/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2011 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @public
 * @toc Standard_C:
 * @file assert.h
 *
 * @brief 'ISO/IEC 9899, Programming Languages - C' interface assert.h
 *
 * @long This header file conditionally defines the assert(exp) macro.
 *
 * @long If an application including this header is compiled with ASSERT
 *       set and without NDEBUG set when #include "assert.h" is used,
 *       a failure in an assert macro (expression is 0) will
 *       generate an @link "ose.h" "error2".
 *       If the assert expression is true (non-zero), nothing happens.
 *
 * @long Without the compile flag combination above, assert will be a NOP.
 *       Any side effects of the evaluation of the expression will thus
 *       not appear in this case.
 */

#ifndef _ASSERT_H
#define _ASSERT_H

#define OSE_ASSERT_H

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _MSC_VER
__declspec( dllexport )
#endif
extern void ose_assertion_failed(const char *expr, 
                 const char *file, 
                 unsigned line);

#ifdef NDEBUG

#ifdef ASSERT
#error Both NDEBUG and ASSERT defined.
#endif

#define assert(exp) ((void)0)
#define ASSERT_CODE(code)

#else
 
/**
 *  @macro assert
 *  @brief Abort with a call to error2() if assertion is false (0).
 *  @long  This macro is a NOP everywhere except where assert.h is
 *         included with ASSERT set and NDEBUG not set at compile
 *         time.
 *  @input exp Expression to assert.
 *  @restriction Also available in interrupt and timer interrupt processes.
 *  @seealso assert.h
 *  @seealso default_restrictions_calling_functions
 */

#define assert(exp) \
   /*lint -save -e506 */ /* Warning 506 Constant value Boolean */\
   if (!(exp)) {\
      ose_assertion_failed(#exp, __FILE__, __LINE__);\
      /*lint -unreachable */\
   } /*lint -restore */
#define ASSERT_CODE(code) code

#endif

#ifdef __cplusplus
}
#endif

#endif
