/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2011 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @public
 * @toc Standard_C:
 * @file errno.h
 *
 * @brief 'ISO/IEC 9899, Programming Languages - C' interface errno.h
 * @long This file enumerates and shortly describes the error codes returned
 *        by file system calls in errno.
 * @long Errnos are borrowed from BSD 4.4 Lite2 and are compatible with INET.
 *
 */

#ifndef _ERRNO_H
#define _ERRNO_H
#define OSE_ERRNO_H

#ifdef __cplusplus
#if !defined (NO_NAME_STD) && defined (__ARMCC_VERSION)
namespace std {
#endif /* NO_NAME_STD */
extern "C" {
#endif /* __cplusplus */

#undef errno

/**
 * @macro errno
 * @brief Get the errno value.
 * @long There is one errno variable for each process.
 * @seealso default_restrictions_calling_functions
 */
#define errno (*ose_errno_ptr())

/**
 * @function ose_errno_ptr
 * @brief Get pointer to the process' errno variable.
 * @return Pointer to the errno variable.
 * @seealso default_restrictions_calling_functions
 */
int *ose_errno_ptr(void);


/**
 * @macro EFS_FIRST_ERROR
 * @brief First error code indicating that operation failed.
 */
#define EFS_FIRST_ERROR    1

/**
 * @macro EFS_LAST_ERROR
 * @brief Last error code indicating that operation failed.
 */
#define EFS_LAST_ERROR    103


/*
 *---------------------------------------------------------------------------
 * ARM ADS Toolkit specific error codes
 *---------------------------------------------------------------------------
 */

#ifdef __ARMCC_VERSION

/* Renumbered error codes according to BSD 4.4 Lite */

# define 	EOK		0	/* Operation successful */

# define	EPERM		96	/* Operation not permitted */
# define	ENOENT		97	/* No such file or directory */
# define	ESRCH		98	/* No such process */
# ifdef ARM_SWAP_EINTR
#  define	EINTR		99	/* Interrupted system call */
# else
#  define	EINTR		4	/* Interrupted system call */
# endif

/* Error codes as defined by ARM ADS */

# define	EDOM		1	/* Numerical argument out of domain */
# define	ERANGE		2	/* Result too large */
# define	ESIGNUM		3	/* Illegal signal number to signal() */
# ifdef ARM_SWAP_EINTR
#  define	EILSEQ		4	/* Multi-byte character invalid */
# else
#  define   EILSEQ    102  /* Multi-byte character invalid */
# endif

/*
 *---------------------------------------------------------------------------
 * Corresponding Error codes for all other environments
 *---------------------------------------------------------------------------
 */

#else

/* Error codes according to BSD 4.4 Lite2 */

/**
 * @macro EOK
 * @brief Operation successful.
 */
#define EOK		0

/**
 * @macro EPERM
 * @brief Operation not permitted.
 */
#define	EPERM		1

/**
 * @macro ENOENT
 * @brief No such file or directory.
 */
#define	ENOENT		2

/**
 * @macro ESRCH
 * @brief No such process.
 */
#define	ESRCH		3

/**
 * @macro EINTR
 * @brief Interrupted system call.
 */
#define	EINTR		4

/* math software */

/**
 * @macro EDOM
 * @brief Numerical argument out of domain.
 */
#define	EDOM		33

/**
 * @macro ERANGE
 * @brief Result too large.
 */
#define	ERANGE		34

/**
 * @macro EILSEQ
 * @brief Multi-byte character invalid.
 */

#define   EILSEQ      102

#endif

/*
 *---------------------------------------------------------------------------
 * Common error codes for all environments
 *---------------------------------------------------------------------------
 */
/* Error codes according to BSD 4.4 Lite2 */

/**
 * @macro EIO
 * @brief Input/output error.
 */
#define	EIO		5

/**
 * @macro ENXIO
 * @brief Device not configured.
 */
#define	ENXIO		6

/**
 * @macro E2BIG
 * @brief Argument list too long.
 */
#define	E2BIG		7

/**
 * @macro ENOEXEC
 * @brief Exec format error.
 */
#define	ENOEXEC		8

/**
 * @macro EBADF
 * @brief Bad file descriptor.
 */
#define	EBADF		9

/**
 * @macro ECHILD
 * @brief No child processes.
 */
#define	ECHILD		10

/**
 * @macro EDEADLK
 * @brief Resource deadlock avoided.
 */
#define	EDEADLK		11

/**
 * @macro ENOMEM
 * @brief Can not allocate memory.
 */
#define	ENOMEM		12

/**
 * @macro EACCES
 * @brief Permission denied.
 */
#define	EACCES		13

/**
 * @macro EFAULT
 * @brief Bad address.
 */
#define	EFAULT		14

/**
 * @macro ENOTBLK
 * @brief Block device required.
 */
#define	ENOTBLK		15

/**
 * @macro EBUSY
 * @brief Device busy.
 */
#define	EBUSY		16

/**
 * @macro EEXIST
 * @brief File exists.
 */
#define	EEXIST		17

/**
 * @macro EXDEV
 * @brief Cross-device link.
 */
#define	EXDEV		18

/**
 * @macro ENODEV
 * @brief Operation not supported by device.
 */
#define	ENODEV		19

/**
 * @macro ENOTDIR
 * @brief Not a directory.
 */
#define	ENOTDIR		20

/**
 * @macro EISDIR
 * @brief Is a directory.
 */
#define	EISDIR		21

/**
 * @macro EINVAL
 * @brief Invalid argument.
 */
#define	EINVAL		22

/**
 * @macro ENFILE
 * @brief Too many open files in system.
 */
#define	ENFILE		23

/**
 * @macro EMFILE
 * @brief Too many open files.
 */
#define	EMFILE		24

/**
 * @macro ENOTTY
 * @brief Inappropriate ioctl for device.
 */
#define	ENOTTY		25

/**
 * @macro ETXTBSY
 * @brief Text file busy.
 */
#define	ETXTBSY		26

/**
 * @macro EFBIG
 * @brief File too large.
 */
#define	EFBIG		27

/**
 * @macro ENOSPC
 * @brief No space left on device.
 */
#define	ENOSPC		28

/**
 * @macro ESPIPE
 * @brief Illegal seek.
 */
#define	ESPIPE		29

/**
 * @macro EROFS
 * @brief Read-only file system.
 */
#define	EROFS		30

/**
 * @macro EMLINK
 * @brief Too many links.
 */
#define	EMLINK		31

/**
 * @macro EPIPE
 * @brief Broken pipe.
 */
#define	EPIPE		32

/* Error codes 33,34 in use - defined above */

/* non-blocking and interrupt i/o */

/**
 * @macro EAGAIN
 * @brief Resource temporarily unavailable.
 */
#define	EAGAIN		35

/**
 * @macro EWOULDBLOCK
 * @brief Operation would block.
 */
#define	EWOULDBLOCK	EAGAIN

/**
 * @macro EINPROGRESS
 * @brief Operation now in progress.
 */
#define	EINPROGRESS	36

/**
 * @macro EALREADY
 * @brief Operation already in progress.
 */
#define	EALREADY	37

/* ipc/network software -- argument errors */

/**
 * @macro ENOTSOCK
 * @brief Socket operation on non-socket.
 */
#define	ENOTSOCK	38

/**
 * @macro EDESTADDRREQ
 * @brief Destination address required.
 */
#define	EDESTADDRREQ	39

/**
 * @macro EMSGSIZE
 * @brief Message too long.
 */
#define	EMSGSIZE	40

/**
 * @macro EPROTOTYPE
 * @brief Protocol wrong type for socket.
 */
#define	EPROTOTYPE	41

/**
 * @macro ENOPROTOOPT
 * @brief Protocol not available.
 */
#define	ENOPROTOOPT	42

/**
 * @macro EPROTONOSUPPORT
 * @brief Protocol not supported.
 */
#define	EPROTONOSUPPORT	43

/**
 * @macro ESOCKTNOSUPPORT
 * @brief Socket type not supported.
 */
#define	ESOCKTNOSUPPORT	44

/**
 * @macro EOPNOTSUPP
 * @brief Operation not supported.
 */
#define	EOPNOTSUPP	45

/**
 * @macro EPFNOSUPPORT
 * @brief Protocol family not supported.
 */
#define	EPFNOSUPPORT	46

/**
 * @macro EAFNOSUPPORT
 * @brief Address family not supported by protocol family.
 */
#define	EAFNOSUPPORT	47

/**
 * @macro EADDRINUSE
 * @brief Address already in use.
 */
#define	EADDRINUSE	48

/**
 * @macro EADDRNOTAVAIL
 * @brief Can not assign requested address.
 */
#define	EADDRNOTAVAIL	49

/* ipc/network software -- operational errors */

/**
 * @macro ENETDOWN
 * @brief Network is down.
 */
#define	ENETDOWN	50

/**
 * @macro ENETUNREACH
 * @brief Network is unreachable.
 */
#define	ENETUNREACH	51

/**
 * @macro ENETRESET
 * @brief Network dropped connection on reset.
 */
#define	ENETRESET	52

/**
 * @macro ECONNABORTED
 * @brief Software caused connection abort.
 */
#define	ECONNABORTED	53

/**
 * @macro ECONNRESET
 * @brief Connection reset by peer.
 */
#define	ECONNRESET	54

/**
 * @macro ENOBUFS
 * @brief No buffer space available.
 */
#define	ENOBUFS		55

/**
 * @macro EISCONN
 * @brief Socket is already connected.
 */
#define	EISCONN		56

/**
 * @macro ENOTCONN
 * @brief Socket is not connected.
 */
#define	ENOTCONN	57

/**
 * @macro ESHUTDOWN
 * @brief Can not send after socket shutdown.
 */
#define	ESHUTDOWN	58

/**
 * @macro ETOOMANYREFS
 * @brief Too many references: can not splice.
 */
#define	ETOOMANYREFS	59

/**
 * @macro ETIMEDOUT
 * @brief Operation timed out.
 */
#define	ETIMEDOUT	60

/**
 * @macro ECONNREFUSED
 * @brief Connection refused.
 */
#define	ECONNREFUSED	61

/**
 * @macro ELOOP
 * @brief Too many levels of symbolic links.
 */
#define	ELOOP		62

/**
 * @macro ENAMETOOLONG
 * @brief File name too long.
 */
#define	ENAMETOOLONG	63

/* should be rearranged */

/**
 * @macro EHOSTDOWN
 * @brief Host is down.
 */
#define	EHOSTDOWN	64

/**
 * @macro EHOSTUNREACH
 * @brief No route to host.
 */
#define	EHOSTUNREACH	65

/**
 * @macro ENOTEMPTY
 * @brief Directory not empty.
 */
#define	ENOTEMPTY	66

/* quotas & mush */

/**
 * @macro EPROCLIM
 * @brief Too many processes.
 */
#define	EPROCLIM	67

/**
 * @macro EUSERS
 * @brief Too many users.
 */
#define	EUSERS		68

/**
 * @macro EDQUOT
 * @brief Disc quota exceeded.
 */
#define	EDQUOT		69

/* Network File System */

/**
 * @macro ESTALE
 * @brief Stale connection to media.
 */
#define	ESTALE		70

/**
 * @macro EREMOTE
 * @brief Too many levels of remote in path.
 */
#define	EREMOTE		71

/**
 * @macro EBADRPC
 * @brief RPC struct is bad.
 */
#define	EBADRPC		72

/**
 * @macro ERPCMISMATCH
 * @brief Wrong RPC version.
 */
#define	ERPCMISMATCH	73

/**
 * @macro EPROGUNAVAIL
 * @brief RPC program not available.
 */
#define	EPROGUNAVAIL	74

/**
 * @macro EPROGMISMATCH
 * @brief Program version wrong.
 */
#define	EPROGMISMATCH	75

/**
 * @macro EPROCUNAVAIL
 * @brief Bad procedure for program.
 */
#define	EPROCUNAVAIL	76

/**
 * @macro ENOLCK
 * @brief No locks available.
 */
#define	ENOLCK		77

/**
 * @macro ENOSYS
 * @brief Function not implemented.
 */
#define	ENOSYS		78

/**
 * @macro EFTYPE
 * @brief Inappropriate file type or format.
 */
#define	EFTYPE		79

/**
 * @macro EAUTH
 * @brief Authentication error.
 */
#define	EAUTH		80

/**
 * @macro ENEEDAUTH
 * @brief Need authenticator.
 */
#define	ENEEDAUTH	81

/* Missing POSIX.1 error codes */

/**
 * @macro ENOLOCK
 * @brief No locks available.
 */
#define ENOLOCK		82

/* Missing POSIX.4 error codes */

/**
 * @macro EBADMSG
 * @brief Corrupted message.
 */
#define EBADMSG		83

/**
 * @macro ECANCELED
 * @brief Operation canceled.
 */
#define ECANCELED	84

/**
 * @macro EFAIL
 * @brief Can not start operation.
 */
#define EFAIL		85

/**
 * @macro EINPROG
 * @brief Operation in progress.
 */
#define EINPROG		86

/**
 * @macro EMTIMERS
 * @brief Too many timers.
 */
#define EMTIMERS	87

/**
 * @macro ENOTSUP
 * @brief Not supported.
 */
#define ENOTSUP		88

/* EFS extensions */

/**
 * @macro EDEVERR
 * @brief Device error.
 */
#define EDEVERR		89

/**
 * @macro EDSKCHG
 * @brief Disk changed.
 */
#define EDSKCHG		90

/**
 * @macro EINTERNAL
 * @brief Internal error.
 */
#define EINTERNAL	91

/**
 * @macro EINVCLT
 * @brief Invalid client.
 */
#define EINVCLT		92

/**
 * @macro ENOFM
 * @brief No such file or format manager.
 */
#define ENOFM		93

/**
 * @macro ENOTMOUNTED
 * @brief Not mounted.
 */
#define ENOTMOUNTED	94

/**
 * @macro EREMOUNT
 * @brief Attempt to remount device.
 */
#define EREMOUNT	95

/* Error codes 96-99 reserved - see ARM ADS above! */

/**
 * @macro EOVERFLOW
 * @brief Value too large to be stored in data type.
 */
#define EOVERFLOW   100

/**
 * @macro ENOATTR
 * @brief No such attribute.
 */
#define ENOATTR     101

/*
 * defined above for non ARM targets
 * #define   EILSEQ      102
 */

/*
 * @macro ENOMSG
 * @brief No message of desired type
 */
#define ENOMSG		103

/*
 ****************************************************************************
 * 9  VARIABLES.
 ****************************************************************************
 */

#ifdef __cplusplus
}
#if !defined (NO_NAME_STD) && defined (__ARMCC_VERSION)
}  /* namespace std */
using namespace ::std;
#endif /* NO_NAME_STD */
#endif /* __cplusplus */

#endif
