/* COPYRIGHT-MIX-SRC-R2 *
 **************************************************************************
 * This software is licensed under more than one license.
 *
 * Some parts of this software have been modified/altered by
 * Enea Software AB. For such modified parts of the software
 * (below the "Software") the following applies.
 *
 * Copyright (C) 2004-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a separate Software License Agreement
 * and may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 *
 * The other licenses that apply to this software are included below
 * this section.
 **************************************************************************
 * COPYRIGHT-END */


/*
Copyright (c) 1994, 1997 Cygnus Solutions.
All rights reserved.

Redistribution and use in source and binary forms are permitted
provided that the above copyright notice and this paragraph are
duplicated in all such forms and that any documentation,
advertising materials, and other materials related to such
distribution and use acknowledge that the software was developed
at Cygnus Solutions. Cygnus Solutions may not be used to
endorse or promote products derived from this software without
specific prior written permission.
THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/

/**
 * @public
 * @toc Standard_C:
 * @file ctype.h
 *
 * @brief 'ISO/IEC 9899, Programming Languages - C' interface ctype.h
 *
 */

#ifndef _CTYPE_H_
#define _CTYPE_H_

#ifdef __cplusplus
#if !defined (NO_NAME_STD) && defined (__ARMCC_VERSION)
namespace std {
#endif /* NO_NAME_STD */
extern "C" {
#endif /* __cplusplus */
   
#include "_ansi.h"

int  isalnum (int __c);
int  isalpha (int __c);
int  iscntrl (int __c);
int  isdigit (int __c);
int  isgraph (int __c);
int  islower (int __c);
int  isprint (int __c);
int  ispunct (int __c);
int  isspace (int __c);
int  isupper (int __c);
int  isxdigit(int __c);
#ifndef __cplusplus
int  tolower (int __c);
int  toupper (int __c);
#endif

#ifndef __STRICT_ANSI__
int  isascii (int __c);
int  toascii (int __c);
int  _tolower (int __c);
int  _toupper (int __c);
#endif

#define	_U	01
#define	_L	02
#define	_N	04
#define	_S	010
#define _P	020
#define _C	040
#define _X	0100
#define	_B	0200

#ifdef __DCC__ /* Diab compiler specials..... */
#define __CTYPE _ctype_
extern const unsigned char __CTYPE[];   
#else   
extern const char _ctype_[];
#endif

#ifndef __cplusplus
/* These macros are intentionally written in a manner that will trigger
   a gcc -Wall warning if the user mistakenly passes a 'char' instead
   of an int containing an 'unsigned char'.  Note that the sizeof will
   always be 1, which is what we want for mapping EOF to __ctype_ptr__[0];
   the use of a raw index inside the sizeof triggers the gcc warning if
   __c was of type char, and sizeof masks side effects of the extra __c.
   Meanwhile, the real index to __ctype_ptr__+1 must be cast to int,
   since isalpha(0x100000001LL) must equal isalpha(1), rather than being
   an out-of-bounds reference on a 64-bit machine.  */
#define _ctype_lookup(c) ((_ctype_+sizeof(""[c]))[(int)(c)])
   
#define	isalpha(c)	(_ctype_lookup(c)&(_U|_L))
#define	isupper(c)	(_ctype_lookup(c)&_U)
#define	islower(c)	(_ctype_lookup(c)&_L)
#define	isdigit(c)	(_ctype_lookup(c)&_N)
#define	isxdigit(c)	(_ctype_lookup(c)&(_X|_N))
#define	isspace(c)	(_ctype_lookup(c)&_S)
#define ispunct(c)	(_ctype_lookup(c)&_P)
#define isalnum(c)	(_ctype_lookup(c)&(_U|_L|_N))
#define isprint(c)	(_ctype_lookup(c)&(_P|_U|_L|_N|_B))
#define	isgraph(c)	(_ctype_lookup(c)&(_P|_U|_L|_N))
#define iscntrl(c)	(_ctype_lookup(c)&_C)
#endif /* __cplusplus */
   
#ifdef __cplusplus
inline int toupper(int c) { return islower(c) ? (c - 'a' + 'A') : c; }
inline int tolower(int c) { return isupper(c) ? (c - 'A' + 'a') : c; }
#elif defined(__GNUC__)
/* Non-gcc versions will get the library versions, and will be
   slightly slower */
#define toupper(c) \
	__extension__({ int __x = (c); islower(__x) ? (__x - 'a' + 'A') : __x;})
#define tolower(c) \
	__extension__({ int __x = (c); isupper(__x) ? (__x - 'A' + 'a') : __x;})
#endif

#ifndef __STRICT_ANSI__
#define isascii(c)	((unsigned)(c)<=0177)
#define toascii(c)	((c)&0177)
#endif

#ifdef __cplusplus
}
#if !defined (NO_NAME_STD) && defined (__ARMCC_VERSION)
}  /* namespace std */
using namespace ::std;
#endif /* NO_NAME_STD */
#endif /* __cplusplus */

#endif /* _CTYPE_H_ */
