/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @public
 * @toc Program_Management:
 * @file ose_err/pm_err.h
 * @version
 * @brief The program manager and RTL error calls.
 */

#ifndef PM_ERR_H_
#define PM_ERR_H_

/***********************************************************
 *
 * INCLUDE FILES
 *
 ***********************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/***********************************************************
 *
 * CONSTANTS AND DEFINITIONS
 *
 ***********************************************************/

/***********************************************************
 *
 * STRUCTURES AND GLOBALS
 *
 ***********************************************************/

/**
 * @type pm_file_line
 *
 * @include "ose_err/pm_err.h"
 *
 * @declaration
 * struct pm_file_line
 * {
 *   const char * file;
 *   unsigned line;
 * };
 *
 * @brief A structure holding the file and line information of an internal
 *        error in PM/RTL.
 * @field file  The name of the file generating the internal error call.
 * @field line  The line in the file generating the internal error call.
 */
struct pm_file_line
{
  const char * file;
  unsigned line;
};

/***********************************************************
 *
 * PGM ERROR DEFINITIONS
 *
 ***********************************************************/

/**
 * @macro PM_ERROR_BASE
 *
 * @include "ose_err/pm_err.h"
 *
 * @declaration
 * #define PM_ERROR_BASE                   0x00e00000
 *
 * @brief Error base for errors generated by program manager.
 */
#define PM_ERROR_BASE                   (0x00e00000)

/**
 * @macro PM_EGMM_LOST
 *
 * @include "ose_err/pm_err.h"
 *
 * @declaration
 * #define PM_EGMM_LOST                    0x00e00002
 *
 * @brief The GMM process was lost (killed).
 *
 * @long Extra: None.
 */
#define PM_EGMM_LOST                    (0x00e00002)

/**
 * @macro PM_EBIOS_INSTALL_FAILED
 *
 * @include "ose_err/pm_err.h"
 *
 * @declaration
 * #define PM_EBIOS_INSTALL_FAILED         0x00e00003
 *
 * @brief The PM BIOS interface could not be installed. 
 *
 * @long Extra: None.
 */
#define PM_EBIOS_INSTALL_FAILED         (0x00e00003)

/**
 * @macro PM_EINTERNAL
 *
 * @include "ose_err/pm_err.h"
 *
 * @declaration
 * #define PM_EINTERNAL                    0x00e00004
 *
 * @brief PM internal error.
 *
 * @long Extra: A struct pm_file_line giving information of where the internal
 *              error was detected.
 */
#define PM_EINTERNAL                    (0x00e00004)

/**
 * @macro PM_EPERSISTENCY_FAILURE
 *
 * @include "ose_err/pm_err.h"
 *
 * @declaration
 * #define PM_EPERSISTENCY_FAILURE         0x00e00006
 *
 * @brief The persistency handling failed.
 *
 * @long Extra: None.
 *       If the error handler returns, some persistent load modules may be
 *       lost.
 */
#define PM_EPERSISTENCY_FAILURE         (0x00e00006)

/***********************************************************
 *
 * RTL ERROR DEFINITIONS
 *
 ***********************************************************/

/**
 * @macro RTL_ERROR_BASE
 *
 * @include "ose_err/pm_err.h"
 *
 * @declaration
 * #define RTL_ERROR_BASE                  0x00e01000
 *
 * @brief Error base for errors generated by run-time loader.
 */
#define RTL_ERROR_BASE                  (0x00e01000)

/**
 * @macro RTL_EPM_CONNECTION_DENIED
 *
 * @include "ose_err/pm_err.h"
 *
 * @declaration
 * #define RTL_EPM_CONNECTION_DENIED       0x00e01003
 *
 * @brief PM denied the connection.
 *
 * @long Extra: Status sent by PM.
 */
#define RTL_EPM_CONNECTION_DENIED       (0x00e01003)

#ifdef __cplusplus
}
#endif

#endif /* #ifndef PM_ERR_H_ */
