/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

/**
 * @public
 * @toc OSE:
 * @file ose_events/rmd_events.h
 * @brief The Run Mode Dumper user event numbers.
 */

#ifndef _RMD_EVENTS_H
#define _RMD_EVENTS_H

#include "stdint.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @macro RMD_EVENT_START_DUMP
 * @brief User event number (10000) for tracing start of run mode dumps.
 * @long Event data is the struct @link "" "RmdStartDumpEvent".
 */
#define RMD_EVENT_START_DUMP (10000) /* !-EVTNO(RmdStartDumpEvent)-! */

/**
 * @macro RMD_EVENT_STOP_DUMP
 * @brief User event number (10001) for tracing stop of run mode dumps.
 * @long Event data is the struct @link "" "RmdStopDumpEvent".
 */
#define RMD_EVENT_STOP_DUMP (10001) /* !-EVTNO(RmdStopDumpEvent)-! */

/**
 * @type RmdStartDumpEvent
 *
 * @brief Start dump event
 * @output pid PID associated with the dump
 * @output reserved0 Not used.
 * @output reserved1 Not used.
 * @output file_name A variable size array containing the dump file name.
 *
 */
typedef struct RmdStartDumpEvent
{
   uint32_t pid;
   uint32_t reserved0;
   uint32_t reserved1;
   char  file_name[1];
} RmdStartDumpEvent;

/**
 * @type RmdStopDumpEvent
 *
 * @brief Stop dump event
 * @output pid PID associated with the dump
 * @output reserved0 Not used.
 * @output reserved1 Not used.
 *
 */
typedef struct RmdStopDumpEvent
{
   uint32_t pid;
   uint32_t reserved0;
   uint32_t reserved1;
} RmdStopDumpEvent;

#ifdef __cplusplus
}
#endif

#endif /* _RMD_EVENTS_H */
