/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @public
 * @toc OSE:
 * @file ose_spi/cpu_hal.h
 */

#ifndef CPU_HAL_H_
#define CPU_HAL_H_

#include "osetypes.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Incomplete type declaration. 
 * The actual type is declared for each architecture in cpu_hal_arch.h
 */
struct arch_cpu_descriptor;


/**
 * @type cpu_descriptor
 * 
 * @brief Generic CPU information.
 *
 * @long This structure hold architecture specific cpu information.
 *
 * @field probe Function pointer. Returns a non-zero value if handled by 
 *               this module, otherwise zero. Target specific and encoded.
 *
 * @field probe.cpu_id The CPU ID given to the probe function.
 *
 * @field version Version of architecture specific interface.
 *
 * @field arch_cpu_descriptor Pointer to architecture specific CPU 
 *                            information. 
 */
struct cpu_descriptor
{
   /*
    * Returns a non-zero value if handled by this module, otherwise
    * zero. Target specific and encoded.
    */
   U32 (*probe)(U32 cpu_id);

   /*
    * Version of architecture specific interface.
    */
   U32 version;

   /*
    * Pointer to architecture specific CPU information. 
    */
   struct arch_cpu_descriptor *arch_cpu_descriptor;
};

#ifdef __cplusplus
}
#endif

#endif /* #ifndef CPU_HAL_H_ */

