/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @toc File_System:
 *
 * @file ose_spi/efstypes.h
 *
 * @brief EFS common types and definitions.
 *
 *
 */

#ifndef _EFSTYPES_H
#define _EFSTYPES_H

#include "osetypes.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 ***********************************************************
 * GENERAL DATA TYPES
 ***********************************************************
 */

/**
 * @type  EfsStatus
 *
 * @brief Return status type in reply signals from all modules in the
 *        Embedded File System
 *
 * @long Status code can have several different sources, e.g. a reply status
 *       from a volume manager might originate from the block device driver
 *       that the volume manager is using. The definition of EFS status codes
 *       is distributed to the following lists:
 * @break
 * @tab  Status codes for file/volume manager, preferred for new designs:
 * @tab  @link "errno.h" "errno"
 * @tab  Status codes for file/volume manager, old:
 * @tab  @link "fm.sig" "FM_ERRBASE"
 * @tab  Block device driver status codes:
 * @tab  @link "ddb.sig" "DDB_ERRBASE"
 * @tab  Status codes for character device drivers:
 * @tab  @link "ddc.sig" "DDC_ERRBASE"
 * @break
 *       See also status codes @link "" "EFS_SUCCESS" and
 *       @link "" "EFS_NO_MORE_ITEMS" in this file.
 *
 * @seealso EFS_SUCCESS
 * @seealso EFS_NO_MORE_ITEMS
 * @seealso efs_status_to_errno
 */
typedef S32 EfsStatus;

/**
 * @type EfsBoolean
 *
 * @brief Boolean type used within EFS
 * @long 1 means True and 0 means False.
 */
typedef U32 EfsBoolean;

/**
 * @type EfsBitmask
 *
 * @brief Bitmask type used within EFS
 *
 * @long  Use this type for future compatibility.
 */
typedef U32 EfsBitmask;

/**
 * @type EfsEnum
 *
 * @brief Typedef for an enum type used within EFS
 *
 * @long  This type is used instead of an enum in interfaces, since the storage
 *        size for enum is compiler dependent.
 *
 * @long  Use this type for future compatibility.
 */
typedef U32 EfsEnum;

/*
 ***********************************************************
 * STATUS/ERROR CODES
 ***********************************************************
*/

/**
 * @macro EFS_SUCCESS
 *
 * @brief Status code of type @link "" "EfsStatus" indicating success.
 *
 * @seealso EfsStatus
 */
#define EFS_SUCCESS		0

/**
 * @macro EFS_NO_MORE_ITEMS
 *
 * @brief Status code of type @link "" "EfsStatus" indicating no more items.
 *
 * @long A reply signal with this status is sent after all known items have
 *       been sent. The reply signal having this status does not contain any
 *       reply data.
 *
 * @seealso EfsStatus
 * @seealso FssListReply
 */
#define EFS_NO_MORE_ITEMS	1

#ifdef __cplusplus
}
#endif

#endif
