/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2008 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef CPU_SPINLOCK_H
#define CPU_SPINLOCK_H

#include "stdint.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef INLINE
#define INLINE __inline__
#endif

typedef struct
{
   volatile uint32_t lock;
} spinlock_t;

#define SPIN_LOCK_INIT		 { 0 }

#define spin_lock_init(x) 	do { (x)->lock = 0; } while(0)
#define spin_is_locked(x)	((x)->lock != 0)
#define spin_unlock_wait(x)	do {} while(spin_is_locked(x))


/* cpu_spin_trylock() returns zero on success. */ 
static INLINE uint32_t
cpu_spin_trylock(spinlock_t* lock)
{
    uint32_t tmp;

    __asm__ volatile
    (
        "   .set push           \n"
        "   .set noreorder      \n"
        "   .set mips2          \n"
        "1: ll      %0, (%1)    \n"
        "   bnez    %0, 2f      \n"
        "   move    %0, %2      \n" /* delay slot */
        "   sc      %0, (%1)    \n"
        "   beqz    %0, 1b      \n"
        "   nop                 \n" /* delay slot */
        "   sync                \n"
        "   move    %0, $0      \n" /* Return zero for success. */
        "2:                     \n"
        "   .set pop            \n"
        : /* output */ "=&r" (tmp)
        : /* input  */ "r" (&lock->lock), "r" (1)
        : /* modify */ "memory"
    );
    return tmp;
}

static INLINE void
cpu_spin_lock(spinlock_t* lock)
{
    uint32_t tmp;

    __asm__ volatile
    (
        "   .set push           \n"
        "   .set noreorder      \n"
        "   .set mips2          \n"
        "1: ll      %0, (%1)    \n"
        "   bnez    %0, 1b      \n"
        "   move    %0, %2      \n" /* delay slot */
        "   sc      %0, (%1)    \n"
        "   beqz    %0, 1b      \n"
        "   nop                 \n" /* delay slot */
        "   sync                \n"
        "   .set pop            \n"
        : /* output */ "=&r" (tmp)
        : /* input  */ "r" (&lock->lock), "r" (1)
        : /* modify */ "memory"
    );
}

static INLINE void
cpu_spin_unlock(spinlock_t* lock)
{
  /* Make updated protected data visible */
   __asm__ __volatile__("sync");
   lock->lock = 0;
   /* Make the update of the lock visible */
   __asm__ __volatile__("sync");
}

#ifdef __cplusplus
}
#endif

#endif /* CPU_SPINLOCK_H */
