/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @public
 * @toc Program_Management:
 * @file ose_spi/ose_conf.h
 * @version
 * @brief Defines constants for LMCONF configuration section
 * @long  There is a special section named LMCONF (sometimes ose_confd.o) which
 *        located is in the first ELF segment. This file defines some constants
 *        if you need to access this section. Normally, configuration stored in
 *        this section is accessed by other means.
 */

#ifndef _OSE_CONF_H
#define _OSE_CONF_H

#include "osarch.h"   /* To determine softOSE */

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @macro CF_CONF_MAGIC
 *
 * @include "ose_spi/ose_conf.h"
 *
 * @declaration
 * #define  CF_CONF_MAGIC      "OSE Configuration"
 *
 * @brief Magic header for LMCONF section.
 *
 * @long  This string is the first thing that exists in the LMCONF
 *        section. It is used for sanity checks.
 */
#define  CF_CONF_MAGIC      "OSE Configuration"

/**
 * @macro CF_CONF_MAGIC_SIZE
 *
 * @include "ose_spi/ose_conf.h"
 *
 * @declaration
 * #define  CF_CONF_MAGIC_SIZE (sizeof (CF_CONF_MAGIC) - 1)
 *
 * @brief Size of configuration magic.
 *
 * @long  This constant is used for those who do sanity checks of the configuration
 *        section.
 */
#define  CF_CONF_MAGIC_SIZE (sizeof (CF_CONF_MAGIC) - 1)

/**
 * @data ose_confd
 *
 * @include "ose_spi/ose_conf.h"
 *
 * @declaration
 * extern const char ose_confd[];
 *
 * @brief Location of the configuration section.
 *
 * @long  This is a pointer to where the configuration section begins.
 */
#ifdef softOSE
extern char ose_confd[];
#else
extern const char ose_confd[];
#endif

#ifdef __cplusplus
}
#endif

/* #ifndef _OSE_CONF_H */
#endif
