/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 1991-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

/**
 * @toc OSE:
 * @file ose_spi/ose_rem.h
 * @brief Undocumented.
 * @long
 * Signal declarations for remote system calls.
 * It should be included in remote call server processes written in C.
 *
 */
/*
******************************************************************************
*									     *
*		OSE MULTITASKING REALTIME OPERATING SYSTEM.		     *
*									     *
******************************************************************************
* @(#) File: ose_rem.h		Id: 08/1946370276339265			     *
* @(#) Revision: 2.7		Revision date: 03/09/25			     *
*
* This file contains signal declarations for remote system calls.
* It should be included in remote call server processes written in C.
*
* History of changes
*
*	Date	Name		Vers	Change
*	------	----		----	-------------------------------
*
*	910901	Bengt Eliasson	2.0	Initial release.
*	920903	Bengt Eliasson	2.1	Added field to rem_hunt.
*	921204	Bengt Eliasson	2.2	Modified fields in rem_set_bp,
*					rem_set_mem and in remr_get_mem.
*	930502	Bengt Eliasson	2.3	Added rem_set_pcb and remr_set_pcb.
*	930806	Bengt Eliasson	2.4	Added some comments.
*	931008	Bengt Eliasson	2.5	Corrected an error in rem_set_pcb.
*	931024	Bengt Eliasson	2.6	The rem_set_uid struct was removed
*                                       along with the set_uid system call.
*       030925 Gunnar Eklund    2.7     Added srcdoc comments.
*/


#ifndef ZZOSE_REM_H
#define ZZOSE_REM_H

#include "ose.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef SIGSELECT OSCALLTYPE;

struct rem_clear_bp
    {
    PROCESS pid;
    OSADDRESS addr;
    };

struct remr_clear_bp
    {
    OSBOOLEAN failed; 
    };

struct rem_create_error_handler
    {
    PROCESS pid;
    OSADDRESS entrypoint;
    OSADDRESS stacksize; 
    };

struct remr_create_error_handler
    {
    OSADDRESS entrypoint;
    };

struct rem_flush
    {
    PROCESS pid;
    PROCESS psel[1]; /* Variable size */
    };

struct rem_get_cpu
    {
    PROCESS pid;
    };

struct remr_get_cpu
    {
    char name[1]; /* Variable size */
    };

struct rem_get_env
    {
    PROCESS pid;
    char name[1]; /* Variable size */
    };

struct remr_get_env
    {
    char value[1]; /* Variable size */
    };

struct rem_get_env_list
    {
    PROCESS pid;
    char first_name[1]; /* Variable size */
    };

struct remr_get_env_list
    {
    char name[1]; /* Variable size */
    };

struct rem_get_fsem
    {
    PROCESS pid;
    };

struct remr_get_fsem
    {
    OSFSEMVAL value;
    };

struct rem_get_mem
    {
    PROCESS pid;
    OSADDRESS from;
    void * to;
    OSADDRESS size;
    };

struct remr_get_mem
    {
    OSBOOLEAN failed;
    };

struct rem_get_pcb
    {
    PROCESS pid;
    };

struct remr_get_pcb
    {
    struct OS_pcb pcb;
    };

struct rem_get_pid_list
    {
    PROCESS bid; 
    };

struct remr_get_pid_list
    {
    OSBUFSIZE count;
    PROCESS pidlist[1]; /* Variable size */
    };

struct rem_get_pri
    {
    PROCESS pid;
    };

struct remr_get_pri
    {
    OSPRIORITY pri;
    };

struct rem_get_signal
    {
    PROCESS pid;
    OSADDRESS mailptr;
    };

struct remr_get_signal
    {
    OSADDRESS nextsig;
    OSBUFSIZE sigsize;		/* Zero if call failed. */
    PROCESS sender;
    PROCESS addressee;
    unsigned char contents[1]; /* Variable size */
    };

struct rem_get_uid
    {
    PROCESS pid;
    };

struct remr_get_uid
    {
    OSUSER uid;
    };

struct rem_hunt
    {
    OSUSER user;
    PROCESS from;
    OSBOOLEAN persistent;
    char name[1]; /* Variable size */
    };

struct remr_hunt
    {
    PROCESS pid;
    OSBOOLEAN found;
    };

struct rem_intercept
    {
    PROCESS pid;
    };

struct rem_resume
    {
    PROCESS pid;
    };

struct rem_set_bp
    {
    PROCESS pid;
    OSADDRESS address;
    OSADDRESS attribute;
    OSBUFSIZE sigsize;
    unsigned char trapsig_contents[1];
    };

struct remr_set_bp
    {
    OSBOOLEAN failed;
    };

struct rem_set_env
    {
    PROCESS pid;
    OSBUFSIZE name;   /* Index into strings. */
    OSBUFSIZE value;  /* Index into strings. */
    char strings[1];  /* Variable size */
    };

struct remr_set_env
    {
    OSBOOLEAN failed;
    };

struct rem_set_fsem
    {
    PROCESS pid;
    OSFSEMVAL value;
    };

struct rem_set_mem
    {
    PROCESS pid;
    void *from;
    OSADDRESS to;
    OSADDRESS size;
    };

struct remr_set_mem
    {
    OSBOOLEAN failed;
    };

struct rem_set_pcb
    {
    PROCESS pid;
    char cpuregs[1]; /* Variable size. */
    };

struct remr_set_pcb
    {
    OSBOOLEAN failed;
    };

struct rem_signal_fsem
    {
    PROCESS pid;
    };

struct rem_start
    {
    PROCESS pid;
    };

struct rem_stop
    {
    PROCESS pid;
    };

union remote_calls
    {
    struct rem_clear_bp rem_clear_bp;
    struct rem_create_error_handler rem_create_error_handler;
    struct rem_flush rem_flush;
    struct rem_get_cpu rem_get_cpu;
    struct rem_get_env rem_get_env;
    struct rem_get_env_list rem_get_env_list;
    struct rem_get_fsem rem_get_fsem;
    struct rem_get_mem rem_get_mem;
    struct rem_get_pcb rem_get_pcb;
    struct rem_get_pid_list rem_get_pid_list;
    struct rem_get_pri rem_get_pri;
    struct rem_get_signal rem_get_signal;
    struct rem_get_uid rem_get_uid;
    struct rem_hunt rem_hunt;
    struct rem_intercept rem_intercept;
    struct rem_resume rem_resume;
    struct rem_set_bp rem_set_bp;
    struct rem_set_env rem_set_env;
    struct rem_set_fsem rem_set_fsem;
    struct rem_set_mem rem_set_mem;
    struct rem_set_pcb rem_set_pcb;
    struct rem_signal_fsem rem_signal_fsem;
    struct rem_start rem_start;
    struct rem_stop rem_stop;
    };

union remote_responses
    {
    struct remr_clear_bp remr_clear_bp;
    struct remr_create_error_handler remr_create_error_handler;
    struct remr_get_cpu remr_get_cpu;
    struct remr_get_env remr_get_env;
    struct remr_get_env_list remr_get_env_list;
    struct remr_get_fsem remr_get_fsem;
    struct remr_get_mem remr_get_mem;
    struct remr_get_pcb remr_get_pcb;
    struct remr_get_pid_list remr_get_pid_list;
    struct remr_get_pri remr_get_pri;
    struct remr_get_signal remr_get_signal;
    struct remr_get_uid remr_get_uid;
    struct remr_hunt remr_hunt;
    struct remr_set_bp remr_set_bp;
    struct remr_set_env remr_set_env;
    struct remr_set_mem remr_set_mem;
    struct remr_set_pcb remr_set_pcb;
    };

#define REM_CLEAR_BP 1
#define REM_CREATE_ERROR_HANDLER 2
#define REM_FLUSH 3
#define REM_GET_CPU 4
#define REM_GET_ENV 5
#define REM_GET_ENV_LIST 6
#define REM_GET_FSEM 7
#define REM_GET_MEM 8
#define REM_GET_PCB 9
#define REM_GET_PID_LIST 10
#define REM_GET_PRI 11
#define REM_GET_SIGNAL 12
#define REM_GET_UID 13
#define REM_HUNT 14
#define REM_INTERCEPT 15
#define REM_RESUME 16
#define REM_SET_BP 17
#define REM_SET_ENV 18
#define REM_SET_FSEM 19
#define REM_SET_MEM 20
#define REM_SIGNAL_FSEM 21
#define REM_START 22
#define REM_STOP 23
#define REM_SET_PCB 24

#define REMR_CLEAR_BP 1
#define REMR_CREATE_ERROR_HANDLER 2
#define REMR_FLUSH 3
#define REMR_GET_CPU 4
#define REMR_GET_ENV 5
#define REMR_GET_ENV_LIST 6
#define REMR_GET_FSEM 7
#define REMR_GET_MEM 8
#define REMR_GET_PCB 9
#define REMR_GET_PID_LIST 10
#define REMR_GET_PRI 11
#define REMR_GET_SIGNAL 12
#define REMR_GET_UID 13
#define REMR_HUNT 14
#define REMR_INTERCEPT 15
#define REMR_RESUME 16
#define REMR_SET_BP 17
#define REMR_SET_ENV 18
#define REMR_SET_FSEM 19
#define REMR_SET_MEM 20
#define REMR_SIGNAL_FSEM 21
#define REMR_START 22
#define REMR_STOP 23
#define REMR_SET_PCB 24

#define REMOTE_CALL (255) /* !-SIGNO(struct remote_call)-! */

struct remote_call
    {
    SIGSELECT sig_no;
    OSCALLTYPE call_type;
    union remote_calls params;
    };

#define REMOTE_RESPONSE (254) /* !-SIGNO(struct remote_response)-! */

struct remote_response
    {
    SIGSELECT sig_no;
    OSCALLTYPE call_type;
    union remote_responses results;
    };

#define REMOTE_CANCEL (253) /* !-SIGNO(struct remote_cancel)-! */

struct remote_cancel
    {
    SIGSELECT sig_no;
    };

#ifdef __cplusplus
}
#endif

#endif
