/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 1991-2014 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @toc OSE:
 * @file ose_spi/ose_spi.h
 * @brief
 * OSE kernel System Programming Interface (SPI), which is mainly
 * intended for platform developers.
 * @long
 * This file contains declarations to be included in prioritized or
 * background processes written in C.
 * @long This file can not be included in the same file as ose_i.h.
 *
 * @seealso ose.h
 * @seealso ose_i.h
 * @seealso ose_err/ose_err.h
 *
 */
/*
******************************************************************************
*									     *
*		OSE MULTITASKING REALTIME OPERATING SYSTEM.		     *
*									     *
******************************************************************************
* @(#) File: ose_spi.h		ID: 14/1946370276339265
* @(#) Revision: 1.37		Revision date: 14/04/22
*
* $Workfile: ose_spi.h $
* Document no: @(#) XXX  $FileRevision:   /main/tb_current_ose5/19  $
*
* The declarations in this file define the OSE kernel's System Programming
* Interface (SPI), which is mainly intended for platform developers.
*
* This file contains declarations to be included in prioritized or
* background processes written in C.
*
* History of changes
*
*	Date	 Name		  Vers	 Change
*	------	 ----		  ----	 -------------------------------
*
*       030522  Fredrik Bredberg  1.0    Initial release.
*       030925  Gunnar Eklund     1.1    Added srcdoc comments.
*       040830  Gunnar Eklund     1.2    Rewrote get_signal() example.
*       050202  Claes Lillieskold 1.3    Added protection from including both
*                                        this file and ose_i.h in the same
*                                        file.
*       050302  Fredrik Bredberg  1.4    Added support for send-hooks.
*       050411  Gunnar Eklund     1.5    clear_bp and set_bp obsolete.
*       050503  Fredrik Bredberg  1.6    Changed the send-hook info struct
*                                        and the send-hook prototype.
*       050518  Fredrik Bredberg  1.7    Changed return type of OSSENDHOOK.
*       050926  Fredrik Bredberg  1.8    Added PPD inquiry functions.
*       051114  Gunnar Eklund     1.9    Added ose_nano_delay.
*       060424  Gunnar Eklund     1.10   Added ose_get_time_resolution and
*                                        ose_set_time_resolution.
*       070314  Fredrik Bredberg  1.11   Added send_hook_restore().
*       070315  Fredrik Bredberg  1.12   Added ose_bind_process().
*       070509  Gunnar Eklund     1.13   Moved get_env_list(),
*                                        get_sig_poolid(), get_stk_poolid()
*                                        and get_poolcb() from ose_spi.h to
*                                        ose.h.
*       070911  Fredrik Bredberg  1.14   Moved get_bid_list(), get_pid_list()
*                                        and get_pool_list() from ose_spi.h to
*                                        ose.h.
*       071115  Fredrik Bredberg  1.15   Added get_pid_info.
*       071220  Gunnar Eklund     1.16   Obsolete clear_bp() and set_bp()
*                                        finally removed.
*       080415  Fredrik Bredberg  1.17   Moved restore() from ose_spi.h to
*                                        ose.h.
*       080425  Mattias Bertilsson 1.18  Added instance address description
*                                        to assign_linkhandler().
*       080910  Fredrik Bredberg  1.19   Added ose_sys_mutex_init().
*       081113  Fredrik Bredberg  1.20   Added ose_sys_mutex_destroy().
*       090204  Fredrik Bredberg  1.21   Added ose_get_curr_procs().
*       090318  Fredrik Bredberg  1.22   Added get_pid_info() documentation.
*       091011  Robin Westberg    1.23   Added system calls ose_disable_vector,
*                                        ose_enable_vector, ose_create_isr,
*                                        ose_kill_isr, ose_kill_interrupt_hook,
*                                        ose_create_interrupt_hook,
*                                        ose_bind_vector, ose_unbind_vector.
*       091120  Robin Westberg    1.24   Added ose_signal_fsem_from_isr().
*       100119  Gunnar Eklund     1.25   Added s_alloc_nil().
*                                        Documented set_segment().
*       100212  Gunnar Eklund     1.26   Removed the experimental private
*                                        calls ose_get_time_resolution and
*                                        ose_set_time_resolution.
*       100217  Gunnar Eklund     1.27   Added ose_set_pool_max_size().
*       100303  Fredrik Bredberg  1.28   Added ose_fsend().
*       100618  Fredrik Bredberg  1.29   Added alloc_brk() and
*                                        ose_set_pool_attr().
*       100831  Anders Hagstrom   1.30   Added documentation for ose_fsend().
*       101014  Anders Hagstrom   1.31   Added ose_set_hw_rcv_poller().
*       101029  Anders Hagstrom   1.32   Added ose_set_fsend_hook().
*       101104  Gunnar Eklund     1.33   Added ose_set_pool_extender().
*                                        Added new info_tags to get_pid_info().
*       110310  Florin Rotaru     1.34   Add the concept of static pools,
*                                        by introducing a new pool atribute
*                                        OSE_POOL_ATTR_IS_STATIC.
*       110930  Fredrik Bredberg  1.35   Added OSE_PI_CURR_NONINT_PROC,
*                                        OSE_PI_LAST_PROC and OSE_PI_CURR_PROC.
*       111028  Hakan Lindqvist   1.36   Updated doc on assign_linkhandler()
*       140422  Gunnar Eklund     1.37   Updated documentation of hunt_from().
*/


#ifndef ZZOSE_SPI_H
#define ZZOSE_SPI_H

#ifndef ZZOSE_H
#include "ose.h"
#endif

/*
** C++ needs to know that this is C code.
*/

#ifdef __cplusplus
extern "C" {
#endif

/*
** Basic types.
*/

#define OSE_PI_LAST_SUPPORTED   ( 0u)
#define OSE_PI_SUPPORTED        ( 1u)
#define OSE_PI_PTYPE            ( 9u)
#define OSE_PI_SUPERVISOR       (10u)
#define OSE_PI_CURR_NONINT_PROC (11u)
#define OSE_PI_LAST_PROC        (12u)
#define OSE_PI_CPU_ID           (13u)
#define OSE_PI_NAME             (15u)
#define OSE_PI_CURR_PROC        (17u)
#define OSE_PI_SIGS_IN_QUEUE    (18u)
#define OSE_PI_POOL_SIZE        (25u)
#define OSE_PI_POOL_MAX_SIZE    (26u)
#define OSE_PI_POOL_FRAG_SIZE   (27u)

#define OSE_ISR_NO_RESCHED      (0u)
#define OSE_ISR_RESCHED         (1u)

#define OSE_POOL_ATTR_TAGEND    (0u)
#define OSE_POOL_ATTR_EXT_ALLOC (3u)
#define OSE_POOL_ATTR_EXT_FREE  (4u)
#define OSE_POOL_ATTR_EXT_HD0   (5u)
#define OSE_POOL_ATTR_EXT_HD1   (6u)
#define OSE_POOL_ATTR_EXT_HD2   (7u)
#define OSE_POOL_ATTR_EXT_HD3   (8u)
#define OSE_POOL_ATTR_EXT_HD4   (9u)
#define OSE_POOL_ATTR_EXT_HD5   (10u)
#define OSE_POOL_ATTR_EXT_HD6   (11u)
#define OSE_POOL_ATTR_EXT_HD7   (12u)
#define OSE_POOL_ATTR_NO_COPY   (13u)
#define OSE_POOL_ATTR_IS_STATIC (14u)

/**
 * @type OSSEGMENT
 * @private
 * @brief Undocumented.
 */
typedef unsigned long OSSEGMENT;

/**
 * @type OSISRHND
 *
 * @declaration typedef unsigned long OSISRHND;
 *
 * @brief An ISR handle.
 *
 * @seealso ose_create_isr
 * @seealso ose_kill_isr
 */
typedef unsigned long OSISRHND;

/**
 * @type OSISR
 *
 * @declaration unsigned long (OSISR) (void *isr_arg);
 *
 * @brief An ISR function.
 *
 * @long See the description of @link "" "ose_create_isr" for details.
 */
typedef unsigned long (OSISR) (void *);

/**
 * @type OSIHHND
 *
 * @declaration typedef unsigned long OSIHHND;
 *
 * @brief An interrupt hook handle.
 *
 * @seealso ose_create_interrupt_hook
 * @seealso ose_kill_interrupt_hook
 */
typedef unsigned long OSIHHND;

/**
 * @type OSIH
 *
 * @declaration typedef void (OSIH) (void *ih_arg);
 *
 * @brief An interrupt hook function.
 *
 * @long See the description of @link "" "ose_create_interrupt_hook" for details.
 */
typedef void (OSIH) (void *);

/**
 * @type OSHWRCVPOLLER
 *
 * @declaration void (OSHWRCVPOLLER)(void);
 *
 * @brief EXPERIMENTAL. A function pointer.
 *
 */
typedef void (OSHWRCVPOLLER)(void);

/**
 * @type OSFSENDHOOK
 *
 * @declaration int (OSFSENDHOOK)(OSPRIORITY, OSADDRESS)
 *
 * @brief Pointer to user-provided fsend callback function.
 *
 * @long This user-provided callback function can be installed with @link ""
 * "ose_set_fsend_hook". If installed, it will be called each time
 * a signal will be queued onto the fsend queues.
 * @long This function may return a non-zero value to kernel stating that
 * the signal shall be silently discarded (dropped) instead of
 * queued. The decision can depend on the priority and the
 * number of elements already queued at that priority.
 * @long The length of the queues may have been configured to
 * a non-default value via the system parameter krn/fsend_queue.
 *
 * @input prio Current priority for the signal.
 * @input elems_in_queue The number of signals present in the current queue.
 * @return Return non-zero to silently drop the signal, 
 *         or 0 to let the kernel attempt to queue it. NOTE that the latter
 *         may result in an error if queue becomes full.
 * @seealso ose_set_fsend_hook
 * @seealso ose_fsend
 */
typedef int (OSFSENDHOOK)(OSPRIORITY prio, OSADDRESS elems_in_queue);

/**
 * @type OSPOOLEXTENDH
 *
 * @declaration typedef OSADDRESS (OSPOOLEXTENDH)(PROCESS pool, OSERRCODE extra);
 *
 * @brief A pool extender callback.
 *
 * @long See the description of @link "" "ose_set_pool_extender" for details.
 * @field pool  The identifier of the pool.
 * @field extra The value to pass as parameter to extend_pool().
 * @seealso extend_pool
 */
typedef OSADDRESS (OSPOOLEXTENDH)(PROCESS pool, OSERRCODE extra);


/*
** Derived types and misc constants.
*/

/**
 * @type OS_sig_info
 *
 * @declaration struct OS_sig_info
 *{
 *   SIGSELECT sig_no;
 *   PROCESS owner;
 *   PROCESS sender_pid;
 *   PROCESS addressee_pid;
 *   OSBUFSIZE sig_size;
 *   OSBUFSIZE size_in_pool;
 *   OSADDRESS next;
 *};
 *
 * @brief Information about a signal.
 *
 * @field sig_no	The signal number.
 *
 * @field owner	        Process ID of the owner of the signal. If this value is
 *                      zero this means that the signal is owned by the system.
 *                      Signals that have been returned to the system by a call
 *                      to free_buf or is located in the signal queue of a
 *                      process are owned by the system and have zero in this
 *                      field.
 * @field sender_pid    The process ID of the sender of the signal. If the
 *                      signal has not been sent, the value is the same as the
 *                      owner field.
 * @field addressee_pid The process ID of the addressee of the signal,
 *                      if the signal has been redirected. If the signal is the
 *                      free list of the pool (not currently used) the
 *                      addressee_pid will be set to the ID of the
 *                      signal pool, and if possible the sender_pid will be set
 *                      to the process that freed the signal. In all other cases
 *                      the addressee_pid will be set to the same value as the
 *                      owner field.
 * @field sig_size      The size value used when the signal was allocated.
 * @field size_in_pool  The number of bytes the signal occupies in the
 *                      user pool. If memory model 1 is used an additional signal
 *                      administration block is allocated in the system pool, the
 *                      size of this signal administration block is typically
 *                      32 bytes.
 * @field next          The address of the next signal buffer owned by the
 *                      specified process in the specified pool. If the 'pid'
 *                      parameter to get_sig_info was set to zero it will be set
 *                      to the next signal buffer owned by anyone.
 *
 * @seealso get_sig_info
 */
struct OS_sig_info
  {
    SIGSELECT sig_no;
    PROCESS   owner;
    PROCESS   sender_pid;
    PROCESS   addressee_pid;
    OSBUFSIZE sig_size;
    OSBUFSIZE size_in_pool;
    OSADDRESS next;
  };

/*
** Prototypes.
*/









/**
 * @function assign_linkhandler
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSBOOLEAN
 * assign_linkhandler(const char *pathname, PROCESS handler);
 *
 * @brief Register and assign a link path.
 *
 * @long  The pathname argument is registered as a link path and either
 *        assigned to a link handler process or assigned as a block link. The
 *        assignment is as a block link if the handler argument is a block ID,
 *        otherwise as a link handler process.
 *
 * @long  A hunt call is matched with a link path if there is no process with
 *        a name that exacly matches the hunt path, but the initial part of
 *        the huntpath matches the link path, and is followed by a slash ("/")
 *        and possibly additional characters.
 *
 * @long  For instance:
 *        The huntpath "helios/robin/peter" matches the link path "helios" as
 *        well as the link path "helios/robin". The huntpath "helios/" also
 *        matches the link path "helios". It is also possible to register an
 *        empty link path which is matched with a hunt path beginning with
 *        slash.
 *
 * @long  When a hunt matches a link path assigned to a link handler process,
 *        the kernel sends the hunt request as an ose_spi/ose_rem.h
 *        "REMOTE_CALL" signal to the link handler process. The main purpose
 *        of link handler processes is to implement links to remote nodes, but
 *        it is also possible to use for other purposes.
 *
 * @long    A link handler process that receives a hunt request has several
 *          options:
 *
 * @long    1. It may reply with an indication that the process was not found.
 *             This should be done immediately, before the link handler
 *             attempts to establish a link to the process. The kernel will
 *             resolve the hunt later if a process with a matching name is
 *             created by the link handler or by someone else.
 * @long    2. It may create a new process or use an existing process with
 *             the desired name and reply with that process ID. The process
 *             ID is propagated to the caller of the hunt call in the name_
 *             variable.
 * @long    3. It may create a new process or use an existing process with
 *             another name and reply with that process ID, indicating to
 *             the caller of hunt that the obtained process ID is a "private"
 *             ID. The process ID is propagated to the caller of hunt in the
 *             name_ variable. This scheme allows the link handler to create
 *             new instances of a service process for each hunt that is issued,
 *             using the path name merely as a request for some service.
 *
 * @long  Hunt calls already pending within the OSE kernel, matching the
 *        new link path, are sent to the new link handler.
 *
 * @long  When a hunt matches a block link, the kernel handles the hunt
 *        internally, and resolves it if there is a process within the block
 *        with a given name matching the part of the hunt path that follows
 *        the link path and a slash. E.g. a hunt for "server12/main" will
 *        resolve to the process "main" in a block with the block link
 *        "server12".
 *
 * @long  Blocks that have an assigned block link are isolated name spaces.
 *        The given names of the contained processes are not visible from
 *        outside the block; only the full name comprising of link name +
 *        "/" + given name. The given names are visible from inside the block.
 *        The main purpose of block links is to allow multiple instances of
 *        the same program to present unique names.
 *
 * @long  A new block inherits the block link name from its parent block if
 *        the parent block has been assigned a block link.
 *
 * @input pathname   The link name that shall be assigned.
 * @input handler    The process ID of the link handler process or block ID
 *                   for a block link.
 * @return           Returns a non-zero value if the call is rejected due to
 *                   error conditions.
 * @restriction      A block link cannot be assigned after the first process
 *                   in the block has been created.
 * @break            It is not possible to assign multiple block links to a
 *                   single block.
 * @break            There is no designated search order if the same link name
 *                   is assigned multiple times.
 * @break            There is no way to deregister a link name. It is in
 *                   effect until the process or the block is killed.
 * @break            A deadlock problem occurs when a link handler
 *                   issues a system call that results in a system request
 *                   towards the same link handler. This problem is easily
 *                   avoided if link handlers are designed to never use system
 *                   calls which can result in requests towards link
 *                   handlers. If this is not possible, such operations should
 *                   be deferred to other processes related to the link handler.
 *
 * @seealso hunt
 * @seealso hunt_from
 * @seealso default_restrictions_calling_functions
 * @example
 * #include "stddef.h"
 * #include "ose_spi/ose_spi.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   char *linkh = "linkhandler";
 *   extern PROCESS linkhand_;
 *   OSBOOLEAN check;
 *   PROCESS proc_id_;
 *
 *   check = assign_linkhandler(linkh,linkhand_);
 *   if (check)
 *     {
 *       // If a non-zero value is returned in check, 
 *       // the call failed.
 *     }
 *
 *   for(;;)
 *     {
 *       check = hunt("linkhandler/proc", 0, &proc_id_, NULL);
 *       if(check)
 *         {
 *           // Code //
 *         }
 *     }
 * }
 */
OSBOOLEAN
assign_linkhandler(const char *pathname, PROCESS handler);

/**
 * @function attach_segment
 * @private
 * @brief Undocumented.
 */
PROCESS
attach_segment(PROCESS pid, OSSEGMENT segid);

/**
 * @function create_pool
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration void
 * create_pool(PROCESS bid, OSADDRESS base, OSADDRESS size,
 * 	    const OSADDRESS sigsize_tab[], const OSADDRESS stacksize_tab[]);
 *
 * @brief Create a pool.
 * @long  This call creates a new pool and attaches it to the specified
 *        block. Using this call, pools in the core module are typically
 *        created by the OSE start code and by the kernel itself.
 *        Program pools are typically created by the program manager.
 * @long  Pool space should be allocated by the memory manager and is
 *        made known to the executive by issuing this call. The pool is
 *        entirely managed by the executive until all blocks using the
 *        pool are dead. At that point, pool space may be reclaimed by
 *        the memory manager without further notice to the executive.
 * @long  The first pool created is the system pool. The system pool is
 *        shared by all supervisor processes and those user processes that
 *        reside in the kernel address space.
 * @long  Other pools are local pools to be used only by the specified block
 *        and its children. (It is possible to have local pools in kernel
 *        space too, which may be a useful feature when separately linked
 *        blocks are utilized in a system with no memory protection hardware.)
 * @long  In systems with simple memory protection hardware (fence registers)
 *        it may be useful to have local pools assigned only for stacks. In
 *        this way achieving a system with improved security without
 *        affecting performance.
 * @long  Sigsize_tab[] is an array of signal buffer sizes, sorted in
 *        ascending order. The first location contains the number of entries
 *        in the array, excluding the count itself (max 8). If count = 0
 *        then signals are allocated from the pool inherited from the creator
 *        or the pool already created for signals.
 * @long  Stacksize_tab[] is an array of stack buffer sizes, sorted in
 *        ascending order. The first location counts the number of entries
 *        (max 8). If count = 0 then stacks are allocated from the pool
 *        inherited from the creator or the pool already created for stacks.
 *
 * @input bid             The ID of the specified block (typically newly
 *                        created) to which the new pool shall be attached.
 * @input base            The lowest address in the pool.
 * @input size            The number of bytes reserved for the pool.
 * @input sigsize_tab     An array of all the available buffer sizes in the
 *                        pool. See the description above.
 * @input stacksize_tab   An array of all the available stack buffer sizes
 *                        in the pool. See the description above.
 *
 * @restriction Create_pool must be called before the first process
 *              in the block is created, or not called at all.
 *      @break  The get_pid_list and get_bid_list system calls require that:
 *              (max buffer size) > (max number of processes * sizeof(PROCESS)).
 *              It is also wise to set the largest buffer size to a large value,
 *              like 65535, which is the largest portable OSE buffer size.
 *
 * @seealso alloc
 * @seealso free_buf
 * @seealso default_restrictions_calling_functions
 */
void
create_pool(PROCESS bid, OSADDRESS base, OSADDRESS size,
	    const OSADDRESS sigsize_tab[], const OSADDRESS stacksize_tab[]);

/**
 * @function flush
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration void
 * flush(PROCESS *psel, PROCESS pid);
 *
 * @brief Remove signals from a queue.
 * @long  The flush call throws away all signals sent by any of a set of
 *        processes specified in psel, from the signal queue of the
 *        process specified in 'pid'.
 * @long  The psel parameter is an array of process IDs. The first location
 *        contains the number of valid entries in the rest of the array.
 * @long  The flush call may be useful when a process is unexpectedly killed,
 *        and other processes involved need to clear any pending transactions
 *        towards the killed process.
 *
 * @input psel   A pointer to an array of processes to flush signals from.
 * @input pid    The process ID of the receiving process.
 * @restriction  The psel array must be built in run-time since process
 *               identities are unknown at compile time.
 * @break        The first location in psel contains the number of entries in
 *               the rest of the array. Since the PROCESS type may not be
 *               compatible to an integer, the count should be cast to the
 *               PROCESS type when building psel.
 *               Ex: psel[0] = (PROCESS) 1;
 * @break        It is illegal to flush the queue of an interrupt or
 *               timer-interrupt process.
 *
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * extern PROCESS proc1_;
 * extern PROCESS proc2_;
 *
 * OS_PROCESS(my_process)
 * {
 *   PROCESS flush_array[2];
 *
 *   flush_array[0] = (PROCESS) 1;
 *   flush_array[1] = proc1_;
 *
 *   for (;;)
 *     {
 *       // Code //
 *
 *       // Throw away all signals sent from proc1 to proc2//
 *       flush(flush_array, proc2_);
 *       // Code //
 *     }
 * }
 */
void
flush(PROCESS *psel, PROCESS pid);

/**
 * @function get_mem
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSBOOLEAN
 * get_mem(PROCESS id, OSADDRESS from, void *to, OSADDRESS size);
 *
 * @brief Read memory contents anywhere.
 * @long  Reads a specified number of bytes from the address space of the
 *        specified process or block.
 * @long  This call is mainly used by debuggers to examine a program being
 *        debugged.
 *
 * @input id	The ID of the specified block or process.
 * @input from	The memory address to read from.
 * @output to	A pointer to the destination buffer.
 * @input size	The number of bytes to read.
 * @return	Returns a non-zero value if an error occurred. Either the
 * 		specified process/block could not be found, or the requested
 * 		size could not be transferred.
 * @restriction	The caller must be aware of the problems that occur when a
 *              program on another CPU type is examined. In particular if that
 *              CPU has another word length.
 *
 * @seealso set_mem
 * @seealso default_restrictions_calling_functions
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * // The declarations of signal numbers and structs are
 * // preferably made in .sig files which are then #included.
 * #define MEM 33
 * struct Mem
 * {
 *   SIGSELECT sig_no;
 *   OSADDRESS address;
 * };
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 *   struct Mem mem;
 * };
 *
 * OS_PROCESS(my_process)
 * {
 *   char to_array[10];
 *   OSADDRESS array;
 *   OSBOOLEAN status;
 *   union SIGNAL *sig;
 *   static const SIGSELECT mem_sig[] = {1, MEM};
 *
 *   for(;;)
 *     {
 *       // Code //
 *       sig = receive(mem_sig);
 *       array = sig->mem.address;
 *       success = get_mem(sender(&sig),
 *                         array,
 *                         to_array,
 *                         sizeof(to_array));
 *
 *       if(status)
 *         {
 *           // Code, Not a successful memory move //
 *         }
 *       else
 *         {
 *           // Code, The memory move was successful //
 *         }
 *       // Code //
 *     }
 * }
 */
OSBOOLEAN
get_mem(PROCESS id, OSADDRESS from, void *to, OSADDRESS size);

/**
 * @function get_pid_info
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSADDRESS
 * get_pid_info(PROCESS pid, OSADDRESS info_tag);
 *
 * @brief Get information about a process, block, pool or segment.
 * @long  Extract one detailed piece of information about an entity that can be
 *        addressed using a process ID.
 * @long  The 'pid' parameter is used to specify which process, block, pool or
 *        segment to extract the information from.
 * @long  The info_tag parameter should contain a number which specifies what
 *        information to extract. Some of the info_tag numbers does not
 *        require any specific PID argument.
 * @long  Below is a list of supported tags and their return values.
 * @break The PID column contains info about what kind of ID is accepted for
 *        that info_tag. (P)rocess, (b)lock, (p)ool, (s)egment, (c)pu number
 *        and finally n/a which means that the PID value is ignored. This
 *        means that "Pbp" should be interpreted as: processes, blocks and
 *        pools are accepted but segments are not.
 * @break
 * @tab PID  @tab INFO_TAG @tab RETURNS
 * @break
 * @tab n/a  @tab OSE_PI_LAST_SUPPORTED @tab Tag number of highest supported info_tag
 *                                           number.
 * @break
 * @tab x    @tab OSE_PI_SUPPORTED      @tab A non-zero value if the info_tag x is
 *                                           supported.
 * @break
 * @tab Pbps @tab OSE_PI_PTYPE          @tab The value 4097 for a pool ID, the value
 *                                           4098 for a segment ID, for other IDs it
 *                                           will return the same value as the
 *                                           @link "ose.h" "get_ptype" system call.
 * @break
 * @tab Pb   @tab OSE_PI_SUPERVISOR     @tab A non-zero value for supervisor-mode
 *                                           processes and supervisor-mode blocks.
 * @break
 * @tab Pb   @tab OSE_PI_CPU_ID         @tab The CPU identifier of the process or
 *                                           block. Use PID zero for current process.
 * @break
 * @tab Pb   @tab OSE_PI_NAME           @tab A buffer obtained from the caller's pool
 *                                           containing the name of the process or
 *                                           block. This buffer must be freed with
 *                                           free_buf by the caller. May return zero
 *                                           if the PID is illegal or dead.
 * @break
 * @tab P    @tab OSE_PI_SIGS_IN_QUEUE  @tab The number of signals currently in the
 *                                           signal queue of the process.
 * @break
 * @tab p    @tab OSE_PI_POOL_SIZE      @tab The current size of the pool.
 * @break
 * @tab p    @tab OSE_PI_POOL_MAX_SIZE  @tab The maximum size of the pool (zero if
 *                                           not set).
 * @break
 * @tab p    @tab OSE_PI_POOL_FRAG_SIZE @tab The extend fragment size of the pool
 *                                           (zero when not known by the kernel).
 * @break
 * @tab c  @tab OSE_PI_CURR_NONINT_PROC @tab The PID of the current non
 *                                           interrupt or timer process, on
 *                                           the CPU pointed out by the pid
 *                                           parameter.
 * @break
 * @tab c    @tab OSE_PI_LAST_PROC      @tab The PID of the background or 
 *                                           prioritized process that was last
 *                                           swapped out, on the CPU pointed
 *                                           out by the pid parameter.
 * @break
 * @tab c    @tab OSE_PI_CURR_PROC      @tab The PID of the current running
 *                                           process, on the CPU pointed out by
 *                                           the pid parameter.
 * @break
 * @input pid       The ID of the specified process, block, pool or segment.
 * @input info_tag  The number specifying what info to extract.
 * @return          Returns the extracted information or zero for not
 *                  supported info_tag numbers.
 *
 * @seealso get_pcb
 * @seealso get_poolcb
 * @seealso get_ptype
 * @seealso get_sig_poolid
 * @seealso get_stk_poolid
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stdio.h"
 * #include "ose_spi/ose_spi.h"
 *
 * extern PROOCESS he;
 *
 * void what_cpu(void)
 * {
 *   printf("I am running on CPU: %u...\n", get_pid_info(0, OSE_PI_CPU_ID));
 *   printf("And he is running on CPU: %u...\n", get_pid_info(he, OSE_PI_CPU_ID));
 * }
 */
OSADDRESS
get_pid_info(PROCESS pid, OSADDRESS info_tag);

/**
 * @function get_sig_info
 * @include "ose_spi/ose_spi.h"
 * @include "ose_i.h"
 *
 * @declaration OSBOOLEAN
 * get_sig_info(struct OS_sig_info *sig_info, PROCESS pool_id, PROCESS pid,
 * 	     OSADDRESS sig);
 *
 * @brief Get information about a signal buffer.
 * @long  Extracts detailed information about a signal buffer and searches
 *        for the next buffer owned by a specified process in a specified pool.
 * @long  The sig_info parameter should contain a pointer to a result buffer.
 *        It is updated with information about the signal buffer pointed out
 *        by the sig parameter. Set sig_info to NULL if not wanting the
 *        information.
 * @long  The 'pool_id' and 'pid' parameters are used to specify how to search
 *        for the next buffer. The pool_id parameter specifies which pool to
 *        search. A value of zero allows the search to continue in any pool.
 *        The 'pid' parameter, if non-zero, specifies that only buffers owned
 *        by that process should be searched for. A value of zero allows any
 *        owner including buffers owned by the system.
 * @long  As a special case, the pool_id parameter may be set to a block ID
 *        to specify the signal pool associated with that block.
 * @long  The 'sig' parameter contains the address of the signal buffer to
 *        check. Set it to zero to request a search for the first signal in
 *        the first pool that matches the 'pool_id' and 'pid' parameters.
 *        Then use the value of the "next" field in the sig_info structure
 *        for subsequent calls until the next field becomes zero. In this way
 *        it is possible to traverse all signals in a pool.
 *
 * @input sig_info   A pointer to a result buffer.
 * @input pool_id    The ID of the pool in which to search.
 * @input pid        Search only for signals owned by the process with this ID.
 * @input sig        A pointer to signal to examine.
 * @return           Returns a status code with either of these values:
 *
 * @break
 * @tab 0 @tab Unimplemented system call. No result is obtained.
 * @break
 * @tab 1 @tab The sig parameter points to a valid signal buffer.
 * @break
 * @tab 2 @tab The sig parameter points to a signal buffer which seems to
 * be OK, apart from the fact that it lacks a valid endmark.
 * @break
 * @tab 3 @tab The sig parameter points to a signal buffer with a broken
 * (somehow faulty) signal administration block.
 * @break
 * @tab 4 @tab The sig parameter is a wild pointer (not pointing at any
 * signal buffer) inside a pool.
 * @break
 * @tab 5 @tab The sig parameter is a wild pointer not inside any pool.
 * @break  If not authorized to look at the specified signal buffer,
 *         the result will be the same as calling
 *         get_sig_info with a wild signal pointer value in the sig
 *         parameter (e.g. 4).
 * @break  If the sig_info parameter is not NULL it should contain a
 *         pointer to a OS_sig_info structure which will be updated with
 *         information about the signal buffer pointed out by the sig
 *         parameter. The structure is described by
 *         struct @link "ose_spi/ose_spi.h" "OS_sig_info"
 * @restriction  Fields related to the sig parameter can be trusted only
 *               if the get_sig_info call returns 1 or 2. The next field
 *               can be trusted only if the get_sig_info call returns
 *               1 or 2, or if the sig parameter was set to zero, in which
 *               case get_sig_info returns 5.
 * @break  It is unwise to call get_sig_info for a process that is not
 *         stopped or intercepted, since signal pointers age rapidly.
 *
 * @seealso OS_sig_info
 * @seealso get_pool_list
 * @seealso get_poolcb
 * @seealso get_sig_poolid
 * @seealso get_stk_poolid
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stdio.h"
 * #include "ose_spi/ose_spi.h"
 *
 * void list_my_sigs(void)
 * {
 *   struct OS_sig_info sig_info;
 *   PROCESS curr = current_process();
 *   PROCESS sig_pool = get_sig_poolid(curr);
 *   OSBOOLEAN status = get_sig_info(&sig_info,sig_pool,curr,0);
 *   if (status == 5 && sig_info.next)
 *     {
 *       do
 *         {
 *           OSADDRESS sigptr = sig_info.next;
 *           status = get_sig_info(&sig_info, sig_pool, curr, sig_info.next);
 *           printf("sigptr........: %#lx \n", sigptr);
 *           switch (status)
 *             {
 *             case 1:
 *               break;
 *             case 2:
 *               printf("No valid endmark found\n");
 *               break;
 *             case 3:
 *               printf("The signal's administration block is faulty\n");
 *               return; // Not safe to continue //
 *             case 4:
 *               printf("The signal's address is wild (inside a pool)\n");
 *               return; // Not safe to continue //
 *             case 5:
 *               printf("The signal's address is wild (not inside any pool)\n");
 *               return; // Not safe to continue //
 *             default:
 *               return;
 *             }
 *           printf(" sig_no.......: %#lx\n", sig_info.sig_no);
 *           printf(" owner........: %#lx\n", sig_info.owner);
 *           printf(" sender_pid...: %#lx\n", sig_info.sender_pid);
 *           printf(" addressee_pid: %#lx\n", sig_info.addressee_pid);
 *           printf(" sig_size.....: %#lx\n", sig_info.sig_size);
 *           printf(" size_in_pool.: %#lx\n", sig_info.size_in_pool);
 *         } while (sig_info.next);
 *     }
 * }
 */
OSBOOLEAN
get_sig_info(struct OS_sig_info *sig_info, PROCESS pool_id, PROCESS pid,
	     OSADDRESS sig);

/**
 * @function get_signal
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSBOOLEAN
 * get_signal(PROCESS id, OSADDRESS mailptr, union SIGNAL **sig_copy,
 * 	   OSADDRESS *nextsig);
 *
 * @brief Get a copy of a signal buffer.
 * @long  This is a debugger call.
 * @long  Returns in sig_copy a copy of the signal located in the signal
 *        queue or attach list for the specified process or block.
 *        (Only processes have signal queues.)
 * @long  The signal copy is returned in a signal buffer obtained by
 *        get_signal. This buffer can be examined with the sender,
 *        sigsize and addressee system calls. The buffer must be freed
 *        with free_buf by the caller when information has been extracted.
 * @long  mailptr is a signal queue pointer obtained from get_pcb or from a
 *        previous get_signal call. (It is not possible to evaluate these
 *        pointers with get_mem since signal buffers are not always stored
 *        in the address space of the addressee process.)
 * @long  The signal structure itself is not altered in any way, just
 *        copied byte by byte. It is important to understand this since
 *        sizeof(SIGSELECT) as well as other signal data may vary between
 *        systems. Thus the caller must realize that if the examined process
 *        runs on another processor, signal data must be interpreted according
 *        to the conventions used in that processor.
 * @long  A pointer to the next signal in the list is returned in nextsig.
 *        A zero value is placed in nextsig if there are no more signals in
 *        the list. Nextsig may be used as input to another get_sig call.
 *
 * @input id        The ID of the specified block or process.
 * @input mailptr   A pointer to the signal queue.
 * @input sig_copy  A pointer to a pointer to the signal buffer in which the
 *                  signal is returned.
 * @input nextsig   A pointer to the next signal in the queue.
 * @return          Returns a non-zero value if the call failed.
 *                  This occurs for instance if the signal queue of the
 *                  specified process has been altered by an intermediate call
 *                  to receive, flush, kill_proc or any other system call that
 *                  alters the contents of the relevant buffer lists. When this
 *                  happens, the list must be re-examined from the top,
 *                  e.g. a new list head must be obtained from get_pcb.
 * @restriction     It is unwise to call get_signal (as well as get_pcb) for a
 *                  process that is not stopped or intercepted, since signal
 *                  pointers age rapidly.
 *
 * @seealso get_pcb
 * @seealso default_restrictions_calling_functions
 * @example
 * #include "stdio.h"
 * #include "ose_spi/ose_spi.h"
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * void list_signal_queue(PROCESS proc_)
 * {
 *   struct OS_pcb *pcbbuf;
 *   OSADDRESS sigptr;
 *   union SIGNAL *sig;
 *   OSADDRESS next;
 *   OSBOOLEAN nosuccess;
 *
 *   printf("Signal queue of process %lu:\n", proc_);
 *
 *   // Stop the process and get a copy of the first signal in the queue.
 *   stop(proc_);
 *   do
 *     {
 *       pcbbuf = get_pcb(proc_);
 *       if(pcbbuf->type != OS_PRI_PROC && pcbbuf->type != OS_BG_PROC)
 *         {
 *           // Neither a prioritized nor background process.
 *           free_buf((union SIGNAL **)&pcbbuf);
 *           start(proc_);
 *           return;
 *         }
 *
 *       sigptr = pcbbuf->sigqueue;
 *       free_buf((union SIGNAL **)&pcbbuf);
 *
 *       if(NULL == sigptr)
 *         {
 *           // The queue was empty.
 *           start(proc_);
 *           return;
 *         }
 *
 *       nosuccess = get_signal(proc_, sigptr, &sig, &next);
 *       // If nosuccess is true, the process has either been killed
 *       // or the signal queue has been manipulated by flush() since
 *       // the get_pcb() call. The get_pcb() call needs to be redone.
 *     } while(nosuccess);
 *
 *   // Print each signal in the queue.
 *   for(;;)
 *     {
 *       printf("sig_no: %lu size: %lu sender: 0x%lx\n",
 *              sig->sig_no, sigsize(&sig), sender(&sig));
 *       free_buf(&sig);
 *       if(next == 0) break;
 *       get_signal(proc_, next, &sig, &next);
 *     }
 *
 *   start(proc_);
 * }
 */
OSBOOLEAN
get_signal(PROCESS id, OSADDRESS mailptr, union SIGNAL **sig_copy,
	   OSADDRESS *nextsig);

/**
 * @function get_uid
 * @private
 * @brief Undocumented.
 */
OSUSER
get_uid(PROCESS id);

/**
 * @function intercept
 * @include "ose_spi/ose_spi.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * intercept(PROCESS id);
 *
 * @brief Intercept a process.
 * @long  This system call is mainly used by debuggers.
 * @long  The intercept call stops a process as if a debugger breakpoint was
 *        reached.
 * @long  Status for the specified process or for all processes in the
 *        specified block is changed to intercepted. This means that
 *        processes are permanently swapped out until allowed to continue with
 *        the resume call.
 * @long  There are no intercept counters in the system. Intercepting a process
 *        that is already intercepted performs no operation.
 *
 * @input id      The ID of the specified block or process.
 * @restriction   Only prioritized, background and timer-interrupt processes
 *                can be intercepted. The intercept and resume calls have no
 *                effect on process types that can not be intercepted.
 * @break         This call is also available in interupt and timer interrupt
 *                processes.
 *
 * @seealso resume
 * @seealso get_pcb
 * @seealso alloc
 * @seealso default_restrictions_calling_functions
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * extern PROCESS proc_;
 *
 * OS_PROCESS(my_process)
 * {
 *   for(;;)
 *     {
 *       // Some code //
 *       intercept(proc_);
 *       // Some more code //
 *       resume(proc_);
 *     }
 * }
 */
void
intercept(PROCESS id);

/**
 * @function ose_bind_process
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSBOOLEAN
 * ose_bind_process(PROCESS id, cpuid_t cpu);
 *
 * @brief Bind a process, block or domain to the specified CPU.
 * @long  A process is implicitly bound to the same CPU as its block at
 *        creation time. This system call can be used to rebind (move)
 *        processes and blocks even after they have been started.
 * @long  If a block is specified, then all processes in that block will be
 *        bound to the specified CPU together with the block itself.
 * @long  If a single process is specified, then only that process will be 
 *        bound.
 * @long  This system call can also be used to bind an entire domain. In this
 *        case all blocks and their processes will be bound.
 * @long  All processes addressed by the ID parameter will be stopped until
 *        ose_bind_process returns.
 * @long  This system call requires actions by the system daemon (ose_sysd) 
 *        that typically has a different priority than the caller. The caller
 *        will be blocked (swapped out) until the system daemon has finished.
 * @long  This system call is only effective in SMP (Symmetric Multi
 *        Processing) systems.
 * @long  A program may be locked to a certain CPU by its program 
 *        configuration. The monolith program is locked to CPU zero.
 *        Generally, it is not permitted to migrate a node locked 
 *        program or parts of it, with one exception: migration of a 
 *        single process in the same block as the calling client. This 
 *        implies that a supervisor mode process can migrate itself.
 * @long  To create and populate a block on another CPU, temporarily
 *        bind the current process to that CPU before creating the block.
 *        The new block inherits the CPU attribute from the creating process.
 *
 * @input id 	The ID of the specified block, process or domain.
 * @input cpu	Identifies the CPU to which the processes / blocks will be
 *              bound.
 * @return 	Returns a non-zero value if one or more processes or blocks was
 * 		successfully bound.
 * @restriction	The 'cpu' parameter must be a number n,
 *              in the range 0 <= n < ose_num_cpus().
 * @break       Only available in supervisor mode processes.
 * @break       The system call will not migrate an interrupt process 
 *              connected to a HW interrupt vector.
 *
 * @seealso cpuid_t
 * @seealso ose_cpu_id
 * @seealso ose_num_cpus
 * @seealso default_restrictions_calling_functions
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * OS_PROCESS(nomad)
 * {
 *
 *   for(;;)
 *     {
 *       ose_bind_process(current_process(), ose_cpu_id()+1 % ose_num_cpus());
 *       printf("I have moved to another home in CPU %u...\n, ose_cpu_id());
 *       delay(10000);
 *     }
 * }
 */

OSBOOLEAN
ose_bind_process(PROCESS id, cpuid_t cpu);

/**
 * @function resume
 * @include "ose_spi/ose_spi.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * resume(PROCESS id);
 *
 * @brief Resume an intercepted process.
 * @long  This system call is mainly used by debuggers.
 * @long  Restart processes previously stopped by the intercept call or by
 *        a debugger breakpoint.
 *        Resume may also be called from interrupt and timer-interrupt
 *        processes.
 * @long  Intercept status for the specified process or for all processes
 *        in the specified block is removed. This means that the process(es)
 *        are allowed to continue operation as implied by the state present
 *        before they were intercepted.
 * @long  There are no intercept counters in the system. Resuming a process
 *        that is not intercepted performs no operation.
 *
 * @input id 	The ID of the specified process.
 * @restriction Only prioritized, background and timer-interrupt processes
 *              can be intercepted. The intercept and resume calls have no
 *              effect on process types that can not be intercepted.
 *
 * @seealso intercept
 * @seealso get_pcb
 * @seealso default_restrictions_calling_functions
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * extern PROCESS proc_;
 *
 * OS_PROCESS(my_process)
 * {
 *   for(;;)
 *     {
 *       // Some code //
 *       intercept(proc_);
 *       // Some more code //
 *       resume(proc_);
 *     }
 * }
 */
void
resume(PROCESS id);

/**
 * @function set_mem
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSBOOLEAN
 * set_mem(PROCESS id, void *from, OSADDRESS to, OSADDRESS size);
 *
 * @brief Set memory contents anywhere.
 * @long  This call is mainly used by memory managers, program loaders and
 *        debuggers to load and modify a program.
 * @long  Writes a specified number of bytes from the address space of the
 *        caller to the address space of the specified process or block.
 *
 * @input id 	The ID of the specified block or process.
 * @input from 	A pointer to the address to read from.
 * @input to 	The address to write to.
 * @input size 	The number of bytes to read.
 * @return 	Returns a non-zero value if an error occurred. Either the
 * 		specified process/block could not be found, or the requested
 *              size could not be transferred.
 * @restriction	The caller must be aware of the problems that occur when a
 *              program on another CPU type is loaded or examined, in
 *              particular if that CPU has another word length. Link handler
 *              conventions should be developed to resolve this.
 *
 * @seealso get_mem
 * @seealso default_restrictions_calling_functions
 */
OSBOOLEAN
set_mem(PROCESS id, void *from, OSADDRESS to, OSADDRESS size);

/**
 * @function set_halt_condition
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSTIME
 * set_halt_condition(OSTIME threshold_time);
 *
 * @brief Set power save threshold.
 * @long  set_halt_condition sets up a timing condition to control the
 *        kernels use of a user provided power save handler.
 * @long  If the system is idle and the kernel can not find any timer
 *        events within the stated threshold_time, it will call the power
 *        save handler.
 *
 * @input threshold_time Minimum time in milliseconds to next planned timer
 * 			 event.
 * @break       0 will activate power save each time the system is idle.
 * @break       -1 will disable the power save functionality.
 * @return      Previous setting of threshold_time.
 * @restriction Only available in supervisor mode processes.
 *
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * OS_PROCESS(power_save_manager)
 * {
 *   OSTIME old_threshold;
 *
 *   for(;;)
 *     {
 *       "...wait for some event...";
 *       // Enable power save whenever idle state is reached //
 *       old_threshold = set_halt_condition(0);
 *
 *       "... wait for some other event...";
 *       // Restore previous power save condition setting //
 *       (void) set_halt_condition(old_threshold);
 *     }
 * }
 */
OSTIME
set_halt_condition(OSTIME threshold_time);

/**
 * @function set_pcb
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSBOOLEAN
 * set_pcb(PROCESS pid, const char *cpuregs);
 *
 * @brief Set CPU registers for a process.
 * @long  Sets the CPU registers of the specified process if the process is
 *        in a state where this is possible.
 * @long  The set_pcb call is mainly used by debuggers.
 * @long  cpuregs contains a string describing the registers to be set.
 *        The string format corresponds to the format used in the cpuregs
 *        string in the get_pcb call, except registers may be in any order
 *        and not all registers need to be present.
 *
 * @input pid 		The ID of the specified process.
 * @input cpuregs 	A pointer to a string describing the registers.
 * @return 		Returns zero if the call was successful.
 * 			Returns a non-zero value if the call failed because the
 *			process is not in a state where the CPU registers can be
 *			set, or the string was in a format not recognized.
 * @restriction	    Can only be applied to prioritized and background
 *                  processes.
 * @break 	    OSE implementations are sensitive to the format of the
 *                  cpuregs string. For example, in an assignment of a CPU
 *                  register, the number of digits of the register value may
 *                  be crucial for the success of the assignment:
 * 		    "PC=0" may not succeed whilst "PC=00000000" would. Take
 *                  care not to violate the rules provided by each processor
 *                  architecture specific OSE implementation.
 *
 * @seealso get_pcb
 * @seealso default_restrictions_calling_functions
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * extern PROCESS proc_;
 *
 * OS_PROCESS(my_process)
 * {
 *   for(;;)
 *     {
 *       // Code //
 *       if(set_pcb(proc_, "A0=000037F5"))
 *         {
 *           error(17);// Set_pcb call was not successful //
 *         }
 *       // Code //
 *     }
 * }
 */
OSBOOLEAN
set_pcb(PROCESS pid, const char *cpuregs);

/**
 * @function set_segment
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSBOOLEAN
 * set_segment(OSBOOLEAN go_home, OSSEGMENT segnum);
 *
 * @brief Set the effective segment number for the calling process.
 * @long  Changes the memory mapping for the calling process to that of the
 *        selected segment.
 * @long  The real segment identifier of the caller is not affected. The real
 *        segment is always the "home" segment of the process.
 * @long  The selected segment is attached to the calling process, so that the
 *        segment can not be deallocated until another segment is selected,
 *        or until the process is killed.
 *
 * @input go_home   The home segment is reselected by a call with the
 *	            go_home flag set to a non-zero value, in which case
 *                  the segnum parameter is ignored.
 * @input segnum    The segment number to visit.
 * @return          Returns zero if the call was successful.
 *                  Returns a non-zero value if the call failed.
 *                  It fails if the segment is not found.
 * @restriction     User mode processes will fail to select other than the home
 *                  segment.
 * @seealso default_restrictions_calling_functions
 */
OSBOOLEAN
set_segment(OSBOOLEAN go_home, OSSEGMENT segnum);

/**
 * @function set_suid
 * @private
 * @brief Undocumented.
 */
void
set_suid(OSBOOLEAN make_super);

/**
 * @function start_OSE
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration void
 * start_OSE(void);
 *
 * @brief Start OSE.
 * @long  This function works in two ways.
 * @long  start_OSE is called during system start to create the kernel and
 *        initialize kernel structures. This function does not return to the
 *  	  caller since the kernel replaces the caller's program.
 * @long  start_OSE is also called by the main process for each separately
 *        linked OSE unit to create the static blocks and processes contained
 *        in that unit. The main process does not return. It waits in the
 *        interface library for any static process to die, and then kills all
 *        blocks in the unit. This behavior relies on the convention that
 *        program loaders keep track of the main process, and kill the domain
 *        when the main process dies. Other conventions are possible and may
 *        appear in certain systems.
 *
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * int main()
 * {
 *   // Code, initialization etc. //
 *   start_OSE();
 * }
 */
void
start_OSE(void);

/**
 * @function tick
 * @include "ose_spi/ose_spi.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * tick(void);
 *
 * @brief Advance system clock.
 * @long  The system tick counter is advanced, timer-interrupt processes are
 *        invoked, process timeslices and timeouts requested by calls to delay,
 *        receive_w_tmo et al. are managed.
 * @long  Timer-interrupt processes are run at the priority of the calling
 *        interrupt process, or at a priority immediately below interrupt
 *        processes if tick was called from a prioritized process.
 *
 * @restriction  Tick should be called periodically at intervals that match
 *               the value returned by system_tick().
 * @break        The tick system call is not re-entrant. Only one process in
 *               the system may call tick.
 * @break        Tick may not be called from a timer-interrupt process.
 * @break        Tick may only be called from a process executing in supervisor
 *               space.
 * @break        This system call may enable interrupts if it is called
 *               with interrupts disabled.
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * // This is an interrupt process used as a timer.
 * // Its only purpose is to run the system clock.
 * OS_PROCESS(ticker)
 * {
 *   if(wake_up())
 *     {
 *       tick();
 *     }
 * }
 */
void
tick(void);

/**
 * @function ose_create_isr
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSISRHND
 * ose_create_isr(OSVECTOR vec, PROCESS owner, OSISR *isr, void *isr_arg);
 *
 * @brief Install an ISR.
 * @long  An ISR presents an alternative to interrupt processing from OSE's
 *        regular interrupt processes. Unlike an interrupt process, an ISR
 *        does not have its own context, and it runs with interrupts
 *        disabled. This avoids the overhead associated with context
 *        switching and interrupt source masking.
 * @long  It is possible to install more than one ISR per vector. When an
 *        interrupt occurs, all ISR functions will be called in an
 *        unspecified order.
 * @long  An ISR can return a value that indicates if the kernel should check
 *        if a process swap should be performed. If the ISR has not performed
 *        any action that would cause this, it can return directly to the
 *        interrupted context, which is more efficient. If more than one ISR
 *        is installed on a vector, it is sufficient that any of them
 *        requests this check for it to be performed.
 * @long  To request a process swap check, the ISR should return
 *        OSE_ISR_RESCHED. If no check is necessary, the ISR should instead
 *        return OSE_ISR_NO_RESCHED.
 * @long  In an SMP system, the ISR will be bound to the same CPU as the
 *        block of the process it is created from.
 *
 * @input vec     The vector number.
 * @input owner   The process (or block or segment) that controls the
 *                life span of the ISR. When the owner is killed, the ISR is
 *                uninstalled. Set to 0 to disable automatic uninstallation.
 * @input isr     The entry point of the ISR function.
 * @input isr_arg The argument that is passed to the ISR function.
 * @return        Returns a handle that can be used to uninstall the ISR.
 * @restriction   It is not possible to use both ISRs and interrupt processes
 *                to service the same vector.
 * @break         Only available in supervisor mode processes.
 *
 * @seealso OSISR
 * @seealso ose_kill_isr
 * @seealso ose_create_interrupt_hook
 * @seealso ose_disable_vector
 * @seealso ose_signal_fsem_from_isr
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * // The vector number will usually have to be determined dynamically
 * static OSVECTOR vector = 10;
 *
 * static unsigned long
 * my_isr(void *arg)
 * {
 *   // Service interrupt and perform EOI if needed
 * }
 *
 * OS_PROCESS(my_isr_installer)
 * {
 *   OSISRHND handle;
 *   handle = ose_create_isr(vector, get_bid(current_process()), &my_isr, NULL);
 *
 *   for(;;)
 *     {
 *       // Code //
 *       ose_kill_isr(handle);
 *     }
 * }
 */
OSISRHND
ose_create_isr(OSVECTOR vec, PROCESS owner, OSISR *isr, void *isr_arg);

/**
 * @function ose_kill_isr
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration void
 * ose_kill_isr(OSISRHND isr_hnd);
 *
 * @brief Uninstall an ISR.
 * @long  Uninstalls an ISR that was previously installed by
 *        ose_create_isr. Since an ISR can be automatically uninstalled,
 *        invalid handles will be silently ignored.
 *
 * @input isr_hnd   The ISR handle returned from ose_create_isr.
 * @restriction     Only available in supervisor mode processes.
 *
 * @seealso OSISRHND
 * @seealso ose_create_isr
 * @seealso default_restrictions_calling_functions
 */
void
ose_kill_isr(OSISRHND isr_hnd);

/**
 * @function ose_create_interrupt_hook
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSIHHND
 * ose_create_interrupt_hook(OSVECTOR vec, PROCESS owner, OSIH *ih, void *ih_arg);
 *
 * @brief Install an interrupt hook.
 * @long  An interrupt hook provides a method of running code when an
 *        interrupt occurs, without being the owner of the interrupt
 *        vector.
 * @long  The interrupt hook executes in vector handler context with
 *        interrupts disabled, before the interrupt process or ISRs installed
 *        on the vector are executed.
 * @long  It is possible to install more than one interrupt hook per
 *        vector. When an interrupt occurs, all interrupt hooks are called in
 *        an unspecified order.
 *
 * @input vec    The vector number.
 * @input owner  The process (or block or segment) that controls the
 *               life span of the interrupt hook. When the owner is killed, the
 *               interrupt hook is uninstalled. Set to 0 to disable automatic
 *               uninstallation.
 * @input ih     The entry point of the interrupt hook function.
 * @input ih_arg The argument that is passed to the interrupt hook function.
 * @restriction  Before installing an interrupt hook, the vector must already be
 *               initialized by either an interrupt process or an ISR.
 * @break        Only available in supervisor mode processes.
 *
 * @seealso OSIH
 * @seealso ose_kill_interrupt_hook
 * @seealso ose_create_isr
 * @seealso ose_disable_vector
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * // The vector number will usually have to be determined dynamically
 * static OSVECTOR vector = 10;
 *
 * static void
 * my_interrupt_hook(void *arg)
 * {
 *   // Perform useful activities
 * }
 *
 * OS_PROCESS(my_interrupt_hook_installer)
 * {
 *   OSIHHND handle;
 *   handle = ose_create_interrupt_hook(vector, get_bid(current_process()),
 *                                      &my_interrupt_hook, NULL);
 *
 *   for(;;)
 *     {
 *       // Code //
 *       ose_kill_interrupt_hook(handle);
 *     }
 * }
 */
OSIHHND
ose_create_interrupt_hook(OSVECTOR vec, PROCESS owner, OSIH *ih, void *ih_arg);

/**
 * @function ose_kill_interrupt_hook
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration void
 * ose_kill_interrupt_hook(OSIHHND ih_hnd);
 *
 * @brief Uninstall an interrupt hook.
 * @long  Uninstalls an interrupt hook that was previously installed by
 *        ose_create_interrupt_hook. Since an interrupt hook can be
 *        automatically uninstalled, invalid handles will be silently ignored.
 *
 * @input ih_hnd  The interrupt hook handle returned from
 *                ose_create_interrupt_hook.
 *
 * @seealso default_restrictions_calling_functions
 *
 * @restriction   Only available in supervisor mode processes.
 *
 * @seealso OSIHHND
 * @seealso ose_create_interrupt_hook
 */
void
ose_kill_interrupt_hook(OSIHHND ih_hnd);

/**
 * @function ose_enable_vector
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration void
 * ose_enable_vector(OSVECTOR vec);
 *
 * @brief Enable interrupts on a vector.
 * @long  This system call re-enables interrupts on a vector after a previous
 *        call to ose_disable_vector. See @link "" "ose_disable_vector" for
 *        more information.
 * @long  The calls are counting, so a number of calls to ose_disable_vector
 *        must be followed by an equal amount of calls to ose_enable_vector.
 *
 * @input vec   The vector number to enable.
 * @restriction Only available in supervisor mode.
 *   @break
 *              Only available in prioritized processes.
 *
 * @seealso ose_disable_vector
 * @seealso default_restrictions_calling_functions
 *
 */
void
ose_enable_vector(OSVECTOR vec);

/**
 * @function ose_disable_vector
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration void
 * ose_disable_vector(OSVECTOR vec);
 *
 * @brief Disable interrupts on a vector.
 * @long  This system call allows atomic installation of several ISRs or
 *        interrupt hooks. By surrounding the region of installation with
 *        calls to ose_disable_vector and ose_enable_vector, no interrupts
 *        will occur on the vector during the installation sequence.
 * @long  The calls are counting, so a number of calls to ose_disable_vector
 *        must be followed by an equal amount of calls to ose_enable_vector.
 * @long  In rare cases if it is necessary to quickly disable a specific
 *        interrupt source in low level interrupt mode code due to a large
 *        amount of spurious interrupts, it is possible to directly call the
 *        function registered as INT_KILL_HANDLER for this vector. It must be
 *        called on the core handling this interrupt.
 *        Using DDA, this function is typically bspIntKillHandler(vector, prio)
 *        where the priority is the priority of the interrupt process
 *        (or 0 if ISRs are used). This should be used with great care because
 *        it violates the state known by the OSE kernel; no interrupts on
 *        this vector are allowed after this. The system error handler might
 *        also receive the fatal OSE_EPRIORITY_ERROR due to interrupts on
 *        wrong priority between the INT_KILL_HANDLER call and the return from
 *        the low level interrupt code.
 *        No calls to ose_enable_vector or ose_disable_vector are allowed
 *        for this vector.
 *        The only allowed subsequent operation regarding that vector is to
 *        restore the state by calling the function registered as
 *        INT_CREATE_HANDLER directly, typically in DDA it is
 *        bspIntCreateHandler(vector,prio). INT_CREATE_HANDLER must be called
 *        on the correct core from either a prioritized supervisor mode process
 *        or from an interrupt process (not from ISR or interrupt hook level).
 *
 * @input vec    The vector number to disable.
 * @restriction  Only available in supervisor mode processes.
 * @break        Only available in prioritized processes.
 *
 * @seealso ose_enable_vector
 * @seealso ose_create_isr
 * @seealso ose_create_interrupt_hook
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * // The vector number will usually have to be determined dynamically
 * static OSVECTOR vector = 10;
 *
 * static unsigned long
 * my_isr(void *arg)
 * {
 *   // Service interrupt and perform EOI if needed
 * }
 *
 * static void
 * my_interrupt_hook(void *arg)
 * {
 *   // Perform useful activities
 * }
 *
 * OS_PROCESS(vector_setup)
 * {
 *   OSISRHND isr;
 *   OSIHHND ih;
 *
 *   ose_disable_vector(vector);
 *   isr = ose_create_isr(vector, 0, &my_isr, NULL);
 *   // No interrupt will be triggered yet
 *   ih = ose_create_interrupt_hook(vector, 0, &my_interrupt_hook, NULL);
 *   ose_enable_vector(vector);
 *   // The interrupt source may now start generating interrupts
 *
 *   for(;;)
 *   {
 *     // Code //
 *   }
 * }
 */
void
ose_disable_vector(OSVECTOR vec);

/**
 * @function ose_bind_vector
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration void
 * ose_bind_vector(OSVECTOR vec, cpuid_t cpu);
 *
 * @brief Bind an interrupt vector to the given CPU.
 * @long  This system call makes it possible to bind an interrupt vector to a
 *        new CPU in an SMP system. The vector is activated on the given CPU
 *        regardless of whether it was previously active on any other. This
 *        means that it is possible for the vector to service interrupts on
 *        more than one CPU.
 * @long  To move an interrupt vector from one CPU to another, it must be
 *        unbound from its original CPU and bound to the new one. To avoid
 *        having the vector bound on multiple CPUs in this case, which may
 *        require special hardware and driver support, perform the unbind
 *        operation first or surround the region with disable / enable vector.
 *
 * @input vec    The vector number to bind.
 * @input cpu    The CPU to bind the vector to.
 * @restriction  This call can only be used with interrupt vectors serviced
 *               by ISRs.
 *       @break  Binding an interrupt vector to more than one CPU may not be
 *               supported by the hardware or driver.
 *       @break  Only available in supervisor mode processes.
 *
 * @seealso ose_create_isr
 * @seealso ose_unbind_vector
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * // The vector number will usually have to be determined dynamically
 * static OSVECTOR vector = 10;
 *
 * static unsigned long
 * my_isr(void *arg)
 * {
 *   // Service interrupt and perform EOI if needed
 * }
 *
 * // Process running in a block bound to CPU 0
 * OS_PROCESS(vector_setup)
 * {
 *   OSISRHND isr;
 *
 *   ose_disable_vector(vector);
 *   // This call creates the ISR and binds it to the current CPU (0)
 *   isr = ose_create_isr(vector, 0, &my_isr, NULL);
 *   // Bind it to CPU 1 as well
 *   ose_bind_vector(vector, 1);
 *   ose_enable_vector(vector);
 *   // The interrupt source may now start generating interrupts on CPU 0 and 1
 *
 *   for(;;)
 *   {
 *     // Code //
 *   }
 * }
 */
void
ose_bind_vector(OSVECTOR vec, cpuid_t cpu);

/**
 * @function ose_unbind_vector
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration void
 * ose_unbind_vector(OSVECTOR vec, cpuid_t cpu);
 *
 * @brief Unbind an interrupt vector from the given CPU.
 * @long  This system call makes it possible to unbind an interrupt vector
 *        from a CPU in an SMP system.
 *
 * @input vec   The vector number to unbind.
 * @input cpu   The CPU to unbind the vector from.
 * @restriction This call can only be used with interrupt vectors serviced
 *              by ISRs.
 * @break       Only available in supervisor mode processes.
 *
 * @seealso ose_create_isr
 * @seealso ose_bind_vector
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * // The vector number will usually have to be determined dynamically
 * static OSVECTOR vector = 10;
 *
 * static unsigned long
 * my_isr(void *arg)
 * {
 *   // Service interrupt and perform EOI if needed
 * }
 *
 * // Process running in a block bound to CPU 0
 * OS_PROCESS(vector_setup)
 * {
 *   OSISRHND isr;
 *
 *   ose_disable_vector(vector);
 *   // This call creates the ISR and binds it to the current CPU (0)
 *   isr = ose_create_isr(vector, 0, &my_isr, NULL);
 *   // Bind it to CPU 1 as well
 *   ose_bind_vector(vector, 1);
 *   // Remove it from CPU 0 where it was bound at creation
 *   ose_unbind_vector(vector, 0);
 *   ose_enable_vector(vector);
 *   // The interrupt source may now start generating interrupts on CPU 1 only
 *
 *   for(;;)
 *   {
 *     // Code //
 *   }
 * }
 */
void
ose_unbind_vector(OSVECTOR vec, cpuid_t cpu);

/**
 * @function ose_signal_fsem_from_isr
 * @include "ose_spi.h"
 *
 * @declaration OSERRCODE
 * ose_signal_fsem_from_isr(PROCESS pid);
 *
 * @brief Signal a fast semaphore from ISR context.
 * @long  Increments the value of the fast semaphore associated with the
 *        specified process.
 * @long  If the result is zero, the specified process is made ready if it is
 *        waiting inside a wait_fsem or receive_fsem call.
 * @long  If the process is an interrupt process, it is scheduled.
 * @long  The ISR calling ose_signal_fsem_from_isr must return the value
 *        OSE_ISR_RESCHED for the rescheduling to be performed immediately.
 *
 * @input pid    The ID of the process for which the fast semaphore is
 *               signaled.
 * @return       Returns 0 on success, otherwise an error code.
 * @restriction  It is illegal to cause a fast semaphore to wrap from
 *               high positive numbers to a negative value for all processes
 *               but interrupt processes (these are allowed to wrap since the
 *               fast semaphore is only used to schedule the process).
 *      @break   This system call is only intended to be used in an ISR context.
 *               Use the regular signal_fsem call from a process context.
 *
 * @seealso signal_fsem
 * @seealso ose_create_isr
 * @example
 * #include "ose_spi.h"
 *
 * extern PROCESS proc_;
 *
 * unsigned long
 * my_isr(void *arg)
 * {
 *    OSBOOLEAN work_to_be_done_in_process_context = 0;
 *
 *    // Code //
 *
 *    if (work_to_be_done_in_process_context)
 *    {
 *       if (ose_signal_fsem_from_isr(proc_))
 *       {
 *          // Handle errors //
 *       }
 *       else
 *       {
 *          // Tell the kernel to check if proc_ should be scheduled //
 *          return OSE_ISR_RESCHED;
 *       }
 *    }
 *
 *    // No need to ask the kernel for a reschedule check //
 *    return OSE_ISR_NO_RESCHED;
 * }
 */
OSERRCODE
ose_signal_fsem_from_isr(PROCESS pid);

/**
 * @function ose_set_pool_max_size
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSADDRESS
 * ose_set_pool_max_size(PROCESS pool_id, OSADDRESS max_size,
 *                       OSADDRESS frag_size);
 *
 * @brief Change the maximum pool size and extend fragment size of a pool.
 * @long  These pool attributes may only be changed for pools that initially
 *        was created as extendable. Refer to the program parameter
 *        documentation of the program handler, and the configuration of
 *        system pool and main pool.
 * @long  Change of the fragment size is only possible for local program pools,
 *        not for the system pool and the main pool.
 * @long  If called from user mode process, the max pool size will not be
 *        changed but the return value will indicate current max size.
 *
 * @input pool_id    The identifier of a pool.
 * @input max_size   The wanted maximum size of the pool.
 * @input frag_size  The wanted extend fragment size of the pool. The
 *                   fragment size must be bigger than the largest signal
 *                   and stack objects allocated from the pool. The value
 *                   zero implies that the previous fragment size is
 *                   kept unchanged.
 * @return           Returns the maximum pool size after the call. If it
 *                   differs from the requested size, the size could not be
 *                   changed. Zero is returned if the pool is not found.
 * @restriction      The effective size limit may be rounded to the fragment
 *                   size.
 *
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose_spi.h"
 * #include "stdio.h"
 *
 * void
 * set_sig_pool_max_size(PROCESS block_id, OSADDRESS new_max)
 * {
 *   OSADDRESS result_size;
 *   PROCESS pool_id = get_sig_poolid(block_id);
 *
 *   result_size = ose_set_pool_max_size(pool_id, new_max, 0);
 *   if(result_size != new_max)
 *     {
 *       if(result_size == 0)
 *         {
 *           printf("The signal pool of block %#x was not found.\n",
 *                  block_id);
 *         }
 *       else
 *         {
 *           printf("Pool max size is %d, not the requested %d.\n",
 *                  result_size, new_max);
 *         }
 *     }
 * }
 */
OSADDRESS
ose_set_pool_max_size(PROCESS pool_id, OSADDRESS max_size,
                      OSADDRESS frag_size);

/**
 * @function ose_set_pool_attr
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSADDRESS
 * ose_set_pool_attr(PROCESS pool_id, const OSADDRESS *tag_list);
 *
 * @brief Set pool attributes on a selected pool.
 * @long
 * This function can be used to set different attributes on a pool.
 * @long
 * Below is a list of supported tags.
 * @break
 * @tab TAG  @tab DESCRIPTION
 * @break
 * @tab OSE_POOL_ATTR_TAGEND  @tab This tag marks the end of the taglist.
 * @break
 * @tab OSE_POOL_ATTR_NO_COPY @tab Disable copying of signals sent from
 *                                 this pool.
 * @break
 * @input pool_id The identifier of a pool.
 * @input taglist Attributes to be set on the selected pool.
 *                The tag list array is always ended with the
 *                OSE_POOL_ATTR_TAGEND tag.
 *
 * @return Returns the number of successfully set attributes.
 * @restriction   Only available in supervisor mode processes.
 *
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose_spi.h"
 * #include "stdio.h"
 *
 * void
 * set_pool_no_copy(PROCESS pool_id)
 * {
 *   static const OSADDRESS taglist[] = { OSE_POOL_ATTR_NO_COPY,
 *                                        OSE_POOL_ATTR_TAGEND };
 *
 *   if (ose_set_pool_attr(pool_id, taglist) == 1)
 *     printf("Successfully set 'no copy' mode on pool %#08x.\n", pool_id);
 *   else
 *     printf("Failed to set 'no copy' mode on pool %#08x.\n", pool_id);
 * }
 */

OSADDRESS
ose_set_pool_attr(PROCESS pool_id, const OSADDRESS *taglist);

/**
 * @function s_alloc_nil
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration union SIGNAL *
 * s_alloc_nil(PROCESS block_id, OSBUFSIZE size, SIGSELECT signo);
 *
 * @brief Allocate a signal buffer from a selected pool.
 * @long  Allocation from a signal pool owned by another program may
 *        inhibit reclaim of memory resources if that program is killed.
 *        The memory can only be reclaimed when all signals in the pool
 *        has been freed.
 *
 * @input block_id  The identifier of a block or process that uses the
 *                  wanted signal pool.
 * @input size      The requested size of the buffer.
 * @input signo     The signal number, will be placed in the allocated buffer.
 * @return          Returns a pointer to the new buffer, or NIL if the
 *                  allocation failed. NIL is returned if the pool ID has been
 *                  killed, if the pool does not support the requested size or
 *                  when a user mode process tries to allocate from a pool in
 *                  another segment.
 * @restriction     The new buffer is owned by the calling process. A new owner
 *                  can be entered only by using one of the system calls that
 *                  operates on the buffer. It is basically illegal to pass
 *                  control of a buffer to another process in any other way
 *                  since buffers may be lost in case of premature
 *                  process termination. If really needing to pass buffers
 *                  around in a disorderly manner, the restore_from system call
 *                  can be used to force registration of a new owner of a buffer.
 *
 * @seealso alloc
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // Example using a dedicated pool with special memory attributes.
 * #include "ose_spi.h"
 *
 * #define XYZ 0x123456 // A SIGNAL number
 * #define XYZ_POOLSIZE 0x10000
 * static const OSADDRESS xyz_sigsize_tab[] =
 *   {8, 31, 63, 127, 255, 511, 1023, 2047, 4095};
 * static const OSADDRESS xyz_stacksize_tab[] = {0};
 * static PROCESS xyz_block_, xyz_sig_pool_;
 * extern PROCESS xyz_consumer_;
 *
 * OS_PROCESS(xyz_producer)
 * {
 *   char *pool_memory;
 *
 *   // Allocate memory for the xyz pool.
 *   // (Not really meaningful to use malloc/free for this purpose!)
 *   pool_memory = malloc(XYZ_POOLSIZE);
 *
 *   // Create the pool and, first, a block as a container for it.
 *   xyz_block_ = create_block("XYZ_BLOCK", 0, 0, 0, 1);
 *   create_pool(xyz_block_, (OSADDRESS)pool_memory, XYZ_POOLSIZE,
 *               xyz_sigsize_tab, xyz_stacksize_tab);
 *   xyz_sig_pool_ = get_sig_poolid(xyz_block_);
 *
 *   // Start the block. This will make the block and pool survive
 *   // if the creating process is killed.
 *   start(xyz_block_);
 *
 *   // Attach to the pool. The pool attach signal will be sent when
 *   // the block has been killed and all signals in the pool has
 *   // been freed. At that point, the pool memory can be reused for
 *   // another purpose.
 *   attach(NULL, xyz_sig_pool_);
 *
 *   for(;;)
 *     {
 *       static const SIGSELECT select_any[] = {0};
 *       union SIGNAL *sig, *xyz_sig;
 *
 *       sig = receive(select_any);
 *
 *       if(sender(&sig) == xyz_sig_pool_)
 *         {
 *           // The pool is gone. Deallocate the memory area and quit.
 *           free(pool_memory);
 *           free_buf(&sig);
 *           kill_proc(current_process());
 *         }
 *
 *       // Use signals from the pool.
 *       xyz_sig = s_alloc_nil(xyz_block_, 4095, XYZ);
 *       if(xyz_sig != NIL)
 *         {
 *           send(&xyz_sig, xyz_consumer_);
 *         }
 *       free_buf(&sig);
 *     }
 * }
 */
union SIGNAL *
s_alloc_nil(PROCESS block_id, OSBUFSIZE size, SIGSELECT signo);

/**
 * @function alloc_brk
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration union SIGNAL *
 * alloc_brk(OSBUFSIZE size, SIGSELECT signo);
 *
 * @brief Allocate a new signal buffer.
 * @long
 * This system call is exactly like the @link "ose.h" "alloc" system
 * call except for the fact that alloc_brk always returns a new unused
 * signal buffer, i.e. it always breaks new memory from the pool.
 *
 * @input size The requested size of the buffer.
 * @input signo The signal number, will be placed in the allocated buffer.
 * @return Returns a pointer to the new signal buffer.
 *
 * @seealso alloc
 * @seealso default_restrictions_calling_functions
 */
union SIGNAL *
alloc_brk(OSBUFSIZE size, SIGSELECT signo);

/**
 * @macro OSE_FSEND_UNCLAIMED
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration #define OSE_FSEND_UNCLAIMED (0x100)
 *
 * @brief ose_fsend flag for sending unclaimed signals in kernel mode.
 * @long In normal application programming, all signals are owned by a process.
 * However, in driver design when using the Network Buffer Concept, signals may
 * be in a state "unclaimed", which means that they are owned by kernel.
 * This flag should in such cases be present in the ose_fsend option field. 
 *
 * @seealso ose_fsend
 * 
 */
#define OSE_FSEND_UNCLAIMED (0x100)

/**
 * @function ose_fsend
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSADDRESS
 * ose_fsend(union SIGNAL **sig, PROCESS from, PROCESS to, OSADDRESS flags);
 *
 * @brief Send a signal, optimized for high-speed inter-cpu (inter-core) communication.
 *
 * @long The purpose of this call is to send signals at a very high traffic
 * load, typically between cores in a multicore system, for example signals
 * containing network payload.
 * This call also works within one core and via LINX also to other nodes.
 * It works similar to send_w_s and send, but carries additional attributes.
 * 
 * @long All cases:
 *   Signals are delivered with no wakeup, i.e. sending signals do not
 *   preempt a running process with lower priority than the destination process.
 *   The use case is optimized for cooperative scheduling mode, where the
 *   signals are polled in a non-blocking receive loop.
 *
 * @long If the signal is sent to an addressee pid on the same cpu:
 *   The signal is delivered reliably just as a regular send but with
 *   no wakeup as described above.
 *   The priority field is "don't care" in the intra-cpu case, and the
 *   inter-cpu queue mechanism described below is not used.
 *   
 * @long If the signal is destined to an addressee pid on another cpu:
 *   The signal will use "light-weight" signal message queues with
 *   configurable lengths of the queues on each priority level.
 *   The inter-cpu signal queue lengths are configurable via the
 *   system parameter krn/fsend_queue.
 *   The order of delivery is, in the case of inter-cpu delivery, therefore
 *   not guaranteed between priorities but retained within each priority level.
 *
 *   @long All priority levels deliver by default signals unreliably,
 *   hence if any queue becomes full, the signal is silently discarded.
 *   However, it is possible to install a custom fsend hook (see
 *   @link "" "ose_set_fsend_hook") that may, on each priority, 
 *   decide to instruct the kernel to either silently discard the signal, or to
 *   implement a fail detection (reliability)
 *   This mechanism allows a platform to optionally implement
 *   customer specific drop algorithms on
 *   specific priority levels by returning a non-zero value in the fsend hook
 *   if the number of elements in the queue reaches the maximum configured (has
 *   to be retrieved by the platform code that installs the hook).
 *
 * @input sig A pointer to the signal buffer.
 * @input from The ID of the process specified as sender.
 * @input to The ID of the process the signal will be sent to. The normal case
 * is that the receiving process is executing on a different cpu than the
 * sender but it could also be executing on the same cpu as the sender.
 * @input flags The least significant byte contains a priority field with the
 * range 0-31. 0 is the highest priority. It is used to select priority between
 * messages that are sent between cores with the ose_fsend call. The number of
 * priorities that are used is configurable (the number of fsend signal
 * queues). A value higher than the configured maximum will use the lowest
 * priority. The priority byte is to be or'ed with the flags. In the current 
 * implementation there is only one flag available FSEND_UNCLAIMED.
 *
 * @return          Returns zero if the call was successful. 
 *
 * @restriction
 * ose_fsend does not fully support redirection tables. It only supports the
 * most basic form, where all signals are redirected to a default process,
 * and this default process does not have an own redirection table. Other forms
 * of redirection will silently free the signal. Rationale: ose_fsend is
 * optimized for performance.
 *
 * @seealso send
 * @seealso send_w_s
 * @seealso OSE_FSEND_UNCLAIMED
 * @seealso ose_set_fsend_hook
 * @seealso default_restrictions_calling_functions
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * // The declarations of signal numbers and structs are
 * // preferably made in .sig-files which are then #included.
 * #define FIFTH 5
 * struct fifth
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * extern PROCESS proc1_;
 * extern PROCESS proc2_;
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *
 *   for(;;)
 *     {
 *       sig = alloc(sizeof(struct fifth), FIFTH);
 *
 *       // send a signal to proc2 specifying proc1 as sender with lowest prio.
 *       ose_fsend(&sig, proc1_, proc2_, 31);
 *       // Code //
 *     }
 * }
 */
OSADDRESS
ose_fsend(union SIGNAL **sig, PROCESS from, PROCESS to, OSADDRESS flags);

/**
 * @function ose_set_hw_rcv_poller
 * @include "ose_spi/ose_spi.h"
 *
 * @brief EXPERIMENTAL. Install a function in the kernel to be called from
 * the receive system call.
 *
 * @input cpu  The cpu that the handler is associated with.
 * @input hw_poller A function pointer. NULL will disable polling.
 *
 * @return Previous function or NULL.
 *
 * @seealso default_restrictions_calling_functions
 */
OSHWRCVPOLLER *
ose_set_hw_rcv_poller(cpuid_t cpu, OSHWRCVPOLLER *hw_poller);

/**
 * @function ose_set_fsend_hook
 * @include "ose_spi/ose_spi.h"
 *
 * @brief Install a user-provided fsend drop callback function in kernel.
 * @long The purpose of this function is to provide external support to
 * implement a custom specific drop algoritm on a given
 * priority in the @link "" "ose_fsend" call.
 * @long The user-supplied callback function is described in @link ""
 * "OSFSENDHOOK".
 *
 * @input sig_prio  The fsend signal priority the function shall be used for.
 * @input hook      A function pointer. NULL will remove the hook.
 *
 * @return          Returns zero if the call was successful.
 *                  Returns a non-zero value if the call failed.
 *
 * @seealso OSFSENDHOOK
 * @seealso ose_fsend
 * @seealso default_restrictions_calling_functions
 */
OSBOOLEAN
ose_set_fsend_hook(int sig_prio, OSFSENDHOOK *hook);

/*
** The "look and feel" of the following system calls will remain as below
** even though they are not declared here in case OS_DEBUG is defined.
*/
#if defined(__cplusplus) && defined(OS_DEBUG)
#else

/**
 * @function hunt_from
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration OSBOOLEAN
 * hunt_from(const char *name, OSUSER user, PROCESS *name_,
 * 	  union SIGNAL **hunt_sig, PROCESS from);
 *
 * @brief Hunt using an alias.
 * @long  This is a system call intended mainly for link handlers and other
 *        network software.
 * @long  This call should not be required unless writing for such software.
 *        Hunt_from does what hunt does, except that the search path is
 *        evaluated for the process specified in the from parameter
 *        (usually a phantom process created by the caller).
 * @long  The optional hunt signal is eventually returned to the caller, not
 *        to the process specified in the from field.
 * @long  The kernel frees the pending hunt signal if the calling process dies
 *        or if the process specified in the from field dies.
 *
 * @input name     A pointer to the specified process name.
 * @input user     Must be 0.
 * @input name_    A pointer to the ID of the found process.
 * @input hunt_sig A pointer to the hunt signal buffer if specified.
 * @input from     The ID of the process specified as hunter.
 * @return         See the hunt system call.
 * @restriction    See the hunt system call.
 *
 * @seealso hunt
 * @seealso attach
 * @seealso detach
 * @seealso assign_linkhandler
 * @seealso default_restrictions_calling_functions
 */
OSBOOLEAN
hunt_from(const char *name, OSUSER user, PROCESS *name_,
	  union SIGNAL **hunt_sig, PROCESS from);

/**
 * @function power_fail
 * @include "ose_spi/ose_spi.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * power_fail(void);
 *
 * @brief Shut down the system.
 * @long  Shuts down the system in a manner which enables immediate
 *        continuation at a subsequent restart. When power reappears,
 *        the calling process (and all others) appears to be continuing
 *        where it left off at the time of the power_fail system call.
 * @long  The call to power_fail causes the current process to be swapped
 *        out, and a user written power fail handler is called to save
 *        memory contents and I/O status. When this is complete,
 *        the handler should shut down the CPU.
 * @long  If power_fail is called from an interrupt process, then the system
 *        shutdown procedure is deferred until execution returns to a
 *        non-interrupt level.
 * @long  When the system restarts, the kernel calls a power on handler which
 *        determines if memory and I/O can be restored. If this is the case,
 *        the kernel immediately swaps in the process that called power_fail,
 *        unless an interrupt process called power_fail, in which case the
 *        kernel swaps in the process that was swapped out when the interrupt
 *        process was called. Otherwise an ordinary cold start is performed.
 *
 * @restriction  Use of the power_fail system call requires careful system
 *               planning since many hardware problems must be correctly
 *               addressed. In particular, it must be possible to
 *               reinitialize all I/O devices to the exact state that was
 *               present at the shutdown.
 * @break  Hints:
 * @break  - A good memory backup power system is required, including
 *         protection against spurious memory cycles during periods of
 *         unstable power.
 * @break  - A reliable power fail interrupt is needed. This interrupt
 *         should not be an NMI since non-maskable interrupts can not
 *         communicate with the kernel.
 * @break  - A watchdog timer is needed since memory backup designs may be
 *         somewhat unreliable.
 * @break  - It is a good idea to have hardware support for determining if
 *         a cold start is required.
 * @break  - Software should be planned in a way that promotes saving the I/O
 *         state. Doing this correctly is an art.
 *
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose_spi/ose_spi.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   for(;;)
 *     {
 *       // Code //
 *       power_fail();
 *       // Code, This is where the process starts again at
 *       // power up on a warm start.
 *     }
 * }
 */
void
power_fail(void);

OSADDRESS
get_sysconf(OSADDRESS f, OSADDRESS p, OSADDRESS q, OSADDRESS r);

union SIGNAL *
receive_with(OSTIME timeout, const SIGSELECT *sel, PROCESS from);

#endif /* defined(__cplusplus) && defined(OS_DEBUG) */

/*
** Debug system calls.
*/

OSADDRESS
zzget_sysconf(ZZPRE_PROTO OSADDRESS f, OSADDRESS p, OSADDRESS q,
	      OSADDRESS r ZZPOST_PROTO);

OSBOOLEAN
zzhunt_from(ZZPRE_PROTO const char *name, OSUSER user, PROCESS *name_,
	    union SIGNAL **hunt_sig, PROCESS from ZZPOST_PROTO);

void
zzpower_fail(ZZMIDDLE_PROTO);

union SIGNAL *
zzreceive_with(ZZPRE_PROTO OSTIME timeout, const SIGSELECT *sel,
	       PROCESS from ZZPOST_PROTO);

#ifdef __cplusplus

} /* end of extern "C" */

#ifdef OS_DEBUG

static inline OSADDRESS
get_sysconf(OSADDRESS f, OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
   return zzget_sysconf(ZZPRE_CPP_ARGS(get_sysconf)
			f, p, q, r
			ZZPOST_CPP_ARGS(get_sysconf));
}

static inline OSBOOLEAN
hunt_from(const char *name, OSUSER user, PROCESS *name_, union SIGNAL **hunt_sig,
	  PROCESS from)
{
   return zzhunt_from(ZZPRE_CPP_ARGS(hunt_from)
		      name, user, name_,
		      hunt_sig, from
		      ZZPOST_CPP_ARGS(hunt_from));
}

static inline void
power_fail(void)
{
   zzpower_fail(ZZMIDDLE_CPP_ARGS(power_fail));
}

static inline union SIGNAL *
receive_with(OSTIME timeout, const SIGSELECT *sel, PROCESS from)
{
   return zzreceive_with(ZZPRE_CPP_ARGS(receive_with)
			 timeout,
			 sel, from
			 ZZPOST_CPP_ARGS(receive_with));
}

#endif /* OS_DEBUG */

extern "C" { /* end of C++ */

#else /* __cplusplus */

#ifdef OS_DEBUG

#define get_sysconf(f,p,q,r)\
	zzget_sysconf(ZZPRE_ARGS f,p,q,r ZZPOST_ARGS)

#define hunt_from(name,user,name_,hunt_sig,from)\
	zzhunt_from(ZZPRE_ARGS name,user,name_,hunt_sig,from ZZPOST_ARGS)

#define power_fail() zzpower_fail(ZZMIDDLE_ARGS)

#define receive_with(timeout,sel,from)\
	zzreceive_with(ZZPRE_ARGS timeout,sel,from ZZPOST_ARGS)

#endif /* OS_DEBUG */

#endif /* __cplusplus */

/*
** The system calls below are considered "processor architecture dependent".
** This means that they are subject to a lower degree of portability than
** other system calls as they are present only in some OSE implementations.
** New system calls that have not yet been incorporated in the generic OSE
** specification are also treated as "processor architecture dependent"
** until it has been decided that they should be globally adopted.
*/

/*@-PRIVATE-CALLS-@*/

/**
 * @function extend_pool
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration void
 * extend_pool(OSERRCODE pool_id, OSADDRESS base, OSADDRESS size);
 *
 * @brief Extend an exhausted pool.
 * @long  This system call is used to provide more memory to exhausted
 *        pools. It can only be called from an error handler, when an exhausted
 *        pool is reported. This pool_id parameter is set from the extra
 *        argument in the error handler; it contains information about the
 *        pool at the time of the error.
 * @long  This functionality will be provided through a new interface in the
 *        future. Therefore, it is strongly recommended only to use
 *        extend_pool in the core module, so the extend pool handling can be
 *        replaced together with a kernel upgrade.
 *
 * @input pool_id  This parameter shall be set to the value of the extra
 *                 error argument received by the error handler.
 * @input base     The lowest address of the new memory fragment,
 *                 to add to the pool.
 * @input size     The number of bytes in the added memory fragment.
 * @seealso ose_set_pool_extender
 */
void
extend_pool(OSERRCODE pool_id, OSADDRESS base, OSADDRESS size);

/**
 * @function ose_set_pool_extender
 * @include "ose_spi/ose_spi.h"
 *
 * @brief Register a pool extender callback function.
 * @long  The extender function is called by the kernel when the pool is 
 *        exhausted. The extender shall preallocate a spare memory area to be
 *        used as a new pool fragment, and call extend_pool() to take the
 *        fragment in use. Normally, the extender will also initiate 
 *        reservation of a new area, to be used at the next extend event.
 * @long  This system call gives a possibility to install a custom
 *        extender, replacing the OSE default extender for the
 *        specified pool.
 * @long  The extender has two arguments, named pool and extra.
 *        Compare @link "" "OSPOOLEXTENDH".
 * @long  "Pool" is the identifier of the pool object. 
 * @long  "Extra" is the value that shall be passed as the first
 *        parameter to extend_pool().
 *        The eight most significant bits of the "extra" parameter is
 *        a pool specific extend counter, modulo 128. It can be
 *        used by the extender to detect if it has been interrupted 
 *        by another process that has already extended the pool.
 *        The counter value in the first call is zero.
 * @long  The extender shall return 0 if the problem remains, resp. 1 when
 *        the kernel should retry the allocation.
 * @long  The extender function may be called with a locked dispatcher,
 *        or in the context of an interrupt process. This sets restrictions
 *        for use of system calls. It is however possible to use the 
 *        kill_proc(), signal_fsem() and signal_sem() calls because these 
 *        calls were designed to work in this situation. The actual
 *        allocation of the new fragment from the memory manager needs
 *        to be done with a separate prioritized process, trigged by
 *        signal_fsem or signal_sem.
 *
 * @input pool_id         The identifier of the pool to extend.
 * @input extend_func     The extender callback.
 * @input nextfrag_handle A pointer to a pointer to a pre-allocated pool 
 *                        fragment that may be used for the pool. Used by 
 *                        the multicore kernel to decide if there is a spare,
 *                        pre-allocated pool fragment, ready to be taken in
 *                        use. Not used by the unicore kernel.
 * @input ext_frag_prio   The priority level used for allocation of new pool
 *                        fragments. Only used by the multicore kernel.
 *
 * @restriction This call is only available in the core module,
 *              i.e. the kernel program.
 * @seealso extend_pool
 * @seealso OSPOOLEXTENDH
 * @seealso default_restrictions_calling_functions
 */
void
ose_set_pool_extender(PROCESS pool_id,
		      OSPOOLEXTENDH *extend_func,
		      void **nextfrag_handle,
		      OSPRIORITY ext_frag_prio);

/**
 * @function set_abstime
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration void
 * set_abstime(OSTICK abs_time, OSTICK ticks, OSTICK microsecs);
 *
 * @brief Set real time to be used by the debug system.
 * @long  This system call is used to provide a binding between the system
 *        tick and real time and is used by the Run-mode Monitor and clients
 *        such as Optima.
 * @long  The real time, or absolute time, is expressed as the number of
 *        seconds since the epoch at 00:00:00 UTC on January 1 1970. The
 *        ticks and microsecs should be the corresponding OSE system time.
 *        The precision of the time should preferably be with a microsecond
 *        resolution.
 * @long  Ideally set_abstime should be called as early as possible, each
 *        time the real time clock is updated, and at least once per tick
 *        wrap period.
 *
 * @input abs_time  Seconds since 0:00:00 UTC on January 1 1970.
 * @input ticks     Corresponding system tick at abs_time.
 * @input microsecs Number of microseconds since the tick at abs_time.
 *
 * @restriction  Only has an effect if there is a Run-mode Monitor in
 *               the system.
 *
 * @seealso get_abstime
 * @seealso default_restrictions_calling_functions
 */
void
set_abstime(OSTICK abs_time, OSTICK ticks, OSTICK microsecs);

/**
 * @function get_abstime
 * @include "ose_spi/ose_spi.h"
 *
 * @declaration void
 * get_abstime(OSTICK *abs_time, OSTICK *ticks, OSTICK *microsecs);
 *
 * @brief Get the last known real time and the corresponding system tick.
 *
 * @long  This system call retrieves the last known real time, or absolute
 *        time, and the corresponding system tick. If there is no known
 *        real time all output variables are set to zero.
 *
 * @output abs_time  Seconds since 0:00:00 UTC on January 1 1970.
 * @output ticks     Corresponding system tick at abs_time.
 * @output microsecs Number of microseconds since the tick at abs_time.
 *
 * @restriction Will always return zeros if no run-mode monitor present.
 *
 * @seealso set_abstime
 * @seealso default_restrictions_calling_functions
 */
void
get_abstime(OSTICK *abs_time, OSTICK *ticks, OSTICK *microsecs);

/* Send hook management. */
struct OS_send_hook_info
{
   OSBUFSIZE sig_size;
   union SIGNAL *sig;
   PROCESS addressee_pid;
   PROCESS sender_pid;
   OSADDRESS zzreserved;
   void *addressee_lnh_hd;
   void *sender_lnh_hd;
   void *zzos_internal_sig;
};

void send_hook_free_buf(union SIGNAL *sig);
void send_hook_restore(union SIGNAL *sig);

typedef OSADDRESS (OSSENDHOOK)(struct OS_send_hook_info *snd_info,
			       OSADDRESS version);

struct OS_send_hook_entry
{
   struct OS_redir_entry fake_redir;
   OSSENDHOOK *snd_hook;
   void *lnh_hd;
};

struct OS_ppd_list
{
  OSBUFSIZE count;
  OSPPDKEY  list[1];
};

struct OS_ppd_list *ose_get_ppd_list(void);
char *ose_get_ppd_keyname(OSPPDKEY key);
/* OSADDRESS ose_get_ppd(PROCESS pid, const char *name); */
OSADDRESS ose_get_ppd_by_key(PROCESS pid, OSPPDKEY key);
void ose_nano_delay(OSADDRESS nanosec);
void ose_sys_mutex_init(MUTEX *mutex, OSADDRESS attr);
void ose_sys_mutex_destroy(MUTEX *mutex);


struct OS_curp
{
  PROCESS   pid;
  OSADDRESS type;
  OSADDRESS pri;
  OSADDRESS _reserved;
};

void ose_get_curr_procs(struct OS_curp *out, cpuid_t cpus);

#ifdef __cplusplus
}
#endif

#endif /* ZZOSE_SPI_H */
