/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @public
 * @toc Program_Management:
 * @file ose_spi/pm.sig
 * @version
 * @brief Program manager signal interface
 */

#ifndef PM_SIG_
#define PM_SIG_

/***********************************************************
 *
 * INCLUDE FILES
 *
 ***********************************************************/

#include "ose.h"
#include "osetypes.h"
#include "pm_defs.h"
#include "mmtypes.h"
#include "heapapi.h"

#ifdef __cplusplus
extern "C" {
#endif

/***********************************************************
 *
 * CONSTANTS AND DEFINITIONS
 *
 ***********************************************************/

/**
 * @macro PM_STRING_LENGTH
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief The maximum string lengths in program manager interface signals
 * including string terminator, unless other string lengths are defined
 * for specific cases.
 *
 * @long Currently this value is 32.
 */
#define PM_STRING_LENGTH             32

/**
 * @macro PM_INSTALL_HANDLE_LENGTH
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Maximum length of install handle name, including the string terminator.
 *
 * @seealso PM_STRING_LENGTH
 */
#define PM_INSTALL_HANDLE_LENGTH     PM_STRING_LENGTH

/**
 * @macro PM_SECTION_NAME_LENGTH
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Maximum length of load module section name, including the string terminator.
 *
 * @seealso PM_STRING_LENGTH
 */
#define PM_SECTION_NAME_LENGTH       PM_STRING_LENGTH

/**
 * @macro PM_PROGRAM_TYPE_PARAM_LENGTH
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Maximum length of program type name, including the string terminator.
 *
 * @long Currently this is set to @link "" "MM_NAME_LENGTH" + 1.
 *
 * @seealso MM_NAME_LENGTH
 */
#define PM_PROGRAM_TYPE_PARAM_LENGTH (MM_NAME_LENGTH + 1)

/**
 * @macro PM_FILE_FORMAT_NAME_LENGTH
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Maximum length of file format name, including the string terminator.
 *
 * @seealso PM_STRING_LENGTH
 */
#define PM_FILE_FORMAT_NAME_LENGTH   PM_STRING_LENGTH

/**
 * @macro PM_PARENT_FILE_NAME_SIZE
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Maximum length of file name, including the string terminator.
 */
#define PM_PARENT_FILE_NAME_SIZE     256


/* The default program regions types. The defines specify the program region
 * types location in the program_type array.
 */

/**
 * @macro PM_TEXT_REGION_TYPE
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Index to the text region.
 *
 * @long This is one of the default program region types. The value of
 *       this constant is an index into an array of program types.
 *
 * @seealso PmProgramTypeInfo
 */
#define PM_TEXT_REGION_TYPE          0

/**
 * @macro PM_DATA_REGION_TYPE
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Index to the data region.
 *
 * @long This is one of the default program region types. The value of
 *       this constant is an index into an array of program types.
 *
 * @seealso PmProgramTypeInfo
 */
#define PM_DATA_REGION_TYPE          1

/**
 * @macro PM_POOL_REGION_TYPE
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Index to the pool region.
 *
 * @long This is one of the default program region types. The value of
 *       this constant is an index into an array of program types.
 *
 * @seealso PmProgramTypeInfo
 */
#define PM_POOL_REGION_TYPE          2

/**
 * @macro PM_HEAP_REGION_TYPE
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Index to the heap region.
 *
 * @long This is one of the default program region types. The value of
 *       this constant is an index into an array of program types.
 *
 * @seealso PmProgramTypeInfo
 */
#define PM_HEAP_REGION_TYPE          3


/* Program execution modes. */

/**
 * @macro PM_EXEC_MODE_USER
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Execute in user mode
 *
 * @long This is one possible value for field 'super' in struct
 *       'PmProgramTypeInfo'.
 *       It indicates that a program should execute in user mode.
 *
 * @seealso PmProgramTypeInfo
 */
#define PM_EXEC_MODE_USER            0

/**
 * @macro PM_EXEC_MODE_SUPERVISOR
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Executes in supervisor mode.
 *
 * @long This is one possible value for field 'super' in struct
 *       'PmProgramTypeInfo'.
 *       It indicates that a program should execute in supervisor mode.
 *
 * @seealso PmProgramTypeInfo
 */
#define PM_EXEC_MODE_SUPERVISOR      1

/* Program states. */

/**
 * @macro PM_PROGRAM_CREATED
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Program is in the state created.
 *
 * @long This value in field 'state' in struct 'PmProgramInfo' indicates that
 *       the program is in 'created' state.
 *
 * @seealso PmProgramInfo
 */
#define PM_PROGRAM_CREATED           0

/**
 * @macro PM_PROGRAM_STARTED
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Program is in the state started.
 *
 * @long This value in field 'state' in struct 'PmProgramInfo' indicates that
 *       the program is in 'started' state.
 *
 * @seealso PmProgramInfo
 */
#define PM_PROGRAM_STARTED           1

/* Section options as part of the section information of a load module. */

/**
 * @macro PM_SECTION_NO_LOAD
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief The load module section is not loaded into memory.
 *
 * @long This value in field 'section_options' in struct
 *       'PmLoadModuleSectionInfo' indicates that the section has not
 *       been loaded into memory.
 *
 * @seealso PmLoadModuleSectionInfo
 */
#define PM_SECTION_NO_LOAD           0

/**
 * @macro PM_SECTION_LOAD
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief The load module section is loaded into memory.
 *
 * @long This value in field 'section_options' in struct
 *       'PmLoadModuleSectionInfo' indicates that the section has been loaded
 *       into memory.
 *
 * @seealso PmLoadModuleSectionInfo
 */
#define PM_SECTION_LOAD              1

/* Installation options. */

/**
 * @macro PM_LOAD_MODULE_PERSISTENT
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief The load module shall be persistent.
 *
 * @seealso PmInstallLoadModuleRequest
 * @seealso PmLoadModuleInfoReply
 */
#define PM_LOAD_MODULE_PERSISTENT     1

/**
 * @macro PM_LOAD_MODULE_ABSOLUTE
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief The load module must be installed at its linked addresses.
 *
 * @seealso PmInstallLoadModuleRequest
 * @seealso PmLoadModuleInfoReply
 */
#define PM_LOAD_MODULE_ABSOLUTE       2

/**
 * @macro PM_LOAD_MODULE_PAGED
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief The load module must be demand paged from file.
 *
 * @seealso PmInstallLoadModuleRequest
 * @seealso PmLoadModuleInfoReply
 */
#define PM_LOAD_MODULE_PAGED          4

/**
 * @macro PM_LOAD_MODULE_KERNEL
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief The load module shall be installed as a kernel module.
 *
 * @seealso PmInstallLoadModuleRequest
 * @seealso PmLoadModuleInfoReply
 */
#define PM_LOAD_MODULE_KERNEL         8

/* Domain related constants. */

/**
 * @macro PM_NEW_DOMAIN
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief The definition of a new protection domain to be created,
 *        used when creating a program.
 *
 * @long When creating a new program, you can specify in which domain you
 *       want to create the program. If you want the program to be
 *       created in a new domain, use this constant as the domain number.
 *
 * @seealso PmCreateProgramRequest
 */
#define PM_NEW_DOMAIN                 0xffffffff

/* Extend program memory options. */

/**
 * @macro PM_SPECIFY_BASE
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Flag meaning that a required base address is specified in the
 *        request.
 *
 * @seealso PmExtendProgramMemory
 */
#define PM_SPECIFY_BASE               1

/**
 * @macro PM_SPECIFY_ALIGN
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Flag meaning that a required alignment is specified in the request.
 *
 * @seealso PmExtendProgramMemory
 */
#define PM_SPECIFY_ALIGN              2

/* PM signal base. */

/**
 * @macro PM_SIGNAL_BASE
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief This is the signal number base for the program manager.
 */
#define PM_SIGNAL_BASE                36200

/* PM event subscription events. */

/**
 * @macro PM_EVENT_UNINSTALL_LOAD_MODULE
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Event signifying the uninstallation of a load module.
 *
 * @long The data element of signal PmEvent when this event has
 *       occurred contains the install handle of the load module that was
 *       uninstalled.
 *
 * @seealso PmEvent
 */
#define PM_EVENT_UNINSTALL_LOAD_MODULE               1

/***********************************************************
 *
 * STRUCTURES AND GLOBALS
 *
 ***********************************************************/

/*
 *===========================================================================
 *                        PmProgramRegionType
 *===========================================================================
 */
/**
 * @type PmProgramRegionType
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief The program region type definition.
 *
 * @long This struct contains information about a region, in a form
 *       understandable by the memory manager. A program type contains
 *       several of these, one for each type of region defined by the
 *       program type.
 * @field phys_mem    The physical memory of the program region type.
 *                        Syntax: The ASCII name of the phys_mem
 *                                as configured in MM.
 *                        Example: "RAM".
 * @field log_mem     The logical memory of the region type.
 *                        Syntax: The ASCII name of a log_mem
 *                                as configured in MM.
 *                        Example: "RAM_SASE".
 * @field permissions The access permissions of the MM region type.
 *                        Syntax: The permission syntax defined in
 *                                mm.sig.
 *                        Example: @link "mm.sig" "SU_RWX_USR_NA".
 * @field cache_mode  The cache attributes of the region type, which
 *                        are a combination of cache and access modes.
 *                        Syntax: The cache syntax defined in mm.sig
 *                                (see cache_mode in
 *                                @link "mm.sig" "MM_CREATE_REGION_REQUEST").
 *                        Example: @link "mm.sig" "MM_CACHE_INHIBITED" |
 *                        @link "mm.sig" "MM_SPECULATIVE_ACCESS"
 *                        or: MM_CACHE_COPY_BACK | MM_SPECULATIVE_ACCESS.
 */
struct PmProgramRegionType
{
  char  phys_mem[PM_PROGRAM_TYPE_PARAM_LENGTH];
  char  log_mem[PM_PROGRAM_TYPE_PARAM_LENGTH];
  U32   permissions;
  U32   cache_mode;
};

/*
 *===========================================================================
 *                        PmLoadModuleSectionInfo
 *===========================================================================
 */

/**
 * @type PmLoadModuleSectionInfo
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief The load modules section information structure.
 *
 * @long This struct holds information about one single ELF section.
 * @field section_base    The base address of the section.
 * @field section_size    The size of the section (bytes).
 * @field section_attr    The attributes of the section (file format specific,
 *                        for ELF compatible load modules the section_attr
 *                        correlates with the contents of sh_flags of
 *                        Elf32_Shdr).
 * @field section_options These options describe whether the section is part
 *                        of a loaded segment or not. Possible values are:
 *                          PM_SECTION_LOAD    - The section is part of a
 *                                               loaded segment.
 *                          PM_SECTION_NO_LOAD - The section is not part of a
 *                                               loaded segment.
 * @field section_name    The name of the section.
 */
struct PmLoadModuleSectionInfo
{
  OSADDRESS   section_base;
  U32         section_size;
  U32         section_attr;
  U32         section_options;
  char        section_name[PM_SECTION_NAME_LENGTH];

};

/*
 *===========================================================================
 *                        PmProgramLoad
 *===========================================================================
 */

/**
 * @type PmProgramLoad
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief The program load report structure.
 *
 * @long This struct holds information about the load of a program.
 *       It is mainly used for profiling purposes.
 *
 * @field id     The program id.
 * @field sum    Time the CPU was occupied with this program (unspecified unit)
 * @field locked Non-zero if the program is locked to a CPU.
 */
struct PmProgramLoad
{
   PROCESS id;
   U32     sum;
   U32     locked;
};

/**
 * @type PmProgramTypeInfo
 *
 * @brief Struct used in signals for handling program types.
 *       See PmAddProgramType and PmProgramTypeInformation.
 *
 * @field sigNo        Signal number.
 * @field status       The status of the operation.
 * @field super        Supervisor mode or user mode. Use the
 *                     macros @link "pm.sig" "PM_EXEC_MODE_USER" and
 *                     @link "pm.sig" "PM_EXEC_MODE_SUPERVISOR"
 *                     to control the execution mode.
 * @field program_type_name The program type name string.
 * @field program_type_size The number of elements of the
 *                          program_type array.
 * @field program_type The program type regions definition as
 *                     an array of @link "" "PmProgramRegionType".
 *                     The array is of variable size.
 *                     Note: This array is at minimum three
 *                     elements large to hold the OSE default
 *                     program region types.
 *                     The three default regions types must be the
 *                     first three elements of the array, in the order:
 *                     text region, data region, pool region.
 *
 * @seealso PmProgramRegionType
 * @seealso PmProgramTypeInformation
 * @seealso PmAddProgramType
 */
struct PmProgramTypeInfo
{
  SIGSELECT                  sigNo;
  PmStatus                   status;
  U32                        super;
  char                       program_type_name[PM_PROGRAM_TYPE_PARAM_LENGTH];
  U32                        program_type_size;
  struct PmProgramRegionType program_type[PM_MINIMAL_NO_OF_REGIONS];
};

/***********************************************************
 *
 * SIGNAL DEFINITIONS
 *
 ***********************************************************/

/*
 *===========================================================================
 *                        PM_INTERFACE_REQUEST,
 *                        PM_INTERFACE_REPLY
 *===========================================================================
 */

/**
 * @signal PmInterface
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Signal to get pm.sig interface information.
 *
 * @long This signal requests
 *       information about the program manager interface.
 *       The reply from this request signal is used to resolve any
 *       compatibility issues when the interface changes.
 *
 * @member PmInterfaceRequest
 * @member PmInterfaceReply
 *
 * @seealso PM_INTERFACE_REQUEST
 * @seealso PM_INTERFACE_REPLY
 */

/**
 * @macro PM_INTERFACE_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Request signal number for getting the pm.sig interface.
 */
#define PM_INTERFACE_REQUEST                      (36200) \
  /* !-SIGNO(struct PmInterfaceRequest)-! */

/**
 * @type PmInterfaceRequest
 *
 * @input sigNo PM_INTERFACE_REQUEST
 *
 * @seealso PmInterfaceReply
 */
struct PmInterfaceRequest
{
  SIGSELECT     sigNo;
};

/**
 * @macro PM_INTERFACE_REPLY
 *
 * @brief Reply signal number for getting the pm.sig interface.
 */
#define PM_INTERFACE_REPLY                        (36201) \
  /* !-SIGNO(struct PmInterfaceReply)-! */

/**
 * @type PmInterfaceReply
 *
 * @output sigNo      PM_INTERFACE_REPLY
 * @output status     The status of the operation, always
 *                    @link "pm.h" "PM_SUCCESS".
 * @output whatStr    The PM what string, limited to 31 char +
 *                    terminator ('\0').
 * @output biosHandle The PM bios handle.
 * @output sigs       The signal numbers of the pm.sig interface requests
 *                    that are supported. (variable size, element ZERO
 *                    holds the number of signals in the array).
 *
 * @seealso PmInterfaceRequest
 */
struct PmInterfaceReply
{
  SIGSELECT     sigNo;
  PmStatus      status;
  char          whatStr[32];
  U32           biosHandle;
  SIGSELECT     sigs[1];
};

/*
 *===========================================================================
 *                        PM_ADD_PROGRAM_TYPE_REQUEST,
 *                        PM_ADD_PROGRAM_TYPE_REPLY
 *===========================================================================
 */

/**
 * @signal PmAddProgramType
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Add or modify program type.
 *
 * @long A program type defines how a program is to be created. It defines
 *       execution mode and memory region types for the program.
 *
 * @long  To add user defined program region types, extend the
 *        Program Manager's program_type array with more elements,
 *        in addition to the default text, data and pool region types.
 *
 * @long Signal @link "pm.sig" "PM_ADD_PROGRAM_TYPE_REQUEST"
 *       is used to add or modify program types. All fields are given
 *       as input.
 *
 * @long Signal @link "pm.sig" "PM_ADD_PROGRAM_TYPE_REPLY" is the reply signal,
 *       with the program type name and status.
 *
 * @member PmProgramTypeInfo
 * @member PmAddProgramTypeReply
 *
 * @seealso PmProgramRegionType
 * @seealso PmProgramTypeInformation
 * @seealso PM_ADD_PROGRAM_TYPE_REQUEST
 * @seealso PM_ADD_PROGRAM_TYPE_REPLY
 */

/**
 * @macro PM_ADD_PROGRAM_TYPE_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Add or modify program type request signal number.
 *
 * @seealso PmAddProgramType
 */
#define PM_ADD_PROGRAM_TYPE_REQUEST               (36202) \
  /* !-SIGNO(struct PmProgramTypeInfo)-! */

/* NOTE: struct PmProgramTypeInfo is a general structure defined in the
 *       beginning of this file.
 */

/**
 * @macro PM_ADD_PROGRAM_TYPE_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Add or modify program type reply signal number.
 *
 * @seealso PmAddProgramType
 */
#define PM_ADD_PROGRAM_TYPE_REPLY                 (36203) \
  /* !-SIGNO(struct PmAddProgramTypeReply)-! */

/**
 * @type PmAddProgramTypeReply
 *
 * @output sigNo   PM_ADD_PROGRAM_TYPE_REPLY
 * @output status  Status of the operation.
 * @output program_type_name  Which program type that was added. Copied
 *                            from the request signal.
 *
 * @seealso PmAddProgramType
 */
struct PmAddProgramTypeReply
{
  SIGSELECT                  sigNo;
  PmStatus                   status;
  char                       program_type_name[PM_PROGRAM_TYPE_PARAM_LENGTH];
};

/*
 *===========================================================================
 *                        PM_PROGRAM_TYPES_REQUEST,
 *                        PM_PROGRAM_TYPES_REPLY
 *===========================================================================
 */

/**
 * @signal PmProgramTypes
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Get information about the currently configured program types.
 *
 * @member PmProgramTypesRequest
 * @member PmProgramTypesReply
 *
 * @seealso PmAddProgramType
 * @seealso PM_PROGRAM_TYPES_REQUEST
 * @seealso PM_PROGRAM_TYPES_REPLY
 */

/**
 * @macro PM_PROGRAM_TYPES_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Get all configured program types request signal number.
 *
 * @seealso PmProgramTypesRequest
 */
#define PM_PROGRAM_TYPES_REQUEST                  (36204) \
  /* !-SIGNO(struct PmProgramTypesRequest)-! */

/**
 * @type PmProgramTypesRequest
 *
 * @input  sigNo     PM_PROGRAM_TYPES_REQUEST
 *
 * @seealso PmProgramTypes
 */
struct PmProgramTypesRequest
{
  SIGSELECT                  sigNo;
};

/**
 * @macro PM_PROGRAM_TYPES_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Get all configured program types reply signal number.
 *
 * @seealso PmProgramTypesReply
 */
#define PM_PROGRAM_TYPES_REPLY                    (36205) \
  /* !-SIGNO(struct PmProgramTypesReply)-! */

/**
 * @type PmProgramTypesReply
 *
 * @output sigNo          PM_PROGRAM_TYPES_REPLY
 * @output status         The status of the operation.
 * @output program_types_no_of_strings  The number of strings in the
 *                                      program_types array.
 * @output program_types  The program type names are stored one in each
 *                        @link "pm.sig" "PM_PROGRAM_TYPE_PARAM_LENGTH"
 *                        sized element of the array.
 *
 * @seealso PmProgramTypes
 */
struct PmProgramTypesReply
{
  SIGSELECT sigNo;
  PmStatus  status;
  U32       program_types_no_of_strings;
  char      program_types[PM_PROGRAM_TYPE_PARAM_LENGTH];
};

/*
 *===========================================================================
 *                        PM_PROGRAM_TYPE_INFO_REQUEST,
 *                        PM_PROGRAM_TYPE_INFO_REPLY
 *===========================================================================
 */

/**
 * @signal PmProgramTypeInformation
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Get information about a program type.
 *
 * @long A program type defines how a program is to be created. It defines
 *       execution mode and region types for the program.
 *
 * @long Signal @link "pm.sig" "PM_PROGRAM_TYPE_INFO_REQUEST" is used to
 *       get information about the program type with the given name string.
 *
 * @long Signal @link "pm.sig" "PM_PROGRAM_TYPE_INFO_REPLY" is the reply
 *       signal with information.
 *
 * @member PmProgramTypeInfoRequest
 * @member PmProgramTypeInfo
 *
 * @seealso PmAddProgramType
 * @seealso PmProgramRegionType
 * @seealso PM_PROGRAM_TYPE_INFO_REQUEST
 * @seealso PM_PROGRAM_TYPE_INFO_REPLY
 */

/**
 * @macro PM_PROGRAM_TYPE_INFO_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Get information about a program type request signal number.
 *
 * @seealso PmProgramTypeInfoRequest
 */
#define PM_PROGRAM_TYPE_INFO_REQUEST              (36206) \
  /* !-SIGNO(struct PmProgramTypeInfoRequest)-! */

/**
 * @type PmProgramTypeInfoRequest
 *
 * @input sigNo              PM_PROGRAM_TYPE_INFO_REQUEST
 * @input program_type_name  Name of the program type to request information about.
 *
 * @seealso PmProgramTypeInformation
 */
struct PmProgramTypeInfoRequest
{
  SIGSELECT sigNo;
  char      program_type_name[PM_PROGRAM_TYPE_PARAM_LENGTH];
};

/**
 * @macro PM_PROGRAM_TYPE_INFO_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Get information about a program type reply signal number.
 *
 * @seealso PmProgramTypeInfo
 */
#define PM_PROGRAM_TYPE_INFO_REPLY                (36207) \
  /* !-SIGNO(struct PmProgramTypeInfo)-! */

/* NOTE: struct PmProgramTypeInfo is a general structure defined in the
 *       beginning of this file.
 */

/*
 *===========================================================================
 *                        PM_FILE_FORMATS_REQUEST,
 *                        PM_FILE_FORMATS_REPLY
 *===========================================================================
 */

/**
 * @signal PmFileFormats
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Get information about the currently supported file formats.
 *
 * @member PmFileFormatsRequest
 * @member PmFileFormatsReply
 *
 * @seealso PM_FILE_FORMATS_REQUEST
 * @seealso PM_FILE_FORMATS_REPLY
 */

/**
 * @macro PM_FILE_FORMATS_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Get information about the supported file formats request
 *        signal number.
 *
 * @seealso PmFileFormatsRequest
 */
#define PM_FILE_FORMATS_REQUEST                   (36208) \
  /* !-SIGNO(struct PmFileFormatsRequest)-! */

/**
 * @type PmFileFormatsRequest
 *
 * @input sigNo    PM_FILE_FORMATS_REQUEST
 *
 * @seealso PmFileFormats
 */
struct PmFileFormatsRequest
{
  SIGSELECT sigNo;
};

/**
 * @macro PM_FILE_FORMATS_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Get information about the supported file formats reply signal number.
 *
 * @seealso PmFileFormatsReply
 */
#define PM_FILE_FORMATS_REPLY                     (36209) \
  /* !-SIGNO(struct PmFileFormatsReply)-! */

/**
 * @type PmFileFormatsReply
 *
 * @output sigNo         PM_FILE_FORMATS_REPLY
 * @output status        The status of the add program type operation.
 * @output file_formats_no_of_strings The number of strings in the
 *                                    file_formats array.
 * @output file_formats  The file format names (the file format strings are
 *                       stored one in each
 *                       @link "pm.sig" "PM_FILE_FORMAT_NAME_LENGTH"
 *                       sized element of the array.
 *
 * @seealso PmFileFormats
 */
struct PmFileFormatsReply
{
  SIGSELECT  sigNo;
  PmStatus   status;
  U32        file_formats_no_of_strings;
  char       file_formats[PM_FILE_FORMAT_NAME_LENGTH];
};

/*
 *===========================================================================
 *                        PM_INSTALL_LOAD_MODULE_REQUEST,
 *                        PM_INSTALL_LOAD_MODULE_REPLY
 *===========================================================================
 */

/**
 * @signal PmInstallLoadModule
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief    Install a load module
 *
 * @long     Install a load module and associate it with a unique
 *           install handle. If the load module can be relocated, it can be
 *           installed multiple times. If multiple programs are needed but the
 *           system does not support MAS region types, one
 *           installation for each program is needed.
 *
 * @member PmInstallLoadModuleRequest
 * @member PmInstallLoadModuleReply
 *
 * @seealso PM_INSTALL_LOAD_MODULE_REQUEST
 * @seealso PM_INSTALL_LOAD_MODULE_REPLY
 */

/**
 * @macro PM_INSTALL_LOAD_MODULE_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Install load module request signal number.
 *
 * @seealso PmInstallLoadModuleRequest
 */
#define PM_INSTALL_LOAD_MODULE_REQUEST            (36210) \
  /* !-SIGNO(struct PmInstallLoadModuleRequest)-! */

/**
 * @type PmInstallLoadModuleRequest
 *
 * @input sigNo           PM_INSTALL_LOAD_MODULE_REQUEST
 * @input install_handle  The handle to use when referencing the load module
 *                        (user defined). Must be unique to the system.
 * @input options         Option field,
 *                        @link "pm.sig" "PM_LOAD_MODULE_PERSISTENT" and
 *                        @link "pm.sig" "PM_LOAD_MODULE_ABSOLUTE" and
 *                        @link "pm.sig" "PM_LOAD_MODULE_PAGED" and
 *                        @link "pm.sig" "PM_LOAD_MODULE_KERNEL"
 *                        are the available options.
 *                        The options can be combined.
 * @input file_format     The File Format of the load module to install.
 * @input file_name       The name and path of the load module to install.
 * @input conf_no_of_strings  The number of strings in the configuration (conf)
 *                            area.
 * @input conf_size       The size of the configuration (conf) area
 *                        (number of bytes).
 * @input conf            The additional configuration of the load module
 *                        (overrides the configuration field of the load
 *                        module), will be stored as load module configuration.
 *                        Each configuration shall have the syntax of "name=value",
 *                        each configuration is separated by the string
 *                        terminator.
 *
 * @seealso PmInstallLoadModule
 */
struct PmInstallLoadModuleRequest
{
  SIGSELECT     sigNo;
  U32           options;
  char          file_format[PM_FILE_FORMAT_NAME_LENGTH];
  char          file_name[PM_PARENT_FILE_NAME_SIZE];
  char          install_handle[PM_INSTALL_HANDLE_LENGTH];

  U32           conf_no_of_strings;
  U32           conf_size;
  char          conf[1];
};

/**
 * @macro PM_INSTALL_LOAD_MODULE_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Install load module reply signal number.
 *
 * @seealso PmInstallLoadModuleReply
 */
#define PM_INSTALL_LOAD_MODULE_REPLY              (36211) \
  /* !-SIGNO(struct PmInstallLoadModuleReply)-! */

/**
 * @type PmInstallLoadModuleReply
 *
 * @output sigNo           PM_INSTALL_LOAD_MODULE_REPLY
 * @output status          The reply status of the installation.
 * @output install_handle  The handle to use when referencing the load module
 *                         (user defined).
 *
 * @seealso PmInstallLoadModule
 */
struct PmInstallLoadModuleReply
{
  SIGSELECT     sigNo;
  PmStatus      status;
  char          install_handle[PM_INSTALL_HANDLE_LENGTH];
};

/*
 *===========================================================================
 *                        PM_UNINSTALL_LOAD_MODULE_REQUEST,
 *                        PM_UNINSTALL_LOAD_MODULE_REPLY
 *===========================================================================
 */

/**
 * @signal PmUninstallLoadModule
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief  Uninstall a load module.
 *
 * @long   Remove a load module and free it memory resources.
 *         There can be no program created using the specific
 *         load module when the load module is uninstalled.
 *
 * @member PmUninstallLoadModule
 *
 * @seealso PM_UNINSTALL_LOAD_MODULE_REQUEST
 * @seealso PM_UNINSTALL_LOAD_MODULE_REPLY
 */

/**
 * @macro PM_UNINSTALL_LOAD_MODULE_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Uninstall load module request signal number.
 *
 * @seealso PmUninstallLoadModule
 */
#define PM_UNINSTALL_LOAD_MODULE_REQUEST          (36212) \
  /* !-SIGNO(struct PmUninstallLoadModule)-! */

/**
 * @macro PM_UNINSTALL_LOAD_MODULE_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Uninstall load module reply signal number.
 *
 * @seealso PmUninstallLoadModule
 */
#define PM_UNINSTALL_LOAD_MODULE_REPLY            (36213) \
  /* !-SIGNO(struct PmUninstallLoadModule)-! */

/**
 * @type PmUninstallLoadModule
 *
 * @input  sigNo           PM_UNINSTALL_LOAD_MODULE_REQUEST
 * @input  install_handle  The install handle of an installed load module.
 *
 * @output sigNo           PM_UNINSTALL_LOAD_MODULE_REPLY
 * @output install_handle  Copied from request signal in the reply signal.
 * @output status          The status of the uninstall operation.
 */
struct PmUninstallLoadModule
{
  SIGSELECT sigNo;
  PmStatus status;
  char    install_handle[PM_INSTALL_HANDLE_LENGTH];
};

/*
 *===========================================================================
 *                        PM_LOAD_MODULE_INSTALL_HANDLES_REQUEST,
 *                        PM_LOAD_MODULE_INSTALL_HANDLES_REPLY
 *===========================================================================
 */

/**
 * @signal PmLoadModuleInstallHandles
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief  Retrieve the install handles for all installed load modules.
 *
 * @member PmLoadModuleInstallHandlesRequest
 * @member PmLoadModuleInstallHandlesReply
 *
 * @seealso PM_LOAD_MODULE_INSTALL_HANDLES_REQUEST
 * @seealso PM_LOAD_MODULE_INSTALL_HANDLES_REPLY
 */

/**
 * @macro PM_LOAD_MODULE_INSTALL_HANDLES_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief  Retrieve handles for all installed load modules request
 *         signal number.
 *
 * @seealso PmLoadModuleInstallHandlesRequest
 */
#define PM_LOAD_MODULE_INSTALL_HANDLES_REQUEST    (36214) \
  /* !-SIGNO(struct PmLoadModuleInstallHandlesRequest)-! */

/**
 * @type PmLoadModuleInstallHandlesRequest
 *
 * @input sigNo            PM_LOAD_MODULE_INSTALL_HANDLES_REQUEST
 *
 * @seealso PmLoadModuleInstallHandles
 */
struct PmLoadModuleInstallHandlesRequest
{
  SIGSELECT   sigNo;
};

/**
 * @macro PM_LOAD_MODULE_INSTALL_HANDLES_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief  Retrieve handles for all installed load modules reply signal number.
 *
 * @seealso PmLoadModuleInstallHandles
 */
#define PM_LOAD_MODULE_INSTALL_HANDLES_REPLY      (36215) \
  /* !-SIGNO(struct PmLoadModuleInstallHandlesReply)-! */

/**
 * @type PmLoadModuleInstallHandlesReply
 *
 * @output sigNo            PM_LOAD_MODULE_INSTALL_HANDLES_REPLY
 * @output status           The status of the operation.
 * @output install_handles_no_of_strings  The number of strings in the install
 *                          handles area.
 * @output install_handles_size  The size (in bytes) of install_handles.
 * @output install_handles  All install handles (separated with string
 *                          terminator, the string array is also ended with a
 *                          string terminator).
 *
 * @seealso PmLoadModuleInstallHandlesRequest
 */
struct PmLoadModuleInstallHandlesReply
{
  SIGSELECT   sigNo;

  PmStatus    status;

  U32         install_handles_no_of_strings;
  U32         install_handles_size;
  char        install_handles[1];
};

/*
 *===========================================================================
 *                        PM_LOAD_MODULE_INFO_REQUEST,
 *                        PM_LOAD_MODULE_INFO_REPLY
 *===========================================================================
 */

/**
 * @signal  PmLoadModuleInfo
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve information about an installed load module.
 *
 * @member  PmLoadModuleInfoRequest
 * @member  PmLoadModuleInfoReply
 *
 * @seealso PM_LOAD_MODULE_INFO_REQUEST
 * @seealso PM_LOAD_MODULE_INFO_REPLY
 */

/**
 * @macro PM_LOAD_MODULE_INFO_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve information about an installed load module request
 *          signal number.
 *
 * @seealso PmLoadModuleInfoRequest
 */
#define PM_LOAD_MODULE_INFO_REQUEST               (36216) \
  /* !-SIGNO(struct PmLoadModuleInfoRequest)-! */

/**
 * @type PmLoadModuleInfoRequest
 *
 * @input sigNo           PM_LOAD_MODULE_INFO_REQUEST
 * @input install_handle  The handle specifying the installed load module
 *                        (user defined).
 *
 * @seealso PmLoadModuleInfo
 */
struct PmLoadModuleInfoRequest
{
  SIGSELECT   sigNo;
  char        install_handle[PM_INSTALL_HANDLE_LENGTH];
};

/**
 * @macro PM_LOAD_MODULE_INFO_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve information about an installed load module reply
 *          signal number.
 *
 * @seealso PmLoadModuleInfoReply
 */
#define PM_LOAD_MODULE_INFO_REPLY                 (36217) \
  /* !-SIGNO(struct PmLoadModuleInfoReply)-! */

/**
 * @type PmLoadModuleInfoReply
 *
 * @output sigNo            PM_LOAD_MODULE_INFO_REPLY
 * @output install_handle   The handle specifying the installed load module
 *                          (user defined). Copied from request signal.
 * @output status           The reply status of the load module info
 *                          operation.
 * @output options          The installation options of the
 *                          load module (
 *                          @link "pm.sig" "PM_LOAD_MODULE_PERSISTENT"
 *                          and @link "pm.sig" "PM_LOAD_MODULE_ABSOLUTE"
 *                          and @link "pm.sig" "PM_LOAD_MODULE_KERNEL"
 *                          or  @link "pm.sig" "PM_LOAD_MODULE_PAGED").
 * @output entrypoint       The entry point of the load module.
 * @output text_base        The base address of the text region.
 * @output text_size        The size of the text region (bytes).
 * @output data_base        The base address of the data region.
 * @output data_size        The size of the data region (bytes).
 * @output no_of_sections   The number of section in the load module.
 * @output no_of_instances  The number of instances currently created from the
 *                          load module.
 *                          Note: The number of instances is increased every
 *                                time a program is created from the load
 *                                module and every time a dynamic library
 *                                (created from the load module) is imported.
 *                                The number is decreased when a program
 *                                (created from the load module) is killed or
 *                                when a program, that previously imported a
 *                                library created from the load module,
 *                                is killed.
 * @output  file_name       The parent file name (including the absolute path).
 * @output  file_format     The file format of the load module.
 *
 * @seealso PmLoadModuleInfo
 */
struct PmLoadModuleInfoReply
{
  SIGSELECT   sigNo;

  PmStatus    status;
  char        install_handle[PM_INSTALL_HANDLE_LENGTH];
  OSADDRESS   entrypoint;
  U32         options;
  OSADDRESS   text_base;
  U32         text_size;
  OSADDRESS   data_base;
  U32         data_size;
  U32         no_of_sections;
  U32         no_of_instances;
  char        file_format[PM_FILE_FORMAT_NAME_LENGTH];
  char        file_name[PM_PARENT_FILE_NAME_SIZE];
};

/*
 *===========================================================================
 *                        PM_LOAD_MODULE_SECTION_INFO_REQUEST,
 *                        PM_LOAD_MODULE_SECTION_INFO_REPLY
 *===========================================================================
 */

/**
 * @signal  PmLoadModuleSectionInfo
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve information about an installed load module's sections.
 *
 * @long    Request information about section from a specific interval of
 *          section numbers. The interval is defined by a start and an end value.
 *          The values describe the section interval to give information about,
 *          including the section information corresponding to the start and end
 *          values.
 * @break   Example:
 * @break
 *          section_interval_start = 0, section_interval_end = 1.
 *          Means reply information about section 0 and section 1.
 *
 * @member  PmLoadModuleSectionInfoRequest
 * @member  PmLoadModuleSectionInfoReply
 *
 * @seealso PM_LOAD_MODULE_SECTION_INFO_REQUEST
 * @seealso PM_LOAD_MODULE_SECTION_INFO_REPLY
 */

/**
 * @macro PM_LOAD_MODULE_SECTION_INFO_REQUEST
 *
 * @brief Retrieve information about an installed load module's sections.
 *
 * @include "ose_spi/pm.sig"
 *
 * @seealso PmLoadModuleSectionInfoRequest
 */
#define PM_LOAD_MODULE_SECTION_INFO_REQUEST       (36218) \
  /* !-SIGNO(struct PmLoadModuleSectionInfoRequest)-! */

/**
 * @type PmLoadModuleSectionInfoRequest
 *
 * @input sigNo                   PM_LOAD_MODULE_SECTION_INFO_REQUEST
 * @input section_interval_start  The start of the section interval (the lowest
 *                                section number).
 * @input section_interval_end    The end of the section interval (the highest
 *                                section number).
 * @input install_handle          The handle referencing the installed load
 *                                module (user defined).
 *
 * @seealso PmLoadModuleSectionInfo
 */
struct PmLoadModuleSectionInfoRequest
{
  SIGSELECT   sigNo;
  char        install_handle[PM_INSTALL_HANDLE_LENGTH];
  U32         section_interval_start;
  U32         section_interval_end;
};

/**
 * @macro PM_LOAD_MODULE_SECTION_INFO_REPLY
 *
 * @brief Reply signal with information about an installed load module's
 *        sections.
 *
 * @include "ose_spi/pm.sig"
 *
 * @seealso PmLoadModuleSectionInfoReply
 */
#define PM_LOAD_MODULE_SECTION_INFO_REPLY         (36219) \
  /* !-SIGNO(struct PmLoadModuleSectionInfoReply)-! */

/**
 * @type PmLoadModuleSectionInfoReply
 *
 * @output sigNo                 PM_LOAD_MODULE_SECTION_INFO_REPLY
 * @output status                The status of the operation.
 * @output section_interval_start  The start of the section interval
 *                               (the lowest section number).
 *                               Copied from request signal.
 * @output section_interval_end  The end of the section interval (the highest
 *                               section number). Copied from a request signal.
 * @output install_handle        The handle referencing the installed
 *                               load module (user defined).
 *                               Copied from a request signal.
 * @output sections              Array of structs containing information about
 *                               the sections.
 *
 * @seealso PmLoadModuleSectionInfo
 */
struct PmLoadModuleSectionInfoReply
{
  SIGSELECT            sigNo;
  PmStatus             status;
  char                 install_handle[PM_INSTALL_HANDLE_LENGTH];
  U32                  section_interval_start;
  U32                  section_interval_end;
  struct PmLoadModuleSectionInfo sections[1];
};

/*
 *===========================================================================
 *                        PM_LOAD_MODULE_CONFIGURATION_REQUEST,
 *                        PM_LOAD_MODULE_CONFIGURATION_REPLY
 *===========================================================================
 */

/**
 * @signal  PmLoadModuleConfiguration
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve information about an installed load modules configuration.
 *
 * @member PmLoadModuleConfigurationRequest
 * @member PmLoadModuleConfigurationReply
 *
 * @seealso PM_LOAD_MODULE_CONFIGURATION_REQUEST
 * @seealso PM_LOAD_MODULE_CONFIGURATION_REPLY
 */

/**
 * @macro PM_LOAD_MODULE_CONFIGURATION_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve information about installed load modules configuration
 *          request signal number.
 *
 * @seealso PmLoadModuleConfigurationRequest
 */
#define PM_LOAD_MODULE_CONFIGURATION_REQUEST      (36220) \
  /* !-SIGNO(struct PmLoadModuleConfigurationRequest)-! */

/**
 * @type PmLoadModuleConfigurationRequest
 *
 * @input sigNo           PM_LOAD_MODULE_CONFIGURATION_REQUEST
 * @input install_handle  The handle referencing the installed load module
 *                        (user defined).
 *
 * @seealso PmLoadModuleConfiguration
 */
struct PmLoadModuleConfigurationRequest
{
  SIGSELECT   sigNo;
  char        install_handle[PM_INSTALL_HANDLE_LENGTH];
};

/**
 * @macro PM_LOAD_MODULE_CONFIGURATION_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve information about installed load modules configuration
 *          reply signal number.
 *
 * @seealso PmLoadModuleConfigurationReply
 */
#define PM_LOAD_MODULE_CONFIGURATION_REPLY        (36221) \
  /* !-SIGNO(struct PmLoadModuleConfigurationReply)-! */

/**
 * @type PmLoadModuleConfigurationReply
 *
 * @output sigNo           PM_LOAD_MODULE_CONFIGURATION_REPLY
 * @output install_handle  The handle referencing the installed load module
 *                         (user defined).
 * @output status          The status of the operation.
 * @output conf_no_of_strings The number of strings in the configuration (conf)
 *                         area.
 * @output conf_size       The size of conf array (in bytes).
 * @output conf            The configuration. Each configuration have the
 *                         syntax of "name=value", each configuration is
 *                         separated by the string terminator.
 *
 * @seealso PmLoadModuleConfiguration
 */
struct PmLoadModuleConfigurationReply
{
  SIGSELECT   sigNo;

  PmStatus    status;
  char        install_handle[PM_INSTALL_HANDLE_LENGTH];

  U32         conf_no_of_strings;
  U32         conf_size;
  char        conf[1];
};

/*
 *===========================================================================
 *                        PM_CREATE_PROGRAM_REQUEST,
 *                        PM_CREATE_PROGRAM_REPLY
 *===========================================================================
 */

/**
 * @signal  PmCreateProgram
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Create a program using an installed load module.
 *
 * @member  PmCreateProgramRequest
 * @member  PmCreateProgramReply
 *
 * @seealso PM_CREATE_PROGRAM_REQUEST
 * @seealso PM_CREATE_PROGRAM_REPLY
 */

/**
 * @macro PM_CREATE_PROGRAM_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Create a program using an installed load module
 *          request signal number.
 *
 * @seealso PmCreateProgramRequest
 */
#define PM_CREATE_PROGRAM_REQUEST                 (36222) \
  /* !-SIGNO(struct PmCreateProgramRequest)-! */

/**
 * @type PmCreateProgramRequest
 *
 * @input sigNo           PM_CREATE_PROGRAM_REQUEST
 * @input install_handle  The handle referencing the
 *                        installed load module (user defined).
 * @input domain          The protection domain to create the program in. If the
 *                        domain is set to @link "" "PM_NEW_DOMAIN", a new
 *                        domain will be created and the domain identifier will be
 *                        returned with the reply. NOTE: The current implementation
 *                        do not support multiple programs in the same domain.
 * @input conf_no_of_strings  The number of strings in the configuration (conf)
 *                        area.
 * @input conf_size       The size (in bytes) of the conf array.
 * @input conf            The additional configuration of the load module
 *                        (overrides the configuration field of the
 *                        load module).
 *                        Overriding the program type configuration has no
 *                        effect, since it is defined in the install load
 *                        module operation. Each configuration shall have the
 *                        syntax of "name=value", each configuration is
 *                        separated by the string terminator.
 *
 * @seealso PmCreateProgram
 */
struct PmCreateProgramRequest
{
  SIGSELECT     sigNo;

  OSDOMAIN      domain;
  char          install_handle[PM_INSTALL_HANDLE_LENGTH];

  U32           conf_no_of_strings;
  U32           conf_size;
  char          conf[1];
};

/**
 * @macro PM_CREATE_PROGRAM_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Create a program using an installed load module
 *          reply signal number.
 *
 * @seealso PmCreateProgramReply
 */
#define PM_CREATE_PROGRAM_REPLY                   (36223) \
  /* !-SIGNO(struct PmCreateProgramReply)-! */

/**
 * @type PmCreateProgramReply
 *
 * @output sigNo           PM_CREATE_PROGRAM_REPLY
 * @output status          The reply status of the create program operation.
 * @output domain          The protection domain number the program is created in.
 * @output install_handle  The handle referencing the installed load module
 *                         (user defined). This is copied from the request
 *                         signal.
 * @output progpid         The program PID (to be used for signal operations on
 *                         the program).
 * @output main_bid        The block id (to be used to set environment
 *                         variables on the program post creation, but before
 *                         the program is started).
 *
 * @seealso PmCreateProgram
 */
struct PmCreateProgramReply
{
  SIGSELECT   sigNo;
  PmStatus    status;
  OSDOMAIN    domain;
  char        install_handle[PM_INSTALL_HANDLE_LENGTH];
  PROCESS     progpid;
  PROCESS     main_bid;
};

/*
 *===========================================================================
 *                        PM_START_PROGRAM_REQUEST,
 *                        PM_START_PROGRAM_REPLY
 *===========================================================================
 */

/**
 * @signal PmStartProgram
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief  Start a created program with given progpid.
 *
 * @member PmStartProgramRequest
 * @member PmStartProgramReply
 *
 * @seealso PM_START_PROGRAM_REQUEST
 * @seealso PM_START_PROGRAM_REPLY
 */

/**
 * @macro PM_START_PROGRAM_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief  Start a created program with given progpid request signal number.
 *
 * @seealso PmStartProgramRequest
 */
#define PM_START_PROGRAM_REQUEST                  (36224) \
  /* !-SIGNO(struct PmStartProgramRequest)-! */

/**
 * @type PmStartProgramRequest
 *
 * @input sigNo    PM_START_PROGRAM_REQUEST
 * @input progpid  The program PID of the program to start. For historical
 *                 reasons a process, block or segment PID associated with the
 *                 segment the program was created in may also be used.
 *                 (Operations on the core module is not allowed).
 *
 * @seealso PmStartProgram
 */
struct PmStartProgramRequest
{
  SIGSELECT                  sigNo;
  PROCESS                  progpid;
};

/**
 * @macro PM_START_PROGRAM_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief  Start a created program with given progpid reply signal number.
 *
 * @seealso PmStartProgramReply
 */
#define PM_START_PROGRAM_REPLY                    (36225) \
  /* !-SIGNO(struct PmStartProgramReply)-! */

/**
 * @type PmStartProgramReply
 *
 * @output sigNo    PM_START_PROGRAM_REPLY
 * @output status   The status of the start program operation.
 * @output progpid  The program PID of the program to start.
 *                  This is copied from the request signal.
 *
 * @seealso PmStartProgram
 */
struct PmStartProgramReply
{
  SIGSELECT                  sigNo;
  PmStatus                  status;
  PROCESS                  progpid;
};


/*
 *===========================================================================
 *                        PM_KILL_PROGRAM_REQUEST,
 *                        PM_KILL_PROGRAM_REPLY
 *===========================================================================
 */

/**
 * @signal  PmKillProgram
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Kill a created or started program.
 *
 * @member  PmKillProgram
 *
 * @seealso PM_KILL_PROGRAM_REQUEST
 * @seealso PM_KILL_PROGRAM_REPLY
 */

/**
 * @macro PM_KILL_PROGRAM_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Kill a created or started program request signal number.
 *
 * @seealso PmKillProgram
 */
#define PM_KILL_PROGRAM_REQUEST                   (36226) \
  /* !-SIGNO(struct PmKillProgram)-! */

/**
 * @macro PM_KILL_PROGRAM_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Kill a created or started program reply signal number.
 *
 * @seealso PmKillProgram
 */
#define PM_KILL_PROGRAM_REPLY                     (36227) \
  /* !-SIGNO(struct PmKillProgram)-! */

/**
 * @type PmKillProgram
 *
 * @input  sigNo   PM_KILL_PROGRAM_REQUEST
 * @input progpid  The program PID of the program to kill. For historical
 *                 reasons a process, block or segment PID associated with the
 *                 segment the program was created in may also be used.
 *                 (Operations on the core module is not allowed).
 *
 * @output sigNo   PM_KILL_PROGRAM_REPLY
 * @output status  The status of the kill program operation.
 * @output progpid This field is left intact in the reply signal.
 */
struct PmKillProgram
{
  SIGSELECT                  sigNo;
  PmStatus                  status;
  PROCESS                  progpid;
};

/*
 *===========================================================================
 *                        PM_ATTACH_PROGRAM_REQUEST,
 *                        PM_ATTACH_PROGRAM_REPLY
 *===========================================================================
 */

/**
 * @signal  PmAttachProgram
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Attach to a program.
 *
 * @long Request an attach to a program. A notification signal is sent
 *       to the process requesting the attach once the program is killed.
 *       When receiving the attach notification, the client is guaranteed that
 *       the program manager has been notified that the program has died.
 *
 * @long If the program has already been killed, the returned signal will
 *       have its status set to PM_EPROGPID.
 *
 * @member  PmAttachProgram
 * @member  PmAttachProgramNotification
 *
 * @seealso PM_ATTACH_PROGRAM_REQUEST
 * @seealso PM_ATTACH_PROGRAM_REPLY
 */

/**
 * @macro PM_ATTACH_PROGRAM_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Attach to a program request signal number.
 *
 * @seealso PmAttachProgram
 */
#define PM_ATTACH_PROGRAM_REQUEST                 (36228) \
  /* !-SIGNO(struct PmAttachProgram)-! */

/**
 * @macro PM_ATTACH_PROGRAM_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Attach to a program reply signal number.
 *
 * @seealso PmAttachProgram
 */
#define PM_ATTACH_PROGRAM_REPLY                   (36229) \
  /* !-SIGNO(struct PmAttachProgram)-! */

/**
 * @type PmAttachProgram
 *
 * @input  sigNo         PM_ATTACH_PROGRAM_REQUEST
 * @input  progpid       The program PID of the program to attach to. For
 *                       historical reasons a process, block or segment
 *                       PID associated with the segment the program was
 *                       created in may also be used.
 * @input  attach_sigNo  The signal number of the attach notification signal.
 * @input  attach_data_size The size in bytes of the attach_data field.
 * @input  attach_data   The client data of the attach notification signal.
 *
 * @output sigNo         PM_ATTACH_PROGRAM_REPLY
 * @output status        The status of the attach program operation.
 * @output progpid       This field is left intact in the reply signal.
 */
struct PmAttachProgram
{
  SIGSELECT                  sigNo;
  PmStatus                  status;
  PROCESS                  progpid;
  SIGSELECT           attach_sigNo;
  U32             attach_data_size;
  char              attach_data[1];
};

/**
 * @type PmAttachProgramNotification
 *
 * @output sigNo    Signal number specified in the attach request.
 * @output progpid  The program PID of the killed program.
 * @output data     The data specified in the PM_ATTACH_PROGRAM_REQUEST signal.
 */
struct PmAttachProgramNotification
{
  SIGSELECT                  sigNo;
  PROCESS                  progpid;
  char                     data[1];
};

/*
 *===========================================================================
 *                        PM_EXTEND_PROGRAM_MEMORY_REQUEST,
 *                        PM_EXTEND_PROGRAM_MEMORY_REPLY
 *===========================================================================
 */

/**
 * @signal PmExtendProgramMemory
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Extend a programs memory by adding a new memory region to
 *          the program's protection domain.
 *
 * @long  The primary purpose of this signal is to extend the programs pool memory. By using
 *        this signal instead of the memory manager directly, the client
 *        will use the program's program type to determine which memory to
 *        allocate from.
 *
 * @member  PmExtendProgramMemory
 *
 * @seealso PM_EXTEND_PROGRAM_MEMORY_REQUEST
 * @seealso PM_EXTEND_PROGRAM_MEMORY_REPLY
 */

/**
 * @macro PM_EXTEND_PROGRAM_MEMORY_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Extend a programs memory request signal number.
 *
 * @seealso PmExtendProgramMemory
 */
#define PM_EXTEND_PROGRAM_MEMORY_REQUEST          (36230) \
  /* !-SIGNO(struct PmExtendProgramMemory)-! */

/**
 * @macro PM_EXTEND_PROGRAM_MEMORY_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Extend a programs memory reply signal number.
 *
 * @seealso PmExtendProgramMemory
 */
#define PM_EXTEND_PROGRAM_MEMORY_REPLY            (36231) \
  /* !-SIGNO(struct PmExtendProgramMemory)-! */

/**
 * @type PmExtendProgramMemory
 *
 * @input  sigNo    PM_EXTEND_PROGRAM_MEMORY_REQUEST
 * @input progpid   The program PID of the program to extend memory in. For
 *                  historical reasons a process, block or segment PID
 *                  associated with the segment the program was created
 *                  in may also be used.
 * @input  base     The required base of the extended memory region.
 * @input  size     The minimal size of the extended memory region.
 * @input  align    The minimal alignment of the extended memory region.
 * @input  options  The options field has the following parameters
 *                  @link "pm.sig" "PM_SPECIFY_BASE", specify a required
 *                  base address of the program region.
 *                  @link "pm.sig" "PM_SPECIFY_ALIGN", specify a required
 *                  alignment of the base address of the program region.
 *                  PM_SPECIFY_BASE and PM_SPECIFY_ALIGN cannot be combined.
 * @input  program_region_type  The program region type number.
 *
 * @output sigNo    PM_EXTEND_PROGRAM_MEMORY_REPLY
 * @output status   The status of the extend program memory operation.
 * @output progpid  This field is left intact in the reply signal.
 * @output base     The actual base of the region.
 * @output size     The actual size of the region.
 * @output align    The actual alignment of the region.
 */
struct PmExtendProgramMemory
{
  SIGSELECT                  sigNo;
  PmStatus                  status;
  PROCESS                  progpid;
  OSADDRESS                   base;
  U32                         size;
  U32                        align;
  U32                      options;
  U32          program_region_type;
};

/*
 *===========================================================================
 *                        PM_PROGRAMS_REQUEST,
 *                        PM_PROGRAMS_REPLY
 *===========================================================================
 */

/**
 * @signal   PmPrograms
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief  Retrieve the program PIDs for all created programs.
 *
 * @member PmProgramsRequest
 * @member PmProgramsReply
 *
 * @seealso PM_PROGRAMS_REQUEST
 * @seealso PM_PROGRAMS_REPLY
 */

/**
 * @macro PM_PROGRAMS_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief  Retrieve the program PIDs for all created programs
 *         request signal number.
 *
 * @seealso PmProgramsRequest
 */
#define PM_PROGRAMS_REQUEST                       (36232) \
  /* !-SIGNO(struct PmProgramsRequest)-! */

/**
 * @type PmProgramsRequest
 *
 * @input sigNo      PM_PROGRAMS_REQUEST
 *
 * @seealso PmPrograms
 */
struct PmProgramsRequest
{
  SIGSELECT   sigNo;
};

/**
 * @macro PM_PROGRAMS_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief  Retrieve the program PIDs for all created programs
 *         reply signal number.
 *
 * @seealso PmProgramsReply
 */
#define PM_PROGRAMS_REPLY                         (36233) \
  /* !-SIGNO(struct PmProgramsReply)-! */

/**
 * @type PmProgramsReply
 *
 * @output sigNo          PM_PROGRAMS_REPLY
 * @output status         The status of the operation.
 * @output progpids_size  The size (number of elements) of progpids.
 * @output progpids       Array with program PIDs.
 *
 * @seealso PmPrograms
 */
struct PmProgramsReply
{
  SIGSELECT   sigNo;

  PmStatus    status;
  U32         progpids_size;
  PROCESS     progpids[1];
};

/*
 *===========================================================================
 *                        PM_PROGRAM_INFO_REQUEST,
 *                        PM_PROGRAM_INFO_REPLY
 *===========================================================================
 */

/**
 * @signal PmProgramInformation
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve information about a program.
 *
 * @member  PmProgramInfo
 *
 * @seealso PM_PROGRAM_INFO_REQUEST
 * @seealso PM_PROGRAM_INFO_REPLY
 */

/**
 * @macro PM_PROGRAM_INFO_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve information about a program request signal number.
 *
 * @seealso PmProgramInfo
 */
#define PM_PROGRAM_INFO_REQUEST                   (36234) \
  /* !-SIGNO(struct PmProgramInfo)-! */

/**
 * @macro PM_PROGRAM_INFO_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve information about a program reply signal number.
 *
 * @seealso PmProgramInfo
 */
#define PM_PROGRAM_INFO_REPLY                     (36235) \
  /* !-SIGNO(struct PmProgramInfo)-! */

/**
 * @type PmProgramInfo
 *
 * @input  sigNo           PM_PROGRAM_INFO_REQUEST
 * @input progpid          The program PID of the program. For
 *                         historical reasons a process, block or segment PID
 *                         associated with the segment the program was created
 *                         in may also be used. This field is left intact
 *                         in the reply signal.
 *
 * @output sigNo           PM_PROGRAM_INFO_REPLY
 * @output status          The status of the operation.
 * @output install_handle  The handle referencing the installed load module
 *                         (user defined).
 * @output progpid         The program PID of the program. This field is left
 *                         intact in the reply signal.
 * @output domain          The protection domain of the program.
 * @output main_block      The main block PID of the program.
 * @output main_process    The main process PID of the program.
 * @output pool_base       The base of the programs stack pool.
 * @output pool_size       The size of the programs stack pool (bytes).
 * @output uid             Reserved, always 0.
 * @output state           The state of the program
 *                         @link "pm.sig" "PM_PROGRAM_CREATED",
 *                         the program is created but not started.
 *                         @link "pm.sig" "PM_PROGRAM_STARTED",
 *                         the program is started.
 * @output segpid          The memory segment identity
 * @output stk_poolid      The identity of the stack pool
 * @output sig_poolid      The identity of the signal pool
 * @output sig_pool_base   The base of the programs signal pool.
 * @output sig_pool_size   The size of the programs signal pool (bytes).
 * @output heap            The programs heap reference.
 */
struct PmProgramInfo
{
  SIGSELECT   sigNo;

  PmStatus    status;
  char        install_handle[PM_INSTALL_HANDLE_LENGTH];
  PROCESS     progpid;
  OSDOMAIN    domain;
  PROCESS     main_block;
  PROCESS     main_process;
  OSADDRESS   pool_base;  /* Stack pool base. */
  U32         pool_size;  /* Stack pool size. */
  OSUSER      uid;
  U32         state;
  PROCESS     segpid;
  PROCESS     stk_poolid;
  PROCESS     sig_poolid;
  OSADDRESS   sig_pool_base;
  U32         sig_pool_size;
  OSHEAP      heap;
};

/*
 *===========================================================================
 *                        PM_PROGRAM_CONFIGURATION_REQUEST,
 *                        PM_PROGRAM_CONFIGURATION_REPLY
 *===========================================================================
 */

/**
 * @signal  PmProgramConfiguration
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve a specific program's configuration.
 *
 * @member  PmProgramConfigurationRequest
 * @member  PmProgramConfigurationReply
 *
 * @seealso PM_PROGRAM_CONFIGURATION_REQUEST
 * @seealso PM_PROGRAM_CONFIGURATION_REPLY
 */

/**
 * @macro PM_PROGRAM_CONFIGURATION_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve a specific program's configuration request signal number.
 *
 * @seealso PmProgramConfigurationRequest
 */
#define PM_PROGRAM_CONFIGURATION_REQUEST          (36236) \
  /* !-SIGNO(struct PmProgramConfigurationRequest)-! */

/**
 * @type PmProgramConfigurationRequest
 *
 * @input sigNo    PM_PROGRAM_CONFIGURATION_REQUEST
 * @input progpid  The program PID of the program to read configuration from.
 *                 For historical reasons a process, block or segment PID
 *                 associated with the segment the program was created
 *                 in may also be used.
 *
 * @seealso PmProgramConfiguration
 */
struct PmProgramConfigurationRequest
{
  SIGSELECT   sigNo;
  PROCESS     progpid;
};

/**
 * @macro PM_PROGRAM_CONFIGURATION_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve a specific program's configuration reply signal number.
 *
 * @seealso PmProgramConfigurationReply
 */
#define PM_PROGRAM_CONFIGURATION_REPLY            (36237) \
  /* !-SIGNO(struct PmProgramConfigurationReply)-! */

/**
 * @type PmProgramConfigurationReply
 *
 * @output sigNo               PM_PROGRAM_CONFIGURATION_REPLY
 * @output status              The status of the operation.
 * @output progpid             The program PID (of the program to read
 *                             configuration from).
 * @output conf_no_of_strings  The number of strings in the configuration
 *                             (conf) area.
 * @output conf_size           The size (in bytes) of configuration.
 * @output conf                The configuration, syntax (for each
 *                             configuration) "name=value" where each
 *                             configuration is separated
 *                             by a string terminator).
 *
 * @seealso PmProgramConfiguration
 */
struct PmProgramConfigurationReply
{
  SIGSELECT   sigNo;
  PmStatus    status;
  PROCESS     progpid;

  U32         conf_no_of_strings;
  U32         conf_size;
  char        conf[1];
};

/*
 *===========================================================================
 *                        PM_INSTALL_HANDLE_TO_PROGRAM_PID_REQUEST,
 *                        PM_INSTALL_HANDLE_TO_PROGRAM_PID_REPLY
 *===========================================================================
 */

/**
 * @signal  PmInstallHandleToProgramPid
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Translate an install handle to a list of corresponding program PIDs.
 *
 * @long This signal will request the progpids of all programs that has been
 *       created from a given install handle.
 *
 * @member  PmInstallHandleToProgramPidRequest
 * @member  PmInstallHandleToProgramPidReply
 *
 * @seealso PM_INSTALL_HANDLE_TO_PROGRAM_PID_REQUEST
 * @seealso PM_INSTALL_HANDLE_TO_PROGRAM_PID_REPLY
 */

/**
 * @macro PM_INSTALL_HANDLE_TO_PROGRAM_PID_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Translate install handle to list of corresponding program PIDs
 *          request signal number.
 *
 * @seealso PmInstallHandleToProgramPidRequest
 */
#define PM_INSTALL_HANDLE_TO_PROGRAM_PID_REQUEST  (36238) \
  /* !-SIGNO(struct PmInstallHandleToProgramPidRequest)-! */

/**
 * @type PmInstallHandleToProgramPidRequest
 *
 * @input sigNo           PM_INSTALL_HANDLE_TO_PROGRAM_PID_REQUEST
 * @input install_handle  Load module install handle.
 *
 * @seealso PmInstallHandleToProgramPid
 */
struct PmInstallHandleToProgramPidRequest
{
  SIGSELECT                  sigNo;
  char  install_handle[PM_INSTALL_HANDLE_LENGTH];
};

/**
 * @macro PM_INSTALL_HANDLE_TO_PROGRAM_PID_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Translate install handle to list of corresponding program PIDs
 *          reply signal number.
 *
 * @seealso PmInstallHandleToProgramPidReply
 */
#define PM_INSTALL_HANDLE_TO_PROGRAM_PID_REPLY    (36239) \
  /* !-SIGNO(struct PmInstallHandleToProgramPidReply)-! */

/**
 * @type PmInstallHandleToProgramPidReply
 *
 * @output sigNo           PM_INSTALL_HANDLE_TO_PROGRAM_PID_REPLY
 * @output install_handle  the install handle of the load module.
 * @output status          The status of the operation.
 * @output progpids_size   The size (number of elements) of progpids.
 * @output progpids        Array of progpids.
 *
 * @seealso PmInstallHandleToProgramPid
 */
struct PmInstallHandleToProgramPidReply
{
  SIGSELECT                  sigNo;
  PmStatus                  status;
  char  install_handle[PM_INSTALL_HANDLE_LENGTH];
  U32                progpids_size;
  PROCESS              progpids[1];
};

/*
 *===========================================================================
 *                        PM_GET_PROGRAM_PID_REQUEST,
 *                        PM_GET_PROGRAM_PID_REPLY
 *===========================================================================
 */

/**
 * @signal  PmGetProgramPid
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Translate a process, block or segment PID to a progpid
 *
 * @member  PmGetProgramPid
 *
 * @seealso PM_GET_PROGRAM_PID_REQUEST
 * @seealso PM_GET_PROGRAM_PID_REPLY
 */

/**
 * @macro PM_GET_PROGRAM_PID_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Translate a process, block or segment PID to a progpid
 *          request signal number.
 *
 * @seealso PmGetProgramPid
 */
#define PM_GET_PROGRAM_PID_REQUEST                (36240) \
  /* !-SIGNO(struct PmGetProgramPid)-! */

/**
 * @macro PM_GET_PROGRAM_PID_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Translate a process, block or segment PID to a progpid
 *          reply signal number.
 *
 * @seealso PmGetProgramPid
 */
#define PM_GET_PROGRAM_PID_REPLY                  (36241) \
  /* !-SIGNO(struct PmGetProgramPid)-! */

/**
 * @type PmGetProgramPid
 *
 * @input  sigNo    PM_GET_PROGRAM_PID_REQUEST
 * @input  pid      The process/block/segment PID. This field is left intact in
 *                  the reply signal.
 *
 * @output sigNo    PM_GET_PROGRAM_PID_REPLY
 * @output status   The status of the get program PID operation.
 * @output progpid  The program PID.
 */
struct PmGetProgramPid
{
  SIGSELECT                  sigNo;
  PmStatus                  status;
  PROCESS                      pid;
  PROCESS                  progpid;
};

/*
 *===========================================================================
 *                        PM_GET_DOMAIN_NUMBER_REQUEST,
 *                        PM_GET_DOMAIN_NUMBER_REPLY
 *===========================================================================
 */

/**
 * @signal  PmGetDomainNumber
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Translate process, block or program PID to a protection domain number.
 *
 * @member  PmGetDomainNumber
 *
 * @seealso PM_GET_DOMAIN_NUMBER_REQUEST
 * @seealso PM_GET_DOMAIN_NUMBER_REPLY
 */

/**
 * @macro PM_GET_DOMAIN_NUMBER_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Translate process, block or program PID to a protection domain number
 *          request signal number.
 *
 * @seealso PmGetDomainNumber
 */
#define PM_GET_DOMAIN_NUMBER_REQUEST              (36242) \
  /* !-SIGNO(struct PmGetDomainNumber)-! */

/**
 * @macro PM_GET_DOMAIN_NUMBER_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Translate process, block or program PID to a protection domain number
 *          reply signal number.
 *
 * @seealso PmGetDomainNumber
 */
#define PM_GET_DOMAIN_NUMBER_REPLY                (36243) \
  /* !-SIGNO(struct PmGetDomainNumber)-! */

/**
 * @type PmGetDomainNumber
 *
 * @input  sigNo   PM_GET_DOMAIN_NUMBER_REQUEST
 * @input  pid     The process/block/program PID.
 *
 * @output sigNo   PM_GET_DOMAIN_NUMBER_REPLY
 * @output status  The status of the get domain number operation.
 * @output domain  The protection domain number.
 * @output  pid    This field is left intact in the reply signal.
 */
struct PmGetDomainNumber
{
  SIGSELECT                  sigNo;
  PmStatus                  status;
  OSDOMAIN                  domain;
  PROCESS                      pid;
};

/*
 *===========================================================================
 *                        PM_IMPORT_DYNAMIC_LIBRARY_REQUEST,
 *                        PM_IMPORT_DYNAMIC_LIBRARY_REPLY
 *===========================================================================
 */

/**
 * @signal  PmImportDynLibrary
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Import a dynamic library into a running program.
 *
 * @member  PmImportDynLib
 *
 * @seealso PM_IMPORT_DYNAMIC_LIBRARY_REQUEST
 * @seealso PM_IMPORT_DYNAMIC_LIBRARY_REPLY
 *
 * @restriction NOT IMPLEMENTED.
 */

/**
 * @macro PM_IMPORT_DYNAMIC_LIBRARY_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Import a dynamic library into a program request signal number.
 *
 * @seealso PmImportDynLib
 *
 * @restriction NOT IMPLEMENTED.
 */
#define PM_IMPORT_DYNAMIC_LIBRARY_REQUEST         (36244) \
  /* !-SIGNO(struct PmImportDynLib)-! */

/**
 * @macro PM_IMPORT_DYNAMIC_LIBRARY_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Import a dynamic library into a program reply signal number.
 *
 * @seealso PmImportDynLib
 *
 * @restriction NOT IMPLEMENTED.
 */
#define PM_IMPORT_DYNAMIC_LIBRARY_REPLY           (36245) \
  /* !-SIGNO(struct PmImportDynLib)-! */

/**
 * @type PmImportDynLib
 *
 * @include "ose_spi/pm.sig"
 *
 * @input  sigNo           PM_IMPORT_DYNAMIC_LIBRARY_REQUEST
 * @input  progpid         The program to import the library in.
 * @input  install_handle  The handle referencing the installed dynamic library
 *                         (user defined).
 *
 * @output sigNo           PM_IMPORT_DYNAMIC_LIBRARY_REPLY
 * @output  progpid        This field is left intact in the reply signal.
 * @output  install_handle This field is left intact in the reply signal.
 * @output status          The reply status of the import dyn lib operation.
 * @output entrypoint      The entrypoint of the imported dynamic library.
 *
 * @seealso PmImportDynLibrary
 *
 * @restriction NOT IMPLEMENTED.
 */
struct PmImportDynLib
{
  SIGSELECT   sigNo;
  PmStatus    status;
  OSADDRESS   entrypoint;
  PROCESS     progpid;
  char        install_handle[PM_INSTALL_HANDLE_LENGTH];
};

/*
 *===========================================================================
 *                        PM_DYNAMIC_LIBRARIES_INFO_REQUEST,
 *                        PM_DYNAMIC_LIBRARIES_INFO_REPLY
 *===========================================================================
 */

/**
 * @signal PmLibrariesInfo
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve the install handles for all libraries imported by a program.
 *
 * @member PmLibrariesInfoRequest
 * @member PmLibrariesInfoReply
 *
 * @seealso PM_DYNAMIC_LIBRARIES_INFO_REQUEST
 * @seealso PM_DYNAMIC_LIBRARIES_INFO_REPLY
 *
 * @restriction NOT IMPLEMENTED.
 */

/**
 * @macro PM_DYNAMIC_LIBRARIES_INFO_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve install handles for all libraries imported by a program
 *          request signal number.
 *
 * @seealso PmLibrariesInfoRequest
 *
 * @restriction NOT IMPLEMENTED.
 */
#define PM_DYNAMIC_LIBRARIES_INFO_REQUEST         (36246) \
  /* !-SIGNO(struct PmLibrariesInfoRequest)-! */

/**
 * @type PmLibrariesInfoRequest
 *
 * @input sigNo    PM_DYNAMIC_LIBRARIES_INFO_REQUEST
 * @input progpid  The program PID of program.
 *
 * @seealso PmLibrariesInfo
 *
 * @restriction NOT IMPLEMENTED.
 */
struct PmLibrariesInfoRequest
{
  SIGSELECT   sigNo;
  PROCESS     progpid;
};

/**
 * @macro PM_DYNAMIC_LIBRARIES_INFO_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Retrieve install handles for all libraries imported by a program
 *          reply signal number.
 *
 * @seealso PmLibrariesInfoReply
 *
 * @restriction NOT IMPLEMENTED.
 */
#define PM_DYNAMIC_LIBRARIES_INFO_REPLY           (36247) \
  /* !-SIGNO(struct PmLibrariesInfoReply)-! */

/**
 * @type PmLibrariesInfoReply
 *
 * @output sigNo                    PM_DYNAMIC_LIBRARIES_INFO_REPLY
 * @output status                   The status of the operation.
 * @output progpid                  The program PID of a program. This field is
 *                                  copied from the request signal.
 * @output libraries_no_of_strings  The number of strings in the libraries
 *                                  area.
 * @output libraries_size           The size (in bytes) of libraries.
 * @output libraries                All install handles of the dynamic
 *                                  libraries (separated by string
 *                                  terminator).
 *
 * @seealso PmLibrariesInfo
 *
 * @restriction NOT IMPLEMENTED.
 */
struct PmLibrariesInfoReply
{
  SIGSELECT   sigNo;
  PmStatus    status;
  PROCESS     progpid;

  U32         libraries_no_of_strings;
  U32         libraries_size;
  char        libraries[1];
};

/*
 *===========================================================================
 *                        PM_LOAD_MODULE_CHECKSUM_REQUEST
 *                        PM_LOAD_MODULE_CHECKSUM_REPLY
 *===========================================================================
 */

/**
 * @signal  PmLoadModuleChecksum
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief    Request/reply for calculation of a load modules checksum
 *           and an comparison of the checksum against the load
 *           module data of the PM.
 *
 * @long    The first request for calculation of the core module
 *          checksum will not result in an comparison against an
 *          expected checksum value and thus fails.
 *
 * @member  PmLoadModuleChecksum
 *
 * @seealso PM_LOAD_MODULE_CHECKSUM_REQUEST
 * @seealso PM_LOAD_MODULE_CHECKSUM_REPLY
 */

/**
 * @macro   PM_LOAD_MODULE_CHECKSUM_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief    Calculate/check a load modules checksum request signal number.
 *
 * @seealso PmLoadModuleChecksum
 */
#define PM_LOAD_MODULE_CHECKSUM_REQUEST           (36248) \
  /* !-SIGNO(struct PmLoadModuleChecksum)-! */

/**
 * @macro   PM_LOAD_MODULE_CHECKSUM_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief    Calculate/check a load modules checksum reply signal number.
 *
 * @seealso PmLoadModuleChecksum
 */
#define PM_LOAD_MODULE_CHECKSUM_REPLY             (36249) \
  /* !-SIGNO(struct PmLoadModuleChecksum)-! */

/**
 * @type     PmLoadModuleChecksum
 *
 * @input  sigNo                    PM_LOAD_MODULE_CHECKSUM_REQUEST
 * @input  install_handle           The load modules install handle.
 *
 * @output sigNo                    PM_LOAD_MODULE_CHECKSUM_REPLY
 * @output status                   The status of the operation.
 * @output checksum                 The calculated checksum.
 */
struct PmLoadModuleChecksum
{
  SIGSELECT   sigNo;
  PmStatus    status;
  char        install_handle[PM_INSTALL_HANDLE_LENGTH];
  U32              checksum;
};

/*
 *===========================================================================
 *                        PM_MOVE_PROGRAM_TO_CPU_REQUEST
 *                        PM_MOVE_PROGRAM_TO_CPU_REPLY
 *===========================================================================
 */

/**
 * @signal  PmMoveProgramToCPU
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Move a program to a specific CPU. For use in SMP systems.
 *
 * @member  PmMoveProgramToCPU
 *
 * @seealso PM_MOVE_PROGRAM_TO_CPU_REQUEST
 * @seealso PM_MOVE_PROGRAM_TO_CPU_REPLY
 */

/**
 * @macro   PM_MOVE_PROGRAM_TO_CPU_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Move a program to a specific CPU (SMP systems) request signal number.
 *
 * @seealso PmMoveProgramToCPU
 */
#define PM_MOVE_PROGRAM_TO_CPU_REQUEST          (36250) \
  /* !-SIGNO(struct PmMoveProgramToCPU)-! */

/**
 * @macro   PM_MOVE_PROGRAM_TO_CPU_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Move a program to a specific CPU (SMP systems) reply signal number.
 *
 * @seealso PmLoadModuleChecksum
 */
#define PM_MOVE_PROGRAM_TO_CPU_REPLY             (36251) \
  /* !-SIGNO(struct PmMoveProgramToCPU)-! */

/**
 * @type     PmMoveProgramToCPU
 *
 * @input  sigNo                    PM_MOVE_PROGRAM_TO_CPU_REQUEST
 * @input  progpid                  The program PID.
 * @input  cpu                      The CPU id the program should be moved to.
 *
 * @output sigNo                    PM_MOVE_PROGRAM_TO_CPU_REPLY
 * @output status                   The status of the operation.
 */
struct PmMoveProgramToCPU
{
  SIGSELECT   sigNo;
  PmStatus    status;
  PROCESS     progpid;
  cpuid_t     cpu;
};


/*
 *===========================================================================
 *                        PM_CONFIGURE_LOAD_MEASUREMENT_REQUEST
 *                        PM_CONFIGURE_LOAD_MEASUREMENT_REPLY
 *===========================================================================
 */

/**
 * @signal  PmConfigureLoadMeasurement
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Configures the per program load measurements performed by the PM.
 *
 * @member  PmConfigureLoadMeasurement
 *
 * @seealso PM_CONFIGURE_LOAD_MEASUREMENT_REQUEST
 * @seealso PM_CONFIGURE_LOAD_MEASUREMENT_REPLY
 */

/**
 * @macro   PM_CONFIGURE_LOAD_MEASUREMENT_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Configures per program load measurements by PM request signal number.
 *
 * @seealso PmConfigureLoadMeasurement
 */
#define PM_CONFIGURE_LOAD_MEASUREMENT_REQUEST          (36256) \
  /* !-SIGNO(struct PmConfigureLoadMeasurement)-! */

/**
 * @macro   PM_CONFIGURE_LOAD_MEASUREMENT_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Configures per program load measurements by PM reply signal number.
 *
 * @seealso PmConfigureLoadMeasurement
 */
#define PM_CONFIGURE_LOAD_MEASUREMENT_REPLY             (36257) \
  /* !-SIGNO(struct PmConfigureLoadMeasurement)-! */

/**
 * @type     PmConfigureLoadMeasurement
 *
 * @input  sigNo                    PM_CONFIGURE_LOAD_MEASUREMENT_REQUEST
 * @input  enable                   Enable the load measurements.
 * @input  sampleInterval           The interval in ms. between load samples.
 * @input  maxProgReports           The maximum number of programs in any one report.
 * @input  priority                 The priority threshold. Only processes with higher priority than
 *                                  the value will contribute to the load.
 *
 * @output sigNo                    PM_CONFIGURE_LOAD_MEASUREMENT_REPLY
 * @output status                   The status of the operation.
 */
struct PmConfigureLoadMeasurement
{
  SIGSELECT   sigNo;
  PmStatus    status;
  U32         enable;
  U32         sampleInterval;
  U32         maxProgReports;
  U32         priority;
};

/*
 *===========================================================================
 *                        PM_GET_LOAD_MEASUREMENT_REQUEST
 *                        PM_GET_LOAD_MEASUREMENT_REPLY
 *===========================================================================
 */

/**
 * @signal  PmGetLoadMeasurement
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Get the per program load measurements.
 *
 * @member  PmGetLoadMeasurement
 *
 * @seealso PM_GET_LOAD_MEASUREMENT_REQUEST
 * @seealso PM_GET_LOAD_MEASUREMENT_REPLY
 */

/**
 * @macro   PM_GET_LOAD_MEASUREMENT_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Get the per program load measurements request signal number.
 *
 * @seealso PmGetLoadMeasurement
 */
#define PM_GET_LOAD_MEASUREMENT_REQUEST          (36258) \
  /* !-SIGNO(struct PmGetLoadMeasurement)-! */

/**
 * @macro   PM_GET_LOAD_MEASUREMENT_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief   Get the per program load measurements reply signal number.
 *
 * @seealso PmGetLoadMeasurement
 */
#define PM_GET_LOAD_MEASUREMENT_REPLY             (36259) \
  /* !-SIGNO(struct PmGetLoadMeasurement)-! */

/**
 * @type     PmGetLoadMeasurement
 *
 * @input  sigNo                    PM_GET_LOAD_MEASUREMENT_REQUEST
 *
 * @output sigNo                    PM_GET_LOAD_MEASUREMENT_REPLY
 * @output status                   The status of the operation.
 * @output interval                 Time of this measurement interval
 *                                  (unspecified unit).
 * @output programsCount            Number of PmProgramLoad structs
 *                                  in programs list.
 * @output programs                 List of PmProgramLoad data.
 */
struct PmGetLoadMeasurement
{
  SIGSELECT             sigNo;
  PmStatus              status;
  U32                   interval;
  U32                   programsCount;
  struct PmProgramLoad  programs[1];
};

/*
 *===========================================================================
 *                        PM_SUBSCRIBE_EVENT_REQUEST,
 *                        PM_SUBSCRIBE_EVENT_REPLY
 *===========================================================================
 */

/**
 * @signal PmSubscribeEvent
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief  Subscribe to a given internal PM event.
 *
 * @member PmSubscribeEventRequest
 * @member PmSubscribeEventReply
 *
 * @seealso PM_SUBSCRIBE_EVENT_REQUEST
 * @seealso PM_SUBSCRIBE_EVENT_REPLY
 */

/**
 * @macro PM_SUBSCRIBE_EVENT_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Subscribe to a given internal PM event request signal number.
 *
 * @seealso PmSubscribeEventRequest
 */
#define PM_SUBSCRIBE_EVENT_REQUEST                  (36260) \
  /* !-SIGNO(struct PmSubscribeEventRequest)-! */

/**
 * @type PmSubscribeEventRequest
 *
 * @input sigNo    PM_SUBSCRIBE_EVENT_REQUEST
 * @input PmEvent  The event to subscribe to.
 *
 * @seealso PmSubscribeEvent
 */
struct PmSubscribeEventRequest
{
  SIGSELECT                  sigNo;
  U32                      PmEvent;
};

/**
 * @macro PM_SUBSCRIBE_EVENT_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Subscribe to a given internal PM event reply signal number.
 *
 * @seealso PmSubscribeEventReply
 */
#define PM_SUBSCRIBE_EVENT_REPLY                    (36261) \
  /* !-SIGNO(struct PmSubscribeEventReply)-! */

/**
 * @type PmSubscribeEventReply
 *
 * @output sigNo    PM_SUBSCRIBE_EVENT_REPLY
 * @output status   The status of the subscribe event operation.
 *
 * @seealso PmSubscribeEvent
 */
struct PmSubscribeEventReply
{
  SIGSELECT                  sigNo;
  PmStatus                  status;
};

/*
 *===========================================================================
 *                        PM_UNSUBSCRIBE_EVENT_REQUEST,
 *                        PM_UNSUBSCRIBE_EVENT_REPLY
 *===========================================================================
 */

/**
 * @signal PmUnsubscribeEvent
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief  Unsubscribe to a given internal PM event.
 *
 * @member PmUnsubscribeEventRequest
 * @member PmUnsubscribeEventReply
 *
 * @seealso PM_UNSUBSCRIBE_EVENT_REQUEST
 * @seealso PM_UNSUBSCRIBE_EVENT_REPLY
 */

/**
 * @macro PM_UNSUBSCRIBE_EVENT_REQUEST
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Unsubscribe to a given internal PM event request signal number.
 *
 * @seealso PmUnsubscribeEventRequest
 */
#define PM_UNSUBSCRIBE_EVENT_REQUEST                  (36262) \
  /* !-SIGNO(struct PmUnsubscribeEventRequest)-! */

/**
 * @type PmUnsubscribeEventRequest
 *
 * @input sigNo    PM_UNSUBSCRIBE_EVENT_REQUEST
 * @input PmEvent  The event to unsubscribe to.
 *
 * @seealso PmUnsubscribeEvent
 */
struct PmUnsubscribeEventRequest
{
  SIGSELECT                  sigNo;
  U32                      PmEvent;
};

/**
 * @macro PM_UNSUBSCRIBE_EVENT_REPLY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Unsubscribe to a given internal PM event reply signal number.
 *
 * @seealso PmUnsubscribeEventReply
 */
#define PM_UNSUBSCRIBE_EVENT_REPLY                    (36263) \
  /* !-SIGNO(struct PmUnsubscribeEventReply)-! */

/**
 * @type PmUnsubscribeEventReply
 *
 * @output sigNo    PM_UNSUBSCRIBE_EVENT_REPLY
 * @output status   The status of the unsubscribe event operation.
 *
 * @seealso PmUnsubscribeEvent
 */
struct PmUnsubscribeEventReply
{
  SIGSELECT                  sigNo;
  PmStatus                  status;
};

/*
 *===========================================================================
 *                        PM_EVENT_NOTIFY
 *===========================================================================
 */

/**
 * @signal PmEventNotify
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Signal sent by PM to client notifying it that an event it
 *        subscribes to has occurred.
 *
 * @member PmEventNotify
 *
 * @seealso PM_EVENT_NOTIFY
 */

/**
 * @macro PM_EVENT_NOTIFY
 *
 * @include "ose_spi/pm.sig"
 *
 * @brief Notify client of PM internal event signal number.
 *
 * @seealso PmEventNotify
 */
#define PM_EVENT_NOTIFY                      (36264) \
  /* !-SIGNO(struct PmEventNotify)-! */

/**
 * @type PmEventNotify
 *
 * @field sigNo    PM_EVENT_NOTIFY
 * @field PmEvent  The event that occurred
 * @field data     Extra information regarding the event that occurred.
 *                 See each event for detailed description of content in
 *                 each case. Event names are of the form PM_EVENT_<>.
 *
 * @seealso PmEventNotify
 */
struct PmEventNotify
{
  SIGSELECT                  sigNo;
  U32                      PmEvent;
  char                     data[1];
};

#ifdef __cplusplus
}
#endif

#endif /* #ifndef PM_SIG_ */
