/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @public
 * @toc Program_Management:
 * @file ose_spi/pm_defs.h
 * @version
 * @brief The program manager status values, load module configurations and
 *        program manager configuration interfaces.
 * @long  This file defines constants for possible status values that can be
 *        returned by the program manager or the OSE ELF run-time loader (RTL).
 *        This file also define constants for load module configuration names
 *        interpreted by the program manager and the RTL.
 */

#ifndef PM_DEFS_H_
#define PM_DEFS_H_

/***********************************************************
 *
 * INCLUDE FILES
 *
 ***********************************************************/

#include "ose.h"
#include "osetypes.h"

#ifdef __cplusplus
extern "C" {
#endif

/***********************************************************
 *
 * CONSTANTS AND DEFINITIONS
 *
 ***********************************************************/

/**
 * @macro PM_PGM_NAME
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The name of the PM program manager process.
 */
#define PM_PGM_NAME          "ose_program_mgr"

/**
 * @macro   PM_CORE_PROGRAM_TYPE_NAME
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The program type name of core.
 */
#define PM_CORE_PROGRAM_TYPE_NAME "SYSTEM"

/**
 * @macro   PM_MINIMAL_NO_OF_REGIONS
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The minimal number of program regions.
 *        The regions are text, data and pool region.
 */
#define PM_MINIMAL_NO_OF_REGIONS  3

/**
 * @type PmStatus
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The PM status (PmStatus) type specifier define the value type of a
 *        program manager operation status.
 */
typedef U32 PmStatus;

/*
 *===========================================================================
 *                        PmStatus definitions
 *===========================================================================
 *
 * Description
 * -----------
 * The PM status definitions describe the status of a program manager
 * operation. The PM status definitions are always of type PmStatus
 * when responded from the Program Manager.
 *
 */

/***********************************************************
 * General statuses.
 ***********************************************************/

/**
 * @macro PM_SUCCESS
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The operation was successful.
 */
#define PM_SUCCESS                               0x00000000

/**
 * @macro PM_EFEATURE_NOT_SUPPORTED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A requested feature was not supported.
 */
#define PM_EFEATURE_NOT_SUPPORTED  	         0x00000001

/**
 * @macro PM_ENOT_IN_CORE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The program manger was not executing as part
 *        of the core program.
 */
#define PM_ENOT_IN_CORE                          0x00000002

/**
 * @macro PM_ENO_PROGRAM_MANAGER
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The program manager process was not found
 * by the functional API.
 */
#define PM_ENO_PROGRAM_MANAGER                   0x00000003


/***********************************************************
 * Program types related statuses.
 ***********************************************************/

/**
 * @macro PM_EILLEGAL_PROGRAM_TYPE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The program type referenced in a signal
 *        was not found.
 */
#define PM_EILLEGAL_PROGRAM_TYPE 	         0x00010000

/**
 * @macro PM_EILLEGAL_REGION_TYPE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The program region type referenced in a
 *        signal was not found.
 */
#define PM_EILLEGAL_REGION_TYPE 	         0x00010001

/**
 * @macro PM_EPROGRAM_TYPE_SYNTAX_ERROR
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The program type has illegal syntax.
 */
#define PM_EPROGRAM_TYPE_SYNTAX_ERROR            0x00010002

/**
 * @macro PM_EPROGRAM_TYPE_PARAM_NAME_CORRUPT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A program type name was too long.
 */
#define PM_EPROGRAM_TYPE_PARAM_NAME_CORRUPT      0x00010003

/**
 * @macro PM_EPROGRAM_TYPE_PARAM_SUPER_CORRUPT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A program execution mode (super parameter in add program type
 *        signals) was not correctly set.
 */
#define PM_EPROGRAM_TYPE_PARAM_SUPER_CORRUPT     0x00010004

/**
 * @macro PM_EPROGRAM_TYPE_PARAM_REGIONS
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief An add program type signal program_type_size
 *        parameter value do not correspond to the size
 *        of the signal.
 */
#define PM_EPROGRAM_TYPE_PARAM_REGIONS           0x00010005

/**
 * @macro PM_EILLEGAL_REGION_LOG_MEM_TYPE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A supervisor mode program type could not be
 *        added since it specified a region type with
 *        MAS-mapped logical memory.
 */
#define PM_EILLEGAL_REGION_LOG_MEM_TYPE          0x00010006

/***********************************************************
 * File format related statuses.
 ***********************************************************/

/**
 * @macro PM_EFILE_FORMAT_NOT_SUPPORTED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The referenced file format was not supported.
 */
#define PM_EFILE_FORMAT_NOT_SUPPORTED            0x00020000

/***********************************************************
 * Install handle related statuses.
 ***********************************************************/

/**
 * @macro PM_EINSTALL_HANDLE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The install handle specified was not found.
 */
#define PM_EINSTALL_HANDLE                       0x00030000

/**
 * @macro PM_EINSTALL_HANDLE_ALREADY_INSTALLED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The install_handle was already in use,
 *        the install operation was aborted.
 */
#define PM_EINSTALL_HANDLE_ALREADY_INSTALLED     0x00030001

/***********************************************************
 * Program related statuses.
 ***********************************************************/

/**
 * @macro PM_EDOMAIN_NOT_FOUND
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The referenced domain was not found.
 */
#define PM_EDOMAIN_NOT_FOUND                     0x00040000

/**
 * @macro PM_EPROGRAM_CONFIGURATION
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The program configuration was not complete
 *        or corrupt.
 */
#define PM_EPROGRAM_CONFIGURATION                0x00040001

/**
 * @macro PM_EOPTION_UNKNOWN
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief An option field of a PM signal included an unknown or
 *        incorrect option.
 * @long  (E.g. the PmExtendProgramMemory.option field).
 */
#define PM_EOPTION_UNKNOWN                       0x00040002

/**
 * @macro PM_EBLOCK_CREATION_FAILED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The main block creation failed during program creation.
 */
#define PM_EBLOCK_CREATION_FAILED                0x00040003

/**
 * @macro PM_EPROCESS_CREATION_FAILED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The main process creation failed during program creation.
 */
#define PM_EPROCESS_CREATION_FAILED              0x00040004

/**
 * @macro PM_EMULTIPLE_PROGRAMS_IN_DOMAIN
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief Multiple programs in one domain is not supported with
 *        the system's memory configuration.
 */
#define PM_EMULTIPLE_PROGRAMS_IN_DOMAIN          0x00040005

/**
 * @macro PM_EINSTALL_HANDLE_IN_USE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The install handle of a create program was already
 *        in use.
 */
#define PM_EINSTALL_HANDLE_IN_USE                0x00040006

/**
 * @macro PM_EOPERATION_ON_CORE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief An disallowed operation was requested on the core program.
 */
#define PM_EOPERATION_ON_CORE                    0x00040007

/**
 * @macro PM_EPROGRAM_ALREADY_STARTED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A start program request could not be performed
 *        because the program has already started.
 */
#define PM_EPROGRAM_ALREADY_STARTED              0x00040008

/* @macro PM_ECORE_PROGRAM_MUST_BE_SUPERVISOR
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief Attempted to create a program with progpid 0 in user mode.
 *        This is not allowed.
 */
#define PM_ECORE_PROGRAM_MUST_BE_SUPERVISOR      0x00040009

/**
 * @macro PM_EPROGRAM_CONFIGURATION_POOL_SIZE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The pool size in the configuration has an illegal value.
 */
#define PM_EPROGRAM_CONFIGURATION_POOL_SIZE      0x0004000A

/**
 * @macro PM_EHEAP_CREATION_FAILED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The Program Manager failed to create the initial heap
 * fragment at program creation, either due to an out of memory
 * situation or an invalid configuration.
 */
#define PM_EHEAP_CREATION_FAILED                 0x0004000B

/**
 * @macro PM_EILLEGAL_POOL_EXTEND_PARAMS
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The pool extend parameters in the configuration
 *        are not consistent with each other.
 *
 * @long  The pool extension related program parameters are
 *        subject to a number of constraints.
 * @long  The initial pool size must be smaller than the
 *        maximum pool size.
 * @long  The pool extend fragment size must be larger than
 *        the largest possible pool object (signals or stacks)
 *        plus 1024 bytes for administration data.
 * @long  If the load module has a supervisor mode program type,
 *        it is not allowed to simultaneously request an
 *        extendible pool (by specifying OSE_LM_POOL_MAX_SIZE).
 */
#define PM_EILLEGAL_POOL_EXTEND_PARAMS           0x0004000C

/***********************************************************
 * Program identifier related statuses.
 ***********************************************************/

/**
 * @macro PM_EPROGPID
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A supplied progpid was not recognized.
 */
#define PM_EPROGPID                              0x00050000

/***********************************************************
 * Memory manager related statuses.
 ***********************************************************/

/**
 * @macro PM_EREGION_DELETION_FAILED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A memory region could not be deleted.
 */
#define PM_EREGION_DELETION_FAILED               0x00060000

/**
 * @macro PM_EREGION_CREATION_FAILED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A memory region could not be created.
 */
#define PM_EREGION_CREATION_FAILED               0x00060001

/**
 * @macro PM_EDOMAIN_DELETION_FAILED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A domain could not be deleted.
 */
#define PM_EDOMAIN_DELETION_FAILED               0x00060002

/**
 * @macro PM_EDOMAIN_CREATION_FAILED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A domain could not be created.
 */
#define PM_EDOMAIN_CREATION_FAILED               0x00060003

/***********************************************************
 * Load Module related statuses.
 ***********************************************************/

/**
 * @macro PM_EUNINSTALL_PROGRAM_EXISTS
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The load module cannot be uninstalled, at least
 *        one program created from the load module exists.
 */
#define PM_EUNINSTALL_PROGRAM_EXISTS             0x00070000

/**
 * @macro PM_ESECTION_INTERVAL
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The section interval does not follow the given syntax.
 */
#define PM_ESECTION_INTERVAL                     0x00070002

/**
 * @macro PM_ECORRUPT_CONFIGURATION
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The configuration of a load module was corrupt.
 */
#define PM_ECORRUPT_CONFIGURATION                0x00070003

/**
 * @macro PM_ENO_MAGIC
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The magic string in the configuration was missing
 *        or it was corrupt. This is an internal error.
 */
#define PM_ENO_MAGIC                             0x00070004

/**
 * @macro PM_EPARAMS_CONF_SIZE_CONF_NO_OF_STRINGS
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief Illegal combination of parameters
 *        'conf_no_of_strings' and 'conf_size'. One
 *         of these variables is set to zero(0).
 */
#define PM_EPARAMS_CONF_SIZE_CONF_NO_OF_STRINGS  0x00070005

/**
 * @macro PM_EPARAM_CONF_SIZE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The variable 'conf_size' is not correct. The
 *        signal allocated for installation is not big
 *        enough to contain configuration with that size.
 */
#define PM_EPARAM_CONF_SIZE                      0x00070006

/**
 * @macro PM_EPARAM_CONF_NO_OF_STRINGS
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The variable 'conf_no_of_strings' is not correct.
 *        The configuration has more or less entries than
 *        the variable implies.
 */
#define PM_EPARAM_CONF_NO_OF_STRINGS             0x00070007

/**
 * @macro PM_ENO_PROGRAM_TYPE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The configuration did not contain any program type,
 *        or there was a syntax error in the entry.
 */
#define PM_ENO_PROGRAM_TYPE                      0x00070008

/**
 * @macro PM_EFILE_NAME_CORRUPT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The file name is too long or not terminated.
 */
#define PM_EFILE_NAME_CORRUPT                    0x00070009

/**
 * @macro PM_EBAD_CHECKSUM
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The checksum of a load module was not correct.
 */
#define PM_EBAD_CHECKSUM                         0x0007000A

/**
 * @macro PM_ENO_CHECKSUM
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The checksum of a load module was not
 *              calculatable (alien load module).
 */
#define PM_ENO_CHECKSUM                         0x0007000B

/***********************************************************
 * RTL related statuses.
 ***********************************************************/

/**
 * @macro PM_ERTL_INTERNAL_ERROR
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The RTL used when installing a load module
 *        reply corrupted parameters.
 */
#define PM_ERTL_INTERNAL_ERROR                   0x00080000

/**
 * @macro PM_ERTL_LOST
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A removed RTL is required for an operation.
 */
#define PM_ERTL_LOST                             0x00080001

/**
 * @macro PM_ENOT_FOUND
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The requested RTL can not be found.
 */
#define PM_ENOT_FOUND      			 0x00080002

/***********************************************************
 * SMP related statuses.
 ***********************************************************/

/**
 * @macro PM_ECPUID
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The supplied CPU ID was not recognized.
 */
#define PM_ECPUID                                0x00090000

/**
 * @macro PM_ELOCKED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The program is locked to the CPU.
 */
#define PM_ELOCKED                               0x00090001

/***********************************************************
 * Measurement related statuses.
 ***********************************************************/

/**
 * @macro PM_EMEAS_NOT_ENABLED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The measurements have not been enabled.
 */
#define PM_EMEAS_NOT_ENABLED                     0x000A0000

/**
 * @macro PM_EMEAS_RMM_ERROR
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The measurements cannot be performed due to an error
 *        in the RMM.
 */
#define PM_EMEAS_RMM_ERROR                       0x000A0001

/**
 * @macro PM_EMEAS_CONFIG_ERROR
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The measurements are not configured correctly.
 */
#define PM_EMEAS_CONFIG_ERROR                       0x000A0002

/***********************************************************
 * Event subscription related statuses.
 ***********************************************************/

/**
 * @macro PM_EEVENT_UNKNOWN
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The event requested is not supported by PM.
 */
#define PM_EEVENT_UNKNOWN                         0x000B0000

/**
 * @macro PM_EALREADY_SUBSCRIBED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The client already subscribes to the event.
 */
#define PM_EALREADY_SUBSCRIBED                    0x000B0001

/**
 * @macro PM_ENOT_SUBSCRIBED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The client does not subscribe to the event.
 */
#define PM_ENOT_SUBSCRIBED                        0x000B0002

/***********************************************************
 * RTL statuses, reported by RTL.
 ***********************************************************/

/**
 * @macro PM_RTL_STATUS_BASE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The RTL status base.
 * @long All RTL related statuses has this status base in the 16 upper bits.
 *       For information about the status message, read the RTL documentation.
 */
#define PM_RTL_STATUS_BASE                       0xFFFF0000

/*
 * General RTL statuses.
 */

/**
 * @macro RTL_EALREADY_CONNECTED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The RTL is already connected.
 */
#define RTL_EALREADY_CONNECTED 			 0xffff0000

/**
 * @macro RTL_EILLEGAL_CONFIGURATION
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The load module configuration or the override
 *        configuration has illegal syntax.
 */
#define RTL_EILLEGAL_CONFIGURATION		 0xffff0001

/**
 * @macro RTL_EREAD_FAIL
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A read error occurred while reading a load module file.
 */
#define RTL_EREAD_FAIL 			         0xffff0002

/**
 * @macro RTL_EOPEN_FAIL
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The load module file could not be opened.
 *
 * @long One example is if the file is not in correct format,
 *       e.g. mismatch in architecture or endian.
 */
#define RTL_EOPEN_FAIL 			         0xffff0003

/**
 * @macro RTL_EUNSUPPORTED_RELOCATION
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief An unsupported relocation was encountered in the load module.
 */
#define RTL_EUNSUPPORTED_RELOCATION 	         0xffff0004

/**
 * @macro RTL_EMEMORY_ALLOC_DENIED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A memory region allocation was not successful.
 */
#define RTL_EMEMORY_ALLOC_DENIED 		 0xffff0005

/**
 * @macro RTL_EINTERNAL_WRITE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief An internal RTL error occurred. RTL was unable to write to a region.
 */
#define RTL_EINTERNAL_WRITE		         0xffff0006

/**
 * @macro RTL_EINTERFACE_MISMATCH
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The RTL signal interface did not match the PM's rtl.sig
 *        signal interface implementation.
 */
#define RTL_EINTERFACE_MISMATCH		         0xffff0007

/**
 * @macro RTL_EPROGRAM_TYPE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The load module configuration does not contain
 *        program type information, e.g. 'OSE_LM_PROGRAM_TYPE'
 *        is missing.
 */
#define RTL_EPROGRAM_TYPE			 0xffff0008

/**
 * @macro RTL_ECHANGE_PERMISSION_DENIED_BY_MM
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief MM denied a region permission change.
 */
#define RTL_ECHANGE_PERMISSION_DENIED_BY_MM 	 0xffff0009

/**
 * @macro RTL_EBACKWARD_REFERENCE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A backward reference in the file.
 * @long The load module is not structured for stream loading.
 *       lm_post_link tool can be used to restructure an ELF
 *       load module for stream loading.
 */
#define RTL_EBACKWARD_REFERENCE 		 0xffff000a

/**
 * @macro RTL_ENO_SU_READ_EXECUTE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief Illegal configuration, a TEXT region must allow read and execute
 *        in supervisor execution mode.
 */
#define RTL_ENO_SU_READ_EXECUTE			 0xffff000b

/**
 * @macro RTL_EFLUSH_OR_CHECKSUM
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The instruction cache could not be flushed, or the checksum could
 *        not be calculated.
 */
#define RTL_EFLUSH_OR_CHECKSUM                   0xffff000c

/**
 * @macro RTL_ECORRUPT_REQUEST_SIGNAL
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The install request signal was corrupt.
 * @long Either a string was NULL, longer than allowed
 *       or the option field had unknown options specified.
 */
#define RTL_ECORRUPT_REQUEST_SIGNAL              0xffff000d

/**
 * @macro RTL_EMISSING_CONFIGURATION
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The configuration of a load module
 *        is missing a mandatory configuration.
 *
 * @long  The RAMLOG specify further what
 *        configuration is missing.
 */
#define RTL_EMISSING_CONFIGURATION		 0xffff000e

/**
 * @macro RTL_ETEXT_NOT_WITHIN_LOG_MEM
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The text for an absolutely linked load module is not within
 *        the log_mem for its program type.
 */
#define RTL_ETEXT_NOT_WITHIN_LOG_MEM             0xffff000f

/*
 * ELF RTL specific statuses.
 */

/**
 * @macro RTL_ELF_ELOAD_BEFORE_PT_DYNAMIC
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The .dynamic section was located in the wrong place.
 *        The load module structure is not correct.
 */
#define RTL_ELF_ELOAD_BEFORE_PT_DYNAMIC 	 0xffff0100

/**
 * @macro RTL_ELF_EELF_HEADER_CORRUPT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The ELF header of the load module is corrupt.
 *
 * @long There can be many reasons. One example is mismatch
 *       in architecture or endian, when loading the file.
 */
#define RTL_ELF_EELF_HEADER_CORRUPT 		 0xffff0101

/**
 * @macro RTL_ELF_ESEGMENT_READ_FAIL
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A backward reference in the load module file. The segments
 *        are not sorted for stream loading.
 */
#define RTL_ELF_ESEGMENT_READ_FAIL    		 0xffff0102

/**
 * @macro RTL_ELF_ESECTION_READ_FAIL
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief A backward reference in the load module file. The sections
 *        are not sorted for stream loading.
 */
#define RTL_ELF_ESECTION_READ_FAIL    		 0xffff0103

/**
 * @macro RTL_ELF_ENO_PROGRAM_HEADERS
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The load module does not contain any program
 *        headers, hence it can not be loaded.
 */
#define RTL_ELF_ENO_PROGRAM_HEADERS              0xffff0104

/**
 * @macro RTL_ELF_ENO_PT_LOAD
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The load module does not contain any load segments.
 */
#define RTL_ELF_ENO_PT_LOAD			 0xffff0105


/**
 * @macro RTL_ELF_EELF_NOT_EXECUTABLE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The elf file lacks execute perimitive(ET_EXEC).
 */
#define RTL_ELF_EELF_NOT_EXECUTABLE              0xffff0106

/**
 * @macro RTL_ELF_ENO_LM_CONF_SEGMENT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The load module lacks the configuration segment
 *        or the configuration has syntax error.
 */
#define RTL_ELF_ENO_LM_CONF_SEGMENT              0xffff0107

/**
 * @macro RTL_ELF_ESEGMENT_BEFORE_HEADER
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The elf segment(s) are places before the program
 *        header(s) in the file. This will lead to backward
 *        references in the file, e.g. not optimal for stream
 *        loading.
 */
#define RTL_ELF_ESEGMENT_BEFORE_HEADER           0xffff0108

/**
 * @macro RTL_ELF_ENO_RELOCATION_INFO
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The load module lacks the dynamic relocation information
 *        segment (non absolute install).
 */
#define RTL_ELF_ENO_RELOCATION_INFO              0xffff0109

/**
 * @macro RTL_ELF_ENO_TEXT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The load module does not contain any text (code or ro data).
 */
#define RTL_ELF_ENO_TEXT                         0xffff010a

/**
 * @macro RTL_ELF_ENO_DATA
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The load module does not contain data (uninitiated data).
 */
#define RTL_ELF_ENO_DATA                         0xffff010b

/**
 * @macro RTL_ELF_EHOLE_BETWEEN_DATA
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The elf has a hole between data segments, and it is being
 *        installed absolute. This leads to unnecessary memory usage.
 */
#define RTL_ELF_EHOLE_BETWEEN_DATA               0xffff010c

/**
 * @macro RTL_ELF_ENO_BIOSENTRY_SECTION
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The load module lacks the BIOS entry section called
 *        ".ose_sfk_biosentry". This error can only occur
 *        in the softOSE environment.
 */
#define RTL_ELF_ENO_BIOSENTRY_SECTION            0xffff010d

/**
 * @macro RTL_ELF_EOPERATION_ABORTED
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The current operation was aborted by a user defined RTL event hook.
 *
 * @seealso RTL_EVENT_HOOK
 */
#define RTL_ELF_EOPERATION_ABORTED               0xffff010e

/*
 *===========================================================================
 *                        Load Module configuration.
 *===========================================================================
 *
 * Description
 * -----------
 * The load module configuration consists of 2 types of configurations. The
 * program configuration, and the PM directives. All PM directives have the
 * prefix of "OSE_LM_". This prefix is reserved by PM and may not be used in
 * program configurations. Some of the PM directives are mandatory, i.e. they
 * must always be defined when installing a load module or creating a program.
 */

/**
 * @macro OSE_LM_PROGRAM_TYPE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The program type defines the memory configuration of
 *        the program created from the load module.
 *
 * @long A program type name defined by the system, the name string shall
 *       contain a maximum of 32 chars including string terminator.
 *
 * @long Mandatory configuration but it is set by RTL_ELF for each
 *          load module if a default program type is set.
 */
#define OSE_LM_PROGRAM_TYPE    "OSE_LM_PROGRAM_TYPE"

/**
 * @macro OSE_LM_POOL_SIZE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The size of a program local pool, or the size of the initial
 *        fragment of an extendible program local pool.
 *
 * @long >0    Any acceptable pool size (acceptable pool sizes are defined by
 *             the OSE kernel).
 * @long 0     means, use the domain's general pool (program creation will
 *             fail if the configuration is set to "0" and the program is
 *             created in a new domain).
 *
 * @long Default  0x10000 bytes.
 */
#define OSE_LM_POOL_SIZE      "OSE_LM_POOL_SIZE"

/**
 * @macro OSE_LM_STACK_SIZES
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The sizes of the program's stack buffers.
 *
 * @long A comma separated list of 8 integers, sorted in ascending order.
 *
 * @long Default   256,512,1024,2048,4096,8192,16384,65536
 */
#define OSE_LM_STACK_SIZES    "OSE_LM_STACK_SIZES"

/**
 * @macro OSE_LM_SIGNAL_SIZES
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The sizes of the program's signal buffers.
 *
 * @long A comma separated list of 8 integers, sorted in ascending order.
 *
 * @long Default   31,63,127,255,1023,4095,16383,65535
 */
#define OSE_LM_SIGNAL_SIZES    "OSE_LM_SIGNAL_SIZES"

/**
 * @macro OSE_LM_DATA_INIT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The configuration forces PM to initialize program static data
 *        before starting a program created from the load module.
 *
 * @long YES means force data init,
 * @long NO means do not force data init.
 *
 * @long Default NO
 */
#define OSE_LM_DATA_INIT       "OSE_LM_DATA_INIT"

/**
 * @macro OSE_LM_BSS_INIT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The configuration forces PM to zero-initialize program BSS
 *        before starting a program created from the load module.
 *
 * @long YES means force BSS init,
 * @long NO means do not force BSS init.
 *
 * @long Default NO
 */
#define OSE_LM_BSS_INIT         "OSE_LM_BSS_INIT"

/**
 * @macro OSE_LM_EXEC_CONTEXT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief This configuration informs PM whether the load module can generate
 *        a program of not.
 * @long  FEATURE NOT IMPLEMENTED.
 *
 * @long YES            means the load module can create a program
 *                       with an execution context
 * @long NO     means the load module cannot create a
 *                       program with an execution context i.e. can
 *                       for instance be used for shared libraries
 *                       that has no execution context of its own.
 *
 * @long Default YES
 */
#define OSE_LM_EXEC_CONTEXT     "OSE_LM_EXEC_CONTEXT"


/**
 * @macro OSE_LM_REENTRANT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief This configuration informs PM whether the load module can generate
 *        a reentrant shared library.
 * @long  FEATURE NOT IMPLEMENTED.
 *
 * @long YES  means the load module can generate a reentrant shared
 *             library.
 * @long NO   means the load module can not generate a reentrant
 *             shared library, i.e. the shared library can not be
 *             created multiple times in the same domain.
 *
 * @long Default YES
 *
 */
#define OSE_LM_REENTRANT        "OSE_LM_REENTRANT"

/**
 * @macro OSE_LM_EXEC_MODEL
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief Private or Shared execution model.
 *
 * @long  This configuration informs PM to configure the program to either
 *        run in private mode (classic OSE) or shared mode (POSIX).
 *
 * @long PRIVATE Sets the execution model for the program to the
 *                private mode, which is the legacy OSE mode. Resources like
 *                heap buffers and file descriptors are private to processes,
 *                and the "exit" call will terminate only the process.
 *                In this mode it is also allowed to create children
 *                OSE blocks.
 *
 * @long SHARED  Sets the execution model for the program to the
 *                shared mode, which is the resource model most familiar
 *                to POSIX/C++ programmers. In this mode, the heap buffers and
 *                file descriptors are shared and accessible by all processes
 *                within the program. Further more, it is also illegal to
 *                create children OSE blocks in this mode.
 *                A call to "exit" will terminate the entire program.
 *
 * @long Default PRIVATE
 *
 */
#define OSE_LM_EXEC_MODEL        "OSE_LM_EXEC_MODEL"

/**
 * @macro OSE_LM_INTERCEPT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief Intercept the program when it is created.
 *
 * @long  This configuration informs PM to intercept the program when it is
 *        created.
 *        The configuration is useful when a programs initialization is
 *        run-mode debugged.
 *
 * @long YES     Intercept the program when it is created.
 *
 * @long NO      Do not intercept the program when it is created.
 *
 * @long Default NO
 */
#define OSE_LM_INTERCEPT        "OSE_LM_INTERCEPT"

/**
 * @macro OSE_LM_MAIN_NAME
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The name of the main process of the program.
 *
 * @long Any ASCII string (except "").
 *
 * @long Default "main"
 */
#define OSE_LM_MAIN_NAME   "OSE_LM_MAIN_NAME"

/**
 * @macro OSE_LM_MAIN_PRIORITY
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The priority of the main process of the program.
 *
 * @long 0-31 Any value from 0 to 31, as a prioritized process
 * @long 32, as a background process
 *
 * @long Default 31
 */
#define OSE_LM_MAIN_PRIORITY    "OSE_LM_MAIN_PRIORITY"

/**
 * @macro OSE_LM_MAIN_STACK_SIZE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The stack size of the main process of the program.
 *
 * @long Any value smaller than the largest configured stack size.
 *
 * @long Default 511
 */
#define OSE_LM_MAIN_STACK_SIZE  "OSE_LM_MAIN_STACK_SIZE"

/**
 * @macro OSE_LM_USER_ID
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The user ID of the program to be created from the load module.
 * @long  FEATURE_NOT_SUPPORTED.
 *
 * @long 0 is the only allowed value.
 *
 * @long Default The user ID of the process that communicates with
 *          the program manager.
 */
#define OSE_LM_USER_ID        "OSE_LM_USER_ID"

/**
 * @macro OSE_LM_CPU_ID
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The CPU ID (in an SMP system) where the program should be created.
 *
 * @long Default The CPU ID of the process that communicates with
 *          the program manager.
 */
#define OSE_LM_CPU_ID        "OSE_LM_CPU_ID"

/**
 * @macro OSE_LM_LOCKED_ON_CPU
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief Control if the program is movable or locked to a CPU in a SMP system.
 *
 * @long Default NO
 */
#define OSE_LM_LOCKED_ON_CPU        "OSE_LM_LOCKED_ON_CPU"

/**
 * @macro OSE_LM_PROGRAM_NAME
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The name of the program and the link handler pathname of the program
 * link handler.
 *
 * @long Any ASCII string is acceptable (except an empty ""). Also avoid
 * special characters. For name rules, see the description of
 * @link "ose.h" "create_process".
 *
 * @long Default no program link handler will be created
 * and the program name will be equal to the load module install handle.
 */
#define OSE_LM_PROGRAM_NAME   "OSE_LM_PROGRAM_NAME"

/**
 * @macro OSE_LM_POOL_EXT_SIZE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The size of the program local pool extend fragment.
 *
 * @long >0    Must be larger than the largest possible pool object
 *             + 1024 bytes. The actual size may differ slightly from
 *             specified size due to alignment.
 * @long 0     The default value will be used.
 *
 *
 * @long Default (OSE_LM_POOL_MAX_SIZE - OSE_LM_POOL_SIZE) / 5.
 */
#define OSE_LM_POOL_EXT_SIZE      "OSE_LM_POOL_EXT_SIZE"

/**
 * @macro OSE_LM_POOL_MAX_SIZE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The maximum possible size of the program local pool.
 *
 * @long >0    Must be larger than initial size, if one is explicitly
 *             defined. The actual size may differ slightly from
 *             specified size due to alignment.
 * @long 0     means, do not enable pool extension for local pools,
 *             only create a static local pool.
 *
 * @long Default 0.
 */
#define OSE_LM_POOL_MAX_SIZE      "OSE_LM_POOL_MAX_SIZE"

/**
 * @macro OSE_LM_POOL_FILL
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @long This is a load module configuration.
 *
 * @brief Control whether a local pool shall be filled with
 *        debug pattern 0xCC or not.
 *
 * @long Default NO.
 */
#define OSE_LM_POOL_FILL        "OSE_LM_POOL_FILL"

/**
 * @macro OSE_LM_TEXT_SEARCH_BASE
 *
 * @include "ose_spi/pm.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The base address where to locate the region for the text segment.
 *
 * @long Only applicable for load modules located in SASE memory. If the given
 * address is already in use, the closest free address above will be used.
 *
 * @long A value of 0 means standard behavior of allocation from the bottom
 * of the address space.
 *
 * @long Default 0
 */
#define OSE_LM_TEXT_SEARCH_BASE     "OSE_LM_TEXT_SEARCH_BASE"

/**
 * @macro OSE_LM_DATA_SEARCH_BASE
 *
 * @include "ose_spi/pm.h"
 *
 * @long This is a load module configuration.
 *
 * @brief The base address where to locate the region for the data segment.
 *
 * @long Only applicable for load modules located in SASE memory. If the given
 * address is already in use, the closest free address above will be used.
 *
 * @long A value of 0 means standard behavior of allocation from the bottom
 * of the address space.
 *
 * @long Default 0
 */
#define OSE_LM_DATA_SEARCH_BASE     "OSE_LM_DATA_SEARCH_BASE"

/*
 *===========================================================================
 *                        RTL ELF configuration.
 *===========================================================================
 */

/**
 * @macro      RTL_DEFAULT_CHUNK_SIZE
 * @brief      The default relocation memory chunk size.
 * @deprecated This symbol is deprecated in OSE5.5 as relocation
 *             chunks are no longer used in the RTL implementation.
 */
#define RTL_DEFAULT_CHUNK_SIZE      0x10000

/**
 * @macro      RTL_MINIMAL_CHUNK_SIZE
 * @brief      The minimal relocation memory chunk possible to set.
 * @deprecated This symbol is deprecated in OSE5.5 as relocation
 *             chunks are no longer used in the RTL implementation.
 */
#define RTL_MINIMAL_CHUNK_SIZE      0x100

/**
 * @macro      RTL_RELOC_CHUNK
 * @brief      The name of the RTL ELF chunk size configuration.
 * @deprecated This symbol is deprecated in OSE5.5 as relocation
 *             chunks are no longer used in the RTL implementation.
 */
#define RTL_RELOC_CHUNK             "ose_rtl_elf_chunk_size"

/**
 * @macro RTL_PROGRAM_TYPE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief Defines the environment string for default program type
 * configuration
 *
 * @long The RTL can be configured to use a default program type, if
 * no program type is defined by the load modules configuration.
 *
 * @long Set the RTL_PROGRAM_TYPE environment variable to the RTL
 * process or block with a program type string to make it the default
 * program type.  E.g. set_env(rtl, RTL_PROGRAM_TYPE, "APP_RAM");
 */
#define RTL_PROGRAM_TYPE             "ose_rtl_elf_program_type"

/**
 * @macro RTL_EVENT_HOOK
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief Defines the environment string for event hook configuration.
 *
 * @long An event hook can be installed by system software to monitor
 * internal events in the RTL. The event hook prototype is defined as:
 * int rtl_event(int event_no, ...);
 *
 * @long The event hook is installed by setting the event hook
 * entry point to the RTL_EVENT_HOOK environment variable on the RTL
 * process or process block before starting the RTL
 * process. Note that set_envp should be used.
 * E.g. set_envp(rtl, RTL_EVENT_HOOK, (OSADDRESS)(rtl_event));
 *
 * @long Each event is assigned an event number.  The event number
 * defines what extra parameters are passed to the event hook during
 * an event. The event number also defines what actions that are taken
 * on event hook return values.
 *
 * @restriction The event hook must be placed in the same build
 * linkage as the RTL.
 *
 * @break The event hook will use the same stack as the RTL.
 *
 * @example
 *
 * #include "stdarg.h"
 *
 * int rtl_event(int event_no, ...)
 * {
 *   int ret = 0;
 *   va_list ap;
 *   va_start(ap, event_no);
 *   switch(event_no)
 *   {
 *     case RTL_INIT_EVT:
 *       ret = handle_init_event();
 *       break;
 *     case RTL_READ_EVT;
 *       {
 *         char *file = va_arg(ap, char*);
 *         char *hand = va_arg(ap, char*);
 *         OSADDRESS segment = va_arg(ap, OSADDRESS);
 *         OSADDRESS offset = va_arg(ap, OSADDRESS);
 *         OSADDRESS base = va_arg(ap, OSADDRESS);
 *         OSADDRESS size = va_arg(ap, OSADDRESS);
 *         ret = handle_read_event(file, hand, segment, offset, data, size);
 *         break;
 *       }
 *     default:
 *       break;
 *   }
 *   va_end(ap);
 *   return ret;
 * }
 *
 * @seealso RTL_INIT_EVT
 * @seealso RTL_INSTALL_EVT
 * @seealso RTL_READ_EVT
 * @seealso RTL_INSTALL_END_EVT
*/
#define RTL_EVENT_HOOK              "ose_rtl_elf_event_hook"

/**
 * @macro RTL_INIT_EVT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief Defines the RTL init event.
 *
 * @long Passed as first argument to the RTL event hook at RTL process
 * initialization. The init event takes no extra parameters.
 *
 * @long The event hook will be called as: int
 * rtl_event(RTL_INIT_EVT);
 *
 * @long The RTL will not take any actions based of the event hook
 * return value for this event.
 *
 * @seealso RTL_EVENT_HOOK
 */
#define RTL_INIT_EVT                1

/**
 * @macro RTL_INSTALL_EVT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief Defines the RTL install event.
 *
 * @long Passed as first argument to the RTL event hook when a load
 * module is about to be installed.
 *
 * @long The event hook will be called as: int
 * rtl_event(RTL_INSTALL_EVT, char *file, char *install_handle);
 *
 * @long If the event hook returns 0 for this event the installation
 * proceeds. Any other returned value will cause the installation to
 * be aborted.
 *
 * @seealso RTL_EVENT_HOOK
 */
#define RTL_INSTALL_EVT             11

/**
 * @macro RTL_READ_EVT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief Defines the RTL read event.
 *
 * @long Passed as first argument to the RTL event hook when a chunk
 * of data has been read from the load module to RAM memory.
 *
 * @long The event hook will be called as: int rtl_event(RTL_READ_EVT,
 * char *file, char *install_handle, OSADDRESS segment_number,
 * OSADDRESS offset_in_segment, OSADDRESS base, OSADDRESS size);
 *
 * @long If the event hook returns 0 for this event the installation
 * proceeds. Any other returned value will cause the installation to
 * be aborted.
 *
 * @seealso RTL_EVENT_HOOK
 */
#define RTL_READ_EVT                12

/**
 * @macro RTL_INSTALL_END_EVT
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief Defines the RTL install end event.
 *
 * @long Passed as first argument to the RTL event hook when a load
 * module has been successfully installed.
 *
 * @long The event hook will be called as: int
 * rtl_event(RTL_INSTALL_END_EVT, char *file, char *install_handle);
 *
 * @long If the event hook returns 0 for this event the installation
 * proceeds. Any other returned value will cause the installation to
 * be aborted.
 *
 * @seealso RTL_EVENT_HOOK
 */
#define RTL_INSTALL_END_EVT         13

/**
 * @macro RTL_CHUNK_SIZE
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief Defines the environment string for chunk size configuration.
 *
 * @long The RTL can optionally be configured to use a max chunk size
 * when reading program segment data from the load file. Default, a
 * complete program segment is read as one single object (chunk).
 *
 * @long The @link "pm_defs.h" "RTL_READ_EVT" will be called for each
 * read chunk if an event hook is configured.
 *
 * @long Set the RTL_CHUNK_SIZE environment variable to the RTL
 * process or block to set the chunk size. Note that set_envp should
 * be used.  E.g. set_envp(rtl, RTL_CHUNK_SIZE, 0x1000);
 */
#define RTL_CHUNK_SIZE             "ose_rtl_elf_chunk_size"

/**
 * @macro RTL_ELF_FF_NAME
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The RTL ELF handles the ELF file format.
 * @long  When installing a load module
 *        using the PM, a file format must be supplied. The file format handled
 *        by the RTL ELF is defined as this string, having the value "ELF".
 */
#define RTL_ELF_FF_NAME             "ELF"

/**
 * @macro RTL_LIP_FF_NAME
 *
 * @include "ose_spi/pm_defs.h"
 *
 * @brief The RTL LIP handles previously loaded (memory mapped) load modules.
 * @long  When installing a memory mapped load module
 *        using the PM, a file format must be supplied. The file format handled
 *        by the RTL LIP is defined as this string, having the value "LIP".
 */
#define RTL_LIP_FF_NAME             "LIP"

/***********************************************************
 *
 * STRUCTURES AND GLOBALS
 *
 ***********************************************************/

/***********************************************************
 *
 * FUNCTION DEFINITIONS
 *
 ***********************************************************/


#ifdef __cplusplus
}
#endif

#endif /* #ifndef PM_DEFS_H_ */
