/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @toc OSE:
 *
 * @file ose_spi/rtc.sig
 *
 * @brief RTC signal interface.
 *
 *	      All rights reserved.
 */

#ifndef _RTC_SIG
#define _RTC_SIG

#include "ose.h"
#include "osetypes.h"
#include "rtc.h"

#ifdef __cplusplus
extern "C" {
#endif

#define TZ_ABBR_BUFF_SIZE 11

/**
 * @signal RtcConfig
 *
 * @brief Signal to configure the RTC local time by giving binary timezone data,
 *        describing the time zone and the daylight savings time
 *        transitions and/or leap seconds.
 *
 * @long  The signal should be allocated, and the parameters filled in,
 *        by the @link "ose_spi/rtc.h" "rtc_tz_compile" call, based
 *        on a text timezone file and one selected timezone name.
 *
 * @long Any previous configuration of the same kind (leap second data
 *        and/or local time zone data, including offset given by
 *        RTC_SET_LOCAL_OFFSET_REQUEST) will be overridden.
 *
 * @member RtcConfig
 * @seealso RTC_CONFIG_REQUEST
 * @seealso RTC_CONFIG_REPLY
 */
 
/**
 * @macro RTC_CONFIG_REQUEST
 *
 * @brief Signal number to configure the RTC local time by giving
 *        binary timezone data
 *
 * @seealso RtcConfig
 */
#define RTC_CONFIG_REQUEST 32720 \
  /* !-SIGNO(struct RtcConfig) */

/**
 * @macro RTC_CONFIG_REPLY
 *
 * @brief Signal number in reply to configure the RTC local time by giving
 *        binary timezone data
 *
 * @seealso RtcConfig
 */
#define RTC_CONFIG_REPLY 32721 \
  /* !-SIGNO(struct RtcConfig) */

/**
 * @signal RtcSetLocalOffset
 *
 * @brief Signal to configure RTC local time by giving a fixed offset.
 *
 * @long No automatic daylight savings time transitions will be made.
 *       Leap second handling will be switched off.
 *       Any configuration made with RTC_CONFIG_REQUEST will be
 *       overridden.
 *
 * @member RtcSetLocalOffset
 * @seealso RTC_SET_LOCAL_OFFSET_REQUEST
 * @seealso RTC_SET_LOCAL_OFFSET_REPLY
 * @seealso RtcConfig
 */
 
/**
 * @macro RTC_SET_LOCAL_OFFSET_REQUEST
 *
 * @brief Signal number to configure RTC local time by giving a fixed offset.
 *
 * @seealso RtcSetLocalOffset
 */
#define RTC_SET_LOCAL_OFFSET_REQUEST 32722 \
  /* !-SIGNO(struct RtcSetLocalOffset) */

/**
 * @macro RTC_SET_LOCAL_OFFSET_REPLY
 *
 * @brief Signal number in reply to configure RTC local time by giving
 *        a fixed offset.
 *
 * @seealso RtcSetLocalOffset
 */
#define RTC_SET_LOCAL_OFFSET_REPLY 32723 \
  /* !-SIGNO(struct RtcSetLocalOffset) */

/**
 * @type RtcSetLocalOffset
 *
 * @brief Type used in RTC_SET_LOCAL_OFFSET_REQUEST
 *
 * @field sigNo       Signal number.
 * @field gmtOffset   Number of seconds to be added to GMT when calculating
 *                    local time
 * @field abbr        Time zone abbreviation
 */
struct RtcSetLocalOffset
{
  SIGSELECT  sigNo;
  S32        gmtOffset;
  char       abbr[TZ_ABBR_BUFF_SIZE];
};

/**
 * @type RtcConfig
 *
 * @brief Type used in RTC_CONFIG_REQUEST
 *
 * @field sigNo       Signal number.
 * @field result      Status of the operation.
 * @field size        Size of tzData.
 * @field tzData      Variable sized binary data, output from the
 *                    rtc_tz_compile function
 */
struct RtcConfig
{
  SIGSELECT  sigNo;
  RtcStatus  result;
  U32        size;
  U8         tzData[1];
};

#ifdef __cplusplus
}
#endif

#endif /* _RTC_SIG */
