/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 ****************************************************************************
 *			  PM SEMI PUBLIC FUNCTION INTERFACE
 *
 *     $Workfile:   rtl.sig  $
 *
 *
 ****************************************************************************
 * Description: The RTL semi public signal interface.
 * 
 */

#ifndef __RTL_SIG
#define __RTL_SIG

/***********************************************************
 *
 * INCLUDE FILES
 *
 ***********************************************************/

#include "osetypes.h"
#include "pm.sig"
#include "pm.h"
#include "stddef.h"

#ifdef __cplusplus
extern "C" {
#endif

/***********************************************************
 *
 * CONSTANTS AND DEFINITIONS
 *
 ***********************************************************/

/* The RTL signal base. (To be changed later.)*/
#define RTL_ELF_SIGNAL_BASE   (36280)

/*
 * Connection options. 
 */

/***********************************************************
 *
 * STRUCTURES AND GLOBALS
 *
 ***********************************************************/

/***********************************************************
 *
 * LOAD MODULE META DATA STRUCTURES
 *
 ***********************************************************/

/* The Text and Data region names. */
#define LM_TEXT "LM_TEXT"
#define LM_DATA "LM_DATA"

/* The PmRoMetaData magic number. */
#define PM_RO_META_DATA_MAGIC_0         ('K')
#define PM_RO_META_DATA_MAGIC_1         ('a')
#define PM_RO_META_DATA_MAGIC_2         ('S')
#define PM_RO_META_DATA_MAGIC_3         ('i')

/* The minimal default region alignment. */
#ifdef OSPP
#define PM_REGION_ALIGN              0x10000
#else
#define PM_REGION_ALIGN              4
#endif
   
/* The RTL options. */
#define RTL_OPT_NORMAL               0
#define RTL_OPT_UNINSTALL            1

/* The memory region options. */
#define PM_REGION_NORMAL             0
#define PM_REGION_ALIEN              1

/* The name of the environment variable that holds the physical memory type
 * used for the LM's text region. Set by RTL at program installation.
 * */
#define RTL_PHYS_MEM_TEXT_VAR          "RTL_PHYS_MEM_TEXT"
/*
 *===========================================================================
 *                        Section_Info
 *===========================================================================
 * 
 * Description
 * -----------
 * Definition of the PM installed load module section information.
 *
 * Parameter description
 * ---------------------
 * base          The base address of the section. 
 * size          The size of the section (bytes). 
 * attr          The attributes of the section. 
 * options       The load options of the section.
 * name          The section name.
 *
 */

typedef struct
{
  U32            base;
  size_t         size;
  U32            attr;
  U32            options;
  char           name[PM_SECTION_NAME_LENGTH];
} Section_Info;

/*
 *===========================================================================
 *                        SectionInfo
 *===========================================================================
 * 
 * Description
 * -----------
 * Definition of the section information overall structure.
 *
 * Parameter description
 * ---------------------
 * sections_size  The number of sections in the load module.
 * sections       The array of section information.
 * 
 */

typedef struct
{
  U32             sections_count;
  Section_Info *  sections;

} SectionInfo;

/*
 *===========================================================================
 *                        MemRegion
 *===========================================================================
 * 
 * Description
 * -----------
 * Definition of a memory region.
 *
 * Parameter description
 * ---------------------
 * base          The (logical) base address of the region.
 * size          The size of the region (bytes).
 * options       The region options, PM_REGION_NORMAL or PM_REGION_ALIEN.
 */

typedef struct
{
  OSADDRESS base;
  U32       size;
  U32       options;
} MemRegion;


/*
 *===========================================================================
 *                        LmRoMetaData
 *===========================================================================
 * 
 * Description
 * -----------
 * Definition of a load module read only meta data.
 *
 * Parameter description
 * ---------------------
 * magic            The structures magic number (PM_RO_META_DATA_MAGIC_x).
 * options          The options field contents of the install request signal.
 * entrypoint       The load module entrypoint (logical).
 * configuration    The pointer to the configuration 
 *                  information of the load module.
 * section_info     The section information pointer.
 * install_handle   The installation handle of the load module.
 * parent_file_name The parent load module name.
 * emfiles          The pointer to the embedded files TOC, if the TOC do not 
 *                  exist, the pointer is set to ZERO.
 * text             The text region information.
 * text_conf        The text region configuration.
 * data             The data region information.
 * data_conf        The data region configuration.
 * uninitiated_data The "bss" data region information.  
 * initiated_data   The initiated data region information.  
 * mmap_fd          The open file descriptor if module is memory mapped.
 * others           The other regions (not text or data) 
 *                  information.
 *
 */

typedef struct
{
  char                        magic[4];
  U32                          options;
  OSADDRESS	            entrypoint;
  const char           * configuration;
  SectionInfo             section_info;
  char                  install_handle[PM_INSTALL_HANDLE_LENGTH];
  char                parent_file_name[PM_PARENT_FILE_NAME_SIZE];	
  char	                   file_format[PM_FILE_FORMAT_NAME_LENGTH];
  OSADDRESS                    emfiles;
  MemRegion                       text; 
  struct PmProgramRegionType text_conf; 
  MemRegion                       data; 
  struct PmProgramRegionType data_conf; 
  MemRegion           uninitiated_data; 
  MemRegion             initiated_data; 
  int                          mmap_fd;
  MemRegion                  others[1];
} LmRoMetaData;



/*
 *===========================================================================
 *                        LmRoMetaDataPers
 *===========================================================================
 * 
 * Description
 * -----------
 * Definition of the LmRoMetaData persistency extension.
 *
 * Parameter description
 * ---------------------
 * meta          The LmRoMetaData structure of the load module.
 * checksum      The checksum of the load module text region.
 * magic         The structures magic number (PM_RO_META_DATA_MAGIC_x).
 *               Placed before the checksum to be guaranteed to be
 *               last in the Meta Data structures. The Magic value is used
 *               when determining if a text region is complete or not during
 *               resurrection.
 *
 */

typedef struct
{
  LmRoMetaData          * meta;
  U32                 checksum;
  char                magic[4];
} LmRoMetaDataPers;

/***********************************************************
 *
 * SIGNAL DEFINITIONS
 *
 ***********************************************************/

/*       The interface reply shall only contain signal requests
         that RTL responds to.
	 Therefore the RTL_ELF_NO_PUBLIC_SIGNALS in the interface reply
	 is not the sum of signals in the rtl.sig.
*/
   
/*
 *===========================================================================
 *                        RTL_INTERFACE_REQUEST, 
 *                        RTL_INTERFACE_REPLY
 *===========================================================================
 *
 * Description:   Request/reply for the rtl.sig interface.
 *
 * Structure:     RtlInterfaceRequest, RtlInterfaceReply
 *
 * Parameters:     I/O: sigNo,     the id of a signal.
 *                 O:   status,    the status of the 
 *                                 operation (always 
 *                                 PM_SUCCESS).
 *                 O:   sigs,      The signal numbers 
 *                                 of the rtl.sig interface 
 *                                 that is supported. 
 *                                 (variable size, element ZERO holds the
 *                                 number of signals in the array).
 */ 

#define RTL_INTERFACE_REQUEST               (36280) \
  /* !-SIGNO(struct RtlInterfaceRequest)-! */

struct RtlInterfaceRequest
{
  SIGSELECT sigNo;
};

#define RTL_INTERFACE_REPLY                 (36281) \
  /* !-SIGNO(struct RtlInterfaceReply)-! */

struct RtlInterfaceReply
{
  SIGSELECT sigNo;
  PmStatus status;
  SIGSELECT sigs[1];    /* The signals implemented in the RTL signal 
			 * interface. */
};

/*
 *===========================================================================
 *                        RTL_CONNECT_REQUEST, 
 *                        RTL_CONNECT_REPLY
 *===========================================================================
 *
 * Description:   Request for connection of a RTL to PM.
 *
 * Structure:     RtlConnectRequest, RtlConnectReply
 *
 * Parameters:    I/O: sigNo,     the id of a signal.
 *                I:   ff_name,   the name of the file 
 *                                format supported.
 *                O:   status,    the status of the 
 *                                operation.
 */ 

#define RTL_CONNECT_REQUEST               (36282) \
  /* !-SIGNO(struct RtlConnectRequest)-! */

struct RtlConnectRequest
{
  SIGSELECT   sigNo;
  char   ff_name[1];
};

#define RTL_CONNECT_REPLY                 (36283) \
  /* !-SIGNO(struct RtlConnectReply)-! */

struct RtlConnectReply
{
  SIGSELECT sigNo;
  PmStatus status;
};

/*
 *===========================================================================
 *                        RTL_COMMIT_LOAD_MODULE_DATA
 *===========================================================================
 *
 * Description:   Commit load module data to the PM.
 *                NOTE: This signal is sent with client sender process as 
 * 		  sender.
 *
 * Structure:     RtlCommitLoadModuleData
 *
 * Parameters:    O:   sigNo,     The id of a signal.
 *		  O:   status,    The reply status of the installation 
 *				  operation.
 *                O:   install_handle,
 *                                The requested install_handle.
 *  		  O:   load_module_ro_data, 
 *                                A pointer to the installed load 
 *				  modules Read Only Meta data.
 */

#define RTL_COMMIT_LOAD_MODULE_DATA        (36284) \
  /* !-SIGNO(struct RtlCommitLoadModuleData)-! */

struct RtlCommitLoadModuleData
{
  SIGSELECT	sigNo;
  PmStatus	status;
  char          install_handle[PM_INSTALL_HANDLE_LENGTH];
  OSADDRESS	load_module_meta_data; 
};

/*
 *===========================================================================
 *                        RTL_INTERNAL_SPAWN_KILLED
 *===========================================================================
 *
 * Description:   Attach signal to a RTL spawn process.
 *
 * Structure:     RtlInternalSpawnKilled
 *
 * Parameters:         sigNo,     The id of a signal.
 *		       client,    The PM client pid. 
 */ 

#define RTL_INTERNAL_SPAWN_KILLED        (36285) \
  /* !-SIGNO(struct RtlInternalSpawnKilled)-! */

struct RtlInternalSpawnKilled
{
  SIGSELECT	sigNo;
  PROCESS       client;
};

/*
 *===========================================================================
 *                        RTL_FIND_MM_REPLY
 *===========================================================================
 *
 * Description:   A hunt MM signal.
 *
 * Structure:     SIGSELECT
 *
 * Parameters:    sigNo     The signal number.
 *
 */ 

/*Internal RTL signal to find MM.*/
#define RTL_FIND_MM_REPLY                 (36286) \
  /* !-SIGNO(SIGSELECT)-! */

/*
 *===========================================================================
 *                        RTL_FIND_PM_REPLY
 *===========================================================================
 *
 * Description:   A hunt PM signal.
 *
 * Structure:     SIGSELECT
 *
 * Parameters:    sigNo     The signal number.
 *
 */ 

/*Internal RTL signal to find PM.*/
#define RTL_FIND_PM_REPLY                 (36287) \
  /* !-SIGNO(SIGSELECT)-! */

/*
 *===========================================================================
 *                        RTL_CONFIGURE_CHILD
 *===========================================================================
 *
 * Description:   Configure the RTL spawn that processes the installs request.
 *
 * Structure:     RtlChildDone.
 *
 * Parameters:    sigNo     The signal number.
 *                att_ref   The attach reference.
 *
 */ 

/*Internal RTL signal configure the an RTL spawn.*/
#define RTL_CONFIGURE_CHILD               (36288) \
  /* !-SIGNO(struct RtlChildDone)-! */

/*
 *===========================================================================
 *                        RTL_CHILD_DONE
 *===========================================================================
 *
 * Description:   The receipt from the spawn that the install request
 *                is handled.
 *
 * Structure:     RtlChildDone.
 *
 * Parameters:    sigNo     The signal number.
 *                att_ref   The attach reference.
 *
 */ 

/*Internal RTL signal configure the an RTL spawn.*/
#define RTL_CHILD_DONE                     (36289) \
  /* !-SIGNO(struct RtlChildDone)-! */

struct RtlChildDone
{
      SIGSELECT sigNo;
      OSATTREF  att_ref;
};

/*
 *===========================================================================
 *                        RTL_FLUSH_AND_CHECKSUM_REQUEST,
 *                        RTL_FLUSH_AND_CHECKSUM_REPLY
 *===========================================================================
 *
 * Description:   The RTL request the PM to flush the specified memory region 
 *                calculate the checksum.
 *
 * Structure:     RtlFlushAndChecksum.
 *
 * Parameters:    sigNo     The signal number.
 *                domain    The domain the region is physically located in.   
 *                base      The base of the region.
 *                size      The size of the region.
 *                checksum  The calculated checksum.
 *                status    The status of the operation.
 *
 */ 

#define RTL_FLUSH_AND_CHECKSUM_REQUEST     (36290) \
/* !-SIGNO(struct RtlFlushAndChecksum)-! */

#define RTL_FLUSH_AND_CHECKSUM_REPLY       (36291) \
/* !-SIGNO(struct RtlFlushAndChecksum)-! */

struct RtlFlushAndChecksum
{
  SIGSELECT sigNo;
  PmStatus  status;
  OSDOMAIN  domain;
  OSADDRESS base;
  U32       size;
  U32       checksum;
};

/*
 *===========================================================================
 *                        RTL_UNINSTALL_LOAD_MODULE_REQUEST,
 *                        RTL_UNINSTALL_LOAD_MODULE_REPLY
 *===========================================================================
 *
 * Description:   The request used to initiate a free all load module 
 *                resources for a specific load module.
 *
 * Structure:     RtlUninstallLoadModule.
 *
 * Parameters:    sigNo     The signal number.
 *                status    The status of the operation.
 *                meta      The load modules meta data pointer.
 *
 */ 

#define RTL_UNINSTALL_LOAD_MODULE_REQUEST     (36292) \
/* !-SIGNO(struct RtlUninstallLoadModule)-! */

#define RTL_UNINSTALL_LOAD_MODULE_REPLY       (36293) \
/* !-SIGNO(struct RtlUninstallLoadModule)-! */

struct RtlUninstallLoadModule
{
  SIGSELECT sigNo;
  PmStatus  status;
  LmRoMetaData * meta;
};

#ifdef __cplusplus
}
#endif

#endif /* #ifndef __RTL_SIG */ 

