/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @public
 * @toc OSE:
 * @file osetypes.h
 *
 * @brief OSE basic type definitions.
 */

#ifndef _OSETYPES_H
#define _OSETYPES_H

#ifdef __cplusplus
extern "C" {
#endif



/*
 * OSE type macros, mainly for internal use.
 */
#define MOV64(dest, src) (dest).lsw = (src).lsw, (dest).msw = (src).msw
#define EQU64(x, y)  ((x).lsw == (y).lsw && (x).msw == (y).msw)
#define NEQ64(x, y)  ((x).lsw != (y).lsw || (x).msw != (y).msw)
#define MAKEU64(dest, mswMA, lswMA) (dest).msw = (mswMA), (dest).lsw = (lswMA)
#define GETU32MSW(l) ((l).msw)
#define GETU32LSW(l) ((l).lsw)


/**
 * @macro OSETYPES_H_ATTRIBUTE_U32
 * @brief  Optional user-defined macro enabling customizable
 *         compiler attributes for the OSE typedef U32.
 *
 * @example
 * For compiler gcc;
 *  #ifdef __GNUC__
 *  #define OSETYPES_H_ATTRIBUTE_U32 __attribute__((__may_alias__))
 *  #endif
 */
#ifndef OSETYPES_H_ATTRIBUTE_U32
#define OSETYPES_H_ATTRIBUTE_U32
#endif
/**
 * @macro OSETYPES_H_ATTRIBUTE_S32
 * @brief  Optional user-defined macro enabling customizable
 *         compiler attributes for the OSE typedef S32.
 *
 * @example
 * For compiler gcc;
 *  #ifdef __GNUC__
 *  #define OSETYPES_H_ATTRIBUTE_S32 __attribute__((__may_alias__))
 *  #endif
 */      
#ifndef OSETYPES_H_ATTRIBUTE_S32
#define OSETYPES_H_ATTRIBUTE_S32   
#endif
/**
 * @macro OSETYPES_H_ATTRIBUTE_U16
 * @brief  Optional user-defined macro enabling customizable
 *         compiler attributes for the OSE typedef U16.
 *
 * @example
 * For compiler gcc;
 *  #ifdef __GNUC__
 *  #define OSETYPES_H_ATTRIBUTE_U16 __attribute__((__may_alias__))
 *  #endif
 */        
#ifndef OSETYPES_H_ATTRIBUTE_U16
#define OSETYPES_H_ATTRIBUTE_U16   
#endif
/**
 * @macro OSETYPES_H_ATTRIBUTE_S16
 * @brief  Optional user-defined macro enabling customizable
 *         compiler attributes for the OSE typedef S16.
 *
 * @example
 * For compiler gcc;
 *  #ifdef __GNUC__
 *  #define OSETYPES_H_ATTRIBUTE_S16 __attribute__((__may_alias__))
 *  #endif
 */        
#ifndef OSETYPES_H_ATTRIBUTE_S16
#define OSETYPES_H_ATTRIBUTE_S16   
#endif
/**
 * @macro OSETYPES_H_ATTRIBUTE_U8
 * @brief  Optional user-defined macro enabling customizable
 *         compiler attributes for the OSE typedef U8.
 *
 * @example
 * For compiler gcc;
 *  #ifdef __GNUC__
 *  #define OSETYPES_H_ATTRIBUTE_U8 __attribute__((__may_alias__))
 *  #endif
 */        
#ifndef OSETYPES_H_ATTRIBUTE_U8
#define OSETYPES_H_ATTRIBUTE_U8
#endif
/**
 * @macro OSETYPES_H_ATTRIBUTE_S8
 * @brief  Optional user-defined macro enabling customizable
 *         compiler attributes for the OSE typedef S8.
 *
 * @example
 * For compiler gcc;
 *  #ifdef __GNUC__
 *  #define OSETYPES_H_ATTRIBUTE_S8 __attribute__((__may_alias__))
 *  #endif
 */        
#ifndef OSETYPES_H_ATTRIBUTE_S8
#define OSETYPES_H_ATTRIBUTE_S8   
#endif
/**
 * @macro OSETYPES_H_ATTRIBUTE_U64
 * @brief  Optional user-defined macro enabling customizable
 *         compiler attributes for the OSE typedef U64.
 *
 * @example
 * For compiler gcc;
 *  #ifdef __GNUC__
 *  #define OSETYPES_H_ATTRIBUTE_U64 __attribute__((__may_alias__))
 *  #endif
 */        
#ifndef OSETYPES_H_ATTRIBUTE_U64
#define OSETYPES_H_ATTRIBUTE_U64
#endif
/**
 * @macro OSETYPES_H_ATTRIBUTE_BOOL
 * @brief  Optional user-defined macro enabling customizable
 *         compiler attributes for the OSE typedef BOOL.
 *
 * @example
 * For compiler gcc;
 *  #ifdef __GNUC__
 *  #define OSETYPES_H_ATTRIBUTE_BOOL __attribute__((__may_alias__))
 *  #endif
 */        
#ifndef OSETYPES_H_ATTRIBUTE_BOOL
#define OSETYPES_H_ATTRIBUTE_BOOL
#endif   
 


/**
 * @type U32
 *
 * @brief OSE unsigned 32-bit type.
 *
 */
typedef unsigned long int OSETYPES_H_ATTRIBUTE_U32 U32;

/**
 * @type U16
 *
 * @brief OSE unsigned 16-bit type.
 *
 */
typedef unsigned short int OSETYPES_H_ATTRIBUTE_U16 U16;

/**
 * @type U8
 *
 * @brief OSE unsigned 8-bit type.
 *
 */
typedef unsigned char OSETYPES_H_ATTRIBUTE_U8 U8;

/**
 * @type S32
 *
 * @brief OSE signed 32-bit type.
 *
 */
typedef long int OSETYPES_H_ATTRIBUTE_S32 S32;

/**
 * @type S16
 *
 * @brief OSE signed 16-bit type.
 *
 */
typedef short int OSETYPES_H_ATTRIBUTE_S16 S16;

/**
 * @type S8
 *
 * @brief OSE signed 8-bit type.
 *
 */
typedef signed char OSETYPES_H_ATTRIBUTE_S8 S8;

/**
 * @type U64
 *
 * @brief OSE unsigned 64-bit type.
 *
 * @long Not compatible with a little endian 64 bit (long long) type, which
 *       is only a problem if mixing U64 and long long.
 *
 * @field msw Most significant 32-bit word.
 *
 * @field lsw Least significant 32-bit word.
 *
 */
typedef struct { U32 msw, lsw; } OSETYPES_H_ATTRIBUTE_U64 U64;

/**
 * @type Boolean
 *
 * @brief OSE boolean type.
 *
 * @long Do not use enums/Boolean in signal definitions, they might not be
 *       compatible between different compilers/CPUs. Use an explicit U32
 *       instead.
 *
 * @field False (0)
 *
 * @field True (1)
 *
 */
typedef enum { False = 0, True = 1 } OSETYPES_H_ATTRIBUTE_BOOL Boolean;

#ifdef __cplusplus
}
#endif

#endif
