/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2011 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @public
 * @toc Standard_C:
 * @file stdio.h
 *
 * @brief 'ISO/IEC 9899, Programming Languages - C' interface stdio.h
 */

#ifndef _STDIO_H
#define _STDIO_H
#define OSE_STDIO_H

#include <stdarg.h>
#include <stddef.h>
#include "osarch.h"

#ifdef __cplusplus
#if !defined (NO_NAME_STD) && defined (__ARMCC_VERSION)
namespace std {
#endif /* NO_NAME_STD */
extern "C" {
#endif /* __cplusplus */

#ifdef softOSE
#define ZZMK_NAME_MANGLING
#include "sfk/sfk_stdio.h"
#endif

/**
 * @macro BUFSIZ
 *
 * @include        "stdio.h"
 *
 * @declaration #define BUFSIZ           4096
 *
 * @brief Default buffer size in bytes used by functions @link "" "setbuf" and
 *        @link "" "setvbuf".
 */
#define BUFSIZ           4096

/**
 * @macro EOF
 *
 * @include        "stdio.h"
 *
 * @declaration #define EOF              (-1)
 *
 * @brief The end of file symbol.
 */
#define EOF              (-1)

/**
 * @macro FILENAME_MAX
 *
 * @include        "stdio.h"
 *
 * @declaration #define FILENAME_MAX     255
 *
 * @brief FILENAME_MAX is the recommended size of a character array intended
 *        to hold a file name string.
 *
 * @long  The actual maximum file name length is dependent on the volume
 *        manager.
 */
#define FILENAME_MAX     255

/**
 * @macro FOPEN_MAX
 *
 * @include        "stdio.h"
 *
 * @declaration #define FOPEN_MAX        128
 *
 * @brief Minimum number of files that the application can expect to be open
 *        simultaneously.
 *
 * @long  It is not recommended to use this value, since its value is volume
 *        manager dependent.
 */
#define FOPEN_MAX        128

/*
 ****************************************************************************
 * Buffer types used as third argument of setvbuf().
 ****************************************************************************
 */

/**
 * @macro   _IOFBF
 *
 * @include        "stdio.h"
 *
 * @declaration #define _IOFBF           0
 *
 * @brief   Buffer type indicating full buffering for the @link "" "setvbuf"
 *          function.
 *
 * @seealso _IOLBF
 * @seealso _IONBF
 */
#define _IOFBF           0

/**
 * @macro   _IOLBF
 *
 * @include        "stdio.h"
 *
 * @declaration #define _IOLBF           1
 *
 * @brief   Buffer type indicating line buffering for the @link "" "setvbuf"
 *          function.
 *
 * @seealso _IOFBF
 * @seealso _IONBF
 */
#define _IOLBF           1

/**
 * @macro   _IONBF
 *
 * @include        "stdio.h"
 *
 * @declaration #define _IONBF           2
 *
 * @brief   Buffer type indicating no buffering for the @link "" "setvbuf"
 *          function.
 *
 * @seealso _IOFBF
 * @seealso _IOLBF
 */
#define _IONBF           2

/*
 ****************************************************************************
 * Constants to be used as third argument in function fseek().
 ****************************************************************************
 */

/*
 * NO SOURCEDOC TAGS BECAUSE ALREADY DEFINED IN unistd.h
 *
 * macro   SEEK_CUR
 * brief   Seek relative to current position.
 * long    Redefinition of SEEK_CUR, a constant to be used as argument whence
 *         in the fseek function.
 * seealso SEEK_END
 * seealso SEEK_SET
 */
#undef SEEK_CUR
#define SEEK_CUR         0

/*
 * NO SOURCEDOC TAGS BECAUSE ALREADY DEFINED IN unistd.h
 *
 * macro   SEEK_END
 * brief   Seek relative to the end of file.
 * long    Redefinition of SEEK_END, a constant to be used as argument whence
 *         in the fseek function.
 * seealso SEEK_CUR
 * seealso SEEK_SET
 */
#undef SEEK_END
#define SEEK_END         1

/*
 * NO SOURCEDOC TAGS BECAUSE ALREADY DEFINED IN unistd.h
 *
 * macro   SEEK_SET
 * brief   Seek relative to the beginning of file.
 * long    Redefinition of SEEK_SET, a constant to be used as argument whence
 *         in the fseek function.
 * seealso SEEK_CUR
 * seealso SEEK_END
 */
#undef SEEK_SET
#define SEEK_SET         2

/*
 ****************************************************************************
 * Constants to be used with function tmpnam().
 ****************************************************************************
 */

/**
 * @macro   L_tmpnam
 *
 * @include        "stdio.h"
 *
 * @declaration #define L_tmpnam 48
 *
 * @brief   The minimum size in characters of the buffer to be used with the
 *          function @link "" "tmpnam".
 */
#undef L_tmpnam
#define L_tmpnam 48

/**
 * @macro   TMP_MAX
 *
 * @include        "stdio.h"
 *
 * @declaration #define TMP_MAX 25
 *
 * @brief   The minimum number of successive calls to the function
 *          @link "" "tmpnam" that will generate unique names.
 */
#undef TMP_MAX
#define TMP_MAX 25

/*
 ****************************************************************************
 * The standard input/output/error streams.
 ****************************************************************************
 */

/**
 * @macro   stdin
 *
 * @include        "stdio.h"
 *
 * @declaration #define stdin  (efs_stdin())
 *
 * @brief   Standard input stream.
 *
 * @long    Macro expression of type @link "" "FILE" * which refer to the
 *          standard input stream of the calling process.
 *          Must be initiated.
 *
 * @return  Pointer to standard input stream of type @link "" "FILE". If an
 *          error condition is detected, NULL will be returned and
 *          @link "errno.h" "errno" will be set to indicate the error.
 *
 * @error   EBADF errno @link "errno.h" "EBADF".
 *          The standard in file descriptor not opened.
 *
 *
 * @seealso stdout
 * @seealso stderr
 * @seealso default_restrictions_calling_functions
 */
#define stdin  (efs_stdin())

/**
 * @macro   stdout
 *
 * @include        "stdio.h"
 *
 * @declaration #define stdout (efs_stdout())
 *
 * @brief   Standard output stream.
 *
 * @long    Macro expression of type @link "" "FILE" * which refer to the
 *          standard output stream of the calling process.
 *          Must be initiated.
 *
 * @return  Pointer to standard output stream of type @link "" "FILE". If an
 *          error condition is detected, NULL will be returned and
 *          @link "errno.h" "errno" will be set to indicate the error.
 *
 * @error   EBADF errno @link "errno.h" "EBADF".
 *          The standard out file descriptor not opened.
 *
 * @seealso stdin
 * @seealso stderr
 * @seealso default_restrictions_calling_functions
 */
#define stdout (efs_stdout())

/**
 * @macro   stderr
 *
 * @include        "stdio.h"
 *
 * @declaration #define stderr (efs_stderr())
 *
 * @brief   Standard error stream.
 *
 * @long    Macro expression of type @link "" "FILE" * which refer to the
 *          standard error stream of the calling process.
 *          Must be initiated.
 *
 * @return  Pointer to standard error stream of type @link "" "FILE". If an
 *          error condition is detected, NULL will be returned and
 *          @link "errno.h" "errno" will be set to indicate the error.
 *
 * @error   EBADF errno @link "errno.h" "EBADF".
 *          The standard error file descriptor not opened.
 *
 *
 * @seealso stdin
 * @seealso stdout
 * @seealso default_restrictions_calling_functions
 */
#define stderr (efs_stderr())

/*
 ****************************************************************************
 * TYPES
 ****************************************************************************
 */

/**
 * @type  fpos_t
 *
 * @include        "stdio.h"
 *
 * @declaration typedef unsigned long int fpos_t;
 *
 * @brief An object type that can store a file position.
 */
#if !defined(_FPOS_T)
#define _FPOS_T
typedef unsigned long int fpos_t;
#endif

/*
** ARM RVCT wchar.h assumes that FILE is a typedef for struct __FILE.
** For this target, we  adapt to that for use of swscanf, vswscanf, swprintf
** and vswprintf.
*/
#if defined(__ARMCC_VERSION)
#define _FILE __FILE
#endif

struct _FILE;

#if !defined(_FILEDEFED)
#define _FILEDEFED
#define _FILE_DEFINED

/**
 * @type  FILE
 *
 * @include        "stdio.h"
 *
 * @declaration typedef struct _FILE FILE;
 *
 * @brief The FILE abstract data type.
 *
 * @long  An object type that can store all the information required for
 *        a stream.
 */
typedef struct _FILE FILE;
#endif

/*
 ****************************************************************************
 * FUNCTION PROTOTYPES
 ****************************************************************************
 */

/*
 * NO SOURCEDOC TAGS BECAUSE PRIVATE FUNCTION.
 *
 * function efs_stdin
 *
 * brief Private function. Used by stdin.
 *
 * long The efs_stdin function returns a reference to the standard input
 *      stream of the calling process.
 *
 * return  Pointer to standard input stream of type FILE.
 *
 * seealso stdin
 * seealso efs_stdout
 * seealso efs_stderr
 * seealso default_restrictions_calling_functions
 */
FILE *   efs_stdin(void);

/*
 * NO SOURCEDOC TAGS BECAUSE PRIVATE FUNCTION.
 *
 * function efs_stdout
 *
 * brief Private function. Used by stdout.
 *
 * long The efs_stdout function returns a reference to the standard output
 *      stream of the calling process.
 *
 * return  Pointer to standard output stream of type FILE.
 *
 * seealso stdout
 * seealso efs_stdin
 * seealso efs_stderr
 * seealso default_restrictions_calling_functions
 */
FILE *   efs_stdout(void);

/*
 * NO SOURCEDOC TAGS BECAUSE PRIVATE FUNCTION.
 *
 * function efs_stderr
 *
 * brief Private function. Used by stderr.
 *
 * long The efs_stderr function returns a reference to the standard error
 *      stream of the calling process.
 *
 * return  Pointer to standard error stream of type FILE.
 *
 * seealso stderr
 * seealso efs_stdin
 * seealso efs_stdout
 * seealso default_restrictions_calling_functions
 */
FILE *   efs_stderr(void);


/**
 * @function clearerr
 *
 * @include        "stdio.h"
 *
 * @declaration void
 * clearerr(FILE *file);
 *
 * @brief Clear the error and end-of-file indicators on a stream.
 *
 * @long The clearerr function clears the error and the end-of-file indicators
 *       on the specified stream.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file A pointer to the corresponding @link "" "FILE" structure for the
 *             stream to clear the error indicator.
 *
 * @return None.
 *
 * @seealso fopen
 * @seealso rewind
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To clear the error indication indicator of the FILE stream st,
 * // do the following:
 *     clearrerr(st);
 */
void     clearerr(FILE *file);


/**
 * @function fclose
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * fclose(FILE *file);
 *
 * @brief Closes a stream.
 *
 * @long The fclose function causes the stream pointed to by file to be flushed
 *       and the associated file to be closed.
 *       Data that is stored in an output buffer that is waiting to be written
 *       to the associated file is written to the file.
 *       Data in an input buffer that has not been given to the client is
 *       discarded.
 *       If the buffer was assigned by the user, with a call to the function
 *       @link "" "setbuf" or to the function @link "" "setvbuf", the buffer
 *       will not automatically be freed.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file A pointer to the @link "" "FILE" structure of the file that is
 *             going to be closed. The @link "" "FILE" structure is created
 *             by the function @link "" "fopen".
 *
 * @return The function returns zero if successful, otherwise it returns
 *         @link "" "EOF" and errno is set to one of the values below to
 *         indicate the error.
 *         fclose will fail and the file will be unchanged if any of the
 *         following conditions are true:
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The stream argument is not a valid file descriptor.
 * @error EINTR  errno @link "errno.h" "EINTR".
 *               The close function was interrupted.
 * @error ESTALE errno @link "errno.h" "ESTALE".
 *               The path points to a remote machine and the link to that
 *               machine is no longer active.
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso close
 * @seealso fopen
 * @seealso remove
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To close a file, ose.txt, do the following:
 *     // The stream pointer fstream was given when ose.txt was created
 *     int OK;
 *     OK = fclose(fstream);
 *     if (OK == 0)
 *     {
 *       // ose.txt was successfully closed.
 *     }
 */
int      fclose(FILE *file);

/**
 * @function fdopen
 *
 * @include        "stdio.h"
 *
 * @declaration FILE *
 * fdopen(int fd, const char *mode);
 *
 * @brief Associates a stream with an open file descriptor.
 *
 * @long The fdopen function call is used to associate a file stream with the
 *       file descriptor of an already opened file. This file could have been
 *       opened with the function call @link "unistd.h" "open" or the function
 *       call @link "unistd.h" "creat".
 * @long The mode argument is a pointer to a character string having one of the
 *       following values (no others) and meaning:
 * @long - "r": Open for reading.
 * @long - "w": Open for writing.
 * @long - "a": Open for writing at end of file.
 * @long - "r+": Open for update (reading and writing).
 * @long - "w+": Open for update (reading and writing).
 * @long - "a+": Open for update (reading and writing) at end of file.
 * @long The meaning of these flags is exactly as specified for @link
 *       "" "fopen", except that "w" and "w+" do not cause truncation of
 *       the file.
 *
 * @restriction Note that the mode must be the same as the one the file was
 *              originally opened with.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input fd   The file descriptor.
 * @input mode A pointer to the file access mode string.
 *
 * @return The function returns a pointer to a stream if successful, otherwise
 *         it returns NULL and errno is set to one of the values below to
 *         indicate the error.
 *         fdopen will fail if any of the following conditions are true:
 *
 * @error EBADF errno @link "errno.h" "EBADF".
 *              The fd argument is not a valid file descriptor.
 *
 * @seealso fileno
 * @seealso creat
 * @seealso open
 * @seealso fopen
 * @seealso efs_clone
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To get the file stream descriptions of the open file ose.txt:
 *     int fd;
 *     FILE *st;
 *     fd = open("ose.txt", O_RDWR)
 *     // File descriptor fd is associated with a FILE description with:
 *     st = fdopen(fd, "r+");
 *     if (st != NULL)
 *     {
 *       // The pointer of the file was successfully fetched.
 *     }
 */
FILE *   fdopen(int fd, const char *mode);

/**
 * @function feof
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * feof(FILE *file);
 *
 * @brief Checks for @link "" "EOF" on a stream.
 *
 * @long The feof function determines if a stream has reached end-of-file
 *       (@link "" "EOF") indicator. When a stream has reached end-of-file,
 *       all read operations return @link "" "EOF" until the stream is
 *       closed or the function @link "" "rewind" or the function
 *       @link "" "fseek" is called.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file The pointer to the @link "" "FILE" structure.
 *
 * @return The function returns non-zero value if the end-of-file flag
 *         associated with the argument stream is set, otherwise it returns
 *         zero.
 *
 * @seealso clearerr
 * @seealso ferror
 * @seealso perror
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To see if the stream pointer of an open file is at end-of-file,
 * // do the following:
 *     OK = feof(st);
 *     if (OK != 0)
 *     {
 *       // Stream pointer was at end-of-file.
 *     }
 */
int      feof(FILE *file);

/**
 * @function ferror
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * ferror(FILE *file);
 *
 * @brief Checks for stream error.
 *
 * @long The ferror function tests a stream's error flag. If an error has
 *       occurred, the error flag remains set until the stream is closed,
 *       or until a call to the function @link "" "rewind" or to the
 *       function @link "" "clearerr" is made.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file The pointer to the @link "" "FILE" structure.
 *
 * @return The function returns a non-zero value if the error flag associated
 *         with stream is set, otherwise it returns zero.
 *
 * @seealso clearerr
 * @seealso feof
 * @seealso perror
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To see if the error flag of an open file is set, do the following:
 *     int OK;
 *     OK = ferror(st);
 *     if (OK != 0)
 *     {
 *       // Error flag was set.
 *     }
 */
int      ferror(FILE *file);

/**
 * @function fflush
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * fflush(FILE *file);
 *
 * @brief Writes buffered (unwritten) data in a stream from the process space
 *        to the volume manager.
 *
 * @long The fflush function writes all data collected in the output buffer of
 *       the stream to the volume manager, but the stream remains open.
 *       If stream is a NULL pointer, the fflush(NULL) function performs this
 *       flushing action on all streams owned by the process.
 *       The fflush function returns as soon as all data has been transferred to
 *       the volume manager and does not wait for the volume manager to write it
 *       to the media.
 * @long NOTE: To make all data persistent on the media, call fflush followed
 *       by @link "unistd.h" "fsync". To make the entire state
 *       of the volume persistent, see also @link "efs.h" "efs_sync".
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file The pointer to the @link "" "FILE" structure.
 *
 * @return If successful, fflush returns zero.
 *         If an error condition is detected, @link "" "EOF" is returned and
 *         errno is set to indicate the error.
 *
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file points to a struct with illegal magic.
 *
 * @seealso fsync
 * @seealso efs_sync
 * @seealso fclose
 * @seealso ungetc
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To flush the stream of an open file:
 *     OK = fflush(st);
 *     if (OK == 0)
 *     {
 *       // Flush was performed successfully.
 *     }
 */
int      fflush(FILE *file);

/**
 * @function fgetc
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * fgetc(FILE *file);
 *
 * @brief Reads next character from stream.
 *
 * @long Returns the next character from the stream pointed to by file and
 *       advances the file position indicator.
 *       If fgetc encounters the end of file, then it returns
 *       @link "" "EOF" and sets the end of file indicator for stream.
 *       Also, if fgetc encounters an input error while attempting to read
 *       input, it returns @link "" "EOF" and sets the error indicator for
 *       stream.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file The pointer to the @link "" "FILE" structure.
 *
 * @return The function returns the next character from the stream if
 *         successful. If a read error occurs, the error indicator for the
 *         stream is set and fgetc returns @link "" "EOF" and errno is set
 *         to one of the values below to indicate the error.
 *         fgetc will fail and the file will be unchanged if any of the
 *         following conditions are true:
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The stream argument is not a valid file descriptor.
 * @error ESTALE errno @link "errno.h" "ESTALE".
 *               Stream points to a remote machine and the link to that machine
 *               is no longer active.
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso getc
 * @seealso getchar
 * @seealso fgets
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To fetch a character from the file associated with the stream pointer st:
 *     char ch;
 *     ch = fgetc(st); // ch is the fetched character.
 */
int      fgetc(FILE *file);

/**
 * @function fgetpos
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * fgetpos(FILE *file, fpos_t *pos);
 *
 * @brief Gets the current position of a stream as an fpos_t structure.
 *
 * @long The fgetpos function stores the current value of the current file
 *       position, for the stream pointed to by file, into the object pointed
 *       to by pos. The value stored contains unspecified information usable
 *       by the @link "" "fsetpos" function for repositioning the stream to
 *       its position at the time of the call to the fgetpos function.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file The pointer to the @link "" "FILE" structure.
 * @input pos  A pointer to an object that shall store the stream position
 *             value.
 *
 * @return The function returns zero if successful, otherwise it returns
 *         @link "" "EOF" and errno is set to one of the values below to
 *         indicate the error.
 *         fgetpos will fail if any of the following conditions are true:
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The stream argument is not a valid file descriptor.
 * @error ESTALE errno @link "errno.h" "ESTALE".
 *               stream points to a remote machine and the link to that
 *               machine is no longer active.
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso fsetpos
 * @seealso fseek
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To fetch the stream pointer position of the file associated
 * // with the stream st, do the following:
 *     fpos_t *posptr;
 *     OK = fgetpos(st, posptr);
 *     if (OK == 0)
 *     {
 *       // The position of pointer was fetched.
 *     }
 */
int      fgetpos(FILE *file, fpos_t *pos);

/**
 * @function fgets
 *
 * @include        "stdio.h"
 *
 * @declaration char *
 * fgets(char *buffer, int bufsize, FILE *file);
 *
 * @brief Reads a line from a stream into a string.
 *
 * @long The fgets function reads at most one less than the number of
 *       characters specified by bufsize from the stream pointed to by
 *       file into the array pointed to by buffer. No additional
 *       characters are read after a new-line character or after end
 *       of file. Note that the new-line character is included in the
 *       buffer. A null character is written immediately after the
 *       last character read into the array.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input buffer  The pointer to a string.
 * @input bufsize The number of characters in the string. bufsize must be larger
 *                than 0.
 * @input file    The pointer to the @link "" "FILE" structure.
 *
 * @return The function returns a pointer to buffer if successful, otherwise it
 *         returns NULL and set errno to indicate the error.
 *
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso fgetc
 * @seealso getchar
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To fetch a string of 10 characters from the file associated with
 * // the file stream st, do the following:
 *     char buffer[100];
 *     char *s = &buffer[0];
 *     s = fgets(s, 10, st); // s is the pointer to the fetched string.
 */
char *   fgets(char *buffer, int bufsize, FILE *file);


/**
 * @function fileno
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * fileno(FILE *file);
 *
 * @brief Returns the POSIX file descriptor used by an open stream.
 *
 * @long The fileno function call is used to fetch the POSIX file descriptor
 *       of a file associated with the given stream pointer.
 *
 * @long Standards: Portable Operating System Interface (POSIX.1).
 *
 * @input file The pointer to the @link "" "FILE" structure.
 *
 * @return The function returns the file descriptor if successful, otherwise it
 *         returns -1 and errno is set to the value below to indicate the error.
 *         fileno will fail if the following condition is true:
 *
 * @error EBADF errno @link "errno.h" "EBADF".
 *              The stream argument is not a valid stream.
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso fdopen
 * @seealso fopen
 * @seealso open
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To fetch the file descriptor, fd, of the file associated with the
 * // stream pointer st, do the following:
 *     fd = fileno(st);
 *     if (fd <!= EOF)
 *     {
 *       // fd contains a valid file descriptor to stream st.
 *     }
 */
int      fileno(FILE *file);


/**
 * @function fopen
 *
 * @include        "stdio.h"
 *
 * @declaration FILE *
 * fopen(const char *filename, const char *mode);
 *
 * @brief Opens a file as a stream.
 *
 * @long The fopen function opens the file whose name is the string
 *       pointed to by filename, and associates a stream with it. If a
 *       file is created with fopen, the files permission attributes
 *       will always become "read and write, masked with current umask",
 *       independent of the open mode specified.
 *
 * @long The mode argument is a pointer to a character string having one
 *       of the following values and meaning:
 * @long - "r": Open text file for reading.
 * @long - "w": Truncate to zero length or create text file for writing.
 * @long - "a": Append; open or create text file for writing at end of file.
 * @long - "rb": Open binary file for reading.
 * @long - "wb": Truncate to zero length or create binary file for writing.
 * @long - "ab": Append; open or create binary file for writing at end of file
 * @long - "r+": Open text file for update (reading and writing).
 * @long - "w+": Truncate to zero length or create text file for update.
 * @long - "a+": Append open or create text file for update, writing at end of
 *               file.
 * @long - "r+b" or "rb+": Open binary file for update (reading and writing).
 * @long - "w+b" or "wb+": Truncate to zero length or create binary file for
 *                         update.
 * @long - "a+b" or "ab+": Append; open or create binary file for update,
 *                         writing at end of file.
 * @long The combinations specified above can also be suffixed by "t", in
 *       which case the file gets opened in transactional mode O_TRANS (see
 *       @link "" "open") in which case any modifications are rolled back in
 *       case of power outage or crash before @link "" "close" or @link ""
 *       "fsync". Similar to O_TRANS, the "t" flag will be silently ignored
 *       if the volume manager does not support transactional mode.
 * @long Opening a file with read mode, "r" as the first character in the mode
 *       argument, fails if the file does not exist or can not be read.
 * @long Opening a file with append mode, "a" as the first character in the
 *       mode argument, causes all subsequent writes to the file to be forced
 *       to the current end of the file, regardless of intervening calls to
 *       @link "" "fseek" function. The file position will be left at end of
 *       file after each write.
 * @long When a file is opened with update mode, "+" as the second or third
 *       character in the above list of mode argument values, both input and
 *       output may be performed on the associated stream.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input filename A pointer to a string with the file name.
 * @input mode     A pointer to a string with the access mode.
 *
 * @return The function returns a pointer to the @link "" "FILE" object
 *         controlling the stream if successful, otherwise it returns NULL
 *         and errno is set to a value to indicate the error. See errno
 *         error codes for @link "unistd.h" "open".
 *
 * @seealso creat
 * @seealso fdopen
 * @seealso fflush
 * @seealso efs_clone
 * @seealso tmpfile
 * @seealso open
 * @seealso remove
 * @seealso umask
 * @seealso wfopen
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To open a text file ose.txt to read from, do the following:
 *     st = fopen("ose.txt","r");
 *     if (st != NULL)
 *     {
 *       // st points at the opened file
 *     }
 */
FILE *   fopen(const char *filename, const char *mode);


/**
 * @function wfopen
 *
 * @include        "stdio.h"
 *
 * @declaration FILE *
 * wfopen(const wchar_t *filename, const char *mode);
 *
 * @brief Opens a file as a stream.
 *
 * @long This function is equivalent to fopen, with the exception that
 *       the pathname is specified as a wide character string. Note that
 *       the mode string is still a char string.
 *
 * @long The wfopen function opens the file whose name is the string
 *       pointed to by filename, and associates a stream with it. If a
 *       file is created with wfopen, the files permission attributes
 *       will always become "read and write, masked with current umask",
 *       independent of the open mode specified.
 *
 * @long The mode argument is a pointer to a character string having one
 *       of the following values and meaning:
 * @long - "r": Open text file for reading.
 * @long - "w": Truncate to zero length or create text file for writing.
 * @long - "a": Append; open or create text file for writing at end of file.
 * @long - "rb": Open binary file for reading.
 * @long - "wb": Truncate to zero length or create binary file for writing.
 * @long - "ab": Append; open or create binary file for writing at end of file
 * @long - "r+": Open text file for update (reading and writing).
 * @long - "w+": Truncate to zero length or create text file for update.
 * @long - "a+": Append open or create text file for update, writing at end of
 *               file.
 * @long - "r+b" or "rb+": Open binary file for update (reading and writing).
 * @long - "w+b" or "wb+": Truncate to zero length or create binary file for
 *                         update.
 * @long - "a+b" or "ab+": Append; open or create binary file for update,
 *                         writing at end of file.
 * @long The combinations specified above can also be suffixed by "t", in
 *       which case the file gets opened in transactional mode O_TRANS (see
 *       @link "" "open") in which case any modifications are rolled back in
 *       case of power outage or crash before @link "" "close" or @link ""
 *       "fsync". Similar to O_TRANS, the "t" flag will be silently ignored
 *       if the volume manager does not support transactional mode.
 * @long Opening a file with read mode, "r" as the first character in the mode
 *       argument, fails if the file does not exist or can not be read.
 * @long Opening a file with append mode, "a" as the first character in the
 *       mode argument, causes all subsequent writes to the file to be forced
 *       to the current end of the file, regardless of intervening calls to
 *       @link "" "fseek" function. The file position will be left at end of
 *       file after each write.
 * @long When a file is opened with update mode, "+" as the second or third
 *       character in the above list of mode argument values, both input and
 *       output may be performed on the associated stream.
 *
 * @long Standards: None - This is an OSE-specific extension.
 *
 * @input filename A pointer to a string with the file name.
 * @input mode     A pointer to a string with the access mode.
 *
 * @return The function returns a pointer to the @link "" "FILE" object
 *         controlling the stream if successful, otherwise it returns NULL
 *         and errno is set to a value to indicate the error. See errno
 *         error codes for @link "unistd.h" "open".
 *
 * @seealso wcreat
 * @seealso fdopen
 * @seealso fflush
 * @seealso efs_clone
 * @seealso tmpfile
 * @seealso wopen
 * @seealso wremove
 * @seealso umask
 * @seealso fopen
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To open a text file ose.txt to read from, do the following:
 *     st = wfopen(L"ose.txt","r");
 *     if (st != NULL)
 *     {
 *       // st points at the opened file
 *     }
 */
FILE * wfopen(const wchar_t *filename, const char *mode);


/**
 * @function fprintf
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * fprintf(FILE *file, const char *format, ...);
 *
 * @brief Prints formatted data to a stream.
 *
 * @long The fprintf function writes output to the stream pointed to by file,
 *       under control of the string pointed to by format that specifies how
 *       subsequent arguments are converted for output. The fprintf function
 *       returns when the end of the format string is encountered.
 * @long NOTE: All XXprintf() format conversions use a common function which
 *       is delivered in a source file, outfmt.c. To use any of the XXprintf()
 *       functions, this source file must be compiled with or without hardware
 *       floating point or without floating point support and linked with the
 *       application.
 * @long The following command strings are valid to compose a format string:
 * @long - "%d": Decimal integer.
 * @long - "%i": Decimal integer.
 * @long - "%u": Unsigned decimal.
 * @long - "%x": Hexadecimal.
 * @long - "%X": Hexadecimal with capital letters.
 * @long - "%o": Octal.
 * @long - "%f": Floating point double value.
 * @long - "%e", "%E": Floating point double value in exponential notation.
 * @long - "%g", "%G": Floating point double value in notation depending on
 *                     precision.
 * @long - "%c": Character.
 * @long - "%s": String.
 * @long - "%p": Pointer.
 * @long - "%m.n": Field width, precision.
 * @long - "%-": Left adjustment.
 * @long - "%+": Explicit '+' for positive numbers.
 * @long - "%space": Extra leading space for positive numbers.
 * @long - "%0": Zero-padding.
 * @long - "%l": Argument is a long int.
 * @long - "%h": Argument is short.
 * @long - "%*.*": Width and precision taken from arguments.
 * @long - "%#":  Alternate format.
 * @long - "%n": Intermediate return of number of characters printed.
 * @long If an attempt is made to print a NULL pointer as a string, "(null)" is
 *       printed.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file   The pointer to the @link "" "FILE" structure.
 * @input format The format command string.
 * @input ...    Are the items to be printed formatted.
 *
 * @return The function returns the number of characters transmitted if
 *         successful, otherwise it returns @link "" "EOF" and errno is
 *         set to one of the values below to indicate the error.
 *         fprintf will fail and the file will be unchanged if any of
 *         the following conditions are true:
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The stream argument is not a valid file descriptor.
 * @error ESTALE errno @link "errno.h" "ESTALE".
 *               Stream points to a remote machine and the link to that
 *               machine is no longer active.
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso fscanf
 * @seealso fread
 * @seealso fwrite
 * @seealso eprintf
 * @seealso printf
 * @seealso sprintf
 * @seealso vprintf
 * @seealso vfprintf
 * @seealso vsprintf
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To print two strings and a character to the file ose.txt associated
 * // with the stream pointer st, do the following:
 *     char *string1, *string2, *ch;
 *     num = fprintf(st,"%s, %s %c", string1, string2, ch);
 *     if (num == (strlen(string1) + strlen(string2) + 4) )
 *     {
 *       // The call was successful.
 *     }
 */
int      fprintf(FILE *file, const char *format, ...);


/**
 * @function fputc
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * fputc(int c, FILE *file);
 *
 * @brief Writes a character to a stream.
 *
 * @long The fputc function writes the character specified by c (converted
 *       to an unsigned char) to the output stream pointed to by file, at
 *       the position indicated by the associated file position indicator
 *       for the stream (if defined), and advances the indicator
 *       appropriately. If the file can not support positioning requests,
 *       or if the stream was opened in append mode, the character is
 *       appended to the output stream.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input c    The character to insert into the stream.
 * @input file The pointer to a @link "" "FILE" structure.
 *
 * @return The function returns the character written if successful,
 *         otherwise it returns @link "" "EOF" and errno is set to one
 *         of the values below to indicate the error.
 *         fputc will fail and the file will be unchanged if any of the
 *         following conditions are true:
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The stream argument is not a valid file descriptor.
 * @error ESTALE errno @link "errno.h" "ESTALE".
 *               Stream points to a remote machine and the link to that
 *               machine is no longer active.
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso fputs
 * @seealso putchar
 * @seealso putc
 * @seealso fgetc
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To put a character into the file ose.txt associated with the stream
 * // pointer st, do the following:
 *     newch = fputc(ch, st);
 *     if (ch == newch)
 *     {
 *       // The call was successful.
 *     }
 */
int      fputc(int c, FILE *file);


/**
 * @function fputs
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * fputs(const char *str, FILE *file);
 *
 * @brief Writes a string to a stream.
 *
 * @long The fputs function writes the string pointed to by str to the stream
 *       pointed to by file. The terminating null character is not written.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input str  The character string to put into the stream. It must be
 *             terminated with a NULL character.
 * @input file The pointer to the @link "" "FILE" structure.
 *
 * @return The function returns the number of characters written if
 *         successful, otherwise it returns @link "" "EOF" and errno is
 *         set to one of the values below to indicate the error.
 *         fputs will fail and the file will be unchanged if any of the
 *         following conditions are true:
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The stream argument is not a valid file descriptor.
 * @error ESTALE errno @link "errno.h" "ESTALE".
 *               Stream points to a remote machine and the link to that
 *               machine is no longer active.
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso puts
 * @seealso fputc
 * @seealso putchar
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To put a character string into the stream st do the following:
 *     int OK;
 *     char mess[] = "Hello World\n"; // fputs does not append a \n like puts.
 *     OK = fputs(mess, st);
 *     if (OK == strlen(mess) )
 *     {
 *       // The call was successful.
 *     }
 */
int      fputs(const char *str, FILE *file);


/**
 * @function fread
 *
 * @include        "stdio.h"
 *
 * @declaration size_t
 * fread(void *buffer, size_t size, size_t nmemb, FILE *file);
 *
 * @brief Reads data from a stream.
 *
 * @long The fread function reads (into the array pointed to by buffer)
 *       up to nmemb elements whose size are specified by size, from the
 *       stream pointed to by file. The file position indicator for the
 *       stream (if defined) is advanced by the number of characters
 *       successfully read.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input buffer The pointer to an array into which the elements are read.
 * @input size   The size of the elements.
 * @input nmemb  The number of elements.
 * @input file   The pointer to the @link "" "FILE" structure.
 *
 * @return The function returns the number of elements successfully read if
 *         successful, otherwise it returns zero and sets errno to indicate
 *         the error. If size or nmemb is zero, zero will be returned and errno
 *         will remain unchanged. To be able to distinguish between end-of-file
 *         condition and error condition, use @link "" "ferror" function.
 *
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso fprintf
 * @seealso fscanf
 * @seealso fwrite
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To read 10 items of size 8 bytes, from the file associated with the stream
 * // pointer st, do the following:
 *     // Allocate buffer. Remember to free it afterwards!
 *     destination = alloc(80, 0);
 *     items = fread(destination, 8, 10, st);
 *     if (items == 10)
 *     {
 *       // The call was successful.
 *     }
 */
size_t   fread(void *buffer, size_t size, size_t nmemb, FILE *file);


/**
 * @function freopen
 *
 * @include        "stdio.h"
 *
 * @declaration FILE *
 * freopen(const char *filename, const char *mode, FILE *file);
 *
 * @brief Opens a file, reusing another open stream.
 *
 * @long The freopen function opens the file whose name is the string
 *       pointed to by filename and associates the stream pointed to by
 *       file with it. The mode argument is used just as in the
 *       @link "" "fopen" function.
 *
 * @long The freopen function first attempts to close any file that is
 *       associated with the specified stream. Failure to close the file
 *       is ignored.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input filename The pointer to a string with the name of the file.
 * @input mode     The pointer to a string with the stream mode.
 *                 See @link "" "fopen".
 * @input file     The pointer to the @link "" "FILE" structure.
 *
 * @return The function returns a pointer to stream if successful, otherwise it
 *         returns NULL and updates @link "errno.h" "errno".
 *
 * @seealso fopen
 * @seealso fdopen
 * @seealso open
 * @seealso wfreopen
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To associate an open stream with a new file ose.txt do the following:
 *     FILE *newst;
 *     newst = freopen("ose.txt", "a", st);
 *     if (newst != NULL)
 *     {
 *       // The call was successful.
 *     }
 */
FILE *   freopen(const char *filename, const char *mode, FILE *file);


/**
 * @function wfreopen
 *
 * @include        "stdio.h"
 *
 * @declaration FILE *
 * wfreopen(const wchar_t *filename, const char *mode, FILE *file);
 *
 * @brief Opens a file, reusing another open stream.
 *
 * @long This function is equivalent to freopen, with the exception that
 *       the pathname is specified as a wide character string. Note that
 *       the mode string is still a char string.
 *
 * @long The wfreopen function opens the file whose name is the string
 *       pointed to by filename and associates the stream pointed to by
 *       file with it. The mode argument is used just as in the
 *       @link "" "fopen" function.
 *
 * @long The wfreopen function first attempts to close any file that is
 *       associated with the specified stream. Failure to close the file
 *       is ignored.
 *
 * @long Standards: None - This is an OSE-specific extension.
 *
 * @input filename The pointer to a string with the name of the file.
 * @input mode     The pointer to a string with the stream mode.
 *                 See @link "" "fopen".
 * @input file     The pointer to the @link "" "FILE" structure.
 *
 * @return The function returns a pointer to stream if successful, otherwise it
 *         returns NULL and updates @link "errno.h" "errno".
 *
 * @seealso wfopen
 * @seealso fdopen
 * @seealso wopen
 * @seealso freopen
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To associate an open stream with a new file ose.txt do the following:
 *     FILE *newst;
 *     newst = wfreopen(L"ose.txt", "a", st);
 *     if (newst != NULL)
 *     {
 *       // The call was successful.
 *     }
 */
FILE * wfreopen(const wchar_t *filename, const char *mode, FILE *file);


/**
 * @function fscanf
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * fscanf(FILE *file, const char *format, ...);
 *
 * @brief Reads formatted data from a stream.
 *
 * @long The fscanf function reads input from the stream pointed to by
 *       file, under control of the string pointed to by format that
 *       specifies the admissible input sequences and how they are to be
 *       converted for assignment, using subsequent arguments as pointers
 *       to the objects to receive the converted input.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file   The pointer to the @link "" "FILE" structure.
 * @input format The format features (see @link "" "fprintf").
 * @input ...    Pointers to objects receiving the converted input.
 *
 * @return The function returns the number of input items assigned if
 *         successful, otherwise it returns @link "" "EOF" and errno is
 *         set to one of the values below to indicate the error.
 *         fscanf will fail if any of the following conditions are true:
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The stream argument is not a valid file descriptor.
 * @error ESTALE errno @link "errno.h" "ESTALE".
 *               Stream points to a remote machine and the link to that
 *               machine is no longer active.
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso fprintf
 * @seealso scanf
 * @seealso fread
 * @seealso fwrite
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To read up to 10 characters from a file associated with the stream
 * // pointer st do the following:
 *     char string[10+1]; // including null termination
 *     num = fscanf(st, "%10s", &string);
 *     if (num == 1)
 *     {
 *       // The call was successful.
 *     }
 */
int      fscanf(FILE *file, const char *format, ...);


/**
 * @function fseek
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * fseek(FILE *file, long int offset, int whence);
 *
 * @brief Sets the position of a stream, relative to beginning/end/current.
 *
 * @long The fseek function sets the file position indicator for the stream
 *       pointed to by file. The new position, measured in characters from
 *       the beginning of the file, is obtained by adding offset to the
 *       position specified by whence.
 * @long The whence specified position is the beginning of the file
 *       if whence is @link "unistd.h" "SEEK_SET",
 *       the current value of the file position indicator
 *       if @link "unistd.h" "SEEK_CUR",
 *       or end-of-file if whence is @link "unistd.h" "SEEK_END".
 *
 * @restriction It depends on the volume manager if the function is supported.
 *       The current volume managers do not support positioning outside the
 *       current file data.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file   The pointer to the @link "" "FILE" structure.
 * @input offset The number of bytes from the position indicated by whence.
 * @input whence A position of the file.
 *
 * @return The function returns zero if successful, otherwise it returns
 *         @link "" "EOF" and errno is set to one of the values below to
 *         indicate the error.
 *         fseek will fail and the file position indicator will be
 *         unchanged if any of the following conditions are true:
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The stream argument is not a valid file descriptor.
 * @error ESTALE errno @link "errno.h" "ESTALE".
 *               Stream points to a remote machine and the link to that
 *               machine is no longer active.
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso fgetpos
 * @seealso fsetpos
 * @seealso lseek
 * @seealso rewind
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To move the file position indicator of a binary file 10 bytes from
 * // the beginning, do the following:
 *     int OK;
 *     OK = fseek(st, 10, SEEK_SET);
 *     if (OK == 0)
 *     {
 *       // The file position indicator was successfully moved.
 *     }
 */
int      fseek(FILE *file, long int offset, int whence);


/**
 * @function fsetpos
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * fsetpos(FILE *file, const fpos_t *pos);
 *
 * @brief Sets the position of a stream as an fpos_t structure.
 *
 * @long The fsetpos function sets the file position indicator for the
 *       stream, pointed to by file, according to the value of the object
 *       pointed to by pos. The pos value must be obtained from an earlier
 *       call to the function @link "" "fgetpos" on the same stream.
 * @long A successful call to fsetpos clears the end-of-file indicator for
 *       the stream and undoes any effect of the function call
 *       @link "" "ungetc" on the same stream.
 *
 * @restriction It depends on the volume manager if the function is supported.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file The pointer to the @link "" "FILE" structure
 * @input pos  A pointer to the position.
 *
 * @return The function returns zero if successful, otherwise it returns
 *         @link "" "EOF" and errno is set to one of the values below to
 *         indicate the error.
 *         fsetpos will fail and the file position indicator will be unchanged
 *         if any of the following conditions are true:
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The stream argument is not a valid file descriptor.
 * @error ESTALE errno @link "errno.h" "ESTALE".
 *               Stream points to a remote machine and the link to that
 *               machine is no longer active.
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso fgetpos
 * @seealso ungetc
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To set the file position of the file associated with the pointer st,
 * // do the following:
 *     fpos_t *posptr;
 *     fgetpos(st, posptr);
 *     // Other things are performed...
 *     OK = fsetpos(st, posptr);
 *     if (OK == 0)
 *     {
 *       // Pointer was set back to the position it was fetched from.
 *     }
 */
int      fsetpos(FILE *file, const fpos_t *pos);


/**
 * @function ftell
 *
 * @include        "stdio.h"
 *
 * @declaration long int
 * ftell(FILE *file);
 *
 * @brief Gets current position of a stream as a long.
 *
 * @long The ftell function obtains the current value of the file position
 *       indicator for the stream pointed to by file.
 *       For a binary stream, the value is the number of characters from the
 *       beginning of the file.
 *       For a text stream, the value does not indicate the number of
 *       characters from the beginning of the file because of carriage-return
 *       and new-line conversion, The file position indicator is usable by
 *       the function @link "" "fseek" for returning the file position
 *       indicator for the stream to its position at the time of the ftell
 *       call; the difference between two such return values is not
 *       necessarily a meaningful measure of the number of characters
 *       written or read.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file The pointer to the @link "" "FILE" structure.
 *
 * @return The function returns the current value of the file position
 *         indicator if successful, otherwise it returns @link "" "EOF" and
 *         errno is set to one of the values below to indicate the error.
 *         ftell will fail if any of the following conditions are true:
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The stream argument is not a valid file descriptor.
 * @error ESTALE errno @link "errno.h" "ESTALE".
 *               Stream points to a remote machine and the link to that
 *               machine is no longer active.
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso fseek
 * @seealso fgetpos
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To fetch the position of a file, do the following:
 *     long int position;
 *     position = ftell(st);
 *     if (position == EOF)
 *     {
 *       // An error has occurred.
 *     }
 */
long int ftell(FILE *file);


/**
 * @function fwrite
 *
 * @include        "stdio.h"
 *
 * @declaration size_t
 * fwrite(const void *buffer, size_t size, size_t nmemb, FILE *file);
 *
 * @brief Writes data to a stream.
 *
 * @long The fwrite function writes, from the array pointed to by buffer, up to
 *       nmemb elements whose size are specified by size, to the stream pointed
 *       to by file. The file position indicator for the stream (if defined) is
 *       advanced by the number of characters successfully written.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input buffer The pointer to an array from which the elements are read.
 * @input size   The size of the elements.
 * @input nmemb  The number of elements.
 * @input file   The pointer to the @link "" "FILE" structure to which the items
 *               are written.
 *
 * @return The function returns the number of successfully written elements.
 *         If an error is detected, the return value might be less than the
 *         requested number of elements and the stream error indicator and
 *         errno will be set. If size or nmemb is zero, the return value is
 *         zero and the stream will remain unchanged.
 *
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso fopen
 * @seealso fprintf
 * @seealso fread
 * @seealso fscanf
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To write 1 item, of size string length, to the file associated with the
 * // stream pointer st, do the following:
 *     char *string = "read this text";
 *     size_t itemcount;
 *     itemcount = fwrite(string, strlen(string), 1, st);
 *     if (!ferror(st))
 *     {
 *       // The call was successful and itemcount holds number of items written
 *       // which is 1 here.
 *     }
 */
size_t   fwrite(const void *buffer, size_t size, size_t nmemb, FILE *file);


/**
 * @function getc
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * getc(FILE *file);
 *
 * @brief Reads next character from a stream. Same as @link "" "fgetc".
 *
 * @long The getc function is equivalent to the @link "" "fgetc" function.
 *
 * @restriction The getc function is implemented with a macro (with
 *              @link "" "fgetc") and thus it is not possible to reference the
 *              address of getc.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file    The file stream to get the character from.
 *
 * @seealso fgetc
 * @seealso fgets
 * @seealso getchar
 * @seealso default_restrictions_calling_functions
 */
/* Function prototype: see getchar. */


/**
 * @function getchar
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * getchar(void);
 *
 * @brief Reads next character from @link "" "stdin".
 *
 * @long The getchar function is equivalent to the function @link "" "fgetc"
 *       with the argument @link "" "stdin".
 *
 * @restriction The getchar function is implemented as a macro (with
 *              @link "" "fgetc").
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @return The function returns the next character from the stream if
 *         successful. If a read error occurs, the error indicator for the
 *         stream is set and getchar returns @link "" "EOF" and errno is
 *         set to one of the values below to indicate the error.
 *         getchar will fail and the file will be unchanged if any of the
 *         following conditions are true:
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The stream @link "" "stdin" is not a valid file descriptor.
 * @error ESTALE errno @link "errno.h" "ESTALE".
 *               @link "" "stdin" points to a remote machine and the link
 *               to that machine is no longer active.
 *
 * @seealso fgetc
 * @seealso fgets
 * @seealso putchar
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To fetch a character from stdin:
 *     char ch;
 *     ch = getchar(); // ch is the fetched character.
 */
#ifdef __cplusplus
inline int getc(FILE *file) { return fgetc(file);  }
inline int getchar(void)    { return fgetc(stdin); }
#undef fgetc
#else
#define getc(f)   fgetc((f))
#define getchar() fgetc(stdin)
#endif

/**
 * @function gets
 *
 * @include        "stdio.h"
 *
 * @declaration char *
 * gets(char *buffer);
 *
 * @brief Reads a line, or until @link "" "EOF", from @link "" "stdin".
 *
 * @long The gets function reads characters from standard input into buffer,
 *       until the next new line or @link "" "EOF". It returns a pointer to
 *       buffer if successful.
 * @long Warning! There is no limit to the number of characters that can
 *       be read into the buffer. Thus it is recommended not to use this
 *       function. Use @link "" "fgets" instead.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input buffer The pointer to a string.
 *
 * @return The function returns a pointer to buffer if successful, otherwise it
 *         returns NULL. NULL is also returned if the stream already is at the
 *         end-of-file before gets.
 *
 * @seealso fgets
 * @seealso default_restrictions_calling_functions
 */
char *   gets(char *buffer); /* Dangerous by design, don't use. */


/**
 * @function perror
 *
 * @include        "stdio.h"
 *
 * @declaration void
 * perror(const char *str);
 *
 * @brief Gets the error message for current errno.
 *
 * @long The perror function maps the error number in the integer expression
 *       errno to an error message. It writes a sequence of characters to
 *       the standard error (@link "" "stderr"). First (if str is not a NULL
 *       pointer or a null string), the string str is output followed by a
 *       colon (:) and a space. Then an appropriate error message string is
 *       output followed by a new-line character. It is the same messages as
 *       in the function @LINK "" "strerror".
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input str A pointer to a string containing a user message.
 *
 * @return The perror function returns no value.
 *
 * @seealso fprintf
 * @seealso ferror
 * @seealso strerror
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To write the error message corresponding to the error code EBADF in the
 * // format "EBADF: Bad file name" to stderr, do the following:
 *     #include "errno.h" // To use the error code mnemonics.
 *     errno = EBADF;
 *     perror("EBADF");
 */
void     perror(const char *str);


/**
 * @function printf
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * printf(const char *format, ...);
 *
 * @brief Prints formatted data to @link "" "stdout".
 *
 * @long The printf function creates formatted output onto @link "" "stdout".
 *       Otherwise its formatting features are the same as for the
 *       @link "" "fprintf" function.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input format A pointer to a string containing formatting code and
 *               characters to include.
 * @input ...    Optional arguments, which must correspond to the formatting
 *               codes in the format string.
 *
 * @return The function returns the number of characters which were output
 *         or a negative value if an error occurred.
 *
 * @seealso eprintf
 * @seealso fprintf
 * @seealso sprintf
 * @seealso vprintf
 * @seealso vfprintf
 * @seealso vsprintf
 * @seealso default_restrictions_calling_functions
 *
 * @example noOfChar = printf("Hello");
 */
int      printf(const char *format, ...);


/**
 * @function putc
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * putc(int c, FILE *file)
 *
 * @brief Writes a character to a stream, same as @link "" "fputc".
 *
 * @long The putc function is equivalent to the function @link "" "fputc".
 *
 * @restriction The putc function is implemented with a macro (with
 *              @link "" "fputc") and thus it is not possible to reference the
 *              address of putc.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input c       The character to put into the @link "" "stdout" stream.
 * @input file    The file stream to put the character in.
 *
 * @seealso getc
 * @seealso putchar
 * @seealso fputc
 * @seealso fgetc
 * @seealso default_restrictions_calling_functions
 */
/* Function prototype: see putchar. */


/**
 * @function putchar
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * putchar(int c)
 *
 * @brief Writes a character to @link "" "stdout".
 *
 * @long The putchar function is equivalent to the function @link "" "fputc"
 *       with the second argument @link "" "stdout".
 *
 * @restriction The putchar function is implemented with a macro (with
 *              @link "" "fputc").
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input c The character to put into the @link "" "stdout" stream.
 *
 * @return The function returns the character written if successful,
 *         otherwise it returns @link "" "EOF" and errno is set to one
 *         of the values below to indicate the error.
 *         putchar will fail and the file will be unchanged if any of
 *         the following conditions are true:
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The stream @link "" "stdout" is not a valid file descriptor.
 * @error ESTALE errno @link "errno.h" "ESTALE".
 *               @link "" "stdout" points to a remote machine and the link
 *               to that machine is no longer active.
 *
 * @seealso getchar
 * @seealso fputc
 * @seealso fgetc
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To write a character to stdout do the following:
 *     char newch, ch =  a ;
 *     newch = putchar(ch);
 *     if (ch == newch)
 *     {
 *       // The call was successful.
 *     }
 */
#ifdef __cplusplus
inline int putc(int c, FILE *file) { return fputc(c,file);   }
inline int putchar(int c)          { return fputc(c,stdout); }
#undef fputc
#else
#define putc(c,f)  fputc((c),(f))
#define putchar(c) fputc((c),stdout)
#endif


/**
 * @function puts
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * puts(const char *str);
 *
 * @brief Writes a string to @link "" "stdout".
 *
 * @long The puts function works like the function @link "" "fputs" with the
 *       second argument @link "" "stdout" and in addition outputs a new-line
 *       character (\n) at the end. The terminating null is NOT written.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input str A pointer to a string of characters to put into the @link ""
 *            "stdout" stream. The string must be terminated with a NULL byte.
 *
 * @return The function returns the number of characters written if successful,
 *         otherwise it returns @link "" "EOF" and errno is set to one of the
 *         values below to indicate the error.
 *         puts will fail and the file will be unchanged if any of the following
 *         conditions are true:
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The stream @link "" "stdout" is not a valid file descriptor.
 * @error ESTALE errno @link "errno.h" "ESTALE".
 *               @link "" "stdout" points to a remote machine and the link
 *               to that machine is no longer active.
 *
 * @seealso fputs
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To write a string to stdout do the following:
 *     int NoOfChar;
 *     char *str = "This is a string";
 *     NoOfChar = puts(str); // puts automatically appends a \n.
 *     if (NoOfChar == strlen(str) + 1)
 *     {
 *       // The call was successful.
 *     }
 */
int      puts(const char *str);


/**
 * @function remove
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * remove(const char *pathname);
 *
 * @brief Removes a file or directory.
 *
 * @long The remove function call is used to delete a specified file
 *       or directory. The remove function causes the file or directory
 *       whose name is the string pointed to by pathname to be no longer
 *       accessible by that name. A subsequent attempt to open that file or
 *       directory using that name will fail, unless it is created again. If
 *       the file or directory is open, the behavior of the remove
 *       function is volume manager dependent.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input pathname  A pointer to a string with the file name to be removed.
 *
 * @return The function returns zero if successful, otherwise it returns
 *         @link "" "EOF" and errno is set to one of the values below to
 *         indicate the error. remove will fail if any of the following
 *         conditions are true:
 *
 * @error EACCES      errno @link "errno.h" "EACCES".
 *                    Search permission is denied for a component of the path
 *                    prefix.
 * @error EBUSY       errno @link "errno.h" "EBUSY".
 *                    The file named by the pathname argument can not be
 *                    removed because it is being used by a client.
 * @error ENODEV      errno @link "errno.h" "ENODEV".
 *                    No such device.
 * @error ENOENT      errno @link "errno.h" "ENOENT".
 *                    The named file does not exist, or the path argument
 *                    points to an empty string.
 * @error ENOTDIR     errno @link "errno.h" "ENOTDIR".
 *                    A component of the path prefix is not a directory.
 * @error ENOTMOUNTED errno @link "errno.h" "ENOTMOUNTED".
 *                    Not mounted.
 * @error EPERM       errno @link "errno.h" "EPERM".
 *                    Operation not permitted.
 * @error EROFS       errno @link "errno.h" "EROFS".
 *                    Read only file system.
 * @error ESTALE      errno @link "errno.h" "ESTALE".
 *                    The path points to a remote machine and the link to
 *                    that machine is no longer active.
 *
 * @seealso rmdir
 * @seealso unlink
 * @seealso wremove
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To remove the file ose.txt do the following:
 *     del = remove("ose.txt");
 *     if (del == 0)
 *     {
 *       // The file was successfully deleted.
 *     }
 */
int      remove(const char *pathname);


/**
 * @function wremove
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * wremove(const wchar_t *pathname);
 *
 * @brief Removes a file or directory.
 *
 * @long This function is equivalent to remove, with the exception that
 *       the pathname is specified as a wide character string.
 *
 * @long The wremove function call is used to delete a specified file
 *       or directory. The wremove function causes the file or directory
 *       whose name is the string pointed to by pathname to be no longer
 *       accessible by that name. A subsequent attempt to open that file or
 *       directory using that name will fail, unless it is created again. If
 *       the file or directory is open, the behavior of the wremove
 *       function is volume manager dependent.
 *
 * @long Standards: None - This is an OSE-specific extension.
 *
 * @input pathname  A pointer to a string with the file name to be removed.
 *
 * @return The function returns zero if successful, otherwise it returns
 *         @link "" "EOF" and errno is set to one of the values below to
 *         indicate the error. remove will fail if any of the following
 *         conditions are true:
 *
 * @error EACCES      errno @link "errno.h" "EACCES".
 *                    Search permission is denied for a component of the path
 *                    prefix.
 * @error EBUSY       errno @link "errno.h" "EBUSY".
 *                    The file named by the pathname argument can not be
 *                    removed because it is being used by a client.
 * @error ENODEV      errno @link "errno.h" "ENODEV".
 *                    No such device.
 * @error ENOENT      errno @link "errno.h" "ENOENT".
 *                    The named file does not exist, or the path argument
 *                    points to an empty string.
 * @error ENOTDIR     errno @link "errno.h" "ENOTDIR".
 *                    A component of the path prefix is not a directory.
 * @error ENOTMOUNTED errno @link "errno.h" "ENOTMOUNTED".
 *                    Not mounted.
 * @error EPERM       errno @link "errno.h" "EPERM".
 *                    Operation not permitted.
 * @error EROFS       errno @link "errno.h" "EROFS".
 *                    Read only file system.
 * @error ESTALE      errno @link "errno.h" "ESTALE".
 *                    The path points to a remote machine and the link to
 *                    that machine is no longer active.
 *
 * @seealso wrmdir
 * @seealso wunlink
 * @seealso remove
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To remove the file ose.txt do the following:
 *     del = wremove(L"ose.txt");
 *     if (del == 0)
 *     {
 *       // The file was successfully deleted.
 *     }
 */
int wremove(const wchar_t *pathname);


/**
 * @function rename
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * rename(const char *oldname, const char *newname);
 *
 * @brief Renames a file or directory.
 *
 * @long The rename function is used to change the name of a specified file,
 *       from the name oldname to the name newname. The destination file
 *       will be unlinked if it exists. If the file is open, the behavior
 *       of the rename function is volume manager dependent.
 * @long The behavior when attempting to rename a directory is volume manager
 *       dependent.
 * @long A file can be renamed to another path. The behavior when renaming
 *       to another path within the same volume is volume manager dependent.
 *
 * @restriction Renaming to a path in another volume is not supported.
 *
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input oldname A pointer to a string with the file name of the file
 *                to rename.
 * @input newname A pointer to a string with the requested new file name.
 *
 * @return The function returns zero if successful, otherwise it returns
 *         @link "" "EOF" and errno is set to one of the values below to
 *         indicate the error.
 *         rename will fail if any of the following conditions are true:
 *
 * @error EAGAIN       errno @link "errno.h" "EAGAIN".
 *                     Lost contact with the volume manager.
 * @error EOPNOTSUPP   errno @link "errno.h" "EOPNOTSUPP".
 *                     Not supported to move across volumes.
 * @error ENOENT       errno @link "errno.h" "ENOENT".
 *                     No such volume.
 * @error EACCES       errno @link "errno.h" "EACCES".
 *                     Search permission is denied for a component of the
 *                     path prefix.
 * @error EBUSY        errno @link "errno.h" "EBUSY".
 *                     The file named by the oldname argument can not be
 *                     removed because it is being used by a client.
 * @error ENAMETOOLONG errno @link "errno.h" "ENAMETOOLONG".
 *                     File name too long.
 * @error ENOTMOUNTED  errno @link "errno.h" "ENOTMOUNTED".
 *                     Not mounted.
 * @error EPERM        errno @link "errno.h" "EPERM".
 *                     Operation not permitted.
 * @error EROFS        errno @link "errno.h" "EROFS".
 *                     Read only file system.
 * @error ESTALE       errno @link "errno.h" "ESTALE".
 *                     The path points to a remote machine and the link to
 *                     that machine is no longer active.
 *
 * @seealso creat
 * @seealso open
 * @seealso wrename
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To change the name of the file ose.txt to efsose.txt do the following:
 *     ren = rename("ose.txt", "efsose.txt");
 *     if (ren == 0)
 *     {
 *       // The file was successfully renamed.
 *     }
 */
int      rename(const char *oldname, const char *newname);


/**
 * @function wrename
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * wrename(const wchar_t *oldname, const wchar_t *newname);
 *
 * @brief Renames a file or directory.
 *
 * @long This function is equivalent to rename, with the exception that
 *       the pathnames are specified as a wide character strings.
 *
 * @long The wrename function is used to change the name of a specified file,
 *       from the name oldname to the name newname. The destination file
 *       will be unlinked if it exists. If the file is open, the behavior
 *       of the wrename function is volume manager dependent.
 * @long The behavior when attempting to rename a directory is volume manager
 *       dependent.
 * @long A file can be renamed to another path. The behavior when renaming
 *       to another path within the same volume is volume manager dependent.
 *
 * @restriction Renaming to a path in another volume is not supported.
 *
 * @long Standards: None - This is an OSE-specific extension.
 *
 * @input oldname A pointer to a string with the file name of the file
 *                to rename.
 * @input newname A pointer to a string with the requested new file name.
 *
 * @return The function returns zero if successful, otherwise it returns
 *         @link "" "EOF" and errno is set to one of the values below to
 *         indicate the error.
 *         wrename will fail if any of the following conditions are true:
 *
 * @error EAGAIN       errno @link "errno.h" "EAGAIN".
 *                     Lost contact with the volume manager.
 * @error EOPNOTSUPP   errno @link "errno.h" "EOPNOTSUPP".
 *                     Not supported to move across volumes.
 * @error ENOENT       errno @link "errno.h" "ENOENT".
 *                     No such volume.
 * @error EACCES       errno @link "errno.h" "EACCES".
 *                     Search permission is denied for a component of the
 *                     path prefix.
 * @error EBUSY        errno @link "errno.h" "EBUSY".
 *                     The file named by the oldname argument can not be
 *                     removed because it is being used by a client.
 * @error ENAMETOOLONG errno @link "errno.h" "ENAMETOOLONG".
 *                     File name too long.
 * @error ENOTMOUNTED  errno @link "errno.h" "ENOTMOUNTED".
 *                     Not mounted.
 * @error EPERM        errno @link "errno.h" "EPERM".
 *                     Operation not permitted.
 * @error EROFS        errno @link "errno.h" "EROFS".
 *                     Read only file system.
 * @error ESTALE       errno @link "errno.h" "ESTALE".
 *                     The path points to a remote machine and the link to
 *                     that machine is no longer active.
 *
 * @seealso wcreat
 * @seealso wopen
 * @seealso rename
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To change the name of the file ose.txt to efsose.txt do the following:
 *     ren = wrename(L"ose.txt", L"efsose.txt");
 *     if (ren == 0)
 *     {
 *       // The file was successfully renamed.
 *     }
 */
int wrename(const wchar_t *oldname, const wchar_t *newname);


/**
 * @function rewind
 *
 * @include        "stdio.h"
 *
 * @declaration void
 * rewind(FILE *file);
 *
 * @brief Rewinds the position of a file stream to the beginning.
 *
 * @long The rewind function sets the file position indicator for the stream
 *       pointed to by file to the beginning of the file. It is equivalent to:
 * @long (void) @link "" "fseek"(stream, 0L, SEEK_SET)
 * @long except that the error indicator for the stream is also cleared.
 *
 * @restriction It depends on the volume manager if the function is supported.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file The pointer to the @link "" "FILE" structure.
 *
 * @return None.
 *
 * @seealso rewinddir
 * @seealso fseek
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To move the stream pointer st to the beginning of the file:
 *     FILE *st;
 *     rewind(st);
 */
void     rewind(FILE *file);


/**
 * @function scanf
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * scanf(const char *format, ...);
 *
 * @brief Reads formatted data from @link "" "stdin".
 *
 * @long The scanf function is equivalent to the function @link "" "fscanf"
 *       with the first argument @link "" "stdin".
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input format The format features (see @link "" "fscanf").
 * @input ...    Items to receive the input.
 *
 * @return The function returns the number of input items assigned if
 *         successful, otherwise it returns @link "" "EOF" and errno is
 *         set to one of the values below to indicate the error.
 *         scanf will fail if any of the following conditions are true:
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The @link "" "stdin" is not a valid file descriptor.
 * @error ESTALE errno @link "errno.h" "ESTALE".
 *               @link "" "stdin" points to a remote machine and the link
 *               to that machine is no longer active.
 *
 * @seealso fscanf
 * @seealso fprintf
 * @seealso sscanf
 * @seealso vsscanf
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To read a string of 10 characters from a file associated with the stdin,
 * // do the following:
 *     char string[10];
 *     num = scanf("%s", &string);
 *     if (num == 1)
 *     {
 *       // This call was successful.
 *     }
 */
int      scanf(const char *format, ...);

/**
 * @function setbuf
 *
 * @include        "stdio.h"
 *
 * @declaration void
 * setbuf(FILE *file, char *buffer);
 *
 * @brief Sets a stream's buffering mode, optionally with user buffer.
 *
 * @long The setbuf function sets the stream to be fully buffered. If the
 *       argument buffer is a NULL pointer, then the file will be unbuffered.
 *       Except that it returns no value, the setbuf function is equivalent
 *       to the @link "" "setvbuf" function invoked with the values @link ""
 *       "_IOFBF" for mode and @link "" "BUFSIZ" for size, or (if buffer is
 *       a NULL pointer) with the value @link "" "_IONBF" for mode.
 * @long The user application is responsible for freeing the buffer after
 *       the stream is closed.
 * @long The setbuf call can replace the @link "" "setvbuf" function
 *       call when the arguments @link "" "_IOFBF" for mode and
 *       @link "" "BUFSIZ" for size are used.
 *       See @link "" "setvbuf" for more details.
 *
 * @restriction The setbuf function call may only be used after a stream
 *              pointer has been associated with an open file and before any
 *              other operation is performed.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file   The pointer to the @link "" "FILE" structure
 * @input buffer The pointer to the supplied buffer, which must have the size
 *               @link "" "BUFSIZ".
 *
 * @return None.
 *
 * @seealso setvbuf
 * @seealso fopen
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To fully buffer the opened file ose.txt with a buffer of size BUFSIZ,
 * // do the following:
 *     char buf[BUFSIZ];
 *     FILE fp;
 *     fp = fopen("ose.txt", "r");
 *     setbuf(fp, &buf); // Always OK.
 */
void     setbuf(FILE *file, char *buffer);


/**
 * @function setvbuf
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * setvbuf(FILE *file, char *buffer, int mode, size_t bufsize);
 *
 * @brief Sets a stream's buffering mode, optionally with user buffer.
 *
 * @long The setvbuf function may be used only after the stream pointed to
 *       by file has been associated with an open file and before any other
 *       operation is performed on the stream. The argument mode determines
 *       how the stream will be buffered, as follows:
 * @long - @link "" "_IOFBF" causes input and output to be fully buffered.
 * @long - @link "" "_IOLBF" causes input and output to be line buffered.
 * @long - @link "" "_IONBF" causes input and output to be unbuffered.
 *         buffer must be NULL.
 * @long If buffer is not a NULL pointer, the area it points to is used
 *       instead of a buffer allocated by the setvbuf function. The area
 *       must be at least bufsize bytes. Any earlier buffer area is freed.
 *       The user application is responsible for freeing the buffer after
 *       the stream is closed.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file    The pointer to the @link "" "FILE" structure.
 * @input buffer  The pointer to the user supplied buffer, or NULL.
 * @input mode    The buffer mode to be used for this stream.
 * @input bufsize The size of the buffer.
 *
 * @return The function setvbuf returns zero if successful, otherwise it
 *         returns @link "" "EOF" and errno is set to one of the values
 *         below to indicate the error.
 *
 * @error EBADF  errno @link "errno.h" "EBADF".
 *               The stream argument is not a valid file descriptor.
 * @error EINVAL errno @link "errno.h" "EINVAL".
 *               The value of mode is invalid.
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso setbuf
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To fully buffer the opened file ose.txt do the following:
 *     #define BSIZ 256
 *     char buf[BSIZ];
 *     int res;
 *     FILE fp;
 *     fp = fopen("ose.txt", "r");
 *     res = setvbuf(fp, &buf, _IOFBF, BSIZ);
 *     if (res == 0)
 *     {
 *       // This call was successful.
 *     }
 */
int      setvbuf(FILE *file, char *buffer, int mode, size_t bufsize);


/**
 * @function snprintf
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * snprintf(char *buffer, size_t bufsize, const char *format, ...);
 *
 * @brief Prints formatted data to a string.
 *
 * @long The snprintf function outputs formatted data like the @link ""
 *       "fprintf" function, but to a memory buffer instead of to a @link
 *       "" "FILE". The snprintf function is identical to the function
 *       @link "" "sprintf" with the addition of the argument bufsize,
 *       which specifies the size of the buffer referred to by buffer.
 * @long Output characters beyond the bufsize-1st are discarded rather
 *       than being written to the buffer, and a null termination character
 *       is written at the end of the characters actually written into the
 *       buffer.
 * @long See @link "" "fprintf" for all details.
 *
 *
 * @long Standards:
 *       ISO/IEC 9899:1999 (E), Programming Languages - C.
 *       NOTE: This function is not part of the 1990 standard.
 *
 * @input buffer  The pointer to a buffer to receive the formatted output.
 * @input bufsize The size of the buffer.
 * @input format  The format command string.
 * @input ...     Are the items to be printed formatted.
 *
 * @return The number the characters printed excluding string terminator.
 *         If an error occur, the function return -1.
 *
 * @seealso sscanf
 * @seealso vsscanf
 * @seealso fprintf
 * @seealso eprintf
 * @seealso printf
 * @seealso vsprintf
 * @seealso vfprintf
 * @seealso default_restrictions_calling_functions
 */
int      snprintf(char *buffer, size_t bufsize, const char *format, ...);


/**
 * @function sprintf
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * sprintf(char *buffer, const char *format, ...);
 *
 * @brief Prints formatted data to a string.
 *
 * @long The sprintf function outputs formatted data like the @link "" "fprintf"
 *       function, but to a memory buffer instead of to a @link "" "FILE".
 * @long Warning: Be sure the receiving buffer is large enough!
 * @long See @link "" "fprintf" for all details.
 *
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input buffer  The pointer to a buffer to receive the formatted output.
 * @input format  The format command string.
 * @input ...     Are the items to be printed formatted.
 *
 * @return The function returns the number of characters written if
 *         successful, otherwise it returns @link "" "EOF" and errno is
 *         set to indicate the error.
 *
 * @seealso sscanf
 * @seealso vsscanf
 * @seealso fprintf
 * @seealso eprintf
 * @seealso printf
 * @seealso vsprintf
 * @seealso vfprintf
 * @seealso default_restrictions_calling_functions
 */
int      sprintf(char *buffer, const char *format, ...);


/**
 * @function sscanf
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * sscanf(const char *buffer, const char *format, ...);
 *
 * @brief Reads formatted data from a string.
 *
 * @long The sscanf function reads values like the function
 *       @link "" "fscanf" but from a string instead.
 * @long See @link "" "fscanf" for more details.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input buffer The pointer to a buffer from which the input is read.
 * @input format The format features (see @link "" "fscanf").
 * @input ...    Items to receive the input.
 *
 * @return The function returns the number of input items assigned if
 *         successful, otherwise it returns @link "" "EOF" and errno is
 *         set to indicate the error.
 *
 * @seealso fscanf
 * @seealso scanf
 * @seealso vsscanf
 * @seealso fprintf
 * @seealso sprintf
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To read up to 10 characters from a buffer, do the following:
 *     char *buf = "This is a string with at least 10 characters";
 *     char string[10+1]; // including null termination
 *     num = sscanf(buf, "%10s", &string);
 *     if (num == 1)
 *     {
 *       // The call was successful.
 *     }
 */
int      sscanf(const char *buffer, const char *format, ...);


/**
 * @function vsscanf
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * vsscanf(const char *buffer, const char *format, va_list ap);
 *
 * @brief Reads formatted data from a string.
 *
 * @long The sscanf function reads values like the function @link "" "sscanf",
 *       but instead of being called with a variable number of arguments,
 *       it is called with an argument list of the type va_list, initiated
 *       in some user functions having a variable number of arguments.
 * @long See the function @link "" "fprintf" for all formatting details.
 * @long See the function @link "" "vfprintf" for all information about va_list.
 *
 * @long See @link "" "fscanf" for more details.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input buffer The pointer to a buffer from which the input is read.
 * @input format The format features (see @link "" "fscanf").
 * @input ap     A pointer to a list of arguments of type va_list.
 *
 * @return The function returns the number of input items assigned if
 *         successful, otherwise it returns @link "" "EOF" and errno is
 *         set to indicate the error.
 *
 * @seealso fscanf
 * @seealso scanf
 * @seealso sscanf
 * @seealso fprintf
 * @seealso sprintf
 * @seealso default_restrictions_calling_functions
 *
 */
int      vsscanf(const char *buffer, const char *format, va_list ap);


/**
 * @function tmpfile
 *
 * @include        "stdio.h"
 *
 * @declaration FILE *
 * tmpfile(void);
 *
 * @brief Creates a temporary file.
 *
 * @long The tmpfile function creates and opens a temporary file using a
 *       name generated by the @link "" "tmpnam" function. The file is
 *       opened in update access mode, i.e. like in @link ""
 *       "fopen"(path, "w+").
 * @long The file is NOT auto deleted when the function @link "stdlib.h"
 *       "close" is called which is a breach of the POSIX standard.
 * @long The EFS_TMPDIR OSE environment variable must exist, either on the
 *       application process or on the application block, with the name of a
 *       temporary directory, which will become the leading part of the file
 *       path.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @return The function returns a pointer to the @link "" "FILE" object
 *         controlling the stream if successful, otherwise it returns NULL.
 *
 * @seealso tmpnam
 * @seealso open
 * @seealso fopen
 * @seealso exit
 * @seealso _exit
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // Create a file with:
 *     FILE * tmp = tmpfile();
 */
FILE *   tmpfile(void);


/**
 * @function tmpnam
 *
 * @include        "stdio.h"
 *
 * @declaration char *
 * tmpnam(char *buffer);
 *
 * @brief Creates a temporary file name.
 *
 * @long The tmpnam function creates a new file name, suitable for use to
 *       create a temporary file. Different file names are generated each
 *       time this function is called.
 * @long The EFS_TMPDIR OSE environment variable must exist, either on the
 *       application process or on the application block, with the name of
 *       a temporary directory, which will become the leading part of the
 *       file path, otherwise a system error is generated.
 *
 * @restriction When buffer is NULL, the resulting string is stored in an
 *              internal static array that can be accessed by the return
 *              value. The content of this string is stored until a
 *              subsequent call to tmpnam() or wtmpnam() function erases it.
 *              Note that when the buffer is NULL the string is NOT process
 *              unique. If process unique string is desirable the buffer
 *              must be allocated by the process.
 *
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input buffer Must be a buffer with a size of at least @link ""
 *               "L_tmpnam" bytes where the resulting file name will be
 *               stored. If buffer is NULL, the result is in an internal
 *               data area.
 *
 * @return The return value is a pointer to the buffer with the new file name.
 *
 * @seealso tmpfile
 * @seealso wtmpnam
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To create a name for a temporary file:
 *     char tmp[L_tmpnam];
 *     char *newName;
 *     newName = tmpnam(tmp);
 */
char *   tmpnam(char *buffer);


/**
 * @function wtmpnam
 *
 * @include        "stdio.h"
 *
 * @declaration wchar_t *
 * wtmpnam(wchar_t *buffer);
 *
 * @brief Creates a temporary file name.
 *
 * @long This function is equivalent to tmpnam, with the exception that
 *       it operates on wide character strings.
 *
 * @long The wtmpnam function creates a new file name, suitable for use to
 *       create a temporary file. Different file names are generated each
 *       time this function is called.
 * @long The EFS_TMPDIR OSE environment variable must exist, either on the
 *       application process or on the application block, with the name of
 *       a temporary directory, which will become the leading part of the
 *       file path, otherwise a system error is generated. Note that
 *       EFS_TMPDIR is defined as a multibyte char string, which will be
 *       converted to wchar_t counterpart when generating the temporary name.
 *
 * @restriction When buffer is NULL, the resulting string is stored in an
 *              internal static array that can be accessed by the return
 *              value. The content of this string is stored until a
 *              subsequent call to wtmpnam() function erases it. Note that
 *              when the buffer is NULL the string is NOT process unique.
 *              If process unique string is desirable the buffer must be
 *              allocated by the process.
 *
 * @long Standards: None - This is an OSE-specific extension.
 *
 * @input buffer Must be a buffer with a size of at least @link ""
 *               "L_tmpnam" wchar_t elements where the resulting file name
 *               will be stored. If buffer is NULL, the result is in an
 *               internal data area.
 *
 * @return The return value is a pointer to the buffer with the new file name.
 *
 * @seealso tmpfile
 * @seealso tmpnam
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To create a name for a temporary file:
 *     wchar_t tmp[L_tmpnam];
 *     wchar_t *newName;
 *     newName = tmpnam(tmp);
 */
wchar_t * wtmpnam(wchar_t *buffer);


/**
 * @function ungetc
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * ungetc(int c, FILE *file);
 *
 * @brief Puts back last character read from a stream.
 *
 * @long The ungetc function pushes the character specified by c (converted
 *       to an unsigned char) back onto the input stream pointed to by file.
 *       The characters pushed back will be returned by subsequent reads on
 *       that stream in the reverse order of their pushing. A successful
 *       intervening call (with the stream pointed to by file) to a file
 *       positioning function (@link "" "fseek", @link "" "fsetpos" or
 *       @link "" "rewind") discards any pushed back characters for the stream.
 * @restriction One character of push back is guaranteed. If the ungetc
 *              function is called too many times on the same data stream
 *              without an intervening read or file positioning operation on
 *              that stream, the operation may fail. If the value of the
 *              specified character equals @link "" "EOF", the operation fails
 *              and the input stream is unchanged.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input c    The specified character.
 * @input file The pointer to the @link "" "FILE" structure for the input
 *             stream to put c back into.
 *
 * @return The function returns the character pushed back after conversion
 *         if successful, otherwise it returns @link "" "EOF" and errno is
 *         set to the value below to indicate the error.
 *         ungetc will fail if the following condition is true:
 *
 * @error EBADF errno @link "errno.h" "EBADF".
 *              The stream argument is not a valid file descriptor.
 * @error EFAULT errno @link "errno.h" "EFAULT".
 *               file is either NULL or points to a struct with illegal magic.
 *
 * @seealso fflush
 * @seealso getc
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To undo a getc call do the following:
 *     char = getc(st);
 *     ch = ungetc(char, st);
 *     if (ch == char)
 *     {
 *       // Operation was successful.
 *     }
 */
int      ungetc(int c, FILE *file);


/**
 * @function vfprintf
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * vfprintf(FILE *file, const char *format, va_list arg);
 *
 * @brief Prints formatted data to a stream, with va_list.
 *
 * @long The vfprintf function outputs formatted data in the same way as
 *       the @link "" "fprintf" function, but instead of being called with
 *       a variable number of arguments, it is called with an argument
 *       list of the type va_list, initiated in some user functions having
 *       a variable number of arguments.
 * @long See the function @link "" "fprintf" for all formatting details.
 * @long The stdarg.h header (automatically included by stdio.h) defines
 *       the va_list type and the macros va_start, va_arg and va_end for
 *       traversing through the va_list.
 *       See the ISO/IEC 9899 C standard for the definition of these.
 *       Below is a summary:
 * @long - va_start(argp, lastFixedArg) initializes the list to begin with
 *         the first argument after the lastFixedArg argument in the current
 *         user defined function.
 * @long - va_arg(argp, argType) retrieves one argument with the indicated
 *         type (e.g. int, char*, ...) and prepares for the next, using the
 *         size of the given type.
 * @long - va_end(argp) resets the argp pointer to NULL, typically after all
 *         arguments have been retrieved.
 * @long See also ISO/IEC 9899 fprintf() documentation for formats and
 *         definitions.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file   The pointer to the @link "" "FILE" structure.
 * @input format The format command string (see @link "" "fprintf").
 * @input arg    A pointer to a list of arguments of type va_list.
 *
 * @return The function returns the number of characters transmitted if
 *         successful, otherwise it returns @link "" "EOF" and errno is
 *         set. See the function @link "" "fprintf" for error codes.
 *
 * @seealso fprintf
 * @seealso vprintf
 * @seealso vsprintf
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // A function printing a variable number of arguments according to the
 * // format string, in addition to a leading character string and an
 * // integer value. Errors are ignored here.
 *    int myPrintThings(char *myName, int *myNumber, char *format, ...)
 *    {
 *     va_list arg;
 *     va_start(arg, format); // va_list starts after the format argument.
 *     (void) fprintf(stderr, "My name: %s, my number: %d\n", myName, myNumber);
 *     (void) vfprintf(stderr, format, arg); // All other arguments.
 *     (void) fprintf(stderr, "That was all! Thanks!\n");
 *     va_end(arg);
 *     return (0);
 *    }
 */
int      vfprintf(FILE *file, const char *format, va_list arg);


/**
 * @function vfscanf
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * vfscanf(FILE * file, const char *format, va_list ap);
 *
 * @brief Reads formatted data from a stream, with va_list.
 *
 * @long The vfscanf function reads formatted data from a stream in the
 *       same way as the @link "" "fscanf" function, but instead of being
 *       called with a variable number of arguments, it is called with an
 *       argument list of the type va_list, initiated in some user
 *       functions having a variable number of arguments.
 * @long See the function @link "" "fprintf" for all formatting details.
 * @long See the function @link "" "vfprintf" for all information about va_list.
 * @long See also ISO/IEC 9899 fscanf() documentation for formats and
 *       definitions.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input file   The pointer to the @link "" "FILE" structure.
 * @input format The format features (see @link "" "fprintf").
 * @input ap     A pointer to a list of arguments of type va_list.
 *
 * @return The function returns the number of input items assigned if
 *         successful, otherwise it returns @link "" "EOF" and errno is
 *         set. See the function @link "" "fscanf" for error codes.
 *
 * @seealso fprintf
 * @seealso scanf
 * @seealso fread
 * @seealso fwrite
 * @seealso default_restrictions_calling_functions
 */
int      vfscanf(FILE * file, const char *format, va_list ap);

/**
 * @function vprintf
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * vprintf(const char *format, va_list arg);
 *
 * @brief Prints formatted data to @link "" "stdout", with va_list.
 *
 * @long The vprintf function is equivalent to the @link "" "vfprintf"
 *       function, but with @link "" "stdout" as the first argument.
 * @long See the function @link "" "fprintf" for all format details.
 * @long See the function @link "" "vfprintf" for all information about va_list.
 * @long See also ISO/IEC 9899 fprintf() documentation for formats and
 *       definitions.
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input format The format command string (see @link "" "fprintf").
 * @input arg    A pointer to a list of arguments of type va_list.
 *
 * @return The function returns the number of characters transmitted if
 *         successful, otherwise it returns  @link "" "EOF" and errno is set.
 *         See the function @link "" "fprintf" for error codes.
 *
 * @seealso fprintf
 * @seealso vfprintf
 * @seealso vsprintf
 * @seealso default_restrictions_calling_functions
 */
int      vprintf(const char *format, va_list arg);


/**
 * @function vsnprintf
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * vsnprintf(char *buffer, size_t bufsize, const char *format, va_list arg);
 *
 * @brief Prints formatted data to a string, with va_list.
 *
 * @long The vsnprintf function outputs formatted data like the @link ""
 *       "vfprintf" function, but to a memory buffer instead of to a
 *       @link "" "FILE". The vsnprintf function is identical to the
 *       function @link "" "vsprintf" with  the addition of the argument
 *       bufsize, which specifies the size of the buffer referred to by buffer.
 * @long Output characters beyond the bufsize-1st are discarded rather
 *       than being written to the buffer, and a null termination character
 *       is written at the end of the characters actually written into the
 *       buffer.
 * @long See the function @link "" "fprintf" for all format details.
 * @long See the function @link "" "vfprintf" for all information about va_list.
 * @long See also ISO/IEC 9899 fprintf() documentation for formats and
 *       definitions.
 *
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input buffer  The pointer to a buffer to receive the formatted output.
 * @input bufsize The size of the buffer.
 * @input format  The format command string (see @link "" "fprintf").
 * @input arg     A pointer to a list of arguments of type va_list.
 *
 * @return The number the characters printed excluding string terminator.
 *         If an error occur, the function return -1.
 *
 * @seealso fprintf
 * @seealso vfprintf
 * @seealso vprintf
 * @seealso vsprintf
 * @seealso sprintf
 * @seealso sscanf
 * @seealso default_restrictions_calling_functions
 */
int    vsnprintf(char *buffer, size_t bufsize, const char *format, va_list arg);


/**
 * @function vsprintf
 *
 * @include        "stdio.h"
 *
 * @declaration int
 * vsprintf(char *buffer, const char *format, va_list arg);
 *
 * @brief Prints formatted data to a string, with va_list.
 *
 * @long The vsprintf function outputs formatted data like the @link ""
 *       "vfprintf" function, but to a memory buffer instead of to a
 *       @link "" "FILE".
 * @long Warning: Be sure the receiving buffer is large enough!
 * @long See the function @link "" "fprintf" for all format details.
 * @long See the function @link "" "vfprintf" for all information about va_list.
 * @long See also ISO/IEC 9899 fprintf() documentation for formats and
 *       definitions.
 *
 *
 * @long Standards:
 *       ISO/IEC 9899:1990 (E), Programming Languages - C.
 *
 * @input buffer The pointer to a buffer to receive the formatted output.
 * @input format The format command string (see @link "" "fprintf").
 * @input arg    A pointer to a list of arguments of type va_list.
 *
 * @return The function returns the number of characters transmitted if
 *         successful, otherwise it returns @link "" "EOF" and errno is set.
 *         See the function @link "" "fprintf" for error codes.
 *
 * @seealso fprintf
 * @seealso vfprintf
 * @seealso vprintf
 * @seealso vsnprintf
 * @seealso sprintf
 * @seealso sscanf
 * @seealso vsscanf
 * @seealso default_restrictions_calling_functions
 */
int      vsprintf(char *buffer, const char *format, va_list arg);


#ifdef softOSE
#define ZZMK_INLINE_FUNCTIONS
#include "sfk/sfk_stdio.h"
#endif

#ifdef __cplusplus
}
#if !defined (NO_NAME_STD) && defined (__ARMCC_VERSION)
}  /* namespace std */
using namespace ::std;
#endif /* NO_NAME_STD */
#endif /* __cplusplus */


/* End #ifndef _STDIO_H */
#endif
