/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2011 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @toc Standard_C:
 *
 * @file sys/stat.h
 *
 * @brief EFS POSIX stat interface.
 *
 *
 */

#ifndef _SYS_STAT_H
#define _SYS_STAT_H
#define OSE_SYS_STAT_H

#include "osarch.h"
#include "sys/types.h"
#include "sys/time.h"

#ifdef softOSE
#define ZZMK_NAME_MANGLING
#include "sfk/sys/sfk_stat.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @macro S_ISDIR
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_ISDIR(mode)       ((mode) & S_IFDIR)
 *
 * @brief Check if the mode of the stat object is a directory.
 *
 * @field mode The mode.
 *
 * @long  Macro for testing object type in the @link "" "stat" structure.
 */
#define S_ISDIR(mode)       ((mode) & S_IFDIR)

/**
 * @macro S_ISCHR
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_ISCHR(mode)       ((mode) & S_IFCHR)
 *
 * @brief Check if the mode of the stat object is a character file (terminal).
 *
 * @field mode The mode.
 *
 * @long  Macro for testing object type in the @link "" "stat" structure.
 */
#define S_ISCHR(mode)       ((mode) & S_IFCHR)

/**
 * @macro S_ISBLK
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_ISBLK(mode)       ((mode) & S_IFBLK)
 *
 * @brief Check if the mode of the stat object is a block file.
 *
 * @field mode The mode.
 *
 * @long  Macro for testing object type in the @link "" "stat" structure.
 */
#define S_ISBLK(mode)       ((mode) & S_IFBLK)

/**
 * @macro S_ISREG
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_ISREG(mode)       ((mode) & S_IFREG)
 *
 * @brief Check if the mode of the stat object is a regular file.
 *
 * @field mode The mode.
 *
 * @long  Macro for testing object type in the @link "" "stat" structure.
 */
#define S_ISREG(mode)       ((mode) & S_IFREG)

/**
 * @macro S_ISFIFO
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_ISFIFO(mode)      ((mode) & S_IFFIFO)
 *
 * @brief Check if the mode of the stat object is a FIFO (pipe).
 *
 * @field mode The mode.
 *
 * @long  Macro for testing object type in the @link "" "stat" structure.
 */
#define S_ISFIFO(mode)      ((mode) & S_IFFIFO)

/**
 * @macro S_ISLNK
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_ISLNK(mode)       ((mode) & S_IFLNK)
 *
 * @brief Check if the mode of the stat object is a link.
 *
 * @field mode The mode.
 *
 * @long  Macro for testing object type in the @link "" "stat" structure.
 */
#define S_ISLNK(mode)       ((mode) & S_IFLNK)

/**
 * @macro S_ISSOCK
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_ISSOCK(mode)      ((mode) & S_IFSOCK)
 *
 * @brief Check if the mode of the stat object is a socket.
 *
 * @field mode The mode.
 *
 * @long  Macro for testing object type in the @link "" "stat" structure.
 */
#define S_ISSOCK(mode)      ((mode) & S_IFSOCK)

/**
 * @macro S_TYPEISMQ
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_TYPEISMQ(a)    0
 *
 * @brief Check if the mode of the stat object is a message queue.
 *
 * @long  Macro for testing object type in the @link "" "stat" structure.
 * @long  NOTE: Required by POSIX, not implemented by OSE.
 *
 * @field a The mode.
 */
#define S_TYPEISMQ(a)    0

/**
 * @macro S_TYPEISSEM
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_TYPEISSEM(a)   0
 *
 * @brief Check if the mode of the stat object is a POSIX semaphore.
 *
 * @long  Macro for testing object type in the @link "" "stat" structure.
 * @long  NOTE: Required by POSIX, not implemented by OSE.
 *
 * @field a The mode.
 */
#define S_TYPEISSEM(a)   0

/**
 * @macro S_TYPEISSHM
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_TYPEISSHM(a)   0
 *
 * @brief Check if the mode of the stat object is shared memory.
 *
 * @long  Macro for testing object type in the @link "" "stat" structure.
 * @long  NOTE: Required by POSIX, not implemented by OSE.
 *
 * @field a The mode.
 *
 */
#define S_TYPEISSHM(a)   0


/**
 * @macro S_IFDIR
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IFDIR       0x00004000
 *
 * @brief Object is a directory.
 *
 * @long  Bitvalue for object types.
 */
#define S_IFDIR       0x00004000

/**
 * @macro S_IFCHR
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IFCHR       0x00008000
 *
 * @brief Object is a character file (terminal).
 *
 * @long  Bitvalue for object types.
 */
#define S_IFCHR       0x00008000

/**
 * @macro S_IFBLK
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IFBLK       0x00010000
 *
 * @brief Object is a block file.
 *
 * @long  Bitvalue for object types.
 */
#define S_IFBLK       0x00010000

/**
 * @macro S_IFREG
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IFREG       0x00020000
 *
 * @brief Object is a regular file.
 *
 * @long  Bitvalue for object types.
 */
#define S_IFREG       0x00020000

/**
 * @macro S_IFFIFO
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IFFIFO      0x00040000
 *
 * @brief Object is a FIFO (pipe).
 *
 * @long  Bitvalue for object types.
 */
#define S_IFFIFO      0x00040000

/**
 * @macro S_IFLNK
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IFLNK       0x00080000
 *
 * @brief Object is a link.
 *
 * @long  Bitvalue for object types.
 */
#define S_IFLNK       0x00080000

/**
 * @macro S_IFSOCK
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IFSOCK      0x00100000
 *
 * @brief Object is a socket.
 *
 * @long  Bitvalue for object types.
 */
#define S_IFSOCK      0x00100000

/**
 * @macro S_IFMT
 *
 * @include        "sys/stat.h"
 *
 * @brief Bitmask of type information in st_mode
 *
 * @long  Bitmask for object types returned in stat.
 */
#define S_IFMT       (S_IFDIR |\
                      S_IFCHR |\
		      S_IFBLK |\
		      S_IFREG |\
		      S_IFFIFO |\
		      S_IFLNK |\
		      S_IFSOCK)


/* Obsolete modes are not documented. */
#ifdef _STATH_OBSOLETE
#define S_ODIR        S_IFDIR /* Obsolete, for backwards compatibility only. */
#define S_OLINK       S_IFLNK /* Obsolete, for backwards compatibility only. */
#endif

/**
 * @macro S_IARCH
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IARCH       0x00000200
 *
 * @brief Object has been modified.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IARCH       0x00000200

/**
 * @macro S_IRUSR
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IRUSR       0x00000100
 *
 * @brief Read access for owner.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IRUSR       0x00000100

/**
 * @macro S_IWUSR
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IWUSR       0x00000080
 *
 * @brief Write access for owner.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IWUSR       0x00000080

/**
 * @macro S_IXUSR
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IXUSR       0x00000040
 *
 * @brief Execute/search access for owner.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IXUSR       0x00000040

/**
 * @macro S_IRGRP
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IRGRP       0x00000020
 *
 * @brief Read access for group.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IRGRP       0x00000020

/**
 * @macro S_IWGRP
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IWGRP       0x00000010
 *
 * @brief Write access for group.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IWGRP       0x00000010

/**
 * @macro S_IXGRP
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IXGRP       0x00000008
 *
 * @brief Execute/search access for group.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IXGRP       0x00000008

/**
 * @macro S_IROTH
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IROTH       0x00000004
 *
 * @brief Read access for other.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IROTH       0x00000004

/**
 * @macro S_IWOTH
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IWOTH       0x00000002
 *
 * @brief Write access for other.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IWOTH       0x00000002

/**
 * @macro S_IXOTH
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IXOTH       0x00000001
 *
 * @brief Execute/search access for other.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IXOTH       0x00000001

/**
 * @macro S_IRWXU
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IRWXU       (S_IRUSR | S_IWUSR | S_IXUSR)
 *
 * @brief All access permissions for owner.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IRWXU       (S_IRUSR | S_IWUSR | S_IXUSR)

/**
 * @macro S_IRWXG
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IRWXG       (S_IRGRP | S_IWGRP | S_IXGRP)
 *
 * @brief All access permissions for group.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IRWXG       (S_IRGRP | S_IWGRP | S_IXGRP)

/**
 * @macro S_IRWXO
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IRWXO       (S_IROTH | S_IWOTH | S_IXOTH)
 *
 * @brief All access permissions for other.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IRWXO       (S_IROTH | S_IWOTH | S_IXOTH)

/**
 * @macro S_IREAD
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IREAD       (S_IRUSR | S_IRGRP | S_IROTH)
 *
 * @brief Read access for owner, group and other.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IREAD       (S_IRUSR | S_IRGRP | S_IROTH)

/**
 * @macro S_IWRITE
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IWRITE      (S_IWUSR | S_IWGRP | S_IWOTH)
 *
 * @brief Write access for owner, group and other.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IWRITE      (S_IWUSR | S_IWGRP | S_IWOTH)

/**
 * @macro S_IEXEC
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_IEXEC       (S_IXUSR | S_IXGRP | S_IXOTH)
 *
 * @brief Execution access for owner, group and other.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_IEXEC       (S_IXUSR | S_IXGRP | S_IXOTH)

/* Obsolete, not documented. */
#ifdef _STATH_OBSOLETE
#define S_IXECUTE     S_IEXEC /* Obsolete, for backwards compatibility only. */
#endif

/**
 * @macro S_ISUID
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_ISUID 0x10000000
 *
 * @brief Set effective user ID on execution as listed in inode.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_ISUID 0x10000000

/**
 * @macro S_ISGID
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_ISGID 0x20000000
 *
 * @brief Set effective group ID on execution as listed in inode.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_ISGID 0x20000000

/**
 * @macro S_ISVTX
 *
 * @include        "sys/stat.h"
 *
 * @declaration #define S_ISVTX 0x40000000
 *
 * @brief Sticky bit; for directories, only superuser or owner can rename
 *        or unlink files. Implementation specific behavior on files
 *        with bit set.
 *
 * @long  Bitvalue for object access permissions.
 */
#define S_ISVTX 0x40000000



/* OSE extension attributes are not documented. */

#define S_IHIDDEN   0x00000400  /* The object name should be hidden */
#define S_IBOOTFILE 0x00000800  /* The object is designated as a boot file. */
#define S_ISTARTUP  S_IBOOTFILE /* The object is designated as a startup file.
                                 */
#define S_IUSER0    0x01000000  /* User application-defined attribute bit 0. */
#define S_IUSER1    0x02000000  /* User application-defined attribute bit 1. */
#define S_IUSER2    0x04000000  /* User application-defined attribute bit 2. */
#define S_IUSER3    0x08000000  /* User application-defined attribute bit 3. */



/**
 * @type stat
 *
 * @include        "sys/stat.h"
 *
 * @brief Structure filled in by @link "" "fstat" and @link "" "stat" functions.
 *
 * @long The structure describes the current state of a file system object.
 *
 * @long The time fields are used differently by different volume managers.
 *       See the "OSE Core Extensions User's Guide" for information about the
 *       different volume managers.
 *
 *
 * @field st_mode    Object type and access mode.
 * @field st_ino     Inode number or similar unique ID for object. The numbers
 *                   are unique within the scope of one volume.
 *                   The value 0 indicates that the volume can not provide a
 *                   unique identity for the object, or that the inode number
 *                   is a 64-bit number. This is a deviation from POSIX
 *                   standard. The POSIX standard states that EOVERFLOW
 *                   should be returned if the inode number can not be
 *                   represented correctly.
 * @field st_dev     Device number, uniquely identifying the volume.
 *                   The value 0 will be used if not possible to give a unique
 *                   identity for the volume.
 * @field st_nlink   Number of links to object (if implemented).
 * @field st_uid     User ID. of object owner.
 * @field st_gid     Group ID. of object owner.
 * @field st_size    Size in bytes if the size can be represented correctly.
 *                   Otherwise EOVERFLOW is returned.
 * @field st_atime   Time of latest access (if implemented and enabled
 *                   in the volume manager).
 * @field st_mtime   Time of latest data modification. A volume manager may
 *                   optionally update this field also at certain status
 *                   changes.
 * @field st_ctime   Time of creation/latest status change, i.e. POSIX ctime
 *                   (if implemented in the volume manager).
 *                   Example: VFAT only updates it at creation time.
 */
struct stat
{
   mode_t st_mode;
   ino_t st_ino;
   dev_t st_dev;
   nlink_t st_nlink;
   uid_t st_uid;
   gid_t st_gid;
   off_t st_size;
   time_t st_atime;
   time_t st_mtime;
   time_t st_ctime;
};

/**
 * @type stat64
 *
 * @include        "sys/stat.h"
 *
 * @brief Structure filled in by @link "" "stat64" and
 *        @link "dirent.h" "readdir_stat64" functions.
 *
 * @long The structure describes the current state of a file system object.
 *
 * @long The st_valid field indicates what other fields of the struct contain
 *       valid values. A set bit indicates that the corresponding fields are
 *       valid; a cleared bit indicates that the corresponding fields contain
 *       undefined values. Note that this design allows future versions of the
 *       stat64 struct to contain more fields, without breaking backwards
 *       compatibility.
 *
 * @long The time fields are used differently by different volume managers.
 *       See the "OSE Core Extensions User's Guide" for information about the
 *       different volume managers.
 *
 * @field st_valid       Mask of valid fields, using
 *                       @link "" "StValidFlags" values.
 * @field st_mode        Object type and access mode.
 * @field st_ino         Inode number or similar unique ID for object. The
 *                       numbers are unique within the scope of one volume.
 *                       The value 0 indicates that the volume can not provide a
 *                       unique identity for the object.
 * @field st_dev         Device number, uniquely identifying the volume.
 *                       The value 0 will be used if not possible to give a
 *                       unique identity for the volume.
 * @field st_nlink       Number of links to object.
 * @field st_uid         User ID of object owner.
 * @field st_gid         Group ID of object owner.
 * @field st_size        Size in bytes.
 * @field st_media_bytes Reserved for future use.
 * @field st_atime       Time of latest access (if implemented and enabled
 *                       in the volume manager).
 * @field st_atimensec   Fractional time of latest access, in nanoseconds.
 * @field st_mtime       Time of latest data modification. A volume manager may
 *                       optionally update this field also at certain status
 *                       changes.
 * @field st_mtimensec   Fractional time of latest modification, in nanoseconds.
 * @field st_ctime       Time of creation/latest status change, i.e. POSIX ctime
 *                       (if implemented in the volume manager).
 *                       Example: VFAT only updates it at creation time.
 * @field st_ctimensec   Fractional time of last status change, in nanoseconds.
 * @field st_blksize     Reserved for future use.
 *
 * @long Standards:
 *       None
 *
 * @long Conformance:
 *       None
 *
 * @restriction In standard OSE, the stat64 structure only applies to
 *        the readdir_stat64 function.
 */
struct stat64
{
   uint32_t st_valid;
   mode_t st_mode;
   ino64_t st_ino;
   dev_t st_dev;
   nlink_t st_nlink;
   uid_t st_uid;
   gid_t st_gid;
   off64_t st_size;
   off64_t st_media_bytes;
   time_t st_atime;
   uint32_t st_atimensec;
   time_t st_mtime;
   uint32_t st_mtimensec;
   time_t st_ctime;
   uint32_t st_ctimensec;
   blksize_t st_blksize;
};

/**
 * @type    StValidFlags
 *
 * @brief   struct stat64 attribute groups select masks
 *
 * @long    This type contains a number of constants that can be OR'ed
 *          together to form a file attribute groups select mask.  @break
 *
 *          These masks are used in struct stat64 and some related
 *          function calls.
 *
 * @restriction This ENUM is not intended to be used as a type, only to define
 *              constants. It is unsafe to use ENUM as a type in public
 *              interfaces, while ENUM constants are safe.
 *
 * @field ST_VALID_INO         Selects the ino field in struct stat64.
 * @field ST_VALID_MODE        Selects the mode field in struct stat64.
 * @field ST_VALID_DEV         Selects the dev field in struct stat64.
 * @field ST_VALID_UID         Selects the uid field in struct stat64.
 * @field ST_VALID_GID         Selects the gid field in struct stat64.
 * @field ST_VALID_NLINK       Selects the nlink field in struct stat64.
 * @field ST_VALID_SIZE        Selects the size field in struct stat64.
 * @field ST_VALID_MEDIA_BYTES Selects the media_bytes field in struct stat64.
 * @field ST_VALID_ATIME       Selects the atime fields in struct stat64.
 * @field ST_VALID_MTIME       Selects the mtime fields in struct stat64.
 * @field ST_VALID_CTIME       Selects the ctime fields in struct stat64.
 * @field ST_VALID_BLKSIZE     Selects the blksize field in struct stat64.
 * @field ST_VALID_ALL_TIME    Selects all time fields in struct stat64.
 * @field ST_VALID_ALL         Selects all fields in struct stat64.
 */
enum StValidFlags
{
   ST_VALID_INO         = (1 << 0 ),
   ST_VALID_MODE        = (1 << 1 ),
   ST_VALID_DEV         = (1 << 2 ),
   ST_VALID_UID         = (1 << 3 ),
   ST_VALID_GID         = (1 << 4 ),
   ST_VALID_NLINK       = (1 << 5 ),
   ST_VALID_SIZE        = (1 << 6 ),
   ST_VALID_MEDIA_BYTES = (1 << 7 ),
   ST_VALID_ATIME       = (1 << 8 ),
   ST_VALID_MTIME       = (1 << 9 ),
   ST_VALID_CTIME       = (1 << 10),
   ST_VALID_BLKSIZE     = (1 << 11),

   ST_VALID_ALL_TIME    = (ST_VALID_ATIME | ST_VALID_MTIME | ST_VALID_CTIME),

   ST_VALID_ALL         = (ST_VALID_INO |
                           ST_VALID_MODE |
                           ST_VALID_DEV |
                           ST_VALID_UID |
                           ST_VALID_GID |
                           ST_VALID_NLINK |
                           ST_VALID_SIZE |
                           ST_VALID_MEDIA_BYTES |
                           ST_VALID_ALL_TIME |
                           ST_VALID_BLKSIZE)
};


/**
 * @function chmod
 *
 * @include        "sys/stat.h"
 *
 * @declaration int
 * chmod(const char *pathname, mode_t attrs);
 *
 * @brief Change the permission attributes for a directory or a file.
 *
 * @long The chmod function changes the access permission attributes,
 *       the mode, of the file or directory specified by path to
 *       the new values in attrs.
 * @long attrs can take the following values:
 * @long - @link "sys/stat.h" "S_IARCH": Object has been modified.
 * @long - @link "sys/stat.h" "S_IRUSR": Read permission bit for file owner.
 * @long - @link "sys/stat.h" "S_IWUSR": Write permission bit for file owner.
 * @long - @link "sys/stat.h" "S_IXUSR": Execute (search) permission bit
 *         for the file owner.
 * @long - @link "sys/stat.h" "S_IRGRP": Read permission bit for
 *         the file group.
 * @long - @link "sys/stat.h" "S_IWGRP": Write permission bit for
 *         the file group.
 * @long - @link "sys/stat.h" "S_IXGRP": Execute(search) permission bit
 *         for the file group.
 * @long - @link "sys/stat.h" "S_IROTH": Read permission bit for others.
 * @long - @link "sys/stat.h" "S_IWOTH": Write permission bit for others.
 * @long - @link "sys/stat.h" "S_IXOTH": Execute(search) permission bit
 *         for others.
 * @long In addition the following combined values can be used in attrs:
 * @long - @link "sys/stat.h" "S_IRWXU": User read/write/search(execute).
 * @long - @link "sys/stat.h" "S_IRWXG": Group read/write/search(execute).
 * @long - @link "sys/stat.h" "S_IRWXO": Others read/write/search(execute).
 * @long - @link "sys/stat.h" "S_IREAD": Read for user, group and others.
 * @long - @link "sys/stat.h" "S_IWRITE": Read for user, group and others.
 * @long - @link "sys/stat.h" "S_IEXEC": Execute(search) for user,
 *         group and others.
 *
 * @restriction Permission classes (owner/group/others) are often not
 *              implemented by the volume manager.
 *
 * @long Standards:
 *       Portable Operating System Interface (POSIX.1).
 * @long Conformance:
 *       ISO/IEC 9945-1:1990 (IEEE 1003.1-1990).
 *
 * @input pathname Specifies the file or directory whose mode will be changed.
 * @input attrs    Specifies the new permission attributes.
 *
 * @return The function returns zero if successful,
 *         otherwise it returns -1 and errno is
 *         set to one of the values below to indicate the error.
 *         chmod will fail and the mode will be unchanged if one or more
 *         of the following conditions are true:
 *
 * @error EACCES       errno @link "errno.h" "EACCES".
 *                     Search permission is denied for any component
 *                     of the path name.
 * @error ENAMETOOLONG errno @link "errno.h" "ENAMETOOLONG".
 *                     The length of the path argument exceeds the maximum
 *                     allowed on the volume.
 * @error ENODEV       errno @link "errno.h" "ENODEV".
 *                     No such device.
 * @error ENOENT       errno @link "errno.h" "ENOENT".
 *                     The named directory does not exist.
 * @error ENOTDIR      errno @link "errno.h" "ENOTDIR".
 *                     A component of the path name is not a directory.
 * @error ENOTMOUNTED  errno @link "errno.h" "ENOTMOUNTED".
 *                     Not mounted.
 * @error EPERM        errno @link "errno.h" "EPERM".
 *                     Operation not permitted.
 * @error EROFS        errno @link "errno.h" "EROFS".
 *                     The named file resides on a read-only file system.
 * @error ESTALE       errno @link "errno.h" "ESTALE".
 *                     The path points to a remote machine and the link
 *                     to that machine is no longer active.
 *
 * @seealso access
 * @seealso open
 * @seealso wchmod
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To change the mode of the file ose.txt to read-only:
 * int ok;
 *
 * ok = chmod("ose.txt", S_IRUSR | S_IRGRP | S_IROTH);
 * if (ok == 0)
 * {
 *    // The permission mode for ose.txt was changed.
 * }
 */

/*
 * chmod should be in <sys/stat.h>, but for backwards compatibility
 * reasons it is also declared in unistd.h.
 */
int chmod(const char *pathname, mode_t attrs);


/**
 * @function wchmod
 *
 * @include        "sys/stat.h"
 *
 * @declaration int
 * wchmod(const wchar_t *pathname, mode_t attrs);
 *
 * @brief Change the permission attributes for a directory or a file.
 *
 * @long This function is equivalent to chmod, with the exception that
 *       the pathname is specified as a wide character string.
 *
 * @long The wchmod function changes the access permission attributes,
 *       the mode, of the file or directory specified by path
 *       to the new values in attrs.
 *
 * @long attrs can take the following values:
 * @long - @link "sys/stat.h" "S_IARCH": Object has been modified.
 * @long - @link "sys/stat.h" "S_IRUSR": Read permission bit for file owner.
 * @long - @link "sys/stat.h" "S_IWUSR": Write permission bit for file owner.
 * @long - @link "sys/stat.h" "S_IXUSR": Execute (search) permission bit
 *         for the file owner.
 * @long - @link "sys/stat.h" "S_IRGRP": Read permission bit for
 *         the file group.
 * @long - @link "sys/stat.h" "S_IWGRP": Write permission bit for
 *         the file group.
 * @long - @link "sys/stat.h" "S_IXGRP": Execute(search) permission bit
 *         for the file group.
 * @long - @link "sys/stat.h" "S_IROTH": Read permission bit for others.
 * @long - @link "sys/stat.h" "S_IWOTH": Write permission bit for others.
 * @long - @link "sys/stat.h" "S_IXOTH": Execute(search) permission bit
 *         for others.
 * @long In addition the following combined values can be used in attrs:
 * @long - @link "sys/stat.h" "S_IRWXU": User read/write/search(execute).
 * @long - @link "sys/stat.h" "S_IRWXG": Group read/write/search(execute).
 * @long - @link "sys/stat.h" "S_IRWXO": Others read/write/search(execute).
 * @long - @link "sys/stat.h" "S_IREAD": Read for user, group and others.
 * @long - @link "sys/stat.h" "S_IWRITE": Read for user, group and others.
 * @long - @link "sys/stat.h" "S_IEXEC": Execute(search) for user,
 *         group and others.
 *
 * @restriction Permission classes (owner/group/others) are often not
 *              implemented by the volume manager.
 *
 * @long Standards:
 *       None - This is an OSE-specific extension.
 * @long Conformance:
 *       None - This is an OSE-specific extension.
 *
 * @input pathname Specifies the file or directory whose mode will be changed.
 * @input attrs    Specifies the new permission attributes.
 *
 * @return The function returns zero if successful,
 *         otherwise it returns -1 and errno is
 *         set to one of the values below to indicate the error.
 *         wchmod will fail and the mode will be unchanged if one or more
 *         of the following conditions are true:
 *
 * @error EACCES       errno @link "errno.h" "EACCES".
 *                     Search permission is denied for any component
 *                     of the path name.
 * @error ENAMETOOLONG errno @link "errno.h" "ENAMETOOLONG".
 *                     The length of the path argument exceeds the maximum
 *                     allowed on the volume.
 * @error ENODEV       errno @link "errno.h" "ENODEV".
 *                     No such device.
 * @error ENOENT       errno @link "errno.h" "ENOENT".
 *                     The named directory does not exist.
 * @error ENOTDIR      errno @link "errno.h" "ENOTDIR".
 *                     A component of the path name is not a directory.
 * @error ENOTMOUNTED  errno @link "errno.h" "ENOTMOUNTED".
 *                     Not mounted.
 * @error EPERM        errno @link "errno.h" "EPERM".
 *                     Operation not permitted.
 * @error EROFS        errno @link "errno.h" "EROFS".
 *                     The named file resides on a read-only file system.
 * @error ESTALE       errno @link "errno.h" "ESTALE".
 *                     The path points to a remote machine and the link
 *                     to that machine is no longer active.
 *
 * @seealso waccess
 * @seealso wopen
 * @seealso chmod
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To change the mode of the file ose.txt to read-only:
 * int ok;
 *
 * ok = wchmod(L"ose.txt", S_IRUSR | S_IRGRP | S_IROTH);
 * if (ok == 0)
 * {
 *    // The permission mode for ose.txt was changed.
 * }
 */
int wchmod(const wchar_t *pathname, mode_t attrs);


/**
 * @function mkdir
 *
 * @include        "sys/stat.h"
 *
 * @declaration int
 * mkdir(const char *pathname, mode_t attrs);
 *
 * @brief Creates a directory.
 *
 * @long The mkdir function creates a new empty directory with the name path.
 *       The file permission bits of the new directory are initialized from
 *       attrs. The file permission bits of the attrs argument are modified by
 *       the file creation mask (@link "" "umask") of the client.
 * @long attrs can take the following values:
 * @long - @link "sys/stat.h" "S_IARCH": Object has been modified.
 * @long - @link "sys/stat.h" "S_IRUSR": Read access for owner.
 * @long - @link "sys/stat.h" "S_IWUSR": Write access for owner.
 * @long - @link "sys/stat.h" "S_IXUSR": Execute/search access for owner.
 * @long - @link "sys/stat.h" "S_IRGRP": Read access for group.
 * @long - @link "sys/stat.h" "S_IWGRP": Write access for group.
 * @long - @link "sys/stat.h" "S_IXGRP": Execute/search access for group.
 * @long - @link "sys/stat.h" "S_IROTH": Read access for other.
 * @long - @link "sys/stat.h" "S_IWOTH": Write access for other.
 * @long - @link "sys/stat.h" "S_IXOTH": Execute/search access for other.
 * @long - @link "sys/stat.h" "S_IRWXU": Read/write execute for user.
 * @long - @link "sys/stat.h" "S_IRWXG": Read/write execute for group.
 * @long - @link "sys/stat.h" "S_IRWXO": Read/write execute for other.
 * @long - @link "sys/stat.h" "S_IREAD": Read for other.
 * @long - @link "sys/stat.h" "S_IWRITE": Write for all.
 * @long - @link "sys/stat.h" "S_IEXEC": Execute for all.
 *
 * @restriction Permission classes (owner/group/others) are often not
 *              implemented by the volume manager.
 *
 * @long Standards:
 *       Portable Operating System Interface (POSIX.1).
 * @long Conformance:
 *       ISO/IEC 9945-1:1990 (IEEE 1003.1-1990).
 *
 * @input pathname A pointer to a string with the path name.
 * @input attrs    Contains the directory permission attribute mode.
 *
 * @return The function returns zero if successful, otherwise it returns -1 and
 *         errno is set to one of the values below to indicate the error.
 *         mkdir will fail if one or more of the following conditions are true:
 *
 * @error EACCES       errno @link "errno.h" "EACCES".
 *                     Search permission is denied for a component of the path
 *                     prefix, or write permission is denied on the parent
 *                     directory of the directory to be created.
 * @error EEXIST       errno @link "errno.h" "EEXIST".
 *                     The named file exists.
 * @error EMLINK       errno @link "errno.h" "EMLINK".
 *                     The link count of the parent directory exceeds maximum.
 * @error ENAMETOOLONG errno @link "errno.h" "ENAMETOOLONG".
 *                     The length of the path argument exceeds the maximum
 *                     allowed on the volume.
 * @error ENODEV       errno @link "errno.h" "ENODEV".
 *                     No such device.
 * @error ENOENT       errno @link "errno.h" "ENOENT".
 *                     A component of the path prefix does not exist, or the
 *                     path argument points to an empty string.
 * @error ENOSPC       errno @link "errno.h" "ENOSPC".
 *                     The file system does contain enough space to hold the
 *                     contents of the new directory or to extend the parent
 *                     directory of the new directory.
 * @error ENOTDIR      errno @link "errno.h" "ENOTDIR".
 *                     A component of the pathname prefix is not a directory.
 * @error ENOTMOUNTED  errno @link "errno.h" "ENOTMOUNTED".
 *                     Not mounted.
 * @error EROFS        errno @link "errno.h" "EROFS".
 *                     The requested link requires writing in a directory on a
 *                     read-only file system.
 * @error ESTALE       errno @link "errno.h" "ESTALE".
 *                     The path points to a remote machine and the link to that
 *                     machine is no longer active.
 * @error ENOTSUP      errno @link "errno.h" "ENOTSUP".
 *                     The creation of the new directory is not supported by
 *                     the target filesystem.
 *
 * @seealso wmkdir
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To create a new directory 'new' do the following:
 * // Store a name in new and mode bits in mode...
 * int ok;
 *
 * ok = mkdir(new, mode);
 * if (ok == 0)
 * {
 *    // The new directory has been created.
 * }
 */
int mkdir(const char *pathname, mode_t attrs);


/**
 * @function wmkdir
 *
 * @include        "sys/stat.h"
 *
 * @declaration int
 * wmkdir(const wchar_t *pathname, mode_t attrs);
 *
 * @brief Creates a directory.
 *
 * @long This function is equivalent to mkdir, with the exception that
 *       the pathname is specified as a wide character string.
 *
 * @long The wmkdir function creates a new empty directory with the name path.
 *       The file permission bits of the new directory are initialized from
 *       attrs. The file permission bits of the attrs argument are modified by
 *       the file creation mask (@link "" "umask") of the client.
 *
 * @long attrs can take the following values:
 * @long - @link "sys/stat.h" "S_IARCH": Object has been modified.
 * @long - @link "sys/stat.h" "S_IRUSR": Read access for owner.
 * @long - @link "sys/stat.h" "S_IWUSR": Write access for owner.
 * @long - @link "sys/stat.h" "S_IXUSR": Execute/search access for owner.
 * @long - @link "sys/stat.h" "S_IRGRP": Read access for group.
 * @long - @link "sys/stat.h" "S_IWGRP": Write access for group.
 * @long - @link "sys/stat.h" "S_IXGRP": Execute/search access for group.
 * @long - @link "sys/stat.h" "S_IROTH": Read access for other.
 * @long - @link "sys/stat.h" "S_IWOTH": Write access for other.
 * @long - @link "sys/stat.h" "S_IXOTH": Execute/search access for other.
 * @long - @link "sys/stat.h" "S_IRWXU": Read/write execute for user.
 * @long - @link "sys/stat.h" "S_IRWXG": Read/write execute for group.
 * @long - @link "sys/stat.h" "S_IRWXO": Read/write execute for other.
 * @long - @link "sys/stat.h" "S_IREAD": Read for other.
 * @long - @link "sys/stat.h" "S_IWRITE": Write for all.
 * @long - @link "sys/stat.h" "S_IEXEC": Execute for all.
 *
 * @restriction Permission classes (owner/group/others) are often not
 *              implemented by the volume manager.
 *
 * @long Standards:
 *       None - This is an OSE-specific extension.
 * @long Conformance:
 *       None - This is an OSE-specific extension.
 *
 * @input pathname A pointer to a UTF-16 string with the path name.
 * @input attrs    Contains the directory permission attribute mode.
 *
 * @return The function returns zero if successful, otherwise it returns -1 and
 *         errno is set to one of the values below to indicate the error.
 *         wmkdir will fail if one or more of the following conditions are
 *         true:
 *
 * @error EACCES       errno @link "errno.h" "EACCES".
 *                     Search permission is denied for a component of the path
 *                     prefix, or write permission is denied on the parent
 *                     directory of the directory to be created.
 * @error EEXIST       errno @link "errno.h" "EEXIST".
 *                     The named file exists.
 * @error EMLINK       errno @link "errno.h" "EMLINK".
 *                     The link count of the parent directory exceeds maximum.
 * @error ENAMETOOLONG errno @link "errno.h" "ENAMETOOLONG".
 *                     The length of the path argument exceeds the maximum
 *                     allowed on the volume.
 * @error ENODEV       errno @link "errno.h" "ENODEV".
 *                     No such device.
 * @error ENOENT       errno @link "errno.h" "ENOENT".
 *                     A component of the path prefix does not exist, or the
 *                     path argument points to an empty string.
 * @error ENOSPC       errno @link "errno.h" "ENOSPC".
 *                     The file system does contain enough space to hold the
 *                     contents of the new directory or to extend the parent
 *                     directory of the new directory.
 * @error ENOTDIR      errno @link "errno.h" "ENOTDIR".
 *                     A component of the pathname prefix is not a directory.
 * @error ENOTMOUNTED  errno @link "errno.h" "ENOTMOUNTED".
 *                     Not mounted.
 * @error EROFS        errno @link "errno.h" "EROFS".
 *                     The requested link requires writing in a directory on a
 *                     read-only file system.
 * @error ESTALE       errno @link "errno.h" "ESTALE".
 *                     The path points to a remote machine and the link to that
 *                     machine is no longer active.
 *
 * @seealso mkdir
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To create a new directory 'new' do the following:
 * // Store a name in new and mode bits in mode...
 * int ok;
 *
 * ok = wmkdir(new, mode);
 * if (ok == 0)
 * {
 *    // The new directory has been created.
 * }
 */
int wmkdir(const wchar_t *pathname, mode_t attrs);


/**
 * @function fstat
 *
 * @include        "sys/stat.h"
 *
 * @declaration int
 * fstat(int fd, struct stat *buf);
 *
 * @brief Get file status.
 *
 * @long Return information about the specific file or directory of fd.
 *
 * @long Standards:
 *       Portable Operating System Interface (POSIX.1).
 *
 * @long Conformance:
 *       ISO/IEC 9945-1:1990 (IEEE 1003.1-1990).
 *
 * @input fd         The file descriptor of the file to get information about.
 * @input buf        A pointer to a @link "" "stat" structure buffer to hold
 *                   the result of fstat.
 *
 * @return Returns zero if the call was successful. If fstat fails,
 *         it shall return -1 and set
 *         @link "errno.h" "errno" for the calling process.
 *
 * @seealso stat
 * @seealso stat64
 * @seealso readdir_stat64
 * @seealso default_restrictions_calling_functions
 */
int fstat(int fd, struct stat *buf);


/**
 * @function stat
 *
 * @include        "sys/stat.h"
 *
 * @brief Get file status.
 *
 * @long Return information about the specific file or directory specified by
 *       the path.
 *
 * @long Standards:
 *       Portable Operating System Interface (POSIX.1).
 *
 * @long Conformance:
 *       ISO/IEC 9945-1:1990 (IEEE 1003.1-1990).
 *
 * @input pathname   The path to the file or directory to stat.
 * @input buf        A pointer to a @link "" "stat" structure buffer to hold
 *                   the result of stat.
 *
 * @return Returns zero if the call was successful. If stat fails,
 *         it shall return -1 and set
 *         @link "errno.h" "errno" for the calling process.
 *
 * @seealso fstat
 * @seealso stat64
 * @seealso readdir_stat64
 * @seealso wstat
 * @seealso default_restrictions_calling_functions
 */
int stat(const char *pathname, struct stat *buf);


/**
 * @function wstat
 *
 * @include       "sys/stat.h"
 *
 * @brief Retrieve file status using wchar_t pathname
 *
 * @long This function is equivalent to stat, with the exception that
 *       the pathname is specified as a wide character string.
 *
 * @long Return information about the specific file or directory specified by
 *       the path.
 *
 * @long Standards:
 *       None - This is an OSE-specific extension.
 * @long Conformance:
 *       None - This is an OSE-specific extension.
 *
 * @input pathname   The path to the file or directory to stat.
 * @input sbuf       A pointer to a @link "" "stat" structure buffer to hold 
 *                   the result of wstat. 
 *
 * @return Returns zero if the call was successful. If wstat fails,
 *         it shall return -1 and set 
 *         @link "errno.h" "errno" for the calling process. 
 *
 * @seealso fstat
 * @seealso stat
 * @seealso stat64
 * @seealso wstat64
 * @seealso readdir_stat64
 * @seealso wreaddir_stat64
 * @seealso default_restrictions_calling_functions
 */
int wstat(const wchar_t *pathname, struct stat *sbuf);


/**
 * @function stat64
 *
 * @include       "sys/stat.h"
 *
 * @brief Retrieve 64-bit file status
 *
 * @long Return information about the specific file or directory specified by
 *       the path.
 *
 * @long Standards:
 *       None - This is an OSE-specific extension.
 * @long Conformance:
 *       None - This is an OSE-specific extension.
 *
 * @input pathname   The path to the file or directory to stat.
 * @input buf        A pointer to a @link "" "stat64" structure buffer to hold
 *                   the result of stat64.
 * @input attr_mask  A bitmask indicating requested attributes, using
 *                   @link "" "StValidFlags" values.
 *
 * @return Returns zero if the call was successful. After a successful call to
 *         stat64, buf->st_valid indicates what fields of *buf contain valid
 *         values: a set bit indicates that the corresponding fields are valid;
 *         a cleared bit indicates that the corresponding fields have not been
 *         modified. Only fields explicitly requested in attr_mask may be
 *         modified by stat64. If stat64 fails, it shall return -1 and set
 *         @link "errno.h" "errno" for the calling process.
 *
 * @seealso fstat
 * @seealso stat
 * @seealso readdir_stat64
 * @seealso wstat64
 * @seealso default_restrictions_calling_functions
 */
int stat64(const char *pathname, struct stat64 *buf, uint32_t attr_mask);


/**
 * @function wstat64
 *
 * @include       "sys/stat.h"
 *
 * @brief Retrieve 64-bit file status using wchar pathname
 *
 * @long This function is equivalent to stat64, with the exception that
 *       the pathname is specified as a wide character string.
 *
 * @long Return information about the specific file or directory specified by
 *       the path.
 *
 * @long Standards:
 *       None - This is an OSE-specific extension.
 * @long Conformance:
 *       None - This is an OSE-specific extension.
 *
 * @input pathname   The path to the file or directory to stat.
 * @input sbuf       A pointer to a @link "" "stat64" structure buffer to hold 
 *                   the result of wstat64. 
 * @input attr_mask  A bitmask indicating requested attributes, using
 *                   @link "" "StValidFlags" values.
 *
 * @return Returns zero if the call was successful. After a successful call to
 *         wstat64, buf->st_valid indicates what fields of *buf contain valid
 *         values: a set bit indicates that the corresponding fields are valid;
 *         a cleared bit indicates that the corresponding fields have not been
 *         modified. Only fields explicitly requested in attr_mask may be
 *         modified by wstat64. If wstat64 fails, it shall return -1 and set
 *         @link "errno.h" "errno" for the calling process.
 *
 * @seealso fstat
 * @seealso stat
 * @seealso wstat
 * @seealso wreaddir_stat64
 * @seealso stat64
 * @seealso default_restrictions_calling_functions
 */
int wstat64(const wchar_t *pathname, struct stat64 *sbuf, uint32_t attr_mask);


/**
 * @function mkfifo
 *
 * @include        "sys/stat.h"
 *
 * @brief Create a FIFO special file, also known as a named pipe.
 *
 * @long mkfifo creates a FIFO special file with name pathname. 
 *       The parameter mode specifies the files permissions, filtered
 *       with umask.
 *
 * @long Standards: Portable Operating System Interface (POSIX.1).
 * @long Conformance: ISO/IEC 9945-1:1990 (IEEE 1003.1-1990), with
 *                   restrictions as noted below.
 *
 * @restriction A pipe file system must be mounted as /pipe and the
 *              pathname of the FIFO special file must be in the /pipe volume,
 *              for this function to be successful.
 *
 * @break       Once created, the new FIFO special file will reside in the
 *              pipe volume manager until explicitly removed. Named pipes are
 *              not persistent(i.e. they are never stored on media).
 *   
 * @input pathname   The path to the FIFO special file. The path must
 *                   correspond to a pipe file system.
 *
 * @input mode       The permissions for the created the FIFO special file.
 *
 * @return           Returns zero if the call was successful.
 *                   In case of error, -1 is returned and errno is set.
 *
 * @seealso wmkfifo
 * @seealso default_restrictions_calling_functions
 */
int mkfifo(const char *pathname, mode_t mode);


/**
 * @function wmkfifo
 *
 * @include        "sys/stat.h"
 *
 * @brief Create a FIFO special file, also known as a named pipe.
 *
 * @long This function is equivalent to mkfifo, with the exception that
 *       the pathname is specified as a wide character string.
 *
 * @long mkfifo creates a FIFO special file with name pathname. 
 *       The parameter mode specifies the files permissions, filtered
 *       with umask.
 *
 * @long Standards: None - This is an OSE-specific extension.
 * @long Conformance: None - This is an OSE-specific extension.
 *
 * @restriction A pipe file system must be mounted as /pipe and the
 *              pathname of the FIFO special file must be in the /pipe volume,
 *              for this function to be successful.
 *
 * @break       Once created, the new FIFO special file will reside in the
 *              pipe volume manager until explicitly removed. Named pipes are
 *              not persistent(i.e. they are never stored on media).
 *   
 * @input pathname   The path to the FIFO special file. The path must
 *                   correspond to a pipe file system.
 *
 * @input mode       The permissions for the created the FIFO special file.
 *
 * @return           Returns zero if the call was successful.
 *                   In case of error, -1 is returned and errno is set.
 *
 * @seealso mkfifo
 * @seealso default_restrictions_calling_functions
 */
int wmkfifo(const wchar_t *pathname, mode_t mode);


/**
 * @function umask
 *
 * @include        "sys/stat.h"
 *
 * @declaration mode_t
 * umask(mode_t cmask);
 *
 * @brief Sets the process's file creation mode mask.
 *
 * @long The umask function sets the file mode creation mask of the client to
 *       cmask and returns the previous value of the mask.
 * @long The file mode creation mask of the client is used during
 *       @link "stdio.h" "fopen", @link "" "open",  @link "" "creat" and
 *       @link "" "mkdir" calls to turn off permission bits in the attrs
 *       argument supplied for a new file or directory.
 *       Bit positions that are set in cmask are cleared in the attrs of the
 *       created file.
 * @long The following modes are used in cmask, the values can be combined
 *       together to form a valid mask:
 * @long - @link "sys/stat.h" "S_IRUSR": Read permission bit for the file
 *         owner.
 * @long - @link "sys/stat.h" "S_IWUSR": Write permission bit for the file
 *         owner.
 * @long - @link "sys/stat.h" "S_IXUSR": Execute (search) permission bit for
 *         the file owner.
 * @long - @link "sys/stat.h" "S_IRGRP": Read permission bit for the file
 *         group.
 * @long - @link "sys/stat.h" "S_IWGRP": Write permission bit for the file
 *         group.
 * @long - @link "sys/stat.h" "S_IXGRP": Execute(search) permission bit for the
 *         file group.
 * @long - @link "sys/stat.h" "S_IROTH": Read permission bit for others.
 * @long - @link "sys/stat.h" "S_IWOTH": Write permission bit for others.
 * @long - @link "sys/stat.h" "S_IXOTH": Execute(search) permission bit for
 *         others.
 *
 * @restriction Permission classes (owner/group/others) are often not
 *              implemented by the volume manager.
 *
 * @long Standards: Portable Operating System Interface (POSIX.1).
 * @long Conformance: ISO/IEC 9945-1:1990 (IEEE 1003.1-1990).
 *
 * @input cmask The file permission bits.
 *
 * @return The umask function returns the previous value of the file mode
 *         creation mask.
 *
 * @seealso open
 * @seealso creat
 * @seealso mkdir
 * @seealso fopen
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // To set the creation mask to read permission for the group:
 * oldmask = umask(S_IRGRP);
 */

mode_t umask(mode_t cmask);



/*
 ****************************************************************************
 * 7  C++ HANDLING FOR SOFT ENVIRONMENTS.
 ****************************************************************************
 */

#ifdef __cplusplus
} /* end of extern "C" */
#endif

#ifdef softOSE
#define ZZMK_INLINE_FUNCTIONS
#include "sfk/sys/sfk_stat.h"
#endif

/* End #ifndef _SYS_STAT_H */
#endif
