/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2011 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @public
 * @toc Standard_C:
 * @file termios.h
 *
 * @brief OSE BSD interface termios.h, by the terminal I/O interfaces.
 *
 */

#ifndef _TERMIOS_H
#define _TERMIOS_H
#define OSE_TERMIOS_H

#include <stddef.h>
#include "osarch.h"

#ifdef softOSE
#define ZZMK_NAME_MANGLING
#include "sfk/sfk_termios.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if defined(__linux__) && defined(_POSIX_SOURCE)
#undef _POSIX_SOURCE
#endif

#define	TCSANOW		0
#define	TCSADRAIN	1
#define	TCSAFLUSH	2

#define TCIFLUSH	1
#define TCOFLUSH	2
#define TCIOFLUSH	3

#define TCOOFF		1
#define TCOON		2
#define TCIOFF		3
#define TCION		4

#define B300          300
#define B600          600
#define B1200        1200
#define B2400        2400
#define B4800        4800
#define B9600        9600
#define B14400	    14400
#define B19200      19200
#define B28800	    28800
#define B38400      38400
#define B57600      57600
#define B76800      76800
#define B115200    115200

/* c_cc element                    c_lflag enabler. */

#define VEOL		0	/* ICANON */
#define	VEOF		1	/* ICANON */
#define	VERASE		2	/* ICANON */
#define VERASERIGHT	3	/* ICANON. EFS extension. */
#define VWERASE 	4	/* ICANON */
#define VKILL		5	/* ICANON */
#define	VREPRINT 	6	/* ICANON */

#define VINTR		7	/* ISIG */
#define VSUSP		8	/* ISIG */

#define VSTART		9	/* IXON, IXOFF */
#define VSTOP		10	/* IXON, IXOFF */

#define	VDISCARD	11	/* IEXTEN */
#define VVERBATIM	12	/* ICANON */
#define VSTATUS		13	/* ICANON */
#define VCURLEFT	14	/* ICANON */
#define VCURRIGHT	15	/* ICANON */
#define VCURBEGIN	16	/* ICANON */
#define VCUREND		17	/* ICANON */
#define VSEARCH		18	/* ICANON */
#define VPREVLINE	19	/* ICANON */
#define VNEXTLINE	20	/* ICANON */

#define VMIN		32	/* !ICANON */
#define VTIME		33	/* !ICANON */

/* Number of elements in c_cc array. */
#define	NCCS		34	/* FM_TCC_SIZE + 2 (VMIN/VTIME), was 22. */

#define _POSIX_VDISABLE	((unsigned char)0xFF)

/* Do some cleanup among the termios flags? */

/*
 * Input flags - software input processing
 */

/**
 * @macro IGNBRK
 * @brief Ignore BREAK condition.
 */
#define	IGNBRK		0x00000001

/**
 * @macro BRKINT
 * @brief Map BREAK to SIGINTR.
 */
#define	BRKINT		0x00000002

/**
 * @macro IGNPAR
 * @brief Ignore (discard) parity errors.
 */
#define	IGNPAR		0x00000004

/**
 * @macro PARMRK
 * @brief Mark parity and framing errors.
 */
#define	PARMRK		0x00000008

/**
 * @macro INPCK
 * @brief Enable checking of parity errors.
 */
#define	INPCK		0x00000010

/**
 * @macro ISTRIP
 * @brief Strip 8th bit off chars.
 */
#define	ISTRIP		0x00000020

/**
 * @macro INLCR
 * @brief Map NL into CR.
 */
#define	INLCR		0x00000040

/**
 * @macro IGNCR
 * @brief Ignore CR.
 */
#define	IGNCR		0x00000080

/**
 * @macro ICRNL
 * @brief Map CR to NL (ala CRMOD).
 */
#define	ICRNL		0x00000100

/**
 * @macro IXON
 * @brief Enable output flow control.
 */
#define	IXON		0x00000200

/**
 * @macro IXOFF
 * @brief Enable input flow control.
 */
#define	IXOFF		0x00000400
#ifndef _POSIX_SOURCE

/**
 * @macro IXANY
 * @brief Any char will restart after stop.
 */
#define	IXANY		0x00000800

/**
 * @macro IMAXBEL
 * @brief Ring bell on input queue full.
 */
#define IMAXBEL		0x00002000
#endif  /*_POSIX_SOURCE */

/*
 * Output flags - software output processing
 */

/**
 * @macro OPOST
 * @brief Enable following output processing.
 */
#define	OPOST		0x00000001
#ifndef _POSIX_SOURCE

/**
 * @macro ONLCR
 * @brief Map NL to CR-NL (ala CRMOD).
 */
#define ONLCR		0x00000002

/**
 * @macro OXTABS
 * @brief Expand tabs to spaces.
 */
#define OXTABS		0x00000004

/**
 * @macro ONOEOT
 * @brief Discard EOTs (^D) on output.
 */
#define ONOEOT		0x00000008
#endif  /*_POSIX_SOURCE */

/*
 * Control flags - hardware control of terminal
 */
#ifndef _POSIX_SOURCE

/**
 * @macro CIGNORE
 * @brief Ignore control flags.
 */
#define	CIGNORE		0x00000001
#endif

/**
 * @macro CSIZE
 * @brief Character size mask.
 */
#define CSIZE		0x00000300

/**
 * @macro CS5
 * @brief 5 bits (pseudo).
 */
#define CS5		    0x00000000

/**
 * @macro CS6
 * @brief 6 bits.
 */
#define CS6		    0x00000100

/**
 * @macro CS7
 * @brief 7 bits.
 */
#define CS7		    0x00000200

/**
 * @macro CS8
 * @brief 8 bits.
 */
#define CS8		    0x00000300

/**
 * @macro CSTOPB
 * @brief Send 2 stop bits.
 */
#define CSTOPB		0x00000400

/**
 * @macro CREAD
 * @brief Enable receiver.
 */
#define CREAD		0x00000800

/**
 * @macro PARENB
 * @brief Parity enable.
 */
#define PARENB		0x00001000

/**
 * @macro PARODD
 * @brief Odd parity, else even.
 */
#define PARODD		0x00002000

/**
 * @macro HUPCL
 * @brief Hang up on last close.
 */
#define HUPCL		0x00004000

/**
 * @macro CLOCAL
 * @brief Ignore modem status lines.
 */
#define CLOCAL		0x00008000
#ifndef _POSIX_SOURCE

/**
 * @macro CRTSCTS
 * @brief RTS/CTS full-duplex flow control.
 */
#define	CRTSCTS		0x00010000

/**
 * @macro CRTS_IFLOW
 * @brief XXX compat.
 */
#define	CRTS_IFLOW	CRTSCTS

/**
 * @macro CCTS_OFLOW
 * @brief XXX compat.
 */
#define	CCTS_OFLOW	CRTSCTS

/**
 * @macro MDMBUF
 * @brief DTR/DCD hardware flow control.
 */
#define	MDMBUF		0x00100000

/**
 * @macro CHWFLOW
 * @brief All types of HW flow control.
 */
#define	CHWFLOW		(MDMBUF|CRTSCTS)
#endif

/*
 * "Local" flags - dumping ground for other state
 *
 * Warning: some flags in this structure begin with
 * the letter "I" and look like they belong in the
 * input flag.
 */

#ifndef _POSIX_SOURCE

/**
 * @macro ECHOKE
 * @brief Visual erase for line kill.
 */
#define	ECHOKE		0x00000001
#endif  /*_POSIX_SOURCE */

/**
 * @macro ECHOE
 * @brief Visually erase chars.
 */
#define	ECHOE		0x00000002

/**
 * @macro ECHOK
 * @brief Echo NL after line kill.
 */
#define	ECHOK		0x00000004

/**
 * @macro ECHO
 * @brief Enable echoing.
 */
#define ECHO		0x00000008

/**
 * @macro ECHONL
 * @brief Echo NL even if ECHO is off.
 */
#define	ECHONL		0x00000010
#ifndef _POSIX_SOURCE

/**
 * @macro ECHOPRT
 * @brief Visual erase mode for hardcopy.
 */
#define	ECHOPRT		0x00000020

/**
 * @macro ECHOCTL
 * @brief Echo control chars as ^(Char).
 */
#define ECHOCTL  	0x00000040
#endif  /*_POSIX_SOURCE */

/**
 * @macro ISIG
 * @brief Enable signals INTR, QUIT, [D]SUSP.
 */
#define	ISIG		0x00000080

/**
 * @macro ICANON
 * @brief Canonicalize input lines.
 */
#define	ICANON		0x00000100
#ifndef _POSIX_SOURCE

/**
 * @macro ALTWERASE
 * @brief Use alternate WERASE algorithm.
 */
#define ALTWERASE	0x00000200
#endif  /*_POSIX_SOURCE */

/**
 * @macro IEXTEN
 * @brief Enable DISCARD and LNEXT.
 */
#define	IEXTEN		0x00000400

/**
 * @macro EXTPROC
 * @brief External processing.
 */
#define EXTPROC         0x00000800

/**
 * @macro TOSTOP
 * @brief Stop background jobs from output.
 */
#define TOSTOP		0x00400000
#ifndef _POSIX_SOURCE

/**
 * @macro FLUSHO
 * @brief Output being flushed (state).
 */
#define FLUSHO		0x00800000

/**
 * @macro NOKERNINFO
 * @brief No kernel output from VSTATUS.
 */
#define	NOKERNINFO	0x02000000

/**
 * @macro PENDIN
 * @brief XXX retype pending input (state).
 */
#define PENDIN		0x20000000
#endif  /*_POSIX_SOURCE */

/**
 * @macro NOFLSH
 * @brief Do not flush after interrupt.
 */
#define	NOFLSH		0x80000000

/* EFS extensions. */
/**
 * @macro SAVELINE
 * @brief Save line in history.
 * @long EFS extension.
 */
#define SAVELINE	0x40000000

/**
 * @type speed_t
 * @brief Used for terminal baud rates.
 */
typedef unsigned long speed_t;

/**
 * @type tcflag_t
 * @brief Used for terminal modes.
 */
typedef unsigned long tcflag_t;

/**
 * @type cc_t
 * @brief Used for terminal special characters.
 */
typedef unsigned char cc_t;

/**
 * @type termios
 * @brief The termios structure.
 * @field c_iflag  Input modes.
 * @field c_oflag  Output modes.
 * @field c_cflag  Control modes.
 * @field c_lflag  Local modes.
 * @field c_cc     Control characters.
 * @field c_ispeed Input speed.
 * @field c_ospeed Output speed.
 */
struct termios
{
	tcflag_t c_iflag;	/* Input modes. */
	tcflag_t c_oflag;	/* Output modes. */
	tcflag_t c_cflag;	/* Control modes. */
	tcflag_t c_lflag;	/* Local modes. */
	cc_t     c_cc[NCCS];	/* Control characters. */
	speed_t  c_ispeed;	/* Input speed. */
	speed_t  c_ospeed;	/* Output speed. */
};


/**
 * @function tcgetattr
 * @brief Get the parameters associated with the terminal.
 * @long Get the parameters associated with the terminal referred
 *       to by fd and store them in the termios structure.
 * @field fd File descriptor.
 * @field termios_p The termios structure.
 *
 * @return Zero if successful, otherwise -1 and errno is set.
 *
 * @error EBADF  fd is not a valid file descriptor.
 * @error ESTALE The file descriptor is stale because contact is lost
 *               with the associated volume manager.
 * @error ENOTTY The file associated with fd is not a terminal.
 *
 * @seealso default_restrictions_calling_functions
 */
int tcgetattr(int fd, struct termios *termios_p);

/**
 * @function tcsetattr
 * @brief Set the parameters associated with the terminal referred to
 *        by fd.
 * @field fd File descriptor.
 * @field optional_actions If set to TCSANOW, the change will
 *                         occur immediately. TCSADRAIN means the change
 *                         will occur after all output written to fd
 *                         is transmitted. This function should be used when
 *                         changing parameters that affect output.
 *                         TCSAFLUSH means the change will occur after all
 *                         output written to fd is transmitted, and all
 *                         input so far received but not read will be
 *                         discarded before the change is made.
 * @field termios_p The termios structure.
 * @return Returns successfully if able to perform any of the
 *         requested actions, even if some of the requested actions could
 *         not be performed. It will set all the attributes that
 *         implementation supports as requested and leave all the attributes
 *         not supported by the implementation unchanged. If no part of the
 *         request can be carried out, it will return -1 and errno is set.
 *
 * @error EBADF  fd is not a valid file descriptor.
 * @error ESTALE The file descriptor is stale because contact is lost
 *               with the associated volume manager.
 * @error EINVAL The optional_actions argument is not a supported value,
 *               or an attempt was made to change an attribute represented
 *               in the termios structure to an unsupported value.
 * @error ENOTTY The file associated with fd is not a terminal.
 *
 * @seealso default_restrictions_calling_functions
 */
int tcsetattr(int fd, int optional_actions, const struct termios *termios_p);

/**
 * @function tcdrain
 * @brief Waits until all output written to the object referred to by
 *        fd is transmitted.
 * @field fd File descriptor.
 * @return Zero if successful, otherwise, -1 and errno is set.
 *
 * @error EBADF  fd is not a valid file descriptor.
 * @error ESTALE The file descriptor is stale because contact is lost
 *               with the associated volume manager.
 * @error ENOTTY The file associated with fd is not a terminal.
 *
 * @seealso default_restrictions_calling_functions
 */
int tcdrain(int fd);

/**
 * @function tcflush
 * @brief Discard data written to the object referred to by fd.
 * @long Discard data written to the object referred to by fd
 *       but not transmitted, or data received but not read,
 *       depending on the value of queue_selector.
 * @field fd File descriptor.
 * @field queue_selector If set to TCIFLUSH it flushes data received
 *                       but not read. TCOFLUSH flushes data written but
 *                       not transmitted. TCIOFLUSH flushes both data
 *                       received but not read and data written but not
 *                       transmitted.
 * @return Zero if successful, otherwise, -1 and errno is set.
 *
 * @error EBADF  fd is not a valid file descriptor.
 * @error ESTALE The file descriptor is stale because contact is lost
 *               with the associated volume manager.
 * @error EINVAL The queue_selector argument is not a supported value.
 * @error ENOTTY The file associated with fd is not a terminal.
 *
 * @seealso default_restrictions_calling_functions
 */
int tcflush(int fd, int queue_selector);

/**
 * @function tcflow
 * @brief Suspend transmission or reception of data on the object referred to
 *        by fd, depending on the value of action.
 * @field fd File descriptor.
 * @field action If set to TCOOFF, output is suspended.
 *               TCOON restarts suspended output.
 *               TCIOFF lets the system transmit a STOP character, which is
 *               intended to cause the terminal device to stop
 *               transmitting data to the system.
 *               TCION lets the system transmit a START character, which is
 *               intended to cause the terminal device to start transmitting
 *               data to the system.
 * @return Zero if successful, otherwise, -1 and errno is set.
 *
 * @error EBADF  fd is not a valid file descriptor.
 * @error ESTALE The file descriptor is stale because contact is lost
 *               with the associated volume manager.
 * @error EINVAL The action argument is not a supported value.
 * @error ENOTTY The file associated with fd is not a terminal.
 *
 * @seealso default_restrictions_calling_functions
 */
int tcflow(int fd, int action);

/**
 * @function cfgetospeed
 * @brief  Extract the output baud rate from the termios structure.
 * @field termios_p The termios data structure.
 * @return If successful, cfgetospeed() returns a value of
 *         type speed_t representing the output baud rate.
 *
 * @seealso default_restrictions_calling_functions
 */
speed_t cfgetospeed(const struct termios *termios_p);

/**
 * @function cfsetospeed
 * @brief Sets the output baud rate stored in the termios structure to speed.
 * @field termios_p The termios data structure.
 * @field speed The desired baud rate.
 * @return Zero if successful, otherwise, -1 and errno is set.
 *
 * @error EINVAL The action argument is not a supported value.
 * @error ESTALE The file descriptor is stale because contact is lost
 *               with the associated volume manager.
 * @error EINVAL The value of speed is outside the range of
 *               possible values.
 *
 * @seealso default_restrictions_calling_functions
 */
int cfsetospeed(struct termios *termios_p, speed_t speed);

/**
 * @function cfgetispeed
 * @brief Extract the input baud rate from the termios structure.
 * @field termios_p The termios structure.
 * @return If successful, cfgetispeed() returns a value of
 *         type speed_t representing the input baud rate.
 *
 * @seealso default_restrictions_calling_functions
 */
speed_t cfgetispeed(const struct termios *termios_p);

/**
 * @function cfsetispeed
 * @brief Set the input baud rate stored in the termios structure to speed.
 * @field termios_p The termios structure.
 * @field speed The desired baud rate.
 * @return Zero if successful, otherwise, -1 and errno is set.
 *
 * @error EINVAL The speed value is not a valid baud rate.
 * @error ESTALE The file descriptor is stale because contact is lost
 *               with the associated volume manager.
 * @error EINVAL The action argument is not a supported value.
 *
 * @seealso default_restrictions_calling_functions
 */
int cfsetispeed(struct termios *termios_p, speed_t speed);

#ifdef __cplusplus
} /* end of extern "C" */
#endif

#ifdef softOSE
#define ZZMK_INLINE_FUNCTIONS
#include "sfk/sfk_termios.h"
#endif

/* End #ifndef _TERMIOS_H */
#endif
