/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @private
 *
 * @brief Interface to fm_dbgserver.c
 */

#include "osetypes.h"
#include "ose.h"

/*
*****************************************************************************
** Constants
*****************************************************************************
*/

#define MAJOR_INTERFACE_VERSION 1
#define MINOR_INTERFACE_VERSION 2

/* Process states as used by the Illuminator (ProcessState.java)
 * see ose.h PROCESS_STATE and these additions. */
#define DBG_RUNNING     1024
#define DBG_READY       2048
#define DBG_KILLED      (0xFFFF)

struct SigHeader
{
  SIGSELECT signo;
  U32 size;
};

/**
 * @function ose_dbg_fm_transfer_bp
 *
 * @brief Used to set a break point by the JTAG debugger.
 *
 * @long This function is called when the transmit buffer is full
 * or the transaction is completed.
 *
 * @long The JTAG debugger shall have a break point at this function,
 * when this function is reached the debugger should read the contents in
 * the transmit buffer and if the transaction is not completed it should
 * resume and let this function return. However if the transaction is
 * completed the debugger may not resume the execution from this point.
 *
 */
void ose_dbg_fm_transfer_bp(void);

/**
 * @function ose_dbg_fm_process_message
 *
 * @declaration ose_dbg_fm_process_message(void);
 *
 * @brief Process one message from the transfer buffer and generate replies.
 *
 */
void ose_dbg_fm_process_message(void);

/**
 * @function ose_dbg_fm_reset_trf_buf
 *
 * @declaration void ose_dbg_fm_reset_trf_buf(void);
 *
 * @brief Reset the transfer buffer.
 */
void ose_dbg_fm_reset_trf_buf(void);

/**
 * @function ose_dbg_fm_alloc
 *
 * @declaration union SIGNAL *ose_dbg_fm_alloc(U32 size, SIGSELECT signo);
 *
 * @brief Allocate buffer from fm_dbgserver transfer buffer.
 */
union SIGNAL *ose_dbg_fm_alloc(U32 size, SIGSELECT signo);

/**
 * @function ose_dbg_fm_send
 *
 * @declaration void ose_dbg_fm_send(union SIGNAL *sig);
 *
 * @brief Transfer a signal to the fm_dbgserver by the buffer.
 */
void ose_dbg_fm_send(union SIGNAL *sig);
