/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @public
 *
 * @file heap_private.h
 *
 * @brief Common functions, defines for core,crt.
 *
 */

#ifndef HEAP_PRIVATE_H
#define HEAP_PRIVATE_H

#include "ose_heap.h"

#ifdef OSMIPS
#include "bios.h"
#define HEAP_BIOS_NAME "HEAP"

/* Handle to BIOS module */
extern unsigned long zzheap_handle;

/**
 * @macro HEAP_CLEANUPD_ENTRYPOINT
 *
 * @declaration #define HEAP_CLEANUPD_ENTRYPOINT
 *
 * @brief The user-mode entrypoint to the cleanup-daemon.
 * @long For each non-system-domain block that has a valid heap,
 *       a heap cleanup-daemon shall be created the first time the heap
 *       is used. A block/program variable containing the entry-point
 *       to the cleanup-daemon shall exist.
 * @long The variable SHOULD NOT be set by a user.
 */
#define HEAP_CLEANUPD_ENTRYPOINT     "heap_cleanupd_entrypoint"

#endif /* #ifdef OSMIPS */

/**
 * @macro HEAP_ALIGNMENT
 *
 * @brief Defines the minimum alignment allowed for heap buffers. 
 */
#define HEAP_ALIGNMENT 8

/*
 *  HEAP bios function codes
 */
#define FC_INIT              0
#define FC_UNSAFE            1  
#define FC_EARLY_NULL        2
#define FC_SAFE              3  
#define FC_FREE              4
#define FC_SIZE              5
#define FC_CLEANUP           6
#define FC_BUFFERINFO        7
#define FC_RESIZE            8
#define FC_EXTEND            9
#define FC_ACTUAL_SIZE       10 /* Returns the buffer size + overhead. */
#define FC_NEXT_SIZE         11 /* Returns the next available size. */
#define FC_REGISTER_CLIENT   12
#define FC_CREATE_HEAP       13
#define FC_CLEANUPD          14
#define FC_SET_DEBUG         15
#define FC_FORCED_FREE       16
#define FC_REMOVE_DEBUG      17
#define FC_NO_EXTEND         18
#define FC_SETMAX            19
#define FC_SHRINK            20
#define FC_FLUSH_CACHE       21
#define FC_ALLOC_TENTATIVE   22
#define FC_FREE_TENTATIVE    23
#define FC_ALLOC_WATERMARK   24

#define FC_PLUG_ALLOC        25
#define FC_PLUG_FREE         26
#define FC_PLUG_RESIZE       27
#define FC_PLUG_COLLECT      28
#define FC_PLUG_QUERY        29
#define FC_PLUG_CONTROL      30

/*
 * The name of the ppd key for the heap ref
 */

#define HEAP_KEY "ose_heap_key"

/*
 * The name of the ppd key for the heap debug data
 */

#define HEAP_DEBUG_PP_KEY "ose_heap_debug_key"

/*
 * returns true if shared mode
 */

#define IS_SHARED(ref) ((ref) & 0x1)

/*
 * mask off shared to get a pointer.
 */

#define HUCB(ref) ((HUCB *)((ref) & 0xFFFFFFFC))

/*
 * current_heap_ref, returns heap reference. 
 */

OSHEAP current_heap_ref(void);

/*
 * heap_alloc and heap_free are interface functions to the monolith
 */
void *heap_alloc(OSHEAP heap, size_t size, PROCESS owner, const char *file,
           unsigned line, int alloc_type, U32 param);
void heap_free(OSHEAP heap, void *ptr, PROCESS owner);

   
/*
 * initialize heap pp data key, MUST BE CALLED before any heap user!! 
 */

void   ose_heap_init(void);

/**
 * @function ose_create_manager_heap
 *
 * @brief  Create a new manager mode heap.
 *         
 * @long   A manager mode heap, or secure heap, is write-protected,
 *         to user mode as well as supervisor mode processes.
 *         
 * @long   Supervisor mode processes may temporarily enter manager mode, using
 *         zzset_manager_mode, to allocate, free or write to a secure heap
 *         buffer. The process should then leave manager mode to reactivate
 *         the write protection.
 *         
 * @long   A manager mode heap can only be created in the system domain.
 *         The heap must be associated with a special logical memory partition,
 *         configured with "manager_mode:yes", from which the heap allocates
 *         new fragments.
 *         
 * @input  log_mem A manager mode logical memory partition, where write access
 *         permissions can be relaxed using manager mode.
 * @input  pid A phantom process acting as heap owner.
 *
 * @return A reference to the newly created secure heap.
 */
OSHEAP ose_create_manager_heap(const char *log_mem, PROCESS pid);

#endif
