/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef _HEAP_PRIVATE_SIG
#define _HEAP_PRIVATE_SIG

#include "ose.h"

/*
 * This is a private heap signal interface. 
 */


/**
 * @type HeapTagId
 *
 * @brief Enum containing HEAP information tag identifiers.
 *        The identifiers are used in the struct
 *        @link "heap_private.sig" "HeapTag" to indicate
 *        requested information. .
 * 
 * @field HEAP_FRAG_INFO_TAG_REQ_SIZE
 *    This is a tag for the signal
 *    @link "heap.sig" "HEAP_FRAG_INFO_REQUEST".
 *    It returns the amount of used heap memory within a single
 *    fragment (excluding slack caused by fragmentation). 
 *
 * @brief HEAP_FRAG_INFO_TAG_USED_SIZE
 *    This is a tag for the signal
 *    @link "heap.sig" "HEAP_FRAG_INFO_REQUEST".
 *    It returns the amount of used heap memory within a single
 *    fragment (including slack caused by fragmentation). 
 *
 * @brief HEAP_FRAG_INFO_TAG_FREE_SIZE
 *    This is a tag for the signal
 *    @link "heap.sig" "HEAP_FRAG_INFO_REQUEST".
 *    It returns the amount of free heap memory within a single
 *    fragment. 
 */
enum HeapTagId
{
   HEAP_FRAG_INFO_TAG_REQ_SIZE          = 0x01,
   HEAP_FRAG_INFO_TAG_USED_SIZE         = 0x02,
   HEAP_FRAG_INFO_TAG_FREE_SIZE         = 0x03

/*
 * The tags below are unimplemented and only listed
 * here as an example of what could be done in the future.
 */
#if 0
 HEAP_INFO_TAG_NUM_SHARED
 HEAP_INFO_TAG_NUM_PRIVATE
 HEAP_INFO_TAG_MAS
 HEAP_INFO_TAG_TOTAL_SIZE
 HEAP_INFO_TAG_REQ_SIZE
 HEAP_INFO_TAG_USED_SIZE
 HEAP_INFO_TAG_USED_HI_SIZE
 HEAP_INFO_TAG_FREE_SIZE
 HEAP_INFO_TAG_NUM_FRAGS
 HEAP_INFO_TAG_LARGEST_FREE
 HEAP_INFO_TAG_MALLOC_FAILED
 HEAP_INFO_TAG_NUM_SIZES

 HEAP_INFO_TAG_BUF_SIZE
 HEAP_INFO_TAG_BUF_NUM_USED
 HEAP_INFO_TAG_BUF_NUM_FREE

 HEAP_INFO_TAG_CFG_MAX_SIZE
 HEAP_INFO_TAG_CFG_SMALL_INIT_SIZE
 HEAP_INFO_TAG_CFG_SMALL_EXT_SIZE
 HEAP_INFO_TAG_CFG_LARGE_THRESHOLD
 HEAP_INFO_TAG_CFG_DOMAIN
 HEAP_INFO_TAG_CFG_PERMISSIONS
 HEAP_INFO_TAG_CFG_CACHE_MODE
 HEAP_INFO_TAG_CFG_ENDMARK_SIZE
 HEAP_INFO_TAG_CFG_ANNOTATION_SIZE
 HEAP_INFO_TAG_CFG_WIPE_ON_FREE
 HEAP_INFO_TAG_CFG_ERROR_ON_NULL
 HEAP_INFO_TAG_CFG_OWNER
 HEAP_INFO_TAG_CFG_SMALL_LOW_LIMIT
 HEAP_INFO_TAG_CFG_LARGE_LOW_LIMIT

 HEAP_FRAG_INFO_TAG_NUM_SHARED
 HEAP_FRAG_INFO_TAG_NUM_PRIVATE
 HEAP_FRAG_INFO_TAG_MAS
 HEAP_FRAG_INFO_TAG_TOTAL_SIZE
 HEAP_FRAG_INFO_TAG_REQ_SIZE
 HEAP_FRAG_INFO_TAG_USED_SIZE
 HEAP_FRAG_INFO_TAG_FREE_SIZE
 HEAP_FRAG_INFO_TAG_NUM_FRAGS
 HEAP_FRAG_INFO_TAG_LARGEST_FREE
#endif
};

#if 0
#endif


/**
 * @type HeapTag
 *
 * @brief Tag identitity and tag value.
 *
 * @field tag_id    Question. Values from enum
 *                  @link "heap_private.sig" "HeapTagId"
 *
 * @field tag_value Answer. Assigned by the heap daemon.
 */
struct HeapTag
{
   OSADDRESS   tag_id;
   OSADDRESS   tag_value;
};



#if 0
/* Currently unimplemented */
struct HeapInfo
{
   SIGSELECT   signo;
   OSHEAP      href;
   OSBUFSIZE   num_tags;
   OSBUFSIZE   err_index;

   struct HeapTag tags[1];
};

struct HeapFragList
{
   SIGSELECT   signo;
   OSHEAP      href;
   OSBUFSIZE   num_frags;

   struct FragAddress {
      OSADDRESS  base;
      OSBUFSIZE  size;
   } frags[1];
};
#endif


/**
 * @signal HEAP_FRAG_INFO_REQUEST
 *
 * @brief  Get detailed information about a heap fragment.
 *
 * @member HeapFragInfo
 *
 * @seealso HEAP_FRAG_INFO_REPLY
 */
#define HEAP_FRAG_INFO_REQUEST (33420ul) /* !-SIGNO(struct HeapFragInfo)-! */


/**
 * @signal HEAP_FRAG_INFO_REPLY
 * 
 * @brief   Signal from heap daemon in reply to
 *          @link "heap_private.sig" "HEAP_FRAG_INFO_REQUEST".
 *
 * @member  HeapFragInfo
 *
 * @seealso HEAP_FRAG_INFO_REQUEST
 */
#define HEAP_FRAG_INFO_REPLY   (33421ul) /* !-SIGNO(struct HeapFragInfo)-! */


/**
 * @type HeapFragInfo
 *
 * @field signo      Signal number.
 *
 * @input href       Heap reference.
 * @input base       Base address of fragment for which information is requested.
 * @input num_tags   Number of information tags to process.
 * @input tags       Information queries in tag format. Variable size.
 *
 * @output status    Status of the operation. 
 *                   HEAP_SUCCESS if the operation was successful.
 *                   HEAP_ENOT_FOUND if the fragment does not exist.
 *                   HEAP_INVALID_REQUEST if an unreqognizable tag identifier
 *                   was used.
 * @output err_index Index of first unreqognized tag identifier, if any.
 * @output tags      Information replies in tag format. Variable size.
 *
 * @seealso HEAP_FRAG_INFO_REQUEST
 * @seealso HEAP_FRAG_INFO_REPLY
 * @seealso HeapTagId
 */
struct HeapFragInfo
{
   SIGSELECT   signo;
   U32         status;
   OSHEAP      href;
   OSADDRESS   base;
   OSBUFSIZE   num_tags;
   OSBUFSIZE   err_index;

   struct HeapTag tags[1];
};

/**
 * @macro sendHeapFragInfoRequest
 *
 * @brief Macro to send heap fragment statistics requests.
 *
 * @long  See @link "" "HEAP_FRAGS_REQUEST".
 *
 * @input hrefMA     Heap reference to query for.
 * @input heapdPidMA Heapd pid to receive the request.
 * @input fragBaseMA Base address of fragment.
 * @input numTagsMA  Number of tags
 * @input tagId0MA   
 * @input tagId1MA   
 * @input tagId2MA   
 * 
 */
#define sendHeapFragInfoRequest(hrefMA, heapdPidMA, fragBaseMA, numTagsMA, tagId0MA, tagId1MA, tagId2MA) \
do\
{ \
   struct HeapFragInfo *req = (struct HeapFragInfo *) alloc(sizeof(struct HeapFragInfo) \
   + sizeof(struct HeapTag)*(numTagsMA), HEAP_FRAG_INFO_REQUEST);\
   req->href = (hrefMA);\
   req->base = (OSADDRESS)(fragBaseMA);\
   req->num_tags = (numTagsMA);\
   if ((numTagsMA)>0) {req->tags[0].tag_id = (tagId0MA);}\
   if ((numTagsMA)>1) {req->tags[1].tag_id = (tagId1MA);}\
   if ((numTagsMA)>2) {req->tags[2].tag_id = (tagId2MA);}\
   send((union SIGNAL **)&req, (heapdPidMA)); \
} while (0)


#endif /* _HEAP_PRIVATE_SIG */
