/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2011 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef _PER_CPU_DATA_H
#define _PER_CPU_DATA_H

/*
*****************************************************************************
** DESCRIPTION.
*****************************************************************************
** This file contains a simple framework for per cpu data in a multicore
** system. This feature shall only be used by components provided in the 
** libcore library. A number of components can define their own private
** view of the common vector, struct per_cpu zzuu[].
**
** For good performance on ARM, the total size of the struct per_cpu 
** should be less or equal 4K bytes (0x1000). Each component part should 
** be cache line aligned.
**
** The struct per_cpu zzcpu[] array contains as many structures of global 
** variables as there are supported CPUs in the SMP system.  
*/
#define	TI_PER_CPU_OFFSET	0x0
#define	TI_PER_CPU_SIZE		0x200
#define	TI_PER_CPU_END		((TI_PER_CPU_OFFSET) + (TI_PER_CPU_SIZE))

#define	POSE_PER_CPU_OFFSET	TI_PER_CPU_END
#define	POSE_PER_CPU_SIZE	0x400
#define	POSE_PER_CPU_END	((POSE_PER_CPU_OFFSET) + (POSE_PER_CPU_SIZE))

#define	MM_PER_CPU_OFFSET	POSE_PER_CPU_END
#define	MM_PER_CPU_SIZE		0x100
#define	MM_PER_CPU_END		((MM_PER_CPU_OFFSET) + (MM_PER_CPU_SIZE))

#define	XYZ_PER_CPU_OFFSET	MM_PER_CPU_END
#define	XYZ_PER_CPU_SIZE	0x100
#define	XYZ_PER_CPU_END		((XYZ_PER_CPU_OFFSET) + (XYZ_PER_CPU_SIZE))

#define PER_CPU_DATA_SIZE	XYZ_PER_CPU_END

#if !defined(__ASM_ONLY__)
/* Structure for multicore per cpu data */
struct per_cpu;

/* ================ PowerPC ================ */
#if defined(OSE_SMP)
#ifdef OSPP
static __inline__ void
zzkrn_set_per_cpu_data(void *ptr)
{
   __asm__ __volatile__ (" mtspr 274,%0" : : "r"(ptr));
}

static __inline__ struct per_cpu *
zzkrn_get_per_cpu_data(void)
{
   struct per_cpu *ptr;

   __asm__ __volatile__ (" mfspr %0,274" :"=r"(ptr):);
   return ptr;
}
#endif	/* OSPP */

/* ================ Mips ================ */
#ifdef OSMIPS
static __inline__ void
zzkrn_set_per_cpu_data(void *ptr)
{
#if 0
   register struct per_cpu *data asm("$28");
   data = ptr;
#else
   __asm__ __volatile__ (" ori $28,%0,0" ::"r"(ptr));
#endif
}

static __inline__ struct per_cpu*
zzkrn_get_per_cpu_data(void)
{
#if 0
   register struct per_cpu *data asm("$28");
   return data;
#else
   struct per_cpu *data;

   __asm__ __volatile__ (" ori %0,$28,0" :"=r"(data):);
   return data;
#endif
}
#endif	/* OSMIPS */

/* ================ x86 ================ */
#ifdef OSX86
void zzkrn_set_per_cpu_data(void *ptr);

static __inline__ struct per_cpu *
zzkrn_get_per_cpu_data(void)
{
    struct per_cpu * val;
    __asm__ __volatile__("mov %%fs:4, %0":"=r"(val):);
    return val;
}
#endif	/* OSX86 */

/* ================ ARM ================ */
#ifdef OSARM
/* Setup per_cpu pointer in Thread id register #4 */
static __inline__ void
zzkrn_set_per_cpu_data(void *ptr)
{
#ifdef __ARMCC_VERSION
  register uint32_t cp15_threadid4 __asm("cp15:0:c13:c0:4");
  cp15_threadid4 = (uint32_t)ptr;
#elif  __GNUC__
  __asm__ __volatile__ (" mcr p15,0,%0,c13,c0,4" : : "r"(ptr));
#endif
}

static __inline__ struct per_cpu *
zzkrn_get_per_cpu_data(void)
{
#ifdef __ARMCC_VERSION
  register struct per_cpu *cp15_threadid4 __asm("cp15:0:c13:c0:4");
#elif  __GNUC__
  struct per_cpu *cp15_threadid4;
  __asm__ __volatile__ (" mrc p15,0,%0,c13,c0,4" :"=r"(cp15_threadid4):);
#endif
  return cp15_threadid4;
}
#endif	/* OSARM */

#endif	/* OSE_SMP */
#endif	/* __ASM_ONLY__ */

#endif /* #ifndef _PER_CPU_DATA_H */

