/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2012 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
*****************************************************************************
**			COMMON BLOCK DEFINITIONS			   **
**									   **
**     Subsystem: 		Portable OSE.				   **
**     Original author:		Bengt Eliasson.				   **
**     Summary:			OSE target interface specification.	   **
**									   **
*****************************************************************************

	CONTENTS
	--------

	1  Description
	2  History of development
	3  Block definitions
	   3.1 Manifest constants
	   3.2 Type definitions
	   3.3 Macros
	4  External references
	   4.1 Functions
	   4.2 Variables
	   4.3 Backdoors
	   4.4 SMP specific

*****************************************************************************
*/

#ifndef _POSE_H
#define _POSE_H

/*
*****************************************************************************
** 1  DESCRIPTION.
*****************************************************************************
** This file contains items that are shared between the OSE portable kernel
** and the CPU dependent code written for a specific OSE port.
**
** This file generates no code.
**
**?? We should some day change all parameter types to 32 bit types since this is
** safer for most compilers. This mostly adds up to changing all occurences
** of OSBOOLEAN and OSPRIORITY to OSADDRESS.
*/

/*
*****************************************************************************
** 2  HISTORY OF DEVELOPMENT.
*****************************************************************************
** date      responsible           notes
** --------  --------------------  ------------------------------------------
** 92/03/20  Bengt Eliasson        Initial release.
** 93/08/06  Bengt Eliasson        Changed uarea parameters and added zzidle().
** 93/09/21  Bengt Eliasson        Added zzget_sysprocnames() and
**                                 zzget_scope_status() in the target interface.
**                                 Also added interface between pose.c and
**                                 the local debugger module.
** 93/10/01  Bengt Eliasson        Added some comments.
** 93/10/14  Bengt Eliasson        Added more comments.
** 93/11/22  Bengt Eliasson        Variables were declared "extern".
** 94/02/20  Bengt Eliasson        Removed some compiler warnings.
** 94/08/21  Bengt Eliasson        Added the zzgo_search entrypoint.
** 94/09/01  Bengt Eliasson        A prototype for the set_clib_pointer()
**                                 was added.
** 94/09/01  Bengt Eliasson        Extra parameters were added to:
**                                 zzset_breakpoint(),
**                                 zzclear_breakpoint() and
**                                 zzcreate_context().
**                                 The sysctlblk struct was extended.
** 94/11/14  Bengt Eliasson        Added the zzdp_look_aside variable.
**                                 The obsolete parameter to zzsave_context()
**                                 was removed.
** 95/04/15  Bengt Eliasson        Added the backdoor sections.
** 95/09/01  Bengt Eliasson        Added the zztivarp and zzldmvarp variable
**                                 pointers for post mortem debugger support.
** 95/09/01  Bengt Eliasson        Added the blockname field in the
**                                 debug_pcb struct.
** 95/09/01  Bengt Eliasson        Added the zzcapture_regs() call.
** 95/09/01  Bengt Eliasson        The sysctlblk struct was extended.
** 96/01/30  Bengt Eliasson        Zzcall_set_segment_mode now returns a value.
** 97/04/04  Bengt Eliasson        zzselect_user_pool() was introduced.
** 97/04/25  Bengt Eliasson        Redefined the debug_pcb structure.
**                                 zzget_parent() was introduced.
** 97/09/20  Bengt Eliasson        The sysctlblk struct was extended to level 2.
** 97/10/21  Bengt Eliasson        The sysctlblk struct was extended to level 3.
** 98/06/26  Fredrik Bredberg      Changed the type for zzldmvarp.
** 98/06/26  Bengt Eliasson        The zzcall_mem_move() call now returns a
**                                 value.
**                                 The new enable_remcalls flag was added
**                                 to the sysctlblk struct.
**                                 Interrupt process profiling is now
**                                 supported using the zzset_inthook_status(),
**                                 zzintswap_in() and zzintswap_out() calls.
**                                 Swap and tick hooks for the profiler was
**                                 also added.
** 98/09/07  Bengt Eliasson        Fixed a problem which caused the kernel to
**                                 generate a few bytes of initialized data.
** 98/10/11  Bengt Eliasson        Added the zzset_tickhook_ldmstatus()
**                                 function to support sampled cpu load rofiling.
** 99/01/11  Bengt Eliasson        Added a pid parameter to zzkill_context().
** 99/10/22  Bengt Eliasson        The zzinit_int() function now returns with
**                                 interrupts disabled, as opposed to using
**                                 LOCK_PUSH().
** 00/05/25  Fredrik Bredberg      Added support for a get_envp() cache, 
**                                 also the zzinit_pflib() hook.
** 01/04/10  Fredrik Bredberg      Added a const to zzset_regs()'s and
**                                 zzunpack_regs()'s cpuregs parameters.
** 01/10/10  Fredrik Bredberg      Added the zzcall_power_save() call.
** 03/10/01  Gunnar Eklund         Added the zzget_register() call.
** 04/01/15  Patrik Stromblad      Added per process data support.
** 04/01/19  Gunnar Eklund         Added zzlock_save() and zzlock_restore()
** 04/02/24  Gunnar Eklund         Changed zzfind_first_bit_set return type
**                                 from U32 to U8 to get faster code in pose.
** 04/05/24  Fredrik Bredberg      Added mutex support.
** 04/07/12  Gunnar Eklund         Added zzget_file_line() for paging.
**                                 Added zzcall_kernel_halted().
** 04/11/09  Fredrik Bredberg      Added config support for timeslicing of
**                                 prioritized processes.
** 05/12/28  Gunnar Eklund         Changed mutex control block to solve 
**                                 problems with mutexes in uninitiated ram.
** 06/02/20  Gunnar Eklund         Added prototypes zzticker_semwakeup,
**                                 zzfind_current_debug_info, create_event
**                                 and zzsignal_event.
** 06/02/22  Fredrik Bredberg      Added prototypes zzkrn_get_pid_info and
**                                 zzkrn_send_hook_free_buf.
** 06/04/24  Fredrik Bredberg      Added a get/set_mem helper function
**                                 called zzkrn_pre_get_set_mem.
** 06/04/24  Gunnar Eklund         Added zzkrn_get_handler and 
**                                 zzkrn_set_handler to control profiling.
**                                 Added zzkrn_get_time_resolution and 
**                                 zzkrn_set_time_resolution.
** 06/09/18  Fredrik Bredberg      Added ose_smp.h include file.
** 06/10/23  Fredrik Bredberg      Added SMP locking prototypes.
** 07/02/20  Petter Ericson        Added prototype for the zzbind_dbghook()
**                                 debugger hook.
**                                 Added 'enable bind hook' bit to field
**                                 debug_flags of struct debug_pcb.
** 07/03/09  Gunnar Eklund         Added zzget_dp_look_aside(cpu) for SMP.
** 07/03/16  Fredrik Bredberg      Added prototype for zzkrn_send_hook_restore.
**                                 Also added SMP functions for cur_p and
**                                 dispatcher lock handling.
** 07/03/20  Gunnar Eklund         Moved zzfind_current_debug_info() to
**                                 another core header file.
** 07/07/20  Fredrik Bredberg      Added zzkrn_dbg_intercept.
** 07/09/07  Fredrik Bredberg      Added zzupdate_err_handl_level.
** 07/12/27  Gunnar Eklund         Added zzguarded_memcpy.
** 08/02/26  Robert Andersson      Added zzset_manager_mode and
**                                 zzget_manager_mode.
** 08/03/19  Gunnar Eklund         Added text for zzkrn_set_handler.
** 08/04/15  Fredrik Bredberg      Added kill_lock()/_unlock() function
**                                 prototypes.
** 08/04/23  Gunnar Eklund         SMP lock infrastructure.
** 08/06/10  Fredrik Bredberg      Added kill_curr_at_unlock().
** 08/09/05  Gunnar Eklund         Added zzkrn_set_semaphore_annotation.
** 08/09/09  Anders Hagstrom       Added zzkrn_alloc_kernelstack.
** 08/09/10  Fredrik Bredberg      Added zzkrn_mutex_init().
** 08/10/03  Jonas Westaker        Added zzkrn_evt_notify() used for
**                                 SMP kernel event handling support.
** 08/10/21  Johan Ekberg          Added zzreceive_tmo_dbghook().
** 08/11/13  Fredrik Bredberg      Added zzkrn_mutex_destroy().
** 08/11/19  Fredrik Bredberg      Added zkrn_bkl_lock_id() for SMP.
** 08/12/02  Fredrik Bredberg      Added new parametes to zzintswap_in/out,
**                                 also rewrote zzset_inthook_ldmstatus()'s
**                                 description.
** 09/02/20  Gunnar Eklund         Added zzkrn_activate_cpu().
** 09/04/15  Gunnar Eklund         Added cache_line_size in struct sysctlblk.
** 09/04/17  Fredrik Bredberg      Added zzkrn_save-/restore-_pcb_state().
** 09/09/24  Fredrik Bredberg      Added zzshutdown_cpu().
** 09/10/11  Robin Westberg        Added ISR related functions.
** 09/11/06  Robin Westberg        Added zzkrn_evt_notify_from_isr.
** 10/04/19  Gunnar Eklund         Added zzkrn_stop_cpu().
** 10/10/29  Anders Hagstrom       Added kernel event prototypes.
** 11/04/12  Gunnar Eklund         Added private block state hooks for use
**                                 by the hypervisor.
** 12/05/14  Gunnar Eklund         Reorganized struct debug_pcb for ARM to
**                                 increase performance.
** 12/09/11  Gunnar Eklund         Added zzsystem_error_dbghook().
** 12/11/26  Gunnar Eklund         Added an argument to zzsystem_error_dbghook.
** 13/01/30  Gunnar Eklund         Added configuration options for disposition
**                                 of queued signals in killed pools.
** 13/08/15  Fredrik Bredberg      Changed struct _pcb_state_buf.
*/

/*
*****************************************************************************
** 3  BLOCKWIDE DEFINITIONS.
*****************************************************************************
*/

/*
**===========================================================================
** 3.1  Manifest constants.
**===========================================================================
*/


/*
**===========================================================================
** 3.2  Type definitions.
**===========================================================================
*/

#include <stddef.h>			/* ANSI C compiler library. */

/*
** We need some OSE types, but not the debugger version.
*/

#ifdef OS_DEBUG
#undef OS_DEBUG
#endif
#include "ose.h"

/*
** SMP specific definitions.
*/
#ifdef OSE_SMP
#include "cpu_smp.h"		/* FIXME need OSE_SMP_MAX_CPUS */

#define SMP_ARRAY [OSE_SMP_MAX_CPUS]
#define SMP_INDEX [zzkrn_cpu_id()]
#define SMP_INDEX_X(x) [x] 
#define NOT_SMP_CODE(x)
#define SMP_CODE(x) x
#define SMP_CODE_YES_NO(x,y) x
#else
#define SMP_ARRAY
#define SMP_INDEX
#define SMP_INDEX_X(x)
#define NOT_SMP_CODE(x) x
#define SMP_CODE(x)
#define SMP_CODE_YES_NO(x,y) y
#endif

/*
** Additional elementary types.
*/
#include "stdint.h"
#ifndef LOCKSTATE_CONFIGURED
#define LOCKSTATE_CONFIGURED
typedef unsigned long LOCKSTATE;
#endif
#ifndef _U32
#define _U32
#ifndef _OSETYPES_H
typedef uint8_t U8;
typedef uint16_t U16;
typedef unsigned long U32;
#endif
#endif


/*
** Structure containing the system control block. This structure is used in
** the zzget_sysctlblk() function.
** This type does probably not need any modification due to porting
** considerations.
**
** The "(Def=0)"-comments indicate suitable default values when the fields
** are not implemented for end user configurability. Fields without this
** comment must be user configurable.
*/

struct sysctlblk
{
    OSADDRESS pool_base;		/* Lowest address in the pool. */
    OSADDRESS pool_size;		/* Number of bytes in the pool. */
    OSADDRESS sigsize_tab[8];		/* Default signal size table. */
    OSADDRESS stacksize_tab[8];		/* Default stack size table. */
    OSADDRESS tracesize;		/* Number of bytes reserved for the trace in the pool. */
    OSADDRESS superv_stack_size;	/* Extra space for superv processes. */
    OSADDRESS max_procs;		/* Contains max number of allowed processes. (Def=4096) */
    OSADDRESS max_attached;		/* Contains max number of attached signals. (Def=4096) */
    OSADDRESS tick_len;			/* Tick length in microseconds. */
    OSADDRESS resident_pri;		/* Virtual mem disabled at this pri and up. (Def=25) */

    OSADDRESS allow_user_intproc;	/* Nonzero to allow user mode proc to create intprocs. (Def=1) */
    OSADDRESS allow_alloc_nil;		/* Nonzero to allow alloc to return NIL upon error handler exit for non sys pools. (Def=1) */
    OSADDRESS skip_stack_allocation;	/* Nonzero to pass NULL in the stack to zzcreate_context(). (Def=0) */
    OSADDRESS linkhandler_timeout;	/* Nonzero enables linkh response timeout error and sets the time (sek). (Def=10) */
    OSADDRESS allow_pool_copy;		/* Nonzero enables intersegment copying when sending between pools. (Def=0) */
    OSADDRESS table_extended;		/* Always 9. Indicates the present table extension level. */

    /* The following fields are present only if table_extended >= 1. */
    OSADDRESS ldm_priority;		/* The priority of the ose_ldm/ process. (Def=1) */
    OSADDRESS huntd_priority;		/* The priority of the ose_huntd process. (Def=10) */
    OSADDRESS main_priority;		/* The priority of the zzmain process. (Def=16) */
    OSADDRESS sysd_priority;		/* The priority of the ose_sysd process. (Def=20) */
    OSADDRESS enable_get_pcb_regs;	/* Nonzero to enable regs in get_pcb(). (Def=1) */

    /* The following fields are present only if table_extended >= 2. */
    OSADDRESS tickd_priority;		/* The priority of the ose_tickd process. (Def=31) */

    /* The following fields are present only if table_extended >= 3. */
    OSADDRESS bgd_timeslice;		/* The default timeslice for background processes. (Def=10) */

    /* The following fields are present only if table_extended >= 4. */
    OSADDRESS enable_remcalls;		/* Flag to enable remote system calls. (Def=1) */

    /* The following fields are present only if table_extended >= 5. */
    OSADDRESS max_ppd_keys;		/* Max number of per process data key strings. */
    OSADDRESS max_ppd_key_size;		/* Max number of characters in key string. */

    /* The following fields are present only if table_extended >= 6. */

    /*
    ** This member is a bit mask. Assigned bit masks are:
    ** 1: Don't warn for user blocks created in the monolith.
    */
    OSADDRESS relax_error_checks;	/* Bit mask */
    OSADDRESS zzmain_stacksize;		/* zzmain stacksize. */

    /* The following fields are present only if table_extended >= 7. */
    OSADDRESS timeslice_priprocs;	/* Nonzero enables timeslicing of prioritized processes. */

    /* The following fields are present only if table_extended >= 8. */
    OSADDRESS tmo_array_length;		/* The length of the */

    /* The following fields are present only if table_extended >= 9. */
    OSADDRESS pool_fill;      /* Controls whether to fill the pool with a debug pattern. */
    
    /* The following fields are present only if table_extended >= 10. */
    OSADDRESS cache_line_size;
    
    /* The following fields are present only if table_extended >= 11. */
    OSADDRESS killed_pool_signal_action; /* default, keep, relocate, kill */
    OSADDRESS killed_pool_signal_action_delay; /* in ms. */
}; /* struct sysctlblk */

/*
** Structure containing the user space mutex control block.
*/
struct _mutex
{
   U16                magic;
   U16                instance;
   U32                attr;
   volatile PROCESS   owner;
   volatile PROCESS   fast_lock;
   U32                count;
   U32	              index;
};

/*
** Structure containing the state that needs to be saved during a
** pagefault.
*/

#define MAGIC_SAVED_PCB_STATE 0x5FCB5718
struct _pcb_state_buf
{
  SIGSELECT  magic;
  OSTICK     tmo_fragment;
  U16        status;  /* pr_state_t */
  OSBOOLEAN  tmo_active;
};

/*
** Structure containing a cache element for the get_envp() cache.
** The cache should be implemented in the c-library (i.e.in pflib.c).
*/

struct envp_cache
{
    PROCESS   owner;
    OSADDRESS value;
};

/*
** Kernel Function structure made readable to user via sys_conf53 syscall.
** It contains pointers to reentrant syscalls possible to call from user 
** space. 
*/

typedef  OSADDRESS (*PPDATA_FUNC)(void);
typedef  PROCESS   (*CURRPID_FUNC)(void);
typedef  OSTICK   (*GET_TICKS_FUNC)(void);
typedef  OSTIME   (*SYSTEM_TICK_FUNC)(void);
typedef  int      (*WAKE_UP_FUNC)(void);

/*
** NOTE: when adding a function here, don't forget to increase
** USRCALLS_COUNT below.
*/

struct krn_usrcalls
{
  OSADDRESS         count; /* used by crt to check kernel version. */
  PPDATA_FUNC       ppd_addr_func;
  CURRPID_FUNC      curr_proc_func;
  GET_TICKS_FUNC    get_ticks_func;
  SYSTEM_TICK_FUNC  system_tick_func;
  WAKE_UP_FUNC      wake_up_func;
};

/*
** Used to pass parameters during creation of ISRs and inthooks.
*/
struct isr_create_data
{
  PROCESS owner;
  void *arg;
};

/*
**===========================================================================
** 3.3  Macros.
**===========================================================================
*/

#define USRCALLS_COUNT  5	/* Supports 5 functions above. */

/*
** Defined subcommands for ose_sysconf_53 system call, private for now.
*/

#define PFLIB_CREATE_PPD_KEY        0x0UL
#define PFLIB_SET_PPD_PTR           0x1UL
#define PFLIB_GET_PPD_PTR           0x2UL
#define PFLIB_GET_PPD_KEYS          0x3UL
#define PFLIB_GET_PPD_KEY_STRING    0x4UL
#define PFLIB_GET_MAX_PPD_KEYS      0x5UL
#define PFLIB_GET_MAX_PPD_KEY_LEN   0x6UL
#define PFLIB_ILLEGAL_PPD_KEY       0x7UL

#define MAGIC_MUTEX           (0x4d75) /* "Mu" */
#define MAGIC_MUTEX_DESTROYED (0x4d64) /* "Md" */

/*
*****************************************************************************
** 4  EXTERNAL REFERENCES.
*****************************************************************************
*/

/*
**===========================================================================
** 4.1  Functions.
**===========================================================================
*/

/*
**===========================================================================
** 4.1.1  Functions implemented by the target interface.
**===========================================================================
*/

/*
** Run power off handler and shut down the system.
** Does not return if a power off handler is present.
** Returns immediately if handler is not present.
*/

void zzpower_off(void);

/*
** Run the power save handler which may shut down the system for
** the stated maximum time.
** If max_time is -1 the power save handler may sleep for as long
** as it wants.
** Returns the actual time the system was sleeping.
** Returns immediately (with value 0) if no power save handler is
** present.
**
** Interrupts are disabled during the call to zzcall_power_save().
*/

OSTICK
zzcall_power_save(OSTICK max_time);

/*
** Set the global interrupt stack pointer.
** This stack is shared among all interrupt processes.
** When a non interrupt process is interrupted, the current stack pointer
** is saved, and the interrupt stack is loaded before control is passed
** to the interrupt process.
** Istack_base points at the lowest address within the stack area.
** size is the number of bytes reserved for the interrupt stack.
*/

void
zzset_interrupt_stack(OSADDRESS istack_base, OSADDRESS size);

/*
** Create a uarea and set up all fields. The process should then be ready to
** swap in with zzrestore_context(). Return a pointer to a uarea allocated
** from zzalloc_kernelmemory(). This uarea is used to keep process specific
** info managed by the target interface. For interrupt or timer processes,
** the stack, stack_size, entrypoint and super_mode parameters are invalid,
** but a uarea may still be required.
** Implementation specific parameters should be propagated to
** the new process, which in most cases means copying them to the
** stack of the new process.
** User mode processes may need a second stack in supervisor space. This stack
** is allocated and managed by the target interface.
** The newpid parameter contains the process id of the new process.
** The parent parameter contains the process id of the parent process or zero
** if not available.
** The clib_handle parameter contains the clib handle of the block in which the
** new process resides, or NULL if no pointer is available.
**
** Returns pointer to the new uarea, or NULL if not needed.
*/

void *zzcreate_context(void *stack, OSADDRESS stack_size, void (*entrypoint)(void),
OSBOOLEAN super_mode, OSBOOLEAN interrupt_type, PROCESS newpid, PROCESS parent,
void *clib_handle);

/*
** Kill a uarea and free it to special free list.
** The uarea parameter contains a pointer to the uarea, or NULL if the process did
** not have a uarea.
** The oldpid parameter contains the process id of the killed process.
*/

void zzkill_context(void *uarea, PROCESS oldpid);

/*
** Push CPU-registers into uarea and jump to restartable dispatcher at
** the zzgo_search symbol.
** Also call swap out handlers if present.
** Interrupts are enabled when this function is called.
** Interrupts are disabled when returning after a subsequent
** process switch.
*/

void zzsave_context(void);

/*
** Pop CPU-registers from uarea.
** Also call swap in handler if present.
** Interrupts are enabled when this function is called.
** Interrupts are disabled when it returns to the new process.
*/

void zzrestore_context(void);

/*
** Associate an interrupt vector with an OSE interrupt
** process. Returns False if the given vector number was unusable.
*/
OSBOOLEAN
zzbind_vector_pcb(OSVECTOR vecno, void *pcb);

/*
** Release an interrupt vector from the associated OSE interrupt
** process.
*/
void
zzrelease_vector_pcb(OSVECTOR vecno, void *pcb);

/*
** Make create_isr visible to the target interface.
*/
OSISRHND
zzkrn_create_isr(OSVECTOR vec, PROCESS owner, OSISR *isr, void *isr_arg);

/*
** Associate an interrupt vector with an ISR or interrupt hook.
** Returns False if the given vector number was unusable.
** 
** Interrupts are enabled when this function is called, but they may be
** temporarily disabled as required.
*/
OSBOOLEAN
zzbind_vector_isr(OSVECTOR vecno, void *isr, uint32_t isr_arg, uint32_t type);

/*
** Release an interrupt vector from the associated ISR or interrupt hook.
** 
** Interrupts are enabled when this function is called, but they may be
** temporarily disabled as required.
*/
void
zzrelease_vector_isr(OSVECTOR vecno, void *isr, uint32_t isr_arg, uint32_t type);

/*
** Programs the interrupt controller to stop generating interrupts on
** the given vector.
*/
void
zzdisable_vector(OSVECTOR vecno);

/*
** Programs the interrupt controller to start generating interrupts on
** the given vector, with the selected hardware priority.
*/
void
zzenable_vector(OSVECTOR vecno, OSPRIORITY pri);

/*
** Simulate a hardware interrupt to the selected process.
** This function is called when a prioritized process requests a wakeup call and
** when an interrupt process requests a wakeup call to an interrupt process
** with a higher priority.
** The wakeup parameter contains the value that should be passed to zzinit_int.
**
** Interrupts are disabled when called from a non interrupt level.
** Interrupts are disabled to the current interrupt level (0-31) when called
** from another interrupt process.
** The current interrupt status register can not be used by the target interface.
** Instead, the current interrupt priority must be determined from the current
** interrupt priority variable in the target interface.
** The current interrupt status register need not be saved by the target interface,
** i.e. a status register corresponding to the current interrupt priority variable
** should be stacked instead.
*/

void zzenter_wakeup(void *uarea, void *pcb, OSPRIORITY pri, OSBOOLEAN wakeup);

/*
** Call an interrupt process as a subroutine in supervisor space from supervisor space,
** using the current supervisor stack. This call is used to execute interrupt processes.
** Must save registers required by the kernel compiler.
*/

void zzcall_intproc(void (*entrypoint)(void));

/*
** Call an error handler as a subroutine in user space from supervisor space,
** using the specified user stack. If the stack parameter is zero, then use the
** current user stack instead.
** The clib_handle is the pointer in the bcb previously stored by zzmain using
** the set_clib_pointer() system call. This pointer points to one or several
** intermediate functions within the current C library.
** This call is used to execute error handlers.
** Must save registers required by the kernel compiler.
*/

OSADDRESS zzcall_user_errh(
   OSADDRESS (*entrypoint)(OSBOOLEAN a, OSERRCODE b, OSERRCODE c),
   OSBOOLEAN user_called, OSERRCODE ecode, OSERRCODE extra, void *stack,
   OSADDRESS stacksize, void *clib_handle);

/*
** Call an error handler as a subroutine in supervisor space from supervisor space,
** using the specified supervisor stack. If the stack parameter is zero, then use
** the current supervisor stack instead.
** The clib_handle is the pointer in the bcb previously stored by zzmain using
** the set_clib_pointer() system call. This pointer points to one or several
** intermediate functions within the current C library.
** The clib_handle is invalid when an early error occurs in one of the
** system daemons. This is indicated by a NULL pointer. In this case, it is
** assumed that the clib handle in the main C library can be found in some
** other way.
** This call is used to execute error handlers.
** Must save registers required by the kernel compiler.
*/

OSADDRESS zzcall_super_errh(
   OSADDRESS (*entrypoint)(OSBOOLEAN a, OSERRCODE b, OSERRCODE c),
   OSBOOLEAN user_called, OSERRCODE ecode, OSERRCODE extra, void *stack,
   OSADDRESS stacksize, void *clib_handle);

/*
** Call the system error handler as a subroutine in supervisor space from
** supervisor space, using the current supervisor stack.
** The stack is the supervisor stack of the current process.
** Stack requirement of the system error handler must be
** reserved on each supervisor stack.
** Must save registers required by the kernel compiler.
*/

OSADDRESS zzcall_system_errh(OSBOOLEAN user_called,
   OSERRCODE ecode, OSERRCODE extra);

/*
** Call the kernel halted handler as a subroutine using the current supervisor 
** stack. The stack is the supervisor stack of the current process.
** Interrupts are disabled by the caller. The handler usually makes a board reset.
** Must save registers required by the kernel compiler if it does return.
*/

void zzcall_kernel_halted(void);

/*
** Call memory management functions in supervisor space using
** the current supervisor stack.
** Must save registers required by the kernel compiler unless they are
** preserved by the customer compiler.
*/

void zzcall_attach_block(PROCESS pid, OSSEGMENT segid);

OSBOOLEAN zzcall_mem_move(
             OSSEGMENT src_segment,
             OSADDRESS src_begin,
             OSADDRESS size,
             OSSEGMENT dest_segment,
             OSADDRESS dest_begin);

void zzcall_select_segment(OSSEGMENT segnum);

OSBOOLEAN zzcall_set_segment_mode(OSSEGMENT segid, OSBOOLEAN contains_int);

/*
** Examine the file and line information for the specified process.
** This function stores line number as an integer into the lineno parameter,
** and the file name into the filename parameter.
** Lineno should be set to zero if file and line info is not available.
** The configurable constant MAX_FILE_NAMELEN contains the worst case number
** of characters allowed. The filename must be truncated if required.
*/

void zzget_filename(void *uarea, OSADDRESS *lineno, char *filename);

/*
** Examine the file and line information for the specified process.
** This function stores line number as an integer into the lineno parameter,
** and the file name pointer into the filename_pointer parameter.
** Lineno should be set to zero if file and line info is not available.
**
** The dispatcher is locked by the caller.
**
** This function is required only for paging systems, and only called
** when ZZGET_FILE_LINE is configured to non-zero. 
*/

void zzget_file_line(void *uarea, OSADDRESS *lineno, char **filename_pointer);

/*
** Examine the current register set for the specified process and return in
** cpuregs a binary image of the registers along with flags indicating the
** current register availability.
** The cpuregs image is later passed to zzget_regs() for reformatting.
** The configurable constant MAX_REGLEN contains the size of the
** cpuregs area.
** The dispatcher is locked while this function is called.
*/

void zzcapture_regs(void *uarea, char *cpuregs);

/*
** Reformat the binary register set for the specified process and return in
** cpuregs a string describing the registers.
** The cpuregs array contains the image previously stored by zzcapture_regs().
** The resulting ascii string should also be in cpuregs, which must be
** accounted for in the algorithm. It may for instance be wise to keep the
** binary image at the tail while building the string from the head of the
** buffer.
** The configurable constant MAX_REGLEN contains the worst case number
** of characters allowed.
** The dispatcher is unlocked while this function is called, which means
** that the function must be reentrant.
**
** If the USE_CAPTURE_REGS flag in pose.con is zero, then zzcapture_regs()
** is not called, and zzget_regs() is assumed to perform the combined
** task of these calls. The dispatcher is locked in this case.
*/

void zzget_regs(void *uarea, char *cpuregs);

/*
** Unpack regstring containing register names and values to a binary format
** recognized by the zzset_regs() function.
** The string format corresponds to the format used in zzget_regs, except
** registers may be in any order and not all registers need be present.
** The result is stored in the cpuregs array, which contains MAX_REGLEN bytes.
** The cpuregs array is subsequently passed without modification to zzset_regs().
**
** Return a nonzero value if a syntax error was encountered in the
** input (regstring).
**
** The dispatcher is not locked during this call.
**
** The zzunpack_regs() function is called only if the USE_CAPTURE_REGS flag
** in pose.con is non zero. If it is zero, then zzset_regs() is assumed to
** perform the combined function of zzunpack_regs() and zzset_regs().
** In this case, cpuregs contains the input ascii string.
*/

OSBOOLEAN zzunpack_regs(char *cpuregs, const char *regstring);

/*
** Modify the current register set for the specified process if the process
** is in a state where this is possible.
** Return a nonzero value if the call failed.
** Cpuregs contains a binary image describing the registers to be set.
** The image was created by zzunpack_regs().
**
** The dispatcher is locked during this call.
**
** The zzunpack_regs() function is called only if the USE_CAPTURE_REGS flag
** in pose.con is non zero. If it is zero, then zzset_regs() is assumed to
** perform the combined function of zzunpack_regs() and zzset_regs().
** In this case, cpuregs contains the input ascii string.
*/

OSBOOLEAN zzset_regs(void *uarea, const char *cpuregs);

/*
** Get address to system control block.
**
** The system control block contains all runtime configurable system parameters.
** These parameters are made known to the kernel through this function.
**
** Returned values are:
**	Kernel pool base address, aligned for any type.
**	Kernel pool size.
**	List of kernel signal sizes.
**	List of kernel stack sizes.
**	Trace buffer size.
**	
*/

struct sysctlblk *
zzget_sysctlblk(void);

/*
** Disable interrupts and return old interrupt status.
** zzlock_save + zzlock_restore may be faster than zzlock_push + zzlock_pop
** because an auto variable may be used. The semantics does however differ 
** for nested operations.
*/

#ifdef OSE_SMP
LOCKSTATE zzbkl_lock_save(void);
#endif
LOCKSTATE zzlock_save(void);

/*
** Restore saved interrupt status.
*/

#ifdef OSE_SMP
void zzbkl_lock_restore(LOCKSTATE saved_state);
#endif
void zzlock_restore(LOCKSTATE saved_state);

/*
** Disable interrupts without saving old value.
*/

#ifdef OSE_SMP
void zzbkl_lock(void);
#endif
void zzlock(void);

/*
** Enable interrupts unconditionally.
*/

#ifdef OSE_SMP
void zzbkl_unlock(void);
#endif
void zzunlock(void);

/*
** Unlock the SMP BKL but keep interrupts disabled.
*/

#ifdef OSE_SMP
#define BKL_UNLOCK_KEEP_INT_MASK() zzbkl_unlock_keep_int_mask()
void zzbkl_unlock_keep_int_mask(void);
#else
#define BKL_UNLOCK_KEEP_INT_MASK() ((void)0)
#endif

/*
** These functions are called in SMP kernel's and can be used to
** implement a mutual exclusion scheme for the the separate schedulers
** (one per cpu).
*/

#ifdef OSE_SMP
void zzsmp_lock_swap(void);
void zzsmp_unlock_swap(void);
cpuid_t zzkrn_bkl_lock_id(void);
#endif


/*
** Cold start entrypoint in the target interface.
**
** This is where the target interface is entered during system start.
** There are two ways to end up here.
** If "System start" option is selected, the reset vector should point
** directly to this entrypoint.
** If "User start" option is selected, this entrypoint is called from the
** start_OSE() function in the portable interface library.
*/

void zzstart_OSE(void);

/*
** Set a breakpoint in the currently selected memory segment.
**
** Interrupts are enabled while this function is called, but they may
** be temporarily disabled when required.
**
** The uarea parameter points to the uarea of the process in which the
** breakpoint was set, or it contains NULL if the breakpoint was set
** for several processes. The intention is that breakpoint hardware 
** contents can be part of each process context when possible.
** 
** Return a nonzero value if the breakpoint could not be set.
*/

OSBOOLEAN zzset_breakpoint(OSADDRESS address, OSADDRESS attribute,
			   void *breakpoint_id, void *uarea);

/*
** Clear a breakpoint in the currently selected memory segment.
**
** Interrupts are disabled by zzlock_push() while this function is called.
**
** The uarea parameter points to the uarea of the process in which the
** breakpoint was set, or it contains NULL if the breakpoint was set
** for several processes.
** 
** return a nonzero value if the breakpoint is unknown.
*/

OSBOOLEAN zzclear_breakpoint(void *breakpoint_id, void *uarea);


/*
** Find first bit set in a 32 bit word.
**
** This function is used by the dispatcher and wakeup functions to search
** look aside flag words.
** The function should search a 32 bit word for the most significant bit set.
** Bit 31 should be found before bit 0.
**
** Return the result of the expression "31 - the found bit number",
** i.e bit 31 set should return the value 0, bit 30 set should return the
** value 1 and so on.
** If no bits are set, the value 32 should be returned.
*/

U8
zzfind_first_bit_set(U32 flagword);

/*
** Return a pointer to a string describing the current CPU.
** This function is used in the get_cpu() system call.
** It is assumed that this string is a constant that can be concurrently
** accessed by several processes.
*/

const char *
zzget_cpu_string(void);

/*
** Read the microsecond clock.
**
** The microsecond clock should be syncronous to the system clock such
** that the system tick is directly derived from the microsecond clock.
** The microsecond clock should NOT be automatically cleared by harware
** when a system tick interrupt is generated. It must continue to run
** until updated with a zzclear_microticks() call from the kernel.
** The microsecond clock is best implemented as a free running counter
** with a 1 MHz clock source, combined with an offset variable in RAM.
**
** If no clock source is available, the zzget_microticks() function
** must return a new value each time it is called, that is it should
** then be implemented as a variable incremented with each read operation.
**
** Interrupts are disabled during the call to zzget_microticks().
*/

U32
zzget_microticks(void);

/*
** Clear the microsecond clock.
**
** There are several ways to implement this function. Which to choose depends
** on the design of the microsecond clock hardware.
**
** If the clock is implemented as a free running counter with an output
** compare register that can cause interrupts then set the offset variable
** in ram to the old value of the output compare register and add one
** system tick period to the output compare register. Note the problem that
** occurs if the timer interrupt execution time exceeds the system tick
** length. If this happens, a system tick must be dropped to regain stability.
** This implementation yields precise system ticks and a microsecond clock
** that is always consistent with the current system tick counter in the kernel.
** It is the best implementation.
**
** If the clock is implemented as a free running counter that is not related to
** the system clock timer, then set the offset variable in ram to the current
** value of the free running counter. This implementation causes temporary
** inconsistencies between the microsecond clock and the system tick counter
** in the kernel. It appears that the time is sometimes moving backwards.
** This implementation is less desirable, but it works.
**
** If the clock is implemented as a timer that stops counting and restarts
** automatically or by software upon each interrupt then just allow it to
** restart. This implementation too causes periodic timer inconsistencies.
**
** Interrupts are disabled during the call to zzclear_microticks().
*/

void
zzclear_microticks(void);

/*
** Idle loop check routine.
** This function is called by the dispatcher when no process is ready.
** It should return whenever an interrupt has occurred.
**
** Interrupts are disabled when this function is entered.
** This is to allow for flag operations within the target interface.
** Interrupts should be reenabled before waiting for an interrupt.
** (The kernel performs an extra UNLOCK() operation in case the
**  target interface doesn't.)
**
** A number of implementations are possible:
**
** A) Do nothing, just return. This is a typical implementation.
**
** B) Soft kernels with polled interrupt detection need to UNLOCK() and
**    scan for interrupts. Return after the first interrupt encountered.
**
** C) On processors with a "wait for interrupt" instruction we need to:
**    1. Set an internal "waiting for int" flag.
**    2. UNLOCK().
**    3. Execute the "wait for interrupt" instruction.
**    4. Clear the internal "waiting for int" flag and return.
**    We must also test on the "waiting for int" flag after each call
**    to zzinit_int that returns 0.
**    If the flag is set, we must clean up the stack as if a
**    normal return from interrupt was made, but we must then jump
**    to step 4 above instead of returning to user code.
**    This procedure fixes the race condition that would otherwise
**    cause the system to miss a process switch when an interrupt
**    occurs immediately before the "wait for interrupt" instruction
**    is executed.
*/

void
zzidle(void);

/*
** The system debugger reads the names of all system processes.
**
** This function is called by the kernel or the ldm during system start
** to obtain a list of the names of all processes that must be present
** for the debugger to operate correctly.
** These processes will never be stopped by the debugger.
**
** Typical system processes are:
**	Linkhandlers used for debugging.
**	The debugger process itself.
**
** Returns the number of process name strings stored in the names pointer.
** The names pointer is updated to contain a pointer to a char array.
** This char array contains packed NUL terminated strings.
**
** The names are typically extracted from a user modified configuration
** file.
*/

U32
zzget_sysprocnames(char **names);

/*
** The system debugger reads the initial status of the breakpoint scope.
**
** This function is called by the kernel or the ldm during cold system start
** to obtain the expected status of non system processes.
** Scope status is typically provided by a user handler configured into
** the target interface. The handler reads a hardware strap or a memory
** flag managed by the handler.
**
** Returns 0 if application processes should be allowed to run.
** Returns non zero if application processes should be immediately intercepted
** when they are created, allowing the system debugger to examine the
** system before it runs.
*/

OSBOOLEAN
zzget_scope_status(void);

/*
** The system debugger enables and disables interrupt process swap hooks.
** The call is needed only when using the LATE_INIT_INT implementation style.
**
** This call is issued whenever cpu load profiling is enabled or disabled
** in the system debugger.
**
** The enable_flag parameter should be copied to local storage in the target
** interface, and used to enable subsequent calls to zzintswap_in() and
** zzintswap_out() whenever it is nonzero.
**
** The zzset_inthook_status() function is called only if the
** ENABLE_INTERRUPT_PROFILING flag in pose.con is non zero.
** Otherwise, the function need not be implemented.
*/

void
zzset_inthook_status(U32 enable_flag);

/*
** This function should update the two elements in an envp_cache 
** structure in one atomic operation. I.e. the target interface
** must implement this by the use of some kind of "move multiple"
** CPU instruction (or similar).
** 
** This function is called from set_envp() and get_envp().
**
** The zzupdate_envp_cache() function is called only if the
** ENVP_CACHE_SIZE flag in pose.con is non zero.
** Otherwise, the function need not be implemented.  
*/

void
zzupdate_envp_cache(struct envp_cache *ecp, PROCESS owner, OSADDRESS value);

/*
** This function is called by efmi to read process specific registers.
** Interrupts are locked out by the caller. The provided uarea is
** an existing, prioritized or background process, and never the current
** process.
** 
** This function shall check if the specified register is available.
** If so, store it in the value argument and return zero.
** If the register is not available, return nonzero.
**
** is_syspool_pointer is a function that should be used to validate
** system pool pointers that are FOLLOWED to extract the current register
** value. The function returns non-zero if the pointer is a pool-aligned
** system pool pointer. The uarea pointer itself is already verified by 
** the caller. The function pointer is passed as an argument mainly to 
** inhibit explicit link-time references from ti into efmi.
*/

OSBOOLEAN
zzget_register(void *uarea, U32 reg_id, U32 *value,
	       OSBOOLEAN (*is_syspool_pointer)(const void *vp));

/*
** This function is called by the kernel when the the current process
** enters (or exits) an error handler nesting level. This function is
** implemented in the target interface and controls that retrival of 
** CPU registers (for debug purposes) are made from the correct scope.
** Only transitions from or to error level 0 are reported by the kernel.
*/
void
zzupdate_err_handl_level(OSBOOLEAN new_err_handl_level);

/*
** Try to copy memory with current memory mapping. Return 0 if
** success, non-zero on failure. The freeze_mode_flag shall be
** zero for run mode, non-zero for freeze mode debug calls.
*/
int zzguarded_memcpy(void *dest, const char *source, size_t size,
		     int freeze_mode_flag);

/*
** This function administrates manager mode for the current process.
** When executing in manager mode memory access permissions are relaxed
** for specific manager mode regions. Manager mode is only available on
** CPUs which support MMU access attribute indirection.
**
** The function returns the previous manager mode state (enabled/disabled),
** as it was before the operation took place.
*/
OSBOOLEAN
zzset_manager_mode(OSBOOLEAN enable);

/*
** Function to determine manager mode state for a process.
** Returns true if the MMU exercises access attribute indirection
** for the process. Returns TRUE if manager mode is enabled,
** FALSE otherwise.
*/
OSBOOLEAN
zzget_manager_mode(void *uareap);

/*
**===========================================================================
** 4.1.2  Functions in the OSE kernel available to the target interface.
**===========================================================================
*/

/*
** Cold start entrypoint in the OSE kernel.
**
** This is where OSE is entered from the target interface in response to
** a previous hardware reset operation or a zzstart_OSE call from the
** interface library.
**
** The mode parameter indicates how system start should be performed.
** mode = 0. Cold start. We know this is a power on situation. The kernel
** will not examine the pool since that might cause parity errors or similar
** problems. The entire system pool will be cleared, all
** internal kernel variables are initialized, and the system is started.
** Debugger variables including the trace buffer will be lost.
** User processes are created.
**
** mode = 1. Warm start. This may be either a cold or a warm start. The kernel
** will examine the pool to determine if system parameters are still valid.
** If they are, a warm restart of the OSE debugger is performed.
** User processes are created.
** If system parameters are not valid, a cold start is performed instead.
**
** mode = 2. Hot start. The was previously shut down with a power_fail
** system call.  The memory is still valid and the kernel will immediately
** swap in the process which called power_fail();
*/

void zzstart_kernel(U32 mode);

/*
** Allocate space for uarea etc.
**
** This memory can be used for uarea and other areas required by the
** target interface.
** The memory can not be freed, so a separate free list must be
** maintained by the target interface for space allocated in this way.
** The memory is allocated from the bottom of the kernel pool (lower
** addresses).
** Returns a pointer to the base of new area.
** Can only be called from a non interrupt process.
*/

void *zzalloc_kernelmemory(OSBUFSIZE size);

/*
** Allocate space for supervisor stack.
**
** This memory should be used for supervisor stacks.
** The memory can not be freed, so a separate free list must be
** maintained by the target interface for space allocated in this way.
** The memory is allocated from the top of the kernel pool (higher
** addresses).
** Returns a pointer to the base of new stack.
** Can only be called from a non interrupt process.
*/

void *zzkrn_alloc_kernelstack(OSBUFSIZE size);

/*
** Enter an interrupt process from hardware int. Wakeup is set if interrupt
** was caused by a wakeup call. Oldint_level is used by OSE to determine if
** we are at lowest int level. It is set to 32 for the first call.
** Int_level contains the interrupt priority of the current interrupt.
** It is used by OSE to determine when wakeup calls to other interrupt processes
** are required.
** Returns with interrupts disabled.
** Returns 0 if a normal interrupt return should be performed.
** Returns 1 if a process switch is required.
** Returns 2 if a wakeup call to a lower priority interrupt is required.
** In this case, the interrupt process priority must be fetched from the
** kernel by reading the zzwakeup_level variable before enabling interrupts.
** zzinit_int() must be called for all interrupt processes except users
** int's. (Users int's may not call zzinit_int)(). Users int's must have
** higher priority than all OS int's and can never use any system calls.)
** All registers not preserved by the C compiler must have been pushed or
** saved in the uarea. If a special interrupt stack is used, then this
** stack must have been selected by the caller.
*/

OSBOOLEAN zzinit_int(void *pcb, OSBOOLEAN wakeup, OSPRIORITY int_level,
		     OSPRIORITY oldint_level);

/*
** Check if a reschedule is needed.
**
** Returns 0 if a normal interrupt return should be performed.
** Returns 1 if a process switch is required.
** Returns 2 if a wakeup call to a lower priority interrupt is required.
*/
OSBOOLEAN
zzkrn_int_dispatch(OSPRIORITY oldint_level);

/*
** Perform a wakeup call to a lower priority in response to a kernel request
** returned by either the zzinit_int() function or another zzdo_wakeup() call.
**
** parameters and return values are defined as for the zzinit_int() function.
*/

OSBOOLEAN zzdo_wakeup(OSPRIORITY int_level, OSPRIORITY oldint_level);

/*
** Breakpoint interrupt entrypoint.
**
** A process was caught in a breakpoint. The target interface is not required to
** understand if the correct process was caught. The kernel checks this.
**
** Returns with interrupts disabled by zzlock_push().
** Returns 0 if a normal interrupt return should be performed. Breakpoint was valid.
** Returns 1 if a process switch is required. Breakpoint was valid.
** Returns 2 if a wakeup call to a lower priority interrupt is required.
** Breakpoint was valid.
** Returns 4 if a normal interrupt return should be performed. Breakpoint was invalid.
** Returns 5 if a process switch is required. Breakpoint was invalid.
** Returns 6 if a wakeup call to a lower priority interrupt is required.
** Breakpoint was invalid.
*/

OSBOOLEAN zzbreakpoint_trap(OSPRIORITY int_level, OSPRIORITY oldint_level,
			     void *breakpoint_id);

/*
** Enter the dispatcher from lowest level interrupt if swap is needed.
** Returns when the caller is again swapped in. Hardware RETI must have
** been performed before the call is made. Interrupts are disabled when
** the call is made.
*/

void zzquit_int(void);

/*
** Called from the target interface for a newly created process.
** It should be called in the context of the new process before
** execution is passed to the process entrypoint.
**
** The purpose is to check if the dispatcher must be restarted
** due to an interrupt during the swap in phase.
**
** Interrupts must be disabled when this function is called.
** They are still disabled upon return, and must be enabled
** before control is transfered to the process entrypoint.
*/

void zzinit_process(void);

/*
** Lock the dispatcher.
**
** After this call has been executed, no further process switches
** will take place until the zzunlock_dispatcher() function is called.
** Any process switch requests are queued internally in the C kernel
** until that time.
**
** This call may be executed with or without interrupts enabled.
**
** This call may be used by the target interface to avoid process switches
** during the single step operations that are required in breakpoint
** processing. This single step operation can not be executed on an
** interrupt level, but it is still crucial that no other non interrupt process
** except the current is allowed to run.
*/

void zzlock_dispatcher(void);

/*
** Unlock the dispatcher.
**
** This call negates the effect of the zzlock_dispatcer() call.
** The call returns a flag set if a process switch is required, in
** which case the caller must execute the zzquit_int() function.
**
** Returns 0 if no process switch is required.
** Returns 1 if a process switch is required.
**
** This call must be executed with interrupts disabled, and it may be
** called only from a non interrupt process (bg or pri).
*/

OSBOOLEAN zzunlock_dispatcher(void);

/*
** Report an OSE detected error.
**
** Allows the target interface to report an error as being found by OSE.
** The return code is supplied by any error handler present.
** fatal = 0 for a warning, 1 for an error that can only be handled by the
** system error handler, 2 for a fatal kernel error.
** interrupts are disabled by zzos_error for fatal kernel errors.
** These errors do not return to the target interface.
*/

OSADDRESS zzos_error(OSBOOLEAN fatal, OSERRCODE ecode, OSERRCODE extra);

/*
** Dispatcher search phase.
** 
** This function is entered from zzsave_context() instead of returning to
** the calling function.
** It replaces the obsolete goto_func parameter to zzsave_context().
** zzgo_search may be entered with a jump instruction since it does not
** return.
*/

void zzgo_search(void);

/*
** The set_clib_pointer() function stores a pointer in the current block
** descriptor. This pointer contains any implementation specific information
** desired, but the main purpose is to obtain the address of an intermediate
** function in the current C library used when calling user mode error
** handlers.
** The set_clib_pointer() should be included in a trap interface as a
** system call. The call should be issued in the zzmain process
** as the first system call from the c-library instance linked to
** osemain.c. It is important that set_clib_pointer() is set before any error
** can occur.
** The clib handle is inherited to child blocks, so each block in the system
** has a valid clib pointer.
** The system block assumes the value of the first clib pointer set, so the
** clib pointer for the system block is not valid until the first zzmain
** process has been started.
*/

void set_clib_pointer(void *clib_handle);

/*
** The zzinit_pflib() function initializes data structures in the POSE
** provided part of the c-library (i.e. in pflib.c) . A call to
** zzinit_pflib() should be issued from zzinit_OSE(), normally it is
** called right after calling set_clib_pointer().
*/

void zzinit_pflib(void);

/*
** This is the zzticker_semwakeup call.
**
** It is called to transform a high priority timer interrupt into an 
** activation of tickd on a more suitable priority, calling tick(). This 
** could have been done by use of the public signal_fsem instead, but in 
** this way we avoid some overhead at each tick.
** 
** The call returns nothing useful.
*/

unsigned long
zzticker_semwakeup(void *);

/*
** This function provides the same functionality as the get_pid_info()
** system call. It's faster since it's not built upon get_sysconf()
** but it's only available to supervisor-mode clients linked to the
** kernel monolith.
*/
OSADDRESS
zzkrn_get_pid_info(PROCESS pid, OSADDRESS info_tag);

/*
** This function provides the same functionality as the
** send_hook_free_buf() system call. It's faster since it's not built
** upon get_sysconf() but it's only available to supervisor-mode
** clients linked to the kernel monolith.
*/
void
zzkrn_send_hook_free_buf(union SIGNAL *sig);

/*
** This function provides the same functionality as the
** send_hook_restore() system call. It's faster since it's not built
** upon get_sysconf() but it's only available to supervisor-mode
** clients linked to the kernel monolith.
*/
void
zzkrn_send_hook_restore(union SIGNAL *sig);

/*
** This is a helper function for user mode get/set_mem.
**
** The target interface should call this function prior to calling
** the get_mem() or set_mem() functions in POSE.
**
** This function decides if a usermode get/set_mem is to be performed
** and if so it validates the id parameter.
**
** If 0 is returned, the the target interface should call POSE's
** get/set_mem() function and return its value.
**
** If 1 is returned,  the supplied id is either faulty or is pointing out
** a process/block which isn't part of the callers segment. In any case
** the target interface implementation of get/set_mem should return 1.
**
** If 2 is returned the target interface should perform a crash safe
** memcpy() in user-space. If any memory access error occurs during
** this memcpy, the target interface implementation of get/set_mem
** should return 1. If the memory is copied without any error 0 should
** be returned.
*/
OSBOOLEAN
zzkrn_pre_get_set_mem(PROCESS id, OSERRCODE subcode);

/*
** This is the kernel function for both ose_mutex_init() and
** ose_sys_mutex_init().  It takes two more parameters than the normal
** API/SPI functions.
**
** Return address is used for tracing purposes and should be either
** zero or the (often compiler supplied) return address from the
** call of the API/SPI function.
**
** The use_sys_seg parameter should be zero when called from the API
** function ose_mutex_init() and non-zero when called from the SPI
** function ose_sys_mutex_init(). The sys mutex is always bound to the
** system segment (regardless of what segment the calleer of the init
** function belongs to), thus it must be explicitly destroyed.
*/
OSERRCODE
zzkrn_mutex_init(struct _mutex *umutex,
		 OSADDRESS attrib,
		 OSADDRESS return_address,
		 OSBOOLEAN use_sys_seg);


/*
** This is the kernel function for both ose_mutex_destroy() and
** ose_sys_mutex_destroy(). It takes two more parameters than the
** normal API/SPI functions.
**
** The extra_p parameter should be a pointer to a variable of type
** OSADDRESS in which the functions writes extra info in case of error.
**
** The sys_mutex parameter should be zero when called from the API
** function ose_mutex_destroy() and non-zero when called from the SPI
** function ose_sys_mutex_destroy(). The sys mutex is always bound to
** the system segment (regardless of what segment the caller of the
** init function belongs to), thus it must be explicitly destroyed.
**
** Returns the value 1 on success, and a value >1 on error, this value
** should be used as the error parameter when calling error...
*/

OSERRCODE
zzkrn_mutex_destroy(struct _mutex *mutex,
		    OSADDRESS *extra_p,
		    OSBOOLEAN sys_mutex);

/*
** Function in the kernel to activate additional SMP cpus.
** This function does not return.
*/
void
zzkrn_activate_cpu(void);

/*
** Save the current process's state.
**
** This function should be called when entering a page fault handler
** if it wants to issue system calls that might end up using the wait
** queue, like any kind of receive that uses a timeout, wait_sem or
** mutex_lock.
*/

void
zzkrn_save_pcb_state(struct _pcb_state_buf *state);

/*
** Restore the current process's state.
**
** This function should be called when exiting a pagefault handler.
**
** When zzkrn_restore_pcb_state() restores the state it will add the
** time spent between saving and restoring the state to the timeout.
*/

void
zzkrn_restore_pcb_state(struct _pcb_state_buf *state);


/*
** This function kills all but three processes running on the calling
** process'c cpu. These three processes are ose_sysdXX, idleXX (in
** both cases XX can not be 00), and the calling process. This is
** usefull when shutting down a core in smp/hypervisor type of
** systems.
*/
void
zzshutdown_cpu(void);

/*
** Temporarily park all cpus except the caller. 
** Also called by the parked cpus to confirm that the action was taken.
** Finally called also to release all other cpus.
**
** The purpose is to establish a stable system state when handling 
** breakpoints and dump generation from the system error handler.
** 
** action:
** 0 A cpu has received an ipi and is entering the stop cpu loop.
**   Also called by epm to poll if some other cpu whants to stop it.
** 1 The master cpu want the other cpus to stop. 
** 2 The master cpu releases the other cpus that where stopped.
** 3 Intercept from public call ose_smp_intercept_other_cpus().
** 4 Resume cpus from public call ose_smp_resume_other_cpus().
**
** invalidate_addr: 
** A breakpoint address to be invalidated on all cpus for action 1.
**
** This function shall be called with interrupts disabled.
*/
int
zzkrn_smp_stop_cpu(int action, OSADDRESS invalidate_addr);

/*
**===========================================================================
** 4.1.3  Signals sent between the system daemon and the local debugger module.
**===========================================================================
*/

/*
** Signal sent from the system daemon to the LDM during system start.
** The ldm extracts the start mode (warm or cold) from this signal,
** initializes ldm operation and sends a response to the system daemon.
** The system daemon then starts the application.
**
** The mode field contains 0 for cold start and 1 for warm start.
*/

#define STARTMODEFLAG (31906)

struct startmodeflag
{
  SIGSELECT sig_no;
  U8 mode;
  void *ldm_world;
};

/*
** Interrupt process swap in and out hooks for cpu load profiling.
**
** These calls should be issued from the target interface just before
** and after each call to an interrupt process entrypoint.
** They are needed only when using the LATE_INIT_INT implementation.
**

** The calls should be made only when previously enabled by a call to
** zzset_inthook_status().
**
** The newpcb and oldpcb parameters are the pointers previously defined
** using the zzbind_vector() call.
**
** The newpcb parameter, always refers to the process that will start
** to run after the hook has been called, the oldpcb always always
** refers to the process that is just about to stop runing.
**
** For zzintswap_in this means that:
**  - The newpcb parameter refers to the interrupt process being
**    called.
**  - The oldpcb parameter refers to the lower level interrupt or
**    other process being swapped back in the end of the interrupt.
**
** For zzintswap_out this means that:
**  - The newpcb parameter refers to the lower level interrupt or
**    other process being swapped back in after the end of the
**    interrupt.
**  - The oldpcb parameter refers to the interrupt process that was
**    being called.
**
** The zzset_inthook_status() function is called only if the
** ENABLE_INTERRUPT_PROFILING flag in pose.con is non zero.
** Otherwise, the function need not be implemented.
*/

void
zzintswap_in(void *newpcb, void *oldpcb);

void
zzintswap_out(void *newpcb, void *oldpcb);

/*
**===========================================================================
** 4.1.4  Functions in the OSE kernel available to the local debugger module.
**===========================================================================
*/

/*
** Process specific debugger control block.
**
** The kernel maintains a structure of this type for each process in the system.
**
** The pid, name, blockname and other fields are initialized by the kernel
** and may not be altered by the ldm.
** Exceptions are the debug_flags and tracename fields, which are initialized
** to zero by the kernel, but are otherwise maintained only by the ldm.
**
** The debug_flags field contains information extracted by the kernel as follows.
** Other bits may be used by the ldm as it finds suitable.
**	D0 = 1 => Enable send hook for this process.
**	D1 = 1 => Enable receive hook for this process.
**	D2 = 1 => Enable swap in hook for this process.
**	D3 = 1 => Enable swap out hook for this process.
**	D4 = 1 => Enable user error hook for this process.
**	D5 = 1 => Enable OSE error hook for this process.
**	D6 = 1 => Enable bind hook for this process.
**      D7 = 1 => Enable timeout hook for this process.
**	D8-D15 are reserved for future expansion. Set them to zero.
*/

struct debug_bcb
{
  U16 magic;
  U16 pindex;
  U16 instance;
};

#ifdef OSARM
/* === ARM ARCHITECTURE ONLY PCB === */
struct debug_pcb
{
  PROCESS pid;		/* [ARM]: */
  void *attached; 	/* [ARM]: */
  OSUSER user;		/* [ARM]: */
  void *padspace0[6];	/* [ARM]: */
  U32 debug_flags;	/* [ARM]: */
  struct stkcb *stack;	/* [ARM]: */
  void *tracename;	/* [ARM]: */
  char *name;
  char *blockname;
  struct debug_bcb *block;
  U32 padspace1[1];	/* [ARM]: */
  void *entrypoint;	/* [ARM]: */
};
/* === ARM ARCHITECTURE ONLY PCB END === */
#else
/* === ALL ARCHITECTURES EXCEPT ARM PCB === */
struct debug_pcb
{
  U32 debug_flags;
  PROCESS pid;
  void *tracename;
  char *name;
  char *blockname;
  struct debug_bcb *block;
  void *padspace1[2];
  OSTIME padspace2;
  void *entrypoint;
};
/* === ALL ARCHITECTURES EXCEPT ARM PCB END === */
#endif

/*
** The ldm sets the master debug status flag.
**
** If the state flag is zero (default), then debugger hooks
** are disabled and the kernel does not perform any system
** debugger services.
**
** If the state flag is non zero, then debugger hooks are called
** as specified by the debug_flags field for each process.
**
** Interrupt disable status is preserved by the function.
*/

void zzset_debug_status(OSBOOLEAN state);

/*
** rmm sets the state of the trace all swaps functionality.
*/

void zzset_trace_swaps_enabled(OSBOOLEAN state);

/*
** Function for the special case when tracing all context 
** switches in a system.
*/

void zzfast_swap_trace(struct debug_pcb *from, struct debug_pcb *to);

/*
** The ldm enables or disables the interrupt process swap hooks
** named zzintswap_in() and zzintswap_out().
**
** If the state flag is zero (default), then debugger hooks
** are disabled and the kernel does not perform any system
** debugger services during interrupt process swaps.
**
** If the state flag is non zero, then different debugger hooks are
** called depending on what bits in the state flag have a non zero
** value.
**
** The state flag bit field is defined as this:
**
** - 0x01: Debug hooks for profiling are called for each interrupt
**         process executed.
**
** - 0x02: Debug hooks for profiling are called for each timer
**         interrupt as well.
**
** - 0x04: Debug hooks for tracing swap-in/out are called for interrupt
**         process executed.
**
** - 0x08: Debug hooks for tracing swap-in/out are called for each
**         timer interrupt process as well.
**
** Interrupt disable status is preserved by the function.
*/

void zzset_inthook_ldmstatus(U32 state);

/*
** The ldm enables or disables the tick debugger hook
** named zzprof_tickhook().
**
** If the state flag is zero (default), then debugger hook is
** disabled and the kernel does not perform any system
** debugger services during the tick() call.
**
** If the state flag is non zero, then debugger hook is called
** for each tick() executed.
**
** Interrupt disable status is preserved by the function.
*/

void
zzset_tickhook_ldmstatus(U32 state);

/*
** The ldm obtains a pointer to the debugger reserved fields of the pcb for the
** specified process.
**
** Returns a pointer to the structure or NULL if the process
** has been killed. The ldm must make sure that invalid process id's are
** never presented to the kernel.
**
** Interrupts must be disabled or the dispatcher must be locked
** while this function is called. I.e the process must not be killed during
** this period.
*/

struct debug_pcb *zzget_debugpcb(PROCESS pid);

/*
** This version of send_w_s is used when a signal must be sent during
** periods when the dispatcher is locked.
** This may be the case in some of the debugger hooks.
*/

void zzsys_send_w_s(union SIGNAL **sig, PROCESS from, PROCESS to);

/*
** The ldm switches to the user pool if required.
**
** The ldm switches from the system pool to the pool of a specified process.
** If pid = 0, the system pool is again selected.
*/

void zzselect_user_pool(PROCESS pid);

/*
** The ldm reads the parent of a process.
**
** The creator of a specified process is returned.
** If the returned pid = 0, then the parent is unknown.
*/

PROCESS zzget_parent(PROCESS pid);

/*
** Get resp. set a function pointer in the kernel handler array.
** The handlers must be located in the core module. They are called
** in supervisor mode.
**
** index 4: Select segment handler. 
** Function prototype: void zzcall_select_segment(OSSEGMENT segnum);
**
** Two indices are available to enable signal allocation profiling.
** The handlers are only called in debug libraries.
** The handlers are called with interrupts enabled or disabled,
** and must therefore preserve the current interrupt mask.
** The handlers may also be called with a locked dispatcher.
** index 12: alloc() handler
** index 13: free_buf() handler.
** These two use a common handler function prototype:
** void (*)(OSBUFSIZE sig_size, void *sig, struct debug_pcb *dpcb,
**          PROCESS pool)
** 
*/
void *zzkrn_get_handler(OSADDRESS index);
void zzkrn_set_handler(OSADDRESS index, void *handler);

/*
** Intecept in the same way as the normal intercept() system call,
** except that system processes, other processes than bg- and
** pri-procs and the calling process wont be intercepted. Returns true
** if the calling process should intercept itself.
*/

OSBOOLEAN zzkrn_dbg_intercept(PROCESS pid);

/*
**===========================================================================
** 4.1.5  Functions and variables implemented in the local debugger module
**        for kernel use.
**===========================================================================
*/

/*
** Pointer to a structure of all internal variables in the ldm.
** This pointer is used by post mortem analyzer tools to obtain information
** relating to the ldm.
** 
** The structure should be implemented so that it contains a magic number
** in the first 32 bit word. This magic number is used by post mortem tools
** to identify the ldm type. Set the magic number to zero if
** the table is unimplemented and no information is available.
*/

extern void * const zzldmvarp[];

/*
** Debugger hook for send events.
** This hook is called from prioritized and background processes.
**
** Returns a pointer to the signal that should actually be sent.
** Returns NULL if the signal was killed by the debugger and the calling
** process should abort the send operation.
**
** The dispatcher is locked when this function is called and it must
** be locked upon return, but it may be unlocked by the debugger while
** performing internal operations. Note that the caller as well as the from and
** to processes may be killed any time while the dispatcher is unlocked.
*/

void *
zzsend_dbghook(OSBUFSIZE sig_size, void *sig, struct debug_pcb *from, struct debug_pcb *to);

/*
** Debugger hook for send events.
** This hook is called from interrupt and timer processes.
**
*/

void *
zzisend_dbghook(OSBUFSIZE sig_size, void *sig, struct debug_pcb *from, struct debug_pcb *to);

/*
** Debugger hook for receive events.
** This hook is called from prioritized and background processes.
**
** Returns a pointer to the signal that should be returned to the process.
** Returns NULL if the signal was killed by the debugger and the calling
** process should retry the receive operation.
**
** The dispatcher is locked when this function is called and it must
** be locked upon return, but it may be unlocked by the debugger while
** performing internal operations. Note that the caller may be killed
** any time while the dispatcher is unlocked.
*/

void *
zzreceive_dbghook(OSBUFSIZE sig_size, void *sig, struct debug_pcb *from, struct debug_pcb *to);

/*
** Debugger hook for receive events.
** This hook is called from interrupt and timer processes.
*/

void *
zzireceive_dbghook(OSBUFSIZE sig_size, void *sig, struct debug_pcb *from, struct debug_pcb *to);

/*
** Debugger hook for receive events.
** This hook is called when the timeout has expired.
*/

void *
zzreceive_tmo_dbghook(struct debug_pcb *to, OSTIME timeout);

/*
** Debugger hook for swap in events.
** This hook is called from within the dispatcher.
** Interrupts are enabled.
** proc_new contains the swapped in process.
** proc_old contains the previously swapped out process, which may
** sometimes have been reported in the last zzswapout_dbghook().
**
** It is mutually exclusive with the swap out hook.
*/

void
zzswapin_dbghook(struct debug_pcb *proc_new, struct debug_pcb *proc_old);

/*
** Debugger hook for swap out events.
** This hook is called from within the dispatcher.
** Interrupts are enabled.
**
** It is mutually exclusive with the swap in hook.
*/

void
zzswapout_dbghook(struct debug_pcb *proc);

/*
** Debugger hook for bind events.
** This hook is called from prioritized and background processes.
** Interrupts are enabled.
**
** Whether the process proc has already been moved from old_cpu to
** new_cpu when zzbind_dbghook is called is undefined.
**
** The dispatcher is locked when this function is called and it must
** remain locked throughout the call.
*/

void
zzbind_dbghook(struct debug_pcb *proc, cpuid_t old_cpu, cpuid_t new_cpu);

/*
** Debugger hook for user error events.
** This hook is called from prioritized and background processes.
**
*/

void
zzerror_dbghook(struct debug_pcb *proc, OSERRCODE ecode, OSERRCODE extra);

/*
** Debugger hook for user error events.
** This hook is called from interrupt and timer processes.
**
*/

void
zzierror_dbghook(struct debug_pcb *proc, OSERRCODE ecode, OSERRCODE extra);

/*
** Debugger hook for OSE error events.
** This hook is called from prioritized and background processes.
**
*/

void
zzoseerror_dbghook(struct debug_pcb *proc, OSERRCODE ecode, OSERRCODE extra);

/*
** Debugger hook for OSE error events.
** This hook is called from interrupt and timer processes.
**
*/

void
zzioseerror_dbghook(struct debug_pcb *proc, OSERRCODE ecode, OSERRCODE extra);

/*
** Debugger hook for system error events.
** This hook is called for kernel and user errors, when not ignored
** by any local process or block error handler.
** This hook is called from interrupt, prioritized and background processes.
** As_intproc is 1 when called from interrupt and timer processes, and
** in other cases where the current dispatcher lock state must be preserved,
** otherwise it is zero.
** The purpose of the hook is primarily to catch system errors in the debugger.
*/

void
zzsystem_error_dbghook(struct debug_pcb *proc, int as_intproc,
		       OSBOOLEAN user_called, OSERRCODE ecode, OSERRCODE extra);
/*
** Debugger hook for create process events.
** This hook is called from the system daemon.
**
** Returns nonzero if the process should be intercepted by the kernel
** upon creation.
** Returns zero otherwise.
*/

OSBOOLEAN
zzcreate_dbghook(struct debug_pcb *proc, OSUSER uid);

/*
** Debugger hook for kill process events.
** This hook is called from the system daemon.
*/

void
zzkill_dbghook(struct debug_pcb *proc);

/*
** This function is called whenever a process is swapped in
** and the profiler is active.
**
** The pri parameter contains 0-31 for prioritized processes,
** 32 for background processes, 33 for the idle process and 34
** for any interrupt or timer process.
**
** The dispatcher may be locked, so no system calls except
** get_systime() may be used.
**
** Interrupts are enabled to the current process level.
*/

void
zzprof_dbghook(OSPRIORITY pri, struct debug_pcb *proc);

/*
** Sample interval supervision for the Profiler subsystem.
**
** This function is called every system tick in the context of the
** current system timer process.
**
** System calls can be issued as from any interrupt process.
*/

void
zzprof_tickhook(void);

/*
** The create_event() and signal_event() system calls supplies user
** defined events to the application.
**
** The functions are stubbed in pose if the RMM is not included, otherwise
** they are implemented in the RMM.
*/

void
create_event(SIGSELECT eventno, OSADDRESS *trig);

void
zzsignal_event(SIGSELECT eventno, const void *data, OSBUFSIZE size);

OSADDRESS
zzose_set_report_val_sysconf_77(OSADDRESS reportno, OSADDRESS reportid,
                                OSADDRESS change);

OSADDRESS
zzose_create_report_sysconf_78(OSADDRESS reportno, OSADDRESS trig,
                               OSADDRESS rpt_flag);

/*
**===========================================================================
** 4.1.6  Functions and variables implemented in the kernel for use in the
**        platform.
**===========================================================================
*/


/*
** These are the undocumented kill_lock(), kill_unlock() and
** kill_curr_at_unlock() system calls.
**
** The purpose of the kill_lock() system call is to make sure that a
** given process won't be killed while it's in "kill locked" state.
**
** The purpose of the kill_unlock() system call is to decrement a
** process' "kill locked" counter and check if it result is zero.  If
** so this function will check if anyone has tried to kill the process
** while it was locked, in which case this function will now call
** kill_proc() upon the former "kill locked" process.
**
** The purpose of kill_curr_at_unlock() system call is to mark the
** current process so it will kill itself as soon as it leaves its
** "kill locked" state.
**
** A process can be locked more than once, which means that it must be
** unlocked the same number of times before it leaves the "kill
** locked" state.
** 
** The pid parameter contains the id of the process to lock.  Setting
** the pid parameter to zero is equivalent to using the pid of the
** current process.
**
** The kill_lock() system call returns nonzero value if the call
** successfully locked the process.
** 
** It is only allowed to call kill_curr_at_unlock() when the current
** process is locked, otherwise it will report an error.
*/

OSBOOLEAN
zzkrn_kill_lock(PROCESS pid);

void
zzkrn_kill_unlock(PROCESS pid);

void
zzkrn_kill_curr_at_unlock(void);

/*
** Set the current segment for the calling process to segid.
** If go_home is nonzero, the segid parameter is ignored and
** the home segment of the process is reselected.
**
** Returns a non zero value if the segid parameter is
** unknown to the caller, i.e. the segment has already
** terminated.
**
** This variant of set_segment should only be used from a
** freeze-mode context and will return a non zero value
** (failure) if interrupts are enabled. 
*/

OSBOOLEAN 
zzkrn_freezemode_set_segment(OSBOOLEAN go_home, OSSEGMENT segid);

/*
** Turn semaphore annotation on or off. Returns the previous flag setting.
** When annotation is on, dynamically created SEMAPHORE structs are extended
** by eight bytes, containing the address of the creating call and the
** creators PROCESS identifier, in that order. Also, the b member of the 
** SEMAPHORE is set to SEMAPHORE_ANNOTATED to indicate that the SEMAPHORE 
** is annotated.
*/
#define SEMAPHORE_ANNOTATED ((void *)0x85522)
OSBOOLEAN zzkrn_set_semaphore_annotation(OSBOOLEAN activate);

/* 
** A prototype of a per block hook to call at start/stop/intercept/resume
** and interface to install resp. uninstall the hook. 
** The operation argument can take the values
** OSE_START, OSE_STOP, OSE_INTERCEPT and OSE_RESUME.
** The hooks can be called in any process context, including interrupt level.
** Interrupts are disabled, and must be so in the hook.
** The dispatcher may be locked by the kernel. 
** Note that the hook can be called concurrently on several cpus,
** so updates to common data must be avoided or serialized.
**
** Currently, no system calls are available, and the return value 
** should be zero.
**
** The hook is called at each state operation on the block, even if the
** operation not really changes the state, for example intercepted twice.
** The kernel does not record the state of blocks.
** 
*/

typedef OSADDRESS (OSSTATEHOOK)(PROCESS block, OSERRCODE operation,
				void *handle);

/* 
** Hook Installation: 
** OSADDRESS hookref;
** hookref = get_sysconf(102, (OSADDRESS) bid, (OSADDRESS) func, 
**                   (OSADDRESS) handle);
** A resulting hookref of zero reports a failure.
**
** Hook Removal:
** if (hookref) 
**   success = get_sysconf(103, (OSADDRESS) bid, (OSADDRESS)&hookref, 0);
*/

/*
**===========================================================================
** 4.2  Variables
**===========================================================================
*/
/*
**===========================================================================
** 4.2.1  Variables available to the target interface.
**===========================================================================
*/

/*
** Pointer to the uarea for the current process. This area can be used to store
** interrupt return status so that registers can later be recovered.
*/

extern void *zzcurrent_uarea;

/*
** When a request for a wakeup call to a lower priority interrupt is made,
** this variable contains the priority that should be passed to the kernel
** in the int_level parameter to the zzdo_wakeup() call.
*/

extern OSPRIORITY zzwakeup_level;

/*
** The dispatcher look aside flag word is a 32 bit word. It  contains one
** bit for each process priority. The most significant bit corresponds to
** the highest priority process, i.e. bit 0 corresponds to priority 31.
** A bit in this word is set if at least one process is
** ready on the corresponding priority. It is cleared otherwise.
**
** The zzdp_look_aside variable is available to the target
** interface and to other OSE software. The reasons are that this
** variable may be needed by the ZZFIND_FIRST_BIT() macro, and it
** can be used for efficient cpu load measurement.
**
** Use zzget_dp_look_aside() to get an SMP-safe access.
*/

extern volatile U32 zzdp_look_aside;
#ifdef OSE_SMP
U32 zzget_dp_look_aside(cpuid_t cpu);
#else
#define zzget_dp_look_aside(cpu) zzdp_look_aside
#endif


/*
**===========================================================================
** 4.2.2  Variables in the target interface, available to the kernel.
**===========================================================================
*/

/*
** Pointer to target interface structure of internal variables.
** This pointer is used by post mortem analyzer tools to obtain information
** relating to the target interface.
** The information that is accessed using this structure is the the cpu
** register contents, the RA pseudo register, and file and line information.
** 
** The structure should be implemented so that it contains a magic number
** in the first 32 bit word. This magic number is used by post mortem tools
** to identify the kernel and cpu type. Set the magic number to zero if
** the table is unimplemented and no information is available.
*/

extern const U32 zztivarp[];

/*
**===========================================================================
** 4.3  Backdoor entries available to trusted applications.
**===========================================================================
*/

/*
** The functions and variables presented here are available to special purpose
** platform applications, including the target interface for certain fixes
** and optimizations.
** If these entries are used by customers, it must be made clear that this
** interface has a very low degree of portability, and may in fact be
** changed from release to release of the kernel.
** 
** These symbols are not subject to name translation in osenames.h, so they
** can be used in a straight forward way between cpu platforms.
*/

/*
** These symbols are used in the target interface when implementing a
** dispatcher with a late call to zzinit_int. Such a dispatcher is
** faster than the standard implementation, but more work needs to
** be done in assembly language.
** Implementing the late zzinit_int policy involves setting the
** LATE_INIT_INT constant to 1 when compiling pose.c. The code in
** zzinit_int that is thereby removed must be implemented in the
** target interface using the proper kernel variables and functions.
**
** By utilizing the zzuu struct, it is possible to entirely eliminate
** the zzinit_int() call, further optimizing interrupt management.
** In fact, the entire dispatcher may be rewritten in assembler
** if required.
**
** Synopsis:
**   #define LATE_INIT_INT 1
**   extern volatile OSBOOLEAN zzwakeflag;
**   extern struct _pcb *zzcur_p;
**   extern struct _pcb *zzcur_ni;
**   extern struct world zzuu;
**
** See also the zzinit_int() function in pose.c.
*/

/*
** The post mortem support table is the handle used to access kernel data
** structures.
**
** It is organized as a header structure in the const section, which points
** to all other data structures.
**
** Synopsis:
**   extern const struct osvarp zzosvarp;
*/


/*
**===========================================================================
** 4.4  SMP specific.
**===========================================================================
*/

#ifdef OSE_SMP
struct _pcb; /* Forward declaration */
extern void zzkrn_set_cur_p(struct _pcb *pcb);
OSBOOLEAN zzkrn_dispatcher_locked(void);
#endif

/*
** Request the kernel to process kernel events posted by other cpus
** in an SMP kernel.
**
** Parameter and return values are defined as for the zzinit_int() function.
*/
#ifdef OSE_SMP
OSBOOLEAN zzkrn_evt_notify(OSPRIORITY oldint_level);
#endif

/*
** Request the kernel to process kernel events posted by other cpus
** in an SMP kernel.
**
** Used when processing IPI interrupts with an ISR.
*/
#ifdef OSE_SMP
void zzkrn_evt_notify_from_isr(void);
#endif

/**
 * @macro OSE_KRN_EVT_MSG_ENTRY_LEN
 *
 * @brief Indicates the max message length of a kernel event.
 */
#define OSE_KRN_EVT_MSG_ENTRY_LEN 2

struct ose_krn_msg
{
   uint32_t data[2];
};

/**
 * @function ose_krn_evt_init
 * @include "ose_spi/ose_smp.h"
 *
 * @brief Initialize event queues used for kernel event handling
 * in an SMP system, and also queues for fsend().
 * @long
 * To be able to communicate between CPUs in an SMP system,
 * kernel events are used.
 * ose_krn_evt_init initializes the required resources
 * in the kernel event queues of the CPUs in the SMP system.
 * @long
 * Implemented by the BSP, called by the kernel.
 * The fsend() queues are mandatory for all kernels.
 *
 * @restriction
 * Must be called before calling ose_krn_evt_enqueue
 * and ose_krn_evt_dequeue.
 *
 * @seealso ose_krn_evt_enqueue
 * @seealso ose_krn_evt_dequeue
 */
void
zzkrn_evt_init(void);

/**
 * @function ose_krn_evt_enqueue
 * @include "ose_spi/ose_smp.h"
 *
 * @brief Enqueue a kernel event for a destination CPU in an SMP system.
 * @long
 * To be able to communicate between CPUs in an SMP system,
 * kernel events are used.
 * ose_krn_evt_enqueue shall put a kernel event
 * in the kernel event queue of the selected peer CPU
 * and generate an interrupt to the destination CPU.
 * A CPU receiving a kernel event in its kernel event queue
 * notifies the kernel that a new kernel event is
 * available in its kernel event queue by returning the magic kernel event
 * vector number OSVECTOR_INT_KRN_EVT
 * in the corresponding interrupt vector handling routine.
 * @long
 * Implemented by the BSP, called by the kernel.
 * @input dst_cpu The ID of the CPU to send the message to.
 * @input krn_evt_buf Pointer to the buffer containing the kernel event.
 * @input count The size of the kernel event located in the buffer.
 * @return 0 on success and 1 if the message queue is full.
 * @restriction
 * Must not be called before ose_krn_evt_init has been called.
 * Must be called with disabled interrupts and the implemented function
 * is not permitted to enable the interrupt.
 *
 * @seealso ose_krn_evt_init
 * @seealso ose_krn_evt_dequeue
 * @seealso OSVECTOR_INT_KRN_EVT
 */
#ifdef OSE_SMP
uint32_t
zzkrn_evt_enqueue(cpuid_t dst_cpu,
		  const uint32_t *krn_evt_buf,
		  uint32_t count);
#endif

/**
 * @function ose_krn_evt_dequeue
 * @include "ose_spi/ose_smp.h"
 *
 * @brief Dequeue a kernel event from the kernel event queue of a CPU
 * in an SMP system.
 * @long
 * To be able to communicate between CPUs in an SMP system,
 * kernel events are used.
 * ose_krn_evt_enqueue generates an interrupt for
 * the selected destination CPU(s).
 *
 * @long
 * Implemented by the BSP, called by the kernel.
 * @input krn_evt_buf Pointer to a buffer into which the kernel event
 * shall be written when dequeuing it from the kernel event queue.
 * @return The size of the kernel event written to the
 * kernel event buffer when dequeuing it from the kernel event queue,
 * or zero if no kernel event was available.
 * @restriction
 * Must not be called before ose_krn_evt_init has been called
 *
 * @seealso ose_krn_evt_init
 * @seealso ose_krn_evt_enqueue
 */
#ifdef OSE_SMP
uint32_t
zzkrn_evt_dequeue(uint32_t *krn_evt_buf);
#endif

/**
 * @function zzose_krn_evt_dump_queue
 * @include "ose_spi/ose_smp.h"
 *
 * @brief EXPERIMENTAL. Dump the kernel event queue for a specific CPU.
 * @long Required but may be empty. Intended to collect information for
 * post mortem analysis.
 * @long This function shall be called with interrupts disabled.
 *
 * @input target_cpu The CPU of the target for which to dump the
 *                   kernel event queue.
 */
#ifdef OSE_SMP
void
zzkrn_evt_dump_queue(cpuid_t target_cpu);
#endif

/**
 * @function zzkrn_force_exception_in_other_cpu
 * @include "ose_spi/ose_smp.h"
 *
 * @brief Force an exception on another cpu if possible.
 * @long Required but may be a dummy. Intended to force another cpu
 * to produce a pmd dump, even if it has disabled ordinary interrrupts.
 * The typical use case is when the kernel event queue is full for
 * a too long time.
 * @long This function shall be called with interrupts disabled.
 *
 * @input target_cpu The CPU of the target for which to dump the
 *                   kernel event queue.
 * @return
 * 0: an exception can not be sent on this hardware, so go ahead and
 * report an error. This is the dummy return value.
 * @break
 * 1: an exception has been sent, do not report an error, just loop
 * waiting for a reset or watchdog exception to occur.
 * @break
 * 2: an exception has been sent, but as the effect is not known,
 * wait a while and report a fatal error anyhow.
 */
#ifdef OSE_SMP
int
zzkrn_force_exception_in_other_cpu(cpuid_t target_cpu);
#endif

uint32_t
zzkrn_msg_enqueue(cpuid_t peer, const struct ose_krn_msg *krn_msg_buf,
		  uint32_t flags);

uint32_t
zzkrn_msg_dequeue(struct ose_krn_msg *krn_msg_buf, uint32_t elems);

int
zzkrn_set_fsend_hook(int sig_prio, OSFSENDHOOK *hook);

#endif /* #ifndef _POSE_H */

