/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
#ifndef PTHREAD_INTERNAL_H_
#define PTHREAD_INTERNAL_H_

#include "heapapi.h"
#include "ose.h"
#include "ose_spi.h"
#include "osetypes.h"
#include "pthread_err.h"

#include "assert.h"
#include "errno.h"
#include "pthread.h"
#include "rtc.h"
#include "string.h"
#include "string.h"
#include "time.h"
#include "splaytree.h"
#include "pthread.sig"
#include "pose.h"

/* Check of execution model not yet active. */
/* #define USING_STILL_UNDECIDED_SHARED_MODEL */

/* Mutex-used values of fast semaphore. */
#define FSEM_INIT     ((OSFSEMVAL) 0)
#define FSEM_DEC      ((OSFSEMVAL) 1)
#define FSEM_WAITING  ((OSFSEMVAL) -1)
#define FSEM_CANCELED ((OSFSEMVAL) 2)

/* Function calls for the bios module trap handler. */
#define OSEPTH_COND_BROADCAST 0
#define OSEPTH_COND_SIGNAL    1
#define OSEPTH_COND_TIMEDWAIT 2
#define OSEPTH_COND_WAIT      3
#define OSEPTH_MUTEX_LOCK     4
#define OSEPTH_MUTEX_TRYLOCK  5
#define OSEPTH_MUTEX_UNLOCK   6
#define OSEPTH_SETCANCELSTATE 7
#define OSEPTH_PTHREAD_CANCEL 8

/* Environment variable names. */
#define PTHREAD_PROCESS_NAME  "pthread"
#define OSE_PTHREAD_BIOS_FLIB "OSE_PTHREAD"
#define PRIO_LOCK_MAX_LEVEL   "PTHREAD_PRIO_LOCK_MAX_LEVEL"
#define PTHREAD_BLOCK_STATE  "ose_pthread_block_state"
#define PTHREAD_THREAD_STATE "ose_pthread_thread_state"
#define KEYVALUES             "keyvalues"
#define CLEANSTACK            "cleanup"

/* Status codes. */
#define OSEPTH_STATUS_OK         0
#define OSEPTH_END_THREAD (-1) /* execute exitnow in right context */

/* Thread state bits. */
#define THREAD_STATE_RUNNING     ((U32) 0x1U)
#define THREAD_STATE_CANCELED    ((U32) 0x2U)
#define THREAD_STATE_JOINED      ((U32) 0x4U)
#define THREAD_STATE_DETACHED    ((U32) 0x8U)
#define THREAD_STATE_CANCELABLE  ((U32) 0x10U)
#define THREAD_STATE_INHERITING  ((U32) 0x20U)
#define THREAD_STATE_CONDWAITING ((U32) 0x40U)
#define THREAD_STATE_JOINING     ((U32) 0x80U)

/* Used when accessing internal mutex details. */
#define MUTEX_INT_TYPE struct _mutex

/* Macro functions. */
#define INTERNAL_ERROR()                                        \
   do                                                           \
   {                                                            \
      error(OSE_PTH_INTERNAL);                                  \
   } while (1)

#define PTHREAD_SMP_LOG(a)

/* New SMP lock debugging. */
#if 0
#define PTHREAD_SMP_DEBUG
#endif

#if defined(PTHREAD_SMP_DEBUG)

#include "ramlog.h"
#undef PTHREAD_SMP_LOG
#define PTHREAD_SMP_LOG(a)                                      \
   do {                           				\
      ramlog_printf a;						\
   } while (0)

#endif /* PTHREAD_SMP_DEBUG */

/* Macros for handling the Big Pthread Lock. */
#define BPL_LOCK()                                         \
   do                                                           \
   {                                                            \
      struct block_state *block_state = osepth_block_state();   \
      PTHREAD_SMP_LOG(("Pthread %x waiting for BPL %s %d\n",	\
                       current_process(), __FILE__, __LINE__)); \
      ose_mutex_lock(&block_state->bpl_mutex);                	\
      PTHREAD_SMP_LOG(("Pthread %x got the BPL %s %d\n",	\
                       current_process(), __FILE__, __LINE__)); \
   } while (0)

#define BPL_UNLOCK()                                  \
   do                                                           \
   {                                                            \
      struct block_state *block_state = osepth_block_state();   \
      PTHREAD_SMP_LOG(("Pthread %x released BPL %s %d\n",	\
                       current_process(), __FILE__, __LINE__)); \
      ose_mutex_unlock(&block_state->bpl_mutex);                \
   } while (0)

#define OSEPTH_ERROR(ecode)                \
  do                                       \
  {                                        \
    for (;;)                               \
    {                                      \
      error((ecode));                      \
    }                                      \
  } while (0)

#define OSEPTH_ERROR2(ecode, extra)  \
  do                                       \
  {                                        \
    for (;;)                               \
    {                                      \
      error2((ecode), (extra));            \
    }                                      \
  } while (0)

struct block_state
{
   Keydata **keys;
   MUTEX key_mutex;
   MUTEX cancel_mutex;
   SplayTree_t cancel_tree;
   OSPRIORITY prio_max;
   MUTEX bpl_mutex;   /* Mutex providing SMP synchronization. */
   PROCESS bid;
};

struct thread_state
{
   U32 waiter; /* PID of pthread that joined this */
   U32 exitval;
   U32 flags;
   U32 cond;
   U32 condlink;
};

struct keyval
{
   void **specifics;
   Keydata **pthkeybase;
};

struct canceled_thread
{
   SplayNode_t node;
   pthread_t tid;
};

union SIGNAL
{
   SIGSELECT sig_no;
   struct configure_thread configure_thread;
   struct signal_cond_var signal_cond_var;
};

void
osepth_end_thread(pthread_t thispth,
                      struct thread_state * thread_state,
                      void *exitval,
                      OSBOOLEAN cancel);

OSBOOLEAN
osepth_attr_is_valid(const pthread_attr_t * attr);

void
osepth_init_flib(void);

long
osepth_trap_handler(long fc,
                    long ecode,
                    long arg1,
                    long arg2,
                    long arg3);

void
initPthreads(void);

struct block_state *
osepth_block_state(void);

struct thread_state *
osepth_private_thread_state(void);

struct thread_state *
osepth_other_thread_state(pthread_t tid);

struct thread_state *
osepth_create_thread_state(pthread_attr_t *attr);

#include "pthtranslate.h"

/* These are defined in mutex.c */
void ose_mutex_init_unsafe(MUTEX *mutex, OSADDRESS attr, OSERRCODE *rc);
void ose_mutex_destroy_unsafe(MUTEX *mutex, OSERRCODE *rc, OSADDRESS *extra);
void ose_mutex_lock_unsafe(MUTEX *mutex, OSERRCODE *rc, OSADDRESS *extra);
int ose_mutex_trylock_unsafe(MUTEX *mutex, OSERRCODE *rc, OSADDRESS *extra);
void ose_mutex_unlock_unsafe(MUTEX *mutex, OSERRCODE *rc, OSADDRESS *extra);

#endif /* #ifndef PTHREAD_INTERNAL_H_ */
