/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2012 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 ****************************************************************************

	CONTENTS
	--------

	1  Description
	2  History of development
	3  Include files
	4  Macros
	5  Types
	6  Variables
	7  Function prototypes


 ****************************************************************************
 */

#ifndef _RTC_PRIVATE_H
#define _RTC_PRIVATE_H

/*
 ****************************************************************************
 * 1  DESCRIPTION.
 ****************************************************************************
 *
 */

/*
 ****************************************************************************
 * 2  HISTORY OF DEVELOPMENT.
 *
 *    $Log:   /rtt/pvcsarch/OSE90/src/src/private.h_v  $
 * 
 *    Rev 1.2   02/18/00 10:54:22   matr
 * 
 *    Rev 1.1   11/11/99 10:37:10   matr
 * 
 *    Rev 1.0   09/19/97 21:49:24   olli
 * Initial revision.
 */

/*
 ****************************************************************************
 * 3  INCLUDE FILES.
 ****************************************************************************
 */

#include <stddef.h>
#include "time.h"

#include "limits.h"
#include "ose.h"

#include "rtc.sig"

#ifdef __cplusplus
extern "C" {
#endif

/*
 ****************************************************************************
 * 4  MACROS
 ****************************************************************************
 */

#define SECSPERMIN  60
#define MINSPERHOUR 60
#define HOURSPERDAY 24
#define DAYSPERWEEK 7
#define DAYSPERNYEAR    365
#define DAYSPERLYEAR    366
#define SECSPERHOUR (SECSPERMIN * MINSPERHOUR)
#define SECSPERDAY  ((long) SECSPERHOUR * HOURSPERDAY)
#define MONSPERYEAR 12

#define TM_SUNDAY   0
#define TM_MONDAY   1
#define TM_TUESDAY  2
#define TM_WEDNESDAY    3
#define TM_THURSDAY 4
#define TM_FRIDAY   5
#define TM_SATURDAY 6

#define TM_JANUARY  0
#define TM_FEBRUARY 1
#define TM_MARCH    2
#define TM_APRIL    3
#define TM_MAY      4
#define TM_JUNE     5
#define TM_JULY     6
#define TM_AUGUST   7
#define TM_SEPTEMBER    8
#define TM_OCTOBER  9
#define TM_NOVEMBER 10
#define TM_DECEMBER 11

#define TM_YEAR_BASE    1900

#define EPOCH_YEAR  1970
#define EPOCH_WDAY  TM_THURSDAY

/*
** Accurate only for the past couple of centuries, that will probably do.
*/


#ifndef isleap
#define isleap(y) ((((y) % 4) == 0 && ((y) % 100) != 0) || ((y) % 400) == 0)
#endif

#define TYPE_BIT(type)  (sizeof (type) * CHAR_BIT)

#define TYPE_SIGNED(type) (((type) -1) < 0)

/*
** 302 / 1000 is log10(2.0) rounded up.
** Subtract one for the sign bit if the type is signed;
** add one for integer division truncation;
** add one more for a minus sign if the type is signed.
*/
#define INT_STRLEN_MAXIMUM(type) \
    ((TYPE_BIT(type) - TYPE_SIGNED(type)) * 302 / 100 + 1 + TYPE_SIGNED(type))

/* Size of the static character buffer. */
#define BUF_SIZE (3 * 2 + 5 * INT_STRLEN_MAXIMUM(int) + 3 + 2 + 1 + 1)

#define INVALID_TIME ((time_t)-1)

#define FALSE 0
#define TRUE 1

/*
 ****************************************************************************
 * 5  TYPES.
 ****************************************************************************
 */

typedef OSTICK (f_get_systime)(OSTICK *microsecs);

/*
*  struct TimeRef
*  Contains three elements that need to stay in synchronization with
*  each other:
*     *  real_time   Seconds and microseconds represents time in the format
*                    maintained by the RTC.
*     *  systime     Ticks and microseconds representing a value pair returned
*                    by the get_systime() system call.
*     *  scale_factor A factor that the systime shall be multiplied by to
*                    convert into real_time, allowing to compensate for
*                    inaccuracies in the kernel systime..
*/
struct TimeRef
{
   struct TimePair   real_time;
   OSTICK            systime_ticks;
   OSTICK            systime_micros;
   U32               scale_factor;
};


/*
*  Bios function code to get globals for whitebox testing.
*  Change if updating RtcGlobal
*/
#ifdef OSE64
#define RTC_WHITEBOX_GET_VERSIONED_GLOBALS (1234000001)
#else
#define RTC_WHITEBOX_GET_VERSIONED_GLOBALS (1235000001)
#endif

/*
*  struct RtcGlobal
*  Contains global variables of the RTC. There are two purposes for
*  keeping these in a struct:
*     *  It is nice to use a "g." prefix for access to global state,
*        making the code more readable.
*     *  It is possible to export the globals as a chunk for witebox
*        testing.
*
*  The contents:
*  micros_per_tick   Number of microsecond per tick; the result from
*                    system_tick().
*  ref_time          A time reference that is updated periodically.
*                    All access is within spin_lock to keep the fields
*                    synchronized with each other.
*/

struct RtcGlobal
{
   OSTICK                  micros_per_tick;
   volatile struct TimeRef ref_time;
   f_get_systime           *get_systime_whitebox_callback;
};


/*
 ****************************************************************************
 * 6  VARIABLES.
 ****************************************************************************
 */

static const int	mon_lengths[2][MONSPERYEAR] = {
	{ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 },
	{ 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }
};

static const unsigned short year_lengths[2] =
{
    DAYSPERNYEAR, DAYSPERLYEAR
};

/*
 ****************************************************************************
 * 7  FUNCTION PROTOTYPES.
 ****************************************************************************
 */

PROCESS rtc_init_flib(void);
PROCESS zzrtc_init_flib(void);

#ifndef softOSE
/* Functions in time.h: */
clock_t zzclock(void);
time_t zzmktime(struct tm *a);
time_t zztime(time_t *a);
char *zzasctime(const struct tm *a);
char *zzasctime_r(const struct tm *a, char *b, size_t c);
char *zzctime(const time_t *a);
char *zzctime_r(const time_t *a, char *b, size_t c);
struct tm *zzgmtime(const time_t *a);
struct tm *zzgmtime_r(const time_t *a, struct tm *b);
struct tm *zzlocaltime(const time_t *a);
struct tm *zzlocaltime_r(const time_t *a, struct tm *b);
size_t zzstrftime(char *a, size_t b, const char *c, const struct tm *d);
int zzgettimeofday(struct timeval *a, struct timezone *b);
int zzsettimeofday(struct timeval *a, struct timezone *b);
void zztzset(void);

#endif

/* Functions in rtc.h: */
void
zzrequest_alarm(ALARM_INFO *a, const time_t *b, time_t c, union SIGNAL **d);
union SIGNAL *zzcancel_alarm(ALARM_INFO *a);
void zzget_time(struct TimePair *a);
void zzset_time(const struct TimePair *a);
void rtc_adjust_time(long micros); /* Obsolete. Use rtc_get_scale instead. */
void zzadjust_time(long a);

/* Dummy entrypoints for backwards compatibility. */
void initRtc(void);
void initRtcFlib(void);

OSENTRYPOINT ose_zic;

/*
 *===========================================================================
 *                           rtc_gmtsub
 *===========================================================================
 *
 * Description: 
 *
 * Parameters:  
 *              
 * Returns:     
 */
void rtc_gmtsub(const time_t *timep,
                long offset,
                struct tm *tmp);
/*
 *===========================================================================
 *                           rtc_localsub
 *===========================================================================
 *
 * Description:
 *
 * Parameters:
 *
 * Returns:
 */

void rtc_localsub(const time_t *timep,
                  long offset,
                  struct tm *tmp);

void rtc_timesub(const time_t *timep,
                 long offset,
                 struct tm *tmp,
                 long corr,
                 int hit);

/*
 *===========================================================================
 *                           rtc_procvar
 *===========================================================================
 *
 * Description:  Return a pointer to the named per process variable.
 *
 * Parameters:   name       Name of the variable.
 *               size       Size of the variable.
 *
 * Returns:      A pointer to the variable.
 */

void *rtc_procvar(const char *name, size_t size);


/*
 *===========================================================================
 *                rtc_getTzInfo
 *===========================================================================
 * Description: Get the local time zone offset. If Daylight savings time is 
 *              active at the given time, it will be included in the offset.
 *
 * Parameters:  timep           I: The time
 *              gmtOffset       O: Time zone offset from GMT in seconds.
 *              isdst           O: Flag for Daylight Savings Time
 *              abbr            O: Time zone abbreviation
 *
 */

void rtc_getTzInfo(time_t t,
                   S32 *gmtOffset,
                   int *isdst,
                   char abbr[TZ_ABBR_BUFF_SIZE]);


/*
 *===========================================================================
 *                rtc_getLsInfo
 *===========================================================================
 *
 * Description: Get the leap second correction factor for a given time.
 *
 * Parameters:  timep      I: The time
 *              corrp      O: Time correction to be applied
 *              hitp       O: Indicates whether this very second is a leap 
 *                            second or not.
 *
 * Returns:
 */


void rtc_getLsInfo(time_t t, S32 *corrp, int *hitp) ;

void rtc_time2subTypeCorr(time_t *t, long offset, int your_isdst);

long rtc_getDstDiff(time_t t, int isdst);

int rtc_tmcomp(const struct tm * const atmp, const struct tm * const btmp);

#endif /* #ifndef _RTC_PRIVATE_H */

/*
 ****************************************************************************
 *                         END OF FILE
 ****************************************************************************
 */
