/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2012 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 ****************************************************************************
 *			   SIGNAL SPECIFICATION
 *
 *     $Workfile:   private.sig  $
 *     Document no: @(#) 512/OSE90-1 $Revision:   1.0  $
 *     $Author:   matr  $
 *     $Date:   02/18/00 10:54:26  $
 *
 *
 ****************************************************************************

	CONTENTS
	--------

	1  Description
	2  History of development
	3  Include files
	4  Macros
	5  Types
	6  Signals

 ****************************************************************************
 */

#ifndef _PRIVATE_SIG
#define _PRIVATE_SIG

/*
 ****************************************************************************
 * 1  DESCRIPTION.
 ****************************************************************************
 *
 * Signals for communication between RTC function library and the RTC.
 *
 * Subject to change without prior notice. This is private interface
 * between the different parts of RTC. Only rtc.h and rtc.sig defines the public
 * interface and carries any responsibility or guarantee.
 */

/*
 ****************************************************************************
 * 2  HISTORY OF DEVELOPMENT.
 *
 *    $Log:   /rtt/pvcsarch/OSE90/src/src/private.siv  $
   
      Rev 1.0   02/18/00 10:54:26   matr
   Initial revision.
 */

/*
 ****************************************************************************
 * 3  INCLUDE FILES.
 ****************************************************************************
 */

#include "ose.h"
#include "osetypes.h"
#include "time.h"
#include "rtc.h"
#include "rtc.sig"

#ifdef OSE64
#include "stdint.h"
#endif


/*
 ****************************************************************************
 * 4  MACROS.
 ****************************************************************************
 */


/*
 ****************************************************************************
 * 5  TYPES.
 ****************************************************************************
 */

/*
 ****************************************************************************
 * 6  SIGNALS.
 ****************************************************************************
 */

#define RTC_REQUEST_32  32700 /* !-SIGNO(struct RtcRequest)-! */
#define RTC_CANCEL_32   32701 /* !-SIGNO(struct RtcCancel_32)-! */
#define RTC_GET_TIME    32702 /* !-SIGNO(struct RtcGetTime)-! */
#define RTC_SET_TIME    32703 /* !-SIGNO(struct RtcSetTime)-! */
#define RTC_ADJUST_TIME 32704 /* !-SIGNO(struct RtcAdjustTime)-! */

#define RTC_READY	32706 /* !-SIGNO(SIGSELECT)-! */
#define RTC_GET_SCALE   32707 /* !-SIGNO(struct RtcGetScale)-! */
#define RTC_SET_SCALE   32708 /* !-SIGNO(struct RtcSetScale)-! */
#define RTC_GET_TZ_INFO_REQUEST     32709 /* !-SIGNO(struct RtcGetTzInfo)-! */
#define RTC_GET_TZ_INFO_REPLY	    32710 /* !-SIGNO(struct RtcGetTzInfo)-! */
#define RTC_GET_LS_INFO_REQUEST	    32711 /* !-SIGNO(struct RtcGetLsInfo)-! */
#define RTC_GET_LS_INFO_REPLY	    32712 /* !-SIGNO(struct RtcGetLsInfo)-! */
#define RTC_T2SUB_TYPE_CORR_REQUEST 32713 /* !-SIGNO(struct RtcT2subTypeCorr)-! */
#define RTC_T2SUB_TYPE_CORR_REPLY   32714 /* !-SIGNO(struct RtcT2subTypeCorr)-! */
#define RTC_GET_DST_DIFF_REQUEST    32715 /* !-SIGNO(struct RtcGetDstDiff)-! */
#define RTC_GET_DST_DIFF_REPLY	    32716 /* !-SIGNO(struct RtcGetDstDiff)-! */
#define RTC_COMPILE_REQUEST         32717 /* !-SIGNO(struct RtcCompileRequest)-! */
#define RTC_COMPILE_REPLY           32718 /* !-SIGNO(struct RtcCompileRequest)-! */

/* 32720-32723: rtc.sig */

#define RTC_REQUEST_64              32724 /* !-SIGNO(struct RtcRequest)-! */
#define RTC_CANCEL_64               32725 /* !-SIGNO(struct RtcCancel_64)-! */

/*
 * 32-bit and 64-bit aspects of dual-mode signals:
 *
 * RTC_CANCEL is the reply signal to RTC_REQUEST and RTC_CANCEL.
 * This needs to be backwards compatible to 32-bit clients.
 */
#ifdef OSE64
#define RTC_REQUEST		RTC_REQUEST_64
#define RTC_CANCEL		RTC_CANCEL_64
#else
#define RTC_REQUEST		RTC_REQUEST_32
#define RTC_CANCEL		RTC_CANCEL_32
#endif

/*
 *===========================================================================
 *                        RtcRequest
 *===========================================================================
 *
 * Description: Request an alarm signal.
 *
 * Parameters:  sigNo           Signal number.
 *              expires
 *              period
 *              client          Client process identifier.
 *              alarmSig        Pointer to client's request_alarm signal.
 *
 */

struct RtcRequest
{
   SIGSELECT    sigNo;
   time_t       expires;
   U32          period;
   PROCESS      client;
   char         alarmSig[1]; /* Variable size. */
};

/*
 *===========================================================================
 *                        RtcCancel
 *===========================================================================
 *
 * Description: Cancel an alarm signal.
 *
 * Parameters:  sigNo           Signal number.
 *              alarmInfo       The alarm information needed for cancellation.
 *
 * struct RtcCancel
 * {
 *    SIGSELECT    sigNo;
 *    ALARM_INFO   alarmInfo;
 *    S32          info;
 * };
 *
 * Return info included in struct by ML2002.
 */

/*
*  Mode-dependent layout of struct RtcCancel
*/
struct RtcCancel
{
    SIGSELECT             sigNo;
    ALARM_INFO            alarmInfo;
    S32                   info;
};

/*
 * 32-bit layout of struct RtcCancel
 */
struct RtcCancel_32
{
   SIGSELECT    sigNo;
   struct alarm_info_32
   {
      SIGSELECT sigNo;
      SIGSELECT instance;
      U32       pointer;
   } alarmInfo;
   S32          info;
};

/*
 * 64-bit layout of struct RtcCancel
 */
#ifdef OSE64
struct RtcCancel_64
{
   SIGSELECT    sigNo;
   S32          info;
   struct alarm_info_64
   {
      SIGSELECT sigNo;
      SIGSELECT instance;
      uint64_t  pointer;
   } alarmInfo;
};
#endif

/*
 *===========================================================================
 *                        RtcGetTime
 *===========================================================================
 *
 * Description: Get the current system time.
 *
 * Parameters:  sigNo           Signal number.
 *              currentTime     Current system time.
 */

struct RtcGetTime
{
   SIGSELECT    sigNo;
   struct TimePair currentTime;
};

/*
 *===========================================================================
 *                        RtcSetTime
 *===========================================================================
 *
 * Description: Set the system time.
 *
 * Parameters:  sigNo           Signal number.
 *              newTime         New system time.
 */

struct RtcSetTime
{
   SIGSELECT    sigNo;
   struct TimePair newTime;
};

/*
 *===========================================================================
 *                        RtcAdjustTime
 *===========================================================================
 *
 * Description: Adjust the system time gracefully.
 *
 * Parameters:  sigNo           Signal number.
 *              micros          Delta time in microseconds.
 */

struct RtcAdjustTime
{
   SIGSELECT    sigNo;
   S32          micros;
};

/*
 *===========================================================================
 *                        RtcGetTzInfo
 *===========================================================================
 *
 * Description: Get the local time zone offset. If Daylight savings time is
 *              active at the given time, it will be included in the offset.
 *
 * Parameters:  sigNo           Signal number.
 *              time            I: The time
 *              gmtOffset       O: Time zone offset from GMT in seconds.
 *              isdst           O: Flag for Daylight Savings Time
 *              abbr            O: Time zone abbreviation
 */

struct RtcGetTzInfo
{
   SIGSELECT    sigNo;
   time_t       time;
   S32          gmtOffset;
   S32          isdst;
   char         abbr[TZ_ABBR_BUFF_SIZE];
};

/*
 *===========================================================================
 *                        RtcGetLsInfo
 *===========================================================================
 *
 * Description: Get the leap second correction factor for a given time.
 *
 * Parameters:  sigNo      Signal number.
 *              time       I: The time
 *              corr       O: Time correction to be applied
 *              hit        O: Indicates whether this very second is a leap
 *                         second or not.
 */

struct RtcGetLsInfo
{
   SIGSELECT    sigNo;
   time_t       time;
   S32          corr;
   S32          hit;
};

/*
 *===========================================================================
 *                        RtcGetScale
 *===========================================================================
 *
 * Description: Get the OSE RTC's scale factor.
 *
 * Parameters:  sigNo           Signal number.
 *              scale           Scale factor will be put here.
 */

struct RtcGetScale
{
   SIGSELECT    sigNo;
   U32     	scale;
};

/*
 *===========================================================================
 *                        RtcSetScale
 *===========================================================================
 *
 * Description: Set the OSE RTC's scale factor.
 *
 * Parameters:  sigNo           Signal number.
 *	 	newScale        New value of the scale.
 */

struct RtcSetScale
{
   SIGSELECT    sigNo;
   U32          new_scale;
};


/*
 *===========================================================================
 *                       RTC_READY
 *===========================================================================
 *
 * Description: Sent from client to server. Returned from server when it is
 *              operational.
 *
 * Parameters:  None.
 */

/*
 *===========================================================================
 *                        RtcT2subTypeCorr
 *===========================================================================
 *
 * Description:
 *
 *
 * Parameters:
 *
 */

struct RtcT2subTypeCorr
{
   SIGSELECT    sigNo;
   time_t       t;
   S32          offset;
   S32          your_isdst;
};

/*
 *===========================================================================
 *                        RtcGetDstDiff
 *===========================================================================
 *
 * Description: Get the difference between DST and Standard time or
 *              vice versa.
 *
 * Parameters:  sigNo           Signal number.
 *              time            I: The time
 *              dstDiff         O: Time difference in seconds between the current
 *                                 time type and the previos time type (with
 *                                 different isdst flag)
 */

struct RtcGetDstDiff
{
  SIGSELECT     sigNo;
  time_t        t;
  S32           isdst;
  S32           dstDiff;
};

/*
 *===========================================================================
 *                        RtcCompileRequest
 *===========================================================================
 *
 * Description:
 *   Internal signal used in client code within the CRT. (No server part).
 *   This signal is mode-dependent. 
 *
 * Parameters:
 */
struct RtcCompileRequest
{
   SIGSELECT    sigNo;
   const char * zoneName;
   const char * timeZoneRules;
   const char * leapSecondRules;
   char *       errorMessage;
   U32          errorMessageSize;
};

#endif

/* #ifndef _PRIVATE_SIG */

/*
 ****************************************************************************
 *                         END OF FILE
 ****************************************************************************
 */
