/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef SHELL_PRIVATE_H
#define SHELL_PRIVATE_H

#include "ose.h"
#include "osetypes.h"
#include "efs.sig"

#define INVALID_ATTREF ((OSATTREF)~0) /* A invalid attach reference. */
#define INVALID_FD -1          /* The return value of a failed open(). */


/* Return codes used by rcmd */
#define SHELL_LHSH_EINVALID_FD     -1
#define SHELL_LHSH_EOPEN_FAILED    -2
#define SHELL_LHSH_EINVALID_LHPATH -3
#define SHELL_LHSH_EINVALID_CMD    -4
#define SHELL_LHSH_ELOST_SHELLD    -5
#define SHELL_LHSH_ELOST_CMD       -6
#define SHELL_LHSH_ECMD_PROXY      -7
#define SHELL_LHSH_EILLEGAL_CMD    -8

struct Execute
{
    const char *cmd; /* Command name. */
    const char *argv; /* Arguments (including command). */
    int fd0, fd1, fd2;
    const char **envs; /* Pointer to array of const char *. */
    Boolean clone_env; /* True => clone environment to child. */
};

void hunt_for_ck_shelld(const char *lhpath);
int shell_execute_ck(const struct Execute *exec, PROCESS shelld, U32 ck_shell);

/* Private shell functions exported by runcmd.c */

struct ChildDone *
shell_wait_child(PROCESS child_pid,
		 OSATTREF *term_ar);

Boolean shell_open_terminal(char *buffer,
			    size_t bufsize,
			    int *term_fd,
			    OSATTREF *term_ar);

void shell_close_terminal(int term_fd,
			  OSATTREF term_ar);

Boolean shell_subscribe_signal(int term_fd,
			       Boolean enable);

struct PreInterpretedCommand
{
    enum ShellRunCmdStatus status;  /* Keeps status between different
                                     *  stages of command execution. */
    const char *cmd_str;               /* Command string without redirection */
    char *name;                        /* Freed by shell_cleanup_command */
    struct ShellResolveCmd *resolve_cmd;  /* Freed by shell_cleanup_command */
};

void shell_interpreted_command_init(struct PreInterpretedCommand * command,
                                      const char *cmd_str);
void shell_interpreted_command_resolve(struct PreInterpretedCommand * command,
                                       OSBOOLEAN default_command);
void shell_interpreted_command_delete(struct PreInterpretedCommand * command);
void shell_run_cmd_2(struct PreInterpretedCommand * command,
                     const OSADDRESS *taglist);

#endif
