/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 ****************************************************************************
 *                            HEADER
 *
 *     $Workfile:   shelld.sig  $
 *     Document no: @(#) 512/OSE52-14 $Revision:   1.5  $
 *     $Author:   nize  $
 *     $Date:   05/31/00 11:34:14  $
 *
 *
 ****************************************************************************
 */

#ifndef _SHELLD_SIG
#define _SHELLD_SIG

#include "ose.h"
#include "osetypes.h"
#include "ose_spi/efstypes.h"

#define SHELL_SIGBASE 30300 /* Start of internal Shell signals. */

/* Add a user (use send_w_s). shelld will supervise the sender. */

#define SHELL_ADD_USER (30310)	/* !-SIGNO(struct ShellAddUser)-! */
struct ShellAddUser
{
    SIGSELECT sigNo;
    OSUSER userId;
    U32 username; /* Index into string pool. */
    U32 terminal; /* Index into string pool. */
    char s[2]; /* Variable size. */
};
#define allocShellAddUser(length) (struct ShellAddUser *)alloc(sizeof(struct ShellAddUser)+(length), SHELL_ADD_USER)

/* List all users, one request, many replies (always one extra at end). */

#define SHELL_LIST_USERS_REQUEST (30311)	/* !-SIGNO(struct ShellListUsersRequest)-! */
struct ShellListUsersRequest
{
    SIGSELECT sigNo;
};
#define allocShellListUsersRequest() (struct ShellListUsersRequest *)alloc(sizeof(struct ShellListUsersRequest), SHELL_LIST_USERS_REQUEST)

#define SHELL_LIST_USERS_REPLY (30312)	/* !-SIGNO(struct ShellListUsersReply)-! */
struct ShellListUsersReply
{
    SIGSELECT sigNo;
    U32 valid; /* True for all but last (dummy) signal. */
    PROCESS shellPid;
    OSUSER userId;
    U32 username; /* Index into string pool. */
    U32 terminal; /* Index into string pool. */
    char s[2]; /* Variable size. */
};
#define allocShellListUsersReply(length) (struct ShellListUsersReply *)alloc(sizeof(struct ShellListUsersReply)+(length), SHELL_LIST_USERS_REPLY)

/* Add an external shell command. */

#define SHELL_ADD_CMD           (30360) /* !-SIGNO(struct ShellAddCmd)-! */
struct ShellAddCmd
{
    SIGSELECT sigNo;
    U32 name;
    U32 usage;
    U32 description;
    PROCESS proxy;
    OSADDRESS entrypoint;
    U32 proc_type;
    U32 priority;
    U32 stack_size;
    char s[3]; /* Variable size. */
};
#define allocShellAddCmd(length) (struct ShellAddCmd *)alloc(sizeof(struct ShellAddCmd)+(length), SHELL_ADD_CMD)

/* Remove an external shell command. */

#define SHELL_REMOVE_CMD        (30314) /* !-SIGNO(struct ShellRemoveCmd)-! */
struct ShellRemoveCmd
{
    SIGSELECT sigNo;
    OSADDRESS entrypoint;
    char name[1]; /* Variable size. */
};
#define allocShellRemoveCmd(length) (struct ShellRemoveCmd *)alloc(sizeof(struct ShellRemoveCmd)+(length), SHELL_REMOVE_CMD)

/* Return function entrypoint for command. */

#define SHELL_RESOLVE_CMD_REQUEST  (30361) /* !-SIGNO(struct ShellResolveCmd)-! */
#define SHELL_RESOLVE_CMD_REPLY    (30362) /* !-SIGNO(struct ShellResolveCmd)-! */
struct ShellResolveCmd
{
    SIGSELECT sigNo;
    OSADDRESS entrypoint; /* O: */
    PROCESS proxy; /* O: */
    U32 proc_type; /* O: */
    U32 priority; /* O: */
    U32 stack_size; /* O: */
    U32 cmd_name; /* I: Offset into stringpool. */
    U32 username; /* I: Offset into stringpool. */
    U32 terminal; /* I: Offset into stringpool. */
    char s[3]; /* Variable size. */
};
#define allocShellResolveCmdRequest(length) (struct ShellResolveCmd *)alloc(sizeof(struct ShellResolveCmd)+(length), SHELL_RESOLVE_CMD_REQUEST)

/* List all shell commands. */

#define SHELL_LIST_CMD_REQUEST  (30363) /* !-SIGNO(SIGSELECT)-! */
#define SHELL_LIST_CMD_REPLY    (30364) /* !-SIGNO(struct ShellListCmd)-! */
struct ShellListCmd
{
    SIGSELECT sigNo;
    U32 valid; /* True for all but last (dummy) signal. */
    OSADDRESS entrypoint; /* O: */
    U32 proc_type; /* O: */
    U32 priority; /* O: */
    U32 stack_size; /* O: */
    U32 name; /* O: Offset into stringpool. */
    U32 usage; /* O: Offset into stringpool. */
    U32 description; /* O: Offset into stringpool. */
    char s[3]; /* Variable size. */
};
#define allocShellListCmdReply(length) (struct ShellListCmd *)alloc(sizeof(struct ShellListCmd)+(length), SHELL_LIST_CMD_REPLY)

/* Obtain information about the shelld interface. */

#define SHELL_INTERFACE_REQUEST   (30340)       /* !-SIGNO(struct ShellInterfaceRequest)-! */
#define SHELL_INTERFACE_REPLY     (30341)       /* !-SIGNO(struct ShellInterfaceReply)-! */
struct ShellInterfaceRequest
{
    SIGSELECT sigNo;
};
struct ShellInterfaceReply
{
    SIGSELECT sigNo;
    EfsStatus status;           /* O: Status of the operation. */
    char whatStr[32];           /* O: What string of product. */
    U32 biosHandle;             /* O: BIOS handle for trap interface. */
    SIGSELECT sigs[1];          /* O: Supported signals. Variable size. */
};
#define sendShellInterfaceRequest(shelldPidMA) \
do \
{ \
    struct ShellInterfaceRequest *sigMA = (struct ShellInterfaceRequest *)alloc(sizeof(struct ShellInterfaceRequest), SHELL_INTERFACE_REQUEST); \
    send((union SIGNAL **)&sigMA, shelldPidMA); \
} \
while (0)

/* Create a child process for specified command.
 * Sender is pid of the created process (in addition to the local pid in the signal). 
 *
 * Returned status and error codes:
 * EFS_SUCCESS                          Success.
 * SHELL_CREATE_CHILD_EUNKNOWN_CMD      Unknown command (0xdeadbabe == -559039810).
 *
 * Added error codes:
 * SHELL_CREATE_CHILD_ECMD_PROXY        Lost contact with command proxy process.
 * SHELL_CREATE_CHILD_EILLEGAL_CMD      The command is not allowed to be run. 
 *                                      Function @link "validate_user.h" "validate_cmd" returned False.
 *                                      Additional status string from validate_cmd() is returned
 *                                      in string pool starting at index 0.
 */
#define SHELL_CREATE_CHILD_EUNKNOWN_CMD       (0xdeadbabe)
#define SHELL_CREATE_CHILD_ECMD_PROXY         (-1)
#define SHELL_CREATE_CHILD_EILLEGAL_CMD       (-2)

#define SHELL_CREATE_CHILD_REQUEST  (30342) /* !-SIGNO(struct ShellCreateChild)-! */
#define SHELL_CREATE_CHILD_REPLY    (30343) /* !-SIGNO(struct ShellCreateChild)-! */
struct ShellCreateChild
{
    SIGSELECT sigNo;
    EfsStatus status;           /* O: Status of the operation. */
    PROCESS child_pid;          /* O: Local pid of the created process. */
    OSATTREF att_ref;           /* O: Attref to be specified in start req. */
    EfsBitmask options;         /* I: Options (none are yet defined). */
    U32 proc_type;              /* I: Default process type. */
    U32 priority;               /* I: Default priority. */
    U32 user_id;                /* I: User ID. */
    U32 cmd_name;               /* I: command name (offset into string pool). */
    U32 proc_name;              /* I: process name (offset into string pool). */
    U32 argv;                   /* I: argument vector (offset into string pool). */
    U32 wdir;                   /* I: working directory (offset into string pool). */
    U32 username;               /* I: user name (offset into string pool). */
    U32 terminal ;              /* I: terminal (offset into string pool). */
    char s[6];                  /* IO: String pool (variable size). */
};
#define sendShellCreateChildRequest(optionsMA, proc_typeMA, priorityMA, user_idMA, _cmd_nameMA, _proc_nameMA, _argvMA, _wdirMA, _usernameMA, _terminalMA, shelldPid) \
do \
{ \
    const char *cmd_nameMA = (_cmd_nameMA); \
    const char *proc_nameMA = (_proc_nameMA); \
    const char *argvMA = (_argvMA); \
    const char *wdirMA = (_wdirMA); \
    const char *usernameMA = (_usernameMA); \
    const char *terminalMA = (_terminalMA); \
    struct ShellCreateChild *sigMA = (struct ShellCreateChild *)alloc(sizeof(struct ShellCreateChild) + strlen(cmd_nameMA) + strlen(proc_nameMA) + strlen(wdirMA) + strlen(argvMA) + strlen(usernameMA) + strlen(terminalMA), SHELL_CREATE_CHILD_REQUEST); \
    sigMA->options = (optionsMA); \
    sigMA->proc_type = (proc_typeMA); \
    sigMA->priority = (priorityMA); \
    sigMA->user_id = (user_idMA); \
    sigMA->cmd_name = 0; \
    sigMA->proc_name = sigMA->cmd_name + strlen(cmd_nameMA) + 1; \
    sigMA->argv = sigMA->proc_name + strlen(proc_nameMA) + 1; \
    sigMA->wdir = sigMA->argv + strlen(argvMA) + 1; \
    sigMA->username = sigMA->wdir + strlen(wdirMA) + 1; \
    sigMA->terminal = sigMA->username + strlen(usernameMA) + 1; \
    strcpy(&sigMA->s[sigMA->cmd_name], cmd_nameMA); \
    strcpy(&sigMA->s[sigMA->proc_name], proc_nameMA); \
    strcpy(&sigMA->s[sigMA->argv], argvMA); \
    strcpy(&sigMA->s[sigMA->wdir], wdirMA); \
    strcpy(&sigMA->s[sigMA->username], usernameMA); \
    strcpy(&sigMA->s[sigMA->terminal], terminalMA); \
    send((union SIGNAL **)&sigMA, (shelldPid)); \
} \
while (0)

/* Set an environment variable on the specified process. */

#define SHELL_SETENV_CHILD (30344) /* !-SIGNO(struct ShellSetenvChild)-! */
struct ShellSetenvChild
{
    SIGSELECT sigNo;
    PROCESS child_pid;          /* I: Local pid of process to receive environment variable. */
    char assignment[1];         /* I: "var=name" (variable size). */
};
#define sendShellSetenvChild(child_pidMA, _assignmentMA, shelldPid) \
do \
{ \
    const char *assignmentMA = (_assignmentMA); \
    struct ShellSetenvChild *sigMA = (struct ShellSetenvChild *)alloc(sizeof(struct ShellSetenvChild) + strlen(assignmentMA), SHELL_SETENV_CHILD); \
    sigMA->child_pid = (child_pidMA); \
    strcpy(sigMA->assignment, assignmentMA); \
    send((union SIGNAL **)&sigMA, (shelldPid)); \
} \
while (0)

/* Set an environment variable on the specified process. */

#define SHELL_SETENV_CHILD_SENDER (30345) /* !-SIGNO(struct ShellSetenvChildSender)-! */
struct ShellSetenvChildSender
{
    SIGSELECT sigNo;
    PROCESS child_pid;          /* I: Local pid of process to receive environment variable. */
    char assignment[1];         /* I: "var=name" (variable size). */
};
#define sendShellSetenvChildSender(child_pidMA, _assignmentMA, senderPid, shelldPid) \
do \
{ \
    const char *assignmentMA = (_assignmentMA); \
    struct ShellSetenvChildSender *sigMA = (struct ShellSetenvChildSender *)alloc(sizeof(struct ShellSetenvChildSender) + strlen(assignmentMA), SHELL_SETENV_CHILD_SENDER); \
    sigMA->child_pid = (child_pidMA); \
    strcpy(sigMA->assignment, assignmentMA); \
    send_w_s((union SIGNAL **)&sigMA, (senderPid), (shelldPid)); \
} \
while (0)

/* Start the specified process. */

#define SHELL_START_CHILD          (30346) /* !-SIGNO(struct ShellStartChild)-! */
struct ShellStartChild
{
    SIGSELECT sigNo;
    PROCESS child_pid;          /* I: Local pid of child process to start. */
    OSATTREF att_ref;           /* I: Attref returned in create child reply. */
};
#define sendShellStartChild(child_pidMA, att_refMA, shelldPid) \
do \
{ \
    struct ShellStartChild *sigMA = (struct ShellStartChild *)alloc(sizeof(struct ShellStartChild), SHELL_START_CHILD); \
    sigMA->child_pid = (child_pidMA); \
    sigMA->att_ref = (att_refMA); \
    send((union SIGNAL **)&sigMA, (shelldPid)); \
} \
while (0)

/* Stop the specified process. */

#define SHELL_STOP_CHILD          (30347) /* !-SIGNO(struct ShellStopChild)-! */
struct ShellStopChild
{
    SIGSELECT sigNo;
    PROCESS child_pid;          /* I: Local pid of child process to stop. */
};
#define sendShellStopChild(child_pidMA, shelldPid) \
do \
{ \
    struct ShellStopChild *sigMA = (struct ShellStopChild *)alloc(sizeof(struct ShellStopChild), SHELL_STOP_CHILD); \
    sigMA->child_pid = (child_pidMA); \
    send((union SIGNAL **)&sigMA, (shelldPid)); \
} \
while (0)

/* Kill the specified process. */

#define SHELL_KILL_CHILD          (30348) /* !-SIGNO(struct ShellKillChild)-! */
struct ShellKillChild
{
    SIGSELECT sigNo;
    PROCESS child_pid;          /* I: Local pid of child process to kill. */
};
#define sendShellKillChild(child_pidMA, shelldPid) \
do \
{ \
    struct ShellKillChild *sigMA = (struct ShellKillChild *)alloc(sizeof(struct ShellKillChild), SHELL_KILL_CHILD); \
    sigMA->child_pid = (child_pidMA); \
    send((union SIGNAL **)&sigMA, (shelldPid)); \
} \
while (0)

/* Read an environment variable from the specified process. */

#define SHELL_GETENV_CHILD_REQUEST (30349) /* !-SIGNO(struct ShellGetenvChildRequest)-! */
#define SHELL_GETENV_CHILD_REPLY   (30350) /* !-SIGNO(struct ShellGetenvChildReply)-! */
struct ShellGetenvChildRequest
{
    SIGSELECT sigNo;
    PROCESS child_pid;          /* Local pid of process to receive environment variable. */
    char name[1];	        /* Name of environment variable to read (variable size). */
};
struct ShellGetenvChildReply
{
    SIGSELECT sigNo;
    EfsBoolean success;		/* Status of the operation, True or False (invalid pid, variable not set). */
    PROCESS child_pid;          /* Local pid of process to receive environment variable. */
    char assignment[1];	        /* "name=value" (variable size). */
};
#define sendShellGetenvChildRequest(child_pidMA, _nameMA, shelldPid) \
do \
{ \
    const char *nameMA = (_nameMA); \
    struct ShellGetenvChildRequest *sigMA = (struct ShellGetenvChildRequest *)alloc(sizeof(struct ShellGetenvChildRequest) + strlen(nameMA), SHELL_GETENV_CHILD_REQUEST); \
    sigMA->child_pid = (child_pidMA); \
    strcpy(sigMA->name, nameMA); \
    send((union SIGNAL **)&sigMA, (shelldPid)); \
} \
while (0)

#endif
