/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 1998-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 ****************************************************************************
 *                            PROGRAM MODULE
 *
 *     $Workfile:   bios_c.c  $
 *     Document no: @(#) 511/OSE129-1 $FileRevision:   /main/tb_current_ose5/5  $
 *     $Author:   anha  $
 *     $Date:   06/01/06 14:28:06  $
 *
 *
 ****************************************************************************

	CONTENTS
	--------

	1  Description
	2  History of development
	3  Include files
	4  Signal include files
	5  Manifest constants
	6  Macros
	7  Types
	8  Function prototypes
	9  Signal definitions
	10 Variables
	11 Functions

 ****************************************************************************
 */

/*
 ****************************************************************************
 * 1  DESCRIPTION.
 *
 * This file implements some maintenance functions for the trap handler (BIOS)
 */

/*
 ****************************************************************************
 * 2  HISTORY OF DEVELOPMENT.
 *
 *    $Log:   /rtt/pvcsarch/OSE129/src/src-common/bios_c.c_v  $
 * 
 *    Rev 1.2   05/25/00 15:58:00   gunnar
 * 
 *    Rev 1.1   09/07/99 18:12:30   gunnar
 * Softkernel adaptions
 * 
 *    Rev 1.0   02/23/99 16:01:26   gunnar
 * Initial revision.
 * 
 */

/*
 ****************************************************************************
 * 3  INCLUDE FILES.
 ****************************************************************************
 */

#include <stddef.h>
#include <string.h>
#include "osarch.h"	
#include "osetypes.h"	
#include "ose.h"	
#include "core_internal.h"	/* Internal Core header. */
#include "exec_start.h"		/* For zzinitBios() */
#ifdef OSE_SMP
#include "cpu_smp.h"
#endif
#include "spinlock.h"
#include "bios.h"		/* BIOS (Trap Handler) interface. */


/*
 ****************************************************************************
 * 4  SIGNAL INCLUDE FILES.
 ****************************************************************************
 */


/*
 ****************************************************************************
 * 5  MANIFEST CONSTANTS.
 ****************************************************************************
 */


/*
 ****************************************************************************
 * 6  MACROS.
 ****************************************************************************
 */
#define SPIN_LOCK_SAVE(lock, sr) spin_lock_irqsave(lock, sr) 
#define SPIN_LOCK_RESTORE(lock, sr) spin_unlock_irqrestore(lock, sr) 

/*
 ****************************************************************************
 * 7  TYPES.
 ****************************************************************************
 */

struct BiosModule		/* Do not rely on sizeof !!! */
{
  BiosFunction *func;		/* Must be first - ref from assembly */
  U8 runlevel;
  U8 filler;
  U16 flags;	
  char name[1];			/* Do not rely on sizeof !!! */
};

/* Data published by bios_s.s */
struct BiosManifest
{
  U32 *table;			/* Table base */
  U32 slots;			/* # entries */
  U32 entrySize;		/* Size of each entry */
  int (* calledFromUserMode)(void); /* Returns zero if supervisor mode called,
				     or if this can't be decided. */
};

/*
 ****************************************************************************
 * 8  FUNCTION PROTOTYPES.
 ****************************************************************************
 */

#ifdef softOSE
static int zzcalledFromUserMode(void); /* Returns zero if supervisor mode called,
					 or if this can't be decided. */
long zzbiosCall(unsigned long handle, long arg1, long arg2, long arg3,
		long arg4, long arg5, long arg6, long arg7);
#endif

long zzbiosError_(long arg1, long arg2, long arg3, long arg4, long arg5,
		  long arg6, long arg7);

/*
 ****************************************************************************
 * 10  VARIABLES.
 ****************************************************************************
 */
#define bios ZZB0107

#ifdef softOSE
#define SLOTS		64
#define ENTRY_SIZE	32
static U32		functions[(SLOTS * ENTRY_SIZE - 1 + sizeof(U32))/sizeof(U32)];
const static struct BiosManifest bios =
{
  functions, SLOTS, ENTRY_SIZE, zzcalledFromUserMode
};
#else
extern struct BiosManifest bios;
#endif

static spinlock_t lock = SPIN_LOCK_INIT;
static U32		nameLength;
static volatile U32	currentRunlevel;
static volatile U32	non_duplicate_install_count;
static int 		initiated;

/*
 ****************************************************************************
 * 11  FUNCTIONS.
 ****************************************************************************
 */

#ifdef softOSE
extern long (*crt_zzbiosCall)(unsigned long handle, long arg1, long arg2, long arg3,
			      long arg4, long arg5, long arg6, long arg7);

long zzbiosCall(unsigned long handle, long arg1, long arg2, long arg3,
		long arg4, long arg5, long arg6, long arg7)
{
  struct BiosModule *module = (struct BiosModule *) &functions[handle*bios.entrySize/sizeof functions[0]];
  return module->func(arg1, arg2, arg3, arg4, arg5, arg6, arg7);
}

static int zzcalledFromUserMode(void) /* Returns zero if supervisor mode called,
					 or if this can't be decided. */
{return 0;}

#endif


/*
** This is a wildcard comparison utility.
** It was written by Peter Bergsten in 1995.
**
** The function compares a string against a wildcard that contains
** string fragments mixed with the usual '*' and '?' characters.
** The e parameter contains the wildcard and the c parameter contains
** the non wildcard string to check.
**
** returns zero if the strings match each other and a nonzero value
** if they don't.
**
** Expansion rules: '*' matches zero or more characters.
**                  '?' matches any single character except the string terminator.
**                   Note that "a*" matches "a", but "a?" does not match "a".
**
** The strings are normally traversed until a mismatch occurs or until the
** end of the e-string. If the wildcard ends with a '*' however, the strings
** are only traversed until the end of the wildcard. Thus the "*" wildcard
** causes no loops at all, only three if-statements.
*/

static int estrcmp(char *e, char *c)
{
  char *r = (char *) 0;
  while(*e || *c)
    {
      if(*e == '*')
	{
	  e++;
	  /* Test on the special case of a '*' at the tail. */
	  if(!*e)
	    {
	      return 0;
	    }
	  r = e;
	}
      else if(*e == '?' && *c)
	{
	  e++;
	  c++;
	}
      else if(*e == *c)
	{
	  e++;
	  c++;
	}
      else if(!*c)
	return 1;
      else if(!r)
	return 1;
      else
	{
	  if(e != r)
	    {
	      c -= e - r - 1;
	      e = r;
	    }
	  else
	    c++;
	}
    }
  return 0;
}

/*
 *===========================================================================
 *                      zzbiosError
 *===========================================================================
 * Description: Handle Bios call with invalid handler argument
 *              Handle was zero or module with the given handle not installed.
 *
 * Parameters:  Any garbage, probably
 *
 * Returns:     -1
 */
long zzbiosError_(long arg1, long arg2, long arg3, long arg4, long arg5,
	       long arg6, long arg7)
{
  /* Touch arguments to remove compiler warnings */
  (void) arg1; (void) arg2; (void) arg3; (void) arg4; (void) arg5;
  (void) arg6; (void) arg7;
  
  return(-1);
}

/*
 *===========================================================================
 *                      zzbiosInstall
 *===========================================================================
 * Description: -
 *
 * Parameters:  name          name of module to install.
 *              func          entrypoint of BIOS module.
 *              flags         bios module flags
 *
 * Returns:     success or error code.
 */

static long zzbiosInstall(const char *name,
			  long (*func)(long arg1, long arg2, long arg3, long arg4, long arg5, long arg6, long arg7),
			  unsigned long flags)
{
  Msr			msr;
  unsigned 		i;
  struct BiosModule 	*mod;
  U32			install_count;
  unsigned		len = strlen(name);
	
  if (len < 1 || len > nameLength) return BIOS_ENAME_TOO_LONG;
  if (initiated && bios.calledFromUserMode()) return BIOS_EUSER_MODE;

  /* NOTE! We will install a first module with handle zero!
     biosOpen will never open this one, but we will end up
     in it when an openBios fails and returns zero, and the
     user does not check that! 
  */

  mod = (struct BiosModule *) bios.table;

 rescan:
  install_count = non_duplicate_install_count;
  for (i = 1; i < bios.slots; i++)
    {				/* skip index 0 here */
      mod = (struct BiosModule *) (bios.entrySize + (char *) mod);
      SPIN_LOCK_SAVE(&lock, msr);
      if (mod->name[0] && (strcmp(name, mod->name) == 0))
	{
	  if (flags & BIOS_FORCE_INSTALL)
	    {
	      mod->func = func;
	      mod->flags = (U16) flags;
	      mod->runlevel = (U8) currentRunlevel;
	      SPIN_LOCK_RESTORE(&lock, msr);
	      return BIOS_SUCCESS; /* FORCED DUPLICATE RETURN */
	    }
	  SPIN_LOCK_RESTORE(&lock, msr);
	  return BIOS_EDUPLICATE_NAME; /* DUPLICATE RETURN */
	}
      SPIN_LOCK_RESTORE(&lock, msr);
    } /* for */

  /* Well, now we know that some time ago there was no function
   * with this name. However, we can not know wheather there is
   * one now! On the other hand, we do not want to run the
   * entire table check with interrupts disabled!
   * So, we check below if we were interrupted here by another
   * process doing biosInstall(). In that case we rescan the
   * entire table.
   */
  mod = (struct BiosModule *) bios.table;
  for (i = 1; i < bios.slots; i++)
    {				/* skip index 0 here */
      mod = (struct BiosModule *) (bios.entrySize + (char *) mod);
      SPIN_LOCK_SAVE(&lock, msr);
      if (mod->name[0] == 0)	/* Found available slot. */
	{
	  if (install_count != non_duplicate_install_count)
	    {
	      SPIN_LOCK_RESTORE(&lock, msr);
	      goto rescan;	/* Someone else has installed, so rescan! */
	    }
	  non_duplicate_install_count++;
	  mod->func = func;
	  mod->flags = (U16) flags;
	  mod->runlevel = (U8) currentRunlevel;
	  strcpy(mod->name, name);
	  SPIN_LOCK_RESTORE(&lock, msr);
	  return BIOS_SUCCESS;	/* SUCCESS RETURN */
	}
      SPIN_LOCK_RESTORE(&lock, msr);
    }
  return BIOS_ENO_TABLE_SPACE;	/* NO SPACE RETURN */
}

/*
 *===========================================================================
 *                      zzbiosOpen
 *===========================================================================
 * Description: -
 *
 * Parameters:  name          name of the BIOS module to open.
 *
 * Returns:     handle to BIOS module, or 0 if module not found.
 */

static unsigned long zzbiosOpen(const char *aname)
{
  Msr			msr;
  unsigned 		handle;
  unsigned long 	flags;
  struct BiosModule 	*mod = (struct BiosModule *) bios.table;
  char name[BIOS_NAME_SIZE];

  /*
  ** Copy the name to local stack, to avoid page fault in the LOCK region
  ** below. Don't care if the null terminator is copied, because we know
  ** that the bios table has a null terminator, that strcmp will find.
  */
  strncpy(name, aname, sizeof(name)); 

  for (handle = 1; handle < bios.slots; handle++)
    {				/* skip index 0 here */
      mod = (struct BiosModule *) (bios.entrySize + (char *) mod);
      SPIN_LOCK_SAVE(&lock, msr);
      if (mod->name[0] && (strcmp(name, mod->name) == 0))
	{
      /* Well, we should check supervisor mode flag here !
	 Better here than for each call. If an application passes
	 a supervisor handle to a user task? I do not think we have to
	 check that for every call, a check at open will probably
	 do fine.
      */
	  flags = mod->flags;
#ifdef OSE_SMP
	  if(flags & BIOS_SMP_UNSAFE)
	    {
	      if(0 != zzkrn_cpu_id())
		{
		  handle = 0;	/* Only permitted on cpu 0! */
		}
	      else
		{
		  /* Inhibit migration of current segment to another cpu. */
		  zzkrn_segment_migration_status(current_process(),
						 MIGRATION_MASK_BIOS,
						 MIGRATION_MASK_BIOS);
		}
	    }
#endif
	  SPIN_LOCK_RESTORE(&lock, msr);

	  if(handle && !(flags & BIOS_USERMODE_ALLOWED) &&
	     0 == zzkrn_get_pid_info(current_process(), 10)) /* user mode */
	    {
	      handle = 0;
	    }
	  
	  return handle;
	}
      SPIN_LOCK_RESTORE(&lock, msr);
    }
  return 0;			/* Not found */
}

/*
 *===========================================================================
 *                      zzbiosUninstall
 *===========================================================================
 * zzbiosUninstall uninstalls a number of handlers matching the wild card
 * argument. Use this function with care. Later calls to biosInstall will
 * reuse the released handles.
 *
 * Wildcard expansion rules: 
 *		'*' matches zero or more characters.
 *		'?' matches any single character except the string terminator.
 *
 * zzbiosUninstall returns the number of functions removed when called from 
 * supervisor mode resp. -1 when illegally called from user mode.
 */
static long zzbiosUninstall(char * wildcardName)
{
  Msr			msr;
  unsigned 		i;
  unsigned 		count = 0;
  struct BiosModule 	*mod = (struct BiosModule *) bios.table;

  if (bios.calledFromUserMode()) return -1;
	
  /* NOTE! We will install a first module with handle zero!
     biosOpen will never open this one, but we will end up
     in it when an openBios fails and returns zero, and the
     user does not check that! 
  */
  for (i = 1; i < bios.slots; i++)
    {				/* skip index 0 here */
      mod = (struct BiosModule *) (bios.entrySize + (char *) mod);
      SPIN_LOCK_SAVE(&lock, msr);
      if (mod->name[0] && estrcmp(wildcardName, mod->name) == 0)
	{
	  mod->func = & zzbiosError_;
	  mod->name[0] = 0;
	  count++;
	}
      SPIN_LOCK_RESTORE(&lock, msr);
    }
  return count;
}

/*
 *===========================================================================
 *                      zzbiosRunlevel
 *===========================================================================
 * zzbiosRunlevel sets the current run level.
 * Runlevel is a number in the range 0:255. 
 *
 * zzbiosRunlevel returns the previous run level.
 * Calls are only permitted from supervisor mode.
 * When called from user mode, -1 is immediately returned.
 *
 * 
 */
static long zzbiosRunlevel(long runlevel)
{
  Msr			msr;
  U32 			prevRunlevel;
  
  if (bios.calledFromUserMode()) return -1;

  SPIN_LOCK_SAVE(&lock, msr);
  prevRunlevel = currentRunlevel;
  currentRunlevel = (runlevel > 255 || runlevel < 0) ? 255 : runlevel;
  SPIN_LOCK_RESTORE(&lock, msr);
  return prevRunlevel;
}

/*
 *===========================================================================
 *                      zzbiosList
 *===========================================================================
 * zzbiosList returns information on installed bios functions.
 * Each call returns one function. The search starts from
 * the prev_handle, normally returned by the previous call.
 * Set prev_handle to zero in the first call.
 * Wildcard expansion rules: 
 *		'*' matches zero or more characters.
 *		'?' matches any single character except the string terminator.
 *
 * zzbiosList returns BIOS_SUCCESS when a function is found
 * and BIOS_ENO_ENTRY for no find.
 */
static long zzbiosList(char * wildcardName, unsigned long prevHandle,
		       struct BiosList * biosInfo)
{
  Msr			msr;
  long 			i;
  struct BiosModule 	*mod = (struct BiosModule *)
    (bios.entrySize * prevHandle + (char *) bios.table);

  /* NOTE! We will install a first module with handle zero!
     biosOpen will never open this one, but we will end up
     in it when an openBios fails and returns zero, and the
     user does not check that! 
  */
  for (i = prevHandle + 1; i < (long) bios.slots; i++)
    {				
      mod = (struct BiosModule *) (bios.entrySize + (char *) mod);
      SPIN_LOCK_SAVE(&lock, msr);
      if (mod->name[0] && estrcmp(wildcardName, mod->name) == 0)
	{
	  biosInfo->entrypoint = mod->func;
	  biosInfo->runlevel = mod->runlevel;
	  biosInfo->handle = i;
	  biosInfo->flags = mod->flags;
	  if (i == 1)
	  {
             /* Modify flags for biosInstall. */
             biosInfo->flags &= ~BIOS_USERMODE_ALLOWED;
	  }
	  strcpy(biosInfo->name, mod->name);
	  SPIN_LOCK_RESTORE(&lock, msr);
	  return BIOS_SUCCESS;
	}
      SPIN_LOCK_RESTORE(&lock, msr);
    }
  return BIOS_ENO_ENTRY;
}


/*
 *===========================================================================
 *                      zzinitBios
 *===========================================================================
 * Description: Initiate bios tables
 *
 * Parameters:  -
 *
 * Returns:     -
 */

void zzinitBios(void)
{
  unsigned i;
  BiosFunction **funcpp = (BiosFunction **) bios.table;

#ifdef LIBWHAT
  LIBWHAT
    ;
#endif

  (void)lock;

  /* Initiate static vars */

  initiated = 0;
  currentRunlevel = 0;
  non_duplicate_install_count = 0;
  nameLength = bios.entrySize - offsetof(struct BiosModule, name) - 1;
  if (nameLength > (BIOS_NAME_SIZE - 1)) nameLength = (BIOS_NAME_SIZE - 1);

  /* Zero table. Set all functions to zzbiosError */
  
  memset(bios.table, 0, (size_t) (bios.slots*bios.entrySize));
  for (i = 0; i < bios.slots; i++)
    {
      *funcpp = & zzbiosError_;
      funcpp = (BiosFunction **)(bios.entrySize + (char *)funcpp);
    }
  /* Install remaining standard handles  (Number 0 already handled.)*/

  /* Note that BiosInstall checks for user mode by itself! */
  zzbiosInstall("biosInstall", (BiosFunction *) zzbiosInstall, BIOS_USERMODE_ALLOWED); 
  zzbiosInstall("biosOpen", (BiosFunction *) zzbiosOpen, BIOS_USERMODE_ALLOWED);
  zzbiosInstall("biosUninstall", (BiosFunction *) zzbiosUninstall, BIOS_SUPERVISOR_ONLY);
  zzbiosInstall("biosRunlevel", (BiosFunction *) zzbiosRunlevel, BIOS_SUPERVISOR_ONLY);
  zzbiosInstall("biosList", (BiosFunction *) zzbiosList, BIOS_USERMODE_ALLOWED);
  initiated = 1;
#ifdef softOSE
  crt_zzbiosCall = zzbiosCall;
#endif
}
