/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

/*
** Executive entry point zzstart_OSE.
**
** The entry point shall be called after initiation of static data
** and clearing of bss sections. The entry point must be called in
** supervisor mode, and with at least 64 bytes stack space available
** on the current stack. Interrupts will be disabled by the kernel
** if the were enabled at the entry.
*/


/*
*****************************************************************************
** Header files
*****************************************************************************
**/

/* Make sure we don't use debug versions of system calls internally. */

#ifdef OS_DEBUG
#undef OS_DEBUG
#endif

#include "string.h"
#include "osetypes.h"		/* OSE types. */
#include "osenames.h"		/* Names for all public symbols.	*/
#include "ose.h"		/* Types and system calls. */
#include "ose_err.h"		/* Error codes. */
#include "pose.con"		/* Configurable types and constants.	*/
#include "pose.h"		/* Target interface.			*/

#include "krn_internal.h"	/* MM interface. */
#include "cpu_hal.h"		/* CPU probing. */
#include "cpu_hal_arch.h"	/* CPU probing. */
#include "krncon.h"		/* KRN configuration. */

#include "exec_start.h"		/* Exec start sequence. */

#include "bios.h"		/* Bios interface. */
#include "cache.h"		/* Cache interface. */
#include "ramlog.h"
#include "ramlog_trap.h"
#include "sysparam_trap.h"
#include "hal.h"


/* FIXME: Put in an include file */
extern void cache_init_bios_interface(void);

/*
*****************************************************************************
** Local definitions
*****************************************************************************
**/

#define END_MARK 0xEE		/* Stack endmark */

typedef void (Start_handler0)(void);
typedef void (Start_handler1)(void);
typedef OSBOOLEAN (Warm_start_handler)(void);

static void exec_start(void);
static void fatal_error(OSERRCODE code, OSERRCODE extra);

/*
*****************************************************************************
** Local variables
*****************************************************************************
**/
/* zzcpu_probe_h_list is declared by krncon.c. */
extern const struct cpu_descriptor *zzcpu_probe_h_list[];

static unsigned char *start_stack_end_mark;
static OSERRH *error_handler;
/* The selected cpu hal: */
static const struct cpu_descriptor *cpu_hal = NULL; 

const struct cpu_descriptor *zzcpu_hal(void)
{
  return cpu_hal;
}

/*
*****************************************************************************
** Architecture Specific stuff and corresponding defaults.
*****************************************************************************
**/

#if defined(OSARM)
#if defined (__GNUC__)
#define SET_STACK_AND_JUMP(entrypoint, start_stack_sp) set_stack_and_jump(entrypoint, start_stack_sp)
static __inline__ void
set_stack_and_jump(void (*entrypoint)(void), OSADDRESS start_stack_sp)
{
    __asm__ volatile
    (
        " mov r13,%0"
        : /*no outputs*/
        : /*inputs*/ "r" (start_stack_sp)
    );
    __asm__ volatile
    (
        " mov r15,%0"
        : /*no outputs*/
        : /*inputs*/ "r" (entrypoint)
	: /*clobbers*/ "r15"
    );
  /* does not return! */
}


#elif defined(__ARMCC_VERSION)	/* Any ARM Ltd. Compiler */
#if __TARGET_ARCH_THUMB < 3
/* Thumb2 code not available. Select ARM code generation. */
#pragma arm
#endif			
#pragma O0			/* Avoid tail call optimizer (iar). */
#define SET_STACK_AND_JUMP(entrypoint, start_stack_sp) set_stack_and_jump(entrypoint, start_stack_sp)
__asm void set_stack_and_jump(void (*entrypoint)(void), OSADDRESS start_stack_sp)
{
  mov	sp,r1		;
  bx	r0		;
  ; does not return;

}
#endif /* compiler */
#endif /* OSARM */

#if defined(OSMIPS)
#if defined (__GNUC__)
#define SET_STACK_AND_JUMP(entrypoint, start_stack_sp) set_stack_and_jump(entrypoint, start_stack_sp)
static __inline__ void
set_stack_and_jump(void (*entrypoint)(void), OSADDRESS start_stack_sp)
{
    __asm__ volatile
    (
        " or $8,$0,%1\n"
        " or $29,$0,%0\n"
	" j $8\n"
	" nop"
        : /*no outputs*/
        : /*inputs*/ "r" (start_stack_sp),  "r" (entrypoint)
    );
    /* does not return! */
}

static __inline__ cpuid_t
get_cpu_id_from_gp(void)
{
   cpuid_t id;
   __asm__ __volatile__ (" ori %0,$28,0" :"=r"(id):);
   return id;
}

#endif
#endif

#if defined(OSPP)
#if defined (__GNUC__)
#define SET_STACK_AND_JUMP(entrypoint, start_stack_sp) set_stack_and_jump(entrypoint, start_stack_sp)
static __inline__ void
set_stack_and_jump(void (*entrypoint)(void), OSADDRESS start_stack_sp)
{
   int zero = 0;
    __asm__ volatile
    (
        " addi %%r1,%0,-16\n"	/* Sp must start inside the stack. */
        " stw %2,0(%%r1)\n"	/* Terminate backtrace. */
	" mtspr %%ctr,%1\n"
	" bctr"
        : /*no outputs*/
        : /*inputs*/ "b" (start_stack_sp), "r" (entrypoint), "r" (zero)
    );
    /* does not return! */
}

/* Init r2 to select non trap mode (supervisor mode). */
#define INIT_REGS() \
	__asm__ volatile(" addis %%r2,%%r0,0xbbbb\n ori %%r2,%%r2,3\n" ::)

#endif
#elif defined(OSARM)
#define INIT_REGS()
#else
#define INIT_REGS()
#endif /* OS architecture */

#if defined(softOSE)
#endif

#if defined(OSX86)
#if defined (__GNUC__)
#define SET_STACK_AND_JUMP(entrypoint, start_stack_sp) set_stack_and_jump(entrypoint, start_stack_sp)
static __inline__ void
set_stack_and_jump(void (*entrypoint)(void), OSADDRESS start_stack_sp)
{
    __asm__ volatile
    (
        " movl %0, %%esp"
        : /*no outputs*/
        : /*inputs*/ "r" (start_stack_sp)
    );
    __asm__ volatile
    (
        " jmp *%0"
        : /*no outputs*/
        : /*inputs*/ "r" (entrypoint)
    );
  /* does not return! */
}

static __inline__ cpuid_t
get_cpu_id_from_gp(void)
{
   cpuid_t id;
   __asm__ __volatile__ (" mov %%cr3, %0" :"=r"(id):);
   return id;
}

#endif /* compiler */
#endif /* OSX86 */


/*
** Continue in next function on the proper start stack.
** This default doesn't use the provided stack!
*/
#ifndef SET_STACK_AND_JUMP
#define SET_STACK_AND_JUMP(entrypoint, start_stack_sp) entrypoint()
#endif

/* Decrementing (1) or incrementing (0) stacks */
#ifndef DECREMENTING_STACKS
#define DECREMENTING_STACKS 1	/* For all supported architectures so far. */
#endif

/* Stack frame alignment */
#ifndef STACK_FRAME_ALIGNMENT
#define STACK_FRAME_ALIGNMENT 16ul /* Architecture dependent frame alignment */
#endif

/*
*****************************************************************************
** Functions
*****************************************************************************
**/

/*
** Call system error handler with a fatal error bit set.
*/
static void fatal_error(OSERRCODE code, OSERRCODE extra)
{
  code |= OSE_EFATAL_MASK;
  for(;;)
    {
      if(error_handler)
	{
	  error_handler(0, code, extra);
	}
    } /* for */
}

/*
** Start the kernel.
**
** This function runs on the OSE start stack with interrupts disabled.
*/

static void exec_start(void)
{
  /* Entrypoint for system call handler. */
  extern long zzsystem_callh(long, long, long, long, long, long, long);
  U32 cpu_id;
  U32 vector_base;		/* Base address of vector area. */
  U32 vector_size;		/* Size of the entire vector area. */
  U32 pose_start_mode;		/* Warm or cold start  */
  struct Ti_ctlblk *ti_conf = &zzconf; /* Configuration of Target Interface. */
  const struct cpu_descriptor **cpu_probe_list = zzcpu_probe_h_list;
  const struct cpu_descriptor **cpu_probe = NULL; 
  struct mm_handlers *mm_handlers; /* chip dependent mm handlers */


  *start_stack_end_mark = END_MARK;
#if defined(OSE_SMP) && defined(OSARM)
  zzkrn_set_cpu_id_arm();
#endif

  /* 
   * Get first parts of configuration data, including list of start
   * handler 0. 
   */
  ti_conf->magic = TI_CTLBLK_MAGIC;
  ti_conf->struct_size = sizeof(*ti_conf);
  zzose_con_0(ti_conf);

  /* Call start_handler0 handlers. */
  {
    Start_handler0 **handler_list =
      (Start_handler0 **) ti_conf->start_handler0_list;

    if (handler_list)
    {
       while (*handler_list)
       {
          (*handler_list)();	/* Call handler. */
          handler_list++;
       }
    }
  }


  /*
  ** Probe CPU HAL with hw chip id (if an id was found and non-zero).
  ** This cpu probing has to be done very early. For example, on ARM1136,
  ** the probe function may need to setup handling of misaligned memory 
  ** accesses before system parameters can be parsed.
  */
  cpu_id = zzget_cpu_id();
  if(cpu_id)
    {
      for(cpu_probe = cpu_probe_list; *cpu_probe; cpu_probe++)
	{
	  if((*cpu_probe)->probe(cpu_id))
	    {
	      goto probe_ready;	/* Successful match found. */
	    }
	} /* for */
    }

  /*
  ** If no match, Probe CPU HAL with configured cpu id converted to pvr.
  */
  cpu_id = zzcpu_to_pvr(ti_conf->cpu);
  if(cpu_id != 0)
    {
      for(cpu_probe = cpu_probe_list; *cpu_probe; cpu_probe++)
	{
	  if((*cpu_probe)->probe(cpu_id))
	    {
	      goto probe_ready;
	    }
	} /* for */
    }

  /*
  ** If no match, Probe CPU HAL with configured cpu assumed given as pvr.
  */
  cpu_id = ti_conf->cpu;
  if(cpu_id != 0)
    {
      for(cpu_probe = cpu_probe_list; *cpu_probe; cpu_probe++)
	{
	  if((*cpu_probe)->probe(cpu_id))
	    {
	      goto probe_ready;
	    }
	} /* for */
    }

 probe_ready:

  /* Setup use of this cpu hal. (A NULL pointer is error reported below.) */
  cpu_hal = *cpu_probe;

  /* Setup flib to handle error and error2 during kernel start. */
  zzkrnflib_init(zzearly_syscallh);

  /* Get rest of configuration data. */
  zzose_con_1(ti_conf);

  /* Pick up the system error handler. */
  error_handler = (OSERRH *) ti_conf->err_handler;

  /*
  ** Check that ALL cpu hals have the proper interface version.
  ** It looks odd to do this test here, after the probing is done,
  ** but there is a reason for this: On some cpus (i.e. ARM11) the probe
  ** must be called BEFORE we can parse the krncon configuration and
  ** other system parameters. And the system parameters must be available
  ** for the normal error handling, that relies on the ramlog.
  */

  for(cpu_probe = cpu_probe_list; *cpu_probe; cpu_probe++)
    {
      if(CPU_HAL_VERSION != (*cpu_probe)->version)
	{
	  fatal_error(OSE_EINCOMPATIBLE_CPU_HAL,	/* Fatal */
		      (OSERRCODE) *cpu_probe);
	}
    } /* for */

  /* Error report if no cpu hal did match. */
  if(NULL == cpu_hal)
    {
      fatal_error(OSE_EUNRECOGNIZED_CPU, (OSERRCODE) ti_conf->cpu); /* Fatal */
    }

  /* Init target interface. */
  vector_size = zzinit_ti(cpu_hal->arch_cpu_descriptor->exec_attributes);

  /* Set base address for the vector area. */
  if (ti_conf->vector_base_set)
  {
      vector_base = ti_conf->vector_base;
      /* Set configurated vector *execution* base in cpu control reg. */
      if(zzset_vector_base(vector_base) != vector_base)
	{			/* Fatal */
	  fatal_error(OSE_EILLEGAL_CONFIGURATION, (OSERRCODE) vector_base);
	}
  }
  else
  {
     /* Get current vector base. */
     vector_base = zzget_vector_base();
  }
  
#ifdef OSARM
  /*  Handle case when "vector *write* base" != "vector *execution* base"
   *  vector_write_base should be used when installing vectors. (CPDX-92) */
  if (ti_conf->vector_write_base_set)
  {
     vector_base = ti_conf->vector_write_base;
     ti_conf->vector_base = vector_base ;
  }
#endif
  
  /* Make vector area writeble. */
  mm_handlers = mm_open_exception_area(cpu_hal,
				       (char *) vector_base,
                                       (size_t) vector_size);

  /* Install proper select segment handler. */
  ti_conf->select_segment_handler =
    (OSADDRESS) mm_handlers->select_segment_hook;

  /* Make pose call the select segment handler directly. */
  zzkrn_set_handler(4, (void *)mm_handlers->select_segment_hook);

  /* Install exception handlers. */
  zzinstall_vectors(vector_base);

  /* Init the bios module. */
  zzinitBios();

  /* Update icache, exceptions can occur in mm_install_exception_handlers. */
  cache_sync(vector_base, vector_size);

  /* MM installs it's exception handlers.
   * icache is synchronized internally. */
  mm_install_exception_handlers();

  /* Install the kernel in bios. */
  if (biosInstall("ose", zzsystem_callh,
		  BIOS_USERMODE_ALLOWED | BIOS_RESERVED_BY_KERNEL) !=
      BIOS_SUCCESS)
  {
     extern void zzkrn_halted_hook(void);
     zzkrn_halted_hook();
  }

  /* Install Cache bios */
  cache_init_bios_interface();
  

  /* Call start_handler1 handlers. */
  {
    Start_handler1 **handler_list =
      (Start_handler1 **) ti_conf->start_handler1_list;

    if (handler_list)
      {
	while (*handler_list)
	  {
	    (*handler_list)();	/* Call handler. */
	    handler_list++;
	  }
      }
  }

#ifndef softOSE
  /* Call devman necessary initialization function. */
  {
     extern WEAK_REF(void bspDdaStartHook(void));
     if (bspDdaStartHook != NULL)
     {
        bspDdaStartHook();
     }
  }
#endif

  /* Call the warm start handler if present. */
  if (ti_conf->warmstart_handler)
    {
      Warm_start_handler *warm_start_handler =
	(Warm_start_handler *) ti_conf->warmstart_handler;

      /* Call handler. */
      pose_start_mode = (*warm_start_handler)();

      /* Allow warm or cold start only. */
      pose_start_mode = pose_start_mode ? 1 : 0;
    }
  else
    {
      /* Warm start if handler not present. */
      pose_start_mode = 1;
    }

  /* install the ramlog BIOS module. */
  ramlog_bios_install();

  /* install the sysparam BIOS module. */
  zzsysparam_bios_install();

  /* Check start-stack endmark. */
  if (END_MARK != *start_stack_end_mark)
    {
      fatal_error(OSE_ESTART_STACK_OVERFLOW, /* Fatal */
		  (OSERRCODE) start_stack_end_mark);
    }

  /* Enter pose. */
  zzstart_kernel(pose_start_mode);
}

#if defined(OSE_SMP) && defined(OSPP)
#include "cpu_access.h"
#endif
/*
** Kernel entry point.
*/
void zzstart_OSE(void)
{
  extern void *zzcur_p;
  OSADDRESS start_stack_sp;

  /* Disable interrupts. */
#if defined(OSARM)
  /* We should disable FIQ interrupts at least until after START_HANDLER1. */
  LOCK_RESTORE(SUPERV_MODE | IRQ_DISABLE | FIQ_DISABLE);
#elif defined(OSE_SMP) && defined(OSPP)
   set_msr(get_msr() & ~MSR_EE);
#elif defined(OSE_SMP) && defined(OSMIPS)
	LOCK();
	__asm__ volatile("la $28, zzcpu");
#else
  LOCK();
#endif

  /* Mark kernel as not started. */
  zzcur_p = 0;

  /*
  ** Decide location of the start stack end mark and root frame, with proper
  ** alignment.
  */
  start_stack_sp = (OSADDRESS) zzstart_stack;
  start_stack_sp += zzget_start_stack_size();
  start_stack_end_mark = (unsigned char *) zzstart_stack;
  start_stack_sp &= ~(STACK_FRAME_ALIGNMENT - 1ul);

  /* Fill start stack with debug pattern. */
  memset(zzstart_stack, 0xee, start_stack_sp - (OSADDRESS)zzstart_stack);

  /* Continue in next function on the proper start stack. */
  SET_STACK_AND_JUMP(exec_start, start_stack_sp);
  /* NOTREACHED */
}

#ifdef OSE_SMP

#undef END_MARK
#include "ose_smp.h"
#include "pose_internal.h"

/* These global variables must be volatile, otherwise the synchronized
 * start of other cpus may fail.
 * This has been observed on the p4080 target compiled with gcc.
 */
static volatile cpuid_t unbooted_cpus;
static volatile Boolean wait_for_callin;      
/*
** Start all CPUs in the SMP kernel.
*/
void
ose_smp_start_other_cpus(void)
{
   OSADDRESS size = zzget_start_stack_size();
   cpuid_t   cpus = zzkrn_num_cpus();
   cpuid_t   i;

   unbooted_cpus = cpus - 1; 
   
   for (i = 1 ; i < cpus ; i++)
   {
      OSADDRESS sp = (OSADDRESS)zzalloc_kernelmemory(size);
      ose_cache(CACHE_FLUSH_D, CACHE_ALL, sp, size);
      wait_for_callin = True;
                  
      ose_smp_boot_cpu(i, (void *)sp, size);
      
      /* Synchronisation of the start of the remaining cpus.
       * Wait here until ose_smp_cpu_callin() has been reached by the new cpu
       * due to the previous call to ose_smp_boot_cpu() here above */
      do
      {
#ifdef OSPP
	 /* Write timebase to variable to be read by other cpu. */
         extern uint32_t zztb_sync_init(void);
	 zztb_sync_init();
#endif
      } while(wait_for_callin);
   }   
}

void
ose_smp_cpu_callin(void)
{
   cpuid_t cpu;
   
#ifdef OSMIPS
   /* Cpu id passed in gp from boot. zzkrn_cpu_id not valid yet. */
   cpu = get_cpu_id_from_gp();
#elif defined(OSX86)
   cpu = get_cpu_id_from_gp();
#elif defined(OSARM)
   zzkrn_set_cpu_id_arm();
   cpu = zzkrn_cpu_id();
   zzkrn_setup_exception_stacks(&zzcpu[cpu]);
#else
   cpu = zzkrn_cpu_id();
#endif
   INIT_REGS();
   zzkrn_set_per_cpu_data(&zzcpu[cpu]);

   /* Depends on zzkrn_set_per_cpu_data for mips. */
   zzkrn_set_cur_p(zzcpu[cpu].curr_p);

#ifdef OSMIPS
   zzkrn_set_cpu_id(cpu); /* Stored in cpu struct. */
#endif

   /* Call bsp to set up Status registers. */
   ose_smp_init_other_cpu(cpu);

#ifdef OSPP
   /* Call mm to init mmu. */
   if (cpu_hal->arch_cpu_descriptor->ppc_family == ppc_book_e)
   {
      hal_init_other_cpu_booke(0);
   }
   else
   {
#if !defined(POWERPC_BOOK3E)
      hal_init_other_cpu(0);
#endif
   }
   {
      /* Set local timebase from a variable written by cpu0. */
      extern uint32_t zztb_sync_set_local(void);
      zztb_sync_set_local();
   }
#else
   hal_init_other_cpu(0UL);
#endif
      
   /* Synchronisation of the core start. */
   unbooted_cpus--;
   wait_for_callin = False;
   
   /* Wait here for all cpus. When the last started cpu arrives
    * here all cores are released almost at the same time.
    */   
   while(unbooted_cpus > 0)
   {
      ;
   }

   ramlog_printf("exec: cpu %u is entering the scheduler.\n", cpu);
   zzkrn_activate_cpu();
  /* NOTREACHED */
}

#endif /* OSE_SMP */
