/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2003-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 */

#include "assert.h"
#include "common.h"
#include "mm.sig"
#include "hal.h"
#include "malloc.h"
#include "mmtypes.h"
#include "unistd.h"
#include "mm_log.h"
#include "cpu_hal_arch.h"

#define VERBOZE(s) log(CATEGORY_HAL, s)

static mm_bool_t d_cache_enabled;
static mm_bool_t i_cache_enabled;

static log_size_t page_sizes[] =
#ifdef OSMIPS
{2 * HAL_SIZE_4K};
#else
{HAL_SIZE_4K};
#endif

static const struct hal_info hal_info =
{
   FALSE, /* No mmu. */
   FALSE, /* No early enable mmu. */
   FALSE, /* MMU not enabled. */
   FALSE, /* No mas. */
   sizeof(page_sizes)/sizeof(page_sizes[0]),
   page_sizes,
   0,     /* Number of resident page sizes */
   NULL,  /* Resident page sizes. */
   0,     /* Number of tag IDs. */
   NULL   /* Tag IDs. */
};




/*
 * See hal.h.
 */
static hal_status_t
nommu_hal_init_1(const struct cpu_descriptor *cpu_descriptor,
                 log_addr_t vector_base,
                 log_size_t vector_size,
                 mm_bool_t enable_d_cache,
                 mm_bool_t enable_i_cache)
{
#ifdef LIBWHAT
   LIBWHAT
#endif
   VERBOZE(("hal_init_1(cpu_descriptor=%p, vector_base=%#lx, "
            "vector_size=%x, enable_d_cache=%d, enable_i_cache=%d)\n",
            cpu_descriptor, vector_base, vector_size,
            enable_d_cache, enable_i_cache));
   (void)vector_base;
   (void)vector_size;
   d_cache_enabled = enable_d_cache;
   i_cache_enabled = enable_i_cache;
   hal_arch_cpu_descriptor = cpu_descriptor->arch_cpu_descriptor;

   /* If supported by cpu hal, call init_cpu function. */
   if (hal_arch_cpu_descriptor->init_cpu != NULL)
   {
      hal_arch_cpu_descriptor->init_cpu();
   }
   
   return HAL_SUCCESS;
}


/*
 * See hal.h.
 */
static hal_status_t
nommu_hal_init_2(void)
{
   VERBOZE(("hal_init_2()\n"));

   assert(hal_arch_cpu_descriptor != NULL);

   /* If supported by cpu hal, enable caches according to configuration. */
   if (hal_arch_cpu_descriptor->enable_caches != NULL)
   {
      hal_arch_cpu_descriptor->enable_caches((U32)d_cache_enabled,
					     (U32)i_cache_enabled);
   }

   return HAL_SUCCESS;
}


/*
 * See hal.h.
 */
static hal_status_t
nommu_hal_init_3(struct partition *partition)
{
   VERBOZE(("hal_init_3(partition=%p)\n", partition));
   (void)partition;
   return HAL_SUCCESS;
}

/*
 * See hal.h.
 */
static void
nommu_hal_init_4(void)
{
   VERBOZE(("hal_init_4(void)\n"));
}

/*
 * See hal.h.
 */
static void
nommu_hal_init_5(void)
{
   VERBOZE(("hal_init_5(void)\n"));
}


/*
 * See hal.h.
 */
static const struct hal_info *
nommu_hal_get_info(void)
{
   VERBOZE(("hal_get_info()\n"));
   return &hal_info;
}


/*
 * See hal.h
 */
static hal_status_t
nommu_hal_actual_permissions(U32 *actual_permissions,
                             U32 requested_permissions,
                             log_addr_t base, U32 options)
{
   (void)base;
   (void)options;
   
   VERBOZE(("hal_actual_permissions(gmm_perm=%x)\n", requested_permissions));
   *actual_permissions = (MM_SU_EXECUTE | MM_SU_WRITE | MM_SU_READ |
                          MM_USR_EXECUTE | MM_USR_WRITE | MM_USR_READ);
   return HAL_SUCCESS;
}


/*
 * See hal.h.
 */
static hal_status_t
nommu_hal_create_domain(unsigned int seg_id,
                        mm_bool_t system_domain,
                        struct hal_domain **domain)
{
   VERBOZE(("hal_create_domain(seg_id=%x, system_domain=%d, hal_domain=%p)\n",
            seg_id, system_domain, domain));
   (void)seg_id;
   (void)system_domain;
   (void)domain;
   return HAL_SUCCESS;
}


/*
 * See hal.h.
 */
static void
nommu_hal_delete_domain(unsigned int seg_id,
                        struct hal_domain *domain)
{
   VERBOZE(("hal_delete_domain(seg_id=%x, domain=%p)\n", seg_id, domain));
   (void)seg_id;
   (void)domain;
}


/*
 * See hal.h.
 */
static void
nommu_hal_select_domain(struct hal_domain *hal_domain)
{
   VERBOZE(("hal_select_domain(hal_domain=%p)\n", hal_domain));
   (void) hal_domain;
}


/*
 * See hal.h.
 */
static hal_status_t
nommu_hal_map_page(struct hal_domain *domain,
                   log_addr_t la,
                   phys_addr_t pa,
                   log_size_t size,
                   int options,
                   U32 access_attr,
                   U32 mem_attr)
{
   VERBOZE(("hal_map_page(domain=%p, la=%#lx, pa=%#lx, size=%x, "
            "options=%x, access_attr=%x, mem_attr=%x)\n",
            domain, la, (U32)pa, size, options, access_attr, mem_attr));
   (void) domain;
   (void) la;
   (void)pa;
   (void)size;
   (void)options;
   (void)access_attr;
   (void)mem_attr;
   return HAL_SUCCESS;
}


/*
 * See hal.h.
 */
static void
nommu_hal_unmap_range(struct hal_domain *domain,
                      log_addr_t         la,
                      log_size_t         size,
                      int                options)
{
   VERBOZE(("hal_unmap_range(domain=%p, la=%#lx, pa=%#lx,size=%x,"
	    "options=%x)\n",
            domain, la, size, options));
   (void) domain;
   (void) la;
   (void) size;
   (void) options;
}

static hal_status_t
nommu_hal_lookup(OSDOMAIN          domain,
                 log_addr_t         la,
                 phys_addr_t       *pa,
                 log_size_t        *size,
                 U32               *perm,
                 U32                options)
{
   (void) options;

   VERBOZE(("hal_lookup(domain=%d, la=%#lx)\n", domain, la));

   if (pa != NULL)
   {
      *pa = la;
   }
   if (size != NULL)
   {
      *size = page_sizes[0];
   }
   if (perm != NULL)
   {
      *perm = SU_RWX_USR_RWX;
   }
   
   return HAL_SUCCESS;
}

#ifdef OSPP
static void nommu_hal_invalidate_tlb_range(log_addr_t base,
                                           log_size_t size,
                                           struct hal_domain *domain,
                                           mm_bool_t domain_invariant)
{
   VERBOZE(("nommu_hal_invalidate_tlb_range(base=%x, size=%x,"
	    " domain=%d, domain_invariant=%d)\n",
	    base, size, domain, domain_invariant));
   (void) base;
   (void) size;
   (void) domain;
   (void) domain_invariant;
}
#endif

static hal_status_t
nommu_hal_commit_range(struct hal_domain *domain,
                       log_addr_t base, log_size_t size, U32 options)
{
   (void) domain;
   (void) base;
   (void) size;
   (void) options;
   return HAL_SUCCESS;
}

static hal_status_t
nommu_hal_uncommit_range(struct hal_domain *domain,
                         log_addr_t base, log_size_t size, U32 options)
{
   (void) domain;
   (void) base;
   (void) size;
   (void) options;
   return HAL_SUCCESS;
}

static hal_status_t
nommu_hal_valid_cmbits(U32 *cbits, U32 *mbits, U32 *cmbits, log_addr_t base)
{
   (void)cbits;
   (void)mbits;
   (void)cmbits;
   (void)base;
   
   return HAL_SUCCESS;
}

static hal_status_t
nommu_hal_legacy_cmbits(U32 cmbits, U32 *cache_mode)
{
   (void)cmbits;
   
   *cache_mode = MM_CACHE_COPY_BACK|MM_SPECULATIVE_ACCESS;
   return HAL_SUCCESS;
}

void OSE_MM_HAL_UNIQUELY_DEFINED(void)
{
}



void install_nommu(void)
{
   hal_init_1 = nommu_hal_init_1; 
   hal_init_2 = nommu_hal_init_2; 
   hal_init_3 = nommu_hal_init_3; 
   hal_init_4 = nommu_hal_init_4; 
   hal_init_5 = nommu_hal_init_5; 
   hal_get_info = nommu_hal_get_info; 
   hal_actual_permissions = nommu_hal_actual_permissions; 
   hal_create_domain = nommu_hal_create_domain; 
   hal_select_domain = nommu_hal_select_domain; 
   hal_delete_domain = nommu_hal_delete_domain; 
   hal_map_page = nommu_hal_map_page; 
   hal_unmap_range = nommu_hal_unmap_range;
   hal_lookup = nommu_hal_lookup;
#ifdef OSPP
   hal_invalidate_tlb_range = nommu_hal_invalidate_tlb_range;
#endif
   hal_commit_range = nommu_hal_commit_range;
   hal_uncommit_range = nommu_hal_uncommit_range;
   hal_valid_cmbits  = nommu_hal_valid_cmbits;
   hal_legacy_cmbits = nommu_hal_legacy_cmbits;   
}
   
