/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#include "monitor_execution.sig"
#include "cpu.h"

#define NUM_REGISTERS 136

typedef struct MipsMonitorGetRegisterInfoRequest 
{
   uint32_t sigNo;
   uint32_t reserved0;
   MonitorScope scope;
   uint32_t reserved1;
   uint32_t registersCount;
   MonitorRegisterId registers[NUM_REGISTERS]; 
} MipsMonitorGetRegisterInfoRequest;

typedef struct FmmSignal
{
   uint32_t size;
   MipsMonitorGetRegisterInfoRequest get_register_info_request;
} FmmSignal;

uint32_t fmd_regdump_registers(struct cpu_regdump *regdump, 
                               MonitorRegisterId *ids, MonitorRegister *regs);

FmmSignal fmd_get_register_info_request = 
{                                           
   sizeof(MipsMonitorGetRegisterInfoRequest),   
   {                                 
      MONITOR_GET_REGISTER_INFO_REQUEST, 
      0,
      {
         0,
         0
      },
      0,                             
      NUM_REGISTERS,                 
      {
         MONITOR_MIPS_REGISTER_R0,
         MONITOR_MIPS_REGISTER_R0_HI,
         MONITOR_MIPS_REGISTER_R0+1,
         MONITOR_MIPS_REGISTER_R0_HI+1,
         MONITOR_MIPS_REGISTER_R0+2,
         MONITOR_MIPS_REGISTER_R0_HI+2,
         MONITOR_MIPS_REGISTER_R0+3,
         MONITOR_MIPS_REGISTER_R0_HI+3,
         MONITOR_MIPS_REGISTER_R0+4,
         MONITOR_MIPS_REGISTER_R0_HI+4,
         MONITOR_MIPS_REGISTER_R0+5,
         MONITOR_MIPS_REGISTER_R0_HI+5,
         MONITOR_MIPS_REGISTER_R0+6,
         MONITOR_MIPS_REGISTER_R0_HI+6,
         MONITOR_MIPS_REGISTER_R0+7,
         MONITOR_MIPS_REGISTER_R0_HI+7,
         MONITOR_MIPS_REGISTER_R0+8,
         MONITOR_MIPS_REGISTER_R0_HI+8,
         MONITOR_MIPS_REGISTER_R0+9,
         MONITOR_MIPS_REGISTER_R0_HI+9,
         MONITOR_MIPS_REGISTER_R0+10,
         MONITOR_MIPS_REGISTER_R0_HI+10,
         MONITOR_MIPS_REGISTER_R0+11,
         MONITOR_MIPS_REGISTER_R0_HI+11,
         MONITOR_MIPS_REGISTER_R0+12,
         MONITOR_MIPS_REGISTER_R0_HI+12,
         MONITOR_MIPS_REGISTER_R0+13,
         MONITOR_MIPS_REGISTER_R0_HI+13,
         MONITOR_MIPS_REGISTER_R0+14,
         MONITOR_MIPS_REGISTER_R0_HI+14,
         MONITOR_MIPS_REGISTER_R0+15,
         MONITOR_MIPS_REGISTER_R0_HI+15,
         MONITOR_MIPS_REGISTER_R0+16,
         MONITOR_MIPS_REGISTER_R0_HI+16,
         MONITOR_MIPS_REGISTER_R0+17,
         MONITOR_MIPS_REGISTER_R0_HI+17,
         MONITOR_MIPS_REGISTER_R0+18,
         MONITOR_MIPS_REGISTER_R0_HI+18,
         MONITOR_MIPS_REGISTER_R0+19,
         MONITOR_MIPS_REGISTER_R0_HI+19,
         MONITOR_MIPS_REGISTER_R0+20,
         MONITOR_MIPS_REGISTER_R0_HI+20,
         MONITOR_MIPS_REGISTER_R0+21,
         MONITOR_MIPS_REGISTER_R0_HI+21,
         MONITOR_MIPS_REGISTER_R0+22,
         MONITOR_MIPS_REGISTER_R0_HI+22,
         MONITOR_MIPS_REGISTER_R0+23,
         MONITOR_MIPS_REGISTER_R0_HI+23,
         MONITOR_MIPS_REGISTER_R0+24,
         MONITOR_MIPS_REGISTER_R0_HI+24,
         MONITOR_MIPS_REGISTER_R0+25,
         MONITOR_MIPS_REGISTER_R0_HI+25,
         MONITOR_MIPS_REGISTER_R0+26,
         MONITOR_MIPS_REGISTER_R0_HI+26,
         MONITOR_MIPS_REGISTER_R0+27,
         MONITOR_MIPS_REGISTER_R0_HI+27,
         MONITOR_MIPS_REGISTER_R0+28,         
         MONITOR_MIPS_REGISTER_R0_HI+28,
         MONITOR_MIPS_REGISTER_SP,
         MONITOR_MIPS_REGISTER_SP_HI,
         MONITOR_MIPS_REGISTER_FP,   
         MONITOR_MIPS_REGISTER_FP_HI,   
         MONITOR_MIPS_REGISTER_RA,
         MONITOR_MIPS_REGISTER_RA_HI,
         MONITOR_MIPS_REGISTER_LO,
         MONITOR_MIPS_REGISTER_LO_HI,
         MONITOR_MIPS_REGISTER_HI,
         MONITOR_MIPS_REGISTER_HI_HI,
         MONITOR_MIPS_REGISTER_PC,
         MONITOR_MIPS_REGISTER_PC_HI,
         MONITOR_MIPS_REGISTER_SR,
         MONITOR_MIPS_REGISTER_SSLIMIT,   
         /* FIXME: Don't try to store FP regs if we don't have them */   
         MONITOR_MIPS_REGISTER_F0_LO,
         MONITOR_MIPS_REGISTER_F0_LO+1,
         MONITOR_MIPS_REGISTER_F0_LO+2,
         MONITOR_MIPS_REGISTER_F0_LO+3,
         MONITOR_MIPS_REGISTER_F0_LO+4,
         MONITOR_MIPS_REGISTER_F0_LO+5,
         MONITOR_MIPS_REGISTER_F0_LO+6,
         MONITOR_MIPS_REGISTER_F0_LO+7,
         MONITOR_MIPS_REGISTER_F0_LO+8,
         MONITOR_MIPS_REGISTER_F0_LO+9,
         MONITOR_MIPS_REGISTER_F0_LO+10,
         MONITOR_MIPS_REGISTER_F0_LO+11,
         MONITOR_MIPS_REGISTER_F0_LO+12,
         MONITOR_MIPS_REGISTER_F0_LO+13,
         MONITOR_MIPS_REGISTER_F0_LO+14,
         MONITOR_MIPS_REGISTER_F0_LO+15,
         MONITOR_MIPS_REGISTER_F0_LO+16,
         MONITOR_MIPS_REGISTER_F0_LO+17,
         MONITOR_MIPS_REGISTER_F0_LO+18,
         MONITOR_MIPS_REGISTER_F0_LO+19,
         MONITOR_MIPS_REGISTER_F0_LO+20,
         MONITOR_MIPS_REGISTER_F0_LO+21,
         MONITOR_MIPS_REGISTER_F0_LO+22,
         MONITOR_MIPS_REGISTER_F0_LO+23,
         MONITOR_MIPS_REGISTER_F0_LO+24,
         MONITOR_MIPS_REGISTER_F0_LO+25,
         MONITOR_MIPS_REGISTER_F0_LO+26,
         MONITOR_MIPS_REGISTER_F0_LO+27,
         MONITOR_MIPS_REGISTER_F0_LO+28,         
         MONITOR_MIPS_REGISTER_F0_LO+29,
         MONITOR_MIPS_REGISTER_F0_LO+30,
         MONITOR_MIPS_REGISTER_F0_LO+31,
         MONITOR_MIPS_REGISTER_F0_HI,
         MONITOR_MIPS_REGISTER_F0_HI+1,
         MONITOR_MIPS_REGISTER_F0_HI+2,
         MONITOR_MIPS_REGISTER_F0_HI+3,
         MONITOR_MIPS_REGISTER_F0_HI+4,
         MONITOR_MIPS_REGISTER_F0_HI+5,
         MONITOR_MIPS_REGISTER_F0_HI+6,
         MONITOR_MIPS_REGISTER_F0_HI+7,
         MONITOR_MIPS_REGISTER_F0_HI+8,
         MONITOR_MIPS_REGISTER_F0_HI+9,
         MONITOR_MIPS_REGISTER_F0_HI+10,
         MONITOR_MIPS_REGISTER_F0_HI+11,
         MONITOR_MIPS_REGISTER_F0_HI+12,
         MONITOR_MIPS_REGISTER_F0_HI+13,
         MONITOR_MIPS_REGISTER_F0_HI+14,
         MONITOR_MIPS_REGISTER_F0_HI+15,
         MONITOR_MIPS_REGISTER_F0_HI+16,
         MONITOR_MIPS_REGISTER_F0_HI+17,
         MONITOR_MIPS_REGISTER_F0_HI+18,
         MONITOR_MIPS_REGISTER_F0_HI+19,
         MONITOR_MIPS_REGISTER_F0_HI+20,
         MONITOR_MIPS_REGISTER_F0_HI+21,
         MONITOR_MIPS_REGISTER_F0_HI+22,
         MONITOR_MIPS_REGISTER_F0_HI+23,
         MONITOR_MIPS_REGISTER_F0_HI+24,
         MONITOR_MIPS_REGISTER_F0_HI+25,
         MONITOR_MIPS_REGISTER_F0_HI+26,
         MONITOR_MIPS_REGISTER_F0_HI+27,
         MONITOR_MIPS_REGISTER_F0_HI+28,         
         MONITOR_MIPS_REGISTER_F0_HI+29,
         MONITOR_MIPS_REGISTER_F0_HI+30,
         MONITOR_MIPS_REGISTER_F0_HI+31         
      }
   }
};

MonitorRegisterId fmd_stack_pointer_id = MONITOR_MIPS_REGISTER_SP;

MonitorCPUClass fmd_cpu_class = MONITOR_CPU_MIPS;

uint32_t
fmd_regdump_registers(struct cpu_regdump *regdump, 
                      MonitorRegisterId *ids, MonitorRegister *regs)
{
   uint32_t count = 0;
   
   if (regdump->magic == CPU_REGDUMP_MAGIC)
   {
      /* FIXME: This entire function needs to be verified */

      ids[count] = regs[count].id = MONITOR_MIPS_REGISTER_PC;
      ose_epm_get_reg(regdump, MONITOR_MIPS_REGISTER_PC, 
                      (U32 *) &(regs[count++].value));
      ids[count] = regs[count].id = MONITOR_MIPS_REGISTER_PC_HI;
      ose_epm_get_reg(regdump, MONITOR_MIPS_REGISTER_PC_HI, 
                      (U32 *) &(regs[count++].value));
      ids[count] = regs[count].id = MONITOR_MIPS_REGISTER_SP;
      ose_epm_get_reg(regdump, MONITOR_MIPS_REGISTER_SP, 
                      (U32 *) &(regs[count++].value));
      ids[count] = regs[count].id = MONITOR_MIPS_REGISTER_SP_HI;
      ose_epm_get_reg(regdump, MONITOR_MIPS_REGISTER_SP_HI, 
                      (U32 *) &(regs[count++].value));
      ids[count] = regs[count].id = MONITOR_MIPS_REGISTER_SR;
      ose_epm_get_reg(regdump, MONITOR_MIPS_REGISTER_SR, 
                      (U32 *) &(regs[count++].value));
      
      /* FIXME: What do we do about these registers? */
      /* dump->fault_data_addr ??? */

      if (regdump->no_sects >= 1)
      {
         struct cpu_dumpsection *sect0 = 
                                    (struct cpu_dumpsection *)&regdump->sect[0];

         if (sect0->identity == 0)
         {
            uint32_t i;
            struct cpu_core *core_regs = (struct cpu_core *)sect0->address;
            uint32_t reg_count = sizeof core_regs->r / sizeof core_regs->r[0];
            
            for (i = 0; i < reg_count; i++)
            {
               ids[count] = regs[count].id = MONITOR_MIPS_REGISTER_R0 + i;
               ose_epm_get_reg(regdump, MONITOR_MIPS_REGISTER_R0 + i,
                               (U32 *) &(regs[count++].value));
               ids[count] = regs[count].id = MONITOR_MIPS_REGISTER_R0_HI + i;
               ose_epm_get_reg(regdump, MONITOR_MIPS_REGISTER_R0_HI + i, 
                               (U32 *) &(regs[count++].value));
            }
            ids[count] = regs[count].id = MONITOR_MIPS_REGISTER_SR;
            ose_epm_get_reg(regdump, MONITOR_MIPS_REGISTER_SR, 
                            (U32 *) &(regs[count++].value));
            ids[count] = regs[count].id = MONITOR_MIPS_REGISTER_LO;
            ose_epm_get_reg(regdump, MONITOR_MIPS_REGISTER_LO, 
                            (U32 *) &(regs[count++].value));
            ids[count] = regs[count].id = MONITOR_MIPS_REGISTER_LO_HI;
            ose_epm_get_reg(regdump, MONITOR_MIPS_REGISTER_LO_HI, 
                            (U32 *) &(regs[count++].value));
            ids[count] = regs[count].id = MONITOR_MIPS_REGISTER_HI;
            ose_epm_get_reg(regdump, MONITOR_MIPS_REGISTER_HI, 
                            (U32 *) &(regs[count++].value));
            ids[count] = regs[count].id = MONITOR_MIPS_REGISTER_HI_HI;
            ose_epm_get_reg(regdump, MONITOR_MIPS_REGISTER_HI_HI, 
                            (U32 *) &(regs[count++].value));
            /* FIXME: What do we do about these registers? */
            /* core_regs->epc ??? */
            /* core_regs->cause ??? */
         }
      }
   }   
   
   return count;
}

