/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 1999-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 ****************************************************************************
 *			      HEADER
 *
 *     $Workfile:   exception.c  $
 *     Document no: @(#) 511/OSE120-4 $FileRevision:   /main/tb_current_ose5/10  $
 *     $Author:   anha  $
 *     $Date:   05/02/06 10:56:42  $
 *
 *
 ****************************************************************************

	CONTENTS
	--------

	1  Description
	2  History of development
	3  Include files
	   3.1 Systemwide definitions
	   3.2 Libraries
	   3.3 Blockwide definitions
	   3.4 Global signals
	   3.5 Local signals
	4  Local definitions
	   4.1 Manifest constants
	   4.2 Type definitions
	   4.3 Macros
	   4.4 Signal composition
	5  External references
	   5.1 Functions
	   5.2 Variables
	   5.3 Forward references
	   5.4 Processes
	6  Local variables
	   6.1 General variables
	   6.2 Receive specifications
	7  Global variables
	8  Local functions
	9  Global functions
	10 Program entrypoint

*****************************************************************************
*/

/*
*****************************************************************************
** 1  DESCRIPTION.
*****************************************************************************
** This file contains the c part of the exception handling for Mips.
**
*/

/*
*****************************************************************************
** 2  HISTORY OF DEVELOPMENT.
*****************************************************************************
*/

/*
*****************************************************************************
** 3  INCLUDE FILES.
*****************************************************************************
*/

/*
**===========================================================================
** 3.1  Systemwide definitions.
**===========================================================================
*/

#include "stdint.h"
#include "string.h"
#include "assert.h"
#include "osenames.h"
#include "cpu_intlock.h"
#include "cpu_regdump.h"
#include "efmi_registers.h"
#include "ose_err.h"
#include "pose.h"
#include "exec_arch.h"
#include "ti_cpu_data.h"
#include "pose_internal.h"
#include "cpu_smp.h"
#include "exception.h"
#include "uarea.h"
#include "tidata.h"
#include "krncon.h"
#include "cache.h"
#include "register.h"

#include "ramlog.h"

/*
**===========================================================================
** 3.2  Libraries.
**===========================================================================
*/

/*
**===========================================================================
** 3.3  Blockwide definitions.
**===========================================================================
*/

/*
**===========================================================================
** 3.4  Global signals.
**	These signals are visible outside the current block of processes.
**===========================================================================
*/

/*
**===========================================================================
** 3.5  Local signals.
**	These signals are used only within the current block of processes.
**===========================================================================
*/

/*
*****************************************************************************
** 4  LOCAL DEFINITIONS.
*****************************************************************************
*/

/*
**===========================================================================
** 4.1  Manifest constants.
**===========================================================================
*/

/*
**===========================================================================
** 4.2  Type definitions.
**===========================================================================
*/

struct exception_block
{
      exception_handler_c *handler;
      struct exception_block *next;
      U32 attribute;
      U32 index;
};

/*
**===========================================================================
** 4.3  Macros.
**===========================================================================
*/

#define EXC_POOL_SIZE 16

#define GPR_SP 29

/*
**===========================================================================
** 4.4  Signal composition.
**===========================================================================
*/

/*
*****************************************************************************
** 5  EXTERNAL REFERENCES.
*****************************************************************************
*/

/*
**===========================================================================
** 5.1  Functions.
**===========================================================================
*/

/* Implemented in ti_s.S */
OSADDRESS zzset_sr(OSADDRESS and, OSADDRESS or);
OSADDRESS zzset_cause(OSADDRESS and, OSADDRESS or);
OSADDRESS zzget_fcr0(void);

/*
**===========================================================================
** 5.2  Variables.
**===========================================================================
*/

extern volatile struct tidata zzti;	/* volatile for fpu_state. */

/*
**===========================================================================
** 5.3  Forward references.
**===========================================================================
*/

void zzexception_c(struct cpu_core *cpu_core, U32 badvaddr);

U32 zzdefault_exception_handler(U32 exception_id, struct cpu_regdump *regdump);

U32 zzsafecopy_exception_handler(U32 exception_id,
				 struct cpu_regdump *regdump);
U32 zzcoprocessor_unusable_handler(U32 exception_id,
				   struct cpu_regdump *regdump);
void zzinstall_vectors(U32 vector_base);

void zzkrn_halted_hook(void);

/*
**===========================================================================
** 5.4  Processes.
**===========================================================================
*/

/*
*****************************************************************************
** 6  LOCAL VARIABLES.
*****************************************************************************
*/

/*
**===========================================================================
** 6.1  General variables.
**===========================================================================
*/

/* Table for General Exception handlers. */
U32 asm_exception_handlers[NO_ASM_VECTORS];

/* Table for C Exception handlers. */
static struct exception_block *c_exception_handlers[NO_C_VECTORS];

/* Pool of blocks for installable exception handlers. */
static struct exception_block exception_blocks[EXC_POOL_SIZE];

/* Free list of c exception handlers. */
static struct exception_block *free_exception_block_list;

/* Index for breaking new memory from exception handler pool. */
static U32 exc_pool_index;

U32 saved_vector_base;

/*
**===========================================================================
** 6.2 Receive specifications.
**===========================================================================
*/

/*
*****************************************************************************
** 7  GLOBAL VARIABLES.
*****************************************************************************
*/

/*
*****************************************************************************
** 8  LOCAL FUNCTIONS.
*****************************************************************************
*/

/*
**===========================================================================
** alloc_exception_block
**===========================================================================
** Description:	
**
** Parameters:	
**
** Returns:	
**
** Globals:	-
*/

static struct exception_block *alloc_exception_block(void)
{
   U32 sr;
   struct exception_block *ptr; 

   LOCK_SAVE(sr);
   if(free_exception_block_list)
   {
      ptr = free_exception_block_list;
      free_exception_block_list = ptr->next;
      LOCK_RESTORE(sr);
   }
   else if(exc_pool_index < EXC_POOL_SIZE)
   {
      /* Break from static memory. */
      ptr = &exception_blocks[exc_pool_index];
      exc_pool_index++;
      LOCK_RESTORE(sr);
   }
   else
   {
      LOCK_RESTORE(sr);
      if(zzcur_p)	/* Check that kernel is running. */
      {
	 /* Alloc from system pool. */
	 ptr = (struct exception_block *)
	    zzalloc_kernelmemory(sizeof(struct exception_block));
      }
      else
      {
	 ptr = NULL;
	 /* FIXME Call error ? */
	 zzkrn_halted_hook();
      }
   }
   return ptr;
}

/*
**===========================================================================
** safecopy_exception_handler
**===========================================================================
** Description:	This function is invoked for data access errors for systems
**              without memory manager. If the exception occurs during
**              safecopy the handler will change the exception return address
**              to make safecopy return an error code.
** Parameters:	regdump
**
** Returns:	Nonzero, for a safecopy exception.

** Globals:	-
*/

U32 zzsafecopy_exception_handler(U32 exception_id, struct cpu_regdump *regdump)
{
   extern void zzsafecopy_error(void);
   UAREA *uarea = (UAREA *) zzcurrent_uarea;
   struct cpu_core *core = (struct cpu_core *) regdump->sect[0].address;

   (void)exception_id;

   /* Check if safecopy freeze mode is in progress. */
   if (zzti.safecopy_freeze_mode_sp.u64)
   {
      /* Setup pc to return in safecopy. */
      core->epc = (U32)zzsafecopy_error;
      core->r[29] = zzti.safecopy_freeze_mode_sp.u64;
      zzti.safecopy_freeze_mode_sp.u64 = 0;
      return 1;
   }
   /* Check if safecopy run mode is in progress. */
   if (uarea && uarea->safecopy_sp.u64)
   {
      assert(uarea->safecopy_sp.u64 != 0);

      core->epc = uarea->safecopy_err.u64;
      core->r[29] = uarea->safecopy_sp.u64;
      uarea->safecopy_sp.u64 = 0;
      return 1;
   }
   return 0;
}

/*
**===========================================================================
** coprocessor_unusable_handler
**===========================================================================
** Description:	This handler is installed for Coprocessor Unusable exceptions.
**              It is used to handle floating point coprocessor exceptions
**              during probing and when processes executes their first
**              fp-instruction.
** Parameters:	regdump
**
** Returns:	Nonzero for handled exceptions.
**
** Globals:	-
*/

U32
zzcoprocessor_unusable_handler(U32 exception_id, struct cpu_regdump *regdump)
{
   struct cpu_core *core = (struct cpu_core *) regdump->sect[0].address;
   OSADDRESS cause = core->cause;

   (void)exception_id;

   if(zzti.fpu_state == FPU_NOT_PRESENT)
   {
      return 0; /* Exception not handled. */
   }

   if((cause & 0x30000000) == 0x10000000)		/* Check CE bits. */
   {
      /* Handle Coprocessor 1 exceptions only. */
      UAREA *uarea = (UAREA *) zzcurrent_uarea;

      /*
       * Current process needs to enable fpu:
       * Enable fpu in sr.
       * Alloc memory for swap area.
       * Enable swaping of fp registers.
       */

      /* Enable coprocessor 1 in current sr. */
      (void) zzset_sr(0xffffffff, 0x20000000);
      
      if(uarea->fpu_reg_file == 0)
      {
	 /* Only alloc memory once, if we get more exceptions,
	  * enable fpu mode again silent.
	  */
	 uarea->fpu_reg_file = zzalloc_uarea_block();
	 uarea->fpu_reg_file->magic = UAREA_MAGIC;
	 
	 /* Init read only register, no need to swap this register. */
	 uarea->fpu_reg_file->fcr0 = zzget_fcr0();
      }
      /* Enable coprocessor 1 in exception frame. */
      core->sr |= 0x20000000;

      /* Restart instruction with fpu enabled. */

      return 1; /* Exception not handled. */
   }
   return 0; /* Exception not handled. */
}

/*
**===========================================================================
** fpu_present
**===========================================================================
** Description:	Probe for floating point unit.
**
** Parameters:	-
**
** Returns:	-
**
** Globals:	-
*/

static int
fpu_present(void)
{
   U32 conf = get32_cp0(C0_CONFIG);

   ramlog_printf("exec: Floating point unit...");
   if (conf & 0x80000000) /* Check if config1 is present. */
   {
      conf = get32_cp0_sel1(C0_CONFIG);
      if (conf & 1) /* Test for fpu. */
      {
	 zzti.fpu_state = FPU_PRESENT;
	 ramlog_printf("present\n");
	 return 1;
      }
   }
   else
   {
      /* Assume floating point is present. */
      zzti.fpu_state = FPU_PRESENT;
      ramlog_printf("assumed present\n");
      return 1;
   }
   zzti.fpu_state = FPU_NOT_PRESENT;
   ramlog_printf("not present\n");
   return 0;
}

/*
*****************************************************************************
** 9  GLOBAL FUNCTIONS.
*****************************************************************************
*/

/*
**===========================================================================
** zzexception_c
**===========================================================================
** Description:	This function is called with interrupts disabled.
**		Call chained handlers for a vector. If a handler is able to
**		handle the exception it can enable interrupts if it is
**		desirable.
** Parameters:	Register dump
**		Access fault address.
** Returns:	-
**
** Globals:	
*/

void zzexception_c(struct cpu_core *core, U32 badvaddr)
{
   struct cpu_regdump regdump; /* Allocate on stack. */
   struct exception_block *block;
   /* Pick up ti per cpu data. */
   struct ticpu *ticpu = &zzcpu[zzkrn_cpu_id()].ticpu;
   struct uarea *current_uarea;

   /* Compensate pc for exceptions in a delay slot. */
   regdump.pc = core->cause & 0x80000000 ? core->epc + 4 : core->epc;
   regdump.sp = core->r[GPR_SP];
   regdump.sr = core->sr;
   if(ticpu->dedicated_vector)
   {
      /* Exception generated from a dedicated vector. */
      regdump.vector = (U32) ticpu->dedicated_vector;
      ticpu->dedicated_vector = 0;
   }
   else
   {
      /* Exception generated from the general exception vector. */
      regdump.vector = (core->cause >> 2) & 0x1f;
   }

   switch(regdump.vector)
   {
      case EXCEPTION_ID_MOD:
      case EXCEPTION_ID_TLBL:
      case EXCEPTION_ID_TLBS:
      case EXCEPTION_ID_ADEL:
      case EXCEPTION_ID_ADES:
      case EXCEPTION_VEC_TLB_REFILL:
      case EXCEPTION_VEC_XTLB:
	 regdump.fault_data_set = 1;
	 regdump.fault_data_addr = badvaddr;
	 break;
      default:
	 regdump.fault_data_set = 0;
	 break;
   }
   regdump.magic = CPU_REGDUMP_MAGIC;
   regdump.no_sects = 1;
   regdump.sect[0].identity = 0;
   regdump.sect[0].size = sizeof(struct cpu_core);
   regdump.sect[0].address = (U32) core;

   if(ticpu->recursive_exception_flag)
   {
      ticpu->recursive_exception_flag = 0;

      /* Report fatal error, does not return. */
      zzos_error(FATAL_ERROR,
		 regdump.vector << 16 | OSE_EUNEXPECTED_EXCEPTION_REGDUMP,
		 (OSERRCODE) &regdump);
      /* NORETURN */
   }

   /* Remember the regdump, for access from ti_c. */
   current_uarea = (struct uarea *)zzcurrent_uarea;
   if(current_uarea && !current_uarea->exception_regdump)
     {
       current_uarea->exception_regdump = (U32 *)&regdump;
     }

   /* Get list of handlers for current vector. */
   block = c_exception_handlers[regdump.vector];

   /* Loop until exception is handled. */
   while(block)
   {
      U32 exception_handled;

      /* Call exception handler. */
      exception_handled = (block->handler)(regdump.vector, &regdump);
      if(exception_handled)
      {
	if(current_uarea &&
	   current_uarea->exception_regdump == (U32 *)&regdump)
	  {
	    current_uarea->exception_regdump = NULL;
	  }
	 return;
      }
      block = block->next; /* Get next handler. */
   }

   /* Exception not handled, report error. */
   zzos_error((regdump.sr & SR_UM) ? WARNING : FATAL_ERROR,
	      regdump.vector << 16 | OSE_EUNEXPECTED_EXCEPTION_REGDUMP,
	      (OSERRCODE) &regdump);
   if(current_uarea &&
      current_uarea->exception_regdump == (U32 *)&regdump)
     {
       current_uarea->exception_regdump = NULL;
     }
}

/*
**===========================================================================
** zzinstall_vectors
**===========================================================================
** Description:	Init data structures for exception handlers.
**		Install a c-handler last in chain for every vector as a
**		fallback if no other handler is able to handle the exception.
**		Install vector code.
** Parameters:	-
**
** Returns:	-
**
** Globals:	
*/

void zzinstall_vectors(U32 vector_base)
{
   extern void zzexception(void); /* Default asm handler. */
   extern void zzexternal_interrupt(void); /* Interrupt handling. */
   extern void zzgeneral_exception_handler(void); /* Vector code for gen exc */
   extern void zzgeneral_exception_handler_end(void);
   extern void zzinterrupt_handler(void); /* Vector code for interrupt. */
   extern void zzinterrupt_handler_end(void);
   extern void zztlb_refill_handler(void); /* Vector code for tlb exception. */
   extern void zztlb_refill_handler_end(void);
   extern void zzxtlb_handler(void); /* Vector code for xtlb exception. */
   extern void zzxtlb_handler_end(void);
   extern void zzcacheerror_handler(void); /* Vector code for tlb exception. */
   extern void zzcacheerror_handler_end(void);
   extern void zzsystem_call(void);
   int i;

   saved_vector_base = vector_base;

   /* Init general exception handlers. */
   for(i = 0; i < NO_ASM_VECTORS; i++)
   {
      asm_exception_handlers[i] = (U32) zzexception;
   }
   asm_exception_handlers[EXCEPTION_ID_INT] = (U32) zzexternal_interrupt;
   asm_exception_handlers[EXCEPTION_ID_SYS] = (U32) zzsystem_call;

#ifdef SET_BP_SUPPORT
   /* set_bp REMOVED !!! */
   /* Handle traps as interrupts for breakpoints. */
   asm_exception_handlers[EXCEPTION_ID_TR]  = (U32) zzexternal_interrupt;
#endif

   /* Copy vector code for the tlb vector. */
   memcpy((void *) (vector_base + 0x000), zztlb_refill_handler,
	  (U32)zztlb_refill_handler_end - (U32)zztlb_refill_handler);

   /* Copy vector code for the xtlb vector. */
   memcpy((void *) (vector_base + 0x080), zzxtlb_handler,
	  (U32)zzxtlb_handler_end - (U32)zzxtlb_handler);

   /* Copy vector code for the cache error vector. */
   memcpy((void *) (vector_base + 0x100), zzcacheerror_handler,
	  (U32)zzcacheerror_handler_end - (U32)zzcacheerror_handler);

   /* Copy vector code for general exception handling. */
   memcpy((void *) (vector_base + 0x180), zzgeneral_exception_handler,
	  (U32)zzgeneral_exception_handler_end -
	  (U32)zzgeneral_exception_handler);

   /* Copy vector code for interrupt handling. */
   memcpy((void *) (vector_base + 0x200), zzinterrupt_handler,
	  (U32)zzinterrupt_handler_end - (U32)zzinterrupt_handler);

   /* Flush cache. */
   cache_sync(vector_base, VECTOR_SIZE);

   /* Clear BEV bit in sr to set vector base in kseg0 (0x80000000). */
   (void) zzset_sr(0xffbfffff, 0);

   /* Set IV bit in cause to enable dedicated interrupt vector. */
   (void) zzset_cause(0xffffffff, 0x00800000);

   /* Install c-handlers. */
   ose_install_c_exception_handler(EXCEPTION_ID_TLBL,
				   zzsafecopy_exception_handler,
				   0);
   ose_install_c_exception_handler(EXCEPTION_ID_ADEL,
				   zzsafecopy_exception_handler,
				   0);
   ose_install_c_exception_handler(EXCEPTION_ID_ADES,
				   zzsafecopy_exception_handler,
				   0);
   ose_install_c_exception_handler(EXCEPTION_ID_DBE,
				   zzsafecopy_exception_handler,
				   0);
   if (fpu_present())
   {
      /* cp1 unusable exception (floating point). */
      ose_install_c_exception_handler(EXCEPTION_ID_CPU,
				      zzcoprocessor_unusable_handler,
				      0);
   }

   ose_install_c_exception_handler(EXCEPTION_VEC_TLB_REFILL,
				   zzsafecopy_exception_handler,
				   0);
#if 0
   /* FIXME: upage handling postponed for now... */
   if(!mm_page_alloc(0x2000, &zzti.upage)) /* FIXME page size */
   {
      U32 extra[4];

      extra[0] = 0;
      extra[1] = 0;
      extra[2] = (U32)__FILE__;
      extra[3] = __LINE__;
      zzcall_system_errh(0, OSE_EEARLY_ERROR, extra); /* FIXME */
   }
#endif
}

/*
**===========================================================================
** ose_install_asm_exception_handler
**===========================================================================
** Description:	Install an asm exception handler.
**		This handler must save all used registers and return
**		from exception when finnished.
** Parameters:	Vector id.
**		Pointer to exception handler.
** Returns:	Pointer to old handler.
**
** Globals:	
*/

exception_handler_asm *
ose_install_asm_exception_handler(U32 exception_id,
				  exception_handler_asm *handler,
				  size_t size)
{
   U32 old_handler;
   U32 sr;

   if (exception_id == EXCEPTION_VEC_TLB_REFILL ||
       exception_id == EXCEPTION_VEC_XTLB)
   {
      U32 vector_addr = exception_id == EXCEPTION_VEC_XTLB ?
	 saved_vector_base + 0x80 : saved_vector_base;

      ramlog_printf("exec: install asm: %x %x %x\n",
		    vector_addr, handler, size);
      memcpy((void *)vector_addr, handler, size);

      /* Flush cache. */
      cache_sync(vector_addr, size);
      return NULL;
   }

   /* Install pointer in general exception vector table. */
   LOCK_SAVE(sr);
   old_handler = asm_exception_handlers[exception_id];

   asm_exception_handlers[exception_id] = (U32) handler;
   LOCK_RESTORE(sr);
   return (exception_handler_asm *) old_handler;
}

/*
**===========================================================================
** ose_install_c_handler
**===========================================================================
** Description:	Install a c exception handler first in the chain for a desired
**              exception id.
** Parameters:	Vector id.
**		Pointer to exception handler.
** Returns:	Handler id.
**
** Globals:	
*/

exception_handler_id
ose_install_c_exception_handler(U32 exception_id,
				exception_handler_c *handler,
				U32 attribute)
{
   U32 sr;
   struct exception_block *block;

   if (exception_id >= NO_C_VECTORS)
   {
      return 0;
   }

   block = alloc_exception_block();
   assert(block != NULL);
   block->handler = handler;
   block->attribute = attribute;

   /* Store index in array to be able to uninstall. */
   block->index = exception_id;

   /* Put new handler first in list. */
   LOCK_SAVE(sr);
   block->next = c_exception_handlers[exception_id];
   c_exception_handlers[exception_id] = block;
   LOCK_RESTORE(sr);
   return (exception_handler_id)block;
}

/*
**===========================================================================
** ose_uninstall_c_handler
**===========================================================================
** Description:	Uninstall a c exception handler.
**
** Parameters:	Handler id.
**
** Returns:	-
**
** Globals:	
*/

void
ose_uninstall_c_exception_handler(exception_handler_id identity)
{
   struct exception_block  *block = (struct exception_block *)identity;
   struct exception_block **block_list;
   U32 sr;

   /* Looks like a handler identity? */
   if (identity == 0 || (identity & 3) != 0)
   {
      /* Error */
      return;
   }

   LOCK_SAVE(sr);

   for(block_list = &c_exception_handlers[block->index]; *block_list;
       block_list = &(*block_list)->next)
   {
      if((*block_list) == block)
      {
	 /* Unlink from handler list. */
	 (*block_list) = block->next;

	 /* Link into free list. */
	 block->next = free_exception_block_list;
	 free_exception_block_list = block;
	 break;
      }
   } /* for */
   
   LOCK_RESTORE(sr);
}


/* 
** Convert register id to U32 offset in struct cpu_core.
** id is given as specified in public interface efmi_registers.h
** Returns ~0 for unknown registers.
*/
#ifdef BIG_ENDIAN
#define HIGH 0
#define LOW 1
#else
#define HIGH 1
#define LOW 0
#endif

U32
zzget_regdump_core_offset(U32 id)
{
  U32 offset;
  
   if (id <= ZZREG_RA)
   {
      /* r0-r31 low word (including SP, FP, RA) */
      return offset = (id * 2) + LOW;
   }
   if (id >= ZZREG_R0_HI && id <= ZZREG_RA_HI)
   {
      /* r0-r31 high word (including SP, FP, RA) */
      return offset = ((id - ZZREG_R0_HI) * 2) + HIGH;
   }
   switch (id)
   {
      case ZZREG_HI:
	 offset = offsetof(struct cpu_core, hi) + (LOW * sizeof(U32));
	 break;
      case ZZREG_HI_HI:
	 offset = offsetof(struct cpu_core, hi) + (HIGH * sizeof(U32));
	 break;
      case ZZREG_LO:
	 offset = offsetof(struct cpu_core, lo) + (LOW * sizeof(U32));
	 break;
      case ZZREG_LO_HI:
	 offset = offsetof(struct cpu_core, lo) + (HIGH * sizeof(U32));
	 break;
      case ZZREG_PC:
	 offset = offsetof(struct cpu_core, epc) + (LOW * sizeof(U32));
	 break;
      case ZZREG_PC_HI:
	 offset = offsetof(struct cpu_core, epc) + (HIGH * sizeof(U32));
	 break;
      case ZZREG_SR:
	 offset = offsetof(struct cpu_core, sr);
	 break;
      default:
	return (U32)~0;
   }
   offset /= sizeof(U32);
   return offset;
}

