/* COPYRIGHT-ENEA-SRC-R2 *
**************************************************************************
* Copyright (C) 2004-2006 by Enea Software AB.
* All rights reserved.
*
* This Software is furnished under a software license agreement and
* may be used only in accordance with the terms of such agreement.
* Any other use or reproduction is prohibited. No title to and
* ownership of the Software is hereby transferred.
*
* PROPRIETARY NOTICE
* This Software consists of confidential information.
* Trade secret law and copyright law protect this Software.
* The above notice of copyright on this Software does not indicate
* any actual or intended publication of such Software.
**************************************************************************
* COPYRIGHT-END */

/*
** This file contains the implementation of the SMP kernel's giant
** lock.  This lock is managed by functions that are piggybacked onto
** the normal lock macros.
**
** The lock is basically a 32-bit variable that is divided into 2 fields:
**
**      16-bit irq lock (in the least significant halfword)
**      16-bit dispatcher lock (in the most significant halfword)
**
** A cpu can accuire any of the two locks recursively.
**
** A lock is unlocked if the value of the lock field is zero.
** The lsb of a lock is the cpu_id when taken.
**
** In order not to end up in a deadlock situation a cpu may *NOT* 
** accuire the dispatcher lock when it holds the irq lock.
** Interrupts will be impicitely enabled by locking the dispatcher.
*/


#define RE_ENABLE_INTERRUPTS	/* Always! */

#if defined(ASSERT)
#define RECURSIVE              
#define MAX_SPIN_CNT 0x1000000 
#define PROTOCOL_VIOLATION
#endif

/* #define RECURSIVE              */ /* Breaks refsys if enabled. */
/* #define MAX_SPIN_CNT 0x1000000 */


#include "stdint.h"
#include "pose.con"
#include "pose.h"
#include "pose_internal.h"
/* typedef  LOCKSTATE Msr; */


#define MSR_EE	0x00000001
#define SWP_LK	0x08000000

#define likely(x)    __builtin_expect(!!(x), 1)
#define unlikely(x)  __builtin_expect(!!(x), 0)

#define CPU_ID_MASK (OSE_SMP_MAX_CPUS-1)

/* Get the return address of the current function (if possible) */
#ifdef __GNUC__
#define RETURN_ADDRESS() ((OSADDRESS) __builtin_return_address(0))
#elif defined(__ARMCC_VERSION)
#define RETURN_ADDRESS() ((OSADDRESS) __return_address())
#else
#define RETURN_ADDRESS() ((OSADDRESS) 0)
#endif

#ifdef RE_ENABLE_INTERRUPTS
#undef RE_ENABLE_INTERRUPTS
#define RE_ENABLE_INTERRUPTS_IS_ON
#define RE_ENABLE_INTERRUPTS(msr)		\
   do {						\
      if (msr & MSR_EE)				\
      {						\
	 set_msr(msr);				\
	 set_msr(msr & ~MSR_EE);		\
      }						\
   } while(0)
#else
#define RE_ENABLE_INTERRUPTS(msr) ((void)0)
#endif


#ifdef MAX_SPIN_CNT
#define CHECK_MAX_SPIN_CNT()			\
   do {						\
      if (--spin_cnt)				\
	 continue;				\
      SMP_LOCK_ERR();				\
   } while(0)
#else
#define CHECK_MAX_SPIN_CNT() ((void)0)
#endif

#ifdef RECURSIVE
#define SET_LOCK_ADDR(x)\
   do \
      zzkrn_bkl_ ## x.lock_addr = RETURN_ADDRESS();	\
   while(0)
#else
#define SET_LOCK_ADDR(x)
#endif

#ifdef RECURSIVE
#define SET_UNLOCK_ADDR(x)\
   do {			  \
      zzkrn_bkl_ ## x.unlock =  zzkrn_bkl_lock_id();	\
      zzkrn_bkl_ ## x.unlock_addr = RETURN_ADDRESS();	\
   } while(0)
#else
#define SET_UNLOCK_ADDR(x)
#endif

extern void ose_polled_write(const char *buf, unsigned  size);
extern int sprintf(char *buf, const char *, ...);
extern int snprintf(char *buf, size_t, const char *, ...);
extern int ramlog_write(const char *buf, unsigned);
extern int ramlog_printf(const char *, ...);
extern unsigned strlen(const char*);
extern void bsp_reset(void); /* What a nice place to put a breakpoint... */

void add_smp_cmds(void); /* Prototype */

volatile struct ZZSmpBkl zzkrn_bkl_irq  __attribute__ ((aligned (32)));
volatile struct ZZSmpBkl zzkrn_bkl_swp  __attribute__ ((aligned (32)));

static inline void
sync(void)
{
   __asm__ __volatile__
      (
	 "sync"
	 : /*no outputs*/
	 : /*no inputs*/
	 : "memory"
	 );
}

/*
** Cavium specific syncw (stype 4) will drain the lazy write buffer
** without stalling the cpu. Needed to make updates visible for 
** other cpus.
** Should we have an unlock() function in the cpu hal instead? Or
** use the sc instruction for the lock release? But that would only
** affect the lock location itself, not making the protected data
** visible for other cpus!
** On Broadcom cpus, the SYNCW executes as a normal SYNC.
*/
static inline void
syncw(void)
{
   __asm__ __volatile__
      (
       ".long 0x10f" /* SYNC with stype 4, i.e. cavium SYNCW */
	 : /*no outputs*/
	 : /*no inputs*/
	 /* Affects, but does not clobber "memory" as long as we
	 ** avoid alias mappings
	 */
	 );
}

static inline Msr
get_msr(void)
{
   Msr msr;
   __asm__ __volatile__
      (
	 "mfc0 %0,$12"
	 : /*outputs*/ "=r" (msr)
	 : /*no inputs*/
	 );
   return msr;
}

#ifdef _MIPS_ARCH_MIPS3
static inline void
set_msr(Msr newMsr)
{
   __asm__ __volatile__ (
      "mtc0 %0,$12\n"
      "nop\n"
      "nop\n"
      "nop\n"
      : /*no outputs*/
      : /*inputs*/ "r" (newMsr) );
}
#else
static inline void
set_msr(Msr newMsr)
{
   __asm__ __volatile__ (
      "mtc0 %0,$12\n"
      "ssnop\n"
      "ssnop\n"
      "ssnop\n"
      : /*no outputs*/
      : /*inputs*/ "r" (newMsr) );
}
#endif

static inline uint32_t
try_lock(volatile uint32_t *addr)
{
   uint32_t tmp;

   __asm__ __volatile__
      (
	 "   .set push           \n"
	 "   .set noat           \n"
	 "   .set noreorder      \n"
	 "   .set mips2          \n"
	 "1:	ll	%[tmp],(%[addr])\n"
	 "	bnez	%[tmp],2f\n"
	 "      nop\n"
	 "	lw	%[tmp],($28)\n"  /* cpu_id in lower byte. */
	 "	ori	%[tmp],0x8800\n" /* Nonzero in upper bytes. */
	 "	sc	%[tmp],(%[addr])\n"
	 "	beqz	%[tmp],1b\n"     /* Retry if not atomic. */
	 "      nop\n"
	 "	sync\n"
	 "	move	%[tmp],$0\n"     /* Return 0 for success. */
	 "2:\n"
	 "   .set pop            \n"
	 : [tmp] "=&r"(tmp)
	 : [addr] "r"(addr)
	 : "memory"
      );
   return tmp;
}

#if defined(RECURSIVE) || defined(MAX_SPIN_CNT)

static void
smp_lock_err(uint32_t addr, const char *func, const char *file, int line)
{
   extern int error_state;
   extern void gdb_stop_here(void);
   volatile int i;
   static char buf[1000];
   
   error_state = 1;

   i  = sprintf(&buf[0], "\nSMP_LOCK_ERR : 0x%x (%s:%s:%d)",
		addr, func, file, line);
   i += sprintf(&buf[i], "\ncurr cpuid       : %u",  zzkrn_cpu_id());
   i += sprintf(&buf[i], "\nbkl_i.lock       : %#x", zzkrn_bkl_irq.lock);
   i += sprintf(&buf[i], "\nbkl_i.lock_addr  : %#x", zzkrn_bkl_irq.lock_addr);
   i += sprintf(&buf[i], "\nbkl_i.unlock     : %#x", zzkrn_bkl_irq.unlock);
   i += sprintf(&buf[i], "\nbkl_i.unlock_addr: %#x", zzkrn_bkl_irq.unlock_addr);

   i += sprintf(&buf[i], "\nbkl_w.lock       : %#x", zzkrn_bkl_swp.lock);
   i += sprintf(&buf[i], "\nbkl_w.lock_addr  : %#x", zzkrn_bkl_swp.lock_addr);
   i += sprintf(&buf[i], "\nbkl_w.unlock     : %#x", zzkrn_bkl_swp.unlock);
   i += sprintf(&buf[i], "\nbkl_w.unlock_addr: %#x", zzkrn_bkl_swp.unlock_addr);

   ramlog_write(buf, i);
   for(i = 0 ; i < 0xFFFFFF ; i++);
   bsp_reset();
}
#define SMP_LOCK_ERR() smp_lock_err(RETURN_ADDRESS(), __func__, __FILE__,__LINE__)

#endif

/*
**********************************************************************
** zzbkl_lock_save
**********************************************************************
*/

LOCKSTATE
zzbkl_lock_save(void)
{
#ifdef MAX_SPIN_CNT
   uint32_t spin_cnt = MAX_SPIN_CNT;
#endif
   Msr msr;
   volatile uint32_t *const lockp = &zzkrn_bkl_irq.lock;

   msr = get_msr();
   set_msr(msr & ~MSR_EE);

   for (;;)
   {
      uint32_t lock = try_lock(lockp);
      
      if (lock == 0)
      {
	 SET_LOCK_ADDR(irq);
	 return msr;
      }
#ifdef RECURSIVE
      else if (unlikely((lock & CPU_ID_MASK) == zzkrn_cpu_id()))
	 SMP_LOCK_ERR();
#endif
      for(;;)
      {
	 if (*lockp == 0)
	    break;
	 RE_ENABLE_INTERRUPTS(msr);
	 CHECK_MAX_SPIN_CNT();
      }
   }
}

/*
**********************************************************************
** zzbkl_lock_restore
**********************************************************************
*/
void
zzbkl_lock_restore(LOCKSTATE msr)
{
#ifdef RECURSIVE
   if (zzkrn_bkl_lock_id())
   {
      uint32_t lock = zzkrn_bkl_irq.lock;
      
      if (unlikely(!lock || ((lock & CPU_ID_MASK) != zzkrn_cpu_id())))
	 SMP_LOCK_ERR();
   }
   SET_UNLOCK_ADDR(irq);
#endif
   syncw();
   zzkrn_bkl_irq.lock = 0;
   syncw();
   set_msr(msr);
}

/*
**********************************************************************
** zzbkl_lock
**********************************************************************
*/

void
zzbkl_lock(void)
{
#ifdef MAX_SPIN_CNT
   uint32_t spin_cnt = MAX_SPIN_CNT;
#endif
   Msr msr;
   volatile uint32_t *const lockp = &zzkrn_bkl_irq.lock;

   msr = get_msr();
   set_msr(msr & ~MSR_EE);

   for (;;)
   {
      uint32_t lock = try_lock(lockp);
      
      if (lock == 0)
      {
	 SET_LOCK_ADDR(irq);
	 return;
      }
#ifdef RECURSIVE
      else if (unlikely((lock & CPU_ID_MASK) == zzkrn_cpu_id()))
	 SMP_LOCK_ERR();
#endif
      for(;;)
      {
	 if (*lockp == 0)
	    break;
	 RE_ENABLE_INTERRUPTS(msr);
	 CHECK_MAX_SPIN_CNT();
      }
   }
}

/*
**********************************************************************
** zzbkl_unlock
**********************************************************************
*/
void
zzbkl_unlock(void)
{
#ifdef RECURSIVE
   if (zzkrn_bkl_lock_id())
   {
      uint32_t lock = zzkrn_bkl_irq.lock;
      
      if (unlikely(!lock || ((lock & CPU_ID_MASK) != zzkrn_cpu_id())))
	 SMP_LOCK_ERR();
   }
   SET_UNLOCK_ADDR(irq);
#endif
   syncw();
   zzkrn_bkl_irq.lock = 0;
   syncw();
   set_msr(get_msr() | MSR_EE);
}

/*
**********************************************************************
** zzbkl_unlock_keep_int_mask
**********************************************************************
*/

void
zzbkl_unlock_keep_int_mask(void)
{
   uint32_t lock = zzkrn_bkl_irq.lock;

   if (likely(lock && ((lock & CPU_ID_MASK) == zzkrn_cpu_id())))
   {
      SET_UNLOCK_ADDR(irq);
      syncw();
      zzkrn_bkl_irq.lock = 0;
   }
#if 0
   else
      SMP_LOCK_ERR();
#endif
}

/*
**********************************************************************
** zzsmp_lock_swap
**********************************************************************
*/

void
zzsmp_lock_swap(void)
{
#ifdef MAX_SPIN_CNT
   uint32_t spin_cnt = MAX_SPIN_CNT;
#endif
   volatile uint32_t *const lockp = &zzkrn_bkl_swp.lock;

#ifdef RECURSIVE
   if(zzcur_p->priority > 32)
      SMP_LOCK_ERR();
#endif

   for (;;)
   {
      uint32_t lock;
      Msr msr;

      /*
      ** Spin and return with interrupts enabled, to avoid deadlock
      ** if we get choked by ipi events, or if other cpu polls for 
      ** effect of sent ipi events with a locked dispatcher.
      ** Note this difference from unicore!
      */
      msr = get_msr() | MSR_EE;

      /* Disable interrupts, to inhibit rebind of me to another cpu. */
      set_msr(msr & ~MSR_EE);
      lock = try_lock(lockp);

      if (lock == 0)
      {
	 SET_LOCK_ADDR(swp);
	 set_msr(msr);
	 return;
      }
      if (unlikely((lock & CPU_ID_MASK) == zzkrn_cpu_id()))
      {
#ifdef RECURSIVE
	 SMP_LOCK_ERR();
#endif
	 set_msr(msr);
	 return;
      }
      set_msr(msr);
      while (*lockp != 0)
      {
	 CHECK_MAX_SPIN_CNT();
      }
      
   }
}

/*
**********************************************************************
** zzsmp_unlock_swap
**********************************************************************
*/


void
(zzsmp_unlock_swap)(void)
{
#ifdef RECURSIVE
   uint32_t lock = zzkrn_bkl_swp.lock;

   if (unlikely(lock && ((lock & CPU_ID_MASK) != zzkrn_cpu_id())))
      SMP_LOCK_ERR();
   SET_UNLOCK_ADDR(swp);
#endif
   syncw();
   zzkrn_bkl_swp.lock = 0;
   syncw();
}

/*
**********************************************************************
** add_smp_cmds
**********************************************************************
*/

#include "stdio.h"

typedef int (* CmdType) (int argc, char ** argv);
void shell_add_cmd(const char *cmd_name, const char *usage,
                   const char *description, CmdType hook);

static int
smpstats(int argc, char *argv[])
{
   (void)argc;
   (void)argv;

   printf("Recursive checking: "
#ifdef RECURSIVE
	  "on\n"
#else
	  "off\n"
#endif
   "Re-enable interrupts:"
#ifdef RE_ENABLE_INTERRUPTS_IS_ON
	  "on\n"
#else
	  "off\n"
#endif
      );
   printf("Max spin cnt: "
#ifdef MAX_SPIN_CNT
	  "%#x\n", MAX_SPIN_CNT
#else
	  "off\n"
#endif
      );
   return 0;
}

void
add_smp_cmds(void)
{
   shell_add_cmd("smpstats", "smpstats", "", smpstats);
}

