/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 1999-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 ****************************************************************************
 *			      HEADER
 *
 *     $Workfile:   ti_c.c  $
 *     Document no: @(#) 511/OSE120-4 $FileRevision:   /main/tb_current_ose5/22  $
 *     $Author:   anha  $
 *     $Date:   05/02/06 10:56:41  $
 *
 *
 ****************************************************************************

	CONTENTS
	--------

	1  Description
	2  History of development
	3  Include files
	   3.1 Systemwide definitions
	   3.2 Libraries
	   3.3 Blockwide definitions
	   3.4 Global signals
	   3.5 Local signals
	4  Local definitions
	   4.1 Manifest constants
	   4.2 Type definitions
	   4.3 Macros
	   4.4 Signal composition
	5  External references
	   5.1 Functions
	   5.2 Variables
	   5.3 Forward references
	   5.4 Processes
	6  Local variables
	   6.1 General variables
	   6.2 Receive specifications
	7  Global variables
	8  Local functions
	9  Global functions
	10 Program entrypoint

*****************************************************************************
*/

/*
*****************************************************************************
** 1  DESCRIPTION.
*****************************************************************************
** This file contains the c part of target intarface for Mips.
**
*/

/*
*****************************************************************************
** 2  HISTORY OF DEVELOPMENT.
*****************************************************************************
** $Log:   /rtt/pvcsarch/OSE120/src/ti_c.c_v  $
 *
 *    Rev 1.6   12/14/00 13:33:44   anha
 *
 *    Rev 1.5   02/15/00 11:32:48   anha
 *
 *    Rev 1.4   06/30/99 23:58:08   anha
 *
 *    Rev 1.3   06/21/99 10:51:16   anha
 *
 *    Rev 1.2   06/04/99 15:21:10   anha
 *
 *    Rev 1.1   05/18/99 08:26:36   anha
 *
 *    Rev 1.0   05/05/99 15:30:02   anha
 * Initial revision.
 *
 *    Rev 1.0   04/29/99 14:42:12   anha
 * Initial revision.
*/

/*
*****************************************************************************
** 3  INCLUDE FILES.
*****************************************************************************
*/

/*
**===========================================================================
** 3.1  Systemwide definitions.
**===========================================================================
*/

#include <stddef.h>
#include <stdint.h>
#include "string.h"

#include "osenames.h"
#include "ose.h"
#include "ose_err.h"
#include "cpu_regdump.h"

#include "hal.h"
#include "pose.con"

#include "register.h"
#include "efmi_registers.h"

#include "uarea.h"
#include "tidata.h"
#include "inline.h"
#include "krncon.h"
#include "pose.h"
#include "ti_cpu_data.h"
#include "pose_internal.h"
#include "bios.h"
#include "exec_start.h"
#include "mm_access.h"

#include "assert.h"
#include "ramlog.h"

#include "cpu_access.h"
#include "cpu_regs.h"

/*
**===========================================================================
** 3.2  Libraries.
**===========================================================================
*/

/*
**===========================================================================
** 3.3  Blockwide definitions.
**===========================================================================
*/

#include "exec_arch.h"

/*
**===========================================================================
** 3.4  Global signals.
**	These signals are visible outside the current block of processes.
**===========================================================================
*/

/*
**===========================================================================
** 3.5  Local signals.
**	These signals are used only within the current block of processes.
**===========================================================================
*/

/*
*****************************************************************************
** 4  LOCAL DEFINITIONS.
*****************************************************************************
*/

/*
**===========================================================================
** 4.1  Manifest constants.
**===========================================================================
*/

#ifndef NULL
#define NULL	((void *) 0)
#endif

/* For run mode breakpoints use instruction; teqi zero,0 */
static const U32 trap = 0x040C0000;

#define RUN_MODE 0

/*
**===========================================================================
** 4.2  Type definitions.
**===========================================================================
*/

/* Help struct for indexing array in clib. */
struct clib_pointer
{
      U32 zzerrh_caller_return;
      U32 zzprocess_ended;
};

#ifdef SET_BP_SUPPORT
/*  Struct contains needed information for each breakpoint. */
struct bp
{
      struct bp *next;
      struct bp *local_next;
      OSADDRESS address;         /* Address for bp. */
      OSADDRESS address_post;    /* Next executed address after bp is hit. */ 
      OSADDRESS attribute;       /* Indicates post or pre bp. */
      void *breakpoint_id;       /* Id, several bp can exist on same address.*/
      void *uarea;                
      OSADDRESS saved_instruction; /* The original instruction that was
				      replaced by the bp. */
      U32 type_post;               /* Shows instruction type of the
				      original instruction. */
};

/* Help struct for call back function used by mm_update_code_forced() */
struct memcpy_arg 
{
   void         *dest;
   const void   *src;
   size_t        size;   
};
#endif

/* Helper typedef to cast call back function, used by mm_update_code_forced().
 */
typedef void (*func_t)(void *);


/* Register struct passed from zzunpack_regs() to zzset_regs() */

#define NO_OF_UNPACKED_U32_IN_UAREA ((offsetof(UAREA, isr) - \
				   offsetof(UAREA, izero)) / 4  + 1)
#define NO_OF_UNPACKED_U32_IN_FPU ((offsetof(struct fpu, fpr31.lsw) - \
		     offsetof(struct fpu, fcr0)) / 4 + 1)

struct unpacked_regs 
{
      UAREA ua;
      struct fpu fpu;
      U32 update_count;

      /* One flag for each 32 bit word in the range
       * [ua.izero : ua.ihi] and
       * [fpu.fcr0 : fpu.fpr31.lsw]
       */
      char update[NO_OF_UNPACKED_U32_IN_UAREA + NO_OF_UNPACKED_U32_IN_FPU];
};

/*
**===========================================================================
** 4.3  Macros.
**===========================================================================
*/
/* BP constants. */
#define ATTR_PRE 0
#define ATTR_POST 1

/* Misc */
#define ADDRESS_UNSET    0xFF
#define STATUS_REGISTER  12

/*
 * Mask codes.
 */
#define MASK_BIT_31_29 0xE0000000
#define MASK_BIT_31_28 0xF0000000
#define MASK_BIT_31_18 0xFFFC0000
#define MASK_BIT_28_26 0x1C000000
#define MASK_BIT_25_0  0x03FFFFFF
#define MASK_BIT_25_24 0x03000000
#define MASK_BIT_23_21 0x00E00000
#define MASK_BIT_25_21 (MASK_BIT_25_24 | MASK_BIT_23_21)
#define MASK_BIT_20_19 0x00180000
#define MASK_BIT_20_16 0x001F0000
#define MASK_BIT_18_16 0x00070000
#define MASK_BIT_17    0x00020000
#define MASK_BIT_15_0  0x0000FFFF
#define MASK_BIT_15_11 0x0000F800
#define MASK_BIT_5_3   0x00000038
#define MASK_BIT_2_0   0x00000007

/*
 * Field codes.
 */
#define RT_HI(x)          ((x & MASK_BIT_20_19) >> 19)
#define RT_LO(x)          ((x & MASK_BIT_18_16) >> 16)
#define FUNC_HI(x)        ((x & MASK_BIT_5_3)   >>  3)
#define FUNC_LO(x)        ((x & MASK_BIT_2_0)   >>  0)
#define RS(x)             ((x & MASK_BIT_25_21) >> 21)
#define RT(x)             ((x & MASK_BIT_20_16) >> 16)
#define RS_HI(x)          ((x & MASK_BIT_25_24) >> 24)
#define RS_LO(x)          ((x & MASK_BIT_23_21) >> 21)
#define OPCODE_HI(x)      ((x & MASK_BIT_31_29) >> 29)
#define OPCODE_LO(x)      ((x & MASK_BIT_28_26) >> 26)
#define CP0_REGISTER(x)   ((x & MASK_BIT_15_11) >> 11)

/* Get address to intframe register rx ( * 2 for 64 bits).
 */
#define GET_INTFRAME_REG(intframe,rx) (&(intframe->zero.u32.l) + rx * 2)

/*
**===========================================================================
** 4.4  Signal composition.
**===========================================================================
*/

/*
*****************************************************************************
** 5  EXTERNAL REFERENCES.
*****************************************************************************
*/

/*
**===========================================================================
** 5.1  Functions.
**===========================================================================
*/

/* Function for reading small data pointer. */
extern void zzsda(OSADDRESS *gp, OSADDRESS *sr);

extern void zzset_sr(OSADDRESS and, OSADDRESS or);
extern OSADDRESS zzset_cause(OSADDRESS and, OSADDRESS or);
extern OSADDRESS zzget_fcr0(void);

#ifdef SET_BP_SUPPORT
/* FIXME
 * -exactly how should this interface be?
 * -this should be declared in an include file
 * For now: return  MM_SUCCESS on success.
 */
#define MM_SUCCESS 0
extern U32
mm_update_code_forced(U32 base, U32 size, void (*func)(void *), void *fa);
#endif

void zzkrn_halted_hook(void);

OSADDRESS
zzti_sysconf_54(OSADDRESS p, OSADDRESS q, OSADDRESS r);

PROCESS
zzti_create_block(const char *name, OSUSER user, OSBOOLEAN use_remote_calls,
		  PROCESS remote_call_server, OSBOOLEAN supervisor_mode);

/*
**===========================================================================
** 5.2  Variables.
**===========================================================================
*/

/*
**===========================================================================
** 5.3  Forward references.
**===========================================================================
*/

void ZZT0201(U32 prid);

U32 zzhandle_breakpoint(struct intframe *intframe, OSPRIORITY int_level,
			OSPRIORITY oldint_level);

static int calledFromUserMode(void);

void zzearlysys_callh_c(U32 fcode, U32 return_address, U32 arg_arr[]);

int zzti_get_mem(PROCESS id, OSADDRESS from, void *to, OSADDRESS size,
		 OSADDRESS phase, OSADDRESS error_address, OSADDRESS sp);
int zzti_set_mem(PROCESS id, void *from, OSADDRESS to, OSADDRESS size,
		 OSADDRESS phase, OSADDRESS error_address, OSADDRESS sp);
/*
**===========================================================================
** 5.4  Processes.
**===========================================================================
*/

/*
*****************************************************************************
** 6  LOCAL VARIABLES.
*****************************************************************************
*/

/*
**===========================================================================
** 6.1  General variables.
**===========================================================================
*/

#ifdef OSE_DEBUG
/*
** Used during kernel start to handle early bios calls.
*/
static UAREA uarea_start;
#endif

#ifdef SET_BP_SUPPORT
/* Break point lists. */
static struct bp *bp_list;
static struct bp *bp_free_list;
#endif
/*
**===========================================================================
** 6.2 Receive specifications.
**===========================================================================
*/

/*
*****************************************************************************
** 7  GLOBAL VARIABLES.
*****************************************************************************
*/

/* Ti per process data (also used for uni core). */
struct per_cpu zzcpu[OSE_SMP_MAX_CPUS] __attribute__ ((aligned (128)));

/* This variable contains all ti data. */
struct tidata zzti __attribute__ ((aligned (128)));

/* This variable contains interrupt vector information. */
void *zzint_table[MAX_INTVEC] __attribute__ ((aligned (32)));

/* This variable contains configuration data. */
struct Ti_ctlblk zzconf;

/* Flag that turns on register save for debug system calls. */
U32 zzdebug_regs_on;

/* Table for bios data. */
struct BiosList zzbios_table[BIOS_TABLE_LEN] __attribute__ ((aligned (32)));

/* Initialize pose ti variable pointer. */
const U32 zztivarp[] = {0x123, (const U32) &zzti};

struct
{
      struct BiosList *table;
      U32              table_len;
      U32              entry_size;
      int              (*called_from_user_mode)(void);
} ZZB0107 = {zzbios_table,
	     BIOS_TABLE_LEN,
	     BIOS_ENTRY_SIZE,
	     calledFromUserMode};
/*
*****************************************************************************
** 8  LOCAL FUNCTIONS.
*****************************************************************************
*/

/*
**===========================================================================
** calledFromUserMode
**===========================================================================
** Description:
**
** Parameters:
**
** Returns:	-
**
** Globals:	-
*/

static int calledFromUserMode(void)
{
   UAREA *uarea = zzcurrent_uarea;

   /* If kernel not started, return supervisor mode. */
   return zzcur_p ? !uarea->super_mode : 0;
}

/*
**===========================================================================
** long2str
**===========================================================================
** Description:	Convert integer to string.
**
** Parameters:	Target string.
**		Integer value to convert.
**
** Returns:	-
**
** Globals:	-
*/

static void long2str(char *s, OSADDRESS l)
{
   static const char table[] = "0123456789ABCDEF";
   int i;

   for(i = 7; i >= 0; i--)
   {
      *(s + i) = table[l & 0xf];
      l >>= 4;
   }
}

/*
**===========================================================================
** fpr2str
**===========================================================================
** Description:	Convert floating point to string.
**
** Parameters:	Target string.
**		Value to convert.
**
** Returns:	stepped target string pointer
**
** Globals:	-
*/

#if OSE_DEBUG
static char *fpr2str(char *s, FPR *f)
{
  long2str(s, f->msw);
  s += 8;
  long2str(s, f->lsw);
  return s + 8;
} /* fpr2str */
#endif

/*
**===========================================================================
** str2long
**===========================================================================
** Description:	Convert a string to integer.
**
** Parameters:	String to convert.
**
** Returns:	Integer value.
**
** Globals:	-
*/

static OSADDRESS str2long(const char *ptr)
{
   static const unsigned char table[32] =
   {0,1,2,3,4,5,6,7,
    8,9,0,0,0,0,0,0,
    0,10,11,12,13,14,15,0,
    0,0,0,0,0,0,0,0};
   OSADDRESS ret = 0;
   int i;

   for(i = 0; i < 8; i++)
   {
      ret <<= 4;
      ret += table[(*ptr-'0') & 31];
      ptr++;
   }
   return ret;
}

/*
**===========================================================================
** alloc_uarea_block
**===========================================================================
** Description: Allocate an uarea.
**              This function must be called from non-int level.
**
** Parameters:  -
**
** Returns:     uarea
**
** Globals:     uarea_free_list
*/

void *zzalloc_uarea_block(void)
{
   UAREA *uarea;

   /* Allocate an uarea from the free list or from the pool if required. */
   BKL_LOCK();
   if(zzti.free_uarea)
   {
      uarea = zzti.free_uarea;
      zzti.free_uarea = uarea->next;
      BKL_UNLOCK();
   }
   else
   {
      BKL_UNLOCK();
      /* Read size of user area and add it to uarea size. */
      uarea = (struct uarea *)
	 zzalloc_kernelmemory((OSBUFSIZE) sizeof(UAREA_UNION) +
			      zzconf.user_area);
      uarea->instance = 0;
   }
   return uarea;
}

/*
**===========================================================================
** Free uarea block
**===========================================================================
** Description: Free an uarea to free list.
**              This function must be called from non-int level.
**
** Parameters:  uarea
**
** Returns:     -
**
** Globals:     uarea_free_list
*/

static void free_uarea_block(UAREA *uarea)
{
   /* Increment number of times used. */
   uarea->instance++;

   BKL_LOCK();

   /* Free the uarea to the local free list. */
   uarea->next = zzti.free_uarea;
   zzti.free_uarea = uarea;
   BKL_UNLOCK();
}

/*
**===========================================================================
** set64
**===========================================================================
** Description: Assign a 64 bit variable from a 32 bit value.
**              Bit 31 is signed extended into bit 32-63.
**
** Parameters: 
**
** Returns:     -
**
** Globals:
*/

static void
set64(REG64 *reg64, U32 value)
{
   reg64->u32.l = value;
   reg64->u32.h = value & 0x80000000 ? ~0 : 0;
}

/*
**===========================================================================
** process_start
**===========================================================================
** Description:	Entrypoint for new processes.
**
** Parameters:	-
**
** Returns:	-
**
** Globals:	-
*/

static void process_start(void)
{
   UAREA *uarea = (UAREA *) zzcurrent_uarea;
   void (*cont_preemption)(void);

   zzinit_process();

   uarea->reg_state = REG_STATE_NA; /* Registers not available. */

   cont_preemption = (void (*)(void)) zzti.cont_preemption;
   (cont_preemption)();	/* Jump to assembler part. */
}

/*
**===========================================================================
** search_bp
**===========================================================================
** Description:
**
** Parameters:	-
**
** Returns:	-
**
** Globals:	-
*/
#ifdef SET_BP_SUPPORT
static struct bp
*search_bp(OSADDRESS address, void *uarea, OSBOOLEAN uarea_valid)
{
   struct bp *bb = bp_list;

   while(bb)
   {
      /* Search main list for address match. */
      if(bb->address == address)
      {
	 /* Address found. */
	 if(!uarea_valid)
	 {
	    return bb;		/* Found any ua */
	 }

	 /* Search local list for uarea match. */
	 while(bb)
	   {
	     if(bb->uarea == uarea)
	       {
		 return bb;	/* Found this ua */
	       }
	     bb = bb->local_next;
	   }
	 return 0;		/* Found address, not this ua */
      }
      bb = bb->next;
   }
   return 0;			/* Adress not found. */
}
#endif
/*
**===========================================================================
** get_bp_block
**===========================================================================
** Description:
**
** Parameters:	-
**
** Returns:	-
**
** Globals:	-
*/
#ifdef SET_BP_SUPPORT
static struct bp *get_bp_block(OSADDRESS address, void *uarea,
			       OSBOOLEAN uarea_valid)
{
   struct bp *bb;

   bb = search_bp(address, uarea, uarea_valid);
   if(uarea_valid)
   {
      if(bb)
      {
	 /* Process specific bp found. */
      }
      else
      {
	 /* Search again for block bp (uarea = 0). */
	 bb = search_bp(address, 0, 1);
      }
   }
   return bb;
}
#endif
/*
 **===========================================================================
 **          zzget_trace_address
 **===========================================================================
 **
 **  Description:
 **                Finish the processing of the instruction when the process
 **                pc == the address of the post breakpoint. This function is
 **                needed when a breakpoint is placed on a conditional branch
 **                or jump via a register.
 **
 **  Parameters:
 **                type        - The type of instruction to process.
 **                instr       - The instruction to trace over.
 **                pc          - The address of the instr.
 **	           addr_post   - The address to set the post breakpoint.
 **
 ** Return:
 **                none.
 **
 */
void zzget_trace_address(U32 type, U32 instr, U32 pc,
			 void *intframe, U32 *post_addr)
{
   struct intframe *iframe = intframe;

   /*
    * Check if the instruction need more processing.
    */
   if((type == INST_JR) || (type == INST_JALR))
   {
      U32 rs = RS(instr);
      int value_rs;

      /* Yes, it is jump via a register. */
      assert(*post_addr == ADDRESS_UNSET);

      value_rs = *GET_INTFRAME_REG(iframe, rs);
      *post_addr = value_rs;

   }
   else if(BC(type))
   {
      /* Yes, it is conditional branch. Note that if the branch shall be taken
       * the *post_addr already keeps the correct branch address which
       * was precalculated in zzpreprocess_trace().
       */
      OSBOOLEAN jump = 0;
      U32 rs = RS(instr);
      U32 rt = RT(instr);
      int value_rs;
      int value_rt;

      value_rs = *GET_INTFRAME_REG(iframe, rs);

      switch(type)
      {
	 case INST_BEQ:
	    /* Calculate value_rt only when needed. */
	    value_rt = *GET_INTFRAME_REG(iframe, rt);
	    jump = ((value_rs == value_rt) ? 1 : 0);
	    break;
	 case INST_BNE:
	    /* Calculate value_rt only when needed. */
	    value_rt = *GET_INTFRAME_REG(iframe, rt);
	    jump = ((value_rs != value_rt) ? 1 : 0);
	    break;
	 case INST_BLEZ:
	    jump = ((value_rs <=  0) ? 1 : 0);
	    break;
	 case INST_BEGZ:
	 case INST_BEGZAL:
	    jump = ((value_rs >=  0) ? 1 : 0);
	    break;
	 case INST_BLTZ:
	 case INST_BLTZAL:
	    jump = ((value_rs <   0) ? 1 : 0);
	    break;
	 case INST_BGTZ:
	    jump = ((value_rs >   0) ? 1 : 0);
	    break;
	 default:
	    assert(0); /* This should never happen. */
      }
      if (!jump)
      {
	  /* The conditional branch shall not be taken, replace the
	   * pre calculated branch address with next consecutive instruction.
	   * Note that next possible instruction to set a bp is the instruction
	   * following the branch delay slot.
	   */
	 *post_addr = pc + 8;
      }
   }
}

/*
 **===========================================================================
 **          memcpy_call_back()
 **===========================================================================
 **
 **  Description:
 **                Wrapper function for memcpy used for callback
 **                from function mm_update_code_forced.
 **
 **  Parameters:  *arg   pointer to arguments for memcpy()
 **
 **
 ** Return:        void
 **
 */
#ifdef SET_BP_SUPPORT
static void
memcpy_call_back(struct memcpy_arg *arg)
{

   (void)memcpy(arg->dest,
		  arg->src,
		  arg->size);
}
#endif
/*
 **===========================================================================
 **          update_code_forced_failed()
 **===========================================================================
 **
 **  Description:
 **                Generic handling if mm_update_code_forced() fails.
 **
 **  Parameters:  void
 **
 **
 ** Return:        void
 **
 */

#ifdef SET_BP_SUPPORT
static void
update_code_forced_failed(OSADDRESS dest)
{
   assert(0);
   /* FIXME, a new err code needs to be defined in ose_err.h.
    * dest should be send as extra param.
    */
   (void)dest;
}
#endif
/*
 **===========================================================================
 **          decode_opcode_special()
 **===========================================================================
 **
 **  Description:
 **                Decode the opcode called special. Only decode
 **                relevant instructions such as jumps, syscall, traps etc.
 **
 **  Parameters:
 **                instr       - The instruction to decode.
 **
 ** Return:        result      - Decoded instruction.
 **
 */
static U32
decode_opcode_special(U32 instr)
{
   U32 result = INST_NORMAL;

   /* opcode '000000' func bits 5-3 '001' */
   if (FUNC_HI(instr) == 1)
   {
      switch(FUNC_LO(instr))
      {
	 case 0:
	    result = INST_JR;
	    break;
	 case 1:
	    result = INST_JALR;
	    break;
	 case 2:
	 case 3:
	    /* Do nothing, result is already set to INST_NORMAL. */
	    break;
	 case 4:
	    result = INST_SYSCALL;
	    break;
	 case 5:
	    /* Do not allow to set bp on BREAK */
	    result = INST_ILLEGAL;
	    break;
	 case 6:
	    result = INST_ILLEGAL;
	    break;
	 case 7:
	    /* Do nothing, result is already set to INST_NORMAL. */
	    break;
	 default:
	    assert(0);  /* should never happen */
	    result = INST_ILLEGAL;
      }
   }

   /* opcode '000000' func bits 5-3 '110' */
   if (FUNC_HI(instr) == 6)
   {
      switch(FUNC_LO(instr))
      {
	 case 0:
	 case 1:
	 case 2:
	 case 3:
	 case 4:
	 case 5:
	 case 6:
	 case 7:
	    /* Do not allow to set bp on other trap instructions
	     * than the one we use.
	     */
	    result = INST_ILLEGAL;
	    break;
	 default:
	    assert(0);  /* should never happen */
	    result = INST_ILLEGAL;
      }
   }
   return result;
}

/*
 **===========================================================================
 **          decode_opcode_regimm()
 **===========================================================================
 **
 **  Description:
 **                Decode the opcode called RegImm. Only decode
 **                relevant instructions such as jumps, syscall, traps etc.
 **
 **  Parameters:
 **                instr       - The instruction to decode.
 **
 ** Return:        result      - Decoded instruction.
 **
 */
static U32
decode_opcode_regimm(U32 instr)
{
   U32 result = INST_NORMAL;

   /* opcode '000001' rt bits 20-19 '00' */
   if (RT_HI(instr) == 0)
   {
      switch(RT_LO(instr))
      {
	 /* Note that BLTZL, BEGZL can be
	  * handled exactly the same way as BLTZ and BEGZ
	  */
	 case 0:
	    result = INST_BLTZ;
	    break;
	 case 1:
	    result = INST_BEGZ;
	    break;
	 case 2:
	    result = INST_BLTZ;
	    break;
	 case 3:
	    result = INST_BEGZ;
	    break;
	 case 4:
	 case 5:
	 case 6:
	 case 7:
	    result = INST_ILLEGAL;
	    break;
	 default:
	    assert(0);  /* should never happen */
	    result = INST_ILLEGAL;
      }
   }
   /* opcode '000001' rt bits 20-19 '01' */
   if (RT_HI(instr) == 1)
   {
      switch(RT_LO(instr))
      {
	 case 0:
	 case 1:
	 case 2:
	 case 3:
	    /* Do not allow to set bp on other trap instructions
	     * than the one we use.
	     */
	    result = INST_ILLEGAL;
	    break;
	 case 4:
	    /* A bp already exists at this address. */
	    result = INST_TEQI;
	    break;
	 case 5:
	 case 6:
	 case 7:
	    /* Do not allow to set bp on other trap instructions
	     * than the one we use.
	     */
	    result = INST_ILLEGAL;
	    break;
	 default:
	    assert(0);  /* should never happen */
	    result = INST_ILLEGAL;
      }
   }

   /* opcode '000001' rt bits 20-19 '10' */
   if (RT_HI(instr) == 2)
   {
      switch(RT_LO(instr))
      {
	 /* Note that BLTZALL, BGEZALL can be
	  * handled exactly the same way as BLTZAL and BGEZAL.
	  */
	 case 0:
	    result = INST_BLTZAL;
	    break;
	 case 1:
	    result = INST_BEGZAL;
	    break;
	 case 2:
	    result = INST_BLTZAL;
	    break;
	 case 3:
	    result = INST_BEGZAL;
	    break;
	 case 4:
	 case 5:
	 case 6:
	 case 7:
	    result = INST_ILLEGAL;
	    break;
	 default:
	    assert(0);  /* should never happen */
	    result = INST_ILLEGAL;
      }
   }

   return result;

}


/*
 **===========================================================================
 **          decode_opcode_COP0()
 **===========================================================================
 **
 **  Description:
 **                Decode the opcode called COP0. Only decode
 **                relevant instructions such as jumps, syscall, traps etc.
 **
 **  Parameters:
 **                instr       - The instruction to decode.
 **
 ** Return:        result      - Decoded instruction.
 **
 */
static U32
decode_opcode_COP0(U32 instr)
{
   U32 result = INST_NORMAL;

   /* opcode '010000' rs bits 25-24 '00' */
   if (RS_HI(instr) == 0)
   {
      switch(RS_LO(instr))
      {
	 case 0:
	    /* This is a MFC0. Check if it is concerning status register. */
	    if (CP0_REGISTER(instr) == STATUS_REGISTER)
	    {
	       result = INST_MFC0_STATUS_REG;
	    }
	    break;
	 case 1:
	 case 2:
	 case 3:
	    result = INST_ILLEGAL;
	    break;
	 case 4:
	    /* This is a MTC0. Check if it is concerning status register. */
	    if (CP0_REGISTER(instr) == STATUS_REGISTER)
	    {
	       result = INST_MTC0_STATUS_REG;
	    }
	    break;
	 case 5:
	 case 6:
	 case 7:
	    result = INST_ILLEGAL;
	    break;
	 default:
	    assert(0);  /* should never happen */
	    result = INST_ILLEGAL;
      }
   }

   /* opcode '010000' rs bits 25_24 '1X' and func bits 5-3 '011' */
   if (RS_HI(instr) >= 2 && FUNC_HI(instr) == 3)
   {
      switch(FUNC_LO(instr))
      {
	 case 0:
	    /* ERET, return from interrupt is not traceable */
	 case 1:
	 case 2:
	 case 3:
	 case 4:
	 case 5:
	 case 6:
	 case 7:
	    /* DERET, return from debug exception is not traceable */
	    result = INST_ILLEGAL;
	    break;
	 default:
	    assert(0);  /* should never happen */
	    result = INST_ILLEGAL;
      }
   }
   return result;
}

/*
 **===========================================================================
 **          zzpreprocess_trace()
 **===========================================================================
 **
 **  Description:
 **                Preprocess as much as possible of a trace emulation to
 **                avoid overhead when tracing over a breakpoint.
 **
 **  Parameters:
 **                instr       - The instruction to trace over.
 **                pc          - The address of the instr.
 **	           addr_post   - The address to set the post breakpoint.
 **
 ** Return:    INST_NORML      - The trace preprocessing is done and the
 **                              address to branch to is in addr.
 **            INST_ILLEGAL    - The instruction (instr) do not allow a trace
 **                              to be done.
 **            INST_BC|X       - A branch conditional instruction of type X,
 **                              need more processing, the branch address is
 **                              put in addr. The branch address may or may
 **                              not be used in the next step of trace
 **                              processing.
 **            INST_BR|X       - A branch instruction f type X, if possible to
 **                              calculate the branch address is put in addr.
 **            INST_SYSCALL    - A system call instruction.
 **            INST_TEQI       - A trap instruction, means that a bp already
 **                              exists on address.
 **       INST_MTCO_STATUS_REG - MTCO instruction to CPU status register.
 **       INST_MFC0_STATUS_REG - MFC0 instruction to CPU status register..
 **
 */
U32
zzpreprocess_trace(U32 instr, U32 pc, U32 *addr_post)
{
   U32 result = INST_NORMAL;
   U32 offset;

   /* opcode bits 31-29 '000' */
   if (OPCODE_HI(instr) == 0)
   {
      switch(OPCODE_LO(instr))
      {
	 case 0:
	    result = decode_opcode_special(instr);
	    break;
	 case 1:
	    result = decode_opcode_regimm(instr);
	    break;
	 case 2:
	    result = INST_J;
	    break;
	 case 3:
	    result = INST_JAL;
	    break;
	 case 4:
	    result = INST_BEQ;
	    break;
	 case 5:
	    result = INST_BNE;
	    break;
	 case 6:
	    result = INST_BLEZ;
	    break;
	 case 7:
	    result = INST_BGTZ;
	    break;
	 default:
	    assert(0);  /* should never happen */
	    result = INST_ILLEGAL;
      }
   }

   /* opcode bits 31-29 '010' */
   if (OPCODE_HI(instr) == 2)
   {
      switch(OPCODE_LO(instr))
      {
	 /* Note that BEQL, BNEL BLEZL and BGTZL can be
	  * handled exactly the same way as BEQ, BNE, BLEZ, and BGTZ
	  */
	 case 0:
	    result = decode_opcode_COP0(instr);
	    break;
	 case 1:
	 case 2:
	 case 3:
	    result = INST_ILLEGAL;
	    break;
	 case 4:
	    result = INST_BEQ;
	    break;
	 case 5:
	    result = INST_BNE;
	    break;
	 case 6:
	    result = INST_BLEZ;
	    break;
	 case 7:
	    result = INST_BGTZ;
	    break;
	default:
	   assert(0);  /* should never happen */
	   result = INST_ILLEGAL;
      }
   }

   /*
    * Instructions are now categorized, go ahead and
    * calculate address to set breakpoint.
    */
   switch(result)
   {
      case INST_J:
      case INST_JAL:
      {
	 U32 instr_index, pc_gprlen_28;

	 instr_index = (instr & MASK_BIT_25_0) << 2;
	 pc_gprlen_28 = pc & MASK_BIT_31_28;
	 *addr_post = instr_index | pc_gprlen_28;
	 break;
      }
      case INST_JR:
      case INST_JALR:
	 /* The jump address depends on register value and
	    can not be precalculated. */
	 *addr_post = ADDRESS_UNSET;
	 break;
      case INST_BEQ:
      case INST_BNE:
      case INST_BLEZ:
      case INST_BGTZ:
      case INST_BLTZ:
      case INST_BEGZ:
      case INST_BLTZAL:
      case INST_BEGZAL:
      {
	 /* All conditional branches calculates address same way;
	  * mask, shift 2 and sign extend to get branch offset.
	  */
	 offset = (instr & MASK_BIT_15_0) << 2;

	 /* check sign bit */
	 if (offset & MASK_BIT_17)
	 {
	    /* Negative relative branch, sign extend with ones. */
	    offset = offset | MASK_BIT_31_18;
	 }
	 /* Note the, the 18-bit signed offset is added to the address
	  * following the branch (not the branch itself).
	  */
	 *addr_post = pc + 4 + offset;

	 break;
      }
      case INST_SYSCALL:
      case INST_NORMAL:
	 *addr_post = pc + 4;
	 break;
      case INST_MTC0_STATUS_REG:
      case INST_MFC0_STATUS_REG:
	 /*
	  * INST_MTC0_STATUS_REG and INST_MFC0_STATUS_REG
	  * instructions are handled in a special way in trap handler.
	  */
	 *addr_post = ADDRESS_UNSET;
	 break;
      case INST_TEQI:
	 /* This means that a bp already exists on this address.
	  */
	 *addr_post = ADDRESS_UNSET;
	 break;
      case INST_ILLEGAL:
	 *addr_post = ADDRESS_UNSET;
	 break;
      default:
	 assert(0);  /* The code is broken if this happens. */
	 *addr_post = ADDRESS_UNSET;
	 result = INST_ILLEGAL;
	 break;
   }

   return result;
}

/*
**===========================================================================
** get_per_cpu_idle_sp
**===========================================================================
** Description:	Get the per cpu idle stack allocated by pose.
**
** Parameters:	cpu id
**
** Returns:	Aligned stack pointer.
**
** Globals:	
*/

#ifdef OSE_SMP
static U32
get_per_cpu_idle_sp(cpuid_t cpu_id)
{
   return ((U32)zzcpu[cpu_id].idle_pcb->stack->top - 16) & ~0xf;
}
#endif


#ifdef OSE_SMP
/*
** Force an exception on another cpu if possible.
** See pose.h
*/
int
zzkrn_force_exception_in_other_cpu(cpuid_t target_cpu)
{
  /* FIXME?? Stub only.
  ** See powerpc/krn/exec/ti_c.c for configuration info.
  */
  return 0;			
}
#endif

/*
**===========================================================================
**
**===========================================================================
*/
static void
init_internal_timer(void)
{
   U32 timer_value = zzconf.count_per_tick;
   U32 sr;

   sr = read_cp0(CP0_STATUS, 0);
   sr |= 1 << 15;
   write_cp0_status(sr);
   write_cp0(CP0_COMPARE, 0, timer_value);
   write_cp0(CP0_COUNT, 0, 0);
}

/*
**===========================================================================
**
**===========================================================================
*/
static unsigned long
internal_timer_proc(void *arg)
{
   U32 timer_value;

   (void)arg;
   timer_value = zzconf.count_per_tick + read_cp0(CP0_COUNT, 0);

   write_cp0(CP0_COMPARE, 0, timer_value);
   return zzticker_semwakeup(NULL);
}

/*
**===========================================================================
** main_init
**===========================================================================
** Description:	This function is called in the context of the main process.
**		in the monolith.
**
** Parameters:	-
**
** Returns:	-
**
** Globals:	
*/

static void
main_init(void)
{
   extern void _zzmain(void);

#ifdef OSE_SMP
   {
      cpuid_t num_cpus = zzkrn_num_cpus();
      cpuid_t i;

      /* Pick up idle stacks from pose. */
      for (i = 1; i < num_cpus; i++)
      {
	 zzcpu[i].ticpu.idle_sp = get_per_cpu_idle_sp(i);
      }
   }
#endif
   /*
    * Install a function for sysconf_54 (usrcalls struct).
    * The monolith will get a the original struct with fast funtions,
    * but load modules will get pointers to trapping calls.
    */
   zzti.sysconf_54 = get_sysconf(SYSCONF_NEW_FUNCTION,
				 54,
				 (OSADDRESS)zzti_sysconf_54,
				 1); /* Allow call from user mode. */

   /*
    * Set error checks level for lib's that support runtime config.
    */
   get_sysconf(SYSCONF_SET_ERROR_CHECK, zzconf.error_checks, 0, 0);

   /* Check if internal timer is configured. */
   if (zzconf.count_per_tick)
   {
      ramlog_printf("exec: Creating isr for tick at vector %d, reload=%u\n",
                    zzconf.internal_timer_vector, zzconf.count_per_tick);
      /* Create tick int process. */
#if 0
      create_process(OS_INT_PROC,
		     "ose_ticker",
		     zzticker_semwakeup,/* Entrypoint for tick process.*/
		     200,               /* Stacksize   */
		     0,	                /* Priority    */
		     0,                 /* Timeslice   */
		     0x10001,           /* Bid:OSE     */
		     0,                 /* Redirection */
		     zzconf.internal_timer_vector,
		     0);                /* User number */
#else
      zzkrn_create_isr(zzconf.internal_timer_vector, 0,
                       internal_timer_proc, NULL);
#endif

      /* Enable internal timer. */
      init_internal_timer();
   }

   /* Jump to real entrypoint. */
   _zzmain();
}

/*
*****************************************************************************
** 9  GLOBAL FUNCTIONS.
*****************************************************************************
*/

/*
**===========================================================================
** Lib check.
**===========================================================================
*/
void __OSE_LIB__MEMORY_MODEL_0(void);
void __OSE_LIB__MEMORY_MODEL_1(void);
void __OSE_LIB__ERROR_CHECKS_0(void);
void __OSE_LIB__ERROR_CHECKS_1(void);
void __OSE_LIB__ERROR_CHECKS_2(void);
void __OSE_LIB__DEBUG_NO(void);
void __OSE_LIB__DEBUG_YES(void);
void __OSE_LIB__REMOTE_CALLS_NO(void);
void __OSE_LIB__REMOTE_CALLS_YES(void);

#if MEMORY_MODEL == 0
void __OSE_LIB__MEMORY_MODEL_0(void) { }
#else
void __OSE_LIB__MEMORY_MODEL_1(void) { }
#endif

#if ERROR_CHECKS == 0
void __OSE_LIB__ERROR_CHECKS_0(void) { }
#else
void __OSE_LIB__ERROR_CHECKS_1(void) { }
void __OSE_LIB__ERROR_CHECKS_2(void) { }
#endif

#if OSE_DEBUG == 0
void __OSE_LIB__DEBUG_NO(void) { ; }
#else
void __OSE_LIB__DEBUG_YES(void) { ; }
#endif

void __OSE_LIB__REMOTE_CALLS_NO(void) { }

/*
**===========================================================================
** void zzpower_off(void)
**===========================================================================
** Description:
**
** Parameters:
**
** Returns:	-
**
** Globals:	-
*/

void zzpower_off(void)
{
   void (*handler)(void) = (void (*)(void)) zzconf.power_off_handler;

   if(handler)
   {
      (handler)();
   }
}

/*
**===========================================================================
** void zzset_interrupt_stack(OSADDRESS istack_base, OSADDRESS size)
**===========================================================================
** Description:	Set the global interrupt stack pointer.
**		This stack is shared among all interrupt processes.
**		When a non interrupt process is interrupted, the current
**		stack pointer is saved, and the interrupt stack is loaded
**		before control is passed to the interrupt process.
**		Istack_base points at the lowest address within the stack area.
**		size is the number of bytes reserved for the interrupt stack.
**
** Parameters:
**
** Returns:	-
**
** Globals:	-
*/

void zzset_interrupt_stack(OSADDRESS istack_base, OSADDRESS size)
{
   cpuid_t cpu = zzkrn_cpu_id();

   *(U8 *) istack_base = END_MARK;
   zzcpu[cpu].ticpu.isp_limit = istack_base;
   /* Compensate isp for the first intframe size. */
   zzcpu[cpu].ticpu.isp = istack_base + size - sizeof(struct intframe);
}

/*
**===========================================================================
** zzcreate_context
**===========================================================================
** Description:
**
** Create a uarea and set up all fields. The process should then be ready to
** swap in with zzrestore_context(). Return a pointer to a uarea allocated
** from zzalloc_kernelmemory(). This uarea is used to keep process specific
** info managed by the target interface. For interrupt or timer processes,
** the stack, stack_size, entrypoint and super_mode parameters are invalid,
** but a uarea may still be required.
** Implementation specific parameters should be propagated to
** the new process, which in most cases means copying them to the
** stack of the new process.
** User mode processes may need a second stack in supervisor space. This stack
** is allocated and managed by the target interface.
** The newpid parameter contains the process id of the new process.
** The parent parameter contains the process id of the parent process or NULL
** if not available.
** The clib_handle parameter contains the clib handle of the block in which the
** new process resides, or NULL if no pointer is available.
**
** MSR bits will have the save state as sysd, ie same as kernel start state,
** exept EE, PR, FP and RI.
**
** Both zzcreate_context and zzkill_context are called from the sysd process.
** Because of this there is no need to disable interrupts when free lists
** are accessed.
**
** Parameters:
**
** Returns:	Returns pointer to the new uarea.
**
** Globals:
*/

void *zzcreate_context(void *stack,
		       OSADDRESS stack_size,
		       void (*entrypoint)(void),
		       OSBOOLEAN super_mode,
		       OSBOOLEAN interrupt_type,
		       PROCESS newpid,
		       PROCESS parent,
		       void *clib_handle)
{
   OSADDRESS gp, sr;
   UAREA *uarea;

   assert(((U32)stack & 0xf) == 0);

#ifdef OSE_SMP
   /* Set idle stack for cpu0. */
   {
      static int done;

      if(!done)
      {
	 done = 1;
	 zzcpu[0].ticpu.idle_sp = get_per_cpu_idle_sp(0);
      }
   }
#endif

   /* Allocate an uarea from the free list or from the pool if required. */
   uarea = zzalloc_uarea_block();

   /* Set up pointer to uarea used during process start-up. */
   set64(&uarea->s1, (U32) uarea);

   /* Cache pid in uarea. */
   uarea->pid = (U32) newpid;

   zzsda(&gp, &sr); 		/* Read registers to inherit. */

   /*
    * Clear CU1, UM, ERL and EXL bits.
    * For MIPS64 both KSU bits are cleared to select kernel mode.
    */
   sr &= 0xdfffffe0;
   sr |= 0x1;			/* Set IE. */

#ifndef OSE_SMP /* gp is a kernel register for smp. */
   set64(&uarea->gp, gp);/* @@@@ remove, compile without gp. */
#endif

   /* Needed for for first time swap in. */
   set64(&uarea->s0, (U32) &zzti);

   /*
   ** If parent exists, SDA reg values should be fetched from parent pid in
   ** order to inherit parent register values.
   */
   if(parent)
   {
      OSADDRESS *pcb_ptr;

      /*
      ** Call zzget_debugpcb
      ** Returns a pointer that points in the middle of a pcb.
      ** Increment pointer to point to uarea.
      */
      pcb_ptr = (OSADDRESS *) zzget_debugpcb(parent);

      if(pcb_ptr != NULL)
      {
	 UAREA *parent_uarea;

	 pcb_ptr += 7;			/* Inc pointer to uarea field. */
	 parent_uarea = (UAREA *) *pcb_ptr;
#ifndef OSE_SMP /* gp is a kernel register for smp. */
	 uarea->igp = uarea->gp = parent_uarea->gp; /* gp for int procs. */
#endif
      }
      else
      {
	 /* The creator has died and POSE will kill this child before
	 ** the create_process() system call ends. However, we want to
	 ** set this register to something other than just a random value,
	 ** just in case.
	 */
#ifndef OSE_SMP /* gp is a kernel register for smp. */
	 set64(&uarea->igp, 0);
#endif
      }
   }
   /*
   ** Parent is zero for system deamons. SDA pointers are set to kernel
   ** start values. The purpose of this is that handlers that are linked
   ** to the kernel and executes in the context of a system deamon should
   ** have correct SDA pointers.
   ** The os code itself is SDA independent.
   */
   else
   {
#ifndef OSE_SMP /* gp is a kernel register for smp. */
      set64(&uarea->igp, gp);
#endif
   }

   uarea->line = 0;
   uarea->reg_state = REG_STATE_ALL | REG_STATE_WRITABLE;
   uarea->fpu_reg_file = (struct fpu *) 0;
   uarea->exception_depth = 0;
   uarea->exception_regdump = NULL;
   set64(&uarea->safecopy_sp, 0);	/* Safecopy not active. */
   set64(&uarea->call_sp, 0);
   set64(&uarea->err_lev_0_fp, 0);
   set64(&uarea->err_lev_0_ra, 0);
   set64(&uarea->err_lev_0_sp, 0);

   /* Setup user area parameter for handlers. */
   if(zzconf.user_area)
   {
      /* Calculate user_area pointer. */
      uarea->user_area = (U32) uarea + sizeof(UAREA_UNION);
   }
   else
   {
      uarea->user_area = 0;	/* Null pointer if no user area. */
   }

   if(interrupt_type)
   {
      /* Init value for kernel debug purpose only. */
      uarea->magic = UAREA_MAGIC_I;
      uarea->super_mode = 1;	/* Used by kill_context */
      uarea->entry_hi = 0;	/* Save a static copy for test. */
   }
   else
   {
      /* Init value for kernel debug purpose only. */
      uarea->magic = UAREA_MAGIC;

      uarea->super_mode = super_mode;	/* Used by kill_context */
      set64(&uarea->iepc, (U32) entrypoint);
      set64(&uarea->ra, (U32) process_start);/* Point to process_start in ti_s. */
      uarea->sr = sr;		/* Used by restore_context first time. */
      set64(&uarea->izero, 0);
      set64(&uarea->ifp, 0);		/* Terminate backtrace ? */
      {
	 /* Use experimental function in pose. */
	 extern OSSEGMENT zzcreate_context_segnum(PROCESS pid);
	 OSSEGMENT segnum = zzcreate_context_segnum(newpid);
	 OSADDRESS asid;

	 /* Check if asid function is installed by hal_mmu. If the function
	  * is not installed we assume we are running a nommu system.
	  */
	 if(!super_mode && /* Supervisor processes always use asid = 0. */
	    parent &&      /* Don't do get_sysconf too early. */
	    get_sysconf(SYSCONF_MMU_HAL_GET_ASID, segnum, (OSADDRESS)&asid, 0))
	 {
#if 0
	    ramlog_printf("zzcreate_context: pid: %lx asid: %d\n",newpid,asid);
#endif
	    uarea->ientry_hi = asid;
	 }
	 else
	 {
	    uarea->ientry_hi = 0;
	 }
	 uarea->entry_hi = uarea->ientry_hi; /* Save a static copy for test. */
      }
#if ERROR_CHECKS > 0
      {
	 /* Debug patterns for register at - k1. */
	 U32 add = 0x01010101;
	 U32 val = add;
	 U32 *ptr = (U32 *)&uarea->iat.u64;
	 U32 i;

	 for(i = 1; i <= 9; i++)
	 {
	    *ptr++ = val;	/* High 32 bits. */
	    *ptr++ = val;	/* Low  32 bits. */
	    val += add;
	 }
	 val = 0x10101010;
	 for(i = 10; i <= 19; i++)
	 {
	    *ptr++ = val;	/* High 32 bits. */
	    *ptr++ = val;	/* Low  32 bits. */
	    val += add;
	 }
	 val = 0x20202020;
	 for(i = 20; i <= 27; i++)
	 {
	    *ptr++ = val;	/* High 32 bits. */
	    *ptr++ = val;	/* Low  32 bits. */
	    val += add;
	 }
      }
      set64(&uarea->ilo, 0x05e05e05);	/* The OSE pattern. */
      set64(&uarea->ihi, 0x05e05e05);
#endif
      /* Setup return address for processes that breakes their main loop. */
      if(clib_handle)
      {
	 /*
	 ** Get function pointer from clib pointer array.
	 ** This function will trap into kernel to a routine that will
	 ** report error and kill current process.
	 */
	 set64(&uarea->ira,
	       (U32)((struct clib_pointer *) clib_handle)->zzprocess_ended);
      }
      else
      {
	 /*
	 ** If clib_handle not available, point direct to entrypoint
	 ** in target interface.
	 */
	 set64(&uarea->ira, zzti.dead_process);
      }

      /* Align size to 16 byte. */
      stack_size &= ~0xf;

      if(super_mode)
      {
	 extern OSENTRYPOINT _zzmain;

	 /* The first _zzmain (monolit) has special entrypoint. */
	 if(entrypoint == _zzmain)
	 {
	    set64(&uarea->iepc, (U32) main_init);
	 }

	 uarea->isr = sr | SR_EXL;

	 /* Setup address to a variable with endmark pattern,
	    to make dummy stack test on. */
	 uarea->usp_base = (U32) &zzti.stack_pattern;

	 uarea->ssp_base = (U32) stack;
	 *(U32 *) stack = STACK_PATTERN;	/* Set end-mark. */

	 /* Reserve 16 bytes for standard stack frame. */
	 set64(&uarea->sp, (U32)stack + (U32)stack_size - 16);
	 set64(&uarea->isp, uarea->sp.u32.l);
	 set64(&uarea->ssp, uarea->sp.u32.l);	/* Setup only for zzssp. */
      }
      else /* user mode */
      {
	 /* Align size to 16 byte. */
	 U32 superv_stack_size = zzconf.superv_stacksize & ~0xf;

	 /* Set user mode: set UX, UM and EXL bits. */
	 uarea->isr = sr | SR_UX | SR_UM | SR_EXL;

	 /* Set supplied stack as user stack. */
	 uarea->usp_base = (U32) stack;
	 *(U8 *) stack = END_MARK;

	 /* Reserve 16 bytes for standard stack frame. */
	 set64(&uarea->isp, (U32)stack + (U32)stack_size - 16);

	 /* Allocate a supervisor stack from the free list or from
	    the pool if required. */
	 if(zzti.free_superv_stack)
	 {
	    uarea->ssp_base = zzti.free_superv_stack;
	    zzti.free_superv_stack = *(U32 *) uarea->ssp_base;
	 }
	 else
	 {
	    uarea->ssp_base = (U32)zzkrn_alloc_kernelstack(superv_stack_size);
	 }
	 /* Fill stack with endmark pattern, for debugger use. */
	 memset((void *)uarea->ssp_base, END_MARK, superv_stack_size);

	 set64(&uarea->sp, uarea->ssp_base + superv_stack_size);

	 /* Create stackframe for bios. */
	 set64(&uarea->ssp, uarea->sp.u32.l - BIOS_FRAME_SIZE);

      }
      assert((uarea->sp.u64  & 0xf) == 0); /* Minimum 16 byte align. */
      assert((uarea->isp.u64 & 0xf) == 0); /* Minimum 16 byte align. */
   }
   /* Call create handlers. */
   if(zzconf.create_handler_list)
   {
      U32 *config_array = (U32 *) zzconf.create_handler_list;
      void (*createh)(U32, PROCESS, PROCESS);

      while(*config_array)		/* Loop until all handlers called. */
      {
	 /* Get address of create handler. */
	 createh = (void (*)) *config_array;
	 (createh)(uarea->user_area, newpid, parent);	/* Call handler. */
	 config_array++;
      }
   }
   return uarea;
}

/*
**===========================================================================
** zzkill_context
**===========================================================================
** Description:	Kill an uarea and free it to special free list.
**		Call kill handlers if present.
**
**		Both zzcreate_context and zzkill_context are called from the
**		sysd process.
**		Because of this there is no need to disable interrupts when
**		free lists are accessed.
**
** Parameters:	uarea
**
** Returns:	-
**
** Globals:	free_uarea, free_superv_stack
*/

void zzkill_context(void *uareap, PROCESS oldpid)
{
   UAREA *uarea = (UAREA *) uareap;
   U32 *config_array;
   void (*killh)(U32, PROCESS);

    /* There is no longer a register dump connected to the process. */
    uarea->exception_regdump = NULL;

   /* Call kill handlers. */

   /* Get address of kill handler array. */
   config_array = (U32 *) zzconf.kill_handler_list;

   if(config_array)
   {
      while(*config_array)		/* Loop until all handlers called. */
      {
	 /* Get address of kill handler. */
	 killh = (void (*)) *config_array;
	 (killh)(uarea->user_area, oldpid);	/* Call handler. */
	 config_array++;
      }
   }

   /* If a user stack pointer is present, a supervisor stack has previously
      been allocated and need to be freed to free list. */
   if(!uarea->super_mode)
   {
      *(U32 *) uarea->ssp_base = zzti.free_superv_stack;
      zzti.free_superv_stack = uarea->ssp_base;
   }

   /* Free fp swap area if present. */
   if(uarea->fpu_reg_file)
   {
      free_uarea_block((UAREA *) uarea->fpu_reg_file);
   }

   /* Free the uarea to the local free list. */
   free_uarea_block(uarea);
}

/*
**===========================================================================
** zzset_manager_mode
**===========================================================================
** Description:	Enables or disables manager mode for the current process.
**
** Parameters:	enable
**
** Returns:	The previous manager mode state (enabled/disabled),
**              as it was before the operation took place.
**
*/
OSBOOLEAN
zzset_manager_mode(OSBOOLEAN enable)
{
   (void)enable;
   return FALSE;
}

/*
**===========================================================================
** zzget_manager_mode
**===========================================================================
** Description:	Obtains manager mode state for a process.
**              Primarily called from within zzkrn_get_pid_info.
**
** Parameters:	Uarea.
**
** Returns:	TRUE if manager mode is enabled for the process context,
**              FALSE otherwise.
**
** Globals:	-
*/
OSBOOLEAN
zzget_manager_mode(void *uareap)
{
   (void)uareap;
   return FALSE;
}

/*
**===========================================================================
** void zzcall_attach_block(PROCESS pid, OSSEGMENT segid)
**===========================================================================
** Description:
**
** Parameters:
**
** Returns:	-
**
** Globals:	-
*/

void zzcall_attach_block(PROCESS pid, OSSEGMENT segid)
{
   void (*handler)(PROCESS, OSSEGMENT);

   handler = (void (*)(PROCESS, OSSEGMENT)) zzconf.attach_block_handler;
   if(handler)
   {
      (handler)(pid, segid);
   }
}

/*
**===========================================================================
** OSBOOLEAN zzcall_mem_move(OSSEGMENT src_segment,
**                           OSADDRESS src_begin,
**                           OSADDRESS size,
**                           OSSEGMENT dest_segment,
**                           OSADDRESS dest_begin)
**===========================================================================
** Description:	Safe memcpy, detecting bus errors, used by run mode debuggers.
**
** Parameters:	src_segment,src_begin,size,dest_segment,dest_begin
**
** Returns:	zero for ok, nonzero for error.
**
** Globals:	-
*/

OSBOOLEAN zzcall_mem_move(OSSEGMENT src_segment,
			  OSADDRESS src_begin,
			  OSADDRESS size,
			  OSSEGMENT dest_segment,
			  OSADDRESS dest_begin)
{
   return mm_access_safecpy(dest_segment,
			    (void *)dest_begin,
			    src_segment,
			    (void *)src_begin,
			    size,
			    RUN_MODE);
}

/*
**===========================================================================
** int
** zzti_get_mem(PROCESS id, OSADDRESS from, void *to, OSADDRESS size,
**              OSADDRESS phase, OSADDRESS error_address, OSADDRESS sp)
**===========================================================================
** Description:	Wrapper function for get_mem.
**
** Parameters:
**
** Returns:     0 get_mem success
**              1 get_mem error
**              2 Perform memcpy in user mode by crt.
**
** Globals:	-
*/
int
zzti_get_mem(PROCESS id, OSADDRESS from, void *to, OSADDRESS size,
	     OSADDRESS phase, OSADDRESS error_address, OSADDRESS sp)
{
   int rc;
#if 0
   ramlog_printf("getm %#x  %#x  %#x  %#x  %#x  %#x  %#x\n",
		 phase,id,from,to,size,error_address,sp);
#endif
   if (phase == 2)
   {
      /* Finish memcpy from user mode. */
      UAREA *uarea = (UAREA *) zzcurrent_uarea;

      set64(&uarea->safecopy_sp, 0);
      return 0;
   }
   rc = zzkrn_pre_get_set_mem(id, OSE_GET_MEM);

   if (rc == 0)
   {
      return get_mem(id, from, to, size);
   }
   if (rc == 2)
   {
      /* Caller is an user mode process. */
      UAREA *uarea = (UAREA *) zzcurrent_uarea;

      set64(&uarea->safecopy_err, error_address);
      set64(&uarea->safecopy_sp, sp);
      return 2;
   }
   return 1;
}

int
zzti_set_mem(PROCESS id, void *from, OSADDRESS to, OSADDRESS size,
	     OSADDRESS phase, OSADDRESS error_address, OSADDRESS sp)
{
   int rc;
#if 0
   ramlog_printf("setm %d  %08x  %08x  %08x  %d  %08x  %08x\n",
		 phase,id,from,to,size,error_address,sp);
#endif
   if (phase == 2)
   {
      /* Finish memcpy from user mode. */
      UAREA *uarea = (UAREA *) zzcurrent_uarea;

      set64(&uarea->safecopy_sp, 0);
      return 0;
   }
   rc = zzkrn_pre_get_set_mem(id, OSE_SET_MEM);
   if (rc == 0)
   {
      return set_mem(id, from, to, size);
   }
   if (rc == 2)
   {
      /* Caller is an user mode process. */
      UAREA *uarea = (UAREA *) zzcurrent_uarea;

      set64(&uarea->safecopy_err, error_address);
      set64(&uarea->safecopy_sp, sp);
      return 2;
   }
   return 1;
}

/*
**===========================================================================
** void zzcall_select_segment(OSSEGMENT segnum)
**===========================================================================
** Description:
**
** Parameters:
**
** Returns:	-
**
** Globals:	-
*/

void zzcall_select_segment(OSSEGMENT segnum)
{
   void (*handler)(OSSEGMENT);

   handler = (void (*)(OSSEGMENT)) zzconf.select_segment_handler;
   if(handler)
   {
      (handler)(segnum);
   }
}

/*
**===========================================================================
** OSBOOLEAN zzcall_set_segment_mode(OSSEGMENT segid, OSBOOLEAN contains_int)
**===========================================================================
** Description:
**
** Parameters:
**
** Returns:	-
**
** Globals:	-
*/

OSBOOLEAN zzcall_set_segment_mode(OSSEGMENT segid, OSBOOLEAN contains_int)
{
   OSBOOLEAN (*handler)(OSSEGMENT, OSBOOLEAN);

   handler = (OSBOOLEAN (*)(OSSEGMENT, OSBOOLEAN))
      zzconf.set_segment_mode_handler;

   if(handler)
   {
      return (handler)(segid, contains_int);
   }
   else
   {
      return 0; /* ??? */
   }
}

/*
**===========================================================================
** zzget_filename
**===========================================================================
** Description:	Examine the file and line information for the specified
**		process. This function stores line number as an integer into
**		the lineno parameter, and the file name into the filename
**		parameter.
**		Lineno should be set to zero if file and line info is not
**		available.
**		The configurable constant MAX_FILE_NAMELEN contains the worst
**		case number of characters allowed. The filename must be
**		truncated if required.
**
** Parameters:	Uarea
**		Line number return variable.
**		Filename return variable.
**
** Returns:	-
**
** Globals:	-
*/

void zzget_filename(void *uarea, OSADDRESS *lineno, char *filename)
{
    UAREA *ua = (UAREA *) uarea;

    *lineno = ua->line;
    if(ua->line)	/* line is zero if file/line not available. */
    {
	int i = 0;
	char *dest = filename;
	char *src = ua->file;
	char c = *src;

	while(c)
	{
	    /* Truncate path and path characters. */
	    if(c == '/' || c == '\\')
	    {
		i = 0;
		dest = filename;
	    }
	    else if(i < MAX_FILE_NAMELEN - 1)
	    {
		*dest++ = c;
		i++;
	    }
	    src++;
	    c = *src;
	}
	*dest = '\0';
    }
}


/*
**===========================================================================
** zzget_file_line
**===========================================================================
** Examine the file and line information for the specified process.
** This function stores line number as an integer into the lineno parameter,
** and the file name pointer into the filename_pointer parameter.
** Lineno should be set to zero if file and line info is not available.
**
** The dispatcher is locked by the caller.
**
** This function is required only for paging systems, and only called
** when ZZGET_FILE_LINE is configured to non-zero.
*/

void zzget_file_line(void *uarea, OSADDRESS *lineno, char **filename_pointer)
{
    UAREA *ua = (UAREA *) uarea;

    *filename_pointer = ua->file;
    *lineno = ua->line;
}

/*
**===========================================================================
** zzget_regs
**===========================================================================
** Description:
**
** Reformat the binary register set for the specified process and return in
** cpuregs a string describing the registers.
** The cpuregs array contains the image previously stored by zzcapture_regs().
** The resulting ascii string should also be in cpuregs, which must be
** accounted for in the algorithm. It may for instance be wise to keep the
** binary image at the tail while building the string from the head of the
** buffer.
** The configurable constant MAX_REGLEN contains the worst case number
** of characters allowed.
** The dispatcher is unlocked while this function is called, which means
** that the function must be reentrant.
**
** If the USE_CAPTURE_REGS flag in pose.con is zero, then zzcapture_regs()
** is not called, and zzget_regs() is assumed to perform the combined
** task of these calls. The dispatcher is locked in this case.
**
**
** Parameters:	Uarea
**		Data area for register contents.
**
** Returns:	-
**
** Globals:	-
*/

void zzget_regs(void *uarea, char *cpuregs)
{
   char *dest = cpuregs;
   (void)uarea;

#if OSE_DEBUG
   /* Start address of binary data saved by capture_regs. */
   UAREA *src_uarea = (UAREA *) *(OSADDRESS *) cpuregs;
   int i, j;

   if(src_uarea->reg_state & REG_STATE_ALL)
   {
      /*
      ** All registers are available.
      */
      static const int sprlen[] = {3,3,3,3};
      static const int sprbinlen[] = {2,2,2,1};	/* Number of words. */
      static const char sprname[] = "PC=LO=HI=SR=";
      static const char loop_end[] = {'9','9','9','1'};/* Loop end values. */

      U32 *src_u32 = (U32 *) &src_uarea->izero;
      char c1, c10 = '0';
      int name_index;

      /* Format GPRs. */
      for(i = 0; i < 4; i++)
      {
	 for(c1 = '0'; c1 <= loop_end[i]; c1++)
	 {
	    *dest++ = 'R';
	    if(c10 != '0')	/* Skip leading zero in regname for R0-R9. */
	    {
	       *dest++ = c10;
	    }
	    *dest++ = c1;
	    *dest++ = '=';
	    /* Convert two 32-bit words for 64 bit register. */
	    long2str(dest, *src_u32);
	    dest += 8;
	    src_u32++;
	    long2str(dest, *src_u32);
	    dest += 8;
	    src_u32++;
	    *dest++ = ' ';
	 }
	 c10++;
      }

      /* Format special purpose registers. */
      name_index = 0;
      src_uarea->isr &= ~SR_EXL;		/* Clear EXL bit. */
      src_u32 = (U32 *) &src_uarea->iepc;
      for(i = 0; i < 4; i++)
      {
	 for(j = 0; j < sprlen[i]; j++)
	 {
	    *dest++ = sprname[name_index++];	/* Copy name. */
	 }
	 long2str(dest, *src_u32);
	 dest += 8;
	 src_u32++;
	 if(sprbinlen[i] == 2)
	 {
	    /* Another 32 bit word for 64 bit regs. */
	    long2str(dest, *src_u32);
	    dest += 8;
	    src_u32++;
	 }
	 *dest++ = ' ';
      }

      /* Format Floating Point */
      if(src_uarea->fpu_reg_file)
      {
	 static const int fcrlen[] = {5,6,6,6,6};
	 static const char fcrname[] = "FCR0=FCR25=FCR26=FCR28=FCR31=";
	 static const char f_loop_end[] = {'9','9','9','1'}; /* Loop limits. */
	 struct fpu *fpu = src_uarea->fpu_reg_file;
	 FPR *src_fpr = &fpu->fpr0;

	 /* Format floating point control registers. */
	 name_index = 0;
	 src_u32 = &fpu->fcr0;
	 for(i = 0; i < 5; i++)
	 {
	    for(j = 0; j < fcrlen[i]; j++)
	    {
	       *dest++ = fcrname[name_index++];	/* Copy name. */
	    }
	    long2str(dest, *src_u32);
	    dest += 8;
	    src_u32++;
	    *dest++ = ' ';
	 }

	 /* Format Floating point data registers */
	 c10 = '0';
	 for(i = 0; i < (int) sizeof(f_loop_end); i++)
	 {
	    for(c1 = '0'; c1 <= f_loop_end[i]; c1++)
	    {
	       *dest++ = 'F';
	       if(c10 != '0') /* Skip leading zero in regname for R0-R9. */
	       {
		  *dest++ = c10;
	       }
	       *dest++ = c1;
	       *dest++ = '=';
	       dest = fpr2str(dest, src_fpr++);
	       *dest++ = ' ';
	    }
	    c10++;
	 }
      } /* FPU reg file */
   }
   else
   {
      /*
      ** Only sp (frame pointer) and return address (RA) are available.
      */

      /* SP */
      strcpy(dest, "R29=");
      dest += 4;
      long2str(dest, src_uarea->isp.u32.h);
      dest += 8;
      long2str(dest, src_uarea->isp.u32.l);
      dest += 8;
      
      /* FP */
      strcpy(dest, " R30=");
      dest += 5;
      long2str(dest, src_uarea->ifp.u32.h);
      dest += 8;
      long2str(dest, src_uarea->ifp.u32.l);
      dest += 8;
      
      /* RA */
      strcpy(dest, " R31=");
      dest += 5;
      long2str(dest, src_uarea->ira.u32.h);
      dest += 8;
      long2str(dest, src_uarea->ira.u32.l);
      dest += 8;
      
      /* PC */
      strcpy(dest, " PC=");
      dest += 4;
      long2str(dest, src_uarea->iepc.u32.h);
      dest += 8;
      long2str(dest, src_uarea->iepc.u32.l);
      dest += 8;
   }
#endif
   *dest = '\0';		/* Terminate string or return null string. */
}

/*
**===========================================================================
** zzset_regs
**===========================================================================
** Description:
**
** Modify the current register set for the specified process if the process
** is in a state where this is possible.
** Return a nonzero value if the call failed.
** Cpuregs contains a binary image describing the registers to be set.
** The image was created by zzunpack_regs().
**
** The dispatcher is locked during this call.
**
** The zzunpack_regs() function is called only if the USE_CAPTURE_REGS flag
** in pose.con is non zero. If it is zero, then zzset_regs() is assumed to
** perform the combined function of zzunpack_regs() and zzset_regs().
** In this case, cpuregs contains the input ascii string.
**
**
** Parameters:	Uarea
**		cpuregs: Binary data to be copied into uarea.
**			 Layout: See struct unpacked_regs.
** Returns:	Nonzero if register can't be written to.
**
** Globals:	-
*/

OSBOOLEAN zzset_regs(void *uarea, const char *cpuregs)
{
   const struct unpacked_regs *regs = (const struct unpacked_regs *) cpuregs;
   UAREA *ua = uarea;
   U32 update_count = 0;

   /* Check if registers are avilable and writable. */
   if((ua->reg_state & (REG_STATE_ALL | REG_STATE_WRITABLE)) ==
      (REG_STATE_ALL | REG_STATE_WRITABLE))
   {
      /* Start address of binary data. */
      const U32 *src = &regs->ua.izero.u32.h;

      /* Destination address of binary data. */
      U32 *dest = &ua->izero.u32.h;

      U32 i;

      for(i = 0; i < NO_OF_UNPACKED_U32_IN_UAREA; i++)
      {
	 /* If flag is set then updata corresponding register in uarea. */
	 if(regs->update[i])
	 {
	   dest[i] = src[i];
	   update_count++;
	 }
      }
      if (ua->fpu_reg_file)
      {
	 U32 j;
	 dest = &ua->fpu_reg_file->fcr0;
	 src = &regs->fpu.fcr0;
	 for (j = 0; j < NO_OF_UNPACKED_U32_IN_FPU; j++)
	 {
	    if (regs->update[i + j])
	    {
	       dest[j] = src[j];
	       update_count++;
	    }
	 } /* for */
      }
      set64(&ua->izero, 0);		/* izero is readonly. */
      return regs->update_count - update_count;	/* Non-zero is internal error*/
   }

   return 1;			/* Registers not writable. */
}

/*
**===========================================================================
** zzget_register
**===========================================================================
** This function is called by efmi to read process specific registers.
** Interrupts are locked out by the caller. The provided uarea is
** an existing, prioritized or background process, and never the current
** process.
**
** This function shall check if the specified register is available.
** If so, store it in the value argument and return zero.
** If the register is not available, return nonzero.
**
** is_syspool_pointer is a function that should be used to validate
** system pool pointers that are FOLLOWED to extract the current register
** value. The function returns non-zero if the pointer is valid. The uarea
** pointer is already verified by the caller.
*/

OSBOOLEAN
zzget_register(void *uarea, U32 reg_id, U32 *value,
	       OSBOOLEAN (*is_syspool_pointer)(const void *vp))
{
   UAREA *ua = (UAREA *) uarea;

   /* Supervisor stack base is always available. */
   if(reg_id == ZZREG_SSLIMIT)
   {
      *value = ua->ssp_base;
      return 0;
   }

   /* Floating point */
   if(reg_id >= ZZREG_F0_LO && reg_id < ZZREG_F0_HI + 32)
   {
      FPR *fpr_arr;

      if(!ua->fpu_reg_file || !is_syspool_pointer(ua->fpu_reg_file))
      {
	 return 1;		/* No floating point available. */
      }

      fpr_arr = &ua->fpu_reg_file->fpr0;
      if(reg_id < ZZREG_F0_LO + 32) /* Least significant half */
      {
	 *value = fpr_arr[reg_id - ZZREG_F0_LO].lsw;
      }
      else			/* Most significant half */
      {
	 *value = fpr_arr[reg_id - ZZREG_F0_HI].msw;
      }
      return 0;
   }

   /* Check if executing in an error handler. */
   if (ua->err_lev_0_ra.u64)
   {
      switch(reg_id)
      {
	 case ZZREG_SP:
	    *value = ua->err_lev_0_sp.u32.l;
	    return 0;
	 case ZZREG_SP_HI:
	    *value = ua->err_lev_0_sp.u32.h;
	    return 0;
	 case ZZREG_RA:
	 case ZZREG_PC:
	    *value = ua->err_lev_0_ra.u32.l;
	    return 0;
	 case ZZREG_RA_HI:
	 case ZZREG_PC_HI:
	    *value = ua->err_lev_0_ra.u32.h;
	    return 0;
	 case ZZREG_FP:
	    *value = ua->err_lev_0_fp.u32.l;
	    return 0;
	 case ZZREG_FP_HI:
	    *value = ua->err_lev_0_fp.u32.h;
	    return 0;
      }
   }

  /* If a register dump exists, fetch registers from it. */
  if(ua->exception_regdump)
    {
      struct cpu_regdump *regdump =(struct cpu_regdump *) ua->exception_regdump;
      U32 *core = (U32 *)regdump->sect[0].address;
      U32 offset = zzget_regdump_core_offset(reg_id);
      if(offset == ~0UL)
	{
	  return 1;		/* FAIL: Bad reg_id */
	}
      *value = core[offset];
      return 0;
    }
  
   /* The rest is not valid for current process. */
   if (uarea == zzcurrent_uarea)
   {
      return 1;
   }

   /* If interrupted or debug_regs_on == true */
   if(ua->reg_state & REG_STATE_ALL)
   {
      /* GPR low word. */
      if(reg_id < ZZREG_R0 + 32)
      {
	 REG64 *gpr_arr = &ua->izero;

	 *value = gpr_arr[reg_id].u32.l;
	 return 0;
      }
      /* GPR high word. */
      if(reg_id >= ZZREG_R0_HI && reg_id <= ZZREG_RA_HI)
      {
	 REG64 *gpr_arr = &ua->izero;

	 reg_id -= ZZREG_R0_HI; /* reg_id == ZZREG_R0_HI -> 0 */
	 *value = gpr_arr[reg_id].u32.h;
	 return 0;
      }

      switch (reg_id)
      {
	 case ZZREG_SR:
	    *value = ua->isr;
	    return 0;
	 case ZZREG_HI:
	    *value = ua->ihi.u32.l;
	    return 0;
	 case ZZREG_HI_HI:
	    *value = ua->ihi.u32.h;
	    return 0;
	 case ZZREG_LO:
	    *value = ua->ilo.u32.l;
	    return 0;
	 case ZZREG_LO_HI:
	    *value = ua->ilo.u32.h;
	    return 0;
	 case ZZREG_PC:
	    *value = ua->iepc.u32.l;
	    return 0;
	 case ZZREG_PC_HI:
	    *value = ua->iepc.u32.h;
	    return 0;
      }
      return 1; /* Not available. */
   }

   /* Registers saved by bios or direct call from supervisor mode. */
   if (ua->call_ra.u64)
   {
      switch (reg_id)
      {
	 case ZZREG_SP:
	    *value = ua->call_sp.u32.l;
	    return 0;
	 case ZZREG_SP_HI:
	    *value = ua->call_sp.u32.h;
	    return 0;
	 case ZZREG_RA:
	 case ZZREG_PC:
	    *value = ua->call_ra.u32.l;
	    return 0;
	 case ZZREG_RA_HI:
	 case ZZREG_PC_HI:
	    *value = ua->call_ra.u32.h;
	    return 0;
	 case ZZREG_FP:
	    *value = ua->call_fp.u32.l;
	    return 0;
	 case ZZREG_FP_HI:
	    *value = ua->call_fp.u32.h;
	    return 0;
      }
   }
   return 1; /* Not available. */
}

/*
**===========================================================================
** struct sysctlblk *zzget_sysctlblk(void)
**===========================================================================
** Description:
** Get address to system control block.
**
** The system control block contains all runtime configurable system
** parameters.
** These parameters are made known to the kernel through this function.
**
** Returned values are:
**	Kernel pool base address, aligned for any type.
**	Kernel pool size.
**	List of kernel signal sizes.
**	List of kernel stack sizes.
**	Trace buffer size.
**
** Parameters:
**
** Returns:	-
**
** Globals:	-
*/

struct sysctlblk *zzget_sysctlblk(void)
{
   extern struct sysctlblk ZZT0102;

   return &ZZT0102;	/* Return struct from krncon.c. */
}

/*
**===========================================================================
** Handle breakpoint
**===========================================================================
** Description:
**
** Parameters:
**
** Returns:	-
**
** Globals:	-
*/
#ifdef SET_BP_SUPPORT
U32 zzhandle_breakpoint(struct intframe *intframe, OSPRIORITY int_level,
			OSPRIORITY oldint_level)
{
   /* Need to store the bp block while tracing.
    * These varibles are placed in BSS unecassry to explicit init to zero.
    */
   static struct bp *saved_bb;
   static OSBOOLEAN saved_action_code;
   static OSADDRESS saved_instruction_trace;
   static U32 saved_ie;
   static OSBOOLEAN trace_active;
   static OSBOOLEAN restore_bp_flag;

   OSBOOLEAN action_code;
   U32    result;
   struct memcpy_arg arg = {NULL,NULL,4};

   /* The zero register is not initiated in intframe, do it here. */
   intframe->zero = 0;

   if(trace_active)  /*lint !e727 */
   {
      trace_active = 0;
      /* Restore IE bit after trace.*/
      intframe->sr |= saved_ie;	 /*lint !e727 */

      /* Sanity check, address must be the same as the EPC */
      if(!saved_bb || saved_bb->address_post != intframe->epc) /*lint !e727 */
      {
	 (void)zzos_error(FATAL_ERROR, 0x1800000 | OSE_EUNEXPECTED_EXCEPTION,
			  intframe->epc);
      }

      /* Remove temporary trace breakpoint and restore saved instruction. */
      arg.dest = (void *)(saved_bb->address_post);
      arg.src = (const void *)&saved_instruction_trace;


      result = mm_update_code_forced((U32)arg.dest, arg.size,
				     (func_t)memcpy_call_back,
				     (void *) &arg);
      if (result != MM_SUCCESS)
      {
	 update_code_forced_failed((OSADDRESS)arg.dest);
      }

      action_code = saved_action_code; /*lint !e727 */

      /* Restore old address breakpoint if needed.
       * I.e. the bp we traced over.
       */
      if(restore_bp_flag)  /*lint !e727 */
      {
	 restore_bp_flag = 0;
	 arg.dest = (void *)(saved_bb->address);
	 arg.src  = (const void *)&trap;

	 /* Restore bp. */
	 result = mm_update_code_forced((U32)arg.dest, arg.size,
					(func_t)memcpy_call_back,
					(void *) &arg);
	 if (result != MM_SUCCESS)
	 {
	    update_code_forced_failed((OSADDRESS)arg.dest);
	 }
      }

      /* Zero the saved_bb to be able to do a sanity check. */
      saved_bb = 0;
   }
   else  /* no trace */
   {
      struct bp *bb, *bb2;
      OSADDRESS address = intframe->epc;	/* Get encountered bp address*/

      bb = get_bp_block(address, zzcurrent_uarea, 1); /* Search, uarea valid.*/
      if(!bb)
      {
	 bb = get_bp_block(address, 0, 0);	/* Search, uarea not valid. */
	 if(!bb)
	 {
	    /* No bp found on current address. */
	    (void)zzos_error(FATAL_ERROR,
			     0x1800000 | OSE_EUNEXPECTED_EXCEPTION,
			     address);
	 }
      }

      action_code = zzbreakpoint_trap(int_level, oldint_level, /*lint !e715 */
				      bb->breakpoint_id);
      if((action_code & 4) || bb->attribute == ATTR_POST)
      {
	 /* Invalid process or post bp, step over. */

	 /* If the instruction is operating on status register,
	  * the instruction must be simulated. I.e. no trace needed.
	  */
	 if(bb->type_post == INST_MTC0_STATUS_REG ||
	    bb->type_post == INST_MFC0_STATUS_REG)
	 {
	    U32 rt;

	    /* Simulate the MFC0/MTC0 instruction. */
	    rt = RT(bb->saved_instruction);

	    if (rt != 0)
	    {
	       U32 *gp_reg;

	       gp_reg = GET_INTFRAME_REG(intframe, rt);

	       if(bb->type_post == INST_MTC0_STATUS_REG)
	       {
		  intframe->sr = *gp_reg;
	       }
	       else
	       {
		  *gp_reg = intframe->sr;
	       }
	    }
	    else
	    {
	       /* No action needed for MFCO to zero register. */
	       if(bb->type_post == INST_MTC0_STATUS_REG)
	       {
		  intframe->sr = 0;
	       }
	    }
	    intframe->epc += 4;
	 }
	 else   /* Not a status register operation, trace needed. */
	 {
	    trace_active = 1;
	     /* Save current bb, pick up after trace. */
	    saved_bb = bb;

	    /* Save for use after trace. */
	    saved_action_code = action_code = action_code & 3;

	    zzget_trace_address(bb->type_post,
				bb->saved_instruction,
				address,
				intframe,
				&bb->address_post);

	    saved_ie = intframe->sr & 0x1;                /* Save ie bit.  */
	    intframe->sr = intframe->sr & 0xfffffffe;	  /* Clear ie bit. */

	    /* Check if any breakpoint still present on current address. */
	    bb2 = get_bp_block(address, 0, 0);
	    if(bb2)
	    {
	       /* Remember to restore bp after trace.*/
	       restore_bp_flag = 1;

	       /* Restore original instruction. */
	       arg.dest = (void *)address;
	       arg.src  = (const void *)&(bb->saved_instruction);
	       result = mm_update_code_forced((U32)arg.dest, arg.size,
					(func_t)memcpy_call_back,
					(void *) &arg);
	       if (result != MM_SUCCESS)
	       {
		  update_code_forced_failed((OSADDRESS)arg.dest);
	       }
	    }
	    /* Save instruction on trace address. */
	    saved_instruction_trace = *(OSADDRESS *) bb->address_post;
	    /* Set temporary trace breakpoint. */
	    arg.dest = (void *)bb->address_post;
	    arg.src  = (const void *)&trap;
	    result = mm_update_code_forced((U32)arg.dest, arg.size,
					   (func_t)memcpy_call_back,
					   (void *) &arg);
	    if (result != MM_SUCCESS)
	    {
	       update_code_forced_failed((OSADDRESS)arg.dest);
	    }
	    action_code = 0;			/* Restart instruction. */

	 } /* if(bb->type_post == INST_MTC0_STATUS_REG .... */

      } /* if((action_code & 4) || bb->attribute == ATTR_POST) */

   }
   return action_code;
}
#endif
/*
**===========================================================================
** OSBOOLEAN zzset_breakpoint(OSADDRESS address, OSADDRESS attribute,
**			      void *breakpoint_id, void *uarea)
**===========================================================================
** Description:
** Set a breakpoint in the currently selected memory segment.
**
** Interrupts are enabled while this function is called, but they may
** be temporarily disabled when required.
**
** The uarea parameter points to the uarea of the process in which the
** breakpoint was set, or it contains NULL if the breakpoint was set
** for several processes. The intention is that breakpoint hardware
** contents can be part of each process context when possible.
**
** Return a nonzero value if the breakpoint could not be set.
**
** Parameters:
**
** Returns:	-
**
** Globals:	-
*/

OSBOOLEAN zzset_breakpoint(OSADDRESS address, OSADDRESS attribute,
			   void *breakpoint_id, void *uarea)
{
#ifdef SET_BP_SUPPORT
   extern void zzsystem_callh(void);
   struct bp *bb, *tmp_bb;
   U32 address_post, type_post;
   OSADDRESS pre_address =  address - 4;

   /* Inhibit stepping into the kernel system calls. */
   if ((attribute & ATTR_POST) && (address == (OSADDRESS) zzsystem_callh))
     {
       UAREA *ua = (UAREA *) uarea;

       /* If the process is halted here, use saved RA as next instruction.
        * Else refuse a post bp on this location.
        */
       if (!(ua->reg_state & REG_STATE_ALL) || (ua->iepc != address))
	 {
	   return 1;		/* Post bp on other location than current PC */
	 }

       attribute = ATTR_PRE;
       address = ua->ira;	/* return point from system call */
     }

   /* check if preceeding instruction is a branch */
   type_post = zzpreprocess_trace(*(OSADDRESS *)pre_address, pre_address,
				&address_post);

   if (BR(type_post) || BC(type_post))
   {
      /* It is not allowed with bp request on instruction executing in
       * branch delay slot.
       */
      return 1;
   }

   type_post = zzpreprocess_trace(*(OSADDRESS *)address, address,
				  &address_post);
   if(type_post == INST_ILLEGAL)
   {
      return 1;
   }
   if(type_post == INST_SYSCALL)
   {
      if(attribute & ATTR_POST)
      {
	 /* Change post bp on sc instruction to pre bp on next instruction.
	  * It is not possible to trace atomic over a system call.
	  */
	 attribute = ATTR_PRE;
	 address = address + 4;
	 /* Recalculate trace parameters. */
	 type_post = zzpreprocess_trace(*(OSADDRESS *)address,
					address, &address_post);
	 if(type_post == INST_SYSCALL)
	 {
	    /* We can't handle tracing over two consecutive sc's. */
	    return 1;
	 }
      }
      else
      {
	 /* We can't set pre bp's on sc's. Not possible to trace. A trace is
	  * needed if the bp is hit by wrong process.
	  */
	 return 1;
      }
   }
   /* Search for existing bp with same address. */
   tmp_bb = search_bp(address, uarea, 0);
   if (tmp_bb)
   {
      assert(type_post == INST_TEQI);

      if (search_bp(address, uarea, 1))
      {
	 return 1;	/* Error: Duplicate bp for address and uarea */
      }
   }

   /* Get a new bp block. */
   if(bp_free_list)
   {
      /* From free list. */
      bb = bp_free_list;
      bp_free_list = bb->next;
   }
   else
   {
      /* Break new memory. */
      bb = (struct bp *) zzalloc_kernelmemory(sizeof(struct bp));
   }

   /* Init new block. */
   bb->next = 0;
   bb->local_next = 0;
   bb->address = address;
   bb->attribute = attribute & 1;	/* Save pre/post bit only. */
   bb->breakpoint_id = breakpoint_id;
   bb->uarea = uarea;

   bb->address_post = address_post;
   bb->type_post = type_post;

   /* Link new block into list. */
   if(tmp_bb)
   {
      /* Copy from other block.
       * - original saved instruction
       * - original post address
       * - original type code
       */
      bb->saved_instruction = tmp_bb->saved_instruction;
      bb->address_post = tmp_bb->address_post;
      bb->type_post = tmp_bb->type_post;

      /* Link into local list. tmp_bb is a local root. */
      bb->local_next = tmp_bb->local_next;
      tmp_bb->local_next = bb;
   }
   else
   {
      U32 result;
      struct memcpy_arg arg;

      /* Link to head of main list. */
      bb->next = bp_list;
      bp_list = bb;

      /* Save instruction, post address and type code. */
      bb->saved_instruction = *(OSADDRESS *) address;
      bb->address_post = address_post;
      bb->type_post = type_post;

      /* Install breakpoint. */
      arg.dest = (void *)address;
      arg.src  = (const void *)&trap;
      arg.size = 4;
      result = mm_update_code_forced((U32)arg.dest, arg.size,
				     (func_t)memcpy_call_back,
				     (void *) &arg);
      if (result != MM_SUCCESS)
      {
	 update_code_forced_failed((OSADDRESS)arg.dest);
      }
   }

   /* Call SET_BP_HANDLER if present. */
   if(zzconf.set_bp_handler)
   {
      void (*handler)(OSADDRESS, OSADDRESS, OSADDRESS);
      OSADDRESS new_data = trap;	/* bp opcode */
      OSADDRESS old_data;

      /* Set old data to bp opcode if bp already exists on current
	 address. */
      old_data = tmp_bb ? new_data : bb->saved_instruction;

      handler = (void (*)(OSADDRESS,OSADDRESS,OSADDRESS))
	 zzconf.set_bp_handler;
      (handler)(address, old_data, new_data);
   }

   return 0;	/* Return ok. */
#else
   (void)address, (void)attribute, (void)breakpoint_id, (void)uarea;
   return 1;	/* Return error always. */
#endif
}

/*
**===========================================================================
** OSBOOLEAN zzclear_breakpoint(void *breakpoint_id, void *uarea)
**===========================================================================
** Description:
** Clear a breakpoint in the currently selected memory segment.
**
** Interrupts are disabled while this function is called.
**
** The uarea parameter points to the uarea of the process in which the
** breakpoint was set, or it contains NULL if the breakpoint was set
** for several processes.
**
** return a nonzero value if the breakpoint is unknown.
**
** Parameters:
**
** Returns:	-
**
** Globals:	-
*/

OSBOOLEAN zzclear_breakpoint(void *breakpoint_id, void *uarea)
{
#ifdef SET_BP_SUPPORT
   struct bp *bb = bp_list, *prev_root = 0, *prev_local = 0, *this_root;
   OSBOOLEAN found = 0, instruction_restored = 0;

   (void) uarea;	/* Kill compiler warning. */

   while(!found && bb)
   {
      this_root = bb;
      prev_local = 0;
      if(bb->breakpoint_id == breakpoint_id)
      {
	 found = 1;
      }
      else
      {
	 while(!found && bb->local_next)
	 {
	    /* Search local list. */
	    prev_local = bb;
	    bb = bb->local_next;

	    if(bb->breakpoint_id == breakpoint_id)
	    {
	       found = 1;
	    }
	 } /* while */
      }
      if(!found)
      {
	 prev_root = this_root;
	 bb = prev_root->next;
      }
   } /* while */

   if(!found)
   {
      return 1;	/* Return error. */
   }

/* HERE:
 * The bp was found (as really was expected). Variables:
 * bb:		The bp to clear
 * prev_root:	The previous root of a local list for bp on one same address
 *		or NULL iff bp is in the very first address list.
 * prev_local:	The previous bp in the list for one single address, or NULL
 *		when bb is the root of this list
 */

   /* Restore instruction only if this the last bp on current address. */
   if(prev_local == 0 && bb->local_next == 0)
   {
      U32 result;
      struct memcpy_arg arg;

      arg.dest = (void *)(bb->address);
      arg.src = (const void *)&(bb->saved_instruction);
      arg.size = 4;
      result = mm_update_code_forced((U32)arg.dest, arg.size,
				     (func_t)memcpy_call_back,
				     (void *) &arg);
      if (result != MM_SUCCESS)
      {
	 update_code_forced_failed((OSADDRESS)arg.dest);
      }
      instruction_restored = 1;
   }

   /* Unlink block  */
   if (prev_local)
   {				/* Local entry */
      prev_local->local_next = bb->local_next;
   }
   else if (prev_root)		/* Root entry but not first */
   {
      if (bb->local_next)
      {			/* Has sublist */
	 prev_root->next = bb->local_next;
	 bb->local_next->next = bb->next;
      }
      else
      {			/* Single entry */
	 prev_root->next = bb->next;
      }
   }
   else				/* First entry in main list. */
   {
      if(bb->local_next)
      {			/* Has sublist */
	 bp_list = bb->local_next;
	 bb->local_next->next = bb->next;
      }
      else
      {			/* Local list empty. */
	 bp_list = bb->next;
      }
   }

   /* Call CLEAR_BP_HANDLER if present. */
   if(zzconf.clear_bp_handler)
   {
      void (*handler)(OSADDRESS, OSADDRESS, OSADDRESS);
      OSADDRESS old_data = trap;	/* bp opcode */
      OSADDRESS new_data;

      new_data = instruction_restored ? bb->saved_instruction : old_data;
      handler = (void (*)(OSADDRESS,OSADDRESS,OSADDRESS))
	 zzconf.clear_bp_handler;
      (handler)(bb->address, old_data, new_data);
   }

   /* Link bp block to free list. */
   bb->next = bp_free_list;
   bp_free_list = bb;

   return 0;	/* Return ok. */
#else
   (void)breakpoint_id, (void)uarea;
   return 1;	/* Return error always. */
#endif
}

/*
**===========================================================================
** OSADDRESS zzcall_system_errh(OSBOOLEAN user_called,
**                              OSERRCODE ecode, OSERRCODE extra)
**===========================================================================
** Description:
** Call the system error handler as a subroutine in supervisor space from
** supervisor space, using the current supervisor stack.
** The stack is the supervisor stack of the current process.
** Stack requirement of the system error handler must be
** reserved on each supervisor stack.
** Must save registers required by the kernel compiler.
**
** Parameters:
**
** Returns:	-
**
** Globals:	-
*/

OSADDRESS zzcall_system_errh(OSBOOLEAN user_called,
			     OSERRCODE ecode, OSERRCODE extra)
{
   OSADDRESS (*handler)(OSBOOLEAN, OSERRCODE, OSERRCODE);

   handler = (OSADDRESS (*)(OSBOOLEAN, OSERRCODE, OSERRCODE))
      zzconf.err_handler;

   if(handler)
   {
      return (handler)(user_called, ecode, extra);
   }
   else
   {
      return 1;			/* Error not handled if handler not present. */
   }
}

/*
**===========================================================================
** zzget_cpu_string
**===========================================================================
** Description:	Return a pointer to a string describing the current CPU.
**		This function is used in the get_cpu() system call.
**		It is assumed that this string is a constant that can be
**		concurrently accessed by several processes.
**
** Parameters:	-
**
** Returns:	Cpu string.
**
** Globals:	-
*/

const char *zzget_cpu_string(void)
{
   static const char cpu_string[]="OSE Delta MIPS32/" STRSTR(BL_LABEL);
   static const char cpuid[] = "/cpuid=00000000";
   static char cpu_string_id[ 8 + (sizeof cpuid)+ (sizeof cpu_string)];

   if (! zzti.cpu_string)
   {
      /* This code must be reentrant. */
      (void) strcpy(cpu_string_id, cpu_string);
      (void) strcpy(&cpu_string_id[(sizeof cpu_string) - 1], cpuid);
      long2str(&cpu_string_id[(sizeof cpuid)+ (sizeof cpu_string) - 2 - 8],
	       get32_cp0(C0_PRID));
      zzti.cpu_string = cpu_string_id;
   }
   return zzti.cpu_string;
}

/*
**===========================================================================
** U32 zzget_microticks(void)
**===========================================================================
** Description:
** Read the microsecond clock.
**
** The microsecond clock should be syncronous to the system clock such
** that the system tick is directly derived from the microsecond clock.
** The microsecond clock should NOT be automatically cleared by harware
** when a system tick interrupt is generated. It must continue to run
** until updated with a zzclear_microticks() call from the kernel.
** The microsecond clock is best implemented as a free running counter
** with a 1 MHz clock source, combined with an offset variable in RAM.
**
** If no clock source is available, the zzget_microticks() function
** must return a new value each time it is called, that is it should
** then be implemented as a variable incremented with each read operation.
**
** Interrupts are disabled during the call to zzget_microticks().
**
** Parameters:
**
** Returns:	-
**
** Globals:	-
*/

U32 zzget_microticks(void)
{
   U32 (*func)(void);

   func = (U32 (*)(void)) zzti.get_microticks; /* Get function pointer. */
   return (func)();
}

/* This function is used if no hw implementation is present. */
static U32 get_microticks_soft(void)
{
   return zzti.softtick++;
}

/*
**===========================================================================
** void zzclear_microticks(void)
**===========================================================================
** Description:
** Clear the microsecond clock.
**
** There are several ways to implement this function. Which to choose depends
** on the design of the microsecond clock hardware.
**
** If the clock is implemented as a free running counter with an output
** compare register that can cause interrupts then set the offset variable
** in ram to the old value of the output compare register and add one
** system tick period to the output compare register. Note the problem that
** occurs if the timer interrupt execution time exceeds the system tick
** length. If this happens, a system tick must be dropped to regain stability.
** This implementation yields precise system ticks and a microsecond clock
** that is always consistent with the current system tick counter in the kernel.
** It is the best implementation.
**
** If the clock is implemented as a free running counter that is not related to
** the system clock timer, then set the offset variable in ram to the current
** value of the free running counter. This implementation causes temporary
** inconsistencies between the microsecond clock and the system tick counter
** in the kernel. It appears that the time is sometimes moving backwards.
** This implementation is less desirable, but it works.
**
** If the clock is implemented as a timer that stops counting and restarts
** automatically or by software upon each interrupt then just allow it to
** restart. This implementation too causes periodic timer inconsistencies.
**
** Interrupts are disabled during the call to zzclear_microticks().
**
** Parameters:
**
** Returns:	-
**
** Globals:	-
*/

void zzclear_microticks(void)
{
   void (*func)(void);

   func = (void (*)(void)) zzti.clear_microticks;/* Get function pointer. */
   (func)();
}

/* This function is used if no hw implementation is present. */
static void clear_microticks_soft(void)
{
   zzti.softtick = 0;
}

/*
**===========================================================================
** zzidle
**===========================================================================
** Description:
**
** Idle loop check routine.
** This function is called by the dispatcher when no process is ready.
** It should return whenever an interrupt has occurred.
**
** Interrupts are disabled when this function is entered.
** This is to allow for flag operations within the target interface.
** Interrupts should be reenabled before waiting for an interrupt.
** (The kernel performs an extra UNLOCK() operation in case the
**  target interface doesn't.)
**
** A number of implementations are possible:
**
** A) Do nothing, just return. This is a typical implementation.
**
** B) Soft kernels with polled interrupt detection need to UNLOCK() and
**    scan for interrupts. Return after the first interrupt encountered.
**
** C) On processors with a "wait for interrupt" instruction we need to:
**    1. Set an internal "waiting for int" flag.
**    2. UNLOCK().
**    3. Execute the "wait for interrupt" instruction.
**    4. Clear the internal "waiting for int" flag and return.
**    We must also test on the "waiting for int" flag after each call
**    to zzinit_int that returns 0.
**    If the flag is set, we must clean up the stack as if a
**    normal return from interrupt was made, but we must then jump
**    to step 4 above instead of returning to user code.
**    This procedure fixes the race condition that would otherwise
**    cause the system to miss a process switch when an interrupt
**    occurs immediately before the "wait for interrupt" instruction
**    is executed.
**
**
** Parameters:	-
**
** Returns:	-
**
** Globals:	-
*/
#if 0
void zzidle(void)
{
}
#endif
/*
**===========================================================================
** zzget_sysprocnames
**===========================================================================
** Description:
**
** The system debugger reads the names of all system processes.
**
** This function is called by the kernel or the ldm during system start
** to obtain a list of the names of all processes that must be present
** for the debugger to operate correctly.
** These processes will never be stopped by the debugger.
**
** Typical system processes are:
**	Linkhandlers used for debugging.
**	The debugger process itself.
**
** The names are typically extracted from a user modified configuration
** file.
**
**
** Parameters:	The names pointer is updated to contain a pointer to a char
**		array. This char array contains packed NUL terminated strings.
**
** Returns:	Returns the number of process name strings stored in the
**		names pointer.
**
** Globals:	ZZT0101 (config array)
*/

U32 zzget_sysprocnames(char **names)
{
   U32 count = zzconf.sysproc_count; /* Read sysproc count. */

   if(count)
   {
      *names = (char *) zzconf.sysprocs; /* Get char array. */
   }
   return count;
}

/*
**===========================================================================
** OSBOOLEAN zzget_scope_status(void)
**===========================================================================
** Description:
** The system debugger reads the initial status of the breakpoint scope.
**
** This function is called by the kernel or the ldm during cold system start
** to obtain the expected status of non system processes.
** Scope status is typically provided by a user handler configured into
** the target interface. The handler reads a hardware strap or a memory
** flag managed by the handler.
**
** Returns 0 if application processes should be allowed to run.
** Returns non zero if application processes should be immediately intercepted
** when they are created, allowing the system debugger to examine the
** system before it runs.
**
** Parameters:
**
** Returns:	-
**
** Globals:	-
*/

OSBOOLEAN zzget_scope_status(void)
{
   OSBOOLEAN (*handler)(void);

   handler = (OSBOOLEAN (*)(void)) zzconf.power_off_handler;
   if(handler)
   {
      return (handler)();
   }
   else
   {
      return 0;
   }
}

/*
**===========================================================================
** zzcapture_regs
**===========================================================================
** Description:
**
** Examine the current register set for the specified process and return in
** cpuregs a binary image of the registers along with flags indicating the
** current register availability.
** The cpuregs image is later passed to zzget_regs() for reformatting.
** The configurable constant MAX_REGLEN contains the size of the
** cpuregs area.
** The dispatcher is locked while this function is called.
**
**
** Parameters:	Uarea
**		Data area to return register contents in.
**
** Returns:	-
**
** Globals:	-
*/

void zzcapture_regs(void *uarea, char *cpuregs)
{
   /* Write binary data at end of cpuregs area.
    * It holds one struct UAREA followed by a struct fpu.
    * MAX_REGLEN should be a multiple of 4.
    */
   OSADDRESS tmp = (OSADDRESS)
      (cpuregs + MAX_REGLEN - sizeof(UAREA) - sizeof(struct fpu));
   UAREA *dest = (UAREA *) (tmp & ~7); /* Align pointer to 64 bit. */
#if OSE_DEBUG
   UAREA *ua = (UAREA *) uarea;

   dest->magic = UAREA_MAGIC;

   /* Save pointer to struct to be picked up by get_regs. */
   *(U32 *) cpuregs = (U32) dest;

   /*
    * Test if registers are available, ie the process has
    * a) been preemted by an interrupt or a bp, or
    * b) swapped out itself when save_debug_regs was turned on.
    */
   if(ua->reg_state & REG_STATE_ALL)
   {
      /* Copy 32-bit regs from interrupt region in uarea to
	 end part of cpuregs. */
      memcpy(&dest->izero, &ua->izero,
	     offsetof(UAREA, isr) - offsetof(UAREA, izero) +
	     sizeof(ua->isr));

      /* Indicate to zzget_regs that all registers are available. */
      dest->reg_state = REG_STATE_ALL;

      /* Save floating point data too. */
      if (ua->fpu_reg_file)
	{
	  dest->fpu_reg_file = (struct fpu *) &dest[1];
	  memcpy(&dest->fpu_reg_file->fcr0, &ua->fpu_reg_file->fcr0,
		 offsetof(struct fpu, fpr31) - offsetof(struct fpu, fcr0) +
		 sizeof(FPR));
	}
      else
	{
	  dest->fpu_reg_file = NULL;
	} /* FPU */

   } /* REG_STATE_ALL */
   else
   {
      /* The process has made a system call without extended register save. */
      dest->isp = ua->call_sp;
      dest->ifp = ua->call_fp;
      dest->ira = ua->call_ra;
      dest->iepc = ua->call_ra;

      /* Indicate to zzget_regs that only sp, fp and RA
	 are available. */
      dest->reg_state = REG_STATE_SUBSET;
   }
#else
   (void)uarea;
   dest->reg_state = REG_STATE_NA;    /* Regs not available. */
#endif
}

/*
**===========================================================================
** zzunpack_regs
**===========================================================================
** Description:
**
** Unpack regstring containing register names and values to a binary format
** recognized by the zzset_regs() function.
** The string format corresponds to the format used in zzget_regs, except
** registers may be in any order and not all registers need be present.
** The result is stored in the cpuregs array, which contains MAX_REGLEN bytes.
** The cpuregs array is subsequently passed without modification to
** zzset_regs().
**
** The dispatcher is not locked during this call.
**
** The zzunpack_regs() function is called only if the USE_CAPTURE_REGS flag
** in pose.con is non zero. If it is zero, then zzset_regs() is assumed to
** perform the combined function of zzunpack_regs() and zzset_regs().
** In this case, cpuregs contains the input ascii string.
**
**
** Parameters:	cpuregs array that is used to return binary register data in.
**		regstring array that contains register values in ascii.
**
** Returns:	Return a nonzero value if a syntax error was encountered in the
**		input (regstring).
**
** Globals:	-
*/

OSBOOLEAN zzunpack_regs(char *cpuregs, const char *regstring)
{
   struct unpacked_regs *regs = (struct unpacked_regs *)cpuregs;
   const char *p, *head;
   U32 syntax_error = 0;
   U32 *location = NULL;
   U32 regnum;
   U32 doubleword;
   U32 update_index;

   /* Mark that no register values stored yet. */
   memset(regs->update, 0, sizeof(regs->update));
   regs->update_count = 0;

   /* Handle each register */
   p = regstring;
   while (*p)
   {
      /* Skip any spaces and start all over again.*/
      if (' ' == *p)
      {
	 p++;
	 continue;
      }

      /* Decide what register */
      head = p;
      doubleword = 0;

      if ('0' <= p[2] && p[2] <= '9') /* Fnn, Rnn */
      {
	 regnum = p[2] - '0' + (p[1] - '0') * 10;
	 if (*head == 'F')	/* Fnn */
	 {
	    doubleword = 1;
	    location = &regs->fpu.fpr0.msw + regnum * 2;
	 }
	 else			/* Rnn */
	 {
	    doubleword = 1;
	    location = &regs->ua.izero.u32.h + regnum * 2;
	 }
	 p += 3;
      }
      else
      {
	 switch (p[2])
	 {
	    case '=':		/* Fn, Rn, PC, SR, LO, HI */
	       if ('0' <= p[1] && p[1] <= '9') /* Fn, Rn */
	       {
		  regnum = p[1] - '0';
		  if (*head == 'F')	/* Fn */
		  {
		     doubleword = 1;
		     location = &regs->fpu.fpr0.msw + regnum * 2;
		  }
		  else			/* Rn */
		  {
		     doubleword = 1;
		     location = &regs->ua.izero.u32.h + regnum * 2;
		  }
	       }
	       else
	       {
		  switch (*head)
		  {
		     case 'P':	/* PC */
			doubleword = 1;
			location = &regs->ua.iepc.u32.h;
			break;

		     case 'S':	/* SR */
			location = &regs->ua.isr;
			break;

		     case 'L':	/* LO */
			doubleword = 1;
			location = &regs->ua.ilo.u32.h;
			break;

		     case 'H':	/* HI */
			doubleword = 1;
			location = &regs->ua.ihi.u32.h;
			break;

		     default:
			syntax_error = 1; /* syntax error */
			break;
		  } /* switch */
	       }
	       p += 2;
	       break;

	    case 'R':		/* FCR0, FCRnn */
	       switch (p[4])
	       {
		  case '5':		/* FCR25 */
		     location = &regs->fpu.fcr25;
		     break;

		  case '6':		/* FCR26 */
		     location = &regs->fpu.fcr26;
		     break;

		  case '8':		/* FCR28 */
		     location = &regs->fpu.fcr28;
		     break;

		  case '1':		/* FCR31 */
		     location = &regs->fpu.fcr31;
		     break;

		  default:		/* FCR0 is readonly. Syntax error */
		     syntax_error = 1;
		     break;
	       } /* switch */
	       p += 5;
	       break;

	    default:		/* Syntax error */
	       syntax_error = 1;
	       break;
	 } /* switch */
      } /* if */

      if (syntax_error || *p != '=')
      {
	 return 1;
      }

      /* Update that register */
      p++;			/* skip '=' */
      /* Decide update flag index. Pointer subtraction produces proper index.*/
      update_index = (location < &regs->fpu.fcr0)
	 ? location - &regs->ua.izero.u32.h
	 : location - &regs->fpu.fcr0 + (int) NO_OF_UNPACKED_U32_IN_UAREA;
      location[0] = str2long(p);
      regs->update_count++;
      regs->update[update_index] = 1;
      p += 8;
      if (doubleword)
      {
	 location[1] = str2long(p);
	 regs->update_count++;
	 regs->update[update_index + 1] = 1;
	 p += 8;
      }

      /* Prepare for next register, if any. */
      if (*p != ' ' && *p != 0)
      {
	 return 1;
      }
   } /* while */
   return syntax_error;
}

/*
**===========================================================================
** void zzset_inthook_status(U32 enable_flag)
**===========================================================================
** Description:
** The system debugger enables and disables interrupt process swap hooks.
** The call is needed only when using the LATE_INIT_INT implementation style.
**
** This call is issued whenever cpu load profiling is enabled or disabled
** in the system debugger.
**
** The enable_flag parameter should be copied to local storage in the target
** interface, and used to enable subsequent calls to zzintswap_in() and
** zzintswap_out() whenever it is nonzero.
**
** The zzset_inthook_status() function is called only if the
** ENABLE_INTERRUPT_PROFILING flag in pose.con is non zero.
** Otherwise, the function need not be implemented.
**
** Parameters:	enable_flag
**
** Returns:	-
**
** Globals:	inthook_flag
*/

void zzset_inthook_status(U32 enable_flag)
{
   zzti.inthook_flag = enable_flag ? 1: 0;
}

/*
** Run the power save handler which may shut down the system for
** the stated maximum time.
** If max_time is -1 the power save handler may sleep for as long
** as it wants.
** Returns the actual time the system was sleeping.
** Returns immediately (with value 0) if no power save handler is
** present.
**
** Interrupts are disabled during the call to zzcall_power_save().
*/

OSTICK
zzcall_power_save(OSTICK max_time)
{
   OSTICK (*func)(OSTICK) = (OSTICK (*)(OSTICK)) zzconf.power_save_handler;

   if (func)
   {
      return func(max_time);
   }
   return 0;
}

/*
**===========================================================================
** zzearlysys_callh_c
**===========================================================================
** Description:	Report error for system calls issued before kernel start.
**
** Parameters:	arg_arr format  0       1       2       3
**                    error:    ecode   -       -       -
**                    error2:   ecode   extra   -       -
**                    zzerror:  ecode   file    line    -
**                    zzerror2: ecode   extra   file    line
**                    others:   -       -       -       -
** Returns:	-
**
** Globals:	-
*/

void zzearlysys_callh_c(U32 fcode, U32 return_address, U32 arg_arr[])
{
   OSERRCODE ecode = OSE_EFATAL_MASK | OSE_EEARLY_ERROR;
   OSERRCODE extra = (OSERRCODE)arg_arr;

   LOCK();
   switch(fcode)
   {
      case 15:	/* error */
	 arg_arr[1] = 0;	/* extra */
      case 76:	/* error2 */
	 arg_arr[2] = 0;	/* file */
	 arg_arr[3] = 0;	/* line */
	 break;
      case 90:	/* zzerror */
	 arg_arr[3] = arg_arr[2];	/* Move line. */
	 arg_arr[2] = arg_arr[1];	/* Move file. */
	 arg_arr[1] = 0;		/* No extra. */
      case 108:	/* zzerror2 */
	 break;
      default:
	 ecode = OSE_EFATAL_MASK | OSE_ESYSCALL_TOO_EARLY;
	 extra = return_address;
	 break;
   }
   for(;;)
   {
      zzcall_system_errh(0, ecode, extra);
   }
}

/*
**===========================================================================
** zzget_cpu_id
**===========================================================================
** Description:	Read cpu id register.
**
** Parameters:	-
**
** Returns:	-
**
** Globals:	-
*/

U32 zzget_cpu_id(void)
{
   return get32_cp0(C0_PRID);
}

/*
**===========================================================================
** zzcpu_to_pvr
**===========================================================================
** Description:	Convert configured cpu value to prid value.
**
** Parameters:	-
**
** Returns:	-
**
** Globals:	-
*/

U32 zzcpu_to_pvr(U32 cpu_model)
{
  (void)cpu_model;
   return 0;
}

/*
**===========================================================================
** vector base
**===========================================================================
** Description:	-
**
** Parameters:	-
**
** Returns:	-
**
** Globals:	-
*/

U32 zzget_vector_base(void)
{
   return get32_cp0(C0_SR) & SR_BEV ? 0xbfc00000 : 0x80000000;
}

U32 zzset_vector_base(U32 vector_base)
{
   /* FIXME Add code for BCM1250. */
   return vector_base;
}

/*
**===========================================================================
** Get usrcalls struct
**===========================================================================
** Description:	This function replaces the default function for sysconf_54
**              installed by pose.
** Parameters:	-
**
** Returns:	-
**
** Globals:	-
*/

OSADDRESS
zzti_sysconf_54(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
   static int has_been_called;

   if(has_been_called == 0)
   {
      OSADDRESS (*sysconf_54)(OSADDRESS, OSADDRESS, OSADDRESS) =
	 (OSADDRESS (*)(OSADDRESS, OSADDRESS, OSADDRESS))zzti.sysconf_54;
      has_been_called = 1;
#if 0
      ramlog_printf("sysconf_54 usrcalls for monolith\n");
#endif
      /* Call the old function first time, to get optimized calls for the
	 monolith. */
      return (sysconf_54)(p, q, r);
   }

#if 0
   ramlog_printf("sysconf_54 usrcalls for load module\n");
#endif
   /* Return 0 to force fallback functions to be used in pflib for
      load modules. */
   return 0;
}

/*
**===========================================================================
** zzti_create_block
**===========================================================================
** Description:	Check mmu mode.
**              mmu    Do nothing.
**              nommu  Set the supervisor parameter to true to force all
**                     processes to supervisor mode.
** Parameters:	-
**
** Returns:	-
**
** Globals:	-
*/

PROCESS
zzti_create_block(const char *name, OSUSER user, OSBOOLEAN use_remote_calls,
		  PROCESS remote_call_server, OSBOOLEAN supervisor_mode)
{
   if (hal_get_info()->has_mmu == 0)
   {
      /* Force supervisor mode for nommu systems. */
      supervisor_mode = 1;
   }
   return create_block(name, user, use_remote_calls,
		       remote_call_server, supervisor_mode);
}

/*
**===========================================================================
** Init ti C part
**===========================================================================
** Description:	Init local and global data.
**
** Parameters:	-
**
** Returns:	vector size
**
** Globals:	zzti
*/

U32 zzinit_ti(U32 prid)
{
   extern void zzinit_ti_s(void);
   int i;

   (void)prid;
   
#ifdef OSE_SMP
   /* Sanity check of structs. */
   if (sizeof(struct ticpu) != offsetof(struct per_cpu, curr_uarea))
   {
      error2(0x12345678, 1);
   }
#endif
   /* Clear ti data. */
   memset(&zzti, 0, sizeof(struct tidata));

   /*
   ** Setup sr.
   ** Clear PX,KX,SX,UX,UM (KSU),ERL,EXL,IE
   ** Set FR
   */
   zzset_sr(0xff7fff00, 0x04000000);
   /* Clear BEV bit in sr to set vector base in kseg0 (0x80000000). */
   (void) zzset_sr(0xffbfffff, 0);

   /* Init asm part. */
   zzinit_ti_s();

#ifdef OSE_SMP
   /* Setup SMP data for core 0. */
   zzkrn_set_cur_p(0);
   zzkrn_set_per_cpu_data(zzcpu);
   zzkrn_set_cpu_id(0);
#endif
#ifdef OSE_DEBUG
   /* Set zzcurrent_uarea during start to handle bios calls. */
   zzcurrent_uarea = &uarea_start;
#endif

   /* Set current_int_level to nonint level. */
   for(i = 0; i < OSE_SMP_MAX_CPUS; i++)
   {
      zzcpu[i].ticpu.current_int_level = 32;
   }

   /* Init interrupt tables. */
	/* FIXME: shall this init be here ? */
   for(i = 0; i <= MAX_INTVEC; i++)
   {
      zzint_table[i] = (void *)0;
   }

   /* Setup function pointers for timer handling. */
   if(zzconf.read_microtick_handler)
   {
      zzti.get_microticks = zzconf.read_microtick_handler;
   }
   else
   {
      zzti.get_microticks = (U32) get_microticks_soft;
   }

   if(zzconf.clear_microtick_handler)
   {
      zzti.clear_microticks = zzconf.clear_microtick_handler;
   }
   else
   {
      zzti.clear_microticks = (U32) clear_microticks_soft;
   }

   zzti.stack_pattern = STACK_PATTERN;

#ifdef LIBWHAT                  /* olli: ollibuilder what string. */
   LIBWHAT
#endif

   return VECTOR_SIZE;
}

/*
**===========================================================================
** zzupdate_err_handl_level
**===========================================================================
** Description: Save register values when entering an error handler so that
**              they won't get trashed by ssystem calls issued from the
**              error handler.
**
** Parameters:  New error level.
**
*/

void
zzupdate_err_handl_level(OSBOOLEAN level)
{
   UAREA *uarea = zzcurrent_uarea;

   if (level)
   {
      /*
       * Going from error level 0 to 1.
       * Save original values
       */
      uarea->err_lev_0_sp = uarea->call_sp;
      uarea->err_lev_0_ra = uarea->call_ra;
      uarea->err_lev_0_fp = uarea->call_fp;
   }
   else
   {
      /* No longer at error level. */
      uarea->err_lev_0_ra.u64 = 0;
   }
}

#ifdef OSE_SMP
void
zzkrn_set_cur_p(struct _pcb* pcb)
{
#if 1
   register struct per_cpu* data asm("$28");
   data->curr_p = pcb;
#else
   __asm__ __volatile__ (" sw %0,68($28)" :: "r"(pcb));
#endif
}
#endif
