/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

/*DEFINE_START*/

#define UAREA_MAGIC   0x1aea
#define UAREA_MAGIC_I 0x2aea

#define REG_STATE_NA       0
#define REG_STATE_ALL      1
#define REG_STATE_WRITABLE 2
#define REG_STATE_SUBSET   4
#define REG_STATE_FP       8

/*DEFINE_STOP*/

/* Floating point general purpose register. */
typedef struct
{
   U32 msw;
   U32 lsw;
} FPR;

struct reg64
{
#ifdef BIG_ENDIAN
   U32 h;
   U32 l;
#else
   U32 l;
   U32 h;
#endif
};

typedef union
{
   struct reg64 u32;
   uint64_t     u64;
} REG64;

struct fpu
{
   /*INC_PREFIX fpu_ */	/* Sync pattern for script. */
   /*INC_START*/	/* Sync pattern for script. */
   U32 magic;
   U32 instance;
   struct uarea *next;
   U32 fcr0;
      
   U32 fcr25;
   U32 fcr26;
   U32 fcr28;
   U32 fcr31;

   FPR fpr0;
   FPR fpr1;
   FPR fpr2;
   FPR fpr3;
   FPR fpr4;
   FPR fpr5;
   FPR fpr6;
   FPR fpr7;
   FPR fpr8;
   FPR fpr9;
   FPR fpr10;
   FPR fpr11;
   FPR fpr12;
   FPR fpr13;
   FPR fpr14;
   FPR fpr15;
   FPR fpr16;
   FPR fpr17;
   FPR fpr18;
   FPR fpr19;
   FPR fpr20;
   FPR fpr21;
   FPR fpr22;
   FPR fpr23;
   FPR fpr24;
   FPR fpr25;
   FPR fpr26;
   FPR fpr27;
   FPR fpr28;
   FPR fpr29;
   FPR fpr30;
   FPR fpr31;
   /*INC_STOP*/	/* Sync pattern for script. */
};      

typedef struct uarea
{
   /*INC_PREFIX uarea_ */	/* Sync pattern for script. */
   /*INC_START*/		/* Sync pattern for script. */
   U32 magic;
   U32 instance;
   struct uarea *next;
   U32 pid;

   U32 usp_base;
   U32 ssp_base;
   REG64 usp;

   REG64 ssp;
   REG64 call_fp;		/* Frame pointer in appl at flib call. */

   REG64 call_ra;		/* Return address to appl at flib call. */
   REG64 call_sp;		/* Stack pointer in appl at flib call. */

   char *file;
   U32 line;
   struct fpu *fpu_reg_file;
   U32 user_area;

   U32 entry_hi;
   U8  super_mode;
   U8  reg_state;
   U8  freebyte2;
   U8  freebyte3;
   /* save/restore context area. */
   REG64 s0;

   REG64 s1;
   REG64 s2;

   REG64 s3;
   REG64 s4;

   REG64 s5;
   REG64 s6;

   REG64 s7;
   REG64 gp;

   REG64 sp;
   REG64 fp;

   REG64 ra;
   U32   sr;
   U32   _pad1;

   /* Interrupt save area. */
   REG64 izero;
   REG64 iat;

   REG64 iv0;
   REG64 iv1;

   REG64 ia0;
   REG64 ia1;

   REG64 ia2;
   REG64 ia3;

   REG64 it0;
   REG64 it1;

   REG64 it2;
   REG64 it3;

   REG64 it4;
   REG64 it5;

   REG64 it6;
   REG64 it7;

   REG64 is0;
   REG64 is1;

   REG64 is2;
   REG64 is3;

   REG64 is4;
   REG64 is5;

   REG64 is6;
   REG64 is7;

   REG64 it8;
   REG64 it9;

   REG64 ik0;
   REG64 ik1;

   REG64 igp;
   REG64 isp;

   REG64 ifp;
   REG64 ira;

   REG64 ilo;
   REG64 ihi;

   REG64 iepc;
   U32   isr;
   U32   ientry_hi;

   U32   exception_depth;
   U32   _pad2;
   REG64 safecopy_sp;

   REG64 safecopy_err;
   REG64 err_lev_0_fp;

   REG64 err_lev_0_ra;
   REG64 err_lev_0_sp;
   U32 *exception_regdump;	/* Pending register dump pointer or NULL. */
   /*INC_STOP*/	/* Sync pattern for script. */
} UAREA;

/* Only used to get max size of uarea and fp register file. */
typedef union
{
   UAREA a;
   struct fpu b;
} UAREA_UNION;
