/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#include "stdio.h"
#include "stdint.h"
#include "osetypes.h"
#include "ramlog.h"
#include "shell.h"
#include "register.h"
#include "cpu_intlock.h"


#define C0_RND        $1
#define C0_WIRED      $6
#define C0_XCONTEXT   $20

struct tlb_dump
{
   uint32_t pagemask;
   uint32_t entryhi;
   uint64_t entrylo0;
   uint64_t entrylo1;
};

static const char *
page_size(U32 pagemask)
{
   switch (pagemask >> 13)
   {
      case 0x0:
	 return "4k  ";
      case 0x3:
	 return "16k ";
      case 0xf:
	 return "64k ";
      case 0x3f:
	 return "256k";
      case 0xff:
	 return "1M  ";
      case 0x3ff:
	 return "4M  ";
      case 0xfff:
	 return "16M ";
      case 0x3fff:
	 return "64M ";
      case 0xffff:
	 return "256M";
      default:
	 return "????";
   }
}

static int
dump_tlb(int argc, char *argv[])
{
   U32 mmu_size;
   U32 asid;
   U32 i;
   U32 c0, c1, d0, d1, v0, v1, g0, g1;
   U32 lock;
   struct tlb_dump dump;

   (void)argc;
   (void)argv;

   if(argc > 1) /* any arg */
	   mmu_size = get32_cp0(C0_CONFIG);
   else
	   mmu_size = get32_cp0_sel1(C0_CONFIG);   /* FIXME select 1 */

   mmu_size = (mmu_size >> 25) & 0x3f;

   printf("ix asid size lpage    ppage0           cdvg ppage1           cdvg\n");
   printf("-----------------------------------------------------------------\n");

   for(i = 0; i <= mmu_size; i++)
   {
      LOCK_SAVE(lock);
      
      asid = get32_cp0(C0_ENTRYHI);
      asid = asid & 0xff;
      
      set32_cp0(C0_INDEX, i);

      __asm__ __volatile__(
	 ".set\tmips3\n\t"
	 ".set\tnoreorder\n\t"
	 "nop;nop;nop;nop\n\t"
	 "tlbr\n\t"
	 "nop;nop;nop;nop\n\t"
	 ".set\treorder\n\t"
	 ".set\tmips0\n\t");

      dump.pagemask = get32_cp0(C0_PAGEMASK);
      dump.entryhi  = get32_cp0(C0_ENTRYHI);
      dump.entrylo0 = get64_cp0(C0_ENTRYLO_0);
      dump.entrylo1 = get64_cp0(C0_ENTRYLO_1);

      /* Restore asid. */
      set32_cp0(C0_ENTRYHI, asid);

      LOCK_RESTORE(lock);

      c0 = (dump.entrylo0 >> 3) & 7;
      c1 = (dump.entrylo1 >> 3) & 7;
      d0 = (dump.entrylo0 >> 2) & 1;
      d1 = (dump.entrylo1 >> 2) & 1;
      v0 = (dump.entrylo0 >> 1) & 1;
      v1 = (dump.entrylo1 >> 1) & 1;
      g0 = dump.entrylo0 & 1;
      g1 = dump.entrylo1 & 1;

      printf("%02d %03d  %s %08x ",
	     i,
	     dump.entryhi & 0xff,
	     page_size(dump.pagemask),
	     dump.entryhi & 0xffffe000);
      printf("%016llx %d%d%d%d ",
	     (dump.entrylo0 & 0xffffffffffffffc0ULL) << 6,
	     c0, d0, v0, g0);
      printf("%016llx %d%d%d%d\n",
	     (dump.entrylo1 & 0xffffffffffffffc0ULL) << 6,
	     c1, d1, v1, g1);
   }
   return 0;
}

void dump_tlb_cmd_reg(void);

void
dump_tlb_cmd_reg(void)
{
   shell_add_cmd("dumptlb", "dumptlb",
		 "Dump tlb", dump_tlb);
}
