/* COPYRIGHT-ENEA-SRC-R2 *
**************************************************************************
* Copyright (C) 2010 by Enea Software AB.
* All rights reserved.
*
* This Software is furnished under a software license agreement and
* may be used only in accordance with the terms of such agreement.
* Any other use or reproduction is prohibited. No title to and
* ownership of the Software is hereby transferred.
*
* PROPRIETARY NOTICE
* This Software consists of confidential information.
* Trade secret law and copyright law protect this Software.
* The above notice of copyright on this Software does not indicate
* any actual or intended publication of such Software.
**************************************************************************
* COPYRIGHT-END */


 /* Enable fls() function when including heap_osal.h */
#define USE_HEAP_FLS
#include "heap_algo_buddy.h"
#include <string.h>


/**************************************************************************
 * Plug-in History
 **************************************************************************
 *
 * IMPORTANT
 * 
 * This comment shall be updated with a description of the change each
 * time the extension version number is increased. The version number
 * shall be increased if incompatible modifications are made to internal
 * structures or after behavioral changes. See HeapExtension below.
 * 
 **************************************************************************
 *
 *  Version  Description
 *  -------  ----------------------------------------------------------
 *   101    Renamed struct Algorithm to struct BuddyAlgorithm.
 *   100    Initial revision.  
 *
 **************************************************************************/


/***************************************************************************
 * Helper Macros
 ***************************************************************************/
#define SIZE0   16
#define SIZE1   16
#define SIZE2   32
#define SIZE3   48
#define SIZE4   80
#define SIZE5   128
#define SIZE6   208
#define SIZE7   336
#define SIZE8   544
#define SIZE9   880
#define SIZE10  1424
#define SIZE11  2304
#define SIZE12  3728
#define SIZE13  6032
#define SIZE14  9760
#define SIZE15  15792
#define SIZE16  25552
#define SIZE17  41344
#define SIZE18  66896
#define SIZE19  108240
#define SIZE20  175136
#define SIZE21  283376
#define SIZE22  458512
#define SIZE23  741888
#define SIZE24  1200400
#define SIZE25  1942288
#define SIZE26  3142688
#define SIZE27  5084976
#define SIZE28  8227664
#define SIZE29  13312640
#define SIZEMAX SIZE28
#define SIZECOUNT 29

#define BF_BITS 32
#define BF_MSB  0x80000000UL
#define BF_LSB  0x00000001UL

#define BUF_NEXT_FRAG_MASK        0xFFFFFFFCul
#define BUF_NEXT_ID_MASK          0x00000003ul
#define BUF_NEXT_FRAG(frag)       ((struct Fragment*)((uint32_t)(frag) & BUF_NEXT_FRAG_MASK))
#define BUF_NEXT_ID(frag)         (((uint32_t)(frag) & BUF_NEXT_ID_MASK) << BUF_ID_SHIFT)
#define BUF_NEXT_MASK(frag, id)   ((struct Fragment*)((uint32_t)(frag)|((id) >> BUF_ID_SHIFT) ))

#define FRAG_NEXT_FRAG_MASK       0xFFFFFFFCul
#define FRAG_NEXT_ID_MASK         0x00000003ul
#define FRAG_NEXT_FRAG(frag)      ((struct Fragment*)((uint32_t)(frag) & FRAG_NEXT_FRAG_MASK))
#define FRAG_NEXT_ID(frag)        (((uint32_t)(frag) & FRAG_NEXT_ID_MASK) << FRAG_ID_SHIFT)
#define FRAG_NEXT_MASK(frag, id)  ((struct Fragment*)((uint32_t)(frag)|((id) >> FRAG_ID_SHIFT) ))

#define MAX(a,b)                  ((a) > (b) ? (a) : (b))


/***************************************************************************
 * Data Structures
 ***************************************************************************/

/*
 * Buffer header. 
 */
struct Buffer
{
   HeapPID   owner;              /* Buffer owner PID. 0 if shared.  */
   uint32_t  control;            /* Buffer control bits. See below. */

/*
 * Description of bits in the buffer control word:
 *
 * - FREE     The buffer is free.
 * - ROOT     The buffer is the first buffer in a fragment,
 *            i.e. the "root" of the fragment buddy tree. 
 * - LESS     The buddy's left sibling is smaller in size.
 * - IONE     The buddy has buffer size index == 1. Read more below.
 * - SIZE     The size of the buffer when allocated, including header
 *            and endmark, or the buddy size index if the buffer is free. Since
 *            the same field is shared for both size and index, there must be
 *            a unique mapping between index and size so that the heap may
 *            switch between them. Buffer size 16 may however have either
 *            of two indexes, 0 and 1. The IONE bit is used to identify the
 *            index in that case.  See buf_aindex();
 * - UCHECK   Lightweight integrity check for allocated buffers.
 * - FCHECK   Heavyweight integrity check for free buffers.
 * - ANNOT    Buffer is annotated with debug information.
 * - GC       Indicates that the buffer is a hostage of the
 *            garbage collector. The buffer cannot be freed. If it
 *            is, the free operation is deferred and carried out by
 *            the garbage collector later. 
 */

#define BUF_ID_MASK              0xC0000000ul
#define BUF_ID_SHIFT             30
#define BUF_FREE_MASK            0x10000000ul
#define BUF_ROOT_MASK            0x20000000ul
#define BUF_LESS_MASK            0x08000000ul
#define BUF_IONE_MASK            0x04000000ul
#define BUF_SIZE_MASK            0x00FFFFFEul
#define BUF_SIZE_SHIFT           1
#define BUF_GC_MASK              0x02000000ul
#define BUF_UCHECK_SRC_MASK      0x08000000ul
#define BUF_UCHECK_SRC_SHIFT     3
#define BUF_UCHECK_MASK          0x01000000ul
#define BUF_UCHECK_SHIFT         24
#define BUF_FCHECK_MAGIC         0x80102040ul
#define BUF_ANNOT_MASK           0x00000001ul

#define buf_id(buf)              ((buf)->control & BUF_ID_MASK)
#define buf_fbit(buf)            ((buf)->control & BUF_FREE_MASK)
#define buf_rbit(buf)            ((buf)->control & BUF_ROOT_MASK)
#define buf_lbit(buf)            ((buf)->control & BUF_LESS_MASK)
#define buf_ione(buf)            ((buf)->control & BUF_IONE_MASK)
#define buf_size(buf)            (((buf)->control & BUF_SIZE_MASK) >> BUF_SIZE_SHIFT)
#define buf_owner(buf)           ((buf)->owner)
#define buf_annotated(buf)       ((buf)->control & BUF_ANNOT_MASK)
#define buf_gc(buf)              ((buf)->control & BUF_GC_MASK)

#define buf_set_id(buf,id)       ((buf)->control |= ((buf)->control & ~BUF_ID_MASK) | (id))
#define buf_set_fbit(buf)        ((buf)->control |= BUF_FREE_MASK)
#define buf_set_rbit(buf)        ((buf)->control |= BUF_ROOT_MASK)
#define buf_set_lbit(buf)        ((buf)->control |= BUF_LESS_MASK)
#define buf_set_ione(buf)        ((buf)->control |= BUF_IONE_MASK)
#define buf_set_owner(buf,pid)   ((buf)->owner = (pid))
#define buf_set_gc(buf)          ((buf)->control |= BUF_GC_MASK)
   
#define buf_clear_fbit(buf)      ((buf)->control &= ~BUF_FREE_MASK)
#define buf_clear_rbit(buf)      ((buf)->control &= ~BUF_ROOT_MASK)
#define buf_clear_lbit(buf)      ((buf)->control &= ~BUF_LESS_MASK)
#define buf_clear_ione(buf)      ((buf)->control &= ~BUF_IONE_MASK)
#define buf_clear_anno(buf)      ((buf)->control &= ~BUF_ANNOT_MASK)
#define buf_clear_gc(buf)        ((buf)->control &= ~BUF_GC_MASK)

#define buf_set_index(buf,size)  ((buf)->control = ((buf)->control & ~BUF_SIZE_MASK) | ((size) << BUF_SIZE_SHIFT))
#define buf_index                buf_size

#define is_rleaf(ix)             ((ix) & 1)        /* Right side leaf. */
#define is_lleaf(ix)             (!is_rleaf(ix))   /* Left  side leaf. */

};

/***************************************************************************/

/*
 * The format of a free buffer.
 * Two words of user data are used as links for the freelists.
 */
struct FreeBuffer
{
   struct Buffer      buf;
   struct FreeBuffer *next;
   struct FreeBuffer *prev;
};

/***************************************************************************/

struct Fragment;

/*
 * External memory region. 
 */
struct Region
{
   struct Region   *next;            /* List link.                      */
   struct Region   *prev;            /* List link.                      */
   struct Fragment *frag_list;       /* First fragment in region.       */
   struct Fragment *frag_last;       /* End of last fragment in region  *
                                      * (not the last fragment header). */
   uint32_t         frag_count;      /* Number of fragments in region.  */
   uint32_t         control;         /* Region control bits. See below. */

/* Bit descriptions:
 *
 * INITIAL - Initial heap region allocated at heap creation.
 *           Can never be removed.
 * ID      - ID of originating allocator.
 * BUF_ID  - ID of algorithm that allocated the region.
 * LOCK    - Region is being traversed and cannot be deallocated.
 */
   
#define REGION_BUF_ID_MASK             0xC0000000ul
#define REGION_BUF_ID_SHIFT            30
#define REGION_ID_MASK                 0x30000000ul
#define REGION_ID_SHIFT                28
#define REGION_INITIAL_MASK            0x08000000ul
#define REGION_LOCK_MASK               0x04000000ul
#define REGION_CHECK_MASK              0x03FFFFFFul
#define region_initial(reg)            ((reg)->control & REGION_INITIAL_MASK)
#define region_set_initial(reg)        ((reg)->control |= REGION_INITIAL_MASK)
#define region_buf_id(reg)             ((reg)->control & REGION_BUF_ID_MASK)
#define region_id(reg)                 ((reg)->control & REGION_ID_MASK)
#define region_set_control(reg,buf_id) ((reg)->control = (buf_id))
#define region_set_lock(reg)           ((reg)->control |= REGION_LOCK_MASK)
#define region_clear_lock(reg)         ((reg)->control &= ~REGION_LOCK_MASK)
#define region_locked(reg)             ((reg)->control & REGION_LOCK_MASK)

#define REGION_CHECKSUM_MAGIC        0x80102040ul
#define region_checksum(_r)                                     \
   ( ((uint32_t)(_r)->next ^                                    \
      (uint32_t)(_r)->prev ^                                    \
      (uint32_t)(_r)->frag_list ^                               \
      (uint32_t)(_r)->frag_last ^                               \
      ((_r)->control & ~REGION_CHECK_MASK) ^                    \
      REGION_CHECKSUM_MAGIC) &                                  \
      REGION_CHECK_MASK)
#define region_set_checksum(_r)    ((_r)->control = ((_r)->control & ~REGION_CHECK_MASK) | region_checksum(_r))
#define region_get_checksum(_r)    ((_r)->control & REGION_CHECK_MASK)
};

/***************************************************************************/

/*
 * A Fibonacci buddy fragment. 
 *
 * Regions are broken down into several fragments, with sizes according to
 * the Fibonacci series. Fragments are required since the root buffer of a
 * buddy tree must have a Fibonacci size, which regions typically don't have.
 * Splitting and merging operations are always confined within a single fragment. 
 */
struct Fragment
{
   struct Fragment *next;              /* List link.                         */
   struct Fragment *prev;              /* List link.                         */
   struct Buffer   *root;              /* First buffer in fragment,          *
                                        * root of  buddy tree.               */ 
   struct Buffer   *sentinel;          /* "Fake" allocated buffer at the end *
                                        * of a fragment. Used to stop merge  *
                                        * operations when reaching the root  *
                                        * buffer.                            */

   struct Region   *region;            /* The region that the fragment belongs to. */
   uint32_t         reserved0;         /* Padding for proper alignment.   */
   uint32_t         root_index;        /* Fibonacci index of root buffer. */
   uint32_t         control;           /* Control bits.                   */

#define FRAG_BUF_ID_MASK               0xC0000000ul
#define FRAG_BUF_ID_SHIFT              30
#define FRAG_ID_MASK                   0x30000000ul
#define FRAG_ID_SHIFT                  28
#define frag_buf_id(frag)              ((frag)->control & FRAG_BUF_ID_MASK)
#define frag_id(frag)                  ((frag)->control & FRAG_ID_MASK)
#define frag_set_control(frag,bid,id)  ((frag)->control = (bid)|(id))

#define FRAG_CHECKSUM_MAGIC        0x80102040ul
#define frag_checksum(_r)                                       \
   ( ((uint32_t)(_r)->next ^                                    \
      (uint32_t)(_r)->prev ^                                    \
      (uint32_t)(_r)->root ^                                    \
      (uint32_t)(_r)->region ^                                  \
      (((_r)->control & (FRAG_ID_MASK|FRAG_BUF_ID_MASK)) >> FRAG_ID_SHIFT) ^ \
      FRAG_CHECKSUM_MAGIC) &                                    \
     ~(FRAG_ID_MASK|FRAG_BUF_ID_MASK))
#define frag_set_checksum(_r)      ((_r)->control = frag_id(_r)|frag_buf_id(_r)|frag_checksum(_r))
#define frag_get_checksum(_r)      ((_r)->control & ~(FRAG_ID_MASK|FRAG_BUF_ID_MASK))
};

/***************************************************************************/

/*
 * Structure used for fragment iteration queries. (list all fragments)
 * All fields shall be zero initialized before the first query and
 * field values must be retained by clients during subsequent queries. 
 */
struct FragmentCursor
{
   struct Fragment *frag_out;     /* Output: Fragment pointer.  */
   size_t           size_out;     /* Output: Size of fragment.  */
   struct Fragment *frag;         /* Determines last position.  */
   struct Fragment *frag_org;     /* Determines last position.  */
};

/***************************************************************************/

/*
 * Administrative data for the algorithm. 
 */
struct BuddyAlgorithm
{
   uint32_t id;                         /* Buffer algorithm identifier.   */
   uint32_t frag_id;                    /* Fragment allocator identifier. */
   
   struct HeapStatistics    *stats;
   struct HeapConfiguration *config;
   struct HeapServiceInfo    info;        /* Plug-in information. */

   struct FreeBuffer   *list[SIZECOUNT];  /* Freelists.                 */
   uint32_t             bitmap;           /* Freelist bitmap indicating
                                           * availability of buffers.   */

   struct Region       *region_list;      /* List of allocated regions. */
   uint32_t             region_count;     /* Number of regions.         */
   uint32_t             frag_count;       /* Number of fragments.       */
};

/***************************************************************************/

/*
 * Structure used for buffer iteration queries. (list all buffers)
 * All fields shall be zero initialized before the first query and
 * field values must be retained by clients during subsequent queries. 
 */
struct BufferCursor
{
   struct Buffer   *buf;          /* Output: Buffer pointer.   */
   struct Fragment *frag;         /* Determines last position. */
   uint32_t         path;         /* Determines last position. */
   uint32_t         depth;        /* Determines last position. */
};

/***************************************************************************/





/***************************************************************************
 * Forward Declarations
 ***************************************************************************/

static inline HeapError
buf_check_endmark(struct Heap *heap, struct BuddyAlgorithm *a,
                  struct Region *reg, struct Buffer *buf);

static inline HeapError
buf_check_owner(struct Heap *heap, struct BuddyAlgorithm *a,
                struct Buffer *buf, HeapPID pid1);

static inline HeapError
buf_check_frag(struct Heap *heap, struct BuddyAlgorithm *a,
               struct Region **frag, struct Buffer *buf);

static inline HeapError
buf_check_header(struct Heap *heap, struct BuddyAlgorithm *a,
                 struct Region *reg, struct Buffer *buf);

static inline HeapError
buf_check_at_free(struct Heap *heap, struct BuddyAlgorithm *a,
                  struct Buffer *buf, HeapPID pid);

static inline HeapError
buf_check_at_query(struct Heap *heap, struct BuddyAlgorithm *a,
                   struct Region **reg, struct Buffer *buf);

static inline HeapError
frag_check(struct Fragment *frag);

static inline struct Fragment *
frag_first(struct Heap *heap, struct Region *region, HeapError *ecode);

static inline struct Fragment *
frag_next(struct Heap *heap, struct Fragment *frag, HeapError *ecode);

static inline struct Region *
region_first(struct Heap *heap, struct BuddyAlgorithm *a, HeapError *ecode);

static inline struct Region *
region_next(struct Heap *heap, struct Region *frag, HeapError *ecode);

static inline struct Region *
region_find(struct Heap *heap, struct BuddyAlgorithm *a,
            void *ptr, uint32_t exact, HeapError *ecode);

static inline HeapError
region_check(struct Region *frag);

static inline struct Fragment *
frag_find(struct Heap *heap, struct Region *reg, void *ptr,
          int exact, HeapError *ecode);



/***************************************************************************
 * Static Data
 ***************************************************************************/

/*
 * Fibonacci Series (multiple of 16)
 */
static const size_t fibonacci[] =
{
    SIZE0, SIZE1, SIZE2, SIZE3, SIZE4, SIZE5, SIZE6, SIZE7, SIZE8,
    SIZE9, SIZE10, SIZE11, SIZE12, SIZE13, SIZE14, SIZE15, SIZE16,
    SIZE17, SIZE18, SIZE19, SIZE20, SIZE21, SIZE22, SIZE23, SIZE24,
    SIZE25, SIZE26, SIZE27, SIZE28, SIZE29
};


/***************************************************************************
 * Private Interface
 ***************************************************************************/

#define BITMAP_SET(bitmap,bit)    ((bitmap) |= (0x80000000ul >> bit))
#define BITMAP_CLEAR(bitmap,bit)  ((bitmap) &= ~(0x80000000ul >> bit))

/***************************************************************************/

static always_inline long
BITMAP_FIND(size_t bitmap, size_t min)
{
   /*
    * Find the most significant bit set in a
    * bitmap. Should return -1 if bitmap is 0.
    *
    * The "min" parameter identifies where
    * the search should start.
    *****************************************/

   bitmap &= (~0ul >> (min));
   if (!bitmap)
      return -1;
   return 31 - fls(bitmap);
}

/***************************************************************************/

static always_inline size_t
size_lindex(size_t size)
{
   int i = 0;

   /* Scan until an index with larger size is found. */
   for (; i<SIZECOUNT && size>fibonacci[i]; i++);

   return i;
}

/***************************************************************************/

static always_inline size_t
size_sindex(size_t size)
{
   int i = SIZECOUNT-1;

   /* Scan until an index with smaller size is found. */
   for (; i>=0 && size<fibonacci[i]; i--);

   return i;
}

/***************************************************************************/



/***************************************************************************/

static always_inline struct Buffer *
buf_init(struct BuddyAlgorithm *a, void *ptr, uint32_t index, uint32_t stats)
{
   struct Buffer *buf = ptr;
   
   buf->owner   = 0;
   buf->control = a->id;
   buf_set_index(buf, index);

   a->stats->buf_num_used[index] += stats;

   return buf;
}

/***************************************************************************/

static always_inline uint32_t
buf_ucheck(struct Buffer *buf)
{
#ifdef ASSERT
   uint32_t chk = buf->control & ~(BUF_UCHECK_MASK|BUF_ANNOT_MASK);
   char c, *p = (char *)&chk;

   c = (p[0] ^ p[2]) ^ (p[1] ^ p[3] ^ BUF_FCHECK_MAGIC);

   chk = c ^ (c >> 2) ^ (c >> 4) ^ (c >> 6) ^ 0x2;

   return (chk & 0x3) << BUF_UCHECK_SHIFT;
#else
   /* "Calculate" the integrity checksum the buffer buffer. */
   return ((~buf->control) & BUF_UCHECK_SRC_MASK) >> BUF_UCHECK_SRC_SHIFT;
#endif
}

/***************************************************************************/

static always_inline uint32_t
buf_get_ucheck(struct Buffer *buf)
{
   return buf->control & BUF_UCHECK_MASK;
}

/***************************************************************************/

static always_inline void
buf_set_ucheck(struct Buffer *buf)
{
   buf->control = (buf->control & ~BUF_UCHECK_MASK) | buf_ucheck(buf);
}

/***************************************************************************/


static always_inline uint32_t
buf_fcheck(struct Buffer *buf)
{
   struct FreeBuffer *fbuf = (struct FreeBuffer *)buf;
   uint32_t chk;

   /* Calculate the intergrity checksum of header and freelist links. */
   chk = (uint32_t)fbuf->next ^ (uint32_t)fbuf->prev ^ fbuf->buf.control;

   return chk ^ BUF_FCHECK_MAGIC;
}

/***************************************************************************/

static always_inline uint32_t
buf_get_fcheck(struct Buffer *buf)
{
   return buf->owner;
}

/***************************************************************************/

static always_inline void
buf_set_fcheck(struct Buffer *buf)
{
   buf->owner = buf_fcheck(buf);
}

/***************************************************************************/

static always_inline void
buf_set_check(struct Buffer *buf)
{
   /* Update integrity checksum. */

   buf_set_ucheck(buf);

   if (buf_fbit(buf))
       buf_set_fcheck(buf);
}

/***************************************************************************/

static always_inline uint32_t
buf_aindex(struct Buffer *buf)
{
   /* Return the actual fibonacci index of the buffer.
    * This has to be calculated from the requested size
    * if the buffer is allocated. A special case exists
    * if the index is 1, which is indicated by a bit in
    * the buffer header. size_lindex would return index
    * 0 even if the index is actually 1. */
   
   if (buf_fbit(buf))
      return buf_index(buf);

   if (buf_ione(buf))
      return 1;

   return size_lindex(buf_size(buf));
}

/***************************************************************************/

static always_inline void
buf_restore_aindex(struct Buffer *buf)
{
   /* Restore the index of an allocated buffer that is
    * being freed. The index is calculated from the
    * requested size. */
   
   buf_set_index(buf, buf_aindex(buf));
   buf_clear_ione(buf);
}


/***************************************************************************/

static always_inline void
buf_set_asize(struct Buffer *buf, size_t ix)
{
   /* Set the requested size of a buffer that has been
    * allocated and is about to be handed out. The function
    * sets the 1-bit if the index is 1 to allow correct
    * recovery of the index during free. */
   
   heap_assert(!buf_fbit(buf), "Attempt to overwrite free buffer's index.");

   if (buf_index(buf) == 1)
      buf_set_ione(buf);

   buf_set_index(buf, ix);
}

/***************************************************************************/

static always_inline size_t 
buf_act_size(struct BuddyAlgorithm *a, struct Buffer *buf)
{
   /* Return the actual size of the buffer, regardless
    * if the buffer is allocated or free. */
   (void)a;

   return fibonacci[buf_aindex(buf)];
}

/***************************************************************************/

static always_inline size_t 
buf_req_size(struct BuddyAlgorithm *a, struct Buffer *buf)
{
   /* Return the requested size of the buffer,
    * only if it is allocated. */
   
   if (unlikely(buf_fbit(buf)))
      return 0;

   return buf_size(buf)-sizeof(struct Buffer)-(a)->config->buf_size_endmark;
}

/***************************************************************************/

static inline void
buf_insert(struct Heap *heap, struct BuddyAlgorithm *a, struct Buffer *buf)
{
   /* Inserts a buffer into the heap freelists and
    * updates the freelist bitmap to indicate that
    * the list has a buffer available.
    *
    * The function calculates a header checksum which
    * is stored in the owner field and later validated
    * by the buf_check_header() function. */
   
   struct FreeBuffer *fb = (struct FreeBuffer *)buf;
   uint32_t index = buf_index(buf);
   HeapError ecode;

   fb->prev = NULL;
   fb->next = (a)->list[index];

   if (fb->next)
   {
#ifdef ASSERT
      ecode = buf_check_header(heap, a, NULL, &fb->next->buf);
#else
      ecode = buf_fcheck(&fb->next->buf) != buf_get_fcheck(&fb->next->buf) ?
         HEAP_ERROR_BUF_LINKS : HEAP_ERROR_NONE;
#endif      
      if (unlikely(ecode != HEAP_ERROR_NONE))
      {
         zzheap_error(heap, HEAP_ERROR_FATAL|ecode, &(&fb->next->buf)[1]);
         return;
      }

      fb->next->prev = fb;
      buf_set_check(&fb->next->buf);
   }

   (a)->list[index] = fb;

   BITMAP_SET((a)->bitmap,(index));
   buf_set_fbit(buf);

   buf_set_check(&fb->buf);

   a->stats->buf_num_free[index]++;
   a->stats->buf_num_used[index]--;
   a->stats->buf_size_free += fibonacci[index];
}

/***************************************************************************/

static inline void
buf_remove(struct Heap *heap, struct BuddyAlgorithm *a, struct Buffer *buf)
{
   /* Removes a buffer from the heap freelists and
    * updates the freelist bitmap to indicate that
    * it is empty, if it has become empty.
    *
    * The integrity checksums of the buffer and its
    * adjacent buffers in the list are validate and
    * updated if affected. 
    * */
   
   struct FreeBuffer *fb = (struct FreeBuffer *)buf;
   uint32_t index = buf_index(buf);
   HeapError ecode;

#ifdef ASSERT
   ecode = buf_check_header(heap, a, NULL, buf);
#else
   ecode = buf_fcheck(buf) != buf_get_fcheck(buf) ?
      HEAP_ERROR_BUF_LINKS : HEAP_ERROR_NONE;
#endif
   if (unlikely(ecode != HEAP_ERROR_NONE))
   {
      zzheap_error(heap, HEAP_ERROR_FATAL|ecode, &buf[1]);
      return;
   }

   if (fb->next)
   {
#ifdef ASSERT
      ecode = buf_check_header(heap, a, NULL, &fb->next->buf);
#else
      ecode = buf_fcheck(&fb->next->buf) != buf_get_fcheck(&fb->next->buf) ?
         HEAP_ERROR_BUF_LINKS : HEAP_ERROR_NONE;
#endif
      if (unlikely(ecode != HEAP_ERROR_NONE))
      {
         zzheap_error(heap, HEAP_ERROR_FATAL|ecode, &(&fb->next->buf)[1]);
         return;
      }

      fb->next->prev = fb->prev;
      buf_set_check(&fb->next->buf);
   }

   if (fb->prev)
   {
#ifdef ASSERT
      ecode = buf_check_header(heap, a, NULL, &fb->prev->buf);
#else
      ecode = buf_fcheck(&fb->prev->buf) != buf_get_fcheck(&fb->prev->buf) ?
         HEAP_ERROR_BUF_LINKS : HEAP_ERROR_NONE;
#endif
      if(unlikely(ecode != HEAP_ERROR_NONE))
      {
         zzheap_error(heap, HEAP_ERROR_FATAL|ecode, &(&fb->prev->buf)[1]);
         return;
      }

      fb->prev->next = fb->next;
      buf_set_check(&fb->prev->buf);
   }

   if (a->list[index] == fb)
      a->list[index] = fb->next;

   fb->next = fb->prev = NULL;

   if (!a->list[index])
      BITMAP_CLEAR(a->bitmap,index);

   buf_clear_fbit(buf);

   buf_set_check(buf);

   a->stats->buf_num_free[index]--;
   a->stats->buf_num_used[index]++;
   a->stats->buf_size_free -= fibonacci[index];
}

/***************************************************************************/

static always_inline struct Buffer *
buf_find(struct Heap *heap, struct BuddyAlgorithm *a, uint32_t index)
{
   /* Searches the freelist bitmap for any buffer that
    * has an index equal to or larger than the requested
    * index. */

   struct FreeBuffer *fbuf;
#ifdef ASSERT
   HeapError ecode;
#endif

   /* Locate a freelist with an available buffer. */
   int i = BITMAP_FIND(a->bitmap, index);
   if (unlikely(i < 0))
      return NULL;

   (void)heap;

   fbuf = a->list[i];

#ifdef ASSERT
   /* Validate the found buffer's header. */
   ecode = buf_check_header(heap, a, NULL, &fbuf->buf);
   if (ecode != HEAP_ERROR_NONE )
   {
      zzheap_error(heap, ecode, &(&fbuf->buf)[1]);
      return NULL;
   }
#endif
   
   return &fbuf->buf;
}

/***************************************************************************/

static inline struct Buffer *
buf_split_right(struct Heap *heap, struct BuddyAlgorithm *a,
                struct Buffer *buf, uint32_t req_ix)
{
   /* This function is called when a buffer is resized to
    * become smaller. The buffer is split multiple times
    * if needed until its index equals the requested index.
    * The right leaf that is left over after a split is
    * inserted into freelists and made available for
    * allocation. */
   
   struct Buffer *right;
   uint32_t ix = buf_index(buf);

   heap_assert(!buf_fbit(buf), "Attempted to split a free buffer.");

   /* Check if the buffer already is as small as it can be. */
   if (unlikely(ix <= 1))
      return buf;

   /* Check that the requested size isn't a right leaf.
    * It's not possible to split a buffer into a right
    * leaf without loosing user data currently present
    * in the left counterpart. NULL is returned and the
    * request is intercepted to force the CRT to allocate
    * a new buffer and copy the data. */
   if (unlikely(is_rleaf(req_ix)))
      return NULL;
   
   a->stats->buf_num_used[ix]--;
   
   while (ix > req_ix)
   {
      if (is_rleaf(ix)) 
      {
         ix -= 1;
         
         right = (struct Buffer *)((char *)buf + fibonacci[ix]);
         buf_init(a, right, ix-1, 1);

         buf_set_index(buf,ix);
      }
      else 
      {
         ix -= 2;
      
         right = (struct Buffer *)((char *)buf + fibonacci[ix]);
         buf_init(a, right, ix+1, 1);
         buf_set_lbit(right);
  
         buf_set_index(buf,ix);
      }

      buf_insert(heap, a, right);
   }

   a->stats->buf_num_used[ix]++;

   buf_set_check(buf);

   return buf;
}

/***************************************************************************/

static inline struct Buffer *
buf_split(struct Heap *heap, struct BuddyAlgorithm *a,
          struct Buffer *buf, uint32_t ix_req)
{
   /* This function is called during regular allocations
    * to split a buffer if the only available buffer found
    * in freelists was larger than requested. The function
    * iteratively splits the buffer until one of the two
    * parts have the requested index.
    *
    * The buffer to be split must have been removed from the
    * freelist and tagged as allocated before this function is
    * called. */
   
   struct Buffer *right;
   uint32_t ix_buf = buf_index(buf), ix_right;

   heap_assert(!buf_fbit(buf), "Attempted to split a free buffer.");

   /* Check if the buffer already is as small as it can get. */
   if (unlikely(ix_buf <= 1))
      return buf;
   

   a->stats->buf_num_used[ix_buf]--;

   while (ix_req < ix_buf && ix_buf > 1)
   {
      if (is_rleaf(ix_buf))
      {
         ix_right = ix_buf - 1;

         right = buf;
         buf = (struct Buffer *)((char *)buf + fibonacci[ix_right]);
         buf_init(a, buf, ix_buf = ix_buf-2, 0);

         buf_set_index(right, ix_right);
      }
      else
      {
         ix_buf -= 2;
      
         right = (struct Buffer *)((char *)buf + fibonacci[ix_buf]);
         buf_init(a, right, ix_right = ix_buf+1, 0);
         buf_set_lbit(right);

         buf_set_index(buf, ix_buf);
      }

      /* Check if we can use 'right' to satisfy request. */
      if (ix_req < ix_right)
      {
         /* Requested buffer is smaller than 'right'.
          * Keep 'buf' and free 'right'. */
         a->stats->buf_num_used[ix_right]++;
         buf_insert(heap, a,right);
      }
      else
      {
         /* 'right' buddy has the correct index, free 'buf'. */
         a->stats->buf_num_used[ix_buf]++;
         buf_insert(heap, a, buf);
         buf = right;
         ix_buf = ix_right;
         break;
      }
   }

   a->stats->buf_num_used[ix_buf]++;

   return buf;
}

/***************************************************************************/

static inline struct Buffer *
buf_merge(struct Heap *heap, struct BuddyAlgorithm *a, struct Buffer *buf)
{
   /* This function merges a buffer recursively with its
    * buddy as far as possible and returns the resulting
    * larger buffer. */
   
   uint32_t ix = buf_index(buf);

   heap_assert(!buf_fbit(buf), "Attempted to merge a free buffer.");

   a->stats->buf_num_used[ix]--;

   for (;;)
   {
      if (is_rleaf(ix))  /* The buffer is a right leaf buddy. */
      {
         uint32_t ix_left = buf_lbit(buf) ? ix-1 : ix+1;
         struct Buffer *left = (struct Buffer *)((char *)buf - fibonacci[ix_left]);

         /* We cannot merge this right leaf buffer with its left side buddy if:
          * 1) this right leaf is the fragment root buffer, or
          * 2) the left buddy is used, or
          * 3) the left buddy has been split, i.e. has incorrect index. 
          */
         if (buf_rbit(buf) || !buf_fbit(left) || (buf_index(left) != ix_left))
         {
            a->stats->buf_num_used[ix]++;
            return buf;
         }

         /*
          * Calculate the index of the merged buddy.
          * It could be calculated as MAX(left_ix, right_ix) + 1.
          */
         ix = (ix + ix_left + 3) / 2;

         buf_remove(heap, a, left);
         buf_set_index(left, ix);

         a->stats->buf_num_used[ix_left]--;

         buf = left;
      }
      else /* The buffer is a left leaf buddy. */
      {
         struct Buffer *right = (struct Buffer *)((char *)buf + fibonacci[ix]);
         uint32_t ix_right = buf_index(right);

         /* We cannot merge this left leaf buddy with its right side buddy if:
          * 1) the right buddy is split, i.e. it has become a left side buddy
          *    and has its own right side buddy.
          * 2) the right buddy is used.
          */
         if (is_lleaf(ix_right) || !buf_fbit(right))
         {
            a->stats->buf_num_used[ix]++;
            return buf;
         }

         /*
          * Calculate the index of the merged buddy.
          * It could be calculated as MAX(left_ix, right_ix) + 1.
          */
         ix = (ix + ix_right + 3) / 2;

         buf_remove(heap, a, right);
         buf_set_index(buf, ix);

         a->stats->buf_num_used[ix_right]--;
      }
   }
}

/***************************************************************************/

static inline struct Buffer *
buf_merge_right(struct Heap *heap, struct BuddyAlgorithm *a,
                struct Buffer *buf, uint32_t ix_req)
{
   /* This function recursively merges a left leaf buffer
    * with its right buddy. The function is called to
    * handled resize requests where a larger buffer is
    * requested.
    *
    * No merge is allowed when the buffer is a right leaf
    * or when merging would not reach the requested index
    * due to an allocated buddy higher up in the tree. */
   
   struct Buffer *right;
   uint32_t ix = buf_index(buf), ix_right;

   heap_assert(!buf_fbit(buf), "Attempted to merge a free buffer.");

   /* Check if the buffer is a right leaf. */
   if (is_rleaf(ix))
      return NULL;

   /* Check if merge is possible up to the requested index. */
   while (ix_req > ix && is_lleaf(ix))
   {
      right = (struct Buffer *)((char *)buf + fibonacci[ix]);
      ix_right = buf_index(right);

      /* Cannot merge the left buddy with 'right' if
       * 'right' is split, i.e. is a left side leaf,
       * or if the right buddy is used. */
      if (!buf_fbit(right) || is_lleaf(ix_right))
         return NULL;

      /*
       * Calculate the index of the larger merged buddy.
       * It could be calculated as MAX(left_ix, right_ix) + 1.
       */
      ix = (ix + ix_right + 3) / 2;
   }

   if (ix_req != ix)
      return NULL;


   /* Everything looks good, proceed and merge. */
   ix = buf_index(buf);
   a->stats->buf_num_used[ix]--;

   while (ix_req > ix)
   {
      right = (struct Buffer *)((char *)buf + fibonacci[ix]);
      ix_right = buf_aindex(right);

      /*
       * Calculate the index of the larger merged buddy.
       * It could be calculated as MAX(left_ix, right_ix) + 1.
       */
      ix = ((ix + ix_right + 3)/2);
      
      buf_remove(heap, a, right);
      buf_set_index(buf, ix);

      a->stats->buf_num_used[ix_right]--;
   }

   a->stats->buf_num_used[ix]++;

   buf_set_check(buf);
   
   return buf;
}

/***************************************************************************/

static inline uint32_t
buf_next_path(uint32_t *path, uint32_t depth)
{
   /*
    * Calculates the next binary path to follow when
    * traversing a buddy fragment as a tree from the
    * root buddy. The path is a bitmap where 0 and 1
    * indicate directions (left,right) at each depth
    * in the tree. The path together with the depth
    * uniquely identify a buddy leaf. This function
    * advances the path to lead to the next leaf in
    * the tree. 
    */
   
   uint32_t path_next;

   if (*path == 0 && depth == 0)
      return 0;
   
   path_next  = *path + (BF_MSB >> (depth - 1));
   path_next &= ~((BF_LSB << (BF_BITS - depth)) - 1);
   
   /* Break if overflow. */
   if (unlikely(path_next <= *path))
   {
      *path = 0;
      return 0;
   }
   
   *path = path_next;

   return 1;
}

/***************************************************************************/

static inline struct Buffer *
buf_from_path(struct Heap *heap,
              struct BuddyAlgorithm *a,
              struct Buffer *buf,
              uint32_t nominal_index,
              uint32_t path, uint32_t *depth)
{
   /*
    * Returns the buddy that is uniquely identified by
    * the tree path and the tree depth as previously
    * calculated in buf_next_path(), i.e. it traverses
    * the buddy tree from the root down to the buddy
    * that is pointed out. 
    */
   
   HeapError ecode;
   
   /* Local copy of depth for speed. */
   uint32_t local_depth = *depth;

   /* Precompute next mask to use. */
   uint32_t next_mask = BF_MSB >> local_depth;

#if 0
   ecode = buf_check_header(heap, a, NULL, buf);
   if (unlikely(ecode != HEAP_ERROR_NONE))
      return NULL;
#endif

   /* Original code reads while (buf->index != nominal_index).
    * However, if the buffer index returned is incorrect due to damaged
    * heap data, the loop will never stop until local_depth becomes incorrect.
    *
    * Previously we called error when the depth became incorrect, but
    * now this function is also used by HFMI where no errors are
    * acceptable. 
    *
    * We instead break the loop when the nominal index becomes too
    * low. The damaged buffer will be returned and can be investigated. */
   while (buf_aindex(buf) < nominal_index)
   {
      uint32_t mask = next_mask;
        
      /* If local_depth is greater than or equal to FIB_ELEMENTS, this
       * indicates either that we have run this loop outside the limit
       * of the tree, or that *depth had a bad value in the call to this
       * function.
       */
      if (local_depth >= SIZECOUNT)
      {
         return NULL;
      }

      /* Adjust depth for one level deeper. */
      local_depth++;

      /* And adjust the next_mask accordingly. */
      next_mask >>= 1;

      if (is_rleaf(nominal_index))	/* Should be odd index. */
      {
         /* The odd buddies on the right side. */

         /* Check if follow right. */
         if ((path & mask) != 0)
         {
            /* Yes, follow right. */
            struct Buffer *right;

            /* FIXME: Maybe call zzheap_error here? */
            ecode = buf_check_header(heap, a, NULL, buf);
            if (unlikely(ecode != HEAP_ERROR_NONE))
               return NULL;

            right = (struct Buffer *)((char *)buf + fibonacci[nominal_index - 1]);

            buf = right;
            nominal_index -= 1;
         }

         nominal_index -= 1;

         /* Else follow left. */
      }
      else /* Should be even index. */
      {
         /* The even buddies on the left side. */

         /* Check if follow right. */
         if ((path & mask) != 0)
         {
            /* Yes follow right. */
            struct Buffer *right;

            /* FIXME: Maybe call zzheap_error here? */
            ecode = buf_check_header(heap, a, NULL, buf);
            if (unlikely(ecode != HEAP_ERROR_NONE))
               return NULL;

            right = (struct Buffer *)((char *)buf + fibonacci[nominal_index - 2]);

            buf = right;
            nominal_index += 1;
         }

         nominal_index -= 2;

         /* Else follow left. */
      }
   }

   /* The updated depth must also be returned. */
   *depth = local_depth;
   return buf;
}

/***************************************************************************/

static inline struct Buffer *
buf_first(struct Heap *heap, struct BuddyAlgorithm *a, struct Fragment *frag)
{
   /* Helper function for linear (non-tree) traversal of
    * a single fragment. The header of each buffer is
    * validated before the buffer pointer is advanced.
    *
    * The heap lock must be aquired for the duration of
    * iterative calls to this function. Any interleaved
    * heap access may make pointers invalid and result in
    * an error or access violation. */

   struct Buffer *buf = frag->root;
   HeapError ecode;
   
   if (unlikely(buf >= frag->sentinel))
      return NULL;

   /* Check integrity of next buddy. */
   ecode = buf_check_header(heap, a, frag->region, buf);
   if (unlikely(ecode != HEAP_ERROR_NONE))
   {
      zzheap_error(heap, ecode, &buf[1]);
      return NULL;
   }

   return buf;
}

/***************************************************************************/

static inline struct Buffer *
buf_right(struct Heap *heap, struct BuddyAlgorithm *a,
          struct Fragment *frag, struct Buffer *buf)
{
   /* Helper function for linear (non-tree) traversal of
    * a single fragment. The header of each buffer is
    * validated before the buffer pointer is advanced.
    *
    * The heap lock must be aquired for the duration of
    * iterative calls to this function. Any interleaved
    * heap access may make pointers invalid and result in
    * an error or access violation. */

   HeapError ecode;
   size_t size;

   size = buf_act_size(a, buf);
   buf  = (struct Buffer *)((char *)buf + size);
   
   if (buf >= frag->sentinel)
      return NULL;

   /* Check integrity of next buddy. */
   ecode = buf_check_header(heap, a, NULL, buf);
   if (unlikely(ecode != HEAP_ERROR_NONE))
   {
      zzheap_error(heap, ecode, &buf[1]);
      return NULL;
   }

   return buf;
}

/***************************************************************************/

static inline struct Buffer *
buf_next(struct Heap *heap, struct BuddyAlgorithm *a,
         struct BufferCursor *cursor, uint8_t advance_frag)
{
   /* Helper function for safe tree traversal of all the
    * algorithm's fragments. It uses a bitmask to find its
    * way to the next buddy. Each bit indicates the direction
    * to go (left/right). If the tree has changed between
    * two calls, a buffer may be skipped or repeated. 
    *
    * This function must not generate any errors since it
    * may be called from ose_heapd in response to signal
    * requests. OSE Optima, for example, may bring down the
    * entire system if an error or exception occurs.
    *
    * If 'advance_frag' is false, iteration will stop when
    * it reaches the end of a fragment. It is set to list
    * buffers in a single fragment only, rather than the
    * whole heap. A fragment to start with must therefore
    * also be provided in the cursor. 
    * */
   
   struct Region   *reg;
   struct Fragment *frag  = BUF_NEXT_FRAG(cursor->frag);
   uint32_t         path  = cursor->path;
   uint32_t         depth = cursor->depth;

   HeapError ecode;


   /* Locate the input fragment and make sure it still
    * exists. If it does, advance to the next fragment if
    * there are no more buffers in the current fragment. */
   if (unlikely(frag == NULL))
   {
      if (unlikely(!advance_frag))
         return NULL;

      /* Start with first buddy fragment. */
      reg = region_first(heap, a, &ecode);
      if (reg == NULL)
         return NULL;

      frag = frag_first(heap, reg, &ecode);
      if (frag == NULL)
         return NULL;

      path = 0;
   }
   else
   {
      reg = region_find(heap, a, frag, 0, &ecode);
      if (reg == NULL)
         return NULL;

      frag = frag_find(heap, reg, frag, 0, &ecode);
      if (frag == NULL)
         return NULL;

      /* Cursor->buf is valid as input only if
       * advance_frag is false. */
      if ((advance_frag || cursor->buf != NULL) &&
          !buf_next_path(&path, depth))
      {
         if (!advance_frag)
            return NULL;
         
         frag = frag_next(heap, frag, &ecode);
         if (frag == NULL)
         {
            reg = region_next(heap, reg, &ecode);
            if (reg != NULL)
               frag = frag_first(heap, reg, &ecode);
         }

         path = 0;
      }
   }

   while (frag != NULL)
   {
      for (;;)
      {
         struct Buffer *buf;

         depth = 0;
         
         buf = buf_from_path(heap, a, frag->root, frag->root_index, path, &depth);
         if (unlikely(buf == NULL)) /* No more buddies in fragment. */
            break;		    /* Try next fragment. */

         cursor->frag  = BUF_NEXT_MASK(&frag[1], a->id);
         cursor->path  = path;
         cursor->depth = depth;
         return buf;
      }

      if (unlikely(!advance_frag))
         return NULL;
      
      frag = frag_next(heap, frag, &ecode);
      if (frag != NULL)
      {
         path = 0;
         continue;
      }

      reg = region_next(heap, reg, &ecode);
      if (reg == NULL)
         return NULL;
               
      frag = frag_first(heap, reg, &ecode);
      path = 0;
   }

   return NULL;
}

/***************************************************************************/

static always_inline HeapError
buf_check_owner(struct Heap *heap, struct BuddyAlgorithm *a,
                struct Buffer *buf, HeapPID pid1)
{
   /* Validates that the pid passed as parameter is the
    * owner of the buffer. Private buffers can only be
    * operated on by its owner. */
   
   HeapPID pid2 = buf_owner(buf);

   (void)heap;
   (void)a;
   
   /* A buffer owner value of 0 (pid2) indicates that the
    * buffer is shared and can be operated on by anyone.
    * An originator pid value of 0 (pid1) means that the
    * operation was forced and should be allowed. */
   
   if (unlikely(pid1 != 0 && pid2 != 0 && pid1 != pid2))
      return HEAP_ERROR_BUF_OWNER;

   return HEAP_ERROR_NONE;
}

/***************************************************************************/

static inline HeapError
buf_check_frag(struct Heap *heap, struct BuddyAlgorithm *a,
               struct Region **reg, struct Buffer *buf)
{
   /* Validates that the buffer pointer resides in one
    * of the algorithm's fragments. If so, it should be
    * safe to access the address. */
   
   struct Region *region;
   HeapError ecode;

   /* Regions/fragments may disappear at any time due to
    * unbreak requests. Locks are needed. */

   region = region_find(heap, a, buf, 0, &ecode);
   if (likely(region != NULL))
   {
      *reg = region;
      return HEAP_ERROR_NONE;
   }

   *reg = NULL;
   
   if (unlikely(ecode != HEAP_ERROR_NONE))
      return ecode;
   
   return HEAP_ERROR_BUF_UNKNOWN;
}

/***************************************************************************/

static inline HeapError
buf_check_header(struct Heap *heap, struct BuddyAlgorithm *a,
                 struct Region *reg, struct Buffer *buf)
{
   /* Validate that the buffer header is reasonable.
    * Free buffers are validated through a checksum
    * of the entire header, including freelist links. 
    * It is stored in the owner field.
    *
    * The validation of allocated buffers is too weak
    * and needs improvment. 
    *
    * If a buffer is not reqognized by an algorithm,
    * the event hook must return the continue action
    * to allow other algorithms to identify it. If no
    * algorithm responds, i.e. all request type event
    * hooks return "continue", the buffer is likely
    * corrupted and the framework will generate an
    * error. */
   
   struct Fragment *frag = (struct Fragment *)buf;

   (void)heap;
   
   if (unlikely(((uintptr_t)buf) & (HEAP_ALIGNMENT-1)))
      return HEAP_ERROR_BUF_UNALIGNED;

   /* If the fragment is unknown, the ID must be verified. */
   if (unlikely(reg == NULL && buf_id(buf) != a->id))
      return HEAP_ERROR_BUF_UNKNOWN;
   /* If the fragment is known, the ID must also be. */
   else if (unlikely(reg != NULL && buf_id(buf) != a->id))
      return HEAP_ERROR_BUF_HEADER;

   if (unlikely(buf_ucheck(buf) != buf_get_ucheck(buf)))
      return HEAP_ERROR_BUF_HEADER;
   
   if (unlikely(buf_rbit(buf) && frag[-1].root != buf))
      return HEAP_ERROR_BUF_HEADER;

   if (buf_fbit(buf))
   {
      if (unlikely(buf_get_fcheck(buf) != buf_fcheck(buf)))
         return HEAP_ERROR_BUF_LINKS;

      if (unlikely(buf_index(buf) > SIZECOUNT))
         return HEAP_ERROR_BUF_HEADER;
   }
   else
   {
      if (unlikely(buf_size(buf) < sizeof(struct Buffer) + a->config->buf_size_endmark))
         return HEAP_ERROR_BUF_HEADER;
   }

   /* Check that size is reasonable. */
   if (unlikely(reg != NULL))
   {
      size_t size = buf_act_size(a, buf);

      if (unlikely(((uintptr_t)buf + size > (uintptr_t)reg->frag_last) ||
                   ((uintptr_t)buf + size < (uintptr_t)reg->frag_list)))
         return HEAP_ERROR_BUF_HEADER;
   }

   return HEAP_ERROR_NONE;
}

/***************************************************************************/

static always_inline HeapError
buf_check_endmark(struct Heap *heap, struct BuddyAlgorithm *a,
                  struct Region *reg, struct Buffer *buf)
{
   /* Validates that the buffer endmark, written
    * at the end of the user data area, is still valid
    * when the buffer is freed. Damages to the endmark
    * indicates that the user has used more memory than
    * it originally allocated or that the size field of
    * header has been damaged but not detected. The next
    * buffer's header may have been damaged as well. */
   
   char *endm = (char *)&buf[1] + buf_req_size(a, buf);
   int i = a->config->buf_size_endmark;

   (void)heap;
   
   /* There will be no endmark if the buffer is free. */
   if (unlikely(buf_fbit(buf)))
      return HEAP_ERROR_NONE;

   if (unlikely(reg != NULL))
   {
      if (unlikely((uint32_t)endm <  (uint32_t)buf ||
                   (uint32_t)endm >= (uint32_t)reg->frag_last))
         return HEAP_ERROR_BUF_HEADER;
   }
   
   while (--i>=0)
   {
      if (unlikely(endm[i] != HEAP_ENDMARK))
         return HEAP_ERROR_BUF_ENDMARK;
   }

   return HEAP_ERROR_NONE;
}

/***************************************************************************/

static always_inline void
buf_set_endmark(struct BuddyAlgorithm *a, struct Buffer *buf)
{
   /* Initializes the buffer endmark, which is used to
    * detect application bugs. If the endmark becomes
    * damaged the application has used more memory in
    * the buffer than it was allowed to. */
   
   char *endm = (char *)&buf[1] + buf_req_size(a, buf);
   uint32_t i = 0;

   for (i=0; i<a->config->buf_size_endmark; i++)
      endm[i] = HEAP_ENDMARK;
}

/***************************************************************************/

static inline HeapError
buf_check_at_free(struct Heap *heap, struct BuddyAlgorithm *a,
                  struct Buffer *buf, HeapPID pid)
{
   /* A collection of all integrity and error checks
    * that are carried out during a free request. The
    * function is also used for resize request which
    * behaves similarly. */

   struct Region *reg;
   HeapError status;
   
   if (unlikely(a->config->frag_check))
   {
      status = buf_check_frag(heap, a, &reg, &buf[1]);
      if (unlikely(status != HEAP_ERROR_NONE))
         return status;
   }
   else
   {
      /* Frag check is disabled, no region to validate against. */
      reg = NULL;
   }
   
   status = buf_check_header(heap, a, reg, buf);
   if (unlikely(status != HEAP_ERROR_NONE))
      return status;

   if (unlikely(buf_fbit(buf)))
      return HEAP_ERROR_BUF_FREE;

   status = buf_check_owner(heap, a, buf, pid);
   if (unlikely(status != HEAP_ERROR_NONE))
      return status;

   status = buf_check_endmark(heap, a, reg, buf);
   if (unlikely(status != HEAP_ERROR_NONE))
      return status;

   return HEAP_ERROR_NONE;
}

/***************************************************************************/

static inline HeapError
buf_check_at_query(struct Heap *heap, struct BuddyAlgorithm *a,
                   struct Region **reg_out, struct Buffer *buf)
{
   struct Region *reg = NULL;
   HeapError status;
   
   /* Frag check is optionally enabled for queries, unless in freeze-mode. */
   if (unlikely(a->config->frag_check || heap->freezed))
   {
      status = buf_check_frag(heap, a, &reg, &buf[1]);
      if (unlikely(status != HEAP_ERROR_NONE))
      {
         *reg_out = NULL;
         return status;
      }
   }

   *reg_out = reg;
   
   status = buf_check_header(heap, a, reg, buf);
   if (unlikely(status != HEAP_ERROR_NONE))
      return status;

   return HEAP_ERROR_NONE;
}

/***************************************************************************/

static inline HeapBufferStatus
buf_status(struct Heap *heap, struct BuddyAlgorithm *a,
           struct Region *reg, struct Buffer *buf)
{
   /* Helper function that investigates the health of
    * an individual buffer. */

   HeapBufferStatus status = 0;
   HeapError ecode;

   ecode = buf_check_header(heap, a, reg, buf);
   if (unlikely(ecode != HEAP_ERROR_NONE))
   {
      if (unlikely(ecode == HEAP_ERROR_BUF_UNALIGNED))
         return HEAP_BUFFER_UNKNOWN;

      if (unlikely(buf_annotated(buf)))
         return HEAP_BUFFER_HEADERS|HEAP_BUFFER_ANNOTATED;

      return HEAP_BUFFER_HEADERS;
   }
   
   if (!buf_fbit(buf))
      status |= HEAP_BUFFER_ALLOCATED;

   ecode = buf_check_endmark(heap, a, reg, buf);
   if (unlikely(ecode != HEAP_ERROR_NONE))
      status |= HEAP_BUFFER_ENDMARK;

   if (unlikely(buf_annotated(buf)))
      status |= HEAP_BUFFER_ANNOTATED;
   
   return status;
}

/***************************************************************************/

static always_inline void 
frag_init(struct BuddyAlgorithm *a, struct Fragment *frag, uint32_t index)
{
   /* Initializes a fragment's descriptor and the
    * fragment's root and sentinel buffers. */
   
   frag_set_control(frag, a->id, a->frag_id);

   /* Free root buffer. */
   frag->root = buf_init(a, &frag[1], index, 0);
   frag->root_index = index;
   buf_set_rbit(frag->root);

   /* Sentinel buffer */
   frag->sentinel = buf_init(a, (char*)&frag[1]+fibonacci[index], index-1, 0);
}

/***************************************************************************/

static always_inline HeapError
frag_check(struct Fragment *frag)
{
   /* Validates the integrity of a fragment header. */
   
   if (unlikely(frag == NULL))
      return HEAP_ERROR_NONE;
   
   if (unlikely(frag_checksum(frag) != frag_get_checksum(frag)))
      return HEAP_ERROR_CORRUPTED;

   return HEAP_ERROR_NONE;
}

/***************************************************************************/

static inline struct Fragment *
frag_find(struct Heap *heap, struct Region *reg, void *ptr,
          int exact, HeapError *ecode)
{
   /* Tries to find the fragment descriptor that contains the
    * ptr. If exact is set, the ptr must be equal to the
    * pointer of the fragment descriptor. */
   
   struct Fragment *frag = frag_first(heap, reg, ecode);
   while (frag != NULL)
   {
      if ((uintptr_t)ptr >= (uintptr_t)frag  &&
          (uintptr_t)ptr <  (uintptr_t)frag->sentinel)
      {
         if (exact && frag != ptr)
            return NULL;
         return frag;
      }

      frag = frag_next(heap, frag, ecode);
   }

   return NULL;
}

/***************************************************************************/

static inline struct Fragment *
frag_first(struct Heap *heap, struct Region *region, HeapError *ecode)
{
   /* Returns the first fragment in a region.
    * Fragment integrity is validated. */
   HeapError tmp_ecode;
   
   tmp_ecode = frag_check(region->frag_list);
   if (ecode != NULL)
      *ecode = tmp_ecode;

   if (unlikely(tmp_ecode != HEAP_ERROR_NONE))
   {
      if (ecode == NULL)
         zzheap_error(heap, tmp_ecode, region->frag_list);

      return NULL;
   }

   return region->frag_list;
}

/***************************************************************************/

static inline struct Fragment *
frag_next(struct Heap *heap, struct Fragment *frag, HeapError *ecode)
{
   /* Returns the next fragment in a region.
    * Fragment integrity is validated. */

   HeapError tmp_ecode;
   
   tmp_ecode = frag_check(frag);
   if (ecode != NULL)
      *ecode = tmp_ecode;

   if (unlikely(tmp_ecode != HEAP_ERROR_NONE))
   {
      if (ecode == NULL)
         zzheap_error(heap, tmp_ecode, frag);
      
      return NULL;
   }

   tmp_ecode = frag_check(frag->next);
   if (ecode != NULL)
      *ecode = tmp_ecode;

   if (unlikely(tmp_ecode != HEAP_ERROR_NONE))
   {
      if (ecode == NULL)
         zzheap_error(heap, tmp_ecode, frag->next);

      return NULL;
   }


   return frag->next;
}

/***************************************************************************/

static always_inline struct Fragment *
frag_from_buf(struct Heap *heap, struct Buffer *buf)
{
   /* The function locates the fragment descriptor
    * that is associated with a root buffer.
    * It is assumed that the caller of this function
    * already has verified that the buffer has the
    * root buffer bit set in its header. 
    */
   
   struct Fragment *f = (struct Fragment *)buf;
   
   f = &f[-1];

   if (unlikely(f->root == buf))
      return f;
   
   /* The fragment descriptor has an invalid root pointer.
    * The heap has been corrupted if the root bit is set
    * in the buffer header. This should already have been
    * checked by the caller, but check it again just to
    * make sure. */
   if (unlikely(buf_rbit(buf)))
      zzheap_error(heap, HEAP_ERROR_CORRUPTED, f);

   return NULL;
}

/***************************************************************************/

static always_inline HeapError
region_check(struct Region *region)
{
   /* Validates the integrity of a region. */

   if (unlikely(region == NULL))
      return HEAP_ERROR_NONE;
   
   if (unlikely(region_checksum(region) != region_get_checksum(region)))
      return HEAP_ERROR_CORRUPTED;
   
   return HEAP_ERROR_NONE;
}

/***************************************************************************/

static always_inline void 
region_init(struct BuddyAlgorithm *a, struct Region *reg, uint8_t init)
{
   /* Initializes a region descriptor. */
   
   region_set_control(reg, a->id);
   if (init)
      region_set_initial(reg);
   reg->frag_list  = NULL;
   reg->frag_last  = NULL;
   reg->frag_count = 0;
}

/***************************************************************************/

static always_inline struct Region *
region_from_buf(struct Heap *heap, struct Buffer *buf)
{
   /* Locates the region descriptor that is associated
    * with a buffer. The region descriptor can be used
    * to determine if the entire region is unused and
    * can be returned to the allocator. */
      
   struct Fragment *f = frag_from_buf(heap, buf);

   return f != NULL ? f->region : NULL;
}

/***************************************************************************/

static always_inline int
region_is_free(struct Heap *heap, struct Region *reg)
{
   /* Traverses all fragments in a region to determine
    * if all fragments are unused. If so, the region can
    * be deallocated using zzheap_unbreak. */
   
   struct Fragment *f;
   HeapError ecode = HEAP_ERROR_NONE;

   /* The initial region can not be reclaimed. */
   if (region_initial(reg))
      return 0;
   
   f = frag_first(heap, reg, &ecode);
   while (f != NULL)
   {
      if (!buf_fbit(f->root) || f->root_index != buf_index(f->root))
         return 0;

      f = frag_next(heap, f, &ecode);
   }

   if (unlikely(ecode != HEAP_ERROR_NONE))
   {
      zzheap_error(heap, ecode, reg);
      return 0;
   }
   
   return 1;
}

/***************************************************************************/

static inline struct Region *
region_first(struct Heap *heap, struct BuddyAlgorithm *a, HeapError *ecode)
{
   HeapError tmp_ecode = region_check(a->region_list);

   if (ecode != NULL)
      *ecode = tmp_ecode;

   if (unlikely(tmp_ecode != HEAP_ERROR_NONE))
   {
      if (ecode == NULL)
         zzheap_error(heap, tmp_ecode, a->region_list);

      return NULL;
   }

   return a->region_list;
}

/***************************************************************************/

static inline struct Region *
region_next(struct Heap *heap, struct Region *reg, HeapError *ecode)
{
   HeapError tmp_ecode = region_check(reg->next);

   if (ecode != NULL)
      *ecode = tmp_ecode;

   if (unlikely(tmp_ecode != HEAP_ERROR_NONE))
   {
      if (ecode == NULL)
         zzheap_error(heap, tmp_ecode, reg->next);

      return NULL;
   }

   return reg->next;
}

/***************************************************************************/

static always_inline void
frag_insert(struct Region *reg, struct Fragment *f)
{
   /* Registers a fragment descriptor with a region to allow
    * traversal of the fragments. Note that the frag_last field
    * of the region descriptor will become invalid as soon as the
    * region is registered with the heap. Once the region is
    * registered, no more fragments can be added.
    *
    * Fragments must be inserted in the order of increasing addresses,
    * so that frag_last points to the fragment with highest address
    * when the region is registered with the heap. 
    */
   
   f->prev = reg->frag_last;
   f->next = NULL;
   if (f->prev)
   {
      f->prev->next = f;
      frag_set_checksum(f->prev);
   }
   if (reg->frag_list == NULL)
      reg->frag_list = f;
   reg->frag_last = f;
   reg->frag_count++;

   f->region = reg;
   frag_set_checksum(f);
}

/***************************************************************************/

static inline void
region_insert(struct Heap *heap, struct BuddyAlgorithm *a, struct Region *reg)
{
   /* Registers a region descriptor with the heap. The region
    * is inserted into a list and all root buffers within fragments
    * of the region are inserted into the heap freelists.
    *
    * The frag_last field of the descriptor is changed to point at
    * the end of the last fragment (instead of the beginning).
    * The field is used by integrity check methods to determine
    * if pointers reside inside the region or not. */
   
   struct Fragment *f;
   HeapLock lock;

   heap_assert(reg->frag_count != 0 &&
               reg->frag_list  != NULL &&
               reg->frag_last  != NULL,
               "Attempted to insert region with no fragments.");


   heap_lock(heap, &lock, 0);
   
   reg->prev = NULL;
   reg->next = a->region_list;
   
   if (reg->next)
   {
      HeapError ecode;
      ecode = region_check(reg->next);
      if (ecode != HEAP_ERROR_NONE)
      {
         heap_unlock(heap, &lock, 0);
         zzheap_error(heap, HEAP_ERROR_FATAL|ecode, reg->next);
         return;
      }

      reg->next->prev = reg;
      region_set_checksum(reg->next);
   }

   a->region_list = reg;
   a->region_count++;
   a->frag_count += reg->frag_count;

   /* Adjust frag_last to point to end of last fragment,
    * optimization to speed up searches. */ 
   reg->frag_last = (struct Fragment *)reg->frag_last->sentinel;

   region_set_checksum(reg);
   
   /* Add all buffers in the region to the heap. */
   f = reg->frag_list;
   while (f != NULL)
   {
      a->stats->buf_num_used[f->root_index]++;
      buf_insert(heap, a, f->root);

      f = f->next;
   }

   heap_unlock(heap, &lock, 0);
}

/***************************************************************************/

static inline void
region_remove(struct Heap *heap, struct BuddyAlgorithm *a, struct Region *reg)
{
   /* Removes/unregisters a region from the heap. All
    * root buffers in the region are removed from freelists.
    * It is assumed that the caller has verified that the
    * region can be removed, i.e. that no buffer is allocated.
    */
   
   struct Fragment *f;
   HeapError ecode = HEAP_ERROR_NONE;
   
   /* Unlink region descriptor. */
   if (reg->prev)
   {
      ecode = region_check(reg->prev);
      if (unlikely(ecode != HEAP_ERROR_NONE))
      {
         zzheap_error(heap, HEAP_ERROR_FATAL|ecode, reg->prev);
         return;
      }

      reg->prev->next = reg->next;
      region_set_checksum(reg->prev);
   }

   if (reg->next)
   {
      ecode = region_check(reg->next);
      if (unlikely(ecode != HEAP_ERROR_NONE))
      {
         zzheap_error(heap, HEAP_ERROR_FATAL|ecode, reg->next);
         return;
      }

      reg->next->prev = reg->prev;
      region_set_checksum(reg->next);
   }

   if (a->region_list == reg)
      a->region_list = reg->next;

   reg->next = reg->prev = NULL;

   a->region_count--;
   a->frag_count -= reg->frag_count;

   /* Remove all buffers from the heap. */
   f = frag_first(heap, reg, &ecode);
   while (f != NULL)
   {
      buf_remove(heap, a, f->root);
      a->stats->buf_num_used[f->root_index]--;

      f = frag_next(heap, f, &ecode);
   }

   if (unlikely(ecode != HEAP_ERROR_NONE))
      zzheap_error(heap, HEAP_ERROR_FATAL|ecode, f);
}

/***************************************************************************/

static inline struct Region *
region_find(struct Heap *heap, struct BuddyAlgorithm *a,
            void *ptr, uint32_t exact, HeapError *ecode)
{
   /* Tries to find the region descriptor that contains the
    * ptr. If exact is set, the ptr must be equal to the
    * pointer of the region descriptor. */

   struct Region *reg = region_first(heap, a, ecode);

   while (reg != NULL)
   {
      if ((uintptr_t)ptr >= (uintptr_t)reg &&
          (uintptr_t)ptr <  (uintptr_t)reg->frag_last)
      {
         if (exact && ptr != reg)
            return NULL;
         return reg;
      }
      
      reg = region_next(heap, reg, ecode);
   }
      
   return NULL;
}

/**************************************************************************/

static inline void
region_reclaim(struct Heap *heap, struct BuddyAlgorithm *a,
               union HeapEvent *event, struct Region *region)
{
   HeapError ecode;
   HeapLock lock;

   /* Interrupt and timer interrupt processes
      are not allowed to do return regions. */
   if (!heap_brkallowed()) {
      return;
   }

   /* Grab the break lock forcibly to prevent new extension attempts. */
   heap_brklock(heap, &lock);

   /* Grab general heap lock to protect freelists, etc. */
   heap_lock(heap, &lock, 0);

   /* Check if the region still exists and if it is free. */
   if (region != NULL
       && region_find(heap, a, region, 1, &ecode)
       && !region_locked(region)
       && region_is_free(heap, region))
   {
      /* The region can be safely deleted.*/

      /* Remove region and buffers from freelists.. */
      region_remove(heap, a, region);

      /* Unlock freelists. */
      heap_unlock(heap, &lock, 0);

      /* Delete region. */
      zzheap_unbreak(heap, region, event ? event->e_free.params : NULL);
   }
   else
   {
      heap_unlock(heap, &lock, 0);
   }

   heap_brkunlock(heap, &lock);
}

/***************************************************************************/

#ifdef ASSERT
static always_inline struct Region *
REGION_ALIGN(struct Region *reg)
{
   return (struct Region *)
      (((uintptr_t)(reg)+HEAP_ALIGNMENT-1) & ~(HEAP_ALIGNMENT-1));
}
#endif

/***************************************************************************/







/***************************************************************************/

static uint32_t
l_frag_break(struct Heap *heap, struct BuddyAlgorithm *a,
             HeapParam *params, size_t size, int init)
{
   struct Fragment *frag;
   struct Region *reg;

   size_t shead =
      sizeof(struct Region) +
      sizeof(struct Fragment) +
      sizeof(struct Buffer);
   size_t smax = 0; /* No limit on maximum fragment size. */
   size_t smin;     /* Minimum fragment size.             */
   size_t sreq;     /* Preferred fragment size.           */

   if (likely(!init))
   {
      /* Align to valid fibonacci size. */
      if (likely(size < SIZEMAX))
         size = fibonacci[size_lindex(size)];

      /* Add overhead size: fragment header + sentinel buffer */
      size += shead;
   }

   smin = size;
   sreq = size;

   /* Align to minimum extension size. */
   if (!init && sreq < a->config->frag_size_default)
      sreq = a->config->frag_size_default;

   reg = zzheap_break(heap, &smin, sreq, &smax, a->id, params);
   if (unlikely(reg == NULL))
   {
      /* Break failed. Possible explainations:
       *    1) No more system memory
       *    2) HEAP_MAX_SIZE was reached.
       */
      return 0;
   }
   else
   {
      /* Allocation was successful. Update size with
       * the size of the fragment that was acquired. */
      size = smin;
   }

   heap_assert(reg == REGION_ALIGN(reg),
               "Unaligned fragment detected.");

   size -= sizeof(struct Region);
   frag  = (struct Fragment *)&reg[1];

   region_init(a, reg, init);

   while (size >= shead + fibonacci[0])
   {
      uint32_t index;

      /* Make room for fragment header and sentinel buffer. */
      size -= shead;
      index = size_sindex(size);

      if (!index)
         break;

      /* Initialize fragment and buffers */
      frag_init(a, frag, index);

      /* Register the fragment within the region. */
      frag_insert(reg, frag);

      size -= fibonacci[index];
      frag  = (struct Fragment *) ((char *)frag + shead + fibonacci[index]);
   }
   
   /* Register the region and all its buffers in the heap. */
   region_insert(heap, a, reg);

   return 1;
}

/***************************************************************************/

static HeapAction
l_buddy_alloc(struct Heap *heap, union HeapEvent *event, void *opaque)
{
   struct BuddyAlgorithm *a = opaque;
   struct Buffer *buf = NULL;
   size_t size = event->e_alloc.size;
   size_t act_size;
   uint32_t ix;

   size += sizeof(struct Buffer) + a->config->buf_size_endmark;
   
   if (unlikely(size > a->config->buf_size_threshold))
      return HEAP_ACTION_CONTINUE;

   ix = size_lindex(size);

   
   do
   {
      HeapLock lock;

      heap_lock(heap, &lock, 0);

      /* Enforce HEAP_MAX_SIZE. */
      if (unlikely(fibonacci[ix] + a->stats->buf_size_used > a->config->buf_size_max))
      {
         heap_unlock(heap, &lock, 0);
         return HEAP_ACTION_CONTINUE;
      }

      /* Locate a free buffer of equal or larger index. */
      buf = buf_find(heap, a, ix);
      if (unlikely(buf == NULL))
      {
         heap_brkprepare(heap);
         heap_unlock(heap, &lock, 0);

         if (!heap_brkallowed())
         {
            return HEAP_ACTION_CONTINUE;
         }

         if (!heap_brklock(heap, &lock))
         {
            heap_brkunlock(heap, &lock);
            continue;
         }
         
         if (unlikely(!l_frag_break(heap, a, event->e_alloc.params, size, 0)))
         {
            heap_brkunlock(heap, &lock);
            return HEAP_ACTION_CONTINUE;
         }

         heap_brkunlock(heap, &lock);
         continue;
      }

      /* Grab buffer. */
      buf_remove(heap, a, buf);

      buf = buf_split(heap, a, buf, ix);

      buf_set_asize(buf, size);
      buf_set_owner(buf, event->e_alloc.owner);
      buf_set_endmark(a, buf);

      /* Update statistics. */
      a->stats->buf_size_used += (act_size = buf_act_size(a, buf));
      a->stats->buf_size_req  += buf_req_size(a, buf);

      if (a->stats->buf_size_used > a->stats->buf_size_used_hi)
         a->stats->buf_size_used_hi = a->stats->buf_size_used;

      if (buf_owner(buf))
         a->stats->buf_num_private += 1;
      else
         a->stats->buf_num_shared += 1;

      buf_set_check(buf);
      
      heap_unlock(heap, &lock, 0);
   } while (unlikely(buf == NULL));

   
   event->e_alloc.ptr  = &buf[1];
   event->e_alloc.size = act_size;
   
   return HEAP_ACTION_INTERCEPT;
}


/***************************************************************************/

static HeapAction
l_buddy_free(struct Heap *heap, union HeapEvent *event, void *opaque)
{
   struct BuddyAlgorithm *a = opaque;
   struct Buffer *buf = event->e_free.ptr;
   size_t act_size;
   HeapError status;
   HeapLock lock;
   
   if (unlikely(buf == NULL))
      return HEAP_ACTION_CONTINUE;

   buf = &buf[-1];

   /* Dummy access to catch bad pointers before locking and
    * potentially entering into kill locked state. */
   (void)*(volatile uint32_t *)buf;
   
   heap_lock(heap, &lock, 0);

   status = buf_check_at_free(heap, a, buf, event->e_free.owner);
   if (unlikely(status == HEAP_ERROR_BUF_UNKNOWN))
   {
      heap_unlock(heap, &lock, 0);
      return HEAP_ACTION_CONTINUE;
   }

   if (unlikely(status != HEAP_ERROR_NONE))
   {
      heap_unlock(heap, &lock, 0);

      zzheap_error(heap, status, &buf[1]);
      return HEAP_ACTION_INTERCEPT;
   }

   if (buf_gc(buf))
   {
      /* The buffer is used by the garbage collector and
       * can't be freed. We simply clear the bit to indicate
       * that we attempted to free it but couldn't. The collector
       * will notice and free the buffer for us. */
      buf_clear_gc(buf);
      heap_unlock(heap, &lock, 0);

      return HEAP_ACTION_INTERCEPT;
   }
  
   /* Update statistics. */
   a->stats->buf_size_used -= (act_size = buf_act_size(a,buf));
   a->stats->buf_size_req  -= buf_req_size(a,buf);
   if (buf_owner(buf))
      a->stats->buf_num_private -= 1;
   else
      a->stats->buf_num_shared -= 1;

   buf_clear_anno(buf);
   buf_restore_aindex(buf);
   
   /* Recursive merge. */
   buf = buf_merge(heap, a, buf);

   buf_insert(heap, a, buf);

   /* If this is the root buffer, try to unbreak entire region. */
   if (unlikely(a->config->frag_reclaim && buf_rbit(buf)))
   {
      struct Region *region = region_from_buf(heap, buf);

      /* Deleting the region requires the heap break lock
       * which can't be locked if the regular heap lock
       * is already locked. The regular lock is therefore
       * unlocked and reacquired in reclaim_region() after
       * grabbing the break lock. */
      heap_unlock(heap, &lock, 0);

      if (region != NULL) {
         region_reclaim(heap, a, event, region);
      }
   } else {
      heap_unlock(heap, &lock, 0);
   }

   event->e_free.size = act_size;

   return HEAP_ACTION_INTERCEPT;
}

/***************************************************************************/
   
static HeapAction
l_buddy_resize(struct Heap *heap, union HeapEvent *event, void *opaque)
{
   struct BuddyAlgorithm *a = opaque;
   struct Buffer *buf = event->e_resize.ptr, *right = NULL;

   size_t req_size = event->e_resize.size +
      sizeof(struct Buffer) + a->config->buf_size_endmark;
   uint32_t req_ix, old_ix;
   size_t old_req, old_act, new_act;

   HeapError status;
   HeapLock lock;

   if (unlikely(buf == NULL))
      return HEAP_ACTION_CONTINUE;

   buf = &buf[-1];

   /* Dummy access to catch bad pointers before locking and
    * potentially entering into kill locked state. */
   (void)*(volatile uint32_t *)buf;

   heap_lock(heap, &lock, 0);
   
   status = buf_check_at_free(heap, a, buf, event->e_free.owner);
   if (unlikely(status == HEAP_ERROR_BUF_UNKNOWN))
   {
      heap_unlock(heap, &lock, 0);
      return HEAP_ACTION_CONTINUE;
   }

   if (unlikely(status != HEAP_ERROR_NONE))
   {
      heap_unlock(heap, &lock, 0);

      zzheap_error(heap, status, &buf[1]);
      event->e_resize.ptr = NULL;
      return HEAP_ACTION_INTERCEPT;
   }

   if (unlikely(req_size > a->config->buf_size_threshold))
   {
      /* The buffer belongs to this algorithm, but the new size cannot
       * be accomodated. Intercept request and force framework to copy. */
      event->e_resize.ptr = NULL;
      return HEAP_ACTION_INTERCEPT;
   }

   req_ix   = size_lindex(req_size);
   old_ix   = buf_aindex(buf);
   old_req  = buf_req_size(a, buf);
   old_act  = buf_act_size(a, buf);
  
   if (unlikely(fibonacci[req_ix] + a->stats->buf_size_used > a->config->buf_size_max))
   {
      heap_unlock(heap, &lock, 0);
      event->e_resize.ptr = NULL;
      return HEAP_ACTION_INTERCEPT;
   }

   buf_restore_aindex(buf);

   if (req_ix < 2 && old_ix < 2)
   {
      /* Do nothing. Cannot split buddies with
       * index 0 or 1 any further. */
   }
   else if (req_ix > old_ix)
   {
      struct Buffer *left = buf;
      left = buf_merge_right(heap, a, buf, req_ix);
      
      if (left == NULL)
         event->e_resize.ptr = NULL;
      else
      {
         heap_assert(buf_index(buf) == req_ix,
                     "Internal error in buddy merge logic.");
      }
   }
   else if (req_ix < old_ix)
   {
      right = buf_split_right(heap, a, buf, req_ix);
      if (right == NULL)
         event->e_resize.ptr = NULL;
      else
      {
         heap_assert(buf_index(buf) == req_ix,
                     "Internal error in buddy split logic.");
      }
   }

   if (event->e_resize.ptr != NULL)
   {
      a->stats->buf_size_req  -= old_req;
      a->stats->buf_size_used -= old_act;

      buf_set_asize(buf, req_size);
      buf_set_endmark(a, buf);
      /* The owner field remains unchanged */

      a->stats->buf_size_req  += buf_req_size(a, buf);
      a->stats->buf_size_used += (new_act = buf_act_size(a, buf));

      event->e_resize.size = new_act;
      event->e_resize.size_prev = old_act;
   }
   else
   {
      buf_set_asize(buf, old_req +
                    sizeof(struct Buffer) +
                    a->config->buf_size_endmark);
   }

   buf_set_check(buf);
   heap_unlock(heap, &lock, 0);
   
   return HEAP_ACTION_INTERCEPT;
}

/***************************************************************************/

static void
buf_collect(struct BuddyAlgorithm *a, struct Buffer **head, struct Buffer *buf)
{
   /* This function links the buffer into
    * a list of all buffers that will be
    * freed by the garbage collector at a
    * later point in time.
    *
    * The buffer is also resized to make
    * room for a pointer in the user data
    * area. The pointer will point to the
    * next buffer in the list of collected
    * buffers. The endmark is moved forward. */

   const size_t oh =
      sizeof(struct Buffer) +
      sizeof(struct Buffer*) + 
      a->config->buf_size_endmark;

   size_t size = buf_size(buf);

   if (size < oh)
   {
      a->stats->buf_size_req += oh - size;

      buf_set_index(buf, oh);
      buf_set_endmark(a, buf);
   }

   buf[1].owner = (HeapPID)*head;
   *head = buf;
}

/**************************************************************************/

static HeapAction
l_buddy_collect(struct Heap *heap, union HeapEvent *event, void *opaque)
{
   HeapPID pid = event->e_collect.owner;
   struct BuddyAlgorithm *a = opaque;

   struct Region *reg;
   struct Fragment *frag;
   struct Buffer *buf;
   HeapLock lock;

   heap_lock(heap, &lock, 0);

   reg = region_first(heap, a, NULL);
   while (reg != NULL)
   {
      /* Prevent reclaim while traversing this region.
       * The minimize interrupt latency, the region
       * is locked which means that it cannot be unbroken
       * if it happens to become unused during traversal.
       * Allows us to traverse it without locks taken. */
      region_set_lock(reg);
      heap_unlock(heap, &lock, 0);

      frag = frag_first(heap, reg, NULL);
      while (frag != NULL)
      {
         struct Buffer *head = NULL;
         uint32_t i;

         /* Mark the first buffer as a hostage of the collector.
          * Attempts to free the buffer will be detected by
          * the collector once heap locks are reacquired. */
         heap_lock(heap, &lock, 0);
         buf = buf_first(heap, a, frag);

         /* Collect and gather all leaked buffers in a list. */
         do {

            /* Work through a hundred buffers at a
             * time before releasing the heap lock. */
            for (i=0; i<100 && buf != NULL; i++)
            {
               if (!buf_fbit(buf) && buf_owner(buf) == pid)
               {
                  buf_collect(a, &head, buf);
               }
               
               buf = buf_right(heap, a, frag, buf);
            }

            /* Scan ahead to the next used buffer.
             * The lock can only be released if.....*/
            while (buf != NULL && buf_fbit(buf))
            {
               buf = buf_right(heap, a, frag, buf);
            }

            if (buf == NULL)
            {
               break;
            }

            /* Mark the buffer as a hostage of the collector.
             * Attempts to free the buffer will be detected by
             * the collector once heap locks are reacquired. */
            buf_set_gc(buf);
            
            heap_unlock(heap, &lock, 0);
            heap_lock(heap, &lock, 0);

            if (!buf_gc(buf))
            {
               buf_collect(a, &head, buf);
               buf = buf_right(heap, a, frag, buf);
            }
            else
            {
               buf_clear_gc(buf);
            }
            
         } while (buf != NULL);

         heap_unlock(heap, &lock, 0);


         /* Now free all buffers in the list. */
         while (head != NULL)
         {
            buf = head;
            head = (struct Buffer *)buf[1].owner;
            
            zzheap_free(heap, &buf[1], 0, NULL, NULL, 0);
         }
         
         frag = frag_next(heap, frag, NULL);
      }
      
      /* Advance to next region. */
      heap_lock(heap, &lock, 0);
      region_clear_lock(reg);

      /* Check if the region became free. Unbreak it if possible. */
      if (unlikely(a->config->frag_reclaim))
      {
         struct Region *tmp = region_next(heap, reg, NULL);
         if (tmp != NULL)
            region_set_lock(tmp);

         /* Deleting the region requires the heap break lock
          * which can't be locked if the regular heap lock
          * is already locked. The regular lock is therefore
          * unlocked and reacquired in reclaim_region() after
          * grabbing the break lock. */
         heap_unlock(heap, &lock, 0);

         region_reclaim(heap, a, NULL, reg);
         
         heap_lock(heap, &lock, 0);
         reg = tmp;
      }
      else
      {
         reg = region_next(heap, reg, NULL);
      }
   }

   heap_unlock(heap, &lock, 0);

   return HEAP_ACTION_CONTINUE;
}

/***************************************************************************/

static HeapAction
l_buddy_query_req(struct Heap *heap, union HeapEvent *event, void *opaque)
{
   HeapParam *p = event->e_query.params;
   struct BuddyAlgorithm *a = opaque;
   HeapLock lock;

   if (p == NULL)
      return HEAP_ACTION_CONTINUE;
   
   if (event->e_query.freezed)
      heap->freezed++;

   while (*p != HPM_END)
   {
      switch (HPM_ID(*p))
      {
         case HPM_QUERY_SERVICE:
         {
            if (!HPM_IS_HANDLED(p[0]) && p[1] == 0)
            {
               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], &a->info);
            }
            else if (p[1] == (HeapParam)&a->info)
            {
               HPM_CLEAR_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], 0);
            }

            break;
         }
         case HPM_QUERY_BUF_ACT_SIZE:
         {
            struct Region *reg;
            struct Buffer *buf = &((struct Buffer *)p[1])[-1];
            
            if (!HPM_IS_HANDLED(p[0]))
            {
               heap_lock(heap, &lock, 0);
               
               if (buf_check_at_query(heap, a, &reg, buf) != HEAP_ERROR_NONE)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], buf_act_size(a, buf));

               heap_unlock(heap, &lock, 0);
            }
            break;
         }
         case HPM_QUERY_BUF_REQ_SIZE:
         {
            struct Region *reg;
            struct Buffer *buf = &((struct Buffer *)p[1])[-1];
           
            if (!HPM_IS_HANDLED(p[0]))
            {
               heap_lock(heap, &lock, 0);

               if (buf_check_at_query(heap, a, &reg, buf) != HEAP_ERROR_NONE)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               HPM_SET_HANDLED(p[0]);
               if (!buf_fbit(buf))
                  HPM_SET_VALUE(p[1], buf_req_size(a, buf));
               else
                  HPM_SET_VALUE(p[1], 0);

               heap_unlock(heap, &lock, 0);
            }
            break;
         }
         case HPM_QUERY_BUF_OWNER:
         {
            struct Region *reg;
            struct Buffer *buf = &((struct Buffer *)p[1])[-1];

            if (!HPM_IS_HANDLED(p[0]))
            {
               heap_lock(heap, &lock, 0);

               if (buf_check_at_query(heap, a, &reg, buf) != HEAP_ERROR_NONE)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }
            
               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], buf->owner);

               heap_unlock(heap, &lock, 0);
            }
            break;
         }
         case HPM_QUERY_BUF_TOT_REQ_SIZE:
         { 
            if (!HPM_IS_HANDLED(p[0]))
            {
               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], a->stats->buf_size_req);
            }
            break;
         }
         case HPM_QUERY_BUF_TOT_USED_SIZE:
         { 
            if (!HPM_IS_HANDLED(p[0]))
            {
               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], a->stats->buf_size_used);
            }
            break;
         }
         case HPM_QUERY_BUF_MAX_USED_SIZE:
         { 
            if (!HPM_IS_HANDLED(p[0]))
            {
               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], a->stats->buf_size_used_hi);
            }
            break;
         }
         case HPM_QUERY_BUF_TOT_FREE_SIZE:
         {
            if (!HPM_IS_HANDLED(p[0]))
            {
               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], a->stats->buf_size_free);
            }
            
            break;
         }
         case HPM_QUERY_BUF_NUM_SIZE:
         {
            uint32_t index = (uint32_t)p[1] + 1;
            
            if (index < SIZECOUNT)
            {
               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[2], fibonacci[index]);
            }
            break;
         }
         case HPM_QUERY_BUF_NUM_FREE:
         {
            uint32_t index = (uint32_t)p[1] + 1;

            if (index < SIZECOUNT)
            {
               HPM_SET_HANDLED(p[0]);
               
               /* Summarize 16 byte indexes. */
               if (index == 1)
                  HPM_ADD_VALUE(p[2], a->stats->buf_num_free[0]);

               HPM_ADD_VALUE(p[2], a->stats->buf_num_free[index]);
            }
            break;
         }
         case HPM_QUERY_BUF_NUM_USED:
         {
            uint32_t index = (uint32_t)p[1] + 1;
            
            if (index < SIZECOUNT)
            {
               HPM_SET_HANDLED(p[0]);

               /* Summarize 16 byte indexes. */
               if (index == 1)
                  HPM_ADD_VALUE(p[2], a->stats->buf_num_used[0]);

               HPM_ADD_VALUE(p[2], a->stats->buf_num_used[index]);
            }
            break;
         }
         case HPM_QUERY_BUF_FRAG_NEXT:
         {
            if (!HPM_IS_HANDLED(p[0]))
            {
               struct BufferCursor *cursor = (struct BufferCursor *)&p[1];
               struct Buffer *buf;

               heap_lock(heap, &lock, 0);
               
               buf = buf_next(heap, a, cursor, 0);
               if (buf == NULL)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], &buf[1]);

               heap_unlock(heap, &lock, 0);
            }
            break;
         }
         case HPM_QUERY_BUF_NEXT:
         {
            /* This algorithm stores fragments in parameter 2. 
             * It doesn't matter if p[2] is a fragment or a buffer since both
             * headers have the algorithm identifier in the same bits.
             * Algorithms may choose which of the two it wants to use for
             * buffer traversals. */

            /* Skip if the request doesn't concern us. */
            if (p[2] != 0 && BUF_NEXT_ID(p[2]) != a->id)
               break;

            if (!HPM_IS_HANDLED(p[0]))
            {
               struct BufferCursor *cursor = (struct BufferCursor *)&p[1];
               struct Buffer *buf;

               heap_lock(heap, &lock, 0);
               
               buf = buf_next(heap, a, cursor, 1);
               if (buf == NULL)
               {
                  /* Initialize query for next algorithm. */
                  cursor->frag  = NULL;
                  cursor->path  = 0;
                  cursor->depth = 0;
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], &buf[1]);

               heap_unlock(heap, &lock, 0);
            }
            break;
         }
         case HPM_QUERY_BUF_TYPE:
         {
            struct Region *reg;
            struct Buffer *buf = &((struct Buffer *)p[1])[-1];

            if (!HPM_IS_HANDLED(p[0]))
            {
               heap_lock(heap, &lock, 0);

               if (buf_check_at_query(heap, a, &reg, buf) != HEAP_ERROR_NONE)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], HEAP_BUFFER_FBUDDY);

               heap_unlock(heap, &lock, 0);
            }
            break;
         }
         case HPM_QUERY_BUF_STATUS:
         {
            struct Region *reg;
            struct Buffer *buf = &((struct Buffer *)p[1])[-1];
            HeapError ecode;

            if (!HPM_IS_HANDLED(p[0]))
            {
               heap_lock(heap, &lock, 0);

               ecode = buf_check_at_query(heap, a, &reg, buf);
               if (ecode == HEAP_ERROR_BUF_UNKNOWN)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], buf_status(heap, a, reg, buf));

               heap_unlock(heap, &lock, 0);
            }
            break;
         }
         case HPM_QUERY_BUF_DATA:
         {
            struct Region *reg;
            struct Buffer *buf = &((struct Buffer *)p[1])[-1];

            if (!HPM_IS_HANDLED(p[0]))
            {
               void  *data = (void *)p[2];
               size_t size = (size_t)p[3];

               heap_lock(heap, &lock, 0);

               if (buf_check_at_query(heap, a, &reg, buf) != HEAP_ERROR_NONE)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               HPM_SET_HANDLED(p[0]);
               memcpy(data, &buf[1], size);

               heap_unlock(heap, &lock, 0);
            }
            break;
         }
         case HPM_QUERY_BUF_LARGEST_FREE:
         {
            int ix = 0;
            size_t bitmap = a->bitmap, largest;

            /* Lazy bit-search */
            while (bitmap <<= 1)
               ix++;

            if (!a->bitmap)
               largest = 0;
            else
               largest = fibonacci[ix];

            if (HPM_IS_HANDLED(p[0]))
            {
               if (largest > (size_t)p[1])
               {
                  HPM_SET_VALUE(p[1], largest);
               }
            }
            else
            {
               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], largest);
            }
            break;
         }
         case HPM_QUERY_BUF_NEXT_SIZE:
         {
            if (!HPM_IS_HANDLED(p[0]))
            {
               size_t s = p[1] + sizeof(struct Buffer) + a->config->buf_size_endmark;

               /* Outside the range of this algorithm? */
               if (s > a->config->buf_size_threshold)
                  break;

               s  = fibonacci[size_lindex(s)];
               s -= (sizeof(struct Buffer) + a->config->buf_size_endmark);

               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], s);
            }
            break;
         }
         case HPM_QUERY_BUF_THRESHOLD:
         {
            if (!HPM_IS_HANDLED(p[0]))
            {
               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], a->config->buf_size_threshold);
            }
            break;
         }
         case HPM_QUERY_BUF_MAX_SIZE:
         {
            if (!HPM_IS_HANDLED(p[0]))
            {
               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], a->config->buf_size_max);
            }
            break;
         }
         case HPM_QUERY_BUF_ENDMARK:
         {
            if (!HPM_IS_HANDLED(p[0]))
            {
               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], a->config->buf_size_endmark);
            }
            break;
         }
         case HPM_QUERY_BUF_NUM_SHARED:
         {
            HPM_SET_HANDLED(p[0]);
            HPM_SET_VALUE(p[1], a->stats->buf_num_shared);
            break;
         }
         case HPM_QUERY_BUF_NUM_PRIVATE:
         {
            HPM_SET_HANDLED(p[0]);
            HPM_SET_VALUE(p[1], a->stats->buf_num_private);
            break;
         }
         case HPM_QUERY_FRAG_SIZE:
         {
            HeapError ecode;
            struct Fragment *frag = &((struct Fragment *)p[1])[-1];
            struct Region *region;

            if (!HPM_IS_HANDLED(p[0]))
            {
               HeapLock lock;
               
               heap_lock(heap, &lock, 0);
               
               region = region_find(heap, a, frag, 0, &ecode);
               if (region == NULL)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               frag = frag_find(heap, region, frag, 1, &ecode);
               if (frag == NULL)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               ecode = frag_check(frag);
               if (ecode != HEAP_ERROR_NONE)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[2], fibonacci[frag->root_index]);

               heap_unlock(heap, &lock, 0);
            }
            break;
         }
         case HPM_QUERY_FRAG_TYPE:
         {
            HeapError ecode;
            struct Fragment *frag = &((struct Fragment *)p[1])[-1];
            struct Region *region;

            if (!HPM_IS_HANDLED(p[0]))
            {
               HeapLock lock;
               
               heap_lock(heap, &lock, 0);
               
               region = region_find(heap, a, frag, 0, &ecode);
               if (region == NULL)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               frag = frag_find(heap, region, frag, 1, &ecode);
               if (frag == NULL)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               ecode = frag_check(frag);
               if (ecode != HEAP_ERROR_NONE)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], HEAP_FRAGMENT_BUDDY);

               heap_unlock(heap, &lock, 0);
            }
            break;
         }
         case HPM_QUERY_FRAG_STATUS:
         {
            HeapError ecode;
            struct Fragment *frag = &((struct Fragment *)p[1])[-1];
            struct Region *region;

            if (!HPM_IS_HANDLED(p[0]))
            {
               HeapLock lock;

               heap_lock(heap, &lock, 0);
               
               region = region_find(heap, a, frag, 0, &ecode);
               if (region == NULL)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               frag = frag_find(heap, region, frag, 1, &ecode);
               if (frag == NULL)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               ecode = frag_check(frag);
               if (ecode == HEAP_ERROR_CORRUPTED)
               {
                  HPM_SET_HANDLED(p[0]);
                  HPM_SET_VALUE(p[2], HEAP_FRAGMENT_HEADERS);
               }
               else
               {
                  HPM_SET_HANDLED(p[0]);
                  HPM_SET_VALUE(p[2], HEAP_FRAGMENT_ALLOCATED);
               }

               heap_unlock(heap, &lock, 0);
            }
            break;
         }
         case HPM_QUERY_FRAG_LMEM:
         case HPM_QUERY_FRAG_PMEM:
         {
            HeapError ecode;
            struct Fragment *frag = &((struct Fragment *)p[1])[-1];
            struct Region *region;

            if (!HPM_IS_HANDLED(p[0]))
            {
               HeapLock lock;
               
               heap_lock(heap, &lock, 0);
               
               region = region_find(heap, a, frag, 0, &ecode);
               if (region == NULL)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               frag = frag_find(heap, region, frag, 1, &ecode);
               if (frag == NULL)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               ecode = frag_check(frag);
               if (ecode != HEAP_ERROR_NONE)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               /* Adjust fragment pointer and let
                * allocator respond to query. */
               HPM_SET_VALUE(p[1], frag->region);

               heap_unlock(heap, &lock, 0);
            }
            break;
         }
         case HPM_QUERY_FRAG_NEXT:
         {
            struct FragmentCursor *cursor = (struct FragmentCursor *)&p[1];

            /* Skip if the request doesn't concern us. */
            if (cursor->frag_org == NULL || FRAG_NEXT_ID(cursor->frag) != a->frag_id)
               break;

            if (!HPM_IS_HANDLED(p[0]))
            {
               struct Region *region;
               struct Fragment *frag;
               HeapError ecode;
               HeapLock lock;

               heap_lock(heap, &lock, 0);

               region = region_find(heap, a, FRAG_NEXT_FRAG(cursor->frag), 0, &ecode);
               if (region == NULL)
               {
                  heap_unlock(heap, &lock, 0);
                  cursor->frag = cursor->frag_org;
                  cursor->frag_org = NULL;
                  break;
               }

               frag = frag_find(heap, region, FRAG_NEXT_FRAG(cursor->frag), 0, &ecode);
               if (frag == NULL)
               {
                  heap_unlock(heap, &lock, 0);
                  cursor->frag = cursor->frag_org;
                  cursor->frag_org = NULL;
                  break;
               }

               frag = frag_next(heap, frag, &ecode);
               if (frag == NULL)
               {
                  heap_unlock(heap, &lock, 0);
                  cursor->frag = cursor->frag_org;
                  cursor->frag_org = NULL;
                  break;
               }

               cursor->frag     = FRAG_NEXT_MASK(frag, a->frag_id);
               cursor->frag_out = &frag[1];
               cursor->size_out = fibonacci[frag->root_index];

               HPM_SET_HANDLED(p[0]);

               heap_unlock(heap, &lock, 0);
            }

            break;
         }
         default:
         {
            break;
         }
      }

      p += (1 + HPM_VALUES(*p));
   }

   if (event->e_query.freezed)
      heap->freezed--;

   return HEAP_ACTION_CONTINUE;
}

/***************************************************************************/

static HeapAction
l_buddy_query_res(struct Heap *heap, union HeapEvent *event, void *opaque)
{
   HeapParam *p = event->e_query.params;
   struct BuddyAlgorithm *a = opaque;

   if (p == NULL)
      return HEAP_ACTION_CONTINUE;
   
   if (event->e_query.freezed)
      heap->freezed++;

   while (*p != HPM_END)
   {
      switch (HPM_ID(*p))
      {
         case HPM_QUERY_FRAG_COUNT:
         {
            if (HPM_IS_HANDLED(p[0]))
            {
               HPM_ADD_VALUE(p[1], a->frag_count);
               HPM_SUB_VALUE(p[1], a->region_count);
            }
            break;
         }
         case HPM_QUERY_FRAG_NEXT:
         {
            struct FragmentCursor *cursor = (struct FragmentCursor *)&p[1];

            if (HPM_IS_HANDLED(p[0]) && cursor->frag_org == NULL)
            {
               struct Region *region;
               struct Fragment *frag;
               HeapError ecode;
               HeapLock lock;

               heap_lock(heap, &lock, 0);

               region = region_find(heap, a, cursor->frag_out, 1, &ecode);
               if (region == NULL)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }
               
               frag = frag_first(heap, region, &ecode);
               if (frag == NULL)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }
               
               cursor->frag_org = cursor->frag;
               cursor->frag     = FRAG_NEXT_MASK(frag, a->frag_id);
               cursor->frag_out = &frag[1];
               cursor->size_out = fibonacci[frag->root_index];

               heap_unlock(heap, &lock, 0);
            }

            break;
         }
         default:
         {
            break;
         }
      }

      p += (1 + HPM_VALUES(*p));
   }

   if (event->e_query.freezed)
      heap->freezed--;

   return HEAP_ACTION_CONTINUE;
}

/***************************************************************************/
   
static HeapAction
l_buddy_control(struct Heap *heap, union HeapEvent *event, void *opaque)
{
   HeapParam *p = event->e_query.params;
   struct BuddyAlgorithm *a = opaque;
   HeapLock lock;

   if (p == NULL)
      return HEAP_ACTION_CONTINUE;
   
   while (HPM_ID(*p) != HPM_END)
   {
      switch (HPM_ID(*p))
      {
         case HPM_CTRL_BUF_THRESHOLD:
         {
            if (!HPM_IS_HANDLED(p[0]))
            {
               size_t s = (size_t)p[1];

               if (s > SIZEMAX)
                  s = SIZEMAX;
               
               HPM_SET_HANDLED(p[0]);
               a->config->buf_size_threshold = s;
            }
            break;
         }
         case HPM_CTRL_BUF_MAX_SIZE:
         {
            if (!HPM_IS_HANDLED(p[0]))
            {
               HPM_SET_HANDLED(p[0]);
               a->config->buf_size_max = (size_t)p[1];
            }
            break;
         }
         case HPM_CTRL_BUF_MAX_USED_SIZE:
         {
            /* Reset used watermark statistics. */
            
            if (!HPM_IS_HANDLED(p[0]))
            {
               HPM_SET_HANDLED(p[0]);
               a->stats->buf_size_used_hi = 0;
            }
            break;
         }
         case HPM_CTRL_BUF_OWNER:
         {
            struct Buffer *buf = &(((struct Buffer*)p[1])[-1]);
            struct Region *reg;
            HeapError ecode;

            if (!HPM_IS_HANDLED(p[0]))
            {
               heap_lock(heap, &lock, 0);
               
               ecode = buf_check_at_query(heap, a, &reg, buf);
               if (ecode != HEAP_ERROR_NONE)
               {
                  heap_unlock(heap, &lock, 0);
                  break;
               }

               buf_set_owner(buf, (HeapPID)p[2]);
               HPM_SET_HANDLED(p[0]);

               heap_unlock(heap, &lock, 0);
            }
            break;
         }
         default:
            break;
      }

      p += (1 + HPM_VALUES(*p));
   }
   
   return HEAP_ACTION_CONTINUE;
}

/***************************************************************************/

static HeapAction
l_buddy_destroy(struct Heap *heap, union HeapEvent *event, void *opaque)
{
   struct BuddyAlgorithm *a = opaque;

   (void)event;

   /* Unregister this plug-in. */
   zzheap_detach_hook(heap, HEAP_EVENT_ALLOC_REQ,   l_buddy_alloc,     a);
   zzheap_detach_hook(heap, HEAP_EVENT_FREE_REQ,    l_buddy_free,      a);
   zzheap_detach_hook(heap, HEAP_EVENT_RESIZE_REQ,  l_buddy_resize,    a);
   zzheap_detach_hook(heap, HEAP_EVENT_QUERY_REQ,   l_buddy_query_req, a);
   zzheap_detach_hook(heap, HEAP_EVENT_QUERY_RES,   l_buddy_query_res, a);
   zzheap_detach_hook(heap, HEAP_EVENT_CONTROL_REQ, l_buddy_control,   a);
   zzheap_detach_hook(heap, HEAP_EVENT_COLLECT_REQ, l_buddy_collect,   a);

   if (a != NULL)
      heap_meta_free(heap, a);

   /* Allocator will free fragments. */

   return HEAP_ACTION_CONTINUE;
}

/***************************************************************************/



/***************************************************************************
 * Public Interface
 ***************************************************************************/

HeapStatus
heap_attach_buddy_algorithm(struct Heap *heap,
                            struct HeapStatistics *stats,
                            struct HeapConfiguration *config,
                            uint32_t id, uint32_t frag_id)
{
   HeapStatus status;
   struct BuddyAlgorithm *a = heap_meta_alloc(
      heap, sizeof(struct BuddyAlgorithm));
   if (a == NULL)
   {
      return HEAP_EOUT_OF_MEMORY;
   }

   bzero(a, sizeof(struct BuddyAlgorithm));

   a->id      = id << BUF_ID_SHIFT;
   a->frag_id = frag_id << FRAG_ID_SHIFT;
   a->stats   = stats;
   a->config  = config;

   a->info.version = 101;
   strcpy(a->info.id, "com.enea.heap.buddy");
   
   if (a->config->buf_size_threshold >= SIZEMAX)
      a->config->buf_size_threshold = SIZEMAX;

   if (config->frag_size_init &&
       l_frag_break(heap, a, NULL, config->frag_size_init, 1) == 0)
   {
      heap_meta_free(heap, a);
      return HEAP_EOUT_OF_MEMORY;
   }
      
   status = zzheap_attach_hook(
      heap, HEAP_EVENT_ALLOC_REQ, l_buddy_alloc, a, HEAP_PRIO_ALGO_NORMAL);
   if (status != HEAP_SUCCESS)
   {
      heap_meta_free(heap, a);
      return status;
   }

   status = zzheap_attach_hook(
      heap, HEAP_EVENT_FREE_REQ, l_buddy_free, a, HEAP_PRIO_ALGO_NORMAL);
   if (status != HEAP_SUCCESS)
   {
      heap_meta_free(heap, a);
      return status;
   }

   status = zzheap_attach_hook(
      heap, HEAP_EVENT_RESIZE_REQ, l_buddy_resize, a, HEAP_PRIO_ALGO_NORMAL);
   if (status != HEAP_SUCCESS)
   {
      heap_meta_free(heap, a);
      return status;
   }

   status = zzheap_attach_hook(
      heap, HEAP_EVENT_QUERY_REQ, l_buddy_query_req, a, HEAP_PRIO_ALGO_NORMAL);
   if (status != HEAP_SUCCESS)
   {
      heap_meta_free(heap, a);
      return status;
   }

   status = zzheap_attach_hook(
      heap, HEAP_EVENT_QUERY_RES, l_buddy_query_res, a, HEAP_PRIO_ALGO_NORMAL);
   if (status != HEAP_SUCCESS)
   {
      heap_meta_free(heap, a);
      return status;
   }

   status = zzheap_attach_hook(
      heap, HEAP_EVENT_COLLECT_REQ, l_buddy_collect, a, HEAP_PRIO_ALGO_NORMAL);
   if (status != HEAP_SUCCESS)
   {
      heap_meta_free(heap, a);
      return status;
   }

   status = zzheap_attach_hook(
      heap, HEAP_EVENT_CONTROL_REQ, l_buddy_control, a, HEAP_PRIO_ALGO_NORMAL);
   if (status != HEAP_SUCCESS)
   {
      heap_meta_free(heap, a);
      return status;
   }

   status = zzheap_attach_hook(
      heap, HEAP_EVENT_DESTROY_RES, l_buddy_destroy, a, HEAP_PRIO_ALGO_NORMAL);
   if (status != HEAP_SUCCESS)
   {
      heap_meta_free(heap, a);
      return status;
   }
   
   return HEAP_SUCCESS;
}

/***************************************************************************/

size_t 
heap_buddy_size(size_t size)
{
   if (size < SIZEMAX)
   {
      size = fibonacci[size_sindex(size)];
   }
   else
   {
      size = SIZEMAX;
   }

   return size;
}

/***************************************************************************/
