/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2010 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#include "heap_filt_nobreak.h"
#include <string.h>

/*****************************************************************************
 * Private Interface
 *****************************************************************************/

static HeapAction
l_nobreak_break(struct Heap *heap, union HeapEvent *event, void *opaque)
{
   HeapParam *p = event->e_break.params;
   
   (void)heap;
   (void)opaque;

   if (p == NULL)
      return HEAP_ACTION_CONTINUE;

   /* Scan for "nobreak" paramater. */
   while (HPM_ID(*p) != HPM_END &&
          HPM_ID(*p) != HPM_BREAK_NO)
   {
      p += (1 + HPM_VALUES(*p));
   }

   if (HPM_ID(*p) == HPM_BREAK_NO)
   {
      heap_assert(event->e_break.base == NULL,
                  "A fragment was leaked when a "
                  "filter cancelled a break event.");

      event->e_break.base = NULL;
      return HEAP_ACTION_ABORT;
   }

   return HEAP_ACTION_CONTINUE;
}

static HeapAction
l_nobreak_query(struct Heap *heap, union HeapEvent *event, void *opaque)
{
   HeapParam *p = event->e_query.params;

   (void)heap;
   (void)opaque;
   
   if (p == NULL)
      return HEAP_ACTION_CONTINUE;
   
   while (*p != HPM_END)
   {
      switch (HPM_ID(*p))
      {
         case HPM_QUERY_SERVICE:
         {
            struct HeapServiceInfo *info = opaque;

            if (!HPM_IS_HANDLED(p[0]) && p[1] == 0)
            {
               HPM_SET_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], info);
            }
            else if (p[1] == (HeapParam)info)
            {
               HPM_CLEAR_HANDLED(p[0]);
               HPM_SET_VALUE(p[1], 0);
            }

            break;
         }
      }

      p += (1 + HPM_VALUES(*p));
   }

   return HEAP_ACTION_CONTINUE;
}

/**************************************************************************/

static HeapAction
l_nobreak_destroy(struct Heap *heap, union HeapEvent *event, void *opaque)
{
   (void)event;

   heap_meta_free(heap, opaque);

   return HEAP_ACTION_CONTINUE;
}

/**************************************************************************/




/*****************************************************************************
 * Public Interface
 *****************************************************************************/

HeapStatus
heap_attach_nobreak_filter(struct Heap *heap)
{
   struct HeapServiceInfo *info;
   HeapStatus status;

   info = heap_meta_alloc(heap, sizeof(struct HeapServiceInfo));
   if (info == NULL)
      return HEAP_EOUT_OF_MEMORY;

   info->version = 101;
   strcpy(info->id, "com.enea.heap.nobreak");
   
   status = zzheap_attach_hook(
      heap, HEAP_EVENT_BREAK_REQ, l_nobreak_break, NULL, HEAP_PRIO_FILTER_NORMAL);
   if (status != HEAP_SUCCESS)
   {
      heap_meta_free(heap, info);
      return status;
   }

   status = zzheap_attach_hook(
      heap, HEAP_EVENT_QUERY_REQ, l_nobreak_query, info, HEAP_PRIO_FILTER_NORMAL);
   if (status != HEAP_SUCCESS)
   {
      heap_meta_free(heap, info);
      return status;
   }

   status = zzheap_attach_hook(
      heap, HEAP_EVENT_DESTROY_RES, l_nobreak_destroy, info, HEAP_PRIO_FILTER_NORMAL);
   if (status != HEAP_SUCCESS)
   {
      heap_meta_free(heap, info);
      return status;
   }
   
   return HEAP_SUCCESS;
}

