/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2014 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef HEAP_ERROR_H
#define HEAP_ERROR_H


typedef uint32_t HeapError;

/* Used when state can't be rolled back or is unknown. */
#define HEAP_ERROR_FATAL            0x00008000U


/*
*  HEAP_ERROR_NONE                  Should never be reported as error.
*
*  HEAP_ERROR_BUF_OWNER             See OSE_HEAP_EPROCESS_NOT_OWNER
*  HEAP_ERROR_BUF_HEADER            See OSE_HEAP_EHEAP_CORRUPTED
*  HEAP_ERROR_BUF_ENDMARK           See OSE_HEAP_EENDMARK_OVERWRITTEN
*  HEAP_ERROR_BUF_UNKNOWN           See OSE_HEAP_EPTR_INVALID
*  HEAP_ERROR_BUF_UNALIGNED   
*  HEAP_ERROR_BUF_FREE              See OSE_HEAP_EBUFFER_FREE
*  HEAP_ERROR_BUF_LINKS             See OSE_HEAP_EFREED_BUFFER_OVERWRITTEN
*  HEAP_ERROR_BUF_TOUCHED           See OSE_HEAP_EFREED_BUFFER_OVERWRITTEN
*     For the above codes, HeapErrorEvent.data is buffer pointer
*     (address after buffer header)
*
*  HEAP_ERROR_CORRUPTED             See OSE_HEAP_EHEAP_CORRUPTED
*     HeapErrorEvent.data is a pointer to the structure that is corrupted.
*     It can be different structures, e.g struct Region, struct Fragment
*
*  HEAP_ERROR_BUF_EXHAUSTED         See OSE_HEAP_EHEAP_EXHAUSTED
*     HeapErrorEvent.data is allocation request size
*
*  HEAP_ERROR_MEM_EXHAUSTED         See OSE_HEAP_ESYSMEM_ALLOC_FAILED
*     HeapErrorEvent.data is break request min size
*
*  HEAP_ERROR_FRAG_HEADER           
*  HEAP_ERROR_FRAG_UNKNOWN     
*  HEAP_ERROR_FRAG_UNALIGNED   
*     For the above codes, HeapErrorEvent.data is fragment pointer
*     (address after fragment header)
*/
#define HEAP_ERROR_NONE             0x00000000U
/* Codes 1-3 currently not used */
#define HEAP_ERROR_BUF_OWNER        0x00000004U
#define HEAP_ERROR_BUF_HEADER       0x00000005U
#define HEAP_ERROR_BUF_ENDMARK      0x00000006U
#define HEAP_ERROR_BUF_UNKNOWN      0x00000007U
#define HEAP_ERROR_BUF_UNALIGNED    0x00000008U
#define HEAP_ERROR_BUF_FREE         0x00000009U
#define HEAP_ERROR_BUF_LINKS        0x0000000AU
#define HEAP_ERROR_CORRUPTED        0x0000000BU
/* Code 0xC currently not used */
#define HEAP_ERROR_BUF_EXHAUSTED    0x0000000DU
#define HEAP_ERROR_MEM_EXHAUSTED    0x0000000EU
#define HEAP_ERROR_BUF_TOUCHED      0x0000000FU
#define HEAP_ERROR_FRAG_HEADER      0x00000010U
#define HEAP_ERROR_FRAG_UNKNOWN     0x00000011U
#define HEAP_ERROR_FRAG_UNALIGNED   0x00000012U


#endif /* HEAP_ERROR_H */

