/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef HEAP_LOCK_H
#define HEAP_LOCK_H

#include "heap_osal.h"

#include "ose.h"
#include "pose.h"
#include "spinlock.h"




/*
 *************************************************************************
 * Public Interface
 *************************************************************************
 */

#if 0
static inline void
heap_lock_init(struct Heap *heap, uint32_t type);
#endif

#define heap_lock_init(heap, type)                              \
do {                                                            \
   ose_mutex_init(&(heap)->brklock, 0);                         \
   switch (type)                                                \
   {                                                            \
      case EHEAP_LOCK_MUTEX:                                    \
         ose_mutex_init(&(heap)->lock.mutex, 0);                \
         break;                                                 \
      case EHEAP_LOCK_SPIN:                                     \
         spin_lock_init(&((heap)->lock.spin.lock));             \
         break;                                                 \
      case EHEAP_LOCK_INT:                                      \
      case EHEAP_LOCK_NONE:                                     \
         break;                                                 \
      default:                                                  \
         (type) = EHEAP_LOCK_NONE;                              \
         break;                                                 \
   }                                                            \
                                                                \
   (heap)->lock_type = (type);                                  \
} while (0)

/*************************************************************************/

#if 0
static inline void
heap_lock(struct Heap *heap, heap_lock_t *lock, uint32_t freeze_mode);
#endif

#define heap_lock(_h, _l, _f)                                   \
do {                                                            \
   *(_l) = 0;                                                   \
   if (!(_h)->freezed)                                          \
   {                                                            \
      switch ((_h)->lock_type)                                  \
      {                                                         \
         case EHEAP_LOCK_MUTEX:                                 \
            zzkrn_kill_lock(0);                                 \
            ose_mutex_lock(&(_h)->lock.mutex);                  \
            break;                                              \
         case EHEAP_LOCK_SPIN:                                  \
         {                                                      \
            PROCESS cp = current_process();                     \
            if ((_h)->lock.spin.owner == cp) {                  \
               (_h)->lock.spin.count++;                         \
            } else {                                            \
               spin_lock_irqsave(&(_h)->lock.spin.lock, *(_l)); \
               (_h)->lock.spin.owner = cp;                      \
               (_h)->lock.spin.count = 1;                       \
            }                                                   \
            break;                                              \
         }                                                      \
         case EHEAP_LOCK_INT:                                   \
            LOCK_SAVE(*(_l));                                   \
            break;                                              \
         default:                                               \
            break;                                              \
      }                                                         \
   }                                                            \
} while (0)

/*************************************************************************/

#if 0
static inline void
heap_unlock(struct Heap *heap, heap_lock_t *lock, uint32_t freeze_mode);
#endif

#define heap_unlock(_h, _l, _f)                                 \
do {                                                            \
   if (!(_h)->freezed)                                          \
   {                                                            \
      switch ((_h)->lock_type)                                  \
      {                                                         \
         case EHEAP_LOCK_MUTEX:                                 \
            ose_mutex_unlock(&(_h)->lock.mutex);                \
            zzkrn_kill_unlock(0);                               \
            break;                                              \
         case EHEAP_LOCK_SPIN:                                  \
         {                                                      \
            if ((_h)->lock.spin.count > 1) {                    \
               (_h)->lock.spin.count--;                         \
            } else {                                            \
               (_h)->lock.spin.owner = 0;                       \
               (_h)->lock.spin.count = 0;                       \
               spin_unlock_irqrestore(&(_h)->lock.spin.lock, *(_l)); \
            }                                                   \
            break;                                              \
         }                                                      \
         case EHEAP_LOCK_INT:                                   \
            LOCK_RESTORE(*(_l));                                \
            break;                                              \
         default:                                               \
            break;                                              \
      }                                                         \
   }                                                            \
} while (0)

/**************************************************************************/

static inline int
heap_brkallowed(void)
{
   enum PROCESS_TYPE ptype = get_ptype(current_process());
   return ptype != OS_INT_PROC && ptype != OS_TI_PROC;
}

/**************************************************************************/

/* The heap lock needs to be acquired before calling this function. */
static inline void
heap_brkprepare(struct Heap *heap)
{
   if (heap->brkproc == 0)  {
      heap->brkproc = current_process();
   }
}

/**************************************************************************/

static inline int
heap_brklock(struct Heap *heap, HeapLock *lock)
{
   zzkrn_kill_lock(0);
   if (ose_mutex_trylock(&heap->brklock) != 0)
   {
      HeapPID brkproc;
      
      heap_lock(heap, lock, 0);
      brkproc = heap->brkproc;
      heap_unlock(heap, lock, 0);

      /* Break lock aquired, but has the process been preempted
       * before it reached this point? If so, return immediately
       * and retry allocation. 
       */
      if (brkproc != current_process())
      {
         heap_lock(heap, lock, 0);
         heap->brkproc = 0;
         heap_unlock(heap, lock, 0);
         return 0;
      }
   }
   else
   {
      /* Failed. */
      ose_mutex_lock(&heap->brklock);

      /* Forces algo to abort and retry allocation before
       * doing another break attempt. */
      return 0;
   }

   return 1; /* Allow algo to proceed with a break request. */
}

/*************************************************************************/

static inline void
heap_brkunlock(struct Heap *heap, HeapLock *lock)
{
   heap_lock(heap, lock, 0);
   heap->brkproc = 0;
   heap_unlock(heap, lock, 0);
   
   ose_mutex_unlock(&heap->brklock);
   zzkrn_kill_unlock(0);
}
 
/*************************************************************************/

#if 0
static inline void
heap_lock_type(struct Heap *heap);
#endif

#define heap_lock_type(_h)   ((_h)->lock_type)

/*************************************************************************/

#if 0
static inline HeapPID
heap_lock_owner(struct Heap *heap);
#endif

#define heap_lock_owner(heap)                           \
   (((heap)->lock_type == EHEAP_LOCK_MUTEX) ?           \
    (((struct _mutex *)(&(heap)->lock.mutex))->owner & ~0x80000000) : \
    ((heap)->lock.spin.owner & ~0x80000000))

/**************************************************************************/

#define heap_brklock_owner(heap)                           \
    (((struct _mutex *)(&(heap)->brklock))->owner & ~0x80000000)

/*************************************************************************/

#if 0
static inline void
heap_lock_destroy(struct Heap *heap);
#endif

#define heap_lock_destroy(heap)                         \
do {                                                    \
   ose_mutex_destroy(&(heap)->brklock);                 \
   switch ((heap)->lock_type)                           \
   {                                                    \
      case EHEAP_LOCK_MUTEX:                            \
         ose_mutex_destroy(&(heap)->lock.mutex);        \
         break;                                         \
      default:                                          \
         break;                                         \
   }                                                    \
} while (0)

/*************************************************************************/

#endif /* HEAP_LOCK_H */
