/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2010 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef HEAP_OSAL_H
#define HEAP_OSAL_H


/*
 *************************************************************************
 * OSE Specific Adaption
 *************************************************************************
 */

#include "ose.h"
#include "mmtypes.h"
#include "stdint.h"
#include "stdlib.h"
#include "assert.h"
#include "atomic.h"
#include "ramlog.h"
#include "cpu.h"
#include "spinlock.h"
#include "compiler.h"
#include "heapapi.h" /* struct HeapExtension */

/*
 *************************************************************************
 * Heap Datatypes
 *************************************************************************
 */

typedef Msr        HeapLock;
typedef MUTEX      HeapMutexLock;
typedef struct     HeapSpinLock
{
   spinlock_t lock;
   uint32_t   count;
   PROCESS    owner;
} HeapSpinLock;

typedef PROCESS    HeapPID;



/*
 *************************************************************************
 * Helper Macros and Functions
 *************************************************************************
 */
#define heap_log ramlog_printf
#ifdef ASSERT
#define heap_assert(condition, message)                                                         \
do {												\
   if (!(condition))										\
   {												\
      ramlog_printf("======================================================================\n"	\
                "HEAP ASSERTION FAILED\n%s\n\n%s:%d\n"						\
                "======================================================================\n",	\
                (message), __FILE__, __LINE__);							\
      error2(0,0);										\
   }												\
} while(0)
#else
#define heap_assert(cond, msg)
#endif

/**************************************************************************/

#ifndef HEAP_ILLEGAL_PID
/* An PID with invalid instance number. */
#define HEAP_ILLEGAL_PID 0xDEAD0001
#endif 

struct Heap;
void *heap_meta_alloc(struct Heap *heap, size_t size);
void  heap_meta_free(struct Heap *heap, void *p);

/**************************************************************************/

/* Use a macro to avoid warnings about an unused function. */
#ifdef USE_HEAP_FLS

static always_inline long
fls(size_t bits)
{

   /*
    * Find the most significant bit set in a
    * bitmap. Should return -1 if bitmap is 0.
    *****************************************/
   
   long tmp;

#if defined(OSPP)

   __asm__ volatile (
      " cntlz %0, %1 \n"
      : "=r"(tmp) : "r"(bits));
   return 31-tmp;

#elif defined(OSARM) && !defined(__thumb)
#   if defined (__GNUC__) 

   __asm__ volatile (
      " clz %0, %1 \n"
      : "=r"(tmp) : "r"(bits));
   return 31-tmp;

#   else

   __asm { clz tmp, bits }
   return 31-tmp;

#   endif
#elif defined(OSMIPS)

#ifdef _MIPS_ARCH_MIPS3
   return 31 - __builtin_clz(bits);
#else
   __asm__ volatile (
      " clz %0, %1 \n"
      : "=r"(tmp) : "r"(bits));
   return 31-tmp;
#endif
   
#elif defined(OSX86)
   if (!bits)
      return -1;
   
   __asm__ volatile (
      " bsr %1, %0 \n" 
      : "=r"(tmp) : "r"(bits));

   return tmp;
#else /* Generic arch. */
   if (!bits)
      return -1;
   
   tmp = 0;
   if ((bits & 0xffff0000) != 0) 
   {  
      bits >>= 16;  
      tmp += 16;
   }
   if ((bits & 0xff00) != 0) 
   {  
      bits >>= 8;  
      tmp += 8;
   }
   if ((bits & 0xf0) != 0) 
   {  
      bits >>= 4;
      tmp += 4;
   }
   if ((bits & 0xc) != 0) 
   {  
      bits >>= 2;
      tmp += 2;
   }
   return tmp + (bits >> 1);
#endif /* Generic arch. */
}
#endif /* USE_HEAP_FLS */

/**************************************************************************/


#endif /* HEAP_OSAL_H */

