/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2009 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */


#include "hfmi.h"
#include "efmi.h"
#include "ose_heapd.h"
#include "pose.h"

#include <string.h>



/*****************************************************************************
 * Helper Macros
 *****************************************************************************/

#define SET_SEGMENT     if (!l_heap_segment(heap, 0)) return HFMI_CALL_INVALID
#define RESTORE_SEGMENT if (!l_heap_segment(heap, 1)) return HFMI_CALL_INVALID



/*****************************************************************************
 * Data Structures
 *****************************************************************************/

struct HeapCursor 
{
   OSHEAP    href;
};

/*****************************************************************************/

struct FragmentCursor 
{
   OSHEAP    href;
   uint32_t  base;
   uint32_t  size;
   uint32_t  frag;
   uint32_t  reserved0;
};

/*****************************************************************************/

struct BufferCursor
{
   OSHEAP    href;
   uint32_t  base;
   uint32_t  frag;
   uint32_t  path;
   uint32_t  depth;
};

/*****************************************************************************/






/*****************************************************************************
 * Helper Functions
 *****************************************************************************/

static int
l_heap_segment(struct OseHeap *heap, OSBOOLEAN restore)
{
   if (heap->config.frag_mas)
   {
      if (restore)
      {
         if (zzkrn_freezemode_set_segment(1, 0))
            return 0;
      }
      else
      {
         if (zzkrn_freezemode_set_segment(0, heap->domain))
            return 0;
      }
   }

   return 1;
}

/*****************************************************************************/

static inline uint32_t
l_heap_alloc_failed(struct OseHeap *heap)
{
   HeapParam p[] = {
      HPM(QUERY_ALLOC_NULL, 1), 0,
      HPM_END
   };

   zzheap_query(&heap->heap, p, 1);

   return HPM_IS_HANDLED(p[0]) ? (uint32_t)p[1] : 0;
}

/*****************************************************************************/

#if 0 /* FIXME Not needed. Does not exist in interface. */
static inline uint32_t
l_heap_break_failed(struct OseHeap *heap)
{
   HeapParam p[] = {
      HPM(QUERY_BREAK_NULL, 1), 0,
      HPM_END
   };

   zzheap_query(&heap->heap, p, 1);

   return HPM_IS_HANDLED(p[0]) ? (uint32_t)p[1] : 0;
}
#endif

/*****************************************************************************/

static inline uint32_t
l_heap_largest_free(struct OseHeap *heap)
{
   HeapParam p[] = {
      HPM(QUERY_BUF_LARGEST_FREE, 1), 0,
      HPM_END
   };

   zzheap_query(&heap->heap, p, 1);

   return HPM_IS_HANDLED(p[0]) ? (uint32_t)p[1] : 0;
}

/*****************************************************************************/

static inline int
l_heap_frag_next(struct OseHeap *heap, struct FragmentCursor *cursor)
{
   HeapParam p[] = {
      HPM(QUERY_FRAG_NEXT, 4), 0, 0, 0, 0,
      HPM_END
   };

   p[1] = (HeapParam)cursor->base;
   p[2] = (HeapParam)cursor->size;
   p[3] = (HeapParam)cursor->frag;
   p[4] = (HeapParam)cursor->reserved0;

   zzheap_query(&heap->heap, p, 1);

   cursor->base      = HPM_IS_HANDLED(p[0]) ? (uint32_t)p[1] : 0;
   cursor->size      = HPM_IS_HANDLED(p[0]) ? (uint32_t)p[2] : 0;
   cursor->frag      = HPM_IS_HANDLED(p[0]) ? (uint32_t)p[3] : 0;
   cursor->reserved0 = HPM_IS_HANDLED(p[0]) ? (uint32_t)p[4] : 0;

   return HPM_IS_HANDLED(p[0]);
}

/*****************************************************************************/

static inline size_t 
l_heap_buf_size(struct OseHeap *heap, struct BufferCursor *cursor)
{
   HeapParam p[] = {
      HPM(QUERY_BUF_ACT_SIZE, 1), 0,
      HPM_END
   };

   p[1] = (HeapParam)cursor->base;

   zzheap_query(&heap->heap, p, 1);

   return HPM_IS_HANDLED(p[0]) ? (size_t)p[1] : 0;
}

/*****************************************************************************/

static inline HeapBufferStatus
l_heap_buf_status(struct OseHeap *heap, struct BufferCursor *cursor)
{
   HeapParam p[] = {
      HPM(QUERY_BUF_STATUS, 1), 0,
      HPM_END
   };

   p[1] = (HeapParam)cursor->base;

   zzheap_query(&heap->heap, p, 1);

   return HPM_IS_HANDLED(p[0]) ? (HeapBufferStatus)p[1] : 0;
}

/*****************************************************************************/

static inline int
l_heap_buf_next(struct OseHeap *heap, struct BufferCursor *cursor)
{
   HeapParam p[] = {
      HPM(QUERY_BUF_NEXT, 4), 0, 0, 0, 0,
      HPM_END
   };

   p[1] = (HeapParam)cursor->base;
   p[2] = (HeapParam)cursor->frag;
   p[3] = (HeapParam)cursor->path;
   p[4] = (HeapParam)cursor->depth;

   zzheap_query(&heap->heap, p, 1);

   cursor->base  = HPM_IS_HANDLED(p[0]) ? (uint32_t)p[1] : 0;
   cursor->frag  = HPM_IS_HANDLED(p[0]) ? (uint32_t)p[2] : 0;
   cursor->path  = HPM_IS_HANDLED(p[0]) ? (uint32_t)p[3] : 0;
   cursor->depth = HPM_IS_HANDLED(p[0]) ? (uint32_t)p[4] : 0;

   return HPM_IS_HANDLED(p[0]);
}

/*****************************************************************************/

static inline uint32_t
l_heap_frag_type(struct OseHeap *heap, struct FragmentCursor *cursor)
{
   HeapParam p[] = {
      HPM(QUERY_FRAG_TYPE, 1), 0,
      HPM_END
   };

   p[1] = (HeapParam)cursor->base;
   
   zzheap_query(&heap->heap, p, 1);

   if (!HPM_IS_HANDLED(p[0]))
      return HFMI_FRAGTYPE_UNKNOWN;

   switch (p[1])
   {
      case HEAP_FRAGMENT_BUDDY:
         return HFMI_FRAGTYPE_HEAP;
      case HEAP_FRAGMENT_REGION:
         return HFMI_FRAGTYPE_MM;
      default:
         return HFMI_FRAGTYPE_UNKNOWN;
   }
}

/*****************************************************************************/

static inline uint32_t
l_heap_frag_status(struct OseHeap *heap, struct FragmentCursor *cursor)
{
   uint32_t status = 0;
   
   HeapParam p[] = {
      HPM(QUERY_FRAG_STATUS, 2), 0, 0,
      HPM_END
   };

   p[1] = (HeapParam)cursor->base;
   
   zzheap_query(&heap->heap, p, 1);

   if (!HPM_IS_HANDLED(p[0]))
      return 0;

   if (p[2] & HEAP_FRAGMENT_HEADERS)
      status |= HFMI_FRAG_BAD_HEADER;
   if (p[2] & HEAP_FRAGMENT_HEADERS)
      status |= HFMI_FRAG_ALLOCATED;

   return status;
}

/*****************************************************************************/

static inline void
l_heap_histo(struct OseHeap *heap, uint32_t ix,
             U32 *num_free, U32 *num_used, U32 *num_size)
{
   HeapParam p[] = {
      HPM(QUERY_BUF_NUM_FREE, 2), 0, 0,
      HPM(QUERY_BUF_NUM_USED, 2), 0, 0,
      HPM(QUERY_BUF_NUM_SIZE, 2), 0, 0,
      HPM_END
   };

   p[1] = p[4] = p[7] = (HeapParam)ix;

   zzheap_query(&heap->heap, p, 1);

   *num_free = HPM_IS_HANDLED(p[0]) ? (U32)p[2] : 0;
   *num_used = HPM_IS_HANDLED(p[3]) ? (U32)p[5] : 0;
   *num_size = HPM_IS_HANDLED(p[6]) ? (U32)p[8] : 0;
}

/*****************************************************************************/

static inline void
l_heap_info(struct OseHeap *heap, struct HfmiHeapInfo *hhip, size_t hi_size)
{
   struct HfmiHeapInfo info;
   
   info.href                 = HREF_FROM_HEAP(heap);
   info.annotation_size      = heap->hucb.annotation_size;
   info.threshold            = heap->config.buf_size_threshold;
   info.endmark_size         = heap->config.buf_size_endmark;
   info.error_on_null        = heap->buf_error_on_null;
   info.owner                = heap->owner;

   info.wipe_on_free         = 0; /* Deprecated. */
   info.use_redundant_bitmap = 0; /* Deprecated. */

   memcpy(hhip, &info, hi_size);
}

/*****************************************************************************/

static inline void
l_heap_stats(struct OseHeap *heap, struct HfmiHeapStatistics *hsp, size_t hs_size)
{
   struct HfmiHeapStatistics hs;
   uint32_t i;

   memset(&hs, 0, sizeof(struct HfmiHeapStatistics));
   
   hs.total_req	      = heap->stats.buf_size_req;
   hs.total_used      = heap->stats.buf_size_used;
   hs.total_free      = heap->stats.buf_size_free;
   hs.total_used_hi   = heap->stats.buf_size_used_hi;
   hs.number_shared   = heap->stats.buf_num_shared;
   hs.number_private  = heap->stats.buf_num_private;
   hs.largest_free    = l_heap_largest_free(heap);
   hs.malloc_fail     = l_heap_alloc_failed(heap);
   
   for(i = 0; i < 32; i++) 
   {
      l_heap_histo(heap, i, &hs.histo_free[i], &hs.histo_used[i], &hs.histo_size[i]);
   }

   memcpy(hsp, &hs, hs_size);
}

/*****************************************************************************/

static inline void
l_heap_frag_info(struct OseHeap *heap, struct FragmentCursor *cursor,
                 struct HfmiFragmentInfo *hfip, size_t hfi_size)
{
   struct HfmiFragmentInfo hfi;
   
   hfi.address = cursor->base;
   hfi.size    = cursor->size;
   hfi.type    = l_heap_frag_type(heap, cursor);
   hfi.status  = l_heap_frag_status(heap, cursor);
   
   memcpy(hfip, &hfi, hfi_size);
}

/*****************************************************************************/

static inline void
l_heap_buf_info(struct OseHeap *heap, struct BufferCursor *cursor,
                struct HfmiBufferInfo *hbip, size_t hbi_size)
{
   struct HfmiBufferInfo hbi;
   HeapParam p[] = {
      HPM(QUERY_BUF_OWNER,    1), 0,
      HPM(QUERY_BUF_REQ_SIZE, 1), 0,
      HPM(QUERY_BUF_ACT_SIZE, 1), 0,
      HPM(QUERY_BUF_STATUS,   1), 0,
      HPM(QUERY_BUF_TYPE,     1), 0,
      HPM_END
   };

   p[1] = p[3] = p[5] = p[7] = p[9] = (HeapParam)cursor->base;

   zzheap_query(&heap->heap, p, 1);

   hbi.data_address       = cursor->base;
   hbi.annotation_address = cursor->base;
   hbi.owner              = HPM_IS_HANDLED(p[0]) ? (PROCESS)p[1] : 0;
   hbi.requested_size     = HPM_IS_HANDLED(p[2]) ? (U32)p[3]     : 0;
   hbi.actual_size        = HPM_IS_HANDLED(p[4]) ? (U32)p[5]     : 0;
   hbi.status             = 0;
   
   /* Translate buffer status bitmask. */
   if (HPM_IS_HANDLED(p[6]))
   {
      if (p[7] & HEAP_BUFFER_ALLOCATED)
         hbi.status |= HFMI_BUFFER_ALLOCATED;
      if (p[7] & HEAP_BUFFER_ENDMARK)
         hbi.status |= HFMI_BUFFER_BAD_ENDMARK;
      if (p[7] & HEAP_BUFFER_HEADERS)
      {
         /* HEAP_BUFFER_HEADERS can be caused by any of these two. */
         hbi.status |= HFMI_BUFFER_BAD_CHECKSUM;
         hbi.status |= HFMI_BUFFER_HEADER_CORRUPT;
      }
      if (p[7] & HEAP_BUFFER_ANNOTATED)
         hbi.status |= HFMI_BUFFER_IS_ANNOTATED;
   }

   /* Translate buffer type to status bitmask. */
   if (HPM_IS_HANDLED(p[8]))
   {
      switch (p[9])
      {
         case HEAP_BUFFER_FBUDDY:
            hbi.status |= HFMI_BUFFER_ALGO_FBUDDY;
            break;
         case HEAP_BUFFER_LARGE:
            hbi.status |= HFMI_BUFFER_ALGO_LARGE;
            break;
         case HEAP_BUFFER_TLSF:
            hbi.status |= HFMI_BUFFER_ALGO_TLSF;
            break;
         case HEAP_BUFFER_POOL:
         default:
            break;
      }
   }

   if ((hbi.status & HFMI_BUFFER_IS_ANNOTATED) != 0)
   {
      hbi.data_address += heap->hucb.annotation_size;
      hbi.status |= HFMI_BUFFER_IS_ANNOTATED;
      hbi.requested_size -= heap->hucb.annotation_size;
   }
   
   memcpy(hbip, &hbi, hbi_size);
}

/*****************************************************************************/

static inline void
l_heap_buf_owner(struct OseHeap *heap, struct BufferCursor *cursor, PROCESS *o)
{
   HeapParam p[] = {
      HPM(QUERY_BUF_OWNER, 1), 0,
      HPM_END
   };

   p[1] = (HeapParam)cursor->base;

   zzheap_query(&heap->heap, p, 1);

   *o = HPM_IS_HANDLED(p[0]) ? (PROCESS)p[1] : 0;
}

/*****************************************************************************/





/*****************************************************************************
 * Public Interface
 *****************************************************************************/

int
ose_hfmi_get_heap(OSHEAP href,
                  struct HfmiHeapCursor* hhc,
                  struct HfmiHeapInfo* hhip, size_t hi_size)
{
   struct HeapCursor* cursor = (struct HeapCursor*)hhc;
   
   struct OseHeap *heap = heapd_heap_find(href);
   if (heap == NULL)
      return HFMI_NOT_FOUND;

   /* Fill in heap information structure. */
   SET_SEGMENT;
   l_heap_info(heap, hhip, hi_size);
   RESTORE_SEGMENT;

   /* Set cursor. */
   cursor->href = href;
   
   return HFMI_OK;
}

/*****************************************************************************/

int
ose_hfmi_get_first_heap(struct HfmiHeapCursor* hhc,
			struct HfmiHeapInfo* hi, size_t hi_size)
{
   struct HeapCursor* cursor = (struct HeapCursor*)hhc;
   
   struct OseHeap *heap = heapd_heap_first();
   if (heap == NULL)
      return HFMI_END_OF_LIST;

   /* Set cursor. */
   cursor->href = HREF_FROM_HEAP(heap);

   /* Fill in heap information structure. */
   SET_SEGMENT;
   l_heap_info(heap, hi, hi_size);
   RESTORE_SEGMENT;
   
   return HFMI_OK;
}

/*****************************************************************************/

int
ose_hfmi_get_next_heap(struct HfmiHeapCursor* hhc,
                       struct HfmiHeapInfo* hi, size_t hi_size)
{
   struct HeapCursor* cursor = (struct HeapCursor*)hhc;
   
   struct OseHeap *heap = heapd_heap_find(cursor->href);
   if (heap == NULL)
      return HFMI_NOT_FOUND;

   heap = heapd_heap_next(heap);
   if (heap == NULL)
      return HFMI_END_OF_LIST;

   /* Set cursor. */
   cursor->href = HREF_FROM_HEAP(heap);

   /* Fill in heap information structure. */
   SET_SEGMENT;
   l_heap_info(heap, hi, hi_size);
   RESTORE_SEGMENT;
   
   return HFMI_OK;
}

/*****************************************************************************/

int
ose_hfmi_get_heap_statistics(OSHEAP href,
			     struct HfmiHeapStatistics* hs, size_t hs_size)
{
   struct OseHeap *heap = heapd_heap_find(href);
   if (heap == NULL)
      return HFMI_NOT_FOUND;

   SET_SEGMENT;
   l_heap_stats(heap, hs, hs_size);
   RESTORE_SEGMENT;
   
   return HFMI_OK;
}

/*****************************************************************************/

int
ose_hfmi_get_first_fragment(OSHEAP href,
			    struct HfmiFragmentCursor* hfc,
			    struct HfmiFragmentInfo* hfi, size_t hfi_size)
{
   struct FragmentCursor* cursor = (struct FragmentCursor*)hfc;
   
   struct OseHeap *heap = heapd_heap_find(href);
   if (heap == NULL)
      return HFMI_NOT_FOUND;

   /* Initialize for first request. */
   cursor->href      = href;
   cursor->base      = 0;
   cursor->size      = 0;
   cursor->frag      = 0;
   cursor->reserved0 = 0;

   /* Do request. */
   SET_SEGMENT;
   if (!l_heap_frag_next(heap, cursor))
   {
      RESTORE_SEGMENT;
      return HFMI_END_OF_LIST;
   }

   l_heap_frag_info(heap, cursor, hfi, hfi_size);
   RESTORE_SEGMENT;

   return HFMI_OK;
}

/*****************************************************************************/

int
ose_hfmi_get_next_fragment(struct HfmiFragmentCursor* hfc,
			   struct HfmiFragmentInfo* hfi, size_t hfi_size)
{
   struct FragmentCursor* cursor = (struct FragmentCursor*)hfc;
   
   struct OseHeap *heap = heapd_heap_find(cursor->href);
   if (heap == NULL)
      return HFMI_NOT_FOUND;

   /* Do request. */
   SET_SEGMENT;
   if (!l_heap_frag_next(heap, cursor))
   {
      RESTORE_SEGMENT;
      return HFMI_END_OF_LIST;
   }

   l_heap_frag_info(heap, cursor, hfi, hfi_size);
   RESTORE_SEGMENT;

   return HFMI_OK;
}

/*****************************************************************************/

int
ose_hfmi_get_first_buffer(OSHEAP href,
			  struct HfmiBufferCursor* hbc,
			  struct HfmiBufferInfo* hbi, size_t hbi_size)
{
   struct BufferCursor* cursor = (struct BufferCursor*)hbc;
   
   struct OseHeap *heap = heapd_heap_find(href);
   if (heap == NULL)
      return HFMI_NOT_FOUND;

   /* Initialize for first request. */
   cursor->href  = href;
   cursor->base  = 0;
   cursor->frag  = 0;
   cursor->path  = 0;
   cursor->depth = 0;

   /* Do request. */
   SET_SEGMENT;
   if (!l_heap_buf_next(heap, cursor))
   {
      RESTORE_SEGMENT;
      return HFMI_END_OF_LIST;
   }

   l_heap_buf_info(heap, cursor, hbi, hbi_size);
   RESTORE_SEGMENT;

   return HFMI_OK;
}

/*****************************************************************************/

int
ose_hfmi_get_next_buffer(struct HfmiBufferCursor* hbc,
                         struct HfmiBufferInfo* hbi, size_t hbi_size)
{
   struct BufferCursor* cursor = (struct BufferCursor*)hbc;
   
   struct OseHeap *heap = heapd_heap_find(cursor->href);
   if (heap == NULL)
      return HFMI_NOT_FOUND;

   /* Do request. */
   SET_SEGMENT;
   if (!l_heap_buf_next(heap, cursor))
   {
      RESTORE_SEGMENT;
      return HFMI_END_OF_LIST;
   }

   l_heap_buf_info(heap, cursor, hbi, hbi_size);
   RESTORE_SEGMENT;

   return HFMI_OK;
}

/*****************************************************************************/

int
ose_hfmi_get_buffer_file_line(struct HfmiBufferCursor* bc,
			      PROCESS* owner,
			      char* file, size_t fileLen,
			      int*  line)
{
   struct BufferCursor* cursor = (struct BufferCursor*)bc;
   struct OseHeap *heap = heapd_heap_find(cursor->href);
   HeapBufferStatus status;
   
   if (heap == NULL)
      return HFMI_NOT_FOUND;

   SET_SEGMENT;

   status = l_heap_buf_status(heap, cursor);
   if ((status & HEAP_BUFFER_ALLOCATED) &&
       (status & HEAP_BUFFER_ANNOTATED))
   {
      DefAnnotation* da = (DefAnnotation*)
         ((char*)cursor->base +
          heap->hucb.annotation_size -
          SIZEOF_DEF_ANNOTATION);

      if(da->file != NULL)
      {
         *line = HDM_GET_LINE(da->control);
         strncpy(file, da->file, fileLen);

         if(strlen(da->file) > (size_t)fileLen - 1)
         {
            RESTORE_SEGMENT;
            return HFMI_RESULT_TRUNCATED;
         }
      }
      else
      {
         /* Empty string */
         *file = 0;
      }
   }

   l_heap_buf_owner(heap, cursor, owner);
   
   RESTORE_SEGMENT;
   return HFMI_OK;
}

/*****************************************************************************/

int
ose_hfmi_find_buffer(OSHEAP href, OSADDRESS address,
		     struct HfmiBufferCursor* hbc,
		     struct HfmiBufferInfo* bi, size_t bi_size)
{
   struct BufferCursor* cursor = (struct BufferCursor*)hbc;
   
   struct OseHeap *heap = heapd_heap_find(href);
   if (heap == NULL)
      return HFMI_NOT_FOUND;

   /* Initialize for first request. */
   cursor->href  = href;
   cursor->base  = 0;
   cursor->frag  = 0;
   cursor->path  = 0;
   cursor->depth = 0;

   SET_SEGMENT;

   while (l_heap_buf_next(heap, cursor))
   {
      OSADDRESS base = (OSADDRESS)cursor->base;

      if (address >= base && address < base + l_heap_buf_size(heap, cursor))
      {
         address = base;
         break;
      }
   }

   if (cursor->base != address)
   {
      RESTORE_SEGMENT;
      return HFMI_NOT_FOUND;
   }

   l_heap_buf_info(heap, cursor, bi, bi_size);

   RESTORE_SEGMENT;

   return HFMI_OK;
}

/*****************************************************************************/

int
ose_hfmi_get_service(OSHEAP href, const struct HfmiServiceInfo **svc)
{
   HeapParam p[] = {
      HPM(QUERY_SERVICE, 1), 0,
      HPM_END
   };

   struct OseHeap *heap = heapd_heap_find(href);
   if (heap == NULL)
      return HFMI_NOT_FOUND;

   p[1] = (U32)*svc;

   zzheap_query(&heap->heap, p, 1);

   if (!HPM_IS_HANDLED(p[0]))
      return HFMI_END_OF_LIST;
   
   *svc = (const struct HfmiServiceInfo *)p[1];

   return HFMI_OK;
}

/**************************************************************************/

static void
env_to_osheap(const char *env, OSHEAP *href_out)
{
   OSADDRESS p[3];
   unsigned char *c, *x;

   memcpy(p, env, sizeof(p));

   c = (unsigned char *) &p[0];
   x = (unsigned char *) &p[1];

   for(;*x; x++, c++)
   {
      if(*c == 0x20)
      {
         *c = 0;
      }
   }

   *href_out = p[0] ^ 0x40404040;
}

/**************************************************************************/

int
ose_hfmi_get_heap_reference(PROCESS pid, OSHEAP *href)
{
   struct EfmiProcessInfo pi;
   struct EfmiEnvVarCursor cursor;
   PROCESS bid;
   char name[20], value[20];
   int rc;

   if (pid == 0)
   {
      struct EfmiCurrent current;

      rc = ose_efmi_get_current(&current, sizeof(current));
      if (rc != EFMI_OK) {
         return HFMI_NOT_FOUND;
      }

      pid = current.process;
   }

   rc = ose_efmi_get_process_info(pid, &pi, sizeof(pi));
   bid = (rc == EFMI_OK) ? pi.block : 0;
   
   rc = ose_efmi_get_first_env_var(pid, &cursor, name, 20, value, 20);
   while (rc == EFMI_OK || rc == EFMI_RESULT_TRUNCATED)
   {
      if (!strcmp(name, HEAPREF_ENVNAME))
      {
         env_to_osheap(value, href);
         return HFMI_OK;
      }

      rc = ose_efmi_get_next_env_var(&cursor, name, value);
   }

   rc = ose_efmi_get_first_env_var(bid, &cursor, name, 20, value, 20);
   while (rc == EFMI_OK || rc == EFMI_RESULT_TRUNCATED)
   {
      if (!strcmp(name, HEAPREF_ENVNAME))
      {
         env_to_osheap(value, href);
         return HFMI_OK;
      }

      rc = ose_efmi_get_next_env_var(&cursor, name, value);
   }
   
   return HFMI_NOT_FOUND;
}

/**************************************************************************/
