/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef OSE_MMAN_INTERNAL_H_
#define OSE_MMAN_INTERNAL_H_

#include "mman_pager.h"
#include "core_rbtree.h"

/* The structure storing all the VMAs in the system. */
struct vmas_t
{
   /* VMAs are stored in this rbtree tree. */
   struct core_rbtree_t tree;

   /* New VMAs are allocated from this slab. */
   struct slab_cache *slab;
};

/* Internal representation of the vm_area_t. */
struct _vm_area_t
{
   /* For the rbtree tree that the VMAs are inserted into. */
   struct core_rbtree_node_t node;

   /* The vm_area_t that is shown to the outside world. */
   struct vm_area_t vm_area;
};

/* Information about the installed pager. */
struct pager_info_t
{
   /* The function in the pager implementing the mlock() function. */
   pager_lock_func_t       *lock_func;

   /* The function in the pager implementing the munlock() function. */
   pager_unlock_func_t     *unlock_func;

   /* The function in the pager that has to be called when a physical page
    * has been moved. */
   pager_invalidate_func_t *invalidate_func;

   /* The function in the pager that is called by the file system when
    * the pager should allocate a page in the page cache. */
   KsssPagerAllocPage      *alloc_page_func;

   /* The function in the pager that removes and frees all resources
    * associated with a virtual memory area. */
   pager_unmap_vma_func_t  *unmap_vma_func;
   
   /* The pager control function used for various testing operations. */
   pager_control_func_t    *control_func;

   /* Set if the pager is installed. */
   uint8_t                  installed;
};

extern struct pager_info_t pager;

#endif /* #ifndef OSE_MMAN_INTERNAL_H_ */
