/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 ****************************************************************************

 CONTENTS
 --------

 1  Description
 2  History of development
 3  Include files
 4  Signal include files
 5  Manifest constants
 6  Macros
 7  Types
 8  Function prototypes
 9  Signal definitions
 10 Variables
 11 Functions
 12 Processes

 ****************************************************************************
 */

/*
 ****************************************************************************
 * 1  DESCRIPTION.
 *
 * 
 */

/*
 ****************************************************************************
 * 2  HISTORY OF DEVELOPMENT.
 *
 *    $Log:   /rtt/pvcsarch/OSE114/src/include/list.h_v  $
 * 
 *    Rev 1.2   09/04/99 18:37:04   olli
 * 
 *    Rev 1.1   01/09/99 01:09:52   olli
 * 
 *    Rev 1.0   12/14/98 19:34:22   olli
 * Initial revision.
 * 
 */

#ifndef _OLDLIST_H
#define _OLDLIST_H

/*
 ****************************************************************************
 * 3  INCLUDE FILES.
 ****************************************************************************
 */

#include <stddef.h>
#include "assert.h"
#include "heapapi.h"

/*
 ****************************************************************************
 * 4  SIGNAL INCLUDE FILES.
 ****************************************************************************
 */

/*
 ****************************************************************************
 * 5  MANIFEST CONSTANTS.
 ****************************************************************************
 */

/*
 ****************************************************************************
 * 6  MACROS.
 ****************************************************************************
 */

#undef INLINE
#if defined __lint
#define INLINE
#elif defined __GNUC__ || defined __DCC__
#define INLINE __inline__
#else
#define INLINE
#endif

/*
 ****************************************************************************
 * 7  TYPES.
 ****************************************************************************
 */

struct Link
{
	struct List *list;		/* Ptr to list, NULL if in no list. */
	struct Link *next;		/* Ptr to next element in list. */
	struct Link *prev;		/* Ptr to previous element in list. */
	size_t size;			/* Size of this element. */
							/* Place for user data. */
};

struct List
{
	struct List *list;		/* Ptr to list, i.e. always points to this struct. */
	struct Link *next;		/* Ptr to first element in list. */
	struct Link *prev;		/* Ptr to last element in list. */
	size_t elements;		/* Number of elements in list. */
};

/*
 ****************************************************************************
 * 8  FUNCTION PROTOTYPES.
 ****************************************************************************
 */

/*
 ****************************************************************************
 * 9  SIGNAL DEFINITIONS.
 ****************************************************************************
 */

/*
 ****************************************************************************
 * 10  VARIABLES.
 ****************************************************************************
 */

/*
 ****************************************************************************
 * 11  FUNCTIONS.
 ****************************************************************************
 */

#ifdef LIST_DEBUG
static void 
assertList(const struct List *list, void (*assertFunc) (const struct Link *))
{
	size_t elements = 0;
	const struct Link *prev;
	const struct Link *next;
	const struct Link *link;
	/* check list head */
	assert(list->list == list);
	assert(list->next != NULL);
	assert(list->prev != NULL);
	/* forward pass, count elements */
	link = list->next;
	prev = (const struct Link *) list;
	while (link != (const struct Link *) list)
	{
		elements++;
		assert(link->list == list);
		assert(link->prev == prev);
		assert(link->size >= sizeof (struct Link));
		prev = link;
		link = link->next;
	}
	assert(elements == list->elements);
	/* backwards pass, use user supplied function to check nodes */
	link = link->prev;
	next = (const struct Link *) list;
	while (link != (const struct Link *) list)
	{
		assert(link->next == next);
		if (assertFunc) assertFunc(link);
		next = link;
		link = link->prev;
	}
}

static void 
assertNode(const void *node, void (*assertFunc) (const struct Link *))
{
	assert(((const struct Link *) node) != NULL);
	if (((const struct Link *) node)->list)
	{
		assert(((const struct Link *) node)->next->prev == ((const struct Link *) node));
		assert(((const struct Link *) node)->prev->next == ((const struct Link *) node));
		assert(((const struct Link *) node)->list == ((const struct Link *) node)->next->list);
		assert(((const struct Link *) node)->list == ((const struct Link *) node)->prev->list);
		assert(((const struct Link *) node)->size >= sizeof (struct Link));
		assertList(((const struct Link *) node)->list, NULL);
	}
	else
	{
		assert(((const struct Link *) node)->next == NULL);
		assert(((const struct Link *) node)->prev == NULL);
		assert(((const struct Link *) node)->size >= sizeof (struct Link));
	}
	if (assertFunc) assertFunc(((const struct Link *) node));
}
#else
#define assertList(list,func)
#define assertNode(node,func)
#endif

INLINE
static struct List *
#ifdef MALLOC_FILELINE
allocList(const char *file, unsigned line)
#define allocList() allocList(__FILE__, __LINE__)
#else
allocList(void)
#endif
{
	struct List *list;
#ifdef MALLOC_FILELINE
	list = (struct List *) heap_alloc_private(sizeof (struct List), file, line);
#else
	list = (struct List *) heap_alloc_private(sizeof (struct List), 0, 0);
#endif
	if (list)
	{
		list->list = list;
		list->next = (struct Link *) list;
		list->prev = (struct Link *) list;
		list->elements = 0;
		assertList(list, NULL);
	}
	return list;
}

INLINE
static void
initList(struct List *list)
{
	assert(list != NULL);
	list->list = list;
	list->next = (struct Link *) list;
	list->prev = (struct Link *) list;
	list->elements = 0;
	assertList(list, NULL);
}

INLINE
static void *
#ifdef MALLOC_FILELINE
allocNode(size_t size, const char *file, unsigned line)
#define allocNode(size) allocNode((size), __FILE__, __LINE__)
#else
allocNode(size_t size)
#endif
{
	struct Link *node;
	assert(size >= sizeof (struct Link));
#ifdef MALLOC_FILELINE
	node = (struct Link *) heap_alloc_private(size, file, line);
#else
	node = (struct Link *) heap_alloc_private(size, 0, 0);
#endif
	if (node)
	{
		node->list = NULL;
		node->next = NULL;
		node->prev = NULL;
		node->size = size;
		assertNode(node, NULL);
	}
	return node;
}

INLINE
static void
initNode(void *node, size_t size)
{
	assert(size >= sizeof (struct Link));
	((struct Link *) node)->list = NULL;
	((struct Link *) node)->next = NULL;
	((struct Link *) node)->prev = NULL;
	((struct Link *) node)->size = size;
	assertNode(((struct Link *) node), NULL);
}

INLINE
static void 
unlinkNode(void *node)
{
	assertNode(((struct Link *) node), NULL);
	if (((struct Link *) node)->list)
	{
		((struct Link *) node)->next->prev = ((struct Link *) node)->prev;
		((struct Link *) node)->prev->next = ((struct Link *) node)->next;
		((struct Link *) node)->list->elements--;
		((struct Link *) node)->list = NULL;
		((struct Link *) node)->next = NULL;
		((struct Link *) node)->prev = NULL;
	}
	assertNode(((struct Link *) node), NULL);
}

INLINE
static void 
freeNode(void *node)			/* implicit unlinkNode() */
{
#ifdef LIST_DEBUG
	unsigned char *p = node;
	size_t size = ((struct Link *) node)->size;
#endif
	unlinkNode(node);			/* asserts node */
#ifdef LIST_DEBUG
	while (size-- != 0) *p++ = 0xCC;	/* fill with easily recognizable garbage. */
#endif
	heap_free_private(node);
}

INLINE
static void 
freeNodes(struct List *list)
{
	assertList(list, NULL);
	while (list->next != (struct Link *) list)
	{
		freeNode(list->next);
	}
}

INLINE
static void 
freeList(struct List *list)		/* implicit freeNode() on nodes in list */
{
	freeNodes(list);
	heap_free_private(list);
}

INLINE
static void 
insertFirst(struct List *list, void *nodeToInsert)
{
	assertList(list, NULL);
	assertNode(((struct Link *) nodeToInsert), NULL);
	assert(((struct Link *) nodeToInsert)->list == NULL);
	((struct Link *) nodeToInsert)->list = list;
	((struct Link *) nodeToInsert)->next = list->next;
	((struct Link *) nodeToInsert)->prev = (struct Link *) list;
	list->next->prev = ((struct Link *) nodeToInsert);
	list->next = ((struct Link *) nodeToInsert);
	list->elements++;
	assertNode(((struct Link *) nodeToInsert), NULL);
	assertList(list, NULL);
}

INLINE
static void 
insertLast(struct List *list, void *nodeToInsert)
{
	assertList(list, NULL);
	assertNode(((struct Link *) nodeToInsert), NULL);
	assert(((struct Link *) nodeToInsert)->list == NULL);
	((struct Link *) nodeToInsert)->list = list;
	((struct Link *) nodeToInsert)->next = (struct Link *) list;
	((struct Link *) nodeToInsert)->prev = list->prev;
	list->prev->next = ((struct Link *) nodeToInsert);
	list->prev = ((struct Link *) nodeToInsert);
	list->elements++;
	assertNode(((struct Link *) nodeToInsert), NULL);
	assertList(list, NULL);
}

INLINE
static void 
insertAfter(void *nodeInList, void *nodeToInsert)
{
	assertNode(((struct Link *) nodeInList), NULL);
	assertNode(((struct Link *) nodeToInsert), NULL);
	assert(((struct Link *) nodeToInsert)->list == NULL);
	((struct Link *) nodeToInsert)->list = ((struct Link *) nodeInList)->list;
	((struct Link *) nodeToInsert)->next = ((struct Link *) nodeInList)->next;
	((struct Link *) nodeToInsert)->prev = ((struct Link *) nodeInList);
	((struct Link *) nodeInList)->next->prev = ((struct Link *) nodeToInsert);
	((struct Link *) nodeInList)->next = ((struct Link *) nodeToInsert);
	((struct Link *) nodeInList)->list->elements++;
	assertNode(((struct Link *) nodeInList), NULL);
	assertNode(((struct Link *) nodeToInsert), NULL);
}

INLINE
static void 
insertBefore(void *nodeInList, void *nodeToInsert)
{
	assertNode(((struct Link *) nodeInList), NULL);
	assertNode(((struct Link *) nodeToInsert), NULL);
	assert(((struct Link *) nodeToInsert)->list == NULL);
	((struct Link *) nodeToInsert)->list = ((struct Link *) nodeInList)->list;
	((struct Link *) nodeToInsert)->prev = ((struct Link *) nodeInList)->prev;
	((struct Link *) nodeToInsert)->next = ((struct Link *) nodeInList);
	((struct Link *) nodeInList)->prev->next = ((struct Link *) nodeToInsert);
	((struct Link *) nodeInList)->prev = ((struct Link *) nodeToInsert);
	((struct Link *) nodeInList)->list->elements++;
	assertNode(((struct Link *) nodeInList), NULL);
	assertNode(((struct Link *) nodeToInsert), NULL);
}

INLINE
static void *
firstNode(const struct List *list)
{
	assertList(list, NULL);
	return list->next != (const struct Link *) list ? list->next : NULL;
}

INLINE
static void *
lastNode(const struct List *list)
{
	assertList(list, NULL);
	return list->prev != (const struct Link *) list ? list->prev : NULL;
}

INLINE
static void *
nextNode(const void *node)
{
	assertNode(((const struct Link *) node), NULL);
	assert(((const struct Link *) node)->list != NULL);	/* nextNode only valid when node is in list */
	return ((const struct Link *) node)->next != (const struct Link *) ((const struct Link *) node)->list ? ((const struct Link *) node)->next : NULL;
}

INLINE
static void *
prevNode(const void *node)
{
	assertNode(((const struct Link *) node), NULL);
	assert(((const struct Link *) node)->list != NULL);	/* prevNode only valid when node is in list */
	return ((const struct Link *) node)->prev != (const struct Link *) ((const struct Link *) node)->list ? ((const struct Link *) node)->prev : NULL;
}

INLINE
static size_t 
howManyNodes(const struct List *list)
{
	assertList(list, NULL);
	return list->elements;
}

INLINE
static struct List *
whichList(const void *node)
{
	assertNode(((const struct Link *) node), NULL);
	return ((const struct Link *) node)->list;
}

static void *
traverseList1(struct List *list, void *(*func) (void *node, void *arg), void *arg)
{
	struct Link *current;
	void *stp = NULL;
	assertList(list, NULL);
	current = firstNode(list);
	while (current)
	{
		struct Link *next = nextNode(current);
		stp = func(current, arg);
		if (stp != NULL) break;
		current = next;
	}
	assertList(list, NULL);
	return stp;
}

#if 0
static void 
insertOrdered(struct List *list, void *nodeToInsert, int (*compare) (const void *, const void *))
{
	struct Link *node;
	assertList(list, NULL);
	assertNode(((struct Link *) nodeToInsert), NULL);
	assert(((struct Link *) nodeToInsert)->list == NULL);
	node = firstNode(list);
	while (node)
	{
		if (compare(((struct Link *) nodeToInsert), node) < 0) break;/* don't break if equal value, skip past */
		node = nextNode(node);
	}
	/* new node is always inserted AFTER equally valued nodes */
	if (node)
	{
		insertBefore(node, ((struct Link *) nodeToInsert));
	}
	else
	{
		insertLast(list, ((struct Link *) nodeToInsert));
	}
	assertNode(((struct Link *) nodeToInsert), NULL);
	assertList(list, NULL);
}
#endif

/*
 ****************************************************************************
 * 12 PROCESSES.
 ****************************************************************************
 */

#endif
