/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2005-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

/*
 ****************************************************************************
 *                            PROGRAM MODULE
 *
 *     $Workfile:     $
 *     Document no: 
 *     $Author:   thol  $
 *     $Date:   04/21/06 15:00:42  $
 *
 *
 ****************************************************************************

  CONTENTS
  --------

  1  Description
  2  History of development
  3  Include files
  4  Signal include files
  5  Manifest constants
  6  Macros
  7  Types
  8  Function prototypes
  9  Signal definitions
  10 Variables
  11 Functions

 ****************************************************************************
 */

/*
 ****************************************************************************
 * 1  DESCRIPTION.
 *
 */

/*
 ****************************************************************************
 * 2  HISTORY OF DEVELOPMENT.
 *
 * Initial revision.
 * 
 */

/*
 ****************************************************************************
 * 3  INCLUDE FILES.
 ****************************************************************************
 */
 
#include "stddef.h"
#include "osetypes.h"
#include "string.h"
#include "assert.h"
#include "ose_conf.h"
#include "pm_conf.h"
#include "stdio.h"
#include "ramlog.h"

/*
 ****************************************************************************
 * 5  MANIFEST CONSTANTS.
 ****************************************************************************
 */

/*
 ****************************************************************************
 * 6  MACROS.
 ****************************************************************************
 */


/*
 ****************************************************************************
 * 7  TYPES.
 ****************************************************************************
 */
 

/*
 ****************************************************************************
 * 8  FUNCTION PROTOTYPES.
 ****************************************************************************
 */

/*
 ****************************************************************************
 * 10  VARIABLES.
 ****************************************************************************
 */
 

/*
 ****************************************************************************
 * 11  FUNCTIONS.
 ****************************************************************************
 */

/* The last occurrence of a configuration takes precedence. */

size_t
pm_lm_conf_get_value(const char  *conf,
                     const char  *variable,
                     const char **value)
{
   size_t value_size = 0;
   const char *ptr;
   const char *last_value;

   if(NULL == variable || NULL == conf || NULL == value)
   {
      return 0;
   }
   *value = NULL;
   last_value = NULL;

   /*check header*/
   if(strncmp(conf, CF_CONF_MAGIC, CF_CONF_MAGIC_SIZE) == 0)   
   {            
      ptr = conf;

      while((*value = strstr(ptr, variable)) != NULL)
      {
	 *value = strchr(*value, '=');
	 if(NULL == *value)
	 {
	    *value = NULL;
	    value_size = 0;	    
	 }
	 else
	 {
	    const char *tmp = *value;
	    tmp++;
	    *value = tmp;
	   
	    tmp = strchr(*value, '\n');
	    if(tmp != NULL)
	    {
	       value_size =  (size_t)(tmp - *value);
	    }
	    /*The last parameter.*/
	    else
	    {
	       tmp = (conf + strlen(conf) + 1);
	       value_size =  (size_t)(tmp - *value);
	    }
            last_value = *value;
	 }
         
         /* Skip the current configuration line and search the rest of the
            configuration, in case there is a later occurrence. */
         ptr = *value;
      }

      /* Restore the last valid value, if we did not find any more entries
         matching the configuration name. */
      if (last_value != NULL)
      {
         *value = last_value;
      }
   }
   return value_size;
}


size_t
pm_strllen(const char * str, size_t len)
{
   size_t i;
   for (i=0; i<len; i++)
   {
      if(str[i] == '\0')
      {
	 return i;
      }
   }
   return len;
} 


size_t
pm_strcnt(const char * str, size_t size)
{
   size_t i;
   size_t strings = 0;

   /* Sanity check. */
   if(NULL == str || 0==size)
   {
      return (size_t) 0;
   }
   if(str[size-1] != '\0')
   {
      /* If half strings are detected, return -1 i.e. impossible
       * number of strings value.
       */
      return (size_t) - 1;
   }

   /* Count the number of strings. */
   for (i=0; i<size; i++)
   {
      if(str[i] == '\0')
      {
	 strings++;
      }
   }

   return strings;
}


Boolean
pm_check_conf(const char * conf, const size_t conf_size)
{
   assert(NULL != conf);
   assert(CF_CONF_MAGIC_SIZE <= conf_size);

   if( ((strlen(conf) + 1) != conf_size))
   {
      ramlog_printf("The size of the configuration do not equal"
                    " the calculated configuration size. (%i!=%i)\n",
                    (strlen(conf) + 1), conf_size);
      return False;
   }

   if (0 != strncmp(conf, CF_CONF_MAGIC, CF_CONF_MAGIC_SIZE))
   {
      char magic[CF_CONF_MAGIC_SIZE+1];
      
      (void)strncmp(magic, conf, CF_CONF_MAGIC_SIZE);
      magic[CF_CONF_MAGIC_SIZE] = '\0';
      ramlog_printf("The LMCONF segment contain an illegal magic, "
                    "(%s!=%s)\n", magic, CF_CONF_MAGIC);
      return False;
   }

   if(False == pm_check_conf_body(conf, conf_size))
   {
      return False;
   }

   if( *(conf + conf_size - 1) != '\0')
   {
      return False;
   }
  
   return True;
}

size_t
pm_conf_to_string(char * conf, size_t size)
{
   size_t no_of_strings = 0;
   char* pos;

   assert(NULL != conf);
   assert((strlen(conf) +1 ) == size);
   
   /*Check the configuration.*/
   if(False == pm_check_conf(conf, size))
   {
      return 0;
   }
   
   /*
    * Count number of strings, and replace \n terminations with \0
    */
   pos = conf;
   while ((pos = strchr (pos, '=')) != NULL)
   {
      pos = strchr (pos, '\n');
      if(pos != NULL)
      {
	 *pos = '\0';
      }
      pos++;	       
      no_of_strings++;
   }
   return no_of_strings;
}      

static Boolean
pm_user_strings_to_conf(char * conf, size_t size, size_t no_of_strings)
{
   size_t i = 0;
   assert(NULL != conf);
   
   /*
    * Replace \0 terminations with \n.
    */
   if(0 != no_of_strings && 0 != size && NULL != conf)
   {
      while (0 != no_of_strings - 1)
      {
	 if (conf[i] == '\0')
	 {
	    conf[i] = '\n';
	    no_of_strings--;
	 }
	 /*Don't go too far.*/
	 if(i > size)
	 {
	    return False;
	 }
	 i++;
      }
   }
   else
   {
      return True;
   }
   return True;
}


Boolean
pm_check_conf_body(const char* body, size_t size)
{
   Boolean status = True;
   const char * equal_p;

   assert(NULL != body);
   assert(0 != size);

#ifndef ASSERT
   (void)size;
#endif   
   
   equal_p = body;
   
   while ((equal_p = strchr (equal_p, '=')) != NULL)
   {
      char * enter_p = strchr(equal_p, '\n');
      equal_p++;
      /* The last configuration may not have '\n'. */
      if(NULL == enter_p)
      {
	 if( NULL == (equal_p = strchr (equal_p, '=')) )
	 {
	    status = True;
	    break;
	 }
	 else
	 {
	    ramlog_printf("The content of LMCONF do not follow the syntax of"
                          " name=[value]\\nname=[value]\\n....name=[value]"
                          " (%s)\n", body);
            status = False;
	    break;
	 }
      }
   }
   return status;
}


Boolean
pm_set_additional_conf(char* dest,
		       const size_t no_of_strings,
		       const size_t conf_size,
		       char * conf)
{
   char * dest_p;
   const char * conf_p;

   assert(NULL != dest);

   conf_p = conf;
   dest_p = dest;

   if (0 == strncmp(dest_p, CF_CONF_MAGIC, CF_CONF_MAGIC_SIZE))
   {
      /* Skip any initial conf magic. */
      dest_p += CF_CONF_MAGIC_SIZE;
   }
   else
   {
      /* Or add one if there is none there. */
      (void)sprintf(dest_p, "%s", CF_CONF_MAGIC);
   }
   
   if((0 !=conf_size) && (0 != no_of_strings) && (NULL != conf))
   {
      if (False == pm_user_strings_to_conf(conf, conf_size, no_of_strings))
      {
	 return False;
      }
      if (False == pm_check_conf_body(conf, conf_size))
      {
	 return False;
      }

      (void)strcat(dest_p, conf_p);
      (void)strcat(dest_p, "\n");
   }
   return True;
}
