/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 * Description: The function interface of the load module conf support in the 
 *              Program Manager. 
 * 
 */

#ifndef __PM_CONF_H
#define __PM_CONF_H

/***********************************************************
 *
 * INCLUDE FILES
 *
 ***********************************************************/

#include "osetypes.h"
#include "sys/types.h"

/* Note: this file is not adapted to C++ since the file was never intended to be
 * included from a C++ source file.
 */

/***********************************************************
 *
 * CONSTANTS AND DEFINITIONS
 *
 ***********************************************************/

/***********************************************************
 *
 * STRUCTURES AND GLOBALS
 *
 ***********************************************************/

/***********************************************************
 *
 * FUNCTION DECLARATIONS
 *
 ***********************************************************/

/*
 *===========================================================================
 *                        pm_lm_conf_get_value
 *===========================================================================
 * 
 * Description
 * -----------
 * Find and return a value corresponding to the configuration name. 
 *
 * Requirement
 * -----------
 * "Load module configuration". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * conf            The OSE configuration.
 * variable        The name of the variable.
 * value           The value of variable.
 *
 * Returns
 * -------
 * The size of value (bytes).
 *
 */

size_t
pm_lm_conf_get_value(const char *conf, const char * variable, 
		     const char ** value);

/*
 *===========================================================================
 *                        pm_strllen
 *===========================================================================
 * 
 * Description
 * -----------
 * Calculate and make sure the string lenght of a string is within bounds , 
 * without reading outside the bounds.
 *
 * Requirement
 * -----------
 * "Sanity Check". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * string                           The string to check.
 * len                              The lenght to check against.
 *
 * Returns
 * -------
 * The number of char in the file as strlen or the limit length, if the 
 * string had no teminator.
 *
 */

size_t
pm_strllen(const char * str, size_t len);

/*
 *===========================================================================
 *                        pm_strcnt
 *===========================================================================
 * 
 * Description
 * -----------
 * Count the number of strings in a char array.
 *
 * Requirement
 * -----------
 * "Sanity Check". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * string                           The string to check.
 * len                              The number of elements in the char array.
 *
 * Returns
 * -------
 * The number of strings in the char array, if an incomplete string is
 * detected, return -1.
 *
 */

size_t
pm_strcnt(const char * str, size_t size);

/*
 *===========================================================================
 *                   pm_check_conf
 *===========================================================================
 * 
 * Description
 * -----------
 * Make sure that the confiration is OK.
 *
 * Requirement
 * -----------
 * "". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * conf                           The configuration.
 * size                           The size of configuration.
 *
 * Returns
 * -------
 * True if OK or False.
 *
 */

Boolean
pm_check_conf(const char * conf, const size_t conf_size);


/*
 *===========================================================================
 *                        pm_conf_to_string
 *===========================================================================
 * 
 * Description
 * -----------
 * Convert OSE configuration to strings.
 *
 * Requirement
 * -----------
 * "". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * conf                           The configuration.
 * size                           The size of configuration.
 *
 * Returns
 * -------
 * The number of configurations converted to strings. 
 *
 */

size_t
pm_conf_to_string(char * conf, size_t size);

/*
 *===========================================================================
 *                        pm_user_string_to_conf
 *===========================================================================
 * 
 * Description
 * -----------
 * Convert user conf strings to OSE configuration.
 *
 * Requirement
 * -----------
 * "". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * conf                           The configuration.
 * size                           The size of configuration.
 * no_of_strings                  # of strings in conf
 *
 * Returns
 * -------
 * True upon success else False. 
 *
 */

Boolean
pm_user_string_to_conf(char * conf, size_t size, size_t no_of_strings);


/*
 *===========================================================================
 *                        pm_check_conf_body
 *===========================================================================
 * 
 * Description
 * -----------
 * Check the main body (after CONF_MAGIC\n of OSE configuration.
 *
 * Requirement
 * -----------
 * "". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * conf                           The configuration.
 * size                           The size of configuration.
 *
 * Returns
 * -------
 * True upon success else False.
 *
 */

Boolean
pm_check_conf_body(const char* body, size_t size);

/*
 *===========================================================================
 *                        pm_check_conf_body
 *===========================================================================
 * 
 * Description
 * -----------
 * Description: Insert user override configuration in OSE configuration.
 *
 * Requirement
 * -----------
 * "". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 *  dest                   Destination for additional configuration.
 *  no_of_strings          No of entries in the confuration.
 *  conf_size              The size of the additionl configuration.
 *  conf                   Client override configuration.
 *
 * Returns
 * -------
 * True upon success else False.
 *
 */
Boolean
pm_set_additional_conf(char* dest,
		       const size_t no_of_strings,
		       const size_t conf_size,
		       char * conf);

#endif /* __PM_CONF_H */
