/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 * Description: The function interface of the dynamic libraries support in the 
 *              Program Manager. 
 * 
 */

#ifndef __PM_DYNAMIC_LIBRARIES_H
#define __PM_DYNAMIC_LIBRARIES_H

/***********************************************************
 *
 * INCLUDE FILES
 *
 ***********************************************************/

#include "pm_internal.h"
#include "pm.sig"

/* Note: this file is not adapted to C++ since the file was never intended to be
 * included from a C++ source file.
 */

/***********************************************************
 *
 * CONSTANTS AND DEFINITIONS
 *
 ***********************************************************/

/***********************************************************
 *
 * STRUCTURES AND GLOBALS
 *
 ***********************************************************/

/***********************************************************
 *
 * FUNCTION DECLARATIONS
 *
 ***********************************************************/

/*
 *===========================================================================
 *                        pm_import_dyn_lib_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_IMPORT_DYN_LIB_REQUEST signal i.e import a library to a running 
 * program.
 *
 * Requirement
 * -----------
 * "Dynamic functionality importation". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_IMPORT_DYN_LIB_REQUEST signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_import_dyn_lib_request(struct PmInternalState * PmInternalState, 
			  struct PmImportDynLib * request);


/*
 *===========================================================================
 *                        pm_no_of_libraries
 *===========================================================================
 * 
 * Description
 * -----------
 * Count the number of libraries created from a specific load module.
 *
 * Requirement
 * -----------
 * "Library information". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * install_handle                   The install handle of the load module.
 *
 * Returns
 * -------
 * The number of libraries created from the load module.
 *
 */

GLOBAL U32
pm_no_of_libraries(struct PmInternalState * pmInternalState, 
		  char * install_handle);

/*
 *===========================================================================
 *                        pm_libraries_info_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_LIBRARIES_INFO_REQUEST signal i.e reply the install handles of
 * all libraries of a running program.
 *
 * Requirement
 * -----------
 * "Dynamic libraries". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_LIBRARIES_INFO_REQUEST signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_libraries_info_request(struct PmInternalState * PmInternalState, 
			  struct PmLibrariesInfoRequest * request);

#endif /* __PM_DYNAMIC_LIBRARIES_H */








