/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2008 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 * Description: The function interface of the event subscription support in 
 *              the Program Manager. 
 *              
 */

#ifndef __PM_EVENTS_H
#define __PM_EVENTS_H

/***********************************************************
 *
 * INCLUDE FILES
 *
 ***********************************************************/

/* Program Management headers. */
#include "pm_internal.h"

/* Program Management block signal interface. */
#include "pm.sig"

/* Note: this file is not adapted to C++ since the file was never intended
 *       to be included from a C++ source file.
 */

/***********************************************************
 *
 * CONSTANTS AND DEFINITIONS
 *
 ***********************************************************/

/***********************************************************
 *
 * STRUCTURES AND GLOBALS
 *
 ***********************************************************/

/*
 *===========================================================================
 *                        EventSubscription
 *===========================================================================
 * 
 * Description
 * -----------
 * Definition of an element in the event subscriptions list
 *
 * Parameter description
 * ---------------------
 * link           The linked list structure.
 * client         The client that subscribes to an event.
 * event          The event that the client subscribes to.
 *
 */

typedef struct
{
  struct Link  link;  
  PROCESS      client;
  U32          event;
  OSATTREF     attref;
} EventSubscription;

/***********************************************************
 *
 * FUNCTION DEFINITIONS
 *
 ***********************************************************/
  
/*
 *===========================================================================
 *                        pm_subscribe_event_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_SUBSCRIBE_EVENT_REQUEST signal.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_SUBSCRIBE_EVENT_REQUEST signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_subscribe_event_request(struct PmInternalState * PmInternalState, 
                           struct PmSubscribeEventRequest *request);

/*
 *===========================================================================
 *                        pm_unsubscribe_event_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_UNSUBSCRIBE_EVENT_REQUEST signal.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_UNSUBSCRIBE_EVENT_REQUEST signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_unsubscribe_event_request(struct PmInternalState * PmInternalState, 
                             struct PmUnsubscribeEventRequest * request);

/*
 *===========================================================================
 *                        pm_event_remove_client
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_ATTACH_EVENT_SUBSCRIPTION signal.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_ATTACH_EVENT_SUBSCRIPTION signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_event_remove_client(struct PmInternalState * PmInternalState, 
                       SIGSELECT * request);

/*
 *===========================================================================
 *                        pm_event_notify
 *===========================================================================
 *
 * Description
 * -----------
 * Notify all clients of an event occurrence.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * PmEvent                          The event that occured.
 * extra                            The extra information to pass to clients
 *                                  in PmOccurredEvent struct. 
 * extra_size                       The size of the extra information.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_event_notify(struct PmInternalState * PmInternalState, 
                U32 PmEvent, 
                void * extra, 
                size_t extra_size);

#endif /* __PM_EVENTS_H */
