/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 * Description: The function interface of the load modules support in the 
 *              Program Manager. 
 * 
 */

#ifndef __PM_LOAD_MODULES_H
#define __PM_LOAD_MODULES_H

/***********************************************************
 *
 * INCLUDE FILES
 *
 ***********************************************************/

/* Program Management headers. */
#include "pm_internal.h"

/* Program Management block signal interfaces. */
#include "pm.sig"
#include "rtl.sig"

/* Note: this file is not adapted to C++ since the file was never intended to be
 * included from a C++ source file.
 */

/***********************************************************
 *
 * CONSTANTS AND DEFINITIONS
 *
 ***********************************************************/

/***********************************************************
 *
 * STRUCTURES AND GLOBALS
 *
 ***********************************************************/

/***********************************************************
 *
 * FUNCTION DEFINITIONS
 *
 ***********************************************************/

/*
 *===========================================================================
 *                        pm_search_lm_install_handle
 *===========================================================================
 * 
 * Description
 * -----------
 * Search for a specific load module using the install handle (as part of 
 * traverseList1).
 *
 * Requirement
 * -----------
 * "Install handle". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * node                             The node (in the list) to compare to.
 * name                             The install handle to search for.
 *
 * Returns
 * -------
 * (void *) 0                       The node did not match the install handle, 
 *                                  ZERO is returned.
 * (void *) node                    The node matched the install handle, the 
 *                                  pointer to the node is returned.
 *
 *
 */

GLOBAL void * 
pm_search_lm_install_handle(void * node, void * install_handle);

/*
 *===========================================================================
 *                        pm_lm_install_handles_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_LOAD_MODULES_INSTALL_HANDLES_REQUEST signal.
 *
 * Requirement
 * -----------
 * "load module information".
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_LOAD_MODULE_INSTALL_HANDLES_REQUEST signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_lm_install_handles_request(struct PmInternalState* PmInternalState, 
			      struct PmLoadModuleInstallHandlesRequest *request);

/*
 *===========================================================================
 *                        pm_lm_info_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_LOAD_MODULE_INFO_REQUEST signal.
 *
 * Requirement
 * -----------
 * "load module information".
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_LOAD_MODULE_INFO_REQUEST signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_lm_info_request(struct PmInternalState * PmInternalState,
		   struct PmLoadModuleInfoRequest * request);

/*
 *===========================================================================
 *                        pm_lm_section_info_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_LOAD_MODULE_SECTION_INFO_REQUEST signal.
 *
 * Requirement
 * -----------
 * "load module information".
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_LOAD_MODULE_SECTION_INFO_REQUEST signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_lm_section_info_request(struct PmInternalState * PmInternalState,
			   struct PmLoadModuleSectionInfoRequest *request);

/*
 *===========================================================================
 *                        pm_lm_configuration_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_LOAD_MODULE_CONFIGURATION_REQUEST signal.
 *
 * Requirement
 * -----------
 * "load module information".
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_LOAD_MODULE_CONFIGURATION_REQUEST signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_lm_configuration_request(struct PmInternalState * PmInternalState, 
			    struct  PmLoadModuleConfigurationRequest *request);

/*
 *===========================================================================
 *                        pm_resurrect_load_modules
 *===========================================================================
 * 
 * Description
 * -----------
 * Reinstall load modules after cold/warm reset.
 *
 * Requirement
 * -----------
 * "Restore load modules".
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void 
pm_resurrect_load_modules(struct PmInternalState * PmInternalState);

/*
 *===========================================================================
 *                        pm_install_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_INSTALL_LOAD_MODULE_REQUEST signal.
 *
 * Requirement
 * -----------
 * "Install load module".
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_INSTALL_LOAD_MODULE_REQUEST signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_install_request(struct PmInternalState * PmInternalState, 
		   struct PmInstallLoadModuleRequest * request);

/*
 *===========================================================================
 *                        pm_uninstall_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_UNINSTALL_LOAD_MODULE_REQUEST signal.
 *
 * Requirement
 * -----------
 * "Uninstall load module".
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_UNINSTALL_LOAD_MODULE_REQUEST signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_uninstall_request(struct PmInternalState * PmInternalState, 
		     struct PmUninstallLoadModule * request);

/*
 *===========================================================================
 *                        pm_commit_load_module_data
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a RTL_COMMIT_LOAD_MODULE_DATA signal. Store the data in the meta 
 * data structure and inform the client of the installed load module.
 *
 * Requirement
 * -----------
 * "Install load module".
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The RTL_COMMIT_LOAD_MODULE_DATA signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
rtl_commit_load_module_data(struct PmInternalState * PmInternalState, 
			    struct RtlCommitLoadModuleData * reply);



/*
 *===========================================================================
 *                        pm_checksum
 *===========================================================================
 * 
 * Description
 * -----------
 * Calculate an adler32 checksum.
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * base                             The base of the region to checksum.
 * size                             The sizo of the region to checksum.
 *
 * Returns
 * -------
 * The checksum.
 *
 */

GLOBAL U32
pm_checksum(OSADDRESS base, U32 size);

/*
 *===========================================================================
 *                        pm_checksum_unt_flush
 *===========================================================================
 * 
 * Description
 * -----------
 * Checksum the region and flush the instruction cache.
 *
 * Requirement
 * -----------
 *
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The RTL_FLUSH_AND_CHECKSUM_REQUEST signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_checksum_unt_flush(struct PmInternalState * pmInternalState,
		      struct RtlFlushAndChecksum *request);

/*
 *===========================================================================
 *                        pm_load_module_checksum_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Checksum the region and compare the checksum to the expected value.
 *
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_LOAD_MODULE_CHECKSUM_REQUEST signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_load_module_checksum_request(struct PmInternalState * pmInternalState,
				struct PmLoadModuleChecksum *request);

#endif /* __PM_LOAD_MODULES_H */
