/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 * Description: The function interface of the program types support in the 
 *              Program Manager. 
 * 
 */

#ifndef __PM_PROGRAM_TYPES_H
#define __PM_PROGRAM_TYPES_H

/***********************************************************
 *
 * INCLUDE FILES
 *
 ***********************************************************/

/* Program Management headers. */
#include "pm_internal.h"

/* Program Management block signal interface. */
#include "pm.sig"

/* Note: this file is not adapted to C++ since the file was never intended to be
 * included from a C++ source file.
 */

/***********************************************************
 *
 * CONSTANTS AND DEFINITIONS
 *
 ***********************************************************/

/***********************************************************
 *
 * STRUCTURES AND GLOBALS
 *
 ***********************************************************/

/***********************************************************
 *
 * FUNCTION DEFINITIONS
 *
 ***********************************************************/
  
/*
 *===========================================================================
 *                        pm_search_prog_type
 *===========================================================================
 * 
 * Description
 * -----------
 * Search for a specific program type using the program type name (as part of 
 * traverseList1).
 *
 * Requirement
 * -----------
 * "Program type handling". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * node                             The node (in the list) to compare to.
 * name                             The name of a program type to search for.
 *
 * Returns
 * -------
 * (void *) 0                       The node did not match the name, ZERO is 
 *                                  returned.
 * (void *) node                    The node matched the name, the pointer to
 *                                  the node is returned.
 *
 *
 */

GLOBAL void * 
pm_search_prog_type(void * node, void * name);

/*
 *===========================================================================
 *                        pm_add_program_type_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_ADD_PROGRAM_TYPE_REQUEST signal. I.e. determine if the 
 * configuration is adding a new program type or if it modifies an existing 
 * one. Store the program type configuration and reply a status message.  
 *
 * Requirement
 * -----------
 * "program type handling". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_ADD_PROGRAM_TYPE_REQUEST signal. 
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_add_program_type_request(struct PmInternalState * PmInternalState, 
			    struct PmProgramTypeInfo * request);


/*
 *===========================================================================
 *                        pm_program_type_info_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_PROGRAM_TYPE_INFO_REQUEST signal
 * 
 * Requirement
 * -----------
 * "program type handling". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_PROGRAM_TYPE_INFO_REQUEST signal. 
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_program_type_info_request(struct PmInternalState * PmInternalState, 
			     struct PmProgramTypeInfoRequest * request);


/*
 *===========================================================================
 *                        pm_program_types_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_PROGRAM_TYPES_REQUEST signal
 * 
 * Requirement
 * -----------
 * "program type handling". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_PROGRAM_TYPES_REQUEST signal. 
 *
 * Returns
 * -------
 * None.
 *
 */
GLOBAL void
pm_program_types_request(struct PmInternalState * PmInternalState, 
			 struct PmProgramTypesRequest * request);


/*
 *===========================================================================
 *                        pm_sanity_check_program_region_type
 *===========================================================================
 * 
 * Description
 * -----------
 * Sanity check the contents of a program region.
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * program_type                  The program type associated with the region.
 * region                        The region to check.
 * region_type                   Type of region (text, data, heap, etc).
 *
 * Returns
 * -------
 * PM_SUCCESS                    The check was successful.
 * PM_EPROGRAM_TYPE_SYNTAX_ERROR There was a syntax error. 
 * PM_EILLEGAL_REGION_LOG_MEM    Region specified MAS-mapped log_mem for
 *                               supervisor mode program type. 
 */

GLOBAL PmStatus
pm_sanity_check_program_region_type(struct PmInternalState * PmInternalState,
                                    ProgramType *program_type,
                                    struct PmProgramRegionType * region,
				    U32 region_type);

#endif /* __PM_PROGRAM_TYPES_H */
