/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 * Description: The function interface of the program support in the 
 *              Program Manager. 
 * 
 */

#ifndef __PM_PROGRAMS_H
#define __PM_PROGRAMS_H

/***********************************************************
 *
 * INCLUDE FILES
 *
 ***********************************************************/

/* Program Management headers. */
#include "pm_internal.h"

/* Program Management block signal interface. */
#include "pm.sig"

/* Note: this file is not adapted to C++ since the file was never intended to be
 * included from a C++ source file.
 */

/***********************************************************
 *
 * CONSTANTS AND DEFINITIONS
 *
 ***********************************************************/

/***********************************************************
 *
 * STRUCTURES AND GLOBALS
 *
 ***********************************************************/

/***********************************************************
 *
 * FUNCTION DEFINITIONS
 *
 ***********************************************************/

/*
 *===========================================================================
 *                        pm_search_by_progpid
 *===========================================================================
 * 
 * Description
 * -----------
 * Search for a specific program using the progam pid (as 
 * part of traverseList1).
 *
 * Requirement
 * -----------
 * "Program pid". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * node                             The node (in the list) to compare to.
 * progpid                          The program pid to search for.
 *
 * Returns
 * -------
 * (void *) 0                       The node did not match the progpid, 
 *                                  ZERO is returned.
 * (void *) node                    The node matched the progpid, the 
 *                                  pointer to the node is returned.
 * 
 */

#if 0
GLOBAL void *
pm_program_search_by_progpid(void * node, void *progpid);
#endif

/*
 *===========================================================================
 *                        pm_search_program_by_install_handle
 *===========================================================================
 * 
 * Description
 * -----------
 * Search for a specific load module using the programs install handle (as 
 * part of traverseList1).
 *
 * Requirement
 * -----------
 * "Programs install handle". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * node                             The node (in the list) to compare to.
 * name                             The install handle to search for.
 *
 * Returns
 * -------
 * (void *) 0                       The node did not match the install handle, 
 *                                  ZERO is returned.
 * (void *) node                    The node matched the install handle, the 
 *                                  pointer to the node is returned.
 *
 *
 */

GLOBAL void * 
pm_search_program_by_install_handle(void * node, void * install_handle);

/*
 *===========================================================================
 *                        pm_program_search_by_mainpid
 *===========================================================================
 * 
 * Description
 * -----------
 * Search for a specific program using the programs mainpid (as 
 * part of traverseList1).
 *
 * Requirement
 * -----------
 * "Programs mainpid". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * node                             The node (in the list) to compare to.
 * name                             The mainpid to search for.
 *
 * Returns
 * -------
 * (void *) 0                       The node did not match the mainpid, 
 *                                  ZERO is returned.
 * (void *) node                    The node matched the mianpid, the 
 *                                  pointer to the node is returned.
 *
 *
 */

#if 0
GLOBAL void *
pm_program_search_by_mainpid(void * node, void *main_pid);
#endif

/*
 *===========================================================================
 *                        pm_create_program_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_CREATE_PROGRAM_REQUEST signal.
 *
 * Requirement
 * -----------
 * "Create program". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_CREATE_PROGRAM_REQUEST signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_create_program_request(struct PmInternalState * PmInternalState, 
			  struct PmCreateProgramRequest * request);

/*
 *===========================================================================
 *                        pm_program_killed
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a killed program.
 *
 * Requirement
 * -----------
 * "Kill program". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The attach signal of the killed program.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_program_killed(struct PmInternalState * PmInternalState, 
		  SIGSELECT * request);


/*
 *===========================================================================
 *                        pm_program_info_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_PROGRAM_INFO_REQUEST signal.
 *
 * Requirement
 * -----------
 * 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * state                  The PM internal state structure.
 * request                The request signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_program_info_request (struct PmInternalState * state,
                         struct PmProgramInfo * request);


/*
 *===========================================================================
 *                        pm_program_configuration_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_PROGRAM_CONFIGURATION_REQUEST signal.
 *
 * Requirement
 * -----------
 * 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * state                  The PM internal state structure.
 * request                The request signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_program_configuration_request ( struct PmInternalState * state,
                                   struct PmProgramConfigurationRequest * 
                                   request);

/*
 *===========================================================================
 *                        pm_programs_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_PROGRAMS_REQUEST signal.
 *
 * Requirement
 * -----------
 * 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * state                  The PM internal state structure.
 * request                The request signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_programs_request ( struct PmInternalState * state,
                      struct PmProgramsRequest * request );


/*
 *===========================================================================
 *                        pm_get_program_pid_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_GET_PROGRAM_PID_REQUEST signal.
 *
 * Requirement
 * -----------
 * 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * state                  The PM internal state structure.
 * request                The request signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_get_program_pid_request (struct PmInternalState * state,
                            struct PmGetProgramPid * request );


/*
 *===========================================================================
 *                        pm_install_handle_to_program_pid_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_INSTALL_HANDLE_TO_PROGRAM_PID_REQUEST signal.
 *
 * Requirement
 * -----------
 * 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * state                  The PM internal state structure.
 * request                The request signal.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_install_handle_to_program_pid_request (struct PmInternalState *state,
                                          struct PmInstallHandleToProgramPidRequest * 
                                          request );

/*
 *===========================================================================
 *                        pm_get_domain_number_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_GET_DOMAIN_NUMBER_REQUEST signal.
 *
 * Requirement
 * -----------
 * 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * state                  The PM internal state structure.
 * request                The request signal.
 *
 * Returns
 * -------
 * None.
 *
 */

extern void
pm_get_domain_number_request ( struct PmInternalState * PmInternalState,
                               struct PmGetDomainNumber * request );

/*
 *===========================================================================
 *                        pm_no_of_programs
 *===========================================================================
 * 
 * Description
 * -----------
 * Count the number of programs created from a specific load module.
 *
 * Requirement
 * -----------
 * "Program information". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * install_handle                   The install handle of the load module.
 *
 * Returns
 * -------
 * The number of programs created from the load module.
 *
 */

GLOBAL U32
pm_no_of_programs(struct PmInternalState * pmInternalState, 
		  char * install_handle);

/*
 *===========================================================================
 *                        pm_init_program_data
 *===========================================================================
 * 
 * Description
 * -----------
 * Initialize the program data at program start.
 *
 * Requirement
 * -----------
 * "Program start". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * progpid                          The started programs progpid.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_init_program_data(struct PmInternalState * PmInternalState, PROCESS progpid);

/*
 *===========================================================================
 *                        pm_start_program_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Request to start a program.
 *
 * Requirement
 * -----------
 * "Program start". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The request of the operation.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_start_program_request(struct PmInternalState * PmInternalState, 
			 struct PmStartProgramRequest * request);

/*
 *===========================================================================
 *                        pm_kill_program_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Request to kill a program.
 *
 * Requirement
 * -----------
 * "Program kill". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The request of the operation.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_kill_program_request(struct PmInternalState * PmInternalState, 
			struct PmKillProgram * request);


/*
 *===========================================================================
 *                        pm_attach_program_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Request to attach to a program.
 *
 * Requirement
 * -----------
 * "Program attach". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The request of the operation.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_attach_program_request(struct PmInternalState * PmInternalState, 
			  struct PmAttachProgram * request);

/*
 *===========================================================================
 *                        pm_extend_program_memory_request
 *===========================================================================
 *
 * Description
 * -----------
 * Request to extend the program memory with a new region.
 *
 * Requirement
 * -----------
 * "Program new memory".
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The request of the operation.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_extend_program_memory_request(struct PmInternalState * PmInternalState, 
                               struct PmExtendProgramMemory * request);

/*
 *===========================================================================
 *                        pm_move_program_to_cpu_request
 *===========================================================================
 *
 * Description
 * -----------
 * Request to move a program to a specific CPU.
 *
 * Requirement
 * -----------
 *
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The request of the operation.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_move_program_to_cpu_request(struct PmInternalState * PmInternalState, 
                               struct PmMoveProgramToCPU * request);

#endif /* __PM_PROGRAMS_H */
