/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
 * Description: The function interface of the rtl management support in the 
 *              Program Manager. 
 * 
 */

#ifndef __PM_VS_RTL_H
#define __PM_VS_RTL_H

/***********************************************************
 *
 * INCLUDE FILES
 *
 ***********************************************************/

/* Program Management headers. */
#include "pm_internal.h"

/* Program Management block signal interface. */
#include "pm.sig"

/* Note: this file is not adapted to C++ since the file was never intended to be
 * included from a C++ source file.
 */

/***********************************************************
 *
 * CONSTANTS AND DEFINITIONS
 *
 ***********************************************************/

/***********************************************************
 *
 * STRUCTURES AND GLOBALS
 *
 ***********************************************************/

/***********************************************************
 *
 * FUNCTION DEFINITIONS
 *
 ***********************************************************/

/*
 *===========================================================================
 *                        pm_rtl_search_by_ff
 *===========================================================================
 * 
 * Description
 * -----------
 * Search for a specific RTL using the file format (as part of 
 * traverseList1).
 *
 * Requirement
 * -----------
 * "RTL file format". 
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * node                             The node (in the list) to compare to.
 * ff                               The name of the file format to search for.
 *
 * Returns
 * -------
 * (void *) 0                       The node did not match the file format, 
 *                                  ZERO is returned.
 * (void *) node                    The node matched the file format, 
 *                                  the pointer to the node is returned.
 *
 *
 */

GLOBAL void *
pm_rtl_search_by_ff(void * node, void *ff);

/*
 *===========================================================================
 *                        pm_rtl_search_by_pid
 *===========================================================================
 * 
 * Description
 * -----------
 * Search for a specific RTL using the RTL pid (as part of 
 * traverseList1).
 *
 * Requirement
 * -----------
 * "RTL management"
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * node                             The node (in the list) to compare to.
 * pid                              The RTL pid to search for.
 *
 * Returns
 * -------
 * (void *) 0                       The node did not match the pid, 
 *                                  ZERO is returned.
 * (void *) node                    The node matched the pid, 
 *                                  the pointer to the node is returned.
 *
 *
 */

GLOBAL void *
pm_rtl_search_by_pid(void * node, void * pid);

/*
 *===========================================================================
 *                        pm_rtl_killed
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a killed RTL.
 *
 * Requirement
 * -----------
 * "RTL management"
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * sig                              The attach signal of the lost RTL.
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
pm_rtl_killed(struct PmInternalState * PmInternalState, 
	      SIGSELECT * sig);

/*
 *===========================================================================
 *                        pm_connect_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a RTL_CONNECT_REQUEST signal.
 *
 * Requirement
 * -----------
 * "RTL management"
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The RTL_CONNECT_REQUEST signal. 
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
rtl_connect_request(struct PmInternalState * PmInternalState, 
		    struct RtlConnectRequest * request);

/*
 *===========================================================================
 *                        pm_interface_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a RTL_INTERFACE_REQUEST signal.
 *
 * Requirement
 * -----------
 * "RTL management"
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * request                          The RTL_INTERFACE_REQUEST signal. 
 *
 * Returns
 * -------
 * None.
 *
 */

GLOBAL void
rtl_interface_request(struct RtlInterfaceRequest * request);


/*
 *===========================================================================
 *                        pm_file_formats_request
 *===========================================================================
 * 
 * Description
 * -----------
 * Handle a PM_FILE_FORMATS_REQUEST signal.
 *
 * Requirement
 * -----------
 * "RTL management"
 *
 * Side effects
 * ------------
 * None.
 *
 * Parameter description
 * ---------------------
 * PmInternalState                  The PM internal state structure.
 * request                          The PM_FILE_FORMATS_REQUEST signal. 
 *
 * Returns
 * -------
 * None.
 *
 */
GLOBAL void 
pm_file_formats_request(struct PmInternalState * PmInternalState,
			struct PmFileFormatsRequest * request);

#endif /* __PM_VS_RTL_H */
