/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
*****************************************************************************
** Description: The header file with macros, types and prototypes for
**              operations on 64 bits integers.
**
** History:     1999-11-06 Initial version
**              1999-11-08 Support for both little and big endian
**              2002-08-01 olli - make use of 'long long' datatype
**              2002-12-13 olli - disable use of 'long long' for Diab.
**
*****************************************************************************
*/

#if !defined(LITTLE_ENDIAN) && !defined(BIG_ENDIAN)
#error LITTLE_ENDIAN or BIG_ENDIAN must be defined
#endif

#ifndef _I64_TYPES_H
#define _I64_TYPES_H

/* Don't enable long long when compiling with Diab (for PowerPC) because
   GCC does not support the 64bit-routines references by Diab. */
#ifdef __DCC__
#undef HAS_LONGLONG
#endif

#define U64 U64_disabled
#include "osetypes.h"
#undef U64

/*
*****************************************************************************
**
** MACROS.
**
*****************************************************************************
*/

/* Make a 64 bits unsigned integer out of two 32 bits unsigned integers. */
#ifdef BIG_ENDIAN
#define I64_MAKE(r, high, low) do{(r)->l[0]=(low);(r)->l[1]=(high);}while(0)
#else
#define I64_MAKE(r, high, low) do{(r)->l[1]=(low);(r)->l[0]=(high);}while(0)
#endif

/* CLeaR x by setting it to zero. */
#define I64_CLR(x) do{(x)->l[0]=(x)->l[1]=0;}while(0)

/* 2 raised to the POWer of e is returned in x. */
#ifdef BIG_ENDIAN
#define I64_POW2(x, e) do{if((e)<=31){(x)->l[1]=1<<(e);(x)->l[0]=0;}else{(x)->l[1]=0;(x)->l[0]=1<<((e)-32);}}while(0)
#else
#define I64_POW2(x, e) do{if((e)<=31){(x)->l[0]=1<<(e);(x)->l[1]=0;}else{(x)->l[0]=0;(x)->l[1]=1<<((e)-32);}}while(0)
#endif

/* SHift x one bit Right. */
#ifdef BIG_ENDIAN
#define I64_SHR1(x) do{(x)->l[1]>>=1;if((x)->l[0]&0x00000001)(x)->l[1]|=0x80000000;(x)->l[0]>>=1;}while(0)
#else
#define I64_SHR1(x) do{(x)->l[0]>>=1;if((x)->l[1]&0x00000001)(x)->l[0]|=0x80000000;(x)->l[1]>>=1;}while(0)
#endif

/* SHift x one bit Left. */
#ifdef BIG_ENDIAN
#define I64_SHL1(x) do{(x)->l[0]<<=1;if((x)->l[1]&0x80000000)(x)->l[0]|=0x00000001;(x)->l[1]<<=1;}while(0)
#else
#define I64_SHL1(x) do{(x)->l[1]<<=1;if((x)->l[0]&0x80000000)(x)->l[1]|=0x00000001;(x)->l[0]<<=1;}while(0)
#endif

/* Is x Less than or Equal to y? */
#ifdef BIG_ENDIAN
#define I64_LE(x, y) (((x)->l[0]!=(y)->l[0])?((x)->l[0]>(y)->l[0])?0:1:((x)->l[1]!=(y)->l[1])?((x)->l[1]>(y)->l[1])?0:1:1)
#else
#define I64_LE(x, y) (((x)->l[1]!=(y)->l[1])?((x)->l[1]>(y)->l[1])?0:1:((x)->l[0]!=(y)->l[0])?((x)->l[0]>(y)->l[0])?0:1:1)
#endif

#define U64add(a,b,c) rtc_U64add(a,b,c)
#define U64sub(a,b,c) rtc_U64sub(a,b,c)
#define U64mul(a,b,c) rtc_U64mul(a,b,c)
#define U64div(a,b,c) rtc_U64div(a,b,c)

#define U64_ADD(r, x, y) U64add(r,x,y)
#define U64_SUB(r, x, y) U64sub(r,x,y)
#define U64_MUL(r, x, y) U64mul(r,x,y)
#define U64_DIV(r, x, y) U64div(r,x,y)

/*
*****************************************************************************
**
** TYPES.
**
*****************************************************************************
*/

/* U64 range 0 - 18 446 744 073 709 551 615 */

typedef union U64
{
#ifdef HAS_LONGLONG
  unsigned long long ll;
#endif
  U32 l[2]; /* long */
  U16 w[4]; /* word */
  U8  b[8]; /* byte */
} U64;


/*
*****************************************************************************
**
** FUNCTION PROTOTYPES.
**
*****************************************************************************
*/

extern void U64add(U64 *r, U64 *x, U64 *y);
extern void U64sub(U64 *r, U64 *x, U64 *y);
extern void U64mul(U64 *r, U64 *x, U64 *y);
extern void U64div(U64 *r, U64 *x, U64 *y);

#endif
