/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2005-2010 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#include "efs.h"
#include "ose.h"
#include "ose_spi/ose_heap.h"
#include "sysparam.h"
#include "compiler.h"

static PROCESS zzose_shared_block;

static const char sysparam_name[] = "core/shared_block";

int zzcore_create_shared_block(OSADDRESS crt_init_func);

WEAK_REF(OS_PROCESS(ose_shared_main));

/* Found in core/crt/src/support/ose_crt_init.c */
void ose_set_shared_model(PROCESS bid);

PROCESS
efs_get_ose_shared_block(void)
{
   return zzose_shared_block;
}

static
OS_PROCESS(zzose_shared_init)
{
   void (*init_func)(void) = (void (*)(void))get_envp(0, "osecrtinit");

   init_func();

   if (ose_shared_main != NULL)
   {
      ose_shared_main();
   }
   else
   {
      for(;;)
      {
	 stop(current_process());
      }
   }
}
   
int
zzcore_create_shared_block(OSADDRESS crt_init_func)
{
   Boolean stopped = False;
   OSHEAP  href;
   PROCESS proc;
   long stack_size;
   long priority;
   
   if (sysparam_readattr_long(sysparam_name, "stack_size", &stack_size) != 0)
      return 1;

   if (sysparam_readattr_long(sysparam_name, "prio", &priority) != 0)
      priority = get_pri(current_process());

   zzose_shared_block = create_block("ose_shared_block",
				      0,  /* user               */
				      0,  /* use_remote_calls   */
				      0,  /* remote_call_server */
				      1); /* supervisor_mode    */

   ose_set_shared_model(zzose_shared_block);

   href = ose_create_heap(zzose_shared_block,
			  0,     /* domain          */
			  True); /* supervisor mode */

   proc = create_process(OS_PRI_PROC,
			 "ose_shared_main",
			 zzose_shared_init,
			 (OSADDRESS)stack_size,
			 (OSPRIORITY)priority,
			 0,    /* timeslice  */
			 zzose_shared_block,
			 NULL, /* redir      */
			 0,    /* vector     */
			 0);   /* user       */

   set_envp(proc, "osecrtinit", crt_init_func);

   (void)sysparam_readattr_bool(sysparam_name, "stopped", &stopped);

   if (stopped == False)
      start(zzose_shared_block);
   return 0;
}
