
/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*

	CONTENTS
	--------

	1  Description
	2  Include files
	3  Common definitions
	   3.1 Manifest constants
	   3.2 Type definitions
	4  Function prototypes
	   4.1 Backdoors provided by the kernel

*****************************************************************************
*/

#ifndef _CORE_INTERNAL_H
#define _CORE_INTERNAL_H

/*
*****************************************************************************
** 1  DESCRIPTION.
*****************************************************************************
** Core internal interfaces and definitions.
*/

/*
*****************************************************************************
** 2  INCLUDE FILES.
*****************************************************************************
*/

#include "ose.h"

/*
*****************************************************************************
** 3  COMMON DEFINITIONS.
*****************************************************************************
*/

#ifdef __cplusplus
extern "C" { 
#endif	/* __cplusplus */
#include "ramlog.h"

/*
 * replace call to ramlog_printf with krintf which requires less stack
 */
#ifdef OSKERNEL
size_t kprintf(const char *fmt, ...);
#define ramlog_printf kprintf
#endif

/*
**===========================================================================
** 3.1  Manifest constants.
**===========================================================================
*/

/*
**===========================================================================
** 3.2  Type definitions.
**===========================================================================
*/
typedef OSADDRESS(OSPOOLEXTENDER)(PROCESS, OSADDRESS, OSADDRESS, 
                                  void*, void**);

/*
*****************************************************************************
** 4  FUNCTION PROTOTYPES.
*****************************************************************************
*/

/*
**===========================================================================
** 4.1  Backdoors provided by the kernel.
**===========================================================================
*/

/*
** Faster version of get_pid_info for supervisor processes in the monolith.
*/
OSADDRESS
zzkrn_get_pid_info(PROCESS pid, OSADDRESS info_tag);

/*
** Read or set the migration_inhibited mask of a segment.
**
** The function is used by the Program Manager and the Bios to
** lock a segment to the currently selected cpu in a SMP system.
** If the set_mask argument is zero, the current value is returned.
** If the set_mask is non-zero, the corresponding bits are taken
** from set_value, and the new value is returned. 
** Mask MIGRATION_MASK_PM is owned by the Program Manager, mask
** MIGRATION_MASK_BIOS by Bios.
** If the segment is not found, ~0 is returned.
*/

#define MIGRATION_MASK_PM 1
#define MIGRATION_MASK_BIOS 2

OSADDRESS
zzkrn_segment_migration_status(PROCESS pid, OSADDRESS set_mask,
			       OSADDRESS set_value);

/*
** Access of per process debug_info of the current process.
** Used by epm.
*/
unsigned char *
zzfind_current_debug_info(void);

/*
** Create a possibly extendible pool. Used by PM.
** The first five parameters are the same as in the system call create_pool().
** The rest are as follows:
**         fill   Fill the pool fragments with initialization pattern?
**     ext_func   Pointer to function in PM that does the region allocation.
**    frag_size   The size of each extend fragment.
**     max_size   Maximum allowed size of the pool.
**  prog_type_p   Pointer to program type struct for the pool's program.
*/
void
zzkrn_create_program_pool(PROCESS bid,
                          OSADDRESS base,
                          OSADDRESS size,
                          const OSADDRESS sigsize_tab[],
                          const OSADDRESS stacksize_tab[],
                          OSBOOLEAN fill,
                          OSPOOLEXTENDER *ext_func,
                          OSADDRESS frag_size,
                          OSADDRESS max_size,
                          void* prog_type_p);

#ifdef __cplusplus
} /* end of extern "C" */
#endif	/* __cplusplus */

#endif	/* _CORE_INTERNAL_H */

