/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef _FMM_H
#define _FMM_H

#include "stdint.h"

/**
 * @macro FMM_STACK_SIZE
 * @brief Size of the internal stack.
 */
#define FMM_STACK_SIZE (2048)

/**
 * @macro FMM_INPUT_SIGNAL_SIZE
 * @brief Max size of a signal in the input buffer.
 */
#define FMM_INPUT_SIGNAL_SIZE (1020)

/**
 * @macro FMM_OUTPUT_SIGNAL_SIZE
 * @brief Max size of a signal in the output buffer.
 */
#define FMM_OUTPUT_SIGNAL_SIZE (8188)

typedef struct FmmSignal FmmSignal;

/**
 * @type FmmSwitchStackArgs
 * @brief Arguments passed to function after switching stack.
 */
typedef volatile struct FmmSwitchStackArgs
{
   uint32_t arg1;
   uint32_t arg2;
   uint32_t arg3;
   uint32_t arg4;
   uint32_t arg5;
   uint32_t arg6;
} FmmSwitchStackArgs;

/**
 * @type FmmProcQueueCnt
 * @brief Structure to hold process id and respective signal queue count.
 */

typedef struct FmmProcQueueCnt
{
   uint32_t pid;
   uint32_t sig_cnt;
} FmmProcQueueCnt;

/**
 * @type FmmFmdProperties
 * @brief Fmd properties to be used by fmm.
 */

typedef struct FmmFmdProperties
{
   uint32_t num_long_queue_procs;
   uint32_t max_signals_in_queue;
   uint32_t max_stack_top_size;
   FmmProcQueueCnt *proc_queue_list;
   uint32_t process_count;
} FmmFmdProperties;


/**
 * @type FmmSwitchStackFunc
 * @brief Function to call after switching stack
 */
typedef uint32_t (*FmmSwitchStackFunc)(uint32_t arg1, uint32_t arg2,
                                       uint32_t arg3, uint32_t arg4,
                                       uint32_t arg5, uint32_t arg6);
                                       
/**
 * @data fmm_stack
 * @brief Internal stack.
 */
extern uint32_t *fmm_stack;

/**
 * @data fmm_input_signal
 * @brief Internal input signal buffer
 */
extern FmmSignal *fmm_input_signal;

/**
 * @data fmm_output_signal
 * @brief Internal output signal buffer
 */
extern FmmSignal *fmm_output_signal;

/**
 * @function fmm_switch_stack
 *
 * @brief Switches the stack pointer, calls the function and then
 * switches the stack back.
 *
 * @input new_sp New stack pointer.
 * @input func Function to call after switching stack.
 * @input args Arguments to pass on to function.
 *
 * @return Return code from the called function.
 */
uint32_t fmm_switch_stack(uint32_t *new_sp, 
                          FmmSwitchStackFunc func,
                          FmmSwitchStackArgs *args);


/**
 * @function fmm_efmi_set_fmd_properties
 *
 * @brief This function is used to set fmd properties for use in fmm.
 * It is called by fmd module.
 * 
 * @input fmm_fmd_set_properties structure pointer.
 *
 * @return None.
 */
void fmm_efmi_set_fmd_properties(FmmFmdProperties *fmm_fmd_set_properties);



#endif
