/* COPYRIGHT-ENEA-SRC-R1 *
 **************************************************************************
 * Copyright (C) 2008 by Enea Embedded Technology AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#include "ose.h"
#include "ramlog.h"
#include "bios.h"
#include "hal.h"
#include "cache_trap.h"
#include "cache_err.h"

/* Struct holding pointers to the fuctions above */
static struct cache_bios_funcs cache_funcs;


/*
 *===========================================================================
 *                        cache_bios_interface
 *===========================================================================
 */
static
long cache_bios_interface(long fc,
                          long arg1,
                          long arg2,
                          long arg3,
                          long arg4,
                          long arg5,
                          long arg6)
{
   long result;

   (void)arg1;
   (void)arg2;
   (void)arg3;
   (void)arg4;
   (void)arg5;
   (void)arg6;
   
   switch (fc)
   {
      case CACHE_TRAP_GET_INTERFACE_VERSION:
      {
	 result = (long)CACHE_TRAP_INTERFACE_VERSION;
	 break;
      }
      case CACHE_TRAP_GET_LAST_FCODE:
      {
	 /* Update when adding function codes to the cache trap interface */
	 result = (long)CACHE_TRAP_LAST_FCODE;
	 break;
      }
      case CACHE_TRAP_GET_FUNCS:
      {
         cache_funcs.zzose_cache = zzose_cache;
         cache_funcs.zzose_cache_query = zzose_cache_query;
         /* Should match the number of elements exported */
         cache_funcs.no_of_elements = CACHE_BIOS_NO_OF_ELEMENTS;
         
         result = (long)&cache_funcs;
	 break;
      }
      default:
      {
	 result = -1;
	 break;
      }
   }
   return result;
}




/*
 *===========================================================================
 *                        cache_init_bios_interface
 *===========================================================================
 */

void
cache_init_bios_interface(void)
{
   long status;

   status = biosInstall(CACHE_BIOS_NAME,
                        cache_bios_interface,
                        BIOS_SUPERVISOR_ONLY);

   if(status != BIOS_SUCCESS)
   {
      /* The install failed. */
      error(CACHE_EBIOS_INSTALL_FAILED);
      stop(current_process());
   }
   ramlog_printf("Cache bios installed.\n");

}
