/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @file epm_internal.h
 *
 * @brief Execution Point Module Private Interface
 */

#ifndef _EPM_INTERNAL_H
#define _EPM_INTERNAL_H

#include "stddef.h"
#include "ose.h"
#include "osetypes.h"
#include "efmi.h"
#include "exception.h"
#include "epm.h"
#include "stdint.h"

void
zzepm_install_exception_handler(exception_handler_c handler);

/* Get next pc to report to the debugger. */
OSADDRESS
zzepm_get_next_pc(struct cpu_regdump *dump, U32 options,
		  OSADDRESS instruction);

/* Get next pc for use as single instruction trace target. */
OSADDRESS
zzepm_get_trace_to_pc(struct cpu_regdump *dump, OSADDRESS instruction);

/* Get pc from register dump. (May differ from dump->pc) */
OSADDRESS
zzepm_get_dump_pc(struct cpu_regdump *dump);

OSADDRESS
zzepm_get_sw_ep(OSADDRESS address, OSADDRESS *opcode, int *bytes_in_opcode);

void
zzepm_set_sw_ep(OSADDRESS address);

void
zzepm_set_opcode(OSADDRESS address, OSADDRESS opcode);

EpmStatus
zzepm_get_opcode(OSADDRESS address, OSADDRESS *opcode);

int
zzepm_emulate_instruction(struct cpu_regdump *dump, OSADDRESS opcode);

U32
zzepm_lock_save_dump(struct cpu_regdump *dump);

void
zzepm_lock_restore_dump(struct cpu_regdump *dump, U32 sr);

void
zzepm_debug_save_regs(void);

void
zzepm_debug_restore_regs(void);

/*
 * curr_opcode - opcode to generate simulation code for
 * address     - where the opcode original was placed
 * type        - Pointer to a 32bit value to indicate how to handle the 
 *               simulation in the callback, this value is only used by 
 *               zzepm_handle_transposed_opcode() to do some smart moves.
 *               If type is NULL the code need to generate code that "works" 
 *               by itself or this functions shall return error, 
 *               This is used for fast ep's where the callback might never be 
 *               called. See zzepm_setup_fast_ep_codearea()
 * opcode_area - where to place the simulation code
 * opcode_area_size - This is the size that the code need to fit into.
 * 
 * Return EPM_OK is all is well, otherwise some other EPM errocode
 * 
 */

EpmStatus
zzepm_transpose_opcode(OSADDRESS curr_opcode, OSADDRESS address,
      uint32_t *type, uint32_t *opcode_area, uint32_t opcode_area_size);

EpmStatus
zzepm_handle_transposed_opcode(struct cpu_regdump *dump, 
                              uint32_t type, uint32_t *opcode_area);

EpmStatus
zzepm_setup_fast_ep_codearea(  OSADDRESS curr_opcode, 
         OSADDRESS address, uint32_t *opcode_area, uint32_t opcode_area_size,
         OSADDRESS check_callback_code_area , OSADDRESS *emulate_pointer );

void
zzemp_handle_veneer_trap(struct cpu_regdump *dump, OSADDRESS exception_pc);

/* Returns true if an ep exists on the supplied address.
 * The saved opcode is returned.
 */
int
epm_get_opcode_from_list(OSADDRESS address, U32 *opcode);

/**
 *    set_ep
 *      A function to set the execution point. E.g. put a breakpoint
 *      instruction at address or setting a breakpoint register.
 *
 *    clear_ep
 *      Removes a breakpoint set by set_ep.
 *
 *    get_opcode_from_state
 *      If the installed breakpoint mechanism is able to extract the opcode
 *      from the state, then this argument shall be a pointer to that function.
 *
 *    init
 *      A function that will be called by epm when the first ep is set. It
 *      shall install an ep handler.
 *
 *    dump_state
 *      A function that will be called by epm if an unrecoverable error
 *      occurs.
 **/

#define SUB_STATE_SIZE 2UL

typedef U32 sub_state_t[SUB_STATE_SIZE];

struct zzepm_bp_conf {
   EpmStatus (*set_bp)(OSADDRESS, sub_state_t);
   EpmStatus (*clear_bp)(OSADDRESS, sub_state_t);
   OSADDRESS (*get_opcode_from_state)(sub_state_t);
   EpmStatus (*init)(exception_handler_c);
   void (*dump_state)(void);
};

extern const struct zzepm_bp_conf *zzepm_hwbp_conf;

/*
** Check an address for functions to avoid stepping into.
** Return zero if the address is considered harmless.
** Exported from the generic layer for use by the architecture specific layer.
*/
int
zzepm_avoid_step_into(OSADDRESS address);

#endif /* _EPM_INTERNAL_H */
