/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2009 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

/*
*****************************************************************************
** HISTORY OF DEVELOPMENT.
*****************************************************************************
** date      responsible           notes
** --------  --------------------  ------------------------------------------
** 09/09/24  Fredrik Bredberg      Initial release.
** 09/10/06  Mona Nilsson          Added ifdef for OSE_SMP
** 10/05/03  Magnus Gille          Changed status for sysd_lite
*/

#include "osenames.h"
#include "ose_smp.h"
#include "core_internal.h"
#include "pose.con"
#include "pose.h"
#include "pose_internal.h"
#include "ose_err.h"
#include "ramlog.h"


extern void zzprof_disable_core(cpuid_t core); /* "rmm/prof.c" */

#define PREFIX "exec: "

void
zzshutdown_cpu(void)
{
#ifdef OSE_SMP
   
  cpuid_t cpu = zzkrn_cpu_id();
  PROCESS curr = current_process();
  struct _pcb *sysd_lite = (struct _pcb *)get_pid_info(zzcpu[cpu].sysd_lite_,
						       7 /* OSE_PI_PCB_PTR */);
  struct _pcb *pcb;
  OSADDRESS pix, saved_pix;

  /*
  ** Make sure we don't try to shutdown cpu 0, also make sure we are
  ** called from a prioritized or background process.
  */
  if(cpu == 0 || zzcur_p->priority > 32)
    {
      zzos_error(0, OSE_EILLEGAL_SYSTEMCALL, (OSERRCODE) 0);	/* Non fatal. */
      return;
    }

  ramlog_printf(PREFIX "Detach cpu%u\n", cpu);

  /* Turn off profiling on the current cpu. */
  ramlog_printf(PREFIX "Disable profiling.\n");
  zzprof_disable_core(cpu);

  /*
  ** Lower our priority to just below sysd_lite_'s in order for it to 
  ** finish all its work before we shut it down.
  */
  if (sysd_lite->priority < 31 && zzcur_p->priority < 31)
    set_pri(sysd_lite->priority + 1);

  do
    {
      pix = zzuu.next_free_pix;
      saved_pix = pix;

      while(pix > 3)
	{
	  LOCKSTATE msr;
	  struct _pcb *pcb = zzuu.pix_list[--pix];
	  PROCESS pid = 0;
  
	  /* Disable interrupts. */
	  BKL_LOCK_SAVE(msr);

	  /* Is it a process on the current cpu? */
	  if(pcb->magic == MAGIC_PCB && pcb->cpu_id == cpu)
	    {
	      pid = pcb->pid;
	    }

	  /* Enable interrupts. */
	  BKL_LOCK_RESTORE(msr);

	  if(pid != 0 &&
	     pid != zzcpu[cpu].sysd_lite_ &&
	     pid != curr &&
	     get_bid(pid) != get_bid(zzuu.sysd_))
	    {
	      char *name = (char *)get_pid_info(pid, OSE_PI_NAME);
	      ramlog_printf(PREFIX "Killing %s [%#x].\n",
			    name != NULL ? name : "???", pid);
	      if (name != NULL)
		free_buf((union SIGNAL **)&name);
	      kill_proc(pid);
	    }
	}

      /* Allow sysd_lite_ to finish all its work. */
      set_pri(get_pri(zzcpu[cpu].sysd_lite_) + 1);
    }
  while(saved_pix != (OSADDRESS)zzuu.next_free_pix);

  /* Stop tick messages to this cpu. */
  ramlog_printf(PREFIX "Cpu%u entering detached state.\n", cpu);
  zzcpu[cpu].state = CPU_STATE_DETACHED;

  ramlog_printf(PREFIX "Stopping ose_sysd%02u\n", cpu);
  stop(zzcpu[cpu].sysd_lite_);
  sysd_lite->priority = 128; /* OS_PHANTOM */
  sysd_lite->status = (pr_state_t)ST_STOPPED;
  
  ramlog_printf(PREFIX "Stopping idle%02u\n", cpu);
  pcb = zzcpu[cpu].idle_pcb;
  pcb->status = (pr_state_t)ST_STOPPED;
  pcb->st_cnt = 1;
  pcb->priority = 128; /* OS_PHANTOM */
  
  zzcur_p->priority = 128; /* OS_PHANTOM */
  /* Now I'm a phantom, but no one can stop me from running! */

  /* Make updated variables visible for other cpus */
  SMP_STORE_MEMBAR();

#endif 
}
