/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2012 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
*****************************************************************************
**                            HEADER MODULE                                **
**                                                                         **
**     Subsystem:               Portable OSE.                              **
**     Original author:         Bengt Eliasson.                            **
**     Summary:                 OSE kernel.                                **
**                                                                         **
*****************************************************************************

	CONTENTS
	--------

	1  Description
	2  History of development
	3  Header files
	4  Local definitions
	   4.1 Manifest constants
	   4.2 Type definitions
	5  Global Variables

*****************************************************************************
*/

/*
*****************************************************************************
** 1  DESCRIPTION.
*****************************************************************************
** This file contains internal macros and types for a portable OSE kernel.
**
*/

/*
*****************************************************************************
** 2  HISTORY OF DEVELOPMENT.
*****************************************************************************
** date      responsible           notes
** --------  --------------------  ------------------------------------------
** 03/03/13  Gunnar Eklund	   Extracted from pose.c
** 04/01/15  Patrik Stromblad      Added per process data and the usrcalls
**                                 array.
** 04/02/18  Fredrik Bredberg      Converted a pad byte to is_deaf in _pcb.
** 04/06/07  Gunnar Eklund	   Padding ready_list_entry to 16 bytes to
**                                 avoid multiplication.
** 04/07/07  Gunnar Eklund	   Adding pagesize to the world struct.
** 04/11/08  Fredrik Bredberg      Added bg_search_next to the world struct.
** 04/12/01  Patrik Stromblad	   Adding shared mode flag to bcb.
** 05/01/26  Gunnar Eklund         Added U8 debug_info to _pcb.
** 05/03/02  Fredrik Bredberg      Added send_hook handlig to _pcb.
** 05/03/17  Gunnar Eklund         Added U8 out_of_pool to _pcb.
** 05/04/12  Karl Morner           Added more send_hook handlig to _pcb.
** 05/11/09  Gunnar Eklund         Changed magic values for expired and 
**                                 detached attach control blocks.
** 05/12/02  Gunnar Eklund         Added free_attempts to poolcb. Added
**                                 zzprof_alloc and zzprof_free_buf to
**                                 handlerp.
** 05/12/28  Gunnar Eklund         Added mutcb control blocks to solve 
**                                 problems with mutexes in uninitiated ram.
** 06/04/24  Fredrik Bredberg      Added pool_delete_cnt to world to solve
**                                 a race in check_buffer().
** 06/05/04  Gunnar Eklund         Added time resolution data to struct world.
** 06/09/18  Fredrik Bredberg      Added the zzcpu array in order to
**                                 vectorize global variables. POSE goes SMP!
** 06/10/16  Fredrik Bredberg      Added cpu_id to struct _block.
** 06/12/19  Gunnar Eklund         Added next_toscb to struct world.
** 07/01/22  Gunnar Eklund         Configurable size of tmolist and 
**                                 tos_tick_list. New MAGIC_KERNEL_TYPE.
** 07/02/22  Fredrik Bredberg      Added alloced_cpu to struct _pcb.
** 07/03/16  Fredrik Bredberg      Getting and setting zzcur_p is now
**                                 handled by functions in SMP systems,
**                                 as is getting the dispatcher_lock_flag.
** 07/03/20  Gunnar Eklund         Added migration_inhibited to struct segcb.
**                                 Changed idle_varp in osvarp from a pointer
**                                 to a handle. Stepped MAGIC_KERNEL_TYPE.
** 07/12/27  Gunnar Eklund         Added mas_mapped to struct segcb.
** 08/04/28  Gunnar Eklund         Added toscb_spin_lock to struct world.
**                                 Adding spin locks per poolcb and per
**                                 signal free list.
**                                 Stepping MAGIC_KERNEL_TYPE for SMP and
**                                 unicore. 
** 08/10/23  Mikael Wanggren       Added entries in struct poolcb related to
**                                 pool extension functionality.
** 08/11/11  Robin Westberg        Added pointer to main pool in world struct
**                                 and in osvarp.
** 08/11/19  Fredrik Bredberg      Added BKL (Big Kernel Lock) stuff for SMP.
** 09/02/20  Gunnar Eklund         Added sig_cnt_queued and  wake_tick_cnt to
**                                 struct _pcb. Added kill_in_progress to
**                                 struct _block. Added ipi_operable and
**                                 sysd_lite_ to struct per_cpu.
** 09/03/19  Gunnar Eklund         Added member prof in struct _block.
**                                 Stepping MAGIC_KERNEL_TYPE for SMP and
**                                 unicore. 
** 09/04/15  Gunnar Eklund         Added member pool_alignment in struct 
**                                 world.
** 09/04/21  Mathias Bergvall      Made per_cpu structure 2048 bytes for x86.
** 09/05/26  Robin Westberg        Added pointer to lnh name to _block.
** 09/10/11  Robin Westberg        Added vector and ISR control blocks, and
**                                 vectorcb array to world struct. Also
**                                 added list of ISRs to pcb, bcb & segcb.
** 09/10/23  Fredrik Bredberg      Added info to the pool control block to
**                                 support a better pool extender on SMP.
**                                 Also added the nil_alloc flag to _pcb.
** 09/11/26  Gunnar Eklund         Renamed called_in to state and added
**                                 enum cpu_state.
** 09/11/20  Robin Westberg        Added the process state ST_RECEIVE_FSEM.
** 09/12/04  Fredrik Bredberg      Changed ticks into a 64-bit variable,
**                                 also removed the LDM world structure
**                                 pointer, thus stepping MAGIC_KERNEL_TYPE
**                                 for both SMP and unicore.
** 10/01/26  Gunnar Eklund         Adding an attach list to each pool.
** 10/03/03  Fredrik Bredberg      Added fsend()/polling receive handlig to
**                                 _pcb.
** 10/03/09  Gunnar Eklund         Added Counters for timer processes, 
**                                 pending timeouts, resp. pending
**                                 timeout signal requests, to speed up 
**                                 entry to the power save handler.
** 10/03/23  Fredrik Bredberg      Added struct ose_krn_msg to per_cpu for
**                                 polling burst receive.
** 10/04/19  Gunnar Eklund         added stop_state to struct per_cpu.
** 10/05/19  Fredrik Bredberg      Added support for external alloc/free_buf
**                                 hooks.
** 10/09/08  Fredrik Bredberg      Added proc_attrs to struct _pcb.
** 10/09/24  Fredrik Bredberg      Stepping MAGIC_KERNEL_TYPE for SMP and
**                                 unicore.
** 10/09/29  Anders Hagstrom       Added function evt_interrupt_generate in
**                                 handlerp.
**                                 Changed prototype for zzmsg_enqueue.
** 10/10/14  Anders Hagstrom       Added hw_poller in the per_cpu struct.
** 10/10/29  Anders Hagstrom       Removed struct ose_krn_msg.
** 10/11/18  Fredrik Bredberg      Added main_block to segcb.
** 11/04/12  Gunnar Eklund         Added the statehookcb control block.
** 11/05/02  Gunnar Eklund         ARM going multicore.
** 11/06/10  Fredrik Bredberg      Added bg_receive_poll to struct per_cpu.
** 11/09/20  Gunnar Eklund         Removed non_rt_free_frag_list.
**                                 Shifted down pool extend_count value.
**                                 Renamed poolcb.realtime_extend_ability
**                                 to realtime_extendable.
** 12/05/14  Gunnar Eklund         Reorganized struct _pcb for ARM to
**                                 increase performance.
** 12/10/03  Gunnar Eklund         Made s_pcb.take_listmodnumber volatile.
** 13/03/05  Gunnar Eklund         Added info on latest error to struct _pcb.
** 13/08/15  Fredrik Bredberg      Added cur_tmo_expires to struct _pcb.
**
*****************************************************************************
*/

#ifndef _POSE_INTERNAL_H
#define _POSE_INTERNAL_H

/*
*****************************************************************************
** 3  HEADER FILES
*****************************************************************************
*/
#include "per_cpu_data.h"
#include "spinlock.h"

/*
*****************************************************************************
** 4  LOCAL DEFINITIONS.
*****************************************************************************
*/

/*
**===========================================================================
** 4.1  Manifest constants.
**===========================================================================
*/


/*
** The system uses the same set of buffers for pool control blocks, block descriptors,
** segment control blocks and process control blocks.
**
** When a process or block id is examined, there must be a way to determine
** what kind of control block is at hand. The kernel uses a magic number for
** this purpose.
**
** Also, when a process etc is killed, the magic number is changed so that
** the MAGIC_KILLED bit is set. This bit actually reflects the state of
** the pcb->killed flag, and is used to increase efficiency in the
** check_pid(), check_pid_bid() and check_pid_bid_seg() etc functions.
**
** Please note that the numbers are selected such that tests can be made by
** masking off the least significant bits as follows:
**   block or process:            if((magic & 0xfffe) == MAGIC_PCB)
**   block or process or segment: if((magic & 0xfffc) == MAGIC_PCB)
**
** The constants defined below are the magic numbers used.
**
** See also: struct poolcb, struct _pcb, struct _block
*/



#define MAGIC_PCB	((U16)0x5718)
#define MAGIC_BCB	((U16)0x5719)
#define MAGIC_SEG	((U16)0x571A)
#define MAGIC_POOL	((U16)0x5520)
#define MAGIC_KILLED	((U16)0x8000)

/*
** The kernel needs a way to check that a semaphore id is legal.
** Therefore, each semaphore control block in kernel space contains a
** magic number, and a semaphore id is accepted only if it points to
** a semaphore control block containing this magic number.
**
** See also: struct semcb
*/

#define MAGIC_SEMAPHORE	((U16)0x5522)

/*
** The kernel uses this magic number to check if a signal administration block
** has been overwritten.
** The value depends on the meory model, as this would affect post mortem
** analyzer tools and other debugger tools.
** See also: struct _signal.
*/

#if MEMORY_MODEL > 0
#define MAGIC_SIGNO	((U16)0x4092)
#else
#define MAGIC_SIGNO	((U16)0x4091)
#endif

/*
** This byte is used to mark the end of the user space in signal buffers.
** The end mark is used to detect buffer overwrite errors.
**
** New stacks are also filled with the end mark. It is used to detect
** stack overflow errors and to calculate stack usage.
**
** The end mark is a single byte. A 16 or 32 bit word might have been safer
** and also faster, but several extra bytes would have been wasted for each
** signal.
*/

#define END_MARK (0xee)

  /* The address alignment of any object in any pool.
  ** DMA may require cache line alignment of signals.
  */
#define POOL_ALIGNMENT (zzuu.pool_alignment)

/*
** This constant indicates the version of the post mortem analysis interface.
** It applies to the pose package, and has nothing to do with the current
** target interface or the compiler used to create the kernel.
**
** It is changed whenever internal kernel structures are modified in a way
** that is incompatible with old post mortem analysis tools.
**
** New tools should support old kernel types as well as new types.
**
** See also: struct osvarp, MAGIC_KERNEL_FEATURES.
*/
#ifdef OSARM
#ifdef OSE_SMP
#define MAGIC_KERNEL_TYPE (0x19481008)  /* ARM specific pcb */
#else
#define MAGIC_KERNEL_TYPE (0x87581261)  /* ARM specific pcb */
#endif
#else
#ifdef OSE_SMP
#define MAGIC_KERNEL_TYPE (0x1968092E)
#else
#define MAGIC_KERNEL_TYPE (0x78457699)
#endif
#endif
/*
** This constant indicates the kernel feature level to post mortem analysis
** tools.
**
** It is incremented whenever a new feature is added to the kernel, as long
** as the new feature still does not affect old tools.
**
** New tools test on this constant to enable new features when they are
** relevant.
**
** The constant is set to zero for each new kernel type, i.e whenever the
** MAGIC_KERNEL_TYPE is changed.
**
** See also: struct osvarp, MAGIC_KERNEL_TYPE.
*/

#define MAGIC_KERNEL_FEATURES (0x0)

/* 
** The kernel needs a way to check the state of expired attach entries.
** We indicate that by setting magic values in the own_bq pointer in an 
** attach entry. Note that these magic values are NOT 32-bit aligned,
** thus the kernel will hopefully crash if ever try to use it as a
** pointer.
**
** See also: struct _attach_entry, the detach() system call.
*/

#define MAGIC_EXPIRED_UNSENT_ATTREF ((void *)0x58C9)
#define MAGIC_EXPIRED_SENT_ATTREF   ((void *)0x58E9)
#define MAGIC_DETACHED_ATTREF       ((void *)0x58D9)

/*
**===========================================================================
** 4.2  Type definitions.
**===========================================================================
*/

enum cpu_state
  {
    CPU_STATE_LIMBO,		/* Not yet active */
    CPU_STATE_ACTIVE,		/* Running OSE. */
    CPU_STATE_DETACHED		/* Activated and then detached. */
  };

/*
**===========================================================================
** 4.2.1  General type definitions.
**===========================================================================
*/

/*
** The time out signal control block (toscb) and friends.
*/

struct tos_head
{
  struct toscb	*head;		/* For active time outs and free list. */
  struct toscb	*tail;		/* Only used for active time outs. */
};

/* 2log of number of toscb per alloc chunk. */
#define TOSCB_ALLOC_2LOG 4
	
/* Number of toscb per alloc chunk. */
#define TOSCB_ALLOC_CHUNK (1 << TOSCB_ALLOC_2LOG) 

struct toscb
{
  PROCESS	pid;		/* Client to receive the signal. */
  struct _signal *sigadm;	/* Timeout signal, valid iff (flags & 2). */
  SIGSELECT	signo;		/* User provided signo of timeout signal. */
  struct toscb	*fq;		/* List of pending tos per pcb,
				   expired tos per pcb, and the free list. */
  struct toscb	*bq;		/* List of pending and expired tos per pcb */
  struct toscb	*tick_fq;	/* List of pending tos per tick slot */
  struct toscb	*tick_bq;	/* List of pending tos per tick slot */
  U32		count;		/* Time left, with unit 256 tick */
  U32		index;		/* Part of return value from request_tmo. */
  U16		instance;	/* Timer instance on this toscb index. */
  U16		tickslot;	/* Tick list slot. */
  U8		flags;		/* 0 == free,
				   1 == pending without allocated signal,
				   3 == pending with user allocated signal,
				   6 == expired with sigadm saved.
				*/
};

/*
** The type of a sort key in a splay tree.
** Redesign this type as required for each application.
*/

typedef const void * KEYTYPE;

/*
** Each node in a splay tree structure must contain at least three fields.
** These are pointers to the left and right children and to the parent node.
** There is also a next pointer used when there are several nodes with the
** same key. These are stored i an linear sublist to the  first node.
** Finally a contents pointer is used to hold the user data o each node.
** This pointer is accessed to find the sort key for the node.
**
** The n pointer is NULL in the last entry of the linear list.
** The p pointer is NULL for the root node.
** The l and r pointers are NULL if no resp children are present.
**
** If p = l and not NULL, then the node is the first node in a linear list.
** n points at the next node and l points at the parent node in the splay
** tree.
**
** If p = r and not NULL, then the node is a node further down a linear list.
** n points at the next node and r points at the previous node.
*/

struct treenode
{
  struct treenode *splay_n;	/* Linear sublist pointer. */
  struct treenode *splay_p;	/* Parent pointer. */
  struct treenode *splay_l;	/* Left child. */
  struct treenode *splay_r;	/* Right child. */
  void *data;			/* Contents pointer. */
};

typedef struct treenode treenode;

/*
** The PCB_INDEX_T type is used for variables holding a process index.
** This type must be a 16 bit unsigned type.
**
** See also zzuu.pix_list[].
*/

typedef U16 PCB_INDEX_T;

/*
** The PCB_INSTANCE_T type is used for variables holding a process instance
** number.
** This type must be a 16 bit unsigned type.
**
** See also zzuu.pix_list[].
*/

typedef U16 PCB_INSTANCE_T;

/*
** The structures below are all related to "signal ports.
** They are used in pcb's and in the s-nodelist allocated by
** each process that tries to receive signals through an s-port.
*/

/* This is the S-port related data in each PCB. */
struct S_pcb
{
  struct S_node *s_nodelist;	/* struct S_nodelist * for non phantoms. */
  U32 add_listmodnumber;	/* Bumped when new sig added to the tail. */
  volatile U32 take_listmodnumber; /* Bumped when a signal is taken from the queue. */
  struct S_node *s_nodetail;	/* Tail pointer for phantoms. */
};

/* A pointer to this structure is in each phantom pcb. */
struct S_node
{
  struct _pcb *phantom_pcb;	/* The phantom with which we are associated or NULL if not in use. */
  struct S_nodelist *base;	/* Ref to start of my S_nodelist. */
  U32 add_listmodnumber;	/* Copy of simil in the phantom pcb. */
  struct S_node *fp_s_nodelist; /* For phantoms. */
  struct S_node *bp_s_nodelist; /* For phantoms. */
  PROCESS phantom_pid;		/* Used to return OS_ATTACH_SIG when an sport dies. */
};

/* A pointer to this structure is in each user pcb. */
#define MAX_SNODELIST 32
struct S_nodelist
{
  struct _pcb *my_pcb;
  U32 maxcount;			/* Size of s_node[]. */
  U32 curcount;			/* Used now in s_node[]. */
  U32 isactive;			/* Set if s_node[] is valid. */
  struct S_node s_node[MAX_SNODELIST];
};

/*
** End of structures related to "signal ports".
*/

/*
** Types used in the ready list.
**
** The ready list is an array of structures, one for each priority.
**
** See also: zzdp_look_aside, zzuu.ready_list[].
*/

struct ready_list_entry
{
  struct _pcb *hd;		/* Head pointer. */
  struct _pcb *tl;		/* Tail pointer. */
  U32 pri_mask;			/* Look aside mask. */
  /*
  ** This padding was introduced and removed again before release of
  ** the 1.19.3 kernel. It was discovered that the change would affect
  ** post mortem tools as well as the SingleStep MTD-extensions, which
  ** was unacceptable at the time.
  ** However, as the padding improves the dispatcher, it is now 
  ** inserted again in 2.20.
  */

  U32 pad;			/* 16 byte alignment to avoid multiplication in go_search. */
};

/*
** The list of background processes.
** This is a circular, double linked list of all background processes 
** in the system. This means "ready" processes as well as "not ready".
**
** The list is "rotated" by the system tick whenever the timeslice for the 
** first process in the list has expired, by moving the head pointer.
**
** Member hd points out the owner of the current time slice.
** Member time_slice holds remaining number of ticks.
** Member search_next, when set, points out where (in the list of
** background processes) to start searching for a ready process that
** will get an opportunity to run even though the current timeslice
** belongs to the first process in the background list. The reason
** why the owner of the timeslice can't utilize it is that it's not
** ready.
*/
struct background_list
{
  struct _pcb *hd;		/* Head pointer. */
  struct _pcb *search_next;	/* Pointer for fair scheduling. */
  OSTIME time_slice;		/* Remaining time slice. */
};

struct ready_entry_in_pcb	/* Goes into field ready_entry in pcb. */
{
  struct _pcb *nxt;		/* Forward pointer. */
  struct _pcb *prev;		/* Backward pointer. */
  struct ready_list_entry *rlist; /* pointer to ready or background list. */
  OSBOOLEAN is_ready;		/* Set if ready. */
  U8 pad1, pad2, pad3;		/* 32 bit alignment. */
};

/*
** Types used in the wakeup list.
**
** The wakeup list is an array of structures, one for each interrupt
** priority.
**
** See also: zzuu.wk_look_aside, zzuu.wakeup_list[], struct ready_entry_in_pcb.
*/

struct wakeup_list_entry
{
  struct _pcb *hd;		/* Head pointer. */
  U32 pri_mask;			/* Look aside mask. */
};

/*
** ISR control block.
**
** Contains information about a single ISR entry.
*/
struct isrcb
{
  void *isr;			/* ISR function entrypoint. */
  uint32_t arg;			/* Argument to pass when calling function. */
  uint8_t type; 		/* 0 == hook, 1 = isr. */
  uint8_t slot;			/* Slot index in parent vectorcb. */
  uint16_t generation; 		/* Used when generating ISR handles. */
  PROCESS owner;		/* Owning pid, or 0 for no owner. */
  struct isrcb *next;		/* Used when linked into an attach list. */
  struct vectorcb *vcb;		/* Pointer back to owning vector. */
};

/*
** Vector control block.
**
** Contains information about an interrupt vector.
*/
struct vectorcb
{  
  OSVECTOR vecno; 		/* Vector number. */
  uint8_t pad11, pad12;
  uint32_t cpus;		/* CPUs the vector is bound to. (bitfield). */   
  uint8_t isrcount;		/* Number of ISRs and hooks installed. */
  uint8_t pcbcount; 		/* Can only be 0 or 1. */
  uint16_t disabled;		/* Counter for disabling the vector. */
  OSPRIORITY priority;		/* Hardware interrupt priority. */
  uint8_t pad23, pad24, pad25;
  struct isrcb isr[7];		/* ISR slots. */
  struct vectorcb *next;	/* Next entry in the vectors array. */
};

/*
** Segment control block.
**
** There is a segment control block for each memory segment in the system.
** A segment is created by the users memory management code, and made
** known to the kernel through the attach_segment system call.
**
** Segment control blocks are stored in the segment list.
**
** The system uses the same set of buffers for block descriptors,
** segment control blocks and process control blocks.
** Free and allocated buffers are kept in the zzuu.pix_list[] array.
** Free buffers are also kept in the zzuu.pcb_freelist linked list.
** The zzuu.pix_list[] array also keeps the pool control blocks, but because
** of their larger size, these block have their own free list.
**
** It has a number of fields:
**	Segment number assigned by the MMU software
**	Number of blocks in the segment
**	Number of pools in the segment
**	Number of high priority processes in the segment
**	List of semaphores in the segment
**	Block id of the last killed block in the segment.
**
** See also: segment_list.
*/

struct segcb
{
  U16 magic;			/* any valid segment contains MAGIC_SEG. */
  PCB_INDEX_T pindex;		/* Index into pcb list where this struct is located. */
  PCB_INSTANCE_T instance;	/* 1 first time struct area is used. */
  OSPRIORITY priority;		/* Not used. Here only for alignment. */
  OSBOOLEAN killed;		/* Not used. Here only for alignment. */
  struct segcb *next;		/* List element in list of segments. */
  struct env_buf *env_list;	/* Environment list. Not used. */
  OSERRH * error_handler;	/* Pointer to error handler. Not used. */
  struct _attach_entry *attached; /* List of signals attached to the segment. */
  /* End of part shared between pool, segment, process and block descriptors. */

  PROCESS last_bid;		/* id of the last block killed in the segment. */
  OSSEGMENT segid;		/* Segment number assigned by the MMU software. */
  struct semcb *semlist;	/* List of semaphores. */
  struct mutcb *mutlist;	/* List of mutexes. */
  U16 block_count;		/* Number of blocks located in the segment. */
  U16 pool_count;		/* Number of pools located in the segment. */
  U16 highpri_count;		/* Number of high pri processes in the segment. Not used yet??*/
  U16 visitors;			/* Number of processes visiting by set_segment(). */
  OSADDRESS migration_inhibited; /* If non-zero, the segment and all its
				 * processes are locked to the current cpu.
				 * The value is a bit mask. Mask 1 is owned
				 * by the Program Manager, mask 2 by Bios.
				 */
  OSBOOLEAN mas_mapped;		/* The segment has any MAS mapped regions. */
  U8 pad1, pad2, pad3;		/* 32 bit alignment. */
  struct isrcb *isr_list;	/* Linked list of ISRs that has attached to this segment. */
  PROCESS main_block;		/* The first block in the segment. */
};

/*
** Stack control block.
**
** There is a stack control block for each stack in the system. Stack control
** blocks are allocated from the system pool, even if the stack itself is in
** some other pool.
**
** A stack control block can be in several lists:
**	Main free list of stack control blocks when not allocated to a pool.
**	Free list in pool control block
** It has a number of fields:
**	Stack top
**	Real stack limit
**	Requested stack limit
**	Memory pool id
*/

struct stkcb
{
  struct stkcb *next;	/* link in free stack list and in pool free list. */
  void *top;		/* Pointer to stack top (one byte beyond stack). */
  void *limit;	/* Pointer to limit as requested by user (lowest addr on stack). */
  struct pool_list_entry *buffer_head;	/* Free list within pool control block. */
};

/*
** Signal administration block.
**
** Signals are organized in two ways depending on the MEMORY_MODEL.
** In memory model 0, there is one signal administration block for each
** signal in the system. It is located right before its associated data buffer
** in the user pool space. The udata field is not used and contain random data.
**
** In memory model 1 and above, there are two signal administration blocks for each
** signal. The main administration block is located in the system pool, and contains
** all system information. The udata field points to the associated data buffer
** in user pool space. The secondary administration block is located right before
** the data buffer in the user pool space. Only the magic_signo and udata fields are
** used. The udata field in this case points to the base of the main administration block.
**
** Note that it is important that the size of the adm block is a multiple
** of 32 bits. Otherwise the signal number would not be properly aligned.
** It is also important that the size is exactly 32 bytes for DMA alignment.
** This works for cache line sizes up to 32 bytes, look out for CPU's with larger lines.
** 
** A signal control block can be in several lists:
**	Free list in pool control block
**	The signal queue of a process
**	List of all signals in the system
** It has a number of fields:
**	Owner
**	Sender
**	Addressee
**	Requested size
**	Pool id and free list id used to free the buffer.
**
** See also: struct short_sigadm.
*/

struct _signal
{
  struct _signal *next;		/* Mailbox pointer. */
  struct _signal *siglist;	/* List of all signals for pool analysis etc. */
  U16 magic_signo;		/* Contains the MAGIC_SIGNO number. This is for free since we need */
				/* this space for alignment anyway. */
  PCB_INDEX_T owner;		/* Process index of owner or 0 if kernel owns it. */
  SIGSELECT *udata;		/* Only used if MEMORY_MODEL > 0. */
  struct pool_list_entry *buffer_head; /* Free list within pool control block. */
  OSBUFSIZE size;		/* Size requested by allocating process. */
  PROCESS dest_id;		/* 0 if not redirected, otherwise addressee. */
  PROCESS sender_id;		/* Sending process or 0 if not sent. */
};

/*
** Semaphore control block.
**
** There is a semaphore control block in supervisor space for each semaphore
** in the system. This control block is allocated from the system pool.
** There is also a related semaphore structure in user space. (struct semaphore
** defined in ose.h)
**
** Semaphore control blocks are stored in the semaphore list for each memory
** segment.
**
** It has a number of fields:
**	A next and prev pointer in the semaphore list per segment.
**	A segcb pointer.
**	Magic number used for error checking.
**	A flag set to indicate a static semaphore that can not be killed.
**	The current semaphore value.
**	Head and tail pointers in the semaphore queue.
**	A pointer to the related user mode semaphore structure.
**
** See also: struct semaphore, MAGIC_SEMAPHORE.
*/

struct semcb
{
  struct semcb *next;		/* List element in list of semaphores. */
  OSSEMVAL value;		/* Current semaphore value. Negative if queue is nonempty. */
  struct _pcb *fq;		/* Semaphore queue forward pointer. NULL at end of list. */
  struct _pcb *bq;		/* Semaphore queue backward pointer. NULL at head of list. */
  struct semaphore *usersem;	/* User mode semaphore structure. */
  struct semcb *prev;		/* List element in list of semaphores. */
  struct segcb *segcb;		/* Belongs to this segment. */
  U16 magic;			/* Any valid semaphore contains MAGIC_SEMAPHORE. */
  OSBOOLEAN static_sm;		/* Nonzero if static, zero if dynamic. */
  U8 end_mark;			/* Must be the last member. Must not be set, 
				   just used in offsetof() to avoid unneeded 
				   cacheline alignment. */
};

/*
** Annotated dynamic user space SEMAPHORE structure.
** See also zzkrn_set_semaphore_annotation in pose.h
*/

struct annotated_semaphore
{
  struct semaphore sem;		/* The usual user mode semaphore. */
  OSADDRESS return_address;	/* The return address of the create call. */
  PROCESS creator;		/* The process that created it. */
};

/*
** Mutex control block.
**
** There is a mutex control block in supervisor space for each mutex
** in the system. This control block is allocated from the system pool.
** There is also a related mutex structure in user space. (MUTEX
** defined in ose.h)
**
** Mutex control blocks are stored in the mutex list for each memory
** segment.
*/
struct mutcb
{
  U16		magic;		/* MAGIC_MUTEX or MAGIC_MUTEX_DESTROYED */
  U16		dummy;		/* Space filler. */
  U16		instance;	/* Increment at mutex_destroy. NEVER READ?? */
  U16		index;		/* Index in mutex pool. Index zero unused. */
  struct segcb	*segcb;		/* needed? Can be reached by sem->f?? */
  SEMAPHORE	*sem;		/* The wait queue. */
  struct _mutex *umutex;	/* NULL for a free mutcb. */
  struct mutcb	*next;		/* Per segment list, or free list link. */
  struct mutcb	*prev;		/* Per segment list link when in use. */
};

/* State hook control block. */
#define STATEHOOK_MAGIC	0xb57a7ecb
struct statehookcb
{
  SIGSELECT	magic;
  struct statehookcb *next;
  OSSTATEHOOK	*func;
  PROCESS	owner;
  PROCESS	victim;
  void		*handle;
};

/*
** Breakpoint control block.
**
** There is a breakpoint control block in supervisor space for each breakpoint set
** in the system. This control block is allocated from the system pool.
**
** Breakpoint control blocks are stored in the global breakpoint list.
**
** It has a number of fields:
**	A next pointer in the breakpoint list.
**	A flag set when the breakpoint is tripped.
**	A flag set if the breakpoint is a software watchpoint (flags = 0).
**	The id of the process that tripped the breakpoint.
**	The id of the process that set the breakpoint (owner).
**	The pcb or bcb of the block or process in which the breakpoint was set (target).
**	The address at which the breakpoint was set.
**	A pointer to the associated trap signal.
**
** See also: break_list.
*/

struct breakcb
{
  struct breakcb *next;		/* List element in list of breakpoints. */
  PROCESS owner;		/* Process that set the breakpoint. */
  union SIGNAL *trapsig;	/* The trap signal. System is the owner. */
  struct _pcb *target;		/* Pcb or bcb of target process/block. */
  OSBOOLEAN softpoint;		/* Set if breakpoint is a watchpoint. */
  OSADDRESS address;		/* Address where the breakpoint was set. */
  OSBOOLEAN tripped;		/* Set when breakpoint is reached. */
  PROCESS caught_pid;		/* Process that reached the soft breakpoint. */
  OSSEGMENT segid;		/* Segment number in which the breakpoint is set. */
  OSADDRESS attribute;		/* The attributes from the set_bp() call. */
  void *uarea;			/* pointer to uarea managed by target interface. NULL = empty. */
  /*
  ** The function below filters breakpoints for attribute bit 2 = 1.
  */
  OSBOOLEAN (*bp_trig_hdlr)(U32 address, PROCESS pid, PROCESS bid, PROCESS sid);
};

/*
** The list of environment variables in a block descriptor or a
** process control block consists of a linked list of environment
** control structures. These are allocated on demand from
** the system pool and the system is the owner of these buffers.
** New entries are always added at the tail of the list.
**
** It has a number of fields:
**	A next pointer in the environment list.
**	A pointer to the current value.
**	The current size of the value buffer.
**	The length of the current value.
**	A hash value representing the name of the variable.
**	The name of the variable.
**
** See also: struct _pcb, struct _block, set_env(), get_env(), get_env_list().
*/

struct env_buf
{
  struct env_buf *next; /* Next buffer in list. */
  char *value;		/* Buffer for value. */
  OSBUFSIZE max_value;	/* Size of current value buffer. */
  U32 cur_value;	/* Size of current value string. */
  char name_hash;	/* Compressed env variable name. */
  char name[1];		/* Name of the env variable. */
};

/*
** The format of each record in the attach pool array.
*/

struct _attach_entry
{
  struct _attach_entry *own_fq;	/* Forward link in owners pcb.
				 * Also free list ptr.
				 */
  struct _attach_entry *own_bq;	/* Backward link in owners pcb.
				 * Also set to MAGIC_EXPIRED_UNSENT_ATTREF
				 * and then MAGIC_EXPIRED_SENT_ATTREF for 
				 * expired attaches, resp.
				 * MAGIC_DETACHED_ATTREF for detached ones.
				 */
  struct _attach_entry *vic_fq;	/* Forward link in victims pcb. */
  struct _attach_entry *vic_bq;	/* Backward link in victims pcb. */
  struct _signal *sig;		/* Sigadm pointer to the attached signal. */
  PROCESS owner;		/* The caller of attach(). */
  PROCESS victim;		/* The pid parameter in attach(). */
  U16 attrid;			/* This entries index into the attach pool. */
  U16 instance;			/* Instance no of reusage cycle. */
};

/*
** Pool control block.
**
** There is a pool control block for each pool in the system.
**
** The system uses the same set of buffers for pool control blocks, block descriptors,
** segment control blocks and process control blocks.
** Free and allocated buffers are kept in the zzuu.pix_list[] array.
** Free buffers are also kept in the zzuu.poolcb_freelist linked list.
**
** It has a number of fields:
**	A magic number used to check if we have a pool, process or a block id
**	An associated entry in the process index translation table
**	An instance number
**	List of free pcb's (used when allocating a new pcb, bcb or pool)
**	Memory segment where pool is located
**	Number of buffers currently allocated
**	Free low pool pointer
**	Free high pool pointer
**	Number of blocks served with stacks/sigs
**	Number of pool extensions performed
**	Sizes of buffers of all types
**	Lists of free buffers of all types
**	Sizes of stacks of all types
**	Lists of free stacks of all types
**	List of all buffers in pool regardless if allocated 
*/

/*
** The short signal adm block is used as the first signal in the pool siglist.
** It is present only to avoid an empty siglist, which would cause an extra
** test in alloc().
*/
struct short_sigadm
{
  struct _signal *next;		/* Mailbox pointer. */
  struct _signal *siglist;	/* List of all signals for pool analysis etc. */
};

struct pool_list_entry
{
  OSADDRESS size;		/* Size of this signal or stack type. */
  void *hd;			/* Head of free list for this type. */
  U32 alloc_count;		/* Total no of signals allocated. */
  struct poolcb *poolcbp;	/* Pointer to the base of the pool control block. */
  OSSEGMENT segid;		/* Segment number where pool is located. */
  spinlock_t smp_spin;		/* Free list spin lock for SMP. */
  void *ext_handle;		/* Handle used for alloc/free_buf hooks. */
};

struct pool_extension
{
  struct pool_extension *frag_list;	/* Forward link in the fragment list. */
  OSADDRESS base;			/* Lowest address in pool. */
  OSADDRESS limit;			/* One byte beyond the pool. */
  U32 magic_frag;			/* Contains MAGIC_POOL. */
  OSADDRESS pool_low_p;			/* Final low brk pointer. */
  OSADDRESS pool_hi_p;			/* Final high brk pointer. */
  U32 pad1, pad2;
};

struct poolcb
{
  U16 magic;			/* any valid pool contains MAGIC_POOL. */
  PCB_INDEX_T pindex;		/* Index into pcb list where this struct is located. */
  PCB_INSTANCE_T instance;	/* 1 first time struct area is used. */
  OSPRIORITY priority;		/* Not used. Here only for alignment. */
  OSBOOLEAN killed;		/* Not used. Here only for alignment. */
  struct poolcb *next;		/* Next pcb in pool_list. */
  spinlock_t smp_spin;		/* Pool spin lock for SMP. */
  struct pool_extension *frag_list; /* List of all pool fragments. For debugging etc. */
  struct _attach_entry *attached; /* List of signals attached to this pool. */
  /*
  ** End of part shared between pool, segment, process and block descriptors.
  ** The attach list must have the same offset for all pixlist objects.
  */

  struct _signal *siglist;	/* List of all signals for pool analysis etc. */
  struct _signal *sigtail;	/* Last buffer in the list of all signals. */
  U16 no_of_stack_users;	/* No of blocks served with stacks. */
  U16 no_of_sig_users;		/* No of blocks served with signals. */
  OSSEGMENT segid;		/* Segment number where pool is located. */
  PROCESS segpid;		/* Segment id where pool is located. */
  OSADDRESS pool_low_p;		/* Current low brk pointer. */
  OSADDRESS pool_hi_p;		/* Current high brk pointer. One byte beyond free space. */
  struct pool_list_entry	/* Free signal lists. */
  signal_list[8];
  struct pool_list_entry	/* Free stack lists. */
  stack_list[8];
  struct short_sigadm
  firstsig;
  U32 free_attempts;		/* Number of attempts to free the pool. */
  OSADDRESS frag_size;          /* Size of each new extend fragment. */
  OSADDRESS max_size;           /* Maximum allowed size of the pool */
  OSADDRESS tot_size;		/* Current pool size. */
  OSPOOLEXTENDH *ext_func;	/* or OSPOOLEXTENDER* for PM user mode extender  */
  void *prog_type_p;            /* Extra parameter registered from and sent back to PM */
  OSBOOLEAN fill;               /* Fill with initial pattern or not. */
  OSPRIORITY ext_frag_prio;     /* The priority of the pool extender. */
  OSPRIORITY no_copy;           /* No copying of signals from this pool. */
  OSPRIORITY is_static;         /* is !=0 when the pool is never killed */
  U8 realtime_extendable;	/* May use a realtime extender. */
  U8 pad1, pad2, pad3;
  void * volatile * volatile ext_frag_pp;  /* Handle to new pool fragments. */
  U32 extend_count;		/* Number of times the pool has been extended. */
  void *(*ext_alloc)(void *ext_handle);          /* External alloc hook. */
  void (*ext_free)(void *ext_handle, void *buf); /* External free hook. */
};

/*
** Process control block.
**
** The system uses the same set of buffers for block descriptors,
** segment control blocks and process control blocks.
** Free and allocated buffers are kept in the zzuu.pix_list[] array.
** Free buffers are also kept in the zzuu.pcb_freelist linked list.
** The zzuu.pix_list[] array also keeps the pool control blocks, but because
** of their larger size, these block have their own free list.
**
** It has a number of fields:
**	A magic number used to check if we have a pool, process or a block id
**	An associated entry in the process index translation table
**	An instance number
**	Pointer in list of free pcb's (used when allocating a new pcb, bcb or pool)
**	process list pointers in the list of all processes (a splay tree).
**	List of environment variables.
**	An error handler entrypoint
**	A user number.
**	Pointer in block specific process list (for hunt, get_pid_list etc).
**	Process name in a list of name buffer[s]
**	A status word
**	A process priority and a process type.
**	A timeslice for bgd, pri and timer processes
**	A current timeslice for timer processes.
**	An entrypoint for interrupt and timer processes.
**	A vector number for interrupt processes.
**	Pointers in timeout list or semaphore queue.
**	Mailbox pointers.
**	A fast semaphore value.
**	A stop counter.
**	Flush restart flag used by receive and flush.
**	List of signals wanted while in the receive call.
**	Pointer to semaphore during the wait_sem call.
**	An associated block id.
**	A redirection table.
**	Flag set if it is a block process.
**	Counter of pending wakeup calls due to signals received.
**	Counter of pending wakeup calls due to semaphore signals.
**	A CPU dependent uarea where cpu regs etc are stored.
**	A process stack and a stack size.
**	Counter of owned signal buffers.
**	Counter of signals in the signal queue.
**	A flag set while killing the process. (used for special cases only.)
**	Ready list pointers (used by dispatcher).
**??	An error handler stack and size (used also for block level error handlers).
**	Flag set if there is a debugger action for this process
**	Name pointer in the debugger trace
**	
** The killed field in the pcb and block structs are required
** to avoid race conditions towards get_bid_list and get_pid_list.
** The flag is used if get_xxx_list is called after a process was killed
** by the kill_proc call, but before the process has been removed from the
** lists by the system daemon.
** There may be a better way to handle this race condition.
**
** The killed field is also used by post mortem debuggers, it protects
** the system against "designed pathological pids" and it is checked in the
** hunt daemon. See also the MAGIC_KILLED constant.
*/

typedef U16 pr_state_t; /* Variables holding an enum pr_state. */

/*
** When clearing the ST_RECEIVE or the ST_FSEMAPHORE status bits,
** always clear both. This is required for the correct operation of
** the receive_fsem system call.
*/
enum pr_state	/* Values used in the status field of the pcb. */
  {
    ST_INIT = 0x00,
    ST_TIMEOUT = 0x01,
    ST_RECEIVE = 0x04,
    ST_READY = 0x04,		/* Process is ready if status < ST_READY */
    ST_DELAY = 0x08,
    ST_SEMAPHORE = 0x10,
    ST_FSEMAPHORE = 0x20,
    ST_RECEIVE_FSEM = 0x24,     /* RECEIVE | FSEMAPHORE */
    ST_REMOTE = 0x40,
    ST_STOPPED = 0x80,
    ST_INTERCEPTED = 0x100
  };

#ifdef OSARM
/* === ARM ARCHITECTURE ONLY PCB === */
/* NOTE! Check pose.h struct debug_pcb if moving members around! */
struct _pcb
{
  U16 magic;			/* any valid pcb contains MAGIC_PCB. */
  PCB_INDEX_T pindex;		/* Index into pcb list where this struct is located. */
  PCB_INSTANCE_T instance;	/* 1 first time struct area is used. */
  OSPRIORITY priority;		/* 0-31=PRI,32=BG,64-95=INT,96-127=TI,128=PHANT */
				/* priority also implies process type. */
  /* Warning! can not move "priority" relative to the debug_pcb. */
  OSBOOLEAN killed;		/* Set while the process is being killed. */
  struct _pcb *next;		/* Next pcb in zzuu.pcb_freelist. */
  struct env_buf *env_list;	/* Environment list. */
  PROCESS pid;			/* [ARM]: Process id for this process (a speed up hack). */
  struct _attach_entry *attached; /* [ARM]: List of signals attached to and by this process. */
  /* End of part shared between pool, segment, process and block descriptors. */
  OSUSER user;			/* [ARM]: The user of this process. */
  /* End of part shared between process and block descriptors. */

  struct _pcb *proclist;	/* [ARM]: List of processes in block. */
  treenode T_procname;		/* [ARM]: Search tree node used in the process list. */
  U32 debug_flags;		/* [ARM]: Debugger status for this process. */
  struct stkcb *stack;		/* [ARM]: Pointer to stack control block. Invalid for int, tim, phant. */
  char *tracename;		/* [ARM]: Process name as stored in the trace. */
  const char *name;		/* [ARM]: Ascii name of the process. Must be placed after tracename*/
  const char *blockname;	/* [ARM]: Ascii name of the parent block. Must be placed here. */
  struct _block *block;		/* [ARM]: Id of parent block. */
  OSFSEMVAL fsemval;		/* [ARM]: Value of fast semaphore. Must be signed. */
  /* End of part needed by phantoms. (Not quite right these days...) */
  void (*entrypoint)(void);	/* [ARM]: Process entrypoint. */
  /* End of debug_pcb */

  OSUSER real_user;		/* [ARM]: The real user of this process while using the kernel pool. */
  PROCESS parentpid;		/* [ARM]: The creator of the process. For the system debugger. */
  OSPRIORITY org_prio;		/* [ARM]: Original prio. (Used in prio inherit mutex.) */
  U8 debug_info;		/* [ARM]: Info saved for debuggers. Zero for new processes. */
  U8 out_of_pool;		/* [ARM]: 0-3: out of space in system(1) or local(2) pool. */
  OSBOOLEAN locked_kill;
  U32 proc_attrs;		/* [ARM]: Process attributes for send hook flavour etc. */
  struct isrcb *isr_list;       /* [ARM]: Linked list of ISRs that has attached to this process. */
  OSADDRESS ppd_addr;		/* [ARM]: Per process data pointer. */
  cpuid_t cpu_id;		/* [ARM]: CPU nr. that executes this process. */
  OSTIME ts_time;		/* [ARM]: Time left in the timeslice for prioritized process. */
  OSTIME tmo;			/* [ARM]: Timeslice (in ticks) for BG and TI processes. */
  struct _pcb *fq;		/* [ARM]: Used for zzuu.tmolist, zzuu.tmslist and (BKL:) semaphore queue. */
  struct _pcb *bq;		/* [ARM]: This one too, but backward list is not used in zzuu.tmslist. */
  struct _signal *mailbox;	/* [ARM]: NULL if empty. */
  struct _signal *mailtail;	/* [ARM]: Undef if empty, otherwise last struct. */
  struct OS_redir_entry *redir;	/* [ARM]: Redirection table. */
  U32 sig_cnt_queued;		/* [ARM]: Number of queued signals. */
  const SIGSELECT *wanted;	/* [ARM]: Valid to debugger only during receive(). Used also during wait_sem(). */
  OSSENDHOOK *snd_hook;		/* [ARM]: Send hook function. (Phantom only.) */
  OSADDRESS sig_cnt_owned;	/* [ARM]: Number of signals owned by the process. */
  struct poolcb *cursigpool;	/* [ARM]: The currently active pool. Used by alloc, set_suid() et al. */
  OSBOOLEAN in_rem_call;	/* [ARM]: Non zero if the process is currently executing a remote system call. */
  OSBOOLEAN flushed;		/* [ARM]: Bool set if queue flushed. Receive looks at it. */
  OSBOOLEAN is_an_sport;	/* [ARM]: This phantom process is a signal port. */
  OSBOOLEAN is_deaf;		/* [ARM]: This int/tim proc won't wake up on signals. */
  void *uarea;			/* [ARM]: pointer to uarea managed by target interface. NULL = empty. */
  OSSEGMENT segid;		/* [ARM]: Segment number where the block is located. */
  struct ready_entry_in_pcb ready_entry; /* [ARM]: Pointers and stuff for the dispatcher. */
  /* Points into zzuu.ready_list for pri, zzuu.wakeup_list for int, invalid otherw. */
  pr_state_t status;		/* [ARM]: Current state of process. enum pr_state. */
  OSVECTOR vector;		/* [ARM]: Vector no associated with int proc. */
  U16 wake_sig_cnt;		/* [ARM]: No of pending send() wakeup calls. */
  U16 wake_sem_cnt;		/* [ARM]: No of pending signal_fsem() wakeup calls. */
  U16 wake_tick_cnt;		/* [ARM]: No of pending tick wakeup calls. (SMP) */
				/* [ARM]: Also used to store zzuu.tmolist index for delay() et.al. */
  OSBOOLEAN tmo_active;		/* [ARM]: Set if a receive_w_tmo or delay is active. Used by kill_proc. */
  OSBOOLEAN err_handl_level;	/* [ARM]: No of recursive calls to an error handler. Detects iterative errors. */
  struct S_pcb s_pcb;		/* [ARM]: Data related to "signal ports". */
  struct tos_head pending_tos;	/* [ARM]: Running time out signals  */
  struct tos_head expired_tos;	/* [ARM]: Expired but still allocated time out signals */
  OSERRH * error_handler;	/* [ARM]: Pointer to error handler. */
  void *lnh_sender_hd;		/* [ARM]: PPD for LNHs, used by snd_hook as sender per process data. */
  void *lnh_dest_hd;		/* [ARM]: PPD for LNHs, used by snd_hook as destination per process data. */
  U32 lock_cnt;			/* [ARM]: Nr. of clients who have locked this PCB. */
  volatile cpuid_t alloced_cpu;	/* [ARM]: Set to cpu_id+1 if proc owns the cpu. */
  U16 cur_tmo;			/* [ARM]: Current index into zzuu.tmslist. Used if timer process is killed. */
  OSBOOLEAN use_kernel_pool;	/* [ARM]: Use the kernel pool for signals if set. Else use default. */
  OSBOOLEAN nil_alloc;		/* [ARM]: Set while the proc is inside alloc_nil(). */
  U32 st_cnt;			/* [ARM]: Stop counter. */
  int (* volatile rec_poller)(void); /* [ARM]: Pointer to a receive poller for ose_fsend. */
  OSADDRESS error_ecode;	/* Latest error code. */
  OSADDRESS error_extra;	/* Latest error extra. */
  OSBOOLEAN error_origin;	/* Latest error type, 0: kernel 1: error() 2: No error */
  U8 pad1[3];
  OSTICK cur_tmo_expires;	/* This is the tick when a delay etc. expires. */
};
/* === ARM ARCHITECTURE ONLY PCB END === */
#else
/* === ALL ARCHITECTURES EXCEPT ARM PCB === */
struct _pcb
{
  U16 magic;			/* any valid pcb contains MAGIC_PCB. */
  PCB_INDEX_T pindex;		/* Index into pcb list where this struct is located. */
  PCB_INSTANCE_T instance;	/* 1 first time struct area is used. */
  OSPRIORITY priority;		/* 0-31=PRI,32=BG,64-95=INT,96-127=TI,128=PHANT */
				/* priority also implies process type. */
  /* Warning! can not move this relative to the debug_pcb. */
  OSBOOLEAN killed;		/* Set while the process is being killed. */
  struct _pcb *next;		/* Next pcb in zzuu.pcb_freelist. */
  struct env_buf *env_list;	/* Environment list. */
  OSERRH * error_handler;	/* Pointer to error handler. */
  struct _attach_entry *attached; /* List of signals attached to and by this process. */
  /* End of part shared between pool, segment, process and block descriptors. */
  OSUSER user;			/* The user of this process. */
  /* End of part shared between process and block descriptors. */

  struct _pcb *proclist;	/* List of processes in block. */
  treenode T_procname;		/* Search tree node used in the process list. */
  U32 debug_flags;		/* Debugger status for this process. */
  PROCESS pid;			/* Process id for this process (a speed up hack). */
  char *tracename;		/* Process name as stored in the trace. */
  const char *name;		/* Ascii name of the process. Must be placed after tracename*/
  const char *blockname;	/* Ascii name of the parent block. Must be placed here. */
  struct _block *block;		/* Id of parent block. */
  struct OS_redir_entry *redir; /* Redirection table. */
  /* End of part needed by phantoms. (Not quite right these days...) */
  void *uarea;			/* pointer to uarea managed by target interface. NULL = empty. */
  OSTIME tmo;			/* Timeslice (in ticks) for BG and TI processes. */
  void (*entrypoint)(void);	/* Process entrypoint. */
  struct _pcb *fq;		/* Used for zzuu.tmolist, zzuu.tmslist and (BKL:) semaphore queue. */
  struct _pcb *bq;		/* This one too, but backward list is not used in zzuu.tmslist. */
  struct _signal *mailbox;	/* NULL if empty. */
  struct _signal *mailtail;	/* Undef if empty, otherwise last struct. */
  OSFSEMVAL fsemval;		/* Value of fast semaphore. Must be signed. */
  U32 sig_cnt_queued;		/* Number of queued signals. */
  const SIGSELECT *wanted;	/* Valid to debugger only during receive(). Used also during wait_sem(). */
  struct stkcb *stack;		/* Pointer to stack control block. Invalid for int, tim, phant. */
  OSADDRESS sig_cnt_owned;	/* Number of signals owned by the process. */
  struct poolcb *cursigpool;	/* The currently active pool. Used by alloc, set_suid() et al. */
  OSUSER real_user;		/* The real user of this process while using the kernel pool. */
  OSSEGMENT segid;		/* Segment number where the block is located. */
  struct ready_entry_in_pcb ready_entry; /* Pointers and stuff for the dispatcher. */
  /* Points into zzuu.ready_list for pri, zzuu.wakeup_list for int, invalid otherw. */
  pr_state_t status;		/* Current state of process. enum pr_state. */
  OSVECTOR vector;		/* Vector no associated with int proc. */
  U16 wake_sig_cnt;		/* No of pending send() wakeup calls. */
  U16 wake_sem_cnt;		/* No of pending signal_fsem() wakeup calls. */
  U16 wake_tick_cnt;		/* No of pending tick wakeup calls. (SMP) */
				/* Also used to store zzuu.tmolist index for delay() et.al. */
  OSBOOLEAN tmo_active;		/* Set if a receive_w_tmo or delay is active. Used by kill_proc. */
  OSBOOLEAN err_handl_level;	/* No of recursive calls to an error handler. Detects iterative errors. */
  OSBOOLEAN in_rem_call;	/* Non zero if the process is currently executing a remote system call. */
  OSBOOLEAN flushed;		/* Bool set if queue flushed. Receive looks at it. */
  OSBOOLEAN is_an_sport;	/* This phantom process is a signal port. */
  OSBOOLEAN is_deaf;		/* This int/tim proc won't wake up on signals. */
  PROCESS parentpid;		/* The creator of the process. For the system debugger. */
  struct S_pcb s_pcb;		/* Data related to "signal ports". */
  struct tos_head pending_tos;	/* Running time out signals  */
  struct tos_head expired_tos;	/* Expired but still allocated time out signals */
  OSADDRESS ppd_addr;		/* Per process data pointer. */
  OSPRIORITY org_prio;		/* Original prio. (Used in prio inherit mutex.) */
  U8 debug_info;		/* Info saved for debuggers. Zero for new processes. */
  U8 out_of_pool;		/* 0-3: out of space in system(1) or local(2) pool. */
  OSBOOLEAN locked_kill;
  OSTIME ts_time;		/* Time left in the timeslice. */
  OSSENDHOOK *snd_hook;		/* Send hook function. (Phantom only.) */
  void *lnh_sender_hd;		/* PPD for LNHs, used by snd_hook as sender per process data. */
  void *lnh_dest_hd;		/* PPD for LNHs, used by snd_hook as destination per process data. */
  U32 lock_cnt;			/* Nr. of clients who have locked this PCB. */
  cpuid_t cpu_id;		/* CPU nr. that executes this process. */
  volatile cpuid_t alloced_cpu;	/* Set to cpu_id+1 if proc owns the cpu. */
  U16 cur_tmo;			/* Current index into zzuu.tmslist. Used if timer process is killed. */
  OSBOOLEAN use_kernel_pool;	/* Use the kernel pool for signals if set. Else use default. */
  OSBOOLEAN nil_alloc;		/* Set while the proc is inside alloc_nil(). */
  U32 st_cnt;			/* Stop counter. */
  struct isrcb *isr_list;       /* Linked list of ISRs that has attached to this process. */
  int (* volatile rec_poller)(void); /* Pointer to a receive poller for ose_fsend. */
  U32 proc_attrs;		/* Process attributes for send hook flavour etc. */
  OSADDRESS error_ecode;	/* Latest error code. */
  OSADDRESS error_extra;	/* Latest error extra. */
  OSBOOLEAN error_origin;	/* Latest error type, 0: kernel 1: error() 2: No error */
  U8 pad1[3];
  OSTICK cur_tmo_expires;	/* This is the tick when a delay etc. expires. */
};
/* === ALL ARCHITECTURES EXCEPT ARM PCB END === */
#endif

/*
** Block descriptor.
**
** The system uses the same set of buffers for block descriptors,
** segment control blocks and process control blocks.
** Free and allocated buffers are kept in the zzuu.pix_list[] array.
** Free buffers are also kept in the zzuu.pcb_freelist linked list.
** The zzuu.pix_list[] array also keeps the pool control blocks, but because
** of their larger size, these block have their own free list.
**
** ?? We need an error handler stack. Allocate per process. Size is the worst case
** needed by either the process level or the block level error handler.
** We never scale down a stack size. The max ever size is stored in both pcb and bcb.
** Look out for error handlers allocated from other processes. We can't allocate stacks
** from another pool than our own, in case we run out of space.
**
** It has a number of fields:
**	A magic number used to check if we have a pool, process or a block id.
**	An associated entry in the process index translation table.
**	An instance number.
**	Pointer in list of free pcb's (used when allocating a new pcb, bcb or pool).
**	process list pointer in list of all blocks. (same as free pointer.)
**	List of environment variables.
**	An error handler entrypoint.
**	A user number.
**	List of all processes in the block and associated modification number.
**	Block name in a list of name buffer[s].
**	A stop counter used to detect when block creation was completed.
**	Id of the parent process during block creation. Also a flag set if no parent.
**	Id of the block process if any.
**	Id of the associated remote call server and a flag set if server present.
**	Flag set if it is a block of supervisor processes.
**	Id of pool from which stacks are allocated.
**	Id of pool from which signals are allocated.
**	A flag set while killing the block. (used for special cases only.)
**	A memory segment id.
**	Name pointer in the debugger trace.
*/

struct _block
{
  U16 magic;			/* any valid pcb contains MAGIC_BCB. */
  PCB_INDEX_T pindex;		/* Index into pcb list where this struct is located. */
  PCB_INSTANCE_T instance;	/* 1 first time struct area is used. */
  OSPRIORITY priority;		/* Not used. Here only for alignment. */
  OSBOOLEAN killed;		/* Set while the block is being killed. */
  struct _block *next;		/* Next bcb in zzuu.bcb_list. */
  struct env_buf *env_list;	/* Environment list. */
  OSERRH * error_handler;	/* Pointer to error handler. */
  struct _attach_entry *attached; /* List of signals attached to this block. */
  /* End of part shared between pool, segment, process and block descriptors. */
  OSUSER user;			/* The default user of all processes in the block. */
  /* End of part shared between process and block descriptors. */

  struct _pcb *proclist;	/* List of processes in block. */
  U32 pcb_list_modno;		/* proclist modification number. Currently not used. Do we need it ??*/
  const char *name;		/* Ascii name of the block. */
  U32 st_cnt;			/* Number of unstarted children. Parent removed when this hits zero. */
  struct _pcb *parent;		/* Father = owner until last start call. NULL after. */
  struct _pcb *block_process;	/* Points to block process or NULL if none. */
  PROCESS remcall_server;	/* Process id of remcall_server. */
  struct poolcb *stackpool;	/* Pool used for stacks. */ 
  struct poolcb *sigpool;	/* Pool used for signals. */
  void *clib_pointer;		/* Clib handle in the C lib associated with this block. NULL if unknown. */
  PROCESS segpid;		/* Segment id where block is located. */
  OSSEGMENT segid;		/* Segment number where the block is located. */
  OSBOOLEAN use_remcalls;	/* Flag set if a remcall_server is present. */
  OSBOOLEAN supervisor_mode;	/* Zero for user mode processes. */
  OSBOOLEAN is_main_block;	/* Set if first block in the segment. Used to auto kill the segment. */
  OSBOOLEAN shared_mode;        /* Set if shared mode execution. */
  cpuid_t cpu_id;		/* CPU nr. for new processes in this block. */
  OSBOOLEAN kill_in_progress;	/* Killing this block. Do not create processes. */
  U8 pad1, pad2, pad3;		/* Word align. */
  U32 prof;			/* Used by rmm for cpu profiling. */
  const char *lnh_name;         /* Linkhandler name assigned to this block, or NULL if none. */
  struct isrcb *isr_list;       /* Linked list of ISRs that has attached to this block. */
  struct statehookcb *statehook_list;  /* State change hook list */
};

/*
** Any segment control block, block control block or process control block.
*/

union large_cb
{
  struct _pcb pcb;		/* Process control block. */ 
  struct _block bcb;		/* Block control block. */
  struct segcb segcb;		/* Segment control block. */
};

/*
** The system handler array.
** this array contains pointers to user supplied handlers or callback
** functions which may be altered in runtime by power users.
*/

struct handlerp
{
  U32 count;	/* No of fields implemented. */
  void (*vector_bind_handler)(U32 vector, U32 priority);
  void (*vector_release_handler)(U32 vector);
  void (*zzcall_attach_block)(PROCESS pid, OSSEGMENT segid);
  OSBOOLEAN (*zzcall_mem_move)(OSSEGMENT src_segment,
			       OSADDRESS src_begin,
			       OSADDRESS size,
			       OSSEGMENT dest_segment,
			       OSADDRESS dest_begin);
  void (*zzcall_select_segment)(OSSEGMENT segnum);
  OSBOOLEAN (*zzcall_set_segment_mode)(OSSEGMENT segid, OSBOOLEAN contains_int);
  OSADDRESS (*zzcall_system_errh)(OSBOOLEAN user_called,
				  OSERRCODE ecode, OSERRCODE extra);
  void (*zzkrn_halted_hook)(void);
  void (*zzget_filename)(void *uarea, OSADDRESS *lineno, char *filename);
  void (*zzcapture_regs)(void *uarea, char *cpuregs);
  void (*zzget_regs)(void *uarea, char *cpuregs);
  OSTICK (*zzcall_power_save)(OSTICK max_time);
  void (*zzprof_alloc)(OSBUFSIZE sig_size, void *sig, struct debug_pcb *dpcb,
		       PROCESS pool);
  void (*zzprof_free_buf)(OSBUFSIZE sig_size, void *sig, struct debug_pcb *dpcb,
		       PROCESS pool);
  int (*zzmsg_enqueue)(cpuid_t dst_cpu,
		       const struct ose_krn_msg *krn_evt_buf, uint32_t flags);

  uint32_t (*zzmsg_dequeue)(struct ose_krn_msg *krn_evt_buf, uint32_t elems);
  void (*evt_interrupt_generate)(cpuid_t cpu);
};

/*
**===========================================================================
** 4.2.2  The kernel world structure
**===========================================================================
*/

/*
** The world structure contains all static variables in the kernel, except
** large local structures.
**
** The world structure is maintained to enable kernel data to be easily
** accessible from the target interface and from post mortem analyzer tools.
*/

struct world
{

  /*
  ** Kernel stable flag.
  ** 
  ** This variable tells post mortem analyzers that the kernel is present
  ** and properly initialzied. The flag is cleared upon each system start,
  ** and set to MAGIC_KERNEL_TYPE when initialization is complete, i.e
  ** just before zzmain is started.
  */

  U32 kernel_initialized;

  /*
  ** Target interface world structure pointer.
  ** This field is redundant and contains NULL.
  ** Use the zztivarp pointer instead.
  */

  void *targint_world; 

  /*
  ** Current time.
  **
  ** This variable contains the number of ticks since system start.
  ** It is examined by get_ticks() and get_systime(). Also
  ** get_sysconf(93, ...) returns the address to this 64-bits variable.
  */

  uint64_t ticks;

  /*
  ** System control block.
  **
  ** The target interface provides a pointer to a structure containing various
  ** system parameters. A pointer to this control block is fetched during
  ** system start and the address of a local copy is placed in the variable 
  ** below.
  ** See also: local_sysctlblk.
  */

  struct sysctlblk *sysctlblk;

  /*
  ** Free list of stack control blocks.
  **
  ** All stacks are managed by a stack control block.
  ** The stack control block is always allocated from the system pool, while
  ** the stack space itself is located in the pool attached to the client
  ** process
  **
  ** Free stacks are kept in the pool to which the stack belongs.
  ** When a pool dies, the stack space is freed, but the stack control blocks
  ** survive and are placed in the free list of stack control blocks.
  ** When a new stack is created in some other pool, the control block is
  ** fetched from the free list if possible.
  */

  struct stkcb *stkcb_free_list;

  /*
  ** The dispatcher is locked when this flag is set.
  ** When set, any swap is postponed until the dispatcher
  ** is again unlocked.
  */

  volatile OSBOOLEAN SMP_CODE_YES_NO(dispatcher_lock_flag_NOSMP,
				     dispatcher_lock_flag);


  /*
  ** The current error check level as set by the system debugger.
  ** It is used only for ERROR_CHECKS = 1.
  ** For other configurations, the error check level can not be set.
  ** The values are used a follows:
  ** error_level1 = nonzero => enable signal buffer error checks.
  ** error_level2 = nonzero => enable all ERROR_CHECKS 2 testing.
  */
  U8 error_level1;	/* Set to 1 on cold start. */
  U8 error_level2;	/* Set to 0 on cold start. */
  U8 pad1;		/* 32 bit alignment. */

  /*
  ** List of all blocks.
  **
  ** All blocks in the system are kept in a linked list.
  ** The last pointer is NULL.
  **
  ** The first block created (the system block) is stored in zzuu.system_block
  ** so that it can easily be located in the set_clib_pointer() function.
  ** This variable is also useful when checking that harmful operations are
  ** not carried out on system processes.
  */

  struct _block *bcb_list;
  struct _block *system_block;

  /*
  ** Free list of process control blocks.
  **
  ** This list is used to quickly find free buffers when a new process control
  ** block, block descriptor, or segment control block is needed.
  ** Because of size, the pool control blocks have their own free list.
  **
  ** Each buffer is permanently allocated to a slot in the process index
  ** translation list (zzuu.pix_list), but it would involve much work to scan 
  ** this list (array) when a new buffer is needed. Instead this list is used.
  **
  ** When a process/bloc/segment is killed, the pcb is freed by the system daemon
  ** when all cleanup work has been finished.
  */

  struct _pcb *pcb_freelist;

  /*
  ** Dispatcher state and dispatcher restart request flag.
  **
  ** The dispatcher state is used by interrupt processes to determine if a
  ** dispatch operation is in progress and how far it has gone.
  ** There are four states:
  **	0  Not in dispatcher.
  **	1 In dispatcher save phase.
  **	2 In dispatcher search phase.
  **	3 In dispatcher restore phase.
  **
  ** The dispatcher restart request flag is set by interrupt processes when a
  ** system call causes:
  ** - A higher priority process to become "ready".
  ** - The current non interrupt process to become "not ready".
  ** - The background process list is rotated while a background process or
  **   the dispatcher serach phase is running.
  ** The flag is cleared by the dispatcher.
  */

  /* Dispatcher state. */
  volatile U8 SMP_CODE_YES_NO(dp_state_NOSMP, dp_state);

  /* Dispatcher (restart) request flag. */
  volatile OSBOOLEAN SMP_CODE_YES_NO(dp_req_NOSMP, dp_req);

  /*
  ** Set by non-nested interrupts. Cleared and tested in the idle loop
  ** to plan power save.
  */
  volatile U8 SMP_CODE_YES_NO(dp_idle_interrupted_NOSMP,
			      dp_idle_interrupted);

  /*
  ** Used to tell the scheduler to poll out any ose_fsend signals
  ** before scheduling any background process.
  */
  OSBOOLEAN bg_receive_poll;

  /*
  ** Timer interrupt priority, timer manager pid.
  **
  ** The timer interrupt priority is the interrupt priority of the timer
  ** processes. It is used when issuing wakeup calls to a timer process.
  ** If the timer manager is an interrupt process, then the timer
  ** interrupt priority is that of the timer manager. If the timer manager is
  ** a non interrupt process, then the timer interrupt priority is set to the
  ** lowest priority possible. If there is no timer manager, then the timer
  ** interrupt priority is set to the highest priority possible.
  ** (Do not confuse "timer process priority" with "timer interrupt priority")
  ** This variable is used only when an entry in the wakeup list is
  ** selected for a timer process, i.e. for the first wakeup request
  ** in a burst.
  **
  ** The timer manager pid is a pointer to the PCB of the process that last
  ** used the tick system call. Only one timer manager is allowed or an error
  ** occurs. Timer manager pid is set to NULL if no manager is present.
  **
  ** The timer interrupt priority and timer manager pid is updated if the
  ** timer manager is killed or when a new process calls tick().
  */

  struct _pcb *timer_manager;
  OSPRIORITY timer_int_priority;
  U8 pad6, pad7, pad8;		/* 32 bit alignment. */

  /*
  ** Background process ready list. See struct background_list_entry.
  */
  struct background_list SMP_CODE_YES_NO(bg_list_NOSMP, bg_list);
  
  /*
  ** Power fail request flag.
  **
  ** This flag is set by the power_fail system call. It causes the dispatcher
  ** to shut the system down instead of entering the dispatcher search phase.
  ** The flag is cleared by the dispatcher search phase after the system has
  ** been restarted.
  */

  volatile OSBOOLEAN pfail_req;	/* Power fail request flag. */
  U8 pad9, pad10, pad11;	/* 32 bit alignment. */

  /*
  ** The hunt list contains all pending hunt requests in the system.
  **
  ** The list is entirely managed by the hunt daemon.
  **
  ** The hunt list is referenced when a hunt request is made, when a new process
  ** is created, when a new linkhandler is created, when the user id of
  ** a process is changed and when a process is killed.
  **
  ** The list is searched either for process name, linkhandler prefix,
  ** owners process id or the process id of the client.
  ** There is a splay tree for each of these sort keys.
  **
  ** The trees contain hunt control blocks of the type struct hunt_request.
  */

  treenode *hunt_list_T_name;
  treenode *hunt_list_T_linkname;
  treenode *hunt_list_T_owner;
  treenode *hunt_list_T_from;

  /*
  ** The linkhandler list contains all linkhandlers in the system.
  **
  ** The list is entirely managed by the hunt daemon.
  **
  ** The linkhandler list is referenced when a hunt request is made, when
  ** a linkhandler is created and when a process is killed.
  **
  ** The list is searched for either the linkhandler name or the process id
  ** of the linkhandler.
  ** There is a splay tree for each of these sort keys.
  **
  ** The trees contain lnkhandler control blocks of the
  ** type struct new_linkhandler.
  */

  treenode *linkh_list_T_link_name;
  treenode *linkh_list_T_pid;

  /*
  ** The process list contains all processes in the system.
  **
  ** The list is entirely managed by the hunt daemon.
  **
  ** The process list is referenced when a process is created, when a
  ** process is killed and when a hunt request is made.
  **
  ** The list is searched for either the process name, or the pcb pointer
  ** of the process.
  ** There is a splay tree only for the process name sort key, since the pcb
  ** pointer can always be located through the zzuu.pix_list array.
  **
  ** The tree contains process control blocks.
  */

  treenode *pcb_list;

  /*
  ** The breakpoint list.
  **
  ** this list contains all currently set breakpoints in the system.
  **
  ** Entries are inserted by set_bp() and they are deleted by clear_bp()
  ** and by the system daemon when breakpoints are removed or reached resp.
  **
  ** Any non int process can trip a hardware breakpoint.
  ** Any int or non int process can trip a software watchpoint by calling
  ** intercept() or resume().
  */

  struct breakcb *break_list;

  /*
  ** Current warm start mode.
  **
  ** When the system starts, on of the follwoing start modes are selected by
  ** the target interface:
  **   0 => Cold start. All memory is initialized.
  **   1 => Warm start. Speed up system start and allow the debugger to
  **        recover internal setup data and the trace.
  **   2 => Hot start. Go directly into the dispatcher since the system was
  **        previously shut down by a power_fail() system call.
  */

  U8 start_mode;

  /*
  ** Pool garbage collection enable flag.
  **
  ** When this flag is nonzero, the system daemon performs periodic checks of
  ** all pools to see if they can be reclaimed.
  **
  ** The flag is set whenever there is a memory segment containing pools only.
  */

  OSBOOLEAN garbage_collection_enabled;
  U8 pad12, pad13;		/* 32 bit alignment. */

  /*
  ** The pcb of the idle process.
  **
  ** This is not a real process, but the pcb exists to make sure there are no
  ** special cases in the routines that manipulate pcb structures.
  */

  /* Pointer to the pcb of the idle process. */
  struct _pcb *SMP_CODE_YES_NO(idle_pcb_NOSMP,idle_pcb);

  /*
  ** Pool list.
  **
  ** There is a linked list containing all pool descriptors in the system.
  ** Pools are created with the create_pool() system call, and they are killed
  ** when the pool is no longer in use by any process.
  **
  ** Pool control blocks are kept along with process control blocks and block
  ** descriptors in the process index translation list. The system pool control
  ** block must be in this list too. We keep it in the first location of
  ** the index list (zzuu.pix_list) since this location is wasted anyway.
  **
  ** See also: create_pool(), zzuu.pix_list, sysd_do_free_pool(), system_pool.
  */

  struct poolcb * volatile pool_list;  /* The global pool list. */

  /*
  ** The segment list.
  **
  ** Address spaces are called segments. The kernel assumes that only
  ** the current segment can be directly accessed from supervisor space,
  ** and that user processes in different segments can not access
  ** each others data.
  **
  ** There is a system segment used by the kernel and supervisor mode
  ** processes. The system pool resides in this segment. The segment
  ** number and id is zero for the system segment. Other segment numbers are 
  ** assigned by user written memory manager software through the 
  ** attach_segment() system call.
  **
  ** Segment descriptors are deleted when they are no longer needed, i.e.
  ** when the last pool or process in the segment dies.
  **
  ** See also: system_segment.
  */

  struct segcb *segment_list;

  /*
  ** Timeout list and current timeout pointer.
  **
  ** The timeout list consists of an array of pointers to process lists.
  ** Each entry corresponds to a point in time. Processes waiting for a timeout
  ** are queued in the appropriate list. Timeouts too large to be managed in
  ** one step are split into several shorter timeouts. The list is allocated
  ** from the system pool. The lenght of the list is a power of two, taken
  ** from the kernel configuration, or defaults to 256.
  */

  struct _pcb **SMP_CODE_YES_NO(tmolist_NOSMP,tmolist);	/* Timeout list. */

  /* Current index in tmolist. */
  volatile U16 SMP_CODE_YES_NO(curtmo_NOSMP,curtmo);

  /* Mask for index in tmolist and tos_tick_list. */
  U16 curtmo_mask;

  /* Used to convert a timeout time to a toscb cycle counter. */
  U16 timeout_shift;
  U8 pad14;
  U8 pad15;

  /*
  ** Timer process scheduling list.
  **
  ** This list is an array similar to the timeout list. It is used to shedule
  ** timer interrupt processes. When a new timer interrupt process is created,
  ** it is placed in the least loaded slot between the current time and the
  ** timeslice allocated for the process. In this way timer processes are
  ** distributed so that work load concentration is somewhat avoided.
  ** The timer scheduling list is accessed through the current timeout pointer.
  */

  struct _pcb *tmslist[256];	/* Timer process list. */

  /*
  ** Prioritized Process ready list. This list consists of an array of
  ** 32 fields, each corresponding to a specific process priority.
  ** Each field contains a head pointer and a tail pointer and a bit mask used
  ** to modify and examine dispatcher look aside flags.
  ** A head pointer is NULL if no process on the priority is ready.
  ** Processes becoming "ready" are added at the tail.
  ** Processes becoming "not ready" are removed from the head.
  ** Only processes with "ready" status are kept in the list.
  ** 
  ** There is also a structure in each pcb. This structure contains a pointer
  ** to the ready list entry that corresponds to the priority of the process.
  ** The pcb contains a ready flag that is set whenever the process is in the
  ** ready list, a pointer to the next pcb in the ready list, a pointer to the
  ** previous pcb in the ready list and a pointer to the ready list entry for
  ** the priority.
  ** 
  ** For background processes, only the is_ready field is used.
  ** 
  ** See also: make_mypri_not_ready(), make_otherpri_not_ready(),
  **           make_otherpri_ready(), make_otherpri_ready_noswap(),
  **           zzdp_look_aside.
  */

  /* Ready list. */
  struct ready_list_entry SMP_CODE_YES_NO(ready_list_NOSMP, ready_list)[32];
  
  /*
  ** Wakeup list for interrupt and timer processes.
  ** Wakeup look aside flag.
  **
  ** This list holds pointers to interrupt processes awaiting wakeup calls.
  ** The list is similar to the ready list for prioritized processes, except
  ** processes are always added and removed at the head, so no backward
  ** link is required in the pcb's.
  ** Each field in the array corresponds to an interrupt priority.
  ** Each field contains a head pointer and a bit mask used
  ** to modify and examine the wakeup look aside flags.
  ** The wakeyp look aside flag word is a 32 bit word. It  contains one
  ** bit for each interrupt priority with a pending wakeup request.
  ** The most significant bit corresponds to
  ** the highest interrupt priority, i.e. bit 0 corresponds to priority 31.
  ** A bit in this word is set if at least one process is
  ** ready on the corresponding priority. It is cleared otherwise.
  ** A head pointer is NULL if no process on the priority is ready.
  ** Processes receiving a "wakeup request" are added at the head.
  ** Processes are removed from the head after execution is complete.
  ** Only processes with at least one "wakeup request" are kept in the list.
  ** 
  ** There is also a structure in each pcb. This structure contains a pointer
  ** to the wakeup list entry that corresponds to the priority of the process.
  ** For timer processes this pointer is valid only when the pcb is linked into
  ** the wakeup list, because the pointer is set when a wakeup list is selected.
  ** The same list fields in th pcb are used for the interrupt wakeup list and
  ** the ready list.
  ** The pcb contains two wakeup counters, which are nonzero whenever the process
  ** is in the wakeup list, a pointer to the next pcb in the wakeup list and
  ** a pointer to the ready list entry for the priority.
  ** 
  ** See also: struct wakeup_list_entry,request_int_semwakeup(),
  **           request_int_sigwakeup(), request_timer_semwakeup(),
  **           request_timer_sigwakeup().
  */

  /* Wakeup list look aside flag word. */
  volatile U32 SMP_CODE_YES_NO(wk_look_aside_NOSMP, wk_look_aside);
  /* Wakeup list. */
  volatile struct wakeup_list_entry SMP_CODE_YES_NO(wakeup_list_NOSMP,
						    wakeup_list)[32];

  /*
  ** Interrupt stack and associated statistics.
  **
  ** All interrupt and timer processes share a single interrupt stack.
  ** The size of this stack is calculated by summing up the worst case
  ** requirements for each of the 32 interrupt priorities and for the
  ** timer interrupt list.
  **
  ** When a new process is created, the worst case requirements are
  ** updated, and the stack is possibly reallocated.
  **
  ** When a process is killed, worst case requirements are also
  ** updated, but the stack is not reallocated.
  */

  /* Base of interrupt stack. */
  struct stkcb *SMP_CODE_YES_NO(interrupt_stack_NOSMP,interrupt_stack);
  OSADDRESS interrupt_stack_size;	/* Size of interrupt stack. */

  OSADDRESS wanted_timer_size;	/* Timer requirements. */
  OSADDRESS wanted_interrupt_size[32]; /* Interrupt requirements. */

  /*
  ** The currently selected segment number.
  **
  ** This variable is updated during each swap in operation.
  ** If it changes to a nonzero value, then the select_segment hook is called.
  */

  OSSEGMENT SMP_CODE_YES_NO(cur_segid_NOSMP,cur_segid);

  /*
  ** Process index translation list, list of free process index translation
  ** slots and list of free process control blocks.
  **
  ** The process index translation list is an array of pointers to the process
  ** control blocks of all processes in the system. Slot zero can not be used
  ** for a real process since index 0 is used for end of various lists.
  ** Virgin slots contain NULL. These make up the list of free process
  ** index translation slots. There is a variable containing the index of
  ** the next free slot. Out of memory if it reaches MAX_PROC.
  ** New slots are allocated from this list if there are no reusable PCB's in
  ** the list of free process control blocks. There is a backward link to the
  ** corresponding translation slot in each PCB.
  **
  ** The system uses the same set of buffers for block descriptors, segment 
  ** control blocks and process control blocks.
  ** Free and allocated buffers are kept in the zzuu.pix_list[] array.
  ** Free buffers are also kept in the zzuu.pcb_freelist linked list.
  ** The zzuu.pix_list[] array also keeps the pool control blocks, but because
  ** of their larger size, these block have their own free list.
  */
  
  PCB_INDEX_T next_free_pix;	/* Next unused index into zzuu.pix_list. Init to 1. */
  U8 pad17, pad18;		/* 32 bit alignment. */
  struct _pcb **pix_list;	/* Array of pointers to all pcb's. size is MAX_PROC */
  OSADDRESS max_proc;		/* Copy of zzuu.sysctlblk->max_procs. */

  /*
  ** The attach pool and free list of attach records.
  **
  ** The attach pool consists of a large array of records.
  ** The free list consists of a circular linked list of
  ** unused entries in the attach pool.
  **
  ** See also: MAX_ATTACH, expired_attach_head.
  */

  struct _attach_entry *free_attach_head;
  struct _attach_entry *free_attach_tail;
  struct _attach_entry **attach_pool;
  OSADDRESS max_attach;		/* Copy of sysctblk->max_attached. */
  OSADDRESS next_free_attach; /* Pointer to next unused attach_pool entry. */
  OSADDRESS free_attach_count; /* Number of records in free_attach_head. */

  /*
  ** Process id's for system daemons.
  **
  ** The kernel has a number of internal processes.
  ** These variables contain the process id's of these daemons.
  ** The huntd2_pcb is a pointer to the process control block of the
  ** huntd2 process. It is used internally in the huntd process.
  **
  ** The ldm is referenced only when OSE_DEBUG > 0.
  */

  PROCESS tickd_;	/* The tick daemon. */
  PROCESS sysd_;	/* The system daemon. */
  PROCESS huntd_;	/* The hunt daemon. */
  PROCESS huntd2_;	/* The non superuser hunt daemon. */
  struct _pcb *huntd2_pcb;

  PROCESS ldm_;	/* The debugger daemon. */

  /*
  ** The pcb of the process that was last swapped out.
  **
  ** This variable is used in the zzswapin_dbghook() to present
  ** the hook with both the new and the old process.
  */

  struct _pcb *SMP_CODE_YES_NO(last_cur_ni_NOSMP,last_cur_ni);

  /*
  ** Master debugger enable flag.
  **
  ** This flag is set to a nonzero value by the system debugger to
  ** enable all system debugger hooks except create and kill.
  ** The debugger hooks are then called as specified by the debug_flags
  ** field in the pcb for each process.
  **
  ** It is set to zero during system start.
  **
  ** The variable is referenced only when OSE_DEBUG > 0.
  */

  OSBOOLEAN debugger_enabled;

  /*
  ** System start debugger enable flag.
  **
  ** This flag is set to a nonzero value when system processes have been
  ** created to enable the system debugger hooks for create.
  **
  ** It is set to zero during early system start.
  **
  ** The variable is referenced only when OSE_DEBUG > 0.
  */

  OSBOOLEAN sysprocs_created;

  /*
  ** Interrupt process swap hook enable flag.
  **
  ** This flag is set to a nonzero value by the system debugger to
  ** enable system debugger hooks for interrupt process swap.
  ** The debugger hooks are then called for each interrupt process.
  **
  ** Setting the flag to a value > 1 causes the same effect also
  ** for timer processes. This facility is used for trace dispatch
  ** operations in the LDM.
  **
  ** It is set to zero during system start.
  **
  ** The variable is referenced only when OSE_DEBUG > 0.
  */

  U8 inthook_enabled;

  /*
  ** Tick debugger hook enable flag.
  **
  ** This flag is set to a nonzero value by the system debugger to
  ** enable system debugger hooks for system ticks.
  ** The debugger hook is called each time a system tick is encountered.
  **
  ** It is set to zero during system start.
  **
  ** See also: zzprof_tickhook(), zzset_tickhook_ldmstatus()
  */

  U8 tickhook_enabled;

  /*
  ** Free list of pool control blocks.
  **
  ** This list is used to quickly find free buffers when a new pool control
  ** block is needed. Because of size, the pool control blocks have their own 
  ** free list.
  **
  ** Each buffer is permanently allocated to a slot in the process index
  ** translation list (zzuu.pix_list), but it would involve much work to scan 
  ** this list (array) when a new buffer is needed. Instead this list is used.
  **
  ** When a pool is killed, the poolcb is freed by the system daemon
  ** when all cleanup work has been finished.
  */

  struct poolcb *poolcb_freelist;

  /*
  ** Free list of pool fragment control blocks. NULL when empty.
  **
  ** One of these blocks is associated with each pool fragment, including
  ** the main fragment passed to create_pool. Fragment control blocks are
  ** always in the system pool. When a pool is killed, the control blocks
  ** are returned to this list. Note that system pool fragments don't have
  ** separate control blocks. Instead the control block is snipped from
  ** each fragment, which doesn't matter to us since the system pool can
  ** not be killed.
  **
  ** The free list must never be allowed to become empty since the
  ** extend_pool() system call needs a control block to be available at
  ** all times. A new control block is allocated each time an error handler
  ** was called for an out of memory error.
  */

  struct pool_extension *free_frag_list;

  /*
  ** Free list of signal administration blocks.
  **
  ** For memory models > 0, we use separate signal administration blocks
  ** in the system pool.
  ** When a pool is killed, these administration blocks go into the free list
  ** for recycling.
  */

  struct _signal *free_sigadm_list;

  /*
  ** The minimum time to next timer event for entering power save mode
  ** and a variable containing the accumulated time spent in the power
  ** save handler.
  **
  ** power_save_threshold:
  ** Is set by calling the set_halt_condition() system call.
  ** A value of -1 indicates that the kernel should never enter power
  ** save mode.
  ** It is set to -1 during system start.
  **
  ** power_saved_ticks:
  ** Is incremented when the power save handler returns.
  ** It is set to 0 during system start and when the global time counter wraps.
  ** The quota (power_saved_ticks / ticks) gives a simple power save measure.
  ** 
  ** The use of these variables is affected by defining USE_POWER_SAVE > 0.
  */

  OSTICK power_save_threshold;
  OSTICK power_saved_ticks;
  OSTICK longest_power_nap;

  /*
  ** Time Out Signal Handling.
  **
  ** The time out signal handling is managed via Time Out Signal Control Block
  ** (toscb) structures. These blocks are accessable by index, by tick slot or
  ** by process. New toscbs are allocated on demand. No longer needed toscbs
  ** are placed in their own free list. Access by index uses pointer
  ** directories in two levels, like MMU hardware table walk. 
  */

  /* Next unused index to control allocation of toscb. Highest index is used
  ** first. Index zero is illegal. The index is divided into 3 bit groups.
  ** The least significant TOSCB_ALLOC_2LOG bits form an index in a
  ** toscb[TOSCB_ALLOC_CHUNK] array.
  ** The next 8 bits are a level 2 index.
  ** Next 8 bits form a level 1 index in tos_index_list[] below.
  */
  volatile U32 next_free_tos_index; 

  /* Free list via fq pointers chain. The 'tail' pointer is undefined when
  ** 'head' is NULL.
  */
  struct tos_head tos_free_list;

  /* Time out signals per tick slot. Each slot has a double linked list.
  ** The list is allocated from the system pool. The lenght of the list
  ** is a power of two, taken from the kernel configuration, or defaults
  ** to 256.
  */
  struct tos_head *tos_tick_list; 

  /*
  ** Last timeout signal control block to check in tick().
  ** Used in SMP, where tick() runs concurrently with prioritized processes.
  ** New entries, created since tick() started shall not fire now, but
  ** at the next scan of the list. 
  ** Cancel and restart operations checks this member for match.
  ** 
  */
  struct toscb * volatile last_toscb;

  /*
  ** SMP spinlock for tos_free_list and tos_tick_list.
  ** The toscb_spin_lock may be taken with an already locked dispatcher.
  */

  spinlock_t toscb_spin_lock;

  /* Level 1 directory to access toscb by index. Level 2 directories are
  ** dynamically allocated. Highest index used first.
  */
  struct toscb **tos_index_list[256];

  /*
  ** Mutex.
  **
  ** The mutex handling is managed via Mutex Control Block (mutcb)
  ** structures. These blocks are accessable by index and by segment.
  ** New mutcbs are allocated on demand. No longer needed mutcbs
  ** are placed in their own free list. Access by index uses pointer
  ** directories in two levels, like MMU hardware table walk. 
  */

  /* Next unused index to control allocation of mutcb. Index zero is
  ** illegal. The index is divided into 2 bit groups.
  ** The 8 least significant bits is a level 2 directory index.
  ** Next 8 bits form a level 1 index in mutex_index_list[] below.
  */
  volatile U32 next_free_mutex_index; 

  /* Free list via next pointers chain. */
  struct mutcb *mutex_free_list;

  /* Level 1 directory to access mutex by index. Level 2 directories are
  ** dynamically allocated.
  */
  struct mutcb **mutex_index_list[256];

  /*
  ** List of free attach entries.
  **
  ** The expired_attach_head is the head of a single linked list of
  ** all attach entries that has expired, but hasn't been confirmed as
  ** reusable. (The list is linked using own_fq as the next pointer.)
  **
  ** The free_attach_agent is used by free_attach() to recognize
  ** situations where it has beed preempted and needs to restart its
  ** operation.
  **
  ** See also: free_attach_head, free_attach().
  */

  struct _attach_entry *expired_attach_head;
  PROCESS free_attach_agent;

  /*
  ** Per Process Data Handling.
  **
  ** ppd_key_buf contains all used key strings, each one word aligned
  ** and nul terminated. ppd_hash_buf contains hash values to
  ** eliminate most string compares. allocated_ppd_key_size is the
  ** maximum string length including nul terminator and word
  ** alignment. ppd_gen_counter is used to detect preemption in the
  ** search procedure. next_free_ppd_index points to the next
  ** unallocated slot. max_ppd_keys is the configured number of keys.
  ** max_ppd_key_size is the maximum string length of a key.
  */

  char *ppd_key_buf;
  char *ppd_hash_buf;
  OSADDRESS allocated_ppd_key_size;
  volatile OSADDRESS ppd_gen_counter;
  OSADDRESS next_free_ppd_index;
  OSADDRESS max_ppd_keys;
  OSADDRESS max_ppd_key_size;

  /*
  ** Paging variables.
  **
  ** A limited model of paging is supported in MEMORY_MODEL 1.
  ** Only text (code and const) areas may be paged.
  ** The pagesize variable is set to the smallest  page size used when
  ** paging  is enabled, zero if not  enabled. The variable is used by
  ** some system calls that may refer to the text segment.
  */

  OSADDRESS pagesize;		/* Smallest pagesize, zero if paging is off.*/

  /*
  ** This variable is incremented each time a pool is deleted. It's
  ** mainly used to overcome a race between check_buffer() and
  ** sysd_check_free_pool().
  */
  volatile OSADDRESS pool_delete_cnt;

  /*
  ** Time resolution per priority level. (DEPRECATED)
  ** Can be used to force a more coarse time resolution on selected priority
  ** levels. The purpose is to collect less time critical events in clusters
  ** to get fewer wake up events from power save, and thus save battery.
  ** Index 32 controls background processes. Zero gives full resolution. 
  ** Other possible values are (2**n - 1) ticks [1,3,7...255].
  */

  U8 timer_resolution[33];	/* (DEPRECATED) */
  U8 pad34;

  U8 annotate_semaphores;

  /*
  ** Enable flag for optimizing of context switch tracing.
  ** When enabled all context switches will be traced if the tracing is enabled.
  ** Controlled by the rmm by calls to zzset_trace_swaps_enabled().
  */      

  OSBOOLEAN trace_all_swaps;

  /* Identifier of the Program Manager. */
  PROCESS prog_mgr_;
   
  /* Process being created by sysd at this very moment, or 0. */
  PROCESS process_under_construction_;

  /*
  ** Pointer to the poolcb of the main pool. Can be NULL if no main pool or if
  ** the pool is not yet created. Updated when setting a pool extension handler
  ** for the main pool through sysconf 67.
  */  
  struct poolcb *main_pool;

  /*
  ** This variable is incremented in tick() and checked in
  ** get_systime() to ensure that the tick and and microticks values
  ** are in sync. The problem is mainly SMP related and was solved
  ** this way because the debugger issues wast amounts of
  ** get_systime() calls which would hog the kernel if it contained any
  ** other type of locking, like BKL or spinlock.
  */
  volatile U32 systime_guard;

  /* The address alignment of any object in any pool.
  ** DMA may require cache line alignment of signals.
  */
  U32 pool_alignment;

  /* Hash table holding data for all interrupt vectors that have been used. */
  struct vectorcb *vectors[256];

  /* Counters for alive timer processes, pending timeouts, resp. pending
  ** timeout signal requests.
  ** These counters are used to speed up entry to the power save handler.
  */
  PCB_INDEX_T timer_process_count;
  PCB_INDEX_T SMP_CODE_YES_NO(timeout_count_NOSMP,timeout_count);
  U32 tmo_signal_count;

  /*
  ** Buffer used for burst receiving ose_fsend() sent signals in unicore.
  */
  struct ose_krn_msg msg[8];

  U32 pad99;

/*
** This is the end of the world structure in case you have lost track by now.
*/

};

/*
**===========================================================================
** 4.2.3  The kernel's per CPU data structure.
**===========================================================================
*/

/*
** The per_cpu structure contains all variables that are local to a
** CPU when POSE is running a SMP type of system. All these variables
** are found (and documented) in the world structure for non SMP
** systems.
*/

#ifdef OSE_SMP

struct per_cpu
{
  struct ticpu            ticpu;	/* Target Interface per cpu data. */

  /* Pose offset 0 bytes */
  
  void                    *curr_uarea;
  struct _pcb             *curr_p;
  struct _pcb             *curr_ni;
  OSBOOLEAN                ipi_operable;
  volatile U8              dp_state;
  volatile OSBOOLEAN       dp_req;
  volatile U8              dp_idle_interrupted;

  /* Pose offset 16 bytes */
  
  OSSEGMENT                cur_segid;
  volatile U32             dp_look_aside;
  volatile U32             wk_look_aside;
  volatile OSBOOLEAN       wakeflag;
  OSPRIORITY               wakeup_level;
  OSBOOLEAN                state;  /* See enum cpu_state. */
  volatile U8              stop_state; /* See pose.c zzkrn_smp_stop_cpu() */
  
  /* Pose offset 32 bytes */
  
  struct background_list  bg_list;
  PROCESS		  sysd_lite_; 

  
  /* Pose offset 48 bytes */
  
  struct _pcb             *idle_pcb;
  struct stkcb            *interrupt_stack;
  struct _pcb             *last_cur_ni;
  PCB_INDEX_T              timeout_count;
  OSBOOLEAN                bg_receive_poll;
  U8                       __pad4[1];


  /* Pose offset 64 bytes */
  /*
  ** The following data should be allocated into a seldome written cache
  ** line. The ticks member of cpu0 is read from other cpus in 
  ** request_timeout() to unify the view of time for tmolist timeouts
  ** better among the cpus, by compensating for a pending tick() call.
  */
  struct _pcb **tmolist; /* Timeout list. */
  OSTICK ticks;		/* Local 32 bit copy of zzuu.ticks. */
  volatile U16 curtmo;	/* Current time in tmolist and tos_tick_list. */
  U8 __pad2[22];

  /* 352-bytes */
  struct          ready_list_entry   ready_list[32];
  volatile struct wakeup_list_entry  wakeup_list[32];
  
  /* Pose offset 864 bytes */
  struct ose_krn_msg msg[8];

  /* Pose offset 928 bytes */
  OSSEGMENT                leave_segid_now;
  void (*hw_receive_poller)(void); /* Hw poller for fast path systems. */

  /* Pose offset 936 bytes */
  /* Reserve exec part (ti+pose) upto 1536 == 0x600 */
  U8 __pad_exec_part[POSE_PER_CPU_END - TI_PER_CPU_SIZE - 936];

  /* Make size power of 2 (2048) */
#if PER_CPU_DATA_SIZE > POSE_PER_CPU_END
  U8 __pad_per_cpu_data[PER_CPU_DATA_SIZE - POSE_PER_CPU_END];
#elif PER_CPU_DATA_SIZE < POSE_PER_CPU_END
#error "Illegal per cpu data sizes, PER_CPU_DATA_SIZE < POSE_PER_CPU_END:" PER_CPU_DATA_SIZE  POSE_PER_CPU_END
#endif
};

/*
** This is the BKL (Big Kernel Lock) stucture.  If the 'lock' member
** is not zero the lower bits should contain the cpu_id of the of the
** cpu that currently owns the lock. The other members are there to help
** debug a BKL implementation (often found in smp_lock.c).
** 
*/
struct ZZSmpBkl
{
   uint32_t lock;
   uint32_t lock_addr;
   uint32_t unlock;
   uint32_t unlock_addr;
   uint32_t _pad[4];
};

/*
** Macro to verify size of struct per_cpu at system start @init_per_cpu().
** Should be a power of 2 for performance reasons. 
*/
#define PER_CPU_SIZE 2048

#define zzww                (*zzkrn_get_per_cpu_data())
#undef  zzcurrent_uarea
#define zzcurrent_uarea     (zzww.curr_uarea)
#undef  zzcur_p
#define zzcur_p             (zzkrn_get_cur_p())
#undef  zzcur_p_usr
#define zzcur_p_usr         (zzkrn_get_cur_p_usr())
#undef  zzcur_ni
#define zzcur_ni            (zzww.curr_ni)
#undef  zzwakeflag
#define zzwakeflag          (zzww.wakeflag)
#undef  zzwakeup_level
#define zzwakeup_level      (zzww.wakeup_level)
#undef  zzdp_look_aside
#define zzdp_look_aside     (zzww.dp_look_aside)
extern int zzdispatcher_locked;
//#define zzdispatcher_locked (zzkrn_bkl_swp.lock == zzkrn_bkl_lock_id())
extern int zzinterrupts_locked;
//#define zzinterrupts_locked (zzkrn_bkl_irq.lock == zzkrn_bkl_lock_id())
#define set_zzcur_p(x)      (zzkrn_set_cur_p(x))

#define ready_list_x(x)     (zzcpu[(x)].ready_list)
#define wakeup_list_x(x)    (zzcpu[(x)].wakeup_list)
#define bg_list_x(x)        (zzcpu[(x)].bg_list)
#define dp_look_aside_x(x)  (zzcpu[(x)].dp_look_aside)
#define zzcur_p_x(x)        (zzcpu[(x)].curr_p)
#define zzcur_ni_x(x)       (zzcpu[(x)].curr_ni)
#define zzlast_cur_ni_x(x)  (zzcpu[(x)].last_cur_ni)
#define zzidle_pcb_x(x)     (zzcpu[(x)].idle_pcb)

#else

#define zzww                zzuu
#define ready_list_x(x)     zzuu.ready_list
#define wakeup_list_x(x)    zzuu.wakeup_list
#define bg_list_x(x)        zzuu.bg_list
#define dp_look_aside_x(x)  zzdp_look_aside
#define zzcur_p_x(x)        zzcur_p
#define zzdispatcher_locked zzuu.dispatcher_lock_flag
#define set_zzcur_p(x)      (zzcur_p = (x))
#define zzcur_ni_x(x)       zzcur_ni
#define zzlast_cur_ni_x(x)  (zzuu.last_cur_ni)
#define zzidle_pcb_x(x)     (zzuu.idle_pcb)

#endif

/*
**===========================================================================
** 4.2.4  The post mortem support table
**===========================================================================
*/

/*
** The post mortem support table is the handle used to access kernel data
** structures.
**
** It is organized as a header structure in the const section, which points
** to all other data structures.
*/

struct osvarp
{
  /*
  ** Kernel type.
  ** 
  ** This variable tells post mortem analyzers that the kernel is present,
  ** the type code of the pose kernel, and the byte order of the cpu.
  ** It is set to MAGIC_KERNEL_TYPE.
  */

  U32 magic_kernel_type;

  /*
  ** This variable indicates the kernel feature level to post mortem analysis
  ** tools. It is set to MAGIC_KERNEL_FEATURES.
  **
  ** It is incremented whenever a new feature is added to the kernel, as long
  ** as the new feature still does not affect old tools.
  **
  ** New tools test on this constant to enable new features when they are
  ** relevant.
  **
  ** The constant is set to zero for each new kernel type, i.e whenever the
  ** MAGIC_KERNEL_TYPE is changed.
  */

  U32 magic_kernel_features;

  /*
  ** Target interface world structure.
  ** It starts with a 32 bit magic number that indicates the target type.
  ** This word may be set to zero to indicate that no information is
  ** present.
  */

  const U32 *zztivarp;

  /*
  ** LDM world structure.
  ** It starts with a 32 bit magic number that indicates the target type.
  ** This word may be set to zero to indicate that no information is
  ** present.
  */

  const void *zzldmvarp;

  /*
  ** Kernel symbol table pointer.
  ** This pointer holds a reference to the symbols used by post mortem tools.
  ** It is set to NULL if not present.
  ** Post mortem tools are supposed to refuse operation when symbols are
  ** unavailable, even if there may be special cases that work anyway.
  */

  void *posesym; 

  /*
  ** Kernel world structure pointer. (defined in symbol table)
  */

  struct world *pose_world;

  /*
  ** Current uarea pointer pointer. (always 32 bit)
  */

  void **uarea_varp;

  /*
  ** Wakeup level pointer.
  */

  OSPRIORITY *wakelevel_varp; 

  /*
  ** Size of wakeup level.
  */

  U32 wakelevel_size;

  /*
  ** Wakeup flag pointer.
  */

  volatile OSBOOLEAN *wakeflag_varp;

  /*
  ** Size of wakeup flag.
  */

  U32 wakeflag_size;

  /*
  ** Current process pointer pointer. (always 32 bit)
  */

  struct _pcb **curp_varp; 

  /*
  ** Current nonint process pointer pointer. (always 32 bit)
  */

  struct _pcb **curni_varp; 

  /*
  ** Dispatcher look aside flag pointer. (always 32 bit)
  */

  volatile U32 *aside_varp;

  /*
  ** System pool pointer. (defined in symbol table)
  */

  struct poolcb *syspool_varp; 

  /*
  ** System segment pointer. (defined in symbol table)
  */

  struct segcb *syseg_varp;

  /*
  ** Pointer to Idle pcb pointer. (defined in symbol table)
  */

  struct _pcb **idle_varp;

  /*
  ** Pointer to what-string describing the current pose release.
  */

  const char *poserev;

  /*
  ** Pointer to the system handler array.
  ** This array contains pointers to user supplied handlers or callback
  ** functions which may be altered in runtime by power users.
  */

  struct handlerp *handler_varp;

  /*
  ** Pointer to the pose configuration array.
  ** This array contains values defined in pose.con.
  */

  const U32 *conf_varp;

  /*
  ** Pointer to the usrcalls array.
  ** This array contains pointers to system calls that can be invoked
  ** directly from user mode.
  */
   
  const struct krn_usrcalls *usrcalls_varp;

  /*
  ** Pointer to the first element in the zzcpu array, or NULL if not a
  ** SMP type of system.
  */
  const struct per_cpu *percpu_varp;

  /*
  ** The size of struct per_cpu.
  */
  U32 percpu_size;

  /*
  ** Number of elements in the zzcpu array.
  */
  U32 percpu_array_elems;

  /*
  ** Pointer to the pointer containing the poolcb of the main pool.
  */
  struct poolcb **main_pool;

  /* Pointer to zzuu.pix_list. */
  struct _pcb ***pix_list_handle;
};


/*
*****************************************************************************
** 5  GLOBAL VARIABLES.
*****************************************************************************
*/


/*
** The zzcpu array contains as many structures of global variables
** as there are CPUs in the SMP system.  
*/
#ifdef OSE_SMP
extern
struct per_cpu zzcpu[];


#else

/*
** Current uarea.
**
** This pointer is made available to the target interface.
** It reflects the current process at all times, unless
** the USE_INTERRUPT_UAREA constant is nonzero, in which
** case the variable reflects the current non interrupt process.
**
** When the idle process is running, the variable contains NULL.
*/

extern
void *zzcurrent_uarea;

/*
** Current wakeup level made known to the target interface.
**
** When a wakeup call is scheduled for a lower priority interrupt process,
** i.e. when a signal is sent from a higher priority to a lower priority
** interrupt process, a special wakeup request code is returned from either
** zzinit_int(), zzdo_wakeup() or zzbreakpoint_trap().
** At that point, the zzwakeup_level variable is set to tell which is
** the interrupt priority for which a wakeup call is requested.
** The target interface should read this variable and run the zzdo_wakeup()
** function.
**
** The variable is used only at this point.
**
** See also: zzdo_wakeup(), zzinit_int(), zzbreakpoint_trap().
*/

extern
OSPRIORITY zzwakeup_level;

/*
** Main wakeup flag.
**
** While a timer or interrupt process is running, this flag is set if
** the current process was invoked through a wakeup call.
** The flag may be examined with the wake_up system call.
** It is set to 1 for bgd and pri processes.
**
** The variable is made available to the target interface for efficient
** implementation of the dispatcher and various system calls implemented
** in assembler.
*/

extern
volatile OSBOOLEAN zzwakeflag;	/* wakeup flag for tim and int processes. */

/*
** Current process and current non interrupt process.
**
** These pointers contains the process control block of
** the currently running process of any kind, and the control block of the
** currently running non interrupt process. The pointers are valid outside
** any LOCKed region.
**
** The pointers are made available to the target interface for efficient
** implementation of the dispatcher and various system calls implemented
** in assembler.
** The pointers may also be useful to trusted applications such as
** cpu load analyzers and others.
*/

extern
struct _pcb *zzcur_p;		/* Current process. */
extern
struct _pcb *zzcur_ni;		/* Current non interrupt or timer process. */

/*
** The dispatcher look aside flag word is a 32 bit word. It  contains one
** bit for each process priority. The most significant bit corresponds to
** the highest priority process, i.e. bit 0 corresponds to priority 31.
** A bit in this word is set if at least one process is
** ready on the corresponding priority. It is cleared otherwise.
**
** The pointers are made available to the target interface for efficient
** implementation of the dispatcher and various system calls implemented
** in assembler.
** The pointers may also be useful to trusted applications such as
** cpu load analyzers and others.
**
** See also: make_mypri_not_ready(), make_otherpri_not_ready(),
**           make_otherpri_ready(), make_otherpri_ready_noswap(),
**           zzuu.ready_list.
*/

extern
volatile U32 zzdp_look_aside;	/* Ready list look aside flag word. */

#endif

/*
** The world structure contains all static variables in the kernel, except
** large local structures.
**
** The world structure is maintained to enable kernel data to be easily
** accessible from the target interface and from post mortem analyzer tools.
*/

extern
struct world zzuu;

/*
** The post mortem support table is the handle used to access kernel data
** structures.
**
** It is organized as a header structure in the const section, which points
** to all other data structures.
*/

extern
const struct osvarp zzosvarp;

/*
** These are the two BKL (Big Kernel Lock) variables, one for the
** interrupt part and one for the scheduling part of BKL.
*/

#ifdef OSE_SMP

extern
volatile struct ZZSmpBkl zzkrn_bkl_irq;

extern
volatile struct ZZSmpBkl zzkrn_bkl_swp;

#endif

#endif

