/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#include "efmi.h"

#include "fmm.h"
#include "fmm_sysparam.h"

#include "sysparam.h"

#include "monitor.sig"

#include "stddef.h"

#define assert(x) if (!(x)) return EFMI_UNAVAILABLE;

union Signal
{
   uint32_t signo;
   MonitorGetInfoRequest               get_info_request;
   MonitorGetSysParamReply             get_sys_param_reply;
   MonitorGetInfoEndmark               get_info_endmark;
};

struct FmmSignal
{
   uint32_t size;
   union Signal data;
};

static EfmiStatus
list_sys_param(int32_t (*reply)(FmmSignal *sig), 
               const MonitorGetInfoRequest *request)
{
   FmmSignal *sig; 
   MonitorGetSysParamReply *info;    
   
   sig = fmm_output_signal;
   info = &sig->data.get_sys_param_reply;

   info->data[0] = 0;
   
   for (;;)
   {
      size_t n, m;
      
      n = sysparam_list_next(info->data, info->data, 
                             FMM_OUTPUT_SIGNAL_SIZE - sizeof(*info));
                             
      if (n == 0)
      {
         return EFMI_OK;
      }
      if (n > FMM_OUTPUT_SIGNAL_SIZE - sizeof (*info))
      {
         return EFMI_RESULT_TRUNCATED;
      }
      
      m = sysparam_read(info->data, &info->data[n + 1], 
                        FMM_OUTPUT_SIGNAL_SIZE - sizeof(*info) - n);  
                        
      if (m == 0)
      {
         /* Apparently, no value was found. */
         info->data[n + 1] = 0;
      }
      if (m > FMM_OUTPUT_SIGNAL_SIZE - sizeof(*info) - n)
      {
         return EFMI_RESULT_TRUNCATED;
      }
         
      info->sigNo = MONITOR_GET_SYS_PARAM_REPLY;
      info->reserved0 = 0;
      info->nameOffset = 0;   
      info->valueOffset = n + 1;
      sig->size = sizeof(*info) + n + m + 1;
      
      if (reply(sig))
      {
         return EFMI_END_OF_LIST;
      }
   }
}

EfmiStatus 
fmm_sysparam_get_sys_param_request(int32_t (*reply)(FmmSignal *sig),
              	                    int32_t (*endmark)(FmmSignal *sig, int32_t status),
                                   const MonitorGetInfoRequest *request)
{
   EfmiStatus ret;

   FmmSignal *sig;
   
   assert(request != NULL);
   assert(request->sigNo == MONITOR_GET_SYS_PARAM_REQUEST);
   
   switch (request->scope.type)
   {
      case MONITOR_SCOPE_SYSTEM:
         ret = list_sys_param(reply, request);       
         break;
      default:
         ret = EFMI_CALL_INVALID;
         break;
   } 

   sig = fmm_output_signal;
   sig->size = sizeof(MonitorGetInfoEndmark);
   sig->data.get_info_endmark.sigNo = MONITOR_GET_SYS_PARAM_ENDMARK;
   
   switch (ret)
   {
      case EFMI_OK:
         sig->data.get_info_endmark.status = MONITOR_STATUS_OK;
         break;
      case EFMI_END_OF_LIST:
         sig->data.get_info_endmark.status = MONITOR_STATUS_CANCELLED;
         break;
      default:
         sig->data.get_info_endmark.status = MONITOR_STATUS_KERNEL_ERROR;
         break;
   }
   
   if (endmark(sig, ret))
   {
      return (ret) ? ret : EFMI_END_OF_LIST;
   }
            
   return ret;
}

              	 	
