/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

/* C runtime include files. */
#include <string.h>

/* OSE public include files. */

/* OSE internal include files. */
#include "assert.h"

/* MM internal include files. */
#include "allocator.h"
#include "partition.h"
#include "common.h"
#include "gmm.h"

/*
 * See allocator.h
 */
void
mm_allocator_init(void)
{
   gmm_state.allocators = NULL;
}


/*
 * See allocator.h
 */
struct allocator *
mm_find_allocator_by_name(const char * name)
{
   struct allocator *a;

   assert(name != NULL);

   for (a = gmm_state.allocators; a != NULL; a = a->next)
   {
      if (!strcmp(a->name, name))
      {
         return a;
      }
   }
   return NULL;
}

/*
 * See allocator.h
 */
struct allocator *
mm_find_allocator_by_range(phys_addr_t base, phys_size_t size)
{
   struct allocator *a;

   for (a = gmm_state.allocators; a != NULL; a = a->next)
   {
      /* Does this allocator handle parts of the range (<base>, <size>)? */
      if (a->overlap(a, base, size))
      {
         /* Yes! */
         return a;
      }
   }
   return NULL;
}

void
dump_allocator_states(int (*print)(const char *, ...)) {
   struct allocator *a;

   /*lint -save -e522 "Expected void type, assignment, increment or decrement"
    */
   print("Current set of allocators:\n");
   for (a = gmm_state.allocators; a != NULL; a = a->next)
   {
      if (a->dump_state) {
         a->dump_state(a, print);
      }
      else {
         print("  %s", a->name);
      }
   }
   /*lint -restore */
}

void mm_register_allocator(struct allocator *a) {
   assert(a);
   a->next = gmm_state.allocators;
   gmm_state.allocators = a;
}
