/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

/* C runtime include files. */
#include <stddef.h>
#include <string.h>

/* OSE public include files. */
#include "assert.h"
#include "ose.h"
#include "ramlog.h"
/*#include "osetypes.h"*/

/* OSE internal include files. */

/* MM internal include files. */
#include "common.h"
#include "compiler_macros.h"

#ifdef softOSE
void mm_kick_wdog(void)
{
   /* don't need this callout functionality on SFK ...*/
}
#else

WEAK_REF(void ose_wdog(void));

void mm_kick_wdog(void)
{
   void (*my_ptr)(void) = ose_wdog; /* Remove arm-rvct warning */
   if(my_ptr)
   {
      ose_wdog();
   }
}
#endif

/*
 **************************************************************************
 * Help functions for logical addresses.
 **************************************************************************
 */
mm_bool_t
mm_log_overlap(log_addr_t base1, log_size_t size1,
               log_addr_t base2, log_size_t size2)
{
   mm_bool_t result;

   result = !(base1 > (base2 + size2 - 1) || (base1 + size1 - 1) < base2);
   return result;
}


mm_bool_t
mm_log_inside(log_addr_t addr, log_addr_t base, log_size_t size)
{
   mm_bool_t result;

   result = addr >= base && addr <= (base + size - 1);
   return result;
}


mm_bool_t
mm_log_sub_area(log_addr_t sub_base, log_size_t sub_size,
		log_addr_t base, log_size_t size)
{
   return
      mm_log_inside(sub_base, base, size) &&
      mm_log_inside(sub_base + sub_size - 1, base, size);
}


mm_bool_t
mm_log_valid_area(log_addr_t base, log_size_t size)
{
   volatile log_addr_t upper_limit = base + size - 1;
   assert(0 != size);
   return upper_limit >= base;
}


log_addr_t
mm_log_align(log_addr_t addr, log_size_t aligned)
{
   log_addr_t result;

   result = ((addr + (aligned - 1)) & ~(aligned - 1));
   return result;
}



/*
 **************************************************************************
 * Help functions for physical addresses.
 **************************************************************************
 */
mm_bool_t
mm_phys_overlap(phys_addr_t base1, phys_size_t size1,
                phys_addr_t base2, phys_size_t size2)
{
   mm_bool_t result;

   result = !(base1 > (base2 + size2 - 1) || (base1 + size1 - 1) < base2);
   return result;
}


mm_bool_t
mm_phys_inside(phys_addr_t addr, phys_addr_t base, phys_size_t size)
{
   mm_bool_t result;

   result = addr >= base && addr <= (base + size - 1);
   return result;
}


mm_bool_t
mm_phys_sub_area(phys_addr_t sub_base, phys_size_t sub_size,
		 phys_addr_t base,     phys_size_t size)
{
   return
      mm_phys_inside(sub_base, base, size) &&
      mm_phys_inside(sub_base + sub_size - 1, base, size);
}


mm_bool_t
mm_phys_valid_area(phys_addr_t base, phys_size_t size)
{
   volatile phys_addr_t upper_limit = base + size - 1;
   assert(0 != size);
   return upper_limit >= base;
}

phys_addr_t
mm_phys_align(phys_addr_t addr, log_size_t aligned)
{
   phys_addr_t result;

   result = ((addr + (aligned - 1)) & ~((phys_addr_t)aligned - 1));
   return result;
}

log_addr_t
mm_log_truncate(log_addr_t addr, log_size_t align_size)
{
   return addr & ~(align_size -1);
}

phys_addr_t
mm_phys_truncate(phys_addr_t addr, log_size_t aligned)
{
   phys_addr_t result = addr & ~((phys_addr_t)aligned - 1);

   return result;
}

mm_bool_t
mm_log_aligned(log_addr_t addr, log_size_t align_size)
{
   return !(addr & (align_size - 1));
}

mm_bool_t
mm_phys_aligned(phys_addr_t addr, log_size_t align_size)
{
   return !(addr & (align_size - 1));
}


/*
 * Blocking-version of receive_from.
 */
union SIGNAL *
receive_from_pid(PROCESS pid, const SIGSELECT *sel)
{
   assert(NULL != sel);
   {
      union SIGNAL *sig = NIL;

      /* Keep receiving until a signal is really received. */
      while(NIL == sig)
      {
	 sig = receive_from((OSTIME)-1, sel, pid);
      }
      return sig;
   }
}
