/* COPYRIGHT-ENEA-SRC-R2 *
**************************************************************************
* Copyright (C) 2004-2006 by Enea Software AB.
* All rights reserved.
*
* This Software is furnished under a software license agreement and
* may be used only in accordance with the terms of such agreement.
* Any other use or reproduction is prohibited. No title to and
* ownership of the Software is hereby transferred.
*
* PROPRIETARY NOTICE
* This Software consists of confidential information.
* Trade secret law and copyright law protect this Software.
* The above notice of copyright on this Software does not indicate
* any actual or intended publication of such Software.
**************************************************************************
* COPYRIGHT-END */

#ifndef _LOG_OBJECT
#define _LOG_OBJECT

#include "mmtypes.h"
#include "partition.h"

#define MM_LOG_OBJECT_OPT_EXCLUSIVE     MM_PARTITION_OPT_EXCLUSIVE
#define MM_LOG_OBJECT_OPT_PHYSICAL      MM_PARTITION_OPT_PHYSICAL
#define MM_LOG_OBJECT_OPT_RESIDENT      MM_PARTITION_OPT_RESIDENT

#define log_object_exclusive(lobj)  \
   (((lobj)->options & MM_LOG_OBJECT_OPT_EXCLUSIVE) != 0)

#define log_object_physical(lobj)   \
   (((lobj)->options & MM_LOG_OBJECT_OPT_PHYSICAL)  != 0)

#define log_object_resident(lobj)   \
   (((lobj)->options & MM_LOG_OBJECT_OPT_RESIDENT)  != 0)


struct freemem;


/*
 * log_object is a container describing a piece
 * of logical memory acquired from a partition allocator. 
 */
struct log_object
{
   log_addr_t  base;
   log_size_t  size;
   uint32_t    options;
   uint32_t    ref_count;
   struct freemem *freemem;
};


/*
 * Init internal state.
 */
void
mm_log_object_init(void);


/*
 * Create an object.
 *
 * If the logical memory described by the object is allocated
 * from an internal logicial allocator, <partition> must be valid.
 *
 */
struct log_object *
mm_create_log_object(void);

/*
 * Attaches to an object, to prevent its destruction.
 *
 * Any client that intends to use the logical memory
 * described by the object must attach to the object.
 *
 * The object reference counter is increased. 
 */
void
mm_attach_log_object(struct log_object *log_object, struct domain *domain);

/*
 * Detaches from an object, to allow its destruction.
 *
 * Any client that no longer intends to use the logical memory
 * described by the object must detach from the object.
 *
 * The object reference counter is decreased. If the counter
 * reaches zero the logical memory is freed and the object
 * destroyed. The client must not reference an object after it
 * has been detached.
 *
 * It is legal to detach an object even if no client references it.
 */
void
mm_detach_log_object(struct log_object *log_object, struct domain *domain);

/*
 * Locates the partition from which the logical memory was allocated.
 *
 * Note that the returned partition only is guaranteed to be correct
 * if the logical memory is exclusive.
 */
struct partition *
mm_log_object_partition(struct log_object *lobj, struct domain *domain);

#endif /* _LOG_OBJECT */

