/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#include "mm_profile.h"


static OSTIME prof_time;
static mm_bool_t prof_active = FALSE;
static OSTIME tick_len;
static OSTIME prof_time_start;

static OSTIME
profile_get_time(void)
{
   OSTICK ticks, usec;
   ticks = get_systime(&usec);

   return ticks * tick_len + usec;
}

#ifdef POSE_PROFILED
/* These varibels needs to be profiles in pose.c */
/* Details can be found in : */
/* /vobs/ose5/core/core/src/krn/exec/pose.c@@/main/tb_current_ose5/ */
/* tb_mi_ose5_3/tb_task045629/1 */
extern int zzose_idle_cnt;
extern int zzose_int_cnt;
extern int zzose_pri_cnt;
#endif

void
mm_profile_start(const char *func_name,
                 const char *reg_name,
                 const U32 reg_size,
                 const char *profile_size)
{
   extern int ose_mm_segnum_cnt;
   struct allocator *a;
   phys_size_t tot_free = 0, tot_mem = 0;

   if (gmm_state.state != mmu_enabled)
   {
      return;
   }


   if (!tick_len)
   {
      tick_len = system_tick();
   }
   
   prof_time = profile_get_time();
   prof_time_start = prof_time;
   
   for (a = gmm_state.allocators; a != NULL; a = a->next)
   {
      tot_free += a->get_free_mem(a);
      tot_mem  += a->get_total_mem(a);
   }
#ifdef POSE_PROFILED   
   ramlog_printf("MMPROF: --- Starting - %s: '%s' size: %#lx > %s "
                 "show time elapsed.\n"
                 "seg_cnt:%ld free_mem:%#llx tot_mem: %#llx "
                 "idle_cnt:%ld  int_cnt: %ld, pri_cnt: %ld\n",
                 func_name, reg_name, reg_size,  profile_size,
                 ose_mm_segnum_cnt, tot_free, tot_mem,
                 zzose_idle_cnt, zzose_int_cnt,
                 zzose_pri_cnt);
#else
   ramlog_printf("MMPROF: --- Starting - %s: '%s' size: %#lx > %s "
                 "show time elapsed.\n"
                 "        seg_cnt:%ld free_mem:%#llx tot_mem: %#llx\n",
                 func_name, reg_name, reg_size,  profile_size,
                 ose_mm_segnum_cnt, tot_free, tot_mem);
#endif   
   
   prof_active = TRUE;
}

void
mm_profile_checkpoint(const char *str1, const char *str2, const U32 value)
{
   OSTIME now;
      
   if (!prof_active)
   {
      return;
   }   
   
   now = profile_get_time();
   
#ifdef POSE_PROFILED
   ramlog_printf("MMPROF: %20s: %10d us (idle_cnt: %ld, "
                 "int_cnt: %ld pri_cnt: %ld)\n",
                 name, now - prof_time, zzose_idle_cnt,
                 zzose_int_cnt, zzose_pri_cnt);
#else
   ramlog_printf("MMPROF: %20s: %10d us; %s: %ld (%#lx)\n",
                 str1, now - prof_time, str2, value, value);
#endif
      
   prof_time = now;
}

void
mm_profile_stop(mm_status_t status, log_addr_t base)
{
   OSTIME now;

   if (!prof_active)
   {
      return;
   }

   now = profile_get_time();

#ifdef POSE_PROFILED   
   ramlog_printf("MMPROF: --- Done - Elapsed: %ld us (MmStatus: %#lx, "
                 "base:%#08lx "
                 "idle_cnt:%ld  int_cnt: %ld, pri_cnt: %ld)\n",
                 now - prof_time_start,
                 status, base, zzose_idle_cnt,
                 zzose_int_cnt, zzose_pri_cnt);
#else
   ramlog_printf("MMPROF: --- Done - Elapsed: %ld us (MmStatus: %#lx, "
                 "base:%#08lx)\n",
                 now - prof_time_start,
                 status, base);
#endif   
   prof_active = FALSE;
}
