/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef _OBJECT_H
#define _OBJECT_H

/* C runtime include files. */

/* OSE public include files. */
#include "list_ext.h"

/* OSE internal include files. */

/* MM internal include files. */

/* Recursive dependency between allocator.h & phys_object.h */
struct phys_object;
struct fragment_container;

#include "allocator.h"

/* Describes one page. */
struct phys_page
{
   phys_addr_t pa;
   phys_size_t size;
};


/* Constants for struct phys_object.data_representation */
#define PHYS_OBJ_NULL             0x320
#define PHYS_OBJ_FRAG_LIST        0x321
#define PHYS_OBJ_DIRECT_PHYS_FRAG 0x322
#define PHYS_OBJ_CLIENT_PAGES     0x323

/*
** Determines the amount of space saved in the fragment_container struct.
**
** Current levels are:
**  0 - Original (24 bytes)
**  1 - 32 bit address containers (12 bytes)
**  2 - 24 bit container for top address (11 bytes)
*/
#if defined(OSPP)
#define FC_PACK_LEVEL 2
#else
#define FC_PACK_LEVEL 1
#endif

/* The frag_container struct is used to represent a fragment of the memory.
 */

#if FC_PACK_LEVEL == 0

struct fragment_container {
   phys_addr_t packed_base; /* the original (and best?) */
   phys_addr_t packed_top; 
   struct fragment_container *packed_next;
};

#define FC_SET_BASE(fc,base) (fc)->packed_base = base
#define FC_GET_BASE(fc) ((fc)->packed_base & (~0xffULL))
#define FC_SET_TOP(fc,top) (fc)->packed_top = top
#define FC_GET_TOP(fc) (fc)->packed_top
#define FC_GET_SIZE(fc) ((fc)->packed_top + 1ULL - (fc)->packed_base)
#define FC_SET_NEXT(fc,next) (fc)->packed_next = next
#define FC_GET_NEXT(fc) (fc)->packed_next
#define FC_SET_PAGE_INDEX(fc,idx) do { \
                                     assert((idx & (~0xffUL)) == 0); \
                                     (fc)->packed_base = ((fc)->packed_base & (~0xffULL)) + (idx); \
                                  } while (0) 
#define FC_GET_PAGE_INDEX(fc) ((fc)->packed_base & (0xffULL))

#elif FC_PACK_LEVEL == 1

struct fragment_container {
   uint32_t packed_base_pn; /* phys_addr_t shifted 4 bits. (low 8 page nr) */
   uint32_t packed_top;     /* phys_addr_t + 1 shifted 4 bits. low 8 unused. */
   struct fragment_container *packed_next;
};

#define FC_WRAP(val) ((val) % 0x1000000000ULL)
#define FC_SET_BASE(fc,base) do { \
                                 assert(((base) & 0xfffffff000000fffULL) == 0); \
                                 (fc)->packed_base_pn = (uint32_t)((base) >> 4); \
                             } while (0)
#define FC_GET_BASE(fc) ((phys_addr_t)((fc)->packed_base_pn & (~0xffUL)) << 4)
#define FC_SET_TOP(fc,top) do { \
                                 assert((FC_WRAP((top) + 1) & 0xfffffff000000fffULL) == 0); \
                                 (fc)->packed_top = (uint32_t)(FC_WRAP((top) + 1) >> 4); \
                           } while (0)
#define FC_GET_TOP(fc) (((phys_addr_t)(fc)->packed_top << 4) - 1)
#define FC_GET_SIZE(fc) ((phys_addr_t)((fc)->packed_top - ((fc)->packed_base_pn & (~0xffUL))) << 4)
#define FC_SET_NEXT(fc,next) (fc)->packed_next = (next)
#define FC_GET_NEXT(fc) (fc)->packed_next
#define FC_SET_PAGE_INDEX(fc,idx) do { \
                                     assert((idx & (~0xffUL)) == 0); \
                                     (fc)->packed_base_pn = ((fc)->packed_base_pn & (~0xffUL)) + (idx); \
                                  } while (0) 
#define FC_GET_PAGE_INDEX(fc) ((fc)->packed_base_pn & (0xffUL))

#elif FC_PACK_LEVEL == 2

struct fragment_container {
   uint8_t X[11];  /* 0-2  = base, phys_addr_t shifted 4 bits
                    *   3  = page_index 
                    * 4-6  = top, phys_addr_t shifted 4 bits
                    * 7-10 = next, struct fragment_container * */
};

#define FC_WRAP(val) ((val) % 0x1000000000ULL)
#define FC_SET_BASE(fc,b) do { \
                             assert(((b) & 0xfffffff000000fffULL) == 0); \
                             (fc)->X[0] = ((b) >> (4 + 8)) & 0xff; \
                             (fc)->X[1] = ((b) >> (4 + 16)) & 0xff; \
                             (fc)->X[2] = ((b) >> (4 + 24)) & 0xff; \
                          } while (0)
#define FC_GET_BASE(fc) (phys_addr_t)( \
                           ((fc)->X[0] << (4 + 8)) + \
                           ((fc)->X[1] << (4 + 16)) + \
                           ((uint64_t)(fc)->X[2] << (4 + 24)) \
                        )
#define FC_SET_TOP(fc,t) do { \
                            assert((FC_WRAP((t) + 1) & 0xfffffff000000fffULL) == 0); \
                            (fc)->X[4] = (FC_WRAP((t) + 1) >> (4 + 8)) & 0xff; \
                            (fc)->X[5] = (FC_WRAP((t) + 1) >> (4 + 16)) & 0xff; \
                            (fc)->X[6] = (FC_WRAP((t) + 1) >> (4 + 24)) & 0xff; \
                         } while (0)
#define FC_GET_TOP(fc) (phys_addr_t)(FC_WRAP( \
                          (((fc)->X[4] << (4 + 8)) + \
                           ((fc)->X[5] << (4 + 16)) + \
                           ((uint64_t)(fc)->X[6] << (4 + 24))) - 1ULL \
                       ))
#define FC_GET_SIZE(fc) (FC_GET_TOP(fc) + 1ULL - FC_GET_BASE(fc))
#define FC_SET_NEXT(fc,n) do { \
                             (fc)->X[7] = ((uint32_t)(n)) & 0xff; \
                             (fc)->X[8] = ((uint32_t)(n) >> 8) & 0xff; \
                             (fc)->X[9] = ((uint32_t)(n) >> 16) & 0xff; \
                             (fc)->X[10] = ((uint32_t)(n) >> 24) & 0xff; \
                          } while (0)
#define FC_GET_NEXT(fc) (struct fragment_container *)( \
                            ((fc)->X[7]) + \
                            ((fc)->X[8] << 8) + \
                            ((fc)->X[9] << 16) + \
                            ((fc)->X[10] << 24) \
                        )
/*#define FC_SET_NEXT(fc,n) *(struct fragment_container **)&(fc)->X[7] = (n)
  #define FC_GET_NEXT(fc) *(struct fragment_container **)&(fc)->X[7] */
#define FC_SET_PAGE_INDEX(fc,idx) do { \
                                     assert((idx & (~0xffUL)) == 0); \
                                     (fc)->X[3] = (idx); \
                                  } while (0) 
#define FC_GET_PAGE_INDEX(fc) (fc->X[3])

#else
#error Illegal pack level..
#endif /* FC_PACK_LEVEL */

/*
 * Data type used when traversing a 'struct phys_container' object.
 */
struct phys_mem_cursor
{
   const struct phys_object *phys_obj;

   /* Get pa and size here. */
   struct phys_page phys_page;
   
   union {
      struct fragment_container *current_frag;
      U32 index;
   } data;
};

/*
 * Container for allocated physical memory and associated allocator.
 * Also contains a reference counter.
 */
struct phys_object
{
   /* Number of instance regions that reference this object. */
   unsigned ref_count;
   
   /*
    * What allocator handles this?
    * <allocator> != NULL -> internal allocator
    * <allocator> == NULL -> client supplied allocator
    */
   struct allocator *allocator;
   
   U32 data_representation;

   U32 meta_options;
   
   union {
      struct fragment_container *frag_list;
      struct phys_page direct_phys_frag;
      struct mm_create_region *client_pages;
   } data;
};

/*
 * Init internal state.
 */
void
mm_phys_object_init(void);


/*
 * Create an object.
 * If the physical memory is allocated from an internal allocator,
 * <allocator> must be a valid.
 * If the physical memory is supplied by the client, <allocator> must
 * equal NULL.
 *
 */
struct phys_object *
mm_create_phys_object(struct allocator *allocator, U32 meta_options);

/*
 * Attach to an object.
 * Increases the reference counter of an object to
 * prevent the physical resources from being freed. 
 */
void
mm_attach_phys_object(struct phys_object *object);

/*
 * Detach from an object.
 * If the number of references drops to zero, the physical memory
 * is freed (with allocator->free if allocator != NULL) and <object> is freed.
 * Otherwise the reference counter is decremented.
 */
void
mm_detach_phys_object(struct phys_object *object);


/*
 * Lookup the physical address for a logical address.
 * The logical address is passed as an offset within a region.
 * If <log_addr> is mapped to physical memory, the physical address
 * is returned.
 * The start address of the physical fragment is also returned.
 * If <log_addr> is not not mapped to physical memory, FALSE is returned.
  */
mm_bool_t
mm_extract_phys_addr(const struct phys_object *object,
                     phys_size_t offset,
                     phys_addr_t *phys_addr,
                     phys_addr_t *frag_start,
                     phys_size_t *frag_len);

/*
 * Description:
 *   Start a traversel of a phys_mem object <phys_mem>.
 *
 * Precondition:
 *   The client must allocate a cursor object <cursor> which is
 *   initialised by this call.
 *
 * Postcondition:
 *   <cursor> object initialized.
 */
void
mm_phys_mem_get_first(const struct phys_object *phys_obj,
                      struct phys_mem_cursor *cursor);

/*
 * Description:
 *   Get next fragment from a of phys_mem object <phys_mem>.
 *
 * Precondition:
 *   The cursor object <cursor> must have been initialised.
 *
 * Postcondition:
 *   not last fragment -> return value = 1 and <cursor> updated with
 *     next fragment.
 *   last fragment -> return value = 0.
 */
int
mm_phys_mem_get_next(struct phys_mem_cursor *cursor);

void print_phys_object(struct phys_object *p);

#endif /* _OBJECT_H */
