/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef _COMMON_H
#define _COMMON_H

#include "stddef.h"

#include "ose.h"
#include "ose_err.h"
#include "ramlog.h"
#include "osetypes.h"

#include "mm.sig"
#include "mm_err.h"


void mm_kick_wdog(void);

#define RAM_NAME                  "RAM"
#define RAM_SASE_NAME             "RAM_SASE"

#define MAX(a,b)          (((a) > (b)) ? (a) : (b))
#define MIN(a,b)          (((a) < (b)) ? (a) : (b))

#define INTERNAL_ERROR() MM_FATAL_ERROR2(("MM internal error.\n"),\
                                         (OSERRCODE)OSE_MM_EINTERNAL_ERROR,\
                                         (OSERRCODE)0)


extern void zzkrn_halted_hook(void);
#define MM_FATAL_ERROR(print_string, ecode)\
  do {\
        (void)ramlog_printf("mm: ");\
        (void)ramlog_printf print_string;\
        error(OSE_EFATAL_MASK | (ecode));\
        zzkrn_halted_hook();\
     } while (0)

#define MM_FATAL_ERROR2(print_string, ecode, extra)\
  do {\
        (void)ramlog_printf("mm: ");\
        (void)ramlog_printf print_string;\
        /* The line parameter in zzerror2() is long for some reason. */ \
        /*lint -save -e747 */ \
        error2((OSE_EFATAL_MASK | (ecode)), (extra)); \
        /*lint -restore */ \
        zzkrn_halted_hook();\
        /*lint -unreachable */\
  } while (0)


#define OVERLAP(b1, s1, b2, s2) \
  mm_log_overlap((log_addr_t)b1, s1, (log_addr_t)b2, s2)

#define ALIGNED(e, s) (!((e) & ((s) - 1)))

#define SUB_AREA(sub_base, sub_size, base, size) \
   mm_phys_sub_area((phys_addr_t)sub_base, sub_size,\
                    (phys_addr_t)base, size)

/* 64-bit safe versions of the formerly used macros
#define ALIGN(a,size)        (((a) + (size) - 1) & ~((size) - 1))
#define ALIGN_DOWN(a,size)   ((a) & ~((size) - 1))
*/
#define ALIGN(a,size)        (((a) + (size) - 1) - (((a) + (size) - 1) & ((size) - 1)))
#define ALIGN_DOWN(a,size)   ((a) - ((a) & (size - 1)))

/* Same as ALIGN, but with a slightly more informative name */
#define ALIGN_UP(a,size)     ALIGN(a,size)

#define TRACE ramlog_printf("<< %s:%d >>\n",__FILE__,__LINE__)

/*
 **************************************************************************
 * Help functions for logical addresses.
 **************************************************************************
 */

/*
 * True if the two areas <base1, size1> and <base2, size2> operlaps.
 */
mm_bool_t
mm_log_overlap(log_addr_t base1, log_size_t size1,
               log_addr_t base2, log_size_t size2);


/*
 * True if address <addr> is inside the area <base, size>.
 */
mm_bool_t
mm_log_inside(log_addr_t addr, log_addr_t base, log_size_t size);

/*
 * True iff the area designated by (<sub_base>, <sub_size>) is completely
 * contained within the area designated by (<base>, <size>).
 */
mm_bool_t
mm_log_sub_area(log_addr_t sub_base, log_size_t sub_size,
		log_addr_t base, log_size_t size);

/*
 * True if the area <base, size> does not wrap around.
 */
mm_bool_t
mm_log_valid_area(log_addr_t base, log_size_t size);

/*
 * Returns the given address aligned to the given alignment.
 */
log_addr_t
mm_log_align(log_addr_t addr, log_size_t alignend);



/*
 **************************************************************************
 * Help functions for physical addresses.
 **************************************************************************
 */

/*
 * True if the two areas <base1, size1> and <base2, size2> operlaps.
 */
mm_bool_t
mm_phys_overlap(phys_addr_t base1, phys_size_t size1,
                phys_addr_t base2, phys_size_t size2);

/*
 * True if address <addr> is inside the area <base, size>.
 */
mm_bool_t
mm_phys_inside(phys_addr_t addr, phys_addr_t base, phys_size_t size);

/*
 * True iff the area designated by (<sub_base>, <sub_size>) is completely
 * contained within the area designated by (<base>, <size>).
 */
mm_bool_t
mm_phys_sub_area(phys_addr_t sub_base, phys_size_t sub_size,
		 phys_addr_t base, phys_size_t size);


/*
 * True if the area <base, size> does not wrap around.
 */
mm_bool_t
mm_phys_valid_area(phys_addr_t base, phys_size_t size);

/*
 * Returns the given address aligned upwards to the given alignment.
 */
phys_addr_t
mm_phys_align(phys_addr_t addr, log_size_t alignend);

/*
 * Returns the given address aligned downwards/truncated to the
 * given alignment.
 */
phys_addr_t
mm_phys_truncate(phys_addr_t addr, log_size_t alignend);

/*
 * Returns the given address aligned downwards/truncated to the
 * given alignment.
 */
log_addr_t
mm_log_truncate(log_addr_t addr, log_size_t alignend);

/*
 * Returns tru if the given logical address is aligned according
 * to the given alignment.
 */
mm_bool_t
mm_log_aligned(log_addr_t addr, log_size_t align_size);

/*
 * Returns tru if the given physical address is aligned according
 * to the given alignment.
 */
mm_bool_t
mm_phys_aligned(phys_addr_t addr, log_size_t align_size);


/*
 * Blocking-version of receive_from.
 */
union SIGNAL *
receive_from_pid(PROCESS pid, const SIGSELECT *sel);

#endif /* _REGION_H */
