/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef _HAL_H
#define _HAL_H

#include "osetypes.h"
#include "mmtypes.h"
#include "stddef.h"
#include "krn_internal.h"
#include "mm_debug.h"
#include "ose.h"
#include "mm.sig"

/* Macros for MM counters.
 * To save some memory (i.e. avoids option_id field for each static counter
 * let counter tag_options coexist with tag_id in sw counters
 * definitions in the MMUHAL in the following way:
 * 
 * Hi  4 bytes  tag_options (counter characteristic)
 * Low 4 bytes: tag_id
 *
 * */
#define MM_SHIFT_TAG_OPT    16   
#define MM_INFOTAG_MASK     0xFFFF0000


/**
 * MM_TAG Option bits > 0x800 reserved for OSE internal used.
 * mm_tag_ext->tag_option.
 */
/* At least one hw couinter is enabled, keep smp gmm agents on other cores. */
#define MM_TAG_SOME_HW_COUNTER_ENABLED 0x1000

#ifdef OSMIPS
#  define MIN_PAGE_SHIFT   13
#else
#  define MIN_PAGE_SHIFT   12
#endif
#define MIN_PAGE_SIZE    (1UL << MIN_PAGE_SHIFT)
#define MAX_ORDER        20

#define HAL_LOG_MEM_NAME          "HAL"
#define HAL_LOG_MEM_SIZE          (32 * HAL_SIZE_1M)



/*
 * Commonly used sizes.
 */
#define HAL_SIZE_1K                  (1024)
#define HAL_SIZE_4K                (4*1024)
#define HAL_SIZE_8K                (8*1024)
#define HAL_SIZE_16K              (16*1024)
#define HAL_SIZE_32K              (32*1024)
#define HAL_SIZE_64K              (64*1024)
#define HAL_SIZE_128K            (128*1024)
#define HAL_SIZE_256K            (256*1024)
#define HAL_SIZE_512K            (512*1024)
#define HAL_SIZE_1M             (1024*1024)
#define HAL_SIZE_2M           (2*1024*1024)
#define HAL_SIZE_4M           (4*1024*1024)
#define HAL_SIZE_8M           (8*1024*1024)
#define HAL_SIZE_16M         (16*1024*1024)
#define HAL_SIZE_32M         (32*1024*1024)
#define HAL_SIZE_64M         (64*1024*1024)
#define HAL_SIZE_128M       (128*1024*1024)
#define HAL_SIZE_256M       (256*1024*1024)
#define HAL_SIZE_512M       (512*1024*1024)
#define HAL_SIZE_1G        (1024*1024*1024)
#define HAL_SIZE_4G   (4*1024*1024*1024ULL)



/*
 * Option constants.
 */
#define HAL_SHARE                    0x00000001
#define HAL_MAS                      0x00000002
#define HAL_CHANGE                   0x00000004
#define HAL_GLOBAL_MAPPING           0x00000008
#define HAL_SKIP_INVALIDATE_TLB      0x00000010
#define HAL_MANAGER_MODE             0x00000020
#define HAL_CONTENTS_TYPE_DATA       0x00000040
#define HAL_CONTENTS_TYPE_TEXT       0x00000080
#define HAL_RESIDENT                 0x00000100
#define HAL_APPEND                   0x00000200
#define HAL_GLOBAL_MAPPING_TRANSIENT 0x00000400




/*
 * Return values from HAL-functions.
 */
typedef enum hal_status
{
   HAL_SUCCESS                           = 0x00,
   HAL_EEND_OF_PHYS_MEMORY               = 0x01,
   HAL_EEND_OF_LOG_MEMORY                = 0x02,
   HAL_EUNSUPPORTED_MMU                  = 0x03,
   HAL_EINTERNAL_POOL_ALLOCATION_FAILED  = 0x04,
   HAL_EINTERNAL_ERROR                   = 0x05, /* HAL private error codes. */
   HAL_ELOOKUP_FAILED                    = 0x06,
   HAL_EMAPPING_EXISTS                   = 0x07,
   HAL_EOUT_OF_RESOURCES                 = 0x08,
   HAL_EINVALID_SIZE                     = 0x09,
   HAL_EINVALID_ALIGNMENT                = 0x0A,
   HAL_EUNSUPPORTED_PHYSICAL_RANGE       = 0x0B,
   HAL_EUNSUPPORTED_TAG                  = 0x0C,
   HAL_EILLEGAL_PERMISSION               = 0x0D,
   HAL_EILLEGAL_CACHE_MODE               = 0x0E,
   HAL_ENOT_FOUND                        = 0x0F,
   HAL_EINFOTAG_ONLY_GLOBAL_COUNTER      = 0x10,
   HAL_EINFOTAG_ONLY_LOCAL_COUNTER       = 0x11
} hal_status_t;


/**************************************************************************/

#ifdef OSX86
#define HAL_X86_CACHE_PCD            0x00000010  /* Inhibit       */
#define HAL_X86_CACHE_PWT            0x00000020  /* Write-through */
#define HAL_X86_CACHE(bits, level)   ((bits) << ((level-1)*4))
#endif

#ifdef OSARM
#define HAL_ARM_MEM_ORDERED          0x00000001  /* Strongly ordered mem. */
#define HAL_ARM_MEM_DEVICE           0x00000002  /* Device / ordered mem  */
#define HAL_ARM_MEM_NORMAL           0x00000004  /* Speculative memory    */
#define HAL_ARM_MEM_SHARED           0x00000008  /* Coherent memory.      */

#define HAL_ARM_CACHE_CI             0x00000010  /* Inhibit               */
#define HAL_ARM_CACHE_WT             0x00000020  /* Write-through         */
#define HAL_ARM_CACHE_WB             0x00000040  /* Write-back            */
#define HAL_ARM_CACHE_WA             0x00000080  /* Write-allocate        */
#define HAL_ARM_CACHE(bits, level)   ((bits) << ((level-1)*4))
#endif

#ifdef OSPP
#define HAL_PPC_MEM_ENDIAN           0x00000001  /* Little endian.        */
#define HAL_PPC_MEM_GUARDED          0x00000002  /* Guarded/Ordered mem   */
#define HAL_PPC_MEM_COHERENT         0x00000004  /* Coherent memory.      */
#define HAL_PPC_CACHE_CI             0x00000008  /* Cache inhibit         */
#define HAL_PPC_CACHE_WT             0x00000010  /* Write-through.        */
#define HAL_PPC_CACHE(bits, level)   ((bits) << ((level-1)*4))
#endif

#ifdef OSMIPS
#define HAL_MIPS_CACHE_ENABLE        0x00000001  /* Cacheable             */
#define HAL_MIPS_CACHE(bits, level)  ((bits) << (level*4))
#endif

/**************************************************************************/

/*
 * Describes a cache type configuration. 
 *
 *   name  - The name of the cache type as read from configuration. 
 *   cache - Architecture specific bits that are used during map().
 *   index - Index of the cache type - used in cache_mode bitfield.
 */
struct hal_cache_type
{
   char name[MM_NAME_LENGTH+1];
   U32  cache;
   U32  index;
};

#define HAL_CTYPE_COUNT 16

/**************************************************************************/

/*
 * Describes a memory type configuration. 
 *
 *   name  - The name of the memory type as read from configuration. 
 *   cache - Architecture specific bits that are used during map().
 *   index - Index of the memory type - used in cache_mode bitfield.
 */
struct hal_memory_type
{
   char name[MM_NAME_LENGTH+1];
   U32  memory;
   U32  index;
};

#define HAL_MTYPE_COUNT 16

/**************************************************************************/



/* We are still here... */
struct mem_state;
struct partition;


/*
 * Contains resources for a HAL domain.
 * The actual definition is architecture dependant.
 */
struct hal_domain;

/*
 * Basic information about the HAL module.
 */
struct hal_info
{
      /*
       * The HAL module uses an MMU.
       * A TRUE value implies that:
       *    - logical memory will be allocated from a HAL log_mem
       *    - physical pages will be allocated from an internal API.
       *    - the MMU will be enabled after hal_init_2()
       */
      mm_bool_t has_mmu;

      /*
       * A TRUE value implies that:
       *    - the MMU will be enabled to map ram memory
       *      before exec needs to install its exception vectors.
       *   (Default is TRUE for ARM, FALSE for PPC.)
       */
      mm_bool_t enable_mmu_early;

      /* If MMU present and used, is it enabled. */
      mm_bool_t enabled_mmu;

      mm_bool_t has_mas;

      /*
       * The number of supported page sizes.
       * The first value designates the minimum page size, used for lots
       * of address calculations in GMM.
       * All HAL modules must have at least a minimum size, even if
       * <has_mmu> is false
       */
      unsigned no_page_sizes;

      /*
       * Pointer to array with page sizes.
       * The first element is the minimum page size.
       */
      const log_size_t *page_sizes;

      /*
      * The number of supported resident page sizes.
      * May be 0 if resident support is unavailable. 
      */
      unsigned no_resident_page_sizes;

      /*
       * Pointer to array with resident page sizes.
       * The first element is the mininum page size.
       * May be NULL if resident support is unavailable.
       */
      const log_size_t *resident_page_sizes;

      /*
       * The number of supported tag id's.
       */
       unsigned no_tag_id;
   
      /*
       * Pointer to array with supported MmTagIds.
       */
       U32 *tag_id_array;
};


/*
 * Contains the arch cpu specific cache functions.
 * Global variable used by cache.c for both mmu/no_mmu.
 */
extern struct arch_cpu_descriptor *hal_arch_cpu_descriptor;



/*
 * Description:
 *    First step in initialisation of a MM HAL module. At least two
 *    things must be performed here:
 *
 *    1  save the pointer to current CPU HAL (<arch_cpu_descriptor>)
 *    2  make the area [<vector_base>, <vector_base> + <vector_size> -
 *       1] writable from supervisor-mode.
 *
 *    Since EXEC has not yet installed it's exception handlers, it is
 *    no use installing exception handlers in HAL since they will
 *    eventually be overwritten. Hence if the MMU is enabled, any MMU
 *    related exceptions can not be handled and will be fatal.
 *
 *    hal_get_info() is operational after this call.
 *
 * Return value:
 *    <status>
 *
 * Precondition:
 * Postcondition:
 *    + the area [<vector_base>, <vector_base> + <vector_size> - 1] is
 *      writable from supervisor-mode
 */

extern hal_status_t (*hal_init_1)(const struct cpu_descriptor *cpu_descriptor,
                                  log_addr_t vector_base,
                                  log_size_t vector_size,
                                  mm_bool_t enable_d_cache,
                                  mm_bool_t enable_i_cache);

/*
 * Description:
 *    Second step in initialisation of a MM HAL module.
 *    HAL modules that need a single large piece of physical memory
 *    may allocate it from this function.
 *
 * Return value:
 *    -
 *
 * Precondition:
 *    + hal_init_1() has been executed
 *    + caches configured
 * Postcondition:
 */
extern hal_status_t (*hal_init_2)(void);



/*
 * Description:
 *    Third step in initialisation of a MM HAL module.
 *    The following calls are operational after this call:
 *       - hal_create_domain()
 *       - hal_delete_domain()
 *       - hal_map_page()
 *       - hal_unmap_range()
 *
 * Return value:
 *    -
 *
 * Precondition:
 *    + mm_malloc()/mm_free() must be operational
 *    + hal_init_2() has been executed
 *
 * Postcondition:
 *    + the system HAL domain exists
 */
extern hal_status_t (*hal_init_3)(struct partition *partition);


/*
 * Description:
 *    Fourth step in initialisation of a MM HAL module.
 *    Possibly enable the MMU.
 *
 * Return value:
 *    -
 *
 * Precondition:
 *    + hal_init_3() has been executed
 *    + The automatic regions have been created.
 *
 * Postcondition: (optional)
 *    + The MMU is enabled with exception area writeable.
 */
extern void (*hal_init_4)(void);

/*
 * Description:
 *    Fourth step in initialisation of a MM HAL module.
 *    - Install the MM exception handlers.
 *    - If not done in hal_init_3 enable the MMU.
 *    - if MMU enabled in hal_init_3 the exception area should now be write protected.
 *
 * Return value:
 *    -
 *
 * Precondition:
 *    + hal_init_4() has been executed
 *    + exception system initialised by EXEC
 *
 * Postcondition:
 *    + The MMU must be enabled
 *    * The MM exception handlers installed.
 *    * The exception area write protected.
 */
extern void (*hal_init_5)(void);

/*
 * Description:
 *    Checks that the HAL can handle the range specified by base & size. The
 *    function is used during startup to detect if the user has configured
 *    physical memory at a location outside the capability of the HAL. If it's
 *    not, the function should not return.
 *    The pointer will be overwritten by HALs that handles larger areas. 
 *    If the HAL does not provide such a function, the pointer shall be set 
 *    to NULL.
 *
 * Return value:
 *    HAL_SUCCESS if the range is ok. Any other value indicates that the range
 *    can not be handled by the HAL.
 *
 * Precondition:
 *    -
 *
 * Postcondition:
 *    -
 */
extern hal_status_t (*hal_validate_phys_range)(phys_addr_t base,
                                               phys_size_t size);

/*
 * Description:
 *    This is the default function for validation of physical range. 
 *    It checks that the range lies within the 32 bit space.
 *
 *   base        - physical base address of the range.
 *   size        - size of the range.
 *
 * Return value:
 *    HAL_SUCCESS or HAL_EUNSUPPORTED_PHYSICAL_RANGE
 */
extern hal_status_t hal_validate_phys_range_32bit(phys_addr_t base,
                                                  phys_size_t size);

/*
 * Description:
 *    Retrieve information about the HAL module.
 *
 * Return value:
 *    <pointer>
 *
 * Precondition:
 *    + hal_init_2() has been executed
 *
 * Postcondition:
 *    -
 */
extern const struct hal_info * (*hal_get_info)(void);


/*
 * Description:
 *    Round requested access permissions <requested_permissions>
 *    to access permissions supported by the hardware
 *    for the logical area in wich the base address is located.
 *
 * Return value:
 *    <status>
 *
 * Precondition:
 *    + hal_init_1() has been executed
 *
 * Postcondition:
 *    + <*actual_permissions> is updated with the permissions
 *      supported by the hardware for the logical area
 *      in wich the base address is located.
 */
extern hal_status_t
(*hal_actual_permissions)(U32 *actual_permissions,
                          U32 requested_permissions,
                          log_addr_t base, U32 options);


/*
 * Description:
 *    Round requested cache mode attributes <requested_cache_mode>
 *    to the cache-mode attributes supported by the hardware for the
 *    logical area in wich the base address is located.
 *
 * Return value:
 *    <status>
 *
 * Precondition:
 *    + hal_init_1() has been executed
 *
 * Postcondition:
 *    + <*actual_cache_mode> is updated with the cache-mode
 *      supported by the hardware for the logical area
 *      in wich the base address is located.
 */
extern hal_status_t
(*hal_actual_cache_mode)(U32 *actual_cache_mode,
                         U32 requested_cache_mode,
                         log_addr_t base);


/*
 * Description:
 *    Register a cache type configuration that controls the behavior
 *    of the CPU caches for a given memory region. This function may
 *    be called repeatedly to add behavior for each supported cache
 *    level, i.e. inner/outer. Predefined cache types, such as
 *    "inhibited" and "copy_back" are also added through this function.
 *    A cache type can be reconfigured by using the same name identifier.
 *
 *    Generic implementation. 
 *    
 * Parameters:
 *    name     - String identifier for the cache type.
 *               Max length: MM_NAME_LENGTH.
 *
 *    cache    - Architecture specific bits to control the
 *               behavior of the cache. See definitions above:
 *                  HAL_PPC_CACHE_*
 *                  HAL_ARM_CACHE_*
 *                  HAL_MIPS_CACHE_*
 *                  HAL_X86_CACHE_*
 *
 *    options  - Available options:
 *
 *               HAL_APPEND
 *               Set this option when appending additional cache level
 *               configurations for an already registered cache type.
 *                  
 * Return value:
 *    <status>
 */
extern hal_status_t
(*hal_add_ctype)(const char *name, U32 cache, U32 options);

/*
 * Description:
 *    Register a memory type configuration that controls the behavior
 *    of memory accesses for a given memory region. Predefined memory
 *    types, such as "ordered_access" and "speculative_access" are also
 *    added through this function. A memory type can be reconfigured by
 *    using the same name identifier. 
 *
 *    Generic implementation. 
 *
 * Parameters:
 *    name     - String identifier for the cache type.
 *               Max length: MM_NAME_LENGTH.
 *
 *    memory   - Architecture specific bits to control the
 *               behavior of the memory. See definitions above:
 *                  HAL_PPC_MEM_*
 *                  HAL_ARM_MEM_*
 *                  HAL_MIPS_MEM_*
 *                  HAL_X86_MEM_*
 *
 * Return value:
 *    <status>
 */
extern hal_status_t
(*hal_add_mtype)(const char *name, U32 memory);

/*
 * Description:
 *    Resolves the names of cache types and memory types into a binary
 *    cache mode bitfield that can be used when creating regions.
 *
 * Parameters:
 *    cache      - Cache type identifier. Mandatory parameter.
 *    mem        - Memory type identifier. Mandatory parameter.
 *    cache_mode - Output. The resolved cache_mode bitfield. 
 *    
 * Return value:
 *    <status>
 */
extern hal_status_t
(*hal_get_cache_mode)(const char *cache, const char *mem, U32 *cache_mode);

/*
 * Description:
 *    Decompiles the names of the cache and memory types from a binary
 *    cache mode bitfield.
 *
 * Parameters:
 *    cache_mode - Input. The cache_mode bitfield to decompile. 
 *    cache      - Cache type identifier. Mandatory parameter.
 *    mem        - Memory type identifier. Mandatory parameter.
 *    
 * Return value:
 *    <status>
 */
extern hal_status_t
(*hal_get_cache_type)(U32 cache_mode, char *cache, char *mem);

/*
 * Description:
 *    Resolves a binary cache mode bitfield into architecture specific
 *    cache and memory bits that are used when mapping pages in the HAL. 
 *
 * Parameters:
 *    cache_mode  - Cache mode bitfield to resolve.
 *    bits        - Output. Architecture specific cache and memory bits.
 *    
 * Return value:
 *    <status>
 */
extern hal_status_t
(*hal_get_cmbits)(U32 cache_mode, U32 *bits);


/*
 * Description:
 *    Translates architecture specific cache and memory bits to
 *    legacy mm.sig cache and access bits. For example:
 *
 *    HAL_PPC_CACHE_WB -> MM_CACHE_COPY_BACK.
 *
 * Paramters:
 *    cmbits     - Architecture specific bits.
 *    cache_mode - Output. The cache mode field into which mm.sig
 *                 options will be or:ed in. 
 *
 * Return value:
 *    <status>
 */
extern hal_status_t
(*hal_legacy_cmbits)(U32 cmbits, U32 *cache_mode);

/*
 * Description:
 *    Validates that the combination of a set of architecture
 *    specific bits is legal. Typically called to verify the system
 *    configuration or combinations of cache and memory when creating
 *    memory regions. All parameters except for 'base' are optional.
 *
 * Parameters:
 *    cbits   - Architecture specific cache bits only. (config)
 *    mbits   - Architecture specific memory bits only. (config)
 *    cmbits  - Or:ed architecture specific cache and memory bits.
 *              (region)
 *
 * Return value:
 *    <status>
 */
extern hal_status_t
(*hal_valid_cmbits)(U32 *cbits, U32 *mbits, U32 *cmbits, log_addr_t base);


/*
 * Description:
 *    Create a HAL domain.
 *
 * Return value:
 *    <status>
 *
 * Precondition:
 *    + hal_init_2() has been executed
 *    + Enough logical and physical memory exists
 *
 * Postcondition:
 *    + All regions (except MAS) are accessible from supervisor-mode
 *      context in the domain
 *    + No regions are accessible from user-mode context in the domain
 *    + <*domain> is updated with a pointer to the HAL domain
 */
extern hal_status_t (*hal_create_domain)(unsigned int seg_id,
                                         mm_bool_t system_domain,
                                         struct hal_domain **domain);


/*
 * Description:
 *    Delete a HAL domain.
 *
 * Return value:
 *    -
 *
 * Precondition:
 *    + hal_init_2() has been executed
 *    + <domain> exists
 *
 * Postcondition:
 *    + All resources of the HAL domain are reclaimed.
 */
extern void (*hal_delete_domain)(unsigned int seg_id,
                                 struct hal_domain *domain);


/*
 * Description:
 *    Switch to domain <hal_domain>.
 *
 * Return value:
 *    -
 *
 * Precondition:
 *    + hal_init_2() has been executed
 *    -
 *
 * Postcondition:
 *    + the domain represented by <hal_domain> is made the current
 *      domain.
 */
extern void (*hal_select_domain)(struct hal_domain *hal_domain);


/*
 * Description:
 *    Map a page in the logical address space.
 *
 * Description of parameters:
 *
 *   domain      - The domain where the mapping should take place. Other
 *                 domains may be affected as well.This depends on the values
 *                 of share and mas.
 *   la          - Logical address for the page.
 *   pa          - Physical address for the page.
 *   size        - Size of the page.
 *   options     - Described below.
 *   access_attr - Permissions for the mapping. The full permissions is only
 *                 given to the specified domains. Other domains may only have
 *                 the corresponding supervisor only permissions.
 *                 These permissions must have been translated to a permission
 *                 that the HAL implementation supports in a call to
 *                 hal_actual_permissions prior to calling hal_map_page.
 *   mem_attr    - GMM cache representation as specified in mm.sig.
 *                 The HAL should try to fulfill this cache request, but may
 *                 ignore it, if it cannot be done.
 *
 *    The caller of this function passes additional information in the
 *    <options> parameter:
 *
 *       HAL_SHARE          Set when <la> already exist in another domain.
 *                          The <domain> in this call wants to share the
 *                          mapping with user mode access.
 *
 *       HAL_MAS             the page is mapped at the same <la> in
 *                           several domains with different <pa>
 *
 *       HAL_CHANGE          <la> is already mapped in
 *                           <domain>. Change <pa> and/or
 *                           <access_attr>.
 *
 * Return value:
 *    <status>
 *
 * Precondition:
 *    + hal_init_2() has been executed
 *    + <domain> exists
 *    + <size> must be a supported page size
 *    + <la> must be a multiple of <size>
 *    + <pa> must be a multiple of <size>
 *    + supervisor-mode access >= user-mode access
 *
 * Postcondition:
 *    + logical and/or physical memory resources may have been
 *      allocated
 *    + A page mapping is established/changed
 */
extern hal_status_t (*hal_map_page)(struct hal_domain *domain,
                                    log_addr_t la,
                                    phys_addr_t pa,
                                    log_size_t size,
                                    int options,
                                    U32 access_attr,
                                    U32 mem_attr);


/*
 * Description:
 *    Unmap the range [<la>, <la> + <size> - 1] from <domain>.
 *    This operation may result in memory resources being reclaimed.
 *    The function should be able to perform unmap of a partially
 *    mapped range. I.e. rollback in fault situation.
 *
 *
 *   domain      - The domain where the unmapping should take place. Other
 *                 domains may be affected as well.This depends on the values
 *                 of share and mas.
 *   la          - Logical base address of the range.
 *   size        - Size of the range.
 *   options     - Described below.
 *
 *   The following three attributes are needed for arm but maybe not for ppc.
 *   The reason for arm is that a unmap in share case leads not unmap but
 *   to map with supervision access only.
 *
 *   access_attr - Permission.
 *   mem_attr    - GMM cache representation.
 *   phys_mem     Array of physical fragments that will be unmapped from
 *                [la, la + size).
 *
 *    The caller of this function passes additional information in the
 *    <options> parameter:
 *
 *       HAL_SHARE          Set when <la> exist in other domain's with
 *                          user mode mapping . I.e. <la> is shared.
 *
 *       HAL_MAS             The page is mapped at the same <la> in
 *                           several domains with different <pa>.
 *
 *       HAL_CHANGE          Not used for unmapping.
 *
 *
 * Return value:
 *    -
 *
 * Precondition:
 *    + hal_init_2() has been executed
 *    + <domain> exists
 *    + <size> must be a multiple of the minimum page size
 *    + <la> must be mapped
 *    + <la> must be a multiple of the minimum page size
 *
 * Postcondition:
 *    + The address range is unmapped
 *    + page table nodes may be reclaimed
 */
extern void (*hal_unmap_range)(struct hal_domain *domain,
                               log_addr_t         la,
                               log_size_t         size,
                               int                options);


/*
 * Description:
 *    Lookup physical page base, page size and permissions
 *    for a logical address
 *
 *    For performance reasons - the lookup will follow the
 *    same way as the HW when taking a page fault. The HAL
 *    implementation might therefor disable the MMU translation
 *    for a short period to be able to follow physical references
 *    in the L1/L2 tree. 
 *
 * Return value:
 *    HAL_SUCCESS or HAL_ELOOKUP_FAILED
 *    Physical page base, size and permissions
 *
 */
extern hal_status_t (*hal_lookup)(OSDOMAIN          domain,
				  log_addr_t        log_addr,
				  phys_addr_t       *page_base,
				  log_size_t        *page_size,
				  U32               *perm,
                                  U32                options);



/*
 * Description:
 *     Retrieve MMU related statistics according to given MM_INFOTAG.
 *     Resources for counters should be activated only after the first
 *     call of this function.
 *
 *     - tag       Tag with tag->tag_id to get statistics from.
 *
 * Return value:
 *     HAL_SUCCESS or
 *     HAL_EUNSUPPORTED, Cpu/MMu has Tag support but does not support this Tag.
 *     HAL_EUNSUPPORTED_MMU, This Cpu/MMu has no Tag support at all.
 *
 * Precondition:
 *    + The MMU is enabled
 *
 * Postcondition: (optional)
 *    
 */
extern hal_status_t (*hal_get_mmu_statistics)(struct mm_tag_ext *tag);


/* Description:
 *     Force pre-allocation of page tables within the specified range.
 *     Once allocated, page tables will never be reclaimed until the 
 *     commitment is removed by using hal_uncommit_range. This feature
 *     is intended for faultable regions and is used to avoid fatal
 *     out of memory situations during execution of demand paged code.
 *
 *     A complete rollback is performed if not enough memory is available.
 *
 * Return value:
 *     HAL_SUCCESS or
 *     HAL_EEND_OF_PHYS_MEMORY, HAL_EEND_OF_PHYS_MEMORY if system memory
 *     is exhausted.
 */
extern hal_status_t
(*hal_commit_range)(struct hal_domain *domain,
                    log_addr_t base, log_size_t size, U32 options);


/* Description:
 *     Remove commitment to keep static page tables, within the specified
 *     range. Page tables no longer needed are garbage collected.
 *
 *     The range must previously have been commited.
 *
 * Return value:
 *     HAL_SUCCESS.
 */
extern hal_status_t
(*hal_uncommit_range)(struct hal_domain *domain,
                      log_addr_t base, log_size_t size, U32 options);


#if 0
/*
 * Use the function <print_func> to dump current MMU mappings.
 */
size_t (*hal_dump_tlb)(print_func_t print_func);
#endif

/*
 * Resident mapping support currently only available for PPC.
 */
extern hal_status_t
(*hal_create_resident_mapping)(struct hal_domain *domain,
                               log_addr_t la,
                               phys_addr_t pa,
                               log_size_t size,
                               U32 options,
                               U32 access_attr,
                               U32 mem_attr);

extern hal_status_t
(*hal_delete_resident_mapping)(struct hal_domain *domain,
                               log_addr_t base,
                               log_size_t size,
                               U32 options);

extern hal_status_t
(*hal_delete_resident_mappings)(log_addr_t base, log_size_t size);


/*
 * Invalidation of Entire range at one sweep only needed for PowerPc
 * with TLB/HASH table.
 */
#ifdef OSPP
extern void (*hal_invalidate_tlb_range)(log_addr_t base,
                                        log_size_t size,
                                        struct hal_domain *domain,
                                        mm_bool_t domain_invariant);
#endif

/* Invalidation of entire TLB. */
extern void (*hal_invalidate_tlb)(void);

/*
 * Description:
 *   Invalidates the TLB on the local core only - for SMP systems.
 *
 *   All pages in the range, in all address spaces are removed from
 *   the MMU cache. 
 */
extern void (*hal_invalidate_tlb_local)(log_addr_t base, log_size_t size);

/*
 * NOTE: The symbol below must only be defined once, i.e. either by nommu hal  
 * or by mmu hal. If the linker signals "multipy defined", we did somehow
 * introduce link dependency to either of the hal:s.
 */

void OSE_MM_HAL_UNIQUELY_DEFINED(void);

/*
 * Install function for nommu.
 */
void install_nommu(void);

/*
 * Install function for mmu.
 */
void install_mmu(void);

#ifdef OSE_SMP
/* No need to put SMP stuff in hal ptr struct since SMP
 * will never be used in a no MMU system.
 */
void
hal_init_other_cpu(unsigned long seg_id);
void
hal_init_other_cpu_booke(unsigned long seg_id);
#endif

#endif /* _HAL_H */
