/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef KRN_INTERNAL_H
#define KRN_INTERNAL_H


#include "cpu_hal.h"
#include "osetypes.h"
#include "stddef.h"


struct mm_handlers
{
   void (*select_segment_hook)(U32 segment);
};


/*
 * Called by EXEC during the start sequence.
 *
 * First function to be called in MM. 
 * 
 * Precondition:
 *   - Unknown with regards to MMU state.
 * Postcondition:
 *   - the memory area 
 *     [<vector_base>, <vector_base> + <vector_size> - 1]
 *     is writable from supervisor mode context.
 *   - the function mm_sync_caches_range() is ready to use.
 */
struct mm_handlers *
mm_open_exception_area(const struct cpu_descriptor *cpu_descriptor,
                       char *vector_base, 
                       size_t vector_size);

/*
 * Called by EXEC during the start sequence.
 *
 * Install exception handlers used by MM.
 *
 * Precondition:
 *   - MMU disabled
 * Postcondition:
 *   - Exception handlers installed and caches synchronised with main
 *     memory. 
 */
void
mm_install_exception_handlers(void);



#endif /* KRN_INTERNAL_H */
