/* COPYRIGHT-ENEA-SRC-R2 * 
 ************************************************************************** 
 * Copyright (C) 2004-2012 by Enea Software AB. 
 * All rights reserved. 
 *
 * This Software is furnished under a software license agreement and 
 * may be used only in accordance with the terms of such agreement. 
 * Any other use or reproduction is prohibited. No title to and 
 * ownership of the Software is hereby transferred. 
 *
 * PROPRIETARY NOTICE 
 * This Software consists of confidential information. 
 * Trade secret law and copyright law protect this Software. 
 * The above notice of copyright on this Software does not indicate 
 * any actual or intended publication of such Software. 
 ************************************************************************** 
 * COPYRIGHT-END */


/* The signal range 40220-40249 has been reserved for this file. */

#ifndef _MM_INTERNAL_SIG
#define _MM_INTERNAL_SIG


/*******************************************************************
 *
 * The following are private flags intended for signals in mm.sig.
 */

/**
 * @macro MM_CREATE_REGION_SEARCH_FORWARD
 * @brief Option bit, set in @link "mm.sig" "MM_CREATE_REGION_REQUEST"
 *
 *        This option is a qualifier for the MM_CREATE_REGION_SPECIFY_LOG 
 *        option, indicating that the nearest higher free address range of
 *        suficient size should be returned if the requested address is
 *        in use.
 *
 * @restriction Implemented only for SAS and MAS
 *
 * @seealso MM_CREATE_REGION_REQUEST
 * @seealso MM_CREATE_REGION_SPECIFY_LOG
 */
#define MM_CREATE_REGION_SEARCH_FORWARD   (0x10000)

/**
 * @macro MM_CREATE_REGION_SEARCH_REVERSE
 * @brief Option bit, set in @link "mm.sig" "MM_CREATE_REGION_REQUEST"
 *
 *        This option is a qualifier for the MM_CREATE_REGION_SPECIFY_LOG 
 *        option, indicating that the nearest lower free address range of
 *        suficient size should be returned if the requested address is
 *        in use.
 *
 *        Use together with MM_CREATE_REGION_SEARCH_FORWARD indicates that
 *        the nearest in either direction should bereturned.
 *
 * @restriction Implemented only for SAS and MAS
 *
 * @seealso MM_CREATE_REGION_REQUEST
 * @seealso MM_CREATE_REGION_SPECIFY_LOG
 */
#define MM_CREATE_REGION_SEARCH_REVERSE   (0x20000)


/*******************************************************************/

/**
 * @signal MM_ADD_PHYS_RESOURCE_REQUEST
 * @brief Signal to add a physical resource. 
 *
 * @member mm_add_phys_resource
 */
#define MM_ADD_PHYS_RESOURCE_REQUEST (40220)\
  /* !-SIGNO(struct mm_add_phys_resource)-! */

/**
 * @signal MM_ADD_PHYS_RESOURCE_REPLY
 * @brief Reply signal from Memory Manager replying to
 *        @link "mm.sig" "MM_ADD_PHYS_RESOURCE_REQUEST".
 *
 * @member mm_add_phys_resource
 */
#define MM_ADD_PHYS_RESOURCE_REPLY   (40221)\
  /* !-SIGNO(struct mm_add_phys_resource)-! */

struct mm_add_phys_resource {
   SIGSELECT            sig_no;
   mm_status_t          status;
   OSDOMAIN             domain;
   OSADDRESS            base;
   char                 phys_mem[MM_NAME_LENGTH + 1];

#if 0
   /* Potential future features. */
   U32                  low_limit;
   U32                  phys_count;        /* I: Length of phys_resources. */
   struct phys_resource phys_resources[1]; /* I: List of phys resources. */
#endif
};

/**
 * @signal MM_REMOVE_PHYS_RESOURCE_REQUEST
 * @brief Signal to remove a physical resource.
 *
 * @member mm_remove_phys_resource
 */
#define MM_REMOVE_PHYS_RESOURCE_REQUEST (40222)\
  /* !-SIGNO(struct mm_remove_phys_resource)-! */

/**
 * @signal MM_REMOVE_PHYS_RESOURCE_REPLY
 * @brief Reply signal from Memory Manager replying to
 *        @link "mm.sig" "MM_REMOVE_PHYS_RESOURCE_REQUEST".
 *
 * @member mm_remove_phys_resource
 */
#define MM_REMOVE_PHYS_RESOURCE_REPLY   (40223)\
  /* !-SIGNO(struct mm_remove_phys_resource)-! */

struct mm_remove_phys_resource {
   SIGSELECT            sig_no;
   mm_status_t          status;
   OSDOMAIN             domain;
   OSADDRESS            base;
};

/**
 * @signal MM_MULTITHREAD_REQUEST
 * @brief Enable or disable multithreading support in MM.
 *
 * @member mm_multithread
 *
 * @seealso MM_MULTITHREAD_REPLY
 */
#define MM_MULTITHREAD_REQUEST (40224)  \
  /* !-SIGNO(struct mm_multithread)-! */

/**
 * @signal MM_MULTITHREAD_REPLY
 * @brief Reply signal from Memory Manager replying to
 *        @link "mm.sig" "MM_MULTITHREAD_REQUEST".
 *
 * @member mm_multithread
 */
#define MM_MULTITHREAD_REPLY   (40225)  \
  /* !-SIGNO(struct mm_multithread)-! */

/**
 * @type mm_multithread
 *
 * @field sig_no Signal number.
 * @input enable Enable (non-zero) or disable (zero) multithreading
 *
 * @output status Status of the operation.
 *         MM_SUCCESS
 *
 */
struct mm_multithread
{
   SIGSELECT   sig_no;
   mm_status_t status; /* Status of the operation */
   mm_bool_t   enable; /* Enable or disable multithreading */
};


/**
 * @signal MM_ALLOC_METADATA_REQUEST
 * @brief Request allocation of hidden metadata
 *
 * @member mm_multithread
 *
 * @seealso MM_ALLOC_METADATA_REPLY
 * @seealso MM_FREE_METADATA_REQUEST
 */
#define MM_ALLOC_METADATA_REQUEST (40226)  \
  /* !-SIGNO(struct mm_alloc_metadata)-! */

/**
 * @signal MM_ALLOC_METADATA_REPLY
 * @brief Reply signal from Memory Manager replying to
 *        @link "mm.sig" "MM_ALLOC_METADATA_REQUEST".
 *
 * @member mm_multithread
 */
#define MM_ALLOC_METADATA_REPLY   (40227)  \
  /* !-SIGNO(struct mm_multithread)-! */

/**
 * @type mm_alloc_metadata
 *
 * @field sig_no Signal number.
 * @input size   Requested size of the new metadata region.
 *
 * @output base   Base address of the created region.
 * @output status Status of the operation.
 *         MM_SUCCESS
 *
 */
struct mm_alloc_metadata
{
   SIGSELECT   sig_no;
   mm_status_t status;
   log_addr_t  base;
   size_t      size;
};


/**
 * @signal MM_FREE_METADATA_REQUEST
 * @brief Request deallocation of hidden metadata.
 *
 * @member mm_free_metadata
 *
 * @seealso MM_FREE_METADATA_REQUEST
 */
#define MM_FREE_METADATA_REQUEST (40228)  \
  /* !-SIGNO(struct mm_free_metadata)-! */

/**
 * @signal MM_FREE_METADATA_REPLY
 * @brief Reply signal from Memory Manager replying to
 *        @link "mm.sig" "MM_FREE_METADATA_REQUEST".
 *
 * @member mm_free_metadata
 */
#define MM_FREE_METADATA_REPLY   (40229)  \
  /* !-SIGNO(struct mm_free_metadata)-! */

/**
 * @type mm_free_metadata
 *
 * @field sig_no Signal number.
 * @input base   Logical address of metadata region to remove. 
 *
 * @output status Status of the operation.
 *         MM_SUCCESS
 *
 */
struct mm_free_metadata
{
   SIGSELECT   sig_no;
   mm_status_t status;
   log_addr_t  base;
   size_t      size;
};

/**
* @signal MM_ALLOC_PHYS_RESOURCE_REQUEST
* @brief Request allocation of physical resource from MM

*/
#define MM_ALLOC_PHYS_RESOURCE_REQUEST (40230)\
/* !-SIGNO(struct mm_alloc_phys_resource)-! */

/**
* @signal MM_ALLOC_PHYS_RESOURCE_REPLY
* @brief Signal from Memory Manager replying to
*        @link "mm_internal.sig" "MM_ALLOC_PHYS_RESOURCE_REQUEST".
*
*/
#define MM_ALLOC_PHYS_RESOURCE_REPLY (40231)\
/* !-SIGNO(struct mm_alloc_phys_resource)-! */

/**
 * @type mm_alloc_phys_resource
 *
 * @field sig_no   Signal number.
 * @input domain   Identifier of recovery domain for which the
 *                 physical resource shall be allocated. 
 * @input align    Alignment requirement for the physical resource
 *                 (must be a power to 2 or zero,
 *                 which implies no alignment requirement).
 * @input size     Size requested.
 * @output size    Size allocated (may be less or equal to the requested size).
 * @input phys_mem Name of the physical RAM-type from which the
 *                 physical resource shall be allocated.
 * @output base    Physical base address of the physical resource
 *                 allocated by MM. 
 * @output status  Status of operation.
 *                 MM_SUCCESS is returned on successful operation.
 *                 MM_ENON_EXISTING_DOMAIN is returned in case the domain
 *                 identifier is unknown to MM.
 *                 MM_ENON_EXISTING_PHYS_MEM is return in case the physical
 *                 RAM-type is unknown to MM.
 *                 MM_EOUT_OF_PHYS_MEM is returned in case MM is out of
 *                 physical memory of the specified type.
 *                 MM_EILLEGAL_AREA is returned if the alignment reqirement
 *                 is not a power to 2, nor zero.
 */
struct mm_alloc_phys_resource {
   SIGSELECT sig_no;
   mm_status_t status;
   OSDOMAIN domain;
   phys_addr_t align;
   phys_size_t size;
   char phys_mem[MM_NAME_LENGTH + 1];
   phys_addr_t base;
};

/**
* @signal MM_FREE_PHYS_RESOURCE_REQUEST
* @brief Request that a physical resource be freed by MM.
*
* @member mm_free_phys_resource
*/
#define MM_FREE_PHYS_RESOURCE_REQUEST (40232)\
/* !-SIGNO(struct mm_free_phys_resource)-! */

/**
* @signal MM_FREE_PHYS_RESOURCE_REPLY
* @brief Signal from Memory Manager replying to
*        @link "mm_internal.sig" "MM_FREE_PHYS_RESOURCE_REQUEST".
*/
#define MM_FREE_PHYS_RESOURCE_REPLY (40233)\
/* !-SIGNO(struct mm_free_phys_resource)-! */

/**
 * @type mm_free_phys_resource
 *
 * @field sig_no Signal number.
 * @input domain Identifier of recovery domain for which the physical resource
 * was allocated. 
 * @input base   Physical base address of physical resource to free. 
 *
 * @output status Status of operation.
 *                MM_SUCCESS is returned on successful operation.
 *                MM_ENON_EXISTING_DOMAIN is returned in case the domain
 *                identifier is unknown to MM.
 *                MM_ENON_EXISTING_PHYS_MEM is returned in case the physical
 *                base address of the physical region is unknown to MM.
 */
struct mm_free_phys_resource {
   SIGSELECT sig_no;
   mm_status_t status;
   OSDOMAIN domain;
   phys_addr_t base;
};

#endif /* _MM_INTERNAL_SIG */
