/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef _SYS_QUEUE_OSE_H_
#define	_SYS_QUEUE_OSE_H_

typedef struct OseListHeadType 
{
   void *first;
   unsigned int const nxt_ofs;
}OseListHeadType;

typedef struct OseListHeadTailType 
{
   void *first;
   void *last;
   unsigned int const nxt_ofs;
}OseListHeadTailType;

typedef void *SLinkType, *DLinkType[2];

#define GENERIC_LIST_HEAD_TYPE            OseListHeadType
#define GENERIC_LIST_HEAD_TAIL_TYPE       OseListHeadTailType

#define	GENERIC_LIST_HEAD(var_name, el_type, field_name)                   \
   OseListHeadType var_name =	                                           \
                { NULL, offsetof(el_type , field_name) } 

#define	GENERIC_LIST_HEAD_TAIL(var_name, el_type, field_name)              \
   OseListHeadTailType var_name =	                                   \
                { NULL, NULL, offsetof(el_type , field_name) } 

#define GENERIC_LIST_HEAD_INIT(head_ptr) do {                              \
   (head_ptr)->first = NULL;                                               \
} while(0);

#define GENERIC_LIST_HEAD_TAIL_INIT(head_ptr) do {                         \
   (head_ptr)->first = NULL;                                               \
   (head_ptr)->last = NULL;                                                \
} while(0);

#define	GENERIC_ENTRY           SLinkType
#define GENERIC_ENTRY_DLINK     DLinkType

#define	GENERIC_EMPTY(head)	((head)->first == NULL)
#define	GENERIC_FIRST(head)	((head)->first)
#define	GENERIC_LAST(head)	((head)->last)

#define GENERIC_NEXT(head, elm)                                            \
   (*(void**)((char*)(elm) + (head)->nxt_ofs))

#define GENERIC_PREV(head, elm)                                            \
   (*(void**)((char*)(elm) + (head)->nxt_ofs + sizeof(void*)))

#define	GENERIC_FOREACH(head, var)                                         \
   for((var) = (head)->first;                                              \
   (var);                                                                  \
   (var) = GENERIC_NEXT((head), (var)))

#define	GENERIC_FOREACH_REVERSE(head, var)                                 \
   for((var) = (head)->last;                                               \
   (var);                                                                  \
   (var) = GENERIC_PREV((head), (var)))

#define GENERIC_REMOVE_HEAD(head)                                          \
   (head)->first = GENERIC_NEXT((head), (head)->first)

#define GENERIC_REMOVE_MIDDLE(head, elm)                                   \
   GENERIC_NEXT((head), elm) =	GENERIC_NEXT((head), GENERIC_NEXT((head), elm))

#define OSE_SLIST_HEAD_TYPE          GENERIC_LIST_HEAD_TYPE
#define	OSE_SLIST_ENTRY              GENERIC_ENTRY
#define OSE_SLIST_EMPTY(head)        GENERIC_EMPTY(head)
#define OSE_SLIST_FIRST(head)        GENERIC_FIRST(head)
#define	OSE_SLIST_FOREACH(head, var) GENERIC_FOREACH(head, var)
#define OSE_SLIST_NEXT(head, elm)    GENERIC_NEXT(head, elm)

#define	OSE_SLIST_HEAD(var_name, el_type, field_name)                      \
   GENERIC_LIST_HEAD(var_name, el_type, field_name)

#define OSE_SLIST_INIT(head_ptr)     GENERIC_LIST_HEAD_INIT(head_ptr)

#define OSE_SLIST_INSERT_AFTER(head, l_elm, elm) do {                      \
   GENERIC_NEXT((head), (elm)) = GENERIC_NEXT((head), (l_elm));            \
   GENERIC_NEXT((head), (l_elm)) = (elm);                                  \
} while (0)

#define	OSE_SLIST_INSERT_HEAD(head, elm) do {                              \
   GENERIC_NEXT((head), (elm)) = GENERIC_FIRST((head));                    \
   GENERIC_FIRST((head)) = (elm);                                          \
} while (0)

#define	OSE_SLIST_REMOVE_HEAD(head) do {                                   \
   GENERIC_REMOVE_HEAD(head);                                              \
} while (0)

#define	OSE_SLIST_REMOVE(head, elm) do {                                   \
   if ((head)->first == (elm)) {                                           \
      GENERIC_REMOVE_HEAD(head);                                           \
   }                                                                       \
   else {                                                                  \
      void *curelm = (head)->first;                                        \
      while (GENERIC_NEXT((head), curelm) != (elm))                        \
         curelm = GENERIC_NEXT((head), curelm);                            \
      GENERIC_REMOVE_MIDDLE((head), curelm);                               \
   }                                                                       \
} while (0)


#define OSE_STAILQ_HEAD_TYPE         GENERIC_LIST_HEAD_TAIL_TYPE
#define	OSE_STAILQ_HEAD(var_name, el_type, field_name)                     \
   GENERIC_LIST_HEAD_TAIL(var_name, el_type, field_name)

#define OSE_STAILQ_INIT(head_ptr)    GENERIC_LIST_HEAD_TAIL_INIT(head_ptr)

#define	OSE_STAILQ_ENTRY                GENERIC_ENTRY
#define	OSE_STAILQ_EMPTY(head)	        GENERIC_EMPTY(head)
#define	OSE_STAILQ_FIRST(head)	        GENERIC_FIRST(head)
#define	OSE_STAILQ_LAST(head)	        GENERIC_LAST(head)
#define OSE_STAILQ_NEXT(head, elm)      GENERIC_NEXT(head, elm)
#define	OSE_STAILQ_FOREACH(head, var)	GENERIC_FOREACH(head, var)

#define	OSE_STAILQ_INSERT_AFTER(head, l_elm, elm) do {                     \
   if ((GENERIC_NEXT((head), (elm)) =                                      \
        GENERIC_NEXT((head), (l_elm))) == NULL)                            \
      (head)->last = (elm);                                                \
   GENERIC_NEXT((head), (l_elm)) = (elm);                                  \
} while (0)

#define	OSE_STAILQ_INSERT_HEAD(head, elm) do {                             \
   if ((GENERIC_NEXT((head), (elm)) = (head)->first) == NULL)              \
      (head)->last = (elm);                                                \
   (head)->first = (elm);                                                  \
} while (0)

#define	OSE_STAILQ_INSERT_TAIL(head, elm) do {                             \
   if ((GENERIC_NEXT((head), (elm)) = NULL) != (head)->first)              \
      (head)->last = GENERIC_NEXT((head), (head)->last) = (elm);           \
   else                                                                    \
      (head)->last = (head)->first = (elm);                                \
} while (0)

#define	OSE_STAILQ_REMOVE_HEAD(head) do {                                  \
   if ((GENERIC_REMOVE_HEAD(head)) == NULL)                                \
      (head)->last = NULL;                                                 \
} while (0)

#define	OSE_STAILQ_REMOVE(head, elm) do {                                  \
   if ((head)->first == (elm)) {                                           \
      OSE_STAILQ_REMOVE_HEAD(head);                                        \
   }                                                                       \
   else {                                                                  \
      void *curelm = (head)->first;                                        \
      while (GENERIC_NEXT((head), curelm) != (elm))                        \
         curelm = GENERIC_NEXT((head), curelm);                            \
      if ((GENERIC_REMOVE_MIDDLE((head), curelm)) == NULL)                 \
         (head)->last = curelm;                                            \
   }                                                                       \
} while (0)

#define	OSE_STAILQ_REMOVE_HEAD_UNTIL(head, elm) do {                       \
   if (((head)->first = GENERIC_NEXT((head), (elm))) == NULL)              \
      (head)->last = NULL;                                                 \
} while (0)


#define OSE_LIST_HEAD_TYPE            GENERIC_LIST_HEAD_TYPE
#define	OSE_LIST_HEAD(var_name, el_type, field_name)                       \
   GENERIC_LIST_HEAD(var_name, el_type, field_name)

#define OSE_LIST_INIT(head_ptr)       GENERIC_LIST_HEAD_INIT(head_ptr)

#define	OSE_LIST_ENTRY                GENERIC_ENTRY_DLINK
#define	OSE_LIST_EMPTY(head)          GENERIC_EMPTY(head)
#define	OSE_LIST_FIRST(head)          GENERIC_FIRST(head)
#define OSE_LIST_NEXT(head, elm)      GENERIC_NEXT(head, elm)
#define	OSE_LIST_FOREACH(head, var)   GENERIC_FOREACH(head, var)

#define	OSE_LIST_INSERT_AFTER(head, l_elm, elm) do {                       \
   if ((GENERIC_NEXT((head), (elm)) = GENERIC_NEXT((head), (l_elm))) != NULL)\
      GENERIC_PREV((head), GENERIC_NEXT((head), (l_elm))) = (elm);         \
   GENERIC_NEXT((head), (l_elm)) = (elm);                                  \
   GENERIC_PREV((head), (elm)) = (l_elm);                                  \
} while (0)

#define	OSE_LIST_INSERT_BEFORE(head, l_elm, elm) do {                      \
   if ((GENERIC_PREV((head), (elm)) = GENERIC_PREV((head), (l_elm))) != NULL)\
   {                                                                       \
      GENERIC_NEXT((head), GENERIC_PREV((head), (l_elm))) = (elm);         \
   }                                                                       \
   else                                                                    \
      (head)->first = (elm);                                               \
   GENERIC_NEXT((head), (elm)) = (l_elm);                                  \
   GENERIC_PREV((head), (l_elm)) = (elm);                                  \
} while (0)

#define	OSE_LIST_INSERT_HEAD(head, elm) do {                               \
   if ((GENERIC_NEXT((head), (elm)) = (head)->first) != NULL)              \
   {                                                                       \
      GENERIC_PREV((head), (head)->first) = (elm);                         \
   }                                                                       \
   (head)->first = (elm);                                                  \
   GENERIC_PREV((head), (elm)) = NULL;                                     \
} while (0)

#define	OSE_LIST_REMOVE(head, elm) do {                                    \
   if (GENERIC_NEXT((head), (elm)) != NULL)                                \
      GENERIC_PREV((head), GENERIC_NEXT((head), (elm))) = GENERIC_PREV((head), (elm));\
   if (GENERIC_PREV((head), (elm)) != NULL)                                \
   {                                                                       \
      GENERIC_NEXT((head), GENERIC_PREV((head), (elm))) = GENERIC_NEXT((head), (elm));\
   }                                                                       \
   else                                                                    \
   {                                                                       \
      (head)->first = GENERIC_NEXT((head), (elm));                         \
   }                                                                       \
} while (0)


#define OSE_TAILQ_HEAD_TYPE            GENERIC_LIST_HEAD_TAIL_TYPE
#define	OSE_TAILQ_HEAD(var_name, el_type, field_name)                      \
   GENERIC_LIST_HEAD_TAIL(var_name, el_type, field_name)

#define OSE_TAILQ_INIT(head_ptr)       GENERIC_LIST_HEAD_TAIL_INIT(head_ptr)

#define	OSE_TAILQ_ENTRY                GENERIC_ENTRY_DLINK
#define	OSE_TAILQ_EMPTY(head)          GENERIC_EMPTY(head)
#define	OSE_TAILQ_FIRST(head)          GENERIC_FIRST(head)
#define	OSE_TAILQ_LAST(head)           GENERIC_LAST(head)
#define OSE_TAILQ_NEXT(head, elm)      GENERIC_NEXT(head, elm)
#define	OSE_TAILQ_FOREACH(head, var)   GENERIC_FOREACH(head, var)
#define	OSE_TAILQ_FOREACH_REVERSE(head, var)   GENERIC_FOREACH_REVERSE(head, var)

#define	OSE_TAILQ_INSERT_AFTER(head, l_elm, elm) do {                      \
   if ((GENERIC_NEXT((head), (elm)) = GENERIC_NEXT((head), (l_elm))) != NULL)\
      GENERIC_PREV((head), GENERIC_NEXT((head), (l_elm))) = (elm);         \
   else                                                                    \
      (head)->last = (elm);                                                \
   GENERIC_NEXT((head), (l_elm)) = (elm);                                  \
   GENERIC_PREV((head), (elm)) = (l_elm);                                  \
} while (0)

#define	OSE_TAILQ_INSERT_BEFORE(head, l_elm, elm) do {                     \
   if ((GENERIC_PREV((head), (elm)) = GENERIC_PREV((head), (l_elm))) != NULL)\
   {                                                                       \
      GENERIC_NEXT((head), GENERIC_PREV((head), (l_elm))) = (elm);         \
   }                                                                       \
   else                                                                    \
      (head)->first = (elm);                                               \
   GENERIC_NEXT((head), (elm)) = (l_elm);                                  \
   GENERIC_PREV((head), (l_elm)) = (elm);                                  \
} while (0)

#define	OSE_TAILQ_INSERT_HEAD(head, elm) do {                              \
   if ((GENERIC_NEXT((head), (elm)) = (head)->first) != NULL)              \
   {                                                                       \
      GENERIC_PREV((head), (head)->first) = (elm);                         \
   }                                                                       \
   else                                                                    \
      (head)->last = (elm);                                                \
   (head)->first = (elm);                                                  \
   GENERIC_PREV((head), (elm)) = NULL;                                     \
} while (0)

#define	OSE_TAILQ_INSERT_TAIL(head, elm) do {                              \
   GENERIC_PREV((head), (elm)) = (head)->last;                             \
   if ((GENERIC_NEXT((head), (elm)) = NULL) != (head)->first)              \
      (head)->last = GENERIC_NEXT((head), (head)->last) = (elm);           \
   else                                                                    \
      (head)->last = (head)->first = (elm);                                \
} while (0)

#define	OSE_TAILQ_REMOVE(head, elm) do {                                   \
   if (GENERIC_NEXT((head), (elm)) != NULL)                                \
      GENERIC_PREV((head), GENERIC_NEXT((head), (elm))) = GENERIC_PREV((head), (elm));\
   else                                                                    \
      (head)->last = GENERIC_PREV((head), (elm));                          \
   if (GENERIC_PREV((head), (elm)) != NULL)                                \
   {                                                                       \
      GENERIC_NEXT((head), GENERIC_PREV((head), (elm))) = GENERIC_NEXT((head), (elm)); \
   }                                                                       \
   else                                                                    \
   {                                                                       \
      (head)->first = GENERIC_NEXT((head), (elm));                         \
   }                                                                       \
} while (0)

#define OSE_CIRCLEQ_HEAD_TYPE            GENERIC_LIST_HEAD_TAIL_TYPE
#define	OSE_CIRCLEQ_HEAD(var_name, el_type, field_name)                    \
   GENERIC_LIST_HEAD_TAIL(var_name, el_type, field_name)

#define OSE_CIRCLEQ_INIT(head_ptr)       GENERIC_LIST_HEAD_TAIL_INIT(head_ptr)

#define	OSE_CIRCLEQ_ENTRY                GENERIC_ENTRY_DLINK
#define	OSE_CIRCLEQ_EMPTY(head)          GENERIC_EMPTY(head)
#define	OSE_CIRCLEQ_FIRST(head)          GENERIC_FIRST(head)
#define	OSE_CIRCLEQ_LAST(head)           GENERIC_LAST(head)
#define OSE_CIRCLEQ_NEXT(head, elm)      GENERIC_NEXT(head, elm)
#define OSE_CIRCLEQ_PREV(head, elm)      GENERIC_PREV(head, elm)
#define	OSE_CIRCLEQ_FOREACH(head, var)                                     \
   for((var) = (head)->first;                                              \
   (var);                                                                  \
   (var) = ((var) != (head)->last) ? GENERIC_NEXT((head), (var)) : NULL)

#define	OSE_CIRCLEQ_FOREACH_REVERSE(head, var)                             \
   for((var) = (head)->last;                                               \
   (var);                                                                  \
   (var) = ((var) != (head)->first) ? GENERIC_PREV((head), (var)) : NULL)

#define	OSE_CIRCLEQ_INSERT_AFTER(head, l_elm, elm) do {                    \
    if ((GENERIC_NEXT((head), (elm)) = GENERIC_NEXT((head), (l_elm))) == (head)->first)\
       (head)->last = (elm);                                               \
   GENERIC_PREV((head), GENERIC_NEXT((head), (l_elm))) = (elm);            \
   GENERIC_NEXT((head), (l_elm)) = (elm);                                  \
   GENERIC_PREV((head), (elm)) = (l_elm);                                  \
} while (0)

#define	OSE_CIRCLEQ_INSERT_BEFORE(head, l_elm, elm) do {                   \
   if ((GENERIC_PREV((head), (elm)) = GENERIC_PREV((head), (l_elm))) == (head)->last)\
      (head)->first = (elm);                                               \
   GENERIC_NEXT((head), GENERIC_PREV((head), (l_elm))) = (elm);            \
   GENERIC_NEXT((head), (elm)) = (l_elm);                                  \
   GENERIC_PREV((head), (l_elm)) = (elm);                                  \
} while (0)

#define	OSE_CIRCLEQ_INSERT_HEAD(head, elm) do {                            \
   if ((GENERIC_NEXT((head), (elm)) = (head)->first) != NULL)              \
   {                                                                       \
      GENERIC_PREV((head), (head)->first) = (elm);                         \
      GENERIC_NEXT((head), (head)->last) = (elm);                          \
   }                                                                       \
   else                                                                    \
      GENERIC_NEXT((head), (elm)) = (head)->last = (elm);                  \
   (head)->first = (elm);                                                  \
   GENERIC_PREV((head), (elm)) = (head)->last;                             \
} while (0)

#define	OSE_CIRCLEQ_INSERT_TAIL(head, elm) do {                            \
   if ((GENERIC_PREV((head), (elm)) = (head)->last) != NULL)               \
   {                                                                       \
      GENERIC_PREV((head), (head)->first) = (elm);                         \
      GENERIC_NEXT((head), (head)->last) = (elm);                          \
   }                                                                       \
   else                                                                    \
      GENERIC_PREV((head), (elm)) = (head)->first = (elm);                 \
   (head)->last = (elm);                                                   \
   GENERIC_NEXT((head), (elm)) = (head)->first;                            \
} while (0)

#define	OSE_CIRCLEQ_REMOVE(head, elm) do {                                 \
   if ((head)->first != (head)->last) {                                    \
      if ((head)->first == (elm)) {                                        \
           (head)->first = GENERIC_NEXT((head), (elm));                    \
      }                                                                    \
      if ((elm) == ((head)->last)) {                                       \
           (head)->last = GENERIC_PREV((head), (elm));                     \
      }                                                                    \
      GENERIC_PREV((head), GENERIC_NEXT((head), (elm))) = GENERIC_PREV((head), (elm));\
      GENERIC_NEXT((head), GENERIC_PREV((head), (elm))) = GENERIC_NEXT((head), (elm));\
   }                                                                       \
   else                                                                    \
      (head)->first = (head)->last = NULL;                                 \
} while (0)

#endif /* _SYS_QUEUE_OSE_H_ */



