/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef ZONE_H
#define ZONE_H

#include "stddef.h"
#include "queue_ose.h"

#define ZONE_INTERRUPT 0x0001	/* If you need to allocate at int time */
#define ZONE_PANICFAIL 0x0002	/* panic if the zalloc fails */
#define ZONE_BOOT      0x0010	/* Internal flag used by zbootinit */
#define ZONE_INIT_2_DONE      0x0010	/* Internal flag used by zbootinit */
#define ZONE_QUEUED_FOR_EXTEND 0x0020   /* Internal */

struct zone
{
      OSE_SLIST_ENTRY all_next;         /* Link all zones. */
      OSE_TAILQ_ENTRY extend_next;
      unsigned          zfreemax;

      void		*zitems;	/* linked list of items */
      int		zfreecnt;	/* free entries */
      int		zfreemin;	/* minimum number of free entries */
      int		add_cnt;	/* number of allocations */
      int		znalloc;	/* number of allocations */
      int		zpagecount;	/* Total # of allocated pages */
      int		zpagemax;	/* Max address space */
      int		zmax;		/* Max number of entries allocated */
      int		ztotal;		/* Total entries allocated now */
      unsigned		zsize;		/* size of each entry */
      int		zalloc;		/* hint for # of pages to alloc */
      int		flags;		/* flags for zone */
      int		zallocflag;	/* flag for allocation */
#if 0
      struct vm_object *zobj;		/* object to hold zone */
#endif
      const char	*zname;		/* name for diags */
};

typedef struct zone *zone_t;


void
mm_zone_late_init(const char *zone_log_mem_name);

/*
 * Perform first part of initialisation.
 */
void
mm_zinit_boot_1(zone_t z,
                const char *name,
                size_t size,
                void *bss,
                int item_count);

/*
 * Perform first part of initialisation.
 */
int
mm_zinit_boot_2(zone_t z,
                unsigned min_item_count,
                unsigned max_item_count);

void
mm_zinit(zone_t z,
         const char *name,
         size_t item_size,
         unsigned min_item_count,
         unsigned max_item_count);

/*
 * Allocate an item from zone <z>. If no memory is available, a NULL
 * pointer is returned.
 */
void *
mm_zalloc(zone_t z);

/*
 * Release item <item> back to zone <z>.
 */
void
mm_zfree(zone_t z,
         void *item);

/*
 * Get the amount of pages in RAM allocated to the zone-allocator.
 */
unsigned
mm_zone_get_page_cnt(void);

#endif /* ZONE_H */
