/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#include "ose.h"
#include "ose_agent.h"
#include "ose_agent.sig"
#include "ramlog.h"
#include "string.h"
#include "cache.h"
#include "cpu_hal_arch.h"
#include "mm_access.h"
#include "mm_util.h"
#include "bios.h"
#include "krncon.h"


#define TAG "OSE test agent: "


#define RAMLOG_PRINTF(x)

#define ERROR(msg) \
  {\
    ramlog_printf(TAG "\n");\
    ramlog_printf("========= ERROR =========\n");\
    ramlog_printf(__FILE__ ":%u. ", __LINE__);\
    ramlog_printf msg;\
    ramlog_printf("=========================\n");\
  }

static const SIGSELECT anySig[] = {0};

union SIGNAL {
  SIGSELECT sigNo;
  struct oseAgentCacheSync            cacheSync;
  struct oseAgentGetHalInfo           getHalInfo;
  struct oseAgentGetIfId              getIfId;
  struct oseAgentInstallErrHook       installErrHook;
  struct oseAgentForcedWrite          forcedWrite;
  struct oseAgentGetMmAttributes      getMmAttributes;

  struct oseAgentEpmAddExecutionPoint    addExecutionPoint;
  struct oseAgentEpmRemoveExecutionPoint removeExecutionPoint;
  struct oseAgentEpmNextInstruction      nextInstruction;
  struct oseAgentEpmDisableAll           disableAll;
  struct oseAgentEpmEnableAll            enableAll;

  struct oseAgentGetCpuDescriptor        getCpuDescriptor;

  struct oseAgentMmMapRange              mapRange;
  struct oseAgentMmUnmapRange            unmapRange;
  struct oseAgentMmMultithread           multithread;

  struct oseAgentCheckSwaph              checkSwaphList;
   
};

extern PROCESS zzmain_pid;
extern struct Ti_ctlblk zzconf;
#include "compiler_macros.h"
#ifndef WIN32
WEAK_REF(void mm_test_swap_in_handler(void * uarea));
#else
void * dummy_swap_in = 0;
void mm_test_swap_in_handler(void * uarea) __attribute__((weak, alias("dummy_swap_in")));
#endif

/****************************************/

static void
sys_err_hnd_hook_install(OSADDRESS (*hook)(OSBOOLEAN, OSERRCODE, OSERRCODE)) {
  (void)get_sysconf((OSADDRESS) 68, (OSADDRESS)hook,0,0);
}

/****************************************/

static OS_PROCESS(ose_agent) {
  OSATTREF attRef;
  PROCESS hookPid = 0;
  union SIGNAL *sig;
  struct oseAgentInstallErrHook *waitingInstallErrHook = 0;

  ramlog_printf(TAG "Agent started. Interface version = %u.%u\n",
                OSE_AGENT_IF_VERSION_MAJOR, OSE_AGENT_IF_VERSION_MINOR);

  for (;;) {
    PROCESS senderPid;

    if (!hookPid && waitingInstallErrHook) {
      sig =  (union SIGNAL*)waitingInstallErrHook;
      waitingInstallErrHook = waitingInstallErrHook->next;
      RAMLOG_PRINTF((TAG "Using queued request\n"));
    }
    else {
      RAMLOG_PRINTF((TAG "Waiting for request\n"));
      sig = receive(anySig);
    }

    senderPid = sender(&sig);
    RAMLOG_PRINTF((TAG "Processing 0x%08lx from 0x%08lx\n",
                   sig->sigNo, senderPid));

    switch (sig->sigNo) {

    case OS_ATTACH_SIG:
      if (hookPid && hookPid == senderPid) {
        RAMLOG_PRINTF((TAG "Uninstalling error hook installed "
                       "by 0x%08lx. (Attach)\n", senderPid));
        hookPid = 0;
      }
      else {
        ERROR(("Unexpected attach signal received from 0x%08lx. "
               "hookPid = 0x%08lx\n", hookPid));
      }
      free_buf(&sig);

      /* Remove pending requests from the killed process */
      while (sig = receive_from(0, anySig, senderPid), sig) { free_buf(&sig); }
      break;

    case OSE_AGENT_INSTALL_ERR_HOOK:
      if (sig->installErrHook.hook) {
        if (hookPid) {
          struct oseAgentInstallErrHook **tmp = &waitingInstallErrHook;
          while (*tmp) tmp = &((*tmp)->next);
          *tmp = &sig->installErrHook;
          sig->installErrHook.next = 0;
          RAMLOG_PRINTF((TAG "Queing OSE_AGENT_INSTALL_ERR_HOOK from "
                         "0x%08lx\n", senderPid));
        }
        else {
          RAMLOG_PRINTF((TAG "sys_err_hnd_hook_install(0x%08lx)\n",
                         sig->installErrHook.hook));
          sys_err_hnd_hook_install(sig->installErrHook.hook);

          RAMLOG_PRINTF((TAG "attach(0x%08lx). hook = 0x%08lx\n",
                         senderPid, sig->installErrHook.hook));
          hookPid = senderPid;
          attRef = attach(0, hookPid);
          send(&sig, senderPid);
        }
      }
      else {
        if (hookPid) {
          RAMLOG_PRINTF((TAG "detach(0x%08lx)\n", hookPid));
          hookPid = 0;
          detach(&attRef);
          RAMLOG_PRINTF((TAG "sys_err_hnd_hook_install(0)\n"));
          sys_err_hnd_hook_install(0);
          delay(1);
        }
        send(&sig, senderPid);
      }

      break;

    case OSE_AGENT_GET_IF_ID:
      free_buf(&sig);
      sig = alloc(sizeof(struct oseAgentGetIfId), OSE_AGENT_GET_IF_ID);
      sig->getIfId.major = OSE_AGENT_IF_VERSION_MAJOR;
      sig->getIfId.minor = OSE_AGENT_IF_VERSION_MINOR;
      send(&sig, senderPid);
      break;

    case OSE_AGENT_CACHE_SYNC:
      cache_sync(sig->cacheSync.base, sig->cacheSync.size);
      send(&sig, senderPid);
      break;

    case OSE_AGENT_GET_HAL_INFO:
      sig->getHalInfo.info = *hal_get_info();
      send(&sig, senderPid);
      break;

    case OSE_AGENT_FORCED_WRITE:
      {
        struct oseAgentForcedWrite *tmp = &sig->forcedWrite;
        tmp->result = mm_access_forced_write(tmp->dst_domain,
                                             tmp->dst,
                                             tmp->src,
                                             tmp->size,
                                             tmp->freeze_mode);
      }
      send(&sig, senderPid);
      break;

      case OSE_AGENT_GET_MM_ATTRIBUTES:
      {
         U32 attr = 0;
         if (hal_arch_cpu_descriptor) {
            attr = hal_arch_cpu_descriptor->mm_attributes;
         }
         sig->getMmAttributes.attr = attr;
      }
      send(&sig, senderPid);
      break;

      case OSE_AGENT_EPM_ADD_EXECUTION_POINT:
      {
         struct oseAgentEpmAddExecutionPoint *tmp = &sig->addExecutionPoint;
         tmp->status = ose_epm_add_execution_point(tmp->ep);

      }
      send(&sig, senderPid);
      break;

      case OSE_AGENT_EPM_REMOVE_EXECUTION_POINT:
      {
         struct oseAgentEpmRemoveExecutionPoint *tmp =
            &sig->removeExecutionPoint;

         tmp->status = ose_epm_remove_execution_point(tmp->ep);

      }
      send(&sig, senderPid);
      break;

      case OSE_AGENT_EPM_NEXT_INSTRUCTION:
      {
         struct  oseAgentEpmNextInstruction *tmp =
            &sig->nextInstruction;

         tmp->status = ose_epm_next_instruction(tmp->dump,
                                                tmp->options,
                                                tmp->next_address);

      }
      send(&sig, senderPid);
      break;

      case OSE_AGENT_EPM_DISABLE_ALL:
         sig->disableAll.status = ose_epm_disable_all();
         send(&sig, senderPid);
         break;

      case OSE_AGENT_EPM_ENABLE_ALL:
         sig->enableAll.status = ose_epm_enable_all();
         send(&sig, senderPid);
         break;

      case OSE_AGENT_GET_CPU_DESCRIPTOR:
         sig->getCpuDescriptor.hal_arch_cpu_descriptor = 
            hal_arch_cpu_descriptor;
         send(&sig, senderPid);
         break;

       case OSE_AGENT_MM_CHECK_SWAPH:
       {
          U32 *config_array;
          U32 swaph_address = 0;
                    
          switch (sig->checkSwaphList.swaph_nr)
          {
             case MM_TEST_SWAP_IN_HANDLER:
                swaph_address = (U32)mm_test_swap_in_handler;
                break;
             default:
                ERROR(("Unknown swap (0x%08lx) received from 0x%08lx\n",
                       sig->checkSwaphList.swaph_nr, senderPid));
                break;
          }

          sig->checkSwaphList.status = MM_ESIGNAL_NOT_IMPLEMENTED;
          if(zzconf.swap_in_handler_list)
          {
             config_array = (U32*) zzconf.swap_in_handler_list;
             
             while(*config_array)		
             {
                if (*config_array == swaph_address)
                {
                   sig->checkSwaphList.status = MM_SUCCESS;
                   break;
                }
                config_array++;
             }
          }
          send(&sig, senderPid);
       }
       break;
       
      case OSE_AGENT_MM_MAP_RANGE:
         sig->mapRange.result = ose_mm_map_range(
            sig->mapRange.domain,
            sig->mapRange.la,
            sig->mapRange.pa,
            sig->mapRange.size,
            sig->mapRange.permission,
            sig->mapRange.cache_mode,
            sig->mapRange.options);
         send(&sig, senderPid);
         break;

      case OSE_AGENT_MM_UNMAP_RANGE:
         sig->unmapRange.result = ose_mm_unmap_range(
            sig->unmapRange.domain,
            sig->unmapRange.la,
            sig->unmapRange.size,
            sig->unmapRange.options);
         send(&sig, senderPid);
         break;

       case OSE_AGENT_MM_MULTITHREAD:
         sig->multithread.result = ose_mm_multithread(
            sig->multithread.enable);
         send(&sig, senderPid);
         break;

      default:
      {
        SIGSELECT sigNo = sig->sigNo;
        free_buf(&sig);
        ERROR(("Unknown signal (0x%08lx) received from 0x%08lx\n",
               sigNo, senderPid));
      }
    }
  }
}

static long biosStartAgent(PROCESS *retPid, U32 *verMaj, U32 *verMin) {
  PROCESS pid = 0;
  PROCESS main_block = get_bid(zzmain_pid);

  if (hunt(OSE_AGENT_NAME, 0, &pid, 0)) {
    ramlog_printf(TAG "Agent \"" OSE_AGENT_NAME "\" already started. "
                  "Pid = 0x%08lx\n", pid);

  }
  else {
    ramlog_printf(TAG "Creating process " OSE_AGENT_NAME "\n");
    pid = create_process(OS_PRI_PROC,
                         OSE_AGENT_NAME,
                         ose_agent,
                         2000,
                         20,
                         (OSTIME)0,
                         main_block,
                         (struct OS_redir_entry *) NULL,
                         (OSVECTOR)0,
                         (OSUSER)0
                         );

    ramlog_printf("Starting 0x%08lx\n", pid);
    start(pid);
    ramlog_printf(OSE_AGENT_NAME " started\n");
  }

  if (retPid) *retPid = pid;
  if (verMaj) *verMaj = OSE_AGENT_IF_VERSION_MAJOR;
  if (verMin) *verMin = OSE_AGENT_IF_VERSION_MINOR;
  return 0;
}

static
long installBiosStartAgent(void) {
  /* This function should be called by the kernel at startup. Save the bid of
   * the current block, which should be the main block. That bid is later used
   * by biosStartAgent() */
  return biosInstall(BIOS_FUNC_START_AGENT,
                     (BiosFunction*)biosStartAgent,
                     BIOS_USERMODE_ALLOWED);
}

/*
** A Coresys start hook, to create the ose_agent process in the system segment.
** We want to avoid creating it from user mode programs, that would require
** relaxed error checks (0xf).
**
** NOTE: Public ose_agent labels must not be refered from other core product 
** library objects. That would pull in unwanted objects in customer built
** monoliths!
*/
void
zzstart_ose_test_agent(void)
{
  PROCESS *retPid = 0;
  U32 *verMaj = 0;
  U32 *verMin = 0;

  /* Create the ose agent from a supervisor process in the monolith. */
  (void)biosStartAgent(retPid, verMaj, verMin);

  /* Make the ose agent available from loadmodules. */
  installBiosStartAgent();
}
