/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2001-2007 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/****************************************************************
 *   $workfile: lmparam.c $ 
 *   Document no: @(#)  $Revision: $
 *   $date: $
 *
 *   Purpose:     C language program file
 *   Description:
 *	This file implements the swunit-internal lmparam.h to read
 *	the parameters encoded in the variable ose_confd, (offline editable
 *	with the OSE Load Module Tools) found in the core module.
 *
 *
 *   REVISION HISTORY:
 *   Date:               Description:
 ****************************************************************
 */


/*
 ****************************************************************************
 * 3  INCLUDE FILES.
 ****************************************************************************
 */

#include "lmparam.h"
#include "ctype.h"
#include "malloc.h"
#include "sysparam.h"
#include "sysparam_err.h"
#include "assert.h"
#include "ose_conf.h"
#include "string.h"
#include "ose.h"
#include "osetypes.h"
#include "cpu.h"
#include "ramlog.h"

/*
 ****************************************************************************
 * 5  MANIFEST CONSTANTS.
 ****************************************************************************
 */

/*
 ****************************************************************************
 * 6  MACROS.
 ****************************************************************************
 */

#define COMMENT_MARK '#'

/* CF_CONF_MAX_NR_ENTRIES can be set through the command line */
#ifndef CF_CONF_MAX_NR_ENTRIES 
#define CF_CONF_MAX_NR_ENTRIES 256 /* allocates CF_CONF_MAX_NR_ENTRIES *
				      2 bytes of static */
#endif

/*
 ****************************************************************************
 * 7  TYPES.
 ****************************************************************************
 */
 

/*
 ****************************************************************************
 * 8  FUNCTION PROTOTYPES.
 ****************************************************************************
 */

/*
 ****************************************************************************
 * 10  VARIABLES.
 ****************************************************************************
 */

static Boolean initialised; /* Initialized to False by crt0. */

static U16 offset[CF_CONF_MAX_NR_ENTRIES];

/*
 ****************************************************************************
 * 11  FUNCTIONS.
 ****************************************************************************
 */

void ose_lmparam_restart(void);

void ose_lmparam_restart(void)
{
   initialised = False;
};

static int
cmp_str_lmp(const char *s, const char *lmp)
{
   char cs;
   char clm;
   do
   {
      cs  = *s++;
      clm = *lmp++;
   } while(cs == clm && clm != '\0');
   
   if (clm == '=')
   {
      clm = '\0';
   }
   return cs - clm;
}

static int
cmp_lmp_lmp(const char *p1, const char *p2)
{
   char c1;
   char c2;
   do
   {
      c1 = *p1++;
      c2 = *p2++;
      assert(c1 != '\0');
   } while (c1 == c2 && c1 != '=');
   if (c1 == '=')
   {
      c1 = '\0';
   }
   if (c2 == '=')
   {
      c2 = '\0';
   }
   return c1 - c2;
}



static size_t
get_lmp_value_len(const char *value)
{
   size_t i = 0;
   char c = value[i];
   while (c != '\0' && c != '\n')
   {
      i++;
      c = value[i];
   }
   return i;
}

static size_t
get_lmp_name_len(const char *name)
{
   const char *name_end = strchr(name, '=');
   size_t len = (size_t)(name_end - name);
   assert(name_end != NULL);
   return len;
}

static void
print_parameter_string(const char *ptr, int length)
{
   while (length--)
   {
      ramlog_printf("%c", *ptr++);
   }
}

/*
 * Calls to ramlog_printf is not done for the softkernels.
 * Softkernels reads the rtose5.conf file when started and 
 * it is not possible to call ramlog_printf at this point. 
 *  
 */
static int
check_initialise(void)
{
   static int nr_params; /* Initialized to 0 by crt0. */
   U32 mask;
   LOCK_SAVE(mask);
   if (!initialised)
   {
      /* Check the ose_confd magic. */
      if(strncmp(ose_confd, CF_CONF_MAGIC, CF_CONF_MAGIC_SIZE) == 0)
      {
         size_t size_name;
         size_t size_value;
         Boolean duplicate;
         Boolean inside_comment;
         Boolean remove_row;
         int min;
         int max;
         int i;
         const char *eol;
         const char *curr_name_end;
	 const char *curr = ose_confd + CF_CONF_MAGIC_SIZE;
	 const char *curr_char;

	 /* Search for the parameter name line by line until a null character
	  * is encountered.
	  */

         inside_comment = False;
	 while (*curr != '\0')
	 {
            /* Find end-of-line position */
	    eol = strchr(curr, '\n');

            /* Check if this is an empty row */
            if (curr == eol || eol == NULL)
            {
               goto next_row;
            }

            curr_char = curr;
            remove_row = False;
            /* Check for C/C++ comment and remove these from parsing */
            while(curr_char < eol)
            {
               if (inside_comment)
               {
                  if (*curr_char == '*')
                  {
                     curr_char++;
                     if (*curr_char == '/')
                     {
                        /* C comment end token detected */
                        inside_comment = False;
                        remove_row = True;
                     }
                  }
               }
               else
               {
                  if (*curr_char == '/')
                  {
                     if (*(curr_char + 1) == '/')
                     {
                        /* C++ comment, remove line */
                        goto next_row;
                     }
                     else if (*(curr_char + 1) == '*')
                     {
                        /* C comment start token detected */
                        inside_comment = True;
                        remove_row = True;
                        /* Continue to parse to look for end comment token on current row */
                     }
                     else
                     {
                        /* Not a comment, continue and parse row as a system parameter */
                        curr = curr_char;
                        break;
                     }
                  }
                  else
                  {
                     if (!isspace((unsigned char)*curr_char) && (!remove_row))
                     {
                        /* Not a comment, continue and parse row as a system parameter */
                        curr = curr_char;
                        break;
                     }
                  }
               }

               curr_char++;
            }

            /* If we are inside a comment or have decided to remove the line, do it */
            if (remove_row || inside_comment)
            {
               goto next_row;
            }
            
            /* Check if the row is commented */
	    if (*curr == COMMENT_MARK)
	    {
               goto next_row;
            }

            /* Check that the formating of the parameter is correct */
            curr_name_end = strchr(curr, '=');
            if (!((curr_name_end != NULL) &&
                   /* '=' on same line ? */
                   (eol > curr_name_end)))
            {
	       const char *cp;

	       /* If the line contains any non-space character, it should also contain a '=' */
	       for(cp = curr + 1; cp < eol; cp++)
	       {
		  if(!isspace((unsigned char)*cp))
		  {
#ifndef softOSE
                     ramlog_printf("WARNING! Ignoring parameter ");
                     print_parameter_string(curr, eol - curr);
                     ramlog_printf(", equal sign missing.\n");
#endif
		     break;
		  }
	       } /* for */

               goto next_row;
            }

            /* Check that the size of the parameter name is within range */
            size_name = (size_t)(curr_name_end - curr);
            if (size_name > SYSPARAM_MAX_NAME_LENGTH)
            {
#ifndef softOSE
               ramlog_printf("WARNING! Ignoring parameter ");
               print_parameter_string(curr, size_name);
               ramlog_printf(", too long name.\n");
#endif
               goto next_row;
            }
            
            /* Check that there are no whitespace around '=' and in the beginning of the row */
            if (isspace((unsigned char)*(curr_name_end - 1)))
            {
#ifndef softOSE
               ramlog_printf("WARNING! Ignoring parameter ");
               print_parameter_string(curr, size_name);
               ramlog_printf(", whitespace preceeding equal sign.\n");
#endif
               goto next_row;
            }

            if (isspace((unsigned char)*(curr_name_end + 1)))
            {
#ifndef softOSE
               ramlog_printf("WARNING! Ignoring parameter ");
               print_parameter_string(curr, size_name);
               ramlog_printf(", whitespace succeeding equal sign.\n");
#endif
               goto next_row;
            }

            /* Check that the size of the parameter value is within range */
            size_value = (size_t)(eol - curr_name_end);
            if (size_value >= SYSPARAM_MAX_VALUE_LENGTH)
            {
#ifndef softOSE
               ramlog_printf("WARNING! Ignoring parameter ");
               print_parameter_string(curr, size_name);
	       ramlog_printf(", too long value.\n");
#endif
               goto next_row;
            }
            
            /* Only parse the parameter if no error on the row found */
            duplicate = False;
            min = 0;
            max = nr_params - 1;
            i = (unsigned)max >> 1;
            
            while(max >= min)
            {
               int compare;
               compare = cmp_lmp_lmp(curr, ose_confd + offset[i]);
               if (compare == 0)
               {
                  /* Name found twice in LM-params, what to do? */
                  duplicate = True;
                  break;
               }
               else if (compare > 0)
               {
                  min = i + 1;
               }
               else
               {
                  max = i - 1;
               }
               i = (unsigned)(min + max) >> 1;
            }
            
            if (!duplicate)
            {
               int j;
               /* initialise offsets and nr_params */
               i = min;
               if (nr_params++ >= CF_CONF_MAX_NR_ENTRIES)
               {
#ifndef softOSE
		  ramlog_printf("WARNING! Max number of parameters reached. Ignoring parameter ");
                  print_parameter_string(curr, size_name);
                  ramlog_printf(" and afterwards.\n");
#endif
                  break;
               }
               /* Make room for inserting the new name by moving all
                * posterior names one step in the list.
                */
               for (j = nr_params - 1; j > i; j--)
               {
                  offset[j] = offset[j - 1];
               }
               /* Insert new entry */
               offset[i] = (U16)(curr - ose_confd);
            }


         next_row:
	    /* Loop for next line */
	    if (eol != NULL)
	    {
	       curr = eol + 1;  /* Skip newline */
	    }
	    else
	    {
	       break; /* end of conf area reached */
	    }
	 } /* while ( ... ) */
      }
      /* Else "ose_confd" did not begin with magic string, leave "nr_params"
       * equal to zero.
       */
      initialised = True;
   }
   /* Else already initalised */
   LOCK_RESTORE(mask);
   return nr_params;
}



static const char *
find_name(const char *name, Boolean list_next)
{
   const char *found = NULL;
   int nr_params = check_initialise();
   if (nr_params > 0)
   {
      static int latest_index; /* Initialized to 0 by crt0. */
      int i = latest_index;
      int min = 0;
      int max = nr_params - 1;
      
      assert(name != NULL);
      
      for (;;)
      {
	 const char *curr = ose_confd + offset[i];
	 int compare = cmp_str_lmp(name, curr);
	 if (compare == 0)
	 {
	    /* found exact match */
	    found = curr;
	    break; /* Exit loop. */
	 }
	 else if (compare > 0)
	 {
	    min = i + 1;
	 }
	 else
	 {
	    max = i - 1;
	 }

	 /* Prepare for next loop */
	 if (max >= min)
	 {
	    i = (unsigned)(min + max) >> 1;
	    /* Loop */
	 }
	 else
	 {
	    i = min; /* set "i" to the entry that would follow */
	    break; /* nothing found */
	 }
	 
      } /* for (;;) */

      if (list_next)
      {
	 if (found != NULL)
	 {
	    i++; /* increment "i" to point at subsequent */
	 }
	 /* Return the subsequent name. */
	 if (i < nr_params)
	 {
	    found = ose_confd + offset[i];
	 }
	 else /* Else end of table reached */
	 {
	    found = NULL;
	 }
      }

      /* Update static variable so we have a good start on next call to this
       * function
       */
      if (found != NULL)
      {
	 latest_index = i;
      }
   }
   /* Else failed */ 
   return found;
}



/*
 *===========================================================================
 *                      lmparam_read
 *===========================================================================
 * Description: Reads the value corresponding to a given parameter name. See
 *	lmparam.h for syntax and semantics description.
 * Side effects: None.
 *
 */
size_t
lmparam_read(const char *name, char *buffer, size_t bufsize)
{
   size_t value_len;
   const char *param = find_name(name, False);
   
   if (param != NULL)
   {
      size_t name_len = strlen(name);
      const char *value = param + name_len + 1;
      value_len = get_lmp_value_len(value);
      /* Store the value in the return buffer. */
      if (bufsize > 0)
      {
	 size_t cpy_sz = value_len < bufsize ? value_len : bufsize - 1;
	 assert(buffer != NULL);
	 memcpy(buffer, value, cpy_sz);
	 buffer[cpy_sz] = '\0';
      }
   }
   else
   {
      value_len = 0;
   }
   return value_len;
}

/*
 *===========================================================================
 *                      lmparam_list_next
 *===========================================================================
 * Description:
 *
 * Used to iterate through the parameter namespace.
 *
 */
size_t
lmparam_list_next(const char *str, char *buffer, size_t bufsize)
{
   size_t len;
   const char *param = find_name(str, True);
   if (param != NULL)
   {
      len = get_lmp_name_len(param);
      if (bufsize > 0)
      {
	 size_t cpy_sz = len < bufsize ? len : bufsize - 1;
	 assert(buffer != NULL);
	 memcpy(buffer, param, cpy_sz);
	 buffer[cpy_sz] = '\0';
      }
   }
   else
   {
      len = 0;
   }
   return len;
}
