/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2006 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

/**************************************************************************
 *
 *   Description:
 *	This file implements the core part of for the sysparam.h API.
 *	Its functions are accessible through trap calls and through
 *	the sysparam_early_-functions. It is basically a switcher to access
 *	the Core module parameters and the board parameters, through
 *	lmparam.h and boardparam.h, respectively. The board parameters
 *	have priority over the core module parameters.
 *
 **************************************************************************
 */

/*
 ****************************************************************************
 * 3  INCLUDE FILES.
 ****************************************************************************
 */

#include "assert.h"
#include "ose.h"
#include "boardparam.h"
#include "lmparam.h"
#include "bios.h"
#include "sysparam.h"
#include "sysparam_trap.h" /* OSE-internal interface */
#include "sysparam_err.h"
#include "string.h"
#include "compiler_macros.h"

#include "stddef.h"
#include "stdlib.h"
#include "osetypes.h"
#include "assert.h"
#ifndef softOSE
#include "ctype.h"
#undef tolower
#undef toupper
#undef isalnum
#undef isalpha
#undef iscntrl
#undef isdigit
#undef isgraph
#undef islower
#undef isprint
#undef ispunct
#undef isspace
#undef isupper
#undef isxdigit
#endif
#include "errno.h"
#include "efs.h"

static Boolean is_safe_mode; /* Initialized to False by crt0. */
size_t sysparam_handle_read(const char *name, char *buffer, size_t bufsize);


WEAK_REF(Boolean sysparam_translate_symbol(const char *symbol, long *value));

static const char *
skip_non_ws(const char *str)
{
   assert(str != NULL);
   while (*str != '\0' && !isspace(*str))
   {
      str++;
   }
   return str;
}

static const char *
skip_ws(const char *str)
{
   assert(str != NULL);
   while (*str != '\0' && isspace(*str))
   {
      str++;
   }
   return str;
}

static Boolean
ws_or_nul(char c)
{
   return c == '\0' || isspace(c);
}

static const char *
find_match(const char *param_value, const char *key, Boolean colon)
{
   const char *ptr;
   size_t key_len = strlen(key);
   
   ptr = skip_ws(param_value);
   while (*ptr != '\0')
   {
      if (strncmp(ptr, key, key_len) == 0)
      {
	 if (colon ? ptr[key_len] == ':' : ws_or_nul(ptr[key_len]))
	 {
	    return ptr + key_len + colon;
	 }
	 /* Else incomplete match, ignore and continue. */
      }
      /* Else no match, continue. */
      ptr = skip_ws(skip_non_ws(ptr));
   }
   /* Keyword not found. */
   return NULL;
}

static int
read_param(const char *param_name, char *buffer, size_t bufsize)
{
   size_t actual_len = sysparam_handle_read(param_name, buffer, bufsize);
   if (actual_len == 0) /* No such parameter. */
   {
      return ENOENT;
   }
   else if (actual_len < bufsize)
   {
      /* Successfully read complete parameter value. */
      assert(strlen(buffer) < bufsize);
      return EOK;
   }
   else /* Parameter value to big/buffer to small. */
   {
      return E2BIG;
   }
}


static int
sysparam_parse_long(const char *param_value, const char *key, long *value)
{
   int rc;
   const char *match = param_value;/*if key is NULL, try to parse param_value */;
   if (key != NULL)
      match = find_match(param_value, key, True);

   if (match != NULL)
   {
      long v;
      if (*match == '@')
      {
	 char  buffer[SYSPARAM_MAX_VALUE_LENGTH];
	 size_t size = skip_non_ws(match) - match - 1;
	 strncpy(buffer, match + 1, size);
	 buffer[size] = '\0';
	 if (sysparam_translate_symbol != NULL &&
             sysparam_translate_symbol(buffer, &v))
	 {
	    /* Only overwrite return variable when successful. */
	    *value = v;
	    rc = EOK;
	 }
	 else /* Else symbolic translation failed. */
	 {
	    rc = EINVAL;
	 }
      }
      else
      {
	 const char *end_ptr = NULL;
	 v = efs_str_to_long(match, &end_ptr);
	 /* Handle "4K", "2M" and "1G" values. */
	 if (end_ptr != NULL)
	 {
	    if (*end_ptr == 'K')
	    {
	       v <<= 10;
	       end_ptr++;
	    }
	    else if (*end_ptr == 'M')
	    {
	       v <<= 20;
	       end_ptr++;
	    }
	    else if (*end_ptr == 'G')
	    {
	       v <<= 30;
	       end_ptr++;
	    }
	 }
	 if (end_ptr == NULL || *end_ptr == '\0' || isspace(*end_ptr))
	 {
	    /* Only overwrite return variable when successful. */
	    *value = v;
	    rc = EOK;
	 }
	 else /* Else numeric translation failed. */
	 {
	    rc = EINVAL;
	 }
      }
   }
   else /* No such keyword/parameter. */
   {
      rc = ENOENT;
   }
   return rc;
}

static int
sysparam_parse_longlong(const char *param_value, const char *key,
                        long long int *value)
{
   int rc;
   const char *match = param_value;/*if key is NULL, try to parse param_value */
   if (key != NULL)
      match = find_match(param_value, key, True);
   if (match != NULL)
   {
      unsigned long long v;
      if (*match == '@')
      {
	 char  buffer[SYSPARAM_MAX_VALUE_LENGTH];
	 size_t size = skip_non_ws(match) - match - 1;
         unsigned long temp;
         
	 strncpy(buffer, match + 1, size);
	 buffer[size] = '\0';
	 if (sysparam_translate_symbol != NULL &&
             sysparam_translate_symbol(buffer, (long *) &temp))
	 {
	    /* Only overwrite return variable when successful. */
	    v = (unsigned long long)temp;
	    *value = v;
	    rc = EOK;
	 }
	 else /* Else symbolic translation failed. */
	 {
	    rc = EINVAL;
	 }
      }
      else
      {
	 const char *end_ptr = NULL;
	 v = efs_str_to_longlong(match, &end_ptr);

	 /* Handle "4K", "2M" and "1G" values. */
	 if (end_ptr != NULL)
	 {
	    if (*end_ptr == 'K')
	    {
	       v <<= 10;
	       end_ptr++;
	    }
	    else if (*end_ptr == 'M')
	    {
	       v <<= 20;
	       end_ptr++;
	    }
	    else if (*end_ptr == 'G')
	    {
	       v <<= 30;
	       end_ptr++;
	    }
	 }
	 if (end_ptr == NULL || *end_ptr == '\0' || isspace(*end_ptr))
	 {
	    /* Only overwrite return variable when successful. */
	    *value = v;
	    rc = EOK;
	 }
	 else /* Else numeric translation failed. */
	 {
	    rc = EINVAL;
	 }
      }
   }
   else /* No such keyword/parameter. */
   {
      rc = ENOENT;
   }
   return rc;
}


static Boolean
copy_string(char *buffer, size_t bufsize, const char *src)
{
   Boolean quote = False;
   for (;;)
   {
      char c = *src++;

      if (c == 0)
      {
         *buffer = '\0';
         return True; /* Buffer ok. */
      }
      else if (c == '"')
      {
	 quote = !quote;
      }
      else if (quote || !isspace(c))
      {
	 if (bufsize > 1)
	 {
	    *buffer++ = c;
	    bufsize--;
	 }
	 else /* No room for character in buffer. */
	 {
	    assert(bufsize == 1);
	    /* NUL-terminate for robustness. */
	    *buffer = '\0';
	    return True; /* True because buffer overwritten. */
	 }
      }
      else
      {
	 assert(bufsize >= 1);
	 *buffer = '\0';
	 return True; /* Buffer ok. */
      }
   }
}

static int
sysparam_parse_string(const char *param_value, const char *key, char *value,
		      size_t bufsize)
{
   int rc;
   const char *match = param_value;/*if key is NULL, try to parse param_value */
   if (key != NULL)
      match = find_match(param_value, key, True);

   if (match != NULL)
   {
      if (bufsize > 1)
      {
	 if (copy_string(value, bufsize, match))
	 {
	    rc = EOK;
	 }
	 else /* Else buffer too small. */
	 {
	    /* LMF: Buffer is overwritten. */
	    rc = ENOSPC;
	 }
      }
      else /* Buffer size must be larger than 1. */
      {
	 rc = EINVAL;
      }
   }
   else /* No such keyword/parameter. */
   {
      rc = ENOENT;
   }
   return rc;
}

static Boolean
translate_enum(const char *name, const struct enumeration *enumeration,
	       long *value)
{
   unsigned i;
   for (i = 0; enumeration[i].name != NULL; i++)
   {
      size_t len = strlen(enumeration[i].name);
      if (strncmp(enumeration[i].name, name, len) == 0 &&
	  ws_or_nul(name[len]))
      {
	 *value = enumeration[i].value;
	 return True;
      }
   }
   return False;
}

static int
sysparam_parse_enum(const char *param_value, const char *key,
		    const struct enumeration *enumeration, long *value)
{
   int rc;
   const char *match = param_value;/*if key is NULL, try to parse param_value */
   if (key != NULL)
      match = find_match(param_value, key, True);

   if (match != NULL)
   {
      if (translate_enum(match, enumeration, value))
      {
	 rc = EOK;
      }
      else /* Symbolic enumeration not found. */
      {
	 rc = EINVAL;
      }
   }
   else /* No such keyword/parameter. */
   {
      rc = ENOENT;
   }
   return rc;
}


/*
 *===========================================================================
 *
 *===========================================================================
 */

static int
handle_readattr_long(const char *param_name, const char *key,
		     long *value, char *buffer)
{
   int rc = read_param(param_name, buffer, SYSPARAM_MAX_VALUE_LENGTH);

   if (rc == EOK)
   {
      rc = sysparam_parse_long(buffer, key, value);
   }
   return rc;
}

int
sysparam_handle_readattr_long(const char *param_name, const char *key,
			      long *value)
{
   char buffer[SYSPARAM_MAX_VALUE_LENGTH];

   return handle_readattr_long(param_name, key, value, buffer);
}


/*
 *===========================================================================
 *
 *===========================================================================
 */

static int
handle_readattr_longlong(const char *param_name, const char *key,
			 long long int *value, char *buffer)
{
   int rc = read_param(param_name, buffer, SYSPARAM_MAX_VALUE_LENGTH);

   if (rc == EOK)
   {
      rc = sysparam_parse_longlong(buffer, key, value);
   }
   return rc;
}

int
sysparam_handle_readattr_longlong(const char *param_name, const char *key,
                                  long long int *value)
{
   char buffer[SYSPARAM_MAX_VALUE_LENGTH];

   return handle_readattr_longlong(param_name, key, value, buffer);
}

/*
 *===========================================================================
 *
 *===========================================================================
 */

static int
handle_readattr_string(const char *param_name,
		       const char *key,
		       char *value,
		       size_t bufsize,
		       char *buffer)
{
   int rc = read_param(param_name, buffer, SYSPARAM_MAX_VALUE_LENGTH);

   if (rc == EOK)
   {
      rc = sysparam_parse_string(buffer, key, value, bufsize);
   }
   return rc;
}

int
sysparam_handle_readattr_string(const char *param_name,
				const char *key,
				char *value,
				size_t bufsize)
{
   char buffer[SYSPARAM_MAX_VALUE_LENGTH];

   return handle_readattr_string(param_name, key, value, bufsize, buffer);
}


/*
 *===========================================================================
 *   
 *===========================================================================
 */

static int
handle_readattr_enum(const char *param_name,
		     const char *key,
		     const struct enumeration *enumeration,
		     long *value,
		     char *buffer)
{
   int rc = read_param(param_name, buffer, SYSPARAM_MAX_VALUE_LENGTH);

   if (rc == EOK)
   {
      rc = sysparam_parse_enum(buffer, key, enumeration, value);
   }
   return rc;
}

int
sysparam_handle_readattr_enum(const char *param_name,
			      const char *key,
			      const struct enumeration *enumeration,
			      long *value)
{
   char buffer[SYSPARAM_MAX_VALUE_LENGTH];

   return handle_readattr_enum(param_name, key, enumeration, value, buffer);
}


/*
 *===========================================================================
 *
 *===========================================================================
 */

size_t
sysparam_handle_read(const char *name, char *buffer, size_t bufsize)
{
   size_t retval = is_safe_mode ? 0 : boardparam_read(name, buffer, bufsize);
   if (retval == 0)
   {
      retval = lmparam_read(name, buffer, bufsize);
   }
   return retval;
}


/*
 *===========================================================================
 *                      sysparam_handle_list_next
 *===========================================================================
 * Used to iterate through the parameter namespace.
 */
static size_t
handle_list_next(const char *str, char *buffer, size_t buffer_size,
		 char *lm_tmp, char *bp_tmp)
{
   size_t ret_len;
   if (is_safe_mode)
   {
      ret_len = lmparam_list_next(str, buffer, buffer_size);
   }
   else
   {
      size_t lm_len = lmparam_list_next(str, lm_tmp, SYSPARAM_MAX_NAME_LENGTH);
      if (lm_len > 0)
      {
	 const char *ret_buf;
	 size_t bp_len =
	    boardparam_list_next(str, bp_tmp, SYSPARAM_MAX_NAME_LENGTH);
	 assert(lm_len < SYSPARAM_MAX_NAME_LENGTH);
	 assert(bp_len < SYSPARAM_MAX_NAME_LENGTH);
	 if (bp_len > 0)
	 {
	    if (strcmp(lm_tmp, bp_tmp) < 0)
	    {
	       ret_buf = lm_tmp;
	       ret_len = lm_len;
	    }
	    else
	    {
	       ret_buf = bp_tmp;
	       ret_len = bp_len;
	    }
	 }
	 else
	 {
	    ret_buf = lm_tmp;
	    ret_len = lm_len;
	 }
	 if (buffer_size > 0)
	 {
	    size_t cpy_sz = ret_len < buffer_size ? ret_len : buffer_size - 1;
	    assert(buffer != NULL);
	    memcpy(buffer, ret_buf, cpy_sz);
	    buffer[cpy_sz] = '\0';
	 }
      }
      else
      {
	 ret_len = boardparam_list_next(str, buffer, buffer_size);
      }
   }
   return ret_len;
}

size_t
sysparam_handle_list_next(const char *str, char *buffer, size_t buffer_size)
{
   char lm_tmp[SYSPARAM_MAX_NAME_LENGTH];
   char bp_tmp[SYSPARAM_MAX_NAME_LENGTH];

   return handle_list_next(str, buffer, buffer_size, lm_tmp, bp_tmp);
}

/*
 *===========================================================================
 *                      sysparam_handle_write
 *===========================================================================
 * Description:
 *
 * Used to write a system parameter to persistent storage.
 *
 */
int
sysparam_handle_write(const char *name, const char *value)
{
   return boardparam_write(name, value);
}


/*
 *===========================================================================
 *                      sysparam_enter_safe_mode
 *===========================================================================
 * Description:
 *
 * Called by system start-up code before any other call to this module
 * if only LM-parameters are to be read, and board-parameters ignored.
 * This can be wanted for example if the system has entered cyclic reboot
 * because of a fatal configuration error in a board parameter.
 */
void
sysparam_enter_safe_mode(void)
{
   is_safe_mode = True;
}

Boolean
sysparam_set_safe_mode(Boolean new_mode)
{
   Boolean old_mode = is_safe_mode;
   is_safe_mode = new_mode;
   return old_mode;
}

/*
 *===========================================================================
 *                      sysparam_trap_handler
 *===========================================================================
 * Description: Alloc buffers from pool to avoid too much supervisor stack
 *              usage.
 */
static long
sysparam_trap_handler(long fc_, long a, long b, long c, long d, long e, long f)
{
   enum SysparamBiosCode fc = (enum SysparamBiosCode) fc_;
   char *buf;
   int rc;
   (void) e;
   (void) f;
   switch (fc)
   {
      case SYSPARAM_BIOS_READ:
	 return (long) sysparam_handle_read((const char *)a, (char *)b,
					    (size_t)c);
      case SYSPARAM_BIOS_LIST_NEXT:
	 buf = (char *)alloc(SYSPARAM_MAX_NAME_LENGTH * 2, 0);
	 rc = handle_list_next((const char *)a,
			       (char *)b,
			       (size_t)c,
			       buf,                             /* 1st half */
			       &buf[SYSPARAM_MAX_NAME_LENGTH]); /* 2nd half */
	 free_buf((union SIGNAL **)&buf);
	 return rc;

      case SYSPARAM_BIOS_WRITE:
	 return sysparam_handle_write((const char *)a,
				      (const char *)b);
      case SYSPARAM_BIOS_READATTR_LONG:
	 buf = (char *)alloc(SYSPARAM_MAX_VALUE_LENGTH, 0);
	 rc = handle_readattr_long((const char *)a,
				   (const char *)b,
				   (long *)c,
				   buf);
	 free_buf((union SIGNAL **)&buf);
	 return rc;

      case SYSPARAM_BIOS_READATTR_LONGLONG:
	 buf = (char *)alloc(SYSPARAM_MAX_VALUE_LENGTH, 0);
	 rc = handle_readattr_longlong((const char *)a,
				       (const char *)b,
				       (long long int *)c,
				       buf);
	 free_buf((union SIGNAL **)&buf);
	 return rc;

      case SYSPARAM_BIOS_READATTR_STRING:
	 buf = (char *)alloc(SYSPARAM_MAX_VALUE_LENGTH, 0);
	 rc = handle_readattr_string((const char *)a,
				     (const char *)b,
				     (char *)c,
				     (size_t)d,
				     buf);
	 free_buf((union SIGNAL **)&buf);
	 return rc;

      case SYSPARAM_BIOS_READATTR_ENUM:
	 buf = (char *)alloc(SYSPARAM_MAX_VALUE_LENGTH, 0);
	 rc = handle_readattr_enum((const char *)a,
				   (const char *)b,
				   (const struct enumeration *)c,
				   (long *)d,
				   buf);
	 free_buf((union SIGNAL **)&buf);
	 return rc;

      default:
	 error2(OSE_SYSPARAM_EUNKNOWN_BIOS_CODE, (OSERRCODE)fc);
	 kill_proc(current_process());
	 return 0; /* make compiler happy */
   }
}




/*
** This initiation function is called by the kernel start sequence in OSE5.1.
*/
void
zzsysparam_bios_install(void)
{
   long result = biosInstall(BIOSNAME_SYSPARAM,
			     sysparam_trap_handler,
			     BIOS_USERMODE_ALLOWED);
   if (result != BIOS_SUCCESS)
   {
      error2(OSE_SYSPARAM_EBIOSINSTALL_FAILED, (OSERRCODE)result);
   }
}

/* 
** This was a spi interface function in OSE5.0. Now a dummy. 
** Keep it a while to be backwards compatible.  
*/
void
sysparam_bios_install(void);

void
sysparam_bios_install(void)
{
}
