/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2009 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

/**
 *  @File prof.h
 *  @brief This is the structures and internal interface of the
 *   profiling for load balancing
 *
 */

#ifndef PROF_H_
#define PROF_H_


#include "ose.h"
#include "monitor_cpu.sig"
#include "pose.h"
#include "prof_timestamp.h"
#include "prof_hash.h"

#define RMM_PROF_PRI 0


typedef enum ProfType
{
   CPU_OVERVIEW_PROF = 0,
   PRIORITY_PROF = 1,
   PROCESS_PROF = 2,
   REPORT_PROF = 3,
   BLOCK_PROF = 4,
   TOTAL_CPU_BLOCK_PROF = 5
} ProfType;

typedef enum prof_status {
	PROF_STATUS_DISABLED = 0,
	PROF_STATUS_ENABLED = 1,
	PROF_STATUS_STOPPING = 2
} ProfStatus;


#define PROF_NEW_SAMPLE (35950) /* !-SIGNO(struct ProfNewSample)-! */
#define PROF_CORE_FREE_BUF (35961) /* !-SIGNO(struct ProfCoreFreeBuf)-! */


typedef struct ProfNewSample
{
   SIGSELECT       sig_no;
   ProfType        type;
   SMP_CODE(   cpuid_t cpu; )
} ProfNewSample;


#ifdef OSE_SMP
typedef struct ProfCoreFreeBuf
{
	   SIGSELECT       sig_no;
	   cpuid_t cpu;
} ProfCoreFreeBuf;
#endif
/************************************************/
/*  SAMPLES                                     */
/************************************************/
/* The structure of each sample in the history list. */
struct prof_pri_sample
{
   struct prof_pri_sample *next;        /* Link in circular list. */
   OSTICK          time_ticks;  /* Start of sample interval, ticks. */
   OSTICK          time_uticks; /* Start of sample interval, microseconds. */

   uint32_t        total_time;  /* Accumulated length of the interval. */
   uint32_t        valid;       /* Zero at the end of the list of valid samples. */
   uint32_t        samples[35]; /* One counter for each pri[32], bgd, idle and int. */
};


/* The structure of each sample in the history list. */
struct prof_cpu_sample
{
   struct prof_cpu_sample *next;        /* Link in circular list. */
   OSTICK          time_ticks;  /* Start of sample interval, ticks. */
   OSTICK          time_uticks; /* Start of sample interval, microseconds. */

   uint32_t        total_time;  /* Accumulated length of the interval. */
   uint32_t        valid;       /* Zero at the end of the list of valid samples. */
   uint32_t        samples;     /* Accumulated load. */
};

struct prof_proc_sample
{
   struct prof_proc_sample *next;       /* Link in circular list. */
   OSTICK         time_ticks;  /* Start of sample interval, ticks. */
   OSTICK        time_uticks; /* Start of sample interval, microseconds. */

   uint32_t        total_time;  /* Accumulated length of the interval. */
   uint32_t        valid;       /* Zero at the end of the list of valid samples. */
   uint32_t        *pids;       /* Pointer to an array of size prof_proc_maxpid */
   uint32_t        *samples;    /* Pointer to an array of size prof_proc_maxpid */
   uint32_t        num_procs;   /* The number of processes measured this sample */
#ifdef OSE_SMP
   uint32_t        rp_count;    /* The number of active report points during this sample */
#endif
   /*
   ** We will use an extra field to count cpu load for all other processes.
   ** The arrays therefore contains prof_proc_maxpid + 1 elements.
   */
};


/* The structure of each sample in the history list. */
struct prof_tcb_sample
{
   struct prof_tcb_sample *next;       /* Link in circular list. */
   OSTICK        time_ticks;  /* Start of sample interval, ticks. */
   OSTICK        time_uticks; /* Start of sample interval, microseconds. */

   uint32_t        total_time;  /* Accumulated length of the interval. */
   uint32_t        valid;       /* Zero at the end of the list of valid samples. */
   uint32_t        *pids;       /* Pointer to an array of size prof_total_cpu_block_maxpid */
   uint32_t        *samples;    /* Pointer to an array of size prof_total_cpu_block_maxpid */
   uint32_t        num_procs;   /* The number of processes measured this sample */
   /*
    ** We will use an extra field to count cpu load for all other processes.
    ** The arrays therefore contains prof_total_cpu_block_maxpid + 1 elements.
    */
};

/************************************************/
/*  CORE SPECIFIC DATA                          */
/************************************************/

struct prof_tcb_core
{
   OSPRIORITY pri_threshold;
   OSBOOLEAN measure;
   volatile ProfStatus core_enabled;


   /* we put the time in the running block */
   volatile uint32_t **bcb_profs;
   PROCESS *bids;
   volatile uint32_t *running_valp;
   PROCESS running_bid;

   /* next free slot */
   uint32_t next;
   /* list of bids that have been used during the interval (so far) */
   MonitorCPUProcess *slots;

   /* max number of slots that are allocated, and thus, can be used */
   uint32_t max_alloc;

   /* pre allocated sample to put the data in the end of the interval */
   MonitorCPUProcessReport *sample; /* per core data sample put here */

   /*time spent in other processes than fit in the slots */
   uint32_t other;

   OSTICK core_interval;

   /* replace tb_data and / ticks uticks */
   struct prof_timestamp timestamp;

   /* replaces tb_interval_start  and interval_start_ticks/uticks */
   struct prof_timestamp interval_start;

   /* to set interval start */
   OSTICK interval_start_ticks;
   OSTICK interval_start_uticks;

   struct debug_pcb *prof_tcb_core_pcb;

#ifdef SAVING_DATA_FOR_MERGE
   uint32_t OSE_block_last_add;
#endif

   uint32_t maxpid;
   OSBOOLEAN unsafe;
};

struct prof_pri_core
{
	volatile ProfStatus prof_pri_core_enabled;
  /* priority prof */
   struct prof_pri_sample prof_pri_sample;
   /* Time of swap in and priority of the currently running process. */
   uint32_t prof_pri_oldpri;
   struct prof_timestamp prof_pri_b_timestamp;

   /* interval start */
   struct prof_timestamp prof_pri_interval_start;

   /* The sample history list along with head and tail pointers. */
   struct prof_pri_sample *prof_pri_samplepool;
   struct prof_pri_sample *prof_pri_firstsample;
   struct prof_pri_sample *prof_pri_cursample;


   OSTICK prof_pri_core_interval;

   SMP_CODE(spinlock_t prof_pri_core_data_lock;)
};

struct prof_cpu_core
{
	 /* cpu profiling */
   volatile ProfStatus prof_cpu_core_enabled;
   OSBOOLEAN prof_cpu_core_issampled;
   OSPRIORITY prof_cpu_core_pri;
   OSTICK prof_cpu_core_interval;

   /* The sample  */
   struct prof_cpu_sample prof_cpu_sample;
   /* Time of swap in and high priority flag of the currently running process. */
   uint32_t prof_cpu_oldpri;        /* nonzero if higher than pri 23 or interrupt. */
   struct prof_timestamp prof_cpu_b_timestamp;

   /* interval start */
   struct prof_timestamp prof_cpu_interval_start;

   /* The sample history list along with head and tail pointers. */
   struct prof_cpu_sample *prof_cpu_samplepool;
   struct prof_cpu_sample *prof_cpu_firstsample;
   struct prof_cpu_sample *prof_cpu_cursample;

   SMP_CODE(spinlock_t prof_cpu_core_data_lock;)
};

struct prof_proc_core
{
   /* duplicated fields has core in their names */
   volatile ProfStatus prof_proc_core_enabled;
   OSPRIORITY prof_proc_core_pri;
   /*Avoid false sharing by putting proc_prof_ and sample counter in the core data */
         /* this should mirrot the  zzprof.proc[x].prof_proc_maxid*/
         /* Number of pid entries in each sample. */
   uint32_t prof_proc_core_maxpid;
   U32 prof_proc_core_sys_process;
   OSTICK prof_proc_core_interval;
  /* OSBOOLEAN prof_proc_core_common_hash; HCHANGE*/

   /* set to 0 for process and 1 for block at init */
   OSBOOLEAN block;

   /* if we use a global hash function and must wait for core 0 to update profiled processes */
   volatile uint32_t hash_sync_counter;

   /* proc and block prof */
   /* The sample */
   struct prof_proc_sample prof_proc_sample;
   /* Time of swap in of the currently running process. */
   struct prof_timestamp prof_proc_b_timestamp;
   struct prof_timestamp prof_proc_interval_start;

   /* The pid of the currently running process or zero if not an interesting one. */
   uint32_t prof_proc_oldpid;
   /*
   ** The index into the sample array where statistics for the prof_proc_oldpid is stored.
   ** Valid only when prof_proc_oldpid is non zero.
   */
   uint32_t      prof_proc_oldpindex;

   struct prof_hash local_hash;

   /* The sample history list along with head and tail pointers. */
   struct prof_proc_sample *prof_proc_samplepool;
   struct prof_proc_sample *prof_proc_firstsample;
   struct prof_proc_sample *prof_proc_cursample;

   SMP_CODE(spinlock_t prof_proc_core_data_lock;)

};


/************************************************/
/*  THE CORE DATA STRUCT                        */
/************************************************/

/* We expect that the cache lines are 64 bytes long */
#define HALF_EXPECTED_CACHE_LINE 16
#define EXPECTED_CACHE_LINE 32

/* This is a struct that each core use to save it ongoing profiling.
 * The struct is placed in an SMP_ARRAY call prof_core_data_entry.
 * By co-locate the data we win cache locality and bring structure to the profiling
 * The rmm (ose_monitor) should never write in this structure (except from initiation)
 * When the interval is closed the data is copied.
 */

struct prof_core_data
{
   SMP_CODE(char pre_alignment[HALF_EXPECTED_CACHE_LINE];)
   volatile OSBOOLEAN core_active;  /* core is not closed down by hypervisor */
   struct prof_pri_core prof_pri_data;
   struct prof_cpu_core prof_cpu_data;
   struct prof_proc_core prof_proc_data[2];
   struct prof_tcb_core prof_tcb_data;
   SMP_CODE(char post_alignment[HALF_EXPECTED_CACHE_LINE];)
};


/************************************************/
/*  NON CORE_SPECIFIC DATA                      */
/************************************************/

struct prof_tcb
{
   volatile OSBOOLEAN enabled;

   OSTICK interval;
   uint32_t reports;
   uint32_t maxpid;
   uint32_t oversize;

   /* Number of ticks until automatic sampling disable. */
   uint32_t disctimer;

   /* samples are central */
   struct prof_tcb_sample *samplepool;
   struct prof_tcb_sample *firstsample;
   struct prof_tcb_sample *cursample;

   OSPRIORITY pri_threshold;

   uint32_t default_interval;
   uint32_t default_reports;
   uint32_t default_priority;
   uint32_t default_maxblocks;
   uint32_t default_oversize;

   union SIGNAL *prof_tcb_reqlist;
};

struct prof_cpu
{
   /* Number of samples in the profiler history list. */
    uint32_t prof_cpu_reports;

    /* Length of sample period in ticks. */
    OSTICK prof_cpu_interval;

    /* Threshold for cpu measurements */
    uint32_t prof_cpu_pri;

    volatile OSBOOLEAN prof_cpu_enabled;

    /* Number of ticks until automatic sampling disable. */
    uint32_t prof_cpu_disctimer;

    /*
    ** This flag is set when a sampled cpu load algorithm is used.
    ** It is zero to use the heavier swap-based algorithm.
    **
    ** This flag is set by a start hook 2 linked to the kernel.
    ** It can not be changed at any other time.
    */
    uint32_t prof_cpu_issampled;


    OSTICK default_cpu_interval;
    uint32_t default_cpu_reports;
    uint32_t default_cpu_priority;

    /* Linked list of pending load report requests. */
    union SIGNAL *prof_cpu_reqlist;
};

struct prof_pri
{
   /* Number of samples in the profiler history list. */
    uint32_t prof_pri_reports;

    /* Length of sample period in ticks. */
    OSTICK prof_pri_interval;

    /* Number of ticks left in the current interval. Zero if sampling disabled. */
    volatile uint32_t prof_pri_enabled;

    /* Number of ticks until automatic sampling disable. */
    uint32_t prof_pri_disctimer;

    uint32_t default_pri_interval;
    uint32_t default_pri_reports;

    union SIGNAL *prof_pri_reqlist;
};

struct prof_proc
{
   /* Threshold for cpu program measurements */
   OSPRIORITY prof_proc_pri;

   /* Number of samples in the profiler history list. */
    uint32_t prof_proc_reports;

    /* Number of pid entries in each sample. */
    uint32_t prof_proc_maxpid;

    /* The structure of each sample in the history list. */

    /* Length of sample period in ticks. */
    OSTICK prof_proc_interval;

    /* Number of ticks left in the current interval. Zero if sampling disabled. */
    volatile uint32_t prof_proc_enabled;

    /* Number of ticks until automatic sampling disable. */
    uint32_t prof_proc_disctimer;

    OSTICK default_proc_interval;
    uint32_t default_proc_reports;
    uint32_t default_proc_maxprocs;


    /* Linked list of pending load report requests. */
    union SIGNAL *prof_proc_reqlist;

    SMP_CODE(OSBOOLEAN prof_proc_common_hash;)
    SMP_CODE(struct prof_hash global_hash;)

    /* if we use a global hash function and must wait for core 0 to update profiled processes */
    volatile uint32_t master_hash_sync_counter;

     /*
     ** This is the currently active signal which contains the wildcard list
     ** for per process profiling.
     ** This pointer is initially NULL, but contains a valid signal pointer
     ** whenever a set status request has been received.
     */
     volatile uint32_t has_prof_proc_spec;
     MonitorCPUProcessReportpointInfo **prof_proc_spec;
#ifdef OSE_SMP
     uint32_t rp_count; /* The number of active report points */
#endif
     /* centralized list of prof proc */
        /* One pid field for each of the 8 prerequested processes. Extra = NULL. */
     uint32_t *prof_proc_pids;
     SMP_CODE(spinlock_t prof_proc_lock;)
};


/************************************************/
/*  THE FULL PROFILER WORLD                     */
/************************************************/

struct prof_world
{
   /*----------------------------------------------------------------*/

   /* The minimum number of microseconds that will be counted as the
    * delta time for process execution time.
    */
   uint32_t prof_all_min_us;

   uint32_t prof_all_num_cpus;  /* Number of CPUs */

   /* Some variables for sampled global cpu profiling is still missing. @@?? */


   /* System tick length in microseconds. */
   uint32_t prof_all_ticklen;

   /* Maximum interval in ticks. */
   uint32_t prof_all_max_interval;

   /* This is core specific data that should not be mixed with the rmm data*/
   SMP_CODE(char pre_alignment_dummy[HALF_EXPECTED_CACHE_LINE];)
   struct prof_core_data prof_core_data_entry SMP_ARRAY;
   SMP_CODE(char post_alignment_dummy[HALF_EXPECTED_CACHE_LINE];)


   struct prof_cpu cpu;
   struct prof_pri pri;
   struct prof_proc proc[2];
   struct prof_tcb  tcb;

};

extern struct prof_world zzprof;

/************************************************/
/*  THE PROCESSES             FIXME extern??    */
/************************************************/

PROCESS _zzrmm_core_pids SMP_ARRAY;
PROCESS _zzrmm_merge_pid;


/* used from the outside */
void zzprof_dbghook(OSPRIORITY pri, struct debug_pcb *proc);

/* used also in prof_timestamp.c */
OSTIME zzprof_get_diff(OSTICK bticks, OSTICK buticks, OSTICK eticks, OSTICK euticks);


/* "external" functions i.e. called from rmm.c which share object file */

static int zzprof_handle_get_cpu_reports_enabled(Offspring *offspring, union SIGNAL *sig);          /* [250] MONITOR_GET_CPU_REPORTS_ENABLED_REQUEST (39250) */
static int zzprof_handle_set_cpu_reports_enabled(Offspring *offspring, union SIGNAL *sig);          /* [252] MONITOR_SET_CPU_REPORTS_ENABLED_REQUEST (39252) */
static int zzprof_handle_get_cpu_pri_reports_enabled(Offspring *offspring, union SIGNAL *sig);      /* [254] MONITOR_GET_CPU_PRI_REPORTS_ENABLED_REQUEST (39254) */
static int zzprof_handle_set_cpu_pri_reports_enabled(Offspring *offspring, union SIGNAL *sig);      /* [256] MONITOR_SET_CPU_PRI_REPORTS_ENABLED_REQUEST (39256) */
static int zzprof_handle_get_cpu_process_reports_enabled(Offspring *offspring, union SIGNAL *sig);  /* [258] MONITOR_GET_CPU_PROCESS_REPORTS_ENABLED_REQUEST (39258) */
static int zzprof_handle_set_cpu_process_reports_enabled(Offspring *offspring, union SIGNAL *sig);  /* [260] MONITOR_SET_CPU_PROCESS_REPORTS_ENABLED_REQUEST (39260) */
static int zzprof_handle_set_cpu_process_reportpoint(Offspring *offspring, union SIGNAL *sig);      /* [262] MONITOR_SET_CPU_PROCESS_REPORTPOINT_REQUEST (39262) */
static int zzprof_handle_clear_cpu_process_reportpoint(Offspring *offspring, union SIGNAL *sig);    /* [264] MONITOR_CLEAR_CPU_PROCESS_REPORTPOINT_REQUEST (39264) */
static int zzprof_handle_get_cpu_process_reportpoint_info(Offspring *offspring, union SIGNAL *sig); /* [266] MONITOR_GET_CPU_PROCESS_REPORTPOINT_INFO_REQUEST (39266) */
static int zzprof_handle_get_cpu_reports(Offspring *offspring, union SIGNAL *sig);                  /* [270] MONITOR_GET_CPU_REPORTS_REQUEST (39270) */
static int zzprof_handle_get_cpu_pri_reports(Offspring *offspring, union SIGNAL *sig);              /* [272] MONITOR_GET_CPU_PRI_REPORTS_REQUEST (39272) */
static int zzprof_handle_get_cpu_process_reports(Offspring *offspring, union SIGNAL *sig);          /* [274] MONITOR_GET_CPU_PROCESS_REPORTS_REQUEST (39274) */
static int zzprof_handle_get_cpu_block_reports(Offspring *offspring, union SIGNAL *sig);            /* [276] MONITOR_GET_CPU_BLOCK_REPORTS_REQUEST (39276) */
static int zzprof_handle_get_total_cpu_block_reports_enabled(Offspring *offspring, union SIGNAL *sig); /* [284] MONITOR_SET_TOTAL_CPU_BLOCK_REPORTS_ENABLED_REQUEST (39284) */
static int zzprof_handle_set_total_cpu_block_reports_enabled(Offspring *offspring, union SIGNAL *sig); /* [286] MONITOR_SET_TOTAL_CPU_BLOCK_REPORTS_ENABLED_REQUEST (39286) */
static int zzprof_handle_get_total_cpu_block_reports(Offspring *offspring, union SIGNAL *sig); /* [288]  MONITOR_GET_TOTAL_CPU_BLOCK_REPORTS_REQUEST (39288) */
static int zzprof_handle_get_cpu_process_reports_enabled(Offspring *offspring, union SIGNAL *sig); /* [290] MONITOR_GET_CPU_BLOCK_REPORTS_ENABLED_REQUEST (39290) */
static int zzprof_handle_set_cpu_process_reports_enabled(Offspring *offspring, union SIGNAL *sig); /* [292] MONITOR_SET_CPU_BLOCK_REPORTS_ENABLED_REQUEST (39292) */
static void zzprof_handle_new_sample(int type);

static void zzprof_remove_from_profiled_processes(struct debug_pcb *proc);
static void zzprof_add_to_profiled_processes(struct debug_pcb *proc);

static void zzprof_disconnect(void);

#ifdef OSE_SMP

static void prof_core_suicide(struct prof_core_data *const core_data);

/* This function can be external if turn on cpus are implemented */
static uint32_t prof_set_core_status(cpuid_t core, OSBOOLEAN enable);

/* called when PROF_CORE_FREE_BUF signal arrives */
static void zzprof_handle_prof_core_free_buf(cpuid_t cpu);


/* This function external in the real sense */
void zzprof_disable_core(cpuid_t core);
void zzzprof_disable_core_free_buf(cpuid_t cpu);
OSBOOLEAN zzprof_get_core_status(cpuid_t core);
#endif

static uint32_t zzprof_get_interval(void);

#endif /* RT_PROF_H_ */
