/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2009 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

/**
 *  @File prof_hash.h
 *  @brief This is the implementation profiling processes hash table used in prof.c
 *
 */

#ifndef PROF_HASH_H_
#define PROF_HASH_H_


#include "ose.h"
#include "pose.h"
#ifdef OSE_SMP
#include "cpu_spinlock.h"
#endif
/*
** To quickly find a process in a process profiling sample a hash list
** is created, containing the processes pids and corresponding position
** in the current sample. It also contains a field 'sample' which tells
** if the process data corresponds to the current or an earlier sample.
** The hash table does not contain linked list for items hashing into the
** same position, instead items hashing into already filled positions are
** added to an overflow list.
*/
typedef struct PidHashItem
{
   uint32_t pid;       /* The PID that the index in this struct points at */
   uint32_t index;     /* The index in the sample process table */
   uint32_t sample;    /* Flag indicating the current sample */
} PidHashItem;

struct prof_hash {
   /*
    ** Index counter for the sample process list where new processes are
    ** added to as they are switched in for the first time during a new
    ** sample. This is the master list for the sample for which the hash
    ** table is used to find specific PIDs in.
    */
    uint32_t index_cnt;
    PidHashItem *pid_hash_table;
    uint32_t hash_list_size;
    uint32_t hash_list_mask;

      /*
      ** The overflow belonging to the hash table is an array containing the
      ** processes overflowing the hash list in the order they are started
      */
     PidHashItem *pid_overflow_list;
     uint32_t overflow_list_size;
     uint32_t overflow_list_pos_cnt;

      /*
      ** The sample counter is updated at every new process profiling sample
      ** started and tells if a sample in the process list is fresh
      */
      volatile uint32_t sample_counter;

      uint32_t hash_maxpid;
      SMP_CODE(spinlock_t hash_lock;)
};

OSBOOLEAN prof_hash_get_pid(uint32_t pid, uint32_t *index, struct prof_hash *data);
void prof_hash_add_pid(const uint32_t pid, uint32_t *index, struct prof_hash *data);
void prof_hash_init(struct prof_hash *data);
OSBOOLEAN prof_hash_create_proc_index_bufs(uint32_t procs, struct prof_hash *data);
void prof_hash_free_proc_index_bufs(struct prof_hash *data);
void prof_hash_reset(struct prof_hash *data);


/* end header file */
#endif /* PROF_HASH_H_ */
