/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2009 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#include "prof_timestamp.h"
#include "ramlog.h"


#ifdef HAS_TB_TIMER
#define WRAP_SAFE_DIFF(NEW,OLD) (((NEW >= OLD) ? (NEW - OLD) : (NEW + ~OLD))>> DIFF_SHIFT)

#ifdef READ_CPO_AS_TIMEBASE

#include "cpu_access.h"
#define PROF_READ_TIMER (read_cp0(CP0_COUNT,0))
#define DIFF_SHIFT 4

#else /* END READ_CP0_AS_TIMEBASE */

extern unsigned long long ose_read_tb(void);
#define PROF_READ_TIMER (ose_read_tb())
#define DIFF_SHIFT 0

#endif

#else /* END HAS_TB_TIMER */
extern OSTIME zzprof_get_delta_time(OSTICK *ticks, OSTICK *uticks);
extern OSTIME zzprof_get_diff(OSTICK bticks, OSTICK buticks, OSTICK eticks, OSTICK euticks);
#endif

extern OSTICK get_systime(OSTICK *n);

#define TRUE 1
#define FALSE 0


#ifdef __GNUCC__
inline
#endif
void prof_timestamp_set(struct prof_timestamp *timestamp)
{
#ifdef HAS_TB_TIMER
   timestamp->tb_time = (prof_timebase_t) PROF_READ_TIMER;
#else
   timestamp->ticks = get_systime(&timestamp->uticks);
#endif
}



#ifdef __GNUCC__
inline
#endif
uint32_t prof_timestamp_get_diff(struct prof_timestamp b, struct prof_timestamp e)
{
#ifdef HAS_TB_TIMER
   prof_timebase_t v = e.tb_time;
   prof_timebase_t old = b.tb_time;
   return (uint32_t) WRAP_SAFE_DIFF(v,old);
#else
   return (uint32_t) zzprof_get_diff(b.ticks,b.uticks,e.ticks,e.uticks);
#endif
}

#ifdef __GNUCC__
inline
#endif
void prof_timestamp_set_interval_start_ticks(OSTICK *ticks, OSTICK *uticks, struct prof_timestamp timestamp)
{
   (void)timestamp;
#ifdef HAS_TB_TIMER
   *ticks = get_systime(uticks);
#else
   *ticks =  timestamp.ticks;
   *uticks = timestamp.uticks;
#endif
}


#ifdef __GNUCC__
inline
#endif
void prof_timestamp_reset(struct prof_timestamp *t)
{
#ifdef HAS_TB_TIMER
   t->tb_time = 0;
#else
   t->ticks = 0;
   t->uticks =0;
#endif
}

void prof_timestamp_print(char *s, struct prof_timestamp t)
{
#ifdef HAS_TB_TIMER
   ramlog_printf("%s: %llu \n", s, t.tb_time);
#else
   ramlog_printf("%s: tick=%lu uticks=%lu", s, t.ticks, t.uticks);
#endif
}

static uint32_t prof_get_max_interval(void)
{
#ifdef HAS_TB_TIMER
   struct prof_timestamp ts0, ts1;
   OSTIME t0, ut0, t1, ut1;
   uint64_t v, d, td;

   /* get first timestamp */
   prof_timestamp_set(&ts0);
   t0 = get_systime(&ut0);

   /* get 1 tick difference in nticks */
   td = (uint64_t)zzprof_get_diff(t0, ut0, t0 + 1, ut0);

   delay(2);
   /* get second profiling timestamp */
   t1 = get_systime(&ut1);
   prof_timestamp_set(&ts1);

   d = (uint64_t)zzprof_get_diff(t0, ut0, t1, ut1);

   /* calculate the maximum ticks per interval
      steps_per_tick equals (timer_diff * td / d)
      the limit is ( 0xffffffff / (steps_per_tick * num_cpus))
    */
   v = (uint64_t)0xffffffff * d;
   v /= (uint64_t)prof_timestamp_get_diff(ts0, ts1) * td *
      (uint64_t)zzprof.prof_all_num_cpus;

   return (uint32_t)v;
#else
   return (OSTIME)0xffffffff /
      (zzprof.prof_all_ticklen * zzprof.prof_all_num_cpus);

#endif

}
