/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2010-2015 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifdef NDEBUG
#undef NDEBUG
#endif
#define ASSERT

#include "osenames.h"

#include "stdio.h"
#include "stdlib.h"
#include "stdint.h"
#include "stdbool.h"
#include "string.h"
#include "assert.h"

#include "ose.h"
#include "ose_spi.h"
#include "ose_err.h"
#include "rmm_err.h"
#include "ramlog.h"
#include "list.h"
#include "sysparam.h"
#include "efmi.h"
#include "compiler.h"
#include "ramdump.h"
#include "pose.con"
#include "pose.h"
#include "profiling_counters.h"


#include "monitor.sig"
#include "monitor_actions.sig"
#include "monitor_counters.sig"
#include "monitor_events.sig"
#include "monitor_execution.sig"
#include "monitor_dumps.sig"
#include "monitor_cpu.sig"
#include "heap.sig"
#include "heap_private.sig"
#include "mm.sig"
#include "rmm_internal.sig"
#include "rmm_tracepoint.h"
#include "rmm_utility.h"
/*
 * Number of replies requested, when using HEAP_BUFFER_INFO_REQUEST
 */
#define HEAP_BUFFER_INFO_BURST 10

union SIGNAL
{
   SIGSELECT sig_no;

   RmmAttSig att_sig;
   MonitorInterfaceReply interface_reply;
   MonitorNameReply name_reply;
   MonitorConnectRequest connect_request;
   MonitorConnectReply connect_reply;
   MonitorFlowControl flow_control;
   MonitorGetInfoRequest get_info_request;
   MonitorGetProcessInfoReply get_process_info_reply;
   MonitorGetProcessInfoLongReply get_process_info_long_reply;
   MonitorGetInfoEndmark get_info_endmark;
   MonitorGetBlockInfoReply get_block_info_reply;
   MonitorGetEnvVarsReply get_env_vars_reply;
   MonitorGetRegisterInfoRequest get_register_info_request;
   MonitorGetRegisterInfoEndmark get_register_info_endmark;
   MonitorGetStackUsageReply get_stack_usage_reply;
   MonitorGetSignalQueueReply get_signal_queue_reply;
   MonitorGetSignalQueueLongReply get_signal_queue_long_reply;
   MonitorGetPoolInfoReply get_pool_info_reply;
   MonitorGetPoolSignalsReply get_pool_signals_reply;
   MonitorGetSegmentInfoReply get_segment_info_reply;
   MonitorGetSegmentInfoLongReply get_segment_info_long_reply;
   MonitorGetConnectedClientsReply get_connected_clients_reply;
   MonitorGetConnectedClientsEndmark get_connected_clients_endmark;
   MonitorActionRequest action_request;
   MonitorActionReply action_reply;
   MonitorCreateProcessRequest create_process_request;
   MonitorCreateProcessReply create_process_reply;
   MonitorSetEnvVarRequest set_env_var_request;
   MonitorSetSysParamRequest set_sys_param_request;
   MonitorSendSignalRequest send_signal_request;
   MonitorSetPriorityRequest set_priority_request;
   MonitorSignalSemaphoreRequest signal_semaphore_request;
   MonitorGetSysParamReply get_sys_param_reply;
   MonitorSetExecutionUnitRequest set_execution_unit_request;

   MonitorRegisterIds register_ids;
   MonitorRegisterValues register_values;

   RmmOffspringInfo rmm_offspring_info;
   RmmGetOffspringsReply get_offsprings_reply;
   RmmGetTPConfInfoRequest get_tp_conf_info_request;
   RmmGetTPConfInfoReply get_tp_conf_info_reply;

#if 0
   /* Evact signals */
   MonitorSetNotifyEnabledRequest set_notify_enabled_request;
   MonitorSetNotifyEnabledReply set_notify_enabled_reply;
   MonitorSetTraceEnabledRequest set_trace_enabled_request;
   MonitorSetTraceEnabledReply set_trace_enabled_reply;
   MonitorSetInterceptEnabledRequest set_intercept_enabled_request;
   MonitorSetInterceptEnabledReply set_intercept_enabled_reply;
   MonitorGetNotifyEnabledRequest get_notify_enabled_request;
   MonitorGetNotifyEnabledReply get_notify_enabled_reply;
   MonitorGetTraceEnabledRequest get_trace_enabled_request;
   MonitorGetTraceEnabledReply get_trace_enabled_reply;
   MonitorGetInterceptEnabledRequest get_intercept_enabled_request;
   MonitorGetInterceptEnabledReply get_intercept_enabled_reply;
   MonitorSetEventStateRequest set_event_state_request;
   MonitorSetEventStateReply set_event_state_reply;
   MonitorGetEventStateRequest get_event_state_request;
   MonitorGetEventStateReply get_event_state_reply;
   MonitorEventActionpointInfo event_actionpoint_info;
   MonitorSetEventActionpointReply set_event_actionpoint_reply;
   MonitorClearEventActionpointRequest clear_event_actionpoint_request;
   MonitorClearEventActionpointReply clear_event_actionpoint_reply;
   MonitorGetEventActionpointsRequest get_event_actionpoints_request;
   MonitorGetEventActionpointsReply get_event_actionpoints_reply;
   MonitorGetEventActionpointsEndmark get_event_actionpoints_endmark;
   MonitorCreateEventStructRequest create_event_struct_request;

   MonitorSetScopeRequest set_scope_request;
   MonitorSetScopeReply set_scope_reply;
   MonitorProcessInfo process_info;
   MonitorSendReceiveInfo send_receive_info;
   MonitorSwapCreateKillInfo swap_create_kill_info;
   MonitorErrorInfo error_info;
   MonitorResetInfo reset_info;
   MonitorUserInfo user_info;
#ifdef OSE_SMP
   MonitorBindInfo bind_info;
#endif
   MonitorAllocFreeBufInfo alloc_free_buf_info;
#ifdef HAS_TIMEOUT_EVENT
   MonitorTimeoutInfo timeout_info;
#endif
   MonitorLossNotify loss_notify;

   /* Trace signals */
   MonitorGetTraceRequest get_trace_request;
   MonitorGetTraceMultipleRequest get_trace_mult_request;
   MonitorGetTraceEndmark get_trace_endmark;
   MonitorClearTraceRequest clear_trace_request;
   MonitorClearTraceReply clear_trace_reply;

   /* Debugging signals */
   MonitorAttachRequest attach_request;
   MonitorAttachReply attach_reply;
   MonitorDetachRequest detach_request;
   MonitorDetachReply detach_reply;

   MonitorInterceptScopeRequest intercept_scope_request;
   MonitorInterceptScopeReply intercept_scope_reply;
   MonitorRegisterIds register_ids;
   MonitorRegisterValues register_values;
#endif
   MonitorGetMemoryRequest get_memory_request;
   MonitorGetMemoryReply get_memory_reply;
   MonitorGetMemoryEndmark get_memory_endmark;
#if 0
   MonitorSetMemoryRequest set_memory_request;
   MonitorSetMemoryReply set_memory_reply;
   MonitorStepProcessRequest step_process_request;
   MonitorStepProcessNotify step_process_notify;
   MonitorSetBreakpointRequest set_breakpoint_request;
   MonitorSetBreakpointReply set_breakpoint_reply;
   MonitorResumeScopeRequest resume_scope_request;
   MonitorResumeScopeReply resume_scope_reply;
   MonitorBreakpointFiredNotify breakpoint_fired_notify;
   MonitorClearBreakpointRequest clear_breakpoint_request;
   MonitorClearBreakpointReply clear_breakpoint_reply;
   MonitorBasicEventActionRequest basic_event_action_request;
   MonitorBasicEventActionReply basic_event_action_reply;
   MonitorBasicErrorNotify basic_error_notify;
   MonitorBasicCreateKillNotify basic_create_kill_notify;
   MonitorBasicLossNotify basic_loss_notify;

   /* Profiling signals. */
   MonitorSetCPUReportsEnabledRequest set_cpu_reports_enabled_request;
   MonitorSetCPUReportsEnabledReply set_cpu_reports_enabled_reply;
   MonitorGetCPUReportsEnabledRequest get_cpu_reports_enabled_request;
   MonitorGetCPUReportsEnabledReply get_cpu_reports_enabled_reply;
   MonitorGetCPUPriReportsEnabledRequest get_cpu_pri_reports_enabled_request;
   MonitorGetCPUPriReportsEnabledReply get_cpu_pri_reports_enabled_reply;
   MonitorSetCPUPriReportsEnabledRequest set_cpu_pri_reports_enabled_request;
   MonitorSetCPUPriReportsEnabledReply set_cpu_pri_reports_enabled_reply;
   MonitorGetCPUProcessReportsEnabledRequest get_cpu_process_reports_enabled_request;
   MonitorGetCPUProcessReportsEnabledReply get_cpu_process_reports_enabled_reply;
   MonitorSetCPUProcessReportsEnabledRequest set_cpu_process_reports_enabled_request;
   MonitorSetCPUProcessReportsEnabledReply set_cpu_process_reports_enabled_reply;
   MonitorGetCPUReportsRequest get_cpu_reports_request;
   MonitorGetCPUReportsReply get_cpu_reports_reply;
   MonitorGetCPUPriReportsRequest get_cpu_pri_reports_request;
   MonitorGetCPUPriReportsReply get_cpu_pri_reports_reply;
   MonitorGetCPUProcessReportsRequest get_cpu_process_reports_request;
   MonitorGetCPUProcessReportsReply get_cpu_process_reports_reply;
   MonitorCPUProcessReportpointInfo cpu_process_reportpoint_info;
   MonitorSetCPUProcessReportpointReply set_cpu_process_reportpoint_reply;
   MonitorClearCPUProcessReportpointRequest clear_cpu_process_reportpoint_request;
   MonitorClearCPUProcessReportpointReply clear_cpu_process_reportpoint_reply;
   MonitorGetCPUProcessReportpointInfoRequest get_cpu_process_reportpoint_info_request;
   MonitorGetCPUProcessReportpointInfoEndmark get_cpu_process_reportpoint_info_endmark;

   MonitorSetUserReportsEnabledRequest set_user_reports_enabled_request;
   MonitorSetUserReportsEnabledReply set_user_reports_enabled_reply;

   MonitorGetUserReportsEnabledRequest get_user_reports_enabled_request;
   MonitorGetUserReportsEnabledReply get_user_reports_enabled_reply;

   MonitorGetUserReportsRequest get_user_reports_request;
   MonitorGetUserReportsReply get_user_reports_reply;

   MonitorCreateUserReportsStructRequest create_user_reports_struct_request;
   MonitorCreateUserReportsStructReply create_user_reports_struct_reply;

   UserReportsAttachSig user_report_attach;
#endif
   MonitorGetHeapInfoReply get_heap_info_reply;
   MonitorGetHeapFragmentInfoReply get_heap_frag_info_reply;
   MonitorGetHeapBuffersReply get_heap_buf_reply;

#ifdef OSE_SMP
   MonitorGetProfilingExecutionUnitEnabledRequest get_prof_eu_enabled_request;
   MonitorGetProfilingExecutionUnitEnabledReply get_prof_eu_enabled_reply;
   MonitorSetProfilingExecutionUnitEnabledRequest set_prof_eu_enabled_request;
   MonitorSetProfilingExecutionUnitEnabledReply set_prof_eu_enabled_reply;
#endif
   /* Source profiling signals. */
   MonitorGetSupportedCounterTypesRequest get_supported_counter_types_request;
   MonitorGetSupportedCounterTypesReply get_supported_counter_types_reply;
   MonitorGetSupportedCounterTypesEndmark get_supported_counter_types_endmark;

   /* Dump signals. */
   MonitorGetRamdumpInfoRequest get_ramdump_info_request;
   MonitorGetRamdumpInfoReply get_ramdump_info_reply;
   MonitorGetRamdumpInfoEndmark get_ramdump_info_endmark;
   MonitorGetRamdumpRequest get_ramdump_request;
   MonitorGetRamdumpReply get_ramdump_reply;
   MonitorGetRamdumpEndmark get_ramdump_endmark;
#if 0
   struct mm_get_domain_info mm_get_domain_info;
   /* Internal signals. */
   UserEventAttachSig user_event_attach_sig;

#ifdef USE_EPM
   RmmCallbackHit rmm_callback_hit;
   RmmScopeAttachSig rmm_scope_attach_sig;
#endif

   ProfNewSample prof_new_sample;

#ifdef OSE_SMP
   ProfCoreFreeBuf prof_core_free_buf;
#endif

   /* LDM event report signal, used by hooks */
   struct event_report_verbose event_report_verbose;

   struct mm_list_regions_reply mm_list_regions_reply;
   struct mm_list_regions_request mm_list_regions_request;
   struct mm_create_region mm_create_region;
#endif
   /* heap.sig interface, used by the Heap Browser */
   struct HeapListReply heap_list_rep;
   struct HeapStatsExReply heap_stats_rep;
   struct HeapConfigReply heap_config_rep;
   struct HeapFragsReply heap_frags_rep;
   struct HeapFragInfo heap_frag_info_rep;
   struct HeapBufferInfoReply heap_buf_info_rep;
};

/**
 * @type Client
 * @brief Hold information about a client.
 * @long Part of a linked list of clients.
 *
 * @field link list adm header.
 * @field magic contains CLIENT_MAGIC for assertion
 * @field pid clients pid.
 * @field attach_ref attach referens as returned by attach
 * @field flow_ctrl_block number of signals we can send to client without blocking
 * @field flow_ctrl_nblock number of signals we can send to client without throwing
 * @field application_scope the client's current application scope
 * @field execution_point the current execution point (if any)
 *
 */
typedef struct Client
{
   int magic_head;
   PROCESS pid;
   PROCESS rmm_;
   PROCESS mm_;
   OSATTREF attach_ref;
   int flow_ctrl_block;
   RmmOffspringInfo *offspring_info;
   int flow_ctrl_nblock;
   TracepointManagerInfo *tracepoint_manager_info;
#if 0
   MonitorScope debug_scope; /* Note: Supports only ID scope types */
   MonitorScope evact_scope;
   OSBOOLEAN intercept_enabled;
   OSATTREF scope_attach_ref;
   int keep_trace_actions;
   int handles_new_events;
   TraceEntryState cursor;

#ifdef USE_EPM
   List *execution_points;
   OSBOOLEAN scope_intercepted;
   uint32_t regdump_free_count;
   List *process_regdumps;
#endif

   struct idbuf notify_pids;

   int mget_tmo;
   union SIGNAL *mget_req;
   MonitorGetTraceMultipleReply *mget_reply;
   struct idbuf mtrace_pids;
   int mget_read;
   uint32_t mget_loss;
#endif

   int magic_tail;
} Client;

static const SIGSELECT all_sigs_filter[] =
{ 0 };
/* static const SIGSELECT no_nblock_sigs_filter[] = { -1, MONITOR_REPORT }; */

OSENTRYPOINT ose_rmm_client;

#define TOUCH(x) (void)x

static void
handle_get_tp_conf_request(Client *client, union SIGNAL **sig);

static void
tracepoint_not_initialized(union SIGNAL *sig, TracepointManagerInfo **tpm_info);

static void
init_tracepoint_manager_not_defined(TracepointManagerInfo **tpm_info,
                                    Client *client,
                                    int *flow_ctrl_block,
                                    int32_t flow_ctrl_blocking,
                                    PROCESS client_pid,
                                    PROCESS rmm_pid);
void
(*tp_handle)(union SIGNAL *sig, TracepointManagerInfo **tpm_info) =
   tracepoint_not_initialized;

void
(*tp_cleanup)(TracepointManagerInfo **tpm_info) = NULL;

void
(*init_tracepoint_manager)(TracepointManagerInfo **tpm_info,
                           Client *client,
                           int *flow_ctrl_block,
                           int32_t flow_ctrl_blocking,
                           PROCESS client_pid,
                           PROCESS rmm_pid) = init_tracepoint_manager_not_defined;

static OSADDRESS
ok_to_die_err_hnd(OSBOOLEAN user_called, OSERRCODE ecode, OSERRCODE extra)
{
   (void) extra; /* Not used */

   return (!user_called && (ecode & OSE_EERROR_MASK)
            == OSE_EKILLED_SYSTEM_DAEMON) ? 1 : 0;
}

extern void
(*tp_cleanup)(TracepointManagerInfo **tpm_info);

static void
suicide(Client *client)
{
   PROCESS curr = current_process();
   if (client && tp_cleanup) {
      tp_cleanup(&client->tracepoint_manager_info);
   }
   /* Make sure pose will let us commit suicide. */
   (void) create_error_handler(curr, (OSERRH *) ok_to_die_err_hnd, 200);

   kill_proc(curr);
}

#define WARN(fmt_str) ramlog_printf("rmm_offspring warn 0x%x (%u): ", current_process(), \
      __LINE__); ramlog_printf fmt_str
#define CHECK_WARN(expr, fmt_str) if (unlikely((expr))) WARN((fmt_str))

#define CHECK_ERROR(expr, fmt_str) do { \
   if (unlikely((expr))) { \
      WARN((fmt_str)); \
      suicide(NULL); \
   } \
} while(0)

static void
assert_client(Client *client)
{
   assert(client);
   assert(client->offspring_info);
   assert(get_ptype(client->pid) != OS_ILLEGAL);
   assert(get_ptype(client->rmm_) == OS_PRI_PROC);
}

/**
 * @function rmm_offspring_send2client
 * See rmm_utility.h for more info 
 */
void
zzrmm_send2client(union SIGNAL **signal, Client *client);

void
zzrmm_send2client(union SIGNAL **signal, Client *client)
{
   union SIGNAL *sig;
   static const SIGSELECT sel_allow_when_blocked[] =
   { 7,
     OS_ATTACH_SIG,
     MONITOR_BLOCKING_FLOW_CONTROL_REPLY,
     RMM_GET_TRACEPOINTS_REQUEST,
     RMM_GET_TRACEPOINT_ACTIONS_REQUEST,
     RMM_GET_TRACEPOINT_VARIABLES_REQUEST,
     RMM_GET_TRACEPOINT_ANNOTATIONS_REQUEST,
     RMM_GET_TP_CONF_INFO_REQUEST
   };

   assert(signal && *signal);
   assert_client(client);
   client->flow_ctrl_block--;

   if (unlikely(client->flow_ctrl_block
            == client->offspring_info->flow_ctrl_blocking / 2))
   {
      union SIGNAL *sig = alloc(sizeof(MonitorFlowControl),
                                MONITOR_BLOCKING_FLOW_CONTROL_REQUEST);
      send(&sig, client->pid);
   }

   while (unlikely(client->flow_ctrl_block <= 0))
   {
      sig = receive(sel_allow_when_blocked);
      switch (sig->sig_no)
      {
         case MONITOR_BLOCKING_FLOW_CONTROL_REPLY:
            if (sender(&sig) == client->pid)
            {
               client->flow_ctrl_block
                        += client->offspring_info->flow_ctrl_blocking / 2 + 1;
            }
            break;

         /* Following signals comes from another offspring that want tracepoint
          * settings from this offspring. These signals will be handled here, to
          * avoid that the communication between this offspring and its client
          * blocks the other offspring.
          */
         case RMM_GET_TRACEPOINTS_REQUEST:
         case RMM_GET_TRACEPOINT_ACTIONS_REQUEST:
         case RMM_GET_TRACEPOINT_VARIABLES_REQUEST:
         case RMM_GET_TRACEPOINT_ANNOTATIONS_REQUEST:
            tp_handle(sig, &(client->tracepoint_manager_info));
            break;
         
         case RMM_GET_TP_CONF_INFO_REQUEST:
            handle_get_tp_conf_request(client, &sig);
            break;

         case OS_ATTACH_SIG:
            suicide(client);
            break;

         default:
            assert(false);
            break;
      }
      free_buf(&sig);
   }
   send(signal, client->pid);
}

static inline bool
is_req_enabled(Client *client, union SIGNAL **signal)
{
   SIGSELECT signo = (*signal)->sig_no;

   /* FIXME what number range should we really check against? */
   if (signo > MONITOR_INTERFACE_REQUEST && signo < 40980)
   {
      uint32_t index = (signo - MONITOR_INTERFACE_REQUEST) / (8
               * sizeof(uint32_t));
      uint32_t bit = (signo - MONITOR_INTERFACE_REQUEST) % (8
               * sizeof(uint32_t));
      return (client->offspring_info->req_disabled_bitmask[index] & (1 << bit))
               == 0;
   }
   return false;
}

/*
 * Expand wild card functions
 *
 */

/*
 ** This is a wildcard comparison utility.
 ** It was written by Peter Bergsten in 1995.
 **
 ** The function compares a string against a wildcard that contains
 ** string fragments mixed with the usual '*' and '?' characters.
 ** The e parameter contains the wildcard and the c parameter contains
 ** the non wildcard string to check.
 **
 ** returns zero if the strings match each other and a nonzero value
 ** if they don't.
 **
 ** Expansion rules: '*' matches zero or more characters.
 **                  '?' matches any single character except the string terminator.
 **                   Note that "a*" matches "a", but "a?" does not match "a".
 **
 ** The strings are normally traversed until a mismatch occurs or until the
 ** end of the e-string. If the wildcard ends with a '*' however, the strings
 ** are only traversed until the end of the wildcard. Thus the "*" wildcard
 ** causes no loops at all, only three if-statements.
 */

static int
estrcmp(const char *e, const char *c)
{
   const char *r = (char *) 0;

   while (*e || *c)
   {
      if (*e == '*')
      {
         e++;
         /* Test on the special case of a '*' at the tail. */
         if (!*e)
         {
            return 0;
         }
         r = e;
      }
      else if (*e == '?' && *c)
      {
         e++;
         c++;
      }
      else if (*e == *c)
      {
         e++;
         c++;
      }
      else if (!*c)
         return 1;
      else if (!r)
         return 1;
      else
      {
         if (e != r)
         {
            c -= e - r - 1;
            e = r;
         }
         else
            c++;
      }
   }
   return 0;
}

static void
decode_env_value(char **src)
{
   char *value = *src;
   if (strlen(value) == 8 && (value[4] == 0x40 || value[4] == 0x60)
            && (value[5] == 0x40 || value[5] == 0x60) && (value[6] == 0x40
            || value[6] == 0x60) && (value[7] == 0x40 || value[7] == 0x60))
   {
      /*
       * The pointer value is obtained by xor-ing the two
       * words in the value.
       */
      OSADDRESS *mval = (OSADDRESS *) value;
      OSADDRESS pval = mval[0] ^ mval[1];

      /* Free the envp encoded value string. */
      free_buf((union SIGNAL **) src);

      /*
       * Allocate a new value string and fill it with
       * the decoded envp value.
       */
      *src = (char *) alloc(16, 0);
      sprintf(*src, "0x%X", pval);
   }
}

static void
handle_get_block_info_request(Client * client, union SIGNAL *sig_r)
{
   struct OS_pid_list *bid_list = NULL;
   OSBUFSIZE bid_count;
   PROCESS *bid;
   MonitorScope scope;
   union SIGNAL *sig;
   struct OS_pcb *pcb;

   assert_client(client);
   assert(sig_r);

   scope = sig_r->get_info_request.scope;

   if (MONITOR_SCOPE_ID == scope.type)
   {
      bid_count = 1;
      bid = (PROCESS *) &scope.id;
   }
   else
   {
      bid_list = get_bid_list(0);
      if (bid_list)
      {
         bid_count = bid_list->count;
         bid = bid_list->list;
      }
      else
      {
         bid_count = 0;
      }
   }

   for (; bid_count; bid_count--, bid++)
   {
      /* get the status of the block. */
      pcb = get_pcb(*bid);

      if (pcb)
      {
         if ((OS_BLOCK == pcb->type) && (MONITOR_SCOPE_SYSTEM == scope.type
                  || MONITOR_SCOPE_ID == scope.type
                  || (MONITOR_SCOPE_SEGMENT_ID == scope.type && get_segid(*bid)
                           == scope.id)))
         {
            uint32_t i;
            uint32_t not = 0;
            MonitorGetInfoRequest *req;
            MonitorGetBlockInfoReply *rep;

            /* Build status info */
            sig = alloc(sizeof(MonitorGetBlockInfoReply) + strlen(
                     &pcb->strings[pcb->name]), MONITOR_GET_BLOCK_INFO_REPLY);
            sig->get_block_info_reply.bid = *bid;
            sig->get_block_info_reply.uid = pcb->user;
            sig->get_block_info_reply.supervisor = 0;
            sig->get_block_info_reply.signalsAttached = 0;
            sig->get_block_info_reply.errorHandler = 0;
            sig->get_block_info_reply.maxSigSize = pcb->max_sigsize;
            sig->get_block_info_reply.euId = (uint16_t) get_pid_info(*bid, 13);
            sig->get_block_info_reply.sid = get_segid(*bid);
            sig->get_block_info_reply.reserved0 = 0;
            sig->get_block_info_reply.reserved1 = 0;
            sig->get_block_info_reply.sigPoolId =
            /* This code replaces get_sig_poolid(bid); */
            (PROCESS) get_sysconf((OSADDRESS) 27, (OSADDRESS) *bid,
                     (OSADDRESS) 0, (OSADDRESS) 0);

            sig->get_block_info_reply.stackPoolId =
            /* This code replaces get_stk_poolid(bid); */
            (uint32_t) get_sysconf((OSADDRESS) 26, (OSADDRESS) *bid,
                     (OSADDRESS) 0, (OSADDRESS) 0);
            strcpy(sig->get_block_info_reply.name, &pcb->strings[pcb->name]);

            req = &sig_r->get_info_request;
            rep = &sig->get_block_info_reply;

            /* Skip blocks according to filter: */
            for (i = 0; i < req->tagsCount; i += req->tags[i].tag.count)
            {
               switch (req->tags[i].tag.type)
               {
                  case MONITOR_BLOCK_FILTER_TAG_UID:
                     if (not ^ ((rep->uid < req->tags[i + 1].num) || (rep->uid
                              >= req->tags[i + 2].num)))
                     {
                        free_buf(&sig);
                     }
                     break;
                  case MONITOR_BLOCK_FILTER_TAG_MAX_SIG_SIZE:
                     if (not ^ ((rep->maxSigSize < req->tags[i + 1].num)
                              || (rep->maxSigSize >= req->tags[i + 2].num)))
                     {
                        free_buf(&sig);
                     }
                     break;
                  case MONITOR_BLOCK_FILTER_TAG_SIG_POOL_ID:
                     if (not ^ (rep->sigPoolId != req->tags[i + 1].num))
                     {
                        free_buf(&sig);
                     }
                     break;
                  case MONITOR_BLOCK_FILTER_TAG_STACK_POOL_ID:
                     if (not ^ (rep->stackPoolId != req->tags[i + 1].num))
                     {
                        free_buf(&sig);
                     }
                     break;
                  case MONITOR_BLOCK_FILTER_TAG_NAME:
                     if (not ^ (0 != estrcmp(req->tags[i + 1].str, rep->name)))
                     {
                        free_buf(&sig);
                     }
                     break;
                  case MONITOR_BLOCK_FILTER_TAG_EU_ID:
                     if (not ^ ((rep->euId < req->tags[i + 1].num)
                              || (rep->euId >= req->tags[i + 2].num)))
                     {
                        free_buf(&sig);
                     }
                     break;
                  default:
                     break;
               }
               if (MONITOR_BLOCK_FILTER_TAG_NOT == req->tags[i].tag.type)
                  not = 1;
               else
                  not = 0;

               if (sig == NULL)
               {
                  break;
               }
            }
            if (sig != NULL)
            {
               zzrmm_send2client(&sig, client);
            }
         }
         free_buf((union SIGNAL **) &pcb);
      }
   }

   if (bid_list)
   {
      free_buf((union SIGNAL **) &bid_list);
   }

   sig = alloc(sizeof(MonitorGetInfoEndmark), MONITOR_GET_BLOCK_INFO_ENDMARK);
   sig->get_info_endmark.status = MONITOR_STATUS_OK;
   send(&sig, client->pid);
}

static void
handle_get_env_vars_request(Client * client, union SIGNAL *sig_r)
{
   union SIGNAL *signal;
   char *names;
   MonitorScope scope;
   enum PROCESS_TYPE ptype;

   scope = sig_r->get_info_request.scope;

   assert_client(client);
   assert(sig_r);
   assert(MONITOR_GET_ENV_VARS_REQUEST == sig_r->sig_no);
   assert(MONITOR_SCOPE_ID == sig_r->get_info_request.scope.type);

   ptype = get_ptype(scope.id);

   switch (ptype)
   {
      case OS_PHANTOM:
      case OS_BG_PROC:
      case OS_PRI_PROC:
      case OS_TI_PROC:
      case OS_INT_PROC:
      case OS_ZOOMBIE:
      case OS_BLOCK:
         /* Get our space separated name list */
         names = get_env_list(scope.id, NULL);
         if (names != NULL)
         {
            char *name = names;
            char *blank = names;
            char *end = names + strlen(names);

            while (blank < end && *blank != ' ')
               blank++;

            *blank = '\0';

            while (name < end)
            {
               char *value = get_env(scope.id, name);

               /*
                * Check if the value is a pointer value set with set_envp().
                * If so, decode it and show as a hexadecimal value.
                */
               if (value != NULL)
               {
                  decode_env_value(&value);
                  /* Build a return signal. */
                  signal = alloc(sizeof(struct MonitorGetEnvVarsReply) + 1
                           + strlen(name) + strlen(value),
                           MONITOR_GET_ENV_VARS_REPLY);
                  signal->get_env_vars_reply.pid = scope.id;
                  signal->get_env_vars_reply.nameOffset = 0;
                  signal->get_env_vars_reply.valueOffset = strlen(name) + 1;
                  signal->get_env_vars_reply.reserved0 = 0;
                  signal->get_env_vars_reply.reserved1 = 0;
                  strcpy(
                           &signal->get_env_vars_reply.data[signal->get_env_vars_reply.nameOffset],
                           name);
                  strcpy(
                           &signal->get_env_vars_reply.data[signal->get_env_vars_reply.valueOffset],
                           value);

                  /* Free the value buffer obtained from get_env(). */
                  free_buf((union SIGNAL **) &value);

                  zzrmm_send2client(&signal, client);
               }

               /* Step forward to next name. */
               name = blank + 1;

               while (blank < end && *blank != ' ')
                  blank++;

               *blank = '\0';
            }
            free_buf((union SIGNAL **) &names);
         }
         signal = alloc(sizeof(MonitorGetInfoEndmark),
                  MONITOR_GET_ENV_VARS_ENDMARK);
         signal->get_info_endmark.status = MONITOR_STATUS_OK;
         break;
      case OS_ILLEGAL:
      default:
         signal = alloc(sizeof(MonitorGetInfoEndmark),
                  MONITOR_GET_ENV_VARS_ENDMARK);
         signal->get_info_endmark.status = MONITOR_STATUS_SCOPE_INVALID;
         break;
   }
   send(&signal, client->pid);
}

#define MAX_MEMORY_XFER_CHUNK (0x100)

/**
 * @function handle_get_memory_request
 * @brief Handle a request to read memory.
 */
static void
handle_get_memory_request(Client * client, union SIGNAL *sig)
{
   union SIGNAL *reply;
   MonitorStatus status = MONITOR_STATUS_OK;
   PROCESS pid = sig->get_memory_request.pid;
   uint32_t address = (uint32_t) sig->get_memory_request.address;
   uint32_t nbr_of_bytes = sig->get_memory_request.size;
   uint32_t chunk;
   enum PROCESS_TYPE ptype;

   ptype = (enum PROCESS_TYPE) get_pid_info(pid, 9);

   if (pid != 0 && (ptype == OS_ZOOMBIE || ptype == OS_ILLEGAL))
   {
      status = MONITOR_STATUS_SCOPE_INVALID;
   }
   else
   {
      while (nbr_of_bytes > 0)
      {
         chunk = (nbr_of_bytes > MAX_MEMORY_XFER_CHUNK) ? MAX_MEMORY_XFER_CHUNK
                                                        : nbr_of_bytes;

         reply = alloc(sizeof(struct MonitorGetMemoryReply) + chunk,
                  MONITOR_GET_MEMORY_REPLY);

         if (get_mem(pid, address, reply->get_memory_reply.data, chunk))
         {
            /* Some error occured. */
            status = MONITOR_STATUS_MEMORY_GET_ERROR;
            free_buf(&reply);
            break;
         }
         else
         {
            /* We got the chunk. */
            nbr_of_bytes -= chunk;

            reply->get_memory_reply.pid = pid;
            reply->get_memory_reply.address = address;
            reply->get_memory_reply.dataCount = chunk;

            /* Update address */
            address += chunk;
            zzrmm_send2client(&reply, client);
         }
      }
   }

   reply = alloc(sizeof(struct MonitorGetMemoryEndmark),
            MONITOR_GET_MEMORY_ENDMARK);
   reply->get_memory_endmark.status = status;
   send(&reply, client->pid);
}

/*
 ** Subfunction to handle_get_pool_info_request().
 **
 ** Display pool statistics about a pool.
 **
 ** Allocates response signals, initializes them and sends them.
 */
static void
build_pool_reply(struct OS_poolcb *pcb, PROCESS pid, Client * client)
{
   union SIGNAL *sig;

   MonitorPoolFragmentInfo *frags;
   MonitorPoolBufferSizeInfo *stacks;
   MonitorPoolBufferSizeInfo *signals;
   uint32_t fragment_count = pcb->values[pcb->fragment_info];
   uint32_t data_size = 0;
   uint32_t *confp;
   uint32_t i;

   assert(pcb);
   assert_client(client);

   /* FIXME: harcoded 100 limit on fragments */
   fragment_count = (fragment_count > 100) ? 100 : fragment_count;
   assert(fragment_count> 0);

   sig = alloc(sizeof(MonitorGetPoolInfoReply)
            + sizeof(MonitorPoolFragmentInfo) * fragment_count
            + sizeof(MonitorPoolBufferSizeInfo) * pcb->stk_conf_entries
            + sizeof(MonitorPoolBufferSizeInfo) * pcb->sig_conf_entries,
            MONITOR_GET_POOL_INFO_REPLY);

   sig->get_pool_info_reply.pid = pid;
   {
      PROCESS sid = get_segid(pid);

      sig->get_pool_info_reply.sid = (sid != pid) ? sid : 0;
   }
   sig->get_pool_info_reply.total = pcb->tot_size;
   sig->get_pool_info_reply.free = pcb->free_size;

   confp = (uint32_t *) get_sysconf(45, 0, 0, 0);
   assert(confp[0] >= 11);
   sig->get_pool_info_reply.signalAlignment = confp[11 /* POOL_ALIGNMENT */];
   sig->get_pool_info_reply.signalAdmSize = 1 + 32; /* TODO: Should be 1 + sizeof(struct _signal) from pose_internal.h */
   sig->get_pool_info_reply.signalInternalAdmSize = 0;
   sig->get_pool_info_reply.stackAlignment = confp[11 /* POOL_ALIGNMENT */];
   sig->get_pool_info_reply.stackAdmSize = 0;
   sig->get_pool_info_reply.stackInternalAdmSize = 1;

   sig->get_pool_info_reply.fragmentsOffset = data_size;
   sig->get_pool_info_reply.fragmentsCount = fragment_count;
   frags = (void *) &sig->get_pool_info_reply.data[data_size];
   data_size += sizeof(MonitorPoolFragmentInfo) / 4 * fragment_count;

   for (i = 0; i < fragment_count; i++)
   {
      OSADDRESS *tmp = &pcb->values[pcb->fragment_info + 1 + 4 * i];

      frags[i].address = *tmp++;
      frags[i].size = *tmp++;
      frags[i].stacks = *tmp++;
      frags[i].signals = *tmp;
      frags[i].reserved0 = 0;
      frags[i].reserved1 = 0;
   }

   sig->get_pool_info_reply.stackSizesOffset = data_size;
   sig->get_pool_info_reply.stackSizesCount = pcb->stk_conf_entries;
   stacks = (void *) &sig->get_pool_info_reply.data[data_size];
   data_size += sizeof(MonitorPoolBufferSizeInfo) / 4 * pcb->stk_conf_entries;

   for (i = 0; i < pcb->stk_conf_entries; i++)
   {
      stacks[i].size = pcb->values[pcb->stk_conf_sizes + i];
      stacks[i].allocated = pcb->values[pcb->stk_alloc_sizes + i];
      stacks[i].free = 0; /* TODO: Should be part of poolcb */
      stacks[i].reserved0 = 0;
   }

   sig->get_pool_info_reply.signalSizesOffset = data_size;
   sig->get_pool_info_reply.signalSizesCount = pcb->sig_conf_entries;
   signals = (void *) &sig->get_pool_info_reply.data[data_size];
   data_size += sizeof(MonitorPoolBufferSizeInfo) / 4 * pcb->sig_conf_entries;

   for (i = 0; i < pcb->sig_conf_entries; i++)
   {
      signals[i].size = pcb->values[pcb->sig_conf_sizes + i];
      signals[i].allocated = pcb->values[pcb->sig_alloc_sizes + i];
      signals[i].free = 0; /* TODO: Should be part of poolcb */
      signals[i].reserved0 = 0;
   }

   sig->get_pool_info_reply.reserved0 = 0;
   sig->get_pool_info_reply.reserved1 = 0;
   sig->get_pool_info_reply.reserved2 = 0;

   zzrmm_send2client(&sig, client);
}

static void
handle_get_pool_info_request(Client * client, union SIGNAL *sig_r)
{
   struct OS_pid_list *pid_list;
   union SIGNAL *signal;
   MonitorScope scope;

   scope = sig_r->get_info_request.scope;

   pid_list = (struct OS_pid_list *) get_sysconf((OSADDRESS) 24, (OSADDRESS) 0,
            (OSADDRESS) 0, (OSADDRESS) 0);

   /* Get all available pool id's */
   if (pid_list)
   {
      OSBUFSIZE pid_count = pid_list->count;
      PROCESS *pid = pid_list->list;

      /* For all pools... */
      for (; pid_count; pid_count--, pid++)
      {
         if ((MONITOR_SCOPE_SYSTEM == scope.type) || ((MONITOR_SCOPE_ID
                  == scope.type) && (scope.id == *pid))
                  || ((MONITOR_SCOPE_SEGMENT_ID == scope.type) && (scope.id
                           == get_segid(*pid))))
         {
            /* Get pool statistics */
            struct OS_poolcb *pcb =
            /* This code replaces get_poolcb(*pid); */
            (struct OS_poolcb *) get_sysconf((OSADDRESS) 25, (OSADDRESS) *pid,
                     (OSADDRESS) 0, (OSADDRESS) 0);

            if (pcb)
            {
               /*
                * Build statistics for this pool and send, check if stop request
                * was received
                */

               build_pool_reply(pcb, *pid, client);

               /* Free process control block */
               free_buf((union SIGNAL **) &pcb);
            }
         }
      }

      free_buf((union SIGNAL **) &pid_list);
   }

   signal = alloc(sizeof(MonitorGetInfoEndmark), MONITOR_GET_POOL_INFO_ENDMARK);
   signal->get_info_endmark.status = MONITOR_STATUS_OK;
   send(&signal, client->pid);
}

/*
 ** This function replaces the similar system call which is not
 ** available directly as it is implemented in osemain.c
 */

static OSBOOLEAN
lget_sig_info(struct OS_sig_info *sig_info, PROCESS pool_id, PROCESS pid,
         OSADDRESS sig)
{
   OSADDRESS pars[3];

   pars[0] = (OSADDRESS) pool_id;
   pars[1] = (OSADDRESS) pid;
   pars[2] = (OSADDRESS) sig;
   return (OSBOOLEAN) get_sysconf((OSADDRESS) 23, (OSADDRESS) sig_info,
            (OSADDRESS) pars, (OSADDRESS) 0);
}

static MonitorProcessState
get_proc_status(PROCESS pid, uint32_t stat)
{
   MonitorProcessState s;

   if (!(current_process() == pid))
   {
      s = (MonitorProcessState) stat;
      if (!((s & OS_STOPPED) || (s & OS_RECEIVE) || (s & OS_DELAY) || (s
               & OS_SEMAPHORE) || (s & OS_FSEMAPHORE) || (s & OS_REMOTE)))
      {
         /* If not any of the suspended states => the process must be ready. */
         s |= MONITOR_PROCESS_STATE_READY;
      }
   }
   else
   {
      s = MONITOR_PROCESS_STATE_RUNNING;
   }
   return s;
}

static union SIGNAL *
build_process_info(struct OS_pcb *pcb, PROCESS pid, PROCESS bid,
         union SIGNAL *sig_r)
{
   union SIGNAL *reply;

   if (pcb)
   {
      uint32_t i;
      uint32_t not = 0;
      MonitorGetInfoRequest *req;
      MonitorGetProcessInfoReply *rep;

      reply = alloc(sizeof(MonitorGetProcessInfoReply) + strlen(
               &pcb->strings[pcb->name]), MONITOR_GET_PROCESS_INFO_REPLY);
      reply->get_process_info_reply.pid = pid;
      reply->get_process_info_reply.bid = bid;
      {
         PROCESS sid = get_segid(bid);

         reply->get_process_info_reply.sid = (sid != bid) ? sid : 0;
      }
      reply->get_process_info_reply.type = (MonitorProcessType) pcb->type;
      reply->get_process_info_reply.state = get_proc_status(pid, pcb->status);
      reply->get_process_info_reply.priority = pcb->priority;
      reply->get_process_info_reply.entrypoint
               = (uint32_t) zzrmm_get_entrypoint(pid);
      reply->get_process_info_reply.properties = zzrmm_is_sysproc(pid);

      reply->get_process_info_reply.reserved1 = 0;

      strcpy(reply->get_process_info_reply.name, &pcb->strings[pcb->name]);

      req = &sig_r->get_info_request;
      rep = &reply->get_process_info_reply;

      /* Skip processes according to filter: */
      for (i = 0; i < req->tagsCount; i += req->tags[i].tag.count)
      {
         switch (req->tags[i].tag.type)
         {
            case MONITOR_PROCESS_FILTER_TAG_TYPE:
               if (not ^ (rep->type != req->tags[i + 1].num))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_STATE:
               if (not ^ (req->tags[i + 1].num != (rep->state
                        & req->tags[i + 1].num)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_PRIORITY:
               if (not ^ ((rep->priority < req->tags[i + 1].num)
                        || (rep->priority >= req->tags[i + 2].num)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_NAME:
               if (not ^ (0 != estrcmp(req->tags[i + 1].str, rep->name)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_SYSTEM:
               if (not ^ (!(rep->properties & 1)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_ENTRYPOINT:
               if (not ^ ((rep->entrypoint < req->tags[i + 1].num)
                        || (rep->entrypoint >= req->tags[i + 2].num)))
               {
                  free_buf(&reply);
               }
               break;
            default:
               break;
         }
         if (MONITOR_PROCESS_FILTER_TAG_NOT == req->tags[i].tag.type)
            not = 1;
         else
            not = 0;

         if (reply == NULL)
         {
            break;
         }
      }
   }
   else
   {
      /* End mark wanted. */
      reply = alloc(sizeof(MonitorGetInfoEndmark),
               MONITOR_GET_PROCESS_INFO_ENDMARK);
      reply->get_info_endmark.status = MONITOR_STATUS_OK;
   }
   return reply;
}

static union SIGNAL *
build_process_long_info(struct OS_pcb *pcb, PROCESS pid, PROCESS bid,
         union SIGNAL *sig_r)
{
   union SIGNAL *reply;

   if (pcb)
   {
      uint32_t name_offset;
      uint32_t data_size = 0;
      uint32_t file_offset = 0;
      int32_t sigsel_length = 0;
      uint32_t not = 0;
      uint32_t i;
      MonitorGetInfoRequest *req;
      MonitorGetProcessInfoLongReply *rep;

      if ((pcb->status & OS_RECEIVE))
      {
         sigsel_length = *((SIGSELECT *) &pcb->strings[pcb->wanted]);
         sigsel_length = (sigsel_length >= 0) ? sigsel_length : -sigsel_length;
         sigsel_length++;
         data_size = sigsel_length * sizeof(SIGSELECT);
      }

      name_offset = data_size;
      data_size += strlen(&pcb->strings[pcb->name]) + 1;

      if (pcb->line)
      {
         file_offset = data_size;
         data_size += strlen(&pcb->strings[pcb->file]) + 1;
      }
      reply = alloc(sizeof(MonitorGetProcessInfoLongReply) + data_size,
               MONITOR_GET_PROCESS_INFO_LONG_REPLY);

      reply->get_process_info_long_reply.pid = pid;
      reply->get_process_info_long_reply.bid = bid;
      {
         PROCESS sid = get_segid(bid);

         reply->get_process_info_long_reply.sid = (sid != bid) ? sid : 0;
      }
      reply->get_process_info_long_reply.type = (MonitorProcessType) pcb->type;
      reply->get_process_info_long_reply.state = get_proc_status(pid,
               pcb->status);
      reply->get_process_info_long_reply.priority = pcb->priority;
      reply->get_process_info_long_reply.uid = pcb->user;
      reply->get_process_info_long_reply.creator = 0;
      reply->get_process_info_long_reply.supervisor = 0;
      reply->get_process_info_long_reply.signalsAttached = 0;
      reply->get_process_info_long_reply.signalsInQueue = pcb->sig_cnt_in_q;

      reply->get_process_info_long_reply.entrypoint
               = (uint32_t) zzrmm_get_entrypoint(pid);
      reply->get_process_info_long_reply.properties = zzrmm_is_sysproc(pid);

      reply->get_process_info_long_reply.semaphoreValue = pcb->fsemvalue;

      switch (pcb->type)
      {
         case OS_PRI_PROC:
         case OS_BG_PROC:
         case OS_INT_PROC:
         case OS_TI_PROC:
            reply->get_process_info_long_reply.stackSize = pcb->stack_top
                     - pcb->stack_limit + 1;
            break;
         case OS_PHANTOM:
         default:
            reply->get_process_info_long_reply.stackSize = 0;
            break;
      }

      reply->get_process_info_long_reply.supervisorStackSize = 0;
      reply->get_process_info_long_reply.lineNumber = pcb->line;
      reply->get_process_info_long_reply.signalsOwned = pcb->sig_cnt_owned;
      reply->get_process_info_long_reply.timeSlice = 0;
      reply->get_process_info_long_reply.vector = 0;
      reply->get_process_info_long_reply.errorHandler = 0;
      reply->get_process_info_long_reply.euId = (uint16_t) zzkrn_get_pid_info(
               pid, 13);
      reply->get_process_info_long_reply.reserved1 = 0;
      reply->get_process_info_long_reply.reserved2 = 0;
      reply->get_process_info_long_reply.sigselectOffset = 0;
      reply->get_process_info_long_reply.sigselectCount = sigsel_length;
      if ((pcb->status & OS_RECEIVE))
      {
         uint32_t *to = (uint32_t *) reply->get_process_info_long_reply.data;
         SIGSELECT *from = ((SIGSELECT *) &pcb->strings[pcb->wanted]);

         while (sigsel_length-- > 0)
         {
            *to++ = *from++;
         }
      }

      reply->get_process_info_long_reply.processNameOffset = name_offset;
      {
         char *from = (char *) &pcb->strings[pcb->name];
         char
                  *to =
                           (char *) &reply->get_process_info_long_reply.data[name_offset];

         strcpy(to, from);
      }

      reply->get_process_info_long_reply.fileNameOffset = file_offset;
      if (file_offset)
      {
         char *from = (char *) &pcb->strings[pcb->file];
         char
                  *to =
                           (char *) &reply->get_process_info_long_reply.data[file_offset];

         strcpy(to, from);
      }

      req = &sig_r->get_info_request;
      rep = &reply->get_process_info_long_reply;

      /* Skip processes according to filter: */
      for (i = 0; i < req->tagsCount; i += req->tags[i].tag.count)
      {
         switch (req->tags[i].tag.type)
         {
            case MONITOR_PROCESS_FILTER_TAG_TYPE:
               if (not ^ (rep->type != req->tags[i + 1].num))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_STATE:
               if (not ^ (req->tags[i + 1].num != (rep->state
                        & req->tags[i + 1].num)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_PRIORITY:
               if (not ^ ((rep->priority < req->tags[i + 1].num)
                        || (rep->priority >= req->tags[i + 2].num)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_UID:
               if (not ^ ((rep->uid < req->tags[i + 1].num) || (rep->uid
                        >= req->tags[i + 2].num)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_SIGNALS_IN_QUEUE:
               if (not ^ ((rep->signalsInQueue < req->tags[i + 1].num)
                        || (rep->signalsInQueue >= req->tags[i + 2].num)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_ENTRYPOINT:
               if (not ^ ((rep->entrypoint < req->tags[i + 1].num)
                        || (rep->entrypoint >= req->tags[i + 2].num)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_SEMAPHORE_VALUE:
               if (not
                        ^ ((rep->semaphoreValue
                                 < (int32_t) req->tags[i + 1].num)
                                 || (rep->semaphoreValue
                                          >= (int32_t) req->tags[i + 2].num)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_STACK_SIZE:
               if (not ^ ((rep->stackSize < (req->tags[i + 1].num)
                        || (rep->stackSize >= req->tags[i + 2].num))))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_SUPERVISOR_STACK_SIZE:
               if (not ^ ((rep->supervisorStackSize < req->tags[i + 1].num)
                        || (rep->supervisorStackSize >= req->tags[i + 2].num)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_LINE_NUMBER:
               if (not ^ ((rep->lineNumber < req->tags[i + 1].num)
                        || (rep->lineNumber >= req->tags[i + 2].num)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_SIGNALS_OWNED:
               if (not ^ ((rep->signalsOwned < req->tags[i + 1].num)
                        || (rep->signalsOwned >= req->tags[i + 2].num)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_SIGSELECT_COUNT:
               if (not ^ ((rep->sigselectCount == 0)
                        || (*(uint32_t *) rep->data < req->tags[i + 1].num)
                        || (*(uint32_t *) rep->data >= req->tags[i + 2].num)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_NAME:
               if (not ^ (0 != estrcmp(req->tags[i + 1].str,
                        (const char *) &rep->data[rep->processNameOffset])))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_FILE_NAME:
               if (not ^ ((rep->fileNameOffset == 0) || estrcmp(
                        req->tags[i + 1].str,
                        (const char *) &rep->data[rep->fileNameOffset])))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_SYSTEM:
               if (not ^ (!(rep->properties & 1)))
               {
                  free_buf(&reply);
               }
               break;
            case MONITOR_PROCESS_FILTER_TAG_EU_ID:
               if (not ^ ((rep->euId < req->tags[i + 1].num) || (rep->euId
                        >= req->tags[i + 2].num)))
               {
                  free_buf(&reply);
               }
               break;
            default:
               break;
         }
         if (MONITOR_PROCESS_FILTER_TAG_NOT == req->tags[i].tag.type)
            not = 1;
         else
            not = 0;

         if (reply == NULL)
         {
            break;
         }
      }
   }
   else
   {
      reply = alloc(sizeof(MonitorGetInfoEndmark),
               MONITOR_GET_PROCESS_INFO_ENDMARK);
      reply->get_info_endmark.status = MONITOR_STATUS_OK;
   }
   return reply;
}

static void
traverse_process_in_block(union SIGNAL *sig_r, PROCESS bid, union SIGNAL *
(*build_display_r)(struct OS_pcb * pcb, PROCESS pid, PROCESS bid,
         union SIGNAL * sig_r), struct procnamelist *plist, Client * client)
{
   struct OS_pid_list *pid_list;
   struct OS_pcb *pcb;
   struct debug_pcb *deb_pcb;

   /* Get all available process id's. */
   pid_list = get_pid_list(bid);
   if (pid_list)
   {
      OSBUFSIZE pid_count = pid_list->count;
      PROCESS *pid = pid_list->list;

      /* For all processes... */
      for (; pid_count; pid_count--, pid++)
      {
         /* Get the debug pcb of the process. */
         LOCK_SWAP();
         deb_pcb = zzget_debugpcb(*pid);
         if (!deb_pcb)
         {
            UNLOCK_SWAP();
            continue;
         }

         /* Match the process against the wildcard name list. */
         if (!zzrmm_match_proc(deb_pcb, plist))
         {
            /* Skip it if no match. */
            UNLOCK_SWAP();
            continue;
         }
         UNLOCK_SWAP();

         /* get the status of the process. */
         pcb = get_pcb(*pid);

         /* Skip the process if there is something wrong with it. */
         if (pcb)
         {
            if (((pcb->type != OS_ILLEGAL) && (pcb->type != OS_ZOOMBIE)
                     && (pcb->type != OS_BLOCK)))
            {
               union SIGNAL *sig;

               /* Build status info and store it for later transmission. */
               sig = build_display_r(pcb, *pid, bid, sig_r);

               /* Skip also if build_* thinks there is something wrong with it */
               if (sig)
               {
                  /* Abort transmission on request. */
                  zzrmm_send2client(&sig, client);
               }
            }
            free_buf((union SIGNAL **) &pcb);
         }
      } /* for(... */

      free_buf((union SIGNAL **) &pid_list);
   } /* if (pid_list) */

   /* return command complete. No flow control request. */
}

/*
 ** This function traverses all blocks in the system and applies
 ** a signal_builder function to it.
 **
 */

static void
traverse_processes(union SIGNAL *sig_r, union SIGNAL *
(*build_display_r)(struct OS_pcb * pcb, PROCESS pid, PROCESS bid,
         union SIGNAL * sig_r), struct procnamelist *plist,
         struct Client * client)
{
   union SIGNAL *sig;
   struct OS_pid_list *bid_list;

   assert(sig_r);
   assert_client(client);

   /* Get all available block id's. */
   bid_list = get_bid_list(0);

   if (bid_list)
   {
      PROCESS *bid = bid_list->list;
      OSBUFSIZE bid_count = bid_list->count;

      /* For all blocks... */
      for (; bid_count; bid_count--, bid++)
      {
         /* get the status of the block. */
         struct OS_pcb *pcb = get_pcb(*bid);

         if (pcb)
         {
            if (pcb->type == OS_BLOCK)
            {
               /* Check all processes in the block. */
               traverse_process_in_block(sig_r, *bid, build_display_r, plist,
                        client);
            }
            free_buf((union SIGNAL **) &pcb);
         }
      } /* for(... */

      free_buf((union SIGNAL **) &bid_list);
   } /* if (bid_list) */

   /* Build and send the end mark */
   sig = build_display_r(NULL, 0, 0, NULL);
   send(&sig, client->pid);
}

static void
handle_get_process_info(Client *client, union SIGNAL *sig)
{
   struct procnamelist *plist = zzrmm_scope_to_plist(
            &sig->get_info_request.scope, sig->get_info_request.tags,
            sig->get_info_request.tagsCount);

   /* Traverse the list using the general display list function. */
   if (plist)
   {
      traverse_processes(sig,
               sig->get_info_request.level ? build_process_long_info
                                           : build_process_info, plist, client);
      free_buf((union SIGNAL **) &plist);
   }
   else
   {
      union SIGNAL *rep = alloc(sizeof(MonitorGetInfoEndmark),
               MONITOR_GET_PROCESS_INFO_ENDMARK);
      rep->get_info_endmark.status = MONITOR_STATUS_SCOPE_INVALID;
      send(&rep, client->pid);
   }
}

static void
handle_get_pool_signals(Client * client, union SIGNAL *sig)
{
   MonitorGetInfoRequest *req = &sig->get_info_request;
   union SIGNAL *signal;
   struct OS_sig_info sig_info;
   uint8_t ret;
   static const uint32_t sigs_per_reply = 10;
   MonitorStatus status = MONITOR_STATUS_OK;
   MonitorScope scope;

   scope = req->scope;

   assert_client(client);
   assert(MONITOR_SCOPE_ID == scope.type);

   ret = lget_sig_info(&sig_info, scope.id, 0, 0);
   if (5 == ret)
   {
      OSADDRESS sigptr;

      sigptr = sig_info.next;
      ret = lget_sig_info(&sig_info, scope.id, 0, sigptr);

      do
      {
         unsigned int i, j;
         MonitorSignalInfo *signals;

         signal = alloc(sizeof(MonitorGetPoolSignalsReply) + sigs_per_reply
                  * sizeof(MonitorSignalInfo), MONITOR_GET_POOL_SIGNALS_REPLY);
         signal->get_pool_signals_reply.pid = scope.id;

         i = 0;
         signals = signal->get_pool_signals_reply.signals;

         while (i < sigs_per_reply && sigptr && (1 == ret || 2 == ret || 3
                  == ret))
         {
            int not = 0;
            int skip = 0;

            signals[i].number = sig_info.sig_no;
            signals[i].owner = sig_info.owner;
            signals[i].sender = sig_info.sender_pid;
            signals[i].addressee = sig_info.addressee_pid;
            signals[i].size = sig_info.sig_size;
            signals[i].bufsize = sig_info.size_in_pool;
            signals[i].address = sigptr;

            if (ret == 1 && !sig_info.owner && sig_info.addressee_pid
                     == scope.id)
               signals[i].status = MONITOR_SIGNAL_STATUS_FREE;
            else
               signals[i].status = ret;

            signals[i].reserved0 = 0;
            signals[i].reserved1 = 0;

            /* Skip signal according to filter: */
            for (j = 0; j < req->tagsCount; j += req->tags[j].tag.count)
            {
               switch (req->tags[j].tag.type)
               {
                  case MONITOR_SIGNAL_FILTER_TAG_NUMBER:
                     if (not ^ ((signals[i].number < req->tags[j + 1].num)
                              || (signals[i].number >= req->tags[j + 2].num)))
                     {
                        skip = 1;
                     }
                     break;
                  case MONITOR_SIGNAL_FILTER_TAG_OWNER_ID:
                     if (not ^ (signals[i].owner != req->tags[j + 1].num))
                     {
                        skip = 1;
                     }
                     break;
                  case MONITOR_SIGNAL_FILTER_TAG_OWNER_NAME:
                  {
                     struct OS_pcb *pcb = get_pcb(signals[i].owner);

                     if (not ^ (!pcb || !pcb->name || estrcmp(
                              req->tags[j + 1].str, &pcb->strings[pcb->name])
                              != 0))
                     {
                        skip = 1;
                     }
                     if (pcb)
                     {
                        free_buf((union SIGNAL **) &pcb);
                     }
                     break;
                  }
                  case MONITOR_SIGNAL_FILTER_TAG_SENDER_ID:
                     if (not ^ (signals[i].sender != req->tags[j + 1].num))
                     {
                        skip = 1;
                     }
                     break;
                  case MONITOR_SIGNAL_FILTER_TAG_SENDER_NAME:
                  {
                     struct OS_pcb *pcb = get_pcb(signals[i].sender);

                     if (not ^ (!pcb || !pcb->name || estrcmp(
                              req->tags[j + 1].str, &pcb->strings[pcb->name])
                              != 0))
                     {
                        skip = 1;
                     }
                     if (pcb)
                     {
                        free_buf((union SIGNAL **) &pcb);
                     }
                     break;
                  }
                  case MONITOR_SIGNAL_FILTER_TAG_ADDRESSEE_ID:
                     if (not ^ (signals[i].addressee != req->tags[j + 1].num))
                     {
                        skip = 1;
                     }
                     break;
                  case MONITOR_SIGNAL_FILTER_TAG_ADDRESSEE_NAME:
                  {
                     struct OS_pcb *pcb = get_pcb(signals[i].addressee);

                     if (not ^ (!pcb || !pcb->name || estrcmp(
                              req->tags[j + 1].str, &pcb->strings[pcb->name])
                              != 0))
                     {
                        skip = 1;
                     }
                     if (pcb)
                     {
                        free_buf((union SIGNAL **) &pcb);
                     }
                     break;
                  }
                  case MONITOR_SIGNAL_FILTER_TAG_SIZE:
                     if (not ^ ((signals[i].size < req->tags[j + 1].num)
                              || (signals[i].size >= req->tags[j + 2].num)))
                     {
                        skip = 1;
                     }
                     break;
                  case MONITOR_SIGNAL_FILTER_TAG_BUFSIZE:
                     if (not ^ ((signals[i].bufsize < req->tags[j + 1].num)
                              || (signals[i].bufsize >= req->tags[j + 2].num)))
                     {
                        skip = 1;
                     }
                     break;
                  case MONITOR_SIGNAL_FILTER_TAG_ADDRESS:
                     if (not ^ ((signals[i].address < req->tags[j + 1].num)
                              || (signals[i].address >= req->tags[j + 2].num)))
                     {
                        skip = 1;
                     }
                     break;
                  case MONITOR_SIGNAL_FILTER_TAG_STATUS:
                     if (not ^ (signals[i].status != req->tags[j + 1].num))
                     {
                        skip = 1;
                     }
                     break;
                  default:
                     break;
               }
               if (MONITOR_SIGNAL_FILTER_TAG_NOT == req->tags[j].tag.type)
                  not = 1;
               else
                  not = 0;

               if (skip)
               {
                  break;
               }
            }

            i += !skip;

            /* If broken sigadm bail out (after i++, so user sees broken sigptr). */

            if (ret == 3)
            {
               status = MONITOR_STATUS_MEMORY_GET_ERROR;
               sigptr = (OSADDRESS) NULL;
               break;
            }
            sigptr = sig_info.next;
            ret = lget_sig_info(&sig_info, scope.id, 0, sigptr);
         }
         if (i > 0)
         {
            signal->get_pool_signals_reply.signalsCount = i;
            signal->get_pool_signals_reply.reserved0 = 0;
            zzrmm_send2client(&signal, client);
         }
         else
         {
            free_buf(&signal);
         }
      } while (sigptr);
      signal = alloc(sizeof(MonitorGetInfoEndmark),
               MONITOR_GET_POOL_SIGNALS_ENDMARK);
      signal->get_info_endmark.status = status;
   }
   else
   {
      /* The kernel does not support listning of pool signals !? */
      signal = alloc(sizeof(MonitorGetInfoEndmark),
               MONITOR_GET_POOL_SIGNALS_ENDMARK);
      signal->get_info_endmark.status = MONITOR_STATUS_KERNEL_ERROR;
   }
   send(&signal, client->pid);
}

static union SIGNAL *
build_register_info(struct OS_pcb *pcb, PROCESS pid, PROCESS bid,
         union SIGNAL *sig)
{
   union SIGNAL *reply;

   if (pcb != NULL)
   {
      uint32_t i;

      assert(sig);
      assert(MONITOR_GET_REGISTER_INFO_REQUEST == sig->sig_no);

      reply = alloc(sizeof(MonitorRegisterValues) + sizeof(MonitorRegister)
               * sig->get_register_info_request.registersCount,
               MONITOR_GET_REGISTERS_REPLY);

      reply->register_values.status = MONITOR_STATUS_OK;
      reply->register_values.pid = pid;
      reply->register_values.registersCount
               = sig->get_register_info_request.registersCount;

      for (i = 0; i < sig->get_register_info_request.registersCount; i++)
      {
         uint32_t value;
         uint32_t error = 1;

         reply->register_values.registers[i].id
                  = sig->get_register_info_request.registers[i];

         /* Read reg from EFMI. */
         error
                  = ose_efmi_get_register(
                           pid,
                           MONITOR_REGISTER_ID(sig->get_register_info_request.registers[i]),
                           (struct EfmiRegisterInfo *) &value);
         if (error)
         {
            reply->register_values.registers[i].id |= MONITOR_REGISTER_INVALID;
            reply->register_values.registers[i].value = 0xffffffff;
         }
         else
         {
            reply->register_values.registers[i].value = value;
         }
      }
   }
   else
   {
      assert(0 == pid);
      assert(0 == bid);
      assert(NULL == sig);

      /* Create the endmark signal */
      reply = alloc(sizeof(MonitorGetRegisterInfoEndmark),
               MONITOR_GET_REGISTER_INFO_ENDMARK);
      reply->get_register_info_endmark.status = MONITOR_STATUS_OK;
   }
   return reply;
}

/**
 * @function handle_get_register_info_request
 * @brief Handle a request to read registers.
 */
static void
handle_get_register_info_request(Client * client, union SIGNAL *sig)
{
   struct procnamelist *plist = zzrmm_scope_to_plist(
            &sig->get_info_request.scope, sig->get_info_request.tags,
            sig->get_info_request.tagsCount);

   /* Traverse the list using the general display list function. */
   if (plist)
   {
      traverse_processes(sig, build_register_info, plist, client);

      free_buf((union SIGNAL **) &plist);
   }
   else
   {
      union SIGNAL *reply = alloc(sizeof(MonitorGetRegisterInfoEndmark),
               MONITOR_GET_REGISTER_INFO_ENDMARK);
      reply->get_register_info_endmark.status = MONITOR_STATUS_REQUEST_INVALID;
      send(&reply, client->pid);
   }
}

#if 0
static bool
connect_mm(Client *client)
{
   if (0 == client->mm_)
   {
      hunt("ose_memory_mgr", 0, &client->mm_, NULL);
      if (0 == client->mm_)
      {
         return false;
      }
      else
      {
         union SIGNAL *sig = alloc(sizeof(SIGSELECT), OS_ATTACH_SIG);
         attach(&sig, client->mm_);
         // TODO: check MM interface signal
      }
   }
   return true;
}
static struct mm_get_domain_info *
get_domain_info(Client *client, OSDOMAIN domain_id)
{
   if (connect_mm(client))
   {
      union SIGNAL *sig = alloc(sizeof(struct mm_get_domain_info),
               MM_GET_DOMAIN_INFO_REQUEST);
      sig->mm_get_domain_info.domain = domain_id;
      sig->mm_get_domain_info.count = 0;
      send(&sig, client->mm_);
      sig = receive_from(~0, all_sigs_filter, client->mm_);
      if (NULL != sig)
      {
         assert(client->mm_ == sender(&sig));
         if (OS_ATTACH_SIG == sig->sig_no)
         {
            WARN(("warning lost connection to MM"));
            client->mm_ = 0;
            goto fail_out;
         }
         assert(MM_GET_DOMAIN_INFO_REPLY == sig->sig_no);
         if (MM_SUCCESS != sig->mm_get_domain_info.status)
         {
            WARN(("mm get domain info failed: status = %u\n",
                              sig->mm_get_domain_info.status));
            goto fail_out;
         }
         return &sig->mm_get_domain_info;
         fail_out: free_buf(&sig);
      }
   }
   return NULL;
}
#endif

static char *
get_mainblock_name(PROCESS sid)
{
   PROCESS bid = (PROCESS) get_pid_info(sid, 28); 
   
   return (char *)get_pid_info(bid, 14);
}

#define SYSTEM_SEG_NAME "System"

static void
handle_get_segment_info_request(Client * client, union SIGNAL *sig)
{
   MonitorGetInfoRequest *req = &sig->get_info_request;
   union SIGNAL *reply;
   struct OS_pid_list *sidlist =
            (struct OS_pid_list *) get_sysconf(71, 0, 0, 0);

   assert(req);

   if (sidlist)
   {
      OSBUFSIZE i;

      for (i = 0; i < sidlist->count; i++)
      {
         if (MONITOR_SCOPE_SYSTEM == req->scope.type || (MONITOR_SCOPE_ID
                  == req->scope.type && sidlist->list[i] == req->scope.id))
         {
            OSDOMAIN domainid = get_pid_info(sidlist->list[i], 16);

            if (0 == req->level)
            {
               reply = alloc(sizeof(MonitorGetSegmentInfoReply),
                        MONITOR_GET_SEGMENT_INFO_REPLY);

               reply->get_segment_info_reply.sid = sidlist->list[i];
               reply->get_segment_info_reply.number = domainid;
               reply->get_segment_info_reply.reserved0 = 0;
               reply->get_segment_info_reply.reserved1 = 0;
               reply->get_segment_info_reply.reserved2 = 0;
            }
            else
            {
#if 0
               struct mm_get_domain_info *domain = get_domain_info(client,
                        domainid);
               int name_len = (domain != NULL) ? strlen(domain->name) : 0;
               if (0 == name_len && 0 == sidlist->list[i])
               {
                  name_len = strlen(SYSTEM_SEG_NAME);
               }
#else
               char *name = get_mainblock_name(sidlist->list[i]);
               int name_len = (name)? strlen(name): 0;
#endif
               reply = alloc(sizeof(MonitorGetSegmentInfoLongReply) + name_len,
                        MONITOR_GET_SEGMENT_INFO_LONG_REPLY);

               reply->get_segment_info_long_reply.sid = sidlist->list[i];
               reply->get_segment_info_long_reply.number = domainid;
               reply->get_segment_info_long_reply.signalsAttached = 0; // TODO: get it with get_pid_info
               reply->get_segment_info_long_reply.masMapped = 0; // TODO: get it with get_pid_info
               reply->get_segment_info_long_reply.reserved0 = 0;
               reply->get_segment_info_long_reply.reserved1 = 0;
               reply->get_segment_info_long_reply.reserved2 = 0;
#if 0
               if (domain)
               {
                  strncpy(reply->get_segment_info_long_reply.name,
                           domain->name, name_len + 1);
               }
#else
               if (name)
               {
                  strncpy(reply->get_segment_info_long_reply.name,
                          name, name_len + 1);
               }
#endif
               else
               {
                  if (0 == sidlist->list[i])
                  {
                     strncpy(reply->get_segment_info_long_reply.name,
                              SYSTEM_SEG_NAME, name_len + 1);
                  }
                  else
                  {
                     reply->get_segment_info_long_reply.name[0] = '\0';
                  }
               }
            }
            zzrmm_send2client(&reply, client);
         }
      }

      free_buf((union SIGNAL **) &sidlist);
   }

   reply = alloc(sizeof(MonitorGetInfoEndmark),
            MONITOR_GET_SEGMENT_INFO_ENDMARK);

   reply->get_info_endmark.status = MONITOR_STATUS_OK;
   send(&reply, client->pid);
}

static union SIGNAL *
build_stack_usage(struct OS_pcb *pcb, PROCESS pid, PROCESS bid,
         union SIGNAL *sig_r)
{
   union SIGNAL *reply;

   if (pcb)
   {
      uint32_t not = 0;
      uint32_t i;
      MonitorGetInfoRequest *req;
      MonitorGetStackUsageReply *rep;

      /* Skip processes without a stack. */
      switch (pcb->type)
      {
         unsigned char *p;
         uint32_t size;
         uint32_t used;

      case OS_PRI_PROC:
      case OS_BG_PROC:
      case OS_INT_PROC:
      case OS_TI_PROC:
         reply = alloc(sizeof(MonitorGetStackUsageReply),
                  MONITOR_GET_STACK_USAGE_REPLY);
         reply->get_stack_usage_reply.pid = pid;

         /* Calculate the stack size. */
         p = (unsigned char *) pcb->stack_limit;
         used = size = (uint32_t) (pcb->stack_top - pcb->stack_limit + 1);

         reply->get_stack_usage_reply.main.address = pcb->stack_limit;
         reply->get_stack_usage_reply.main.size = size;

         /* Set the number of bytes used on the stack. */
         while (used-- > 0)
         {
            if (*p++ != 0xee)
            {
               break;
            }
         }
         reply->get_stack_usage_reply.main.used = used + 1;
         reply->get_stack_usage_reply.main.bufsize = 0;
         {
            /* This code replaces get_stk_poolid(bid); */
            PROCESS poolid = (PROCESS) get_sysconf((OSADDRESS) 26,
                     (OSADDRESS) bid, (OSADDRESS) 0, (OSADDRESS) 0);
            struct OS_poolcb *poolcb = get_poolcb(poolid);

            if (poolcb)
            {
               OSADDRESS i;

               for (i = 0; i < poolcb->stk_conf_entries; i++)
               {
                  OSADDRESS conf = poolcb->values[poolcb->stk_conf_sizes + i];

                  if (size <= conf)
                  {
                     reply->get_stack_usage_reply.main.bufsize = conf;
                     break;
                  }
               }
               free_buf((union SIGNAL **) &poolcb);
            }

         }
         reply->get_stack_usage_reply.main.reserved1 = 0;
         reply->get_stack_usage_reply.main.reserved2 = 0;

         reply->get_stack_usage_reply.supervisor.address = 0;
         reply->get_stack_usage_reply.supervisor.size = 0;
         reply->get_stack_usage_reply.supervisor.used = 0;
         reply->get_stack_usage_reply.supervisor.bufsize = 0;
         reply->get_stack_usage_reply.supervisor.reserved1 = 0;
         reply->get_stack_usage_reply.supervisor.reserved2 = 0;

         break;
      case OS_PHANTOM:
      default:
         reply = NULL;
         break;
      }

      if (reply)
      {
         req = &sig_r->get_info_request;
         rep = &reply->get_stack_usage_reply;

         /* Skip stacks according to filter: */
         for (i = 0; i < req->tagsCount; i += req->tags[i].tag.count)
         {
            switch (req->tags[i].tag.type)
            {
               case MONITOR_STACK_FILTER_TAG_OWNER_ID:
                  if (not ^ (rep->pid != req->tags[i + 1].num))
                  {
                     free_buf(&reply);
                  }
                  break;
               case MONITOR_STACK_FILTER_TAG_OWNER_NAME:
                  if (not ^ (estrcmp(req->tags[i + 1].str,
                           &pcb->strings[pcb->name]) != 0))
                  {
                     free_buf(&reply);
                  }
                  break;
               case MONITOR_STACK_FILTER_TAG_SIZE:
                  if (not ^ ((rep->main.size < req->tags[i + 1].num)
                           || (rep->main.size >= req->tags[i + 2].num)))
                  {
                     free_buf(&reply);
                  }
                  break;
               case MONITOR_STACK_FILTER_TAG_BUFSIZE:
                  if (not ^ ((rep->main.bufsize < req->tags[i + 1].num)
                           || (rep->main.bufsize >= req->tags[i + 2].num)))
                  {
                     free_buf(&reply);
                  }
                  break;
               case MONITOR_STACK_FILTER_TAG_ADDRESS:
                  if (not ^ ((rep->main.address < req->tags[i + 1].num)
                           || (rep->main.address >= req->tags[i + 2].num)))
                  {
                     free_buf(&reply);
                  }
                  break;
               case MONITOR_STACK_FILTER_TAG_USED:
                  if (not ^ ((rep->main.used < req->tags[i + 1].num)
                           || (rep->main.used >= req->tags[i + 2].num)))
                  {
                     free_buf(&reply);
                  }
                  break;
               case MONITOR_STACK_FILTER_TAG_UNUSED:
               {
                  uint32_t unused = rep->main.size - rep->main.used;

                  if (not ^ ((unused < req->tags[i + 1].num) || (unused
                           >= req->tags[i + 2].num)))
                  {
                     free_buf(&reply);
                  }
                  break;
               }
               case MONITOR_STACK_FILTER_TAG_SUPERVISOR_SIZE:
                  if (not ^ ((rep->supervisor.size < req->tags[i + 1].num)
                           || (rep->supervisor.size >= req->tags[i + 2].num)))
                  {
                     free_buf(&reply);
                  }
                  break;
               case MONITOR_STACK_FILTER_TAG_SUPERVISOR_BUFSIZE:
                  if (not
                           ^ ((rep->supervisor.bufsize < req->tags[i + 1].num)
                                    || (rep->supervisor.bufsize >= req->tags[i
                                             + 2].num)))
                  {
                     free_buf(&reply);
                  }
                  break;
               case MONITOR_STACK_FILTER_TAG_SUPERVISOR_ADDRESS:
                  if (not
                           ^ ((rep->supervisor.address < req->tags[i + 1].num)
                                    || (rep->supervisor.address >= req->tags[i
                                             + 2].num)))
                  {
                     free_buf(&reply);
                  }
                  break;
               case MONITOR_STACK_FILTER_TAG_SUPERVISOR_USED:
                  if (not ^ ((rep->supervisor.used < req->tags[i + 1].num)
                           || (rep->supervisor.used >= req->tags[i + 2].num)))
                  {
                     free_buf(&reply);
                  }
                  break;
               case MONITOR_STACK_FILTER_TAG_SUPERVISOR_UNUSED:
               {
                  uint32_t unused = rep->supervisor.size - rep->supervisor.used;

                  if (not ^ ((unused < req->tags[i + 1].num) || (unused
                           >= req->tags[i + 2].num)))
                  {
                     free_buf(&reply);
                  }
                  break;
               }
               default:
                  break;
            }
            if (MONITOR_STACK_FILTER_TAG_NOT == req->tags[i].tag.type)
               not = 1;
            else
               not = 0;

            if (reply == NULL)
            {
               break;
            }
         }
      }
   }
   else
   {
      /* End mark wanted. */
      reply = alloc(sizeof(MonitorGetInfoEndmark),
               MONITOR_GET_STACK_USAGE_ENDMARK);
      reply->get_info_endmark.status = MONITOR_STATUS_OK;
   }
   return reply;
}

static void
handle_get_signal_queue(Client * client, union SIGNAL *sig)
{
   MonitorGetInfoRequest *req = &sig->get_info_request;
   union SIGNAL *reply;
   struct OS_pcb *pcb;
   MonitorScope scope;

   scope = req->scope;

   assert(req);
   assert(MONITOR_GET_SIGNAL_QUEUE_REQUEST == req->sigNo);
   assert(MONITOR_SCOPE_ID == scope.type);
   assert_client(client);

   pcb = get_pcb(scope.id);

   if (likely(pcb))
   {
      if ((likely(OS_PRI_PROC == pcb->type || OS_BG_PROC == pcb->type
               || OS_PHANTOM == pcb->type)))
      {
         uint32_t entry;
         OSADDRESS nextsig = pcb->sigqueue;

         for (entry = 0; nextsig; entry++)
         {
            union SIGNAL *sig_copy;
            union SIGNAL *sig_addr = (void *) nextsig;
            struct MonitorSignalInfo *sig_info;

            /* Get the next signal copy. */
            /* done if signal queue has changed while reading. */
            if (likely(!get_signal(scope.id, nextsig, &sig_copy, &nextsig)))
            {
               uint32_t length = (uint32_t) sigsize(&sig_copy);

               if (req->level)
               {
                  reply = alloc(
                           sizeof(MonitorGetSignalQueueLongReply) + length,
                           MONITOR_GET_SIGNAL_QUEUE_LONG_REPLY);

                  reply->get_signal_queue_long_reply.pid = scope.id;
                  reply->get_signal_queue_long_reply.dataCount = length;
                  reply->get_signal_queue_long_reply.reserved0 = 0;
                  memcpy(reply->get_signal_queue_long_reply.data, sig_copy,
                           length);
                  sig_info = &reply->get_signal_queue_long_reply.signal;
               }
               else
               {
                  reply = alloc(sizeof(MonitorGetSignalQueueReply),
                           MONITOR_GET_SIGNAL_QUEUE_REPLY);

                  reply->get_signal_queue_reply.pid = scope.id;
                  sig_info = &reply->get_signal_queue_reply.signal;
               }
               sig_info->number = (uint32_t) sig_copy->sig_no;
               sig_info->owner = 0; /* Kernel owns signals in queues */
               sig_info->sender = (uint32_t) sender(&sig_copy);
               sig_info->addressee = (uint32_t) addressee(&sig_copy);
               sig_info->size = length;
               sig_info->bufsize = 0; /* TODO: Use get_sig_info()? */

               if ((OS_ZOOMBIE != get_ptype(sig_info->sender)) &&
                   (0 == get_segid(sig_info->sender)) &&
                   (0 == get_segid(scope.id)))
               {
                  sig_info->address = (uint32_t) sig_addr;
               }
               else
               {
                  sig_info->address = 0;
               }

               sig_info->status = MONITOR_SIGNAL_STATUS_UNKNOWN;
               sig_info->reserved0 = 0;
               sig_info->reserved1 = 0;

               /* Free the signal queue copy. */
               free_buf(&sig_copy);

               zzrmm_send2client(&reply, client);
            }
            else
            {
               nextsig = 0UL;
            }
         }
      }
      free_buf((union SIGNAL **) &pcb);
      reply = alloc(sizeof(MonitorGetInfoEndmark),
               MONITOR_GET_SIGNAL_QUEUE_ENDMARK);
      reply->get_info_endmark.status = MONITOR_STATUS_OK;
   }
   else
   {
      reply = alloc(sizeof(MonitorGetInfoEndmark),
               MONITOR_GET_SIGNAL_QUEUE_ENDMARK);
      reply->get_info_endmark.status = MONITOR_STATUS_ID_INVALID;
   }

   send(&reply, client->pid);
}

static void
handle_get_stack_usage(Client *client, union SIGNAL *sig)
{
   struct procnamelist *plist = zzrmm_scope_to_plist(
            &sig->get_info_request.scope, sig->get_info_request.tags,
            sig->get_info_request.tagsCount);

   /* Traverse the list using the general display list function. */
   if (plist)
   {
      traverse_processes(sig, build_stack_usage, plist, client);

      free_buf((union SIGNAL **) &plist);
   }
   else
   {
      union SIGNAL *reply = alloc(sizeof(MonitorGetInfoEndmark),
               MONITOR_GET_STACK_USAGE_ENDMARK);
      reply->get_info_endmark.status = MONITOR_STATUS_REQUEST_INVALID;
      send(&reply, client->pid);
   }
}

static void
handle_get_sys_param_request(Client * client, union SIGNAL *sig)
{
   MonitorGetInfoRequest *req = &sig->get_info_request;
   union SIGNAL *reply;
   char *name = (void *) alloc(SYSPARAM_MAX_NAME_LENGTH + 1, 0);

   (void) req;

   assert(MONITOR_SCOPE_SYSTEM == req->scope.type);

   for (;;)
   {
      size_t namelen;
      size_t valuelen;

      namelen = sysparam_list_next(name, name, SYSPARAM_MAX_NAME_LENGTH + 1);

      if (namelen)
      {
         reply = alloc(sizeof(MonitorGetSysParamReply)
                  + SYSPARAM_MAX_NAME_LENGTH + SYSPARAM_MAX_VALUE_LENGTH + 1,
                  MONITOR_GET_SYS_PARAM_REPLY);

         reply->get_sys_param_reply.nameOffset = 0;

         strcpy(reply->get_sys_param_reply.data, name);
         reply->get_sys_param_reply.valueOffset = namelen + 1;

         valuelen = sysparam_read(name,
                  &reply->get_sys_param_reply.data[namelen + 1],
                  SYSPARAM_MAX_VALUE_LENGTH + 1);
         set_sigsize(&reply, sizeof(MonitorGetSysParamReply) + namelen
                  + valuelen + 1);
         reply->get_sys_param_reply.reserved0 = 0;

         zzrmm_send2client(&reply, client);
      }
      else
      {
         break;
      }
   }
   free_buf((union SIGNAL **) &name);

   reply = alloc(sizeof(MonitorGetInfoEndmark), MONITOR_GET_SYS_PARAM_ENDMARK);

   reply->get_info_endmark.status = MONITOR_STATUS_OK;
   send(&reply, client->pid);
}

/*
 * Handling of heap related request.
 */
static bool
apply_filter_to_buffer(struct HeapBufferInfoReply *buf_info,
         struct MonitorGetInfoRequest *req)
{
   bool skip = false;
   unsigned int i;
   int not = 0;

   for (i = 0; i < req->tagsCount; i += req->tags[i].tag.count)
   {
      switch (req->tags[i].tag.type)
      {
         case MONITOR_HEAP_BUFFER_FILTER_TAG_OWNER_ID:
            if (not ^ (buf_info->owner != req->tags[i + 1].num))
            {
               skip = true;
            }
            break;
         case MONITOR_HEAP_BUFFER_FILTER_TAG_OWNER_NAME:
         {
            struct OS_pcb *pcb = get_pcb(buf_info->owner);

            if (not ^ (!pcb || !pcb->name || estrcmp(req->tags[i + 1].str,
                     &pcb->strings[pcb->name]) != 0))
            {
               skip = true;
            }
            if (pcb)
            {
               free_buf((union SIGNAL **) &pcb);
            }
            break;
         }
         case MONITOR_HEAP_BUFFER_FILTER_TAG_SHARED:
            if (not ^ (req->tags[i + 1].num != 0) ^ (buf_info->owner == 0))
            {
               skip = true;
            }
            break;
         case MONITOR_HEAP_BUFFER_FILTER_TAG_ADDRESS:
            if (not ^ ((buf_info->address < req->tags[i + 1].num)
                     || (buf_info->address >= req->tags[i + 2].num)))
            {
               skip = true;
            }
            break;
         case MONITOR_HEAP_BUFFER_FILTER_TAG_SIZE:
            if (not ^ ((buf_info->act_size < req->tags[i + 1].num)
                     || (buf_info->act_size >= req->tags[i + 2].num)))
            {
               skip = true;
            }
            break;
         case MONITOR_HEAP_BUFFER_FILTER_TAG_REQ_SIZE:
            if (not ^ ((buf_info->req_size < req->tags[i + 1].num)
                     || (buf_info->req_size >= req->tags[i + 2].num)))
            {
               skip = true;
            }
            break;
         case MONITOR_HEAP_BUFFER_FILTER_TAG_FILE_NAME:
            if (buf_info->file != ~0UL && buf_info->line != 0)
            {
               if (not ^ (estrcmp(req->tags[i + 1].str,
                        (const char *) (buf_info->data + buf_info->file)) != 0))
               {
                  skip = true;
               }
            }
            else
            {
               skip = true;
            }
            break;
         case MONITOR_HEAP_BUFFER_FILTER_TAG_LINE_NUMBER:
            if (not ^ ((buf_info->line < req->tags[i + 1].num)
                     || (buf_info->line >= req->tags[i + 2].num)))
            {
               skip = true;
            }
            break;
         case MONITOR_HEAP_BUFFER_FILTER_TAG_STATUS:
            if (MONITOR_HEAP_BUFFER_STATUS_VALID == req->tags[i + 1].num)
            {
               if (not ^ (!buf_info->endmark_ok))
               {
                  skip = true;
               }
            }
            else if (MONITOR_HEAP_BUFFER_STATUS_ENDMARK == req->tags[i + 1].num)
            {
               if (not ^ buf_info->endmark_ok)
               {
                  skip = true;
               }
            }
            break;
         default:
            break;
      }
      if (MONITOR_HEAP_BUFFER_FILTER_TAG_NOT == req->tags[i].tag.type)
         not = 1;
      else
         not = 0;
      if (skip)
      {
         break;
      }
   }
   return skip;
}

static void
build_heap_buffers_reply(Client *client, PROCESS heapd_pid, OSHEAP href,
         MonitorGetInfoRequest *request)
{
   static const SIGSELECT heap_sigsel[] =
   { 1, HEAP_BUFFER_INFO_REPLY };
   union SIGNAL *sig, *reply;
   unsigned int i, entry_counter, char_counter, counter;
   bool skip = false;

   sendFirstHBIRequest(href, 0, 16, HEAP_BUFFER_INFO_BURST, heapd_pid);

   counter = 0;
   while (1)
   {
      for (i = 0; i < HEAP_BUFFER_INFO_BURST; i++)
      {
         sig = receive(heap_sigsel);
         switch (sig->sig_no)
         {
            case HEAP_BUFFER_INFO_REPLY:

               if (sig->heap_buf_info_rep.status != HEAP_SUCCESS)
               {
                  if (counter)
                  {
                     /* truncate the reply signal */
                     set_sigsize(
                              &reply,
                              sizeof(MonitorGetHeapBuffersReply)
                                       + (sizeof(MonitorHeapBufferInfo)
                                                * (reply->get_heap_buf_reply.buffersCount)));
                     zzrmm_send2client(&reply, client);
                  }

                  if (sig->heap_buf_info_rep.status != HEAP_NO_MORE_ITEMS)
                  {

                     reply = alloc(sizeof(MonitorGetInfoEndmark),
                              MONITOR_GET_HEAP_BUFFERS_ENDMARK);
                     reply->get_info_endmark.status = MONITOR_STATUS_CANCELLED;
                     WARN(("unsuccessful heap request, status %lu!\n",
                                       sig->heap_buf_info_rep.status));
                     send(&reply, client->pid);
                     free_buf(&sig);
                     return;
                  } /* we have successfully iterate through all buffers */
                  free_buf(&sig);
                  return;
               }
               else
               {
                  skip = apply_filter_to_buffer(&sig->heap_buf_info_rep,
                           request);

                  if (!skip)
                  {
                     char_counter = entry_counter = 0;
                     if (sig->heap_buf_info_rep.file != ~0UL)
                     {
                        char_counter = strlen(
                                 (const char *) (sig->heap_buf_info_rep.data
                                          + sig->heap_buf_info_rep.file));
                        char_counter += 1; /* add space for '\0' */
                        entry_counter
                                 = 1
                                          + ((char_counter
                                                   / sizeof(MonitorHeapBufferInfo)
                                                   == 1)
                                                         ? 0
                                                         : (char_counter
                                                                 / sizeof(MonitorHeapBufferInfo)));
                     }

                     /* if this one will not fit, send what we have until now */
                     if (counter + entry_counter >= HEAP_BUFFER_INFO_BURST)
                     {
                        if (counter)
                        {
                           /* truncate the reply signal */
                           set_sigsize(
                                    &reply,
                                    sizeof(MonitorGetHeapBuffersReply)
                                             + (sizeof(MonitorHeapBufferInfo)
                                                      * (reply->get_heap_buf_reply.buffersCount)));
                           zzrmm_send2client(&reply, client);
                           counter = 0;
                        }
                     }

                     /* alloc here the reply if not done so yet */
                     if (counter == 0)
                     {
                        /* add extra space for filename information */
                        reply = alloc(sizeof(MonitorGetHeapBuffersReply)
                                 + sizeof(MonitorHeapBufferInfo)
                                          * (HEAP_BUFFER_INFO_BURST
                                                   + entry_counter),
                                 MONITOR_GET_HEAP_BUFFERS_REPLY);

                        reply->get_heap_buf_reply.id = href;
                        reply->get_heap_buf_reply.buffersCount = counter = 0;
                        reply->get_heap_buf_reply.reserved0 = 0;
                     }

                     reply->get_heap_buf_reply.buffersCount += entry_counter
                              + 1;
                     reply->get_heap_buf_reply.buffers[counter].info.owner
                              = sig->heap_buf_info_rep.owner;
                     (sig->heap_buf_info_rep.owner == 0)
                                                         ? (reply->get_heap_buf_reply.buffers[counter].info.shared
                                                                 = true)
                                                         : (reply->get_heap_buf_reply.buffers[counter].info.shared
                                                                 = false);

                     reply->get_heap_buf_reply.buffers[counter].info.address
                              = sig->heap_buf_info_rep.address;
                     reply->get_heap_buf_reply.buffers[counter].info.size
                              = sig->heap_buf_info_rep.act_size;
                     reply->get_heap_buf_reply.buffers[counter].info.reqSize
                              = sig->heap_buf_info_rep.req_size;
                     reply->get_heap_buf_reply.buffers[counter].info.lineNumber
                              = sig->heap_buf_info_rep.line;
                     reply->get_heap_buf_reply.buffers[counter].info.reserved0
                              = 0;
                     reply->get_heap_buf_reply.buffers[counter].info.reserved1
                              = 1;

                     (sig->heap_buf_info_rep.endmark_ok == true)
                                                                 ? (reply->get_heap_buf_reply.buffers[counter].info.status
                                                                         = MONITOR_HEAP_BUFFER_STATUS_VALID)
                                                                 : (reply->get_heap_buf_reply.buffers[counter].info.status
                                                                         = MONITOR_HEAP_BUFFER_STATUS_ENDMARK);

                     reply->get_heap_buf_reply.buffers[counter].info.fileNameCount
                              = entry_counter;

                     if (sig->heap_buf_info_rep.file != ~0UL)
                     {
                        memcpy(
                                 reply->get_heap_buf_reply.buffers[counter + 1].str,
                                 sig->heap_buf_info_rep.data
                                          + sig->heap_buf_info_rep.file,
                                 char_counter);
                     }
                     counter = reply->get_heap_buf_reply.buffersCount;
                  }
               }
               break;
            default:
               WARN(("received unexpected signal when obtaining fragments"
                                 "from the heap! signo: %u\n", sig->sig_no))
               ;
               break;
         }
         if (i == HEAP_BUFFER_INFO_BURST - 1)
         {
            sendNextHBIRequest(&sig->heap_buf_info_rep, href, 0, 16, HEAP_BUFFER_INFO_BURST, heapd_pid);
         }
         free_buf(&sig);
      }
   }
}

static void
handle_get_heap_buffers_request(Client * client, union SIGNAL *sig_req)
{
   union SIGNAL *sig;
   PROCESS heapd_pid;
   OSHEAP heap_ref;

   MonitorGetInfoRequest *mon_req = &sig_req->get_info_request;
   MonitorScope scope = mon_req->scope;

   if (hunt("ose_heapd", 0, &heapd_pid, NULL))
   {
      heap_ref = scope.id;
      if (!heap_ref && (MONITOR_SCOPE_ID != scope.type))
      {
         sig = alloc(sizeof(MonitorGetInfoEndmark),
                  MONITOR_GET_HEAP_BUFFERS_ENDMARK);
         sig->get_info_endmark.status = MONITOR_STATUS_ID_INVALID;
         send(&sig, client->pid);
         return;
      }
      build_heap_buffers_reply(client, heapd_pid, heap_ref, mon_req);

      sig = alloc(sizeof(MonitorGetInfoEndmark),
               MONITOR_GET_HEAP_BUFFERS_ENDMARK);
      sig->get_info_endmark.status = MONITOR_STATUS_OK;
      send(&sig, client->pid);
   }
   else
   {
      sig = alloc(sizeof(MonitorGetInfoEndmark),
               MONITOR_GET_HEAP_BUFFERS_ENDMARK);
      sig->get_info_endmark.status = MONITOR_STATUS_CANCELLED;
      send(&sig, client->pid);
   }
}

static void
build_heap_fragment_info_reply(Client *client, PROCESS heapd_pid, OSHEAP href)
{
   static const SIGSELECT heap_sigsel[] =
   { 2, HEAP_FRAGS_REPLY, HEAP_FRAG_INFO_REPLY };
   union SIGNAL *sig, *sig2, *reply;
   unsigned int i = 0;
   struct HeapFragInfo *req;

   sendHeapFragsRequest(href, heapd_pid);
   sig = receive(heap_sigsel);

   switch (sig->sig_no)
   {
      case HEAP_FRAGS_REPLY:
         if (sig->heap_frags_rep.status == HEAP_SUCCESS)
         {
            reply = alloc(sizeof(MonitorGetHeapFragmentInfoReply)
                     + (sizeof(MonitorHeapFragmentInfo)
                              * (sig->heap_frags_rep.num_frags - 1)),
                     MONITOR_GET_HEAP_FRAGMENT_INFO_REPLY);

            reply->get_heap_frag_info_reply.id = href;
            reply->get_heap_frag_info_reply.reserved0 = 0;
            reply->get_heap_frag_info_reply.fragmentsCount
                     = sig->heap_frags_rep.num_frags;

            for (i = 0; i < sig->heap_frags_rep.num_frags; i++)
            {
               reply->get_heap_frag_info_reply.fragments[i].size
                        = sig->heap_frags_rep.frags[i].size;
               reply->get_heap_frag_info_reply.fragments[i].address
                        = sig->heap_frags_rep.frags[i].addr;

               req = (struct HeapFragInfo *) alloc(sizeof(struct HeapFragInfo)
                        + sizeof(struct HeapTag), HEAP_FRAG_INFO_REQUEST);
               req->href = href;
               req->base = sig->heap_frags_rep.frags[i].addr;
               req->num_tags = 2;
               req->tags[0].tag_id = HEAP_FRAG_INFO_TAG_USED_SIZE;
               req->tags[1].tag_id = HEAP_FRAG_INFO_TAG_REQ_SIZE;
               send((union SIGNAL **) &req, heapd_pid);
               sig2 = receive(heap_sigsel);
               switch (sig2->sig_no)
               {
                  case HEAP_FRAG_INFO_REPLY:
                     if (sig2->heap_frag_info_rep.status == HEAP_SUCCESS)
                     {
                        reply->get_heap_frag_info_reply.fragments[i].usedSize
                                 = sig2->heap_frag_info_rep.tags[0].tag_value;
                        reply->get_heap_frag_info_reply.fragments[i].reqSize
                                 = sig2->heap_frag_info_rep.tags[1].tag_value;
                        reply->get_heap_frag_info_reply.fragments[i].reserved0
                                 = 0;
                        reply->get_heap_frag_info_reply.fragments[i].reserved1
                                 = 0;
                     }
                     else
                     {
                        reply->get_heap_frag_info_reply.fragments[i].usedSize
                                 = 0;
                        reply->get_heap_frag_info_reply.fragments[i].reqSize
                                 = 0;
                        reply->get_heap_frag_info_reply.fragments[i].reserved0
                                 = 0;
                        reply->get_heap_frag_info_reply.fragments[i].reserved1
                                 = 0;
                     }
                     break;
                  default:
                     WARN(("received unexpected signal when obtaining fragment"
                                       "information from the heap! signo: %u\n",
                                       sig2->sig_no))
                     ;
                     break;
               }
               free_buf(&sig2);
            }
         }
         else
         {
            reply = alloc(sizeof(MonitorGetInfoEndmark),
                     MONITOR_GET_HEAP_FRAGMENT_INFO_ENDMARK);
            reply->get_info_endmark.status = MONITOR_STATUS_CANCELLED;
            ramlog_printf("rmm: unsuccessful heap request, status %lu!\n",
                     sig->heap_frags_rep.status);
            send(&reply, client->pid);
            free_buf(&sig);
            return;
         }
         break;
      default:
         WARN(("received unexpected signal when obtaining fragment"
                           "information from the heap! signo: %u\n",
                           sig->sig_no))
         ;
         break;
   }
   free_buf(&sig);
   zzrmm_send2client(&reply, client);
}

static int
handle_get_heap_fragment_info_request(Client * client, union SIGNAL *sig_req)
{
   union SIGNAL *sig;
   PROCESS heapd_pid;
   OSHEAP heap_ref;

   MonitorGetInfoRequest *mon_req = &sig_req->get_info_request;
   MonitorScope scope = mon_req->scope;

   if (hunt("ose_heapd", 0, &heapd_pid, NULL))
   {
      heap_ref = scope.id;
      if (!heap_ref && (MONITOR_SCOPE_ID != scope.type))
      {
         sig = alloc(sizeof(MonitorGetInfoEndmark),
                  MONITOR_GET_HEAP_FRAGMENT_INFO_ENDMARK);
         sig->get_info_endmark.status = MONITOR_STATUS_ID_INVALID;
         assert(sig);
         send(&sig, client->pid);
         return 0;
      }
      build_heap_fragment_info_reply(client, heapd_pid, heap_ref);

      sig = alloc(sizeof(MonitorGetInfoEndmark),
               MONITOR_GET_HEAP_FRAGMENT_INFO_ENDMARK);
      sig->get_info_endmark.status = MONITOR_STATUS_OK;
      assert(sig);
      send(&sig, client->pid);
   }
   else
   {
      sig = alloc(sizeof(MonitorGetInfoEndmark),
               MONITOR_GET_HEAP_FRAGMENT_INFO_ENDMARK);
      sig->get_info_endmark.status = MONITOR_STATUS_CANCELLED;
      assert(sig);
      send(&sig, client->pid);
      WARN(("ose_heapd not found!\n"))
      ;
   }
   return 0;
}

static void
build_heap_info_reply(Client *client, PROCESS heapd_pid, PROCESS ownerpid,
         OSHEAP href)
{
   static const SIGSELECT heap_sigsel[] =
   { 2, HEAP_STATSEX_REPLY, HEAP_CONFIG_REPLY };
   union SIGNAL *sig, *reply;
   unsigned int i;

   sendHeapStatsExRequest(href, heapd_pid);
   sig = receive(heap_sigsel);

   if (sig->sig_no == HEAP_STATSEX_REPLY)
   {
      if (sig->heap_stats_rep.status == HEAP_SUCCESS)
      {
         reply = alloc(sizeof(MonitorGetHeapInfoReply)
                  + (sizeof(MonitorHeapBufferSizeInfo)
                           * (sig->heap_stats_rep.num_sizes - 1)),
                  MONITOR_GET_HEAP_INFO_REPLY);

         reply->get_heap_info_reply.id = href;
         reply->get_heap_info_reply.owner = ownerpid;
         reply->get_heap_info_reply.sid = get_segid(ownerpid);
         reply->get_heap_info_reply.size = sig->heap_stats_rep.total_used
                  + sig->heap_stats_rep.total_free;
         reply->get_heap_info_reply.usedSize = sig->heap_stats_rep.total_used;
         reply->get_heap_info_reply.peakUsedSize
                  = sig->heap_stats_rep.total_used_hi;
         reply->get_heap_info_reply.reqSize = sig->heap_stats_rep.total_req;
         reply->get_heap_info_reply.largestFree
                  = sig->heap_stats_rep.largest_free;
         reply->get_heap_info_reply.private
                  = sig->heap_stats_rep.number_private;
         reply->get_heap_info_reply.shared = sig->heap_stats_rep.number_shared;
         reply->get_heap_info_reply.mallocFailed
                  = sig->heap_stats_rep.malloc_failed;
         reply->get_heap_info_reply.bufferSizesCount
                  = sig->heap_stats_rep.num_sizes;
         reply->get_heap_info_reply.reserved0 = 0;
         reply->get_heap_info_reply.reserved1 = 0;
         reply->get_heap_info_reply.reserved2 = 0;
         reply->get_heap_info_reply.reserved3 = 0;
         for (i = 0; i < sig->heap_stats_rep.num_sizes; i++)
         {
            reply->get_heap_info_reply.bufferSizes[i].size
                     = sig->heap_stats_rep.size[i];
            reply->get_heap_info_reply.bufferSizes[i].allocated
                     = sig->heap_stats_rep.num_used[i];
            reply->get_heap_info_reply.bufferSizes[i].free
                     = sig->heap_stats_rep.num_free[i];
            reply->get_heap_info_reply.bufferSizes[i].reserved0 = 0;
         }
      }
      else
      {
         WARN(("unsuccessful heap request, status %lu!\n",
                           sig->heap_config_rep.status));
         goto error;
      }
   }
   else
   {
      ramlog_printf(
               "rmm: received unexpected signal when obtaining statistics from heap!\n");
      goto error;
   }

   free_buf(&sig);
   sendHeapConfigRequest(href, heapd_pid);
   sig = receive(heap_sigsel);

   if (sig->sig_no == HEAP_CONFIG_REPLY)
   {
      if (sig->heap_config_rep.status == HEAP_SUCCESS)
      {
         reply->get_heap_info_reply.largeThreshold
                  = sig->heap_config_rep.large_threshold;
      }
      else
      {
         ramlog_printf("rmm: unsuccessful heap request, status %lu!\n",
                  sig->heap_config_rep.status);
         free_buf(&reply);
         goto error;
      }
   }
   else
   {
      ramlog_printf(
               "rmm: received unexpected signal when obtaining statistics from heap!\n");
      free_buf(&reply);
      goto error;
   }

   assert(sig);
   free_buf(&sig);
   zzrmm_send2client(&reply, client);
   return;

   error: assert(sig);
   free_buf(&sig);
   reply = alloc(sizeof(MonitorGetInfoEndmark), MONITOR_GET_HEAP_INFO_ENDMARK);
   reply->get_info_endmark.status = MONITOR_STATUS_CANCELLED;
   send(&reply, client->pid);
}

static int
handle_get_heap_info_request(Client * client, union SIGNAL *sig_req)
{
   static const SIGSELECT heap_sigsel[] =
   { 1, HEAP_LIST_REPLY };
   union SIGNAL *sig;
   PROCESS heapd_pid, heap_ownerpid;
   OSHEAP heap_ref;
   unsigned int i;

   MonitorGetInfoRequest *mon_req = &sig_req->get_info_request;
   MonitorScope scope = mon_req->scope;

   if (hunt("ose_heapd", 0, &heapd_pid, NULL))
   {
      sig = alloc(sizeof(struct HeapListRequest), HEAP_LIST_REQUEST);
      send(&sig, heapd_pid);
      sig = receive(heap_sigsel);

      if (sig->sig_no == HEAP_LIST_REPLY)
      {
         if (sig->heap_list_rep.status == HEAP_SUCCESS)
         {
            for (i = 0; i < sig->heap_list_rep.num_heaps; i++)
            {
               heap_ref = sig->heap_list_rep.heaps[i].href;
               heap_ownerpid = sig->heap_list_rep.heaps[i].ownerpid;

               if ((MONITOR_SCOPE_SYSTEM == scope.type) || ((MONITOR_SCOPE_ID
                        == scope.type) && (scope.id == heap_ref))
                        || ((MONITOR_SCOPE_SEGMENT_ID == scope.type)
                                 && (scope.id == get_segid(heap_ownerpid))))
               {
                  build_heap_info_reply(client, heapd_pid, heap_ownerpid,
                           heap_ref);
               }
            }
            free_buf(&sig);
            sig = alloc(sizeof(MonitorGetInfoEndmark),
                     MONITOR_GET_HEAP_INFO_ENDMARK);
            sig->get_info_endmark.status = MONITOR_STATUS_OK;
            send(&sig, client->pid);
         }
         else
         {
            ramlog_printf("rmm: unsuccessful heap request, status %lu!\n",
                     sig->heap_list_rep.status);
            free_buf(&sig);
            sig = alloc(sizeof(MonitorGetInfoEndmark),
                     MONITOR_GET_HEAP_INFO_ENDMARK);
            sig->get_info_endmark.status = MONITOR_STATUS_CANCELLED;
            send(&sig, client->pid);
         }
      }
      else
      {
         ramlog_printf("rmm: received unexpected signal from the heap!\n");
         free_buf(&sig);
         sig = alloc(sizeof(MonitorGetInfoEndmark),
                  MONITOR_GET_HEAP_INFO_ENDMARK);
         sig->get_info_endmark.status = MONITOR_STATUS_CANCELLED;
         send(&sig, client->pid);
      }
   }
   else
   {
      ramlog_printf("rmm: ose_heapd not found!\n");
      sig = alloc(sizeof(MonitorGetInfoEndmark), MONITOR_GET_HEAP_INFO_ENDMARK);
      sig->get_info_endmark.status = MONITOR_STATUS_CANCELLED;
      send(&sig, client->pid);
   }
   return 0;
}

#ifdef OSE_SMP
/*
 * Handling of request signals for profile enable/disable execution unit
 */

static int
handle_get_prof_eu_enabled_request(Client *client, union SIGNAL *sig)
{
/* There is nothing in profiler to return such information */
   cpuid_t cpu;
   union SIGNAL *reply;
   cpuid_t cpu_i;

   cpu = sig->get_prof_eu_enabled_request.eu;
   reply = alloc (sizeof(MonitorGetProfilingExecutionUnitEnabledReply), MONITOR_GET_PROFILING_EXECUTION_UNIT_ENABLED_REPLY);

   if (cpu == MONITOR_ALL_EXECUTION_UNITS)
   {
      reply->get_prof_eu_enabled_reply.status = MONITOR_STATUS_OK;
      reply->get_prof_eu_enabled_reply.enabled = True;
      /* check corest 1 to ose_num_cpus(). */
      for (cpu_i = 1; cpu_i < ose_num_cpus(); cpu_i++)
      {
         if (!zzprof_get_core_status(cpu_i))
         {
            reply->get_prof_eu_enabled_reply.enabled = False;
            break;
         }
      }
   }
   else if (cpu < ose_num_cpus())
   {
      reply->get_prof_eu_enabled_reply.status = MONITOR_STATUS_OK;
      reply->get_prof_eu_enabled_reply.enabled = zzprof_get_core_status(cpu);
   }
   else
   {
      reply->get_prof_eu_enabled_reply.status = MONITOR_STATUS_INVALID_EU;
   }

   send(&reply, client->pid);
   return 0;
}


static int
handle_set_prof_eu_enabled_request(Client *client, union SIGNAL *sig)
{
   cpuid_t cpu;
   MonitorBoolean enable;
   union SIGNAL *reply;
   cpuid_t cpu_i;

   cpu = sig->set_prof_eu_enabled_request.eu;
   enable = sig->set_prof_eu_enabled_request.enabled;

   reply = alloc (sizeof(MonitorSetProfilingExecutionUnitEnabledReply), MONITOR_SET_PROFILING_EXECUTION_UNIT_ENABLED_REPLY);

   /* Currently this can only be used to disable cores */
   if (enable)
   {
      reply->set_prof_eu_enabled_reply.status = MONITOR_STATUS_REQUEST_INVALID;
   }
   else if (cpu == MONITOR_ALL_EXECUTION_UNITS)
   {
      for (cpu_i = 1; cpu_i < ose_num_cpus(); cpu_i++)
      {
         zzprof_disable_core(cpu_i);
      }
      reply->set_prof_eu_enabled_reply.status = MONITOR_STATUS_OK;
   }
   else if (cpu == 0 || cpu >= ose_num_cpus())
   {
      reply->set_prof_eu_enabled_reply.status = MONITOR_STATUS_INVALID_EU;
   }
   else
   {
      zzprof_disable_core(cpu);
      reply->set_prof_eu_enabled_reply.status = MONITOR_STATUS_OK;
   }

   send(&reply, client->pid);
   return 0;
}
#endif
/*
 * Handling of request signals from monitor_action.sig
 */
static int
handle_create(Client * client, union SIGNAL *sig)
{
   union SIGNAL *reply;
   MonitorStatus status = MONITOR_STATUS_OK;
   MonitorCreateProcessRequest *req = &sig->create_process_request;

   char *name = NULL;
   struct OS_redir_entry *redir = NULL;

   reply = alloc(sizeof(MonitorCreateProcessReply),
            MONITOR_CREATE_PROCESS_REPLY);

   /* Get the tags */
   {
      uint32_t i;

      for (i = 0; i < req->tagsCount; i += req->tags[i].tag.count)
      {
         switch (req->tags[i].tag.type)
         {
            case MONITOR_CREATE_PROCESS_TAG_NAME:
               name = req->tags[i + 1].str;
               break;
            case MONITOR_CREATE_PROCESS_TAG_REDIRECTION:
               redir = (void *) &req->tags[i + 1].num;
               break;
            default:
               break;
         }
      }
   }

   /* Some basic checks */
   if (name == NULL)
   {
      status = MONITOR_STATUS_REQUEST_INVALID;
   }
   if (req->uid != 0)
   {
      status = MONITOR_STATUS_REQUEST_INVALID;
   }

   switch (req->type)
   {
      case MONITOR_PROCESS_TYPE_PRIORITIZED:
         if (req->priority > 31 || req->vector != 0)
         {
            status = MONITOR_STATUS_REQUEST_INVALID;
         }
         break;
      case MONITOR_PROCESS_TYPE_BACKGROUND:
         if (req->priority != 0 || req->vector != 0)
         {
            status = MONITOR_STATUS_REQUEST_INVALID;
         }
         break;
      case MONITOR_PROCESS_TYPE_INTERRUPT:
         if (req->priority > 31 || req->timeSlice != 0)
         {
            status = MONITOR_STATUS_REQUEST_INVALID;
         }
         break;
      case MONITOR_PROCESS_TYPE_TIMER_INTERRUPT:
         if (req->priority != 0 || req->vector != 0)
         {
            status = MONITOR_STATUS_REQUEST_INVALID;
         }
         break;
      case MONITOR_PROCESS_TYPE_PHANTOM:
         if (req->priority != 0 || req->timeSlice != 0 || req->vector != 0)
         {
            status = MONITOR_STATUS_REQUEST_INVALID;
         }
         break;
      default:
         status = MONITOR_STATUS_REQUEST_INVALID;
         break;
   }

   /* If still OK, create the process */
   if (!status)
   {
      if (get_ptype(req->bid) == OS_BLOCK)
      {
         PROCESS pid;

         pid = create_process(req->type, name,
                  (OSENTRYPOINT *) req->entrypoint, req->stackSize,
                  req->priority, req->timeSlice, req->bid, redir, req->vector,
                  req->uid);

         reply->create_process_reply.pid = pid;
         reply->create_process_reply.bid = req->bid;
         reply->create_process_reply.sid = get_segid(pid);
      }
      else
      {
         status = MONITOR_STATUS_REQUEST_INVALID;
      }
   }

   reply->create_process_reply.status = status;
   send(&reply, client->pid);
   return 0;
}

static int
handle_kill(Client * client, union SIGNAL *sig)
{
   MonitorActionRequest *req = &sig->action_request;
   PROCESS pid = req->scope.id;
   enum PROCESS_TYPE ptype;

   union SIGNAL *reply = alloc(sizeof(MonitorActionReply),
            MONITOR_KILL_SCOPE_REPLY);

   ptype = get_pid_info(pid, 9);
   switch (ptype)
   {
      case OS_PHANTOM:
      case OS_BG_PROC:
      case OS_PRI_PROC:
      case OS_TI_PROC:
      case OS_INT_PROC:
      case OS_ZOOMBIE:
      case OS_BLOCK:
      case OS_ILLEGAL + 2U: /* Segment type */
         kill_proc(pid);
         reply->action_reply.status = MONITOR_STATUS_OK;
         break;

      case OS_ILLEGAL:
      default:
         reply->action_reply.status = MONITOR_STATUS_SCOPE_INVALID;
         ramlog_printf("rmm: client tried to kill an illegal pid 0x%X"
            " of type 0x%X\n", pid, ptype);
         break;
   }
   send(&reply, client->pid);
   return 0;
}

static int
handle_send(Client * client, union SIGNAL *sig)
{
   union SIGNAL *s;
   union SIGNAL *reply;

   /* Switch to the user pool. */
   zzselect_user_pool((PROCESS) sig->send_signal_request.addressee);

   /* Allocate and build the signal. */
   s = alloc(sizeof(SIGSELECT) + sig->send_signal_request.dataCount,
            (SIGSELECT) sig->send_signal_request.number);
   memcpy((char *) s + sizeof(SIGSELECT), sig->send_signal_request.data,
            sig->send_signal_request.dataCount);
   send_w_s(&s, (PROCESS) sig->send_signal_request.sender,
            (PROCESS) sig->send_signal_request.addressee);

   /* Switch back to the system pool. */
   zzselect_user_pool((PROCESS) 0);

   reply = alloc(sizeof(struct MonitorActionReply), MONITOR_SEND_SIGNAL_REPLY);

   reply->action_reply.status = MONITOR_STATUS_OK;
   send(&reply, client->pid);
   return 0;
}

static void
handle_set_env_var_request(Client * client, union SIGNAL *sig_r)
{
   MonitorStatus status = MONITOR_STATUS_ID_INVALID;
   MonitorSetEnvVarRequest *r = &sig_r->set_env_var_request;

   union SIGNAL *signal;

   if (get_ptype(r->pid) != OS_ILLEGAL)
   {
      if (r->valueOffset == ~0UL)
      {
         set_env(r->pid, &r->data[r->nameOffset], NULL);
      }
      else if (r->valueOffset == 0UL)
      {
         set_envp(r->pid, &r->data[r->nameOffset], r->value);
      }
      else
      {
         set_env(r->pid, &r->data[r->nameOffset], &r->data[r->valueOffset]);
      }

      status = MONITOR_STATUS_OK;
   }

   signal = alloc(sizeof(MonitorActionReply), MONITOR_SET_ENV_VAR_REPLY);
   signal->action_reply.status = status;
   send(&signal, client->pid);
}

static void
handle_set_priority_request(Client *client, union SIGNAL *sig_r)
{
   MonitorStatus status = MONITOR_STATUS_OK;
   MonitorSetPriorityRequest *r = &sig_r->set_priority_request;
   enum PROCESS_TYPE process_type = get_ptype(r->pid);
   union SIGNAL *signal;

   if (process_type == OS_PRI_PROC)
   {
      if (r->priority <= 31)
      {
         OSPRIORITY prio = set_pri_for(r->pid, r->priority);

         if (prio != r->priority)
         {
            /* The resulting prio is not the requested prio. */
            status = MONITOR_STATUS_REQUEST_INVALID;
         }
      }
   }
   else
   {
      /* Process is not a prioritized process. */
      status = MONITOR_STATUS_ID_INVALID;
   }

   signal = alloc(sizeof(MonitorActionReply), MONITOR_SET_PRIORITY_REPLY);
   signal->action_reply.status = status;
   send(&signal, client->pid);
}

static void
handle_signal_semaphore_request(Client *client, union SIGNAL *sig_r)
{
   MonitorStatus status = MONITOR_STATUS_ID_INVALID;
   MonitorSignalSemaphoreRequest *r = &sig_r->signal_semaphore_request;
   enum PROCESS_TYPE process_type = get_ptype(r->pid);
   union SIGNAL *signal;

   switch (process_type)
   {
      case OS_BG_PROC:
      case OS_PRI_PROC:
      case OS_INT_PROC:
      case OS_TI_PROC:
      case OS_PHANTOM:
      {
         OSFSEMVAL fsem = get_fsem(r->pid);

         if ((process_type != OS_INT_PROC) && (fsem > (OSFSEMVAL) 0) && (fsem
                  + (OSFSEMVAL) 1 < (OSFSEMVAL) 0))
         {
            /* It is illegal to cause a fast semaphore to wrap
             from high positive numbers to a negative value for all
             processes but interrupt processes.*/
            status = MONITOR_STATUS_REQUEST_INVALID;
         }
         else
         {
            signal_fsem(r->pid);
            status = MONITOR_STATUS_OK;
         }
         break;
      }

      default:
         break;
   }

   signal = alloc(sizeof(MonitorActionReply), MONITOR_SIGNAL_SEMAPHORE_REPLY);
   signal->action_reply.status = status;
   send(&signal, client->pid);
}

static OSADDRESS
rmm_error_handler(OSBOOLEAN user_called, OSERRCODE ecode, OSERRCODE extra)
{
   (void) extra;
   if (!user_called && !(ecode & OSE_EFATAL_MASK)
            && (ecode & OSE_ESUBCODE_MASK) == OSE_START && (ecode
            & OSE_EERROR_MASK) == OSE_EALREADY_STARTED)
   {
      /* Non fatal error generated due to the start system call so ignore it */
      return 1; /* Error handled */
   }
   else
   {
      return 0; /* Error not handled */
   }
}

static int
handle_start(Client * client, union SIGNAL *sig)
{
   MonitorActionRequest *req = &sig->action_request;
   PROCESS pid = req->scope.id;

   union SIGNAL *reply = alloc(sizeof(MonitorActionReply),
            MONITOR_START_SCOPE_REPLY);

   switch (get_pid_info(pid, 9))
   {
      case OS_PRI_PROC:
      case OS_BG_PROC:
      case OS_TI_PROC:
      {
         struct OS_pcb *pcb;
         /*
          ** Skip the process if it is not stopped.
          ** This is not water proof since some other process may issue a start
          ** call between our test and start calls.
          ** We still need the error handler.
          */
         pcb = get_pcb(pid);

         if (pcb)
         {
            if (pcb->status & OS_STOPPED)
            {
               /* Try to start the process. */
               start(pid);
               reply->action_reply.status = MONITOR_STATUS_OK;
            }
            else
            {
               reply->action_reply.status = MONITOR_STATUS_SCOPE_STATE_INVALID;
            }
            free_buf((union SIGNAL **) &pcb);
         }
         else
         {
            reply->action_reply.status = MONITOR_STATUS_ID_INVALID;
         }
      }
         break;

      case OS_ILLEGAL + 2u: /* Segment type */
      case OS_BLOCK:
         start(pid);
         reply->action_reply.status = MONITOR_STATUS_OK;
         break;

      default:
         reply->action_reply.status = MONITOR_STATUS_SCOPE_INVALID;
         break;
   }
   send(&reply, client->pid);
   return 0;
}

static int
handle_stop(Client * client, union SIGNAL *sig)
{
   MonitorActionRequest *req = &sig->action_request;
   PROCESS pid = req->scope.id;

   union SIGNAL *reply = alloc(sizeof(MonitorActionReply),
            MONITOR_STOP_SCOPE_REPLY);

   switch (get_pid_info(pid, 9))
   {
      case OS_PRI_PROC:
      case OS_BG_PROC:
      case OS_BLOCK:
      case OS_TI_PROC:
      case OS_ILLEGAL + 2u: /* Segment type */
         stop(pid);
         reply->action_reply.status = MONITOR_STATUS_OK;
         break;
      default:
         reply->action_reply.status = MONITOR_STATUS_SCOPE_INVALID;
         break;
   }
   send(&reply, client->pid);
   return 0;
}

static int
handle_set_execution_unit(Client * client, union SIGNAL *sig)
{
   MonitorSetExecutionUnitRequest *req = &sig->set_execution_unit_request;
   PROCESS pid = req->pid;
   cpuid_t cpu = req->euId;

   union SIGNAL *reply = alloc(sizeof(MonitorActionReply),
            MONITOR_SET_EXECUTION_UNIT_REPLY);

   switch (get_pid_info(pid, 9))
   {
      case OS_PRI_PROC:
      case OS_BG_PROC:
      case OS_BLOCK:
      case OS_INT_PROC:
      case OS_TI_PROC:
         if (cpu < ose_num_cpus())
         {
            if (ose_bind_process(pid, cpu))
            {
               reply->action_reply.status = MONITOR_STATUS_OK;
            }
            else
            {
               reply->action_reply.status = MONITOR_STATUS_BIND_FAILED;
            }
         }
         else
         {
            /* Execution unit is out of range */
            reply->action_reply.status = MONITOR_STATUS_INVALID_EU;
         }
         break;
         
      case OS_ZOOMBIE:
         reply->action_reply.status = MONITOR_STATUS_KILLED;
         break;
         
      default:
         reply->action_reply.status = MONITOR_STATUS_ID_INVALID;
         break;
   }
   send(&reply, client->pid);
   return 0;
}

/*
 * Ramdump related functions.
 */

/**
 * @function ramdump_status
 * @brief Map a ramdump status value to a monitor status value
 */
static MonitorStatus
ramdump_status(RamdumpStatus status)
{
   switch (status)
   {
      case RAMDUMP_SUCCESS:
         return MONITOR_STATUS_OK;
      case RAMDUMP_EMEMORY_ACCESS_NOK:
         return MONITOR_STATUS_MEMORY_GET_ERROR;
      case RAMDUMP_EPOS_OR_DUMP_NOT_AVAILABLE:
      case RAMDUMP_EINVALID_ARGUMENT:
      case RAMDUMP_ERAMDUMP_NOT_INITIALIZED:
         return MONITOR_STATUS_REQUEST_INVALID;
      default:
         return MONITOR_STATUS_INTERNAL_ERROR;
   }
}

/**
 * @function handle_get_ramdump_info_request
 * @brief Handle a request to get RAM dump information
 */
static void
handle_get_ramdump_info_request(Client * client, union SIGNAL *sig)
{
   MonitorGetRamdumpInfoRequest *req = &sig->get_ramdump_info_request;
   union SIGNAL *signal;
   struct RamdumpDumpInfo dump_info;
   RamdumpStatus status = RAMDUMP_SUCCESS;
   uint32_t i;

   assert_client(client);

   /* Get dump info for dumps in requested interval */
   for (i = req->from; i < req->to; i++)
   {
      /* Retrieve dump id corresponding to index */
      RamdumpDumpId dump_id = ramdump_get_dump_id(i);

      if (!dump_id)
      {
         /* End of dumps, break out of loop. */
         break;
      }

      /* Retrieve information corresponding to dump id */
      status = ramdump_get_dump_info(dump_id, &dump_info);

      if (status == RAMDUMP_SUCCESS)
      {
         /* Fill out reply signal fields */
         signal = alloc(sizeof(MonitorGetRamdumpInfoReply),
                  MONITOR_GET_RAMDUMP_INFO_REPLY);
         signal->get_ramdump_info_reply.dumpId = dump_info.dump_id;
         signal->get_ramdump_info_reply.size = dump_info.size;
         signal->get_ramdump_info_reply.sec = dump_info.clock;
         signal->get_ramdump_info_reply.microsec = dump_info.microseconds;

         zzrmm_send2client(&signal, client);
      }
      else
      {
         /* Ramdump info retrieval failed for some reason.
          * Break out of loop. */
         break;
      }
   }

   /* Send endmark */
   signal = alloc(sizeof(MonitorGetRamdumpInfoEndmark),
            MONITOR_GET_RAMDUMP_INFO_ENDMARK);
   signal->get_ramdump_info_endmark.status = ramdump_status(status);
   send(&signal, client->pid);
}

/**
 * @function handle_get_ramdump_request
 * @brief Handle a request to get RAM dump contents
 */
static void
handle_get_ramdump_request(Client * client, union SIGNAL *sig)
{
   MonitorGetRamdumpRequest *req = &sig->get_ramdump_request;
   union SIGNAL *signal;
   struct RamdumpDumpInfo dump_info;
   RamdumpStatus dump_status;
   MonitorStatus mon_status = MONITOR_STATUS_OK;

   assert_client(client);

   /* Get information for requested dump */
   dump_status = ramdump_get_dump_info(req->dumpId, &dump_info);

   if (dump_status == RAMDUMP_SUCCESS)
   {
      /* Dump exists */

      uint32_t offset = 0;
      uint32_t byte_count = dump_info.size;

      while (byte_count > 0)
      {
         int bytes_read;
         uint32_t chunk_size;

         /* There are more bytes left to read, keep reading... */

         /* Read memory in chunks of at most MAX_MEMORY_XFER_CHUNK bytes */
         chunk_size
                  = (byte_count > MAX_MEMORY_XFER_CHUNK)
                                                         ? MAX_MEMORY_XFER_CHUNK
                                                         : byte_count;

         /* Allocate signal and read dump data directly to signal buffer */
         signal = alloc(sizeof(struct MonitorGetRamdumpReply) + chunk_size,
                  MONITOR_GET_RAMDUMP_REPLY);

         bytes_read = ramdump_read_dump(req->dumpId, offset,
                  (void *) signal->get_ramdump_reply.data, chunk_size);

         if (bytes_read == RAMDUMP_EMEMORY_ACCESS_NOK || bytes_read
                  == RAMDUMP_EPOS_OR_DUMP_NOT_AVAILABLE || bytes_read
                  == RAMDUMP_ERAMDUMP_NOT_INITIALIZED || bytes_read < 0)
         {
            /* Unable to read dump, error reported */
            mon_status = ramdump_status(bytes_read);
            free_buf(&signal);
            break;
         }

         if (bytes_read <= 0 || chunk_size < (uint32_t) bytes_read)
         {
            /* No bytes were read, or too many */
            mon_status = MONITOR_STATUS_MEMORY_GET_ERROR;
            free_buf(&signal);
            break;
         }

         /* Fill out the remaining fields of the reply signal */
         signal->get_ramdump_reply.dumpId = req->dumpId;
         signal->get_ramdump_reply.offset = offset;
         signal->get_ramdump_reply.dataCount = bytes_read;

         /* Update loop counters */
         offset += (uint32_t) bytes_read;
         byte_count -= (uint32_t) bytes_read;

         zzrmm_send2client(&signal, client);
      }
   }
   else
   {
      /* Unable to get dump info for specified dump_id */
      mon_status = ramdump_status(dump_status);
   }

   /* Send endmark */
   signal
            = alloc(sizeof(MonitorGetRamdumpEndmark),
                    MONITOR_GET_RAMDUMP_ENDMARK);
   signal->get_ramdump_endmark.status = mon_status;
   send(&signal, client->pid);
}


void
(*tp_settings)(union SIGNAL **sig, TracepointManagerInfo **tpm_info) = NULL;

static void
get_tp_settings(union SIGNAL **sig, Client *client)
{
   if (tp_settings)
   {
      tp_settings(sig, &client->tracepoint_manager_info);
   }
   else
   {
      (*sig)->get_tp_conf_info_reply.nbrOfTracepointActions = 0;
      (*sig)->get_tp_conf_info_reply.nbrOfTracepointVariables = 0;
      (*sig)->get_tp_conf_info_reply.nbrOfTracepoints = 0;
      (*sig)->get_tp_conf_info_reply.tracepointBufferSize = 0;
      (*sig)->get_tp_conf_info_reply.tracepointBufferType = MONITOR_TRACEPOINT_BUFFER_NONE;
   }
}

static void
handle_get_tp_conf_request(Client *client, union SIGNAL **sig)
{
   union SIGNAL *reply;

   reply = alloc(sizeof(RmmGetTPConfInfoReply), RMM_GET_TP_CONF_INFO_REPLY);
   get_tp_settings(&reply, client);
   send(&reply, (*sig)->get_tp_conf_info_request.offspringPid);
}

static int
handle_get_connected_clients_request(Client *client, union SIGNAL *sig)
{
   MonitorStatus status = MONITOR_STATUS_OK;
   union SIGNAL *get_offsprings;
   union SIGNAL *get_tp_info;
   union SIGNAL *reply;
   union SIGNAL *endmark;
   RmmGetOffspringsReply *offspring = NULL;
   RmmGetTPConfInfoReply *tp_info = NULL;
   MonitorBoolean gotEndmark = False;
   SIGSELECT get_offsprings_filter[] =
   { 2,
     RMM_GET_OFFSPRINGS_REPLY,
     RMM_GET_OFFSPRINGS_ENDMARK };
   SIGSELECT get_tp_info_filter[] =
   { 3,
     RMM_GET_TP_CONF_INFO_REPLY,
     RMM_GET_TP_CONF_INFO_REQUEST,
     OS_ATTACH_SIG };

   /*
    * Ask the monitor to send all offsprings. For each reported offspring:
    * - Copy the offspring information given by the monitor
    * - Request the tracepoint information from the offspring and copy it.
    * - Create a reply signal and send the copied information to the client.
    */
   get_offsprings = alloc(sizeof(SIGSELECT), RMM_GET_OFFSPRINGS_REQUEST);
   send(&get_offsprings, client->rmm_);
   while (!gotEndmark)
   {
      get_offsprings = receive_from(10000, get_offsprings_filter, client->rmm_);
      if (get_offsprings == NIL)
      {
         status = MONITOR_STATUS_INTERNAL_ERROR;
         goto done;
      }
      switch (get_offsprings->sig_no)
      {
         case RMM_GET_OFFSPRINGS_REPLY:
            offspring = &get_offsprings->get_offsprings_reply;
            /* The monitor has no knowledge about the tracepoints the
             * offsprings. Request the tracepoints configurations directly
             * from the reported offspring, if it is another offspring than
             * this.
             */
            if (offspring->offspringPid != current_process())
            {
               OSATTREF attref;
               MonitorBoolean Found = False;
               
               get_tp_info = alloc(sizeof(RmmGetTPConfInfoRequest),
                                        RMM_GET_TP_CONF_INFO_REQUEST);
               attref = attach(NULL, offspring->offspringPid);
               get_tp_info->get_tp_conf_info_request.offspringPid
                        = current_process();
               send_w_s(&get_tp_info,
                     client->rmm_,
                     offspring->offspringPid);

               while (Found == False)
               {
                  get_tp_info = receive_w_tmo(10000, get_tp_info_filter);

                  if (get_tp_info == NIL)
                  {
                     status = MONITOR_STATUS_INTERNAL_ERROR;
                     free_buf(&get_offsprings);
                     goto done;
                  }
                  switch (get_tp_info->sig_no)
                  {
                     case RMM_GET_TP_CONF_INFO_REQUEST:
                        handle_get_tp_conf_request(client, &get_tp_info);
                        free_buf(&get_tp_info);
                        break;
                     case RMM_GET_TP_CONF_INFO_REPLY:
                        if (sender(&get_tp_info) == offspring->offspringPid)
                        {
                           tp_info = &get_tp_info->get_tp_conf_info_reply;
                           Found = True;
                        }
                        else
                        {
                           status = MONITOR_STATUS_INTERNAL_ERROR;
                           ramlog_printf("(0x%x): Error - received RMM_GET_TP_CONF_INFO_REPLY "
                                         "from unexpected process(0x%x)\n",
                                         current_process(), offspring->offspringPid);
                           free_buf(&get_tp_info);
                           free_buf(&get_offsprings);
                           goto done;
                        }
                        break;
                     case OS_ATTACH_SIG:
                        /*
                         * Offspring died. Ignore this offspring and continue with
                         * other offsprings
                         */
                        goto offspring_died;
                  }
               }
               detach(&attref);
            }
            else
            {
               /* The reported offspring is this. Gather the information
                * without sending a request.
                */
               get_tp_info = alloc(sizeof(RmmGetTPConfInfoReply), 0);
               get_tp_settings(&get_tp_info, client);
               tp_info = &get_tp_info->get_tp_conf_info_reply;
            }

            /* Create a reply signal and copy both the offspring and the tracepoint
             * information
             */
            reply = alloc(sizeof(MonitorGetConnectedClientsReply)
                                   + offspring->nameLength,
                          MONITOR_GET_CONNECTED_CLIENTS_REPLY);
            reply->get_connected_clients_reply.clientPid = offspring->clientPid;
            reply->get_connected_clients_reply.scope = offspring->scope;
            reply->get_connected_clients_reply.interceptScope
                     = offspring->interceptScope;
            reply->get_connected_clients_reply.offspringPid
                     = offspring->offspringPid;
            reply->get_connected_clients_reply.nbrOfBreakpoints
                     = offspring->nbrOfBreakpoints;
            reply->get_connected_clients_reply.nameLength
                     = offspring->nameLength;
            strncpy((char *) reply->get_connected_clients_reply.clientName,
                    (const char *) offspring->clientName,
                    offspring->nameLength);
            reply->get_connected_clients_reply.nbrOfTracepointActions
                     = tp_info->nbrOfTracepointActions;
            reply->get_connected_clients_reply.nbrOfTracepointVariables
                     = tp_info->nbrOfTracepointVariables;
            reply->get_connected_clients_reply.nbrOfTracepoints
                     = tp_info->nbrOfTracepoints;
            reply->get_connected_clients_reply.tracepointBufferSize
                     = tp_info->tracepointBufferSize;
            reply->get_connected_clients_reply.tracepointBufferType
                     = tp_info->tracepointBufferType;
            zzrmm_send2client(&reply, client);
offspring_died:
            free_buf(&get_tp_info);
            free_buf(&get_offsprings);
            break;
         case RMM_GET_OFFSPRINGS_ENDMARK:
            free_buf(&get_offsprings);
            gotEndmark = True;
            break;
      }
   }

   done:
   endmark = alloc(sizeof(MonitorGetConnectedClientsEndmark),
                         MONITOR_GET_CONNECTED_CLIENTS_ENDMARK);
   endmark->get_connected_clients_endmark.status = status;
   send(&endmark, sender(&sig));
   return 0;
}

/*
 * handle_get_supported_counter_types_request(Client *client, union SIGNAL *sig)
 *
 * Signal handler, sends a signals containing
 * supported counter types.
 *
 * client - client.
 * sig - request signal.
 */
static void
handle_get_supported_counter_types_request(Client *client, union SIGNAL *sig)
{
   uint32_t type_count = 0;
   const ProfilingPerfCounterEventType *types;
   union SIGNAL *endmark;
   uint32_t i;

   /* SFK does not support weak references for linking functions
    * and source profiling feature can be disabled for soft kernels. */
#ifndef softOSE
   /* Get the supported event types from the counter API. */
   types = profiling_get_supported_counter_event_types(&type_count);

   /* Send a reply signal to the client for each event type. */
   for (i = 0; i < type_count; i++)
   {
      size_t description_length = strlen(types[i].description);
      OSBUFSIZE reply_size = sizeof(MonitorGetSupportedCounterTypesReply)
                             + description_length;

      union SIGNAL *reply = alloc(reply_size,
                                  MONITOR_GET_SUPPORTED_COUNTER_TYPES_REPLY);

      reply->get_supported_counter_types_reply.counterType = types[i].type;
      reply->get_supported_counter_types_reply.reserved0 = 0;
      strncpy(reply->get_supported_counter_types_reply.name,
                                types[i].description, description_length + 1);
      zzrmm_send2client(&reply, client);
   }

#endif
   /* Send an endmark after all event type replies have been sent. */
   endmark = alloc(sizeof(MonitorGetSupportedCounterTypesEndmark),
                      MONITOR_GET_SUPPORTED_COUNTER_TYPES_ENDMARK);

   endmark->get_supported_counter_types_endmark.status = MONITOR_STATUS_OK;
   zzrmm_send2client(&endmark, client);
}

static void
tracepoint_not_initialized(union SIGNAL *sig, TracepointManagerInfo **tpm_info)
{
   TOUCH(sig);
   TOUCH(tpm_info);
   ramlog_printf("rmm_offspring: Tracepoints are not initialized \n");
   assert(0);
}

static void
init_tracepoint_manager_not_defined(TracepointManagerInfo **tpm_info,
                                    Client *client,
                                    int *flow_ctrl_block,
                                    int32_t flow_ctrl_blocking,
                                    PROCESS client_pid,
                                    PROCESS rmm_pid)
{
   *tpm_info = NULL;
   TOUCH(client);
   TOUCH(flow_ctrl_block);
   TOUCH(flow_ctrl_blocking);
   TOUCH(rmm_pid);
   TOUCH(client_pid);
}

void
ose_rmm_client(void)
{
   union SIGNAL *signal = receive(all_sigs_filter);
   Client client;
#ifdef OSE_SMP
   cpuid_t current_cpu = zzkrn_get_pid_info(current_process(), 13);
#endif

   client.rmm_ = sender(&signal);
   client.pid = signal->rmm_offspring_info.client_;
   client.mm_ = 0;
   client.offspring_info = &signal->rmm_offspring_info;
   client.flow_ctrl_block = client.offspring_info->flow_ctrl_blocking;
   client.flow_ctrl_nblock = client.offspring_info->flow_ctrl_blocking;
   init_tracepoint_manager(&client.tracepoint_manager_info,
                           &client,
                           &(client.flow_ctrl_block),
                           client.offspring_info->flow_ctrl_blocking,
                           client.pid,
                           client.rmm_);

   /*
    * Install a rmm_offspring local error handler.
    * The error handler is used to ignore errors due to bad requests.
    */
   create_error_handler(current_process(), rmm_error_handler, 512);

   /*
    * NOTE: 'signal' is not freed since a copy of '&signal->rmm_offspring_info'
    * is kept in 'client'. 'signal' will be freed when the offspring dies.
    */
   signal = alloc(sizeof(RmmAttSig), OS_ATTACH_SIG);
   signal->att_sig.client = &client;
   attach(&signal, client.pid);
   for (;;)
   {
      PROCESS sender_;
      signal = receive(all_sigs_filter);
      sender_ = sender(&signal);

      if (sender_ != client.pid && sender_ != client.rmm_)
      {
        ramlog_printf ("rmm_offspring: error; received signal from unknown process: 0x%X\n", sender_);
        error (OSE_RMM_EUNEXPECTED_SIGNAL_SENDER);
        free_buf (&signal);
        continue;
      }

      if (sender_ == client.pid)
      {
#ifdef OSE_SMP
         cpuid_t sender_cpu = zzkrn_get_pid_info(sender_, 13);
         if (unlikely(current_cpu != sender_cpu && sender_cpu != ~0U))
         {
            /* Try to follow the client process to the same CPU core
             * as it executes at.
             */

            ose_bind_process(current_process(), sender_cpu);
            current_cpu = sender_cpu;
         }
#endif

         switch (signal->sig_no)
         {
            case OS_ATTACH_SIG:
               ramlog_printf("rmm_offspring: client killed: 0x%X\n", sender_);
            case MONITOR_DISCONNECT_REQUEST:
               ramlog_printf("rmm_offspring: disconnecting: 0x%X\n", sender_);

               /* Kill our self and let the ose_monitor and kernel clean up after us. */
               suicide(&client);
               break;

            case MONITOR_INTERFACE_REQUEST:
            case MONITOR_NAME_REQUEST:
            case MONITOR_NON_BLOCKING_FLOW_CONTROL_REPLY:
               send(&signal, client.rmm_);
               break;

            case MONITOR_BLOCKING_FLOW_CONTROL_REPLY:
               client.flow_ctrl_block
                        += client.offspring_info->flow_ctrl_blocking / 2 + 1;
               break;

            default:
               /* Here is all signals handled that might be disabled. */
               if (likely(is_req_enabled(&client, &signal)))
               {
                  /* Monitor Tracepoint Interface */
                  //#ifdef USE_EPM
                  if (signal->sig_no > 40729 && signal->sig_no < 40774)
                  {
                     tp_handle(signal, &client.tracepoint_manager_info);
                  }
                  //#endif

                  else
                  {
                     switch (signal->sig_no)
                     {
                        /* monitor.sig signals */

                        case MONITOR_GET_BLOCK_INFO_REQUEST:
                           handle_get_block_info_request(&client, signal);
                           break;

                        case MONITOR_GET_ENV_VARS_REQUEST:
                           handle_get_env_vars_request(&client, signal);
                           break;

                        case MONITOR_GET_MEMORY_REQUEST:
                           handle_get_memory_request(&client, signal);
                           break;

                        case MONITOR_GET_POOL_INFO_REQUEST:
                           handle_get_pool_info_request(&client, signal);
                           break;

                        case MONITOR_GET_POOL_SIGNALS_REQUEST:
                           handle_get_pool_signals(&client, signal);
                           break;

                        case MONITOR_GET_PROCESS_INFO_REQUEST:
                           handle_get_process_info(&client, signal);
                           break;

                        case MONITOR_GET_REGISTER_INFO_REQUEST:
                           handle_get_register_info_request(&client, signal);
                           break;

                        case MONITOR_GET_SEGMENT_INFO_REQUEST:
                           handle_get_segment_info_request(&client, signal);
                           break;

                        case MONITOR_GET_SIGNAL_QUEUE_REQUEST:
                           handle_get_signal_queue(&client, signal);
                           break;

                        case MONITOR_GET_STACK_USAGE_REQUEST:
                           handle_get_stack_usage(&client, signal);
                           break;

                        case MONITOR_GET_SYS_PARAM_REQUEST:
                           handle_get_sys_param_request(&client, signal);
                           break;

                           /* Heap signals */
                        case MONITOR_GET_HEAP_INFO_REQUEST:
                           handle_get_heap_info_request(&client, signal);
                           break;

                        case MONITOR_GET_HEAP_FRAGMENT_INFO_REQUEST:
                           handle_get_heap_fragment_info_request(&client,
                                    signal);
                           break;

                        case MONITOR_GET_HEAP_BUFFERS_REQUEST:
                           handle_get_heap_buffers_request(&client, signal);
                           break;
#ifdef OSE_SMP
                           /* profiler enable/disable execution unit Signals */
                        case MONITOR_GET_PROFILING_EXECUTION_UNIT_ENABLED_REQUEST:
                           handle_get_prof_eu_enabled_request(&client, signal);
                           break;

                        case MONITOR_SET_PROFILING_EXECUTION_UNIT_ENABLED_REQUEST:
                           handle_set_prof_eu_enabled_request(&client, signal);
                           break;
#endif
                         /* Get supported counter types for counter profiling */
                        case MONITOR_GET_SUPPORTED_COUNTER_TYPES_REQUEST:
                           handle_get_supported_counter_types_request(&client,
                                                                        signal);
                           break;

                           /* monitor_actions.sig Signals */

                        case MONITOR_CREATE_PROCESS_REQUEST:
                           handle_create(&client, signal);
                           break;

                        case MONITOR_KILL_SCOPE_REQUEST:
                           handle_kill(&client, signal);
                           break;

                        case MONITOR_SEND_SIGNAL_REQUEST:
                           handle_send(&client, signal);
                           break;

                        case MONITOR_SET_ENV_VAR_REQUEST:
                           handle_set_env_var_request(&client, signal);
                           break;

                        case MONITOR_SET_PRIORITY_REQUEST:
                           handle_set_priority_request(&client, signal);
                           break;

                        case MONITOR_SIGNAL_SEMAPHORE_REQUEST:
                           handle_signal_semaphore_request(&client, signal);
                           break;

                        case MONITOR_START_SCOPE_REQUEST:
                           handle_start(&client, signal);
                           break;

                        case MONITOR_STOP_SCOPE_REQUEST:
                           handle_stop(&client, signal);
                           break;

                        case MONITOR_SET_EXECUTION_UNIT_REQUEST:
                           handle_set_execution_unit(&client, signal);
                           break;
                           
                           // TODO: handle_set_sys_param_reques
                           
                           /* Ramdump signals */
                        case MONITOR_GET_RAMDUMP_REQUEST:
                           handle_get_ramdump_request(&client, signal);
                           break;

                        case MONITOR_GET_RAMDUMP_INFO_REQUEST:
                           handle_get_ramdump_info_request(&client, signal);
                           break;

                        case MONITOR_GET_CONNECTED_CLIENTS_REQUEST:
                           handle_get_connected_clients_request(&client, signal);
                           break;

                        case MONITOR_ATTACH_REQUEST:
                           /* Just tap out the scope and let the rmm take care of the rest
                            */
                           if (client.tracepoint_manager_info)
                           {
                              client.tracepoint_manager_info->scope = ((MonitorAttachRequest *)signal)->scope;
                           }
                           /*I have all intention to go to the default from here. */
                        default:
                           /* We don't know how to deal with this signal so pass it on
                            * to the main monitor process.
                            */
                           send(&signal, client.rmm_);
                           break;
                     }
                  }
               }
               else
               {
                  ramlog_printf("rmm: *** Disabled signal %u\n", signal->sig_no);
               }
               break;
         }
      }
      else if (sender_ == client.rmm_)
      {
         switch (signal->sig_no)
         {
            // TODO: this should not be needed when everything is handled in this process.
            case MONITOR_BLOCKING_FLOW_CONTROL_REQUEST:
               signal->sig_no = MONITOR_BLOCKING_FLOW_CONTROL_REPLY;
               send(&signal, client.rmm_);
               break;

            case MONITOR_GET_PROCESS_TRACE_REPLY:
            case MONITOR_GET_SEND_TRACE_REPLY:
            case MONITOR_GET_RECEIVE_TRACE_REPLY:
            case MONITOR_GET_CREATE_TRACE_REPLY:
            case MONITOR_GET_KILL_TRACE_REPLY:
            case MONITOR_GET_SWAP_TRACE_REPLY:
            case MONITOR_GET_ERROR_TRACE_REPLY:
            case MONITOR_GET_BIND_TRACE_REPLY:
            case MONITOR_GET_USER_TRACE_REPLY:
            case MONITOR_GET_ALLOC_TRACE_REPLY:
            case MONITOR_GET_FREE_BUF_TRACE_REPLY:
            case MONITOR_GET_TIMEOUT_TRACE_REPLY:
            case MONITOR_GET_LOSS_TRACE_REPLY:
               zzrmm_send2client(&signal, &client);
               break;
            case RMM_GET_TRACEPOINTS_REQUEST:
               tp_handle(signal, &client.tracepoint_manager_info);
               break;

            case RMM_GET_TRACEPOINT_ACTIONS_REQUEST:
               tp_handle(signal, &client.tracepoint_manager_info);
               break;

            case RMM_GET_TRACEPOINT_VARIABLES_REQUEST:
               tp_handle(signal, &client.tracepoint_manager_info);
               break;

            case RMM_GET_TRACEPOINT_ANNOTATIONS_REQUEST:
               tp_handle(signal, &client.tracepoint_manager_info);
               break;

            case RMM_GET_TP_CONF_INFO_REQUEST:
               handle_get_tp_conf_request(&client, &signal);
               break;

            case MONITOR_ATTACH_REPLY:
               if (client.tracepoint_manager_info && ((MonitorAttachReply *)signal)->status != MONITOR_STATUS_OK)
               {
                  client.tracepoint_manager_info->scope.type = MONITOR_SCOPE_VOID;
               }
            default:
               send(&signal, client.pid);
               break;
         }
      }
      else
      {
         ramlog_printf(
                  "rmm: got signal from unknown process: pid: 0x%X signal: 0x%X signo: %u",
                  sender_, signal, signal->sig_no);
      }

      if (signal)
      {
         free_buf(&signal);
      }
   }
}
