/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2010-2011 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef RMM_TRACEPOINT_H_
#define RMM_TRACEPOINT_H_

#include "ose.h"
#include "monitor_tracepoint.sig"
#include "list.h"
#include "tracepoint_buffer.h"
#include "epm.h"
#include "atomic.h"

#define DEFAULT_TRACE_BUFFER_SIZE (4096)

enum state {
   INIT,
   CONFIGURED,
   RUNNING,
   FULL_PASSCNT,
   STOP,
};

static const uint32_t save_mem_header = sizeof(uint64_t) + sizeof(uint16_t);

/*Pointer to a memcpy function used in callbacks.*/
extern int
(*tracepoint_memcpy)(void * to, const void *from, size_t size);


typedef struct TracepointManagerInfo
{
   List *enabled_tracepoint_list;
   List *disabled_tracepoint_list;
   List *variables_list;
   int64_t *variable_table;
   uint32_t tracepoint_id_counter;
   uint32_t action_id_counter;
   uint32_t variable_id_counter;
   void *buffer;
   enum MonitorTraceStatus status;
   volatile enum state state;
   TraceBuffer *trace_buffer_handle;
   MonitorScope scope;
   uint32_t nr_of_actions;
   int32_t *block_flow_ctrl_counter;
   uint32_t block_flow_ctrl_limit;
   PROCESS client_pid;
   void *client;
   PROCESS rmm_pid;
   uint32_t configured_size;
   uint32_t convert_bytecode_variable_ids;
   uint32_t *active_callbacks;
} TracepointManagerInfo;

typedef struct TracepointAnnotations
{
   uint32_t no_of_annotations;
   uint32_t annotations_size;
   char annotations[1];
} TracepointAnnotations;

typedef struct Tracepoint
{
   ListLink link;
   MonitorTracepoint id;
   uint32_t address;
   List* action_list;
   TracepointAnnotations *annotations;
   MonitorScope scope;
   uint32_t enabled;
   uint32_t passCount;
   atomic_t passCountCurr;
   uint32_t tracepointType;
   uint32_t byteCodeLength;
   uint32_t byteCodeUpdated;
   uint8_t byteCode[1];
} Tracepoint;

typedef struct ExecutionPointTraceInfo
{
   ListLink link;
   struct EpmExecutionPoint ep;
   uint32_t address;
   TracepointManagerInfo *tpmi;
   Tracepoint tp;
} ExecutionPointTraceInfo;

typedef struct TracepointAction
{
   ListLink link;
   MonitorTracepoint tracepointId;
   MonitorTracepointAction actionId;
   MonitorTracepointAction whileSteppingId;
   List* action_list;
   uint32_t size;
   uint32_t actionType;
   uint32_t byteCodeUpdated;
   ExecutionPointTraceInfo *ept;
   MonitorTracepointActionData tracepointAction;
} TracepointAction;

typedef struct TracepointVariable
{
   ListLink link;
   MonitorTracepointVariable id;
   int64_t initial_value;
   volatile int64_t value;
   uint32_t client_side_id;
} TracepointVariable;

struct whilestepping_s
{
   struct EpmExecutionPoint ep;
   List* action_list;
   int stepcount;
   TracepointManagerInfo *tpmi;
   MonitorTracepoint tracepointId;
};


#endif /* RMM_TRACEPOINT_H_*/
