/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2010-2011 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#include "rmm_utility.h"

static Boolean
check_pid_bid_collision(PROCESS pid0, PROCESS bid0)
{
   struct OS_pid_list *pid_list;
   OSBUFSIZE       pid_count = 0;
   PROCESS        *pid;

   if (pid_list = get_pid_list(bid0), pid_list)
   {
      pid_count = pid_list->count;
      pid = pid_list->list;

      for (; pid_count; pid_count--, pid++)
      {
         if (*pid == pid0)
         {
            free_buf((union SIGNAL **) &pid_list);
            return True;
         }
      }
      free_buf((union SIGNAL **) &pid_list);
   }
   return False;
}

static Boolean
check_bid_sid_collision(PROCESS bid, PROCESS sid)
{
   return (get_segid(bid) == sid);
}

static Boolean
check_pid_sid_collision(PROCESS pid, PROCESS sid)
{
   return (get_segid(pid) == sid);
}

OSBOOLEAN
zzrmm_process_in_scope(PROCESS pid, MonitorScope * scope)
{
   if (scope->type == MONITOR_SCOPE_VOID)
   {
      /* Scope is empty */
      return (OSBOOLEAN) 0;
   }

   if (scope->type == MONITOR_SCOPE_SYSTEM)
   {
      return (OSBOOLEAN) 1;
   }

   if (scope->type == MONITOR_SCOPE_ID && scope->id == pid)
   {
      return (OSBOOLEAN) 1;
   }

   if (scope->type == MONITOR_SCOPE_BLOCK_ID && check_pid_bid_collision(pid, scope->id))
   {
      return (OSBOOLEAN) 1;
   }

   if (scope->type == MONITOR_SCOPE_SEGMENT_ID && check_pid_sid_collision(pid, scope->id))
   {
      return (OSBOOLEAN) 1;
   }

   return (OSBOOLEAN) 0;
}

Boolean
zzrmm_check_scope_collision(MonitorScope * existing_scope, MonitorScope * new_scope)
{
   if (existing_scope->type == MONITOR_SCOPE_VOID || new_scope->type == MONITOR_SCOPE_VOID)
   {
      /* An empty scope never collides. */
      return False;
   }

   /* We know now that neither the requested nor the existing
    * scope is empty. */
   if (existing_scope->type == MONITOR_SCOPE_SYSTEM || new_scope->type == MONITOR_SCOPE_SYSTEM)
   {
      /* We have a collision. */
      return True;
   }

   if (existing_scope->type == new_scope->type && existing_scope->id == new_scope->id)
   {
      /* We have a collision. */
      return True;
   }

   if ((existing_scope->type == MONITOR_SCOPE_ID &&
        new_scope->type == MONITOR_SCOPE_BLOCK_ID &&
        check_pid_bid_collision(existing_scope->id, new_scope->id)) ||
       (existing_scope->type == MONITOR_SCOPE_BLOCK_ID &&
        new_scope->type == MONITOR_SCOPE_ID && check_pid_bid_collision(new_scope->id, existing_scope->id)))
   {
      return True;
   }

   if ((existing_scope->type == MONITOR_SCOPE_BLOCK_ID &&
        new_scope->type == MONITOR_SCOPE_SEGMENT_ID &&
        check_bid_sid_collision(existing_scope->id, new_scope->id)) ||
       (existing_scope->type == MONITOR_SCOPE_SEGMENT_ID &&
        new_scope->type == MONITOR_SCOPE_BLOCK_ID && check_bid_sid_collision(new_scope->id, existing_scope->id)))
   {
      return True;
   }

   if ((existing_scope->type == MONITOR_SCOPE_SEGMENT_ID &&
        new_scope->type == MONITOR_SCOPE_ID &&
        check_pid_sid_collision(new_scope->id, existing_scope->id)) ||
       (existing_scope->type == MONITOR_SCOPE_ID &&
        new_scope->type == MONITOR_SCOPE_SEGMENT_ID && check_pid_sid_collision(existing_scope->id, new_scope->id)))
   {
      return True;
   }

   return False;
}

/**
 * @function get_scope_intersection
 * @brief Calculate the intersection between two scopes.
 */
MonitorScope
zzrmm_get_scope_intersection(MonitorScope * scope_1, MonitorScope * scope_2)
{
   MonitorScope resulting;

   if (zzrmm_check_scope_collision(scope_1, scope_2))
   {
      /* The scopes have some intersection, calculate it. */
      if (scope_1->type <= scope_2->type)
      {
         resulting.type = scope_2->type;
         resulting.id = scope_2->id;
      }
      else
      {
         resulting.type = scope_1->type;
         resulting.id = scope_1->id;
      }
   }
   else
   {
      resulting.type = MONITOR_SCOPE_VOID;
      resulting.id = 0;
   }
   return resulting;
}
