/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2010-2011 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#ifndef TRACEPOINT_BUFFER_H_
#define TRACEPOINT_BUFFER_H_

#include "ose.h"
#include "monitor.sig"
#include "monitor_tracepoint.sig"

#include <stdint.h>
#include <stdlib.h>

#include "list.h"

enum tp_type {
   TP,
   COLLECT,
};

enum collect_e {
   COLLECT_REGISTER,
   COLLECT_MEMORY,
   COLLECT_VARIABLE,
   COLLECT_ERROR,
};

typedef struct MemoryType
{
   uint64_t address;
   uint16_t length;
   uint8_t data[1];
} MemoryType;

typedef struct RegisterType
{
   int dummy;
} RegisterType;

typedef struct VariableType
{
   int64_t value;
   uint32_t id;
} VariableType;

typedef struct Error_s
{
   uint32_t errorcode;
   uint32_t extra;
}TError;

typedef struct Tp_s
{
   uint32_t process;
   uint32_t timestamp_hi;
   uint32_t timestamp_lo;
   uint32_t tpid;
   TError error;
} Tp;

/*type = MEMORY; REGISTER; VARIABLE;*/

typedef struct Collects_s
{
   uint32_t type;
   union
   {
      MemoryType mem;
      RegisterType reg;
      VariableType var;
      TError error;
   } u;

} Collect;


typedef struct TraceEntryHead_s
{
   ListLink dlink;
   uint32_t real_size;
   uint32_t data_size;
   uint32_t frameid;
   uint32_t commit;
   uint32_t type;
   uint32_t padding;
} TraceEntryHead;


typedef struct TraceEntry_s
{
   ListLink dlink;
   uint32_t real_size;
   uint32_t data_size;
   uint32_t frameid;
   uint32_t commit;
   uint32_t type;
   uint32_t padding;
   union
   {
      Tp tp;
      Collect collect;
   } u;
   uint8_t data[1];
} TraceEntry;

typedef struct TraceBuffer_s
{
   OSADDRESS start;
   OSADDRESS end;
   List *dlink;
   OSADDRESS next_free;

   uint32_t junk;
   void *add_junkpoint;
   uint32_t size;
   int freespace;
   uint32_t frames;
   uint32_t created;
   uint32_t sorted;
   enum MonitorTracepointBufferTypeValue buffer_type;   /* Circular or Linjer*/
} TraceBuffer;

enum TraceBuferCommitStatus
{
   TRACE_ENTRY_COMMITED,
   TRACE_ENTRY_NOT_COMMITED,
};

enum TraceConfigTag
{
   TRACEBUFFER_CONFIG_BUFFER_TYPE,
};

TraceBuffer *
tracepoint_buffer_init(void *buffer, uint32_t trace_size);

void *
tracepoint_buffer_destroy(TraceBuffer *trace);

void
tracepoint_buffer_clear(TraceBuffer *trace);

enum MonitorTraceStatus
tracepoint_buffer_alloc(TraceBuffer *trace,
                        uint32_t tracepoint_id,
                        size_t size,
                        TraceEntry **buff);

void
tracepoint_buffer_config(TraceBuffer *trace, uint32_t tag, uint32_t val);

void
tracepoint_buffer_sort(TraceBuffer *tracearray);

#endif /* TRACEPOINT_BUFFER_H_ */
