/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2010-2011 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */


#include "tracepoint_buffer.h"
#include "tracepoint_bytecode.h"
#include "string.h"

#define TP_STACK_SIZE 		(32)

#define TP_OP_FLOAT  		(0x01)
#define TP_OP_ADD		(0x02)
#define TP_OP_SUB  		(0x03)
#define TP_OP_MUL		(0x04)
#define TP_OP_DIV_SIGNED	(0x05)
#define TP_OP_DIV_UNSIGNED  	(0x06)
#define TP_OP_REM_SIGNED  	(0x07)
#define TP_OP_REM_UNSIGNED 	(0x08)
#define TP_OP_LSH  		(0x09)
#define TP_OP_RSH_SIGNED  	(0x0a)
#define TP_OP_RSH_UNSIGNED  	(0x0b)
#define TP_OP_LOG_NOT  		(0x0e)
#define TP_OP_BIT_AND  		(0x0f)
#define TP_OP_BIT_OR  		(0x10)
#define TP_OP_BIT_XOR  		(0x11)
#define TP_OP_BIT_NOT  		(0x12)
#define TP_OP_EQUAL  		(0x13)
#define TP_OP_LESS_SIGNED  	(0x14)
#define TP_OP_LESS_UNSIGNED 	(0x15)
#define TP_OP_EXT 		(0x16)
#define TP_OP_ZERO_EXT  	(0x2a)
#define TP_OP_REF8  		(0x17)
#define TP_OP_REF16  		(0x18)
#define TP_OP_REF32  		(0x19)
#define TP_OP_REF64  		(0x1a)
#define TP_OP_REF_FLOAT  	(0x1b)
#define TP_OP_REF_DOUBLE  	(0x1c)
#define TP_OP_REF_LONG_DOUBLE   (0x1d)
#define TP_OP_L_TO_D  		(0x1e)
#define TP_OP_D_TO_L  		(0x1f)
#define TP_OP_DUP  		(0x28)
#define TP_OP_SWAP  		(0x2b)
#define TP_OP_POP  		(0x29)
#define TP_OP_IF_GOTO  		(0x20)
#define TP_OP_GOTO  		(0x21)
#define TP_OP_CONST8  		(0x22)
#define TP_OP_CONST16  		(0x23)
#define TP_OP_CONST32  		(0x24)
#define TP_OP_CONST64  		(0x25)
#define TP_OP_REG  		(0x26)
#define TP_OP_TRACE  		(0x0c)
#define TP_OP_TRACE_QUICK  	(0x0d)
#define TP_OP_TRACE16  		(0x30)
#define TP_OP_TRACEV            (0x2e)
#define TP_OP_SETV		(0x2d)
#define TP_OP_GETV              (0x2c)
#define TP_OP_END	    	(0x27)
/*TODO this constants may be altered */
#define TP_OP_CONST32_ADDR (0x34)
#define TP_OP_CONST64_ADDR (0x35)

#ifdef PPC
/* Register lookup functions. */
extern int
read_register_for_id(struct cpu_core *dump, OSADDRESS gdb_reg_id, OSADDRESS *address);
#else
/* Dummy register lookup function */
static int
read_register_for_id(void *dump, OSADDRESS gdb_reg_id, OSADDRESS *value)
{
   TOUCH(dump);
   TOUCH(gdb_reg_id);
   TOUCH(value);
   return 0;
}
#endif

#if defined(softOSE) || defined(__i386)

#else


/************** FUNCTION ************************************************/

static int
tracepoint_get_mem(uint64_t *ret, uint64_t addr, size_t size)
{
   uint64_t temp = 0;
   int status = tracepoint_memcpy(&temp, (void *)(int)addr, size);
#ifdef BIG_ENDIAN
   temp >>= ((8 - size) * 8);
#endif
   *ret = temp;
   return status;
}

static int
trace_mem(TraceBuffer *trace, uint32_t frameid, uint64_t addr, size_t size)
{
   TraceEntry *entry;
   enum MonitorTraceStatus status;
   MemoryType *mem;
   int tstatus = 0;
   status = tracepoint_buffer_alloc(trace,
                                    0,
                                    sizeof(Collect) + size,
                                    &entry);
   if (TRACEPOINT_RUNNING == status)
   {
      mem = &entry->u.collect.u.mem;
      entry->type = COLLECT;
      entry->frameid = frameid;
      entry->u.collect.type = COLLECT_MEMORY;

      mem->address = addr;
      mem->length = size;
      tstatus = tracepoint_memcpy(mem->data, (void *) (int)addr, size);

      entry->commit = TRACE_ENTRY_COMMITED;
   }
   return tstatus;
}

static int
trace_variable(TraceBuffer *trace, uint32_t frameid, int64_t *tvariable ,uint32_t id)
{
   TraceEntry *entry;
   enum MonitorTraceStatus status;
   VariableType *var;

   status = tracepoint_buffer_alloc(trace, 0, sizeof(Collect), &entry);
   if (TRACEPOINT_RUNNING == status)
   {
      var = &entry->u.collect.u.var;
      entry->type = COLLECT;
      entry->frameid = frameid;
      entry->u.collect.type = COLLECT_VARIABLE;

      var->id = id;
      var->value = tvariable[id];

      entry->commit = TRACE_ENTRY_COMMITED;
   }
   return 0;
}

static uint32_t 
get_variable_id(TracepointManagerInfo *tpm_info, uint32_t client_side_id)
{
   TracepointVariable *variable;

   /* Traverse the list of variables looking for the ID */
   variable = list_head(tpm_info->variables_list);
   while (variable)
   {
      if (variable->client_side_id == client_side_id)
      {
         return (uint32_t)variable->id;
      }
      variable = list_next(tpm_info->variables_list, &variable->link);
   }
   return 0;
}

unsigned long
tracepoint_bytecode_update_variable_ids( TracepointManagerInfo *tpm_info,
                                         unsigned char *tp_bytecode,
                                         int max_len,
                                         uint64_t *extra)
{
   int tp_pc;
   uint32_t argument;
   uint32_t variable_id; 

   tp_pc = 0;

   while (tp_pc < max_len)
   {
      switch (tp_bytecode[tp_pc]) {
      /* Operations have different bytecode sizes, so we need to advance
       * the pc based on the op code.
       */
      case TP_OP_FLOAT:
      case TP_OP_ADD:
      case TP_OP_SUB:
      case TP_OP_MUL:
      case TP_OP_DIV_SIGNED:
      case TP_OP_DIV_UNSIGNED:
      case TP_OP_REM_SIGNED:
      case TP_OP_REM_UNSIGNED:
      case TP_OP_LSH:
      case TP_OP_RSH_SIGNED:
      case TP_OP_RSH_UNSIGNED:
      case TP_OP_LOG_NOT:
      case TP_OP_BIT_AND:
      case TP_OP_BIT_OR:
      case TP_OP_BIT_XOR:
      case TP_OP_BIT_NOT:
      case TP_OP_EQUAL:
      case TP_OP_LESS_SIGNED:
      case TP_OP_LESS_UNSIGNED:
      case TP_OP_REF8:
      case TP_OP_REF16:
      case TP_OP_REF32:
      case TP_OP_REF64:
      case TP_OP_REF_FLOAT:
      case TP_OP_REF_DOUBLE:
      case TP_OP_REF_LONG_DOUBLE:
      case TP_OP_L_TO_D:
      case TP_OP_D_TO_L:
      case TP_OP_DUP:
      case TP_OP_SWAP:
      case TP_OP_POP:
      case TP_OP_END:
         break;
      case TP_OP_EXT:
      case TP_OP_ZERO_EXT:
         tp_pc++;
         break;
      case TP_OP_IF_GOTO:
      case TP_OP_GOTO:
         tp_pc+=2;
         break;
      case TP_OP_CONST8:
         tp_pc++;
         break;
      case TP_OP_CONST16:
         tp_pc+=2;
         break;
      case TP_OP_CONST32:
      case TP_OP_CONST32_ADDR:
         tp_pc+=4;
         break; 
      case TP_OP_CONST64:
      case TP_OP_CONST64_ADDR:
         tp_pc+=8;
         break;
      case TP_OP_REG:
         tp_pc+=2;
         break;
      case TP_OP_TRACE:
         break;
      case TP_OP_TRACE_QUICK:
         tp_pc++;
         break;
      case TP_OP_TRACE16:
         tp_pc+=2;
         break;
      case TP_OP_TRACEV:
      case TP_OP_GETV:
      case TP_OP_SETV:
         /* replace ID */
         argument = tp_bytecode[tp_pc+1] << 8;
         argument += tp_bytecode[tp_pc+2];

         variable_id = get_variable_id(tpm_info, argument);
         if (variable_id == 0)
         {
            *extra = argument;
            return(TRACEPOINT_ERROR_BAD_TVARIABLE_ADDRESS);
         }
         tp_bytecode[tp_pc+2] = (unsigned char) (variable_id & 0x00FF);
         tp_bytecode[tp_pc+1] = (unsigned char) ( (variable_id >> 8) & 0x00FF);

         tp_pc+=2; 
         break;
      default:
         *extra = tp_bytecode[tp_pc];
         return(TRACEPOINT_ERROR_UNKNOWN_OP_CODE); 
      }
      /* Advance to next operation */
      tp_pc++;
   }
   return TRACEPOINT_OK;
}


unsigned long
tracepoint_bytecode_eval( TracepointManagerInfo *tpm_info,
                          uint32_t frameid,
                          unsigned char *tp_bytecode,
                          int max_len,
                          struct cpu_core *regbank,
                          uint64_t *stack_top,
                          int64_t *tvariable,
                          uint32_t tvariable_length)
{
   int tp_pc;
   uint64_t stack[TP_STACK_SIZE];
   int tp_sp = TP_STACK_SIZE;

   uint64_t value,argument;
   uint32_t reg_value = 0;

    tp_pc = 0;
    stack[--tp_sp]= 0xfeaf; /* Dummy at stacktop */


while ( tp_pc  < max_len  && tp_sp > 0 && tp_sp < TP_STACK_SIZE) {

	switch (  tp_bytecode[tp_pc]) {
	case TP_OP_REG:
		argument =   tp_bytecode[++tp_pc] << 8 ;
		argument +=  tp_bytecode[++tp_pc] ;
		if ( read_register_for_id(regbank, argument, (OSADDRESS *)&reg_value) )
		{
		   *stack_top = argument;
         return(TRACEPOINT_ERROR_REGISTER);
		}
		stack[--tp_sp] = reg_value;
      break;

        case TP_OP_CONST8:
		tp_sp--;
                stack[tp_sp] = tp_bytecode[++tp_pc] ;
        	break;

        case TP_OP_CONST16:
		tp_sp--;
                stack[tp_sp] = tp_bytecode[++tp_pc] ;
                stack[tp_sp] = (stack[tp_sp] << 8 ) + tp_bytecode[++tp_pc] ;
		break;

        case TP_OP_CONST32:
        case TP_OP_CONST32_ADDR:
		tp_sp--;
                stack[tp_sp] = tp_bytecode[++tp_pc] ;
                stack[tp_sp] = (stack[tp_sp] << 8 ) + tp_bytecode[++tp_pc] ;
                stack[tp_sp] = (stack[tp_sp] << 8 ) + tp_bytecode[++tp_pc] ;
                stack[tp_sp] = (stack[tp_sp] << 8 ) + tp_bytecode[++tp_pc] ;
                break;

	case TP_OP_CONST64:
  case TP_OP_CONST64_ADDR:
		tp_sp--;
                stack[tp_sp] = tp_bytecode[++tp_pc] ;
                stack[tp_sp] = (stack[tp_sp] << 8 ) + tp_bytecode[++tp_pc] ;
                stack[tp_sp] = (stack[tp_sp] << 8 ) + tp_bytecode[++tp_pc] ;
                stack[tp_sp] = (stack[tp_sp] << 8 ) + tp_bytecode[++tp_pc] ;
                stack[tp_sp] = (stack[tp_sp] << 8 ) + tp_bytecode[++tp_pc] ;
                stack[tp_sp] = (stack[tp_sp] << 8 ) + tp_bytecode[++tp_pc] ;
                stack[tp_sp] = (stack[tp_sp] << 8 ) + tp_bytecode[++tp_pc] ;
                stack[tp_sp] = (stack[tp_sp] << 8 ) + tp_bytecode[++tp_pc] ;
        	break;

        case TP_OP_REF8:
		if ( tracepoint_get_mem(&value, stack[tp_sp],1) )
		{
		   *stack_top = stack[tp_sp];
			return(TRACEPOINT_ERROR_MEMORY_READ);
		}
		stack[tp_sp] = value;
		break;

	case TP_OP_REF16:
		if ( tracepoint_get_mem(&value, stack[tp_sp],2) )
      {
         *stack_top = stack[tp_sp];
			return(TRACEPOINT_ERROR_MEMORY_READ);
      }
		stack[tp_sp] = value;
		break;

	case TP_OP_REF32:

		if ( tracepoint_get_mem( &value, stack[tp_sp],4) )
      {
         *stack_top = stack[tp_sp];
			return(TRACEPOINT_ERROR_MEMORY_READ);
      }
		stack[tp_sp] = value;
		break;

	case TP_OP_REF64:
		if ( tracepoint_get_mem( &value, stack[tp_sp],8) )
      {
         *stack_top = stack[tp_sp];
			return(TRACEPOINT_ERROR_MEMORY_READ);
      }
		stack[tp_sp] = value;
		break;


       case TP_OP_EXT:
		argument = tp_bytecode[++tp_pc];
		if ( argument < (sizeof(int64_t ) * 8) ) {
 		     	int64_t svalue = (1 << ( argument - 1 ));
 			stack[tp_sp] &= ((int64_t) 1 << argument) -1 ;
			stack[tp_sp] = (stack[tp_sp] ^ svalue) - svalue;
		}
         	break;

        case TP_OP_MUL:
		stack[tp_sp+1] = stack[tp_sp] * stack[tp_sp+1];
                tp_sp++;
		break;

        case TP_OP_ADD:
		stack[tp_sp+1] = stack[tp_sp] + stack[tp_sp+1];
                tp_sp++;
        	break;

	case TP_OP_SUB:
		stack[tp_sp+1] = stack[tp_sp+1] - stack[tp_sp];
                tp_sp++;
 		break;

	case TP_OP_END:
		/* Done EXIT */
	   if ( stack[TP_STACK_SIZE-1] != 0xfeaf )
	      return(TRACEPOINT_ERROR_STACK_UNDERFLOW);
		*stack_top = stack[tp_sp];
		return(TRACEPOINT_OK);

        case TP_OP_EQUAL:
		stack[tp_sp+1] = (stack[tp_sp+1] == stack[tp_sp] );
                tp_sp++;
		break;

	case TP_OP_LESS_SIGNED:
   		stack[tp_sp+1] = ( (int64_t ) stack[tp_sp+1] < (int64_t ) stack[tp_sp]) ;
                tp_sp++;
 		break;

	case TP_OP_LESS_UNSIGNED:
  		stack[tp_sp+1] = ( stack[tp_sp+1] < stack[tp_sp]) ;
                tp_sp++;
		break;

	case TP_OP_BIT_AND:
		stack[tp_sp+1] = stack[tp_sp] &  stack[tp_sp+1];
                tp_sp++;
	      	break;

	case TP_OP_BIT_OR:
 		stack[tp_sp+1] = stack[tp_sp] |  stack[tp_sp+1];
                tp_sp++;
		break;

	case TP_OP_BIT_XOR:

		stack[tp_sp+1] = stack[tp_sp] ^  stack[tp_sp+1];
                tp_sp++;
		break;

	case TP_OP_BIT_NOT:
		stack[tp_sp] = ~stack[tp_sp]  ;
		break;

	case TP_OP_SWAP:

		value =  stack[tp_sp];
		stack[tp_sp]=  stack[tp_sp+1];
		stack[tp_sp+1] = value;
		break;

	case TP_OP_DUP:
  		value = stack[tp_sp];
		stack[--tp_sp] = value;
		break;

	case TP_OP_POP:
                tp_sp++;
        	break;

        case TP_OP_DIV_SIGNED:
               if ( (int64_t ) stack[tp_sp]) {
                stack[tp_sp+1] = (int64_t ) stack[tp_sp+1] / (int64_t ) stack[tp_sp];
                tp_sp++;
	       }  else {
		       return(TRACEPOINT_ERROR_DIV_BY_ZERO);
	       }
	       break;

	case TP_OP_DIV_UNSIGNED:
               if ( stack[tp_sp]) {
	       stack[tp_sp+1] =  stack[tp_sp+1] /  stack[tp_sp];
               tp_sp++;
	       }  else {
		       return(TRACEPOINT_ERROR_DIV_BY_ZERO);
	       }
	       break;

	case TP_OP_REM_SIGNED:
               if ( (int64_t ) stack[tp_sp]) {
                stack[tp_sp+1] = (int64_t ) stack[tp_sp+1] % (int64_t ) stack[tp_sp];
                tp_sp++;
	       } else {
		       return(TRACEPOINT_ERROR_DIV_BY_ZERO);
	       }
	       break;

	case TP_OP_REM_UNSIGNED:
              if ( stack[tp_sp]) {
		    stack[tp_sp+1] =  stack[tp_sp+1] %  stack[tp_sp];
	            tp_sp++;
	       } else {
		       return(TRACEPOINT_ERROR_DIV_BY_ZERO);
	       }
               break;

	case TP_OP_LSH:
		stack[tp_sp+1] = stack[tp_sp+1] << stack[tp_sp];
                tp_sp++;
		break;

	case TP_OP_RSH_SIGNED:
 		stack[tp_sp+1] =  (int64_t ) stack[tp_sp+1] >>  stack[tp_sp];
                tp_sp++;
		break;

	case TP_OP_RSH_UNSIGNED:
		stack[tp_sp+1] =  stack[tp_sp+1] >>  stack[tp_sp];
                tp_sp++;
		break;

        case TP_OP_LOG_NOT:
		stack[tp_sp] =  ! stack[tp_sp];
		break;

        case TP_OP_ZERO_EXT:
		argument = tp_bytecode[++tp_pc];
		if ( argument < sizeof ( int64_t) * 8 ) {
		  stack[tp_sp] &= ((int64_t) 1 << argument)-1;
		}
		break;

        case TP_OP_IF_GOTO:
 	      	if (  stack[tp_sp++]) {
		tp_pc =  (tp_bytecode[tp_pc+1] << 8 )+ tp_bytecode[tp_pc+2] -1;
		  if ( tp_pc >= max_len ) {
		     *stack_top = tp_pc;
		     return(TRACEPOINT_ERROR_ILLEGAL_GOTO_ADDRESS);
                }
		} else {
		tp_pc +=2;
		}
                break;

        case TP_OP_GOTO:
		tp_pc =  (tp_bytecode[tp_pc+1] << 8 ) + tp_bytecode[tp_pc+2] -1 ;
 		 if ( tp_pc >= max_len ) {
          *stack_top = tp_pc;
		    return(TRACEPOINT_ERROR_ILLEGAL_GOTO_ADDRESS);
		 }
		break;


        case TP_OP_TRACE:

           if (trace_mem( tpm_info->trace_buffer_handle, frameid, stack[tp_sp+1],(size_t) stack[tp_sp]) )
           {
              *stack_top = stack[tp_sp+1];
              return(TRACEPOINT_ERROR_MEMORY_READ);
           }
	              tp_sp +=2;
		      break;


        case TP_OP_TRACE_QUICK:
		argument = tp_bytecode[++tp_pc];
		if ( trace_mem ( tpm_info->trace_buffer_handle, frameid, stack[tp_sp], (size_t) argument))
		{
		   *stack_top = stack[tp_sp];
 		 	return(TRACEPOINT_ERROR_MEMORY_READ);
		}
		break;

        case TP_OP_TRACE16:
		argument = tp_bytecode[++tp_pc] << 8 ;
		argument += tp_bytecode[++tp_pc] ;
		if ( trace_mem ( tpm_info->trace_buffer_handle, frameid, stack[tp_sp], (size_t) argument))
		{
         *stack_top = stack[tp_sp];
 		 	return(TRACEPOINT_ERROR_MEMORY_READ);
		}
		break;



        case TP_OP_TRACEV:
                argument =   tp_bytecode[++tp_pc] << 8 ;
		argument +=  tp_bytecode[++tp_pc] ;
		if ( argument >= tvariable_length )
		{
	       *stack_top = argument;
		   return(TRACEPOINT_ERROR_BAD_TVARIABLE_ADDRESS);
		}
		trace_variable(tpm_info->trace_buffer_handle, frameid, tvariable,argument);
        	break;

        case TP_OP_GETV:
                argument =   tp_bytecode[++tp_pc] << 8 ;
		argument +=  tp_bytecode[++tp_pc] ;
		if ( argument >= tvariable_length )
      {
          *stack_top = argument;
         return(TRACEPOINT_ERROR_BAD_TVARIABLE_ADDRESS);
      }
      stack[--tp_sp] =  tvariable[argument];
        	break;

        case TP_OP_SETV:
                argument =   tp_bytecode[++tp_pc] << 8 ;
		argument +=  tp_bytecode[++tp_pc] ;
		if ( argument >= tvariable_length )
      {
          *stack_top = argument;
         return(TRACEPOINT_ERROR_BAD_TVARIABLE_ADDRESS);
      }
        	tvariable[argument] =   stack[tp_sp];
	       	break;

        case TP_OP_FLOAT:
        case TP_OP_REF_FLOAT:
        case TP_OP_REF_DOUBLE:
        case TP_OP_REF_LONG_DOUBLE:
        case TP_OP_L_TO_D:
        case TP_OP_D_TO_L:
           *stack_top = tp_bytecode[tp_pc];
	        return(TRACEPOINT_ERROR_INSTRUCTION_NOT_IMPLEMENTED);

        default:
           *stack_top = tp_bytecode[tp_pc];
           return(TRACEPOINT_ERROR_UNKNOWN_OP_CODE);



        } /*switch */
	/* next bytecode */
	tp_pc++;

} /* while */
/* some error occurred */

	if ( tp_pc >= max_len )	{
	   *stack_top = tp_pc;
      return(TRACEPOINT_ERROR_PROGRAM_TOO_LARGE);
	}
	if ( tp_sp >= TP_STACK_SIZE ) {
	   *stack_top = tp_sp;
      return(TRACEPOINT_ERROR_STACK_UNDERFLOW);
   }
   return(TRACEPOINT_ERROR_STACK_OVERFLOW);
}


#endif
