/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 1991-2013 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/**
 * @toc OSE:
 * @file ose.h
 * @brief
 * OSE kernel Application Programming Interface (API).
 * @long
 * This file contains declarations to be included in prioritized or
 * background processes written in C.
 *
 * @seealso ose_i.h
 * @seealso ose_spi/ose_spi.h
 * @seealso ose_err/ose_err.h
 *
 */
/*
******************************************************************************
* @(#) File: ose.h		Id: 06/1946370276339265			     *
* @(#) Revision: 3.49.3		Revision date: 13/08/15			     *
*
* The declarations in this file define the OSE kernel's Application
* Level Interface (API).
*
* This file contains declarations to be included in prioritized or
* background processes written in C.
*
* History of changes
*
*	Date	Name		Vers	Change
*	------	----		----	-------------------------------
*
*	930401	Bengt Eliasson	3.0	Initial OSExc release.
*	930806	Bengt Eliasson	3.1	Moved system calls from ose_mmu.h
*	931008	Bengt Eliasson	3.2	Assign_linkhandler() returns OSBOOLEAN.
*	931022	Bengt Eliasson	3.3	Altered the order of fields in the
*                                       OS_pcb struct to avoid alignment
*                                       problems. Added user parameter to
*                                       create_process(). Removed the set_uid()
*                                       call.
*	931214	Bengt Eliasson	3.4	Added the set_segment() call and
*                                       modified the return type of the
*                                       attach_segment() call.
*                                       Added the OSE_LEVEL_x constants.
*	940210	Bengt Eliasson	3.5	Added the set_pcb() call.
*	940220	Bengt Eliasson	3.6	Replaced bad NIL declaration.
*                                       Added conditional C++ support.
*	940821	Bengt Eliasson	3.7	Added the set_suid declaration.
*                                       Modified the OS_pcb struct according
*                                       to generic spec changes.
*	940901	Bengt Eliasson	3.8	Added support for the obsolete
*                                       create_proc() system call present
*                                       in the old os68 Classic kernel.
*                                       Added prototypes for the implementation
*                                       specific system calls extend_pool(), error2(),
*                                       and set_redirection().
*	950901	Bengt Eliasson	3.9	The zz-prototypes for debugger calls are
*                                       available also without the OS_DEBUG flag.
*                                       Added prototypes for the implementation
*                                       specific system calls set_pri_for(),
*                                       set_abstime(), get_abstime(), get_segid(),
*                                       create_event() and signal_event().
*	951115	Bengt Eliasson	3.10	Fixed a C++ syntax error.
*                                       Added the ZZFILE_AT_HEAD constant. It is
*                                       intended only for internal OSE use, and
*                                       it is defined in implementations that
*                                       places the file and line parameters of
*                                       system calls at the beginning of the
*                                       parameter list instead of at the end.
*                                       User code should not attempt to use it.
*                                       Added the implementation dependent
*                                       system calls exit_error_handler() and
*                                       receive_with().
*	960329	Bengt Eliasson	3.11	Removed bad parameters in zzpower_fail().
*	960925	Bengt Eliasson	3.12	Added the implementation dependent
*                                       system calls alloc_nil() and get_sysconf().
*                                       Changed the type of system_tick() to OSTIME.
*	961101	Bengt Eliasson	3.13	Removed a compiler warning in ZZOS_MAIN.
*	                                Added the @-PRIVATE-CALLS-@ tag.
*	970718	Bengt Eliasson	3.14	Added the implementation dependent
*                                       system call receive_sport().
*	970920	Bengt Eliasson	3.15	Added the implementation dependent
*                                       system calls set_sigsize(),
*                                       get_sig_info(), get_pool_list(),
*                                       get_poolcb(), get_stk_poolid(),
*                                       get_sig_poolid(), get_envp() and set_envp().
*       990127 Fredrik Bredberg 3.16    Added zz-prototypes for system
*                                       calls signal_fsem(), signal_sem(),
*                                       wait_fsem(), wait_sem(), error2(),
*                                       receive_with(), get_sysconf(),
*                                       addressee(), sender(), sigsize(),
*                                       set_pri_for() and hunt_from().
*       000525 Fredrik Bredberg 3.17    A major rewrite in order to support
*                                       C++. Credits to Claes Lillieskold
*                                       and Gunnar Eklund for research and
*                                       moral support.
*       010116 Fredrik Bredberg 3.18    The definition of the system type
*                                       is now imported from "osarch.h".
*       010410 Fredrik Bredberg 3.19    Added const to some prototypes.
*                                       Moved some prototypes from the
*                                       private calls section. Declared
*                                       "union SIGNAL;" for both C and C++.
*       011010 Fredrik Bredberg 3.20    Added the implementation dependent
*                                       system call cond_halt().
*       011101 Gunnar Eklund    3.21    Added the implementation dependent
*                                       timeout signal handling system calls.
*                                       Renamed cond_halt() to
*                                       set_halt_condition().
*       020221 Gunnar Eklund    3.22    Let error() and error2() provide
*                                       file and line information also when
*                                       OS_DEBUG is not defined.
*       020313 Gunnar Eklund    3.23    With C++, inline error() and error2()
*                                       only when OS_DEBUG defined.
*       020925 Fredrik Bredberg 3.24    Fixed a C++ incompatibility problem.
*       030522 Fredrik Bredberg 3.25    Moved SPI calls to ose_spi.h and
*                                       OSE Classic related declarations to
*                                       ose_classic.h
*       030925 Gunnar Eklund    3.26    Added srcdoc comments.
*       040115 Fredrik Bredberg 3.27    Added implementation dependent per
*                                       process data handling system calls.
*       040524 Fredrik Bredberg 3.28    Added mutex system calls.
*       040607 Gunnar Eklund    3.29    Added srcdoc comments for mutex.
*       040707 Gunnar Eklund    3.30    A few comments changed.
*       040730 Karl Morner      3.31    Added NO_ZZZSEND to make it possible to
*                                       disable zzzsend name mangling for the
*                                       softOSE environment.
*       041109 Fredrik Bredberg 3.32    Modified the timeslice parameter
*                                       description of create_process().
*       041117 Gunnar Eklund    3.33    A few comments changed.
*       050228 Gunnar Eklund    3.34    Added {} in OSE_MUTEX_INITIALIZER.
*       051109 Gunnar Eklund    3.35    Attach and detach now permitted for
*                                       interrupt processes.
*       070103 Gunnar Eklund    3.36    Added macros OSE_ILLEGAL_PROCESS,
*                                       OSE_ILLEGAL_ATTREF and
*                                       OSE_ILLEGAL_TMOREF.
*       070305 Fredrik Stenling 3.37    Added constants OSREPORTVAL and
*                                       OSREPORTID. Also added new
*                                       prototypes; ose_create_report()
*                                       and ose_set_report_val().
*       070315 Gunnar Eklund    3.38    Added SMP system calls.
*       070509 Gunnar Eklund    3.39    Moved get_env_list(),
*                                       get_sig_poolid(), get_stk_poolid()
*                                       and get_poolcb() from ose_spi.h to
*       070911 Fredrik Bredberg 3.40    Moved get_bid_list(), get_pid_list()
*                                       and get_pool_list() from ose_spi.h to
*                                       ose.h.
*       080415 Fredrik Bredberg 3.41    Moved restore() from ose_spi.h to
*                                       ose.h.
*       081021 Fredrik Bredberg 3.42    Added send_w_sender() and restore_from().
*       091111 Costel Grosu     3.43    Added free_buf_type() macro.
*       091120 Robin Westberg   3.44    Added receive_fsem().
*       100507 Gunnar Eklund    3.45    Improved documentation of
*                                       ose_mutex_destroy().
*       110720 Gunnar Eklund    3.46    Local error handler stacks are not 
*                                       implemented.
*       111006 Gunnar Eklund    3.47    Added the macro OSE_NO_TIMEOUT.
*       111028 Hakan Lindqvist  3.48    Updated doc on hunt().
*       111110 Mats Liljegren   3.49    Updated doc to refer to default
*                                       restrictions.
*                                       Made send_w_sender() appear in
*                                       reference documentation.
*                                       Updated documentation for get_fsem(),
*                                       receive(), send_w_s(), signal_fsem(),
*                                       signal_sem(), wait_fsem()
*       120319 Gunnar Eklund    3.49.1  Added C++ prototype for send_w_sender.
*       130305 Gunnar Eklund    3.49.2  Added ose_get_error_info() and
*                                       ose_reset_error_info().
*       130815 Fredrik Bredberg 3.49.3  Declared receive_sport() as extern "C".
*
*/
#ifndef ZZOSE_H
#define ZZOSE_H

/*
** This symbol designates a prioritized or background process.
*/
/**
 * @macro OSE
 * @declaration #define OSE
 *
 * @brief This manifest constant is defined by ose.h.
 *
 * @long
 * The value of this constant is unimportant. It is intended for use in an
 * #ifdef or #ifndef statement.
 * @long
 * It may be used by an application or an OSE system simulator to find out
 * which OSE header file is being used.
 * @long
 * When a run-time decision is needed, for example in code shared by different
 * process types, use the system call get_ptype(current_process()) instead.
 *
 * @include "ose.h"
 *
 * @restriction
 * The OSE manifest constant is not available to interrupt processes,
 * if the interrupt processes includes ose_i.h instead of ose.h.
 *
 * @seealso OSE_I
 * @seealso OSPP
 * @seealso OSARM
 * @seealso OSMIPS
 * @seealso softOSE
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 * #ifdef OSE
 *   for(;;)
 *     {
 *       // Code, "I am a prioritized process" //
 *     }
 * #else
 *   if(wake_up())
 *     {
 *       // Code, "I am an interrupt process" //
 *     }
 * #endif
 * }
 */
#define OSE

/*
** This include file defines a preprocessor symbol that states the
** current processor type. I.e. it will define either OS68, OSARM,
** OSMIPS, OSPP, softOSE or a similar symbol in future systems.
**
** It may also define a symbol called ZZFILE_AT_HEAD to state that the
** first (hidden) parameters to the debug versions of the system calls
** should be __FILE__ and __LINE__. If ZZFILE_AT_HEAD is not defined,
** file and line info will be passed as the last parameters. Of the
** above listed system types, OS68 and softOSE are the only ones that
** define this symbol.
*/
#include "osarch.h"

/*
******************************************************************************
** Target versions of OS types and functions.
******************************************************************************
*/

/*
** These symbols indicate that definitions previously found in ose.h are
** now moved to other separate files.
**
** New code can be written to adopt to an old ose.h like this:
**
**    #include "ose.h"
**    #ifdef OSE_HAS_SPI
**    #include "ose_spi/ose_spi.h"
**    #endif
**
** Old code can be compiled with the new ose.h/ose_spi.h/ose_classic.h
** files by defining OSE_H_SPI at compile time (i.e. with the compiler
** flag -DOSE_H_SPI).
**
** The same goes for the OSE Classic compatibility (i.e. -DOSE_H_CLASSIC).
*/
/**
 * @macro OSE_HAS_CLASSIC
 *
 * @declaration #define OSE_HAS_CLASSIC
 *
 * @brief Defined if this OSE version contains a separate ose_classic.h header.
 *
 * @long If OSE_HAS_CLASSIC is defined by ose.h, then the header ose_classic.h
 * is available for inclusion.
 *
 * @example
 * #include "ose.h"
 * #ifdef OSE_HAS_CLASSIC
 * #include "ose_spi/ose_classic.h"
 * #endif
 */
#define OSE_HAS_CLASSIC /* Classic definitions are found in ose_classic.h */

/**
 * @macro OSE_HAS_SPI
 *
 * @declaration #define OSE_HAS_SPI
 *
 * @brief Defined if this OSE version contains a separate ose_spi.h header.
 *
 * @long If OSE_HAS_SPI is defined by ose.h, then the header ose_spi.h
 * is available for inclusion.
 *
 * @example
 * #include "ose.h"
 * #ifdef OSE_HAS_SPI
 * #include "ose_spi/ose_spi.h"
 * #endif
 */
#define OSE_HAS_SPI     /* SPI related definitions are found in ose_spi.h */

/*
** C++ needs to know that this is C code.
*/

#ifdef __cplusplus
extern "C" {
#endif

/*
** Basic types.
*/

/**
 * @type OSPRIORITY
 *
 * @declaration typedef unsigned char OSPRIORITY;
 *
 * @brief A process priority.
 *
 * @long Priority values for prioritized processes are in the range 0 .. 31.
 * See the priority parameter of @link "" "create_process" for a
 * more general description of process priorities.
 */
typedef unsigned char OSPRIORITY;

/**
 * @type OSBOOLEAN
 *
 * @declaration typedef unsigned char OSBOOLEAN;
 *
 * @brief A Boolean value, non-zero is true, zero is false.
 */
typedef unsigned char OSBOOLEAN;

/**
 * @type OSVECTOR
 *
 * @declaration typedef unsigned short OSVECTOR;
 *
 * @brief An interrupt vector identity.
 */
typedef unsigned short OSVECTOR;

/**
 * @type PROCESS
 *
 * @declaration typedef unsigned long PROCESS;
 *
 * @brief A process, block, domain or pool identity.
 */
typedef unsigned long PROCESS;

/**
 * @type SIGSELECT
 *
 * @declaration typedef unsigned long SIGSELECT;
 *
 * @brief A signal number.
 */
typedef unsigned long SIGSELECT;

/**
 * @type OSBUFSIZE
 *
 * @declaration typedef unsigned long OSBUFSIZE;
 *
 * @brief The number of bytes in a signal.
 */
typedef unsigned long OSBUFSIZE;

/**
 * @type OSUSER
 *
 * @declaration typedef unsigned long OSUSER;
 *
 * @brief A user number.
 */
typedef unsigned long OSUSER;

/**
 * @type OSERRCODE
 *
 * @declaration typedef unsigned long OSERRCODE;
 *
 * @brief An error code.
 */
typedef unsigned long OSERRCODE;

/**
 * @type OSADDRESS
 *
 * @declaration typedef unsigned long OSADDRESS;
 *
 * @brief A memory address.
 */
typedef unsigned long OSADDRESS;

/**
 * @type OSTIME
 *
 * @declaration typedef unsigned long OSTIME;
 *
 * @brief A time given in milliseconds.
 */
typedef unsigned long OSTIME;

/**
 * @type OSTICK
 *
 * @declaration typedef unsigned long OSTICK;
 *
 * @brief Time as reported by get_systime() and get_ticks().
 */
typedef unsigned long OSTICK;

/**
 * @type OSATTREF
 *
 * @declaration typedef unsigned long OSATTREF;
 *
 * @brief An attach() identity.
 *
 * @seealso attach
 */
typedef unsigned long OSATTREF;

/**
 * @type OSPPDKEY
 *
 * @declaration typedef unsigned long OSPPDKEY;
 *
 * @brief A per process data key.
 *
 * @seealso ose_create_ppdata
 */
typedef unsigned long OSPPDKEY;

/**
 * @type OSFSEMVAL
 *
 * @declaration typedef signed long OSFSEMVAL;
 *
 * @brief A fast semaphore value.
 */
typedef signed long OSFSEMVAL;

/**
 * @type OSSEMVAL
 *
 * @declaration typedef signed long OSSEMVAL;
 *
 * @brief A semaphore value.
 */
typedef signed long OSSEMVAL;

/**
 * @type OSENTRYPOINT
 *
 * @declaration typedef void (OSENTRYPOINT)(void);
 *
 * @brief A process entrypoint.
 */
typedef void (OSENTRYPOINT)(void);

/**
 * @type OSERRH
 *
 * @brief The typedef of a block or process error handler.
 *
 * @long The error handler function will be called with three
 *       parameters. For syntax details, see the description of
 *       @link "" "create_error_handler".
 */
typedef OSADDRESS (OSERRH)(OSBOOLEAN, OSERRCODE, OSERRCODE);

/**
 * @type OSREPORTVAL
 *
 * @declaration typedef signed long OSREPORTVAL;
 *
 * @brief A profiling report value.
 */
typedef signed long OSREPORTVAL;

/**
 * @type OSREPORTID
 *
 * @declaration typedef unsigned long OSREPORTID;
 *
 * @brief A profiling report value identifier.
 */
typedef unsigned long OSREPORTID;

/**
 * @type cpuid_t
 *
 * @declaration typedef unsigned long cpuid_t;
 *
 * @brief Identifies a CPU entity. Also used to report the number of CPUs
 * available in the system.
 *
 * @long Primarily used in SMP (Symmetric Multi Processing) systems.
 *
 * @seealso ose_cpu_id
 * @seealso ose_num_cpus
 */
typedef unsigned long cpuid_t;

/*
** Derived types and miscellaneous constants.
*/

#ifdef __cplusplus
#define OS_PROCESS(name) extern "C" void name(void)
#else
/**
 * @macro OS_PROCESS
 *
 * @declaration #define OS_PROCESS(name)
 *
 * @brief Macro for defining the entrypoint of a process.
 *
 * @long
 * It is defined in ose.h and ose_i.h.
 * @long
 * It is wise to assume that OS_PROCESS is differently defined in ose.h and
 * ose_i.h, even if this is not normally the case.
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @field name The process name.
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   for(;;)
 *     {
 *       // Code //
 *     }
 * }
 */
#define OS_PROCESS(name) void name(void)

/* This macro is obsolete, exist only because the default
 * osemain.c file uses it. DO NOT USE!
 */
#define ZZOS_MAIN(name) void name(void)
#endif

/**
 * @macro NULL
 *
 * @declaration #define NULL
 *
 * @brief The NULL value.
 *
 * @long NULL is standardized zero constant, used throughout in ose.h.
 * The NULL value may also be defined in Std C.
 */
#ifndef NULL
#define NULL 0L
#endif

/**
 * @macro NIL
 *
 * @declaration #define NIL
 *
 * @brief The "no signal" value of signal pointers.
 *
 * @long NIL is a manifest constant, defining a  "no signal" value for
 * SIGNAL pointers.  NIL is defined in ose.h and in ose_i.h.
 * @long
 * NIL is the value returned from receive, receive_w_tmo, receive_from
 * and receive_fsem when no signal buffer could be received.
 * @long
 * NIL is stored in the caller's signal pointer by the send, send_w_s and
 * free_buf system calls.
 * @long
 * NIL is not equivalent to NULL, and should be used only to test on
 * received buffers.
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @restriction
 * NIL is specified such that it is never confused with a
 * legal signal pointer.
 * Furthermore, it is an advantage if NIL points to a valid scratch pad
 * area. This enables the kernel to correctly report several user errors
 * that would otherwise lead to bus error or memory parity error.
 *
 * @example
 * #include "ose.h"
 *
 * static const SIGSELECT any_sig[] = {0};
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *   for(;;)
 *     {
 *       sig = receive_w_tmo(1000, (SIGSELECT *)any_sig);
 *       if(sig != NIL)
 *         {
 *           // Code //
 *           free_buf(&sig);
 *         }
 *     }
 * }
 */
#define NIL ((union SIGNAL *) 0)

/**
 * @macro OSE_ILLEGAL_PROCESS
 *
 * @declaration #define OSE_ILLEGAL_PROCESS 0
 *
 * @brief An illegal process identifier.
 *
 * @long
 * The macro OSE_ILLEGAL_PROCESS may be used to initiate and test application
 * variables of type PROCESS. This value indicates that the variable is
 * currently not in use.
 */
#define OSE_ILLEGAL_PROCESS 0

/**
 * @macro OSE_ILLEGAL_ATTREF
 *
 * @declaration #define OSE_ILLEGAL_ATTREF 0xFFFFFFFFUL
 *
 * @brief An illegal attach identifier.
 *
 * @long
 * The macro OSE_ILLEGAL_ATTREF may be used to initiate and test application
 * variables of type OSATTREF. This value indicates that the variable is
 * currently not in use.
 */
#define OSE_ILLEGAL_ATTREF 0xFFFFFFFFUL

/**
 * @macro OSE_ILLEGAL_TMOREF
 *
 * @declaration #define OSE_ILLEGAL_TMOREF 0
 *
 * @brief An illegal timeout identifier.
 *
 * @long
 * The macro OSE_ILLEGAL_TMOREF may be used to initiate and test application
 * variables of type OSTMOREF. This value indicates that the variable is
 * currently not in use.
 */
#define OSE_ILLEGAL_TMOREF 0

/**
 * @macro OSE_NO_TIMEOUT
 *
 * @brief A system call timeout parameter value, meaning eternal timeout.
 *
 * @long
 * This macro can be used in calls to receive_from(), receive_sport(),
 * receive_fsem() when a timeout return is not wanted.
 * A call of receive_w_tmo() with this parameter will be equivalent to
 * calling a plain receive(), i.e. it will only return when a signal has
 * arrived. 
 * @long For earlier versions than OSE5.5.3, eternal timeouts where only
 * implemented for receive_fsem().
 */
#define OSE_NO_TIMEOUT (~(OSTIME)0)

/**
 * @macro SYSTEM_TICK
 *
 * @declaration #define SYSTEM_TICK (1000L)
 *
 * @brief This manifest constant is always 1000L.
 *
 * @long
 * It is present only for compatibility with old OSE kernels (it should not
 * be used when writing applications for OSE), where the delay and
 * receive_w_tmo calls specify timeout in number of system ticks instead of
 * in number of milliseconds.
 * @long
 * An old way of saying "delay 50 milliseconds" is:
 * @break
 * delay((OSTIME) ((50 * 1000L) / SYSTEM_TICK));
 * @long
 * This design also works with new OSE versions since the SYSTEM_TICK
 * constant balances the expression.
 * (It is, of course, more efficient to calculate the expression in advance
 * and save the result for repeated use.)
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @seealso delay
 * @seealso receive_w_tmo
 */
#define SYSTEM_TICK 1000L

/**
 * @macro OSE_MUTEX_PRIO_INHERIT
 *
 * @declaration #define OSE_MUTEX_PRIO_INHERIT  0x00000100
 *
 * @brief Attribute to initiate a mutex with priority inheritance.
 *
 * @include "ose.h"
 *
 * @seealso ose_mutex_init
 */
#define OSE_MUTEX_PRIO_INHERIT  0x00000100

/**
 * @macro OSE_MUTEX_RECURSIVE
 *
 * @declaration #define OSE_MUTEX_RECURSIVE     0x00000200
 *
 * @brief Attribute to initiate a counting mutex.
 *
 * @include "ose.h"
 *
 * @seealso ose_mutex_init
 */
#define OSE_MUTEX_RECURSIVE     0x00000200

/**
 * @macro OSE_MUTEX_INITIALIZER
 *
 * @declaration #define OSE_MUTEX_INITIALIZER(attributes)
 *
 * @field attributes Mutex attributes, a combination of the wanted
 * attribute bits in @link "" "ose_mutex_init", or zero.
 *
 * @brief Initializer macro for a statically allocated mutex.
 *
 * @include "ose.h"
 *
 * @example
 * // Create and initialize a mutex
 * MUTEX mtx=OSE_MUTEX_INITIALIZER(OSE_MUTEX_PRIO_INHERIT | OSE_MUTEX_RECURSIVE);
 * .....
 * .....
 * ose_mutex_lock(&mtx);
 * // do something ..... and when ready:
 * ose_mutex_unlock(&mtx);
 *
 * @seealso ose_mutex_init
 * @seealso OSE_MUTEX_PRIO_INHERIT
 * @seealso OSE_MUTEX_RECURSIVE
 */
#define OSE_MUTEX_INITIALIZER(attributes) {{0x4d754d75UL, (OSADDRESS)(attributes)}}

/**
 * @macro free_buf_type
 *
 * @field t The type of the pointer.
 * @field b A pointer to a pointer to the signal buffer to free.
 *
 * @brief Use this instead of free_buf() when freeing a pointer that is
 * not a union SIGNAL pointer.
 *
 * @long
 * This macro can be used when freeing pointers that are not union SIGNAL
 * pointers. By using the macro, warnings regarding incompatible pointer type
 * and warnings regarding breaking the strict aliasing rules are avoided.
 * It is also possible to avoid these warnings by including the type of the
 * pointer in the SIGNAL union.
 *
 * @include "ose.h"
 *
 * @example
 * #include "ose.h"
 * void check_env(void)
 * {
 *   char *envbuf = get_env(current_process(), "my_env");
 *
 *   if(envbuf != NULL)
 *     {
 *       free_buf_type( char *, &envbuf);
 *     }
 * }
 */
#define free_buf_type(t, b) \
	do \
	{ \
	   union U \
	   { \
	      union SIGNAL **s; \
	      t *x; \
	   } u; \
	   u.x = (b); \
	   free_buf(u.s); \
	} while(0)

/**
 * @type SIGNAL
 *
 * @declaration union SIGNAL;
 *
 * @brief The SIGNAL union.
 *
 * @long When writing an application, it is necessary to declare the
 * "union SIGNAL" type. This must be done after including the ose.h header
 * file. All signals that are used in a file must be part of the union
 * signal declaration in that particular file.
 *
 * @example
 * #include "ose.h"
 * #include "signals.sig"
 * union SIGNAL
 * {
 *   SIGSELECT            sig_no;
 *   struct    First      first;
 *   struct    Testsignal testsignal;
 * };
 */
union SIGNAL; /* Forward declaration. */

/**
 * @macro OS_ATTACH_SIG
 *
 * @declaration #define OS_ATTACH_SIG 252
 *
 * @brief Default attach signal number.
 *
 * @long A manifest constant defining the signal number of the default
 * signal created by the attach system call when invoked with a signal
 * pointer set to NULL.
 * @long
 * It may be used by an application to determine if such a signal was
 * received. The value of this constant is always 252.
 * @long
 * It is defined in the header files ose.h and ose_i.h.
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @seealso attach
 * @seealso detach
 *
 * @example
 * #include "ose.h"
 *
 * static const SIGSELECT any_sig[] = {0};
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *   for(;;)
 *     {
 *       sig = receive((SIGSELECT *) any_sig);
 *       switch(sig->sig_no)
 *         {
 *         case OS_ATTACH_SIG:
 *           // Code //
 *           break;
 *         default:
 *           // Code //
 *           break;
 *         }
 *       free_buf(&sig);
 *     }
 * }
 */
#define OS_ATTACH_SIG (252)	/* !-SIGNO(SIGSELECT)-! */

/**
 * @macro OS_RCV_FSEM
 *
 * @declaration #define OS_RCV_FSEM
 *
 * @brief The return value from receive_fsem when the fast semaphore
 * has been signaled.
 *
 * @include "ose.h"
 *
 * @restriction
 * OS_RCV_FSEM is specified such that it is never confused with a
 * legal signal pointer.
 *
 * @seealso receive_fsem
 */
#define OS_RCV_FSEM ((union SIGNAL *)(-1))

/**
 * @type PROCESS_TYPE
 *
 * @declaration enum PROCESS_TYPE
 *{
 *   OS_PRI_PROC = 0, OS_BG_PROC = 64, OS_INT_PROC = 128, OS_TI_PROC = 256,
 *    OS_PHANTOM = 512, OS_BLOCK = 1024, OS_ZOOMBIE = 2048, OS_ILLEGAL = 4096
 *};
 *
 * @brief The type of a process.
 *
 * @field OS_PRI_PROC A Prioritized process.
 * @field OS_BG_PROC  A Background process.
 * @field OS_INT_PROC An Interrupt process.
 * @field OS_TI_PROC  A Timer Interrupt process.
 * @field OS_PHANTOM  A Phantom process.
 * @field OS_BLOCK    A Block.
 * @field OS_ZOOMBIE  A killed process or block.
 * @field OS_ILLEGAL  A return value when operating on an illegal process identity.
 *
 * @seealso create_process
 * @seealso get_ptype
 */
enum PROCESS_TYPE
    {
    OS_PRI_PROC = 0,
    OS_BG_PROC = 64,
    OS_INT_PROC = 128,
    OS_TI_PROC = 256,
    OS_PHANTOM = 512,
    OS_BLOCK = 1024,
    OS_ZOOMBIE = 2048,
    OS_ILLEGAL = 4096
    };

/**
 * @type PROCESS_STATUS
 *
 * @declaration enum PROCESS_STATUS
 *{
 *   OS_RECEIVE = 1, OS_DELAY = 2, OS_SEMAPHORE = 4, OS_FSEMAPHORE = 8,
 *    OS_REMOTE = 16, OS_STOPPED = 32, OS_INTERCEPTED = 64
 *};
 *
 * @brief Process status bits.
 *
 * @long These bits describe the current state of the process. Note that
 * several flag bits may be set for a single process.
 *
 * @field OS_RECEIVE     The process is waiting in a receive call.
 * @field OS_DELAY       The process is waiting in a delay call.
 * @field OS_SEMAPHORE   The process is waiting on a semaphore or mutex.
 * @field OS_FSEMAPHORE  The process is waiting on its fast semaphore.
 * @field OS_REMOTE      The process is waiting on a remote call to complete.
 * @field OS_STOPPED     The process is stopped.
 * @field OS_INTERCEPTED The process is intercepted.
 *
 * @seealso OS_pcb
 */
enum PROCESS_STATUS
    {
    OS_RECEIVE = 1,
    OS_DELAY = 2,
    OS_SEMAPHORE = 4,
    OS_FSEMAPHORE = 8,
    OS_REMOTE = 16,
    OS_STOPPED = 32,
    OS_INTERCEPTED = 64
    };

/**
 * @type OS_redir_entry
 *
 * @declaration struct OS_redir_entry
 *{
 *   SIGSELECT sig;
 *   PROCESS pid;
 *};
 *
 * @brief A redirection table entry. A signal to be redirected to a process ID.
 *
 * @long The first entry in a redirection table is special. There the
 *       fields 'sig' and 'pid' have other meanings. The first 'sig' contains
 *       the total number of entries (including the first), while
 *       the first 'pid' contains the default process ID, to which
 *       undefined signals (not in the redirection table) shall
 *       be redirected.
 * @long As a special case, in a signal port redirection table, the normal
 *       redirection table is prepended by another special entry, with sig=0,
 *       see @link "ose.h" "receive_sport".
 *
 * @restriction
 *       A redirection table can not point out a target that is a phantom
 *       representing a link handler connection. In that case, the signal is
 *       silently discarded.
 *
 * @field sig The signal number that should be redirected
 *            (sig in first entry is the total number of entries in table).
 * @field pid Process ID of the process that the signal should be redirected to
 *            ('pid' in first entry is the default process).
 *
 * @seealso create_process
 * @seealso set_redirection
 * @seealso receive_sport
 */
struct OS_redir_entry
    {
    SIGSELECT sig;
    PROCESS pid;
    };

/**
 * @type semaphore
 *
 * @declaration struct semaphore
 *{
 *   OSSEMVAL value;
 *   void * f;
 *   void * b;
 *};
 *
 * @brief A basic semaphore struct.
 *
 * @long The contents of this struct is really internal, and exposed here
 * to make the size and alignment known to compilers, when static semaphores
 * are used.
 *
 * @field value
 * @field f
 * @field b
 */
struct semaphore
    {
    OSSEMVAL value;
    void *f;
    void *b;
    };

/**
 * @type SEMAPHORE
 *
 * @declaration typedef struct semaphore SEMAPHORE;
 *
 * @brief The SEMAPHORE type as used in system call interface.
 *
 * @seealso create_sem
 */
typedef struct semaphore SEMAPHORE;

/**
 * @type OS_pid_list
 *
 * @declaration struct OS_pid_list
 *{
 *   OSBUFSIZE count;
 *   PROCESS list[1];
 *};
 *
 * @brief A list of process, block and pool identities.
 *
 * @field count The number of elements in the list array.
 * @field list A variable size array containing the process, block and pool
 * identities.
 *
 * @seealso get_bid_list
 * @seealso get_pid_list
 * @seealso get_pool_list
 */
struct OS_pid_list
    {
    OSBUFSIZE count;
    PROCESS list[1]; /* Variable size */
    };

/**
 * @type OS_mutex
 *
 * @declaration struct OS_mutex
 *{
 *   OSADDRESS m[8];
 *};
 *
 * @brief A basic mutex struct.
 *
 * @long The contents of this struct is really internal, and exposed here
 * to make the size and alignment known to compilers.
 *
 * @field m
 */
struct OS_mutex
{
   OSADDRESS m[8];
};

/**
 * @type MUTEX
 *
 * @declaration typedef struct OS_mutex MUTEX;
 *
 * @brief The MUTEX type as used in system call interface.
 *
 * @seealso ose_mutex_init
 */
typedef struct OS_mutex MUTEX;

/**
 * @type OS_pcb
 *
 * @declaration struct OS_pcb
 *{
 *   OSADDRESS type;
 *   OSADDRESS status;
 *   OSADDRESS priority;
 *   OSUSER user;
 *   OSFSEMVAL fsemvalue;
 *   OSADDRESS sigqueue;
 *   OSADDRESS attach_list;
 *   OSADDRESS stack_top;
 *   OSADDRESS stack_limit;
 *   PROCESS remote_server;
 *   OSADDRESS sig_cnt_in_q;
 *   OSADDRESS sig_cnt_owned;
 *   OSADDRESS max_sigsize;
 *   OSADDRESS sigsel_size;
 *   OSADDRESS line;
 *   OSADDRESS file;
 *   OSADDRESS name;
 *   OSADDRESS cpuregs;
 *   OSADDRESS wanted;
 *   char strings[1];
 *};
 *
 * @brief A Process Control block, as reported by get_pcb().
 *
 * @long The OS_pcb shows the status of the specified process or block.
 * @long
 * The following table describes the amount of information returned
 * depending on the type of calling process and the type of object
 * investigated.
 * @long
 * @tab Calling Process @tab Prioritized or Background @tab Prioritized or
 *   Background @tab Interrupt or Timer Interrupt @tab Interrupt or Timer
 *   Interrupt @break
 * @tab Investigated Object @tab Process @tab Block @tab Process @tab Block @break
 * @tab type          @tab YES @tab YES @tab YES @tab YES @break
 * @tab status        @tab YES @tab     @tab YES @tab     @break
 * @tab priority      @tab YES @tab     @tab YES @tab     @break
 * @tab user          @tab YES @tab YES @tab     @tab     @break
 * @tab fsemvalue     @tab YES @tab     @tab YES @tab     @break
 * @tab siqqueue      @tab YES @tab     @tab     @tab     @break
 * @tab attach_list   @tab YES @tab YES @tab     @tab     @break
 * @tab stack_top     @tab YES @tab     @tab YES @tab     @break
 * @tab stack_limit   @tab YES @tab     @tab YES @tab     @break
 * @tab remote_server @tab YES @tab YES @tab YES @tab YES @break
 * @tab sig_cnt_in_q  @tab YES @tab     @tab     @tab     @break
 * @tab sig_cnt_owned @tab YES @tab     @tab YES @tab     @break
 * @tab max_sigsize   @tab YES @tab YES @tab     @tab     @break
 * @tab sigsel_size   @tab YES @tab YES @tab YES @tab YES @break
 * @tab line          @tab YES @tab     @tab     @tab     @break
 * @tab file          @tab YES @tab     @tab     @tab     @break
 * @tab name          @tab YES @tab YES @tab YES @tab YES @break
 * @tab cpuregs       @tab YES @tab     @tab     @tab     @break
 * @tab wanted        @tab YES @tab     @tab     @tab     @break
 *
 * @field type
 * Type contains the process type. Legal values are
 * @break
 * @tab OS_BLOCK    @tab A block was specified. @break
 * @tab OS_PRI_PROC @tab A prioritized process. @break
 * @tab OS_BG_PROC  @tab A background process. @break
 * @tab OS_INT_PROC @tab An interrupt process. @break
 * @tab OS_TI_PROC  @tab A timer-interrupt process. @break
 * @tab OS_PHANTOM  @tab A phantom process. @break
 * @tab OS_ZOOMBIE  @tab A killed process or block. @break
 * @tab OS_ILLEGAL  @tab An invalid process ID was specified. @break
 * The process type field must always be tested to validate other
 * information.
 * @field status
 * Process status. This is a bit field with zero, one, two or three bits
 * set. If no bits are set, the process is ready or running. Otherwise the process is
 * suspended for one or more reasons. The OS_STOPPED and OS_INTERCEPTED
 * bits can be independently set. Other bits are mutually exclusive.
 * Legal bit masks are:
 * @break
 * @tab OS_RECEIVE     @tab Waiting for one of the receive calls. @break
 * @tab OS_DELAY       @tab Waiting for a delay call. @break
 * @tab OS_SEMAPHORE   @tab Waiting at a semaphore. @break
 * @tab OS_FSEMAPHORE  @tab Waiting at a fast semaphore. @break
 * @tab OS_REMOTE      @tab Waiting for a remote system call. @break
 * @tab OS_STOPPED     @tab The process was stopped. @break
 * @tab OS_INTERCEPTED @tab A breakpoint was reached. @break
 *
 * @field priority The priority of the process. Legal values are 0 - 31.
 *
 * @field user The user number of the process.
 *
 * @field fsemvalue The current value of the fast semaphore attached to the
 * process. The value is negative if the process is waiting at the semaphore
 * or if the process does not have a fast semaphore. Use the status field
 * to determine which is the case.
 *
 * @field sigqueue The address of the first signal in the signal queue.
 * The signal queue can be examined by subsequent get_signal calls.
 * Sigqueue = 0 if no signal is presently in the queue.
 *
 * @field attach_list The address of the first signal in the queue of
 * attached signals. This list contains signals registered by other
 * processes, by use of the attach() system call to supervise the state
 * of this process. The attach queue can be examined by subsequent
 * get_signal calls. Attach_list = 0 if no signal is attached to the process.
 *
 * @field stack_top The highest address on the stack. (The byte at the
 * specified address is part of the stack.)
 *
 * @field stack_limit The lowest address on the stack. (The byte at the
 * specified address is part of the stack.) The stack_top and stack_limit
 * fields are equal if the kernel can not provide stack information.
 *
 * @field remote_server The ID of the process that manages remote system
 * calls directed towards the examined process.
 * Remote_server is the process ID of the examined process itself if no
 * remote call server is present.
 *
 * @field sig_cnt_in_q The number of signals currently in the signal queue
 * of the process. The address of the first signal is found in the sigqueue
 * field. Please note that the sig_cnt_in_q field should only be used for
 * statistics, not when traversing the signal queue itself.
 *
 * @field sig_cnt_owned The number of signals currently owned by the process.
 * This includes signals allocated by the process and signals attached by
 * the process, but not signals in the signal queue or signals handed to
 * the hunt system call.
 *
 * @field max_sigsize The number of bytes in the largest signal that the
 * specified process or block is able to allocate. This is also the largest
 * signal that can be successfully sent to a process in the block. (Other
 * signals are lost.)
 *
 * @field sigsel_size  Contains sizeof(SIGSELECT) for the specified process
 * or block.
 *
 * @field line  An integer containing the source code line from which the
 * current system call was issued. Line is 0 if no line and file information
 * is available. Line and file information is available if the specified
 * process was compiled with the constant OS_DEBUG defined.(e.g. a
 * -DOS_DEBUG flag was set when compiling).
 *
 * @field file Offset into the strings field, indicating the file name from
 * which the current system call was issued. This field is invalid if line
 * is 0. Line and file information is available if the specified process
 * was compiled with the constant OS_DEBUG defined.
 *
 * @field name Offset into the strings field, indicating the name of the
 * process or block.
 *
 * @field cpuregs Offset into the strings field, indicating all known CPU
 * register values.
 * @break
 * The names of these are CPU dependent.
 * @break
 * A sample string might look like "PC=00001D34 SP=000089AB D1=0000A89".
 * @break
 * Registers are separated by a space. All values are in upper case hex
 * format. Registers that can not be determined at a certain point are
 * excluded from the list.
 * @break
 * There may be a pseudo register named "RA". It contains the return
 * address to user code while a system call is in progress. Information
 * on CPU register values is only available for Kernel libraries supporting
 * debugging.
 *
 * @field wanted  Offset into the strings field to a copy of the currently
 * active sigselect array passed to receive by the examined process. The array
 * was converted to a byte order suitable for the caller. The size of
 * each entry must be calculated by the caller using the sigsel_size field
 * described above. The wanted field is invalid if process status does not
 * indicate OS_RECEIVE.
 * @break
 * If the sigsel array is unresolvable large it will be truncated.
 *
 * @field strings  A variable size array containing the string pool. The
 * string pool is used to store strings for file, process or block name and
 * CPU registers. It also holds the wanted array. The values of those
 * fields are not pointers, but indexes into the string pool.
 *
 * @seealso get_pcb
 * @seealso PROCESS_TYPE
 * @seealso PROCESS_STATUS
 */
struct OS_pcb
    {
    OSADDRESS type;     /* enum PROCESS_TYPE */
    OSADDRESS status;   /* enum PROCESS_STATUS */
    OSADDRESS priority; /* OSPRIORITY */
    OSUSER user;
    OSFSEMVAL fsemvalue;
    OSADDRESS sigqueue;
    OSADDRESS attach_list;
    OSADDRESS stack_top;
    OSADDRESS stack_limit;
    PROCESS remote_server;
    OSADDRESS sig_cnt_in_q;
    OSADDRESS sig_cnt_owned;
    OSADDRESS max_sigsize;
    OSADDRESS sigsel_size;
    OSADDRESS line;
    OSADDRESS file;
    OSADDRESS name;
    OSADDRESS cpuregs;
    OSADDRESS wanted;
    char strings[1];
    };

/**
 * @type OS_poolcb
 *
 * @declaration struct OS_poolcb
 *{
 *   OSADDRESS tot_size;
 *   OSADDRESS free_size;
 *   OSADDRESS fragment_info;
 *   OSADDRESS stk_conf_entries;
 *   OSADDRESS sig_conf_entries;
 *   OSADDRESS stk_conf_sizes;
 *   OSADDRESS sig_conf_sizes;
 *   OSADDRESS stk_alloc_sizes;
 *   OSADDRESS sig_alloc_sizes;
 *   OSADDRESS values[1];
 *};
 *
 * @brief Information about a pool (Pool Control Block).
 *
 * @field tot_size	The total memory size (in bytes) of all pool fragments.
 *
 * @field free_size	The number of bytes never used in the pool.
 *
 * @field fragment_info	An index into the values array where detailed
 * information about all the pool fragments can be found. The layout of
 * this information is as follows:
 * @break
 * @tab values[fragment_info+0] @tab The number of fragments in the pool
 * @break
 * @tab values[fragment_info+1+4*f] @tab The base address of fragment
 * number f @break
 * @tab values[fragment_info+2+4*f] @tab The size of fragment number f
 * @break
 * @tab values[fragment_info+3+4*f] @tab The number of bytes used for
 * stacks in fragment number f @break
 * @tab values[fragment_info+4+4*f] @tab The number of bytes used for
 * signals in fragment number f. For the system pool, this figure
 * includes space used by different kernel control blocks. @break
 * A pool is initially created with one fragment, called fragment number zero.
 *
 * @field stk_conf_entries The number of configured stack sizes for this pool.
 *
 * @field sig_conf_entries The number of configured signal buffer sizes for
 * this pool.
 *
 * @field stk_conf_sizes An index into the values array where to find an
 * array (of size stk_conf_entries) with information about how the stack
 * sizes has been configured for this pool. This field is only valid if the
 * value of the stk_conf_entries field is greater than zero. The smallest
 * configured stack size can be found at "values[stk_conf_sizes]" and
 * largest configured stack size can be found at
 * "values[stk_conf_sizes + stk_conf_entries - 1]".
 *
 * @field sig_conf_sizes An index into the values array where to find an
 * array (of size sig_conf_entries) with information about how the signal
 * buffer sizes has been configured for this pool. This field is only valid
 * if the value of the sig_conf_entries field is greater than zero. The
 * smallest configured signal buffer size can be found
 * at "values[sig_conf_sizes]" and the largest configured signal buffer size
 * can be found at "values[sig_conf_sizes + sig_conf_entries - 1]".
 *
 * @field stk_alloc_sizes An index into the values array where to find an
 * array (of size stk_conf_entries) with information about how many stacks
 * (of a certain configured size) are currently in use in the pool. This
 * field is only valid if the value of the stk_conf_entries field is greater
 * than zero. The number of currently used stacks of the smallest configured
 * stack size in the pool can be found at "values[stk_alloc_sizes]" and
 * "values[stk_alloc_sizes + stk_conf_entries - 1]" contains the number of
 * currently used stacks of the largest configured stack size in the pool.
 *
 * @field sig_alloc_sizes An index into the values array where to find an
 * array (of size sig_conf_entries) with information about how many signal
 * buffers (of a certain configured size) are currently allocated in the
 * pool. This field is only valid if the value of the sig_conf_entries field
 * is greater than zero. The number of currently allocated signal buffers of
 * the smallest configured signal buffer size in the pool can be found at
 * "values[sig_alloc_sizes]" and "values[sig_alloc_sizes + sig_conf_entries
 * - 1]" contains the number of currently allocated signal buffers of
 * the largest configured signal buffer size in the pool.
 *
 * @field values A variable size array containing most of the requested pool
 * information.
 *
 * @seealso get_poolcb
 */
struct OS_poolcb
  {
    OSADDRESS tot_size;
    OSADDRESS free_size;
    OSADDRESS fragment_info;
    OSADDRESS stk_conf_entries; /* [0-8] -> 0 == Not a stack pool  */
    OSADDRESS sig_conf_entries; /* [0-8] -> 0 == Not a signal pool */
    OSADDRESS stk_conf_sizes;
    OSADDRESS sig_conf_sizes;
    OSADDRESS stk_alloc_sizes;
    OSADDRESS sig_alloc_sizes;
    OSADDRESS values[1];
};

/*
** Prototypes.
*/

/**
 * @function alloc
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration union SIGNAL *
 * alloc(OSBUFSIZE size, SIGSELECT signo);
 *
 * @brief Allocate a signal buffer.
 *
 * @long
 * Allocates a signal buffer of the specified size from the pool available
 * to the caller's block. The specified signo (signal number) is entered in
 * the first location in the new buffer.
 *
 * @long
 * Another signal number may later be assigned to the buffer by simply
 * storing a new number in the first location.
 *
 * @long
 * The new buffer is owned by the calling process. A new owner can be
 * entered only by using one of the system calls that operates on the
 * buffer. It is basically illegal to pass control of a buffer to another
 * process in any other way since buffers may be lost in case of premature
 * process termination. If really needing to pass buffers around in a
 * disorderly manner, the restore system call can be used to force
 * registration of a new owner of a buffer.
 * Note that a signal buffer can never be owned by a block, i.e. the shared
 * execution model does not affect allocation of signal buffers.
 *
 * @long The buffer will be aligned at a cache line boundary in systems
 *       that include caches.
 *
 * @long Requests for signal buffers smaller than sizeof(SIGSELECT) are
 * rounded up to sizeof(SIGSELECT) in some kernel configurations.
 * However, the value of the signal number always needs to fit in the buffer,
 * otherwise the signo value can not be relied upon to reach the destination.
 *
 * @input size The requested size of the signal buffer.
 * @input signo The signal number, will be placed in the allocated buffer.
 *
 * @return Returns a pointer to the new buffer.
 *
 * @restriction
 * It is an error to allocate a buffer larger than the largest buffer size
 * available in the pool. Use the get_pcb system call to gain information
 * about the largest possible buffer size for a process.
 * This error will be reported as the non-fatal kernel
 * error OSE_EBUFFER_TOO_LARGE. The largest configured size will be
 * allocated instead of the requested size if the error handler
 * allows the current process to continue executing on this error.
 * @break
 * Also available in interrupt and timer interrupt processes.
 *
 * @seealso free_buf
 * @seealso restore
 * @seealso sender
 * @seealso addressee
 * @seealso sigsize
 * @seealso create_pool
 * @seealso set_sigsize
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * // The declarations of signal numbers and structs are
 * // preferably made in .sig-files which are then #included.
 * #define THIRD 3
 * struct Third
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 *   struct Third third;
 * };
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *   // Process ID for the static process named "proc".
 *   extern PROCESS proc_;
 *
 *   for(;;)
 *     {
 *       sig = alloc(sizeof(struct Third),THIRD);
 *       send(&sig,proc_);
 *       delay(100);
 *     }
 * }
 */
union SIGNAL *
alloc(OSBUFSIZE size, SIGSELECT signo);

/**
 * @function create_error_handler
 *
 * @include "ose.h"
 *
 * @declaration OSERRH *
 * create_error_handler(PROCESS id, OSERRH *entrypoint, OSADDRESS stack_size);
 *
 * @brief Create a local error handler.
 *
 * @long
 * An error handler is created for the specified process or block. 
 * The stack_size argument is currently not used, but it is recommended
 * to give a decent estimate of the size needed.
 * @long
 * Any previously defined handler on that error level is replaced, and the
 * old error handler's entrypoint is returned to the caller.
 * @long
 * The entrypoint is a location within the address space of the specified
 * block. Set entrypoint to NULL to remove a previously created error
 * handler.
 * @long
 * An error handler behaves as a subroutine called from the currently running
 * process. 
 * @long
 * Error handlers may use the same set of system calls as the process
 * running when the error handler was invoked.
 * @long
 * The error handler is passed parameters containing error information and
 * a flag set to a non-zero value if the call was made from user code and
 * not from the kernel. The handler should return a non-zero
 * value if the error was managed.
 *
 * @long
 * An error handler declaration looks like this:
 * @long
 * OSADDRESS err_handler(OSBOOLEAN user_called,
 * OSERRCODE ecode,
 * OSERRCODE extra);
 * @long
 * @tab user_called = 0
 * @tab The kernel reported the error.
 * Error codes are listed in ose_err.h.
 * @break
 * @tab user_called = 1
 * @tab The error was generated by
 * the @link "" "error" or @link "" "error2" system calls.
 * Error codes are subdivided into ranges per OSE component and
 * free ranges should be used by applications.
 * Bit 31 is used as a fatal bit. It is recommended to use
 * bits 30-16 to define the range while bits 15-0 contain the error code.
 * Ranges for OSE components and more information about error handlers
 * and error codes are found in the Error Codes appendix
 * of the "OSE Architecture User's Guide".
 * @break
 * @tab ecode
 * @tab Error code.
 * @break
 * @tab extra
 * @tab Extra error code, available when the error was
 * generated by the error2() system call.
 * @break
 *
 * @long
 * There are four levels of error handlers: process, block, system, and
 * kernel halt error handlers.
 * @long
 * The process error handler is called first if it is present. If the
 * process error handler returns a non-zero value, the kernel will return to
 * user code. If the error handler returns zero, or there is no process error
 * handler for the process, then the error is propagated to the next level.
 * Errors considered fatal by the kernel are always propagated to the next
 * level.
 * @long
 * The next level is the block error handler. The block error handler
 * functions exactly like a process error handler. If the block error handler
 * returns zero, or there is no block error handler for this process, then
 * the error is propagated to the next level.
 * @long
 * The next level is the system error handler. The system error handler is
 * invoked in case the previous levels have been unable to solve the problem,
 * and for system errors that do not pertain to a particular process or block.
 * The system error handler is part of the kernel and is configured at compile
 * time. If the system error handler returns zero, or if the fatal bit was set
 * in the error code, then the error is propagated to the next level.
 * @long
 * The last level is the kernel halt error handler. It is always present in
 * the system. This error handler simply halts all processing in the system.
 * The hardware watchdog timer is then expected to reset and restart the
 * system.
 *
 * @input id The ID of the specified process.
 * @input entrypoint The address of the error handler.
 * @input stack_size This parameter is currently not used. It is recommended
 * to give a decent estimate of the size needed.
 * @return Returns the entrypoint of any previously defined error handler,
 * or NULL if no previous error handler was defined.
 *
 * @restriction
 * Error handlers used by several processes must be re-entrant, e.g. they
 * must use only stack variables.
 * @break
 * An error handler must return when it is finished. In particular, it must
 * not use the long jmp directive defined in the C language.
 * @break
 * It is recommended that error handlers are created only for processes or
 * blocks that are closely related to the caller. It is illegal to create
 * an error handler for a block or process in another domain.
 * @break
 * Error codes provided by the OSE kernel as well as all symbols defined in
 * ose_err.h are implementation dependent.
 *
 * @seealso OSERRH
 * @seealso error
 * @seealso error2
 * @seealso OSE
 * @seealso ose_err/ose_err.h
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * OSADDRESS pr_err_hand(OSBOOLEAN user_called,
 *                       OSERRCODE ecode,
 *                       OSERRCODE extra);
 *
 * OSADDRESS bl_err_hand(OSBOOLEAN user_called,
 *                       OSERRCODE ecode,
 *                       OSERRCODE extra);
 *
 * PROCESS proc_;
 * PROCESS block_;
 *
 * OS_PROCESS(my_process)
 * {
 *   OSERRH *old_pr_errhand, *old_block_errhand;
 *
 *   // Creating a process error handler. The entrypoint
 *   // to an error handler is returned in errhand if one
 *   // is previously defined otherwise NULL is returned.
 *   // The return value may be used when temporarily
 *   // creating a new error handler for a specific code
 *   // sequence, and restoring the old one after completion.
 *   old_pr_errhand = create_error_handler(proc_,(OSERRH *)
 *                                         pr_err_hand,200);
 *
 *   // Creating a block error handler. The entrypoint to
 *   //  an error handler is returned in errhand if one is
 *   //  previously defined otherwise NULL is returned.
 *   old_block_errhand = create_error_handler(block_,(OSERRH *)
 *                                            bl_err_hand,200);
 *
 *   // Any code that requires the new process error handler here ...
 *
 *   // Restore the previous process error handler setting.
 *   // The stack size parameter may be zero here.
 *   (void)create_error_handler(proc_,old_pr_errhand,0);
 *
 *   for(;;)
 *     {
 *       // Code //
 *     }
 * }
 */
OSERRH *
create_error_handler(PROCESS id, OSERRH *entrypoint, OSADDRESS stack_size);

/**
 * @function create_sem
 *
 * @include "ose.h"
 *
 * @declaration SEMAPHORE *
 * create_sem(OSSEMVAL initial_val);
 *
 * @brief Create a semaphore.
 *
 * @long
 * Creates a dynamic semaphore structure and initializes it to the value
 * specified in initial_val. The initial value must be zero or a positive
 * value. Typical values are 0 and 1.
 * @long
 * The semaphore is dynamically allocated from the current pool. It may be
 * returned to the pool when it is no longer in use by calling kill_sem.
 * Semaphores are implicitly killed when all blocks in the domain
 * of the creator terminates, i.e. when the domain is reclaimed.
 * @long
 * Semaphores may also be statically defined by simply declaring a static
 * semaphore structure in the program and initializing it. Such semaphores
 * can not be killed. They are removed from the system when the blocks
 * using a semaphore have terminated and the memory manager software
 * reclaims the domain.
 * @long
 * Semaphore structures contain three fields. The first must be initialized
 * to the initial (positive) value of the semaphore. The second and third
 * fields must be initialized to zero, and are required for internal use by
 * the kernel.
 * @long
 * The three fields of a semaphore structure are named "value", "f" and "b"
 * respectively. The value field is of type OSSEMVAL. The types of the
 * other fields are private to the OSE implementation.
 *
 * @input initial_val The initial value of the created semaphore.
 *
 * @return A pointer to the created semaphore structure.
 *
 * @restriction
 * A semaphore can only be shared by processes within the same domain
 * since the semaphore dies when the domain is reclaimed.
 * If it is known that the domain is never killed, then a semaphore can be
 * shared among all processes with access to the address space of the
 * domain. It is the user's responsibility to ensure that the semaphore is
 * not killed while it is still in use, and that a process is not killed
 * while operating on a semaphore that is expected to live. Serious
 * problems may otherwise arise.
 *
 * @seealso wait_sem
 * @seealso signal_sem
 * @seealso get_sem
 * @seealso kill_sem
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   SEMAPHORE *sem;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       sem = create_sem(1);
 *       // Code //
 *     }
 * }
 */
SEMAPHORE *
create_sem(OSSEMVAL initial_val);

/**
 * @function current_process
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration PROCESS
 * current_process(void);
 *
 * @brief Get the identity of the current process.
 *
 * @long
 * Determines which process is currently running.
 *
 * @return Returns the process ID of the calling process.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * Will not change interrupt mask unless error handler is called.
 *
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   PROCESS proc_;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       proc_ = current_process();
 *       // Code //
 *     }
 * }
 */
PROCESS
current_process(void);

/**
 * @function get_bid
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration PROCESS
 * get_bid(PROCESS pid);
 *
 * @brief Get the block ID of the specified process.
 *
 * @long Find the block ID that the specified process is part of.
 *
 * @input pid The process ID.
 *
 * @return Returns the block ID.
 * If 'pid' is a block ID or the ID of a dead process, then the value
 * specified in 'pid' is returned from the get_bid call. Use get_ptype to
 * test on these cases if needing to.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * Will not change interrupt mask unless error handler is called.
 *
 * @seealso create_block
 * @seealso get_bid_list
 * @seealso get_ptype
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stddef.h"
 * #include "ose.h"
 *
 * OSENTRYPOINT new_process;
 * extern PROCESS proc_;
 *
 * OS_PROCESS(my_process)
 * {
 *   PROCESS bid;
 *   PROCESS process_;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       bid = get_bid(proc_);
 *
 *       // Add new_process to the blocks to which proc_ belongs //
 *       process_ = create_process(OS_PRI_PROC,
 *                                 "process",
 *                                 new_process,
 *                                 200,
 *                                 16,
 *                                 (OSTIME) 0,
 *                                 bid,
 *                                 (struct OS_redir_entry *) NULL,
 *                                 (OSVECTOR) 0,
 *                                 (OSUSER) 0);
 *
 *       start(process_);
 *       // Code //
 *     }
 * }
 */
PROCESS
get_bid(PROCESS pid);

/**
 * @function get_bid_list
 *
 * @include "ose.h"
 *
 * @declaration struct OS_pid_list *
 * get_bid_list(OSUSER user);
 *
 * @brief List blocks.
 *
 * @long
 * Lists all blocks that are available.
 *
 * @input user Must be 0.
 *
 * @return Returns NULL if there are no blocks available.
 * Otherwise get_bid_list allocates and returns a signal buffer with
 * a structure containing the list of block IDs.
 * @break
 * The result buffer must be freed by the caller when information has been
 * extracted.
 * @break
 * The structure is described by struct @link "ose.h" "OS_pid_list"
 *
 * @seealso OS_pid_list
 * @seealso get_bid
 * @seealso get_pcb
 * @seealso get_pid_list
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   struct OS_pid_list *bidlist;
 *   OSBUFSIZE count;
 *   PROCESS block1_;
 *   OSBOOLEAN exist;
 *   int i;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       bidlist = get_bid_list(0);
 *       count = bidlist->count;
 *       exist = 0;
 *
 *       for(i = 0; i < count; i++)
 *         {
 *           if(bidlist->list[i] == block1_)
 *             {
 *               exist = 1;
 *               break;
 *             }
 *         }
 *
 *       if(exist)
 *         {
 *           // Code, The block exists //
 *         }
 *       else
 *         {
 *           block1_ = create_block("block1",
 *                                  (OSUSER) 0,
 *                                  (OSBOOLEAN) 0,
 *                                  (PROCESS) 0,
 *                                  (OSBOOLEAN) 0);
 *           // Code //
 *         }
 *     }
 * }
 */
struct OS_pid_list *
get_bid_list(OSUSER user);

/**
 * @function get_cpu
 *
 * @include "ose.h"
 *
 * @declaration char *
 * get_cpu(PROCESS pid);
 *
 * @brief Get CPU ID string of specified process.
 *
 * @long Returns a string describing the operating system on which the
 * specified process is executing.
 * @long
 * In many cases, this information is also used indirectly to find out the
 * CPU type. The string is contained in a buffer obtained from the caller's
 * pool. This buffer must be returned to the pool with the free_buf call
 * when the string has been extracted.
 * @long
 * The returned null-terminated string has the following general format:
 * @long
 * "<OSE_product>/<revision>/<other_info>"
 * @long
 * The "/" character is a general field separator.
 * Only the first two fields have specified contents.
 *
 * @input pid The process ID of the specified process.
 *
 * @return Operating system description string
 *
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "string.h"
 * #include "ose.h"
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * OS_PROCESS(my_process)
 * {
 *   char *cpu1;
 *   char *cpu2;
 *   extern PROCESS proc_;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       cpu1 = (char*) get_cpu(current_process());
 *       cpu2 = (char*) get_cpu(proc_);
 *       if(strcmp(cpu1, cpu2) == 0)
 *         {
 *           // Code when processes run in exactly the same
 *           // operating systems (need not be in the same CPU)
 *         }
 *       free_buf( (union SIGNAL **) &cpu1);
 *       free_buf( (union SIGNAL **) &cpu2);
 *     }
 * }
 */
char *
get_cpu(PROCESS pid);

/**
 * @function get_env
 *
 * @include "ose.h"
 *
 * @declaration char *
 * get_env(PROCESS id, const char *name);
 *
 * @brief Get environment variable.
 *
 * @long Get the contents of the named environment variable for the
 * specified process or block.
 * @long
 * Variables set with the set_envp() call can be read with the get_env()
 * call, but the string returned is not easily interpreted
 *
 * @input id The ID of the specified process.
 * @input name A pointer to the environment variable name.
 *
 * @return Returns a buffer obtained from the caller's pool containing the
 * value of the specified environment variable. The value is always a
 * null-terminated string. This buffer must be freed with free_buf by the
 * caller when information has been extracted.
 * @break
 * Returns NULL if the specified environment variable was not found.
 * @restriction
 * Strings that are too long to be passed in a signal buffer are truncated.
 *
 * @seealso get_env_list
 * @seealso get_envp
 * @seealso set_env
 * @seealso set_envp
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include <stdio.h>
 * #include "ose.h"
 *
 * void print_term(void)
 * {
 *   char *envbuf = get_env(current_process(), "TERM");
 *
 *   if(envbuf != NULL)
 *     {
 *       printf("TERM=%s\n", envbuf);
 *       free_buf( (union SIGNAL **) &envbuf);
 *     }
 * } // print_term //
 */
char *
get_env(PROCESS id, const char *name);

/**
 * @function get_envp
 *
 * @include "ose.h"
 *
 * @declaration OSADDRESS
 * get_envp(PROCESS id, const char *name);
 *
 * @brief Get environment variable with binary value.
 *
 * @long Reads a 32 bit pointer from a named environment variable for the
 * specified process or block.
 * @long
 * This call assumes that the value was set with the set_envp call, which
 * means that it was stored in a special format. Variables set with the
 * set_env call can be read with the get_envp call only if the value
 * consists of exactly 8 characters.
 *
 * @input id The ID of the specified block or process. Setting the 'id'
 * parameter to zero is equivalent to using the PID of the current
 * process.
 * @input name A pointer to the environment variable name.
 *
 * @return Returns the 32 bit pointer value of the environment variable.
 * @break
 * Returns zero if the specified environment variable was not found.
 *
 * @restriction
 * The get_envp call is available also to interrupt processes, but there is
 * a possible interrupt latency penalty doing so. The penalty depends on
 * the number of environment variables owned by the investigated process.
 *
 * @seealso get_env
 * @seealso get_env_list
 * @seealso set_env
 * @seealso set_envp
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stdio.h"
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   int proc_data = 0xCAFE;
 *   OSADDRESS pa;
 *
 *   for(;;)
 *     {
 *       set_envp(0, "pd", (OSADDRESS)&proc_data);
 *       pa = get_envp(0, "pd");
 *       if (pa != 0)
 *         printf("pa = %d\n", *(int *)pa);
 *       // Code //
 *     }
 * }
 */
OSADDRESS
get_envp(PROCESS id, const char *name);

/**
 * @function get_env_list
 *
 * @include "ose.h"
 *
 * @declaration char *
 * get_env_list(PROCESS id, const char *first_name);
 *
 * @brief List environment variables.
 *
 * @long List the environment variables available in the specified
 * process or block.
 * @long
 * If first_name is NULL then the returned list starts at the first
 * variable found.
 * @long
 * If first_name is a valid name, then the returned list starts at the
 * first variable name found after first_name. (In this way a long list can
 * be managed.)
 *
 * @input id The ID of the specified process or block.
 * @input first_name A pointer to an environment variable name.
 *
 * @return Returns a buffer obtained from the caller's pool containing the
 * names of the environment variables available in the specified process or
 * block.
 * @break
 * Names are separated by a single space and the entire string is
 * null-terminated.
 * @break
 * Names that are too long for a single buffer are discarded from the
 * return list. (Truncated names are never returned).
 * @break
 * The buffer must be freed by the caller when information has been
 * extracted.
 * @break
 * Returns an empty string "" if the process or block has terminated or if
 * no variables were found.
 *
 * @restriction
 * There is only one way to know if the environment contains more strings
 * than the ones returned. A second get_env_list call must always be
 * issued, specifying the last name in the previous list as first_name.
 *
 * @seealso set_env
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stdio.h"
 * #include "string.h"
 * #include "ose.h"
 *
 * // Print the names of all environment variables
 * // for a selected process
 *
 * void print_envnames(PROCESS pid)
 * {
 *   char *envbuf, *next_envbuf, *name, *blank;
 *
 *   envbuf = get_env_list(pid, NULL);
 *   name = envbuf;
 *   while(*name)
 *     {
 *       blank = strchr(name, ' ');
 *       if(blank)
 *         {
 *           // Not the last name in the buffer //
 *           while (name != blank) putchar(*name++);
 *           printf("\n");
 *           name = blank + 1;
 *         }
 *       else
 *         {
 *           // The last name in this buffer //
 *           printf("%s\n", name);
 *           next_envbuf = get_env_list(pid, name);
 *           free_buf((union SIGNAL **) &envbuf);
 *           envbuf = next_envbuf;
 *           name = envbuf;
 *         }
 *     } // while //
 *
 *   // Free the last, always empty, environment buffer //
 *   free_buf((union SIGNAL **) &envbuf);
 * }
 */
char *
get_env_list(PROCESS id, const char *first_name);

/**
 * @function get_fsem
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration OSFSEMVAL
 * get_fsem(PROCESS pid);
 *
 * @brief Get the fast semaphore value of the specified process.
 *
 * @input pid The process ID of the specified process.
 *
 * @return Returns the current value of the fast semaphore.
 *
 * @restriction Also available in interrupt and timer interrupt processes.
 *
 * @seealso signal_fsem
 * @seealso wait_fsem
 * @seealso set_fsem
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   OSFSEMVAL fsemvalue;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       fsemvalue = get_fsem(current_process());
 *       //Code //
 *     }
 * }
 */
OSFSEMVAL
get_fsem(PROCESS pid);

/**
 * @function get_pcb
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration struct OS_pcb *
 * get_pcb(PROCESS id);
 *
 * @brief Get the status of the specified process or block.
 *
 * @long
 * The get_pcb call is mainly used by debuggers, but the process name, for
 * instance, may also be of interest in some applications.
 * @long
 * Get_pcb may be called also from interrupt and timer-interrupt processes.
 * Calling get_pcb from interrupt processes is allowed mainly to aid
 * error handlers in obtaining the name of the current process.
 * @long
 * The terminated processes may be examined, although no information of interest
 * except process type is available.
 * @long
 * The process type field must always be tested to validate other
 * information.
 * @long
 * Note that it is legal to pass an invalid process ID to this system call.
 * This is legal only for the get_pcb and get_ptype calls. The returned
 * 'type' field of the struct will then be set to OS_ILLEGAL.
 *
 * @input id The ID of the specified block or process.
 * @return Returns a buffer obtained from the caller's pool containing the
 * OS_pcb structure.
 * @break
 * This buffer must be freed with free_buf when it is no longer needed.
 * @break
 * If the get_pcb call was not successful, a NULL pointer is returned.
 * @break
 * See type @link "ose.h" "OS_pcb" for a full description of the data structure.
 * @restriction There must be a buffer available in the pool that is large
 * enough to hold the returned structure. Otherwise the get_pcb call will
 * fail.
 * @break
 * In the Soft Kernel the return value for cpuregs will always be an empty
 * string.
 * @break
 * Also available in interrupt and timer interrupt processes.
 *
 * @seealso get_signal
 * @seealso get_mem
 * @seealso get_ptype
 * @seealso set_pcb
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "string.h"
 * #include "ose.h"
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * OS_PROCESS(my_process)
 * {
 *   struct OS_pcb *pcbbuf;
 *   static const char *name = "my_process";
 *
 *   for(;;)
 *     {
 *       // Code //
 *       pcbbuf = get_pcb(current_process());
 *       if(pcbbuf)
 *         {
 *           switch((enum PROCESS_TYPE)pcbbuf->type)
 *             {
 *             case OS_PRI_PROC:
 *               if(strcmp(&pcbbuf->strings[pcbbuf->name], name))
 *                 {
 *                   // Code //
 *                 }
 *               break;
 *
 *             default:
 *               // Code //
 *               break;
 *             }
 *           free_buf( (union SIGNAL **) &pcbbuf);
 *         }
 *       else
 *         {
 *           // Code, Not a successful call //
 *         }
 *     }
 * }
 */
struct OS_pcb *
get_pcb(PROCESS id);

/**
 * @function get_pid_list
 *
 * @include "ose.h"
 *
 * @declaration struct OS_pid_list *
 * get_pid_list(PROCESS bid);
 *
 * @brief List processes.
 *
 * @long List all processes that are part of the specified block and are
 * available to the calling process.
 *
 * @input bid The ID of the specified block.
 *
 * @return Returns NULL if the block has no processes available to the
 * caller.
 * @break
 * Otherwise get_pid_list allocates and returns a signal buffer with a
 * structure containing process IDs available to the caller in the block.
 * @break
 * The result buffer must be freed by the caller when information has been
 * extracted.
 * @break
 * The structure is described by struct @link "ose.h" "OS_pid_list"
 *
 * @restriction
 * If the block contains processes that are not available to the caller,
 * these processes are not shown.
 * @break
 * If the block contains more processes than can be presented in the
 * largest signal buffer, then the list is truncated at that size.
 *
 * @seealso OS_pid_list
 * @seealso get_pcb
 * @seealso get_bid_list
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * // The declarations of signal numbers and structs are
 * // preferably made in .sig-files which are then #included.
 * #define START 10
 * struct Start
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 *   struct Start start;
 * };
 *
 * extern PROCESS some_block_;
 *
 * OS_PROCESS(my_process)
 * {
 *   struct OS_pid_list *pidlist;
 *   OSBUFSIZE count;
 *   union SIGNAL *sig;
 *   int i;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       pidlist = get_pid_list(some_block_);
 *       if(pidlist)
 *         {
 *           count = pidlist->count;
 *           for(i =0 ;i<count ;i++)
 *             {
 *               // Send the start signal to all
 *               // processes in the block.
 *               sig = alloc(sizeof(struct Start), START);
 *               send(&sig, pidlist->list[i]);
 *             }
 *           free_buf( (union SIGNAL **) &pidlist);
 *           // Code //
 *         }
 *       else
 *         {
 *           // Code, no processes available to the caller
 *           // in specified block
 *         }
 *     }
 * }
 */
struct OS_pid_list *
get_pid_list(PROCESS bid);

/**
 * @function get_pool_list
 *
 * @include "ose.h"
 *
 * @declaration struct OS_pid_list *
 * get_pool_list(OSUSER user);
 *
 * @brief List pools.
 *
 * @long List all pools that are available.
 *
 * @input user Must be 0.
 *
 * @return Returns a signal buffer with a structure containing the list of
 * pool IDs. The buffer must be freed by the caller when information has
 * been extracted.
 * @break
 * The structure is described by struct @link "ose.h" "OS_pid_list"
 *
 * @seealso OS_pid_list
 * @seealso get_poolcb
 * @seealso get_sig_info
 * @seealso get_sig_poolid
 * @seealso get_stk_poolid
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stdio.h"
 * #include "ose.h"
 *
 * void list_pools(void)
 * {
 *   struct OS_pid_list *list = get_pool_list(0);
 *
 *   if (list != NULL)
 *     {
 *       OSBUFSIZE count = list->count;
 *       int i;
 *
 *       printf("Pools: ");
 *       for(i=0 ;i <count ;i++)
 *         {
 *           printf("%#lx ", list->list[i]);
 *         }
 *       printf("\n");
 *     }
 * }
 */
struct OS_pid_list *
get_pool_list(OSUSER user);

/**
 * @function get_poolcb
 *
 * @include "ose.h"
 *
 * @declaration struct OS_poolcb *
 * get_poolcb(PROCESS pool_id);
 *
 * @brief Get the status of the specified pool.
 *
 * @long
 * NOTE: It is legal to pass an invalid pool ID to this system call.
 *
 * @input pool_id The ID of the specified pool.
 * @return Returns a buffer obtained from the caller's pool containing an
 * OS_poolcb structure with information about a specified pool. The buffer
 * must be returned to the system with free_buf when the information is no
 * longer needed.
 * @break
 * If the get_poolcb call was not successful, a NULL pointer is returned.
 * @break
 * See struct @link "" "OS_poolcb" for a detailed description.
 *
 * @restriction
 * There must be a buffer available in the callers pool that is large
 * enough to hold the returned structure. Otherwise the get_poolcb call
 * will fail.
 *
 * @seealso OS_poolcb
 * @seealso get_pool_list
 * @seealso get_sig_info
 * @seealso get_sig_poolid
 * @seealso get_stk_poolid
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stdio.h"
 * #include "ose.h"
 *
 * void list_pool_info(PROCESS pool_id)
 * {
 *   struct OS_poolcb *pool = get_poolcb(pool_id);
 *   OSADDRESS *value;
 *   OSADDRESS loop;
 *
 *   if (pool == NULL)
 *     {
 *       printf("Error: %#lx is not a valid pool identifier.\n", pool_id);
 *       return;
 *     }
 *   printf("Pool id.....: %#lx\n", pool_id);
 *   printf("Total size..: %lu\n", pool -> tot_size);
 *   printf("Never used..: %lu\n", pool -> free_size);
 *   printf("Stack sizes.: %lu\n", pool -> stk_conf_entries);
 *
 *   if (pool -> stk_conf_entries)
 *     {
 *       value = &pool -> values[pool -> stk_conf_sizes];
 *       loop = pool -> stk_conf_entries;
 *       printf( " Conf.: ");
 *       while (loop--) printf( "%7lu", *value++);
 *       printf( "\n Alloc: ");
 *       value = &pool -> values[pool -> stk_alloc_sizes];
 *       loop = pool -> stk_conf_entries;
 *       while (loop--) printf( "%7lu", *value++);
 *       printf("\n");
 *     }
 *
 *   printf("Signal sizes: %lu\n", pool -> sig_conf_entries);
 *   if (pool -> sig_conf_entries)
 *     {
 *       value = &pool -> values[pool -> sig_conf_sizes];
 *       loop = pool -> sig_conf_entries;
 *       printf( " Conf.: ");
 *       while (loop--)
 *         printf( "%7lu", *value++);
 *       printf( "\n Alloc: ");
 *       value = &pool -> values[pool -> sig_alloc_sizes];
 *       loop = pool -> sig_conf_entries;
 *       while (loop--)
 *         printf( "%7lu", *value++);
 *       printf("\n");
 *     }
 *
 *   loop = pool -> values[pool -> fragment_info];
 *   value = &pool -> values[pool -> fragment_info + 1];
 *   printf("Fragments: %lu\n", loop);
 *   printf(" %10s %10s %10s %10s %10s\n",
 *          "Fragment", "BaseAddr", "Size", "StkUsed", "SigUsed");
 *   do
 *     {
 *       printf(" %10lu %#010lx %10lu %10lu %10lu\n",
 *              loop, value[0], value[1], value[2], value[3]);
 *       value += 4;
 *     } while (--loop);
 *   free_buf((union SIGNAL **)&pool);
 * }
 */
struct OS_poolcb *
get_poolcb(PROCESS pool_id);

/**
 * @function get_pri
 *
 * @include "ose.h"
 *
 * @declaration OSPRIORITY
 * get_pri(PROCESS pid);
 *
 * @brief Get the current priority of the specified process.
 *
 * @input pid The process ID of the specified process.
 *
 * @return Returns the priority. Priority 0 is the highest priority.
 *
 * @restriction
 * Returns 0 when applied to background processes.
 *
 * @seealso set_pri
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   OSPRIORITY prio;
 *
 *   for(;;)
 *     {
 *       prio = get_pri(current_process());
 *       if(prio != 31)
 *         {
 *           set_pri(++prio);
 *         }
 *       else
 *         {
 *           set_pri(0);
 *         }
 *       delay(100);
 *     }
 * }
 */
OSPRIORITY
get_pri(PROCESS pid);

/**
 * @function get_ptype
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration enum PROCESS_TYPE
 * get_ptype(PROCESS pid);
 *
 * @brief Get the process type of the specified process or block.
 *
 * @long
 * This call is intended for use in code that is shared by several
 * processes of different types. For instance, it can be used in common
 * code that needs to react in different ways to calls from interrupt
 * processes and other types.
 * @long
 * Note that it is legal to pass an invalid process ID to this system call.
 * This is legal only for the get_pcb and get_ptype calls. The returned
 * process type will then be OS_ILLEGAL.
 *
 * @input pid The ID of the specified block or process.
 *
 * @return Returns the process type. Legal values are:
 * @tab OS_BLOCK    @tab A block was specified. @break
 * @tab OS_PRI_PROC @tab A prioritized process. @break
 * @tab OS_BG_PROC  @tab A background process. @break
 * @tab OS_INT_PROC @tab An interrupt process. @break
 * @tab OS_TI_PROC  @tab A timer-interrupt process @break
 * @tab OS_PHANTOM  @tab A phantom or dummy process. @break
 * @tab OS_ZOOMBIE  @tab A killed process or block. @break
 * @tab OS_ILLEGAL  @tab An invalid process ID was specified. @break
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * Will not change interrupt mask unless error handler is called.
 *
 * @seealso get_pcb
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   enum PROCESS_TYPE ptype;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       ptype = get_ptype(current_process());
 *       if(ptype == OS_BG_PROC)
 *         {
 *           // Code, specific to a background process //
 *         }
 *       else
 *         {
 *           // Code //
 *         }
 *     }
 * }
 */
enum PROCESS_TYPE
get_ptype(PROCESS pid);

/**
 * @function get_segid
 *
 * @include "ose.h"
 *
 * @declaration PROCESS
 * get_segid(PROCESS pid);
 *
 * @brief Get the domain (program) ID of the specified block, process or pool.
 *
 * @long The domain (program) ID is here defined as
 *       the segment ID of the block, containing the process ID,
 *       or the segment ID of the block ID
 *       or the segment ID of the pool ID.
 *       ID = 0 means the system segment.
 *
 * @input pid The ID of the specified block or process or pool.
 *
 * @return Returns the domain ID.
 * If 'pid' is the ID of a dead block or process or invalid pool, the value
 * specified in 'pid' is returned from the get_segid call.
 *
 * @restriction Will not change interrupt mask unless error handler is called.
 *
 * @seealso attach
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stdio.h"
 * #include "ose.h"
 *
 * extern PROCESS proc_;
 *
 * OS_PROCESS(my_process)
 * {
 *   static const SIGSELECT att_sig[] = { 1, OS_ATTACH_SIG };
 *   union SIGNAL *sig;
 *   PROCESS sid;
 *
 *   sid = get_segid(proc_);
 *   (void)attach(NULL, sid);
 *   for(;;)
 *     {
 *       sig = receive(att_sig);
 *       if (sender(&sig) == sid)
 *         {
 *           printf("The domain containing proc_ has died!\n");
 *         }
 *       free_buf(&sig);
 *       stop(current_process());
 *     }
 * }
 */
PROCESS
get_segid(PROCESS pid);

/**
 * @function get_sem
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration OSSEMVAL
 * get_sem(SEMAPHORE *sem);
 *
 * @brief Get the current value of a semaphore.
 *
 * @input sem A pointer to the specified semaphore.
 *
 * @return Returns the current semaphore value.
 *
 * @restriction Also available in interrupt and timer interrupt processes.
 *
 * @seealso wait_sem
 * @seealso signal_sem
 * @seealso create_sem
 * @seealso kill_sem
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * extern SEMAPHORE *sem;
 *
 * OS_PROCESS(my_process)
 * {
 *   OSSEMVAL semvalue;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       semvalue = get_sem(sem);
 *       if(semvalue == 5)
 *         {
 *           // Code //
 *         }
 *       else
 *         {
 *           // Code //
 *         }
 *     }
 * }
 */
OSSEMVAL
get_sem(SEMAPHORE *sem);

/**
 * @function get_sig_poolid
 *
 * @include "ose.h"
 *
 * @declaration PROCESS
 * get_sig_poolid(PROCESS bid);
 *
 * @brief Get the signal pool ID of a process.
 *
 * @long Returns the ID of the signal pool associated with a specified
 * block or process.
 *
 * @input bid The ID of the specified block or process.
 *
 * @return The ID of the signal pool associated with a specified block or
 * process.
 * @break
 * If 'bid' is the ID of a dead block or process, then the value specified in
 * 'bid' is returned from the get_sig_poolid call.
 *
 * @restriction Also available in interrupt and timer interrupt processes.
 *
 * @seealso get_pool_list
 * @seealso get_poolcb
 * @seealso get_sig_info
 * @seealso get_stk_poolid
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stdio.h"
 * #include "ose.h"
 *
 * void show_my_sig_poolid(void)
 * {
 *   PROCESS poolid = get_sig_poolid(current_process());
 *   printf("My signal pool ID: %#lx", poolid);
 * }
 */
PROCESS
get_sig_poolid(PROCESS bid);

/**
 * @function get_stk_poolid
 *
 * @include "ose.h"
 *
 * @declaration PROCESS
 * get_stk_poolid(PROCESS bid);
 *
 * @brief Get the stack pool ID of a process.
 *
 * @long Returns the ID of the stack pool associated with a specified
 * block or process.
 *
 * @input bid The ID of the specified block or process.
 *
 * @return The ID of the stack pool associated with a specified block or
 * process.
 * @break
 * If 'bid' is the ID of a dead block or process, then the value specified in
 * 'bid' is returned from the get_stk_poolid call.
 *
 * @restriction Also available in interrupt and timer interrupt processes.
 *
 * @seealso get_pool_list
 * @seealso get_poolcb
 * @seealso get_sig_info
 * @seealso get_sig_poolid
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stdio.h"
 * #include "ose.h"
 *
 * void show_my_stk_poolid(void)
 * {
 *   PROCESS poolid = get_stk_poolid(current_process());
 *
 *   printf("My stack pool ID: %#lx", poolid);
 * }
 */
PROCESS
get_stk_poolid(PROCESS bid);

/**
 * @function get_systime
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration OSTICK
 * get_systime(OSTICK *microsecs);
 *
 * @brief Get the elapsed time since system start.
 *
 * @long
 * This call is mainly used by debuggers. Shows the number of system ticks
 * since system start and the number of microseconds since the last tick.
 * @long
 * The system ticks counter is reset to zero each time the kernel is
 * restarted. It is then incremented each time the system clock is advanced
 * i.e. when the tick() system call is issued. The counter wraps after a
 * fairly long period of time.
 *
 * @input microsecs A pointer to the returned number of microseconds. If
 * microsecs == NULL, then the number of microseconds is not returned.
 *
 * @return Returns the number of system ticks since system start. Returns
 * the number of microseconds since the last tick in the microsecs
 * parameter. Multiple calls to get_systime can return the same values,
 * depending on the timer counter resolution.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * Few systems can actually provide an accurate timing reference at the
 * microsecond level. The actual resolution of the microseconds counter
 * must therefore be determined for each target hardware.
 * @break
 * Will not change interrupt mask unless error handler is called.
 *
 * @seealso get_ticks
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   OSTICK systick;
 *   OSTICK micro;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       systick = get_systime(&micro);
 *       // Code //
 *     }
 * }
 */
OSTICK
get_systime(OSTICK *microsecs);

/**
 * @function get_ticks
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration OSTICK
 * get_ticks(void);
 *
 * @brief Get the number of system ticks since system start.
 *
 * @long
 * The system ticks counter is set to zero when time the kernel is started.
 * @long
 * The counter is incremented each time the system clock is advanced (i.e.
 * when the tick system call is issued). The counter wraps after a fairly
 * long period of time.
 * @long
 * Will not change interrupt mask unless error handler is called.
 *
 * @return Returns system time in ticks.
 *
 * @seealso tick
 * @seealso get_systime
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   OSTICK tick;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       tick = get_ticks();
 *       // Code //
 *     }
 * }
 */
OSTICK
get_ticks(void);

/**
 * @function kill_sem
 *
 * @include "ose.h"
 *
 * @declaration void
 * kill_sem(SEMAPHORE *sem);
 *
 * @brief Deletes a semaphore.
 *
 * @long The semaphore that was created by create_sem is returned to the pool.
 *
 * @input sem A pointer to the semaphore.
 *
 * @restriction
 * Make sure no process is using the semaphore when it is killed.
 * @break
 * Never kill a statically declared semaphore structure by calling
 * kill_sem. (A statically declared semaphore is initialized at
 * declaration).
 *
 * @seealso create_sem
 * @seealso signal_sem
 * @seealso wait_sem
 * @seealso get_sem
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   SEMAPHORE *sem;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       sem = create_sem(1);
 *       // Code //
 *       kill_sem(sem);
 *     }
 * }
 */
void
kill_sem(SEMAPHORE *sem);

/**
 * @function restore
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * restore(union SIGNAL *sig);
 *
 * @brief Steal a signal buffer.
 *
 * @long Registers the caller as the current owner of a signal buffer and
 * clears buffer redirection information from it.
 * @long
 * Use this call if:
 * @long
 * 1. A redirected buffer is to be reused without reallocation or
 * @long
 * 2. A buffer has been illegally transferred from one process to another.
 * @long
 * The restore call should only be needed on rare occasions.
 *
 * @input sig A pointer to the specified signal buffer.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * Caution! This call has an unusual parameter syntax.
 * @break
 * In systems with multiple address space, it is an error to restore a
 * buffer that is not in the callers address space.
 * @break
 * If a killed process is part of a design which passes buffers around
 * without proper ownership control, then the following problem may arise:
 * the killed process owns a buffer which is currently in use by some other
 * process. Once the owner is killed, the buffer is returned to the pool
 * without the knowledge of the process which uses the buffer. If that
 * process chooses to send or free the buffer, the system may crash.
 * @break
 * Summary: If using kill_proc extensively, never transfer buffers between
 * processes without using one of the send system calls.
 *
 * @seealso alloc
 * @seealso free_buf
 * @seealso send
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * static const SIGSELECT any_sig[] = {0};
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * OS_PROCESS(my_linkhandler)
 * {
 *   union SIGNAL *sig;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       sig = receive(any_sig);
 *       if(addressee(&sig) != current_process())
 *         {
 *           restore(sig);
 *           // Code //
 *         }
 *       free_buf(&sig);
 *     }
 * }
 */
void
restore(union SIGNAL *sig);

/*
** Take over the ownership of a certain signal from a specified
** process.
**
** If sig is currently owned by curr_owner the ownership of the signal
** will be changed to the calling process and any redirection flag
** will be cleared.
**
** Returns zero if the owner of the signal was successfully changed to
** the PID of the calling process, otherwise a non-zero value is
** returned.
*/
OSBOOLEAN
restore_from(union SIGNAL *sig, PROCESS curr_owner);

/**
 * @function ose_create_ppdata
 *
 * @include "ose.h"
 *
 * @declaration void
 * ose_create_ppdata(const char *key_name, OSPPDKEY *key);
 *
 * @brief Create a key for a per process key name string.
 *
 * @long
 * This function creates a globally unique per process data key. The
 * same key_name string always renders the same (system wide) key value.
 * @long
 * The key should be used as an argument to ose_get_ppdata() which
 * provides fast access to a particular per process object.
 * @long
 * If the total number of keys or the maximum key string length is
 * exceeded, an error is reported.
 * @long
 * This system call can be used from any (executing) type of process.
 *
 * @input key_name The string name of the key.
 *
 * @output key A non-zero key reference to be used when in ose_get_ppdata().
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * The total number of keys is a system parameter and it can not
 * be modified after system start.
 *
 * @seealso ose_get_ppdata
 * @seealso default_restrictions_calling_functions
 */
void
ose_create_ppdata(const char *key_name, OSPPDKEY *key);

/**
 * @function ose_get_ppdata
 *
 * @include "ose.h"
 *
 * @declaration OSADDRESS *
 * ose_get_ppdata(OSPPDKEY key);
 *
 * @brief Returns the address to a per process data pointer.
 *
 * @long
 * The per process data pointer is initialized to NULL. The user
 * "owns" this pointer, and is responsible for the contents in it. In
 * most cases the contents is used for storing a pointer to a data
 * structure that is allocated for a particular purpose. This system
 * call can be used from any type of process.
 * @long
 * The OSPPDKEY key variable has a non-zero value range when it
 * is initialized.
 *
 * @input key The fast-access key obtained in the ose_create_ppdata() call.
 *
 * @return Returns the address to a per process data pointer, uniquely
 * identified by the key.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * The address (but not the content) of a per process data pointer can
 * be changed during the lifetime of a process, i.e. ose_get_ppdata()
 * should ALWAYS be the only way to gain access to the pointer value.
 *
 * @seealso ose_create_ppdata
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * static OSPPDKEY my_pp_key;
 *
 * OS_PROCESS(my_process)
 * {
 *   struct MyProcVars **var_ptr;
 *
 *   // The check and init of my_pp_key may also be done in another
 *   // scope (startup) in order to remove the checks below.
 *   if(!my_pp_key)
 *   {
 *      ose_create_ppdata("my_proc_vars",&my_pp_key);
 *   }
 *   var_ptr = (struct MyProcVars **)ose_get_ppdata(my_pp_key);
 *   if(*var_ptr == NULL)
 *   {
 *      *var_ptr = (struct MyProcVars *)
 *                 alloc(sizeof(struct MyProcVars),0);
 *      // initialize structure
 *   }
 *   // use structure *var_ptr.....
 * }
 */
OSADDRESS *
ose_get_ppdata(OSPPDKEY key);

/**
 * @function ose_mutex_init
 *
 * @include "ose.h"
 *
 * @declaration void
 * ose_mutex_init(MUTEX *mutex, OSADDRESS attr);
 *
 * @brief Initialize the specified mutex.
 *
 * @long
 * Upon successful initialization, the state of the mutex becomes
 * initialized and unlocked.
 *
 * @input mutex A pointer to the mutex.
 * @input attr    Optional mutex initializing attribute bits, which can
 *                be a combination of the following. The default
 *                behavior without attributes gives least system overhead.
 *
 *        @break
 *        OSE_MUTEX_PRIO_INHERIT which specifies the priority
 *        inheritance and the way to manage the queue. Setting this 
 *        attribute bit specifies a mutex with priority inheritance
 *        and a process queue ordered by priority. Without this
 *        attribute, this mutex have the default behavior, which
 *        is a mutex with a "best effort priority order" of the queue.
 *        Best effort here means that the priority is undefined and it
 *        is not always respected; it might happen that some other process
 *        takes it before the waiting queue is examined.
 *        @break
 *        OSE_MUTEX_RECURSIVE which specifies that a process can lock the
 *        mutex more than once. With this attribute set, this mutex remains
 *        locked until it is explicitly unlocked the same number of times.
 *        Without this attribute, it is an error if a process tries to
 *        lock this mutex multiple times.
 *
 * @restriction
 * The time-sliced background processes do not take part in the priority
 * inheritance scheme, and are not recommended to be used with
 * OSE_MUTEX_PRIO_INHERIT mutexes.
 *
 * @error OSE_EMUTEX_ALREADY_INITIALIZED
 * See @link "" "OSE_EMUTEX_ALREADY_INITIALIZED"
 * @error OSE_EBAD_PARAMETER
 * See @link "" "OSE_EBAD_PARAMETER"
 *
 * @example
 * // Create and initialize a mutex
 * ose_mutex_init(&mtx, OSE_MUTEX_PRIO_INHERIT | OSE_MUTEX_RECURSIVE); 
 * ose_mutex_lock(&mtx); 
 * // do something .... and when ready:
 * ose_mutex_unlock(&mtx); 
 *
 * @seealso ose_mutex_lock
 * @seealso ose_mutex_unlock
 * @seealso ose_mutex_trylock
 * @seealso ose_mutex_destroy
 * @seealso default_restrictions_calling_functions
 */

void
ose_mutex_init(MUTEX *mutex, OSADDRESS attr);

/**
 * @function ose_mutex_destroy
 *
 * @include "ose.h"
 *
 * @declaration void
 * ose_mutex_destroy(MUTEX *mutex);
 *
 * @brief Destroy the specified mutex.
 *
 * @long
 * The mutex becomes uninitialized. A destroyed mutex can be reinitialized
 * using ose_mutex_init().
 *
 * @input mutex A pointer to the mutex.
 *
 * @restriction
 * An attempt to destroy a mutex that is currently in use will cause a
 * OSE_EMUTEX_LOCKED or OSE_EMUTEX_IN_USE error. An error handler may
 * decide to ignore these errors.
 *
 * @error OSE_EMUTEX_LOCKED
 * See @link "" "OSE_EMUTEX_LOCKED"
 * @error OSE_EMUTEX_IN_USE
 * See @link "" "OSE_EMUTEX_IN_USE"
 * @error OSE_EMUTEX_INVALID
 * See @link "" "OSE_EMUTEX_INVALID"
 *
 * @seealso ose_mutex_lock
 * @seealso ose_mutex_unlock
 * @seealso ose_mutex_trylock
 * @seealso ose_mutex_init
 * @seealso default_restrictions_calling_functions
 */

void
ose_mutex_destroy(MUTEX *mutex);

/**
 * @function ose_mutex_lock
 *
 * @include "ose.h"
 *
 * @declaration void
 * ose_mutex_lock(MUTEX *mutex);
 *
 * @brief Lock the specified mutex with the calling process as its owner.
 *
 * @long
 * If the mutex is already locked, the calling process blocks until the mutex
 * becomes available.
 * @long
 * The result of killing a process that has locked a mutex is undefined.
 *
 * @input mutex A pointer to the mutex.
 *
 * @error OSE_EMUTEX_INVALID
 * See @link "" "OSE_EMUTEX_INVALID"
 * @error OSE_EMUTEX_DEADLOCK
 * See @link "" "OSE_EMUTEX_DEADLOCK"
 * @error OSE_EMUTEX_OWNER_DEAD
 * See @link "" "OSE_EMUTEX_OWNER_DEAD"
 *
 * @seealso ose_mutex_init
 * @seealso ose_mutex_unlock
 * @seealso ose_mutex_trylock
 * @seealso ose_mutex_destroy
 * @seealso default_restrictions_calling_functions
 */

void
ose_mutex_lock(MUTEX *mutex);

/**
 * @function ose_mutex_trylock
 *
 * @include "ose.h"
 *
 * @declaration int
 * ose_mutex_trylock(MUTEX *mutex);
 *
 * @brief Try to lock a mutex, but return immediately on failure.
 *
 * @long
 * This function is equivalent to ose_mutex_lock(), except that if the mutex
 * is currently locked by another process, the call returns immediately.
 *
 * @input mutex A pointer to the mutex.
 *
 * @return If the lock on the mutex is acquired the function returns non-zero,
 * otherwise the function returns zero.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 *
 * @error OSE_EMUTEX_INVALID
 * See @link "" "OSE_EMUTEX_INVALID"
 * @error OSE_EMUTEX_DEADLOCK
 * See @link "" "OSE_EMUTEX_DEADLOCK"
 *
 * @seealso ose_mutex_lock
 * @seealso ose_mutex_unlock
 * @seealso ose_mutex_init
 * @seealso ose_mutex_destroy
 * @seealso default_restrictions_calling_functions
 */

int
ose_mutex_trylock(MUTEX *mutex);

/**
 * @function ose_mutex_unlock
 *
 * @include "ose.h"
 *
 * @declaration void
 * ose_mutex_unlock(MUTEX *mutex);
 *
 * @brief Unlock the specified mutex.
 *
 * @long
 * If there are multiple processes blocked on the mutex, the order of
 * acquisition is depending on the mutex attribute.
 *
 * @input mutex A pointer to the mutex.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 *
 * @error OSE_EMUTEX_INVALID
 * See @link "" "OSE_EMUTEX_INVALID"
 * @error OSE_EMUTEX_NOT_OWNER
 * See @link "" "OSE_EMUTEX_NOT_OWNER"
 *
 * @seealso ose_mutex_init
 * @seealso ose_mutex_lock
 * @seealso ose_mutex_trylock
 * @seealso ose_mutex_destroy
 * @seealso default_restrictions_calling_functions
 */

void
ose_mutex_unlock(MUTEX *mutex);

/**
 * @function send_w_sender
 *
 * @include "ose.h"
 *
 * @declaration OSBOOLEAN
 * send_w_sender(union SIGNAL **sig, PROCESS from, PROCESS to);
 *
 * @brief Send a signal using an alias.
 *
 * @long Short for send with sender. Works like send, except that the
 * signal buffer is tagged with the specified 'from' process ID in place of
 * the caller's.
 * @long
 * If the 'to' or 'from' process has terminated the signal will be free'd.
 *
 * @input sig   A pointer to the signal buffer.
 * @input from  The ID of the process specified as sender.
 * @input to    The ID of the process the signal will be sent to.
 *
 * @return
 * @tab 0    @tab Signal successfully sent. @break
 * @tab != 0 @tab Signal free'd because the 'from' process has died. @break
 *
 * @restriction Also available in interrupt and timer interrupt processes.
 *
 * @seealso send
 * @seealso receive
 * @seealso receive_w_tmo
 * @seealso receive_from
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * // The declarations of signal numbers and structs are
 * // preferably made in .sig-files which are then #included.
 * #define FIFTH 5
 * struct fifth
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * extern PROCESS proc1_;
 * extern PROCESS proc2_;
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *
 *   for(;;)
 *     {
 *       sig = alloc(sizeof(struct fifth), FIFTH);
 *
 *       // send a signal to proc2 specifying proc1 as sender//
 *       if (send_w_sender(&sig, proc1_, proc2_) == 0)
 *       {
 *         // Successful!
 *       }
 *       else
 *       {
 *         // Signal free'd because proc1_ was dead.
 *       }
 *     }
 * }
 */
OSBOOLEAN
send_w_sender(union SIGNAL **sig, PROCESS from, PROCESS to);

/**
 * @function set_env
 *
 * @include "ose.h"
 *
 * @declaration OSBOOLEAN
 * set_env(PROCESS id, const char *name, const char *value);
 *
 * @brief Set (or unset) an environment variable.
 *
 * @long Create, update or delete an environment string for the specified
 * process or block.
 * @long
 * The environment variable specified by name is set to the value specified
 * in the value parameter. A value set to NULL removes the variable.
 *
 * @input id The ID of the specified block or process.
 * @input name A pointer to the environment variable name.
 * @input value A pointer to the value.
 *
 * @return Returns a non-zero value if out of environment space for the
 * specified process or block.
 *
 * @restriction
 * If the specified name or value is longer than the maximum size of a
 * signal buffer, then the environment variable is not set.
 *
 * @seealso get_env_list
 * @seealso get_env
 * @seealso get_envp
 * @seealso set_envp
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * extern PROCESS proc_;
 *
 * OS_PROCESS(my_process)
 * {
 *   for(;;)
 *     {
 *       set_env(current_process(), "appvar", "xyz21");
 *       set_env(proc_, "number", "2000");
 *       // Code //
 *     }
 * }
 */
OSBOOLEAN
set_env(PROCESS id, const char *name, const char *value);

/**
 * @function set_envp
 *
 * @include "ose.h"
 *
 * @declaration OSBOOLEAN
 * set_envp(PROCESS id, const char *name, OSADDRESS value);
 *
 * @brief Set a binary environment variable.
 *
 * @long Store a 32 bit pointer in a named environment variable.
 * @long
 * The value is stored in a special (mangled) format so that it can be
 * efficiently read with the get_envp call.
 * @long
 * Variables set with the set_envp call can be read with the get_env call,
 * but the string returned is not easily interpreted.
 *
 * @input id The ID of the specified block or process. Setting the id
 * parameter to zero is equivalent to using the PID of the current process.
 * @input name A pointer to the environment variable name.
 * @input value A pointer to the value.
 *
 * @return Returns a non-zero value if the call was successful. Returns
 * zero if the kernel does not support the call or if the call failed due
 * to an ignored error report.
 *
 * @restriction
 * Setting a variable to zero is less useful as that value can not be
 * properly read.
 * @break
 * If the specified name is longer than the maximum size of a signal
 * buffer, then the environment variable is not set.
 *
 * @seealso get_env
 * @seealso get_env_list
 * @seealso get_envp
 * @seealso set_env
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stdio.h"
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   int proc_data = 0xCAFE;
 *   OSADDRESS pa;
 *
 *   for(;;)
 *     {
 *       set_envp(0, "pd", (OSADDRESS)&proc_data);
 *       pa = get_envp(0, "pd");
 *       if (pa != 0)
 *         printf("pa = %d\n", *(int *)pa);
 *       // Code //
 *     }
 * }
 */
OSBOOLEAN
set_envp(PROCESS id, const char *name, OSADDRESS value);

/**
 * @function set_fsem
 *
 * @include "ose.h"
 *
 * @declaration void
 * set_fsem(OSFSEMVAL value, PROCESS pid);
 *
 * @brief Initialize a fast semaphore.
 *
 * @input value The initial value.
 * @input pid The ID of the process for which to set the fast semaphore.
 *
 * @restriction The fast semaphore may not be accessed in any way before it
 * is initialized.
 * @break
 * The first initialization of a fast semaphore, done by the owner
 * process should be by a call to signal_fsem(current_process()) and
 * not using set_fsem, because otherwise if a higher priority process or
 * a process in another core in a multicore system manages to signal this
 * fast semaphore, this event can be lost if set_fsem forces the value to 0.
 * @break
 * It is illegal to set a fast semaphore to a negative value.
 * @break
 * Some OSE implementations behave badly if a fast semaphore is
 * reinitialized while the owner is waiting for it. Portable code should
 * avoid reinitializing fast semaphores.
 *
 * @seealso signal_fsem
 * @seealso wait_fsem
 * @seealso get_fsem
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * extern PROCESS proc_;
 *
 * OS_PROCESS(my_process)
 * {
 *   for(;;)
 *     {
 *       set_fsem((OSFSEMVAL) 1, proc_);
 *       // Code //
 *     }
 * }
 */
void
set_fsem(OSFSEMVAL value, PROCESS pid);

/**
 * @function set_redirection
 *
 * @include "ose.h"
 *
 * @declaration void
 * set_redirection(PROCESS pid, struct OS_redir_entry *redir);
 *
 * @brief Set or replace the redirection table of a process.
 *
 * @long
 * The redir parameter is a pointer to a redirection table that describes
 * how signals sent to the process pointed out by the 'pid' parameter should
 * be redirected. The redirection table contains an array of signal/process
 * pairs. When a signal is sent to the process, the redirection table is
 * searched and if present in the table, the signal is redirected to the
 * process that should receive the signal.
 * @long
 * The redirection table consists of an array of structures of type
 * @link "ose.h" "OS_redir_entry".
 * @long
 * The first structure in a normal redirection table contains a count of the
 * number of entries in the table including the first entry, and a process
 * ID to which signals not specified in the redirection table should be
 * sent. The count is always at least 1, and the default process ID may be
 * set to zero to specify the process to which the redirection table is
 * attached.
 * @long
 * The kernel builds a copy of the table in kernel memory.
 *
 * @input pid The ID of the specified process.
 * @input redir A pointer to the redirection table.
 * @restriction
 * The redirection table must be built in run-time since process identities
 * are unknown at compile time.
 * @break
 * A redirection table can not point out a target that is a phantom
 * representing a link handler connection. In that case, the signal is
 * silently discarded.
 *
 * @seealso create_process
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * #define REDIRECTED_SIG 0xCAFE
 * extern PROCESS proc_;
 *
 * void redir_to_proc(void)
 * {
 *   struct OS_redir_entry redir_tbl[2];
 *
 *   // Set number of entries in the redir table //
 *   redir_tbl[0].sig = 2;
 *
 *   // Set default addressee for signals not found in the redir
 *   // table. Setting this to zero means that the default
 *   // addressee will be set to the process pointed out by the
 *   // 'pid' parameter when calling set_redirection().
 *   redir_tbl[0].pid = 0;
 *
 *   // State that if REDIRECTED_SIG is sent it should be directed
 *   // to the proc_ process.
 *   redir_tbl[1].sig = REDIRECTED_SIG;
 *   redir_tbl[1].pid = proc_;
 *
 *   // Set the redirection table for the current process //
 *   set_redirection(current_process(), redir_tbl);
 * }
 */
void
set_redirection(PROCESS pid, struct OS_redir_entry *redir);

/**
 * @function set_sigsize
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration OSBOOLEAN
 * set_sigsize(union SIGNAL **sig, OSBUFSIZE newsize);
 *
 * @brief Resize an existing signal buffer without reallocation.
 *
 * @long
 * This system call attempts to change the size of a signal buffer without
 * actually reallocate and copy it. This can only be done if the requested
 * size is smaller than the actual maxsize of the buffer.
 *
 * @input sig A pointer to a pointer to a signal buffer.
 * @input newsize The requested new size of the buffer.
 *
 * @return Returns a non-zero value if the operation was successful.
 * Returns zero if the operation failed.
 *
 * @restriction Also available in interrupt and timer interrupt processes.
 *
 * @seealso alloc
 * @seealso sigsize
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * static const SIGSELECT any_sig[] = {0};
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *
 *   for(;;)
 *     {
 *       sig = receive(any_sig);
 *       if (set_sigsize(&sig, sigsize(&sig) + 4))
 *         {
 *           // Successfully enlarged the signal //
 *         }
 *       else
 *         {
 *           // Could not enlarge the signal //
 *         }
 *       // Code //
 *     }
 * }
 */
OSBOOLEAN
set_sigsize(union SIGNAL **sig, OSBUFSIZE newsize);

/**
 * @function system_tick
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration OSTIME
 * system_tick(void);
 *
 * @brief Get the system tick length in microseconds.
 *
 * @return Returns the number of microseconds per system tick.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * This call relies on the tick system call being issued at intervals
 * corresponding to the system tick length configured into the kernel. This
 * can not be ensured by the kernel itself.
 * @break
 * Will not change interrupt mask unless error handler is called.
 *
 * @seealso system_tick
 * @seealso tick
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   OSTIME microsec_per_tick = system_tick();
 *
 *   for(;;)
 *     {
 *       // Code //
 *     }
 * }
 */
OSTIME
system_tick(void);

/**
 * @function wake_up
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration int
 * wake_up(void);
 *
 * @brief Get the reason for activation of an interrupt process.
 *
 * @long
 * An interrupt process can be invoked in two ways; either by a hardware
 * interrupt or by a software event. In both cases the interrupt process is
 * invoked with interrupts masked up to and including the interrupt
 * priority assigned to the process.
 * @long
 * The wake_up system call informs the interrupt process of the way in
 * which it was invoked. This is useful because receive calls are fairly
 * expensive and can often not be afforded at each hardware interrupt.
 * @long
 * Software events that cause interrupt processes to be invoked are signal
 * transmissions to an interrupt process and signal_fsem calls specifying
 * an interrupt process.
 * @long
 * Hardware interrupts are delayed until the wake_up call has completed.
 *
 * @return
 * Returns 0 (zero) if the interrupt process was invoked by a
 * hardware interrupt.
 * @break
 * Returns 1 (one) if the interrupt process was invoked by a signal sent to
 * it. The signal is ready to be fetched with a receive call.
 * @break
 * Returns 2 if the interrupt process was invoked in response to a
 * signal_fsem call issued by some other process. (The value of the fast
 * semaphore is not important and can not be read with the get_fsem system
 * call.)
 * @break
 * If called by a process of a type other than an interrupt process, the
 * wake_up function returns 1 (one).
 *
 * @restriction Also available in interrupt and timer interrupt processes.
 *
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * OS_PROCESS(my_interrupt_process)
 * {
 *   static const SIGSELECT all[] = {0};
 *   union SIGNAL *sig;
 *
 *   switch(wake_up())
 *     {
 *     case 0:
 *       // hardware interrupt invoked the interrupt process //
 *       break;
 *
 *     case 1:
 *       // A signal sent to this process caused the wake_up //
 *       sig = receive(all);
 *       free_buf(&sig);
 *       break;
 *
 *     case 2:
 *       // signal_fsem() on this process caused the wake_up //
 *       break;
 *
 *     default:
 *       // this can never happen //
 *       break;
 *     }
 * }
 */
int
wake_up(void);

#if defined(softOSE) && !defined(NO_ZZZSEND)
void
zzzsend(union SIGNAL **sig, PROCESS to);
#endif

/*
** The "look and feel" of the following system calls will remain as below
** even though they are not declared here in case OS_DEBUG is defined.
*/
#if defined(__cplusplus) && defined(OS_DEBUG)
#else

/**
 * @function addressee
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration PROCESS
 * addressee(union SIGNAL **sig);
 *
 * @brief Get the original addressee of a redirected signal.
 *
 * @long
 * Examines a signal buffer to find out which process the buffer was
 * originally sent to by the sender.
 *
 * @input sig A pointer to a pointer to a signal buffer.
 *
 * @return The process ID of the caller is returned in case the signal was
 * sent without redirections. If sig has been redirected, the addressee
 * returns the process ID that was stated in the send or send_w_s call that
 * caused the buffer redirection.
 * Returns owner's ID if the signal was never sent.
 *
 * @restriction Also available in interrupt and timer interrupt processes.
 *
 * @seealso restore
 * @seealso sender
 * @seealso send
 * @seealso send_w_s
 *
 * @example
 * #include "ose.h"
 *
 * static const SIGSELECT any_sig[] = {0};
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *   PROCESS addr;
 *
 *   for(;;)
 *     {
 *       sig = receive(any_sig);
 *       addr = addressee(&sig);
 *       if(addr == current_process ())
 *         {
 *           // The signal was sent directly to this process //
 *         }
 *       else
 *         {
 *           // Signal was sent to the process with process ID
 *           // "addr" but was redirected to this process through
 *           // the use of a redirection table
 *         }
 *     }
 * }
 */
PROCESS
addressee(union SIGNAL **sig);

/**
 * @function attach
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration OSATTREF
 * attach(union SIGNAL **sig, PROCESS id);
 *
 * @brief Register supervision of another process.
 *
 * @long
 * This call is used in defensive programming to detect and resolve
 * unexpected situations that occur when processes die during signal
 * transactions.
 * @long
 * Attach stores a signal buffer owned by the caller in the control block
 * of the specified process or block.
 * @long
 * If the sig parameter is set to NULL, the kernel automatically allocates
 * a signal with signal number OS_ATTACH_SIG.
 * @long
 * If the attached process or block is killed, this buffer is sent back to
 * the caller by the kernel. The buffer will be sent back immediately to
 * the caller if the process or block is already dead when issuing the
 * attach.
 * @long
 * When the calling process issues a detach call for the previously
 * attached process or block, the buffer is freed by the kernel using the
 * reference ID obtained from attach. This still works even if the attached
 * process has been killed and the attached buffer waits in the caller's
 * signal queue.
 * @long
 * Normal buffer examination calls, like sender and addressee, work on the
 * returned buffer. Sender is set to the process ID of the killed process.
 * @long
 * When a process dies, OSE ensures that attached signals are the last
 * signals sent from the killed process. A supervising process can safely
 * clear signal queues after it has received an attached signal.
 * @long
 * The attach system call can also be used by memory manager software to
 * supervise a domain. A domain ID obtained from either the
 * attach_segment system call or the get_segid system call can be attached
 * to. A memory manager can then be conveniently notified when a killed
 * domain can be reclaimed.
 *
 * @input sig A pointer to a pointer to a signal buffer.
 * @input id The ID of the specified block, process or domain to attach.
 *
 * @return Returns a reference ID that may be used in a subsequent call to
 * detach.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * When a process dies, the order in which attached signals are returned is
 * unspecified. A user can not assume that attached signals are returned
 * immediately when a process dies. A delay is caused because process
 * housekeeping is performed by system daemons as a lower priority job.
 *
 * @seealso detach
 * @seealso kill_proc
 * @seealso OS_ATTACH_SIG
 * @seealso get_segid
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * // The declarations of signal numbers and structs are
 * // preferably made in .sig-files which are then #included.
 * #define FOURTH 4
 * struct Fourth
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 *   struct Fourth fourth;
 * };
 *
 * static const SIGSELECT any_sig[] = {0};
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *   OSATTREF attref;
 *   extern PROCESS proc_;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       sig = alloc(sizeof(struct Fourth), FOURTH);
 *       attref = attach(&sig,proc_);
 *
 *       sig = receive(any_sig);
 *       if (sig->sig_no == FOURTH)
 *         {
 *           // Code, The process has been killed //
 *         }
 *       else
 *         {
 *           // Code //
 *         }
 *     }
 * }
 */
OSATTREF
attach(union SIGNAL **sig, PROCESS id);

/**
 * @function create_block
 *
 * @include "ose.h"
 *
 * @declaration PROCESS
 * create_block(const char *name, OSUSER user, OSBOOLEAN use_remote_calls,
 *              PROCESS remote_call_server, OSBOOLEAN supervisor_mode);
 *
 * @brief Create a block.
 *
 * @long
 * Creates a block descriptor.
 * A new block inherits copies of all environment variables from its parent
 * block, i.e. the kernel will copy all the environment variables in the
 * creator's block into the newly created block. The memory domain is
 * inherited from the caller, i.e. all processes in the new block execute
 * in the same memory domain as the creator. Another memory domain can be
 * attached to the block by a memory manager prior to the first process
 * being created. This is desirable when a new, separately linked software
 * unit (load module) is being loaded into memory for execution.
 * @long
 * New block descriptors are given "stopped" status so an environment can
 * be initialized before operation begins. The first set of processes in a
 * new block are "stopped" until either a start call for the block or when
 * each process in the block is started separately.
 * @long
 * Until the last start call is issued, the block is owned by its creator.
 * If the creator is killed before this happens, the child block is also
 * killed. This mechanism ensures that partially created blocks are never
 * left abandoned in the system.
 * @long
 * A block descriptor is removed from the system when either the block is
 * explicitly killed, or the last process executing in the block is killed.
 *
 * @input name A pointer to the name of the block. This is the name by
 * which the block will be known to the system and to the debugger.
 * See @link "" "create_process" for reserved characters that shall not be
 * used in a block name.
 * @input user Must be 0.
 * @input use_remote_calls Must be 0.
 * @input remote_call_server Must be 0.
 * @input supervisor_mode The specified mode, 0 means user mode, non-zero
 * means supervisor mode. Supervisor mode processes must reside in kernel
 * address space and can only be created by a process executing in supervisor
 * mode.
 *
 * @return Returns the block ID for the new block.
 *
 * @restriction
 * Supervisor mode blocks can only be created by a process
 * executing in supervisor mode itself.
 * @break
 * The create_block call is not allowed from a block that uses the shared
 * execution model. An attempt to call it returns an error code.
 * @break
 * Creation of user mode processes running in a supervisor module
 * is not meaningful in a system using memory protection, and will
 * likely cause memory exceptions when such processes are started.
 *
 * @seealso create_process
 * @seealso get_bid
 * @seealso get_bid_list
 * @seealso kill_proc
 * @seealso start
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stddef.h"
 * #include "ose.h"
 *
 * OSENTRYPOINT new_process;
 *
 * OS_PROCESS(my_process)
 * {
 *   PROCESS block_;
 *   PROCESS proc_;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       block_ = create_block("block",
 *                             (OSUSER) 0,
 *                             (OSBOOLEAN) 0,
 *                             (PROCESS) 0,
 *                             (OSBOOLEAN) 0);
 *
 *       proc_=create_process(OS_PRI_PROC,
 *                            "proc_",
 *                            new_process,
 *                            200,
 *                            16,
 *                            (OSTIME) 0,
 *                            block_,
 *                            (struct OS_redir_entry *) NULL,
 *                            (OSVECTOR) 0,
 *                            (OSUSER) 0);
 *
 *       start(block_);
 *       // Code //
 *     }
 * }
 */
PROCESS
create_block(const char *name, OSUSER user, OSBOOLEAN use_remote_calls,
	     PROCESS remote_call_server, OSBOOLEAN supervisor_mode);

/**
 * @function create_process
 *
 * @include "ose.h"
 *
 * @declaration PROCESS
 * create_process(enum PROCESS_TYPE proc_type, const char *name,
 *	          OSENTRYPOINT *entrypoint, OSADDRESS stack_size,
 *	          OSPRIORITY priority, OSTIME timeslice, PROCESS block,
 *	          struct OS_redir_entry *router_table, OSVECTOR vec,
 * 	          OSUSER user);
 *
 * @brief Create a process.
 *
 * @long
 * Prioritized, background, and timer-interrupt processes must be
 * subsequently started with the start system call, or the entire block can
 * be started if only new processes are contained in it.
 * @long
 * The actual creation of the process will be
 * handled by the system daemon (ose_sysd) that typically has a different
 * priority than the caller. The caller will be blocked (swapped out) until the
 * system daemon has created the child process, at the priority of the system
 * daemon.
 *
 * @input proc_type Specifies the type of the process.
 * See @link "ose.h" "PROCESS_TYPE" for available types.
 *
 * @input name A pointer to the name of the process.
 * The name is a zero-terminated ASCII string.
 * This string may in some implementations be copied by the kernel and
 * stored in kernel memory. The name is used as a tag by other processes
 * when searching for a process with the hunt call. The name may contain
 * all printable characters, but some characters are reserved for
 * internal use. It is recommended to use characters in the range
 * a-z A-Z 0-9 or the characters _ (underline) or - (hyphen) in names.
 * @break
 * The slash (/) has special significance because it is recognized by the
 * kernel as a path separator. It separates components in the name, which
 * usually represents a network routing path or a block link name (program
 * name). A link handler may however choose to use the pathname in any way
 * desired.
 * The colon (:) is used by the kernel as a separator and the question
 * mark (?) and the asterisk (*) are used as wildcards. These must be
 * avoided in block, process, or program names.
 * See @link "ose_spi/monitor.sig" "MonitorScopeTypeValue".
 * It is also highly recommended to avoid special characters like
 * &, @, #, $, !, \, ", and ', these are internally used in a few cases.
 * The create_process call however, does not manipulate the name string,
 * it is just stored away, path and all.
 *
 * @input entrypoint The address of where the process will start execution.
 * Define the entrypoint with the @link "" "OS_PROCESS" macro.
 *
 * @input stack_size Specifies the size of the stack.
 * Stack_size is the number of bytes that should be allocated for the
 * stack. Stacks are allocated according to a complex set of rules.
 * Supervisor type interrupt and timer-interrupt processes share a common
 * interrupt stack in kernel space. The size of this stack is automatically
 * adjusted to maximum requirements. Prioritized and background processes
 * allocate the user stack from the pool of the block they belong to, the
 * supervisor stack is allocated from the system pool.
 *
 * @input priority The priority level for the process.
 * Legal values are 0-31. 0 is the highest priority. The priority
 * is interpreted in different ways for the various process types.
 * For interrupt processes it means hardware priority. The logical
 * interrupt priority (0-31) maps to actual hardware priority in
 * a processor architecture specific way. For prioritized processes, it
 * reflects the real-time criticality of the process.
 * For timer-interrupt, background and
 * phantom processes, the priority is not used (but the
 * parameter must be set to 0).
 *
 * @input timeslice Number of milliseconds in the timeslice.
 * Timeslice is used for timer-interrupt processes, prioritized
 * processes, and background processes. Set it to zero for interrupt
 * processes. For timer-interrupt processes the timeslice represents
 * the number of milliseconds between each activation of the
 * process.
 * For background and prioritized processes it represents the maximum
 * number of milliseconds the process is allowed to run before the next
 * background or prioritized process is scheduled. @break
 * The stipulated time is rounded upwards to an integral number of
 * system tick periods by the kernel. A resulting timeslice of 1 means
 * execute until the next system tick. A timeslice of 0 means the
 * default timeslice for background processes and a period of one
 * system tick for timer-interrupt processes. For prioritized processes,
 * a timeslice of 0 means that timeslicing is disabled. @break
 * Any attempt to create a background process with a timeslice longer
 * than the default value causes the process to be created with the
 * default timeslice for background processes (see restriction).
 *
 * @input block Block is the ID of the block that contains the new process.
 * The block ID indicates among other things in which domain the
 * process executes. Set
 * block ID to zero for the new process to become part of the caller's block.
 *
 * @input router_table A pointer to the redirection table, if present.
 * Set to NULL to disable redirection. If non-NULL, this structure contains an
 * array of signal/process pairs. When a signal is sent to the created
 * process, the redirection table is searched and if the signal number is
 * present in the table, the signal is redirected to the process that is
 * indicated in the redirection table.
 * @break
 * The redirection table consists of an array of structures of
 * type @link "ose.h" "OS_redir_entry".
 * The first structure in a normal redirection table contains a count of the
 * number of entries in the table including the first entry, and a process
 * ID to which signals not specified in the redirection table should be
 * sent. The count is always at least 1, and the default process ID may be
 * set to zero to specify the created process.
 * @break
 * The kernel builds a copy of the redirection table in kernel memory.
 *
 * @input vec The interrupt vector number. The vector is valid for
 * interrupt processes only. The board-specific vector handler identifies
 * each hardware interrupt, and returns a vector. The vector is used
 * by the kernel to select the interrupt process that serves the
 * actual interrupt.
 * @break
 * The vector number ~0, a bitwise  negated zero, causes the kernel to
 * skip hardware vector generation. Such a process can only be invoked
 * by sending a signal to it or signaling its fast semaphore. Use the
 * wake_up() system call to gain knowledge about why the interrupt process
 * was invoked.
 *
 * @input user Must be 0.
 *
 * @return Returns the process ID for the new process.
 *
 * @restriction
 * The redirection table must be built in run-time since process identities
 * are unknown at compile-time. This system call may enable interrupts if
 * it is called  with interrupts disabled.
 * @break
 * Supervisor mode processes (part of a supervisor mode block) can only be
 * created by a process executing in supervisor mode itself.
 * @break
 * Some implementations can not create timer-interrupt processes with an
 * invocation period (timeslice value) larger than 256 times the length
 * of a tick, i.e. 256 * (system_tick() / 1000) milliseconds.
 * @break
 * The default and maximum timeslice for background processes is
 * 10 ticks, i.e. 10 * (system_tick() / 1000) milliseconds.
 *
 * @seealso current_process
 * @seealso create_block
 * @seealso kill_proc
 * @seealso OS_PROCESS
 * @seealso set_redirection
 * @seealso start
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stddef.h"
 * #include "ose.h"
 *
 * extern OSENTRYPOINT new_process;
 *
 * OS_PROCESS(my_process)
 * {
 *   PROCESS proc_;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       proc_ = create_process(OS_PRI_PROC,
 *                              "proc",
 *                              new_process,
 *                              200,
 *                              16,
 *                              (OSTIME) 0,
 *                              (PROCESS) 0,
 *                              (struct OS_redir_entry *) NULL,
 *                              (OSVECTOR) 0,
 *                              (OSUSER) 0);
 *       start(proc_);
 *
 *       // Code //
 *     }
 * }
 */
PROCESS
create_process(enum PROCESS_TYPE proc_type, const char *name,
	       OSENTRYPOINT *entrypoint, OSADDRESS stack_size,
	       OSPRIORITY priority, OSTIME timeslice, PROCESS block,
	       struct OS_redir_entry *router_table, OSVECTOR vec,
	       OSUSER user);

/**
 * @function delay
 *
 * @include "ose.h"
 *
 * @declaration void
 * delay(OSTIME timeout);
 *
 * @brief Suspend the current process.
 *
 * @long
 * Suspends the calling process for the number of milliseconds specified in
 * the timeout parameter.
 * @long
 * The specified timeout period is converted to a number of actual system
 * ticks according to the following rules:
 * @long
 * - A value of zero returns immediately.
 * @long
 * - A value equal to or smaller than one system tick period returns at the
 * next system timer event, i.e. after anything between 0 and 1 system tick.
 * @long
 * - A value higher than one system tick period is rounded upwards to the
 * closest number of system ticks. The actual delay will vary between the
 * calculated number of ticks and this value less one tick period.
 * @long
 * The result is that the actual delay may be up to one system tick period
 * shorter than specified.
 *
 * @input timeout Specifies the number of milliseconds to suspend the
 * process.
 *
 * @seealso receive_w_tmo
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * void set_led(int on);
 *
 * OS_PROCESS(my_process)
 * {
 *
 *   for(;;)
 *     {
 *       set_led(1);
 *       delay(100);
 *       set_led(0);
 *       delay(100);
 *     }
 * }
 */
void
delay(OSTIME timeout);

/**
 * @function detach
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * detach(OSATTREF *sigref);
 *
 * @brief Remove supervision of another process.
 * @long
 * Removes a signal previously attached by the caller. The sigref parameter
 * contains a pointer to the reference ID which was returned by the
 * previous call to attach. This still works even if the attached process
 * has been killed and the attached signal already waits in the caller's
 * signal queue.
 * @long With OSE5.5.2 and later versions, detach() sets the ATTREF 
 * variable to OSE_ILLEGAL_ATTREF to inform the user that it is no 
 * longer in use.
 *
 * @input sigref A pointer to the reference ID of the previously attached
 * signal.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * It is illegal to detach a signal already received and freed by the
 * caller.
 * @break
 * It is illegal for a process to use a sigref parameter obtained by another
 * process, i.e. corresponding attach and detach calls must be made by the
 * same process.
 *
 * @seealso attach
 * @seealso OS_ATTACH_SIG
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stddef.h"
 * #include "ose.h"
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * static const SIGSELECT attach_sig[2] = {1, OS_ATTACH_SIG};
 * extern PROCESS proc_;
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *   OSATTREF attref;
 *
 *   for(;;)
 *     {
 *       attref = attach(NULL, proc_);
 *       // Code //
 *
 *       sig = receive_from(0, attach_sig, proc_);
 *       if(sig == NIL)
 *         {
 *           detach(&attref);// Process not dead //
 *         }
 *       else
 *         {
 *           free_buf(&sig); // Process is dead //
 *         }
 *     }
 * }
 */
void
detach(OSATTREF *sigref);

/**
 * @function error
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * error(OSERRCODE ecode);
 *
 * @brief Report an error.
 *
 * @long
 * Reports an error to the error handlers.
 * @long
 * It is suggested that error codes are grouped into ranges for
 * classification of the error. For instance, an error code with the most
 * significant bit set to zero can be interpreted as a warning while other
 * error codes can be considered fatal for the calling process.
 * @long
 * Errors generated by this call is considered user_called (1) and non-fatal,
 * even if the fatal bit (bit 31) is set in the ecode value.
 * User defined error handlers might use bit 31 as a soft fatal bit,
 * to decide what to do. Users/applications issuing error calls,
 * should use error codes in free ranges as listed in the appendix of the
 * "OSE Architecture User's Guide".
 *
 * @input ecode The error number to be passed to the error handler.
 *
 * @return The error call may or may not return, depending on how the error
 * handler policy is implemented by the user.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 *
 * @seealso error2
 * @seealso create_error_handler
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   for(;;)
 *     {
 *       // Code //
 *       error(6);
 *       // Code //
 *     }
 * }
 */
void
error(OSERRCODE ecode);

/**
 * @function error2
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * error2(OSERRCODE ecode, OSERRCODE extra);
 *
 * @brief Report an error with additional information.
 *
 * @long
 * Reports an error to the error handlers.
 * @long
 * The ecode parameter contains a user-defined numerical error code. This
 * code is passed to the error handler for evaluation.
 * @long
 * It is suggested that error codes are grouped into ranges for
 * classification of the error. For instance, an error code with the most
 * significant bit set to zero can be interpreted as a warning while other
 * error codes can be considered fatal for the calling process.
 * @long
 * Errors generated by this call is considered user_called (1) and non-fatal,
 * even if the fatal bit (bit 31) is set in the ecode value.
 * User defined error handlers might use bit 31 as a soft fatal bit,
 * to decide what to do. Users/applications issuing error calls,
 * should use error codes in free ranges as listed in the appendix of the
 * "OSE Architecture User's Guide".
 * @long
 * The extra parameter is there for passing extra information to the
 * error handler. It might be a pointer to some information that should
 * be available when handling the error in the error handler.
 *
 * @input ecode The error number to be passed to the error handler.
 * @input extra This parameter will be passed to the error handler, and show
 * up as the extra parameter in the error handler.
 *
 * @return The error call may or may not return, depending on how the error
 * handler policy is implemented by the user.
 *
 * @restriction
 * This system call may enable interrupts if it is called with interrupts
 * disabled.
 *
 * @seealso error
 * @seealso create_error_handler
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   for(;;)
 *     {
 *       const char *error_str = "This is no good";
 *
 *       // Code //
 *       error2(6, (OSERRCODE)error_str);
 *       // Code //
 *     }
 * }
 */
void
error2(OSERRCODE ecode, OSERRCODE extra);

/**
 * @function free_buf
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * free_buf(union SIGNAL **sig);
 *
 * @brief Release a signal buffer.
 *
 * @long
 * The signal buffer, that is no longer needed, is freed to the pool.
 * NIL is entered into the caller's signal pointer
 * to avoid accidental reuse of the buffer. It is an error to call free_buf
 * with a NIL signal pointer.
 *
 * @input sig A pointer to a pointer to the signal buffer to free.
 *
 * @restriction It is an error to free a buffer owned by another process.
 * @break
 * Also available in interrupt and timer interrupt processes.
 * @break
 * Note that many OSE calls return buffers that are allocated by the
 * kernel. These buffers are typed as required by the various calls and
 * must be cast to (union SIGNAL **) when freed.
 *
 * @seealso alloc
 * @seealso restore
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * static const SIGSELECT any_sig[] = {0};
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *
 *   for(;;)
 *     {
 *       // Code //
 *       sig = receive(any_sig);
 *       // Code //
 *       free_buf(&sig);
 *     }
 * }
 */
void
free_buf(union SIGNAL **sig);

/**
 * @function hunt
 *
 * @include "ose.h"
 *
 * @declaration OSBOOLEAN
 * hunt(const char *name, OSUSER user, PROCESS *name_,
 *      union SIGNAL **hunt_sig);
 *
 * @brief Search for a process.
 *
 * @long Searches for a process by the given name.
 *
 * @long
 * The actual search will be handled by the hunt daemon (ose_huntd) that
 * typically has a different priority than the caller. The caller will be
 * blocked (swapped out) while the hunt daemon process executes on it's
 * own priority until it can respond with a positive or negative answer.
 *
 * @long
 * In case there already is a process present with the specified name, hunt
 * returns after finding the local process (or an already existing local
 * phantom representing a remote process).
 * If the name_ parameter is not NULL, the process ID of the found process
 * is stored in name_. If both name_ and hunt_sig are NULL, the caller gets
 * no information about the PID of the found process.
 *
 * @long
 * If hunt_sig is not NULL, the specified signal will be sent to the
 * caller as soon as the process is found. If the sought process
 * already exists, the signal will be sent immediately. If not, the hunt_sig
 * will be saved in the pending hunts queue, to be sent later when the
 * sought process is located or created.
 *
 * @long
 * If the process with the specified name is not already present,
 * the available link handlers are scanned, but first hunt returns
 * unsuccessfully, and the hunt_sig is added to the pending hunts queue.
 * To handle this situation, the caller should provide a hunt_sig in
 * the hunt call, in which case the hunt daemon later will send the
 * hunt_sig to the caller when the process has been found or created.
 *
 * @long
 * If no hunt_sig was specified, the caller needs to try again at
 * a later time with a new hunt, but should avoid too frequent calls
 * to not cause an unneccessary load on the system.
 *
 * @long
 * Processes are searched for in the following order:
 * @long
 * 1. Processes within the caller's block.
 * @long
 * 2. Processes that are "block processes" in other blocks.
 * A block process is a process with the same name as the block
 * it belongs to.
 * @long
 * 3. Processes in other blocks, where the block does not have an assigned
 * block link. A block link can be assigned with @link "" "assign_linkhandler"().
 * @long
 * 4. If there is no match in the previously listed places, the registered
 * link paths are searched for a match. If a matched path is assigned to a link
 * handler process, the link handler process is asked for the process ID.
 * A link handler can cause the hunt call to return unsuccessful, to allow the
 * calling process to continue, while the link handler works on the pending
 * hunt to find the process.
 *
 * @long
 * If the matched link path is a block link, the kernel searches processes
 * directly in assigned block for names matching the part of the hunt path
 * following the link name. If the process is found, the hunt_sig is sent
 * to the hunt caller.
 * @long
 * If the matched link path is through a link handler which handles
 * remote systems, the link handler connects to the remote system and hunts
 * there for the process. If the remote process is found, the link handler
 * and the remote link handler create phantoms and the hunt_sig is sent
 * to the hunt caller.
 *
 * @long
 * When the process is found or created, the hunt signal is sent to the
 * calling process, and the process ID of the found process can be extracted
 * with the @link "" "sender" call when the signal has been received.
 * @long
 * The kernel frees the pending hunt signal if the calling process dies.
 *
 * @input name A pointer to the specified process name.
 * @input user Must be 0.
 * @input name_ A pointer to where to store the ID of the found process or NULL.
 * @input hunt_sig A pointer to the hunt signal buffer or NULL. The kernel
 * takes ownership of the signal buffer.
 *
 * @output name_ If name_ is not NULL, the referenced variable is updated with
 * the found process ID, or with @link "" "OSE_ILLEGAL_PROCESS" if the process
 * was not immediately found.
 *
 * @return The call returns zero (unsuccessful) if the process could not be
 * found immediately. In that case, any hunt signal given in the hunt_sig
 * argument is stored in the kernel as a pending hunt until a matching process
 * appears at a later time.
 * @break
 * Returns a non-zero value if the process was found immediately. In that case,
 * any hunt signal provided in the hunt_sig argument is also sent to the calling
 * process.
 * @break
 * The returned value is further subdivided to indicate the scope of the
 * found process. This is important only to link handlers when propagating
 * hunt requests.
 * @break
 * If the hunt call returns 1, then the found process is "public", meaning
 * that access to it is not restricted to any particular process.
 * @break
 * If the hunt call returns 2, then the found process is "private", meaning
 * that access is restricted to the client process that hunted for it. This
 * restriction is not enforced by the kernel, but a link handler is assumed
 * to delete the private process when the client terminates.
 *
 * @restriction
 * The kernel takes ownership of the hunt signal and may reallocate the
 * space occupied by the hunt signal. Thus, the pointer to the hunt signal
 * is not valid following the call to hunt.
 *
 * @seealso attach
 * @seealso detach
 * @seealso assign_linkhandler
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stddef.h"
 * #include "ose.h"
 *
 * // The declarations of signal numbers and structs are
 * // preferably made in .sig-files which are then #included.
 * #define SIXTH 6
 * struct Sixth
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 *   struct Sixth sixth;
 * };
 *
 * OS_PROCESS(my_process)
 * {
 *   static const SIGSELECT sel_sixth[] = {1,SIXTH};
 *   union SIGNAL *huntsignal;
 *   PROCESS huntproc_;
 *   OSATTREF attref;
 *
 *   for(;;)
 *     {
 *       // ...Code... //
 *       // Hunt for the process and wait for response signal //
 *       huntsignal = alloc(sizeof(struct Sixth), SIXTH);
 *       (void) hunt("proc", 0, (PROCESS *) NULL, &huntsignal);
 *       huntsignal = receive(sel_sixth);
 *
 *       // Save the found process identity //
 *       huntproc_ = sender(&huntsignal);
 *       free_buf(&huntsignal);
 *
 *       // Supervise the existence of the found process //
 *       attref = attach((union SIGNAL **) NULL, huntproc_);
 *       // ...Code... //
 *     }
 * }
 */
OSBOOLEAN
hunt(const char *name, OSUSER user, PROCESS *name_,
     union SIGNAL **hunt_sig);

/**
 * @function kill_proc
 *
 * @include "ose.h"
 *
 * @declaration void
 * kill_proc(PROCESS id);
 *
 * @brief Kill a process, block or domain.
 *
 * @long
 * If a block is specified, then all processes in that block are killed.
 * If there are no processes left, the block descriptor is then deleted.
 * @long
 * If a single process is specified, then that process only is killed,
 * unless it is the last process in a block, in which case the block is
 * deleted too, as described above.
 * @long
 * In any case, all signals attached to killed processes and blocks and
 * domains are eventually returned to their respective owners.
 * @long
 * If a killed process owns signal buffers, hunt signals etc., these are
 * freed and the memory returned to the system. If a killed process was
 * a link handler, then that information is deleted from the kernel lists.
 * @long
 * The kill_proc system call can also be used by a memory manager to kill
 * a domain. A domain ID obtained from either the
 * attach_segment system call or the get_segid system call can be passed to
 * kill_proc, which causes deletion of all processes and blocks in the
 * domain.
 *
 * @input id The ID of the specified block, process or domain.
 *
 * @restriction
 * Killing a static process is allowed by the kernel only when the process
 * is part of a separately linked software unit (program), otherwise it is
 * an illegal operation.
 * @break
 * By default, killing a static process causes the load or core module
 * to terminate. Other rules may apply as desired for each system.
 * @break
 * When the kill_proc system call is executed, the kernel removes the
 * affected process(es) and blocks from system tables and invokes the lower
 * priority system daemon to return attached signals and clean up at a
 * later time. Kill_proc returns to the caller before this work begins.
 * This has some important implications:
 * @break
 * 1. A significant amount of time may elapse before attached signals are
 * returned.
 * @break
 * 2. Excessive process creation and destruction may exhaust system
 * resources.
 *
 * @seealso create_block
 * @seealso create_process
 * @seealso attach
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stddef.h"
 * #include "ose.h"
 *
 * OSENTRYPOINT new_process;
 *
 * OS_PROCESS(my_process)
 * {
 *   PROCESS proc_;
 *
 *   for(;;)
 *     {
 *       proc_ = create_process(OS_PRI_PROC,
 *                              "process",
 *                              new_process,
 *                              200,
 *                              16,
 *                              (OSTIME) 0,
 *                              (PROCESS) 0,
 *                              (struct OS_redir_entry *) NULL,
 *                              (OSVECTOR) 0,
 *                              (OSUSER) 0);
 *       // Code //
 *       kill_proc(proc_);
 *     }
 * }
 */
void
kill_proc(PROCESS id);

/**
 * @function receive
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration union SIGNAL *
 * receive(const SIGSELECT *sigsel);
 *
 * @brief Receive a signal from the caller's signal queue.
 *
 * @long
 * If an appropriate signal is not immediately available, the caller pends
 * until an appropriate signal is sent to the caller by some other process.
 * @long
 * Sigsel points to an array containing a list of signal numbers to be
 * received. Receive returns to the caller when a signal matching any of
 * the specified signal numbers is found.
 * @long
 * The first position in sigsel contains the number of entries in the list
 * that follows.
 * @long
 * If the number of entries is zero, any signal number is accepted.
 * @long
 * When called from an interrupt or timer-interrupt process, receive always
 * returns immediately. If no requested signal was immediately available in
 * the signal queue, NIL is returned. This works in a manner similar to
 * receive_w_tmo with a 0 timeout. (Note that receive_w_tmo is not
 * available to interrupt and timer-interrupt processes.)
 * @long
 * A "negative receive specification" feature is supported. This means that
 * if the first location of the sigsel array contains a negative count,
 * then any signal except those that match the sigsel array is received.
 * This feature effectively inverts the semantics of the receive,
 * receive_w_tmo and receive_from calls, providing a means of holding back
 * certain signals.
 * @long
 * The SIGSELECT type is an unsigned type, so using a negative receive
 * specification requires the count in the sigsel array to be initialized
 * with the cast (SIGSELECT).
 *
 * @input sigsel Pointer to an array of the signal numbers to receive.
 *
 * @return Returns a pointer to the received signal buffer. This buffer is
 * owned by the caller from that moment.
 * @break
 * If caller is an interrupt process and there is no signal immediately
 * available that matches sigsel, NIL is returned.
 *
 * @restriction Also available in interrupt and timer interrupt processes.
 *
 * @seealso send
 * @seealso send_w_s
 * @seealso receive_w_tmo
 * @seealso receive_from
 * @seealso alloc
 * @seealso free_buf
 * @seealso sender
 * @seealso sigsize
 * @seealso addressee
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * // The declarations of signal numbers and structs are
 * // preferably made in .sig-files which are then #included.
 * #define THIRD 3
 * struct Third
 * {
 *   SIGSELECT sig_no;
 * };
 * #define FIFTH 5
 * struct Fifth
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * // A union SIGNAL declaration must include a
 * // signal number and the signal structures of
 * // the signals that will be used by the program.
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 *   struct Third third;
 *   struct Fifth fifth;
 * };
 *
 * static const SIGSELECT any_sig[] = {0};
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *
 *   for(;;)
 *     {
 *       sig = receive(any_sig);
 *       switch(sig->sig_no)
 *         {
 *         case THIRD:
 *           // Code //
 *           break;
 *         case FIFTH:
 *           // Code //
 *           break;
 *         default:
 *           // Code //
 *           break;
 *         }
 *       free_buf(&sig);
 *     }
 * }
 *
 *
 *
 * // Example with negative selection, to receive all signals
 * // except those listed
 * static const SIGSELECT allButTheseSigs[] = {(SIGSELECT)-4, MY_NEW_SIG,
 *        MY_LOST_SIG, MY_HUNT_SIG, MY_ATTACH_SIG};
 *
 */
union SIGNAL *
receive(const SIGSELECT *sigsel);

/**
 * @function receive_from
 *
 * @include "ose.h"
 *
 * @declaration union SIGNAL *
 * receive_from(OSTIME timeout, const SIGSELECT *sel, PROCESS from);
 *
 * @brief Receive a signal from a specific sender.
 *
 * @long
 * Works like receive_w_tmo, but signals are only accepted from a specific
 * process.
 * @long
 * This call is used, for instance, in interface libraries which hide
 * signal transactions towards a server process from a user. In this case
 * it must be possible to define signals to the server without conflicts
 * with user signal numbers.
 * @long
 * The receive_from call should be used with some caution since extensive
 * use can easily cause a messy system design and lead to debugging
 * problems.
 *
 * @input timeout The number of milliseconds to wait. 
 * With OSE5.5.3 and later versions, OSE_NO_TIMEOUT can be used to inhibit
 * return until a wanted signal has arrived.
 * @input sel Pointer to an array of the signal numbers to receive.
 * See @link "" "receive" for the signal array format.
 * @input from The ID of the process to receive from.
 *
 * @return Returns a pointer to the received signal buffer or the NIL
 * pointer if no signal arrived within the specified timeout.
 *
 * @seealso alloc
 * @seealso free_buf
 * @seealso addressee
 * @seealso send
 * @seealso send_w_s
 * @seealso receive
 * @seealso receive_w_tmo
 * @seealso OSE_NO_TIMEOUT
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * // The declarations of signal numbers and structs are
 * // preferably made in .sig-files which are then #included.
 * #define FIRST 1
 * struct First
 * {
 *   SIGSELECT sig_no;
 * };
 * #define THIRD 3
 * struct Third
 * {
 *   SIGSELECT sig_no;
 * };
 *
 *
 * // A union SIGNAL declaration must include a
 * // signal number and the signal structures of
 * // the signals that will be used by the program.
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 *   struct First first;
 *   struct Third third;
 * };
 *
 * static const SIGSELECT any_sig[] = {0};
 * static const SIGSELECT sig_one[] = {1, FIRST};
 * extern PROCESS sending_proc_;
 * extern PROCESS sending_proc2_;
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *
 *   sig = receive_from(10, any_sig, sending_proc_);
 *   if(sig != NIL)
 *     {
 *       if(sig->sig_no == THIRD)
 *         {
 *           free_buf(&sig);
 *           sig = receive_from(55, sig_one, sending_proc2_);
 *           if(sig != NIL)
 *             {
 *               // Code //
 *               free_buf(&sig);
 *             }
 *         }
 *       else
 *         {
 *           // Code //
 *           free_buf(&sig);
 *         }
 *     }
 *   for(;;)
 *     {
 *       // Code //
 *     }
 * }
 */
union SIGNAL *
receive_from(OSTIME timeout, const SIGSELECT *sel, PROCESS from);

/**
 * @function receive_w_tmo
 *
 * @include "ose.h"
 *
 * @declaration union SIGNAL *
 * receive_w_tmo(OSTIME timeout, const SIGSELECT *sel);
 *
 * @brief Receive with timeout.
 *
 * @long
 * Works like receive, except that the caller is suspended no longer than
 * the number of milliseconds specified in the timeout parameter.
 * If the requested signal has not arrived by then, the NIL pointer is
 * returned.
 * @long
 * The specified timeout period is converted to a number of actual system
 * ticks according to the following rules:
 * @long
 * - A value of zero returns immediately.
 * @long
 * - A value equal to or smaller than one system tick period returns at the
 * next system timer event, i.e. after anything between 0 and 1 system tick.
 * @long
 * - A value higher than one system tick period is rounded upwards to the
 * closest number of system ticks. The actual delay may vary between the
 * calculated number of ticks and this value less than one tick period.
 * @long
 * The result is that the actual delay may be up to one system tick period
 * shorter than specified.
 *
 * @input timeout The number of milliseconds to wait.
 * With OSE5.5.3 and later versions, OSE_NO_TIMEOUT can be used to inhibit
 * return until a wanted signal has arrived.
 * @input sel A pointer to an array of the signal numbers to receive.
 * See @link "" "receive" for the signal array format.
 *
 * @return Returns a pointer to the received signal buffer or the NIL
 * pointer if no signal arrived within the specified timeout.
 *
 * @seealso alloc
 * @seealso addressee
 * @seealso free_buf
 * @seealso send
 * @seealso sender
 * @seealso send_w_s
 * @seealso receive
 * @seealso receive_from
 * @seealso OSE_NO_TIMEOUT
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * static const SIGSELECT any_sig[] = {0};
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *
 *   for(;;)
 *     {
 *       //Wait for any signal in 10 ms time //
 *       sig = receive_w_tmo((OSTIME) 10, any_sig);
 *
 *       // Check if a signal was received. If no signal was
 *       // received in 10 ms time the receive_w_tmo()
 *       // call will return NIL.
 *       if (sig != NIL)
 *         {
 *           // Code //
 *           free_buf(&sig);
 *         }
 *       else
 *         {
 *           // Code //
 *         }
 *     }
 * }
 */
union SIGNAL *
receive_w_tmo(OSTIME timeout, const SIGSELECT *sel);

/**
 * @function receive_fsem
 * @include "ose.h"
 *
 * @declaration union SIGNAL *
 * receive_fsem(OSTIME timeout, const SIGSELECT *sigsel, OSFSEMVAL count);
 *
 * @brief Waits for either a signal, a fast semaphore, or a timeout.
 *
 * @long
 * The call receive_fsem combines wait_fsem with receive_w_tmo. The
 * return value indicates which type of event caused the call to
 * return. It is also possible to avoid timeouts completely, by
 * passing -1 as the time to wait. This makes the call more similar to
 * the regular receive system call.
 * @long
 * The fast semaphore associated with the calling process is decreased
 * by count on entry to receive_fsem. The call will return immediately
 * if the fast semaphore is still positive, i.e. if the fast semaphore
 * had been signaled at least count times before the call to
 * receive_fsem. The call can return because of fast semaphore
 * signaling as soon as the fast semaphore is positive.
 * @long
 * If the call returns because of another event than the fast
 * semaphore, the fast semaphore is increased by count before the call
 * returns. This ensures that no fast semaphore event is missed even
 * if there are several simultaneous events.
 * @long
 * Receive_fsem will receive signals in exactly the same manner as
 * receive. All signals will remain in the signal queue if a return is
 * caused by a fast semaphore signaling or a timeout.
 * @long
 * Several events may appear to occur simultaneously, e.g. if a wanted
 * signal is sent to the process and the fast semaphore is signaled
 * during execution of higher or equal priority processes. It can not
 * be known which of several simultaneous events will cause
 * receive_fsem to return. The following is however guaranteed:
 * @long
 *  - When return is caused by a signal, the first wanted signal in
 * the signal queue will be returned.
 * @long
 *  - The return value will not indicate a timeout if a signal arrives
 * or the fast semaphore is signaled within the specified time.
 * @long
 *  - If the fast semaphore has been signaled enough times before the
 * entry to the call, the return value will be OS_RCV_FSEM.
 *
 * @input timeout The number of milliseconds to wait. Specify OSE_NO_TIMEOUT
 * to wait forever.
 * @input sigsel A pointer to an array of the signal numbers to receive.
 * See @link "" "receive" for the signal array format.
 * @input count The number to decrease the fast semaphore with.
 *
 * @return Returns a pointer to the received signal buffer or the NIL
 * pointer if no signal arrived within the specified timeout. If the
 * fast semaphore was signaled, OS_RCV_FSEM is returned.
 *
 * @seealso alloc
 * @seealso free_buf
 * @seealso send
 * @seealso receive
 * @seealso receive_w_tmo
 * @seealso OS_RCV_FSEM
 * @seealso NIL
 * @seealso OSE_NO_TIMEOUT
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * static const SIGSELECT any_sig[] = {0};
 *
 * union SIGNAL
 * {
 *    SIGSELECT sig_no;
 * };
 *
 * OS_PROCESS(my_process)
 * {
 *    union SIGNAL *sig;
 *
 *    for(;;)
 *    {
 *       //Wait for any signal or fast semaphore in 10 ms time //
 *       sig = receive_fsem((OSTIME) 10, any_sig, 1);
 *
 *       // Determine what caused the call to return //
 *       if (sig == NIL)
 *       {
 *          // Timeout has occurred //
 *       }
 *       else if (sig == OS_RCV_FSEM)
 *       {
 *          // Semaphore became non-negative //
 *       }
 *       else
 *       {
 *          // A signal was received //
 *          free_buf(&sig);
 *       }
 *    }
 * }
 */
union SIGNAL *
receive_fsem(OSTIME timeout, const SIGSELECT *sigsel, OSFSEMVAL count);

#ifndef softOSE
/**
 * @function send
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * send(union SIGNAL **sig, PROCESS to);
 *
 * @brief Send a signal.
 *
 * @long Sends a signal buffer to a specified addressee process and enters
 * NIL into the caller's signal pointer to avoid accidental reuse of the
 * buffer.
 * @long
 * The calling process is no longer the owner of the signal.
 * @long
 * Signals are redirected if the addressee process has a redirection table
 * that matches the sent signal. (The signal is not redirected if the
 * redirection table chain is infinite or indicates that the signal should
 * be sent to the caller.)
 * @long
 * If the addressee process has terminated, the signal is silently freed.
 * The caller will never know.
 *
 * @input sig A pointer to the signal buffer.
 * @input to The ID of the process the signal will be sent to.
 *
 * @restriction Also available in interrupt and timer interrupt processes.
 *
 * @seealso send_w_s
 * @seealso receive
 * @seealso receive_w_tmo
 * @seealso receive_from
 *
 * @seealso default_restrictions_calling_functions
 * @example
 * #include "ose.h"
 *
 * // The declarations of signal numbers and structs are
 * // preferably made in .sig-files which are then #included.
 * #define FORTH 4
 * struct Forth
 * {
 *   SIGSELECT sig_no;
 *   char data;
 * };
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 *   struct Forth forth;
 * };
 *
 * extern PROCESS proc_;
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *
 *   for(;;)
 *     {
 *       sig = alloc(sizeof(struct Forth), FORTH);
 *       sig->forth.data ='x';
 *       send(&sig,proc_);
 *       // Code //
 *     }
 * }
 */
void
send(union SIGNAL **sig, PROCESS to);
#endif

/**
 * @function sender
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration PROCESS
 * sender(union SIGNAL **sig);
 *
 * @brief Get the sender of a signal.
 *
 * @long Finds the process which last sent a specified signal buffer.
 *
 * @input sig A pointer to the signal buffer.
 *
 * @return Returns the process ID of the sender or the owner's ID if the
 * signal was never sent.
 * @break
 * The sender is not affected by signal redirection, i.e. the sender is
 * always the process which called send or the process ID that was
 * specified in send_w_s.
 *
 * @restriction Also available in interrupt and timer interrupt processes.
 *
 * @seealso addressee
 * @seealso restore
 * @seealso send
 * @seealso send_w_s
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * static const SIGSELECT any_sig[] = {0};
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *
 *   for(;;)
 *     {
 *       sig = receive(any_sig);
 *       // Code //
 *       send(&sig, sender(&sig));
 *     }
 * }
 */
PROCESS
sender(union SIGNAL **sig);

/**
 * @function send_w_s
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * send_w_s(union SIGNAL **sig, PROCESS from, PROCESS to);
 *
 * @brief Send a signal using an alias.
 *
 * @long Short for send with sender. Works like send, except that the
 * signal buffer is tagged with the specified 'from' process ID in place of 
 * the caller's.
 * @long
 * If the addressee process or the specified sender process has terminated,
 * the signal is silently freed. The caller will never know.
 *
 * @input sig A pointer to the signal buffer.
 * @input from The ID of the process specified as sender.
 * @input to The ID of the process the signal will be sent to.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * In a multicore environment, there is no way of knowing if the signal was
 * dropped because the 'from' process is killed. Use @link "" "send_w_sender"
 * instead.
 *
 * @seealso send_w_sender
 * @seealso receive
 * @seealso receive_w_tmo
 * @seealso receive_from
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * // The declarations of signal numbers and structs are
 * // preferably made in .sig-files which are then #included.
 * #define FIFTH 5
 * struct fifth
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * extern PROCESS proc1_;
 * extern PROCESS proc2_;
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *
 *   for(;;)
 *     {
 *       sig = alloc(sizeof(struct fifth), FIFTH);
 *
 *       // send a signal to proc2 specifying proc1 as sender//
 *       send_w_s(&sig, proc1_, proc2_);
 *       // Code //
 *     }
 * }
 */
void
send_w_s(union SIGNAL **sig, PROCESS from, PROCESS to);

/**
 * @function set_pri
 *
 * @include "ose.h"
 *
 * @declaration OSPRIORITY
 * set_pri(OSPRIORITY newpri);
 *
 * @brief Set a new priority level for the calling process.
 *
 * @long
 * The calling process is moved to the tail of the ready list on the new
 * priority, which means that the process is swapped out until all other
 * processes with that priority have had an opportunity to execute.
 * @long
 * The newpri parameter may be any priority level allowed in the
 * create_process system call.
 *
 * @input newpri The priority level to set.
 *
 * @return Returns the resulting priority. This is the old priority if the
 * call failed.
 *
 * @restriction Only available in prioritized processes.
 *
 * @seealso create_process
 * @seealso get_pri
 * @seealso set_pri_for
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   OSPRIORITY org_prio;
 *   OSPRIORITY prio;
 *
 *   for(;;)
 *     {
 *       org_prio = get_pri(current_process());
 *       prio = set_pri(31);
 *       if(prio == org_prio && org_prio != 31)
 *         {
 *           // Code, priority could not be set //
 *         }
 *       else
 *         {
 *           // Code //
 *           // Reset the original priority //
 *           prio = set_pri(org_prio);
 *         }
 *     }
 * }
 */
OSPRIORITY
set_pri(OSPRIORITY newpri);

/**
 * @function set_pri_for
 *
 * @include "ose.h"
 *
 * @declaration OSPRIORITY
 * set_pri_for(PROCESS pid, OSPRIORITY newpri);
 *
 * @brief Set a new priority level for the specified process.
 *
 * @long
 * The specified process is moved to the tail of the ready list on the new
 * priority, which means that the process is swapped out until all other
 * processes with that priority have had an opportunity to execute.
 * @long
 * The newpri parameter may be any priority level allowed in the
 * create_process system call.
 *
 * @input pid The ID of the specified process.
 * @input newpri The priority level to set.
 *
 * @return Returns the resulting priority. This is the old priority if the
 * call failed.
 *
 * @restriction
 * The target process specified with the 'pid' parameter must be a
 * prioritized process.
 * @break
 * Some implementations place restrictions on how priority levels may be
 * used.
 *
 * @seealso create_process
 * @seealso get_pri
 * @seealso set_pri
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   OSPRIORITY org_prio;
 *   OSPRIORITY prio;
 *
 *   for(;;)
 *     {
 *       org_prio = get_pri(current_process());
 *       prio = set_pri_for(current_process(), 31);
 *       if(prio == org_prio)
 *         {
 *           // Code, priority could not be set //
 *         }
 *       else
 *         {
 *           // Code //
 *           // Reset the original priority //
 *           prio = set_pri(org_prio);
 *         }
 *     }
 * }
 */
OSPRIORITY
set_pri_for(PROCESS pid, OSPRIORITY newpri);

/**
 * @function sigsize
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration OSBUFSIZE
 * sigsize(union SIGNAL **sig);
 *
 * @brief Get signal size.
 *
 * @long Examines a signal buffer and reports the size that was requested
 * when the buffer was allocated.
 *
 * @input sig A pointer to a pointer to a signal buffer.
 *
 * @return Returns the number of bytes requested when the buffer was
 * allocated.
 *
 * @restriction Also available in interrupt and timer interrupt processes.
 *
 * @seealso alloc
 * @seealso set_sigsize
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * static const SIGSELECT any_sig[] = {0};
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * OS_PROCESS(my_process)
 * {
 *   union SIGNAL *sig;
 *   union SIGNAL *sig2;
 *   OSBUFSIZE size;
 *
 *   for(;;)
 *     {
 *       sig = receive(any_sig);
 *       size = sigsize(&sig);
 *       sig2 = alloc(size,sig->sig_no);
 *       // Code //
 *     }
 * }
 */
OSBUFSIZE
sigsize(union SIGNAL **sig);

/**
 * @function signal_fsem
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * signal_fsem(PROCESS pid);
 *
 * @brief Signal the fast semaphore.
 *
 * @long Increments the value of the fast semaphore associated with the
 * specified process.
 * @long
 * If the result is zero, the specified process is made ready if it is
 * waiting inside a wait_fsem call.
 * @long
 * If pid refers to an interrupt process, it is scheduled and the
 * fast semaphore value is left untouched.
 *
 * @input pid The ID of the process for which the fast semaphore is
 * signaled.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * It is illegal to cause a fast semaphore to wrap from high positive
 * numbers to a negative value.
 *
 * @seealso set_fsem
 * @seealso wait_fsem
 * @seealso get_fsem
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * extern PROCESS proc_;
 *
 * OS_PROCESS(my_process)
 * {
 *   for(;;)
 *     {
 *       // Code //
 *       signal_fsem(proc_);
 *       // Code //
 *     }
 * }
 */
void
signal_fsem(PROCESS pid);

/**
 * @function signal_sem
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * signal_sem(SEMAPHORE *sem);
 *
 * @brief Signal a semaphore.
 *
 * @long Increments the value of the specified semaphore and releases the
 * first process waiting at the semaphore.
 *
 * @input sem A pointer to the semaphore to be signaled.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * It is illegal to cause a semaphore to wrap from a positive to a negative
 * value.
 *
 * @seealso wait_sem
 * @seealso get_sem
 * @seealso create_sem
 * @seealso kill_sem
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * extern SEMAPHORE *sem;
 *
 * OS_PROCESS(my_process)
 * {
 *   for(;;)
 *     {
 *       // Code //
 *       signal_sem(sem);
 *       // Code //
 *     }
 * }
 */
void
signal_sem(SEMAPHORE *sem);

/**
 * @function start
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * start(PROCESS id);
 *
 * @brief Start a block or process.
 *
 * @long
 * Each new process (or block) must be started to enable execution. 
 * The start function is also used to let a process (or block) resume
 * execution after it has been stopped by a stop call.
 * @long
 * The kernel keeps a record of how many times each process has been
 * stopped. A process is allowed to execute again only when the same number
 * of start calls have been issued.
 * @long
 * Starting a block is equivalent to issuing a start call to each of the
 * processes contained in the block. It is important that no process is
 * started individually before the block is started, since starting a
 * process twice is illegal.
 * @long
 * Starting a timer-interrupt process means that it from now on will be
 * scheduled to run at its specified time interval.
 *
 * @input id The ID of the block or process to start.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * Only prioritized, background and timer-interrupt processes can be
 * stopped. The start and stop calls have no effect on process types that
 * can not be stopped.
 * @break
 * Starting a process that is not in stopped state is illegal. Except for
 * the initial start, this implies that it is illegal to start a stopped
 * process more times than it was stopped.
 * @break
 * This implies that it is always illegal to start the current process or
 * the current block.
 *
 * @error OSE_EALREADY_STARTED Non-fatal error if start is called for
 * an already started process (or block).
 *
 * @seealso stop
 * @seealso create_process
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "stddef.h"
 * #include "ose.h"
 *
 * OSENTRYPOINT new_process;
 *
 * OS_PROCESS(my_process)
 * {
 *   PROCESS proc1_;
 *   PROCESS proc2_;
 *   PROCESS block_;
 *
 *   for(;;)
 *     {
 *       proc1_ = create_process(OS_PRI_PROC,
 *                               "proc1",
 *                               new_process,
 *                               200,
 *                               16,
 *                               (OSTIME) 0,
 *                               (PROCESS) 0,
 *                               (struct OS_redir_entry *) NULL,
 *                               (OSVECTOR) 0,
 *                               (OSUSER) 0);
 *
 *       start(proc1_);
 *
 *       block_ = create_block("block",
 *                             (OSUSER) 0,
 *                             (OSBOOLEAN) 0,
 *                             (PROCESS) 0,
 *                             (OSBOOLEAN) 0);
 *
 *       proc2_ = create_process(OS_PRI_PROC,
 *                               "proc2",
 *                               new_process,
 *                               200,
 *                               16,
 *                               (OSTIME) 0,
 *                               block_,
 *                               (struct OS_redir_entry *) NULL,
 *                               (OSVECTOR) 0,
 *                               (OSUSER) 0);
 *
 *       start(block_);
 *       // Code //
 *     }
 * }
 */
void
start(PROCESS id);

/**
 * @function stop
 *
 * @include "ose.h"
 * @include "ose_i.h"
 *
 * @declaration void
 * stop(PROCESS id);
 *
 * @brief Suspend a process or block.
 *
 * @long Suspend the execution of a single process or all processes in a
 * block.
 * @long
 * It is legal to stop a process any (reasonable) number of times.
 * @long
 * The kernel keeps a record of how many times each process has been
 * stopped. A process is allowed to execute again only when the same number
 * of start calls have been issued.
 * @long
 * Stopping a block is equivalent to issuing a stop call to each of the
 * processes contained in the block.
 * @long
 * Stopping a timer-interrupt process means that it will no longer be
 * scheduled to run at its specified time interval.
 *
 * @input id The ID of the block or process to stop.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * Only prioritized, background and timer-interrupt processes can be
 * stopped. The start and stop calls have no effect on process types that
 * can not be stopped.
 * @break
 * Note that a timer-interrupt process will continue to execute until its
 * end, even after issuing a stop(current_process()), but it will no longer
 * be scheduled to run at its specified time interval.
 *
 * @seealso start
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * extern PROCESS proc_;
 *
 * OS_PROCESS(my_process)
 * {
 *   for(;;)
 *     {
 *       stop(proc_);
 *       // Code //
 *       start(proc_);
 *     }
 * }
 */
void
stop(PROCESS id);

/**
 * @function wait_fsem
 *
 * @include "ose.h"
 *
 * @declaration void
 * wait_fsem(OSFSEMVAL count);
 *
 * @brief Wait at the fast semaphore.
 *
 * @long Wait at the fast semaphore associated with the caller.
 * @long
 * The semaphore value is decreased by the specified amount. If the result
 * is negative, the caller is suspended until some other process has issued
 * enough signal_fsem calls for the value to become non-negative.
 *
 * @input count The number to decrease the fast semaphore with.
 *
 * @restriction It is illegal to cause the semaphore value to wrap, as this
 * might cause unexpected behavior.
 *
 * @seealso signal_fsem
 * @seealso get_fsem
 * @seealso set_fsem
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * OS_PROCESS(my_process)
 * {
 *   for(;;)
 *     {
 *       // Code //
 *       wait_fsem((OSFSEMVAL) 1);
 *       // Code //
 *     }
 * }
 */
void
wait_fsem(OSFSEMVAL count);

/**
 * @function wait_sem
 *
 * @include "ose.h"
 *
 * @declaration void
 * wait_sem(SEMAPHORE *sem);
 *
 * @brief Wait at the specified semaphore.
 *
 * @long
 * The semaphore value is decremented. If the result is negative, the
 * caller is suspended and queued at the end of the semaphore queue.
 * @long
 * Each time some other process signals the semaphore, the value is
 * incremented and the first process queued at the semaphore is released.
 *
 * @input sem A pointer to the semaphore.
 *
 * @seealso signal_sem
 * @seealso get_sem
 * @seealso create_sem
 * @seealso kill_sem
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * SEMAPHORE *sem;
 *
 * OS_PROCESS(my_process)
 * {
 *   for(;;)
 *     {
 *       // Code //
 *       wait_sem(sem);
 *       // Code //
 *     }
 * }
 */
void
wait_sem(SEMAPHORE *sem);

#endif /* defined(__cplusplus) && defined(OS_DEBUG) */

/*
** Debug system calls.
*/

#ifdef ZZFILE_AT_HEAD
#define ZZPRE_PROTO		const char *file, long line,
#define ZZPOST_PROTO
#define ZZPRE_ARGS		__FILE__,__LINE__,
#define ZZPRE_CPP_ARGS(n)	#n "@ose.h", __LINE__,
#define ZZPOST_ARGS
#define ZZPOST_CPP_ARGS(n)
#else
#define ZZPRE_PROTO
#define ZZPOST_PROTO		,const char *file, long line
#define ZZPRE_ARGS
#define ZZPRE_CPP_ARGS(n)
#define ZZPOST_ARGS		,__FILE__,__LINE__
#define ZZPOST_CPP_ARGS(n)	, #n "@ose.h", __LINE__
#endif
#define ZZMIDDLE_PROTO          const char *file, long line
#define ZZMIDDLE_ARGS		__FILE__,__LINE__
#define ZZMIDDLE_CPP_ARGS(n)	#n "@ose.h", __LINE__

PROCESS
zzaddressee(ZZPRE_PROTO union SIGNAL **sig ZZPOST_PROTO);

OSATTREF
zzattach(ZZPRE_PROTO union SIGNAL **sig, PROCESS id ZZPOST_PROTO);

PROCESS
zzcreate_block(ZZPRE_PROTO const char *name, OSUSER user,
	       OSBOOLEAN use_remote_calls,
	       PROCESS remote_call_server, OSBOOLEAN supervisor_mode
	       ZZPOST_PROTO);

PROCESS
zzcreate_process(ZZPRE_PROTO enum PROCESS_TYPE proc_type, const char *name,
		 OSENTRYPOINT *entrypoint, OSADDRESS stack_size,
		 OSPRIORITY priority, OSTIME timeslice, PROCESS block,
		 struct OS_redir_entry *router_table, OSVECTOR vec,
		 OSUSER user ZZPOST_PROTO);

void
zzdelay(ZZPRE_PROTO OSTIME timeout ZZPOST_PROTO);

void
zzdetach(ZZPRE_PROTO OSATTREF *sigref ZZPOST_PROTO);

void
zzerror(ZZPRE_PROTO OSERRCODE ecode ZZPOST_PROTO);

void
zzerror2(ZZPRE_PROTO OSERRCODE ecode, OSERRCODE extra ZZPOST_PROTO);

void
zzfree_buf(ZZPRE_PROTO union SIGNAL **sig ZZPOST_PROTO);

OSBOOLEAN
zzhunt(ZZPRE_PROTO const char *name, OSUSER user, PROCESS *name_,
       union SIGNAL **hunt_sig ZZPOST_PROTO);

void
zzkill_proc(ZZPRE_PROTO PROCESS id ZZPOST_PROTO);

union SIGNAL *
zzreceive(ZZPRE_PROTO const SIGSELECT *sigsel ZZPOST_PROTO);

union SIGNAL *
zzreceive_from(ZZPRE_PROTO OSTIME timeout, const SIGSELECT *sel,
	       PROCESS from ZZPOST_PROTO);

union SIGNAL *
zzreceive_w_tmo(ZZPRE_PROTO OSTIME timeout, const SIGSELECT *sel
		ZZPOST_PROTO);

void
zzsend(ZZPRE_PROTO union SIGNAL **sig, PROCESS to ZZPOST_PROTO);

PROCESS
zzsender(ZZPRE_PROTO union SIGNAL **sig ZZPOST_PROTO);

void
zzsend_w_s(ZZPRE_PROTO union SIGNAL **sig, PROCESS from,
	   PROCESS to ZZPOST_PROTO);

OSPRIORITY
zzset_pri(ZZPRE_PROTO OSPRIORITY newpri ZZPOST_PROTO);

OSPRIORITY
zzset_pri_for(ZZPRE_PROTO PROCESS pid, OSPRIORITY newpri
	      ZZPOST_PROTO);

OSBUFSIZE
zzsigsize(ZZPRE_PROTO union SIGNAL **sig ZZPOST_PROTO);

void
zzsignal_fsem(ZZPRE_PROTO PROCESS pid ZZPOST_PROTO);

void
zzsignal_sem(ZZPRE_PROTO SEMAPHORE *sem ZZPOST_PROTO);

void
zzstart(ZZPRE_PROTO PROCESS id ZZPOST_PROTO);

void
zzstop(ZZPRE_PROTO PROCESS id ZZPOST_PROTO);

void
zzwait_fsem(ZZPRE_PROTO OSFSEMVAL count ZZPOST_PROTO);

void
zzwait_sem(ZZPRE_PROTO SEMAPHORE *sem ZZPOST_PROTO);

#ifdef __cplusplus

} /* end of extern "C" */

#ifdef OS_DEBUG

static inline PROCESS
addressee(union SIGNAL **sig)
{
   return zzaddressee(ZZPRE_CPP_ARGS(addressee)
		      sig
		      ZZPOST_CPP_ARGS(addressee));
}

static inline OSATTREF
attach(union SIGNAL **sig, PROCESS id)
{
   return zzattach(ZZPRE_CPP_ARGS(attach)
		   sig, id
		   ZZPOST_CPP_ARGS(attach));
}

static inline PROCESS
create_block(const char *name, OSUSER user, OSBOOLEAN use_remote_calls,
	     PROCESS remote_call_server, OSBOOLEAN supervisor_mode)
{
   return zzcreate_block(ZZPRE_CPP_ARGS(create_block)
			 name, user, use_remote_calls,
			 remote_call_server, supervisor_mode
			 ZZPOST_CPP_ARGS(create_block));
}

static inline PROCESS
create_process(enum PROCESS_TYPE proc_type,const char *name,
	       OSENTRYPOINT *entrypoint, OSADDRESS stack_size,
	       OSPRIORITY priority, OSTIME timeslice, PROCESS block,
	       struct OS_redir_entry *router_table, OSVECTOR vec,
	       OSUSER user)
{
   return zzcreate_process(ZZPRE_CPP_ARGS(create_process)
			   proc_type, name, entrypoint, stack_size,
			   priority, timeslice, block, router_table,
			   vec, user
			   ZZPOST_CPP_ARGS(create_process));
}

static inline void
delay(OSTIME timeout)
{
   zzdelay(ZZPRE_CPP_ARGS(delay)
	   timeout
	   ZZPOST_CPP_ARGS(delay));
}

static inline void
detach(OSATTREF *sigref)
{
   zzdetach(ZZPRE_CPP_ARGS(detach)
	    sigref
	    ZZPOST_CPP_ARGS(detach));
}

static inline void
error(OSERRCODE ecode)
{
   zzerror(ZZPRE_CPP_ARGS(error)
	   ecode
	   ZZPOST_CPP_ARGS(error));
}

static inline void
error2(OSERRCODE ecode, OSERRCODE extra)
{
   zzerror2(ZZPRE_CPP_ARGS(error2)
	    ecode, extra
	    ZZPOST_CPP_ARGS(error2));
}

static inline void
free_buf(union SIGNAL **sig)
{
   zzfree_buf(ZZPRE_CPP_ARGS(free_buf)
	      sig
	      ZZPOST_CPP_ARGS(free_buf));
}

static inline OSBOOLEAN
hunt(const char *name, OSUSER user, PROCESS *name_, union SIGNAL **hunt_sig)
{
   return zzhunt(ZZPRE_CPP_ARGS(hunt)
		 name, user, name_, hunt_sig
		 ZZPOST_CPP_ARGS(hunt));
}

static inline void
kill_proc(PROCESS id)
{
   zzkill_proc(ZZPRE_CPP_ARGS(kill_proc)
	       id
	       ZZPOST_CPP_ARGS(kill_proc));
}

static inline union SIGNAL *
receive(const SIGSELECT *sigsel)
{
   return zzreceive(ZZPRE_CPP_ARGS(receive)
		    sigsel
		    ZZPOST_CPP_ARGS(receive));
}

static inline union SIGNAL *
receive_from(OSTIME timeout, const SIGSELECT *sel, PROCESS from)
{
   return zzreceive_from(ZZPRE_CPP_ARGS(receive_from)
			 timeout, sel, from
			 ZZPOST_CPP_ARGS(receive_from));
}

static inline union SIGNAL *
receive_w_tmo(OSTIME timeout, const SIGSELECT *sel)
{
   return zzreceive_w_tmo(ZZPRE_CPP_ARGS(receive_w_tmo)
			  timeout, sel
			  ZZPOST_CPP_ARGS(receive_w_tmo));
}

static inline void
send(union SIGNAL **sig, PROCESS to)
{
   zzsend(ZZPRE_CPP_ARGS(send)
	  sig, to
	  ZZPOST_CPP_ARGS(send));
}

static inline PROCESS
sender(union SIGNAL **sig)
{
   return zzsender(ZZPRE_CPP_ARGS(sender)
		   sig
		   ZZPOST_CPP_ARGS(sender));
}

static inline void
send_w_s(union SIGNAL **sig, PROCESS from, PROCESS to)
{
   zzsend_w_s(ZZPRE_CPP_ARGS(send_w_s)
	      sig, from, to
	      ZZPOST_CPP_ARGS(send_w_s));
}

static inline OSPRIORITY
set_pri(OSPRIORITY newpri)
{
   return zzset_pri(ZZPRE_CPP_ARGS(set_pri)
		    newpri
		    ZZPOST_CPP_ARGS(set_pri));
}

static inline OSPRIORITY
set_pri_for(PROCESS pid, OSPRIORITY newpri)
{
   return zzset_pri_for(ZZPRE_CPP_ARGS(set_pri_for)
			pid, newpri
			ZZPOST_CPP_ARGS(set_pri_for));
}

static inline OSBUFSIZE
sigsize(union SIGNAL **sig)
{
   return zzsigsize(ZZPRE_CPP_ARGS(sigsize)
		    sig
		    ZZPOST_CPP_ARGS(sigsize));
}

static inline void
signal_fsem(PROCESS pid)
{
   zzsignal_fsem(ZZPRE_CPP_ARGS(signal_fsem)
		 pid
		 ZZPOST_CPP_ARGS(signal_fsem));
}

static inline void
signal_sem(SEMAPHORE *sem)
{
   zzsignal_sem(ZZPRE_CPP_ARGS(signal_sem)
		sem
		ZZPOST_CPP_ARGS(signal_sem));
}

static inline void
start(PROCESS id)
{
   zzstart(ZZPRE_CPP_ARGS(start)
	   id
	   ZZPOST_CPP_ARGS(start));
}

static inline void
stop(PROCESS id)
{
   zzstop(ZZPRE_CPP_ARGS(stop)
	  id
	  ZZPOST_CPP_ARGS(stop));
}

static inline void
wait_fsem(OSFSEMVAL count)
{
   zzwait_fsem(ZZPRE_CPP_ARGS(wait_fsem)
	       count
	       ZZPOST_CPP_ARGS(wait_fsem));
}

static inline void
wait_sem(SEMAPHORE *sem)
{
   zzwait_sem(ZZPRE_CPP_ARGS(wait_sem)
	      sem
	      ZZPOST_CPP_ARGS(wait_sem));
}

#endif /* OS_DEBUG */

extern "C" { /* end of C++ */

#else /* __cplusplus */
#if !defined(ZZNO_FILE_LINE_INFO_IN_ERROR)

#define error(ecode) zzerror(ZZPRE_ARGS ecode ZZPOST_ARGS)

#define error2(ecode,extra) zzerror2(ZZPRE_ARGS ecode,extra ZZPOST_ARGS)

#endif
#ifdef OS_DEBUG

#define addressee(sig) zzaddressee(ZZPRE_ARGS sig ZZPOST_ARGS)

#define attach(sig,id) zzattach(ZZPRE_ARGS sig,id ZZPOST_ARGS)

#define create_block(name,user,use_remote_calls,remote_call_server,\
		     supervisor_mode) \
	zzcreate_block(ZZPRE_ARGS name,user,\
		       use_remote_calls,remote_call_server,\
		       supervisor_mode ZZPOST_ARGS)

#define create_process(proc_type,name,entrypoint,stack_size,priority,\
		       timeslice,block,router_table,vec,user) \
	zzcreate_process(ZZPRE_ARGS proc_type,name,entrypoint,\
			 stack_size,priority,timeslice,block,router_table,\
			 vec,user ZZPOST_ARGS)

#define delay(timeout) zzdelay(ZZPRE_ARGS timeout ZZPOST_ARGS)

#define detach(id) zzdetach(ZZPRE_ARGS id ZZPOST_ARGS)

#define free_buf(sig) zzfree_buf(ZZPRE_ARGS sig ZZPOST_ARGS)

#define hunt(name,user,name_,hunt_sig)\
	zzhunt(ZZPRE_ARGS name,user,name_,hunt_sig ZZPOST_ARGS)

#define kill_proc(id) zzkill_proc(ZZPRE_ARGS id ZZPOST_ARGS)

#define receive(sigsel) zzreceive(ZZPRE_ARGS sigsel ZZPOST_ARGS)

#define receive_from(timeout,sel,from)\
	zzreceive_from(ZZPRE_ARGS timeout,sel,from ZZPOST_ARGS)

#define receive_w_tmo(timeout,sel)\
	zzreceive_w_tmo(ZZPRE_ARGS timeout,sel ZZPOST_ARGS)

#define send(sig,to) zzsend(ZZPRE_ARGS sig,to ZZPOST_ARGS)

#define sender(sig) zzsender(ZZPRE_ARGS sig ZZPOST_ARGS)

#define send_w_s(sig,from,to) zzsend_w_s(ZZPRE_ARGS sig,from,to ZZPOST_ARGS)

#define set_pri(newpri) zzset_pri(ZZPRE_ARGS newpri ZZPOST_ARGS)

#define set_pri_for(pid,newpri) zzset_pri_for(ZZPRE_ARGS pid,newpri \
					      ZZPOST_ARGS)

#define sigsize(sig) zzsigsize(ZZPRE_ARGS sig ZZPOST_ARGS)

#define signal_fsem(id) zzsignal_fsem(ZZPRE_ARGS id ZZPOST_ARGS)

#define signal_sem(sem) zzsignal_sem(ZZPRE_ARGS sem ZZPOST_ARGS)

#define start(id) zzstart(ZZPRE_ARGS id ZZPOST_ARGS)

#define stop(id) zzstop(ZZPRE_ARGS id ZZPOST_ARGS)

#define wait_fsem(count) zzwait_fsem(ZZPRE_ARGS count ZZPOST_ARGS)

#define wait_sem(sem) zzwait_sem(ZZPRE_ARGS sem ZZPOST_ARGS)

#endif /* OS_DEBUG */

#endif /* __cplusplus */

#if defined(softOSE) && !defined(OS_DEBUG) && !defined(NO_ZZZSEND)

#ifdef __cplusplus

} /* end of extern "C" */

static inline void
send(union SIGNAL **sig, PROCESS to)
{
   zzzsend(sig, to);
}

extern "C" { /* end of C++ */

#else
#define send(sig, to) zzzsend(sig, to)
#endif /* __cplusplus */

#endif /* defined(softOSE) && !defined(OS_DEBUG) && !defined(NO_ZZZSEND) */

/*
** The system calls below are considered "processor architecture dependent".
** This means that they are subject to a lower degree of portability than
** other system calls as they are present only in some OSE implementations.
** New system calls that have not yet been incorporated in the generic OSE
** specification are also treated as "processor architecture dependent"
** until it has been decided that they should be globally adopted.
*/

/*@-PRIVATE-CALLS-@*/


/**
 * @function receive_sport
 *
 * @include "ose.h"
 *
 * @declaration union SIGNAL *
 * receive_sport(OSTIME timeout, const SIGSELECT *sel, PROCESS *sport_list);
 *
 * @brief Receive from a signal port.
 *
 * @long
 * A signal port is basically a phantom process with an extra attribute set
 * that causes signals sent to the process to remain in the signal queue of
 * the phantom instead of being silently freed. Any process can send to a
 * signal port and any prioritized or background process can read from the
 * queue of a signal port.
 * @long
 * This system call works like receive_w_tmo() except that signals are
 * fetched from the signal queues of any of the processes in the
 * sport_list. The list may only contain process IDs of signal ports.
 * The first location in the sport_list array contains the number of entries
 * in the rest of the array. Since the PROCESS type may not be an integer type,
 * the count should be cast to the PROCESS type when building sport_list.
 * E.g.: sport_list[0] = (PROCESS) 1;
 * @long
 * A signal port is created with the create_process() system call, using a
 * special redirection table format. The normal table is preceded with an entry
 * containing redir.sig=0 and redir.pid=1. The redir.pid field contains the
 * attribute bits and bit 0 specifies the signal port behavior.
 *
 * @input timeout The number of milliseconds to wait.
 * With OSE5.5.3 and later versions, OSE_NO_TIMEOUT can be used to inhibit
 * return until a wanted signal has arrived.
 * @input sel A pointer to an array of the signal numbers to receive.
 * See @link "" "receive" for the signal array format.
 * @input sport_list A pointer to an array of sports to receive signals
 * from.
 *
 * @return Returns a pointer to the received signal buffer or the NIL
 * pointer if no signal arrived within the specified timeout.
 *
 * @restriction
 * The size of the sport_list is limited to
 * 32 entries.
 *
 * @seealso alloc
 * @seealso addressee
 * @seealso free_buf
 * @seealso send
 * @seealso sender
 * @seealso send_w_s
 * @seealso receive
 * @seealso receive_from
 * @seealso receive_w_tmo
 * @seealso create_process
 * @seealso OSE_NO_TIMEOUT
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include <stdio.h>
 * #include "ose.h"
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * OS_PROCESS(gummi)
 * {
 *   static const struct OS_redir_entry redir[2] =
 *     {
 *       {
 *         0, // 0 == Set signal attribute //
 *         1 // Attribute 1 is create signal port //
 *       },
 *       {
 *         1, // One entry in redirection table //
 *         0 // 0 makes the sport store all signals //
 *       }
 *     };
 *
 *   PROCESS sport_ = create_process(
 *                       OS_PHANTOM, // processtype //
 *                       "sport", // name //
 *                       NULL, // entrypoint //
 *                       0, // stacksize //
 *                       0, // priority //
 *                       0,0, // not used //
 *                       (struct OS_redir_entry *)&redir,
 *                       0,0); // not used //
 *
 *   for(;;)
 *     {
 *       static const SIGSELECT sig_sel[]={0};
 *       PROCESS sport_list[2];
 *       union SIGNAL *sig;
 *       sport_list[0] = 1;
 *       sport_list[1] = sport_;
 *       sig = receive_sport(1000,
 *                           sig_sel,
 *                           sport_list);
 *       if (sig != NIL)
 *         {
 *           printf("Got signal %lx\n", sig->sig_no);
 *           free_buf(&sig);
 *         }
 *       else
 *         {
 *           printf("Got no signal (timeout)\n");
 *         }
 *     }
 * }//gummi//
 */
union SIGNAL *
receive_sport(OSTIME timeout, const SIGSELECT *sel, PROCESS *sport_list);

/**
 * @function create_event
 *
 * @include "ose.h"
 *
 * @declaration void
 * create_event(SIGSELECT eventno, OSADDRESS *trig);
 *
 * @brief Register a trigger variable for an event number, for
 * subsequent checking before signal_event().
 *
 * @long The kernel will set the trigger variable to either 0 or 1
 * to discourage or encourage (respectively) calls to signal_event()
 * with the associated event number, depending on if any action have
 * been registered for this event number. This allows the application to
 * avoid calls to signal_event() for event numbers for which there is no
 * interest.
 *
 * @long Once a trigger variable is registered, the application should
 * not modify the value of the trigger variable.
 *
 * @long The trigger variable must exist as long as the application
 * exist. This usually means that it has to be static or of file
 * scope.
 *
 * @long It is legal to call signal_event() with an event number for
 * which create_event() has not been called. The application can thus
 * use a single create_event() call and trigger variable to decide
 * also to call signal_event() for other event numbers.
 *
 * @long Each definition of a user event number, for events carrying data,
 * in the header files should be followed on the line by a C comment
 * with the format
 * / *  !-EVTNO(struct xxxxxx)-! * /
 * to indicate the event data struct or void.
 * Use EVTNO(xxxxx) without "struct", if there is a typedef and the
 * struct does not have a name.
 * This definition can be decoded by the sigdb host tool for use
 * during debugging. Compare the SigDB tool description.
 *
 * @long Applications should use event numbers above OSE_EVENTS_ENEA_END.
 *
 * @input eventno The event number for which the trigger shall be registered.
 * @input trig    The address of a (non-local) trigger variable.
 *
 * @seealso signal_event
 * @seealso MonitorSetUserActionpointRequest
 * @seealso default_restrictions_calling_functions
 * @seealso OSE_EVENTS_ENEA_END
 *
 * @example
 * #include "ose.h"
 * #include "string.h"
 *
 * #define MY_FUNC_ENTER 30040    // !-EVTNO(struct MyFuncParams)-! //
 * #define MY_FUNC_EXIT  30041    // !-EVTNO(void)-! //
 *
 * struct MyFuncParams
 * {
 *    char str[1];
 * };
 *
 * // Initialize trigger //
 * static OSADDRESS my_func_trigger = ~0;
 *
 * static void my_function(const char *str)
 * {
 *    // Call create_event() on demand //
 *    if (my_func_trigger == ~0)
 *    {
 *       // Can also be done ahead of time (e.g. in a start hook) //
 *       create_event(MY_FUNC_ENTER, &my_func_trigger);
 *    }
 *    // Signal function entry if there is interest //
 *    if (my_func_trigger)
 *    {
 *       signal_event(MY_FUNC_ENTER, str, strlen(str) + 1)
 *    }
 *
 *    // CODE //
 *
 *    // Signal function exit (reusing trigger variable) //
 *    if (my_func_trigger)
 *    {
 *       signal_event(MY_FUNC_EXIT, NULL, 0);
 *    }
 * }
 */
void create_event(SIGSELECT eventno, OSADDRESS *trig);

/**
 * @function signal_event
 * @include "ose.h"
 *
 * @declaration void
 * signal_event(SIGSELECT event_id, void *data, OSBUFSIZE size);
 *
 * @brief Issue an application specific user event.
 *
 * @long Applications can use this call to log important events and
 * allow system debug tools to act upon specific application states.
 * Arbitrary data can be associated with the event.
 *
 * @long The event number is used to uniquely describe the type of event
 * and how the event data is to be interpreted.
 *
 * @long A trigger variable can be registered for a user event number,
 * with create_event(), allowing an application to avoid calling
 * signal_event() unless there is any interest of this event number.
 *
 * @long The size of the data must be less than 64 KB.
 *
 * @long The RMM Run-Mode Monitor supports actions triggered by
 * user events.
 *
 * @input eventid The event number.
 * @input data The address to the event data.
 * @input size The size of the event data.
 *
 * @restriction Also available in interrupt and timer interrupt processes.
 *
 * @seealso create_event
 * @seealso default_restrictions_calling_functions
 */
void
signal_event(SIGSELECT eventid, const void *data, OSBUFSIZE size);

/**
 * @function exit_error_handler
 *
 * @declaration void exit_error_handler(void);
 * @brief Undocumented.
 */
void
exit_error_handler(void);

union SIGNAL *
alloc_nil(OSBUFSIZE size, SIGSELECT signo);

#define ZZGET_SYSCONF

/**
 * @type OSTMOREF
 *
 * @declaration typedef OSADDRESS OSTMOREF;
 *
 * @brief The identity of a timeout request.
 * @long This identity is used to cancel or restart of the timer.
 *
 * @seealso request_tmo
 * @seealso request_tmo_sig
 */
typedef OSADDRESS OSTMOREF;

/**
 * @function request_tmo_sig
 *
 * @include "ose.h"
 *
 * @declaration OSTMOREF
 * request_tmo_sig(OSTIME timeout, union SIGNAL **tmosig);
 *
 * @brief Request a timeout with a specified signal.
 *
 * @long
 * The signal will be sent to the application when the requested time has
 * elapsed. Information necessary for canceling the timeout is returned.
 * @long
 * The actual timeout time, timeout, is rounded upward to the next larger
 * tick because a timeout can only trigger when a system clock tick is
 * received. Hence, the routine guarantees at least the number of
 * milliseconds requested, but may add a few more depending on the tick
 * resolution. When several timeout signals are generated at the same
 * clock tick, the order is unspecified.
 *
 * @input timeout Timeout in milliseconds.
 * @input tmosig User supplied signal to return at timeout.
 *
 * @return Identification of timeout for cancellation
 *
 * @restriction
 * Timeout signals are not affected by redirection tables. Instead, these
 * signals will always end up in the receive queue of the process issuing
 * the timeout request. A time interval of at least 50 hours can be used.
 *
 * @seealso request_tmo
 * @seealso cancel_tmo
 * @seealso cancel_tmo_sig
 * @seealso request_tmo
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // Example with request_tmo_sig and cancel_tmo_sig
 * // Request a timeout after 1 second.
 * // Cancel it if it is not needed any more.
 *
 * #include "ose.h"
 *
 * #define MY_TMO_SIG (13) // Client defined //
 *
 * struct my_tmo
 * {
 *   SIGSELECT sig_no;
 *   // Add application specific data here! //
 * };
 *
 * #define MY_EVENT_SIG (14) // Client defined //
 * struct my_event
 * {
 *   SIGSELECT sig_no;
 * };
 *
 * union SIGNAL
 * {
 *   SIGSELECT sig_no;
 *   struct my_tmo my_tmo;
 *   struct my_event my_event;
 * };
 *
 * OS_PROCESS(my_tmo_process)
 * {
 *   OSTMOREF tmoref;
 *   const SIGSELECT sigsel[] = {0};
 *   union SIGNAL *sig;
 *
 *   // In a more realistic example there would probably be a
 *   // linked list of free timeout signals below.
 *   union SIGNAL *tmo_sig = alloc(sizeof(struct my_tmo),
 *                                 MY_TMO_SIG);
 *
 *   for(;;)
 *     {
 *       // Put application specific data //
 *       // in the timeout signal here. //
 *       tmoref = request_tmo_sig(1000, &tmo_sig);
 *       sig = receive(sigsel);
 *       switch(sig->sig_no)
 *         {
 *         case MY_EVENT_SIG:
 *           // Event occurred in time, all is well //
 *           free_buf(&sig);
 *           // Cancel timeout request and //
 *           // get the signal back for reuse. //
 *           tmo_sig = cancel_tmo_sig(&tmoref);
 *           break;
 *
 *         case MY_TMO_SIG:
 *           // Time out occurred, handle that and continue //
 *           tmo_sig = sig;
 *           break;
 *
 *         default:
 *           // Unexpected signal received //
 *           error2(4711, sender(&sig));
 *           free_buf(&sig);
 *         }
 *     }
 * }
 */
OSTMOREF
request_tmo_sig(OSTIME timeout, union SIGNAL **tmosig);

/**
 * @function request_tmo
 *
 * @include "ose.h"
 *
 * @declaration OSTMOREF
 * request_tmo(OSTIME timeout, SIGSELECT tmosigno);
 *
 * @brief Request a timeout with a specified signal number.
 *
 * @long Request a signal with the specified signal number to the calling
 * process after the specified timeout.
 * @long
 * Information necessary for canceling the timeout is returned.
 * @long
 * This function call is often faster than "request_tmo_sig" because there
 * is no need to allocate a signal until the timeout actually occurs. As a
 * result, it should be used on all occasions where the contents of the
 * timeout signal are irrelevant and when cancellations are frequent.
 * @long
 * The actual timeout time, timeout, is rounded upward to the next larger
 * tick because a timeout can only trigger when a system clock tick is
 * received. Hence, the routine guarantees at least the number of
 * milliseconds requested, but may add a few more depending on the tick
 * resolution. When several timeout signals are generated at the same
 * clock tick, the order is unspecified.
 *
 * @input timeout Timeout time in milliseconds.
 * @input tmosigno Signal number for the wanted signal.
 *
 * @return Identification of timeout for cancellation.
 *
 * @restriction
 * Timeout signals are not affected by redirection tables. Instead, these
 * signals will always end up in the receive queue of the process issuing
 * the timeout request. A time interval of at least 50 hours can be used.
 *
 * @seealso cancel_tmo
 * @seealso request_tmo_sig
 * @seealso cancel_tmo_sig
 * @seealso restart_tmo
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // Example with request_tmo and cancel_tmo
 * // Request a timeout after 1 second.
 * // Cancel it if it is not needed any more.
 *
 * #include "ose.h"
 *
 * #define MY_TMO_SIG (13) // Client defined //
 * int no_longer_needed;
 *
 * OS_PROCESS(my_tmo_process)
 * {
 *   OSTMOREF tmoref;
 *
 *   for(;;)
 *     {
 *       tmoref = request_tmo(1000, MY_TMO_SIG);
 *       // Cancel timeout request after some work //
 *       if(no_longer_needed)
 *         {
 *           cancel_tmo(&tmoref);
 *         }
 *     }
 * }
 */
OSTMOREF
request_tmo(OSTIME timeout, SIGSELECT tmosigno);

/**
 * @function cancel_tmo_sig
 *
 * @include "ose.h"
 *
 * @declaration union SIGNAL *
 * cancel_tmo_sig(OSTMOREF *tmoref);
 *
 * @brief Cancel a timeout and get the signal back.
 *
 * @long
 * The timeout is identified by the tmoref parameter, which was set by
 * request_tmo_sig or request_tmo.
 *
 * @input tmoref Identification of timeout to cancel.
 *
 * @return The signal specified in the request_tmo_sig call or the default
 * signal (with user specified signo) if request_tmo was used.
 *
 * @restriction
 * Both cancel_tmo_sig and cancel_tmo can only cancel timeouts that were
 * requested by the same process.
 * @break
 * It is an error to cancel an already received or canceled timeout. If an
 * error handler chooses to ignore this error, a NIL pointer is returned.
 *
 * @seealso cancel_tmo
 * @seealso request_tmo
 * @seealso request_tmo_sig
 * @seealso restart_tmo
 * @seealso default_restrictions_calling_functions
 *
 * @long See the example in the @link "ose.h" "request_tmo_sig" description.
 */
union SIGNAL *
cancel_tmo_sig(OSTMOREF *tmoref);

/**
 * @function cancel_tmo
 *
 * @include "ose.h"
 *
 * @declaration void
 * cancel_tmo(OSTMOREF *tmoref);
 *
 * @brief Cancel a timeout.
 *
 * @long
 * Cancels a timeout and frees the timeout signal if a signal does exist.
 * The timeout is identified by the tmoref parameter, which was filled in
 * by request_tmo_sig or request_tmo.
 *
 * @input tmoref Identification of timeout to cancel.
 *
 * @restriction
 * Both cancel_tmo_sig and cancel_tmo can only cancel timeouts that were
 * requested by the same process.
 * @break
 * It is an error to cancel an already received or canceled timeout.
 *
 * @seealso cancel_tmo_sig
 * @seealso request_tmo
 * @seealso request_tmo_sig
 * @seealso restart_tmo
 * @seealso default_restrictions_calling_functions
 *
 * @long See the example in the @link "ose.h" "request_tmo" description.
 */
void
cancel_tmo(OSTMOREF *tmoref);

/**
 * @function restart_tmo
 * @include "ose.h"
 *
 * @declaration void
 * restart_tmo(OSTMOREF *tmoref, OSTIME timeout);
 *
 * @brief Restart the specified timeout.
 * @long
 *
 * @input tmoref Identification of timeout to restart.
 * @input timeout New timeout in milliseconds.
 *
 * @restriction This call can only be used by the same process that issued
 * the timeout request in the first place.
 * @break
 * It is an error to restart an already received or canceled timeout.
 *
 * @seealso cancel_tmo_sig
 * @seealso cancel_tmo
 * @seealso request_tmo
 * @seealso request_tmo_sig
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * // Example with request_tmo and restart_tmo
 * // Request a timeout after 1 second.
 * // Restart it after some application event.
 *
 * #include "ose.h"
 *
 * #define MY_TMO_SIG (13) // Client defined //
 * int some_event;
 *
 * OS_PROCESS(my_tmo_process)
 * {
 *   OSTMOREF tmoref = request_tmo(1000, MY_TMO_SIG);
 *
 *   for(;;)
 *     {
 *       // Restart the timeout request after some event //
 *       if(some_event)
 *         {
 *           restart_tmo(&tmoref, 1000);
 *         }
 *     }
 * }
 */
void
restart_tmo(OSTMOREF *tmoref, OSTIME timeout);


/**
 * @function ose_create_report
 *
 * @declaration void
 * ose_create_report(SIGSELECT reportno, OSADDRESS *trig, OSBOOLEAN multiple,
 *                   OSBOOLEAN continuous, OSBOOLEAN maxmin);
 *
 * @brief Register a report class (reportno) and trigger variable.
 *
 * @long The report class number is used to uniquely describe the
 * type of the report and how the report data is to be interpreted.
 *
 * @long The application shall never modify the trig variable; it is done by
 * the kernel. The kernel will set the trigger variable to either 0 or 1
 * to discourage or encourage (respectively) calls to
 * @link "" "ose_set_report_val"() with the associated report number.
 * This allows the application to avoid calls to update report values,
 * for which there is no registered interest.
 * @long The trigger variable must exist as long as the application
 * exists. This usually means that it has to be static or of file
 * scope.
 *
 * @long The RMM Run-Mode Monitor supports profiling based on
 * user reports.
 *
 * @long Applications can use any user report numbers, except the range
 * reserved for OSE, see @link "" "OSE_REPORTS_ENEA_END".
 *
 * @input reportno The report number to identify the report class.
 * @input trig Pointer to a non-local boolean variable.
 * @input multiple Set to one to create a report class with multiple values
 * (for instance one per process). Set to zero to create a report class with
 * a single value.
 * @input continuous Set to one to have the previous value copied to the next
 * interval at interval start. Set to zero to have the value cleared at
 * interval start.
 * @input maxmin Set to one to have set_report_val track the maximum and
 * minimum during each interval. Set to zero to track the current value only.
 *
 * @seealso ose_set_report_val
 * @seealso default_restrictions_calling_functions
 */
void
ose_create_report(SIGSELECT reportno, OSADDRESS *trig, OSBOOLEAN multiple,
                  OSBOOLEAN continuous, OSBOOLEAN maxmin);


/**
 * @function ose_set_report_val
 *
 * @declaration void
 * set_report_val(SIGSELECT reportno, OSREPORTID id, OSREPORTVAL change);
 *
 * @brief Update a report value, e.g. to indicate an application state change.
 *
 * @long The report number (class identifier) is used to uniquely describe the
 * type of the report and how the report data is to be interpreted.
 *
 * @input reportno The report number (class identifier).
 * @input id The ID for which to change the reported value. Ignored if
 * reportno indicates a single value report class.
 * @input change The amount (+/-) by which to change the reported value.
 *
 * @restriction Also available in interrupt and timer interrupt processes.
 *
 * @seealso ose_create_report
 * @seealso default_restrictions_calling_functions
 *
 * @example
 * #include "ose.h"
 *
 * #define HWREGA (1)
 * #define HWREGA_ID (9)
 *
 * static OSADDRESS hwrega_trig = 0;
 * OSBOOLEAN multiple = 0;
 * OSBOOLEAN continuous = 0;
 * OSBOOLEAN maxmin = 1;
 * OSREPORTVAL hw_value = 0;
 *
 * OS_PROCESS(my_process)
 * {
 *    // Code //
 *    ose_create_report(HWREGA, &hwrega_trig, multiple, continuous, maxmin);
 *
 *    for(;;)
 *    {
 *       hw_value = get_hwrega_value();
 *       // Code ... OSE will set the trig value if reports are expected //
 *       if (hwrega_trig)
 *       {
 *          ose_set_report_val(HWREGA, HWREGA_ID, hw_value);
 *       }
 *    }
 * }
 */
void
ose_set_report_val(SIGSELECT reportno, OSREPORTID id, OSREPORTVAL change);

/**
 * @function ose_num_cpus
 *
 * @include "ose.h"
 *
 * @declaration cpuid_t
 * ose_num_cpus(void);
 *
 * @brief Get the number of CPUs available in the system.
 *
 * @return The value 1 for non-SMP (Symmetric Multi Processing) systems
 * and a value >= 1 for SMP systems.
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * Will not change interrupt mask unless error handler is called.
 *
 * @seealso cpuid_t
 * @seealso ose_cpu_id
 * @seealso default_restrictions_calling_functions
 *
 */
cpuid_t
ose_num_cpus(void);

/**
 * @function ose_cpu_id
 *
 * @include "ose.h"
 *
 * @declaration cpuid_t
 * ose_cpu_id(void);
 *
 * @brief Get the identifier of the current CPU.
 *
 * @return A number n, in the range 0 <= n < ose_num_cpus().
 *
 * @restriction
 * Also available in interrupt and timer interrupt processes.
 * @break
 * Will not change interrupt mask unless error handler is called.
 *
 * @seealso cpuid_t
 * @seealso ose_num_cpus
 * @seealso default_restrictions_calling_functions
 *
 */
cpuid_t
ose_cpu_id(void);

/**
 * @function ose_get_error_info
 * @include "ose.h"
 *
 * @brief Get information on the latest error for a process.
 * @long Returns the same information as an error handler gets, except that
 *       recursive errors are here reported as if they were not recursive. 
 *       This gives the kernel error handler a way to access the original 
 *       value of the "extra" parameter.
 * @long If the "extra" parameter provides a pointer to additional error
 *       information, then it is unsafe to dereference it while not running
 *       inside an error handler. Such information is often temporal, it may
 *       for example be placed in the process stack or another volatile or
 *       even unreadable location.
 *
 * @input  pid      The process to investigate (ID of any living process).
 * @output ecode    The latest error code.
 * @output extra    The corresponding extra information.
 *
 * @return          The return value is either the user_called flag
 *                  or indicates an unsuccessful result. @break 
 *                  0: user_called=0. The kernel detected the error.
 *                     Error codes are listed in ose_err.h. @break
 *                  1: user_called=1. error() or error2() was called. @break
 *                  2: No error info stored for the selected process. @break
 *                  3: No such process found.
 *
 * @restriction
 * ose_get_error_info can be called from any process type or error handlers,
 * even in freeze mode.
 *
 * @seealso ose_reset_error_info
 * @seealso create_error_handler
 */
OSBOOLEAN ose_get_error_info(PROCESS pid, OSERRCODE *ecode, OSERRCODE *extra);

/**
 * @function ose_reset_error_info
 * @include "ose.h"
 *
 * @brief Clear the information on the latest error for the current process.
 * @long This system call erases the information on the latest error, thus
 *       making it invisible for ose_get_error_info(). ose_reset_error_info
 *       may be called for example from a process error handler.
 *
 * @seealso ose_get_error_info
 * @seealso create_error_handler
 * @seealso default_restrictions_calling_functions
 */

void ose_reset_error_info(void);

#ifdef __cplusplus
}
#endif

/*
** This includes the SPI (System Programmers Interface) calls if there
** is a need to be fully compatible with the programming model used
** before the birth of the ose_spi.h header file.
*/
#ifdef OSE_H_SPI
#include "ose_spi/ose_spi.h"
#endif

/*
** This includes the OSE Classic calls if there is a need to be fully
** compatible with the programming model used before the birth of the
** ose_classic.h header file.
*/
#ifdef OSE_H_CLASSIC
#include "ose_spi/ose_classic.h"
#endif

#endif /* ZZOSE_H */
