/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2002-2014 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */
/*
*****************************************************************************
**			      PROGRAM MODULE				   **
**									   **
**@(#) File: pose.c		Id: 02/1946370276339265			   **
**@(#) Revision: 2.182.19	Revision date: 14/01/31	        	   **
**     Subsystem: 		Portable OSE.				   **
**     Original author:		Bengt Eliasson.				   **
**     Summary:			OSE kernel.				   **
**									   **
**									   **
*****************************************************************************

	CONTENTS
	--------

	1  Description
	2  History of development
	3  Header files
	   3.1 Internal definitions
	   3.2 Libraries
	4  Local definitions
	   4.1 Manifest constants
	   4.2 Type definitions
	   4.3 Macros
	   4.4 Signal composition
	5  External references
	   5.1 Functions
	   5.2 Variables
	   5.3 Forward references
	6  Global variables
	7  Local definitions
	   7.1 Local variables
	   7.4 Local helper functions 
	8  System start
	9  Dispatcher and associated functions
	10 Block management
	11 Process management
	12 Pool management
	13 Error management
	14 Timer mangement
	15 Start/stop
	16 Send/receive
	17 attach/detach
	18 hunt and associates
	19 Fast semaphores
	20 Semaphores
	21 Environment variables and per process data
	22 Breakpoints
	23 System interrogation
	24 Splay tree management
	25 Miscellaneous
	26 Process entrypoints
	27 Compatibility glue

*****************************************************************************
*/

/*
*****************************************************************************
** 1  DESCRIPTION.
*****************************************************************************
** This file contains the source code for a portable OSE kernel.
**
** The C kernel requires a CPU dependent target interface that contains
** various low level functions. This interface is defined in pose.h
**
** The C kernel can be configured by editing pose.con. The configuration
** file contains items that may be CPU dependent, and which must be known
** to the C kernel at the time of compilation.
**
** All public and external names in the C kernel can be changed. This may
** be desirable for instance if multiple versions of the kernel should be
** selected from a library. Actual names are defined in the file osenames.h.
**
** The kernel is written in ANSI C.
** 
*/

/*
*****************************************************************************
** 2  HISTORY OF DEVELOPMENT.
*****************************************************************************
** date      responsible           notes
** --------  --------------------  ------------------------------------------
** 92/03/20  Bengt Eliasson        Initial release.
** 93/08/06  Bengt Eliasson        Debugger added.
** 93/09/21  Bengt Eliasson        A number of debugger related changes.
** 93/10/08  Bengt Eliasson        More debugger hooks.
**                                 Assign_linkhandler() retuns OSBOOLEAN.
** 93/10/22  Bengt Eliasson        Fixed problem in kill_proc(), get_pri()
**                                 and
** 93/10/27  Bengt Eliasson        Fixed problem in get_pcb().
** 93/11/22  Bengt Eliasson        Killed processes are now imediately
**                                 recognized by the huntd. Signals redirected
**                                 are no longer lost.
** 93/11/30  Bengt Eliasson        Fixed the problem that caused kill_proc,
**                                 stop and intercept to crash if the affected
**                                 process is ready.
**                                 Made some variables volatile.
** 93/12/10  Bengt Eliasson        The problem that caused set_pri() to fail
**                                 at rare occasions has been fixed.
** 93/12/11  Bengt Eliasson        Start/stop and get_pcb are now available
**                                 also to interrupt processes.
**                                 The error2 system call was added.
** 94/02/10  Bengt Eliasson        Reordered internal variables to allow
**                                 assembly versions of system calls and
**                                 dispatcher.
** 94/02/20  Bengt Eliasson        Removed some compiler warnings.
**                                 Corrected extra information passed
**                                 to error handlers.
** 94/06/19  Bengt Eliasson        A problem that sometimes caused loss
**                                 of attached signals has been fixed.
** 94/08/21  Bengt Eliasson        Added the zzgo_search entrypoint.
**                                 The attach_segment call now returns a
**                                 proper segment id.
**                                 The attach_block and select_segment hooks
**                                 are now functional.
**                                 Zzfind_first_bit_set() is now called
**                                 through a macro FIND_FIRST_BIT().
**                                 The set_suid() call was added.
**                                 Space for attach and process lists are now
**                                 allocated from the pool as specified in
**                                 the sysctlblk structure.
**                                 When a static block is killed,the kernel
**                                 now kills the entire segment, unless it
**                                 is the supervisor segment.
** 94/09/01  Bengt Eliasson        Added the non portable set_redirection()
**                                 system call.
**                                 Added the OSE_EKILLED_STATIC_PROCESS,
**                                 OSE_EILLEGAL_REDIRCOUNT and
**                                 OSE_ECORRUPTED_POOL error codes.
**                                 The send(), send_w_s() and zzsys_send_w_s()
**                                 were changed so that the addressee field is
**                                 always set. Fixed negative receive count
**                                 in get_pcb().
** 94/10/28  Bengt Eliasson        The size of the pcb is now constant
**                                 regardless of debug configuration.
** 94/11/14  Bengt Eliasson        Fixed missed calls to timer processes when
**                                 more than one process was expected to run.
**                                 Interrupts are enabled during early restart.
** 94/11/14  Bengt Eliasson        Added the zzdp_look_aside variable.
**                                 The obsolete parameter to zzsave_context()
**                                 was removed.
**                                 The USE_INTERRUPT_UAREA and
**                                 CALL_INTERRUPTS_DIRECTLY flags were added
**                                 for speed optimization.
** 94/11/14  Bengt Eliasson        Fixed a problem that caused failure to
**                                 start the segment garbage collector, which
**                                 in turn caused segments to remain
**                                 indefinitely in memory.
** 95/04/05  Bengt Eliasson        A problem that caused the kernel to crash
**                                 during system start was fixed.
** 95/04/15  Bengt Eliasson        Fixed a bug in detach that caused random
**                                 signals at times. (Really fixed in 1.13,
**                                 but undocumented until 1.14.)
**                                 The error recover procedure from pool
**                                 crashes was improved.
**                                 The common_hunt symbol is now local.
**                                 The pool_dumper for development test was
**                                 added.
**                                 The set_env() call now accepts a value
**                                 parameter set to NULL to remove the
**                                 variable.
** 95/06/07  Anders Hagstrom       Fixed a problem that caused a pool leak when
**                                 breakpoints are used. (anha 950531)
** 95/09/01  Bengt Eliasson        Signal ownership counters are now
**                                 modified only when interrupts are disabled.
**                                 This was a problem on cpu types that do
**                                 not have memory increment and decrement
**                                 instructions. (Patch by "mape".)
** 95/09/01  Bengt Eliasson        Added the blockname field in the
**                                 _pcb struct. To be used by the ldm.
** 95/09/01  Bengt Eliasson        Now allows '/'-characters in linkh names.
** 95/09/01  Bengt Eliasson        Supports the attach_list field in get_pcb().
** 95/09/01  Bengt Eliasson        Added the USE_CAPTURE_REGS flag. The
**                                 get_pcb() call was rewritten.
** 95/09/01  Bengt Eliasson        Added the implementation
**                                 specific system calls set_pri_for(),
**                                 set_abstime(), get_abstime(), get_segid(),
**                                 create_event() and zzsignal_event().
** 96/01/30  Bengt Eliasson        Fixed problem that caused osesysd to hang
**                                 when using a linkhandler that is owned
**                                 by another user.
** 96/03/05  Bengt Eliasson        Fixed a debugger related problem that
**                                 caused the kernel to hang after killing a
**                                 signal caught in a receive call.
** 96/09/25  Bengt Eliasson        Fixed a large number of prblems described
**                                 in the readme file for pose 1.17.2.
** 96/11/01  Bengt Eliasson        Added 32 bit internal structure alignment
**                                 padding. Signal control blocks and
**                                 pool fragment control blocks are now
**                                 located in supervisor space for memory
**                                 models > 0. Other fixes as described in
**                                 the readme file for pose 1.18.
** 96/11/10  Bengt Eliasson        Fixed a bug when deleting a pool with no
**                                 signals. Other fixes as described in
**                                 the readme file for pose 1.18.1.
** 96/11/13  Bengt Eliasson        Fixed a bug in the use of redir tables.
** 96/12/05  Fredrik Bredberg      Improved error reporting.
** 96/12/05  Bengt Eliasson        Fixed a slight problem in the attach() call.
**                                 Other fixes as described in the readme
**                                 file for pose 1.18.3.
** 96/12/17  Fredrik Bredberg      Killed an OSE_EILLEGAL_SUPER_MODE error
**                                 reporting bug.
** 96/12/17  Bengt Eliasson        Fixed a rare race condition in alloc.
** 97/04/25  Fredrik Bredberg      Fixed a number of bugs/improvements found
**                                 during the certification project.
** 97/09/04  Fredrik Bredberg      Fixed a compilation error in set_fsem()
**                                 that occured when ERROR_CHECKS == 0.
** 97/09/16  Fredrik Bredberg      Fixed so that signal ports can be
**                                 examined with get_pcb()/get_signal().
** 97/09/20  Bengt Eliasson        Fixed a bug in get_pcb() when using
**                                 USE_CAPTURE_REGS set to 0 in pose.con.
** 97/10/21  Bengt Eliasson        Added padding in struct ready_list_entry.
**                                 The ose_tickd daemon was optimized for
**                                 error levels 0 and 1.
** 97/11/24  Fredrik Bredberg      Fixed some bugs in pool analyze syscalls.
** 98/01/08  Bengt Eliasson        The new "Illuminator" tool is supported.
** 98/06/26  Fredrik Bredberg      Changed the type for zzldmvarp in the
**                                 world struct.
** 98/08/07  Bjorn Johansson       Fixed some problems in the CPU profiler
**                                 subsystem.
** 98/08/14  Fredrik Bredberg      Optimized code, corrected bugs, added
**                                 checks and removed compiler warnings.
** 98/08/20  Fredrik Bredberg      Optimized code and removed compiler
**                                 warnings.
** 98/09/07  Bengt Eliasson        Fixed a problem which caused the system
**                                 debugger to clear the system trace and the
**                                 action list on every warm restart.
** 98/10/11  Bengt Eliasson        Now supports sampled cpu load profiling
**                                 towards applications.
** 98/10/25  Bengt Eliasson        Fixed a bug in the set_env(x,y,NULL) form
**                                 of the set_env() call.
** 98/11/11  Fredrik Bredberg      Fixed some bugs which were introduced 
**                                 when a few strcpy was changed to memcpy.
** 99/01/11  Bengt Eliasson        Added breakpoint trigger handlers.
**                                 Added a pid parameter to zzkill_context().
**                                 Intercept() and resume() now handle segpids.
** 99/02/10  Fredrik Bredberg      Rewrote some allocation code which used to
**                                 waste memory. Corrected two bugs in
**                                 set_bp4(). Corrected error subcodes in
**                                 set_pri_for() and receive_with(). Fixed a
**                                 bug in sysconf_33(). Removed compiler
**                                 warnings.
** 99/03/14  Fredrik Bredberg      Fixed a bug in set_bp4() when using
**                                 multiple hook breakpoints. Added handling
**                                 of a new return code from a breakpoint
**                                 trigger handler.
** 99/05/07  Fredrik Bredberg      Fixed bugs in get_sig_info(), flush(),
**                                 get_pcb() and in code related to segment
**                                 killing. Optimized set_envp() and redir
**                                 handling.
** 99/08/27  Fredrik Bredberg      Optimized hunt() and common_hunt().
** 99/10/22  Bengt Eliasson        Improved interrupt latency.
** 99/12/08  Fredrik Bredberg      Fixed an all_wanted bug in receive_from()
**                                 and receive_with(). Added attach_list
**                                 functionality to get_pcb() and
**                                 get_signal(). Added error subcodes.
** 00/05/31  Fredrik Bredberg      Added freelist info to get_sig_info().
** 00/08/17  Fredrik Bredberg      Removed compiler warnings.
** 00/12/19  Fredrik Bredberg      Fixed a race bug in set_bp() & set_bp4().
**                                 Made check_pid() et al. more waterproof.
**                                 Added a redir check in create_process().
** 01/04/10  Fredrik Bredberg      Added const to some syscall prototypes.
**                                 Removed compiler warnings.
** 01/10/10  Fredrik Bredberg      Added the cond_halt() system call.
** 01/11/21  Fredrik Bredberg      At last I added safe_send_w_s().
** 01/11/27  Gunnar Eklund         Added the timeout signal system calls.
**                                 Renamed cond_halt() to set_halt_condition().
**                                 Also added subcodes
**                                 OSE_SET_REDIRECTION, OSE_GET_STK_POOLID,
**                                 OSE_GET_SIG_POOLID, OSE_RECEIVE_SPORT.
** 02/02/20  Gunnar Eklund         Optimize timeout signals. Set client as
**                                 sender in timeout signals.
** 02/03/13  Gunnar Eklund         POSE 2.12 (Changes only made in ose.h).
** 02/04/19  Fredrik Bredberg      Fixed a bug get_attached_signal().
** 02/06/10  Gunnar Eklund         POSE 2.14.
**				   Added the first waiting pid in the extra 
**				   argument for error 
**				   OSE_EKILLED_SEMAPHORE_IN_USE.
**				   Fixed OSEhq08811: kill_sem() is slow if many
**				   semaphores exist in the segment.
**				   Fixed OSEhq08538: set_env causes a 8 bit 
**				   write to address 0.
**				   Changed check_pid to check_pid_bid_seg
**				   when checking sender in receive_from()
**				   to permit selection of attach signals 
**				   for non-process objects.
**				   Added check_pid_bid in get_mem and set_mem.
**				   Corrected segid passed to zzcall_mem_move()
**				   when operating on blocks.
**				   Now recovering saved zzcur_p, zzwakeflag and
**				   zzcurrent_uarea when OSE_ESPURIOUS_INTERRUPT
**				   is detected by zzinit_int and that error is 
**				   ignored. This only affects the combination
**				   LATE_INIT_INT == 0 and ERROR_CHECKS == 2.
** 02/09/26  Gunnar Eklund         POSE 2.15
** 02/10/08  Fredrik Bredberg      Fixed an "orphan process" glitch in
**                                 create_process().
** 03/03/31  Fredrik Bredberg      Fixed OSEhq12958 by rewriting the handling
**                                 of free attach entries, also fixed
**                                 OSEhq12785 by adding OSE_ENOT_ATTREF_OWNER.
** 03/03/13  Gunnar Eklund	   Extracted manifest constants and local 
**				   types to pose_internal.h
** 03/05/06  Gunnar Eklund         Fixed False OSE_EATTACHED_TO_CALLER error
**                                 and Detach memory leak, TR OSEhq13322 
**                                 and OSEhq13323.
** 03/06/16  Fredrik Bredberg      Fixed OSEhq13673 by rewriting the handling
**                                 of recursive fatal errors in os_error().
** 03/07/16  Gunnar Eklund         Rev 2.18.2
** 03/08/21  Fredrik Bredberg      Fixed two alignment bugs in init_pool().
**                                 (Merged from 2.13.1 by Gunnar)
** 03/11/28  Gunnar Eklund         Removed struct poolcb from the large_cb
**                                 union, and added a separate poolcb free list
**                                 to save 96 byte/process. 
**                                 Moved check_nonint_caller to allow inlining.
**                                 Added INLINE for a few minor functions, see
**                                 pose.con.
** 04/01/15  Patrik Stromblad      Added per process data support.
** 04/01/28  Gunnar Eklund         Swapped order of zzget_microticks() and
**                                 zzuu.ticks access in get_systime() to solve
**                                 optimizer trouble.
** 04/02/18  Fredrik Bredberg      Added support for deaf int/tim processes.
**                                 Corrected a bug in os_error().
** 04/03/24  Fredrik Bredberg      Added support OSE_EEARLY_ERROR handling.
** 04/04/24  Fredrik Bredberg      Added the mutex system calls.
** 04/05/06  Gunnar Eklund         Added sysconf 58.
**                                 Rewrote set_env() to permit get_envp() from
**                                 interrupt processes. 
**                                 Changed get_ptype() to return OS_ILLEGAL 
**                                 instead of OS_ZOOMBIE when the PROCESS 
**                                 argument is less than 0x10000.
** 04/05/12  Gunnar Eklund         Made get_envp available for interrupt 
**                                 processes.
** 04/06/07  Gunnar Eklund         Mutex: proper priority inheritance added,
**                                 mutex error information improved.
** 04/07/12  Gunnar Eklund         Fixed a bug with dying blocks in
**                                 create_process. Adding error checks in 
**                                 create_process for user mode callers.
**                                 Refuse restore() when owner has died.
**                                 Repair the user sigadm in collect_pool.
**                                 Adding sysconf_57.
**                                 Rewrite get_pcb for paging text.
**                                 Modified all environment variable functions,
**                                 create_pool and per-process data for paging
**                                 text.
**                                 Added a call of zzcall_kernel_halted() from
**                                 zzkrn_halted_hook().
** 04/08/30  Gunnar Eklund         Fixed a bug in get_poolcb() causing a loop
**                                 when the fragment counter wraps at 128 
**                                 pool extend.
** 04/09/08  Gunnar Eklund         OSEhq16004: Check magic in zzget_debugpcb
**                                 to prevent match on non-process objects.
**                                 Added zeroing of cur_value when deleting
**                                 environment variables, to safely handle a
**                                 concurrent get_envp() from an interrupt 
**                                 process.
** 04/11/09  Fredrik Bredberg      Added timeslicing of pri procs. Rewrote
**                                 the dispatcher to get more "fair share"
**                                 scheduling of bg procs.
** 04/11/17  Gunnar Eklund         Added support for recovery from user stack
**                                 mark overwrite.
**                                 Added sysconf_56() for receive operations
**                                 in exception handlers.
**                                 Removed unneeded code in tick().
**                                 Correcting the extra parameter for error 
**                                 OSE_EMUTEX_LOCKED. 
** 04/11/29  Gunnar Eklund         Made make_mypri_not_ready() capable of
**                                 handling that the current process was 
**                                 already set "not ready" by an interrupt
**                                 process. Also handle that the current
**                                 process is ready, but not at the head
**                                 of the ready list (a timeslice victim).
**                                 Finally rewrote make_mypri_not_ready 
**                                 as a function.
**                                 Changed make_otherpri_not_ready() to
**                                 request a new dispatch if it did 
**                                 interrupt the dispatcher search phase. 
**                                 Added a check of the head of the ready 
**                                 list in the timeslice handling of
**                                 prioritized processes in tick(). Also 
**                                 narrowing a LOCK region in the same 
**                                 section. 
**                                 Fixed a signal queue leak in detach(). 
**                                 Made attach signals insensitive of 
**                                 redirection tables.
** 04/12/01  Patrik Stromblad	   Adding the sharing flag to bcb.
**                                 Checks on this in create_block.
**                                 Added sysconf_63/64.
** 04/12/15  Gunnar Eklund         Suppressed reports of deleted variables in
**                                 get_env_list. Also corrected the handling
**                                 of very long names in get_env_list.
**                                 Added checks for blocks that are being
**                                 killed in sysconf_63/64.
**                                 Changed OSE_RECEIVE_FROM to 
**                                 OSE_RECEIVE_SPORT in sysconf_20.
**                                 Added an experimental set_segment()
**                                 implementation.
** 05/01/26  Gunnar Eklund         Added the zzfind_current_debug_info()
**                                 function.
** 05/01/26  Fredrik Bredberg      Added sysconf_55 for reading/setting
**                                 the timeslice value of a process.
** 05/01/27  Anders Hagstrom       clib_pointer is cleared in attach_segment.
** 05/03/02  Fredrik Bredberg      Added support for send-hooks and per
**                                 process data for link handlers, which can
**                                 be managed with sysconf_65.
** 05/03/17  Gunnar Eklund         Send_hook() now ignores sending to blocks.
**                                 Repaired handling of redirection tables in
**                                 Create_process() after send_hook disorder.
**                                 Making pool_extend possible when allocation
**                                 fails in an error handler.
** 05/04/11  Gunnar Eklund         Pool_extend possible when allocation fails
**                                 in an error handler, now also for interrupt 
**                                 processes.
**                                 No longer forcing errors in interrupt 
**                                 processes to become fatal just because the
**                                 dispatcher was locked.
** 05/04/17  Karl Morner           Added a second per process data slot for
**                                 link handlers.
** 05/05/03  Fredrik Bredberg      Rewrote the send-hook code (and its
**                                 friends) to be more bullit proof.
** 05/05/18  Fredrik Bredberg      Changed the handling of return values in
**                                 send_hook().
** 05/06/16  Fredrik Bredberg      Changed LOCK_PUSH/_POP to LOCK_SAVE/
**                                 _RESTORE. Made alloc() more compact and
**                                 removed ipool_brk_lo(). Also removed
**                                 several aliasing (and other) warnings.
** 05/07/22  Claes Lillieskold     Fixed OSEhq20190: Moved assignment of the
**                                 the local variable rlist in set_pri inside
**                                 the interrupt lock.
** 05/08/15  Gunnar Eklund         Permit attach() for interrupt processes. 
**                                 Still no detach. Also fixed an odd memory 
**                                 leak in create_ppd_key() for paging 
**                                 systems.
** 05/08/15  Fredrik Bredberg      Added the missing per processes data
**                                 functionality in sysconf_53.
** 05/10/03  Gunnar Eklund         Added calls of zzcall_select_segment to
**                                 handle dying segments in sysd_delete_block,
**                                 sysd_do_free_pool and attach_segment.
**                                 Correcting dispatcher lock region in
**                                 set_segment().
**                                 Get_sig_info() now returns 1 instead of 4
**                                 for signals not yet freed by a dying 
**                                 process.
**                                 Added LOCK UNLOCK in kill_block().
** 05/11/09  Gunnar Eklund         Detach available for interrupt processes.
** 05/11/14  Gunnar Eklund         Added sysconf_69 (nano delay).
**                                 Changed removal of segments to co-exist 
**                                 with set_segment. Fixed a NULL pointer
**                                 problem intoduced in the detach rewrite.
** 05/11/16  Patrik Stromblad      Added sysconf 67/68, hooks for test and
**                                 pool extend in system error handling.
**                                 Changed user process entrypoint from zzmain
**                                 to _zzmain (to give core the entrypoint).
** 05/12/14  Gunnar Eklund         Fix a memory leak in the latest "detach
**                                 for interrupt process" stuff.
** 06/01/13  Gunnar Eklund         Added mutcb control blocks to solve 
**                                 problems with mutexes in uninitiated ram.
**                                 Moved reading of old priority into the 
**                                 LOCK region of set_pri.
**                                 Simplified attach_segment.
** 06/01/23  Gunnar Eklund         Copy queued signals from dying pools into
**                                 the system pool to be able to deallocate
**                                 killed segments. Affects also attach and
**                                 request_tmo_sig, as expired attach and tmo
**                                 signals must not be relocated.
**                                 Added profiling hooks for allocation and 
**                                 deallocation of signal buffers.
**                                 Improved ose_nano_delay.
** 06/02/20  Gunnar Eklund         Fixed a race in mutex_raise_priority.
**                                 Added assert code in attach/detach.
**                                 Added sysconf_71 (alias get_segid_list).
**                                 Changed get_segid() to accept a poolid.
** 06/02/22  Fredrik Bredberg      Force garbage collection of attach
**                                 entries when we hit MAX_ATTACH.
**                                 Alloc() now preserves the dispatcher-lock
**                                 state, making locked_alloc() a one-liner.
**                                 It is now possible to call get_signal()
**                                 to obtain signals attached to blocks.
**                                 Reduced the codesize of start(),stop(),
**                                 intercept() and resume().
**                                 Optimized the pri proc execution path
**                                 of the dispatcher search phase.
**                                 Improved error information on
**                                 OSE_ETOO_MANY-_PROCESSES/_ATTACHED.
**                                 Prevented kill_proc() from returning
**                                 when killing the current segment.
**                                 Added new syscall get_pid_info().
** 06/03/27  Gunnar Eklund         Added sysconf_73, reporting some system 
**                                 limits and statistics.
** 06/04/18  Gunnar Eklund         Removed "assignment in condition" warnings
**                                 for the ARM RVCT compiler.
** 06/04/24  Fredrik Bredberg      Added a get/set_mem helper function.
**                                 Also added zzuu.pool_delete_cnt to
**                                 solve a race in check_buffer().
** 06/04/24  Gunnar Eklund         Added ose_get_time_resolution and
**                                 ose_set_time_resolution.
**                                 Added zzkrn_get_handler and 
**                                 zzkrn_set_handler to control profiling.
** 06/05/15  Fredrik Bredberg      Added functionality to get_pid_info().
** 06/05/23  Gunnar Eklund         Fixed a race condition in mutex_dequeue 
**                                 triggered by nested mutexes using 
**                                 priority enheritance.
**                                 Removed duplicated signal and stack sizes
**                                 in the poolcb, to let get_poolcb return
**                                 proper size tables.
** 06/06/14  Gunnar Eklund         Added functionality to get_pid_info().
** 06/06/19  Gunnar Eklund         Fixed a NULL pointer problem in 
**                                 sysd_check_free_pool.
** 06/09/13  Gunnar Eklund         Fixed a problem in error checks of signal
**                                 buffers when full checks was enabled and 
**                                 pools were killed.
** 06/09/18  Fredrik Bredberg      Vectorize global vars. POSE goes SMP!
** 06/10/16  Fredrik Bredberg      Added new syscall ose_smp_bind_process(),
**                                 also added more SMP infrastructure.
** 06/10/23  Fredrik Bredberg      Added the use of SMP locks.
** 06/10/24  Gunnar Eklund         Fixes in init_per_cpu(). Get_pcb no
**                                 longer tries to fetch registers for
**                                 running (current) processes.
** 06/12/06  Gunnar Eklund         The background process is new double linked.
**                                 Background process scheduling algorithm
**                                 modified to guarantee time for the 
**                                 timeslice owner. Changed the insert point
**                                 of new background processes from first to
**                                 last in the background list.
**                                 SMP: Added dynamic bind to cpu.
**                                 SMP: Interrupt and timer processes fixed
**                                 at cpu 0.
**                                 SMP: Implemented timeslicing of prioritized
**                                 and background processes.
** 06/10/23  Fredrik Bredberg      SMP: Stabilisation task: make proper use of
**                                 the idle stack, and don't delete stacks
**                                 that are in use (on any CPU).
**                                 Corrected a SMP related bug in
**                                 make_otherpri_not_ready().
** 06/12/19  Gunnar Eklund         Permit spontanous extend_pool().
**                                 Added SMP version of expire_tmo_signals().
**                                 Disabled obsolete breakpoint code.
** 07/01/16  Fredrik Bredberg      Fixed a scheduler bug for background procs.
**                                 Adopted the new SMP locking scheme.
** 07/01/16  Hans Hedin            Extended user mode block check in
**                                 create_block.
** 07/01/24  Fredrik Bredberg      SMP: Fixed a int-proc wakeup problem and a
**                                 locking scheme violation in mutex_enqueue.
** 07/02/01  Gunnar Eklund         Fixed SMP-related problems in detach().
**                                 Also, let make_old_attref() try to allocate
**                                 an expired attcb to save system pool.
**                                 Added name access in get_pid_info.
** 07/02/07  Hans Hedin            Added retrieval of segnum in get_pid_info().
** 07/02/09  Magnus Karlsson       Extended start, stop and smp_bind_process
**                                 to also operate on segments.
**                                 SMP: smp_bind_process now moves all
**                                 processes in a block or segment that can be
**                                 moved. 
**                                 Merged start/resume and stop/intercept into
**                                 one function each, as they were almost
**                                 identical. 
** 07/02/19  Fredrik Bredberg      SMP: Added a spinlock to protect
**                                 zzuu.tmolist.
** 07/02/20  Petter Ericson        SMP: Added zzbind_dbghook() support to
**                                 smp_bind_process().
** 07/02/21  Gunnar Eklund         Configurable size of tmolist and 
**                                 tos_tick_list.
**                                 Restore make_otherpri_ready_noswap() as
**                                 a separate function.
**                                 SMP fix in mutex_dequeue().
**                                 Call the select segment handler even when
**                                 switching to the system segment but only
**                                 for the PowerPC architecture.
** 07/02/22  Fredrik Bredberg      SMP: Added synchronisation to ensure
**                                 that it's safe to move a processes to
**                                 another CPU. Fixed a SMP lock
**                                 inconsistency problem in SMP_LOCK_SWAP().
** 07/02/26  Fredrik Bredberg      SMP: removed zzgo_search_idle and
**                                 improved the "move proc synchronisation".
** 07/02/28  Gunnar Eklund         Let an error handler decide if it is
**                                 permitted to create a new block when
**                                 the caller lives in shared mode.
**                                 Fixed an SMP dispatcher lock error in 
**                                 receive_sport.
** 07/03/02  Fredrik Bredberg      Fixed a glitch in intercept().
** 07/03/09  Gunnar Eklund         Add zzget_dp_look_aside(cpu).
** 07/03/14  Fredrik Stenling      Renamed ose_set_report_val() and 
**                                 ose_create_report() to
**                                 zzose_set_report_val_sysconf_77() and
**                                 zzose_create_report_sysconf_78().
** 07/03/16  Fredrik Bredberg      Stabilised process migration.
** 07/03/20  Gunnar Eklund         Added zzkrn_segment_migration_status().
**                                 Added sysconf_79 to register a Program
**                                 Manager. 
**                                 Moved idle_pcb into the system pool.
**                                 Changed idle_varp in osvarp from a pointer
**                                 to a handle. Stepped MAGIC_KERNEL_TYPE.
** 07/08/23  Fredrik Bredberg      Added zzkrn_dbg_intercept(), optimized
**                                 safe_send_w_s().
** 07/09/07  Fredrik Bredberg      Added calls to zzupdate_err_handl_level()
**                                 and new functionality to get_pid_info().
** 07/11/20  Fredrik Bredberg      Fixed a parameter validating problem
**                                 in attach(). Added functionality to
**                                 get_pid_info(). Made zzur_ni and
**                                 zzcur_p more consistent in the
**                                 scheduler. Patched a hole in
**                                 send_hook(). Fixed a SMP related
**                                 bug in get_pcb(). Added an
**                                 experimental function for CPU load
**                                 mesurements.
** 07/11/26  Fredrik Bredberg      Prepare for larger pid-space.
** 07/12/28  Gunnar Eklund         Added mas_mapped attribute per segcb, 
**                                 and sysconf_80 to set it.
**                                 Modified get_pcb to handle mas mapped
**                                 data sections.
**                                 Fixed a glitch with lost hunt signals
**                                 from dying processes.
**                                 Permit system calls in create handlers
**                                 to refer to the process being created.
** 08/02/08  Gunnar Eklund         Limit power naps to current tick when
**                                 cpu profiling is running.
** 08/03/17  Anders Hagstrom       SMP related optimisations.
** 08/03/12  Robert Andersson      Extended get_pid_info with manager mode
**                                 process state.
** 08/04/15  Fredrik Bredberg      Added semi public "kill lock" functions.
**                                 Fixed small things in setup_bcb() and
**                                 and get_pid_info().
** 08/04/23  Gunnar Eklund         Locking infrastructure. See readme.
** 08/04/28  Gunnar Eklund         Added toscb_spin_lock.
**                                 Added sysconf_81 to fetch pointers and 
**                                 sizes, for example for sysctlblk.
**                                 Added spin locks per pool and per 
**                                 signal free list.
** 08/05/12  Gunnar Eklund         Made get_ptype and check_pid and friends
**                                 LOCK-free. Disabled power_save for SMP.
**                                 Repaired a broken lock region in
**                                 make_mypri_current(), caused by SMP
**                                 related changes in cancel_timeout().
** 08/06/04  Fredrik Bredberg      Fixed kill_proc() on blocks which
**                                 contain "kill locked" processes. Also
**                                 added kill_curr_at_unlock() and extended
**                                 get_pid_info() with kill lock info.
** 08/07/02  Fredrik Bredberg      SMP: Interrupt processes can now run on
**                                 any cpu, i.e. they are no longer forced
**                                 to run on cpu 0. It is now ok to call
**                                 get_pid_info() from a create handler.
** 08/07/18  Gunnar Eklund         Permit a supervisor mode process to 
**                                 migrate processes in its own block
**                                 even if the segment is locked to a cpu.
** 08/08/25  Gunnar Eklund         Truncate buffer in get_pid_list.
** 08/09/02  Mattias Bertilsson    Added a virtual link handler concept 
**                                 for blocks. Changes made in most huntd 
**                                 functions and in assign_linkhandler(),
**                                 which now accepts block pids.
** 08/09/05  Gunnar Eklund         Added annotation of created semaphores.
** 08/09/08  Gunnar Eklund         Fixes for swap handlers refering to
**                                 data in MAS. See OSECR-4172 in readme.
** 08/09/09  Anders Hagstrom       Added zzkrn_alloc_kernelstack.
** 08/09/12  Fredrik Bredberg      Added zzkrn_mutex_init.
** 08/10/02  Robin Westberg        Fix get_pid_list returning uninitialized
**                                 data. Unlock dispatcher during alloc.
** 08/10/03  Jonas Westaker        Added zzkrn_evt_notify.
** 08/10/21  Fredrik Bredberg      Added send_w_sender() and restore_from().
**                                 SMP: Make sure that idle loops don't
**                                 continue to use the address mapping of a
**                                 deleted segment.
** 08/10/23  Mikael Wanggren       Wrote the pool extender for local pools.
** 08/11/03  Gunnar Eklund         Making SMP interrupt stack info observable.
** 08/11/04  Fredrik Bredberg      Rewrote "kill lock" handling in both
**                                 kill_proc() and sysd().
** 08/11/11  Robin Westberg        Decouple automatic pool extension from
**                                 error handling.
** 08/11/13  Fredrik Bredberg      Added zzkrn_mutex_destroy.
** 08/11/20  Fredrik Bredberg      Removed recursive locking and optimized
**                                 get_systime().
** 08/11/26  Gunnar Eklund         Fix an SMP problem with timer request
**                                 concurrently with tick(). Adjust
**                                 set_suid() to select the system pool as 
**                                 the home pool for user interrupt processes.
**                                 Spin-lock fixes in collect_pool() and
**                                 extend_pool(). 
** 08/12/02  Fredrik Bredberg      The debugger can now trace swap-in/out of
**                                 interrupt porcesses.
** 09/01/12  Fredrik Bredberg      Optimized alloc(), removed all asm stuff.
** 09/02/04  Fredrik Bredberg      Fixed a bug in stop_or_intercept(). Made
**                                 ose_get_curr_procs() official. Fixed a
**                                 redir problem in hunt().
** 09/02/16  Gunnar Eklund         Refuse to create interrupt processes in
**                                 user mode programs with MAS or a local,
**                                 dynamically extendable pool.
** 09/02/20  Mikael Wanggren       Added debug pattern fill support for
**                                 the system pool.
** 09/02/11  Gunnar Eklund         The old sourdough:
**                                 Reduce number of BKL lock regions.
**                                 Wide use of kernel event messages.
**                                 Finally handling SMP wakeup between cpus.
**                                 Added the per-cpu sysd_lite daemon.
**                                 Dynamic allocation of SMP interrupt stacks.
**                                 Removed SMP_ prefix in LOCK_SWAP() et al.
**                                 Changed SMP_ prefix to BKL_ in LOCK() et al.
**                                 Added missing LOCK region in alloc_pixcb().
** 09/03/09  Gunnar Eklund         Corrected handling of parent and st_cnt
**                                 members of a block in create_process() and
**                                 stop_one_proc(). 
**                                 Implemented reallocation of queued signals
**                                 from dying pools for SMP.
**                                 Added check of sender_id in unlink_attached.
**                                 Finally deleted obsoleted breakpoint code.
** 09/03/19  Gunnar Eklund         Minor bug fixes in zzkrn_bind_process().
**                                 Add prof to struct _block for use by rmm.
**                                 Stepping  MAGIC_KERNEL_TYPE.
** 09/03/23  Gunnar Eklund         Minor bug fix in zzkrn_send_hook_free_buf().
** 09/03/27  Fredrik Bredberg      Interrupt processes now end up in the
**                                 correct wakeup_list when migrated. Fixed
**                                 a send-debug-hook related bug.
** 09/04/09  Gunnar Eklund         SMP fix in send_finalize() to handle a
**                                 dying receiver. Optimize wake_up_usr()
**                                 for SMP. POOL_ALIGNMENT now decided in
**                                 runtime. Removed some BKL lock regions.
** 09/04/17  Fredrik Bredberg      Added zzkrn_save-/restore-_pcb_state()
**                                 which should be used during a pagefault.
** 09/05/15  Gunnar Eklund         Fix a problem with a lost attach signal
**                                 from a segment, introduced at 09/02/11.
** 09/05/26  Gunnar Eklund         Fix a bug in interrupt stack administration
**                                 for more than 2 SMP cpus.
** 09/05/26  Robin Westberg        Block link prefixes are now added
**                                 to process names in get_pid_info() and
**                                 get_pcb() outside the process' own block.
** 09/06/01  Robin Westberg        Hide processes in blocks with block links
**                                 from being found by hunt without the
**                                 prefix outside the process' own block.
** 09/05/26  Gunnar Eklund         Disable interrupts in tick when sending
**                                 KRN_EVT_RESCHEDULE.
** 09/06/16  Gunnar Eklund         Added public experimental calls
**                                 ose_smp_intercept_other_cpus() and
**                                 ose_smp_resume_other_cpus().
**                                 Now managing the background list in
**                                 sysd_lite on the local cpu.
**                                 Timeslicing of background processes
**                                 handled per cpu.
**                                 BKL_LOCK removed from zzgo_search.
**                                 Process migration now always performed by
**                                 sysd.
**                                 Reorganized flush() for SMP.
** 09/07/31  Gunnar Eklund         Minor update in kill_attach.
**                                 Improved assert code and ramlog output.
** 09/08/25  Gunnar Eklund         Ignore KRN_EVT_TIMEOUT events when
**                                 target no longer has tmo_active set,
**                                 i.e. when it has received a signal.
** 09/09/02  Gunnar Eklund         Added MEMBAR() in tick() for MIPS SMP
**                                 to fix syncronization with get_systime()
**                                 for Cavium targets. 
** 09/09/24  Fredrik Bredberg      Allow system daemons to commit suicide.
**                                 Optimized send_attach_signal() for SMP.
** 09/09/24  Gunnar Eklund         Use MEMBAR() in tick() for all SMP
**                                 architectures. Made kill_attach more
**                                 robust, by checking the owner of pre-
**                                 allocated attach signals. Removed the
**                                 dispatcher lock from the normal flow of
**                                 the send() call. Improved handling of
**                                 send from interrupt level to dying 
**                                 processes on another cpu. Split of
**                                 send_finalize into send_finalize and
**                                 locked_send_finalize. Introduced
**                                 likely() and unlikely(). Corrected return
**                                 value for get_pid_info(pool,9).
** 09/10/11  Robin Westberg        Added interrupt vector control to kernel,
**                                 and system calls for handling ISRs and
**                                 interrupt hooks.
** 09/10/23  Fredrik Bredberg      Improved extend pool mechanism for SMP.
**                                 Fixed a proper alloc_nil() implementation.
** 09/11/03  Fredrik Bredberg      Fixed a scheduler locking problem in
**                                 pool_brk_lo/hi.
** 09/11/06  Robin Westberg        Added zzkrn_evt_notify_from_isr and
**                                 a get_sysconf to fetch its entrypoint.
** 09/11/10  Gunnar Eklund         Correct SMP lock regions for the timeout
**                                 signal concept.
** 09/11/20  Robin Westberg        Added the system calls receive_fsem() and
**                                 ose_signal_fsem_from_isr().
** 09/11/25  Fredrik Bredberg      Fixed two signal leak problems in
**                                 common_hunt().
** 09/11/17  Gunnar Eklund         Let ose_bind_process return OK status
**                                 for some more unicore cases.
**                                 Changed the per_cpu called_in flag to state.
**                                 Inhibit running of processes on detached
**                                 cpus.
**                                 Minor optimization of signal_fsem from
**                                 interrupt level.
**                                 Some lock scheme changes to avoid deadlock,
**                                 in extend_realtime_pool, handle_error, 
**                                 receive_20.
** 09/12/04  Fredrik Bredberg      The zzuu.ticks is now a 64-bit variable,
**                                 also added sysconf 93 to read out the
**                                 address to it.
** 09/12/07  Robin Westberg        Unbreak receive_from and receive_with.
** 09/12/14  Gunnar Eklund         Corrected lock scheme in mutex_wait_sem.
**                                 SMP fixes in handling of tmolist and 
**                                 the tmo_active flag.
**                                 Canceling timer in rec_sig_dup to
**                                 avoid odd errors in pool extend hooks.
** 10/01/07  Gunnar Eklund         Distribute a tmolist to each cpu.
**                                 Safe_free_buf() now silently ignores 
**                                 buffers not owned by the current process.
** 10/01/28  Gunnar Eklund         Added system call s_alloc_nil().
**                                 Attach() can now operate on pool objects.
**                                 Receive_from() can now receive from a pool.
**                                 Starting a block with no process but 
**                                 with a signal pool disconnects the parent,
**                                 thus it survives when the parent dies.
** 10/02/12  Gunnar Eklund         safe_free_buf() now ignores signals with
**                                 wrong owner.
**                                 Corrected and simplified get_mem and 
**                                 set_mem called from a supervisor process.
**                                 Request_timeout() on cpuX now pays
**                                 attention to a pending tick() on cpu0.
**                                 Added assert code for signals and pools.
**                                 Added some more likely/unlikely.
** 10/02/15  Fredrik Bredberg      We can now create interrupt processes in
**                                 programs with MAS if the proper relax bit
**                                 (0x40) is set in relax_error_checks.
** 10/03/01  Gunnar Eklund         Added system call ose_set_pool_max_size().
** 10/03/03  Fredrik Bredberg      Added the ose_fsend()/polling receive
**                                 concept.
** 10/03/09  Gunnar Eklund         Added Counters for timer processes, 
**                                 pending timeouts, resp. pending
**                                 timeout signal requests, to speed up 
**                                 entry to the power save handler.
** 10/03/23  Fredrik Bredberg      Started using compiler.h for inline
**                                 macros. The receive_core_sig_queue()
**                                 function now handles burst receive.
**                                 Removed "unused param" warnings.
** 10/03/29  Fredrik Bredberg      Added a 'sender' parameter to ose_fsend().
**                                 Changed errorhandling of MAS processes
**                                 when relax bit 0x40 is not set.
** 10/04/16  Gunnar Eklund         Removed broadcast of kernel events in 
**                                 ose_smp_stop_cpu() and renamed to
**                                 zzkrn_smp_stop_cpu().
**                                 Relocate_queued_signals() now tries 
**                                 s_alloc_nil() to copy signals to the pool
**                                 of the addressee.
** 10/05/07  Gunnar Eklund         Use atomic_cas in mutex functions to
**                                 permit access over cpu borders.
**                                 Start kill-locked processes when trying
**                                 to kill them to let them finish.
**                                 Make get_pid_info more robust for pid 0
**                                 parameters.
**                                 Fixed a problem in sys_power_save.
** 10/06/09  Gunnar Eklund         mutex_wait_sem now called with BKL taken.
**                                 Fixed a NULL pointer exception in
**                                 background process timeslicing.
** 10/06/23  Fredrik Bredberg      Added support for external alloc/free_buf
**                                 hooks (i.e. hardware allocated buffers),
**                                 ose_set_pool_attr() and alloc_brk().
** 10/07/12  Gunnar Eklund         Corrected no_copy attribute check in 
**                                 all receive calls.
** 10/07/15  Robin Westberg        Added a version of ose_mutex_init that
**                                 returns an error instead of using os_error.
** 10/07/15  Gunnar Eklund         Refined the handling of sending to a
**                                 full kernel event queue.
** 10/09/08  Fredrik Bredberg      Added a new "unlocked" flavor of the send
**                                 hook, also made ose_fsend() send hook
**                                 aware.
** 10/09/29  Anders Hagstrom       Added call to evt_interrupt_generate.
**                                 Changed prototype for zzmsg_enqueue.
** 10/09/29  Anders Hagstrom       Added sysconf_101 and the hw_receive_poller.
** 10/10/29  Anders Hagstrom       Added new subcode to sysconf_101.
**                                 Renamed kernel event functions.
** 10/11/04  Gunnar Eklund         Added public ose_set_pool_extender().
**                                 Added three pool info tags to get_pid_info.
**                                 Separated handling of free fragment list 
**                                 for non realtime pools.
**                                 Detach now sets the attref to 
**                                 OSE_ILLEGAL_ATTREF instead of MAX_ATTACH.
**                                 Added name mangling of s_alloc_nil and 
**                                 get_pid_info.
** 10/11/16  Fredrik Bredberg      The main_block can now be found via
**                                 get_pid_info. Also fixed a few "pid
**                                 under construction" problems in
**                                 get_pid_info().
** 10/11/29  Gunnar Eklund         Fixed incomplete kill of a block with
**                                 processes running on more than one cpu. 
**                                 Added validation of common attributes
**                                 when allocating pix list objects from
**                                 the free lists.
** 11/01/31  Gunnar Eklund         Moved a call of zzkrn_evt_init() from
**                                 _zzmain.c into zzstart_kernel().
** 11/02/15  Gunnar Eklund         Minor adaptions to the new type atomic_t.
** 11/03/10  Gunnar Eklund         Remove LOCK/UNLOCK_SWAP() from s_alloc_nil
**           Fredrik Bredberg      for static pools.
**           Florin Rotaru
** 11/04/12  Gunnar Eklund         Added private block state hooks for use 
**                                 by the hypervisor.
**                                 Setting dp_idle_interrupted at ISR events.
**                                 Issue a KRN_EVT_RESCHEDULE in 
**                                 delete_segment().
** 11/05/02  Gunnar Eklund         ARM going multicore.
**                                 Moved MEMBAR definitions to pose.con.
** 11/05/06  Gunnar Eklund         Corrected pattern fill of the unicore
**                                 interrupt stack.
** 11/05/10  Fredrik Bredberg      Added limited/fake ose_bind_process()
**                                 functionality to non SMP environments.
** 11/05/12  Gunnar Eklund         Added some debug code for ARM MCE.
** 11/06/09  Fredrik Bredberg      Fixed an ose_fsend polling related bug.
** 11/06/14  Fredrik Bredberg      Fixed an ose_fsend migrating related bug.
** 11/06/21  Fredrik Bredberg      Fixed an alloc_nil() bug for
**                                 MEMORY_MODEL 0.
** 11/07/05  Gunnar Eklund         Corrected return value for user mode
**                                 call to select_segment.
**                                 Removed unneded interrupt generating in
**                                 send_krn_event().
** 11/07/20  Gunnar Eklund         With current fsend() model, the call of
**                                 zzkrn_evt_init() is only needed if OSE_SMP.
** 11/09/20  Gunnar Eklund         Fixed a memory leak of 32 bytes/program.
**                                 Removed non_rt_free_frag_list.
**                                 Shifted down pool extend_count.
** 11/10/05  Robert Andersson      Added sysconf_104 (invalidate tlb). 
** 11/10/13  Gunnar Eklund         Treating OSE_NO_TIMEOUT in receive_* calls
**                                 as an eternal timeout. Error report use
**                                 of OSE_NO_TIMEOUT in delay() and timeout
**                                 signal requests. In create_process(),
**                                 interpret OSE_NO_TIMEOUT depending on 
**                                 process type.
** 11/10/14  Fredrik Bredberg      Removed curtmo from world and per_cpu.
**                                 Fixed two scheduler bugs, one when
**                                 bg-procs are waking up from a delay()
**                                 or receive_w_tmo() and another in 
**                                 receive_core_sig_queue().
** 11/10/24  Gunnar Eklund         Split zzkrn_bind_process into one client
**                                 and one ose_sysd function. Permit a 
**                                 supervisor process to migrate an empty
**                                 block in the own segment even when it is
**                                 locked to a cpu.
** 11/10/27  Fredrik Bredberg      Fixed a problem with deleting blocks
**                                 in the system domain if there where
**                                 more than one segment in the system
**                                 domain.
** 11/10/28  Hakan Lindqvist       OSECR-12073 Fixed a race condition with 
**                                 assign_linkhandler() on block links.
**                                 Renamed various convolutions of "virtual
**                                 block link handler" as "block link".
** 11/10/28  Gunnar Eklund         Fix zzkrn_bind_process for softkernel.
** 11/11/04  Gunnar Eklund         Fixed signal relocation from dying pools
**                                 also for multicore sports.
**                                 Optimized get_pid_info 18=sig_cnt_in_queue.
**                                 Added get_pid_info tags 29-33.
**                                 Permitted user process to use s_alloc_nil
**                                 on a static pool.
** 11/12/02  Gunnar Eklund         Fix a problem with killing blocks with 
**                                 kill-locked and stopped/intercepted 
**                                 that will never die. 
** 11/12/16  Gunnar Eklund         Added a multicore membar in tick().
** 12/01/09  Gunnar Eklund         Changed change_sig_owner to decrement
**                                 sig_cnt_owned on cpu 0 for dead process.
** 12/01/18  Gunnar Eklund         Fixed a number of bugs in ose_fsend. See
**                                 readme for mor info. Minor fix of error
**                                 reporting in send_hook_free_buf and
**                                 send_hook_restore.
** 12/01/18  Gunnar Eklund         Removed check for signal in free list
**                                 in ose_fsend, since a signal allocated by
**                                 hardware still has the pool id in dest_id.
** 12/04/03  Gunnar Eklund         Minor optimizations.
** 12/04/11  Gunnar Eklund         Minor optimizations.
** 12/04/25  Gunnar Eklund         Added an option to force exception at
**                                 other cpu when kernel event queue full.
** 12/05/14  Gunnar Eklund         Reorganized struct _pcb for ARM to
**                                 increase performance.
** 12/06/12  Fredrik Bredberg      Fixed a nasty signal relocation bug.
**                                 Minor typecast improvements.
** 12/06/19  Gunnar Eklund         Improved error information for corrupted
**                                 signal control blocks.
** 12/09/11  Gunnar Eklund         Added call of rmm zzsystem_error_dbghook.
** 12/09/11  Gunnar Eklund         Optimized zzgo_search. Credit: anha.
** 12/10/03  Gunnar Eklund         Made s_pcb.take_listmodnumber volatile.
** 12/11/26  Gunnar Eklund         Added an argument to zzsystem_error_dbghook.
**                                 Learned extend_pool to extend when creating
**                                 a process stack in another program.
** 13/01/08  Gunnar Eklund         Changed LOCK to LOCK_SAVE in signal_fsem.
** 13/01/23  Mahesh Patil          Use pcb_to_debug_pcb macro in the 
**                                 zzsystem_error_dbghook function call.
** 13/01/30  Gunnar Eklund         Added configuration options for disposition
**                                 of queued signals in killed pools.
** 13/03/05  Gunnar Eklund         Added ose_get_error_info() and
**                                 ose_reset_error_info().
**                                 Corrected next_pid_value().
** 13/06/14  Fredrik Bredberg      Make free_attach() more "nice".
** 13/08/15  Fredrik Bredberg      Added missing error information in
**                                 zzkrn_mutex_destroy().
**                                 Simplified the delay() timeout handling.
** 13/08/27  Gunnar Eklund         Some more error checks in change_sig_owner.
** 13/12/30  Gunnar Eklund         Corrected handling of user_called errors
**                                 0x2, 0x12 and 0x20.
** 14/01/07  Fredrik Bredberg      Fixed a nasty bug in expire_timeouts().
** 14/01/31  Gunnar Eklund         Added error reports for 
**                                 OSE_ECONCURRENT_TICK_CALL and multicore
**                                 KRN_EVT_TICK out of order.
*/

/*
*****************************************************************************
** 3  HEADER FILES.
*****************************************************************************
*/

/*
**===========================================================================
** 3.1  Internal definitions.
**===========================================================================
*/

/* The product id of the pose product for traceability. */

#define POSEREV "2.182.19"

/* Define if get_mem error checks 2 included in send hooks or not. */
#define SEND_HOOK_GET_MEM_CHECK 0
/* Define if the power off hook is called from zzgo_search or not.  */
#define POWERFAIL_HOOK_CALLED 0

#if 0
#define ASSERT
#define GUNNAR_DEBUG 
#endif
#define LESO_DEBUG 0
#define LESO_PRINTF(...) if(LESO_DEBUG) ramlog_printf("pose: " __VA_ARGS__)

/* 
** ?? REMAINING MULTICORE PROBLEMS:
**	- Reenable power_save for smp, this will affect both target
**	  power and simics performance, and eliminate some IPIs.
**	- mutex_raise_priority not implemented for other cpu.
**	  (well, really coded, but disabled because not yet tested)
**	- krnstress test:
**        User stack overflow error handling (works several 100 times 
**	  but eventually fails, varying errors but often an illegal pid,
**	  when running on cpu1. Even if no processes are migrated.
**	  Fredrik detected that the problem is trigged by locked dispatcher.)
**	- Interrupt process specific code assumes that no process can die,
**	  but that is only true for processes on the current cpu. A known
**	  spot is the reading of redirection tables in send() calls. Also
**	  consider set_redirection(). The issue is LOCK_SWAP and
**	  SILENT_UNLOCK_SWAP().
**	  
** IMPROVEMENTS to consider:
**      - Use a free fragment list per cpu (i.e. remove BKL).
**	- public int ose_started() (0 == system calls not available)
**	- The dispatcher lock has got a new role. Replace by a R/W lock?
**      - Optimize sports and semaphores.
*/


/* Make sure we don't use debug versions of system calls internally. */

#ifdef OS_DEBUG
#undef OS_DEBUG
#endif

#include "osenames.h"		/* Names for all public symbols.	*/

#include "ose_rem.h"		/* OSE standard headers.		*/
#include "ose_smp.h"		/* OSE standard headers.		*/
#include "core_internal.h"	/* Internal Core header.		*/

/* START_SIGDESC */
#include "ose_mmu.h"
#include "ose_err.h"

#include "pose.con"		/* Configurable types and constants.	*/
#include "pose.h"		/* Target interface.		 	*/
#include "pose_internal.h"	/* Local Manifest constants and types.  */

#include "atomic.h"
#ifdef OSE_SMP
#include "spinlock.h"
#endif
#include "compiler.h"

#include "ramlog.h"
/* END_SIGDESC */

/*
**===========================================================================
** 3.2  Libraries.
**===========================================================================
*/

#include <stddef.h>		/* ANSI C compiler library.	*/
#include <string.h>		/* ANSI C compiler library.	*/

/*
*****************************************************************************
** 4  LOCAL DEFINITIONS.
*****************************************************************************
*/

/*
**===========================================================================
** 4.1  Manifest constants.
**===========================================================================
*/

/*
** The NULL pointer is required in several places.
**
** We define it here in case it is not defined elsewhere.
** (We would like to get rid of the ANSI headers some day for
** portability reasons.?? )
*/

#ifndef NULL
#define NULL (void *)0
#endif

/*
** These defines are needed until we have taken the big step of
** removing the two members instance and pindex from all
** proc/block/seg/pool control blocks.
*/

#define PCB_INSTANCE_SHIFT 16
#define PCB_INDEX_MASK 0x000FFFFu
#define live_pid pid

/*
**===========================================================================
** 4.2  Type definitions.
**===========================================================================
*/

/* This enum sets up the flags that can be used to send information to 
 * stop_or_intercept() on which types of processes are to be exempt from
 * stopping or intercepting in this call.
 * Several flags can be used at the same time.
 */
typedef enum
  {
    BUT_CURRENT_PROC   = 0x1,	/* Do not stop the current process. */
    BUT_INTERRUPT_PROC = 0x2,	/* Do not stop (timer) interrupt processes. */
    BUT_SYSTEM_PROC    = 0x4,	/* Do not stop system processes. */
    BUT_KILL_LOCKED    = 0x8	/* Do not stop kill locked processes. */
  } stop_or_intercept_bit_mask;

/*
** The test error handler hook is installed through sysconf 68.
*/

typedef OSBOOLEAN (*SYSERR_HOOK) (OSBOOLEAN user_called,
                                  OSERRCODE ecode,
                                  OSERRCODE extra);

/*
**===========================================================================
** 4.2.1  General type definitions.
**===========================================================================
*/

/* START_SIGDESC */


/*
**===========================================================================
** 4.2.2  The symbol root type used to generate symbols with confsig.
**===========================================================================
*/

/*
** This section is used in conjunction with the OSE confsig program to
** generate the symbol map in the post mortem support table.
**
** The extra structure level is needed since the confsig program strips the
** first field from each top level struct, because it thinks that this is
** a signal number.
**
** See also: zzosvarp.
*/

#define SYMROOT	(1)	/* !-SIGNO(struct symroot)-! */

struct symroot
{
  SIGSELECT sig_no;
  struct world world;
  struct _pcb pcb;
  struct _block block;
  struct poolcb poolcb;
  struct segcb segcb;
  struct stkcb stkcb;
  struct _signal signal;
  struct semcb semcb;
  struct breakcb breakcb;
  struct env_buf env_buf;
  struct _attach_entry attach_entry;
  struct pool_extension pool_extension;
  struct sysctlblk sysctlblk;
};

/* END_SIGDESC */

/*
**===========================================================================
** 4.3  Macros.
**===========================================================================
*/

/*
** Max no of processes.
**
** This constant determines the size of the process index translation table.
** It also implicitely defines the size of the PCB_INDEX_T type, since this
** type must be large enough for the highest possible pcb_index.
**
** See also: zzuu.pix_list, zzuu.max_proc.
*/

#define MAX_PROC zzuu.max_proc

/*
** Max no of attached signals.
**
** This constant determines the maximum number of active attach calls in the
** system.
**
** See also: attach_pool, zzuu.max_attach.
*/

#define MAX_ATTACH zzuu.max_attach

/*
** Min no of attached signals.
**
** The free list of attach records should never be empty.
** Otherwise, we would reuse old entries too quickly.
**
** See also: attach_pool, zzuu.max_attach.
*/

#define MIN_ATTACHED (20)

/*
** Max size of the timeout lists, tmolist and tos_tick_list.
*/

#define MAX_TIMEOUT_ARRAY (8*1024)
/*
** Create a process id that seems to have been killed.
**
** This id is returned from create_process() when process creation failed
** and errors were ignored by an error handler.
**
** The id is created by adding 1 to the instance number of the idle
** process who has index 1.
**
** If PCB_INDEX_MASK is 0xFFFF (and PCB_INSTANCE_SHIFT is 16)
** any_killed_pid will get value 0x20001.
*/

#define any_killed_pid() ((PROCESS) (2*(PCB_INDEX_MASK+1)+1))

/*
** Convert a pcb pointer to a process id.
**
** The most significant bits of the process id is the instance number.
** The least significant bits is the process index.
*/

#ifndef LARGE_PID_SPACE
#ifdef ARM_RVCT_OPT_PCB_TO_PID
/* ARM V7 RVCT little endian version to avoid external function call. */
#define pcb_to_pid(pcb) (*(PROCESS*)(&(pcb)->pindex))
#else
#define pcb_to_pid(pcb) ((((PROCESS)((pcb)->instance))<<PCB_INSTANCE_SHIFT) | \
			 ((PROCESS)((pcb)->pindex)))
#endif
#else
#define pcb_to_pid(pcb) ((pcb)->pid)
#endif

/* Convert pcb pointer to debug_pcb pointer */
#ifdef OSARM
#define pcb_to_debug_pcb(pcb) ((struct debug_pcb *) &(pcb)->pid)
#else
#define pcb_to_debug_pcb(pcb) ((struct debug_pcb *) &(pcb)->debug_flags)
#endif

/*
** Extract instance number or process index from a process id or pcb
** pointer.
**
** The most significant bits of the process id is the instance number.
** The least significant bits is the process index.
*/

#define pid_to_instance(pid) ((PCB_INSTANCE_T)((pid) >> PCB_INSTANCE_SHIFT))

#ifndef LARGE_PID_SPACE
#define pid_to_pindex(pid)   ((PCB_INDEX_T)   (pid))
#define pcb_to_instance(cb)  ((cb)->instance)
#define pcb_to_pindex(cb)    ((cb)->pindex)
#else
#define pid_to_pindex(pid)   ((PCB_INDEX_T)   ((pid) & PCB_INDEX_MASK))
#define pcb_to_instance(cb)  ((PCB_INSTANCE_T)((cb)->pid>>PCB_INSTANCE_SHIFT))
#define pcb_to_pindex(cb)    ((PCB_INDEX_T)   ((cb)->pid) & PCB_INDEX_MASK)
#endif

/*
** Extract a pcb pointer from a process id.
*/

#define pid_to_pcb(pid) zzuu.pix_list[pid_to_pindex(pid)]
#define pid_to_segcb(pid) ((!pid) ? &system_segment : \
			   (struct segcb *)zzuu.pix_list[pid_to_pindex(pid)])

/*
** Construct a ready list mask for a prioritized process (See zzdp_look_aside)
*/
#define RLIST_PRI_MASK(pcb) (0x80000000U>>(pcb)->priority)

/*
** Lock and unlock the dispatcher. No process switches
** will occur between the LOCK_SWAP() and UNLOCK_SWAP() macros.
** The zzww.dp_req flag may be set by either an interrupt routine or
** the current non interrupt process to request a process switch.
** If zzww.dp_req is set at the time of UNLOCK_SWAP(), a process switch
** immediately occurs.
**
** The SILENT_UNLOCK_SWAP() macro is used instead of UNLOCK_SWAP()
** when interrupts are disabled at the time, and we don't want them
** reenabled. In this case, the caller must reenable interrupts at a
** later time with the CHECK_UNLOCK_SWAP() macro, which also checks
** zzww.dp_req and swaps if required.
**
** LOCKED_UNLOCK_SWAP() is an UNLOCK_SWAP() for situations where it
** is known that interrupts are disabled.
**
** These macros are used mostly to make sure that no
** processes are killed during the region enclosed between
** LOCK_SWAP() and UNLOCK_SWAP().
**
** There is an important feature in this mechanism.
** If a process uses LOCK_SWAP(), then makes itself not ready and
** calls zzquit_int() to do a swap, then the swap is postponed
** until the UNLOCK_SWAP macro is used.
*/

#ifdef OSE_SMP

#define LOCK_SWAP() zzsmp_lock_swap()
#define SILENT_UNLOCK_SWAP() zzsmp_unlock_swap()

#else  /* ! OSE_SMP */

#define LOCK_SWAP() do {			\
    zzuu.dispatcher_lock_flag = (OSBOOLEAN) 1;	\
  } while (0)

#define SILENT_UNLOCK_SWAP() do {		\
    zzuu.dispatcher_lock_flag = (OSBOOLEAN) 0;	\
  } while (0)

#endif  /* ! OSE_SMP */


#define UNLOCK_SWAP() do {			\
    LOCK();					\
    SILENT_UNLOCK_SWAP();			\
    if(zzww.dp_req)				\
      {						\
	zzquit_int(); /* Swap. */		\
      }						\
    UNLOCK();					\
  } while (0)

#define LOCKED_UNLOCK_SWAP() do {		\
    SILENT_UNLOCK_SWAP();			\
    if(zzww.dp_req)				\
      {						\
	zzquit_int(); /* Swap. */		\
      }						\
    UNLOCK();					\
  } while (0)


#define CHECK_UNLOCK_SWAP() do {		\
    if(zzww.dp_req)				\
      {						\
	zzquit_int(); /* Swap. */		\
      }						\
    UNLOCK();					\
  } while (0)


/*
** These macros are used to move back and forth between a signal administration block
** and the corresponding data field in user space. No checks are performed.
**
** The macros are needed because signals are stored differently depending on the
** selected memory model.
**
** For memory model 0, signal administration blocks are located immediately
** before the data buffer in user space. This scheme minimizes overhead and
** maximizes performance.
**
** For memory model 1 and above, signal administration blocks are allocated
** separately from the system pool, while data space is located in the user pool.
** There is also a control block in the user pool, but this block contains
** only a magic_number and a backward pointer to the real control block
** in supervisor space.
*/

#if MEMORY_MODEL > 0
#define sig_to_adm(sig) ((struct _signal *) (((struct _signal *)(sig))-1)->udata)

#define adm_to_sig(adm) ((union SIGNAL *) (((struct _signal *)(adm))->udata))

#else

#define sig_to_adm(sig) (((struct _signal *) (sig)) - 1)

#define adm_to_sig(adm) ((union SIGNAL *) (((struct _signal *) (adm)) + 1))

#endif

/*
** These macros are used in various error checks to test if the check
** should be performed. The macros test on the error_level variables
** maintained by the system debugger.
*/

#if ERROR_CHECKS == 0

#define ON_ERRLEVEL_2() if(0)
#endif

#if ERROR_CHECKS == 1
#define ON_ERRLEVEL_2() if(unlikely(zzuu.error_level2))
#endif

#if ERROR_CHECKS == 2
#define ON_ERRLEVEL_2() if(1)
#endif

/*
** We need these macros to ensure that we don't break the strict aliasing
** rules of the C language.
*/
#define sys_free_buf_type(t, b) \
	do \
	{ \
	   union U \
	   { \
	      union SIGNAL **s; \
	      t *x; \
	   } u; \
	   u.x = b; \
           change_sig_owner(sig_to_adm(*(u.s)), zzcur_p);	\
	   free_buf(u.s); \
	} while(0)

#define zzsys_send_w_s_type(t, b, f, o) \
	do \
	{ \
	   union U \
	   { \
	      union SIGNAL **s; \
	      t *x; \
	   } u; \
	   u.x = b; \
           zzsys_send_w_s(u.s, (f), (o)); \
	} while(0)

#define	CAST(type, var) \
  ((type)(ptrdiff_t)(var))

#define	DECONST(type, var) \
  ((type)(ptrdiff_t)(const void *)(var))

#define	DEVOLATILE(type, var) \
  ((type)(ptrdiff_t)(volatile void *)(var))

/*
**===========================================================================
** 4.4  Signal composition.
**===========================================================================
*/

/*
** The following section defines internal OSE signals used between
** client processes and the OSE daemons.
** These signals are placed in the range 31900-31910.
*/

/*
** The hunt_request signal is sent from a client process to the hunt daemon
** to execute a hunt() or hunt_from() system call.
** The system call is executed by the hunt daemon, and the result is
** returned to the client in a remote_response signal containing a
** remr_hunt structure.
**
** The fields sig_no, sig, user, from, owner and name are defined
** by the client.
** Other fields are used for internal housekeeping by the hunt daemon.
*/

#define HUNT_REQUEST	(31900)	/* 0x7c9c */

struct hunt_request
{
  SIGSELECT sig_no;
  struct _signal *hsig;	/* Pointer to user provided hunt sigadm if any. */
  OSUSER user;		/* user field in hunt call. */
  PROCESS from;		/* from field in hunt_from call. */
  PROCESS owner;	/* caller of the hunt call. */
  treenode T_name;	/* Search tree nodes used in the list of pending hunts. */
  treenode T_linkname;
  treenode T_owner;
  treenode T_from;
  char name[1];	/* Variable size */
};

/*
** The new_linkhandler signal is sent from a client process to the hunt daemon
** to execute an assign_linkhandler() system call.
**
** NOTE: The reply signal is a bit further down in the file...
**
** This signal buffer lives on in the linkhandler search tree as long as
** the linkhandler exists.
**
** This is basically wrong since assign_linkhandler now returns a flag set if the ??
** linkhandler could not be created. However, there seem to be a movement towards
** the old specification where several linkhandlers were allowed to have the
** same name. No change is required if this is the case.
*/

#define NEW_LINKHANDLER (31901)	/* 0x7c9d */

struct new_linkhandler
{
  SIGSELECT sig_no;
  PROCESS new_linkhdlr;
  OSUSER user;          /* User number of the new linkhandler. */
  treenode T_link_name; /* Search tree nodes used in list of linkhandlers. */
  treenode T_pid;
  int expecting_reply;  /* Flag set if the client expects a reply sig.
                        ** NOTE: offsetof this field must be after sizeof
                        ** new_linkhandler_reply */
  OSADDRESS link_name_len; 
  char link_name[1];	/* Variable size. */
};

/*
** The new_process signal is sent from a client process to the system daemon
** to execute a create_process() system call.
** The resulting process id is returned to the client in a response signal
** of the same type (only the pid and error fields are valid).
**
** The new_process signal is also sent from the system daemon to the hunt
** daemon. It is processed by the hunt daemon at a later time.
** No signal is returned to the system daemon. 
**
** The new_process signal is also sent from the client process to the hunt 
** daemon when a block is created. It is processed by the hunt daemon at a 
** later time. No signal is returned to the sender.
**
** The pid field is set by the system daemon and examined by the client and by
** the hunt daemon. The ecode and extra fields are set by the system daemon
** and examined by the client. The block field is set by the client process 
** and examined by the system daemon and the hunt deamon. 
** All other fields are set by the client process and examined by the
** system daemon.
**
*/

#define NEW_PROCESS (31902)	/* 0x7c9e */

struct new_process
{
  SIGSELECT sig_no;
  PROCESS pid;		/* Resulting proces id. */
  OSERRCODE ecode;	/* Set if error occurred. */
  OSERRCODE extra;	/* More info to os_error(). */
  /* create_parameters. */
  enum PROCESS_TYPE proc_type;
  char *name;		/* Name buffer allocated in system pool. */
  OSENTRYPOINT *entrypoint;
  struct stkcb *stack;	/* stack allocated by client. */
  OSADDRESS stack_size;
  OSPRIORITY priority;
  OSTIME tmo;
  PROCESS block;	/* Id of parent block. */
  struct OS_redir_entry *redir; /* Buffer allocated by client. */
  U32 proc_attrs;	/* Process attributes for signal ports etc from the redir table. */
  OSVECTOR vector;
  OSSENDHOOK *snd_hook;
  void *lnh_dest_hd;
};

/*
** The delete_process signal is sent from a client process to the system daemon
** to execute a kill_proc() system call.
** No signal is returned to the client.
**
** The delete_process signal is also sent from the system daemon to the hunt
** daemon.
** The hunt daemon deletes the process from the process list and 
** removes it from other lists managed by the hunt daemon.
** Finally the hunt daemon frees the pcb.
** No signal is returned to the system daemon.
**
** The pid field is set by the client and examined by the system daemon and by
** the hunt daemon. All other fields are set by the client process and
** examined by the system daemon.
*/

#define DELETE_PROCESS (31903)	/* 0x7c9f */

struct delete_process
{
  SIGSELECT sig_no;
  PROCESS pid;
};

/*
** The new_uid signal is sent from a client process to the hunt daemon
** to reexecute pending hunt requests when a set_uid() system call ??
** has been executed.
** No signal is returned to the client.
**
*/

#define NEW_UID (31904)	/* 0x7ca0 */

struct new_uid
{
  SIGSELECT sig_no;
  PROCESS pid;		/* Process with new user number. */
  OSUSER user;		/* New user number. */
};

/*
** Signal to tell the system daemon that a garbage collection may be required.
** garbage collection involves scanning all known pools for a pool that
** no longer has any users.
** There is no data field in this signal.
**
** See also: sysd_do_free_pool(), sysd_check_free_pool().
*/

#define START_GARBAGE_COLLECTOR (31905)	/* 0x7ca1 */

/*
** Signal sent from the system daemon to the LDM during system start.
** The ldm extracts the start mode (warm or cold) from this signals,
** initializes ldm operation and sends a response to the system daemon.
** The system daemon then starts the application.
**
** The mode field contains 0 for cold start and 1 for warm start.
*/

#define START_MODEFLAG (31906)	/* 0x7ca2 */

struct start_modeflag
{
  SIGSELECT sig_no;
  U8 mode;
};

/*
** Signal to tell the system daemon that a breakpoint was reached.
** The system daemon then scans the breakpoint list and removes
** any tripped breakpoints.
** There is no data field in this signal.
*/
/* ===DEPRECATED=== */
#define KILL_BREAK (31907)	/* 0x7ca3 */

/*
** Signal sent from kill_proc to the system daemon to kill an entire segment.
** The system daemon issues a kill request for all blocks in the segment.
**
** This signal can not be larger than the DELETE_PROCESS signal.
** See kill_proc().
*/

#define KILL_SEG_REQ (31908)	/* 0x7ca4 */

struct kill_seg_req
{
  SIGSELECT sig_no;
  PROCESS segid;
};

/*
** Signal sent from a client to the system daemon to request a 
** re-bind of a process or block to a cpu. The signal is used
** to re-bind the currently executing process or block.
**
** The pid is never illegal, but may be dead.
** Sysd returns the the signal with member status set.
*/

#define SMP_BIND_PROCESS (31909) /* 0x7ca5 */

struct smp_bind_process
{
  SIGSELECT sig_no;
  PROCESS pid;
  cpuid_t cpu;
  OSADDRESS status;
};

/*
** Signal sent from a client to the sysd_lite daemon to request a 
** kill_proc to be carried out on the cpu of the victim.
** The victim is a block or a process, not a segment.
**
** The pid is never illegal, but may be dead.
** Sysd_lite returns a reply signal when ready. This may cause
** priority inversion??
*/

#define SMP_KILL_PROC_REQUEST (31910) /* 0x7ca6 */
#define SMP_KILL_PROC_REPLY (31911) /* 0x7ca7 */

struct smp_kill_proc
{
  SIGSELECT sig_no;
  PROCESS pid;
};

/*
** Signal sent from sysd to the sysd_lite daemon to request a 
** new interrupt stack to be allocated on the cpu of the receiver.
** The signal is sent back to sysd to confirm that the operation is completed.
*/

#define SMP_ALLOC_INT_STACK (31912) /* 0x7ca8 */

struct smp_alloc_int_stack
{
  SIGSELECT sig_no;
  OSADDRESS stack_size;
};

/*
** Signal sent from sysd to the sysd_lite daemon to request a 
** send of an attach signal to an owner on another cpu.
** The signal is sent back to sysd when handled.
*/

#define SMP_META_ATTACH (31913) /* 0x7ca9 */

struct smp_meta_attach
{
  SIGSELECT sig_no;
  struct _attach_entry *attcb;	/* Attach entry to check, verify matching
				** instance with member below. Discard the
				** attach signal if instance differs!
				*/
  struct _signal *att_sig;	/* Sigadm pointer to the attach signal. */
  PROCESS owner;
  PROCESS victim;
  U16 instance;			/* The proper attcb instance. */
};

/*
** Signal sent to the sysd_lite daemon to request a free_buf of a 
** signal sent to a dead process.
*/

#define SMP_FREE_ME (31914) /* 0x7caa */

struct smp_free_me
{
  SIGSELECT sig_no;
};

/*
** Signal sent to the sysd_lite daemon to request a flush of a 
** signal queue.
*/

#define SMP_FLUSH_REQUEST (31915) /* 0x7cab */
#define SMP_FLUSH_REPLY (31916) /* 0x7cac */

struct smp_flush
{
  SIGSELECT sig_no;
  PROCESS pid;
  PROCESS psel[1];
};

/*
** Signal sent to the sysd_ daemon to request distribution of attach signals
** for a deleted segment.
*/

#define KILL_ATTACH (31917) /* 0x7cad */

struct kill_attach
{
  SIGSELECT sig_no;
  PROCESS pid;
};

/*
** Dispose sent and queued signals from a dying pool.
** The signal is sent by sysd_ to sysd_lite_ on other cpus.
** sysd_lite_ responds with the disposal_count filled in.
** The signals are either relocated (moved to another pool) or just killed
** (unlinked and freed). The purpose is to be able to deallocate the memory
** occupied by the pool.
*/

#define SMP_DISPOSE_QUEUED_SIGNALS_REQUEST (31918) /* 0x7cae */
#define SMP_DISPOSE_QUEUED_SIGNALS_REPLY (31919) /* 0x7caf */

struct smp_dispose_queued_signals
{
  SIGSELECT sig_no;
  PROCESS poolid;
  U32 disposal_count;
};

/*
** Request signals sent from sysd to sysd_lite to request
** updates of the cpu specific background process lists
** and timeout lists.
** SMP_PROC_MOVE_REQUEST is sent by zzkrn_bind_process()
** and the addressee is the current cpu of the process.
** SMP_PROC_ADD_REQUEST is sent by sysd_create_process.
** Both requests are always confirmed by one SMP_PROC_ADD_REPLY to sysd.
** An SMP_PROC_MOVE_REQUEST to one sysd_lite is usually 
** transformed to an SMP_PROC_ADD_REQUEST and sent to
** another sysd_lite, that will answer directly to sysd.
**
** 
** 
*/

#define SMP_PROC_MOVE_REQUEST (31920) /* 0x7cb0 */
#define SMP_PROC_ADD_REQUEST (31921) /* 0x7cb1 */
#define SMP_PROC_ADD_REPLY (31922) /* 0x7cb2 */

struct smp_move_proc
{
  SIGSELECT sig_no;
  PROCESS pid;
  cpuid_t cpu;
  OSTIME tmo_fragment;	    /* Only used in SMP_BG/PRI_PROC_ADD_REQUEST */
  OSBOOLEAN request_tmo;    /* Only used in SMP_BG/PRI_PROC_ADD_REQUEST */
};

/*
** A signal sent from sysd to sysd_lite. sysd_lite returns the signal.
** For test purpose.
*/
#define SMP_SYNC (31923) /* 0x7cb3 */

/*
** Signal sent from a client to the sysd daemon to perform the call
** ose_vector_disable in an SMP system.
*/

#define SMP_DISABLE_VECTOR_REQUEST (31924) /* 0x7cb4 */
#define SMP_DISABLE_VECTOR_REPLY (31925) /* 0x7cb5 */

struct smp_disable_vector
{
  SIGSELECT sig_no;
  OSVECTOR vec;
};

/*
** Signal sent from a client to the sysd daemon, or from sysd to
** sysd_lite, to perform the call ose_kill_isr or
** ose_kill_interrupt_hook in an SMP system.
*/
#define SMP_KILL_ISR_REQUEST (31926) /* 0x7cb6 */
#define SMP_KILL_ISR_REPLY (31927) /* 0x7cb7 */

struct smp_kill_isr
{
  SIGSELECT sig_no;
  OSVECTOR vec;
  uint8_t slot;
  uint16_t generation;
  uint8_t type;
};

/*
** Signal sent from a client to the sysd daemon to perform the call
** ose_enable_vector in an SMP system.
*/
#define SMP_ENABLE_VECTOR_REQUEST (31928) /* 0x7cb8 */
#define SMP_ENABLE_VECTOR_REPLY (31929) /* 0x7cb9 */

struct smp_enable_vector
{
  SIGSELECT sig_no;
  OSVECTOR vec;
  OSBOOLEAN success;
};

/*
** Signal sent from a client to the sysd daemon, or from sysd to
** sysd_lite, to perform the call ose_create_isr or
** ose_create_interrupt_hook in an SMP system.
*/
#define SMP_CREATE_ISR_REQUEST (31930) /* 0x7cba */
#define SMP_CREATE_ISR_REPLY (31931) /* 0x7cbb */

struct smp_create_isr
{
  SIGSELECT sig_no;
  OSVECTOR vec;
  OSERRCODE error;
  uint32_t handle;
  PROCESS owner;
  void *isr;
  void *arg;
  uint8_t type;
  cpuid_t cpu;
};

/*
** Signal sent from a client to the sysd daemon to perform the call
** ose_bind_vector in an SMP system.
*/
#define SMP_BIND_VECTOR_REQUEST (31932) /* 0x7cbc */
#define SMP_BIND_VECTOR_REPLY (31933) /* 0x7cbd */
struct smp_bind_vector
{
  SIGSELECT sig_no;
  OSVECTOR vec;
  cpuid_t cpu;
  OSERRCODE success;
};

/*
** Signal sent from a client to the sysd daemon to perform the call
** ose_unbind_vector in an SMP system.
*/
#define SMP_UNBIND_VECTOR_REQUEST (31934) /* 0x7cbe */
#define SMP_UNBIND_VECTOR_REPLY (31935) /* 0x7cbf */
struct smp_unbind_vector
{
  SIGSELECT sig_no;
  OSVECTOR vec;
  cpuid_t cpu;
  OSERRCODE success;
};

/*
** Signal sent from sysd to sysd_lite to perform the call to
** zzenable_vector in the target interface on the correct core.
*/
#define SMP_ZZENABLE_VECTOR_REQUEST (31936) /* 0x7cc0 */
#define SMP_ZZENABLE_VECTOR_REPLY (31937) /* 0x7cc1 */
struct smp_zzenable_vector
{
  SIGSELECT sig_no;
  OSVECTOR vec;
  OSPRIORITY pri;
};

/*
** Signal sent from sysd to sysd_lite to perform the call to
** zzdisable_vector in the target interface on the correct core.
*/
#define SMP_ZZDISABLE_VECTOR_REQUEST (31938) /* 0x7cc2 */
#define SMP_ZZDISABLE_VECTOR_REPLY (31939) /* 0x7cc3 */
struct smp_zzdisable_vector
{
  SIGSELECT sig_no;
  OSVECTOR vec;
};


/*
** The new_linkhandler_reply signal is sent from the hunt daemon to a
** client process that has requested assign_linkhandler.
**
** The main purpose of the signal is to synchronise the client with the
** hunt daemon. Otherwise, the client would potentially create processes
** in the linkhandler block before the assignment is committed. (OSECR-12073)
**
** Possible error codes are returned to the client. The intention is that
** errors should be reported from the client process, not the daemon.
** Errors are only reported when assigning link handler paths for blocks. 
*/
#define NEW_LINKHANDLER_REPLY (31940) /* 0x7cc4 */
struct new_linkhandler_reply
{
  SIGSELECT sig_no;
  OSERRCODE ecode;      /* Set if error occurred, otherwise 0 */
  OSERRCODE extra;      /* More info to os_error. */
};

/*
** The union of all signals known to the system processes.
**
** Any OSE process must define this union. The definition in this file
** applies to the system processes "tickd", "huntd" and "sysd".
*/

union SIGNAL
{
  SIGSELECT sig_no;
  struct remote_call remote_call;
  struct remote_response remote_response;
  struct remote_cancel remote_cancel;
  struct hunt_request hunt_request;
  struct new_linkhandler new_linkhandler;
  struct new_process new_process;
  struct delete_process delete_process;
  struct new_uid new_uid;
  struct start_modeflag start_modeflag;
  struct kill_seg_req kill_seg_req;
  struct smp_bind_process smp_bind_process;
  struct smp_kill_proc smp_kill_proc;
  struct smp_alloc_int_stack smp_alloc_int_stack;
  struct smp_meta_attach smp_meta_attach;
  struct smp_free_me smp_free_me;
  struct smp_flush smp_flush;
  struct kill_attach kill_attach;
  struct smp_dispose_queued_signals smp_dispose_queued_signals;
  struct smp_move_proc smp_move_proc;
  struct smp_disable_vector smp_disable_vector;
  struct smp_kill_isr smp_kill_isr;
  struct smp_enable_vector smp_enable_vector;
  struct smp_create_isr smp_create_isr;
  struct smp_bind_vector smp_bind_vector;
  struct smp_unbind_vector smp_unbind_vector;
  struct smp_zzenable_vector smp_zzenable_vector;
  struct smp_zzdisable_vector smp_zzdisable_vector;
  struct new_linkhandler_reply new_linkhandler_reply;
};


/*
*****************************************************************************
** 5  EXTERNAL REFERENCES.
*****************************************************************************
*/

/*
**===========================================================================
** 5.1  Functions.
**===========================================================================
*/

/*
**===========================================================================
** 5.2  Variables.
**===========================================================================
*/

/*
**===========================================================================
** 5.3  Forward references.
**===========================================================================
*/

static OSADDRESS handle_error(int fatal, OSERRCODE ecode, OSERRCODE extra,
			      OSBOOLEAN user_called);

static void set_uid(PROCESS pid, OSUSER user);

static void init_attach(void);

#if USE_REMOTE_CALLS
static union SIGNAL *make_remote_call(union SIGNAL *sig,
				      OSCALLTYPE response_type,
				      PROCESS handler);
#endif

static struct _pcb *check_pid_bid_seg(PROCESS pid, OSERRCODE errcode);
static struct _pcb *check_pid_bid(PROCESS pid, OSERRCODE errcode);
static struct _pcb *check_pid(PROCESS pid, OSERRCODE errcode);
static struct _block *check_bid(PROCESS bid, OSERRCODE errcode);

static struct _pcb *safe_pid_to_pcb(PROCESS pid, int skip_magic_check);
static struct segcb *find_segcb(struct _pcb *pcb);

static void add_to_pix_list(struct _pcb *pcb);
static void bump_pcb_instance(struct _pcb *pcb);
static PROCESS next_pid_value(PROCESS pid);
static union large_cb *alloc_pcb(OSERRCODE subcode);
static struct poolcb *alloc_poolcb(void);
static void free_pcb(void *pcb_p);
static void free_poolcb(struct poolcb *poolcb_p);

static struct stkcb * stack_alloc(struct poolcb *inpool, OSADDRESS size);
static void stack_free(struct stkcb *stack);

static struct poolcb *get_home_sigpool(PROCESS pid_bid);

static PROCESS
sysd_create_process(
		    enum PROCESS_TYPE proc_type,
		    const char *name,
		    OSENTRYPOINT *entrypoint,
		    struct stkcb *stack,
		    OSADDRESS stack_size,
		    OSPRIORITY priority,
		    OSTIME tmo,
	            PROCESS block,
		    struct OS_redir_entry *redir,
		    PROCESS parent,
		    U32 proc_attrs,
		    OSSENDHOOK *snd_hook,
		    void *lnh_dest_hd
		    );
static void
setup_pcb(struct _pcb *pcb,
	  const char *name,
	  OSTIME tmo,
	  OSPRIORITY priority,
	  OSENTRYPOINT *entrypoint,
	  struct _block *block,
	  struct OS_redir_entry *redir
	  );

static void
setup_bcb(struct _block *bcb,
	  OSUSER user,
	  const char *name,
	  struct _pcb *parent,
	  PROCESS remcall_server,
	  OSBOOLEAN use_remcalls,
	  OSBOOLEAN supervisor_mode,
	  struct poolcb *stackpool,
	  struct poolcb *sigpool,
	  OSSEGMENT segid,
	  PROCESS segpid
	  );

static void sysd(void);
static void huntd(void);
static void tickd(void);
static void get_kernel_configuration(void);
static void init_timer(void);
static void init_pool(void);
static void init_dispatcher(void);
#ifdef OSE_SMP
static void init_per_cpu(void);
#endif
static void init_ppd(void);
static void init_mutex(void);

static union SIGNAL *sys_alloc(OSBUFSIZE size, SIGSELECT signo);
static union SIGNAL *locked_alloc(OSBUFSIZE size, SIGSELECT signo);

static void change_sig_owner(struct _signal *sigadm, struct _pcb *new_owner);

static void request_timeout(struct _pcb *pcb, OSTICK tmo);
static void cancel_timeout(void);
static void cancel_other_timeout(struct _pcb *pcb);

static void *sig_dup(union SIGNAL *sig);
#if MEMORY_MODEL > 0
static union SIGNAL *rec_sig_dup(struct _signal *sigadm_in);
#endif
static struct OS_pcb *
prepare_pcb_buffer(OSBUFSIZE stringlen, enum PROCESS_TYPE type, PROCESS remserver);

static OSADDRESS os_error(int fatal, OSERRCODE ecode, OSERRCODE extra);

static void splay(treenode *root);

static treenode *splay_find(KEYTYPE k, treenode **root,
			    int (*cmp_key)(treenode *T, KEYTYPE k));
static treenode *splay_delete(treenode *node, treenode **root);
static treenode *splay_insert(KEYTYPE k, treenode *newnode, treenode **root,
			      int (*cmp_key)(treenode *T, KEYTYPE k));
static treenode *splay_lin_delete(treenode *node, treenode **root);
static void splay_lin_insert(KEYTYPE k, treenode *newnode, treenode **root,
			     int (*cmp_key)(treenode *T, KEYTYPE k));

static void init_hunt(void);
static void init_break(void);

static void cond_kill_sem(struct segcb *segcb);
static void cond_kill_mutex(struct segcb *segcb);

static OSBOOLEAN sysd_check_free_pool(void);
static void sysd_do_kill_seg_req(union SIGNAL *sigr);
static void kill_proc_cancel_sem(struct _pcb *pcb);
static void kill_attach(PROCESS killed);
static void kill_proc_cancel_env(struct _pcb *pcb);
static void kill_proc_abort_sport(struct _pcb *pcb);
static void kill_timeout_signals(struct _pcb *pcb);
static void sys_free_buf(union SIGNAL **sig);
static void safe_send_w_s(struct _signal *sigadm, PROCESS from, PROCESS to, U32 flags,
			  OSBOOLEAN (*discard_function)(struct _signal *sigadm, void *discard_arg),
			  void *discard_arg);
static void delete_segment(struct segcb *seg);

#if ERROR_CHECKS > 0
#if MEMORY_MODEL > 0
struct _signal *zzrecover_from_pool_crash(union SIGNAL *sig, OSERRCODE errcode);
#else
struct _signal *zzrecover_from_pool_crash(struct _signal *sigadm, OSERRCODE errcode);
#endif
#endif

static void init_sysconf(void);

static void *pool_brk_lo(struct poolcb *inpool, OSBUFSIZE sz);

static void local_kill_proc(PROCESS pid, OSBOOLEAN block_too);

void zzkrn_halted_hook(void);
OSSEGMENT zzcreate_context_segnum(PROCESS pid);

static OSADDRESS get_ppd_addr(void);
#ifdef OSE_SMP
static int wake_up_usr(void);
static PROCESS current_process_usr(void);
static void sysd_lite(void);
static void set_global_zzcur_p(struct _pcb *pcb);
static void local_krn_evt_notify(void);
#endif
static struct _attach_entry *free_attach(OSBOOLEAN give_me_first_free);

static OSBOOLEAN
stop_or_intercept(PROCESS pid, enum pr_state operation, OSERRCODE errcode,
                  OSADDRESS exclude);

static void
start_or_resume(PROCESS pid, enum pr_state operation, OSERRCODE errcode);
static int
start_one_proc(struct _pcb *pcb, int *wasnt_stopped, enum pr_state operation);
static void safe_free_buf(struct _signal *sigadm);
static always_inline void
link_free_buf(struct poolcb *pool,
	      struct pool_list_entry *hd,
	      struct _signal *sigadm,
	      struct _pcb *pcb);
static void handle_timeslicing(void);
static void expire_timeouts(void);
static struct mutcb *index_to_mutcb(U32 index);


#ifdef assert
#undef assert
#endif
#ifdef ASSERT_CODE
#undef ASSERT_CODE
#endif

#ifdef ASSERT
#define assert(expr)  assert_from((expr), 0)
#define assert_from(expr, fromline) \
	/*lint -save -e506 */ /* Warning 506 Constant value Boolean */ \
  ((expr) ? 0 : zzassert_from(#expr, __LINE__, fromline)) \
	/*lint -restore */
#define ASSERT_CODE(...) __VA_ARGS__

static int
zzassert_from(const char *expr, int line, int fromline)
{
  ramlog_printf("***assertion failed: '%s' @ " __FILE__ " line %d",
		expr, line);
  if(0 != fromline)
      ramlog_printf(" called from line %d", fromline);
  ramlog_printf("***\n");
  os_error(1, OSE_ECORRUPTED_KERNEL, (OSERRCODE) ~0); /* Non fatal */
  os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) ~0); /* Fatal */
  /* NOTREACHED */
  return 1;
}



#else  /* ASSERT */
#define assert(expr) ((void)0)
#define assert_from(expr, fromline)  ((void)0)
#undef  ASSERT_CODE
#define ASSERT_CODE(...)
#endif /* ASSERT */

static void
dump_mem(void *vp, size_t size)
{
  U32 *p = (U32 *)(~3 & (OSADDRESS)vp);
  int wc = (size + 3) / 4;
  while(wc > 0)
    {
      ramlog_printf("%#010x: %#010x %#010x %#010x %#010x\n",
		    p, p[0], p[1], p[2], p[3]);
      p += 4;
      wc -= 4;
    } /* while */
}

static void
dump_pcb(struct _pcb *pcb)
{
  ramlog_printf("pcb @%#x instance,pindex %#x,%#x:\n",
		pcb, pcb_to_instance(pcb), pcb_to_pindex(pcb));
  dump_mem(pcb, sizeof(struct _pcb));
}

#if defined(GUNNAR_DEBUG) | defined(ASSERT)
static void
dump_sigadm(int print_header, struct _signal *s)
{
  if(print_header)
    {
      ramlog_printf("%-8.8s %-8.8s %-8.8s %-8.8s %-8.8s %-8.8s %-8.8s %-8.8s %-8.8s\n",
		    "address", "next", "siglist", "mag-ownr",
		    "udata", "head", "size", "dest", "sender");
    }
  if(s)
    {
      ramlog_printf("%08x %08x %08x %04x%04x %08x %08x %08x %08x %08x\n",
		    s, s->next, s->siglist, s->magic_signo, s->owner,
		    s->udata, s->buffer_head, s->size, s->dest_id,
		    s->sender_id);
    }
}
#endif

static noinline union SIGNAL *
alloc_needs_pool_brk(OSBUFSIZE size,
		     SIGSELECT signo,
		     struct pool_list_entry *hd);

static void
process_isr_attach_list(struct isrcb *isr_list);

static void
detach_isr_or_hook(struct vectorcb *vcb, uint8_t slot);

static int
dummy_msg_enqueue(cpuid_t dst_cpu, const struct ose_krn_msg *krn_evt_buf,
		  uint32_t flags);

static uint32_t
dummy_msg_dequeue(struct ose_krn_msg *krn_evt_buf, uint32_t elems);

static void
dummy_evt_interrupt_generate(cpuid_t cpu);

static void
zzkrn_claim_ext_sig(union SIGNAL *sig, OSBUFSIZE size);

static void
zzkrn_release_ext_sig(union SIGNAL *sig);

static always_inline int
check_receive_poll(struct _pcb *pcb, int check_hw_poll);

/*
*****************************************************************************
** 6  GLOBAL VARIABLES.
*****************************************************************************
*/

/*
** For descriptions, see pose_internal.h
*/

#ifdef OSE_SMP
/* zzcpu is declared in TI. */
#else

void *zzcurrent_uarea;

OSPRIORITY zzwakeup_level;

volatile OSBOOLEAN zzwakeflag;	/* Wakeup flag for tim and int processes. */

struct _pcb *zzcur_p;		/* Current process. */

struct _pcb *zzcur_ni;		/* Current non interrupt or timer process. */

volatile U32 zzdp_look_aside;	/* Ready list look aside flag word. */
#endif

struct world zzuu;		/* Contains most static kernel variables. */

/*
*****************************************************************************
** 7  LOCAL DEFINITIONS.
*****************************************************************************
*/

/*
**===========================================================================
** 7.1  Local variables.
**===========================================================================
*/

/*
** Local copy of the system control block provided by the target
** interface. This copy is slightly modified to provide all necessary fields
** even if the target interface is an old design.
** See also: zzuu.sysctlblk.
*/

static struct sysctlblk local_sysctlblk;

/*
** Support for detection of recursive calls to tick().
**
** This variable is initially zero. It is set when the tick() function is
** entered, and cleared when tick() returns.
** If it is already set when tick() is called, an error is reported.
*/

static volatile int concurrent_check;

/* The product id of the pose product for traceability. */

static const char poserev[] = "@(#)POSE " POSEREV "\0"
     "Original author Bengt Eliasson.";

/*
**===========================================================================
** 7.1  Large local structures
**===========================================================================
*/

/*
** The system pool is always present, while other pool control blocks are
** dynamically allocated from the system pool.
**
** See also: pool_list, zzuu.pix_list.
*/

static struct poolcb system_pool; /* The system pool control block. */

/*
** The system segment.
**
** It is used by the kernel and supervisor mode processes.
** The system pool resides in this segment. The segment
** number and ID is zero for the system segment. Other segment IDnumbers are assigned
** by user written memory manager software through the attach_segment()
** system call.
**
** The system segment can never die. This is ensured by setting the
** usage counters in the system pool to a value higher than the
** number of users. There are also other rules that will have the
** same effect.
**
** See also: segment_list.
*/

static struct segcb system_segment;

/*
** The system handler array.
** This array contains pointers to user supplied handlers or callback
** functions which may be altered in runtime by power users.
*/

static struct handlerp handlerp = {
  17,				/* No of pointers in the struct. */
  (void (*)(U32 vector, U32 priority)) 0, /* vector_bind_handler */
  (void (*)(U32 vector)) 0,	/* vector_release_handler */
  zzcall_attach_block,		/* zzcall_attach_block */
  zzcall_mem_move,		/* zzcall_mem_move */
  zzcall_select_segment,	/* zzcall_select_segment */
  zzcall_set_segment_mode,	/* zzcall_set_segment_mode */
  zzcall_system_errh,		/* zzcall_system_errh */
  zzkrn_halted_hook,		/* zzkrn_halted_hook */
  zzget_filename,		/* zzget_filename */
#if USE_CAPTURE_REGS
  zzcapture_regs,		/* zzcapture_regs */
#else
  (void (*)(void *, char *)) 0,	/* zzcapture_regs */
#endif
  zzget_regs,			/* zzget_regs */
#if USE_POWER_SAVE > 0
  zzcall_power_save,		/* zzcall_power_save */
#else
  (OSTICK (*)(OSTICK)) 0,	/* zzcall_power_save */
#endif
  (void (*)(OSBUFSIZE, void *, struct debug_pcb *, PROCESS)) 0,	/* zzprof_alloc */
  (void (*)(OSBUFSIZE, void *, struct debug_pcb *, PROCESS)) 0,	/* zzprof_free_buf */
  dummy_msg_enqueue,
  dummy_msg_dequeue,
  dummy_evt_interrupt_generate
};

/*
** This structure contains system calls that can be invoked directly
** from user mode. It is described in pose.h.
** 
** The structure is (for now) located in supervisor read-only memory,
** for this reason it must be possible to both read (and execute) from
** this memory space in user mode.
**
** This works fine in the current OSE Memory model for ARM and
** PPC. When MAS (Muliple Address Space) or higer level of security
** (like MMU support for MIPS) becomes part of the plan, this
** structure can be moved to a "special" user readable area, and the
** functions adopted appropritely.
*/
#ifdef OSE_SMP
static const struct krn_usrcalls usrcalls = {
  USRCALLS_COUNT,
  &get_ppd_addr,
  &current_process_usr,
  &get_ticks,
  &system_tick,
  &wake_up_usr
};
#else
static const struct krn_usrcalls usrcalls = {
  USRCALLS_COUNT,
  &get_ppd_addr,
  &current_process,
  &get_ticks,
  &system_tick,
  &wake_up
};
#endif

/*
** The pose configuration array.
** This array contains values defined in pose.con.
*/

static const U32 confp[] = {
  18, /* No of entries in the table. */
  ENABLE_INTERRUPT_PROFILING,
  USE_MEM_MOVE,
  LATE_INIT_INT,
  OSE_DEBUG,
  ERROR_CHECKS,
  USE_REMOTE_CALLS,
  MEMORY_MODEL,
  USE_INTERRUPT_UAREA,		/* Obsoleted. Always true. */
  CALL_INTERRUPTS_DIRECTLY,
  USE_CAPTURE_REGS,
  32,				/* Obsolete, POOL_ALIGNMENT set in runtime */
  MAX_ENV_LIST,
  MAX_REGLEN,
  MAX_FILE_NAMELEN,
  MIN_INTERRUPT_STACKSIZE,
  HUNTD_STACKSIZE,
  SYSD_STACKSIZE,
  ZZMAIN_STACKSIZE
};

static SYSERR_HOOK test_err_handler;

/*
**===========================================================================
** 7.2  The kernel symbol table
**===========================================================================
*/

/*
** The kernel symbol table supports post mortem analyzer tools.
**
** It provides symbolic information on all kernel data structures.
**
** The symbol table is generated for each pose-version by applying the
** the confsig program supplied with the OSE classic kernels.
** Some header information is then stripped from the result before it is used
** here.
**
** Finally, the symbol table is compiled with the target kernel and used with
** a special analyzer tool to produce pose_sym.h.
**
** The SUPPORT_POSTMORTEM_SYMBOLS constant should be added to pose.con ??
** whenever post mortem symbols are actually implemented (if ever...).
** The SKIP_SIGDESC constant is used when running confsig.
*/

#ifdef SUPPORT_POSTMORTEM_SYMBOLS
#ifndef SKIP_SIGDESC
#include "pose_sym.h"
#endif
#endif

/*
**===========================================================================
** 7.3  The post mortem support table
**===========================================================================
*/

/*
** The post mortem support table is the handle used to access kernel data
** structures.
**
** It is organized as a header structure in the const section, which points
** to all other data structures.
*/

const struct osvarp zzosvarp = {
  MAGIC_KERNEL_TYPE,
  MAGIC_KERNEL_FEATURES,
  zztivarp,
#if OSE_DEBUG > 0
  zzldmvarp,
#else
  (const U32 *) 0,
#endif
#ifdef SUPPORT_POSTMORTEM_SYMBOLS
  (void *) &zzpose_symbols, /* pose symbols with debugger. */
#else
  NULL,	/* pose symbols when not present. */
#endif
  &zzuu,
  SMP_CODE_YES_NO(&(zzcpu[0].curr_uarea),&zzcurrent_uarea),
  SMP_CODE_YES_NO(&(zzcpu[0].wakeup_level),&zzwakeup_level),
  sizeof(zzwakeup_level),
  SMP_CODE_YES_NO(&(zzcpu[0].wakeflag),&zzwakeflag),
  sizeof(zzwakeflag),
  SMP_CODE_YES_NO(&(zzcpu[0].curr_p),&zzcur_p),
  SMP_CODE_YES_NO(&(zzcpu[0].curr_ni),&zzcur_ni),
  SMP_CODE_YES_NO(&(zzcpu[0].dp_look_aside),&zzdp_look_aside),
  &system_pool,
  &system_segment,
  SMP_CODE_YES_NO(&zzcpu[0].idle_pcb, &zzuu.idle_pcb),
  poserev,
  &handlerp,
  confp,
  &usrcalls,
  SMP_CODE_YES_NO(&zzcpu[0],NULL),
  SMP_CODE_YES_NO(sizeof(zzcpu[0]),0),
  SMP_CODE_YES_NO(OSE_SMP_MAX_CPUS,0),
  &zzuu.main_pool,
  &zzuu.pix_list
};

/*
**===========================================================================
** 7.4  Local helper functions 
**===========================================================================
*/


/*
** With, for example, gcc-3.3.2, only already defined functions can be
** inlined. Here is the place to define such functions with a module-wide
** usage. Functions with more local usage may find another definition context.
*/

/*
** Check that the caller is a prioritized or background process.
**
** Several system calls are illegal for interrupt or timer processes.
** An error is generated if these calls are used from unauthorized processes.
** Bits of error code representing system call is supplied by caller,
** e.g. the OSE_SEND constant or a similar code.
**
** Return nonzero if error occurred and was handled.
*/

#if ERROR_CHECKS > 0
static inline int
check_nonint_caller(OSERRCODE errcode)
{
  if(unlikely(zzcur_p->priority > 32))
    {
      os_error(0, OSE_EILLEGAL_SYSTEMCALL | errcode, (OSERRCODE) 0);	/* Non fatal. */
      return 1;
    }
  else
    {
      return 0;
    }
}
#endif


/* 
** Kernel Events 
*/

/*
** Tiny events are 4 bits, extended are 12 bits with tiny bits all zero.
** This permits us to pack a kernel object pointer and a tiny event in
** a single word. Extended events may pass 20 bits of data in the word.
*/

enum KRN_EVT
  {
    /* Masks */
    KRN_EVT_TINY_MASK	= 0x00f,
    KRN_EVT_EXTENDED_MASK = 0xfff,
    /* Tiny events */
    KRN_EVT_EXTENDED	= 0x0,	/* Look for an extended event! */
    KRN_EVT_SIG_SEND	= 0x1,	/* pid, sigadm* */
    KRN_EVT_MUTEX_RAISE	= 0x2,  /* pid, prio<<4 | (instance&0x3f)<<10 | index<<16 */
    KRN_EVT_FLUSH_TLB	= 0x3,  /* base, size */
    
    
    /* Extended events */
    KRN_EVT_START	= 0x010, /* pid, NULL */
    KRN_EVT_STOP	= 0x020, /* pid, NULL */
    KRN_EVT_INTERCEPT	= 0x030, /* pid, NULL */
    KRN_EVT_RESUME	= 0x040, /* pid, NULL */
    KRN_EVT_SIGNAL_FSEM	= 0x050, /* pid, NULL */
    KRN_EVT_SET_PRI_FOR	= 0x060, /* pid, prio<<16 */

    KRN_EVT_TICK_TI_PROC= 0x080, /* pid, NULL */
    KRN_EVT_RESCHEDULE	= 0x090, /* 0, NULL */
    KRN_EVT_YIELD	= 0x0b0, /* 0, NULL */
    KRN_EVT_TICK	= 0x0c0, /* ticks, NULL */
    KRN_EVT_TIMEOUT	= 0x0d0, /* pid, NULL (@Deprecated@) */
    KRN_EVT_SIGNAL_SEM	= 0x0e0, /* pid, NULL */
    KRN_EVT_MIGRATE	= 0x0f0, /* pid, to_cpu<<16 */
    KRN_EVT_SIG_COUNT	= 0x100, /* pid, signed_increment<<16 */
    KRN_EVT_SEND_SPORT	= 0x110, /* pid, NULL */
    KRN_EVT_BLOCK_HOOK	= 0x120 /* hookcb, operation */
  };



struct krn_event
{
  OSADDRESS	event;
  PROCESS	data;
  void		*obj;
};

/*
** Send a kernel event.
**
** Local interrupts are disabled by the caller.
** NOTE! The current implementation truncates the lower bits in the obj 
** parameter to permit a 64 bit hw channel!
*/

#ifdef OSE_SMP

static void
send_krn_event(cpuid_t cpu, uint32_t krn_event, PROCESS data, void *obj)
{
  uint32_t msg[2];
  const uint32_t wordcount = sizeof(msg) / sizeof(msg[0]);
  uint32_t err_status;

  /* Assert that event and obj may pe packed in one word. */
  assert(krn_event < KRN_EVT_EXTENDED_MASK);
  assert((KRN_EVT_TINY_MASK & (uint32_t)obj) == 0);
  assert(krn_event <= KRN_EVT_TINY_MASK
	 || (KRN_EVT_EXTENDED_MASK & (uint32_t)obj) == 0);

  msg[0] = krn_event | (uint32_t)obj;
  msg[1] = (uint32_t)data;
  
  if(unlikely(cpu == zzkrn_cpu_id()))
    {
      /* Something is very wrong here, should never send a event to myself.
       * It may be of some help  to have the queue in the ramlog.
       */
      zzkrn_evt_dump_queue(cpu);
      os_error(2, OSE_ECORRUPTED_KERNEL, 28); /* Fatal system error */
    }

  
  err_status = zzkrn_evt_enqueue(cpu, msg, wordcount);
  handlerp.evt_interrupt_generate(cpu); /* Intercept receiving core. */
  if(unlikely(err_status))
    {
      uint32_t retry;
      cpuid_t cpu_ix;
      int exception_status;

      /*
      ** The queue full, retry again a number of times.
      ** To avoid a deadlock situation empty my own queue in case
      ** the cpu I try to send to also sends events to me and therefor
      ** can not empty his own queue.
      */
      retry = 1000000;
      while(err_status && retry--)
        {
	  local_krn_evt_notify();
	  err_status = zzkrn_evt_enqueue(cpu, msg, wordcount);
        }

      if(err_status == 0)
	{
	  handlerp.evt_interrupt_generate(cpu); /* Interrupt receiving core. */
	  return;		/* SUCCESS */
	}
      
      /* Dump queue and current info for all cpus to the ramlog */

      /* Dump the queue to the ramlog and then report fatal error */
      ramlog_printf("Failed at zzkrn_evt_enqueue to cpu: %u\n", cpu);
      zzkrn_evt_dump_queue(cpu);
      /* Dump current processes on all cpus. */
      for(cpu_ix = 0; cpu_ix < zzkrn_num_cpus(); ++cpu_ix)
	{
	  struct per_cpu *cur_cpu = &zzcpu[cpu_ix];
	  
	  ramlog_printf("cpu: %u, cur_p: %#x, name: %s, cur_ni: %#x, name: %s\n",
			cpu_ix,
			pcb_to_pid(cur_cpu->curr_p), cur_cpu->curr_p->name,
			pcb_to_pid(cur_cpu->curr_ni), cur_cpu->curr_ni->name);
	} /* for */

      /* Try to generate an exception at the target cpu */
      exception_status = zzkrn_force_exception_in_other_cpu(cpu);
      if(exception_status)
	{
	  ramlog_printf("Trying to generate an exception at cpu: %u\n", cpu);

	  if(exception_status == 1)
	    {
	      /* Just wait for a reset or a watchdog exception. */
	      for(;;)
		{
		  local_krn_evt_notify();
		  err_status = zzkrn_evt_enqueue(cpu, msg, wordcount);
		}
	      /*NOTREACHED*/
	    }

	  /* Wait a while to let that cpu build a dump if it can. */
	  retry = 10000000;
	  while(err_status && retry--)
	    {
	      local_krn_evt_notify();
	      err_status = zzkrn_evt_enqueue(cpu, msg, wordcount);
	    }
	}
      else
	  ramlog_printf("Can not generate an exception at cpu: %u\n", cpu);

      /* Report a fatal error */
      os_error(2, (OSERRCODE)OSE_EKRN_EVT_QUEUE_FULL, cpu); /* Fatal system error */
    }
}
#endif

/*
** Receive a kernel event.
**
** Local interrupts are disabled by the caller.
** Returns non_zero if an event was found.
*/
#ifdef OSE_SMP
static int
receive_krn_event(struct krn_event *evt)
{
  uint32_t length;
  uint32_t msg[2];

  length = zzkrn_evt_dequeue(msg);
  assert(length < 3);
  if(length != 0)
    {
      uint32_t event = msg[0] & KRN_EVT_TINY_MASK;

      if(event == KRN_EVT_EXTENDED)
	{
	  evt->event = msg[0] & KRN_EVT_EXTENDED_MASK;
	  evt->obj = (void *) (msg[0] & ~KRN_EVT_EXTENDED_MASK);
	}
      else
	{
	  evt->event = msg[0] & KRN_EVT_TINY_MASK;
	  evt->obj = (void *) (msg[0] & ~KRN_EVT_TINY_MASK);
	}
      evt->data = msg[1];
    }
  return length;
}
#endif

static int
dummy_msg_enqueue(cpuid_t dst_cpu, const struct ose_krn_msg *krn_msg_buf,
		  uint32_t flags)
{
  (void)dst_cpu;
  (void)krn_msg_buf;
  (void)flags;

  /* Return a non zero value to tell ose_fsend() that it failed. */
  return 1;
}

static uint32_t
dummy_msg_dequeue(struct ose_krn_msg *krn_msg_buf, uint32_t elems)
{
  (void)krn_msg_buf;
  (void)elems;

  /*
  ** Return zero to tell receive_core_sig_queue() that there was
  ** nothing in the queue.
  */
  return 0u;
}

static void
dummy_evt_interrupt_generate(cpuid_t cpu)
{
  (void)cpu;
}

/*
*****************************************************************************
** 8  SYSTEM START.
*****************************************************************************
*/

/*
** Cold start entrypoint in the OSE kernel.
**
** This is where OSE is entered from the target interface in response to
** a previous hardware reset operation or a zzstart_OSE call from the
** interface library.
**
** The mode parameter indicates how system start should be performed.
** mode = 0. Cold start. We know this is a power on situation. The kernel
** will not examine the pool since that might cause parity errors or similar
** problems. The entire system pool will be cleared, all
** internal kernel variables are initialized, and the system is started.
** Debugger variables including the trace buffer will be lost.
** User processes are created.
**
** mode = 1. Warm start. This may be either a cold or a warm start. The kernel
** will examine the pool to determine if system parameters are still valid.
** If they are, a warm restart of the OSE debugger is performed.
** User processes are created.
** If system parameters are not valid, a cold start is performed instead.
**
** mode = 2. Hot start. The system was previously shut down with a power_fail
** system call.  The memory is still valid and the kernel will immediately
** swap in the process which called power_fail();
*/

void 
zzstart_kernel(U32 mode)
{
  static U32 magic;

  /*
  ** Trust the hot start indicator only if we can recover a previously saved
  ** magic number.
  */
  if(mode == 2 && magic != 0x61051910)
    {
      mode = 1;
    }
  magic = 0x61051910;

  /* Jump into dispatcher if hot start is seleceted. */
  if(mode == 2)
    zzgo_search();

  /*
  ** Clear the kernel stable flag to indicate to post mortem tools
  ** that we do not have a stable situation yet.
  **
  ** Also clear the variable world pointers provided by the target
  ** interface.  Disable paging support. New values may be provided at
  ** a later stage.
  */

  zzuu.kernel_initialized = 0;
  zzuu.targint_world = NULL;
  zzuu.pagesize = 0;

  /*
  ** Clear the error check level flags to default values.
  */

#if ERROR_CHECKS > 0
  zzuu.error_level1 = (U8) 1;
#else
  zzuu.error_level1 = (U8) 0;
#endif

#if ERROR_CHECKS > 1
  zzuu.error_level2 = (U8) 1;
#else
  zzuu.error_level2 = (U8) 0;
#endif

  /* Save the warm start mode for later use by the debugger. */
  zzuu.start_mode = (U8) mode;

  /* Disable all debugger event hooks. */
  zzuu.debugger_enabled = (OSBOOLEAN) 0;
  zzuu.sysprocs_created = (OSBOOLEAN) 0;

  /* Disable interrupt process swap hooks. */
  zzuu.inthook_enabled = (U8) 0;
  zzuu.tickhook_enabled = (U8) 0;
#if ENABLE_INTERRUPT_PROFILING > 0
#if LATE_INIT_INT > 0
  zzset_inthook_status(0);
#endif
#endif

  /* Enable interrupts during the restart procedure. */
  SMP_CODE(BKL_LOCK());
  BKL_UNLOCK();

  /*
  ** The initialization functions below depend on each other.
  ** They can not be called in any other order.
  */

  get_kernel_configuration();

  /* First initialize the system pool and other memory management functions. */
  init_pool();

  /* Set up the breakpoint system. */
  init_break();

  /* Set up the system timer. */
  init_timer();

  /* Initialize the dispatcher and create idle process and system daemon. */
  init_dispatcher();

  /* Set up the attach list. */
  init_attach();

  /* Set up the hunt daemon. */
  init_hunt();

  /* Set up the per process data. */
  init_ppd();

  /* Set up the sysconf function array. */
  init_sysconf();

  /* Set up mutex services. */
  init_mutex();

  /* Initate event/msg queues for multicore. */
  SMP_CODE(zzkrn_evt_init());

  /* Swap in the system daemon. */
  zzgo_search();
}

#ifdef OSE_SMP

/* Create the sysd_lite daemon for each cpu. */

static void
sysd_create_sysd_lite(OSPRIORITY prio)
{
  static const char sysd_lite_name[] = "ose_sysd00";
  OSADDRESS stack_size = local_sysctlblk.superv_stack_size;
  struct stkcb *stack;
  cpuid_t cpu;
  struct _pcb *pcb;
  char *name;

  for(cpu = 0; cpu < zzkrn_num_cpus(); cpu++)
    {
      name = (char *)alloc(sizeof(sysd_lite_name), 0);
      memcpy(name, sysd_lite_name, sizeof(sysd_lite_name));
      name[(sizeof(sysd_lite_name)) - 3] = (char)('0' + (cpu / 10));
      name[(sizeof(sysd_lite_name)) - 2] = (char)('0' + (cpu % 10));
      LOCK_SWAP(); 
      stack = stack_alloc(&system_pool, stack_size);
      UNLOCK_SWAP();

      memset((char *) stack->limit, END_MARK,
	     (size_t) ((char *)stack->top - (char *)stack->limit));

      zzcpu[cpu].sysd_lite_ = 
	sysd_create_process(
			    OS_PRI_PROC,
			    name,
			    sysd_lite,
			    stack,
			    stack_size,
			    prio,
			    (OSTIME) 0, /* No timeslicing. */
			    pcb_to_pid(zzuu.bcb_list),
			    NULL,
			    current_process(),
			    (U32) 0,
			    NULL,
			    NULL
			    );
      UNLOCK_SWAP();

      /* Transfer the process to the proper cpu. */
      pcb = pid_to_pcb(zzcpu[cpu].sysd_lite_);
      pcb->cpu_id = cpu;
      pcb->ready_entry.rlist = (struct ready_list_entry *)
        &ready_list_x(cpu)[pcb->priority];
    } /* for */
}
#endif

/*
*****************************************************************************
** 9  DISPATCHER AND ASSOCIATED FUNCTIONS.
*****************************************************************************
*/

/*
**===========================================================================
** 9.1  Initialization.
**===========================================================================
*/

/*
** Subfunction to init_dispatcher().
** This function is present only as a bug guard. It is called if the idle
** process for some reason is actually scheduled by the dispacher.
*/

static void
internal_error(void)
{
  /* Internal error if we run the idle process. */
  os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 1);	/* Fatal. */
}

/*
** Set up the dispatcher and create the first process.
*/

static void
init_dispatcher(void)
{
  struct _pcb *pcb;
  struct stkcb *stack;

  /* Initialize dispatcher variables. */
#ifndef OSE_SMP
  zzuu.dispatcher_lock_flag = (OSBOOLEAN) 0;
#endif
  zzwakeflag = (OSBOOLEAN) 1;
  zzuu.pfail_req = (OSBOOLEAN) 0;
  zzww.dp_req = (OSBOOLEAN) 0;
  zzww.dp_state = (U8) 0;
  zzww.cur_segid = 0;
  SMP_CODE(zzww.leave_segid_now = 0);
  zzuu.power_save_threshold = (OSTICK) -1;
  zzuu.power_saved_ticks = (OSTICK) 0;
  zzuu.longest_power_nap = (OSTICK) 0;

  /* Clear list of background processes. */
  zzww.bg_list.hd = NULL;
  zzww.bg_list.time_slice = 1;
  zzww.bg_list.search_next = NULL;

  /* Build the ready list and the wakeup list. */
  zzdp_look_aside = 0;
  zzww.wk_look_aside = 0;
    {
      int i;
      U32 pri_mask = 1;
      for(i = 32; i;)
	{
	  i--;
	  zzww.ready_list[i].hd = NULL;
	  zzww.ready_list[i].tl = NULL;
	  zzww.ready_list[i].pri_mask = pri_mask;
	  zzww.wakeup_list[i].hd = NULL;
	  zzww.wakeup_list[i].pri_mask = pri_mask;
	  pri_mask <<= 1;
	}
    }

  /* Create the system block descriptor. */
  zzuu.system_block = zzuu.bcb_list = &alloc_pcb(OSE_CREATE_BLOCK)->bcb;

  /* Check that we got a block descriptor. */
  if(!zzuu.system_block)
    {
      /* Fatal system error. */
      os_error(2, OSE_ENO_KERN_SPACE | OSE_START_OSE,
	       (OSERRCODE) ((system_pool.extend_count & 0x7f) << 24) |
	       (OSERRCODE) pcb_to_pindex(&system_pool));
    }

  setup_bcb(zzuu.bcb_list,
	    (OSUSER)0,
	    "OSE",
	    NULL,
	    (PROCESS) 0,
	    (OSBOOLEAN) 0,
	    (OSBOOLEAN) 1,
	    &system_pool,
	    &system_pool,
	    (OSSEGMENT) 0,
	    (PROCESS) 0
	    );
  zzuu.bcb_list->next = NULL;
  zzuu.bcb_list->magic = MAGIC_BCB;
  zzuu.bcb_list->killed = (OSBOOLEAN) 0;

  /* Build a pcb for the idle process and make it current. */
  pcb = zzww.idle_pcb;
  add_to_pix_list(pcb);
  setup_pcb(pcb, "idle", (OSTIME) 1, (OSPRIORITY) 32, internal_error, zzuu.bcb_list, NULL);
  pcb->magic = MAGIC_PCB;
  pcb->killed = (OSBOOLEAN) 0;
  pcb->status = (pr_state_t) (pcb->status & (~ST_STOPPED));
  pcb->stack = stack = stack_alloc(&system_pool,
				   local_sysctlblk.superv_stack_size);
#if ERROR_CHECKS > 0
  memset((char *) stack->limit, END_MARK, (size_t) ((char *)stack->top - (char *)stack->limit));
#else
  memset((char *) stack->limit, END_MARK, (size_t) 4);
#endif
  pcb->block->proclist = pcb;
  set_zzcur_p(pcb);
  zzcur_ni = zzww.last_cur_ni = pcb;
  pcb->uarea =
    zzcreate_context(pcb->stack->limit,
		     (OSADDRESS) ((char *)pcb->stack->top - (char *)pcb->stack->limit),
		     pcb->entrypoint,
		     pcb->block->supervisor_mode,
		     (OSBOOLEAN) 0,
		     (PROCESS) pcb_to_pid(pcb),
		     (PROCESS) 0,
		     (void *) 0
		     );
  zzcurrent_uarea = pcb->uarea;

  /* The idle process needs to be a manager in case it
   * executes on someone's secure stack. */
  zzset_manager_mode(1);
  
  /* Tell sysd that the hunt daemon is still nonexistent. */
  zzuu.huntd_ = (PROCESS) 0;

  /* Create pcb for the system daemon. */
  stack = stack_alloc(&system_pool, SYSD_STACKSIZE + local_sysctlblk.superv_stack_size);
#if ERROR_CHECKS > 0
  memset((char *) stack->limit, END_MARK, (size_t) ((char *)stack->top - (char *)stack->limit));
#else
  memset((char *) stack->limit, END_MARK, (size_t) 4);
#endif

  {
#ifdef SYSD_PRIORITY
    OSPRIORITY sysd_pri = SYSD_PRIORITY;
#else
    OSPRIORITY sysd_pri = (OSPRIORITY) 20;
#endif
    /*
    ** Get the zzmain priority configured by the user.
    ** Shouldn't we assure local_sysctlblk.sysd_priority < 32
    ** and issue a ``os_error(0, OSE_EILLEGAL_CONFIGURATION,
    ** (OSERRCODE)local_sysctlblk.sysd_priority);'', this
    ** check should probably also be added for
    ** huntd_priority, ldm_priority, tickd_priority etc.
    ** @frbr@ ???
    */
    if(local_sysctlblk.table_extended >= 1)
      sysd_pri = (OSPRIORITY)local_sysctlblk.sysd_priority;

    zzuu.sysd_ = 
      sysd_create_process(
			  OS_PRI_PROC,
			  "ose_sysd",
			  sysd,
			  stack,
			  (OSADDRESS) SYSD_STACKSIZE,
			  sysd_pri,
			  (OSTIME) 0, /* No timeslicing. */
			  pcb_to_pid(zzuu.bcb_list),
			  NULL,
			  (PROCESS) 0,
			  (U32) 0,
			  NULL,
			  NULL
			  );

  }

  /* Unlock the dispatcher since sysd_create_process() left it locked. */
  UNLOCK_SWAP();

  /*
  ** Start the system daemon.
  */

  pcb = pid_to_pcb(zzuu.sysd_);
  pcb->st_cnt = 0;

  /*
  ** Make the system daemon ready.
  */

  pcb->status = (pr_state_t) ST_INIT;
    {
      register struct ready_list_entry *rlist;
      pcb->ready_entry.is_ready = (OSBOOLEAN) 1;
      if(pcb->priority < 32)
	{
	  rlist = pcb->ready_entry.rlist;
	  rlist->hd = rlist->tl = pcb;
	  pcb->ready_entry.prev = NULL;
	  zzdp_look_aside |= RLIST_PRI_MASK(pcb);
	  pcb->ready_entry.nxt = NULL;
	}
    }

  /*
  ** Some old tools check the global (and only) zzcur_p pointer to see
  ** if the kernel is started. We'd better keep em happy. But is this
  ** the right place to call set_global_zzcur_p()? @frbr@
  */
  SMP_CODE(set_global_zzcur_p(pcb));
}

/*
** Set up all per-CPU local data for SMP kernels.
*/

#ifdef OSE_SMP
static void
init_per_cpu(void)
{
  cpuid_t cpus = zzkrn_num_cpus();
  cpuid_t i, j;
  size_t tmolist_size;

  if(PER_CPU_SIZE != sizeof(zzcpu[0]))
    os_error(2, OSE_START_OSE | OSE_ECORRUPTED_KERNEL, (OSERRCODE) sizeof(zzcpu[0])); /* Fatal */

  zzcpu[0].state = CPU_STATE_ACTIVE; 

  for (i = 1 ; i < cpus ; i++)
    {
      struct per_cpu *cpu = &zzcpu[i];
      struct _pcb *pcb = (struct _pcb *)pool_brk_lo(&system_pool, sizeof(struct _pcb));
      struct stkcb *stack;
      char *name;

      /* Create "idleXX" name.  */
      name = (char *)pool_brk_lo(&system_pool, 8);
      name[0] = 'i';
      name[1] = 'd';
      name[2] = 'l';
      name[3] = 'e';
      name[4] = (char)('0' + (i / 10));
      name[5] = (char)('0' + (i % 10));
      name[6] = '\0';
	
      add_to_pix_list(pcb);
      setup_pcb(pcb, name, (OSTIME) 1, (OSPRIORITY) 32,
		internal_error, zzuu.bcb_list, NULL);
      /* Override cpu_id set by setup_pcb */
      pcb->cpu_id = i;
      pcb->magic = MAGIC_PCB;
      pcb->killed = (OSBOOLEAN) 0;
      pcb->status = (pr_state_t) (pcb->status & (~ST_STOPPED));
      pcb->stack = stack = stack_alloc(&system_pool,
				       local_sysctlblk.superv_stack_size);
#if ERROR_CHECKS > 0
      memset((char *) stack->limit, END_MARK, (size_t) ((char *)stack->top - (char *)stack->limit));
#else
      memset((char *) stack->limit, END_MARK, (size_t) 4);
#endif
      pcb->uarea =
	zzcreate_context(pcb->stack->limit,
			 (OSADDRESS) ((char *)pcb->stack->top - (char *)pcb->stack->limit),
			 pcb->entrypoint,
			 pcb->block->supervisor_mode,
			 (OSBOOLEAN) 0,
			 (PROCESS) pcb_to_pid(pcb),
			 (PROCESS) 0,
			 (void *) 0
			 );
      pcb->proclist = pcb->block->proclist;
      pcb->block->proclist = pcb;

      cpu->curr_uarea = pcb->uarea;
      cpu->curr_p = pcb;
      cpu->curr_ni = pcb;
      cpu->last_cur_ni = pcb;
      cpu->idle_pcb = pcb;
      cpu->interrupt_stack = NULL; /*@SMP@*/
      cpu->leave_segid_now = 0;
      cpu->ipi_operable = (OSBOOLEAN) 0;
      cpu->dp_state = (U8) 0;
      cpu->dp_req = (OSBOOLEAN) 0;
      cpu->dp_idle_interrupted = (OSBOOLEAN) 0;
      cpu->cur_segid = 0;
      cpu->dp_look_aside = 0;
      cpu->wk_look_aside = 0;
      cpu->wakeflag = (OSBOOLEAN) 1;
      cpu->wakeup_level = (OSPRIORITY) 0;
      cpu->state = CPU_STATE_LIMBO;
      cpu->timeout_count = 0;
      cpu->hw_receive_poller = NULL;
      cpu->bg_receive_poll = (OSBOOLEAN) 0;

      /* Clear list of background processes. */
      cpu->bg_list.hd = NULL;
      cpu->bg_list.search_next = NULL;
      cpu->bg_list.time_slice = 1;

      /* Build the ready list and the wakeup list. */
      for(j=0; j<32; j++)
	{
	  U32 pri_mask = 0x80000000UL >> j;
	  
	  cpu->ready_list[j].hd = NULL;
	  cpu->ready_list[j].tl = NULL;
	  cpu->ready_list[j].pri_mask = pri_mask;
	  cpu->wakeup_list[j].hd = NULL;
	  cpu->wakeup_list[j].pri_mask = pri_mask;
	}
      
      /* Obtain space for the local timeout list and clear it. */
      tmolist_size = local_sysctlblk.tmo_array_length * sizeof *cpu->tmolist;
      cpu->tmolist = (struct _pcb **)pool_brk_lo(&system_pool, tmolist_size);
      memset(cpu->tmolist, 0, tmolist_size);
    }
}
#endif
  
/*
**===========================================================================
** 9.2  Local functions.
**===========================================================================
*/

#ifdef BENGT_DEBUG
/* This is junk code which can be safely removed at any time. ?? */
#include <stdio.h>
/* How to print specific signals on the terminal. */
void print_sig(union SIGNAL **sig, PROCESS from, PROCESS to)
{
  register struct _signal *sigadm;
/*    if(**((SIGSELECT **)sig) != 49) */
      {
      unsigned int i;
      unsigned char *p;
      printf("%8lx ", *sig);
      if(get_ptype(from) != OS_ILLEGAL)
	printf("%12s(%x) -> ",pid_to_pcb(from)->name,from);
      else
	printf("%12s(%x) -> ", "    <illegal/seg>",from);
      if(get_ptype(to) != OS_ILLEGAL)
	{
	  if(get_ptype(to) != OS_ZOOMBIE)
	    printf("%12s(%x) : ",pid_to_pcb(to)->name,to);
	  else
	    printf("%12s(%x) : ", "    <killed>",to);
	}
      else
	printf("%12s(%x): ","    <illegal/seg>",to);
      printf("%8ld ", **((SIGSELECT **)sig));
      p = *((unsigned char **)sig) + sizeof(SIGSELECT);
      sigadm = sig_to_adm(*sig);
      for(i = sigadm->size + sizeof(unsigned char) - sizeof(SIGSELECT); i; i--)
	{
	if(i > 2048)
	  i = 2048;
	printf("%2x ", *p++);
	}
      printf("\r\n");
      }
  }

/*??*/
#endif

/*
** Insert a background process in the background list of the current cpu. 
**
** The dispatcher must be locked by the caller. The process must be alive.
** Caller is sysd or (SMP) syd_lite.
*/
static void
insert_pcb_in_bg_list(struct _pcb *pcb)
{
  struct background_list *bg = &zzww.bg_list;
  LOCKSTATE msr;

  LOCK_SAVE(msr);
  assert(zzdispatcher_locked);
  assert(pcb->ready_entry.rlist == NULL);
  pcb->ready_entry.rlist = (struct ready_list_entry *)bg;
#ifdef OSE_SMP
  pcb->cpu_id = zzkrn_cpu_id();
#endif
  /* 
  ** Changed insert point from first to last Thu Dec 7 2006. /Gunnar
  */
  if(bg->hd)
    {
      pcb->ready_entry.nxt = bg->hd;
      pcb->ready_entry.prev = bg->hd->ready_entry.prev;
      pcb->ready_entry.nxt->ready_entry.prev = pcb;
      pcb->ready_entry.prev->ready_entry.nxt = pcb;
    }
  else
    {
      bg->hd = pcb->ready_entry.nxt = pcb->ready_entry.prev = pcb;
      bg->time_slice = pcb->tmo;
    }
  LOCK_RESTORE(msr);
}

/*
** Remove a background process from the background list of the current cpu.
**
** Dispatcher and local interrupts must be disabled by the caller.
** Caller is sysd or (SMP) syd_lite.
*/
static void
unlink_pcb_from_bg_list(struct _pcb *pcb)
{
  struct background_list *bg = &bg_list_x(pcb->cpu_id);

  assert(zzdispatcher_locked);
  assert(pcb->ready_entry.rlist == (struct ready_list_entry *)bg);

  /*
  ** Check if we are deleting the process which is pointed out as
  ** the next one that may steal some background time. If so, pass
  ** on that possiblity to another one. 
  */
  if(bg->search_next == pcb)
    bg->search_next = pcb->ready_entry.nxt;

  /* Remove the killed process. */
  if(pcb->ready_entry.nxt == pcb)
    {
      /* This was the only process. */
      bg->hd = bg->search_next = NULL;
    }
  else
    {
      /* Unlink from the list. */
      pcb->ready_entry.nxt->ready_entry.prev = pcb->ready_entry.prev;
      pcb->ready_entry.prev->ready_entry.nxt = pcb->ready_entry.nxt;

      /*
       * If the unlinked process owned the current time slot, then
       * give it to the next one.
       */
      if(bg->hd == pcb)
	{
	  bg->hd = pcb->ready_entry.nxt;
	  bg->time_slice = bg->hd->tmo;
	}
    }
  pcb->ready_entry.rlist = NULL;
}


/*
** Make the calling non interrupt process not ready and
** run the dispatcher.
**
** The pcb pointer argument must be the current process. It is
** present only to optimize performance.
**
** The state variable in the pcb must have been set by the caller
** to some "not ready" state prior to this function being executed.
** Note that the current process may be in a not-ready state already
** if it is executing in a dispatcher lock region.
**
** Interrupts must be disabled when this function is called.
** Interrupts are disabled when it returns.
**
** See also: set_pri()
*/
static void
make_mypri_not_ready(struct _pcb *pcb)
{
  register struct ready_list_entry *rlist;

  if(pcb->ready_entry.is_ready)
    {
      if(pcb->priority < 32)
	{
	  rlist = pcb->ready_entry.rlist;
	  if(pcb->ready_entry.prev)
	    {
              pcb->ready_entry.prev->ready_entry.nxt = pcb->ready_entry.nxt;
	      if(pcb->ready_entry.prev->ready_entry.nxt)
		pcb->ready_entry.nxt->ready_entry.prev = pcb->ready_entry.prev;
	      else
		rlist->tl = pcb->ready_entry.prev;
	    }
	  else
	    {
              rlist->hd = pcb->ready_entry.nxt;
	      if(rlist->hd)
		rlist->hd->ready_entry.prev = NULL;
	      else
		zzdp_look_aside ^= RLIST_PRI_MASK(pcb);
	    }
	}
      pcb->ready_entry.is_ready = (OSBOOLEAN) 0;
    }
  zzquit_int();	/* swap. */  
}

/*
** Make some other non interrupt process not ready.
**
** The caller may be any process type.
** The target process must not be the calling process.
** It is safe to call this function even if the target process
** was "not ready" prior to the call.
**
** The state variable in the pcb must have been set by the caller
** to some "not ready" state prior to this function being executed.
**
** Interrupts must be disabled when this function is called.
** Interrupts are disabled when it returns.
*/

static void
make_otherpri_not_ready(struct _pcb *pcb)
{
  register struct ready_list_entry *rlist;

#ifdef OSE_SMP
  assert(pcb->cpu_id == zzkrn_cpu_id());
#endif

  if(pcb->ready_entry.is_ready)
    {
      if(pcb->priority < 32)
	{
	  rlist = pcb->ready_entry.rlist;
	  if(pcb->ready_entry.prev)
	    {
              pcb->ready_entry.prev->ready_entry.nxt = pcb->ready_entry.nxt;
	      if(pcb->ready_entry.prev->ready_entry.nxt)
		pcb->ready_entry.nxt->ready_entry.prev = pcb->ready_entry.prev;
	      else
		rlist->tl = pcb->ready_entry.prev;
	    }
	  else
	    {
              rlist->hd = pcb->ready_entry.nxt;
	      if(rlist->hd)
		rlist->hd->ready_entry.prev = NULL;
	      else
		zzdp_look_aside ^= RLIST_PRI_MASK(pcb);
	    }
	}
      pcb->ready_entry.is_ready = (OSBOOLEAN) 0;

      /*
      ** We must request a dispatch if operating on the current
      ** non_interrupt process or if we have interrupted the dispatcher
      ** search phase.
      */
      if(zzcur_ni == pcb || zzww.dp_state == 2)
	zzww.dp_req = (OSBOOLEAN) 1;
      return;
    }
}

/*
** Abort a pending swapout, making current non interrupt process current again.
**
** The caller must be a prioritized or background process, and it
** must currently be in the swap out phase of zzquit_int. The caller 
** must further not have killed himself.
**
** The purpose of returning to the ready state is to be able to 
** handle non-fatal errors detected in the dispatcher save phase,
** e.g. user stack mark errors.
**
** The calling process may be ready or not ready.
**
** The status variable in the pcb will be set to a "ready" state 
** by this function at return.
**
** Interrupts must be disabled when this function is called.
** Interrupts are still disabled when it returns.
**
*/

static void
make_mypri_current(void)
{ 
  register struct _pcb *qpcb = zzcur_p; 
  register struct ready_list_entry *rlist = qpcb->ready_entry.rlist;

  /* This is the current non-interrupt process. */
  zzcur_ni = qpcb;

  /*
  ** Remove the process from the ready list.
  ** If we got time sliced, or did a set_pri yield, the process is ready but 
  ** not at the head of the ready list. We should not run a process that isn't
  ** at the head of the list.
  */
  if(qpcb->ready_entry.is_ready)
    {
      if(qpcb->priority < 32) 
	{ 
	  if(qpcb->ready_entry.nxt)
	    {
	      qpcb->ready_entry.nxt->ready_entry.prev = qpcb->ready_entry.prev;
	    }
	  else
	    {
	      rlist->tl = qpcb->ready_entry.prev;
	    }
	  if(qpcb->ready_entry.prev)
	    {
	      qpcb->ready_entry.prev->ready_entry.nxt = qpcb->ready_entry.nxt;
	    }
	  else
	    {
	      rlist->hd = qpcb->ready_entry.nxt;
	      
	      if(NULL == rlist->hd)
		{
		  zzdp_look_aside ^= RLIST_PRI_MASK(qpcb);
		}
	    }
	}

      /*
      ** If we have got a timeout while swapping out, tmo_active is still
      ** set. We must clear it here, to prevent kill_proc from trying to
      ** cancel the already removed timer. But if the timer has not yet
      ** expired, we must cancel it instead.
      */
      if(qpcb->tmo_active)
	{
	  if(qpcb->status & ST_TIMEOUT)
	    {
	      /* The timer has expired. */
	      qpcb->tmo_active = 0;
	    }
	  else
	    {
	      /* The timer has not yet expired. */
	      cancel_timeout();
	    }
	}
    }
   else
    {
      /* If the process wasn't ready, cancel any pending timeout. */
      if(qpcb->tmo_active)
	{
	  cancel_timeout();
	}
      /*
      ** If the process was waiting for a semaphore, remove it from the
      ** semaphore queue.
      */
      else if(qpcb->status & ST_SEMAPHORE)
	{
	  kill_proc_cancel_sem(qpcb);
	}
    }
  
  
  /*
  ** Make the current process ready.
  */
  /* Set state to ready */
  qpcb->status = (pr_state_t) ST_INIT;
  qpcb->ready_entry.is_ready = (OSBOOLEAN) 1;

  /* Insert a prioritized process into the proper ready list. */
  if(qpcb->priority < 32) 
    { 
      if(rlist->hd) 
	{ 
	  qpcb->ready_entry.nxt = rlist->hd; 
	  rlist->hd->ready_entry.prev = qpcb; 
	} 
      else 
	{ 
	  rlist->tl = qpcb; 
	  qpcb->ready_entry.nxt = NULL; 
	  zzdp_look_aside |= RLIST_PRI_MASK(qpcb); 
	} 
      rlist->hd = qpcb; 
      qpcb->ready_entry.prev = NULL; 
    } 

  /* We are no longer in the dispatcher. */
  zzww.dp_state = 0;
}


/*
** Make some other non interrupt process ready and run the dispatcher
** if required.
**
** The caller may be any process type.
** The target process must not be the calling process.
** It is safe to call this function even if the target process
** was "ready" prior to the call.
**
** The state variable in the pcb must have been set by the caller
** to some "ready" state prior to this function being executed.
** That is, pcb->status < ST_READY.
**
** Interrupts must be disabled when this function is called.
** Interrupts are disabled when it returns.
**
** static void make_otherpri_ready(struct _pcb *pcb);
*/

static void
make_otherpri_ready(struct _pcb *pcb)
{
  register struct ready_list_entry *rlist;

#ifdef OSE_SMP
  assert(pcb->cpu_id == zzkrn_cpu_id());
#endif

  if(!pcb->ready_entry.is_ready)
    {
      pcb->ready_entry.is_ready = (OSBOOLEAN) 1;
      if(pcb->priority < 32)
	{
	  rlist = pcb->ready_entry.rlist;
	  if(rlist->hd)
	    {
	      pcb->ready_entry.prev = rlist->tl;
	      rlist->tl->ready_entry.nxt = pcb;
	      rlist->tl = pcb;
	    }
	  else
	    {
	      rlist->hd = rlist->tl = pcb;
	      pcb->ready_entry.prev = NULL;
	      zzdp_look_aside |= RLIST_PRI_MASK(pcb);
	    }
	  pcb->ready_entry.nxt = NULL;
	  if(zzcur_ni->priority > pcb->priority)
	    {
	      if(zzcur_p->priority > 32)
		zzww.dp_req = (OSBOOLEAN) 1;
	      else
		zzquit_int();	/* swap. */
	    }
	}
    }
}

/*
** Make some other non interrupt process ready but don't run the dispatcher.
**
** The caller may be any process type.
** The target process must not be the calling process.
** It is safe to call this function even if the target process
** was "ready" prior to the call.
**
** The state variable in the pcb must have been set by the caller
** to some "ready" state prior to this function being executed.
** That is, pcb->status < ST_READY.
**
** Interrupts must be disabled when this function is called.
** Interrupts are disabled when it returns.
*/

static void
make_otherpri_ready_noswap(struct _pcb *pcb)
{
  register struct ready_list_entry *rlist;

#ifdef OSE_SMP
  assert(pcb->cpu_id == zzkrn_cpu_id());
#endif

  if(!pcb->ready_entry.is_ready)
    {
      pcb->ready_entry.is_ready = (OSBOOLEAN) 1;
      if(pcb->priority < 32)
	{
	  rlist = pcb->ready_entry.rlist;
	  if(rlist->hd)
	    {
	      pcb->ready_entry.prev = rlist->tl;
	      rlist->tl->ready_entry.nxt = pcb;
	      rlist->tl = pcb;
	    }
	  else
	    {
	      rlist->hd = rlist->tl = pcb;
	      pcb->ready_entry.prev = NULL;
	      zzdp_look_aside |= RLIST_PRI_MASK(pcb);
	    }
	  pcb->ready_entry.nxt = NULL;
	  if(zzcur_ni->priority > pcb->priority)
	    {
	      zzww.dp_req = (OSBOOLEAN) 1;
	    }
	}
    }
}

/*
** Request a later wakeup call for an interrupt process due to
** a fast semaphore operation.
**
** This wakeup utility function may be called only from interrupt level
** and with interrupts completely disabled.
**
** The target process must have the same or lower interrupt priority
** than the caller.
** (Otherwise a wakeup call should be immediately executed instead of
** being queued.)
** The target process may be the calling process, even if this is an
** odd case.
*/

static void
request_int_semwakeup(struct _pcb *pcb)
{
  register U16 wake_sig_cnt;
  register U16 wake_sem_cnt;
  register struct wakeup_list_entry *rlist;

  /* Get current counters of requested wakeup calls. */
  wake_sig_cnt = pcb->wake_sig_cnt;
  wake_sem_cnt = pcb->wake_sem_cnt;

  /* Bump counter of requested wakeup calls. */
  pcb->wake_sem_cnt++;

  /* insert pcb into wakeup list if this is the first wakeup call. */
  if(!(wake_sig_cnt | wake_sem_cnt))
    {
      rlist = DEVOLATILE(struct wakeup_list_entry *,
			 SMP_CODE_YES_NO(&zzww.wakeup_list[pcb->priority & 31],
					 pcb->ready_entry.rlist));

      /* Flag pri level as requested if previously unflagged. */
      zzww.wk_look_aside |= rlist->pri_mask;
      pcb->ready_entry.nxt = rlist->hd;
      rlist->hd = pcb;
    }
}

/*
** Request a later wakeup call for an interrupt process due to
** a signal sent.
**
** This wakeup utility function may be called only from interrupt level
** and with interrupts completely disabled.
**
** The target process must have the same or lower interrupt priority
** than the caller.
** (Otherwise a wakeup call should be immediately executed instead of
** being queued.)
** The target process may be the calling process, even if this is an
** odd case.
*/

static void
request_int_sigwakeup(struct _pcb *pcb)
{
  register U16 wake_sig_cnt;
  register U16 wake_sem_cnt;
  register struct wakeup_list_entry *rlist;

  /*
  ** If the process' is_deaf flag is set it means that we shouldn't
  ** wake it up when sending signals to it.
  */
  if(pcb->is_deaf)
    return;
  
  /* Get current counters of requested wakeup calls. */
  wake_sig_cnt = pcb->wake_sig_cnt;
  wake_sem_cnt = pcb->wake_sem_cnt;

  /* Bump counter of requested wakeup calls. */
  pcb->wake_sig_cnt++;

  /* insert pcb into wakeup list if this is the first wakeup call. */
  if(!(wake_sig_cnt | wake_sem_cnt))
    {
      rlist = DEVOLATILE(struct wakeup_list_entry *,
			 SMP_CODE_YES_NO(&zzww.wakeup_list[pcb->priority & 31],
					 pcb->ready_entry.rlist));
      
      /* Flag pri level as requested if previously unflagged. */
      zzww.wk_look_aside |= rlist->pri_mask;
      pcb->ready_entry.nxt = rlist->hd;
      rlist->hd = pcb;
    }
}

#ifdef OSE_SMP
/*
** Request a later wakeup call for a timer process due to a tick call.
**
** This wakeup utility function may be called only from interrupt level
** and with interrupts completely disabled.
**
** The target process must have the same or lower interrupt priority
** than the caller.
** (Otherwise a wakeup call should be immediately executed instead of
** being queued.)
*/

static void
request_timer_tickwakeup(struct _pcb *pcb)
{
  U16 queued;
  struct wakeup_list_entry *rlist;

  /* Decide if the process is already present in the wakeup list. */
  queued = pcb->wake_sig_cnt | pcb->wake_sem_cnt | pcb->wake_tick_cnt;

  /* Bump counter of requested wakeup calls. */
  pcb->wake_tick_cnt++;

  /* insert pcb into wakeup list if this is the first wakeup call. */
  if(!queued)
    {
      rlist = DEVOLATILE(struct wakeup_list_entry *,
			 &zzww.wakeup_list[zzuu.timer_int_priority]);
      /* Flag pri level as requested if previously unflagged. */
      zzww.wk_look_aside |= rlist->pri_mask;
      pcb->ready_entry.nxt = rlist->hd;
      rlist->hd = pcb;
    }
}

#endif

/*
** Request a later wakeup call for a timer process due to
** a fast semaphore operation.
**
** This wakeup utility function may be called only from interrupt level
** and with interrupts completely disabled.
**
** The target process must have the same or lower interrupt priority
** than the caller.
** (Otherwise a wakeup call should be immediately executed instead of
** being queued.)
** The target process may be the calling process, even if this is an
** odd case.
*/

static void
request_timer_semwakeup(struct _pcb *pcb)
{
  U16 queued;
  struct wakeup_list_entry *rlist;

  /* Decide if the process is already present in the wakeup list. */
  queued = pcb->wake_sig_cnt | pcb->wake_sem_cnt |
    SMP_CODE_YES_NO(pcb->wake_tick_cnt, 0);

  /* Bump counter of requested wakeup calls. */
  pcb->wake_sem_cnt++;

  /* insert pcb into wakeup list if this is the first wakeup call. */
  if(!queued)
    {
      rlist = DEVOLATILE(struct wakeup_list_entry *,
			 &zzww.wakeup_list[zzuu.timer_int_priority]);
      /* Flag pri level as requested if previously unflagged. */
      zzww.wk_look_aside |= rlist->pri_mask;
      pcb->ready_entry.nxt = rlist->hd;
      rlist->hd = pcb;
    }
}

/*
** Request a later wakeup call for a timer process due to
** a signal sent.
**
** This wakeup utility function may be called only from interrupt level
** and with interrupts completely disabled.
**
** The target process must have the same or lower interrupt priority
** than the caller.
** (Otherwise a wakeup call should be immediately executed instead of
** being queued.)
** The target process may be the calling process, even if this is an
** odd case.
*/

static void
request_timer_sigwakeup(struct _pcb *pcb)
{
  U16 queued;
  struct wakeup_list_entry *rlist;

  /*
  ** If the process' is_deaf flag is set it means that we shouldn't
  ** wake it up when sending signals to it.
  */
  if(pcb->is_deaf)
    return;
  
  /* Decide if the process is already present in the wakeup list. */
  queued = pcb->wake_sig_cnt | pcb->wake_sem_cnt |
    SMP_CODE_YES_NO(pcb->wake_tick_cnt, 0);

  /* Bump counter of requested wakeup calls. */
  pcb->wake_sig_cnt++;

  /* insert pcb into wakeup list if this is the first wakeup call. */
  if(!queued)
    {
      rlist = DEVOLATILE(struct wakeup_list_entry *,
			 &zzww.wakeup_list[zzuu.timer_int_priority]);
      /* Flag pri level as requested if previously unflagged. */
      zzww.wk_look_aside |= rlist->pri_mask;
      pcb->ready_entry.nxt = rlist->hd;
      rlist->hd = pcb;
    }
}

#ifdef ASSERT
  /* Assert ready list */
static void
assert_ready_list(void)
{
  OSPRIORITY pri;
  PCB_INDEX_T pindex;
  U32 pri_mask = zzdp_look_aside;
  struct _pcb *pcb;
  int errcount = 0;

  for(pri = 0; pri < 32; pri++)
    {
      if(((pri_mask & 0x80000000) && !zzww.ready_list[pri].hd) ||
	 (!(pri_mask & 0x80000000) && zzww.ready_list[pri].hd))
	{

	  if(errcount == 0)
	    {
	      ramlog_printf("\nzzgo_search: cpu %d\n"
			    "   zzww.last_cur_ni %#lx ni->magic %#x ni->priority %d\n"
			    "   ni->status %#x ni->name %.14s.\n",
			    zzkrn_cpu_id(),
			    zzww.last_cur_ni, zzww.last_cur_ni->magic,
			    zzww.last_cur_ni->priority, zzww.last_cur_ni->status,
			    DECONST(char *, zzww.last_cur_ni->name));
	    }
	  errcount++;
	  ramlog_printf("\nzzgo_search: mismatch on prio %d zzdp_look_aside %#.8lx,\n"
			"   zzww.ready_list[pri].hd %#lx zzww.ready_list[pri].tl %#lx,\n",
			pri, zzdp_look_aside,
			zzww.ready_list[pri].hd, zzww.ready_list[pri].tl);
	  if(zzww.ready_list[pri].hd)
	    ramlog_printf("Pid at hd: %#x\n", zzww.ready_list[pri].hd->pid);
	  if(zzww.ready_list[pri].tl)
	    ramlog_printf("Pid at tl: %#x\n", zzww.ready_list[pri].tl->pid);
	  
	  ramlog_printf("ready list prio %d @ cpu%d:%s\n",
			pri, zzkrn_cpu_id(),
			zzww.ready_list[pri].hd ? "" : " Empty");
	  for(pcb = zzww.ready_list[pri].hd; pcb; pcb = pcb->ready_entry.nxt)
	    {
	      ramlog_printf("   pcb %#lx pid %#lx pri %d state %#x prev %#lx nxt %#lx name %.14s.\n",
			    pcb, pcb->pid, pcb->priority, pcb->status,
			    pcb->ready_entry.prev, pcb->ready_entry.nxt,
			    DECONST(char *, pcb->name));
	    } /* for */
	  ramlog_printf("All processes @ priority %d:\n", pri);
	  for(pindex = zzuu.next_free_pix - 1; pindex; --pindex)
	    {
	      pcb = zzuu.pix_list[pindex];
	      if((pcb->magic) == (MAGIC_PCB) && pcb->priority == pri)
		{
		  ramlog_printf("pid %x cpu %d blk %x cpu %d status %#x",
				pcb->pid, pcb->cpu_id, pcb_to_pid(pcb->block),
				pcb->block->cpu_id, pcb->status);
		  ramlog_printf(" is_ready %d rlist %x name %s\n",
				pcb->ready_entry.is_ready,
				pcb->ready_entry.rlist, pcb->name);
		  if(pcb->status < ST_READY)
		    {
		      if(pcb->ready_entry.prev)
			ramlog_printf("ready_entry.prev->pid: %#x\n",
				      pcb->ready_entry.prev->pid);
		      if(pcb->ready_entry.nxt)
			ramlog_printf("ready_entry.nxt->pid: %#x\n",
				      pcb->ready_entry.nxt->pid);
		      if(pcb->mailbox)
			ramlog_printf("sig_cnt_queued: %d first signo %#x\n",
				      pcb->sig_cnt_queued,
				      adm_to_sig(pcb->mailbox)->sig_no);
		    }
		}
	    } /* for */
	}
      pri_mask = pri_mask << 1;
    } /* for */
  assert(0 == errcount);
}
#else
#define assert_ready_list()
#endif	/* ASSERT */

/*
** Check the timer event lists and call the power save handler if the
** time to the next event is longer than (or equal to) the threshold
** value stated in an earlier call to set_halt_condition().
**
** Return a nonzero value to indicate a dispatcher restart request.
*/

#if USE_POWER_SAVE > 0
static int
sys_power_save(void)
{
  enum timer_mask { TI_PROC_BIT = 1, TIMEOUT_BIT = 2, TMO_SIG_BIT = 4};
  int timer_mask = 0;
  OSTICK idle_ticks;

  /* Is power save mode disabled? */
  if(zzuu.power_save_threshold == (OSTICK)-1)
    {
      /* Yes. Just return. */
      return 0;
    }

  /* Disable interrupts. */
  LOCK();
  
  /* If there are no planned timer events, sleep forever. */
  if(zzuu.timer_process_count)
    timer_mask |= TI_PROC_BIT;
  if(zzww.timeout_count)
    timer_mask |= TIMEOUT_BIT;
  if(zzuu.tmo_signal_count)
    timer_mask |= TMO_SIG_BIT;
  if(!timer_mask)
    {
      /* Check if we need to jump back to the scheduler. */
      if(zzww.dp_idle_interrupted)
	{
	  /* Enable interrupts. */
	  UNLOCK();

	  /* Dispatcher restart request. */
	  return 1;
	}

      /* Tell the power save handler that it may sleep as long as it wants. */
      idle_ticks = (OSTICK)-1;
    }
  else
    /* Find next timer event. */
    {
      unsigned oldtmo = zzww.ticks & zzuu.curtmo_mask;
      unsigned slot = oldtmo;

      idle_ticks = 0u;

      /* Scan the timer event lists. */
      for(;;)
	{
	  slot = (slot + 1) & zzuu.curtmo_mask;

	  /* Check if we need to jump back to the scheduler. */
	  if(zzww.dp_idle_interrupted)
	    {
	      /* Enable interrupts. */
	      UNLOCK();

	      /* Dispatcher restart request. */
	      return 1;
	    }

	  /*
	  ** We don't use the tmolist spinlock here since powersave
	  ** handling is not feasible in SMP type of systems anyhow.
	  ** Is this correct? @frbr@
	  */
	  /*
	  ** End the loop at the first found timer event.
	  ** The tmslist has 256 entries, tmolist and tos_tick_list
	  ** may be longer.
	  */
	  if((timer_mask & TI_PROC_BIT) && idle_ticks < 256 &&
	     zzuu.tmslist[slot & 255])
	    break;
	  if((timer_mask & TIMEOUT_BIT) && zzww.tmolist[slot])
	    break;
	  if((timer_mask & TMO_SIG_BIT) && zzuu.tos_tick_list[slot].head)
	    break;

	  /*
	  ** Have we tested all slots whithout finding any future
	  ** timer events?
	  */
	  if(slot == oldtmo)
	    {
	      /*
	      ** Normally, we should not arrive here. It indicates that
	      ** a timer list and its counter are not updated atomically.
	      ** Tell the power save handler to sleep as long as it wants.
	      */
	      idle_ticks = (OSTICK)-1;
	      break;
	    }

	  /* Enable interrupts. */
	  UNLOCK();
      
	  idle_ticks++;

	  /* Disable interrupts. */
	  LOCK();
	}
    }

  assert(zzww.dp_idle_interrupted == 0);

  /* Interrupts are still disabled here. */
  
  /* Decide if we are to sleep or not. */
  if(idle_ticks >= zzuu.power_save_threshold)
    {
      OSTICK t;
      
      /*
      ** When the cpu profiler is running, we must process each tick to
      ** get proper results.
      */
      if(zzuu.tickhook_enabled)
	{
	  idle_ticks = 0;
	}
      
      /* Zzzzzz..... */
      t = handlerp.zzcall_power_save(idle_ticks);

      /* Did we sleep longer than excpected? */
      if(t > idle_ticks)
	{
	  /* What should we do then? Just truncate? Or?... @frbr@ */
	  t = idle_ticks;
	}

      /* Update the kernel's time. */
      zzuu.ticks += t;
      zzuu.power_saved_ticks += t;
      if(zzuu.longest_power_nap < t)
	zzuu.longest_power_nap = t;

      /* Enable interrupts. */
      UNLOCK();
      
      /* Dispatcher restart request. */
      return 1;
    }

  /* Enable interrupts. */
  UNLOCK();
  
  /* We have done nothing, so we should still remain in the idle loop. */
  return 0;
}
#endif

/*
** Dispatcher search phase.
**
** Locate the highest priority non interrupt process and run it.
** If no ready process can be found then dwell in the idle loop.
**
** During this phase, the current process is set to be the
** idle process. This is done so that we won't miss any
** dispatch operations. The idle process has a priority lower than
** any other process, so any process becoming ready at this time
** generates a dispatch request (zzww.dp_req).
*/

void
zzgo_search(void)
{
  register PCB_INDEX_T found_p;
  register struct _pcb *found_pcb;
  register struct _pcb *end_search;
  int first_time = 1;

  /* The dispatcher state in zzww.dp_state must be updated before UNLOCK(). */
  LOCK();

  /*
  ** The process (formely know as the current) is no longer in
  ** possession of the CPU and the target interface has made sure that
  ** we are currently using the idle process' stack.
  */
  SMP_CODE(zzcur_p->alloced_cpu = 0);

 go_search_again:

#if POWERFAIL_HOOK_CALLED
  /* Call power fail handler if required. */

  if(unlikely(zzuu.pfail_req))
    {
      /* Drop the power fail required flag. */
      zzuu.pfail_req = (OSBOOLEAN) 0;

      /* Call the power fail handler. */
      zzpower_off();
    }
#endif	/* POWERFAIL_HOOK_CALLED */

  /* Clear restart request flag. */
  
  zzww.dp_req = (OSBOOLEAN) 0;

  assert_ready_list();

  /* Search for next prioritized process to run if any is ready.
  ** The most significant bit of the look aside register corresponds to
  ** the highest priority process, i.e. bit 31 set corresponds to priority 0
  ** being ready.
  ** The goal is to find the first most significant bit set, and subtract it
  ** from 31. The result is the priority of the process to run.
  */
  found_p = (PCB_INDEX_T) FIND_FIRST_BIT(zzdp_look_aside);

  if(found_p != 32)
    {
      found_pcb = zzww.ready_list[found_p].hd;
      assert(found_pcb);
    }
  else
    {
#ifdef  BG_TIMESLICE_TRACE
      int ran_bg = 0;
#endif

      /* Note that we are now in dispatcher search phase. */
      zzww.dp_state = (U8) 2;
      
      /* Set current process to idle process. */
      zzcur_ni = zzww.idle_pcb;
      set_zzcur_p(zzcur_ni);

      /*
      ** Else search for next background process to run.
      **
      ** We search all background processes. If none ready then
      ** we go back and search again. This is important, because we don't
      ** get a zzww.dp_req when an interrupt process makes another background
      ** process ready at this point.
      **
      ** We could do this differently, but that would cost us more
      ** work in the interrupts or in lots of other places, neither of
      ** which was desired.
      */

      UNLOCK();

      /*
      ** Check if we need to poll out any ose_fsend signals before
      ** scheduling a background process. If we don't do this a
      ** background process that won't give up the cpu will prevent
      ** any further fsend polling.
      */
      if(unlikely(zzww.bg_receive_poll))
	{
	  /* Do not check the hw poll. */
	  if(check_receive_poll(zzcur_p, 0))
	    {
	      zzww.bg_receive_poll = (OSBOOLEAN) 0;
	      LOCK();
	      goto go_search_again;
	    }
	  zzww.bg_receive_poll = (OSBOOLEAN) 0;
	}

      /* No interrupt has occured yet in the idle loop. */
      zzww.dp_idle_interrupted = 0;

      found_pcb = zzww.bg_list.hd;
      end_search = (found_pcb != NULL)
	? found_pcb->ready_entry.prev
	: NULL;

      /* This is the idle loop. */
      for(;;)
	{
#ifdef OSE_SMP
	  /*
	  ** During the boot, poll the kernel event queue.
	  ** This may be required before the ipi mecanism is operable.
	  ** We are not serving wakeup calls as we are on the wrong stack.
	  ** Any wakeup calls will be queued until the first hardware
	  ** interrupt is received.
	  */
	  if(unlikely(!zzww.ipi_operable))
	    {
	      LOCK();
	      (void)local_krn_evt_notify();
	      UNLOCK();
	    }
#endif
	  if(zzww.dp_req)
	    {
	      LOCK();
	      goto go_search_again;
	    }

	  if(found_pcb)
	    {
	      struct _pcb *search_next_copy;
	      
	      if(found_pcb->status < 4) /* Ready if status word < 4. */
		{
		  LOCK();
		  if(!zzww.dp_req)
		    {
#ifdef  BG_TIMESLICE_TRACE
		      ramlog_printf("RUN BG tick %lu pid 0x%x"
				    " name %s (slice owner 0x%x)\n",
				    zzuu.ticks,
				    found_pcb->pid,
				    found_pcb->name,
				    zzww.bg_list.hd->pid);
		      ran_bg = 1;
#endif
		      zzww.bg_list.search_next = found_pcb->ready_entry.nxt;
		      break;              /* Ok to swap in found_pcb. */
		    }
		  goto go_search_again;
		}

	      /*
	      ** Choose another background process, until we find one that
	      ** is ready. If the current timeslice expires, tick will set
	      ** dp_req to request a dispatch. In that case, the selection
	      ** of a found_pcb below will not matter.
	      ** First, take a sample of the search_next.
	      */
	      search_next_copy = zzww.bg_list.search_next;
	      if(found_pcb == zzww.bg_list.hd && search_next_copy)
		{
		  if(found_pcb != search_next_copy)
		    {
		      found_pcb = search_next_copy;

		      /*
		      ** found_pcb is not NULL here, but
		      ** zzww.bg_list.search_next can have been set to NULL
		      ** by a timeslicing tick interrupt. In that case,
		      ** dp_req is also set, so it is OK.
		      */
		    }
		  else
		    found_pcb = found_pcb->ready_entry.nxt;
		  end_search = found_pcb->ready_entry.prev; 
		  zzww.bg_list.search_next = NULL;
		}
	      else if(found_pcb == end_search)
		{
		  found_pcb = NULL; /* idle */
		}
	      else
		{
		  found_pcb = found_pcb->ready_entry.nxt;
		}

#ifdef  BG_TIMESLICE_TRACE
	      if(ran_bg && found_pcb == NULL && !zzww.dp_req)
		{
		  ramlog_printf("IDLE @ tick %lu\n", zzuu.ticks);
		  ran_bg = 0;
		}
#endif
	    }
	  else
	    {
	      if(first_time)
		{
		  first_time = 0;

#if OSE_DEBUG > 0
		  if(likely(zzuu.trace_all_swaps))
		    {
		      zzfast_swap_trace(pcb_to_debug_pcb(zzww.last_cur_ni),
					pcb_to_debug_pcb(zzcur_ni));
		    }
		  else if(zzuu.debugger_enabled)
		    {
		      /* check for trace swap in request. */
		      if((zzcur_ni->debug_flags | zzww.last_cur_ni->debug_flags) & 0x04)
			zzswapin_dbghook(pcb_to_debug_pcb(zzcur_ni),
					 pcb_to_debug_pcb(zzww.last_cur_ni));
		    }
		  /* Call the swap in hook in the profiler, indicating the idle process. */
		  if(zzuu.inthook_enabled & 0x01)
		    zzprof_dbghook((OSPRIORITY) 33, pcb_to_debug_pcb(zzcur_ni));
#endif
		  /* Save the previous process for the debugger. */
		  zzww.last_cur_ni = zzcur_ni;
		}

              
              /* Check if we need to poll out any ose_fsend() signal. */
              /* Do not check the hw poll. */
              if(check_receive_poll(zzcur_p, 0)) 
              {
                 LOCK();
                 goto go_search_again;
              }
              
	      /* Idle loop is running. */
#if USE_POWER_SAVE > 0
	      /* Try to enter power save mode. */
	      if(sys_power_save())
		{
		  /* Got a dispatch request and/or an interrupt here. */
		  LOCK();
		  goto go_search_again;
		}
#endif
	      /* Allow the target interface to check for interrupts. */
	      SMP_CODE(if(likely(zzww.ipi_operable)))
	        {
		  if(likely((U32)zzuu.ticks != 0))
		    {
		      LOCK();
		      if(zzww.dp_req || zzww.dp_idle_interrupted)
			goto go_search_again;
#ifdef OSE_SMP
                      if(!zzww.hw_receive_poller)
#endif
                         zzidle();
		      UNLOCK();
		    }
	        }
	      /* Prepare a new scan of all background processes */
	      found_pcb = zzww.bg_list.hd;
	      end_search = (found_pcb != NULL)
		? found_pcb->ready_entry.prev
		: NULL;
	    }

#ifdef OSE_SMP
	  /*
	  ** We need to make sure that idle loop doesn't continue to use an
	  ** address mapping of a deleted segment. See: delete_segment().
	  */
	  if(unlikely(zzww.leave_segid_now))
	    {
	      if(zzww.leave_segid_now == zzww.cur_segid)
		{
		  /*
		  ** Switch to the system segment since the segment we are
		  ** currently running on is 'soon to be removed'.
		  **
		  ** Note: We do not set cur_segid before calling the
		  ** select segment hook. May this cause any problem for
		  ** the hook? @frbr@
		  */
		  handlerp.zzcall_select_segment(0);

		  /*
		  ** Set cur_segid to zero to tell delete_segment() that
		  ** we are no longer using the segment.
		  */
		  zzww.cur_segid = 0;
		}
	    }
#endif

	} /* End of idle loop */
    }

  /* Note that we are now in restore phase of dispatcher. */
  zzww.dp_state = (U8) 3;

  /* Set current process to the swapped in process. */
  set_zzcur_p(found_pcb);
  zzcur_ni = found_pcb;
  zzcurrent_uarea = found_pcb->uarea;

  /* The current process is now in possession of the CPU. */
  SMP_CODE(zzcur_p->alloced_cpu = zzkrn_cpu_id()+1);
  
  UNLOCK();

#if OSE_DEBUG > 0
  /* Call the debugger hook. */
  if(likely(zzuu.trace_all_swaps))
    {
      zzfast_swap_trace(pcb_to_debug_pcb(zzww.last_cur_ni),
			pcb_to_debug_pcb(zzcur_ni));
    }
  else if(zzuu.debugger_enabled)
    {
      /* check for trace swap in request. */
      if((found_pcb->debug_flags | zzww.last_cur_ni->debug_flags) & 0x04)
	zzswapin_dbghook(pcb_to_debug_pcb(zzcur_ni),
			 pcb_to_debug_pcb(zzww.last_cur_ni));
    }

  /*
  ** Call the swap in hook in the profiler, indicating a prioritized
  ** or background process.
  */
  if(unlikely(zzuu.inthook_enabled & 0x01))
    zzprof_dbghook((OSPRIORITY) zzcur_ni->priority,
		   pcb_to_debug_pcb(zzcur_ni));
#endif
  
  /* Select the memory segment of the current process. */
#if defined(OSPP) || defined(OSE_SMP)
  /* On PPC PEM, always select proper segment to keep the hash table clean. */
  if(zzww.cur_segid != found_pcb->segid)
    {
      handlerp.zzcall_select_segment(zzww.cur_segid = found_pcb->segid);
    }
#else
  if(found_pcb->segid)
    {
      if(zzww.cur_segid != found_pcb->segid)
	{
	  handlerp.zzcall_select_segment(zzww.cur_segid = found_pcb->segid);
	}
    }
#endif

  /* Pop current context from uarea. */
  zzrestore_context();
}


#ifdef OSE_SMP
void 
zzkrn_activate_cpu(void)
{
  struct _pcb *pcb;

  zzww.ticks = (OSTICK)zzuu.ticks;
  zzww.state = CPU_STATE_ACTIVE;

  /* Start sysd_lite on this cpu. */
  pcb = pid_to_pcb(zzww.sysd_lite_);
  if(!--pcb->st_cnt)
    {
      pcb->status &= ~ST_STOPPED;
      if(pcb->status < ST_READY)
	{
	  make_otherpri_ready_noswap(pcb); /* noswap */
	}
    }

  zzgo_search();
  /* NOTREACHED */
}
#endif

/*
**===========================================================================
** 9.2.x		zzquit_int()
**===========================================================================
** Description:	Swaps out the current process if needed and starts search
**		for a new process to run.
**		This function is only called from prioritized or background
**		processes. Interrupts are already disabled and process status
**		are set to indicate why the caller is not ready to run.
**
** Parameters:	-
**
** Returns:	Does not return until the calling process is again swapped
**		in. Interrupts are still disabled.
**
** Globals:	
*/

#ifdef _BullseyeCoverage
#pragma BullseyeCoverage off
#endif

void
zzquit_int(void)
{
  /*
   * Skip swap if dispatcher is locked.
   * Set zzww.dp_req in case a non interrupt process is running.
   */

  if(zzdispatcher_locked)
    {
      zzww.dp_req = (OSBOOLEAN) 1;
      /*
      ** This flag was set in zzinit_int(), assuming that a dispatch
      ** operation is about to execute. Since we are now aborting that operation
      ** we need to reset the dispatcher to the idle state so that future
      ** checks on dp_state will cause a dispatch request.
      */
      zzww.dp_state = (U8) 0;
      return;
    }

restart_quit_int:

  /* Note that we are in the dispatcher save phase so we won't be interrupted. */
  zzww.dp_state = (U8) 1;

  /* Save the previous process for the debugger. */
  zzww.last_cur_ni = zzcur_ni;

  /* Enable interrupts. */
  UNLOCK();
#ifdef MORE_BENGT_DEBUG
/* This is junk code which can be safely removed at any time. ?? */
  ramlog_printf("CPU%d: swapped out %#x %s state: %#x\n", zzkrn_cpu_id(),
		pcb_to_pid(zzcur_p), zzcur_p->name, zzcur_p->status);
#endif

  /* Call the debugger hook. */
  /* This hook is not used by the current ldm, so we disable it for now. */
#if 0
#if OSE_DEBUG > 0
  if(zzuu.debugger_enabled)
    {
      /* check for trace swap out request. */
      if(zzcur_p->debug_flags & 0x08)
	zzswapout_dbghook(pcb_to_debug_pcb(zzcur_p));
    }
#endif
#endif

  /* Push current context into uarea and go search for new process to run. */
  zzsave_context();

#if 0
#ifdef OSE_SMP
  /* We need this in order to keep the SMP kernel's lock in sync. ?? */
  LOCK();
#endif
#endif

  /* Restart the swap function if dispatcher restart flag was set. */
  if(zzww.dp_req)
      goto restart_quit_int;

#ifdef MORE_BENGT_DEBUG
/* This is junk code which can be safely removed at any time. ?? */
      ramlog_printf("CPU%d: swapped in  %#x %s\r\n", zzkrn_cpu_id(),
		    pcb_to_pid(zzcur_p), zzcur_p->name);
#endif
  /* Note that we are no longer in dispatcher. */
  zzww.dp_state = (U8) 0;
}

#ifdef _BullseyeCoverage
#pragma BullseyeCoverage on
#endif

/*
** Called from the target interface for a newly created process.
**
** The purpose is to check if the dispatcher must be restarted
** due to an interrupt during the swap in phase.
**
** Interrupts must be disabled when this function is called.
*/

void
zzinit_process(void)
{
  /* Restart the swap function if dispatcher restart flag was set. */
#ifdef MORE_BENGT_DEBUG
  /* This is junk code which can be safely removed at any time. ?? */
  ramlog_printf("CPU%d: swapped in %s\r\n", zzkrn_cpu_id(), zzcur_p->name);
#endif
      
  if(zzww.dp_req)
    zzquit_int();

  /* Note that we are no longer in dispatcher. */
  else
    zzww.dp_state = (U8) 0;
}


#ifdef OSE_SMP
U32
zzget_dp_look_aside(cpuid_t cpu)
{
  return dp_look_aside_x(cpu);
}
#endif

#ifdef OSE_SMP
#if 0
/*
** Sysd syncs with all sysd_lite using a signal tranaction.
** For test and debug purpose. May be removed at any time.
*/
static void
sysd_sync(void)
{
  static const SIGSELECT sel_sync[] ={1, SMP_SYNC};
  union SIGNAL *sync_sig = alloc(sizeof(SIGSELECT), SMP_SYNC);
  cpuid_t cpu;

  for(cpu = 0; cpu < zzkrn_num_cpus(); cpu++)
    {
      send(&sync_sig, zzcpu[cpu].sysd_lite_);
      sync_sig = receive(sel_sync);
    } /* for */
  
  free_buf(&sync_sig);
}
#endif
#endif
/*
**===========================================================================
** 9.3  System calls.
**===========================================================================
*/

/*
** This is the wake_up system call.
**
** It reads the current wakeup status, which is:
** Always 1 for bgd and pri processes.
** 0 if a hardware interrupt occurred.
** 1 if wakeup call was caused by a signal sent to the process.
** 2 if wakeup call was caused by a fast semaphore operation.
*/

int
wake_up(void)
{
  return (int) zzwakeflag;
}

#ifdef OSE_SMP
static int
wake_up_usr(void)
{
  struct _pcb *pcb = zzcur_p_usr;

  /*
  ** Prioritized and backgrond processes may run in user mode, and
  ** can then possibly not read the zzwakeflag macro on all 
  ** architectures, so we handle them explicitely.
  */
  if(pcb->priority <= 32)
    {
      return 1;
    }

  /* Handle interrupt processes, running in supervisor mode. */
  return (int) zzwakeflag;
}
#endif

/*
** This is the set_halt_condition system call.
**
** The system call uses the get_sysconf() framework, which means that
** there is no public entrypoint to this function.
** This system call uses function 46 of get_sysconf().
** The user level prototype is:
**    OSTIME set_halt_condition(OSTIME threshold_time);
*/

static OSADDRESS
sysconf_46(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  LOCKSTATE msr;
  OSTICK old_value;
  
  /* Remove ``not used args'' compiler warnings. */
  (void) q;
  (void) r;

  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);

  old_value = zzuu.power_save_threshold;
  
#if USE_POWER_SAVE > 0
  zzuu.power_save_threshold = (OSTICK)p;
#else
  (void) p;
#endif

  /* Enable interrupts. */
  BKL_LOCK_RESTORE(msr);

  return (OSADDRESS)old_value;
}


/*
** This is a helper function to ose_bind_process.
** This function performs the actual move of a process
** from one CPU to another.
** It is called by sysd.
** The dispatcher must be locked by the caller. This will 
** make it possible to protect other code from process migration.
** The dispatcher is temporarily unlocked when migrating background
** processes. As the caller is sysd, we may still be able to follow
** process lists and block lists in other sysd functions.
** Returns 0 if the move is not possible.
** Returns 1 if the move was executed successfully.
*/

#if defined(OSE_SMP_MAX_CPUS) && (OSE_SMP_MAX_CPUS > 1)
static OSBOOLEAN
move_process_to_cpu(struct _pcb *pcb, cpuid_t cpu)
{
  if(pcb->killed)
    return 1; /* Ok, since it's already dead! */

  if(pcb->cpu_id == cpu)
    return 1; /* Ok, since it's already reached its destination! */

  /*
  ** If the process is an interrupt process we don't allow it to
  ** migrate if it's bound to a hardware vector.  I.e. only
  ** soft-interrupt processes (vector == -1) can migrate.
  */
  if(pcb->priority > 32 && pcb->priority < 96 && pcb->vector != 0xffff)
    {
      return 0;
    }

#ifdef OSE_SMP

#if OSE_DEBUG > 0 
  if(pcb->priority < 96 || pcb->priority >= 128)
    {
       /* Only report processes that are still alive. */
      if(pcb->magic == MAGIC_PCB)
        {
          if(pcb->debug_flags & 0x40)
            zzbind_dbghook(pcb_to_debug_pcb(pcb),
                           pcb->cpu_id,
                           cpu);
        }
    }
#endif

  /*
  ** Make sure that the process we want to move is not currently in
  ** possession of it's CPU.  If it is we just have to wait until it
  ** is not. Trying to move someone from one CPU to another who's
  ** currently running (and using the stack) is no good, trust me,
  ** I've tried.../frbr
  ** This only detects running prioritized and background processes./Gunnar
  */
  while(pcb->alloced_cpu)
    {
      ; /* Empty while body */
    }

  if(pcb->priority < 32) /* Priority process */
    {
#if 1
      /*
      ** Currently, we don't boot without this if(). ??
      ** The error may be somewhere else??
      ** This code does not handle timeouts??
      */
      if(zzcpu[cpu].state != CPU_STATE_ACTIVE)
	{
	  cpuid_t old_cpu = pcb->cpu_id;

	  /*
	  ** If the process is on another cpu, tell that one to handle the
	  ** migration.
	  */
	  if(old_cpu != zzkrn_cpu_id())
	    {
	      LOCK();
	      send_krn_event(old_cpu, KRN_EVT_MIGRATE, pcb_to_pid(pcb),
			     (void *)(cpu << 16));
	      UNLOCK();
	    }
	  else
	    {
	      /* The process is on the current cpu. */
	      LOCK();
	      pcb->cpu_id = cpu;
	      pcb->ready_entry.rlist = (struct ready_list_entry *)
		&ready_list_x(cpu)[pcb->priority];
	      UNLOCK();
	    }
	}
      else
#endif
	{
	  union SIGNAL *sigr;
	  PROCESS pid = pcb_to_pid(pcb);
	  static const SIGSELECT sel_reply[] = {1, SMP_PROC_ADD_REPLY};

	  /*
	  ** Order sysd_lite to move it.
	  ** No danger to unlock the dispatcher here, sysd_lite will LOCK_SWAP.
	  */
	  UNLOCK_SWAP();
	  sigr = alloc(sizeof(struct smp_move_proc), SMP_PROC_MOVE_REQUEST);
	  sigr->smp_move_proc.pid = pid;
	  sigr->smp_move_proc.cpu = cpu;
	  send(&sigr,zzcpu[pcb->cpu_id].sysd_lite_);
	  sigr = receive(sel_reply);
	  free_buf(&sigr);
	  LOCK_SWAP();
	}
    }
  else if(pcb->priority == 32) /* Background process */
    {
      union SIGNAL *sigr;
      PROCESS pid = pcb_to_pid(pcb);
      static const SIGSELECT sel_reply[] = {1, SMP_PROC_ADD_REPLY};

      /*
      ** Order sysd_lite to move it.
      ** No danger to unlock the dispatcher here, sysd_lite will LOCK_SWAP.
      */
      UNLOCK_SWAP();
      sigr = alloc(sizeof(struct smp_move_proc), SMP_PROC_MOVE_REQUEST);
      sigr->smp_move_proc.pid = pid;
      sigr->smp_move_proc.cpu = cpu;
      send(&sigr,zzcpu[pcb->cpu_id].sysd_lite_);
      sigr = receive(sel_reply);
      free_buf(&sigr);
      LOCK_SWAP();
    }
  else if(pcb->priority < 96)
    {
      /* Target process is an interrupt process. */
      LOCK();
      pcb->cpu_id = cpu;
      /*
      ** We only set 'rlist' here for debug purposes, the real value
      ** (which is '&zzww.wakeup_list[pcb->priority & 31]'), will be
      ** evaluated each time it's used. This way 'pcb->cpu_id' and
      ** 'pcb->ready_entry.rlist' can't get out of sync when
      ** migrating. @frbr@
      */
      pcb->ready_entry.rlist =
	DEVOLATILE(struct ready_list_entry *,
		   &wakeup_list_x(cpu)[pcb->priority & 31]);
      UNLOCK();
    }
  else if(pcb->priority < 128)
    {
      /*
      ** Target process is a timer process.
      ** It is stopped, so it is not scheduled by the tick, only by 
      ** software wakeup. Can this cause a dual instance problem??
      */
      LOCK();
      pcb->cpu_id = cpu;
      /*
      ** We only set 'rlist' here for debug purposes, the real value
      ** (which is '&zzww.wakeup_list[zzuu.timer_int_priority]'), will
      ** be evaluated each time it's used. This way 'pcb->cpu_id' and
      ** 'pcb->ready_entry.rlist' can't get out of sync when
      ** migrating. @frbr@
      */
      pcb->ready_entry.rlist =
	DEVOLATILE(struct ready_list_entry *,
		   &wakeup_list_x(cpu)[zzuu.timer_int_priority]);
      UNLOCK();
    }
  else
    {
      /* Target process is a phantom process */
      pcb->cpu_id = cpu;
    }

#else
  pcb->cpu_id = cpu;
#endif /* OSE_SMP */

  return 1; /* success */
}
#endif /* defined(OSE_SMP_MAX_CPUS) && (OSE_SMP_MAX_CPUS > 1) */

/*
** sysd performs the actions of ose_bind_process system call.
** This function will move all the processes that belong
** to the provided pid, bid or seg.
** 
** Returns 0 if it was not possible to move a single process or block.
** Returns 1 if one or more processes or blocks could be moved.
*/

#if defined(OSE_SMP_MAX_CPUS) && (OSE_SMP_MAX_CPUS > 1)
static OSBOOLEAN
sysd_bind_process(PROCESS pid, cpuid_t cpu)
{
  struct _pcb *pcb = pid_to_pcb(pid);
  OSBOOLEAN status = 0;
  
  /* If pid is alive now, keep him alive while we do some checks. */
  LOCK_SWAP();

#ifdef OSE_SMP
  /* If the cpu is detached, report failure. */
  if(zzcpu[cpu].state == CPU_STATE_DETACHED)
    {
      status = 0;
      goto out;
    }
#endif

  /* If the object is dead, report success, analogue to send() etc! */
  if(pcb_to_pid(pcb) != pid)
    {
      status = 1;
      goto out;
    }

  /*
  ** Stop the object.
  ** Note: the dispatcher is locked, but the stop will affect other cpus,
  ** but also local timer processes.
  */
  (void)stop_or_intercept(pid, ST_STOPPED, OSE_BIND_PROCESS, 0);
  /* SMP FIXME stop count > 1 ?? */

  if(pcb->magic == MAGIC_PCB) /* Process */
    {
      status = move_process_to_cpu(pcb, cpu);      
    }
  else if(pcb->magic == MAGIC_BCB) /* Block. */
    {
      struct _pcb *tmp_pcb;
      struct _block *bcb = (struct _block *)pcb;

      /* Move all the processes in the block. */
      for(tmp_pcb = bcb->proclist; tmp_pcb; tmp_pcb = tmp_pcb->proclist)
	(void)move_process_to_cpu(tmp_pcb, cpu);

      bcb->cpu_id = cpu;
      status = 1;
    }
  else /* Segment */
    {
      struct _block *bcb;
      int at_least_one_moved_block = 0;

      /* Move all the processes in the segment. */
      for(bcb = zzuu.bcb_list; bcb; bcb = bcb->next)
	{
	  /* Operate only on blocks that are in the segment. */
	  if(bcb->segpid == pid)
            {
              struct _pcb *tmp_pcb;

              for(tmp_pcb = bcb->proclist; tmp_pcb;
		  tmp_pcb = tmp_pcb->proclist)
		{
		  (void)move_process_to_cpu(tmp_pcb, cpu);
                }

              at_least_one_moved_block = 1;
	      bcb->cpu_id = cpu;
            }
        }

      if(at_least_one_moved_block)
        status = 1;
    }

  start_or_resume(pid, ST_STOPPED, OSE_BIND_PROCESS);
 out: ;
  UNLOCK_SWAP();
  return status;
}
#endif	/* if defined(OSE_SMP_MAX_CPUS) && (OSE_SMP_MAX_CPUS > 1) */

/*
** This is the ose_bind_process system call.
** This function will move all the processes that belong
** to the provided pid, bid or seg.
** 
** Returns 0 if it was not possible to move a single process or block.
** Returns 1 if one or more processes or blocks could be moved.
*/

static OSBOOLEAN
zzkrn_bind_process(PROCESS pid, cpuid_t cpu)
{
  struct _pcb *pcb;
  struct _pcb *cur_p;
  OSBOOLEAN status = 0;
  
  cur_p = zzcur_p;

  /*
  ** Refuse to move anything for user mode callers.
  ** These are normally stopped already in the get_sysconf framework.
  */
  if(!cur_p->block->supervisor_mode)
    return 0;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_BIND_PROCESS))
    return 0;
  
  if(cpu >= zzkrn_num_cpus())
    {
      os_error(0, OSE_BIND_PROCESS | OSE_EBAD_PARAMETER, cpu);
      return 0;
    }
#endif

  /* An attempt to move the system segment will result in a warning. */
  pcb = check_pid_bid_seg(pid, OSE_BIND_PROCESS);
  if(pcb == NULL)
    return 0;

  /* If pid is alive now, keep him alive while we do some checks. */
  BKL_LOCK();

  /* If the object is dead, report success, analogue to send() etc! */
  if(pcb_to_pid(pcb) != pid)
    {
      BKL_UNLOCK();
      return 1;
    }

  /* Refuse to move the system block and its processes from the outside. */
  if(cur_p->block != zzuu.system_block)
    {
      if(pcb == (struct _pcb *)zzuu.system_block ||
	 (MAGIC_PCB == pcb->magic && zzuu.system_block == pcb->block))
	{
	  BKL_UNLOCK();
	  return 0;
	}
    }
  
#if defined(OSE_SMP_MAX_CPUS) && (OSE_SMP_MAX_CPUS > 1)
  /*
  ** Refuse to move stuff in a locked segment, with a few exceptions below.
  ** Supervisor caller checked earlier.
  */
  {
    struct segcb *segcb = find_segcb(pcb);

    if(segcb && segcb->migration_inhibited)
      {
	/*
	** The program manager is trusted to bind a program being created
	** in the locked system segment. Eventually, it will get a segment
	** of its own. 
	*/
	if(cur_p->pid == zzuu.prog_mgr_ && segcb == &system_segment)
	  {
	    ;			/* OK, continue */
	  }
	/*
	** Permit a process to migrate single processes in its own block,
	** for example itself.
	*/
	else if(MAGIC_PCB == pcb->magic && cur_p->block == pcb->block)
	  {
	    ;			/* OK, continue */
	  }
	/* Permit a process to migrate an empty block in the own segment. */
	else if(MAGIC_BCB == pcb->magic &&
		cur_p->block->segpid == ((struct _block*)pcb)->segpid &&
		((struct _block*)pcb)->proclist == NULL)
	  {
	    ;			/* OK, continue */
	  }
	  else
	  {
	    /* Return fail status due to locked segment. */
	    BKL_UNLOCK();
	    return 0;
	  }
      }
  }

  BKL_UNLOCK();

  /*
  ** We need to serialize with create_process without locking the
  ** dispatcher for the complete job. This will let us migrate
  ** background processes in a safe way, and to avoid a BKL in
  ** zzgo_search. And, by the way, make shure ose_sysd avoids sending
  ** a signal to itself, if some hook tries to migrate processes.
  */
  if(zzcur_p->live_pid == zzuu.sysd_)
    {
      sysd_bind_process(pid, cpu);
    }
  else
    {
      static const SIGSELECT sel_bind[] = {1, SMP_BIND_PROCESS};
      union SIGNAL *sig;

      sig = alloc(sizeof(struct smp_bind_process), SMP_BIND_PROCESS);
      sig->smp_bind_process.pid = pid;
      sig->smp_bind_process.cpu = cpu;
      send(&sig, zzuu.sysd_);

      sig = receive_from(OSE_NO_TIMEOUT, sel_bind, zzuu.sysd_);
      status = sig->smp_bind_process.status;
      free_buf(&sig);
    }

#else
  /* Non-SMP, no move really needed, but we want a decent return value. */
  UNLOCK();
  status = 1;
#endif

  return status;
}

/*
** This is the ose_bind_process system call.
*/
static OSADDRESS
sysconf_75(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  (void)r;

  return (OSADDRESS)zzkrn_bind_process((PROCESS)p, (cpuid_t)q);
}

#ifdef OSE_SMP

#include "cache.h"

static void
do_cache_invalidate_i(OSADDRESS iaddr)
{
#ifdef OSMIPS
  /* Workaround for mips, OSECR-6919. */
  (void)iaddr;
  cache_invalidate_i_complete(0);
#else
  const size_t breakpoint_size = 4; /* How wide is a breakpoint? */
  cache_invalidate_i(iaddr, breakpoint_size);
#endif
}

/*
** Temporarily park all cpus except the caller. 
** Also called to confirm the parking operation.
** Finally called also to release all other cpus.
**
** The purpose is to establish a stable system state when handling 
** breakpoints and dump generation from the system error handler.
** 
** action:
** 0 A cpu has received an ipi and is entering the stop cpu loop.
**   Also called by epm to poll if some other cpu whants to stop it.
** 1 The master cpu want the other cpus to stop. 
** 2 The master cpu releases the other cpus that where stopped.
** 3 Intercept from public call ose_smp_intercept_other_cpus().
** 4 Resume cpus from public call ose_smp_resume_other_cpus().
**
** invalidate_addr: 
** A breakpoint address to be invalidated on all cpus for action 1.
**
** This function shall be called with interrupts disabled.
*/
int
zzkrn_smp_stop_cpu(int action, OSADDRESS invalidate_addr)
{
  /*
  ** stop_state:
  ** 0 done     Not active.
  ** 1 posted   One cpu (master) has requested all other cpus to stop.
  ** 2 arrived  The cpu has started spinning in ipi context (stopped).
  */
  static OSADDRESS iaddr = (OSADDRESS)~0;
  static volatile uint32_t intercept_level; /* level * 0x10000 | 1 + cpu_id  */
  cpuid_t num_cpus    = zzkrn_num_cpus();
  cpuid_t i;
  LOCKSTATE msr;

  /* Disable interrupts, if the caller didn't. Prevents us from migration. */
  LOCK_SAVE(msr);

  switch(action)
    {
      /*
      ** Intercept my cpu. Triggered by a kernel event. Also called by EPM 
      ** to poll if some other cpu tries to stop us.
      */
    case 0:
      {
	int cpu_was_stopped = 0;

	/*
	** If my cpu has stopped the others, return directly.
	** Otherwise, spin until master sets stop_state to done (0).
	*/
	if((intercept_level & 0xffff) != 1 + zzkrn_cpu_id())
	  {
	    while(zzww.stop_state)
	      {
		cpu_was_stopped = 1;
		if(zzww.stop_state == 1)
		  zzww.stop_state = 2; /* Tell master that we have arrived */
	      }
	    /*
	    ** Invalidate the icache for one instruction.
	    ** This cpu was stopped because the instruction memory was
	    ** updated due to a breakpoint was set or cleared.
	    */
	    if(cpu_was_stopped && iaddr != (OSADDRESS)~0)
	      do_cache_invalidate_i(iaddr);
	  }
	LOCK_RESTORE(msr);
	return cpu_was_stopped;
      }

    case 1: /* Intercept all cpus but mine. Called by EPM. */
    case 3: /* Intercept from public call ose_smp_intercept_other_cpus(). */

      if((intercept_level & 0xffff) == 1 + zzkrn_cpu_id())
	{
	  /*
	  ** Detected a nested attempt to stop other cpus from this one.
	  ** This may be caused by an internal error in the breakpoint
	  ** handling of the EPM component, or, (more likely?) by
	  ** a recursive error generated by queue overflow in the kernel
	  ** event queue driver, leading to a situation when no event
	  ** can be passed to some of the cpus. Unwilling cpus are likely
	  ** sitting in some spin_lock and will probably not cause any
	  ** harm for PMD generating. We chose to continue.
	  */
	  intercept_level += 0x10000;
	}
      else
	{
	  /* Try  to become master of the world. */
	  while(atomic_cas(DEVOLATILE(atomic_t *, &intercept_level), 0,
			   0x10000 + 1 + zzkrn_cpu_id()))
	    {
	      /* Failed, another cpu is master. Spin until we may continue. */
	      if(zzww.stop_state == 1)
		zzww.stop_state = 2;	/* Tell I have arrived waiting. */
	      while(zzww.stop_state)	/* Wait until I may proceede. */
		;			/* NOP */

	      /*
	      ** Invalidate the icache for one instruction.
	      ** This cpu was stopped because the instruction memory was
	      ** updated due to a breakpoint was set or cleared.
	      */
	      if(iaddr != (OSADDRESS)~0)
		do_cache_invalidate_i(iaddr);
	    }

	  /* Now, I am the master! Tell all other to spin! */
	  for(i = 0; i < num_cpus; i++)
	    {
	      if(CPU_STATE_DETACHED != zzcpu[i].state)
		{
		  zzcpu[i].stop_state = 1; /* posted */

		  /* Tell cpu i to yield. */
		  if(i != zzkrn_cpu_id())
		    send_krn_event(i, KRN_EVT_YIELD, 0, NULL);
		}
	    }
	}

      /* Set master to arrived */
      zzww.stop_state = 2;

      /* Resumed cpus should invalidate their icache at this address. */
      if(action == 1)
	iaddr = invalidate_addr;

      /*
      ** Loop until all non-detached cpus has arrived, i.e. entered
      ** spinning in this function.
      */
      {
	int spin_count = 1000000;

	for(;;)
	  {
	    int all_stopped;

	    for(i = 0, all_stopped = 1; i < num_cpus; i++)
	      {
		if(CPU_STATE_DETACHED != zzcpu[i].state)		
		  all_stopped = all_stopped && zzcpu[i].stop_state == 2;
	      }

	    /*
	    ** Ready if all cpus are intercepted, or the retry counter
	    ** reaches zero.
	    */
	    if(all_stopped || (action == 3 && spin_count == 0))
	      {
		LOCK_RESTORE(msr);
		return 0;
	      }
	    --spin_count;
	  }
	/*NOTREACHED*/
      }

    case 2: /* Resume all cpus. Called by EPM. */
    case 4: /* Resume cpus from public call ose_smp_resume_other_cpus(). */
      if((intercept_level & 0xffff) == 1 + zzkrn_cpu_id())
	{
	  if((intercept_level & 0xffff0000) == 0x10000)
	    {
	      for(i = 0; i < num_cpus; i++)
		{
		  zzcpu[i].stop_state = 0; /* done */
		}
	      SMP_STORE_MEMBAR();
	      intercept_level = 0;
	    }
	  else
	    intercept_level -= 0x10000;
	}
      break;
    }

  LOCK_RESTORE(msr);
  return 0;
}
#endif

/*
 * @function ose_smp_intercept_other_cpus
 * @include "ose_spi/ose_smp.h"
 * 
 * @brief EXPERIMENTAL. Intended for use by the kernel error handler 
 * in an SMP system. Performs no action in single cpu systems.
 */
void
ose_smp_intercept_other_cpus(void)
{
  SMP_CODE(zzkrn_smp_stop_cpu(3, ~0u));
}

/*
 * @function ose_smp_resume_other_cpus
 * @include "ose_spi/ose_smp.h"
 * 
 * @brief EXPERIMENTAL. Intended for use by the kernel error handler 
 * in an SMP system. Performs no action in single cpu systems.
 */
void
ose_smp_resume_other_cpus(void)
{
  SMP_CODE(zzkrn_smp_stop_cpu(4, ~0u));
}

/*
**===========================================================================
** 9.4  Undocumented system calls.
**===========================================================================
*/

/*
** Enter an interrupt process from hardware int. 
** zzinit_int() must be called for all interrupt processes except users
** int's. Users int's may not call zzinit_int(). Users int's must have
** higher priority than all OS int's and can never use any system calls.
** The wakeup parameter is set if the interrupt was caused by a wakeup call.
** Oldint_level is used by OSE to determine if we are at lowest int level.
** It is set to 32 for the first call.
** Int_level contains the interrupt priority of the current interrupt.
** It is used by OSE to determine when wakeup calls to other interrupt processes
** are required.
**
** Returns with interrupts disabled.
** Returns 0 if a normal interrupt return should be performed.
** Returns 1 if a process switch is required.
** Returns 2 if a wakeup call to a lower priority interrupt is required.
*/

OSBOOLEAN
zzinit_int(void *pcb, OSBOOLEAN wakeup, OSPRIORITY int_level, OSPRIORITY oldint_level)
{
#if LATE_INIT_INT != 0
   (void)pcb;
   (void)wakeup;
   (void)int_level;
#elif LATE_INIT_INT == 0
  register struct _pcb *saved_cur_p;
  register OSBOOLEAN saved_wakeup;

  /* Save old current process on stack. */
  saved_cur_p = zzcur_p;

  /* Save old wakeup flag on stack. */
  saved_wakeup = zzwakeflag;

  /* Store wakeup flag to be returned by the wakeup function. */
  zzwakeflag = wakeup;

  /* Store new process as current process. */
  set_zzcur_p((struct _pcb *)pcb);

  /* The target interface may require a uarea for interrupt and timer processes. */
  zzcurrent_uarea = ((struct _pcb *) pcb)->uarea;

#if ERROR_CHECKS > 0
  ON_ERRLEVEL_2()
    {
      /*
      ** Check that the pcb supplied looks good. Assume a
      ** spurious interrupt if not.
      */
      if(unlikely(((struct _pcb *) pcb)->magic != MAGIC_PCB))
	{
	  os_error(1, OSE_ESPURIOUS_INTERRUPT, (OSERRCODE) int_level);	/* Non fatal system error. */
	  LOCK();
	  /* Pop old wakeup flag from stack. */
	  zzwakeflag = saved_wakeup;

	  /* Pop old current process from stack. */
	  set_zzcur_p(saved_cur_p);

	  /* The target interface may require a uarea for interrupt and timer processes. */
	  zzcurrent_uarea = saved_cur_p->uarea;
	  return (OSBOOLEAN) 0;
	}

      /*
      ** Check that the current priority matches the priority of
      ** the interrupt process and report error if not.
      ** Modify the wakeup priority if the error handler returns.
      */
      if((int_level | (OSPRIORITY) 64) != ((struct _pcb *) pcb)->priority &&
	 ((struct _pcb *) pcb)->priority < (OSPRIORITY) 96)
	  {
	    os_error(1, OSE_EBAD_INTLEVEL, (OSERRCODE) int_level);	/* Non fatal system error. */
	    ((struct _pcb *) pcb)->priority = (int_level | (OSPRIORITY) 64);
	  }
    }
#endif

  /* This code is for systems without the debugger. */

#if OSE_DEBUG == 0
      /* Call interrupt process. */
#if CALL_INTERRUPTS_DIRECTLY > 0
      (*zzcur_p->entrypoint)();
#else
      zzcall_intproc(zzcur_p->entrypoint);
#endif
#endif
  
  /* This code is for systems with debugger present and interrupt profiling enabled. */

#if OSE_DEBUG > 0
#if ENABLE_INTERRUPT_PROFILING > 0
  if(zzuu.inthook_enabled)
    {
      /* Call interrupt process swap in hook. */
      zzintswap_in(pcb, saved_cur_p);
      /* Call interrupt process. */
#if CALL_INTERRUPTS_DIRECTLY > 0
      (*zzcur_p->entrypoint)();
#else
      zzcall_intproc(zzcur_p->entrypoint);
#endif
      /* Call interrupt process swap out hook. */
      zzintswap_out(saved_cur_p, pcb);
    }
  else
    {
      /* Call interrupt process. */
#if CALL_INTERRUPTS_DIRECTLY > 0
      (*zzcur_p->entrypoint)();
#else
      zzcall_intproc(zzcur_p->entrypoint);
#endif
    }
#endif
#endif

  /* This code is for systems with debugger present and interrupt profiling disabled. */

#if OSE_DEBUG > 0
#if ENABLE_INTERRUPT_PROFILING == 0
  /* Call interrupt process. */
#if CALL_INTERRUPTS_DIRECTLY > 0
  (*zzcur_p->entrypoint)();
#else
  zzcall_intproc(zzcur_p->entrypoint);
#endif
#endif
#endif

  /* Pop old wakeup flag from stack. */
  zzwakeflag = saved_wakeup;

  /* Pop old current process from stack. */
  set_zzcur_p(saved_cur_p);

  /* The target interface may require a uarea for interrupt and timer processes. */
  zzcurrent_uarea = saved_cur_p->uarea;

/* if LATE_INIT_INT == 0 */
#endif
  /*
  ** Issue any wakeup calls queued during the execution of the
  ** interrupt process, but only those with same or lower priority than
  ** int_level and higher priority than oldint_level.
  ** (It is an internal error if calls are queued at a higher priority.)
  ** We need to search lists with interrupts disabled, which would cause
  ** unintentional delay.
  ** Instead we only find out which priority to go to, put that in a
  ** global variable and return a wakeup code.
  ** The wakeup priority is searched when we arrive at the common
  ** wakeup entrypoint.
  */
  LOCK();

  if(zzww.wk_look_aside)	/* Any wakup at all? */
    {
      /* Find highest level that needs a wakeup. */
      /* Give it to target interface if higher than oldint_level. */
      zzwakeup_level = (OSPRIORITY) FIND_FIRST_BIT(zzww.wk_look_aside);
      if(zzwakeup_level < oldint_level)
	return (OSBOOLEAN) 2;
    }

  /* Return from interrupt unless we are at lowest interrupt level. */
  if(oldint_level == 32)
    {
      /* Tell the idle loop that an interrupt has occurred. */
      /* Possibly move this setting after if(zzww.dp_req){...} ? @guek@ */
      zzww.dp_idle_interrupted = 1;

      /* Test if interrupt requested a swap. */
      if(zzww.dp_req)
	{
	  /* If so, make sure we are not already in dispatcher. */
	  switch(zzww.dp_state)
	    {
	    case 0:
	      {
		/*
		** Setting dp_state to 1 allows the caller to enable interrupts
		** almost immediately as a dispatch operation will seem to
		** be in progress, causing no further dispatch requests from
		** subsequent interrupt processes.
		*/
		zzww.dp_state = 1;
		return (OSBOOLEAN) 1;
	      }
	      
	      /* Otherwise, action depends on dispatcher phase. */
	      /* In dispatcher save phase? */
	    case 1:
	      {
		/* Clear dispatcher restart flag. */
		zzww.dp_req = (OSBOOLEAN) 0;
	      }
	      
	      /* In dispatcher search or restore phase? */
	    default: /* case 2 or 3: */
	      {
		/* Leave dispatcher to continue. */
		return (OSBOOLEAN) 0;
	      }
	    }
	}
    }
  /* Leave dispatcher to continue. */
  return (OSBOOLEAN) 0;
}

/*
** Check if a reschedule is needed.
**
** Returns 0 if a normal interrupt return should be performed.
** Returns 1 if a process switch is required.
** Returns 2 if a wakeup call to a lower priority interrupt is required.
*/
OSBOOLEAN
zzkrn_int_dispatch(OSPRIORITY oldint_level)
{
  /*
  ** Interrupts are still disabled.
  ** Issue any wakeup calls queued during interrupt process, but only
  ** those with same or lower priority than int_level and higher priority
  ** than oldint_level.
  ** (It is an internal error if calls are queued at a higher priority.)
  ** We need to search lists with interrupts disabled, which would cause
  ** unintentional delay.
  ** Instead we only find out which priority to go to, put that in a
  ** global variable and return a wakeup code.
  ** The wakeup priority is searched when we arrive at the common
  ** wakeup entrypoint.
  */

  if(zzww.wk_look_aside)	/* Any wakup at all? */
    {
      /* Find highest level that needs a wakeup. */
      /* Give it to target interface if higher than oldint_level. */
      zzwakeup_level = (OSPRIORITY) FIND_FIRST_BIT(zzww.wk_look_aside);
      if(zzwakeup_level < oldint_level)
	return (OSBOOLEAN) 2;
    }

  /*
  ** Request a dispatch if the current interrupt is not nested,
  ** the dispatcher is not locked and no dispatch is ongoing already.
  */
  if(zzww.dp_req == 1 && oldint_level == 32
     && (zzdispatcher_locked | zzww.dp_state) == 0)
    return 1;			/* Switch process. */

  return 0;			/* Return from interrupt. */
}

/*
** Perform a wakeup call to a lower priority in response to a kernel request
** returned by either the zzinit_int() function or another zzdo_wakeup() call.
** Wakeup calls for semaphores are issued before calls due to
** signal transmissions.
**
** parameters and return values are defined as for the zzinit_int() function.
*/

OSBOOLEAN
zzdo_wakeup(OSPRIORITY int_level, OSPRIORITY oldint_level)
{
  register LOCKSTATE msr;
  register struct _pcb *pcb;
  register volatile struct wakeup_list_entry * rlist;
  register struct _pcb *saved_cur_p;
  register OSBOOLEAN saved_wakeup;

  /* Save old current process on stack. */
  saved_cur_p = zzcur_p;

  /* Save old wakeup flag on stack. */
  saved_wakeup = zzwakeflag;
  
  /*
  ** Scan the specified interrupt level until no more interrupts on that level.
  ** Scan the timer list if timers are on this level.
  ** Note that the wakeup list is continuously updated by
  ** higher priority interrupts, so interrupts must be disabled each time the
  ** wakeup list is examined.
  */

  /* Get the wakeup list head for the current interrupt priority. */
  rlist = (volatile struct wakeup_list_entry *) &zzww.wakeup_list[int_level];

  /* Disable interrupts to protect wakeup lists from higher interrupts. */
  LOCK_SAVE(msr);

  /* Loop for all processes on the priority with pending wakeups. */
  for(pcb = rlist->hd; pcb; pcb = rlist->hd)
    {
      /* Run any pending semaphore wakeup calls for this process. */
      if(pcb->wake_sem_cnt)
	{
	  /* Set a proper wakeup flag. */
	  zzwakeflag = (OSBOOLEAN) 2; /* Semaphore was signalled. */

	  /* Decrement number of pending wakeup calls. */
	  if(!--(pcb->wake_sem_cnt))
	    {
	      /* Get rid of process when no more pending calls. */
	      if(!(pcb->wake_sig_cnt |
		   SMP_CODE_YES_NO(pcb->wake_tick_cnt, 0)))
		{
                  rlist->hd = pcb->ready_entry.nxt;
		  if(!pcb->ready_entry.nxt)
		    zzww.wk_look_aside ^= rlist->pri_mask;
		}
	    }
	}
      /* Run any pending signal wakeup calls for this process. */
      else if(pcb->wake_sig_cnt)
	{
	  /* Set a proper wakeup flag. */
	  zzwakeflag = (OSBOOLEAN) 1; /* Got a signal. */

	  /* Decrement number of pending wakeup calls. */
	  if(!--(pcb->wake_sig_cnt))
	    {
	      /* Get rid of process when no more pending calls. */
	      if(!SMP_CODE_YES_NO(pcb->wake_tick_cnt, 0))
		{
                  rlist->hd = pcb->ready_entry.nxt;
		  if(!pcb->ready_entry.nxt)
		    zzww.wk_look_aside ^= rlist->pri_mask;
		}
	    }
	}
      /*
      ** Run any pending tick wakeup calls for this process.
      **
      ** This code is used by timer processes running on another SMP cpu 
      ** than the one that consumes the tick() interrupts.
      */
#ifdef OSE_SMP
      else if(pcb->wake_tick_cnt)
	{
	  /* Set a proper wakeup flag. */
	  zzwakeflag = (OSBOOLEAN) 0; /* Hardware interrupt. */

	  /* Decrement number of pending wakeup calls. */
	  if(!--(pcb->wake_tick_cnt))
	    {
	      /* Get rid of process when no more pending calls. */
              rlist->hd = pcb->ready_entry.nxt;
	      if(!pcb->ready_entry.nxt)
		zzww.wk_look_aside ^= rlist->pri_mask;
	    }
	}
#endif
      else
	{
	  assert(0);
	}
      
      
      /* Store new process as current process. */
      set_zzcur_p(pcb);

      /* The target interface may require a uarea for interrupt and timer processes. */
      zzcurrent_uarea = pcb->uarea;
	  
      /* Enable higher priority interrupts. */
      LOCK_RESTORE(msr);

      /* This code is for systems without the debugger. */

#if OSE_DEBUG == 0
      /* Call interrupt process. */
#if CALL_INTERRUPTS_DIRECTLY > 0
      (*pcb->entrypoint)();
#else
      zzcall_intproc(pcb->entrypoint);
#endif
#endif
  
      /* This code is for systems with debugger present and interrupt profiling enabled. */
	  
#if OSE_DEBUG > 0
#if ENABLE_INTERRUPT_PROFILING > 0
      if(zzuu.inthook_enabled)
	{
	  /* Call interrupt process swap in hook. */
	  zzintswap_in(pcb, saved_cur_p);
	  /* Call interrupt process. */
#if CALL_INTERRUPTS_DIRECTLY > 0
	  (*pcb->entrypoint)();
#else
	  zzcall_intproc(pcb->entrypoint);
#endif
	  /* Call interrupt process swap out hook. */
	  zzintswap_out(saved_cur_p, pcb);
	}
      else
	{
	  /* Call interrupt process. */
#if CALL_INTERRUPTS_DIRECTLY > 0
	  (*pcb->entrypoint)();
#else
	  zzcall_intproc(pcb->entrypoint);
#endif
	}
#endif
#endif
	  
      /* This code is for systems with debugger present and interrupt profiling disabled. */
	  
#if OSE_DEBUG > 0
#if ENABLE_INTERRUPT_PROFILING == 0
      /* Call interrupt process. */
#if CALL_INTERRUPTS_DIRECTLY > 0
      (*pcb->entrypoint)();
#else
      zzcall_intproc(pcb->entrypoint);
#endif
#endif
#endif

      /* Disable interrupts again. */
      LOCK_SAVE(msr);
    } /* for */

  /* Pop old wakeup flag from stack. */
  zzwakeflag = saved_wakeup;

  /* Pop old current process from stack. */
  set_zzcur_p(saved_cur_p);

  /* The target interface may require a uarea for interrupt and timer processes. */
  zzcurrent_uarea = saved_cur_p->uarea;

  /*
  ** Interrupts are still disabled.
  ** Issue any wakeup calls queued during
  ** interrupt process, but only those with same or lower priority than
  ** int_level and higher priority than oldint_level.
  ** (It is an internal error if calls are queued at a higher priority.)
  ** We need to search lists with interrupts disabled, which would cause
  ** unintentional delay.
  ** Instead we only find out which priority to go to, put that in a
  ** global variable and return a wakeup code.
  ** The wakeup priority is searched when we arrive at the common
  ** wakeup entrypoint.
  */

  if(zzww.wk_look_aside)	/* Any wakup at all? */
    {
      /* Find highest level that needs a wakeup. */
      /* Give it to target interface if higher than oldint_level. */
      zzwakeup_level = (OSPRIORITY) FIND_FIRST_BIT(zzww.wk_look_aside);
      if(zzwakeup_level < oldint_level)
	return (OSBOOLEAN) 2;
    }

  /* Return from interrupt unless we are at lowest interrupt level. */
  if(oldint_level == 32)
    {
      /* Tell the idle loop that an interrupt has occurred. */
      zzww.dp_idle_interrupted = 1;

      /* Test if interrupt requested a swap. */
      if(zzww.dp_req)
	{
	  /* If so, make sure we are not already in dispatcher. */
	  switch(zzww.dp_state)
	    {
	    case 0:
	      {
		/*
		** Setting dp_state to 1 allows the caller to enable interrupts
		** almost immediately as a dispatch operation will seem to
		** be in progress, causing no further dispatch requests from
		** subsequent interrupt processes.
		*/
		zzww.dp_state = 1;
		return (OSBOOLEAN) 1;
	      }

	      /* Otherwise, action depends on dispatcher phase. */
	      /* In dispatcher save phase? */
	    case 1:
	      {
		/* Clear dispatcher restart flag. */
		zzww.dp_req = (OSBOOLEAN) 0;
	      }
	      
	      /* In dispatcher search or restore phase? */
	    default: /* case 2 or 3: */
	      {
		/* Leave dispatcher to continue. */
		return (OSBOOLEAN) 0;
	      }
	    }
	}
    }
  return (OSBOOLEAN) 0;
}

/*
** KRN_EVT ISR functions for different events.
**
** These functions are invoked normally invoked by a hardware interrupt
** request from another SMP cpu, when it requests some change of 
** cpu specific state. During the boot phase, before interrupts can be 
** served, these functions are also invoked from the idle loop when it
** polls the cpu specific event queue.
**
** These functions shall be regared as context-less ISR functions.
** For example, they should not report errors.
**
** Most events will cause a dispatcher action, typically
** make_otherpri_ready_noswap() or make_otherpri_not_ready().
**
** These functions are called with, and returns with interrupts disabled.
** Spinlock operations must be avoided.
*/

#ifdef OSE_SMP
/* KRN_EVT_SIG_SEND:	 pid, sigadm* */
static void
krn_evt_sig_send(struct krn_event *evt)
{
  struct _pcb *pcb = pid_to_pcb(evt->data);
  struct _signal *sigadm = (struct _signal *)evt->obj;

  /* Receiver dead or alive? */
  if(pcb_to_pid(pcb) != evt->data || pcb->magic != MAGIC_PCB)
    {
      /* Receiver is dead. Let sysd_lite free the signal. */
      adm_to_sig(sigadm)->sig_no = SMP_FREE_ME;
      pcb = pid_to_pcb(zzww.sysd_lite_);
    }
  else
    {
      cpuid_t pcb_cpu = pcb->cpu_id;

      if(pcb_cpu != zzkrn_cpu_id())  
	{
	  /* The process has migrated, pass the event to the proper cpu. */
	  send_krn_event(pcb_cpu, evt->event, evt->data, evt->obj);
	  return;
	}
    }
  
  /* Insert the signal into the queue. */
  sigadm->next = NULL;
  if(pcb->mailbox)
    {
      pcb->mailtail->next = sigadm;
    }
  else
    {
      pcb->mailbox = sigadm;
    }
  pcb->mailtail = sigadm;
  pcb->sig_cnt_queued++;

  /* Do a process type dependent swap request. */

  if(pcb->priority < 33)
    {
      /*
      ** The target process is a prioritized or background process.
      ** Make the process ready and maybe swap.
      */
      if(pcb->status & ST_RECEIVE)
	{
	  pcb->status = (pr_state_t) (pcb->status & (~ST_RECEIVE_FSEM));
	      
	  /* If the process becomes ready then do a swap. */
	  if(pcb->status < ST_READY)
	    {
	      make_otherpri_ready_noswap(pcb); /* noswap */
	    }
	}
    }
  else if(pcb->priority < 96)
    {
      /*
      ** The target process is an interrupt process.
      ** Request a later wakeup call. The is_deaf flag is
      ** checked inside request_int_sigwakeup().
      */
      request_int_sigwakeup(pcb);
    }
  else
    {
      /*
      ** The target process is a timer process.
      ** Request a later wakeup call. The is_deaf flag is checked
      ** inside request_timer_sigwakeup().
      */
      request_timer_sigwakeup(pcb);
    }
}

/* KRN_EVT_SIGNAL_SEM:  pid, NULL */
static void
krn_evt_signal_sem(struct krn_event *evt)
{
  struct _pcb *pcb = pid_to_pcb(evt->data);
  cpuid_t pcb_cpu;

  /* Still alive? */
  if(pcb_to_pid(pcb) == evt->data)
    {
      pcb_cpu = pcb->cpu_id;
      /* If the process has migrated, pass the event to the proper cpu. */
      if(pcb_cpu != zzkrn_cpu_id())
	{
	  send_krn_event(pcb_cpu, evt->event, evt->data, evt->obj);
	  return;
	}

      /* Request a dispatch if required. */
      pcb->status  = (pr_state_t) (pcb->status & (~ST_SEMAPHORE));
      if((pcb->status < ST_READY) && (pcb->priority < 33))
	{
	  make_otherpri_ready_noswap(pcb); /* noswap */
	}
    }
}

/* KRN_EVT_MIGRATE:	pid, to_cpu<<16 */
static void
krn_evt_migrate(struct krn_event *evt)
{
  struct _pcb *pcb = pid_to_pcb(evt->data);
  cpuid_t cpu = ((OSADDRESS)evt->obj) >> 16;
  
  /*
  ** The sender has taken the dispatcher lock. Therefore, we know that
  ** the victim is alive, stopped and really located in this cpu.
  ** Still, Note that a victim of interrupt type could be the current
  ** process(sic!), but at the moment we only use this event for 
  ** prioritized processes.
  */

  assert(pcb_to_pid(pcb) == evt->data);
  assert(pcb->cpu_id == zzkrn_cpu_id());
  assert(cpu < zzkrn_num_cpus());

  pcb->cpu_id = cpu;
  if (pcb->priority < 32) 
    {
      /* Priority process */
      pcb->ready_entry.rlist = (struct ready_list_entry *)
        &ready_list_x(cpu)[pcb->priority];
    }
#if 0
  else if(pcb->priority == 32) 
    {
      /* Background process */
    }
  else if(pcb->priority < 96)
    {
      /* Target process is an interrupt process */
      /* Can it be affected if it is currently running?? */
    }
  else if(pcb->priority < 128)
    {
      /* Target process is a timer process */
      /* Can it be affected if it is currently running?? */
    }
  else
    {
      /* Target process is a phantom process */
    }
#endif

  /*
  ** There is no need to request a dispatch here, as the process is in
  ** state stopped.
  */
}

/* KRN_EVT_SIG_COUNT:	pid, signed_increment<<16 */
/*
** This event shall affect dead processes as well as alive!
** Will this do? Or do we open up a window that harms sysd_delete_process??
**
** This event is unusual, as it is applied even to killed processes.
*/
static void
krn_evt_sig_count(struct krn_event *evt)
{
  struct _pcb *pcb = pid_to_pcb(evt->data);
  cpuid_t pcb_cpu = pcb->cpu_id;
  W16 signed_increment;

  /* If the process has migrated, pass the event to the proper cpu. */
  if(pcb_cpu != zzkrn_cpu_id())
    {
      send_krn_event(pcb_cpu, evt->event, evt->data, evt->obj);
      return;
    }

  /* Increment/Decrement number of owned signals. */
  signed_increment = (W16)(((W32)evt->obj) >> 16);
  pcb->sig_cnt_owned += signed_increment;
}

/* KRN_EVT_START:	 pid, NULL */
/* KRN_EVT_RESUME:	 pid, NULL */
static void
krn_evt_start_resume(struct krn_event *evt)
{
  enum pr_state operation;
  struct _pcb *pcb = pid_to_pcb(evt->data);
  cpuid_t pcb_cpu = pcb->cpu_id;

  operation = evt->event == KRN_EVT_START ? ST_STOPPED : ST_INTERCEPTED;
  
  /* Still alive? */
  if(pcb_to_pid(pcb) == evt->data)
    {
      /* If the process has migrated, pass the event to the proper cpu. */
      if(pcb_cpu != zzkrn_cpu_id())
	{
	  send_krn_event(pcb_cpu, evt->event, evt->data, evt->obj);
	  return;
	}
  
      /* Any action really needed? */
      if(operation & pcb->status)
	{
	  if(operation == ST_STOPPED)
	    {
	      if(--pcb->st_cnt == 0)
		pcb->status &= ~operation;
	    }
	  else
	    pcb->status &= ~operation;

          if((pcb->status < ST_READY) && (pcb->priority < 33))
            {
	      make_otherpri_ready_noswap(pcb); /* noswap */
	      return;
            }
	}
    }
}

/* KRN_EVT_STOP:	 pid, NULL */
/* KRN_EVT_INTERCEPT:	 pid, NULL */
static void
krn_evt_stop_intercept(struct krn_event *evt)
{
  enum pr_state operation;
  struct _pcb *pcb = pid_to_pcb(evt->data);
  cpuid_t pcb_cpu = pcb->cpu_id;

  operation = evt->event == KRN_EVT_STOP ? ST_STOPPED : ST_INTERCEPTED;
  
  /* Still alive? */
  if(pcb_to_pid(pcb) == evt->data)
    {
      /* If the process has migrated, pass the event to the proper cpu. */
      if(pcb_cpu != zzkrn_cpu_id())
	{
	  send_krn_event(pcb_cpu, evt->event, evt->data, evt->obj);
	  return;
	}
  
      if(!pcb->st_cnt || operation == ST_INTERCEPTED)
	{
	  pcb->status |= operation;
	  if(pcb->priority < 33)
	    {
	      make_otherpri_not_ready(pcb);
	    }
	}

      if(operation == ST_STOPPED)
        {
	  U32 st_cnt = pcb->st_cnt;

          /* Increment the process stop counter, but don't let it wrap. */
          if(++st_cnt)
	    pcb->st_cnt = st_cnt;
	}
    }
}


/* KRN_EVT_SIGNAL_FSEM:  pid, NULL */
static void
krn_evt_signal_fsem(struct krn_event *evt)
{
  struct _pcb *pcb = pid_to_pcb(evt->data);
  cpuid_t pcb_cpu = pcb->cpu_id;

  /* Still alive? */
  if(pcb_to_pid(pcb) == evt->data)
    {
      /* If the process has migrated, pass the event to the proper cpu. */
      if(pcb_cpu != zzkrn_cpu_id())
	{
	  send_krn_event(pcb_cpu, evt->event, evt->data, evt->obj);
	  return;
	}

      /* If target process is a prioritized or background process */
      /* then signal the semaphore and maybe swap. */
      if(pcb->priority < 33)
        {
	  if(++(pcb->fsemval) == 0)
	    {
	      if(pcb->status & ST_FSEMAPHORE)
	        {
		  pcb->status = (pr_state_t) (pcb->status & (~ST_RECEIVE_FSEM));
		  
		  /* If the process becomes ready then do a swap. */
		  if(pcb->status < ST_READY)
		    {
		      make_otherpri_ready_noswap(pcb); /* noswap */
		    }
	        }
	    }
        }

      /* If target process is an interrupt process then */
      /* request a wakeup call. */
      else if(pcb->priority < 96)
        {
	  request_int_semwakeup(pcb);
        }

      /* If the target process is a timer process then request a wakeup call. */
      else if(pcb->priority < 128)
        {
	  request_timer_semwakeup(pcb);
        }
    }
}

/* KRN_EVT_SET_PRI_FOR:  pid, prio<<16 */
static void
krn_evt_set_pri_for(struct krn_event *evt)
{
  OSPRIORITY newpri = (OSPRIORITY) (((OSADDRESS)evt->obj) >> 16);
  struct _pcb *pcb = pid_to_pcb(evt->data);
  cpuid_t pcb_cpu = pcb->cpu_id;

  /* Still alive? */
  if(pcb_to_pid(pcb) == evt->data)
    {
      /* If the process has migrated, pass the event to the proper cpu. */
      if(pcb_cpu != zzkrn_cpu_id())
	{
	  send_krn_event(pcb_cpu, evt->event, evt->data, evt->obj);
	  return;
	}
      /* Remove the process from the ready list if it is ready. */
      make_otherpri_not_ready(pcb);

      /* Set a new priority in the pcb. */
      pcb->priority = newpri;
      pcb->ready_entry.rlist = (struct ready_list_entry *) &zzww.ready_list[newpri];

      /* Run the process at the new priority if it is ready. */
      if(pcb->status < ST_READY)
	make_otherpri_ready_noswap(pcb); /* noswap */
    }
}

/* KRN_EVT_MUTEX_RAISE:  pid, prio<<4 | (instance&0x3f)<<10 | index <<16 */
/*
** Note that this process must not access the user mode mutex, as it may 
** lack a current mapping.
** Restrictions:
** - Not yet functional.
*/
static void
krn_evt_mutex_raise(struct krn_event *evt)
{
  /* krn_evt_mutex_raise: Not yet functional ?? */
#if 1
  (void)evt;
#else
  OSADDRESS arg = (OSADDRESS)evt->obj;
  OSPRIORITY newpri = (OSPRIORITY) ((arg >> 4) & 0x1F);
  struct _pcb *pcb = pid_to_pcb(evt->data);
  cpuid_t pcb_cpu = pcb->cpu_id;

  /* Is the process still alive? */
  if(pcb_to_pid(pcb) == evt->data)
    {
      /* If the process has migrated, pass the event to the proper cpu. */
      if(pcb_cpu != zzkrn_cpu_id())
	{
	  send_krn_event(pcb_cpu, evt->event, evt->data, evt->obj);
	  return;
	}

      /* Ignore background processes? */
      if(pcb->priority > 31)
	{
	  return;
	}
      
      /* Need to raise priority? */
      if(pcb->priority > newpri)
	{
	  OSADDRESS mindex = arg >> 16;
	  struct mutcb *mutcb;

	  /* Validate the mutex index and mutex. */
	  if(mindex == 0 || mindex >= zzuu.next_free_mutex_index)
	    {
	      /* The user space mutex was not valid at sending. */
	      return;
	    }

	  mutcb = index_to_mutcb(mindex);
	  if(((mutcb->instance ^ (arg >> 10)) & 0x3F) ||
	     mutcb->magic != MAGIC_MUTEX)
	    {
	      /* The mutcb is no longer valid. */
	      return;
	    }

	  /*
	  ** Check that the mutex is still owned by the process.
	  ** How can we do that without touching the user space mutex?
	  */

	  /* Save the original priority if not done yet. */
	  if(!pcb->org_prio)
	    {
	      /* No atomic needed, org_prio is only set from the same cpu. */
	      pcb->org_prio = pcb->priority & 31;
	    }

	  /* Remove the process from the ready list if it is ready. */
	  make_otherpri_not_ready(pcb);

	  /* Set a new priority in the pcb. */
	  pcb->priority = newpri;
	  pcb->ready_entry.rlist =
	    (struct ready_list_entry *) &zzww.ready_list[newpri];

	  /* Run the process at the new priority if it is ready. */
	  if(pcb->status < ST_READY)
	    make_otherpri_ready_noswap(pcb); /* noswap */
	}
    }
#endif	/* krn_evt_mutex_raise: Not yet functional?? */
}

/* KRN_EVT_TICK_TI_PROC:  pid, NULL */
static void
krn_evt_wake_ti_proc(struct krn_event *evt)
{
  struct _pcb *pcb = pid_to_pcb(evt->data);
  cpuid_t pcb_cpu = pcb->cpu_id;

  /* Still alive? */
  if(pcb_to_pid(pcb) == evt->data)
    {
      /* If the process has migrated, pass the event to the proper cpu. */
      if(pcb_cpu != zzkrn_cpu_id())
	{
	  send_krn_event(pcb_cpu, evt->event, evt->data, evt->obj);
	  return;
	}
      /* Request a timer wakeup call. */
      request_timer_tickwakeup(pcb);
      return;
    }
}

/* KRN_EVT_TICK:  ticks, NULL */
static void
krn_evt_tick(struct krn_event *evt)
{
  if(unlikely(zzww.ticks + 1 != evt->data))
    {
      ramlog_printf("<FATAL>(krn_evt_tick) bad counter, got %u @ tick %u.\n",
		    (unsigned)evt->data, (unsigned)zzww.ticks);
      os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 31);  /* Fatal */
    }
  /* Save a local copy of the tick counter. */
  zzww.ticks = evt->data;
  /* Expire timeouts */
  expire_timeouts();
  /* Handle Timeslicing of prioritized and background processes. */
  handle_timeslicing();
}

/* KRN_EVT_TIMEOUT:  pid, NULL */
static void
krn_evt_timeout(struct krn_event *evt)
{
#if 0 				/* OBSOLETE KRN_EVT_TIMEOUT */
  struct _pcb *pcb = pid_to_pcb(evt->data);
  cpuid_t pcb_cpu = pcb->cpu_id;

  /* Still alive? */
  if(pcb_to_pid(pcb) == evt->data)
    {
      /* If the process has migrated, pass the event to the proper cpu. */
      if(pcb_cpu != zzkrn_cpu_id())
	{
	  send_krn_event(pcb_cpu, evt->event, evt->data, evt->obj);
	  return;
	}
      /* The process may have received a signal by now.
      ** ST_TIMEOUT must only be set if tmo_active is still set here.
      ** 
      */
      if(pcb->tmo_active == 2)
	{
	  pcb->tmo_active = 0;
	  pcb->status = (pr_state_t) ((pcb->status | ST_TIMEOUT) &
				      (~(ST_RECEIVE_FSEM | ST_DELAY)));
	  /* Make process ready if required. */
	  if(pcb->status < ST_READY)
	    make_otherpri_ready_noswap(pcb); /* noswap */
	}
    }
#else
  (void)evt;
#endif
}

/* KRN_EVT_SEND_SPORT:  pid, NULL */
static void
krn_evt_send_sport(struct krn_event *evt)
{
  struct _pcb *pcb = pid_to_pcb(evt->data);
  cpuid_t pcb_cpu = pcb->cpu_id;

  /* Still alive? */
  if(pcb_to_pid(pcb) == evt->data)
    {
      /* If the process has migrated, pass the event to the proper cpu. */
      if(pcb_cpu != zzkrn_cpu_id())
	{
	  send_krn_event(pcb_cpu, evt->event, evt->data, evt->obj);
	  return;
	}

      /* Request a dispatch if required. */
      pcb->status  = (pr_state_t) (pcb->status & (~ST_RECEIVE_FSEM));
      if(pcb->status < ST_READY)
	{
	  make_otherpri_ready_noswap(pcb); /* noswap */
	}
    }
}

/* KRN_EVT_FLUSH_TLB:	 base, size */
static void
krn_evt_flush_tlb(struct krn_event *evt)
{
  extern void (*hal_invalidate_tlb_local)(U32 base, U32 size);

  U32 base = (U32)evt->obj;
  U32 size = evt->data;

  if(NULL != hal_invalidate_tlb_local)
    {
      hal_invalidate_tlb_local(base, size);
    }
  else
    {
      ramlog_printf("Unhandled KRN_EVT_FLUSH_TLB\n");  /* ?? */
      assert(0);
    }
}

/* KRN_EVT_BLOCK_HOOK: hookcb, operation */
static void
krn_evt_block_hook(struct krn_event *evt)
{
  struct statehookcb *hookcb = (struct statehookcb *)evt->data;
  OSERRCODE operation = (OSERRCODE)evt->obj;
  PROCESS victim = hookcb->victim;

  /* Still alive? */
  if(hookcb->magic == STATEHOOK_MAGIC &&
     hookcb->victim == pcb_to_pid(pid_to_pcb(victim)))
    {
      /* Call the hook. */
      hookcb->func(victim, operation, hookcb->handle);
    }
}


/*
** Request the kernel to process kernel events posted by other cpus
** in an SMP kernel. This function is called with, and returns with
** interrupts disabled.
**
** Usually this function is called from the interrupt code in the
** target interface. It can also be called from an ISR in an IPI driver.
**
** In the boot phase, this function is called from the idle loop too.
** Wakeup calls are not processed by the idle loop, but they are queued
** until the first IPI HW interrupt arrives.
*/

static void
local_krn_evt_notify(void)
{
  struct krn_event evt;

  /* Get the kernel events from the BSP. */
  while(receive_krn_event(&evt))
    {
#if 0
      ramlog_printf("Cpu %d got evt %#x %#lx %#lx\n", zzkrn_cpu_id(),
		   evt.event, evt.data, (OSADDRESS)evt.obj);
#endif
      /* 
      ** Optimized by using one tiny and one extended>>4 switch statement
      ** to get better code out of gcc.
      */

      if((evt.event & KRN_EVT_TINY_MASK) != KRN_EVT_EXTENDED)
	{
	  if(likely(evt.event == KRN_EVT_SIG_SEND))
	    {
	      krn_evt_sig_send(&evt); /* pid, sigadm* */
	    }
	  else if(evt.event == KRN_EVT_MUTEX_RAISE)
	    {
	      krn_evt_mutex_raise(&evt);
	    }
	  else if (evt.event == KRN_EVT_FLUSH_TLB)
            {
              krn_evt_flush_tlb(&evt);
            }
          else
	      ramlog_printf("Wild KRN_EVT: %#lx.\n", evt.event);  
	}
      else
	{
	  /* >>4 to improve gcc object code. */
	  switch(evt.event >> 4)
	    {
	    case KRN_EVT_RESCHEDULE>>4: /* pid, NULL (background timeslice) */
	      zzww.dp_req = 1;
	      break;
	  
	    case KRN_EVT_YIELD>>4:	/* 0, NULL (2 b used in handle_error??) */
	      zzkrn_smp_stop_cpu(0, 0);
	      break;

	    case KRN_EVT_SIGNAL_SEM>>4: /* 0, sigcb* */
	      krn_evt_signal_sem(&evt);
	      break;
	  
	    case KRN_EVT_SIG_COUNT>>4:	/* pid, NULL */
	      krn_evt_sig_count(&evt);
	      break;
	  
	    case KRN_EVT_START>>4:	/* pid, NULL */
	    case KRN_EVT_RESUME>>4:	/* pid, NULL */
	      krn_evt_start_resume(&evt);
	      break;
	  
	    case KRN_EVT_STOP>>4:	/* pid, NULL */
	    case KRN_EVT_INTERCEPT>>4:	/* pid, NULL */
	      krn_evt_stop_intercept(&evt);
	      break;
	  
	    case KRN_EVT_SIGNAL_FSEM>>4: /* pid, NULL */
	      krn_evt_signal_fsem(&evt);
	      break;
	  
	    case KRN_EVT_SET_PRI_FOR>>4: /* pid, prio<<16 */
	      krn_evt_set_pri_for(&evt);
	      break;
	  
	    case KRN_EVT_TICK_TI_PROC>>4: /* pid, NULL */
	      krn_evt_wake_ti_proc(&evt);
	      break;
	  	  
	    case KRN_EVT_TIMEOUT>>4:	/* pid, NULL */
	      krn_evt_timeout(&evt);
	      break;

	    case KRN_EVT_TICK>>4:	/* 0, NULL */
	      krn_evt_tick(&evt);
	      break;

	    case KRN_EVT_MIGRATE>>4:	/* pid, to_cpu<<16 */
	      krn_evt_migrate(&evt);
	      break;
	      
	    case KRN_EVT_SEND_SPORT>>4: /* pid, NULL */
	      krn_evt_send_sport(&evt);
	      break;
	  
	    case KRN_EVT_BLOCK_HOOK>>4: /* pid, NULL */
	      krn_evt_block_hook(&evt);
	      break;
	  
	    default:
	      ramlog_printf("Wild KRN_EVT: %#lx.\n", evt.event);
	      break;
	    } /* switch */
	}
    } /* while */
}

/*
** Request the kernel to process kernel events posted by other cpus
** in an SMP kernel. This function is called with, and returns with
** interrupts disabled.
**
** Return values are defined as for the zzinit_int() function. This
** function is used when handling IPIs by returning the special magic
** IPI vector number 0xff01 from the vector handler.
*/

OSBOOLEAN 
zzkrn_evt_notify(OSPRIORITY oldint_level)
{
  if(zzww.ipi_operable == 0)
    ramlog_printf("IPI operable @ cpu%d\n", zzkrn_cpu_id());
  zzww.ipi_operable = 1;
  /* Tell the idle loop that an interrupt has occurred. */
  zzww.dp_idle_interrupted = 1;

  local_krn_evt_notify();
  return zzkrn_int_dispatch(oldint_level);
}

/*
** Request the kernel to process kernel events posted by other cpus
** in an SMP kernel. This function is called with, and returns with
** interrupts disabled.
**
** This function is used when handling IPIs with ISRs that handle the
** call to zzkrn_int_dispatch by themselves.
*/

void
zzkrn_evt_notify_from_isr(void)
{
  if(zzww.ipi_operable == 0)
    ramlog_printf("IPI operable @ cpu%d (isr)\n", zzkrn_cpu_id());
  zzww.ipi_operable = 1;
  /* Tell the idle loop that an interrupt has occurred. */
  zzww.dp_idle_interrupted = 1;
  local_krn_evt_notify();
}

#endif

/*
** Breakpoint interrupt entrypoint.
**
** A process was caught in a breakpoint. The target interface is not required to
** understand if the correct process was caught. The kernel checks this.
**
** Returns with interrupts disabled by zzlock_push().
** Returns 0 if a normal interrupt return should be performed. Breakpoint was valid.
** Returns 1 if a process switch is required. Breakpoint was valid.
** Returns 2 if a wakeup call to a lower priority interrupt is required.
** Breakpoint was valid.
** Returns 4 if a normal interrupt return should be performed. Breakpoint was invalid.
** Returns 5 if a process switch is required. Breakpoint was invalid.
** Returns 6 if a wakeup call to a lower priority interrupt is required.
** Breakpoint was invalid.
*/

/* ===DEPRECATED=== */
OSBOOLEAN
zzbreakpoint_trap(OSPRIORITY int_level, OSPRIORITY oldint_level,
		  void *breakpoint_id)
{
  (void)int_level;
  (void)oldint_level;
  (void)breakpoint_id;
  
  os_error(1, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 23); /* Non fatal */
  os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 23); /* Fatal */
  /* NOTREACHED */
  return 0;
}

/*
** Lock the dispatcher.
**
** After this call has been executed, no further process switches
** will take place until the zzunlock_dispatcher() function is called.
** Any process switch requests are queued internally in the C kernel
** until that time.
**
** This call may be executed with or without interrupts enabled.
**
** This call may be used by the target interface to avoid process switches
** during the single step operations that are required in breakpoint
** processing. This single step operation can not be executed on an
** interrupt level, but it is still crucial that no other non interrupt process
** except the current is allowed to run.
*/

void 
zzlock_dispatcher(void)
{
  LOCK_SWAP();
}

/*
** Unlock the dispatcher.
**
** This call negates the effect of the zzlock_dispatcher() call.
** The call returns a flag set if a process switch is required, in
** which case the caller must execute the zzquit_int() function.
**
** Returns 0 if no process switch is required.
** Returns 1 if a process switch is required.
**
** This call must be executed with interrupts disabled, and it may be
** called only from a non interrupt process (bg or pri).
*/

OSBOOLEAN 
zzunlock_dispatcher(void)
{
  SILENT_UNLOCK_SWAP();
  if(zzww.dp_req)
    return (OSBOOLEAN) 1;
  else
    return (OSBOOLEAN) 0;
}

/*
** Save the current process's state.
**
** This function should be called when entering a page fault handler
** if it wants to issue system calls that might end up using the
** pcb->bq and pcb->fq queue, like any kind of receive that uses a
** timeout, wait_sem or mutex_lock.
**
** This because a page fault may occur in for instance receive_w_tmo()
** when iswanted() is reading the SIGSELECT mask. Since
** receive_w_tmo() has issued a request_timeout() which uses the
** pcb->bq and pcb->fq queue, that timeout *MUST* be canceled before
** any other system calls need to use the fq/bq pointers.
**
** The ST_TIMEOUT bit is also saved and cleared by this function.
**
** When zzkrn_restore_pcb_state() restores the state it will add the
** time spent between saving and restoring the state to the timeout.
*/

void
zzkrn_save_pcb_state(struct _pcb_state_buf *state)
{
  struct _pcb *pcb = zzcur_p;
  LOCKSTATE msr;

  /* Mark the state data as "under construction". */
  state->magic = 123;
  
  /* Save current timeout status and remove any pending or expired timeout.*/
  LOCK_SAVE(msr);
  state->status = pcb->status;
  state->tmo_active = pcb->tmo_active;
  if(state->tmo_active)
    {
      state->tmo_fragment = pcb->cur_tmo_expires - zzww.ticks;
      cancel_timeout();
    }
  else
    {
      pcb->status &= (pr_state_t) ~ST_TIMEOUT;

      /* Unnecessary assign, but it will remove a compiler warning. */
      state->tmo_fragment = 0; 
    }
  LOCK_RESTORE(msr);
  state->magic = MAGIC_SAVED_PCB_STATE;
}

/*
** Restore the current process's state.
**
** This function should be called when exiting a pagefault
** handler. Its purpose is described in detail above, at
** zzkrn_save_pcb_state().
*/

void
zzkrn_restore_pcb_state(struct _pcb_state_buf *state)
{
  LOCKSTATE msr;

  if(state->magic == MAGIC_SAVED_PCB_STATE)
    {
      state->magic |= MAGIC_KILLED;
      /* Restore old timeout state */
      if(state->status & ST_TIMEOUT)
	{
	  /* Restore an expired timer */
	  LOCK_SAVE(msr);
	  zzcur_p->status |= (pr_state_t) ST_TIMEOUT;
	  LOCK_RESTORE(msr);
	}
      else if(state->tmo_active)
	{
	  /* Restore a pending timer */
	  LOCK_SAVE(msr);
	  request_timeout(zzcur_p, state->tmo_fragment);
	  LOCK_RESTORE(msr);
	}
    }
  else
    {
      os_error(2, OSE_EBAD_PARAMETER, (OSERRCODE)state);	/* Fatal system error. */
    }
}

/*
*****************************************************************************
** 10 BLOCK MANAGEMENT.
*****************************************************************************
*/

/*
**===========================================================================
** 10.1  Initialization.
**===========================================================================
*/

/*
**===========================================================================
** 10.2  Local functions.
**===========================================================================
*/

/*
** Check that a block id is valid and alive.
** Return a pointer to the block control block.
** Return NULL if killed.
** Report error and return NULL if invalid.
** A process id is considered invalid.
** Bits of error code representing the system call is supplied by the caller.
**
** Note that the block may be killed while this function is executed, so the
** caller must always check the instance number again later to be certain that
** the block exists. This check should be done with a locked dispatcher.
**
** The dispatcher must not be locked when this function is called since an
** error may be generated.
**
** The most significant 16 bits of the process id is the instance number.
** The least significant 16 bits is the process index.
*/

static struct _block *
check_bid(PROCESS bid, OSERRCODE errcode)
{
  const PCB_INDEX_T pindex = pid_to_pindex(bid);
  const PCB_INSTANCE_T instance = pid_to_instance(bid);
  register struct _block *bcb;

  /* Check that the process index is reasonable. */
  if(unlikely(pindex >= zzuu.next_free_pix))
    {
      os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) bid);	/* Non fatal. */
      return NULL;
    }

  /* Pick up pcb pointer. */
  bcb = (struct _block *) zzuu.pix_list[pindex];

  /* Return NULL if the block has been killed. */
  if(unlikely(pcb_to_instance(bcb) != instance))
    {
      /* But make an extra error check on pid = 0, which is illegal. */
      if(!pindex || !instance)
	os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) bid);	/* Non fatal. */
      return NULL;
    }

  /* Check that the magic number indicates a block. */
  if(unlikely(bcb->magic != MAGIC_BCB))
    {
      goto check_bid1;
    }

  /* Check that the user numbers match. */
  /*
  ** Note that the specification states that a block is visible if it is
  ** owned by another user but it contains at least one process of the
  ** same user.
  ** We don't support this now for reasons of efficiency, but we must
  ** in the future. There is a similar problem in get_bid_list(). ??
  */
  if(unlikely(zzcur_p->user))
    {
      if(zzcur_p->user != bcb->user)
	{
	  goto check_bid1;
	}
    }

  /* Return a pointer to the block contol block if no problem was found. */
  return bcb;

  /*
  ** Report error if an object of wrong type or with an invisible user
  ** number was found. Always return NULL.
  */
 check_bid1:
  {
    volatile struct _pcb *vbcb = (volatile struct _pcb*)bcb;
    if(instance == pcb_to_instance(vbcb) && !vbcb->killed)
      {
	os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) bid);	/* Non fatal. */
      }
  }
  return NULL;
}

/*
** Set up fields in a block descriptor.
**
** A block descriptor supplied by the caller is initialized.
** The caller supplies important fields as parameters.
** Name buffer should have been allocated by the caller.
**
** The pindex and instance fields were set by alloc_pcb().
** The next field is not set.
**
** Must be called with a locked dispatcher since it traverses the
** zzuu.segment_list.
*/

static void
setup_bcb(struct _block *bcb,
	  OSUSER user,
	  const char *name,
	  struct _pcb *parent,
	  PROCESS remcall_server,
	  OSBOOLEAN use_remcalls,
	  OSBOOLEAN supervisor_mode,
	  struct poolcb *stackpool,
	  struct poolcb *sigpool,
	  OSSEGMENT segid,
	  PROCESS segpid
	  )
{
  register struct segcb *segp;

  bcb->env_list = NULL;
  bcb->error_handler = (OSERRH *) NULL;
  bcb->attached = NULL;
  bcb->user = user;
  bcb->proclist = NULL;
  bcb->pcb_list_modno = 0;
  bcb->name = name;
  bcb->st_cnt = 0;
  bcb->parent = parent;
  bcb->block_process = NULL;
  bcb->remcall_server = remcall_server;
  bcb->use_remcalls = use_remcalls;
  bcb->supervisor_mode = supervisor_mode;
  bcb->stackpool = stackpool;
  bcb->sigpool = sigpool;
  bcb->segid = segid;
  bcb->segpid = segpid;
  bcb->is_main_block = (OSBOOLEAN) 0;
  bcb->shared_mode = (OSBOOLEAN) 0;
  bcb->kill_in_progress = (OSBOOLEAN) 0;
  bcb->prof = 0;
  bcb->lnh_name = NULL;
  bcb->isr_list = NULL;
  bcb->statehook_list = NULL;

  /*
  ** Inherit clib pointer from parent block if there is a parent.
  ** Inherit the cpu_id from parent, if there is one. Thus, the cpu_id 
  ** will not be a detached cpu.
  */
  if(parent)
    {
      bcb->clib_pointer = parent->block->clib_pointer;
      bcb->cpu_id = parent->cpu_id;
    }
  else
    {
      bcb->clib_pointer = NULL;
      bcb->cpu_id = (cpuid_t)0;
    }

  /* Increment user counters in pool descriptor. */
  stackpool->no_of_stack_users++;
  sigpool->no_of_sig_users++;

  /* Locate the segment descriptor. */
  if(segpid)
    {
      segp = pid_to_segcb(segpid);
      /* Increment the usage counter in the segment descriptor. */
      segp->block_count++;
      return;
    }
  else if (segid == 0)
    {
      /* Segid zero is the same as the system segment. */
      system_segment.block_count++;
      return;
    }
  else
    {
      for(segp = zzuu.segment_list; segp; segp = segp->next)
	{
	  if(segp->segid == segid)
	    {
	      /* Increment the usage counter in the segment descriptor. */
	      segp->block_count++;
	      return;
	    }
	}
    }

  /* Internal error if segment was not found. */
  os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 0);	/* Fatal. */
}

/*
**===========================================================================
** 10.3  System calls.
**===========================================================================
*/

/*
** This is the get_bid() system call.
*/

PROCESS
get_bid(PROCESS pid)
{
  LOCKSTATE msr;
  register struct _pcb *pcb;

  /* Check the process id. Report error if invalid. */
  pcb = check_pid_bid(pid, OSE_GET_BID);
  if(pcb == NULL)
    {
      return pid;
    }

  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);

  /* Check that the block/process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      BKL_LOCK_RESTORE(msr);
      return pid;
    }

  /* Return the pid unaltered if the pid is a block. */
  if(pcb->magic == MAGIC_BCB)
    {
      BKL_LOCK_RESTORE(msr);
      return pid;
    }

  /* Calculate the block id of the process. */
  pid = pcb_to_pid(pcb->block);
  BKL_LOCK_RESTORE(msr);

  /* Return the block id of the process. */
  return pid;
}


/*
** Create a new block.
*/

PROCESS
create_block(const char *name,
	     OSUSER user,
	     OSBOOLEAN use_remcalls,
	     PROCESS remcall_server,
	     OSBOOLEAN supervisor_mode)
{
  char *name_buf;
  struct _block *bcb;
  register struct _pcb *pcb;
  
#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_CREATE_BLOCK))
    return any_killed_pid();
#endif

  if(use_remcalls)
    {
      /* Read the instance number and check the remcall server id. Report error if invalid. */
      pcb = check_pid(remcall_server, OSE_CREATE_BLOCK);
      if(pcb == NULL)
	{
	  return any_killed_pid();
	}
      /* Locate the process descriptor. */
      pcb = pid_to_pcb(remcall_server);
    }
  else
    {
      pcb = NULL;
    }

  /* May create another user only if caller is the superuser. */
  user = zzcur_p->user ? zzcur_p->user : user;

  /*
  ** May create a new block in shared exec mode only if 
  ** the system configuration permits it.
  */
  if(zzcur_p->block->shared_mode &&
     0 == (0x10 & local_sysctlblk.relax_error_checks))
    {
      /*
      ** The error code OSE_EPRIVILEGED_SYSCALL hints that the extra
      ** parameter contains bits required in the system parameter.
      ** Non-fatal error, may continue.
      */
      os_error(0, OSE_CREATE_BLOCK | OSE_EPRIVILEGED_SYSCALL, 0x10);
    }

  /*
  ** May create a supervisor mode process only if the caller is the superuser
  ** executing a supervisor mode process.
  */
  if(supervisor_mode)
    {
      if(zzcur_p->user || !zzcur_p->block->supervisor_mode)
	{
#if ERROR_CHECKS > 1
	  ON_ERRLEVEL_2()
	    os_error(0, OSE_EILLEGAL_SUPER_MODE | OSE_CREATE_BLOCK, (OSERRCODE) name);	/* Non fatal. */
#endif
	  supervisor_mode = (OSBOOLEAN) 0; 
	}
    }
  else
    {
#if ERROR_CHECKS > 0
      /*
      ** Warning if the caller is a supervisor process  
      ** and user mode not accepted there. The new block is likely to cause 
      ** protection exceptions. Program managers will then need a local 
      ** error handler to ignore this warning. 
      */
      if(0 == (local_sysctlblk.relax_error_checks & 1) &&
	 zzcur_p->block->supervisor_mode)
	{
	  os_error(0, OSE_EILLEGAL_USER_MODE | OSE_CREATE_BLOCK, (OSERRCODE) name); /* Non fatal. */
	  /* This was just a warning, so continue and create the block. */
	}
#endif
    }
  

  /* Allocate a name buffer in the system pool and copy the name to it. */
  {
    size_t namelen = strlen(name);
    /* Make sure that the name can be fitted in the system pool. */
    if(system_pool.signal_list[7].size < (OSADDRESS) (sizeof(SIGSELECT) + namelen))
      {
	os_error(0, OSE_EBAD_PARAMETER | OSE_CREATE_BLOCK, (OSERRCODE) name);	/* Non fatal. */
	return any_killed_pid();
      }
    name_buf = (char *) sys_alloc((OSBUFSIZE) (sizeof(SIGSELECT) + namelen), (SIGSELECT) 0);
    memcpy(name_buf, name, namelen);
    name_buf[namelen] = (char) '\0';	/* String terminator just in case... */
  }

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the remcall handler is still present. */
  if((pcb != NULL) &&
     (pcb_to_pid(pcb) != remcall_server))
    {
      UNLOCK_SWAP();
      free_buf_type(char *, &name_buf);
      return any_killed_pid();
    }

  /* Allocate a new block descriptor. */
  bcb = &alloc_pcb(OSE_CREATE_BLOCK)->bcb;

  /* Check that we got a block descriptor. */
  if(!bcb)
    {
      /* No, abort the system call. */
      UNLOCK_SWAP();
      free_buf_type(char *, &name_buf);
      return any_killed_pid();
    }

  /* Change the name buffer owner. */
  change_sig_owner(sig_to_adm(name_buf), NULL);

  /* Initialize the new block descriptor. */
  setup_bcb(bcb,
	    user,
	    name_buf,
	    zzcur_p,
	    remcall_server,
	    use_remcalls,
	    supervisor_mode,
	    zzcur_p->block->stackpool,
	    zzcur_p->block->sigpool,
	    zzcur_p->block->segid,
	    zzcur_p->block->segpid
	    );

  /* Link new block descriptor into system block list. */
  /*
  ** Note that there is no race condition here since we only add things
  ** at the head of the list. Other processes traversing the list may
  ** miss the new entry, but won't crash.
  */
  bcb->next = zzuu.bcb_list;
  zzuu.bcb_list = bcb;

  /* Make the block visible. */
  bcb->magic = MAGIC_BCB;
  bcb->killed = (OSBOOLEAN) 0;
  
  /* Unlock the dispatcher. */
  UNLOCK_SWAP();

  /*
  ** Skip environment variables if a remcall handler is present.
  ** They can not be read anyway.
  */
  if(!bcb->use_remcalls)
    {
      register struct _block *parent = zzcur_p->block;
      register struct env_buf *envbuf;
      PROCESS bid = pcb_to_pid(bcb);

      /* Copy all environment variables from the parent block. */
      for(envbuf = parent->env_list; envbuf; envbuf = envbuf->next)
	{
	  if(envbuf->value)
	    set_env(bid, envbuf->name, envbuf->value);
	}
    }
  
  set_suid((OSBOOLEAN) 1);
  /* Notify the MMU of the new block. */
  handlerp.zzcall_attach_block(pcb_to_pid(bcb), bcb->segid);

  /* Notify the hunt daemon of the new block */
    {
      union SIGNAL *sig; 
  
      sig = alloc(sizeof(struct new_process), NEW_PROCESS);
      sig->new_process.pid = pcb_to_pid(bcb);
      /* 
      ** Using pcb_to_pid() on the block pointer will result in 
      ** an unexpected pid if the block has been killed. However, 
      ** here we do it for current process so we should be safe.
      */ 
      sig->new_process.block = pcb_to_pid(zzcur_p->block);
      send(&sig, zzuu.huntd_);
    }
  set_suid((OSBOOLEAN) 0);

  /* Return the new block id to the caller. */
  return pcb_to_pid(bcb);
}

/*
**===========================================================================
** 10.4  Undocumented system calls.
**===========================================================================
*/

/* call to set shared mode flag for a given block. Return value not used. */
static OSADDRESS
sysconf_63(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  LOCKSTATE msr;
  struct _block *bcb;
  PROCESS bid = (PROCESS)p;

  (void)r;
   
  /* Locate the block or process descriptor. */
  /* Read instance number and check the process id. Report error if invalid. */
  bcb = check_bid(bid, 0);
  if(bcb == NULL)
    {
      return 0;
    }
   
  BKL_LOCK_SAVE(msr);

  /* Set the shared mode flag if the block is still alive. */
  if(pcb_to_pid(bcb) == bid)
    bcb->shared_mode = (OSBOOLEAN) q;
   
  BKL_LOCK_RESTORE(msr);
  return 0;
}

/* return shared mode flag. */
static OSADDRESS
sysconf_64(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  LOCKSTATE msr;
  struct _block *bcb;
  PROCESS bid = (PROCESS)p;
  OSADDRESS shared_mode = 0;

  (void)q;
  (void)r;
   
  /* Locate the block or process descriptor. */
  /* Read instance number and check the process id. Report error if invalid. */
  bcb = check_bid(bid, 0);
  if(bcb == NULL)
    {
      return 0;
    }
   

  BKL_LOCK_SAVE(msr);

  if(pcb_to_pid(bcb) == bid)
    shared_mode = (OSADDRESS) bcb->shared_mode;
  BKL_LOCK_RESTORE(msr);
  return shared_mode;
}

/*
*****************************************************************************
** 11 PROCESS MANAGEMENT.
*****************************************************************************
*/

/*
**===========================================================================
** 11.1  Initialization.
**===========================================================================
*/

/*
**===========================================================================
** 11.2  Local functions.
**===========================================================================
*/

/* Returns a pointer to a vector control block. If allocate is set, a
 * new entry will be allocated if needed. Should be called with locked
 * dispatcher. */
static struct vectorcb *
get_vectorcb(OSVECTOR vecno, OSBOOLEAN allocate)
{
  struct vectorcb *veccb;
  uint8_t slot = vecno % 256;
    
  /* Check the linked list of vectors. */
  for (veccb = zzuu.vectors[slot]; veccb != NULL; veccb = veccb->next)
    {
      if (veccb->vecno == vecno)
        {
          return veccb;
        }
    }

  /* No matching entry. */
  if (!allocate)
    {
      return NULL;
    }

  /* Allocate a new entry. */
  veccb = (struct vectorcb *)pool_brk_lo(&system_pool,
                                         sizeof(struct vectorcb));
  veccb->next = zzuu.vectors[slot];
  zzuu.vectors[slot] = veccb;

  /* Clear and initialize the new entry. */
  memset(veccb, 0, sizeof(struct vectorcb));
  veccb->vecno = vecno;
  return veccb;
} 

/* Removes interrupt hooks from a vector control block that has no ISR
 * or interrupt process remaining. The dispatcher should be locked
 * when calling this function. */
static void
clear_vectorcb_hooks(struct vectorcb *veccb)
{
  int i;

  /* Should only call this function on an empty vector. */
  assert(veccb->isrcount == 0);
  assert(veccb->pcbcount == 0);

  for (i = 0; i < 7; ++i)
    {
      if ((veccb->isr[i].isr != NULL) && (veccb->isr[i].type == 0))
        {
          detach_isr_or_hook(veccb, i);
          veccb->isr[i].isr = NULL;
        }
    }

  /* Can also reset the bind information for this vector. */
  veccb->cpus = 0;
}

/* Only designed to be called from devman, which in turn will have
 * been called with the dispatcher locked. Devman uses this to figure
 * out if it has to send a create request to a different core. This in
 * turn is only ever needed for interrupt processes that calls devman
 * from sysd on core 0. */
static cpuid_t
get_vectorcb_cpu(OSVECTOR vecno)
{
#ifndef OSE_SMP
  (void)vecno;
  return 0;
#else
  struct vectorcb *veccb;
  uint8_t slot = vecno % 256;
  cpuid_t retval = 0;
  
  /* Check the linked list of vectors. */
  for (veccb = zzuu.vectors[slot]; veccb != NULL; veccb = veccb->next)
    {
      if (veccb->vecno == vecno)
        {
          /* For ISRs we always think we are on the correct core. */
          if (veccb->isrcount > 0)
            return zzkrn_cpu_id();

          /* Otherwise only one cpu should be bound. */
          retval = 31 - FIND_FIRST_BIT(veccb->cpus);
          break;
        }
    }

  return retval;
#endif /* OSE_SMP */  
}

/*
** Check that a block or process or segment id is valid and alive.
** Return a pointer to the control block.
** Return NULL if killed.
** Report error and return NULL if invalid.
** Bits of error code representing the system call is supplied by the caller.
**
** Note that the block or process may be killed while this function is executed,
** so the caller must always check the instance number again later to be
** certain that the block/process exists.
** This check should be done with a locked dispatcher.
**
** The dispatcher must not be locked when this function is called since an
** error may be generated.
**
** The most significant 16 bits of the process id is the instance number.
** The least significant 16 bits is the process index.
*/

static struct _pcb *
check_pid_bid_seg(PROCESS pid, OSERRCODE errcode)
{
  struct _pcb *cur_p = zzcur_p;
  const PCB_INDEX_T pindex = pid_to_pindex(pid);
  const PCB_INSTANCE_T instance = pid_to_instance(pid);
  register struct _pcb *pcb;

  /* Check that the process index is reasonable. */
  if(unlikely(pindex >= zzuu.next_free_pix))
    {
      os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) pid);	/* Non fatal. */
      return NULL;
    }

  /* Pick up pcb pointer. */
  pcb = zzuu.pix_list[pindex];

  /* Return NULL if the process/block has been killed. */
  if(unlikely(pcb_to_instance(pcb) != instance))
    {
      /* Check for illegal PID values. */
      if(!pindex || !instance)
	{
	  os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) pid); /* Non fatal. */
	}
      return NULL;
    }

  /* Check that the magic number indicates a live process, block or segment. */
  if((pcb->magic & ~3) != MAGIC_PCB)
    {
      goto check_pid1;
    }

  /* Check that either the user numbers or blocks matches. */
  /*
  ** For blocks, note the user visibility problem as in get_bid_list().??
  */
  if(unlikely(cur_p->user))
    {
      if(pcb->magic == MAGIC_PCB)
	{
	  if(cur_p->user != pcb->real_user &&
	     cur_p->block != pcb->block)
	    {
	      goto check_pid2;
	    }
	}
      else
	{
	  if(cur_p->user != pcb->user &&
	     cur_p->block != (struct _block *)pcb &&
	     cur_p->block->segpid != pid)
	    {
	      goto check_pid2;
	    }
	}
    }

  /* Return a pointer to the control block if no problem was found. */
  return pcb;

  /*
  ** Found a killed process/block or another type of object.
  ** Accept a reference of the process being created when called from
  ** a create_handler, i.e. by sysd, for any call but kill_proc(). 
  */
 check_pid1:
  if(pcb_to_pid(cur_p) == zzuu.sysd_ && errcode != OSE_KILL_PROC)
    {
      if(zzuu.process_under_construction_ == pid &&
	 pcb->magic == (MAGIC_PCB | MAGIC_KILLED))
	{
	  return pcb;
	}
    }
  
  /*
  ** Report error if an object of wrong type or with an invisible user
  ** number was found. Always return NULL.
  */
 check_pid2:
  {
    volatile struct _pcb *vpcb = (volatile struct _pcb*)pcb;
    if(instance == pcb_to_instance(vpcb) && !vpcb->killed)
      {
	os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) pid); /* Non fatal. */
      }
  }
  return NULL;
}

/*
** Check that a block or process id is valid and alive.
** Return a pointer to the control block.
** Return NULL if killed.
** Report error and return NULL if invalid.
** Bits of error code representing the system call is supplied by the caller.
**
** Note that the block or process may be killed while this function is executed,
** so the caller must always check the instance number again later to be
** certain that the block/process exists.
** This check should be done with a locked dispatcher.
**
** The dispatcher must not be locked when this function is called since an
** error may be generated.
**
** The most significant 16 bits of the process id is the instance number.
** The least significant 16 bits is the process index.
**
** There seems to be a bug here and in get_pid, get_bid, get_ptype and
** possibly get_pcb.??
** It should be OK if the examined process is part of the callers block,
** even if the user numbers are different. This code renders this situation
** invalid.
*/


static struct _pcb *
check_pid_bid(PROCESS pid, OSERRCODE errcode)
{
  struct _pcb *cur_p = zzcur_p;
  const PCB_INDEX_T pindex = pid_to_pindex(pid);
  const PCB_INSTANCE_T instance = pid_to_instance(pid);
  register struct _pcb *pcb;

  /* Check that the process index is reasonable. */
  if(unlikely(pindex >= zzuu.next_free_pix))
    {
      os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) pid);	/* Non fatal. */
      return NULL;
    }

  /* Pick up pcb pointer. */
  pcb = zzuu.pix_list[pindex];

  /* Return NULL if the process/block has been killed. */
  if(unlikely(pcb_to_instance(pcb) != instance))
    {
      /* Check for illegal PID values. */
      if(!pindex || !instance)
	os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) pid); /* Non fatal. */
      return NULL;
    }

  /* Check that the magic number indicates a live block or process. */
  if(unlikely((pcb->magic & ~1) != MAGIC_PCB))
    {
      goto check_pid1;
    }

  /* Check that either the user numbers or blocks matches. */
  /*
  ** For blocks, note the user visibility problem as in get_bid_list().??
  */
  if(unlikely(cur_p->user))
    {
      if(pcb->magic == MAGIC_PCB)
	{
	  if(cur_p->user != pcb->real_user &&
	     cur_p->block != pcb->block)
	    {
	      goto check_pid2;
	    }
	}
      else
	{
	  if(cur_p->user != pcb->user &&
	     cur_p->block != (struct _block *)pcb)
	    {
	      goto check_pid2;
	    }
	}
    }

  /* Return a pointer to the control block if no problem was found. */
  return pcb;

  /*
  ** Found a killed process/block or another type of object.
  ** Accept a reference of the process being created when called from
  ** a create_handler, i.e. by sysd, for any call but kill_proc(). 
  */
 check_pid1:
  if(pcb_to_pid(cur_p) == zzuu.sysd_ && errcode != OSE_KILL_PROC)
    {
      if(zzuu.process_under_construction_ == pid &&
	 pcb->magic == (MAGIC_PCB | MAGIC_KILLED))
	{
	  return pcb;
	}
    }
  
  /*
  ** Report error if an object of wrong type or with an invisible user
  ** number was found. Always return NULL.
  */
 check_pid2:
  {
    volatile struct _pcb *vpcb = (volatile struct _pcb*)pcb;
    if(instance == pcb_to_instance(vpcb) && !vpcb->killed)
      {
	os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) pid); /* Non fatal. */
      }
  }
  return NULL;
}

/*
** Check that a process id is valid and alive.
** Return a pointer to the process control block.
** Return NULL if killed.
** Report error and return NULL if invalid.
** A block id is considered invalid.
** Bits of error code representing the system call is supplied by the caller.
**
** Note that the process may be killed while this function is executed,
** so the caller must always check the instance number again later to be
** certain that the process exists.
** This check should be done with a locked dispatcher.
**
** The dispatcher must not be locked when this function is called since an
** error may be generated.
**
** The most significant bits of the process id is the instance number.
** The least significant bits is the process index.
*/

static struct _pcb *
check_pid(PROCESS pid, OSERRCODE errcode)
{
  struct _pcb *cur_p = zzcur_p;
  const PCB_INDEX_T pindex = pid_to_pindex(pid);
  const PCB_INSTANCE_T instance = pid_to_instance(pid);
  register struct _pcb *pcb;

  /* Check that the process index is reasonable. */
  if(unlikely(pindex >= zzuu.next_free_pix))
    {
      os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) pid);	/* Non fatal. */
      return NULL;
    }

  /* Pick up pcb pointer. */
  pcb = zzuu.pix_list[pindex];

  /* Return NULL if the process has been killed. */
  if(unlikely(pcb_to_instance(pcb) != instance))
    {
      /* Check for illegal PID values. */
      if(!pindex || !instance)
	os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) pid); /* Non fatal. */
      return NULL;
    }

  /* Check that the magic number indicates a live process. */
  if(unlikely(pcb->magic != MAGIC_PCB))
    {
      goto check_pid1;
    }

  /* Check that either the user numbers or blocks matches. */
  if(unlikely(cur_p->user))
    {
      if(cur_p->user != pcb->real_user &&
	 cur_p->block != pcb->block)
	{
	  goto check_pid2;
	}
    }

  /* Return a pointer to the process control block if no problem was found. */
  return pcb;

  /*
  ** Found a killed process or an object of any other type.
  ** Accept a reference of the process being created when called from
  ** a create_handler, i.e. by sysd, for any call but kill_proc(). 
  */
 check_pid1:
  if(pcb_to_pid(cur_p) == zzuu.sysd_ && errcode != OSE_KILL_PROC)
    {
      if(zzuu.process_under_construction_ == pid &&
	 pcb->magic == (MAGIC_PCB | MAGIC_KILLED))
	{
	  return pcb;
	}
    }
  
  /*
  ** Report error if an object of wrong type or with an invisible user
  ** number was found. Always return NULL.
  */
 check_pid2:
  {
    volatile struct _pcb *vpcb = (volatile struct _pcb*)pcb;
    if(instance == pcb_to_instance(vpcb) && !vpcb->killed)
      {
	os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) pid); /* Non fatal. */
      }
  }
  return NULL;
}

/*
** Check that a process (or other control block) id is valid and alive.
**
** If skip_magic_check is 0:
**   Return a pointer to the process' PCB iff (if and only if) the id
**   is a valid/alive process id, otherwise return NULL.
**
** If skip_magic_check is 1:
**   Return a pointer to a control block iff (if and only if) the
**   id's index points out a valid/alive control block, otherwise
**   return NULL. This is used for getting a pointer to the
**   controlblock regardless of its type.
**
** If skip_magic_check is 2:
**   Returns the same as for skip_magic_check == 1 but with the ability
**   to return pointers to dead control blocks.
**
** The dispatcher as well as interrups may be (and probably should be)
** locked when this function is called.
**
** This function do not call any error handlers.
*/

static struct _pcb *
safe_pid_to_pcb(PROCESS pid, int skip_magic_check)
{
  PCB_INDEX_T pindex = pid_to_pindex(pid);
  struct _pcb *pcb;

  /* Check that the process index is reasonable. */
  if(likely(pindex < zzuu.next_free_pix))
    {
      /* Pick up pcb pointer. */
      pcb = zzuu.pix_list[pindex];

      if(skip_magic_check == 0)
	{
	  /* Check that the magic number indicates a process. */
	  if(likely(pcb->magic == MAGIC_PCB))
	    {
	      /* Make sure it is the process we want. */
	      if(likely(pcb->live_pid == pid))
		return pcb;
	    }
	}
      else if(skip_magic_check == 1)
	{
	  if(likely(!(pcb->magic & MAGIC_KILLED) &&
		    (pcb_to_pid(pcb) == pid)))
	    {
	      return pcb;
	    }
	}
      else if(skip_magic_check == 2)
	{
	  return pcb;
	}
    }

  /* We can not find any such process. */
  return NULL;
}

/*
** Find the segment control block related to a pcb/bcb/poolcb/segcb
** pointer.
**
** If the pcb object is dead, NULL is returned. If the argument
** is a segment control block, a pointer to this object is returned.
**
** Either interrupts or dispatcher should be locked by the caller.
*/
static struct segcb *
find_segcb(struct _pcb *pcb)
{
  struct segcb *segcb;

  switch(pcb->magic)
    {
    case MAGIC_PCB:
      segcb = pid_to_segcb(pcb->block->segpid);
      break;
      
    case MAGIC_BCB:
      segcb = pid_to_segcb(((struct _block *)pcb)->segpid);
      break;
      
    case MAGIC_POOL:
      segcb = pid_to_segcb(((struct poolcb *)pcb)->segpid);
      break;

    case MAGIC_SEG:
      segcb = (struct segcb *)pcb;
      break;
      
    default:
      segcb = NULL;
      break;
    } /* switch */
  return segcb;
}

/*
** Set up basic fields in a process control block.
**
** A control block supplied by the caller is initialized.
** The caller supplies important fields as parameters.
** Name buffer should have been allocated by the caller.
**
** The pindex and instance fields were set by alloc_pcb().
**
** The following fields are left incomplete:
**   next, proclist, cur_tmo, vector, uarea, stack, killed
**
** Block descriptor fields are not updated.
*/

static void
setup_pcb(struct _pcb *pcb,
	  const char *name,
	  OSTIME tmo,
	  OSPRIORITY priority,
	  OSENTRYPOINT *entrypoint,
	  struct _block *block,
	  struct OS_redir_entry *redir
	  )
{
  pcb->env_list = NULL;
  pcb->error_handler = (OSERRH *) NULL;
  pcb->attached = NULL;
  pcb->real_user = pcb->user = block->user;
  pcb->proclist = NULL;
  pcb->debug_flags = (U32) 0;
  pcb->live_pid = pcb_to_pid(pcb);
  pcb->tracename = NULL;
  pcb->name = name;
  if(priority < 33)
    {
      pcb->status = (pr_state_t) (ST_INIT | ST_STOPPED);
      pcb->st_cnt = 1;
    }
  else if(priority < 96)
    {
      pcb->status = (pr_state_t) ST_INIT;
      pcb->st_cnt = 0;
    }
  else if(priority < 128)
    {
      pcb->status = (pr_state_t) (ST_INIT | ST_STOPPED);
      pcb->st_cnt = 1;
    }
  else
    {
      pcb->status = (pr_state_t) ST_INIT;
      pcb->st_cnt = 0;
    }

  pcb->priority = priority;
  pcb->org_prio = 0;
  pcb->tmo = tmo;

  /* Init timeslice counter for prioritized processes. */
  pcb->ts_time = priority < 32 ? tmo : (OSTIME) 0;
  pcb->cur_tmo = (U16) 0;
  pcb->tmo_active = (OSBOOLEAN) 0;
  pcb->entrypoint = entrypoint;
  pcb->vector = (OSVECTOR) -1;
  pcb->fq = NULL;
  pcb->bq = NULL;
  pcb->mailbox = NULL;
  pcb->mailtail = NULL;
  pcb->fsemval = -1;
  pcb->sig_cnt_queued = 0;
  pcb->flushed = (OSBOOLEAN) 0;
  pcb->wanted = NULL;
  pcb->block = block;
  pcb->blockname = block->name;
  pcb->redir = redir;
  pcb->wake_sig_cnt = (U16) 0;
  pcb->wake_sem_cnt = (U16) 0;
  pcb->wake_tick_cnt = (U16) 0;
  pcb->uarea = NULL;
  pcb->stack = NULL;
  pcb->sig_cnt_owned = 0;
  pcb->use_kernel_pool = (OSBOOLEAN) 0;
  pcb->cursigpool = block->sigpool;
  pcb->err_handl_level = (OSBOOLEAN) 0;
  pcb->out_of_pool = (U8) 0;
  pcb->in_rem_call = (OSBOOLEAN) 0;
  pcb->cpu_id = block->cpu_id;
  pcb->alloced_cpu = 0;
  pcb->ready_entry.nxt = NULL;
  pcb->ready_entry.prev = NULL;
  
  if(priority < 32)
    {
      /* PRI */
      pcb->ready_entry.rlist =
	(struct ready_list_entry *) &ready_list_x(pcb->cpu_id)[priority];
    }
  else if(priority < 33)
    {
      /* BG */
      pcb->ready_entry.rlist = NULL;
    }
  else if(priority < 96)
    {
      /* INT */
      /*
      ** We only set 'rlist' here for debug purposes, the real value
      ** will be evaluated each time it's used. This way 'pcb->cpu_id'
      ** and 'pcb->ready_entry.rlist' can't get out of sync when
      ** migrating. @frbr@
      */
      pcb->ready_entry.rlist =
	DEVOLATILE(struct ready_list_entry *,
		   &wakeup_list_x(pcb->cpu_id)[priority & 31]);
    }
  else if(priority < 128)
    {
      /* TIM */
      /*
      ** We only set 'rlist' here for debug purposes, the real value
      ** will be evaluated each time it's used. This way 'pcb->cpu_id'
      ** and 'pcb->ready_entry.rlist' can't get out of sync when
      ** migrating. @frbr@
      */
      pcb->ready_entry.rlist =
	DEVOLATILE(struct ready_list_entry *,
		   &wakeup_list_x(pcb->cpu_id)[zzuu.timer_int_priority]);
    }
  else
    {
      /* PHANT */
      pcb->ready_entry.rlist = NULL;
    }
  
  pcb->ready_entry.is_ready = (OSBOOLEAN) 0;

  pcb->segid = block->segid;

  /* Convert the default process id in the redirection table if required. */
  if(redir && !redir->pid)
      redir->pid = pcb_to_pid(pcb);

  /* Set up signal port related fields. */
  pcb->s_pcb.s_nodelist = NULL;
  pcb->s_pcb.s_nodetail = NULL;
  pcb->s_pcb.add_listmodnumber = 0;
  pcb->s_pcb.take_listmodnumber = 0;
  pcb->is_an_sport = (OSBOOLEAN) 0;
  pcb->is_deaf = (OSBOOLEAN) 0;
  pcb->pending_tos.head = pcb->pending_tos.tail = NULL;
  pcb->expired_tos.head = pcb->expired_tos.tail = NULL;
  pcb->ppd_addr = 0;
  pcb->debug_info = 0;
  pcb->lock_cnt = 0;
  pcb->locked_kill = (OSBOOLEAN)0;
  pcb->isr_list = NULL;
  pcb->nil_alloc = (OSBOOLEAN)0;
  pcb->rec_poller = NULL;
  pcb->error_origin = 2;	/* No error info saved. */
  pcb->error_ecode = 0;
  pcb->error_extra = 0;
}

/*
** The following function is called from the system daemon to create a
** process. It is called through the intermediate function sysd_new_process().
**
** Processes are created by the system daemon on request from other processes.
** This allows long operations without a need for locking the dispatcher.
**
** The block as well as the parent block may be killed while this function
** is executing. The kill operation is however restricted to setting the
** killed flag and modifying the instance number. This means that it is
** still safe to access the block descriptor and the parent pcb.
**
** The block id was checked by the caller before the call to this function,
** so we know that it was alive at that point.
**
** It is possible to call this function directly during system start, provided
** that no interrupt vector is needed, and that the stack is allocated by the
** caller.
**
** The function returns the new process id or the special purpose process id
** any_killed_pid() if the create operation failed.
** If the operation failed, the caller must free the name buffer, the
** redirection table and the stack.
**
** The function leaves the dispatcher locked upon return. This is so that
** the caller can bind an interrupt vector before anyone gets a chance to
** kill the new process.
**
** We should use the highpri_count field in the segment descriptor.??
*/

static PROCESS
sysd_create_process(enum PROCESS_TYPE proc_type,
		    const char *name,
		    OSENTRYPOINT *entrypoint,
		    struct stkcb *stack,
		    OSADDRESS stack_size,
		    OSPRIORITY priority,
		    OSTIME tmo,
	            PROCESS bid,
		    struct OS_redir_entry *redir,
		    PROCESS parent,
		    U32 proc_attrs,
		    OSSENDHOOK *snd_hook,
   		    void *lnh_dest_hd)
{
  PROCESS new_pid;
  register struct _pcb *pcb;
  register struct _block *block = (struct _block *) pid_to_pcb(bid);
  int is_block_process;

  /*
  ** Make sure the timeslice is at least a tick for non prioritized
  ** processes.
  */
  if(proc_type != OS_PRI_PROC)
    {
      tmo = tmo ? tmo : 1;
    }
  else
    {
      /* Check if we allow timeslicing for prioritized processes. */
      if((local_sysctlblk.table_extended < 7) ||
	 !local_sysctlblk.timeslice_priprocs)
	{
	  tmo = 0; /* No we don't, so we will disable it. */
	}
    }

  /* Translate process type to priority. */
  priority = (priority > 31) ? (OSPRIORITY) 31 : priority;
  switch((int) proc_type)
    {
    case OS_BG_PROC:
      {
	priority = (OSPRIORITY) 32;
	break;
      }
    case OS_PHANTOM:
      {
	priority = (OSPRIORITY) 128;
	break;
      }
    case OS_TI_PROC:
      {
	priority += 32;
	/*FALLTHROUGH*/
      }
    case OS_INT_PROC:
      {
	priority += 64;
	/*FALLTHROUGH*/
      }
    case OS_PRI_PROC:
      {
	break;
      }
    default:
      {
	/* Internal bug found, treat it as a fatal error. */
	os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 17);	/* Fatal. */
      }
    }

  LOCK_SWAP();

#ifdef OSE_SMP
  /*
  ** If the block belongs to a detached cpu, refuse to create any process.
  */
  if(zzcpu[((struct _block*)pid_to_pcb(bid))->cpu_id].state == CPU_STATE_DETACHED)
    {
      return any_killed_pid();
    }
#endif

  /* Allocate a process control block and determine new process id. */
  pcb = &alloc_pcb(OSE_CREATE_PROCESS)->pcb;

  /* Check that we got the control block. */
  if(!pcb)
    {
      /* No, abort the system call. */
      return any_killed_pid();
    }

 /* Avoid a race condition with pathological designed pid */
  pcb->magic |= MAGIC_KILLED;
  UNLOCK_SWAP();
  new_pid = pcb_to_pid(pcb);

  /* Initialize process control block. */
  setup_pcb(pcb, name, tmo, priority, entrypoint, block, redir); /* patch */

  /*
  ** Save process attributes, so we for instance can distinguish the
  ** flavour of any send hook etc.
  */
  pcb->proc_attrs = proc_attrs;

  /* See if we should create a signal port. */
  if(proc_attrs & 0x0001)
    pcb->is_an_sport = (OSBOOLEAN) 1;

  /* See if we should create a deaf int/tim process. */
  if(proc_attrs & 0x0002)
    pcb->is_deaf = (OSBOOLEAN) 1;

  /* Install send hook. */
  pcb->snd_hook = snd_hook;
  pcb->lnh_dest_hd = lnh_dest_hd;
  pcb->lnh_sender_hd = NULL;
  
  /* Look for a new block process and note if found. */
  /* Note that pointer in the block descriptor is not set yet. */
  if(!strcmp((const char *)name, (const char *)block->name))
    {
      is_block_process = 1;
    }
  else
    {
      is_block_process = 0;
    }

  /* Set up a stack for non interrupt processes. */
  pcb->stack = stack;

  /*
  ** Store pid created in zzuu to permit system calls in create_handlers.
  ** Store the parent pid for the debugger.
  **
  ** Warning! this code breaks if we implement short phantom pcb's. ??
  */
  zzuu.process_under_construction_ = pcb_to_pid(pcb);
  pcb->parentpid = parent;

  /* Create a context. */
  if((proc_type == OS_BG_PROC) || (proc_type == OS_PRI_PROC))
    {
      pcb->uarea =
	zzcreate_context(pcb->stack->limit,
			 (OSADDRESS) ((char *) pcb->stack->top - (char *) pcb->stack->limit),
			 entrypoint,
			 pcb->block->supervisor_mode,
			 (OSBOOLEAN) 0,
			 (PROCESS) pcb_to_pid(pcb),
			 (PROCESS) parent,
			 (void *) pcb->block->clib_pointer
			 );
    }
  if((proc_type == OS_TI_PROC) || (proc_type == OS_INT_PROC))
    {
      pcb->uarea = zzcreate_context(NULL,
				    (OSADDRESS) 0,
				    entrypoint,
				    (OSBOOLEAN) 1,
				    (OSBOOLEAN) 1,
				    (PROCESS) pcb_to_pid(pcb),
				    (PROCESS) parent,
				    (void *) pcb->block->clib_pointer
				    );
    }

  zzuu.process_under_construction_ = (PROCESS) 0;
  
  /* Recalculate the interrupt stack. */
  if((proc_type == OS_TI_PROC) || (proc_type == OS_INT_PROC))
    {
      register int need_recalc = 0;

      /* See if worst case size on the selected priority changed. */
      if(proc_type == OS_TI_PROC)
	{
	  if(zzuu.wanted_timer_size < stack_size)
	    {
	      zzuu.wanted_timer_size = stack_size;
	      need_recalc = 1;
	    }
	}
      else
	{
	  if(zzuu.wanted_interrupt_size[pcb->priority & 31] < stack_size)
	    {
	      zzuu.wanted_interrupt_size[pcb->priority & 31] = stack_size;
	      need_recalc = 1;
	    }
	}

      /* If it did, check if the allocated interrupt stack is too small. */
      if(need_recalc)
	{
	  register OSADDRESS *p;
	  register int i;
	  register OSADDRESS new_int_size =
	    zzuu.wanted_timer_size + MIN_INTERRUPT_STACKSIZE;
	  for(p = &zzuu.wanted_interrupt_size[0], i = 32; i; i--, p++)
	    {
	      new_int_size += *p;
	    }

	  /* If too small, allocate the closest suitable size. */
	  if(zzuu.interrupt_stack_size < new_int_size)
	    {
	      struct stkcb *interrupt_stack;	/* Base of old int stack. */

	      /*
	      ** Align the stack size to a multiple of 32 bytes.
	      ** Just for the test, stack_alloc() does it too.
	      */
	      new_int_size = (new_int_size +
		  (POOL_ALIGNMENT-1)) /*& (OSADDRESS) ~(POOL_ALIGNMENT-1)*/;

	      /* See if the system pool has a stack large enough. */
	      if(new_int_size > system_pool.stack_list[7].size)
		  {
		  /* If it hasn't then report error... */
		  os_error(1, OSE_EINTSTACK_TOO_LARGE | OSE_CREATE_PROCESS,
		      (OSERRCODE) new_int_size);	/* Non fatal. */

		  /* and trim to the largest available stack. */
		  new_int_size = system_pool.stack_list[7].size;
		  }
	      
	      /* Don't loose the old interrupt stack. */
	      interrupt_stack = zzww.interrupt_stack;

	      /* Allocate a new interrupt stack for the current SMP cpu. */
	      LOCK_SWAP();
	      zzww.interrupt_stack = stack_alloc(&system_pool, new_int_size);

	      /*
	      ** Calculate actual size instead of requested to avoid
	      ** frequent reallocations.
	      */
	      zzuu.interrupt_stack_size = 
		zzww.interrupt_stack->buffer_head->size;
	      UNLOCK_SWAP();

	      /*
	      ** Fill the stack with end marks.
	      */
	      memset(((char *) zzww.interrupt_stack->top) - zzuu.interrupt_stack_size,
		     END_MARK,
		     ERROR_CHECKS ? (size_t) zzuu.interrupt_stack_size : (size_t) 4);

	      /* Make the stack limit known to get_pcb() etc. */
	      zzww.interrupt_stack->limit = (char *)zzww.interrupt_stack->top
		- zzuu.interrupt_stack_size;

	      /* Make new stack known to the target interface. */
	      zzset_interrupt_stack((OSADDRESS) zzww.interrupt_stack->limit,
				    zzuu.interrupt_stack_size);

	      /* Free the old interrupt stack if we had one. */
	      LOCK_SWAP();
	      if(interrupt_stack)
		stack_free(interrupt_stack);
	      UNLOCK_SWAP();
#ifdef OSE_SMP
	      /*
	      ** Install an interrupt stacks of same size on all other cpus.
	      */
	      {
		static const SIGSELECT sel_alloc_int_stack[] =
		  {1, SMP_ALLOC_INT_STACK};
		cpuid_t num_cpus = zzkrn_num_cpus();
		cpuid_t cpu;
		union SIGNAL *sig = alloc(sizeof(struct smp_alloc_int_stack),
					  SMP_ALLOC_INT_STACK);
		for(cpu = 0; cpu < num_cpus; cpu++)
		  {
		    /* Send order to all other cpus, but the current one. */
		    if(cpu != zzkrn_cpu_id() &&
		       zzcpu[cpu].state == CPU_STATE_ACTIVE)
		      {

			sig->smp_alloc_int_stack.stack_size =
			  zzuu.interrupt_stack_size;
			send(&sig, zzcpu[cpu].sysd_lite_);
			do
			  {
			    sig = receive_from(~0u, sel_alloc_int_stack,
					       zzcpu[cpu].sysd_lite_);
			  } while(sig == NIL);
		      }
		  } /* for */
		free_buf(&sig);
	      }
#endif	      
	    }
	}
    }

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Show post mortem debugger that the process is now available. */
  pcb->killed = (OSBOOLEAN) 0;
  pcb->magic = MAGIC_PCB;

  /* Kill the process again if the block was killed. */
  if(pcb->block->killed || pcb->block->kill_in_progress)
    {
      /* Set kill flag and bump instance number. */
      bump_pcb_instance(pcb);

      /* Kill the process context. */
      zzkill_context(pcb->uarea, new_pid);

      /* Set return value to indicate failure. */
      new_pid = any_killed_pid();

      /* Unlock the dispatcher. */
      UNLOCK_SWAP();

      /* Free the process control block. */
      free_pcb(pcb);

      /* The dispatcher is left locked. */
      LOCK_SWAP();

      /* Return the already killed new process id to the caller. */
      return new_pid;
    }

  /* Link the pcb into the list in the block descriptor. Bump modno. */
  pcb->proclist = block->proclist;
  block->proclist = pcb;
  block->pcb_list_modno++;

  /* Update the block process pointer in the block descriptor. */
  if(is_block_process)
    {
      /* Insert the new block process. */
      block->block_process = pcb;
    }

  /*
  ** If the block still has a parent, then update stop counter.
  **
  ** Protect the stop counter from start() issued by interrupt processes,
  ** even if this is a really odd case.
  ** Note that this counter can not wrap as the number of processes
  ** is limited to 64k.
  ** See also: stop()
  */
  if(pcb->st_cnt)
    {
      BKL_LOCK();
      if(block->parent)
	block->st_cnt++;
      BKL_UNLOCK();
    }

  /* Insert a background process into the background process list. */
  if(proc_type == OS_BG_PROC)
    {
#ifdef OSE_SMP
      {
	union SIGNAL *sig_insert;
	const SIGSELECT sel_smp_bg_list_reply[] = {1, SMP_PROC_ADD_REPLY};

	/* Need help by the local sysd_lite on the proper cpu. */
	UNLOCK_SWAP();
	sig_insert = alloc(sizeof(struct smp_move_proc),
			   SMP_PROC_ADD_REQUEST);
	sig_insert->smp_move_proc.pid = new_pid;
	sig_insert->smp_move_proc.cpu = pcb->cpu_id;
	sig_insert->smp_move_proc.request_tmo = 0;
	send(&sig_insert, zzcpu[pcb->cpu_id].sysd_lite_);
	sig_insert = receive(sel_smp_bg_list_reply);
	free_buf(&sig_insert);
	LOCK_SWAP();
      }
#else
      insert_pcb_in_bg_list(pcb);
#endif
    }

  /* Select a timer scheduler slot for a timer process. */
  /* Disable interrupts while entering the process into the scheduler list. */
  if(proc_type == OS_TI_PROC)
    {
      register U8 oldtmo;
      register U8 nexttmo;
      register U8 i;
      U32 safety_counter = 3;

    restart_scheduler_search:
      oldtmo = (U8) zzuu.ticks;
      nexttmo = oldtmo;

      /* Search for free slot. */
      i = (U8)tmo;
      do
	{
	  nexttmo++;
	  if(!zzuu.tmslist[nexttmo])
	    break;
	}
      while(--i);

      /* Insert timer process into scheduler slot. */
      BKL_LOCK();
      /* restart search if timer int occurred while searching. */
      if(oldtmo != (U8) zzuu.ticks)
	{
	  /* But drop it after a few retries if system is overloaded. */
	  if(safety_counter--)
	    {
	      BKL_UNLOCK();
	      goto restart_scheduler_search;
	    }
	  else
	    {
	      nexttmo = (U8)(zzuu.ticks + tmo);
	    }
	}
      pcb->cur_tmo = nexttmo;
      pcb->fq = zzuu.tmslist[nexttmo];
      zzuu.tmslist[nexttmo] = pcb;

      ++zzuu.timer_process_count;
      BKL_UNLOCK();
    }

  /* The dispatcher is left locked. */
  /* Return the new process id to the caller. */
  return new_pid;
}

/*
** This function can be called from within zzcreate_context() if the
** target interface needs to know the which segment the process belongs
** to.
*/
OSSEGMENT
zzcreate_context_segnum(PROCESS pid)
{
  return pid_to_pcb(pid)->segid;
}

/*
** Subfunction to the system daemon.
**
** It is called in response to a create_process() call issued
** by a client process. This call causes a new_process signal
** to be sent to the system daemon, which in turn calls this function.
*/
static void
sysd_new_process(union SIGNAL *sigr)
{
  PROCESS client = sender(&sigr);

  /* Clear error code in return signal. */
  sigr->new_process.ecode = (OSERRCODE) 0;

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the client is still alive. */
  if(get_ptype(client) == OS_ZOOMBIE)
    {
      /* Clean up and exit if not. */
      /* The name and redir buffers were reclaimed by the kernel. */
      /* Free the new process stack if there is one. */
      if(sigr->new_process.stack)
	stack_free(sigr->new_process.stack);
      /* Unlock the dispatcher. */
      UNLOCK_SWAP();
      /* Free the signal from the dead client. */
      free_buf(&sigr);
      /* Exit. */
      return;
    }

  /* Change ownership of the name and redir buffers. */
  change_sig_owner(sig_to_adm(sigr->new_process.name), zzcur_p);
  if(sigr->new_process.redir)
    change_sig_owner(sig_to_adm(sigr->new_process.redir), zzcur_p);

  /* Unlock the dispatcher. */
  UNLOCK_SWAP();

  /* Fill the stack with a pattern that can be used to calculate stack usage in the
  ** debugger. The lowest few bytes are also used for stack overflow checks.
  ** There is a bug here. The stack can not be reached without moving to the target segment. ??
  ** There is another bug too?? If there is a bus error while accessing the new stack, then
  ** the sysd may be damaged.
  ** We tried to fix the latter problem in create_pool()/extend_pool(), but
  ** the question must be revisited in a future level-D implementation.
  */
#if ERROR_CHECKS > 0
  if(sigr->new_process.stack)
    memset((char *) sigr->new_process.stack->limit,
	   END_MARK,
	   (size_t) ((char *)sigr->new_process.stack->top - (char *)sigr->new_process.stack->limit));
#else
  if(sigr->new_process.stack)
    memset((char *) sigr->new_process.stack->limit, END_MARK, (size_t) 4);
#endif

  /* Create the process. */
  sigr->new_process.pid =
    sysd_create_process(
			sigr->new_process.proc_type,
			sigr->new_process.name,
			sigr->new_process.entrypoint,
			sigr->new_process.stack,
			sigr->new_process.stack_size,
			sigr->new_process.priority,
			sigr->new_process.tmo,
			sigr->new_process.block,
			sigr->new_process.redir,
			(PROCESS) sender(&sigr),
			sigr->new_process.proc_attrs,
			sigr->new_process.snd_hook,
			sigr->new_process.lnh_dest_hd
			);

  /* Check if successful. Otherwise abort. */
  if(sigr->new_process.pid == any_killed_pid())
    {
      /* Free the stack. */
      if(sigr->new_process.stack)
	{
	  /* Already locked LOCK_SWAP(); */
	  stack_free(sigr->new_process.stack);
	}

      /* Unlock the dispatcher since sysd_create_process() left it locked. */
      UNLOCK_SWAP();

      /* Free the name buffer. */
      free_buf((union SIGNAL **) &(sigr->new_process.name));

      /* Free the redirection table. */
      if(sigr->new_process.redir)
	free_buf((union SIGNAL **) &(sigr->new_process.redir));
    }
  else
    {
      register struct _pcb *pcb = pid_to_pcb(sigr->new_process.pid);

      /* Allocate an interrupt vector if one is requested. */
      if(sigr->new_process.proc_type == OS_INT_PROC)
	{
	  if((sigr->new_process.vector & 0xffff) != 0xffff)	/* If vector is not -1. */
	    {
	      register OSPRIORITY new_pri;
	      pcb->vector = sigr->new_process.vector;

	      /*
	      ** Store pid created in zzuu to permit system calls in
	      ** int_create_handlers.
	      */
	      zzuu.process_under_construction_ = sigr->new_process.pid;

              {
                struct vectorcb *vcb = get_vectorcb(pcb->vector, 1);

                /* We don't have to check which cpu the vector is
                 * allocated to, because a different cpu would mean
                 * the vector is already in use, which is also an
                 * error. We still have to ensure that we insert the
                 * correct cpu id since the vector is now assigned. */
                vcb->cpus = (1u << pcb->cpu_id);
                
                /* The vector may not be in use. */
                if (vcb->isrcount + vcb->pcbcount > 0)
                  {
                    new_pri = 255;
                  }                    
                /* The target interface is responsible for
                   enabling the vector on the correct core. */
                else if (!zzbind_vector_pcb(pcb->vector, pcb))
                  {
                    new_pri = 255;
                  }
                else
                  {
                    /* Setup the vector entry. */
                    new_pri = pcb->priority & 31;
                    vcb->pcbcount = 1;
                    vcb->priority = new_pri;
                    
                    /* Check if we should enable the vector. */
                    if (vcb->disabled == 0)
                      {
                        SMP_CODE(UNLOCK_SWAP());  /* FIXME ?? */
                        zzenable_vector(pcb->vector, pcb->priority & 31);
                        SMP_CODE(LOCK_SWAP());  /* FIXME ?? */
                      }
                  }
              }              

	      zzuu.process_under_construction_ = (PROCESS) 0;

	      /*
	      ** Report error if vector already allocated.
	      ** Create a wakeup only process in this case.
	      */
	      if(new_pri == 255)
		{
		  sigr->new_process.ecode = OSE_EVECTOR_IN_USE | OSE_CREATE_PROCESS;
		  sigr->new_process.extra = (OSERRCODE) sigr->new_process.vector;	/* Non fatal. */
		  pcb->vector = (OSVECTOR) -1;
		}

	      /*
	      ** Call the vector bind handler if there is one.
	      ** This handler is allowed to enable the interrupt controller
	      ** for the specified vector and priority.
	      */
	      if(handlerp.vector_bind_handler)
		handlerp.vector_bind_handler((U32)pcb->vector, (U32) (pcb->priority & 31));

	      /*??
	      ** There is a bug here. We get a recursive wakeup call if an interrupt
	      ** process invokes a wakeup call to itself.
	      ** We can fix this in send() and signal_fsem() or in zzinit_int().
	      */
	    }
	}

      /* Unlock the dispatcher since sysd_create_process() left it locked. */
      UNLOCK_SWAP();

      /* Call the debugger hook and intercept the process if required. */
#if OSE_DEBUG > 0
      if(zzuu.sysprocs_created)
	{
	  if( zzcreate_dbghook(pcb_to_debug_pcb(pcb), pcb->user) )
	    {
	      /* Intercept the process if requested by the debugger. */
	      /* Interrupt processes and phantoms can not be intercepted. */
	      if(pcb->priority < 33)
		{
		  pcb->status = (pr_state_t) (pcb->status | ST_INTERCEPTED);
		}
	      else if(pcb->priority < 96)
	        {
		  /*EMPTY*/
		}
	      else if(pcb->priority < 128)
		{
		  pcb->status = (pr_state_t) (pcb->status | ST_INTERCEPTED);
		}
	    }
	}
#endif
  
      /*
      ** Ask the hunt daemon to scan the list of pending hunt signals and
      ** process those that match the name of the new process.
      */
      {
	union SIGNAL *sigh;
	sigh = alloc(sizeof(struct new_process), NEW_PROCESS);
	sigh->new_process.pid = sigr->new_process.pid;
        sigh->new_process.block = sigr->new_process.block;
	send(&sigh, zzuu.huntd_);
      }
    }


  /*
  ** Send a response containing the resulting process id and possibly
  ** an error code to the client.
  ** We use safe_send_w_s() and make sure we don't use the redirection
  ** table of the client. 
  */
  if (get_ptype(client) != OS_ZOOMBIE)
    {
      /* Send response back if the client is still alive. */
      safe_send_w_s(sig_to_adm(sigr), zzcur_p->pid, client, 1ul, NULL, NULL);
    }
  else
    {
      /* If the client is dead we might as well kill the new
       * process, since no one will start it. I.e. if we do not kill
       * it, it will become an orphan process.
       */
      kill_proc(sigr->new_process.pid);
      
      /* Since we do not send the signal back, we need to free it. */
      free_buf(&sigr);
    }
}

/*
** Subfunction to the sysd_delete_one_proc() function.
**
** Perform pool garbage collection in a specified pool.
*/

#if MEMORY_MODEL > 0

static void
collect_pool(struct _pcb *pcb, struct poolcb *pool)
{
  PROCESS pool_pid = pcb_to_pid(pool);
  register OSADDRESS sig_cnt_owned = pcb->sig_cnt_owned;
  register struct _signal *next;
  struct _signal *ssigadm;
  struct _signal *usigadm;
  register PCB_INDEX_T pindex = pcb_to_pindex(pcb);

  for(ssigadm = pool->siglist->siglist; ssigadm; ssigadm = next)
    {
      next = ssigadm->siglist;
      /*
      ** Check that the buffer looks right.
      ** Because of the memory model, we only check the administration
      ** block. Don't care about other errors since the process is
      ** dead anyway. In fact, it was probably killed becaused it has
      ** a damaged buffer...
      */
#if ERROR_CHECKS > 0
      if(unlikely(ssigadm->magic_signo != MAGIC_SIGNO ||
		  (/*(POOL_ALIGNMENT-1) &*/
		   ((OSADDRESS)next | (OSADDRESS)ssigadm->udata))))
	{
	  uint32_t *wp = (uint32_t *)ssigadm; 
	  int cnt = (int)(sizeof(*ssigadm) / sizeof(*wp));
	  int ix;

	  ramlog_printf("OSE_ECORRUPTED_KERNEL: Invalid signal control block:\n");
	  for(ix = 0; ix < cnt; ix += 8, wp += 8)
	    {
	      ramlog_printf("%#zx: %08x %08x %08x %08x %08x %08x %08x %08x\n",
			    wp, wp[0], wp[1], wp[2], wp[3],
			    wp[4], wp[5], wp[6], wp[7]);
	    } /* for */

	  /* We can not recover from a corrupted system pool. */
	  os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 11);	/* Fatal. */
	}
#endif
      if(unlikely(pindex == ssigadm->owner))
	{
	  /* Repair the user space sigadm, to avoid later errors. */
	  usigadm = -1 + (struct _signal *)ssigadm->udata;
#if ERROR_CHECKS > 0
	  /* Set the magic number in the user mode buffer. */
	  usigadm->magic_signo = MAGIC_SIGNO;
#endif
	  usigadm->udata = (SIGSELECT *) ssigadm;

	  /*
	  ** Free the buffer. We do this by just adding it to the free list
	  ** and dropping the ownership counters.
	  */
	  {
	    register struct pool_list_entry *hd;
	    register LOCKSTATE msr;

#if OSE_DEBUG > 0
	    if(handlerp.zzprof_free_buf)
	      handlerp.zzprof_free_buf(ssigadm->size, adm_to_sig(ssigadm),
				       pcb_to_debug_pcb(pcb),
				       pool_pid);
#endif
	    /* Locate free list. */
	    hd = ssigadm->buffer_head;

	    /*
	    ** By setting the dest_id field (e.g. the addressee field)
	    ** to the PID of the signal pool, we tell pool analyzers
	    ** that this signal buffer is now in the freelist. The sender
	    ** filed is set to indicate who used to own it.
	    */
	    ssigadm->dest_id = pool_pid;
	    ssigadm->sender_id = pcb->live_pid;  /* @frbr@ MUST BE live_pid */

	    /* Disable interrupts. */
	    SPIN_LOCK_SAVE(&hd->smp_spin, msr);

	    /* Make system the owner. */
	    ssigadm->owner = (PCB_INDEX_T) 0;

	    if(likely(hd->ext_handle == NULL))
	      {
		/* Link into free list. */
		ssigadm->next = (struct _signal *) hd->hd;
		hd->hd = ssigadm;

		/* Decrement no of signals allocated in pool. */
		hd->alloc_count--;
	      }
	    else
	      {
		/* Call external free function. */
		pool->ext_free(hd->ext_handle, adm_to_sig(ssigadm));

		/* Decrement no of signals allocated in pool. */
		(void)atomic_dec((atomic_t *)&hd->alloc_count);
	      }
	    
#ifdef OSE_SMP
	    /*
	    ** Decrement owned counter in owners (=killed) pcb. 
	    **
	    ** The process is dead. Noone else should steal buffers now.
	    ** Attach signals have been sent out synchronized.
	    ** Therefore, we can handle this from cpu0.
	    */
	    sig_cnt_owned = --pcb->sig_cnt_owned;

	    /* Enable interrupts. */
	    SPIN_LOCK_RESTORE(&hd->smp_spin, msr);
#else
	    /* Enable interrupts. */
	    SPIN_LOCK_RESTORE(&hd->smp_spin, msr);

	    /* Decrement owned counter in owners (=killed) pcb. */
	    sig_cnt_owned--;
#endif
	    /* Done if the killed process has no more signals. */
	    if(!sig_cnt_owned)
	      break;
	  }
	}
    }
#ifndef OSE_SMP
  pcb->sig_cnt_owned = sig_cnt_owned;
#endif
}

#else

static void
collect_pool(struct _pcb *pcb, struct poolcb *pool)
{
  PROCESS pool_pid = pcb_to_pid(pool);
  register OSADDRESS sig_cnt_owned = pcb->sig_cnt_owned;
  register struct _signal *next;
  struct _signal *sig;
  register PCB_INDEX_T pindex = pcb_to_pindex(pcb);
  for(sig = pool->siglist->siglist; sig; sig = next)
    {
      next = sig->siglist;
      if(pindex == sig->owner)
	{
	  /*
	  ** Check that the buffer looks right.
	  ** Must do this to avoid getting caught in check_buffer().
	  */
#if ERROR_CHECKS > 0
	  if(sig->magic_signo != MAGIC_SIGNO)
	    {
	      /* Try to locate the buffer that caused the failure. */
	      zzrecover_from_pool_crash(sig, OSE_KILL_PROC);
	      /* We can not recover from a corrupted pool at this point. */
	      os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 11);	/* Fatal. */
	    }
	  /*
	  ** We might also check that sender, addressee etc are reasonable.
	  ** We won't do that because we expect linear overwrite errors.
	  ** We assume that if the highest and lowest few bytes are OK,
	  ** then the rest is probably OK too.
	  ** See also:  check_buffer().
	  */
	  /* Check buffer end mark. */
	  if(END_MARK != *(U8 *)(sig->size + (U8 *) adm_to_sig(sig)))
	    {
	      /* Try to locate the buffer that caused the failure. */
	      zzrecover_from_pool_crash(sig, OSE_KILL_PROC);
	      /* We can not recover from a corrupted pool at this point. */
	      os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 12);	/* Fatal. */
	    }
#endif
	  /*
	  ** Free the buffer. We do this by just adding it to the free list
	  ** and dropping the ownership counters.
	  */
	  {
	    register struct pool_list_entry *hd;

#if OSE_DEBUG > 0
	    if(handlerp.zzprof_free_buf)
	      handlerp.zzprof_free_buf(sig->size, adm_to_sig(sig),
				       pcb_to_debug_pcb(pcb),
				       pool_pid);
#endif
	    /* Locate free list. */
	    hd = sig->buffer_head;

	    /*
	    ** By setting the dest_id field (e.g. the addressee field)
	    ** to the PID of the signal pool, we tell pool analyzers
	    ** that this signal buffer is now in the free list. The sender
	    ** filed is set to indicate who used to own it.
	    */
	    sig->dest_id = pool_pid;
	    sig->sender_id = pcb->live_pid;  /* @frbr@ MUST BE live_pid */

	    /* Disable interrupts. */
	    BKL_LOCK();

	    /* Make system the owner. */
	    sig->owner = (PCB_INDEX_T) 0;

	    /* Link into free list. */
	    sig->next = (struct _signal *) hd->hd;
	    hd->hd = sig;

	    /* Decrement no of signals allocated in pool. */
	    hd->alloc_count--;
	    
	    /* Enable interrupts. */
	    BKL_UNLOCK();

	    /* Decrement owned counter in owners (=killed) pcb. */
	    sig_cnt_owned--;

	    /* Done if the killed process has no more signals. */
	    if(!sig_cnt_owned)
	      break;
	  }
	}
    }
  pcb->sig_cnt_owned = sig_cnt_owned;
}

#endif

/*
** Subfunction to the sysd_delete_process() function.
**
** Called from sysd to kill one process.
**
** Retuns a non zero value if no processes where killed because it
** was "kill locked".
*/

static unsigned
sysd_delete_one_proc(struct _pcb *pcb, PROCESS pid)
{
  register struct _block *bcb;
  register struct _pcb *tpcb;
  union SIGNAL *sig;

  /* Disable interrupts. */
  BKL_LOCK();

  /* Check if the process is "kill locked".... */
  if(pcb->magic == MAGIC_PCB &&
     pcb_to_instance(pcb) == pid_to_instance(pcb->pid) &&
     pcb->lock_cnt != 0)
    {
      BKL_UNLOCK();
      /* ...yes it is. */
      return 1;
    }

  /* Reenable interrupts. */
  BKL_UNLOCK();

  /*
  ** Make sure that the process we want to delete is not currently in
  ** possession of it's CPU.  If it is we just have to wait until it
  ** is not. Trying to delete someone from who's currently running
  ** (and using the stack) is no good, trust me, I've tried...
  */
  SMP_CODE(while(pcb->alloced_cpu) { ; /* Spin! */ });

  /* Free all timeout signal control blocks of the process. */
  kill_timeout_signals(pcb);

  /* Pick up the block descriptor. */
  bcb = pcb->block;

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Decrement the segment visitor count for processes using set_segment(). */
  if(bcb->segid != pcb->segid)
    {
      struct segcb *seg;

      if(pcb->segid == 0)
	{
	  /*
	  ** The pcb process was visiting the system segment.
	  ** (Segid zero is the same as the system segment.)
	  */
	  system_segment.visitors--;

	  /*
	  ** We never need to call delete_segment() since the system
	  ** segment can never die.
	  */
	}
      else
	{
	  for(seg = zzuu.segment_list; ; seg = seg->next)
	    {
	      if(!seg)
		{
		  /* Segment not found. */
		  break;
		}
	    
	      if(seg->segid == pcb->segid)
		{
		  seg->visitors--;

		  /* If the segment has no more references, then delete it. */
		  if(!(seg->block_count || seg->pool_count || seg->visitors))
		    {
		      delete_segment(seg);
		    }
		  break;
		}
	    }
	}
    }
  

  /* Remove a process from the per block process list. */
  if(bcb->proclist == pcb)
    bcb->proclist = pcb->proclist;
  else
    for(tpcb = bcb->proclist; tpcb->proclist; tpcb = tpcb->proclist)
      if(tpcb->proclist == pcb)
	{
	  tpcb->proclist = pcb->proclist;
	  break;
	}

  /*
  ** If the killed process was a block process then
  ** remove the link from the block descriptor to the killed process.
  */
  if(bcb->block_process == pcb)
    bcb->block_process = NULL;

  /* Unlock the dispatcher. */
  UNLOCK_SWAP();

  /* Remove all ISRs that uses this process as owner. */
  process_isr_attach_list(pcb->isr_list);
  
  /*
  ** return all signals attached to the process/block and
  ** kill those attached by the process.
  ** Must do this before pool garbage collection since attached signals
  ** would otherwise be freed by the garbage collector.
  */
  kill_attach(pid);

  /*
  ** Free the s-nodelist allocated by processes who have been receiving
  ** signals through a signal port.
  ** Must do this before pool garbage collection since attached signals
  ** would otherwise be freed by the garbage collector.
  */
  if(pcb->s_pcb.s_nodelist)
    {
      sys_free_buf((union SIGNAL **) &pcb->s_pcb.s_nodelist);
    }
  pcb->s_pcb.s_nodelist = NULL;

  /*
  ** Call the debugger hook.
  ** Must do this after the return of attached signals since the name is needed
  ** by the debugger during signal transmission.
  */
#if OSE_DEBUG > 0
  zzkill_dbghook(pcb_to_debug_pcb(pcb));
#endif

  /* Scan the local pool for signals owned by the killed process. */
  if(pcb->sig_cnt_owned)
    {
      collect_pool(pcb, pcb->block->sigpool);
    }

  /* Also scan the system pool if there are still signals left. */
  if(pcb->sig_cnt_owned && pcb->block->sigpool != &system_pool)
    collect_pool(pcb, &system_pool);

  /*
  ** Finally scan all other pools. This is real heavy work, but there
  ** seem to be no other way out.
  ** Do it on a very low priority?
  ** Should there be a background supervision process??
  ** Maybe there should be a "check_system" system call that
  ** a user can call periodically?
  */
  if(pcb->sig_cnt_owned)
    {
      struct poolcb *pool;
      for(pool = zzuu.pool_list; pool; pool = pool->next)
	{
	  if(!pcb->sig_cnt_owned)
	    break;

	  if( (pool != &system_pool) && (pool != pcb->block->sigpool) )
	    collect_pool(pcb, pool);
	}
    }

  /* Report an error if some signals can not be freed. This is really a kernel error. */
  if(pcb->sig_cnt_owned)
    {
      /* What should we do if we can not free all owned signals??
      ** The current action is to report a fatal error, but the real reason is
      ** probably a kernel bug, unless a pool was truncated. We should be able
      ** to kill a damaged program. Maybe skip collect_pool for damaged pools?
      */
#ifdef OSE_SMP
#ifdef GUNNAR_DEBUG				/* FIXME ?? */
      {
	extern Boolean error_state;
	error_state = True;

	/* How catch this problem?? */
	LOCK();
	ramlog_printf("pose: Collect_pool %s pid %#x @cpu %d.\n",
		      pcb->name, pcb->live_pid, pcb->cpu_id);
	zzkrn_evt_dump_queue(pcb->cpu_id);
	UNLOCK();
      }
#endif
#endif
      os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 0x10000 +
	       pcb->sig_cnt_owned);	/* Fatal. */
    }

  /* Delete the process context in the target interface. */
  if(pcb->uarea)
    zzkill_context(pcb->uarea, pid);

  /*
  ** Free all environment variables.
  ** These buffers are all owned by the system (pindex 0).
  */
  kill_proc_cancel_env(pcb);

  /* Free all signals in the mailbox. */
  {
    register struct _signal *next;
    struct _signal *sigadm;
    for(sigadm = pcb->mailbox; sigadm; sigadm = next)
      {
	next = sigadm->next;
	/*
	** Check that the buffer looks right.
	** We only check the administration block, since we can not handle
	** overwrite errors in other pools gracefully at this point.
	*/
#if ERROR_CHECKS > 0
	if(sigadm->magic_signo != MAGIC_SIGNO)
	  {
	    /* Try to locate the buffer that caused the failure or simply kill the system. */
#if MEMORY_MODEL > 0
	    os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 10);	/* Fatal. */
#else
	    zzrecover_from_pool_crash(sigadm, OSE_KILL_PROC); /* @frbr@ Change to os_error? */
	    os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 10);	/* Fatal. */
#endif
	  }
#endif
	/*
	** Free the buffer. We do this by just adding it to the free list
	** and dropping the ownership counters.
	*/
	{
	  register struct pool_list_entry *hd = sigadm->buffer_head;
	  struct poolcb *pool = hd->poolcbp;

#if OSE_DEBUG > 0
	  if(handlerp.zzprof_free_buf)
	    handlerp.zzprof_free_buf(sigadm->size, adm_to_sig(sigadm),
				     pcb_to_debug_pcb(pcb),
				     pcb_to_pid(pool));
#endif
	  /*
	  ** By setting the dest_id field (e.g. the addressee field)
	  ** to the PID of the signal pool, we tell pool analyzers
	  ** that this signal buffer is now in the free list. 
	  ** We don't change the sender field here, thus it will look
	  ** like "the real sender" of the signal just freed it. I.e. it
	  ** will look like the sender sent it after the addressee died.
	  */
	  sigadm->dest_id = pcb_to_pid(pool);

	  /* Link into free list. */
	  link_free_buf(pool, hd, sigadm, NULL);
	}
      }
  }

  /* Free any redirection table. */
  if(pcb->redir)
    sys_free_buf((union SIGNAL **) &pcb->redir);

  /* Free the stack occupied by the process. */
  if(pcb->stack)
    {
      LOCK_SWAP();
      stack_free(pcb->stack);
      UNLOCK_SWAP();
    }

  /*
  ** If a high priority process was killed, notify the MMU if
  ** the address space need no longer be resident.??
  */

  /* Send delete_process signals to the hunt daemon. */
  sig = alloc(sizeof(struct delete_process), DELETE_PROCESS);
  sig->delete_process.pid = pid;
  send(&sig, zzuu.huntd_);

  /*
  ** Kill all blocks for which the killed process is a remote call server.
  ** Kill all owned blocks (blocks partially created by the killed proc).
  */
  for(bcb = zzuu.bcb_list; bcb; bcb = bcb->next)
    {
      register PROCESS ppid = pcb_to_pid(bcb);
      if(bcb->use_remcalls && bcb->remcall_server == pid)
	{
	  if(!bcb->killed)
	    kill_proc(ppid);
	}
      if(bcb->parent == pcb && !bcb->killed)
	{
	    kill_proc(ppid);
	}
    }

  return 0;
}

/*
** Subfunction to the sysd_delete_process() function.
**
** The sysd kills an entire block.
*/

static void
sysd_delete_block(struct _block *bcb, PROCESS pid)
{
  OSADDRESS locked_procs = 0;
  register struct _pcb *pcb;
  register struct _pcb *tpcb;
  register struct _block *tbcb;
  struct segcb *segcb;

  /* Disable interrupts. */
  BKL_LOCK();

  /*
  ** Make sure we only delete this block once. If we have kill locked
  ** procs in the block sysd might (due to race conditions) get
  ** several delete block requests. It's importante that we only carry
  ** out the first one.
  */

  if(pcb_to_pid(bcb) != pid &&               /* Still alive. */
     pcb_to_pid(bcb) != next_pid_value(pid)) /* Killed but not deleted. */
    {
      BKL_UNLOCK();
      return;
    }

  /* Reenable interrupts. */
  BKL_UNLOCK();
  
  /* Kill each process in the block. */
  for(pcb = bcb->proclist; pcb; pcb = tpcb)
    {
      /* Save the next pcb pointer since we will kill the current. */
      tpcb = pcb->proclist;

      locked_procs += sysd_delete_one_proc(pcb, pcb->live_pid); /* @frbr@ MUST BE live_pid */
    }

  /*
  ** Since there is at least one "kill locked" processes left in the
  ** block we return without actually killing the block now. It will
  ** die eventually when the last locked process in the block unlocks
  ** itsef. See: zzkrn_kill_unlock().
  */
  if(locked_procs != 0)
    {
      return;
    }

  /*
  ** Bump the instance number of the block (one more than usual) to
  ** make sure that we don't service any more delete block requests on
  ** this block.
  */
  BKL_LOCK();
  bump_pcb_instance((struct _pcb *)bcb);
  BKL_UNLOCK();

  /* Kill the segment if the main block died. */
  if(bcb->is_main_block && bcb->segpid)
    {
      struct kill_seg_req kill_seg;

      kill_seg.sig_no = KILL_SEG_REQ;
      kill_seg.segid = bcb->segpid;
      sysd_do_kill_seg_req((union SIGNAL *)&kill_seg);
    }

  /* Remove all ISRs that uses this block as owner. */
  process_isr_attach_list(bcb->isr_list);
  
  /* Return all signals attached to the block. */
  kill_attach(pid);

  /* Remove the block descriptor from the system block list. */
  /* zzuu.bcb_list is changed here!!!! Race conditions??*/
  /* Must lock all processes except sysd when traversing this list. */
  LOCK_SWAP();
  if(bcb == (tbcb = zzuu.bcb_list))
    {
      zzuu.bcb_list = bcb->next;
    }
  else
    {
      for(; tbcb; tbcb = tbcb->next)
	{
	  if(tbcb->next == bcb)
	    {
	      tbcb->next = bcb->next;
	      break;
	    }
	}
    }
  UNLOCK_SWAP();

  /* Free all environment variables. */
  kill_proc_cancel_env((struct _pcb *) bcb);

  /* Free the name buffer. */
  sys_free_buf_type(const char *, &bcb->name);

  /* Free linkhandler name if assigned. */
  if(bcb->lnh_name)
    {
      sys_free_buf_type(const char *, &bcb->lnh_name);
    }
  
  /*
  ** Free the state hook list if present.
  ** No lock needed, as the block is dead, and it can not die while calling
  ** the hooks.
  */
  if(bcb->statehook_list)
    {
      struct statehookcb *next;
      struct statehookcb *hookcb;

      for(hookcb = bcb->statehook_list; hookcb; hookcb = next)
	{
	  next = hookcb->next;
	  hookcb->magic++;
	  sys_free_buf_type(struct statehookcb *, &hookcb);
	}
    }
  
  /* Locate the program segment of the block. */
  LOCK_SWAP();
  if(bcb->segpid)
    {
     segcb = (struct segcb *) pid_to_pcb(bcb->segpid);
    }
  else if(bcb->segid == 0)
    {
      /* Segid zero is the same as the system segment. */
      segcb = &system_segment;
    }
  else
    {
      for(segcb = zzuu.segment_list; segcb; segcb = segcb->next)
	{
	  if(segcb->segid == bcb->segid)
	    {
	      break;
	    }
	}
    }
  UNLOCK_SWAP();

  /* Decrement the block reference counter in the segment control block. */
  BKL_LOCK();
  segcb->block_count--;
  BKL_UNLOCK();

  /* Store the last killed block in the segment control block. */
  segcb->last_bid = pid;

  /*
  ** Free all semaphores and mutexes in the segment if all blocks in the
  ** segment are dead.
  ** (Error if any process is still waiting at the semaphore.)
  */
  cond_kill_sem(segcb);
  cond_kill_mutex(segcb);

  /* Free the program segment if required. */
  /* If no more blocks or pools or visitors then delete the segment. */
  if(!(segcb->block_count || segcb->pool_count || segcb->visitors))
    {
      /* Lock the dispatcher. */
      LOCK_SWAP();

      delete_segment(segcb);

      /* Unlock the dispatcher. */
      UNLOCK_SWAP();
    }

  /* Decrement reference counts for stack and signal pools. */
  BKL_LOCK();
  if(!(--bcb->stackpool->no_of_stack_users))
    zzuu.garbage_collection_enabled = (OSBOOLEAN) 1;
  if(!(--bcb->sigpool->no_of_sig_users))
    zzuu.garbage_collection_enabled = (OSBOOLEAN) 1;
  BKL_UNLOCK();

  /* Free the stack and signal pools if required. */
  if(zzuu.garbage_collection_enabled)
    zzuu.garbage_collection_enabled = sysd_check_free_pool();

  /* Send delete_process signal to the hunt daemon. */
  /* The hunt daemon will free the pcb. */
    {
      union SIGNAL *sig;  
  
      sig = alloc(sizeof(struct delete_process), DELETE_PROCESS);
      sig->delete_process.pid = pid;
      send(&sig, zzuu.huntd_);
    }
}

/*
** Subfunction to the system daemon.
**
** It is called in response to a kill_proc() call issued
** by a client process. This call causes a delete_process signal
** to be sent to the system daemon, which in turn calls this function.
*/

static void
sysd_delete_process(PROCESS pid)
{
  register struct _pcb *pcb;

  assert(pcb_to_pid(zzcur_p) == zzuu.sysd_);

  /* Locate the block or process descriptor. */
  pcb = pid_to_pcb(pid);

  /* If a block was specified, then kill it. */
  if((pcb->magic & (~MAGIC_KILLED)) == MAGIC_BCB)
    sysd_delete_block((struct _block *) pcb, pid);

  /* If a single process was specified, then kill it. */
  else
    sysd_delete_one_proc(pcb, pid);

}

/*
** Subfunction to the kill_proc() system call.
**
** Kills one process if it's not "kill locked".
**
** Retuns zero if the processes was not killed because it is was "kill
** locked". Returns the value 1 if the process was successfully killed.
**
** The dispatcher is locked while this function is called.
**
** In an SMP environment, this function is always called on the cpu of
** the victim, either by a client process, sysd or proper sysd_lite.
*/

static unsigned
kill_one_proc(struct _pcb *pcb)
{
  register OSPRIORITY priority;

  /* Disable interrupts. */
  BKL_LOCK();

  /* Check if the the process has been kill locked. */
  if(pcb->lock_cnt != 0)
    {
      /*
      ** The killing will have to be deferred to just after the process
      ** has been unlocked.
      */
      pcb->locked_kill = (OSBOOLEAN)1;
      
      /* Reenable interrupts. */
      BKL_UNLOCK();

      /* We can not kill a locked process. */
      return 0;
    }

  /* Reenable interrupts. */
  BKL_UNLOCK();

  /* Read process priority. */
  priority = pcb->priority;

  /* Extract process type from process priority. */
  /* Is it a non interrupt process? */
  if(priority < 33)
    {
      /*
      ** Remove a background process from the background list
      ** (of the currently running SMP cpu).
      */
      if(pcb->priority == 32)
	{
	  LOCK();
	  if(pcb->ready_entry.rlist)
	    unlink_pcb_from_bg_list(pcb);
	  UNLOCK();
	}

      /* Disable interrupts. */
      BKL_LOCK();

      /* Check if the process have touched the signal port system. */
      if(pcb->s_pcb.s_nodelist)
	/* Remove a non int process from any signal port receive chains. */
	kill_proc_abort_sport(pcb);

      /* Remove a non int process from any semaphore queue. */
      if(pcb->status & ST_SEMAPHORE)
	kill_proc_cancel_sem(pcb);

      /* Make reasonably sure this process won't be swapped in again. */
      pcb->status = (pr_state_t) (pcb->status | ST_STOPPED);
      pcb->st_cnt += 0xdead;

      /* Make a non int process not ready and remove it from the ready list. */
      if(pcb == zzcur_p)
	{
	  make_mypri_not_ready(pcb);
	}
      else
	{
	  make_otherpri_not_ready(pcb);
	}

      /* Is a timeout requested? */
      if(pcb->tmo_active)
	/* Cancel any pending timeout for a non interrupt process. */
	cancel_other_timeout(pcb);
    }

  /* Is it an interrupt process? */
  else if(priority < 96)
    {
      /*
      ** Call the vector release handler if there is one.
      ** This handler is assumed to disable the interrupt controller
      ** for the specified vector.
      */
      if((pcb->vector & 0xffff) != 0xffff)	/* If vector is not -1. */
	{
	  BKL_LOCK();
	  if(handlerp.vector_release_handler)
	    handlerp.vector_release_handler((U32)pcb->vector);
	  BKL_UNLOCK();

          {
            /* Fetch the vector control block if it exists. */
            struct vectorcb *vcb = get_vectorcb(pcb->vector, 0);
            if (vcb != NULL)
              {
                /* Check if we should disable the vector. In the SMP
                 * case we are called from sysd lite, so we are on the
                 * correct core. */
                if (vcb->disabled == 0)
                  {
                    zzdisable_vector(pcb->vector);
                  }

                /* Remove the interrupt process. */
                zzrelease_vector_pcb(pcb->vector, pcb);
                vcb->pcbcount = 0;
                clear_vectorcb_hooks(vcb);
              }
          }           
	}

      /* Disable interrupts. */
      BKL_LOCK();
    }

  /* Is it a timer process? */
  else if(priority < 128)
    {
      /* Disable interrupts. */
      BKL_LOCK();

      /*
      ** Cancel any pending wakeup calls for stopped timer processes?
      ** Wakeup calls are not kept pending at all for stopped processes.
      */

      /* Remove a timer process from the timer list. */
      --zzuu.timer_process_count;
      if(pcb == zzuu.tmslist[pcb->cur_tmo])
	{
	  zzuu.tmslist[pcb->cur_tmo] = pcb->fq;
	}
      else
	{
	  register struct _pcb *ti_pcb;
	  for(ti_pcb = zzuu.tmslist[pcb->cur_tmo]; ti_pcb; ti_pcb = ti_pcb->fq)
	    {
	      if(pcb == ti_pcb->fq)
		{
		  ti_pcb->fq = pcb->fq;
		  break;
		}
	    }
	}
    }
  /* Is it a phantom process? */
  else
    {
      /* Disable interrupts. */
      BKL_LOCK();
      /* Is the phantom a signal port? */
      if(pcb->is_an_sport)
	{
	  struct S_node *s_node =
	    pcb->s_pcb.s_nodelist;
	  pcb->s_pcb.s_nodelist = NULL;
	  /* Unlink all processes waiting at the signal port. */
	  while(s_node)
	    {
	      struct _pcb *cli_pcb;
	      /*
	      ** Set the phantom_pcb field in all clients to NULL to
	      ** indicate that an sport died during the receive_sport() call.
	      */
	      s_node->phantom_pid = pcb_to_pid(pcb); /* The killed phantom. */
	      s_node->phantom_pcb = NULL;
	      /* Wake up the client so that it can return an OS_ATTACH_SIG. */
	      cli_pcb = s_node->base->my_pcb;
	      s_node = s_node->fp_s_nodelist;
	      /* We know that the target process is a prioritized or background process.
	      ** If it is in receive then make it ready for subsequent swap.
	      */
	      if(cli_pcb->status & ST_RECEIVE)
		{
		  cli_pcb->status = (pr_state_t) (cli_pcb->status & (~ST_RECEIVE_FSEM));
		  
		  /* If the process becomes ready then do a swap. */
		  if(cli_pcb->status < ST_READY)
		    {
		      make_otherpri_ready_noswap(cli_pcb); /* noswap */
		    }
		}

	    } /* end while. */
	} /* end if(pcb->is_an_sport) */
    }

  /*
  ** Deregister the timer manager if it was killed.
  ** Wakeup priority for timer processes is set to highest possible.
  */
  if(pcb == zzuu.timer_manager)
    {
      zzuu.timer_manager = NULL;
      zzuu.timer_int_priority = (OSPRIORITY) 0;
    }

  /* Set the killed flag and bump the instance number. */
  bump_pcb_instance(pcb);

  /*
  ** We also change the type of the pcb so that check_pid() et. al understands
  ** and refuses pathological designed pids.
  */
  pcb->magic |= MAGIC_KILLED;

  /* Reenable interrupts. */
  BKL_UNLOCK();

  /* The process was not locked and thus successfully killed. */
  return 1;
}

/*
** Subfunction to the kill_proc() system call.
**
** Kills an entire block.
**
** Note: if there is any "kill locked" processes in the block they
** will not be killed and they will also force us to keep the block.
**
** Returns a non zero value if no processes where killed because they
** where all "kill locked".
**
** The dispatcher is locked while this function is called.
**
** There is a problem here.
** We should manage mixed user blocks in kill_proc??
** We should kill the entire block if the bcb is in the callers
** user space.
** We should kill only processes visible to the caller if the
** bcb is of another user id.
** How do we tell the system daemon which processes are dead?
*/

#ifndef OSE_SMP
static OSBOOLEAN
kill_block(struct _block *bcb)
{
  register struct _pcb *pcb;
  OSADDRESS killed_procs = 0;
  OSADDRESS locked_procs = 0;

  /* Kill each process in the block. */
  for(pcb = bcb->proclist; pcb; pcb = pcb->proclist)
    {
      /* Handle only processes that are still alive. */
      if(!pcb->killed)
	{
	  if (kill_one_proc(pcb))
	    killed_procs++;
	  else
	    {
	      int wasnt_stopped = 0;
	      
	      locked_procs++;

	      /* Resume and start it if needed to make it die. */
	      if(pcb->status & ST_INTERCEPTED)
		start_one_proc(pcb, &wasnt_stopped, ST_INTERCEPTED);
	      if(pcb->st_cnt)
		{
		  pcb->st_cnt = 1;
		  start_one_proc(pcb, &wasnt_stopped, ST_STOPPED);
		}
	    }
	}
    }

  /*
  ** Since there is at least one "kill locked" processes left in the
  ** block we return without actually killing the block now. It will
  ** die eventually when the last locked process in the block unlocks
  ** itsef. See: zzkrn_kill_unlock().
  */
  if(locked_procs != 0)
    {
      return (killed_procs != 0)
	? (OSBOOLEAN) 0  /* We killed at least one process. */
	: (OSBOOLEAN) 1; /* All processes where "kill locked". */
    }
  
  /* Set the killed flag and bump the instance number. */
  BKL_LOCK();
  bump_pcb_instance((struct _pcb *)bcb);

  /*
  ** We also change the type of the pcb so that check_pid() et. al understands
  ** and refuses pathological designed pids.
  */
  bcb->magic |= MAGIC_KILLED;
  BKL_UNLOCK();

  /* All processes as well as the block is now officially gone. */
  return (OSBOOLEAN) 0;
}
#endif

/*
**===========================================================================
** 11.3  System calls.
**===========================================================================
*/

/*
** Return the process id of the calling process.
*/

PROCESS
current_process(void)
{
  return zzcur_p->pid;
}

#ifdef OSE_SMP
static PROCESS
current_process_usr(void)
{
  return zzcur_p_usr->pid;
}
#endif

/*
** This is the create_process system call.
**
** The user parameter is currently incorrectly handled. This is an error??
*/

PROCESS
create_process(enum PROCESS_TYPE proc_type,
	       const char *name,
	       OSENTRYPOINT *entrypoint,
	       OSADDRESS stack_size,
	       OSPRIORITY priority,
	       OSTIME tmo,
	       PROCESS block,
	       struct OS_redir_entry *redir,
	       OSVECTOR vector,
	       OSUSER user)
{
  struct _block *bcb;
  struct new_process *sig;
  PROCESS newpid = any_killed_pid();
  
#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_CREATE_PROCESS))
    return (PROCESS) newpid;
#endif

#if ERROR_CHECKS > 0
  /* Check that the priority parameter has a legal value. */
  if(priority > 31)
    {
      os_error(0, OSE_EBAD_PARAMETER | OSE_CREATE_PROCESS, (OSERRCODE) priority);	/* Non fatal. */
      return (PROCESS) newpid;
    }
#else
  priority &= 31;
#endif

  /* Check and possibly adjust the given tmo value. */
  switch(proc_type)
    {
    case OS_TI_PROC:
      /* Error report an illegal timeout parameter for a timer process. */
      if(tmo > 256)
	{
	  os_error(0, OSE_EBAD_PARAMETER | OSE_CREATE_PROCESS, (OSERRCODE) tmo); /* Non fatal. */
	  return any_killed_pid();
	}
      break;
      
    case OS_BG_PROC:
      /*
      ** Zero or a  too big or background timeslice is replaced by the
      ** system default timeslice.
      */
      if(!tmo)
	tmo = local_sysctlblk.bgd_timeslice;
      if(tmo > local_sysctlblk.bgd_timeslice)
	tmo = local_sysctlblk.bgd_timeslice;
      break;

    default:
      /*
      ** OS_PRI_PROC, OS_INT_PROC, OS_PHANTOM:
      ** Replace OSE_NO_TIMEOUT by a zero timeslice.
      */
      if(OSE_NO_TIMEOUT == tmo)
	tmo = 0;
      break;
    } /* switch */
  
#if ERROR_CHECKS > 0
  /* Check the process type. */
  switch(proc_type)
    {
    case OS_PRI_PROC:
    case OS_BG_PROC:
    case OS_INT_PROC:
    case OS_TI_PROC:
    case OS_PHANTOM:
    case OS_BLOCK:
	break;
    case OS_ZOOMBIE:
    case OS_ILLEGAL:
    default:
      {
        os_error(0, OSE_EBAD_PROCESS_TYPE | OSE_CREATE_PROCESS, proc_type); /* Non fatal. */
        return newpid;
      }
    }
#endif

  /* Check the block id provided or create a suitable default block id. */
  if(block)
    {
      if(!check_bid(block, OSE_CREATE_PROCESS))
	{
	  return newpid;
	}
    }
  else
    block = pcb_to_pid(zzcur_p->block);
  
#ifdef OSE_SMP
  /* If the block belongs to a detached cpu, refuse to create any process. */
  if(zzcpu[((struct _block*)pid_to_pcb(block))->cpu_id].state ==
     CPU_STATE_DETACHED)
    {
      /* The block id was most likely explicitely given by application here. */
      os_error(0, OSE_EILLEGAL_PROCESS_ID | OSE_CREATE_PROCESS, block); /* Non fatal. */
      return newpid;
    }
#endif

  /*
  ** Reject to create interrupt processes in programs with MAS mappings
  ** unless the 0x40 bit is set in "krn/relax_error_checks".
  */
  if(OS_INT_PROC == proc_type || OS_TI_PROC == proc_type)
    {
      LOCK_SWAP();
      bcb = (struct _block *)pid_to_pcb(block);
      if(pcb_to_pid(bcb) == block)
	{
	  struct segcb *segcb = bcb->segpid == 0 ?
	    &system_segment : (struct segcb *)(pid_to_pcb(bcb->segpid));
	  
	  if(segcb->mas_mapped)
	    {
	      UNLOCK_SWAP();
	      
	      if(0 == (0x40 & local_sysctlblk.relax_error_checks))
		{
		  /*
		  ** This program is not allowed to contain interrupt
		  ** processes.
		  */
		  os_error(0, OSE_CREATE_PROCESS | OSE_EBAD_PROCESS_TYPE, proc_type);	/* Non fatal. */
		  return newpid;
		}
	      else
		{
		  /*
		  ** This program is allowed to contain interrupt
		  ** processes if the system errorhandler says ok.
		  */
		  os_error(1, OSE_CREATE_PROCESS | OSE_EBAD_PROCESS_TYPE, proc_type); /* Non fatal system error. */
		}
	    }
	}
      if(zzdispatcher_locked)
	UNLOCK_SWAP();
    }
      
  /* Allocated a signal to be sent to sysd. */
  sig = (struct new_process *)
      sys_alloc(sizeof(struct new_process), NEW_PROCESS);

  /* Allocate a name buffer in the system pool and copy the name to it. */
  {
    size_t namelen = strlen(name);
    /* Make sure that the name can be fitted in the system pool. */
    if(system_pool.signal_list[7].size < (OSADDRESS) (sizeof(SIGSELECT) + namelen))
      {
	os_error(0, OSE_EBAD_PARAMETER | OSE_CREATE_PROCESS, (OSERRCODE) name);	/* Non fatal. */
	free_buf_type(struct new_process *, &sig);
	return any_killed_pid();
      }
    sig->name = (char *) sys_alloc((OSBUFSIZE) (sizeof(SIGSELECT) + namelen), (SIGSELECT) 0);
    memcpy(sig->name, name, namelen);
    sig->name[namelen] = (char) '\0';	/* String terminator just in case... */
  }

  /*
  ** Extract process attribute flags from the redirection table.
  ** These flags are present if the first word is zero.  Flags are
  ** used for instance to define a "signal port" phantom or a "deaf"
  ** timer or interrupt process (one that isn't woken up when sending
  ** signals to it).
  */
  sig->proc_attrs = 0;
  sig->snd_hook = NULL;
  sig->lnh_dest_hd = NULL;
  if(redir)
    {
      if(!redir->sig)
	{
	  sig->proc_attrs = (U32) redir->pid;
	  /* Check that the process attributes are legal. */
	  if(sig->proc_attrs == 1)
	    {
	      /* Create a sport (signal port). */
	      if(proc_type != OS_PHANTOM)
		sig->proc_attrs = 0;
	    }
	  else if(sig->proc_attrs == 2)
	    {
	      /* Create a deaf tim/int process. */
	      if( (proc_type != OS_TI_PROC) && (proc_type != OS_INT_PROC) )
		sig->proc_attrs = 0;
	    }
	  else if(sig->proc_attrs == 4 || sig->proc_attrs == 8)
	    {
	      /* Create a phantom (with a send hook). */
	      if(proc_type == OS_PHANTOM)
		{
		  struct OS_send_hook_entry *hentry =
		    (struct OS_send_hook_entry *)redir;
		  sig->snd_hook = hentry->snd_hook;
		  sig->lnh_dest_hd = hentry->lnh_hd;
		  redir = NULL;
		  goto no_redir;
		}
	      else
		sig->proc_attrs = 0;
	    }
	  else
	    {
	      /* Illegal process attribute. */
	      sig->proc_attrs = 0;
	    }
	    
	  /* If legal attributes advance redir ptr to "real" redir tab. */
	  if(sig->proc_attrs)
	    redir++;
	}
      /* Remove a useless redirection table */
      if(redir->sig == 1 && redir->pid == 0)
	redir = NULL;
    }
 no_redir:
  
  /*
  ** Allocate a redirection table buffer in the system pool.
  ** Copy the redirection table to the system buffer.
  */
  sig->redir = redir;
  if(redir)
    {
      OSBUFSIZE redir_len = redir->sig * sizeof(struct OS_redir_entry);
      sig->redir = (struct OS_redir_entry *)
	sys_alloc(sizeof(SIGSELECT) + redir_len, (SIGSELECT) 0);
      memcpy((void *) sig->redir, (void *) redir, (size_t) (redir_len ? redir_len : 1));
      redir = sig->redir;
    }

  /*
  ** Check that all process id's in the redirection table are legal from
  ** the point of view that they refer to either a killed or
  ** present process.
  ** Block id's are illegal.
  ** Redirection through a block id could be a way to create hot program handover ??.
  */
  if(redir)
    {
      OSADDRESS count = redir->sig;
      struct OS_redir_entry *r = redir + 1;
      PROCESS defaultpid = redir->pid;

      /* Check that the count is nonzero. */
      if(count)
	count--;
      else
	{
	  os_error(0, OSE_EILLEGAL_REDIRCOUNT | OSE_CREATE_PROCESS, (OSERRCODE) 0);	/* Non fatal. */
	  free_buf((union SIGNAL **) &sig->redir);
	  free_buf((union SIGNAL **) &sig->name);
	  free_buf_type(struct new_process *, &sig);
	  return any_killed_pid();
	}

      /* Check that the default pid is valid or zero. */
      if(defaultpid && !check_pid(defaultpid, OSE_CREATE_PROCESS))
	{
	  free_buf((union SIGNAL **) &sig->redir);
	  free_buf((union SIGNAL **) &sig->name);
	  free_buf_type(struct new_process *, &sig);
	  return any_killed_pid();
	}

      /* Check each pid in the list to see if it is a valid process id. */
      while(count--)
	{
	  if(!check_pid(r->pid, OSE_CREATE_PROCESS))
	    {
	      free_buf((union SIGNAL **) &sig->redir);
	      free_buf((union SIGNAL **) &sig->name);
	      free_buf_type(struct new_process *, &sig);
	      return any_killed_pid();
	    }
	  r++;
	}
    }

  /* Set up the remaining fields in the request signal. */
  sig->proc_type = proc_type;
  sig->entrypoint = entrypoint;
  sig->stack_size = stack_size;
  sig->priority = priority;
  sig->tmo = tmo;
  sig->block = block;
  sig->vector = vector;

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* 
  ** If the block is dead, return a killed pid. 
  ** The check is needed because we must dereference some block attributes.
  */
  bcb = (struct _block *)pid_to_pcb(block);
  if(pcb_to_pid(bcb) != block) /* The block was killed. */
    {
      UNLOCK_SWAP();
      if(sig->redir)
	free_buf((union SIGNAL **) &sig->redir);
      free_buf((union SIGNAL **) &sig->name);
      free_buf_type(struct new_process *, &sig);
      return any_killed_pid();
    }

#if ERROR_CHECKS > 0
  
  /*
  ** Reject to create interrupt processes in a user mode program with a 
  ** program local, non-realtime (pm) signal pool extender.
  */
  if(OS_INT_PROC == proc_type || OS_TI_PROC == proc_type)
    {
      if(!bcb->supervisor_mode &&
	 !bcb->sigpool->realtime_extendable)
	{
      	  UNLOCK_SWAP();
	  if(sig->redir)
	    free_buf((union SIGNAL **) &sig->redir);
	  free_buf((union SIGNAL **) &sig->name);
	  free_buf_type(struct new_process *, &sig);

	  /* This program can not contain interrupt processes. */
	  os_error(0, OSE_CREATE_PROCESS | OSE_EBAD_PROCESS_TYPE, proc_type); 
	  return any_killed_pid();
	}
    }

  /* Do privilege checking when called from user mode. */
  if(!zzcur_p->block->supervisor_mode)
    {
      OSADDRESS priv_mask = 0;

      /* Create an interrupt process? */
      if(OS_INT_PROC == proc_type || OS_TI_PROC == proc_type)
	{
	  priv_mask |= 2;
	}

      /* The wanted block is supervisor_mode? */
      if(bcb->supervisor_mode)
	{
	  priv_mask |= 4;
	}
	  
      /* The wanted block is in another segment? */
      if(zzcur_p->block->segid != bcb->segid)
	{
	  priv_mask |= 8;
	}

      /* Is the system configured to permit this operation? */
      if(priv_mask & ~local_sysctlblk.relax_error_checks)
	{
	  UNLOCK_SWAP();
	  if(sig->redir)
	    free_buf((union SIGNAL **) &sig->redir);
	  free_buf((union SIGNAL **) &sig->name);
	  free_buf_type(struct new_process *, &sig);

	  /*
	  ** The error code OSE_EPRIVILEGED_SYSCALL hints that the extra
	  ** parameter contains bits required in the system parameter.
	  */
	  os_error(0, OSE_CREATE_PROCESS | OSE_EPRIVILEGED_SYSCALL, priv_mask); 
	  return any_killed_pid();
	}
    }
#endif

  /* Allocate a stack for non int processes. (Locked dispatcher.) */
  if((proc_type == OS_BG_PROC) || (proc_type == OS_PRI_PROC))
    {
      /* Add extra for supervisor processes. */
      if(bcb->supervisor_mode)
	{
	  sig->stack =
	    stack_alloc(bcb->stackpool,
			stack_size + local_sysctlblk.superv_stack_size);
	}
      else
	{
	  sig->stack = stack_alloc(bcb->stackpool, stack_size);
	}
    }
  else
    {
      /* No stack for other process types. */
      sig->stack = NULL;
    }

  /* Send to sysd with dispatcher locked so we won't loose the stack. */
  set_suid((OSBOOLEAN) 1);
  zzsys_send_w_s_type(struct new_process *, &sig,
		      current_process(), zzuu.sysd_);

  /* Unlock the dispatcher. */
  UNLOCK_SWAP();

  /* Wait for a response from the system daemon. */
  {
    static const SIGSELECT sig_sel[] = {1, NEW_PROCESS};
    while(NIL == (union SIGNAL *) (sig = (struct new_process *)
				   receive_from((OSTIME) -1, sig_sel, zzuu.sysd_)))
      {
        ; /* Empty loop body. */
      }
  }
  set_suid((OSBOOLEAN) 0);

  /*
  ** Call error if sysd failed to create the process.
  ** Return a killed process id if an error occurred.
  ** Name and redir buffers were freed by sysd.
  */
  if(sig->ecode)
    {
      /* Kill the new process. */
      local_kill_proc(sig->pid, (OSBOOLEAN) 0); /* Leave the block even if it's empty. */
      /* Report the error in the proper process context. */
      os_error(0, sig->ecode | OSE_CREATE_PROCESS, sig->extra);	/* Non fatal. */
      /* Free the response buffer from sysd. */
      free_buf_type(struct new_process *, &sig);
      /* Return a killed process id to the caller. */
      return newpid;
    }

  /* Pick up the pid of the new process. */
  newpid = sig->pid;
  
  /* Free the response buffer from sysd. */
  free_buf_type(struct new_process *, &sig);
  
  /* Set the user id of the new process if required. */
  /* this should really be done before the process is made public?? */
  if(user)
    set_uid(newpid, user);

  /* Return the new process id to the caller. */
  return newpid;
}

/*
** This is a SMP helper for the kill_proc system call, to kill a process on
** another cpus than the kill_proc caller. 
**
** It is called by the sysd_lite process instance on the cpu of the victim,
** with locked dispatcher.
** If sig_delete_process is not NIL at return, it shall be disposed by the
** caller.
*/

#ifdef OSE_SMP
static void
sys_kill_proc(struct _pcb *pcb, union SIGNAL **sig_delete_process)
{
  PROCESS pid = pcb_to_pid(pcb);

#if 0 				/* gunnar: Removed */
  /* Check if the PCB is locked. */
  if((pcb->magic == MAGIC_PCB) && pcb->lock_cnt)
    {
      /*
      ** We can't kill a locked PCB, so we set the locked_kill
      ** flag instead. Which means that the killing is deferred to
      ** just after the PCB has been unlocked. Start the process
      ** to let it die.
      */
      pcb->locked_kill = (OSBOOLEAN)1;
      UNLOCK_SWAP();
      while(pid_to_instance(pid) == pcb->instance && pcb->st_cnt)
	start(pcb->live_pid);
      LOCK_SWAP();
      return;
    }
#endif

  /* Try to kill it. */
  if(!kill_one_proc(pcb))
    {
      /*
      ** The process was kill_locked. It is also stopped by the killing
      ** cliant, so we have to start it to let it die.
      */
      UNLOCK_SWAP();
      while(pid_to_instance(pid) == pcb->instance && pcb->st_cnt)
	start(pcb->live_pid);
      LOCK_SWAP();
    }
  else
    {
      /*
      ** The process is killed.
      ** Tell the system daemon to detete it.
      ** Using zzsys_send_w_s, because the dispatcher is still locked.
      */
      (*sig_delete_process)->delete_process.pid = pid;
      zzsys_send_w_s(sig_delete_process, current_process(), zzuu.sysd_);
      *sig_delete_process = NIL;
    }
}
#endif

/*
** This is a SMP helper for the kill_proc system call, to kill blocks,
** possibly with processes on other cpus than the kill_proc caller. 
**
** It is called by the sysd_lite process instance on the cpu of the victim,
** with locked dispatcher. 
** If processes on another cpu is detected, a cpu_id is returned, and 
** sys_kill_block() must be called at that cpu too.
** Otherwise, the block is now gone and ~0 is returned.
** 
** If sig_delete_process is not NIL at return, it shall be disposed by the
** caller.
*/

#ifdef OSE_SMP
static cpuid_t
sys_kill_block(struct _block *bcb, union SIGNAL **sig_delete_process)
{
  PROCESS bid = pcb_to_pid(bcb);
  struct _pcb *pcb;
  struct _pcb *pcb_on_other_cpu = NULL;
  struct _pcb *kill_locked_pcb = NULL;
  
  for(pcb = bcb->proclist; pcb; pcb = pcb->proclist)
    {
      if(!pcb->killed)
	{
	  if(pcb->cpu_id != zzkrn_cpu_id())
	    {
	      /*
	      ** If the pcb has not taken the kill-lock, or has the
	      ** kill-lock but no locked_kill flag, then handle it on the
	      ** proper cpu.
	      ** (Each kill-locked process should just be kicked once.)
	      */
	      if(pcb->lock_cnt == 0 || pcb->locked_kill == 0)
		pcb_on_other_cpu = pcb;
	    }
	  else
	    {
	      int wasnt_stopped = 0;

	      /* Check if the PCB is kill-locked. */
	      if(!kill_one_proc(pcb))
		{      
		  /*
		  ** The PCB is kill-locked.
		  ** We can't kill a locked PCB, so we did set the locked_kill
		  ** flag instead. Which means that the killing is deferred to
		  ** just after the PCB has been unlocked.
		  **
		  ** Now, resume and start it if needed to make it die.
		  */
		  kill_locked_pcb = pcb;
		  if(pcb->status & ST_INTERCEPTED)
		    start_one_proc(pcb, &wasnt_stopped, ST_INTERCEPTED);
		  if(pcb->st_cnt)
		    {
		      pcb->st_cnt = 1;
		      start_one_proc(pcb, &wasnt_stopped, ST_STOPPED);
		    }
		}
	    }
	}
    } /* for */

  /*
  ** If there are any processes on another cpu, they must
  ** be killed on that cpu.
  */
  if(pcb_on_other_cpu)
    return pcb_on_other_cpu->cpu_id;

  /*
  ** If there are kill_locked processes, we can't kill the block yet.
  ** The last kill_locked process will do that when it dies. That has
  ** not yet happended because the dispatcher is still locked.
  */
  if(kill_locked_pcb)
    {
      return (cpuid_t)~0;
    }
  
  /* Set the killed flag and bump the instance number. */
  BKL_LOCK();
  bump_pcb_instance((struct _pcb *)bcb);

  /*
  ** We also change the type of the pcb so that check_pid() et. al understands
  ** and refuses pathological designed pids.
  */
  bcb->magic |= MAGIC_KILLED;
  BKL_UNLOCK();

  /* All processes as well as the block is now officially gone. */

  /*
  ** Tell the system daemon to kill the process/block.
  ** Using zzsys_send_w_s, because the dispatcher is still locked.
  */
  (*sig_delete_process)->delete_process.pid = bid;
  zzsys_send_w_s(sig_delete_process, current_process(), zzuu.sysd_);
  *sig_delete_process = NIL;

  return (cpuid_t)~0;
}
#endif

/*
** This is the kill_proc system call.
** We should use the highpri_count field in the segment descriptor.??
*/

static void
local_kill_proc(PROCESS pid, OSBOOLEAN block_too)
{
  register struct _pcb *pcb;
  register struct _block *bcb;
  register PROCESS cur_pid;
  union SIGNAL *sig;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_KILL_PROC))
    {
      return;
    }
#endif

  /* Save the callers process id in case we kill the current process. */
  cur_pid = zzcur_p->pid;

  /* Locate the block or process descriptor. */
  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid_bid_seg(pid, OSE_KILL_PROC);
  if(pcb == NULL)
    {
      if ((pid_to_pindex(pid) == pid_to_pindex(zzcur_p->live_pid))
	  && (zzcur_p->magic & MAGIC_KILLED))
	{
	  LOCK();
	  for(;;)
	    {
	      zzquit_int();
	      os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 21);	/* Fatal. */
	    }
	    
	}
      return;
    }

  /* Enter superuser mode. */
  set_suid((OSBOOLEAN) 1);

  /*
  ** Allocate a signal to be sent to the system daemon.
  ** Note that it may be used for DELETE_PROCESS, KILL_SEG_REQ or
  ** SMP_KILL_PROC_REQUEST signals.
  */
  sig = alloc(sizeof(struct delete_process), DELETE_PROCESS);

  /* Lock the dispatcher. */
  if(!zzdispatcher_locked)
    LOCK_SWAP();

  /* Check that the block/process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      UNLOCK_SWAP();
      /* Exit superuser mode. */
      set_suid((OSBOOLEAN) 0);
      free_buf(&sig);
      return;
    }

#ifdef GUNNAR_DEBUG
  {
    const char *name = "?";
    
    switch(pcb->magic)
      {
      case MAGIC_PCB:
	name = pcb->name;
	break;
	
      case MAGIC_BCB:
	name = ((struct _block *) pcb)->name;
	ramlog_printf("killing block(%#x [%s]) st_cnt: %d\n",
		      pcb_to_pid(pcb), (char *)name,
		      ((struct _block *) pcb)->st_cnt);
	break;
	
      case MAGIC_SEG:
	name = "Segment!";
	break;
	
      } /* switch */
    
    ramlog_printf("kill_proc(%#x [%s]) from %#x [%s]\n",
		  pcb_to_pid(pcb), (char *)name,
		  pcb_to_pid(zzcur_p), (char *)zzcur_p->name);
  }
#endif

  /* Disable interrupts. */
  BKL_LOCK();
  
  /* Check if the PCB is locked. */
  if((pcb->magic == MAGIC_PCB) && pcb->lock_cnt)
    {      
      if(pid != cur_pid)
	{
	  /*
	  ** We can't kill a locked PCB, so we set the locked_kill
	  ** flag instead. Which means that the killing is deferred to
	  ** just after the PCB has been unlocked.
	  */
	  pcb->locked_kill = (OSBOOLEAN)1;
	  BKL_UNLOCK();
	  UNLOCK_SWAP();
	  /* Exit superuser mode. */
	  set_suid((OSBOOLEAN) 0);
	  free_buf(&sig);
	  return;
	}
      else
	{
	  /*
	  ** Oops! It looks like we are locked but we want to commit
	  ** suicide. We probably ended up in the error handler and
	  ** decided that we can't continue to run this faulty
	  ** process.
	  **
	  ** Unfortunately this also forces us to halt/reboot the
	  ** entire system. Since if we:
	  **
	  ** 1. Don't set pcb->lock_cnt to zero before calling
	  **    os_error() we will end up in a recursive error
	  **    madness.
	  **
	  ** 2. Unlock the dispatcher (and set pcb->lock_cnt
	  **    to zero) before calling os_error() some other 
	  **    process can be swapped in and issue a kill_proc()
	  **    upon this faulty process and succeed, without
	  **    informing anyone about the fact that the killing
	  **    was done in the middle of a kill-locked region.
	  ** 
	  ** This lack of recovery has a smell of the dark ages, it's
	  ** not worthy a modern OS...  Oh good I do hate this...
	  ** @frbr@
	  */
	locked_suicide:
	  os_error(2, OSE_EBAD_PARAMETER | OSE_KILL_PROC, (OSERRCODE) pid);	/* Fatal system error. */
	  /*NOTREACHED*/
	}
    }

  /* Reenable interrupts. */
  BKL_UNLOCK();
  
  /* Filter out kill segment requests and send them on to sysd. */
  /* Shouldn't we do the killing here as for other types?? */
  /*
  ** We don't have the code to check user privileges when killing a segment. ??
  */
  if(pcb->magic == MAGIC_SEG)
    {
      struct segcb *segcb = (struct segcb *)pcb;
      
      UNLOCK_SWAP();
      sig->kill_seg_req.sig_no = KILL_SEG_REQ;
      sig->kill_seg_req.segid = pid;
      if(zzcur_p->pid != zzuu.sysd_)
	{
	  /* If we are not called from sysd we send a signal to it.... */
	  send(&sig, zzuu.sysd_);
	}
      else
	{
	  /* ...otherwise we do the job here and now. */
	  sysd_do_kill_seg_req(sig);
	  free_buf(&sig);
	}
      /* Exit superuser mode. */
      set_suid((OSBOOLEAN) 0);
      
      /*
      ** Return now if we are not killing the current segment. Note
      ** that we need to check the current block's segid since the
      ** current process might have done a set_segment() system call.
      */
      if(segcb->segid != zzcur_p->block->segid)
	{
	  return;
	}
	  
      /*
      ** If we *are* killing the current segment we need to prevent
      ** the caller from returning out of the kill_proc() system call.
      */
      for (;;)
	{
	  stop_or_intercept(zzcur_p->block->segpid, ST_INTERCEPTED,
			    OSE_INTERCEPT, BUT_KILL_LOCKED);
	  /* Still running are we!? Oh, no, maybe we are "kill locked"? */
	  if(zzcur_p->lock_cnt != 0)
	    goto locked_suicide;
	}
      /* NOTREACHED */
    }

  /* Ignore any attempt to kill a system daemon or a similar process. */
  if(pcb->magic == MAGIC_BCB)
    bcb = (struct _block *) pcb;
  else
    bcb = pcb->block;
  if(bcb == zzuu.system_block)
    {
      UNLOCK_SWAP();
      os_error(0, OSE_EKILLED_SYSTEM_DAEMON | OSE_KILL_PROC,
	  (OSERRCODE) pid);	/* Non fatal. */

      /*
      ** Allow a process in the system block to commit suicide if it
      ** returns ok from the error handler.
      */
      if(pcb != zzcur_p)
	{
	  free_buf(&sig);
	  /* Exit superuser mode. */
	  set_suid((OSBOOLEAN) 0);
	  return;
	}
      LOCK_SWAP();
    }

  /* If pid is the last process in the block, then kill the block. */
  if(block_too)
    {
      register struct _pcb *tpcb;
      register int count;
      
      /*
      ** Killing last process if there is exactly one nonkilled
      ** process in the block.
      */
      for(count = 0, tpcb = bcb->proclist; tpcb; tpcb = tpcb->proclist)
	if(!tpcb->killed)
	  count++;
      if(count == 1)
	{
	  pid = pcb_to_pid(bcb);
	  pcb = (struct _pcb *) bcb;
	}
    }

  /* If a block was specified, then kill it. */
  if(pcb->magic == MAGIC_BCB)
    {
      /*
      ** Just make sure we are not "kill locked" if we are part the
      ** block being killed.
      */
      if((struct _pcb *)(zzcur_p->block) == pcb && zzcur_p->lock_cnt != 0)
	goto locked_suicide;

      /* Mark the block as being killed. No new processes will be created. */
      ((struct _block*)pcb)->kill_in_progress = 1;

#ifdef OSE_SMP
      /*
      ** Stop all processes in the block (except the caller) so sysd_lite 
      ** isn't starved out by the block being killed.
      */
      stop_or_intercept(pid, ST_STOPPED, OSE_STOP, BUT_CURRENT_PROC);

      {
	static const SIGSELECT sig_kill_proc[] = {1, SMP_KILL_PROC_REPLY};
	PROCESS sysd_lite_;

	/*
	** Send kill request to sysd_lite on current cpu if the caller belongs
	** to the block, otherwise to the cpu of the block.
	*/
	sysd_lite_ = zzcpu[zzcur_p->block == (struct _block *)pcb ?
			   zzkrn_cpu_id() : bcb->cpu_id
			   ].sysd_lite_;

	UNLOCK_SWAP();
   
	sig->sig_no = SMP_KILL_PROC_REQUEST;
	sig->smp_kill_proc.pid = pid;
	send(&sig, sysd_lite_);
	do {sig = receive_from(~0u, sig_kill_proc, zzuu.sysd_);} while(sig == NIL);
	/* Exit superuser mode. */
	set_suid((OSBOOLEAN) 0);
	free_buf(&sig);
	return;
      }
#else  /* !OSE_SMP */
      if (kill_block(bcb))
	{
	  /*
	  ** Oops, all processes in the block where "kill locked"
	  ** which means that there is no use in telling sysd.
	  */
	  UNLOCK_SWAP();
	  free_buf(&sig);
	  sig = NULL;
	  /* Exit superuser mode. */
	  set_suid((OSBOOLEAN) 0);
	  return;
	}
#endif	/* !OSE_SMP */
    }
  else
    {
      /* If a single process was specified, then kill it. */
#ifdef OSE_SMP
      if(pcb->cpu_id != zzkrn_cpu_id())
	{
	  PROCESS helper = zzcpu[pcb->cpu_id].sysd_lite_;
	  static const SIGSELECT sig_kill_proc[] = {1, SMP_KILL_PROC_REPLY};

	  UNLOCK_SWAP();

	  /* Stop the victim process, so sysd_lite isn't starved out by it. */
	  stop_or_intercept(pid, ST_STOPPED, OSE_STOP, BUT_CURRENT_PROC);
	  
	  sig->sig_no = SMP_KILL_PROC_REQUEST;
	  sig->smp_kill_proc.pid = pid;
	  send(&sig, helper);
	  do {sig = receive_from(~0u, sig_kill_proc, zzuu.sysd_);} while(sig == NIL);
#if 0 				/* frbr: */
	  /* frbr: */
	  if(pcb->magic == MAGIC_PCB &&
	     pcb_to_pid(pcb) == pid &&
	     pcb->locked_kill)
	    {
	      start(pid);
	    }
#endif	/* frbr */
	  /* Exit superuser mode. */
	  set_suid((OSBOOLEAN) 0);
	  free_buf(&sig);
	  return;
	}
#endif
      kill_one_proc(pcb);
    }

  if(sig != NULL)
    {
      /* Tell the system daemon to kill the process/block. */
      sig->delete_process.pid = pid;
      zzsys_send_w_s(&sig, cur_pid, zzuu.sysd_);
      
      /* Exit superuser mode. */
      set_suid((OSBOOLEAN) 0);
    }

  /* Unlock the dispatcher and return. */
  /* This also terminates the caller if we killed the current process. */
  UNLOCK_SWAP();
}

void
kill_proc(PROCESS pid)
{
  register struct _pcb *current;

  /*
  ** Check that the caller isn't in a recursive fatal error trying to kill
  ** the current process. This check is just an extra safeguard against
  ** a fairly common mistake that is hard to track down.
  */
#if ERROR_CHECKS > 0
  if(zzcur_p->err_handl_level > 2)
    {
      LOCK();
      handlerp.zzkrn_halted_hook();
    }
#endif

  /*
  ** If the current process is a user mode process, check the stack end mark
  ** The reason is: If the user process intends to kill itself, we don't want
  ** to detect the error when the process is already dead. This would
  ** effectively inhibit system recovery by just killing the current
  ** segment/block.
  **
  ** Of the same reason, we simply ignore the stack mark error if we are
  ** already processing another error in the current process. 
  */
  current = zzcur_p;
  if(!current->block->supervisor_mode &&
     current->err_handl_level == 0)
    {
      unsigned char *limit = (unsigned char *) current->stack->limit;
      
      if(END_MARK != *limit)
	{
	  (void) handle_error(1, OSE_EUSER_STACK_OVERFLOW, /* Non-fatal */
			      (OSERRCODE) limit,
			      (OSBOOLEAN) 0); /* Not user_called */
	  (void) handle_error(2, OSE_EUSER_STACK_OVERFLOW, /* Fatal */
			      (OSERRCODE) limit,
			      (OSBOOLEAN) 0); /* Not user_called */
	  /* NOTREACHED */
	}
    }

  /* Take the block with us if required. */
  local_kill_proc(pid, (OSBOOLEAN) 1);
#if defined(GUNNAR_DEBUG)
  if(current->block->pindex == pid_to_pindex(pid))
    {
      ramlog_printf("pose.c: kill_proc(%#lx): Returns! error_level %d, dp_locked %d dp_state %d.\n",
		    pid, current->err_handl_level, zzdispatcher_locked, zzww.dp_state);
      dump_pcb((struct _pcb *)current->block);
      dump_pcb(current);
    }
#endif
}

/*
** This is the get_pri system call.
*/

OSPRIORITY 
get_pri(PROCESS pid)
{
  register struct _pcb *pcb;
  OSPRIORITY result;

  if(pid == zzcur_p->pid)
    return (OSPRIORITY)(zzcur_p->priority & 31);

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  ON_ERRLEVEL_2()
    if(check_nonint_caller(OSE_GET_PRI))
      return (OSPRIORITY) 0;
#endif

  /* Locate the process descriptor. */
  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid(pid, OSE_GET_PRI);
  if(pcb == NULL)
    {
      return (OSPRIORITY) 0;
    }

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the block/process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      UNLOCK_SWAP();
      return (OSPRIORITY) 0;
    }

#if USE_REMOTE_CALLS
    /* Issue a remote system call if required. */
  if(local_sysctlblk.enable_remcalls)
    {
    union SIGNAL *sig;
    register OSBOOLEAN use_remcalls;
    register PROCESS handler;
    if(pcb->magic == MAGIC_PCB)
      {
	use_remcalls = pcb->block->use_remcalls;
	handler = pcb->block->remcall_server;
      }
    else
      {
	use_remcalls = ((struct _block *)pcb)->use_remcalls;
	handler = ((struct _block *)pcb)->remcall_server;
      }
    if(use_remcalls)
      {
	UNLOCK_SWAP();
	set_suid((OSBOOLEAN) 1);
	sig = alloc(sizeof(struct remote_call), REMOTE_CALL);
	sig->remote_call.call_type = REM_GET_PRI;
	sig->remote_call.params.rem_get_pri.pid = pid;
	
	sig = make_remote_call(sig, (OSCALLTYPE) REMR_GET_PRI, handler);
	set_suid((OSBOOLEAN) 0);
	if(sig)
	  {
	    result = sig->remote_response.results.remr_get_pri.pri;
	    free_buf(&sig);
	    return result;
	  }
	return (OSPRIORITY) 0;
      }
    }
#endif

  /* Check that the process is a prioritized process. Done if not. */
  result = (OSPRIORITY)(pcb->priority & 31);
  UNLOCK_SWAP();
  return result;
}

/*
** This is the set_pri system call.
*/

OSPRIORITY 
set_pri(OSPRIORITY newpri)
{
  register struct _pcb *pcb = zzcur_p;
  register struct ready_list_entry *rlist;
  register OSPRIORITY oldpri;

#if ERROR_CHECKS > 0
  /* Check that the caller is a prioritized process. */
  if(unlikely(pcb->priority > 31))
    {
      os_error(0, OSE_EILLEGAL_PRISYSTEMCALL | OSE_SET_PRI, (OSERRCODE) 0);	/* Non fatal. */
      return pcb->priority & 31;
    }

  /* Check that the new priority is valid. */
  if(unlikely(newpri > 31))
    {
      os_error(0, OSE_EBAD_PARAMETER | OSE_SET_PRI, (OSERRCODE) newpri);	/* Non fatal. */
      return pcb->priority & 31;
    }
#endif

  /* Disable interrupts. */
  LOCK();

  /* Pick up the old priority. */
  oldpri = pcb->priority;

  /* Pick up the current ready list. */
  rlist = pcb->ready_entry.rlist;

  /* Remove the process from the ready list. */
  rlist->hd = pcb->ready_entry.nxt;
  if(rlist->hd)
    rlist->hd->ready_entry.prev = NULL;
  else
    zzdp_look_aside ^= RLIST_PRI_MASK(pcb);

  /* Set a new priority in the pcb. */
  pcb->priority = newpri;
  rlist = pcb->ready_entry.rlist = (struct ready_list_entry *) &zzww.ready_list[newpri];

  /* Insert the process into the ready list. */
  if(rlist->hd)
    {
      pcb->ready_entry.prev = rlist->tl;
      rlist->tl->ready_entry.nxt = pcb;
      rlist->tl = pcb;
    }
  else
    {
      rlist->hd = rlist->tl = pcb;
      pcb->ready_entry.prev = NULL;
      zzdp_look_aside ^= RLIST_PRI_MASK(pcb);
    }
  pcb->ready_entry.nxt = NULL;

  /* Swap if required, enable interrupts and return. */
  if(newpri >= oldpri)
    {
      zzquit_int();
    }
  UNLOCK();
  return newpri;
}

#ifdef OSE_SMP
/*
** Helper function for disabling a vector in an SMP system. Called
** from sysd. */
static void
sysd_disable_vector_helper(OSVECTOR vec)
{
  struct vectorcb *vcb;
  cpuid_t cur_cpu;
  uint32_t cpus;

  LOCK_SWAP();
  vcb = get_vectorcb(vec, 0);

  /* Check that the work is still valid. */
  if (vcb->isrcount + vcb->pcbcount == 0)
    {
      vcb->disabled++;
      UNLOCK_SWAP();
      return;
    }

  cpus = vcb->cpus;
  UNLOCK_SWAP();

  /* Enter superuser mode to allocate from the system pool. */
  set_suid((OSBOOLEAN) 1);

  for (cur_cpu = 0; cpus != 0; cur_cpu++)
    {
      PROCESS helper = zzcpu[cur_cpu].sysd_lite_;
      static const SIGSELECT sig_zzdisable_vector[] = {1, SMP_ZZDISABLE_VECTOR_REPLY};
      union SIGNAL *sig;

      /* Skip all cpus that are not bound to this vector. */
      if ((cpus & (1u << cur_cpu)) == 0)
        continue;

      /* This core is now in need of running disable. */
      sig = alloc(sizeof(struct smp_zzdisable_vector), SMP_ZZDISABLE_VECTOR_REQUEST);
      sig->smp_zzdisable_vector.vec = vec;
      send(&sig, helper);
      do {sig = receive_from(~0u, sig_zzdisable_vector, helper);} while(sig == NIL);
      free_buf(&sig);

      /* Clear the bind flag to be able to exit the loop asap. */
      cpus &= ~(1U << cur_cpu);            
    }
                   
  /* Exit superuser mode. */
  set_suid((OSBOOLEAN) 0);

  /* Increase disabled counter and return. */
  vcb->disabled++;
}
     
#endif

/*
** This is the ose_disable_vector system call. 
*/
static void
zzkrn_disable_vector(OSVECTOR vec)
{
  struct vectorcb *vcb;
  
  /* Fetch a vectorcb, allocate if neccessary. */
  LOCK_SWAP();
  vcb = get_vectorcb(vec, 1);

  /* Check if there is a need to call zzdisable_vector. */
  if ((vcb->disabled == 0) && (vcb->isrcount + vcb->pcbcount > 0))
    {      
#ifdef OSE_SMP
      /* In the general case we will have to send signals to several
       * cores. We cannot lock the dispatcher, but we don't want to
       * die during this either, so we'll pass it on to sysd. */
      static const SIGSELECT sig_disable_vector[] = {1, SMP_DISABLE_VECTOR_REPLY};
      union SIGNAL *sig;

      UNLOCK_SWAP();
      
      set_suid((OSBOOLEAN) 1);
      sig = alloc(sizeof(struct smp_disable_vector), SMP_DISABLE_VECTOR_REQUEST);
      sig->smp_disable_vector.vec = vec;
      send(&sig, zzuu.sysd_);
      do {sig = receive_from(~0u, sig_disable_vector, zzuu.sysd_);} while(sig == NIL);
      free_buf(&sig);
      set_suid((OSBOOLEAN) 0);
      return;      
#else
      /* In the non SMP case we can simply disable the vector. */
      zzdisable_vector(vec);
#endif      
    }

  /* Increase the disabled counter and return. */
  vcb->disabled++;        
  UNLOCK_SWAP();
}

#ifdef OSE_SMP
/*
** Helper function for enabling a vector in an SMP system. Called from
** sysd, so it will never be called concurrently.
*/
static OSBOOLEAN
sysd_enable_vector_helper(OSVECTOR vec)
{
  struct vectorcb *vcb;
  cpuid_t cur_cpu;
  uint32_t cpus;

  LOCK_SWAP();
  vcb = get_vectorcb(vec, 0);

  /* Check that we won't trigger a spurious enable. */
  if (vcb->disabled == 0)
    {
      UNLOCK_SWAP();
      return 0;
    }

  /* Check that the work is still valid. If not, just decrease. */
  if (vcb->pcbcount + vcb->isrcount == 0)
    {
      vcb->disabled--;
      UNLOCK_SWAP();
      return 1;
    }
  
  cpus = vcb->cpus;  
  UNLOCK_SWAP();

  /* Enter superuser mode to allocate from the system pool. */
  set_suid((OSBOOLEAN) 1);
        
  for (cur_cpu = 0; cpus != 0; cur_cpu++)
    {
      PROCESS helper = zzcpu[cur_cpu].sysd_lite_;
      static const SIGSELECT sig_zzenable_vector[] = {1, SMP_ZZENABLE_VECTOR_REPLY};
      union SIGNAL *sig;
            
      /* Skip all cpus that are not bound to this vector. */
      if ((cpus & (1u << cur_cpu)) == 0)
        continue;

      /* We now know that this core should get an enable. */
      sig = alloc(sizeof(struct smp_zzenable_vector), SMP_ZZENABLE_VECTOR_REQUEST);
      sig->smp_zzenable_vector.vec = vec;
      sig->smp_zzenable_vector.pri = vcb->priority;
      send(&sig, helper);
      do {sig = receive_from(~0u, sig_zzenable_vector, helper);} while(sig == NIL);
      free_buf(&sig);
            
      /* Clear the bind flag to be able to exit the loop asap. */
      cpus &= ~(1U << cur_cpu);
    }

  /* Exit superuser mode. */
  set_suid((OSBOOLEAN) 0);

  /* All done, decrease the disabled count. */
  vcb->disabled--;
  
  return 1;
}
#endif

/*
** This is the ose_enable_vector system call.
*/
static void
zzkrn_enable_vector(OSVECTOR vec)
{
  struct vectorcb *vcb;

  LOCK_SWAP();
  vcb = get_vectorcb(vec, 0);

  /* Check for spurious enable vector. */
  if ((vcb == NULL) || (vcb->disabled == 0))
    {
      UNLOCK_SWAP();
      os_error(0, OSE_ESPURIOUS_ENABLE_VECTOR | OSE_ENABLE_VECTOR, (OSERRCODE) vec);	/* Non fatal. */
      return;
    }

  /* Check if there is a need to call zzenable_vector. */            
  if ((vcb->disabled == 1) && (vcb->isrcount + vcb->pcbcount > 0))
    {      
#ifdef OSE_SMP
      /* In the general case we will have to send signals to several
       * cores. We cannot lock the dispatcher, but we don't want to
       * die during this either, so we'll pass it on to sysd. */
      static const SIGSELECT sig_enable_vector[] = {1, SMP_ENABLE_VECTOR_REPLY};
      union SIGNAL *sig;

      UNLOCK_SWAP();
      
      set_suid((OSBOOLEAN) 1);
      sig = alloc(sizeof(struct smp_enable_vector), SMP_ENABLE_VECTOR_REQUEST);
      sig->smp_enable_vector.vec = vec;
      send(&sig, zzuu.sysd_);
      do {sig = receive_from(~0u, sig_enable_vector, zzuu.sysd_);} while(sig == NIL);

      /* It's possible that it ended up being a spurious enable. */
      if (!sig->smp_enable_vector.success)
        {
          os_error(0, OSE_ESPURIOUS_ENABLE_VECTOR | OSE_ENABLE_VECTOR, (OSERRCODE) vec);	/* Non fatal. */
        }
      
      free_buf(&sig);
      set_suid((OSBOOLEAN) 0);
      return;      
#else
      /* Non SMP case, we'll simply just enable the vector. */
      zzenable_vector(vec, vcb->priority);
#endif      
    }

  /* Decrease the disabled counter and return. */
  vcb->disabled--;      
  UNLOCK_SWAP();
}

/*
** Returns a pointer to the isr_list member in the control block
** belonging to the the given pid, which is located differently
** depending on whether the pid indicates a process, block or
** segment. Dispatcher shall be locked by the caller.
*/
static struct isrcb**
get_isr_list_from_pid(PROCESS pid)
{
  struct _pcb *pcb = safe_pid_to_pcb(pid, 1);

  if (!pcb)
    {
      return NULL;
    }

  /* Figure out the linked list location based on magic. */
  switch (pcb->magic)
    {
    case MAGIC_PCB:
      return &pcb->isr_list;

    case MAGIC_BCB:
      return &((struct _block *)pcb)->isr_list;

    case MAGIC_SEG:
      return &((struct segcb *)pcb)->isr_list;

    default:
      return NULL;
    }  
}

/*
** Inserts an entry into the correct control block based on the owner
** pid. Dispatcher shall be locked by the caller.
*/
static OSBOOLEAN
attach_isr_or_hook(struct vectorcb *vcb, uint8_t slot)
{
  struct isrcb **isr_list;
  
  /* Owner 0 is a nop. */
  if (vcb->isr[slot].owner == 0)
    {
      return 1;
    }

  /* Fetch the correct list. */
  isr_list = get_isr_list_from_pid(vcb->isr[slot].owner);
  if (!isr_list)
    {
      return 0;
    }
  
  /* Link it in. */
  vcb->isr[slot].next = *isr_list;
  *isr_list = &vcb->isr[slot];

  return 1;
}

/*
** Removes an entry from the correct control block based on the owner
** pid. Dispatcher shall be locked by the caller.
*/
static void
detach_isr_or_hook(struct vectorcb *vcb, uint8_t slot)
{
  struct isrcb **isr_list;
  struct isrcb **isr;

  if (vcb->isr[slot].owner == 0)
    {
      return;
    }
  
  /* Fetch the correct list. */
  isr_list = get_isr_list_from_pid(vcb->isr[slot].owner);
  if (!isr_list)
    {
      return;
    }

  /* Unlink it. */
  for (isr = isr_list; *isr; isr = &(*isr)->next)
    {
      if (*isr == &vcb->isr[slot])
        {
          *isr = vcb->isr[slot].next;
          break;
        }
    }
  
  return;  
}

/*
** Helper function to create_isr_or_hook. Executed on the core the ISR
** should reside on. Can be called from sysd_lite. Should be called
** with locked dispatcher. Returns an ISR handle. If an error occurs,
** the error argument is set to the error code. It is set to 0 on
** success. Otherwise the error code should be propagated to os_error
** as a non-fatal error.
*/
static uint32_t
create_isr_or_hook_helper(struct vectorcb *vcb, PROCESS owner, void *isr,
                          void *arg, uint8_t type, OSERRCODE *error,
                          cpuid_t cpu)
{
  int slot;
  
  /* Check if the vector is usable. */
  if (type == 0)
    {
      /* An ISR or interrupt process must be installed first. */
      if (vcb->isrcount + vcb->pcbcount == 0)
        {
          *error = OSE_EVECTOR_NOT_INITIALIZED;
          return 0;
        }
    }
  else
    {
      /* An ISR and an interrupt process cannot coexist. */
      if (vcb->pcbcount > 0)
        {
          *error = OSE_EVECTOR_IN_USE;
          return 0;          
        }
    }
    
  /* Find an empty isr slot in the vectorcb. */
  for (slot = 0; slot < 7; ++slot)
    {
      if (vcb->isr[slot].isr == NULL)
        break;
    }

  /* Make sure that we are not out of slots. */
  if (slot == 7)
    {
      if (type == 0)
        *error = OSE_ETOO_MANY_INTERRUPT_HOOKS;
      else
        *error = OSE_ETOO_MANY_ISRS;
      return 0;
    }

  /* Fill in all isrcb fields. */
  vcb->isr[slot].isr = isr;
  vcb->isr[slot].arg = (uint32_t)arg;
  vcb->isr[slot].owner = owner;
  vcb->isr[slot].type = type;
  vcb->isr[slot].slot = slot;
  vcb->isr[slot].vcb = vcb;
  
  /* Attach to the owner process. */
  if (!attach_isr_or_hook(vcb, slot))
    {
      vcb->isr[slot].isr = NULL;
      *error = OSE_EILLEGAL_OWNER;
      return 0;
    }

  /* The vectorcb may not yet have been assigned to the correct cpu. */
  if (type == 1)
    {
      if (vcb->isrcount == 0)
        {
          /* The first ISR should clear any previous contents. */
          vcb->cpus = (1U << cpu);
        }
      else
        {
          /* Additional ISRs (not hooks) always enables the cpu. */
          vcb->cpus |= (1U << cpu);
        }
    }
  
  /* Register with the target interface. */
  if (!zzbind_vector_isr(vcb->vecno, (void *)isr, (uint32_t)arg, type))
    {
      /* Our mission is a failure. */
      vcb->isr[slot].isr = NULL;
      if (type == 0)
        *error = OSE_EINTERRUPT_HOOK_INSTALLATION;
      else
        *error = OSE_EISR_INSTALLATION;
      return 0;
    }
  
  /* Bump the generation counter, wrap it to the limited range. */
  vcb->isr[slot].generation++;
  if (vcb->isr[slot].generation > 0xfff)
    {
      /* Setting it to 1 ensures that 0 is never a valid handle. */
      vcb->isr[slot].generation = 1;
    }

  /* If we are the first ISR to arrive, enable the vector. */
  if (type == 1)
    {
      vcb->isrcount++;
      if ((vcb->disabled == 0) && (vcb->isrcount == 1))
        {
          zzenable_vector(vcb->vecno, 0);
        }
    }      
  
  /* Construct a suitable ISR handle. */
  *error = 0;
  return ((uint32_t)(vcb->vecno << 16u) |
	  (uint32_t)(vcb->isr[slot].generation << 4u) |
	  (uint32_t)slot);
}

#ifdef OSE_SMP
/*
** Called from sysd.
*/
static uint32_t
sysd_create_isr_or_hook_helper(OSVECTOR vec, PROCESS owner, void *isr,
                               void *arg, uint8_t type, OSERRCODE *error,
                               cpuid_t cpu)
{
  struct vectorcb *vcb;
  uint32_t handle;

  LOCK_SWAP();
  vcb = get_vectorcb(vec, 1);
  
  /* If the vector is disabled already (or not bound to any cpu at
   * all, or to cpu 0 only where we are now, and we are attempting to
   * install the new one on cpu 0 as well), we can install without
   * any problem. */
  if ((vcb->disabled > 0) || ((vcb->cpus < 2) && (cpu == 0)))
    {
      handle = create_isr_or_hook_helper(vcb, owner, isr, arg, type, error, cpu);
      UNLOCK_SWAP();
      return handle;
    }

  /* Disabling an active vector source is something we would like to
   * avoid if possible since it will be very bad for the interrupt
   * latency on that vector. If it is bound to one cpu only, we can
   * perform the work on the correct core through sysd_lite. */
  if ((vcb->cpus == 0) || (vcb->cpus == (1u << cpu)))
    {
      PROCESS helper = zzcpu[cpu].sysd_lite_;
      static const SIGSELECT sig_create_isr[] = {1, SMP_CREATE_ISR_REPLY};      
      union SIGNAL *sig;
      
      UNLOCK_SWAP();      
      sig = alloc(sizeof(struct smp_create_isr), SMP_CREATE_ISR_REQUEST);
      sig->smp_create_isr.vec = vec;
      sig->smp_create_isr.owner = owner;
      sig->smp_create_isr.isr = isr;
      sig->smp_create_isr.arg = arg;
      sig->smp_create_isr.type = type;
      sig->smp_create_isr.cpu = cpu;

      send(&sig, helper);
      do {sig = receive_from(~0u, sig_create_isr, helper);} while(sig == NIL);
      *error = sig->smp_create_isr.error;
      handle = sig->smp_create_isr.handle;
      free_buf(&sig);
      
      return handle;
    }
  
  /* The vector is active, we have to disable it before we can do
   * anything with it, since we cannot lock interrupts on all cores it
   * may be active on. */
  UNLOCK_SWAP();

  /* We know that the vector will not be enabled or disabled while we
   * are executing, since these operations are performed through sysd as
   * well. */
  sysd_disable_vector_helper(vec);

  /* Now we can update the data structures from any core, such as this one. */
  LOCK_SWAP();
  vcb = get_vectorcb(vec, 0);
  
  handle = create_isr_or_hook_helper(vcb, owner, isr, arg, type, error, cpu);

  /* Re-enable the vector. */
  UNLOCK_SWAP();
  sysd_enable_vector_helper(vec);

  return handle;
}
#endif

/*
** Helper function to zzkrn_create_isr and zzkrn_create_interrupt_hook.
*/
static uint32_t
create_isr_or_hook(OSVECTOR vec, PROCESS owner, void *isr, void *arg, uint32_t type)
{
  OSERRCODE errcode;
  uint32_t handle;

  /* Sanity check parameters. */
  if (isr == NULL)
    {
      errcode = OSE_EBAD_PARAMETER;
      handle = 0;
      goto done;
    }

#ifdef OSE_SMP
  /* We may have to call disable and enable vector if the vector is
   * active on more than one cpu, since disabling interrupts cannot
   * help us. We will continue in sysd context to avoid being killed
   * during this operation, since we will have to unlock the dispatcher. */
  {
    static const SIGSELECT sig_create_isr[] = {1, SMP_CREATE_ISR_REPLY};
    union SIGNAL *sig;
      
    /* Enter superuser mode to allocate from the system pool. */
    set_suid((OSBOOLEAN) 1);
      
    sig = alloc(sizeof(struct smp_create_isr), SMP_CREATE_ISR_REQUEST);
    sig->smp_create_isr.vec = vec;
    sig->smp_create_isr.owner = owner;
    sig->smp_create_isr.isr = isr;
    sig->smp_create_isr.arg = arg;
    sig->smp_create_isr.type = type;
    sig->smp_create_isr.cpu = zzcur_p->block->cpu_id;
      
    send(&sig, zzuu.sysd_);
    do {sig = receive_from(~0u, sig_create_isr, zzuu.sysd_);} while(sig == NIL);
      
    /* Exit superuser mode. */
    set_suid((OSBOOLEAN) 0);

    /* Update status variables and return. */
    errcode = sig->smp_create_isr.error;
    handle = sig->smp_create_isr.handle;
    free_buf(&sig);
    goto done;
  }  
#else  
  /* Fetch the vector control block, allocate a new if necessary. */
  LOCK_SWAP();
  {
    struct vectorcb *vcb = get_vectorcb(vec, 1);

  /* Perform the installation. */
  handle = create_isr_or_hook_helper(vcb, owner, isr, arg, type, &errcode, 0);
  }
  UNLOCK_SWAP();
#endif

 done:  
  /* Check for problems. */
  if (errcode != 0)
    {
      /* Throw a non-fatal error with correct function subcode. */
      if (type == 0)
        os_error(0, errcode | OSE_CREATE_INTERRUPT_HOOK, (OSERRCODE) vec);
      else
        os_error(0, errcode | OSE_CREATE_ISR, (OSERRCODE) vec);
      return 0;
    }
  return handle;
}

/*
** This is the ose_create_isr system call.
*/
OSISRHND
zzkrn_create_isr(OSVECTOR vec, PROCESS owner, OSISR *isr, void *isr_arg)
{
  return (OSISRHND)create_isr_or_hook(vec, owner, CAST(void *,isr), isr_arg, 1);
}

/*
** This is the ose_create_interrupt_hook system call.
*/
static OSIHHND
zzkrn_create_interrupt_hook(OSVECTOR vec, PROCESS owner, OSIH *ih, void *ih_arg)
{
  return (OSIHHND)create_isr_or_hook(vec, owner, CAST(void *,ih), ih_arg, 0);
}

/*
** Performs the actual killing of an ISR or an interrupt hook. Must be
** called from the correct core. Can be called from sysd_lite. Called
** with locked dispatcher. */
static void
kill_isr_or_hook_helper(struct vectorcb *vcb, uint8_t slot, uint8_t type)
{
  /* If this is the last ISR, disable the vector. */
  if (type == 1)
    {
      if ((vcb->disabled == 0) && (vcb->isrcount == 1))
        {
          /* Don't call disable if it is not actually bound anywhere. */
          if (vcb->cpus > 0)
            {
              zzdisable_vector(vcb->vecno);
            }
        }
    }
  
  /* Unregister the ISR from the target interface. */
  zzrelease_vector_isr(vcb->vecno, (void *)vcb->isr[slot].isr, vcb->isr[slot].arg, type);

  /* Detach from owner pid. */
  detach_isr_or_hook(vcb, slot);
  
  vcb->isr[slot].isr = NULL;

  if (type == 1)
    {
      vcb->isrcount--;
      /* If this is the last ISR, also remove all hooks. */
      if (vcb->isrcount == 0)
        {
          clear_vectorcb_hooks(vcb);
        }
    }
}

#ifdef OSE_SMP
/*
** SMP Helper function to kill_isr_or_hook, called from sysd.
*/
static void
sysd_kill_isr_or_hook_helper(OSVECTOR vec, uint32_t slot, uint32_t type, uint32_t generation)
{
  struct vectorcb *vcb;
  cpuid_t cpu;

  /* Fetch the vectorcb, it will always exist since we would not be
   * called otherwise. */
  LOCK_SWAP();
  vcb = get_vectorcb(vec, 0);

  /* Make sure the ISR slot is still valid, it could have been killed
   *  by someone else by now.  */
  if (vcb->isr[slot].generation != generation)
    {
      UNLOCK_SWAP();
      return;
    }
  
  /* If the vector is disabled already (or not bound to any cpu at
   * all, or only cpu 0 where we are now), we can just kill it. */
  if ((vcb->disabled > 0) || (vcb->cpus < 2))
    {
      kill_isr_or_hook_helper(vcb, slot, type);
      UNLOCK_SWAP();
      return;
    }

  /* Disabling an active vector source is something we would like to
   * avoid if possible since it will be very bad for the interrupt
   * latency on that vector. If it is bound to one cpu only, we can
   * perform the work on the correct core through sysd_lite. */
  cpu = 31u - FIND_FIRST_BIT(vcb->cpus);
  if (vcb->cpus == (1u << cpu))
    {
      PROCESS helper = zzcpu[cpu].sysd_lite_;
      static const SIGSELECT sig_kill_isr[] = {1, SMP_KILL_ISR_REPLY};      
      union SIGNAL *sig;
      
      UNLOCK_SWAP();      
      sig = alloc(sizeof(struct smp_kill_isr), SMP_KILL_ISR_REQUEST);
      sig->smp_kill_isr.vec = vec;
      sig->smp_kill_isr.slot = slot;
      sig->smp_kill_isr.type = type;
      sig->smp_kill_isr.generation = generation;

      send(&sig, helper);
      do {sig = receive_from(~0u, sig_kill_isr, helper);} while(sig == NIL);
      free_buf(&sig);
      
      return;
    }
  
  /* The vector is active, we have to disable it before we can do
   * anything with it, since we cannot lock interrupts on all cores it
   * is active on. */
  UNLOCK_SWAP();

  /* We know that the vector will not be enabled or disabled while we
   * are executing, since these operations are performed through sysd as
   * well. */
  sysd_disable_vector_helper(vec);

  /* Now we can update the data structures from any core, such as this one. */
  LOCK_SWAP();
  vcb = get_vectorcb(vec, 0);
  
  kill_isr_or_hook_helper(vcb, slot, type);

  /* It's possible that there are other ISRs remaining though, so
   * reenable the vector in that case. */
  UNLOCK_SWAP();
  sysd_enable_vector_helper(vec);
}
#endif

/*
** Helper function to zzkrn_kill_isr and zzkrn_kill_interrupt_hook.
*/
static void
kill_isr_or_hook(OSISRHND isr_hnd, uint8_t type)
{
  struct vectorcb *vcb;
  OSVECTOR vecno;
  uint8_t slot;
  uint16_t gen;

  /* Interpret the ISR handle */
  vecno = (isr_hnd & 0xffff0000) >> 16;
  slot = (isr_hnd & 0x0000000f);
  gen = (isr_hnd & 0x0000fff0) >> 4;
  
  /* Fetch the indicated vectorcb. */
  LOCK_SWAP();
  vcb = get_vectorcb(vecno, 0);

  /* Check that the given ISR slot is valid. */
  if ((vcb == NULL) ||
      (slot >= 7) ||
      (vcb->isr[slot].isr == NULL) ||
      (vcb->isr[slot].type != type) ||
      (vcb->isr[slot].generation != gen))
    {
      UNLOCK_SWAP();

      /* Invalid handles are silently ignored. */
      return;
    }
 
#ifdef OSE_SMP

  /* We may have to call disable and enable vector if the vector is
   * active on more than one cpu, since disabling interrupts cannot
   * help us. We will continue in sysd context to avoid being killed
   * during this operation, since we will have to unlock the dispatcher. */
  {
    static const SIGSELECT sig_kill_isr[] = {1, SMP_KILL_ISR_REPLY};
    union SIGNAL *sig;

    UNLOCK_SWAP();
    
    /* Enter superuser mode to allocate from the system pool. */
    set_suid((OSBOOLEAN) 1);

    sig = alloc(sizeof(struct smp_kill_isr), SMP_KILL_ISR_REQUEST);
    sig->smp_kill_isr.vec = vecno;
    sig->smp_kill_isr.slot = slot;
    sig->smp_kill_isr.generation = gen;
    sig->smp_kill_isr.type = type;
      
    send(&sig, zzuu.sysd_);
    do {sig = receive_from(~0u, sig_kill_isr, zzuu.sysd_);} while(sig == NIL);
      
    /* Exit superuser mode. */
    set_suid((OSBOOLEAN) 0);
    free_buf(&sig);

    return;
  }
#else
  /* Perform the killing. */
  kill_isr_or_hook_helper(vcb, slot, type);
  UNLOCK_SWAP();
#endif
}

/*
** This is the ose_kill_isr system call.
*/
static void
zzkrn_kill_isr(OSISRHND isr_hnd)
{
  kill_isr_or_hook(isr_hnd, 1);
}

/*
** This is the ose_kill_interrupt_hook system call.
*/
static void
zzkrn_kill_interrupt_hook(OSIHHND ih_hnd)
{
  kill_isr_or_hook(ih_hnd, 0);
}

/*
** Walks an attach list of ISRs and kills them. Always called from
** sysd context with unlocked dispatcher.
*/
static void
process_isr_attach_list(struct isrcb *isr_list)
{
  struct isrcb *isr;
  struct isrcb *next;

  /* Attach will not be accessing this list, since it will not
   * interfere with pcb's that have been marked as KILLED. */
  isr = isr_list;
  while (isr)
    {
      /* Fetch the next entry before killing. */
      next = isr->next;
      
#ifdef OSE_SMP
      sysd_kill_isr_or_hook_helper(isr->vcb->vecno, isr->slot, isr->type, isr->generation);
#else
      LOCK_SWAP();
      kill_isr_or_hook_helper(isr->vcb, isr->slot, isr->type);
      UNLOCK_SWAP();
#endif      
      isr = next;
    }
}

#ifdef OSE_SMP

/*
** This is a helper process to the ose_bind_vector system call. It is
** called from sysd context. Returns 1 if ok, otherwise 0.
*/
static OSBOOLEAN
sysd_bind_vector_helper(OSVECTOR vec, cpuid_t cpu)
{
  struct vectorcb *vcb;
  
  LOCK_SWAP();
  vcb = get_vectorcb(vec, 0);
  if ((!vcb) || (vcb->isrcount == 0))
    {
      /* ERROR! Cannot bind a vector without ISRs. */
      UNLOCK_SWAP();
      return 0;
    }
    
  /* Check if this is a change, otherwise this is a NOP. */
  if ((vcb->cpus & (1U << cpu)) == 0)
    {
      /* Mark it as bound to this cpu. */
      vcb->cpus |= (1U << cpu);      
      
      /* Now check if we should perform the enable call here. */
      if (vcb->disabled == 0)
        {
          /* We may have to perform it on a different core. Sysd is
             always core 0. */
          if (cpu != 0)
            {
              PROCESS helper = zzcpu[cpu].sysd_lite_;
              static const SIGSELECT sig_zzenable_vector[] = {1, SMP_ZZENABLE_VECTOR_REPLY};
              union SIGNAL *sig;
      
              UNLOCK_SWAP();

              /* Enter superuser mode to allocate from the system pool. */
              set_suid((OSBOOLEAN) 1);
      
              sig = alloc(sizeof(struct smp_zzenable_vector), SMP_ZZENABLE_VECTOR_REQUEST);
              sig->smp_zzenable_vector.vec = vec;
              sig->smp_zzenable_vector.pri = vcb->priority;
      
              send(&sig, helper);
              do {sig = receive_from(~0u, sig_zzenable_vector, helper);} while(sig == NIL);
      
              /* Exit superuser mode. */
              set_suid((OSBOOLEAN) 0);
              free_buf(&sig);
              return 1;
            }
          else
            {
              /* Just do it locally. */
              zzenable_vector(vec, vcb->priority);              
            }
        }
    }
  
  UNLOCK_SWAP();
  return 1;
}

/*
** This is the ose_bind_vector system call.
*/
static void
zzkrn_bind_vector(OSVECTOR vec, cpuid_t cpu)
{
  /* Send the job to sysd to serialize it with the other ISR related calls. */
  static const SIGSELECT sig_bind_vector[] = {1, SMP_BIND_VECTOR_REPLY};
  union SIGNAL *sig;
      
  /* Enter superuser mode to allocate from the system pool. */
  set_suid((OSBOOLEAN) 1);
      
  sig = alloc(sizeof(struct smp_bind_vector), SMP_BIND_VECTOR_REQUEST);
  sig->smp_bind_vector.vec = vec;
  sig->smp_bind_vector.cpu = cpu;
      
  send(&sig, zzuu.sysd_);
  do {sig = receive_from(~0u, sig_bind_vector, zzuu.sysd_);} while(sig == NIL);

  /* Check for errors. */
  if (!sig->smp_bind_vector.success)
    {
      os_error(0, OSE_EVECTOR_NOT_REBINDABLE | OSE_BIND_VECTOR, (OSERRCODE) vec);	/* Non fatal. */      
    }
  
  /* Exit superuser mode. */
  set_suid((OSBOOLEAN) 0);
  free_buf(&sig);
}

/*
** This is the ose_unbind_vector system call.
*/
static OSBOOLEAN
sysd_unbind_vector_helper(OSVECTOR vec, cpuid_t cpu)
{
  struct vectorcb *vcb;

  LOCK_SWAP();
  vcb = get_vectorcb(vec, 0);
  if ((!vcb) || (vcb->isrcount == 0))
    {
      /* ERROR! Cannot bind a vector without ISRs. */
      UNLOCK_SWAP();
      return 0;
    }

  /* Check if this is a change, otherwise this is a NOP. */
  if ((vcb->cpus & (1U << cpu)) != 0)
    {
      /* Flag the vcb as not bound to this cpu any more. */
      vcb->cpus &= ~(1U << cpu);

      /* Now check if we should perform the disable call here. */
      if (vcb->disabled == 0)
        {
          /* We may have to perform it on a different core. */
          if (cpu != zzkrn_cpu_id())
            {
              PROCESS helper = zzcpu[cpu].sysd_lite_;
              static const SIGSELECT sig_zzdisable_vector[] = {1, SMP_ZZDISABLE_VECTOR_REPLY};
              union SIGNAL *sig;
      
              UNLOCK_SWAP();

              /* Enter superuser mode to allocate from the system pool. */
              set_suid((OSBOOLEAN) 1);
      
              sig = alloc(sizeof(struct smp_zzdisable_vector), SMP_ZZDISABLE_VECTOR_REQUEST);
              sig->smp_zzdisable_vector.vec = vec;
      
              send(&sig, helper);
              do {sig = receive_from(~0u, sig_zzdisable_vector, helper);} while(sig == NIL);
      
              /* Exit superuser mode. */
              set_suid((OSBOOLEAN) 0);
              free_buf(&sig);
              return 1;
            }
          else
            {
              /* Just do it locally. */
              zzdisable_vector(vec);
            }
        }
    }
  
  UNLOCK_SWAP();
  return 1;
}

static void
zzkrn_unbind_vector(OSVECTOR vec, cpuid_t cpu)
{
  /* Send the job to sysd to serialize it with the other ISR related calls. */
  static const SIGSELECT sig_unbind_vector[] = {1, SMP_UNBIND_VECTOR_REPLY};
  union SIGNAL *sig;
      
  /* Enter superuser mode to allocate from the system pool. */
  set_suid((OSBOOLEAN) 1);
      
  sig = alloc(sizeof(struct smp_unbind_vector), SMP_UNBIND_VECTOR_REQUEST);
  sig->smp_unbind_vector.vec = vec;
  sig->smp_unbind_vector.cpu = cpu;
      
  send(&sig, zzuu.sysd_);
  do {sig = receive_from(~0u, sig_unbind_vector, zzuu.sysd_);} while(sig == NIL);

  /* Check for errors. */
  if (!sig->smp_unbind_vector.success)
    {
      os_error(0, OSE_EVECTOR_NOT_REBINDABLE | OSE_UNBIND_VECTOR, (OSERRCODE) vec);	/* Non fatal. */      
    }
  
  /* Exit superuser mode. */
  set_suid((OSBOOLEAN) 0);
  free_buf(&sig);  
}

#endif

/*
**===========================================================================
** 11.4  Undocumented system calls.
**===========================================================================
*/

/*
** Access of the handlerp structure - see pose.h.
*/

void *zzkrn_get_handler(OSADDRESS index)
{
  switch(index)
    {
    case 12:
      return (void *) handlerp.zzprof_alloc;

    case 13:
      return (void *) handlerp.zzprof_free_buf;

    case 14:
      return (void *) handlerp.zzmsg_enqueue;

    case 15:
      return (void *) handlerp.zzmsg_dequeue;

    default: 
      break;
    } /* switch */
  
  return NULL;
}

void zzkrn_set_handler(OSADDRESS index, void *handler)
{
  switch(index)
    {
    case 12:
      handlerp.zzprof_alloc = (void (*)(OSBUFSIZE, void *, struct debug_pcb *, PROCESS))handler;
      return;

    case 13:
      handlerp.zzprof_free_buf = (void (*)(OSBUFSIZE, void *, struct debug_pcb *, PROCESS))handler;
      return;

    case 14:
      handlerp.zzmsg_enqueue = (int  (*)(cpuid_t,
                                         const struct ose_krn_msg *,
                                         uint32_t))handler;
      return;

    case 15:
      handlerp.zzmsg_dequeue = (uint32_t (*)(struct ose_krn_msg *,
                                             uint32_t))handler;
      return;

    default: 
      break;
    } /* switch */
}

/*
** Access of per process debug_info of the current process - see pose.h.
*/

unsigned char *
zzfind_current_debug_info(void)
{
  if(zzcur_p)
    {
      return (unsigned char *)&zzcur_p->debug_info;
    }
  return (U8 *) NULL;
}

/*
** The ldm reads the parent of a process.
**
** The creator of a specified process is returned.
** If the returned pid = 0, then the parent is unknown.
*/

PROCESS
zzget_parent(PROCESS pid)
{
  register struct _pcb *pcb;

  /* Get pcb and check instance number. */
  BKL_LOCK();
  pcb = pid_to_pcb(pid);
  if(pcb_to_pid(pcb) == pid)
    {
      PROCESS parentpid = pcb->parentpid;
      BKL_UNLOCK();
      return parentpid;
    }

  /* return zero if instance numbers do not match. */
  BKL_UNLOCK();
  return (PROCESS) 0;
}

/*
** This is an experimental system call that can set the priority
** of another process.
**
** It works as set_pri(), but any process id can be supplied.
**
** This call was requested by a university for experimental
** reasons, but it seems to have caught on in a wider perspective now.
*/

OSPRIORITY 
set_pri_for(PROCESS pid, OSPRIORITY newpri)
{
  register PCB_INSTANCE_T instance;
  register struct _pcb *pcb;

  /* Convert to set_pri for current process. */
  if(pid == current_process())
    return set_pri(newpri);

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
    if(check_nonint_caller(OSE_SET_PRI_FOR))
      return (OSPRIORITY) 0; /* We can't return "pcb->priority" in a safe way here. */

  /* Check that the new priority is valid. */
    if(unlikely(newpri > 31))
    {
      os_error(0, OSE_EBAD_PARAMETER | OSE_SET_PRI_FOR, (OSERRCODE) newpri);	/* Non fatal. */
      return (OSPRIORITY) 0; /* We can't return "pcb->priority" in a safe way here. */
    }
#endif

  /* Read instance number and check the process id. Report error if invalid. */
  pcb = pid_to_pcb(pid);
  instance = pid_to_instance(pid);
  if(pcb_to_instance(pcb) != instance)
    return newpri;

  /* Check that the target is a prioritized process. */
  if(unlikely(pcb->priority > 31))
  {
    os_error(0, OSE_EBAD_PARAMETER | OSE_SET_PRI_FOR, (OSERRCODE) pid);	/* Non fatal. */
    return (OSPRIORITY) 0; /* We can't return "pcb->priority" in a safe way here. */
  }

  /* Disable interrupts. */
  LOCK();

  /* Check that the block/process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      UNLOCK();
      return newpri; /* Pretend that the call worked on a killed process. */
    }

#ifdef OSE_SMP
      /* If the process runs on another cpu, let that one do the job. */
  {
    cpuid_t cpu = pcb->cpu_id;
    
    if(cpu != zzkrn_cpu_id())
      {
	send_krn_event(cpu, KRN_EVT_SET_PRI_FOR, pid, (void *)(newpri<<16));
	UNLOCK();
	return newpri;
      }
  }
#endif

  /* Remove the process from the ready list if it is ready. */
  make_otherpri_not_ready(pcb);

  /* Set a new priority in the pcb. */
  pcb->priority = newpri;
  pcb->ready_entry.rlist = (struct ready_list_entry *) &zzww.ready_list[newpri];

  /* Run the process at the new priority if it is ready. */
  if(pcb->status < ST_READY)
    make_otherpri_ready(pcb); /* swap */

  /* Enable interrupts and quit. */
  UNLOCK();
  return newpri;
}

/*
** Replace the redirection table of a process.
*/

void
set_redirection(PROCESS pid, struct OS_redir_entry *redir)
{
  struct OS_redir_entry *oldredir;
  struct OS_redir_entry *newredir;
  struct _pcb *pcb;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller((OSERRCODE) 0))
    return;
#endif

  /*
  ** Check for the NULL pointer.
  */
  if(!redir)
    {
      os_error(0, OSE_EILLEGAL_REDIRCOUNT | OSE_SET_REDIRECTION, (OSERRCODE) 0); /* Non fatal. */
      return;
    }

  /* Locate the process descriptor. */
  /* Read the instance number and check the process id. Report error if invalid. */
  pcb = check_pid(pid, (OSERRCODE) 0);
  if(pcb == NULL)
    {
      return;
    }

  /*
  ** Allocate a redirection table buffer in the system pool.
  ** Copy the redirection table to the system buffer.
  */
  {
    OSBUFSIZE redir_len = redir->sig * sizeof(struct OS_redir_entry);
    /* Make sure that the redirection table can be fitted in the system pool. */
    if(!redir_len || system_pool.signal_list[7].size < (OSADDRESS) redir_len)
      {
	os_error(0, OSE_EBAD_PARAMETER | OSE_SET_REDIRECTION, (OSERRCODE) redir); /* Non fatal. */
	return;
      }
    newredir = (struct OS_redir_entry *)
      sys_alloc(redir_len, (SIGSELECT) 0);
    memcpy((void *) newredir, (void *) redir, redir_len ? (size_t) redir_len : 1);
    redir = newredir;
  }

  /*
  ** Check that all process id's in the redirection table are legal from
  ** the point of view that they refer to either a killed or
  ** present process.
  ** Block id's are illegal.
  ** Redirection through a block id could be a way to create hot program handover ??.
  ** See also: the same test in create_process()
  */
  {
    OSADDRESS count = redir->sig;
    PROCESS defaultpid = redir->pid;

    /* Check that the count is nonzero. */
    if(count)
      count--;
    else
	{
	  os_error(0, OSE_EILLEGAL_REDIRCOUNT | OSE_SET_REDIRECTION, (OSERRCODE) 0); /* Non fatal. */
	  free_buf_type(struct OS_redir_entry *, &redir);
	  return;
	}

    /* Check that the default pid is valid or zero. */
    if(defaultpid)
      {
	if(!check_pid(defaultpid, (OSERRCODE) 0))
	  {
	    free_buf_type(struct OS_redir_entry *, &redir);
	    return;
	  }
      }
    /* If it is zero, replace it with the target pid. */
    else
      {
	redir->pid = pid; /* defaultpid */
      }
    
    /* Check each pid in the list to see if it is a valid process id. */
    for(newredir = redir + 1; count; newredir++, count--)
      {
	if(!check_pid(newredir->pid, (OSERRCODE) 0))
	  {
	    /*
	    ** At this point, we need to find out if the table contained an
	    ** illegal pid, or if it just contains a dead pid.
	    ** If a pid was illegal, the we should just return.
	    ** If no pid was illegal, then we should kill the target process
	    ** since the spec says that all processes involved in a dead chain
	    ** should be killed.
	    */
	    count = redir->sig;
	    for(newredir = redir + 1; count; newredir++, count--)
	      if(get_ptype(newredir->pid) == OS_ILLEGAL)
		{
		  /* Free the new redirection table. */
		  free_buf_type(struct OS_redir_entry *, &redir);
		  return;
		}

	    /* Kill the process that now would have a bad redirection table. */
	    kill_proc(pid);
	    /* Free the new redirection table. */
	    free_buf_type(struct OS_redir_entry *, &redir);
	    return;
	  }
      }
  }

  /* Lock the dispatcher. */
  LOCK_SWAP();
  
  /* Check that the target process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      UNLOCK_SWAP();
      free_buf_type(struct OS_redir_entry *, &redir);
      return;
    }

  /* Set the owner of the new redirection buffer. */
  change_sig_owner(sig_to_adm(redir), pid_to_pcb(zzuu.sysd_));

  /* Disable interrupts. */
  LOCK();

  /* Get the old buffer. */
  oldredir = pcb->redir;

  /* Activate the new buffer. */
  pcb->redir = redir;

  /* Get the old buffer and change its ownership. */
  if(oldredir)
    change_sig_owner(sig_to_adm(oldredir), zzcur_p);

  /* Unlock the dispatcher and enable interrupts. */
  LOCKED_UNLOCK_SWAP();

  /* Free the old redirection buffer. */
  if(oldredir)
    free_buf_type(struct OS_redir_entry *, &oldredir);
}

/*
** This is the undocumented kill_lock() system call.  The purpose of
** this system call is to make sure that a given process won't be
** killed while it's in "kill locked" state.
**
** A process can be locked more than once, which means that it must be
** unlocked the same number of times before it leaves the "kill
** locked" state.
** 
** The pid parameter contains the id of the process to lock.  Setting
** the pid parameter to zero is equivalent to using the pid of the
** current process.
**
** Returns a nonzero value if the call successfully locked the process.
** 
*/

OSBOOLEAN
zzkrn_kill_lock(PROCESS pid)
{
  LOCKSTATE msr;

  BKL_LOCK_SAVE(msr);
  if(pid == 0)
    {
      zzcur_p->lock_cnt++;
      BKL_LOCK_RESTORE(msr);
      return 1;
    }
  else
   {
     struct _pcb *pcb = safe_pid_to_pcb(pid, 0);

     if (pcb != NULL)
       {
	 pcb->lock_cnt++;
	 BKL_LOCK_RESTORE(msr);
	 return 1;
       }
   }
  BKL_LOCK_RESTORE(msr);
  (void)check_pid(pid, 0);
  return 0;
}

/*
** This is the undocumented kill_unlock() system call.  The purpose of
** this system call is to decrement a process' "kill locked" counter
** and check if it result is zero.  If so this function will check if
** anyone has tried to kill the process while it was locked, in which
** case this function will now call kill_proc() upon the former "kill
** locked" process.
** 
** A process can be locked more than once, which means that it must be
** unlocked the same number of times before it leaves the "kill
** locked" state.
**
** The pid parameter contains the id of the process to unlock.
** Setting the pid parameter to zero is equivalent to using the pid of
** the current process.
*/

void
zzkrn_kill_unlock(PROCESS pid)
{
  LOCKSTATE msr;
  struct _pcb *pcb;

  BKL_LOCK_SAVE(msr);
  if(pid == 0)
    {
      pcb = zzcur_p;
      pid = pcb->pid;
    }
  else
   {
     pcb = safe_pid_to_pcb(pid, 0);

     if(unlikely(pcb == NULL))
       {
	 BKL_LOCK_RESTORE(msr);
	 os_error(0, OSE_EILLEGAL_PROCESS_ID, (OSERRCODE)pid);/* Non fatal. */
	 return;
       }
   }

  if(unlikely(pcb->lock_cnt == 0))
    {
      BKL_LOCK_RESTORE(msr);
      os_error(0, OSE_EBAD_PARAMETER, (OSERRCODE) pid);	/* Non fatal. */
      return;
    }
  else if(--pcb->lock_cnt == 0)
    {
      if(pcb->locked_kill)
	{
	  BKL_LOCK_RESTORE(msr);

	  /*
	  ** Someone tried to kill the proc while it where locked.
	  ** Better make that someone happy...
	  */
	  kill_proc(pid);
	}
    }
  BKL_LOCK_RESTORE(msr);
}

/*
** This is the undocumented kill_curr_at_unlock() system call.  The
** purpose of this system call is to mark the current process so it
** will kill itself as soon as it leaves its "kill locked" state.
**
** The need for this function arose when trying to solve some tricky
** cleanup issues.  The the public system call kill_proc() can not be
** used since it will report an error if the current process tries to
** kill itself when it is locked.
**
** It is only allowed to call this function when the current process
** is locked, otherwise it will report an error.
*/

void
zzkrn_kill_curr_at_unlock(void)
{
  LOCKSTATE msr;
  struct _pcb *pcb = zzcur_p;

  BKL_LOCK_SAVE(msr);
  if(pcb->lock_cnt != 0)
    {
      pcb->locked_kill = 1;
      BKL_LOCK_RESTORE(msr);
      return;
    }
  BKL_LOCK_RESTORE(msr);
  os_error(0, OSE_EILLEGAL_SYSTEMCALL | OSE_KILL_PROC, (OSERRCODE) 0);	/* Non fatal. */
}

/*
*****************************************************************************
** 12 POOL MANAGEMENT.
*****************************************************************************
*/

/*
** There has been a number of suggestions on how OSE could be improved to
** handle memory shortage more gracefully.
** I have made a summary below.
**
** Only suggestion A) has been implemented in this kernel since I feel that
** this is the only reasonable proposal. It is fairly efficient, and there
** are no dangerous race conditions that a user might run into.
**
** There are a number of possible ways in which memory shortage
** can be managed.
** 
** A)
** 
** An error handler frees some memory or creates new space.
** 
** When a memory shortage occurs, a warning is generated for each
** process that fails. The error handler is passed a sequence number
** stating the number of times the error has been managed.
** 
** The kernel does not perform any process switches while the error handler
** is executing, so the error handler can be multiply called only by
** interrupt processes.
** 
** The first error handler called is responsible for managing the
** error by extending the pool or by freeing a sufficient number of
** signals or stacks.
** 
** These operations must all be performed with interrupts disabled if
** there is a chance that an interrupt process uses the same pool.
** 
** In case error processing is interrupted before interrupts were disabled,
** the error handler must test the sequence number after disabling interrupts.
** If the passed sequence number is not larger than the most previous sequence
** number managed by the error handler, the error handler should return without
** doing anything. Else the error should be managed with interrupts disabled,
** and the last managed sequence number should be updated before interrupts
** are reenabled.
** 
** When the error has been processed, all involved processes retry
** the alloc operation. If error managing was successful, allocation
** succeeds. Otherwise a new warning is generated.
** 
** The error handler is not allowed to cause a process switch, i.e calls to
** receive, delay, wait_(f)sem etc are illegal. Send, free_buf and extend_pool
** are all OK to use.
** 
** B)
** 
** A memory warning is issued before space is actually exhausted.
** 
** An error handler may inform the entire system that a memory limit has
** been reached.
** 
** If space becomes completely exhausted in spite of the early warning,
** an error according to A) is reported.
** 
** When free memory again falls below the warning limit, a new warning is
** issued. An error handler may now inform the system that memory is
** once again available. This warning should not be propagated from the
** error handler to the rest of the system for at least a few
** system ticks in order to avoid heavy system load due to oscillation.
** 
** There are efficiency problems with this implementation. There must
** be a separate low water counter and limit variable for each buffer type
** and stack type.
** If any of these is exceeded, a memory low warning is issued.
** A memory OK warning can be issued only when all buffer types have
** again reached a normal free space level. Much kernel overhead is
** introduced in the alloc and free_buf system calls.
** 
** C)
** 
** The alloc system call is allowed to return NIL.
** 
** This policy causes horrible crashes whenever users fail to
** test on the value returned by alloc. (the malloc syndrome.)
** 
** Performance is badly hurt by all the if-statements scattered
** all over application code.
** 
** Program complexity becomes much higher because every process MUST
** have an "out of memory mode" which allows for reasonable system
** operation until memory is again available. It is very hard to detect
** cases where users "forgot" to test on the value returned.
** 
** Program testing becomes exceedingly expensive since program
** behaviour in every out of memory case must be separately verified.
** 
** D)
** 
** Processes simply hang until memory is freed by some other process.
** 
** This policy was tested in early OSE implementations, but was
** abandoned because serious deadlocks occurred nearly every time
** memory was exhausted.
** 
** The policy also does not handle interrupt processes,
** which can not wait.
** 
** E)
** 
** There is a new system call to check free space before attempting
** allocation.
** 
** The disadvantage is that there are several kinds of free space.
** One still does not know if a specific buffer or stack type is
** available until allocation is actually attempted.
** 
** Another problem is that higher priority processes may use up
** the free memory while a lower priority process is making the
** decision to allocate the memory.
** 
** This policy is not safe unless combined with something else.
** It is also just as complicated and expensive to use as E).
*/

/*
**===========================================================================
** 12.1  Initialization.
**===========================================================================
*/

  /* Read the system control block. */
static void
get_kernel_configuration(void)
{
  local_sysctlblk = *zzget_sysctlblk();
  zzuu.sysctlblk = &local_sysctlblk;

  /* Adjust control block fields not supported by the target interface. */
  if(local_sysctlblk.table_extended < 3)
    local_sysctlblk.bgd_timeslice = 10;
  if(local_sysctlblk.table_extended < 4)
    local_sysctlblk.enable_remcalls = 1;
  if(local_sysctlblk.table_extended < 5)
    {
      local_sysctlblk.max_ppd_keys = 32;
      local_sysctlblk.max_ppd_key_size = 32;
    }
  if(local_sysctlblk.table_extended < 6)
    {
      /* For old default behavoir, disable all added error checks. */
      local_sysctlblk.relax_error_checks = ~0u;
      
      local_sysctlblk.zzmain_stacksize = ZZMAIN_STACKSIZE;
    }

  zzuu.timeout_shift = 8;
  if(local_sysctlblk.table_extended < 8)
    {
      local_sysctlblk.tmo_array_length = 256;
    }
  else
    {
      OSADDRESS conf_length = local_sysctlblk.tmo_array_length;
      OSADDRESS selected_length;
      
      for(selected_length = 256; ; selected_length *= 2)
	{
	  if(selected_length * 2 > MAX_TIMEOUT_ARRAY ||
	     selected_length >= conf_length)
	    break;
	  zzuu.timeout_shift++;
	} /* for */
      local_sysctlblk.tmo_array_length = selected_length;
    }
  zzuu.curtmo_mask = local_sysctlblk.tmo_array_length - 1;

  /* Decide pool_alignment */
  zzuu.pool_alignment = (local_sysctlblk.table_extended < 10 ||
			 local_sysctlblk.cache_line_size == 0)
    ? 32 : local_sysctlblk.cache_line_size;

  /* Decide destiny of sent signals in killed pools. */
  {
    const char * const actions[] = {"?", "keep", "relocate", "kill"};

#if MEMORY_MODEL == 0
    local_sysctlblk.killed_pool_signal_action = 1;  /* keep */
    local_sysctlblk.killed_pool_signal_action_delay = 0;
#else
    if(local_sysctlblk.table_extended < 11)
      {
	local_sysctlblk.killed_pool_signal_action = 2;  /* relocate */
	local_sysctlblk.killed_pool_signal_action_delay = 0;
      }
    else  
      {
	if(local_sysctlblk.killed_pool_signal_action == 0) /* default */
	  local_sysctlblk.killed_pool_signal_action = 2;  /* relocate */
      }
#endif
    ramlog_printf("exec: memory_model=%d table_extend=%tu "
		  "queued_signal_action=%s\n",
		  MEMORY_MODEL, local_sysctlblk.table_extended,
		  actions[local_sysctlblk.killed_pool_signal_action &
			  (OSADDRESS)3]);
  }
  /*
  ** Make sure that fields in the system control block have
  ** reasonable values.
  ** There are probably more fields to check here. ??
  */
  {
     OSADDRESS badval = 0;
     if(local_sysctlblk.max_procs > 0xffff)
     {
	badval = (OSADDRESS) local_sysctlblk.max_procs;
	local_sysctlblk.max_procs = 0xffff;
     }
     if(local_sysctlblk.max_attached > 0xffff)
     {
	badval = (OSADDRESS) local_sysctlblk.max_attached;
	local_sysctlblk.max_attached = 0xffff;
     }

     /* Report an ignorable error if we had to adjust any value. */
     if(badval)
     {
	os_error(0, OSE_EILLEGAL_CONFIGURATION | OSE_START_OSE, (OSERRCODE) badval);	/* Non fatal. */
     }
  }
}

/*
** Helper for init_pool to break memory from the system pool.
** The object is pool aligned, as well as the next pool break pointer.
*/

static void *
early_syspool_brk_lo(OSADDRESS chunk)
{
  struct poolcb *pool = &system_pool;
  OSADDRESS pool_old_lo = pool->pool_low_p;
  OSADDRESS align_before =
    ((pool_old_lo + (POOL_ALIGNMENT-1))/* & (OSADDRESS) ~(POOL_ALIGNMENT-1)*/) -
    pool_old_lo;

  /* Prepare for pool alignment of the NEXT pool_low_p. */
  chunk = (chunk + (POOL_ALIGNMENT-1))/* & (OSADDRESS) ~(POOL_ALIGNMENT-1)*/;

  while((pool->pool_hi_p - pool->pool_low_p) <= chunk + align_before)
     os_error(2, OSE_ENO_KERN_SPACE | OSE_START_OSE,
	      (OSERRCODE) ((pool->extend_count & 0x7f) << 24) |
	      (OSERRCODE) pcb_to_pindex(pool)); /* Fatal system error. */
  pool->pool_low_p += chunk + align_before;
  return (void *)(pool_old_lo + align_before);
}

/*
** Create the system segment and the system pool.
*/

static void
init_pool(void)
{
  int i;
  OSADDRESS chunk;
  register struct poolcb *pool;
  register struct sysctlblk *p = zzuu.sysctlblk;
  struct pool_extension *ext;

  /* Clear the list of free stack control blocks. */

  zzuu.stkcb_free_list = NULL;

  /*
  ** Allocate new process indexes from 1.
  ** The first slot is reserved for the system pool since
  ** No process or block can use this index.
  */

  zzuu.next_free_pix = (PCB_INDEX_T) 1;
  zzuu.process_under_construction_ = (PROCESS) 0;

  /*
  ** Clear the list of free process control blocks,
  ** pool descriptors or block descriptors.
  **
  ** Please note that this implementation does not allow
  ** the system segment to have a (PROCESS) segment id since it is not
  ** in the zzuu.pix_list. Thus we can not easily implement a get_segid() ??
  ** system call or some such without returning zero for the system segment.
  ** All segment id's must be obtained from attach_segment().
  */

  zzuu.pcb_freelist = NULL;

  /*
  ** Clear the list of free signal administration blocks.
  */
  zzuu.free_sigadm_list = NULL;


  /* Clear the counter that keeps track of pool deletions. */
  zzuu.pool_delete_cnt = 0;

  /* Clear the pointer to the main pool's poolcb. */
  zzuu.main_pool = NULL;
  
  /* Create the system segment. This can not be killed. */

  zzuu.segment_list = &system_segment;
  system_segment.magic = MAGIC_SEG;
  system_segment.killed = 0;
#ifndef LARGE_PID_SPACE
  system_segment.pindex = (PCB_INDEX_T) 0;
  system_segment.instance = (PCB_INSTANCE_T) 0;
#else
  system_segment.pid = 0;
#endif
  system_segment.attached = NULL;
  system_segment.next = NULL;
  system_segment.segid = 0;
  system_segment.block_count = (U16) 1;
  system_segment.pool_count = (U16) 1;
  system_segment.highpri_count = (U16) 0;
  system_segment.visitors = (U16) 0;
  system_segment.mas_mapped = 0;
  system_segment.semlist = NULL;
  system_segment.mutlist = NULL;
  system_segment.main_block = 0;

  /*
  ** We must initially be able to move IPI processes to other cpus.
  ** We could inhibit migration of system segment processes in a later
  ** phase. Note however that PM must be able to migrate virgin blocks.
  */
  zzuu.prog_mgr_ = 0;
  system_segment.migration_inhibited = 0; 

  /* Get a register pointer for efficiency. */
  pool = &system_pool;

  /* Set up all simple fields in the system pool. */
  pool->magic = MAGIC_POOL;
#ifndef LARGE_PID_SPACE
  pool->pindex = (PCB_INDEX_T) 0;
  pool->instance = (PCB_INSTANCE_T) 1;
#else
  pool->pid = PCB_INDEX_MASK + 1;
#endif
  pool->next = NULL;
  SMP_CODE(spin_lock_init(&pool->smp_spin));
  pool->attached = NULL;
  pool->extend_count = 0;
  pool->siglist = (struct _signal *) &(pool->firstsig);
  pool->sigtail = (struct _signal *) &(pool->firstsig);
  pool->firstsig.siglist = (struct _signal *) NULL;
  pool->no_of_stack_users = (U16) 2;
  pool->no_of_sig_users = (U16) 2;
  pool->segid = 0;
  pool->segpid = (PROCESS) 0;
  pool->pool_low_p = (OSADDRESS) ((p->pool_base + (POOL_ALIGNMENT-1))/* & (OSADDRESS) ~(POOL_ALIGNMENT-1)*/);
  pool->pool_hi_p = (OSADDRESS) ((p->pool_base + p->pool_size)/* & (OSADDRESS) ~(POOL_ALIGNMENT-1)*/);
  pool->frag_size = 0;		/* Extend fragment size not known. */
  pool->max_size = 0;		/* Max size not set. */
  pool->tot_size = pool->pool_hi_p - pool->pool_low_p;  /* Current size. */
  pool->ext_func = NULL;
  pool->prog_type_p = NULL;
  pool->no_copy = 0;
  pool->is_static=0;
  pool->realtime_extendable = 1;
  pool->ext_frag_pp = NULL;
  pool->ext_alloc = NULL;
  pool->ext_free = NULL;
  pool->free_attempts = 0;

#if OSE_DEBUG > 0
  /* Reserve space for the system trace. */
  /* WARNING! The minimum size must be matched with the ldm code.!! */
  if(p->tracesize < 8192)
    {
      if((pool->pool_hi_p - pool->pool_low_p) <= 8192)
	os_error(2, OSE_ENO_KERN_SPACE | OSE_START_OSE,
		 (OSERRCODE) ((pool->extend_count & 0x7f) << 24) |
		 (OSERRCODE) pcb_to_pindex(pool));	/* Fatal system error. */
      else
	pool->pool_low_p += 8192;
    }
  else
    {
      if((pool->pool_hi_p - pool->pool_low_p) <= (p->tracesize /*& (OSADDRESS) ~(POOL_ALIGNMENT-1)*/) )
	os_error(2, OSE_ENO_KERN_SPACE | OSE_START_OSE,
		 (OSERRCODE) ((pool->extend_count & 0x7f) << 24) |
		 (OSERRCODE) pcb_to_pindex(pool));	/* Fatal system error. */
      else
        pool->pool_low_p += (p->tracesize/* & (OSADDRESS) ~(POOL_ALIGNMENT-1)*/);
    }
#endif

  if(local_sysctlblk.table_extended >= 9)
    {
      pool->fill = p->pool_fill;
    }
  else
    {
#if ERROR_CHECKS > 0
      pool->fill = 1; /* TRUE */
#else
      pool->fill = 0; /* FALSE */
#endif
    }
  if(pool->fill)
    {
       /* Initialize the pool to a debug pattern on each cold start. */
       if(!zzuu.start_mode)
         {
            memset((char *) p->pool_base, 0xcc, (size_t)(pool->pool_hi_p - p->pool_base));
         }
       else
         {
            memset((char *) pool->pool_low_p, 0xcc, (size_t)(pool->pool_hi_p - pool->pool_low_p));
         }
    }

  /* Build a fragment record for the system pool. */
  ext = (struct pool_extension *) pool->pool_low_p;
  ext->base = (OSADDRESS) ext;
  ext->limit = pool->pool_hi_p;
  chunk = (sizeof(struct pool_extension) + (POOL_ALIGNMENT-1))/* &
    (OSADDRESS) ~(POOL_ALIGNMENT-1)*/;
  if((pool->pool_hi_p - pool->pool_low_p) <= chunk)
    os_error(2, OSE_ENO_KERN_SPACE | OSE_START_OSE,
	     (OSERRCODE) ((pool->extend_count & 0x7f) << 24) |
	     (OSERRCODE)pcb_to_pindex(pool));	/* Fatal system error. */
  pool->pool_low_p += chunk;
  ext->frag_list = NULL;
  ext->magic_frag = MAGIC_POOL;
  pool->frag_list = ext;

  /* Set reasonable values for the final pool low and high pointers. */
  ext->pool_low_p = ext->base;
  ext->pool_hi_p = ext->limit;

  /* Build a fragment record for free list of fragment records.. */
  ext = (struct pool_extension *)
    early_syspool_brk_lo(sizeof(struct pool_extension));
  ext->frag_list = NULL;
  ext->magic_frag = MAGIC_POOL;
  zzuu.free_frag_list = ext;

  /* Set up free lists in the system pool. */
  {
    int highest_sig_i;
    int highest_stack_i;

#if ERROR_CHECKS > 0
    if(p->sigsize_tab[0])
      {
        /* Check that the size array is sorted on size. */
        for(i = 1; i < 8 ; i++)
	  {
	    if (p->sigsize_tab[i] < p->sigsize_tab[i - 1])
	      {
	        os_error(2, OSE_EUNREASONABLE_SIZES | OSE_START_OSE, 0);	/* Fatal system error. */
	      }
	  }
      }
    else
      {
	/* 0 is not a reasonable size... */
        os_error(2, OSE_EUNREASONABLE_SIZES | OSE_START_OSE, 0);	/* Fatal system error. */
      }
  
    if(p->stacksize_tab[0])
      {
        /* Check that the size array is sorted on size. */
        for(i = 1; i < 8 ; i++)
	  {
	    if (p->stacksize_tab[i] < p->stacksize_tab[i - 1])
	      {
	        os_error(2, OSE_EUNREASONABLE_SIZES | OSE_START_OSE, 0);	/* Fatal system error. */
	      }
	  }
      }
    else
      {
	/* 0 is not a reasonable size... */
        os_error(2, OSE_EUNREASONABLE_SIZES | OSE_START_OSE, 0);	/* Fatal system error. */
      }
#endif

    highest_sig_i = 0;
    highest_stack_i = 0;

    for(i = 0; i < 8; i++)
      {
	if(p->sigsize_tab[i])
	  {
	    highest_sig_i = i;
	  }

	if(p->stacksize_tab[i])
	  {
	    highest_stack_i = i;
	  }
	
	pool->signal_list[i].size = ((p->sigsize_tab[highest_sig_i] + 1 + (POOL_ALIGNMENT-1))/* & (OSADDRESS) ~(POOL_ALIGNMENT-1)*/) - 1;
	pool->stack_list[i].size = (p->stacksize_tab[highest_stack_i] + (POOL_ALIGNMENT-1)) /*& (OSADDRESS) ~(POOL_ALIGNMENT-1)*/;
	pool->signal_list[i].hd = NULL;
	pool->stack_list[i].hd = NULL;
	pool->signal_list[i].alloc_count = 0;
	pool->stack_list[i].alloc_count = 0;
	pool->signal_list[i].poolcbp = pool;
	pool->stack_list[i].poolcbp = pool;
	pool->signal_list[i].segid = 0;
	pool->signal_list[i].ext_handle = NULL;
      }
  }
  
  /* Initialize interrupt stack management. */
  zzww.interrupt_stack = NULL;
  zzuu.interrupt_stack_size = 0;
  zzuu.wanted_timer_size = 0;
  for(i = 0; i < 32; i++)
    zzuu.wanted_interrupt_size[i] = 0;

  /*
  ** Initialize global list of processes.
  ** Create kernel pool in process index table 0.
  ** This index is known to be "invalid" so it can not be used for any
  ** other purpose anyway.
  */
  zzuu.max_proc = local_sysctlblk.max_procs;
  zzuu.pix_list = (struct _pcb **)
    early_syspool_brk_lo(zzuu.max_proc * sizeof(struct _pcb *));
  zzuu.pix_list[0] = (struct _pcb *) pool;
  zzuu.pool_list = pool;

  /* Obtain space for the attach pool. */
  zzuu.max_attach = local_sysctlblk.max_attached;
  zzuu.attach_pool = (struct _attach_entry **)
    early_syspool_brk_lo(zzuu.max_attach * sizeof(struct _attach_entry *));

  /* Obtain space for the timeout list and clear it. */
  zzww.tmolist = early_syspool_brk_lo(local_sysctlblk.tmo_array_length *
				      sizeof *zzww.tmolist);
  memset(zzww.tmolist, 0,
	 local_sysctlblk.tmo_array_length * sizeof(*zzww.tmolist));


  /* Obtain space for the timeout signal per tick list. */
  zzuu.tos_tick_list = early_syspool_brk_lo(local_sysctlblk.tmo_array_length *
					    sizeof *zzuu.tos_tick_list);

  /* Obtain space for per process data handling. */
  zzuu.max_ppd_keys = local_sysctlblk.max_ppd_keys;
  zzuu.max_ppd_key_size = local_sysctlblk.max_ppd_key_size;
  /* Include nul terminator and word align per process data names */
  zzuu.allocated_ppd_key_size =
    (local_sysctlblk.max_ppd_key_size + sizeof(OSADDRESS)) &
    (OSADDRESS) ~(sizeof(OSADDRESS) - 1);
  zzuu.ppd_key_buf  = (char *)
    early_syspool_brk_lo(zzuu.max_ppd_keys * zzuu.allocated_ppd_key_size);
  zzuu.ppd_hash_buf  = (char *)
    early_syspool_brk_lo(zzuu.max_ppd_keys * sizeof(*zzuu.ppd_hash_buf));

  /* Allocate the idle pcb */
  zzww.idle_pcb = (struct _pcb *)early_syspool_brk_lo(sizeof(union large_cb));
}

/*
**===========================================================================
** 12.2  Local functions.
**===========================================================================
*/

/*
** Subfunction to check_buffer().
**
** This function attempts to refine the error code by examining the damaged pool.
*/

#if ERROR_CHECKS > 0

#if MEMORY_MODEL > 0

/*
** This version works with complex memory models.
** In this case we expect to be able to recover from many types
** of errors, so we need to tread very carefully here.
**
** We do our best to find out if the problem is in the user pool or
** in the system pool. System pool errors are always fatal.
*/

struct _signal *
zzrecover_from_pool_crash(union SIGNAL *sig, OSERRCODE errcode)
{
  register struct _signal *ssigadm;
  register struct _signal *usigadm;
  struct poolcb *pool;
  struct pool_extension *frag;
  OSBOOLEAN fatal = 1;
  struct _signal *prev_buf;
  struct _signal *buf;
  volatile OSADDRESS deleted_pool;
  PCB_INSTANCE_T pool_instance;
  LOCKSTATE msr;

  /*
  ** Locate the pool that contains the bad buffer.
  ** Note that there might be several in various segments if we have
  ** allow pools to be in virtual memory.
  */
  do
    {
      frag = NULL;
      /* Keep track of any delete operations in the pool list. */
      deleted_pool = zzuu.pool_delete_cnt;

      for(pool = zzuu.pool_list; pool; pool = pool->next)
	{
	  /* Check that this pool looks ok. */
	  if(pool->magic != MAGIC_POOL)
	    os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 5);	/* Fatal. */
	  for(frag = pool->frag_list; frag; frag = frag->frag_list)
	    {
	      /* Check that this fragment looks ok. */
	      if(frag->magic_frag != MAGIC_POOL)
		os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 4);	/* Fatal. */
	      /* Check if the buffer is in this fragment. */
	      if(
		 (frag->base <= (OSADDRESS) sig) &&
		 (frag->limit > (OSADDRESS) sig)
		 )
		break;
	    }
	  if(frag)
	    break;
	}
      /*
      ** Keep track of the pool's instance number so we can check later
      ** if it is killed.
      */
      pool_instance = pcb_to_instance(pool);
      /*
      ** We need to restart the search loop if any pool has been deleted
      ** since we stared looping.
      */
    }
  while(deleted_pool != zzuu.pool_delete_cnt);

  /*
  ** Report an error if the pool could not be found. This means a wild
  ** pointer.
  */
  if(!frag)
    {
      os_error(0, OSE_EWILD_SIG_POINTER | errcode, (OSERRCODE) sig);	/* Non fatal. */
      return NULL;
    }

  /* Errors reported should be fatal if the system pool was found. */
  if(pool == &system_pool)
    fatal = (OSBOOLEAN) 2;

  /* Pick up the user space sigadm pointer. */
  usigadm = ((struct _signal *) sig) - 1;

  /*
  ** Traverse the pool looking for the administration block that
  ** matches the buffer.
  */
  for(ssigadm = pool->siglist->siglist; ssigadm; ssigadm = ssigadm->siglist)
    {
      /* Check that this adm block is still in the system pool. */
      {
	for(frag = system_pool.frag_list; frag; frag = frag->frag_list)
	  {
	    /* Check if the adm block is in this fragment. */
	    if(
	       (frag->base <= (OSADDRESS) ssigadm) &&
	       (frag->limit > (OSADDRESS) ssigadm)
	       )
	      break;
	  }
	/*
	** Report a fatal error if the system pool was corrupted
	** before the current buffer.
	*/
	if(!frag)
	  {
	    /* We have a fatal error if the system pool was damaged. */
	    os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 14);	/* Fatal. */
	  }
      }
      /* Check the supervisor space magic number. */
      if(ssigadm->magic_signo != MAGIC_SIGNO)
	{
	  /* We have a fatal error if the system pool was damaged. */
	  os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 13);	/* Fatal. */
	}
      /* See if we found the correct adm block. */
      if(ssigadm->udata == (SIGSELECT *) sig)
	break;
    }

  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);
  
  /*
  ** Report a wild pointer error if we couldn't find the adm block or
  ** if the pool has been killed.
  */
  if(!ssigadm || pcb_to_instance(pool) != pool_instance)
    {
      /* Enable interrupts. */
      BKL_LOCK_RESTORE(msr);
      os_error(0, OSE_EWILD_SIG_POINTER | errcode, (OSERRCODE) sig);	/* Non fatal. */
      return NULL;
    }

  /* Report a buffer overwrite error if the user mode adm block looks ok. */
  if(usigadm->magic_signo == MAGIC_SIGNO)
    if(usigadm->udata == (SIGSELECT *) ssigadm)
      if(END_MARK != *(U8 *)(ssigadm->size + (U8 *) sig))
	{
	  /* Enable interrupts. */
	  BKL_LOCK_RESTORE(msr);
	  /*
	  ** Should we repair the endmark if we return from the error
	  ** handler and return the ssigadm pointer? If so we could
	  ** also repair the MAGIC_SIGNO and udata ptr in usigadm
	  ** (after reporting error). (Apr 26, 2006) @frbr@
	  */
	  os_error(fatal, OSE_ENO_BUFFER_END_MARK | errcode, (OSERRCODE) sig);	/* Fatal system error. */
	  return NULL;
	}

  /* Enable interrupts. */
  BKL_LOCK_RESTORE(msr);

  /* Traverse the pool looking for the last buffer with a correct adm block. */
  for(buf = prev_buf = pool->siglist->siglist; buf; buf = buf->siglist)
    {
      register struct _signal *uadm;

      /* Disable interrupts. */
      BKL_LOCK_SAVE(msr);

      /* Check if the pool has been killed. */
      if(pcb_to_instance(pool) != pool_instance)
	{
	  /* Enable interrupts. */
	  BKL_LOCK_RESTORE(msr);
	  
	  os_error(0, OSE_EWILD_SIG_POINTER | errcode, (OSERRCODE) sig);	/* Non fatal. */
	  return NULL;
	}

      uadm = ((struct _signal *) buf->udata) - 1;
      
      /* Is the next sigadm block OK? */
      if(uadm->magic_signo != MAGIC_SIGNO)
	{
	  /* No, report an error for the previous buffer. */
	  buf = prev_buf;
	  break;
	}
      /* Is the next sigadm block OK? */
      if(uadm->udata != (SIGSELECT *)buf)
	{
	  /* No, report an error for the previous buffer. */
	  buf = prev_buf;
	  break;
	}
      /* Yes, See if the end mark was overwritten. */
      if(END_MARK != *(U8 *)(buf->size + (U8 *)buf->udata))
	{
	  /* Yes, report an error for the current buffer. */
	  break;
	}

      /* Enable interrupts. */
      BKL_LOCK_RESTORE(msr);

      /* No, check the next buffer. */
      prev_buf = buf;
    }

  /* Report a buffer failure if a buffer was found. */
  {
    PROCESS extra[4];

    if(!buf)
      {
	/* Will this ever happen? @frbr@ */
	
	/* Disable interrupts. */
	BKL_LOCK_SAVE(msr);

	/* Check if the pool has been killed. */
	if(pcb_to_instance(pool) != pool_instance)
	  {
	    /* Enable interrupts. */
	    BKL_LOCK_RESTORE(msr);
	  
	    os_error(0, OSE_EWILD_SIG_POINTER | errcode, (OSERRCODE) sig);	/* Non fatal. */
	    return NULL;
	  }
	buf = ssigadm;
      }

    /* Report an error for the found buffer. */
    extra[0] = (PROCESS) 0;
    extra[1] = (PROCESS) 0;
    extra[2] = (PROCESS) (buf->udata);
    extra[3] = (PROCESS) (sig);
    if(buf->owner)
      {
	/* Show owner field if present. */
	extra[0] = (PROCESS) pcb_to_pid(zzuu.pix_list[buf->owner]);
      }
    else
      {
	/* Otherwise show the sender field or zero if unknown. */
	extra[0] = buf->sender_id;
      }

    /* Enable interrupts. */
    BKL_LOCK_RESTORE(msr);

    if(extra[0] && get_ptype(extra[0]) != OS_ZOOMBIE)
      extra[1] = (PROCESS) (pid_to_pcb(extra[0]))->name;
    os_error(fatal, OSE_ECORRUPTED_POOL | errcode, (OSERRCODE) extra);	/* Non fatal system error. */
    return NULL;
  }
}

#else

struct _signal *
zzrecover_from_pool_crash(struct _signal *sigadm, OSERRCODE errcode)
{
  struct poolcb *pool;
  struct pool_extension *frag = NULL;
  OSBOOLEAN fatal = (OSBOOLEAN) 1;
  struct _signal *prev_buf;
  struct _signal *next_buf;

  /* Locate the pool that contains the bad buffer. */
  /* Look out! There might be several in various segments when virtual memory ??*/
  for(pool = zzuu.pool_list; pool; pool = pool->next)
    {
      /* Check that this pool looks ok. */
      if(pool->magic != MAGIC_POOL)
	os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 5);	/* Fatal. */
      for(frag = pool->frag_list; frag; frag = frag->frag_list)
	{
	  /* Check that this fragment looks ok. */
	  if(frag->magic_frag != MAGIC_POOL)
	    os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 4);	/* Fatal. */
	  /* Check if the buffer is in this fragment. */
	  if(
	     (frag->base <= (OSADDRESS) sigadm) &&
	     (frag->limit > (OSADDRESS) sigadm)
	     )
	    break;
	}
      if(frag)
	break;
    }

  /* Report an error if the pool could not be found. This means a wild pointer. */
  if((!frag) || (!pool)) /* Can pool ever be NULL if frag != NULL? @frbr@ */
    {
      os_error(0, OSE_EWILD_SIG_POINTER | errcode, (OSERRCODE) (adm_to_sig(sigadm)));	/* Non fatal. */
      return NULL;
    }

  /* Errors reported should be fatal if the system pool was found. */
  if(pool == &system_pool)
    fatal = (OSBOOLEAN) 2;

  /* traverse the pool looking for the last buffer with a correct adm block. */
  for(next_buf = prev_buf = pool->siglist->siglist; next_buf; prev_buf = next_buf, next_buf = next_buf->siglist)
    {
      /* Is the next sigadm block OK? */
      if(next_buf->magic_signo != MAGIC_SIGNO)
	{
	  /* No, report an error for the previous buffer. */
	  next_buf = prev_buf;
	  break;
	}
      /* Yes, See if the end mark was overwritten. */
      if(END_MARK != *(U8 *)(next_buf->size + (U8 *) (adm_to_sig(next_buf))))
	{
	  /* Yes, report an error for the current buffer. */
	  break;
	}
      /* No, check the next buffer. */
    }

  /* Report a buffer failure if a buffer was found. */
  if(next_buf)
    {
      PROCESS extra[4];

      /* Clean up the pool by killing the current free lists. */
      pool->signal_list[0].hd = NULL;
      pool->signal_list[1].hd = NULL;
      pool->signal_list[2].hd = NULL;
      pool->signal_list[3].hd = NULL;
      pool->signal_list[4].hd = NULL;
      pool->signal_list[5].hd = NULL;
      pool->signal_list[6].hd = NULL;
      pool->signal_list[7].hd = NULL;

      /* Report an error for the found buffer. */
      extra[0] = (PROCESS) 0;
      extra[1] = (PROCESS) 0;
      extra[2] = (PROCESS) (adm_to_sig(next_buf));
      extra[3] = (PROCESS) (adm_to_sig(sigadm));
      if(next_buf->owner)
	{
	  /* Show owner field if present. */
	  extra[0] = (PROCESS) pcb_to_pid(zzuu.pix_list[next_buf->owner]);
	}
      else
	{
	  /* Otherwise show the sender field or zero if unknown. */
	  extra[0] = next_buf->sender_id;
	}
      if(extra[0])
	extra[1] = (PROCESS) (pid_to_pcb(extra[0]))->name;
      os_error(fatal, OSE_ECORRUPTED_POOL | errcode, (OSERRCODE) extra);	/* Non fatal system error. */
    }
  /* If the pointer is in the pool, but not on a signal boundary, then report a wild pointer. */
  else
    {
      os_error(0, OSE_EWILD_SIG_POINTER | errcode, (OSERRCODE) (adm_to_sig(sigadm)));	/* Non fatal. */
      return NULL;
    }

  /* Finally kill the system by reporting a general failure. */
  os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 2);	/* Fatal. */
  return NULL;
}

/* endif MEMORY_MODEL > 0 */
#endif

/* endif ERROR_CHECKS > 0 */
#endif

/*
** Check out a signal pointer. Issues an error if not OK.
** Bits of error code representing the system call is supplied by the caller,
** e.g. the OSE_SEND constant or a similar code.
** Returns zero if error an occurred and was ignored.
** Otherwise returns a pointer to the main signal administration block.
** Please note that buffer overwrite errors are fatal for all processes
** using the pool, and for the entire system if it is the kernel pool.
**
** There are a number of different versions of this code depending on
** the memory model and error check level selected.
*/

#if ERROR_CHECKS > 0

#if MEMORY_MODEL > 0

/*
** This version of check_buffer() is used with the complex memory model
** when full error checks are needed. In this case we really want to
** leave no stone unturned to locate errors, no matter how odd.
** Since all those checks can be *very* time consuming, we implement
** the really bad ones only on error check level 2.
*/

static struct _signal *
check_buffer(union SIGNAL **sigptr, OSERRCODE errcode)
{
  register struct _signal *ssigadm;
  register struct _signal *usigadm;
  register union SIGNAL *sig;

  /* Done here if error checks are disabled. */
  if(!zzuu.error_level1)
    {
      ssigadm = sig_to_adm(*sigptr);
      return ssigadm;
    }

  /* Check for the NULL pointer. */
  if(unlikely(!sigptr))
    {
      os_error(0, OSE_EBAD_PARAMETER | errcode, (OSERRCODE) sigptr);	/* Non fatal. */
      return NULL;
    }

  /* Pick up the signal pointer. */
  sig = *sigptr;

  /* Check for the NIL pointer. */
  if(unlikely(sig == NIL))
    {
      os_error(0, OSE_EUSED_NIL_POINTER | errcode, (OSERRCODE) sigptr);	/* Non fatal. */
      return NULL;
    }

#if ERROR_CHECKS > 0
  /* Check that the signal pointer has a correct aligment. */
  if(unlikely(((OSADDRESS)sig) /*& (OSADDRESS)(POOL_ALIGNMENT-1)*/))
    {
      os_error(0, OSE_EWILD_SIG_POINTER | errcode, (OSERRCODE) sig);	/* Non fatal. */
      return NULL;
    }
#endif
  
  /* Pick up the user space sigadm pointer. */
  usigadm = ((struct _signal *) sig) - 1;

  /*
  ** Check that this pointer is in one of the user pools or in the
  ** system pool.
  */
#if ERROR_CHECKS > 0
  ON_ERRLEVEL_2()
    {
      struct poolcb *pool;
      struct pool_extension *frag;
    
      /* Check the callers pool. */
      assert(zzcur_p->cursigpool->magic == MAGIC_POOL);
      for(frag = zzcur_p->cursigpool->frag_list; frag; frag = frag->frag_list)
	{
	  assert(frag->magic_frag == MAGIC_POOL);
	  /* Check if the buffer is in this fragment. */
	  if(
	     (frag->base <= (OSADDRESS) usigadm) &&
	     (frag->limit > (OSADDRESS) usigadm)
	     )
	    break;
	}

      /*
      ** Check all other pools if we can't find it in the callers
      ** pool. This is probably illegal, but we can't report it as an
      ** error because it might be OK in some systems with a low
      ** profile memory protection policy. Also, supervisor processes
      ** might need to use signals in other pools.
      */
      if(!frag)
	{
	  volatile OSADDRESS deleted_pool;

	  do
	    {
	      frag = NULL;
	      /* Keep track of any delete operations in the pool list. */
	      deleted_pool = zzuu.pool_delete_cnt;

	      for(pool = zzuu.pool_list; pool; pool = pool->next)
		{
		  if(pool != zzcur_p->cursigpool)
		    {
		      for(frag = pool->frag_list; frag; frag = frag->frag_list)
			{
			  assert(frag->magic_frag == MAGIC_POOL);

			  /* Check if the buffer is in this fragment. */
			  if(
			     (frag->base <= (OSADDRESS) usigadm) &&
			     (frag->limit > (OSADDRESS) usigadm)
			     )
			    break;
			}
		      if(frag)
			break;
		    }
		}
	      /*
	      ** We need to restart the search loop if any pool has
	      ** been deleted since we stared looping.
	      */
	    }
	  while(deleted_pool != zzuu.pool_delete_cnt);
	}
      
      /* Report an error if we can't find the signal buffer in any pool. */
      if(unlikely(!frag))
	{
	  os_error(0, OSE_EWILD_SIG_POINTER | errcode, (OSERRCODE) sig);	/* Non fatal. */
	  return NULL;
	}
    }
#endif

  /* Check the user space magic number. */
  if(usigadm->magic_signo != MAGIC_SIGNO)
    {
      /* Try to locate the buffer that caused the failure. */
      return zzrecover_from_pool_crash(sig, errcode);
    }

  /* Pick up the supervisor space sigadm pointer. */
  ssigadm = (struct _signal *) usigadm->udata;

  /* Check that this pointer is in the system pool. */
#if ERROR_CHECKS > 0
  ON_ERRLEVEL_2()
    {
      struct pool_extension *frag;
    
      /* Check the system pool. */
      for(frag = system_pool.frag_list; frag; frag = frag->frag_list)
	{
	  /* Check if the adm block is in this fragment. */
	  if(
	     (frag->base <= (OSADDRESS) ssigadm) &&
	     (frag->limit > (OSADDRESS) ssigadm)
	     )
	    break;
	}
    
      /* Report an error if we can't find the adm block in the system pool. */
      if(!frag)
	{
	  zzrecover_from_pool_crash(sig, errcode);
	  return NULL;
	}
    }
#endif

  /* Check the supervisor space magic number. */
  if(ssigadm->magic_signo != MAGIC_SIGNO)
    {
      /* Try to locate the buffer that caused the failure. */
      zzrecover_from_pool_crash(sig, errcode);
      return NULL;
    }

  /* Check that the backward pointer refers to the user space signo field. */
  if(ssigadm->udata != (SIGSELECT *) sig)
    {
      zzrecover_from_pool_crash(sig, errcode);
      return NULL;
    }

  /* Check signal ownership. */
  if(unlikely(ssigadm->owner != pcb_to_pindex(zzcur_p)))
    {
      /* Ignore this error if a restore system call is in progress. */
      if(errcode != OSE_RESTORE)
	{
	  /* Otherwise report as another owner. */
#ifdef GUNNAR_DEBUG
	  {
	    extern Boolean error_state;
	    error_state = True;
	    ramlog_printf("pose: check_buffer: current = %#x owner = %#x\n",
			  pcb_to_pindex(zzcur_p), ssigadm->owner);
	  }
#endif
	  os_error(0, OSE_ENOT_SIG_OWNER | errcode, (OSERRCODE) sig);	/* Non fatal. */
	  return NULL;
	}
    }

  /* Check user space end mark. */
  if(END_MARK != *(U8 *)(ssigadm->size + (U8 *) sig))
    {
      return zzrecover_from_pool_crash(sig, errcode);
    }

  /* Return a pointer to the supervisor space signal administration block. */
  return ssigadm;
}

#else

/*
** This checkbuffer() version is used in memory models with a single
** buffer administration block and full error checking.
*/

static struct _signal *
check_buffer(union SIGNAL **sigptr, OSERRCODE errcode)
{
  register union SIGNAL *sig;
  register struct _signal *sigadm;

  /* Done here if error checks are disabled. */
  if(!zzuu.error_level1)
    {
      sigadm = sig_to_adm(*sigptr);
      return sigadm;
    }

  /* Check if NIL pointer. */
  sig = *sigptr;
  if(unlikely(sig == NIL))
    {
      os_error(0, OSE_EUSED_NIL_POINTER | errcode, (OSERRCODE) sigptr);	/* Non fatal. */
      return NULL;
    }
  /* Check the magic number in the buffer. */
  /* It is important that we check this first, or we might report a misleading error code. */
  sigadm = ((struct _signal *) sig) - 1;
  if(sigadm->magic_signo != MAGIC_SIGNO)
    {
      /* Try to locate the buffer that caused the failure. */
      return zzrecover_from_pool_crash(sigadm, errcode);
    }

  /* Check owner of signal. May also be a wild pointer. */
  if(unlikely(sigadm->owner != pcb_to_pindex(zzcur_p)))
    {
      /* Ignore this error if a restore system call is in progress. */
      if(errcode != OSE_RESTORE)
	{
	  /* Otherwise report as another owner. */
	  os_error(0, OSE_ENOT_SIG_OWNER | errcode, (OSERRCODE) sig);	/* Non fatal. */
	  return NULL;
	}
    }
  /*
  ** We might also check that sender, addressee etc are reasonable.
  ** We won't do that because we expect linear overwrite errors.
  ** We assume that if the highest and lowest few bytes are OK,
  ** then the rest is probably OK too.
  ** See also:  collect_pool().
  */
  /* Check buffer end mark. */
  if(unlikely(END_MARK != *(U8 *)(sigadm->size + (U8 *) sig)))
    {
      os_error(1, OSE_ENO_BUFFER_END_MARK | errcode, (OSERRCODE) sig);	/* Non fatal system error. */
      /* Finally kill the system by reporting a general failure. */
      os_error(2, OSE_ENO_BUFFER_END_MARK | errcode, (OSERRCODE) sig);	/* Fatal system error. */
      return NULL; /* Not reached. */
    }
  return sigadm;
}

/* endif MEMORY_MODEL > 0 */
#endif

#else

#if MEMORY_MODEL > 0

/*
** This version of check_buffer() is used with the complex memory model
** when no error checks are desired. The keyword here is *speed*, but
** the code is somewhat complex. We are therefore going to provide a function
** that uses the sig_to_adm() macro. In this way we get the complex code in
** only one place.
*/

static struct _signal *
check_buffer(union SIGNAL **sigptr, OSERRCODE errcode)
{
  register struct _signal *sigadm;
  sigadm = sig_to_adm(*sigptr);
  return sigadm;
}

#else

/*
** This version of check_buffer() is used with the simplest memory model
** when no error checks are desired. The keyword here is *high speed*, so
** we do it all as an inline macro.
*/

#define check_buffer(sigptr,errcode) (((struct _signal *) *sigptr) - 1)

/* endif MEMORY_MODEL > 0 */
#endif

/* endif ERROR_CHECKS > 0 */
#endif

/*
** This function is a helper function to pool_brk_lo() and pool_brk_hi()
** and does all the work needed to extend the pool with a new fragment.
** Returns zero if the the pool was successfully extended.
**
** The function is only called by prioritized and background processes.
** It is called, and returns with a locked dispatcher, even if unlocked 
** internally.
*/

static OSERRCODE
extend_non_realtime_pool(struct poolcb *pool, U32 extend_count)
{
  register LOCKSTATE msr;
  OSADDRESS extend_base;
  OSADDRESS request_size = pool->frag_size;
  OSADDRESS actual_size;
  struct pool_extension *ext;
  OSERRCODE err;

  assert(zzcur_p->priority <= 32);
  assert(!pool->realtime_extendable);
  assert(zzdispatcher_locked);

  if(!zzdispatcher_locked)
    LOCK_SWAP();

  /* Report error if maximum extend size reached. */
  if(pool->tot_size >= pool->max_size)
    return 1;
  
  /* Ok, go on to extend the pool. */
  /* Last extension up to maximum size may have to be a smaller fragment. */
  if(pool->tot_size + pool->frag_size > pool->max_size)
    {
      request_size = pool->max_size - pool->tot_size;

      /* Check that last pool fragment isn't too small */
      if(request_size < 1024)
        return 1;
    }

  /* 
  ** Get a fragment control block. We may however not leave the free fragment
  ** list empty, as that would cause trouble for the real time extender!
  */
  BKL_LOCK();
  ext = zzuu.free_frag_list;
  if(ext && ext->frag_list)
    {
      /* The free list will still not be empty if we take the first one. */
      zzuu.free_frag_list = ext->frag_list;
      BKL_UNLOCK();

      /* Check that this fragment looks ok. */
      if(ext->magic_frag != MAGIC_POOL)
        os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 9);	/* Fatal. */
    }
  else
    {
      BKL_UNLOCK();
      ext = (struct pool_extension *)
	pool_brk_lo(&system_pool, sizeof(struct pool_extension));
      ext->magic_frag = MAGIC_POOL;
    }
  /*
  ** The dispatcher is locked, so we can't be killed here and leak the ext
  ** object. Now, kill-lock the current process to avoid memory leaks when
  ** we unlock the dispatcher further below.
  */
  (void)zzkrn_kill_lock(0);
  
  /* Switch to system pool for signals since the local pool is full. */
  set_suid((OSBOOLEAN) 1);
  
  /*
  ** Stop all other processes in the program to avoid other process
  ** from concurrently entering the pool extender.
  */
  stop_or_intercept(zzcur_p->block->segpid, ST_STOPPED, 
                    OSE_STOP, BUT_CURRENT_PROC);

  /* 
  ** Now that current process is the only one running in this program, make
  ** sure that no other process in the same program has extended before us.
  */
  if(extend_count != pool->extend_count)
    {
      /* Clean up and return without extending. */
      BKL_LOCK();
      ext->frag_list = zzuu.free_frag_list;
      zzuu.free_frag_list = ext;
      BKL_UNLOCK();

      start(zzcur_p->block->segpid);
      set_suid((OSBOOLEAN) 0);
      (void)zzkrn_kill_unlock(0);
      return 0;
    }

  /* Unlock the dispatcher, since we will need to send signals to MM. */
  UNLOCK_SWAP(); 

  /* Use callback to allocate memory. */
  {
    /* The local extender uses a OSPOOLEXTENDER - see core_internal.h
    ** for a prototype!
    */
    OSPOOLEXTENDER *local_ext_func = (OSPOOLEXTENDER *)pool->ext_func;
    actual_size = local_ext_func(pcb_to_pid(pool), 
				 (OSADDRESS)extend_count, 
				 request_size, 
				 pool->prog_type_p,
				 (void**)&extend_base);
  }
  if(actual_size >= request_size)
    {
      /* We got the memory, and it is the correct size */
   
      /* Fill the pool with an initialization pattern if requested */
      if(pool->fill)
        {
          memset((char *) extend_base, 0xcc, (size_t) request_size);
        }

      /* Initialize parts of new fragment header */
      ext->base = extend_base;
      ext->limit = extend_base + request_size;
      
      /* Disable interrupts. */
      SPIN_LOCK_SAVE(&pool->smp_spin, msr);

      /* Save the final pool low and high pointers of the previous fragment. */
      pool->frag_list->pool_low_p = pool->pool_low_p;
      pool->frag_list->pool_hi_p = pool->pool_hi_p;
      
      /* Update pool low and high pointers. */
      pool->pool_low_p = ext->base; /* Already aligned. */
      pool->pool_hi_p = ext->limit;
      
      /* Add the memory fragment to the list of all fragments in the pool. */
      ext->frag_list = pool->frag_list;
      pool->frag_list = ext;

      /* Update the total pool size attribute */
      pool->tot_size = pool->tot_size + (ext->limit - ext->base);

      /* Increment the pool extend count. */
      pool->extend_count++;

      /* Enable interrupts. */
      SPIN_LOCK_RESTORE(&pool->smp_spin, msr);

      /* No error. */
      err = 0;
    }
  else
    {
      /* Memory allocation failed. */
      err = 1;
    }

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* It is now safe to start all other processes in our program again */
  start(zzcur_p->block->segpid);

  /* Switch back to user pool for signals */
  set_suid((OSBOOLEAN) 0);
  
  /* Unlock the kill-lock since we are done */
  (void)zzkrn_kill_unlock(0);
 
  return err;
}

/*
** Used in pool_brk_hi and pool_brk_lo when calling a pool extension hook
** for pools with realtime extend capability.
** It sets the out_of_pool flag for the current process to allow the code
** in extend_pool to recognize the context in which the call is made.
**
** The dispatcher is locked when called by a prioritized or background process.
*/

static void
extend_realtime_pool(struct poolcb *pool, OSERRCODE extender_extra)
{
  assert(pool->realtime_extendable);
  
  /*
  ** Flag this process as being out of pool just like the error handler would,
  ** and invoke the callback.
  */
  zzcur_p->out_of_pool |= (pool == &system_pool ? 1 : 2);  

#ifdef OSE_SMP
  {
    /*
    ** This code is mainly here to support pool extend in SMP type of
    ** systems. If the pool extender has registered a handle to a new
    ** fragment, we want to make sure there is a new fragment in place
    ** if we have a priority below the memory manager. Otherwise we spin
    ** until there is. If we do not do this, a low priority process on
    ** any CPU (not hosting the memory manager) could call alloc() in a
    ** loop and thereby consume all of the memory in a fresh fragment
    ** before the memory manager gets the chance to allocate a new
    ** fragment.
    ** This is not an issue if the current process runs on cpu0,
    ** where it will be preempted by the fragment allocator process.
    */
    if(zzkrn_cpu_id() != 0 && pool->ext_frag_pp != NULL)
      {
	OSPRIORITY my_pri = zzcur_p->priority;
	
	/*
	** Check if I'm a bg/pri-proc with a lower pri than the
	** memory mgr.
	*/
	if(my_pri <= 32 && my_pri > pool->ext_frag_prio)
	  {
	    void * volatile * fpp = pool->ext_frag_pp;

	    /*
	    ** Spin until a new fragment is found or someone deregister
	    ** the fragment handle.
	    */
	    while(fpp != NULL && *fpp == NULL)
	      {
		fpp = pool->ext_frag_pp;
	      }
	  }
      }
  }
#endif
  
  /* Don't extend if the size limit has been reached. */
  if(pool->tot_size < pool->max_size)
    {
      /* Call the extender. */
      (void)pool->ext_func(pcb_to_pid(pool), extender_extra);
    }
  /*
  ** We may not actually have fixed the problem, but if we fall back to an
  ** error handler without clearing this flag, it will think it has already
  ** tried to fix the problem and report a recursive error.
  */
  zzcur_p->out_of_pool &= (pool == &system_pool ? ~1 : ~2);
}

/*
** Get a memory area from high end of pool. Error if out of space.
** May be called only from a non interrupt process.
** The dispatcher shall be locked by the caller to avoid memory leaks.
*/

static void *
pool_brk_hi(struct poolcb *pool, OSBUFSIZE sz)
{
  register LOCKSTATE msr;
  register OSADDRESS mem;
  register OSADDRESS size = (OSADDRESS) sz;
  register OSERRCODE extender_extra;
  register U32 seq;
  U32 extend_count;

  /* Assert: bootstrap or dispatcher locked */
  assert(zzuu.tickd_ == 0 || zzdispatcher_locked);

  /* Align size to a multiple of 32 bytes. */
  size = (size + (POOL_ALIGNMENT-1))/* & (OSADDRESS) ~(POOL_ALIGNMENT-1)*/;

  /* We will retry the operation 10 times. */  
  for(seq = 0; ; seq++)
    {
      /* Disable interrupts. */
      SPIN_LOCK_SAVE(&pool->smp_spin, msr);
      /* Check that there is space. */
      mem = (pool->pool_hi_p - size);
      if((pool->pool_hi_p - pool->pool_low_p) > size)
        {
          /* If space then take memory. */
          pool->pool_hi_p = mem;
          /* Enable interrupts. */
          SPIN_LOCK_RESTORE(&pool->smp_spin, msr);
          /* Return new space. */
          return (void *) mem;
        }
      /* If no space then read old extension counter. */
      extend_count = pool->extend_count;
      extender_extra = ((extend_count & 0x7f) << 24) | pcb_to_pindex(pool);
      /* Enable interrupts. */
      SPIN_LOCK_RESTORE(&pool->smp_spin, msr);

      /* Give up if there are too many errors. */
      if(seq == 10)
        break;
      
      /* Handle out of pool depending on what kind of pool it is. */
      if(pool->realtime_extendable)
        {
          /* Call realtime pool extension callback if present. */
          if(pool->ext_func)
            {
              extend_realtime_pool(pool, extender_extra);
            }
          
          /* Non fatal error if no callback or the callback failed to fix the problem. */
          if(!pool->ext_func || (seq == 9))
            {
              /* Out of system pool is a system error, out of main pool is not. */
              if (pool == &system_pool)            
                os_error(1, OSE_ENO_KERN_SPACE, extender_extra);
              else
		{
		  OSBOOLEAN unlock_lock = zzdispatcher_locked &&
		    (zzcur_p->priority <= 32);
		  
		  /*
		  ** By unlocking the sheduler befor calling the error
		  ** handler we put less restraints on what a user can do
		  ** in the error hander, send/receive for instance. We do
		  ** however open up for greater concurrency in the the
		  ** extend_pool handling, which might lead to more "lost"
		  ** fragments since extend_pool do not report any
		  ** failiure to install new fragments.
		  */
		  if(unlock_lock)
		    UNLOCK_SWAP();

		  os_error(0, OSE_ENO_USER_STACKSPACE, extender_extra);

		  if(unlock_lock)
		    LOCK_SWAP();
		}
            }      
        }  
      else
        { 
          /* Look for a pool extender callback. */
          if(pool->ext_func)
            {
              /* If found, use it. */
              if(extend_non_realtime_pool(pool, extend_count))
                {
                  /*
                  ** Report error if maximum extend size reached.
                  ** First give system error handler a chance to kill
                  ** program.
		  ** By unlocking the sheduler before calling the error
		  ** handler we put less restraints on what a user can
		  ** do in the error hander.
		  */
		  if(zzdispatcher_locked && (zzcur_p->priority <= 32))
		    UNLOCK_SWAP();
		  goto  pool_extension_failed;
                }
            }
          else
            {
	      /* Should not be reached. */
              os_error(0, OSE_ENO_USER_STACKSPACE, extender_extra);	/* Non fatal. */
            }
        }
      /* Replace the pool fragment record if the error handler took it. */
      if(!zzuu.free_frag_list)
        {
          struct pool_extension *ext;
          ext = (struct pool_extension *)
            pool_brk_lo(&system_pool, sizeof(struct pool_extension));
          ext->magic_frag = MAGIC_POOL;
          BKL_LOCK();
          ext->frag_list = zzuu.free_frag_list;
          zzuu.free_frag_list = ext;
          BKL_UNLOCK();
        }
    }

 pool_extension_failed:
  {
    OSERRCODE extra;
    if(pool == &system_pool)
      extra = OSE_ENO_KERN_SPACE;
    else
      extra = OSE_ENO_USER_STACKSPACE;
  os_error(1, OSE_EPOOL_EXTENSION_FAILED, (OSERRCODE) extra);	/* Non fatal system error. */
  os_error(2, OSE_EPOOL_EXTENSION_FAILED, (OSERRCODE) extra);	/* Fatal system error. */
  }
  /* NOTREACHED */
  return NULL;
}

/*
** Get a memory area from low end of pool. Error if out of space.
** May be called from any process type. When called from a non-interrupt
** process, the caller shall lock the dispatcher to avoid memory leaks.
*/

static void *
pool_brk_lo(struct poolcb *pool, OSBUFSIZE sz)
{
  register LOCKSTATE msr;
  register OSADDRESS mem;
  register OSADDRESS size = (OSADDRESS) sz;
  register OSERRCODE extender_extra;
  register U32 seq;
  U32 extend_count;

  /* Assert: bootstrap or dispatcher locked or running an interrupt process */
  assert(zzuu.tickd_ == 0 || zzdispatcher_locked || zzcur_p->priority > 32);

  /* Align size to a multiple of 32 bytes. */
  size = (size + (POOL_ALIGNMENT-1))/* & (OSADDRESS) ~(POOL_ALIGNMENT-1)*/;

  /* We will retry the operation 10 times. */
  for(seq = 0; ; seq++)
    {
      /* Disable interrupts. */
      SPIN_LOCK_SAVE(&pool->smp_spin, msr);
      /* Check that there is space. */
      mem = pool->pool_low_p; 
      if((pool->pool_hi_p - mem) > size)
        {
          /* If space then take memory. */
          pool->pool_low_p += size;
          /* Enable interrupts. */
          SPIN_LOCK_RESTORE(&pool->smp_spin, msr);
          /* Return new space. */
          return (void *) mem;
        }
      /* If no space then read old extension counter. */
      extend_count = pool->extend_count;
      extender_extra = ((extend_count & 0x7f) << 24) | pcb_to_pindex(pool);
      /* Enable interrupts. */
      SPIN_LOCK_RESTORE(&pool->smp_spin, msr);

      /* Check if we are in an alloc_nil() call. */
      if(zzcur_p->nil_alloc)
	 return NULL;
      
      /* Give up if there are too many errors. */
      if(seq == 10)
        break;
      
      /* Handle out of pool depending on what kind of pool it is. */
      if(pool->realtime_extendable)
        {
          /* Call realtime pool extension callback if present. */
          if(pool->ext_func)
            {
              extend_realtime_pool(pool, extender_extra);
            }
          
          /* Non fatal error if no callback or the callback failed to fix the problem. */
          if(!pool->ext_func || (seq == 9))
            {
              /* Out of system pool is a system error, out of main pool is not. */
              if (pool == &system_pool)            
                os_error(1, OSE_ENO_KERN_SPACE, extender_extra);
              else
		{
		  OSBOOLEAN unlock_lock = zzdispatcher_locked &&
		    (zzcur_p->priority <= 32);
		  
		  /*
		  ** By unlocking the sheduler befor calling the error
		  ** handler we put less restraints on what a user can do
		  ** in the error hander, send/receive for instance. We do
		  ** however open up for greater concurrency in the the
		  ** extend_pool handling, which might lead to more "lost"
		  ** fragments since extend_pool do not report any
		  ** failiure to install new fragments.
		  */
		  if(unlock_lock)
		    UNLOCK_SWAP();

		  os_error(0, OSE_ENO_USER_SIGSPACE, extender_extra);

		  if(unlock_lock)
		    LOCK_SWAP();
		}
            }      
        }
      else
        {
          /* Look for a pool extender callback. */
          if(pool->ext_func)
            {
              /* If found, use it. */
              if(extend_non_realtime_pool(pool, extend_count))
                {
                  /*
                  ** Report error if maximum extend size reached.
                  ** First give system error handler a chance to kill
                  ** program.
		  ** By unlocking the sheduler before calling the error
		  ** handler we put less restraints on what a user can
		  ** do in the error hander.
		  */
		  if(zzdispatcher_locked && (zzcur_p->priority <= 32))
		    UNLOCK_SWAP();
		  goto  pool_extension_failed;
                }
            }
          else
            {
	      /* Should not be reached. */
              os_error(0, OSE_ENO_USER_SIGSPACE, extender_extra);	/* Non fatal. */
            }
        }

      /* Replace the pool fragment record if the error handler took it. */
      if(!zzuu.free_frag_list)
        {
          struct pool_extension *ext;
          ext = (struct pool_extension *)
            pool_brk_lo(&system_pool, sizeof(struct pool_extension));
          ext->magic_frag = MAGIC_POOL;
          BKL_LOCK_SAVE(msr);
          ext->frag_list = zzuu.free_frag_list;
          zzuu.free_frag_list = ext;
          BKL_LOCK_RESTORE(msr);
        }
    }

 pool_extension_failed:
  {
    OSERRCODE extra;
    if(pool == &system_pool)
      extra = OSE_ENO_KERN_SPACE;
    else
      extra = OSE_ENO_USER_SIGSPACE;
    if(zzcur_p->priority <= 32)
      {
	/* For non interrupt processes we first report a non fatal error. */
	os_error(1, OSE_EPOOL_EXTENSION_FAILED, (OSERRCODE) extra); /* Non fatal system error. */
      }
    os_error(2, OSE_EPOOL_EXTENSION_FAILED, (OSERRCODE) extra); /* Fatal system error. */
  }
  /* NOTREACHED */
  return NULL;
}

/*
** Free a buffer to its native pool regardless of current owner.
** I.e. set the owner to the current process and do a
** normal free_buf().
**
** It is OK to call this function while the dispatcher is locked,
** provided that we know that no error will occur in check_buffer().
*/

static void
sys_free_buf(union SIGNAL **sig)
{
  change_sig_owner(sig_to_adm(*sig), zzcur_p);
  free_buf(sig);
}

/*
** Link a signal into the pool's free list, or give it back to the
** external allocator if there is a handle for such.  If the pcb
** argument is NULL this function will not decrement the number of
** owned signals for that process.
*/

static always_inline void
link_free_buf(struct poolcb *pool,
	      struct pool_list_entry *hd,
	      struct _signal *sigadm,
	      struct _pcb *pcb)
{
  LOCKSTATE msr;

  /* Check if we have an external allocator. */
  if(likely(hd->ext_handle == NULL))
    {
      /* Disable interrupts. */
      SPIN_LOCK_SAVE(&hd->smp_spin, msr);
  
      /* Make system the owner. */
      sigadm->owner = (PCB_INDEX_T) 0;
  
      /* Decrement owned counter in owners (=current) pcb. */
      if(pcb != NULL)
	pcb->sig_cnt_owned--;
  
      /* Link into free list. */
      sigadm->next = (struct _signal *) hd->hd;
      hd->hd = sigadm;
  
      /* Decrement no of signals allocated in pool. */
      hd->alloc_count--;
  
      /* Enable interrupts. */
      SPIN_LOCK_RESTORE(&hd->smp_spin, msr);
    }
  else
    {
      /* Disable interrupts. */
      LOCK_SAVE(msr);
  
      /* Make system the owner. */
      sigadm->owner = (PCB_INDEX_T) 0;
  
      /* Decrement owned counter in owners (=current) pcb. */
      if(pcb != NULL)
	pcb->sig_cnt_owned--;
  
      /* Call external free function. */
      pool->ext_free(hd->ext_handle, adm_to_sig(sigadm));

      /* Decrement no of signals allocated in pool. */
      (void)atomic_dec((atomic_t *)&hd->alloc_count);
  
      /* Enable interrupts. */
      LOCK_RESTORE(msr);
    }
}

/* Claim ownership of an external signal. */
static void
zzkrn_claim_ext_sig(union SIGNAL *sig, OSBUFSIZE size)
{
  struct _pcb *cur_pp = zzcur_p;
  struct _signal *sigadm = sig_to_adm(sig);
  LOCKSTATE msr;
  struct pool_list_entry *hd;
  
  if(unlikely(sigadm->magic_signo != MAGIC_SIGNO))
    {
      (void)check_buffer(&sig, 0);
      return;
    }

  hd = sigadm->buffer_head;
  if(unlikely(hd->ext_handle == NULL))
    {
      os_error(0, OSE_EBAD_PARAMETER, (OSERRCODE) sig);	/* Non fatal. */
      return;
    }

  /* Disable interrupts. */
  LOCK_SAVE(msr);
  
  /* Set sender_id, dest_id and size fields. */
  sigadm->sender_id = (PROCESS) 0;
  sigadm->dest_id = (PROCESS) 0;

  /* Set owner field. */
  sigadm->owner = pcb_to_pindex(cur_pp);
  
  /* Increment owned counter in callers pcb. */
  cur_pp->sig_cnt_owned++;
  
  /* Increment number of allocated signals. */
  (void)atomic_inc((atomic_t *)&hd->alloc_count);

  /* Too large buffer size. */
  if(unlikely(sigadm->buffer_head->size < size))
    {
      /* Report a warning. Trim to largest allowed. */
      os_error(0, OSE_EBUFFER_TOO_LARGE, (OSERRCODE) size);	/* Non fatal. */
      size = sigadm->buffer_head->size;
    }
  
  /* Update the end mark. */
#if ERROR_CHECKS > 0
  *(((U8 *) sig) + size) = (U8) END_MARK;
#endif

  /* Set the new size field. */
  sigadm->size = size;

  /* Enable interrupts. */
  LOCK_RESTORE(msr);
}

/* Release ownership of an external signal. */
static void
zzkrn_release_ext_sig(union SIGNAL *sig)
{
  struct _signal *sigadm = sig_to_adm(sig);
  LOCKSTATE msr;
  struct pool_list_entry *hd;
  
  if(unlikely(sigadm->magic_signo != MAGIC_SIGNO))
    {
      (void)check_buffer(&sig, 0);
      return;
    }

  hd = sigadm->buffer_head;
  if(unlikely(hd->ext_handle == NULL))
    {
      os_error(0, OSE_EBAD_PARAMETER, (OSERRCODE) sig);	/* Non fatal. */
      return;
    }

  /* Disable interrupts. */
  LOCK_SAVE(msr);
  
  /* Make system the owner. */
  sigadm->owner = (PCB_INDEX_T) 0;
  
  /* Decrement owned counter in owners (=current) pcb. */
  zzcur_p->sig_cnt_owned--;
  
  /* Decrement no of signals allocated in pool. */
  (void)atomic_dec((atomic_t *)&hd->alloc_count);

  /*
  ** Set buffer end mark and size.  Note that if we do not set the
  ** end mark first get_sig_info() might erroneously report that
  ** this buffer's end mark is overwritten.
  */
#if ERROR_CHECKS > 0
  *(hd->size + (U8 *) sig) = (U8) END_MARK;
#endif
  sigadm->size = hd->size;

  /* Enable interrupts. */
  LOCK_RESTORE(msr);
}

/*
** Sysconf 99 is used to copy out function pointers for
** zzkrn_claim_ext_sig and zzkrn_release_ext_sig. It is used like
** this:
**
** static void (*claim_ext_sig)(union SIGNAL *sig, OSBUFSIZE size) = NULL;
** static void (*release_ext_sig)(union SIGNAL *sig) = NULL;
** 
** void
** init_claim_release_functions(void)
** {
**    void *f[3];
**
**    if(get_sysconf(99, (OSADDRESS) f, sizeof(f), 0) == sizeof(f))
**    {
**       ramlog_printf("Found claim/release-signal functions. "
**                     "(Interface version: %lu)\n", (OSADDRESS)f[0]);
**       claim_ext_sig   = (void (*)(union SIGNAL *, OSBUFSIZE)) f[1];
**       release_ext_sig = (void (*)(union SIGNAL *))            f[2];
**    }
**    else
**    {
**       ramlog_printf("Could not get hold of claim/release-signal"
**                     "functions\n");
**    }
** }
*/

static OSADDRESS
sysconf_99(OSADDRESS fout, OSADDRESS size, OSADDRESS not_used)
{
  static void *f[3] = { (void *)1, /* Version number. */
                        (void *)zzkrn_claim_ext_sig,
                        (void *)zzkrn_release_ext_sig };

  (void) not_used;

  if((size < sizeof(f)) || zzguarded_memcpy((void *)fout,
					    (const char *)f, sizeof(f), 0))
    {
      return 0;
    }
  return sizeof(f);
}

/*
** Free a buffer to its native pool. No checks are performed.
**
** This is a free_buf which allows us to avoid touching user space data.
** This call is used when we need to free buffers in other memory segments.
**
** The input is a pointer to the signal administration block and not to
** the signal data area.
*/

static void
safe_free_buf(struct _signal *sigadm)
{
  register struct pool_list_entry *hd;
  register struct _pcb *cur_pp = zzcur_p;
  struct poolcb *pool;
  PROCESS poolid;

#ifdef ASSERT
  if(sigadm->owner != pcb_to_pindex(cur_pp) ||
     sigadm->magic_signo != MAGIC_SIGNO)
    {
      dump_sigadm(1, sigadm);
      assert(sigadm->magic_signo == MAGIC_SIGNO);
      assert(sigadm->owner == pcb_to_pindex(cur_pp));
    }
#endif

  /* Make sure we only free a buffer if we own it. */
  if(sigadm->owner == pcb_to_pindex(cur_pp))
    {
      /* Locate free list. */
      hd = sigadm->buffer_head;
      pool = hd->poolcbp;
      poolid = pcb_to_pid(pool);
  
#if OSE_DEBUG > 0
      if(handlerp.zzprof_free_buf)
	handlerp.zzprof_free_buf(sigadm->size, adm_to_sig(sigadm),
				 pcb_to_debug_pcb(cur_pp),
				 poolid);
#endif

      /*
      ** By setting the dest_id field (e.g. the addressee field)
      ** to the PID of the signal pool, we tell pool analyzers
      ** that this signal buffer is now in the freelist. The sender
      ** filed is set to indicate who used to own it.
      */
      sigadm->dest_id = poolid;
      sigadm->sender_id = cur_pp->pid;

      /* Link into free list. */
      link_free_buf(pool, hd, sigadm, cur_pp);
    }
}

/*
** Change signal buffer ownership.
**
** Performs a function similar to the restore system call.
** The new owner may be either a specified process or the
** operating system if new_owner == NULL.
** The dispatcher may or may not be locked.
*/


static noinline void
change_sig_owner_error(struct _signal *sigadm)
{
  union SIGNAL *sigp = (union SIGNAL *) sigadm->udata;
      
  ramlog_printf("krn: Bad signal header:\n");
  dump_mem(sigadm, sizeof(*sigadm));
  if(zzdispatcher_locked)
    {
      SILENT_UNLOCK_SWAP();
    }
  /* Try to locate the buffer that caused the failure. */
  check_buffer(&sigp, OSE_RESTORE);
  os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 30); /* Fatal system error */
}

static void
change_sig_owner(struct _signal *sigadm, struct _pcb *new_owner)
{
  LOCKSTATE msr;
  PCB_INDEX_T victim_pindex;
  SMP_CODE(cpuid_t cpu);

  /* sanity check of sig adm. */
  if(unlikely(sigadm->magic_signo != MAGIC_SIGNO))
    {
      change_sig_owner_error(sigadm);
      /*NOTREACHED*/
    }

  /* Disable interrupts. */
  LOCK_SAVE(msr);
  victim_pindex = sigadm->owner;
  /* Decrement owned counter in owners pcb if owner is not 0. */
  if(victim_pindex)
    {
      register struct _pcb *old_pcb;
      
      /* Get and check pcb of old owner. */
      old_pcb = zzuu.pix_list[victim_pindex];
      if(unlikely((old_pcb->magic & (~MAGIC_KILLED)) != MAGIC_PCB))
	{
	  /* It is likely that the signal is defect, not the pcb. */
	  change_sig_owner_error(sigadm);
	  /*NOTREACHED*/
	}

#ifdef OSE_SMP
      /*
      ** If the process is alive and runs on another cpu, let that one do 
      ** the update. If the owner is dead, do the update at cpu0.
      */
      cpu = old_pcb->magic == MAGIC_PCB ? old_pcb->cpu_id : 0;
      if(cpu != zzkrn_cpu_id())
	{
	  send_krn_event(cpu, KRN_EVT_SIG_COUNT, pcb_to_pid(old_pcb),
			 (void*)(-1 << 16));
	}
      else
	old_pcb->sig_cnt_owned--;
#else
      old_pcb->sig_cnt_owned--;
#endif
    }

  /* Increment owned counter in new owners pcb if new owner <> NULL. */
  if(new_owner)
    {
#ifdef OSE_SMP
      /* If the process runs on another cpu, let that one do the job. */
      /* This only happens in set_redirection. Signal transaction there instead?? */
      cpu = new_owner->cpu_id;
      if(cpu != zzkrn_cpu_id())
	{
	  send_krn_event(cpu, KRN_EVT_SIG_COUNT, pcb_to_pid(new_owner),
			 (void*)(1 << 16));
	}
      else
	new_owner->sig_cnt_owned++;
#else
      new_owner->sig_cnt_owned++;
#endif

      /* Set new owner. */
      sigadm->owner = pcb_to_pindex(new_owner);
    }
  else
    {
      /* Set new owner. */
      sigadm->owner = (PCB_INDEX_T) 0;
    }
  /* Enable interrupts. */
    LOCK_RESTORE(msr);
}

/*
** Allocate a new stack from the high end of the specified pool.
**
** The stack control block is allocated from the low end of the
** kernel pool.
** This function can only be called from a prioritized process.
**
** The dispatcher must be locked when this function is called.
*/

static struct stkcb *
stack_alloc(struct poolcb *inpool, OSADDRESS size)
{
  register struct poolcb *pool;
  register struct pool_list_entry *hd;
  register struct stkcb *stack;

  /* Align size to a multiple of 32 bits. */
  size = (size + (POOL_ALIGNMENT-1))/* & (OSADDRESS) ~(POOL_ALIGNMENT-1)*/;

  /* Find out which size to use. Error if too large. */
  pool = inpool;
  if(size <= pool->stack_list[3].size)
    {
      if(size <= pool->stack_list[0].size)
	hd = &pool->stack_list[0];
      else if(size <= pool->stack_list[1].size)
	hd = &pool->stack_list[1];
      else if(size <= pool->stack_list[2].size)
	hd = &pool->stack_list[2];
      else
	hd = &pool->stack_list[3];
    }
  else
    {
      if(size <= pool->stack_list[4].size)
	hd = &pool->stack_list[4];
      else if(size <= pool->stack_list[5].size)
	hd = &pool->stack_list[5];
      else if(size <= pool->stack_list[6].size)
	hd = &pool->stack_list[6];
      else if(size <= pool->stack_list[7].size)
	hd = &pool->stack_list[7];
      else
	{
	  /* Too large stack requested. Trim to largest allowed. */
	  UNLOCK_SWAP();
	  os_error(0, OSE_ESTACK_TOO_LARGE, (OSERRCODE) size);	/* Non fatal. */
	  LOCK_SWAP();
	  size = pool->stack_list[7].size;
	  hd = &pool->stack_list[7];
	}
    }

  /* Get buffer from free list if available. */
  if(hd->hd)
    {
      stack = (struct stkcb *) hd->hd;
      hd->hd = stack->next;
    }
  else
    {
      /* Get stack control block from free list if possible. */
      stack = zzuu.stkcb_free_list;
      if(stack)
	{
	  zzuu.stkcb_free_list = stack->next;
	}

      /* Else break it from base of the kernel pool. */
      else
	{
	  stack = (struct stkcb *) pool_brk_lo(&system_pool,
					       sizeof(struct stkcb));
	}

      /* Break new stack from top of the user stack pool. */
      stack->top = hd->size + (char *) pool_brk_hi(pool, hd->size);

      /* Set pool and buffer_head fields. */
      stack->buffer_head = hd;
    }

  /* Set real_limit field. */
  stack->limit = ((char *) stack->top) - size;

  /* Increment number of allocated stacks. */
  hd->alloc_count++;

  /* Return buffer. */
  return stack;
}

/*
** Free a stack to its native pool.
**
** This function can only be called from a prioritized process.
** The dispatcher must be locked when this function is called.
*/

static void
stack_free(struct stkcb *stack)
{
  register struct pool_list_entry *hd;

  /* Locate free list. */
  hd = stack->buffer_head;

  /* Link into free list. */
  stack->next = (struct stkcb *) hd->hd;
  hd->hd = stack;

  /* Decrement no of stacks allocated in the pool. */
  hd->alloc_count--;
}

/*
** Helper function for alloc_pixcb() and kernel start up functions.
**
** Adds a fresh controlblock to the pix_list, initialises its
** pid (instance and pindex) value. It also increases the hi-water mark
** of used controlblocks (next_free_pix).
*/
static void
add_to_pix_list(struct _pcb *pcb)
{
#ifndef LARGE_PID_SPACE
  pcb->pindex = zzuu.next_free_pix;
  pcb->instance = (PCB_INSTANCE_T) 1;
#else
  pcb->pid = zzuu.next_free_pix + (PCB_INDEX_MASK + 1);
#endif
  /* Avoid race on access of a partly initiated object: */
  pcb->killed = 1;
  pcb->magic = MAGIC_PCB | MAGIC_KILLED;

  zzuu.pix_list[zzuu.next_free_pix] = pcb;
  zzuu.next_free_pix++;
}
     
/*
** Helper functions to all kill related operations.
**
** Set kill flag and and increase instance number part of the control
** block's pid. After this the former pid will be recognized as a
** zombie.
*/
static void
bump_pcb_instance(struct _pcb *pcb)
{
  /* Set kill flag and bump instance number. */
  pcb->killed = 1;
#ifndef LARGE_PID_SPACE
  pcb->instance++;
  if(!pcb->instance) /* Avoid instance 0 since it is illegal. */
    pcb->instance++;
#else
  PROCESS instance = pcb->pid & ~PCB_INDEX_MASK;

  instance += (PCB_INDEX_MASK + 1);

  /* Avoid instance 0 since it is illegal. */
  if(!instance)
    instance = (PCB_INDEX_MASK + 1);
  pcb->pid = instance + (pcb->pid & PCB_INDEX_MASK);
#endif
}

/*
** Helper functions that calculates the next valid process id for a
** certain pcb.
*/
static PROCESS
next_pid_value(PROCESS pid)
{
  PROCESS instance;
  
#ifndef LARGE_PID_SPACE
  PROCESS pindex = pid_to_pindex(pid);

  instance = (pid_to_instance(pid) + 1) << PCB_INSTANCE_SHIFT;
  if(!instance) /* Avoid instance 0 since it is illegal. */
    instance = (PCB_INDEX_MASK + 1);

  return instance | pindex;
#else
  instance = pcb->pid & ~PCB_INDEX_MASK;

  instance += (PCB_INDEX_MASK + 1);

  /* Avoid instance 0 since it is illegal. */
  if(!instance)
    instance = (PCB_INDEX_MASK + 1);
  return instance + (pcb->pid & PCB_INDEX_MASK);
#endif
}

/*
** Helper function for alloc_pcb(), alloc_poolcb().
**
** The dispatcher must have been locked by the caller.
*/

static void *
alloc_pixcb(struct _pcb **freelist, OSBUFSIZE size, OSERRCODE subcode)
{
  register struct _pcb *pcb;

  /* Take buffer from free list if not empty. */
  BKL_LOCK();
  pcb = *freelist;
  if(pcb)	/* If any pcb in free list then get it. */
    {
      PCB_INDEX_T next_free_pix = zzuu.next_free_pix;

      *freelist = pcb->next;
      BKL_UNLOCK();

      assert(pcb != *freelist);
      if(!pcb->killed ||
	 !(pcb->magic & MAGIC_KILLED) ||
	 pcb->pindex >= next_free_pix ||
	 zzuu.pix_list[pcb->pindex] != pcb)
	{
	    os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 29);	/* Fatal. */
	}
    }

  /* Else break it from low kernel pool. */
  else
    {
      BKL_UNLOCK();

      /* Error if we used up the last process slot. */
      if(zzuu.next_free_pix >= MAX_PROC)
	{
	  UNLOCK_SWAP();
	  os_error(1,
		   subcode | OSE_ETOO_MANY_PROCESSES,
		   (OSERRCODE) MAX_PROC);	/* Non fatal system error. */
	  return NULL;
	}

      pcb = (struct _pcb *) pool_brk_lo(&system_pool, size);

      /* Set up basic fields. */
      add_to_pix_list(pcb);
    }

  /* Return found control block. */
  return (void *) pcb;
}

/*
** Allocate a process control block.
**
** Must be called from a prioritized process.
** The dispatcher must have been locked by the caller.
** This function is used for processes, blocks and segments.
** The fields common to pcb/bcb/poolcb/segcb are maintained by 
** alloc_pcb/alloc_poolcb. 
*/

static union large_cb *
alloc_pcb(OSERRCODE subcode)
{
  return (union large_cb *) alloc_pixcb(&zzuu.pcb_freelist,
					sizeof(union large_cb),
					subcode);
}

/*
** Allocate a pool control block.
**
** Must be called from a prioritized process.
** The dispatcher must have been locked by the caller.
** The fields common to pcb/bcb/poolcb/segcb are maintained by 
** alloc_pcb/alloc_poolcb. 
*/

static struct poolcb *
alloc_poolcb(void)
{
  return (struct poolcb *) alloc_pixcb((struct _pcb **)&zzuu.poolcb_freelist,
				       sizeof(struct poolcb),
				       OSE_CREATE_POOL);
}


/*
** Subfunction of free_pcb(), free_poolcb()
*/

static void 
free_pixcb(void *pcb_p, struct _pcb **freelist)
{
  register struct _pcb *pcb = (struct _pcb *) pcb_p;

  assert(pcb != *freelist);
  BKL_LOCK();
  pcb->killed = (OSBOOLEAN) 1;
  pcb->next = *freelist;
  *freelist = pcb;

  /*
  ** We also change the type of the pcb so that check_pid() et. al understands
  ** and refuses pathological designed pids.
  */
  pcb->magic |= MAGIC_KILLED;

  BKL_UNLOCK();
}

/*
** Free a process control block, a block descriptor or a segment descriptor
** to the proper free list.
**
** Must be called from a prioritized process.
** The dispatcher may or may not be locked by the caller.
** The fields common to pcb/bcb/segcb/poolcb are maintained by 
** free_pcb/free_poolcb, except for the instance number, which must be 
** incremented by the caller.
*/

static void 
free_pcb(void *pcb_p)
{
  free_pixcb(pcb_p, &zzuu.pcb_freelist);
}

/*
** Free a pool control block to the proper free list.
**
** Must be called from a prioritized process.
** The dispatcher may or may not be locked by the caller.
** The fields common to pcb/bcb/segcb/poolcb are maintained by 
** free_pcb/free_poolcb, except for the instance number, which must be 
** incremented by the caller.
*/

static void 
free_poolcb(struct poolcb *poolcb_p)
{
  free_pixcb(poolcb_p, (struct _pcb **) &zzuu.poolcb_freelist);
}

/*
** Allocate a signal from the kernel pool. Set signo and owner.
**
** This function is used internally by the kernel when buffers in the kernel
** are needed in system calls from user processes.
**
** See also: alloc(), locked_alloc().
*/

static union SIGNAL *
sys_alloc(OSBUFSIZE size, SIGSELECT signo)
{
  union SIGNAL *sig;
  set_suid((OSBOOLEAN) 1);
  sig = alloc(size, signo);
  set_suid((OSBOOLEAN) 0);
  if(unlikely(sigsize(&sig) < size))
    {
      /*
      ** Internal functions can not tolerate truncated buffers that result
      ** from user error handlers ignoring the OSE_EBUFFER_TOO_LARGE
      ** error.
      */
      os_error(2, OSE_EBUFFER_TOO_LARGE, (OSERRCODE) size);	/* Fatal. */
    }
  return sig;
}

/*
** Same as alloc() except it must be called with a locked dispatcher and
** only from a non interrupt process.
**
** It is a subfunction to system calls such as get_env() get_env_list(),
** get_pid_list() and others. These calls need to calculate a size,
** allocate a buffer and fill it as an atomic operation.
**
** There is a slight problem in this function. If a size larger than available
** is requested, the dispatcher must be unlocked before the error handlers
** are called. If the error handler fails to kill the caller, then a smaller
** buffer is allocated, and it is returned at a point where the dispatcher has
** been unlocked.
** This may in (very) rare cases cause the kernel to crash if the manipulated
** data structures were concurrently changed. In any case, a subsequent buffer
** overwrite error will occur. We won't bother to solve this weird problem.
** Instead we report the error as fatal, so a restart is forced.
**
** There is a problem in many system calls. Note that a page fault may occur
** whenever a user buffer is referenced. This means that a user pool buffer
** can not be referenced with a locked dispatcher. We must therefore in several
** cases build an intermediate image in the kernel pool and copy it to
** user space as an extra operation.??
**
** See also: sys_alloc(), alloc().
*/

static union SIGNAL *
locked_alloc(OSBUFSIZE insize, SIGSELECT signo)
{
  return alloc(insize, signo);
}

/*
** Free a pool that is no longer in use.
** This function is called by the system daemon when the last block
** served by a pool is killed, unless there are still signals allocated
** in the pool, in which case this function is called by the
** periodically activated garbage collector.
** The pool has already been unlinked from the pool list by the caller.
** When a pool is killed, the pool administration block is freed to the
** list of free process control blocks, pool memory space is reclaimed by the
** memory manager and all signal buffer control blocks are placed in the
** list of free signal buffer control blocks.
*/

static void
sysd_do_free_pool(struct poolcb *pool)
{
  struct stkcb *tail;
  struct segcb *old_seg;
  int i;

  /* Move all stack control blocks to the system list of free control blocks. */
  for(i = 0; i < 8; i++)
    {
      if(pool->stack_list[i].hd)
	{
	  for(tail = (struct stkcb *) pool->stack_list[i].hd;
	      tail->next;
	      tail = tail->next)
	    {
	      ; /* Empty loop body. */
	    }
	  /* Link the chain into the main free list. */
	  BKL_LOCK();
	  tail->next = zzuu.stkcb_free_list;
	  zzuu.stkcb_free_list = (struct stkcb *) pool->stack_list[i].hd;
	  BKL_UNLOCK();
	}
    }

  /* Move pool fragment control blocks to the free list. */
  {
    struct pool_extension *ext, *top, *last;
    for(top = last = ext = pool->frag_list; ext; ext = ext->frag_list)
      {
	/* Check that this fragment looks ok. */
	if(ext->magic_frag != MAGIC_POOL)
	  os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 7);	/* Fatal. */
	/* Update pointer to the last block in the chain. */
	if(last->frag_list)
	  last = last->frag_list;
      }
    /* Link the chain into the main free list of fragment control blocks. */
    BKL_LOCK();
    last->frag_list = zzuu.free_frag_list;
    zzuu.free_frag_list = top;
    BKL_UNLOCK();
  }

  /* Move signal administration blocks to the free list. */
#if MEMORY_MODEL > 0
  if(pool->siglist->siglist)
    {
      struct _signal *sigadm, *top, *last;
      for(top = last = sigadm = pool->siglist->siglist; sigadm; sigadm = sigadm->siglist)
	{
	  /* Check that this adm block looks ok. */
#if ERROR_CHECKS > 0
	  if(sigadm->magic_signo != MAGIC_SIGNO)
	    os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 15);	/* Fatal. */
#endif
	  /* Update pointer to the last block in the chain. */
	  if(last->siglist)
	    last = last->siglist;
	}
      /*
      ** Check that the last field in the pool control block actually points
      ** to the last element.
      */
      if(pool->sigtail != last)
	os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 16);	/* Fatal. */
      /* Link the chain into the main free list of fragment control blocks. */
      BKL_LOCK();
      last->siglist = zzuu.free_sigadm_list;
      zzuu.free_sigadm_list = top;
      BKL_UNLOCK();
    }
#endif

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Locate the applicable segment descriptor. */
  if(pool->segpid)
    {
      old_seg = (struct segcb *) pid_to_pcb(pool->segpid);
    }
  else if(pool->segid == 0)
    {
      /* Segid zero is the same as the system segment. */
      old_seg = &system_segment;
    }
  else
    {
      for(old_seg = zzuu.segment_list; old_seg; old_seg = old_seg->next)
	{
	  if(old_seg->segid == pool->segid)
	    {
	      break;
	    }
	}
    }

  /* Decrement no of pools in the memory segment. */
  old_seg->pool_count--;

  /* If no more blocks or pools or visitors then delete the segment. */
  if(!(old_seg->block_count || old_seg->pool_count || old_seg->visitors))
    {
      delete_segment(old_seg);
    }

  /* Unlock the dispacher. */
  UNLOCK_SWAP();

  /* Link the pool descriptor into free poolcb list. */
  free_poolcb(pool);
}

/*
** A utility called by sysd and sysd_lite to relocate or kill signals sent but
** not yet  received signals from dying pools. If intersegment copy is not 
** configured, no action is taken.
** The relocated signals will land in the main pool when possible, otherwise
** in the system pool.
**
** Returns the number of handled signals.
*/

#if MEMORY_MODEL > 0
static U32
dispose_queued_signals(struct poolcb *pool)
{
  U32 pix;
  U32 dispose_count = 0;

  /* Only meaningful to call this function if action expected. */
  assert(local_sysctlblk.killed_pool_signal_action > 1);

  /*
  ** It is only meaningful to move/kill not yet received signals to another
  ** pool if already received signals were also copied.
  */
  if(!local_sysctlblk.allow_pool_copy)
    {
      return dispose_count;
    }

  /* Scan the process message queues for signals allocated from this pool. */
  pix = zzuu.next_free_pix;
  while(--pix)
    {
      struct _pcb *pcb = zzuu.pix_list[pix];

      LOCK_SWAP();
      
      /*
      ** We handle sports and prioritized and background processes, with
      ** a non-empty queue. For SMP, treat only processes on the current cpu.
      */
      if(pcb->magic == MAGIC_PCB &&
#ifdef OSE_SMP
	 pcb->cpu_id == zzkrn_cpu_id() &&
#endif
	 pcb->mailbox &&
	 (pcb->priority <= 32 || (pcb->priority == 128 && pcb->is_an_sport)))
	{
	  int kill_signals = local_sysctlblk.killed_pool_signal_action == 3;
	  int flushed = 0;
	  struct _signal *sigadm;
	  struct _signal *prev_sigadm;
	  
	  /* Scan the mailbox for signals allocated from the cleaned pool. */

	  sigadm = pcb->mailbox;
	  prev_sigadm = NULL;
	  while(sigadm)
	    {
	      if(sigadm->buffer_head->poolcbp == pool)
		{
		  if(kill_signals)
		    {
		      /* === KILL the signal === */
		      struct _signal *next_sigadm;

#ifdef OSE_SMP
		      if(pcb->is_an_sport)
			BKL_LOCK();
		      else
			LOCK();
#else
		      LOCK();
#endif
		      next_sigadm = sigadm->next;
		      if(!next_sigadm)
			pcb->mailtail = prev_sigadm;
		      if(prev_sigadm)
			{
			  prev_sigadm->next = next_sigadm;
			}
		      else
			{
			  pcb->mailbox = next_sigadm;
			}
		      pcb->sig_cnt_queued--;

		      /*
		      ** Notify processes waiting at signal ports.
		      ** For SMP, this has to be done inside current lock region.
		      */
		      if(pcb->is_an_sport)
			pcb->s_pcb.take_listmodnumber++;
#ifdef OSE_SMP
		      if(pcb->is_an_sport)
			BKL_UNLOCK();
		      else
			UNLOCK();
#else
		      UNLOCK();
#endif
		      /* Free the old buffer. */
		      change_sig_owner(sigadm, zzcur_p);
		      safe_free_buf(sigadm);

		      /* One signal less remains in the pool. */
		      dispose_count++;

		      /* Remember that we have flushed this queue. */
		      flushed = 1;
		      sigadm = next_sigadm;
		      /* prev_sigadm is preserved here */
		    }
		  else /* Not kill_signals, i.e. RELOCATE them instead. */
		    {
		      struct _signal *new_sigadm;
		      union SIGNAL *new_sig;

		      if(sigadm->size <= system_pool.signal_list[7].size)
			{
			  /*
			  ** === Relocate the signal to another pool ===
			  ** Replace the signal with a copy in the main pool
			  ** or in the system pool.
			  */
			  if(pcb->cursigpool != &system_pool)
			    {
			      new_sig = NIL;
			      if(zzuu.main_pool)
				{
				  struct _pcb *cur_pp = zzcur_p;

				  cur_pp->cursigpool = zzuu.main_pool;
				  new_sig = alloc_nil(sigadm->size, 0);
				  cur_pp->cursigpool = &system_pool;
				}
			      if(!new_sig)
				new_sig = locked_alloc(sigadm->size, 0);
			    }
			  else
			    new_sig = locked_alloc(sigadm->size, 0);

			  /* Copy signal payload. */
			  memcpy(new_sig, sigadm->udata, (size_t)sigadm->size);

			  /*
			  ** Set the system as owner to the new buffer, and link
			  ** it into the signal queue.
			  */
			  new_sigadm = sig_to_adm(new_sig);
			  change_sig_owner(new_sigadm, NULL);
#ifdef OSE_SMP
			  if(pcb->is_an_sport)
			    BKL_LOCK();
			  else
			    LOCK();
#else
			  LOCK();
#endif
			  new_sigadm->dest_id = sigadm->dest_id;
			  new_sigadm->sender_id = sigadm->sender_id;
			  new_sigadm->next = sigadm->next;
			  if(new_sigadm->next == NULL)
			    pcb->mailtail = new_sigadm;
			  if(prev_sigadm)
			    {
			      prev_sigadm->next = new_sigadm;
			    }
			  else
			    {
			      pcb->mailbox = new_sigadm;
			    }
			  /*
			  ** Notify processes waiting at signal ports.
			  ** For SMP, this has to be done inside current lock region.
			  */
			  if(pcb->is_an_sport)
			    pcb->s_pcb.take_listmodnumber++;
#ifdef OSE_SMP
			  if(pcb->is_an_sport)
			    BKL_UNLOCK();
			  else
			    UNLOCK();
#else
			  UNLOCK();
#endif
			  /* Free the old buffer. */
			  change_sig_owner(sigadm, zzcur_p);
			  safe_free_buf(sigadm);

			  /* One signal less remains in the pool. */
			  dispose_count++;

			  /* Remember that we have flushed this queue. */
			  flushed = 1;
			  prev_sigadm = new_sigadm;
			  sigadm = new_sigadm->next;
			}
		      else
			{
			  /* The signal was too big, so it stays. */
			  prev_sigadm = sigadm;
			  sigadm = sigadm->next;
			}
		    } /* disposition type */
		} /* The signal was located in the killed pool. */
	      else
		{
		  /* The signal is not in killed pool, so it stays here. */
		  prev_sigadm = sigadm;
		  sigadm = sigadm->next;
		}
	    } /* while(sigadm) */

	      /* Notify the process if we have changed its signal queue. */
	  if(flushed)
	    {
	      /* Set the flushed flag in the pcb of the flushed process. */
	      pcb->flushed = (OSBOOLEAN) 1;
	    }
	}
      UNLOCK_SWAP();
    } /* while(--pix) */

  /* Return number of handled signals. */
  return dispose_count;
}
#endif

/*
** A utility called by sysd to clean dying pools from sent but not yet received
** signals. Such signals are either kept, killed or relocated to another pool.
** If intersegment copy is not configured, no action is performed. 
**
** Returns zero if we know that there are no signals allocated. 
** A non-zero return value just means "don't know, try again later". 
** Handle all pools or only pools in other segments than the system segment??
** See also: attach, sysconf_48 (attach and tmo signals must not be relocated).
*/

static U32
sysd_clean_killed_pool(struct poolcb *pool)
{
  U32 alloc_count;

  assert(pcb_to_pid(zzcur_p) == zzuu.sysd_);

  alloc_count = (pool->signal_list[0].alloc_count +
		 pool->signal_list[1].alloc_count +
		 pool->signal_list[2].alloc_count +
		 pool->signal_list[3].alloc_count +
		 pool->signal_list[4].alloc_count +
		 pool->signal_list[5].alloc_count +
		 pool->signal_list[6].alloc_count +
		 pool->signal_list[7].alloc_count);

  /* Count number of free attempts */
  if(pool->free_attempts < 255)
    {
      pool->free_attempts++;
    }

  /* If no signals are allocated, then we are ready. */
  if(alloc_count == 0)
    {
      return alloc_count;
    }

#if MEMORY_MODEL > 0
#ifdef ASSERT				/* POOL_DEBUG */

  /* This is debug code, not to leave the building! */

  /* Dump info for evil pools. */
  if(pool->free_attempts == 255)
    {
      int print_header = 1;
      PROCESS pool_id = pcb_to_pid((struct _pcb *)pool);
      struct _signal *sigadm = pool->siglist->siglist;

      ramlog_printf("POSE:%d: pool %#x alloc_count %d\n",
		    __LINE__, pool_id, alloc_count);
      while(sigadm)
	{
	  if(sigadm->dest_id != pool_id)
	    {
	      dump_sigadm(print_header, sigadm);
	      print_header = 0;
	    }
	  if(sigadm->siglist == pool->sigtail)
	    break;
	} /* while */
    }
#endif	/* POOL_DEBUG */

  /*
  ** It is only meaningful to go on if a kill or move action is configured.
  */
  if(local_sysctlblk.killed_pool_signal_action <= 1)
    return alloc_count;

  /*
  ** Delay the relocation action (2) a while, and do it just once,
  ** but perform the kill action (3) immediately.
  */
  if(local_sysctlblk.killed_pool_signal_action == 2 && pool->free_attempts != 2)
    return alloc_count;
  
  /*
  ** It is only meaningful to kill or move not yet received signals to another
  ** pool if already received signals were also copied.
  */
  if(!local_sysctlblk.allow_pool_copy)
    return alloc_count;

  /* Try to move still queued signals to another pool. */
#ifdef OSE_SMP
  /* SMP, sysd makes the job only for cpu 0. */
  alloc_count -= dispose_queued_signals(pool);

  /* While there are signals left, relocate on other cpus too. */
  if(alloc_count)
    {
      static const SIGSELECT sel_reloc_reply[] =
	{1, SMP_DISPOSE_QUEUED_SIGNALS_REPLY};
      cpuid_t cpu_id = zzkrn_num_cpus();
      union SIGNAL *sigr =
	alloc(sizeof(struct smp_dispose_queued_signals), 0);
	
      /* For all other cpus than # 0. */
      while(alloc_count && --cpu_id)
	{
	  sigr->sig_no = SMP_DISPOSE_QUEUED_SIGNALS_REQUEST;
	  sigr->smp_dispose_queued_signals.poolid = pcb_to_pid(pool);
	  send(&sigr,zzcpu[cpu_id].sysd_lite_);
	  sigr = receive(sel_reloc_reply);
	  alloc_count -= sigr->smp_dispose_queued_signals.disposal_count;
	} /* while */
      free_buf(&sigr);
    }
#else
  /* Non-SMP, sysd makes the job. */
  alloc_count -= dispose_queued_signals(pool);
#endif
#endif	/* MEMORY_MODEL */

  /* Return zero only if we know that there are no signals allocated now. */
  return alloc_count;
}

/*
** Pool garbage collector.
** This function is periodically called by the
** system daemon to check if any pool has become obsolete.
** Those found are freed.
** Return nonzero if the garbage collector should be called again at
** a later time.
** Return zero if there is no longer a potential garbage collect situation.
** In this case garbage collection need be resumed only when the last block
** of another pool has been killed while some signals are still owned by other
** processes or are in a queue.
*/

static OSBOOLEAN
sysd_check_free_pool(void)
{
  register struct poolcb *pool;
  register struct poolcb *prev_pool = NULL;
  register struct poolcb *next_pool;
  register OSBOOLEAN ret_flag;
  PROCESS pool_id;

  /* Clear the return flag. */
  ret_flag = (OSBOOLEAN) 0;

  /* Check all pools... */
  for(pool = zzuu.pool_list; pool; pool = next_pool)
    {
      /*
      ** ...but only those with neither stack users nor signal users
      ** and without the static attribute.
      */
      if(!(pool->no_of_sig_users || pool->no_of_stack_users || pool->is_static))
	{
	  /* Set return flag if pool has any signals still allocated. */
	  if(sysd_clean_killed_pool(pool))
	    {
	      ret_flag = (OSBOOLEAN) 1;
	      prev_pool = pool;
	      next_pool = pool->next;
	    }
	  else
	    {
	      /* Otherwise free the pool. */
	      next_pool = pool->next;

	      /*
	      ** Unlink the pool descriptor from zzuu.pool_list. Note
	      ** that we must lock, bump the deletion counter and
	      ** instance number here, otherwise we wont be able to
	      ** restart the pool traversal operations in
	      ** check_buffer() and zzrecover_from_pool_crash().
	      ** (Apr 26, 2006) @frbr@
	      */
	      LOCK_SWAP();
	      BKL_LOCK();
	      if(pool == zzuu.pool_list)
		{
		  zzuu.pool_list = next_pool;
		}
	      else
		{
		  /*
		  ** Prev_pool can be NULL here if we were interrupted by
		  ** someone calling create_pool(). If so, we cancel the
		  ** current pool scan, and retry on a later garbage
		  ** collect attempt, by just returning non-zero.
		  */
		  if(prev_pool == NULL)
		    {
		      BKL_UNLOCK_KEEP_INT_MASK();
		      LOCKED_UNLOCK_SWAP();
		      return 1;
		    }

		  prev_pool->next = next_pool;
		}
	      pool_id = pcb_to_pid((struct _pcb *)pool);
	      zzuu.pool_delete_cnt++;

	      /* Set kill flag and bump instance number. */
	      bump_pcb_instance((struct _pcb *)pool);
	      pool->magic |= MAGIC_KILLED;

	      BKL_UNLOCK_KEEP_INT_MASK();
	      LOCKED_UNLOCK_SWAP();

	      /* Distribute attach signals requested on the pool. */
	      kill_attach(pool_id);

	      /* Free the pool. */
	      sysd_do_free_pool(pool);
	    }
	}
      else
	{
	  prev_pool = pool;
	  next_pool = pool->next;
	}
    }

  /* Return the accumulated flag. */
  return ret_flag;
}


/*
** Subfunction to the system daemon.
**
** A request was made through kill_proc to kill an entire segment.
** This request was forwarded to sysd, who performs it here.
** All processes in all blocks are still running.
*/
static void
sysd_do_kill_seg_req(union SIGNAL *sigr)
{
  OSSEGMENT segid;
  register PROCESS bcb_list_bid;
  struct segcb *segcb;
  PROCESS segpid = sigr->kill_seg_req.segid;

  LOCK_SWAP();

  /* Obtain a pointer to the controlblock. */
  segcb = (struct segcb *) safe_pid_to_pcb(segpid, 1);

  /* Done if the segment is already killed or if the types mismatch. */
  if(segcb == NULL || segcb->magic != MAGIC_SEG)
    {
      UNLOCK_SWAP();
      return;
    }

  /* Locate the segment number corresponding to the segment id received. */
  segid = segcb->segid;

  UNLOCK_SWAP();

  /*
  ** Kill all blocks in the specified segment.
  ** We use the BID of the first block in the global block list to see if
  ** new blocks has been created, and if so we restart scanning the list
  ** for blocks to kill.
  */
  do
    {
      register struct _block *bcb;

      bcb_list_bid = pcb_to_pid(zzuu.bcb_list);
      for(bcb = zzuu.bcb_list; bcb; bcb = bcb->next)
	{
	  register PROCESS ppid = pcb_to_pid(bcb);
	  if(bcb->segpid)
	    {
	      if(bcb->segpid == segpid && !bcb->killed)
		kill_proc(ppid);
	    }
	  else
	    {
	      if(bcb->segid == segid && !bcb->killed)
		kill_proc(ppid);
	    }
	}
    }
  while(bcb_list_bid != pcb_to_pid(zzuu.bcb_list));
}

/*
** Subfunction of attach_segment, sysd_delete_block, sysd_do_free_pool and 
** sysd_delete_one_proc.
**
** delete_segment must be called with locked dispatcher.
*/
static void
delete_segment(struct segcb *seg)
{
  PROCESS segpid;

  /* Unlink the segment descriptor from the segment list. */
  if(seg == zzuu.segment_list)
    {
      zzuu.segment_list = seg->next;
    }
  else
    {
      struct segcb *tmpseg;
	  
      for(tmpseg = zzuu.segment_list; tmpseg->next; tmpseg = tmpseg->next)
	if(tmpseg->next == seg)
	  {
	    tmpseg->next = seg->next;
	    break;
	  }
    }

  /*
  ** Make sure the select_segment calling mechanism knows that the
  ** segment has died.
  */
  if(seg->segid == zzww.cur_segid)
    {
      handlerp.zzcall_select_segment(zzww.cur_segid = zzcur_p->segid);
    }

#ifdef OSE_SMP
  /*
  ** We need to make sure that any idle loop isn't still using the
  ** address mapping of the segment we want to delete. See:
  ** zzgo_search(). The system segment (segid zero) is always
  ** avaliable and won't be deleted.
  */
  if(seg->segid != 0)
    {
      /* Check all cpus except cpu 0. */
      cpuid_t cpu_id = zzkrn_num_cpus();

      while(--cpu_id != 0)
	{
	  /* Does this use the 'soon to be removed' segment? */
	  if(seg->segid == zzcpu[cpu_id].cur_segid)
	    {
	      /* Yes, so tell that cpu's idle loop to leave the segment now. */
	      zzcpu[cpu_id].leave_segid_now = seg->segid;

	      /*
	      ** Kick it to reschedule. This will wake it up from power
	      ** save or a wait for interrupt state.
	      */
	      {
		LOCKSTATE msr;
		LOCK_SAVE(msr);
		send_krn_event(cpu_id, KRN_EVT_RESCHEDULE, 0, NULL);
		LOCK_RESTORE(msr);
	      }
	      /* Wait until the cpu has left the segment. */
	      while(seg->segid == *(volatile OSSEGMENT *)(&zzcpu[cpu_id].cur_segid))
		{
		  ; /* Empty loop. */
		}
	      /* Reset the flag. */
	      zzcpu[cpu_id].leave_segid_now = 0;
	    }
	}
    }
#endif

  segpid = pcb_to_pid(seg);

  /* Set kill flag and bump instance number. */
  bump_pcb_instance((struct _pcb *)seg);

  /*
  ** Notify the MMU.
  ** We do not want to send the attach signals with the dispatcher locked,
  ** so we tell sysd to do it later.
  ** (Yes, this code usually run by sysd, but also from set_segment and
  ** attach_segment.)
  */
    {
      union SIGNAL *sig = sys_alloc(sizeof(struct kill_attach), KILL_ATTACH);
      sig->kill_attach.pid = segpid;
      /* Use a send suitable for a locked dispatcher. */
      zzsys_send_w_s(&sig, current_process(), zzuu.sysd_);
    }
}

/*
**===========================================================================
** 12.3  System calls.
**===========================================================================
*/

/*
** Create a new pool for a specified block.
**
** The pool is created in the memory segment currently attached to the
** block.
*/

/* Helper for create_pool, to validate size arrays. */
static OSADDRESS
increasing_sizes(OSADDRESS count, OSADDRESS *sizes)
{
  unsigned ix;

  /* An empty list is permitted. */
  if(count == 0)
    return 1;

  /* Check that the size array is sorted on size. */
  for(ix = 1; ix < count; ix++)
    {
      if (sizes[ix] < sizes[ix - 1])
	return 0;
    }

  /* If non-empty list, check that at least one size is non-zero. */
  return sizes[count - 1] != 0;
}

/*
** Helper for create_pool.
**
** Align the sizes, then eliminate duplicate sizes. Finally fill
** remaining elements with the greatest size.
*/
static void
align_pad_sizes(OSADDRESS *size_tab, int reserved_endmark_size)
{
  OSADDRESS count = size_tab[0];
  OSADDRESS *sizes = &size_tab[1];
  unsigned ix, iq;
  
  if(count)
    {
      /* Align each size. */
      for(ix = 0; ix < count; ix++)
	{
	  sizes[ix] = ((sizes[ix] + reserved_endmark_size + (POOL_ALIGNMENT-1))
		       /*& (OSADDRESS) ~(POOL_ALIGNMENT-1)*/) - reserved_endmark_size;
	} /* for */

      /* Eliminate duplicates. */
      iq = 0;
      for(ix = 1; ix < count; ix++)
	{
	  if(sizes[iq] != sizes[ix])
	    {
	      iq++;
	    }
	  sizes[iq] = sizes[ix];
	} /* for */

      /* Pad to eight sizes, duplicating the biggest one. */
      for(ix = iq + 1; ix < 8; ix++)
	{
	  sizes[ix] = sizes[iq];
	} /* for */

      /* Update the number of distinct sizes. */
      size_tab[0] = iq + 1;
    }
}

void
zzkrn_create_program_pool(PROCESS bid,
                          OSADDRESS base,
                          OSADDRESS size,
                          const OSADDRESS sigsize_tab[],
                          const OSADDRESS stacksize_tab[],
                          OSBOOLEAN fill,
                          OSPOOLEXTENDER *ext_func,
                          OSADDRESS frag_size,
                          OSADDRESS max_size,
                          void *prog_type_p)
{
  register struct _block *bcb;
  register struct poolcb *pool;
  OSADDRESS i;
  struct pool_extension *ext;
  struct size_tables
  {
    OSADDRESS sig[9];
    OSADDRESS stack[9];
  } *size_tab;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_CREATE_POOL))
    return;
#endif

  /* Locate the block descriptor. */
  /* Read instance number and check the block id. Report error if invalid. */
  bcb = check_bid(bid, OSE_CREATE_POOL);
  if(bcb == NULL)
    {
      return;
    }

  /* Check that the supplied memory area is reasonably large. */
#if ERROR_CHECKS > 0
  if(size < 1024)
    {
      os_error(0, OSE_EFRAGMENT_TOO_SMALL | OSE_CREATE_POOL, size);	/* Non fatal. */
      return;
    }
#endif

  /* Check that the supplied memory fragment does not extend beyond the
   * address space. We use a separete "end" variable due to the fact that
   * the compiler might optimize "if (b+s <= b)" to "if(s <= 0)", and that
   * wouldn't be so good in for the usefulness of our test.
   */
#if ERROR_CHECKS > 0
  {
    OSADDRESS end = base + size;
     
    if(end <= base)
      {
        os_error(0,
		 OSE_EFRAGMENT_TOO_LARGE | OSE_CREATE_POOL,
		 (OSERRCODE) size);	/* Non fatal. */
	return;
      }
  }
#endif

  /*
  ** Make sure that the supplied memory is properly aligned.
  */
  {
    OSADDRESS tmp = base;
    base = ((base + (POOL_ALIGNMENT-1))/* & (OSADDRESS) ~(POOL_ALIGNMENT-1)*/);
    size = size - (base - tmp);
    size = size /*& (OSADDRESS) ~(POOL_ALIGNMENT-1)*/;
  }

  /*
  ** Check that the memory fragment is not allready in use by another pool
  ** this is a common error when users don't realize that pools live long
  ** after the kill_proc() call that killed the block that was using the pool.
  */
#if ERROR_CHECKS > 0
  {
    struct pool_extension *frag;
    for(pool = zzuu.pool_list; pool; pool = pool->next)
      {
	for(frag = pool->frag_list; frag; frag = frag->frag_list)
	  {
	    /* Check if the buffer is in this fragment. */
	    if(
	       (frag->base <= (OSADDRESS) base) &&
	       (frag->limit > (OSADDRESS) base)
	       )
	      break;
	  }
	/* Report an error and abort if the pool fragment is already in use. */
	if(frag)
	  {
	    os_error(0, OSE_EFRAGMENT_IN_USE | OSE_CREATE_POOL, base);	/* Non fatal. */
	    return;
	  }
      }
  }  
#endif

  /* Fill memory area with debug pattern if caller wants us to. */
  if(fill)
    {
      /* By filling the memory area we also verify that it is available. */
      memset((char *) base, 0xcc, (size_t) size);
    }
  else
    {
#if ERROR_CHECKS > 0
      /* 
      ** Check that the supplied memory is available. We do this by reading
      ** and writing back to the first and last byte.
      */
      *((volatile char *) base) = *((volatile char *) base);
      *((volatile char *) (base + size - 1)) = *((volatile char *) 
                                                 (base + size - 1));
#endif
    }
  
  /* 
  ** Allocate and populate a temporary buffer for the size tables to avoid 
  ** page faults with a locked dispatcher. 
  */

  size_tab = (struct size_tables *) sys_alloc(sizeof(struct size_tables),
						 0);
  for(i = 0; i < 9; i++)
    {
      if(sigsize_tab[0] >= i)
	{
	  size_tab->sig[i] = sigsize_tab[i];
	}
      if(stacksize_tab[0] >= i)
	{
	  size_tab->stack[i] = stacksize_tab[i];
	}
    } /* for */
  
  /* Check that size tables are reasonable. */
  if((size_tab->sig[0] > 8) ||
     (size_tab->stack[0] > 8) ||
     !(size_tab->sig[0] || size_tab->stack[0])
     )
    {
      free_buf_type(struct size_tables *, &size_tab);
      os_error(0, OSE_EUNREASONABLE_SIZES | OSE_CREATE_POOL, 0); /* Non fatal. */
      return;
    }

  /*
  ** Check that non-empty size lists are sorted and hold at least one non-zero
  ** size.
  */
  if(!increasing_sizes(size_tab->sig[0], &size_tab->sig[1]) ||
     !increasing_sizes(size_tab->stack[0], &size_tab->stack[1]))
    {
      free_buf_type(struct size_tables *, &size_tab);
      os_error(0, OSE_EUNREASONABLE_SIZES | OSE_CREATE_POOL, 0); /* Non fatal. */
      return;
    }

  /*
  ** Align the sizes, then eliminate duplicate sizes. Finally fill
  ** remaining elements with the greatest size.
  */
  align_pad_sizes(size_tab->sig, sizeof(char));
  align_pad_sizes(size_tab->stack, 0);
  
  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the block is still present. */
  if(pcb_to_pid(bcb) != bid)
    {
      UNLOCK_SWAP();
      free_buf_type(struct size_tables *, &size_tab);
      return;
    }

  /* Check that the block contains no processes. */
  if(bcb->proclist)
    {
      UNLOCK_SWAP();
      free_buf_type(struct size_tables *, &size_tab);
      os_error(0, OSE_EBLOCK_HAS_PROCESSES | OSE_CREATE_POOL, 0); /* Non fatal. */
      return;
    }

  /* Allocate a pool control block. */
  pool = alloc_poolcb();

  /* Check that we got the descriptor. */
  if(!pool)
    {
      /* No, abort the system call. */
      UNLOCK_SWAP();
      free_buf_type(struct size_tables *, &size_tab);
      return;
    }
  /*
  ** Check the free list for a new fragment record, or break a new one.
  ** Remember that we can not take the last one.
  */
  BKL_LOCK();
  if(zzuu.free_frag_list && zzuu.free_frag_list->frag_list)
    {
      ext = zzuu.free_frag_list;
      zzuu.free_frag_list = zzuu.free_frag_list->frag_list;
      BKL_UNLOCK();
    }
  else
    {
      BKL_UNLOCK();
      ext = (struct pool_extension *)
	pool_brk_lo(&system_pool, sizeof(struct pool_extension));
      ext->magic_frag = MAGIC_POOL;
    }

  /* Check that this fragment looks ok. */
  if(ext->magic_frag != MAGIC_POOL)
    os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 6);	/* Fatal. */

  /* Fill in the new fragment record. */
  ext->base = (OSADDRESS) base;
  ext->limit = (OSADDRESS) ((base + size)/* & (OSADDRESS) ~(POOL_ALIGNMENT-1)*/);
  ext->frag_list = NULL;

  /* Set reasonable values for the final pool low and high pointers. */
  ext->pool_low_p = ext->base;
  ext->pool_hi_p = ext->limit;

  /* Set up pool control block. */
  SMP_CODE(spin_lock_init(&pool->smp_spin));
  pool->magic = MAGIC_POOL;
  pool->extend_count = 0;
  pool->frag_list = ext;
  pool->siglist = (struct _signal *) &(pool->firstsig);
  pool->sigtail = (struct _signal *) &(pool->firstsig);
  pool->firstsig.siglist = (struct _signal *) NULL;
  pool->no_of_stack_users = (U16) 0;
  pool->no_of_sig_users = (U16) 0;
  pool->segid = bcb->segid;
  pool->segpid = bcb->segpid;
  pool->free_attempts = 0;
  pool->pool_low_p = ext->base +
    ((sizeof(struct pool_extension) + (POOL_ALIGNMENT-1))/* & (OSADDRESS) ~(POOL_ALIGNMENT-1)*/);
  pool->pool_hi_p = ext->limit;
  pool->frag_size = frag_size;
  pool->max_size = max_size;
  pool->tot_size = ext->limit - ext->base;
  pool->prog_type_p = prog_type_p;
  pool->fill = fill;
  pool->ext_frag_prio = 0;
  pool->ext_frag_pp = NULL;
  pool->attached = NULL;
  /*
  ** Local User mode pools may use the non-realtime extender in the 
  ** program manager, that is activated through the PM program parameters.
  ** Other pools have realtime extend ability, either by a registred callback
  ** (via sysconf_67) or the legacy way, from an the error handler.
  ** Here we detect a program manager extender:
  */
  pool->realtime_extendable = bcb->supervisor_mode || (ext_func == NULL);
  pool->ext_func = pool->realtime_extendable ?
    NULL : (OSPOOLEXTENDH *)ext_func; 

  pool->ext_alloc = (void *(*)(void *)) NULL;
  pool->ext_free = (void (*)(void *, void *)) NULL;
  pool->no_copy = 0;
  pool->is_static=0;
  
  /* Locate the segment descriptor attached to the block. */
  {
    register struct segcb *seg;
    if(bcb->segpid)
      {
	seg = (struct segcb *) pid_to_pcb(bcb->segpid);
	/* Increment pool counter in the segment descriptor. */
	seg->pool_count++;
      }
    else if(bcb->segid == 0)
      {
	/* Segid zero is the same as the system segment. */
	seg = &system_segment;
	/* Increment pool counter in the segment descriptor. */
	seg->pool_count++;
      }
    else
      {
	for(seg = zzuu.segment_list; seg; seg = seg->next)
	  {
	    if(seg->segid == pool->segid)
	      {
		/* Increment pool counter in the segment descriptor. */
		seg->pool_count++;
		break;
	      }
	  }
      }
  }

  /* Store control block in list of pools. */
  pool->next = zzuu.pool_list;
  zzuu.pool_list = pool;

  /* Fill the size list. */
  for(i = 0; i < 8; i++)
    {
      if(size_tab->stack[0])
	pool->stack_list[i].size = size_tab->stack[i + 1];
      pool->stack_list[i].hd = NULL;
      pool->stack_list[i].alloc_count = 0;
      pool->stack_list[i].poolcbp = pool;
      if(size_tab->sig[0])
	pool->signal_list[i].size = size_tab->sig[i + 1];
      pool->signal_list[i].hd = NULL;
      pool->signal_list[i].alloc_count = 0;
      pool->signal_list[i].poolcbp = pool;
      pool->signal_list[i].segid = pool->segid;
      pool->signal_list[i].ext_handle = NULL;
      SMP_CODE(spin_lock_init(&pool->signal_list[i].smp_spin));
    }
  
  /* If new pool is a stack pool... */
  if(size_tab->stack[0])
    {
      /* Decrement stack usage counter in old pool. */
      bcb->stackpool->no_of_stack_users--;

      /* If the old pool now has no users, then start garbage collection. */
      /* (We might do the job here, but that would cause a race condition towards the garbage collector.) */
      if(!
	 ((bcb->stackpool->no_of_sig_users) |
	 (bcb->stackpool->no_of_stack_users))
	 )
	{
	  union SIGNAL *sig;
	  set_suid((OSBOOLEAN) 1);
	  sig = alloc(sizeof(SIGSELECT), START_GARBAGE_COLLECTOR);
	  send(&sig, zzuu.sysd_);
	  set_suid((OSBOOLEAN) 0);
	}

      /* Store new pool in clients block descriptor. */
      bcb->stackpool = pool;

      /* Increment stack usage counter in pool. */
      pool->no_of_stack_users++;
    }

  /* If new pool is a signal pool... */
  if(size_tab->sig[0])
    {
      /* Decrement signal usage counter in old pool. */
      bcb->sigpool->no_of_sig_users--;
      /* If old pool now has no users, then start garbage collector. */
      if(!
	 ((bcb->sigpool->no_of_sig_users) |
	 (bcb->sigpool->no_of_stack_users))
	 )
	{
	  union SIGNAL *sig;
	  set_suid((OSBOOLEAN) 1);
	  sig = alloc(sizeof(SIGSELECT), START_GARBAGE_COLLECTOR);
	  send(&sig, zzuu.sysd_);
	  set_suid((OSBOOLEAN) 0);
	}

      /* Store new pool in clients block descriptor. */
      bcb->sigpool = pool;

      /* Increment signal usage counter in pool. */
      pool->no_of_sig_users++;
    }

  /* Clear the killed flag for documentation purpose. */
  pool->killed = 0;

  /* Unlock dispatcher. */
  UNLOCK_SWAP();
  free_buf_type(struct size_tables *, &size_tab);
}

void
create_pool(PROCESS bid,
            OSADDRESS base,
            OSADDRESS size,
            const OSADDRESS sigsize_tab[],
            const OSADDRESS stacksize_tab[])
{
  /* Call extended version of create_pool() with default extra values.
  ** For backwards compatibility reasons the fill parameter is dependent 
  ** on whether ERROR_CHECKS is on or not.
  */
#if ERROR_CHECKS > 0
  zzkrn_create_program_pool(bid, base, size, sigsize_tab, stacksize_tab,
                            1, NULL, 0, 0, NULL);
#else
  zzkrn_create_program_pool(bid, base, size, sigsize_tab, stacksize_tab,
                            0, NULL, 0, 0, NULL);
#endif
}

/* ose_set_pool_attr */
static OSADDRESS
sysconf_97(OSADDRESS poolid, OSADDRESS tag_list_p, OSADDRESS not_used)
{
  LOCKSTATE msr;
  struct poolcb *pool;
  OSADDRESS success = 0;

  (void)not_used;
  
  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);

  /* Obtain a pointer to the controlblock. */
  pool = get_home_sigpool((PROCESS)poolid);

  if(pool != NULL && tag_list_p != 0)
    {
      OSADDRESS *tag_list = (OSADDRESS *)tag_list_p;
      OSADDRESS action = *tag_list;
      
      while(action != 0)
	{
	  switch(action)
	    {
	    case 1: /* Create hook. */
	      break;
	    case 2: /* Kill hook. */
	      break;
	    case 3:
	      pool->ext_alloc = (void *(*)(void *)) *++tag_list;
	      ++success;
	      break;
	    case 4:
	      pool->ext_free = (void (*)(void *, void *)) *++tag_list;
	      ++success;
	      break;
	    case 5: case  6: case  7: case  8:
	    case 9: case 10: case 11: case 12:
	      pool->signal_list[action-5].ext_handle = (void *) *++tag_list;
	      ++success;
	      break;
	    case 13:
	      pool->no_copy = 1;
	      ++success;
	      break;
	    case 14:
	      pool->is_static=1;
	      ++success;
	      break;
	    default:
	       os_error(0, OSE_EBAD_PARAMETER, action);	/* Non fatal. */
	      break;
	    }
	  action = *++tag_list;
	}
      /* Reenable interrupts. */
      BKL_LOCK_RESTORE(msr);
      
      return success;
    }
  else
    {
      /* Reenable interrupts. */
      BKL_LOCK_RESTORE(msr);
      
      return 0;
    }
}

/*
** Obtain the segment id for the specified process or block or pool.
** The system segment is indicated as zero.
** The segment number can not be obtained.
*/

PROCESS
get_segid(PROCESS pid)
{
  LOCKSTATE msr;
  register struct _pcb *pcb;

  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);

  /* Obtain a pointer to the controlblock. */
  pcb = safe_pid_to_pcb(pid, 1);

  if(pcb != NULL)
    {
      switch(pcb->magic)
	{
	case MAGIC_PCB:
	  /*
	  ** Found a valid and alive process. Return the block's
	  ** segment identity.
	  */
	  pid = pcb->block->segpid;
	  break;
	case MAGIC_BCB:
	  /* Found a valid and alive block. Return the segment identity. */
	  pid = ((struct _block *)pcb)->segpid;
	  break;
	case MAGIC_POOL:
	  /* Found a valid and alive pool. Return the segment identity. */
	  pid = ((struct poolcb *)pcb)->segpid;
	  break;
	default:
	  pcb = NULL;
	  break;
	}
    }
  
  /* Reenable interrupts. */
  BKL_LOCK_RESTORE(msr);
  
  if(pcb == NULL)
    {
      /* Report error if invalid. */
      (void)check_pid_bid(pid, 0);
    }

  /* Return the segment id. May be zero to indicate the system segment. */
  return pid;
}

/*
** Set the current segment for the calling process to segid.
** If go_home is nonzero, the segid parameter is ignored and
** the home segment of the process is reselected.
**
** Returns a non zero value if the segid parameter is
** unknown to the caller, i.e. the segment has already
** terminated, or the caller is not permitted to access it.
*/

OSBOOLEAN 
set_segment(OSBOOLEAN go_home, OSSEGMENT segid)
{
  struct segcb *old_seg;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_SET_SEGMENT))
    return 1;			/* Failed */
#endif

/* 
** Gunnar Eklund @ 2008-09-08
** Set_segment() now returns non-zero for user mode processes
** going to other segments, to resolve a problem with swap 
** handlers refering data in MAS mapped data. 
** Affected features are somewhat obscure: 
** 1) get_pcb, called by user process, target process has MAS: 
** The sigselect arry may be invisible.
** 2) create_heap from user mode with invisible target heap space.
** 3) creating a mm region from user mode, in another domain.
*/
  if(!zzcur_p->block->supervisor_mode)
    {
      if(go_home || segid == zzcur_p->block->segid)
	return 0;		/* Success */
      return 1;			/* Failed */
    }
  

  LOCK_SWAP();

  if(go_home)
    {
      segid = zzcur_p->block->segid;
    }
  else
    {
      /* Check the segment descriptor if not going to the home segment. */
      if(segid != zzcur_p->block->segid)
	{
	  struct segcb *new_seg;

	  if(segid == 0)
	    {
	      /* Remember that the system segment has a visitor. */
	      system_segment.visitors++;
	    }
	  else
	    {
	      for(new_seg = zzuu.segment_list; ; new_seg = new_seg->next)
		{
		  if(!new_seg)
		    {
		      /* Segment not found. */
		      UNLOCK_SWAP();
		      return 1;
		    }
	    
		  if(new_seg->segid == segid)
		    {
		      /* Remember that the segment has a visitor. */
		      new_seg->visitors++;
		      break;
		    }
		}
	    }
	}
    }

  /*
  ** If we are leaving another segment entered through set_segment, then
  ** decrement the visitor counter of that segment. 
  */
  old_seg = NULL;
  if(zzcur_p->segid != zzcur_p->block->segid)
    {
      if(zzcur_p->segid == 0)
	{
	  old_seg = &system_segment;
	}
      else
	{
	  for(old_seg = zzuu.segment_list; ; old_seg = old_seg->next)
	    {
	      if(!old_seg)
		{
		  /* Segment not found. */
		  UNLOCK_SWAP();
		  os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 20);	/* Fatal. */
		  return 1;		/* NOTREACHED */
		}
	      
	      if(old_seg->segid == zzcur_p->segid)
		{
		  break;
		}
	    }
	}
      old_seg->visitors--;
    }
	  
  zzcur_p->segid = segid;
  if(zzww.cur_segid != segid)
    {
      zzww.cur_segid = segid;
      handlerp.zzcall_select_segment(segid);
    }

  /*
  ** If we leave another segment than the home segment, and there are no
  ** more blocks or pools or visitors then delete the segment.
  */
  if(old_seg &&
     !(old_seg->block_count || old_seg->pool_count || old_seg->visitors))
    delete_segment(old_seg);

  UNLOCK_SWAP();
  return (OSBOOLEAN) 0;
}


/*
** Set the current segment for the calling process to segid.
** If go_home is nonzero, the segid parameter is ignored and
** the home segment of the process is reselected.
**
** Returns a non zero value if the segid parameter is
** unknown to the caller, i.e. the segment has already
** terminated.
**
** This variant of set_segment should only be used from a
** freeze-mode context and will return a non zero value
** (failure) if interrupts are enabled. 
*/

OSBOOLEAN 
zzkrn_freezemode_set_segment(OSBOOLEAN go_home, OSSEGMENT segid)
{
  struct segcb *old_seg;
  LOCKSTATE msr;

  LOCK_SAVE(msr);

  /* It is illegal to switch segment if interrupts are enabled.
   * This function should only be called from a freeze-mode context. */
  if (IRQ_ENABLED(msr))
    {
      LOCK_RESTORE(msr);
      return 1;
    }

  if(go_home)
    {
      segid = zzcur_p->block->segid;
    }
  else
    {
      /* Check the segment descriptor if not going to the home segment. */
      if(segid != zzcur_p->block->segid)
	{
	  struct segcb *new_seg;

	  if(segid == 0)
	    {
	      /* Remember that the system segment has a visitor. */
	      system_segment.visitors++;
	    }
	  else
	    {
	      for(new_seg = zzuu.segment_list; ; new_seg = new_seg->next)
		{
		  if(!new_seg)
		    {
		      /* Segment not found. */
		      LOCK_RESTORE(msr);
		      return 1;
		    }
		  
		  if(new_seg->segid == segid)
		    {
		      /* Remember that the segment has a visitor. */
		      new_seg->visitors++;
		      break;
		    }
		}
	    }
	}
    }

  /*
  ** If we are leaving another segment entered through set_segment, then
  ** decrement the visitor counter of that segment. 
  */
  old_seg = NULL;
  if(zzcur_p->segid != zzcur_p->block->segid)
    {
      if(zzcur_p->segid == 0)
	{
	  old_seg = &system_segment;
	}
      else
	{
	  for(old_seg = zzuu.segment_list; ; old_seg = old_seg->next)
	    {
	      if(!old_seg)
		{
		  /* Segment not found. */
		  LOCK_RESTORE(msr);
		  os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 20);	/* Fatal. */
		  return 1;		/* NOTREACHED */
		}
	      
	      if(old_seg->segid == zzcur_p->segid)
		{
		  break;
		}
	    }
	}
      old_seg->visitors--;
    }
  
  zzcur_p->segid = segid;
  if(zzww.cur_segid != segid)
    {
      zzww.cur_segid = segid;
      handlerp.zzcall_select_segment(segid);
    }

  LOCK_RESTORE(msr);
  return (OSBOOLEAN) 0;
}


/* get_sysconf entry for setting the mas_mapped attribute of segments. */
static OSADDRESS
sysconf_80(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  struct segcb *segcb;
  OSSEGMENT segnum = (OSSEGMENT)p;
  OSBOOLEAN mas_mapped = (OSBOOLEAN)(q != 0);

  (void)r;

  if(mas_mapped)
    {
      LOCK_SWAP();
      if(segnum == 0)
	{
	  /*
	  ** Is it ok to MAS map the system segment? @frbr@
	  */
	  system_segment.mas_mapped = mas_mapped;
	}
      else
	{
	  for(segcb = zzuu.segment_list; segcb; segcb = segcb->next)
	    {
	      if(segcb->segid == segnum)
		{
		  segcb->mas_mapped = mas_mapped;
		}
	    }
	}
      UNLOCK_SWAP();
    }
  return 0;
}

/*
** Move the specified block to a new memory segment.
** Create the segment.
** Delete the old memory segment if it is now unused.
** Otherwise set the last killed field in the segment descriptor.
** It is assumed that the caller of attach_segment is never killed.
*/

PROCESS
attach_segment(PROCESS bid, OSSEGMENT segid)
{
  register struct _block *bcb;
  struct segcb *new_seg;
  struct segcb *old_seg;
  PROCESS sid;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_ATTACH_SEGMENT))
    return any_killed_pid();
#endif

  /* Locate the block descriptor. */
  /* Read instance number and check the block id. Report error if invalid. */
  bcb = check_bid(bid, OSE_ATTACH_SEGMENT);
  if(bcb == NULL)
    {
      return any_killed_pid();
    }


  /* Locate the blocks new segment descriptor if segment zero. */
  new_seg = NULL;
  if(segid)
    {
      for(new_seg = zzuu.segment_list; new_seg; new_seg = new_seg->next)
	{
	  if(new_seg->segid == segid)
	    {
	      break;
	    }
	}
    }

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the block is still present. */
  if(pcb_to_pid(bcb) != bid)
    {
      UNLOCK_SWAP();
      return any_killed_pid();
    }

  /* Check that the block contains no processes. */
  if(bcb->proclist)
    {
      UNLOCK_SWAP();
      os_error(0, OSE_EBLOCK_HAS_PROCESSES | OSE_ATTACH_SEGMENT, 0);	/* Non fatal. */
      return any_killed_pid();
    }

  /* Create it if necessary. */
  if(!new_seg)
    {
      /* Allocate a buffer from the kernel pool. */
      new_seg = &alloc_pcb(OSE_ATTACH_SEGMENT)->segcb;

      /* Check that we got a segment descriptor. */
      if(!new_seg)
	{
	  /* No, abort the system call. */
	  UNLOCK_SWAP();
	  return any_killed_pid();
	}

      /* Make it the main block in the new segment if not in supervisor space. */
      if(segid)
	bcb->is_main_block = (OSBOOLEAN) 1;

      /* Initialize fields of segment descriptor. */
      new_seg->magic = MAGIC_SEG;
      new_seg->killed = (OSBOOLEAN) 0;
      new_seg->attached = NULL;

      new_seg->segid = segid;
      new_seg->block_count = (U16) 0;
      new_seg->pool_count = (U16) 0;
      new_seg->highpri_count = (U16) 0;
      new_seg->visitors = (U16) 0;
      new_seg->mas_mapped = 0;
      new_seg->semlist = NULL;
      new_seg->mutlist = NULL;
      new_seg->isr_list = NULL;
      new_seg->main_block = bid;
      
      /*
      ** We create the segment as locked by setting the MIGRATION_MASK_PM bit.
      ** The Program Handler may clear it to permit dynamic bind_process. 
      */
      new_seg->migration_inhibited = MIGRATION_MASK_PM;

      /* Link into list of segments. */
      new_seg->next = zzuu.segment_list;
      zzuu.segment_list = new_seg;
    }

  /*
  ** Locate the blocks old segment descriptor. The static system_segment
  ** is identified by a segpid that is zero.
  */
  if(bcb->segpid)
    old_seg = (struct segcb *) pid_to_pcb(bcb->segpid);
  else
    old_seg = &system_segment;

  /* Calculate the segment id of the new segment. */
  sid = pcb_to_pid(new_seg);

  /* Increment the number of attached blocks in the new segment. */
  new_seg->block_count++;

  /* Set the new segment id in the block descriptor. */
  bcb->segid = segid;
  bcb->segpid = pcb_to_pid(new_seg);

  /*
  ** Clear the clib pointer to tell zzcreate_context that this is a main block
  ** in a new segment.
  */
  bcb->clib_pointer = NULL;

  /* Set the last killed field in the old segment descriptor. */
  old_seg->last_bid = bid;

  /* Decrement the number of attached blocks in the old segment. */
  old_seg->block_count--;

  /* If no more blocks or pools or visitors then delete the segment. */
  if(!(old_seg->block_count || old_seg->pool_count || old_seg->visitors))
    delete_segment(old_seg);

  /* Unlock the dispacher. */
  UNLOCK_SWAP();

  return(sid);
}

/*
** Allocate a signal buffer either from the system pool or from
** the current signal pool. Set signal number in the new buffer.
**
** See also: alloc_nil(), s_alloc_nil(), sys_alloc(), locked_alloc().
*/

union SIGNAL *
alloc(OSBUFSIZE size, SIGSELECT signo)
{
  LOCKSTATE msr;
  register struct pool_list_entry *hd;
  register struct _signal *sig;
  register struct _pcb *cur_pp = zzcur_p;
  union SIGNAL *usig;

#if ERROR_CHECKS > 0
  /*
  ** Make sure the signal is large enough for the signal number.
  ** If we don't do this we will overwrite the debuggers end mark.
  ** There may of course also be some other problems, but we
  ** will ignore these unless error check overhead is accepted.
  */
  if(unlikely(size < sizeof(SIGSELECT)))
     size = sizeof(SIGSELECT);
#endif

  /* Find out which size to use. Error if too large. */
  hd = &cur_pp->cursigpool->signal_list[3];
  if(size <= hd->size)
    {
      hd-=3;				/* hd = &pool->signal_list[0]; */
      if(size > hd->size)		/* Size > list[0]. */
	{
	  hd++;
	  if(size > hd->size)		/* Size > list[1]. */
	    {
	      hd++;
	      if(size > hd->size)	/* Size > list[2]. */
		{
		  hd++;
		}
	    }
	}
    }
  else
    {
      hd++;				/* hd = &pool->signal_list[4]; */
      if(size > hd->size)		/* Size > list[4]. */
	{
	  hd++;
	  if(size > hd->size)		/* Size > list[5]. */
	    {
	      hd++;
	      if(size > hd->size)	/* Size > list[6]. */
		{
		  hd++;
		  if(size > hd->size)	/* Size > list[7]. */
		    {
		      /*
		      ** Too large buffer requested.
		      ** Return NIL if we are in an alloc_nil() call.
		      */
		      if(cur_pp->nil_alloc)
			return NIL;

		      /* Report a warning. Trim to largest allowed. */
		      os_error(0, OSE_EBUFFER_TOO_LARGE, (OSERRCODE) size);	/* Non fatal. */
		      hd = &cur_pp->cursigpool->signal_list[7];
		      size = hd->size;
		    }
		}
	    }
	}
    }

  /* Disable interrupts. */
  LOCK_SAVE(msr);

  if(unlikely(hd->ext_handle))
    {
      usig = cur_pp->cursigpool->ext_alloc(hd->ext_handle);

      if(usig == NULL)
	{
	  LOCK_RESTORE(msr);
	  goto pool_brk;
	}

      /* Increment number of allocated signals. */
      /* hd->alloc_count++; */
      (void)atomic_inc((atomic_t *)&hd->alloc_count);

      sig = sig_to_adm(usig);
    }
  else
    {
      SPIN_LOCK(&hd->smp_spin);
      
      /* Get buffer from free list if available. */
      sig = (struct _signal *) hd->hd;
      if(likely(sig))
	{
	  hd->hd = sig->next;

	  /* Increment number of allocated signals. */
	  hd->alloc_count++;

	  SPIN_UNLOCK(&hd->smp_spin);
	}
      else
	{
	  SPIN_UNLOCK(&hd->smp_spin);
	  LOCK_RESTORE(msr);
	  goto pool_brk;
	}
    }

  /* Set owner field. */
  sig->owner = pcb_to_pindex(cur_pp);

  /* Increment owned counter in callers pcb. */
  cur_pp->sig_cnt_owned++;

  /* Set sender_id, dest_id and size fields. */
  sig->sender_id = (PROCESS) 0;
  sig->dest_id = (PROCESS) 0;

  /* Fill in the signal number. */
  usig = adm_to_sig(sig);
  usig->sig_no = signo;

  /*
  ** Set buffer end mark and size.  Note that if we do not set the
  ** end mark first get_sig_info() might erroneously report that
  ** this buffer's end mark is overwritten.
  */
#if ERROR_CHECKS > 0
  *(size + (U8 *) usig) = (U8) END_MARK;
#endif
  sig->size = size;

  /* Enable interrupts. */
  LOCK_RESTORE(msr);

#if OSE_DEBUG > 0
  if(handlerp.zzprof_alloc)
    handlerp.zzprof_alloc(size, usig,
			  pcb_to_debug_pcb(cur_pp),
			  pcb_to_pid(cur_pp->cursigpool));
#endif
  
  return usig;

  /* Break a virgin signal from the current pool. */
pool_brk:
  return alloc_needs_pool_brk(size, signo, hd);
}

/* union SIGNAL *alloc_brk(OSBUFSIZE size, SIGSELECT signo) */
static OSADDRESS
sysconf_98(OSADDRESS size, OSADDRESS signo, OSADDRESS nil_alloc)
{
  register struct pool_list_entry *hd;
  
#if ERROR_CHECKS > 0
  /*
  ** Make sure the signal is large enough for the signal number.
  ** If we don't do this we will overwrite the debuggers end mark.
  ** There may of course also be some other problems, but we
  ** will ignore these unless error check overhead is accepted.
  */
  if(unlikely(size < sizeof(SIGSELECT)))
     size = sizeof(SIGSELECT);
#endif

  /* Find out which size to use. Error if too large. */
  hd = &zzcur_p->cursigpool->signal_list[3];
  if(size <= hd->size)
    {
      hd-=3;				/* hd = &pool->signal_list[0]; */
      if(size > hd->size)		/* Size > list[0]. */
	{
	  hd++;
	  if(size > hd->size)		/* Size > list[1]. */
	    {
	      hd++;
	      if(size > hd->size)	/* Size > list[2]. */
		{
		  hd++;
		}
	    }
	}
    }
  else
    {
      hd++;				/* hd = &pool->signal_list[4]; */
      if(size > hd->size)		/* Size > list[4]. */
	{
	  hd++;
	  if(size > hd->size)		/* Size > list[5]. */
	    {
	      hd++;
	      if(size > hd->size)	/* Size > list[6]. */
		{
		  hd++;
		  if(unlikely(size > hd->size))	/* Size > list[7]. */
		    {
		      /*
		      ** Too large buffer requested.
		      ** Report a warning. Trim to largest allowed.
		      */
		      os_error(0, OSE_EBUFFER_TOO_LARGE, (OSERRCODE) size);	/* Non fatal. */
		      hd = &zzcur_p->cursigpool->signal_list[7];
		      size = hd->size;
		    }
		}
	    }
	}
    }

  if(nil_alloc)
    {
      union SIGNAL *sig;
      
      zzcur_p->nil_alloc = (OSBOOLEAN)1;
      sig = alloc_needs_pool_brk(size, signo, hd);
      zzcur_p->nil_alloc = (OSBOOLEAN)0;
      return (OSADDRESS)sig;
    }
  else
    {
      return (OSADDRESS)alloc_needs_pool_brk(size, signo, hd);
    }
}

static noinline union SIGNAL *
alloc_needs_pool_brk(OSBUFSIZE size,
		     SIGSELECT signo,
		     struct pool_list_entry *hd)
{
  LOCKSTATE msr;
  register struct _signal *sig;
  register struct _pcb *cur_pp = zzcur_p;
  register struct poolcb *pool = cur_pp->cursigpool;
#if MEMORY_MODEL > 0
  struct _signal *usigadm;
#endif
  union SIGNAL *usig;
  OSBOOLEAN dispatcher_locked;

  /*
  ** Save dispatcher locked flag so we can emulate the
  ** locked_alloc() behaviour.
  */
  dispatcher_locked = zzdispatcher_locked;

  if(cur_pp->priority <= 32 && !dispatcher_locked)
    {
      /* Non interrupt process. Lock dispatcher. */
      LOCK_SWAP();
    }

#if MEMORY_MODEL > 0
  /* allocate a new signal administration block. */
  BKL_LOCK_SAVE(msr);
  
  sig = zzuu.free_sigadm_list;
  if(sig)
    {
      zzuu.free_sigadm_list = sig->siglist;
      BKL_LOCK_RESTORE(msr);
    }
  else
    {
      BKL_LOCK_RESTORE(msr);
      /* Break new buffer administration block from the pool. */
      sig = (struct _signal *) pool_brk_lo(&system_pool,
					   sizeof(struct _signal));
      if(sig == NULL)
	{
	  /* We are in an alloc_nil() and the pool fragment is full. */
	  if(cur_pp->priority <= 32 && !dispatcher_locked)
	    UNLOCK_SWAP();
	  return NULL;
	}
    }
  /* Break new buffer from pool. */
  usigadm = (struct _signal *) pool_brk_lo(pool,
					   hd->size + 1 +
					   sizeof(struct _signal));
  if(usigadm == NULL)
    {
      /* We are in an alloc_nil() and the pool fragment is full. */

      /* Link the chain into the main free list of fragment control blocks. */
      BKL_LOCK_SAVE(msr);
      sig->siglist = zzuu.free_sigadm_list;
      zzuu.free_sigadm_list = sig;
      BKL_LOCK_RESTORE(msr);
      if(cur_pp->priority <= 32 && !dispatcher_locked)
	UNLOCK_SWAP();
      return NULL;
    }

#if ERROR_CHECKS > 0
  /* Set the magic number in the user mode buffer. */
  usigadm->magic_signo = MAGIC_SIGNO;
#endif
  usigadm->udata = (SIGSELECT *) sig;
  sig->udata = (SIGSELECT *) (usigadm + 1);
#else
  /* Break new buffer from pool. */
  sig = (struct _signal *) pool_brk_lo(pool,
				       hd->size + 1 + sizeof(struct _signal));
  if(sig == NULL)
    {
      /* We are in an alloc_nil() and the pool fragment is full. */
      if(cur_pp->priority <= 32 && !dispatcher_locked)
	UNLOCK_SWAP();
      return NULL;
    }
#endif
  
  /* Set buffer_head. */
  sig->buffer_head = hd;
  sig->next = NULL;
  
  /* Set sender_id, dest_id and size fields. */
  sig->sender_id = (PROCESS) 0;
  sig->dest_id = (PROCESS) 0;
  sig->size = size;
  
  /* Set owner field. */
  sig->owner = pcb_to_pindex(cur_pp); /* Note! Owner is 16-bits. @frbr@ */
  
  /* Fill in the signal number. */
  usig = adm_to_sig(sig);
  usig->sig_no = signo;

#if ERROR_CHECKS > 0
  /* Set the magic number in the buffer. */
  sig->magic_signo = MAGIC_SIGNO;
  /* Set buffer end mark. */
  *(size + (U8 *) usig) = (U8) END_MARK;
#endif
  
  /* Insert buffer in signal buffer list. */
  /* Disable interrupts. */
  sig->siglist = NULL;
  LOCK_SAVE(msr);

  SPIN_LOCK(&pool->smp_spin);

  pool->sigtail->siglist = sig;
  pool->sigtail = sig;
  
  /* Increment owned counter in callers pcb. */
  cur_pp->sig_cnt_owned++;
  
  /* Increment number of allocated signals. */
  SPIN_LOCK(&hd->smp_spin);
  hd->alloc_count++;
  SPIN_UNLOCK(&hd->smp_spin);

  SPIN_UNLOCK(&pool->smp_spin);
  
  if(cur_pp->priority > 32 || dispatcher_locked)
    {
      /*
      ** Called by an interrupt process or with locked dispatcher.
      ** Enable interrupts.
      */
      LOCK_RESTORE(msr);
    }
  else
    {
      /* Unlock dispatcher and enable interrupts. */
      LOCKED_UNLOCK_SWAP();
    }

#if OSE_DEBUG > 0
  if(handlerp.zzprof_alloc)
     handlerp.zzprof_alloc(size, usig,
			   pcb_to_debug_pcb(cur_pp),
			   pcb_to_pid(pool));
#endif

  return usig;
}

/*
** Change owner of a signal buffer and clear any redirection flag.
**
** It is legal to call this function with interrupts disabled or
** with the dispatcher locked, but not with interrupt status pushed.
*/

void
restore(union SIGNAL *sig)
{
  LOCKSTATE msr;
  register struct _signal *sigadm;
  register struct _pcb *old_pcb;
  union SIGNAL *sigp;

  /*
  ** There is a potential race condition here, but we will assume
  ** that the current buffer owner is aware of the restore operation.
  */
  /* Check out buffer pointer and get the signal control block. */
  sigp = sig;
  sigadm = check_buffer(&sigp, OSE_RESTORE);
  if(!sigadm)
    {
      /* Abort on buffer error. */
      return;
    }
  
  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);

  /*
  ** Check for illegal intersegment restore in complex memory models. ??
  ** (We currently don't do this since it might break existing
  ** supervisor mode applications (INET) which operate by restoring buffers
  ** owned by user mode clients.)
  */
#if 0
#if ERROR_CHECKS > 0
#if MEMORY_MODEL == 1
  ON_ERRLEVEL_2()
    {
      /*
      ** Allow a supervisor mode process to do anything in memory model 1.
      ** This is probably OK since that model only supports a single
      ** address space, and few users have memory protection in
      ** supervisor mode.
      */
      if(!zzcur_p->block->supervisor_mode)
	/* Otherwise check that the signal is in the callers pool. */
	if(zzcur_p->cursigpool->segid != sigadm->buffer_head->segid)
	  {
	    /* Enable interrupts. */
	    BKL_LOCK_RESTORE(msr);
	    /* Report illegal intersegment restore error. */
	    os_error(0, OSE_ECANNOT_RESTORE | OSE_RESTORE, (OSERRCODE) sig);	/* Non fatal. */
	    return;
	  }
    }
#endif
#endif
#endif

  /* Skip attempts to restore a free buffer. */
  if(likely(sigadm->owner))
    {
      /* Get pcb of old owner. */
      old_pcb = zzuu.pix_list[sigadm->owner];

      /* Do not restore if owner has died but the pool cleanup has not completed. */
      if(old_pcb->magic == MAGIC_PCB)
	{
	  cpuid_t cpu;
	  (void)cpu;
	  
	  /* Decrement owned counter in owners pcb if owner <> 0. */
#ifdef OSE_SMP
	  /* If the process runs on another cpu, let that one do the job. */
	  cpu = old_pcb->cpu_id;
	  if(cpu != zzkrn_cpu_id())
	    send_krn_event(cpu, KRN_EVT_SIG_COUNT, pcb_to_pid(old_pcb),
			   (void*)(-1 << 16));
	  else
	    old_pcb->sig_cnt_owned--;
#else
	  old_pcb->sig_cnt_owned--;
#endif

	  /* Increment owned counter in new owners pcb. */
	  zzcur_p->sig_cnt_owned++;

	  /* Clear redirection flag if set. */
	  sigadm->dest_id = (PROCESS) 0;

	  /* Set new owner. */
	  sigadm->owner = pcb_to_pindex(zzcur_p); /* Note! Owner is 16-bits. @frbr@ */

	  /* Enable interrupts and quit. */
	  BKL_LOCK_RESTORE(msr);
	  return;
	}
    }

  /* Enable interrupts and report a restore of free buffer error. */
  BKL_LOCK_RESTORE(msr);
  os_error(0, OSE_ECANNOT_RESTORE | OSE_RESTORE, (OSERRCODE) sig);	/* Non fatal. */
}

/*
** Take over the ownership of a certain signal from a specified process.
**
** If sig is currently owned by curr_owner the ownership of the signal will be
** changed to the calling process and any redirection flag will be cleared.
**
** Returns zero if the owner of the signal was sucessfully changed to
** the pid of the calling process, otherwise a non zero value is returned.
*/

static OSBOOLEAN
zzkrn_restore_from(union SIGNAL *sig, PROCESS curr_owner)
{
  LOCKSTATE msr;
  struct _signal *sigadm;
  struct _pcb *owner_pcb;

  /* Check out buffer pointer and get the signal control block. */
  sigadm = check_buffer(&sig, OSE_RESTORE);
  if(!sigadm)
    {
      /* Abort on buffer error. */
      return (OSBOOLEAN)1;
    }
  
  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);

  owner_pcb = safe_pid_to_pcb(curr_owner, 0);

  if(owner_pcb != NULL)
    {
      if(sigadm->owner == pcb_to_pindex(owner_pcb))
	{
	  change_sig_owner(sigadm, zzcur_p);

	  /* Clear redirection flag if set. */
	  sigadm->dest_id = (PROCESS) 0;

	  /* Enable interrupts. */
	  BKL_LOCK_RESTORE(msr);
	  return (OSBOOLEAN) 0;
	}
    }

  (void)check_pid(curr_owner, OSE_RESTORE);

  /* Enable interrupts. */
  BKL_LOCK_RESTORE(msr);
  return (OSBOOLEAN) 1;
}

/*
** Free a buffer to its native pool. Check owner.
**
** It is OK to call this function while the dispatcher is locked,
** provided that we know that no error will occur in check_buffer().
*/

void
free_buf(union SIGNAL **sig)
{
  register struct _signal *sigadm;
  register struct pool_list_entry *hd;
  register struct _pcb *cur_pp;
  struct poolcb *pool;
  PROCESS poolid;

  /* Check out buffer pointer and get the signal control block. */
  sigadm = check_buffer(sig, OSE_FREE_BUF);
  if(!sigadm)
    {
      return;
    }

  /* Locate free list. */
  hd = sigadm->buffer_head;
  pool = hd->poolcbp;
  poolid = pcb_to_pid(pool);

  /* Locate current PCB. */
  cur_pp = zzcur_p;
  
#if OSE_DEBUG > 0
  if(handlerp.zzprof_free_buf)
    handlerp.zzprof_free_buf(sigadm->size, adm_to_sig(sigadm),
			     pcb_to_debug_pcb(cur_pp),
			     poolid);
#endif

  /*
  ** By setting the dest_id field (e.g. the addressee field)
  ** to the PID of the signal pool, we tell pool analyzers
  ** that this signal buffer is now in the freelist. The sender
  ** filed is set to indicate who used to own it.
  */
  sigadm->dest_id = poolid;
  sigadm->sender_id = cur_pp->pid;

  /* Take buffer from user. */
  *sig = NIL;

  /* Link into free list. */
  link_free_buf(pool, hd, sigadm, cur_pp);

  /*
    If no more signals are allocated in the pool and no blocks are
    using the pool, then the pool should be freed.
    This is done by the system daemon, which periodically scans
    all pools to determine if there are dead pools.
    Another possible solution would be to send a signal from free_buf
    to the system daemon, but this would hurt worst case performance
    in free_buf, so we won't do that.
    Refer to create_pool for an alternate approach.
  */
}

/*
** OSADDRESS
** ose_set_pool_max_size(PROCESS pool_id, OSADDRESS max_size,
**                       OSADDRESS frag_size);
** See ose_spi.h.
*/
static OSADDRESS
sysconf_95(OSADDRESS p, OSADDRESS max_size, OSADDRESS frag_size)
{
  PROCESS poolid = (PROCESS)p;
  struct poolcb *poolcb;

  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_SET_POOL_MAX_SIZE))
    return 0;			/* Fail */

  /* Does the pool exist? */
  poolcb = (struct poolcb *)safe_pid_to_pcb(poolid, 1);
  LOCK_SWAP();
  if(!poolcb || poolcb->magic != MAGIC_POOL ||
     poolcb->instance != pid_to_instance(poolid))
    {
      UNLOCK_SWAP();
      return 0;			/* Pool not found. */
    }

  /*
  ** Just return the actual size if the pool is not extendable.
  ** Return the maximum size if caller is a user mode process.
  */
  if(!poolcb->ext_func)
    {
      /* Not extendable but fixed size. Return the actual size. */
      max_size = poolcb->tot_size;
    }
  else if(!zzcur_p->block->supervisor_mode)
    {
      /* User mode. Return the actual max size. */
      max_size = poolcb->max_size;
    }
  else
    {
      /* Set the max size if it is bigger than the current size. */
      if(poolcb->tot_size > max_size)
	{
	  max_size = poolcb->tot_size;
	}
      poolcb->max_size = max_size;

      /*
      ** Set the fragment size if given in the call and there was one before.
      ** The selected fragment size must manage the largest stack and signal.
      ** NOTE THAT SYSTEM AND MAIN POOL DOES NOT USE THE FRAGMENT SIZE
      ** ATTRIBUTE, i.e. it shall be set to zero!
      */
      if(poolcb->frag_size && frag_size)
	{
	  OSADDRESS min_frag;

	  min_frag = poolcb->signal_list[7].size;
	  if(min_frag < poolcb->stack_list[7].size)
	    min_frag = poolcb->stack_list[7].size;
	  /* Align like the program manager does. */
	  min_frag += 1024; 
	  frag_size = (frag_size > min_frag) ? frag_size : min_frag;
	  poolcb->frag_size = frag_size/* & (OSADDRESS) ~(POOL_ALIGNMENT-1)*/;
	}
    }
  
  UNLOCK_SWAP();
  return max_size;
}

/*
**===========================================================================
** 12.4  Undocumented system calls.
**===========================================================================
*/

/* Register myself as the one and only Program Manager. 
** Also, permanently lock the system segment to cpu zero.
*/
static OSADDRESS
sysconf_79(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) p;
  (void) q;
  (void) r;
  
  zzuu.prog_mgr_ = zzcur_p->pid;
  system_segment.migration_inhibited = MIGRATION_MASK_PM;
  return 1;
}

/*
** Read or set the migration_inhibited mask of a segment.
**
** The function is used by the Program Manager and the Bios to
** lock a segment to the currently selected cpu in a SMP system.
** If the set_mask argument is zero, the current value is returned.
** If the set_mask is non-zero, the corresponding bits are set to
** set_value, and the new value is returned. 
** Mask 1 is owned by the Program Manager, mask 2 by Bios.
** If the segment is not found, ~0 is returned.
*/
OSADDRESS
zzkrn_segment_migration_status(PROCESS pid, OSADDRESS set_mask,
			       OSADDRESS set_value)
{
  LOCKSTATE msr;
  struct _pcb *pcb;
  struct segcb *segcb;
  OSADDRESS migration_inhibited = ~0u;

  BKL_LOCK_SAVE(msr);

  /* Find the segment control block. */
  pcb = safe_pid_to_pcb(pid, 1);

  if(pcb)
    {
      segcb = find_segcb(pcb);
      
      if(segcb)
	{
	  migration_inhibited = (segcb->migration_inhibited & ~set_mask) |
	    (set_mask & set_value);
	  segcb->migration_inhibited = migration_inhibited;
	}
    }
  
  BKL_LOCK_RESTORE(msr);
  return migration_inhibited;
}

/*
** Free a buffer to its native pool regardless of current owner.
** This is also the only system call that can be used to free a signal
** that has been given to a send hook.
*/
void
zzkrn_send_hook_free_buf(union SIGNAL *sig)
{
  LOCKSTATE msr;
  register PROCESS cur_pid = zzcur_p->pid;
  register struct _signal *sigadm;
  register struct pool_list_entry *hd;
  struct poolcb *pool;
  PROCESS poolid;
  OSADDRESS safe_read = 0;

  (void)safe_read;

#if ERROR_CHECKS > 0
  ON_ERRLEVEL_2()
    {
      /* Check that we can read the memory pointed out by sig. */
      if ((OSADDRESS)sig/* & (POOL_ALIGNMENT-1)*/
#if SEND_HOOK_GET_MEM_CHECK > 0
	  || get_mem(cur_pid,
		     (OSADDRESS)sig,
		     &safe_read,
		     sizeof(OSADDRESS))
#endif
	  )
	{
	  os_error(0, OSE_EWILD_SIG_POINTER | OSE_FREE_BUF, (OSERRCODE)sig);	/* Non fatal. */
	  return;
	}
    }
#endif

  /* Pick up a pointer to the signal administration block. */
  sigadm = sig_to_adm(sig);
  
#if MEMORY_MODEL > 0

#if ERROR_CHECKS > 0
  ON_ERRLEVEL_2()
    {
      /*
      ** If MEMORY_MODEL > 0 we need to check that we can read the sigadm,
      ** if not this indicates that the udata pointer in the user sigadm
      ** is broken.
      */
      if ((OSADDRESS)sigadm/* & (POOL_ALIGNMENT-1)*/
#if SEND_HOOK_GET_MEM_CHECK > 0
	  || get_mem(cur_pid,
		     (OSADDRESS)sigadm,
		     &safe_read,
		     sizeof(OSADDRESS))
#endif
	  )
	{
	  os_error(0, OSE_EWILD_SIG_POINTER | OSE_FREE_BUF, (OSERRCODE)sig);	/* Non fatal. */
	  return;
	}
    }

  if(adm_to_sig(sigadm) != sig)
    {
      /* Try to locate the buffer that caused the failure. */
      zzrecover_from_pool_crash(sig, OSE_FREE_BUF);
      return;
    }
#endif

#endif

  /* Locate free list. */
  hd = sigadm->buffer_head;
  pool = hd->poolcbp;
  poolid = pcb_to_pid(pool);

  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);

  /*
  ** Check if the signal was sent given to a send hook and that it has
  ** not already been freed.
  */
  if(((OSADDRESS)(sigadm->next) & 0xFFFF) == 0x44A3)
    {
#if OSE_DEBUG > 0
      if(handlerp.zzprof_free_buf) /* zzkrn_send_hook_free_buf in LOCK_SAVE region?? */
	handlerp.zzprof_free_buf(sigadm->size, adm_to_sig(sigadm), 
				 pcb_to_debug_pcb(zzcur_p),
				 poolid);
#endif
      /*
      ** By setting the dest_id field (e.g. the addressee field)
      ** to the PID of the signal pool, we tell pool analyzers
      ** that this signal buffer is now in the freelist. The sender
      ** filed is set to indicate who used to own it.
      */
      sigadm->dest_id = poolid;
      sigadm->sender_id = cur_pid;

      if(likely(hd->ext_handle == NULL))
	{
	  /* Link into free list. */
	  SPIN_LOCK(&hd->smp_spin);
	  sigadm->next = (struct _signal *) hd->hd;
	  hd->hd = sigadm;
	  
	  /* Decrement no of signals allocated in pool. */
	  hd->alloc_count--;
	  SPIN_UNLOCK(&hd->smp_spin);
	}
      else
	{
	  /* Call external free function. */
	  pool->ext_free(hd->ext_handle, adm_to_sig(sigadm));
	  
	  /* Decrement no of signals allocated in pool. */
	  (void)atomic_dec((atomic_t *)&hd->alloc_count);
	}

      /* Enable interrupts. */
      BKL_LOCK_RESTORE(msr);
      return;
    }
  else
    {
#if ERROR_CHECKS > 0
      ON_ERRLEVEL_2()
	{
	  /* Check the magic number and user space end mark. */
	  if((sigadm->magic_signo != MAGIC_SIGNO) ||
	     (END_MARK != *(U8 *)(sigadm->size + (U8 *) sig)))
	    {
	      /* Try to locate the buffer that caused the failure. */
	      check_buffer(&sig, OSE_FREE_BUF);
	    }
	}
#endif

      /* Decrement owned counter in owners pcb if owner <> 0. */
      if(likely(sigadm->owner))
	{
	  register struct _pcb *old_pcb;
	  
	  /* Get pcb of old owner. */
	  old_pcb = zzuu.pix_list[sigadm->owner];

	  /*
	  ** Do not restore if owner has died but the pool cleanup has
	  ** not completed.
	  */
	  if(old_pcb->magic == MAGIC_PCB)
	    {
#if OSE_DEBUG > 0
	      if(handlerp.zzprof_free_buf) /* zzkrn_send_hook_free_buf in LOCK_SAVE region?? */
		handlerp.zzprof_free_buf(sigadm->size, adm_to_sig(sigadm), 
					 pcb_to_debug_pcb(zzcur_p),
					 poolid);
#endif
	      /* Decrement owned counter in owner's pcb. */
#ifdef OSE_SMP
	      /*
	      ** If the process runs on another cpu, let that one do the job.
	      */
	      {
		cpuid_t cpu = old_pcb->cpu_id;
		if(cpu != zzkrn_cpu_id())
		  {
		    send_krn_event(cpu, KRN_EVT_SIG_COUNT, pcb_to_pid(old_pcb),
				   (void*)(-1 << 16));
		  }
		else
		  old_pcb->sig_cnt_owned--;
	      }
#else
	      old_pcb->sig_cnt_owned--;
#endif

	      /* Make system the owner. */
	      sigadm->owner = (PCB_INDEX_T) 0;

	      /*
	      ** By setting the dest_id field (e.g. the addressee field)
	      ** to the PID of the signal pool, we tell pool analyzers
	      ** that this signal buffer is now in the freelist. The sender
	      ** filed is set to indicate who used to own it.
	      */
	      sigadm->dest_id = poolid;
	      sigadm->sender_id = cur_pid;

	      if(likely(hd->ext_handle == NULL))
		{
		  /* Link into free list. */
		  SPIN_LOCK(&hd->smp_spin);
		  sigadm->next = (struct _signal *) hd->hd;
		  hd->hd = sigadm;
		  
		  /* Decrement no of signals allocated in pool. */
		  hd->alloc_count--;
		  SPIN_UNLOCK(&hd->smp_spin);
		}
	      else
		{
		  /* Call external free function. */
		  pool->ext_free(hd->ext_handle, adm_to_sig(sigadm));
		  
		  /* Decrement no of signals allocated in pool. */
		  (void)atomic_dec((atomic_t *)&hd->alloc_count);
		}

	      /* Enable interrupts. */
	      BKL_LOCK_RESTORE(msr);
	      return;
	    }
	}
      
      /* Enable interrupts and report a restore of free buffer error. */
      BKL_LOCK_RESTORE(msr);
      os_error(0, OSE_ECANNOT_RESTORE | OSE_FREE_BUF, (OSERRCODE) sig);	/* Non fatal. */
    }
}

/*
** Change owner of a signal buffer that has been sent trough a
** send_hook.
*/
void
zzkrn_send_hook_restore(union SIGNAL *sig)
{
  LOCKSTATE msr;
  register struct _signal *sigadm;
  OSADDRESS safe_read = 0;

  (void)safe_read;

#if ERROR_CHECKS > 0
  ON_ERRLEVEL_2()
    {
      /* Check that we can read the memory pointed out by sig. */
      if ((OSADDRESS)sig/* & (POOL_ALIGNMENT-1)*/
#if SEND_HOOK_GET_MEM_CHECK > 0
	  || get_mem(zzcur_p->pid,
		     (OSADDRESS)sig,
		     &safe_read,
		     sizeof(OSADDRESS))
#endif
	  )
	{
	  os_error(0, OSE_EWILD_SIG_POINTER | OSE_RESTORE, (OSERRCODE)sig);	/* Non fatal. */
	  return;
	}
    }
#endif

  /* Pick up a pointer to the signal administration block. */
  sigadm = sig_to_adm(sig);
  
#if MEMORY_MODEL > 0

#if ERROR_CHECKS > 0
  ON_ERRLEVEL_2()
    {
      /*
      ** If MEMORY_MODEL > 0 we need to check that we can read the sigadm,
      ** if not this indicates that the udata pointer in the user sigadm
      ** is broken.
      */
      if ((OSADDRESS)sigadm/* & (POOL_ALIGNMENT-1)*/
#if SEND_HOOK_GET_MEM_CHECK > 0
	  || get_mem(zzcur_p->pid,
		     (OSADDRESS)sigadm,
		     &safe_read,
		     sizeof(OSADDRESS))
#endif
	  )
	{
	  os_error(0, OSE_EWILD_SIG_POINTER | OSE_RESTORE, (OSERRCODE)sig);	/* Non fatal. */
	  return;
	}
    }

  if(adm_to_sig(sigadm) != sig)
    {
      /* Try to locate the buffer that caused the failure. */
      check_buffer(&sig, OSE_RESTORE);
      return;
    }
#endif

#endif

  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);

  /*
  ** Check if the signal was sent given to a send hook and that it has
  ** not already been freed.
  */
  if(likely(((OSADDRESS)(sigadm->next) & 0xFFFF) == 0x44A3))
    {
      /* Increment owned counter in new owners pcb. */
      zzcur_p->sig_cnt_owned++;
      
      /* Set new owner. */
      sigadm->owner = pcb_to_pindex(zzcur_p); /* Note! Owner is 16-bits. @frbr@ */

      /*
      ** Make sure we destroy the magic send-hook value in the next
      ** pointer.
      */
      sigadm->next = NULL;

      /* Enable interrupts and quit. */
      BKL_LOCK_RESTORE(msr);
      return;
    }
  
  /* Enable interrupts and report a restore of free buffer error. */
  BKL_LOCK_RESTORE(msr);
  os_error(0, OSE_ECANNOT_RESTORE | OSE_RESTORE, (OSERRCODE) sig);	/* Non fatal. */
}

/*
** Enter or exit a region where the kernel pool is always used for signals,
** and user number zero is selected.
** The region may be entered several times as long as it is exited the
** same number of times.
**
** This function may be used by user implemented supervisor mode functions
** to ensure that signals allocated and received are always located in the
** kernel pool.
**
** A typical case is a page fault handler. When a page fault occurs, the
** current process is trapped to supervisor mode. The page fault trap handler
** may want to send a signal to a file system, requesting some pages to be
** loaded from secondary storage. It is important to avoid new page faults 
** while doing this, such as may occur if the user mode pool is used.
**
** Other cases may involve user implemented system calls that use signals to
** communicate with various server processes
**
** CAUTION! The target interface may activate this function only if the caller
** is in supervisor mode.
**
** The function may be called from any process type and also when the
** dispatcher is locked.
*/

void 
set_suid(OSBOOLEAN make_super)
{
  LOCKSTATE msr;
  register struct _pcb *cur_pp = zzcur_p;
  LOCK_SAVE(msr);
  if(make_super)
    {
      cur_pp->user = (OSUSER) 0;
      cur_pp->use_kernel_pool++;
      cur_pp->cursigpool = &system_pool;
    }
  else
    {
      cur_pp->use_kernel_pool--;
      if(!cur_pp->use_kernel_pool)
	{
	  cur_pp->user = cur_pp->real_user;
	  cur_pp->cursigpool = cur_pp->block->sigpool;
	}
    }
  LOCK_RESTORE(msr);
}

/*
** Allocate space on request from the target interface.
**
** This memory can be used for uarea and other areas required by the
** target interface.
** The memory can not be freed, so a separate free list must be
** maintained by the target interface for space allocated in this way.
** The memory is allocated from the bottom of the kernel pool.
** Returns a pointer to the base of new area.
*/

void *
zzalloc_kernelmemory(OSBUFSIZE size)
{
  if(!zzdispatcher_locked)
    {
      void *p;      
      LOCK_SWAP();
      p = pool_brk_lo(&system_pool, size);
      UNLOCK_SWAP();
      return p;
    }
  else
    {
      return pool_brk_lo(&system_pool, size);
    }
}

/*
** Allocate space on request from the target interface.
**
** This memory should be used for supervisor stacks.
** The memory can not be freed, so a separate free list must be
** maintained by the target interface for space allocated in this way.
** The memory is allocated from the top of the kernel pool.
** Returns a pointer to the base of new stack.
*/

void *
zzkrn_alloc_kernelstack(OSBUFSIZE size)
{
  void *p;
  LOCK_SWAP();
  p = pool_brk_hi(&system_pool, size);
  UNLOCK_SWAP();
  return p;
}

/*
** Extend a pool.
**
** This function is called by an error handler to append a new
** memory fragment to a pool when it has become exhausted. 
**
** The function must be called with interrupts disabled if there is
** a chance of interrupt processes using the pool.
** If called from a non interrupt process, the dispatcher was locked
** by the kernel prior to the error handler invokation.
** The pool_id parameter is an exact copy of the extra information
** parameter given to the error handler.
**
** Extend_pool can also be called spontanously to extend a pool used 
** by the current process. This feature is intentionally undocumented.
** The pool_id is in this case the plain pool pid. In this case,
** the dispatcher is not locked! Hopefully, this use case is now
** only history...
*/

void
extend_pool(OSERRCODE pool_id, OSADDRESS base, OSADDRESS size)
{
  LOCKSTATE msr;
  register struct poolcb *pool;
  struct pool_extension *ext;
  int context = 0;		/* 1=trigged by kernel, 2=spontanous, 0=invalid */
  OSBOOLEAN unlock_dispatcher = 0;
  struct _pcb *pcb = zzcur_p;

  /* Check that the supplied memory area is reasonably large. */
#if ERROR_CHECKS > 0
  if(unlikely(size < 1024))
    {
      os_error(0, OSE_EFRAGMENT_TOO_SMALL | OSE_EXTEND_POOL, size);	/* Non fatal. */
      return;
    }
#endif

  /* Check that the supplied memory fragment does not extend beyond the
   * address space. We use a separete "end" variable due to the fact that
   * the compiler might optimize "if (b+s <= b)" to "if(s <= 0)", and that
   * wouldn't be so good for the usefulness of our test.
   */
#if ERROR_CHECKS > 0
  {
    OSADDRESS end = base + size;
     
    if(unlikely(end <= base))
      {
        os_error(0, OSE_EFRAGMENT_TOO_LARGE | OSE_EXTEND_POOL, (OSERRCODE) size);	/* Non fatal. */
	return;
      }
  }
#endif

  /* Lock the dispatcher if the caller didn't. */
  if(pcb->priority <= 32 && !zzdispatcher_locked)
    {
      unlock_dispatcher = 1;
      LOCK_SWAP();
    }

  /* Locate the pool. */
  pool = (struct poolcb *)safe_pid_to_pcb(pool_id, 2);

  /* Check that the pool contains a proper magic number. */
  if(unlikely(pool->magic != MAGIC_POOL))
    {
      if(unlock_dispatcher)
	UNLOCK_SWAP();
      os_error(1, OSE_EILLEGAL_POOL | OSE_EXTEND_POOL, pool_id);	/* Non fatal system error. */
      return;
    }

  /*
  ** Check if this is a proper call from an error handler.
  ** Note that creating a process in another program may cause a pool extend
  ** in another pool than the current stack pool. This is an unusual scenario.
  */
  if((pool == &system_pool && (pcb->out_of_pool & 1)) ||
     (pcb->out_of_pool & 2))
    {
      /*
      ** Extend action triggered by the kernel. We still need to check
      ** the extend counter below to detect if someone else has
      ** already done an extend.
      */
      context = 1;
    }
  else
    {
      /* Check if this is a proper spontanous call, i.e. a runtime
      ** one-shot reconfiguration of the pool size. This was never
      ** an intended feature, but has been used in some systems...
      */
      if(pool_id == pcb_to_pid(pool) &&
	 (pool == pcb->cursigpool || pool == pcb->block->stackpool) &&
	 (pool != &system_pool || pcb->block->supervisor_mode))
	{
	  /* Proper spontanous call. */
	  context = 2;
	}	
    }

  if(unlikely(context == 0))
    {
      /* Checks failed. Report error and return. */
      if(unlock_dispatcher)
	UNLOCK_SWAP();
      os_error(1, OSE_EILLEGAL_POOL | OSE_EXTEND_POOL, pool_id); /* Non fatal system error. */
      return;
    }
  
  /*
  ** Make sure that the supplied memory is properly aligned.
  */
  {
    OSADDRESS tmp = base;
    base = ((base + (POOL_ALIGNMENT-1))/* & (OSADDRESS) ~(POOL_ALIGNMENT-1)*/);
    size = size - (base - tmp);
    size = size /*& (OSADDRESS) ~(POOL_ALIGNMENT-1)*/;
  }

  /* Disable interrupts in case the user didn't. */
  BKL_LOCK_SAVE(msr);

  /*
  ** If the sequence number does not match the pool extend count,
  ** someone has interrupted us and already performed an extend,
  ** so we just return silently.
  */
  if(context == 1 && (pool_id & 0x7F000000) !=
     ((pool->extend_count << 24) & 0x7F000000))
    {
      goto done;
    }

  /*
  ** Check that the supplied memory is available.
  ** We do this by writing a few locations with the debug pattern.
  ** We don't write all of it, because that would yield
  ** unreasonable interrupt latency.
  ** Also, we only write back existing patterns. This is important
  ** since the pool fragment may already be in use.
  ** We use volatile pointers here to make sure the compiler does not
  ** optimize away this.
  */
  *((volatile char *) base) = *((volatile char *) base);
  *((volatile char *) (base + size - 1)) = *((volatile char *) (base + size - 1));

  /* Build a new fragment record if we are extending the system pool. */
  if(pool == &system_pool)
    {
      ext = (struct pool_extension *) base;
      ext->base = (OSADDRESS) ext;
      ext->limit = base + size;
      ext->magic_frag = MAGIC_POOL;

      SPIN_LOCK(&pool->smp_spin);

      /* Save the final pool low and high pointers in the previous fragment. */
      pool->frag_list->pool_low_p = pool->pool_low_p;
      pool->frag_list->pool_hi_p = pool->pool_hi_p;

      /* Update pool low and high pointers. */
      pool->pool_low_p = ext->base + ((sizeof(struct pool_extension) + (POOL_ALIGNMENT-1)) /*& (OSADDRESS) ~(POOL_ALIGNMENT-1)*/);
      pool->pool_hi_p = ext->limit;
    }
  else
    {
      /*
      ** Not the system pool. Check that we have a free fragment list.
      */
      if(unlikely(zzuu.free_frag_list == NULL))
	{
	  os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 8);	/* Fatal. */
	  /*NOTREACHED*/
	}

      ext = zzuu.free_frag_list;
      zzuu.free_frag_list = ext->frag_list;

      /* Check that this fragment looks ok. */
      if(unlikely(ext->magic_frag != MAGIC_POOL))
	os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 9);	/* Fatal. */

      ext->base = (OSADDRESS) base;
      ext->limit = base + size;

      SPIN_LOCK(&pool->smp_spin);

      /* Save the final pool low and high pointers in the previous fragment. */
      pool->frag_list->pool_low_p = pool->pool_low_p;
      pool->frag_list->pool_hi_p = pool->pool_hi_p;

      /* Update pool low and high pointers. */
      pool->pool_low_p = ext->base; /* Already aligned. */
      pool->pool_hi_p = ext->limit;
    }

  /* Add the memory fragment to the list of all fragments in the pool. */
  ext->frag_list = pool->frag_list;
  pool->frag_list = ext;

  /* Increment the pool extend count. */
  pool->extend_count++;
  pool->tot_size += size;

  SPIN_UNLOCK(&pool->smp_spin);

  /* If this was a spontanous extend_pool, then restore one free fragment. */
  if(context == 2 && NULL == zzuu.free_frag_list)
    {
      BKL_LOCK_RESTORE(msr);
      ext = (struct pool_extension *)
	pool_brk_lo(&system_pool, sizeof(struct pool_extension));
      ext->magic_frag = MAGIC_POOL;
      
      BKL_LOCK_SAVE(msr);
      ext->frag_list = zzuu.free_frag_list;
      zzuu.free_frag_list = ext;
    }

 done:
  
  /* Enable interrupts. */
  BKL_LOCK_RESTORE(msr);

  if(unlock_dispatcher)
    {
      /*
      ** Unlock dispatcher if it was not locked at the entry of
      ** extend_pool().
      */
      UNLOCK_SWAP();
    }
}

/*
** Allocate a signal buffer either from the system pool or from
** the current signal pool. Set signal number in the new buffer.
**
** This version of alloc returns NIL if no pool space can be obtained.
**
** See also: alloc(), s_alloc_nil(), sys_alloc(), locked_alloc().
*/

union SIGNAL *
alloc_nil(OSBUFSIZE size, SIGSELECT signo)
{
  struct _pcb *curr_p = zzcur_p;
  union SIGNAL *sig;

  curr_p->nil_alloc = (OSBOOLEAN)1;
  sig = alloc(size, signo);
  curr_p->nil_alloc = (OSBOOLEAN)0;

  return sig;
}

/*
** Get the signal pool for a process or block.
** The dispatcher is locked by the caller.
*/
static struct poolcb *
get_home_sigpool(PROCESS pid_bid)
{
  struct _pcb *pcb;

  /* Get any alive object. */
  pcb = safe_pid_to_pcb(pid_bid, 1);
  if(pcb)
    {
      switch(pcb->magic)
	{
	case MAGIC_PCB:
	  return pcb->block->sigpool;

	case MAGIC_BCB:
	  return ((struct _block *)pcb)->sigpool;

	case MAGIC_POOL:
	  return (struct poolcb *)pcb;

	default: 
	  break;
	} /* switch */
    }
  return NULL;
}

/*
** Allocate a signal buffer from any signal pool.
** Set signal number in the new buffer.
**
** This version of alloc returns NIL if no pool space can be obtained
** from the current fragment of the pool, i.e. the call will not
** cause a pool extend operation.
** Add Support for interrupt processes in SMP environment, i.e.
** a kill_lock scheme for dying pools on another cpu??
**
** See also: alloc(), alloc_nil(), sys_alloc(), locked_alloc().
*/

union SIGNAL *
s_alloc_nil(PROCESS pid_bid, OSBUFSIZE size, SIGSELECT signo)
{
  union SIGNAL *sig = NIL;
  struct poolcb *requested_pool;
  struct poolcb *saved_pool;
  struct _pcb *curr_p = zzcur_p;
  OSBOOLEAN do_unlock = 0;
  volatile struct poolcb *v_poolcb;

  /* Optimize for explicit pool identifier. */
  v_poolcb = (struct poolcb *)safe_pid_to_pcb(pid_bid, 2);

  if(v_poolcb != NULL && 
     v_poolcb->magic == MAGIC_POOL && 
     v_poolcb->is_static &&
     v_poolcb->instance == pid_to_instance(pid_bid))
    {
      requested_pool = DEVOLATILE(struct poolcb *, v_poolcb);      
    }
  else
    {	
      /* Lock the dispatcher to keep the selected pool alive.  */
      if(curr_p->priority <= 32 && !zzdispatcher_locked)
	{
	  LOCK_SWAP();
	  do_unlock = 1;
	}
      requested_pool = get_home_sigpool(pid_bid);
    }

  /* We will not find a pool for a segment pid or a dead pid. */
  if(likely(requested_pool != NULL))
    {
      /* User mode clients may not use a non-static pool in another segment */
      if(requested_pool->is_static ||
	 curr_p->block->supervisor_mode ||
	 curr_p->segid == requested_pool->segid)
	{
	  saved_pool = curr_p->cursigpool;
	  curr_p->cursigpool = requested_pool;

	  curr_p->nil_alloc = (OSBOOLEAN)1;
	  sig = alloc(size, signo);
	  curr_p->nil_alloc = (OSBOOLEAN)0;

	  curr_p->cursigpool = saved_pool;
	}
    }

  /* Make sure we unlock only if we previously locked. */
  if(do_unlock)
    UNLOCK_SWAP();

  return sig;
}

static OSADDRESS
sysconf_94(OSADDRESS pid_bid, OSADDRESS size, OSADDRESS signo)
{
  return (OSADDRESS)s_alloc_nil((PROCESS)pid_bid, (OSBUFSIZE)size,
				      (SIGSELECT)signo);
}

/*
** The ldm switches to the user pool if required.
**
** The ldm switches from the system pool to the pool of a specified process.
** If pid = 0, the system pool is again selected.
*/

void zzselect_user_pool(PROCESS pid)
{
  struct _pcb *pcb;

  /*
  ** Always switch for memory model 0.
  ** Check the enable flag otherwise.
  */
#if MEMORY_MODEL > 0
  if(local_sysctlblk.allow_pool_copy)
    return;
#endif

  /* Revert to the system pool if no pid provided. */
  if(!pid)
    {
      zzcur_p->cursigpool = zzcur_p->block->sigpool;
      return;
    }

  /* Switch the pool if the target process actually exists. */
  BKL_LOCK();
  if(zzget_debugpcb(pid))
    {
      pcb = pid_to_pcb(pid);
      zzcur_p->cursigpool = pcb->block->sigpool;
    }
  BKL_UNLOCK();
}

/*
** This is the get_sig_info system call.
**
** The call extracts detailed information about a signal buffer
** and searches for the next buffer owned by a specified
** process in a specified pool.
** 		
** The sig_info parameter contains a pointer to a result buffer.
** It is updated with information about the signal buffer.
** Set sig_info to NULL if you don't want that information.
** please note that fields related to the current sigptr can be trusted
** only if the get_sig_info call returns 1 or 2.
** The next field can be trusted only if the get_sig_info call
** returns 1 or 2 or if the sigptr was set to zero.
** 
** The sigptr parameter contains the address of the signal
** buffer to check. Set it to zero to request a search for
** the first signal in the first pool that matches the pool_id
** and pid parameters.
** When traversing a pool, set sigptr to 0 for the first signal.
** Then use the value of the "next" field in the sig_info structure
** for the subsequent calls until the next field becomes zero.
**
** The pool_id and pid parameters are used to specify how to
** search for the next buffer.
** The pool_id parameter specifies which pool to search. A value
** of zero allows the search to continue in any pool.
** The pid parameter, if nonzero, specifies that only buffers
** owned by that process should be searched for. A value of zero
** allows any owner including buffers owned by the system.
**
** As a special case, the pool_id parameter may be set to a
** block id to specify the signal pool associated with that
** block.
**
** Returns a status code with either of these values:
** 
** 0 = Unimplemented system call. No result is obtained.
** 1 = sigptr points to a valid signal buffer
** 2 = sigptr points to a signal buffer which seems to
**     be ok, apart from the fact that it lacks a
**     valid endmark.
** 3 = sigptr points to a signal buffer with a broken
**     (somehow faulty) signal administration block.
** 4 = sigptr is a wild pointer (not pointing at any
**     signal buffer) inside a pool.
** 5 = sigptr is a wild pointer not inside any pool.
** 
** If you are unauthorised to look at the specified signal
** buffer (you don't have the right user number) the result
** will be the same as feeding calling get_sig_info with a
** wild sigptr (i.e. 4).
**
** The system call uses the get_sysconf() framework, which means that
** there is no public entrypoint to this function.
** This system call uses function 23 of get_sysconf().
** The user level prototype is:
** OSBOOLEAN get_sig_info(struct OS_sig_info *sig_info,
** 	     PROCESS pool_id,
** 	     PROCESS pid,
** 	     OSADDRESS sig);
*/

static OSADDRESS
get_nextsig(PROCESS pool_id, PROCESS pid, struct _signal *sigadm, LOCKSTATE msr)
{
  struct _signal *nextsig;
  struct poolcb *pool;
  struct _pcb *pcb;
  PCB_INDEX_T pindex;
  PCB_INSTANCE_T instance;

  /* Find out which pool we are searching. */
  if(pool_id)
    {
      /* Get the controlblock. */
      pool = (struct poolcb *)safe_pid_to_pcb(pool_id, 1);

      /* Check if it's alive. */
      if(pool == NULL)
	return 0;

      /* If we have a block id, then pick up the signal pool of that block. */
      if(pool->magic == MAGIC_BCB)
	pool = ((struct _block *) pool)->sigpool;
      /* Check that we have a proper process id. */
      else if(pool->magic != MAGIC_POOL)
	return 0;
      /*
      ** Terminate the search if we were asked to continue the search in a
      ** pool other than that of the current signal.
      ** We could perhaps treat this consistency problem in several ways,
      ** but the situation is probably an indication of a user level mistake.
      */
      if(sigadm)
	{
	  if(pool != sigadm->buffer_head->poolcbp)
	    return 0;
	}
      else
	{
	  /*
	  ** This is a first search request, get the first dummy signal
	  ** pointer.
	  */
	  sigadm = pool->siglist;
	}
    }
  else
    {
      if(sigadm)
	{
	  /* Search the pool of the last signal if nothing else is said. */
	  pool = sigadm->buffer_head->poolcbp;
	}
      else
	{
	  /*
	  ** Scan all pools if we have neither a pool id nor a signal
	  ** pointer.
	  */
	  pool = zzuu.pool_list;

	  /*
	  ** This is a first search request, get the first dummy signal
	  ** pointer.
	  */
	  sigadm = pool->siglist;
	}
    }

  /* Unpack the process id of the owner to filter out. */
  if(pid)
    {
      /* Does pid point out a live process? */
      pcb = safe_pid_to_pcb(pid, 0);

      /* If not return now. */
      if(pcb == NULL)
	return 0;

      pindex = pcb_to_pindex(pcb);
      
    }
  else
    pindex = (PCB_INDEX_T) 0; /* Remove compiler warning */

  /* Pick up the first siglist next pointer. */
  nextsig = sigadm->siglist;

  /* Pick up the instance number of the pool. */
  instance = pcb_to_instance(pool);

  /* Loop over all pools. */
  while(pool)
    {
      /*
      ** Enable interrupts, redisable it and check that the pool
      ** is still present.
      ** Abort if a pool is lost in the middle of a search.
      */
      BKL_LOCK_RESTORE(msr); BKL_LOCK_SAVE(msr);
      if(instance != pcb_to_instance((volatile struct poolcb *)pool))
	return 0;
      /* Switch to the next pool if we are at the end of the current. */
      if(!nextsig)
	{
	  /* But only if the pool was unspecified. */
	  if(pool_id)
	    return 0;
	  pool = pool->next;
	  if(pool)
	    {
	      /* FIXME! Shouldn't we check if the new pool is alive @frbr@ */
	      instance = pcb_to_instance(pool);
	      nextsig = pool->siglist->siglist;
	      continue;
	    }
	  return 0;
	}
      /* Check if the signal owner matches the specified pid. */
      if(pid)
	{
	  if(nextsig->owner == pindex) /* Note! Owner is 16-bits. @frbr@ */
	    break;
	  else
	    nextsig = nextsig->siglist;
	}
      /* Take any signal if no owner pid was specified. */
      else
	break;
    }

  /* Return the found signal if any. */
  return (OSADDRESS) adm_to_sig(nextsig);
}

static OSADDRESS
sysconf_23(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  LOCKSTATE msr;
  OSADDRESS *pars = (OSADDRESS *) q;
  struct OS_sig_info *sig_info = (struct OS_sig_info *) p;
  PROCESS pool_id = (PROCESS) pars[0];
  PROCESS pid = (PROCESS) pars[1];
  OSADDRESS sig = (OSADDRESS) pars[2];

  struct poolcb *pool;
  struct pool_extension *frag = (struct pool_extension *) 0;
  OSBOOLEAN ecode = (OSBOOLEAN) 1;
  struct _signal *ssigadm;
  struct _signal *usigadm;
  
  (void) r; /* Remove ``not used arg'' compiler warning. */
  
  /* Set fields in the output structure to safe values. */
  if(sig_info)
    {
      sig_info->next = (OSADDRESS) 0;
    }

  /*
  ** Check if we should only perform a search.
  */
  if(!sig)
    {
      ecode = (OSBOOLEAN) 5;
      ssigadm = 0;
      goto search_only;
    }

  /* Locate the pool that contains the buffer. */
  /* Look out! There might be several in various segments when virtual memory ??*/
  for(pool = zzuu.pool_list; pool; pool = pool->next)
    {
      for(frag = pool->frag_list; frag; frag = frag->frag_list)
	{
	  /* Check if the buffer is in this fragment. */
	  if(
	     (frag->base <= (OSADDRESS) sig) &&
	     (frag->limit > (OSADDRESS) sig)
	     )
	    break;
	}
      if(frag)
	break;
    }

  /* We have a wild pointer outside any pool if no fragment matches the pointer. */
  if(!frag)
    {
      /* We are done here if sigptr is not zero. */
      return 5;
    }

  /* Pick up the user space sigadm pointer. */
  usigadm = ((struct _signal *) sig) - 1;

  /* Traverse the pool looking for the administration block that matches the buffer. */
#if MEMORY_MODEL > 0
  for(ssigadm = pool->siglist->siglist; ssigadm; ssigadm = ssigadm->siglist)
    {
      /* Check that this adm block is still in the system pool. */
      {
	for(frag = system_pool.frag_list; frag; frag = frag->frag_list)
	  {
	    /* Check if the adm block is in this fragment. */
	    if(
	       (frag->base <= (OSADDRESS) ssigadm) &&
	       (frag->limit > (OSADDRESS) ssigadm)
	       )
	      break;
	  }
	/* Report a fatal error if the system pool was corrupted before the current buffer. */
	if(!frag)
	  /* Report as a wild pointer in a pool if the system pool was damaged. */
	  return 4;
      }
      /* Check the supervisor space magic number. */
#if ERROR_CHECKS > 0
      if(ssigadm->magic_signo != MAGIC_SIGNO)
	{
	  /* Report as a wild pointer in a pool if the system pool was damaged. */
	  return 4;
	}
#endif
      /* See if we found the correct adm block. */
      if(ssigadm->udata == (SIGSELECT *) sig)
	break;
    }
#else
  /* These are the same in memory model 0. */
  ssigadm = usigadm;
#endif

  /* Report a wild pointer error if we couldn't find the adm block. */
  if(!ssigadm)
    {
      /* Report as a wild pointer in a pool if the system pool was damaged. */
      return 4;
    }

  /*
  ** Check for errors in the user mode adm block.
  ** This is a fatal error in memory model 0.
  */
#if MEMORY_MODEL > 0
#if ERROR_CHECKS > 0
  if(usigadm->magic_signo != MAGIC_SIGNO)
    ecode = 3;
#endif
  if(usigadm->udata != (SIGSELECT *) ssigadm)
    ecode = 3;
#else
#if ERROR_CHECKS > 0
  if(usigadm->magic_signo != MAGIC_SIGNO)
    return 4;
#endif
#endif

#if ERROR_CHECKS > 0
  /* Check for a missing end mark. This is not a fatal error. */
  if(END_MARK != *(U8 *)(ssigadm->size + (U8 *) sig))
    ecode = (OSBOOLEAN) 2;
#endif

  /* Time to update the sig_info structure. */
  if(sig_info)
    {
      PCB_INDEX_T pindex;
      PROCESS owner = 0;
      
      /*
      ** Check that the owner's process index is valid. Otherwise we'll
      ** report a wild pointer in a pool.
      */
      pindex = ssigadm->owner;
      if (pindex >= zzuu.next_free_pix)
	  return 4;
      if(pindex)
	{
	  /* Pick up pcb pointer. */
	  struct _pcb *pcb = (struct _pcb *) zzuu.pix_list[pindex];
	  
	  /* Check that we have a proper process id. */
	  if((pcb->magic & ~MAGIC_KILLED) != MAGIC_PCB)
	    return 4;

	  /* We have a valid owner. */
	  owner = pcb->pid;
	}
      
      /*
      ** Check if the adressee field is set to the poolcb, in which
      ** case we will set the owner to 0 and consider the signal to be
      ** in the free list. We will report a wild pointer in a pool if
      ** the dest_id (i.e. addressee) field is invalid.
      */
      pindex = pid_to_pindex(ssigadm->dest_id);
      if (pindex >= zzuu.next_free_pix)
	return 4;
      else if(pindex)
	{
	  /* Pick up pcb pointer. */
	  struct _pcb *pcb = (struct _pcb *) zzuu.pix_list[pindex];
	  
	  /* Check that we have a pool id. */
	  if(pcb->magic == MAGIC_POOL)
	    owner = 0;
	}
      
      sig_info->sig_no = *(SIGSELECT *) sig;
      sig_info->owner = owner;
      sig_info->sender_pid = ssigadm->sender_id
	? ssigadm->sender_id
	: owner;
      sig_info->addressee_pid = ssigadm->dest_id
	? ssigadm->dest_id
	: owner;
      sig_info->sig_size = (OSBUFSIZE) ssigadm->size;
      
      /*
      ** Is this the correct value. Should we account for the system
      ** pool adm block as well??
      */
      sig_info->size_in_pool = (OSBUFSIZE) (ssigadm->buffer_head->size +
					    sizeof(struct _signal) + 1);
    }

search_only:

  /* done here if no search is needed. */
  if(!sig_info)
    return ecode;

  /* Search for the next buffer available. */
  /* I guess we shouldn't disable interrupts here, but what other solution is there ?? */
  BKL_LOCK_SAVE(msr);
  sig_info->next = get_nextsig(pool_id, pid, ssigadm, msr);
  BKL_LOCK_RESTORE(msr);

  /* Return the result. */
  return ecode;
}

/*
** This is the get_pool_list system call.
**
** Lists all pools that are available to the specified
** user number. If user 0 is specified, this indicates
** the caller's user number. If user 0 is specified and
** the caller is a superuser, then all pools in the
** system are listed.
**
** get_pool_list allocates and returns a signal buffer
** with a structure containing the list of pool ID's. The
** buffer must be freed by the caller when information
** has been extracted.
**
** The system call uses the get_sysconf() framework, which means that
** there is no public entrypoint to this function.
** This system call uses function 24 of get_sysconf().
** The user level prototype is:
** struct OS_pid_list *get_pool_list(OSUSER user);
*/

static OSADDRESS
sysconf_24(OSADDRESS pp, OSADDRESS q, OSADDRESS r)
{
  LOCKSTATE msr;
#if 0
  OSUSER user = (OSUSER) pp;
#endif
  struct OS_pid_list *pool_list;
  struct poolcb *pool;
  U32 count;
  U32 scount;
  PROCESS *p;

  /* Remove ``not used args'' compiler warnings. */
  (void) pp;
  (void) q;
  (void) r;

  /* Count the number of pools. */
  count = 0;
  BKL_LOCK_SAVE(msr);
  for(pool = zzuu.pool_list; pool; pool = pool->next)
    {
      /* We should check user privileges here. ??*/
      count++;
    }
  BKL_LOCK_RESTORE(msr);

  /* Allocate a return buffer. */
 retry:
  pool_list = (struct OS_pid_list *)
    alloc(sizeof(struct OS_pid_list) + count * sizeof(PROCESS), 0);

  /* Get the pool id list. */
  scount = 0;
  p = pool_list->list;
  BKL_LOCK_SAVE(msr);
  for(pool = zzuu.pool_list; pool; pool = pool->next)
    {
      /* We should check user privileges here. ??*/
      scount++;
      /* Store the pid only if we have space for it. */
      if(scount <= count)
	*p++ = pcb_to_pid(pool);
    }
  BKL_LOCK_RESTORE(msr);

  /* Retry the whole operation if there are more pools now than before. */
  if(scount > count)
    {
      count = scount;
      free_buf_type(struct OS_pid_list *, &pool_list);
      goto retry;
    }

  /* Update actual list size and exit. */
  pool_list->count = scount;
  return (OSADDRESS) pool_list;
}

/*
** This is the get_poolcb system call.
**
** Interrogates the status of the specified pool. This
** information is returned to the caller in a buffer.
** Note that it is legal to pass an invalid pool ID to
** this system call.
**
** Returns either NULL if the call failed or a buffer
** containing an OS_poolcb structure with information
** about a specified pool. The buffer must be freed 
** by the caller when information has been extracted.
**
** The values returned in the OS_poolcb structure are:
**
**	tot_size:
**        ---------
**        The total memory size (in bytes) of all pool fragments.
**	        
**        free_size:
**        ----------
**        The number of bytes available for further use in the pool.
**        Note that this do not include free buffers in free-lists.
**	        
**        fragment_info:
**        --------------
**        values[fragment_info + 0] == Number of fragments
**	        
**        values[fragment_info + 1] == BaseAddr of fragment 0
**        values[fragment_info + 2] == Size of fragment 0
**        values[fragment_info + 3] == Bytes used for stacks in
**	        			     fragment 0
**        values[fragment_info + 4] == Bytes used for signals in
**	        			     fragment 0
**        values[fragment_info + 6] == BaseAddr of fragment 1
**        ...
**              
**        stk_conf_entries:
**        -----------------
**        Number of configured stacksizes for this pool.
**	        
**        sig_conf_entries:
**        -----------------
**        Number of configured signalsizes for this pool.
**	        
**        stk_conf_sizes:
**        ---------------
**        Only valid if stk_conf_entries > 0
**	        
**        values[stk_conf_sizes] ==
**           Smallest configured stack size in this pool.
**	        
**        values[stk_conf_sizes + stk_conf_entries - 1] ==
**           Largest configured stack size in this pool.
**	        
**        sig_conf_sizes:
**        ---------------
**        See stk_conf_sizes, change stk to sig and stack to
**        signal.
**	        
**        stk_alloc_sizes:
**        ----------------
**        Only valid if stk_conf_entries > 0
**	        
**        values[stk_alloc_sizes] ==
**           Number of alloced stacks of the smallest configured
**           stack size in this pool.
**	        
**        values[stk_alloc_sizes + stk_conf_entries - 1] ==
**           Number of alloced stacks of the largest configured
**           stack size in this pool.
**	        
**        sig_alloc_sizes:
**        ----------------
**        See stk_alloc_sizes, change stk to sig and stack to
**        signal.
**	        
**        values:
**        -------
**        Variable sized array used to store all other info.
**
** The system call uses the get_sysconf() framework, which means that
** there is no public entrypoint to this function.
** This system call uses function 25 of get_sysconf().
** The user level prototype is:
** struct OS_poolcb *get_poolcb(PROCESS pool_id);
*/

static OSADDRESS
sysconf_25(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  LOCKSTATE msr;
  PROCESS pid = (PROCESS) p;
  struct OS_poolcb *pc;
  PCB_INSTANCE_T instance;
  struct poolcb *pool;
  OSADDRESS tot_size;
  OSADDRESS frag_count;
  struct pool_extension *frag;
  OSADDRESS *value;

  /* Remove ``not used args'' compiler warnings. */
  (void) q;
  (void) r;

  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);

  /* Get the controlblock. */
  pool = (struct poolcb *)safe_pid_to_pcb(pid, 1);
  
  /* Check that we have a proper pool id. */
  if(pool == NULL || pool->magic != MAGIC_POOL)
    {
      /* ...otherwise we'll return NULL. */
      BKL_LOCK_RESTORE(msr);
      return (OSADDRESS) NULL;
    }

  /* We might check user numbers here. ?? */

  /* Save the instance number for later. */
  instance = pcb_to_instance(pool);
  
  /* Calculate the total pool size. */
 retry:
  tot_size = 0;
  frag_count = 0;
  for(frag = pool->frag_list; frag; frag = frag->frag_list)
    {
      /* Check that this fragment looks ok. */
      if(frag->magic_frag != MAGIC_POOL)
	os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 18);	/* Fatal. */
      frag_count++;
      tot_size += (frag->limit - frag->base);
    }

  /* Allocate a user level pool control block structure. */
  BKL_LOCK_RESTORE(msr);
  pc = (struct OS_poolcb *) alloc(sizeof(struct OS_poolcb) +
				  sizeof(OSADDRESS) * (32 + 1 + (frag_count * 4)), 0);

  /* Set basic structure fields. */
  pc->tot_size = tot_size;
  pc->free_size = pool->pool_hi_p - pool->pool_low_p;
  pc->stk_conf_entries = 0;
  pc->sig_conf_entries = 0;
  pc->stk_conf_sizes = 0;
  pc->sig_conf_sizes = 8;
  pc->stk_alloc_sizes = 16;
  pc->sig_alloc_sizes = 24;
  pc->fragment_info = 32;

  value = &pc->values[0];
  
  /* Get the stack size list. */
  value[0] = pool->stack_list[0].size;
  value[1] = pool->stack_list[1].size;
  value[2] = pool->stack_list[2].size;
  value[3] = pool->stack_list[3].size;
  value[4] = pool->stack_list[4].size;
  value[5] = pool->stack_list[5].size;
  value[6] = pool->stack_list[6].size;
  value[7] = pool->stack_list[7].size;
  
  /* Get the signal size list. */
  value[8] = pool->signal_list[0].size;
  value[9] = pool->signal_list[1].size;
  value[10] = pool->signal_list[2].size;
  value[11] = pool->signal_list[3].size;
  value[12] = pool->signal_list[4].size;
  value[13] = pool->signal_list[5].size;
  value[14] = pool->signal_list[6].size;
  value[15] = pool->signal_list[7].size;
  
  /* Get the current stack count list. */
  value[16] = pool->stack_list[0].alloc_count;
  value[17] = pool->stack_list[1].alloc_count;
  value[18] = pool->stack_list[2].alloc_count;
  value[19] = pool->stack_list[3].alloc_count;
  value[20] = pool->stack_list[4].alloc_count;
  value[21] = pool->stack_list[5].alloc_count;
  value[22] = pool->stack_list[6].alloc_count;
  value[23] = pool->stack_list[7].alloc_count;
  
  /* Get the current signal count list. */
  value[24] = pool->signal_list[0].alloc_count;
  value[25] = pool->signal_list[1].alloc_count;
  value[26] = pool->signal_list[2].alloc_count;
  value[27] = pool->signal_list[3].alloc_count;
  value[28] = pool->signal_list[4].alloc_count;
  value[29] = pool->signal_list[5].alloc_count;
  value[30] = pool->signal_list[6].alloc_count;
  value[31] = pool->signal_list[7].alloc_count;
  
  /* Check that the pool is still present and that no extension took place. */
  BKL_LOCK_SAVE(msr);
  if(pcb_to_instance(pool) != instance)
    {
      /* Return NULL if the pool has been killed. */
      BKL_LOCK_RESTORE(msr);
      free_buf_type(struct OS_poolcb *, &pc);
      return (OSADDRESS) NULL;
    }

  /* Retry the operation if a pool extension took place. */
  if(pool->extend_count != (frag_count - 1))
    {
      BKL_LOCK_RESTORE(msr);
      free_buf_type(struct OS_poolcb *, &pc);
      BKL_LOCK_SAVE(msr);
      goto retry;
    }

  /* Get the fragment list. */
  {
    OSADDRESS *t = &value[32];
    *t++ = frag_count;
    frag = pool->frag_list;
    *t++ = frag->base; /* Lowest addr in fragment. */
    *t++ = frag->limit - frag->base; /* No of bytes in fragment. */
    *t++ = frag->limit - pool->pool_hi_p; /* Bytes used for stacks. */
    *t++ = pool->pool_low_p - frag->base; /* Bytes used for signals. */
    frag = frag->frag_list;
    for(; frag; frag = frag->frag_list)
      {
	*t++ = frag->base;		 /* Lowest addr in fragment. */
	*t++ = frag->limit - frag->base; /* No of bytes in fragment. */
	*t++ = frag->limit - frag->pool_hi_p; /* Bytes used for stacks. */
	*t++ = frag->pool_low_p - frag->base; /* Bytes used for signals. */
      }
  }

  /* Enable interrupts. */
  BKL_LOCK_RESTORE(msr);

  /* Adjust xxx_conf_entries to actual values. */
  if(pool->no_of_stack_users)
    {
      int i;
      pc->stk_conf_entries++;
      for(i = 1; i < 8; i++)
	{
	  if( pool->stack_list[i].size != pool->stack_list[i - 1].size )
	    pc->stk_conf_entries++;
	}
    }

  if(pool->no_of_sig_users)
    {
      int i;
      pc->sig_conf_entries++;
      for(i = 1; i < 8; i++)
	{
	  if( pool->signal_list[i].size != pool->signal_list[i - 1].size )
	    pc->sig_conf_entries++;
	}
    }

  /* Done. */
  return (OSADDRESS) pc;
}

/*
** This is the get_stk_poolid system call.
**
** Returns the ID of the stack-pool associated with a
** specified block.
**
** The system call uses the get_sysconf() framework, which means that
** there is no public entrypoint to this function.
** This system call uses function 26 of get_sysconf().
** The user level prototype is:
** PROCESS get_stk_poolid(PROCESS bid);
*/

static OSADDRESS
sysconf_26(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  LOCKSTATE msr;
  register PROCESS pid = (PROCESS) p;
  register struct _pcb *pcb;

  /* Remove ``not used args'' compiler warnings. */
  (void) q;
  (void) r;

  /* Optimized check on current process. */
  if(pid == zzcur_p->pid)
    return pcb_to_pid(zzcur_p->block->stackpool);

  /* Locate the block or process descriptor. */
  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid_bid(pid, OSE_GET_STK_POOLID);
  if(pcb == NULL)
    {
      return pid;
    }

  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);

  /* Check that the block/process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      BKL_LOCK_RESTORE(msr);
      return pid;
    }

  if(pcb->magic != MAGIC_BCB)
    {
      /* Pick up the block id. */
      pid = pcb_to_pid(pcb->block);
    }

  /* Pick up the stack pool. */
  pid = pcb_to_pid(((struct _block *) pid_to_pcb(pid))->stackpool);

  BKL_LOCK_RESTORE(msr);

  /* Return the id of the stack pool. */
  return pid;
}

/*
** This is the get_sig_poolid system call.
**
** Returns the ID of the signal-pool associated with a
** specified block.
**
** The system call uses the get_sysconf() framework, which means that
** there is no public entrypoint to this function, except for the soft kernel,
** where the purify integration requires it to be exported.
** This system call uses function 27 of get_sysconf().
** The user level prototype is:
** PROCESS get_sig_poolid(PROCESS bid);
*/

static OSADDRESS
sysconf_27(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  LOCKSTATE msr;
  register PROCESS pid = (PROCESS) p;
  register struct _pcb *pcb;

  /* Remove ``not used args'' compiler warnings. */
  (void) q;
  (void) r;

  /* Optimized check on current process. */
  if(pid == zzcur_p->pid)
    return pcb_to_pid(zzcur_p->block->sigpool);

  /* Locate the block or process descriptor. */
  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid_bid(pid, OSE_GET_SIG_POOLID);
  if(pcb == NULL)
    {
      return pid;
    }

  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);

  /* Check that the block/process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      BKL_LOCK_RESTORE(msr);
      return pid;
    }

  if(pcb->magic != MAGIC_BCB)
    {
      /* Pick up the block id. */
      pid = pcb_to_pid(pcb->block);
    }

  /* Pick up the signal pool. */
  pid = pcb_to_pid(((struct _block *) pid_to_pcb(pid))->sigpool);

  BKL_LOCK_RESTORE(msr);

  /* Return the id of the signal pool. */
  return pid;
}

/*
*****************************************************************************
** 13 ERROR MANAGEMENT.
*****************************************************************************
*/

/*
**===========================================================================
** 13.1  Initialization.
**===========================================================================
*/

/*
**===========================================================================
** 13.2  Local functions.
**===========================================================================
*/

/*
** This function wraps the callout to the test error handler
** before calling the system error handler as the last handler.
** dbghook_info:
** 1:  when called from interrupt and timer processes, and in other cases
**     where the current dispatcher lock state must be preserved.
** 0:  when called from a prioritized or background process.
** -1: when the dbghook must not be called.
*/

static OSADDRESS
handle_system_error(int dbghook_info,
		    OSBOOLEAN user_called,
                    OSERRCODE ecode,
                    OSERRCODE extra)
{
#if OSE_DEBUG > 0
  struct _pcb *curp = zzcur_p;

  if(curp && zzuu.debugger_enabled && dbghook_info >= 0)
    {
      /* Call the system error debug hook if actual error type enabled. */
      if(curp->debug_flags & (U32)(user_called ? 0x10 : 0x20))
	{
	  zzsystem_error_dbghook(pcb_to_debug_pcb(curp), dbghook_info,
				 user_called, ecode, extra);
	}
    }
#else
  (void)dbghook_info;
#endif
  
  /* Run the test error handler if present. (Installed by sysconf_68) */
  if (test_err_handler && test_err_handler(user_called, ecode, extra))
  {
     return 1;
  }

  return handlerp.zzcall_system_errh(user_called, ecode, extra);   
}

/*
** This is the main error handling routine in POSE.
**
** It is called from os_error() and error2().
**
** fatal = 0 for a warning, 1 for an error that can only be handled by
** the system error handler, 2 for a fatal kernel error.
**
** user_called is 1 when this function is called from any publicly
** available error reporting system calls (such as error2()). In these
** cases are the fatal flag always set to 0.
**
** Return code is supplied by the error handler.
**
** Interrupts are disabled by handle_error() for fatal kernel errors.
**
*/

static OSADDRESS
handle_error(int       fatal,
	     OSERRCODE ecode,
	     OSERRCODE extra,
	     OSBOOLEAN user_called)
{
  void *stack;
  OSADDRESS stacksize;
  int dispatcher_locked;
  U8 out_of_pool = 0;
  struct _pcb *cur_p;

  /*
  ** Make sure that we have an error handler and a kernel halted hook.
  ** This might not be the case if an error occurs in a start handler
  ** or as a result of a spurious interrupt.
  */
  if(!handlerp.zzcall_system_errh)
    handlerp.zzcall_system_errh = zzcall_system_errh;
  if(!handlerp.zzkrn_halted_hook)
    handlerp.zzkrn_halted_hook = zzkrn_halted_hook;

  /*
  ** Make sure the kernel is up and running, otherwise issue an
  ** fatal OSE_EEARLY_ERROR.
  */
  cur_p = zzcur_p;
  if(cur_p == NULL)
    {
      LOCK();
      /*
      ** Check if the error is already in the correct OSE_EEARLY_ERROR
      ** form, or if we need to "rebuild" it.
      */
      if(user_called || ((ecode & OSE_EERROR_MASK) != OSE_EEARLY_ERROR))
	{
	  OSADDRESS xtra[4];
	  
	  xtra[0] = ecode; /* Original error code. */
	  xtra[1] = extra; /* Original extra code. */
	  xtra[2] = 0UL;   /* No file info.        */
	  xtra[3] = 0UL;   /* No line info.        */
	  ecode = OSE_EEARLY_ERROR | OSE_EFATAL_MASK;
	  extra = (OSERRCODE)xtra;
	}
      (void)handle_system_error(-1, (OSBOOLEAN) 0, ecode, extra);
      LOCK();
      handlerp.zzkrn_halted_hook();
    }
  
  /*
  ** Check that the dispatcher is not locked. This can sometimes happen
  ** if an error occurs in zzsys_send_w_s() or sys_free_buf(), or as
  ** a result of unexpected kernel bugs. To avoid big bang crashes, we check
  ** the lock flag here, and if it is set we set the fatal parameter to 2.
  **
  ** Note however that the "out of memory" errors should be called with a
  ** locked dispatcher, so we must treat those codes separately.
  */
  dispatcher_locked = zzdispatcher_locked;

  if(!user_called)
    {
      switch(ecode & OSE_EERROR_MASK)
	{
	case OSE_ENO_KERN_SPACE:
	  out_of_pool = 1;	/* system pool */
	  break;
	case OSE_ENO_USER_SIGSPACE:
	case OSE_ENO_USER_STACKSPACE:
	  out_of_pool = 2;	/* local pool */
	  break;
	default:
	  if(dispatcher_locked && cur_p->priority <= 32)
	    {
	      fatal = 2;
	    }
	  break;
	}
    }
  else if(dispatcher_locked && cur_p->priority <= 32)
    {
      /* User called and locked dispatcher seems like an odd case? */
      fatal = 2;
    }

  /*
  ** Disable interrupts permanently for fatal errors.
  */
  if(fatal > 1)
    {
#ifdef OSE_SMP
      /*
      ** Drop the BKL to avoid errors in system calls.
      ** Test this code with/without lock assert??
      */
      if(zzinterrupts_locked)
	BKL_UNLOCK_KEEP_INT_MASK();
#endif
      LOCK();

      /* Add the fatal flag to the error_code. */
      ecode |= OSE_EFATAL_MASK;
    }

  /* Save error info for ose_get_error_info(). */
  cur_p->error_origin = user_called;
  cur_p->error_ecode = ecode;
  cur_p->error_extra = extra;

  /*
  ** Trace the error to the debugger.
  ** Different entrypoints depending on if we are in an interrupt or
  ** non interrupt process.
  **
  ** We treat fatal errors as if they appear in an interrupt process.
  ** This is because the LDM will then preserve interrupt status
  ** properly.
  **
  ** We must also treat errors that appear while the dispatcher is
  ** locked as if they happened inside an interrupt process, thereby
  ** preventing the *_dbghook() to accidently unlock the scheduler.
  */
#if OSE_DEBUG > 0
  if(zzuu.debugger_enabled)
    {
      if (!user_called)
	{
	  /* Check for trace OSE error request. */
	  if(cur_p->debug_flags & 0x20)
	    {
	      if(fatal > 1 || cur_p->priority > 32 || dispatcher_locked)
		zzioseerror_dbghook(pcb_to_debug_pcb(cur_p),
				    ecode, extra);
	      else
		zzoseerror_dbghook(pcb_to_debug_pcb(cur_p),
				   ecode, extra);
	    }
	}
      else
	{
	  /* Check for trace user error request. */
	  if(cur_p->debug_flags & 0x10)
	    {
	      if(cur_p->priority > 32)
		zzierror_dbghook(pcb_to_debug_pcb(cur_p),
				 ecode, extra);
	      else
		zzerror_dbghook(pcb_to_debug_pcb(cur_p),
				ecode, extra);
	    }
	}
    }
#endif

  /*
  ** Report error to the system error handler directly if recursive
  ** error calls were detected. Pass attempted error code to error
  ** handler as a parameter.
  **
  ** An out-of-pool error is treated as recursive only if we have already
  ** reported it for the same type of pool, system resp. local pool.
  ** This is because we need to extend pools also when we ran out of pool
  ** in an error handler. Also note that when we extend a local pool,
  ** this may cause a recursive extend of the system pool. 
  **
  ** All other errors are treated as recursive if we were already
  ** handling an error.
  */
  if((cur_p->out_of_pool & out_of_pool) ||
     (cur_p->err_handl_level != 0 && out_of_pool == 0))
    {
      OSERRCODE recode = user_called
	 ? OSE_ERECURSIVE_ERROR
	 : OSE_ERECURSIVE_SYSERROR;

      /* Make a recursive out-of-pool for the same pool type fatal. */
      if(out_of_pool && (cur_p->err_handl_level < 2))
	{
	  if(0 == cur_p->err_handl_level)
	    zzupdate_err_handl_level(2);
           cur_p->err_handl_level = 2;
	}
      

      /* Add the fatal flag if we get several levels of recursive errors. */
      if(cur_p->err_handl_level > 1)
	{
	  LOCK();
	  recode |= OSE_EFATAL_MASK;
	}

      /*
      ** Loop for ever if the system error handler could not handle
      ** the error.
      */
      for(;;)
	{
	  /*
	  ** Don't call the error handler again if it ignored the last
	  ** fatal error.
	  */
	  if(cur_p->err_handl_level > 2)
          {
             handlerp.zzkrn_halted_hook();
          }
	  if(1 == ++cur_p->err_handl_level)
	    zzupdate_err_handl_level(1);

	  if(fatal > 1 || cur_p->priority > 32 || dispatcher_locked)
	    (void)handle_system_error(1, (OSBOOLEAN)0, recode, ecode);
	  else
	    (void)handle_system_error(0, (OSBOOLEAN)0, recode, ecode);
	  LOCK();
	  recode |= OSE_EFATAL_MASK;
	}
    }

  /*
  ** Report fatal errors directly to the system error handler.  By
  ** setting the err_handl_level in the process to 2, we make sure
  ** that any new error trigged from within the system error handler
  ** will be reported as being both fatal and recursive.
  */
  if(fatal > 1)
    {
      if(0 == cur_p->err_handl_level)
	zzupdate_err_handl_level(2);
      cur_p->err_handl_level = 2;

      (void)handle_system_error(1, (OSBOOLEAN) 0, ecode, extra);

      LOCK();
      handlerp.zzkrn_halted_hook();
    }

  /*
  ** Allocate an error handler stack if required.
  ** We don't have error handler stacks yet.??
  */
  stack = NULL;
  stacksize = (OSADDRESS) 0;

  /*
  ** Note that we are now in error processing.
  */
  if(out_of_pool)
    {
      cur_p->out_of_pool |= out_of_pool;      
    }
  else
    {
      if(1 == ++cur_p->err_handl_level)
	zzupdate_err_handl_level(1);
    }

  /*
  ** Call process level error handler if available, then call block
  ** level error handler if available.
  */
  if (!fatal)
    {
      OSERRH *errh;
      int proc_errh;

      if (cur_p->error_handler != NULL)
	{
	  errh = cur_p->error_handler;
	  proc_errh = 1;
	}
      else
	{
	  errh = cur_p->block->error_handler;
	  proc_errh = 0;
	}

      while (errh != NULL)
	{
	  /*
	  ** Check if we should call the error handler in supervisor
	  ** mode or not. Note that interrupt processes are per
	  ** definition always running in supervisor mode (even if
	  ** they happen to be located in a user mode block)
	  */
	  if(cur_p->block->supervisor_mode || cur_p->priority > 32)
	    {
	      if(zzcall_super_errh(errh, user_called,
				   ecode, extra,
				   stack, stacksize,
				   cur_p->block->clib_pointer
				   ))
		{
		  /* Done if error was handled. */
		  goto done;
		}
	    }
	  else
	    {
	      /*
	      ** The current running process is a background or prioritized
	      ** process running in user mode.
	      */
	      if(zzcall_user_errh(errh, user_called,
				  ecode, extra,
				  stack, stacksize,
				  cur_p->block->clib_pointer
				  ))
		{
		  /* Done if error was handled. */
		  goto done;
		}
	    }

	  /* Check if we just ran the process error handler... */
	  if(proc_errh)
	    {
	      /* ..if so we now want to run the block error handler. */
	      errh = cur_p->block->error_handler;
	      proc_errh = 0;
	    }
	  else
	    break;
	}
    }

  /*
  ** Perform system error handling. 
  */
  if(!handle_system_error((fatal > 1 || cur_p->priority > 32 ||
			   dispatcher_locked),
			  user_called, ecode, extra))
    {
      /* Loop for ever if system error handling could not handle the error. */
      LOCK();
      handlerp.zzkrn_halted_hook();
    }

 done:
  if(out_of_pool)
    {
      cur_p->out_of_pool &= ~out_of_pool;
    }
  else
    {
      if(0 == --cur_p->err_handl_level)
	zzupdate_err_handl_level(0);
    }
  

  /*
  ** If the dispatcher was locked when handle_error() was called, it
  ** must still be locked if we want to return.
  */
  if(dispatcher_locked && !zzdispatcher_locked)
    {
      LOCK();
      handle_system_error(-1, (OSBOOLEAN) 0,
			  OSE_EFATAL_MASK | OSE_ECORRUPTED_KERNEL, (OSERRCODE) 19);
      LOCK();
      handlerp.zzkrn_halted_hook();
    }
  return (OSADDRESS) 1;
}

/*
** Report an error found by the OSE kernel.
**
** fatal = 0 for a warning, 1 for an error that can only be handled by
** the system error handler, 2 for a fatal kernel error.
**
** Return code is supplied by the error handler.
**
** Interrupts are disabled by handle_error() for fatal kernel errors.
*/

static OSADDRESS
os_error(int fatal, OSERRCODE ecode, OSERRCODE extra)
{
  return handle_error(fatal, ecode, extra,
		      (OSBOOLEAN) 0); /* Not user_called */
}

/*
** Allow the target interface to report an error as being found by OSE.
**
** See also: os_error().
*/

OSADDRESS
zzos_error(OSBOOLEAN fatal, OSERRCODE ecode, OSERRCODE extra)
{
  if((ecode & OSE_EERROR_MASK) == OSE_EUSER_STACK_OVERFLOW)
    {
      /*
      ** Report user stack overflow from zzsave_context().
      **
      ** This function is here called from the dispatcher save phase.
      ** It makes the current process running again before the error
      ** is processed. 
      **
      ** The error must be processes by the kernel error handler only.
      **
      ** The process can only be made ready if it has not killed itself.
      ** To avoid fatal recursive errors, we ignore the error when
      ** already in error processing. The target interface shall
      ** continue with the context save if the call returns.
      */

      if(zzww.dp_state)
	{
	  BKL_LOCK();
	  if(zzcur_p->killed || zzcur_p->err_handl_level != 0)
	    {
	      /* Ignore the stack mark error. */
	      BKL_UNLOCK();
	      return 1;
	    }

	  /* Restore myself as the current process. */
	  make_mypri_current();
	  BKL_UNLOCK();
	}

      /*
      ** Local error handlers may not process this error, the user
      ** stack is broken.
      */
      if(fatal == 0)
	{
	  fatal = 1;	/* Call system error handler only. */
	}
  
      (void) handle_error(fatal, ecode, extra,
			  (OSBOOLEAN) 0); /* Not user_called */
      
      /* If still here, treat it as a fatal error. */
      fatal = 2;
    }

  return handle_error(fatal, ecode, extra,
		      (OSBOOLEAN) 0); /* Not user_called */
}

/*
** This is the fatal error loop.
** It is called whenever the kernel needs to halt the kernel in response
** to an error which was not properly handled by the system error handler.
*/

void
zzkrn_halted_hook(void)
{
  for(;;)
    {
      LOCK();
      zzcall_kernel_halted();
    }
}

/*
**===========================================================================
** 13.3  System calls.
**===========================================================================
*/

/*
** This is the create_error_handler system call.
*/

OSERRH *
create_error_handler(PROCESS pid,
		     OSERRH *entrypoint,
		     OSADDRESS stack_size)
{
  register struct _pcb *pcb;
  OSERRH *old_entrypoint;

  (void)stack_size;
  
#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_CREATE_ERROR_HANDLER))
    return NULL;
#endif

  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid_bid(pid, OSE_CREATE_ERROR_HANDLER);
  if(pcb == NULL)
    {
      return NULL;
    }

  /* Locate the block or process descriptor. */
  pcb = pid_to_pcb(pid);

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the block/process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      UNLOCK_SWAP();
      return NULL;
    }

#if USE_REMOTE_CALLS
  /* Issue a remote system call if required. */
  if(local_sysctlblk.enable_remcalls)
  {
    union SIGNAL *sig;
    register OSBOOLEAN use_remcalls;
    register PROCESS handler;
    if(pcb->magic == MAGIC_PCB)
      {
	use_remcalls = pcb->block->use_remcalls;
	handler = pcb->block->remcall_server;
      }
    else
      {
	use_remcalls = ((struct _block *)pcb)->use_remcalls;
	handler = ((struct _block *)pcb)->remcall_server;
      }
    if(use_remcalls)
      {
	UNLOCK_SWAP();
	set_suid((OSBOOLEAN) 1);
	sig = alloc(sizeof(struct remote_call), REMOTE_CALL);
	sig->remote_call.call_type = REM_CREATE_ERROR_HANDLER;
	sig->remote_call.params.rem_create_error_handler.pid = pid;
	sig->remote_call.params.rem_create_error_handler.entrypoint = (OSADDRESS) entrypoint;
	sig->remote_call.params.rem_create_error_handler.stacksize = stack_size;
	
	sig = make_remote_call(sig, (OSCALLTYPE) REMR_CREATE_ERROR_HANDLER, handler);
	if(sig)
	  {
	    OSADDRESS result = sig->remote_response.results.remr_create_error_handler.entrypoint;
	    free_buf(&sig);
	    set_suid((OSBOOLEAN) 0);
	    return (OSERRH *) result;
	  }
	else
	  {
	    set_suid((OSBOOLEAN) 0);
	    return NULL;
	  }
      }
  }
#endif

  /*
  ** We should select a new stack size as appropriate.
  ** We don't use error handler stacks for now??
  */

  /* Read the old error handler and register the new. */
  if(pcb->magic == MAGIC_BCB)
    {
      struct _block *block = (struct _block *)pcb;

      old_entrypoint = block->error_handler;
      block->error_handler = entrypoint;
    }
  else
    {
      old_entrypoint = pcb->error_handler;
      pcb->error_handler = entrypoint;
    }

  /* Unlock the dispatcher and return to caller. */
  UNLOCK_SWAP();
  return old_entrypoint;
}

/*
** This is the error system call.
*/

void
error(OSERRCODE ecode)
{
  (void)handle_error(0,              /* Non fatal. */
		     ecode,
		     (OSERRCODE) 0,
		     (OSBOOLEAN) 1); /* User called */
}

/*
** This is the error2 system call.
*/

void
error2(OSERRCODE ecode, OSERRCODE extra)
{
  (void)handle_error(0,              /* Non fatal. */
		     ecode, extra,
		     (OSBOOLEAN) 1); /* User called */
}

static OSADDRESS
sysconf_107(OSADDRESS subcode, OSADDRESS q, OSADDRESS r)
{
  /* ose_get_error_info() */
  if(subcode == 1)
    {
      PROCESS pid = (PROCESS)q;
      uint64_t *values = (uint64_t *)r;
      struct _pcb *pcb = safe_pid_to_pcb(pid, 0);
      OSADDRESS result;
      
      if(pcb == NULL)
	return 3;		/* No such process */
      
      if(pcb->error_origin > 1)
	result = 2;		/* No error info stored. */
      else
	{
	  values[0] = (uint64_t)pcb->error_ecode;
	  values[1] = (uint64_t)pcb->error_extra;
	  result = (OSADDRESS)pcb->error_origin;
	}
      /* Process still present? */
      if(pcb_to_pid((volatile struct _pcb *)pcb) != pid)
	result = 3;		/* No such process */

      return result;
    }
  
  /* ose_reset_error_info() */
  if(subcode == 2)
    {
      zzcur_p->error_origin = 2;  /* Clear error info */
    }
  
   return (OSADDRESS) 0;   
}

/*
**===========================================================================
** 13.4  Undocumented system calls.
**===========================================================================
*/


/*
** This is the exit_error_handler() system call.
** It is a no-operation call if the current process is not executing
** in an error handler.
**
** It clears the error handler recursion counter for the current process
** and reinitializes the supervisor stack pointer if the current process
** is a user mode process.
**
** The last operation is expected to be implemented in the target interface
** before or after calling this function.
** Note that the return trap from user mode error handlers need also check
** a flag set by exit_error_handler(), reporting an error if an error handler
** returns instead of doing the intended longjmp.
*/

void
exit_error_handler(void)
{
  /* Clear the error handler level counter in the pcb of the caller. */
  if(zzcur_p->err_handl_level)
    zzupdate_err_handl_level(0);      
  zzcur_p->err_handl_level = (OSBOOLEAN) 0;
}

/*
*****************************************************************************
** 14 TIMER MANAGEMENT.
*****************************************************************************
*/

/*
**===========================================================================
** 14.1  Initialization.
**===========================================================================
*/

static void
init_timer(void)
{
  /* There is no timer manager process yet. */
  /* Wakeup priority for timer processes is set to highest possible. */
  zzuu.timer_manager = NULL;
  zzuu.timer_int_priority = (OSPRIORITY) 0;

  /* Clear the timer interrupt scheduling list. */
  memset(zzuu.tmslist, 0, sizeof(zzuu.tmslist));

  /* Setup full time resolution for all priorities. */
  memset(zzuu.timer_resolution, 0, sizeof(zzuu.timer_resolution));

  /*
   * Clear timeout signal per tick list and set other timeout signal variables.
   */
  memset(zzuu.tos_tick_list, 0,
	 local_sysctlblk.tmo_array_length * sizeof(*zzuu.tos_tick_list));
  zzuu.next_free_tos_index = (~0u) >> (16u - TOSCB_ALLOC_2LOG);
  zzuu.last_toscb = NULL;
  zzuu.tos_free_list.head = NULL;
  zzuu.tos_free_list.tail = NULL;
  SMP_CODE(spin_lock_init(&zzuu.toscb_spin_lock));

  zzuu.ticks = 0;
  zzuu.systime_guard = (U32) 0;

  /* There are no pending timer events at this moment. */
  zzuu.timer_process_count = 0;
  zzww.timeout_count = 0;
  zzuu.tmo_signal_count = 0;

  /* Clear the recursive tick detection support flag. */
  concurrent_check = 0;
}

/*
**===========================================================================
** 14.2  Local functions.
**===========================================================================
*/
#ifdef ASSERT
#define DO_ASSERT_TMOLIST
#endif

#ifndef DO_ASSERT_TMOLIST
#define assert_tmolist(head, curr, expect_find_me, line)
#define assert_no_active_timer(pcb)
#else
static void tmo_dump_pcb(struct _pcb *c)
{
  ramlog_printf("@@ pcb %#x pid %#x cpu %d pri %d stat %#x fq %#x bq %#x",
		c, c->pid, c->cpu_id, c->priority, c->status, c->fq, c->bq);
  ramlog_printf(" '%s'\n", c->name);
}

static void
assert_no_active_timer(struct _pcb *pcb)
{
  if(!pcb->tmo_active && !(pcb->status & ST_TIMEOUT))
    return;
  
  if(pcb->tmo_active)
    ramlog_printf("@@TMOLIST: tmo_active at tmo request\n");

  if(pcb->status & ST_TIMEOUT)
    ramlog_printf("@@TMOLIST: ST_TIMEOUT set at tmo request\n");
  
  tmo_dump_pcb(pcb);
  os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 24); /* Fatal. */
  return;			/*NOTREACHED*/
}

static void
assert_tmolist(struct _pcb *head, struct _pcb *curr, int expect_find_me, unsigned line)
{
  int found = 0;
  int errcnt = 0;
  int cnt = 0;
  int circular = 0;
  struct _pcb *p;
  struct _pcb *q = NULL;
  cpuid_t cpu_id = zzkrn_cpu_id();

  if(expect_find_me && curr && curr->cpu_id != cpu_id)
    {
      errcnt++;
      ramlog_printf("@@TMOLIST: Wrong cpu:\n");
      tmo_dump_pcb(curr);
    }
  
  if(head && head->bq)
    {
      errcnt++;
      ramlog_printf("@@TMOLIST: list head has bq set:\n");
      tmo_dump_pcb(head);
      tmo_dump_pcb(head->bq);
    }
  
  for(p = head; p; p = p->fq)
    {
      ++cnt;
      if(p == p->fq) circular = cnt;
      
      if(p->cpu_id != cpu_id)
	{
	  errcnt++;
	  ramlog_printf("@@TMOLIST: wrong cpu, expected: %d\n", cpu_id);
	  tmo_dump_pcb(p);
	}
      if(p->bq != q)
	{
	  errcnt++;
	  ramlog_printf("@@TMOLIST: bad bq, expected: %#x\n", q);
	  tmo_dump_pcb(p);
	  if(q) tmo_dump_pcb(q);
	}
      if(!p->tmo_active)
	{
	  errcnt++;
	  ramlog_printf("@@TMOLIST: tmo_active is false\n", q);
	  tmo_dump_pcb(p);
	}
      if(p->status & ST_TIMEOUT)
	{
	  errcnt++;
	  ramlog_printf("@@TMOLIST: ST_TIMEOUT set\n", q);
	  tmo_dump_pcb(p);
	}
      if(curr == p)
	{
	  if(found)
	    circular = cnt;
	  else
	    found = cnt;
	}

      if(cnt > 100) circular = cnt;

      if(circular)
	{
	  errcnt++;
	  ramlog_printf("@@TMOLIST: circular list detected:\n");
	  for(q = head; cnt--; q = q->fq) tmo_dump_pcb(q);
	  break;
	}
      q = p;
    } /* for */
  
  if(curr && !!expect_find_me != !!found)
    {
      errcnt++;
      ramlog_printf("@@TMOLIST: Unexpected %s of pcb in list:\n",
		    expect_find_me ? "nofind" : "find");
      tmo_dump_pcb(curr);
    }
  if(errcnt)
    {
      ramlog_printf("@@TMOLIST: called from: %d. Queue:%s\n",
		    line, head ? "" : " Empty.");
      for(q = head; cnt--; q = q->fq) tmo_dump_pcb(q);

      /* Also dump the local event queue. */
#ifdef OSE_SMP
      /* zzkrn_evt_dump_queue(zzkrn_cpu_id()); */
#endif

      os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 25);	/* Fatal. */
    }
}
#endif	/* DO_ASSERT_TMOLIST */

/*
** Request a non-zero timeout for a process on the the current cpu. 
**
** This function is called from delay(), receive_w_tmo() and receive_from()
** to start a timeout, and also by sysd_lite to restart a timeout on
** a new cpu after process migration.
**
** The timeout must be cleared with either cancel_timeout() or
** cancel_other_timeout().
**
** The ST_TIMEOUT bit in the process status field is set whenever a timeout has
** occurred.
**
** Interrupts must be disabled by the caller.
*/

static void
request_timeout(struct _pcb *pcb, OSTICK tmo)
{
  register struct _pcb **head;
  register U16 nexttmo;
  register OSTICK tmo_expires;

#ifdef OSE_SMP
  /*
  ** If cpu0 has incremented the tick counter, but we have not yet got the
  ** corresponding tick event, add one tick to the timeout.
  ** Otherwise, load generators polling get_ticks() will wakeup too soon,
  ** generating more cpu load than expected.
  */
  tmo_expires = ((OSTICK)zzcpu[0].ticks + tmo);
#else
  tmo_expires = (zzww.ticks + tmo);
#endif

  /* Tell kill_proc that we have a timeout running. */
  assert_no_active_timer(pcb);
  pcb->tmo_active = (OSBOOLEAN) 1;

  /*
  ** Save the index of the timer list for cancel_timeout() and
  ** cancel_other_timeout().
  */
  nexttmo = (U16)(tmo_expires & zzuu.curtmo_mask);
  pcb->cur_tmo = nexttmo;
  pcb->cur_tmo_expires = tmo_expires;

  /* Enter pcb into timer list. */
  ++zzww.timeout_count;
  head = &zzww.tmolist[nexttmo];
  assert_tmolist(*head, pcb, 0, __LINE__);
  pcb->fq = *head;
  if(pcb->fq)
    (*head)->bq = pcb;
  pcb->bq = NULL;
  *head = pcb;
  assert_tmolist(*head, pcb, 1, __LINE__);
}

/*
** Cancel a timeout for the current process.
**
** This function is used in receive_w_tmo() and receive_from().
** If a timeout is currently requested for the calling process, then
** it is canceled.
**
** The timeout is not canceled if the ST_TIMEOUT flag is set since the timer
** has already canceled it.
** The ST_TIMEOUT bit in the process status field is cleared at return.
**
** This function *MUST* be called with interrupts disabled.
*/

static void
cancel_timeout(void)
{
  register struct _pcb *pcb = zzcur_p;

  if(pcb->tmo_active)
    {
      /* Not already expired? */
      if(!(pcb->status & ST_TIMEOUT))
	{
	  assert_tmolist(zzww.tmolist[pcb->cur_tmo], pcb, 1, __LINE__);

	  /* No, remove it. */
	  if(pcb->bq)
	    {
	      pcb->bq->fq = pcb->fq;
	    }
	  else
	    {
	      zzww.tmolist[pcb->cur_tmo] = pcb->fq;
	    }
	  if(pcb->fq)
	    {
	      pcb->fq->bq = pcb->bq;
	    }
	  assert_tmolist(zzww.tmolist[pcb->cur_tmo], pcb, 0, __LINE__);
	}

      /* Tell kill_proc that there is no timeout. */
      pcb->tmo_active = (OSBOOLEAN) 0;
      --zzww.timeout_count;
    }
  pcb->status = (pr_state_t) (pcb->status & (~ST_TIMEOUT));
}

/*
** Cancel any pending timeout for another process.
**
** This function is used in kill_proc().
** If a timeout is currently requested for the specified process, then
** it is canceled.
**
** The timeout is not canceled if the ST_TIMEOUT flag is set since the timer
** has already canceled it.
** The state of the ST_TIMEOUT flag is not altered.
**
** Interrupts shall be disabled by the caller.
** cancel_other_timeout must be called on the cpu of the victim process.
*/

static void
cancel_other_timeout(struct _pcb *pcb)
{
  if(pcb->tmo_active)
    {
      /* Already expired? */
      if(!(pcb->status & ST_TIMEOUT))
	{
	  assert_tmolist(zzww.tmolist[pcb->cur_tmo], pcb, 1, __LINE__);
	  if(pcb->bq)
	    {
	      pcb->bq->fq = pcb->fq;
	    }
	  else
	    {
	      zzww.tmolist[pcb->cur_tmo] = pcb->fq;
	    }
	  if(pcb->fq)
	    {
	      pcb->fq->bq = pcb->bq;
	    }
	  assert_tmolist(zzww.tmolist[pcb->cur_tmo], pcb, 0, __LINE__);
	}

      /* Tell kill_proc that there is no timeout. */
      pcb->tmo_active = (OSBOOLEAN) 0;
      --zzww.timeout_count;
    }
}

/*
** Insert timeout signal control block last in tick slot list. 
**
** Interrupts shall be disabled by the caller.
** Proper tick slot is calculated from timeout time requested.
*/

static inline void
append_to_tickslot(struct toscb *toscb, OSTICK timeout)
{
  U16 tickslot = (U16) ((zzuu.ticks + timeout + 1) & zzuu.curtmo_mask);

  toscb->count = timeout >> zzuu.timeout_shift;
  toscb->tickslot = tickslot;

  toscb->tick_fq = NULL;
  if(zzuu.tos_tick_list[tickslot].tail)
    {
      toscb->tick_bq = zzuu.tos_tick_list[tickslot].tail;
      zzuu.tos_tick_list[tickslot].tail->tick_fq = toscb;
    }
  else
    {
      toscb->tick_bq = NULL;  
      zzuu.tos_tick_list[tickslot].head = toscb;
    }
  zzuu.tos_tick_list[tickslot].tail = toscb;
}

/*
** Insert timer last in pcb list. 
**
** Interrupts shall be disabled by the caller.
*/

static inline void
append_toscb_to_pcb_list(struct toscb *toscb, struct tos_head *list)
{
  if(list->tail)
    {
      toscb->bq = list->tail;
      list->tail->fq = toscb;
    }
  else
    {
      toscb->bq = NULL;
      list->head = toscb;
    }
  list->tail = toscb;
  toscb->fq = NULL;
}

/*
** Unlink pending toscb from tick list.
**
** Interrupts shall be disabled by the caller.
*/

static inline void 
unlink_toscb_from_tick_list(struct toscb *toscb)
{
#ifdef OSE_SMP
  if(zzuu.last_toscb == toscb)	
    {
      /* Tick() was preempted by current process and wants this toscb next.
       * Tell tick() to skip this block.
       */
      zzuu.last_toscb = toscb->tick_bq;
    }
#endif
  if(toscb->tick_fq)
    toscb->tick_fq->tick_bq = toscb->tick_bq;
  else
    zzuu.tos_tick_list[toscb->tickslot].tail = toscb->tick_bq;
  
  if(toscb->tick_bq)
    toscb->tick_bq->tick_fq = toscb->tick_fq;
  else
    zzuu.tos_tick_list[toscb->tickslot].head = toscb->tick_fq;
}

/*
** Unlink toscb from pcb list. 
**
** Interrupts shall be disabled by the caller.
*/

static inline void 
unlink_toscb_from_pcb_list(struct toscb *toscb, struct tos_head *list)
{
  if(toscb->fq)
    toscb->fq->bq = toscb->bq;
  else
    list->tail = toscb->bq;
  
  if(toscb->bq)
    toscb->bq->fq = toscb->fq;
  else
    list->head = toscb->fq;
}

/*
** Get a toscb from the free list. Allocate new ones if neccessary.
** This is a subfunction to the request_tmo_sig() and request_tmo() call.
**
** It uses toscb_spin_lock internally to lock the toscb lists.
** pool_brk_lo() used for allocation to avoid fragmentation. Thus, the
** dispatcher is locked temporarily to inhibit memory leaks caused by
** the calling process being killed.
*/

static struct toscb *
alloc_toscb(OSTICK timeout, struct _signal *sigadm, SIGSELECT signo)
{
  LOCKSTATE     msr;
  struct toscb	*tos;
  struct toscb	*tosarr;
  struct toscb	**tos_index_level2 = NULL;
  unsigned int	i;
  U32		free_index;
  U32		old_free_tos_index;
  U32		chunk_index;

  SPIN_LOCK_SAVE(&zzuu.toscb_spin_lock, msr);

  ++zzuu.tmo_signal_count;

  tos  = zzuu.tos_free_list.head;
  if(tos)
    {
      zzuu.tos_free_list.head = tos->fq;

      tos->pid = zzcur_p->pid;
      tos->flags = sigadm ? 3 : 1;
      tos->sigadm = sigadm;
      tos->signo = signo;

      /* Insert it in the toscb list of current process. */
      append_toscb_to_pcb_list(tos, &zzcur_p->pending_tos);
  
      /* Insert it into the proper time slot. */
      append_to_tickslot(tos, timeout);

      SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);
      return tos;
    }

  SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);

  /* Allocate a level 2 tos directory chunk in the system pool. */
  tos_index_level2 = (struct toscb **) sys_alloc(256 * sizeof(struct toscb *), 0);

  /* Protect current process from being killed (no toscb memory leaks). */
  LOCK_SWAP();

  old_free_tos_index = zzuu.next_free_tos_index;
  if(unlikely(old_free_tos_index >> TOSCB_ALLOC_2LOG == 0))
    {
      UNLOCK_SWAP();
      os_error(1, OSE_ETOO_MANY_TMO_REQUESTS, 255*256*TOSCB_ALLOC_CHUNK);
      /* If still alive, escalate to FATAL error: */
      os_error(2, OSE_ETOO_MANY_TMO_REQUESTS, 255*256*TOSCB_ALLOC_CHUNK);
    }
  free_index = old_free_tos_index - TOSCB_ALLOC_CHUNK;

  /* Allocate a new tos chunk in the system pool, owned by the system. */
  tosarr = (struct toscb *)
    pool_brk_lo(&system_pool, TOSCB_ALLOC_CHUNK * sizeof(struct toscb));

  /* Initiate the new toscb chunk. */
  for(i = 0; i < TOSCB_ALLOC_CHUNK; i++)
    {
      tosarr[i].fq = &tosarr[i - 1];	/* tosarr[0] reiniated below. */
      tosarr[i].instance = 0;
      tosarr[i].flags = 0;
      tosarr[i].index = free_index + 1u + i;
    } /* for */

  /* Reserve and initate one control block for the caller. */
  tos = &tosarr[TOSCB_ALLOC_CHUNK - 1];
  tos->pid = zzcur_p->pid;
  tos->flags = sigadm ? 3 : 1;
  tos->sigadm = sigadm;
  tos->signo = signo;

  /* If a new level 2 directory is needed, then insert it at level 1. */
  chunk_index = old_free_tos_index >> TOSCB_ALLOC_2LOG;
  if((chunk_index & 255) == 255)
    {
      change_sig_owner(sig_to_adm((union SIGNAL *) tos_index_level2), NULL);
      zzuu.tos_index_list[chunk_index >> 8] = tos_index_level2;
      tos_index_level2 = NULL;
    }
  
  (zzuu.tos_index_list[chunk_index >> 8])[chunk_index & 255] = tosarr;
  zzuu.next_free_tos_index = free_index;

  /* Take proper lock for the toscb structures. */
  SPIN_LOCK_SAVE(&zzuu.toscb_spin_lock, msr);

  /* Add the new elements, except the last one, to the free list. */
  tosarr[0].fq = zzuu.tos_free_list.head;
  if(! zzuu.tos_free_list.head)
    zzuu.tos_free_list.tail = &tosarr[0];
  zzuu.tos_free_list.head = &tosarr[TOSCB_ALLOC_CHUNK - 2];

  /* Insert it in the toscb list of current process. */
  append_toscb_to_pcb_list(tos, &zzcur_p->pending_tos);
  
  /* Insert it into the proper time slot. */
  append_to_tickslot(tos, timeout);

  SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);
  UNLOCK_SWAP();
    
  /* If the level 2 directory was not needed, free it again. */
  if(tos_index_level2)
    {
      union SIGNAL *p = (union SIGNAL *) tos_index_level2;
      free_buf(&p); 
    }
  
  return tos;
}

/*
** Put a toscb into the free list.
**
** This function may be called by any kind of process.
** The caller shall disable interrupts resp. take the SMP toscb_spin_lock.
*/

static inline void
free_toscb(struct toscb *toscb)
{
  toscb->flags = 0;		/* Zero means "free". */
  toscb->instance++;		/* Make it not match the last tmo reference. */
  toscb->fq = NULL;		/* Enter last in list, so no next. */

  if(zzuu.tos_free_list.head)
    zzuu.tos_free_list.tail->fq = toscb;
  else
    zzuu.tos_free_list.head = toscb;
  zzuu.tos_free_list.tail = toscb;

  --zzuu.tmo_signal_count;
}

/*
** Free toscb control blocks of a killed process.
**
** Subfunction of sysd_delete_one_proc(). Current process is sysd, and will
** not die.
*/

static void 
kill_timeout_signals(struct _pcb *pcb)
{
  LOCKSTATE msr;
  struct toscb *toscb, *toscb_next;

  /* Kill still counting timeout signal control blocks. */
  toscb = pcb->pending_tos.head;
  pcb->pending_tos.head = pcb->pending_tos.tail = NULL;
  while(toscb)
    {
      toscb_next = toscb->fq;
      SPIN_LOCK_SAVE(&zzuu.toscb_spin_lock, msr);
      unlink_toscb_from_tick_list(toscb);
      free_toscb(toscb);
      SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);
      toscb = toscb_next;
    } /* for */

  /* Kill expired timer control blocks. */
  toscb = pcb->expired_tos.head;
  pcb->expired_tos.head = pcb->expired_tos.tail = NULL;
  while(toscb)
    {
      toscb_next = toscb->fq;
      SPIN_LOCK_SAVE(&zzuu.toscb_spin_lock, msr);
      free_toscb(toscb);
      SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);
      toscb = toscb_next;
    } /* while */
}

/*
** Expire time out signals of current tick slot.
**
** Subfunction of tick(). 
*/

#ifdef OSE_SMP
static void 
expire_tmo_signals(void)
{
  struct _pcb *pcb;
  struct toscb *toscb;
  struct toscb *next_toscb;
  struct toscb *last_toscb;

  /*
  ** Handle each timer signal in this time slot.
  ** Break when next timer signal entry is added after the current tick
  ** was taken; those should not fire at the current scan of the list.
  ** Lock out cancel requests during the scan of the timer list.
  ** Note that we do not disable higher interrupts here!
  **
  ** Should we change the cancel operations to mark the toscb instead of
  ** unlink it??
  */

  SPIN_LOCK(&zzuu.toscb_spin_lock);
  
  /* Get the last timeout control block to handle. */
  last_toscb = zzuu.last_toscb;
  if(last_toscb)
    {
      zzuu.last_toscb = NULL;
      toscb = zzuu.tos_tick_list[((U32)zzuu.ticks) & zzuu.curtmo_mask].head;
      assert(toscb);
      for(;;)
	{
	  next_toscb = toscb->tick_fq;

	  if(toscb->count)	
	    {
	      /* Not expired. Decrement counter. */
	      toscb->count--;
	    }
	  else
	    {
	      /* Check if the process is killed. */
	      pcb = pid_to_pcb(toscb->pid);
	      if(pcb_to_pid(pcb) != toscb->pid)
		{
		  /* The process was killed. Sysd will do all cleanup. */
		}
	      else
		{
		  /*
		  ** The client is still alive. We keep the kernel lock for
		  ** the entire state transition, because we don't want to
		  ** race with cancel and reset operations on other cpus.
		  */
		  unlink_toscb_from_tick_list(toscb);
		  unlink_toscb_from_pcb_list(toscb, &pcb->pending_tos);

		  /* Put timer into expired tos list. */
		  append_toscb_to_pcb_list(toscb, &pcb->expired_tos);

		  /* Get and send the signal, without redirection. */
		  if(toscb->flags == 3)
		    {
		      /* User provided signal */
		      change_sig_owner(toscb->sigadm, zzcur_p);
		    }
		  else
		    {
		      /* Kernel provided signal */
		      toscb->sigadm =
			sig_to_adm(alloc(sizeof(SIGSELECT), toscb->signo));
		    }
		  safe_send_w_s(toscb->sigadm, toscb->pid, toscb->pid,
				1ul, NULL, NULL); /* No redirection */
		  /* Mark as Expired */
		  toscb->flags = 6;		
		}
	    }
	  /*
	  ** Quit after handling of all timeout entries that were present
	  ** when the current tick was taken.
	  */
	  if(toscb == last_toscb)
	    {
	      break;
	    }
	  assert(next_toscb);
	  toscb = next_toscb;
	} /* for */
    }
  SPIN_UNLOCK(&zzuu.toscb_spin_lock);
}
#else
/* Non-SMP */
static void 
expire_tmo_signals(void)
{
  struct _pcb *pcb;
  struct toscb *toscb;

  /* Handle each timer signal in this time slot */
  for(toscb = zzuu.tos_tick_list[(U32)zzuu.ticks & zzuu.curtmo_mask].head;
      toscb;
      toscb = toscb->tick_fq)
    {
      if(toscb->count)	
	{
	  /* Not expired. Decrement counter. */
	  toscb->count--;
	}
      else
	{
	  /* Check if the process is killed. */
	  pcb = pid_to_pcb(toscb->pid);
	  if(pcb_to_pid(pcb) != toscb->pid)
	    {
	      /* The process was killed. Sysd will do all cleanup needed. */
	    }
	  else
	    {
	      /* The client is still alive. */
	      unlink_toscb_from_tick_list(toscb);
	      unlink_toscb_from_pcb_list(toscb, &pcb->pending_tos);

	      /* Put timer into expired tos list. */
	      append_toscb_to_pcb_list(toscb, &pcb->expired_tos);

	      /* Get and send the signal, without redirection. */
	      if(toscb->flags == 3)
		{
		  /* User provided signal */
		  change_sig_owner(toscb->sigadm, zzcur_p);
		}
	      else
		{
		  /* Kernel provided signal */
		  toscb->sigadm =
		    sig_to_adm(alloc(sizeof(SIGSELECT), toscb->signo));
		}
	      safe_send_w_s(toscb->sigadm, toscb->pid, toscb->pid,
			    1ul, NULL, NULL); /* No redirection */
	      /* Mark as Expired */
	      toscb->flags = 6;		
	    }
	}
    } /* for */
}
#endif

/*
** Free toscb of received signals for current process.
**
** Disabling of interrupts handled locally.
*/

static void 
free_received_toscb(void)
{
  LOCKSTATE     msr;
  struct toscb *toscb;
  struct toscb *prev_toscb;
  struct toscb *next_toscb;
  struct _pcb *pcb = zzcur_p;

  if(pcb->expired_tos.head == NULL)
    return;

  /*
  ** Protect prioritized and background processes from beeing killed
  ** here.
  */
  LOCK_SWAP();

  SPIN_LOCK_SAVE(&zzuu.toscb_spin_lock, msr);
  if(!pcb->mailbox)		
    {
      /* Empty mailbox - free all toscb blocks. */
      toscb = pcb->expired_tos.head;
      pcb->expired_tos.head = pcb->expired_tos.tail = NULL;
      
      SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);
      while(toscb)
	{
	  next_toscb = toscb->fq;
	  SPIN_LOCK_SAVE(&zzuu.toscb_spin_lock, msr);
	  free_toscb(toscb);
	  SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);
	  toscb = next_toscb;
	} /* while */
      /* Unlock dispatcher */
      UNLOCK_SWAP();
      return;
    }
  SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);

  /* Unlock dispatcher */
  UNLOCK_SWAP();
  toscb = pcb->expired_tos.head;
  while(toscb)
    {
      struct _signal *sigadm;

      sigadm = toscb->sigadm;
      
      /* Check if corresponding signal is present in current signal queue. */
      if(sigadm->sender_id == toscb->pid && /* (a) */
	 sigadm->dest_id == 0 && /* (b) */
	 sigadm->owner == 0 && /* (c) */
	 adm_to_sig(sigadm)->sig_no == toscb->signo) /* (d) */
	{
	  /* Now, we have verified that the signal is:
	   * (a) - either sent or freed by the proper process,
	   * (b) - neither freed nor redirected,
	   * (c) - not owned, i. e. either freed, in inqueue or in use by pose,
	   * (d) - and has correct signal number.
	   * Summing up: It is sent by the proper process, not redirected, 
	   * not yet received, and contains the proper signo. 
	   * Still, we do not know if it is in the queue of the current process.
	   * For example, the client may have sent the timeout signal to
	   * another process, that has not yet received it. At the moment,
	   * we ignore this. We do not want to scan the signal queue. 
	   * The toscb will be deallocated later when either that 2nd
	   * process has received the signal, or the current process has an 
	   * empty signal queue.
	   */

	  /* Signal not received yet - keep this toscb. Check next one. */
	  toscb = toscb->fq;
	}
      else
	{
	  /* Signal already received - free the toscb. */
	  prev_toscb = toscb->bq;
	  SPIN_LOCK_SAVE(&zzuu.toscb_spin_lock, msr);
	  unlink_toscb_from_pcb_list(toscb, &pcb->expired_tos);
	  free_toscb(toscb);
	  SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);
	  toscb = prev_toscb ? prev_toscb->fq : pcb->expired_tos.head;
      	}
    } /* while */
}

/*
** Register a time out signal request.
** This is a subfunction to the request_tmo_sig() and request_tmo() call.
**
** This function may only be called by background or prioritized
** processes.
*/

static OSTMOREF
sys_request_tmo(OSTICK timeout, struct _signal *sigadm, SIGSELECT signo)
{
  OSTMOREF	tmoid;
  struct toscb	*toscb;

  /* Deallocate any toscb representing an already received signal. */
  free_received_toscb();

  /* Get a timer signal control block and initiate it. */
  toscb = alloc_toscb(timeout, sigadm, signo);

  tmoid = toscb->index | (toscb->instance << (16 + TOSCB_ALLOC_2LOG));
  return tmoid;
}

/*
** Unlink a signal from the receive queue of the current process.
** This is a subfunction to sys_cancel_tmo() and sys_restart_tmo().
** 
** The signal identity is checked to improve detection of user errors,
** i. e. cancelling of already received timeouts, where the recived 
** signal happens to be reused for a new expired timeout for the same
** process. In this case, the signal is allocated by the kernel, and
** the signal numbers must be different. 
**
** Returns the sigadm pointer when signal found in mailbox, else NULL.
** The function locks the dispatcher internally.
*/

static struct _signal *
unlink_tmo_from_mailbox(struct _signal *sigadm, SIGSELECT signo)
{
  struct _signal *sigadm_q;

  /* Protect the signal queue from flush() by other process */
  LOCK_SWAP();
  sigadm_q = zzcur_p->mailbox;

  if(sigadm == sigadm_q)
    {
      if(adm_to_sig(sigadm)->sig_no != signo)
	{
	  /* User error. Operating on an already received timer. */
	  sigadm = NULL;
	}
      else
	{
	  LOCK();
	  zzcur_p->mailbox = sigadm->next;
	  zzcur_p->sig_cnt_queued--;
	  /* Make current process the owner now, to inhibit memory leaks. */
	  change_sig_owner(sigadm, zzcur_p);
	  UNLOCK();
	}
      UNLOCK_SWAP();
      return sigadm;
    }
  for(; sigadm_q; sigadm_q = sigadm_q->next)
    {
      /* If proper signal found, remove it from mailbox. */
      if(sigadm_q->next == sigadm) /* Found it. */
	{
	  if(adm_to_sig(sigadm)->sig_no != signo)
	    {
	      /* User error. Operating on an already received timer. */
	      sigadm = NULL;
	    }
	  else
	    {
	      LOCK();
	      sigadm_q->next = sigadm->next;
	      if(!sigadm_q->next)
		{
		  zzcur_p->mailtail = sigadm_q;
		}
	      zzcur_p->sig_cnt_queued--;
	      /* Make current process the owner, to inhibit memory leaks. */
	      change_sig_owner(sigadm, zzcur_p);
	      UNLOCK();
	    }
	  UNLOCK_SWAP();
	  /* No need to set pcb->flushed, we are in our own mailbox. */
	  return sigadm;
	}
    } /* for */
  /* No find */
  UNLOCK_SWAP();
  return (struct _signal *) NULL;
}

/*
** Cancel a timeout signal request.
** This is a subfunction to the cancel_tmo_sig() and cancel_tmo() call.
**
*/
static struct _signal *
sys_cancel_tmo(OSTMOREF tmoref, OSADDRESS want_signal, OSERRCODE syscall)
{
  LOCKSTATE	msr;
  SIGSELECT	signo;
  struct _signal *sigadm;
  struct toscb	*toscb;
  OSADDRESS	index;
  OSADDRESS	instance = tmoref >> (16 + TOSCB_ALLOC_2LOG);

  index = tmoref ^ (instance << (16 + TOSCB_ALLOC_2LOG));

  if(unlikely(index <= zzuu.next_free_tos_index)) 
    {
      /* Index is zero when timer already cancelled. See pflib.c. */
      os_error(0, OSE_ETMO_ALREADY_CANCELLED | syscall, tmoref);
      return NULL;
    }

  toscb = &((zzuu.tos_index_list[index >> (8 + TOSCB_ALLOC_2LOG)])
	    [(index >> TOSCB_ALLOC_2LOG) & 255])[index & (TOSCB_ALLOC_CHUNK - 1)];
  signo = toscb->signo;
  SPIN_LOCK_SAVE(&zzuu.toscb_spin_lock, msr);
  sigadm = toscb->sigadm;

  if(unlikely((toscb->instance ^ instance) << (16 + TOSCB_ALLOC_2LOG) ||
	      toscb->flags == 0 || toscb->pid != zzcur_p->pid))
    {
      SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);
      os_error(0, OSE_ETMO_ALREADY_RECEIVED | syscall, tmoref);
      return NULL;
    }

  switch(toscb->flags)
    {
    case 1:			/* Pending without signal */
				/*FALLTHROUGH*/
    case 3:			/* Pending with user provided signal */
      /* Unlink from tick list. */
      unlink_toscb_from_tick_list(toscb);

      /* Unlink from pcb list.
       * tick() can not expire this timer now, but may still expire another 
       * toscb, so continue with interrupts disabled.
       */
      unlink_toscb_from_pcb_list(toscb, &zzcur_p->pending_tos);

      /* Free the toscb. */
      free_toscb(toscb);
      SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);

      if(sigadm && !want_signal)
	{
	  safe_free_buf(sigadm); /* The client is still the owner here. */
	  sigadm = NULL;
	}
      else if(!sigadm && want_signal)
	{
	  sigadm = sig_to_adm(alloc(sizeof(SIGSELECT), signo));
	}
      return sigadm;

    case 6:			/* Expired. */
      /* Unlink toscb from pcb */
      unlink_toscb_from_pcb_list(toscb, &zzcur_p->expired_tos);

      /* Free the toscb. */
      free_toscb(toscb);
      SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);

#ifdef OSE_SMP
      /*
      ** For SMP, if cancel_tmo was called with interrupts disabled,
      ** then the signal may not yet have landed in the mailqueue.
      ** UNLOCK() to make it happen!
      */
      UNLOCK();
#endif

      /* Unlink the signal from the receive queue. Sender is checked first to
       * safer detect cancellation of received and freed timeouts, which is
       * illegal.
       */
      if(unlikely((sigadm->sender_id != zzcur_p->pid) ||
		  !unlink_tmo_from_mailbox(sigadm, signo)))
	{
	  os_error(0, OSE_ETMO_ALREADY_RECEIVED | syscall, tmoref); 
	  sigadm = NULL;
	}

      /* Handle the signal and return. */
      if(sigadm && !want_signal)
	{
	  safe_free_buf(sigadm); /* Current process is owner here. */
	  sigadm = NULL;
	}

      return sigadm;
      
    default:			/* Error in kernel data */
      /* Report that as a fatal error. Some better way possible? @guek@ */
      os_error(2, OSE_ECORRUPTED_KERNEL | syscall, (OSERRCODE) toscb);	
      break;			/* NOTREACHED */
    } /* switch */
		
  return (struct _signal *) NULL; /* NOTREACHED */
}

/*
** Restart a timeout signal request.
** This is a subfunction to the restart_tmo() call.
**
*/
static void
sys_restart_tmo(OSTMOREF tmoref, OSTICK timeout,  OSERRCODE syscall)
{
  LOCKSTATE	msr;
  struct _signal *sigadm;
  struct toscb	*toscb;
  OSADDRESS	index;
  OSADDRESS	instance = tmoref >> (16 + TOSCB_ALLOC_2LOG);

  index = tmoref ^ (instance << (16 + TOSCB_ALLOC_2LOG));

  if(unlikely(index <= zzuu.next_free_tos_index)) 
    {
      /* Index is zero when timer already cancelled. See pflib.c. */
      os_error(0, OSE_ETMO_ALREADY_CANCELLED | syscall, tmoref);
      return;
    }

  toscb = &((zzuu.tos_index_list[index >> (8 + TOSCB_ALLOC_2LOG)])
	    [(index >> TOSCB_ALLOC_2LOG) & 255])[index & (TOSCB_ALLOC_CHUNK - 1)];

  SPIN_LOCK_SAVE(&zzuu.toscb_spin_lock, msr);
  if(unlikely((toscb->instance ^ instance) << (16 + TOSCB_ALLOC_2LOG) ||
	      toscb->flags == 0 || toscb->pid != zzcur_p->pid))
    {
      SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);
      os_error(0, OSE_ETMO_ALREADY_RECEIVED | syscall, tmoref); 
      return;
    }

  switch(toscb->flags)
    {
    case 1:			/* Pending with default signal */
				/*FALLTHROUGH*/
    case 3:			/* Pending with user provided signal */
      /* Unlink from tick list. */
      unlink_toscb_from_tick_list(toscb);

      /* Insert it into the proper time slot. */
      append_to_tickslot(toscb, timeout);
      SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);
      return;

    case 6:			/* Expired. */
      SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);

      sigadm = toscb->sigadm;

#ifdef OSE_SMP
      /*
      ** For SMP, if cancel_tmo was called with interrupts disabled,
      ** then the signal may not yet have landed in the mailqueue.
      ** UNLOCK() to make it happen!
      */
      UNLOCK();
#endif

      /* Unlink the signal from the receive queue. Sender is checked first to
       * safer detect restart of received and freed timeouts, which is
       * illegal.
       */
      if(unlikely((sigadm->sender_id != zzcur_p->pid) ||
		  !unlink_tmo_from_mailbox(sigadm, toscb->signo)))
	{
	  os_error(0, OSE_ETMO_ALREADY_RECEIVED | syscall, tmoref); 
	  return;
	}

      /* Save the signal data in toscb. */
      toscb->flags = 3;

      SPIN_LOCK_SAVE(&zzuu.toscb_spin_lock, msr);

      /* Unlink toscb from pcb */
      unlink_toscb_from_pcb_list(toscb, &zzcur_p->expired_tos);

      /* Insert it in the pending toscb list of current process. */
      append_toscb_to_pcb_list(toscb, &zzcur_p->pending_tos);
  
      /* Insert it into the proper time slot. */
      append_to_tickslot(toscb, timeout);
      SPIN_LOCK_RESTORE(&zzuu.toscb_spin_lock, msr);
      return;
      
    default:			/* Error in kernel data */
      /* Report that as a fatal error. Some better way possible? @guek@ */
      os_error(2, OSE_ECORRUPTED_KERNEL | syscall, (OSERRCODE) toscb);	
      break;			/* NOTREACHED */
    } /* switch */
  /* NOTREACHED */
}

/*
** Handle timeslicing of prioritized processes.
**
** Called by the tick interrupt, and, in SMP slave cpus, by the event ISR.
** When called by an ISR, all interrupts are disabled by the caller.
*/
static void
handle_prio_timeslicing(void)
{
  LOCKSTATE msr;
  struct _pcb *pcb = zzcur_ni; /* The current non int process. */

  /*
  ** Check if the current running (non int) process has enabled
  ** timeslicing and if it is a prioritized process.
  */
  if(pcb->tmo != 0 && pcb->priority < 32)
    {
      /* Has the process' timeslice expired? */
      if(!(--pcb->ts_time))
	{
	  struct ready_list_entry *rlist; 
	    
	  /* Give the process a fresh timeslice. */
	  pcb->ts_time = pcb->tmo;

	  /* Pick up the process' ready list. */
	  rlist = pcb->ready_entry.rlist; 

	  LOCK_SAVE(msr);

	  /*
	  ** Only rotate the process to the end of the ready list if
	  ** the current process is first in the proper ready list
	  ** and there are other processes in the list. The current
	  ** process may not be part of any ready list if the
	  ** dispatcher is locked. In that case, it will itself
	  ** request a dispatch operation when it unlocks the
	  ** dispatcher.
	  */
	  if((rlist->hd == pcb) && pcb->ready_entry.nxt)
	    {
	      /* Remove the process from the head of ready list. */
	      rlist->hd = pcb->ready_entry.nxt;
	      rlist->hd->ready_entry.prev = NULL; 
	    
	      /* Insert the process back into the tail of ready list. */
	      pcb->ready_entry.prev = rlist->tl;
	      rlist->tl->ready_entry.nxt = pcb;
	      rlist->tl = pcb;
	      pcb->ready_entry.nxt = NULL;

	      /* Request a dispatch operation. */
	      zzww.dp_req = (OSBOOLEAN) 1;
	    }
	    
	  LOCK_RESTORE(msr);
	}
    }
}

/*
** Timeslice background processes. 
**
** If a timeslice has expired, rotate background process list by the
** moving head to the tail.
** Running in the context of the caller of tick, or, for SMP, as
** a KRN_EVT isr.
*/
static void
handle_background_timeslicing(void)
{
  if(!(--zzww.bg_list.time_slice))
    {
      register struct _pcb *tmp = zzww.bg_list.hd;
      if(tmp)
	{
	  zzww.bg_list.hd = tmp->ready_entry.nxt;
	  zzww.bg_list.time_slice = tmp->ready_entry.nxt->tmo;

	  if(zzww.bg_list.hd == zzww.bg_list.search_next)
	    zzww.bg_list.search_next = NULL;

	  /*
	  ** Swap if current non interrupt process is a background or
	  ** idle process. The new timeslice owner may want to run.
	  */
	  if(zzcur_ni->priority == 32)
	    zzww.dp_req = (OSBOOLEAN) 1;
	}
      else
	{
	  zzww.bg_list.time_slice = 1;
	}
    }
}

/* Handle timeslicing of prioritized and background processes. */
static void
handle_timeslicing(void)
{
  /* Handle Timeslicing of prioritized processes. */
  handle_prio_timeslicing();

  /* Handle Timeslicing of background processes. */
  handle_background_timeslicing();

  /*
  ** Tell the scheduler to poll out any ose_fsend signals before
  ** scheduling any background process. If we don't do this a
  ** background process that won't give up the cpu will prevent any
  ** further fsend polling.
  */
  if(zzww.idle_pcb->rec_poller)
    zzww.bg_receive_poll = (OSBOOLEAN) 1;
}

/*
** Expire requested timeouts.
** Called by tick() and at KRN_EVT_TICK events on multicore cpus.
*/
static void
expire_timeouts(void)
{
  LOCKSTATE msr;
  register struct _pcb *cur_ti;
  U16 curtmo;

  /* Update the current tmolist index, vital for timer clients. */
  curtmo = (U16)zzww.ticks & zzuu.curtmo_mask;
  cur_ti = zzww.tmolist[curtmo];

  assert_tmolist(cur_ti, NULL, 0, __LINE__);
  
  while(cur_ti)
    {
      /* LOCK_SAVE only needed when called from tick(). */
      LOCK_SAVE(msr);

      /*
      ** Detect a forgotten timer.
      ** In powerpc, the expiration must be within 2 G ticks from now.
      ** otherwise it is a forgotten timer.
      */
      if(unlikely(cur_ti->cur_tmo_expires - (OSTICK)zzww.ticks > 0x80000000U))
	{
	  ramlog_printf("<TMO>(expire_timeouts) tmo %u @ tick %u pid %#x %s.\n",
			(unsigned)cur_ti->cur_tmo_expires,
			(unsigned)zzww.ticks, cur_ti->pid, cur_ti->name);
	  dump_pcb(cur_ti);

	  if(cur_ti->tmo_active && cur_ti->cur_tmo == curtmo)
	    {
	      /* Expire the timeout NOW, even if it is late! */
	      cur_ti->cur_tmo_expires = zzww.ticks;
	    }
	  else
	    {
	      os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 32); /* Fatal. */
	      /* NORETURN */
	    }
	}

      /* Check if the process's timeout expires on this tick.
      ** NOTE: We have to cast the zzww.ticks to 32-bits OSTICK
      ** since on unicore zzww.ticks is mapped to zzuu.ticks
      ** which is 64-bits, if we don't all timeouts will stop
      ** working after 49-days.
      */
      if(cur_ti->cur_tmo_expires == (OSTICK)zzww.ticks)
	{
	  assert_tmolist(zzww.tmolist[cur_ti->cur_tmo], cur_ti, 1, __LINE__);

	  cur_ti->status = (pr_state_t) ((cur_ti->status | ST_TIMEOUT) &
					 (~(ST_RECEIVE_FSEM | ST_DELAY)));
	  /* Make process ready if required. */
	  if(cur_ti->status < ST_READY)
	    {
	      make_otherpri_ready_noswap(cur_ti); /* noswap */
	    }
	  
	  /* Remove it. */
	  if(cur_ti->bq)
	    {
	      cur_ti->bq->fq = cur_ti->fq;
	    }
	  else
	    {
	      zzww.tmolist[cur_ti->cur_tmo] = cur_ti->fq;
	    }
	  if(cur_ti->fq)
	    {
	      cur_ti->fq->bq = cur_ti->bq;
	    }
	  assert_tmolist(zzww.tmolist[cur_ti->cur_tmo], cur_ti, 0, __LINE__);
	}

      LOCK_RESTORE(msr);

      /* Look at next timer process. */
      cur_ti = cur_ti->fq;
    }
}

/*
**===========================================================================
** 14.3  System calls.
**===========================================================================
*/

/*
** This is the get_ticks() system call.
*/

OSTICK 
get_ticks(void)
{
  return (OSTICK) zzuu.ticks;
}

/*
** This is the system_tick() system call.
*/

OSTIME
system_tick(void)
{
  return (OSTIME) local_sysctlblk.tick_len;
}

/*
** This is the get_systime() system call.
**
** It is legal to call this function with a locked dispatcher.
*/

OSTICK
get_systime(OSTICK *microsecs)
{
  LOCKSTATE msr;
  register OSTICK lticks;
  register OSTICK microticks;

  if(microsecs)
    {
      U32 guard1, guard2;

      do
	{
	  LOCK_SAVE(msr);
	  guard1 = zzuu.systime_guard;
	  microticks = zzget_microticks();
	  lticks = (OSTICK) zzuu.ticks;
	  guard2 = zzuu.systime_guard;
	  LOCK_RESTORE(msr);
	}
      while(guard1 & 1 ||       /* Started inside guard region. */
	    guard1 != guard2);  /* A complete guard region during this try. */

      if(microticks >= local_sysctlblk.tick_len)
	{
	  microticks -= local_sysctlblk.tick_len;
	  lticks++;
	}
      *microsecs = microticks;
      return lticks;
    }
  else
    {
      return (OSTICK) zzuu.ticks;
    }
}

/*
** This is the ose_nano_delay system call.
**
** It is currently implemented as a busy loop with micro second resolution at
** best, depending on the timer hardware. 
*/
static OSADDRESS
sysconf_69(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  OSTICK u_sleep, t_sleep;
  OSTICK u_now, t_now;
  OSTICK u_out, t_out;
  OSTICK tick_len = (OSTICK)local_sysctlblk.tick_len;

  /* Remove ``not used args'' compiler warnings. */
  (void) q;
  (void) r;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_NANO_DELAY))
    return 0;

  /*
  ** Check that the timer is running, or, more precise, that we have taken
  ** at least one tick interrupt.
  */
  if(unlikely(zzuu.timer_manager == NULL))
    {
      os_error(0, OSE_ESYSCALL_TOO_EARLY | OSE_NANO_DELAY, (OSERRCODE) sysconf_69); /* Non fatal. */
      return 0;
    }
#endif

  /*
  ** The current implementation has, at best, micro second resolution,
  ** and only when the hardware is capable of that.
  **
  ** Decide earliest permitted return time.
  */
  u_sleep = (OSTICK)((p + 999) / 1000);
  t_out = get_systime(&u_out);
  u_out += u_sleep;

  /* If the return time is in another tick, do a usual delay() to that tick. */
  if(u_out >= tick_len)
    {
      t_sleep = u_out / tick_len;
      delay(t_sleep);
      u_out -= t_sleep * tick_len;
      t_out += t_sleep;
    }

  /*
  ** Use a busy wait loop until the micro second wanted is reached. 
  ** (This holds even for interrupt processes, but only if we read out
  ** micros too. See get_systime!)
  */
  do
    {
      t_now = get_systime(&u_now);
    }
  while(u_now < u_out && t_now == t_out);

  return 1;
}

/*
** This is the tick() system call.
**
** This function must be executed on interrupt level.
** If the caller is not an interrupt process, then the tick
** daemon, a special purpose interrupt process, is invoked to
** do the job.
**
** Perhaps the tick daemon should always be invoked to make sure
** that timer processes are executed on the lowest possible interrupt 
** level?
** On the other hand, this could easily be implemented by a user
** if so desired.
*/

void
tick(void)
{
  LOCKSTATE msr;
  register struct _pcb *saved_cur_p;
  cpuid_t cpu_id;
  cpuid_t num_cpus;


  (void)cpu_id;			/* SMP variable */
  (void)num_cpus;		/* SMP variable */

  /*
  ** Check that the caller is a superuser.
  ** Ignore the call if not.
  */
#if ERROR_CHECKS > 1
  ON_ERRLEVEL_2()
    if(unlikely(zzcur_p->user))
      {
	os_error(0, OSE_EILLEGAL_USER_CALL | OSE_TICK, (OSERRCODE) 0);	/* Non fatal. */
	return;
      }
#endif

  /* Check if tick is called from a non interrupt process. */
  if(zzcur_p->priority < 33)
    {
      /*
      ** Check that the caller is a superuser.
      ** Ignore the call if not.
      */
      if(!(zzcur_p->user))
	{
	  signal_fsem(zzuu.tickd_);
	}
      else
	{
	  /* @frbr@ call os_error() ? */
	}
      return;
    }

  /*
  ** Check that tick is not called concurrently by several processes.
  ** (Usually this will trigger an interrupt priority error in the TI layer.)
  */
#if ERROR_CHECKS > 0
  if(unlikely(concurrent_check))
    os_error(2, OSE_ECONCURRENT_TICK_CALL | OSE_TICK, (OSERRCODE) 0); /* Fatal. */
  concurrent_check = 1;
#endif

  /* Make sure that timer processes won't see the wakeup flag. */
  zzwakeflag = (OSBOOLEAN) 0;

  /* Update timer interrupt priority if a new caller is detected. */
  if(zzuu.timer_manager != zzcur_p)
    {
      zzuu.timer_manager = zzcur_p;
      zzuu.timer_int_priority = (zzcur_p->priority & 31);
    }

  /* Save old current process on stack. */
  saved_cur_p = zzcur_p;

  /*
  ** Update free running system tick counters and microsecond clock.
  ** The SMP toscb spinlock and zzuu.last_toscb stuff permits new timeout
  ** signal requests while we run timer interrupt processes below. 
  */
#ifdef OSE_SMP
  SPIN_LOCK(&zzuu.toscb_spin_lock);
  BKL_LOCK_SAVE(msr);
  ++zzuu.systime_guard; /* odd */ 

  /* Make the systime_guard variable visible for other cpus */
  SMP_STORE_MEMBAR();
    
  ++zzuu.ticks;
  zzww.ticks = zzuu.ticks;
  zzclear_microticks();

  /* Save reference to last Time Out Signal Control Block to handle. */
  zzuu.last_toscb =
    zzuu.tos_tick_list[((U32)zzuu.ticks) & zzuu.curtmo_mask].tail;
  /* Make current updates visible to other cores before systime_guard. */
  SMP_STORE_MEMBAR();
  ++zzuu.systime_guard; /* even */ 

  SMP_STORE_MEMBAR();
  BKL_LOCK_RESTORE(msr);
  SPIN_UNLOCK(&zzuu.toscb_spin_lock);

#else  /* Not SMP */
  LOCK_SAVE(msr);
  ++zzuu.ticks;
  zzclear_microticks();
  LOCK_RESTORE(msr);
#endif /* Not SMP */

  /*
  ** Restart the ``power save duration'' bookkeeping if the time has
  ** wrapped.
  */
#if USE_POWER_SAVE > 0
  if(!zzuu.ticks)
    zzuu.power_saved_ticks = (OSTICK) 0;
#endif

  /* Update profiling sample periods in the LDM. */
#if OSE_DEBUG > 0
  if(zzuu.tickhook_enabled)
    zzprof_tickhook();
#endif

  /* Run all timer processes from highest pri to lowest. */
  {
    U8 curtms = (U8) zzuu.ticks;
    struct _pcb *cur_ti;
    struct _pcb *next;

    cur_ti = zzuu.tmslist[curtms];
    zzuu.tmslist[curtms] = NULL;

    for(; cur_ti; cur_ti = next)
      {
	register U8 nexttmo = (U8) (curtms + (U8) cur_ti->tmo);

	/* Pick up the forward pointer before we destroy it. */
	next = cur_ti->fq;

	/* Reschedule the timer process for another period. */
	cur_ti->cur_tmo = nexttmo;
	cur_ti->fq = zzuu.tmslist[nexttmo];
	zzuu.tmslist[nexttmo] = cur_ti;

#ifdef OSE_SMP
	/* If the timer process is on another cpu, run it on that cpu. */
	{
	  cpuid_t cpu = cur_ti->cpu_id;
	  if(cpu != zzkrn_cpu_id())
	    {
	      if(cur_ti->status < ST_READY)
		{
		  LOCK_SAVE(msr);
		  send_krn_event(cpu, KRN_EVT_TICK_TI_PROC,
				 pcb_to_pid(cur_ti), NULL);
		  LOCK_RESTORE(msr);
		}
	      /* CONTINUE with next timer process. */
	      continue;
	    }
	}
#endif
	
	/* Run the timer process unless it is stopped. */
	if(cur_ti->status < ST_READY)
	  {
	     set_zzcur_p(cur_ti);

	    /* The target interface may require a uarea for interrupt and timer processes. */
	    zzcurrent_uarea = cur_ti->uarea;

	    /* This code is for systems without the debugger. */

#if OSE_DEBUG == 0
	    /* Call interrupt process. */
#if CALL_INTERRUPTS_DIRECTLY > 0
	    (*cur_ti->entrypoint)();
#else
	    zzcall_intproc(cur_ti->entrypoint);
#endif
#endif
  
	    /* This code is for systems with debugger present and interrupt profiling enabled. */
	  
#if OSE_DEBUG > 0
#if ENABLE_INTERRUPT_PROFILING > 0
	    if(zzuu.inthook_enabled & 0x0a)
	      {
		/* Call interrupt process swap in hook. */
		zzintswap_in(cur_ti, saved_cur_p);
		/* Call interrupt process. */
#if CALL_INTERRUPTS_DIRECTLY > 0
		(*cur_ti->entrypoint)();
#else
		zzcall_intproc(cur_ti->entrypoint);
#endif
		/* Call interrupt process swap out hook. */
		zzintswap_out(saved_cur_p, cur_ti);
	      }
	    else
	      {
		/* Call interrupt process. */
#if CALL_INTERRUPTS_DIRECTLY > 0
		(*cur_ti->entrypoint)();
#else
		zzcall_intproc(cur_ti->entrypoint);
#endif
	      }
#endif
#endif
	    
	    /* This code is for systems with debugger present and interrupt profiling disabled. */
	    
#if OSE_DEBUG > 0
#if ENABLE_INTERRUPT_PROFILING == 0
	    /* Call interrupt process. */
#if CALL_INTERRUPTS_DIRECTLY > 0
	    (*cur_ti->entrypoint)();
#else
	    zzcall_intproc(cur_ti->entrypoint);
#endif
#endif
#endif
	  }
      }	/* for */
  }

  /* Pop old current process from stack. */
  set_zzcur_p(saved_cur_p);


  /* The target interface may require a uarea for interrupt and timer processes. */
  zzcurrent_uarea = saved_cur_p->uarea;

  /*
  ** Inform other cpus, so they can handle timeouts and timeslicing.
  */
#ifdef OSE_SMP
  {
    PROCESS ticks_lsw = (PROCESS) zzuu.ticks; 
    num_cpus = zzkrn_num_cpus();
    for(cpu_id = 0; cpu_id < num_cpus; cpu_id++)
      {
	LOCK_SAVE(msr);
	/* The ticks counter is present in the event for debug purpose only. */
	if(cpu_id != zzkrn_cpu_id() && zzcpu[cpu_id].state == CPU_STATE_ACTIVE)
	  send_krn_event(cpu_id, KRN_EVT_TICK, ticks_lsw, NULL);
	LOCK_RESTORE(msr);
      }
  }
#endif

  /* Scan all set timers and expire those that reach timeout. */
  expire_timeouts();

  /* Timeslice prioritized and background processes. */
  handle_timeslicing();

  /* Expire time out signals of current tick slot. */
  expire_tmo_signals();

  /*
  ** Maintain check that tick is not called concurrently by several
  ** processes.
  */
#if ERROR_CHECKS > 0
  concurrent_check = 0;
#endif
}

/*
** This is the delay system call.
*/

void
delay(OSTIME timeout)
{
  register struct _pcb *pcb = zzcur_p;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  ON_ERRLEVEL_2()
    if(check_nonint_caller(OSE_DELAY))
      return;
#endif

  /* Delay forever is illegal. */
  if(OSE_NO_TIMEOUT == timeout)
    {
      os_error(0, OSE_EBAD_PARAMETER | OSE_DELAY, (OSERRCODE) timeout); /* Non fatal. */
      return;
    }

  /* Wait for as long a timeout as requested. */
  if(timeout)
    {
      /* Set up timeout. */
      LOCK();
      request_timeout(pcb, timeout);

      /* Change process status and swap. */
      pcb->status = (pr_state_t) ST_DELAY;
      make_mypri_not_ready(pcb);

      /* Clear the timeout. */
      cancel_timeout();
      UNLOCK();
    }
}

/*
** Absolute time references.
**
** The abs_sec and abs_tick variables contain the last known absolute time.
** These values are used to time stamp trace, monitor and catch events in
** the ldm.
** The values are manipulated with the set_abstime() and get_abstime()
** system calls.
** If abs_sec = 0, then the time was never set.
*/

#if OSE_DEBUG == 0
static U32 abs_sec;	/* Seconds since 0:0:0 1 jan-1970 GMT. */
static U32 abs_tick;	/* The corresponding system tick value. */
static U32 abs_usec;	/* The corresponding microseconds subtick value. */
#endif

/*
** This is the set_abstime() system call.
**
** It is used to make the last known absoulte time available to the kernel.
** The call sets a reference point in time, consisting of an absolute time
** and the corresponding kernel relative time obtained from get_systime().
**
** abs_time contains the current time expressed as a number of seconds
** after 00:00:00 1-jan 1970 GMT. (traditional unix format.)
** tick and microsecs contain the values obtained from get_systime() at
** the time corresponding to abs_time.
*/

#if OSE_DEBUG == 0
void
set_abstime(OSTICK abs_time, OSTICK tick, OSTICK microsecs)
{
  LOCKSTATE msr;
  BKL_LOCK_SAVE(msr);
  abs_sec = (U32) abs_time;
  abs_tick = (U32) tick;
  abs_usec = (U32) microsecs;
  BKL_LOCK_RESTORE(msr);
}
#endif

/*
** This is the get_abstime() system call.
**
** It returns the last time set with set_abstime).
**
** abs_time contains the current time expressed as a number of seconds
** after 00:00:00 1-jan 1970 GMT. (traditional unix format.)
** tick and microsecs contain the values obtained from get_systime() at
** the time corresponding to abs_time.
**
** If abs_time is zero, then the time was never set.
*/

#if OSE_DEBUG == 0
void
get_abstime(OSTICK *abs_time, OSTICK *tick, OSTICK *microsecs)
{
  LOCKSTATE msr;
  BKL_LOCK_SAVE(msr);
  *abs_time = (OSTICK) abs_sec;
  *tick = (OSTICK) abs_tick;
  *microsecs = (OSTICK) abs_usec;
  BKL_LOCK_RESTORE(msr);
}
#endif

/*
** This is the request_tmo_sig() call.
**
** Argument p is the timeout.
** Argument q is a signal pointer.
**
** Returns an identification of the created time out event.
** The signal pointer in user space is set to NIL by the interface library.
*/
static OSADDRESS
sysconf_48(OSADDRESS p, OSADDRESS q, OSADDRESS group)
{
  OSTICK	timeout = (OSTICK) p;
  union SIGNAL	*tmosig = (union SIGNAL *) q;
  struct _signal *sigadm;

  (void) group;			/* Remove compiler warning. */

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_REQUEST_TMO_SIG))
    return (OSADDRESS) OSE_ILLEGAL_TMOREF;
#endif

  /* Check out buffer pointer and get the signal control block. */
  sigadm = check_buffer(&tmosig, OSE_REQUEST_TMO_SIG);
  if(!sigadm)
    {
      return (OSADDRESS) OSE_ILLEGAL_TMOREF;
    }

  /* An infinite timeout is a stupid request. */
  if(OSE_NO_TIMEOUT == timeout)
    {
      os_error(0, OSE_EBAD_PARAMETER | OSE_REQUEST_TMO_SIG, (OSERRCODE) timeout); /* Non fatal. */
      return (OSADDRESS) OSE_ILLEGAL_TMOREF;
    }
  
#if MEMORY_MODEL > 0
  /*
  ** Copy a timeout signal in a foreign pool to the system segment.
  ** We do this to get rid of dying pools. An expired and sent timeout 
  ** signal may not be relocated to a new address in another pool. That
  ** would cause trouble in sys_cancel_tmo().
  ** A timeout signal in a pool of a foreign segment must have been
  ** stolen by restore(). This is a very odd case.
  ** See also: attach, sysd_clean_killed_pool.
  */
  if(local_sysctlblk.allow_pool_copy)
    {
      struct poolcb *pool = sigadm->buffer_head->poolcbp;
      
      if(pool != zzcur_p->block->sigpool && pool != &system_pool &&
	 sigadm->size <= system_pool.signal_list[7].size)
	{
	  struct _signal *new_sigadm = sig_to_adm(sys_alloc(sigadm->size, 0));
	  
	  /* Memcpy will not work for MAS pools?? */
	  memcpy(new_sigadm->udata, sigadm->udata, sigadm->size);
	  safe_free_buf(sigadm);
	  sigadm = new_sigadm;
	}
    }
#endif

  return (OSADDRESS) sys_request_tmo(timeout, sigadm, tmosig->sig_no);
}

/*
** This is the request_tmo() call.
**
** Argument p is the timeout.
** Argument q is a signal number.
**
** Returns an identification of the created time out event.
*/
static OSADDRESS
sysconf_49(OSADDRESS p, OSADDRESS q, OSADDRESS group)
{
  OSTICK	timeout = (OSTICK) p;
  SIGSELECT	tmosigno = (SIGSELECT) q;

  (void) group;			/* Remove compiler warning. */

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_REQUEST_TMO))
    return (OSADDRESS) OSE_ILLEGAL_TMOREF;
#endif

  /* An infinite timeout is a stupid request. */
  if(OSE_NO_TIMEOUT == timeout)
    {
      os_error(0, OSE_EBAD_PARAMETER | OSE_REQUEST_TMO, (OSERRCODE) timeout); /* Non fatal. */
      return (OSADDRESS) OSE_ILLEGAL_TMOREF;
    }
  
  return (OSADDRESS) sys_request_tmo(timeout, NULL, tmosigno);
}

/*
** This is the cancel_tmo_sig() call.
**
** Argument p is reserved for future use, currently always 1.
** Argument q is a timeout event identity.
**
** The call returns the corresponding time out signal pointer.
*/
static OSADDRESS
sysconf_50(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  OSTMOREF	tmoid = (OSTMOREF) q;
  struct _signal *sigadm;

  /* Remove ``not used args'' compiler warnings. */
  (void) p;
  (void) r;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_CANCEL_TMO_SIG))
    return (OSADDRESS) NIL;
#endif

  sigadm = sys_cancel_tmo(tmoid, 1, OSE_CANCEL_TMO_SIG);
  if(!sigadm)
    {
      return (OSADDRESS) NIL;
    }
  return (OSADDRESS) adm_to_sig(sigadm);
}

/*
** This is the cancel_tmo() call.
**
** Argument p is reserved for future use, currently always 1.
** Argument q is a timeout event identity.
** 
** An existing signal will be freed. The call returns nothing useful.
*/
static OSADDRESS
sysconf_51(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  OSTMOREF	tmoid = (OSTMOREF) q;

  /* Remove ``not used args'' compiler warnings. */
  (void) p;
  (void) r;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_CANCEL_TMO))
    return (OSADDRESS) 0;
#endif

  (void) sys_cancel_tmo(tmoid, 0, OSE_CANCEL_TMO);
  return (OSADDRESS)0;
}

/*
** This is the restart_tmo() call.
**
** Argument p is a timeout event identity.
** Argument q is a new timeout time.
** 
** The call returns nothing useful.
*/
static OSADDRESS
sysconf_52(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  OSTMOREF	tmoid = (OSTMOREF) p;
  OSTICK	new_time = (OSTICK) q;

  /* Remove ``not used args'' compiler warnings. */
  (void) r;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_RESTART_TMO))
    return (OSADDRESS) 0;
#endif

  /* Check the new_time parameter. */
  if(new_time == (OSTICK)OSE_NO_TIMEOUT)
    {
      os_error(0, OSE_EBAD_PARAMETER | OSE_RESTART_TMO, (OSERRCODE) new_time); /* Non fatal. */
      return (OSADDRESS)0;
    }

  sys_restart_tmo(tmoid, new_time, OSE_RESTART_TMO);
  return (OSADDRESS)0;
}

/*
**===========================================================================
** 14.4  Undocumented system calls.
**===========================================================================
*/

/*
** This is the zzticker_semwakeup call.
**
** It is called to transform a high priority timer interrupt into an 
** activation of tickd on a more suitable priority, calling tick(). This 
** could have been done by use of the public signal_fsem instead, but in 
** this way we avoid some overhead at each tick.
** 
** The call returns nothing useful.
*/

unsigned long
zzticker_semwakeup(void *arg) 
{
  (void)arg;
  request_int_semwakeup(pid_to_pcb(zzuu.tickd_));
  return OSE_ISR_RESCHED;
}

/*
** This is the undocumented set/get_timeslice system call.
**
** Argument p should be zero for just reading out the current timeslice
**            value and non zero for setting a new timeslice value.
** Argument q is the pid of the process (or zero for the calling process).
** Argument r is the new timeslice value (only used if p != 0).
** 
** This function returns the current timeslice value if p == 0 and
** the old timeslice value if p != 0.
**
** Note that all timeslice values are represented as number of ticks.
*/
static OSADDRESS
sysconf_55(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  LOCKSTATE msr;
  struct _pcb *cur_pp = zzcur_p;
  struct _pcb *pcb;
  OSPRIORITY pri;
  OSTIME tmo;

  if(q)
    {
      if(q != cur_pp->pid)
	{
	  PROCESS pid = (PROCESS)q;

	  /*
	  ** Read instance number and check the process id. Report error
	  ** if invalid.
	  */
	  pcb = check_pid(pid, 0);
	  if(pcb == NULL)
	    {
	      return (OSADDRESS) 0;
	    }

	  /* Locate the process descriptor. */
	  pcb = pid_to_pcb(pid);

	  /*
	  ** If the calling process is either BG or PRI we must lock the
	  ** scheduler.
	  */
	  if(cur_pp->priority < 33)
	    {
	      /* Lock the dispatcher. */
	      LOCK_SWAP();
	  
	      /* Check that the block/process is still present. */
	      if(pcb_to_pid(pcb) != pid)
		{
		  UNLOCK_SWAP();
		  return (OSPRIORITY) 0;
		}
	    }
	}
      else
	{
	  q = 0; /* Disable the UNLOCK_SWAP() below. */
	  pcb = cur_pp;
	}
    }
  else
    {
      pcb = cur_pp;
    }

  pri = pcb->priority;
  
  /* 0-31=PRI,32=BG,64-95=INT,96-127=TI,128=PHANT */
  if(((pri & 96) != 64) && pri < 128)
    {
      /* Process type is OS_BG_PROC or OS_PRI_PROC or OS_TI_PROC. */
      BKL_LOCK_SAVE(msr);

      /* Pick up the current tmo value */
      tmo = pcb->tmo;

      /* If p is non zero we should set a new timeout value. */
      if(p)
	{
	  if(pri < 32)
	    {
	      /* OS_PRI_PROC */
	      /* Check if we allow timeslicing for prioritized processes. */
	      if((local_sysctlblk.table_extended >= 7) &&
		 local_sysctlblk.timeslice_priprocs)
		{
		  pcb->tmo = (OSTIME)r;
		}
	    }
	  else
	    {
	      /* OS_BG_PROC and OS_TI_PROC. */
	      pcb->tmo = (OSTIME)(r ? r : 1);
	    }
	}
      BKL_LOCK_RESTORE(msr);
    }
  else
    {
      /* Process type is OS_INT_PROC or OS_PHANTOM. */
      tmo = 0;
    }

  if(q && (cur_pp->priority < 33))
    UNLOCK_SWAP();
  return tmo;
}

/* 
** The deprecated ose_get_time_resolution() and ose_set_time_resolution()
** system calls 
*/
static OSADDRESS
sysconf_74(OSADDRESS function, OSADDRESS prio, OSADDRESS res)
{
  (void)(function + prio + res);
  
  /* Deprecated */
  return 0;			/* Failed */
}

/*
*****************************************************************************
** 15 START/STOP.
*****************************************************************************
*/

/*
**===========================================================================
** 15.1  Initialization.
**===========================================================================
*/

/*
**===========================================================================
** 15.2  Local functions.
**===========================================================================
*/

/*
** Subfunction to start.
**
** Called for each process in a block when a block is started.
** A process can not start itself so we need not check for this case.
**
** Works even if the caller is an interrupt or timer process.
**
** Works for timer processes and non interrupt processes.
**
** Returns 1 if a process switch is required.
** Returns 0 if not.
**
** These values must be accumulated by the caller.
** The wasnt_stopped flag is set if the process wasn't stopped.
**
** The dispatcher is locked when this function is called from a
** non interrupt process.
*/

static int
start_one_proc(struct _pcb *pcb, int *wasnt_stopped, enum pr_state operation)
{
  LOCKSTATE msr;
  unsigned start_process = 0;
  
  /* Start current process makes no sense. */
  if(pcb == zzcur_p)
    return 0;

  /*
  ** Ignore call for interrupt process or phantom process.
  ** We also ignore processes that are killed and
  ** processes with a bad user number.
  ** We need to check the pcb->killed and zzcur_p->user
  ** fields here since we traverse a list that was not
  ** previously checked.
  */
  if((pcb->priority != 128) &&
     (
      (pcb->priority < 33) ||
      (pcb->priority > 95)
      ) &&
     !(pcb->killed) &&
     (zzcur_p->user == 0 || zzcur_p->user == pcb->real_user)
     )
    {
      LOCK_SAVE(msr);
      if(operation == ST_STOPPED)
        {
          /*
          ** Check if the block still has a parent. If it has, then it is
          ** a new block which is still owned by its parent, meaning
          ** that if the parent dies, the child should also be removed.
          ** Unlink the block from the parent when all processes in the
          ** block were started.
          */
#ifdef OSE_SMP
          if(pcb->block->parent)
            {
	      LOCKSTATE msr2;
	      volatile struct _block *vblock = pcb->block;

	      BKL_LOCK_SAVE(msr2);
	      if(vblock->parent)
                {
		  if(!(--vblock->st_cnt))
		    vblock->parent = NULL;
                }
	      BKL_LOCK_RESTORE(msr2);
            }
#else  /* !OSE_SMP */
          if(pcb->block->parent)
            {
              if(!(--pcb->block->st_cnt))
                {
                  pcb->block->parent = NULL;
                }
            }
#endif
	}

#ifdef OSE_SMP
      {
	cpuid_t cpu = pcb->cpu_id;
	/*  */
	if(cpu != zzkrn_cpu_id())
	  {
#if 0 /* A stop may be pending in the event pipe?? */
	    if(operation == ST_STOPPED && pcb->st_cnt == 0)
	      {
		*wasnt_stopped = 1;
		LOCK_RESTORE(msr);
		return 0;		/* No swap */
	      }
#endif

	    send_krn_event(cpu,
			   operation == ST_STOPPED
			   ? KRN_EVT_START : KRN_EVT_RESUME,
			   pcb_to_pid(pcb), NULL);
	    LOCK_RESTORE(msr);
      
	    return 0;		/* No swap */
	  }
      }
      /*
      ** It is vital that the process hasn't been moved to another cpu here.
      ** We know that, because interrupts are masked since that test.
      */
#endif

      if(operation == ST_STOPPED)
        {
          /*
          ** Decrement process stop counter and change the process state
          ** if required.
          ** We won't report any started already error now since the
          ** dispatcher is locked.
          ** Instead we set a flag so that the error can be reported
          ** later.
          ** Here we just ignore any illegal start calls.
          */
#if ERROR_CHECKS > 0
          /* 
          ** If we are starting an interrupt process or a system
          ** process we do not know if it was actually stopped or if
          ** it was explicitly excluded during the stop_or_intercept()
          ** call. So we do not perform the check in these cases.
	  */
          if((pcb->priority < 33) || !(pcb->debug_flags & 0x10000))
            {
              /* Check if the process is started. */
              if(!pcb->st_cnt)
                *wasnt_stopped = 1;
            }
#endif

	  if(pcb->st_cnt)
	    {         
	      if (!(--pcb->st_cnt))
		start_process = 1;
	    }
        }
      else if(operation == ST_INTERCEPTED && (pcb->status & ST_INTERCEPTED))
        {
          start_process = 1;
        }
      
      if(start_process)
        {
          pcb->status = (pr_state_t) (pcb->status & (~operation));
          if((pcb->status < ST_READY) && (pcb->priority < 33))
            {
              make_otherpri_ready_noswap(pcb); /* noswap */
            
              /* Return nonzero if a process switch is needed. */
              if(zzcur_ni->priority > pcb->priority)
                {
                  LOCK_RESTORE(msr);
                  return 1;
                }
            }
        }
      LOCK_RESTORE(msr);
    }
  return 0;
}

/*
** Subfunction to stop.
**
** Called for each process in a block when a block is stopped.
** It is not used to stop the current process.
**
** Works even if the caller is an interrupt or timer process.
**
** Works for timer processes and non interrupt processes.
**
** The dispatcher is locked when this function is called from a
** non interrupt process.
**
** Returns a non zero value (1) if we need to call stop_my_proc().
*/

static int
stop_one_proc(struct _pcb *pcb, enum pr_state operation, OSADDRESS exclude)
{
  LOCKSTATE msr;

  /* Check if the process matched any exclude criteria. */
   if(  ((exclude & BUT_KILL_LOCKED)    && (pcb->lock_cnt != 0))
     || ((exclude & BUT_INTERRUPT_PROC) && (pcb->priority > 32)) 
     || ((exclude & BUT_SYSTEM_PROC)    && (pcb->debug_flags & 0x10000)))
    {
      return 0;
    }
  
  /* If the caller should be stopped, tell the caller to do so itself. */
  if(pcb == zzcur_p)
    {
      if(exclude & BUT_CURRENT_PROC)
        return 0;
      else
        return 1;
    }
  
  /*
  ** Ignore call for interrupt process or phantom process.  We also
  ** ignore processes that are killed and processes with a bad user
  ** number.  We need to check the pcb->killed and zzcur_p->user
  ** fields here since we might be traversing a list that was not
  ** previously checked.
  */
  if((pcb->priority != 128) &&
     (
      (pcb->priority < 33) ||
      (pcb->priority > 95)
      ) &&
     !(pcb->killed) &&
     (zzcur_p->user ? (zzcur_p->user == pcb->real_user) : 1)
     )
    {
      LOCK_SAVE(msr);

#ifdef OSE_SMP
      {
	cpuid_t cpu_id = pcb->cpu_id;
	
	if(cpu_id != zzkrn_cpu_id())
	  {
	    volatile struct _pcb *vpcb = (volatile struct _pcb *)pcb;
      
	    send_krn_event(cpu_id,
			   operation == ST_STOPPED
			   ? KRN_EVT_STOP : KRN_EVT_INTERCEPT,
			   pcb_to_pid(pcb), NULL);
	    LOCK_RESTORE(msr);

	    /* 
	    ** For non-interrupt callers, spin until the victim is stopped.
	    ** Interrupt process callers shall not spin.
	    ** Loops if someone else starts it again before we see it stopped??
	    **
	    ** We spin here with locked dispatcher. This could cause deadlock
	    ** if some partner tries to lock the dispatcher with interrupts
	    ** disabled or if IPI interrupts not yet operable on the cpu of
	    ** the victim. Therefore, we enable interrupts inside LOCK_SWAP().
	    ** Either remove int-locked LOCK_SWAP or this spin loop.
	    ** 
	    ** Perhaps skip the poll loop if interrupts disabled by caller??
	    */
	    if(zzcur_p->priority <= 32)
	      {
		while(!(vpcb->status & operation) &&
		      zzcpu[cpu_id].ipi_operable)
		  ; /* empty while body */
	      }    

	    return 0;
	  }
      }
      /*
      ** It is vital that the process hasn't been moved to another cpu here.
      ** We know that, because interrupts are masked since that test.
      */
#endif

      /* Change process state if required. */
      if(!pcb->st_cnt || operation == ST_INTERCEPTED)
	{
	  pcb->status = (pr_state_t) (pcb->status | operation);
	  if(pcb->priority < 33)
	    {
	      make_otherpri_not_ready(pcb);
	    }
	}

      if(operation == ST_STOPPED)
        {
          /* Increment process stop counter. */
          pcb->st_cnt++;
         
          /* Check that the stop counter doesn't wrap. */
#if ERROR_CHECKS > 0
          if(unlikely(!pcb->st_cnt))
            {
              pcb->st_cnt--;
              LOCK_RESTORE(msr);
              os_error(0, OSE_ESTOP_OVERFLOW | OSE_STOP, (OSERRCODE) pcb_to_pid(pcb));	/* Non fatal. */
              return 0;
            }
#endif
        }
      LOCK_RESTORE(msr);
    }
  return 0;
}

/*
** Subfunction to stop.
**
** Called to stop the current process.
**
** Works even if the caller is an interrupt or timer process.
**
** Works for timer processes and non interrupt processes, but the caller
** must filter off interrupt processes and phantoms.
**
** The dispatcher is locked when this function is called from a
** non interrupt process.
** This causes make_mypri_not_ready() to skip the swap and set zzww.dp_req
** instead. UNLOCK_SWAP() does the dispatch operation later.
*/

static void
stop_my_proc(enum pr_state operation)
{
  LOCKSTATE msr;
  register struct _pcb *pcb = zzcur_p;

  LOCK_SAVE(msr);

  if(operation == ST_STOPPED)
    {
      /* Increment process stop counter. */
      pcb->st_cnt++;
     
      /* Check that the stop counter doesn't wrap. */
#if ERROR_CHECKS > 0
      if(unlikely(!pcb->st_cnt))
        {
          pcb->st_cnt--;
          LOCK_RESTORE(msr);
          os_error(0, OSE_ESTOP_OVERFLOW | OSE_STOP, (OSERRCODE) pcb_to_pid(pcb));	/* Non fatal. */
          return;
        }
#endif
    }

  /* Change process state if required. */
  pcb->status = (pr_state_t) (pcb->status | operation);
  if(pcb->priority < 33)
    {
      make_mypri_not_ready(pcb);
    }

  LOCK_RESTORE(msr);
}

/* 
** Call the state hooks of a block. The operation argument identifies the 
** actual system call. The dispatcher is locked by the caller.
** This function should be called while the processes of the block are all
** stopped or intercepted.
** The hook is called by a kernel event, on all cpus that are in use by 
** the block. On the current cpu, the hook is called directly, not in an 
** ISR context.
** Should we open up for local kernel events??
*/
static void
call_block_state_hooks(struct _block *bcb, OSERRCODE operation)
{
  struct statehookcb *hook = bcb->statehook_list;
  LOCKSTATE msr;

  while(hook)
    {
#ifdef OSE_SMP
      /*
      ** Call the hook if the owner is still alive.
      ** We have to issue one call for each cpu with processes in the block.
      */
      if(hook->owner == pcb_to_pid(pid_to_pcb(hook->owner)))
	{
	  cpuid_t my_cpu = zzkrn_cpu_id();
	  cpuid_t cpu_id;
	  struct _pcb *pcb;
#if OSE_SMP_MAX_CPUS <= 32
	  uint32_t cpumask;
#else
#error "FIXME > 32 cpus"
#endif
	  cpumask = 0;
	  for(pcb = bcb->proclist; pcb; pcb = pcb->proclist)
	    {
	      if(pcb->magic == MAGIC_PCB)
		cpumask |= 1u << pcb->cpu_id;
	    } /* for */

	  cpu_id = 0;
	  while(cpumask)
	    {
	      if(cpumask & 1)
		{
		  LOCK_SAVE(msr);
		  if(cpu_id == my_cpu)
		    (void)hook->func(pcb_to_pid(bcb), operation, hook->handle);
		  else
		    send_krn_event(cpu_id, KRN_EVT_BLOCK_HOOK, (PROCESS)hook,
				   (void *)operation);
		  LOCK_RESTORE(msr);
		}
	      cpumask = cpumask >> 1;
	      cpu_id++;
	    } /* while */
	}
#else
      /* Call the hook if the owner is still alive. */
      if(hook->owner == pcb_to_pid(pid_to_pcb(hook->owner)))
	{
	  LOCK_SAVE(msr);
	  hook->func(pcb_to_pid(bcb), operation, hook->handle);
	  LOCK_RESTORE(msr);
	}
#endif	/* OSE_SMP */
      hook = hook->next;
    } /* while */
}

/*
** This is the main workhorse for both start and resume.
*/

static void
start_or_resume(PROCESS pid, enum pr_state operation, OSERRCODE errcode)
{
  register struct _pcb *pcb;
  int wasnt_stopped;
  register OSBOOLEAN already_locked;
  
  /* Read instance number and check the process id. Report error if invalid. */
#if ERROR_CHECKS > 0
  pcb = check_pid_bid_seg(pid, errcode);
  if(pcb == NULL)
    return;

  /* Locate the block or process descriptor. */
  pcb = pid_to_pcb(pid);
#else
  pcb = pid_to_pcb(pid);
  if(pcb_to_pid(pcb) != pid)
    return;
#endif

  /*
  ** Report an error if we try to start a process that isn't stopped.
  */
  if(operation == ST_STOPPED)
    {
#if ERROR_CHECKS > 0
      wasnt_stopped = 0;
#endif
    }

  /* Check if the caller is a prioritized or background process. */
  if(zzcur_p->priority <= 32)
    {
      /*
      ** We need to save the dispatcher lock flag in order to maintain
      ** its value when we return.
      */
      already_locked = zzdispatcher_locked;
       
      /* Lock the dispatcher. */
      if(!already_locked)
	LOCK_SWAP();

      /* Check that the block/process is still present. */
      if(pcb_to_pid(pcb) != pid)
	{
	  if(!already_locked)
            UNLOCK_SWAP();
	  return;
	}

#if USE_REMOTE_CALLS
      /* Issue a remote system call if required. */
      if(local_sysctlblk.enable_remcalls)
	{
	  union SIGNAL *sig;
	  register OSBOOLEAN use_remcalls;
	  register PROCESS handler;
	  if(pcb->magic == MAGIC_PCB)
	    {
	      use_remcalls = pcb->block->use_remcalls;
	      handler = pcb->block->remcall_server;
	    }
	  else if(pcb->magic == MAGIC_BCB)
	    {
	      use_remcalls = ((struct _block *)pcb)->use_remcalls;
	      handler = ((struct _block *)pcb)->remcall_server;
	    }
	  else /* Skip remote calls for segments. */
	    {
	      use_remcalls = 0;
	      handler = 0; /* Kill compiler warning. */
	    }
          if(use_remcalls)
            {
              UNLOCK_SWAP();
              set_suid((OSBOOLEAN) 1);
              sig = alloc(sizeof(struct remote_call), REMOTE_CALL);
              if(operation == ST_STOPPED)
                sig->remote_call.call_type = REM_START;
              else
                sig->remote_call.call_type = REM_RESUME;
              sig->remote_call.params.rem_start.pid = pid;
	    
              sig = make_remote_call(sig, (OSCALLTYPE) REMR_START, handler);
              if(sig)
                {
                  free_buf(&sig);
                }
              set_suid((OSBOOLEAN) 0);
              return;
            }
	}
#endif

    }
  else
    {
      /* Not really needed but will remove a compiler warning. */
      already_locked = 0;
    }
  
  /* Check if a block is started. Then start all processes. */
  if(pcb->magic == MAGIC_BCB)
    {
      struct _block *bcb = (struct _block *)pcb;

      /* Call the block hooks with stopped/intercepted processes. */
      if(bcb->statehook_list)
	call_block_state_hooks(bcb, errcode);

      for(pcb = bcb->proclist; pcb != NULL; pcb = pcb->proclist)
	{
	  /*
	  ** Start the process. We set zzww.dp_req if a process
	  ** switch is needed.  This causes UNLOCK_SWAP() to
	  ** call the dispatcer.
	  */
	  if(start_one_proc(pcb, &wasnt_stopped, operation))
	    zzww.dp_req = (OSBOOLEAN) 1;
	}

      /* 
      ** Give us a way to keep a block without any process when the parent
      ** dies. The empty block may be used as a owner of a signal pool to
      ** be used by s_alloc_nil().
      */
      if(bcb->st_cnt == 0 && bcb->sigpool)
	{
	  bcb->parent = NULL;
	}
    }
  else if(pcb->magic == MAGIC_PCB)
    {
      /* Start a single process. */
      /*
      ** Start the process. We set zzww.dp_req if a process switch
      ** is needed.  This causes UNLOCK_SWAP() to call the
      ** dispatcer.
      */
      if(start_one_proc(pcb, &wasnt_stopped, operation))
	zzww.dp_req = (OSBOOLEAN) 1;
    }
  else
    {
      /* Start a segment. */
      struct _block *bcb;

      /* Scan all blocks for a match. */
      for(bcb = zzuu.bcb_list; bcb; bcb = bcb->next)
	{
	  /* Operate only on blocks that are in the segment. */
	  if(bcb->segpid == pid)
	    {
	      /* Call the block hooks with stopped/intercepted processes. */
	      if(bcb->statehook_list)
		call_block_state_hooks(bcb, errcode);

	      for(pcb = bcb->proclist; pcb; pcb = pcb->proclist)
		{
		  /*
		  ** Resume the process. We set zzww.dp_req if a
		  ** process switch is needed. This causes
		  ** UNLOCK_SWAP() to call the dispatcer.
		  */
		  if(start_one_proc(pcb, &wasnt_stopped, operation))
		    zzww.dp_req = (OSBOOLEAN) 1;
		}
	    }
	}
    }

  /*
  ** Unlock the dispatcher if the caller is a prioritized or
  ** background process, but only if the dispatcher was not already
  ** locked when we entered this function.
  */
  if(zzcur_p->priority <= 32 && !already_locked)
    UNLOCK_SWAP();

  if(operation == ST_STOPPED && errcode != OSE_BIND_PROCESS)
    {
#if ERROR_CHECKS > 0
      /* Report an error if we try to start a process that isn't stopped. */
      if(unlikely(wasnt_stopped))
        os_error(0, OSE_EALREADY_STARTED | OSE_START, (OSERRCODE) pid);	/* Non fatal. */
#endif
    }
}

/*
** This is the main workhorse for both stop and intercept.
*/

static OSBOOLEAN
stop_or_intercept(PROCESS pid, enum pr_state operation, OSERRCODE errcode,
                  OSADDRESS exclude)
{
  register struct _pcb *pcb;
  register int stop_caller = 0;
  register OSBOOLEAN already_locked;
  OSADDRESS bit_mask;

  /*
  ** Only the debugger may intercept the system segment (pid == 0). So
  ** if pid points out the system segment, then all flags must be set
  ** for the interception to take place.
  */
  bit_mask = (BUT_CURRENT_PROC | BUT_INTERRUPT_PROC | BUT_SYSTEM_PROC);
  if (pid == 0 && ((exclude & bit_mask) == bit_mask))
    {
      pcb = (struct _pcb *)&system_segment;
    }
  else
    {
      /* Check the process id. Report error if invalid. */
#if ERROR_CHECKS > 0
      pcb = check_pid_bid_seg(pid, errcode);
      if(pcb == NULL)
	return 0;
#else
      pcb = pid_to_pcb(pid);
      if(pcb_to_pid(pcb) != pid)
	return 0;
#endif
    }

  /* Check if the caller is a prioritized or background process. */
  if(zzcur_p->priority <= 32)
    {
      /*
      ** We need to save the dispatcher lock flag in order to maintain
      ** its value when we return.
      */
      already_locked = zzdispatcher_locked;

      /* Lock the dispatcher. */
      if(!already_locked)
	LOCK_SWAP();
  
      /* Check that the block/process is still present. */
      if(pcb_to_pid(pcb) != pid)
	{
          if(!already_locked)
            UNLOCK_SWAP();
	  return 0;
	}
  
#if USE_REMOTE_CALLS
      /* Issue a remote system call if required. */
      if(local_sysctlblk.enable_remcalls)
	{
	  union SIGNAL *sig;
	  register OSBOOLEAN use_remcalls;
	  register PROCESS handler;
	  if(pcb->magic == MAGIC_PCB)
	    {
	      use_remcalls = pcb->block->use_remcalls;
	      handler = pcb->block->remcall_server;
	    }
	  else if(pcb->magic == MAGIC_BCB)
	    {
	      use_remcalls = ((struct _block *)pcb)->use_remcalls;
	      handler = ((struct _block *)pcb)->remcall_server;
	    }
	  else /* Skip remote calls for segments. */
	    {
	      use_remcalls = 0;
	      handler = 0; /* Kill compiler warning. */
	    }
          if(use_remcalls)
	    {
	      /*
	      ** Skip the remote call if the dispatcher is locked.
	      ** This occurs when a debugger hook in the ldm calls
	      ** intercept. Ignoring the call is the correct action
	      ** since a remote process Shouldn't have been in the
	      ** breakpoint scope in the first place.
	      */
	      if(operation == ST_INTERCEPTED && already_locked)
		return 0;

	      /* Do the remote call. */
              UNLOCK_SWAP();
	      set_suid((OSBOOLEAN) 1);
	      sig = alloc(sizeof(struct remote_call), REMOTE_CALL);
              if(operation == ST_STOPPED)
                sig->remote_call.call_type = REM_STOP;
              else
                sig->remote_call.call_type = REM_INTERCEPT;
	      sig->remote_call.params.rem_stop.pid = pid;
	  
	      sig = make_remote_call(sig, (OSCALLTYPE) REMR_STOP, handler);
	      if(sig)
		{
		  free_buf(&sig);
		}
	      set_suid((OSBOOLEAN) 0);
	      return 0;
	    }
	}
#endif

    }
  else
    {
      /* Not really needed but will remove a compiler warning. */
      already_locked = 0;
    }

  /* Check if a block is stopped. Then stop all processes. */
  if(pcb->magic == MAGIC_BCB)
    {
      struct _block *bcb = (struct _block *) pcb;

      for(pcb = bcb->proclist; pcb != NULL; pcb = pcb->proclist)
	{
	  /* Stop the process. */
	  stop_caller |= stop_one_proc(pcb, operation, exclude);
	}

      /* Call the block hooks with stopped/intercepted processes. */
      if(bcb->statehook_list)
	call_block_state_hooks(bcb, errcode);
    }
  else if(pcb->magic == MAGIC_PCB)
    {
      /* Stop the process. */
      stop_caller = stop_one_proc(pcb, operation, exclude);
    }
  else
    {
      /* Stop a segment. */
      struct _block *bcb;
      
      /* Scan all blocks for a match. */
      for(bcb = zzuu.bcb_list; bcb; bcb = bcb->next)
	{
	  /* Operate only on blocks that are in the segment. */
	  if(bcb->segpid == pid)
	    {
	      for(pcb = bcb->proclist;
		  pcb != NULL;
		  pcb = pcb->proclist)
		{
		  /* Stop the process. */
		  stop_caller |= stop_one_proc(pcb, operation, exclude);
		}
	      /* Call the block hooks with stopped/intercepted processes. */
	      if(bcb->statehook_list)
		call_block_state_hooks(bcb, errcode);
	    }
	}
    }
  
  /* Stop the calling process if required. */
  if(stop_caller)
    {
      stop_my_proc(operation);
    }
  
  /*
  ** Unlock the dispatcher if the caller is a prioritized or
  ** background process, but only if the dispatcher was not already
  ** locked when we entered this function.
  */
  if(zzcur_p->priority <= 32 && !already_locked)
    UNLOCK_SWAP();

  return (OSBOOLEAN)stop_caller;
}

/*
**===========================================================================
** 15.3  System calls.
**===========================================================================
*/

/*
** Start a process, a block or a segment.
**
** Do we check user numbers so that only processes with an allowed
** user number are started? Is this the same for other system calls
** that operate on blocks??
*/


void
start(PROCESS pid)
{
  start_or_resume(pid, ST_STOPPED, OSE_START);
}

/*
** Stop a process, a block or a segment.
*/

void
stop(PROCESS pid)
{
  (void)stop_or_intercept(pid, ST_STOPPED, OSE_STOP, 0);
}


/*
**===========================================================================
** 15.4  Undocumented system calls.
**===========================================================================
*/

/*
*****************************************************************************
** 16 SEND/RECEIVE.
*****************************************************************************
*/

/*
**===========================================================================
** 16.1  Initialization.
**===========================================================================
*/

/*
**===========================================================================
** 16.2  Local functions.
**===========================================================================
*/

/*
** Check if a signal number matches a sigselect list.
**
** We know that the sigselect array is not empty.
** Return a nonzero value if the signal should be accepted.
*/
static int
iswanted(SIGSELECT received_signo, const SIGSELECT *sigsel)
{
  register W16 count;
  register SIGSELECT signo;
  register const SIGSELECT *wanted;

  /* Pick up parameters to registers. */
  signo = received_signo;
  wanted = sigsel;

  /*
  ** Check if positive or negative test should be executed.
  **
  ** We do this test by truncating to a 16 bit variable before
  ** testing the sign bit. The reason for this is that we want
  ** to make sure that everything works even for pose ports
  ** with 16 bit signal numbers.
  ** The code breaks for select arrays with more than 32k entries, ??
  ** wich is an unreasonable size.
  */
  count = (W16) *wanted++;
  if(count > 0)
    {
      /* Check received signo against each wanted signo. */
      do
	{
	  /* Accept the first match. */
	  if(signo == *wanted++)
	    return 1;
	}
      while(--count);
      /* Not found. Reject the signal. */
      return 0;
    }
  else
    {
      /* Check received signo against each wanted signo. */
      do
	{
	  /* Reject the first match. */
	  if(signo == *wanted++)
	    return 0;
	}
      while(++count);
      /* Not found. Accept the signal. */
      return 1;
    }
}

/*
** Subfunction to local_receive_with().
** Check if a signal number and second signal field
** matches a sigselect list.
**
** We know that the sigselect array is not empty.
*/
static int
iswanted_with(SIGSELECT received_signo,
	      SIGSELECT second,
	      const SIGSELECT *sigsel)
{
  register W16 count;
  register SIGSELECT signo;
  register const SIGSELECT *wanted;

  /* Pick up parameters to registers. */
  signo = received_signo;
  wanted = sigsel;

  /* Check if positive or negative test should be executed. */
  count = (W16) *wanted++;
  if(count > 0)
    {
      /* Check received signo against each wanted signo. */
      do
	{
	  /* Accept the first match. */
	  if(signo == *wanted++)
	    {
	      if(second == *wanted++)
		return 1;
	    }
	  else
	    wanted++;
	}
      while(--count);
      /* Not found. Reject the signal. */
      return 0;
    }
  else
    {
      /* Check received signo against each wanted signo. */
      do
	{
	  /* Reject the first match. */
	  if(signo == *wanted++)
	    {
	      if(signo == *wanted++)
		return 0;
	    }
	  else
	    wanted++;
	}
      while(++count);
      /* Not found. Accept the signal. */
      return 1;
    }
}

/*
** Subfunction to send() and send_w_s().
**
** Handles signals sent to phantoms with the S-port attribute set.
** Such send operations have different semantics as there may be many
** processes waiting at the signal queue.
** At this point, we know that the target is a phantom process.
**
** This version of send_to_sport() works when the caller is any
** type of process.
** The dispatcher should be locked when called from a non interrupt process.
*/

static void
send_to_sport(union SIGNAL **sig, struct _signal *sigadm, struct _pcb *pcb)
{
  LOCKSTATE msr;
  struct S_node *s_node;
  struct S_nodelist *s_nodelist;

  /* Done if the phantom is not an s-port. */
  if(!pcb->is_an_sport)
    return;

  /* Take buffer from user. */
  *sig = NIL;

  /* Make system the signal owner. */
  sigadm->owner = (PCB_INDEX_T) 0;

  /* Clear the forward pointer in preparation for putting it into the queue. */
  sigadm->next = NULL;

  /* Disable interrupts if the caller is a non interrupt process. */
  BKL_LOCK_SAVE(msr);

  /* Decrement owned counter in callers pcb. */
  zzcur_p->sig_cnt_owned--;

  /* Link signal into mailbox of target process. */
  if(pcb->mailbox)
    {
      pcb->mailtail->next = sigadm;
      pcb->mailtail = sigadm;
    }
  else
    {
      pcb->mailbox = sigadm;
      pcb->mailtail = sigadm;
    }
  pcb->sig_cnt_queued++;

  /* Bump signal queue modification counter. */
  pcb->s_pcb.add_listmodnumber++;

  /* Make all waiting processes ready. */ 
  for(s_node = pcb->s_pcb.s_nodelist; s_node; s_node = s_node->fp_s_nodelist)
    {
      /* Pick up the pcb of a process waiting. */
      s_nodelist = s_node->base;
      pcb = s_nodelist->my_pcb;
      
      /* We know that the target process is a prioritized or background process.
      ** If it is in receive then make it ready for subsequent swap.
      */
#ifdef OSE_SMP
      {
	cpuid_t target_cpu = pcb->cpu_id;
	if(target_cpu != zzkrn_cpu_id())
	  {
	    send_krn_event(target_cpu, KRN_EVT_SEND_SPORT, pcb_to_pid(pcb),
			   NULL);
	    /* CONTINUE with next pcb. */
	    continue;
	  }
      }
#endif
      if(pcb->status & ST_RECEIVE)
	{
	  pcb->status = (pr_state_t) (pcb->status & (~ST_RECEIVE_FSEM));
	  
	  /* If the process becomes ready then do a swap. */
	  if(pcb->status < ST_READY)
	    {
	      make_otherpri_ready_noswap(pcb); /* noswap */
	    }
	}
    }

  /* Reenable interrupts if the caller is an int or timer process. */
  BKL_LOCK_RESTORE(msr);
}

/*
** Subfunction to receive_from.
**
** Does same job as receive(), except the from field is checked
** before a signal is accepted.
**
** We know that the caller is not an interrupt or timer process.
*/

static union SIGNAL *
local_receive_from(const SIGSELECT *sigsel, PROCESS from)
{
  register struct _pcb *pcb;
  register const SIGSELECT *wanted;
  register struct _signal *prevbuf;
  register struct _signal *thisbuf;
  register SIGSELECT all_wanted;
  SIGSELECT sig_no = 0; /* Kill "may be used uninitialized" warning. */
  
#if (OSE_DEBUG + MEMORY_MODEL) > 0
  /* Retry here if a received signal was killed by the debugger. */
 retry:
#endif

  /* Get important pointers to registers. */
  pcb = zzcur_p;
  wanted = sigsel;
      
  /* Create a flag that says if we need to  check the sigsel array. */
  all_wanted = !*wanted;
      
  /* Give list of wanted signals to the debugger. */
  pcb->wanted = wanted;
      
  /* Disable interrupts. */
  LOCK();
      
  /* Restart search here if the queue was flushed. */
 restart_receive:
      
  /* Wait for first buffer to arrive. */
  prevbuf = pcb->mailbox;
  if(prevbuf)
    {
      /*
      ** We need to read out the signal number from the user pool
      ** with interrupts disabled, otherwise the user pool might
      ** be reclaimed and thus the memory will become unmapped,
      ** causing the access to the signal number to crash.
      */
      if (!all_wanted)
	sig_no =  *(SIGSELECT *)(adm_to_sig(prevbuf));

      /* Buffer present. Check it. */
      UNLOCK();
      if((from == prevbuf->sender_id) &&
	 (all_wanted || iswanted(sig_no, wanted)))
	{
	  /* If wanted, unlink and return the signal unless flushed. */
	  LOCK();
	  if(pcb->flushed)
	    {
	      /* Clear flushed flag and restart if flushed. */
	      pcb->flushed = (OSBOOLEAN) 0;
	      goto restart_receive;
	    }
	  pcb->mailbox = prevbuf->next;
	  pcb->sig_cnt_queued--;

	  /* Set new signal owner. */
	  prevbuf->owner = pcb_to_pindex(pcb); /* Note! Owner is 16-bits. @frbr@ */

	  /* Increment owned counter in callers pcb. */
	  pcb->sig_cnt_owned++;
	  UNLOCK();

	  /* Trace the signal. */
#if OSE_DEBUG > 0
	  if(zzuu.debugger_enabled)
	    {
	      register struct _pcb *from_pcb;
	      /* Lock the dispatcher. */
	      LOCK_SWAP();
	      /* Check that the sender is still alive. */
	      from_pcb = pid_to_pcb(prevbuf->sender_id);
	      if(pcb_to_pid(from_pcb) == prevbuf->sender_id)
		{
		  /* check for trace receive request. */
		  if((pcb->debug_flags | from_pcb->debug_flags) & 0x02)
		    {
		      /* We must check if the signal was killed by the debugger. */
		      if(!zzreceive_dbghook((OSBUFSIZE) prevbuf->size,
					    adm_to_sig(prevbuf),
					    pcb_to_debug_pcb(from_pcb),
					    pcb_to_debug_pcb(pcb)))
			{
			  /* Retry receive if the ldm killed the signal. */
			  UNLOCK_SWAP();
			  goto retry;
			}
		    }
		}
	      /* Unlock the dispatcher. */
	      UNLOCK_SWAP();
	    }
#endif

	  /* Copy the signal to the callers segment if required. */
#if MEMORY_MODEL == 0
	  return adm_to_sig(prevbuf);
#else
	  {
	    union SIGNAL *nsig;
	    /* Check if intersegment copying is enabled. */
	    if(!local_sysctlblk.allow_pool_copy ||
	       prevbuf->buffer_head->poolcbp->no_copy)
	      /* Done otherwise. */
	      return adm_to_sig(prevbuf);
	    /*
	    ** Check if the segment of the received signal is the one
	    ** in which the callers pool resides.
	    */
	    if(zzcur_p->cursigpool->segid == prevbuf->buffer_head->segid)
	      /* Done if they are the same. */
	      return adm_to_sig(prevbuf);
	    /* Copy signal contents to a new buffer. */
	    nsig = rec_sig_dup(prevbuf);
	    if(!nsig)
	      /* Restart the receive operation if the copy failed. */
	      goto retry;
	    /* Return the new buffer. */
	    return nsig;
	  }
#endif
	}
      else
	{
	  /* If not wanted, check next signal in the queue. */
	  LOCK();
	  thisbuf = prevbuf->next;
	}
    }
  else
    {
      /* No buffer present. Swap while waiting. */
      /* Done if a timeout occurred. */
      if(pcb->status & ST_TIMEOUT)
	{
	  UNLOCK();
	  return NIL;
	}
      pcb->status = (pr_state_t) ST_RECEIVE;
      make_mypri_not_ready(pcb);
      goto restart_receive;
    }
  
  /* Check second and following buffers. (Ints disabled.) */
  for(;;)
    {
      /* Clear flushed flag and restart if flushed. */
      if(pcb->flushed)
	{
	  pcb->flushed = (OSBOOLEAN) 0;
	  goto restart_receive;
	}
      
      /* Is there a buffer to check? */
      if(thisbuf)
	{
	  /*
	  ** We need to read out the signal number from the user pool
	  ** with interrupts disabled, otherwise the user pool might
	  ** be reclaimed and thus the memory will become unmapped,
	  ** causing the access to the signal number to crash.
	  */
	  if(!all_wanted)
	    sig_no =  *(SIGSELECT *)(adm_to_sig(thisbuf));

	  /* Yes, check it. */
	  UNLOCK();
          if((from == thisbuf->sender_id) &&
	     (all_wanted || iswanted(sig_no, wanted)))
	    {
	      /* If wanted, unlink and return the signal unless flushed. */
	      LOCK();
	      if(pcb->flushed)
		{
		  /* Clear flushed flag and restart if flushed. */
		  pcb->flushed = (OSBOOLEAN) 0;
		  goto restart_receive;
		}
              prevbuf->next = thisbuf->next;
	      if(!prevbuf->next)
		{
		  pcb->mailtail = prevbuf;
		}
	      pcb->sig_cnt_queued--;

	      /* Set new signal owner. */
	      thisbuf->owner = pcb_to_pindex(pcb); /* Note! Owner is 16-bits. @frbr@ */
	      /* Increment owned counter in callers pcb. */
	      pcb->sig_cnt_owned++;
	      UNLOCK();

	      /* Trace the signal. */
#if OSE_DEBUG > 0
	      if(zzuu.debugger_enabled)
		{
		  register struct _pcb *from_pcb;
		  /* Lock the dispatcher. */
		  LOCK_SWAP();
		  /* Check that the sender is still alive. */
		  from_pcb = pid_to_pcb(thisbuf->sender_id);
		  if(pcb_to_pid(from_pcb) == thisbuf->sender_id)
		    {
		      /* check for trace receive request. */
		      if((pcb->debug_flags | from_pcb->debug_flags) & 0x02)
			{
			  /* We must check if the signal was killed by the debugger. */
			  if(!zzreceive_dbghook((OSBUFSIZE) thisbuf->size,
						adm_to_sig(thisbuf),
						pcb_to_debug_pcb(from_pcb),
						pcb_to_debug_pcb(pcb)))
			    {
			      /* Retry receive if the ldm killed the signal. */
			      UNLOCK_SWAP();
			      goto retry;
			    }
			}
		    }
		  /* Unlock the dispatcher. */
		  UNLOCK_SWAP();
		}
#endif
	      /* Copy the signal to the callers segment if required. */
#if MEMORY_MODEL == 0
	      return adm_to_sig(thisbuf);
#else
	      {
		union SIGNAL *nsig;
		/* Check if intersegment copying is enabled. */
		if(!local_sysctlblk.allow_pool_copy ||
		   thisbuf->buffer_head->poolcbp->no_copy)
		  /* Done otherwise. */
		  return adm_to_sig(thisbuf);
		/*
		** Check if the segment of the received signal is the one
		** in which the callers pool resides.
		*/
		if(zzcur_p->cursigpool->segid == thisbuf->buffer_head->segid)
		  /* Done if they are the same. */
		  return adm_to_sig(thisbuf);
		/* Copy signal contents to a new buffer. */
		nsig = rec_sig_dup(thisbuf);
		if(!nsig)
		  /* Restart the receive operation if the copy failed. */
		  goto retry;
		/* Return the new buffer. */
		return nsig;
	      }
#endif
	    }
	  else
	    {
	      /* If not wanted, check next signal in the queue. */
	      prevbuf = thisbuf;
	      LOCK();
	      thisbuf = prevbuf->next;
	    }
	}
      else
	{
	  /* No, wait for next buffer to arrive. */
	  /* Done if a timeout occurred. */
	  if(pcb->status & ST_TIMEOUT)
	    {
	      UNLOCK();
	      return NIL;
	    }
	  pcb->status = (pr_state_t) ST_RECEIVE;
	  make_mypri_not_ready(pcb);
	  thisbuf = prevbuf->next;
	}
    }
}

/*
** Subfunction to receive_with.
**
** Does same job as receive(), except the from field is checked
** before a signal is accepted, and the second signal field
** is checked as required by receive_with().
**
** We know that the caller is not an interrupt or timer process.
*/

static union SIGNAL *
local_receive_with(const SIGSELECT *sigsel, PROCESS from)
{
  register struct _pcb *pcb;
  register const SIGSELECT *wanted;
  register struct _signal *prevbuf;
  register struct _signal *thisbuf;
  register SIGSELECT all_wanted;
  SIGSELECT sig_no0 = 0; /* Kill "may be used uninitialized" warning. */
  SIGSELECT sig_no1 = 0; /* Kill "may be used uninitialized" warning. */
  
#if (OSE_DEBUG + MEMORY_MODEL) > 0
  /* Retry here if a received signal was killed by the debugger. */
 retry:
#endif

  /* Get important pointers to registers. */
  pcb = zzcur_p;
  wanted = sigsel;
      
  /* Create a flag that says if we need to check the sigsel array. */
  all_wanted = !*wanted;
      
  /* Give list of wanted signals to the debugger. */
  /*
  ** The debugger can't understand this array, but we are going to ??
  ** ignore that fact since this is only a prototype call just now.
  */
  pcb->wanted = wanted;
      
  /* Disable interrupts. */
  LOCK();
      
  /* Restart search here if the queue was flushed. */
 restart_receive:
      
  /* Wait for first buffer to arrive. */
  prevbuf = pcb->mailbox;
  if(prevbuf)
    {
      /*
      ** We need to read out the signal number from the user pool
      ** with interrupts disabled, otherwise the user pool might
      ** be reclaimed and thus the memory will become unmapped,
      ** causing the access to the signal number to crash.
      */
      if(!all_wanted)
	{
	  sig_no0 = *(((SIGSELECT *)(adm_to_sig(prevbuf))) + 0);
	  sig_no1 = *(((SIGSELECT *)(adm_to_sig(prevbuf))) + 1);
	}

      /* Buffer present. Check it. */
      UNLOCK();
      if(((!from) || (from == prevbuf->sender_id)) &&
	 (all_wanted || iswanted_with(sig_no0, sig_no1, wanted)))
	{
	  /* If wanted, unlink and return the signal unless flushed. */
	  LOCK();
	  if(pcb->flushed)
	    {
	      /* Clear flushed flag and restart if flushed. */
	      pcb->flushed = (OSBOOLEAN) 0;
	      goto restart_receive;
	    }
	  pcb->mailbox = prevbuf->next;
	  pcb->sig_cnt_queued--;

	  /* Set new signal owner. */
	  prevbuf->owner = pcb_to_pindex(pcb); /* Note! Owner is 16-bits. @frbr@ */
	  /* Increment owned counter in callers pcb. */
	  pcb->sig_cnt_owned++;
	  UNLOCK();

	  /* Trace the signal. */
#if OSE_DEBUG > 0
	  if(zzuu.debugger_enabled)
	    {
	      register struct _pcb *from_pcb;
	      /* Lock the dispatcher. */
	      LOCK_SWAP();
	      /* Check that the sender is still alive. */
	      from_pcb = pid_to_pcb(prevbuf->sender_id);
	      if(pcb_to_pid(from_pcb) == prevbuf->sender_id)
		{
		  /* check for trace receive request. */
		  if((pcb->debug_flags | from_pcb->debug_flags) & 0x02)
		    {
		      /* We must check if the signal was killed by the debugger. */
		      if(!zzreceive_dbghook((OSBUFSIZE) prevbuf->size,
					    adm_to_sig(prevbuf),
					    pcb_to_debug_pcb(from_pcb),
					    pcb_to_debug_pcb(pcb)))
			{
			  /* Retry receive if the ldm killed the signal. */
			  UNLOCK_SWAP();
			  goto retry;
			}
		    }
		}
	      /* Unlock the dispatcher. */
	      UNLOCK_SWAP();
	    }
#endif

	  /* Copy the signal to the callers segment if required. */
#if MEMORY_MODEL == 0
	  return adm_to_sig(prevbuf);
#else
	  {
	    union SIGNAL *nsig;
	    /* Check if intersegment copying is enabled. */
	    if(!local_sysctlblk.allow_pool_copy ||
	       prevbuf->buffer_head->poolcbp->no_copy)
	      /* Done otherwise. */
	      return adm_to_sig(prevbuf);
	    /*
	    ** Check if the segment of the received signal is the one
	    ** in which the callers pool resides.
	    */
	    if(zzcur_p->cursigpool->segid == prevbuf->buffer_head->segid)
	      /* Done if they are the same. */
	      return adm_to_sig(prevbuf);
	    /* Copy signal contents to a new buffer. */
	    nsig = rec_sig_dup(prevbuf);
	    if(!nsig)
	      /* Restart the receive operation if the copy failed. */
	      goto retry;
	    /* Return the new buffer. */
	    return nsig;
	  }
#endif
	}
      else
	{
	  /* If not wanted, check next signal in the queue. */
	  LOCK();
	  thisbuf = prevbuf->next;
	}
    }
  else
    {
      /* No buffer present. Swap while waiting. */
      /* Done if a timeout occurred. */
      if(pcb->status & ST_TIMEOUT)
	{
	  UNLOCK();
	  return NIL;
	}
      pcb->status = (pr_state_t) ST_RECEIVE;
      make_mypri_not_ready(pcb);
      goto restart_receive;
    }
  
  /* Check second and following buffers. (Ints disabled.) */
  for(;;)
    {
      /* Clear flushed flag and restart if flushed. */
      if(pcb->flushed)
	{
	  pcb->flushed = (OSBOOLEAN) 0;
	  goto restart_receive;
	}
      
      /* Is there a buffer to check? */
      if(thisbuf)
	{
	  /*
	  ** We need to read out the signal number from the user pool
	  ** with interrupts disabled, otherwise the user pool might
	  ** be reclaimed and thus the memory will become unmapped,
	  ** causing the access to the signal number to crash.
	  */
	  if(!all_wanted)
	    {
	      sig_no0 = *(((SIGSELECT *)(adm_to_sig(thisbuf))) + 0);
	      sig_no1 = *(((SIGSELECT *)(adm_to_sig(thisbuf))) + 1);
	    }

	  /* Yes, check it. */
	  UNLOCK();
	  if(((!from) || (from == thisbuf->sender_id)) && 
	     (all_wanted || iswanted_with(sig_no0, sig_no1, wanted)))
	    {
	      /* If wanted, unlink and return the signal unless flushed. */
	      LOCK();
	      if(pcb->flushed)
		{
		  /* Clear flushed flag and restart if flushed. */
		  pcb->flushed = (OSBOOLEAN) 0;
		  goto restart_receive;
		}
              prevbuf->next = thisbuf->next;
	      if(!prevbuf->next)
		{
		  pcb->mailtail = prevbuf;
		}
	      pcb->sig_cnt_queued--;

	      /* Set new signal owner. */
	      thisbuf->owner = pcb_to_pindex(pcb); /* Note! Owner is 16-bits. @frbr@ */
	      /* Increment owned counter in callers pcb. */
	      pcb->sig_cnt_owned++;
	      UNLOCK();

	      /* Trace the signal. */
#if OSE_DEBUG > 0
	      if(zzuu.debugger_enabled)
		{
		  register struct _pcb *from_pcb;
		  /* Lock the dispatcher. */
		  LOCK_SWAP();
		  /* Check that the sender is still alive. */
		  from_pcb = pid_to_pcb(thisbuf->sender_id);
		  if(pcb_to_pid(from_pcb) == thisbuf->sender_id)
		    {
		      /* check for trace receive request. */
		      if((pcb->debug_flags | from_pcb->debug_flags) & 0x02)
			{
			  /* We must check if the signal was killed by the debugger. */
			  if(!zzreceive_dbghook((OSBUFSIZE) thisbuf->size,
						adm_to_sig(thisbuf),
						pcb_to_debug_pcb(from_pcb),
						pcb_to_debug_pcb(pcb)))
			    {
			      /* Retry receive if the ldm killed the signal. */
			      UNLOCK_SWAP();
			      goto retry;
			    }
			}
		    }
		  /* Unlock the dispatcher. */
		  UNLOCK_SWAP();
		}
#endif
	      /* Copy the signal to the callers segment if required. */
#if MEMORY_MODEL == 0
	      return adm_to_sig(thisbuf);
#else
	      {
		union SIGNAL *nsig;
		/* Check if intersegment copying is enabled. */
		if(!local_sysctlblk.allow_pool_copy ||
		   thisbuf->buffer_head->poolcbp->no_copy)
		  /* Done otherwise. */
		  return adm_to_sig(thisbuf);
		/*
		** Check if the segment of the received signal is the one
		** in which the callers pool resides.
		*/
		if(zzcur_p->cursigpool->segid == thisbuf->buffer_head->segid)
		  /* Done if they are the same. */
		  return adm_to_sig(thisbuf);
		/* Copy signal contents to a new buffer. */
		nsig = rec_sig_dup(thisbuf);
		if(!nsig)
		  /* Restart the receive operation if the copy failed. */
		  goto retry;
		/* Return the new buffer. */
		return nsig;
	      }
#endif
	    }
	  else
	    {
	      /* If not wanted, check next signal in the queue. */
	      prevbuf = thisbuf;
	      LOCK();
	      thisbuf = prevbuf->next;
	    }
	}
      else
	{
	  /* No, wait for next buffer to arrive. */
	  /* Done if a timeout occurred. */
	  if(pcb->status & ST_TIMEOUT)
	    {
	      UNLOCK();
	      return NIL;
	    }
	  pcb->status = (pr_state_t) ST_RECEIVE;
	  make_mypri_not_ready(pcb);
	  thisbuf = prevbuf->next;
	}
    }
}

/*
** Subfunction to receive_fsem.
**
** Does same job as receive(), except it also returns when the fast
** semaphore of the process is signalled.
**
** We know that the caller is not an interrupt or timer process.
*/

static union SIGNAL *
local_receive_fsem(const SIGSELECT *sigsel, OSFSEMVAL count)
{
  register struct _pcb *pcb;
  register const SIGSELECT *wanted;
  register struct _signal *prevbuf;
  register struct _signal *thisbuf;
  register SIGSELECT all_wanted;
  SIGSELECT sig_no = 0; /* Kill "may be used uninitialized" warning. */
  
#if (OSE_DEBUG + MEMORY_MODEL) > 0
  /* Retry here if a received signal was killed by the debugger. */
 retry:
#endif

  /* Get important pointers to registers. */
  pcb = zzcur_p;
  wanted = sigsel;
      
  /* Create a flag that says if we need to  check the sigsel array. */
  all_wanted = !*wanted;
      
  /* Give list of wanted signals to the debugger. */
  pcb->wanted = wanted;
      
  /* Disable interrupts. */
  LOCK();

  /* Start by decreasing the semaphore value. */
  pcb->fsemval -= count;
  
  /* Restart search here if the queue was flushed. */
 restart_receive:
  
  /* Check the fast semaphore first. */
  if(pcb->fsemval >= 0)
    {
      UNLOCK();
      return OS_RCV_FSEM;
    }
        
  /* Wait for first buffer to arrive. */
  prevbuf = pcb->mailbox;
  if(prevbuf)
    {
      /*
      ** We need to read out the signal number from the user pool
      ** with interrupts disabled, otherwise the user pool might
      ** be reclaimed and thus the memory will become unmapped,
      ** causing the access to the signal number to crash.
      */
      if (!all_wanted)
	sig_no = *(SIGSELECT *)(adm_to_sig(prevbuf));

      /* Buffer present. Check it. */
      UNLOCK();
      if(all_wanted || iswanted(sig_no, wanted))
	{
	  /* If wanted, unlink and return the signal unless flushed. */
	  LOCK();
	  if(pcb->flushed)
	    {
	      /* Clear flushed flag and restart if flushed. */
	      pcb->flushed = (OSBOOLEAN) 0;
	      goto restart_receive;
	    }
	  pcb->mailbox = prevbuf->next;
	  pcb->sig_cnt_queued--;

	  /* Set new signal owner. */
	  prevbuf->owner = pcb_to_pindex(pcb); /* Note! Owner is 16-bits. @frbr@ */

	  /* Increment owned counter in callers pcb. */
	  pcb->sig_cnt_owned++;

          /* We will return a signal or retry, undo the semaphore decrease. */
          pcb->fsemval += count;
          
	  UNLOCK();

	  /* Trace the signal. */
#if OSE_DEBUG > 0
	  if(zzuu.debugger_enabled)
	    {
	      register struct _pcb *from_pcb;
	      /* Lock the dispatcher. */
	      LOCK_SWAP();
	      /* Check that the sender is still alive. */
	      from_pcb = pid_to_pcb(prevbuf->sender_id);
	      if(pcb_to_pid(from_pcb) == prevbuf->sender_id)
		{
		  /* check for trace receive request. */
		  if((pcb->debug_flags | from_pcb->debug_flags) & 0x02)
		    {
		      /* We must check if the signal was killed by the debugger. */
		      if(!zzreceive_dbghook((OSBUFSIZE) prevbuf->size,
					    adm_to_sig(prevbuf),
					    pcb_to_debug_pcb(from_pcb),
					    pcb_to_debug_pcb(pcb)))
			{
			  /* Retry receive if the ldm killed the signal. */
			  UNLOCK_SWAP();
			  goto retry;
			}
		    }
		}
	      /* Unlock the dispatcher. */
	      UNLOCK_SWAP();
	    }
#endif

	  /* Copy the signal to the callers segment if required. */
#if MEMORY_MODEL == 0
	  return adm_to_sig(prevbuf);
#else
	  {
	    union SIGNAL *nsig;
	    /* Check if intersegment copying is enabled. */
	    if(!local_sysctlblk.allow_pool_copy ||
	       prevbuf->buffer_head->poolcbp->no_copy)
	      /* Done otherwise. */
	      return adm_to_sig(prevbuf);
	    /*
	    ** Check if the segment of the received signal is the one
	    ** in which the callers pool resides.
	    */
	    if(zzcur_p->cursigpool->segid == prevbuf->buffer_head->segid)
	      /* Done if they are the same. */
	      return adm_to_sig(prevbuf);
	    /* Copy signal contents to a new buffer. */
	    nsig = rec_sig_dup(prevbuf);
	    if(!nsig)
	      /* Restart the receive operation if the copy failed. */
	      goto retry;
	    /* Return the new buffer. */
	    return nsig;
	  }
#endif
	}
      else
	{
	  /* If not wanted, check next signal in the queue. */
	  LOCK();
	  thisbuf = prevbuf->next;
	}
    }
  else
    {
      /* No buffer present or semaphore ready. Swap while waiting. */
      /* Done if a timeout occurred. */
      if(pcb->status & ST_TIMEOUT)
	{
          /* Undo the semaphore decrease. */
          pcb->fsemval += count;          
	  UNLOCK();
	  return NIL;
	}
      pcb->status = (pr_state_t) ST_RECEIVE_FSEM;
      make_mypri_not_ready(pcb);
      goto restart_receive;
    }
  
  /* Check second and following buffers. (Ints disabled.) */
  for(;;)
    {
      /* Clear flushed flag and restart if flushed. */
      if(pcb->flushed)
	{
	  pcb->flushed = (OSBOOLEAN) 0;
	  goto restart_receive;
	}
      
      /* Is there a buffer to check? */
      if(thisbuf)
	{
	  /* Yes, check it. */

	  /*
	  ** We need to read out the signal number from the user pool
	  ** with interrupts disabled, otherwise the user pool might
	  ** be reclaimed and thus the memory will become unmapped,
	  ** causing the access to the signal number to crash.
	  */
	  sig_no = *(SIGSELECT *)(adm_to_sig(thisbuf));

          /*
	  ** We don't need to check all_wanted here since we know it
	  ** is zero.
	  */
	  UNLOCK();
	  if(iswanted(sig_no, wanted))
	    {
	      /* If wanted, unlink and return the signal unless flushed. */
	      LOCK();
	      if(pcb->flushed)
		{
		  /* Clear flushed flag and restart if flushed. */
		  pcb->flushed = (OSBOOLEAN) 0;
		  goto restart_receive;
		}
              prevbuf->next = thisbuf->next;
	      if(!prevbuf->next)
		{
		  pcb->mailtail = prevbuf;
		}
	      pcb->sig_cnt_queued--;

	      /* Set new signal owner. */
	      thisbuf->owner = pcb_to_pindex(pcb); /* Note! Owner is 16-bits. @frbr@ */
	      /* Increment owned counter in callers pcb. */
	      pcb->sig_cnt_owned++;

              /* We will return a signal or retry, undo the semaphore decrease. */
              pcb->fsemval += count;
              
	      UNLOCK();

	      /* Trace the signal. */
#if OSE_DEBUG > 0
	      if(zzuu.debugger_enabled)
		{
		  register struct _pcb *from_pcb;
		  /* Lock the dispatcher. */
		  LOCK_SWAP();
		  /* Check that the sender is still alive. */
		  from_pcb = pid_to_pcb(thisbuf->sender_id);
		  if(pcb_to_pid(from_pcb) == thisbuf->sender_id)
		    {
		      /* check for trace receive request. */
		      if((pcb->debug_flags | from_pcb->debug_flags) & 0x02)
			{
			  /* We must check if the signal was killed by the debugger. */
			  if(!zzreceive_dbghook((OSBUFSIZE) thisbuf->size,
						adm_to_sig(thisbuf),
						pcb_to_debug_pcb(from_pcb),
						pcb_to_debug_pcb(pcb)))
			    {
			      /* Retry receive if the ldm killed the signal. */
			      UNLOCK_SWAP();
			      goto retry;
			    }
			}
		    }
		  /* Unlock the dispatcher. */
		  UNLOCK_SWAP();
		}
#endif
	      /* Copy the signal to the callers segment if required. */
#if MEMORY_MODEL == 0
	      return adm_to_sig(thisbuf);
#else
	      {
		union SIGNAL *nsig;
		/* Check if intersegment copying is enabled. */
		if(!local_sysctlblk.allow_pool_copy ||
		   thisbuf->buffer_head->poolcbp->no_copy)
		  /* Done otherwise. */
		  return adm_to_sig(thisbuf);
		/*
		** Check if the segment of the received signal is the one
		** in which the callers pool resides.
		*/
		if(zzcur_p->cursigpool->segid == thisbuf->buffer_head->segid)
		  /* Done if they are the same. */
		  return adm_to_sig(thisbuf);
		/* Copy signal contents to a new buffer. */
		nsig = rec_sig_dup(thisbuf);
		if(!nsig)
		  /* Restart the receive operation if the copy failed. */
		  goto retry;
		/* Return the new buffer. */
		return nsig;
	      }
#endif
	    }
	  else
	    {
	      /* If not wanted, check next signal in the queue. */
	      prevbuf = thisbuf;
	      LOCK();
	      thisbuf = prevbuf->next;
	    }
	}
      else
	{
          /* No signal buffer present, check the fast semaphore. */
          if(pcb->fsemval >= 0)
            {
              UNLOCK();
              return OS_RCV_FSEM;
            }
          
	  /* No, wait for next buffer to arrive, or the fast semaphore. */
	  /* Done if a timeout occurred. */
	  if(pcb->status & ST_TIMEOUT)
	    {
              /* Undo the semaphore decrease. */
              pcb->fsemval += count;
	      UNLOCK();
	      return NIL;
	    }
	  pcb->status = (pr_state_t) ST_RECEIVE_FSEM;
	  make_mypri_not_ready(pcb);
	  thisbuf = prevbuf->next;
	}
    }
}

/*
** Subfunction to the receive_sport() system call.
**
** This function creates and OS_ATTACH_SIGNAL and sets the sender field
** to the provided pid.
*/

static union SIGNAL *
make_os_attach_sig(PROCESS pid)
{
  union SIGNAL *sig;
  struct _signal *sigadm;

  sig = alloc(sizeof(SIGSELECT), OS_ATTACH_SIG);
  sigadm = sig_to_adm(sig);
  sigadm->sender_id = pid;
  return sig;
}

/*
** Subfunction to the receive_sport() system call.
**
** This function waits for a signal to appear in any of the
** currently selected s-port phantoms.
** NIL is returned if a timeout occurs.
*/

static union SIGNAL *
receive_20(SIGSELECT *sigsel)
{
  register struct _pcb *pcb;
  register struct _pcb *mypcb;
  register SIGSELECT *wanted;
  register struct _signal *prevbuf;
  register struct _signal *thisbuf;
  
  struct S_node *s_node;
  struct S_nodelist *s_nodelist;
  
  SIGSELECT all_wanted;

#if (OSE_DEBUG + MEMORY_MODEL) > 0
  /* Retry here if a received signal was killed by the debugger. */
 retry:
#endif

  /* Get important pointers to registers. */
  mypcb = zzcur_p;
  wanted = sigsel;
  
  /* Give list of wanted signals to the debugger. */
  mypcb->wanted = wanted;
  
  /* Create a flag that says if we need to check the sigsel array. */
  all_wanted = !*wanted;
      
  /* Pick up the list of s-ports to scan. */
  s_nodelist = (struct S_nodelist *) mypcb->s_pcb.s_nodelist;

  /* Disable interrupts. */
#ifdef OSE_SMP
  /*
  ** Obey the SMP lock protocol: Don't LOCK_SWAP() with locked BKL locked.
  */
  LOCK_SWAP();
  BKL_LOCK();
#else
  LOCK();
#endif


  /* Loop until timeout or a signal found. */
  for(;;)
    {
      register U32 count;
      /* Restart search here if a queue changed while traversing it. */
    restart_receive:
      /* Look for changes in any of the input queues. */
      for(count = s_nodelist->curcount, s_node = s_nodelist->s_node;
	  count; count--, s_node++)
	{
	  register PCB_INSTANCE_T instance;
	  register U32 take_listmodnumber;
	  /* Skip inactive s-nodes. */
	  pcb = s_node->phantom_pcb;
	  if(!pcb)
	    {
	      /*
	      ** An sport died. Check if we should return an OS_ATTACH_SIG.
	      */
	      if(all_wanted || iswanted(OS_ATTACH_SIG, wanted))
		{
#ifdef OSE_SMP
		  BKL_UNLOCK_KEEP_INT_MASK();
		  LOCKED_UNLOCK_SWAP();
#else
		  UNLOCK();
#endif
		  return make_os_attach_sig(s_node->phantom_pid);
		}
	      /* No, just continue the search loop. */
	      continue;
	    }
	  /* Skip nodes with no new signals in the queue. */
	  if(s_node->add_listmodnumber == pcb->s_pcb.add_listmodnumber)
	    continue;
	  /* Skip nodes with an empty signal queue. */
	  if(!pcb->mailbox)
	    {
	      /* Update the new signal arrived counter. */
	      s_node->add_listmodnumber = pcb->s_pcb.add_listmodnumber;
	      continue;
	    }
	  /* Pick up the instance number of the process so we can see if it dies. */
	  instance = pcb_to_instance(pcb);
	  /* Pick up the take_counter so we can see if the queue changed. */
	  take_listmodnumber = pcb->s_pcb.take_listmodnumber;
	  /* Scan the queue for a new signal. */
	  for(prevbuf = thisbuf = pcb->mailbox; thisbuf; prevbuf = thisbuf, thisbuf = thisbuf->next)
	    {
	      int was_wanted = all_wanted;

	      if(!all_wanted)
		{
		  /*
		  ** We need to read out the signal number from the user pool
		  ** with interrupts disabled, otherwise the user pool might
		  ** be reclaimed and thus the memory will become unmapped,
		  ** causing the access to the signal number to crash.
		  */
		  SIGSELECT sig_no = *(SIGSELECT *)(adm_to_sig(thisbuf));

		  /* Enable interrupts. */
#ifdef OSE_SMP
		  BKL_UNLOCK_KEEP_INT_MASK();
		  LOCKED_UNLOCK_SWAP();
#else
		  UNLOCK();
#endif

		  /* Check if the buffer is wanted. */
		  was_wanted = iswanted(sig_no, wanted);

		  /* Disable interrupts. */
#ifdef OSE_SMP
		  LOCK_SWAP();
		  BKL_LOCK();
#else
		  LOCK();
#endif
		  /* Did someone take a signal from the queue while we looked at it? */
		  if(take_listmodnumber != pcb->s_pcb.take_listmodnumber)
		    /* Yes, abort search and restart from the top. */
		    goto restart_receive;
		  /* Did the phantom process die? */
		  if(instance != pcb_to_instance(pcb))
		    /* Yes, abort search and restart from the top. */
		    goto restart_receive;
		}

	      /* Did we want the buffer? */
	      if(was_wanted)
		{
		  /* Yes, unlink it from the signal queue. */
		  if(thisbuf == prevbuf)
		    pcb->mailbox = thisbuf->next;
		  else
		    prevbuf->next = thisbuf->next;
		  if(!thisbuf->next)
		    pcb->mailtail = prevbuf;
		  pcb->sig_cnt_queued--;

		  /* Bump the take counter in the pcb. */
		  pcb->s_pcb.take_listmodnumber++;
		  /* Set new signal owner. */
		  thisbuf->owner = pcb_to_pindex(mypcb); /* Note! Owner is 16-bits. @frbr@ */
		  /* Increment owned counter in callers pcb. */
		  mypcb->sig_cnt_owned++;

		  /* Enable interrupts. */
#if OSE_DEBUG == 0
#ifdef OSE_SMP
		  BKL_UNLOCK_KEEP_INT_MASK();
		  LOCKED_UNLOCK_SWAP();
#else
		  UNLOCK();
#endif
#endif
		  /* Enable interrupts if the debugger is off, otherwise lock the dispatcher. */
#if OSE_DEBUG > 0
		  if(!zzuu.debugger_enabled)
		    {
#ifdef OSE_SMP
		      BKL_UNLOCK_KEEP_INT_MASK();
		      LOCKED_UNLOCK_SWAP();
#else
		      UNLOCK();
#endif
		    }
		  else
		    {
		      register struct _pcb *from_pcb;
		      /* Lock the dispatcher and enable interrupts. */
#ifdef OSE_SMP
		      BKL_UNLOCK();
#else
		      LOCK_SWAP();
		      UNLOCK();
#endif
		      /* Check that the sender is still alive. */
		      from_pcb = pid_to_pcb(thisbuf->sender_id);
		      if(pcb_to_pid(from_pcb) == thisbuf->sender_id)
			{
			  /* check for trace receive request. */
			  if((pcb->debug_flags | from_pcb->debug_flags) & 0x02)
			    {
			      /* We must check if the signal was killed by the debugger. */
			      if(!zzreceive_dbghook((OSBUFSIZE) thisbuf->size,
						    adm_to_sig(thisbuf),
						    pcb_to_debug_pcb(from_pcb),
						    pcb_to_debug_pcb(pcb)))
				{
				  /* Retry receive if the ldm killed the signal. */
				  UNLOCK_SWAP();
				  goto retry;
				}
			    }
			}
		      /* Unlock the dispatcher. */
		      UNLOCK_SWAP();
		    }
#endif
		  /* Copy the signal to the callers segment if required. */
#if MEMORY_MODEL == 0
		  return adm_to_sig(thisbuf);
#else
		  {
		    union SIGNAL *nsig;
		    /* Check if intersegment copying is enabled. */
		    if(!local_sysctlblk.allow_pool_copy ||
		       thisbuf->buffer_head->poolcbp->no_copy)
		      /* Done otherwise. */
		      return adm_to_sig(thisbuf);
		    /*
		    ** Check if the segment of the received signal is the one
		    ** in which the callers pool resides.
		    */
		    if(zzcur_p->cursigpool->segid == thisbuf->buffer_head->segid)
		      /* Done if they are the same. */
		      return adm_to_sig(thisbuf);
		    /* Copy signal contents to a new buffer. */
		    nsig = rec_sig_dup(thisbuf);
		    if(!nsig)
		      /* Restart the receive operation if the copy failed. */
		      goto retry;
		    /* Return the new buffer. */
		    return nsig;
		  }
#endif
		}
	    }
	  
	  /* Update the new signal arrived counter. */
	  s_node->add_listmodnumber = pcb->s_pcb.add_listmodnumber;
	  /*
	  ** Restart the search from the top as we enabled interrupts
	  ** and therefore don't know the status of all queues.
	  */
	  goto restart_receive;
	}      
      /* No new buffer found. Check for timeout expired. */
      if(mypcb->status & ST_TIMEOUT)
	{
#ifdef OSE_SMP
	  BKL_UNLOCK_KEEP_INT_MASK();
	  LOCKED_UNLOCK_SWAP();
#else
	  UNLOCK();
#endif
	  return NIL;
	}
      /* Timeout not yet expired. Swap while waiting for the next signal. */
#ifdef OSE_SMP
      BKL_UNLOCK_KEEP_INT_MASK();
      SILENT_UNLOCK_SWAP();
#endif
      mypcb->status = (pr_state_t) ST_RECEIVE;
      make_mypri_not_ready(mypcb);

#ifdef OSE_SMP
      /*
      ** Obey the SMP lock protocol: Don't LOCK_SWAP() with locked BKL locked.
      */
      UNLOCK();
      LOCK_SWAP();
      BKL_LOCK();
#endif
    }
}

/*
** Subfunction to send() and send_w_s() and zzsys_send_w_s().
**
** This function follows a redirection table as long as possible.
** If a redirection is made, the addressee field in the signal is set.
** The function returns a pointer to the new addressee pcb.
**
** We should kill all processes in the redirection chain if the last process is dead??
** How do we do this for calling interrupt processes?
*/

static noinline struct _pcb *
non_simple_redir(struct _signal *sigadm,
		 struct _pcb *destpcb,
		 struct _pcb *senderpcb)
{
  register SIGSELECT signo = *((SIGSELECT *)(adm_to_sig(sigadm)));
  register struct _pcb *from = senderpcb;
  register struct _pcb *to = destpcb;
  register OSUSER sender_uid = from ? from->user : 0;
  register struct OS_redir_entry *redir;
  register int entries;
  register PROCESS defaultp;
  register PROCESS cur_def;
  int hops;

  /*
  ** Loop until target found or redirection loop detected.
  ** We allow only 10 hops for now, but this might be configurable
  ** in the future.??
  */
  for(hops = 0; hops < 10; hops++)
    {
      /* Scan the redirection table for a new target. */
      redir = to->redir;
      cur_def = to->pid;
      defaultp = redir->pid;
      entries = (int) redir->sig - 1;
      redir++;

      /* Scan all entries in the current redirection table. */
      for(;entries; entries--, redir++)
	{
	  /* see if signal number matches. */
	  if(redir->sig == signo)
	    {
	      defaultp = redir->pid;
	      break; /* Drop scan loop and search next table if present. */
	    }
	}

      /* Check for redirection loop. */
      from = to;
      to = pid_to_pcb(defaultp);
      if(to == senderpcb)
	break;

      /* Check that the target is still alive. */
      /* Shouldn't the OS kill the phantoms if there is a problem here?? */
      if(pcb_to_pid(to) != defaultp)
	return NULL;

      /* Check that the target is user compatible with the caller. */
      if(sender_uid)
	{
	  if(from && to->block != from->block && to->real_user != from->user)
	    return NULL;
	  sender_uid = to->user;
	}

      /* Set addressee field in the signal buffer. */
      if(!sigadm->dest_id)
	sigadm->dest_id = pcb_to_pid(destpcb);

      /*
      ** We have reached the final destination if the table points to
      ** the process to which the table is attached.
      */
      if(defaultp == cur_def)
	return to;

      /* Check for more redirection tables. Done if none. */
      if(!to->redir)
	{
	  /* Return final process id. */
	  return to;
	}
    }

  /* Detected a redirection loop or a user incompatibility error. */
  sigadm->dest_id = (PROCESS) 0; /* The buffer should be non-redirected. */
  return destpcb;
}

/*
** Like send_redir() but can only handle the simplest redir table, i.e:
** {1, default_pid}
*/
static always_inline struct _pcb *
simple_redir(struct _signal *sigadm,
	     struct _pcb *destpcb,
	     struct _pcb *senderpcb)
{
  struct OS_redir_entry *redir = destpcb->redir;

  if(likely(redir->sig == 1))
    {
      PROCESS destpid = redir->pid;
      struct _pcb *defaultp = pid_to_pcb(destpid);

      if(likely(defaultp != senderpcb &&
		pcb_to_instance(defaultp) == pid_to_instance(destpid) &&
		defaultp->redir == NULL))
	{
	  sigadm->dest_id = destpcb->pid;
	  return defaultp;
	}
    }
  return NULL;
}

static always_inline struct _pcb *
send_redir(struct _signal *sigadm,
	   struct _pcb *destpcb,
	   struct _pcb *senderpcb)
{
   struct _pcb *pcb = simple_redir(sigadm, destpcb, senderpcb);

   if(unlikely(pcb == NULL))
      pcb = non_simple_redir(sigadm, destpcb, senderpcb);
   return pcb;
}

/*
** Subfunction to send() and send_w_s().
**
** This function picks up the send_hook that was installed via the
** redirection table and calls it with a struct containing the signal
** pointer and some other useful meta data. It returns a non zero value
** if the hook was successfully called.
**
** There are two flavors of the send hook:
**  - Locked hook, set with proc_attr: 4.
**  - Unlocked hook, set with proc_attr: 8.
** 
** Locked hook:
** ============
** We must make the system the owner of signals that are given to the
** send hook to ensure that they are not reclaimed if the caller is
** killed. This is because the signal might not be delivered "to the
** other side" when the caller is killed. That's why we invented
** send_hook_free_buf() which can free system owned signals.  The
** problem with this solution is that if we commit suicide in the send
** hook before actually sending the signal we will have a signal left
** in the pool without any owner which will prevent us from reclaiming
** the pool. To solve this we mark signals that are given to the send
** hook, and if the pool reclaiming problem arises we should be able
** to scan for such lost signals and free them?? @frbr@
**
** Unlocked hook:
** ==============
** This flavor of the send hook does not use any locking at all, it
** also does not make the system the owner of the signal. This means
** that the send hook *must* therefore somehow consume the signal for
** instance by using send() or free_buf(). If the signal is left
** untouched by the send hook, the process calling the send hook will
** "leak" the signal, i.e. it will still be the owner of the signal
** but it may not find it since the signal pointer in user space has
** become NULL after the send system call. This mechanism was
** developed primarely for use in the ip-based fast-path project.
**
** See also: send_hook_free_buf(),send_hook_internal_free_buf(),
**           local_kill_proc(), send() and send_w_s().
*/

static int
send_hook(struct _signal *sigadm,
	  PROCESS from,
	  PROCESS to,
	  struct _pcb *to_pcb)
{
  LOCKSTATE msr = 0; /* Init to avoid compiler warning. */
  register struct _pcb *cur_pp = zzcur_p;
  struct _pcb *from_pcb;
  OSADDRESS rv;
  int status;
  struct OS_send_hook_info shi;
  OSSENDHOOK *snd_hook;
  int locked_hook = to_pcb->proc_attrs == 4 ? 1 : 0;

  /* Check if we should use locks or not. */
  if(locked_hook)
    {
      /* Disable interrupts. */
      BKL_LOCK_SAVE(msr);
    }
  
  if(from == 0)
    {
      /* Send() case: the caller is the sender. */
      from_pcb = cur_pp;
      from = from_pcb->pid;
    }
  else
    {
      /* Send_w_s() case: the processes index is reasonable. */
      if(pid_to_pindex(from) >= zzuu.next_free_pix)
	goto bad_participantes;

      from_pcb = pid_to_pcb(from);
      
      /* Check if the 'from' process is alive. */
      if(pcb_to_pid(from_pcb) != from)
	goto bad_participantes;

      /* Check that the magic number indicates a block or a process. */
      if((from_pcb->magic & 0xfffe) != MAGIC_PCB)
	goto bad_participantes;
    }

  /*
  ** We don't need to check that the 'to' processes index is reasonable.
  ** since that should be done befor this function is called.
  */

  /* Check if the 'to' process has been killed. */
  if(pcb_to_pid(to_pcb) != to)
    goto bad_participantes;
 
  /* Check that the magic number indicates a process. */
  if((to_pcb->magic) != MAGIC_PCB)
    goto bad_participantes;
  
  /* Check that the send hook has not been removed. */
  if(to_pcb->snd_hook == NULL)
    goto bad_participantes;

  /*
  ** Make sure we have a copy of the send hook address if it's removed
  ** after we enable interrupts but before we actually call the hook.
  */
  snd_hook = to_pcb->snd_hook;
    
  shi.sig_size          = sigadm->size;
  shi.sig               = adm_to_sig(sigadm);
  shi.addressee_pid     = to; /* Should this be: sigadm->dest_id @frbr@ */
  shi.sender_pid        = from;
  shi.zzreserved        = 0;  /* Reserved for id-propagation or whatever. */
  shi.addressee_lnh_hd  = to_pcb->lnh_dest_hd;
  shi.sender_lnh_hd     = from_pcb->lnh_sender_hd;
  shi.zzos_internal_sig = sigadm;

  /* If we have a non locking hook... */
  if(!locked_hook)
    {
      /* ...we must check that the 'to' process is still alive... */
      if(to_pcb->magic == MAGIC_PCB && to_pcb->pid == to)
	{
	  /*
	  ** ...if so we just call the hook...
	  ** The upper 16 bits of the version number is the signal
	  ** magic number which enables an advanced user to correctly
	  ** interpret the sigadm found in shi.zzos_internal_sig. The
	  ** lower 16-bits is the version number of the hook
	  ** itself. Current version for the unlocking type of send
	  ** hook is 0x0010.
	  */
	  (void)snd_hook(&shi, (((OSADDRESS)MAGIC_SIGNO) << 16) | 0x0010);
	  /* ...and we're done as quickly as that! */
	  return 1;
	}
      /*
      ** The caller will handle cases where the 'to' process is dead
      ** or invalid
      */
      return 0;
    }

  /* Make the system the signal owner... */
  sigadm->owner = (PCB_INDEX_T) 0;
  /*
  ** ...and mark it as "currently in use by a send hook".
  ** Maybe we should place the lower 16 bits of zzuu.ticks in the
  ** upper 16 bits of this magic value to enable us to write a "time
  ** safe" garbage collector for "lost send hook signals"?? @frbr@
  */
  sigadm->next = (void *)0x44A3;

  /* Decrement owned counter in callers pcb. */
  cur_pp->sig_cnt_owned--;
  /* Make sure we don't get killed while in the send hook. */
  cur_pp->lock_cnt++;

  /* Enable interrupts. */
  BKL_LOCK_RESTORE(msr);

  /* Trace the signal. */
#if OSE_DEBUG > 0
  if(zzuu.debugger_enabled)
    {
      /* Check for trace send request. */
      if((cur_pp->debug_flags | to_pcb->debug_flags) & 0x01)
	{
	  /* Check it with the ldm. */
	  zzisend_dbghook((OSBUFSIZE) sigadm->size,
			  adm_to_sig(sigadm),
			  pcb_to_debug_pcb(from_pcb),
			  pcb_to_debug_pcb(to_pcb));
	}
    }
#endif

  /* Call the hook.
  ** The upper 16 bits of the version number is the signal magic
  ** number which enables an advanced user to correctly interpret the
  ** sigadm found in shi.zzos_internal_sig. The lower 16-bits is the
  ** version number of the hook itself. Current version for the locking
  ** type of send hook is 0x0000.
  */
  rv = snd_hook(&shi, (((OSADDRESS)MAGIC_SIGNO) << 16) | 0x0000);

  /* Assume that everything is ok. */
  status = 1;
  BKL_LOCK_SAVE(msr);

  if(rv)
    {
      /*
      ** The send hook want us to report an error.
      ** Thus we must return the signal to it's original state.
      */
      
      /* Make sure that the signal has not already been freed. */
      if(((OSADDRESS)(sigadm->next) & 0xFFFF) == 0x44A3)
	{
	  sigadm->owner = pcb_to_pindex(cur_pp); /* Note! Owner is 16-bits. @frbr@ */
	  cur_pp->sig_cnt_owned++;
	  sigadm->next = NULL;
	  status = 0;
	}
    }
  
  if(--cur_pp->lock_cnt == 0)
    {
      if(cur_pp->locked_kill)
	{
	  BKL_LOCK_RESTORE(msr);
	  /*
	  ** Someone tried to kill us while we where locked.
	  ** Better make that someone happy...
	  ** Goodbye earth!
	  */
	  kill_proc(zzcur_p->pid);
	  /*NOTREACHED*/
	  return 0;
	}
    }
  BKL_LOCK_RESTORE(msr);
  return status;
  
 bad_participantes:
  if(locked_hook)
    {
      BKL_LOCK_RESTORE(msr);
    }
  return 0;
}

/*
** Perform final actions to perform a send from an interrupt or timer process.
*/
static void
int_send_finalize(struct _signal *sigadm, PROCESS to, struct _pcb *cur_pp)
{
  struct _pcb *pcb = pid_to_pcb(to);
  LOCKSTATE msr;

  /*
  ** Now, the receiver may already be dead, but he can neiter die or migrate
  ** from here on if he is located on the current cpu.
  */

  /* If the receiver is dead, just free the buffer. */
  if(unlikely(to != pcb_to_pid(pcb)))
    {
      safe_free_buf(sigadm);
      return;
    }
  
  sigadm->next = NULL;

  LOCK_SAVE(msr);

  /* Make system the signal owner. */
  sigadm->owner = (PCB_INDEX_T) 0;

  /* Decrement owned counter in callers pcb. */
  cur_pp->sig_cnt_owned--;

  /*
  ** If the process is on another cpu, send an event to it.
  ** The target cpu is responible for handling dying and migrated receivers.
  */
#ifdef OSE_SMP
  {
    cpuid_t target_cpu = pcb->cpu_id;
    if(target_cpu != zzkrn_cpu_id())
      {
	send_krn_event(target_cpu, KRN_EVT_SIG_SEND, to, sigadm);
	LOCK_RESTORE(msr);
	return;
      }
  }
#endif
  
  /* Link signal into mailbox of target process. */
  if(pcb->mailbox)
    {
      pcb->mailtail->next = sigadm;
      pcb->mailtail = sigadm;
    }
  else
    {
      pcb->mailbox = sigadm;
      pcb->mailtail = sigadm;
    }
  pcb->sig_cnt_queued++;
  
  /* Do process type dependent swap request. */

  if(pcb->priority < 33)
    /*
    ** The target process is a prioritized or background process.
    ** Make the process ready and maybe swap.
    */
    {
      if(pcb->status & ST_RECEIVE)
	{
	  pcb->status = (pr_state_t) (pcb->status & (~ST_RECEIVE_FSEM));
	      
	  /* If the process becomes ready then do a swap. */
	  if(pcb->status < ST_READY)
	    {
	      make_otherpri_ready_noswap(pcb); /* noswap */
	    }
	}

      /* Enable interrupts and quit. */
      LOCK_RESTORE(msr);
      return;
    }
  else if(pcb->priority < 96)
    /*
    ** The target process is an interrupt process.
    ** Request a wakeup call.
    */
    {
      OSPRIORITY priority = cur_pp->priority;

      /*
      ** If caller is a timer proc then use the timer int priority
      ** instead.
      */
      if(priority > 95)
	priority = (OSPRIORITY) (64 + zzuu.timer_int_priority);


      if(priority > pcb->priority)
	/* The caller is a lower priority int. Do wakeup call now. */
	{
	  /* Enable interrupts. */
	  LOCK_RESTORE(msr);

	  /*
	  ** If the process' is_deaf flag is set it means that we
	  ** shouldn't wake it up when sending signals to it.
	  */
	  if(!pcb->is_deaf)
	    {
	      zzenter_wakeup(pcb->uarea, pcb, pcb->priority & 31,
			     (OSBOOLEAN) 1);
	    }
	  return;
	}
      else
	/*
	** The caller is a higher or same priority int.
	** Request a later wakeup call. The is_deaf flag is
	** checked inside request_int_sigwakeup().
	*/
	{
	  request_int_sigwakeup(pcb);
	  LOCK_RESTORE(msr);
	  return;
	}
    }
  else
    /*
    ** The target process is a timer process.
    ** Request a wakeup call.
    */
    {
      OSPRIORITY priority = cur_pp->priority;

      if(priority > 95)
	/* The caller is a timer proc then request later wakeup call. */
	{
	  /*
	  ** Note that the is_deaf flag is checked inside
	  ** request_timer_sigwakeup().
	  */
	  request_timer_sigwakeup(pcb);
	  LOCK_RESTORE(msr);
	  return;
	}
      else if((priority & 31) > zzuu.timer_int_priority)
	/*
	** The caller has a priority lower than timer int priority.
	** Do wakeup call now.
	*/
	{
	  /* Enable interrupts. */
	  LOCK_RESTORE(msr);

	  /*
	  ** If the process' is_deaf flag is set it means that we
	  ** shouldn't wake it up when sending signals to it.
	  */
	  if(!pcb->is_deaf)
	    {
	      zzenter_wakeup(pcb->uarea, pcb, zzuu.timer_int_priority,
			     (OSBOOLEAN) 1);
	    }
	  return;
	}
      else
	/*
	** The caller has a higher or same priority as timer int.
	** Request later call. The is_deaf flag is checked
	** inside request_timer_sigwakeup().
	*/
	{
	  request_timer_sigwakeup(pcb);
	  LOCK_RESTORE(msr);
	  return;
	}
    }
}

/*
** Perform final actions to perform a send from a prioritized or background 
** process.
**
** The two discard arguments are used by the attach concept. If the sending
** of an attach signal is interrupted by an interrupt process doing detach,
** the discard function reports that the signal shall be freed instead of
** sent.
**
** The dispatcher must be locked by the caller. 
** The dispatcher is unlocked at return iff leave_dispatcher_locked is zero.
*/
static void 
locked_send_finalize(struct _signal *sigadm,
		     struct _pcb *pcb,
		     struct _pcb *cur_pp,
		     OSBOOLEAN leave_dispatcher_locked,
		     OSBOOLEAN (*discard_function)(struct _signal *, void *),
		     void *discard_arg)
{
#ifdef OSE_SMP
  cpuid_t target_cpu;
  PROCESS pid = pcb_to_pid(pcb);
#endif

  /* Disable interrupts. */
  LOCK();

  /* Unlock the dispatcher, but don't swap now. */
  if(!leave_dispatcher_locked)
    SILENT_UNLOCK_SWAP();

  /* If the signal should be discarded, free it and return. */
  if(unlikely(discard_function && discard_function(sigadm, discard_arg)))
    {
      if(leave_dispatcher_locked)
	UNLOCK();
      else
	CHECK_UNLOCK_SWAP();

      safe_free_buf(sigadm);
      return;
    }

  /*
  ** If the process is on another cpu, send an event to it.
  ** The target cpu is responible for handling dying and migrated receivers.
  */
#ifdef OSE_SMP
  target_cpu = pcb->cpu_id;
  if(target_cpu != zzkrn_cpu_id())
    {
      struct poolcb *dest_pool = pcb->block->sigpool;

      /*
      ** If the pool extender has registered a handle to a new
      ** fragment, we want to make sure there is a new fragment in
      ** place if we have a priority below the memory manager.
      ** Otherwise we spin until there is. If we do not do this, a low
      ** priority process on any CPU (not hosting the memory manager)
      ** could call send() in a loop and thereby make a receiver in
      ** another segment consume all of the memory in a fresh fragment
      ** before the memory manager gets the chance to allocate a new
      ** fragment.
      ** We only need to do all this if the target process runs on cpu0,
      ** where it can starve out the extender and mm.
      */
      if(target_cpu == 0 && dest_pool->ext_frag_pp != NULL)
	{
	  OSPRIORITY my_pri = cur_pp->priority;

	  /*
	  ** Check if I'm a bg/pri-proc with a lower pri than the memory
	  ** manager.
	  */
	  if(my_pri <= 32 && my_pri > dest_pool->ext_frag_prio)
	    {
	      /*
	      ** Check if the signal will trig an alloc on the
	      ** receiving side.
	      */
	      if(dest_pool->segid != sigadm->buffer_head->poolcbp->segid)
		{
		  void * volatile * fpp = dest_pool->ext_frag_pp;
		  PCB_INSTANCE_T instance = pcb_to_instance(pcb);
		  
		  /*
		  ** Spin until a new fragment is found or someone
		  ** deregister the fragment handle. Also stop
		  ** spinning if the addressee is killed or migrated.
		  */
		  while(fpp != NULL && *fpp == NULL)
		    {
		      volatile struct _pcb *vpcb = pcb;
			
		      /* Unlock to receive kernel events while spinning. */
		      UNLOCK();
		      LOCK();

		      target_cpu = vpcb->cpu_id;
		      if(pcb_to_instance(vpcb) != instance)
			{
			  /* The addressee has died. */
			  if(leave_dispatcher_locked)
			    UNLOCK();
			  else
			    CHECK_UNLOCK_SWAP();
			  safe_free_buf(sigadm);
			  return;
			}

		      if(target_cpu != 0)
			{
			  /* The addressee has migrated from cpu 0. */
			  break;
			}
		      fpp = dest_pool->ext_frag_pp;
		    }

		  /* Make sure we are still the owner of the signal. */
		  if(sigadm->owner != pcb_to_pindex(cur_pp))
		    {
		      if(leave_dispatcher_locked)
			UNLOCK();
		      else
			CHECK_UNLOCK_SWAP();
		      return;
		    }
		}
	    }
	}

	/* Current and/or the adressee process might have migrated now. */
      if(target_cpu != zzkrn_cpu_id())
	{
	  /* Make system the signal owner. */
	  sigadm->owner = (PCB_INDEX_T) 0;
	  sigadm->next = NULL;
      
	  /* Decrement owned counter in callers pcb. */
	  cur_pp->sig_cnt_owned--;
      
	  send_krn_event(target_cpu, KRN_EVT_SIG_SEND, pid, sigadm);

	  if(leave_dispatcher_locked)
	    UNLOCK();
	  else
	    CHECK_UNLOCK_SWAP();
	  return;
	}
    }
#endif

  /* Make system the signal owner. */
  sigadm->owner = (PCB_INDEX_T) 0;
  sigadm->next = NULL;

  /* Decrement owned counter in callers pcb. */
  cur_pp->sig_cnt_owned--;

  /* Link signal into mailbox of target process. */
  if(pcb->mailbox)
    {
      pcb->mailtail->next = sigadm;
      pcb->mailtail = sigadm;
    }
  else
    {
      pcb->mailbox = sigadm;
      pcb->mailtail = sigadm;
    }
  pcb->sig_cnt_queued++;

  /* Do process type dependent swap. */

  if(pcb->priority < 33)
  /*
  ** The target process is a prioritized or background process.
  **  Make the process ready and maybe swap.
  */
    {
      if(pcb->status & ST_RECEIVE)
	{
	  pcb->status = (pr_state_t) (pcb->status & (~ST_RECEIVE_FSEM));
	      
	  /* If the process becomes ready then do a swap. */
	  if(pcb->status < ST_READY)
	    {
	      make_otherpri_ready_noswap(pcb); /* noswap */
	    }
	}

      /* Enable interrupts and quit. */
      if(leave_dispatcher_locked)
	UNLOCK();
      else
	CHECK_UNLOCK_SWAP();
      return;
    }
  else if(pcb->priority < 96)
  /*
  ** The target process is an interrupt process.
  ** Request a wakeup call.
  */
    {
      /*
      ** If the process' is_deaf flag is set it means that we
      ** shouldn't wake it up when sending signals to it.
      */
      if(!pcb->is_deaf)
	{
	  zzenter_wakeup(pcb->uarea, pcb, pcb->priority & 31,
			 (OSBOOLEAN) 1);
	  /*
	  ** We don't need to do a CHECK_UNLOCK_SWAP() here since
	  ** The invoked interrupt process will do this job in
	  ** zzinit_int().
	  */
	  UNLOCK();
	  return;
	}
	  
      /* Enable interrupts and quit. */
      if(leave_dispatcher_locked)
	UNLOCK();
      else
	CHECK_UNLOCK_SWAP();
      return;
    }
  else
  /*
  ** The target process is a timer process.
  ** Request a wakeup call.
  */
    {
      /*
      ** If the process' is_deaf flag is set it means that we
      ** shouldn't wake it up when sending signals to it.
      */
      if(!pcb->is_deaf)
	{
	  zzenter_wakeup(pcb->uarea, pcb, zzuu.timer_int_priority,
			 (OSBOOLEAN) 1);
	  /*
	  ** We don't need to do a CHECK_UNLOCK_SWAP() here since
	  ** The invoked interrupt process will do this job in
	  ** zzinit_int().
	  */
	  UNLOCK();
	  return;
	}

      /* Enable interrupts and quit. */
      if(leave_dispatcher_locked)
	UNLOCK();
      else
	CHECK_UNLOCK_SWAP();
      return;
    }
}

/*
** Perform final actions to perform a send from a prioritized or background 
** process.
**
** This function must NOT be called with a locked dispatcher.
** 
*/
static void 
send_finalize(struct _signal *sigadm, PROCESS to)
{
  struct _pcb *pcb = pid_to_pcb(to);

  /* Disable interrupts. */
  LOCK();

  /*
  ** Now, the receiver may already be dead, but he can neither die or migrate
  ** from here on if he is located on the current cpu.
  */
  
  /* If the receiver is dead, just free the buffer. */
  if(unlikely(to != pcb_to_pid(pcb)))
    {
      UNLOCK();
      safe_free_buf(sigadm);
      return;
    }

  /*
  ** If the process is on another cpu, send an event to it.
  ** The target cpu is responible for handling dying and migrated receivers.
  */
#ifdef OSE_SMP
  {
    cpuid_t target_cpu = pcb->cpu_id;
    if(target_cpu != zzkrn_cpu_id())
      {
	struct poolcb *dest_pool = pcb->block->sigpool;
	struct _pcb *cur_pp = zzcur_p;

	/*
	** If the pool extender has registered a handle to a new
	** fragment, we want to make sure there is a new fragment in
	** place if we have a priority below the memory manager.
	** Otherwise we spin until there is. If we do not do this, a low
	** priority process on any CPU (not hosting the memory manager)
	** could call send() i a loop and thereby make a receiver in
	** another segment consume all of the memory in a fresh fragment
	** before the memory manager gets the chance to allocate a new
	** fragment.
	** We only need to do all this if the target process runs on cpu0,
	** where it can starve out the extender and mm.
	*/
	if(target_cpu == 0 && dest_pool->ext_frag_pp != NULL)
	  {
	    OSPRIORITY my_pri = cur_pp->priority;

	    /*
	    ** Check if I'm a bg/pri-proc with a lower pri than the
	    ** memory manager.
	    */
	    if(my_pri <= 32 && my_pri > dest_pool->ext_frag_prio)
	      {
		/*
		** Check if the signal will trig an alloc on the
		** receiving side.
		*/
		if(dest_pool->segid != sigadm->buffer_head->poolcbp->segid)
		  {
		    void * volatile * fpp = dest_pool->ext_frag_pp;
		    PCB_INSTANCE_T instance = pcb_to_instance(pcb);
		    
		    /*
		    ** Spin until a new fragment is found or someone
		    ** deregister the fragment handle. Also stop
		    ** spinning if the addressee is killed or migrated.
		    */
		    while(fpp != NULL && *fpp == NULL)
		      {
			volatile struct _pcb *vpcb = pcb;
			
			/* Unlock to receive kernel events while spinning. */
			UNLOCK();
			LOCK();

			target_cpu = vpcb->cpu_id;
			if(pcb_to_instance(vpcb) != instance)
			  {
			    /* The addressee has died. */
			    UNLOCK();
			    safe_free_buf(sigadm);
			    return;
			  }

			if(target_cpu != 0)
			  {
			    /* The addressee has migrated from cpu 0. */
			    break;
			  }
			fpp = dest_pool->ext_frag_pp;
		      }

		    /* Make sure we are still the owner of the signal. */
		    if(sigadm->owner != pcb_to_pindex(cur_pp))
		      {
			UNLOCK();
			return;
		      }
		  }
	      }
	  }

	/* Current and/or the adressee process might have migrated now. */
	if(target_cpu != zzkrn_cpu_id())
	  {
	    /* Make system the signal owner. */
	    sigadm->owner = (PCB_INDEX_T) 0;
	    sigadm->next = NULL;
      
	    /* Decrement owned counter in callers pcb. */
	    cur_pp->sig_cnt_owned--;

	    send_krn_event(target_cpu, KRN_EVT_SIG_SEND, to, sigadm);

	    UNLOCK();
	    return;
	  }
      }
  }
#endif

  /* Make system the signal owner. */
  sigadm->owner = (PCB_INDEX_T) 0;
  sigadm->next = NULL;

  /* Decrement owned counter in callers pcb. */
  zzcur_p->sig_cnt_owned--;

  /* Link signal into mailbox of target process. */
  if(pcb->mailbox)
    {
      pcb->mailtail->next = sigadm;
      pcb->mailtail = sigadm;
    }
  else
    {
      pcb->mailbox = sigadm;
      pcb->mailtail = sigadm;
    }
  pcb->sig_cnt_queued++;

  /* Do process type dependent swap. */

  if(pcb->priority < 33)
  /*
  ** The target process is a prioritized or background process.
  ** Make the process ready and maybe swap.
  */
    {
      if(pcb->status & ST_RECEIVE)
	{
	  pcb->status = (pr_state_t) (pcb->status & (~ST_RECEIVE_FSEM));
	      
	  /* If the process becomes ready then do a swap. */
	  if(pcb->status < ST_READY)
	    {
	      make_otherpri_ready(pcb); /* swap */
	    }
	}
    }
  else if(pcb->priority < 96)
  /*
  ** The target process is an interrupt process.
  ** Request a wakeup call.
  */
    {
      /*
      ** If the process' is_deaf flag is set it means that we
      ** shouldn't wake it up when sending signals to it.
      */
      if(!pcb->is_deaf)
	{
	  zzenter_wakeup(pcb->uarea, pcb, pcb->priority & 31,
			 (OSBOOLEAN) 1);
	}
    }
  else
  /*
  ** The target process is a timer process.
  ** Request a wakeup call.
  */
    {
      /*
      ** If the process' is_deaf flag is set it means that we
      ** shouldn't wake it up when sending signals to it.
      */
      if(!pcb->is_deaf)
	{
	  zzenter_wakeup(pcb->uarea, pcb, zzuu.timer_int_priority,
			 (OSBOOLEAN) 1);
	}
    }

  /* Enable interrupts and quit. */
  UNLOCK();
  return;
}


/*
** Send_w_s for internal use.
**
** The input is a pointer to the signal administration block and not to
** the signal data area, which allows us to avoid touching user space data.
**
** The normal send_w_s() call checks that the sender is still alive
** and drops the signal if not. This is the reason why send_w_s() can
** not be used to return attached signals to owners when a process has
** been killed.
**
** The flags parameter is a bit-field that allow us to change the
** normal behaviour of this function. The bits are as follows:
**    0x00000001 = skip_redir.
**
** The "skip redir" functionality is needed when a system daemon is
** sending back a return value to a system call (like
** create_process()) where it's essential that the signal is not
** redirected.
**
** Safe_send_w_s() can be called with a locked dispatcher, and will in
** that case not change the dispatchers locked state.
**
** The caller must make sure that the buffer is owned by the current
** process and that it is thoroughly checked.
**
** This routine cannot send signals to a sport, neither directly nor
** through a redirection table. Any a attempts to do so will cause the
** signal to be silently discarded.
*/

static void
safe_send_w_s(struct _signal *sigadm, PROCESS from, PROCESS to, U32 flags,
	      OSBOOLEAN (*discard_function)(struct _signal *sigadm,
					    void *discard_arg),
	      void *discard_arg)
{
  PROCESS addressee_pid = to;
  register struct _pcb *pcb;
  register struct _pcb *cur_pp = zzcur_p;
  register OSBOOLEAN dispatcher_locked = zzdispatcher_locked;

  (void)addressee_pid;
  
  /* Set the sender and addressee fields in the signal. */
  sigadm->sender_id = from;
  sigadm->dest_id = (PROCESS) 0;

  /*
  ** If the from process isn't the current running process we don't
  ** know if it's alive.
  */
  if (from != cur_pp->pid)
    {
      /*
      ** If the from process is dead, then use the callers pid for the
      ** debugger.
      */
      switch(get_ptype(from))
	{
	case OS_ILLEGAL: /* Why is this? For live segments, or? @frbr@  */
	case OS_ZOOMBIE: /* Dead refs, i.e. for sending attach signals. */
	  from = cur_pp->pid;
	  break;
	default:
	  break;
	}
    }

  /* Locate the destination block or process descriptor. */
  pcb = pid_to_pcb(to);

  /* Run the interrupt version if the caller is an int or timer process. */
  if(cur_pp->priority > 32)
    {
      /*
      ** Read instance number of the dest process and check the process id.
      ** Report error if invalid.
      */
      if(!(check_pid_bid(to, OSE_SEND_W_S)))
	{
	  safe_free_buf(sigadm);
	  return;
	}

      /* If block descriptor then send to block process instead. */
      if(pcb->magic == MAGIC_BCB)
	{
	  if(((struct _block *) pcb)->block_process)
	    {
	      pcb = ((struct _block *) pcb)->block_process;
	      if(pcb->killed)
		{
		  /* Drop signal if no block process is present. */
		  safe_free_buf(sigadm);
		  return;
		}
	      to = addressee_pid = pcb_to_pid(pcb);
	    }
	  else
	    {
	      /* Drop signal if no block process is present. */
	      safe_free_buf(sigadm);
	      return;
	    }
	}
      
      /*
      ** Use redirection table if present.
      ** pcb is set to new target or NULL if not found.
      ** The addressee field is updated if required.
      ** We can not call send_redir with a dead from_field, so use NULL instead.
      */
      if(pcb->redir && !(flags & 0x00000001))
	{
	  pcb = send_redir(sigadm, pcb,
			   sigadm->sender_id == from
			   ? pid_to_pcb(from) : NULL);
	  if(!pcb)
	    {
	      /* Terminate system call if final addresse not found. */
	      safe_free_buf(sigadm);
	      return;
	    }
	  to = pcb_to_pid(pcb);
	}

      /* Skip operation if target is a phantom process. */
      if(pcb->priority == 128)
	{
	  /* Terminate system call if so. */
	  /*
	  ** Note that we do not check if the target is an sport.
	  ** That would cost us lots of code and we currently don't use
	  ** zzsys_send_w_s() in a context where the target can be an sport.
	  ** If we had a function like this it would probably be called
	  ** sys_send_to_sport(); We should fix this @frbr@.
	  */
	  safe_free_buf(sigadm);
	  return;
	}

      /*
      ** Check for intersegment send to interrupt process.
      ** This is a lengthy test so it should be removed whenever possible.??
      ** We consider this an illegal operation only for memory models which
      ** implement demand paging.
      */
#if ERROR_CHECKS > 0
#if MEMORY_MODEL > 1
      ON_ERRLEVEL_2()
	{
	  if(pcb->priority > 32)
	    {
	      if(pcb->block->sigpool->segid !=
		 sigadm->buffer_head->poolcbp->segid)
		{
		  os_error(0, OSE_EINTERSEGMENT_SEND | OSE_SEND_W_S, (OSERRCODE) 0);	/* Non fatal. */
		  safe_free_buf(sigadm);
		  return;
		}
	    }
	}
#endif
#endif

      /* Trace the signal. */
#if OSE_DEBUG > 0
      if(zzuu.debugger_enabled)
	{
	  register struct _pcb *to_pcb = pid_to_pcb(addressee_pid);
	  register struct _pcb *from_pcb = pid_to_pcb(from);

	  /* check for trace send request. */
	  if((from_pcb->debug_flags | to_pcb->debug_flags) & 0x01)
	    {
	      /* Check it with the ldm. */
	      zzisend_dbghook((OSBUFSIZE) sigadm->size,
			      adm_to_sig(sigadm),
			      pcb_to_debug_pcb(from_pcb),
			      pcb_to_debug_pcb(to_pcb));
	    }
	}
#endif

      /* Take buffer from user. */
      /* *sig = NIL; */

      /* Send the signal. */
      int_send_finalize(sigadm, to, cur_pp);
    }
  else
    {
      /*
      ** The caller is not an interrupt or timer process.
      **
      ** Read instance number of the dest process and check the process id.
      ** Report error if invalid.
      */
      pcb = check_pid_bid(to, OSE_SEND_W_S);
      if(pcb == NULL)
	{
	  safe_free_buf(sigadm);
	  return;
	}

      /* Lock the dispatcher. */
      if(!dispatcher_locked)
	LOCK_SWAP();

      /* Check that the dest process is still present. */
      if(pcb_to_pid(pcb) != to)
	{
	  /* Terminate system call if not. */
	  if(!dispatcher_locked)
	    UNLOCK_SWAP();
	  safe_free_buf(sigadm);
	  return;
	}
      
      /* Check that the from process is still present. */
      if((sigadm->sender_id == from) &&
	 (pcb_to_pid(pid_to_pcb(from)) != from))
	{
	  /* If the from process is dead, then use the callers pid for the debugger. */
	  from = current_process();
	}

      /* If block descriptor then send to block process instead. */
      if(pcb->magic == MAGIC_BCB)
	{
	  if(((struct _block *) pcb)->block_process)
	    {
	      pcb = ((struct _block *) pcb)->block_process;
	      if(pcb->killed)
		{
		  /* Drop signal if no block process is present. */
		  if(!dispatcher_locked)
		    UNLOCK_SWAP();
		  safe_free_buf(sigadm);
		  return;
		}
	      to = addressee_pid = pcb_to_pid(pcb);
	    }
	  else
	    {
	      /* Drop signal if no block process is present. */
	      if(!dispatcher_locked)
		UNLOCK_SWAP();
	      safe_free_buf(sigadm);
	      return;
	    }
	}

      /*
      ** Use redirection table if present.
      ** pcb is set to new target or NULL if not found.
      ** The addressee field is updated if required.
      ** We can not call send_redir with a dead from_field, so use NULL instead.
      */
      if(pcb->redir && !(flags & 0x00000001))
	{
	  pcb = send_redir(sigadm, pcb,
			   sigadm->sender_id == from
			   ? pid_to_pcb(from) : NULL);
	  if(!pcb)
	    {
	      /* Terminate system call if final addresse not found. */
	      if(!dispatcher_locked)
		UNLOCK_SWAP();
	      safe_free_buf(sigadm);
	      return;
	    }
	  to = pcb_to_pid(pcb);
	}

      /* Skip operation if target is a phantom process. */
      if(pcb->priority == 128)
	{
	  /* Terminate system call if so. */
	  /*
	  ** Note that we do not check if the target is an sport.
	  ** That would cost us lots of code and we currently don't
	  ** use zzsys_send_w_s() in a context where the target can be
	  ** an sport.  If we had a function like this it would
	  ** probably be called sys_send_to_sport(); We should fix
	  ** this @frbr@.
	  */
	  if(!dispatcher_locked)
	    UNLOCK_SWAP();
	  safe_free_buf(sigadm);
	  return;
	}

      /*
      ** Check for intersegment send to interrupt process.
      ** This is a lengthy test so it should be removed whenever possible.??
      ** We consider this an illegal operation only for memory models which
      ** implement demand paging.
      */
#if ERROR_CHECKS > 0
#if MEMORY_MODEL > 1
      ON_ERRLEVEL_2()
	{
	  if(pcb->priority > 32)
	    {
	      if(pcb->block->sigpool->segid !=
		 sigadm->buffer_head->poolcbp->segid)
		{
		  if(!dispatcher_locked)
		    UNLOCK_SWAP();
		  os_error(0, OSE_EINTERSEGMENT_SEND | OSE_SEND_W_S, (OSERRCODE) 0);	/* Non fatal. */
		  safe_free_buf(sigadm);
		  return;
		}
	    }
	}
#endif
#endif

      /* Trace the signal. Exit if it was killed by the debugger. */
      /* Is it possible to trace or catch signals sent from a killed process?? */
      /* Check all occurrences of zzsys_send_w_s for proper ownership management. ??*/
      /* What happens with name buffers etc?. */
#if OSE_DEBUG > 0
      if(zzuu.debugger_enabled)
	{
	  register struct _pcb *to_pcb = pid_to_pcb(addressee_pid);

	  /* check for trace send request. */
	  if((to_pcb->debug_flags) & 0x01)
	    {
	      /* Save instance number in case the addressee is killed during trace. */
	      PCB_INSTANCE_T instance = pcb_to_instance(pcb);

	      /* Check it with the ldm, but not if the sender is a segment. */
	      if(((pid_to_pcb(from))->magic & (~MAGIC_KILLED)) == MAGIC_PCB)
		{
		  if(!zzsend_dbghook((OSBUFSIZE) sigadm->size,
				     adm_to_sig(sigadm),
				     pcb_to_debug_pcb((pid_to_pcb(from))),
				     pcb_to_debug_pcb(to_pcb)))
		    {
		      /* Done if the ldm killed the signal. */
		      if(!dispatcher_locked)
			UNLOCK_SWAP();
		      return;
		    }
		}
	      /* Send from the owner for signals returned from a killed segment. */
	      else
		{
		  if(!zzsend_dbghook((OSBUFSIZE) sigadm->size,
				     adm_to_sig(sigadm),
				     pcb_to_debug_pcb(to_pcb),
				     pcb_to_debug_pcb(to_pcb)))
		    {
		      /* Done if the ldm killed the signal. */
		      if(!dispatcher_locked)
			UNLOCK_SWAP();
		      return;
		    }
		}
	      if(instance != pcb_to_instance(pcb))
		{
		  /* Done if the addressee was killed. */
		  if(!dispatcher_locked)
		    UNLOCK_SWAP();
		  safe_free_buf(sigadm);
		  return;
		}
	    }
	}
#endif

      /* Take buffer from user. */
      /* *sig = NIL; */

      /* Send the signal. */
      locked_send_finalize(sigadm, pcb, cur_pp, dispatcher_locked,
			   discard_function, discard_arg);
    }
}

/*
** This is like safe_send_w_s() but it takes a normal signal pointer
** and don't make any use of the behaviour changing flag-bits.
*/

void
zzsys_send_w_s(union SIGNAL **sig, PROCESS from, PROCESS to)
{
  safe_send_w_s(sig_to_adm(*sig), from, to, 0ul, NULL, NULL);
}

/*
** Helper function to receive() and receive_w_tmo() that polls the
** special ose_fsend signal queue. Is also called from the idle loop.
**
** A nonzero value is returned when called from a receive related
** system call when a new signal is placed in the calling process'
** in-queue.
**
** A nonzero value can also be returned when called from the idle
** loop, in which case it means that a re-schedule is requested.
*/

static int
receive_core_sig_queue(void)
{
  struct _pcb *pcb;
  uint32_t length, i;
  LOCKSTATE msr;
  int status = 0;
  int i_am_idle = zzww.idle_pcb == zzcur_p;

  length = i_am_idle
    ? 1
    : sizeof(zzww.msg)/sizeof(zzww.msg[0]);
  
  LOCK_SAVE(msr);

  length = handlerp.zzmsg_dequeue(zzww.msg, length);

  for (i = 0 ; i < length ; i++)
    {
      struct ose_krn_msg *msg = &zzww.msg[i];
      struct _signal *sigadm =
	(struct _signal *)(msg->data[0] & ~KRN_EVT_TINY_MASK);
      PROCESS pid = msg->data[1];

      if(unlikely((msg->data[0] & KRN_EVT_TINY_MASK) != KRN_EVT_SIG_SEND))
	{
	  os_error(2, OSE_ECORRUPTED_KERNEL, (OSERRCODE) 26);	/* Fatal. */
	}
      
      pcb = pid_to_pcb(pid);
	      
      /* Receiver dead or alive? */
      if(unlikely(pcb_to_pid(pcb) != pid ||
		  pcb->magic != MAGIC_PCB))
	{
#ifdef OSE_SMP
	  /* Receiver is dead. Let sysd_lite free the signal. */
	  adm_to_sig(sigadm)->sig_no = SMP_FREE_ME;
	  pcb = pid_to_pcb(zzww.sysd_lite_);
#else
	  LOCK_RESTORE(msr);
	  safe_free_buf(sigadm);
	  continue;
#endif
	}
#ifdef OSE_SMP
      else
	{
	  cpuid_t pcb_cpu = pcb->cpu_id;
		  
	  if(unlikely(pcb_cpu != zzkrn_cpu_id()))
	    {
	      union SIGNAL *sig = adm_to_sig(sigadm);
	       
	      /*
	      ** The process has migrated. We need to pass the signal
	      ** on to the fsend queue on the proper cpu.
	      */
	      change_sig_owner(sigadm, zzcur_p);
	      (void)ose_fsend(&sig,
			      sigadm->sender_id, /* from  */
			      pcb_to_pid(pcb),   /* to    */
			      0);                /* flags */
	      continue;
	    }
	}
#endif		
	      
      /* Insert the signal into the queue. */
      sigadm->next = NULL;
      if(pcb->mailbox)
	{
	  pcb->mailtail->next = sigadm;
	}
      else
	{
	  pcb->mailbox = sigadm;
	}
      pcb->mailtail = sigadm;
      pcb->sig_cnt_queued++;

      /* Check if the signal was sendt to the current process. */
      if(pcb == zzcur_p)
	{
	  status = 1;
	  continue;
	}

      /* Do a process type dependent swap request. */
      if(likely(pcb->priority < 33))
	{
	  /*
	  ** The target process is a prioritized or background
	  ** process. Make the process ready.
	  */
	  if(pcb->status & ST_RECEIVE)
	    {
	      pcb->status &= (pr_state_t)(~ST_RECEIVE_FSEM);
		      
	      if(pcb->status < ST_READY)
		{
		  make_otherpri_ready_noswap(pcb); /* noswap */

		  /* If called from the idle loop, go seach again. */
		  status += i_am_idle;
		}
	    }
	}
      else if(pcb->priority < 96)
	{
	  /*
	  ** The target process is an interrupt process. Request a
	  ** later wakeup call.
	  */
	  request_int_sigwakeup(pcb);
	}
      else if(pcb->priority < 128)
	{
	  /*
	  ** The target process is a timer process. Request a later
	  ** wakeup call.
	  */
	  request_timer_sigwakeup(pcb);
	}
      else
	{
	  /* What if the target is a sport? */
	}
    }

  /* Check if the current process is a background or prioritized process. */
  if(likely(zzcur_p->priority < 33))
    {
      struct _signal *mailtail = zzcur_p->mailtail;

      /*
      ** Check if we found any signals to timer or interrupt
      ** processes. If so, we wake up the one with the highest
      ** priority. The rest will be woken up as part of the
      ** zzenter_wakeup() epilogue.
      */
      if(unlikely(zzww.wk_look_aside))	
	{
	  /* Find highest level that needs a wakeup. */
	  OSPRIORITY int_level=(OSPRIORITY)FIND_FIRST_BIT(zzww.wk_look_aside);
	  
	  /* Get the wakeup list head for the current interrupt priority. */
	  pcb = zzww.wakeup_list[int_level].hd;
	  
	  int_level = (pcb->priority < 96)
	    ? pcb->priority & 31
	    : zzuu.timer_int_priority;
	  
	  zzenter_wakeup(pcb->uarea, pcb, int_level, (OSBOOLEAN) 1);
	}

      /*
      ** Check if a swap is requested. But do not swap if called from
      ** the idle loop.
      */
      if(zzww.dp_req && !i_am_idle)
	{
	  zzquit_int(); /* Swap. */
	}

      /*
      ** We need to tell the caller of this function if any of the
      ** higher prioritized processes (pri/ti/int) that was woken up
      ** by this function sent a signal to the current process.
      */
      if(unlikely(mailtail != zzcur_p->mailtail))
	{
	  status = 1;
	}
    }

  LOCK_RESTORE(msr);
  return status;
}

/*
** Call the hw poll function if present.
** Call the receive poll function if set for the calling process.
*/
static always_inline int
check_receive_poll(struct _pcb *pcb, int check_hw_poll)
{
  int (*rec_poller)(void) = pcb->rec_poller;
#ifdef OSE_SMP
  if (check_hw_poll)
  {
     void (*hw_receive_poller)(void) = zzww.hw_receive_poller;
     if (hw_receive_poller)
     {
        (hw_receive_poller)();
     }
  }
#else
  (void)check_hw_poll;
#endif
  return (rec_poller && (rec_poller)());
}

/*
**===========================================================================
** 16.3  System calls.
**===========================================================================
*/

/*
** The sender system call.
*/

PROCESS
sender(union SIGNAL **sig)
{
  register struct _signal *sigadm;

  /* Check out buffer pointer and get the signal control block. */
  sigadm = check_buffer(sig, OSE_SENDER);
  if(!sigadm)
    {
      return (PROCESS) 0;
    }

  /* Return the sender field. */
  if(sigadm->sender_id)
    return sigadm->sender_id;

  /*
  ** Or the owner field if the buffer was never sent.
  **
  ** There is an OSE Delta specification "feature" here.
  ** It is illegal to operate on a buffer that is not owned
  ** by the current process. The only exception to this rule
  ** is the restore() system call. Ownership was checked by the
  ** check_buffer() statement above, so when error checks are
  ** enabled we already know the owner.
  ** When error checks are off, it really doesn't matter
  ** what we return because the system is going down anyway if
  ** the caller operates further on a buffer he doesn't own.
  */
  return zzcur_p->pid;
}

/*
** The addressee system call.
*/

PROCESS
addressee(union SIGNAL **sig)
{
  register struct _signal *sigadm;

  /* Check out buffer pointer and get the signal control block. */
  sigadm = check_buffer(sig, OSE_ADDRESSEE);
  if(!sigadm)
    {
      return (PROCESS) 0;
    }

  /* Return addressee field. */
  if(sigadm->dest_id)
    return sigadm->dest_id;
  else
    return zzcur_p->pid;
}

/*
** The sigsize system call.
*/

OSBUFSIZE
sigsize(union SIGNAL **sig)
{
  register struct _signal *sigadm;


  /* Check out buffer pointer and get the signal control block. */
  sigadm = check_buffer(sig, OSE_SIGSIZE);
  if(!sigadm)
    {
      return (OSBUFSIZE) 0;
    }

  /* Return size field. */
    return sigadm->size;
}

/*
** Helper for flush() and (SMP) sysd_lite.
**
** Performs a flush on a prioritized or background process running on the
** current cpu.
** The dispatcher is locked by the caller. Any error checks shall be done
** by the caller.
*/
static void
local_flush(PROCESS *psel, struct _pcb *pcb)
{
  register struct _signal *sigadm;
  register U32 count;
  PROCESS *p;
  union SIGNAL *sig;
  int flushed = 0;

  /* Unlink and free all matching signals at the head of the signal queue. */
 flush_head:
  sigadm = pcb->mailbox;
  if(sigadm)
    {
      for(count = (U32) *psel, p = psel + 1; count; count--)
	{
	  if(sigadm->sender_id == *p++)
	    {
	      sig = adm_to_sig(sigadm);
	      BKL_LOCK();
	      pcb->mailbox = sigadm->next;
	      pcb->sig_cnt_queued--;
	      BKL_UNLOCK();
	      sys_free_buf(&sig);
	      flushed = 1;
	      goto flush_head;
	    }
	}
      
      /*
      ** Unlink and free all matching signals in the rest of the signal
      ** queue.
      */
    flush_tail:
      while(sigadm->next)
	{
	  for(count = (U32) *psel, p = psel + 1; count; count--)
	    {
	      if(sigadm->next->sender_id == *p++)
		{
		  sig = adm_to_sig(sigadm->next);
		  BKL_LOCK();
		  sigadm->next = sigadm->next->next;
		  if(!sigadm->next)
		    pcb->mailtail = sigadm;
		  pcb->sig_cnt_queued--;
		  BKL_UNLOCK();
		  sys_free_buf(&sig);
		  flushed = 1;
		  goto flush_tail;
		}
	    }
	  sigadm = sigadm->next;
	  if(!sigadm)
	    break;
	}
    }

  /* Notify the process if we have changed its signal-queue. */
  if(flushed)
    {
      /* Set the flushed flag in the pcb of the flushed process. */
      pcb->flushed = (OSBOOLEAN) 1;
      
      /* Notify processes waiting at signal ports as well. */
      BKL_LOCK();
      pcb->s_pcb.take_listmodnumber++;
      BKL_UNLOCK();
    }
}

/*
** The flush system call.
*/

void
flush(PROCESS *psel, PROCESS pid)
{
  register PROCESS *p;
  register struct _pcb *pcb;
  register U32 count;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_FLUSH))
    return;
#endif

  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid(pid, OSE_FLUSH);
  if(pcb == NULL)
    {
      return;
    }

  /* Check that all process id's in the psel array are legal. */
  for(count = (U32) *psel, p = psel + 1; count; count--)
    {
      if(get_ptype(*p++) == OS_ILLEGAL)
	{
	os_error(0, OSE_EILLEGAL_FLUSH_SENDER | OSE_FLUSH, (OSERRCODE) 0);	/* Non fatal. */
	return;
	}
    }

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      UNLOCK_SWAP();
      return;
    }

#if USE_REMOTE_CALLS
  /* Issue a remote system call if required. */
  if(local_sysctlblk.enable_remcalls)
  {
    register OSBOOLEAN use_remcalls;
    register PROCESS handler;
    if(pcb->magic == MAGIC_PCB)
      {
	use_remcalls = pcb->block->use_remcalls;
	handler = pcb->block->remcall_server;
      }
    else
      {
	use_remcalls = ((struct _block *)pcb)->use_remcalls;
	handler = ((struct _block *)pcb)->remcall_server;
      }
    if(use_remcalls)
      {
	union SIGNAL *sig;
	register PROCESS *q;

	UNLOCK_SWAP();
	set_suid((OSBOOLEAN) 1);
	sig = alloc(sizeof(struct remote_call) +
		    ((1 + *psel) * sizeof(PROCESS)),
		    REMOTE_CALL);
	sig->remote_call.call_type = REM_FLUSH;
	sig->remote_call.params.rem_flush.pid = pid;
	for(count = (U32) *psel, p = psel, q = sig->remote_call.params.rem_flush.psel;
	    count; count--)
	  *q++ = *p++;

	sig = make_remote_call(sig, (OSCALLTYPE) REMR_FLUSH, handler);
	if(sig)
	  {
	    free_buf(&sig);
	  }
	set_suid((OSBOOLEAN) 0);
	return;
      }
  }
#endif

  /*
  ** Skip the flush operation for all process types but prioritized,
  ** background and sports.
  */
  if(pcb->priority > 32 && !(pcb->priority == 128 && pcb->is_an_sport))
    {
      /* Unlock the dispatcher. */
      UNLOCK_SWAP();
      return;
    }

#ifdef OSE_SMP
  /* If the victim is on another cpu, perform the job there. */
  if(pcb->cpu_id != zzkrn_cpu_id())
    {
      static const SIGSELECT sel_flush_reply[] = {1, SMP_FLUSH_REPLY};
      union SIGNAL *sigr;
      
      /* Unlock the dispatcher. */
      UNLOCK_SWAP();

      sigr = alloc((sizeof(struct smp_flush)) + *psel * sizeof(PROCESS),
		   SMP_FLUSH_REQUEST);
      sigr->smp_flush.pid = pid;
      memcpy(sigr->smp_flush.psel, psel, *psel * sizeof(PROCESS));
      /*
      ** Do we need to wait until completed here? 
      ** Yes, stop the victim and wait for response signal, otherwise we
      ** may flush yet unsent signals! 
      */
      stop(pid);
      send(&sigr, zzcpu[pcb->cpu_id].sysd_lite_);
      sigr = receive(sel_flush_reply);
      free_buf(&sigr);
      start(pid);
      return;
    }
#endif

  /* The victim runs on the current cpu. Flush it. */
  local_flush(psel, pcb);

  /* Unlock the dispatcher. */
  UNLOCK_SWAP();
}

/*
** The receive system call.
*/

union SIGNAL *
receive(const SIGSELECT *sigsel)
{
  LOCKSTATE msr;
  register struct _pcb *pcb;
  register const SIGSELECT *wanted;
  register struct _signal *prevbuf;
  register struct _signal *thisbuf;
  register SIGSELECT all_wanted;
  
#if (OSE_DEBUG + MEMORY_MODEL) > 0
  /* Retry here if a received signal was killed by the debugger. */
 retry:
#endif

  /* Get important pointers to registers. */
  pcb = zzcur_p;
  wanted = sigsel;
      
  /* Create a flag that says if we need to  check the sigsel array. */
  all_wanted = !*wanted;
      
  /* Run the interrupt version if the caller is an int or timer process. */
  if(pcb->priority > 32)
    {
      /*
      ** The receive system call for interrupt and timer processes.
      **
      ** We know that the queue can not be flushed and the debugger
      ** is not used to examine the process.
      */

    int_restart_receive:
      /* Look at first buffer. */
      prevbuf = pcb->mailbox;
      if(prevbuf)
	{
	  /* Buffer present. Check it. */
	  if(likely(all_wanted ||
		    iswanted(*((SIGSELECT *) (adm_to_sig(prevbuf))), wanted)))
	    {
	      /* If wanted, unlink and return the signal. */
	      LOCK_SAVE(msr);
	      pcb->mailbox = prevbuf->next;
	      pcb->sig_cnt_queued--;

	      /* Set new signal owner. */
	      prevbuf->owner = pcb_to_pindex(pcb); /* Note! Owner is 16-bits. @frbr@ */
	      /* Increment owned counter in callers pcb. */
	      pcb->sig_cnt_owned++;
	      LOCK_RESTORE(msr);

	      /* Trace the signal. */
#if OSE_DEBUG > 0
	      if(unlikely(zzuu.debugger_enabled))
		{
		  register struct _pcb *from_pcb;
		  /* Check that the sender is still alive. */
		  from_pcb = pid_to_pcb(prevbuf->sender_id);
		  if(pcb_to_pid(from_pcb) == prevbuf->sender_id)
		    {
		      /* check for trace receive request. */
		      if((pcb->debug_flags | from_pcb->debug_flags) & 0x02)
			{
			  zzireceive_dbghook((OSBUFSIZE) prevbuf->size,
					     adm_to_sig(prevbuf),
					     pcb_to_debug_pcb(from_pcb),
					     pcb_to_debug_pcb(pcb));
			}
		    }
		}
#endif

	      /* Copy the signal to the callers segment if required. */
#if MEMORY_MODEL == 0
	      return adm_to_sig(prevbuf);
#else
	      {
		union SIGNAL *nsig;
		/* Check if intersegment copying is enabled. */
		if(!local_sysctlblk.allow_pool_copy ||
		   prevbuf->buffer_head->poolcbp->no_copy)
		  /* Done otherwise. */
		  return adm_to_sig(prevbuf);
		/*
		** Check if the segment of the received signal is the one
		** in which the callers pool resides.
		*/
		if(zzcur_p->cursigpool->segid == prevbuf->buffer_head->segid)
		  /* Done if they are the same. */
		  return adm_to_sig(prevbuf);
		/* Copy signal contents to a new buffer. */
		nsig = rec_sig_dup(prevbuf);
		if(!nsig)
		  /* Restart the receive operation if the copy failed. */
		  goto retry;
		/* Return the new buffer. */
		return nsig;
	      }
#endif
	    }
	  else
	    {
	      /* If not wanted, check next signal in the queue. */
	      thisbuf = prevbuf->next;
	    }
	}
      else
	{
          if(check_receive_poll(pcb, 1))
	    goto int_restart_receive;
	  /* No buffer present. Return NIL. */
	  return NIL;
	}
      
      /* Check second and following buffers. */
      for(;;)
	{
	  /* Is there a buffer to check? */
	  if(thisbuf)
	    {
	      /* Yes, check it. */
	      /* We don't need to check all_wanted here since we know it is zero. */
	      if(iswanted(*((SIGSELECT *) (adm_to_sig(thisbuf))), wanted))
		{
		  /* If wanted, unlink and return the signal. */
		  LOCK_SAVE(msr);
                  prevbuf->next = thisbuf->next;
		  if(!prevbuf->next)
		    {
		      pcb->mailtail = prevbuf;
		    }
		  pcb->sig_cnt_queued--;

		  /* Set new signal owner. */
		  thisbuf->owner = pcb_to_pindex(pcb); /* Note! Owner is 16-bits. @frbr@ */
		  /* Increment owned counter in callers pcb. */
		  pcb->sig_cnt_owned++;
		  LOCK_RESTORE(msr);

	      /* Trace the signal. */
#if OSE_DEBUG > 0
	      if(unlikely(zzuu.debugger_enabled))
		{
		  register struct _pcb *from_pcb;
		  /* Check that the sender is still alive. */
		  from_pcb = pid_to_pcb(thisbuf->sender_id);
		  if(pcb_to_pid(from_pcb) == thisbuf->sender_id)
		    {
		      /* check for trace receive request. */
		      if((pcb->debug_flags | from_pcb->debug_flags) & 0x02)
			{
			  zzireceive_dbghook((OSBUFSIZE) thisbuf->size,
					     adm_to_sig(thisbuf),
					     pcb_to_debug_pcb(from_pcb),
					     pcb_to_debug_pcb(pcb));
			}
		    }
		}
#endif
		  /* Copy the signal to the callers segment if required. */
#if MEMORY_MODEL == 0
		  return adm_to_sig(thisbuf);
#else
		  {
		    union SIGNAL *nsig;
		    /* Check if intersegment copying is enabled. */
		    if(!local_sysctlblk.allow_pool_copy ||
		       thisbuf->buffer_head->poolcbp->no_copy)
		      /* Done otherwise. */
		      return adm_to_sig(thisbuf);
		    /*
		    ** Check if the segment of the received signal is the one
		    ** in which the callers pool resides.
		    */
		    if(zzcur_p->cursigpool->segid == thisbuf->buffer_head->segid)
		      /* Done if they are the same. */
		      return adm_to_sig(thisbuf);
		    /* Copy signal contents to a new buffer. */
		    nsig = rec_sig_dup(thisbuf);
		    if(!nsig)
		      /* Restart the receive operation if the copy failed. */
		      goto retry;
		    /* Return the new buffer. */
		    return nsig;
		  }
#endif
		}
	      else
		{
		  /* If not wanted, check next signal in the queue. */
		  prevbuf = thisbuf;
		  thisbuf = prevbuf->next;
		}
	    }
	  else
	    {
	      /* Check if we need to poll out any ose_fsend() signal. */
              if(!check_receive_poll(pcb, 1))
		{
		  /* No buffer present. Return NIL. */
		  return NIL;
		}
	      thisbuf = prevbuf->next;
	    }
	}
    }
  /* The caller is not an interrupt or timer process. */
  else
    {
      /* Give list of wanted signals to the debugger. */
      pcb->wanted = wanted;
      
      /* Disable interrupts. */
      LOCK();
      
      /* Restart search here if the queue was flushed. */
    restart_receive:
      
      /* Wait for first buffer to arrive. */
      prevbuf = pcb->mailbox;
      if(prevbuf)
	{
	  SIGSELECT sig_no = 0; /* Kill "may be used uninitialized" warning. */

	  /*
	  ** We need to read out the signal number from the user pool
	  ** with interrupts disabled, otherwise the user pool might
	  ** be reclaimed and thus the memory will become unmapped,
	  ** causing the access to the signal number to crash.
	  */
	  if(!all_wanted)
	    sig_no = *(SIGSELECT *)(adm_to_sig(prevbuf));

	  /* Buffer present. Check it. */
	  UNLOCK();
	  if(all_wanted || iswanted(sig_no, wanted))
	    {
	      /* If wanted, unlink and return the signal unless flushed. */
	      LOCK();
	      if(pcb->flushed)
		{
		  /* Clear flushed flag and restart if flushed. */
		  pcb->flushed = (OSBOOLEAN) 0;
		  goto restart_receive;
		}

	      pcb->mailbox = prevbuf->next;
	      pcb->sig_cnt_queued--;

	      /* Set new signal owner. */
	      prevbuf->owner = pcb_to_pindex(pcb); /* Note! Owner is 16-bits. @frbr@ */
	      /* Increment owned counter in callers pcb. */
	      pcb->sig_cnt_owned++;
	      UNLOCK();

	      /* Trace the signal. */
#if OSE_DEBUG > 0
	      if(zzuu.debugger_enabled)
		{
		  register struct _pcb *from_pcb;
		  /* Lock the dispatcher. */
		  LOCK_SWAP();
		  /* Check that the sender is still alive. */
		  from_pcb = pid_to_pcb(prevbuf->sender_id);
		  if(pcb_to_pid(from_pcb) == prevbuf->sender_id)
		    {
		      /* check for trace receive request. */
		      if((pcb->debug_flags | from_pcb->debug_flags) & 0x02)
			{
			  /* We must check if the signal was killed by the debugger. */
			  if(!zzreceive_dbghook((OSBUFSIZE) prevbuf->size,
					     adm_to_sig(prevbuf),
						pcb_to_debug_pcb(from_pcb),
						pcb_to_debug_pcb(pcb)))
			    {
			      /* Retry receive if the ldm killed the signal. */
			      UNLOCK_SWAP();
			      goto retry;
			    }
			}
		    }
		  /* Unlock the dispatcher. */
		  UNLOCK_SWAP();
		}
#endif

	      /* Copy the signal to the callers segment if required. */
#if MEMORY_MODEL == 0
	      return adm_to_sig(prevbuf);
#else
	      {
		union SIGNAL *nsig;
		/* Check if intersegment copying is enabled. */
		if(!local_sysctlblk.allow_pool_copy ||
		   prevbuf->buffer_head->poolcbp->no_copy)
		  /* Done otherwise. */
		  return adm_to_sig(prevbuf);
		/*
		** Check if the segment of the received signal is the one
		** in which the callers pool resides.
		*/
		if(zzcur_p->cursigpool->segid == prevbuf->buffer_head->segid)
		  /* Done if they are the same. */
		  return adm_to_sig(prevbuf);
		/* Copy signal contents to a new buffer. */
		nsig = rec_sig_dup(prevbuf);
		if(!nsig)
		  /* Restart the receive operation if the copy failed. */
		  goto retry;
		/* Return the new buffer. */
		return nsig;
	      }
#endif
	    }
	  else
	    {
	      /* If not wanted, check next signal in the queue. */
	      LOCK();
	      thisbuf = prevbuf->next;
	    }
	}
      else
	{
	  /* Check if we need to poll out any ose_fsend() signal. */
          if(check_receive_poll(pcb,1))
	    goto restart_receive;
	  /* No buffer present. Swap while waiting. */
	  /* Done if a timeout occurred. */
	  if(pcb->status & ST_TIMEOUT)
	    {
	      UNLOCK();
	      return NIL;
	    }
	  pcb->status = (pr_state_t) ST_RECEIVE;
	  make_mypri_not_ready(pcb);
	  goto restart_receive;
	}
      
      /* Check second and following buffers. (Ints disabled.) */
      for(;;)
	{
	  /* Clear flushed flag and restart if flushed. */
	  if(pcb->flushed)
	    {
	      pcb->flushed = (OSBOOLEAN) 0;
	      goto restart_receive;
	    }
	  
	  /* Is there a buffer to check? */
	  if(thisbuf)
	    {
	      /* Yes, check it. */
	      
	      /*
	      ** We need to read out the signal number from the user pool
	      ** with interrupts disabled, otherwise the user pool might
	      ** be reclaimed and thus the memory will become unmapped,
	      ** causing the access to the signal number to crash.
	      */
	      SIGSELECT sig_no = *(SIGSELECT *)(adm_to_sig(thisbuf));
	       
	      /*
	      ** We don't need to check all_wanted here since we know
	      ** it is zero.
	      */
	      UNLOCK();
	      if(iswanted(sig_no, wanted))
		{
		  /* If wanted, unlink and return the signal unless flushed. */
		  LOCK();
		  if(pcb->flushed)
		    {
		      /* Clear flushed flag and restart if flushed. */
		      pcb->flushed = (OSBOOLEAN) 0;
		      goto restart_receive;
		    }

                  prevbuf->next = thisbuf->next;
		  if(!prevbuf->next)
		    {
		      pcb->mailtail = prevbuf;
		    }
		  pcb->sig_cnt_queued--;

		  /* Set new signal owner. */
		  thisbuf->owner = pcb_to_pindex(pcb); /* Note! Owner is 16-bits. @frbr@ */
		  /* Increment owned counter in callers pcb. */
		  pcb->sig_cnt_owned++;
		  UNLOCK();

	      /* Trace the signal. */
#if OSE_DEBUG > 0
	      if(zzuu.debugger_enabled)
		{
		  register struct _pcb *from_pcb;
		  /* Lock the dispatcher. */
		  LOCK_SWAP();
		  /* Check that the sender is still alive. */
		  from_pcb = pid_to_pcb(thisbuf->sender_id);
		  if(pcb_to_pid(from_pcb) == thisbuf->sender_id)
		    {
		      /* check for trace receive request. */
		      if((pcb->debug_flags | from_pcb->debug_flags) & 0x02)
			{
			  /* We must check if the signal was killed by the debugger. */
			  if(!zzreceive_dbghook((OSBUFSIZE) thisbuf->size,
					     adm_to_sig(thisbuf),
						pcb_to_debug_pcb(from_pcb),
						pcb_to_debug_pcb(pcb)))
			    {
			      /* Retry receive if the ldm killed the signal. */
			      UNLOCK_SWAP();
			      goto retry;
			    }
			}
		    }
		  /* Unlock the dispatcher. */
		  UNLOCK_SWAP();
		}
#endif
		  /* Copy the signal to the callers segment if required. */
#if MEMORY_MODEL == 0
		  return adm_to_sig(thisbuf);
#else
		  {
		    union SIGNAL *nsig;
		    /* Check if intersegment copying is enabled. */
		    if(!local_sysctlblk.allow_pool_copy ||
		       thisbuf->buffer_head->poolcbp->no_copy)
		      /* Done otherwise. */
		      return adm_to_sig(thisbuf);
		    /*
		    ** Check if the segment of the received signal is the one
		    ** in which the callers pool resides.
		    */
		    if(zzcur_p->cursigpool->segid == thisbuf->buffer_head->segid)
		      /* Done if they are the same. */
		      return adm_to_sig(thisbuf);
		    /* Copy signal contents to a new buffer. */
		    nsig = rec_sig_dup(thisbuf);
		    if(!nsig)
		      /* Restart the receive operation if the copy failed. */
		      goto retry;
		    /* Return the new buffer. */
		    return nsig;
		  }
#endif
		}
	      else
		{
		  /* If not wanted, check next signal in the queue. */
		  prevbuf = thisbuf;
		  LOCK();
		  thisbuf = prevbuf->next;
		}
	    }
	  else
	    {
	      /* Check if we need to poll out any ose_fsend() signal. */
              if(!check_receive_poll(pcb,1))
		{
		  /* No, wait for next buffer to arrive. */
		  /* Done if a timeout occurred. */
		  if(pcb->status & ST_TIMEOUT)
		    {
		      UNLOCK();
		      return NIL;
		    }
		  pcb->status = (pr_state_t) ST_RECEIVE;
		  make_mypri_not_ready(pcb);
		}
	      thisbuf = prevbuf->next;
	    }
	}
    }
}

/*
** The receive_from system call.
*/

union SIGNAL *
receive_from(OSTIME timeout,
	     const SIGSELECT *sig_sel,
	     PROCESS from)
{
  register struct _pcb *pcb = zzcur_p;
  register union SIGNAL *sig;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  ON_ERRLEVEL_2()
    if(check_nonint_caller(OSE_RECEIVE_FROM))
      return NIL;
#endif

  /*
  ** Check if the "from" object is a valid sender, dead or alive.
  ** We can safely ignore any user number here.
  **
  ** Don't return now if the "from" pid is legal but killed.
  ** This can lead to busy looping in constructions like this:
  **
  **   while (NIL == (sig = receive_from((OSTIME) -1, sig_sel, pid)));
  **
  ** @frbr@ is this the right way to do this?
  */
  if(unlikely(!safe_pid_to_pcb(from, 2) || !pid_to_instance(from)))
    {
      /* Report an error for this illegal pid. */
      os_error(0, OSE_EILLEGAL_PROCESS_ID | OSE_RECEIVE_FROM, (OSERRCODE) from); /* Non fatal. */
      return NIL;
    }

  /* If timeout is zero then check the queue but don't wait. */
  if(!timeout)
    {
      LOCK();
      pcb->status = (pr_state_t) (pcb->status | ST_TIMEOUT);
      UNLOCK();
      sig = local_receive_from(sig_sel, from);
      LOCK();
      pcb->status = (pr_state_t) (pcb->status & (~ST_TIMEOUT));
      UNLOCK();
      return sig;
    }

  /* If eternal timeout, wait for signal in local_receive_from(). */
  if(OSE_NO_TIMEOUT == timeout)
    {
      sig = local_receive_from(sig_sel, from);
      return sig;
    }

  /* Set up timeout. */
  LOCK();
  request_timeout(pcb, timeout);
  UNLOCK();

  /* Try to receive a signal. */
  sig = local_receive_from(sig_sel, from);

  /* Clear the timeout. */
  LOCK();
  cancel_timeout();
  UNLOCK();

  /* Done if a signal was received. */
  if(sig)
    return sig;

#if OSE_DEBUG > 0
  if(zzuu.debugger_enabled)
    {
       /* Lock the dispatcher. */
       LOCK_SWAP();
	     
       /* check for trace receive_w_tmo request. */
       if(pcb->debug_flags & 0x80)
         {
	   zzreceive_tmo_dbghook(pcb_to_debug_pcb(pcb), timeout);
	 }
		
	 /* Unlock the dispatcher. */
	 UNLOCK_SWAP();
    }
#endif
  
  /* Return NIL if no signal was found. */
  return NIL;
}

/*
** The receive_fsem system call.
*/

static union SIGNAL *
zzkrn_receive_fsem(OSTIME timeout,
	     const SIGSELECT *sig_sel,
	     OSFSEMVAL count)
{
  register struct _pcb *pcb = zzcur_p;
  register union SIGNAL *sig;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  ON_ERRLEVEL_2()
    if(check_nonint_caller(OSE_RECEIVE_FSEM))
      return NIL;
#endif

  /* If timeout is zero then check the queue but don't wait. */
  if(!timeout)
    {
      LOCK();
      pcb->status = (pr_state_t) (pcb->status | ST_TIMEOUT);
      UNLOCK();
      sig = local_receive_fsem(sig_sel, count);
      LOCK();
      pcb->status = (pr_state_t) (pcb->status & (~ST_TIMEOUT));
      UNLOCK();
      return sig;
    }

  /* If eternal timeout, wait for signal or sem in local_receive_fsem(). */
  if(OSE_NO_TIMEOUT == timeout)
    {
      sig = local_receive_fsem(sig_sel, count);
      return sig;
    }

  /* Set up timeout. */
  LOCK();
  request_timeout(pcb, timeout);
  UNLOCK();

  /* Try to receive a signal. */
  sig = local_receive_fsem(sig_sel, count);

  /* Clear the timeout. */
  LOCK();
  cancel_timeout();
  UNLOCK();

  /* Done if a signal or fast semaphore was received. */
  if(sig != NIL)
    return sig;

#if OSE_DEBUG > 0
  if(zzuu.debugger_enabled)
    {
       /* Lock the dispatcher. */
       LOCK_SWAP();
	     
       /* check for trace receive_w_tmo request. */
       if(pcb->debug_flags & 0x80)
         {
	   zzreceive_tmo_dbghook(pcb_to_debug_pcb(pcb),timeout);
	 }
		
	 /* Unlock the dispatcher. */
	 UNLOCK_SWAP();
    }
#endif
  
  /* Return NIL if no signal was found. */
  return NIL;
}

/*
** The sysconf_56 system call.
**
** Like receive_from but without timeout, and restoring any previously setup 
** timeout. This call is intended to be used from a page fault exception
** handler. Because a page fault may occur in any taste of receive() calls,
** the current timeout state is saved and restored. The timeout is prolonged
** by the time spent in the current receive operation.
**
** Argument p is the SIGSELECT pointer, addressing unpaged memory.
** Argument q is the wanted sender.
**
*/

static OSADDRESS
sysconf_56(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  struct _pcb_state_buf pcb_state;
  PROCESS from = (PROCESS) q;
  register union SIGNAL *sig;

  (void) r;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  ON_ERRLEVEL_2()
    if(check_nonint_caller(OSE_RECEIVE_FROM))
      return (OSADDRESS) NIL;
#endif

  /* Check the process id. Report error if invalid. */
  check_pid_bid_seg(from, OSE_RECEIVE_FROM);

  /* Save current timeout status and remove any pending or expired timeout.*/
  zzkrn_save_pcb_state(&pcb_state);
  
  sig = local_receive_from((const SIGSELECT *) p, from);

  /* Restore old timeout state */
  zzkrn_restore_pcb_state(&pcb_state);
  return (OSADDRESS) sig;
}

/*
** The receive_w_tmo system call.
*/

union SIGNAL *
receive_w_tmo(OSTIME timeout, const SIGSELECT *sig_sel)
{
  register struct _pcb *pcb = zzcur_p;
  register union SIGNAL *sig;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  ON_ERRLEVEL_2()
    if(check_nonint_caller(OSE_RECEIVE_W_TMO))
      return NIL;
#endif

  /* If timeout is zero then check the queue but don't wait. */
  if(!timeout)
    {
      LOCK();
      pcb->status = (pr_state_t) (pcb->status | ST_TIMEOUT);
      UNLOCK();
      sig = receive(sig_sel);
      LOCK();
      pcb->status = (pr_state_t) (pcb->status & (~ST_TIMEOUT));
      UNLOCK();
      return sig;
    }

  /* If eternal timeout, wait for signal in receive(). */
  if(OSE_NO_TIMEOUT == timeout)
    {
      sig = receive(sig_sel);
      return sig;
    }

  /* Set up timeout. */
  LOCK();
  request_timeout(pcb, timeout);
  UNLOCK();

  /* Try to receive a signal. */
  sig = receive(sig_sel);

  /* Clear the timeout. */
  LOCK();
  cancel_timeout();
  UNLOCK();

  /* Done if a signal was received. */
  if(sig)
    return sig;

#if OSE_DEBUG > 0
  if(zzuu.debugger_enabled)
    {
      /* Lock the dispatcher. */
      LOCK_SWAP();
	     
      /* check for trace receive_w_tmo request. */
      if(pcb->debug_flags & 0x80)
        {
	  zzreceive_tmo_dbghook(pcb_to_debug_pcb(pcb), timeout);
	}
		
      /* Unlock the dispatcher. */
      UNLOCK_SWAP();
    }
#endif

  /* Return NIL if no signal was found. */
  return NIL;
}

/*
** The send system call.
*/

void
send(union SIGNAL **sig, PROCESS to)
{
  PROCESS addressee_pid = to;	/* Pid used in debug hooks. */
  register struct _pcb *pcb;
  register struct _signal *sigadm;
  register struct _pcb *cur_pp = zzcur_p;

  (void)addressee_pid;
  
#ifdef BENGT_DEBUG
  /* This is junk code which can be safely removed at any time. ?? */
  print_sig(sig, current_process(), to);
#endif

  /* Check out buffer pointer and get the signal control block. */
  sigadm = check_buffer(sig, OSE_SEND);
  if(unlikely(!sigadm))
    {
      return;
    }

  /* Set the sender and addressee fields in the signal. */
  sigadm->sender_id = cur_pp->pid;
  sigadm->dest_id = (PROCESS) 0;
  
  /* Locate the block or process descriptor. */
  pcb = pid_to_pcb(to);

  /* Use send hook if present. */
  if(likely(pid_to_pindex(to) < zzuu.next_free_pix) && unlikely(pcb->snd_hook))
    {
      if(likely(send_hook(sigadm, 0, to, pcb)))
	{
	  *sig = NIL;
	  return;
	}
    }

  /* Run the interrupt version if the caller is an int or timer process. */
  if(cur_pp->priority > 32)
    {
      /* Read instance number and check the process id. Report error if invalid. */
#if ERROR_CHECKS == 0
      if(unlikely(pcb_to_pid(pcb) != to))
	{
	  free_buf(sig);
	  return;
	}
#endif

#if ERROR_CHECKS == 1
      if(zzuu.error_level1)
	{
	  if(unlikely(!(check_pid_bid(to, OSE_SEND))))
	    {
	      free_buf(sig);
	      return;
	    }
	}
      else
	{
	  if(unlikely(pcb_to_pid(pcb) != to))
	    {
	      free_buf(sig);
	      return;
	    }
	}
#endif

#if ERROR_CHECKS == 2
      if(unlikely(!(check_pid_bid(to, OSE_SEND))))
	{
	  free_buf(sig);
	  return;
	}
#endif

      /* If block descriptor then send to block process instead. */
      if(unlikely(pcb->magic == MAGIC_BCB))
	{
	  struct _block *bcb = (struct _block *) pcb;
	  
	  if(bcb->block_process)
	    {
	      pcb = bcb->block_process;
	      if(pcb->killed || pcb->block != bcb)
		{
		  /* Drop signal if no block process is present. */
		  free_buf(sig);
		  return;
		}
	      to = addressee_pid = pcb_to_pid(pcb);
	    }
	  else
	    {
	      /* Drop signal if no block process is present. */
	      free_buf(sig);
	      return;
	    }
	}

      /*
      ** Use redirection table if present.
      ** pcb is set to new target or NULL if not found.
      ** The addressee field is updated if required.
      ** Protect from updates in SMP??
      */
      if(unlikely(pcb->redir))
	{
	  pcb = send_redir(sigadm, pcb, cur_pp);
	  if(unlikely(!pcb))
	    {
	      /* Terminate system call if final addresse not found. */
	      free_buf(sig);
	      return;
	    }
	  to = pcb_to_pid(pcb);
	}

      /* Skip operation if target is a phantom process. */
      if(unlikely(pcb->priority == 128))
	{
	  /* Trace the signal. */
#if OSE_DEBUG > 0
	  if(unlikely(zzuu.debugger_enabled))
	    {
	      register struct _pcb *to_pcb = pid_to_pcb(addressee_pid);

	      /* check for trace send request. */
	      if((cur_pp->debug_flags | to_pcb->debug_flags) & 0x01)
		{
		  /* Check it with the ldm. */
		  zzisend_dbghook((OSBUFSIZE) sigadm->size,
				  adm_to_sig(sigadm),
				  pcb_to_debug_pcb(cur_pp),
				  pcb_to_debug_pcb(to_pcb));
		}
	    }
#endif

	  /* Check if the phantom is an S-port. */
	  send_to_sport(sig, sigadm, pcb);
	  /* Free the signal if the phantom is not an S-port. */
	  if(*sig)
	    free_buf(sig);
	  return;
	}

      /*
      ** Check for intersegment send to interrupt process.
      ** This is a lengthy test so it should be removed whenever possible.
      ** We consider this an illegal operation only for memory models which
      ** implement demand paging.
      */
#if ERROR_CHECKS > 0
#if MEMORY_MODEL > 1
      ON_ERRLEVEL_2()
	{
	  if(pcb->priority > 32)
	    {
	      if(pcb->block->sigpool->segid !=
		 sigadm->buffer_head->poolcbp->segid)
		{
		  os_error(0, OSE_EINTERSEGMENT_SEND | OSE_SEND, (OSERRCODE) 0);	/* Non fatal. */
		  free_buf(sig);
		  return;
		}
	    }
	}
#endif
#endif

      /* Trace the signal. */
#if OSE_DEBUG > 0
      if(unlikely(zzuu.debugger_enabled))
	{
	  register struct _pcb *to_pcb = pid_to_pcb(addressee_pid);

	  /* check for trace send request. */
	  if((cur_pp->debug_flags | to_pcb->debug_flags) & 0x01)
	    {
	      /* Check it with the ldm. */
	      zzisend_dbghook((OSBUFSIZE) sigadm->size,
			      adm_to_sig(sigadm),
			      pcb_to_debug_pcb(cur_pp),
			      pcb_to_debug_pcb(to_pcb));
	    }
	}
#endif

      /* Take buffer from user. */
      *sig = NIL;

      /* Send the signal. */
      int_send_finalize(sigadm, to, cur_pp);
    }
  else
    {
      /*
      ** The caller is not an interrupt or timer process.
      **
      ** Check the process id. Report error if invalid.
      */

#if ERROR_CHECKS == 1
      if(zzuu.error_level1)
	{
	  pcb = check_pid_bid(to, OSE_SEND);
	  if(unlikely(pcb == NULL))
	    {
	      free_buf(sig);
	      return;
	    }
	}
#endif

#if ERROR_CHECKS == 2
      pcb = check_pid_bid(to, OSE_SEND);
      if(unlikely(pcb == NULL))
	{
	  free_buf(sig);
	  return;
	}
#endif

      /* If block descriptor then send to block process instead. */
      if(unlikely(pcb->magic == MAGIC_BCB))
	{
	  struct _block *bcb = (struct _block *) pcb;
	  
	  LOCK_SWAP();
	  pcb = bcb->block_process;

	  /*
	  ** If the block is still present and it has a block process and
	  ** the block process is still alive, then signal it.
	  */
	  if(pcb_to_pid(bcb) == to && pcb && !pcb->killed)
	    {
	      to = addressee_pid = pcb_to_pid(pcb);
	      UNLOCK_SWAP();
	    }
	  else
	    {
	      /* Drop signal if no block process is present. */
	      UNLOCK_SWAP();
	      free_buf(sig);
	      return;
	    }
	}

      /*
      ** Use redirection table if present.
      ** pcb is set to new target or NULL if not found.
      ** The addressee field is updated if required.
      */
      if(unlikely(pcb->redir))
	{
	  LOCK_SWAP();
	  /* Check that the process is still present. */
	  if(unlikely(pcb_to_pid(pcb) != to))
	    {
	      /* Terminate system call if not. */
	      UNLOCK_SWAP();
	      free_buf(sig);
	      return;
	    }
	  pcb = send_redir(sigadm, pcb, cur_pp);
	  if(!pcb)
	    {
	      /* Terminate system call if final addresse not found. */
	      UNLOCK_SWAP();
	      free_buf(sig);
	      return;
	    }
	  to = pcb_to_pid(pcb);
	  UNLOCK_SWAP();
	}

      /* Skip operation if target is a phantom process. */
      if(unlikely(pcb->priority == 128))
	{
	  LOCK_SWAP();
	  /* Check that the process is still present. */
	  if(unlikely(pcb_to_pid(pcb) != to))
	    {
	      /* Terminate system call if not. */
	      UNLOCK_SWAP();
	      free_buf(sig);
	      return;
	    }

	  /*
	  ** There is a problem with this trace operation??
	  ** The dispatcher must be unlocked by the ldm during data copy
	  ** since a page fault may occur or the signal may be caught.
	  ** It must also be unlocked if the signal is caught.
	  ** When we return from the ldm, the sender and addressee must be 
	  ** rechecked and the signal dropped if any process was killed.
	  ** Note that the ldm must read all it needs from the sender and
	  ** addressee pcb's before unlocking the dispatcher.
	  */

	  /* Trace the signal. Exit if it was killed by the debugger. */
#if OSE_DEBUG > 0
	  if(unlikely(zzuu.debugger_enabled))
	    {
	      /* If block descriptor then trace block process instead. */
	      register struct _pcb *to_pcb = pid_to_pcb(addressee_pid);

	      /* check for trace send request. */
	      if((cur_pp->debug_flags | to_pcb->debug_flags) & 0x01)
		{
		  /* Save instance number in case the addressee is killed during trace. */
		  PCB_INSTANCE_T instance = pcb_to_instance(pcb);

		  /* Check it with the ldm. */
		  if(!zzsend_dbghook((OSBUFSIZE) sigadm->size,
				     adm_to_sig(sigadm),
				     pcb_to_debug_pcb(cur_pp),
				     pcb_to_debug_pcb(to_pcb)))
		    {
		      /* Done if the ldm killed the signal. */
		      UNLOCK_SWAP();
		      return;
		    }
		  if(instance != pcb_to_instance(pcb))
		    {
		      /* Done if the addressee were killed. */
		      UNLOCK_SWAP();
		      free_buf(sig);
		      return;
		    }
		}
	    }
#endif
	  
	  /* Check if the phantom is an S-port. */
	  send_to_sport(sig, sigadm, pcb);
	  UNLOCK_SWAP();
	  /* Free the signal if the phantom is not an S-port. */
	  if(*sig)
	    free_buf(sig);
	  return;
	}

      /*
      ** Check for intersegment send to interrupt process.
      ** This is a lengthy test so it should be removed whenever possible.??
      ** We consider this an illegal operation only for memory models which
      ** implement demand paging.
      */
#if ERROR_CHECKS > 0
#if MEMORY_MODEL > 1
      ON_ERRLEVEL_2()
	{
	  if(pcb->priority > 32)
	    {
	      if(pcb->block->sigpool->segid !=
		 sigadm->buffer_head->poolcbp->segid)
		{
		  UNLOCK_SWAP(); /* @frbr@ unbalanced UNLOCK_SWAP() ??? */
		  os_error(0, OSE_EINTERSEGMENT_SEND | OSE_SEND, (OSERRCODE) 0);	/* Non fatal. */
		  free_buf(sig);
		  return;
		}
	    }
	}
#endif
#endif

      /*
      ** There is a problem with this trace operation??
      ** The dispatcher must be unlocked by the ldm during data copy
      ** since a page fault may occur or the signal may be caught.
      ** It must also be unlocked if the signal is caught.
      ** When we return from the ldm, the sender and addressee must be
      ** rechecked and the signal dropped if any process was killed.
      ** Note that the ldm must read all it needs from the sender and
      ** addressee pcb's before unlocking the dispatcher.
      */

      /* Trace the signal. Exit if it was killed by the debugger. */
#if OSE_DEBUG > 0
      if(unlikely(zzuu.debugger_enabled))
	{
	  /* If block descriptor then trace block process instead. */
	  register struct _pcb *to_pcb = pid_to_pcb(addressee_pid);

	  LOCK_SWAP();
	  /* Check that the process is still present. */
	  if(pcb_to_pid(pcb) != to)
	    {
	      /* Terminate system call if not. */
	      UNLOCK_SWAP();
	      free_buf(sig);
	      return;
	    }

	  /* check for trace send request. */
	  if((cur_pp->debug_flags | to_pcb->debug_flags) & 0x01)
	    {
	      /* Save instance number in case the addressee is killed during trace. */
	      PCB_INSTANCE_T instance = pcb_to_instance(pcb);

	      /* Check it with the ldm. */
	      if(!zzsend_dbghook((OSBUFSIZE) sigadm->size,
				 adm_to_sig(sigadm),
				 pcb_to_debug_pcb(cur_pp),
				 pcb_to_debug_pcb(to_pcb)))
		{
		  /* Done if the ldm killed the signal. */
		  UNLOCK_SWAP();
		  return;
		}
	      if(instance != pcb_to_instance(pcb))
		{
		  /* Done if the addressee were killed. */
		  UNLOCK_SWAP();
		  free_buf(sig);
		  return;
		}
	    }
	  UNLOCK_SWAP();
	}
#endif

      /* Take buffer from user. */
      *sig = NIL;

      /* Send the signal. */
      send_finalize(sigadm, to);
    }
}

/*
** This is the ose_fsend() system call.
*/

OSADDRESS
ose_fsend(union SIGNAL **sig_p, PROCESS from, PROCESS to, OSADDRESS flags)
{
  static const int freeze_mode = 1;
  static const OSERRCODE errcode = OSE_SEND;
  struct ose_krn_msg msg;
  union SIGNAL *sig;
  struct _pcb *pcb;
  struct _signal *sigadm;
  struct _pcb *cur_pp = zzcur_p;
  OSADDRESS status;
  LOCKSTATE msr;
#if ERROR_CHECKS > 0
  OSADDRESS safe_read;
#endif

  /*
  ** Check out buffer pointer and get the signal control block.
  */

  /* Check for the NULL pointer. */
  if(unlikely(!sig_p))
    {
      os_error(0, OSE_EBAD_PARAMETER | errcode, (OSERRCODE) sig_p);	/* Non fatal. */
      return 0;
    }

  /* Pick up the signal pointer. */
  sig = *sig_p;

  /* Check for the NIL pointer. */
  if(unlikely(sig == NIL))
    {
      os_error(0, OSE_EUSED_NIL_POINTER | errcode, (OSERRCODE) sig_p);	/* Non fatal. */
      return 0;
    }

#if ERROR_CHECKS > 0
  ON_ERRLEVEL_2()
    {
      /* Check that we can read the memory pointed out by sig. */
      if(zzguarded_memcpy(&safe_read, (const char *)sig, sizeof(safe_read), !freeze_mode))
	{
	  os_error(0, OSE_EWILD_SIG_POINTER | errcode, (OSERRCODE) sig_p);	/* Non fatal. */
	  return 0;
	}
    }
#endif

  /* Pick up a pointer to the signal administration block. */
  sigadm = sig_to_adm(sig);

#if MEMORY_MODEL > 0

#if ERROR_CHECKS > 0
  ON_ERRLEVEL_2()
    {
      /*
      ** If MEMORY_MODEL > 0 we need to check that we can read the sigadm,
      ** if not this indicates that the udata pointer in the user sigadm
      ** is broken.
      */
      if(zzguarded_memcpy(&safe_read, (const char *)sigadm, sizeof(safe_read), !freeze_mode))
	{
	  os_error(0, OSE_EWILD_SIG_POINTER | errcode, (OSERRCODE) sig_p);	/* Non fatal. */
	  return 0;
	}
    }

  if(unlikely(adm_to_sig(sigadm) != sig))
    {
      os_error(0, OSE_EWILD_SIG_POINTER | errcode, (OSERRCODE) sig_p);	/* Non fatal. */
      return 0;
    }
#endif

#endif

  /* Take buffer from user. */
  *sig_p = NIL;

  /* Check that the "to" process index is reasonable. */
  if(unlikely(pid_to_pindex(to) >= zzuu.next_free_pix))
    {
      os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) to);	/* Non fatal. */
      safe_free_buf(sigadm);
      return 0;
    }

  /* Claim a HW pool signal owned by the system. */
  if((flags & OSE_FSEND_UNCLAIMED) &&
     sigadm->buffer_head->ext_handle != NULL)
    {
      if(sigadm->owner == 0)
	{
	  /*
	  ** The system is owner, we should check that the signal is not
	  ** in the free signal list. We often use dest_id for that.
	  ** But when the HW allocates the signal, dest_id is not updated.
	  ** Currently we have to depend on the client setting of the
	  ** OSE_FSEND_UNCLAIMED flag. ??
	  */
	  if(1)
	    {
	      /* Claim the signal. */
	      LOCK_SAVE(msr);
	      sigadm->owner = pcb_to_pindex(cur_pp);
	      cur_pp->sig_cnt_owned++;

	      /* Increment number of allocated signals. */
	      (void)atomic_inc((atomic_t *)&sigadm->buffer_head->alloc_count);

	      /* Set the end mark. */
#if ERROR_CHECKS > 0
	      *(((U8 *) sig) + sigadm->buffer_head->size) = (U8) END_MARK;
#endif

	      /* Set the size */
	      sigadm->size = sigadm->buffer_head->size;
	      LOCK_RESTORE(msr);
	    }
	  else
	    {
	      /* Error: the buffer is in the free list. */
	      os_error(0, OSE_EWILD_SIG_POINTER | errcode, (OSERRCODE) sig_p);	/* Non fatal. */
	      return 0;
	    }
	}
      else if(unlikely(sigadm->owner != pcb_to_pindex(cur_pp)))
	{
	  /* Error, got a signal owned by someone else: */
	  os_error(0, OSE_ENOT_SIG_OWNER | errcode, (OSERRCODE) sig_p);	/* Non fatal. */
	  return 0;
	}
    }

  /* Locate the receivers process descriptor. */
  pcb = pid_to_pcb(to);

  /* Use send hook if present. */
  if(unlikely(pcb->snd_hook))
    {
      sigadm->dest_id = (PROCESS) 0;
      if(likely(send_hook(sigadm, from, to, pcb)))
	{
	  return 0;
	}
      /*
      ** An error was detected, most likely related to the "from" or "to"
      ** parameters. The following code will report the error.
      */
    }
  
  /* Check that the receiver is alive. */
  if(unlikely(pcb_to_instance(pcb) != pid_to_instance(to)))
    {
      safe_free_buf(sigadm);
      return 0;
    }

  /* Check that the receiver is a process, but not a sport. */
  if(unlikely((pcb->magic) != MAGIC_PCB ||
	      (pcb->priority == 128 && pcb->is_an_sport)))
    {
      os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) to);	/* Non fatal. */
      safe_free_buf(sigadm);
      return 0;
    }

  /* Check the "from" pid. */
  if(likely(from == 0))
    {
      /* The caller is the sender. */
      from = cur_pp->pid;
    }
  else if(from != pcb_to_pid(cur_pp))
    {
      struct _pcb *from_pcb;
    
      /* Check that the process index is reasonable. */
      if(unlikely(pid_to_pindex(from) >= zzuu.next_free_pix))
	{
	  os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) from);	/* Non fatal. */
	  safe_free_buf(sigadm);
	  return 0;
	}

      /* Locate the process descriptor. */
      from_pcb = pid_to_pcb(from);

      /* If the "from process" is dead, just free the buffer. */
      if(unlikely(from != pcb_to_pid(from_pcb)))
	{
	  safe_free_buf(sigadm);
	  return 0;
	}
      
      /* Check that the magic number indicates a process. */
      if(unlikely((from_pcb->magic) != MAGIC_PCB))
	{
	  os_error(0, OSE_EILLEGAL_PROCESS_ID | errcode, (OSERRCODE) from);	/* Non fatal. */
	  safe_free_buf(sigadm);
	  return 0;
	}
    }

  /* Set the sender and addressee fields in the signal. */
  sigadm->sender_id = from;
  sigadm->dest_id = (PROCESS) 0;

  /*
  ** Use redirection table if present.
  ** Protect from updates in SMP.
  */
  if(unlikely(pcb->redir))
    {
      BKL_LOCK_SAVE(msr);
      pcb = simple_redir(sigadm, pcb, cur_pp);
      if(unlikely(!pcb))
	{
	  BKL_LOCK_RESTORE(msr);
	  /* Terminate system call if final addresse not found. */
	  safe_free_buf(sigadm);
	  return 0;
	}
      to = pcb_to_pid(pcb);
      BKL_LOCK_RESTORE(msr);
    }

  /* If the receiver is dead, just free the buffer. */
  if(unlikely(to != pcb_to_pid(pcb)))
    {
      safe_free_buf(sigadm);
      return 0;
    }

  /*
  ** Make sure the receiver (and the idle process at the receiving
  ** cpu) has a receive poller function.
  */
  if(unlikely(pcb->rec_poller == NULL))
    {
      BKL_LOCK_SAVE(msr);
      if(likely(to == pcb_to_pid(pcb)))
	{
	  pcb->rec_poller = receive_core_sig_queue;
	  (SMP_CODE_YES_NO(zzcpu[pcb->cpu_id].idle_pcb, zzuu.idle_pcb)->
	   rec_poller) = receive_core_sig_queue;
	}
      BKL_LOCK_RESTORE(msr);
    }

  /* Trace the signal. */
#if OSE_DEBUG > 0
  if(zzuu.debugger_enabled)
    {
      /* Check for trace send request. */
      if(unlikely((cur_pp->debug_flags | pcb->debug_flags) & 0x01))
	{
	  /* Check it with the ldm. */
	  zzisend_dbghook((OSBUFSIZE) sigadm->size,
			  adm_to_sig(sigadm),
			  pcb_to_debug_pcb(zzcur_p),
			  pcb_to_debug_pcb(pcb));
	}
    }
#endif

  /* Disable interrupts. */
  LOCK_SAVE(msr);

  /* Check signal ownership. */
  if(unlikely(sigadm->owner != pcb_to_pindex(cur_pp)))
    {
      LOCK_RESTORE(msr);
      *sig_p = sig;		/* OPS ?? */
      os_error(0, OSE_ENOT_SIG_OWNER | errcode, (OSERRCODE) sig_p);	/* Non fatal. */
      return 0;
    }

  /* Make system the signal owner. */
  sigadm->owner = (PCB_INDEX_T) 0;
  sigadm->next = NULL;
  
  /* Decrement owned counter in callers pcb. */
  cur_pp->sig_cnt_owned--;

  if(likely(pcb->cpu_id == zzkrn_cpu_id()))
    {
      /*
      ** Perform local send with no swap. 
      ** Link signal into mailbox of target process.
      */
      if(pcb->mailbox)
	{
	  pcb->mailtail->next = sigadm;
	  pcb->mailtail = sigadm;
	}
      else
	{
	  pcb->mailbox = sigadm;
	  pcb->mailtail = sigadm;
	}
      pcb->sig_cnt_queued++;

      if(likely(pcb->priority < 33))
        /*
        ** The target process is a prioritized or background process.
        **  Make the process ready but no swap.
        */
	{
	  if(pcb->status & ST_RECEIVE)
	    {
	      pcb->status = (pr_state_t) (pcb->status & (~ST_RECEIVE_FSEM));
           
	      /* If the process becomes ready then do a swap. */
	      if(pcb->status < ST_READY)
		{
		  make_otherpri_ready_noswap(pcb); /* noswap */
		}
	    }
	}
      /* treat int/timer processes as "deaf" */
      LOCK_RESTORE(msr);
      return 0;
    }
  else
    {
      /* Peform remote send via msg queues */

      msg.data[0] = KRN_EVT_SIG_SEND | (uint32_t)sigadm;
      msg.data[1] = (uint32_t)to;
  
      status = handlerp.zzmsg_enqueue(pcb->cpu_id, &msg, flags);

      if(likely(status == 0))
	{
	  /* Enable interrupts. */
	  LOCK_RESTORE(msr);
	  return 0;
	}
    }

  /*
  **  Failed to send signal. Free the buffer, that is currently owned by 
  ** the system. 
  */
  sigadm->owner = pcb_to_pindex(cur_pp); /* Note! Owner is 16-bits. @frbr@ */
  cur_pp->sig_cnt_owned++;
  sigadm->next = NULL;

  /* Enable interrupts. */
  LOCK_RESTORE(msr);

  /* Free the signal buffer. */
  safe_free_buf(sigadm);
  return 0;
}

/*
** The send_w_sender system call works like send_w_s() but it returns
** a non zero value if the signal was silently freed because the
** specified from/sender process has terminated.
*/

static OSBOOLEAN
zzkrn_send_w_sender(union SIGNAL **sig, PROCESS from, PROCESS to)
{
  PROCESS addressee_pid = to;	/* Pid used in debug hooks. */
  register struct _pcb *pcb;
  register struct _signal *sigadm;
  register struct _pcb *cur_pp = zzcur_p;
  register struct _pcb *from_pcb;

  (void)addressee_pid;
  
#ifdef BENGT_DEBUG
  /* This is junk code which can be safely removed at any time. ?? */
  print_sig(sig, from, to);
#endif

  /* Check out buffer pointer and get the signal control block. */
  sigadm = check_buffer(sig, OSE_SEND_W_S);
  if(!sigadm)
    {
      return (OSBOOLEAN)0;
    }

  /* Set the sender and addressee fields in the signal. */
  sigadm->sender_id = from;
  sigadm->dest_id = (PROCESS) 0;

  /* Locate the destination block or process descriptor. */
  pcb = pid_to_pcb(to);

  /* Use send hook if present. */
  if(likely(pid_to_pindex(to) < zzuu.next_free_pix) && unlikely(pcb->snd_hook))
    {
      if(likely(send_hook(sigadm, from, to, pcb)))
	{
	  *sig = NIL;
	  return (OSBOOLEAN)0;
	}
    }

  /* Locate the sender block or process descriptor. */
  from_pcb = pid_to_pcb(from);

  /* Run the interrupt version if the caller is an int or timer process. */
  if(cur_pp->priority > 32)
    {
      /*
      ** Read instance number of the dest process and check the process id.
      ** Report error if invalid.
      */
      ON_ERRLEVEL_2()
	{
	  if(!(check_pid_bid(to, OSE_SEND_W_S)))
	    {
	      free_buf(sig);
	      return (OSBOOLEAN)0;
	    }
	}
      else
	{
	  if(pcb_to_pid(pcb) != to)
	    {
	      free_buf(sig);
	      return (OSBOOLEAN)0;
	    }
	}

      /*
      ** Read instance number of sender process and check the process id.
      ** Report error if invalid.
      */
      ON_ERRLEVEL_2()
	{
	  if(!(check_pid_bid(from, OSE_SEND_W_S)))
	    {
	      free_buf(sig);
	      return (OSBOOLEAN)1;
	    }
	}
      else
	{
	  if(pcb_to_pid(from_pcb) != from)
	    {
	      free_buf(sig);
	      return (OSBOOLEAN)1;
	    }
	}

      /* If block descriptor then send to block process instead. */
      if(pcb->magic == MAGIC_BCB)
	{
	  if(((struct _block *) pcb)->block_process)
	    {
	      pcb = ((struct _block *) pcb)->block_process;
	      if(pcb->killed)
		{
		  /* Drop signal if no block process is present. */
		  free_buf(sig);
		  return (OSBOOLEAN)0;
		}
	      to = addressee_pid = pcb_to_pid(pcb);
	    }
	  else
	    {
	      /* Drop signal if no block process is present. */
	      free_buf(sig);
	      return (OSBOOLEAN)0;
	    }
	}

      /*
      ** Use redirection table if present.
      ** pcb is set to new target or NULL if not found.
      ** The addressee field is updated if required.
      */
      if(pcb->redir)
	{
	  pcb = send_redir(sigadm, pcb, from_pcb);
	  if(!pcb)
	    {
	      /* Terminate system call if final addresse not found. */
	      free_buf(sig);
	      return (OSBOOLEAN)0;
	    }
	  to = pcb_to_pid(pcb);
	}

      /* Skip operation if target is a phantom process. */
      if(pcb->priority == 128)
	{
	  /* Trace the signal. */
#if OSE_DEBUG > 0
	  if(zzuu.debugger_enabled)
	    {
	      register struct _pcb *to_pcb = pid_to_pcb(addressee_pid);

	      /* check for trace send request. */
	      if((cur_pp->debug_flags | to_pcb->debug_flags) & 0x01)
		{
		  /* Check it with the ldm. */
		  zzisend_dbghook((OSBUFSIZE) sigadm->size,
				  adm_to_sig(sigadm),
				  pcb_to_debug_pcb(from_pcb),
				  pcb_to_debug_pcb(to_pcb));
		}
	    }
#endif

	  /* Check if the phantom is an S-port. */
	  send_to_sport(sig, sigadm, pcb);
	  /* Free the signal if the phantom is not an S-port. */
	  if(*sig)
	    free_buf(sig);
	  return (OSBOOLEAN)0;
	}

      /*
      ** Check for intersegment send to interrupt process.
      ** This is a lengthy test so it should be removed whenever possible.??
      ** We consider this an illegal operation only for memory models which
      ** implement demand paging.
      */
#if ERROR_CHECKS > 0
#if MEMORY_MODEL > 1
      ON_ERRLEVEL_2()
	{
	  if(pcb->priority > 32)
	    {
	      if(pcb->block->sigpool->segid !=
		 sigadm->buffer_head->poolcbp->segid)
		{
		  os_error(0, OSE_EINTERSEGMENT_SEND | OSE_SEND_W_S, (OSERRCODE) 0);	/* Non fatal. */
		  free_buf(sig);
		  return (OSBOOLEAN)0;
		}
	    }
	}
#endif
#endif

      /* Trace the signal. */
#if OSE_DEBUG > 0
      if(zzuu.debugger_enabled)
	{
	  register struct _pcb *to_pcb = pid_to_pcb(addressee_pid);

	  /* check for trace send request. */
	  if((cur_pp->debug_flags | to_pcb->debug_flags) & 0x01)
	    {
	      /* Check it with the ldm. */
	      zzisend_dbghook((OSBUFSIZE) sigadm->size,
			      adm_to_sig(sigadm),
			      pcb_to_debug_pcb(from_pcb),
			      pcb_to_debug_pcb(to_pcb));
	    }
	}
#endif

      /* Take buffer from user. */
      *sig = NIL;

      /* Send the signal. */
      int_send_finalize(sigadm, to, cur_pp);
    }
  /* The caller is not an interrupt or timer process. */
  else
    {
      /*
      ** Read instance number of the dest process and check the process id.
      ** Report error if invalid.
      */
      ON_ERRLEVEL_2()
	{
	  if(check_pid_bid(to, OSE_SEND_W_S) == NULL)
	    {
	      free_buf(sig);
	      return (OSBOOLEAN)0;
	    }
	}

      /*
      ** Read instance number of sender process and check the process id.
      ** Report error if invalid.
      */
      ON_ERRLEVEL_2()
	{
	  if(check_pid_bid(from, OSE_SEND_W_S) == NULL)
	    {
	      free_buf(sig);
	      return (OSBOOLEAN)1;
	    }
	}

      /* Lock the dispatcher. */
      LOCK_SWAP();

      /* Check that the from processes is still present. */
      if(pcb_to_pid(from_pcb) != from)
	{
	  /* Terminate system call if not. */
	  UNLOCK_SWAP();
	  free_buf(sig);
	  return (OSBOOLEAN)1;
	}

      /* Check that the to processes is still present. */
      if(pcb_to_pid(pcb) != to)
	{
	  /* Terminate system call if not. */
	  UNLOCK_SWAP();
	  free_buf(sig);
	  return (OSBOOLEAN)0;
	}

      /* If block descriptor then send to block process instead. */
      if(pcb->magic == MAGIC_BCB)
	{
	  if(((struct _block *) pcb)->block_process)
	    {
	      pcb = ((struct _block *) pcb)->block_process;
	      if(pcb->killed)
		{
		  /* Drop signal if no block process is present. */
		  UNLOCK_SWAP();
		  free_buf(sig);
		  return (OSBOOLEAN)0;
		}
	      to = addressee_pid = pcb_to_pid(pcb);
	    }
	  else
	    {
	      /* Drop signal if no block process is present. */
	      UNLOCK_SWAP();
	      free_buf(sig);
	      return (OSBOOLEAN)0;
	    }
	}

      /*
      ** Use redirection table if present.
      ** pcb is set to new target or NULL if not found.
      ** The addressee field is updated if required.
      */
      if(pcb->redir)
	{
	  pcb = send_redir(sigadm, pcb, from_pcb);
	  if(!pcb)
	    {
	      /* Terminate system call if final addresse not found. */
	      UNLOCK_SWAP();
	      free_buf(sig);
	      return (OSBOOLEAN)0;
	    }
	  to = pcb_to_pid(pcb);
	}

      /* Skip operation if target is a phantom process. */
      if(pcb->priority == 128)
	{
	  /*
	  ** There is a problem with this trace operation??
	  ** The dispatcher must be unlocked by the ldm during data copy
	  ** since a page fault may occur or the signal may be caught.
	  ** It must also be unlocked if the signal is caught.
	  ** When we return from the ldm, the sender and addressee must be rechecked
	  ** and the signal dropped if any process was killed.
	  ** Note that the ldm must read all it needs from the sender and addressee pcb's
	  ** before unlocking the dispatcher.
	  */

	  /* Trace the signal. Exit if it was killed by the debugger. */
#if OSE_DEBUG > 0
	  if(zzuu.debugger_enabled)
	    {
	      register struct _pcb *to_pcb = pid_to_pcb(addressee_pid);
	      
	      /* check for trace send request. */
	      if((from_pcb->debug_flags | to_pcb->debug_flags) & 0x01)
		{
		  /* Save instance number in case the addressee is killed during trace. */
		  PCB_INSTANCE_T instance = pcb_to_instance(pcb);

		  /* Check it with the ldm. */
		  if(!zzsend_dbghook((OSBUFSIZE) sigadm->size,
				     adm_to_sig(sigadm),
				     pcb_to_debug_pcb(from_pcb),
				     pcb_to_debug_pcb(to_pcb)))
		    {
		      /* Done if the ldm killed the signal. */
		      UNLOCK_SWAP();
		      return (OSBOOLEAN)0;
		    }
		  if((pcb_to_pid(from_pcb) != from) ||
		     (instance != pcb_to_instance(pcb)))
		    {
		      /* Done if the sender or addressee were killed. */
		      UNLOCK_SWAP();
		      free_buf(sig);
		      return (OSBOOLEAN)(pcb_to_pid(from_pcb) != from);
		    }
		}
	    }
#endif

	  /* Check if the phantom is an S-port. */
	  send_to_sport(sig, sigadm, pcb);
	  UNLOCK_SWAP();
	  /* Free the signal if the phantom is not an S-port. */
	  if(*sig)
	    free_buf(sig);
	  return (OSBOOLEAN)0;
	}

      /*
      ** Check for intersegment send to interrupt process.
      ** This is a lengthy test so it should be removed whenever possible.??
      ** We consider this an illegal operation only for memory models which
      ** implement demand paging.
      */
#if ERROR_CHECKS > 0
#if MEMORY_MODEL > 1
      ON_ERRLEVEL_2()
	{
	  if(pcb->priority > 32)
	    {
	      if(pcb->block->sigpool->segid !=
		 sigadm->buffer_head->poolcbp->segid)
		{
		  UNLOCK_SWAP();
		  os_error(0, OSE_EINTERSEGMENT_SEND | OSE_SEND_W_S, (OSERRCODE) 0);	/* Non fatal. */
		  free_buf(sig);
		  return (OSBOOLEAN)0;
		}
	    }
	}
#endif
#endif

      /*
      ** There is a problem with this trace operation??
      ** The dispatcher must be unlocked by the ldm during data copy
      ** since a page fault may occur or the signal may be caught.
      ** It must also be unlocked if the signal is caught.
      ** When we return from the ldm, the sender and addressee must be rechecked
      ** and the signal dropped if any process was killed.
      ** Note that the ldm must read all it needs from the sender and addressee pcb's
      ** before unlocking the dispatcher.
      */

      /* Trace the signal. Exit if it was killed by the debugger. */
#if OSE_DEBUG > 0
      if(zzuu.debugger_enabled)
	{
	  register struct _pcb *to_pcb = pid_to_pcb(addressee_pid);

	  /* check for trace send request. */
	  if((from_pcb->debug_flags | to_pcb->debug_flags) & 0x01)
	    {
	      /* Save instance number in case the addressee is killed during trace. */
	      PCB_INSTANCE_T instance = pcb_to_instance(pcb);
	      /* Check it with the ldm. */
	      if(!zzsend_dbghook((OSBUFSIZE) sigadm->size,
				 adm_to_sig(sigadm),
				 pcb_to_debug_pcb(from_pcb),
				 pcb_to_debug_pcb(to_pcb)))
		{
		  /* Done if the ldm killed the signal. */
		  UNLOCK_SWAP();
		  return (OSBOOLEAN)0;
		}
	      if((pcb_to_pid(from_pcb) != from) ||
		 (instance != pcb_to_instance(pcb)))
		{
		  /* Done if the sender or addressee were killed. */
		  UNLOCK_SWAP();
		  free_buf(sig);
		  return (OSBOOLEAN)(pcb_to_pid(from_pcb) != from);
		}
	    }
	}
#endif

      /* Take buffer from user. */
      *sig = NIL;

      /* Send the signal. */
      locked_send_finalize(sigadm, pcb, cur_pp, 0, NULL, NULL);
    }
  return 0;
}

/*
** The send_w_s system call.
*/

void
send_w_s(union SIGNAL **sig, PROCESS from, PROCESS to)
{
  (void)zzkrn_send_w_sender(sig, from, to);
}

/*
**===========================================================================
** 16.4  Undocumented system calls.
**===========================================================================
*/

/*
** The receive_with system call.
** 
** It receives any signal in which the two first words matches the sel
** specification. The sel array contains one count, followed by an array
** of pairs of signo and subcode. e.g. static SIGSELECT sel[] = 2,3,4,5,6
** is a specification that matches either a signal which starts with
** the two words 3 and 4, or a signal which start with the words 5 and 6.
** The from field is set to zero to allow any sender.
*/

union SIGNAL *
receive_with(OSTIME timeout,
	     const SIGSELECT *sig_sel,
	     PROCESS from)
{
  register struct _pcb *pcb = zzcur_p;
  register union SIGNAL *sig;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  ON_ERRLEVEL_2()
    if(check_nonint_caller(OSE_RECEIVE_WITH))
      return NIL;
#endif

  /* Check the process id. Report error if invalid. */
  if(!check_pid(from, OSE_RECEIVE_WITH))
    {
      /* Don't return now if the "from" process is killed.
       * This can lead to busy looping in constructions like this:
       *
       *   while (NIL == (sig = receive_from((OSTIME) -1, sig_sel, pid)));
       *
       * @frbr@ is this the right way to do this?
       */
      if (get_ptype(from) != OS_ZOOMBIE)
	{
	  return NIL;
	}
    }

  /* If timeout is zero then check the queue but don't wait. */
  if(!timeout)
    {
      LOCK();
      pcb->status = (pr_state_t) (pcb->status | ST_TIMEOUT);
      UNLOCK();
      sig = local_receive_with(sig_sel, from);
      LOCK();
      pcb->status = (pr_state_t) (pcb->status & (~ST_TIMEOUT));
      UNLOCK();
      return sig;
    }

  /* If eternal timeout, wait for signal in local_receive_with(). */
  if(OSE_NO_TIMEOUT == timeout)
    {
      sig = local_receive_with(sig_sel, from);
      return sig;
    }

  /* Set up timeout. */
  LOCK();
  request_timeout(pcb, timeout);
  UNLOCK();

  /* Try to receive a signal. */
  sig = local_receive_with(sig_sel, from);

  /* Clear the timeout. */
  LOCK();
  cancel_timeout();
  UNLOCK();

  /* Done if a signal was received. */
  if(sig)
    return sig;

#if OSE_DEBUG > 0
  if(zzuu.debugger_enabled)
    {
      /* Lock the dispatcher. */
      LOCK_SWAP();
	     
      /* check for trace receive_w_tmo request. */
      if(pcb->debug_flags & 0x80)
        {
	  zzreceive_tmo_dbghook(pcb_to_debug_pcb(pcb), timeout);
	}
		
      /* Unlock the dispatcher. */
      UNLOCK_SWAP();
    }
#endif

  /* Return NIL if no signal was found. */
  return NIL;
}

/*
** The receive_sport() system call.
**
** The kernel supports a "signal port" concept.
** A signal port is basically a phantom process with an extra attribute
** set that causes signals sent to the process to remain in the signal
** queue of the phantom instead of being silently freed.
** Any process can send to a signal port and any process can read from
** the queue of a signal port.
** A signal port is created with the create_process() system call, using
** a new redirection table format. The normal table is preceded with an
** entry containing redir.sig=0 and redir.pid=1. The redir.pid field
** contains the attribute bits and bit 0 specifies the signal port
** behaviour.
** Signals are received through signal ports using the new system call:
** union SIGNAL *
** receive_sport(OSTIME timeout, SIGSELECT *sel, PROCESS *sport_list);
** It works like receive_w_tmo() except that signals are fetched from
** the signal queues of any of the processes in the sport_list, which has
** a layout similar to the psel array in the flush() system call.
** The list may only contain id's of signal ports, and it is limited to
** a reasonable size (currently 32).
** Note that the receive_sport() call is not a "real time" function as
** its execution time depends on the length of the sel and sport_list
** parameters.
**
** The system call uses the get_sysconf() framework, which means that
** there is no public entrypoint to a receive_sport() function.
** The receive_sport() call uses function 20 of get_sysconf().
** The user level prototype is:
** union SIGNAL *receive_sport(OSTIME timeout, SIGSELECT *sel,
**                             PROCESS *sport_list);
*/

static OSADDRESS
sysconf_20(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  OSTIME timeout = (OSTIME) p;
  SIGSELECT *sig_sel = (SIGSELECT *) q;
  PROCESS *sport_list = (PROCESS *) r;

  union SIGNAL *sig = NIL;
  register struct S_node *s_node;
  register struct S_nodelist *s_nodelist;

  PROCESS *pid;
  register struct _pcb *pcb;
  PCB_INSTANCE_T instance;
  register U32 count;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  ON_ERRLEVEL_2()
    if(check_nonint_caller(OSE_RECEIVE_SPORT))
      return (OSADDRESS) NIL;
#endif

  /* Check that the sport_list has a reasonable size. */
  if(*sport_list > MAX_SNODELIST)
    {
      os_error(0, OSE_EBAD_PARAMETER | OSE_RECEIVE_SPORT, (OSERRCODE) sport_list);	/* Non fatal. */
      delay(1);
      return (OSADDRESS) NIL;
    }

  /* Allocate an empty s-nodelist for the current process if we don't have one. */
  s_nodelist = (struct S_nodelist *) zzcur_p->s_pcb.s_nodelist;
  if(!s_nodelist)
    {
      /* Allocate a buffer from the system pool. */
      s_nodelist = (struct S_nodelist *)
	sys_alloc(sizeof(struct S_nodelist), 0);

      /* Clear it to an inactive state and set up read only fields. */
      s_nodelist->my_pcb = zzcur_p;
      s_nodelist->maxcount = MAX_SNODELIST;
      s_nodelist->curcount = 0;
      s_nodelist->isactive = 0;
      for(count = MAX_SNODELIST, s_node = s_nodelist->s_node; count; count--, s_node++)
	{
	  /* Is it really necessary to clear the phantom_pcb field since curcount = 0 ?? */
	  s_node->phantom_pcb = NULL;
	  s_node->base = s_nodelist;
	}
      /* Insert it into the current pcb. */
      zzcur_p->s_pcb.s_nodelist = (struct S_node *)
	s_nodelist;
    }

  /* Activate the empty s-nodelist. */
  s_nodelist->curcount = 0;
  s_nodelist->isactive = 1;

  /* Enter the s-nodelist into the s-nodechain of all phantom processes. */
  for(count = (U32) *sport_list, pid = sport_list + 1, s_node = s_nodelist->s_node;
      count; count--, s_node++, pid++)
    {
      /* Pick up the expected instance number. */
      instance = pid_to_instance(*pid);

      /* Check that the phantom pid is reasonable. */
#if ERROR_CHECKS > 0
      if(!instance)
	goto bad_sport;
      if( ((PCB_INSTANCE_T) *pid) >= zzuu.next_free_pix)
	goto bad_sport;
#endif

      /* Pick up the pcb pointer. */
      pcb = zzuu.pix_list[pid_to_pindex(*pid)];

      /* Skip phantoms that are dead. */
      BKL_LOCK();
      if(pcb_to_instance(pcb) != instance)
	{
	  BKL_UNLOCK();
	  /*
	  ** An sport is dead. Check if we should return an OS_ATTACH_SIG. 
	  */
	  if((!*sig_sel) || iswanted(OS_ATTACH_SIG, sig_sel))
	    {
	      sig = make_os_attach_sig(*pid);
	      goto done;
	    }
	  /* No, just ignore the pid. */
	  continue;
	}

      /* Check that the magic number indicates a process. */
      if(pcb->magic != MAGIC_PCB)
	goto bad_sport;

      /* Check that the phantom really has the S-port attribute set. */
      if(!pcb->is_an_sport)
	goto bad_sport;

      /* Add an s-node to the s-nodechain of the selected phantom. */
      /* Add to a non empty list. */
      if(pcb->s_pcb.s_nodelist)
	{
	  s_node->fp_s_nodelist = NULL;
	  s_node->bp_s_nodelist = pcb->s_pcb.s_nodetail;
	  s_node->phantom_pcb = pcb;
	  pcb->s_pcb.s_nodetail->fp_s_nodelist = s_node;
	  pcb->s_pcb.s_nodetail = s_node;
	}
      /* Add to an empty list. */
      else
	{
	  s_node->fp_s_nodelist = NULL;
	  s_node->bp_s_nodelist = NULL;
	  s_node->phantom_pcb = pcb;
	  pcb->s_pcb.s_nodelist = pcb->s_pcb.s_nodetail = s_node;
	}
      /* Get a "touched" copy of the current signal queue add counter. */
      s_node->add_listmodnumber = pcb->s_pcb.add_listmodnumber - 1;
      /* Bump the counter of active s_nodes in the s-nodelist of the current process. */
      s_nodelist->curcount++;
      BKL_UNLOCK();
    }
  
  /* If timeout is zero then check the queue but don't wait. */
  if(!timeout)
    {
      LOCK();
      zzcur_p->status = (pr_state_t) (zzcur_p->status | ST_TIMEOUT);
      UNLOCK();
      sig = receive_20(sig_sel);
      LOCK();
      zzcur_p->status = (pr_state_t) (zzcur_p->status & (~ST_TIMEOUT));
      UNLOCK();
      goto done;
    }

  /* If eternal timeout, wait for signal in receive_20(). */
  if(OSE_NO_TIMEOUT == timeout)
    {
      sig = receive_20(sig_sel);
      goto done;
    }
  else
    {
      /* Set up timeout. */
      LOCK();
      request_timeout(zzcur_p, timeout);
      UNLOCK();

      /* Try to receive a signal. */
      sig = receive_20(sig_sel);

      /* Clear the timeout. */
      LOCK();
      cancel_timeout();
      UNLOCK();
    }

  /* Remove the s-nodelist from the s-nodechain of all phantom processes. */
 done:
  for(count = s_nodelist->curcount, s_node = s_nodelist->s_node;
      count; count--, s_node++)
    {
      BKL_LOCK();
      /* Skip inactive s-nodes. (Killed during the call.) */
      pcb = s_node->phantom_pcb;
      if(pcb)
	{
	  /* Unlink the s-node from the chain of each phantom. */
	  /* Unlink at head of list. */
	  if(!s_node->bp_s_nodelist)
	    {
	      pcb->s_pcb.s_nodelist = s_node->fp_s_nodelist;
	      if(s_node->fp_s_nodelist)
		s_node->fp_s_nodelist->bp_s_nodelist = NULL;
	    }
	  /* Unlink within the list. */
	  else
	    {
	      s_node->bp_s_nodelist->fp_s_nodelist = s_node->fp_s_nodelist;
	      if(s_node->fp_s_nodelist)
		s_node->fp_s_nodelist->bp_s_nodelist = s_node->bp_s_nodelist;
	      else
		pcb->s_pcb.s_nodetail = s_node->bp_s_nodelist;
	    }
	  /* Make the s-node inactive by removing the phantom pcb pointer. */
	  s_node->phantom_pcb = NULL;
	}
      BKL_UNLOCK();
    }
  /* Deactivate the s-nodelist. */
  s_nodelist->isactive = 0;

#if OSE_DEBUG > 0
  if(sig == NIL)
    {
       if(zzuu.debugger_enabled)
         {
	   /* Lock the dispatcher. */
	   LOCK_SWAP();

	   pcb = zzcur_p;
  
	   /* check for trace receive_w_tmo request. */
	   if(pcb->debug_flags & 0x80)
	     {
	       zzreceive_tmo_dbghook(pcb_to_debug_pcb(pcb), timeout);
	     }
		
	   /* Unlock the dispatcher. */
	   UNLOCK_SWAP();
	 }
    }
#endif

  /* Return the found signal or NIL on timeout or ignored error. */
  return (OSADDRESS) sig;

  /* This subsection is used when there is a bad pid in the sport_list. */
  bad_sport:
  BKL_UNLOCK();
  os_error(0, OSE_EBAD_PARAMETER | OSE_RECEIVE_SPORT, (OSERRCODE) sport_list);	/* Non fatal. */
  delay(1);
  goto done;
}

/*
** The set_sigsize system call.
**
** This system call attempts to change the size of a signal buffer without
** actually reallocate and copy it.
** This can only be done if the requested size is smaller than the actual
** size of the buffer.
**
** Returns a nonzero value if the operation was successful.
** returns zero if the operation failed.
**
** The system call uses the get_sysconf() framework, which means that
** there is no public entrypoint to a set_sigsize() function.
** The set_sigsize() call uses function 22 of get_sysconf().
** The user level prototype is:
** OSBOOLEAN set_sigsize(union SIGNAL **sig, OSBUFSIZE newsize);
*/

static OSADDRESS
sysconf_22(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  union SIGNAL **sig = (union SIGNAL **) p;
  OSBUFSIZE newsize = (OSBUFSIZE) q;
  register struct _signal *sigadm;
  
  (void) r; /* Remove ``not used arg'' compiler warning. */

  /* Check out buffer pointer and get the signal control block. */
  sigadm = check_buffer(sig, 0);
  if(!sigadm)
    {
      return (OSBUFSIZE) 0;
    }

  /* Abort if no space in the buffer. */
  if(sigadm->buffer_head->size < newsize)
    return 0;

  /* Update the end mark. */
#if ERROR_CHECKS > 0
  *(((U8 *) *sig) + newsize) = (U8) END_MARK;
#endif

  /* Set the new size field. */
  sigadm->size = newsize;

  /* The operation was succesful. */
  return (OSADDRESS) 1;
}

/*
*****************************************************************************
** 17 ATTACH/DETACH.
*****************************************************************************
*/

/*
** The global list of attached signals consists of an array of records,
** the attach pool.
** The size of this array determines the maximum number of attached
** signals that can be handled simultaneously.
** The absolute maximum size of the array is 65536 entries.
**
** Unused records in the array are stored in a linked list from which
** records are allocated when an attach request is issued by an
** application.
**
** Records can be in one of three states: attached, expired and free.
**
** By setting the own_bg field in an attach entry to a magic value we
** can distinguish attached from expired entries.
**
** The free_attach() routine is used to move attach entries from the
** expired list too the free list.
**
** It is important that free entries are not quickly reused otherwise
** the kernel may not be able to issue the correct error code (see:
** OSE_EDETACHED_TOO_LATE in ose_err.h).
**
** We currently allow some slack in the reusage cycle by entering
** expired entries at the tail of the free list and allocating new
** entries from the head.
**
** This works only if there are always a reasonable number of free entries.
**
** Each process control block carries a list of signals attached to
** the process and by the process. Thus, each attach entry is linked
** into two pcb's. The callers and the "victims". The lists are doubly
** linked so that any entry can be removed at any time.
*/

/*
**===========================================================================
** 17.1  Initialization.
**===========================================================================
*/

/*
** System start function to format the attach pool and create the
** free list of attach records.
*/

static void
init_attach(void)
{
  OSADDRESS i;

  /* Initialize free attach list structure. */
  zzuu.free_attach_head = NULL;
  zzuu.free_attach_tail = NULL;
  zzuu.next_free_attach = 0;
  zzuu.free_attach_count = 0;
  zzuu.expired_attach_head = NULL;
  zzuu.free_attach_agent = 0;

  /* Initialize all attach entries and build free list. */
  for(i = MAX_ATTACH; i; i--)
    {
      zzuu.attach_pool[i - 1] = NULL;
    }
}

/*
**===========================================================================
** 17.2  Local functions.
**===========================================================================
*/
#ifndef ASSERT
#define assert_attach_free_list(line)
#else

static void
ramlog_print_attcb(struct _attach_entry *attcb)
{
  ramlog_printf("POSE: attcb: 0x%08lx\n", attcb);
  ramlog_printf("  own_fq:    0x%08lx\n", attcb->own_fq);
  ramlog_printf("  own_bq:    0x%08lx\n", attcb->own_bq);
  ramlog_printf("  vic_fq:    0x%08lx\n", attcb->vic_fq);
  ramlog_printf("  vic_bq:    0x%08lx\n", attcb->vic_bq);
  ramlog_printf("  sigadm:    0x%08lx\n", attcb->sig);
  ramlog_printf("  owner:     0x%08lx\n", attcb->owner);
  ramlog_printf("  victim:    0x%08lx\n", attcb->victim);
  ramlog_printf("  attrid:    0x%04x\n",  attcb->attrid);
  ramlog_printf("  instance:  0x%04x\n",  attcb->instance);
}

static char assert_attach_stamp[0x10000]; /* Used by assert_attach_free_list() */

static void
assert_attach_free_list(int line)
{
  LOCKSTATE msr;
  unsigned ix;
  unsigned expired = 0, freed = 0, errors = 0;
  struct _attach_entry *attcb;

  BKL_LOCK_SAVE(msr);

  assert_from(zzuu.next_free_attach <= MAX_ATTACH, line);
  assert_from(zzuu.free_attach_count <= zzuu.next_free_attach, line);
  if(zzuu.free_attach_count)
    {
      assert_from(zzuu.free_attach_head, line);
      assert_from(zzuu.free_attach_tail, line);

      for(ix = 0; ix < zzuu.next_free_attach; ix++)
	assert_attach_stamp[ix] = 0;

      /* Assert all attcb->attrid members */
      for(ix = 0; ix < zzuu.next_free_attach; ix++)
	{
	  if(zzuu.attach_pool[ix])
	    assert_from(zzuu.attach_pool[ix]->attrid == ix, line);
	} /* for */

      /* Assert the free list */
      for(attcb = zzuu.free_attach_head; attcb; attcb = attcb->own_fq)
	{
	  assert_from(assert_attach_stamp[attcb->attrid] == 0, line);
	  assert_attach_stamp[attcb->attrid] = 1;
#if 1
	  if(((((OSADDRESS)attcb->own_bq) ^ (OSADDRESS)MAGIC_DETACHED_ATTREF)
	      & ~0xf0) != 0)
	    {
	      ramlog_printf("POSE: Strange own_bq in free list, line %d\n",
			    __LINE__);
	      ramlog_print_attcb(attcb);
	      errors++;
	    }
#else
	  assert_from(((((OSADDRESS)attcb->own_bq) ^ (OSADDRESS)MAGIC_DETACHED_ATTREF) & ~0xf0) == 0, line);
#endif
	  freed++;
	} /* for */
      assert_from(freed == zzuu.free_attach_count, line);
      assert_from(zzuu.free_attach_tail->own_fq == NULL, line);
      assert_from(assert_attach_stamp[zzuu.free_attach_tail->attrid] == 1, line);

      /* Assert the expired list */
      for(attcb = zzuu.expired_attach_head; attcb; attcb = attcb->own_fq)
	{
	  assert_from(assert_attach_stamp[attcb->attrid] == 0, line);
	  assert_attach_stamp[attcb->attrid] = 2;
	  assert_from(((((OSADDRESS)attcb->own_bq) ^ (OSADDRESS)MAGIC_DETACHED_ATTREF) & ~0xf0) == 0, line);
	  expired++;
	} /* for */
    }
  else
    {
      assert_from(zzuu.free_attach_head == NULL, line);
    }

  if(errors)
    {
      ramlog_printf("POSE: Attach errors: %d\n", errors);
      assert(0);
    }
  BKL_LOCK_RESTORE(msr); 
}
#endif

/*
** Allocate an attach_entry.
**
** For use by non-interrupt processeses. The dispatcher shall be locked.
** Subfunction to make_old_attref and attach.
*/

static struct _attach_entry *
alloc_attach_entry(void)
{
  struct _attach_entry *attcb;

  BKL_LOCK();
      
  /*
  ** Create another attach_entry if free list is too small and there
  ** is room.
  */
  if(zzuu.free_attach_count < MIN_ATTACHED &&
     zzuu.next_free_attach < MAX_ATTACH)
    {
      BKL_UNLOCK();

      /*
      ** We don't want to increase zzuu.next_free_attach before allocating
      ** space for the attach entry since we might successfully kill ourself
      ** in pool_brk_lo.
      */
      attcb = (struct _attach_entry *)
	pool_brk_lo(&system_pool, sizeof(struct _attach_entry));

      BKL_LOCK();
      if(zzuu.next_free_attach < MAX_ATTACH)
	{
	  OSADDRESS my_attach_index = zzuu.next_free_attach++;
	  
	  zzuu.attach_pool[my_attach_index] = attcb;
	  attcb->attrid = (U16) my_attach_index;
	  attcb->instance = (U16) 1;
	  BKL_UNLOCK();
	  return attcb;
	}
    }

  if(!zzuu.free_attach_head)
    {
      BKL_UNLOCK();

      /* Try to free one attach entry. */
      attcb = free_attach(2);
      
      /* Return immediately if we got one. */
      if (attcb != NULL)
	return attcb;

      /* Unlock the dispatcher. */
      UNLOCK_SWAP();

      os_error(1, OSE_ETOO_MANY_ATTACHED | OSE_ATTACH, (OSERRCODE) MAX_ATTACH);	/* Non fatal system error. */
      for(;;)
	{
	  os_error(2, OSE_ETOO_MANY_ATTACHED | OSE_ATTACH, (OSERRCODE) MAX_ATTACH); /* Fatal. */
	  /* NOTREACHED */
	}
    }
  else
    {
      /* Allocate an attach entry and tick instance no. */
      attcb = zzuu.free_attach_head;
      zzuu.free_attach_count--;
      zzuu.free_attach_head = zzuu.free_attach_head->own_fq;
      attcb->instance++;
      BKL_UNLOCK();
    }
  return attcb;
}

/*
** Allocate an attach_entry.
**
** For use by interrupt processeses.
** Subfunction to make_old_attref and attach.
*/

static struct _attach_entry *
ialloc_attach_entry(void)
{
  struct _attach_entry *attcb;
  LOCKSTATE msr;

  BKL_LOCK_SAVE(msr);
      
  /* Create another attach_entry if free list is empty. */
  if(unlikely(!zzuu.free_attach_head))
    {
      /* But only if there is room for it. */
      if(zzuu.next_free_attach < MAX_ATTACH)
	{
	  OSADDRESS my_attach_index = zzuu.next_free_attach++;
	  
	  BKL_LOCK_RESTORE(msr);

	  attcb = zzuu.attach_pool[my_attach_index] = (struct _attach_entry *)
	    pool_brk_lo(&system_pool, sizeof(struct _attach_entry));
	  attcb->attrid = (U16) my_attach_index;
	  attcb->instance = (U16) 1;
	}
      else
	{
	  BKL_LOCK_RESTORE(msr);

	  /* Try to free one attach entry. */
	  attcb = free_attach(2);
      
	  /* Return immediately if we got one. */
	  if (attcb != NULL)
	    return attcb;

	  /*
	  ** An interrupt process can not call kill_proc() so this is
	  ** a fatal system error.
	  */
	  os_error(2, OSE_ETOO_MANY_ATTACHED | OSE_ATTACH, (OSERRCODE) MAX_ATTACH); /* Fatal. */
	  /* NOTREACHED */
	}
    }
  else
    {
      /* Allocate an attach entry and tick instance no. */
      attcb = zzuu.free_attach_head;
      zzuu.free_attach_count--;
      zzuu.free_attach_head = zzuu.free_attach_head->own_fq;
      attcb->instance++;
      BKL_LOCK_RESTORE(msr);
    }
  return attcb;
}

/*
** Subfunction to attach.
**
** This function is called when a request is made to attach to a
** killed or invalid process.
** In this case, the attached signal is immediately sent to the
** caller and an expired osattref value is returned to the caller.
** The sigadm parameter may be NULL.
**
** The dispatcher can not be locked when this function is called.
** The caller has to NIL a signal pointer in user space.
*/

static OSATTREF
make_old_attref(struct _signal *sigadm, PROCESS pid)
{
  OSATTREF ref;
  struct _attach_entry *attcb;
  PROCESS curr = zzcur_p->pid;
  LOCKSTATE msr;
  int i_am_an_int_proc;

  /* Allocate a signal if required. */
  if(!sigadm)
    {
      sigadm = sig_to_adm(alloc(sizeof(SIGSELECT), OS_ATTACH_SIG));
    }

  i_am_an_int_proc = zzcur_p->priority > 32;

  if(i_am_an_int_proc)
    {
      attcb = ialloc_attach_entry();
    }
  else
    {
      /*
      ** To avoid allocation of all available attcb in a static system
      ** (where free_attach() is never called with a zero argument),
      ** we first try to get an expired attcb.
      */
      attcb = free_attach(1);

      /* Lock the dispatcher. */
      LOCK_SWAP();
      if(!attcb)
	attcb = alloc_attach_entry();
    } /* End of non-interrupt version */

 
  /* Calculate the resulting user reference value. */
  ref = (OSATTREF) ((( (U32) attcb->instance) << 16) | (attcb->attrid));

  /* Fill attach entry info needed by free_attach(). */
  attcb->owner = curr;
  attcb->victim = pid;

  /* Store the signal used in the expired attach entry. */
  attcb->sig = sigadm;

  /*
  ** Send the signal to the caller.
  ** This is done with a safe-send which don't touch userspace.
  */
  safe_send_w_s(sigadm, pid, curr, 1ul, /* No redirection */
		NULL, NULL);

  /* Expire the attach entry (but do not free it). */
  attcb->own_bq = MAGIC_EXPIRED_SENT_ATTREF;

  BKL_LOCK_SAVE(msr);
  attcb->own_fq = zzuu.expired_attach_head;
  zzuu.expired_attach_head = attcb;

  BKL_LOCK_RESTORE(msr);

  if(!i_am_an_int_proc)
    UNLOCK_SWAP();

  assert_attach_free_list(__LINE__);
  return ref;
}

/*
** Subfunction to detach().
**
** Scan the signal queue of the calling proces and unlink the
** attached signal indirectly specified by the attcb parameter.
**
** Returns nonzero if the signal could not be found (error).
** Returns zero if OK.
**
** The dispatcher must be locked since the flush() system call
** might otherwise interfere with the operation.
*/

static OSBOOLEAN
unlink_attached(struct _attach_entry *attcb)
{
  LOCKSTATE msr;
  register struct _signal *prevbuf;
  register struct _signal *thisbuf;
  register struct _signal *sig;

  /* Get signal adm pointer of signal to be located in the queue. */
  sig = attcb->sig;

  /* Get the head of the callers signal queue. */
  prevbuf = zzcur_p->mailbox;
	
  /* Error if no buffer in the sig queue. */
  if (!prevbuf)
    {
      return (OSBOOLEAN) 1;
    }

  /* Check first buffer. */
  if(sig == prevbuf)
    {
      /* Check if the attach signal has been received and then reused. */
      if(sig->sender_id != attcb->victim)
	{
	  return 2;		/* reused attach signal */
	}
      
      /* Unlink found signal and return. */
      LOCK_SAVE(msr);
      zzcur_p->mailbox = prevbuf->next;
      zzcur_p->sig_cnt_queued--;
      LOCK_RESTORE(msr);
      return (OSBOOLEAN) 0;
    }
	
  /* Check second and following buffers. */
  thisbuf = prevbuf->next;
  for(;;)
    {
      if(!thisbuf)
	return (OSBOOLEAN) 1;
      if(sig == thisbuf)
	{
	  /* Check if the attach signal has been received and then reused. */
	  if(sig->sender_id != attcb->victim)
	    {
	      return 2;		/* reused attach signal */
	    }
      
	  /* Unlink and return signal. */
	  LOCK_SAVE(msr);
	  prevbuf->next = thisbuf->next;
	  if(!thisbuf->next)
	    {
	      zzcur_p->mailtail = prevbuf;
	    }
	  zzcur_p->sig_cnt_queued--;
	  LOCK_RESTORE(msr);
	  return (OSBOOLEAN) 0;
	}
      prevbuf = thisbuf;
      thisbuf = thisbuf->next;
    }
}    

/*
** Helper to kill_attach(). This function is a call-back from safe_send_w_s().
** It returns zero iff the signal shall be sent, non-zero if it should be
** freed instead.
** Interrupts are disabled during the call.
*/

struct send_attach_info
{
  struct _attach_entry *attcb;
  U16 instance;			/* The attcb instance. */
};

static OSBOOLEAN
discard_attach_sig(struct _signal *sigadm, void *discard_info)
{
  struct smp_meta_attach *meta_attach = (struct smp_meta_attach*)discard_info;

  SMP_CODE(assert(zzdispatcher_locked));

  /*
  ** Drop the signal if the owner has detached it or if the attcb has been
  ** freed and now is allocated for another attach user. (The later also
  ** implies that the client did a detach. Dead clients are already
  ** sorted out by send().)
  */
  if(meta_attach->attcb->own_bq != MAGIC_EXPIRED_UNSENT_ATTREF ||
     meta_attach->attcb->instance != meta_attach->instance)
    {
      return 1;
    }

  /* Put the signal into the attcb for use by detach. */
  meta_attach->attcb->sig = sigadm;
  meta_attach->attcb->own_bq = MAGIC_EXPIRED_SENT_ATTREF;
  return 0;
}

/*
** Send the attached signal to the owner.
**
** Helper for kill_attach() and sysd_lite.
**
** When called from sysd, via kill_attach(), sigr is a plain struct pointer.
** When called by sysd_lite, the sigr struct is a signal pointer, to be sent
** back to sysd when ready.
**
** The attach signal is sent with a safe-send which doesn't touch 
** userspace.
** Note that the owner may have detached from the attach signal
** before we get a chance to send it. This is handled by the
** discard_attach_sig() callback. This requires that the send is
** performed on the proper SMP cpu.
**
** If the owner is dead, we could take a shortcut, but the send call
** will also sort this out.
*/


#ifdef OSE_SMP
static void
send_attach_signal(struct smp_meta_attach *sigr)
{
  struct _attach_entry *attcb = sigr->attcb;
  struct _pcb *pcb = pid_to_pcb(sigr->owner);
  cpuid_t cpu_id;
  
  /* Inhibit migration, detach and kill_proc. */
  LOCK_SWAP();

  /* Decide if the attach signal shall be handled on another cpu. */
  cpu_id = pcb->cpu_id;
  if(cpu_id != zzkrn_cpu_id())
    {
      if(pcb_to_pid(zzcur_p) == zzuu.sysd_)
	{
	  /*
	  ** The following code is only run by sysd (on cpu 0).  The
	  ** sigr is a plain struct pointer (see kill_attach), so we
	  ** need to transform it into a signal before we can send it.
	  */
	  static const SIGSELECT sel_meta_attach[] = {1, SMP_META_ATTACH};
	  union SIGNAL *tmpsig;

	  /*
	  ** If sysd allocated the attach signal (an OS_ATTACH_SIG),
	  ** make the system owner of it. This will make the job
	  ** easier for the other cpu, because otherwise it whould
	  ** have to issue a change_sig_owner() to steal it from sysd_
	  ** which whould require an ipi transaction.
	  */
	  if(sigr->att_sig->owner == zzcur_p->pindex)
	    change_sig_owner(sigr->att_sig, NULL);

	  UNLOCK_SWAP();

	  /* Check if the owner is still alive. */
	  if(pcb_to_instance(pcb) == pid_to_instance(sigr->owner))
	    {
	      /* Yes it is, so talk to sysd_lite on that cpu. */
	      tmpsig = alloc(sizeof(struct smp_meta_attach), SMP_META_ATTACH);
	      memcpy(tmpsig, sigr, sizeof(struct smp_meta_attach));
	      send(&tmpsig, zzcpu[cpu_id].sysd_lite_);
	      tmpsig = receive(sel_meta_attach);
	      free_buf(&tmpsig);
	    }
	  else
	    {
	      /*
	      ** No it is dead and we don't want to occupy valuable
	      ** ipi bandwidth. If the system is the owner of the
	      ** signal (an OS_ATTACH_SIG) we free it here, if the
	      ** signal is owned by the attacher it will be freed when
	      ** sysd garbage collects the attacher's pool.
	      */
	      if(sigr->att_sig->owner == 0)
		{
		  change_sig_owner(sigr->att_sig, zzcur_p);
		  safe_free_buf(sigr->att_sig);
		}
	    }
	}
      else
	{
	  /*
	  ** The owner process has moved, so we must forward the
	  ** signal so it can be handled on the right cpu.
	  */
	  UNLOCK_SWAP();
	  (void)zzkrn_send_w_sender((union SIGNAL **)&sigr, zzuu.sysd_,
				    zzcpu[cpu_id].sysd_lite_);
	}
      return;
    }

  /*
  ** We run on the proper cpu, the same as the owner, or the owner is dead.
  ** Claim the attach signal; this makes no harm if the owner is dead,
  ** because it is not yet freed by sysd.
  */
  LOCK();
  change_sig_owner(sigr->att_sig, zzcur_p);

  /*
  ** Drop the signal if the owner has detached it or if the attcb has been
  ** freed and now is allocated for another attach user. 
  */
  if(attcb->own_bq != MAGIC_EXPIRED_UNSENT_ATTREF ||
     attcb->instance != sigr->instance)
    {
      LOCKED_UNLOCK_SWAP();
      safe_free_buf(sigr->att_sig);
    }
  else
    {
      /*
      ** The attach owner must not be migrated here, so we send the signal
      ** with the dispatcher locked.
      */
      UNLOCK();
      safe_send_w_s(sigr->att_sig, sigr->victim, sigr->owner,
		    1ul, /* No redirection */
		    discard_attach_sig, sigr);
      UNLOCK_SWAP();
    }

  /* Tell sysd to proceede. */
  if(pcb_to_pid(zzcur_p) != zzuu.sysd_)
    send((union SIGNAL **)&sigr, zzuu.sysd_);
}

#else
/* The simple NON_SMP case */
static void
send_attach_signal(struct smp_meta_attach *sigr)
{
  struct _attach_entry *attcb = sigr->attcb;
  
  LOCK();
  if(sigr->att_sig->owner != zzcur_p->pindex)
    change_sig_owner(sigr->att_sig, zzcur_p);

  if(attcb->own_bq != MAGIC_EXPIRED_UNSENT_ATTREF ||
     attcb->instance != sigr->instance)
    {
      /* Detached by the owner. Free the attach signal. */
      UNLOCK();
      safe_free_buf(sigr->att_sig);
    }
  else
    {
      UNLOCK();
      safe_send_w_s(sigr->att_sig, attcb->victim, attcb->owner,
		    1ul, /* No redirection */
		    discard_attach_sig, sigr);
    }
}
#endif	/* OSE_SMP */

/*
** Return attached signals to their owners when a process
** or block is killed.
**
** This is a utility called from sysd.
** The pcb or bcb of the killed process or block contains a list
** of attached signals related to the killed process.
** This list contains signals attached to the killed process and
** signals attached by the killed process.
** Signals attached by the killed process are freed.
** Signals attached to the killed process are sent back to
** their resp owners.
**
** The dispatcher should NOT be locked by the caller.
*/

static void
kill_attach(PROCESS killed)
{
  register struct _attach_entry *attcb;
  register struct _attach_entry *next_attcb;
  register PROCESS killed_pid;
  register struct _pcb *killed_pcb;
  register struct _pcb *victim;
  register struct _pcb *owner;
  struct _signal *att_sig;

  assert(!zzdispatcher_locked);

  /* Locate the killed block or process descriptor. */
  killed_pid = killed;
  killed_pcb = pid_to_pcb(killed_pid);

  /* Scan the list of attached signals. */
  for(;;)
    {
      /* Lock out other processes from doing attach and detach. */
      BKL_LOCK();

      attcb = killed_pcb->attached;

      if(!attcb)
	{
	  BKL_UNLOCK();
	  assert_attach_free_list(__LINE__);
	  return;
	}

      /* If the owner was killed. */
      if(killed_pid == attcb->owner)
	{
	  /* Update attach list in the owners pcb. */
	  killed_pcb->attached = next_attcb = attcb->own_fq;
	  if(next_attcb)
	    {
	      if(next_attcb->owner == killed_pid)
		next_attcb->own_bq = NULL;
	      else
		next_attcb->vic_bq = NULL;
	    }

	  /* Update attach list in the victims pcb. */
	  victim = pid_to_pcb(attcb->victim);
	  if(victim->attached == attcb)
	      victim->attached = attcb->vic_fq;

	  /* Unlink the descriptor from the pcb of the target. */
	  if(attcb->vic_bq)
	    {
	    if(attcb->vic_bq->victim == attcb->victim)
		attcb->vic_bq->vic_fq = attcb->vic_fq;
	    else
		attcb->vic_bq->own_fq = attcb->vic_fq;
	    }

	  if(attcb->vic_fq)
	    {
	    if(attcb->vic_fq->victim == attcb->victim)
		attcb->vic_fq->vic_bq = attcb->vic_bq;
	    else
		attcb->vic_fq->own_bq = attcb->vic_bq;
	    }

	  /* Remember the signal and owner */
	  att_sig = attcb->sig;
	  owner = pid_to_pcb(attcb->owner);

	  /*
	  ** Expire the attach entry and put it at tail of fifo free list.
	  ** Protect this from allocation from the free list by other
	  ** processes.
	  */
	  attcb->own_bq = MAGIC_DETACHED_ATTREF;
	  attcb->own_fq = NULL;

	  if(zzuu.free_attach_head)
	    {
	      zzuu.free_attach_tail->own_fq = attcb;
	    }
	  else
	    {
	      zzuu.free_attach_head = attcb;
	    }
	  zzuu.free_attach_tail = attcb;
	  zzuu.free_attach_count++;
	  BKL_UNLOCK();

	  /*
	  ** Free the attached signal if present.
	  ** Not really needed, but may speed up collect_pool().
	  ** But first check that the attach owner really still owns the
	  ** signal, and is not trying to deceive us.
	  */
	  if(att_sig)
	    {
	      assert(att_sig->owner == pcb_to_pindex(owner));

	      if(att_sig->owner == pcb_to_pindex(owner))
		{
		  /*
		  ** The owner is dead, so noone else will update his pcb.
		  ** Steal the signal.
		  */
		  LOCK();
		  owner->sig_cnt_owned--;
		  zzcur_p->sig_cnt_owned++;
		  att_sig->owner = pcb_to_pindex(zzcur_p);
		  UNLOCK();
	      
		  safe_free_buf(att_sig);
		}
	    }
	}
      /* If the target was killed. */
      else
	{
	  /* Update attach list in the targets pcb. */
	  killed_pcb->attached = next_attcb = attcb->vic_fq;
	  if(next_attcb)
	    {
	      if(next_attcb->victim == killed_pid)
		next_attcb->vic_bq = NULL;
	    else
		next_attcb->own_bq = NULL;
	    }

	  /* Update attach list in the owners pcb. */
	  owner = pid_to_pcb(attcb->owner);
	  if(owner->attached == attcb)
	    owner->attached = attcb->own_fq;

	  /* Unlink the descriptor from the pcb of the owner. */
	  if(attcb->own_bq)
	    {
	      if(attcb->own_bq->owner == attcb->owner)
		attcb->own_bq->own_fq = attcb->own_fq;
	      else
		attcb->own_bq->vic_fq = attcb->own_fq;
	    }
	  
	  if(attcb->own_fq)
	    {
	      if(attcb->own_fq->owner == attcb->owner)
		attcb->own_fq->own_bq = attcb->own_bq;
	      else
		attcb->own_fq->vic_bq = attcb->own_bq;
	    }

	  /* Expire the attach entry (but do not free it). */
	  attcb->own_bq = MAGIC_EXPIRED_UNSENT_ATTREF;
	  attcb->own_fq = zzuu.expired_attach_head;
	  zzuu.expired_attach_head = attcb;

	  /*
	  ** We must save some info from the attach control block in case
	  ** an owner of interrupt process type interrupts us and detaches
	  ** it before we send the signal.
	  */
	  {
	    struct smp_meta_attach meta_attach;

	    meta_attach.sig_no = SMP_META_ATTACH;
	    meta_attach.attcb = attcb;
	    meta_attach.att_sig = attcb->sig;
	    meta_attach.owner = attcb->owner;
	    meta_attach.victim = attcb->victim;
	    meta_attach.instance = attcb->instance;
	    
	    BKL_UNLOCK();
	    /*
	    ** If the client didn't provide a signal, or is trying to deceive
	    ** us by freeing the signal, then send and OS_ATTACH_SIG.
	    */
	    if(meta_attach.att_sig == NULL ||
	       meta_attach.att_sig->owner != pid_to_pindex(meta_attach.owner))
	      {
		meta_attach.att_sig =
		  sig_to_adm(alloc(sizeof(SIGSELECT), OS_ATTACH_SIG));
	      }
	  
	    /* Send the attach signal, if attcb not detached or reused by now. */
	    send_attach_signal(&meta_attach);
	  }
	}
    } /* for(;;) */
}

/*
** Move attach control blocks from the list of expired attach entries
** to the fifo free list.
**
** If the 'give_me_first_free' argument is non zero we abandon the
** execution at the first reusable entry we find and return that entry
** without moving it to the free list, but with an incremented instance.
**
** If the 'give_me_first_free' argument is less than 2, we will have
** certain restrictions on how many loops/retries we will tolerate
** before we give up, this will make this function behave more nice in
** non critical situations.
**
** This is a utility called with a zero as argument from within the
** sysd and huntd daemons. It is called with a non zero argument from
** the alloc_attach_entry functions.
**
** It is safe to call this function from an interrupt process:
**
** The real trick of this routine lays in looking at an expired attach
** entry and realizing that it can be freed. (See more about this
** below.)
*/

static struct _attach_entry *
free_attach(OSBOOLEAN give_me_first_free)
{
  LOCKSTATE msr;
  PROCESS curr = zzcur_p->pid;
  struct _attach_entry *attcb;

  /* Execution restrictions used when (give_me_first_free < 2). */
  int restarts = 3;
  int loop_max = 1024;

  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);

 restart:

  /*
  ** Check if we really need an attach entry quick and someone else
  ** has put one on the free list.
  */
  if((give_me_first_free == 2) && zzuu.free_attach_head)
    {
    alloc_from_free_list:
       /* Allocate an attach entry and tick instance no. */
      attcb = zzuu.free_attach_head;
      zzuu.free_attach_count--;
      zzuu.free_attach_head = zzuu.free_attach_head->own_fq;
      attcb->instance++;
      BKL_LOCK_RESTORE(msr);
      assert_attach_free_list(__LINE__);
      return attcb;
    }

  /* Tell the world that we are starting to process the expired list. */
  zzuu.free_attach_agent = curr;

  /* Pick up the first entry in the expired list. */
  attcb = zzuu.expired_attach_head;
  
  while (attcb && attcb->own_fq)
    {
      struct _attach_entry *free_ref = attcb->own_fq;
      struct _signal       *sigadm   = free_ref->sig;
      PROCESS               owner    = free_ref->owner;

      /*
      ** We now do some checks to see if we can move the attach entry
      ** from the expired list to the free list.
      **   0. The owner has detached it. Perhaps no signal was allocated.
      **   1. Owner is dead. It's important that this test is made before
      **      any sigadm tests, since we don't want to touch the signal
      **      administration block if the pool is gone, which can otherwise
      **      happen in memory model 0 systems.
      **   2. If the attach has expired, but the signal is not yet sent,
      **      then keep the expired attref. The signal is perhaps not
      **      allocated yet, so we must check this before the sigadm.
      **      Otherwise, if any of the following (3-5) is true, move it 
      **      to the free list.
      **   3. The sender_id field has changed so the signal must clearly
      **      have been received (or freed due to killing or flushing).
      **   4. The owner field is not zero so it's not laying around in
      **      the signal queue, which implies that it's been received.
      **   5. If the signal is free.
      */
      if(free_ref->own_bq == MAGIC_DETACHED_ATTREF ||               /* 0 */
	 pcb_to_pid(pid_to_pcb(owner)) != owner ||                  /* 1 */
	 (free_ref->own_bq != MAGIC_EXPIRED_UNSENT_ATTREF &&        /* 2 */
	  (sigadm->sender_id != free_ref->victim ||                 /* 3 */
	   sigadm->owner != 0 ||                                    /* 4 */
	   (sigadm->dest_id != 0 &&                                 /* 5 */
	    pid_to_pcb(sigadm->dest_id)->magic == MAGIC_POOL))))
	{
	  /*
	  ** It appears to be safe to move the attach entry to the
	  ** free list.
	  */
	  
	  /* Remove attach entry from expired list.*/
	  attcb->own_fq = free_ref->own_fq;

	  /*
	  ** Return immediately if we were called from within
	  ** an alloc_attach_entry function.
	  */
	  if(give_me_first_free)
	  {
	    free_ref->instance++;
	    BKL_LOCK_RESTORE(msr);
	    return free_ref;
	  }

	  /* Put attach entry in fifo free list. */
	  free_ref->own_fq = NULL;
	  if(zzuu.free_attach_head)
	    {
	      zzuu.free_attach_tail->own_fq = free_ref;
	      zzuu.free_attach_tail = free_ref;
	    }
	  else
	    {
	      zzuu.free_attach_head = zzuu.free_attach_tail = free_ref;
	    }
	  zzuu.free_attach_count++;

	  /* Enable interrupts. */
	  BKL_LOCK_RESTORE(msr);

	  /*
	  ** We can now be preempted by a higher prioritized process
	  ** issuing a free_attach() call.
	  */
	  
	  /* Disable interrupts. */
	  BKL_LOCK_SAVE(msr);

	  /* Check if we where preempted by another "free attach agent". */
	  if(zzuu.free_attach_agent != curr)
	    {
	      /* Check is we should give up. */
	      if((give_me_first_free < 2) && (restarts-- == 0))
		break;
	      goto restart;
	    }

	  /*
	  ** Since we have just removed an element, we do not need to
	  ** move forward in the list here.
	  */
	}
      else
	{
	  /* Enable interrupts. */
	  BKL_LOCK_RESTORE(msr);

	  /*
	  ** We can now be preempted by a higher prioritized process
	  ** issuing a free_attach() call.
	  */

	  /* Disable interrupts. */
	  BKL_LOCK_SAVE(msr);

	  /* Check if we where preempted by another "free attach agent". */
	  if(zzuu.free_attach_agent != curr)
	    {
	      /* Check is we should give up. */
	      if((give_me_first_free < 2) && (restarts-- == 0))
		break;
	      goto restart;
	    }

	  /* Move forward one list item. */
	  attcb = attcb->own_fq;
	}

      /* Check if it's time to stop garbage collecting for now. */
      if((give_me_first_free < 2) && (--loop_max == 0))
	break;
    }

  /*
  ** Check if we need an attach entry and someone else has put one on
  ** the free list.
  */
  if((give_me_first_free != 0) && zzuu.free_attach_head)
    goto alloc_from_free_list;

  /* Enable interrupts. */
  BKL_LOCK_RESTORE(msr);
  assert_attach_free_list(__LINE__);
  return NULL;
}

/*
**===========================================================================
** 17.3  System calls.
**===========================================================================
*/

/*
** The attach system call.
*/

OSATTREF
attach(union SIGNAL **sig, PROCESS pid)
{
  OSATTREF ref;
  register struct _attach_entry *attcb;
  register struct _pcb *victim;
  register struct _pcb *owner = zzcur_p;
  struct _signal *sigadm = NULL;
  LOCKSTATE msr;
  int i_am_an_int_proc;

#if ERROR_CHECKS > 0
  /* Check out the buffer pointer. */
  ON_ERRLEVEL_2()
    {
      if(sig && !check_buffer(sig, OSE_ATTACH))
	{
	  return (OSATTREF) 0;
	}
    }

  /*
  ** Ensure that we don't try to attach to ourself (the caller).
  ** This would cause the kernel to crash.
  */
  if(unlikely(owner->pid == pid))
    {
      os_error(0, OSE_EATTACHED_TO_CALLER | OSE_ATTACH, (OSERRCODE) 0);	/* Non fatal. */
      if(sig)
	free_buf(sig);
      return (OSATTREF) 0;
    }
#endif

  if(sig)
    {
      sigadm = sig_to_adm(*sig);

#if MEMORY_MODEL > 0
      /*
      ** Copy an attach signal in a foreign pool to the system segment.
      ** We do this to get rid of dying pools. An expired and sent attach 
      ** signal may not be relocated to a new address in another pool. That
      ** would cause trouble in detach().
      ** An attach signal in a pool of a foreign segment must have been
      ** allocated by s_alloc_nil or stolen by restore(). This is an odd case.
      ** See also: sysconf_48, sysd_clean_killed_pool.
      */
      if(local_sysctlblk.allow_pool_copy)
	{
	  struct poolcb *pool = sigadm->buffer_head->poolcbp;
      
	  if(pool != owner->block->sigpool && pool != &system_pool &&
	     sigadm->size <= system_pool.signal_list[7].size)
	    {
	      struct _signal *new_sigadm = sig_to_adm(sys_alloc(sigadm->size,
								0));

	      /* Memcpy will not work for MAS pools?? */
	      memcpy(new_sigadm->udata, sigadm->udata, sigadm->size);
	      safe_free_buf(sigadm);
	      sigadm = new_sigadm;
	    }
	}
#endif
      /* Take the signal from the user. */
      *sig = NIL;
    }


  /* Locate the object descriptor of the victim, dead or alive. */
  victim = safe_pid_to_pcb(pid, 2);
   
  /*
  ** Check if the victim object is a valid sender, dead or alive.
  ** We can safely ignore any user number here.
  */
  if(unlikely(!victim || !pid_to_instance(pid)))
    {
      /* Report an error for this illegal pid. */
      os_error(0, OSE_EILLEGAL_PROCESS_ID | OSE_ATTACH, (OSERRCODE) pid); /* Non fatal. */
      if(sigadm)
	safe_free_buf(sigadm);
      return (OSATTREF) 0;
    }

  if(pcb_to_pid(victim) != pid)
    {
      return make_old_attref(sigadm, pid);
    }

  i_am_an_int_proc = owner->priority > 32;
  if(i_am_an_int_proc)
    {
      /* Interrupt version. */
      attcb = ialloc_attach_entry();
    }
  else
    {				      
      /* Non-interrupt version. Lock the dispatcher. */
      LOCK_SWAP();

      /* Check that the block/process is still present. */
      if(pcb_to_pid(victim) != pid)
	{
	  UNLOCK_SWAP();
	  return make_old_attref(sigadm, pid);
	}

      attcb = alloc_attach_entry();
      
    } /* End of non-interrupt version */

  /* Calculate the resulting user reference value. */
  ref = (OSATTREF) ((( (U32) attcb->instance) << 16) | (attcb->attrid));

  /* Fill attach entry and make it unexpired. */
  attcb->sig = sigadm;
  attcb->owner = owner->pid;
  attcb->victim = pid;
  attcb->own_bq = NULL;
  attcb->vic_bq = NULL;

  /* Protect linked list operations from interrupt processes doing attach. */
  BKL_LOCK_SAVE(msr);

  /* Insert entry into callers pcb. */
  attcb->own_fq = owner->attached;
  if(attcb->own_fq)
    {
      if(owner->attached->owner == attcb->owner)
	owner->attached->own_bq = attcb;
      else
	owner->attached->vic_bq = attcb;
    }
  owner->attached = attcb;

  /* Insert entry into victims pcb. */
  attcb->vic_fq = victim->attached;
  if(attcb->vic_fq)
    {
      if(victim->attached->victim == attcb->victim)
	victim->attached->vic_bq = attcb;
      else
	victim->attached->own_bq = attcb;
    }
  victim->attached = attcb;

  BKL_LOCK_RESTORE(msr);

  if(!i_am_an_int_proc)
    UNLOCK_SWAP();

  /* Return the reference value. */
  assert_attach_free_list(__LINE__);
  return ref;
}

/*
** The detach system call.
*/

static void
detach_unlock_all(LOCKSTATE msr, struct _pcb *pcb)
{
  BKL_LOCK_RESTORE(msr);
#ifdef OSE_SMP
  if(pcb->priority <= 32)
    UNLOCK_SWAP();
#else
  (void)pcb;
#endif
}

void
detach(OSATTREF *attref)
{
  register struct _attach_entry *attcb;
  register struct _pcb *victim;
  register struct _pcb *owner = zzcur_p;
  U16 attrid = (U16) *attref;
  U16 instance = (U16) (*attref >> 16);
  struct _signal *sig;
  LOCKSTATE msr;

  /* Check attref. Error if illegal. */
  if(unlikely(attrid >= zzuu.next_free_attach))
    {
      os_error(0, OSE_EDETACHED_TWICE | OSE_DETACH, (OSERRCODE) 0);	/* Non fatal. */
      return;
    }

  /*
  ** Check attcb. Error if bad instance no.
  ** This error implies that the attached process has been killed, and
  ** the caller failed to notice this for a period long enough for the
  ** attach entry to be reused.
  ** This is not really a user error, but rather an indication that the number
  ** of attach entries is too small.
  */
  attcb = zzuu.attach_pool[attrid];

  SMP_CODE(if(owner->priority <= 32) LOCK_SWAP());
  BKL_LOCK_SAVE(msr);

  if(unlikely(instance != attcb->instance))
    {
      detach_unlock_all(msr, owner);
      assert_attach_free_list(__LINE__);
      os_error(0, OSE_EDETACHED_TOO_LATE | OSE_DETACH, (OSERRCODE) 0);	/* Non fatal. */
      return;
    }

#if ERROR_CHECKS > 0
  if(unlikely(attcb->owner != owner->pid))
    {
      detach_unlock_all(msr, owner);
      os_error(0, OSE_ENOT_ATTREF_OWNER | OSE_DETACH, attcb->owner);	/* Non fatal. */
      return;
    }
#endif
  
  /* Make the users attcb illegal to detect multiple detach calls. */
  *attref = OSE_ILLEGAL_ATTREF;

  switch((OSADDRESS)attcb->own_bq)
    {
    case (OSADDRESS)MAGIC_DETACHED_ATTREF:
      detach_unlock_all(msr, owner);
      os_error(0, OSE_EDETACHED_TWICE | OSE_DETACH, (OSERRCODE) 0);	/* Non fatal. */
      return;

    case (OSADDRESS)MAGIC_EXPIRED_UNSENT_ATTREF:
      /*
      ** Tell the interrupted daemon (ose_sysd) to discard the signal sending
      ** and just free the signal. Catch a 2nd detach attempt where the user 
      ** has taken a copy of the attref variable.
      */
      attcb->own_bq = MAGIC_DETACHED_ATTREF;
      detach_unlock_all(msr, owner);
      assert_attach_free_list(__LINE__);
      return;

    case (OSADDRESS)MAGIC_EXPIRED_SENT_ATTREF:
      /*
      ** Catch a 2nd detach attempt where the user has taken a copy of the
      ** attref variable.
      */
      attcb->own_bq = MAGIC_DETACHED_ATTREF;
      
      /*
      ** Search the callers signal queue for a signal with the same pointer.
      ** It is important that we really get exactly the signal pointer that
      ** was stored in the attach() call or in the send() call that was
      ** made by the kernel at the time the target process was killed.
      */

      if(owner->priority > 32)
	{
	  /* Interrupt version. */

	  /* Let higher interrupts run. */
	  BKL_LOCK_RESTORE(msr);

	  if(unlikely(unlink_attached(attcb)))
	    {
	      /* Error if not found. */
	      os_error(0, OSE_EDETACH_AFTER_RECEIVE | OSE_DETACH, (OSERRCODE) 0);	/* Non fatal. */
	      return;
	    }

	  /* Get a pointer to the attach signal. */
	  sig = attcb->sig;

	  /* Make us the owner of the signal. */
	  change_sig_owner(sig, owner);
	}
      else
	{
	  /* Prioritized/background version. */

	  /* This should run with DISPATCHER LOCK only! */
#ifndef OSE_SMP
	  LOCK_SWAP();
#endif
	  BKL_LOCK_RESTORE(msr);

	  if(unlikely(unlink_attached(attcb)))
	    {
	      /* Error if not found. */
	      UNLOCK_SWAP();
	      os_error(0, OSE_EDETACH_AFTER_RECEIVE | OSE_DETACH, (OSERRCODE) 0);	/* Non fatal. */
	      return;
	    }

	  /* Get a pointer to the attached signal. */
	  sig = attcb->sig;

	  /* 
	  ** Make us the owner of the signal to prevent memory leak in case we
	  ** are being killed after unlock of the dispatcher.
	  */
	  change_sig_owner(sig, zzcur_p);
	  UNLOCK_SWAP();
	}
      assert_attach_free_list(__LINE__);

      /* Free the signal buffer. */
      safe_free_buf(sig);
      return;

    default:
      /* If not expired then... */

      /* Get the victims pcb pointer. */
      victim = pid_to_pcb(attcb->victim);

      /* Update attach list in the owners pcb. */
      if(owner->attached == attcb)
	owner->attached = attcb->own_fq;

      /* Update attach list in the victims pcb. */
      if(victim->attached == attcb)
	victim->attached = attcb->vic_fq;

      /* Unlink the entry from the callers pcb. */
      if(attcb->own_bq)
	{
	  if(attcb->own_bq->owner == attcb->owner)
	    attcb->own_bq->own_fq = attcb->own_fq;
	  else
	    attcb->own_bq->vic_fq = attcb->own_fq;
	}
      if(attcb->own_fq)
	{
	  if(attcb->own_fq->owner == attcb->owner)
	    attcb->own_fq->own_bq = attcb->own_bq;
	  else
	    attcb->own_fq->vic_bq = attcb->own_bq;
	}

      /* Unlink the entry from the victims pcb. */
      if(attcb->vic_bq)
	{
	  if(attcb->vic_bq->victim == attcb->victim)
	    attcb->vic_bq->vic_fq = attcb->vic_fq;
	  else
	    attcb->vic_bq->own_fq = attcb->vic_fq;
	}
      if(attcb->vic_fq)
	{
	  if(attcb->vic_fq->victim == attcb->victim)
	    attcb->vic_fq->vic_bq = attcb->vic_bq;
	  else
	    attcb->vic_fq->own_bq = attcb->vic_bq;
	}

      /* Get a pointer to the attached signal if any. */
      sig = attcb->sig;

      /* Expire attach entry and put in fifo free list. */
      attcb->own_bq = MAGIC_DETACHED_ATTREF;
      attcb->own_fq = NULL;

      if(zzuu.free_attach_head)
	{
	  zzuu.free_attach_tail->own_fq = attcb;
	  zzuu.free_attach_tail = attcb;
	}
      else
	{
	  zzuu.free_attach_head = zzuu.free_attach_tail = attcb;
	}
      zzuu.free_attach_count++;

      detach_unlock_all(msr, owner);
      assert_attach_free_list(__LINE__);
      
      /* Free signal buffer if present. */
      if(sig)
	safe_free_buf(sig);
      return;
    }
}

/*
**===========================================================================
** 17.4  Undocumented system calls.
**===========================================================================
*/

/*
*****************************************************************************
** 18 HUNT AND ASSOCIATES.
*****************************************************************************
*/

/*
**===========================================================================
** 18.1  Initialization.
**===========================================================================
*/

/*
** Initialize static variables associated with the hunt daemon.
** This includes the process list, the linkhandler list,
** the list of pending hunt requests.
*/

static void
init_hunt(void)
{
  /* Clear the process list. */
  zzuu.pcb_list = NULL;

  /* Clear the linkhandler list. */
  zzuu.linkh_list_T_link_name = NULL;
  zzuu.linkh_list_T_pid = NULL;

  /* Clear the hunt request list. */
  zzuu.hunt_list_T_name = NULL;
  zzuu.hunt_list_T_linkname = NULL;
  zzuu.hunt_list_T_owner = NULL;
  zzuu.hunt_list_T_from = NULL;
}

/*
**===========================================================================
** 18.2  Local functions.
**===========================================================================
*/

/*
** This is a splay tree key comparison utility.
**
** It compares the name field in a process control block against a
** key that is a zero terminated character string.
**
** It returns zero if the key matches the node.
** It returns >0 if the node is less than the key.
** It returns <0 if the node is greater than the key.
*/

static int
splaycmp_pcb_name(treenode *T, KEYTYPE k)
{
  return strcmp((const char *) k, ((struct _pcb *)(T->data))->name);
}

/*
** This is a splay tree key comparison utility.
**
** It compares the name field in a hunt control block against a
** key that is a zero terminated character string.
**
** It returns zero if the key matches the node.
** It returns >0 if the node is less than the key.
** It returns <0 if the node is greater than the key.
*/

static int
splaycmp_hunt_name(treenode *T, KEYTYPE k)
{
  return strcmp((const char *) k, ((struct hunt_request *)(T->data))->name);
}

/*
** This is a splay tree key comparison utility.
**
** It compares the name field in a hunt control block against a
** key that is a character string terminated by a '/' character.
**
** It returns zero if the key matches the node.
** It returns 1 if the node is less than the key.
** It returns -1 if the node is greater than the key.
*/

static int
splaycmp_hunt_linkname(treenode *T, KEYTYPE k)
{
  register const char *p, *q;
  for(p = ((struct hunt_request *)(T->data))->name, q = (const char *) k;
      (*p == *q) && *p && (*p != '/');
      p++, q++)
    {
      ; /* Empty loop body. */
    }
  if(*p != *q)
    {
      if(*p > *q)
	return -1;
      else
	return 1;
    }
  return 0;
}

/*
** This is a splay tree key comparison utility.
**
** It compares the owner field in a hunt control block against a
** key that is a process id.
**
** It returns zero if the key matches the node.
** It returns -1 if the node is less than the key.
** It returns  1 if the node is greater than the key.
*/

static int
splaycmp_hunt_owner(treenode *T, KEYTYPE k)
{
  PROCESS key = ((struct hunt_request *)(T->data))->owner;
  if(((PROCESS) k) != key)
    {
      if(((PROCESS) k) > key)
	return -1;
      else
	return 1;
    }
  return 0;
}

/*
** This is a splay tree key comparison utility.
**
** It compares the from field in a hunt control block against a
** key that is a process id.
**
** It returns zero if the key matches the node.
** It returns -1 if the node is less than the key.
** It returns  1 if the node is greater than the key.
*/

static int
splaycmp_hunt_from(treenode *T, KEYTYPE k)
{
  PROCESS key = ((struct hunt_request *)(T->data))->from;
  if(((PROCESS) k) != key)
    {
      if(((PROCESS) k) > key)
	return -1;
      else
	return 1;
    }
  return 0;
}

/*
** This is a splay tree key comparison utility.
**
** It compares the link_name field in a linkhandler control block against a
** key that is a character string terminated by a '/' character.
**
** It returns zero if the key matches the node.
** It returns 1 if the node is less than the key.
** It returns -1 if the node is greater than the key.
*/

static int
splaycmp_linkh_link_name(treenode *T, KEYTYPE k)
{
  register const char *p, *q;
  for(p = ((struct new_linkhandler *)(T->data))->link_name, q = (const char *) k;
      (*p == *q) && *p && (*p != '/');
      p++, q++)
    {
      ; /* Empty loop body. */
    }
  if(*p != *q)
    {
      if(*p > *q)
	return -1;
      else
	return 1;
    }
  return 0;
}

/*
** This is a splay tree key comparison utility.
**
** It compares the pid field in a linkhandler control block against a
** key that is a process id.
**
** It returns zero if the key matches the node.
** It returns -1 if the node is less than the key.
** It returns  1 if the node is greater than the key.
*/

static int
splaycmp_linkh_owner(treenode *T, KEYTYPE k)
{
  PROCESS key = ((struct new_linkhandler *)(T->data))->new_linkhdlr;
  if(((PROCESS) k) != key)
    {
      if(((PROCESS) k) > key)
	return -1;
      else
	return 1;
    }
  return 0;
}

/*
** Subfunction to the hunt daemon.
**
** This function checks that a longer string actually begins with
** a shorter string.
**
** Returns a nonzero value if there is a match. Returns zero if not.
**
** The function is used to check that a complete linkhandler name actually
** matches the first part of a hunt request.
** We need to do this because the splay tree of linkhandlers is only sorted
** on the characters before the first '/'.
** Since we allow linkhandler names with several '/'-characters, there
** is a possible mismatch.
*/
static int
substr_match(char *substr, char *str)
{
  while(*substr && (*substr == *str))
    {
      substr++; str++;
    }
  return (int) !*substr;
}

/*
** Subfunction to the hunt daemon.
**
** Check that the user numbers of a hunt request match a linkhandler
** so that a hunt request can be sent to the linkhandler.
** Returns a nonzero value if the numbers match.
**
** The linkhandler must either be a block or:
** The linkhandler must be able to see both the user and from fields.
** The linkhandler need not be able to see the owner field.
** Neither the owner nor the from field need be able to see the linkhandler.
** The owner must be able to see the from field.
** The owner need not be able to see the user.
** The from field need not be able to see the owner.
** The from field must be able to see the user.
*/

static int
check_hunt_user(OSUSER user, PROCESS linkh, PROCESS from, PROCESS owner)
{
  struct _pcb *l = pid_to_pcb(linkh);
  struct _pcb *f = pid_to_pcb(from);
  struct _pcb *o = pid_to_pcb(owner);
  int match = 1;

  if ((l->magic & ~MAGIC_KILLED) != MAGIC_BCB)
    {
      /* The linkhandler must be able to see the user field. */
      if(l->real_user && l->real_user != user)
        match = 0;

      /* The linkhandler must be able to see the from field. */
      if(l->real_user && l->real_user != f->real_user && l->block != f->block)
        match = 0;

      /* The owner must be able to see the from field. */
      if(o->real_user && o->real_user != f->real_user && o->block != f->block)
        match = 0;

      /* The from field must be able to see the user. */
      if(f->real_user && f->real_user != user)
        match = 0;
    }
    
  /* OK if all conditions are satisfied. */
  return match;
}

/*
** Subfunction to the hunt daemon.
**
** Search the process list for a process with a matching name and user number.
** The processes in the owner and from fields must both be able to see the found 
** process. The found process need not be able to see either of the from or 
** owner fields.
**
** If not found, then ask all linkhandlers with a user number that matches both
** the user field and the from field in the hunt request.
**
** If the linkhandlers don't know of the process, the hunt request is inserted
** into the list of pending hunt requests.
**
** While the linkhandler call is in progress, new hunt requests are processed
** only if they do not require the service of a linkhandler. Other requests are
** placed in an internal queue for processing when the current linkhandler call
** completes.
**
** ??Is there a deadlock problem here? What if a hunt asks a linkhandler which
** makes another hunt?
** Also in huntd_assign_linkhandler.
*/

static void
huntd_hunt_request(union SIGNAL *sigr)
{
  PROCESS client;
  treenode *T, *T2 = NULL;
  /* Request currently processed by linkhandler. */
  struct hunt_request *current_linkhandler_request = NULL;

  /* List of requests waiting to be sent to linkhandler. */
  struct hunt_request *pending_hunt_request = NULL;
  struct hunt_request *pending_tail = NULL;

  /* Used when attached to linkhandler. */
  OSATTREF attach_id;

  /* Linkhandler currently being asked. */
  struct new_linkhandler *linkh = NULL; 

  static const SIGSELECT sig_sel[] = {4, HUNT_REQUEST,
					REMOTE_RESPONSE,
					REMOTE_CANCEL,
					OS_ATTACH_SIG};

  /* We supervise linkhandler responses and report error if > 10 sec timeout. */
  OSTICK hunt_time = 0;	/* The time when the hunt request was sent. */
  OSTICK hunt_timeout;	/* Number of ticks that corresponds to 10 sec. */

  hunt_timeout = 10000000u / system_tick();	/* This is 10 seconds. */

 search_process_list:
  client = sender(&sigr);

#if 0
  /*
  ** We currently don't support this "./" notation.
  */
  
  /* Handle "./" substitution. */
  if (sigr->hunt_request.name[0] == '.' && sigr->hunt_request.name[1] == '/')
    {
      /* Check if the block of the process is a linkhandler.
      **
      ** Using pcb_to_pid() on the block pointer will result in 
      ** an unexpected pid if the block has been killed. However, 
      ** that should be no problem here since it means that the 
      ** process is dead and no hunt signal is needed.
      */   
      T = splay_find((KEYTYPE) pcb_to_pid(pid_to_pcb(client)->block), 
                     &zzuu.linkh_list_T_pid,
                     splaycmp_linkh_owner);

      /* Substitute with matching entry in the linkhandler list. */
      if (T)
        {
          union SIGNAL *sig; 
          struct new_linkhandler *l = (struct new_linkhandler *)T->data; 
        
          sig = alloc(sigsize(&sigr) - 2 + l->link_name_len, HUNT_REQUEST);
          sig->hunt_request.hsig = sigr->hunt_request.hsig;
          sig->hunt_request.user = sigr->hunt_request.user;
          sig->hunt_request.from = sigr->hunt_request.from;
          sig->hunt_request.owner = sigr->hunt_request.owner;
          strcpy(sig->hunt_request.name, l->link_name);
          strcpy(sig->hunt_request.name + l->link_name_len,
                 sigr->hunt_request.name + 2);
                 
          free_buf(&sigr);
          sigr = sig;
        }
    }
#endif

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the client is still alive. Clean up and exit if not. */
  /* Any hunt signal buffer was reclaimed by the kernel. */
  if(get_ptype(client) == OS_ZOOMBIE)
    {
      /* Unlock the dispatcher. */
      UNLOCK_SWAP();
  
      /* Free the signal from the dead client. */
      free_buf(&sigr);
  
      /* Exit if no linkhandler call is pending. */
      if(!current_linkhandler_request)
	return;

      /* Otherwise wait for linkhandler response. */
      goto receive_again;
    }

  /* Change the ownership of any hunt signal. */
  if(sigr->hunt_request.hsig)
    change_sig_owner(sigr->hunt_request.hsig, zzcur_p);

  /* Unlock the dispatcher. */
  UNLOCK_SWAP();

  /* search the process list for a process with a matching name. */
  T = splay_find((KEYTYPE) sigr->hunt_request.name,
		 &zzuu.pcb_list,
		 splaycmp_pcb_name);

  /* Traverse the list of processes with that name. */
  {
    struct _pcb *found_pcb = NULL;
    int found_class = 0;	/* 2 = in callers bloc, 1 = other block-proc, 0 =  other. */
    struct _block *current_caller = pid_to_pcb(sigr->hunt_request.owner)->block;

    for(; T; T = T->splay_n)
      {
	struct _pcb *pcb = (struct _pcb *) T->data;

	/*
	** We must check the killed flag in each pcb. Some processes in the
        ** pcb list may actually be dead, and since the instance number has
        ** already been ticked for these, we must not report them.
	*/
	if(!pcb->killed)
	  /* Check that the user number of the found process matches request. */
	  /* Is this correctly checked?? */
	  if(sigr->hunt_request.user == pcb->real_user)
	    {
	      /* Check if found in the callers block. */
	      if(pcb->block == current_caller)
		{
		  found_pcb = pcb;
		  found_class = 2;
		}
	      /* Check if we found another block process. */
	      else if(pcb == pcb->block->block_process)
		{
		  if(!found_class)
		    {
                      /* If the block of the process has a block link
                       * assigned, the process should be invisible. It will
                       * only be seen when using the block link prefix. */
                      if(!pcb->block->lnh_name)
                        {                           
                          found_pcb = pcb;
                          found_class = 1;
                        }
		    }
		}
	      /* Check if we found it outside our block. */
	      else
		{
		  if(!found_class)
		    {
                      /* Same restriction as above for a block process. */
                      if(!pcb->block->lnh_name)
                        {
                          found_pcb = pcb;
                          found_class = 0;
                        }
		    }
		}
	    }
      }
      
    /* Notify the caller if we found a process. */
    if(found_pcb)
      {
	/*
	** If the process was found then return any hunt signal to the client.
	** NOTE! The found process may be dead by now! We will here depend on
	** the fact that huntd has higher priority than sysd. Thus, we know
	** that:
	** 1: The found pcb is not yet disposed or reused but valid.
	** 2: Sysd has not yet sent out any attach signals from that process.
	**    (A hunt reply may not be sent after an attach signal)
	** So, we can ignore the fact that it is dead, and still send the
	** hunt signal back. The caller will attach to it and will find out
	** that it has died. That send, however requires use safe_send_w_s,
	** that is capable of sending from dead objects.
	*/
	if(sigr->hunt_request.hsig)
	  {
	    safe_send_w_s(sigr->hunt_request.hsig, found_pcb->live_pid,
			  sigr->hunt_request.owner, 0, NULL, NULL);
	  }

	/* Also give the client a remote response with the found pid. */
	{
	  PROCESS client = sigr->hunt_request.owner;
	  sigr->remote_call.sig_no = REMOTE_RESPONSE;
	  sigr->remote_call.call_type = REMR_HUNT;
	  sigr->remote_response.results.remr_hunt.found = (OSBOOLEAN) 1;
	  sigr->remote_response.results.remr_hunt.pid = pcb_to_pid(found_pcb);
	  /*
	  ** We must use safe_send_w_s() to make sure we don't use the
	  ** redirection table of the client. Otherwise this response
	  ** might not reach the receive_from() inside common_hunt().
	  */
	  safe_send_w_s(sig_to_adm(sigr), zzcur_p->pid, client, 1ul, NULL, NULL);
	}

	/* Go receive again if still no response from the linkhandler. */
	if(current_linkhandler_request)
	  {
	    goto receive_again;
	  }

	/* Otherwise we are done. */
	return;
      }
  }

  /* If not found and linkhandler is busy, then leave the request in the internal queue. */
  if(current_linkhandler_request)
    {
      sigr->hunt_request.T_name.splay_n = NULL;
      if(pending_hunt_request)
	{
	  pending_tail->T_name.splay_n = (struct treenode *) sigr;
	  pending_tail = (struct hunt_request *) sigr;
	}
      else
	{
	  pending_tail = (struct hunt_request *) sigr;
	  pending_hunt_request = (struct hunt_request *) sigr;
	}

      /* Go receive again if still no response from the linkhandler. */
      goto receive_again;
    }

  /* Otherwise search the linkhandler list for a suitable linkhandler. */
  current_linkhandler_request = (struct hunt_request *) sigr;
 search_linkhandler_list:
  T2 = splay_find((KEYTYPE) current_linkhandler_request->name,
		  &zzuu.linkh_list_T_link_name,
		  splaycmp_linkh_link_name);

  /* Traverse the list of linkhandlers with the correct name. */
 continue_linkhandler_search:
  if(T2)
    {
      /* Save the currently outstanding request. */
      linkh = (struct new_linkhandler *) T2->data;
      /*
      ** Check that the complete linkhandler name actually matches the first
      ** part of the hunt request. We need to do this because the splay tree
      ** of linkhandlers is only sorted on the characters before the first '/'.
      ** Since we allow linkhandler names with several '/'-characters, there
      ** is a possible mismatch.
      **
      ** Check that the linkhandler user number matches the request. 
      */
      if(substr_match(linkh->link_name, current_linkhandler_request->name) 
         && check_hunt_user(current_linkhandler_request->user,
			    linkh->new_linkhdlr,
			    current_linkhandler_request->from,
			    current_linkhandler_request->owner))
	{
          struct _pcb *lnh_pcb;
                
	  /*
	  ** Make sure the linkhandler can see the hunt daemon.
	  ** We do this by making all requests from a phantom process
	  ** acting in the user space of the linkhandler.
	  ** We can protect the kernel since this phantom can not be killed and
	  ** no other system calls would affect a phantom.
	  ** ?? This phantom mechanism must be changed if we decide to accept
	  ** several concurrent linkhandler calls in the future.
	  ** See the deadlock problem in huntd_hunt_request().
	  */
	  zzuu.huntd2_pcb->real_user = linkh->user;

          lnh_pcb = pid_to_pcb(linkh->new_linkhdlr);
          
          /* Check if we have link handler process or a block link */
          if ((lnh_pcb->magic & ~MAGIC_KILLED) != MAGIC_BCB)
	    {
              /* Make a remote system call to the link handler process. */
	      union SIGNAL *sig;

              /* Attach to the linkhandler. */
              attach_id = attach(NULL, linkh->new_linkhdlr);
	    
              sig = alloc(sizeof(struct remote_call) +
		      strlen(current_linkhandler_request->name), REMOTE_CALL);
	      sig->remote_call.call_type = REM_HUNT;
	      sig->remote_call.params.rem_hunt.user =
		      current_linkhandler_request->user;
	      sig->remote_call.params.rem_hunt.from = 
		      current_linkhandler_request->from;
	      sig->remote_call.params.rem_hunt.persistent =
		      current_linkhandler_request->hsig ?
			      (OSBOOLEAN) 1: (OSBOOLEAN) 0;
	      strcpy(sig->remote_call.params.rem_hunt.name,
		      (const char *) current_linkhandler_request->name);
	      send_w_s(&sig, zzuu.huntd2_, linkh->new_linkhdlr);
	      /* Set the 10 sec deadlock timeout. */
	      hunt_time = get_ticks();
	    }
          else
            {
	      /* Block link */
              char *name;
              struct _pcb *pcb;  
              union SIGNAL *sig;

              name = current_linkhandler_request->name + linkh->link_name_len;
              
              /* Check the list of processes in the block and search 
              ** for a match.
              **  
              ** Both the block and processes in the proclist may be dead by 
              ** now. However, the proclist field is not touched when processes 
              ** are killed, and PCB:s are freed by us (huntd), so the list 
              ** should still be safe to traverse.
              **  
              ** Even so, we do depend on the fact that huntd has 
              ** higher priority than sysd. We must know that sysd has not 
              ** yet sent out any attach signals from a dead process.
              ** (A hunt reply may not be sent after an attach signal)
              **
              ** FIXME: This looks like a bug - priority is unreliable for
              ** synchronisation as huntd sometimes blocks waiting for signals.
              ** Is there some status info in the PCB that indicates if the
              ** attach sig has already been sent? Then we should use it
              ** instead of relying on prio. 
              */                            
              for (pcb = lnh_pcb->proclist; pcb; pcb = pcb->proclist)
                {
                  if (!strcmp(pcb->name, name)
                      && (current_linkhandler_request->user == pcb->user))
                    {
                      break;
                    }
                }
                
              /* Attach to the linkhandler. */
              attach_id = attach(NULL, linkh->new_linkhdlr);

              /* Send ourselves a REMOTE_RESPONSE */
              sig = alloc(sizeof(struct remote_response), REMOTE_RESPONSE);
              sig->remote_call.sig_no = REMOTE_RESPONSE;
              sig->remote_call.call_type = REMR_HUNT;
              if (pcb)
                {
                  sig->remote_response.results.remr_hunt.found = (OSBOOLEAN) 1;
                  sig->remote_response.results.remr_hunt.pid = pcb->live_pid;
                }
              else
                {
                  sig->remote_response.results.remr_hunt.found = (OSBOOLEAN) 0;
                  sig->remote_response.results.remr_hunt.pid = (PROCESS) 0;
                }
              send_w_s(&sig, linkh->new_linkhdlr, zzuu.huntd_ /* curr_proc */);

              /* Set the 10 sec deadlock timeout. */
              hunt_time = get_ticks();                         
            }
	}
      else
	{
	  /* Ask the next linkhandler if the present doesn't match after all. */
	  T2 = T2->splay_n;
	  goto continue_linkhandler_search;
	}
    }

  /* If no more linkhandlers, then enter the request into the pending list. */
  else
    {
      union SIGNAL *sig;

      /* Send a negative response to the client waiting in the hunt call. */
      sig = alloc(sizeof(struct remote_response), REMOTE_RESPONSE);
      sig->remote_call.sig_no = REMOTE_RESPONSE;
      sig->remote_call.call_type = REMR_HUNT;
      sig->remote_response.results.remr_hunt.found = (OSBOOLEAN) 0;
      sig->remote_response.results.remr_hunt.pid = (PROCESS) 0;
      send(&sig, current_linkhandler_request->owner);

      /* Enter the request into the list of pending hunt requests. */
      if(current_linkhandler_request->hsig)
	{
      	current_linkhandler_request->T_linkname.data = (void *) current_linkhandler_request;
	splay_lin_insert((KEYTYPE) current_linkhandler_request->name,
			 &current_linkhandler_request->T_linkname,
			 &zzuu.hunt_list_T_linkname,
			 splaycmp_hunt_linkname);
      
	current_linkhandler_request->T_name.data = (void *) current_linkhandler_request;
	splay_lin_insert((KEYTYPE) current_linkhandler_request->name,
			 &current_linkhandler_request->T_name,
			 &zzuu.hunt_list_T_name,
			 splaycmp_hunt_name);

	current_linkhandler_request->T_from.data = (void *) current_linkhandler_request;
	splay_lin_insert((KEYTYPE) current_linkhandler_request->from,
			 &current_linkhandler_request->T_from,
			 &zzuu.hunt_list_T_from,
			 splaycmp_hunt_from);

	current_linkhandler_request->T_owner.data = (void *) current_linkhandler_request;
	splay_lin_insert((KEYTYPE) current_linkhandler_request->owner,
			 &current_linkhandler_request->T_owner,
			 &zzuu.hunt_list_T_owner,
			 splaycmp_hunt_owner);
        }
      /* Otherwise free the buffer containing the hunt request. */
      else
	{
	  free_buf_type(struct hunt_request *, &current_linkhandler_request);
	}
      
      /* If another hunt request is pending, then ask the linkhandlers about it. */
      current_linkhandler_request = pending_hunt_request;
      if(current_linkhandler_request)
	{
	  pending_hunt_request = (struct hunt_request *)
	    pending_hunt_request->T_name.splay_n;
	  goto search_linkhandler_list;
	}
      else
	{
	  /* Otherwise we are done. */
	  return;
	}
    }

  /*
  ** Wait for a linkhandler response or another hunt request.
  */
 receive_again:
  sigr = receive_w_tmo((OSTIME) 2510, sig_sel);

  /*
  ** Check if the deadlock timeout expired and report an error.
  ** Resume operation if the error was ignored.
  */
  if((get_ticks() - hunt_time) > hunt_timeout)
    {
      os_error(1, OSE_ELINKHANDLER_DEADLOCK,
	       (OSERRCODE) linkh->new_linkhdlr);  /* Non fatal system error. */
      /* Restart the 10 sec deadlock timeout. */
      hunt_time = get_ticks();
    }

  /* Receive again if we didn't get a signal. */
  if(!sigr)
    goto receive_again;

  /* Execute the received signal. */
  switch((int) sigr->sig_no)
    {
      /* A hunt request causes a new search. */
    case HUNT_REQUEST:
      {
	goto search_process_list;
      }
      /* A response from a linkhandler was received. */
    case REMOTE_RESPONSE:
      /* A response from a linkhandler was canceled. */
    case REMOTE_CANCEL:
      {
	/* Check the pid of the linkhandler. 
        ** Ignore and report an error if invalid. 
        */
	if(sender(&sigr) != linkh->new_linkhdlr)
	  {    
	    os_error(1, OSE_ESPURIOUS_SYSTEM_SIGNAL, (OSERRCODE) sender(&sigr)); /* Non fatal system error. */
	    free_buf(&sigr);
	    goto receive_again;
	  }

	/* Detach from the linkhandler. */
	detach(&attach_id);
	/* Fall through to the next case. */
	/*FALLTHROUGH*/
      }
      /* A linkhandler died. Abort any outstanding remote calls. */
    case OS_ATTACH_SIG:
      {
	break;
      }
    default:
      {
	/* We got an unexpected signal. This should not be possible. */
	os_error(1, OSE_ESPURIOUS_SYSTEM_SIGNAL, (OSERRCODE) sender(&sigr)); /* Non fatal system error. */
	free_buf(&sigr);
	goto receive_again;
      }
    }

  /*
  ** Restore the secondary hunt daemon to superuser.
  ** We do this only to make the process look nicer in the
  ** debugger when it is not in use.
  */
  zzuu.huntd2_pcb->real_user = (OSUSER) 0;

  /* If the linkhandler finds the process then return the hunt signal */
  if((sigr->sig_no == REMOTE_RESPONSE) &&
     (sigr->remote_response.results.remr_hunt.found))
    {
      /* Send a hunt response to the owner of the hunt request... */
      if(current_linkhandler_request->hsig)
	safe_send_w_s(current_linkhandler_request->hsig,
		      sigr->remote_response.results.remr_hunt.pid,
		      current_linkhandler_request->owner, 0, NULL, NULL);

      /* Give the client a remote response. */
      send(&sigr, current_linkhandler_request->owner);
      
      /* Free the buffer containing the hunt request. */
      free_buf_type(struct hunt_request *, &current_linkhandler_request);

      /* Take care of any pending hunt requests. */
      current_linkhandler_request = pending_hunt_request;
      if(current_linkhandler_request)
        {
	  pending_hunt_request = (struct hunt_request *)
	    pending_hunt_request->T_name.splay_n;
	  goto search_linkhandler_list;
        }

    }
  /* If the linkhandler does not know... */
  else
    {
      /* Free the linkhandler response signal. */
      free_buf(&sigr);

      /* and ask the next linkhandler. */
      T2 = T2->splay_n;
      goto continue_linkhandler_search;
    }
}
  
/*
** Subfunction to the hunt daemon.
**
** Register a new process and honor any pending hunt requests that matches
** the name and user number of the new process.
*/

static void
huntd_create_process(union SIGNAL *sigr)
{
  PROCESS pid;
  register treenode *T, *next;
  register struct _pcb *pcb;
  struct hunt_request *request;
  struct new_linkhandler *linkh;
   
  /* Locate the process descriptor. */
  pid = sigr->new_process.pid;
  pcb = pid_to_pcb(pid);

  /* Insert the new process control block into the process list. */
  pcb->T_procname.data = pcb;
  splay_lin_insert((KEYTYPE) pcb->name,
		   &pcb->T_procname,
		   &zzuu.pcb_list,
		   splaycmp_pcb_name);

  /* Search the list of pending hunt requests for a matching name. */
  T = splay_find((KEYTYPE) pcb->name,
		 &zzuu.hunt_list_T_name,
		 splaycmp_hunt_name);
  
  /* Check each request in the found list for a matching user number. */
  while(T)
    {
      /* If match, send a hunt response to the owner of the hunt request. */
      request = (struct hunt_request *) T->data;
      if(request->user == pcb->user)
	{
          struct _pcb *caller = safe_pid_to_pcb(request->owner, 0);

          /* If the new process lives in a block with a block link,
           * we won't report it to outsiders. */
          if(pcb->block->lnh_name)
            {
              if((caller) && (caller->block != pcb->block))
                {
                  T = T->splay_n;
                  continue;
                }
            }
          
	  /* Send hunt response. */
	  if(request->hsig)
	    safe_send_w_s(request->hsig, pid, request->owner,
			  0, NULL, NULL);
	  
          /* Delete the hunt request. */
	  next = T->splay_n;
	  splay_lin_delete(&request->T_name, &zzuu.hunt_list_T_name);
	  splay_lin_delete(&request->T_linkname, &zzuu.hunt_list_T_linkname);
	  splay_lin_delete(&request->T_owner, &zzuu.hunt_list_T_owner);
	  splay_lin_delete(&request->T_from, &zzuu.hunt_list_T_from);

          /* Free the buffer containing the hunt request. */
	  free_buf_type(struct hunt_request *, &request);
	  T = next;
	}
      else
	{
	  T = T->splay_n;
	}
    }
  
  /* Check if the block of the process is a linkhandler. */
  T = splay_find((KEYTYPE) sigr->new_process.block,
                 &zzuu.linkh_list_T_pid,
                 splaycmp_linkh_owner);

  /* Check matching entry in the linkhandler list. */
  if (T)
    {
      linkh = (struct new_linkhandler *) T->data;

      /*
      ** Check if there are any pending hunt requests that match the name of 
      ** the current linkhandler.
      */
      T = splay_find((KEYTYPE) linkh->link_name,
                     &zzuu.hunt_list_T_linkname,
                     splaycmp_hunt_linkname);

      /*
      ** Check each request in the found list for a matching user number and 
      ** name.
      */
      while(T)
        {
          char *name;
          
          /* Pick up data pointer. */
          request = (struct hunt_request *) T->data;
          name = request->name + linkh->link_name_len;
          
          /*
          ** Check that the complete linkhandler name actually matches the first
          ** part of the hunt request. We need to do this because the splay tree
          ** of linkhandlers is only sorted on the characters before the first '/'.
          ** Since we allow linkhandler names with several '/'-characters, there
          ** is a possible mismatch.
          ** Test for a matching process name. 
          ** Test for a matching user number.
          */
          if(substr_match(linkh->link_name, request->name)
             && !strcmp(pcb->name, name)
             && (request->user == pcb->user))
            {
              /* Send hunt response. */
              if(request->hsig)
                safe_send_w_s(request->hsig, pid, request->owner,
                              0, NULL, NULL);
          
              /* Delete the hunt request. */
              next = T->splay_n;
              splay_lin_delete(&request->T_name, &zzuu.hunt_list_T_name);
              splay_lin_delete(&request->T_linkname,&zzuu.hunt_list_T_linkname);
              splay_lin_delete(&request->T_owner, &zzuu.hunt_list_T_owner);
              splay_lin_delete(&request->T_from, &zzuu.hunt_list_T_from);

              /* Free the buffer containing the hunt request. */
              free_buf_type(struct hunt_request *, &request);
              T = next;
            }
          else
            {
              T = T->splay_n;  
            }           
        }
    }
  
  /* Free the new_process signal. */
  free_buf(&sigr);
}

/*
** Subfunction to the hunt daemon.
**
** A new user number was assigned to a process.
** Reevaluate all pending hunt requests matching the name of the process
** and honor those that also matches the new user number.
** This function is obsolete. See set_uid()??
*/

static void
huntd_set_uid(union SIGNAL *sigr)
{
  PROCESS pid;      
  register treenode *T, *next;
  register struct _pcb *pcb;
  struct hunt_request *request;
  struct new_linkhandler *linkh;
  
  /* Locate the process descriptor. */
  pid = sigr->new_uid.pid;
  pcb = pid_to_pcb(pid);
  
  /* Free the new_uid signal. */
  free_buf(&sigr);

  /* Search the list of pending hunt requests for a matching name. */
  /* Done if not found. */
  T = splay_find((KEYTYPE) pcb->name,
		 &zzuu.hunt_list_T_name,
		 splaycmp_hunt_name);

  /* Check each request in the found list for a matching user number. */
  while(T)
    {
      /* If match, send a hunt response to the owner of the hunt request. */
      request = (struct hunt_request *) T->data;
      if(request->user == pcb->user)
	{
	  /* Send hunt response. */
	  safe_send_w_s(request->hsig, pid, request->owner, 0, NULL, NULL);
	  
          /* Delete the hunt request. */
	  next = T->splay_n;
	  splay_lin_delete(&request->T_name, &zzuu.hunt_list_T_name);
	  splay_lin_delete(&request->T_linkname, &zzuu.hunt_list_T_linkname);
	  splay_lin_delete(&request->T_owner, &zzuu.hunt_list_T_owner);
	  splay_lin_delete(&request->T_from, &zzuu.hunt_list_T_from);
	  
          /* Free the buffer containing the hunt request. */
	  free_buf_type(struct hunt_request *, &request);
	  T = next;
	}
      else
	{
	  T = T->splay_n;
	}
    }

  /* Check if the block of the process is a linkhandler.
  ** 
  ** Using pcb_to_pid() on the block pointer will result in 
  ** an unexpected pid if the block has been killed. However, 
  ** that should be  no problem here since it means that the 
  ** process is dead and no hunt signal is needed.
  */   
  T = splay_find((KEYTYPE) pcb_to_pid(pcb->block), 
                 &zzuu.linkh_list_T_pid,
                 splaycmp_linkh_owner);

  /* Check matching entry in the linkhandler list. */
  if (T)
    {
      /* Delete the linkhandler definition entry. */
      linkh = (struct new_linkhandler *) T->data;

      /*
      ** Check if there are any pending hunt requests that match the name of 
      ** the current linkhandler.
      */
      T = splay_find((KEYTYPE) linkh->link_name,
                     &zzuu.hunt_list_T_linkname,
                     splaycmp_hunt_linkname);

      /*
      ** Check each request in the found list for a matching user number and 
      ** name.
      */
      while(T)
        {
          char *name;
          
          /* Pick up data pointer. */
          request = (struct hunt_request *) T->data;
          name = request->name + linkh->link_name_len;
          
          /*
          ** Check that the complete linkhandler name actually matches the first
          ** part of the hunt request. We need to do this because the splay tree
          ** of linkhandlers is only sorted on the characters before the first '/'.
          ** Since we allow linkhandler names with several '/'-characters, there
          ** is a possible mismatch.
          ** Test for a matching process name. 
          ** Test for a matching user number.
          */
          if(substr_match(linkh->link_name, request->name)
             && !strcmp(pcb->name, name)
             && (request->user == pcb->user))
            {
              /* Send hunt response. */
              if(request->hsig)
                safe_send_w_s(request->hsig, pid, request->owner,
                              0, NULL, NULL);
          
              /* Delete the hunt request. */
              next = T->splay_n;
              splay_lin_delete(&request->T_name, &zzuu.hunt_list_T_name);
              splay_lin_delete(&request->T_linkname,&zzuu.hunt_list_T_linkname);
              splay_lin_delete(&request->T_owner, &zzuu.hunt_list_T_owner);
              splay_lin_delete(&request->T_from, &zzuu.hunt_list_T_from);

              /* Free the buffer containing the hunt request. */
              free_buf_type(struct hunt_request *, &request);
              T = next;
            }
          else
            {
              T = T->splay_n;  
            }           
        }
    }  
}

/*
** Subfunction to the hunt daemon.
**
** A new linkhandler was created with the assign_linkhandler() system call.
** The hunt daemon inserts the new process into the list of linkhandlers.
** Any pending hunt requests that match the new linkhandler are sent to it.
**
** ??Should we maintain a list of huntd2 phantoms? One for each user space
** containing a linkhandler.
*/

static void
huntd_assign_linkhandler(union SIGNAL *sigr, PROCESS client)
{
  register treenode *T, *next;
  struct hunt_request *request;
  OSATTREF attach_id;  /* Used when attached to linkhandler. */
  OSBOOLEAN found;
  PROCESS found_pid;
  PROCESS lnh_pid;
  struct _pcb *lnh_pcb;
  union SIGNAL *sig;

  /*
  ** This function relies on process priority to lock out the sysd.
  */
  lnh_pid = sigr->new_linkhandler.new_linkhdlr;
  lnh_pcb = pid_to_pcb(lnh_pid);
  
  if(client)
    {
      /*
      ** Block links should be allowed if:
      ** 1) No link for the block already exists. We can't accept multiple
      **    links because it would lead to ambiguous naming of the contained
      **    processes
      ** 2) No processes have been created in the block. The creation of a
      **    block link changes the naming convention for the contents, and
      **    we don't want any processes to suddenly change name.
      */     
      if((lnh_pcb->magic & ~MAGIC_KILLED) == MAGIC_BCB)
        {
          char *namebuf;

          /* Check if pcb is already assigned a link */
          T = splay_find((KEYTYPE) lnh_pid,
                         &zzuu.linkh_list_T_pid,
                         splaycmp_linkh_owner);
          if(T)
            {
              /* Re-use sigr as reply signal */
              sigr->new_linkhandler_reply.ecode =
                  OSE_ASSIGN_LINKHANDLER | OSE_ELINKHANDLER_ALREADY_ASSIGNED;
              sigr->new_linkhandler_reply.extra = lnh_pid;
              goto send_error_sig;
            }

          /* Check that the block is empty. */
          if(lnh_pcb->proclist)
            {
              /* Re-use sigr as reply signal */
              sigr->new_linkhandler_reply.ecode =
                  OSE_ASSIGN_LINKHANDLER | OSE_EBLOCK_HAS_PROCESSES;
              sigr->new_linkhandler_reply.extra = lnh_pid;
              goto send_error_sig;
            }

          /*
          ** Here is a possible optimization: the BCB lnh_name field could
          ** point to the same memory as is used in the search tree instead
          ** of a separate buffer. The exact lifetimes of each needs to be
          ** investigated before introducing the optimization.
          */
          /* Make sure that the name fits in the system pool. */
          if(system_pool.signal_list[7].size <
             (OSADDRESS) (sizeof(SIGSELECT) +
                          sigr->new_linkhandler.link_name_len))
            {
              /* Re-use sigr as reply signal */
              sigr->new_linkhandler_reply.ecode =
                  OSE_ASSIGN_LINKHANDLER | OSE_EBAD_PARAMETER;
              sigr->new_linkhandler_reply.extra =
                  (OSERRCODE) sigr->new_linkhandler.link_name;
              /* Falling through to common code for all the errors */

            send_error_sig:
              sigr->new_linkhandler_reply.sig_no = NEW_LINKHANDLER_REPLY;
              if (sigr->new_linkhandler.expecting_reply)
                {
                  send(&sigr, client);
                }
              else
                {
                  os_error(0, /* 0=Warning */
                           sigr->new_linkhandler_reply.ecode,
                           sigr->new_linkhandler_reply.extra);
                  free_buf(&sigr);
                }
              return;
            }

          /*
          ** pcb is a block. Flag it as having a block link.
          */
          namebuf = (char *) sys_alloc(
               sizeof(SIGSELECT) + sigr->new_linkhandler.link_name_len, 0);
          strcpy(namebuf, sigr->new_linkhandler.link_name);
          ((struct _block *)lnh_pcb)->lnh_name = namebuf;
        }
    }
    
  /*
  ** Insert the new process into the list of linkhandlers.
  ** The sigr buffer is henceforth owned by the list.
  */
  sigr->new_linkhandler.T_pid.data = (void *) sigr;
  splay_lin_insert((KEYTYPE) lnh_pid,
        	   &sigr->new_linkhandler.T_pid,
		   &zzuu.linkh_list_T_pid,
		   splaycmp_linkh_owner);

  sigr->new_linkhandler.T_link_name.data = (void *) sigr;
  splay_lin_insert((KEYTYPE) sigr->new_linkhandler.link_name,
		   &sigr->new_linkhandler.T_link_name,
		   &zzuu.linkh_list_T_link_name,
		   splaycmp_linkh_link_name);

  if(sigr->new_linkhandler.expecting_reply)
    {
      /*
      ** Signal to synchronise with client to avoid the OSECR-12073 race
      ** condition, where the client would continue and create processes
      ** in the block before we had committed to processing this request.
      **
      ** The reply signal applies only to block links because
      ** we would get a potential deadlock situation if a regular link
      ** handler were to wait for this reply while registering itself or
      ** a subordinate phantom as a linkhandler.
      **
      ** We use a field in the signal to determine if a reply is needed;
      ** we can't assume the test for linkhandler type now would be
      ** identical to the type when the client generated the request.
      */
      sig = alloc(sizeof(struct new_linkhandler_reply),
                  NEW_LINKHANDLER_REPLY);
      sig->new_linkhandler_reply.ecode = 0;
      send(&sig, client);
    }

  /*
  ** Check if there are any pending hunt requests that match the name of 
  ** the new linkhandler.
  */
  T = splay_find((KEYTYPE) sigr->new_linkhandler.link_name,
		 &zzuu.hunt_list_T_linkname,
		 splaycmp_hunt_linkname);

  /*
  ** Check each request in the found list for a matching user
  ** number and name. Send a hunt request to the new linkhandler
  ** for each pending hunt request.
  */
  while(T)
    {
      /* Pick up next pointer and data pointer. */
      request = (struct hunt_request *) T->data;
      next = T->splay_n;

      /*
      ** Check that the complete linkhandler name actually matches
      ** the first part of the hunt request. We need to do this
      ** because the splay tree of linkhandlers is only sorted on
      ** the characters before the first '/'.  Since we allow
      ** linkhandler names with several '/'-characters, there is a
      ** possible mismatch.  Test for a matching user number.  Ask
      ** the linkhandler if allowed to.
      */
      if(substr_match(sigr->new_linkhandler.link_name, request->name)
         && check_hunt_user(request->user, lnh_pid, 
                            request->from, request->owner))
	{
          /*
          ** Make sure the linkhandler can see the hunt daemon.
          ** We do this by making all requests from a phantom process
          ** acting in the user space of the linkhandler.
          ** We can protect the kernel since this phantom can not be 
          ** killed and no other system calls would affect a phantom.
          ** ?? This phantom mechanism must be changed if we decide to 
          ** accept several concurrent linkhandler calls in the future.
          ** See the deadlock problem in huntd_hunt_request().
          */
          zzuu.huntd2_pcb->real_user = sigr->new_linkhandler.user;

          found = (OSBOOLEAN) 0;
          found_pid = (PROCESS)0;

          /* Real link handler process */
          if ((lnh_pcb->magic & ~MAGIC_KILLED) != MAGIC_BCB)
            {                  
              /* attach to the linkhandler. */	  
              attach_id = attach(NULL, lnh_pid);

	      /* Make a remote system call to the linkhandler. */
	      sig = alloc(sizeof(struct remote_call) + strlen(request->name), 
                          REMOTE_CALL);
	      sig->remote_call.call_type = REM_HUNT;
	      sig->remote_call.params.rem_hunt.user = request->user;
	      sig->remote_call.params.rem_hunt.from = request->from;
	      sig->remote_call.params.rem_hunt.persistent = request->hsig ?
	          (OSBOOLEAN) 1: (OSBOOLEAN) 0;
	      strcpy(sig->remote_call.params.rem_hunt.name, 
                     (const char *) request->name);
	      send_w_s(&sig, zzuu.huntd2_, lnh_pid);

	      /* Wait for a response. */
	      {
	        static const SIGSELECT sig_sel[] = {3, REMOTE_RESPONSE,
						      REMOTE_CANCEL,
						      OS_ATTACH_SIG};

	        while(NIL == (sig = receive_from((OSTIME) 10000, 
                                                  sig_sel,lnh_pid)))
	          {
                    /* Non fatal system error. */
                    os_error(1, OSE_ELINKHANDLER_DEADLOCK, (OSERRCODE) lnh_pid);
	          }
	        switch((int) sig->sig_no)
	          {
		    /* A response from a linkhandler was received. */
	          case REMOTE_RESPONSE:
		    {
		      /*
		      ** Check that the linkhandler returned a valid 
                      ** process id. Ignore the response otherwise.
		      */
		      found = sig->remote_response.results.remr_hunt.found;
		      found_pid = sig->remote_response.results.remr_hunt.pid;

                      if(found && get_ptype(found_pid) == OS_ILLEGAL)
		        found = (OSBOOLEAN) 0;
		      /* Fall through to the next case and detach. */
		      /*FALLTHROUGH*/
		    }
		    /* A response from a linkhandler was canceled. */
	          case REMOTE_CANCEL:
		    {
		      detach(&attach_id);
		      break;
		    }
		    /* A linkhandler died. Abort outstanding remote calls */
	          case OS_ATTACH_SIG:
	          default:
		    break;
	          }
	      }                  

              /* Free the linkhandler response. */
              /* There was once a bug here, (ref lost signal 243) */
              free_buf(&sig);
            }

          /* Did we find a process? */
          if(found)
            {
              /* Yes, send a hunt response to the owner of the hunt request. */
              if(request->hsig)
                safe_send_w_s(request->hsig, found_pid,
                              request->owner, 0, NULL, NULL);

              /* and delete the hunt request. */
              splay_lin_delete(&request->T_name, &zzuu.hunt_list_T_name);
              splay_lin_delete(&request->T_linkname,
                               &zzuu.hunt_list_T_linkname);
              splay_lin_delete(&request->T_owner, &zzuu.hunt_list_T_owner);
              splay_lin_delete(&request->T_from, &zzuu.hunt_list_T_from);

              /* Free the buffer containing the hunt request. */
              free_buf_type(struct hunt_request *, &request);
            }                                            

          /*
          ** Restore the secondary hunt daemon to superuser.
          ** We do this only to make the process look nicer in the
          ** debugger when it is not in use.
          */
          zzuu.huntd2_pcb->real_user = (OSUSER) 0;
	}

      T = next;
    }
}

/*
** Subfunction to the hunt daemon.
**
** A process was killed.
** The hunt daemon removes the deleted process from the process list.
** Pending hunt requests issued by or for the removed process are
** discarded.
** The process control block is returned to the list of free process
** control blocks.
**
** If the killed process was a linkhandler, it is removed from the
** linkhandler list.
*/

static void
huntd_kill_proc(union SIGNAL *sigr)
{
  register PROCESS pid;
  register struct _pcb *pcb;
  register treenode *T, *next;
  struct hunt_request *request;
  struct new_linkhandler *linkh;
  
  /* Get the process id and the pcb pointer of the killed process. */
  pid = sigr->delete_process.pid;
  pcb = pid_to_pcb(pid);

  /* Free the delete_process signal. */
  free_buf(&sigr);

  /* Unlink the pcb from the process list. */
  splay_lin_delete(&pcb->T_procname, &zzuu.pcb_list);

  /* Discard all pending hunt requests issued by the killed process. */
  T = splay_find((KEYTYPE) pid,
		 &zzuu.hunt_list_T_owner,
		 splaycmp_hunt_owner);
  while(T)
    {
      /* Free the hunt signal if it is present. */
      request = (struct hunt_request *) T->data;
      if(request->hsig)
	safe_free_buf(request->hsig);

      /* Delete the hunt request. */
      next = T->splay_n;
      splay_lin_delete(&request->T_name, &zzuu.hunt_list_T_name);
      splay_lin_delete(&request->T_linkname, &zzuu.hunt_list_T_linkname);
      splay_lin_delete(&request->T_owner, &zzuu.hunt_list_T_owner);
      splay_lin_delete(&request->T_from, &zzuu.hunt_list_T_from);

      /* Free the buffer containing the hunt request. */
      free_buf_type(struct hunt_request *, &request);
      T = next;
    }

  /* Discard all pending hunt requests issued for (from) the killed process. */
  T = splay_find((KEYTYPE) pid,
		 &zzuu.hunt_list_T_from,
		 splaycmp_hunt_from);
  while(T)
    {
      /* Free the hunt signal if it is present. */
      request = (struct hunt_request *) T->data;
      if(request->hsig)
	safe_free_buf(request->hsig);

      /* Delete the hunt request. */
      next = T->splay_n;
      splay_lin_delete(&request->T_name, &zzuu.hunt_list_T_name);
      splay_lin_delete(&request->T_linkname, &zzuu.hunt_list_T_linkname);
      splay_lin_delete(&request->T_owner, &zzuu.hunt_list_T_owner);
      splay_lin_delete(&request->T_from, &zzuu.hunt_list_T_from);

      /* Free the buffer containing the hunt request. */
      free_buf_type(struct hunt_request *, &request);
      T = next;
    }

  /* Check if the killed process was a linkhandler. */
  T = splay_find((KEYTYPE) pid,
		 &zzuu.linkh_list_T_pid,
		 splaycmp_linkh_owner);

  /* Discard all matching entries in the linkhandler list. */
  while(T)
    {
      /* Delete the linkhandler definition entry. */
      linkh = (struct new_linkhandler *) T->data;
      next = T->splay_n;
      splay_lin_delete(&linkh->T_pid, &zzuu.linkh_list_T_pid);
      splay_lin_delete(&linkh->T_link_name, &zzuu.linkh_list_T_link_name);

      /* Free the buffer containing the hunt request. */
      free_buf_type(struct new_linkhandler *, &linkh);
      T = next;
    }

  /* Free the process name buffer. */
  /* We can't do this in the sysd since huntd uses it all the time. */
  sys_free_buf_type(const char *, &pcb->name);

  /* Free the process control block. */
  free_pcb(pcb);
}

/*
** Subfunction to the hunt daemon.
**
** Register a new block to honor the inheritance rule of block links.
*/

static void
huntd_create_block(union SIGNAL *sigr)
{  
  PROCESS bid;
  PROCESS parent;  
  OSUSER user;
  treenode *T;
  struct new_linkhandler *linkh;
  
  bid = sigr->new_process.pid;
  parent = sigr->new_process.block;
  user = pid_to_pcb(bid)->user;
  
  /* Check if the parent block is a linkhandler. */
  T = splay_find((KEYTYPE) parent,
                 &zzuu.linkh_list_T_pid,
                 splaycmp_linkh_owner);

  /* Inherit matching entry in the linkhandler list. */
  if(T)
    {
      union SIGNAL *sig;
      
      linkh = (struct new_linkhandler *) T->data;
      
      sig = alloc(sizeof(struct new_linkhandler) + linkh->link_name_len, 
                  NEW_LINKHANDLER);
      sig->new_linkhandler.new_linkhdlr = bid;
      sig->new_linkhandler.user = user;
      sig->new_linkhandler.expecting_reply = 0; /* False */
      sig->new_linkhandler.link_name_len = linkh->link_name_len;
      strcpy(sig->new_linkhandler.link_name, linkh->link_name);
      
      /*
      ** REVIEW REMARK: The code to flag a block as a link handler is
      ** only activated if the client parameter of huntd_assign_linkhandler
      ** is non-zero. Is this the intention? (hali)
      ** This could be a bug, doesn't look like the relevant test case checks
      ** what get_pcb for a process created in a secondary block returns..
      ** There are more things to fix in this area as well (processes in both
      ** blocks should see each other without prefix for example). Correcting
      ** this was deemed not a priority a long time ago I think. (rowe)
      */
      huntd_assign_linkhandler(sig, 0);
    }
  
  /* Free the new_process signal. */
  free_buf(&sigr);
}

/*
** Subfunction to the hunt daemon.
**
** A block was killed.
** The hunt daemon removes the deleted block from the list of block links.
*/

static void
huntd_kill_block(union SIGNAL *sigr)
{
  PROCESS pid;
  struct _pcb *pcb;
  treenode *T;
  struct new_linkhandler *linkh;

  /* Get the block id and the bcb pointer of the killed block. */
  pid = sigr->delete_process.pid;
  pcb = pid_to_pcb(pid);

  /* Check if the killed block had a block link. */
  T = splay_find((KEYTYPE) pid,
                 &zzuu.linkh_list_T_pid,
                 splaycmp_linkh_owner);

  /* Discard matching entry in the linkhandler list. */
  if (T)
    {
      /* Delete the linkhandler definition entry. */
      linkh = (struct new_linkhandler *) T->data;
      splay_lin_delete(&linkh->T_pid, &zzuu.linkh_list_T_pid);
      splay_lin_delete(&linkh->T_link_name, &zzuu.linkh_list_T_link_name);

      /* Free the buffer containing the hunt request. */
      free_buf_type(struct new_linkhandler *, &linkh);
    }

  /* Free the delete_process signal. */
  free_buf(&sigr);

  /* Free the block control block. */
  free_pcb(pcb);
}

/*
**===========================================================================
** 18.3  System calls.
**===========================================================================
*/

/*
** This is the assign_linkhandler system call.
**
** A request is sent to the hunt daemon, which registers the
** new linkhandler.
*/

OSBOOLEAN
assign_linkhandler(const char *pathname, PROCESS handler)
{
  static const SIGSELECT sig_sel[] = {1, NEW_LINKHANDLER_REPLY};
  union SIGNAL *sig;
  register U32 namelen = strlen(pathname) + 1;
  register char *d;
  register struct _pcb *pcb;
  OSUSER user;
  OSERRCODE ecode = 0;
  int handler_is_block;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_ASSIGN_LINKHANDLER))
    {
      return (OSBOOLEAN) 1;
    }
#endif

#if ERROR_CHECKS > 0
  /* Check the process id. Report error if invalid. */
  if(!check_pid_bid(handler, OSE_ASSIGN_LINKHANDLER))
    {
      return (OSBOOLEAN) 1;
    }
#endif
  
  /* Save the linkhandler user number before changing to user 0. */
  pcb = pid_to_pcb(handler);
  user = pcb->real_user;
  handler_is_block = ((pcb->magic & ~MAGIC_KILLED) == MAGIC_BCB);

  /* Enter superuser mode. */
  set_suid((OSBOOLEAN) 1);

  /* Send a new_linkhandler request to the hunt daemon. */
  sig = alloc(sizeof(struct new_linkhandler) + namelen, NEW_LINKHANDLER);
  sig->new_linkhandler.new_linkhdlr = handler;
  sig->new_linkhandler.user = user;
  sig->new_linkhandler.expecting_reply = handler_is_block;
  sig->new_linkhandler.link_name_len = namelen;
  strcpy(sig->new_linkhandler.link_name, pathname);

  /* Append a '/' character to the pathname. */
  d = &sig->new_linkhandler.link_name[namelen - 1];
  *d++ = (char) '/';
  *d = (char) '\0';
  send(&sig, zzuu.huntd_);

  /*
  ** Wait for a reply signal.
  **
  ** We only do this if the call is a request to assign a block link.
  ** We MUST have the synchronization when assigning block links (OSECR-12073)
  ** otherwise the caller could race ahead and create processes in the block
  ** before the huntd gets around to handling the request.
  **
  ** We CANNOT have synchronization when assigning link handler processes
  ** because the caller is often a process that acts as the link handler, and
  ** we would get a deadlock if this call would block in that case.
  **
  ** We use a field in the signal to indicate if we want a reply because the
  ** block could potentially be dead by the time the huntd gets around to
  ** handling the request, and it would in that case not be able to use the
  ** process type to make a correct decision as to whether the reply is needed.
  */
  if(handler_is_block)
    {
      while(NIL == (sig = receive_from((OSTIME) -1, sig_sel, zzuu.huntd_)))
        {
          ; /* Empty loop body. */
        }
      ecode = sig->new_linkhandler_reply.ecode;
      if (ecode != 0)
        {
          os_error(0, /* 0=Warning */
                   ecode,
                   sig->new_linkhandler_reply.extra);
        }
      free_buf(&sig);
    }

  /* Exit superuser mode. */
  set_suid((OSBOOLEAN) 0);

  /*
  ** There is a bug here?? assign_linkhandler should return something useful.
  ** Note however that we are moving towards a specification which allows
  ** multiple linkhandlers with the same name.
  */
  /*
  ** Note the race condition that occurs if a new linkhandler is created after
  ** the old one has just been killed. The killed flag in the pcb fixes this problem.
  */
  return (OSBOOLEAN) (ecode != 0);
}

/*
** This is the hunt_from system call.
**
** Give the job to the hunt daemon and wait for a response.
*/

static OSBOOLEAN
common_hunt(const char *name, 
	  OSUSER user,
	  PROCESS *name_, 
	  union SIGNAL **hunt_sig,
	  PROCESS from,
	  OSERRCODE ecode)
{
  union SIGNAL *sig;
  OSBOOLEAN result;
  static const SIGSELECT sig_sel[] = {1, REMOTE_RESPONSE};
  size_t namelen = strlen(name);

#if ERROR_CHECKS > 0
  /* Check out the buffer pointer. */
  if(hunt_sig && !check_buffer(hunt_sig, ecode))
      {
	if(name_)
	  *name_ = (PROCESS) 0;
	return (OSBOOLEAN) 0;
      }
#endif
  
#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(ecode))
    {
      /* Take buffer from user. */
      if(hunt_sig)
	free_buf(hunt_sig);
      if(name_)
	*name_ = (PROCESS) 0;
      return (OSBOOLEAN) 0;
    }
#endif

#if ERROR_CHECKS > 0
  /* Check the process id. Report error if invalid. */
  if(!check_pid(from, ecode))
    {
      /* Take buffer from user. */
      if(hunt_sig)
	free_buf(hunt_sig);
      if(name_)
        *name_ = (PROCESS) 0;
      return (OSBOOLEAN) 0;
    }
#endif
  
  /*
  ** A user number of zero indicates the callers user number.
  ** In the case of a hunt_from system call, user number zero
  ** indicates the user number of the process in the from field.
  **
  ** Should we check further and ignore impossible requests??
  */
  if(!user)
    user = (pid_to_pcb(from))->user;

  /* Enter superuser mode. */
  set_suid((OSBOOLEAN) 1);

  /* Send hunt request to the hunt daemon. */
  /* @@ Store the pointer to the adm block in the .sig fields. */
  sig = alloc(sizeof(struct hunt_request) + namelen, HUNT_REQUEST);
  sig->hunt_request.user = user;
  sig->hunt_request.from = from;
  sig->hunt_request.owner = current_process();
  sig->hunt_request.hsig = hunt_sig ? sig_to_adm(*hunt_sig) : NULL; /* @frbr@ Copy the hunt_sig with sys_alloc? */
  memcpy(sig->hunt_request.name, name, namelen + 1);
  send(&sig, zzuu.huntd_);

  /* Get a response. The response is contained in a remote_response signal.  */
  while(NIL == (sig = receive_from((OSTIME) -1, sig_sel, zzuu.huntd_)))
    {
      ; /* Empty loop body. */
    }
  result = sig->remote_response.results.remr_hunt.found;
  if(name_)
    {
      if(result)
	*name_ = sig->remote_response.results.remr_hunt.pid;
      else
	*name_ = (PROCESS) 0;
    }
  free_buf(&sig);

  /* Exit superuser mode. */
  set_suid((OSBOOLEAN) 0);

  /* Take buffer from user. */
  if(hunt_sig)
    *hunt_sig = NIL;
  return result;
}

/*
** This is the hunt system call.
*/

OSBOOLEAN
hunt(const char *name, 
     OSUSER user,
     PROCESS *name_, 
     union SIGNAL **hunt_sig)
{
  /* Convert to a more general system call. */
  return common_hunt(name, user, name_, hunt_sig, zzcur_p->pid, OSE_HUNT);
}

/*
** This is the hunt_from system call.
**
** Give the job to the hunt daemon and wait for a response.
*/

OSBOOLEAN
hunt_from(const char *name, 
	  OSUSER user,
	  PROCESS *name_, 
	  union SIGNAL **hunt_sig,
	  PROCESS from)
{
  /* Convert to a more general system call. */
  return common_hunt(name, user, name_, hunt_sig, from, OSE_HUNT_FROM);
}

/*
**===========================================================================
** 18.4  Undocumented system calls.
**===========================================================================
*/

/*
*****************************************************************************
** 19 FAST SEMAPHORES.
*****************************************************************************
*/

/*
**===========================================================================
** 19.1  Initialization.
**===========================================================================
*/

/*
**===========================================================================
** 19.2  Local functions.
**===========================================================================
*/

/*
**===========================================================================
** 19.3  System calls.
**===========================================================================
*/

OSFSEMVAL 
get_fsem(PROCESS pid)
{
  register struct _pcb *pcb;
  register OSFSEMVAL semval;

  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid(pid, OSE_GET_FSEM);
  if(pcb == NULL)
    {
      return (OSFSEMVAL) 0;
    }

  /* Is the caller an interrupt or timer process? */
  if(zzcur_p->priority > 32)
    {
      return pcb->fsemval;
    }
  else
    {
      /* The caller is NOT an interrupt or timer process. */

      BKL_LOCK();

      /* Check that the process is still present. */
      if(pcb_to_pid(pcb) != pid)
	{
	  /* Terminate system call if not. */
	  BKL_UNLOCK();
	  return (OSFSEMVAL) 0;
	}

#if USE_REMOTE_CALLS
      /* Issue a remote system call if required. */
      if(local_sysctlblk.enable_remcalls)
      {
	union SIGNAL *sig;
	register PROCESS handler;
	if(pcb->block->use_remcalls)
	  {
	    handler = pcb->block->remcall_server;
	    BKL_UNLOCK();
	    set_suid((OSBOOLEAN) 1);
	    sig = alloc(sizeof(struct remote_call), REMOTE_CALL);
	    sig->remote_call.call_type = REM_GET_FSEM;
	    sig->remote_call.params.rem_get_fsem.pid = pid;
	    
	    sig = make_remote_call(sig, (OSCALLTYPE) REMR_GET_FSEM, handler);
	    if(sig)
	      {
		OSFSEMVAL result = sig->remote_response.results.remr_get_fsem.value;
		free_buf(&sig);
		set_suid((OSBOOLEAN) 0);
		return result;
	      }
	    else
	      {
		set_suid((OSBOOLEAN) 0);
		return (OSFSEMVAL) 0;
	      }
	  }
      }
#endif

      /* Read the fast semaphore. */
      semval = pcb->fsemval;

      /* Enable interrupts. */
      BKL_UNLOCK();

      /* Back to caller. */
      return semval;
    }
}

void 
set_fsem(OSFSEMVAL value, PROCESS pid)
{
  register struct _pcb *pcb;

#if ERROR_CHECKS > 0
  ON_ERRLEVEL_2()
    {
      /* Check that the caller is not an interrupt or timer process. */
      if(check_nonint_caller(OSE_SET_FSEM))
	return;

      /* Check for an attempt to set a negative value. */
      if(unlikely(value < 0))
	{
	  os_error(0, OSE_EBAD_PARAMETER | OSE_SET_FSEM, (OSERRCODE) value);	/* Non fatal. */
	  return;
	}
    }
#endif

  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid(pid, OSE_SET_FSEM);
  if(pcb == NULL)
    {
      return;
    }

  /* Locate the process descriptor. */
  pcb = pid_to_pcb(pid);
      
  /* Disable interrupts. */
  BKL_LOCK();

  /* Check that the process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      /* Terminate system call if not. */
       BKL_UNLOCK();
      return;
    }

#if USE_REMOTE_CALLS
  /* Issue a remote system call if required. */
  if(local_sysctlblk.enable_remcalls)
  {
    union SIGNAL *sig;
    register PROCESS handler;
    if(pcb->block->use_remcalls)
      {
	handler = pcb->block->remcall_server;
	BKL_UNLOCK();
	set_suid((OSBOOLEAN) 1);
	sig = alloc(sizeof(struct remote_call), REMOTE_CALL);
	sig->remote_call.call_type = REM_SET_FSEM;
	sig->remote_call.params.rem_set_fsem.pid = pid;
	sig->remote_call.params.rem_set_fsem.value = value;
	
	sig = make_remote_call(sig, (OSCALLTYPE) REMR_SET_FSEM, handler);
	if(sig)
	  {
	    free_buf(&sig);
	  }
	set_suid((OSBOOLEAN) 0);
	return;
      }
  }
#endif

  /* Set the fast semaphore. */
  pcb->fsemval = value;

  /* Enable interrupts and quit. */
  BKL_UNLOCK();
}

void 
wait_fsem(OSFSEMVAL count)
{
  register struct _pcb *pcb = zzcur_p;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  ON_ERRLEVEL_2()
    if(check_nonint_caller(OSE_WAIT_FSEM))
      return;
#endif

  /* Disable interrupts. */
  LOCK();

  /* Subtract requested value from semaphore and swap if needed. */
  if((pcb->fsemval -= count) < 0)
    {
      pcb->status  = (pr_state_t) (pcb->status | ST_FSEMAPHORE);
      make_mypri_not_ready(pcb);
    }

  /* Enable interrupts and exit. */
  UNLOCK();
}

void 
signal_fsem(PROCESS pid)
{
  LOCKSTATE msr;
  register struct _pcb *pcb;

  /* Is the caller an interrupt or timer process? Yes, it is. */
  if(zzcur_p->priority > 32)
    {
      /* Read instance number and check the process id. Report error if invalid. */
      pcb = check_pid(pid, OSE_SIGNAL_FSEM); 
      if(!pcb)
	{
	  return;
	}

      /* Disable interrupts. */
      LOCK_SAVE(msr);

#if ERROR_CHECKS > 0
      /*
      ** Check that the operation wouldn't cause the semaphore value to wrap from
      ** positive to negative values.
      */
      ON_ERRLEVEL_2()
	if(unlikely(pcb->priority < 33 && pcb->fsemval == 0x7fffffff))
	  {
	    LOCK_RESTORE(msr);
	    os_error(0, OSE_ESEM_OVERFLOW | OSE_SIGNAL_FSEM, (OSERRCODE) pid); /* Non fatal. */
	    /* Terminate the system call. */
	    return;
	  }
#endif

#ifdef OSE_SMP
      /* If the process runs on another cpu, let that one do the job. */
      {
	cpuid_t cpu = pcb->cpu_id;
	if(cpu != zzkrn_cpu_id())
	  {
	    send_krn_event(cpu, KRN_EVT_SIGNAL_FSEM, pid, NULL);
	    LOCK_RESTORE(msr);
	    return;
	  }
      }
#endif

      /* If target process is a prioritized or background process */
      /* then signal the semaphore and maybe swap. */
      if(pcb->priority < 33)
        {
	  if(++(pcb->fsemval) == 0)
	    {
	      if(pcb->status & ST_FSEMAPHORE)
	        {
		  pcb->status = (pr_state_t) (pcb->status & (~ST_RECEIVE_FSEM));
		  
		  /* If the process becomes ready then do a swap. */
		  if(pcb->status < ST_READY)
		    {
		      make_otherpri_ready_noswap(pcb); /* noswap */
		    }
	        }
	    }
        }

      /* If target process is an interrupt process then */
      /* request a wakeup call. */
      else if(pcb->priority < 96)
        {
	  OSPRIORITY priority = zzcur_p->priority;
	  /* If caller is a timer proc then use the timer int priority instead. */
	  if(priority > 95)
	    priority = (OSPRIORITY) (64 + zzuu.timer_int_priority);
	  /* If caller is a lower priority int then do wakeup call now. */
	  if(priority > pcb->priority)
	    {
	      LOCK_RESTORE(msr);
	      zzenter_wakeup(pcb->uarea, (void *) pcb, pcb->priority & 31, (OSBOOLEAN) 2);
	      return;
	    }
	  /* If caller is a higher or same priority int then request later wakeup call. */
	  else
	    {
	      request_int_semwakeup(pcb);
	      LOCK_RESTORE(msr);
	      return;
	    }
        }

      /* If the target process is a timer process then request a wakeup call. */
      /*
      ** A stopped timer process will get called anyway.?? This seems to be the
      ** most reasonable interpretation of the call.
      */
      else if(pcb->priority < 128)
        {
	  OSPRIORITY priority = zzcur_p->priority;
	  /* If caller is a timer proc then request later wakeup call. */
	  if(priority > 95)
	    {
	      request_timer_semwakeup(pcb);
	      LOCK_RESTORE(msr);
	      return;
	    }
	  /* If caller has a priority lower than timer int pri then do wakeup call now. */
	  else if((priority & 31) > zzuu.timer_int_priority)
	    {
	      LOCK_RESTORE(msr);
	      zzenter_wakeup(pcb->uarea, (void *) pcb, zzuu.timer_int_priority, (OSBOOLEAN) 2);
	      return;
	    }
	  /* If caller has a higher or same priority as timer int then request later call. */
	  else
	    {
	      request_timer_semwakeup(pcb);
	      LOCK_RESTORE(msr);
	      return;
	    }
        }

      /* Enable interrupts and quit. */
      LOCK_RESTORE(msr);
      return;
    }

  /* The caller is not an interrupt or timer process. */

  else
    {
      /* Read instance number and check the process id. Report error if invalid. */
      pcb = check_pid(pid, OSE_SIGNAL_FSEM);
      if(pcb == NULL)
	{
	  return;
	}

      /* Disable interrupts. */
      LOCK_SAVE(msr);

      /* Check that the process is still present. */
      if(pcb_to_pid(pcb) != pid)
	{
	  /* Terminate system call if not. */
	  LOCK_RESTORE(msr);
	  return;
	}

#if USE_REMOTE_CALLS
      /* Issue a remote system call if required. */
      if(local_sysctlblk.enable_remcalls)
      {
	union SIGNAL *sig;
	register PROCESS handler;
	if(pcb->block->use_remcalls)
	  {
	    handler = pcb->block->remcall_server;
	    LOCK_RESTORE(msr);
	    set_suid((OSBOOLEAN) 1);
	    sig = alloc(sizeof(struct remote_call), REMOTE_CALL);
	    sig->remote_call.call_type = REM_SIGNAL_FSEM;
	    sig->remote_call.params.rem_signal_fsem.pid = pid;
	
	    sig = make_remote_call(sig, (OSCALLTYPE) REMR_SIGNAL_FSEM, handler);
	    if(sig)
	      {
		free_buf(&sig);
	      }
	    set_suid((OSBOOLEAN) 0);
	    return;
	  }
      }
#endif

#if ERROR_CHECKS > 0
      /*
      ** Check that the operation wouldn't cause the semaphore value to wrap from
      ** positive to negative values.
      */
      ON_ERRLEVEL_2()
	if(unlikely(pcb->priority < 33 && pcb->fsemval == 0x7fffffff))
	  {
	    LOCK_RESTORE(msr);
	    os_error(0, OSE_ESEM_OVERFLOW | OSE_SIGNAL_FSEM, (OSERRCODE) pid);	/* Non fatal. */
	    /* Terminate the system call. */
	    return;
	  }
#endif

#ifdef OSE_SMP
      /* If the process runs on another cpu, let that one do the job. */
      {
	cpuid_t cpu = pcb->cpu_id;
	if(cpu != zzkrn_cpu_id())
	  {
	    send_krn_event(cpu, KRN_EVT_SIGNAL_FSEM, pid, NULL);
	    LOCK_RESTORE(msr);
	    return;
	  }
      }
#endif

      /* If target process is a prioritized or background process */
      /* then signal the semaphore and maybe swap. */
      if(pcb->priority < 33)
        {
	  if(++(pcb->fsemval) == 0)
	    {
	      if(pcb->status & ST_FSEMAPHORE)
	        {
		  pcb->status = (pr_state_t) (pcb->status & (~ST_RECEIVE_FSEM));
		  
		  /* If the process becomes ready then do a swap. */
		  if(pcb->status < ST_READY)
		    {
		      make_otherpri_ready(pcb); /* swap */
		    }
	        }
	    }
        }

      /* If target process is an interrupt process then */
      /* request a wakeup call. */
      else if(pcb->priority < 96)
        {
	  /*
	  ** There is a potential race condition here.
	  ** If we enable interrupts before entering wakeup, it is possible
	  ** that the caller is killed before the wakeup call takes places,
	  ** and the wakeup call is missed.
	  ** to avoid this, zzenter_wakeup is called with interrupts
	  ** still disabled. I.e. the zzenter_wakeup function can not look at
	  ** the current status register to determine the old priority. It must
	  ** rely on the current interrupt level stored in the target interface.
	  ** Each time zzenter_wakeup is called from non int level, interrupts must
	  ** be disabled. When called from interrupt level, interrupts need
	  ** not be disabled higher than to the current level.
	  */
	  zzenter_wakeup(pcb->uarea, (void *) pcb, pcb->priority & 31, (OSBOOLEAN) 2);
	  LOCK_RESTORE(msr);
	  return;
        }

      /* If target process is a timer process then */
      /* request a wakeup call. */
      else if(pcb->priority < 128)
        {
	  zzenter_wakeup(pcb->uarea, (void *) pcb, zzuu.timer_int_priority, (OSBOOLEAN) 2);
	  LOCK_RESTORE(msr);
	  return;
        }

      /* Enable interrupts and quit. */
      LOCK_RESTORE(msr);
      return;
    }
}

/*
** This is the signal_fsem_from_isr system call. It is only valid to
** call from an ISR context. 
*/
static OSERRCODE
zzkrn_signal_fsem_from_isr(PROCESS pid)
{
  register struct _pcb *pcb;
  Msr msr;

  /* Lock interrupts just in case we are called from non ISR context. */
  LOCK_SAVE(msr);
  
  /* Read instance number and check the process id. Report error if invalid. */
  pcb = safe_pid_to_pcb(pid, 0); 
  if(!pcb)
    {
      LOCK_RESTORE(msr);
      return OSE_EILLEGAL_PROCESS_ID;
    }

  /* Interrupts are locked. */

#if ERROR_CHECKS > 0
  /*
  ** Check that the operation wouldn't cause the semaphore value to wrap from
  ** positive to negative values.
  */
  ON_ERRLEVEL_2()
    if(pcb->priority < 33 && pcb->fsemval == 0x7fffffff)
      {
        LOCK_RESTORE(msr);
        return OSE_ESEM_OVERFLOW;
      }
#endif

#ifdef OSE_SMP
  /* If the process runs on another cpu, let that one do the job. */
  {
    cpuid_t cpu = pcb->cpu_id;
    if(cpu != zzkrn_cpu_id())
      {
        send_krn_event(cpu, KRN_EVT_SIGNAL_FSEM, pid, NULL);
        LOCK_RESTORE(msr);
        return 0;
      }
  }
#endif

  /* If target process is a prioritized or background process */
  /* then signal the semaphore and make it ready. */
  if(pcb->priority < 33)
    {
      if(++(pcb->fsemval) == 0)
        {
          if(pcb->status & ST_FSEMAPHORE)
            {
              pcb->status = (pr_state_t) (pcb->status & (~ST_RECEIVE_FSEM));
		  
              /* Make it ready. */
              if(pcb->status < ST_READY)
                {
                  /* But we don't want to swap here! */
                  make_otherpri_ready_noswap(pcb); 
                }
            }
        }
    }

  /* If target process is an interrupt process then */
  /* request a wakeup call. */
  else if(pcb->priority < 96)
    {
      request_int_semwakeup(pcb);
      LOCK_RESTORE(msr);
      return 0;
    }

  /* If the target process is a timer process then request a wakeup call. */
  /*
  ** A stopped timer process will get called anyway.?? This seems to be the
  ** most reasonable interpretation of the call.
  */
  else if(pcb->priority < 128)
    {
      request_timer_semwakeup(pcb);
      LOCK_RESTORE(msr);
      return 0;
    }

  /* Return */
  LOCK_RESTORE(msr);
  return 0;
}

/*
**===========================================================================
** 19.4  Undocumented system calls.
**===========================================================================
*/

/*
*****************************************************************************
** 20 SEMAPHORES.
*****************************************************************************
*/

/*
** A semaphore is attached to a memory segment. Only processes in this segment
** can use the semaphore. The semaphore is killed when the last block in the
** segment dies or when a semphore is explicitely killed.
** 
** A semaphore consists of a user space semaphore structure and a supervisor
** space kernel structure.
** 
** The user semaphore structure is in user space. It is either a static structure
** or allocated from the pool attached to the creating process. The kernel is
** the owner of this structure so it is not deallocated as a direct result
** of a process being killed. The first field in this structure contains the initial
** positive value of the semaphore. The second field contains a pointer to the kernel
** structure or NULL if no kernel structure has yet been allocated. The third field
** contains zero and is not used by the kernel.
** 
** The kernel structure is always allocated from the system pool, either
** on demand or when the semaphore is created. The kernel is the owner
** of this structure too. The kernel structure contains a magic number indicating
** a valid semaphore, a flag indicating a static semaphore, the current semaphore
** value, backward and forward pointers in the semaphore queue, and a pointer to
** the user semaphore structure.
*/

/*
**===========================================================================
** 20.1  Initialization.
**===========================================================================
*/

/* Initiate mutex handling. */

static void
init_mutex(void)
{
  /* Mutex index zero is never used, and the free mutex list is empty. */
  zzuu.next_free_mutex_index = 1;
  zzuu.mutex_free_list = NULL;

  /* Don't annotate dynamic semaphores. */
  zzuu.annotate_semaphores = 0;
}

/*
**===========================================================================
** 20.2  Local functions.
**===========================================================================
*/

#ifndef ASSERT
#define assert_mutex_freelist(line)
#define assert_mutlist(segcb, line)
#else

static void
assert_mutex_freelist(int line)
{
  struct mutcb *mutcb;
  U32 cnt = 0;

  for(mutcb = zzuu.mutex_free_list; mutcb; mutcb = mutcb->next)
    {
      cnt++;
      assert_from(cnt < zzuu.next_free_mutex_index, line);
      assert_from(mutcb->index < zzuu.next_free_mutex_index, line);
      assert_from(NULL == mutcb->umutex, line);
      assert_from(MAGIC_MUTEX_DESTROYED == mutcb->magic, line);
    } /* for */
}

static void
assert_mutlist(struct segcb *segcb, int line)
{
  struct mutcb *mutcb;
  U32 cnt = 0;

  for(mutcb = segcb->mutlist; mutcb; mutcb = mutcb->next)
    {
      cnt++;
      assert_from(cnt < zzuu.next_free_mutex_index, line);
      assert_from(mutcb->index < zzuu.next_free_mutex_index, line);
      assert_from(NULL != mutcb->umutex, line);
      assert_from(MAGIC_MUTEX == mutcb->magic, line);
      assert_from(segcb == mutcb->segcb, line);
      assert_from(NULL != mutcb->sem, line);
      if(cnt == 1)
	{
	  assert_from(NULL == mutcb->prev, line);
	}
      else
	{
	  assert_from(NULL != mutcb->prev, line);
	  assert_from(mutcb == mutcb->prev->next, line);
	}
      if(mutcb->next)
	{
	  assert_from(mutcb == mutcb->next->prev, line);
	}
    } /* for */
}
#endif

/* 
** Kill all mutexes in a segment if needed.
** Subfunction to the system daemon sysd.
**
** This function is called each time a block has been killed. If there
** are no more blocks in the segment, then the mutex list is
** killed.
** Must be called after the block reference count in the segment
** control block has been decremented.
*/
static void 
cond_kill_mutex(struct segcb *segcb)
{
  struct mutcb *mutcb;
  register struct mutcb *next_mutcb;

  assert_mutlist(segcb, __LINE__);

  /* Done if there are still blocks in the segment. */
  if(segcb->block_count)
    return;

  /* Get the mutex list. */
  mutcb = segcb->mutlist;
  segcb->mutlist = NULL;

  /* Free each mutex in the list of the segment. */
  for(; mutcb; mutcb = next_mutcb)
    {
#ifdef ASSERT
      ramlog_printf("cond_kill_mutex index %#x instance %#x mutex %#x segment %#x\n",
		mutcb->index, mutcb->instance, mutcb->umutex, pcb_to_pid(mutcb->segcb));
#endif
      /* Pick up the next mutex in the list and save it for later. */
      next_mutcb = mutcb->next;

      /* Scratch the magic number and bump instance. */
      mutcb->magic = MAGIC_MUTEX_DESTROYED;
      mutcb->instance++;
      mutcb->umutex = NULL;

      /* Put the mutex into the free list. */
      BKL_LOCK();
      assert_mutex_freelist(__LINE__);

      mutcb->next = zzuu.mutex_free_list;
      zzuu.mutex_free_list = mutcb;

      assert_mutex_freelist(__LINE__);
      assert_mutlist(segcb, __LINE__);
      BKL_UNLOCK();
    }
}

/*
** Kill all semaphores in a segment if needed.
** Subfunction to the system daemon sysd.
**
** This function is called each time a block has been killed. If there
** are no more blocks in the segment, then the semaphore list is
** killed.
** Must be called after the block reference count in the segment
** control block has been decremented.
**
** There is an error reporting problem if processes are still waiting at some
** semaphore. Errors can not be reported by the process causing the error, so
** the system daemon must report a system error.
** This error must be managed by killing the process indicated in the extra
** parameter.
*/

static void
cond_kill_sem(struct segcb *segcb)
{
  struct semcb *semcb;
  register struct semcb *next_semcb;
  PROCESS pid;

  /* Done if there are still blocks in the segment. */
  if(segcb->block_count)
    return;

  /* Get the semaphore list. */
  semcb = segcb->semlist;
  segcb->semlist = NULL;

  /* For each semaphore in the list. */
  for(; semcb; semcb = next_semcb)
    {
      /* Pick up the next semaphore in list and save it for later. */
      next_semcb = semcb->next;

      /* Scratch the magic number. */
      semcb->magic = (U16) 0;

      /* Check that no processes are waiting at the semaphore. */
    cond_kill_sem1:
      BKL_LOCK();
      if(semcb->value < 0)
	{
	  /* Get the id of the offending process. */
	  pid = pcb_to_pid(semcb->fq);
	  BKL_UNLOCK();

	  /* Report error. */
	  os_error(1, OSE_EINTERSEGMENT_SEMAPHORE_REF, (OSERRCODE) pid);	/* Non fatal system error. */

	  /* Kill the process just in case the error handler did not do it. */
	  kill_proc(pid);

	  /* Loop until no more processes waiting. */
	  goto cond_kill_sem1;
	}
      BKL_UNLOCK();

      /* Free the buffers involved. */
      if(!semcb->static_sm)
	sys_free_buf((union SIGNAL **) &semcb->usersem);
      sys_free_buf_type(struct semcb *, &semcb);
    }
}

/*
** Subfunction to kill_proc.
** Called for each non interrupt process killed.
**
** If a process being killed is waiting in a receive_sport() call
** then unlink it from all s_nodelists in all phantoms.
**
** Must be called with interrupts disabled.
*/

static void
kill_proc_abort_sport(struct _pcb *pcb)
{
  struct S_node *s_node;
  struct S_nodelist *s_nodelist;
  U32 i;

  /* Get the s_nodelist structure. */
  s_nodelist = (struct S_nodelist *) pcb->s_pcb.s_nodelist;
  /* See if the process is in a receive_sport() call. */
  if(s_nodelist->isactive)
    {
      /* Yes, unlink it from all phantoms. */
      s_nodelist->isactive = 0;
      for(s_node = s_nodelist->s_node, i = s_nodelist->curcount; i; i--, s_node++)
	{
	  struct _pcb *ppcb = s_node->phantom_pcb;
	  /* Skip inactive entries. (sports killed during a receive_sport() call) */
	  if(ppcb)
	    {
	      /* Unlink at head of list. */
	      if(!s_node->bp_s_nodelist)
		{
		  ppcb->s_pcb.s_nodelist = s_node->fp_s_nodelist;
		  if(s_node->fp_s_nodelist)
		    s_node->fp_s_nodelist->bp_s_nodelist = NULL;
		}
	      /* Unlink within the list. */
	      else
		{
		  s_node->bp_s_nodelist->fp_s_nodelist = s_node->fp_s_nodelist;
		  if(s_node->fp_s_nodelist)
		    s_node->fp_s_nodelist->bp_s_nodelist = s_node->bp_s_nodelist;
		  else
		    ppcb->s_pcb.s_nodetail = s_node->bp_s_nodelist;
		}
	    }
	}
    }
}

/*
** Subfunction to kill_proc.
** Called for each non interrupt process killed.
** (Safe to call also for other process types.)
**
** If a process being killed is waiting at a semaphore then unlink it from
** the semaphore queue and compensate the semaphore value.
** The process is not made ready.
** The wanted field in the process control block points to the semaphore
** in use.
**
** Must be called with interrupts disabled.
*/

static void
kill_proc_cancel_sem(struct _pcb *pcb)
{
  register struct semcb *semcb;

  semcb = DECONST(struct semcb *, pcb->wanted);

  /* Compensate semaphore value. */
  semcb->value++;

  /* Unlink pcb from semaphore queue. */
  if(pcb->bq)
    {
      pcb->bq->fq = pcb->fq;
    }
  else
    {
      semcb->fq = pcb->fq;
    }
  if(pcb->fq)
    {
      pcb->fq->bq = pcb->bq;
    }
  else
    {
      semcb->bq = pcb->bq;
    }
  if(semcb->fq)
    {
      semcb->fq->bq = semcb->bq->fq = NULL;
    }
}

/*
** Subfunction to wait_sem.
**
** A static semaphore is used for the first time and we need to create
** the kernel part of the semaphore.
**
** There is a race condition here, so we must check if some other
** process also creates the kernel structure and back off if we turn
** out to be too late.
**
** This function is called with interrupts enabled and it should return with
** interrupts enabled.
**
** Only non interrupt processes call this function.
*/

static void
create_static_sem(SEMAPHORE *usersem)
{
  struct semcb *semcb;
  register struct segcb *segcb;

  /*
  ** Allocate a kernel space structure. The size given as offset of
  ** the last (and unused) member, end_mark, to avoid unneccessary
  ** cache line alignment.
  */
  semcb = (struct semcb *)sys_alloc(offsetof(struct semcb, end_mark),
				    (SIGSELECT) 0);

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Free the kernel structure again if someone else already created it. */
  if(usersem->f)
    {
      /* Unlock the dispatcher. */
      UNLOCK_SWAP();

      /* Free the redundant kernel semaphore structure. */
      free_buf_type(struct semcb *, &semcb);

      /* Retry the entire semaphore operation. */
      return;
    }

  /* Make the system the owner of the new kernel structure. */
  change_sig_owner(sig_to_adm(semcb), NULL);

  /* Initialize the kernel space structure. */
  semcb->magic = (U16) MAGIC_SEMAPHORE;
  semcb->fq = NULL;
  semcb->bq = NULL;
  semcb->usersem = usersem;
  semcb->static_sm = (OSBOOLEAN) 1;

  /* Disable interrupts. */
  BKL_LOCK();

  /* Copy the initial value to the kernel space structure. */
  semcb->value = usersem->value;

  /* Initialize the user space structure. */
  usersem->f = (void *) semcb;

  /* Enable interrupts. */
  BKL_UNLOCK();

  /* Locate the memory segment descriptor of the calling process. */
  if(zzcur_p->block->segpid)
    segcb = (struct segcb *) pid_to_pcb(zzcur_p->block->segpid);
  else
    segcb = &system_segment;

  /* Link the semaphore into the segment control block. */
  semcb->prev = NULL;
  semcb->segcb = segcb;
  semcb->next = segcb->semlist;
  if(semcb->next)
    {
      semcb->next->prev = semcb;
    }
  segcb->semlist = semcb;

  /* Unlock the dispatcher and return. */
  UNLOCK_SWAP();
}


/*
** Subfunction of mutex operations.
**
** Creates a semaphore in the system pool, connected to the callers segment.
** The system pool is used because we want to avoid page faults in paged
** pools. The user space semaphore object is only referenced from supervisor
** mode.
** 
** The dispatcher must be locked.
*/

static SEMAPHORE *
locked_create_sem(OSSEMVAL  initial_val,
		  OSADDRESS return_address,
		  OSBOOLEAN use_sys_seg)
{
  register struct semcb *semcb;
  register struct semaphore *usersem;
  register struct segcb *segcb;
  U8 annotate = zzuu.annotate_semaphores;

  /* Use the system pool. */
  set_suid(1);

  /* Allocate a user space structure. */
  usersem = (struct semaphore *)
    locked_alloc(annotate ? sizeof(struct annotated_semaphore)
		 : sizeof(struct semaphore),
		 (SIGSELECT) 0);

  /*
  ** Allocate a kernel space structure. The size given as offset of
  ** the last (and unused) member, end_mark, to avoid unneccessary
  ** cache line alignment.
  */
  semcb = (struct semcb *) locked_alloc(offsetof(struct semcb, end_mark),
					(SIGSELECT) 0);

  /* Use the application pool again. */
  set_suid(0);

  /* Initialize the user space structure. */
  usersem->value = initial_val;
  usersem->f = (void *) semcb;
  usersem->b = NULL;
  if(annotate)
    {
      struct annotated_semaphore *anno_sem;
      usersem->b = SEMAPHORE_ANNOTATED;
      anno_sem = (struct annotated_semaphore *)usersem;
      anno_sem->return_address = return_address;
      anno_sem->creator = pcb_to_pid(zzcur_p);
    }

  /* Initialize the kernel space structure. */
  semcb->magic = (U16) MAGIC_SEMAPHORE;
  semcb->value = initial_val;
  semcb->fq = NULL;
  semcb->bq = NULL;
  semcb->usersem = usersem;
  semcb->static_sm = (OSBOOLEAN) 0;

  /* Locate the memory segment descriptor of the calling process. */
  if(!use_sys_seg && zzcur_p->block->segpid)
    segcb = (struct segcb *) pid_to_pcb(zzcur_p->block->segpid);
  else
    segcb = &system_segment;

  /* Make the system the owner of both the kernel and user structures. */
  change_sig_owner(sig_to_adm(usersem), NULL);
  change_sig_owner(sig_to_adm(semcb), NULL);

  /* Link the semaphore into the segment control block. */
  semcb->prev = NULL;
  semcb->segcb = segcb;
  semcb->next = segcb->semlist;
  if(semcb->next)
    {
      semcb->next->prev = semcb;
    }
  segcb->semlist = semcb;

  return usersem;
}

static SEMAPHORE *
local_create_sem(OSSEMVAL initial_val, OSADDRESS return_address)
{
  register struct semcb *semcb;
  register struct semaphore *usersem;
  register struct segcb *segcb;
  U8 annotate = zzuu.annotate_semaphores;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_CREATE_SEM))
    return NULL;
#endif

  /* Check that the initial value is non negtive. */
  if(initial_val < 0)
    {
      /* Error if attempt to initialize a semaphore to a neg value. */
      os_error(0, OSE_EILLEGAL_SEM_VALUE | OSE_CREATE_SEM, (OSERRCODE) 0);	/* Non fatal. */
      initial_val = 0;
    }    

  /* Allocate a user space structure. */
  usersem = (struct semaphore *)
    alloc(annotate ? sizeof(struct annotated_semaphore)
	  : sizeof(struct semaphore),
	  (SIGSELECT) 0);

  /*
  ** Allocate a kernel space structure. The size given as offset of the last
  ** (and unused) member, end_mark, to avoid unneccessary cache line alignment.
  */
  semcb = (struct semcb *) sys_alloc(offsetof(struct semcb, end_mark), (SIGSELECT) 0);

  /* Initialize the user space structure. */
  usersem->value = initial_val;
  usersem->f = (void *) semcb;
  usersem->b = NULL;
  if(annotate)
    {
      struct annotated_semaphore *anno_sem;
      usersem->b = SEMAPHORE_ANNOTATED;
      anno_sem = (struct annotated_semaphore *)usersem;
      anno_sem->return_address = return_address;
      anno_sem->creator = pcb_to_pid(zzcur_p);
    }

  /* Initialize the kernel space structure. */
  semcb->magic = (U16) MAGIC_SEMAPHORE;
  semcb->value = initial_val;
  semcb->fq = NULL;
  semcb->bq = NULL;
  semcb->usersem = usersem;
  semcb->static_sm = (OSBOOLEAN) 0;

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Locate the memory segment descriptor of the calling process. */
  if(zzcur_p->block->segpid)
    segcb = (struct segcb *) pid_to_pcb(zzcur_p->block->segpid);
  else
    segcb = &system_segment;

  /* Make the system the owner of both the kernel and user structures. */
  change_sig_owner(sig_to_adm(usersem), NULL);
  change_sig_owner(sig_to_adm(semcb), NULL);

  /* Link the semaphore into the segment control block. */
  semcb->prev = NULL;
  semcb->segcb = segcb;
  semcb->next = segcb->semlist;
  if(semcb->next)
    {
      semcb->next->prev = semcb;
    }
  segcb->semlist = semcb;

  /* Unlock the dispatcher and return the new semaphore. */
  UNLOCK_SWAP();
  return usersem;
}

/*
**===========================================================================
** 20.3  System calls.
**===========================================================================
*/

/*
** This is the original create_sem system call.
*/

SEMAPHORE *
create_sem(OSSEMVAL initial_val)
{
  return local_create_sem(initial_val, 0);
}

/*
** This is the newer create_sem system call, for annotation of returnpoint.
*/

static OSADDRESS
sysconf_82(OSADDRESS p, OSADDRESS return_address, OSADDRESS r)
{
  OSSEMVAL initial_val = (OSSEMVAL)p;

  (void)r;
  return (OSADDRESS)local_create_sem(initial_val, return_address);
}


/*
** Helper function to the kill_sem system call.
*/

static void
sys_kill_sem(SEMAPHORE *usersem, OSBOOLEAN is_sys_sem)
{
  struct semcb *semcb;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_KILL_SEM))
    return;
#endif

  /* Check for an attempt to pass us the NULL pointer. */
  if(!usersem)
    {
      os_error(0, OSE_EILLEGAL_SEMAPHORE | OSE_KILL_SEM, (OSERRCODE) 0);	/* Non fatal. */
      return;
    }

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Locate the kernel semaphore structure. */
  semcb = (struct semcb *) usersem->f;

  /* Check that it is not an uninitialized static semaphore. */
  if(!semcb)
    {
      UNLOCK_SWAP();
      os_error(0, OSE_EKILLED_STATIC_SEMAPHORE | OSE_KILL_SEM, (OSERRCODE) 0);	/* Non fatal. */
      return;
    }

  /* Check the semaphore magic number. */
  if(semcb->magic != MAGIC_SEMAPHORE)
    {
      UNLOCK_SWAP();
      os_error(0, OSE_EILLEGAL_SEMAPHORE | OSE_KILL_SEM, (OSERRCODE) 0);	/* Non fatal. */
      return;
    }

  /* Check that it is not a static semaphore. */
  if(semcb->static_sm)
    {
      UNLOCK_SWAP();
      os_error(0, OSE_EKILLED_STATIC_SEMAPHORE | OSE_KILL_SEM, (OSERRCODE) 0);	/* Non fatal. */
      return;
    }

  /* Disable interrupts. */
  BKL_LOCK();

  /* Check that no process is waiting at the semaphore. */
  if(semcb->fq)
    {
      PROCESS waiting_pid = pcb_to_pid(semcb->fq);
      
      LOCKED_UNLOCK_SWAP();
      os_error(0, OSE_EKILLED_SEMAPHORE_IN_USE | OSE_KILL_SEM, /* Non fatal. */
	       (OSERRCODE) waiting_pid);
      return;
    }

  /* Scratch the magic number. */
  semcb->magic = (U16) 0;

  /* Enable interrupts. */
  BKL_UNLOCK();

  /* Check if the semaphore belongs to the segment of the current process */
  if(( is_sys_sem && semcb->segcb->segid) ||
     (!is_sys_sem && (semcb->segcb->segid != zzcur_p->block->segid)))
    {
      UNLOCK_SWAP();
      os_error(0, OSE_EILLEGAL_SEMAPHORE | OSE_KILL_SEM, (OSERRCODE) usersem); /* Non fatal. */
      return;
    }

  /* Unlink semaphore from the segment list */
  if(semcb->next)
    {
      semcb->next->prev = semcb->prev;
    }
  if(semcb->prev)
    {
      semcb->prev->next = semcb->next;
    }
  else
    {
      semcb->segcb->semlist = semcb->next;
    }

  /* Unlock the dispatcher and free the buffers involved. */
  change_sig_owner(sig_to_adm(usersem), zzcur_p);
  change_sig_owner(sig_to_adm(semcb), zzcur_p);
  UNLOCK_SWAP();
  free_buf_type(struct semcb *, &semcb);
  free_buf_type(SEMAPHORE *, &usersem);
}


/*
** This is the kill_sem system call.
*/

void
kill_sem(SEMAPHORE *usersem)
{
  sys_kill_sem(usersem, (OSBOOLEAN)0);
}

/*
** This is the get_sem system call.
*/

OSSEMVAL
get_sem(SEMAPHORE *usersem)
{
  OSSEMVAL value;
  register struct semcb *semcb;
  LOCKSTATE msr;

  /* Check for an attempt to pass us the NULL pointer. */
#if ERROR_CHECKS > 0
  ON_ERRLEVEL_2()
    if(unlikely(!usersem))
      {
	os_error(0, OSE_EILLEGAL_SEMAPHORE | OSE_GET_SEM, (OSERRCODE) 0);	/* Non fatal. */
	return (OSSEMVAL) 0;
      }
#endif

  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);

  /* Read the initial value if no kernel structure is present. */
  semcb = (struct semcb *) usersem->f;
  if(!semcb)
    {
      value = usersem->value;
      BKL_LOCK_RESTORE(msr);
      return value;
    }

  /* Check the magic number of the kernel structure. */
#if ERROR_CHECKS > 0
  if(unlikely(semcb->magic != MAGIC_SEMAPHORE))
    {
      BKL_LOCK_RESTORE(msr);
      os_error(0, OSE_EILLEGAL_SEMAPHORE | OSE_GET_SEM, (OSERRCODE) 0);	/* Non fatal. */
      return (OSSEMVAL) 0;
    }
#endif

  /* Read the current value. */
  value = semcb->value;

  /* Reenable interrupts and exit. */
  BKL_LOCK_RESTORE(msr);
  return value;
}

/*
** This is the signal_sem system call.
*/

void
signal_sem(SEMAPHORE *usersem)
{
  register struct semcb *semcb;
  register struct _pcb *pcb;
  LOCKSTATE msr;

  /* Check for an attempt to pass us the NULL pointer. */
#if ERROR_CHECKS > 0
  if(unlikely(!usersem))
    {
      os_error(0, OSE_EILLEGAL_SEMAPHORE | OSE_SIGNAL_SEM, (OSERRCODE) 0);	/* Non fatal. */
      return;
    }
#endif

  /* Disable interrupts. */
  BKL_LOCK_SAVE(msr);

#if ERROR_CHECKS > 0
  /*
  ** Check that the operation wouldn't cause the semaphore value to wrap from
  ** positive to negative values.
  */
  ON_ERRLEVEL_2()
    if(unlikely(usersem->value == 0x7fffffff))
      {
	BKL_LOCK_RESTORE(msr);
	os_error(0, OSE_ESEM_OVERFLOW | OSE_SIGNAL_SEM, (OSERRCODE) usersem);	/* Non fatal. */
	/* Terminate the system call. */
	return;
      }
#endif

  /*
  ** Increment the initial value if no kernel structure is present.
  ** This also implies that no process is waiting.
  */
  semcb = (struct semcb *) usersem->f;
  if(!semcb)
    {
      usersem->value++;
      BKL_LOCK_RESTORE(msr);
      return;
    }

  /* Check the magic number of the kernel structure. */
#if ERROR_CHECKS > 0
  if(unlikely(semcb->magic != MAGIC_SEMAPHORE))
    {
      BKL_LOCK_RESTORE(msr);
      os_error(0, OSE_EILLEGAL_SEMAPHORE | OSE_SIGNAL_SEM, (OSERRCODE) 0);	/* Non fatal. */
      return;
    }
#endif

  /* Increment sem value and exit if no process is waiting. */
  if(++semcb->value > 0)
    {
      BKL_LOCK_RESTORE(msr);
      return;
    }

  /* Unlink first waiting process from semaphore queue. */
  pcb = semcb->fq;
  semcb->fq = pcb->fq;
  if(semcb->fq)
    {
      semcb->fq->bq = NULL;
    }

  /* Let go BKL, but keep the interrupt mask. */
  BKL_UNLOCK_KEEP_INT_MASK();

  /* Make the unlinked process ready and request dispatch if required. */
  {
#ifdef OSE_SMP
    cpuid_t cpu = pcb->cpu_id;
    /* If the process runs on another cpu, let that one do the job. */
    if(cpu != zzkrn_cpu_id())
      {
	send_krn_event(cpu, KRN_EVT_SIGNAL_SEM, pcb_to_pid(pcb),
		       NULL);
      }
    else
#endif
      {
	pcb->status  = (pr_state_t) (pcb->status & (~ST_SEMAPHORE));
	if(pcb->status < ST_READY)
	  {
	    make_otherpri_ready(pcb); /* swap */
	  }
      }
  }
  /* Reenable interrupts. */
  /* Is the caller an interrupt or timer process? Yes, it is. */
  if(zzcur_p->priority > 32)
    {
      LOCK_RESTORE(msr);
    }
  else
    {
      /* No, it is not. */
      UNLOCK();
    }
}

/*
** This is the wait_sem system call.
*/

void
wait_sem(SEMAPHORE *usersem)
{
  register struct semcb *semcb;
  register struct _pcb *pcb;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  ON_ERRLEVEL_2()
    if(check_nonint_caller(OSE_WAIT_SEM))
      return;
#endif

  /* Check for an attempt to pass us the NULL pointer. */
#if ERROR_CHECKS > 0
  if(unlikely(!usersem))
    {
      os_error(0, OSE_EILLEGAL_SEMAPHORE | OSE_WAIT_SEM, (OSERRCODE) 0);	/* Non fatal. */
      return;
    }
#endif

wait_sem1:

  /* Disable interrupts. */
  BKL_LOCK();

  /*
  **  Allocate a kernel structure if not there.
  ** This situation occurs when a static semaphore is used
  ** for the first time.
  */
  semcb = (struct semcb *) usersem->f;
  if(!semcb)
    {
      BKL_UNLOCK();
      create_static_sem(usersem);
      goto wait_sem1;
    }

  /* Check the magic number of the kernel structure. */
#if ERROR_CHECKS > 0
  if(unlikely(semcb->magic != MAGIC_SEMAPHORE))
    {
      BKL_UNLOCK();
      os_error(0, OSE_EILLEGAL_SEMAPHORE | OSE_WAIT_SEM, (OSERRCODE) 0);	/* Non fatal. */
      return;
    }
#endif

  /* Decrement current value. */
  if((--semcb->value) >= 0)
    {
      /* Done if still positive. */
      BKL_UNLOCK();
      return;
    }
  
  /* Link caller into semaphore list. */
  pcb = zzcur_p;
  if(semcb->value == -1)
    {	/* List empty. */
      pcb->fq = pcb->bq = NULL;
      semcb->fq = semcb->bq = pcb;
    }
  else
    {	/* List not empty. */
      pcb->fq = NULL;
      pcb->bq = semcb->bq;
      semcb->bq->fq = pcb;
      semcb->bq = pcb;
    }

  /* Let go BKL, but keep the interrupt mask. */
  BKL_UNLOCK_KEEP_INT_MASK();

  /*
  ** Store the semaphore pointer in the pcb in case we are killed during
  ** the wait_sem() call.
  */
  pcb->wanted = (SIGSELECT *) semcb;

  /* Make not ready and swap. */
  pcb->status  = (pr_state_t) (pcb->status | ST_SEMAPHORE);
  make_mypri_not_ready(pcb);

  /* Enable interrupts and exit. */
  UNLOCK();
}

/*
**===========================================================================
** 20.4  Undocumented system calls.
**===========================================================================
*/

/*
** Turn semaphore annotation on or off. Returns the previous flag setting.
** When annotation is on, dynamically created SEMAPHORE structs are extended
** by eight bytes, containing the address of the creation call and the
** creators PROCESS identifier, in that order. Also, the b member of the 
** SEMAPHORE is set to SEMAPHORE_ANNOTATED to indicate that the SEMAPHORE 
** is annotated.
*/

OSBOOLEAN zzkrn_set_semaphore_annotation(OSBOOLEAN activate)
{
  OSBOOLEAN old_value = zzuu.annotate_semaphores;
  zzuu.annotate_semaphores = activate;
  return old_value;
}

/* Make zzkrn_set_semaphore_annotation() available for test programs. */
static OSADDRESS
sysconf_83(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  (void)q;
  (void)r;
  return (OSADDRESS)zzkrn_set_semaphore_annotation((OSBOOLEAN)p);
}


/*
** This is the mutex taste of the wait_sem system call.
**
** The mutex semaphore has a queue that is ordered in priority order.
** This kind of semaphore is only signalled and waited on by non-interrupt
** processes. 
** As the queue is ordered, wait is not a fix time call.
** mutex_wait_sem() is called with, and returns with BKL_LOCK().
** We should use a local lock for semaphores??
*/

static void
mutex_wait_sem(SEMAPHORE *usersem)
{
  register struct semcb *semcb = (struct semcb *) usersem->f;
  register struct _pcb *pcb;


  /* Check the magic number of the kernel structure. */
#if ERROR_CHECKS > 0
  if(unlikely(semcb->magic != MAGIC_SEMAPHORE))
    {
      BKL_UNLOCK();
      os_error(1, OSE_EILLEGAL_SEMAPHORE | OSE_WAIT_SEM, (OSERRCODE) 0); /* Non fatal. */
      os_error(2, OSE_EILLEGAL_SEMAPHORE | OSE_WAIT_SEM, (OSERRCODE) 0); /* Fatal. */
      /*NOTREACHED*/
    }
#endif

  /* Decrement current value. */
  if((--semcb->value) >= 0)
    {
      /* Done if still positive. */
      return;
    }
  
  /* Link caller into semaphore list. */
  pcb = zzcur_p;
  if(semcb->value == -1)
    {	/* List empty. */
      pcb->fq = pcb->bq = NULL;
      semcb->fq = semcb->bq = pcb;
    }

  /* List not empty. Maintain priority order. */
  else
    {	
      OSPRIORITY prio = pcb->priority;
      struct _pcb *qpcb = semcb->fq;

      if(prio < qpcb->priority)
	{
	  /* Enter first in non-empty list. */
	  pcb->fq = qpcb;
	  pcb->bq = NULL;
	  semcb->fq = qpcb->bq = pcb;
	}
      else
	{
	  for(;;)
	    {
	      qpcb = qpcb->fq;
	      if(!qpcb)
		{
		  /* Enter last in the non-empty list. */
		  qpcb = semcb->bq;
		  pcb->fq = NULL;
		  pcb->bq = qpcb;
		  qpcb->fq = semcb->bq = pcb;
		  break;
		}
	      if(prio < qpcb->priority)
		{
		  /* Enter inside the list. */
		  pcb->fq = qpcb;
		  pcb->bq = qpcb->bq;
		  qpcb->bq->fq = pcb;
		  qpcb->bq = pcb;
		  break;
		}
	    } /* for */
	}
    }

  /*
  ** Store the semaphore pointer in the pcb in case we are killed during
  ** the wait_sem() call.
  */
  pcb->wanted = (SIGSELECT *) semcb;

  /* Make not ready and swap. */
  BKL_UNLOCK_KEEP_INT_MASK();

  pcb->status  = (pr_state_t) (pcb->status | ST_SEMAPHORE);
  make_mypri_not_ready(pcb);

  BKL_LOCK();
}

static PCB_INSTANCE_T
mutex_check_owner(PROCESS pid)
{
  const PCB_INDEX_T pindex = pid_to_pindex(pid);
  PCB_INSTANCE_T instance = pid_to_instance(pid);
  register struct _pcb *pcb;

  /* Check that the process index is reasonable. */
  if(pindex >= zzuu.next_free_pix)
    {
      return (PCB_INSTANCE_T) 0;
    }

  /* Pick up pcb pointer. */
  pcb = zzuu.pix_list[pindex];

  /* Disable interrupts. */
  BKL_LOCK();

  /* Return zero if the process has been killed. */
  /* FIXME! for LARGE_PID_SPACE @frbr@ */
  if((pcb_to_instance(pcb) & 0x7FFF) != (instance & 0x7FFF))
    {
      /* Enable interrupts and quit. */
      BKL_UNLOCK();
      return (PCB_INSTANCE_T) 0;
    }

  /* Check that the magic number indicates a process. */
  if(pcb->magic != MAGIC_PCB)
    {
      /* Enable interrupts and quit. */
      BKL_UNLOCK();
      return (PCB_INSTANCE_T) 0;
    }

  /* Pick up the 'real' instance number. */
  instance = pcb_to_instance(pcb);

  /* Enable interrupts. */
  BKL_UNLOCK();
  
  /* Return instance number if no problem was found. */
  return instance;
}

/*
** Subfunction to sysconf_59.
** This function is called by prioritized and background processes.
** Returns: 0 = Enter the mutex queue, 1 = Try to get the mutex, 
** or > 1 = Error code.
** Interrupts are locked by the caller. Interrupts are enabled internally, 
** but are again locked at return.
*/
static OSERRCODE
mutex_raise_priority(struct mutcb *mutcb)
{
  struct _mutex *mutex = mutcb->umutex;
  struct _pcb *curr = zzcur_p;
  struct _pcb *pcb;
  volatile PROCESS owner;
  PCB_INSTANCE_T instance;
  SMP_CODE(cpuid_t cpu);

  /*
  ** Check that we are a prioritized process with a higher priority
  ** than the lowest.
  */
  if(curr->priority > 30)
    {
      /* We return with interrupts disabled. */
      return 0; /* Enter the mutex queue. */
    }

 again:
  /* Sample the mutex owner's process id. */
  owner = mutex->owner;

  if(!owner)
    {
      /*
      ** The mutex is currently not locked.
      ** We return with interrupts disabled.
      */
      return 1; /* Try to get the mutex. */
    }

  BKL_UNLOCK();
  
  /* Check if the stated owner is a valid (and alive) process. */
  instance = mutex_check_owner(owner);
  if(!instance)
    {
      /* Lock the dispatcher. */
      LOCK_SWAP();

      if((mutex->owner & 0x7FFFFFFFUL) == (owner & 0x7FFFFFFFUL))
	{
	  /*
	  ** The mutex is still locked but the owner has died.
	  */
	dead_owner:
	  UNLOCK_SWAP();
	  BKL_LOCK();
	  return OSE_EMUTEX_OWNER_DEAD;
	}
      else
	{
	  /*
	  ** The mutex may be locked by a new owner. We therefor
	  ** need to take a new owner sample.
	  */
	  UNLOCK_SWAP();
	  BKL_LOCK();
	  goto again;
	}
    }

  /* Locate the mutex owner's process descriptor. */
  pcb = pid_to_pcb(owner);

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check if the mutex is still locked by owner. */
  if(mutex->owner != owner)
    {
      /* If not, unlock the dispatcher and exit. */
      UNLOCK_SWAP();
      /* We return with interrupts disabled. */
      BKL_LOCK();
      return 1; /* Try to get the mutex. */
    }

  /* Check that the process is still present. */
  if(instance != pcb_to_instance(pcb))
    {
      if((mutex->owner & 0x7FFFFFFFUL) == (owner & 0x7FFFFFFFUL))
	{
	  /* The owner has died. */
	  goto dead_owner; 
	}
      /* Unlock the dispatcher and restart. */
      UNLOCK_SWAP();
      /* Disable interrupts. */
      BKL_LOCK();
      goto again;
    }

  /*
  ** Check that the owner process is a prioritized process with a
  ** lower priority than the caller's.
  */
  if(pcb->priority > 31 || pcb->priority < curr->priority)
    {
      /*
      ** No, we shall not raise the priority.
      ** Set a flagbit to indicate we will enter mutex waiting
      ** queue. This will force a call to mutex_dequeue() in
      ** ose_mutex_unlock().
      */
      if(atomic_cas(DEVOLATILE(atomic_t *, &mutex->owner), owner,
		    owner | 0x80000000)
	 != owner)
	{
	  /*
	  ** Update failed because the lock word was modified. Take
	  ** a new owner sample. Don't worry, no action needed.
	  */
	}

      /* Unlock the dispatcher and exit. */
      UNLOCK_SWAP();
      /* Disable interrupts. */
      BKL_LOCK();
      /*
      ** The owner may have become ready by an interrupt, and may have
      ** unlocked the mutex by now. 
      */
      /* We return with interrupts disabled. */
      if(mutex->owner)
	return 0; /* Enter the mutex queue. */
      else
	return 1; /* Try to get the mutex. */
    }

  /*
  ** It looks like it's ok to raise the owner's priority.
  */

  /* Disable interrupts. */
  LOCK();

#ifdef OSE_SMP
  /*
  ** Check if the owner is running on another cpu.
  ** The dispatcher is locked, so the mutex owner can't be migrated.
  */

  cpu = pcb->cpu_id;
  if(cpu != zzkrn_cpu_id())
    {
#if 0 /* KRN_EVT_MUTEX_RAISE: Not yet functional?? */
      OSADDRESS arg;
      /*
      ** Problem: If we send the event below, and the mutex has
      ** been released when the other cpu handles the event,
      ** that must be detected by the other cpu, without touching
      ** the user space mutex??
      */
      if(!(owner | 0x80000000))
	{
	  /*
	  ** Set a flagbit to indicate we will enter mutex waiting
	  ** queue. This will force a call to mutex_dequeue() in
	  ** ose_mutex_unlock().
	  */
	  if(atomic_cas(DEVOLATILE(atomic_t *, &mutex->owner), owner,
			owner | 0x80000000)
	     != owner)
	    {
	      /*
	      ** Update failed because the lock word was modified. Take
	      ** a new owner sample. Don't worry, no action needed.
	      */
	      goto mutex_already_modified;
	    }
	}
      arg = (zzcur_p->priority << 4) | ((mutcb->instance & 0x3F) << 10) |
	(mutcb->index <<16);
      send_krn_event(cpu, KRN_EVT_MUTEX_RAISE, pcb_to_pid(pcb),
		     (void *)arg);
    mutex_already_modified: ;
#endif
    }
  else
#endif	/* OSE_SMP */
    {
      /* Save the owner's old priority, if it was not saved before. */
      if(!pcb->org_prio)
	{
	  /* No atomic needed, org_prio is only set from the same cpu. */
	  pcb->org_prio = pcb->priority & 31;

	  /*
	  ** Set a flagbit to indicate we will enter mutex waiting
	  ** queue. This will force a call to mutex_dequeue() in
	  ** ose_mutex_unlock().
	  */
	  if(atomic_cas(DEVOLATILE(atomic_t *, &mutex->owner), owner,
			owner | 0x80000000)
	     != owner)
	    {
	      /*
	      ** Update failed because the lock word was modified. Take
	      ** a new owner sample. Don't worry, no action needed.
	      */
	      goto mutex_already_modified2;
	    }
	}

      /* Remove the process from the ready list if it is ready. */
      make_otherpri_not_ready(pcb);

      /* Set a new priority in the pcb. */
      pcb->priority = curr->priority;
      pcb->ready_entry.rlist =
	(struct ready_list_entry *)&zzww.ready_list[pcb->priority];

      /* Run the process at the new priority if it is ready. */
      if(pcb->status < ST_READY)
	make_otherpri_ready_noswap(pcb); /* noswap */

    mutex_already_modified2: ;
    }
  /*
  ** Unlock the dispatcher and check if a swap was requested.
  */
  SILENT_UNLOCK_SWAP();
  if(zzww.dp_req)
    zzquit_int(); /* Swap. */

  /* We return with interrupts disabled. */
  BKL_LOCK();
  if(mutex->owner)
    return 0; /* Enter the mutex queue. */
  else
    return 1; /* Try to get the mutex. */
}

static struct _pcb *
sem_queue_waiting(SEMAPHORE *usersem)
{
  register struct semcb *semcb = (struct semcb *) usersem->f;
  
  if (semcb == NULL)
    return NULL;
  
  return semcb->fq;
}

/* Get a mutcb pointer from a mutex index. */
static struct mutcb *
index_to_mutcb(U32 index)
{
  U8 L1_index = (U8)(index >> 8);
  U8 L2_index = (U8)index;

  return zzuu.mutex_index_list[L1_index][L2_index];
}

static void 
free_mutcb(struct mutcb *mutcb)
{
  BKL_LOCK();

  assert_mutlist(mutcb->segcb, __LINE__);

  mutcb->magic = MAGIC_MUTEX_DESTROYED;
  mutcb->instance++;
  mutcb->umutex = NULL;

  /* Remove mutex from segment list. */
  if(mutcb->next)
    mutcb->next->prev = mutcb->prev;
  
  if(mutcb->prev)
    mutcb->prev->next = mutcb->next;
  else
    mutcb->segcb->mutlist = mutcb->next;

  assert_mutlist(mutcb->segcb, __LINE__);

  /* Add mutcb to free list. */
  assert_mutex_freelist(__LINE__);

  mutcb->next = zzuu.mutex_free_list;
  zzuu.mutex_free_list = mutcb;

  assert_mutex_freelist(__LINE__);
  BKL_UNLOCK();
}

/* 
** Create a kernel space mutex object.
**
** The dispatcher is already locked by the caller.
** The caller has to decide what to do with the owner, fast_lock and count
** members in the user space umutex object.
*/
static struct mutcb * 
create_mutcb(struct _mutex *umutex,
	     OSERRCODE subcode,
	     OSADDRESS return_address,
	     OSBOOLEAN use_sys_seg)
{
  struct mutcb *mutcb;

  assert_mutex_freelist(__LINE__);
  mutcb = zzuu.mutex_free_list;
  if(mutcb)
    zzuu.mutex_free_list = mutcb->next;
  else
    {
      U8 L1_index, L2_index;

      if(unlikely(zzuu.next_free_mutex_index == 0xffff))
	{
	  UNLOCK_SWAP();
      	  os_error(1, OSE_ETOO_MANY_MUTEXES | subcode, (OSERRCODE) 0); /* Non fatal system error. */
	  os_error(2, OSE_ETOO_MANY_MUTEXES | subcode, (OSERRCODE) 0); /* Fatal. */
	  /*NOTREACHED*/
	  return NULL;
	}
      /*
      ** Mutcb index zero is unused. Therefore, we must allocate the very
      ** first L2-array in this special way.
      */
      if(zzuu.next_free_mutex_index == 1)
	{
      	  zzuu.mutex_index_list[0] =
	    pool_brk_lo(&system_pool, 256 * sizeof(struct mutcb *));
	
	  zzuu.mutex_index_list[0][0] = NULL;
	}

      mutcb = pool_brk_lo(&system_pool, sizeof(struct mutcb));
      mutcb->instance = 1;
      mutcb->index = zzuu.next_free_mutex_index++;
      L2_index = mutcb->index;
      L1_index = mutcb->index >> 8;
      if(0 == L2_index)
	{
	  /* Need a new L2-array */

	  zzuu.mutex_index_list[L1_index] =
	    pool_brk_lo(&system_pool, 256 * sizeof(struct mutcb *));
	}
      zzuu.mutex_index_list[L1_index][L2_index] = mutcb;
    }

  mutcb->sem = locked_create_sem(0, return_address, use_sys_seg);
  mutcb->umutex = umutex;
  mutcb->segcb = ((struct semcb *)mutcb->sem->f)->segcb; /* needed??*/
  mutcb->prev = NULL;
  mutcb->next = mutcb->segcb->mutlist;
  if(mutcb->next)
    mutcb->next->prev = mutcb;
  mutcb->segcb->mutlist = mutcb;
  umutex->index = mutcb->index;
  
  mutcb->magic = MAGIC_MUTEX;
  umutex->magic = MAGIC_MUTEX;

  assert_mutlist(mutcb->segcb, __LINE__);
  return mutcb;
}

/* mutex_enqueue(struct _mutex *, timeout, OSADDRESS *extra); */
static OSADDRESS
sysconf_59(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  struct _mutex *mutex = (struct _mutex *)p;
  struct mutcb *mutcb;
  OSADDRESS *extra_p = (OSADDRESS *)r;
  U32 attr = mutex->attr;
  PROCESS owner = zzcur_p->pid & ~0x80000000u;
  volatile PROCESS *lock;
  U16 instance = mutex->instance;

  /* Use fast_lock if no prio inherit. */
  lock = attr & OSE_MUTEX_PRIO_INHERIT
     ? &mutex->owner
     : &mutex->fast_lock;

  /* Interrupt process version */
  if(zzcur_p->priority > 32)
    {
      LOCKSTATE msr;

      /* Interrupt processes can not wait. */
      if(q != 0)
	{
	  *extra_p = 0;
	  return OSE_EILLEGAL_SYSTEMCALL;
	}

      BKL_LOCK_SAVE(msr);
      if(mutex->magic != MAGIC_MUTEX)
	{
	  if(mutex->magic == MAGIC_MUTEX_DESTROYED)
	    {
	      BKL_LOCK_RESTORE(msr);
	      *extra_p = (OSADDRESS) mutex;
	      return OSE_EMUTEX_DESTROYED;
	    }
	  BKL_LOCK_RESTORE(msr);
	  *extra_p = (OSADDRESS) mutex;
	  return OSE_EMUTEX_INVALID;
	}

      /* Try to take the mutex from an interrupt process. */
      if(*lock == 0)
        {
	  /* The lock was released now. */
	  if((attr & OSE_MUTEX_PRIO_INHERIT) == 0)
	    {
	      if(atomic_cas(DEVOLATILE(atomic_t *, lock), 0, 1) == 0)
		{
		  mutex->owner = owner;		  
		  BKL_LOCK_RESTORE(msr);
		  /* The lock was successfully acquired. */
		  return 1; 
		}
	    }
	  else
	    {
	      if(atomic_cas(DEVOLATILE(atomic_t *, lock), 0, owner) == 0)
		{
		  BKL_LOCK_RESTORE(msr);
		  /* The lock was successfully acquired. */
		  return 1; 
		}
	    }
	}
      BKL_LOCK_RESTORE(msr);
      return 2; /* Trylock failed */
    } /* Interrupt process */

  /* Prioritized and background processes */

  /* Create a mutcb to handle the wait queue for the mutex. */
  if(mutex->magic == MAGIC_MUTEX && mutex->index == 0)
    {
      volatile struct _mutex *vm = mutex;
      
      LOCK_SWAP();
      if(vm->magic == MAGIC_MUTEX && vm->index == 0)
	create_mutcb(mutex, OSE_MUTEX_LOCK, 0, 0);
      UNLOCK_SWAP();
    }
  mutcb = index_to_mutcb(mutex->index);

  BKL_LOCK();
  for(;;)
    {
      if(mutex->magic != MAGIC_MUTEX)
	{
	  if(mutex->magic == MAGIC_MUTEX_DESTROYED)
	    {
	      BKL_UNLOCK();
	      return OSE_EMUTEX_DESTROYED;
	    }
	  BKL_UNLOCK();
	  *extra_p = (OSADDRESS) mutex;
	  return OSE_EMUTEX_INVALID;
	}

      /*
      ** Is the mutex both destroyed and reinitiated, or is the mutcb index
      ** invalid?
      */
      if(mutex->instance != instance || !mutex->index ||
	 mutex->index >= zzuu.next_free_mutex_index)
	{
	  BKL_UNLOCK();
	  *extra_p = (OSADDRESS) mutex;
	  return OSE_EMUTEX_DESTROYED;
	}
      
      /* Try to take the mutex from an prioritized/bg process. */
      if((attr & OSE_MUTEX_PRIO_INHERIT) == 0)
	{
	  if(atomic_cas(DEVOLATILE(atomic_t *, lock), 0, 1) == 0)
	    {
	      mutex->owner = owner;		  
	      if(sem_queue_waiting(mutcb->sem))
		{
		  /*
		  ** Set a flagbit to indicate that there is still at least
		  ** one more process waiting to get the mutex. This will
		  ** force a call to mutex_dequeue() in ose_mutex_unlock().
		  ** A plain or will do, as no other cpu can make any
		  ** other update.
		  */
		  *lock |= 0x80000000UL;
		}
	      BKL_UNLOCK();
	      /* The lock was successfully acquired. */
	      return 1; 
	    }
	}
      else
	{
	  if(atomic_cas(DEVOLATILE(atomic_t *, lock), 0, owner) == 0)
	    {
	      if(sem_queue_waiting(mutcb->sem))
		{
		  /*
		  ** Set a flagbit to indicate that there is still at least
		  ** one more process waiting to get the mutex. This will
		  ** force a call to mutex_dequeue() in ose_mutex_unlock().
		  ** A plain or will do, as no other cpu can make any
		  ** other update.
		  */
		  *lock |= 0x80000000UL;
		}
	      BKL_UNLOCK();
	      /* The lock was successfully acquired. */
	      return 1; 
	    }
	}

      /* Failed to take the lock. Return failure status or enter wait queue. */
      if(q == 0)
        {
	  BKL_UNLOCK();
	  return 2; /* Trylock failed */
	}

      if(attr & OSE_MUTEX_PRIO_INHERIT)
	{
	  /* The attributes indicate that we want priority inheritance. */
	  int rc = mutex_raise_priority(mutcb);
	  if(rc == 1)
	    continue;
	  else if (rc != 0)
	    {
	      PCB_INDEX_T pindex = pid_to_pindex(mutex->owner);

	      /* FIXME! Can we safely use pindex in pix_list here? @frbr@ */
	      PCB_INSTANCE_T pcb_instance = pcb_to_instance(zzuu.pix_list[pindex]);
	      PCB_INSTANCE_T owner_instance = pid_to_instance(mutex->owner);
	      OSADDRESS extra;

	      BKL_UNLOCK();

	      /*
	      ** Reconstruct the dead pid for the error information.
	      ** The most significant bit in the instance number may
	      ** be wrong. 
	      */
	      if((pcb_instance & 0x7fff) > (owner_instance & 0x7fff))
		{
		  owner_instance = ((owner_instance & 0x7fff) |
				    (pcb_instance & 0x8000));
		}
	      else
		{
		  owner_instance = ((owner_instance & 0x7fff) |
				    (~pcb_instance & 0x8000));
		}
	      extra = owner_instance;
	      extra = (extra << 16) | (mutex->owner & 0xffff);
	      *extra_p = extra;
	      return OSE_EMUTEX_OWNER_DEAD;
	    }
	  /* rc == 0 => go on and enter the mutex waiting queue. */
	}

      {
	PROCESS old_lock;

	/*
	** If the lock is occupied, then wait for it.
	** it.
	*/
	old_lock = *lock;
	while(old_lock)
	  {
	    /*
	    ** Set a flagbit to indicate we will enter mutex waiting
	    ** queue. This will force a call to mutex_dequeue() in
	    ** ose_mutex_unlock().
	    */
	    if(old_lock == atomic_cas(DEVOLATILE(atomic_t *, lock),
				      old_lock,
				      old_lock | 0x80000000))
	      {
		/* Enter the waiting queue. */
		mutex_wait_sem(mutcb->sem);
	      }
	    old_lock = *lock;
	  }
      }
      /* Retry to take the mutex. BKL_LOCK still taken here. */
    } /* for(;;) */
  /*NOTREACHED*/
}

/* mutex_dequeue(struct _mutex *, OSADDRESS *extra); */
static OSADDRESS
sysconf_60(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  struct _mutex *mutex = (struct _mutex *)p;
  OSADDRESS *extra_p;
  OSPRIORITY org_prio;
  U32 attr;

  (void)r;

  if(zzcur_p->priority > 32)
    {
#ifdef OSE_SMP
      /*
      ** SMP: Prioritized/background processes on other cpus have queued
      ** while we locked the mutex. If the mutex is still present now,
      ** we signal the semaphore.
      */
      if(mutex->magic == MAGIC_MUTEX && mutex->index &&
	 mutex->index < zzuu.next_free_mutex_index)
	{
	  struct mutcb *mutcb = index_to_mutcb(mutex->index);
	  if(mutcb->umutex == mutex)
	    {
	      signal_sem(mutcb->sem);
	      return 1;			/* Success */
	    }
	}
#endif
      /*
      ** Interrupt processes not allowed to use this call. 
      ** Either an internal error in the mutex implementation or mutex
      ** data invalid.
      **/
      extra_p = (OSADDRESS *)q;
      *extra_p = (OSADDRESS) mutex;
      return OSE_EMUTEX_INVALID;
    }

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /*  If the mutex is still present now, we signal the semaphore. */
  if(mutex->magic == MAGIC_MUTEX && mutex->index &&
     mutex->index < zzuu.next_free_mutex_index)
    {
      struct mutcb *mutcb = index_to_mutcb(mutex->index);
      if(mutcb->umutex == mutex)
	{
	  signal_sem(mutcb->sem);
	}
    }

  /*
  ** Save mutex attributes in case the mutex will be destroyed.
  ** Do we know that it is still present here?? Don't we need to enter
  ** here while the mutex is still taken? New crt trap function?
  */
  attr = mutex->attr;
  org_prio = zzcur_p->org_prio;

  /* Unlock dispatcher. */
  UNLOCK_SWAP();

  /*
  ** Does this mutex use priority inheritance and has anyone raised my
  ** priority?
  */
  if((attr & OSE_MUTEX_PRIO_INHERIT) && org_prio)
    {
      /* If so, lower it. */
      zzcur_p->org_prio = 0;
      set_pri(org_prio);
    }
  return 1;			/* Success */
}

/*
** This is the function used by zzkrn_mutex_init and the error
** returning version of mutex_init. It takes an extra parameter where
** the additional error information is returned. See zzkrn_mutex_init
** for further parameter descriptions.
**
** It returns 1 on success, otherwise an error code suitable for use
** with the os_error system call.
*/
static OSERRCODE
zzkrn_mutex_init_unsafe(struct _mutex *umutex,
                        OSADDRESS attrib,
                        OSADDRESS return_address,
                        OSBOOLEAN use_sys_seg,
                        OSADDRESS *extra)
{
  struct mutcb *mutcb;

#if ERROR_CHECKS > 0
  /* Check that caller is a non-interrupt process. */
  if(unlikely(zzcur_p->priority > 32))
    {
      return OSE_EILLEGAL_SYSTEMCALL;
    }
#endif

  /* Return an error on unknown mutex attributes. */
  if (unlikely(attrib & ~(OSE_MUTEX_PRIO_INHERIT | OSE_MUTEX_RECURSIVE)))
    {
      *extra = attrib;
      return OSE_EBAD_PARAMETER;
    }

  LOCK_SWAP();

  /*
  ** Create the internal mutex control block if not already done.
  ** This is needed if:
  ** 1. The user mutex is uninitiated or already destroyed.
  ** 2. The user mutex is only initiated by a static initializer.
  ** 3. The mutex is fully initialized and used in an earlier lifetime,
  **    either before latest reset, or by a killed program (segment).
  **    The reset scenario further requires that the mutex is stored
  **    in some kind of uninitated ram, for example the heap.
  */
  if(umutex->magic != MAGIC_MUTEX ||			/* 1 */
     umutex->index == 0 ||				/* 2 */
     umutex->index >= zzuu.next_free_mutex_index ||	/* 3 */
     umutex != index_to_mutcb(umutex->index)->umutex)	/* 3 */
    {
      umutex->attr = attrib;
      mutcb = create_mutcb(umutex, OSE_MUTEX_INIT,
			   return_address, use_sys_seg);
    }
  else
    {
      /* The mutex was already initiated, and the index is valid. */
      mutcb = index_to_mutcb(umutex->index);

      /* Check if a non sys_mutex is referenced over a segment boundary. */
      if(mutcb->segcb->segid && (zzcur_p->segid != mutcb->segcb->segid))
	{
	  UNLOCK_SWAP();
          *extra = (OSADDRESS) umutex;
          return OSE_EBAD_PARAMETER;
	}

      /*
      ** If attributes are identical, return silently.
      ** Otherwise report error, we can't change behavior while the mutex
      ** is in use.
      */
      if (umutex->attr != attrib)
	{
	  UNLOCK_SWAP();
          *extra = (OSADDRESS) umutex;
          return OSE_EMUTEX_ALREADY_INITIALIZED;
	}
    }

  /*
  ** Initiate the lock state members in the user space mutex block.
  ** As the umutex may be in uninitiated RAM, we must make it unlocked here.
  ** (A locked state can have been set in an earlier life, or by random data.)
  ** If it really was a valid lock, an error will be reported when the locker
  ** tries to unlock it. Thus, it is illegal to re-initiate a locked mutex.
  */
  BKL_LOCK();
  umutex->owner = 0;
  umutex->fast_lock = 0;
  umutex->count = 0;

  BKL_UNLOCK_KEEP_INT_MASK();
  /* UNLOCK() implicitely done by LOCKED_UNLOCK_SWAP() */
  LOCKED_UNLOCK_SWAP();
  return 1;			/* Success */
}

/*
** This is the kernel function for both ose_mutex_init() and
** ose_sys_mutex_init().  It takes two more parameters than the normal
** API/SPI functions.
**
** Return address is used for tracing purposes and should be either
** zero or the (often compiler supplied) return address from the
** call of the API/SPI function.
**
** The use_sys_seg parameter should be zero when called from the API
** function ose_mutex_init() and non-zero when called from the SPI
** function ose_sys_mutex_init(). The sys mutex is always bound to the
** system segment (regardless of what segment the caller of the init
** function belongs to), thus it must be explicitly destroyed.
*/
OSERRCODE
zzkrn_mutex_init(struct _mutex *umutex,
                 OSADDRESS attrib,
                 OSADDRESS return_address,
                 OSBOOLEAN use_sys_seg)
{
  OSADDRESS extra = 0;
  OSERRCODE rc;

  /* Report a warning on unknown mutex attributes. */
  if (attrib & ~(OSE_MUTEX_PRIO_INHERIT | OSE_MUTEX_RECURSIVE))
    {
      os_error(0, OSE_MUTEX_INIT | OSE_EBAD_PARAMETER, attrib);	/* Non fatal. */
      attrib &= (OSE_MUTEX_PRIO_INHERIT | OSE_MUTEX_RECURSIVE);
    }
  
  rc = zzkrn_mutex_init_unsafe(umutex, attrib, return_address,
                               use_sys_seg, &extra);
  if (rc > 1)
    {
      os_error(0, OSE_MUTEX_INIT | rc, (OSERRCODE) extra);	/* Non fatal. */
      return 0;
    }

  return 1;			/* Success */
}

/*
** OSERRCODE mutex_init(struct _mutex *umutex, OSADDRESS attrib,
**                      OSADDRESS return_address);
*/
static OSADDRESS
sysconf_70(OSADDRESS umutex, OSADDRESS attrib, OSADDRESS return_address)
{
  return (OSADDRESS)zzkrn_mutex_init((struct _mutex *)umutex,
				     attrib,
				     return_address,
				     0 /* OSBOOLEAN use_sys_seg */ );
}

/*
** OSERRCODE mutex_init_unsafe(struct _mutex *umutex, OSADDRESS attrib,
**                             OSADDRESS return_address);
*/
static OSADDRESS
sysconf_100(OSADDRESS umutex, OSADDRESS attrib, OSADDRESS return_address)
{
  OSADDRESS extra = 0;
  return (OSADDRESS)zzkrn_mutex_init_unsafe((struct _mutex *)umutex,
                                            attrib,
                                            return_address,
                                            0, /* OSBOOLEAN use_sys_seg */
                                            &extra);
}

/*
** This is the kernel function for both ose_mutex_destroy() and
** ose_sys_mutex_destroy(). It takes two more parameters than the
** normal API/SPI functions.
**
** The extra_p parameter should be a pointer to a variable of type
** OSADDRESS in which the functions writes extra info in case of error.
**
** The sys_mutex parameter should be zero when called from the API
** function ose_mutex_destroy() and non-zero when called from the SPI
** function ose_sys_mutex_destroy(). The sys mutex is always bound to
** the system segment (regardless of what segment the caller of the
** init function belongs to), thus it must be explicitly destroyed.
**
** Returns the value 1 on success, and a value >1 on error, this value
** should be used as the error parameter when calling error...
*/

OSERRCODE
zzkrn_mutex_destroy(struct _mutex *mutex,
		    OSADDRESS     *extra_p,
		    OSBOOLEAN      sys_mutex)
{
  struct mutcb *mutcb;
  SEMAPHORE *sem;

  /* Check that caller is a non-interrupt process. */
  if(zzcur_p->priority > 32)
    return OSE_EILLEGAL_SYSTEMCALL;

  /* Disable interrupts. */
  BKL_LOCK();
  
  if(mutex->magic != MAGIC_MUTEX)
    {
      if (mutex->magic == MAGIC_MUTEX_DESTROYED)
	{
	  /* Enable interrupts. */
	  BKL_UNLOCK();
	  /* The mutex is already destroyed. */
	  *extra_p = (OSADDRESS) mutex;
	  return OSE_EMUTEX_DESTROYED;
	}
      else
	{
	  /* Enable interrupts. */
	  BKL_UNLOCK();
	  /*
	  ** Can't destroy since the mutex pointer does not point to a
	  ** valid mutex.
	  */
	  *extra_p = (OSADDRESS) mutex;
	  return OSE_EMUTEX_INVALID;
	}
    }
  
  if(mutex->owner | mutex->fast_lock)
    {
      PCB_INDEX_T pindex = pid_to_pindex(mutex->owner);

      /* Get the pid of the owner */
      /* FIXME! Should we check the pindex here?? @frbr@ */
      *extra_p = (OSADDRESS) zzuu.pix_list[pindex]->live_pid;

      /* Enable interrupts. */
      BKL_UNLOCK();
      /* Can't destroy since mutex is currently locked. */
      return OSE_EMUTEX_LOCKED;
    }

  /* If there is a mutcb allocated, check the semaphore queue. */
  mutcb = NULL;
  sem = NULL;
  if(mutex->index)
    {
      struct _pcb *qpcb;

      /* Validate the index. */
      if(mutex->index >= zzuu.next_free_mutex_index ||
	 (mutcb = index_to_mutcb(mutex->index))->umutex != mutex)
	{
	  BKL_UNLOCK();
	  *extra_p = (OSADDRESS) mutex;
	  return OSE_EMUTEX_INVALID;
	}

      sem = mutcb->sem;
      qpcb = sem_queue_waiting(sem);

      if(qpcb)
	{
	  *extra_p = qpcb->live_pid;
	  
	  /* Enable interrupts. */
	  BKL_UNLOCK();
	  /*
	  ** Can't destroy since there are processes waiting in the mutex
	  ** queue.
	  */
	  return OSE_EMUTEX_IN_USE;
	}
    }

  /* Ok to destroy mutex. */
  mutex->magic = MAGIC_MUTEX_DESTROYED;
  mutex->instance++;
  mutex->index = 0;
  BKL_UNLOCK();

  /* Free the mutex waiting queue (if we have one). */
  if(mutcb)
    {
      /* semleak?? Freed with the segment. */
      sys_kill_sem(sem, sys_mutex);
      free_mutcb(mutcb);
    }
  /* The mutex was successfully destroyed. */
  return 1;
}

/* OSERRCODE mutex_destroy(struct _mutex *p, OSADDRESS *extra); */
static OSADDRESS
sysconf_61(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  (void)r;

  return (OSADDRESS)zzkrn_mutex_destroy((struct _mutex *)p,
					(OSADDRESS *)q,
					(OSBOOLEAN) 0 /* Not sys_mutex */);
}

/* mutex_error(fatal, OSERRCODE, struct _mutex *); */
static OSADDRESS
sysconf_62(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  return handle_error((int) p,        /* fatal. */
		      (OSERRCODE) q,  /* ecode */
		      (OSERRCODE) r,  /* extra */
		      (OSBOOLEAN) 0); /* Not user_called */
}

/*
*****************************************************************************
** 21 ENVIRONMENT VARIABLES AND PER PROCESS DATA.
*****************************************************************************
*/

/*
** The list of environment variables in a block descriptor or a
** process control block consists of a linked list of environment
** control structures. These are allocated on demand from
** the system pool and the system is the owner of these buffers.
** New entries are always added at the tail of the list.
**
** No entries are removed from the list except that the entire list
** is deallocated when the block/process is killed.
**
** Note that the OSE specification requires that the order of entries
** in a list is not changed. This is a requirement for the get_env_list()
** system call.
** It would be possible to keep name buffers in a parallel tree for quick
** search in the set_env() and get_env() system calls. We don't do this
** right now. Instead we compress names into a hash value to improve
** search speed.
**
** See also: struct _pcb, struct _block, set_env(), get_env(), get_env_list().
*/

/*
**===========================================================================
** 21.1  Initialization.
**===========================================================================
*/

/*
** Initialization of per process data stuctures.
*/

static void
init_ppd(void)
{
  OSADDRESS keys = zzuu.max_ppd_keys;

  memset(zzuu.ppd_key_buf, 0, keys * zzuu.allocated_ppd_key_size);
  /* Set hash keys to a value that isn't the hash of the empty string "" */
  memset(zzuu.ppd_hash_buf, 255, keys);
}

/*
**===========================================================================
** 21.2  Local functions.
**===========================================================================
*/

/*
** Deallocate an environment list for a block or a process.
** Subfunction to the system daemon sysd.
*/

static void
kill_proc_cancel_env(struct _pcb *pcb)
{
  struct env_buf *envbuf;
  register struct env_buf *next_envbuf;

  /* For all environment variables. */
  for(envbuf = pcb->env_list; envbuf; envbuf = next_envbuf)
    {
      /* Pick up next pointer and save it. */
      next_envbuf = envbuf->next;

      /* Free the name buffer. */
      if(envbuf->value)
	sys_free_buf((union SIGNAL **) &(envbuf->value));

      /* free the environment control structure. */
      sys_free_buf_type(struct env_buf *, &envbuf);
    }
}

/*
**===========================================================================
** 21.3  System calls.
**===========================================================================
*/

/*
** This is the set_env system call.
**
** The value parameter is allowed to be set to NULL by the caller.
** This means that the variable is removed, or really hidden.
** It is still present in the lists, but it is no longer displayed or
** inherited to child blocks.
*/

OSBOOLEAN
set_env(PROCESS pid, const char *name, const char *value)
{
  register struct _pcb *pcb;
  register char hash;
  register const char *p;
  register struct env_buf *envbuf;
  register struct env_buf *prev_envbuf;
  register U32 cur_value = 0;
  register U32 namelen;
  union SIGNAL *tmpbuf = NULL;

  /*
  ** Pick up the string length of the value.
  */
  if(value)
    cur_value = strlen(value);

#if ERROR_CHECKS > 0
  ON_ERRLEVEL_2()
    {
      /* Check that the caller is not an interrupt or timer process. */
      if(check_nonint_caller(OSE_SET_ENV))
	return (OSBOOLEAN) 0;

      /* Check that the name is not the NULL pointer. */
      if(unlikely(!name))
	{
	  os_error(0, OSE_EILLEGAL_ENVNAME | OSE_SET_ENV, (OSERRCODE) 0);	/* Non fatal. */
	  return (OSBOOLEAN) 1; /* Return with the error flag set. */
	}
    }
#endif

  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid_bid(pid, OSE_SET_ENV);
  if(pcb == NULL)
    {
      return (OSBOOLEAN) 0;
    }

  /* Locate the block or process descriptor. */
  pcb = pid_to_pcb(pid);

  /* Calculate hash value and string length from name. */
  for(hash = (char) 0, p = name; *p; p++)
    {
      hash ^= *p;
    }
  namelen = (U32) (p - name);

  /* Use the kernel pool from now on. */
  set_suid((OSBOOLEAN) 1);

  /*
  ** If paging is enabled, save name and value in a temporary buffer in the
  ** system pool to avoid page faults with a locked dispatcher.
  */
  if(zzuu.pagesize != 0)
    {
      tmpbuf = alloc(namelen + 1u + cur_value + 1u + sizeof(SIGSELECT), 0);
      memcpy(tmpbuf, name, namelen + 1u);
      name = (const char *) tmpbuf;
      if(cur_value != 0)
	{
	  char *tmp_value = namelen + 1u + (char *)tmpbuf;
	  
	  memcpy(tmp_value, value, cur_value + 1u);
	  value = (const char *) tmp_value;
	}
    }

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the block/process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      UNLOCK_SWAP();

      /* Free the temporary buffer used in paging systems. */
      if(tmpbuf != NULL)
	{
	  free_buf(&tmpbuf);
	}
      return (OSBOOLEAN) 0;
    }

#if USE_REMOTE_CALLS
  /* Issue a remote system call if required. */
  if(local_sysctlblk.enable_remcalls)
    {
      union SIGNAL *sig;
      register OSBOOLEAN use_remcalls;
      register PROCESS handler;
      if(pcb->magic == MAGIC_PCB)
	{
	  use_remcalls = pcb->block->use_remcalls;
	  handler = pcb->block->remcall_server;
	}
      else
	{
	  use_remcalls = ((struct _block *)pcb)->use_remcalls;
	  handler = ((struct _block *)pcb)->remcall_server;
	}
      if(use_remcalls)
	{
	  UNLOCK_SWAP();
	  sig = alloc(sizeof(struct remote_call) + namelen + cur_value + 1, REMOTE_CALL);
	  sig->remote_call.call_type = REM_SET_ENV;
	  sig->remote_call.params.rem_set_env.pid = pid;
	  sig->remote_call.params.rem_set_env.name = 0;
	  memcpy(&sig->remote_call.params.rem_set_env.strings[0], name, (size_t)namelen + 1);
	  /* Set the value field to point at the value string if we have one. */
	  if(value)
	    {
	      sig->remote_call.params.rem_set_env.value = namelen + 1;
	      memcpy(&sig->remote_call.params.rem_set_env.strings[namelen + 1], value, (size_t)cur_value + 1);
	    }
	  /* Else set the value field in the signal to zero to indicate that we should remove the variable. */
	  else
	    {
	      sig->remote_call.params.rem_set_env.value = 
		sig->remote_call.params.rem_set_env.name;
	    }

	  sig = make_remote_call(sig, (OSCALLTYPE) REMR_SET_ENV, handler);
	  set_suid((OSBOOLEAN) 0);

	  /* Free the temporary buffer used in paging systems. */
	  if(tmpbuf != NULL)
	    {
	      free_buf(&tmpbuf);
	    }
	  if(sig)
	    {
	      OSBOOLEAN result = sig->remote_response.results.remr_set_env.failed;
	      free_buf(&sig);
	      return result;
	    }
	  return (OSBOOLEAN) 0;
	}
    }
#endif

  /* Search for the variable. */
  prev_envbuf = NULL;
  for(envbuf = pcb->env_list; envbuf; envbuf = envbuf->next)
    {
      if(envbuf->name_hash == hash)
	{
	  if(!strcmp(envbuf->name, name))
	    {
	      break;
	    }
	}
      prev_envbuf = envbuf;
    } /* for */

  /* Handle delete request */
  /* NOTE that interrupt processes may use get_envp while doing this. */
  if(!value)
    {
      /* 
       * Delete the value, but only if it did exist. We can not remove the
       * envbuf, because that could cause trouble in a segmented get_env_list.
       */
      if(envbuf && envbuf->value)
	{
	  
	  /* 
	   * First, change cur_value to inhibit get_envp from interrupt
	   * processes to find an already freed value buffer.
	   */
	  envbuf->cur_value = 0;
	  sys_free_buf((union SIGNAL **) &(envbuf->value));
	  envbuf->value = NULL;
	  envbuf->max_value = 0;
	}
    }

  /* Handle set request. */
  /* NOTE that interrupt processes may use get_envp while doing this. */
  else
    {
      /* Create the variable if it was not found. */
      if(!envbuf)
	{
	  envbuf = (struct env_buf *)
	    locked_alloc(sizeof(struct env_buf) + namelen, 0);
	  change_sig_owner(sig_to_adm(envbuf), NULL);
	  envbuf->next = NULL;
	  envbuf->name_hash = hash;
	  envbuf->cur_value = cur_value;
	  envbuf->value = (char *) locked_alloc(cur_value + 1, 0);
	  change_sig_owner(sig_to_adm(envbuf->value), NULL);
	  envbuf->max_value = cur_value + 1;
	  memcpy(envbuf->name, name, (size_t)namelen + 1);
	  
	  /* Set the variable value. */
	  memcpy(envbuf->value, value, (size_t)cur_value + 1);

	  /* Make the variable visible for other processes. */
	  if(prev_envbuf)
	    {
	      prev_envbuf->next = envbuf;
	    }
	  else
	    {
	      pcb->env_list = envbuf;
	    }
	}

      /* Update an existing variable. */
      else
	{
	  /* Check size of data buffer. Reallocate if too small. */
	  if(!envbuf->value || envbuf->max_value <= cur_value)
	    {
	      char *new_value = (char *) locked_alloc(cur_value + 1, 0);
	      char *old_value = envbuf->value;

	      memcpy(new_value, value, (size_t)cur_value + 1);
	      change_sig_owner(sig_to_adm(new_value), NULL);
	      BKL_LOCK();
	      envbuf->value = new_value;
	      envbuf->cur_value = cur_value;
	      envbuf->max_value = cur_value + 1;
	      BKL_UNLOCK();
	      if(old_value)
		sys_free_buf_type(char *, &(old_value));
	    }

	  /* The old value buffer will do. Fill in the value. */
	  else
	    {
	      /* Remember that 8 byte values can be envp variables.
	       * Such values must be updated atomically.
	       */
	      if(cur_value == 8)
		{
		  BKL_LOCK();
		  envbuf->cur_value = cur_value;
		  memcpy(envbuf->value, value, (size_t)cur_value + 1);
		  BKL_UNLOCK();
		}

	      /* Not an envp variable. Atomic not needed */
	      else
		{
		  envbuf->cur_value = cur_value;
		  memcpy(envbuf->value, value, (size_t)cur_value + 1);
		}
	    }
	}
    }

  /* Move back to the user pool, unlock the dispatcher and exit. */
  set_suid((OSBOOLEAN) 0);
  UNLOCK_SWAP();

  /* Free the temporary buffer used in paging systems. */
  if(tmpbuf != NULL)
    {
      free_buf(&tmpbuf);
    }
  return (OSBOOLEAN) 0;
}

/*
** This is the get_env system call.
*/

char *
get_env(PROCESS pid, const char *name)
{
  register struct _pcb *pcb;
  register struct env_buf *envbuf;
  register char hash;
  register const char *p;
  register char *r;
  OSBUFSIZE bufsize;
  size_t namelen;
  union SIGNAL *tmpbuf = NULL;

#if ERROR_CHECKS > 0
  ON_ERRLEVEL_2()
    {
      /* Check that the caller is not an interrupt or timer process. */
      if(check_nonint_caller(OSE_GET_ENV))
	return NULL;

      /* Check that the name is not the NULL pointer. */
      if(unlikely(!name))
	{
	  os_error(0, OSE_EILLEGAL_ENVNAME | OSE_GET_ENV, (OSERRCODE) 0);	/* Non fatal. */
	  return NULL;	/* The variable was not found. */
	}
    }
#endif

  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid_bid(pid, OSE_GET_ENV);
  if(pcb == NULL)
    {
      return NULL;
    }

  /* Locate the block or process descriptor. */
  pcb = pid_to_pcb(pid);

  /* Calculate hash value from name. */
  for(hash = (char) 0, p = name; *p; p++)
    hash ^= *p;
  namelen = p - name;

  /*
  ** If paging is enabled, save the name in a temporary buffer in the
  ** system pool to avoid page faults with a locked dispatcher.
  */

  if(zzuu.pagesize != 0)
    {
#if MEMORY_MODEL > 1
      /* Use the kernel pool for next alloc. */
      tmpbuf = sys_alloc(namelen + 1 + sizeof(SIGSELECT), 0);
#else
      tmpbuf = alloc(namelen + 1 + sizeof(SIGSELECT), 0);
#endif
      memcpy(tmpbuf, name, namelen + 1);
      name = (const char *) tmpbuf;
    }

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the block/process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      UNLOCK_SWAP();
      if(tmpbuf != NULL)
	{
	  free_buf(&tmpbuf);
	}
      return NULL;
    }

#if USE_REMOTE_CALLS
  /* Issue a remote system call if required. */
  if(local_sysctlblk.enable_remcalls)
    {
      union SIGNAL *sig;
      register OSBOOLEAN use_remcalls;
      register PROCESS handler;
      if(pcb->magic == MAGIC_PCB)
	{
	  use_remcalls = pcb->block->use_remcalls;
	  handler = pcb->block->remcall_server;
	}
      else
	{
	  use_remcalls = ((struct _block *)pcb)->use_remcalls;
	  handler = ((struct _block *)pcb)->remcall_server;
	}
      if(use_remcalls)
	{
	  UNLOCK_SWAP();
	  set_suid((OSBOOLEAN) 1);
	  sig = alloc(sizeof(struct remote_call) + namelen, REMOTE_CALL);
	  sig->remote_call.call_type = REM_GET_ENV;
	  sig->remote_call.params.rem_get_env.pid = pid;
	  memcpy(sig->remote_call.params.rem_get_env.name, name, namelen + 1);
	  if(tmpbuf != NULL)
	    {
	      free_buf(&tmpbuf);
	    }
	  sig = make_remote_call(sig, (OSCALLTYPE) REMR_GET_ENV, handler);
	  set_suid((OSBOOLEAN) 0);
	  if(sig)
	    {
	      char *dst;

	      dst = (char *)
		alloc((OSBUFSIZE)(sizeof(SIGSELECT) +
				  strlen(sig->remote_response.results.remr_get_env.value)),
		      (SIGSELECT) 0);
	      strcpy(dst, (const char *) sig->remote_response.results.remr_get_env.value);
	      free_buf(&sig);
	      return (char *) dst;
	    }
	  return NULL;
	}
    }
#endif /* USE_REMOTE_CALLS */

  /* Search for the indicated variable. */
  envbuf = pcb->env_list;
  if(!envbuf)
    {
      /* Not found if env list in pcb is empty. */
      UNLOCK_SWAP();
      if(tmpbuf != NULL)
	{
	  free_buf(&tmpbuf);
	}
      return NULL;
    }

  for(;;)
    {
      if(envbuf->name_hash == hash)
	{
	  if(!strcmp(envbuf->name, name))
	    {
	      break;
	    }
	}
      envbuf = envbuf->next;
      if(!envbuf)
	{
	  /* Not found. Tell the caller so. */
	  UNLOCK_SWAP();
	  if(tmpbuf != NULL)
	    {
	      free_buf(&tmpbuf);
	    }
	  return NULL;
	}
    }

  /* We don't need the name buffer any more. */
  if(tmpbuf != NULL)
    {
      free_buf(&tmpbuf);
    }

  /*
  ** Done if the variable was previously deleted.
  ** This is indicated by the value field being set to NULL.
  */
  if(!envbuf->value)
    {
      /* Not found. Tell the caller so. */
      UNLOCK_SWAP();
      return NULL;
    }

  /* Calculate size of variable value. */
  bufsize = envbuf->cur_value + sizeof(SIGSELECT);

  /* Use the system pool. */
  set_suid(1);

  /* Allocate a buffer large enough for the list. */
  r = (char *) locked_alloc(bufsize, 0);

  /* Use the application pool again. */
  set_suid(0);

  /* Copy found string into data buffer. */
  strcpy(r, envbuf->value);

  /* Unlock the dispatcher and return the value. */
  UNLOCK_SWAP();
  return (char *) sig_dup((union SIGNAL *)r);
  
}

/*
** This is the get_env_list system call.
*/

char *
get_env_list(PROCESS pid, const char *first_name)
{
  register struct _pcb *pcb;
  register struct env_buf *envbuf;
  register char hash;
  register const char *p;
  register char *q, *r;
  OSBUFSIZE bufsize;
  U32 no_of_env;
  size_t first_namelen = 0;
  union SIGNAL *tmpbuf = NULL;


#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_GET_ENV_LIST))
    return (char *) alloc((OSBUFSIZE) sizeof(SIGSELECT), (SIGSELECT) 0);
#endif

  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid_bid(pid, OSE_GET_ENV_LIST);
  if(pcb == NULL)
    {
      return (char *) alloc((OSBUFSIZE) sizeof(SIGSELECT), (SIGSELECT) 0);
    }

  /* Locate the block or process descriptor. */
  pcb = pid_to_pcb(pid);

  /* Calculate hash value from name. */
  if(first_name)
    {
      for(hash = (char) 0, p = first_name; *p; p++)
	hash ^= *p;
      first_namelen = p - first_name;
    }
  else
    {
      /* Unnecessary assign, but it will remove a compiler warning. */
      hash = (char)0;
    }

  /*
  ** If paging is enabled, save the first_name in a temporary buffer in the
  ** system pool to avoid page faults with a locked dispatcher.
  */

  if(zzuu.pagesize != 0 && first_namelen != 0)
    {
#if MEMORY_MODEL > 1
      /* Use the kernel pool for next alloc. */
      tmpbuf = sys_alloc(first_namelen + 1 + sizeof(SIGSELECT), 0);
#else
      tmpbuf = alloc(first_namelen + 1 + sizeof(SIGSELECT), 0);
#endif
      memcpy(tmpbuf, first_name, first_namelen + 1);
      first_name = (const char *) tmpbuf;
    }

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the block/process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      UNLOCK_SWAP();
      if(tmpbuf != NULL)
	{
	  free_buf(&tmpbuf);
	}
      return (char *) alloc((OSBUFSIZE) sizeof(SIGSELECT), (SIGSELECT) 0);
    }

#if USE_REMOTE_CALLS
  /* Issue a remote system call if required. */
  if(local_sysctlblk.enable_remcalls)
    {
      union SIGNAL *sig;
      register OSBOOLEAN use_remcalls;
      register PROCESS handler;

      if(pcb->magic == MAGIC_PCB)
	{
	  use_remcalls = pcb->block->use_remcalls;
	  handler = pcb->block->remcall_server;
	}
      else
	{
	  use_remcalls = ((struct _block *)pcb)->use_remcalls;
	  handler = ((struct _block *)pcb)->remcall_server;
	}
      if(use_remcalls)
	{
	  UNLOCK_SWAP();
	  set_suid((OSBOOLEAN) 1);
	  sig = alloc(sizeof(struct remote_call) + first_namelen, REMOTE_CALL);
	  sig->remote_call.call_type = REM_GET_ENV_LIST;
	  sig->remote_call.params.rem_get_env_list.pid = pid;
	  if(first_namelen)
	    {
	      memcpy(sig->remote_call.params.rem_get_env_list.first_name,
		     first_name, first_namelen + 1);
	    }
	  else
	    sig->remote_call.params.rem_get_env_list.first_name[0] = 0;

	  if(tmpbuf != NULL)
	    {
	      free_buf(&tmpbuf);
	    }
	  sig = make_remote_call(sig, (OSCALLTYPE) REMR_GET_ENV_LIST, handler);
	  set_suid((OSBOOLEAN) 0);
	  if(sig)
	    {
	      char *dst;

	      dst = (char *)
		alloc((OSBUFSIZE)(sizeof(SIGSELECT) +
				  strlen(sig->remote_response.results.remr_get_env_list.name)),
		      (SIGSELECT) 0);
	      strcpy(dst, (const char *) sig->remote_response.results.remr_get_env_list.name);
	      free_buf(&sig);
	      return (char *) dst;
	    }
	  return (char *) alloc((OSBUFSIZE) sizeof(SIGSELECT), (SIGSELECT) 0);
	}
    }
#endif

  /* Search for the variable after the specified one. */
  if(!pcb->env_list)
    {
      /* Not found if env list in pcb is empty. */
      UNLOCK_SWAP();
      if(tmpbuf != NULL)
	{
	  free_buf(&tmpbuf);
	}
      return (char *) alloc((OSBUFSIZE) sizeof(SIGSELECT), (SIGSELECT) 0);
    }

  envbuf = pcb->env_list;
  /* Search only if a first_name was given. */
  if(first_name)
    {
      for(;;)
	{
	  if(envbuf->name_hash == hash)
	    {
	      if(!strcmp(envbuf->name, first_name))
		{
		  envbuf = envbuf->next;
		  if(!envbuf)
		    {
		      /* Not found if last variable was in first_name. */
		      UNLOCK_SWAP();
		      if(tmpbuf != NULL)
			{
			  free_buf(&tmpbuf);
			}
		      return (char *) alloc((OSBUFSIZE) sizeof(SIGSELECT), (SIGSELECT) 0);
		    }
		  break;
		}
	    }
	  if(!envbuf->next)
	    {
	      /* Not found. Tell caller so. */
	      UNLOCK_SWAP();
	      if(tmpbuf != NULL)
		{
		  free_buf(&tmpbuf);
		}
	      return (char *) alloc((OSBUFSIZE) sizeof(SIGSELECT), (SIGSELECT) 0);
	    }
	  envbuf = envbuf->next;
	}
    }

  /* first_name no longer needed. */
  if(tmpbuf != NULL)
    {
      free_buf(&tmpbuf);
    }

  /*
  ** Accumulate size of names in rest of list. 
  ** Result in bufsize and no_of_env.
  */
  {
    register struct env_buf *e;
    OSBUFSIZE bz;

  restart_scan_after_skipping_a_too_long_name:

    no_of_env = 0;
    bufsize = bz = sizeof(SIGSELECT);
    for(e = envbuf; e ; e = e->next)
      {
	if(e->value)
	  {
	    bz += strlen(e->name) + 1;
	    if(bz > MAX_ENV_LIST)
	      break;
	    bufsize = bz;
	    /* Count names with a value buffer only. */
	    no_of_env++;
	  }
      }

    /*
    ** If the first variable with a value string has a too long name to be
    ** reported, then we skip this variable and restart the scan on the next,
    ** if there is one.
    */
    if(bz > MAX_ENV_LIST && no_of_env == 0 && e->next)
      {
	envbuf = e->next;
	goto restart_scan_after_skipping_a_too_long_name;
      }
  }
  

  /* Use the system pool. */
  set_suid(1);

  /* Allocate a buffer large enough for the list. */
  q = (char *) locked_alloc(bufsize, 0);
  p = q;

  /* Use the application pool again. */
  set_suid(0);

  /* Concatenate names in rest of list into data buffer. */
  for(; no_of_env; envbuf = envbuf->next)
    {
      if(envbuf->value)
	{
          r = envbuf->name;
          while(*r)
            {
              *q++ = *r++;
            }
          *q = *r;

          if(--no_of_env)
	    *q++ = (char) ' ';
	}
    }
  *q = (char) '\0';

  /* Unlock the dispatcher and return list. */
  UNLOCK_SWAP();
  return (char *) sig_dup(DECONST(union SIGNAL *, p));

}

/*
**===========================================================================
** 21.4  Undocumented system calls.
**===========================================================================
*/

/*
** This is the get_envp system call.
**
** This call reads a 32 bit pointer from a named environment variable.
** This call assumes that the value was set with the set_envp() call,
** which means that it was stored in a special format.
** variables set with the old set_env() call can be read with the
** get_envp() call only if the value consists of exactly 8 characters.
**
** The pid parameter contains the id of the process or block from which
** the variable should be read. Setting the pid parameter to zero is
** equivalent to using the pid of the current process.
**
** returns zero if the variable was not found. Otherwise the value
** of the environment variable is returned.
**
** The system call uses the get_sysconf() framework, which means that
** there is no public entrypoint to this function.
** This system call uses function 28 of get_sysconf().
** The user level prototype is:
** OSADDRESS get_envp(PROCESS pid, char *name);
*/

static OSADDRESS
sysconf_28(OSADDRESS pp, OSADDRESS q, OSADDRESS r)
{
  PROCESS pid = (PROCESS) pp;
  char *name = (char *) q;

  struct _pcb *pcb;
  struct env_buf *envbuf;
  char hash;
  char *p;
  size_t namelen;
  union SIGNAL *tmpbuf = NULL;
  
  (void) r; /* Remove ``not used arg'' compiler warning. */

#if ERROR_CHECKS > 0
  ON_ERRLEVEL_2()
    {
      /* Check that the name is not the NULL pointer. */
      if(unlikely(!name))
	{
	  os_error(0, OSE_EILLEGAL_ENVNAME | OSE_GET_ENV, (OSERRCODE) 0);	/* Non fatal. */
	  return (OSADDRESS) 0;	/* The variable was not found. */
	}
    }
#endif

  /* Calculate hash value from name. */
  for(hash = (char) 0, p = name; *p; p++)
    hash ^= *p;
  namelen = p - name;

  /* Run the interrupt version if the caller is an int or timer process. */
  if(zzcur_p->priority > 32)
    {
      /* Check if we should use the current_process value. */
      if(!pid)
	{
	  pcb = zzcur_p;
	}
      else
	{
	  /* Read instance number and check the process id. Report error if invalid. */
	  pcb = check_pid_bid(pid, OSE_GET_ENV);
	  if(pcb == NULL)
	    {
	      return (OSADDRESS) 0;
	    }

	  /* Locate the block or process descriptor. */
	  pcb = pid_to_pcb(pid);
	}

      /* Search for the indicated variable. */
      envbuf = pcb->env_list;
      if(!envbuf)
	{
	  /* Not found if env list in pcb is empty. */
	  return (OSADDRESS) 0;
	}

      for(;;)
	{
	  if(envbuf->name_hash == hash)
	    {
	      if(envbuf->cur_value == 8 && !strcmp(envbuf->name, name))
		{
		  break;
		}
	    }
	  envbuf = envbuf->next;
	  if(!envbuf)
	    {
	      /* Not found. Tell the caller so. */
	      return (OSADDRESS) 0;
	    }
	}

      /*
      ** Done if the variable was previously deleted.
      ** This is indicated by the value field being set to NULL.
      */
      if(!envbuf->value)
	{
	  /* Not found. Tell the caller so. */
	  return (OSADDRESS) 0;
	}

      /* Read and return the value. */
      {
	OSADDRESS *mval = (OSADDRESS *) envbuf->value;
	OSADDRESS result;

	/* The result is obtained by xor-ing the two words in the value. */
	result = mval[0] ^ mval[1];
	return result;
      }
    }

  /* Run the non-interrupt version if the caller isn't int or timer process. */
  else
    {
      /*
      ** If paging is enabled, save the name in a temporary buffer in the
      ** system pool to avoid page faults with a locked dispatcher.
      */

      if(zzuu.pagesize != 0)
	{
#if MEMORY_MODEL > 1
	  /* Use the kernel pool for next alloc. */
	  tmpbuf = sys_alloc(namelen + 1 + sizeof(SIGSELECT), 0);
#else
	  tmpbuf = alloc(namelen + 1 + sizeof(SIGSELECT), 0);
#endif
	  memcpy(tmpbuf, name, namelen + 1);
	  name = (char *) tmpbuf;
	}

      /* Check if we should use the current_process value. */
      if(!pid)
	{
	  pcb = zzcur_p;
	  /* Lock the dispatcher. */
	  LOCK_SWAP();
	}
      else
	{
	  /* Read instance number and check the process id. Report error if invalid. */
	  pcb = check_pid_bid(pid, OSE_GET_ENV);
	  if(pcb == NULL)
	    {
	      return (OSADDRESS) 0;
	    }

	  /* Locate the block or process descriptor. */
	  pcb = pid_to_pcb(pid);

	  /* Lock the dispatcher. */
	  LOCK_SWAP();

	  /* Check that the block/process is still present. */
	  if(pcb_to_pid(pcb) != pid)
	    {
	      UNLOCK_SWAP();
	      if(tmpbuf != NULL)
		{
		  free_buf(&tmpbuf);
		}
	      return (OSADDRESS) 0;
	    }
	}

      /* Search for the indicated variable. */
      envbuf = pcb->env_list;
      if(!envbuf)
	{
	  /* Not found if env list in pcb is empty. */
	  UNLOCK_SWAP();
	  if(tmpbuf != NULL)
	    {
	      free_buf(&tmpbuf);
	    }
	  return (OSADDRESS) 0;
	}

      for(;;)
	{
	  if(envbuf->name_hash == hash)
	    {
	      if(envbuf->cur_value == 8 && !strcmp(envbuf->name, name))
		{
		  break;
		}
	    }
	  envbuf = envbuf->next;
	  if(!envbuf)
	    {
	      /* Not found. Tell the caller so. */
	      UNLOCK_SWAP();
	      if(tmpbuf != NULL)
		{
		  free_buf(&tmpbuf);
		}
	      return (OSADDRESS) 0;
	    }
	}

      /* The name buffer is no more needed. */
      if(tmpbuf != NULL)
	{
	  free_buf(&tmpbuf);
	}

      /*
      ** Done if the variable was previously deleted.
      ** This is indicated by the value field being set to NULL.
      */
      if(!envbuf->value)
	{
	  /* Not found. Tell the caller so. */
	  UNLOCK_SWAP();
	  return (OSADDRESS) 0;
	}

      /* Read and return the value. */
      {
	OSADDRESS *mval = (OSADDRESS *) envbuf->value;
	OSADDRESS result;

	/* The result is obtained by xor-ing the two words in the value. */
	result = mval[0] ^ mval[1];

	/* Unlock the dispatcher and exit. */
	UNLOCK_SWAP();
	return result;
      }
    }
}

/*
** This is the set_envp system call.
**
** This call stores a 32 bit pointer in a named environment variable.
** This call stores the value in a special (mangled) format so that it can be
** efficiently read with the get_envp() call.
**
** variables set with the set_envp() call can be read with the get_env() call,
** but the string returned is not easily interpreted.
**
** The pid parameter contains the id of the process or block in which
** the variable should be set. Setting the pid parameter to zero is
** equivalent to using the pid of the current process.
**
** Returns a nonzero value if the call was successful.
** Returns zero if the kernel does not support the call or if the call
** failed due to an ignored error report.
**
** Restrictions: Setting a variable to zero is less useful as that value
** can not be properly read.
**
** The system call uses the get_sysconf() framework, which means that
** there is no public entrypoint to this function.
** This system call uses function 29 of get_sysconf().
** The user level prototype is:
** OSBOOLEAN set_envp(PROCESS pid, char *name, OSADDRESS value);
*/

static OSADDRESS
sysconf_29(OSADDRESS pp, OSADDRESS q, OSADDRESS r)
{
  PROCESS pid = (PROCESS) pp;
  char *name = (char *) q;
  OSADDRESS value = (OSADDRESS) r;

  OSADDRESS p[3];
  unsigned char *c, *x;

  /* Check if we should use the current_process value. */
  if(!pid)
    pid = current_process();

  /*
  ** Ascii-fy each byte of the pointer.
  ** We do this according to the following rules:
  ** c = 0x40    => c = c xor 0x60
  ** c = other   => c = c xor 0x40
  */
  p[0] = value ^ 0x40404040;
  p[1] = 0x40404040;
  p[2] = 0;
  c = (unsigned char *) &p[0];
  x = (unsigned char *) &p[1];
  for(;*x; x++, c++)
    if(!*c)
      {
	*x = 0x60;
	*c = 0x20;
      }
  return (OSADDRESS) !set_env(pid, name, (char *) p);
}


/* 
** This funtion creates a unique key value for a given string.
*/

static OSADDRESS
create_ppd_key(const char *key_string)
{
  LOCKSTATE msr;
  char * const hash_array = zzuu.ppd_hash_buf;
  OSADDRESS max_ppd_keys = zzuu.max_ppd_keys;
  char hash = (char)0;
  const char *cp;
  size_t key_size;
  unsigned long this_gen;
  OSADDRESS slot;
  char *key_buf_pointer;
  union SIGNAL *tmpbuf = NULL;

  for(cp = key_string; *cp; cp++)
    hash ^= *cp;
  key_size = cp - key_string;
  hash ^= (char) key_size;

  if(unlikely(key_size > zzuu.max_ppd_key_size))
    {
      os_error(0, OSE_ETOO_LONG_PPD_KEY_STRING,
	       (OSERRCODE) zzuu.max_ppd_key_size); /* Non fatal. */
      /*
      ** If the error was ignored we return a index value which is
      ** caught as illegal if any one tries to use it.
      */
      return 0xFFFFUL; /* @frbr@ We must handle this in pflib.c */
    }

  /* Add one to include nul termination. */
  key_size++;
  
  /* Update the generation counter to indicate a possible race. */
  BKL_LOCK_SAVE(msr);
 restart:
  /* Increase and sample the generation counter. */
  this_gen = ++zzuu.ppd_gen_counter; 
  BKL_LOCK_RESTORE(msr);
    
  /* First check if the key string exist, if so return index. */
  key_buf_pointer = zzuu.ppd_key_buf;
  for(slot=0ul; slot<max_ppd_keys; slot++)
    {
      if (hash == hash_array[slot] &&
	  memcmp(key_string, key_buf_pointer, key_size) == 0)
	{
	  goto done;
	}
      key_buf_pointer += zzuu.allocated_ppd_key_size;
    }

  if(unlikely(zzuu.next_free_ppd_index >= max_ppd_keys))
    {
      os_error(0, OSE_ETOO_MANY_PPD_KEYS, (OSERRCODE)max_ppd_keys); /* Non fatal. */
      /*
      ** If the error was ignored we return a index value which is
      ** caught as illegal if any one tries to use it.
      */
      slot = 0xFFFFUL; /* @frbr@ We must handle this in pflib.c */
      goto done;
    }

  /*
  ** If paging is enabled, save the name in a temporary buffer in the
  ** system pool to avoid page faults with a locked dispatcher.
  ** With paged pools, this buffer must be in the system pool.
  */

  if(zzuu.pagesize != 0 && tmpbuf == NULL)
    {
#if MEMORY_MODEL > 1
	  /* Use the kernel pool for next alloc. */
	  tmpbuf = sys_alloc(key_size + sizeof(SIGSELECT), 0);
#else
	  tmpbuf = alloc(key_size + sizeof(SIGSELECT), 0);
#endif
      memcpy(tmpbuf, key_string, key_size);
      key_string = (char *) tmpbuf;
    }

  /*
  ** We will put the key in the next_free slot. However, we need to
  ** lock and check the generation counter to see that no one
  ** interrupted us and did the same. In that case, we must restart
  ** the search.
  */

  BKL_LOCK_SAVE(msr);
  if(this_gen != zzuu.ppd_gen_counter)
    {
      goto restart;
    }
  slot = zzuu.next_free_ppd_index++;
  hash_array[slot] = hash;
  memcpy(&zzuu.ppd_key_buf[slot * zzuu.allocated_ppd_key_size], key_string,
	 key_size);
  BKL_LOCK_RESTORE(msr);

 done:
  if(tmpbuf != NULL)
    {
      free_buf(&tmpbuf);
    }
  return slot;
}

/*
** This function returns the address to the (user space allocated)
** array of per process data pointers.
*/

static OSADDRESS
get_ppd_addr(void)
{
#ifdef OSE_SMP
  return zzcur_p_usr->ppd_addr;
#else
  return zzcur_p->ppd_addr;
#endif
}

/*
** This is the sysconf_53() call which is the main interface for
** handling (fast usermode) per process data.
**
** Argument cmd is a command, see pose.h
** Argument arg is a the generic argument.
*/

static OSADDRESS
sysconf_53(OSADDRESS cmd, OSADDRESS arg, OSADDRESS r)
{
  LOCKSTATE msr;
  static struct _pcb *pcb;
  OSADDRESS rv;

  /* Remove ``not used args'' compiler warnings. */
  (void) r;

  switch(cmd)
    {
    case PFLIB_CREATE_PPD_KEY:
      rv = create_ppd_key((const char *) arg);
      break;

    case PFLIB_SET_PPD_PTR:
      zzcur_p->ppd_addr = arg;
      rv = 1u;
      break;

    case PFLIB_GET_PPD_PTR:
      if(arg == 0)
	{
	  rv = zzcur_p->ppd_addr;
	}
      else
	{
	  BKL_LOCK_SAVE(msr);
	  pcb = safe_pid_to_pcb((PROCESS)arg, 0);
	  rv = pcb != NULL
	    ? pcb->ppd_addr
	    : (OSADDRESS)0;
	  BKL_LOCK_RESTORE(msr);
	}
      break;

    case PFLIB_GET_PPD_KEYS:
      rv = zzuu.next_free_ppd_index;
      break;

    case PFLIB_GET_PPD_KEY_STRING:
      if (arg < zzuu.next_free_ppd_index)
	{
	  OSBUFSIZE size = zzuu.max_ppd_key_size + 1u;
	  char *str = (char *)alloc(size, 0);

	  BKL_LOCK_SAVE(msr);
	  memcpy(str,
		 &zzuu.ppd_key_buf[arg * zzuu.allocated_ppd_key_size],
		 size);
	  str[size - 1] = '\0'; /* Just to make sure. */
	  BKL_LOCK_RESTORE(msr);
	  rv = (OSADDRESS)str;
	}
      else
	rv = (OSADDRESS)0; /* Not found. */
      break;

    case PFLIB_GET_MAX_PPD_KEYS:
      rv = zzuu.max_ppd_keys;
      break;
 
    case PFLIB_GET_MAX_PPD_KEY_LEN:
      rv = zzuu.max_ppd_key_size;
      break;

    case PFLIB_ILLEGAL_PPD_KEY:
      os_error(0,OSE_EILLEGAL_PPDKEY,(OSERRCODE) arg);
      rv = 1;
      break;
 
    default:
      rv = (OSADDRESS)-1;	/* Illegal operation. */
    }
  return rv;  
}

/*
*****************************************************************************
** 22 BREAKPOINTS.
*****************************************************************************
*/

/*
**===========================================================================
** 22.1  Initialization.
**===========================================================================
*/

/*
** Initialize global variables associated with the breakpoint system.
*/

static void
init_break(void)
{
  /* Clear the global list of breakpoints. */
  zzuu.break_list = NULL;
}

/*
**===========================================================================
** 22.2  Local functions.
**===========================================================================
*/

/*
**===========================================================================
** 22.3  System calls.
**===========================================================================
*/

/*
** This is the set_bp system call.
*/
/* ===DEPRECATED=== */

/* Prototype to kill warnings. */
OSBOOLEAN
set_bp(PROCESS pid,
       OSADDRESS address,
       OSADDRESS attribute,
       union SIGNAL **trapsig);

OSBOOLEAN
set_bp(PROCESS pid,
       OSADDRESS address,
       OSADDRESS attribute,
       union SIGNAL **trapsig)
{
  (void)pid;
  (void)address;
  (void)attribute;
  (void)trapsig;
  
  return (OSBOOLEAN) 1;		/* ===DEPRECATED=== */
}

/*
** This is the clear_bp system call.
*/
/* ===DEPRECATED=== */

/* Prototype to kill warnings. */
OSBOOLEAN
clear_bp(PROCESS pid, OSADDRESS addr);

OSBOOLEAN
clear_bp(PROCESS pid, OSADDRESS addr)
{
  (void)pid;
  (void)addr;
  return (OSBOOLEAN) 1;		/* ===DEPRECATED=== */
}

/*
** This is the intercept system call.
**
** This function may be called also with the dispatcher locked.
** This is a feature needed by the LDM since it wants to operate
** on the breakpoint scope as an atomic operation.
**
** We can't maintain the swap lock on intercept/resume when
** remote calls occur.
** This will cause the debugger to fail in performing intercept
** operations atomically over several cpu's, but we
** won't do anything about that.
**
** The intercept and resume calls should scan the breakpoint list.??
** Any untripped soft breakpoint matching the process should be marked tripped.
** If a trip occurs, a kill_break signal is sent to sysd, which will remove
** the breakpoint and send any unsent trapsigs to the owners.
*/

void
intercept(PROCESS pid)
{
  (void)stop_or_intercept(pid, ST_INTERCEPTED, OSE_INTERCEPT, 0);
}

/*
** This is the resume system call.
**
** This function may be called also with the dispatcher locked.
** This is a feature needed by the LDM since it wants to operate
** on the breakpoint scope as an atomic operation.
**
** We can't maintain the swap lock on intercept/resume when
** remote calls occur.
** This will cause the debugger to fail in performing intercept
** operations atomically over several cpu's, but we
** won't do anything about that.
*/

void
resume(PROCESS pid)
{
  start_or_resume(pid, ST_INTERCEPTED, OSE_RESUME);
}

/*
**===========================================================================
** 22.4  Undocumented system calls.
**===========================================================================
*/

/*
** Intercept in the same way as the normal intercept() system call,
** except that system processes, other processes than bg- and
** pri-procs and the calling process won't be intercepted. Returns true
** if the calling process should intercept itself.
*/

OSBOOLEAN
zzkrn_dbg_intercept(PROCESS pid)
{
  return stop_or_intercept(pid, ST_INTERCEPTED, OSE_INTERCEPT, BUT_CURRENT_PROC
                           | BUT_INTERRUPT_PROC | BUT_SYSTEM_PROC);
}

/*
*****************************************************************************
** 23 SYSTEM INTERROGATION.
*****************************************************************************
*/

/*
**===========================================================================
** 23.1  Initialization.
**===========================================================================
*/

/*
**===========================================================================
** 23.2  Local functions.
**===========================================================================
*/

/*
** Subfunction to the get_cpu() system call.
**
** Allocate a signal buffer and return in it a copy of a string describing
** the current cpu.
*/

static char *
get_my_cpu(void)
{
  char *dst;
  const char *cpu = zzget_cpu_string();

  dst = (char *) alloc((OSBUFSIZE)(sizeof(SIGSELECT) + strlen(cpu)),
		       (SIGSELECT) 0);
  strcpy(dst, (const char *) cpu);
  return (char *) dst;
}

/*
** Subfunction to get_bid_list().
**
** Check if a block contains any processes with the
** designated  user number.
**
** This function is very inefficient in a large system. Perhaps
** we should introduce a pcb-counter pinpointing the blocks that contain
** processes of other users than the block itself.
** This variable could aid the check_bid() and check_pid_bid() calls too.??
**
** The dispatcher is locked while this function is called.
*/

static int
check_if_bid_visible(OSUSER req_user, struct _block *bcb)
{
  /* Check if any process in a block is visible. */
  register OSUSER user = req_user;
  register struct _pcb *pcb;
  for(pcb = bcb->proclist; pcb; pcb = pcb->proclist)
    if(user == pcb->real_user)
      return 1;
  return 0;
}

/*
** Subfunction to prepare_pcb_buffer() in get_pcb().
** Enters safe default values into all return structure fields
** except the type and remote_server fields.
*/

static void
setup_pcb_buffer(struct OS_pcb *buffer)
{
  static unsigned long dummy_stack[4];

  buffer->status = (OSADDRESS) 0;
  buffer->priority = 0;
  buffer->user = 0;
  buffer->fsemvalue = -1;
  buffer->sigqueue = 0;
  buffer->attach_list = 0;
  if(zzuu.interrupt_stack_size && zzww.interrupt_stack != NULL)
    {
      buffer->stack_top = (OSADDRESS) zzww.interrupt_stack->top - 1; /*@SMP@*/
      buffer->stack_limit = (OSADDRESS) zzww.interrupt_stack->limit; /*@SMP@*/
    }
  else
    {
      buffer->stack_top = (OSADDRESS) &dummy_stack[1];
      buffer->stack_limit = (OSADDRESS) dummy_stack;
    }
  buffer->sig_cnt_in_q = (OSADDRESS) 0;
  buffer->sig_cnt_owned = (OSADDRESS) 0;
  buffer->max_sigsize = (OSADDRESS) 0;
  buffer->sigsel_size = (OSADDRESS) sizeof(SIGSELECT);
  buffer->line = (OSADDRESS) 0;
  buffer->file = 0;
  buffer->name = 0;
  buffer->cpuregs = 0;
  buffer->wanted = 0;
  buffer->strings[0] = (char) '\0';
}

/*
** Subfunction to get_pcb().
** Allocates a buffer from the system pool and enters
** safe default values into all fields.
**
** The dispatcher must be locked while this function is called.
*/

static struct OS_pcb *
prepare_pcb_buffer(OSBUFSIZE stringlen,
		   enum PROCESS_TYPE type,
		   PROCESS remserver)
{
  register struct OS_pcb *buffer;

  set_suid((OSBOOLEAN) 1);
  buffer = (struct OS_pcb *) locked_alloc(sizeof(struct OS_pcb) + stringlen, 0);
  set_suid((OSBOOLEAN) 0);

  setup_pcb_buffer(buffer);

  buffer->type = (OSADDRESS) type;
  buffer->remote_server = remserver;

  return buffer;
}

/*
** Subfunction to get_pcb().
**
** Same as prepare_pcb_buffer(), except it is called from an
** interrupt or timer process.
*/

static struct OS_pcb *
int_prepare_pcb_buffer(OSBUFSIZE stringlen,
		       enum PROCESS_TYPE type,
		       PROCESS remserver)
{
  register struct OS_pcb *buffer;

  buffer = (struct OS_pcb *) alloc(sizeof(struct OS_pcb) + stringlen, 0);

  setup_pcb_buffer(buffer);

  buffer->type = (OSADDRESS) type;
  buffer->remote_server = remserver;

  return buffer;
}

/*
** Subfunction to get_pcb()
**
** Get the filename of the pending system call. 
** This function is used in a paging environment so page faults may occur.
** The dispatcher is not locked, so the segment may die. Therefore, we use
** zzguarded_memcpy. 
** The directory portions of the path are trimmed off. The filename is 
** truncated when longer than the result buffer. The result is always 
** zero-terminated. 
** The source file string may have any length, and may cross page borders. 
** The pid is not illegal but may be dead. The caller provides a reasonable
** scratch area.
** The directory delimiter characters should be possible configure??
*/

static void
get_file(const char *source,	/* from area */
	 char *dest, OSBUFSIZE dest_size, /* to area */
	 char *temp, OSBUFSIZE temp_size) /* scratch area */
{
  const int freeze_mode = 1;
  OSADDRESS from = (OSADDRESS) source;
  char *to = dest;
  char *cp;
  char *to_limit = dest + dest_size - 1;
  char *temp_limit = temp + temp_size - 1;
  char c;
  
  for(;;)
    {
      if(zzguarded_memcpy(temp, (const char *)from, temp_size, !freeze_mode))
	{
	  OSADDRESS from_pagelimit;

	  /* 
	  ** zzguarded_memcpy failed. Either the process is killed or we
	  ** exceeded the segment border. Issue a new zzguarded_memcpy()
	  ** that refers only one page.
	  */
	  
	  /* Give up if we are not using paging. */
	  if(zzuu.pagesize == 0)
	    break;		

	  from_pagelimit = (from + zzuu.pagesize)/* & ~(zzuu.pagesize - 1)*/;

	  /* Give up if we did not pass a page border. */
	  if(temp_size <= from_pagelimit - from)
	    break;		

	  /* Issue a read from a single page. */
	  temp_size = from_pagelimit - from;
	  if(zzguarded_memcpy(temp, (const char *)from, temp_size,
			      !freeze_mode))
	    {
	      /* zzguarded_memcpy failed. The process was killed. */
	      break;
	    }
	}

      from += temp_size;
      
      for(cp = temp; cp < temp_limit; cp++)
	{
	  c = *cp;
	  if (c == 0)
	    {
	      goto done;
	    }
	  if(c == '/' || c == '\\')
	    to = dest;
	  else
	    {
	      if (to < to_limit)
		*to++ = c;
	    }
	} /* for */
    } /* for */

  /* The process was killed */
  to = dest;

  /* Always put a string terminator */
 done:
  *to = 0;
}


/*
** Subfunction to the get_signal() system call.
**
** Checks if the handle given to the get_signal() system call is an
** attach list handle and creates a copy of the next attached
** signal.
**
** There is a bug here similar to the one in get_pcb().??
** We access the data part of a signal in another process
** while the dispatcher is locked.
**
** Must be called with a locked dispatcher. Returns with the dispatcher
** unlocked.
**
** All sanity checks of the caller and the supplied pid must be
** carried out by the caller.
**
** Returns zero if the call was successful.
** Returns a nonzero value if the call failed because we couldn't 
** find any attach victim entries in the attach list chain starting 
** att the entry pointed out by the attach_list parameter.
*/

static OSBOOLEAN
get_attached_signal(PROCESS pid,
		    OSADDRESS attach_list,
		    union SIGNAL **sig_copy,
		    OSADDRESS *nextsig)
{
  register struct _attach_entry *attent = pid_to_pcb(pid)->attached;
  register OSBOOLEAN seen_handle = 0;
  struct _signal *sigadm;
  union SIGNAL *sig;

  /* Scan the list of attached signals. */
  while(attent)
    {
      if(attent == (struct _attach_entry *)attach_list)
	seen_handle = 1;
      if(pid == attent->owner)
	attent = attent->own_fq;
      else
	{
	  /* assert(pid == attent->victim); */
	  /*
	  ** Don't match an attach entry if it's found before the 
	  ** supplied attach_list handle.
	  */
	  if (seen_handle)
	    break;
	  attent = attent->vic_fq;
	}
    }

  /* Unlock the dispatcher and return error flag if not found. */
  if(!attent)
    {
      UNLOCK_SWAP();
      return (OSBOOLEAN) 1;
    }

  /* Allocate a return buffer in kernel space. */
  set_suid((OSBOOLEAN) 1);
  if(attent->sig)
    {
      /* Create copy of user supplied attach signal. */
      sigadm = attent->sig;
      sig = locked_alloc(sigadm->size, (SIGSELECT) 0);
      /* Copy the data field. This is where the bug is?? */
      memcpy(sig, adm_to_sig(sigadm), (size_t) sigadm->size);
    }
  else
    {
      /* Create implicit attach signal */
      sig = locked_alloc(sizeof(SIGSELECT), OS_ATTACH_SIG);
    }
  set_suid((OSBOOLEAN) 0);

  /* Unlock the dispatcher. */
  UNLOCK_SWAP();

  /* Copy the buffer to the callers user pool. */
  sig = sig_dup(sig);
  sigadm = sig_to_adm(sig); /*@@ maybe check_sig here or short inline check. */

  /* Set sender and addressee fields. */
  sigadm->dest_id = attent->owner;
  sigadm->sender_id = attent->owner;

  /* Return the new signal buffer. */
  *sig_copy = sig;
  *nextsig = (OSADDRESS) attent->vic_fq;
  return (OSBOOLEAN) 0;
}

/*
**===========================================================================
** 23.3  System calls.
**===========================================================================
*/

/*
** This is the get_ptype system call.
**
** It is legal to call this function with interrupts disabled or
** with the dispatcher locked, but not with interrupt status pushed.
*/

enum PROCESS_TYPE
get_ptype(PROCESS pid)
{
  const PCB_INDEX_T pindex = pid_to_pindex(pid);
  const PCB_INSTANCE_T instance = pid_to_instance(pid);
  register struct _pcb *pcb;
  register enum PROCESS_TYPE type;
  register OSPRIORITY priority;
  struct _pcb *cur_p;

  /* Check that the pid is reasonable. */
  if((instance == 0) || (pindex == 0) || (pindex >= zzuu.next_free_pix))
    {
      return OS_ILLEGAL;
    }

  /* Pick up pcb pointer. */
  pcb = zzuu.pix_list[pindex];

  /* Done if the block/process/pool has been killed. */
  if(pcb_to_instance(pcb) != instance)
    {
      return OS_ZOOMBIE;
    }

  /* Check that the user numbers match. */
  /*
  ** Note user number visibility problem as in check_bid().??
  */
  cur_p = zzcur_p;
  if(cur_p->user)
    {
      if(pcb->magic == MAGIC_PCB)
	{
	  if(cur_p->user != pcb->real_user)
	    {
	      type = OS_ILLEGAL;
	      goto get_ptype1;
	    }
	}
      else
	{
	  if(cur_p->user != pcb->user)
	    {
	      type = OS_ILLEGAL;
	      goto get_ptype1;
	    }
	}
    }

  if(pcb->magic == MAGIC_BCB)
    {
      /* Found a block. */
      type = OS_BLOCK;
      goto get_ptype1;
    }

  /*
  ** Done if it is not a process.
  ** But accept a reference of the process being created when called from
  ** a create_handler!
  */
  if(pcb->magic != MAGIC_PCB)
    {
      if(zzuu.process_under_construction_ != pid ||
	 pcb->magic != (MAGIC_PCB | MAGIC_KILLED) ||
	 pcb_to_pid(cur_p) != zzuu.sysd_)
	{
	  type = OS_ILLEGAL;
	  goto get_ptype1;
	}
    }

  /* Extract process type from process priority. */
  priority = pcb->priority;
  if(priority < 32)
    type = OS_PRI_PROC;
  else if(priority < 33)
    type = OS_BG_PROC;
  else if(priority < 96)
    type = OS_INT_PROC;
  else if(priority < 128)
    type = OS_TI_PROC;
  else
    type = OS_PHANTOM;

  /* Return the process type. */
 get_ptype1:
  /* Check if the object has been killed by now. */
  if(pcb_to_instance((volatile struct _pcb*)pcb) != instance)
    {
      type = OS_ZOOMBIE;
    }
  return type;
}

/*
** This is the get_cpu system call.
*/

char *
get_cpu(PROCESS pid)
{
  register struct _pcb *pcb;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_GET_CPU))
    {
      return get_my_cpu();
    }
#endif

  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid_bid(pid, OSE_GET_CPU);
  if(pcb == NULL)
    {
      return get_my_cpu();
    }

  /* Locate the block or process descriptor. */
  pcb = pid_to_pcb(pid);

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the block/process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      UNLOCK_SWAP();
      return get_my_cpu();
    }

#if USE_REMOTE_CALLS
  /* Issue a remote system call if required. */
  if(local_sysctlblk.enable_remcalls)
  {
    union SIGNAL *sig;
    register OSBOOLEAN use_remcalls;
    register PROCESS handler;
    if(pcb->magic == MAGIC_PCB)
      {
	use_remcalls = pcb->block->use_remcalls;
	handler = pcb->block->remcall_server;
      }
    else
      {
	use_remcalls = ((struct _block *)pcb)->use_remcalls;
	handler = ((struct _block *)pcb)->remcall_server;
      }
    if(use_remcalls)
      {
	UNLOCK_SWAP();
	set_suid((OSBOOLEAN) 1);
	sig = alloc(sizeof(struct remote_call), REMOTE_CALL);
	sig->remote_call.call_type = REM_GET_CPU;
	sig->remote_call.params.rem_get_cpu.pid = pid;
	
	sig = make_remote_call(sig, (OSCALLTYPE) REMR_GET_CPU, handler);
	if(sig)
	  {
	    char *dst;
	    set_suid((OSBOOLEAN) 0);
	    dst = (char *) alloc(
				 (OSBUFSIZE)(sizeof(SIGSELECT) +
					     strlen(sig->remote_response.results.remr_get_cpu.name)),
				 (SIGSELECT) 0);
	    strcpy(dst, (const char *) sig->remote_response.results.remr_get_cpu.name);
	    free_buf(&sig);
	    return (char *) dst;
	  }
	else
	  {
	    set_suid((OSBOOLEAN) 0);
	    return get_my_cpu();
	  }
      }
    }
#endif

  /* Unlock the dispatcher and return the current cpu string. */
      UNLOCK_SWAP();
      return get_my_cpu();
}

/*
** This is the get_uid system call.
*/

OSUSER
get_uid(PROCESS pid)
{
  register struct _pcb *pcb;
  OSUSER user;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_GET_UID))
    return zzcur_p->user;
#endif

  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid_bid(pid, OSE_GET_UID);
  if(pcb == NULL)
    {
    return zzcur_p->user;
    }

  /* Locate the block or process descriptor. */
  pcb = pid_to_pcb(pid);

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the block/process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      UNLOCK_SWAP();
      return zzcur_p->user;
    }

#if USE_REMOTE_CALLS
  /* Issue a remote system call if required. */
  if(local_sysctlblk.enable_remcalls)
  {
    union SIGNAL *sig;
    register OSBOOLEAN use_remcalls;
    register PROCESS handler;
    if(pcb->magic == MAGIC_PCB)
      {
	use_remcalls = pcb->block->use_remcalls;
	handler = pcb->block->remcall_server;
      }
    else
      {
	use_remcalls = ((struct _block *)pcb)->use_remcalls;
	handler = ((struct _block *)pcb)->remcall_server;
      }
    if(use_remcalls)
      {
	UNLOCK_SWAP();
	set_suid((OSBOOLEAN) 1);
	sig = alloc(sizeof(struct remote_call), REMOTE_CALL);
	sig->remote_call.call_type = REM_GET_UID;
	sig->remote_call.params.rem_get_uid.pid = pid;
	
	sig = make_remote_call(sig, (OSCALLTYPE) REMR_GET_UID, handler);
	if(sig)
	  {
	    user = sig->remote_response.results.remr_get_uid.uid;
	    free_buf(&sig);
	    set_suid((OSBOOLEAN) 0);
	    return user;
	  }
	else
	  {
	    set_suid((OSBOOLEAN) 0);
	    return zzcur_p->user;
	  }
      }
  }
#endif

  /* Return the user number of the specified process or block. */
  if(pcb->magic == MAGIC_PCB)
    {
      user = pcb->real_user;
    }
  else
    user = pcb->user;
  UNLOCK_SWAP();
  return user;
}

/*
** This is the set_uid system call.
**
** The set_uid() call is obsolete. Delete it and modify create_process()??
*/

static void
set_uid(PROCESS pid, OSUSER user)
{
  register struct _pcb *pcb;
  union SIGNAL *sig;

  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid(pid, OSE_CREATE_PROCESS);
  if(pcb == NULL)
    {
    return;
    }

  /* Locate the process descriptor. */
  pcb = pid_to_pcb(pid);

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the caller is a superuser process. Done if not. */
  if(zzcur_p->user)
    {
      UNLOCK_SWAP();
      return;
    }

  /* Check that the block/process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      UNLOCK_SWAP();
      return;
    }

  /* Set the user number of the specified process. */
  pcb->user = pcb->real_user = user;
  UNLOCK_SWAP();

  /*
  ** Return attached signals to processes that can no longer see the process.
  ** ??
  */

  /*
  ** Return hunt signals to processes that can now see the changed process.
  ** We do this by sending a request to the hunt daemon.
  */

  set_suid((OSBOOLEAN) 1);
  sig = alloc(sizeof(struct new_uid), NEW_UID);
  sig->new_uid.pid = pid;
  sig->new_uid.user = user;
  send(&sig, zzuu.huntd_);
  set_suid((OSBOOLEAN) 0);
}

/*
** This is the get_pid_list system call.
*/

struct OS_pid_list *
get_pid_list(PROCESS bid)
{
  register struct _pcb *pcb;
  struct _block *bcb;
  struct OS_pid_list *buffer;
  register PROCESS *p;
  register OSBUFSIZE count;
  OSBUFSIZE max_count;
  OSUSER user = zzcur_p->user;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_GET_PID_LIST))
    return NULL;
#endif

  /* Locate the block descriptor. */
  /* Read instance number and check the block id. Report error if invalid. */
  bcb = check_bid(bid, OSE_GET_PID_LIST);
  if(bcb == NULL)
    {
      return NULL;
    }

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the block is still present. */
  if(pcb_to_pid(bcb) != bid)
    {
      UNLOCK_SWAP();
      return NULL;
    }

#if USE_REMOTE_CALLS
  /* Issue a remote system call if required. */
  if(local_sysctlblk.enable_remcalls)
    {
    union SIGNAL *sig;
    register OSBOOLEAN use_remcalls;
    register PROCESS handler;
    use_remcalls = bcb->use_remcalls;
    handler = bcb->remcall_server;
    if(use_remcalls)
      {
	UNLOCK_SWAP();
	set_suid((OSBOOLEAN) 1);
	sig = alloc(sizeof(struct remote_call), REMOTE_CALL);
	sig->remote_call.call_type = REM_GET_PID_LIST;
	sig->remote_call.params.rem_get_pid_list.bid = bid;
	
	sig = make_remote_call(sig, (OSCALLTYPE) REMR_GET_PID_LIST, handler);
	set_suid((OSBOOLEAN) 0);
	if(sig)
	  {
	    PROCESS *d, *s;
	    count = ((struct OS_pid_list *) sig)->count =
	      sig->remote_response.results.remr_get_pid_list.count;
	    for(d = ((struct OS_pid_list *) sig)->list, s = sig->remote_response.results.remr_get_pid_list.pidlist;
		count; count--)
	      *d++ = *s++;
	    return (struct OS_pid_list *) sig;
	  }
	return NULL;
      }
    }
#endif

  /* Scan the blocks process list and count number of available processes. */
  if(user)
    {
      for(count = 0, pcb = bcb->proclist; pcb; pcb = pcb->proclist)
	{
	  if(!pcb->killed && user == pcb->real_user)
	    count++;
	}
    }
  else
    {
      for(count = 0, pcb = bcb->proclist; pcb; pcb = pcb->proclist)
	{
	  if(!pcb->killed)
	    count++;
	}
    }

  /* Return NULL if no processes were found. */
  if(!count)
  {
    UNLOCK_SWAP();
    return NULL;
  }

  /* Truncate the count if no suitable buffer exists. */
  max_count = 1 + (zzcur_p->cursigpool->signal_list[7].size -
		   sizeof(struct OS_pid_list)) / sizeof(PROCESS);
  if(count > max_count)
    {
      count = max_count;
    }
  else
    {
      max_count = count;
    }
  
  /* Unlock the dispatcher and allocate a return buffer. */
  UNLOCK_SWAP();
  buffer = (struct OS_pid_list *)
    alloc(sizeof(struct OS_pid_list) + sizeof(PROCESS) * (count - 1), 0);
  LOCK_SWAP();

  /* Check that the block is still present. */
  if(pcb_to_pid(bcb) != bid)
    {
      UNLOCK_SWAP();
      free_buf((union SIGNAL **)&buffer);
      return NULL;
    }
  
  /* The number of processes may have changed. */
  if(user)
    {
      for(count = 0, pcb = bcb->proclist; pcb; pcb = pcb->proclist)
	{
	  if(!pcb->killed && user == pcb->real_user)
	    count++;
	}
    }
  else
    {
      for(count = 0, pcb = bcb->proclist; pcb; pcb = pcb->proclist)
	{
	  if(!pcb->killed)
	    count++;
	}
    }
  
  /*
  ** Copy all process identities to the new buffer. If the current count
  ** won't fit in the buffer, we drop the newest created processes.
  */
  buffer->count = count < max_count ? count : max_count;
  if(user)
    {
      for(p = buffer->list, pcb = bcb->proclist; count > 0; pcb = pcb->proclist)
	{
	  if(!pcb->killed && user == pcb->real_user)
	    {
              if (count-- <= max_count)
                *p++ = pcb_to_pid(pcb);
	    }
	}
    }
  else
    {
      for(p = buffer->list, pcb = bcb->proclist; count > 0; pcb = pcb->proclist)
	if(!pcb->killed)
          {
            if (count-- <= max_count)
              *p++ = pcb_to_pid(pcb);
          }
    }

  /* Return the buffer. */
  UNLOCK_SWAP();
  return buffer;
}

/*
** This is the get_bid_list system call.
*/

struct OS_pid_list *
get_bid_list(OSUSER req_user)
{
  OSBUFSIZE count;
  register struct _block *bcb;
  register OSUSER user = req_user;
  struct OS_pid_list *buffer;
  PROCESS *pid;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_GET_BID_LIST))
    return NULL;
#endif

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Select the proper user number. */
  if(zzcur_p->user)
    {
      user = zzcur_p->user;
    }

  /*
  ** Scan global block list and count the number of recognized blocks.
  ** If the caller is a superuser requesting user zero, then all blocks
  ** in the system are listed.
  */
  if(user)
    {
      for(count = 0, bcb = zzuu.bcb_list; bcb; bcb = bcb->next)
	{
	  /*
	  ** Check for correct user number and skip those
	  ** that don't match the caller.
	  ** Note that a block should be visible if at least one process
	  ** in the block is visible.
	  */
	  if(!bcb->killed)
	    {
	      if(user == bcb->user)
		{
		  count++;
		}
	      else
		{
		  if(check_if_bid_visible(user, bcb))
		    {
		      count++;
		    }
		}
	    }
	}
    }
  else
    {
      for(count = 0, bcb = zzuu.bcb_list; bcb; bcb = bcb->next)
	{
	  if(!bcb->killed)
	    {
	      count++;
	    }
	}
    }

  /* Return NULL if no blocks were found. */
  if(!count)
    {
      UNLOCK_SWAP();
      return NULL;
    }

  /* Use the system pool. */
  set_suid(1);

  /* Allocate a return buffer. */
  buffer = (struct OS_pid_list *)
    locked_alloc(sizeof(struct OS_pid_list) + ((count - 1) * sizeof(PROCESS)), 0);

  /* Use the application pool again. */
  set_suid(0);

  /* Copy all block identities to the new buffer. */
  buffer->count = count;
  if(user)
    {
      for(pid = buffer->list, bcb = zzuu.bcb_list; bcb; bcb = bcb->next)
	{
	  /*
	  ** Check for correct user number and skip those
	  ** that don't match the caller.
	  ** Note that a block should be visible if at least one process
	  ** in the block is visible.
	  */
	  if(!bcb->killed)
	    {
	      if(user == bcb->user)
		{
		  *pid++ = pcb_to_pid(bcb);
		}
	      else
		{
		  if(check_if_bid_visible(user, bcb))
		    {
		      *pid++ = pcb_to_pid(bcb);
		    }
		}
	    }
	}
    }
  else
    {
      for(pid = buffer->list, bcb = zzuu.bcb_list; bcb; bcb = bcb->next)
	{
	  if(!bcb->killed)
	    {
	      *pid++ = pcb_to_pid(bcb);
	    }
	}
    }

  /* Return the buffer. */
  UNLOCK_SWAP();
  return (struct OS_pid_list *) sig_dup((union SIGNAL *)buffer);
}

/*
** This is the get_pcb system call.
**
** We do not support examination of the signal queue for timer
** or interrupt processes.
**
** In a paging system, it is possible to construct use cases where the
** filename and wanted array may contain misleading information?? This
** requires interrupts scheduling the get_pcb victim process during this
** call, as well as non-constant filename resp. sigselect arrays. For
** the filename, it also requires that file and line are explicitely
** given by the user as debug call parameters. For the wanted array,
** the victim process must also issue a new receive call from
** another stack frame. 
**
** Another option would be to stop the examined process (with a new
** invisible start/stop pair) and disable the kill_proc system call
** so that the examined process as well as the caller remains intact.
**
** When inspecting a mas_mapped object, the caller may be transfered to
** its own home segment?? 
*/

struct OS_pcb *
get_pcb(PROCESS pid)
{
  register struct _pcb *pcb;
  register struct _block *bcb;
  const SIGSELECT *s;
  SIGSELECT *d;
  W16 count = 0;
  OSBOOLEAN process_object_in_other_mas_mapped_segment;
  struct OS_pcb *buffer;
  enum PROCESS_TYPE type;
  OSBUFSIZE stringlen = 0; /* Space needed in strings incl termin. */

  /* Actual offsets into string pool for the various fields. */
  OSADDRESS file = 0;
  OSADDRESS name; /* Need not to be initialized */
  OSADDRESS cpuregs = 0;
  OSADDRESS wanted = 0;
  const W16 max_wanted_elements = 50; /* Max wanted array item count. */



  /* Run the interrupt version if the caller is an int or timer process. */
  if(zzcur_p->priority > 32)
    {
      /* Read the type of the process or block. */
      type = get_ptype(pid);

      /* Done if an illegal id or a killed process/block. */
      if((type == OS_ILLEGAL) || (type == OS_ZOOMBIE))
	{
	  buffer = int_prepare_pcb_buffer(stringlen, type, pid);
	  return buffer;
	}

      /* Locate the block or process descriptor. */
      pcb = pid_to_pcb(pid);
      bcb = (struct _block *) pcb;

      /*
      ** Calculate size of the strings field in the return structure and
      ** check for a remote call handler.
      */
      name = 1;
      if(type == OS_BLOCK)
	{
	  stringlen = 1 + strlen(bcb->name);
	}
      else
	{
	  stringlen = 1 + strlen(pcb->name);

          /* Append optional instance name if process is outside our block. */
          if((pcb->block->lnh_name) && (zzcur_p->block != pcb->block))
            {
              stringlen += strlen(pcb->block->lnh_name);
            }          
	}
      
      /*
      ** Always allow space for a wildcard sigselect array.
      ** We do it this way since we cannot afford to read the
      ** user space from an interrupt process and possibly cause a
      ** page fault.
      ** Sorry about the somewhat misleading receive status.
      */
      /* Align actual address. */
      wanted = ((4 + stringlen) & (OSBUFSIZE) ~3);
      stringlen = (OSBUFSIZE) (wanted - 1);
      stringlen += sizeof(SIGSELECT);

      /* Allocate a buffer of suitable size. */
      buffer = int_prepare_pcb_buffer(stringlen + 1, type, pid);

      /* Set up all pointers into the strings field. */
      buffer->name = name;
      buffer->wanted = wanted;
      
      /* Gather process or block information. */
      if(type == OS_BLOCK)
	{
	  /* Get a copy of the block name. */
	  strcpy(&buffer->strings[name], (const char *) bcb->name);
	}
      else
	{
	  /* Get a copy of the process name. */
          if((pcb->block->lnh_name) && (zzcur_p->block != pcb->block))
            {
              strcpy(&buffer->strings[name], (const char *) pcb->block->lnh_name);
              strcpy(&buffer->strings[name + strlen(pcb->block->lnh_name)], (const char *) pcb->name);
            }
          else
            {
              strcpy(&buffer->strings[name], (const char *) pcb->name);
            }           
	  
	  /* Get process status for all process types. */
	  buffer->status = (OSADDRESS) ((pcb->status & 0x1fc) >> 2);
	  
	  /* Get the process priority for all process types. */
	  buffer->priority = pcb->priority & 31;
	  
	  if(type != OS_PHANTOM)
	    {
	      /* Count number of signals owned by any process. */
	      buffer->sig_cnt_owned = pcb->sig_cnt_owned;

	      /* Get the current fast semaphore value. */
	      buffer->fsemvalue = pcb->fsemval;
	    }
	}

#ifdef OSE_SMP
      if((type == OS_INT_PROC) || (type == OS_TI_PROC))
	{
	  /* Get the current stack limits. */
	  buffer->stack_top =
	    (OSADDRESS)zzcpu[pcb->cpu_id].interrupt_stack->top - 1;
	  buffer->stack_limit =
	    (OSADDRESS)zzcpu[pcb->cpu_id].interrupt_stack->limit;
	}
#endif
      if((type == OS_PRI_PROC) || (type == OS_BG_PROC))
	{
	  /* Get the current stack limits. */
	  buffer->stack_top = (OSADDRESS) pcb->stack->top - 1;
	  buffer->stack_limit = (OSADDRESS) pcb->stack->limit;
	  
	  /* Get a dummy wanted array in case the process is in a receive call. */
	  if(buffer->status & OS_RECEIVE)
	    {
	      /* Modify process status if we are in a remote system call. */
	      if(pcb->in_rem_call)
		buffer->status = (OSADDRESS)
		    ((buffer->status & ~(OSADDRESS)OS_RECEIVE) | OS_REMOTE);
	      
	      /* Create the dummy wanted array. */
	      *((SIGSELECT *) &buffer->strings[wanted]) = (SIGSELECT) 0;
	    }
	}
      
      /* Return the buffer to the caller. */
      return buffer;
    }
  /* Run the non interrupt version if the caller is a prioritized or background process. */
  else
    {
      /* SMP: enable interrupts to survive smp_lock assert code. */
      SMP_CODE(UNLOCK());

      /* Lock the dispatcher. */
      LOCK_SWAP();

      /* Read the type of the process or block. */
      type = get_ptype(pid);

      /* Done if an illegal id or a killed process/block. */
      if((type == OS_ILLEGAL) || (type == OS_ZOOMBIE))
	{
	  buffer = prepare_pcb_buffer(stringlen, type, pid);
	  UNLOCK_SWAP();
	  return (struct OS_pcb *) sig_dup((union SIGNAL *)buffer);
	}

      /* Locate the block or process descriptor. */
      pcb = pid_to_pcb(pid);
      bcb = (struct _block *) pcb;

#if USE_REMOTE_CALLS
      /* Issue a remote system call if required. */
      if(local_sysctlblk.enable_remcalls)
      {
	union SIGNAL *sig;
	register OSBOOLEAN use_remcalls;
	register PROCESS handler;
	if(pcb->magic == MAGIC_PCB)
	  {
	    use_remcalls = pcb->block->use_remcalls;
	    handler = pcb->block->remcall_server;
	  }
	else
	  {
	    use_remcalls = ((struct _block *)pcb)->use_remcalls;
	    handler = ((struct _block *)pcb)->remcall_server;
	  }
	if(use_remcalls)
	  {
	    UNLOCK_SWAP();
	    set_suid((OSBOOLEAN) 1);
	    sig = alloc(sizeof(struct remote_call), REMOTE_CALL);
	    sig->remote_call.call_type = REM_GET_PCB;
	    sig->remote_call.params.rem_get_pcb.pid = pid;
	
	    sig = make_remote_call(sig, (OSCALLTYPE) REMR_GET_PCB, handler);
	    if(sig)
	      {
		ptrdiff_t offset = 
		  &sig->remote_response.results.remr_get_pcb.pcb - (struct OS_pcb *) sig;
		int len = (int) sigsize(&sig) - (int) offset;
		register char *dd, *ss;
		for(dd = (char *) sig, ss = (char *) &sig->remote_response.results.remr_get_pcb.pcb;
		    len; len--)
		  *dd++ = *ss++;
		set_suid((OSBOOLEAN) 0);
		return (struct OS_pcb *) sig_dup((void *) sig);
	      }
	    else
	      {
		/*
		** Give actual type of process if no response from handler.
		** We must duplicate some of the work previously done since the
		** process may have been killed.
		*/
		set_suid((OSBOOLEAN) 0);
		LOCK_SWAP();

		/* Read the type of the process or block. */
		type = get_ptype(pid);

		/* Done if the process was killed. */
		if(type == OS_ZOOMBIE)
		  {
		    buffer = prepare_pcb_buffer(stringlen, type, pid);
		    UNLOCK_SWAP();
		    return (struct OS_pcb *)sig_dup((union SIGNAL *)buffer);
		  }
	      }
	  }
      }
#endif

      /*
      ** Calculate size of the strings field in the return structure and
      ** check for a remote call handler.
      */
      name = 1;
      if(type == OS_BLOCK)
	{
	  stringlen = 1 + strlen(bcb->name);
	}
      else
	{
	  stringlen = 1 + strlen(pcb->name);

          /* Append optional instance name if process is outside our block. */
          if((pcb->block->lnh_name) && (zzcur_p->block != pcb->block))
            {
              stringlen += strlen(pcb->block->lnh_name);
            }          
	}

      if((type == OS_PRI_PROC) || (type == OS_BG_PROC))
	{
#if USE_CAPTURE_REGS
	  cpuregs = (4 + stringlen) & (OSBUFSIZE) ~3;
	  stringlen = cpuregs + MAX_REGLEN - 1;
#else
	  cpuregs = (1 + stringlen);	/* Original line. */
	  stringlen += MAX_REGLEN;
#endif
	  file = 1 + stringlen;
	  stringlen += MAX_FILE_NAMELEN + 1;
	}

      if((type == OS_PRI_PROC) || (type == OS_BG_PROC))
      {
	 /*
	 ** Allocate space for sigselect entries. Unused space
	 ** should be stripped off later, by calling set_sigsize().
	 */
	 
	 /* Align actual address. */
	 wanted = ((4 + stringlen) & (OSBUFSIZE) ~3);
	 stringlen = (OSBUFSIZE) (wanted - 1);
	 stringlen += (sizeof(SIGSELECT) * max_wanted_elements);
      }

      /* Allocate a buffer of suitable size. */
      buffer = prepare_pcb_buffer(stringlen + 1, type, pid);

    restart:
      
      /*
      ** DRIVE CAREFULLY!
      ** 1) In a system with paging .text we must not reference the text 
      ** section with locked dispatcher, because the page fault handling 
      ** may require a context switch. The .text section may contain
      ** sourcecode filename and/or a sigselect array.
      ** 2) If the investigated object has MAS mapped regions, they
      ** may currently not be visible. A mas mapped .data section may
      ** contain a sigselect array.
      ** 3) Access of Block objects are not affected by mas or paging.
      */
      process_object_in_other_mas_mapped_segment =
	type != OS_BLOCK &&
	zzcur_p->segid != pcb->block->segid &&
	(pcb->block->segid == 0 ? system_segment.mas_mapped
	 : ((struct segcb*)pid_to_pcb(pcb->block->segpid))->mas_mapped);

      if(type != OS_BLOCK)
	{
	  /* Get process status for all process types. */
	  buffer->status = (OSADDRESS) ((pcb->status & 0x1fc) >> 2);
	}

      if((type == OS_PRI_PROC) || (type == OS_BG_PROC))
	{
	  /* Endmark index for shrinking the buffer. */
	  int strings_endmark_index = wanted + sizeof(SIGSELECT);
	  
	  /* Get the first signal queue pointer. */
	  buffer->sigqueue = (OSADDRESS) pcb->mailbox;
	  
	  if(zzuu.pagesize || process_object_in_other_mas_mapped_segment)
	    {	

	      /*
	      ** Paging is enabled, or we refer a process in another segment
	      ** than the current, and that segment contains mas mapping.
	      ** If that process did set_segment() and receive(), then
	      ** zzguarded_memcpy() below may fail, and will force us to 
	      ** return an empty sigselect array.
	      */

	      OSSEGMENT object_segid = pcb->block->segid;
	      char *filename = NULL; /* Silent compiler warning. */
	      const SIGSELECT *sigsel_ptr;
	      SIGSELECT sigsel_cnt;
	      OSBOOLEAN make_an_empty_sigselect;

	      /*
	      ** A user mode program will not read a sigselect mask of
	      ** another program with data in MAS. (or perhaps: A program with
	      ** MAS data will not read a sigselect from another MAS program.)
	      */
	      make_an_empty_sigselect = !zzcur_p->block->supervisor_mode
		&& process_object_in_other_mas_mapped_segment;

	      /* Save the SIGSELECT array pointer. */
	      sigsel_ptr = pcb->wanted;

	      /*
	      ** Get file and line of the current system call.
	      */
	      buffer->line = 0;
	      buffer->strings[file] = (char) 0;
#if ZZGET_FILE_LINE
	      if(pcb->uarea)
		zzget_file_line(pcb->uarea, &buffer->line, &filename);
#endif

	      /* Unlock dispatcher before refering user space. */
	      UNLOCK_SWAP();

	      /*
	      ** If mas mapping involved, select that segment.
	      ** This may fail if the segment is dead, but we don't care,
	      ** the failure is detected later anyhow.
	      */
	      if(process_object_in_other_mas_mapped_segment
		 && zzcur_p->block->supervisor_mode)
		{
		  (void)set_segment(0, object_segid);
		}

	      /* Get filename of the current system call. */
	      if(buffer->line)
		{
		  get_file(filename, /* from */
			   &buffer->strings[file], MAX_FILE_NAMELEN, /* to */
			   &buffer->strings[wanted], /* scratch */
			   max_wanted_elements * sizeof(SIGSELECT));
		}

	      /* copy the wanted array if the process is in a receive call. */
	      if(buffer->status & OS_RECEIVE && !make_an_empty_sigselect)
		{
		  const int freeze_mode = 1;

		  /*
		  ** Pick up the size of the sigselect array. Make it positive.
		  **
		  ** We do this test by truncating to a 16 bit variable before
		  ** testing the sign bit. The reason for this is that we want
		  ** to make sure that everything works even for pose ports
		  ** with 16 bit signal numbers.
		  ** The code breaks for select arrays with more than 32k entries, ??
		  ** which is an unreasonable size.
		  */
		  if(zzguarded_memcpy(&sigsel_cnt, (const char *)sigsel_ptr,
				      sizeof(sigsel_cnt), !freeze_mode))
		    {
		      /* Failed. The process has been killed. */
		      sigsel_cnt = 0;
		    }
		  count = (W16) sigsel_cnt;
		  if(count < 0)
		    count = (W16) -count;
		  count++;
		  if(count > max_wanted_elements)
		    count = max_wanted_elements;

		  /* Copy the wanted array. The first word is fixed up later. */
		  if(zzguarded_memcpy(&buffer->strings[wanted],
				      (const char *)sigsel_ptr, 
				      count * sizeof(SIGSELECT), !freeze_mode))
		    {
		      /* Failed. The process has been killed. */
		      sigsel_cnt = 0;
		      /*
		      ** Just in case the process really is alive still, but
		      ** we fail to access the sigselect array, report it as
		      ** empty.
		      */
		      *(SIGSELECT*)&buffer->strings[wanted] = 0;
		    }
		}

	      /* If mas mapping involved, return to the home segment. */
	      if(process_object_in_other_mas_mapped_segment
		 && zzcur_p->block->supervisor_mode)
		{
		  (void)set_segment(1, 0); /* Go home. */
		}

	      /* Lock the dispatcher again. */
	      LOCK_SWAP();
	      
	      /* If the process has been killed, return a proper buffer. */
	      type = get_ptype(pid);

	      /* Done if the process was killed. */
	      if(type == OS_ZOOMBIE)
		{
		  union SIGNAL *old_buffer = (union SIGNAL *) buffer;

		  free_buf(&old_buffer);
		  stringlen = 0;
		  buffer = prepare_pcb_buffer(stringlen, type, pid);
		  UNLOCK_SWAP();
		  return (struct OS_pcb *) sig_dup((void *) buffer);
		}

	      /* Retrieve current process status. */
	      buffer->status = (OSADDRESS) ((pcb->status & 0x1fc) >> 2);

	      /* Update the sigselect array depending on the current status. */
	      if((pcb->status & ST_RECEIVE) && !make_an_empty_sigselect)
		{
		  SIGSELECT new_sigsel_cnt =
		    *(SIGSELECT *) &buffer->strings[wanted];
		  W16 new_count;

		  new_count = (W16) new_sigsel_cnt;
		  if(new_count < 0)
		    new_count = (W16) -new_count;
		  new_count++;
		  if(new_count > max_wanted_elements)
		    new_count = max_wanted_elements;

		  /* Check if the count in the wanted array was changed. */
		  if(count >= new_count)
		    {
		      /* New sigselect is shorter or equal. Truncate array. */
		      count = new_count;
		      sigsel_cnt = new_sigsel_cnt;
		    }
		  else
		    {
		      /*
		      ** New sigselect array is longer.
		      ** We reread the wanted array. There is no risc for 
		      ** an eternal loop since the array can't grow forever.
		      ** (We could truncate the array instead, by taking the
		      ** abs value of the old count with the new count sign.) 
		      */
		      goto restart;
		    }

		  /* Modify process status if we are in a remote system call. */
		  if(pcb->in_rem_call)
		    buffer->status = (OSADDRESS)
		      ((buffer->status & ~(OSADDRESS)OS_RECEIVE) | OS_REMOTE);
		}
	      else
		{
		  /* No longer in receive. Make wanted array empty. */
		  sigsel_cnt = 0;
		  count = 1;
		}

	      /* Set the wanted sigselect count */
	      *(SIGSELECT *) &buffer->strings[wanted] = (0 <= (W16) sigsel_cnt)
		?  (SIGSELECT) (count - 1)
		: -(SIGSELECT) (count - 1);

	      /* Calculate where to put the buffer endmark. */
	      strings_endmark_index = wanted + (U16)count * sizeof(SIGSELECT);

	    } /* Paging */
	  else
	    {
	      /*
	      ** Paging is disabled. Select the fast and easy way.
	      */

	      /*
	      ** This is needed in SMP kernels since receive related
	      ** system calls does not use the scheduler lock, 
	      ** 
	      */
	      SMP_CODE(BKL_LOCK());
	      
	      if(pcb->status & ST_RECEIVE)
		{
		  /*
		  ** Pick up the size of the sigselect array. Make it positive.
		  **
		  ** We do this test by truncating to a 16 bit variable before
		  ** testing the sign bit. The reason for this is that we want
		  ** to make sure that everything works even for pose ports
		  ** with 16 bit signal numbers.
		  ** The code breaks for select arrays with more than 32k entries, ??
		  ** which is an unreasonable size.
		  */
		  count = (W16) *(pcb->wanted);
		  if(count < 0)
		    count = (W16) -count;
		  count++;
		  if(count > max_wanted_elements)
		    count = max_wanted_elements;

		  /* Calculate where to put the buffer endmark. */
		  strings_endmark_index = wanted + (U16)count * sizeof(SIGSELECT);

		  /* Copy the wanted array. */
		  d = (SIGSELECT *) &buffer->strings[wanted];
		  s = pcb->wanted;
		  *d++ = ((W16) *(pcb->wanted)) >= 0
		    ?  (SIGSELECT) (--count)
		    : -(SIGSELECT) (--count);
		  s++;
		  while(count--)
		    *d++ = *s++;
		  
		  if(pcb->in_rem_call)
		    {
		      /*
		      ** Modify process status if we are in a remote
		      ** system call.
		      */
		      buffer->status = (OSADDRESS)
			((buffer->status & ~(OSADDRESS)OS_RECEIVE) | OS_REMOTE);
		    }
		  else
		    {
		      /* Make sure the buffer indicates state OS_RECEIVE. */
		      buffer->status |= OS_RECEIVE;
		    }
		}
	      else
		{
		  /*
		  ** Make sure the buffer does *NOT*  indicates state
		  ** OS_RECEIVE.
		  */
		  buffer->status &= ~(OSADDRESS)OS_RECEIVE;
		}

	      SMP_CODE(BKL_UNLOCK());
		
	      /* Get file and line of the current system call. */
	      buffer->strings[file] = (char) 0;
	      if(pcb->uarea)
		zzget_filename(pcb->uarea, &buffer->line,
			       &buffer->strings[file]);
	    } /* Non-paging */
	  
	  /*
	  ** Shrink the buffer to proper size and set the buffer endmark.
	  ** The wanted array is placed last in the strings array. 
	  */
#if ERROR_CHECKS > 0
	  ((U8 *) buffer->strings)[strings_endmark_index] = (U8) END_MARK;
#endif
	  /* Set the new size. */
	  sig_to_adm(buffer)->size = (OSBUFSIZE)\
	    (&buffer->strings[strings_endmark_index] - (char *) buffer);

	  /*
	  ** Get cpu registers.
	  ** We do this in two phases. First we capture the current registers
	  ** in a binary format. Then we translate them to a string after
	  ** unlocking the dispatcher.
	  **
	  ** We allow the old implementation where all work is done in
	  ** zzget_regs() if USE_CAPTURE_REGS is zero.
	  ** See also: pose.con.
	  **
	  ** Current (running) processes return an empty register string,
	  ** as these registers are not saved in the uarea.
	  */
	  buffer->strings[cpuregs] = (char) 0;

	  /* The uarea is not accessable for a process under construction. */
	  if(pcb->uarea)
	    {
#if USE_CAPTURE_REGS
	      if(local_sysctlblk.table_extended
		 && local_sysctlblk.enable_get_pcb_regs
		 && zzcur_p_x(pcb->cpu_id) != pcb)
		{
		  /* Read the binary register values into strings[cpuregs]. */
		  zzcapture_regs(pcb->uarea, &buffer->strings[cpuregs]);
		  /* Allow other applications in for a while. */
		  UNLOCK_SWAP();
		  /* Reformat the registers to an ascii string. */
		  zzget_regs(pcb->uarea, &buffer->strings[cpuregs]);
		}
	      else
		{
		  /* Allow other applications in for a while. */
		  UNLOCK_SWAP();
		  /* Insert an empty string. */
		  buffer->strings[cpuregs] = (char) 0;
		}
#else
	      if(local_sysctlblk.table_extended
		 && local_sysctlblk.enable_get_pcb_regs
		 && zzcur_p_x(pcb->cpu_id) != pcb)
		{
		  /* The old policy: read and reformat in one operation. */
		  zzget_regs(pcb->uarea, &buffer->strings[cpuregs]);
		  /* Allow other applications in for a while. */
		  UNLOCK_SWAP();
		}
	      else
		{
		  /* Allow other applications in for a while. */
		  UNLOCK_SWAP();
		  /* Insert an empty string. */
		  buffer->strings[cpuregs] = (char) 0;
		}
#endif
	    } /* uarea exists */
	  else
	    {
	      /* Allow other applications in for a while. */
	      UNLOCK_SWAP();
	    }
	} /* PRI or BG */
      else if(type == OS_PHANTOM)
        {
	  /* Get the first signal queue pointer. */
	  buffer->sigqueue = (OSADDRESS) pcb->mailbox;

	  /* Allow other applications in for a while. */
	  UNLOCK_SWAP();
	}
      else
	{
	  /* Allow other applications in for a while. */
	  UNLOCK_SWAP();
	}
      
      /* Set up all pointers into the strings field. */
      buffer->file = file;
      buffer->name = name;
      buffer->cpuregs = cpuregs;
      buffer->wanted = wanted;
      
      /* Lock the dispatcher again. */
      LOCK_SWAP();

      /* Check that we still have the checked process. */
      if(type != get_ptype(pid))
	{
	  buffer->type = OS_ZOOMBIE;
	  UNLOCK_SWAP();
	  return (struct OS_pcb *) sig_dup((void *) buffer);
	}

      /* Gather less unstable process or block information. */
      if(type == OS_BLOCK)
	{
	  /* Pick up the remote call server. */
	  if(bcb->use_remcalls)
	    buffer->remote_server = bcb->remcall_server;

	  /* Set the attach_list if there is anything attached to the block. */
	  buffer->attach_list = (bcb->attached ? (OSADDRESS)bcb->attached : 0);

	  /* Get the user number. */
	  buffer->user = bcb->user;
	  
	  /* Get a copy of the block name. */
	  strcpy(&buffer->strings[name], (const char *) bcb->name);
	  
	  /* Get the largest buffer size available to the block. */
	  buffer->max_sigsize = bcb->sigpool->signal_list[7].size;
	}
      else
	{
	  /* Pick up the remote call server. */
	  if(pcb->block->use_remcalls)
	     buffer->remote_server = pcb->block->remcall_server;

	  /* Find out if there is a signal attached to the process. */
	  {
	    struct _attach_entry *sigref = pcb->attached;
	    for(; sigref; sigref = sigref->own_fq)
	      if(pid == sigref->victim)
		{
		  buffer->attach_list = (OSADDRESS)sigref;
		  break;
		}
	  }

	  /* Get the user number. */
	  buffer->user = pcb->real_user;
	  
	  /* Get a copy of the process name. */
          if((pcb->block->lnh_name) && (zzcur_p->block != pcb->block))
            {
              strcpy(&buffer->strings[name], (const char *) pcb->block->lnh_name);
              strcpy(&buffer->strings[name + strlen(pcb->block->lnh_name)], (const char *) pcb->name);
            }
          else
            {
              strcpy(&buffer->strings[name], (const char *) pcb->name);
            }          
	  
	  /* Get the largest buffer size available to the process. */
	  buffer->max_sigsize = pcb->block->sigpool->signal_list[7].size;
	  
	  /* Get the process priority for all process types. */
	  buffer->priority = pcb->priority & 31;
	  
	  if(type != OS_PHANTOM)
	    {
	      /* Count number of signals owned by any process. */
	      buffer->sig_cnt_owned = pcb->sig_cnt_owned;

	      /* Get the current fast semaphore value. */
	      buffer->fsemvalue = pcb->fsemval;
	    }
	}

#ifdef OSE_SMP
      if((type == OS_INT_PROC) || (type == OS_TI_PROC))
	{
	  /* Get the current stack limits. */
	  buffer->stack_top =
	    (OSADDRESS)zzcpu[pcb->cpu_id].interrupt_stack->top - 1;
	  buffer->stack_limit =
	    (OSADDRESS)zzcpu[pcb->cpu_id].interrupt_stack->limit;
	}
#endif
      if((type == OS_PRI_PROC) || (type == OS_BG_PROC) || (type == OS_PHANTOM))
	{
	  if (type != OS_PHANTOM)
	    {
	      /* Get the current stack limits. */
	      buffer->stack_top = (OSADDRESS) pcb->stack->top - 1;
	      buffer->stack_limit = (OSADDRESS) pcb->stack->limit;
	    }
	  
	  /*
	  ** Get number of signals in the queue. Make it consistent with
	  ** the sigqueue member.
	  */
	  if(buffer->sigqueue)
	    {
	      buffer->sig_cnt_in_q = pcb->sig_cnt_queued;
	      if(buffer->sig_cnt_in_q == 0)
		{
		  buffer->sigqueue = 0;
		}
	    }
	}

      /* Make a copy of the built buffer and return it to the caller. */
      UNLOCK_SWAP();
      return (struct OS_pcb *) sig_dup((void *) buffer);
    }
}

/*
** This is the set_pcb system call.
**
** Sets the CPU registers of the specified process if the process
** is in a state where this is possible.
**
** Cpuregs contains a string describing the registers to be set.
** The string format corresponds to the format used in get_pcb, except
** registers may be in any order and not all registers need be present.
**
** Returns zero if the call was successful.
** Returns a nonzero value if the call failed because the process
** is not in a state where the CPU registers can be set.
**
*/

OSBOOLEAN
set_pcb(PROCESS pid, const char *cpuregs)
{
  register struct _pcb *pcb;
  register OSBOOLEAN result;
#if USE_CAPTURE_REGS
  char *binregs;
#endif

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_SET_PCB))
    return (OSBOOLEAN) 1;
#endif

  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid(pid, OSE_SET_PCB);
  if(pcb == NULL)
    {
      return (OSBOOLEAN) 1;
    }

#if USE_CAPTURE_REGS
  /* Allocate a side buffer for the binary register image. */
  binregs = (char *) alloc(MAX_REGLEN, 0);
  /* Reformat the string to a binary image. */
  result = zzunpack_regs(binregs, cpuregs);
#endif

  /* Locate the process descriptor of the flushed process. */
  pcb = pid_to_pcb(pid);

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      UNLOCK_SWAP();
#if USE_CAPTURE_REGS
      /* Free the  side buffer for the binary register image. */
      free_buf_type(char *, &binregs);
#endif
      return (OSBOOLEAN) 1;
    }

#if USE_REMOTE_CALLS
  /* Issue a remote system call if required. */
  /* Many remote calls can not occur for blocks. Trim off the tests for MAGIC_PCB ??*/
  if(local_sysctlblk.enable_remcalls)
  {
    union SIGNAL *sig;
    register OSBOOLEAN use_remcalls;
    register PROCESS handler;
    if(pcb->magic == MAGIC_PCB)
      {
	use_remcalls = pcb->block->use_remcalls;
	handler = pcb->block->remcall_server;
      }
    else
      {
	use_remcalls = ((struct _block *)pcb)->use_remcalls;
	handler = ((struct _block *)pcb)->remcall_server;
      }
    if(use_remcalls)
      {
	UNLOCK_SWAP();
	set_suid((OSBOOLEAN) 1);
	sig = alloc(sizeof(struct remote_call) + strlen(cpuregs), REMOTE_CALL);
	sig->remote_call.call_type = REM_SET_PCB;
	sig->remote_call.params.rem_set_pcb.pid = pid;
	strcpy(sig->remote_call.params.rem_set_pcb.cpuregs, (const char *) cpuregs);
	
	sig = make_remote_call(sig, (OSCALLTYPE) REMR_SET_PCB, handler);
#if USE_CAPTURE_REGS
	/* Free the  side buffer for the binary register image. */
	free_buf((union SIGNAL **) &binregs);
#endif
	if(sig)
	  {
	    set_suid((OSBOOLEAN) 0);
	    result = sig->remote_response.results.remr_set_pcb.failed;
	    free_buf(&sig);
	    return result;
	  }
	else
	  {
	    set_suid((OSBOOLEAN) 0);
	    return (OSBOOLEAN) 1;
	  }
      }
  }
#endif

  /* Ignore attempts to set registers for non interrupt processes. */
  if(pcb->priority > 32)
    {
      UNLOCK_SWAP();
#if USE_CAPTURE_REGS
      /* Free the  side buffer for the binary register image. */
      free_buf_type(char *, &binregs);
#endif
      return (OSBOOLEAN) 1;
    }

  /* Call the target interface. */
#if USE_CAPTURE_REGS
  if(!result)
    result = zzset_regs(pcb->uarea, binregs);
#else
  result = zzset_regs(pcb->uarea, cpuregs);
#endif

  /* Unlock the dispatcher. */
  UNLOCK_SWAP();

#if USE_CAPTURE_REGS
  /* Free the  side buffer for the binary register image. */
  free_buf_type(char *, &binregs);
#endif

  /* Return the result provided by the target interface. */
  return result;
}

/*
** This is the get_pid_info system call.
**
** Note that we currently only implement the first (mandatory) info
** tags and that the definition of the rest may change when they are
** finaly implemented. @frbr@
**
** Prototype for user-mode and loadmodule clients.
** 
**    OSADDRESS
**    get_pid_info(PROCESS pid, OSADDRESS info_tag);
**
**    This is is implemented as get_sysconf(72, pid, info_tag, 0).
**
** Faster prototype only available to supervisor-mode clients linked
** to the kernel monolith (defined in pose.h):
**    
**    OSADDRESS
**    get_pid_info(PROCESS pid, OSADDRESS info_tag)
** 
** New struct:
** 
**     struct stack_info
**     {
**        OSADDRESS base;
**        OSADDRESS size;
**        OSADDRESS sp;     // Zero if not available.
**        OSADDRESS s_base; // Zero for supervisor-mode (and int) procs.
**        OSADDRESS s_size; // Zero for supervisor-mode (and int) procs.
**        OSADDRESS s_sp;   // Zero if not available.
**     };
** 
** Supported tags and their return values:
** 
**                         N/A = PID Argument is ignored.
**                         P   = Process
**                         B   = Block
**                         O   = Pool
**                         S   = Segment
**                         |
**                         V
** 
** NUM INFO_TAG            PID   RETURNS INFO
** --------------------------------------------------------------------------
**  0  info_tags           N/A   Tag number of last supported info_tag.
**  1  info_tag_supported  (tag) tag => 1 (supported) ? 0 (not supported)
**  2  pid_index_mask      N/A   0x0000fffful;
**  3  pid_instance_mask   N/A   0xffff0000ul;
**  4  max_proc            N/A   (OSADDRESS)zzuu.max_proc;
**  5  next_free_pix       N/A   (OSADDRESS)zzuu.next_free_pix;
**  6  pid_at_index        N/A   (PROCESS) pcb_to_pid(pid_to_pcb(index))
**  7  pcb                 PBOS  struct _pcb *pcb;
**  8  k_uarea             P---  void *uarea; (Kernel/Target-interface)
**  9  pcb_type            PBOS  ptype + pool & segment.
** 10  supervisor_mode     PB--  OSBOOLEAN supervisor_mode
** 11  cur_ni              N/A   PROCESS zzcur_ni[cpu]
** 12  last_cur_ni         N/A   PROCESS zzww.last_cur_ni[cpu]
** 13  cpu_id              PB--  (cpid_t) cpu_id (pid zero for current process)
** 14  name                PB--  const char *name; (Raw pointer to string)
** 15  name_buf            PB--  alloced(const char *name); Respects instances.
** 16  segnum              PBOS  OSSEGMENT segnum;
** 17  cur_pid             N/A   PROCESS cur_p[cpu]
** 18  sig_cnt_in_queue    P---  (OSADDRESS) (count mailbox len)
** 19  redir_buf           P---  alloced(struct OS_redir_entry *redir);
** 20  manager_mode        P---  OSBOOLEAN manager_mode
** 21  lock_cnt/_killed    P---  OSADDRESS (lock_cnt << 1) | (locked_kill & 1)
** 22  new_pid             N/A   PROCESS pid under construction
** 23  instance_len        P---  Length of instance prefix in process name.
** 24  tmo_active          P---  OSBOOLEAN tmo_active
** 25  current_size        --O-  OSADDRESS Current Pool size
** 26  max_size            --O-  OSADDRESS Maximum Pool size
** 27  fragment_size       --O-  OSADDRESS Pool extend fragment size
** 28  main_block          PBOS  PROCESS main_pid; (PID of mainblock)
** 29  entrypoint          P---  void (*entrypoint)(void);
** 30  vector              P---  OSVECTOR vector;
** 31  is_an_sport         P---  OSBOOLEAN is_an_sport;
** 32  is_deaf             P---  OSBOOLEAN is_deaf;
** 33  err_handl_level     P---  OSBOOLEAN err_handl_level;
** 
**  -  The following tags are not implemented and may therefore be changed -
** 
**     first_proc          PB--  (PROCESS) (First process in main block)
**     parent_pid          PBOS  PROCESS parentpid;
**     swap_in_cnt         P---  (OSADDRESS) (Swap in counter)
**     stack               P---  struct stack_info *stack;
**     error_handler       PB--  OSERRH *error_handler;
**     tmo                 P---  OSTIME tmo;
**     u_uarea             P---  void *uarea; (User defined)
**     is_main_block       P---  OSBOOLEAN is_main_block;
**     shared_mode         PB--  OSBOOLEAN shared_mode;
*/
 
OSADDRESS
get_pid_info(PROCESS pid, OSADDRESS info_tag)
{
  LOCKSTATE msr;
  struct _pcb *pcb;

  switch (info_tag)
    {
    case 0:
      /* Tag number of last supported info_tag. */
      return (OSADDRESS)33;
    case 1:
      /* Return 1 if tag is supported and 0 if tag is unsupported. */
      return (OSADDRESS)(pid <= 33 ? 1 : 0);
    case 2:
      /* pid_index_mask */
      return (OSADDRESS)PCB_INDEX_MASK;
    case 3:
      /* pid_instance_mask */
      return (OSADDRESS)(((PROCESS)~0u) ^ (PROCESS)(PCB_INDEX_MASK));
    case 4:
      return (OSADDRESS)zzuu.max_proc;
    case 5:
      return (OSADDRESS)zzuu.next_free_pix;
    case 6:
      {
	/* pid_at_index */
	PCB_INDEX_T pindex = pid_to_pindex(pid);
	
	return (pindex < zzuu.next_free_pix)
	  ? pcb_to_pid(zzuu.pix_list[pindex])
	  : (OSADDRESS)0ul;
      }
    case 7:
      {
	/* Return pointer to Process Control Block at index. */
	PCB_INDEX_T pindex = pid_to_pindex(pid);
	
	return (pindex < zzuu.next_free_pix)
	  ? (OSADDRESS)zzuu.pix_list[pindex]
	  : (OSADDRESS)0ul;
      }
    case 8:
      {
	/* Return the target-interface's uarea. */
	OSADDRESS uarea = 0ul;
	
	/* Disable interrupts. */
	BKL_LOCK_SAVE(msr);
	
	pcb = safe_pid_to_pcb(pid, 0);
	if (pcb != NULL)
	  uarea = (OSADDRESS)(pcb->uarea);

	/* Enable interrupts. */
	BKL_LOCK_RESTORE(msr);

	return uarea;
      }
    case 9:
      {
	/*
	** Returns the same as get_ptype() but can also return OS_POOL
	** and OS_SEGMENT.
	*/
	OSADDRESS type = OS_ILLEGAL;

	if(pid_to_instance(pid) == 0)
	  return OS_ILLEGAL;

	/* Disable interrupts. */
	BKL_LOCK_SAVE(msr);
	
	pcb = safe_pid_to_pcb(pid, 2);
	if (pcb != NULL)
	  {
	    if (!(pcb->magic & MAGIC_KILLED))
	      {
		switch (pcb->magic)
		  {
		  case MAGIC_PCB:
		    {
		      OSPRIORITY priority = pcb->priority;
		      
		      /* Extract process type from process priority. */
		      if(priority < 32)
			type = OS_PRI_PROC;
		      else if(priority < 33)
			type = OS_BG_PROC;
		      else if(priority < 96)
			type = OS_INT_PROC;
		      else if(priority < 128)
			type = OS_TI_PROC;
		      else
			type = OS_PHANTOM;
		    }
		    break;
		  case MAGIC_BCB:
		    type = OS_BLOCK;
		    break;
		  case MAGIC_POOL:
		    type = (OSADDRESS)OS_ILLEGAL + 1u;  /* 4097 */
		    break;
		  case MAGIC_SEG:
		    type = (OSADDRESS)OS_ILLEGAL + 2u;  /* 4098 */
		    break;
		  default:;
		  }
	      }
	    else
	      {
		type = OS_ZOOMBIE;
	      }
	    if(type != OS_ILLEGAL)
	      {
		if (pcb_to_pid(pcb) != pid)
		  type = OS_ZOOMBIE;
	      }
	  }
	
	/* Enable interrupts. */
	BKL_LOCK_RESTORE(msr);

	return type;
      }
  
    case 10:
      /* OSBOOLEAN supervisor_mode */
      {
	OSBOOLEAN supervisor_mode;

	if(pid == 0)
	  return (OSADDRESS)zzcur_p->block->supervisor_mode;
	
	supervisor_mode = 0;
	   
	/* Disable interrupts. */
	BKL_LOCK_SAVE(msr);
	
	pcb = safe_pid_to_pcb(pid, 1);
	if (pcb != NULL)
	  {
	    switch (pcb->magic)
	      {
	      case MAGIC_PCB:
		supervisor_mode = pcb->block->supervisor_mode;
		break;
	      case MAGIC_BCB:
		supervisor_mode = ((struct _block *)pcb)->supervisor_mode;
		break;
	      default:
		;
	      }
	  }
	else if(pid == zzuu.process_under_construction_)
	  supervisor_mode = pid_to_pcb(pid)->block->supervisor_mode;

	/* Enable interrupts. */
	BKL_LOCK_RESTORE(msr);

	return (OSADDRESS)supervisor_mode;
      }

    case 11:
      /* PROCESS zzcur_ni[cpu] */
      {
#ifdef OSE_SMP
	PROCESS cur_ni = any_killed_pid();

	if(pid < zzkrn_num_cpus())
	  {
	    struct per_cpu *cpu = &zzcpu[pid];

	    BKL_LOCK_SAVE(msr);
	    
	    if(cpu->state == CPU_STATE_ACTIVE)
	      cur_ni = cpu->curr_ni->pid;
	    
	    BKL_LOCK_RESTORE(msr);
	  }
	return (OSADDRESS)cur_ni;
#else
	return (OSADDRESS) (pid == 0
			    ? zzcur_ni->pid
			    : any_killed_pid());
#endif
      }

    case 12:
      /* PROCESS zzww.last_cur_ni[cpu] */
      {
	PROCESS last = any_killed_pid();
	
#ifdef OSE_SMP
	if(pid < zzkrn_num_cpus())
	  {
	    struct per_cpu *cpu = &zzcpu[pid];
	    
	    BKL_LOCK_SAVE(msr);
	    if(cpu->state == CPU_STATE_ACTIVE &&
	       cpu->last_cur_ni->magic == MAGIC_PCB)
	      last = cpu->last_cur_ni->pid;
	    BKL_LOCK_RESTORE(msr);
	  }
#else
	BKL_LOCK_SAVE(msr);
	if(zzuu.last_cur_ni->magic == MAGIC_PCB)
	  last = zzuu.last_cur_ni->pid;
	BKL_LOCK_RESTORE(msr);
#endif
	return (OSADDRESS)last;
      }
      
    case 13:
      /* cpu_id of process/block */
      {
	cpuid_t cpu_id;

	if(pid == 0)
	  {
	    return (OSADDRESS)zzcur_p->cpu_id;
	  }
      
	cpu_id = ~0u;

	/* Disable interrupts. */
	BKL_LOCK_SAVE(msr);
	
	pcb = safe_pid_to_pcb(pid, 1);
	if (pcb != NULL)
	  {
	    switch (pcb->magic)
	      {
	      case MAGIC_PCB:
		cpu_id = pcb->cpu_id;
		break;
	      case MAGIC_BCB:
		cpu_id = ((struct _block *)pcb)->cpu_id;
		break;
	      default:
		;
	      }
	  }
	else if(pid == zzuu.process_under_construction_)
	  cpu_id = pid_to_pcb(pid)->cpu_id;

	/* Enable interrupts. */
	BKL_LOCK_RESTORE(msr);

	return (OSADDRESS)cpu_id;
      } 

    case 14:
      /* Name pointer of process/block */
      {
	const char *name = NULL;

	/* Disable interrupts. */
	BKL_LOCK_SAVE(msr);
	
	pcb = safe_pid_to_pcb(pid, 1);
	if (pcb != NULL)
	  {
	    switch (pcb->magic)
	      {
	      case MAGIC_PCB:
		name = pcb->name;
		break;
	      case MAGIC_BCB:
		name = ((struct _block *)pcb)->name;
		break;
	      default:
		;
	      }
	  }
	else if(pid && pid == zzuu.process_under_construction_)
	  name = pid_to_pcb(pid)->name;

	/* Enable interrupts. */
	BKL_LOCK_RESTORE(msr);

	return (OSADDRESS)name;
      } 

    case 15:
      /* Name buffer copy of process/block. Prefixes instance name of block if
       * applicable. */
      {
	OSPRIORITY priority = zzcur_p->priority;
	const char *name = NULL;
        const char *instance = NULL;
	char *buf = NULL;

	if(priority <= 32)
	  LOCK_SWAP();

	pcb = safe_pid_to_pcb(pid, 1);
	if (pcb != NULL)
	  {
	    switch (pcb->magic)
	      {
	      case MAGIC_PCB:
                /* Instance prefixes are for foreign blocks only. */
                if (zzcur_p->block != pcb->block)
                  instance = pcb->block->lnh_name;
		name = pcb->name;
		break;
	      case MAGIC_BCB:
		name = ((struct _block *)pcb)->name;
		break;
	      default:
		;
	      }
	  }
	else if(pid && pid == zzuu.process_under_construction_)
	  name = pid_to_pcb(pid)->name;
	if(name)
	  {
            if(instance)
              {
                /* The instance name contains a trailing slash. */
                size_t instance_len = strlen(instance);
                buf = (char *)alloc(strlen(name) + sizeof(SIGSELECT) +
                                    instance_len, 0);
                strcpy(buf, instance);
                strcpy(&buf[instance_len], name);
              }
            else
              {                    
                buf = (char *)alloc(strlen(name) + sizeof(SIGSELECT), 0);
                strcpy(buf, name);
              }                
          }

	if(priority <= 32)
	  UNLOCK_SWAP();
	return (OSADDRESS)buf;
      }

      case 16:
      {
	/* OSSEGMENT segnum */
	OSSEGMENT segid = ~0u;

	/* The system segment is segment nuber zero. */
	if(pid == 0)
	  return 0;

	/* Disable interrupts. */
	BKL_LOCK_SAVE(msr);

	pcb = safe_pid_to_pcb(pid, 1);
	if (pcb != NULL)
	  {
	    switch (pcb->magic)
	      {
	      case MAGIC_PCB:
		segid = pcb->segid;
		break;
	      case MAGIC_BCB:
		segid = ((struct _block*) pcb)->segid;
		break;
	      case MAGIC_SEG:
		segid = ((struct segcb*) pcb)->segid;
		break;
	      case MAGIC_POOL:
		segid = ((struct poolcb*) pcb)->segid;
		break;
	      default:
		;
	      }
	  }
	else if(pid == zzuu.process_under_construction_)
	  segid = pid_to_pcb(pid)->segid;

	/* Enable interrupts. */
	BKL_LOCK_RESTORE(msr);

	return (OSSEGMENT)segid;
      }

      case 17:
      {
	/* PROCESS cur_p[cpu] */
#ifdef OSE_SMP
	PROCESS cur_p = any_killed_pid();

	if(pid < zzkrn_num_cpus())
	  {
	    BKL_LOCK_SAVE(msr);
	    
	    if(zzcpu[pid].state == CPU_STATE_ACTIVE)
	      cur_p = zzcpu[pid].curr_p->pid;
	    
	    BKL_LOCK_RESTORE(msr);
	  }
	return (OSADDRESS)cur_p;
#else
	return (OSADDRESS) (pid == 0
			    ? zzcur_p->pid
			    : any_killed_pid());
#endif
      }

    case 18:
      {
	/* OSADDRESS sig_cnt_in_queue. */
	OSADDRESS in_q = 0ul;
	
	pcb = safe_pid_to_pcb(pid, 0);
	if (pcb != NULL)
	  in_q = pcb->sig_cnt_queued;

	if(pcb_to_pid((volatile struct _pcb *)pcb) != pid)
	  in_q = 0;

	return in_q;
      }

    case 19:
      /* get_redirection(PROCESS pid) */
      {
	struct OS_redir_entry *redir_copy = NULL;

      retry_get_redirection:
	/* Disable interrupts. */
	BKL_LOCK_SAVE(msr);

	pcb = (pid && pid == zzuu.process_under_construction_)
	  ? pid_to_pcb(pid)
	  : safe_pid_to_pcb(pid, 0);

	if (pcb != NULL)
	  {
	    struct OS_redir_entry *redir = pcb->redir;

	    if(redir)
	      {
		OSBUFSIZE size = sig_to_adm(redir)->size;

		/* Enable interrupts. */
		BKL_LOCK_RESTORE(msr);

		redir_copy = (struct OS_redir_entry *)alloc(size, 0);

		/* Disable interrupts. */
		BKL_LOCK_SAVE(msr);

		/*
		** Check if the process is still alive and that it has
		** not changed it's redirection table by using the
		** set_redirection system call.
		*/
		if (pid == pcb_to_pid(pcb) &&
		    redir == pcb->redir)
		  {
		    memcpy(redir_copy, redir, size);
		  }
		else
		  {
		    /* Enable interrupts. */
		    BKL_LOCK_RESTORE(msr);
		    free_buf((union SIGNAL **)&redir_copy);
		    goto retry_get_redirection;
		  }
	      }
	  }

	/* Enable interrupts. */
	BKL_LOCK_RESTORE(msr);
	
	return (OSADDRESS)redir_copy;
      }

    case 20:
      {
	/* OSBOOLEAN manager_mode */

	/* Disable interrupts. */
	BKL_LOCK_SAVE(msr);

	pcb = (pid && pid == zzuu.process_under_construction_)
	  ? pid_to_pcb(pid)
	  : safe_pid_to_pcb(pid, 0);

	if (pcb != NULL)
	  {
            OSBOOLEAN state = zzget_manager_mode(pcb->uarea);

	    /* Enable interrupts. */
            BKL_LOCK_RESTORE(msr);
            
            return (OSADDRESS)state;
	  }
	
	/* Enable interrupts. */
	BKL_LOCK_RESTORE(msr);
        
	return (OSADDRESS)~0;
      }

    case 21:
      {
	/* OSADDRESS (lock_cnt << 1) + locked_kill */

	OSADDRESS lock_info;
	
	/* Disable interrupts. */
	BKL_LOCK_SAVE(msr);
	pcb = safe_pid_to_pcb(pid, 0);
        
	lock_info = (pcb != NULL)
	  ? (pcb->lock_cnt << 1) | (pcb->locked_kill & 1)
	  : ~0UL;
	    
	/* Enable interrupts. */
	BKL_LOCK_RESTORE(msr);
        
	return lock_info;
      }
      
    case 22:
      {
	/* PROCESS new_pid */

	return zzuu.process_under_construction_;
      }

    case 23:
      {
        /* Length of instance name prefix for a process. Can be 0. */
	OSPRIORITY priority = zzcur_p->priority;
        OSADDRESS prefix_len = 0;

	if(priority <= 32)
	  LOCK_SWAP();

	pcb = safe_pid_to_pcb(pid, 1);

        /* Only consider living processes. */
	if((pcb != NULL) && (pcb->magic == MAGIC_PCB))
          {
            /* Processes outside our block with a block link have a prefix. */
            if((pcb->block->lnh_name) && (zzcur_p->block != pcb->block))
              {
                prefix_len = strlen(pcb->block->lnh_name);
              }
          }

	if(priority <= 32)
	  UNLOCK_SWAP();
        
        return prefix_len;
      }

    case 24:
      {
        /* OSBOOLEAN tmo_active */
        
        OSBOOLEAN tmo_active;
        
	BKL_LOCK_SAVE(msr);
	pcb = safe_pid_to_pcb(pid, 0);
        tmo_active = (OSBOOLEAN)((pcb != NULL)
				 ? pcb->tmo_active
				 : 0);

        BKL_LOCK_RESTORE(msr);
        return tmo_active;
      }

    case 25:
    case 26:
    case 27:
      {
	/* Pool size, pool max size, pool extend fragment size */
	OSADDRESS value = 0;

	/* Disable interrupts. */
	BKL_LOCK_SAVE(msr);

	pcb = safe_pid_to_pcb(pid, 1);
	if (pcb != NULL)
	  {
	    struct poolcb *poolcb = (struct poolcb *)pcb;
	    if(poolcb->magic == MAGIC_POOL)
	      {
		switch(info_tag)
		  {
		  case 25:
		    value = poolcb->tot_size;  /* Current pool size */
		    break;

		  case 26:
		    value = poolcb->max_size;  /* Current max size */
		    break;

		  case 27:
		    /* When is the fragment size known by the kernel?
		    ** Really only for local user mode pools.
		    ** Main and system pool fragments are defined by the
		    ** extender, provided either by OSE or the customer.
		    */
		    if(poolcb->realtime_extendable)
		      {
			struct pool_extension *cur_frag = poolcb->frag_list;
			if(cur_frag->frag_list)
			  {
			    /* This pool has been extended at least once. */
			    value = cur_frag->limit - cur_frag->base;
			  }
		      }
		    else
		      {
			/* A local non-realtime extender registered. */
			value = poolcb->frag_size;
		      }
		    break;
		    
		  default:
		    /*NOTREACHED*/
		    break;
		  }
	      }
	  }

	/* Enable interrupts. */
	BKL_LOCK_RESTORE(msr);

	return value;
      }

    case 28:
      /* PROCESS main_block */
      {
	PROCESS main_block = any_killed_pid();
	struct segcb *seg;
	
	if(pid == 0)
	  return (OSADDRESS)system_segment.main_block;
	   
	/* Disable interrupts. */
	BKL_LOCK_SAVE(msr);
	
	pcb = safe_pid_to_pcb(pid, 1);
	if (pcb != NULL)
	  {
	    seg = find_segcb(pcb);
	  }
	else if(pid == zzuu.process_under_construction_)
	  {
	    seg = pid_to_segcb(pid_to_pcb(pid)->block->segpid);
	  }
	else
	  {
	    seg = NULL;
	  }

	if(seg != NULL)
	  {
	    main_block = seg->main_block;
	  }

	/* Enable interrupts. */
	BKL_LOCK_RESTORE(msr);

	return (OSADDRESS)main_block;
      }

    case 29:
    case 30:
    case 31:
    case 32:
    case 33:
      {
	/* Some simple PCB attributes. */
	OSADDRESS attribute = 0ul;
	
	pcb = safe_pid_to_pcb(pid, 0);
	if (pcb != NULL)
	  {
	    switch(info_tag)
	      {
	      case 29:
		attribute = (OSADDRESS)pcb->entrypoint;
		break;
	      
	      case 30:
		attribute = (OSADDRESS)pcb->vector;
		break;
	      
	      case 31:
		attribute = (OSADDRESS)pcb->is_an_sport;
		break;
	      
	      case 32:
		attribute = (OSADDRESS)pcb->is_deaf;
		break;
	      
	      case 33:
		attribute = (OSADDRESS)pcb->err_handl_level;
		break;
	      
	      default: 
		break;
	      } /* switch */

	    if(pcb_to_pid((volatile struct _pcb *)pcb) != pid)
	      {
		attribute = 0;
	      }
	  }	
	return attribute;
      }

    default:
      return 0ul;
    }
}

/* This is the get_sysconf entry for get_pid_info(). */
static OSADDRESS
sysconf_72(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void)r;

  return get_pid_info((PROCESS)p, q);
}

/*
** This is a function that can be used for implementing a polled CPU
** profiler that shows the priority and type of the currently
** executing process.
*/

void
ose_get_curr_procs(struct OS_curp *out, cpuid_t cpus)
{
  out = &out[cpus - 1];
  while(cpus)
    {
      SMP_CODE_YES_NO(struct per_cpu *cpu = &zzcpu[--cpus], --cpus);
      
      if(SMP_CODE_YES_NO(cpu->state == CPU_STATE_ACTIVE, cpus == 0))
	{
	  OSPRIORITY priority;

#ifdef OSE_SMP
	  PROCESS pid;

	  do
	    {
	      pid = cpu->curr_p->pid;
	      priority = cpu->curr_p->priority;
	    }
	  while(cpu->curr_p->pid != pid);
	  out->pid = cpu->curr_p->pid;
#else
	  priority = zzcur_p->priority;
	  out->pid = zzcur_p->pid;
#endif
	  /* Extract process type from process priority. */
	  if(priority < 32)
	    out->type = OS_PRI_PROC;
	  else if(priority < 33)
	    out->type = OS_BG_PROC;
	  else if(priority < 96)
	    out->type = OS_INT_PROC;
	  else if(priority < 128)
	    out->type = OS_TI_PROC;
	  else
	    out->type = OS_ILLEGAL;

	  out->pri = priority & 31;
	}
      else
	{
	  out->pid  = any_killed_pid();
	  out->type = OS_ILLEGAL; 
	  out->pri  = 0;
	}
      out->_reserved = 0xa05e05ea; /* For future use. */
      --out;
    }
}

/*
** This is the get_signal system call.
**
** There is a bug here similar to the one in get_pcb().??
** We access the data part of a signal in another process
** while the dispatcher is locked.
*/

OSBOOLEAN
get_signal(PROCESS pid,
	   OSADDRESS mailptr,
	   union SIGNAL **sig_copy,
	   OSADDRESS *nextsig)
{
  register struct _pcb *pcb;
  union SIGNAL *sig_r;
  register struct _signal *sigadm;
  struct _signal *newadm;
  struct _signal *next;
  PROCESS sender_id;
  PROCESS dest_id;

  /* Set output variables to safe values in case of error. */
  *sig_copy = NIL;
  *nextsig = (OSADDRESS) 0;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_GET_SIGNAL))
    return (OSBOOLEAN) 1;
#endif

  /* Read instance number and check the process id. Report error if invalid. */
  pcb = check_pid_bid(pid, OSE_GET_SIGNAL);
  if(pcb == NULL)
    {
    return (OSBOOLEAN) 1;
    }

  /* Locate the process descriptor. */
  pcb = pid_to_pcb(pid);

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the block/process is still present. */
  if(pcb_to_pid(pcb) != pid)
    {
      UNLOCK_SWAP();
      return (OSBOOLEAN) 1;
    }

#if USE_REMOTE_CALLS
  /* Issue a remote system call if required. */
  if(local_sysctlblk.enable_remcalls)
  {
    union SIGNAL *sig;
    register OSBOOLEAN use_remcalls;
    register PROCESS handler;
    OSBUFSIZE sig_size;
    if(pcb->magic == MAGIC_PCB)
      {
	use_remcalls = pcb->block->use_remcalls;
	handler = pcb->block->remcall_server;
      }
    else
      {
	use_remcalls = ((struct _block *)pcb)->use_remcalls;
	handler = ((struct _block *)pcb)->remcall_server;
      }
    if(use_remcalls)
      {
	UNLOCK_SWAP();
	set_suid((OSBOOLEAN) 1);
	sig = alloc(sizeof(struct remote_call), REMOTE_CALL);
	sig->remote_call.call_type = REM_GET_SIGNAL;
	sig->remote_call.params.rem_get_signal.pid = pid;
	sig->remote_call.params.rem_get_signal.mailptr = mailptr;
	
	sig = make_remote_call(sig, (OSCALLTYPE) REMR_GET_SIGNAL, handler);
	set_suid((OSBOOLEAN) 0);
	if(sig)
	  {
	    sig_size = sig->remote_response.results.remr_get_signal.sigsize;
	    if(sig_size)
	      {
		sig_r = alloc(sig_size, 0);
		sigadm = sig_to_adm(sig_r); /*@@ maybe check_sig here. */

		sigadm->sender_id = sig->remote_response.results.remr_get_signal.sender;
		sigadm->dest_id = sig->remote_response.results.remr_get_signal.addressee;
		memcpy((void *) sig_r,
		       (void *) sig->remote_response.results.remr_get_signal.contents,
		       (size_t) sig_size);
		*sig_copy = sig_r;
		*nextsig = sig->remote_response.results.remr_get_signal.nextsig;
		free_buf(&sig);
		return (OSBOOLEAN) 0;
	      }
	    else
	      {
		free_buf(&sig);
		return (OSBOOLEAN) 1;
	      }
	  }
	return (OSBOOLEAN) 1;
      }
  }
#endif

  /* Is it a process? */
  if (pcb->magic == MAGIC_PCB)
    {
      /* Search for the specified buffer. */
      for(sigadm = pcb->mailbox; sigadm; sigadm = sigadm->next)
	{
	  if(sigadm == (struct _signal *) mailptr)
	    {
	      break;
	    }
	}
    }
  else
    sigadm = NULL;

  /*
  ** If the mailptr handle didn't match any signal in the queue we 
  ** will continue to search for it in the attach lists.
  */
  if(!sigadm)
    {
      /* The dispatcher will be unlocked by get_attached_signal(). */
      return get_attached_signal(pid, mailptr, sig_copy, nextsig);
    }

  /* Save sender, addressee and next fields from the original signal. */
  dest_id = sigadm->dest_id;
  sender_id = sigadm->sender_id;
  next = sigadm->next;

  /* Allocate a similar buffer in kernel space. */
  set_suid((OSBOOLEAN) 1);
  sig_r = locked_alloc(sigadm->size, (SIGSELECT) 0);
  set_suid((OSBOOLEAN) 0);

  /* Copy the data field. This is where the bug is?? */
  memcpy((void *) sig_r, (void *) (adm_to_sig(sigadm)), (size_t) sigadm->size);

  /* Unlock the dispatcher. */
  UNLOCK_SWAP();

  /* Copy the buffer to the callers user pool. */
  sig_r = (union SIGNAL *) sig_dup((void *) sig_r);
  newadm = sig_to_adm(sig_r); /*@@ maybe check_sig here or short inline check. */

  /* Copy sender and addressee fields. */
  newadm->dest_id = dest_id ? dest_id : pid;
  newadm->sender_id = sender_id;

  /* Return the new signal buffer. */
  *sig_copy = sig_r;
  *nextsig = (OSADDRESS) next;
  return (OSBOOLEAN) 0;
}

/*
**===========================================================================
** 23.4  Undocumented system calls.
**===========================================================================
*/

/* struct OS_pid_list *get_segid_list(void) */
static OSADDRESS
sysconf_71(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  OSBUFSIZE count;
  register struct segcb *segcb;
  struct OS_pid_list *buffer;
  PROCESS *pidp;

  (void)p; (void)q; (void)r;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_GET_BID_LIST)) /* new subcode?? */
    return (OSADDRESS)NULL;
#endif

  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Scan global segment list and count the number of members. */
  for(count = 0, segcb = zzuu.segment_list; segcb; segcb = segcb->next)
    {
      count++;
    }

  /* Use the system pool. */
  set_suid(1);

  /* Allocate a return buffer. */
  buffer = (struct OS_pid_list *)
    locked_alloc(sizeof(struct OS_pid_list) + ((count - 1) * sizeof(PROCESS)), 0);

  /* Use the application pool again. */
  set_suid(0);

  /* Copy all segment identities to the new buffer. */
  buffer->count = count;
  pidp = buffer->list;
  for(segcb = zzuu.segment_list; segcb; segcb = segcb->next)
    {
      *pidp++ = pcb_to_pid(segcb);
    }

  /* Return the buffer. */
  UNLOCK_SWAP();
  return (OSADDRESS)(struct OS_pid_list *) sig_dup((union SIGNAL *)buffer);
}

/* Get system limits and statistics */
static OSADDRESS
sysconf_73(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  LOCKSTATE msr;

  (void)r;
  switch(p)
    {
    case 0:			/* Highest implemented function */
      return 5;

    case 1:			/* Attach */
      switch(q)
	{
	case 0:
	  return 3;		/* Highest implemented subfunction. */

	case 1:			/* System limit. */
	  return MAX_ATTACH;
	  
	case 2:			/* Watermark. */
	  return zzuu.next_free_attach;

	case 3:			/* Current usage count. */
	  {
	    OSADDRESS attcb_in_use_count;
	    BKL_LOCK_SAVE(msr);
	    attcb_in_use_count = zzuu.next_free_attach - zzuu.free_attach_count;
	    BKL_LOCK_RESTORE(msr);
	    return attcb_in_use_count;
	  }
	  
	default: 
	  break;
	} /* switch */
      return 0;
      
    case 2:			/* Timeout signals */
      switch(q)
	{
	case 0:
	  return 2;		/* Highest implemented subfunction. */

	case 1:			/* System limit. */
	  return 255*256*TOSCB_ALLOC_CHUNK;
	  
	case 2:			/* Watermark. Indices are decreementing. */
	  return (255*256*TOSCB_ALLOC_CHUNK) - zzuu.next_free_tos_index;
	  
	default: 
	  break;
	} /* switch */
      return 0;
      
    case 3:			/* Mutex */
      switch(q)
	{
	case 0:
	  return 2;		/* Highest implemented subfunction. */

	case 1:			/* System limit. */
	  return 0xffff - 1;
	  
	case 2:			/* Watermark. */
	  return zzuu.next_free_mutex_index;
	  
	default: 
	  break;
	} /* switch */
      return 0;
      
    case 4:			/* SMP */
      switch(q)
	{
	case 0:
	  return 2;		/* Highest implemented subfunction. */

	case 1:			/* Number of cpus. */
	  return (OSADDRESS) zzkrn_num_cpus(); /* Name mangled! */
	  
	case 2:			/* state of cpu number r. */
#ifdef OSE_SMP
	  return (OSADDRESS) (r < zzkrn_num_cpus() ? zzcpu[r].state :
			      CPU_STATE_LIMBO);
#else
	  return (OSADDRESS) (r == 0 ? CPU_STATE_ACTIVE : CPU_STATE_LIMBO);
#endif
	  
	default: 
	  break;
	} /* switch */
      return 0;
      
      
    case 5:			/* Timeout array */
      switch(q)
	{
	case 0:
	  return 2;		/* Highest implemented subfunction. */

	case 1:			/* Max size. */
	  return MAX_TIMEOUT_ARRAY;
	  
	case 2:			/* Configured size. */
	  return local_sysctlblk.tmo_array_length;
	  
	default: 
	  break;
	} /* switch */
      return 0;
      
    default: 
      break;
    } /* switch */
  return 0;  
}

/* Private call to get system configuration etc. */
static OSADDRESS
sysconf_81(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  (void)r;

  switch(p)
    {
    case 0:
      return 1;			/* Highest p. */

    case 1:
      return q ? (OSADDRESS)(sizeof(*zzuu.sysctlblk))
	: (OSADDRESS)zzuu.sysctlblk;
      
    default: 
      return 0;
    } /* switch */
}


/*
*****************************************************************************
** 24 SPLAY TREE MANAGEMENT.
*****************************************************************************
*/

/*
** The splay tree package was adapted from an article in
** the Dec 1992 issue of Dr. Dobbs Journal.
** A number of bugs were fixed and the code was reorganized.
**
** Splay trees are useful when a small number of nodes are frequently
** accessed since the tree tends to concentrate these near the root.
** Splay trees behave badly when the nodes are sequentially accessed since
** doing this transforms the tree into a linear list which causes the worst
** possible performance.
**
** Predictable behaviour can not be expected from a splay tree.
*/

/*
**===========================================================================
** 24.1  Initialization.
**===========================================================================
*/

/*
**===========================================================================
** 24.2  Local functions.
**===========================================================================
*/

/*
** Splay a splay tree.
**
** Input is the node that should be promoted to the root.
*/

static void
splay(treenode *root)
{
  register treenode *t = root;

  /*
  ** t points to the node that should be the new root.
  ** Promote the node to the root, i.e. reorganize the tree until
  ** the parent pointer becomes NULL.
  */

  while(t->splay_p)
    {
      if(t->splay_p->splay_p)
	{
	  if(t->splay_p->splay_p->splay_l == t->splay_p)
	    {
	      if(t->splay_p->splay_l == t)
		/* zig_zig_left(t); */
		{
		  register treenode *p, *pr, *g, *r, *gp;
		  p = t->splay_p;
		  g = p->splay_p;
		  gp = g->splay_p;
		  r = t->splay_r;
		  pr = p->splay_r;
		  p->splay_r = g;
		  p->splay_p = t;
		  g->splay_l = pr;
		  g->splay_p = p;
		  t->splay_p = gp;
		  t->splay_r = p;
		  p->splay_l = r;
		  if(r)
		    r->splay_p = p;
		  if(pr)
		    pr->splay_p = g;
		  if(gp)
		    {
		      if(gp->splay_l == g)
			gp->splay_l = t;
		      else
			gp->splay_r = t;
		    }
		}
	      else
		/* zig_zag_left(t); */
		{
		  register treenode *p, *gp, *ggp, *l, *r;
		  p = t->splay_p;
		  gp = p->splay_p;
		  ggp = gp->splay_p;
		  l = t->splay_l;
		  r = t->splay_r;
		  t->splay_p = ggp;
		  t->splay_l = p;
		  t->splay_r = gp;
		  gp->splay_p = t;
		  p->splay_p = t;
		  p->splay_r = l;
		  gp->splay_l = r;
		  if(l)
		    l->splay_p = p;
		  if(r)
		    r->splay_p = gp;
		  if(ggp)
		    {
		      if(ggp->splay_l == gp)
			ggp->splay_l = t;
		      else
			ggp->splay_r = t;
		    }
		}
	    }
	  else
	    {
	      if(t->splay_p->splay_r == t)
		/* zig_zig_right(t); */
		{
		  register treenode *p, *pl, *g, *l, *gp;
		  p = t->splay_p;
		  g = p->splay_p;
		  gp = g->splay_p;
		  l = t->splay_l;
		  pl = p->splay_l;
		  p->splay_l = g;
		  p->splay_p = t;
		  g->splay_r = pl;
		  g->splay_p = p;
		  t->splay_p = gp;
		  t->splay_l = p;
		  p->splay_r = l;
		  if(l)
		    l->splay_p = p;
		  if(pl)
		    pl->splay_p = g;
		  if(gp)
		    {
		      if(gp->splay_l == g)
			gp->splay_l = t;
		      else
			gp->splay_r = t;
		    }
		}
	      else
		/* zig_zag_right(t); */
		{
		  register treenode *p, *gp, *ggp, *l, *r;
		  p = t->splay_p;
		  gp = p->splay_p;
		  ggp = gp->splay_p;
		  l = t->splay_l;
		  r = t->splay_r;
		  t->splay_p = ggp;
		  t->splay_l = gp;
		  t->splay_r = p;
		  gp->splay_p = t;
		  p->splay_p = t;
		  gp->splay_r = l;
		  p->splay_l = r;
		  if(l)
		    l->splay_p = gp;
		  if(r)
		    r->splay_p = p;
		  if(ggp)
		    {
		      if(ggp->splay_l == gp)
			ggp->splay_l = t;
		      else
			ggp->splay_r = t;
		    }
		}
	    }
	}
      else
	{
	  if(t->splay_p->splay_l == t)
	    /* zig_left(t); */
	    {
	      register treenode *p, *r;
	      p = t->splay_p;
	      r = t->splay_r;
	      t->splay_r = p;
	      t->splay_p = (treenode *) 0;
	      if(r)
		r->splay_p = p;
	      p->splay_l = r;
	      p->splay_p = t;
	    }
	  else
	    /* zig_right(t); */
	    {
	      register treenode *p, *l;
	      p = t->splay_p;
	      l = t->splay_l;
	      t->splay_l = p;
	      t->splay_p = (treenode *) 0;
	      if(l)
		l->splay_p = p;
	      p->splay_r = l;
	      p->splay_p = t;
	    }
	}
    }
}

/*
** Locate the node in the splay tree that matches the selected key.
** Returns a pointer to the node or NULL if not found.
** If the node has a linear sublist, this list must be accessed by
** following the next pointers in the obtained node.
**
** cmp_key is the external key compare function.
** It returns zero if the key matches the node.
** It returns -1 if the node is less than the key.
** It returns  1 if the node is greater than the key.
*/

static treenode *
splay_find(KEYTYPE k, treenode **root, int (*cmp_key)(treenode *T, KEYTYPE k))
{
  register treenode *T = *root;
  register int tmp;

  /* Done if the tree is empty. */
  if(!*root)
    return (treenode *) 0;

  /*
  ** Traverse the tree looking for the node containing the key k.
  ** Exit if the node could not be found.
  */
  while((tmp = cmp_key(T, k)) != 0)
    {
      if(tmp > 0)
	{
	  if(T->splay_l)
	    T = T->splay_l;
	  else
	    return (treenode *) 0;
	}
      else
	{
	  if(T->splay_r)
	    T = T->splay_r;
	  else
	    return (treenode *) 0;
	}
    }

  /* Select the found node as new root and splay the tree. */
  splay(*root = T);
  return T;
}

/*
** Delete the selected node from the tree.
**
** The node must be in the tree.
** The NULL pointer is detected and ignored.
** The node may have been obtained by a call to splay_find(), but it can
** also have been obtained in some other convenient way.
** It is not required that the node to be deleted is the current root.
**
** Returns the input parameter node.
** The caller must free the space occupied by the deleted node.
*/

static treenode *
splay_delete(treenode *node, treenode **root)
{
  register treenode *l, *r;
  register treenode *T = node;

  /* Done if the NULL pointer was deleted. */
  if(!T)
    return (treenode *) 0;

  /* Promote the node to be deleted to the root. */
  splay(T);

  /* Get the left and right sub trees. */
  l = T->splay_l;
  r = T->splay_r;

  /*
  ** If there is a left subtree then make the inorder
  ** predecessor to the deleted node the new root.
  ** Attach the right subtree.
  */
  if(l)
    {
      l->splay_p = (treenode *) 0;
      while(l->splay_r)
	l = l->splay_r;
      splay(*root = l);
      if(r)
	{
	  r->splay_p = l;
	  l->splay_r = r;
	}
    }
  
  /*
  ** If there is only a right subtree then make the inorder
  ** successor to the deleted node the new root.
  */
  else if(r)
    {
      r->splay_p = (treenode *) 0;
      while(r->splay_l)
	r = r->splay_l;
      splay(*root = r);
    }

  /*
  ** If there are no subtrees left then remove the entire tree.
  */
  else
    *root = (treenode *) 0;
	  
  /* Return the deleted node. */
  return T;
}

/*
** Insert a new node at the location matching the selected key.
** The caller must allocate space for the new node before calling this function.
** The node is not inserted if a node with the specified key already exists.
**
** Returns zero if the operation was successful.
** Returns a pointer to the duplicate node if the operation failed.
**
** cmp_key is the external key compare function.
** It returns zero if the key matches the node.
** It returns -1 if the node is less than the key.
** It returns  1 if the node is greater than the key.
*/

static treenode *
splay_insert(KEYTYPE k, treenode *newnode, treenode **root, int (*cmp_key)(treenode *T, KEYTYPE k))
{
  register treenode *T = *root;
  register int tmp;

  /* Create new root if first node. */
  newnode->splay_p = (treenode *) 0;
  newnode->splay_l = (treenode *) 0;
  newnode->splay_r = (treenode *) 0;
  newnode->splay_n = (treenode *) 0;
  if(!T)
    {
      *root = newnode;
      return (treenode *) 0;
    }
  /*
  ** Traverse the tree looking for the node containing the key k,
  ** or the inorder successor or predecessor to k.
  ** Then insert the new node at that point.
  */
  while((tmp = cmp_key(T, k)) != 0)
    {
      if(tmp > 0)
	{
	  if(T->splay_l)
	    T = T->splay_l;
	  else
	    {
	      /* Insert the new node. */
	      T->splay_l = newnode;
	      newnode->splay_p = T;
	      /* Promote the new node to the root and exit. */
	      splay(*root = newnode);
	      return (treenode *) 0;
	    }
	}
      else
	{
	  if(T->splay_r)
	    T = T->splay_r;
	  else
	    {
	      /* Insert the new node. */
	      T->splay_r = newnode;
	      newnode->splay_p = T;
	      /* Promote the new node to the root and exit. */
	      splay(*root = newnode);
	      return (treenode *) 0;
	    }
	}
    }
  /*
  ** We found an already existing node with the same key.
  ** Promote it to the root and return a pointer to it.
  */
  splay(*root = T);
  return T;
}

/*
** Delete the selected node from the tree or from a linear sublist.
**
** The node must be in the tree or in a sublist to some node in the tree.
** The NULL pointer is detected and ignored.
** The node may have been obtained by a call to splay_find(), but it can
** also have been obtained in some other convenient way.
** It is not required that the node to be deleted is the current root.
**
** Returns the input parameter node.
** The caller must free the space occupied by the deleted node.
*/

static treenode *
splay_lin_delete(treenode *node, treenode **root)
{
  register treenode *next;
  register treenode *T = node;

  /* Done if the NULL pointer was deleted. */
  if(!T)
    return (treenode *) 0;

  /* Is the node in the splay tree? */
  if((T->splay_p) && ((T->splay_p == T->splay_l) || (T->splay_p == T->splay_r)))
      {
      /* No, the node is in a sublist. Is it the top? */
      T->splay_p->splay_n = T->splay_n;
      if(T->splay_p == T->splay_l)
	  {
	  /* Yes, delete it. */
	    next = T->splay_n;
	    if(next)
	      {
	      next->splay_p = T->splay_p;
	      next->splay_l = T->splay_p;
	      next->splay_r = (treenode *) 0;
	      }
	  }
      else
	  {
	  /* No, delete it in a nother way. */
	    next = T->splay_n;
	    if(next)
	      {
	      next->splay_p = T->splay_p;
	      next->splay_r = T->splay_p;
	      }
	  }
      }
  else
      {
      /* Yes, Sublist present? */
	next = T->splay_n;
	if(next)
	  {
          /* Yes, replace the node by the sublist. */
	  if(T->splay_p)
	      {
	      if(T->splay_p->splay_l == T)
		  T->splay_p->splay_l = next;
	      else
		  T->splay_p->splay_r = next;
	      }
	  else
	      {
	      *root = next;
	      }
	  next->splay_p = T->splay_p;
	  next->splay_l = T->splay_l;
	  if(next->splay_l)
	      next->splay_l->splay_p = next;
	  next->splay_r = T->splay_r;
	  if(next->splay_r)
	      next->splay_r->splay_p = next;
	  next = next->splay_n;
	  if(next)
	      {
	      next->splay_l = next->splay_p;
	      next->splay_r = (treenode *) 0;
	      }
	  }
      else
	  {
          /* No, delete the node. */
	  splay_delete(node, root);
	  }
      }
	  
  /* Return the deleted node. */
  return T;
}

/*
** Insert a new node at the location matching the selected key.
** If the splay tree location representing the key is already taken, then the
** new node is placed in the sublist of that node.
** The caller must allocate space for the new node before calling this function.
** cmp_key is the external key compare function.
** It returns zero if the key matches the node.
** It returns -1 if the node is less than the key.
** It returns  1 if the node is greater than the key.
*/

static void
splay_lin_insert(KEYTYPE k, treenode *newnode, treenode **root, int (*cmp_key)(treenode *T, KEYTYPE k))
{
  register treenode *T;
  register treenode *next;

  /* Try to insert as a new splay tree node. */
  T = splay_insert(k, newnode, root, cmp_key);
  if(!T)
    return;

  /* If already present, then insert node into sublist of the present node. */
  next = T->splay_n;
  if(next)
      {
      next->splay_p = newnode;
      next->splay_l = (treenode *) 0;
      next->splay_r = newnode;
      }

  T->splay_n = newnode;
  newnode->splay_n = next;
  newnode->splay_p = T;
  newnode->splay_l = T;
  newnode->splay_r = (treenode *) 0;
}

/*
**===========================================================================
** 24.3  System calls.
**===========================================================================
*/

/*
**===========================================================================
** 24.4  Undocumented system calls.
**===========================================================================
*/

/*
*****************************************************************************
** 25 MISCELLANEOUS.
*****************************************************************************
*/

/*
**===========================================================================
** 25.1  Initialization.
**===========================================================================
*/

/*
**===========================================================================
** 25.2  Local functions.
**===========================================================================
*/

/*
** Make a remote system call to the indicated system call handler.
**
** The caller must be a superuser process, i.e it must have been
** using the set_suid() call to enter superuser mode.
*/

#if USE_REMOTE_CALLS
static union SIGNAL *
make_remote_call(union SIGNAL *sig, OSCALLTYPE response_type, PROCESS handler)
{
  static const SIGSELECT sig_sel[] = {3, 
	REMOTE_RESPONSE, REMOTE_CANCEL, OS_ATTACH_SIG};
  OSATTREF attach_id;

  /* Tell get_pcb that we are in a remote call. */
  zzcur_p->in_rem_call = (OSBOOLEAN) 1;

  attach_id = attach(NULL, handler);
  send(&sig, handler);
  while(NIL == (sig = receive_from((OSTIME) 10000, sig_sel, handler)))
    {
      os_error(1, OSE_ELINKHANDLER_DEADLOCK,
	       (OSERRCODE) handler);	/* Non fatal system error. */
    }

  /* Tell get_pcb that we are no longer in a remote call. */
  zzcur_p->in_rem_call = (OSBOOLEAN) 0;

  switch((int) sig->sig_no)
    {
    case REMOTE_RESPONSE:
      {
	if(sig->remote_response.call_type != response_type)
	  {
	    /* The process received a bad remote response from a handler. Drop it. */
	    free_buf(&sig);
	    return (union SIGNAL *) 0;
	  }
	detach(&attach_id);
	return sig;
      }
    case REMOTE_CANCEL:
      {
	detach(&attach_id);
	/* Fall into OS_ATTACH_SIG clause... */
	/*FALLTHROUGH*/
      }
    default:
      {
	free_buf(&sig);
	return (union SIGNAL *) 0;
      }
    }
}
#endif

/*
** Make a signal buffer copy in user space.
**
** This function is passed a pointer to a signal buffer in
** the system pool.
** If the calling process is not attached to the system pool,
** Then the buffer is replaced by a copy in the callers pool.
**
** This function is used in a number of system calls that must
** build a signal while the dispatcher is locked.
** This signal must be in the system pool since a page fault
** might otherwise occur with a locked dispatcher, which is
** illegal.
**
** The dispatcher may not be locked when this function is called.
** Only non interrupt processes may use this function.
*/

static void *
sig_dup(union SIGNAL *sig)
{
  void *new_sig;
  OSBUFSIZE size = sigsize(&sig);

  /* Done if the caller is attached to the system pool (optimization).?? */

  /* Allocate a new buffer. */
  new_sig = (void *) alloc(size, (SIGSELECT) 0);

  /* Copy buffer data to the new buffer. */
  memcpy(new_sig, sig, (size_t) size);

  /* Free the old buffer. */
  free_buf(&sig);

  /* Return the new buffer. */
  return new_sig;
}

/*
** Make a signal buffer copy in user space.
**
** This function is passed a pointer to a signal buffer in
** a segment other than the callers.
** The buffer is replaced by a copy in the callers pool.
**
** This function is used in the receive-type system calls
** when intersegment copying of received signals is required.
**
** returns zero if the callers pool can not handle a signal this
** large and an error handler ignored the error.
**
** The dispatcher may not be locked when this function is called.
*/

#if MEMORY_MODEL > 0
static union SIGNAL *
rec_sig_dup(struct _signal *sigadm_in)
{
  union SIGNAL *sig;
  struct _signal *sigadm_out;
  union SIGNAL *new_sig;
  OSBUFSIZE size;
  struct _pcb *pcb = zzcur_p;

  /* Get the input signal control block pointer. */
  sig = adm_to_sig(sigadm_in);

  /* Check that the callers pool has a suitable buffer. */
  size =  sigadm_in->size;
  if(unlikely(zzcur_p->cursigpool->signal_list[7].size < size))
    {
      /* Report an error and return zero if not. */
      os_error(0, OSE_EINTERSEGMENT_COPY_FAILED, (OSERRCODE) size);	/* Non fatal. */
      return (union SIGNAL *) 0;
    }

  /*
  ** Cancel any pending timeout status to avoid trouble in pool extend.
  ** A pending timeout or the ST_TIMEOUT status bit can otherwise cause
  ** receive() to return NIL.
  */
  if(pcb->tmo_active)
    {
      if(pcb->priority <= 32)
	{
	  /* No interrupt processes enter here. */
	  LOCK();
	  cancel_timeout();
	  UNLOCK();
	}
    }
  else
    {
      /* We must also clear ST_TIMEOUT for receive_from(0,...) calls. */
      pcb->status &= ~ST_TIMEOUT;
    }
  
  /* Allocate a similar buffer in the callers pool. */
  new_sig = alloc(size, (SIGSELECT) 0);

  /* Copy buffer data to the new buffer. */
  memcpy((void *) new_sig, (void *) sig, (size_t) size);

  /* Get the output signal control block pointer. */
  sigadm_out = sig_to_adm(new_sig); /*@@ maybe check_sig here. Or a simpler inline check, or safe_alloc. */

  /* Copy sender and addresse fields to the new buffer. */
  sigadm_out->dest_id = sigadm_in->dest_id;
  sigadm_out->sender_id = sigadm_in->sender_id;

  /* Free the old buffer without causing an error. */
  safe_free_buf(sigadm_in);

  /* Return the new buffer. */
  return new_sig;
}
#endif


/* 
** Helper for set_mem and get_mem. 
**
** Here we only treat supervisor mode callers. Supervisor mode sees
** all logical memory exept MAS mapped regions in user mode programs,
** so we only need to set_segment for user mode programs with MAS.
** The id argument has been validated by the caller. Still, the id
** object can be dead now.
**
** A user mode caller will not run this function, instead it is
** handled completely in user space by the target interface.
*/
static OSBOOLEAN
zzkrn_set_get_mem(PROCESS id,
	    const void *from,
	    void *to,
	    size_t size,
	    OSERRCODE syscall)
{
  struct _pcb *victim = NULL;
  OSSEGMENT victim_segnum = 0;
  int status;

  /* If operating on the current process, use the current mapping. */
  if(id == pcb_to_pid(zzcur_p))
    id = 0;
  
  /* Id zero affects the system segment, else check for a valid id. */
  victim_segnum = 0;
  if(id != 0)
    {
      victim = safe_pid_to_pcb(id, 1);

      if(unlikely(!victim))
	{
	  /* Error report if we got an invalid id. */
	  check_pid_bid_seg(id, syscall);
	  return 1;		/* Fail */
	}

      LOCK_SWAP();
      if(victim->instance == pid_to_instance(id))
	{
	  struct segcb *victim_segcb = find_segcb(victim);
	  if(victim_segcb && victim_segcb->mas_mapped)
	    victim_segnum = victim_segcb->segid;
	}
      UNLOCK_SWAP(); 
    }

  /* Copy the data, possibly using another mapping. */
  if(victim_segnum)
    {
      if(set_segment(0, victim_segnum))
	{
	  /* The segment is dead. */
	  return 1;		/* Fail */
	}
    }

  /* Copy if the victim is alive, else report failure. */
  if(id == 0 || victim->instance == pid_to_instance(id))
    status = zzguarded_memcpy(to, (const char *)from, size, 0);
  else
    status = 1;			/* Fail */

  if(victim_segnum)
    set_segment(1, 0);	/* Go home */

  /* Return zero if success. */
  return status ? 1 : 0;
}

/*
**===========================================================================
** 25.3  System calls.
**===========================================================================
*/

/*
** This is the power_fail system call.
**
** We allow this call to be used from an interrupt process too, which is not
** standard yet, but should be for backward OSE compatibility.
*/

void 
power_fail(void)
{
  /* Set the power fail request flag and do a process switch. */
  zzuu.pfail_req = (OSBOOLEAN) 1;

  /* Run the interrupt version if the caller is an int or timer process. */
  if(zzcur_p->priority > 32)
    {
      zzww.dp_req = (OSBOOLEAN) 1;
    }
  else
    {
      LOCK();
      zzquit_int();	/* swap. */
      UNLOCK();
    }
}

/*
** This is the set_mem system call.
*/

OSBOOLEAN
set_mem(PROCESS id,
	void *from,
	OSADDRESS to,
	OSADDRESS size)
{
  return zzkrn_set_get_mem(id, from, (void *)to, (size_t) size, OSE_SET_MEM);
}

/*
** This is the get_mem system call.
*/

OSBOOLEAN
get_mem(PROCESS id,
	OSADDRESS from,
	void *to,
	OSADDRESS size)
{
  return zzkrn_set_get_mem(id, (const void *)from, to, (size_t) size, OSE_GET_MEM);
}

/*
** The create_event() and signal_event() system calls supplies user
** defined events to the application.
**
** The functions are stubbed out here if the RMM is not included, otherwise
** they are implemented in the RMM.
**
** The trig variable is maintained by the RMM, and it is set to a nonzero value
** to indicate to the application that signal_event calls should be
** enabled for a class. (There need be only one trig variable for each class.)
** It is assumed that applications do not modify trig variables.
**
*/

#if OSE_DEBUG == 0
void
create_event(SIGSELECT eventno, OSADDRESS *trig)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) eventno;

  /* Set the event trig variable to zero so that the application
     do not call signal_event. */
  *trig = 0;
}

void
zzsignal_event(SIGSELECT eventno, const void *data, OSBUFSIZE size)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) eventno;
  (void) data;
  (void) size;
}

/*
** The zzose_create_report_sysconf_78() and zzose_set_report_val_sysconf_77()
** are the system name functions for ose_create_report() and
** ose_set_report_val().
**
** The functions are stubbed out here if the RMM is not included, otherwise
** they are implemented in the RMM.
**
** The trig variable is maintained by the RMM, and it is set to a nonzero value
** to indicate to the application that signal_event calls should be
** enabled for a class. (There need be only one trig variable for each class.)
** It is assumed that applications do not modify trig variables.
**
*/

OSADDRESS
zzose_create_report_sysconf_78(OSADDRESS reportno, OSADDRESS trig, OSADDRESS rpt_flag)
{
   /* Remove ``not used args'' compiler warnings. */
   (void) reportno;
   (void) trig;
   (void) rpt_flag;
   
   /* The report trig variable is initialized by pflib.c in the crt. */
   
   return (OSADDRESS) 0;
}

OSADDRESS
zzose_set_report_val_sysconf_77(OSADDRESS reportno, OSADDRESS reportid, OSADDRESS change)
{
   /* Remove ``not used args'' compiler warnings. */
   (void) reportno;
   (void) reportid;
   (void) change;

   return (OSADDRESS) 0;   
}

#endif

/*
**===========================================================================
** 25.4  Undocumented system calls.
**===========================================================================
*/

/*
** The set_clib_pointer() system call stores a pointer in the current block
** descriptor. This pointer contains any implementation specific information
** desired, but the main purpose is to obtain the address of an intermediate
** function in the current C library used when calling user mode error
** handlers.
** The set_clib_pointer() system call should be used in the zzmain process
** before any other system call is used, specifically before any error can
** occur.
** The clib pointer is inherited to child blocks, so each block in the system
** has a valid clib pointer.
** The system block assumes the value of the first clib pointer set, so the
** clib pointer for the system block is not valid until the first zzmain
** process has been started.
*/

void
set_clib_pointer(void *clib_pointer)
{
  /* Store pointer in the current block descriptor. */
  zzcur_p->block->clib_pointer = clib_pointer;

  /* First time store the pointer in the system block too. */
  if(!(zzuu.system_block->clib_pointer))
    zzuu.system_block->clib_pointer = clib_pointer;
}

/*
** This is a helper function for the target interface. It decides if a
** usermode get/set_mem is to be performed and if so it validates the
** id parameter.
**
** Returns 0 if the caller is a supervisor mode process.
** Returns 1 (error) if the caller is a user mode process and the
**           supplied id is outside the current segment.
** Returns 2 if the caller is a user mode process and the supplied id
**           is inside the current segment.
**
** If 2 is returned the target interface will perform a crash safe
** memcpy() in user-space. This way we lock out all usermode processes
** from getting/setting memory in other segments than its own,
** whithout the need for time costly access-right-checks in the memory
** manager.
*/
OSBOOLEAN
zzkrn_pre_get_set_mem(PROCESS id, OSERRCODE subcode)
{
  struct _pcb *cur_p = zzcur_p;
  struct _pcb *victim;

  /*
  ** If the calling process is a supervisor mode process we will
  ** return zero to tell the target interface to call get/set_mem.
  */
  if(likely(cur_p->block->supervisor_mode))
    {
      /* Call get/set_mem. */
      return 0;
    }

  /* Access of current user mode process is always permitted. */
  if(likely(id == pcb_to_pid(cur_p)))
    {
      /* Perform memcpy in user mode. */
      return 2;
    }

  /* Other victims must be alive and reside in the same segment. */
  victim = safe_pid_to_pcb(id, 1);
  if(likely(victim && (victim->magic & ~3) == MAGIC_PCB))
    {
      LOCK_SWAP();
      if(likely(victim->instance == pid_to_instance(id)))
	{
	  struct segcb *victim_segcb = find_segcb(victim);

	  if(victim_segcb && victim_segcb->segid == cur_p->segid)
	    {
	      /*
	      ** Caller is user mode, and caller and victim in same segment.
	      ** Perform copy in user mode.
	      */
	      UNLOCK_SWAP();
	      return 2;
	    }
	}
      UNLOCK_SWAP();
    }
  else
    {
      /* Error report if id is invalid. */
      check_pid_bid_seg(id, subcode);
    }

  /* Return failure status. */
  return 1;
}

/*
** This is the get_sysconf() system call.
**
** This function provides a path for using various special purpose
** features in the operating system. The intention is that the
** system call should be used by kernel related platform products
** for proprietary functions of limited scope.
**
** The f parameter contains a function code between 0 and N, where
** N = MAX_SYSCONF - 1, and can be read with function code 0.
** Unimplemented functions return zero.
**
** Currently supplied functions:
**  f = 0: Read the highest function code available. Returns N.
**  f = 1: Read a function pointer. p = func code. Returns zero if uninstalled.
**  f = 2: Write a function pointer. p = func code, q = pointer or NULL,
**         r = 0 indicates that only supervisor processes can use the function.
**         Returns the old pointer.
**  f = 3: Set the error check level. p = level (0-2). Returns old value.
**  f = 4: Read the zzosvarp symbol.
**  f = 10-19: Reserved for REOS system calls according to:
**         10 OS_def_dev()
**         11 OS_get_desc()
**         12 OS_call()
**         13 OS_set()
**         14 OS_reset()
**         15-19 Unassigned.
**  f = 20: The public receive_sport() call.
**  f = 21: The public get_handlers() call.
**  f = 22: The public set_sigsize() call.
**  f = 23: The public get_sig_info() call.
**  f = 24: The public get_pool_list() call.
**  f = 25: The public get_poolcb() call.
**  f = 26: The public get_stk_poolid() call.
**  f = 27: The public get_sig_poolid() call.
**  f = 28: The public get_envp() call.
**  f = 29: The public set_envp() call.
**  f = 30: Read the privilege flag of a function pointer. p = func code.
**          Returns zero if superuser only or uninstalled.
**  f = 31: Enable sampled cpu load measurement. p = q = r = unused.
**          Returns a pointer to the sampling function called periodically
**          from a user provided interrupt process.
**          Returns zero if sampled measurement is not supported.
**  f = 32: Return the number of strings in the system process name list.
**          Returns zero if no list is available.
**  f = 33: Return a pointer to the system process name list.
**          Returns zero if no list is available.
**  f = 34-44: Reserved for use by target interface implementations.
**  f = 45: Returns a pointer to a kernel structure which lists the
**          configuration fields (pose.con values) for the current kernel.
**  f = 46: The public set_halt_condition() call.
**  f = 47: Returns power save information. p = 0 returns valid p range.
**          p = 1 returns power save threshold. p = 2 returns power saved
**          ticks.
**  f = 48: The public request_tmo_sig() call.
**  f = 49: The public request_tmo() call.
**  f = 50: The public cancel_tmo_sig() call.
**  f = 51: The public cancel_tmo() call.
**  f = 52: The public restart_tmo() call.
**  f = 53: Per process data handling.
**  f = 54: Returns a pointer to the usrcalls array.
**  f = 55: The public set/get_timeslice system call.
**  f = 56: A receive_from without timeout, for use in exception handlers.
**  f = 57: Read or set zzuu.pagesize. Setting it non-zero enables paging.
**  f = 58: Returns dispatcher lock flag.
**  f = 59: mutex_enqueue
**  f = 60: mutex_dequeue
**  f = 61: mutex_destroy
**  f = 62: mutex_error
**  f = 63: call to set shared mode flag for a given block.
**  f = 64: return shared mode flag for a block.
**  f = 65: Read or write the LNH specific PPD (Per Process Data).
**  f = 66: send_hook_free_buf.
**  f = 67: Function for setting pool extender callout.
**  f = 68: Function for setting a syserr test callout.
**  f = 69: The public ose_nano_delay system call.
**  f = 70: mutex_init
**  f = 71: The private get_segid_list call.
**  f = 72: The private get_pid_info call.
**  f = 73: The private get_system_statistics call.
**  f = 74: The public ose_get/set_time_resolution call. (OBSOLETE)
**  f = 75: The public ose_bind_process call.
**  f = 76: The public send_hook_restore call.
**  f = 77: The public set_report_val call 
**  f = 78: The public create_report call
**  f = 79: Function to register a Program Manager.
**  f = 80: Private call to set mas_mapped segcb attribute. 
**  f = 81: Private call to get a pointer to the sysctlblk configuration etc.
**  f = 82: Public call to do create_sem with annotation data.
**  f = 83: Public call to control create_sem annotation.
**  f = 84: zzkrn_restore_from
**  f = 85: zzkrn_send_w_sender
**  f = 90: Private call to dispatch isr related calls.
**  f = 91: zzkrn_receive_fsem
**  f = 92: zzkrn_signal_fsem_from_isr
**  f = 93: zzkrn_get_ticks_addr
**  f = 94: The public s_alloc_nil call.
**  f = 95: ose_set_pool_max_size
**  f = 96: ose_uninstall_c_exception_handler
**  f = 97: ose_set_pool_attr
**  f = 98: alloc_brk
**  f = 99: Function for getting hold of zzkrn_claim_ext_sig() and
**          zzkrn_release_ext_sig();
**  f = 100: ose_mutex_init_unsafe
**  f = 101: Handle the receive hardware poll hook.
**  f = 102: Install block state hook.
**  f = 103: Uninstall block state hook.
**  f = 104: Invalidate TLB address range.
**  f = 105: RESERVED.
**  f = 106: RESERVED.
**  f = 107: Public calls ose_get_error_info() and ose_reset_error_info().
*/

#define MAX_SYSCONF 128

static OSADDRESS (*sysconf[MAX_SYSCONF])(OSADDRESS p, OSADDRESS q, OSADDRESS r);
static U8 sysconf_privil[MAX_SYSCONF];

static OSADDRESS
noop_sysconf(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) p;
  (void) q;
  (void) r;

  /* This is the default function for uninstalled slots. */
  return (OSADDRESS) 0;
}

static OSADDRESS
sysconf_0(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) p;
  (void) q;
  (void) r;

  return (OSADDRESS) MAX_SYSCONF - 1 ;
}

static OSADDRESS
sysconf_1(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) q;
  (void) r;

  if(p >= MAX_SYSCONF)
    return (OSADDRESS) 0;
  if(sysconf[p] == noop_sysconf)
    return (OSADDRESS) 0;
  else
    return (OSADDRESS) sysconf[p];
}

static OSADDRESS
sysconf_2(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  OSADDRESS o = (OSADDRESS) 0;

  /* Check if out of bounds. */
  if(p >= MAX_SYSCONF)
    return (OSADDRESS) 0;

  /* Return the old function pointer or NULL. */
  if(sysconf[p] != noop_sysconf)
    o = (OSADDRESS) sysconf[p];

  /* Set supervisor only privileges for uninstalled functions. */
  if(!p)
    r = 0;

  /* Install the new pointer. */
  sysconf[p] = (OSADDRESS (*)(OSADDRESS, OSADDRESS, OSADDRESS)) q;

  /* Enable user mode calls if required. */
  sysconf_privil[p] = (U8) (r ? 1 : 0);

  return o;
}

static OSADDRESS
sysconf_30(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) q;
  (void) r;

  /* Check if out of bounds. */
  if(p >= MAX_SYSCONF)
    return (OSADDRESS) 0;

  /* return the current privilege level. */
  return (OSADDRESS) sysconf_privil[p];
}

static OSADDRESS
sysconf_32(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  char *names;
  U32 count;

  /* Remove ``not used args'' compiler warnings. */
  (void) p;
  (void) q;
  (void) r;

  /* Read system process information. */
  count = zzget_sysprocnames(&names);

  /* Return the number of strings in the system process name list. */
  return (OSADDRESS) count;
}

static OSADDRESS
sysconf_33(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  char *names;
  U32 count;

  /* Remove ``not used args'' compiler warnings. */
  (void) p;
  (void) q;
  (void) r;

  /* Read system process information. */
  count = zzget_sysprocnames(&names);

  /* Return a pointer to the system process name list. */
  return (OSADDRESS) (count ? names : 0);
}

static OSADDRESS
sysconf_3(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) q;
  (void) r;
  
#if ERROR_CHECKS == 0
  return (OSADDRESS) 0;
#endif

#if ERROR_CHECKS == 1
  {
    OSADDRESS o = (OSADDRESS) 0;
    o += (zzuu.error_level1 ? 1 : 0);
    o += (zzuu.error_level2 ? 1 : 0);
  
    zzuu.error_level1 = (U8) (p >= 1);
    zzuu.error_level2 = (U8) (p >= 2);
    return o;
  }
#endif

#if ERROR_CHECKS == 2
  return (OSADDRESS) 2;
#endif
}

static OSADDRESS
sysconf_4(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) p;
  (void) q;
  (void) r;

  return (OSADDRESS) &zzosvarp;
}

/*
** This is the struct handlerp *get_handlers(void) system call.
** Returns a pointer or NULL if not implemented.
*/
static OSADDRESS
sysconf_21(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) p;
  (void) q;
  (void) r;

  return (OSADDRESS) &handlerp;
}

/*
**  This function returns a pointer to a kernel structure which lists the
**  configuration fields (pose.con values) for the current kernel.
*/
static OSADDRESS
sysconf_45(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) p;
  (void) q;
  (void) r;

  return (OSADDRESS) &confp;
}

/*
**  This function returns power save information.
*/
static OSADDRESS
sysconf_47(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) q;
  (void) r;

  switch(p)
    {
    case 0:
      /*
      ** Tell the caller that this function can deliver useful
      ** information for values of [0..n] in parameter p.
      */
      return (OSADDRESS)3;
    case 1:
      /*
      ** Return the minimum time to next timer event for entering
      ** power save mode.
      */
      return (OSADDRESS)zzuu.power_save_threshold;
    case 2:
      /* Return the accumulated time spent in the power save handler. */
      return (OSADDRESS)zzuu.power_saved_ticks;
    case 3:
      /* Return the longest time spent in the power save handler. */
      return (OSADDRESS)zzuu.longest_power_nap;
    default:
      return (OSADDRESS)0;
    }
}

/*
** This funtion returns a pointer to the usrcalls array, which
** contains pointers to system calls that can be invoked directly from
** user mode.
*/
static OSADDRESS
sysconf_54(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) p;
  (void) q;
  (void) r;

  return (OSADDRESS) &usrcalls;
}

/*
** This function sets or reads the pagesize variable in the world struct.
**
** Setting the pagesize to non-zero enables a limited paging support for 
** the text segment. The value must be a power of 2, defining the smallest
** page used. It must be set before any paging processes are created.
*/
static OSADDRESS
sysconf_57(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) r;
  switch(p)
    {
    case 0:
      /*
      ** Tell the caller that this function can deliver useful
      ** information for values of [0..2] in parameter p.
      */
      return (OSADDRESS)2;

    case 1:
      /* Return the current pagesize. */
      return zzuu.pagesize;

    case 2:
      /*
      ** Set the pagesize. Enables paging support when non-zero.
      ** Return the new pagesize, or zero if paging not supported.
      */
      zzuu.pagesize = q;
      return q;

    default:
      return (OSADDRESS)0;
    }
}

/*
** This function returns dispatcher lock flag.
*/
static OSADDRESS
sysconf_58(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void) p;
  (void) q;
  (void) r;

  return (OSADDRESS) zzdispatcher_locked;
}

/*
** Read or write the LNH specific PPD (Per Process Data).
**
** Argument p should be:
**            0 for reading lnh_sender_hd
**            1 for setting lnh_sender_hd
**            2 for reading lnh_dest_hd
**            3 for setting lnh_dest_hd
** Argument q is the pid of the process.
** Argument r is the new lnh_*_hd value (only used if p==1 or p==3).
**
** Returns the current value of lnh_sender_hd.
*/
static OSADDRESS
sysconf_65(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  LOCKSTATE msr;
  register PROCESS pid = (PROCESS)q;
  register PCB_INDEX_T pindex;
  register struct _pcb *pcb;

  /* Split process id into process index and instance number. */
  pindex = (PCB_INSTANCE_T) pid;

  /* Check that the process index is reasonable. */
  if(pindex >= zzuu.next_free_pix)
    {
      return 0;
    }

  pcb = pid_to_pcb(pid);

  /* Check that the magic number indicates a process. */
  if(pcb->magic != MAGIC_PCB)
    return 0;

  BKL_LOCK_SAVE(msr);
  
  /* Check if the process has been killed. */
  if(pcb_to_pid(pcb) != pid)
    {
      BKL_LOCK_RESTORE(msr);
      return 0;
    }
  
  if((p&1) == 0)
    {
      /* Read. */
      if((p>>1) == 0)
	r = (OSADDRESS)(pcb->lnh_sender_hd); /* p == 0 */
      else
	r = (OSADDRESS)(pcb->lnh_dest_hd);   /* p == 2 */
    }
  else
    {
      /* Write. */
      if((p>>1) == 0)
	pcb->lnh_sender_hd = (void *)r;      /* p == 1 */
      else
	pcb->lnh_dest_hd = (void *)r;        /* p == 3 */
    }

  BKL_LOCK_RESTORE(msr);
  return r;
}

/* This is the get_sysconf entry for send_hook_free_buf. */
static OSADDRESS
sysconf_66(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void)q;
  (void)r;

  zzkrn_send_hook_free_buf((union SIGNAL *)p);
  return 1;
}

/* This is the get_sysconf entry for setting a pool extender callout. 
** Case 1 and 2 no longer used in later extenders.
** Case 3 installs an extender with realtime capacity, for unicore and
** multicore.
*/
static OSADDRESS
sysconf_67(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  struct poolcb *poolcb;
  OSPOOLEXTENDH *h = NULL;
  LOCKSTATE msr;

  switch (q)
    {
    case 1:
      /* Obsoleted: Register a system pool extender for unicore only. */
      poolcb = &system_pool;
      h = poolcb->ext_func;
      poolcb->ext_func = (OSPOOLEXTENDH *)p;
      poolcb->realtime_extendable = 1;

      /* If no max_size has been set, allow any size. */
      if(!poolcb->max_size)
	poolcb->max_size = (OSADDRESS)~0;
      
      break;

    case 2:
      /* Obsoleted: Register a realtime "main pool extender" for unicore only. */
      BKL_LOCK_SAVE(msr);
      poolcb = (struct poolcb *)safe_pid_to_pcb((PROCESS)r, 1);
      if (poolcb && poolcb->magic == MAGIC_POOL)
	{
	  h = poolcb->ext_func;
	  poolcb->ext_func = (OSPOOLEXTENDH *)p;
	  poolcb->realtime_extendable = 1;
	  /* If no max_size has been set, allow any size. */
	  if(!poolcb->max_size)
	    poolcb->max_size = (OSADDRESS)~0;

	  if(zzuu.main_pool == NULL)
	    zzuu.main_pool = poolcb;
	}
      BKL_LOCK_RESTORE(msr);
      break;

    case 3:
      /*
      ** Register a realtime pool extender that handles both unicore and
      ** multicore systems.
      */
      {
	OSADDRESS 	*args          = (OSADDRESS *)p;
	PROCESS          pool_id       = (PROCESS)(args[1]);
	OSPOOLEXTENDH   *ext_func      = (OSPOOLEXTENDH *)(args[2]);
	void * volatile *ext_frag_pp   = (void * volatile  *)(args[3]);
	OSPRIORITY       ext_frag_prio = (OSPRIORITY)(args[4]);

	if(args[0] != 5)
	  {
	    os_error(0, OSE_EBAD_PARAMETER, p);
	    return 0;
	  }

	BKL_LOCK_SAVE(msr);
	/*
	** Older multicore aware pool_extend.c did refer to system pool
	** by id zero, kept for compatibility only.
	*/
	if(pool_id == 0)
	  pool_id = pcb_to_pid(&system_pool);  

	poolcb = (struct poolcb *)safe_pid_to_pcb(pool_id, 1);

	if(poolcb && poolcb->magic == MAGIC_POOL &&
	   pool_id == pcb_to_pid(poolcb))
	  {
	    h = poolcb->ext_func;
	    poolcb->ext_func      = ext_func;
	    poolcb->ext_frag_prio = ext_frag_prio;
	    poolcb->ext_frag_pp   = ext_frag_pp;
	    poolcb->realtime_extendable = 1;

	    /* If no max_size has been set, allow any size. */
	    if(!poolcb->max_size)
	      poolcb->max_size = (OSADDRESS)~0;

	    /* Register the first pool as the main pool. */
	    if(poolcb != &system_pool && zzuu.main_pool == NULL)
	      zzuu.main_pool = poolcb;
	  }
	BKL_LOCK_RESTORE(msr);
	break;
      }

    case 4:
      {
	OSADDRESS *args = (OSADDRESS *)p;
	struct poolcb *poolcb;
	
	if(args[0] != 5)
	  {
	    os_error(0, OSE_EBAD_PARAMETER, p);
	    return 0;
	  }

	BKL_LOCK_SAVE(msr);
	poolcb = (struct poolcb *)safe_pid_to_pcb((PROCESS) r, 1);
	if(poolcb != NULL && poolcb->magic == MAGIC_POOL)
	  {
	    args[0] = 5;
	    args[1] = (OSADDRESS)pcb_to_pid(poolcb);
	    args[2] = (OSADDRESS)poolcb->ext_func;
	    args[3] = (OSADDRESS)poolcb->ext_frag_prio;
	    args[4] = (OSADDRESS)poolcb->ext_frag_pp;
	    BKL_LOCK_RESTORE(msr);
	    return 5;
	  }
	BKL_LOCK_RESTORE(msr);
	return 0;
      }

    default:
      os_error(0, OSE_EBAD_PARAMETER, q);      
      h = NULL;
      break;
    }
  
  /*return old handler */
  return (OSADDRESS) h;
}

/*
**===========================================================================
**                           ose_set_pool_extender()
**===========================================================================
**
** Description: Register a pool extender.
**
** FIXME Return type??
*/


void
ose_set_pool_extender(PROCESS pool_id,
		      OSPOOLEXTENDH *extend_func,
		      void **nextfrag_handle,
		      OSPRIORITY ext_frag_prio)
{
  struct extender_info {
    OSADDRESS	members;
    PROCESS	pool;
    OSPOOLEXTENDH *func;
    void	**nextfrag_handle;
    OSPRIORITY	ext_frag_prio;
  } info;

  info.members = 5;
  info.pool = pool_id;
  info.func = extend_func;
  info.nextfrag_handle = nextfrag_handle;
  info.ext_frag_prio = ext_frag_prio;
  sysconf_67((OSADDRESS)&info, 3, 0);
}
  

/* This is the get_sysconf entry for setting a syserr test callout. */
static OSADDRESS
sysconf_68(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  SYSERR_HOOK h = test_err_handler;
  /* Remove ``not used args'' compiler warnings. */
  (void)q;
  (void)r;
  /* set new handler */
  test_err_handler = (SYSERR_HOOK)p;
  /*return old handler */
  return (OSADDRESS) h;
}

/* This is the get_sysconf entry for send_hook_restore. */
static OSADDRESS
sysconf_76(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Remove ``not used args'' compiler warnings. */
  (void)q;
  (void)r;

  zzkrn_send_hook_restore((union SIGNAL *)p);
  return 1;
}

/* This is the get_sysconf entry for zzkrn_restore_from. */
static OSADDRESS
sysconf_84(OSADDRESS sig, OSADDRESS pid, OSADDRESS not_used)
{
  (void)not_used;  /* Remove compiler warning. */
  
  return (OSADDRESS)zzkrn_restore_from((union SIGNAL *)sig, (PROCESS)pid);
}

/* This is the get_sysconf entry for zzkrn_send_w_sender. */
static OSADDRESS
sysconf_85(OSADDRESS sig, OSADDRESS from, OSADDRESS to)
{
  return (OSADDRESS)zzkrn_send_w_sender((union SIGNAL **)sig,
					(PROCESS)from,
					(PROCESS)to);
}

/* This is the get_sysconf entry for ose_install_c_exception_handler. */
static OSADDRESS
sysconf_86(OSADDRESS id, OSADDRESS handler, OSADDRESS attr)
{
   OSADDRESS ose_install_c_exception_handler(OSADDRESS,
                                             OSADDRESS,
                                             OSADDRESS);
   
   return (OSADDRESS)ose_install_c_exception_handler(id, handler, attr);
}

/* This is the get_sysconf entry for ose_shutdown_cpu. */
static OSADDRESS
sysconf_87(OSADDRESS entrypoint, OSADDRESS sp, OSADDRESS arg)
{
#ifdef OSPP   
   void ose_shutdown_cpu(void *, void *, void *);
   ose_shutdown_cpu((void*)entrypoint, (void*)sp, (void*)arg);
   return 1;
#else
   (void)entrypoint; (void)sp; (void)arg;
   return 0;
#endif   
}

/* Should not be used: This function used to return a function pointer to
 * ose_restore_core_dump. However, this function has been moved
 * to the hypervisor library.
 */
static OSADDRESS
sysconf_88(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  (void)p;
  (void)q;
  (void)r;
  
   return 0;
}

/* This is the get_sysconf entry to zzkrn_segment_migration_status. */
static OSADDRESS
sysconf_89(OSADDRESS pid, OSADDRESS set_mask, OSADDRESS set_value)
{
   OSADDRESS zzkrn_segment_migration_status(PROCESS pid,
                                            OSADDRESS set_mask,
                                            OSADDRESS set_value);
   return zzkrn_segment_migration_status((OSADDRESS)pid,set_mask,set_value);
}

/* This is the get_sysconf entry for ISR related calls. */
static OSADDRESS
sysconf_90(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  OSADDRESS retval = 0;

  switch (p >> 16)
    {
    case 0:
      retval = zzkrn_create_isr(p, ((struct isr_create_data *)r)->owner,
                                (OSISR *)q, ((struct isr_create_data *)r)->arg);
      break;
    case 1:
      zzkrn_kill_isr(q);
      break;
    case 2:
      retval = zzkrn_create_interrupt_hook(p & 0xffff,
                                           ((struct isr_create_data *)r)->owner,
                                           (OSIH *)q,
                                           ((struct isr_create_data *)r)->arg);
      break;
    case 3:
      zzkrn_kill_interrupt_hook(q);
      break;
    case 4:
      zzkrn_enable_vector(q);
      break;
    case 5:
      zzkrn_disable_vector(q);
      break;
    case 6:
      retval = get_vectorcb_cpu(q);
      break;
    case 7:
       /* Used to be zzkrn_signal_fsem_from_isr, may be reused. */
      break;
#ifdef OSE_SMP      
    case 8:
      zzkrn_bind_vector(q, r);
      break;
    case 9:
      zzkrn_unbind_vector(q, r);
      break;
    case 10:
      retval = (OSADDRESS)&zzkrn_evt_notify_from_isr;
      break;
    case 11:
      handlerp.evt_interrupt_generate = (void (*)(cpuid_t))q;
      retval = 1;
      break;
#endif      
    }
  
  return retval;
}

/* This is the get_sysconf entry for zzkrn_receive_fsem. */
static OSADDRESS
sysconf_91(OSADDRESS timeout, OSADDRESS sigsel, OSADDRESS count)
{
  return (OSADDRESS)zzkrn_receive_fsem((OSTIME)timeout,
                                       (SIGSELECT *)sigsel,
                                       (OSFSEMVAL)count);
}

/* This is the get_sysconf entry for zzkrn_receive_fsem. */
static OSADDRESS
sysconf_92(OSADDRESS pid, OSADDRESS q, OSADDRESS r)
{
  (void)q;
  (void)r;
  return (OSADDRESS)zzkrn_signal_fsem_from_isr((PROCESS)pid);
}

/* This is the get_sysconf entry for ose_uninstall_c_exception_handler. */
static OSADDRESS
sysconf_96(OSADDRESS id, OSADDRESS not_used_1, OSADDRESS not_used_2)
{
   void ose_uninstall_c_exception_handler(OSADDRESS id);
   
   (void)not_used_1;
   (void)not_used_2;
   
   ose_uninstall_c_exception_handler(id);
   
   return 0;
}

/*
** This is the get_sysconf entry for returning a pointer to the 64-bit
** ticks variable.
*/
static OSADDRESS
zzkrn_get_ticks_addr(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  (void)(p + q + r); /* Go figure! */
  return (OSADDRESS)&zzuu.ticks;
}

/* This is the get_sysconf entry for handling fast path functions.
 */
static OSADDRESS
sysconf_101(OSADDRESS fcode, OSADDRESS q, OSADDRESS r)
{
#ifdef OSE_SMP
  OSADDRESS old;

  switch (fcode)
    {
    case 1:
      /* Set new receive hw poll handler. Set to NULL to remove. */
      old = (OSADDRESS)zzcpu[q].hw_receive_poller;
      zzcpu[q].hw_receive_poller = (void(*)(void))r;

      /* Return the previous poll handler. */
      return old;
    case 2:
      /* Read the receive hw poll handler. */
      return (OSADDRESS)zzcpu[q].hw_receive_poller;
    case 3:
      /* Set fsend hook. The hook decides if the signal should be queued
       * or dropped. */
      return (OSADDRESS)zzkrn_set_fsend_hook((int)q, (OSFSENDHOOK *)r);
    }
#else
  (void)(fcode + q + r);
#endif
  return 0;
}

/* 
** This is the get_sysconf entry block state hook installation.
*/
static OSADDRESS
sysconf_102(OSADDRESS bid, OSADDRESS func_a, OSADDRESS handle_a)
{
  struct statehookcb *hookcb;
  struct _block *bcb;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_GET_UID))
    return 0;
#endif
  /* Locate the block descriptor. */
  /* Read instance number and check the block id. Report error if invalid. */
  bcb = check_bid(bid, OSE_GET_UID);
  if(bcb == NULL)
    return 0;

  hookcb = (struct statehookcb *)sys_alloc(sizeof(struct statehookcb),
					   STATEHOOK_MAGIC);
  /* Lock the dispatcher. */
  LOCK_SWAP();

  /* Check that the block is still present. */
  if(pcb_to_pid(bcb) != bid)
    {
      UNLOCK_SWAP();
      free_buf_type(struct statehookcb *, &hookcb);
      return 0;
    }

  /* Donate the cb to the system and fill in cb data. */
  change_sig_owner(sig_to_adm(hookcb), NULL);
  hookcb->func = (OSSTATEHOOK *)func_a;
  hookcb->owner = zzcur_p->live_pid;
  hookcb->victim = bid;
  hookcb->handle = (void *)handle_a;

  /* Link into the bcb and return. */
  BKL_LOCK();
  hookcb->next = bcb->statehook_list;
  bcb->statehook_list = hookcb;
  BKL_UNLOCK_KEEP_INT_MASK();
  LOCKED_UNLOCK_SWAP();
  return (OSADDRESS)hookcb;
}

/* 
** This is the get_sysconf entry block state hook uninstallation.
*/
static OSADDRESS
sysconf_103(OSADDRESS bid, OSADDRESS hookref_a, OSADDRESS q)
{
  struct statehookcb **hookref = (struct statehookcb **)hookref_a;
  struct statehookcb *hookcb;
  struct _block *bcb;

  (void) q;

#if ERROR_CHECKS > 0
  /* Check that the caller is not an interrupt or timer process. */
  if(check_nonint_caller(OSE_GET_UID))
    return 0;
#endif
  /* Locate the block descriptor. */
  /* Read instance number and check the block id. Report error if invalid. */
  bcb = check_bid(bid, OSE_GET_UID);
  if(bcb == NULL)
    return 0;

  if(!hookref)
    return 0;

  hookcb  = *hookref;
  if(!hookcb)
    return 0;

  LOCK_SWAP();
  if(hookcb->magic != STATEHOOK_MAGIC ||
     hookcb->victim != bid ||
     pcb_to_pid(bcb) != bid)
    {
      UNLOCK_SWAP();
      return 0;
    }

  /* Unlink the cb from the block. */
  BKL_LOCK();
  {
    struct statehookcb *cb = bcb->statehook_list;
    if(cb == hookcb)
      {
	bcb->statehook_list = hookcb->next;
      }
    else
      {
	for(;;)
	  {
	    if(!cb)
	      {
		/* Should really not happen, unlinked twice? */
		BKL_UNLOCK();
		UNLOCK_SWAP();
		return 0;
	      }
	    
	    if(cb->next == hookcb)
	      {
		/* Unlink the hookcb from the block. */
		cb->next = hookcb->next;
		break;
	      }
	    cb = cb->next;
	  }
      }
  }
  BKL_UNLOCK();
  UNLOCK_SWAP();
  sys_free_buf_type(struct statehookcb *, &hookcb);
  return 1;
}

static OSADDRESS
sysconf_104(OSADDRESS cpu, OSADDRESS base, OSADDRESS size)
{
#ifdef OSE_SMP
  LOCKSTATE msr;

  base &= ~KRN_EVT_TINY_MASK;
  
  LOCK_SAVE(msr);
  send_krn_event((cpuid_t)cpu, KRN_EVT_FLUSH_TLB, size, (void*)base);
  LOCK_RESTORE(msr);
#else
  (void)(cpu+base+size);
#endif

  return 0;
}


static void
init_sysconf(void)
{
  int i;
  for(i = MAX_SYSCONF - 1; i; i--)
    {
      sysconf[i] = noop_sysconf;
      sysconf_privil[i] = (U8) 0; /* Disallow all calls from user mode. */
    }

  sysconf[0] = sysconf_0;
  sysconf_privil[0] = (U8) 1; /* Allow user mode calls. */
  sysconf[1] = sysconf_1;
  sysconf_privil[1] = (U8) 1; /* Allow user mode calls. */
  sysconf[2] = sysconf_2;
  sysconf[3] = sysconf_3;
  sysconf[4] = sysconf_4;
  sysconf_privil[4] = (U8) 1; /* Allow user mode calls. */
  sysconf[20] = sysconf_20;
  sysconf_privil[20] = (U8) 1; /* Allow user mode calls. */
  sysconf[21] = sysconf_21;
  sysconf[22] = sysconf_22;
  sysconf_privil[22] = (U8) 1; /* Allow user mode calls. */
  sysconf[23] = sysconf_23;
  sysconf_privil[23] = (U8) 1; /* Allow user mode calls. */
  sysconf[24] = sysconf_24;
  sysconf_privil[24] = (U8) 1; /* Allow user mode calls. */
  sysconf[25] = sysconf_25;
  sysconf_privil[25] = (U8) 1; /* Allow user mode calls. */
  sysconf[26] = sysconf_26;
  sysconf_privil[26] = (U8) 1; /* Allow user mode calls. */
  sysconf[27] = sysconf_27;
  sysconf_privil[27] = (U8) 1; /* Allow user mode calls. */
  sysconf[28] = sysconf_28;
  sysconf_privil[28] = (U8) 1; /* Allow user mode calls. */
  sysconf[29] = sysconf_29;
  sysconf_privil[29] = (U8) 1; /* Allow user mode calls. */
  sysconf[30] = sysconf_30;
  /* The LDM registers code 31. */
  sysconf[32] = sysconf_32;
  sysconf[33] = sysconf_33;
  /* Codes 34-44 are defined by the target interface. */
  sysconf[45] = sysconf_45;
  sysconf_privil[45] = (U8) 1; /* Allow user mode calls. */
  sysconf[46] = sysconf_46;
  sysconf[47] = sysconf_47;
  sysconf_privil[47] = (U8) 1; /* Allow user mode calls. */
  sysconf[48] = sysconf_48;
  sysconf_privil[48] = (U8) 1; /* Allow user mode calls. */
  sysconf[49] = sysconf_49;
  sysconf_privil[49] = (U8) 1; /* Allow user mode calls. */
  sysconf[50] = sysconf_50;
  sysconf_privil[50] = (U8) 1; /* Allow user mode calls. */
  sysconf[51] = sysconf_51;
  sysconf_privil[51] = (U8) 1; /* Allow user mode calls. */
  sysconf[52] = sysconf_52;
  sysconf_privil[52] = (U8) 1; /* Allow user mode calls. */
  sysconf[53] = sysconf_53;
  sysconf_privil[53] = (U8) 1; /* Allow user mode calls. */
  sysconf[54] = sysconf_54;
  sysconf_privil[54] = (U8) 1; /* Allow user mode calls. */
  sysconf[55] = sysconf_55;
  sysconf_privil[55] = (U8) 1; /* Allow user mode calls. */
  sysconf[56] = sysconf_56;
  sysconf_privil[56] = (U8) 1; /* Allow user mode calls. */
  sysconf[57] = sysconf_57;
  sysconf[58] = sysconf_58;
  sysconf_privil[58] = (U8) 1; /* Allow user mode calls. */
  sysconf[59] = sysconf_59;
  sysconf_privil[59] = (U8) 1; /* Allow user mode calls. */
  sysconf[60] = sysconf_60;
  sysconf_privil[60] = (U8) 1; /* Allow user mode calls. */
  sysconf[61] = sysconf_61;
  sysconf_privil[61] = (U8) 1; /* Allow user mode calls. */
  sysconf[62] = sysconf_62;
  sysconf_privil[62] = (U8) 1; /* Allow user mode calls. */
  sysconf[63] = sysconf_63;
  sysconf_privil[63] = (U8) 1; /* Allow user mode calls. */
  sysconf[64] = sysconf_64;
  sysconf_privil[64] = (U8) 1; /* Allow user mode calls. */
  sysconf[65] = sysconf_65;
  sysconf_privil[65] = (U8) 1; /* Allow user mode calls. */
  sysconf[66] = sysconf_66;
  sysconf_privil[66] = (U8) 1; /* Allow user mode calls. */
  sysconf[67] = sysconf_67;
  sysconf[68] = sysconf_68;  
  sysconf[69] = sysconf_69;
  sysconf_privil[69] = (U8) 1; /* Allow user mode calls. */
  sysconf[70] = sysconf_70;
  sysconf_privil[70] = (U8) 1; /* Allow user mode calls. */
  sysconf[71] = sysconf_71;
  sysconf_privil[71] = (U8) 1; /* Allow user mode calls. */
  sysconf[72] = sysconf_72;
  sysconf_privil[72] = (U8) 1; /* Allow user mode calls. */
  sysconf[73] = sysconf_73;
  sysconf_privil[73] = (U8) 1; /* Allow user mode calls. */
  sysconf[74] = sysconf_74;
  sysconf[75] = sysconf_75;
  sysconf[76] = sysconf_76;
  sysconf[77] = zzose_set_report_val_sysconf_77;
  sysconf_privil[77] = (U8) 1; /* Allow user mode calls. */
  sysconf[78] = zzose_create_report_sysconf_78;
  sysconf_privil[78] = (U8) 1; /* Allow user mode calls. */
  sysconf[79] = sysconf_79;
  sysconf[80] = sysconf_80;
  sysconf[81] = sysconf_81;
  sysconf[82] = sysconf_82;
  sysconf_privil[82] = (U8) 1; /* Allow user mode calls. */
  sysconf[83] = sysconf_83;
  sysconf_privil[83] = (U8) 1; /* Allow user mode calls. */
  sysconf[84] = sysconf_84;
  sysconf_privil[84] = (U8) 1; /* Allow user mode calls. */
  sysconf[85] = sysconf_85;
  sysconf_privil[85] = (U8) 1; /* Allow user mode calls. */
  sysconf[86] = sysconf_86;
  sysconf[87] = sysconf_87;
  sysconf[88] = sysconf_88;
  sysconf[89] = sysconf_89;  
  sysconf[90] = sysconf_90;
  sysconf[91] = sysconf_91;
  sysconf_privil[91] = (U8) 1; /* Allow user mode calls. */
  sysconf[92] = sysconf_92;
  sysconf_privil[92] = (U8) 1; /* Allow user mode calls. */
  sysconf[93] = zzkrn_get_ticks_addr;
  sysconf_privil[93] = (U8) 1; /* Allow user mode calls. */
  sysconf[94] = sysconf_94;
  sysconf_privil[94] = (U8) 1; /* Allow user mode calls. */
  sysconf[95] = sysconf_95;
  sysconf_privil[95] = (U8) 1; /* Allow user mode calls. */
  sysconf[96] = sysconf_96;
  sysconf[97] = sysconf_97;
  sysconf[98] = sysconf_98;
  sysconf_privil[98] = (U8) 1; /* Allow user mode calls. */
  sysconf[99] = sysconf_99;
  sysconf[100] = sysconf_100;
  sysconf_privil[100] = (U8) 1; /* Allow user mode calls. */
  sysconf[101] = sysconf_101;
  sysconf[102] = sysconf_102;
  sysconf[103] = sysconf_103;
  sysconf[104] = sysconf_104;
  sysconf[107] = sysconf_107;
  sysconf_privil[107] = (U8) 1; /* Allow user mode calls. */
}

OSADDRESS
get_sysconf(OSADDRESS f, OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
  /* Check for function code out of range and return an error code. */
  if(f >= MAX_SYSCONF)
    return (OSADDRESS) 0;
    
  /* Allow user mode calls only if explictly enabled. */
  if(!sysconf_privil[f] && !zzcur_p->block->supervisor_mode)
    return (OSADDRESS) 0;

  /* Call the requested function. */
  return (*sysconf[f])(p, q, r);
}

/*
**===========================================================================
** 25.5  System debugger utilities.
**===========================================================================
*/

/*
** The ldm sets the master debug status flag.
**
** If the state flag is zero (default), then debugger hooks
** are disabled and the kernel does not perform any system
** debugger services.
**
** If the state flag is non zero, then debugger hooks are called
** as specified by the debug_flags field for each process.
**
** Interrupt disable status is preserved by the function.
*/

void zzset_debug_status(OSBOOLEAN state)
{
  /* Update the debugger master enable flag. */
  zzuu.debugger_enabled = state;
}

/*
** rmm sets the state of the trace all swaps functionality.
*/

void zzset_trace_swaps_enabled(OSBOOLEAN state)
{
   zzuu.trace_all_swaps = state;
}

/*
** The ldm enables or disables the tick debugger hook
** named zzprof_tickhook().
**
** If the state flag is zero (default), then debugger hook is
** disabled and the kernel does not perform any system
** debugger services during the tick() call.
**
** If the state flag is non zero, then debugger hook is called
** for each tick() executed.
**
** Interrupt disable status is preserved by the function.
*/

void
zzset_tickhook_ldmstatus(U32 state)
{
  /* Update the hook enable flag. */
  zzuu.tickhook_enabled = (U8)state;
}

/*
** The ldm enables or disables the interrupt process swap hooks
** named zzintswap_in() and zzintswap_out().
**
** If the state flag is zero (default), then debugger hooks
** are disabled and the kernel does not perform any system
** debugger services during interrupt process swaps.
**
** If the state flag is non zero, then debugger hooks are called
** for each interrupt process executed.
**
** If the state flag is > 1, then debugger hooks are called
** for each timer process as well.
**
** Interrupt disable status is preserved by the function.
*/

void
zzset_inthook_ldmstatus(U32 state)
{
  /* Update the hook enable flag. */
  zzuu.inthook_enabled = (U8)state;

  /* Update the target interface copy as well. */
#if ENABLE_INTERRUPT_PROFILING > 0
#if LATE_INIT_INT > 0
  zzset_inthook_status(state);
#endif
#endif
}

/*
** The target interface calls interrupt process swap hooks
** when using the LATE_INIT_INT implementation.
** These hooks are called from within the kernel as well.
**
** Note that these functions can be interrupted at any time by a
** higher priority interrupt.
**
** See also: pose.h
*/

void
zzintswap_in(void *newpcb, void *oldpcb)
{
#if OSE_DEBUG == 0
  (void)newpcb; (void)oldpcb;
#else
  struct _pcb *new = (struct _pcb *) newpcb;

  if(zzuu.inthook_enabled & 0x03)
    {
      /* Call the swap in hook in the profiler, indicating an interrupt process. */
      zzprof_dbghook((OSPRIORITY) 34, pcb_to_debug_pcb(new));
    }

  /* Need to support sampled per cpu profiler as well. @@??*/
  /* Need to support trace hook in the debugger as well. @@??*/

  if(zzuu.inthook_enabled & 0x0c)
    {
      struct _pcb *old = (struct _pcb *) oldpcb;
      
      /* check for trace swap in request. */
      if(likely(zzuu.trace_all_swaps))
        {
	  zzfast_swap_trace(pcb_to_debug_pcb(old),
			    pcb_to_debug_pcb(new));
        }
      else if((new->debug_flags | old->debug_flags) & 0x04)
	zzswapin_dbghook(pcb_to_debug_pcb(new),
			 pcb_to_debug_pcb(old));
    }
#endif
}

void
zzintswap_out(void *newpcb, void *oldpcb)
{
#if OSE_DEBUG == 0
  (void)newpcb; (void)oldpcb;
#else
  struct _pcb *new = (struct _pcb *) newpcb;


  if(zzuu.inthook_enabled & 0x03)
    {
      OSPRIORITY pri = new->priority;

      /* We need to find the previous' process priority. */
      if(pri > 32)
	pri = (OSPRIORITY) 34; /* Interrupt or timer process */
      else
	{ 
	  /*
	  ** Non-interrupt or background process.
	  **
	  ** Check for idle process, pcb passed does not include the idle
	  ** process, but zzcur_ni indicates if the system was idle when
	  ** the interrupt was taken
	  */
	  if(zzcur_ni == zzww.idle_pcb)
	    {
	      pri = (OSPRIORITY) 33;
	      new = zzww.idle_pcb;
	    }
	}
      zzprof_dbghook((OSPRIORITY) pri, pcb_to_debug_pcb(new));
    }

  /* Need to support sampled per cpu profiler as well. @@??*/
  /* Need to support trace hook in the debugger as well. @@??*/

  if(zzuu.inthook_enabled & 0x0c)
    {
      struct _pcb *old = (struct _pcb *) oldpcb;
      
      /* check for trace swap in request. */
      if(likely(zzuu.trace_all_swaps))
        {
	  zzfast_swap_trace(pcb_to_debug_pcb(old),
			    pcb_to_debug_pcb(new));
        }
      else if((new->debug_flags | old->debug_flags) & 0x04)
	zzswapin_dbghook(pcb_to_debug_pcb(new),
			 pcb_to_debug_pcb(old));
    }
#endif
}

/*
** The ldm obtains a pointer to the debugger reserved fields for the
** specified process.
**
** Returns a pointer to the structure or NULL if the process
** has been killed. The ldm must make sure that invalid process id's are
** never presented to the kernel.
**
** Interrupts must be disabled or the dispatcher must be locked
** while this function is called. I.e the process must not be killed during
** this period.
*/

struct debug_pcb *
zzget_debugpcb(PROCESS pid)
{
  register struct _pcb *pcb;

  /*
   * Get pcb and check instance number. We also check magic, to prevent
   * match on non-process objects. We must also return pointers for processes
   * under construction; these still have MAGIC_KILLED set.
   */
  pcb = pid_to_pcb(pid);
  if((pcb->magic & ~MAGIC_KILLED) == MAGIC_PCB &&
     pcb_to_pid(pcb) == pid)
    {
      return pcb_to_debug_pcb(pcb);
    }
  /* return NULL if the object did not match. */
  return NULL;
}

/*
*****************************************************************************
** 26 PROCESS ENTRYPOINTS.
*****************************************************************************
*/

/*
**===========================================================================
** 26.1  The tick daemon.
**===========================================================================
*/

/*
** The system tick daemon.
**
** The tick system call relies on being executed on interrupt level.
** If this is not true, we must simulate an interrupt by causing
** a wakeup call to a special internal interrupt process, which
** in turn can run the tick call.
** The wakeup flag is not set when this wakeup call is issued.
**
** We don't use the OS_PROCESS macro here. That would be dangerous since
** this is an interrupt process and we included ose.h.
** This should work unless OS_PROCESS is redefined in ose_i.h.
*/

/* OS_PROCESS(tickd) */
static void tickd(void)
{
  /* Check that we weren't activated by a signal sent to us. */
#if ERROR_CHECKS > 1
  if(unlikely(zzcur_p->mailbox))
    {
      static const SIGSELECT sig_sel[] = {0};
      union SIGNAL *sig = receive(sig_sel);
      /* Report an error and free the buffer if so. */
      os_error(1, OSE_ESPURIOUS_SYSTEM_SIGNAL, (OSERRCODE) sender(&sig));	/* Non fatal system error. */
      free_buf(&sig);
      return;
    }
#endif

  /* Run the system timer. */
  tick();
}

/*
**===========================================================================
** 26.2  The hunt daemon.
**===========================================================================
*/

/*
** The hunt daemon performs the following tasks:
**
** When a new process is created, it is inserted into the process list.
** If the name of the new process matches any pending hunt request, then
** those requests are honored.
**
** When a process is assigned a new user id, pending hunt requests that match
** the name of the process are honored. No linkhandler calls are generated.
**
** When a new linkhandler is created it is inserted into the list of
** linkhandlers. The new linkhandler is handed any pending hunt requests
** matching the new linkhandler.
**
** When a process is deleted, it is removed from the process list.
** Pending hunt requests issued by or for the killed process are discarded.
** The hunt daemon checks if the process was a linkhandler.
** The pcb is freed.
**
** When a linkhandler is deleted, it is removed from the linkhandler list.
**
** When a hunt request is issued, the process list is searched for a matching name.
** If not found, The linkhandler list is searched and all matching linkhandlers
** are asked for the process. While doing this, only hunt requests and new user
** number assignments are evaluated, and only those that don't call upon
** a linkhandler.
** Those requiring linkhandler service are stored in an internal fifo.
** (It might be possible to keep one outstanding request for each linkhandler
** by using flags in the linkhandler list.)
**
** We don't use the OS_PROCESS macro here. The reason for this is that
** we want the symbol to be static, and OS_PROCESS would make it global.
*/

/* OS_PROCESS(huntd) */
static void huntd(void)
{
  static union SIGNAL *sigr;
  static const SIGSELECT sig_sel[] = {0};

  for(;;)
    {
      sigr = receive(sig_sel);
      switch((int) sigr->sig_no)
	{
	  /* Execute a hunt() or hunt_from() system call. */
	case HUNT_REQUEST:
	  {
	    (void)free_attach(0);
	    huntd_hunt_request(sigr);
	    break;
	  }
	  /* Execute an assign_linkhandler() system call. */
	case NEW_LINKHANDLER:
	  {
	    (void)free_attach(0);
	    huntd_assign_linkhandler(sigr, sender(&sigr));
	    break;
	  }
	  /* A new process or block was created. */
	case NEW_PROCESS:
	  {
            struct _pcb *pcb = pid_to_pcb(sigr->new_process.pid);
            
            if ((pcb->magic & ~MAGIC_KILLED) != MAGIC_BCB)
	      huntd_create_process(sigr);
            else
              huntd_create_block(sigr);            
	    break;
	  }
	  /* A process or block was killed. */
	case DELETE_PROCESS:
	  {
            struct _pcb *pcb = pid_to_pcb(sigr->new_process.pid);

            if ((pcb->magic & ~MAGIC_KILLED) != MAGIC_BCB)
              huntd_kill_proc(sigr);
            else
              huntd_kill_block(sigr);                           
	    break;
	  }
	  /* A new user id was assigned to a process. */
	case NEW_UID:
	  {
	    huntd_set_uid(sigr);
	    break;
	  }
	  /* An unknown signal was received. Report an error and drop it. */
	default:
	  {
	    os_error(1, OSE_ESPURIOUS_SYSTEM_SIGNAL, (OSERRCODE) sender(&sigr));	/* Non fatal system error. */
	    free_buf(&sigr);
	    break;
	  }
	}
    }
}

/*
**===========================================================================
** 26.3  The system daemon.
**===========================================================================
*/

/*
** The system daemon performs general housekeeping for operations like:
** When a process is created.
** When a process is killed.
** When a block is killed.
**
** Periodic garbage collection of the system pools whenever required.
**
** We don't use the OS_PROCESS macro here. The reason for this is that
** we want the symbol to be static, and OS_PROCESS would make it global.
*/

/* OS_PROCESS(sysd) */
void 
sysd(void)
{
  extern OSENTRYPOINT _zzmain;
  PROCESS main_block;
  PROCESS zzmain_;
  struct stkcb *stack;
  static OSTIME one_second;
  OSPRIORITY huntd_pri;

  SMP_CODE(init_per_cpu());

  /* Initialize static variables. */
  zzuu.garbage_collection_enabled = (OSBOOLEAN) 0;
  one_second = 1000000u / (OSTIME)local_sysctlblk.tick_len;

  /* Create the tick daemon. */
  /*?? We should check HUNTD_PRIORITY et all for bgd process if configured pri = 32. */
  zzuu.tickd_ =
    sysd_create_process(
			OS_INT_PROC,
			"ose_tickd",
			tickd,
			NULL,
			(OSADDRESS) MIN_INTERRUPT_STACKSIZE,	/* Make sure we get an initial int stack. */
			(OSPRIORITY) ( (local_sysctlblk.table_extended >= 2) ?
				       local_sysctlblk.tickd_priority : 31),
			(OSTIME) 0,
			pcb_to_pid(zzuu.bcb_list),
			NULL,
			current_process(),
			(U32) 0,
			NULL,
			NULL
			);

  /* Create the hunt daemon. */
  /* Already locked by sysd_create_process() LOCK_SWAP(); */
  stack = stack_alloc(&system_pool, HUNTD_STACKSIZE + local_sysctlblk.superv_stack_size);
  UNLOCK_SWAP();
#if ERROR_CHECKS > 0
  memset((char *) stack->limit, END_MARK, (size_t) ((char *)stack->top - (char *)stack->limit));
#else
  memset((char *) stack->limit, END_MARK, (size_t) 4);
#endif
  {
#if ERROR_CHECKS > 0
    OSPRIORITY sysd_pri = get_pri(current_process());
#endif
#ifdef HUNTD_PRIORITY
    huntd_pri = HUNTD_PRIORITY;
#else
    huntd_pri = (OSPRIORITY) 10;
#endif
    /* Get the zzmain priority configured by the user. */
    if(local_sysctlblk.table_extended >= 1)
      huntd_pri = (OSPRIORITY)local_sysctlblk.huntd_priority;

    /* Check that the huntd priority is higher than the sysd. */
#if ERROR_CHECKS > 0
    if(huntd_pri >= sysd_pri)
      {
	/* No, report an error if the huntd priority is too low. */
	os_error(0, OSE_EBAD_HUNT_PRI | OSE_START_OSE, (OSERRCODE) 0);	/* Non fatal. */
	/* We also need to set things right. */
	if(!sysd_pri)
	  {
	    sysd_pri = (OSPRIORITY) 1;
	    set_pri(sysd_pri);
	  }
	huntd_pri = (OSPRIORITY)(sysd_pri - 1);
      }
#endif

    zzuu.huntd_ = 
      sysd_create_process(
			  OS_PRI_PROC,
			  "ose_huntd",
			  huntd,
			  stack,
			  (OSADDRESS) HUNTD_STACKSIZE,
			  huntd_pri,
			  (OSTIME) 0, /* No timeslicing. */
			  pcb_to_pid(zzuu.bcb_list),
			  NULL,
			  current_process(),
			  (U32) 0,
			  NULL,
			  NULL
			  );
    /* Unlock the dispatcher since sysd_create_process() left it locked. */
    UNLOCK_SWAP();
  }
  {
    static struct OS_redir_entry redir;
    redir.sig = (SIGSELECT) 1;
    redir.pid = zzuu.huntd_;
    zzuu.huntd2_ = 
      sysd_create_process(
			  OS_PHANTOM,
			  "ose_huntd",
			  huntd,
			  NULL,
			  (OSADDRESS) HUNTD_STACKSIZE,
			  (OSPRIORITY) 0,
			  (OSTIME) 0,
			  pcb_to_pid(zzuu.bcb_list),
			  &redir,
			  (PROCESS) 0,
			  (U32) 0,
			  NULL,
			  NULL
			  );
    /* Unlock the dispatcher since sysd_create_process() left it locked. */
    UNLOCK_SWAP();
  }


  /* Start the hunt daemon. */
  zzuu.huntd2_pcb = pid_to_pcb(zzuu.huntd2_);
  start(zzuu.huntd_);

#ifdef OSE_SMP
  /* Create the sysd_lite daemon for each cpu. */
  sysd_create_sysd_lite(huntd_pri);

  /* Start the sysd_lite instance for the current cpu. */
  start(zzww.sysd_lite_);
#endif

#if OSE_DEBUG > 0
  /* Create the local debugger daemon. */
  {
  extern OSENTRYPOINT zzose_rmm;
  union SIGNAL *sigh;

  LOCK_SWAP();
  stack = stack_alloc(&system_pool, HUNTD_STACKSIZE + local_sysctlblk.superv_stack_size);
  UNLOCK_SWAP();
#if ERROR_CHECKS > 0
  memset((char *) stack->limit, END_MARK, (size_t) ((char *)stack->top - (char *)stack->limit));
#else
  memset((char *) stack->limit, END_MARK, (size_t) 4);
#endif
  {
    OSPRIORITY ldm_pri = (OSPRIORITY) 1;
    /* Get the ldm priority configured by the user. */
    if(local_sysctlblk.table_extended >= 1)
      ldm_pri = (OSPRIORITY)local_sysctlblk.ldm_priority;

    zzuu.ldm_ = 
      sysd_create_process(
			  OS_PRI_PROC,
			  "ose_monitor",
			  zzose_rmm,
			  stack,
			  (OSADDRESS) HUNTD_STACKSIZE,
			  ldm_pri,
			  (OSTIME) 0, /* No timeslicing. */
			  pcb_to_pid(zzuu.bcb_list),
			  NULL,
			  current_process(),
			  (U32) 0,
			  NULL,
			  NULL
			  );
  }

  /* Unlock the dispatcher since sysd_create_process() left it locked. */
  UNLOCK_SWAP();

  /* Ask the ldm to initialize. */
  sigh = alloc(sizeof(struct start_modeflag), START_MODEFLAG);
  sigh->start_modeflag.mode = zzuu.start_mode;
  send(&sigh, zzuu.ldm_);

  /* Make this process known to the hunt daemon. */
  sigh = alloc(sizeof(struct new_process), NEW_PROCESS);
  sigh->new_process.pid = zzuu.ldm_;
  sigh->new_process.block = pcb_to_pid(zzuu.bcb_list);  
  send(&sigh, zzuu.huntd_);
}
#endif

  /* Create the application block. */
  {
    register char *name = (char *) alloc((OSBUFSIZE) 5, (SIGSELECT) 0);
    strcpy(name, (const char *) "main");
    main_block = create_block("main",(OSUSER)0,(OSBOOLEAN)0,(PROCESS)0,(OSBOOLEAN)1);

    /* Publish this as the main_block of the system segment. */
    system_segment.main_block = main_block;

    /* Create and start the boot application. */
    LOCK_SWAP();
    stack = stack_alloc(&system_pool, local_sysctlblk.zzmain_stacksize +
			local_sysctlblk.superv_stack_size);
    UNLOCK_SWAP();
#if ERROR_CHECKS > 0
  memset((char *) stack->limit, END_MARK, (size_t) ((char *)stack->top - (char *)stack->limit));
#else
  memset((char *) stack->limit, END_MARK, (size_t) 4);
#endif

    {
#ifdef ZZMAIN_PRIORITY
    OSPRIORITY main_pri = ZZMAIN_PRIORITY;
#else
    OSPRIORITY main_pri = (OSPRIORITY) 16;
#endif
    /* Get the zzmain priority configured by the user. */
    if(local_sysctlblk.table_extended >= 1)
      main_pri = (OSPRIORITY)local_sysctlblk.main_priority;

      zzmain_ =  
	sysd_create_process(
			    OS_PRI_PROC,
			    name,
			    _zzmain,
			    stack,
			    local_sysctlblk.zzmain_stacksize,
			    main_pri,
			    (OSTIME) 0, /* No timeslicing. */
			    main_block,
			    NULL,
			    current_process(),
			    (U32) 0,
			    NULL,
			    NULL
			    );
      /* Unlock the dispatcher since sysd_create_process() left it locked. */
      UNLOCK_SWAP();
    }

    /* Supervise the zzmain process so an error can be reported if it dies. */
    (void) attach(NULL, zzmain_);

    /* Need to give zzmain to huntd so that it can be killed. */
    {
      union SIGNAL *sigh;
      sigh = alloc(sizeof(struct new_process), NEW_PROCESS);
      sigh->new_process.pid = zzmain_;
      send(&sigh, zzuu.huntd_);
    }
  }

#if OSE_DEBUG > 0
  /* Start the ldm and allow it to receive the start_modeflag signal. */
  start(zzuu.ldm_);
#else
  /*
  ** Set the kernel stable flag to indicate to post mortem tools
  ** that the kernel is up and running.
  */
  zzuu.kernel_initialized = MAGIC_KERNEL_TYPE;

  /* Start the application directly if no ldm is present. */
  start(zzmain_);
#endif

  for(;;)
    {
      static union SIGNAL *sigr;
      static const SIGSELECT sig_sel[] = {0};

      /*
      ** Do a garbage collection each second for as long as there are
      ** abandoned pools around.
      */
      if(zzuu.garbage_collection_enabled)
	sigr = receive_w_tmo(one_second, sig_sel);
      else
	sigr = receive(sig_sel);

      if(zzuu.garbage_collection_enabled)
	{
	  (void)free_attach(0);
	  zzuu.garbage_collection_enabled = sysd_check_free_pool();
	}

      if(sigr != NIL)
	{
	  switch((int) sigr->sig_no)
	    {
	      /* Execute a create_process() system call. */
	    case NEW_PROCESS:
	      {
		(void)free_attach(0);
		sysd_new_process(sigr);
		break;
	      }
	      /* Execute a kill_proc() system call. */
	    case DELETE_PROCESS:
	      {
		register PROCESS pid = sigr->delete_process.pid;
		free_buf(&sigr);
		sysd_delete_process(pid);
		break;
	      }
	      /* Start garbage collection of pools. */
	    case START_GARBAGE_COLLECTOR:
	      {
		zzuu.garbage_collection_enabled = (OSBOOLEAN) 1;
		free_buf(&sigr);
		break;
	      }

	      /* Clean up any old entries in the breakpoint list. */
	      /* ===DEPRECATED=== */
	    case KILL_BREAK:
	      {
		free_buf(&sigr);
		break;
	      }

#if OSE_DEBUG > 0
	      /* Start the application when the ldm finished initializing. */
	    case START_MODEFLAG:
	      {
		/* Enable the create debugger hook. */
		zzuu.sysprocs_created = (OSBOOLEAN) 1;

		/*
		** Set the kernel stable flag to indicate to post mortem tools
		** that the kernel is up and running.
		*/
		zzuu.kernel_initialized = MAGIC_KERNEL_TYPE;

		/* start the application. */
		start(zzmain_);
		free_buf(&sigr);
		break;
	      }
#endif
	      /* Kill an entire segment. */
	    case KILL_SEG_REQ:
	      {
		sysd_do_kill_seg_req(sigr);
		free_buf(&sigr);
		break;
	      }

	      /* The zzmain process was killed. */
	    case OS_ATTACH_SIG:
	      {
		if(sender(&sigr) == zzmain_)
		  {
		    os_error(0, OSE_EKILLED_STATIC_PROCESS, (OSERRCODE) 0);	/* Non fatal. */
		  }
		free_buf(&sigr);
		break;
	      }

	      /* Bind a process or block to a cpu. */
#if defined(OSE_SMP_MAX_CPUS) && (OSE_SMP_MAX_CPUS > 1)
	    case SMP_BIND_PROCESS:
	      {
		sigr->smp_bind_process.status =
		  sysd_bind_process(sigr->smp_bind_process.pid,
				    sigr->smp_bind_process.cpu);
		send(&sigr, sender(&sigr));
		break;
	      }
#endif
	      /* Distribute attach signals for a dead segment. */
	    case KILL_ATTACH:
	      {
                struct segcb *segcb = (struct segcb *)pid_to_pcb(sigr->kill_attach.pid);

                /* Remove all ISRs that uses this segment as owner. */
                process_isr_attach_list(segcb->isr_list);
                 
		kill_attach(sigr->kill_attach.pid);
		/* Free the segment descriptor. */
		free_pcb(segcb);
		free_buf(&sigr);
		break;
	      }

#ifdef OSE_SMP              
            case SMP_ENABLE_VECTOR_REQUEST:
              {
                sigr->smp_enable_vector.success =
                  sysd_enable_vector_helper(sigr->smp_enable_vector.vec);
                sigr->sig_no = SMP_ENABLE_VECTOR_REPLY;
                send(&sigr, sender(&sigr));
                break;
              }

            case SMP_DISABLE_VECTOR_REQUEST:
              {
                sysd_disable_vector_helper(sigr->smp_disable_vector.vec);
                sigr->sig_no = SMP_DISABLE_VECTOR_REPLY;
                send(&sigr, sender(&sigr));
                break;
              }

              /* Execute an ose_kill_isr or ose_kill_interrupt_hook system call. */
            case SMP_KILL_ISR_REQUEST:
              {
                sysd_kill_isr_or_hook_helper(sigr->smp_kill_isr.vec,
                                             sigr->smp_kill_isr.slot,
                                             sigr->smp_kill_isr.type,
                                             sigr->smp_kill_isr.generation);
                sigr->sig_no = SMP_KILL_ISR_REPLY;
                send(&sigr, sender(&sigr));                                        
              }
              break;

              /* Execute an ose_create_isr or ose_create_interrupt_hook system call. */
            case SMP_CREATE_ISR_REQUEST:
              {
                uint32_t handle;
                OSERRCODE error;
                handle = sysd_create_isr_or_hook_helper(sigr->smp_create_isr.vec,
                                                        sigr->smp_create_isr.owner,
                                                        sigr->smp_create_isr.isr,
                                                        sigr->smp_create_isr.arg,
                                                        sigr->smp_create_isr.type,
                                                        &error,
                                                        sigr->smp_create_isr.cpu);
                sigr->sig_no = SMP_CREATE_ISR_REPLY;
                sigr->smp_create_isr.handle = handle;
                sigr->smp_create_isr.error = error;
                send(&sigr, sender(&sigr));
              }
              break;

              /* Execute an ose_bind_vector system call. */
            case SMP_BIND_VECTOR_REQUEST:
              {
                sigr->smp_bind_vector.success =
                  sysd_bind_vector_helper(sigr->smp_bind_vector.vec,
                                          sigr->smp_bind_vector.cpu);
                sigr->sig_no = SMP_BIND_VECTOR_REPLY;
                send(&sigr, sender(&sigr));
              }
              break;

              /* Execute an ose_unbind_vector system call. */
            case SMP_UNBIND_VECTOR_REQUEST:
              {
                sigr->smp_unbind_vector.success =
                  sysd_unbind_vector_helper(sigr->smp_unbind_vector.vec,
                                            sigr->smp_unbind_vector.cpu);
                sigr->sig_no = SMP_UNBIND_VECTOR_REPLY;
                send(&sigr, sender(&sigr));
              }
              break;
              
#endif /* OSE_SMP */
              
	      /* An unknown signal was received. Report an error and drop it. */
	    default:
	      {
		os_error(1, OSE_ESPURIOUS_SYSTEM_SIGNAL, (OSERRCODE) sender(&sigr));	/* Non fatal system error. */
		free_buf(&sigr);
		break;
	      }
	    }
	}
    }
}

/*
**===========================================================================
** 26.4  The sysd_lite SMP system daemon helper.
**===========================================================================
*/

/*
** Helper for sysd-lite.
** Allocate, pattern fill and install a new interrupt stack on current cpu.
** The old stack must be disposed by the caller.
*/
#ifdef OSE_SMP
static void
install_interrupt_stack(OSADDRESS interrupt_stack_size)
{
  struct stkcb *new_stack;

  LOCK_SWAP();
  new_stack = stack_alloc(&system_pool, interrupt_stack_size);
  UNLOCK_SWAP();

#if ERROR_CHECKS > 0
  memset(((char *) new_stack->top) - interrupt_stack_size,
	 END_MARK,
	 (size_t) interrupt_stack_size);
#else
  memset(((char *) new_stack->top) - interrupt_stack_size,
	 END_MARK,
	 (size_t) 4);
#endif
  zzww.interrupt_stack = new_stack;
		
  /* Make the new stack known to the target interface. */
  zzset_interrupt_stack((OSADDRESS)zzww.interrupt_stack->limit,
			interrupt_stack_size);
}
#endif	/* OSE_SMP */

/*
** The sysd_lite daemon performs general housekeeping of local cpu resources
** for operations like:
** When a process or block is killed.
**
** We don't use the OS_PROCESS macro here. The reason for this is that
** we want the symbol to be static, and OS_PROCESS would make it global.
**
** BEWARE: sysd_lite() may cause a system deadlock if it ever calls
** create_process(), kill_proc(), zzkrn_bind_process(), flush() or hunt().
*/
/* OS_PROCESS(sysd_lite) */
#ifdef OSE_SMP
static void 
sysd_lite(void)
{
  union SIGNAL *sigr;
  static const SIGSELECT sig_sel[] = {0};
  OSADDRESS interrupt_stack_size = 0;

  /* Allocate a new interrupt stack if current cpu is nonzero. */
  if(zzkrn_cpu_id())
    {
      interrupt_stack_size = zzuu.interrupt_stack_size;
      install_interrupt_stack(interrupt_stack_size);
    }
      

  for(;;)
    {
      sigr = receive(sig_sel);
      switch((int) sigr->sig_no)
	{
	  /* Sender wants me to respond. */
	case SMP_SYNC:
	  {
	    send(&sigr, sender(&sigr));
	  }
	  break;

	  /* Allocate a new interrupt stack. */
	case SMP_ALLOC_INT_STACK:
	  {
	    if(interrupt_stack_size < sigr->smp_alloc_int_stack.stack_size)
	      {
		struct stkcb *old_stackcb;

		interrupt_stack_size = sigr->smp_alloc_int_stack.stack_size;

		/* Don't loose the old interrupt stack. */
		old_stackcb = zzww.interrupt_stack;

		install_interrupt_stack(interrupt_stack_size);

		/* Free the old interrupt stack. */
		LOCK_SWAP();
		stack_free(old_stackcb);
		UNLOCK_SWAP();
	      }
	    send(&sigr, sender(&sigr));
	  }
	  break;
	  
	  /* Free a signal that was dropped by an ISR. */
	case SMP_FREE_ME:
	  /* Consider safe_free_buf() here ?? */
	  free_buf(&sigr);
	  break;

	  /* Send an attach signal to the owner. */
	case SMP_META_ATTACH:
	  send_attach_signal(&sigr->smp_meta_attach);
	  /* The sigr signal is disposed by now. */
	  break;

	  /* Relocate queued signals in a dying pool. */
	case SMP_DISPOSE_QUEUED_SIGNALS_REQUEST:
	  {
	    PROCESS poolid = sigr->smp_dispose_queued_signals.poolid;
	    struct poolcb *poolcb;

	    poolcb = (struct poolcb *)pid_to_pcb(poolid);
	    assert(poolcb->instance == pid_to_instance(poolid));
	    sigr->sig_no = SMP_DISPOSE_QUEUED_SIGNALS_REPLY;
	    sigr->smp_dispose_queued_signals.disposal_count = 
	      dispose_queued_signals(poolcb);
	    send(&sigr, zzuu.sysd_);
	  }
	  break;

	  /* Move from the local background and timeout list. */
	case SMP_PROC_MOVE_REQUEST:
	  {
	    PROCESS pid = sigr->smp_move_proc.pid;
	    struct _pcb *pcb = pid_to_pcb(pid);
	    cpuid_t cpu = sigr->smp_move_proc.cpu;

	    LOCK_SWAP();
	    if(pcb_to_pid(pcb) != pid)
	      {
		/* The victim is dead. Just send reply to sysd. */
		sigr->sig_no = SMP_PROC_ADD_REPLY;
		UNLOCK_SWAP();
		send(&sigr, zzuu.sysd_);
	      }
	    else
	      {
		LOCK();
		/*
		** Save a pending timeout request for the new cpu and
		** cancel it on the current cpu.
		*/
		sigr->smp_move_proc.request_tmo = 0;
		if(pcb->tmo_active)
		  {
		    sigr->smp_move_proc.request_tmo = 1;
		    sigr->smp_move_proc.tmo_fragment =
		      pcb->cur_tmo_expires - zzww.ticks;

		    /* Cancel the timer but preserve ST_TIMEOUT the bit. */
		    cancel_other_timeout(pcb);
		  }

		if(pcb->priority == 32)
		  {
		    /* Unlink from the local background list. */
		    unlink_pcb_from_bg_list(pcb);
		  }
		else
		  {
		    pcb->ready_entry.rlist = (struct ready_list_entry *)
		      &ready_list_x(cpu)[pcb->priority];
		  }
		pcb->cpu_id = cpu;
		
		LOCKED_UNLOCK_SWAP();

		/* Order target syd_lite to insert it in the bg list. */
		sigr->sig_no = SMP_PROC_ADD_REQUEST;
		send(&sigr, zzcpu[cpu].sysd_lite_);
	      }
	  }
	  break;
	  
	  /* Insert in the local background and timeout list. */
	case SMP_PROC_ADD_REQUEST:
	  {
	    PROCESS pid = sigr->smp_move_proc.pid;
	    struct _pcb *pcb = pid_to_pcb(pid);

	    LOCK_SWAP();
	    if(pcb_to_pid(pcb) == pid)
	      {
		/* Insert background process in our background list. */
		if(pcb->priority == 32)
		  insert_pcb_in_bg_list(pcb);
		
		/* Request a timeout if needed */
		if(sigr->smp_move_proc.request_tmo &&
		   !(pcb->status & ST_TIMEOUT))
		  {
		    LOCK();
		    request_timeout(pcb, sigr->smp_move_proc.tmo_fragment);
		    UNLOCK();
		  }
	      }
	    /*
	    ** If the process was migrated to this CPU and it has a
	    ** receive poller function we need to enable polling for
	    ** the idle process on this CPU as well. (OSECR-11443) @frbr@
	    */
	    if(pcb->rec_poller != NULL)
	      {
		LOCKSTATE msr;
		
		BKL_LOCK_SAVE(msr);
		zzcpu[zzkrn_cpu_id()].idle_pcb->rec_poller =
		  receive_core_sig_queue;
		BKL_LOCK_RESTORE(msr);
	      }
	    UNLOCK_SWAP();

	    /* Always respond to sysd */
	    sigr->sig_no = SMP_PROC_ADD_REPLY;
	    send(&sigr, zzuu.sysd_);
	  }
	  break;

	  /* Perform a flush requested by another cpu. */
	case SMP_FLUSH_REQUEST:
	  {
	    struct _pcb *pcb = pid_to_pcb(sigr->smp_flush.pid);

	    /* Inhibit migration and killing. */
	    LOCK_SWAP();

	    /* If the victim now is dead, just respond to the client. */
	    if(pcb_to_pid(pcb) != sigr->smp_flush.pid)
	      {
		UNLOCK_SWAP();

		/* Respond to client. */
		sigr->sig_no = SMP_FLUSH_REPLY;
		send(&sigr, sender(&sigr));
	      }
	    else
	      {
		/*
		** If the victim has migrated to another cpu, forward
		** the flush request to that cpu.
		*/
		if(pcb->cpu_id != zzkrn_cpu_id())
		  {
		    UNLOCK_SWAP();
		    zzsys_send_w_s(&sigr, sender(&sigr),
				   zzcpu[pcb->cpu_id].sysd_lite_);
		  }
		else
		  {
		    /* Flush the local victim. */
		    local_flush(sigr->smp_flush.psel, pcb);

		    UNLOCK_SWAP();

		    /* Respond to client. */
		    sigr->sig_no = SMP_FLUSH_REPLY;
		    send(&sigr, sender(&sigr));
		  }
	      }
	  }
	  break;

	  /* Execute a kill_proc() system call. */
	case SMP_KILL_PROC_REQUEST:
	  {
	    PROCESS client = sender(&sigr);
	    PROCESS pid = sigr->smp_kill_proc.pid;
	    struct _pcb *pcb;
	    union SIGNAL *sig_delete_process;
	    
	    /* Allocate a signal to be sent to the system daemon. */
	    sig_delete_process = alloc(sizeof(struct delete_process),
				       DELETE_PROCESS);

	    LOCK_SWAP();
	    pcb = safe_pid_to_pcb(pid, 1);

	    if(!pcb)
	      {
		/* Someone else killed it, just send response. */
		UNLOCK_SWAP();
		free_buf(&sig_delete_process);
		sigr->sig_no = SMP_KILL_PROC_REPLY;
		send_w_s(&sigr, zzuu.sysd_, client);
		break;
	      }
	    
	    switch(pcb->magic)
	      {
	      case MAGIC_BCB:
		{
		  struct _block *bcb = (struct _block*)pcb;
		  cpuid_t other_cpu;
		  /*
		  ** The client stopped the entire block, except itself.
		  ** The block is marked as being killed, so no new processes 
		  ** can be created in it. 
		  ** Now, stop the client too, if it belongs to the block.
		  */
		  if(pid_to_pcb(client)->block == bcb)
		    {
		      stop(client);
		    }

		  other_cpu = sys_kill_block(bcb, &sig_delete_process);
#if defined(GUNNAR_DEBUG)
		  ramlog_printf("pose.c:%d: ose_sysd%02d sys_kill_block(%#x) returns %#x, client %#lx.\n",
				__LINE__, zzkrn_cpu_id(), bid, other_cpu,
				client);
#endif
		  UNLOCK_SWAP();

		  /* Check if more cpus are involved in the kill. */
		  if(other_cpu == (cpuid_t)~0U)
		    {
		      /* The killing is completed, or will at least be when
		      ** the last kill_locked process dies. The client should
		      ** attach the block to detect when kill completes.
		      */
		      sigr->sig_no = SMP_KILL_PROC_REPLY;
		      send_w_s(&sigr, zzuu.sysd_, client);
		    }
		  else
		    {
		      /*
		      ** Kill more processes on another cpu. Using
		      ** safe_send_w_s since the client is likely killed
		      */
		      safe_send_w_s(sig_to_adm(sigr), client,
				    zzcpu[other_cpu].sysd_lite_,
				    0, NULL, NULL);
		      sigr = NIL;
		    }

		  if(sig_delete_process != NIL)
		    {
		      free_buf(&sig_delete_process);
		    }
		}
		break;
		
	      case MAGIC_PCB:
		{
		  cpuid_t cpu = pcb->cpu_id;

		  if(cpu != zzkrn_cpu_id())
		    {
		      /* On another cpu by now, pass the kill request on. */
		      UNLOCK_SWAP();
		      send_w_s(&sigr, client, zzcpu[cpu].sysd_lite_);
		    }
		  else
		    {
		      sys_kill_proc(pcb, &sig_delete_process); 
		      UNLOCK_SWAP();
		      sigr->sig_no = SMP_KILL_PROC_REPLY;
		      send_w_s(&sigr, zzuu.sysd_, client);
		    }

		  if(sig_delete_process != NIL)
		    {
		      free_buf(&sig_delete_process);
		    }
		}
		break;

	      default:
		/* Internal error. Corrupted kernel?? */
		UNLOCK_SWAP();
		os_error(1, OSE_ESPURIOUS_SYSTEM_SIGNAL, (OSERRCODE) client); /* Non fatal system error. */
		free_buf(&sigr);
		break;
	      }	/* switch */

	    /* If the delete process signal wasn't sent, then free it. */
	    if(sig_delete_process != NIL)
	      {
		free_buf(&sig_delete_process);
	      }
	    
	  } /* case */
	  break;

          /* Execute an ose_disable_vector system call. */
        case SMP_ZZDISABLE_VECTOR_REQUEST:
          {
            zzdisable_vector(sigr->smp_zzdisable_vector.vec);
            sigr->sig_no = SMP_ZZDISABLE_VECTOR_REPLY;
	    send(&sigr, sender(&sigr));            
          }
          break;

          /* Execute an ose_enable_vector system call. */
        case SMP_ZZENABLE_VECTOR_REQUEST:
          {
            zzenable_vector(sigr->smp_zzenable_vector.vec,
                            sigr->smp_zzenable_vector.pri);
            sigr->sig_no = SMP_ZZENABLE_VECTOR_REPLY;
	    send(&sigr, sender(&sigr));            
          }
          break;

          /* Execute ose_kill_isr on behalf of sysd. */
        case SMP_KILL_ISR_REQUEST:
          {
            struct vectorcb *vcb;
            LOCK_SWAP();
            vcb = get_vectorcb(sigr->smp_kill_isr.vec, 0);
            /* We don't have to check validity since we are called
             * from sysd, so nothing can happen in the meantime. */
            kill_isr_or_hook_helper(vcb,
                                    sigr->smp_kill_isr.slot,
                                    sigr->smp_kill_isr.type);
            UNLOCK_SWAP();
            sigr->sig_no = SMP_KILL_ISR_REPLY;
            send(&sigr, sender(&sigr));
          }
          break;

          /* Execute ose_create_isr on behalf of sysd. */
        case SMP_CREATE_ISR_REQUEST:
          {
            struct vectorcb *vcb;
            OSERRCODE error;
            uint32_t handle;
            LOCK_SWAP();
            vcb = get_vectorcb(sigr->smp_create_isr.vec, 0);
            /* We don't have to check validity since we are called
             * from sysd, so nothing can happen in the meantime. */
            handle = create_isr_or_hook_helper(vcb,
                                               sigr->smp_create_isr.owner,
                                               sigr->smp_create_isr.isr,
                                               sigr->smp_create_isr.arg,
                                               sigr->smp_create_isr.type,
                                               &error,
                                               sigr->smp_create_isr.cpu);
            UNLOCK_SWAP();
            sigr->sig_no = SMP_CREATE_ISR_REPLY;
            sigr->smp_create_isr.error = error;
            sigr->smp_create_isr.handle = handle;
            send(&sigr, sender(&sigr));
          }
          break;
          
	  /* An unknown signal was received. Report an error and drop it. */
	default:
	  {
	    os_error(1, OSE_ESPURIOUS_SYSTEM_SIGNAL, (OSERRCODE) sender(&sigr)); /* Non fatal system error. */
	    free_buf(&sigr);
	    break;
	  }
	} /* switch */
    } /* for */
} /* sysd_lite */
#endif

/*
*****************************************************************************
** 27 COMPATIBILITY GLUE.
*****************************************************************************
*/

/*
**===========================================================================
** 27.1  Compatibility glue for SMP/non-SMP kernels.
**===========================================================================
*/

/*
** Some old tools check the global (and only) zzcur_p pointer to see
** if the kernel is started. We keep those clients happy by keeping the
** global zzcur_p variable even though we don't use it in the SMP kernel.
*/
#ifdef OSE_SMP
#undef       zzcur_p  /* Make sure we get rid of any name mangling. */
struct _pcb *zzcur_p; /* This is the global variable */
static void
set_global_zzcur_p(struct _pcb *pcb)
{
  zzcur_p = pcb;
}
#endif
