/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2004-2012 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

#include "compiler_macros.h"
#ifdef PPC
long long ose_read_tb(void);
#else
WEAK_REF(long long ose_read_tb(void));
#endif

/* C runtime include files. */
#include "string.h"

/* OSE public include files. */
#include "assert.h"
#include "cache.h"
#include "cpu_hal.h"
#include "cpu_access.h"
#include "mm.sig"
#include "mm_internal.sig"
#include "mm_cmd.h"
#include "mm_err.h"
#include "ose.h"
#include "ramlog.h"
#include "shell.h"
#include "sysparam.h"
#include "stdio.h"
#include "mm.h"

/* OSE internal header files. */
#include "core_rbtree.h"

/* KRN. */
#include "krn_internal.h"
#include "pose.h"

/* Block internal include files. */
#include "allocator.h"
#include "configure.h"
#include "compat.h"
#include "domain.h"
#include "gmm.h"
#include "hal.h"
#include "slab.h"
#include "mm_access_internal.h"
#include "mm_util_internal.h"
#include "mm_trap_internal.h"
#include "partition.h"
#include "phys_object.h"
#include "ram_allocator.h"
#include "mm_log.h"
#include "zone.h"
#include "mm_cmd.h"
#include "mm_private.h"
#include "page.h"
#include "codec.h"
#include "mm_profile.h"



#ifdef OSE_SMP
#  include "cpu_smp.h" /* OSE_SMP_MAX_CPUS */
#endif

/* More of these may be needed later... */
#define PRINT_START(s)            log(CATEGORY_START, s)
#define PRINT_SIGNAL(s)           log(CATEGORY_SIGNAL, s)
#define PRINT_MAS(s)              log(CATEGORY_MAS, s)
#define PRINT_FAILED_REQUEST(s)   log(CATEGORY_FAILED_REQUEST, s)

#define IMPLY(a,b)    (!(a) || (b))


/* Forward references */
static union SIGNAL *delete_region(struct gmm_vars *, union SIGNAL *);

#ifdef OSE_SMP
/******************************************************************************
 * Interface between gmm and the agents on cpu 1..N.
 */
#define HAL_CREATE_RESIDENT_MAPPING   5677
#define HAL_DELETE_RESIDENT_MAPPING   5678
#define HAL_DELETE_RESIDENT_MAPPINGS  5679

#define HAL_GET_CORE_MMU_STAT         5680
#endif

struct hal_get_core_mmu_stat {
   SIGSELECT sig_no;
   hal_status_t status;
   struct mm_tag_ext tags[1];   /* I/O: Queries/Answers in tag format */
};

struct hal_create_resident_mapping {
   SIGSELECT sig_no;
   struct hal_domain *hal_domain;
   log_addr_t base;
   log_size_t size;
   U32 hal_options;
   U32 permissions;
   U32 mem_attr;
};

struct hal_delete_resident_mapping {
   SIGSELECT sig_no;
   struct hal_domain *hal_domain;
   log_addr_t base;
   log_size_t size;
   U32 hal_options;
};

struct hal_delete_resident_mappings {
   SIGSELECT sig_no;
   log_addr_t base;
   log_size_t size;
};

/*****************************************************************************/
#ifdef OSPP
static union SIGNAL *
make_resident(struct gmm_vars *gmm_vars, union SIGNAL *request);

static union SIGNAL *
make_unresident(struct gmm_vars *gmm_vars, union SIGNAL *request);

static mm_status_t
make_unresident_range(struct gmm_vars *gmm_vars, log_addr_t base, log_size_t size);
#endif /* OSPP */
/*****************************************************************************/

/* The set of signals to handle. */
union SIGNAL
{
   SIGSELECT sig_no;
   
   struct mm_create_region           mm_create_region;
   struct mm_delete_region           mm_delete_region;
   struct mm_change_permissions      mm_change_permissions;
   struct mm_share_region            mm_share_region;
   /* struct mm_create_domain           mm_create_domain; */
   struct mm_delete_domain           mm_delete_domain;
   struct mm_write                   mm_write;
   /* struct mm_interface_info_request  mm_interface_info_request; */
   struct mm_interface_info_reply    mm_interface_info_reply;
   /* struct mm_list_domains_request    mm_list_domains_request; */
   struct mm_list_domains_reply      mm_list_domains_reply;
   struct mm_list_regions_request    mm_list_regions_request;
   struct mm_list_regions_reply      mm_list_regions_reply;
   /* struct mm_list_log_mem_request    mm_list_log_mem_request; */
   struct mm_list_log_mem_reply      mm_list_log_mem_reply;
   /* struct mm_list_phys_mem_request   mm_list_phys_mem_request; */
   struct mm_list_phys_mem_reply     mm_list_phys_mem_reply;
   struct mm_translate_address       mm_translate_address;
   /*struct mm_list_page_sizes_request mm_list_page_sizes_request; */
   struct mm_list_page_sizes_reply   mm_list_page_sizes_reply;
   struct mm_get_mmu_statistics      mm_get_mmu_statistics;
   struct mm_get_phys_mem_info       mm_get_phys_mem_info;
   struct mm_get_domain_info         mm_get_domain_info;
   struct mm_get_log_mem_info        mm_get_log_mem_info;
   struct mm_get_log_mem_bank_info   mm_get_log_mem_bank_info;
   struct mm_get_phys_mem_type_info  mm_get_phys_mem_type_info;
   struct mm_multithread             mm_multithread;
   struct mm_alloc_metadata          mm_alloc_metadata;
   struct mm_free_metadata           mm_free_metadata;
   struct mm_add_phys_resource       mm_add_phys_resource;
   struct mm_remove_phys_resource    mm_remove_phys_resource;
   struct mm_alloc_phys_resource     mm_alloc_phys_resource;
   struct mm_free_phys_resource      mm_free_phys_resource;
   struct mm_make_resident           mm_make_resident;
   struct mm_get_max_domains         mm_get_max_domains;
   struct mm_forced_write            mm_forced_write;

   /* PMM signals */
   struct pmm_assign                  pmm_assign;
   struct pmm_restore_start_reply     pmm_restore_start_reply;
   struct pmm_restore_region_request  pmm_restore_region_request;
   struct pmm_restore_region_reply    pmm_restore_region_reply;
   struct pmm_restore_binding_request pmm_restore_binding_request;
   struct pmm_restore_binding_reply   pmm_restore_binding_reply;
   struct pmm_alloc_sase_request      pmm_alloc_sase_request;
   struct pmm_alloc_sase_reply        pmm_alloc_sase_reply;
   struct pmm_alloc_sas_request       pmm_alloc_sas_request;
   struct pmm_alloc_sas_reply         pmm_alloc_sas_reply;
   struct pmm_change                  pmm_change;
   struct pmm_free                    pmm_free;

   /* Local interpretations. */
   struct hal_create_resident_mapping  hal_create_resident_mapping;
   struct hal_delete_resident_mapping  hal_delete_resident_mapping;
   struct hal_delete_resident_mappings hal_delete_resident_mappings;
   struct hal_get_core_mmu_stat        hal_get_core_mmu_stat;
};

/*****************************************************************************/
/* MAX length of Phys resource array.
 */
#define PHYS_RES_MAX 128

struct regionData {
   struct pmm_restore_binding_reply *bindingSig;
   struct region                    *region;
   struct phys_object               *pobj;

   /* Used to store fragment list and allocator until the phys_object has been
    * allocated. */
   struct fragment_container *tmp_frag_list;
   struct allocator *tmp_allocator;
};

static const SIGSELECT select_all[] = {0}; /* accept all signals */

#if defined(OSE_SMP) && defined(OSPP)

/* Information about the MM agents on cpu 1..N. */
static struct {
   cpuid_t num_cpus;
   
   /* The array is shifted down one step since no agent is needed on cpu 0.
    * Use cpu_id-1 as index.
    */
   PROCESS pid[OSE_SMP_MAX_CPUS-1];
} smp_agents;

#endif

struct restartData {
  U32 entryCnt;
  struct pmm_restore_start_reply *restartSig;
  struct regionData regions[1];
};

static struct mm_handlers handlers =
{
   mm_select_domain
};

void
init_mman(void);

static mm_status_t
mm_pmm_alloc_sase(struct region *region);

static mm_status_t
mm_pmm_alloc_sas(struct region *region);

static mm_status_t
mm_pmm_change_attr(struct region *region);

#if defined(OSE_SMP) && defined(OSPP)
static void
create_and_start_cpu_agents(void);
static void
stop_and_kill_cpu_agents(void);
#endif

struct mm_handlers *
mm_open_exception_area_internal(const struct cpu_descriptor *cpu_descriptor,
                                char *vector_base,
                                size_t vector_size);

void
mm_install_exception_handlers_internal(void);

mm_bool_t
mm_compiled_with_assert(void)
{
   mm_bool_t rc = FALSE;
#ifdef ASSERT
   rc = TRUE;
#endif

   return rc;
}

mm_bool_t
mm_compiled_with_optimize_gcc(void)
{
   mm_bool_t rc = FALSE;
#if defined(__GNUC__) && defined(__OPTIMIZE__)
   rc = TRUE;
#endif

   return rc;
}

mm_status_t
mm_hal_status_to_mm_status(hal_status_t hal_status)
{
   mm_status_t mm_status = MM_SUCCESS;
   
   switch (hal_status)
   {
      case HAL_SUCCESS:
         mm_status = MM_SUCCESS;
         break;
      case HAL_EEND_OF_PHYS_MEMORY:
      case HAL_EINTERNAL_POOL_ALLOCATION_FAILED:
         mm_status = MM_EOUT_OF_PHYS_MEM;
         break;
      case HAL_EEND_OF_LOG_MEMORY:
         mm_status = MM_EOUT_OF_LOG_MEM;
         break;
      case HAL_EINTERNAL_ERROR:
         mm_status = MM_EINTERNAL_ERROR;
         break;
      case HAL_ELOOKUP_FAILED:
         mm_status = MM_ENO_PHYSICAL_MAPPING;
         break;
      case HAL_EMAPPING_EXISTS:
         mm_status = MM_EOVERLAPPING_ADDRESS;
         break;
      case HAL_EUNSUPPORTED_MMU:
      default:
         INTERNAL_ERROR();
   }

   return mm_status;
}

/*
 * Try to reclaim memory allocated by other components in the system
 */
size_t
ose_mm_reclaim_component_memory(struct gmm_vars *gmm_state,
                                U32 required_size,
                                const char *phys_mem)
{
   Msr sr;
   struct mm_reclaim_physical_event_t event;
   enum PROCESS_TYPE type;
   extern OSBOOLEAN ose_started(void);

   if (!ose_started())
   {
      /* Quit if exec is not yet initiated, since
       * no ose system calls are yet available.
       * */
      return 0;
   }

   type = get_ptype(current_process());

   /* No reclaim for processes with interrupts disabled as the reclaim
    * could enable interrupts. */
   LOCK_SAVE(sr);
   LOCK_RESTORE(sr);
   if (!IRQ_ENABLED(sr) || type == OS_INT_PROC || type == OS_TI_PROC)
   {
      return 0;
   }

   if (mm_get_log_mask() & CATEGORY_SIGNAL)
   {
      struct allocator *allocator =
         mm_find_allocator_by_name(phys_mem);

      if (allocator != NULL &&
          allocator->get_free_mem != NULL &&
          allocator->get_biggest_frag != NULL)
      {
         size_t avail_size = allocator->get_free_mem(allocator);
         size_t max_phys_frag = allocator->get_biggest_frag(allocator);

         PRINT_SIGNAL(("gmm: ose_mm_reclaim_component_memory required "\
                       "size: %#lx, phys_mem %s, \n "
                       "(MM avail size: %#lx max_phys_frag %#lx)\n",
                       required_size, phys_mem, avail_size, max_phys_frag));
      }
   }

   event.domain = MM_EVENT_ANY_DOMAIN;
   event.phys_mem = phys_mem;
   event.internal_identifier = RECLAIM_ALL_INTERNAL;
   event.size = required_size;
   event.inquiry = 0;
   event.event_no = MM_EVENT_RECLAIM_REGION_PHYSICAL;
   call_reclaim_region_event(&event);

   if (event.size > 1 &&
       event.identifier != COMMON_MM_COMPONENT)
   {
      struct mm_delete_region *reg = (struct mm_delete_region *)
         alloc(sizeof(struct mm_delete_region), 0);

      /* Delete the region if it is not from the slab.
       * It handles the deletion by itself. */
      /* FIXME: validate the returned base address before delete. */
      reg->domain = event.domain;
      reg->base = event.base;
      assert(reg->base != 0);

      PRINT_SIGNAL(("gmm: ose_mm_reclaim_component_memory, found mem base: "\
                    "%#010lx size: %#010lx domain: %ld\n",
                    event.base, event.size, event.domain));
      
      delete_region(gmm_state, (union SIGNAL *)reg);

      if (reg->status != MM_SUCCESS)
      {
         ramlog_printf("Component id %x base %x size %x\n", event.identifier,
                       event.base, event.size);
         MM_FATAL_ERROR2(("The reclaim of the specified region failed\n\n"),
                         OSE_MM_EINTERNAL_ERROR, 0);
      }

      free_buf((union SIGNAL**)&reg);
   }
   
   return event.size;
}

/*
 * Try to reclaim memory allocated by other components in the system.
 * Interface available in private/mm_private.h
 */
mm_status_t
ose_mm_reclaim_memory(U32 required_size, const char *phys_mem)
{
   size_t size;

   size = ose_mm_reclaim_component_memory(&gmm_state, required_size, phys_mem);
   if (size >= required_size)
   {
      return MM_SUCCESS;
   }
   else
   {
      return MM_EOUT_OF_PHYS_MEM;
   }
}

/*
 * Syncronized gmm hal init.
 * Finally the mmu (if present) is enabled.
 */
static void
gmm_hal_sync_init(void)
{
   /* Do the rest in another file. */
   mm_config_init();
}

/*
 * First function to be called in MM.
 * See krn_internal.h
 */
   
struct mm_handlers *
mm_open_exception_area_internal(const struct cpu_descriptor *cpu_descriptor,
                                char *vector_base,
                                size_t vector_size)
{
   hal_status_t rc;

#ifndef softOSE
   if (ose_read_tb != NULL)
   {
      gmm_stats.open_exception_area.start = ose_read_tb();
   }
#endif

   gmm_state.mas_cnt = 0;
   mm_configure_log_level();

   PRINT_START(("mm: mm_open_exception_area(cpu_descriptor=%p,"\
                "vector_base=%#lx, vector_size=%x) \n",
                cpu_descriptor, vector_base, vector_size));

   mm_configure_cache_mode();

  /* Validate MMU and open up exception area. */
   rc = hal_init_1(cpu_descriptor,
                   (log_addr_t)vector_base,
                   (log_size_t)vector_size,
                   gmm_state.d_cache,
                   gmm_state.i_cache);

   if (rc != HAL_SUCCESS)
   {
      MM_FATAL_ERROR2(("MMU not supported\n\n"),
                      OSE_MM_EHAL_SUBHAL_NO_MATCH, 0);
   }

   /* Retrieve basic information from HAL. */
   gmm_state.hal_info = hal_get_info();

   PRINT_START(("mm: mm_open_exception_area MMU=%d\n",
                gmm_state.hal_info->has_mmu));

   if (gmm_state.hal_info->enable_mmu_early)
   {
      /* If arm do an early enable of mmu.
       * On for example xscale it is neccessary to to map
       * exception area before exec can install its exception handlers.
       */
      PRINT_START(("mm: mm_open_exception_area early\n"));

      gmm_hal_sync_init();
   }

#ifndef softOSE
   if (ose_read_tb != NULL)
   {
      gmm_stats.open_exception_area.end = ose_read_tb();
   }
#endif

   return &handlers;
}

/*
 * Second function to be called in MM.
 * See krn_internal.h
 */

void
mm_install_exception_handlers_internal(void)
{
   extern void mm_init_bios_interface(void);

#ifndef softOSE
   if (ose_read_tb != NULL)
   {
      gmm_stats.install_exception_handlers.start = ose_read_tb();
   }
#endif

   PRINT_START(("mm: mm_install_exception_handlers: entry\n"));

   if (!gmm_state.hal_info->enable_mmu_early)
   {
      /* In ppc do a late enable of mmu.
       * It is for debug reasons desirable to execute as much
       * code as possible after exec has installed its exception handlers.
       */
      gmm_hal_sync_init();
   }

   /* Install mm vectors and there after write protect vector area.  */
   hal_init_5();
   gmm_state.state = mmu_enabled;
   init_slab(HAL_LOG_MEM_NAME);
   mm_util_init();
   init_slab2();

   mm_kick_wdog();

   mm_access_init(mm_find_domain_partition_by_name(
                     gmm_state.system_domain, "HAL"));

   /* Install the MM trap interface */
   mm_init_bios_interface();
   init_mman();

#ifndef softOSE
   if (ose_read_tb != NULL)
   {
      gmm_stats.install_exception_handlers.end = ose_read_tb();
   }
#endif
   
   ramlog_printf("mm: initialization completed.\n");
}

/*
 * Catch obvious errors in permission parameter.
 */
mm_bool_t
mm_check_permission(U32 perm)
{
   U32 should_be_zero_mask;
   U32 super = perm >> MM_SU_SHIFT;
   U32 user = perm & 7;

   should_be_zero_mask = ~(MM_USR_EXECUTE | MM_USR_WRITE | MM_USR_READ | \
			   MM_SU_EXECUTE  | MM_SU_WRITE  | MM_SU_READ);
   /* Only releveant bits allowed. */
   if ((perm & should_be_zero_mask) != 0)
   {
      return FALSE;
   }

   /* Supervisor mode must have same or better permissions than user mode. */
   if (((user & MM_PERM_READ)    && !(super & MM_PERM_READ))  ||
       ((user & MM_PERM_WRITE)   && !(super & MM_PERM_WRITE)) ||
       ((user & MM_PERM_EXECUTE) && !(super & MM_PERM_EXECUTE)))
   {
      return FALSE;
   }

   /* What should we do here? Extra checks by asking HAL? */
   /* FIXME */
   return TRUE;
}



/*
 * Check cache mode parameter.
 */
mm_bool_t
mm_check_cache_mode(U32 cache)
{
   mm_bool_t result;

   cache &= (MM_CACHE_INHIBITED|MM_CACHE_WRITE_THROUGH|
             MM_CACHE_COPY_BACK|MM_ORDERED_ACCESS|
             MM_SPECULATIVE_ACCESS|MM_CACHE_MEMORY_COHERENCE);

   result =
      cache == (MM_CACHE_INHIBITED     | MM_SPECULATIVE_ACCESS) ||
      cache == (MM_CACHE_WRITE_THROUGH | MM_SPECULATIVE_ACCESS) ||
      cache == (MM_CACHE_COPY_BACK     | MM_SPECULATIVE_ACCESS) ||
      cache == (MM_CACHE_COPY_BACK     | MM_CACHE_MEMORY_COHERENCE | MM_SPECULATIVE_ACCESS) ||
      cache == (MM_CACHE_INHIBITED     | MM_ORDERED_ACCESS)     ||
      cache == (MM_CACHE_WRITE_THROUGH | MM_ORDERED_ACCESS)     ||
      cache == (MM_CACHE_COPY_BACK        | MM_ORDERED_ACCESS)     ||
      cache == (MM_CACHE_MEMORY_COHERENCE | MM_ORDERED_ACCESS)     
      ;
   return result;
}



/*
 * Validate a region identifier pair (<domain_id>, <base>).
 */
static mm_status_t
validate_region_identifiers(struct gmm_vars *gmm_vars,
                            OSDOMAIN domain_id,
                            OSADDRESS base,
                            mm_bool_t exact_match,
                            struct domain **out_domain,
                            struct region **out_region)
{
   struct domain *domain;
   struct region *region;

   assert(NULL != gmm_vars);
   assert(NULL != out_domain);
   assert(NULL != out_region);

   /* Look up domain. */
   
   domain = mm_find_domain_by_domid(&gmm_vars->domains, domain_id);
   if (domain == NULL)
   {
      return MM_ENON_EXISTING_DOMAIN;
   }

   /* Look up instance region. */
   region = mm_find_region_by_addr(
      &gmm_vars->domains, domain->segnum, base, exact_match);

   if (region == NULL)
   {
      return MM_ENON_EXISTING_REGION;
   }

   /* Instance region found -> update out variables .*/
   *out_domain = domain;
   *out_region = region;

   return MM_SUCCESS;
}

/*
 * Check options against mapping type.
 */
static mm_status_t
check_options(struct mm_create_region *request, mapping_type_t mapping_type)
{
   mm_status_t status = MM_SUCCESS;
   U32 options = request->options;

   switch (mapping_type)
   {
      case SASE:
      {
	 /*
	  * SASE does not support logical-only or duplicate regions
	  */
	 if (options & MM_CREATE_REGION_LOGICAL &&
             !(options & MM_ALLOW_LOGICAL_SASE))
	 {
            
            PRINT_SIGNAL(("gmm: Option MM_CREATE_REGION_LOGICAL "
                          "not allowed for SASE region\n"));
	    status = MM_EOPTION_NOT_SUPPORTED;
	 }

         if ((options & MM_CREATE_REGION_FAULTABLE))
	 {
            PRINT_SIGNAL(("gmm: Option MM_CREATE_REGION_FAULTABLE not allowed "
                          "for SASE region\n"));
	    status = MM_EOPTION_NOT_SUPPORTED;
	 }

	 if (options & MM_CREATE_REGION_DUPLICATE)
	 {
            PRINT_SIGNAL(("gmm: Option MM_CREATE_REGION_DUPLICATE not allowed "
                          "for SASE region\n"));
	    status = MM_EILLEGAL_SASE_OPTION;
	 }
	 break;
      }
      case SAS:
      {
         /* Option MM_CREATE_REGION_DUPLICATE must be used in conjunction
          * with the option MM_CREATE_REGION_SPECIFY_PHYS. */        
	 if(options & MM_CREATE_REGION_DUPLICATE)
	 {
           if(!(options & MM_CREATE_REGION_SPECIFY_PHYS))
           {
              PRINT_SIGNAL(("gmm: Option MM_CREATE_REGION_DUPLICATE must be "
                            "used in conjunction with the option "
                            "MM_CREATE_REGION_SPECIFY_PHYS.\n"));
              
	      status = MM_EILLEGAL_OPTION_COMBINATION;
           }
         }        
	 break;
      }
      case MAS:
      {
         if (request->domain == MM_KERNEL_DOMAIN &&
             (options & MM_CREATE_REGION_LOGICAL) == 0 &&
             (options & MM_CREATE_REGION_FAULTABLE) == 0)
	 {
            /* Only allow logical MAS regions (used by PGM to reserve
             * data for loaded LM's).
             *
             * FIXME(rand) - No MAS at all in system domain ?
             */
            PRINT_SIGNAL(("gmm: In system domain only logical MAS regions are "
                          "allowed.\n"));
	    status = MM_EOPTION_NOT_SUPPORTED;
	 }
         
         /* Option MM_CREATE_REGION_DUPLICATE must be used in conjunction
          * with the option MM_CREATE_REGION_SPECIFY_PHYS. */        
	 if(options & MM_CREATE_REGION_DUPLICATE)
	 {
           if(!(options & MM_CREATE_REGION_SPECIFY_PHYS))
           {
              PRINT_SIGNAL(("gmm: Option MM_CREATE_REGION_DUPLICATE must be "
                            "used in conjunction with the option "
                            "MM_CREATE_REGION_SPECIFY_PHYS.\n"));
              
	      status = MM_EILLEGAL_OPTION_COMBINATION;
           }
	 }
	 break;
      }
      default:
      {
	 INTERNAL_ERROR();
      }
   }
   /* Do basic sanity checks that are valid for all mapping types.*/
   if ((options & MM_CREATE_REGION_LOGICAL) ||
       (options & MM_CREATE_REGION_FAULTABLE))
   {
      /* By definition a logical only region shall not carry
       * information of any physical memory.
       */
      if (options & MM_CREATE_REGION_SPECIFY_PHYS ||
          options & MM_CREATE_REGION_PERSISTENT)
      {
         status = MM_EILLEGAL_OPERATION;
      }
   }
   return status;
}


/*
 * Make sure input strings are valid C-style string, i.e. '\0'-terminated.
 */
static void
truncate_strings(struct mm_create_region *req)
{
   /* Truncate region name, or make it an empty string. */
   if (req->options & MM_CREATE_REGION_SPECIFY_NAME)
   {
      req->name[MM_NAME_LENGTH] = '\0';
   }
   else
   {
      req->name[0] = '\0';
   }

   /* Make sure other identifiers have terminators (implicit truncation). */
   req->phys_mem[MM_NAME_LENGTH] = '\0';
   req->log_mem[MM_NAME_LENGTH] = '\0';
}


/*
 * Create a sase region.
 */
static mm_status_t
create_sase_region(struct gmm_vars *gmm_vars,
		   struct mm_create_region *request,
		   struct partition *partition,
		   struct allocator *allocator,
                   struct domain *domain)
{
   struct phys_object *pobj = NULL;
   struct log_object *lobj = NULL;
   struct region *region = NULL;
   mm_status_t status;
   U32 log_base_valid;
   U32 persistent;
   log_addr_t log_base;
   log_size_t size;
   U32 meta_options = 0;
   int alloc_phys;

   log_base_valid = (request->options & MM_CREATE_REGION_SPECIFY_LOG);
   log_base   = request->base;
   size       = request->size;
   
   persistent = (request->options & MM_CREATE_REGION_PERSISTENT) != 0;

   assert(IMPLY(!log_base_valid && persistent, allocator != NULL));
   assert(IMPLY(!log_base_valid,allocator != NULL));

   PRINT_SIGNAL(("gmm: create_sase_region  name: %s base: %#x size: %#lx "\
                 "log_mem: %s, allocator: %#x\n"\
                 "persistent: %d partition->name %s\n",
                 request->name, log_base, size,
                 request->log_mem,
                 allocator, persistent, partition->class->name));
   /*
    * For SASE regions,
    * MM_CREATE_REGION_SPECIFY_PHYS -> MM_CREATE_REGION_SPECIFY_LOG
    */
   if (!log_base_valid && (request->options & MM_CREATE_REGION_SPECIFY_PHYS))
   {
      return MM_EILLEGAL_OPERATION;
   }

   if (persistent)
   {
      meta_options |= MM_REGION_OPT_PERSISTENT;
   }

   if (request->options & MM_CREATE_REGION_LOGICAL &&
       request->options & MM_ALLOW_LOGICAL_SASE)
   {
      meta_options |= MM_REGION_OPT_LOGICAL;
      meta_options |= MM_REGION_OPT_LOGICAL_SASE;
      alloc_phys = 0;
   }
   else
   {
      alloc_phys = 1;
      meta_options |= MM_REGION_OPT_MAPPED;
   }
   

   if (!alloc_phys) {
      goto skip_alloc_phys; 
   }
   
   pobj = mm_create_phys_object(allocator, meta_options);
   if (pobj == NULL)
   {
      status = MM_EOUT_OF_PHYS_MEM;
      goto cleanup;
   }

   mm_attach_phys_object(pobj);
   
   /*
    * Allocate physical memory.
    * A special case exists here, namely arbitrary allocations from
    * stateful allocators. That memory has already been allocated
    * (thats the way it is).
    */

   /*
    * Client supplied logical *and* physical address ->
    * create simple phys_mem object. Used for offset mappings.
    */
   if (request->options & MM_CREATE_REGION_SPECIFY_PHYS)
   {
      if (request->phys_count != 1) {
         PRINT_SIGNAL(("gmm: User provided physical memory for SASE regions"
                       " must be contained in exactly one fragment. Got %lu in"
                       " request.\n",
                       request->phys_count));

         status = MM_EMISMATCH_FRAGMENTS;
         goto cleanup;
      }

      if (size != request->phys_resources[0].size) {
         PRINT_SIGNAL(("gmm:Size of user provided physical memory (0x%08lx)"
                       " does not match size of region (0x%08lx).\n",
                       request->phys_resources[0].size, size));

         status = MM_EMISMATCH_FRAGMENTS;
         goto cleanup;
      }

      pobj->data_representation = PHYS_OBJ_DIRECT_PHYS_FRAG;
      pobj->data.direct_phys_frag.pa = request->phys_resources[0].base;
      pobj->data.direct_phys_frag.size = request->phys_resources[0].size;
      status = MM_SUCCESS;
   }
   else if (NULL == allocator) {
      if (!log_base_valid) {
         status = MM_EILLEGAL_OPERATION;
      }
      else
      {
         /*
          * Client supplied logical address but no phys_mem ->
          * create simple physmem object with identical physical
          * address.
          */
         phys_addr_t phys_base = log_base; /* SASE region */
         
#ifdef OSMIPS
         
         if ((log_base & KSEG_MASK) == KSEG0_BASE) {
            phys_base -= KSEG0_BASE;
         }
         else if ((log_base & KSEG_MASK) == KSEG1_BASE) {
            phys_base -= KSEG1_BASE;
         }
         
#endif
         
         pobj->data_representation = PHYS_OBJ_DIRECT_PHYS_FRAG;
         pobj->data.direct_phys_frag.pa   = phys_base;
         pobj->data.direct_phys_frag.size = size;
         status = MM_SUCCESS;
      }
   }
   else
   {
      /* These variables specifies a range in which the memory shall be
       * allocated. This is only needed for mips, but always keeping it
       * avoids one #ifdef. */
      log_addr_t range_base;
      log_size_t range_size;
      log_size_t log_offset = 0;
      
      
      if (partition->class->get_bank_info == NULL)
      {
         /* Assume no restrictions. */
         range_base = 0;
         range_size = 0;
      }
      else
      {
         int exists = partition->class->get_bank_info(
            partition->class, 0, &range_base, &range_size);
         if (!exists)
         {
            range_base = 0;
            range_size = 0;
         }

#ifdef OSMIPS
         
         if ((range_base & KSEG_MASK) == KSEG0_BASE) {
            log_offset = KSEG0_BASE;
         }
         else if ((range_base & KSEG_MASK) == KSEG1_BASE) {
            log_offset = KSEG1_BASE;
         }

         range_base -= log_offset;
         
#endif
      }
      
      if (log_base_valid) {
         /* This case used alloc_f earlier.
          * New behavior: MM_CREATE_REGION_SPECIFY_LOG is treated as a hint.
          *  If specified address is not free, MM will perform an incremental
          *  search for a free region on higher addresses. 
          *  */

         /* Create_region will only use a partition that contains log_base. */
         assert(range_base <= log_base);
         assert(range_base + range_size - 1 >= log_base);            
         
         while (1) {
            U32 try_reclaim = 0;
            
            /* First try [log_base .. range_base + range_size] */
            {
               phys_size_t tmp_size = (range_base + range_size) - log_base;
               
               status = allocator->alloc_e(allocator, pobj,
                                           size, request->align,
                                           (phys_addr_t)log_base, tmp_size);

               if (MM_SUCCESS == status) {
                  break;
               }

               if (status == MM_EOUT_OF_PHYS_MEM) {
                  try_reclaim = 1;
               }
            }
            
            {
               /* Not successful, try [range_base .. log_base-1] */
               status = allocator->alloc_e(allocator, pobj,
                                           size, request->align,
                                           range_base, log_base - range_base);
               
               if (MM_SUCCESS == status) {
                  break;
               }
               if (status == MM_EOUT_OF_PHYS_MEM) {
                  try_reclaim = 1;
               }
            }

            if (try_reclaim &&
                ose_mm_reclaim_component_memory(gmm_vars,
                                                size,
                                                request->phys_mem))
            {
               continue;
            }
            goto cleanup;
         }
      }
      else {
         assert(NULL != allocator->alloc_e);
         while (1)
         {
            status = allocator->alloc_e(allocator, pobj,
                                        size, request->align,
                                        (phys_addr_t)range_base,
                                        (phys_size_t)range_size);
            if (status == MM_EOUT_OF_PHYS_MEM &&
                ose_mm_reclaim_component_memory(gmm_vars, size,
                                                request->phys_mem))
            {
               continue;
            }
            if (status != MM_SUCCESS)
            {
               goto cleanup;
            }
            break;
         }
      }

      {
         phys_addr_t ph_base = FC_GET_BASE(pobj->data.frag_list);
         assert(ph_base == (log_addr_t)ph_base);
         log_base = (log_addr_t)ph_base + log_offset;
      }
   }

   if (status != MM_SUCCESS)
   {
      goto cleanup;
   }

skip_alloc_phys:   
   lobj = mm_create_log_object();
   if (lobj == NULL)
   {
      status = MM_EOUT_OF_PHYS_MEM;
      goto cleanup;
   }

   status = partition->class->alloc_f(
      partition, lobj, size, log_base, MM_PARTITION_OPT_SASE);
   if (status != MM_SUCCESS)
   {
      goto cleanup;
   }

   mm_attach_log_object(lobj, domain);
   
   status = mm_add_region(
      request->domain,
      lobj,
      pobj,
      request->permissions,
      request->cache_mode,
      request->options,
      meta_options,
      request->name,
      request->log_mem,
      &region);
   if (status != MM_SUCCESS)
   {
      goto cleanup;
   }

   /* Inform client of actual permissions and cache_mode. */
   request->permissions = region->permissions;
   request->cache_mode = region->cache_mode;

   /* Not possible to invalidate a SU_NA_USR_NA since it has no mapping */
   if(SU_NA_USR_NA != request->permissions)
   {
      /*
       * If this region is cache inhibit, ensure that no dirty lines
       * exist that can asynchronously be pushed out !!
       */
      if(request->cache_mode & MM_CACHE_INHIBITED)
      {
         cache_invalidate_d(log_base, request->size);
      }
   }
   /* Let client know address. */
   request->base = log_base;

   if (persistent)
   {
      /* Persistency attribute is updated by mm_pmm_alloc_sase. */
      mm_pmm_alloc_sase(region);
   }

   return MM_SUCCESS;

  cleanup:

   if (region != NULL)
   {
      mm_remove_region(&gmm_vars->domains, domain, region);
   }

   /* It is important that the physical memory always
    * is freed first since the logical allocator might
    * need it while freeing the logical resources. */
   if (pobj != NULL)
   {
      mm_detach_phys_object(pobj);
   }

   if (lobj != NULL)
   {
      mm_detach_log_object(lobj, domain);
   }

   return status;
}


static mm_status_t
validate_duplicate_mapping(struct mm_create_region *request)
{          
   struct region *region;                            
   struct region *prev_region = NULL;
   log_addr_t base;
   U32        size;
   U32        i, domid;
   OSBUFSIZE  ssize;
   U32 phys_resources_size;
   U32 nr_of_phys_resources;
   const U32 max_domains = gmm_state.max_domains;

   /* Check that the size of the phys_resources match phys_count. */
   ssize = sigsize((union SIGNAL **)&request);
   phys_resources_size = (ssize - sizeof(struct mm_create_region)) +
      sizeof(struct phys_resource);
   nr_of_phys_resources = phys_resources_size / sizeof(struct phys_resource);
  
   if(nr_of_phys_resources < request->phys_count)
   {
      return MM_EPHYS_FRAG_MISMATCH;
   }

   for(i = 0; i < request->phys_count; i++)
   {
      /* We can treat the physical address in phys_resources[i].base as a
       * logical address since it is SASE. */
      base = (log_addr_t)request->phys_resources[i].base;
      size = request->phys_resources[i].size;

      for (domid = 0; domid < max_domains; domid++)
      {
         int part_of_region;

         region = mm_find_region_by_addr(
            &gmm_state.domains, domid, base, FALSE);
         if(region == NULL)
         {
            /* Region not found in current domain, move on to the next */
            continue;
         }
            
	 /* The region owning the physical resources must be physical */
	 if (!region_is_physical(region))
	 {
            return MM_EPHYS_FRAG_ILLEGAL_SASE;	 
	 }
	 
         /* All supplied fragments must be part of the same region */
         if(i > 0)
         {
            if(region != prev_region)
            {
               PRINT_SIGNAL(("gmm: The supplied phys_resources "
                             "are not part of the same SASE region.\n"));
               return MM_EPHYS_FRAG_NOT_SAME_SASE;
            }     
         }

         prev_region = region;

         /* The supplied phys_resource must fit within the region. */
         part_of_region = mm_log_sub_area(base, size,
                                          region->log_object->base,
                                          region->log_object->size);
         if(!part_of_region)
         {
            PRINT_SIGNAL(("gmm: The supplied phys_resource(0x%08lx) "
                          "is not within the SASE region\n", base));
            return MM_EPHYS_FRAG_TOO_BIG;
         }

         if(region->permissions != SU_NA_USR_NA)
         {
            PRINT_SIGNAL(("gmm: The supplied phys_resource(0x%08lx) "
                          "maps a SASE region that is not SU_NA_USR_NA\n", base));
            return MM_EPHYS_FRAG_ILLEGAL_SASE;
         }

         /* Passed all checks. Move on to next fragment. */
         break;
      }

      /* Make sure a region was found in one of the domains */
      if (domid >= max_domains)
      {
         PRINT_SIGNAL(("gmm: The supplied phys_resource (0x%08lx)"
                       "is not part of any SASE region.\n", base));
         return MM_EPHYS_FRAG_OUTSIDE_SASE;
      }
   }                     

   return MM_SUCCESS;
}

/*
 * Create a SAS or MAS region.
 */
static mm_status_t
create_sas_or_mas_region(struct gmm_vars *gmm_state,
                         struct mm_create_region *request,
                         struct partition *partition,
                         struct allocator *allocator,
                         struct domain *domain,
                         mapping_type_t map_type)
{
   struct phys_object *pobj = NULL;
   struct log_object  *lobj = NULL;
   struct region *region    = NULL;
   log_addr_t bank_base = 0;
   int alloc_phys;
   U32 meta_options = 0, map_type_bits = 0;
   mm_status_t status = MM_SUCCESS;
   hal_status_t hal_status;
   mm_bool_t persistent;

   assert(gmm_state != NULL);
   assert(request != NULL);
   assert(partition != NULL);
   assert(partition->class->get_bank_info != NULL);
   

   /***************************************************************************
    *  Verify options
    */
   status = partition->class->get_bank_info(
      partition->class, 0, &bank_base, NULL);
   if (!status)
   {
      /* Partition exists, but has no configured logical bank */
      return MM_ENON_EXISTING_LOG_MEM;
   }   

   hal_status = hal_actual_permissions(
      &request->permissions, request->permissions, bank_base, 0);
   if (hal_status != HAL_SUCCESS)
   {
      return MM_EILLEGAL_PERMISSION;
   }

   hal_status = hal_actual_cache_mode(
      &request->cache_mode, request->cache_mode, bank_base);
   if (hal_status != HAL_SUCCESS)
   {
      return MM_EILLEGAL_CACHE_MODE;
   }

   switch (map_type)
   {
      case SASE:
         map_type_bits = MM_PARTITION_OPT_SASE;
         break;
      case SAS:
         map_type_bits = MM_PARTITION_OPT_SAS;
         break;
      case MAS:
         map_type_bits = MM_PARTITION_OPT_MAS;
         break;
      default:
         INTERNAL_ERROR();
   }

   /* Note that if client supplied the physical page addresses (the unusual
    * case ) allocator ==  NULL , i.e. avoid null references.
    */
   persistent = (request->options & MM_CREATE_REGION_PERSISTENT);
   alloc_phys = 1;

   if (persistent)
   {
      meta_options |= MM_REGION_OPT_PERSISTENT;
   }

   if (request->options & MM_CREATE_REGION_LOGICAL)
   {
      meta_options |= MM_REGION_OPT_LOGICAL;
      alloc_phys = 0;
   }

   if (request->options & MM_CREATE_REGION_FAULTABLE)
   {
      /* Faultable implies logical. */
      meta_options |= MM_REGION_OPT_LOGICAL | MM_REGION_OPT_FAULTABLE;
      alloc_phys = 0;
   }


   PRINT_SIGNAL(("gmm: create_sas_or_mas_region name: %s, base: %#x "\
                 "size: %#lx log_mem: %s allocator %#lx\n"\
                 "persistent %d alloc_phys: %d partition->name %s\n",
                 request->name, request->base, request->size,
                 request->log_mem,
                 allocator, persistent, alloc_phys, partition->class->name));

   /*
    * MAS intro checks.
    */
   if (map_type == MAS)
   {
      PRINT_MAS(("mm: MAS region detected. (permissions: %#lx)\n",
                 request->permissions));
      
      if (hal_get_info()->has_mas)
      {
         /* Only allow mas for data. (Const data should be part of text.) */
         if ( ! ((request->permissions & MM_SU_WRITE)  ||
                 (request->permissions & MM_USR_WRITE)))
         {
            /* FIXME(rand) - Remove when text in MAS is introduced */
            return MM_EMAS_DATA_ONLY;
         }
      }
      else
      {
         return MM_EMAS_NOT_SUPPORTED;
      }

      /* MAS data regions are not allowed in the system domain
       * since that will prevent creation of interrupt and timer
       * interrupt processes. */
      if (request->domain == MM_KERNEL_DOMAIN)
      {
         return MM_EMAS_NOT_SUPPORTED;
      }
   }

   /***************************************************************************
    *  Allocate logical resources.
    */
   lobj = mm_create_log_object();
   if (lobj == NULL)
   {
      status = MM_EOUT_OF_PHYS_MEM;
      goto cleanup;
   }

   do
   {
      if (request->options & MM_CREATE_REGION_SPECIFY_LOG)
      {
         uint32_t alloc_f_options =
            ((request->options & MM_CREATE_REGION_SEARCH_FORWARD) ?
             MM_PARTITION_OPT_SEARCH_FORWARD : 0) |
            ((request->options & MM_CREATE_REGION_SEARCH_REVERSE) ?
             MM_PARTITION_OPT_SEARCH_REVERSE : 0);

         assert(partition->class->alloc_f != NULL);

         status = partition->class->alloc_f(
            partition, lobj, request->size, request->base,
            map_type_bits | alloc_f_options);

         /* 
          * The if-case below is only here to support program loading
          * as it is currently implemented by PM. PM reserves logical 
          * memory in the system domain for a program's .data region
          * during the install phase (to prevent program creation from
          * failing if SAS mapping type is used).
          *
          * When a program is created, PM creates a new domain and uses
          * mm_create_region to create the program's .data-region in the
          * new domain at the same logical address. 
          *
          * The new logical allocator does not allow reallocation of the logical
          * memory in the new domain since it is considered used by the
          * logical region in the kernel domain. 
          *
          * Therefore we allow the creation of the region in the new domain
          * in the code below, but with these restrictions:
          *
          *   - The logical memory required must be used
          *   - The logical memory must belong to a system domain region
          *     and have exactly the same base and size.
          *   - The logical memory must not already be shared between system
          *     domain and another domain.
          * 
          * This behaviour should be changed as it allows anyone to reuse
          * logical SAS memory in a MAS fashion. 
          */
         if (alloc_phys && status == MM_EOVERLAPPING_ADDRESS)
         {
            struct region *local_region = mm_find_region_by_addr(
               &gmm_state->domains, request->domain, request->base, TRUE);
            struct region *region = mm_find_region_by_addr(
               &gmm_state->domains, MM_KERNEL_DOMAIN, request->base, TRUE);

            /* Not legal to populate a logical region in the same domain,
             * with mm_create_region. (Possible with mm_add_phys_resources) */
            if (region == NULL || local_region != NULL)
            {
               break;
            }

            /* A region was found in the system domain.
             * Make sure it is equally large, ...*/
            if (region->log_object->size != request->size)
            {
               break;
            }

            /* ... logical ... */
            if (region->phys_object != NULL)
            {
               break;
            }

            /* ... and not already shared or populated in another domain. */
            if (region->log_object->ref_count > 1)
            {
               break;
            }

            /* The logical region found in the system domain will now be
             * shared to the destination domain and populated with physical
             * resources, even though the logical SAS-memory was in use by
             * a system domain region.
             */

            lobj = region->log_object;
            status = MM_SUCCESS;
         }
      }
      else if (request->options & MM_CREATE_REGION_SPECIFY_ALIGN)
      {
         assert(partition->class->alloc != NULL);

         status = partition->class->alloc(
            partition, lobj, request->size, request->align, map_type_bits);
      }
      else
      {
         assert(partition->class->alloc   != NULL);

         status = partition->class->alloc(
            partition, lobj, request->size, 0, map_type_bits);
      }
      
      if (status == MM_EOUT_OF_LOG_MEM || status == MM_EOUT_OF_PHYS_MEM)
      {
         int reclaim = ose_mm_reclaim_component_memory(
            gmm_state, request->size, request->phys_mem);
         if (!reclaim)
         {
            break;
         }
      }

   } while (status == MM_EOUT_OF_LOG_MEM || status == MM_EOUT_OF_PHYS_MEM);

   
   if (status != MM_SUCCESS)
   {
      goto cleanup;
   }

   mm_attach_log_object(lobj, domain);

   request->base = lobj->base;


   /***************************************************************************
    *  Allocate physical resources.
    */

   MM_PROFILE_CHECKPOINT("logical region added", __FILE__, __LINE__);
   
   if (alloc_phys)
   {
      pobj = mm_create_phys_object(allocator, meta_options);
      if (pobj == NULL)
      {
         status = MM_EOUT_OF_PHYS_MEM;
         goto cleanup;
      }
   }

   MM_PROFILE_CHECKPOINT("phys alloc",  __FILE__, __LINE__);

   if (request->options & MM_CREATE_REGION_SPECIFY_PHYS)
   {
      if (request->options & MM_CREATE_REGION_DUPLICATE)
      {        
         status = validate_duplicate_mapping(request);
         if(status != MM_SUCCESS)
         {
            goto cleanup;
         }
      }
      
      /*
       * The client supplied the physical page addresses.
       * No need to allocate physical memory.
       */
      {
         union SIGNAL *sig;
         OSBUFSIZE size = sigsize((union SIGNAL **)&request);
         
         /*
          * Allocate new signal for saving supplied client pages.
          */
         sig = alloc(size, MM_CREATE_REGION_REQUEST);
         memcpy(sig, request, size);
         pobj->data.client_pages = &sig->mm_create_region;
         pobj->data_representation = PHYS_OBJ_CLIENT_PAGES;
      }
   }
   else if (alloc_phys)
   {
      /*
       * We need to allocate physical memory from an internal allocator.
       */

      /* Allocate physical memory. */
      assert(allocator != NULL);
      assert(allocator->alloc_s != NULL);

      do
      {
         status = allocator->alloc_s(
            allocator, pobj, request->size, lobj->base);
         
         MM_PROFILE_CHECKPOINT("alloc_s 1",  __FILE__, __LINE__);;

         
         if (status == MM_EOUT_OF_PHYS_MEM)
         {
             int reclaim = ose_mm_reclaim_component_memory(
                gmm_state, request->size, request->phys_mem);
             if (!reclaim)
             {
                break;
             }
         }
      } while (status == MM_EOUT_OF_PHYS_MEM);
      
      if (status != MM_SUCCESS)
      {
	 goto cleanup;
      }
   }

   if (pobj != NULL)
   {
      mm_attach_phys_object(pobj);
   }


   /***************************************************************************
    *  Tie resources together in a region.
    */
   
   /* Create a region. */
   meta_options |= MM_REGION_OPT_MAPPED;

   status = mm_add_region(
      request->domain,
      lobj,
      pobj,
      request->permissions,
      request->cache_mode,
      request->options,
      meta_options,
      request->name,
      request->log_mem,
      &region);

   MM_PROFILE_CHECKPOINT("phys done",  __FILE__, __LINE__);

   if (status != MM_SUCCESS)
   {
      goto cleanup;
   }

   /* Inform client of actual permissions and cache_mode. */
   request->permissions = region->permissions;
   request->cache_mode = region->cache_mode;

   /***************************************************************************
    *  1. Maintain cache coherency
    *  2. Maintain ram persistency
    */

   /* Invalidate data cache if and only if physical pages where mapped. */
   if (alloc_phys)
   {
      /* Not possible to invalidate a SU_NA_USR_NA since it has no mapping */
      if(request->permissions != SU_NA_USR_NA)
      {
         /*
          * If this region is cache inhibit, ensure that no dirty lines
          * exist that can asynchronously be pushed out !!
          */
         if(request->cache_mode & MM_CACHE_INHIBITED)
         {
            OSBOOLEAN rc;

            MM_PROFILE_CHECKPOINT("D-Cache Invalidate", "size",request->size);

            /* FIXME(rand) - Single mapping type assumption */
            if (!region_is_exclusive(region))
            {
               PRINT_SIGNAL(("gmm: cache invalidation base: %#lx size: %#lx\n",
                             lobj->base, request->size));
               /* Temporarily go to mas domain to be able to
                * do cache invalidate.
                */
               rc = set_segment(0, request->domain);

               if (rc != 0)
               {
                  ramlog_printf("mm: Invalidation of cache inhibit MAS region "\
                                "failed since the domain is not active. base: "\
                                "%#lx size: %#lx domain: %lx \n",
                                lobj->base, request->size, request->domain);
                  status = MM_EDOMAIN_NOT_MAS_ACTIVE;
                  goto cleanup;
               }

               cache_invalidate_d(lobj->base, request->size);
               rc = set_segment(1,0);                         /* Go back. */
               assert(rc == 0);
            }
            else
            {
               cache_invalidate_d(lobj->base, request->size);
            }
         }
      } /* if(request->permissions != SU_NA_USR_NA) */
     
      if (persistent)
      {
         /* Persistency attribute is updated by mm_alloc_sas. */
         mm_pmm_alloc_sas(region);
      }
   }

   MM_PROFILE_CHECKPOINT("D-Cache invalidate done", __FILE__, __LINE__);

   return MM_SUCCESS;

  cleanup:
   PRINT_SIGNAL(("gmm: sas_or_mas cleanup with logical resource: %#x "
                 "status : %#x\n", lobj, status));

   if (region != NULL)
   {
      mm_remove_region(&gmm_state->domains, domain, region);
   }

   /* It is important that the physical memory always
    * is freed first since the logical allocator might
    * need it while freeing the logical resources. */
   if (pobj != NULL)
   {
      mm_detach_phys_object(pobj);
   }

   if (lobj != NULL)
   {
      mm_detach_log_object(lobj, domain);
   }

   return status;
}


/*
 * Returns the total physical size occupied by the specified
 * physical fragments.
 * No checks for overlaps.
 */
static phys_size_t
total_frag_size(struct phys_resource *fragments,
                unsigned              no_fragments)
{
   unsigned   i = 0;
   phys_size_t total_size = 0;

   assert(NULL != fragments);

   while (i < no_fragments)
   {
      total_size += (phys_size_t)fragments[i].size;
      ++i;
   }

   return total_size;
}


/*
***********************************************************************
 *              MM_CREATE_REGION
 ***********************************************************************
 */
static union SIGNAL *
create_region(struct gmm_vars *gmm_state, union SIGNAL *request_generic)
{
   struct mm_create_region *request =
      (struct mm_create_region *) request_generic;
   struct partition *partition;
   struct domain *domain;
   mapping_type_t map_type;

   /* An internal allocator might be used. */
   struct allocator *allocator = NULL;

   mm_bool_t internal_allocator;

   assert(NULL != gmm_state);
   assert(NULL != request);

   /* This variable may be updated below. */
   internal_allocator = !(request->options & MM_CREATE_REGION_SPECIFY_PHYS);
#if 0
   PRINT_SIGNAL(("gmm: create_region called name: %s base: %#x size: %#lx "\
                 "cache: %#x permission: %#x option: %#x domain: %ld "\
                 "align: %#x log_mem: %s internal_allocator %d client: %#x\n",
                 request->name, request->base, request->size,
                 request->cache_mode, request->permissions ,request->options,
                 request->domain, request->align, request->log_mem,
                 internal_allocator, sender((union SIGNAL **)&request)));
#endif
   /* Make sure names has limited length. */
   truncate_strings(request);


   /* Only start profiling for regions above threshold. */
   if (request->size > PROFILE_SIZE)
   {
      MM_PROFILE_START("create_region", request->name,
                       request->size, XSTR(PROFIL_SIZE));
   }
   

   /***************************************************************************
    *  Verify options 
    */
   
   /* Find domain. */
   domain = mm_find_domain_by_domid(&(gmm_state->domains), request->domain);
   if (NULL == domain)
   {
      request->status = MM_ENON_EXISTING_DOMAIN;
      goto exit_error;
   }

   /*
    * Look up logmem element.
    * Either the <log_mem> field is used or the logmem element is
    * deduced from the <base> field.
    */
   if (request->options & MM_LOGMEM_FROM_ADDRESS &&
       request->options & MM_CREATE_REGION_SPECIFY_LOG)
   {
      request->log_mem[0] = '\0';
      partition = mm_find_domain_partition_by_addr(domain, request->base);

      if (partition == NULL)
      {
         request->status = MM_EOUTSIDE_LOG_MEM;
         goto exit_error;
      }

      request->status = mm_find_partition_mapping_type(
         partition->class, &request->log_mem[0], &map_type);
      if (request->status != MM_SUCCESS)
      {
         goto exit_error;
      }
   
   }
   else
   {
      partition = mm_find_domain_partition_by_name(
         domain, &request->log_mem[0]);

      if (partition == NULL)
      {
         request->status = MM_ENON_EXISTING_LOG_MEM;
         goto exit_error;
      }

      request->status = mm_find_partition_mapping_type(
         partition->class, &request->log_mem[0], &map_type);
      if (request->status != MM_SUCCESS)
      {
         goto exit_error;
      }
   }

   /* Check the attributes of resident regions */
   if (partition_is_resident(partition->class))
   {
      hal_status_t hal_rc;
      U32 perm;
            
      assert(!partition_is_implicit(partition->class));
      
      hal_rc = hal_actual_permissions(
         &perm,
         request->permissions,
         partition->class->banks->base,
         HAL_RESIDENT);
      if (hal_rc != HAL_SUCCESS)
      {
         request->status = MM_EILLEGAL_PERMISSION;
         goto exit_error;
      }

      hal_rc = hal_actual_cache_mode(
         &request->cache_mode,
         request->cache_mode,
         partition->class->banks->base);
      if (hal_rc != HAL_SUCCESS)
      {
         request->status = MM_EILLEGAL_CACHE_MODE;
         goto exit_error;
      }
      
      if(request->cache_mode != partition->class->res_cache)
      {
         request->status = MM_EILLEGAL_CACHE_MODE;
         goto exit_error;
      }

      if(map_type != SASE)
      {
         request->status = MM_EOPTION_NOT_SUPPORTED;
         goto exit_error;
      }
   }

   /* Check options against the mapping type of the partition.
    */
   request->status = check_options(request, map_type);
   if (request->status != MM_SUCCESS)
   {
      goto exit_error;
   }


   /***************************************************************************
    *  Create a new region.
    *
    *  1. Locate allocators
    *  2. Check options
    *  3. Create the region
    */

   if (internal_allocator)
   {
      allocator = mm_find_allocator_by_name(&request->phys_mem[0]);

      if (allocator == NULL)
      {
         /*
          * Special case: SASE regions without valid <phys_mem>
          * fields get accepted for 'ease-of-use' reasons.
          * Check that the address don't overlap with existing
          * allocators in order to catch typos (misspelled
          * <phys_mem>). Not valid for persistence regions.
          */
         if (!(request->options & MM_CREATE_REGION_SPECIFY_LOG))
         {
            request->status = MM_ENON_EXISTING_PHYS_MEM;
            goto exit_error;
         }

         /* Mapping type must be SASE. */
         if (map_type != SASE)
         {
            request->status = MM_ENON_EXISTING_PHYS_MEM;
            goto exit_error;
         }

         /* Persistent regions must have a valid phys_mem */
         if (request->options & MM_CREATE_REGION_PERSISTENT)
         {
            request->status = MM_ENON_EXISTING_PHYS_MEM;
            goto exit_error;
         }

#ifdef OSMIPS
         if ((request->base & KSEG_MASK) == KSEG0_BASE)
         {
            allocator = mm_find_allocator_by_range(
               request->base - KSEG0_BASE, request->size);
         }
         else if ((request->base & KSEG_MASK) == KSEG1_BASE)
         {
            allocator = mm_find_allocator_by_range(
               request->base - KSEG1_BASE, request->size);
         }
         else
#endif
         {
            allocator = mm_find_allocator_by_range(request->base, request->size);
         }

         if (allocator != NULL)
         {
            request->status = MM_ENON_EXISTING_PHYS_MEM;
            goto exit_error;
         }

         /* Request qualifies for implicit creation without phys_mem */
         internal_allocator = FALSE;
      }
   }

   /* Check that persistence in options matches persistence of allocator */
   {
      mm_bool_t request_persistent =
	 request->options & MM_CREATE_REGION_PERSISTENT;

      /* Persistent regions are only allowed in the system domain. */
      if (request_persistent && request->domain != MM_KERNEL_DOMAIN)
      {
	 request->status = MM_EILLEGAL_DOMAIN;
	 goto exit_error;
      }
   }

   /* Make sure that size is greater than zero */
   if (request->size == 0)
   {
      request->status = MM_ESIZE_IS_ZERO;
      goto exit_error;
   }

   /* Round size up to allocator alignment. */
   {
      log_size_t allocator_alignment =
	 internal_allocator ?
	 allocator->alignment : gmm_state->hal_info->page_sizes[0];

      log_size_t size_alignment =
	 MAX(allocator_alignment, gmm_state->hal_info->page_sizes[0]);

      request->size = mm_log_align(request->size, size_alignment);

      /* Make sure that base is properly aligned */
      if (request->options & MM_CREATE_REGION_SPECIFY_LOG &&
          !mm_log_aligned(request->base, size_alignment))
      {
         request->status = MM_EILLEGAL_BASE_ADDRESS;
         goto exit_error;
      }

      /* Make sure that the alignment is a power of two. */
      if (request->options & MM_CREATE_REGION_SPECIFY_ALIGN &&
          (request->align & (request->align - 1)) != 0)
      {
         request->status = MM_EILLEGAL_ALIGNMENT;
         goto exit_error;
      }

      if (request->options & MM_CREATE_REGION_SPECIFY_ALIGN &&
          request->align < gmm_state->hal_info->page_sizes[0])
      {
         request->align = gmm_state->hal_info->page_sizes[0];
      }
   }

   /* Make sure that size is greater than zero also after aligment */
   if (request->size == 0)
   {
      request->status = MM_EOUT_OF_LOG_MEM;
      goto exit_error;
   }

   /* Watch out for address wrap-around. */
   if (request->options & MM_CREATE_REGION_SPECIFY_LOG &&
       !mm_log_valid_area(request->base, request->size))
   {
      request->status = MM_EILLEGAL_AREA;
      goto exit_error;
   }

   /* Check permissions. */
   if (!mm_check_permission(request->permissions))
   {
      request->status = MM_EILLEGAL_PERMISSION;
      goto exit_error;
   }

   /* Check cache mode. */
   if (!mm_check_cache_mode(request->cache_mode))
   {
      request->status = MM_EILLEGAL_CACHE_MODE;
      goto exit_error;
   }

   {
      log_size_t actual_alignment;
      log_size_t allocator_alignment =
	 internal_allocator ?
	 allocator->alignment : gmm_state->hal_info->page_sizes[0];

      /* Calculate actual alignment. */
      if (request->options & MM_CREATE_REGION_SPECIFY_ALIGN)
      {
	 actual_alignment = MAX(request->align, allocator_alignment);
      }
      else
      {
         actual_alignment = allocator_alignment;
      }

      /* Let the client know actual alignment. */
      request->align = actual_alignment;
   }

   if (!internal_allocator)
   {
      /* Some extra checks for user-supplied physical memory. */

      PRINT_SIGNAL(("gmm: partition->options: %x "
                    "!request->options & MM_CREATE_REGION_SPECIFY_PHYS: %d\n",
                    partition->class->options,
                    !(request->options & MM_CREATE_REGION_SPECIFY_PHYS)));

      /* Due to the Special case: SASE regions without valid <phys_mem>
       * fields get accepted for 'ease-of-use' reasons.
       * On SASE we only perform checks if is explicitly set of
       * MM_CREATE_REGION_SPECIFY_PHYS.
       */
      if (map_type == SASE && 
          !(request->options & MM_CREATE_REGION_SPECIFY_PHYS))
      {
         goto sase_skip_user_supplied_checks;
      }

      /*
       * Very first check:
       * SASE must be only fragment supplied by client.
       */
      if (map_type == SASE)
      {
         if (request->phys_count > 1)
         {
            request->status = MM_EMISMATCH_FRAGMENTS;
            PRINT_SIGNAL(("gmm: mapping_type SASE must not have more than one "
                          "fragment, phys_count=%ld\n", request->phys_count));
            goto exit_error;
         }
      }
      /*
       * First check:
       * Verify that the sum of all physical size pages equals
       * the requested region size (after alignment).
       */
      {
         phys_size_t phys_frag_size =
            total_frag_size(&request->phys_resources[0],
                            request->phys_count);

         if (phys_frag_size != request->size)
         {
            request->status = MM_EMISMATCH_FRAGMENTS;
            PRINT_SIGNAL(("gmm: phys_frag_size(0x%08lx) must equals "
                          "request->size(0x%08lx)\n", phys_frag_size, request->size));
            goto exit_error;
         }
      }
      /*
       * Second check:
       * For all physical fragments, check if an internal allocator manages
       * the fragment, unless the option MM_CREATE_REGION_DUPLICATE is set. 
       */
      if (!(request->options & MM_CREATE_REGION_DUPLICATE))
      {
         unsigned i;

         for (i = 0; i < request->phys_count; i++)
         {
            struct phys_resource *phys_resource =
               &request->phys_resources[i];

            if (NULL != mm_find_allocator_by_range(phys_resource->base,
                                                   phys_resource->size))
            {
               /* Physical fragment overlaps with memory managed by
                * an internal allocator. */
               if (MM_SUCCESS == in_domain_phys_resource(&gmm_state->domains,
                                                         request->domain,
                                                         phys_resource->base,
                                                         phys_resource->size))
               {
                  /* Physical fragment not allocated to any region,
                   * i.e. in the physical resource list of the domain. */
                  continue;
               }

               /* Physical fragment allocated to another region -> error */

               request->status = MM_EPHYS_FRAG_OVERLAP_PHYS_MEM;
               goto exit_error;
            }
         }
      }
   }

  sase_skip_user_supplied_checks:

   /* Tentative regions are no longer supported, always clear the
    * tentative flag if clients still try to use the feature. */
   request->options &= ~MM_CREATE_REGION_TENTATIVE;

   /*
    * SASE
    */
   if (map_type == SASE)
   {
      request->status = create_sase_region(
         gmm_state, request, partition, allocator, domain);
   }
   else /* SAS, MAS */
   {
      request->status = create_sas_or_mas_region(
         gmm_state, request, partition, allocator, domain, map_type);
   }

   
   /* Handle low limit option. Note that this only can be done
    * after creation of the region since an unknown amount of
    * additional metadata might be required in order to create the
    * region. */
   if ((request->status == MM_SUCCESS) &&
       (request->options & MM_CHECK_LOW_LIMIT &&
        request->low_limit > 0))
   {
      /* Prepare a delete_region signal in case the
       * limit was exceeded and the region has to be
       * deleted. */
      struct mm_delete_region *delete_sig = (struct mm_delete_region *)
         alloc(sizeof(struct mm_delete_region), 0);
      delete_sig->sig_no = MM_DELETE_REGION_REQUEST;
      delete_sig->domain = request->domain;
      delete_sig->base = request->base;

      if (allocator->get_free_mem != NULL)
      {

         while (1)
         {
            size_t avail_size;
            size_t reclaim_size;

            avail_size = allocator->get_free_mem(allocator);

            /* A bit backwards but now we have to check that the
               newly created region did not lower remaining memory
               below the requested low_limit. */
            PRINT_SIGNAL(("gmm: create_region  low_limit flag, avail_size: "
                          "%#010lx  request->low_limit: %#010lx\n,",
                          avail_size, request->low_limit));

            if (avail_size < request->low_limit)
            {
               reclaim_size = request->low_limit - avail_size;
               /* Check if there is memory allocated to any instance regions,
                * which may be reclaimed. */
               if (ose_mm_reclaim_component_memory(
                      gmm_state, reclaim_size, request->phys_mem))
               {
                  continue;
               }

               
               delete_region(gmm_state, (union SIGNAL *)delete_sig);
               free_buf((union SIGNAL **)&delete_sig);
               request->status = MM_EOUT_OF_PHYS_MEM;
               goto exit_error;
            }

            free_buf((union SIGNAL **)&delete_sig);
            break;
         }
      }
      else
      {
         /* allocator do not support free_mem() -> let user know. */
         delete_region(gmm_state, (union SIGNAL *)delete_sig);
         free_buf((union SIGNAL **)&delete_sig);
         request->status = MM_EILLEGAL_OPERATION;
         goto exit_error;
      }
   }

  exit_error:

   if (request->status != MM_SUCCESS)
   {
      PRINT_FAILED_REQUEST(
         ("mm: mm_create_region failed with status=%#x, client=%#x\n",
          request->status, sender((union SIGNAL **)&request)));
      
      PRINT_FAILED_REQUEST(
         ("mm:    log_mem=%s, phys_mem=%s, options=%#x\n",
          request->log_mem, request->phys_mem, request->options));
      
      PRINT_FAILED_REQUEST(
         ("mm:    base=%#x, size=%#x, perm=%#x, cache=%#x, align=%#x\n",
          request->base, request->size, request->permissions,
          request->cache_mode, request->align));
   }

   request->sig_no = MM_CREATE_REGION_REPLY;
#if 0
   PRINT_SIGNAL(("gmm: create_region  exits with base: %#x "
                 "size: %#x cache: %#x permission: %#x option: %#x "\
                 "align: %#x result status: %#x client: %#x\n",
                 request->base, request->size, request->cache_mode,
                 request->permissions ,request->options, request->align,
                 request->status, sender((union SIGNAL **)&request)));
#endif
   MM_PROFILE_STOP(request->status, request->base);
   
   return request_generic;
}

mm_status_t
gmm_create_region(  
   U32                  *size,
   const OSDOMAIN       domain,
   const char           name[MM_NAME_LENGTH + 1],
   const char           phys_mem[MM_NAME_LENGTH + 1],
   const char           log_mem[MM_NAME_LENGTH + 1],
   U32                  *permissions,
   U32                  *cache_mode,
   OSADDRESS            *base,            
   U32                  *align,           /* optional, can be NULL */
   const U32             options,         /* optional, can be zero */
   const U32             low_limit,       /* optional, can be zero */
   U32                  *phys_count,      /* optional, can be NULL */
   struct phys_resource *phys_resources   /* optional, can be NULL */
   )

{   
   /* FIXME, would be better if create_region called this function
    * instead of the reverse. If done this function should not have do the
    * detour of embedding all function arguments in an allocated signal struct.
    */
   union SIGNAL *request, *reply;
   mm_status_t status;
   mm_bool_t   signal_allocated_from_pool = TRUE;

   extern OSBOOLEAN ose_started(void);
   (void) domain;
   
   /* If exec is not yet initiated, we cannot
    * use system calls.
    * */
   if (ose_started())
   {
      if (current_process() != gmm_state.gmm_pid)
      {
         *base = 0;
         status = MM_ENOT_GMM_PROCESS;
         return status;
      }
      request = alloc(sizeof(struct mm_create_region) +
                      (sizeof(struct phys_resource) * (*phys_count - 1)),
                      MM_CREATE_REGION_REQUEST);
   }
   else
   {
      mm_bool_t rc;
      
      /* Allow early region creations,
       * for example create hash table region. */
      rc = mm_page_alloc(gmm_state.hal_info->page_sizes[0],
                         (log_addr_t *) &request);
      if (rc != TRUE)
      {
         return MM_EOUT_OF_PHYS_MEM;
      }
      request->sig_no = MM_CREATE_REGION_REQUEST;
      signal_allocated_from_pool = FALSE;
   }

   /* Mandatory arguments. */
   request->mm_create_region.size = *size;
   request->mm_create_region.domain = domain;
   strcpy(request->mm_create_region.name, name);
   strcpy(request->mm_create_region.phys_mem, phys_mem);
   strcpy(request->mm_create_region.log_mem, log_mem);
   request->mm_create_region.permissions = *permissions;
   request->mm_create_region.cache_mode = *cache_mode;
   request->mm_create_region.base = *base;
   
   /* Optional arguments, ptrs have to be checked. */
   if (align == NULL) {
      request->mm_create_region.align = 0;
   } else {
      request->mm_create_region.align = *align;
   }

   request->mm_create_region.options = options;
   request->mm_create_region.low_limit = low_limit;

   if (phys_count == NULL ||  phys_resources == NULL) {
      request->mm_create_region.phys_count = 0;
   } else {
      request->mm_create_region.phys_count = *phys_count;
      memcpy(request->mm_create_region.phys_resources,
             phys_resources, *phys_count * (sizeof(struct phys_resource)));
   }
   
   reply = create_region(&gmm_state, request);

   /* Fill output arguments. */
   *size =  reply->mm_create_region.size;
   strcpy(reply->mm_create_region.phys_mem, phys_mem);
   strcpy(reply->mm_create_region.log_mem, log_mem);
   *permissions = reply->mm_create_region.permissions;
   *cache_mode = reply->mm_create_region.cache_mode;
   *base = reply->mm_create_region.base;
   
   /* Optional arguments, ptrs have to be checked. */
   if (align != NULL) {
      *align = reply->mm_create_region.align;
   }

   if (phys_count != NULL &&  phys_resources != NULL) {
      *phys_count = reply->mm_create_region.phys_count;
      memcpy(phys_resources,
             reply->mm_create_region.phys_resources,
             reply->mm_create_region.phys_count  *
             (sizeof(struct phys_resource)));
   }

   status = reply->mm_create_region.status;
   
   if (signal_allocated_from_pool)
   {
      free_buf(&reply);
   }
   else
   {
      mm_page_free((log_addr_t)reply, gmm_state.hal_info->page_sizes[0]);  
   }
   
   return status;
}


/*
 ***********************************************************************
 *              MM_DELETE_REGION
 ***********************************************************************
 */

static union SIGNAL *
delete_region(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   union SIGNAL *reply = request;
   OSDOMAIN dom_id;
   struct region *region;
   log_addr_t base = request->mm_delete_region.base;
   struct domain *domain;

   assert(gmm_vars != NULL);
   assert(request != NULL);
   reply->sig_no = MM_DELETE_REGION_REPLY;

   dom_id = request->mm_delete_region.domain;

   PRINT_SIGNAL(("gmm: delete_region  domain: %#x base:0x%08lx client: %#x\n", 
   				 dom_id, base, sender(&reply)));

   /* Is the domain valid? */
   domain = mm_find_domain_by_domid(&gmm_vars->domains, dom_id);
   if (domain == NULL)
   {
      reply->mm_delete_region.status = MM_ENON_EXISTING_DOMAIN;
      goto exit_error;
   }

   /* Is there an instance region in the domain? */
   region = mm_find_region_by_addr(
      &gmm_vars->domains, dom_id, base, TRUE); /* Exact match. */
   if (region == NULL)
   {
      reply->mm_delete_region.status = MM_ENON_EXISTING_REGION;
      goto exit_error;
   }

   /* Only start profiling for regions above threshold. */
   if (region->log_object->size > PROFILE_SIZE)
   {
      MM_PROFILE_START("delete_region", region->name,
                       region->log_object->size,
                       XSTR(PROFIL_SIZE));
   }


   /* Are we alloved to remove this region? */
   if (!region_is_static(region))
   {
      struct log_object  *lobj = region->log_object;
      struct phys_object *pobj = region->phys_object;

#ifdef OSPP
      make_unresident_range(gmm_vars, lobj->base, lobj->size);
#endif
      
      mm_remove_region(&gmm_vars->domains, domain, region);

      /* It is important that the physical memory always
       * is freed first since the logical allocator might
       * need it while freeing the logical resources. */
      if (pobj != NULL)
      {
         mm_detach_phys_object(pobj);
      }

      if (lobj != NULL)
      {
         /* Free logical resources */        
         mm_detach_log_object(lobj, domain);
      }

      reply->mm_delete_region.status = MM_SUCCESS;
   }
   else
   {
      reply->mm_delete_region.status = MM_ESYSTEM_REGION;
   }

  exit_error:

   PRINT_SIGNAL(("gmm: delete_region  exits with base: %#x "
                 "domain: %#x result status : %#x client: %#x\n",
                 base, dom_id, reply->mm_delete_region.status,
                 sender(&reply)));

   MM_PROFILE_STOP(reply->mm_delete_region.status,
                   reply->mm_delete_region.base);
   return reply;
}

/*
 ***********************************************************************
 *              MM_SHARE_REGION
 ***********************************************************************
 */

/*
 * Returns true if perm1 has a permission that perm 2 lacks.
 */
static mm_bool_t
compare_umode_perms(U32 perm1, U32 perm2)
{
   mm_bool_t read;
   mm_bool_t execute;
   mm_bool_t write;

   read = (perm1 & MM_USR_READ) && !(perm2 & MM_USR_READ);
   write = (perm1 & MM_USR_WRITE) && !(perm2 & MM_USR_WRITE);
   execute = (perm1 & MM_USR_EXECUTE) && !(perm2 & MM_USR_EXECUTE);

   return read | write | execute;
}

/*
 * This will return true if the new requested user mode permissions could
 * be implemented in the current HAL without changing the supervisor mode
 * access permissions. The result can be described like this.
 * Return TRUE and the new user mode permissions (result) if all of the
 * following is true:
 *
 *   - Please put something here. This has been redisgned about
 *     five time which makes me less inclined to write about it.
 */
static mm_bool_t
get_new_share_perms(U32 old_perms,
                    U32 new_requested_umode_perms,
                    U32 *result,
                    log_addr_t base)
{
   U32 new_requested_perms; /* Su-perm from log reg + req umode permissions. */
   U32 new_actual_perms; /* HAL-negotiated new su+usr permissions. */
   U32 old_actual_perms; /* HAL-negotiated old su+usr permissions. */
   U32 new_actual_umode_perms;
   U32 old_sumode_perms; /* Original su permissions. */
   U32 old_umode_perms; /* Original usr permissions. */
   U32 new_actual_sumode_perms; /* Su-perm of HAL-negotiated perms . */
   hal_status_t hal_status;

   new_requested_perms = (old_perms & ~0x7) | new_requested_umode_perms;

   PRINT_SIGNAL(("gmm: Set New umode perms, old_perms: 0x%x,  new_requested_perms: 0x%x\n",
                 old_perms, new_requested_perms));
   /*
    * When changning permissions, these must always be negotiated
    * with the HAL.
    */
   hal_status = hal_actual_permissions(
      &new_actual_perms, new_requested_perms, base, 0);
   PRINT_SIGNAL(("gmm: New hal permissions: 0x%x\n", new_actual_perms));

   if (hal_status != HAL_SUCCESS)
   {
      /*
       * HAL could not fulfill the new requested permissions.
       * Leave it unchanged (use old logical region permissions).
       */
      return FALSE;
   }

   hal_status = hal_actual_permissions(&old_actual_perms, old_perms, base, 0);
   (void) hal_status; /* Should not be a problem since region exists. */

   old_sumode_perms = old_perms & (0x7 << MM_SU_SHIFT);
   old_umode_perms = old_perms & 0x7;
   new_actual_sumode_perms = new_actual_perms & (0x7 << MM_SU_SHIFT);
   new_actual_umode_perms = new_actual_perms & 0x7;

   if (old_sumode_perms != new_actual_sumode_perms)
   {
      /*
       * Could not get the new permissions without changing
       * the supervisor mode permissions
       */
      ramlog_printf("Had to change supervisor mode permissions.\n");
      ramlog_printf("old_su_perms: 0x%x, new_actual_sumode_perms: 0x%x",
                    old_perms, new_actual_sumode_perms);

      return FALSE;
   }

   if (compare_umode_perms(new_actual_umode_perms, old_umode_perms))
   {
      /*
       * The new user mode permission can't be allowed to be better
       * than the original user mode permissions.
       */
      ramlog_printf("New actual user mode permissions better than original\n");
      ramlog_printf("new_actual_umode_perms: 0x%x, old_umode_perms: 0x%x\n",
                    new_actual_umode_perms, old_umode_perms);

      return FALSE;
   }

   if (!mm_check_permission(new_actual_perms))
   {
      /* The new permissions do not preserve the rule that supervisor mode
       * should be more powerful.
       */

      ramlog_printf("Did not pass check_permission check\n");

      return FALSE;
   }

   /*
    * We managed to change the user mode permissions for the region
    * in the destination domain with preserved supervisor mode
    * permissions.
    */
   *result = new_actual_umode_perms;

   return TRUE;
}

static union SIGNAL *
share_region(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   union SIGNAL               *reply = request;
   struct mm_share_region     *req = &request->mm_share_region;
   OSDOMAIN                    source = req->source;
   OSDOMAIN                    destination = req->destination;
   log_addr_t                  base = req->base;
   struct log_object          *lobj = NULL;
   struct phys_object         *pobj = NULL;
   struct region              *region = NULL;
   struct domain              *src_domain;
   struct domain              *dst_domain;
   mm_status_t                 status;
   mm_bool_t                   perm_change_possible = FALSE;
   U32 new_permissions = 0;   /* Remove arm-rvct warning. */
   U32 new_meta_options = 0;

   PRINT_SIGNAL(("gmm: share_region called base: %#x src domain: %d "\
                 "dest domain: %d options: %#x permission: %#x client: %#x\n ",
                 req->base, req->source, req->destination,
                 req->options, req->permission, sender(&reply)));

   reply->sig_no = MM_SHARE_REGION_REPLY;

   /***************************************************************************
    *  Verify preconditions and options
    */

   /* Make sure the existance of source as well as destination domains. */
   src_domain = mm_find_domain_by_domid(&gmm_vars->domains, source);
   dst_domain = mm_find_domain_by_domid(&gmm_vars->domains, destination);
   if (src_domain == NULL || dst_domain == NULL)
   {
      reply->mm_share_region.status = MM_ENON_EXISTING_DOMAIN;
      goto exit_error;
   }

   /* Make sure <src_domain> != <dest_domain>. */
   if (source == destination)
   {
      reply->mm_share_region.status = MM_EILLEGAL_DOMAIN;
      goto exit_error;
   }

   /* Look up the region in the source domain (exact match). */
   region = mm_find_region_by_addr(
      &gmm_vars->domains, source, base, TRUE);

   /* Make sure the source region exists. */
   if (region == NULL)
   {
      reply->mm_share_region.status = MM_ENON_EXISTING_REGION;
      goto exit_error;
   }

   /* Verify that the destination domain has no region in the share area. */
   {
      struct region *dest_region = mm_find_region_by_addr(
         &gmm_vars->domains, destination, base, TRUE);

      if (NULL != dest_region)
      {
         reply->mm_share_region.status = MM_ETOO_MANY_INSTANCES;
         goto exit_error;
      }
   }   

   /* Only start profiling for regions above threshold. */
   if (region->log_object->size > PROFILE_SIZE)
      {
      MM_PROFILE_START("share_region", region->name,
                       region->log_object->size, XSTR(PROFIL_SIZE));
   }

   if(!region_is_resident(region)) 
   {

      /* If the src domain is the system domain mapped with global PID=0.
       * avoid double hit in TLB (OSECR-8881) by making the region in system
       * domain non-global mapped with TID=1. */
      if (source == gmm_state.system_domain->segnum &&
          region_is_tlb_global(region))
      {
         int sys_region_options;
         hal_status_t hal_status;

         /* Set region to be non TLB global, must be done before
          * call map_area, since it is the global bit we want to disable
          * in the change in the hal mmu mappings.
          */
         region->meta_options &= ~MM_REGION_OPT_TLB_GLOBAL;
         /* Indicate that region is changing from global to non global makes
            the tlb_invalidation faster for some architectures. (PPC PEM) */ 
         region->meta_options |=  MM_REGION_OPT_TLB_GLOBAL_TRANSIENT;

         
         /* Call HAL to change the MMU mappings to be non TLB global
          */
         sys_region_options = mm_set_hal_options(gmm_state.system_domain,
                                                 region, TRUE /* Change */); 
         
         hal_status = mm_map_area(gmm_state.system_domain->hal_domain,
                                  region->phys_object,
                                  region->log_object->base,
                                  sys_region_options,
                                  region->permissions,
                                  region->cache_mode);
         
         region->meta_options &= ~MM_REGION_OPT_TLB_GLOBAL_TRANSIENT;

         
         if (hal_status != HAL_SUCCESS) {
            INTERNAL_ERROR();
         }
      }
   }
   /***************************************************************************
    *  Obtain logical resources, or allocate new. 
    */

   lobj = region->log_object;
   assert(lobj != NULL);

#if 0 /* FIXME(rand) - Enable when true MAS is introduced */
   /* Allocate new logical memory if it is non-exclusive */
   if (!region_is_exclusive(region))
   {
      struct partition *partition =
         mm_find_domain_partition_by_addr(dst_domain, base);

      lobj = NULL;
      
      if (partition == NULL)
      {
         reply->mm_share_region.status = MM_ENON_EXISTING_REGION;
         goto exit_error;
      }
      
      lobj = mm_create_log_object(partition);
      if (lobj == NULL)
      {
         reply->mm_share_region.status = MM_EOUT_OF_PHYS_MEM;
         goto exit_error;
      }

      /* FIXME(rand) - Assuming straight share. Add offset sharing. */
      status = partition->class->alloc_f(
         partition, lobj,
         region->log_object->size,
         region->log_object->base,
         region->log_object->options & MM_PARTITION_MAPTYPE_MASK);
      if (status != MM_SUCCESS)
      {
         reply->mm_share_region.status = status;
         goto exit_error;
      }
   }
#else
   /* Sharing a MAS region is not allowed */
   if (!region_is_exclusive(region))
   {
      lobj = NULL;  /* Not yet attached */
      reply->mm_share_region.status = MM_EILLEGAL_OPERATION;
      goto exit_error;
   }
#endif

   mm_attach_log_object(lobj, dst_domain);


   /***************************************************************************
    *  Obtain physical resources. 
    */

   /* Look up physical memory object of the source region. */
   pobj = region->phys_object;

   /* A regions must have physical memory in order to be shared. */
   if (pobj == NULL)
   {
      reply->mm_share_region.status = MM_EREGION_LOGICAL;
      goto exit_error;
   }

   mm_attach_phys_object(pobj);


   if (req->options & MM_SHARE_REGION_UPERM)
   {
      U32 new_actual_umode_perms = 0;

      perm_change_possible = get_new_share_perms(
         region->permissions, req->permission & 0x7,
         &new_actual_umode_perms, base);

#if 0
      ramlog_printf("perm_change_possible=%d\n", perm_change_possible);
      ramlog_printf("new_actual_umode_perms=0x%x\n", new_actual_umode_perms);
#endif

      if (!perm_change_possible)
      {
         /*
          * Changing to new requested user mode permissions with preserved
          * user mode permissions was not possible with preserved supervisor
          * mode permissions.
          *
          * User mode permissions from the original region are reused. 
          */
         reply->mm_share_region.permission = 0;
         reply->mm_share_region.status = MM_ESHARE_UMODE_RESTRICT_FAILED;
         goto exit_error;
      }
      else
      {
         /* Set the possibly new user mode permission in the reply signal. */
         reply->mm_share_region.permission = new_actual_umode_perms & 0x007;
         new_permissions  = region->permissions & 0x700;
         new_permissions |= new_actual_umode_perms & 0x007;
      }
   }
   else
   {
      new_permissions = region->permissions;
      reply->mm_share_region.permission = new_permissions & 0x7;
   }

#ifdef OSPP
   if (region->permissions != new_permissions)
   {
      make_unresident_range(gmm_vars, lobj->base, lobj->size);
   }
#endif
   

   /* Destination region is not static, nor persistent */
   new_meta_options  = region->meta_options;
   new_meta_options &= ~MM_REGION_OPT_INITIAL_STATIC;
   new_meta_options &= ~MM_REGION_OPT_PERSISTENT;
   
   /* Create a new instance region in the destination domain. */
   status = mm_add_region(destination,
                          lobj,
                          pobj,
                          new_permissions,
                          region->cache_mode,
                          region->options,
                          new_meta_options,
                          region->name,
                          region->log_mem,
                          &region);

   reply->mm_share_region.status = status;

   if (status == MM_SUCCESS)
   {
      goto exit_sucess;
   }

  exit_error:
   /* It is important that the physical memory always
    * is freed first since the logical allocator might
    * need it while freeing the logical resources. */
   if (pobj != NULL)
   {
      mm_detach_phys_object(pobj);
   }
   
   if (lobj != NULL)
   {
      mm_detach_log_object(lobj, dst_domain);
   }

  exit_sucess:   
   PRINT_SIGNAL(("gmm: share_region  performed with base %#x "
                 "src domain: %d dest domain: %d "\
                 "result status: %#x "\
                 "new_actual_umode_perms: %#x client: %#x\n",
                 base, source, destination, reply->mm_share_region.status,
                 new_permissions & 0x7, sender(&reply)));

    MM_PROFILE_STOP(reply->mm_share_region.status,
                    reply->mm_share_region.base);
   return reply;
}



/*
 ***********************************************************************
 *              MM_WRITE
 ***********************************************************************
 */


static mm_status_t
write_wrapper(struct phys_object *phys_object,
	      log_addr_t src,
	      log_addr_t dest,
	      log_size_t len)
{
   struct allocator *allocator;

   assert(NULL != phys_object);
   assert(NULL != phys_object->allocator);

   allocator = phys_object->allocator;
   return allocator->write(allocator, src, dest, len);
}


static union SIGNAL *
write_region(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   struct mm_write *req = &request->mm_write;
   struct region *dst_region;
   struct domain *dst_domain;
   OSDOMAIN dst_domain_id = MM_KERNEL_DOMAIN;
   log_size_t accessible_length;
   unsigned int validation_result;

   assert(gmm_vars != NULL);
   assert(request != NULL);

   PRINT_SIGNAL(("gmm: write_region  target: %#lx source: %#lx size: %#lx "\
                 "client: %#x\n",
                 req->target, req->source, req->size, sender(&request)));

   /* Deny zero-sized requests. */
   if (req->size == 0)
   {
      req->status = MM_ESIZE_IS_ZERO;
      goto exit_error;
   }

   /* Deny overlapping addresses. Such addresses either indicate that
    * the user wishes to copy data from one MAS region to another MAS
    * region which is not supported, or that the user wishes to copy
    * data within the same region in which case memcpy would suffice.
    */
   if (mm_log_inside(req->target, req->source, req->size))
   {
      req->status = MM_EILLEGAL_OPERATION;
      goto exit_error;
   }

   /* Validate source region accessibility.
    * The source must be readable an not MAS-mapped. */
   accessible_length = (log_addr_t)mm_validate_buffer(
      req->source, req->size, 0,
      MM_PERM_READ, MM_VALIDATE_IN_MAS, &validation_result);
   if (accessible_length < req->size)
   {
      req->status = MM_ENO_READ_PERMISSION;
      goto exit_error;
   }

   if (validation_result & MM_VALIDATE_IN_MAS)
   {
      req->status = MM_EMAS_NOT_SUPPORTED;
      goto exit_error;
   }

   /* Find domain, error if fail */
   dst_domain = mm_find_domain_by_domid(&gmm_vars->domains, req->domain);
   if (dst_domain == NULL)
   {
      req->status = MM_ENON_EXISTING_DOMAIN;
      goto exit_error;
   }

   /* Find destination region. */
   dst_region = mm_find_region_by_addr(
      &gmm_vars->domains, dst_domain->segnum, req->target, FALSE); /* Fuzzy match. */
   if (dst_region == NULL)
   {
      req->status = MM_ENON_EXISTING_REGION;
      goto exit_error;
   }

   /* Can we write to the target region? */
   if ((dst_region->permissions & MM_SU_WRITE) == 0)
   {
      /* No. Bail out. */
      req->status = MM_ENO_WRITE_PERMISSION;
      goto exit_error;
   }

   /* Is dst region a MAS region? If so, indicate need to set_segment() */
   if (!region_is_exclusive(dst_region))
   {
      dst_domain_id = dst_domain->segnum;
   }

   /*
    * Now perform the actual write operation.
    */
   {
      struct phys_object *pobj = dst_region->phys_object;
      struct allocator *allocator;

      /* Does the region have any physical memory? */
      if (pobj == NULL)
      {
	 req->status = MM_EREGION_LOGICAL;
	 goto exit_error;
      }

      /* We have an phys_object, i.e. the region has physical memory. */
      allocator = pobj->allocator;

      /* Do we have an allocator attached? */
      if (allocator == NULL)
      {
	    /*
	     * No allocator -> client supplied physical memory.
	     * Since the physical memory located at client-supplied
	     * addresses can be of any kind, i.e. read-only, we must
	     * take the careful stance and deny write operations.
	     */
	    req->status = MM_EUSER_SUPPLIED_PHYSICAL;
	    goto exit_error;
      }

      /*
       * Yes. This implies that the physical memory was allocated
       * from an internal allocator.
       * Now we must check if the allocator permits writing.
       * If the write-function has a valid address,
       * we can write (RAM, FLASH, etc.).
       * If the write-function == NULL, we can't write
       * (memory-mapped I/O or some other esoteric physical memory).
       */
      if (allocator->write == NULL)
      {
	 req->status = MM_EWRITE_NOT_IMPLEMENTED;
	 goto exit_error;
      }

      /* Write is permitted. */
      if (dst_domain_id != MM_KERNEL_DOMAIN)
      {
         set_segment(0, dst_domain_id);
      }
      
      req->status = write_wrapper(
         pobj, req->source, req->target, req->size);

      if (dst_domain_id != MM_KERNEL_DOMAIN)
      {
         set_segment(1, 0);
      }
  }

  exit_error:
   /* The request signal is also used as the reply signal. */
   req->sig_no = MM_WRITE_REPLY;
   return request;
}



/*
 ***********************************************************************
 *              MM_CREATE_DOMAIN
 ***********************************************************************
 */
static union SIGNAL *
create_domain(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   struct mm_create_domain *req_sig = (struct mm_create_domain *) request;

   assert(NULL != request);
   assert(NULL != gmm_vars);

   /* Make sure domain name has limited length. */
   if (req_sig->options & MM_CREATE_DOMAIN_SPECIFY_NAME)
   {
      req_sig->name[MM_NAME_LENGTH] = '\0';
   }
   else
   {
      req_sig->name[0] = '\0';
   }

   /* Try to allocate a new domain. */
   {
      struct domain *domain;
      mm_status_t status =
         mm_add_domain(&gmm_vars->domains, req_sig->name, &domain);

      /* Update status code in reply signal. */
      req_sig->status = status;

      if (MM_SUCCESS == status)
      {
	 /* Success. Show new domain to client. */
	 req_sig->domain = domain->segnum;
      }
   }

   /* The request signal is also used as the reply signal. */
   req_sig->sig_no = MM_CREATE_DOMAIN_REPLY;

   return request;
}



/*
 ***********************************************************************
 *              MM_DELETE_DOMAIN
 ***********************************************************************
 */
static union SIGNAL *
delete_domain(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   union SIGNAL *reply = request;
   OSDOMAIN      domain = request->mm_delete_domain.domain;

   if (MM_KERNEL_DOMAIN == domain)
   {
      /* Cannot delete the system domain. */
      reply->mm_delete_domain.status = MM_ESYSTEM_DOMAIN;
   }
   else
   {
      /* First check if <domain> exists. */
      if (NULL != mm_find_domain_by_domid(&gmm_vars->domains, domain))
      {

         /* Yes it does. Try to remove it. */
         if (mm_remove_domain(&gmm_vars->domains, domain))
         {
            /* Ok. */
            reply->mm_delete_domain.status = MM_SUCCESS;
         }
         else
         {
            /* Failure. Tried to remove non-removable domain (kernel domain) */
            reply->mm_delete_domain.status = MM_EILLEGAL_OPERATION;
         }
      }
      else
      {
         reply->mm_delete_domain.status = MM_ENON_EXISTING_DOMAIN;
      }
   }

   /* The request signal is also used as the reply signal. */
   reply->sig_no = MM_DELETE_DOMAIN_REPLY;
   return reply;
}


/*
 ***********************************************************************
 *              MM_TRANSLATE_ADDRESS
 ***********************************************************************
 */
static union SIGNAL *
translate_address(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   int status;
   struct mm_translate_address *req_sig = &request->mm_translate_address;
   struct domain *domain;
   struct region *region;
   struct phys_object *pobj;
   phys_addr_t phys_addr;
   phys_addr_t start_phys_addr;
   phys_size_t frag_len;
   phys_size_t offset;
   

   assert(NULL != gmm_vars);
   assert(NULL != request);

   req_sig->sig_no = MM_TRANSLATE_ADDRESS_REPLY;

   /* First check if <domain> exists. */
   domain = mm_find_domain_by_domid(&gmm_vars->domains, req_sig->domain);
   if (domain == NULL)
   {
      req_sig->status = MM_ENON_EXISTING_DOMAIN;
      return (union SIGNAL*)req_sig;
   }

   /* Look up region that maps <log_addr>. (Fuzzy match) */
   region =  mm_find_region_by_addr(
      &gmm_vars->domains, req_sig->domain, req_sig->log_addr, FALSE);
   if (region == NULL)
   {
      req_sig->status = MM_ENON_EXISTING_REGION;
      return (union SIGNAL*)req_sig;
   }

   /*
    * Check if region has physical resources. 
    */
   pobj = region->phys_object;
   if (pobj == NULL)
   {
      req_sig->status = MM_EREGION_LOGICAL;
      return (union SIGNAL*)req_sig;
   }

   offset = req_sig->log_addr - region->log_object->base;

   /* Translate. */
   status = mm_extract_phys_addr(
      pobj, offset, &phys_addr, &start_phys_addr, &frag_len);
   if (!status)
   {
      /* This should not happen.*/
      INTERNAL_ERROR();
   }

   /* The request signal is also used as the reply signal. */
   req_sig->phys_addr = phys_addr;
   req_sig->phys_page = start_phys_addr;
   req_sig->status = MM_SUCCESS;
   return (union SIGNAL*)req_sig;
}


/*
 ***********************************************************************
 *              MM_GET_DOMAIN_INFO
 ***********************************************************************
 */
static union SIGNAL *
get_domain_info(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   struct mm_get_domain_info *info_request;
   struct domain *domain;

   info_request = &request->mm_get_domain_info;
   request->sig_no = MM_GET_DOMAIN_INFO_REPLY;
   domain = mm_find_domain_by_domid(
      &gmm_vars->domains, info_request->domain);

   if (domain == NULL)
   {
      info_request->status = MM_ENON_EXISTING_DOMAIN;
      return request;
   }

   strlcpy(info_request->name, &domain->name[0], sizeof(domain->name));
   if (info_request->count > 0)
   {
      info_request->status = MM_EILLEGAL_TAG;
      info_request->err_index = 0;
   }
   else
   {
      info_request->status = MM_SUCCESS;
   }

   return request;
}


/*
 ***********************************************************************
 *              MM_GET_REGION_INFO
 ***********************************************************************
 */

static U32
get_options(struct region *region)
{
   U32 options = 0; /* Init option field. */

   /* Volatile memory is default memory flavor,
    * check if the region is of any of the other two possible flavor's:
    * persistence or logical.
    */
   if (region_is_logical(region))
   {
      options |= MM_CREATE_REGION_LOGICAL;
   }

   if (region_is_faultable(region))
   {
      /* Faultable implies logical */
      options |= MM_CREATE_REGION_LOGICAL | MM_CREATE_REGION_FAULTABLE;
   }

   if (region_is_persistent(region))
   {
      options |= MM_CREATE_REGION_PERSISTENT;
   }

   if (region->name[0] != '\0')
   {
      options |= MM_CREATE_REGION_SPECIFY_NAME;
   }

   if (region->phys_object != NULL &&
       region->phys_object->data_representation == PHYS_OBJ_CLIENT_PAGES)
   {
      options |= MM_CREATE_REGION_SPECIFY_PHYS;
   }

   if(region_is_resident(region))
   {
      options |= MM_CREATE_REGION_RESIDENT;
   }

   /* FIXME it would be more accurate to read this info from pte.
    * this requires mmu_hal_lookup to be extended
    */
   if(region_is_tlb_global(region))
   {
      options |= MM_CREATE_REGION_TLB_GLOBAL;
   }

   if(region_is_static(region))
   {
      options |= MM_CREATE_REGION_INITIAL_STATIC;
   }

   if(region_is_initial_dynamic(region))
   {
      options |= MM_CREATE_REGION_INITIAL_DYNAMIC;
   }
   
   return options;
}

static struct mm_create_region *
alloc_reply_sig(struct region *region, log_addr_t base, OSDOMAIN domain)
{
   struct mm_create_region *sig;

   assert(region != NULL);

   sig = (struct mm_create_region *)alloc(
      sizeof(*sig) + (PHYS_RES_MAX - 1) * sizeof(sig->phys_resources),
      MM_GET_REGION_INFO_REPLY);

   sig->status = MM_SUCCESS;
   sig->domain = domain;
   sig->base = base;
   sig->size = region->log_object->size;

   sig->permissions = region->permissions;
   sig->cache_mode  = region->cache_mode;
   strlcpy(sig->name, region->name, sizeof(sig->name));
   strlcpy(sig->phys_mem,
           mm_get_phys_mem(region),
           sizeof(sig->phys_mem));
   strlcpy(sig->log_mem, region->log_mem, sizeof(sig->log_mem));

   sig->options = get_options(region);
   sig->align = 0xFFFFFFFF; /* Alignment is undefinded in this signal. */

   sig->phys_count = 0;
   return sig;
}

static union SIGNAL *
get_region_info(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   mm_status_t status;
   struct domain *domain;
   struct region *region;
   log_addr_t base;
   struct phys_object *pobj;
   struct mm_create_region *reply;

   assert(gmm_vars != NULL);
   assert(request != NULL);

   reply = &request->mm_create_region;
   reply->sig_no = MM_GET_REGION_INFO_REPLY;


   /* First validate the domain. */
   domain = mm_find_domain_by_domid(
      &gmm_vars->domains, request->mm_create_region.domain);
   if (domain == NULL)
   {
      status = MM_ENON_EXISTING_DOMAIN;
      goto exit_error;
   }
   

   base = reply->base;

   /* Then look up region. */
   region = mm_find_region_by_addr(
      &gmm_vars->domains, domain->segnum, base, TRUE);
   if (region == NULL)
   {
      status = MM_ENON_EXISTING_REGION;
      goto exit_error;
   }


   pobj = region->phys_object;

   assert(region->log_object != NULL);

   /* Allocate reply signal with region information filled in */
   reply = alloc_reply_sig(region, base, domain->segnum);

   /* If region has no physical resources, reply immediately */
   if (pobj == NULL)
   {
      free_buf(&request);
      reply->status = MM_SUCCESS;
      return (union SIGNAL*)reply;
   }


   /* Add info about physical resources */
   {
      unsigned i;
      struct phys_mem_cursor cursor;
      struct mm_create_region *prev_reply;
      PROCESS client_;

      i = 0;
      mm_phys_mem_get_first(pobj, &cursor);
      prev_reply = NULL;
      reply->phys_resources[i].base = cursor.phys_page.pa;
      reply->phys_resources[i].size = cursor.phys_page.size;
      reply->phys_count = ++i;
      client_ = sender(&request);

      /*
       * Traverse through all physical fragments and stuff them
       * into reply signals. Since the signal numbers indicate
       * if there are more signals to come, we must use a
       * lookahead mechanism.
       */
      while (mm_phys_mem_get_next(&cursor))
      {
         if (reply == NULL)
         {
            reply = alloc_reply_sig(region, base, domain->segnum);
            if (prev_reply != NULL)
            {
               prev_reply->sig_no = MM_GET_REGION_INFO_NOT_LAST;
               send((union SIGNAL **)&prev_reply, client_);
               prev_reply = NULL;
            }
         }
         reply->phys_resources[i % PHYS_RES_MAX].base =
            cursor.phys_page.pa; /*lint !e661 */
         reply->phys_resources[i % PHYS_RES_MAX].size =
            cursor.phys_page.size; /*lint !e661 */
         reply->phys_count = ++i;
         
         if (i % PHYS_RES_MAX == 0)
         {
            prev_reply = reply;
            reply = NULL;
            i = 0;
         }
      }

      assert((reply != NULL && prev_reply == NULL) ||
             (prev_reply != NULL && reply == NULL));

      if (region_is_persistent(region) && domain == MM_KERNEL_DOMAIN)
      {
         reply->options |= MM_CREATE_REGION_PERSISTENT;
      }

      if (reply == NULL)
      {
         reply = prev_reply;
      }
   }
   
   free_buf(&request);
   /* The type of sig should maybe be changed */
   return (union SIGNAL*)(void*)reply;

  exit_error:

   reply->status = status;
   return (union SIGNAL*)(void*)reply;
}



/*
 ***********************************************************************
 *              MM_GET_PHYS_MEM_INFO
 ***********************************************************************
 */
union SIGNAL *
mm_get_phys_mem_info(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   union SIGNAL     *reply;
   struct allocator *allocator;

   (void)gmm_vars;
   
   reply = request;

   /* Make sure name is terminated. */
   request->mm_get_phys_mem_info.name[MM_NAME_LENGTH] = '\0';

   /* Look up allocator. */
   allocator = mm_find_allocator_by_name(request->mm_get_phys_mem_info.name);
   if (NULL != allocator)
   {
      reply->mm_get_phys_mem_info.status = MM_SUCCESS;
      allocator->info(allocator, request, &reply);
   }
   else
   {
      reply->mm_get_phys_mem_info.status = MM_ENON_EXISTING_PHYS_MEM;
   }

   reply->sig_no = MM_GET_PHYS_MEM_INFO_REPLY;

   return reply;
}



/*
 ***********************************************************************
 *              MM_GET_PHYS_MEM_TYPE_INFO
 ***********************************************************************
 */

static union SIGNAL *
get_phys_mem_type_info(union SIGNAL *request)
{
   union SIGNAL     *reply;
   struct allocator *allocator;

   reply = request;
   reply->sig_no = MM_GET_PHYS_MEM_TYPE_INFO_REPLY;

   /* Make sure name is terminated. */
   request->mm_get_phys_mem_type_info.name[MM_NAME_LENGTH] = '\0';

   /* Look up allocator. */
   allocator = mm_find_allocator_by_name(request->mm_get_phys_mem_type_info.name);
   if (NULL != allocator)
   {
      unsigned ix;

      reply->mm_get_phys_mem_type_info.status = MM_SUCCESS;

      for (ix = 0; ix < request->mm_get_phys_mem_type_info.count; ix++)
      {
         switch (request->mm_get_phys_mem_type_info.info[ix].tag)
         {
            case MM_TAG_INFO_PHYS_MEM_TOTAL_MEMORY:
            {
               reply->mm_get_phys_mem_type_info.info[ix].value =
                  allocator->get_total_mem(allocator);
               break;
            }
            case MM_TAG_INFO_PHYS_MEM_FREE_MEMORY:
            {
               reply->mm_get_phys_mem_type_info.info[ix].value =
                  allocator->get_free_mem(allocator);
               break;
            }
            case MM_TAG_INFO_PHYS_MEM_FRAG_COUNT:
            {
               reply->mm_get_phys_mem_type_info.info[ix].value =
                  allocator->get_frag_count(allocator);
               break;
            }

            case MM_TAG_INFO_PHYS_MEM_BIGGEST_FRAG:
            {
               reply->mm_get_phys_mem_type_info.info[ix].value =
                  allocator->get_biggest_frag(allocator);
               break;
            }
            case MM_TAG_INFO_PHYS_MEM_INACTIVE_MEMORY: /* Deprecated */
            {
               reply->mm_get_phys_mem_type_info.info[ix].value = 0;
               break;
            }
            case MM_TAG_INFO_PHYS_MEM_FREE_WATERMARK:
            {
               reply->mm_get_phys_mem_type_info.info[ix].value =
                  allocator->get_free_watermark(allocator);
               break;
            }
            case MM_TAG_INFO_PHYS_MEM_METADATA_SIZE:
            {
               reply->mm_get_phys_mem_type_info.info[ix].value +=
                  allocator->get_metadata_size(allocator);
               break;
            }            
            default:
            {
               reply->mm_get_phys_mem_type_info.status = MM_EILLEGAL_TAG;
               reply->mm_get_phys_mem_type_info.err_index = ix;
               return reply;
            }
         }
      }
   }
   else
   {
      reply->mm_get_phys_mem_type_info.status = MM_ENON_EXISTING_PHYS_MEM;
   }

   return reply;
}


/*
 ***********************************************************************
 *              MM_GET_LOG_MEM_INFO
 ***********************************************************************
 */
static union SIGNAL *
get_log_mem_info(union SIGNAL *request)
{
   mm_status_t status;
   union SIGNAL *reply = request;
   struct partition_class *partition_class;

   /* Make sure name is terminated. */
   reply->mm_get_log_mem_info.name[MM_NAME_LENGTH] = '\0';

   PRINT_SIGNAL(("gmm: get_log_mem_info called with log_mem: %s\n "\
                 "tag count: %ld tags[0].tags_id: %#lx client: %#x\n",
                 reply->mm_get_log_mem_info.name,
                 reply->mm_get_log_mem_info.count,
                 reply->mm_get_log_mem_info.tags[0].tag_id,
                 sender(&reply)));

   /* Look up allocator. */
   partition_class =
      mm_find_partition_by_name(reply->mm_get_log_mem_info.name);
   if (partition_class != NULL)
   {
      unsigned tag_index;
      struct mm_tag *tags = &reply->mm_get_log_mem_info.tags[0];
      OSBUFSIZE my_sigsize;
      U32 err_index = (U32)-1;

      status = MM_SUCCESS;

      /* Check that reply->count holds a reasonable value. */
      my_sigsize = sigsize(&request);

      if (reply->mm_get_log_mem_info.count * (sizeof(struct mm_tag)) <=
          (my_sigsize - (sizeof(struct mm_get_log_mem_info)) +
           (sizeof(struct mm_tag))))
      {
         /* Traverse tag list. */
         for (tag_index = 0;
              tag_index < reply->mm_get_log_mem_info.count;
              tag_index++)
         {
            enum MmTagId tag_id = (enum MmTagId) tags[tag_index].tag_id;
            switch (tag_id)
            {
               case MM_TAG_INFO_LOG_MEM_IMPLICIT:
               {
                  /* The condition NULL == partition->get_base might not be
                   * future safe.
                   */
                  tags[tag_index].tag_value =
                     partition_is_implicit(partition_class);
                  break;
               }
               case MM_TAG_INFO_LOG_MEM_BASE_ADDRESS:
               {
                  if (partition_class->get_bank_info == NULL)
                  {
                     tags[tag_index].tag_value = MM_EILLEGAL_TAG;
                     status = MM_EILLEGAL_TAG;
                     if (tag_index < err_index)
                     {
                        err_index = tag_index;
                     }
                  }
                  else
                  {
                     log_addr_t base;
                     int status = partition_class->get_bank_info(
                        partition_class, 0, &base, NULL);

                     if (!status)
                     {
                        tags[tag_index].tag_value = MM_EILLEGAL_TAG;
                        status = MM_EILLEGAL_TAG;
                        if (tag_index < err_index)
                        {
                           err_index = tag_index;
                        }
                     }
                     else
                     {
                        tags[tag_index].tag_value = base;
                     }
                  }
                  break;
               }
               case MM_TAG_INFO_LOG_MEM_SIZE:
               {
                  if (partition_class->get_bank_info == NULL)
                  {
                     tags[tag_index].tag_value = MM_EILLEGAL_TAG;
                     status = MM_EILLEGAL_TAG;
                     if (tag_index < err_index)
                     {
                        err_index = tag_index;
                     }
                  }
                  else
                  {
                     log_size_t size;
                     int index = 0;
                     
                     tags[tag_index].tag_value = 0;

                     while(partition_class->get_bank_info(
                              partition_class, index, NULL, &size))
                     {
                        tags[tag_index].tag_value += size;
                        index++;
                     }
                  }
                  break;
               }
               case MM_TAG_INFO_LOG_MEM_TYPE:
               {
                  mapping_type_t maptype;
                  
                  status = mm_find_partition_mapping_type(
                     partition_class, reply->mm_get_log_mem_info.name, &maptype);
                  if (status != MM_SUCCESS)
                  {
                     tags[tag_index].tag_value = MM_EILLEGAL_TAG;
                     status = MM_EILLEGAL_TAG;
                     break;
                  }

                  tags[tag_index].tag_value = (U32)maptype;

                  break;
               }
               case MM_TAG_INFO_LOG_MEM_MANAGER_MODE:
               {
                  tags[tag_index].tag_value = (U32)
                     partition_is_manager_mode(partition_class);
                  break;
               }
               case MM_TAG_INFO_LOG_MEM_BANK_COUNT:
               {
                  U32 count = 0;
                  int success = 1;

                  while (success && partition_class->get_bank_info != NULL)
                  {
                     success = partition_class->get_bank_info(
                        partition_class, count, NULL, NULL);
                     if (success)
                        count++;
                  }

                  tags[tag_index].tag_value = count;
                  break;
               }
               case MM_TAG_INFO_LOG_MEM_RESIDENT:
               {
                  tags[tag_index].tag_value = (U32)
                     partition_is_resident(partition_class);
                  break;
               }
               case MM_TAG_INFO_LOG_MEM_PERMISSIONS:
               {
                  if (partition_is_resident(partition_class))
                  {
                     tags[tag_index].tag_value = (U32)
                        partition_class->res_perm;
                  }
                  else
                  {
                     tags[tag_index].tag_value = (U32)SU_NA_USR_NA;
                  }
                  break;
               }
               case MM_TAG_INFO_LOG_MEM_CACHE_MODE:
               {
                  if (partition_is_resident(partition_class))
                  {
                     tags[tag_index].tag_value = (U32)
                        partition_class->res_cache;
                  }
                  else
                  {
                     tags[tag_index].tag_value = (U32)0;
                  }
                  break;
               }
               default:
               {
                  /* Calculate index of unknown tag and break loop. */
                  reply->mm_get_log_mem_info.err_index = tag_index;
                  status = MM_EILLEGAL_TAG;
                  break;
               }
            }
         }
      }
      else
      {
         status = MM_ETOO_SMALL_SIGNAL_SIZE;;
         err_index = reply->mm_get_log_mem_info.count;
      }
      reply->mm_get_log_mem_info.err_index = err_index;
   }
   else
   {
      status = MM_ENON_EXISTING_LOG_MEM;
   }

   reply->mm_get_log_mem_info.status = status;
   reply->mm_get_log_mem_info.sig_no = MM_GET_LOG_MEM_INFO_REPLY;
   return reply;
}


/*
 ***********************************************************************
 *              MM_GET_LOG_MEM_BANK_INFO
 ***********************************************************************
 */
static union SIGNAL *
get_log_mem_bank_info(union SIGNAL *request)
{
   mm_status_t status;
   union SIGNAL *reply = request;
   struct partition_class *partition_class;
   unsigned tag_index;
   struct mm_tag *tags = &reply->mm_get_log_mem_bank_info.tags[0];
   OSBUFSIZE my_sigsize;
   U32 err_index = (U32)-1;

   /* Make sure name is terminated. */
   reply->mm_get_log_mem_bank_info.name[MM_NAME_LENGTH] = '\0';

   PRINT_SIGNAL(("gmm: get_log_mem_bank_info called with log_mem: %s\n "\
                 "tag count: %ld tags[0].tags_id: %#lx\n",
                 reply->mm_get_log_mem_bank_info.name,
                 reply->mm_get_log_mem_bank_info.count,
                 reply->mm_get_log_mem_bank_info.tags[0].tag_id));

   /* Locate partition (log_mem). */
   partition_class =
      mm_find_partition_by_name(reply->mm_get_log_mem_bank_info.name);
   if (partition_class == NULL)
   {
      reply->mm_get_log_mem_bank_info.status = MM_ENON_EXISTING_LOG_MEM;
      reply->mm_get_log_mem_bank_info.sig_no = MM_GET_LOG_MEM_BANK_INFO_REPLY;
      return reply;
   }
   
   /* Check that reply->count holds a reasonable value. */
   my_sigsize = sigsize(&request);

   if (reply->mm_get_log_mem_bank_info.count * (sizeof(struct mm_tag)) >
       (my_sigsize - (sizeof(struct mm_get_log_mem_bank_info)) +
        (sizeof(struct mm_tag))))
   {
      reply->mm_get_log_mem_bank_info.err_index =
         reply->mm_get_log_mem_bank_info.count;
      reply->mm_get_log_mem_bank_info.status = MM_ETOO_SMALL_SIGNAL_SIZE;
      reply->mm_get_log_mem_bank_info.sig_no = MM_GET_LOG_MEM_BANK_INFO_REPLY;
      return reply;
   }
   

   status = MM_SUCCESS;

   /* Traverse tag list. */
   for (tag_index = 0;
        tag_index < reply->mm_get_log_mem_bank_info.count;
        tag_index++)
   {
      U32 bank_index = reply->mm_get_log_mem_bank_info.bank;
      enum MmTagId tag_id = (enum MmTagId) tags[tag_index].tag_id;

      switch (tag_id)
      {
         case MM_TAG_INFO_LOG_MEM_BANK_BASE:
         {
            if (partition_class->get_bank_info == NULL)
            {
               tags[tag_index].tag_value = MM_EILLEGAL_TAG;
               status = MM_EILLEGAL_TAG;
               if (tag_index < err_index)
               {
                  err_index = tag_index;
               }
            }
            else
            {
               log_addr_t base;
               int status = partition_class->get_bank_info(
                  partition_class, bank_index, &base, NULL);
               
               if (!status)
               {
                  tags[tag_index].tag_value = MM_EILLEGAL_TAG;
                  status = MM_EILLEGAL_TAG;
                  if (tag_index < err_index)
                  {
                     err_index = tag_index;
                  }
               }
               else
               {
                  tags[tag_index].tag_value = base;
               }
            }
            break;
         }
         case MM_TAG_INFO_LOG_MEM_BANK_SIZE:
         {
            if (partition_class->get_bank_info == NULL)
            {
               tags[tag_index].tag_value = MM_EILLEGAL_TAG;
               status = MM_EILLEGAL_TAG;
               if (tag_index < err_index)
               {
                  err_index = tag_index;
               }
            }
            else
            {
               log_size_t size;
               int status = partition_class->get_bank_info(
                  partition_class, bank_index, NULL, &size);
               
               if (!status)
               {
                  tags[tag_index].tag_value = MM_EILLEGAL_TAG;
                  status = MM_EILLEGAL_TAG;
                  if (tag_index < err_index)
                  {
                     err_index = tag_index;
                  }
               }
               else
               {
                  tags[tag_index].tag_value = size;
               }
            }
            break;
         }
         default:
         {
            /* Calculate index of unknown tag and break loop. */
            reply->mm_get_log_mem_bank_info.err_index = tag_index;
            status = MM_EILLEGAL_TAG;
            break;
         }
      }
   }

   reply->mm_get_log_mem_bank_info.status = status;
   reply->mm_get_log_mem_bank_info.sig_no = MM_GET_LOG_MEM_BANK_INFO_REPLY;
   return reply;
}


/*
 ***********************************************************************
 *      Help functions  MM_GET_MMU_STATISTICS
 ***********************************************************************
 */
/*
 * Check mm_tag_ext
 */
static mm_status_t
check_tag(struct mm_tag_ext *tag)
{
   mm_status_t  status = MM_SUCCESS;
   U32 supported_in_options;
   
   supported_in_options =
      MM_TAG_RESET_COUNTER | MM_TAG_COUNTER_ENABLED | MM_TAG_COUNTER_DISABLED
      | MM_TAG_PER_CORE_COUNTER;
   
   /*
    * Check options.
    */
   if (tag->tag_options & ~supported_in_options)
   {
      status = MM_EOPTION_NOT_SUPPORTED;
   }

   /* Cannot handle both disable and enable. */
   if ((tag->tag_options & MM_TAG_COUNTER_ENABLED) &&
       (tag->tag_options & MM_TAG_COUNTER_DISABLED))
   {
      status = MM_EOPTION_NOT_SUPPORTED;
   }
   
   /*
    * Check cpu id.
    */
   if (tag->cpu_id != 0)
   {
#if defined(OSE_SMP)
      if (tag->cpu_id > (ose_num_cpus() - 1)) {
         status = MM_EILLEGAL_CPU_ID;
      }
#else
      tag->cpu_id = 0;  /* Unicore, always cpu_id = 0 */
#endif      
   }
   return status;
}
 

/*
 * Arch generic handling of enable/disable counter.
 */
static U32
update_counter_enable_state(const U32 old_counter_state)
{

   U32 new_counter_state = old_counter_state;
   
   /* Enable/disable for HW counters should be controlled
    * by hal_mmu perf counter state. */
   if (old_counter_state & MM_TAG_HW_COUNTER && 
       (old_counter_state & MM_TAG_COUNTER_DISABLED ||
        old_counter_state & MM_TAG_COUNTER_ENABLED))
   {
      INTERNAL_ERROR();
   }
   
   
   if (old_counter_state & MM_TAG_SW_COUNTER)
   {
      /* For now sw counters always enabled. */
      new_counter_state |= MM_TAG_COUNTER_ENABLED;
   }
   return new_counter_state;
}

/*
 * Arch generic inheritance of all tag_options defined by the counter
 * into client request.
 *
 */
static U32
inherit_tag_options(U32 old_client_tag_options, const U32 counter_state)
{
   U32 new_client_tag_options = old_client_tag_options;
   
   assert(((counter_state & MM_TAG_SW_COUNTER) &&
           (counter_state & MM_TAG_HW_COUNTER)) == 0);
   
   new_client_tag_options |= counter_state;


   if (new_client_tag_options & MM_TAG_SW_COUNTER &&
       new_client_tag_options & MM_TAG_COUNTER_DISABLED)
   {
      /* For now sw counters cannot be disabled, ignore this disable option. */
      new_client_tag_options &= ~MM_TAG_COUNTER_DISABLED;
   }
   
   
   return new_client_tag_options;
}


/*
 * Search for a given tag_id in mmuhal.
 * Handle characteristic bits and tag_options.
 */
static hal_status_t  
ose_mm_tagId_search(struct mm_tag_ext *tag, const struct hal_info *hal_info)
{
   unsigned i;
   hal_status_t  status = HAL_EUNSUPPORTED_TAG;
   U32 counter_state, new_counter_state, new_tag_options;
   
   
   for (i = 0; i < hal_info->no_tag_id; i++)
   {
      if ((hal_info->tag_id_array[i] & ~MM_INFOTAG_MASK)
          == tag->tag_id)
      {         
         counter_state = (hal_info->tag_id_array[i] & MM_INFOTAG_MASK)
            >> MM_SHIFT_TAG_OPT;
         
         /* Distinguish between per core and global counters with
          * same tag_id, for example see ppce500 */
         if (tag->tag_options & MM_TAG_PER_CORE_COUNTER &&
             !(counter_state & MM_TAG_PER_CORE_COUNTER))
         {
            status = HAL_EINFOTAG_ONLY_GLOBAL_COUNTER;
            continue; /*client request local core counter but we found global*/
         }
         if (!(tag->tag_options & MM_TAG_PER_CORE_COUNTER) &&
             (counter_state & MM_TAG_PER_CORE_COUNTER))
         {
            status = HAL_EINFOTAG_ONLY_LOCAL_COUNTER;
            continue; /*client request a global counter but we found local. */
         }

         
         new_counter_state = update_counter_enable_state(counter_state);
         
         /* Inherit all tag_options defined by the updated counter state. */
         new_tag_options = inherit_tag_options(tag->tag_options,
                                               new_counter_state);

         /* Update tag_options */
         tag->tag_options = new_tag_options;

         if (new_counter_state != counter_state) {
            /* counter state changed,
             * update without destroying the counter tag_id */
            hal_info->tag_id_array[i] =
               ((new_counter_state << MM_SHIFT_TAG_OPT) & MM_INFOTAG_MASK) |
               (hal_info->tag_id_array[i] & ~MM_INFOTAG_MASK);
         }
         status = HAL_SUCCESS;
         
         break;
      }
   }
   return status;
}
#if defined(OSE_SMP) && defined(OSPP)
hal_status_t  
get_mmu_stat_other_core(struct mm_tag_ext *tag)
{
   hal_status_t rc;
   static const SIGSELECT sel[] = {1, HAL_GET_CORE_MMU_STAT};
   union SIGNAL *sig;

   if (!(tag->tag_options & MM_TAG_PER_CORE_COUNTER)) {
      INTERNAL_ERROR();  /* Catch recursion. */
   }
   
   create_and_start_cpu_agents(); 
   
   sig = alloc(sizeof(struct hal_get_core_mmu_stat), HAL_GET_CORE_MMU_STAT);

   sig->hal_get_core_mmu_stat.tags[0] = *tag;  /* Struct assign. */

   send(&sig, smp_agents.pid[tag->cpu_id - 1]);
   sig = receive(sel);

   *tag = sig->hal_get_core_mmu_stat.tags[0];
   rc = sig->hal_get_core_mmu_stat.status;
   
   free_buf(&sig);

   if (!(tag->tag_options & MM_TAG_SOME_HW_COUNTER_ENABLED)){
      stop_and_kill_cpu_agents(); 
   }
   
   return rc;
}
#endif
/*
 ***********************************************************************
 *              MM_GET_MMU_STATISTICS
 ***********************************************************************
 */
static union SIGNAL *
get_mmu_statistics(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   struct  mm_get_mmu_statistics *reply;
   hal_status_t rc;
   OSBUFSIZE my_sigsize;
   U32 ix;
   U32 count_max;
   mm_bool_t infotag_mmu_all = FALSE;

   PRINT_SIGNAL(("gmm: First tag->tag_id[0] %#lx, cpu_id:%ld options:%#lx\n",
                 request->mm_get_mmu_statistics.tags[0].tag_id,
                 request->mm_get_mmu_statistics.tags[0].cpu_id,
                 request->mm_get_mmu_statistics.tags[0].tag_options));   

   reply = &request->mm_get_mmu_statistics;
   reply->sig_no = MM_GET_MMU_STATISTICS_REPLY;
   
   if (gmm_vars->hal_info->tag_id_array == NULL ||
       hal_get_mmu_statistics == NULL)
   {
      /* MMUHAL has no tag support. */
      reply->status = MM_ENO_INFOTAG_COUNTER_SUPPORT_FOR_CPU;
      goto exit_mmu_stat;
   }
   
   if (request->mm_get_mmu_statistics.tags[0].tag_id == MM_INFOTAG_MMU_ALL)
   {      
      /* Client desired *all* avail MmTag's. Alloc and build new reply
       * signal with all MmTag's that the MMUHAL supports.
       * Scale-able: 
       * SMP *all* means all global but only the local counters for
       * given cpu_id. This avoids the signal from becoming larger the
       * greater number of cpu's
       */
      infotag_mmu_all =  TRUE;
      count_max = gmm_vars->hal_info->no_tag_id;
      reply = (struct mm_get_mmu_statistics *)
          alloc(sizeof(struct mm_get_mmu_statistics)
                + (count_max - 1) * sizeof(struct mm_tag_ext),
                MM_GET_MMU_STATISTICS_REPLY);

      for (ix = 0; ix < count_max; ix++)
      {
         /* Mask away internal characteristic bits in tag_id from client. */
         reply->tags[ix].tag_id =
            gmm_vars->hal_info->tag_id_array[ix] & ~MM_INFOTAG_MASK;

         /* Request tag_option[0] from client should be set in all tags. */
         reply->tags[ix].tag_options =
         request->mm_get_mmu_statistics.tags[0].tag_options;

         if ((gmm_vars->hal_info->tag_id_array[ix] >> MM_SHIFT_TAG_OPT) &
             MM_TAG_PER_CORE_COUNTER) {
            /* Help to distinguish per core and global counters with
             * same tag_id for example see ppce500 */
            reply->tags[ix].tag_options |= MM_TAG_PER_CORE_COUNTER;
         }
         reply->tags[ix].cpu_id =
         request->mm_get_mmu_statistics.tags[0].cpu_id;
         
      }
      free_buf(&request);      
      reply->count = count_max;
   }
   else        /* Client specified one ore severel desired MmTag's.*/
   {
      /* Check that reply->count holds a reasonable value. */
      if (reply->count == 0)
      {
         /* A.I. code:) The signal struct always includes one tags field. */
         reply->count = 1; 
      }

      /* Here we check:
       * a) That count holds a reasonable value.
       * b) That old signal format of mm_get_mmu_statistics with
       *    "struct mm_tag" is not used.
       */
      {
         /* Use &request ptr (of type union **SIGNAL) instead of reply
          * to not break strict-aliasing.
          */
         my_sigsize = sigsize(&request);
         
         if (reply->count * (sizeof(struct mm_tag_ext)) >
             (my_sigsize - (sizeof(struct mm_get_mmu_statistics)) +
              (sizeof(struct mm_tag_ext))))
         {
            reply->status = MM_ETOO_SMALL_SIGNAL_SIZE;
            reply->err_index =
               reply->count;
            
            goto exit_mmu_stat;
         }         
      }
   }      

   reply->status = MM_SUCCESS;
   reply->err_index = 0;
   
   for (ix = 0; ix < reply->count; ix++)
   {
      mm_status_t status;
      
      PRINT_SIGNAL(("gmm: tag->tag_id[%ld] = %#lx\n",
                    ix, reply->tags[ix].tag_id));

      status = check_tag(&(reply->tags[ix]));

      if (status != MM_SUCCESS)
      {
         /* Calculate index of unknown tag option and break loop. */
         reply->tags[ix].tag_value = OSE_ILLEGAL_ATTREF;
         reply->status = status;
         reply->err_index = ix;
         goto exit_mmu_stat;
      }

      switch (reply->tags[ix].tag_id)
      {

         /* Note here we list all tags that are valid for all architectures.
          * This makes it possible to differentiate if input tag is illegal
          * or just not implemented for this architecture.
          * 
          * */
         case MM_INFOTAG_TLB_MISS_COUNT:
         {
            strlcpy(reply->tags[ix].tag_desc, MM_INFOTAG_TLB_MISS_COUNT_DESC,
                    sizeof reply->tags[ix].tag_desc);
            break;
         }
         case MM_INFOTAG_DTLB_MISS_COUNT:
         {
            strlcpy(reply->tags[ix].tag_desc, MM_INFOTAG_DTLB_MISS_COUNT_DESC,
                    sizeof reply->tags[ix].tag_desc);
            break;
         }
         case MM_INFOTAG_ITLB_MISS_COUNT:
         {
            strlcpy(reply->tags[ix].tag_desc, MM_INFOTAG_ITLB_MISS_COUNT_DESC,
                    sizeof reply->tags[ix].tag_desc);
            break;
         }            
         case MM_INFOTAG_DHTAB_MISS_COUNT:
         {
            strlcpy(reply->tags[ix].tag_desc, MM_INFOTAG_DHTAB_MISS_COUNT_DESC,
                    sizeof reply->tags[ix].tag_desc);
            break;
         }  
         case MM_INFOTAG_IHTAB_MISS_COUNT:
         {
            strlcpy(reply->tags[ix].tag_desc, MM_INFOTAG_IHTAB_MISS_COUNT_DESC,
                    sizeof reply->tags[ix].tag_desc);
            break;
         }  
         case MM_INFOTAG_L1_ICACHE_MISS_COUNT:
         {
            strlcpy(reply->tags[ix].tag_desc,
                    MM_INFOTAG_L1_ICACHE_MISS_COUNT_DESC,
                    sizeof reply->tags[ix].tag_desc);
            break;
         }  
         case MM_INFOTAG_L1_DCACHE_MISS_COUNT:
         {
            strlcpy(reply->tags[ix].tag_desc,
                    MM_INFOTAG_L1_DCACHE_MISS_COUNT_DESC,
                    sizeof reply->tags[ix].tag_desc);
            break;
         }  
         case MM_INFOTAG_L2_INSTRUCTION_MISS_COUNT:
         {
            strlcpy(reply->tags[ix].tag_desc,
                    MM_INFOTAG_L2_INSTRUCTION_MISS_COUNT_DESC,
                    sizeof reply->tags[ix].tag_desc);
            break;
         }  
         case MM_INFOTAG_L2_DATA_MISS_COUNT:
         {
            strlcpy(reply->tags[ix].tag_desc,
                    MM_INFOTAG_L2_DATA_MISS_COUNT_DESC,
                    sizeof reply->tags[ix].tag_desc);
            break;
         }  
         case MM_INFOTAG_DTLB_HW_TABLE_WALK_CYCLES:
         {
            strlcpy(reply->tags[ix].tag_desc,
                    MM_INFOTAG_DTLB_HW_TABLE_WALK_CYCLES_DESC,
                    sizeof reply->tags[ix].tag_desc);
            break;
         }  
         case MM_INFOTAG_ITLB_HW_TABLE_WALK_CYCLES:
         {
            strlcpy(reply->tags[ix].tag_desc,
                    MM_INFOTAG_ITLB_HW_TABLE_WALK_CYCLES_DESC,
                    sizeof reply->tags[ix].tag_desc);
            break;
         }  
         case MM_INFOTAG_INSTRUCTIONS_COMPLETED:
         {
            strlcpy(reply->tags[ix].tag_desc,
                    MM_INFOTAG_INSTRUCTIONS_COMPLETED_DESC,
                    sizeof reply->tags[ix].tag_desc);
            break;
         }  
         case MM_INFOTAG_L1_DCACHE_HIT_COUNT:
         {
            strlcpy(reply->tags[ix].tag_desc, MM_INFOTAG_L1_DCACHE_HIT_COUNT_DESC,
                    sizeof reply->tags[ix].tag_desc);                 
            break;
         }
         case MM_INFOTAG_DATA_LOAD_AND_STORES:
         {
            strlcpy(reply->tags[ix].tag_desc,
                    MM_INFOTAG_DATA_LOAD_AND_STORES_DESC,
                    sizeof reply->tags[ix].tag_desc);                 
            break;
         }
         case MM_INFOTAG_L2_CASTOUT:
         {
            strlcpy(reply->tags[ix].tag_desc, MM_INFOTAG_L2_CASTOUT_DESC,
                    sizeof reply->tags[ix].tag_desc);                 
            break;
         }
         case MM_INFOTAG_L1_DC_LOAD_MISS_CYCLES:
         {
            strlcpy(reply->tags[ix].tag_desc,
                    MM_INFOTAG_L1_DC_LOAD_MISS_CYCLES_DESC,
                    sizeof reply->tags[ix].tag_desc);                 
            break;
         }
         case MM_INFOTAG_HTAB_OVERFLOW_COUNT:
         {
            strlcpy(reply->tags[ix].tag_desc,
                    MM_INFOTAG_HTAB_OVERFLOW_COUNT_DESC,
                    sizeof reply->tags[ix].tag_desc);                 
            break;
         }
         default:
         {
            /* Calculate index of unknown tag and break loop. */
            reply->tags[ix].tag_value = OSE_ILLEGAL_ATTREF;
            reply->err_index = ix;
            reply->status = MM_EILLEGAL_TAG;
            goto exit_mmu_stat;
         }
      }

      /* We know that Tag Id is valid according to mm.sig,
       * now check if Tag Id is supported by ppc family
       * sub architecture. 
       */
      rc = ose_mm_tagId_search(&(reply->tags[ix]), gmm_vars->hal_info);
      if (rc == HAL_SUCCESS)
      {
#if defined(OSE_SMP) && defined(OSPP)
         struct mm_tag_ext *tag = &reply->tags[ix];
         if ((tag->tag_options & MM_TAG_HW_COUNTER) &&
             (tag->tag_options & MM_TAG_PER_CORE_COUNTER) &&
             (tag->cpu_id > 0))
         {
            /* Detected a local hw counter on another core. */
            rc = get_mmu_stat_other_core(tag);
         }
         else
#endif            
         {
            /* Request can be handled in GMM context. */
            rc = hal_get_mmu_statistics(&(reply->tags[ix]));
         }
         assert(reply->tags[ix].tag_options & MM_TAG_COUNTER_DISABLED ||
                reply->tags[ix].tag_options & MM_TAG_COUNTER_ENABLED);
         
         assert((reply->tags[ix].tag_options & MM_TAG_COUNTER_DISABLED &&
                 reply->tags[ix].tag_options & MM_TAG_COUNTER_ENABLED) == 0);
      }
      
      if (rc != HAL_SUCCESS)   
      {
         if (infotag_mmu_all)
         {
            /* Really wrong! The client asked MM to give all tags supported
             * but then counter driver cannot read one or several of them. */
            assert(0);
            /* Calculate index of unknown tag and break loop. */
            reply->tags[ix].tag_value = OSE_ILLEGAL_ATTREF;
            reply->err_index = ix;
            reply->status = MM_EILLEGAL_TAG;
            goto exit_mmu_stat;
                  
         }
         else if (rc == HAL_EUNSUPPORTED_MMU)
         {
            /* FAMILY(get_mmu_statistics) == NULL.
             * i.e. no counters support at all  for this MMUHAL. */
            /* Calculate index of unsupported tag and break loop. */
            reply->tags[ix].tag_value = OSE_ILLEGAL_ATTREF;
            reply->err_index = ix;
            reply->status = MM_ENO_INFOTAG_COUNTER_SUPPORT_FOR_CPU;
            goto exit_mmu_stat;
         }
         else if (rc == HAL_EUNSUPPORTED_TAG)
         {
            /* FAMILY(get_mmu_statistics) exists
             * but this TAG is not  supported for this MMUHAL. */
            /* Calculate index of unsupported tag and break loop. */
            reply->tags[ix].tag_value = OSE_ILLEGAL_ATTREF;
            reply->err_index = ix;
            reply->status = MM_EINFOTAG_UNSUPPORTED_FOR_CPU;
            goto exit_mmu_stat;
         }
         else if (rc == HAL_EINFOTAG_ONLY_LOCAL_COUNTER)
         {
            reply->tags[ix].tag_value = OSE_ILLEGAL_ATTREF;
            reply->err_index = ix;
            reply->status = MM_EINFOTAG_ONLY_LOCAL_COUNTER;              
         }
         else if (rc == HAL_EINFOTAG_ONLY_GLOBAL_COUNTER)
         {
            reply->tags[ix].tag_value = OSE_ILLEGAL_ATTREF;
            reply->err_index = ix;
            reply->status = MM_EINFOTAG_ONLY_GLOBAL_COUNTER;   
         }
         else
         {
            assert(0);
            reply->status = MM_EINTERNAL_ERROR;
            goto exit_mmu_stat;
         }
      }
      
   }

  exit_mmu_stat:

   PRINT_SIGNAL(("gmm: Return tag->tag_id[0] %#lx, cpu_id:%#lx options:%ld "
                 "status %#lx\n",
                 reply->tags[0].tag_id,
                 reply->tags[0].cpu_id,
                 reply->tags[0].tag_options,
                 reply->status));  
   return (union SIGNAL *)reply;
}



/*
 ***********************************************************************
 *              MM_LIST_LOG_MEM
 ***********************************************************************
 */

static union SIGNAL *
list_log_mem(union SIGNAL *request)
{
   union SIGNAL *reply;
   struct partition_class *p;
   struct partition_alias *alias;
   char * dest;
   unsigned count;
   size_t total_size;

   free_buf(&request);

   count = 0;
   total_size = 0;

   /* First calculate total string length. */
   for (p = gmm_state.partition_classes; p != NULL; p = p->next)
   {
      for (alias = p->aliases; alias != NULL; alias = alias->next)
      {
         count++;
         total_size += strlen(alias->name) + 1;
      }
   }

   /* Allocate a buffer big enough for all strings. */
   reply = alloc(sizeof(struct mm_list_log_mem_reply) + total_size - 1,
            MM_LIST_LOG_MEM_REPLY);

   reply->mm_list_log_mem_reply.count = count;
   dest = reply->mm_list_log_mem_reply.names;

   /* Second traversel: Copy partition names. */
   for (p = gmm_state.partition_classes; p != NULL; p = p->next)
   {
      for (alias = p->aliases; alias != NULL; alias = alias->next)
      {
         /* Copy string. */
         strcpy(dest, alias->name);
         
         /* Update state. */
         dest += strlen(alias->name) + 1;
         count--;
      }
   }

   assert(0 == count);

   return reply;
}



/*
 ***********************************************************************
 *              MM_LIST_DOMAINS
 ***********************************************************************
 */
static union SIGNAL *
list_domains(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   union SIGNAL *reply;
   struct domains *domains = &gmm_vars->domains;
   U32 source_pos = 0;
   U32 reply_pos = 0;
   const U32 max_domains = gmm_state.max_domains;

   free_buf(&request);
   reply = alloc(sizeof(struct mm_list_domains_reply) +
		 (max_domains - domains->no_free) * sizeof(OSSEGMENT),
		 MM_LIST_DOMAINS_REPLY);

   while (source_pos < max_domains)
   {
      struct domain *domain = domains->domain_list[source_pos];

      if (domain != NULL)
      {
	 reply->mm_list_domains_reply.domains[reply_pos] = domain->segnum;
	 ++reply_pos;
      }
      ++source_pos;
   }

   reply->mm_list_domains_reply.count = reply_pos;
   reply->sig_no = MM_LIST_DOMAINS_REPLY;
   return reply;
}


/*
 ***********************************************************************
 *              MM_LIST_REGIONS
 ***********************************************************************
 */
static union SIGNAL *
list_regions(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   union SIGNAL *reply;
   struct domain *domain;
   core_rbtree_t      *regions = NULL;
   core_rbtree_node_t *current_region;
   unsigned no_regions = 0;

   domain = mm_find_domain_by_domid(
      &gmm_vars->domains, request->mm_list_regions_request.domain);
   if (domain == NULL)
   {
      request->mm_list_regions_reply.sig_no = MM_LIST_REGIONS_REPLY;
      request->mm_list_regions_reply.status = MM_ENON_EXISTING_DOMAIN;
      return request;
   }

   regions = &domain->regions;

   /* Need to count the number of regions to know how large reply
    * signal to allocate. */
   for (current_region = core_rbtree_first_no_lock(regions);
        current_region != NULL;
        current_region = core_rbtree_next_no_lock(regions, current_region))
   {
      no_regions++;
   }

   /* Allocate reply signal. */
   reply = alloc(sizeof(struct mm_list_regions_reply) +
                 (no_regions - 1) * sizeof reply->mm_list_regions_reply.regions,
                 MM_LIST_REGIONS_REPLY);

   /* Copy fixed information. */
   reply->mm_list_regions_reply.count = no_regions;

   /* Traverse list with instance regions and copy base addresses. */
   {
      unsigned i = 0;
      struct region *region = (struct region*)
         core_rbtree_first_no_lock(regions);

      while (region != NULL)
      {
         reply->mm_list_regions_reply.regions[i++]
            = region->log_object->base;
         region = (struct region*)
            core_rbtree_next_no_lock(regions, &region->node);
      }
   }

   free_buf(&request);

   reply->mm_list_regions_reply.status = MM_SUCCESS;
   return reply;
}


/*
 ***********************************************************************
 *              MM_LIST_INACTIVE_REGIONS
 ***********************************************************************
 * This interface is deprecated and this function only exists to
 * respond to applications that still use the request.
 *              
 */
static union SIGNAL *
list_inactive_regions(union SIGNAL *request)
{
   union SIGNAL *reply;

   /* Signal is deprecated and should not be used,
    * but send reply to keep old clients happy. */
   reply = alloc(sizeof(struct mm_list_regions_reply),
                 MM_LIST_INACTIVE_REGIONS_REPLY);

   /* No inactive regions at all */
   reply->mm_list_regions_reply.count = 0;

   free_buf(&request);
      
   reply->mm_list_regions_reply.status = MM_SUCCESS;
   return reply;
}


/*
 ***********************************************************************
 *              MM_LIST_PHYS_MEM
 ***********************************************************************
 */
static union SIGNAL *
list_phys_mem(union SIGNAL *request)
{
   union SIGNAL *reply;
   struct allocator *a;
   char *dest;
   unsigned count;
   size_t total_size;

   free_buf(&request);
   count = 0;
   total_size = 0;

   /* First traversal: calculate total string length. */
   for (a = gmm_state.allocators; a != NULL; a = a->next)
   {
      count++;
      total_size += strlen(a->name) + 1;
   }

   /* Allocate a buffer big enough for all strings. */
   reply = alloc(sizeof(struct mm_list_phys_mem_reply) + total_size - 1,
            MM_LIST_PHYS_MEM_REPLY);
   reply->mm_list_phys_mem_reply.count = count;

   dest = reply->mm_list_phys_mem_reply.names;
   for (a = gmm_state.allocators; a != NULL; a = a->next)
   {
      strcpy(dest, a->name);
      dest += strlen(a->name) + 1;
      count--;
   }

   assert(0 == count);

   return reply;
}

/*
 ***********************************************************************
 *              MM_LIST_PAGE_SIZES
 ***********************************************************************
 */
static union SIGNAL *
list_page_sizes(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   union SIGNAL *reply;
   log_size_t    page_sizes_length;

   free_buf(&request);
   page_sizes_length = gmm_vars->hal_info->no_page_sizes * sizeof(int);
   reply = alloc(sizeof(struct mm_list_page_sizes_reply) + page_sizes_length,
		 MM_LIST_PAGE_SIZES_REPLY);
   /* Copy the number of supported page sizes to reply. */
   reply->mm_list_page_sizes_reply.count = gmm_vars->hal_info->no_page_sizes;

   /* Copy all supported page sizes to reply. */
   memcpy(&reply->mm_list_page_sizes_reply.sizes[0],
	  gmm_vars->hal_info->page_sizes, page_sizes_length);
   return reply;
}


/*
 ***********************************************************************
 *              MM_CHANGE_PERMISSIONS
 ***********************************************************************
 */
static union SIGNAL *
change_permissions(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   mm_status_t status;
   struct domain *domain;
   struct region *region;
   union SIGNAL *reply = request;
   U32 requested_perm = request->mm_change_permissions.permissions;

   /* Remove compiler warnings. */
   domain = NULL;
   region = NULL;

  

   /* Validate region identifiers. */
   status = validate_region_identifiers(gmm_vars,
                                        request->mm_change_permissions.domain,
                                        request->mm_change_permissions.base,
                                        TRUE,        /* exact_match */
                                        &domain,
                                        &region);
   if (status != MM_SUCCESS)
   {
      goto exit;
   }

   /* Only start profiling for regions above threshold. */
   if (region->log_object->size > PROFILE_SIZE)
   {
      MM_PROFILE_START("change_permission",
                       region->name,
                       region->log_object->size,
                       XSTR(PROFIL_SIZE));
   }
   
   /* An instance region was found. */

   /* Can it be changed? */
   if (region_is_static(region))
   {
      /* System regions can't be changed -> error. */
      status = MM_ESYSTEM_REGION;
      goto exit;
   }

   /* A region must have physical memory in order to be changed. */
   if (region->phys_object == NULL)
   {
      status = MM_EREGION_LOGICAL;
      goto exit;
   }

   if (!mm_check_permission(requested_perm))
   {
      status = MM_EILLEGAL_PERMISSION;
      goto exit;
   }

   assert(region->log_object != NULL);

   /* The region must only exist in one domain in order to be changed.
    *
    * FIXME(rand) - This is probably not a valid limitation any more, remove */
   if (region->log_object->ref_count != 1)
   {
      status = MM_ETOO_MANY_INSTANCES;
      goto exit;
   }


   
   {
      int options;
      U32 actual_new_permissions;
      hal_status_t hal_status;

      /* If the new permissions is equal to the current just return. */
      if(requested_perm == region->permissions)
      {
         status = MM_SUCCESS;
         goto exit;  
      }

      /* Get the actual permissions supported by HAL. */
      hal_status = hal_actual_permissions(
         &actual_new_permissions,
         requested_perm,
         region->log_object->base,
         region_is_resident(region) ? HAL_RESIDENT : 0);

      if (hal_status != HAL_SUCCESS)
      {
         status = MM_EILLEGAL_PERMISSION;
         goto exit;
      }
            
      /*
       * Return old permissions in signal.
       */
      reply->mm_change_permissions.permissions = region->permissions;

      options = mm_set_hal_options(domain, region, TRUE);

      if(!region_is_resident(region))
      {
	 /* Call HAL to change the MMU mappings.
	  */
	 hal_status = mm_map_area(domain->hal_domain,
				  region->phys_object,
				  region->log_object->base,
				  options,
				  actual_new_permissions,
				  region->cache_mode);
	 
	 status = mm_hal_status_to_mm_status(hal_status);
	 
	 /* On success update the permissions with the new values 
	  * otherwise let them be SU_NA_USR_NA */
	 if(hal_status == HAL_SUCCESS)
	 {
	    region->permissions = actual_new_permissions;
	 }
	 else
	 { 
	    /* The mm_map_area() call can fail if MM runs out of meta data
	     * during the operation. This can happen if the change is from
	     * SU_NA_USR_NA to something else since the NA_NA permissions
	     * doesn't have any page tables.
	     * The assert() is in case if mm_map_area() fails when the
	     * old permissions is something else than SU_NA_USR_NA. */
	    assert(region->permissions == SU_NA_USR_NA);
	    region->permissions = SU_NA_USR_NA;
	    goto exit;
	 }
      }
      else
      {
         if (SU_NA_USR_NA == requested_perm)
         {
            /*
             * Do not round NA_NA permissions if the region is located
             * in a resident log_mem. The region's permissions would be
             * rounded to match those of the log_mem which would be
             * problematic if the region is to be used for duplicate
             * regions. Duplicate regions are only allowed with a NA_NA
             * region as source.
             */
            region->permissions = SU_NA_USR_NA;
         }
         else
         {
            region->permissions = actual_new_permissions;
         }
      }      
#ifdef OSPP
      /* Remove resident mapping on all cores. */
      make_unresident_range(gmm_vars,
                            region->log_object->base,
                            region->log_object->size);
#endif
            
      if (region_is_persistent(region))
      {
         if (mm_pmm_change_attr(region) != MM_SUCCESS)
         {
            INTERNAL_ERROR();
         }
      }
   }

  exit:
   reply->mm_change_permissions.status = status;
   reply->sig_no = MM_CHANGE_PERMISSIONS_REPLY;

   MM_PROFILE_STOP(reply->mm_change_permissions.status,
                   reply->mm_change_permissions.base);
   
   
   return reply;
}


/*
 ***********************************************************************
 *              MM_INTERFACE_INFO
 ***********************************************************************
 */
static union SIGNAL *
interface_info(union SIGNAL *request)
{
   U32 signal_count;
   union SIGNAL *sig;

   signal_count =
      (MM_MAKE_UNRESIDENT_REPLY - MM_CREATE_REGION_REQUEST) / 2 + 1;
   sig = alloc(sizeof(struct mm_interface_info_reply)
               + signal_count * sizeof(SIGSELECT),
	    MM_INTERFACE_INFO_REPLY);
   sig->mm_interface_info_reply.status = MM_SUCCESS;
   strncpy(sig->mm_interface_info_reply.what_str,
	   "OSE Memory Manager/VERSION",
	   sizeof(sig->mm_interface_info_reply.what_str));
   /* For some reason, mmu_mode and has_mmu are of different types. The !!
    * operation are short for (x ? 1 : 0). */
   sig->mm_interface_info_reply.mmu_mode = !!gmm_state.hal_info->has_mmu;
   sig->mm_interface_info_reply.sigs[0] = signal_count;

   /* Copy all signal values. */
   {
      unsigned i;
      for (i = 0; i < signal_count; i++)
      {
	 sig->mm_interface_info_reply.sigs[i+1]
            = MM_CREATE_REGION_REQUEST+2*i; /*lint !e661 !e662 */
      }
   }
   free_buf(&request);
   return sig;
}

void
mm_hal_info_cmd(void)
{
   if (gmm_state.hal_info->has_mmu)
   {
      unsigned i;

      printf("HAL module attributes: ");
      printf("\n supported page sizes: [");
      for (i = 0; i < gmm_state.hal_info->no_page_sizes; i++)
      {
         printf("%#lx", (U32)gmm_state.hal_info->page_sizes[i]);
         if (i < gmm_state.hal_info->no_page_sizes - 1)
         {
            printf(",");
         }
      }
      printf("]\n MAS support: %s\n\n",
             gmm_state.hal_info->has_mas ? "yes" : "no");

      printf("\n MAS region global counter: %lu\n\n", gmm_state.mas_cnt);
   }
   else
   {
      printf("MMU not used by MM\n\n");
   }
}

/*****************************************************************************
 *
 * Retrive meta-data for a region and create a region descriptor.
 */
static struct region *
get_region_metadata(log_addr_t base)
{
   PROCESS pmm_ = gmm_state.pmm;
   struct region *region = NULL;
   static const SIGSELECT sig_select[] = {1, PMM_RESTORE_REGION_REPLY};
   union SIGNAL *sig = alloc(sizeof(struct pmm_restore_region_request),
                             PMM_RESTORE_REGION_REQUEST);

   sig->pmm_restore_region_request.base = base;
   send(&sig, pmm_);
   sig = receive_from_pid(pmm_, sig_select);

   /* Decode meta-data into a logical region descriptor. */
   region = mm_decode_metadata(sig->pmm_restore_region_reply.metadata, base);

   if (!region) {
      dump_allocator_states((int (*)(const char *,  ...))ramlog_printf);
      MM_FATAL_ERROR2(("Corrupt meta data \"%s\" for persistent region"\
                       "with base = 0x%08lx\n",
                       sig->pmm_restore_region_reply.metadata, base),
                      OSE_MM_PMM_RESTORE_FAILED,
                      (OSERRCODE) 0);
   }

   if (PMM_BOUND != sig->pmm_restore_region_reply.region_type) {
      dump_allocator_states((int (*)(const char *, ...))ramlog_printf);
      MM_FATAL_ERROR2(("Unbound persistent RAM regions are illegal. "
                      "Base 0x%08lx\n", base),
                     OSE_MM_PMM_RESTORE_FAILED,
                     (OSERRCODE) 0);
   }

   free_buf(&sig);

   return region;
}

/*****************************************************************************
 *
 * Retrive meta-data for a binding of a region.
 * Create and return a struct phys_mem object.
 */

static struct pmm_restore_binding_reply *
get_binding_metadata(log_addr_t base)
{
   PROCESS pmm_ = gmm_state.pmm;
   static const SIGSELECT sig_select[] = {1, PMM_RESTORE_BINDING_REPLY};
   union SIGNAL *sig = alloc(sizeof(struct pmm_restore_binding_request),
                             PMM_RESTORE_BINDING_REQUEST);

   /* Send request and wait for reply. */
   sig->pmm_restore_binding_request.base = base;

   send(&sig, pmm_);
   sig = receive_from_pid(pmm_, sig_select);

   return &sig->pmm_restore_binding_reply;
}

/*****************************************************************************/

static void
reserve_phys_fragments(struct regionData *data,
                       struct pmm_restore_binding_reply *restore)
{
   struct fragment_container *frag_list = NULL;
   struct fragment_container *cur_entry = NULL;
   struct fragment_container *next = NULL;
   struct allocator *prev_allocator = NULL;
   U32 i;

   for (i = 0; i < restore->no_fragments; i++)
   {
      phys_addr_t fragBase = restore->fragments[i].base;
      phys_size_t fragSize = restore->fragments[i].size;
      struct allocator *allocator
         = mm_find_allocator_by_range(fragBase, fragSize);
      
      assert(allocator != NULL);

      /* All fragment should belong to the same allocator. */
      assert(prev_allocator == NULL || prev_allocator == allocator);
      prev_allocator = allocator;
      
      if (MM_SUCCESS != allocator->alloc_f(allocator,
                                           &next,
                                           fragBase,
                                           fragSize))
      {
         
         dump_allocator_states((int (*)(const char *, ...))ramlog_printf);
         MM_FATAL_ERROR2(("Failure to reserve persistent fragment with base "
                          "address : 0x%x size: 0x%x in %s.\n",
                          fragBase, fragSize, allocator->name),
                         OSE_MM_PMM_RESTORE_FAILED,
                         (OSERRCODE) 0);
      }
      if (cur_entry)
      {
         FC_SET_NEXT(cur_entry, next);
         cur_entry = next;
      }
      else
      {
         frag_list = next;
         cur_entry = next;
      }
   }
   data->tmp_allocator = prev_allocator;
   data->tmp_frag_list = frag_list;
}

/*****************************************************************************/

static void
restore_regions_pass1(struct restartData *restartData)
{
   struct region *region;
   
   U32 i;
   log_addr_t base;
   
   for (i = 0; i < restartData->entryCnt; i++) {
      struct regionData *data;
      
      data = &restartData->regions[i];
      base = restartData->restartSig->regions[i];
      
      /* get_region_metadata(.) allocates a log-region element, which requires
       * an element from the partition zone. At this point, there should be
       * enough elements left to avoid extending the zone */
      region = get_region_metadata(base);
      
      data->region = region;
      
      /* PMM_RESTORE_BINDING */
      data->bindingSig = get_binding_metadata(base);
      
      {
         if (region_is_exclusive(region))
         {
            reserve_phys_fragments(data, data->bindingSig);
            free_buf((union SIGNAL**)&data->bindingSig);
         }
         else if (!(region_is_exclusive(region) &&
                    region_is_physical(region)))
         {
            dump_allocator_states((int (*)(const char *, ...))ramlog_printf);
            MM_FATAL_ERROR2(("Persistent regions with mapping type %d is"
                             " not allowed",
                             region->meta_options),
                            OSE_MM_PMM_RESTORE_FAILED,
                            (OSERRCODE) 0);
         }
      }
   }
}

/*****************************************************************************/

static void
restore_regions_pass2(struct restartData* restartData)
{
   U32 i;
   struct regionData *data;
   mm_status_t status;

   for (i = 0; i < restartData->entryCnt; i++)
   {
      data = &restartData->regions[i];
      
      /* Create object. */
      data->pobj = mm_create_phys_object(data->tmp_allocator,
                                         MM_REGION_OPT_PERSISTENT);
      
      data->pobj->data_representation = PHYS_OBJ_FRAG_LIST;
      data->pobj->data.frag_list = data->tmp_frag_list;

      mm_attach_phys_object(data->pobj);
      
      data->region->phys_object = data->pobj;

      /* Map region. */
      status = mm_map_region(MM_KERNEL_DOMAIN, data->region);

      if (status != MM_SUCCESS)
      {
         ramlog_printf("Failure to add physical resources to region at: 0x%x\n",
                       data->region->log_object->base);
         dump_allocator_states((int (*)(const char *, ...))ramlog_printf);
         MM_FATAL_ERROR2(("ram_allocator: Add instance region.\n"),
                         OSE_MM_PMM_RESTORE_FAILED,
                         (OSERRCODE) 0);
      }
      data->region->meta_options |= MM_REGION_OPT_PERSISTENT;
      
      ramlog_printf("mm: Restored Warm region %#010lx->%#010lx:%s\n",
                    data->region->log_object->base,
                    data->region->log_object->base + data->region->log_object->size -1,
                    data->region->name);
   }
}

/*****************************************************************************/

/*
 * Connect to RAM PMM.
 */
static void
connect_ram_pmm(void)
{
   PROCESS pmm_ = gmm_state.pmm;

   /* FIXME: GMM shall handle the warm-start - not the RAM-allocator */
   /* Connect RAM-PMM to RAM allocator. */

   /* Send PMM_INIT_REQUEST signal to PMM and wait for reply. */
   {
      union SIGNAL *sig;
      static const SIGSELECT sig_select[] = {1, PMM_INIT_REPLY};
      sig = alloc(sizeof(struct pmm_init), PMM_INIT_REQUEST);
      
      /* Send init signal and wait for reply. */
      send(&sig, pmm_);
      sig = receive_from_pid(pmm_, sig_select);
      
      free_buf(&sig);
   }
   
   /* Send RESTORE_START signal and get list with persistent regions. */
   {
      struct restartData *list;
      U32 listSize, entryCnt;
      static const SIGSELECT sig_select[] = {1, PMM_RESTORE_START_REPLY};
      union SIGNAL *sig;
      
      sig = alloc(sizeof(SIGSELECT), PMM_RESTORE_START_REQUEST);
      send(&sig, pmm_);
      
      sig = receive_from_pid(pmm_, sig_select);
      
      entryCnt = sig->pmm_restore_start_reply.no_regions;
      
      listSize = sizeof(struct restartData) +
         (entryCnt-1)*sizeof(struct regionData);
      
      list = (struct restartData *)alloc(listSize, entryCnt);
      
      list->restartSig = (struct pmm_restore_start_reply *)sig;
      
      /* Reserve all physical fragments */
      restore_regions_pass1(list);
      
      /* Complete the region creation */
      restore_regions_pass2(list);
      
      free_buf((union SIGNAL **)(void*)&list->restartSig);
      free_buf((union SIGNAL **)(void*)&list);
   }
   
   /* Complete the restore phase by handshaking with PMM. */
   {
      union SIGNAL *sig;

      static const SIGSELECT sig_select[] = {1, PMM_RESTORE_COMPLETE_REPLY};
      sig = alloc(sizeof(SIGSELECT), PMM_RESTORE_COMPLETE_REQUEST);
      
      /* Send PMM_RESTORE_COMPLETE_REQUEST signal to pmm. */
      send(&sig, pmm_);
      
      /* Wait for reply. This signal completes the handshake. */
      sig = receive_from_pid(pmm_, sig_select);
      free_buf(&sig);
   }
}

/*****************************************************************************/

static void
connect_pmm(union SIGNAL *sig)
{
   const char *pmm_name;

   assert(sig != NULL);

   /*
    * The name of the PMM is used as a key when looking up
    * configuration variables.
    */
   pmm_name = &sig->pmm_assign.memory_type[0];

   gmm_state.pmm = sender(&sig);
   /*
    * if RAM, we must connect to the default RAM allocator.
    */
   if(strcmp(pmm_name,"RAM") == 0)
   {
      mm_partition_prepare_for_warm_start(sig->pmm_assign.max_nr_of_regions);
      
      /* Connect RAM-PMM to RAM allocator. */
      connect_ram_pmm();
      return;
   }
   
   MM_FATAL_ERROR2(("Only RAM PMM is supported.\n"),
                   OSE_MM_EHAL_SUBHAL_NO_MATCH, 0);
}

/*****************************************************************************/

void
mm_pmm_free(log_addr_t base) {
   
   if (!gmm_state.pmm) return;
   
   {
      U32 pmm_status;
      PROCESS pmm_ = gmm_state.pmm;
      static const SIGSELECT sig_select[] = {1, PMM_FREE_REPLY};
      union SIGNAL *sig;

      sig = alloc(sizeof(struct pmm_free), PMM_FREE_REQUEST);
      
      /* Send free request and wait for reply. */
      sig->pmm_free.base = base;
      send(&sig, pmm_);

      sig = receive_from_pid(pmm_, sig_select);
      pmm_status = sig->pmm_free.ecode;
      
      free_buf(&sig);
      
      /* Check error codes. */
      if (pmm_status != PMM_EOK) {
         ramlog_printf("PMM error: 0x%x\n",pmm_status);
         INTERNAL_ERROR();
      }
   }
}

/*****************************************************************************/

static mm_status_t
mm_pmm_alloc_sase(struct region *region)
{
   mm_status_t status = MM_SUCCESS;
   
   if (!gmm_state.pmm)
   {
      region->meta_options &= ~MM_REGION_OPT_PERSISTENT;
      return status;
   }

   {
      PROCESS pmm_ = gmm_state.pmm;
      char *meta_data;
      size_t meta_data_buf_len;
      static const SIGSELECT sig_select[] = {1, PMM_ALLOC_SASE_REPLY};
      union SIGNAL *sig;
      
      /* Send request and wait for answer. */
      /* Encode meta-data for logical region. */
      meta_data = mm_encode_metadata(region);
      
      /* Calculate length of meta-data. */
      meta_data_buf_len = strlen(meta_data) + 1;

      sig = alloc(sizeof(struct pmm_alloc_sase_request)
                  + meta_data_buf_len - 1,
                  PMM_ALLOC_SASE_REQUEST);
      
      {
         struct pmm_alloc_sase_request *sase_req;

         sase_req= &sig->pmm_alloc_sase_request;
         sase_req->base = region->log_object->base;
         sase_req->region_size = region->log_object->size;
         sase_req->metadata_size = meta_data_buf_len;
         memcpy(sase_req->metadata, meta_data, meta_data_buf_len);
      }

      /* Free buffer allocated in mm_encode. */
      free_buf((union SIGNAL **)(void*)&meta_data);

      /* Send SASE request and wait for a reply signal. */
      send(&sig, pmm_);
      sig = receive_from_pid(pmm_, sig_select);

      switch (sig->pmm_alloc_sase_reply.ecode) {
         
         case PMM_ETOO_MANY_REGIONS:
            status = MM_ETOO_MANY_PERSISTENT_REGIONS;
            break;
            
         case PMM_EOUT_OF_MEMORY:
            status = MM_EPMM_OUT_OF_MEMORY;
            break;
            
         case PMM_EOK:
            region->meta_options |= MM_REGION_OPT_PERSISTENT;
            status = MM_SUCCESS;
            break;
            
         default:
            ramlog_printf("PMM error: 0x%x\n", status);
            INTERNAL_ERROR();
      }
      
      free_buf(&sig);
   }

   return status;
}

/*****************************************************************************/

static mm_status_t
mm_pmm_alloc_sas(struct region *region)
{
   mm_status_t status = MM_SUCCESS;
   
   if (!gmm_state.pmm)
   {
      region->meta_options &= ~MM_REGION_OPT_PERSISTENT;
      return status;
   }

   {
      PROCESS pmm_ = gmm_state.pmm;
      struct phys_mem_cursor cursor;
      unsigned phys_count = 0;
      struct phys_object *phys_object = region->phys_object;

      char *meta_data;
      size_t meta_data_buf_len;

      union SIGNAL *sig;
      static const SIGSELECT sig_select[] = {1, PMM_ALLOC_SAS_REPLY};
      
      /* Send request and wait for answer. */
      /* Encode meta-data for logical region. */
      meta_data = mm_encode_metadata(region);
      
      /* Calculate length of meta-data. */
      meta_data_buf_len = strlen(meta_data) + 1;
      
      mm_phys_mem_get_first(phys_object, &cursor);
      do {
         phys_count++;
      } while (mm_phys_mem_get_next(&cursor));
      
      sig = alloc(sizeof(struct pmm_alloc_sas_request)
                  + phys_count * sizeof(struct phys_resource),
                  PMM_ALLOC_SAS_REQUEST);
      
      {
         struct pmm_alloc_sas_request *sas_req = &sig->pmm_alloc_sas_request;

         sas_req->base        = region->log_object->base;
         sas_req->region_size = region->log_object->size;
         sas_req->metadata_size = meta_data_buf_len;
         memcpy(sas_req->metadata, meta_data, meta_data_buf_len);
         
         /* Free buffer allocated in mm_encode. */
         free_buf((union SIGNAL **)(void*)&meta_data);
         
         /* Set the number of calculated fragments. */
         sas_req->no_fragments = phys_count;
      
         /*
          * Fill the fragment list.
          */
         {
            unsigned i = 0;
            
            mm_phys_mem_get_first(phys_object, &cursor);
            do
            {
               sas_req->fragments[i].base = cursor.phys_page.pa;
               sas_req->fragments[i].size = cursor.phys_page.size;
               i++;
               
            } while (mm_phys_mem_get_next(&cursor));
            assert(i == phys_count);
         }
      }
      
      /* Send SAS request. */
      send(&sig, pmm_);
      
      sig = receive_from_pid(pmm_, sig_select);
      
      switch (sig->pmm_alloc_sas_reply.ecode) {
         
         case PMM_ETOO_MANY_REGIONS:
         {
            struct allocator *allocator;

            allocator = region->phys_object->allocator;
            allocator->free(allocator, region->phys_object);
            status = MM_ETOO_MANY_PERSISTENT_REGIONS;
         }
         break;
         
         case PMM_EOUT_OF_MEMORY:
         {
            struct allocator *allocator;

            allocator = region->phys_object->allocator;
            allocator->free(allocator, region->phys_object);
            status = MM_EPMM_OUT_OF_MEMORY;
         }
         break;
         
         case PMM_EOK:
            region->meta_options |= MM_REGION_OPT_PERSISTENT;
            status = MM_SUCCESS;
            break;
            
         default:
            ramlog_printf("PMM error: 0x%x\n", status);
            INTERNAL_ERROR();
      }
      
      free_buf(&sig);
   }
   return status;
}

/*****************************************************************************/

static mm_status_t
mm_pmm_change_attr(struct region *region)
{
   mm_status_t status = MM_SUCCESS;
   
   if (!gmm_state.pmm) return status;

   assert(NULL != region);
   {
      PROCESS pmm_ = gmm_state.pmm;
      U32 pmm_status;
      /* Send request and wait for answer. */
      /* Encode meta-data for region. */
      char *meta_data = mm_encode_metadata(region);

      /* Calculate length of meta-data. */
      size_t meta_data_buf_len = strlen(meta_data) + 1;
      static const SIGSELECT sig_select[] = {1, PMM_CHANGE_REPLY};
      union SIGNAL *sig =  alloc(sizeof(struct pmm_change)
                                 + meta_data_buf_len - 1,
                                 PMM_CHANGE_REQUEST);
      
      /* Assign fields in struct. */
      {
         struct pmm_change *change_req = &sig->pmm_change;

         change_req->base = region->log_object->base;
         change_req->metadata_size = meta_data_buf_len;
         memcpy(change_req->metadata, meta_data, meta_data_buf_len);
      }
      
      /* Free buffer allocated in mm_encode. */
      free_buf((union SIGNAL **)(void*)&meta_data);
      
      /* Send change request and wait for a reply signal. */
      send(&sig, pmm_);

      sig = receive_from_pid(pmm_, sig_select);
      pmm_status = sig->pmm_change.ecode;
      free_buf(&sig);

      /* Check error code. */
      if (pmm_status != PMM_EOK) {
         ramlog_printf("PMM error: 0x%x\n", pmm_status);
         INTERNAL_ERROR();
      }
   }
   return status;
}

/*****************************************************************************/
static union SIGNAL *
mm_multithread(struct gmm_vars *gmm_state, mm_bool_t enable)
{
   struct mm_multithread *reply = (struct mm_multithread *)
      alloc(sizeof(struct mm_multithread), MM_MULTITHREAD_REPLY);

   if (enable)
   {
      gmm_state->multithread_enable++;
      reply->status = MM_SUCCESS;
   }
   else
   {
      /* FIXME: Implement safe disable
       *
       * gmm_state.multithread_enable--;
       */
      reply->status = MM_SUCCESS;
   }

   reply->enable = gmm_state->multithread_enable != 0;
   
   return (union SIGNAL*)reply;
}

/*****************************************************************************/

static union SIGNAL *
mm_add_phys_resource(union SIGNAL *request)
{
   mm_status_t status = ~MM_SUCCESS;
   struct region *region;
   struct allocator *allocator;
   struct log_object  *lobj = NULL;
   struct phys_object *pobj = NULL;

   /* Then look up instance region. */
   region = mm_find_region_by_addr(&gmm_state.domains,
                                   request->mm_add_phys_resource.domain,
                                   request->mm_add_phys_resource.base,
                                   TRUE);  /* Exact match. */
   
   if (region->phys_object != NULL)
   {
      ramlog_printf("Attempt to add physical resource to a region that already"
                    " has one.\n");
      status = MM_EILLEGAL_OPERATION;
      goto _return_;
   }
   
   lobj = region->log_object;

   /* I have not looked into the possibility to perform this operation om SASE
    * or MAS mapped regions. */
   if (!(region_is_exclusive(region) && !region_is_physical(region)))
   {
      ramlog_printf("MM_ADD_PHYS_RESOURCE_REQUEST is only supported for "
                    "regions in SAS mapped partitions: %x.\n",
                    region->log_object->options);
      status = MM_EILLEGAL_OPERATION;
      goto _return_;
   }
   
   /* FIXME: The code below is extracted from create_sas_or_mas_region. It
    * Should be put in a separate function that could be used by this
    * function and create_sas_or_mas_region. */
   allocator = mm_find_allocator_by_name(request->mm_add_phys_resource.phys_mem);
   
   if (allocator == NULL)
   {
      ramlog_printf("Allocator for phys_mem '%s' not found.\n",
                    request->mm_add_phys_resource.phys_mem);
      status = MM_ENON_EXISTING_PHYS_MEM;
      goto _return_;
   }

   pobj = mm_create_phys_object(allocator, 0);
   
   if (pobj == NULL)
   {
      ramlog_printf("Failed to create physical object.\n");
      status = MM_EOUT_OF_PHYS_MEM;
      goto _return_;
   }

   mm_attach_phys_object(pobj);

   status = allocator->alloc_s(allocator, pobj, lobj->size, lobj->base);

   if (status != MM_SUCCESS)
   {
      ramlog_printf("Failed to allocate physical memory.\n");
      goto _return_;
   }

   region->phys_object = pobj;
   status = mm_map_region(request->mm_add_phys_resource.domain, region);
   if (status != MM_SUCCESS)
   {
      ramlog_printf("Failed to map instance region.\n");
      goto _return_;
   }
   
   status = MM_SUCCESS;
   
  _return_:
   if (status != MM_SUCCESS)
   {
      error(0);
      /* Cleanup */
      if (pobj != NULL)
      {
         mm_detach_phys_object(pobj);
      }
      region->phys_object = NULL;
   }
   
   request->sig_no = MM_ADD_PHYS_RESOURCE_REPLY;
   request->mm_add_phys_resource.status = status;
   
   return request;
}

/*****************************************************************************/

static union SIGNAL *
mm_remove_phys_resource(union SIGNAL *request)
{
   mm_status_t status = MM_SUCCESS;
   struct domain *domain;
   struct region *region;

   OSDOMAIN domainId = request->mm_remove_phys_resource.domain;

   log_addr_t log_base = request->mm_remove_phys_resource.base;

   domain = mm_find_domain_by_domid(&gmm_state.domains, domainId);
   if (domain == NULL)
   {
      status = MM_ENON_EXISTING_DOMAIN;
      goto _return_;
   }
   
   /* Then look up instance region. */
   region = mm_find_region_by_addr(
      &gmm_state.domains, domainId, log_base, TRUE);  /* Exact match. */
   if (region == NULL)
   {
      status = MM_ENON_EXISTING_REGION;
      goto _return_;
   }
   
   if (region->phys_object == NULL)
   {
      ramlog_printf("Attempt to free physical resource from to a logical "
                    "region.\n");
      status = MM_EILLEGAL_OPERATION;
      goto _return_;
   }

   if (!(region_is_exclusive(region) && !region_is_physical(region)))
   {
      ramlog_printf("MM_REMOVE_PHYS_RESOURCE_REQUEST is only supported for "
                    "regions in SAS mapped partitions: %x.\n",
                    region->log_object->options);
      status = MM_EILLEGAL_OPERATION;
      goto _return_;
   }
   
   /* Unmap region. */
   mm_unmap_region(domain, region);
   
   mm_detach_phys_object(region->phys_object);
   region->phys_object = NULL;
   
  _return_:
   request->sig_no = MM_REMOVE_PHYS_RESOURCE_REPLY;
   request->mm_remove_phys_resource.status = status;
   
   return request;
}

/*****************************************************************************/

static union SIGNAL *
mm_alloc_phys_resource(struct gmm_vars *gmm, union SIGNAL *request)
{
   struct mm_alloc_phys_resource *reply = &request->mm_alloc_phys_resource;

   reply->sig_no = MM_ALLOC_PHYS_RESOURCE_REPLY;
   reply->status =
      domain_alloc_phys_resource(&(gmm->domains),
                                 request->mm_alloc_phys_resource.domain,
                                 &(request->mm_alloc_phys_resource.base),
                                 request->mm_alloc_phys_resource.size,
                                 request->mm_alloc_phys_resource.align,
                                 request->mm_alloc_phys_resource.phys_mem);

   return (union SIGNAL *)reply;
}

/*****************************************************************************/

static union SIGNAL *
mm_free_phys_resource(struct gmm_vars *gmm, union SIGNAL *request)
{
   struct mm_free_phys_resource *reply = &request->mm_free_phys_resource;

   reply->sig_no = MM_FREE_PHYS_RESOURCE_REPLY;
   reply->status =
      domain_free_phys_resource(&(gmm->domains),
                                request->mm_free_phys_resource.domain,
                                request->mm_free_phys_resource.base);

   return (union SIGNAL *)reply;
}

/*****************************************************************************/

static union SIGNAL *
mm_alloc_metadata_region(union SIGNAL *request)
{
   struct mm_alloc_metadata *reply = &request->mm_alloc_metadata;
   struct partition *hal_partition = mm_find_domain_partition_by_name(
      gmm_state.system_domain, HAL_LOG_MEM_NAME);

   reply->sig_no = MM_ALLOC_METADATA_REPLY;
   
   if (gmm_state.hal_info->has_mmu)
   {
      log_addr_t la, pa, tmp_pa, page;

      if (hal_partition == NULL)
      {
         reply->status = MM_ENON_EXISTING_LOG_MEM;
         return (union SIGNAL*)reply;
      }

      if (!mm_alloc_log_mem2(hal_partition,
                             request->mm_alloc_metadata.size, &la))
      {
         reply->status = MM_EOUT_OF_LOG_MEM;
         return (union SIGNAL*)reply;
      }

      if (!mm_page_alloc(request->mm_alloc_metadata.size, &tmp_pa))
      {
         mm_free_log_mem(hal_partition, la, request->mm_alloc_metadata.size);

         reply->status = MM_EOUT_OF_PHYS_MEM;
         return (union SIGNAL*)reply;
      }
      
      pa = tmp_pa;
      
      for (page = 0; page < request->mm_alloc_metadata.size; page += PAGE_SIZE)
      {
         hal_status_t status;

         status = hal_map_page(gmm_state.system_domain->hal_domain,
                               la + page, pa + page, PAGE_SIZE, 0,
                               gmm_state.internal_access_mode,
                               DEFAULT_CACHE_MODE);
         if (status != HAL_SUCCESS)
         {
            size_t size = page * PAGE_SIZE;
            if (size > 0)
            {
               hal_unmap_range(gmm_state.system_domain->hal_domain, la, size, 0);
            }

            mm_page_free(pa, request->mm_alloc_metadata.size);
            mm_free_log_mem(hal_partition, la, request->mm_alloc_metadata.size);

            reply->status = mm_hal_status_to_mm_status(status);
            return (union SIGNAL*)reply;
         }
      }

      reply->base   = la;
      reply->status = 0;
   }
   else
   {
      log_addr_t la;
      
      if (!mm_page_alloc(request->mm_alloc_metadata.size, &la))
      {
         reply->status = MM_EOUT_OF_PHYS_MEM;
         return (union SIGNAL*)reply;
      }

      reply->base   = la;
      reply->status = 0;
   }
   
   return (union SIGNAL*)reply;
}

/*****************************************************************************/

static union SIGNAL *
mm_free_metadata_region(union SIGNAL *request)
{
   struct mm_free_metadata *reply = &request->mm_free_metadata;
   struct partition *hal_partition = mm_find_domain_partition_by_name(
      gmm_state.system_domain, HAL_LOG_MEM_NAME);
   phys_addr_t phys;

   reply->sig_no = MM_FREE_METADATA_REPLY;

   if (hal_partition == NULL && gmm_state.hal_info->has_mmu)
   {
      reply->status = MM_ENON_EXISTING_LOG_MEM;
      return (union SIGNAL*)reply;
   }
   
   if (hal_lookup(gmm_state.system_domain->segnum,
                  request->mm_free_metadata.base, &phys, NULL, NULL, 0)
       != HAL_SUCCESS)
   {
      reply->status = MM_EOUTSIDE_REGION;
      return (union SIGNAL*)reply;
   }

   hal_unmap_range(gmm_state.system_domain->hal_domain,
                   request->mm_free_metadata.base,
                   request->mm_free_metadata.size, 0);

   mm_page_free((log_addr_t)phys, request->mm_free_metadata.size);

   if (gmm_state.hal_info->has_mmu)
   {
      mm_free_log_mem(hal_partition,
                      request->mm_free_metadata.base,
                      request->mm_free_metadata.size);
   }

   reply->status = 0;
   
   return (union SIGNAL*)reply;
}

/*****************************************************************************
 * Description:
 * Find all physical fragments that belongs to the supplied logical address.
 * If there is more than one physical fragment the input signal is freed
 * and a new one large enough to hold all fragments is allocated and returned.
 * 
 */

static union SIGNAL *
prepare_reply_phys(union SIGNAL *request)
{
  union SIGNAL *reply;
  struct region *region;
  struct phys_object *pobj;
  
  /* Look up instance region. */
  region = mm_find_region_by_addr(
     &gmm_state.domains,
     request->mm_create_region.domain,
     request->mm_create_region.base,
     TRUE);  /* Exact match. */

  
  assert(region != NULL);
  pobj = region->phys_object;

  /* Return if there is no physical memory attached to the instance region. */
  if (pobj == NULL)
  {
    request->mm_create_region.phys_count = 0;
    request->mm_create_region.phys_resources[0].base = 0;
    request->mm_create_region.phys_resources[0].size = 0;
    return request;          
  }
            
  {  
    unsigned i;
    struct phys_mem_cursor cursor;

    /* Get the first fragmanet. */
    mm_phys_mem_get_first(pobj, &cursor);
    request->mm_create_region.phys_resources[0].base = cursor.phys_page.pa;
    request->mm_create_region.phys_resources[0].size = cursor.phys_page.size;
    request->mm_create_region.phys_count = 1;
    
    /* If there is only one fragment we can use the original signal. */
    if(0 == mm_phys_mem_get_next(&cursor))
    {
      return request;      
    }
    
    /* More than one fragment, need to allocate a larger signal.
     * To get simpler code we allocate a signal large enough to
     * hold the maximum number of phys_resources. */    
    reply = alloc(sizeof(struct mm_create_region) + (PHYS_RES_MAX - 1) *
                  sizeof(struct phys_resource), 0);

    /* Copy the signal data from the original signal to the new signal so we 
     * can free the request signal.
     */
    memcpy(reply, request, sizeof(struct mm_create_region));
    reply->sig_no = MM_CREATE_REGION_REPLY;
    free_buf(&request);
        
    /*
     * Traverse through all physical fragments except for the first fragment
     * since it is already copied in to the new signal.
     */
    for(i = 1; i < PHYS_RES_MAX; i++)
    {
      reply->mm_create_region.phys_resources[i].base = cursor.phys_page.pa;
      reply->mm_create_region.phys_resources[i].size = cursor.phys_page.size;
      reply->mm_create_region.phys_count++;
      if(0 == mm_phys_mem_get_next(&cursor))
      { /* No more fragments, break out. */
        break;
      }
    }
    /* If there is more fragments than PHYS_RES_MAX notify client by
     * setting phys_count to -1  */
    if(i >= PHYS_RES_MAX)
    {
      reply->mm_create_region.phys_count = (U32)-1;
    }
             
  }
  return reply;  
}


#ifdef OSPP
/*
 ***********************************************************************
 *              Helpers to MM_MAKE_RESIDENT/MM_MAKE_UNRESIDENT
 ***********************************************************************
 */

static U32
mm_opts2hal_opts(U32 mm_opts, U32 *hal_opts)
{
   U32 hal_opts_tmp = 0;
   U32 mm_opts_orig = mm_opts;
   
   if (MM_MAKE_RESIDENT_DATA & mm_opts)
   {
      hal_opts_tmp |= HAL_CONTENTS_TYPE_DATA;
      mm_opts &= ~MM_MAKE_RESIDENT_DATA;
   }
   
   if (MM_MAKE_RESIDENT_CODE & mm_opts)
   {
      hal_opts_tmp |= HAL_CONTENTS_TYPE_TEXT;
      mm_opts &= ~MM_MAKE_RESIDENT_CODE;
   }

   if (0 != mm_opts)
   {
      ramlog_printf("Failed to translate mm_options 0x%08lx to HAL options\n",
                    mm_opts_orig);
      return 0;
   }

   *hal_opts = hal_opts_tmp;
   return 1;
}

/******************************************************************************
 * This function checks if a range can be residently mapped. The criteria is:
 * The range is entirely covered by regions in the system domain with identical
 * permissions and cache mode (incl. access mode).
 */
static struct region *
range_can_be_resident(log_addr_t base0, log_size_t size)
{
   struct region *reg0;
   log_addr_t base = base0;
   log_addr_t top = base+size-1;
   
   if (base > top) { /* Wrap */
      return NULL;
   }
   
   /* Find the first region. This region will be used to verify the
    * permissions and cache modes of the succeeding regions. */
   
   /* Last parameter is FALSE indicating that the region can start below the
    * proveded base. Exact match not required */
   reg0 = mm_find_region_by_addr(&gmm_state.domains,
                                  MM_KERNEL_DOMAIN,
                                  base,
                                  FALSE);
   if (NULL == reg0)
   {
      /* base is not a located in any region in the system domain. */
      return NULL;
   }
   
   /* Only 1-1 mappings are supported */
   /* FIXME: What about shifted SASE regions? */
   if (!(region_is_exclusive(reg0) && region_is_physical(reg0)))
   {
      return NULL;
   }
   
   /* The previous value of base could be located inside ireg0 =>
    * base += size is incorrect.
    */
   base = reg0->log_object->base+reg0->log_object->size;
   
   
   {
      /* Verify succeeding regions. */
      
      U32 exp_permissions = reg0->permissions;
      U32 exp_cache_mode  = reg0->cache_mode;
      
      /* About the while condition:
       * + Base might wrap here. This is detected by (base > base0) being
       *   false
       */
      while (base <= top && base > base0) {
         struct region *reg;
         
         /* Last parameter is TRUE indicating that the region's base must be
          * the provided base */
         reg = mm_find_region_by_addr(&gmm_state.domains,
                                       MM_KERNEL_DOMAIN,
                                       base,
                                       TRUE);
         if (NULL == reg)
         {
            /* There is no region in the system domain starting at base. */
            return NULL;
         }
         
         if (exp_permissions != reg->permissions) {
            return NULL;
         }
         
         if (exp_cache_mode != reg->cache_mode) {
            return NULL;
         }
         
         /* Only 1-1 mappings are supported. Note that this must be verified
          * for all succeeding regions because the requested range might
          * contain more than one log_mem. */
         /* FIXME: What about shifted SASE regions? */
         if (reg->options != reg0->options)
         {
            return NULL;
         }
         
         base += reg->log_object->size;
      }
      
      /* The range is entirely covered with system domain regions with
       * identical permissions & cache modes. */
      
      return reg0;
   }
}

/*
 ***********************************************************************
 *              MM_MAKE_RESIDENT
 ***********************************************************************
 */

/******************************************************************************
 * create_resident_mapping is a wrapper to hal_create_resident_mapping that
 * ensures that the input parameters and return values are handled in the same
 * way on all cpus.
 */
static mm_status_t
create_resident_mapping(struct hal_create_resident_mapping create)
{
   hal_status_t hal_status;
   struct partition_class *p;

   /* Adding resident pages that are part
    * of a resident log_mem is illegal. */
   p = mm_space_is_resident(create.base, create.size);
   if (p != NULL)
   {
      return MM_EILLEGAL_OPERATION;
   }


   hal_status = hal_create_resident_mapping(create.hal_domain,
                                            create.base,
                                            (phys_addr_t)create.base,
                                            create.size,
                                            create.hal_options,
                                            create.permissions,
                                            create.mem_attr);
   if (HAL_SUCCESS == hal_status)
   {
      return MM_SUCCESS;
   }

   ramlog_printf("hal_create_resident_mapping returned 0x%lx,"
                 " expected 0x%lx\n",
                 hal_status, HAL_SUCCESS);
   
   if (HAL_EINVALID_SIZE == hal_status)
   {
      ramlog_printf("Size %#x is not supported by MMU.\n", create.size);
      return MM_EILLEGAL_AREA;
   }
   
   if (HAL_EINVALID_ALIGNMENT == hal_status)
   {
      ramlog_printf("Base address %#x has illegal alignment.\n", create.base);
      return MM_EILLEGAL_AREA;
   }
   
   ramlog_printf("Out of resident mapping resources\n");
   return MM_EOUT_OF_RESOURCES;
}

/*****************************************************************************/

static union SIGNAL *
make_resident(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   struct hal_create_resident_mapping create;

   (void)gmm_vars;
   
   {
      struct region *region;
      U32 hal_options;
      
      request->sig_no = MM_MAKE_RESIDENT_REPLY;
      
      if (NULL == hal_create_resident_mapping)
      {
         /* The current HAL does not support hal_create_resident_mapping() */
         request->mm_make_resident.status = MM_EOUT_OF_RESOURCES;
         return request;
      }
      
      region = range_can_be_resident(request->mm_make_resident.base,
                                     request->mm_make_resident.size);
      if (region == NULL)
      {
         request->mm_make_resident.status = MM_EILLEGAL_AREA;
         return request;
      }
      
      if (!mm_opts2hal_opts(request->mm_make_resident.options, &hal_options))
      {
         ramlog_printf("Attempt to create a resident mapping with"
                       " unsupported value for 'options' (0x%08lx)"
                       " rejected.\n",
                       request->mm_make_resident.options);
         request->mm_make_resident.status = MM_EOPTION_NOT_SUPPORTED;
         return request;
      }

      /* Deny if hal_superpage_threshold is confed. */
      if (gmm_vars->make_resident_disabled) {
         request->mm_make_resident.status = MM_EOUT_OF_RESOURCES;
         return request;
      }
      
      /* Put all parameters in one struct to ensure that the same parameters
       * are used on all cores. */
      create.hal_domain  = gmm_vars->system_domain->hal_domain;
      create.base        = request->mm_make_resident.base;
      create.size        = request->mm_make_resident.size;
      create.hal_options = hal_options;
      create.permissions = region->permissions;
      create.mem_attr    = region->cache_mode;
      
      /* sig_no is only used in the SMP case */
      create.sig_no = 0; /* Avoid compiler warning. */
   }
   
   {
      mm_status_t status = create_resident_mapping(create);
      
#if defined(OSE_SMP) && defined(OSPP)
      create_and_start_cpu_agents();
      create.sig_no = HAL_CREATE_RESIDENT_MAPPING;

      if (MM_SUCCESS == status)
      {
         cpuid_t id = smp_agents.num_cpus;
         
         while (--id > 0)
         {
            static const SIGSELECT sel[] = {1, HAL_CREATE_RESIDENT_MAPPING};
            
            /* sig_no will be overwritten in the assugnment after alloc, so I
             * use 0 to increase readability.
             */
            union SIGNAL *sig = alloc(
               sizeof(struct hal_create_resident_mapping), 0);
            
            sig->hal_create_resident_mapping = create;

            send(&sig, smp_agents.pid[id-1]);
            sig = receive(sel);
            free_buf(&sig);
         }
      }
      
#endif
      
      request->mm_make_resident.status = status;
      return request;
   }
} /* make_resident() */


/*
 ***********************************************************************
 *              MM_MAKE_UNRESIDENT
 ***********************************************************************
 */

/******************************************************************************
 * delete_resident_mapping is a wrapper to hal_delete_resident_mapping that
 * ensures that the input parameters are handled in the same way on all cpus.
 */
static mm_status_t
delete_resident_mapping(struct hal_delete_resident_mapping delete)
{
   hal_status_t hal_status;
   struct partition_class *p;

   /* Removing resident pages that are part
    * of a resident log_mem is illegal. */
   p = mm_space_is_resident(delete.base, delete.size);
   if (p != NULL)
   {
      return MM_EILLEGAL_OPERATION;
   }

   hal_status = hal_delete_resident_mapping(
      delete.hal_domain, delete.base, delete.size, delete.hal_options);
   if (HAL_SUCCESS == hal_status)
   {
      return MM_SUCCESS;
   }

#if 0
   ramlog_printf("hal_delete_resident_mapping returned 0x%lx,"
                 " expected 0x%lx\n",
                 hal_status, HAL_SUCCESS);
#endif

   return MM_EOUT_OF_RESOURCES;
}

/******************************************************************************
 * delete_resident_mappings is a wrapper to hal_delete_resident_mappings that
 * ensures that the input parameters are handled in the same way on all cpus.
 */
static mm_status_t
delete_resident_mappings(struct hal_delete_resident_mappings delete)
{
   hal_status_t hal_status;

   hal_status = hal_delete_resident_mappings(delete.base, delete.size);
   if (HAL_SUCCESS == hal_status)
   {
      return MM_SUCCESS;
   }

#if 0
   ramlog_printf("hal_delete_resident_mapping returned 0x%lx,"
                 " expected 0x%lx\n", hal_status, HAL_SUCCESS);
#endif
   
   return MM_EOUT_OF_RESOURCES;
}

/*****************************************************************************/

static union SIGNAL *
make_unresident(struct gmm_vars *gmm_vars, union SIGNAL *request)
{
   struct hal_delete_resident_mapping delete;
   {
      U32 hal_options;
      
      request->sig_no = MM_MAKE_UNRESIDENT_REPLY;
      
      if (NULL == hal_delete_resident_mapping)
      {
         /* The current HAL does not support hal_delete_resident_mapping() */
         request->mm_make_resident.status = MM_EOUT_OF_RESOURCES;
         return request;
      }
      
      /* The options are needed since data and instruction mappings can be
       * removed independently. */
      if (!mm_opts2hal_opts(request->mm_make_resident.options, &hal_options))
      {
         ramlog_printf("Attempt to delete a resident mapping with"
                       " unsupported value for 'options' (0x%08lx)"
                       " rejected.\n",
                       request->mm_make_resident.options);
         request->mm_make_resident.status = MM_EOPTION_NOT_SUPPORTED;
         return request;
      }

      /* Deny if hal_superpage_threshold is confed. */
      if (gmm_vars->make_resident_disabled) {
         request->mm_make_resident.status = MM_EOUT_OF_RESOURCES;
         return request;
      }
      
      /* Put all parameters in one struct to ensure that the same parameters
       * are used on all cores. */
      delete.hal_domain = gmm_vars->system_domain->hal_domain;
      delete.base = request->mm_make_resident.base;
      delete.size = request->mm_make_resident.size;
      delete.hal_options = hal_options;
      
      /* sig_no is only used in the SMP case */
      delete.sig_no = 0; /* Avoid compiler warning. */
   }
   
   {
      mm_status_t status = delete_resident_mapping(delete);
      
#if defined(OSE_SMP)
      create_and_start_cpu_agents();
      delete.sig_no = HAL_DELETE_RESIDENT_MAPPING;
      
      if (MM_SUCCESS == status)
      {
         cpuid_t id = smp_agents.num_cpus;
         
         while (--id > 0)
         {
            static const SIGSELECT sel[] = {1, HAL_DELETE_RESIDENT_MAPPING};
            
            /* sig_no will be overwritten in the assignment after alloc, so I
             * use 0 to increase readability.
             */
            union SIGNAL *sig = alloc(
               sizeof(struct hal_delete_resident_mapping), 0);
            
            sig->hal_delete_resident_mapping = delete;

            send(&sig, smp_agents.pid[id-1]);
            sig = receive(sel);
            free_buf(&sig);
         }
      }
      
#endif
      
      request->mm_make_resident.status = status;
      
      return request;
   }
}
/* make_unresident() */


/*****************************************************************************/

static mm_status_t
make_unresident_range(struct gmm_vars *gmm_vars,
                      log_addr_t base, log_size_t size)
{
   struct hal_delete_resident_mappings delete;
   struct partition_class *p;

   (void)gmm_vars;

   if (NULL == hal_delete_resident_mapping)
   {
      /* The current HAL does not support hal_delete_resident_mapping() */
      return MM_SUCCESS;
   }

   /* Removing resident pages that are part
    * of a resident log_mem is illegal. */
   p = mm_space_is_resident(base, size);
   if (p != NULL)
   {
      return MM_EILLEGAL_OPERATION;
   }

   /* If hal_superpage_threshold is confed, TLB (E500 TLB1) invalidation is
    * performed in mmu_hal_unmap_range() */
   if (gmm_vars->make_resident_disabled) {
      return MM_SUCCESS;
   }
   
   
   /* Put all parameters in one struct to ensure that the same parameters
    * are used on all cores. */
   delete.base = base;
   delete.size = size;

   /* sig_no is only used in the SMP case */
   delete.sig_no = 0; /* Avoid compiler warning. */

   {
      mm_status_t status = delete_resident_mappings(delete);
      
#ifdef OSE_SMP
      delete.sig_no = HAL_DELETE_RESIDENT_MAPPINGS;
      
      if (MM_SUCCESS == status && 0 != smp_agents.num_cpus)
      {
         cpuid_t id = smp_agents.num_cpus;
         
         while (--id > 0)
         {
            static const SIGSELECT sel[] = {1, HAL_DELETE_RESIDENT_MAPPINGS};
            
            /* sig_no will be overwritten in the assignment after alloc, so I
             * use 0 to increase readability.
             */
            union SIGNAL *sig = alloc(
               sizeof(struct hal_delete_resident_mappings), 0);
            
            sig->hal_delete_resident_mappings = delete;

            send(&sig, smp_agents.pid[id-1]);
            sig = receive(sel);
            free_buf(&sig);
         }
      }
#endif
      
      return status;
   }
}
/* make_unresident() */

#endif /* OSPP */


#if defined(OSE_SMP) && defined(OSPP)

/******************************************************************************
 * MM agent for core 1 ..
 *****************************************************************************/

static OS_PROCESS(gmm_agent)
{
   for (;;)
   {
      union SIGNAL *sig = receive(select_all);
      
      switch (sig->sig_no)
      {
         case HAL_CREATE_RESIDENT_MAPPING:
         {
            mm_status_t status
               = create_resident_mapping(sig->hal_create_resident_mapping);
            
            if (MM_SUCCESS != status)
            {
               error2(OSE_MM_EINTERNAL_ERROR, (OSERRCODE)status);
               free_buf(&sig);
               continue;
            }
            
            break;
         }

         case HAL_DELETE_RESIDENT_MAPPING:
         {
            mm_status_t status
               = delete_resident_mapping(sig->hal_delete_resident_mapping);
            
            if (MM_SUCCESS != status)
            {
               error2(OSE_MM_EINTERNAL_ERROR, (OSERRCODE)status);
               free_buf(&sig);
               continue;
            }
            
            break;
         }

         case HAL_DELETE_RESIDENT_MAPPINGS:
         {
            delete_resident_mappings(sig->hal_delete_resident_mappings);


            break;
         }
         
         case HAL_GET_CORE_MMU_STAT:
         {
            sig->hal_get_core_mmu_stat.status =
               hal_get_mmu_statistics(sig->hal_get_core_mmu_stat.tags);

            break;
         }

         default:
	    error2(OSE_MM_EUNKNOWN_SIGNAL, (OSERRCODE) sig);
            free_buf(&sig);
            continue; /* Avoid send(NIL) if for some reason error2 returns. */
      }
      send(&sig, sender(&sig));
   }
}

/*****************************************************************************/

static void
create_and_start_cpu_agents(void)
{
   cpuid_t id = ose_num_cpus();

   if (smp_agents.num_cpus != 0)  {
      /* Create agents only once. */
      return;
   }

   smp_agents.num_cpus = id;
   
   while (--id > 0) /* (num_cpus-1) .. 1 */
   {
      /* The purpose of the initiation is to make sure that the buffer is
       * large enough. 4 digits "should be enough for everyone".
       */
      char name[] = GMM_NAME_STRING "XXXX";
      PROCESS agent_pid;
      
      snprintf(name, sizeof name, GMM_NAME_STRING "%02lu", id);
      agent_pid = create_process(OS_PRI_PROC,
                                 name,
                                 gmm_agent,
                                 0x100,
                                 get_pri(current_process()),
                                 (OSTIME)0,
                                 get_bid(current_process()),
                                 (struct OS_redir_entry *)NULL,
                                 (OSVECTOR)0,
                                 (OSUSER)0);
      
      ose_bind_process(agent_pid, id);
      start (agent_pid);
      smp_agents.pid[id-1] = agent_pid;
   }
}

static void
stop_and_kill_cpu_agents(void)
{
   cpuid_t id = smp_agents.num_cpus;

   if (smp_agents.num_cpus == 0)  {
      /* Kill agents only when they exist. */
      return;
   }

   smp_agents.num_cpus = 0; 
   
   while (--id > 0) /* (num_cpus-1) .. 1 */
   {
      stop(smp_agents.pid[id-1]);
      kill_proc(smp_agents.pid[id-1]);
   }
}

#endif

/*
 ***********************************************************************
 *              MM main process/signal engine
 ***********************************************************************
 */

/* Identifiers with external linkage must have a prototype. */
OS_PROCESS(ose_memory_mgr);

OS_PROCESS(ose_memory_mgr)
{
   int pmm_connected;
#ifdef LIBWHAT
   LIBWHAT;
#endif

   pmm_connected = 0;
   ose_mutex_init(&reclaim_region_mutex, OSE_MUTEX_PRIO_INHERIT);
   
   gmm_state.gmm_pid = current_process();
   gmm_state.data_accessable = TRUE;   
   
   for (;;)
   {
      union SIGNAL *reply = NULL;
      union SIGNAL *request;
      PROCESS client;

      request = receive((const SIGSELECT *)select_all);

      /* Access of MM data in freeze mode will be denied */
      gmm_state.data_accessable = FALSE;

      client = sender(&request);

      PRINT_SIGNAL(("gmm: received signal: %ld client_pid: %#lx\n",
                    request->sig_no, client));

      switch(request->sig_no)
      {
         case PMM_ASSIGN:
         {
            if (pmm_connected)
            {
               error2(OSE_MM_EMULTIPLE_PMM_CONNECT, 0);
            }
            else
            {
#ifndef softOSE
               if (ose_read_tb != NULL)
               {
                  gmm_stats.connect_pmm.start = ose_read_tb();
               }
#endif
               connect_pmm(request);

#ifndef softOSE
               if (ose_read_tb != NULL)
               {
                  gmm_stats.connect_pmm.end = ose_read_tb();
               }
#endif

               free_buf(&request);
               pmm_connected = 1;
            }
	    break;
         }
	 case MM_CREATE_DOMAIN_REQUEST:
	 {
	    reply = create_domain(&gmm_state, request);
	    break;
	 }
	 case MM_DELETE_DOMAIN_REQUEST:
	 {
	    reply = delete_domain(&gmm_state, request);
	    break;
	 }
	 case MM_CREATE_REGION_REQUEST:
	 {
#if (defined(OSPP) || defined(OSMIPS))
            /*
             * For PPC and MIPS, it is not possible to create CI regions with
             * write disallowed, since we are invalidating the region.
             * Therefore we must set the su-write permission, create the region
             * and then perform a change permission back to the intended
             * permission.
             */
            int write_allow = False;
            /* save original permissions */
            U32 perm = request->mm_create_region.permissions;

            /* First of all check the original permissions. */
            if (!mm_check_permission(perm))
            {
               /* Bailout before creation of tmp regions. */
               reply = request;
               reply->mm_create_region.status = MM_EILLEGAL_PERMISSION;
               reply->mm_create_region.sig_no = MM_CREATE_REGION_REPLY;
               break;
            }

            if((request->mm_create_region.cache_mode & MM_CACHE_INHIBITED) &&
               (request->mm_create_region.permissions & MM_SU_WRITE) == 0 &&
               (request->mm_create_region.options & MM_CREATE_REGION_LOGICAL) == 0 &&
               (request->mm_create_region.options & MM_CREATE_REGION_FAULTABLE) == 0)
            {
               write_allow = True;
               /* temporarily add SU_READ and SU_WRITE */
               request->mm_create_region.permissions |=
                  (MM_SU_READ | MM_SU_WRITE);
            }
            
	    reply = create_region(&gmm_state, request);
            if(write_allow && reply->mm_create_region.status == MM_SUCCESS)
            {
               union SIGNAL *sig;

               sig = alloc(sizeof (struct mm_change_permissions),
                        MM_CHANGE_PERMISSIONS_REQUEST);
               {
                  struct mm_change_permissions *change_sig;
                  change_sig = &sig->mm_change_permissions;
                  
               /* restore intended permission. */
               change_sig->permissions = perm;
               change_sig->domain = reply->mm_create_region.domain;
               change_sig->base = reply->mm_create_region.base;
               }
               sig = change_permissions(&gmm_state, sig);
               
               if(sig->mm_change_permissions.status != MM_SUCCESS)
               {
                  INTERNAL_ERROR();
               }
               if (HAL_SUCCESS !=
                   hal_actual_permissions(&reply->mm_create_region.permissions,
                                          perm,
                                          reply->mm_create_region.base, 0))
               {
                  INTERNAL_ERROR();
               }
               free_buf(&sig);
            }
#else
	    reply = create_region(&gmm_state, request);
#endif
            /* Return the allocated phys_resources only if the client wants
             * to have it and the creation was successful. */
            if( (reply->mm_create_region.options & MM_CREATE_REGION_RETURN_PHYS) &&
                (MM_SUCCESS == reply->mm_create_region.status ) )
            {
              reply = prepare_reply_phys(reply);
            }
	    break;
	 }
	 case MM_DELETE_REGION_REQUEST:
	 {
	    reply = delete_region(&gmm_state, request);
	    break;
	 }
	 case MM_SHARE_REGION_REQUEST:
	 {
	    reply = share_region(&gmm_state, request);
	    break;
	 }
	 case MM_WRITE_REQUEST:
	 {
            OSADDRESS supervisor;

            supervisor = zzkrn_get_pid_info(sender(&request), 10);
            if (!supervisor)
            {
               request->mm_write.sig_no = MM_WRITE_REPLY;
               request->mm_write.status = MM_EILLEGAL_OPERATION;
               break;
            }

	    reply = write_region(&gmm_state, request);
	    break;
	 }
	 case MM_LIST_DOMAINS_REQUEST:
	 {
	    reply = list_domains(&gmm_state, request);
	    break;
	 }
	 case MM_LIST_REGIONS_REQUEST:
	 {
	    reply = list_regions(&gmm_state, request);
	    break;
	 }
	 case MM_LIST_INACTIVE_REGIONS_REQUEST: /* Deprecated */
	 {
	    reply = list_inactive_regions(request);
	    break;
	 }
	 case MM_GET_REGION_INFO_REQUEST:
	 {
	    reply = get_region_info(&gmm_state, request);
	    break;
	 }
	 case MM_TRANSLATE_ADDRESS_REQUEST:
	 {
	    reply = translate_address(&gmm_state, request);
	    break;
	 }
	 case MM_LIST_PAGE_SIZES_REQUEST:
	 {
	    reply = list_page_sizes(&gmm_state, request);
	    break;
	 }
	 case MM_GET_MMU_STATISTICS_REQUEST:
	 {
	    reply = get_mmu_statistics(&gmm_state, request);
	    break;
	 }
	 case MM_GET_DOMAIN_INFO_REQUEST:
	 {
	    reply = get_domain_info(&gmm_state, request);
	    break;
	 }
	 case MM_GET_PHYS_MEM_INFO_REQUEST:
	 {
	    reply = mm_get_phys_mem_info(&gmm_state, request);
	    break;
	 }
	 case MM_GET_PHYS_MEM_TYPE_INFO_REQUEST:
	 {
	    reply = get_phys_mem_type_info(request);
	    break;
	 }
	 case MM_GET_LOG_MEM_INFO_REQUEST:
	 {
	    reply = get_log_mem_info(request);
	    break;
	 }
	 case MM_GET_LOG_MEM_BANK_INFO_REQUEST:
	 {
	    reply = get_log_mem_bank_info(request);
	    break;
	 }
	 case MM_LIST_LOG_MEM_REQUEST:
	 {
	    reply = list_log_mem(request);
	    break;
	 }
	 case MM_LIST_PHYS_MEM_REQUEST:
	 {
	    reply = list_phys_mem(request);
	    break;
	 }
	 case MM_INTERFACE_INFO_REQUEST:
	 {
	    reply = interface_info(request);
	    break;
	 }

	 case MM_CHANGE_PERMISSIONS_REQUEST:
	 {
	    reply = change_permissions(&gmm_state, request);
	    break;
	 }

#ifdef OSPP         
         case MM_MAKE_RESIDENT_REQUEST:
         {
            reply = make_resident(&gmm_state, request);
            break;
         }

         case MM_MAKE_UNRESIDENT_REQUEST:
         {
            reply = make_unresident(&gmm_state, request);
            break;
         }
#endif /* OSPP */

         case MM_FORCED_WRITE_REQUEST:
         {
            OSADDRESS supervisor;

            supervisor = zzkrn_get_pid_info(sender(&request), 10);
            if (!supervisor)
            {
               request->mm_forced_write.sig_no = MM_FORCED_WRITE_REPLY;
               request->mm_forced_write.status = MM_EILLEGAL_OPERATION;
               break;
            }
            
            reply = mm_access_forced_write_runmode(request);
            break;
         }
         case MM_MULTITHREAD_REQUEST:
         {
            reply = mm_multithread(&gmm_state, request->mm_multithread.enable);
            break;
         }

         case MM_ADD_PHYS_RESOURCE_REQUEST:
         {
            reply = mm_add_phys_resource(request);
            break;
         }

         case MM_REMOVE_PHYS_RESOURCE_REQUEST:
         {
            reply = mm_remove_phys_resource(request);
            break;
         }

         case MM_ALLOC_PHYS_RESOURCE_REQUEST:
         {
            reply = mm_alloc_phys_resource(&gmm_state, request);
            break;
         }

         case MM_FREE_PHYS_RESOURCE_REQUEST:
         {
            reply = mm_free_phys_resource(&gmm_state, request);
            break;
         }

         case MM_ALLOC_METADATA_REQUEST:
         {
            reply = mm_alloc_metadata_region(request);
            break;
         }

         case MM_FREE_METADATA_REQUEST:
         {
            reply = mm_free_metadata_region(request);
            break;
         }
         
         case MM_GET_MAX_DOMAINS_REQUEST:
         {
            PRINT_SIGNAL(("gmm: received signal: %ld client_pid: %#lx\n",
                           request->sig_no, client));
            reply = request;
            reply->sig_no = MM_GET_MAX_DOMAINS_REPLY;
            reply->mm_get_max_domains.status = MM_SUCCESS;            
            reply->mm_get_max_domains.max_domains = gmm_state.max_domains;
            
            break;
         }
         
#if defined(OSE_SMP) && defined(OSPP)
         case OS_ATTACH_SIG:
         {
            cpuid_t cpu_id;
            PROCESS sender_pid = sender(&request);
            
            for (cpu_id = 1; cpu_id < ose_num_cpus(); cpu_id++)
            {
               if (smp_agents.pid[cpu_id-1] == sender_pid)
               {
                  ramlog_printf("MM agent on CPU %lu terminated.\n", cpu_id);
               }
            }
            error2(OSE_MM_EUNKNOWN_SIGNAL, (OSERRCODE) request);
            free_buf(&request);
            break;
         }
#endif
         
	 /*
	  * Unknown signals cause non-fatal errors.
	  * If the error handler returns, the offending signal is freed.
	  */
	 default:
	 {
	    error2(OSE_MM_EUNKNOWN_SIGNAL, (OSERRCODE) request);
            free_buf(&request);
	    break;
	 }
      }
      
      /* MM internal data is again accessable in freeze mode */
      gmm_state.data_accessable = TRUE;   

      /* Don't send replies to unknown signals. */
      if (NULL != reply)
      {
         send(&reply, client);
      }
   }
}
