/* COPYRIGHT-ENEA-SRC-R2 *
 **************************************************************************
 * Copyright (C) 2009 - 2010 by Enea Software AB.
 * All rights reserved.
 *
 * This Software is furnished under a software license agreement and
 * may be used only in accordance with the terms of such agreement.
 * Any other use or reproduction is prohibited. No title to and
 * ownership of the Software is hereby transferred.
 *
 * PROPRIETARY NOTICE
 * This Software consists of confidential information.
 * Trade secret law and copyright law protect this Software.
 * The above notice of copyright on this Software does not indicate
 * any actual or intended publication of such Software.
 **************************************************************************
 * COPYRIGHT-END */

/**
 *  @File prof.c
 *  @brief This is the implementation of the run time load profiling for now included in the rmm.c
 *
 */

#include "prof_timestamp.h"
#include "prof.h"
#include "atomic.h"
#include "pose_internal.h"
#include "ramlog.h"
#include "rmm_utility.h"

#define TOUCH(foo) foo=foo

/* NLOG is no log */
#define NLOG(msg)

#if 0
#define LOG(msg) \
    {\
        ramlog_printf msg;\
    }
#else
#define LOG(msg)
#endif


#define IS_CPU_ACTIVE(X) ((zzprof.prof_core_data_entry SMP_INDEX_X((X))).core_active == TRUE)


/* locking macros for the summations*/
/* pri*/
#define LOCK_ALL_PRI {\
   cpuid_t cpu; \
   for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++) \
   {\
	   if ( IS_CPU_ACTIVE(cpu) ) \
	   {\
         SPIN_LOCK(&(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_pri_data.prof_pri_core_data_lock);\
       }\
   }\
}


#define UNLOCK_ALL_PRI { \
   cpuid_t cpu; \
   for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++) \
   {\
	   if (IS_CPU_ACTIVE(cpu)) \
	   {\
          SPIN_UNLOCK(&(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_pri_data.prof_pri_core_data_lock); \
	   }\
   }\
}

/*cpu*/

#define LOCK_ALL_CPU {\
   cpuid_t cpu; \
   for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++) \
   {\
	   if ( IS_CPU_ACTIVE(cpu)) \
	   {\
		   SPIN_LOCK(&(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_cpu_data.prof_cpu_core_data_lock);\
	   }\
   }\
}


#define UNLOCK_ALL_CPU {\
   cpuid_t cpu;\
   for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)\
   {\
	   if (IS_CPU_ACTIVE(cpu)) \
	   {\
		   SPIN_UNLOCK(&(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_cpu_data.prof_cpu_core_data_lock);\
	   }\
   }\
}

/*proc*/

#define LOCK_ALL_PROC(block) {\
   cpuid_t cpu; \
   for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++) \
   {\
	   if (IS_CPU_ACTIVE(cpu)) \
	   {\
		   SPIN_LOCK(&(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_proc_data[ (block) ].prof_proc_core_data_lock);\
	   }\
   }\
}


#define UNLOCK_ALL_PROC(block) {\
   cpuid_t cpu;\
   for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)\
   {\
	   if ( IS_CPU_ACTIVE(cpu))\
	   {\
		   SPIN_UNLOCK(&(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_proc_data[(block)].prof_proc_core_data_lock);\
	   }\
   }\
}


/* THE global variable */
struct prof_world zzprof;

/* internal signal defs */
/* BLOCK profiling */
#define PROF_TCB_CLOSE_SIG (2)
#define PROF_TCB_START_SIG (3)
#define PROF_TCB_STOP_SIG (4)
#define PROF_TCB_MERGE_SIG (5)

/* CPU / CORE profiling */
#define PROF_CPU_CLOSE_SIG (6)
#define PROF_CPU_START_SIG (7)
#define PROF_CPU_STOP_SIG (8)

/* PRI / CORE profiling */
#define PROF_PRI_CLOSE_SIG (9)
#define PROF_PRI_START_SIG (10)
#define PROF_PRI_STOP_SIG (11)

/* BLOCK / CORE profiling */
#define PROF_PROC_CLOSE_SIG (12)
#define PROF_PROC_START_SIG (13)
#define PROF_PROC_STOP_SIG (14)

/* BLOCK / CORE profiling */
#define PROF_BLOCK_CLOSE_SIG (15)
#define PROF_BLOCK_START_SIG (16)
#define PROF_BLOCK_STOP_SIG (17)

/* signal to kill core */
#define PROF_CORE_KILL_SIG (18)
#define PROF_CORE_ATTACH_SIG (19)

/* maybe we win something when including these */
#include "prof_timestamp.c"
#include "prof_hash.c"

#ifdef NEED_TO_PROTOTYPE_RMM_C_FUNCTIONS
/* These functions left in rmm.c *
 * as long as prof.c is included in rmm.c there is no reason to duplicate these
 * they cannot be moved as the userreports are left in the rmm.c
 */
static OSBOOLEAN get_max_sigsize( uint32_t req_size );

static OSBOOLEAN lset_sigsize(union SIGNAL **sig, OSBUFSIZE newsize);
static Offspring  *find_offspring_by_pid(const PROCESS pid);


/* non static from rmm used also in prof_hash.c */
union SIGNAL *alloc_sample_pool(uint32_t size);
void free_sample_pool(union SIGNAL ** pool);
#endif

/****************************************************************************************************
 * All slave dbghooks
 */


static
#ifdef __GNUCC__
inline
#endif
void
prof_tcb_dbghook(OSPRIORITY pri,
         struct debug_pcb *proc,
         struct prof_tcb_core *const data)
{
   struct prof_timestamp e_timestamp;

   if (data->measure == TRUE)
   {
      uint32_t time_spent;

      if (pcb_to_bid(proc) == data->running_bid &&  (pri < data->pri_threshold || pri == 34))
      {
         NLOG ( ("Same measurement keep, bid %u returning\n ", data->running_bid) );
         return;
      }

   /* close sample if ongoing measurement */
      prof_timestamp_set(&e_timestamp);
      time_spent = prof_timestamp_get_diff(data->timestamp, e_timestamp);
      data->timestamp = e_timestamp;

#ifdef SAVING_DATA_FOR_MERGE
      if (proc->pid == data->core_rmm_pid)
      {
         data->last_add_bid = data->running_bid;
         data->last_add_val = time_spent;
         return;
      }
#endif

#ifdef OSE_SMP
      if ( atomic_cas((atomic_t *)data->running_valp, 0, (uint32_t)time_spent) == 0)
      {
#else
      if (*data->running_valp == 0)
      {
         *data->running_valp = time_spent;
#endif
         /* ... and remember the block */
         if (data->next < data->max_alloc)
         {
            NLOG ( ("Adding bid %u to cpu %u has value %u, time_spent = %u\n ", data->running_bid, zzkrn_cpu_id(), *(data->running_valp) , time_spent ) );
            data->bids[data->next] = data->running_bid;
            data->bcb_profs[data->next] = data->running_valp;
            data->next++;
         }
         else
         {
            /* rare case reset happens if we cannot handle more blocks */
            *data->running_valp = 0;
            data->other += time_spent;
         }
      }
      else
      {
       NLOG ( ("bid %u adds %u to %u \n ", data->running_bid,  time_spent, *(data->running_valp)) );
         /* normally we add the time spent to the value */
   #ifdef OSE_SMP
         atomic_add(time_spent, (atomic_t *)data->running_valp);
   #else
         *data->running_valp += time_spent;
   #endif
        NLOG(("bid %u after add= %u \n ", data->running_bid, *(data->running_valp)));
      }
   }

   /* prepare next measurement */
   if (pri < data->pri_threshold || pri == 34)
   {
      struct _block *b = (struct _block *)proc->block;
      data->running_valp = (volatile uint32_t *)&(b->prof);
      data->running_bid = pcb_to_bid(proc);
      NLOG( ("bid %u has value %d before add\n ", data->running_bid, *(data->running_valp) ) );
      if (data->measure == FALSE)
      {
         prof_timestamp_set(&data->timestamp);
      }
      data->measure = TRUE;
   }
   else
   {
      data->measure = FALSE;
   }

}


static
#ifdef __GNUCC__
inline
#endif
void
prof_pri_dbghook(OSPRIORITY pri, struct prof_pri_core *data)
{
   struct prof_timestamp e_timestamp;
   struct prof_timestamp b_timestamp;

   /* See if we swapped to a new priority. */
   if ( data->prof_pri_oldpri != pri)
   {
      prof_timestamp_set(&e_timestamp);
      b_timestamp = data->prof_pri_b_timestamp;

      /* if the interval has been closed since we started we add time since the start of the interval */
      data->prof_pri_sample.samples[data->prof_pri_oldpri]
         += prof_timestamp_get_diff(b_timestamp, e_timestamp);
      /* Update the start of measurement time stamp. */
      data->prof_pri_b_timestamp = e_timestamp;
      /* Update the current priority. */
      data->prof_pri_oldpri = pri;
   }
}

static
#ifdef __GNUCC__
inline
#endif
void
prof_cpu_dbghook(OSPRIORITY pri, struct prof_cpu_core *data)
{
   OSPRIORITY      cpu_pri;
   struct prof_timestamp e_timestamp;

   if ((pri == 34) || (pri < data->prof_cpu_core_pri))
   {
      cpu_pri = 1;
   }
   else
   {
      cpu_pri = 0;
   }

   if (data->prof_cpu_oldpri != cpu_pri)
   {
      prof_timestamp_set(&e_timestamp);

      if (cpu_pri)
      {
         /* We went from idle/low priority to high priority
          * so initiate a new measurement. */
         data->prof_cpu_b_timestamp = e_timestamp;
      }
      else
      {
         /* We went from high priority to idle/low priority
          * so add time spent. */
         data->prof_cpu_sample.samples
            += prof_timestamp_get_diff(data->prof_cpu_b_timestamp,e_timestamp);
      }

      /* Update the current priority. */
      data->prof_cpu_oldpri = cpu_pri;
   }
}

static
#ifdef __GNUCC__
inline
#endif
void
prof_proc_dbghook(OSPRIORITY pri, struct debug_pcb *proc, struct prof_proc_core *data)
{
   struct prof_timestamp e_timestamp;
   struct prof_timestamp b_timestamp;
   PROCESS pid = 0;
   uint32_t pindex = 0;

   prof_timestamp_set(&e_timestamp);
   b_timestamp = data->prof_proc_b_timestamp;

  /* LOG(("proc_dbghook: entering le hook debug pri=%u prof_proc_core_pri=%u\n",pri,data->prof_proc_core_pri)); */
   /* See if we swapped out a process with an active measurement period. */
   if (data->prof_proc_oldpid)
   {
      /* Add execution time to the current sample. */
      data->prof_proc_sample.samples[data->prof_proc_oldpindex]
                              += prof_timestamp_get_diff(b_timestamp, e_timestamp);

      NLOG(("Ending measurement for pid=%x pindex=%u report pid=%x\n",
            data->prof_proc_oldpid,
            data->prof_proc_oldpindex,
            data->prof_proc_sample.pids[data->prof_proc_oldpindex]));
      /* Note that we don't have an active measurement. */
      data->prof_proc_oldpid = 0;
   }

   /* We are done here if we are running a process that is less prioritized than the threshold */
   if ((pri < data->prof_proc_core_pri) || (pri == 34))
   {
      struct prof_hash *hash;

      pid = (!data->block ? proc->pid : pcb_to_bid(proc));

      if (!data->block && (proc->debug_flags & data->prof_proc_core_sys_process))
      {
       NLOG(("proc_dbghook:Process %x was a system process\n", pid));
         /* Use the "other" slot for system processes. */
         pindex = data->prof_proc_core_maxpid;
      }
      else
      {
         /* not a system process */

         /* add (if needed) to the active hash table */
         hash = SMP_CODE_YES_NO( (zzprof.proc[data->block].prof_proc_common_hash == TRUE ? &zzprof.proc[data->block].global_hash: &data->local_hash ), &data->local_hash);
         NLOG(("proc_dbghook:Process %x will look for hash\n", pid));
         if (!prof_hash_get_pid(pid, &pindex, hash))
         {
            NLOG(("proc_dbghook:Process %x did not have a hash value\n", pid));
            /* prof_proc_core_sys_process shall be copied from rw.sys_process */

#ifdef OSE_SMP
            if (zzprof.proc[data->block].prof_proc_common_hash == TRUE)
            {
               /* if all locked LOCK_SAVE(msr); */
               SPIN_LOCK(&hash->hash_lock);
               prof_hash_add_pid(pid, &pindex, hash); /* global must be protected */
               NLOG(("Starting new measurement for pid=%x in pindex=%u \n", pid , pindex));
               SPIN_UNLOCK(&hash->hash_lock);
               /* if all locked LOCK_RESTORE(msr); */
            }
            else
#endif
            {
               prof_hash_add_pid(pid, &pindex, hash);
            }
            data->prof_proc_sample.pids[pindex] = pid;
            /* TODO: Should not be needed: */
            //data->prof_proc_sample.samples[pindex] = 0;
         }
#ifdef OSE_SMP
         else if (0 == data->prof_proc_sample.pids[pindex])
         {
            data->prof_proc_sample.pids[pindex] = pid;
         }
#endif
      }
      NLOG(("proc_dbghook:Process %x at pindex=%u\n", pid, pindex));
      /* Note that we have an active measurement. */
      data->prof_proc_oldpindex = pindex;
      data->prof_proc_oldpid = pid;
      /* Update the start of interval time stamp. */
      data->prof_proc_b_timestamp = e_timestamp;
   }
}

/*
** This function is called from the kernel whenever a process
** is swapped in and the profiler is active.
**
** The pri parameter contains 0-31 for prioritized processes,
** 32 for background processes, 33 for the idle process and 34
** for any interrupt or timer process.
**
** The dispatcher may be locked, so no system calls except
** get_systime() may be used.
**
** Interrupts are enabled to the current process level.
*/

void
zzprof_dbghook(OSPRIORITY pri, struct debug_pcb *proc)
{
   LOCKSTATE msr;
   SMP_CODE(cpuid_t cpu = zzkrn_cpu_id();)
   struct prof_core_data * const core_data =
         &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu));

   if (!core_data->core_active)
   {
      return;
   }

   LOCK_SAVE(msr);
   if (core_data->prof_tcb_data.core_enabled == PROF_STATUS_ENABLED /* zzprof.tcb.enabled*/)
   { /* This is safe for swap? guess so because we are called from the dispatcher */
      prof_tcb_dbghook(pri, proc, &core_data->prof_tcb_data);
   }

   if (core_data->prof_pri_data.prof_pri_core_enabled == PROF_STATUS_ENABLED /* zzprof.pri.prof_pri_enabled */)
   { /* This is safe for swap? guess so because we are called from the dispatcher */
      prof_pri_dbghook(pri, &core_data->prof_pri_data);
   }

   if (core_data->prof_cpu_data.prof_cpu_core_enabled == PROF_STATUS_ENABLED /* zzprof.cpu.prof_cpu_enabled */
   && !core_data->prof_cpu_data.prof_cpu_core_issampled /* !zzprof.cpu.prof_cpu_issampled*/)
   { /* This is safe for swap? guess so because we are called from the dispatcher */
      prof_cpu_dbghook(pri, &core_data->prof_cpu_data);
   }

   if (core_data->prof_proc_data[0].prof_proc_core_enabled
         == PROF_STATUS_ENABLED /* zzprof.proc[0].prof_proc_enabled */)
   { /* This is safe for swap? guess so because we are called from the dispatcher */
      prof_proc_dbghook(pri, proc, &(core_data->prof_proc_data[0]));
   }

   if (core_data->prof_proc_data[1].prof_proc_core_enabled
         == PROF_STATUS_ENABLED /*zzprof.proc[1].prof_proc_enabled*/)
   { /* This is safe for swap? guess so because we are called from the dispatcher */
      prof_proc_dbghook(pri, proc, &(core_data->prof_proc_data[1]));
   }
   LOCK_RESTORE(msr);
}




/****************************************************************************************
 *  Disable functions
 */

/*
 * Disable profiling
 * called locked at core 0
 */

static void
prof_tcb_disable(LOCKSTATE *msr)
{
   union SIGNAL *disable;
   SMP_CODE( cpuid_t cpu;  );


   NLOG(("TCB enabled is set to false"));
   zzprof.tcb.enabled = FALSE;
   zzprof.tcb.firstsample->valid = 0;
   zzprof.tcb.disctimer = 0;
   NLOG(("disctimer is 0"));
/*
 * we have not removed the data so don't
    zzprof.tcb.reports = 0;
   zzprof.tcb.maxpid = 0;
*/
   SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++))
   {
	   SMP_CODE( if (IS_CPU_ACTIVE(cpu) ))
	   {
		   /* Get core specific data */
		   struct prof_tcb_core *const data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_tcb_data;

		   /* stop the sampling */
		   data->core_enabled = PROF_STATUS_STOPPING;
	   }
   }

   update_int_proc_hook();

   LOCK_RESTORE(*msr);
   disable = alloc(sizeof(SIGSELECT), PROF_TCB_STOP_SIG);
   send(&disable, _zzrmm_merge_pid);
   LOCK_SAVE(*msr);

/*
   free_buf ((union SIGNAL **)&data->bcb_profs);
   free_buf ((union SIGNAL **)&data->slots);
   free_buf ((union SIGNAL **)&data->bids);
   free_buf ((union SIGNAL **)&data->sample);
   */
}

/* called locked at core 0 */

static void
prof_pri_disable(void)
{
   SMP_CODE(cpuid_t cpu;)

   zzprof.pri.prof_pri_disctimer = 0;
   /* Abort the current sample. */
   zzprof.pri.prof_pri_enabled = 0;

   SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++))
   {
	   SMP_CODE( if (IS_CPU_ACTIVE(cpu)) )
	   {
		   struct prof_pri_core *data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_pri_data;
		   SMP_CODE(SPIN_LOCK(&data->prof_pri_core_data_lock);)
		   data->prof_pri_firstsample->valid = 0;
		   data->prof_pri_core_enabled = PROF_STATUS_STOPPING;
		   SMP_CODE(SPIN_UNLOCK(&data->prof_pri_core_data_lock);)
	   }
   }
   update_int_proc_hook();

   /* signal to all cores that the measurement is done */
   SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++))
   {
	   SMP_CODE(if (IS_CPU_ACTIVE(cpu)))
	   {
		   union SIGNAL *stopsig;

		   stopsig = alloc(sizeof(SIGSELECT), PROF_PRI_STOP_SIG);
		   send(&stopsig,   _zzrmm_core_pids SMP_INDEX_X(cpu) );
	   }
   }
}

/*
 * Disable cpu profiling
 * called locked at core 0
 */

static void
prof_cpu_disable(void)
{
   SMP_CODE(cpuid_t cpu;)

   /* Abort the current sample. */
    zzprof.cpu.prof_cpu_enabled = FALSE;

    SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++))
    {
 	   SMP_CODE(if (IS_CPU_ACTIVE(cpu)) )
 	   {
 		   struct prof_cpu_core *data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_cpu_data;
 		   SMP_CODE (SPIN_LOCK(&data->prof_cpu_core_data_lock);)
 		   data->prof_cpu_firstsample->valid = 0;
 		   data->prof_cpu_core_enabled = PROF_STATUS_STOPPING;
 		   SMP_CODE (SPIN_UNLOCK(&data->prof_cpu_core_data_lock);)
 	   }
    }

    /* Disable the kernel and target interface swap hooks. */
    update_int_proc_hook();

    SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++))
    {
    	 SMP_CODE( if (IS_CPU_ACTIVE(cpu)) )
    	 {
    		 union SIGNAL *stopsig;

    		 stopsig = alloc(sizeof(SIGSELECT), PROF_CPU_STOP_SIG);
    		 send(&stopsig,   _zzrmm_core_pids SMP_INDEX_X(cpu) );
    	 }
    }


}

/*
 * Disable proc/block profiling
 * called LOCKED at core 0
 */

static void
prof_proc_disable(int block)
{
   SMP_CODE(cpuid_t cpu;)

   /* Abort the current sample. */
    zzprof.proc[block].prof_proc_enabled = FALSE;

    SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++))
    {
    	 SMP_CODE( if (IS_CPU_ACTIVE(cpu)) )
    	 {
    		 union SIGNAL *stopsig;
    		 struct prof_proc_core *data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_proc_data[block];
    		 data->prof_proc_firstsample->valid = 0;
    		 data->prof_proc_core_enabled = PROF_STATUS_STOPPING;

    		 stopsig = alloc(sizeof(SIGSELECT), (block == 1 ?PROF_BLOCK_STOP_SIG: PROF_PROC_STOP_SIG));
    		 send(&stopsig,   _zzrmm_core_pids SMP_INDEX_X(cpu) );
    	 }
    }

    /* Disable the kernel and target interface swap hooks. */
    update_int_proc_hook();
}
/* EMPTY REQ funcs *******************************************************************
 *
 */

/*
** Empty pending request queues to offsprings by sending disabled response
*/
static void
prof_empty_pending_pri_req(OSBOOLEAN send_req)
{
  Offspring         *c;
  union SIGNAL   *rsig, *sig;
  struct prof_pri *data = &zzprof.pri;


  /* Return any pending cpu requests to clients if there are any */
  while (data->prof_pri_reqlist)
  {
	  /* cpu must be < num_cpus to be here i.e. safe */
	  uint32_t cpu = rsig->get_cpu_pri_reports_request.reserved0;

     rsig = data->prof_pri_reqlist;
     data->prof_pri_reqlist = (union SIGNAL *) rsig->sig_no;    /* Next pointer. */
     if (send_req == TRUE)
	 {
    	 c = find_offspring_by_pid(sender(&rsig));
    	 if (c)
    	 {
    		 sig = alloc(sizeof (MonitorGetCPUPriReportsReply), MONITOR_GET_CPU_PRI_REPORTS_REPLY);
    		 sig->get_cpu_pri_reports_reply.status = 
    		    ( (cpu == MONITOR_ALL_EXECUTION_UNITS || IS_CPU_ACTIVE(cpu)) ? 
    		      MONITOR_STATUS_OK : MONITOR_STATUS_CPU_INACTIVE); /* non OSE core */
    		 sig->get_cpu_pri_reports_reply.enabled = 0;
    		 sig->get_cpu_pri_reports_reply.reportsCount = 0;
    		 sig->get_cpu_pri_reports_reply.euId = cpu;
    		 sig->get_cpu_pri_reports_reply.reserved0 = 0;
    		 sig->get_cpu_pri_reports_reply.reserved1 = 0;

    		 send(&sig, c->pid);
    	 }
     }
     free_buf(&rsig);
  }
}


/*
** Empty pending request queues to clients by sending disabled response
*/
static void
prof_empty_pending_proc_req(MonitorBoolean block, OSBOOLEAN send_req)
{
   Offspring         *c;
   union SIGNAL   *rsig, *sig;
   struct prof_proc *data = &zzprof.proc[block];

  /* Return any pending cpu requests to clients if there are any */
  while (data->prof_proc_reqlist)
  {
	  /* cpu must be < num_cpus to be here i.e. safe */
	  uint32_t cpu = rsig->get_cpu_pri_reports_request.reserved0;

	  rsig = data->prof_proc_reqlist;
    data->prof_proc_reqlist = (union SIGNAL *) rsig->sig_no;    /* Next pointer. */

    if (send_req )
    {
       c = find_offspring_by_pid(sender(&rsig));
       if (c)
       {
          sig = alloc(sizeof (MonitorGetCPUProcessReportsReply),
                      block ? MONITOR_GET_CPU_BLOCK_REPORTS_REPLY
                      : MONITOR_GET_CPU_PROCESS_REPORTS_REPLY);
          sig->get_cpu_process_reports_reply.status = 
             ( (cpu == MONITOR_ALL_EXECUTION_UNITS || IS_CPU_ACTIVE(cpu)) ? 
                MONITOR_STATUS_OK : MONITOR_STATUS_CPU_INACTIVE);
          sig->get_cpu_process_reports_reply.enabled = 0;
          sig->get_cpu_process_reports_reply.euId = cpu;
          sig->get_cpu_process_reports_reply.reserved0 = 0;
          sig->get_cpu_process_reports_reply.reserved1 = 0;
          sig->get_cpu_process_reports_reply.reserved2 = 0;
          sig->get_cpu_process_reports_reply.reportsCount = 0;
          sig->get_cpu_process_reports_reply.processesCount = 0;
          send(&sig, c->pid);
       }
    }
    free_buf(&rsig);
  }
}

/*
** Empty pending request queues to clients by sending disabled response
*/
static void
prof_empty_pending_tcb_req(OSBOOLEAN send_req)
{
  Offspring         *c;
  union SIGNAL   *rsig, *sig;
  struct prof_tcb *data = &zzprof.tcb;

  /* Return any pending proc requests to clients if there are any */
  while (data->prof_tcb_reqlist)
  {

    rsig = data->prof_tcb_reqlist;
    data->prof_tcb_reqlist = (union SIGNAL *) rsig->sig_no;    /* Next pointer. */

    if (send_req == TRUE)
    {
       c = find_offspring_by_pid(sender(&rsig));
       if (c)
       {
          sig = alloc(sizeof (MonitorGetCPUProcessReportsReply),
                      MONITOR_GET_TOTAL_CPU_BLOCK_REPORTS_REPLY);
          sig->get_cpu_process_reports_reply.status = MONITOR_STATUS_OK;
          sig->get_cpu_process_reports_reply.enabled = 0;
          sig->get_cpu_process_reports_reply.euId = 0;
          sig->get_cpu_process_reports_reply.reserved0 = 0;
          sig->get_cpu_process_reports_reply.reserved1 = 0;
          sig->get_cpu_process_reports_reply.reserved2 = 0;
          sig->get_cpu_process_reports_reply.reportsCount = 0;
          sig->get_cpu_process_reports_reply.processesCount = 0;
          send(&sig, c->pid);
       }
    }
    free_buf(&rsig);
  }
}


/*
** Empty pending request queues to clients by sending disabled response
*/
static void
prof_empty_pending_cpu_req(OSBOOLEAN send_req)
{
  Offspring         *c;
  union SIGNAL   *rsig, *sig;
  uint32_t cpu;

  struct prof_cpu *data = &zzprof.cpu;

  while (data->prof_cpu_reqlist)
  {
     rsig = data->prof_cpu_reqlist;
     cpu = rsig->get_cpu_reports_request.reserved0;
     data->prof_cpu_reqlist = (union SIGNAL *) rsig->sig_no;    /* Next pointer. */

     if (send_req == TRUE)
     {
        c = find_offspring_by_pid(sender(&rsig));
        if (c)
        {
           sig = alloc(sizeof (MonitorGetCPUReportsReply), MONITOR_GET_CPU_REPORTS_REPLY);
           sig->get_cpu_reports_reply.status =
              ( (cpu == MONITOR_ALL_EXECUTION_UNITS || IS_CPU_ACTIVE(cpu)) ?
                    MONITOR_STATUS_OK : MONITOR_STATUS_CPU_INACTIVE);
           sig->get_cpu_reports_reply.enabled = 0;
           sig->get_cpu_reports_reply.euId = cpu;
           sig->get_cpu_reports_reply.reserved0 = 0;
           sig->get_cpu_reports_reply.reserved1 = 0;
           sig->get_cpu_reports_reply.reportsCount = 0;
           send(&sig, c->pid);
        }
     }
    free_buf(&rsig);
  }
}


/********************************************************************************************************
 *  INTERVAL CLOSING
 **/

static void
prof_pri_close_interval(struct prof_pri_core *data)
{
   LOCKSTATE msr;
   uint32_t i;
   struct prof_timestamp e_timestamp;
   struct prof_timestamp b_timestamp;
   OSTICK ticks;
   OSTICK uticks;
   struct prof_pri_sample *old, *new, *pri_sample;


   LOCK_SAVE(msr);
   SMP_CODE( SPIN_LOCK(&data->prof_pri_core_data_lock); )

   prof_timestamp_set(&e_timestamp);
   prof_timestamp_set_interval_start_ticks(&ticks,&uticks,e_timestamp);

   b_timestamp = data->prof_pri_b_timestamp;
   old = data->prof_pri_cursample;
   new = data->prof_pri_cursample->next;

   if (new == data->prof_pri_firstsample)
               data->prof_pri_firstsample = data->prof_pri_firstsample->next;
   new->valid = 0;
   /* copy samples */
   pri_sample = &data->prof_pri_sample;
   for (i = 0; i < 35; ++i)
   {
      old->samples[i] = pri_sample->samples[i];
      pri_sample->samples[i] = 0;
   }

   /* Add the execution time to the current sample. */
   old->samples[data->prof_pri_oldpri] +=
           prof_timestamp_get_diff(b_timestamp, e_timestamp);
   new->time_ticks = ticks;
   new->time_uticks = uticks;

   old->total_time = prof_timestamp_get_diff(data->prof_pri_interval_start, e_timestamp);
   /* Switch from the old to the new sample. */
   old->valid = 1;
   data->prof_pri_cursample = data->prof_pri_cursample->next;
   data->prof_pri_b_timestamp = e_timestamp; /* not needed? */
   data->prof_pri_interval_start = e_timestamp;

   SMP_CODE( SPIN_UNLOCK(&data->prof_pri_core_data_lock); )

   SMP_CODE(if (0 == zzkrn_cpu_id()))
   {
      if (zzprof.pri.prof_pri_disctimer <= zzprof.pri.prof_pri_interval)
      {
         zzprof.pri.prof_pri_disctimer = 0;
         prof_pri_disable();
         /* we will not loose interrupt lock */
         prof_empty_pending_pri_req(FALSE);
      }
      else
      {
         zzprof.pri.prof_pri_disctimer -= zzprof.pri.prof_pri_interval;

         /* If a request is pending, send a notification to the RMM. */
         if (zzprof.pri.prof_pri_reqlist)
         {
            union SIGNAL   *sig;

            LOCK_RESTORE(msr);
            sig = alloc(sizeof (ProfNewSample), PROF_NEW_SAMPLE);
            sig->prof_new_sample.type = PRIORITY_PROF;
            send(&sig, rw.zzose_rmm_);
            LOCK_SAVE(msr);
         }
      }
   }
   LOCK_RESTORE(msr);
}


static void
prof_sampled_cpu_close_interval(void)
{
   SMP_CODE(cpuid_t cpu;)
   LOCKSTATE msr;
   OSTICK ticks;
   OSTICK uticks;
   struct prof_cpu_sample *old, *new;

   /* start new transaction */
   /* lock the monitor's profiling activities AND turn off interrupts */

   LOCK_SAVE(msr);

   /* Update the counter and see if interval expired. */
   /* Restart the counter. */
   /* do not call get_systime several times */
   ticks = get_systime(&uticks);

   /* For sampled we run everything from core 0. */
   SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++))
   {
	   SMP_CODE( if (IS_CPU_ACTIVE(cpu)))
	   {
		   struct prof_cpu_core *data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_cpu_data;
		  /* Close the old sample and allocate a fresh one. */
		  old = data->prof_cpu_cursample;
		  new = data->prof_cpu_cursample->next;
		  if (new == data->prof_cpu_firstsample)
		  data->prof_cpu_firstsample = data->prof_cpu_firstsample->next;
		  new->valid = 0;
		  new->total_time = 0;

		  /* Clear the values of the new sample. */
		  new->samples = 0;

		  new->time_ticks = ticks;
		  new->time_uticks = uticks;

		  /* Switch from the old to the new sample. */
		  old->valid = 1;
		  data->prof_cpu_cursample = new;
		  LOG(("SAMPLED interval closed, samples: %u, total_time: %u\n", old->samples, old->total_time));
	   }
   }

   if (zzprof.cpu.prof_cpu_disctimer <= zzprof.cpu.prof_cpu_interval)
   {
      zzprof.cpu.prof_cpu_disctimer = 0;
      prof_cpu_disable();
      LOCK_RESTORE(msr);
      prof_empty_pending_cpu_req(FALSE);
      LOCK_SAVE(msr);
   }
   else
   {
      zzprof.cpu.prof_cpu_disctimer -= zzprof.cpu.prof_cpu_interval;

      /* If a request is pending, send a notification to the RMM. */
      if (zzprof.cpu.prof_cpu_reqlist )
      {
	 union SIGNAL   *sig;

	 LOCK_RESTORE(msr);
	 sig = alloc(sizeof (ProfNewSample),
		     PROF_NEW_SAMPLE);
	 sig->prof_new_sample.type = CPU_OVERVIEW_PROF;
	 send(&sig, rw.zzose_rmm_);
	 LOCK_SAVE(msr);
      }
   }
   LOCK_RESTORE(msr);
}

static void
prof_cpu_close_interval(struct prof_cpu_core *data)
{
   struct prof_cpu_sample *old, *new;
   struct prof_timestamp e_timestamp;
   struct prof_timestamp b_timestamp;
   LOCKSTATE msr;

   LOCK_SAVE(msr);
   SMP_CODE( SPIN_LOCK(&data->prof_cpu_core_data_lock); )


   old = data->prof_cpu_cursample;
   new = data->prof_cpu_cursample->next;
   new->valid = 0;
   if (new == data->prof_cpu_firstsample)
                   data->prof_cpu_firstsample = data->prof_cpu_firstsample->next;

   prof_timestamp_set(&e_timestamp);
   prof_timestamp_set_interval_start_ticks(&new->time_ticks, &new->time_uticks, e_timestamp);

   b_timestamp = data->prof_cpu_b_timestamp;

   if (data->prof_cpu_oldpri)
   {
      /* Add the execution time to the current sample. */
      data->prof_cpu_sample.samples += prof_timestamp_get_diff(b_timestamp, e_timestamp);
   }

  /* finish the old sample (old's ticks/uticks are already set) */
  /********************************************************/
  /* copy the samples */
   old->samples = data->prof_cpu_sample.samples;

   /* old->total_time += old->samples[0] + old->samples[1]; */
   old->total_time = prof_timestamp_get_diff(data->prof_cpu_interval_start, e_timestamp);

   /* the total_time was checked and is exactly the same as the sum of the samples (tb_timer) */
   LOG(("total_time = %u, samples=%u", old->total_time, old->samples))

   /* prepare for the next sample */
   /*********************************************************/

   /* Update the start of interval time stamp. */
   data->prof_cpu_b_timestamp = e_timestamp;
   data->prof_cpu_interval_start = e_timestamp;

   /* prof_cpu_oldpri is kept; no update*/

   data->prof_cpu_sample.samples = 0;

   /* Switch from the old to the new sample. */
   old->valid = 1;
   data->prof_cpu_cursample = new;

   /* Reenable profiling */
   /* unlock the monitor's profiling activities AND turn off interrupts */

   SMP_CODE( SPIN_UNLOCK(&data->prof_cpu_core_data_lock); )


   SMP_CODE( if (0 == zzkrn_cpu_id()) )
   {
       if (zzprof.cpu.prof_cpu_disctimer <= zzprof.cpu.prof_cpu_interval)
       {
          zzprof.cpu.prof_cpu_disctimer = 0;
          prof_cpu_disable();
          LOCK_RESTORE(msr);
          prof_empty_pending_cpu_req(TRUE);
          LOCK_SAVE(msr);
       }
       else
       {
          zzprof.cpu.prof_cpu_disctimer -= zzprof.cpu.prof_cpu_interval;

         /* If a request is pending, send a notification to the LDM. */
         if (zzprof.cpu.prof_cpu_reqlist)
         {
            union SIGNAL   *sig;
            LOCK_RESTORE(msr);
            sig = alloc(sizeof (ProfNewSample), PROF_NEW_SAMPLE);
            sig->prof_new_sample.type = CPU_OVERVIEW_PROF;
            send(&sig, rw.zzose_rmm_);
            LOCK_SAVE(msr);
         }

       }
   }
   LOCK_RESTORE(msr);
}

#ifdef OSE_SMP
/* called by core 0 */
static void
wait_slaves(struct prof_proc *g, uint32_t block)
{
   cpuid_t cpu;
   for (cpu = 1; cpu < zzprof.prof_all_num_cpus ; cpu++)
   {
      if (IS_CPU_ACTIVE(cpu))
      {
    	  struct prof_core_data *const core_data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu));
		  while (core_data->prof_proc_data[block].hash_sync_counter ==  g->master_hash_sync_counter)
		  {
		  }
      }
   }
   return;
}

/* called by core != 0 */
static void
wait_master(struct prof_proc *g, struct prof_proc_core *data)
{
   while(g->master_hash_sync_counter != data->hash_sync_counter)
   { /* wait for update */

   }
   return;
}

#endif /* OSE_SMP */

/****************************************************************************************
 * Helper at interval starts that adds the profiled processes to the a hash table
 */

static void
prof_add_profiled_pids(struct prof_proc_core *data, struct prof_hash *hash)
{
   uint32_t i;
   SMP_CODE(SPIN_LOCK(&zzprof.proc[data->block].prof_proc_lock);)
   for (i = 0; i < data->prof_proc_core_maxpid; i++)
   {
      if( zzprof.proc[data->block].prof_proc_pids[i] != 0 )
      {
         uint32_t pindex;
         uint32_t pid = zzprof.proc[data->block].prof_proc_pids[i];
         prof_hash_add_pid( pid, &pindex, hash);
         data->prof_proc_sample.pids[pindex] = pid;
         data->prof_proc_sample.samples[pindex] = 0;
      }
   }
   SMP_CODE(SPIN_UNLOCK(&zzprof.proc[data->block].prof_proc_lock);)
}

static void
prof_proc_close_interval(struct prof_proc_core *data)
{
   LOCKSTATE msr;
   uint32_t i;
   struct prof_timestamp e_timestamp;
   struct prof_timestamp b_timestamp;
   struct prof_proc_sample *old, *new;
   struct prof_hash *hash;

   LOCK_SAVE(msr);
   SMP_CODE( SPIN_LOCK(&data->prof_proc_core_data_lock); )
   hash = SMP_CODE_YES_NO(( zzprof.proc[data->block].prof_proc_common_hash == TRUE ? &zzprof.proc[data->block].global_hash: &data->local_hash), &data->local_hash);
#ifdef OSE_SMP
   data->prof_proc_cursample->rp_count = zzprof.proc[data->block].rp_count;
#endif
   old = data->prof_proc_cursample;
   new = data->prof_proc_cursample->next;

   if (new == data->prof_proc_firstsample)
   {
      data->prof_proc_firstsample =
         data->prof_proc_firstsample->next;
   }
   new->valid = 0;
   b_timestamp = data->prof_proc_b_timestamp;
   prof_timestamp_set(&e_timestamp);
   prof_timestamp_set_interval_start_ticks(
                        (OSTICK *)&new->time_ticks,
                        (OSTICK *)&new->time_uticks,
                        e_timestamp);

   /* Normally we have some swaps during the interval  */
   /* See if we have a process with an active measurement period. */
   if (data->prof_proc_oldpid)
   {
      /* Add execution time to the current sample. */
      data->prof_proc_sample.samples[data->prof_proc_oldpindex] += prof_timestamp_get_diff(b_timestamp, e_timestamp);

   }

   NLOG(("we have %u pids\n", hash->index_cnt ));
   for (i = 0; i < data->prof_proc_core_maxpid + 1 ; i++)
   {
      old->pids[i] = data->prof_proc_sample.pids[i];
      data->prof_proc_sample.pids[i]= 0;
      old->samples[i] = data->prof_proc_sample.samples[i];
      data->prof_proc_sample.samples[i] = 0;
      NLOG(("[%u] pid[%d]=%x has  sample[%d]=%u\n", zzkrn_cpu_id(), i, old->pids[i], i, old->samples[i]));
   }

   old->total_time = prof_timestamp_get_diff(data->prof_proc_interval_start, e_timestamp);

#ifdef OSE_SMP
   if (zzprof.proc[data->block].prof_proc_common_hash == TRUE)
   {

      if (0 == zzkrn_cpu_id())
      {
         wait_slaves(&zzprof.proc[data->block], (uint32_t) data->block);
         SPIN_LOCK(&hash->hash_lock);
         NLOG(("rmm: prof: master_hash_sync_counter: %u\n",
               zzprof.proc[data->block].master_hash_sync_counter));
         old->num_procs = hash->index_cnt;
         prof_hash_reset(hash);
         prof_add_profiled_pids(data, hash);
         zzprof.proc[data->block].master_hash_sync_counter++;
         SPIN_UNLOCK(&hash->hash_lock);
      }
      else
      {
         SPIN_LOCK(&hash->hash_lock);
         NLOG(("rmm: prof: hash_sync_counter: %u\n",
               data->hash_sync_counter));
         old->num_procs = hash->index_cnt;
         SPIN_UNLOCK(&hash->hash_lock);

         data->hash_sync_counter++;
         /* let core 0 update profiled pids here */
         wait_master(&zzprof.proc[data->block], data);
      }
   }
   else
#endif
   {

	  old->num_procs = hash->index_cnt;
      prof_hash_reset(hash);
      prof_add_profiled_pids(data, hash);
   }

   /* Switch from the old to the new sample. */
   old->valid = 1;

   data->prof_proc_cursample = data->prof_proc_cursample->next;;
   data->prof_proc_b_timestamp = e_timestamp;
   data->prof_proc_interval_start = e_timestamp;

   /* Add the profiled processes to the new sample before we let
    * anything new be added to the sample */

   SMP_CODE( SPIN_UNLOCK(&data->prof_proc_core_data_lock); )

   SMP_CODE(if (0 == zzkrn_cpu_id()))
   {
      if (zzprof.proc[data->block].prof_proc_disctimer <= zzprof.proc[data->block].prof_proc_interval)
      {
         zzprof.proc[data->block].prof_proc_disctimer = 0;
         prof_proc_disable(data->block);
         LOCK_RESTORE(msr);
         prof_empty_pending_proc_req(data->block,TRUE);
         LOCK_SAVE(msr);
      }
      else
      {
         zzprof.proc[data->block].prof_proc_disctimer -= zzprof.proc[data->block].prof_proc_interval;

         /* If a request is pending, send a notification to the RMM. */
         if (zzprof.proc[data->block].prof_proc_reqlist)
         {
            union SIGNAL   *sig;

            LOCK_RESTORE(msr);
            sig= alloc(sizeof (ProfNewSample), PROF_NEW_SAMPLE);
            sig->prof_new_sample.type = data->block ? BLOCK_PROF : PROCESS_PROF;
            send(&sig, rw.zzose_rmm_);
            LOCK_SAVE(msr);
         }
      }
   }

   LOCK_RESTORE(msr);
}


/*
 * Qsort comparer for prof_tcb_close_interval and prof_send_cpu_proc_reports_sum
 *
 **/
static int
prof_tcb_compslot (const void *e1, const void *e2)
{
   const MonitorCPUProcess *t1 = (const MonitorCPUProcess*)e1;
   const MonitorCPUProcess *t2 = (const MonitorCPUProcess*)e2;
   return  (int)(((int64_t)t2->sum) - ((int64_t)t1->sum));
}


/* the end of interval
 * each core will call this
 * must have lock on this ... */

/* FOR NOW we expect that:
 * The interval_close have to be called after the last has been swapped out
 * How to achieve that?
 * nr_of_slots does not include other .....
 */
static void
prof_tcb_close_interval(struct prof_tcb_core *const data)
{
   uint32_t i, j, bids_in_use;
   U32 other;
   MonitorCPUProcessReport *sample = data->sample;
   struct prof_timestamp e_timestamp;
#ifdef SAVING_DATA_FOR_MERGE
   struct prof_timestamp b_timestamp;
#endif
   OSTICK ticks, uticks;
   LOCKSTATE       msr;


   LOCK_SAVE(msr);
   /* Take timestamp */
   prof_timestamp_set(&e_timestamp);
   prof_timestamp_set_interval_start_ticks(&ticks, &uticks, e_timestamp);
#ifdef SAVING_DATA_FOR_MERGE
   b_timestamp = data->timestamp;
#endif

   /* TRANSFER VALUES TO THE SLOTS */
   bids_in_use = data->next;
   other = data->other;

   for (i = 0, j = 0;i < bids_in_use; i++)
   {
      if (get_ptype(data->bids[i]) == OS_BLOCK)
      {
         /* references takes time  */
         uint32_t sum = (uint32_t) *(data->bcb_profs[i]);

#ifdef OSE_SMP
         while ( atomic_cas((atomic_t *)data->bcb_profs[i], sum, 0) != sum)
         {
            sum = (uint32_t) *data->bcb_profs[i];
         }
#else
         *(data->bcb_profs[i]) = 0;
#endif
         /* now we put it in the referenced variable */
         data->slots[j].sum = sum;
         data->slots[j].id = data->bids[i];
         NLOG(("before qsort  %x %u [%u]\n", data->slots[j].id, data->slots[j].sum, j));
         j++;
      }
   }
   /* Adjust to actually live block count */
   bids_in_use = j;

   data->next =0;
   data->other = 0;


   /* set start time for next interval */
   /* use our now timestamp to mark the end of last interval and start of the next */
   sample->interval = prof_timestamp_get_diff(data->interval_start, e_timestamp);
   sample->time.tick = data->interval_start_ticks;
   sample->time.ntick = data->interval_start_uticks;

   data->interval_start_ticks = ticks;
   data->interval_start_uticks = uticks;
   data->interval_start = e_timestamp;
   data->timestamp = e_timestamp;


#ifdef SAVING_DATA_FOR_MERGE
   /* close sample if ongoing measurement */
   if (data->measure == TRUE)
   {
      uint32_t time_spent = prof_timestamp_get_diff(b_timestamp, e_timestamp);
      /* this can be added to the OSE block in the merge */
      data->OSE_block_last_add = time_spent;
   }
#endif


   /* post processing starts now no lock needed only data->slots to be used*/

   NLOG( ("close interval bids_in_use=%u \n ", bids_in_use ) );

   qsort(data->slots, bids_in_use, sizeof(MonitorCPUProcess), prof_tcb_compslot );

   sample->processesCount = 0;
   sample->sumOther = 0;
   for (i = 0;i < bids_in_use; i++)
   {
      if (i < data->maxpid)
      {
         sample->processes[i].id = data->slots[i].id;
         sample->processes[i].sum = data->slots[i].sum;

         NLOG(("after qsort  %x %u [%u]\n", data->slots[i].id, data->slots[i].sum, i));
         NLOG( ("core[%u][%u] id=%u, sum=%u \n", zzkrn_cpu_id(), i, data->slots[i].id, data->slots[i].sum));

         data->slots[i].sum = 0;
         data->slots[i].id = 0;
         sample->processesCount++;
      }
      else {
         NLOG( ("in sumOther: [%u] id=%u, sum=%u \n", i, data->slots[i].id, data->slots[i].sum) );
         sample->sumOther+= data->slots[i].sum;
      }
   }

   sample->sumOther += (uint32_t)other;
   data->unsafe = (OSBOOLEAN) (other > 0 ? 1 : 0);

   /* to be sure that merge not will come before we are done hold the processor until here */
   LOCK_RESTORE(msr);
}


/*
 *  Helper to prof_tcb_merge_interval
 */

#ifdef OSE_SMP
static
#ifdef __GNUCC__
/* inline */
#endif
MonitorCPUProcess*
prof_tcb_choose(MonitorCPUProcessReport **sample_list , uint32_t *index )
{
   cpuid_t cpu;
   MonitorCPUProcess * result = NULL;
   cpuid_t chosen_cpu = 0 ;

   for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
   {
	   if ( IS_CPU_ACTIVE(cpu))
	   {
		   uint32_t cpu_index  = index[cpu];
		   MonitorCPUProcessReport *candidate = sample_list[cpu];

		   if ( cpu_index < candidate->processesCount )
		   {
			   if (result == NULL || result->sum < candidate->processes[cpu_index].sum)
			   {
				   result = &(candidate->processes[cpu_index]);
				   chosen_cpu = cpu;
			   }
		   }
	   }
   }

   /* update the index for the chosen one */
   if (result != NULL)
   {
      uint32_t *chosen_index  = &index[chosen_cpu];
      (*chosen_index)++;
   }

   return result;
}
#endif

/*
 * The closing of the closed intervals for the tcb profiling  Delivers to the samples
 */
/* read all reports from the cores after they have closed */
static void
prof_tcb_merge_interval(MonitorCPUProcessReport **sample_list, struct prof_tcb_sample *old)
{
   LOCKSTATE msr;
   uint32_t i;
   SMP_CODE(cpuid_t cpu;)
   MonitorCPUProcessReport *first_sample = *sample_list;
   uint32_t index SMP_ARRAY;
   MonitorCPUProcess *choice = NULL;

   NLOG( ("MERGE: entering\n") );
   LOCK_SAVE(msr);

   /* set the indexes to 0 */
   SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++))
   {
	   SMP_CODE( if (IS_CPU_ACTIVE(cpu)))
	   {
		   index SMP_INDEX_X(cpu) = 0;
	   }
   }

   old->time_ticks = first_sample->time.tick;
   old->time_uticks = first_sample->time.ntick;
   old->num_procs = 0;
   old->samples[zzprof.tcb.maxpid] = 0; /* pointer not allocated !!!*/
   old->total_time = first_sample->interval;
   choice = SMP_CODE_YES_NO(
               prof_tcb_choose(sample_list , &index[0]) , /* SMP */
               ((*sample_list)->processesCount > index ?  & (*sample_list)->processes[index++] : NULL) /* NO SMP */
            );
   for (i=0; choice != NULL && i< zzprof.tcb.maxpid ; i++)
   {
      NLOG( ("Add bid %x with value %u to index %u \n", choice->id, choice->sum, i) );
      old->pids[i] = choice->id;
      old->samples[i] = choice->sum;
      old->num_procs++;
      choice = SMP_CODE_YES_NO(
                  prof_tcb_choose(sample_list , &index[0]),
                  ((*sample_list)->processesCount > index ?  & (*sample_list)->processes[index++] : NULL)
               );
   }

   /* add the rest to other */
   while (choice != NULL )
   {
      NLOG( ("Add bid %u to extra \n", choice->id));
      old->samples[zzprof.tcb.maxpid] += choice->sum;
      choice = SMP_CODE_YES_NO(
                  prof_tcb_choose(sample_list , &index[0]),
                  ((*sample_list)->processesCount > index ?  & (*sample_list)->processes[index++] : NULL)
               );
   }
   /* add all other ....    */
 /*  LOG(("Enter loop sumOther to extraAdd\n", choice->id));
*/
   SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++) )
   {
	   SMP_CODE (if (IS_CPU_ACTIVE(cpu)))
	   {
		   MonitorCPUProcessReport *sample = *(sample_list SMP_CODE(+ cpu));
  /*    	LOG ( ("Adding all sumOther to the extra slot before %u added %u\n", old->samples[zzprof.tcb.maxpid], sample->sumOther)); */
		   old->samples[zzprof.tcb.maxpid] += sample->sumOther;
	   }
   }

 /* this is core 0 */
   if (zzprof.tcb.disctimer <= zzprof.tcb.interval)
   {
      NLOG(("TIMEOUT disctimer is now %u", zzprof.tcb.disctimer));
      prof_tcb_disable(&msr);
      prof_empty_pending_tcb_req(FALSE);
   }
   else
   {
      zzprof.tcb.disctimer -= zzprof.tcb.interval;
      NLOG(("disctimer is now %u", zzprof.tcb.disctimer));

      if (zzprof.tcb.prof_tcb_reqlist)
      {
         union SIGNAL   *sig;

         LOCK_RESTORE(msr);
         sig = alloc(sizeof (ProfNewSample), PROF_NEW_SAMPLE);
         sig->prof_new_sample.type = TOTAL_CPU_BLOCK_PROF;
         send(&sig, rw.zzose_rmm_);
         LOCK_SAVE(msr);
      }
   }

   LOCK_RESTORE(msr);
 /*  LOG (("MERGE: exit\n") ); */
}


/********************************************************************************************************
 * Prepare start functions
 */

static void
prof_prepare_proc_start(struct prof_proc_core *data)
{
        LOCKSTATE msr;
   struct prof_proc_sample *first_sample;
   struct prof_proc_sample *sample;
   uint32_t i;
   struct prof_hash *hash;

   LOCK_SAVE(msr);
   if (0 == zzkrn_cpu_id())
   {
      /* Start the disconnect timer. */
      zzprof.proc[data->block].prof_proc_disctimer = 100000000L / zzprof.prof_all_ticklen;    /* 100 sec. */
   }

   first_sample = data->prof_proc_firstsample
               = data->prof_proc_cursample
               = data->prof_proc_samplepool;

   first_sample->valid = 0;

   prof_timestamp_set(&data->prof_proc_b_timestamp);
   prof_timestamp_set_interval_start_ticks(&first_sample->time_ticks, &first_sample->time_uticks, data->prof_proc_b_timestamp);
   data->prof_proc_interval_start = data->prof_proc_b_timestamp;

   /* no measurement is going on */
   data->prof_proc_oldpid = 0;
   data->prof_proc_oldpindex = 0;

   /* reset the internal sample */
   sample = &data->prof_proc_sample;
   sample->valid = 0;
   sample->total_time = 0;
   for (i = 0; i < data->prof_proc_core_maxpid +1; ++i)
   {
           /* include the  "all other processes" at data->prof_proc_core_maxpid. That's why +1*/
      sample->pids[i] = 0; /* don't put the prof_proc_pids here but add then to the hash table */
      sample->samples[i] = 0;
   }

   hash = SMP_CODE_YES_NO( (zzprof.proc[data->block].prof_proc_common_hash == TRUE ? &zzprof.proc[data->block].global_hash: &data->local_hash ), &data->local_hash);

#ifdef OSE_SMP
   if (zzprof.proc[data->block].prof_proc_common_hash == TRUE)
   {
      if (0 == zzkrn_cpu_id())
      {
                  /* no need to wait */
          SPIN_LOCK(&hash->hash_lock);
          prof_hash_reset(hash);
          prof_add_profiled_pids(data, hash);
          zzprof.proc[data->block].master_hash_sync_counter++;
          SPIN_UNLOCK(&hash->hash_lock);
      }
      else
      {
          data->hash_sync_counter = 1;
           /* let core 0 update here */
           wait_master(&zzprof.proc[data->block], data);
      }
   }
   else
#endif
   {
           prof_hash_reset(hash);
           prof_add_profiled_pids(data, hash);
   }

   /* SET UP duplicates */
   /*  speed up by having local versions of these */
   data->prof_proc_core_pri = zzprof.proc[data->block].prof_proc_pri;
   NLOG(("prepare set prof_proc_core_pri to %u   "));

   assert( data->prof_proc_core_maxpid == zzprof.proc[data->block].prof_proc_maxpid);
   data->prof_proc_core_sys_process = rw.sys_process;
   data->prof_proc_core_interval = zzprof.proc[data->block].prof_proc_interval;


    /* let the fun begin */
    data->prof_proc_core_enabled = PROF_STATUS_ENABLED;
    LOCK_RESTORE(msr);
    NLOG(("prepare: prof_proc_core_enabled is set\n"));
}

static void
prof_prepare_cpu_start(struct prof_cpu_core *data)
{
   struct prof_cpu_sample *first_sample;
   struct prof_cpu_sample *sample;
   OSPRIORITY pri;

   if (0 == zzkrn_cpu_id())
   {
      /* Start the disconnect timer. */
      zzprof.cpu.prof_cpu_disctimer = 100000000L / zzprof.prof_all_ticklen;    /* 100 sec. */
   }

   first_sample = data->prof_cpu_firstsample
               = data->prof_cpu_cursample
               = data->prof_cpu_samplepool;

   first_sample->valid = 0;

   /* These inits needed for sampled measurement flavor */
   first_sample->samples = 0;
   first_sample->total_time = 0;

   prof_timestamp_set(&data->prof_cpu_b_timestamp);
   prof_timestamp_set_interval_start_ticks(&first_sample->time_ticks, &first_sample->time_uticks, data->prof_cpu_b_timestamp);
   data->prof_cpu_interval_start = data->prof_cpu_b_timestamp;

   /* no measurement is going on */
   pri = get_pri(current_process());
   data->prof_cpu_oldpri = (((pri == 34) || ( pri < data->prof_cpu_core_pri)) ? 1 : 0);

   /* reset the internal sample */
   sample = &data->prof_cpu_sample;
   sample->samples = 0;


   /* SET UP duplicates */
   /*  speed up by having local versions of these */
   data->prof_cpu_core_pri = zzprof.cpu.prof_cpu_pri;
   data->prof_cpu_core_issampled = zzprof.cpu.prof_cpu_issampled;
   data->prof_cpu_core_interval = zzprof.cpu.prof_cpu_interval;


    /* let the fun begin */
    data->prof_cpu_core_enabled = PROF_STATUS_ENABLED;
}


static void
prof_prepare_pri_start(struct prof_pri_core *data)
{
   struct prof_pri_sample *first_sample;
   struct prof_pri_sample *sample;
   uint32_t i;

   if (0 == zzkrn_cpu_id())
   {
      /* Start the disconnect timer. */
      zzprof.pri.prof_pri_disctimer = 100000000L / zzprof.prof_all_ticklen;    /* 100 sec. */
   }

   first_sample = data->prof_pri_firstsample
               = data->prof_pri_cursample
               = data->prof_pri_samplepool;

   first_sample->valid = 0;

   prof_timestamp_set(&data->prof_pri_b_timestamp);
   prof_timestamp_set_interval_start_ticks(&first_sample->time_ticks, &first_sample->time_uticks, data->prof_pri_b_timestamp);
   data->prof_pri_interval_start = data->prof_pri_b_timestamp;

   /* no measurement is going on */
   data->prof_pri_oldpri = 0;

   /* reset the internal sample */
   sample = &data->prof_pri_sample;
   for (i = 0; i < 35; ++i)
         sample->samples[i] = 0;


   /* SET UP duplicates */
   /*  speed up by having local versions of these */

   data->prof_pri_core_interval = zzprof.pri.prof_pri_interval;

    /* let the fun begin */
    data->prof_pri_core_enabled = PROF_STATUS_ENABLED;
}

/*
 * Helper to get_delta_time and get_diff for ticks time counting
 */

static OSTIME
prof_ticks_to_nticks(OSTICK t0, OSTICK ut0, OSTICK t1, OSTICK ut1, OSTIME tlen)
{
   OSTIME dt;
   /* Ensure utick difference >= 0 */
   if (ut1 < ut0)
   {
      ut1 += tlen;
      t1--;
   }
   dt = ut1 - ut0 + (t1 - t0) * tlen;
   if (dt < zzprof.prof_all_min_us)
   {
      dt = zzprof.prof_all_min_us;
   }
   return dt;
}

/********************************************************************************************
 *  external functions (used by prof_timestamp)
 **/

/* extern
 *  e shall be later then b
 **/
#ifdef __GNUCC__
inline
#endif
OSTIME
zzprof_get_diff(OSTICK bticks, OSTICK buticks, OSTICK eticks, OSTICK euticks)
{
    return prof_ticks_to_nticks(bticks, buticks, eticks, euticks, zzprof.prof_all_ticklen );
}


/***********************************************************************
 * Functions to set all data to 0 at startup
 */
static void
prof_tcb_core_init(struct prof_tcb_core *const data)
{
   data->bcb_profs = NULL;
   data->slots = NULL;
   data->sample = NULL;
   data->next = 0;
   data->max_alloc = 0;
   data->measure = FALSE;
   data->other = 0;
   data->pri_threshold = 0;
   prof_timestamp_reset(&data->timestamp);
   prof_timestamp_reset(&data->interval_start);
   data->running_valp = NULL;
   data->running_bid = 0;
}

static void
prof_pri_core_init(struct prof_pri_core *const data)
{
   data->prof_pri_core_enabled = PROF_STATUS_DISABLED;
  /* priority prof */
   memset(&data->prof_pri_sample, 0, sizeof(struct prof_pri_sample));
   /* Time of swap in and priority of the currently running process. */
   data->prof_pri_oldpri = 0;
   prof_timestamp_reset(&data->prof_pri_b_timestamp);

   /* interval start */
   prof_timestamp_reset(&data->prof_pri_interval_start);

   /* The sample history list along with head and tail pointers. */
   data->prof_pri_samplepool = NULL;
   data->prof_pri_firstsample = NULL;
   data->prof_pri_cursample = NULL;
   data->prof_pri_core_interval = 0;

   SMP_CODE(spin_lock_init(&data->prof_pri_core_data_lock));
}

static void
prof_cpu_core_init(struct prof_cpu_core *const data)
{
   /* cpu profiling */
   data->prof_cpu_core_enabled = PROF_STATUS_DISABLED;
   data->prof_cpu_core_issampled = 0;
   data->prof_cpu_core_pri = 0;
   data->prof_cpu_core_interval = 0;
   memset(&data->prof_cpu_sample, 0, sizeof(struct prof_cpu_sample));
   data->prof_cpu_oldpri = 0;
   prof_timestamp_reset(&data->prof_cpu_b_timestamp);
   prof_timestamp_reset(&data->prof_cpu_interval_start);
   data->prof_cpu_samplepool = NULL;
   data->prof_cpu_firstsample = NULL;
   data->prof_cpu_cursample = NULL;

   SMP_CODE(spin_lock_init(&data->prof_cpu_core_data_lock));
}

static void
prof_proc_core_init(struct prof_proc_core *const data, OSBOOLEAN block)
{
   /* duplicated fields has core in their names */
     data->prof_proc_core_enabled = PROF_STATUS_DISABLED;
     data->prof_proc_core_pri = 0;
     data->prof_proc_core_maxpid = 0;
     data->prof_proc_core_sys_process = 0;
     data->prof_proc_core_interval = 0;

     /* (possible) non zero defaults */
     data->block = block;

     data->hash_sync_counter = 0;
     memset(&data->prof_proc_sample,0, sizeof(struct prof_proc_sample)) ;
     prof_timestamp_reset(&data->prof_proc_b_timestamp);
     prof_timestamp_reset(&data->prof_proc_interval_start);
     data->prof_proc_oldpid = 0;
     data->prof_proc_oldpindex = 0;
     prof_hash_init(&data->local_hash);
     data->prof_proc_samplepool = NULL;
     data->prof_proc_firstsample = NULL;
     data->prof_proc_cursample  = NULL;
     SMP_CODE(spin_lock_init(&data->prof_proc_core_data_lock));
}

/* NON per core data init basically set to 0 */

static void
prof_tcb_init(void)
{
   zzprof.tcb.cursample = NULL;
   zzprof.tcb.enabled = 0;
   zzprof.tcb.firstsample = NULL;
   zzprof.tcb.interval = 0;
   zzprof.tcb.maxpid = 0;
   zzprof.tcb.pri_threshold = 0;
   zzprof.tcb.reports = 0;
   zzprof.tcb.samplepool = NULL;
   zzprof.tcb.prof_tcb_reqlist = NULL;
   zzprof.tcb.oversize = zzprof.tcb.default_oversize;
}

static void
prof_cpu_init(void)
{
      zzprof.cpu.prof_cpu_reports = 0;
      zzprof.cpu.prof_cpu_interval = 0;
      zzprof.cpu.prof_cpu_pri = 0;
      zzprof.cpu.prof_cpu_enabled = 0;
      zzprof.cpu.prof_cpu_disctimer = 0;
      zzprof.cpu.prof_cpu_issampled = FALSE;
      zzprof.cpu.prof_cpu_reqlist = NULL;
}

static void
prof_pri_init(void)
{
      zzprof.pri.prof_pri_reports = 0;
      zzprof.pri.prof_pri_interval = 0;
      zzprof.pri.prof_pri_enabled = 0;
      zzprof.pri.prof_pri_disctimer = 0;
      zzprof.pri.prof_pri_reqlist = NULL;
}

static void
prof_proc_init(int block)
{
   zzprof.proc[block].prof_proc_pri = 33;
   /* set at enable
   zzprof.proc[block].prof_proc_reports= 0;
   zzprof.proc[block].prof_proc_maxpid= 0;
   zzprof.proc[block].prof_proc_interval= 0;
   */
   zzprof.proc[block].prof_proc_enabled= 0;
   zzprof.proc[block].prof_proc_disctimer= 0;
   zzprof.proc[block].prof_proc_reqlist = NULL;
   SMP_CODE(prof_hash_init(&zzprof.proc[block].global_hash);)
   zzprof.proc[block].has_prof_proc_spec = 0;
   zzprof.proc[block].prof_proc_spec = NULL;
   zzprof.proc[block].prof_proc_pids = NULL;
   SMP_CODE(spin_lock_init(&zzprof.proc[block].prof_proc_lock));
}

static void
prof_free_tcb_core_data(struct prof_tcb_core *const  data)
{
   free_sample_pool( (union SIGNAL **) (&data->bcb_profs));
   free_sample_pool( (union SIGNAL **) (&data->bids));
   free_sample_pool( (union SIGNAL **) (&data->slots));
   free_sample_pool( (union SIGNAL **) (&data->sample));
}

static MonitorStatus
prof_tcb_setup_core_data(struct prof_tcb_core *const data, uint32_t nr_of_slots, OSPRIORITY pri, OSTICK interval)
{
   MonitorStatus status = MONITOR_STATUS_OK;

   data->next = 0;
   data->measure = FALSE;
   data->other = 0;
   data->pri_threshold = pri;
   NLOG(("prof: data->pri_threshold = %u\n", data->pri_threshold));
   prof_timestamp_reset(&data->interval_start);
   data->interval_start_ticks = 0;
   data->interval_start_uticks =0;
   data->running_valp = NULL;
   data->running_bid = 0;
   data->core_interval = interval;
   data->maxpid = nr_of_slots;

   /* if (data->max_alloc == (nr_of_slots +1)* zzprof.tcb.oversize) return status; */
   if (data->max_alloc != 0)
   {
      prof_free_tcb_core_data(data);
   }

   data->bcb_profs = (volatile uint32_t **) alloc_sample_pool(sizeof(void *) * (nr_of_slots +1) * zzprof.tcb.oversize );
   data->bids = (PROCESS *) alloc_sample_pool(sizeof(PROCESS) * (nr_of_slots +1) * zzprof.tcb.oversize );
   data->slots = (MonitorCPUProcess *)alloc_sample_pool(sizeof(  MonitorCPUProcess)*  (nr_of_slots +1)* zzprof.tcb.oversize);
   data->sample = (MonitorCPUProcessReport *)alloc_sample_pool( sizeof (MonitorCPUProcessReport) + ((nr_of_slots+1) * sizeof (MonitorCPUProcess)));

   if (data->bcb_profs == NULL || data->bids == NULL || data->slots == NULL || data->sample == NULL)
   {
      data->max_alloc = 0;
      status = get_last_memory_error();
      prof_free_tcb_core_data(data);
   }
   else
   {
      data->max_alloc = (nr_of_slots + 1)* zzprof.tcb.oversize;
      data->sample->processesCount =0;
      data->sample->sumOther = 0;
   }

   return status;
}


static void
prof_tcb_reset(struct prof_tcb_core *const data, OSBOOLEAN start_new_interval)
{
   uint32_t i;

   /* undo earlier measurements */
   for (i = 0;i < data->next; i++)
   {
      *data->bcb_profs[i] = 0;
/*
 * data->slots[i].id =0;
 * data->slots[i].sum=0
 * data->bids[i] =0
*/
   }
   data->next = 0;
   data->other = 0;
   data->measure = FALSE;
   if (start_new_interval) {
      prof_timestamp_set(&data->timestamp);
      prof_timestamp_set_interval_start_ticks(&data->interval_start_ticks, &data->interval_start_uticks, data->timestamp);
      data->interval_start = data->timestamp;
   }
   else{
      data->interval_start_ticks = 0;
      data->interval_start_ticks = 0;
      prof_timestamp_reset(&data->interval_start);
   }
}

/* Allocation samples (not per core) */

static MonitorStatus
prof_tcb_setup_sample_data(uint32_t max_reports, uint32_t slots)
{
   MonitorStatus status = MONITOR_STATUS_OK;
   uint32_t alloc_size = 0;
   uint32_t sample_data_size = 2 * (slots + 1) * sizeof(uint32_t);



   /* Allocate size for all samples and their process lists */
   alloc_size = max_reports * sizeof (struct prof_tcb_sample);
   alloc_size += sample_data_size * max_reports;
   /* free if not NULL */
   free_sample_pool((union SIGNAL **)&zzprof.tcb.samplepool);
   zzprof.tcb.samplepool = (struct prof_tcb_sample *)alloc_sample_pool(alloc_size);

   if (zzprof.tcb.samplepool != NULL)
   {
      struct prof_tcb_sample *samplep;
      char *bufp;
      uint32_t total_sample_size = sizeof(struct prof_tcb_sample) +
                         sample_data_size;

      for (bufp = (char*)zzprof.tcb.samplepool;
            bufp < (char*)zzprof.tcb.samplepool + alloc_size;
                     bufp += total_sample_size)
      {
         /* Set the current sample pointer to point at where the buf pointer points */
         samplep = (struct prof_tcb_sample *)bufp;

         /* next pointer points at the location a full sample ahead of the current*/
         samplep->next = (void *)(bufp + total_sample_size);
         /* pids points at the location at the beginning of the process data section */
         samplep->pids = (uint32_t *)( bufp + sizeof(struct prof_tcb_sample) );
         /* samples points at the location in the middle of the process data section*/
         samplep->samples = (uint32_t *)( bufp + sizeof(struct prof_tcb_sample)
                  + sample_data_size / 2);
         samplep->valid = 0;
      }
      /* step back and correct the next pointer for the last */
      assert(bufp == ((char*)zzprof.tcb.samplepool + alloc_size) );
      samplep = (struct prof_tcb_sample *)(bufp - total_sample_size);
      samplep->next = zzprof.tcb.samplepool;

      /* Change global buffer size params */
      zzprof.tcb.reports = max_reports;
      zzprof.tcb.maxpid = slots;

      zzprof.tcb.firstsample = zzprof.tcb.cursample = zzprof.tcb.samplepool;
   }
   else {
      zzprof.tcb.reports = 0;
      zzprof.tcb.maxpid = 0;
      status = MONITOR_STATUS_MEMORY_EXHAUSTED;
   }
   return status;
}



/* Check if this signal is from a client with setting buffer capabilities etc */
static MonitorStatus
prof_tcb_enable(union SIGNAL *sig)
{
   uint32_t max_reports = zzprof.tcb.default_reports;
   uint32_t slots = zzprof.tcb.default_maxblocks;
   MonitorStatus status = MONITOR_STATUS_OK;
   MonitorSetCPUProcessReportsEnabledRequest *req;
   OSPRIORITY pri = 33;
   union SIGNAL *enable;
   SMP_CODE(cpuid_t cpu;)
   struct OS_pid_list *bids;
   uint32_t i = 0;

   req = &sig->set_cpu_process_reports_enabled_request;

   if(sigsize(&sig) >= sizeof(MonitorSetCPUProcessReportsEnabledRequest))
   {
#ifdef HAS_PROF_PRIORITY_THRESHOLD
      if (req->reserved1 < 0x10 )
      {
         if (req->reserved1 != 0)
         {
            zzprof.tcb.oversize = (req->reserved1 & (uint16_t) 0x000F);
         }
         else {
            zzprof.tcb.oversize = zzprof.tcb.default_oversize;
         }
         NLOG (("TCB STARTED with oversize %u \n", zzprof.tcb.oversize));
         /* Check that values are reasonable */
#else
      if (0 == (req->reserved0))
      {
#endif
         if(req->maxReports >= MIN_PROF_REPORTS
               && req->maxReports <= MAX_PROF_REPORTS
               && req->maxProcsReport <= zzkrn_get_pid_info(0, 4))
         {
            max_reports = req->maxReports;
            slots = req->maxProcsReport;
         }

#ifdef HAS_PROF_PRIORITY_THRESHOLD
         if (req->usePriority)
         {
            if (req->priority <= MAX_PROF_PRIORITY)
            {
               pri= (OSPRIORITY) req->priority;
            }
            else
            {
               pri = (OSPRIORITY) zzprof.tcb.default_priority;
            }
         }
         else
#endif
         {
            pri = 33;
         }
      } /* if reserved1 is crap free */
   }

   /* The priority is pushed out to the core but we save it for get_enable replies */
   zzprof.tcb.pri_threshold = pri;
   NLOG (("Set zzprof.tcb.pri_threshold to %u \n", zzprof.tcb.pri_threshold));

   bids = get_bid_list(0);
   if (bids)
   {
      for (i = 0; i < bids->count; i++)
      {
         struct _block *bcb;
         BKL_LOCK();
         bcb = (struct _block *)zzkrn_get_pid_info(bids->list[i], 7);
         if (bcb && zzkrn_get_pid_info(bids->list[i], 6) == bids->list[i])
         {
            bcb->prof = 0;
         }
         BKL_UNLOCK();
      }
   }
   else
   {
      return MONITOR_STATUS_MEMORY_EXHAUSTED;
   }

   if (max_reports != zzprof.tcb.reports || slots !=zzprof.tcb.maxpid)
   {
      status = prof_tcb_setup_sample_data(max_reports, slots);
   }

   if (status != MONITOR_STATUS_OK)
   {
      return status;
   }

   /* Start setting up the profiling */
   if(req->interval >= MIN_PROF_INTERVAL &&
      req->interval <= zzprof_get_interval())
   {
      zzprof.tcb.interval = req->interval;
   }
   else
   {
      zzprof.tcb.interval = zzprof.tcb.default_interval;
   }

   /*
    * if not -1 we will always get an extra tick before the timeout because some time has been spent since the tick went off
    */
   /* zzprof.tcb.interval_ms =  zzprof.tcb.interval * (zzprof.prof_all_ticklen - 1) / 1000; */

   SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus && status == MONITOR_STATUS_OK; cpu++))
   {
	   SMP_CODE (if (IS_CPU_ACTIVE(cpu)))
	   {
		   /* Get core specific data */
		   struct prof_tcb_core *const data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_tcb_data;

		   /* will free if necessary */
		   status = prof_tcb_setup_core_data(data, zzprof.tcb.maxpid, zzprof.tcb.pri_threshold, zzprof.tcb.interval);
	   }
   }

   if (status != MONITOR_STATUS_OK)
   {
#ifdef OSE_SMP
      cpuid_t i;
      NLOG (("TCB STARTUP STATUS not OK aftersetup core data will freed\n"));

      for (i = 0; i < cpu; i++)
      {
    	 if (IS_CPU_ACTIVE(cpu))
    	 {
    		 prof_free_tcb_core_data(&(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_tcb_data);
    	 }
      }
#endif
      return status;
   }

   zzprof.tcb.firstsample->valid = 0;

   zzprof.tcb.disctimer = 100000000L / zzprof.prof_all_ticklen;   /* 100 sec. */

   enable = alloc(sizeof(SIGSELECT), PROF_TCB_START_SIG);
   send(&enable, _zzrmm_merge_pid);
   return status;
}


/* needed to set the interval in ticks */
static OSTMOREF
prof_request_tmo_tick( OSTICK ticks, SIGSELECT tmosigno)
{
   return (OSTMOREF) get_sysconf (
            (OSADDRESS) 49,
            (OSADDRESS) ticks,
            (OSADDRESS) tmosigno,
            (OSADDRESS) 0);
}
OSENTRYPOINT zzose_rmm_core_prof;
/**
 * @function zzose_rmm_core
 *
 * @brief The rmm process_core entry point.
 */
OS_PROCESS(zzose_rmm_core_prof)
{
   OSTMOREF total_cpu_block_tmoref, cpu_tmoref, proc_tmoref, block_tmoref, pri_tmoref;
   const SIGSELECT sigsel[] = {0};
   union SIGNAL *sig;
   SMP_CODE( cpuid_t cpu = zzkrn_cpu_id(); )
   struct prof_core_data *const core_data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)) ;
   struct prof_tcb_core *const tcb_data = &core_data->prof_tcb_data;
   struct prof_proc_core *const proc_data = &core_data->prof_proc_data[0];
   struct prof_proc_core *const block_data = &core_data->prof_proc_data[1];
   struct prof_cpu_core *const cpu_data = &core_data->prof_cpu_data;
   struct prof_pri_core *const pri_data = &core_data->prof_pri_data;
   OSBOOLEAN has_cputmo = 0;
   OSBOOLEAN has_pritmo = 0;
   OSBOOLEAN has_proctmo = 0;
   OSBOOLEAN has_blocktmo =0;
   OSBOOLEAN has_tcbtmo = 0;

   for (;;)
   {
      sig = receive(sigsel);
      switch(sig->sig_no)
      {
#ifdef OSE_SMP
      case PROF_CORE_KILL_SIG:
    	  prof_core_suicide(core_data);
    	  /* this point cannot be reached */
    	  break;
#endif
/* TCB */
         case PROF_TCB_CLOSE_SIG:
         {
            if (tcb_data->core_enabled == PROF_STATUS_ENABLED )
            {
                prof_tcb_close_interval(tcb_data);
            }
            total_cpu_block_tmoref = prof_request_tmo_tick( tcb_data->core_interval -1 , PROF_TCB_CLOSE_SIG);

         }
         break;

         /* the START_TCB_INTERVAL will come at a tick */
         case PROF_TCB_START_SIG:
            LOG( ("prof TCB [%u] START_SIG set timer at %u ms \n", zzkrn_cpu_id(), tcb_data->core_interval));
            prof_tcb_reset(tcb_data, TRUE);
            tcb_data->core_enabled = PROF_STATUS_ENABLED;
            total_cpu_block_tmoref = prof_request_tmo_tick( tcb_data->core_interval - 1 , PROF_TCB_CLOSE_SIG);
            has_tcbtmo = 1;
            break;

         case PROF_TCB_STOP_SIG:
            LOG( ("prof TCB STOP_SIG sender=%u\n", sender(&sig) ));
            prof_tcb_reset(tcb_data, FALSE);
            if (has_tcbtmo) {
              cancel_tmo(&total_cpu_block_tmoref);
            }
            has_tcbtmo = 0;
            tcb_data->core_enabled = PROF_STATUS_DISABLED;
            LOG( ("prof TCB  STOPPED OK\n"));
            break;

 /* CPU  */
         case PROF_CPU_CLOSE_SIG:
            if (cpu_data->prof_cpu_core_enabled ==  PROF_STATUS_ENABLED )
            {
              if ( cpu_data->prof_cpu_core_issampled == TRUE)
              {
            	  SMP_CODE(if (0 == cpu))
            	  {
            		  /* If sampled then only run on one core. */
            		  prof_sampled_cpu_close_interval();
            	  }
              }
              else
              {
                 prof_cpu_close_interval(cpu_data);
              }
              cpu_tmoref = prof_request_tmo_tick( cpu_data->prof_cpu_core_interval - 1, PROF_CPU_CLOSE_SIG);
           }
            else
            {
               cpu_tmoref = 0;
            }
           break;

                 /* the START_TCB_INTERVAL will come at a tick */
         case PROF_CPU_START_SIG:
                    LOG( ("prof [%u] CPU START_SIG set timer to %u ticks \n", zzkrn_cpu_id(), cpu_data->prof_cpu_core_interval));
                    prof_prepare_cpu_start(cpu_data);
                    cpu_tmoref = prof_request_tmo_tick( cpu_data->prof_cpu_core_interval - 1, PROF_CPU_CLOSE_SIG);
                    has_cputmo =1;
                    break;

         case PROF_CPU_STOP_SIG:
                    LOG( ("prof [%u] CPU STOP_SIG \n", zzkrn_cpu_id()));
                    /* prof_reset_cpu_prof(cpu_data); */
                    if (has_cputmo)
                    {
                       cancel_tmo(&cpu_tmoref);
                       has_cputmo = 0;
                    }
                    cpu_data->prof_cpu_core_enabled = PROF_STATUS_DISABLED;
                    break;
/***** PROCESS */
         case PROF_PROC_CLOSE_SIG:
            if (proc_data->prof_proc_core_enabled ==  PROF_STATUS_ENABLED )
            {
               prof_proc_close_interval(proc_data);
               /* LOG( ("prof [%u] PROC interval closed \n", zzkrn_cpu_id()) ); */
               proc_tmoref = prof_request_tmo_tick( proc_data->prof_proc_core_interval -1, PROF_PROC_CLOSE_SIG);
            }
            else {
               has_proctmo = 0;
            }
           break;
         case PROF_PROC_START_SIG:
            LOG( ("prof PROC_START_SIG \n"));
            prof_prepare_proc_start(proc_data);
            proc_tmoref = prof_request_tmo_tick( proc_data->prof_proc_core_interval -1, PROF_PROC_CLOSE_SIG);
            has_proctmo = 1;
            break;
         case PROF_PROC_STOP_SIG:
             LOG( ("prof PROC STOP_SIG \n"));
             /* prof_reset_proc_prof(proc_data); */
             if (has_proctmo)  {  has_proctmo = 0; cancel_tmo(&proc_tmoref);  }
             proc_data->prof_proc_core_enabled = PROF_STATUS_DISABLED;
             break;
/*** BLOCKS */
         case PROF_BLOCK_CLOSE_SIG:
           if (block_data->prof_proc_core_enabled == PROF_STATUS_ENABLED )
           {
              prof_proc_close_interval(block_data);
              block_tmoref = prof_request_tmo_tick( block_data->prof_proc_core_interval -1, PROF_BLOCK_CLOSE_SIG);
           }
           else
           {
              has_blocktmo = 0;
           }
           break;
         case PROF_BLOCK_START_SIG:
            prof_prepare_proc_start(block_data);
            block_tmoref = prof_request_tmo_tick( block_data->prof_proc_core_interval -1, PROF_BLOCK_CLOSE_SIG);
            has_blocktmo = 1;
            break;
         case PROF_BLOCK_STOP_SIG:
             LOG( ("prof [%u] BLOCK STOP_SIG \n", zzkrn_cpu_id()));
            /* prof_reset_proc_prof(block_data); */
             if (has_blocktmo)
             {
                cancel_tmo(&block_tmoref);
                has_blocktmo = 0;
             }
             block_data->prof_proc_core_enabled = PROF_STATUS_DISABLED;
             break;
/* pri */
         case PROF_PRI_CLOSE_SIG:
           if (pri_data->prof_pri_core_enabled == PROF_STATUS_ENABLED)
           {
              prof_pri_close_interval(pri_data);
              pri_tmoref = prof_request_tmo_tick( pri_data->prof_pri_core_interval -1 , PROF_PRI_CLOSE_SIG);
           }
           else
           {
              has_pritmo = 0;
           }
           break;
         case PROF_PRI_START_SIG:
            prof_prepare_pri_start(pri_data);
            pri_tmoref = prof_request_tmo_tick( pri_data->prof_pri_core_interval -1, PROF_PRI_CLOSE_SIG);
            has_pritmo = 1;
            break;
         case PROF_PRI_STOP_SIG:
             LOG( ("prof [%u] PRI STOP_SIG \n", zzkrn_cpu_id()));
             /* prof_reset_pri_prof(pri_data); */
             if(has_pritmo)
             {
                cancel_tmo(&pri_tmoref);
                has_pritmo = 0;
             }
             pri_data->prof_pri_core_enabled = PROF_STATUS_DISABLED;
             break;

         default:
            ramlog_printf("RMM_CORE_PROF unknown signal %u\n", sig->sig_no);
      }

      free_buf(&sig);
   }
}


static void
prof_start_core(cpuid_t cpu)
{
	char proc_name[15];
	PROCESS proc;
	struct debug_pcb *deb_pcb;
        TOUCH(cpu);

#ifndef OSE_SMP
	(void)cpu;
#endif

	sprintf(proc_name, "ose_rmm_prof%02d", SMP_CODE_YES_NO(cpu,0) );

	/* Create a process, bind it to cpu 0 and start it */
	proc = create_process(OS_PRI_PROC,            /* Process type */
			proc_name,              /* name         */
			zzose_rmm_core_prof,                /* entrypoint   */
			1024,               /* stack size   */
			RMM_PROF_PRI,                /* priority     */
			0,                      /* timeslice    */
			0 /*block*/,              /* block        */
			0,                      /* redir entry  */
			0,                      /* SKIP vector       */
			0); /* user         */
	_zzrmm_core_pids SMP_INDEX_X(cpu) = proc;

	deb_pcb = zzget_debugpcb(proc);
        assert(deb_pcb);
        if (deb_pcb)
        {
           deb_pcb->debug_flags |= SYS_PROCESS + 0x30; /* make it a sysproc */
        }
	(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_tcb_data.prof_tcb_core_pcb = deb_pcb;

	SMP_CODE (get_sysconf( (OSADDRESS) 75,
			(OSADDRESS) proc,
			(OSADDRESS) cpu,
			(OSADDRESS) 0); )
	LOG(("Will now start process %u, _zzrmm_core_pids[%u]=%u\n",proc, SMP_CODE_YES_NO(cpu,0), _zzrmm_core_pids SMP_INDEX_X(cpu) ));
	start(proc);
}

/* the functionality in the zzrmm_merge may be included in the rmm OR be promoted to zzrmm_prof */


OSENTRYPOINT zzose_rmm_merge_prof;
/**
 * @function zzose_rmm_core
 *
 * @brief The rmm process_core entry point.
 */
OS_PROCESS(zzose_rmm_merge_prof)
{
   LOCKSTATE msr;
   union SIGNAL *sig;
   OSTMOREF total_cpu_block_merge_tmoref = (OSTMOREF) 0;
   cpuid_t cpu = 0;
   MonitorCPUProcessReport *sample_list SMP_ARRAY;
   const SIGSELECT sigsel[] = {0};
   uint32_t running_status = 0;

   _zzrmm_merge_pid = current_process();

   SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++) )
   {
	   /* just for the compiler */
	   sample_list SMP_INDEX_X(cpu) = (MonitorCPUProcessReport *) NULL;

	   prof_start_core(cpu);
	   zzprof.prof_core_data_entry SMP_INDEX_X(cpu).core_active = TRUE;
   }


  /* wait for profiling start */

   for (;;)
   {
      sig = receive(sigsel);
      switch(sig->sig_no)
      {
         case PROF_TCB_MERGE_SIG:
            {
               struct prof_tcb_sample *old, *new;
               if (running_status == 1) {
                  LOCK_SAVE(msr);
                  old = zzprof.tcb.cursample;
                  new = zzprof.tcb.cursample->next;
                  new->valid = 0;

                  prof_tcb_merge_interval((MonitorCPUProcessReport **)&(sample_list SMP_INDEX_X(0)), old);
                  total_cpu_block_merge_tmoref = prof_request_tmo_tick( zzprof.tcb.interval - 1, PROF_TCB_MERGE_SIG);
                  if (new == zzprof.tcb.firstsample)
                  {
                       zzprof.tcb.firstsample = zzprof.tcb.firstsample->next;
                  }

                  old->valid = 1;
                  zzprof.tcb.cursample = zzprof.tcb.cursample->next;
                  LOCK_RESTORE(msr);
               }
            }
            break;

         case PROF_TCB_START_SIG:
            LOG(("merge: START_SIG \n"));
            running_status = 1;
            zzprof.tcb.firstsample = zzprof.tcb.cursample = zzprof.tcb.samplepool;
            zzprof.tcb.firstsample->valid = 0;

            SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++) )
            {
            	SMP_CODE( if (IS_CPU_ACTIVE(cpu) ))
            	{
            		union SIGNAL *startsig;
            		struct prof_core_data *const core_data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu));
            		/* sample size may have changed between the enabling of total_cpu_block */

            		sample_list SMP_INDEX_X(cpu) = core_data->prof_tcb_data.sample;
            		startsig = alloc(sizeof(SIGSELECT), PROF_TCB_START_SIG);
            		LOG(("merge: PROF_START_SIG sent to cpu %u process %u\n", SMP_CODE_YES_NO(cpu,0), _zzrmm_core_pids SMP_INDEX_X(cpu) ));
            		send(&startsig,   _zzrmm_core_pids SMP_INDEX_X(cpu) );
            	}
            }
            total_cpu_block_merge_tmoref = prof_request_tmo_tick( zzprof.tcb.interval /* NO -1 */ , PROF_TCB_MERGE_SIG);
            break;

         case PROF_TCB_STOP_SIG:
            LOG(("merge: STOP_SIG \n"));
            running_status = 0;
            SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++) )
            {
            	SMP_CODE( if (IS_CPU_ACTIVE(cpu) ))
            	{
            		union SIGNAL *stopsig;
            		stopsig = alloc(sizeof(SIGSELECT), PROF_TCB_STOP_SIG);
            		send(&stopsig,   _zzrmm_core_pids SMP_INDEX_X(cpu) );
            	}
            }
            cancel_tmo(&total_cpu_block_merge_tmoref);
            break;
         default:
            break;
      }
      free_buf(&sig);
   }
}


/*
** This is the prof_sample_cpuload() system call.
**
** It is called at regular intervals by a high priority interrupt
** process.
** Each call increments a counter corresponding to the priority
** of the currently running prioritized process.
*/

static void
prof_sample_cpuload(void)
{
   U32             pri;

   SMP_CODE(cpuid_t cpu;)

   /* Exit if disabled. */
   if (!zzprof.cpu.prof_cpu_enabled)
      return;

   SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++))
   {
	   SMP_CODE( if (IS_CPU_ACTIVE(cpu) ))
	   {
		   struct prof_cpu_core *data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_cpu_data;
		   /* Pick up the highest ready priority. This will only consider
		   priority processes and the threshold can not be set to include
		   background processes. Interrupt processes is also ignored. */
		   pri = (U32) FIND_FIRST_BIT(zzget_dp_look_aside(cpu));

		   /* Update the activity counter for the current priority. */
		   if (pri < zzprof.cpu.prof_cpu_pri)
			   data->prof_cpu_cursample->samples += 1;

		   /* Accumulate time in the current sample. */
		   data->prof_cpu_cursample->total_time += 1;
	   }
   }
}

/*
** The prof_enable_sample_cpuload() system call is issued from a
** start hook 2 by an application which supplies a sample timer.
** The call returns the address of the prof_sample_cpuload() function,
** or zero if sampled cpu load measurement is not supported by
** the current kernel.
** It is implemented using the get_sysconf() mechanism using function 31.
**
** The interrupt process associated with the timer calls
** the prof_sample_cpuload() function periodially to maintain the
** statistics provided.
*/
static OSADDRESS
prof_enable_sample_cpuload(OSADDRESS p, OSADDRESS q, OSADDRESS r)
{
   SMP_CODE(cpuid_t cpu;)
   /* Remove ``not used args'' compiler warnings. */
   (void) p;
   (void) q;
   (void) r;

   /* Select the timer based statistics algorithm. */
   zzprof.cpu.prof_cpu_issampled = 1;

   SMP_CODE(for (cpu =0; cpu < zzprof.prof_all_num_cpus ; cpu++))
   {
	   SMP_CODE( if (IS_CPU_ACTIVE(cpu) ))
	   {
		   struct prof_cpu_core *data = &(zzprof.prof_core_data_entry SMP_INDEX_X (cpu)).prof_cpu_data;
		   data->prof_cpu_core_issampled = 1;
	   }
   }
   /* Return the function to call periodically. */
   return (OSADDRESS) prof_sample_cpuload;
}

static void
prof_basic_core_init(cpuid_t cpu)
{
#ifndef OSE_SMP
        (void)cpu;
#endif

	prof_tcb_core_init(&(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_tcb_data);
	prof_pri_core_init(&(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_pri_data);
	prof_cpu_core_init(&(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_cpu_data);
	prof_proc_core_init(&(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_proc_data[0],0);
	prof_proc_core_init(&(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_proc_data[1],1);
}

static uint32_t
zzprof_get_interval(void)
{
   if(zzprof.prof_all_max_interval == 0)
   {
      zzprof.prof_all_max_interval = prof_get_max_interval();
   }
   return (uint32_t)zzprof.prof_all_max_interval;
}

/*
** This function initializes the profiling subsystem upon restart.
*/

static void
zzprof_init(void)
{
   PROCESS proc;
   cpuid_t cpu = 0;
   struct debug_pcb *deb_pcb;
   TOUCH(cpu);

   /* Initialize common variables. */
   zzprof.prof_all_ticklen = system_tick();

   zzprof.prof_all_num_cpus = SMP_CODE_YES_NO(zzkrn_num_cpus(), 1);

   /* mark the maximum interval as unitinitalized */
   zzprof.prof_all_max_interval = 0;

   /* Install the prof_enable_sample_cpuload() system call. */
   get_sysconf(2, (OSADDRESS) 31, (OSADDRESS) prof_enable_sample_cpuload, 0);

   prof_tcb_init();
   prof_cpu_init();
   prof_pri_init();
   prof_proc_init(0);
   prof_proc_init(1);

   SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++) )
   {
	   /* initiate all here no check for core_active */
      prof_basic_core_init(cpu);
   }

   proc = create_process(OS_PRI_PROC,            /* Process type */
             "ose_rmm_merge_prof",              /* name         */
             zzose_rmm_merge_prof,                /* entrypoint   */
             128,               /* stack size   */
             0,                     /* priority     */
             0,                      /* timeslice    */
             0,              /* block        */
             0,                      /* redir entry  */
             (OSVECTOR)~0,                      /* SKIP vector       */
             0); /* user         */
   deb_pcb = zzget_debugpcb(proc);
   assert(deb_pcb);
   if (deb_pcb)
   {
      deb_pcb->debug_flags |= SYS_PROCESS + 0x30; /* make it a sysproc */
   }
   start(proc);
}



/*
** Frees the buffers for pre-requested processes in process profiling
*/
static void
prof_free_prereq_bufs(struct prof_proc *data)
{
  free_sample_pool((union SIGNAL **)&data->prof_proc_pids);
  free_sample_pool((union SIGNAL **)&data->prof_proc_spec);
}


/*
** Sets up and intializes (or re-initializes existing) buffers for
** pre-requested processes and wildcard list for per process profiling
*/
static OSBOOLEAN
prof_create_prereq_bufs( uint32_t max_nr_pids, struct prof_proc *data)
{
  OSBOOLEAN retval = (OSBOOLEAN) 0;
  uint32_t i;
 /* data->prof_proc_maxpid is not set yet !! */

  /* Free any existing pre request process buffers */
  prof_free_prereq_bufs(data);
  data->prof_proc_pids = (void *) alloc_sample_pool( max_nr_pids * sizeof(uint32_t) );
  data->prof_proc_spec = (void *) alloc_sample_pool( max_nr_pids * sizeof(MonitorCPUProcessReportpointInfo * ) );

  if ((data->prof_proc_pids == NULL) || (data->prof_proc_spec == NULL))
  {
     prof_free_prereq_bufs(data);
     retval = (OSBOOLEAN) 0;
  }
  else
  {
    /* Initialize the buffers and variables*/
    for (i = 0; i < max_nr_pids; i++)
    {
      data->prof_proc_pids[i] = 0;
      data->prof_proc_spec[i] = 0;
    }
    retval = (OSBOOLEAN)1;
  }

  /* Initialize other variables */
  data->has_prof_proc_spec = 0;


  return retval;
}

/*
** Frees the process reports buffer for per process profiling
*/
static void
prof_free_proc_report_bufs(struct prof_proc_core *data)
{
  free_sample_pool((union SIGNAL **) &data->prof_proc_samplepool);
  free_sample_pool((union SIGNAL **) &data->prof_proc_sample.pids );
}

/*
** Build circular sample list for per process profiling. An extra
** field ( the +1 ) is used to count cpu load for all other processes
** in samples where the list is filled up
*/
static OSBOOLEAN
prof_create_proc_report_bufs(uint32_t reports, uint32_t procs, struct prof_proc_core *data)
{
   uint32_t alloc_size = 0;
   uint32_t sample_data_size = 2 * (procs + 1) * sizeof(uint32_t);


   /* Free any existing process report buffers based on the
    * current value of prof_proc_reports (dont't change it before free)*/
   prof_free_proc_report_bufs(data);

   /* Change global buffer size params */
   /*data->prof_proc_reports = reports;*/
   data->prof_proc_core_maxpid = procs;

   /* Allocate size for all samples and their process lists */
   alloc_size = reports * sizeof (struct prof_proc_sample);
   alloc_size += sample_data_size * reports;

   data->prof_proc_samplepool = (void *) alloc_sample_pool(alloc_size);
    /*
     * From the buffer aquired we construct the sample list by creating
     * prof_proc_reports number of sections containing first the sample struct and
     * then a following (procs+1)*uint32 section for the PIDs and another
     * (procs+1)*uint32 section for the corresponing sample values.
     */
   if (data->prof_proc_samplepool != NULL)
   {
      struct prof_proc_sample *samplep;
      char *bufp;
      uint32_t total_sample_size = sizeof(struct prof_proc_sample) +
                                   sample_data_size;

      for (bufp = (char*)data->prof_proc_samplepool;
           bufp < (char*)data->prof_proc_samplepool + alloc_size;
           bufp += total_sample_size)
      {
         /* Set the current sample pointer to point at where the buf pointer points */
         samplep = (struct prof_proc_sample *)bufp;

         /* next pointer points at the location a full sample ahead of the current*/
         samplep->next = (void *)(bufp + total_sample_size);
         /* pids points at the location at the beginning of the process data section */
         samplep->pids = (uint32_t *)( bufp + sizeof(struct prof_proc_sample) );
         /* samples points at the location in the middle of the process data section*/
         samplep->samples = (uint32_t *)( bufp + sizeof(struct prof_proc_sample)
                            + sample_data_size / 2);
         samplep->valid = 0;
      }

      /* bufp now points at the end of the allocated buffer, so make the last sample's
       * next pointer (which currently is a wild pointer) point at the first sample to
       * create the ring in the ring buffer */
      assert(bufp == ((char*) data->prof_proc_samplepool + alloc_size) );
      samplep = (struct prof_proc_sample *)(bufp - total_sample_size);
      samplep->next = data->prof_proc_samplepool;

      /* Reset the buffer pointers and return*/
      data->prof_proc_firstsample = data->prof_proc_cursample = data->prof_proc_samplepool;

      bufp = (char *) alloc_sample_pool(sample_data_size);
      if (bufp)
      {
         data->prof_proc_sample.pids = (uint32_t *) bufp ;
         data->prof_proc_sample.samples = (uint32_t *)( bufp + sample_data_size / 2);
      }
      else
      {
         /* Failed for some reason??  Reset variables */
          /*data->prof_proc_reports = 0;*/
          data->prof_proc_core_maxpid = 0;
          return 0;
      }
   }
   else
   {
      /* Failed due to too large buffer wanted. Reset variables */
     /* data->prof_proc_reports = 0;*/
      data->prof_proc_core_maxpid = 0;
      return 0;
   }
   return 1;
}

static OSBOOLEAN
prof_setup_cpu_report_bufs_core(struct prof_cpu_core *data)
{
   uint32_t i;
   OSBOOLEAN retval = (OSBOOLEAN) 0;

   SMP_CODE(SPIN_LOCK(&data->prof_cpu_core_data_lock); )

   free_sample_pool((union SIGNAL **)&data->prof_cpu_samplepool );

/* Build circular sample list for per core cpu profiling. */
   NLOG(("allocating \n"));
   data->prof_cpu_samplepool  = (void *) alloc_sample_pool(zzprof.cpu.prof_cpu_reports * sizeof (struct prof_cpu_sample));
   if( data->prof_cpu_samplepool  != NULL)
   {
	   for (i = 0; i < zzprof.cpu.prof_cpu_reports; i++)
      {
         data->prof_cpu_samplepool  [i].next = & (data->prof_cpu_samplepool[i + 1]);
         data->prof_cpu_samplepool  [i].valid = 0;
      }
      data->prof_cpu_samplepool  [zzprof.cpu.prof_cpu_reports - 1].next = &(data->prof_cpu_samplepool[0]);
      data->prof_cpu_firstsample =
         data->prof_cpu_cursample =
            data->prof_cpu_samplepool;

      /* done creating the cpu reports buf */
      retval = (OSBOOLEAN) 1;
   }
   SMP_CODE(SPIN_UNLOCK(&data->prof_cpu_core_data_lock); )

   return retval;
}


/*
** Sets up and intializes (or re-initializes existing) report
** buffers for the overall cpu load monitoring
*/
static OSBOOLEAN
prof_setup_cpu_report_bufs(uint32_t reports)
{
   OSBOOLEAN retval = (OSBOOLEAN) 1;

   SMP_CODE(cpuid_t cpu;)

   /* Turn off CPU profiling*/
   zzprof.cpu.prof_cpu_enabled = 0;

   /* Set the new number of CPU profiling reports */
   zzprof.cpu.prof_cpu_reports = reports;

   NLOG(("setup cpu report bufs \n"));

   /* Free any previously allocated buffers*/
   SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus && (retval == 1); cpu++)) 
   {
      SMP_CODE ( if (IS_CPU_ACTIVE(cpu ) ))
      {
         struct prof_core_data *const core_data = &(zzprof.prof_core_data_entry SMP_INDEX_X (cpu));
         retval = prof_setup_cpu_report_bufs_core(&(core_data->prof_cpu_data));
      }
   }

#ifdef OSE_SMP
   if (retval == 0)
   {
      cpuid_t i;
      for (i = 0 ; i < cpu ; i++ )
      {
    	  SMP_CODE ( if (IS_CPU_ACTIVE(cpu ) ))
    	  {
    		  struct prof_cpu_core *const data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_cpu_data;
    		  free_sample_pool((union SIGNAL **) &data->prof_cpu_samplepool);
    	  }
      }
   }
#endif


#if 0
   /* Initialize other global cpu variables. */
   rw.prof_cpu_interval = 100000 / system_tick();
   rw.prof_cpu_pri = 24;
   rw.prof_cpu_reqlist = (void *) 0;
#endif
   NLOG(("empty pending cpu_req\n"));
  /* Send message to pending client (possible with interrupts off?)*/
  prof_empty_pending_cpu_req(TRUE);

  return (OSBOOLEAN) retval;
}

static OSBOOLEAN
prof_setup_pri_report_bufs_core(struct prof_pri_core *data)
{
	OSBOOLEAN retval = TRUE;
	uint32_t i;
	struct prof_pri *const pri_data = &zzprof.pri;

	SMP_CODE(SPIN_LOCK(&data->prof_pri_core_data_lock); )
	/* Free the sample pool if previously allocated */
	free_sample_pool((union SIGNAL **) &data->prof_pri_samplepool);

	/* Build circular sample list for per priority profiling */
	data->prof_pri_samplepool  = (void *) alloc_sample_pool(pri_data->prof_pri_reports * sizeof (struct prof_pri_sample));
	if(data->prof_pri_samplepool != NULL)
	{
		for (i = 0; i < pri_data->prof_pri_reports; i++)
		{
			data->prof_pri_samplepool [i].next = &data->prof_pri_samplepool [i + 1];
			data->prof_pri_samplepool [i].valid = 0;
		}
		data->prof_pri_samplepool [pri_data->prof_pri_reports - 1].next = &data->prof_pri_samplepool [0];
		data->prof_pri_firstsample = data->prof_pri_cursample = data->prof_pri_samplepool;
	}
	else
	{
		retval = (OSBOOLEAN) 0;
	}
	SMP_CODE(SPIN_UNLOCK(&data->prof_pri_core_data_lock); )
	return retval;
}


/*
** Sets up and intializes (or re-initializes existing) report
** buffers for the per priority  monitoring
*/
static OSBOOLEAN
prof_setup_pri_report_bufs( uint32_t reports )
{
   OSBOOLEAN retval = (OSBOOLEAN) 1;
   struct prof_pri * const pri_data = &zzprof.pri;
   SMP_CODE(cpuid_t cpu;)

   /* Turn off profiling */
   pri_data->prof_pri_enabled = 0;

   /* Set the new number of reports*/
   pri_data->prof_pri_reports = reports;

   SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus && (retval == 1); cpu++)) 
   {
      SMP_CODE ( if (IS_CPU_ACTIVE(cpu ) ))
      {
         struct prof_pri_core *const data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_pri_data;
         retval = prof_setup_pri_report_bufs_core(data);
      }
   }

#ifdef OSE_SMP
   if (retval == 0)
   {
      cpuid_t i;
      for (i = 0; i < cpu; i++ )
      {
         if (IS_CPU_ACTIVE(cpu))
         {
            struct prof_pri_core *const data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_pri_data;
            free_sample_pool((union SIGNAL **) &data->prof_pri_samplepool);
         }
      }
   }
#endif

   /* Reset other per priority variables. */
   prof_empty_pending_pri_req(TRUE);
   pri_data->prof_pri_reqlist = (void *) 0;
   
   return retval;
}

static OSBOOLEAN
prof_setup_proc_report_bufs_core( uint32_t reports, uint32_t procs, struct prof_proc_core *data)
{
	OSBOOLEAN retval = (OSBOOLEAN) TRUE;
	/* create the process report ring buffers*/
	if( prof_create_proc_report_bufs(reports, procs, data) )
	{
		/* Set up the indexing buffers for new number of processes */
		if ( !prof_hash_create_proc_index_bufs(procs, &data->local_hash) )
		{
			/* Free the already allocated process report bufs  */
			prof_free_proc_report_bufs(data);
			retval = (OSBOOLEAN) 0;
		}
	}
	else
	{
		retval = (OSBOOLEAN) 0;
	}
	return retval;
}


/*
** Sets up and intializes (or re-initializes existing) report
** buffers for the per process monitoring
*/
static OSBOOLEAN
prof_setup_proc_report_bufs( uint32_t reports, uint32_t procs, MonitorBoolean block)
{
  OSBOOLEAN retval = (OSBOOLEAN) 1;

  SMP_CODE(cpuid_t cpu;)

  /* Turn off profiling */
  zzprof.proc[block].prof_proc_enabled = FALSE;

  if(!prof_create_prereq_bufs(procs, &zzprof.proc[block]))
  {
     return (OSBOOLEAN) 0;
     /* better return here !!
      * do not want to go into the loop at all
      * the retval == 1 test is no good for unicore
      */
  }
#ifdef OSE_SMP
  if (!prof_hash_create_proc_index_bufs(procs, &zzprof.proc[block].global_hash) )
  {
	 prof_free_prereq_bufs(&zzprof.proc[block]);
	 return (OSBOOLEAN) 0;
  }
#endif

  SMP_CODE( for (cpu= 0; cpu <zzprof.prof_all_num_cpus && retval == 1 ; cpu++ ) )
  {
	  SMP_CODE( if (IS_CPU_ACTIVE(cpu)))
	  {
		  struct prof_proc_core *data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_proc_data[block];
          /* if not disabled we should not be here data->prof_proc_core_enabled = FALSE; */
		  retval = prof_setup_proc_report_bufs_core(reports, procs, data);
	  }
  }

   if (retval == 0) {
      SMP_CODE(cpuid_t i;)
      prof_free_prereq_bufs(&zzprof.proc[block]);
      SMP_CODE(prof_hash_free_proc_index_bufs(&zzprof.proc[block].global_hash);)

      /* free earlier SMP laps */
      SMP_CODE(
         for (i= 0; i < cpu ; i++ )
         {
        	  if (IS_CPU_ACTIVE(cpu))
        	  {
        		  struct prof_proc_core *data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_proc_data[block];
        		  prof_free_proc_report_bufs(data);
        		  prof_hash_free_proc_index_bufs(&data->local_hash);
        	  }
         }
      ) /* SMP_CODE ends here */
   }

  /* Send responses to pending profiling requests*/
   prof_empty_pending_proc_req(block, TRUE);

   /* just what we did for all cores zzprof.proc[block].prof_proc_reqlist = (void *) 0; */

  return retval;
}


/*
 * Helper to prof_update_prof_proc_one_pids
 */

static int
prof_match_prof_proc_spec(struct debug_pcb *proc)
{
   uint32_t        i, j;
   const char     *process, *block, *segment;

   for (i = 0; i < zzprof.proc[0].prof_proc_maxpid; i++)
   {
      MonitorCPUProcessReportpointInfo *r = zzprof.proc[0].prof_proc_spec[i];

      if (r)
      {
         switch (r->scope.type)
         {
            case MONITOR_SCOPE_ID:
               if (r->scope.id == proc->pid)
                  return 1;

               break;
            case MONITOR_SCOPE_NAME_PATTERN:
               for (j = 0; j < r->tagsCount; j += r->tags[j].tag.count)
               {
                  if (MONITOR_SCOPE_TAG_NAME == r->tags[j].tag.type)
                  {
                     split_string(&segment, &block, &process, r->tags[j + 1].str);

                     {
                        uint32_t        pmatch = process ? !estrcmp(process, proc->name) : 1;
                        uint32_t        bmatch = block ? !eestrcmp(block, proc->blockname) : 1;

                        if (bmatch && pmatch)
                           return 1;
                        else
                           break;
                     }
                  }
               }
               break;
            default:
               break;
         }
      }
   }

   return 0;
}

/*
** Subfunction to prof_update_prof_proc_pids().
**
** This function traverses all processes in a block, checks for a match and
** saves the pids in the stored_pids temporary list.
*/
static void
prof_update_prof_proc_one_pids(PROCESS bid, struct OS_pid_list *stored_pids)
{
   struct OS_pid_list *pid_list;
   OSBUFSIZE       pid_count;
   PROCESS        *pid;
   struct debug_pcb *deb_pcb;
   struct prof_proc *data = &zzprof.proc[0];

   /* NOTE prof_pid_lock() should be active when entering this function */

   /* Get all available process id's. */
   if ((pid_list = get_pid_list(bid)))
   {
      pid_count = pid_list->count;
      pid = pid_list->list;
   }
   else
   {
      pid_count = 0;
      pid = NULL;
   }

   /* For all processes... */
   for (; pid_count; pid_count--, pid++)
   {
      /* Get the debug pcb of the process. */
      deb_pcb = zzget_debugpcb(*pid);
      if (!deb_pcb)
      {
         continue;
      }

      /* Match the process against the wildcard name list. */
      if (!prof_match_prof_proc_spec(deb_pcb))
      {
         /* Skip it if no match. */
         continue;
      }

      /* Save the pid in the temporary list */
      if (stored_pids->count < data->prof_proc_maxpid)
      {
         stored_pids->list[stored_pids->count] = *pid;
         stored_pids->count++;
      }
   }

   /* Free the block id list if we obtained one. */
   if (pid_list)
   {
      free_buf((union SIGNAL **) &pid_list);
   }
}

/*
** Subfunction to do_modify for prof_set_procm_status.
**
** This function traverses all processes in the system and stores any
** matching process id's in the profiler selection array.
*/

static struct OS_pid_list * 
prof_update_prof_proc_pids(void)
{
   struct OS_pid_list *bid_list;
   OSBUFSIZE       bid_count = 0;
   PROCESS        *bid;
   struct OS_pcb  *pcb = (struct OS_pcb *) 0;
   struct OS_pid_list *stored_pids;

   stored_pids = (struct OS_pid_list *)alloc(sizeof(struct OS_pid_list) + (zzprof.proc[0].prof_proc_maxpid - 1) * sizeof(PROCESS), 0);
   stored_pids->count = 0;

   /* Get all available block id's. */
   if ((bid_list = get_bid_list(0)))
   {
      bid_count = bid_list->count;
      bid = bid_list->list;
   }
   else
   {
      ramlog_printf("rmm: prof_update_prof_proc_pids: could not obtain bid list\n");
      return stored_pids;
   }

   /* For all blocks... */
   for (; bid_count; bid_count--, bid++)
   {
      /* get the status of the block. */
      pcb = get_pcb(*bid);

      /* Skip the block if there is something wrong with it. */
      if ((!pcb) || (pcb->type != OS_BLOCK))
      {
         /* Free the block control block if we obtained one. */
         if (pcb)
            free_buf((union SIGNAL **) &pcb);
      }
      else
      {
         /* Check all processes in the block. */
         prof_update_prof_proc_one_pids(*bid, stored_pids);

         /* Free the block control block if we obtained one. */
         if (pcb)
            free_buf((union SIGNAL **) &pcb);
      }
   }

   /* Free the block id list if we obtained one. */
   if (bid_list)
   {
      free_buf((union SIGNAL **) &bid_list);
   }
   return stored_pids;
}


#if defined(HAS_PROF_ALL_EXECUTION_UNITS) && defined(OSE_SMP)
static void
send_cpu_reports_sum(union SIGNAL **rsig, int link_in, Offspring * offspring)
{
   LOCKSTATE msr;
   union SIGNAL *sig;
   cpuid_t cpu;
   uint32_t count;
   uint32_t tick = (*rsig)->get_cpu_reports_request.time.tick;
   uint32_t utick = (*rsig)->get_cpu_reports_request.time.ntick;
   MonitorCPUReport *d;
   volatile struct prof_cpu_sample *s SMP_ARRAY;

   /* Allocate a worst case response signal. */
   sig = alloc(sizeof(MonitorGetCPUReportsReply) + zzprof.cpu.prof_cpu_reports
         * sizeof(MonitorCPUReport), MONITOR_GET_CPU_REPORTS_REPLY);
   sig->get_cpu_reports_reply.enabled = 1;
   sig->get_cpu_reports_reply.euId = MONITOR_ALL_EXECUTION_UNITS;
   sig->get_cpu_reports_reply.reserved0 = 0;
   sig->get_cpu_reports_reply.reserved1 = 0;

   /* Disable interrupts to avoid conflicts with the tick call etc. */
   LOCK_SAVE(msr);
   LOCK_ALL_CPU
   /* Search for the specified time stamp if non zero. */
   d = sig->get_cpu_reports_reply.reports;
   for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
   {
      if (IS_CPU_ACTIVE(cpu))
      {
         s SMP_INDEX_X(cpu) = (zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_cpu_data.prof_cpu_firstsample;
      }
   }
   /* Is it non zero? */
   if (tick || utick)
   {
      while (s SMP_INDEX_X(0)->valid)
      {
         /* Did we find the specified entry. */
         if ((tick == s SMP_INDEX_X(0)->time_ticks) && (utick == s SMP_INDEX_X(0)->time_uticks))
         {
            /* Found it. Point to the next, yet unread sample. */
            for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
            {
               if (IS_CPU_ACTIVE(cpu))
               {
                  s SMP_INDEX_X(cpu) = s SMP_INDEX_X(cpu)->next;
               }
           }
           break;
         }
         for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
         {
            if (IS_CPU_ACTIVE(cpu))
            {
               s SMP_INDEX_X(cpu) = s SMP_INDEX_X(cpu)->next;
            }
         }
      }
   }

   if (!s SMP_INDEX_X(0)->valid)
   {
      if (link_in)
      {
         if (zzprof.cpu.prof_cpu_enabled)
         {
            /* Link it into the request queue. */
            (*rsig)->sig_no = (SIGSELECT) zzprof.cpu.prof_cpu_reqlist;
            zzprof.cpu.prof_cpu_reqlist = *rsig;
            SMP_CODE(UNLOCK_ALL_CPU)
            LOCK_RESTORE(msr);
            free_buf(&sig);
            *rsig = NULL;
            return;
         }
         else
         {
            sig->get_cpu_reports_reply.enabled = 0;
            goto done;
         }
      }
      else
      {
         /*
          ** If the sample list is now empty, we will assume that the
          ** input signal contained a bad time stamp and send the entire
          ** backlog. This should make any user look for the bug.
          */
         for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
         {
            if (IS_CPU_ACTIVE(cpu))
            {
               s SMP_INDEX_X(cpu) = (zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_cpu_data.prof_cpu_firstsample;
            }
         }
      }
   }

   /* Count samples and copy them into the response. */
   count = 0;
   while (s SMP_INDEX_X(0)->valid)
   {
      d->time.tick = s SMP_INDEX_X(0)->time_ticks;
      d->time.ntick = s SMP_INDEX_X(0)->time_uticks;
      d->interval = s SMP_INDEX_X(0)->total_time;
      d->sum = s SMP_INDEX_X(0) ->samples;

      for (cpu = 1; cpu < zzprof.prof_all_num_cpus; cpu++)
      {
         if (IS_CPU_ACTIVE(cpu))
         {
            d->sum += s SMP_INDEX_X(cpu)->samples;
         }
      }
      /* Temporarily enable interrupts. */
      UNLOCK_ALL_CPU
      LOCK_RESTORE(msr);
      ++count;
      /* Yield */
      set_pri(get_pri(current_process()));
      LOCK_SAVE(msr);
      LOCK_ALL_CPU
      /* Check that we have a valid state after the unlock/lock. */
      if (!(s SMP_INDEX_X(0)->valid) || (d->time.tick != s SMP_INDEX_X(0)->time_ticks))
      {
         /* We got overrun so return with what we have */
         break;
      }

      ++d;
      for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
      {
         if (IS_CPU_ACTIVE(cpu))
         {
            s SMP_INDEX_X(cpu) = s SMP_INDEX_X(cpu)->next;
         }
      }
   }

   done:
   /* Enable interrupts. */
   UNLOCK_ALL_CPU
   LOCK_RESTORE(msr);

   /* Finalize the counter and truncate the signal. */
   lset_sigsize(&sig, sizeof(MonitorGetCPUReportsReply) + count
         * sizeof(MonitorCPUReport));
   sig->get_cpu_reports_reply.reportsCount = count;
   sig->get_cpu_reports_reply.status = MONITOR_STATUS_OK;

   send_w_s(&sig, offspring->pid, offspring->client_);
   free_buf(rsig);
}
#endif

static void
prof_send_cpu_load(union SIGNAL **rsig, int link_in, Offspring * offspring)
{
   LOCKSTATE msr;
   union SIGNAL   *sig;
   uint32_t cpu = (*rsig)->get_cpu_reports_request.reserved0;

   assert(rsig);
   assert(*rsig);
   assert(link_in == 0 || link_in == 1);
   assert_offspring(offspring);

   /* For integrity resons this code *must* run on core 0 */
#ifdef HAS_PROF_ALL_EXECUTION_UNITS
   if (cpu == MONITOR_ALL_EXECUTION_UNITS && zzprof.cpu.prof_cpu_enabled)
   {
#ifdef OSE_SMP
      /* delegate core summarized reporting */
      send_cpu_reports_sum(rsig,link_in,offspring);
      return;
#else
      cpu = 0;
#endif
   }
#endif
   
   /* If measurement is inactive, return requests with the active flag set to zero. */
   if (!zzprof.cpu.prof_cpu_enabled ||  cpu >= zzprof.prof_all_num_cpus || (!IS_CPU_ACTIVE(cpu)))
   {
      sig = alloc(sizeof (MonitorGetCPUReportsReply), MONITOR_GET_CPU_REPORTS_REPLY);
      if (cpu < zzprof.prof_all_num_cpus && !IS_CPU_ACTIVE(cpu))
      {
    	  sig->get_cpu_pri_reports_reply.status = MONITOR_STATUS_CPU_INACTIVE; /* non OSE core */
      }
      else
      {
    	  sig->get_cpu_reports_reply.status = ((cpu >= zzprof.prof_all_num_cpus && 
    	        cpu != MONITOR_ALL_EXECUTION_UNITS)
                ? MONITOR_STATUS_REQUEST_INVALID
                : MONITOR_STATUS_OK);
      }
      sig->get_cpu_reports_reply.enabled = zzprof.cpu.prof_cpu_enabled;
      sig->get_cpu_reports_reply.reportsCount = 0;
      sig->get_cpu_reports_reply.euId = cpu;
      sig->get_cpu_reports_reply.reserved0 = 0;
      sig->get_cpu_reports_reply.reserved1 = 0;

      send_w_s(&sig, offspring->pid, offspring->client_);
   }
   else
   {
      uint32_t        count;
      uint32_t        tick = (*rsig)->get_cpu_reports_request.time.tick;
      uint32_t        utick = (*rsig)->get_cpu_reports_request.time.ntick;
      MonitorCPUReport *d;
      volatile struct prof_cpu_sample *s;
      struct prof_cpu_core *const data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_cpu_data;

      /* Allocate a worst case response signal. */
      sig = alloc(sizeof (MonitorGetCPUReportsReply) +
                     zzprof.cpu.prof_cpu_reports * sizeof (MonitorCPUReport), MONITOR_GET_CPU_REPORTS_REPLY);
      sig->get_cpu_reports_reply.enabled = 1;
      sig->get_cpu_reports_reply.euId = cpu;
      sig->get_cpu_reports_reply.reserved0 = 0;
      sig->get_cpu_reports_reply.reserved1 = 0;

      /* Disable interrupts to avoid conflicts with the tick call etc. */

      count = 0;
      d = sig->get_cpu_reports_reply.reports;

      LOCK_SAVE(msr);
      SMP_CODE(SPIN_LOCK(&data->prof_cpu_core_data_lock);)

      /* Search for the specified time stamp if non zero. */
      s = data->prof_cpu_firstsample;
      /* Is it non zero? */
      if (tick || utick)
      {
         while (s->valid)
         {
            /* Did we find the specified entry. */
            if ((tick == s->time_ticks) && (utick == s->time_uticks))
            {
               /* Found it. Point to the next, yet unread sample. */
               s = s->next;
               break;
            }
            s = s->next;
         }
      }

      if (!s->valid)
      {
         if (link_in)
         {
            /* Link it into the request queue. */
            (*rsig)->sig_no = (SIGSELECT) zzprof.cpu.prof_cpu_reqlist;
            zzprof.cpu.prof_cpu_reqlist = *rsig;
            SMP_CODE(SPIN_UNLOCK(&data->prof_cpu_core_data_lock);)
            LOCK_RESTORE(msr);
            free_buf(&sig);
            *rsig = NULL;
            return;
         }
         else
         {
            /*
             ** If the sample list is now empty, we will assume that the
             ** input signal contained a bad time stamp and send the entire
             ** backlog. This should make any user look for the bug.
             */
               s = data->prof_cpu_firstsample;
         }
      }

      /* Count samples and copy them into the response. */
      while (s->valid)
      {
         d->time.tick = s->time_ticks;
         d->time.ntick = s->time_uticks;
         d->interval = s->total_time;
         d->sum = s->samples;
         /* Temporarily unlock to avoid long interrupt lock */
         SMP_CODE(SPIN_UNLOCK(&data->prof_cpu_core_data_lock);)
         LOCK_RESTORE(msr);
         ++count;
         LOCK_SAVE(msr);
         SMP_CODE(SPIN_LOCK(&data->prof_cpu_core_data_lock);)
         /* Check that we have a valid state after the unlock/lock. */
         if (!(zzprof.cpu.prof_cpu_enabled && s->valid && d->time.tick == s->time_ticks))
         {
            break;
         }
         ++d;
         s = s->next;
      }

      /* Reenable interrupts. */
      SMP_CODE(SPIN_UNLOCK(&data->prof_cpu_core_data_lock);)
      LOCK_RESTORE(msr);


      /* Finalize the counter and truncate the signal. */
      lset_sigsize(&sig, sizeof (MonitorGetCPUReportsReply) + count * sizeof (MonitorCPUReport));
      sig->get_cpu_reports_reply.reportsCount = count;
      sig->get_cpu_reports_reply.status = MONITOR_STATUS_OK;

      send_w_s(&sig, offspring->pid, offspring->client_);
   }
   free_buf(rsig);
}

/**
 * @function prof_send_cpu_pri_reports_sum
 *
 * @brief send pending priority cpu load reports.
 *
 * @long if link_in is true the request will be saved for later use
 * if no pending reports exists that matches the request.
 * @long
 * *rsig will be set to NULL and the caller should not operate on the
 * signal any more.
 *
 */
#if defined (HAS_PROF_ALL_EXECUTION_UNITS) && defined(OSE_SMP)
static void
prof_send_cpu_pri_reports_sum(union SIGNAL **rsig, int link_in, Offspring * offspring)
{
   LOCKSTATE msr;
   union SIGNAL   *sig;
   uint32_t        count;
   uint32_t        cpu;
   uint32_t        tick = (*rsig)->get_cpu_pri_reports_request.time.tick;
   uint32_t        utick = (*rsig)->get_cpu_pri_reports_request.time.ntick;
   struct MonitorCPUPriReport *d;
   volatile struct prof_pri_sample *s SMP_ARRAY;

   assert(rsig);
   assert(*rsig);
   assert(link_in == 0 || link_in == 1);
   assert_offspring(offspring);

   /* we have already checked that we have an active session */

   /* Allocate a worst case response signal. */
   sig = alloc(sizeof (MonitorGetCPUPriReportsReply) +
                 zzprof.pri.prof_pri_reports * sizeof (MonitorCPUPriReport), MONITOR_GET_CPU_PRI_REPORTS_REPLY);
   sig->get_cpu_pri_reports_reply.enabled = 1;
   sig->get_cpu_pri_reports_reply.euId = MONITOR_ALL_EXECUTION_UNITS;
   sig->get_cpu_pri_reports_reply.reserved0 = 0;
   sig->get_cpu_pri_reports_reply.reserved1 = 0;

   d = sig->get_cpu_pri_reports_reply.reports;
   count = 0;

   /* Disable interrupts to avoid conflicts with the tick call etc. */
   LOCK_SAVE(msr);
   LOCK_ALL_PRI
   /* Search for the specified time stamp if non zero. */
   for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
   {
           if (IS_CPU_ACTIVE(cpu))
           {
                   s SMP_INDEX_X(cpu) = (zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_pri_data.prof_pri_firstsample;
           }
   }
   /* Is it non zero? */
   if (tick || utick)
   {
      while ( s SMP_INDEX_X(0)->valid)
      {
         /* Did we find the specified entry. */
         if ((tick == s SMP_INDEX_X(0)->time_ticks) && (utick == s SMP_INDEX_X(0)->time_uticks))
         {
            /* Found it. Point to the next, yet unread sample. (forall)*/
            for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
            {
                if (IS_CPU_ACTIVE(cpu))
                {
                        s SMP_INDEX_X(cpu)= s SMP_INDEX_X(cpu)->next;
                }
            }

            break;
         }
         /* jump to next (forall)*/
         for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
         {
                 if (IS_CPU_ACTIVE(cpu))
                 {
                         s SMP_INDEX_X(cpu)= s SMP_INDEX_X(cpu)->next;
                 }
         }
      }
   }
   if (!s SMP_INDEX_X(0)->valid)
   {
      if (link_in)
      {
         if (zzprof.pri.prof_pri_enabled)
         {
            (*rsig)->sig_no = (SIGSELECT) zzprof.pri.prof_pri_reqlist;
            zzprof.pri.prof_pri_reqlist = *rsig;
            *rsig = NULL;
            UNLOCK_ALL_PRI
            LOCK_RESTORE(msr);
            free_buf(&sig);
            return;
         }
         else
         {
            goto done;
         }
      }
      else
      {
         /*
          ** If the sample list is now empty, we will assume that the
          ** input signal contained a bad time stamp and send the entire
          ** backlog. This should make any user look for the bug.
          */
         for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
         {
                 if (IS_CPU_ACTIVE(cpu))
                 {
                         s SMP_INDEX_X(cpu) = (zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_pri_data.prof_pri_firstsample;
                 }
         }
      }
   }
   /* Count samples and copy them into the response. */
   while (s SMP_INDEX_X(0)->valid)
   {
      int             i;
      int core_count = 1;

      d->time.tick = s SMP_INDEX_X(0)->time_ticks;
      d->time.ntick = s SMP_INDEX_X(0)->time_uticks;
      d->interval = s SMP_INDEX_X(0)->total_time;

      /* initiate with the first */
      d->sumInterrupt = s SMP_INDEX_X(0)->samples[34];
      d->sumBackground = s SMP_INDEX_X(0)->samples[32];

      for (i = 0; i < 32; i++)
      {
        d->sumPrioritized[i] = s SMP_INDEX_X(0)->samples[i];
      }

      /* it does not work to SMP_CODE the for loop's header */
      for (cpu = 1; cpu < zzprof.prof_all_num_cpus; cpu++)
      {
         if (IS_CPU_ACTIVE(cpu))
         {
            d->sumInterrupt += s SMP_INDEX_X(cpu)->samples[34];
            d->sumBackground += s SMP_INDEX_X(cpu)->samples[32];
            d->interval += s SMP_INDEX_X(cpu)->total_time;
            core_count++;

            for (i = 0; i < 32; i++)
            {
               d->sumPrioritized[i] += s SMP_INDEX_X(cpu)->samples[i];
            }
         }
      }

      UNLOCK_ALL_PRI
      LOCK_RESTORE(msr);
      d->interval /= core_count;
      d->reserved0 = 0;
      ++count;
      LOCK_SAVE(msr);
      LOCK_ALL_PRI
      /* Check that we have a valid state after the unlock/lock. */
      if (!(s SMP_INDEX_X(0)->valid) || (d->time.tick != s SMP_INDEX_X(0)->time_ticks))
      {
         /* Profiling got overrun so return with what we have */
         break;
      }
      ++d;
      for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
      {
          if (IS_CPU_ACTIVE(cpu))
                  {
                  s SMP_INDEX_X(cpu) = s SMP_INDEX_X(cpu)->next;
                  }
      }
   }

   done:
   /* Reenable interrupts. */
   UNLOCK_ALL_PRI
   LOCK_RESTORE(msr);
   /* Finalize the counter and truncate the signal. */
   lset_sigsize(&sig, sizeof (MonitorGetCPUPriReportsReply) + count * sizeof (MonitorCPUPriReport));
   sig->get_cpu_pri_reports_reply.reportsCount = count;
   sig->get_cpu_pri_reports_reply.status = MONITOR_STATUS_OK;
   send_w_s(&sig, offspring->pid, offspring->client_);

   free_buf(rsig);
}
#endif

/**
 * @function prof_send_cpu_pri_reports
 *
 * @brief send pending priority cpu load reports.
 *
 * @long if link_in is true the request will be saved for later use
 * if no pending reports exists that matches the request.
 * @long
 * *rsig will be set to NULL and the caller should not operate on the
 * signal any more.
 *
 */
static void
prof_send_cpu_pri_reports(union SIGNAL **rsig, int link_in, Offspring *offspring)
{
   LOCKSTATE msr;
   union SIGNAL   *sig;
   uint32_t cpu = (*rsig)->get_cpu_pri_reports_request.reserved0;

   assert(rsig);
   assert(*rsig);
   assert(link_in == 0 || link_in == 1);
   assert_offspring(offspring);

   if (!zzprof.pri.prof_pri_enabled)
   {
      sig = alloc(sizeof (MonitorGetCPUPriReportsReply), MONITOR_GET_CPU_PRI_REPORTS_REPLY);
      sig->get_cpu_pri_reports_reply.status = MONITOR_STATUS_OK;
      sig->get_cpu_pri_reports_reply.enabled = 0;
      sig->get_cpu_pri_reports_reply.reportsCount = 0;
      sig->get_cpu_pri_reports_reply.euId = cpu;
      sig->get_cpu_pri_reports_reply.reserved0 = 0;
      sig->get_cpu_pri_reports_reply.reserved1 = 0;

      send_w_s(&sig, offspring->pid, offspring->client_);
      return;
   }
#ifdef HAS_PROF_ALL_EXECUTION_UNITS
   if (cpu == MONITOR_ALL_EXECUTION_UNITS)
   {
#ifdef OSE_SMP
      prof_send_cpu_pri_reports_sum(rsig,link_in,offspring);
      return;
#else
      cpu = 0;
#endif
   }
#endif

   /* If measurement is inactive, return requests with the active flag set to zero. */
   if (cpu >= zzprof.prof_all_num_cpus || !IS_CPU_ACTIVE(cpu))
   {
      sig = alloc(sizeof (MonitorGetCPUPriReportsReply), MONITOR_GET_CPU_PRI_REPORTS_REPLY);
      if (!IS_CPU_ACTIVE(cpu))
      {
          sig->get_cpu_pri_reports_reply.status = MONITOR_STATUS_CPU_INACTIVE; /* non OSE core */
      }
      else
      {
          sig->get_cpu_pri_reports_reply.status = MONITOR_STATUS_REQUEST_INVALID;
      }
      sig->get_cpu_pri_reports_reply.enabled = 1;
      sig->get_cpu_pri_reports_reply.reportsCount = 0;
      sig->get_cpu_pri_reports_reply.euId = cpu;
      sig->get_cpu_pri_reports_reply.reserved0 = 0;
      sig->get_cpu_pri_reports_reply.reserved1 = 0;

      send_w_s(&sig, offspring->pid, offspring->client_);
   }
   /* If request is valid, return any samples found. */
   else
   {
      uint32_t        count = 0;
      uint32_t        tick = (*rsig)->get_cpu_pri_reports_request.time.tick;
      uint32_t        utick = (*rsig)->get_cpu_pri_reports_request.time.ntick;
      struct MonitorCPUPriReport *d;
      struct prof_pri_sample *s;
      struct prof_pri_core *const data= &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_pri_data;

      /* Allocate a worst case response signal. */
      sig = alloc(sizeof (MonitorGetCPUPriReportsReply) +
                  zzprof.pri.prof_pri_reports * sizeof (MonitorCPUPriReport), MONITOR_GET_CPU_PRI_REPORTS_REPLY);
      sig->get_cpu_pri_reports_reply.enabled = 1;
      sig->get_cpu_pri_reports_reply.euId = cpu;
      sig->get_cpu_pri_reports_reply.reserved0 = 0;
      sig->get_cpu_pri_reports_reply.reserved1 = 0;

      d = sig->get_cpu_pri_reports_reply.reports;

      /* Disable interrupts to avoid conflicts with the tick call etc. */

      LOCK_SAVE(msr);
      SMP_CODE(SPIN_LOCK(&data->prof_pri_core_data_lock);)

      /* Search for the specified time stamp if non zero. */
      s = data->prof_pri_firstsample;
      /* Is it non zero? */
      if (tick || utick)
      {
         while (s->valid)
         {
            /* Did we find the specified entry. */
            if ((tick == s->time_ticks) && (utick == s->time_uticks))
            {
               /* Found it. Point to the next, yet unread sample. */
               s = s->next;
               break;
            }
            s = s->next;
         }
      }
      if (!s->valid)
      {
         if (link_in)
         {
            (*rsig)->sig_no = (SIGSELECT) zzprof.pri.prof_pri_reqlist;
            zzprof.pri.prof_pri_reqlist = *rsig;
            *rsig = NULL;
            SMP_CODE(SPIN_UNLOCK(&data->prof_pri_core_data_lock);)
            LOCK_RESTORE(msr);
            free_buf(&sig);
            return;
         }
         else
         {
            /*
             ** If the sample list is now empty, we will assume that the
             ** input signal contained a bad time stamp and send the entire
             ** backlog. This should make any user look for the bug.
             */
            s = data->prof_pri_firstsample;
         }
      }


      while (s->valid)
      {
         int             i;

         d->time.tick = s->time_ticks;
         d->time.ntick = s->time_uticks;
         d->sumInterrupt = s->samples[34];
         d->sumBackground = s->samples[32];
         d->interval = s->total_time;
         for (i = 0; i < 32; i++)
         {
            d->sumPrioritized[i] = s->samples[i];
         }
         SMP_CODE(SPIN_UNLOCK(&data->prof_pri_core_data_lock);)
         LOCK_RESTORE(msr);
         d->reserved0 = 0;
         ++count;
         LOCK_SAVE(msr);
         SMP_CODE(SPIN_LOCK(&data->prof_pri_core_data_lock);)
         /* Check that we have a valid state after the unlock/lock. */
         if (!(s->valid) || (d->time.tick != s->time_ticks))
         {
            /* Profiling got overrun so return with what we have */
            break;
         }
         ++d;
         s = s->next;
      }

      /* Reenable interrupts. */
      SMP_CODE(SPIN_UNLOCK(&data->prof_pri_core_data_lock);)
      LOCK_RESTORE(msr);
      /* Finalize the counter and truncate the signal. */
      lset_sigsize(&sig, sizeof (MonitorGetCPUPriReportsReply) + count * sizeof (MonitorCPUPriReport));
      sig->get_cpu_pri_reports_reply.reportsCount = count;
      sig->get_cpu_pri_reports_reply.status = MONITOR_STATUS_OK;
      send_w_s(&sig, offspring->pid, offspring->client_);
   }
   free_buf(rsig);
}

/**
 * @function send_cpu_proc_reports_sum
 *
 * @brief send pending process cpu load reports (sum)
 *
 * @long if link_in is true the request will be saved for later use
 * if no pending reports exists that matches the request.
 * @long
 * *rsig will be set to NULL and the caller should not operate on the
 * signal any more.
 *
 */
#if defined(HAS_PROF_ALL_EXECUTION_UNITS) && defined(OSE_SMP)

static void send_cpu_proc_reports_sum(union SIGNAL **rsig, int link_in,
                                      Offspring * offspring,
                                      MonitorBoolean block)
{
   LOCKSTATE msr;
   union SIGNAL *sig;
   cpuid_t cpu;
   uint32_t tick = (*rsig)->get_cpu_process_reports_request.time.tick;
   uint32_t utick = (*rsig)->get_cpu_process_reports_request.time.ntick;
   MonitorCPUProcessReport *d;
   struct prof_proc_sample *s SMP_ARRAY;
   uint32_t rep_size = 0;
   uint32_t rep_nr = 0;
   MonitorStatus status = MONITOR_STATUS_OK;
   uint32_t total_processes;
   MonitorCPUProcess *final_list_ptr1;

   assert(rsig);
   assert(*rsig);
   assert(link_in == 0 || link_in == 1);
   assert_offspring(offspring);

   /* Calculate the total number processes to be considered */
   total_processes = zzprof.prof_all_num_cpus
         * (zzprof.proc[block].prof_proc_maxpid);

   /* create common buffer for holding sample values while sorting */
   final_list_ptr1 = (MonitorCPUProcess *) alloc(sizeof(MonitorCPUProcess) * total_processes,
         0);

   /* we have already checked that we have an active session */

   /* Allocate a worst case response signal. */
   sig = alloc(sizeof(MonitorGetCPUProcessReportsReply)
         + zzprof.proc[block].prof_proc_reports
               * (sizeof(MonitorCPUProcessReport)
                     + (zzprof.proc[block].prof_proc_maxpid)
                           * sizeof(MonitorCPUProcess)),
         block ? MONITOR_GET_CPU_BLOCK_REPORTS_REPLY
               : MONITOR_GET_CPU_PROCESS_REPORTS_REPLY);

   sig->get_cpu_process_reports_reply.enabled = 1;
   sig->get_cpu_process_reports_reply.euId = MONITOR_ALL_EXECUTION_UNITS;
   sig->get_cpu_process_reports_reply.reserved0 = 0;
   sig->get_cpu_process_reports_reply.reserved1 = 0;
   sig->get_cpu_process_reports_reply.reserved2 = 0;

   d = (void *) sig->get_cpu_process_reports_reply.processes;

   /* Disable interrupts to avoid conflicts with the tick call etc. */
   LOCK_SAVE(msr);
   LOCK_ALL_PROC(block)

   /* Search for the specified time stamp if non zero. */
   for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
   {
      if (IS_CPU_ACTIVE(cpu))
      {
         s SMP_INDEX_X(cpu) = (zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_proc_data[block].prof_proc_firstsample;
      }
   }

   /* Is it non zero? */
   if (tick || utick)
   {
      while (s SMP_INDEX_X(0)->valid)
      {
         /* Did we find the specified entry. */
         if ((tick == s SMP_INDEX_X(0)->time_ticks) && (utick == s SMP_INDEX_X(0)->time_uticks))
         {
            /* Found it. Point to the next, yet unread sample. (forall)*/
            for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
            {
               if (IS_CPU_ACTIVE(cpu))
               {
                  s SMP_INDEX_X(cpu) = s SMP_INDEX_X(cpu)->next;
               }
            }
            break;
         }
         /* jump to next (forall)*/
         for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
         {
            if (IS_CPU_ACTIVE(cpu))
            {
               s SMP_INDEX_X(cpu) = s SMP_INDEX_X(cpu)->next;
            }
         }
      }
   }

   /* Execute different code for global hash and local hash from here on */
   if (TRUE == zzprof.proc[block].prof_proc_common_hash)
   {
      if (!s SMP_INDEX_X(0)->valid)
      {
         if (link_in)
         {
            (*rsig)->sig_no = (SIGSELECT) zzprof.proc[block].prof_proc_reqlist;
            zzprof.proc[block].prof_proc_reqlist = *rsig;
            *rsig = NULL;
            UNLOCK_ALL_PROC(block);
            LOCK_RESTORE(msr);
            free_buf(&sig);
            free_buf((union SIGNAL **) &final_list_ptr1);
            return;
         }
         else
         {
            /*
             ** If the sample list is now empty, we will assume that the
             ** input signal contained a bad time stamp and send the entire
             ** backlog. This should make any user look for the bug.
             */
            for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
            {
               if (IS_CPU_ACTIVE(cpu))
               {
                  s SMP_INDEX_X(cpu) = (zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_proc_data[block].prof_proc_firstsample;
               }
            }
         }
      }

      while (s SMP_INDEX_X(0)->valid)
      {
         uint32_t i;

         d->time.tick = s SMP_INDEX_X(0)->time_ticks;
         d->time.ntick = s SMP_INDEX_X(0)->time_uticks;
         d->interval = s SMP_INDEX_X(0)->total_time;

         d->processesCount = s SMP_INDEX_X(0)->num_procs;
         d->reserved0 = 0;

         /* initiate with the first */
         d->sumOther = s SMP_INDEX_X(0)->samples[zzprof.proc[block].prof_proc_maxpid];

         for (i = 0; i < s SMP_INDEX_X(0)->num_procs; i++)
         {
            if (s SMP_INDEX_X(0)->pids[i] != 0)
            {
               d->processes[i].id = s SMP_INDEX_X(0)->pids[i];
               d->processes[i].sum = s SMP_INDEX_X(0)->samples[i];
            }
            else
            {
               d->processes[i].id = 0;
               d->processes[i].sum = 0;
            }
         }

         for (cpu = 1; cpu < zzprof.prof_all_num_cpus; cpu++)
         {
            if (IS_CPU_ACTIVE(cpu))
            {
               d->sumOther += s SMP_INDEX_X(cpu)->samples[zzprof.proc[block].prof_proc_maxpid];

               for (i = 0; i < s SMP_INDEX_X(cpu)->num_procs; i++)
               {
                  if ((s SMP_INDEX_X(cpu)->pids[i]) != 0)
                  {
                     if (d->processes[i].id != 0 && d->processes[i].id
                           != s SMP_INDEX_X(cpu)->pids[i])
                     {
                        ramlog_printf("rmm: prof: internal error:\n"
                           "   missmatch of pid in cpu proc/block reports between cores.\n");
                        status = MONITOR_STATUS_INTERNAL_ERROR;
                        goto done;
                     }
                     /* in case of MONITOR_ALL_EXECUTION_UNITS all pids must be the same */
                     d->processes[i].id = s SMP_INDEX_X(cpu)->pids[i];
                     d->processes[i].sum += s SMP_INDEX_X(cpu)->samples[i];
                  }
                  if (cpu + 1 == zzprof.prof_all_num_cpus && d->processes[i].id
                        == 0)
                  {
                     ramlog_printf("rmm: prof: internal error:\n"
                        "   process slot allocated in report but not used.\n");
                     status = MONITOR_STATUS_INTERNAL_ERROR;
                     goto done;
                  }
               }
            }
         }

         /* Update the total report size and then process next report. The
          * reportsize is increased with the number process entries less
          * one which is already included in the MonitorCPUProcessReport */
         rep_size += (s SMP_INDEX_X(0)->num_procs - 1) * sizeof(MonitorCPUProcess)
               + sizeof(MonitorCPUProcessReport);
         rep_nr++;
         d = (void *) &d->processes[s SMP_INDEX_X(0)->num_procs];
         for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
         {
            if (IS_CPU_ACTIVE(cpu))
            {
               s SMP_INDEX_X(cpu) = s SMP_INDEX_X(cpu)->next;
            }
         }
      }
   }
   else
   {

      MonitorCPUProcess *final_list_ptr2 = final_list_ptr1;

      for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
      {
         if (IS_CPU_ACTIVE(cpu))
         {
            if (!s SMP_INDEX_X(cpu)->valid)
            {
               if (link_in)
               {
                  if (zzprof.proc[block].prof_proc_enabled)
                  {
                     (*rsig)->sig_no
                           = (SIGSELECT) zzprof.proc[block].prof_proc_reqlist;
                     zzprof.proc[block].prof_proc_reqlist = *rsig;
                     *rsig = NULL;
                     UNLOCK_ALL_PROC(block);
                     LOCK_RESTORE(msr);
                     free_buf(&sig);
                     free_buf((union SIGNAL **) &final_list_ptr1);
                     return;
                  }
                  else
                  {
                     sig->get_cpu_process_reports_reply.enabled = 0;
                     goto done;
                  }
               }
               else
               {
                  /*
                   ** If the sample list is now empty, we will assume that the
                   ** input signal contained a bad time stamp and send the entire
                   ** backlog. This should make any user look for the bug.
                   */
                  s SMP_INDEX_X(cpu) = (zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_proc_data[block].prof_proc_firstsample;

               }
            }
         }
      }

      while (s SMP_INDEX_X(0)->valid)
      {
         uint64_t interval = 0;
         uint32_t i, inactive_cpu_count, k;
         uint32_t j = 0;
         i = 0;
         k = 0;
         inactive_cpu_count = 0;

         /* check whether the sample is valid for all other cores */
         for (cpu = 1; cpu < zzprof.prof_all_num_cpus; cpu++)
         {
            if (IS_CPU_ACTIVE(cpu))
            {
               if (!(s SMP_INDEX_X(cpu)->valid))
               {
                  /* This sample was not ready on all cores so send what we got. */
                  goto done;
               }
            }
            else
            {
               inactive_cpu_count++;
            }
         }
         /* Flush/Reset the common array */
         for (j = 0; j < total_processes; j++)
         {
            (final_list_ptr1 + j)->id = 0;
            (final_list_ptr1 + j)->sum = 0;
         }
         /* get report points count for this sample */
         k = s SMP_INDEX_X(0)->rp_count;

         d->sumOther = 0;
         /* Let us select core 0 - sample timestamp, as the final value,
          do not care about other cores */
         d->time.tick = s SMP_INDEX_X(0)->time_ticks;
         d->time.ntick = s SMP_INDEX_X(0)->time_uticks;
         /* copy all cores sample into a common array */
         j = 0;
         for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
         {
            if (IS_CPU_ACTIVE(cpu))
            {
               interval += s SMP_INDEX_X(cpu)->total_time;
               /* Add all cores "sumOther" sample, but its not the final "sumOther" */
               for (i = 0; i < s SMP_INDEX_X(cpu)->num_procs; i++)
               {
                  if (s SMP_INDEX_X(cpu)->pids[i] != 0)
                  {
                     (final_list_ptr1 + j)->id = s SMP_INDEX_X(cpu)->pids[i];
                     (final_list_ptr1 + j)->sum = s SMP_INDEX_X(cpu)->samples[i];
                     j++;
                  }
                  else
                  {
                     ramlog_printf("rmm: prof: internal error:\n"
                        "   PID value Zero on core %u.\n", cpu);
                     status = MONITOR_STATUS_INTERNAL_ERROR;
                     goto done;
                  }
               }
               d->sumOther += s SMP_INDEX_X(cpu)->samples[zzprof.proc[block].prof_proc_maxpid];
            }
         }
         d->processesCount = j;
         
         /* Temporarily enable interrupts. */
         UNLOCK_ALL_PROC(block)
         LOCK_RESTORE(msr);
         /* Yield */
         set_pri(get_pri(current_process()));

         /* Do the following work without interrupt lock. */

         /* average out the total sample time from active cores */
         d->interval = interval / (zzprof.prof_all_num_cpus - inactive_cpu_count);

         d->reserved0 = 0;

         /* add similar pids sample values in the common array  */
         for (i = 0; i < total_processes - 1; i++)
         {
            if ((final_list_ptr1 + i)->id != 0)
            {
               for (j = i + 1; j < total_processes; j++)
               {
                  if (((final_list_ptr2 + j)->id != 0) && ((final_list_ptr2
                        + j)->id == (final_list_ptr1 + i)->id))
                  {
                     (final_list_ptr1 + i)->sum
                           += (final_list_ptr2 + j)->sum;
		     (final_list_ptr2 + j)->id = 0;
                     (final_list_ptr2 + j)->sum = 0;
                     d->processesCount--;
                  }
               }
            }
         }

         /* check whether the total processes on all cores for current sample is greater then maxpid */
         if (d->processesCount > (zzprof.proc[block].prof_proc_maxpid))
         {
            d->processesCount = zzprof.proc[block].prof_proc_maxpid;
         }

         /* check whether report points are enabled, then make sure that they stay at the top in the common array list
          even though they are not the highest cpu time consumed processes, so send appropriate index of the array
          from where sorting as to be done   */

#if 0
         ramlog_printf("rmm: calling qsort k = %u, total processes = %u processesCount = %u\n",
                       k, total_processes, d->processesCount);
#endif
         qsort(final_list_ptr1 + k, total_processes - k, sizeof(MonitorCPUProcess), prof_tcb_compslot);
#if 0
         { /* FIXME remove this block! Debug code only! */
            uint32_t prev_sum = ~0;
            for (i = k; i < total_processes; i++)
            {
               if (prev_sum < (final_list_ptr1 + i)->sum)
               {
                  ramlog_printf("rmm: **************** unsorted data: index %u pid 0x%X value %u\n",
                        i, (final_list_ptr1 + i)->id, (final_list_ptr1 + i)->sum);
                  for (i = 0; i < total_processes; i++)
                  {
                     ramlog_printf("rmm: content index %u pid 0x%X value %u\n",
                                             i, (final_list_ptr1 + i)->id, (final_list_ptr1 + i)->sum);
                  }
                  *((int *)1) = 0; /* fixme MAKE IT EXPLODE! */
               }
               prev_sum = (final_list_ptr1 + i)->sum;
            }
         } /* FIXME remove this block! */
#endif
         /* add left out other processes to "sumOther" */
         for (i = d->processesCount; i < total_processes; i++)
         {
            d->sumOther += (final_list_ptr1 + i)->sum;
         }

         /* copy the pids and corresponding sample values from common array to sample report */
         for (i = 0; i < d->processesCount; i++)
         {
            d->processes[i] = *(final_list_ptr1 + i);
         }

         /* Update the total report size and then process next report. The
          * reportsize is increased with the number process entries less
          * one which is already included in the MonitorCPUProcessReport */
         rep_size += (d->processesCount - 1) * sizeof(MonitorCPUProcess)
               + sizeof(MonitorCPUProcessReport);
         rep_nr++;

         /* Take the lock again. */
         LOCK_SAVE(msr);
         LOCK_ALL_PROC(block)

         /* Check that we have a valid state after the unlock/lock. */
         if (!(s SMP_INDEX_X(0)->valid) || (d->time.tick != s SMP_INDEX_X(0)->time_ticks))
         {
            /* We got overrun so return with what we have */
            goto done;
         }
         d = (void *) &d->processes[d->processesCount];
         for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++)
         {
            if (IS_CPU_ACTIVE(cpu))
            {
               s SMP_INDEX_X(cpu) = s SMP_INDEX_X(cpu)->next;
            }
         }
      }
   }

   done:
   /* Enable interrupts. */
   UNLOCK_ALL_PROC(block)
   LOCK_RESTORE(msr);

   sig->get_cpu_process_reports_reply.status = status;
   if (MONITOR_STATUS_OK == status)
   {
      /* Finalize the counter and truncate the signal. */
      lset_sigsize(&sig, sizeof(MonitorGetCPUProcessReportsReply) + rep_size);
      sig->get_cpu_process_reports_reply.processesCount = rep_size
            / sizeof(MonitorCPUProcess);
      sig->get_cpu_process_reports_reply.reportsCount = rep_nr;
   }
   else
   {
      lset_sigsize(&sig, sizeof(MonitorGetCPUProcessReportsReply));
      sig->get_cpu_process_reports_reply.processesCount = 0;
      sig->get_cpu_process_reports_reply.reportsCount = 0;
   }
   send_w_s(&sig, offspring->pid, offspring->client_);
   /* Free the request signal. */
   free_buf(rsig);
   /* free common_array buffer */
   free_buf((union SIGNAL **) &final_list_ptr1);
}
#endif

/**
 * @function prof_send_cpu_process_reports
 *
 * @brief send pending process cpu load reports.
 *
 * @long if link_in is true the request will be saved for later use
 * if no pending reports exists that matches the request.
 * @long
 * *rsig will be set to NULL and the caller should not operate on the
 * signal any more.
 *
 */
static void
prof_send_cpu_process_reports(union SIGNAL **rsig, int link_in, Offspring * offspring, MonitorBoolean block)
{
   union SIGNAL   *sig;
   uint32_t cpu = (*rsig)->get_cpu_process_reports_request.reserved0;
   LOCKSTATE       msr;

   assert(rsig);
   assert(*rsig);
   assert(link_in == 0 || link_in == 1);
   assert_offspring(offspring);

   if (!zzprof.proc[block].prof_proc_enabled)
   {
      sig = alloc(sizeof (MonitorGetCPUProcessReportsReply),
		  block ? MONITOR_GET_CPU_BLOCK_REPORTS_REPLY
                        : MONITOR_GET_CPU_PROCESS_REPORTS_REPLY);
      sig->get_cpu_process_reports_reply.status = MONITOR_STATUS_OK;
      sig->get_cpu_process_reports_reply.enabled = 0;
      sig->get_cpu_process_reports_reply.euId = cpu;
      sig->get_cpu_process_reports_reply.reserved0 = 0;
      sig->get_cpu_process_reports_reply.reserved1 = 0;
      sig->get_cpu_process_reports_reply.reserved2 = 0;
      sig->get_cpu_process_reports_reply.reportsCount = 0;
      sig->get_cpu_process_reports_reply.processesCount = 0;

      send_w_s(&sig, offspring->pid, offspring->client_);
      return;
   }
   
#ifdef HAS_PROF_ALL_EXECUTION_UNITS
   if (cpu == MONITOR_ALL_EXECUTION_UNITS)
   {
#ifdef OSE_SMP
      /* For all execution units, reports should be summed even in case of local hash */
      send_cpu_proc_reports_sum(rsig, link_in, offspring, block);
      return;
#else
      cpu = 0;
#endif
   }
#endif


   /* If measurement is inactive, return requests with the active flag set to zero. */
   if ( cpu >= zzprof.prof_all_num_cpus || !IS_CPU_ACTIVE(cpu))
   {
      sig = alloc(sizeof (MonitorGetCPUProcessReportsReply),
                  block ? MONITOR_GET_CPU_BLOCK_REPORTS_REPLY
                        : MONITOR_GET_CPU_PROCESS_REPORTS_REPLY);
      if (cpu >= zzprof.prof_all_num_cpus)
      {
    	  sig->get_cpu_process_reports_reply.status = MONITOR_STATUS_REQUEST_INVALID;
      }
      else
      {
	 sig->get_cpu_process_reports_reply.status =  MONITOR_STATUS_CPU_INACTIVE;
      }
      sig->get_cpu_process_reports_reply.enabled = 1;
      sig->get_cpu_process_reports_reply.euId = cpu;
      sig->get_cpu_process_reports_reply.reserved0 = 0;
      sig->get_cpu_process_reports_reply.reserved1 = 0;
      sig->get_cpu_process_reports_reply.reserved2 = 0;
      sig->get_cpu_process_reports_reply.reportsCount = 0;
      sig->get_cpu_process_reports_reply.processesCount = 0;
      send_w_s(&sig, offspring->pid, offspring->client_);
   }
   else                         /* If measurement is active, return any samples found. */
   {
      uint32_t        tick = (*rsig)->get_cpu_process_reports_request.time.tick;
      uint32_t        utick = (*rsig)->get_cpu_process_reports_request.time.ntick;
      MonitorCPUProcessReport *d;
      volatile struct prof_proc_sample *s;
      uint32_t rep_size = 0;
      uint32_t rep_nr = 0;
      struct prof_proc_core *data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_proc_data[block];


      /* Allocate a worst case response signal. */
      sig = alloc(sizeof (MonitorGetCPUProcessReportsReply) +
                  zzprof.proc[block].prof_proc_reports * (sizeof (MonitorCPUProcessReport) +
                                           (zzprof.proc[block].prof_proc_maxpid - 1) *
                                           sizeof (MonitorCPUProcess)),
                  block ? MONITOR_GET_CPU_BLOCK_REPORTS_REPLY
                        : MONITOR_GET_CPU_PROCESS_REPORTS_REPLY);

      sig->get_cpu_process_reports_reply.enabled = 1;
      sig->get_cpu_process_reports_reply.euId = cpu;
      sig->get_cpu_process_reports_reply.reserved0 = 0;
      sig->get_cpu_process_reports_reply.reserved1 = 0;
      sig->get_cpu_process_reports_reply.reserved2 = 0;

      d = (void *) sig->get_cpu_process_reports_reply.processes;

      /* Disable interrupts to avoid conflicts with the tick call etc. */

      LOCK_SAVE(msr);
      SMP_CODE(SPIN_LOCK(&data->prof_proc_core_data_lock);)

      /* Search for the specified time stamp if non zero. */
      s = data->prof_proc_firstsample;
      /* Is it non zero? */
      if (tick || utick)
      {
         while (s->valid)
         {
            /* Did we find the specified entry. */
            if ((tick == s->time_ticks) && (utick == s->time_uticks))
            {
               /* Found it. Point to the next, yet unread sample. */
               s = s->next;
               break;
            }
            s = s->next;
         }
      }
      /*
       ** If the sample list is now empty, we will assume that the
       ** input signal contained a bad time stamp and send the entire
       ** backlog. This should make any user look for the bug.
       */
      if (!s->valid)
      {
         if (link_in)
         {

            (*rsig)->sig_no = (SIGSELECT) zzprof.proc[block].prof_proc_reqlist;
            zzprof.proc[block].prof_proc_reqlist = *rsig;
            *rsig = NULL;

            SMP_CODE( SPIN_UNLOCK(&data->prof_proc_core_data_lock);)
            LOCK_RESTORE(msr);
            free_buf(&sig);
            return;
         }
         else
         {
            s = data->prof_proc_firstsample;
         }
      }

      /* Count samples and copy them into the response. */
      while (s->valid)
      {
         unsigned int i;
         unsigned int j;

         d->time.tick = s->time_ticks;
         d->time.ntick = s->time_uticks;
         d->interval = s->total_time;
         d->sumOther = s->samples[data->prof_proc_core_maxpid];

         for (i = j = 0; i < s->num_procs; i++)
         {
            /* The slot for a proc might be unused on this cpu core
             * and hence pid == 0*/
            if (0 != s->pids[i])
            {
               d->processes[j].id = s->pids[i];
               d->processes[j].sum = s->samples[i];
               j++;
            }
         }

         /* Temporarily enable interrupts. */
         SMP_CODE( SPIN_UNLOCK(&data->prof_proc_core_data_lock);)
         LOCK_RESTORE(msr);
         d->reserved0 = 0;
         d->processesCount = j;
         rep_nr++;

         LOCK_SAVE(msr);
         SMP_CODE(SPIN_LOCK(&data->prof_proc_core_data_lock);)
         /* Check that we have a valid state after the unlock/lock. */
         if (!zzprof.proc[block].prof_proc_enabled || !(s->valid) || (d->time.tick != s->time_ticks))
         {
            /* Profiling got disabled or we got overrun so return with what we have */
            break;
         }

         d = (MonitorCPUProcessReport *) &d->processes[d->processesCount];
         s = s->next;
      }

      /* Reenable interrupts. */
      SMP_CODE (SPIN_UNLOCK(&data->prof_proc_core_data_lock);)
      LOCK_RESTORE(msr);

      /* Finalize the counter and truncate the signal. */
      rep_size = ((char *)d) - ((char *)sig->get_cpu_process_reports_reply.processes);
      lset_sigsize(&sig, sizeof (MonitorGetCPUProcessReportsReply) + rep_size);

      sig->get_cpu_process_reports_reply.status = MONITOR_STATUS_OK;
      sig->get_cpu_process_reports_reply.processesCount = rep_size/sizeof(MonitorCPUProcess);
      sig->get_cpu_process_reports_reply.reportsCount = rep_nr;
      send_w_s(&sig, offspring->pid, offspring->client_);
   }
   /* Free the request signal. */
   free_buf(rsig);
}


static void
prof_send_total_cpu_block_reports(union SIGNAL **rsig, int link_in, Offspring *offspring)
{

    union SIGNAL   *sig;
    LOCKSTATE       msr;
    /* Restart the disconnect timer. */
    /* prof_proc_disctimer[1] = 100000000L / zzprof.tcb.ticklen; */ /* 100 sec. */


    assert(rsig);
    assert(link_in == 0 || link_in == 1);
    assert_offspring(offspring);

    /* If measurement is inactive, return requests with the active flag set to zero. */
    if (!zzprof.tcb.enabled )
    {
       sig = alloc(sizeof (MonitorGetCPUProcessReportsReply),
                   MONITOR_GET_TOTAL_CPU_BLOCK_REPORTS_REPLY);
       sig->get_cpu_process_reports_reply.status = MONITOR_STATUS_OK;
       sig->get_cpu_process_reports_reply.enabled = (zzprof.tcb.enabled != 0);
       sig->get_cpu_process_reports_reply.euId = 0;
       sig->get_cpu_process_reports_reply.reserved0 = 0;
       sig->get_cpu_process_reports_reply.reserved1 = 0;
       sig->get_cpu_process_reports_reply.reserved2 = 0;
       sig->get_cpu_process_reports_reply.reportsCount = 0;
       sig->get_cpu_process_reports_reply.processesCount = 0;
       send_w_s(&sig, offspring->pid, offspring->client_);
    }
    else                         /* If measurement is active, return any samples found. */
    {
       MonitorGetCPUProcessReportsRequest *req = (MonitorGetCPUProcessReportsRequest *)&(*rsig)->get_cpu_process_reports_request;
       uint32_t tick = req->time.tick;
       uint32_t utick = req->time.ntick;
       MonitorCPUProcessReport *d;
       struct prof_tcb_sample *s;
       uint32_t rep_size = 0;
       uint32_t rep_nr = 0;

       /* Allocate a worst case response signal. */
       sig = alloc(sizeof (MonitorGetCPUProcessReportsReply) +
                   zzprof.tcb.reports * (sizeof (MonitorCPUProcessReport) +
                                            (zzprof.tcb.maxpid - 1) *
                                            sizeof (MonitorCPUProcess)),
                                            MONITOR_GET_TOTAL_CPU_BLOCK_REPORTS_REPLY);

       sig->get_cpu_process_reports_reply.enabled = 1;
       sig->get_cpu_process_reports_reply.euId = 0;
       sig->get_cpu_process_reports_reply.reserved0 = 0;
       sig->get_cpu_process_reports_reply.reserved1 = 0;
       sig->get_cpu_process_reports_reply.reserved2 = 0;


       /* Disable interrupts to avoid conflicts with the tick call etc. */

       LOCK_SAVE(msr); /* lock disables the merge */

       /* Search for the specified time stamp if non zero. */
       d = (void *) sig->get_cpu_process_reports_reply.processes;
       s = zzprof.tcb.firstsample;
       /* Is it non zero? */
       if (tick || utick)
       {
          while (s->valid)
          {
             /* Did we find the specified entry. */
             if ((tick == s->time_ticks) && (utick == s->time_uticks))
             {
                /* Found it. Point to the next, yet unread sample. */
                s = s->next;
                break;
             }
             s = s->next;
          }
       }
       /*
        ** If the sample list is now empty, we will assume that the
        ** input signal contained a bad time stamp and send the entire
        ** backlog. This should make any user look for the bug.
        */
       if (!s->valid)
       {
          if (link_in)
          {
             (*rsig)->sig_no = (SIGSELECT) zzprof.tcb.prof_tcb_reqlist;
             zzprof.tcb.prof_tcb_reqlist = *rsig;
             (*rsig) = NULL;
             free_buf(&sig);
             LOCK_RESTORE(msr);
             return;
          }
          else
          {
             s = zzprof.tcb.firstsample;
          }
       }

       /* Count samples and copy them into the response. */
       while (s->valid)
       {
          uint32_t             i;

          d->time.tick = s->time_ticks;
          d->time.ntick = s->time_uticks;
          d->interval = s->total_time;
          d->sumOther = s->samples[zzprof.tcb.maxpid];
          d->processesCount = s->num_procs;
          d->reserved0 = 0;

          for (i = 0; i < s->num_procs; i++)
          {
             d->processes[i].id = s->pids[i];
             d->processes[i].sum = s->samples[i];
          }

          /* Update the total report size and then process next report. The
           * reportsize is increased with the number process entries less
           * one which is already included in the MonitorCPUProcessReport */
          rep_size += (s->num_procs - 1) * sizeof (MonitorCPUProcess) +
                      sizeof (MonitorCPUProcessReport);
          rep_nr++;
          d = (void *) &d->processes[s->num_procs];
          s = s->next;
       }

       LOCK_RESTORE(msr); /* now it is OK to merge again */
       /* Finalize the counter and truncate the signal. */
       lset_sigsize(&sig, sizeof (MonitorGetCPUProcessReportsReply) + rep_size);

       sig->get_cpu_process_reports_reply.status = MONITOR_STATUS_OK;
       sig->get_cpu_process_reports_reply.processesCount = rep_size/sizeof(MonitorCPUProcess);
       sig->get_cpu_process_reports_reply.reportsCount = rep_nr;
       send_w_s(&sig, offspring->pid, offspring->client_);
    }
    /* Free the request signal. */
    free_buf(rsig);


   return;
}

/***************************************************************************************************
 * Handle get * reports functions
 */

static int
zzprof_handle_get_total_cpu_block_reports(Offspring *offspring, union SIGNAL *sig)
{
   zzprof.tcb.disctimer = 100000000L / zzprof.prof_all_ticklen;   /* 100 sec. */
   NLOG(("DISCTIMER is %u\n", zzprof.tcb.disctimer))
   prof_send_total_cpu_block_reports(&sig, 1, offspring);
   return 1;
}

static int
zzprof_handle_get_cpu_reports(Offspring *offspring, union SIGNAL *sig)
{
   /* Restart the disconnect timer. */
   zzprof.cpu.prof_cpu_disctimer = 100000000L / zzprof.prof_all_ticklen;   /* 100 sec. */
   prof_send_cpu_load(&sig, 1, offspring);
   return 1;
}

static int
zzprof_handle_get_cpu_pri_reports(Offspring *offspring, union SIGNAL *sig)
{
   /* Restart the disconnect timer. */
   zzprof.pri.prof_pri_disctimer = 100000000L / zzprof.prof_all_ticklen;   /* 100 sec. */
   prof_send_cpu_pri_reports(&sig, 1, offspring);
   return 1;
}

static int
zzprof_handle_get_cpu_process_reports(Offspring *offspring, union SIGNAL *sig)
{
   /* Restart the disconnect timer. */
   zzprof.proc[0].prof_proc_disctimer = 100000000L / zzprof.prof_all_ticklen;  /* 100 sec. */
   prof_send_cpu_process_reports(&sig, 1, offspring, 0);
   return 1;
}

/****************************************************************************************
 *  Handle get * enabled functions
 *
 */


static int
zzprof_handle_get_total_cpu_block_reports_enabled(Offspring *offspring, union SIGNAL *sig)
{
   union SIGNAL   *sig_r;
   (void)sig;
   
   /* Send a reply to the offspring. */
   sig_r = alloc(sizeof (MonitorGetCPUProcessReportsEnabledReply) + 1,
                 MONITOR_GET_TOTAL_CPU_BLOCK_REPORTS_ENABLED_REPLY);
   sig_r->get_cpu_process_reports_enabled_reply.enabled = zzprof.tcb.enabled;
   if(sig_r->get_cpu_process_reports_enabled_reply.enabled)
   {
      sig_r->get_cpu_process_reports_enabled_reply.interval = zzprof.tcb.interval;
      sig_r->get_cpu_process_reports_enabled_reply.maxProcsReport = zzprof.tcb.maxpid;
      sig_r->get_cpu_process_reports_enabled_reply.maxReports = zzprof.tcb.reports;
#ifdef HAS_PROF_PRIORITY_THRESHOLD
      sig_r->get_cpu_process_reports_enabled_reply.priority = (uint8_t) zzprof.tcb.pri_threshold;
      sig_r->get_cpu_process_reports_enabled_reply.usePriority = True;
#endif
   }
   else
   {
      sig_r->get_cpu_process_reports_enabled_reply.interval = zzprof.tcb.default_interval;
      sig_r->get_cpu_process_reports_enabled_reply.maxProcsReport = zzprof.tcb.default_maxblocks;
      sig_r->get_cpu_process_reports_enabled_reply.maxReports = zzprof.tcb.default_reports;
#ifdef HAS_PROF_PRIORITY_THRESHOLD
      /* reuse the threshold from the cpu measurement */
      sig_r->get_cpu_process_reports_enabled_reply.priority = (uint8_t) zzprof.tcb.default_priority;
      sig_r->get_cpu_process_reports_enabled_reply.usePriority = True;
#endif
   }

   get_abstime((OSTICK *)&sig_r->get_cpu_process_reports_enabled_reply.sec,
         (OSTICK *)&sig_r->get_cpu_process_reports_enabled_reply.sectick,
         (OSTICK *)&sig_r->get_cpu_process_reports_enabled_reply.secntick);

#ifdef HAS_PROF_PRIORITY_THRESHOLD
   sig_r->get_cpu_process_reports_enabled_reply.reserved1 = 0;
#else
   sig_r->get_cpu_process_reports_enabled_reply.reserved0 = 0;
#endif

   send_w_s(&sig_r, offspring->pid, offspring->client_);
   return 0;
}


static int
zzprof_handle_get_cpu_reports_enabled(Offspring *offspring, union SIGNAL *sig)
{
   union SIGNAL   *sig_r;
   (void)sig;
   
   sig_r = alloc(sizeof (MonitorGetCPUReportsEnabledReply), MONITOR_GET_CPU_REPORTS_ENABLED_REPLY);
   sig_r->get_cpu_reports_enabled_reply.enabled = zzprof.cpu.prof_cpu_enabled;
   if(sig_r->get_cpu_reports_enabled_reply.enabled)
   {
      sig_r->get_cpu_reports_enabled_reply.interval = zzprof.cpu.prof_cpu_interval;
      sig_r->get_cpu_reports_enabled_reply.priority = zzprof.cpu.prof_cpu_pri;
      sig_r->get_cpu_reports_enabled_reply.maxReports = zzprof.cpu.prof_cpu_reports;
   }
   else
   {
      sig_r->get_cpu_reports_enabled_reply.interval = zzprof.cpu.default_cpu_interval;
      sig_r->get_cpu_reports_enabled_reply.priority = zzprof.cpu.default_cpu_priority;
      sig_r->get_cpu_reports_enabled_reply.maxReports = zzprof.cpu.default_cpu_reports;
   }
   get_abstime((OSTICK *)&sig_r->get_cpu_reports_enabled_reply.sec,
         (OSTICK *)&sig_r->get_cpu_reports_enabled_reply.sectick,
         (OSTICK *)&sig_r->get_cpu_reports_enabled_reply.secntick);
   sig_r->get_cpu_reports_enabled_reply.reserved0 = 0;
   send_w_s(&sig_r, offspring->pid, offspring->client_);
   return 0;
}

static int
zzprof_handle_get_cpu_pri_reports_enabled(Offspring *offspring, union SIGNAL *sig)
{
   union SIGNAL   *sig_r;
   (void)sig;
   
   /* Send a reply to the offspring. */
   sig_r = alloc(sizeof (MonitorGetCPUPriReportsEnabledReply),
                 MONITOR_GET_CPU_PRI_REPORTS_ENABLED_REPLY);
   sig_r->get_cpu_pri_reports_enabled_reply.enabled = zzprof.pri.prof_pri_enabled ;
   if(sig_r->get_cpu_pri_reports_enabled_reply.enabled)
   {
      sig_r->get_cpu_pri_reports_enabled_reply.interval = zzprof.pri.prof_pri_interval;
      sig_r->get_cpu_pri_reports_enabled_reply.maxReports = zzprof.pri.prof_pri_reports;
   }
   else
   {
      sig_r->get_cpu_pri_reports_enabled_reply.interval = zzprof.pri.default_pri_interval;
      sig_r->get_cpu_pri_reports_enabled_reply.maxReports = zzprof.pri.default_pri_reports;
   }
   get_abstime((OSTICK *)&sig_r->get_cpu_pri_reports_enabled_reply.sec,
         (OSTICK *)&sig_r->get_cpu_pri_reports_enabled_reply.sectick,
         (OSTICK *)&sig_r->get_cpu_pri_reports_enabled_reply.secntick);
   sig_r->get_cpu_pri_reports_enabled_reply.reserved0 = 0;
   send_w_s(&sig_r, offspring->pid, offspring->client_);
   return 0;
}

static int
zzprof_handle_get_cpu_process_reports_enabled(Offspring *offspring, union SIGNAL *sig)
{
   MonitorBoolean block = sig->sig_no == MONITOR_GET_CPU_BLOCK_REPORTS_ENABLED_REQUEST;
   union SIGNAL   *sig_r;
   struct prof_proc *data = &zzprof.proc[block];

   /* Send a reply to the offspring. */
   sig_r = alloc(sizeof (MonitorGetCPUProcessReportsEnabledReply),
                 block ? MONITOR_GET_CPU_BLOCK_REPORTS_ENABLED_REPLY
                       : MONITOR_GET_CPU_PROCESS_REPORTS_ENABLED_REPLY);
   sig_r->get_cpu_process_reports_enabled_reply.enabled = data->prof_proc_enabled;
   if(sig_r->get_cpu_process_reports_enabled_reply.enabled)
   {
      sig_r->get_cpu_process_reports_enabled_reply.interval = data->prof_proc_interval;
      sig_r->get_cpu_process_reports_enabled_reply.maxProcsReport = data->prof_proc_maxpid;
      sig_r->get_cpu_process_reports_enabled_reply.maxReports = data->prof_proc_reports;
#ifdef HAS_PROF_PRIORITY_THRESHOLD
      sig_r->get_cpu_process_reports_enabled_reply.priority = (uint8_t) data->prof_proc_pri;
      sig_r->get_cpu_process_reports_enabled_reply.usePriority = True;
#endif
   }
   else
   {
      sig_r->get_cpu_process_reports_enabled_reply.interval = data->default_proc_interval;
      sig_r->get_cpu_process_reports_enabled_reply.maxProcsReport = data->default_proc_maxprocs;
      sig_r->get_cpu_process_reports_enabled_reply.maxReports = data->default_proc_reports;
#ifdef HAS_PROF_PRIORITY_THRESHOLD
      /* reuse the threshold from the cpu measurement */
      sig_r->get_cpu_process_reports_enabled_reply.priority = (uint8_t) zzprof.cpu.default_cpu_priority;
      sig_r->get_cpu_process_reports_enabled_reply.usePriority = True;
#endif
   }

#ifdef HAS_PROF_PRIORITY_THRESHOLD
   /* set reserved1 to 1 instead of 0 */
   //sig_r->get_cpu_process_reports_enabled_reply.reserved1 = 0;
   sig_r->get_cpu_process_reports_enabled_reply.reserved1 = 1;

#else
   sig_r->get_cpu_process_reports_enabled_reply.reserved0 = 0;
#endif

   get_abstime((OSTICK *)&sig_r->get_cpu_process_reports_enabled_reply.sec,
         (OSTICK *)&sig_r->get_cpu_process_reports_enabled_reply.sectick,
         (OSTICK *)&sig_r->get_cpu_process_reports_enabled_reply.secntick);

   send_w_s(&sig_r, offspring->pid, offspring->client_);
   return 0;
}

/********************************************************************************************
 * Helpers to the _enable functions to see that all core have been disabled properly
 */

static OSBOOLEAN
prof_cpu_isdisabled(void)
{
   SMP_CODE(cpuid_t cpu;)
   SMP_CODE(for(cpu = 0 ; cpu < zzprof.prof_all_num_cpus; cpu ++) )
   {
	   SMP_CODE ( if ( IS_CPU_ACTIVE(cpu)))
	   {
		   if ((zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_cpu_data.prof_cpu_core_enabled != PROF_STATUS_DISABLED )
		   {
			   return FALSE;
		   }
	   }
   }
   return TRUE;
}

static OSBOOLEAN
prof_pri_isdisabled(void)
{
  SMP_CODE(cpuid_t cpu;)
  SMP_CODE(for(cpu = 0 ; cpu < zzprof.prof_all_num_cpus; cpu++) )
  {
	  SMP_CODE ( if ( IS_CPU_ACTIVE(cpu)))
	  {
		  if ((zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_pri_data.prof_pri_core_enabled != PROF_STATUS_DISABLED )
		  {
			  return FALSE;
		  }
	  }
  }
  return TRUE;
}

static OSBOOLEAN
prof_proc_isdisabled(OSBOOLEAN block)
{
  SMP_CODE(cpuid_t cpu;)
  SMP_CODE(for(cpu = 0 ; cpu < zzprof.prof_all_num_cpus; cpu++) )
  {
	  SMP_CODE ( if ( IS_CPU_ACTIVE(cpu)))
	  {
		  if ((zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_proc_data[block].prof_proc_core_enabled != PROF_STATUS_DISABLED )
		  {
			  return FALSE;
		  }
	  }
  }
  return TRUE;
}

static OSBOOLEAN
prof_tcb_isdisabled(void)
 {
   SMP_CODE(cpuid_t cpu;)
   SMP_CODE(for(cpu = 0 ; cpu < zzprof.prof_all_num_cpus; cpu++) )
   {
      SMP_CODE ( if ( IS_CPU_ACTIVE(cpu)))
      {
    	  if ((zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_tcb_data.core_enabled != PROF_STATUS_DISABLED )
    	  {
    		  return FALSE;
    	  }
      }
   }
   return TRUE;
 }



/********************************************************************************************************
 * handle set * enabled functions
 */

 static int
 zzprof_handle_set_total_cpu_block_reports_enabled(Offspring *offspring, union SIGNAL *sig)
 {
    union SIGNAL   *sig_r;
    MonitorStatus status = MONITOR_STATUS_OK;
    MonitorSetCPUProcessReportsEnabledRequest *req;
    LOCKSTATE msr;

    req = &sig->set_cpu_process_reports_enabled_request;

    NLOG(("GOT %s request for TCB \n", (req->enabled ? "enable" : "disable") ));

    LOCK_SAVE(msr);

    /* Enable profiling */
    if (req->enabled)
    {
       /* But only if previously disabled. */
        if (prof_tcb_isdisabled() && (zzprof.tcb.enabled == FALSE))
        {
           if (MONITOR_STATUS_OK == (status = prof_tcb_enable(sig)))
           {
              zzprof.tcb.enabled = TRUE;
              NLOG(("TCB enabled is set to true"));
              update_int_proc_hook();
           }
        }
        else
        {
           status = MONITOR_STATUS_IN_USE;
        }
     }
     /* Disable profiling. */
     else
     {
        /* But only if previously enabled. */
        if ( zzprof.tcb.enabled == TRUE)
        {
           prof_tcb_disable(&msr); /* the core_enabled is set to false in disable */
           LOCK_RESTORE(msr);
           prof_empty_pending_tcb_req(TRUE);
           LOCK_SAVE(msr);
        }
     }

    LOCK_RESTORE(msr);

     /* Send a reply to the offspring. */
     sig_r = alloc(sizeof (MonitorSetCPUProcessReportsEnabledReply),
                    MONITOR_SET_TOTAL_CPU_BLOCK_REPORTS_ENABLED_REPLY);

     sig_r->set_cpu_process_reports_enabled_reply.status = status;
     send_w_s(&sig_r, offspring->pid, offspring->client_);
     return 0;
  }


 static int
 zzprof_handle_set_cpu_pri_reports_enabled(Offspring *offspring, union SIGNAL *sig)
 {
    union SIGNAL   *sig_r;
    MonitorStatus status = MONITOR_STATUS_OK;
    MonitorSetCPUPriReportsEnabledRequest *req;
    LOCKSTATE msr;


    SMP_CODE(cpuid_t cpu;)

    req = &sig->set_cpu_pri_reports_enabled_request;
    LOCK_SAVE(msr);

    /* Enable profiling. */
    if (req->enabled)
    {
       /* But only if previously disabled. */
       if (prof_pri_isdisabled() && (zzprof.pri.prof_pri_enabled == FALSE))
       {
          uint32_t max_reports = zzprof.pri.default_pri_reports;

          /* Check if this signal is from a client with setting buffer capabilities */
          if(sigsize(&sig) >= sizeof(MonitorSetCPUPriReportsEnabledRequest))
          {
             if(req->reserved0 == 0)
             {
                /* Check that value is reasonable */
                if (in_range(req->maxReports, rw.pri_reports_range))
                {
                   max_reports = req->maxReports;
                }
             }
          }

          /* Update buffer sizes only if they have changed and are larger than zero */
          if(max_reports != zzprof.pri.prof_pri_reports)
          {
             if(!prof_setup_pri_report_bufs(max_reports))
             {
                LOCK_RESTORE(msr);
                /* Allocation of new buffers failed ! Respond client and break out*/
                sig_r = alloc(sizeof (MonitorSetCPUPriReportsEnabledReply),
                              MONITOR_SET_CPU_PRI_REPORTS_ENABLED_REPLY);
                sig_r->set_cpu_pri_reports_enabled_reply.status = get_last_memory_error();
                send_w_s(&sig_r, offspring->pid, offspring->client_);
                return 0;
             }
          }

          if(in_range(req->interval, rw.pri_interval_range) &&
             req->interval <= zzprof_get_interval())
          {
             zzprof.pri.prof_pri_interval = req->interval;
          }
          else
          {
             zzprof.pri.prof_pri_interval = zzprof.pri.default_pri_interval;
          }

          /* Enable sampling. */
          zzprof.pri.prof_pri_enabled = TRUE;
          update_int_proc_hook();

          LOCK_RESTORE(msr);

          SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++))
          {
             SMP_CODE ( if ( IS_CPU_ACTIVE(cpu)) )
             {
            	 union SIGNAL *startsig;

            	 startsig = alloc(sizeof(SIGSELECT), PROF_PRI_START_SIG );
            	 send(&startsig,   _zzrmm_core_pids SMP_INDEX_X(cpu) );
             }
          }
          LOCK_SAVE(msr); /* always leave with lock on */

       }
       else
       {
          status = MONITOR_STATUS_IN_USE;
       }
    }
    /* Disable profiling. */
    else
    {
       /* But only if previously enabled. */
       if (zzprof.pri.prof_pri_enabled)
       {
          prof_pri_disable();
          prof_empty_pending_pri_req(TRUE);
       }
    }
    LOCK_RESTORE(msr);

    sig_r = alloc(sizeof (MonitorSetCPUPriReportsEnabledReply),
                  MONITOR_SET_CPU_PRI_REPORTS_ENABLED_REPLY);
    sig_r->set_cpu_pri_reports_enabled_reply.status = status;
    send_w_s(&sig_r, offspring->pid, offspring->client_);
    return 0;
 }


 static int
 zzprof_handle_set_cpu_reports_enabled(Offspring *offspring, union SIGNAL *sig)
 {
    union SIGNAL   *sig_r;
    MonitorStatus status = MONITOR_STATUS_OK;
    MonitorSetCPUReportsEnabledRequest *req;
    LOCKSTATE msr;

    SMP_CODE(cpuid_t cpu;)

    req = &sig->set_cpu_reports_enabled_request;

    LOCK_SAVE(msr);

    /* Enable profiling */
    if(req->enabled)
    {

       NLOG(("got enable cpu sig\n"));
       /* But only if previously disabled */
       if(  prof_cpu_isdisabled() && (zzprof.cpu.prof_cpu_enabled == FALSE))
       {
          uint32_t max_reports = zzprof.cpu.default_cpu_reports;
          /* Check if this signal is from a client with setting buffer capabilities */
          if(sigsize(&sig) >= sizeof(MonitorSetCPUReportsEnabledRequest))
          {
             if(req->reserved0 == 0)
             {
                /* Check that value is reasonable */
                if (in_range(req->maxReports, rw.cpu_reports_range))
                {
                   max_reports = req->maxReports;
                }
             }
          }
          NLOG (("RMM max reports %u", max_reports  ));
          /* Update buffers sizes only if they have changed */
          if(max_reports != zzprof.cpu.prof_cpu_reports)
          {
             if(!prof_setup_cpu_report_bufs(max_reports))
             {
                /* Turn off cpu profiling */
                zzprof.cpu.prof_cpu_enabled = FALSE;

                LOCK_RESTORE(msr);
                LOG (("RMM ALLOCATION FAILED for CPU"  ));
                /* Allocation failed! Respond to client and break out!*/
                sig_r = alloc(sizeof (MonitorSetCPUReportsEnabledReply), MONITOR_SET_CPU_REPORTS_ENABLED_REPLY);
                sig_r->set_cpu_reports_enabled_reply.status =  get_last_memory_error();
                send_w_s(&sig_r, offspring->pid, offspring->client_);
                return 0;
             }
          }

          if (in_range(req->interval, rw.cpu_interval_range))
          {
             zzprof.cpu.prof_cpu_interval = req->interval;
          }
          else
          {
             zzprof.cpu.prof_cpu_interval = zzprof.cpu.default_cpu_interval;
          }

          if (in_range(req->priority, rw.cpu_priority_range))
          {
             zzprof.cpu.prof_cpu_pri = req->priority;
          }
          else
          {
             zzprof.cpu.prof_cpu_pri = zzprof.cpu.default_cpu_priority;
          }
          SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++))
          {
             SMP_CODE( if (IS_CPU_ACTIVE(cpu) ))
             {
            	 union SIGNAL *startsig;

            	 LOG (("RMM SEND CPU_START_SIG"  ));
            	 startsig = alloc(sizeof(SIGSELECT), PROF_CPU_START_SIG );
            	 send(&startsig,   _zzrmm_core_pids SMP_INDEX_X(cpu) );
             }
          }
          /* don't enable prof */
          zzprof.cpu.prof_cpu_enabled = TRUE;
          /* Enable the kernel and target interface swap hooks. */
          update_int_proc_hook();
       }
       else
       {
          status = MONITOR_STATUS_IN_USE;
       }
    }
    /* Disable profiling */
    else
    {
       /* But only if previously enabled. */
       if (zzprof.cpu.prof_cpu_enabled)
       {
          prof_cpu_disable();
          prof_empty_pending_cpu_req(TRUE);
       }
    }
    LOCK_RESTORE(msr);

    NLOG (("RMM CPU ENABLED returns"  ));
    sig_r = alloc(sizeof (MonitorSetCPUReportsEnabledReply), MONITOR_SET_CPU_REPORTS_ENABLED_REPLY);
    sig_r->set_cpu_reports_enabled_reply.status = status;
    send_w_s(&sig_r, offspring->pid, offspring->client_);
    return 0;
 }


static int
zzprof_handle_set_cpu_process_reports_enabled(Offspring *offspring, union SIGNAL *sig)
{
   MonitorBoolean block = sig->sig_no == MONITOR_SET_CPU_BLOCK_REPORTS_ENABLED_REQUEST;
   union SIGNAL   *sig_r;
   MonitorStatus status = MONITOR_STATUS_OK;
   MonitorSetCPUProcessReportsEnabledRequest *req;
   LOCKSTATE msr;
   struct prof_proc *data = &zzprof.proc[block];

   SMP_CODE(cpuid_t cpu;)

   req = &sig->set_cpu_process_reports_enabled_request;

   /* set reserved1 to 1 , to select local hash instead of global hash */
   SMP_CODE(req->reserved1 = 1;)

   LOCK_SAVE(msr);

   /* Enable profiling */
   if (req->enabled)
   {
      /* But only if previously disabled. */
      if ( prof_proc_isdisabled(block) && (data->prof_proc_enabled == FALSE) )
      {
         uint32_t max_reports = data->default_proc_reports;
         uint32_t max_procs_report = data->default_proc_maxprocs;

         /* we are fully disabled here no more than the interrupt lock is needed _*/

         /* Check if this signal is from a client with setting buffer capabilities etc */
         if(sigsize(&sig) >= sizeof(MonitorSetCPUProcessReportsEnabledRequest))
         {
#ifdef HAS_PROF_PRIORITY_THRESHOLD
            if ((0 == req->reserved1) || (req->reserved1 == 1)) /* only one bit allowed */
#else
            if (req->reserved0 == 0)
#endif
            {

               NLOG(("Nothing strange with the reserved field      "))

#ifdef HAS_PROF_PRIORITY_THRESHOLD
               SMP_CODE(zzprof.proc[block].prof_proc_common_hash = ( req->reserved1 == 1) ? FALSE : TRUE;)
#else
               SMP_CODE(zzprof.proc[block].prof_proc_common_hash = TRUE;)
#endif

               /* Check that values are reasonable */
               if (in_range(req->maxReports, block ? rw.block_reports_range : rw.proc_reports_range) &&
                   in_range(req->maxProcsReport, block ? rw.block_maxprocs_range : rw.proc_maxprocs_range))
               {
                  max_reports = req->maxReports;
                  max_procs_report = req->maxProcsReport;
               }
#ifdef HAS_PROF_PRIORITY_THRESHOLD
                   if (req->usePriority)
               {
                  if (in_range(req->priority, rw.cpu_priority_range))
                  {
                     data->prof_proc_pri = (OSPRIORITY) req->priority;
                  }
                  else
                  {
                     data->prof_proc_pri = (OSPRIORITY) zzprof.cpu.default_cpu_priority;
                  }
               }
               else
#endif
               {
                  data->prof_proc_pri = 33;
               }
            } /* reserved1 != 0 */
         }
         /* Update buffer sizes only if they have changed */
         if((max_reports != data->prof_proc_reports) || (max_procs_report != data->prof_proc_maxpid))
         {
            if(!prof_setup_proc_report_bufs(max_reports, max_procs_report, block))
            {
               /* The allocation failed so we do not have any reports and reportpoints allocated. */
               data->prof_proc_maxpid = 0;
               data->prof_proc_reports = 0;
               LOCK_RESTORE(msr);
               /* Send a not ok responde to offspring and break out of case*/
               sig_r = alloc(sizeof (MonitorSetCPUProcessReportsEnabledReply),
                             block ? MONITOR_SET_CPU_BLOCK_REPORTS_ENABLED_REPLY
                                   : MONITOR_SET_CPU_PROCESS_REPORTS_ENABLED_REPLY);

               sig_r->set_cpu_process_reports_enabled_reply.status = get_last_memory_error();
               send_w_s(&sig_r, offspring->pid, offspring->client_);
               return 0;
            }
            else {
               /* values were accepted */
               zzprof.proc[block].prof_proc_maxpid = max_procs_report;
               zzprof.proc[block].prof_proc_reports = max_reports;
            }
         } /* reset the pid entry in the global hash table */
#ifdef OSE_SMP
	 else if(zzprof.proc[block].prof_proc_common_hash == TRUE)
         {
	    uint32_t i;

	    for(i = 0; i < data->global_hash.hash_list_size; i++)
	    {
	       if(data->global_hash.pid_hash_table[i].pid)
	       {
		  data->global_hash.pid_hash_table[i].pid = 0;
	       }
	    }
         }
#endif

         /* Start setting up the profiling */
         if(in_range(req->interval, block ? rw.block_interval_range : rw.proc_interval_range) &&
            req->interval <= zzprof_get_interval())
         {
            data->prof_proc_interval = req->interval;
         }
         else
         {
            data->prof_proc_interval = data->default_proc_interval;
         }

         data->master_hash_sync_counter = 0;
         SMP_CODE(data->global_hash.sample_counter = 1;)

         SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++))
         {
        	 SMP_CODE(if (IS_CPU_ACTIVE(cpu)))
        	 {
        	 	struct prof_proc_core *cdata = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_proc_data[block];
                /*reset the pid entry in the local hash table ***/
        	 	uint32_t j;

				for(j = 0; j < cdata->local_hash.hash_list_size; j++)
				{
				   if(cdata->local_hash.pid_hash_table[j].pid)
				   {
				       cdata->local_hash.pid_hash_table[j].pid = 0;
				   }
				}

        	 	cdata->local_hash.sample_counter = 1;
                cdata->hash_sync_counter = 0;
        	 }
         }

         /* now send start sig */
         SMP_CODE(for (cpu = 0; cpu < zzprof.prof_all_num_cpus; cpu++))
         {
        	 SMP_CODE(if (IS_CPU_ACTIVE(cpu)) )
        	 {
        		 union SIGNAL *startsig;

        		 startsig = alloc(sizeof(SIGSELECT), (block == 1 ? PROF_BLOCK_START_SIG :PROF_PROC_START_SIG));
        		 send(&startsig,   _zzrmm_core_pids SMP_INDEX_X(cpu) );
        	 }
         }

         /* mark that we are enabled, the sampling is enabled per core. */
         zzprof.proc[block].prof_proc_enabled = TRUE;
         /* Enable the kernel and target interface swap hooks. */
         update_int_proc_hook();
      }
      else
      {
         status = MONITOR_STATUS_IN_USE;
      }
   }
   /* Disable profiling. */
   else
   {
      /* But only if previously enabled. */
      if (zzprof.proc[block].prof_proc_enabled == TRUE)
      {
         prof_proc_disable(block);
         LOCK_RESTORE(msr);
         prof_empty_pending_proc_req(block,TRUE);
         LOCK_SAVE(msr);
      }
   }
   LOCK_RESTORE(msr);

   /* Send a reply to the client. */
   sig_r = alloc(sizeof (MonitorSetCPUProcessReportsEnabledReply),
                 block ? MONITOR_SET_CPU_BLOCK_REPORTS_ENABLED_REPLY
                       : MONITOR_SET_CPU_PROCESS_REPORTS_ENABLED_REPLY);

   sig_r->set_cpu_process_reports_enabled_reply.status = status;
   send_w_s(&sig_r, offspring->pid, offspring->client_);
   return 0;
}




/********************************************************************************************
 * handle reportpoint functions
 */

static int
zzprof_handle_set_cpu_process_reportpoint(Offspring *offspring, union SIGNAL *sig)
{
   uint32_t        i;
   union SIGNAL   *sig_r;
   MonitorBoolean  block;
   MonitorCPUProcessReportpointInfo *req;
   MonitorStatus   status = MONITOR_STATUS_MEMORY_EXHAUSTED;
   LOCKSTATE msr;
   struct prof_proc *data;

   req = &sig->cpu_process_reportpoint_info;
   block = req->scope.type == MONITOR_SCOPE_ID && get_ptype(req->scope.id) == OS_BLOCK;
   data = &zzprof.proc[block];

   LOCK_SAVE(msr);
   SMP_CODE( SPIN_LOCK(&zzprof.proc[block].prof_proc_lock);)
   for (i = 0; i < data->prof_proc_maxpid; i++)
   {
      if (!data->prof_proc_spec[i])
      {
         uint32_t        j;

         sig->cpu_process_reportpoint_info.id = block ? 0x10000 + i : i;

         data->prof_proc_spec[i] = &sig->cpu_process_reportpoint_info;
#ifdef OSE_SMP
         /* Increment the number of report points */
         data->rp_count++;
#endif
         data->has_prof_proc_spec++;

         status = MONITOR_STATUS_OK;

         if (block)
         {
            data->prof_proc_pids[i] = req->scope.id;
         }
         else
         {
            struct OS_pid_list * proc_slots;
            /* prof_update_prof_proc_pids uses system calls that re-enable
               interrupts, so we need to release the lock while in the call
               and put the results in a temporary list*/

            SMP_CODE (SPIN_UNLOCK(&zzprof.proc[block].prof_proc_lock);)
            LOCK_RESTORE(msr);

            proc_slots = prof_update_prof_proc_pids();

            LOCK_SAVE(msr);
            SMP_CODE( SPIN_LOCK(&zzprof.proc[block].prof_proc_lock);)
            for (j = 0; j < data->prof_proc_maxpid; j++)
            {
               if (j < proc_slots->count)
               {
                  data->prof_proc_pids[j] = proc_slots->list[j];
               }
               else
               {
                  data->prof_proc_pids[j] = 0;
               }
            }
            SMP_CODE (SPIN_UNLOCK(&zzprof.proc[block].prof_proc_lock);)
            LOCK_RESTORE(msr);

            free_buf ((union SIGNAL **)&proc_slots);

            LOCK_SAVE(msr);
            SMP_CODE( SPIN_LOCK(&zzprof.proc[block].prof_proc_lock);)
         }
         break;
      }
   }
   SMP_CODE( SPIN_UNLOCK(&zzprof.proc[block].prof_proc_lock);)
   LOCK_RESTORE(msr);
   sig_r = alloc(sizeof (MonitorSetCPUProcessReportpointReply),
                 MONITOR_SET_CPU_PROCESS_REPORTPOINT_REPLY);

   sig_r->set_cpu_process_reportpoint_reply.status = status;
   sig_r->set_cpu_process_reportpoint_reply.id = req->id;
   send_w_s(&sig_r, offspring->pid, offspring->client_);
   return 1;
}

static int
zzprof_handle_clear_cpu_process_reportpoint(Offspring *offspring, union SIGNAL *sig)
{
   union SIGNAL   *sig_r;
   MonitorBoolean  block;
   MonitorStatus   status = MONITOR_STATUS_REQUEST_INVALID;
   uint32_t        i = sig->clear_cpu_process_reportpoint_request.id;
   LOCKSTATE msr;
   struct prof_proc *data;

   /* Clear all reportpoints on id ~0 */
   if (i == ~0U)
   {
      for (block = 0; block < 2; block++)
      {
         uint32_t        j;

         data = &zzprof.proc[block];
         LOCK_SAVE(msr);
         SMP_CODE( SPIN_LOCK(&data->prof_proc_lock);)
         data->has_prof_proc_spec = 0;
         for (j = 0; j < data->prof_proc_maxpid; j++)
         {
            if (data->prof_proc_spec[j])
            {
               free_buf((union SIGNAL **) &data->prof_proc_spec[j]);
#ifdef OSE_SMP
               /* reset the number of report points */
               data->rp_count = 0;
#endif

            }
         }
         status = MONITOR_STATUS_OK;

         for (j = 0; j < data->prof_proc_maxpid; j++)
         {
            data->prof_proc_pids[j] = 0;
         }
         SMP_CODE( SPIN_UNLOCK(&data->prof_proc_lock);)
         LOCK_RESTORE(msr);
      }
   }
   else
   {
      block = (0x10000UL & i) == 0x10000UL;
      i &= ~0x10000UL;
      data = &zzprof.proc[block];

      LOCK_SAVE(msr);
      SMP_CODE( SPIN_LOCK(&data->prof_proc_lock);)
      /* Clear selected reportpoint */
      if (i < data->prof_proc_maxpid && data->prof_proc_spec[i])
      {
         uint32_t        j;

         data->has_prof_proc_spec--;
         free_buf((union SIGNAL **) &data->prof_proc_spec[i]);
#ifdef OSE_SMP
         /* Decrement the number of report points */
         data->rp_count--;
#endif

         status = MONITOR_STATUS_OK;
         if (block)
         {
            data->prof_proc_pids[i] = 0;
         }
         else
         {
            struct OS_pid_list * proc_slots;
            /* prof_update_prof_proc_pids uses system calls that re-enable
               interrupts, so we need to release the lock while in the call
               and put the results in a temporary list*/

            SMP_CODE (SPIN_UNLOCK(&zzprof.proc[block].prof_proc_lock);)
            LOCK_RESTORE(msr);

            proc_slots = prof_update_prof_proc_pids();

            LOCK_SAVE(msr);
            SMP_CODE( SPIN_LOCK(&zzprof.proc[block].prof_proc_lock);)
            for (j = 0; j < data->prof_proc_maxpid; j++)
            {
               if (j < proc_slots->count)
               {
                  data->prof_proc_pids[j] = proc_slots->list[j];
               }
               else
               {
                  data->prof_proc_pids[j] = 0;
               }
            }
            SMP_CODE (SPIN_UNLOCK(&zzprof.proc[block].prof_proc_lock);)
            LOCK_RESTORE(msr);

            free_buf ((union SIGNAL **)&proc_slots);

            LOCK_SAVE(msr);
            SMP_CODE( SPIN_LOCK(&zzprof.proc[block].prof_proc_lock);)
         }
      }
      SMP_CODE( SPIN_UNLOCK(&data->prof_proc_lock);)
      LOCK_RESTORE(msr);
   }
   sig_r = alloc(sizeof (MonitorClearCPUProcessReportpointReply),
                 MONITOR_CLEAR_CPU_PROCESS_REPORTPOINT_REPLY);

   sig_r->clear_cpu_process_reportpoint_reply.status = status;
   send_w_s(&sig_r, offspring->pid, offspring->client_);
   return 0;
}

static int
zzprof_handle_get_cpu_process_reportpoint_info(Offspring *offspring, union SIGNAL *sig)
{
   uint32_t        i;
   union SIGNAL   *sig_r;
   MonitorBoolean  block;
   struct prof_proc *data;
   (void)sig;

   /* safe process cannot swap out itself to realloc the maxpid size */
   for (block = 0; block < 2; block++)
   {
      data = &zzprof.proc[block];
      for (i = 0; i < data->prof_proc_maxpid; i++)
      {
         MonitorCPUProcessReportpointInfo *r = data->prof_proc_spec[i];

         if (r)
         {
            uint32_t        size = sigsize((union SIGNAL **) &r);

            sig_r = alloc(size, 0);
            memcpy(sig_r, r, size);
            sig_r->sig_no = MONITOR_GET_CPU_PROCESS_REPORTPOINT_INFO_REPLY;
            send2offspring(&sig_r, offspring);
         }
      }
   }

   sig_r = alloc(sizeof (MonitorGetCPUProcessReportpointInfoEndmark),
         MONITOR_GET_CPU_PROCESS_REPORTPOINT_INFO_ENDMARK);
   sig_r->get_cpu_process_reportpoint_info_endmark.status = MONITOR_STATUS_OK;
   send(&sig_r, offspring->pid);
   return 0;
}

/*******************************************************************************************
 * add/remove processes to the set of profiled processes in real time
 * called from the create/kill dbghooks
 */


/* this function is called with swap_lock */
static void
zzprof_add_to_profiled_processes(struct debug_pcb *proc)
{
        struct prof_proc *data = &zzprof.proc[0];
  /* Decide if the process matches the per process profiling request list. */
  if (data->has_prof_proc_spec)      /* Have a list? */
  {
     if (prof_match_prof_proc_spec(proc))
     {
        uint32_t             i;
        LOCKSTATE msr;

        LOCK_SAVE(msr);
        /* Store the pid in a free entry of the pid array. */
        SMP_CODE( SPIN_LOCK(&data->prof_proc_lock);)
        for (i = 0; i < data->prof_proc_maxpid; ++i)
        {
           if (!data->prof_proc_pids[i])
           {
              data->prof_proc_pids[i] = proc->pid;
              break;
           }
        }
        SMP_CODE( SPIN_UNLOCK(&data->prof_proc_lock);)
        LOCK_RESTORE(msr);
     }
  }
}

/*
  ** Remove the process from the per process profiler list if
  ** we have such a list.
  */
/* this function is called with swap_lock */
static void
zzprof_remove_from_profiled_processes(struct debug_pcb *proc)
{
   struct prof_proc *data = &zzprof.proc[0];

   if (data->has_prof_proc_spec)
   {
      uint32_t             i, j;
      LOCKSTATE msr;

      LOCK_SAVE(msr);
      SMP_CODE( SPIN_LOCK(&data->prof_proc_lock);)
      /* Look for the killed pid in the profiler pre-selection list. */
      for (i = 0; i < data->prof_proc_maxpid; i++)
      {
         if (data->prof_proc_pids[i] == proc->pid)
         {
            /* Remove the found entry. */
            data->prof_proc_pids[i] = 0;

            /* Pull up the rest of the list. */
            for (j = i + 1; j < data->prof_proc_maxpid; j++)
            {
               data->prof_proc_pids[i++] = data->prof_proc_pids[j];
               data->prof_proc_pids[j] = 0;
            }

            /* don't continue the outer loop since i is gone. */
            break;
         }
      }
      SMP_CODE( SPIN_UNLOCK(&data->prof_proc_lock);)
      LOCK_RESTORE(msr);
   }
}

int
zzprof_handle_get_cpu_block_reports(Offspring *offspring, union SIGNAL *sig)
{
   /* Restart the disconnect timer. */
   zzprof.proc[1].prof_proc_disctimer = 100000000L / zzprof.prof_all_ticklen;  /* 100 sec. */
   prof_send_cpu_process_reports(&sig, 1, offspring, 1);
   return 1;
}

/*
** This function is called whenever a new sample arrives
** in the profiler.
**
** Pending sample report requests are returned to clients
** with the new sample data. If measurement is inactive,
** the reports are returned with the active flag cleared.
**
** Returns a non zero value if the sig_r pointer should be freed by
** the caller.
*/

void
zzprof_handle_new_sample(int type)
{
   /* FIXME if pointer is 64 bit invaluable data has been overwritten */

   /* Process per priority reqests. */
   if (type == PRIORITY_PROF)
   {
      struct prof_pri *data = &zzprof.pri;
      while (data->prof_pri_reqlist)
      {
         Offspring         *offspring;
         union SIGNAL   *rsig;

         /* Unlink the next request. */
         rsig = data->prof_pri_reqlist;
         data->prof_pri_reqlist = (union SIGNAL *) rsig->sig_no;      /* Next pointer. */

         offspring = find_offspring_by_pid(sender(&rsig));
         if (NULL != offspring)
         {
            prof_send_cpu_pri_reports(&rsig, 0, offspring);
         }
         else
         {
            /* No client anymore so free the request signal. */
            free_buf(&rsig);
         }
      }
   }
   /* Process global cpu load requests. */
   else if (type == CPU_OVERVIEW_PROF)
   {
      struct prof_cpu *data = &zzprof.cpu;
      while (data->prof_cpu_reqlist)
      {
         Offspring         *offspring;
         union SIGNAL   *rsig;

         /* Unlink the next request. */
         rsig = data->prof_cpu_reqlist;
         data->prof_cpu_reqlist = (union SIGNAL *) rsig->sig_no;      /* Next pointer. */

         offspring = find_offspring_by_pid(sender(&rsig));
         if (NULL != offspring)
         {
            prof_send_cpu_load(&rsig, 0, offspring);
         }
         else
         {
            /* We have lost the offspring that requested this. */
            free_buf(&rsig);
         }
      }
   }
   /* Process per process requests. */
   else if (type == PROCESS_PROF || type == BLOCK_PROF)
   {
      struct prof_proc *data = &zzprof.proc[(type==BLOCK_PROF ? 1 :0)];
      while (data->prof_proc_reqlist)
      {
         Offspring         *offspring;
         union SIGNAL   *rsig;

         /* Unlink the next request. */
         rsig = data->prof_proc_reqlist;
         data->prof_proc_reqlist = (union SIGNAL *) rsig->sig_no;     /* Next pointer. */

         offspring = find_offspring_by_pid(sender(&rsig));
         if (NULL != offspring)
         {
            prof_send_cpu_process_reports(&rsig, 0, offspring, type == BLOCK_PROF);
         }
         else
         {
            /* We have lost the offspring that requested this. */
            free_buf(&rsig);
         }
      }
   }

   else if (type == TOTAL_CPU_BLOCK_PROF)
   {
      /* data is not core specific */
      struct prof_tcb *data = &zzprof.tcb;
      while (data->prof_tcb_reqlist)
      {
         Offspring         *offspring;
         union SIGNAL   *rsig;

         /* Unlink the next request. */
         rsig = data->prof_tcb_reqlist;
         data->prof_tcb_reqlist = (union SIGNAL *) rsig->sig_no;     /* Next pointer. */

         offspring = find_offspring_by_pid(sender(&rsig));
         if (NULL != offspring)
         {
            prof_send_total_cpu_block_reports(&rsig, 0, offspring);
         }
         else
         {
            /* We have lost the offspring that requested this. */
            free_buf(&rsig);
         }
      }
   }
}

/*
** This function disables profiling when the offspring connection is lost.
**
** Pending sample reports from the offspring are discarded.
** Pending requests from other clients remain in the list.
**
** Per process and per priority measurement is disabled.
** Global CPU measurement remains active.
*/

static void
zzprof_disconnect(void)
{
   MonitorBoolean block;
   LOCKSTATE msr;

   LOCK_SAVE(msr);

   LOG(("Got disconnect\n"));

   /* Stop per priority profiling. */
   if (zzprof.pri.prof_pri_enabled)
   {
      prof_pri_disable();
      /* only free interrupt lock is safe */
      prof_empty_pending_pri_req(FALSE);
   }

   /* Stop cpu profiling. */
     if (zzprof.cpu.prof_cpu_enabled)
     {
        prof_cpu_disable();
        /* only free interrupt lock is safe */
        prof_empty_pending_cpu_req(FALSE);
     }

   /* Stop per process / block profiling. */
   for (block = 0; block < 2; block++)
   {
           struct prof_proc *gdata = &zzprof.proc[block];
      if (gdata->prof_proc_enabled)
      {
         prof_proc_disable(block);
         /* only free interrupt lock is safe */
         prof_empty_pending_proc_req(block, FALSE);
      }
   }

   /* Stop per total cpu block profiling. */
   if (zzprof.tcb.enabled)
   {
           prof_tcb_disable(&msr);
           /* only free interrupt lock is safe */
           prof_empty_pending_tcb_req(FALSE);
   }


   for (block = 0; block < 2; block++)
   {
      uint32_t        i;
      struct prof_proc *data = &zzprof.proc[block];

      data->has_prof_proc_spec = 0;

      for (i = 0; i < data->prof_proc_maxpid; i++)
      {
         if (data->prof_proc_spec[i])
         {
            /* we don't loose the interrupt lock at free_buf */
            free_buf((union SIGNAL **) &data->prof_proc_spec[i]);
         }
      }

      SMP_CODE(SPIN_LOCK(&data->prof_proc_lock);)
      for (i = 0; i < data->prof_proc_maxpid; i++)
      {
          data->prof_proc_pids[i] = 0;
      }
      SMP_CODE(SPIN_UNLOCK(&data->prof_proc_lock);)
   }


    LOCK_RESTORE(msr);
}



#ifdef OSE_SMP
/* this function is not static */

static OSADDRESS
ok_to_die_err_hnd(OSBOOLEAN user_called,
                  OSERRCODE ecode,
                  OSERRCODE extra)
{
   (void) extra;
   return ((user_called == 0 && (ecode & OSE_EERROR_MASK)
         == OSE_EKILLED_SYSTEM_DAEMON)) ? 1 : 0;
}

static void
prof_core_suicide(struct prof_core_data *const core_data)
{
   PROCESS curr = current_process();
   (void) core_data;

   /* Make sure pose will let us commit suicide. */
   (void) create_error_handler(curr, (OSERRH *) ok_to_die_err_hnd, 200);

   kill_proc(curr);
   /* the end */
}

static uint32_t
prof_set_core_status(cpuid_t cpu, OSBOOLEAN enable)
{
   struct prof_core_data * const core_data =
         &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu));
   if (enable == FALSE)
   {
      if (core_data->core_active == TRUE)
      {
         union SIGNAL *killsig;
         union SIGNAL *attachsig;
         union SIGNAL *sig;
         const SIGSELECT sigsel[] =
         { 1, PROF_CORE_ATTACH_SIG};

         /* deactivate all actions for the core (the hooks will still be running) */
         core_data->core_active = FALSE;

         /* attach signal setup */
         attachsig = alloc(sizeof(SIGSELECT), PROF_CORE_ATTACH_SIG);
         attach(&attachsig, _zzrmm_core_pids SMP_INDEX_X(cpu));

         /* send to */
         killsig = alloc(sizeof(SIGSELECT), PROF_CORE_KILL_SIG);NLOG(("kill core: PROF_CORE_KILL_SIG cpu=%u, pid=%pid\n", cpu, zzrmm_core_pids SMP_INDEX_X(cpu) ));
         send(&killsig, _zzrmm_core_pids SMP_INDEX_X(cpu) );

         /* wait for process to die */
         sig = receive(sigsel);
         free_buf(&sig);
         _zzrmm_core_pids SMP_INDEX_X(cpu) = 0;

         /*
          * Reports will be freed when z#endifzzprof_disable_core_free_buf is called
          */
         return 0;
      }
      else
      {
         return 1;
      }
   }
   else
   {

      return 1; /* ERROR NOT IMPLEMENTED */
#if 0
      uint32_t block = 0;

      prof_basic_core_init(cpu);
      core_data->prof_cpu_data.prof_cpu_core_issampled = zzprof.cpu.prof_cpu_issampled;

      /* not implemented */
      prof_start_core(cpu);

      /* CPU */
      /* allocate memory as for the other cores */
      if (zzprof.cpu.prof_cpu_enabled == TRUE)
      {
         /*			prof_prepare_cpu_start(&core_data->prof_cpu_data); */
      }

      /* PRI */
      /* allocate memory as for the other cores */
      /* ... */
      /* start */
      if (zzprof.pri.prof_pri_enabled == TRUE)
      {
         /*			prof_prepare_pri_start(&core_data->prof_pri_data); */
      }

      /* BLOCK o PROC */
      for (block = 0; block<2;block++)
      {
         struct prof_proc *data = &(zzprof.proc[block]);
         struct prof_proc_core *data_core = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).prof_proc_data[block];
         /* if not disabled we should not be here data->prof_proc_core_enabled = FALSE; */

         /* now do the allocation */
         prof_setup_proc_report_bufs_core( data->prof_proc_reports, data->prof_proc_maxpid, data_core);
      }

      /*
       * Start process
       * possible initiation of profiling
       * possible allocation of reports
       * catchup with reports
       *
       */
      (zzprof.prof_core_data_entry SMP_INDEX_X(cpu)).core_active = TRUE;
#endif
   }
   return 0;
}


void
zzzprof_disable_core_free_buf(cpuid_t cpu)
{
   /* send free core_prof_data to rmm that has allocated it */
   union SIGNAL *freesig;

   freesig = alloc(sizeof(ProfCoreFreeBuf), PROF_CORE_FREE_BUF);
   freesig->prof_core_free_buf.cpu = cpu;
   send(&freesig, rw.zzose_rmm_);
}

static void
zzprof_handle_prof_core_free_buf(cpuid_t cpu)
{
   struct prof_core_data *const core_data = &(zzprof.prof_core_data_entry SMP_INDEX_X(cpu));

   /* free sample_pool check for NULL pointer */
   free_sample_pool((union SIGNAL **)&core_data->prof_cpu_data.prof_cpu_samplepool );
   free_sample_pool((union SIGNAL **)&core_data->prof_pri_data.prof_pri_samplepool );

   prof_free_tcb_core_data(&(core_data->prof_tcb_data));
   prof_free_proc_report_bufs(&core_data->prof_proc_data[0]);
   prof_free_proc_report_bufs(&core_data->prof_proc_data[1]);
}

void
zzprof_disable_core(cpuid_t cpu)
{
   if (cpu == 0)
   {
      return;
   }

   /* We free the data since it is not possible to re-enable */
   if(prof_set_core_status(cpu, FALSE) == 0)
   {
      zzzprof_disable_core_free_buf(cpu);
   }
}

OSBOOLEAN
zzprof_get_core_status(cpuid_t cpu)
{
   return IS_CPU_ACTIVE(cpu);
}

#endif
